using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portalpages.common.constants;
using com.plumtree.portaluiinfrastructure.search;
using com.plumtree.portaluiinfrastructure.statichelpers;
using PT_SERVERCONSTANTS=com.plumtree.server.PT_SERVERCONSTANTS;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.admin.editors.bestbets
{
  /// <author>CraigS, based on JulieF template</author>
  public class BestBetEditorView : EditorView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
     "com.plumtree.portalpages.admin.editors.bestbets.BestBetEditorView");
    public const String STR_MVC_CLASS_NAME = "BestBetEditorView";
    //2464 public static final String _ui_NotSelected = "Not Selected";
    // 2455 public static final String _ui_SearchColon = "Search:";
    // 2448 public static final String _ui_Results = "Results";
    // 2449 public static final String _ui_Rank = "Rank";
    // 2450 public static final String _ui_DeleteSelected = "Delete Selected Items";
    // ptmsgs_portaladminmsgs 1674:   public static final String _ui_HelpText = "Enter the search string that will trigger this Best Bet.  Then run another search to find up to three items to be returned at the top of the search results list.";
    // HTML Object names
    // Best Bet trigger string.
    public const String SEARCH_STRING_SETTING = "in_tx_searchstring";
    public const String ADD_TARGET_BUTTON = "in_bu_addtarget";
    public const String DEL_ALLTARGET_CB = "in_cb_delalltarg";
    public const String DEL_TARGET_CB = "in_cb_deltarg";
    public const String NBSP = "&nbsp;";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new BestBetEditorView();
    }
    
    /// <summary>
    /// Returns the HTML for the view
    /// </summary>
    /// <returns>HTMLElement</returns>
    public override HTMLElement Display()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      IBestBetModelRO bbModel = (IBestBetModelRO) m_model;
      // Get the search model as well
      IBaseSearchResultModelRO searchModel = (IBaseSearchResultModelRO) m_asOwner.GetModel(SearchResultModel.STR_MVC_CLASS_NAME);
      try
      {
        HTMLTable myTable = new HTMLTable();
        myTable.SetWidth("100%");
        myTable.SetBorder("0");
        myTable.SetCellPadding("5");
        myTable.SetCellSpacing("0");
        result.AddInnerHTMLElement(myTable);
        HTMLTableRow myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        HTMLTableCell myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        AddTriggerInstructions(myCell, bbModel);
        AddBestBets(myCell, bbModel, searchModel);
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        AddSearchBoxAndResults(myCell, bbModel, searchModel);
      }
      catch (Exception e)
      {
        log.Error(e, e.Message);
        throw XPException.GetInstance(e);
      }
      return result;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    protected virtual bool CanMoveUp(int id)
    {
      IBestBetModelRO bbModel = (IBestBetModelRO) m_model;
      int numTargets = bbModel.GetNumTargets();
      if (numTargets == 1)
      {
        return false;
      }
      return (id > 1);
    }
    protected virtual bool CanMoveDown(int id)
    {
      IBestBetModelRO bbModel = (IBestBetModelRO) m_model;
      int numTargets = bbModel.GetNumTargets();
      if (numTargets == 1)
      {
        return false;
      }
      return (id < numTargets);
    }
    protected virtual void CreateButtons(HTMLTableCell holder, int id)
    {
      holder.AddInnerHTMLString(NBSP);
      HTMLElement a = CreateMoveElement(BestBetRepostControl.GetMoveTopCode(id),
       GetTopArrow(CanMoveUp(id)), m_asOwner.GetString(2396, ResourceConstants.PORTALADMINMSGS),
       CanMoveUp(id));
      holder.AddInnerHTMLElement(a);
      a = CreateMoveElement(BestBetRepostControl.GetMoveUpCode(id), GetUpArrow(CanMoveUp(id)),
       m_asOwner.GetString(2397, ResourceConstants.PORTALADMINMSGS), CanMoveUp(id));
      holder.AddInnerHTMLElement(a);
      a = CreateMoveElement(BestBetRepostControl.GetMoveDownCode(id), GetDownArrow(CanMoveDown(id)),
       m_asOwner.GetString(2398, ResourceConstants.PORTALADMINMSGS), CanMoveDown(id));
      holder.AddInnerHTMLElement(a);
      a = CreateMoveElement(BestBetRepostControl.GetMoveBottomCode(id), GetBottomArrow(CanMoveDown(id)),
       m_asOwner.GetString(2399, ResourceConstants.PORTALADMINMSGS), CanMoveDown(id));
      holder.AddInnerHTMLElement(a);
      holder.AddInnerHTMLString(NBSP);
    }
    protected virtual PTImageType GetTopArrow(bool enabled)
    {
      if (enabled)
      {
        return PTImageType.ICON_ARROW_TOP;
      } else
      {
        return PTImageType.ICON_ARROW_TOP_DISABLED;
      }
    }
    protected virtual PTImageType GetUpArrow(bool enabled)
    {
      if (enabled)
      {
        return PTImageType.ICON_ARROW_UP;
      } else
      {
        return PTImageType.ICON_ARROW_UP_DISABLED;
      }
    }
    protected virtual PTImageType GetDownArrow(bool enabled)
    {
      if (enabled)
      {
        return PTImageType.ICON_ARROW_DOWN;
      } else
      {
        return PTImageType.ICON_ARROW_DOWN_DISABLED;
      }
    }
    protected virtual PTImageType GetBottomArrow(bool enabled)
    {
      if (enabled)
      {
        return PTImageType.ICON_ARROW_BOTTOM;
      } else
      {
        return PTImageType.ICON_ARROW_BOTTOM_DISABLED;
      }
    }
    protected virtual HTMLElement CreateMoveElement(int postToSelfCode, PTImageType imageType,
     String altText, bool enabled)
    {
      HTMLImg image = SearchUIUtility.GetImage(this.m_asOwner, imageType,
       altText, "15", "20");
      if (!enabled)
      {
        return image;
      }
      HTMLAnchor retval = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
      // mj: removed 'Javascript:' from function name
      retval.SetOnClick("postToSelf(" + postToSelfCode + ");");
      retval.SetPreComment(altText);
      retval.AddInnerHTMLElement(image);
      return retval;
    }
    protected virtual void AddTriggerInstructions(HTMLElement container, IBestBetModelRO bbModel)
    {
      HTMLElement spacer = SearchUIUtility.makeSpacer(this.m_asOwner, "10");
      container.AddInnerHTMLElement(spacer);
      HTMLTable myTable = new HTMLTable();
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellPadding("0");
      myTable.SetCellSpacing("0");
      container.AddInnerHTMLElement(myTable);
      HTMLTableRow myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.OBJECT_HEADER_BG);
      HTMLTableCell myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetHeight("20");
      myCell.SetWidth("100%");
      myCell.SetVAlign("middle");
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      HTMLSpan mySpan = new HTMLSpan();
      mySpan.SetStyleClass(PTStyleClass.OBJECT_HEADER);
      mySpan.AddInnerHTMLString(m_asOwner.GetString(2392, ResourceConstants.PORTALADMINMSGS));
      myCell.AddInnerHTMLElement(mySpan);
      myCell = new HTMLTableCell();
      myCell.AddInnerHTMLString("&nbsp;");
      myCell.SetWidth("10");
      myRow.AddInnerHTMLElement(myCell);
      spacer = SearchUIUtility.makeSpacer(this.m_asOwner, "5");
      container.AddInnerHTMLElement(spacer);
      myTable = new HTMLTable();
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellPadding("5");
      myTable.SetCellSpacing("0");
      container.AddInnerHTMLElement(myTable);
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetWidth("100%");
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      myCell.AddInnerHTMLString(m_asOwner.GetString(2393, ResourceConstants.PORTALADMINMSGS));
      spacer = SearchUIUtility.makeSpacer(this.m_asOwner, "5");
      container.AddInnerHTMLElement(spacer);
      myTable = new HTMLTable();
      container.AddInnerHTMLElement(myTable);
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellPadding("5");
      myTable.SetCellSpacing("0");
      // 1st Row begin
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.LIST_ITEM_ONE_BG);
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetNoWrap(true);
      myCell.SetWidth("100%");
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      HTMLInput myInput = new HTMLInput(HTMLInputTypes.TEXT, SEARCH_STRING_SETTING,
       "");
      myCell.AddInnerHTMLElement(myInput);
      myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
      myInput.SetValue(bbModel.GetSearchString());
      myInput.SetMaxLength(PT_SERVERCONSTANTS.PT_MAX_NAMELENGTH);
      myInput.SetSize(30);
      spacer = SearchUIUtility.makeSpacer(this.m_asOwner, "10");
      container.AddInnerHTMLElement(spacer);
    }
    protected virtual void AddBestBets(HTMLElement container, IBestBetModelRO bbModel,
     IBaseSearchResultModelRO searchModel)
    {
      HTMLTable myTable = new HTMLTable();
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellPadding("0");
      myTable.SetCellSpacing("0");
      container.AddInnerHTMLElement(myTable);
      HTMLTableRow myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.OBJECT_HEADER_BG);
      HTMLTableCell myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetHeight("20");
      myCell.SetWidth("100%");
      myCell.SetVAlign("middle");
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      HTMLSpan mySpan = new HTMLSpan();
      mySpan.SetStyleClass(PTStyleClass.OBJECT_HEADER);
      mySpan.AddInnerHTMLString(m_asOwner.GetString(2394, ResourceConstants.PORTALADMINMSGS));
      myCell.AddInnerHTMLElement(mySpan);
      myCell = new HTMLTableCell();
      myCell.AddInnerHTMLString("&nbsp;");
      myCell.SetWidth("10");
      myRow.AddInnerHTMLElement(myCell);
      myTable = new HTMLTable();
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellPadding("0");
      myTable.SetCellSpacing("0");
      container.AddInnerHTMLElement(myTable);
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetHeight("25");
      myCell.SetNoWrap(true);
      myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
      // add image href
      bool enabled = true;
      //_ui_DeleteSelected
      HTMLElement el = CreateDeleteElement(PTImageType.ACTION_DELETE, m_asOwner.GetString(2450,
       ResourceConstants.PORTALADMINMSGS), "25", "25", enabled);
      myCell.AddInnerHTMLElement(el);
      myCell.SetWidth("25");
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetHeight("25");
      myCell.SetNoWrap(true);
      myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_DIVIDER);
      HTMLElement spacer = SearchUIUtility.makeSpacer(this.m_asOwner, "1");
      myCell.AddInnerHTMLElement(spacer);
      myCell = new HTMLTableCell();
      myCell.AddInnerHTMLString("&nbsp;");
      myCell.SetWidth("100%");
      myCell.SetHeight("25");
      myRow.AddInnerHTMLElement(myCell);
      int numTargets = bbModel.GetNumTargets();
      // Skip this table if there are no results.
      //if(numTargets == 0)
      //  return;
      myTable = new HTMLTable();
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellPadding("0");
      myTable.SetCellSpacing("2");
      container.AddInnerHTMLElement(myTable);
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
      myCell.SetAlign("center");
      myCell.SetVAlign("top");
      myCell.SetWidth("20");
      // Add overall delete checkbox
      HTMLInput checkB = new HTMLInput(HTMLInputTypes.CHECKBOX, DEL_ALLTARGET_CB,
       "");
      // mj: removed 'Javascript:' from function name
      checkB.SetOnClick("DoToggleAllDeletes();");
      myCell.AddInnerHTMLElement(checkB);
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
      myCell.SetHeight("20");
      myCell.SetWidth("95%");
      //_ui_Results
      myCell.AddInnerHTMLString(m_asOwner.GetString(2448, ResourceConstants.PORTALADMINMSGS));
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
      myCell.SetHeight("20");
      myCell.SetWidth("5%");
      myCell.SetColSpan("1");
      //_ui_Rank
      myCell.AddInnerHTMLString(m_asOwner.GetString(2449, ResourceConstants.PORTALADMINMSGS));
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
      myCell.SetHeight("20");
      myCell.SetWidth("5%");
      myCell.SetColSpan("1");
      // 1st Row end
      for (int j = 1;j <= numTargets;j++)
      {
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass((j % 2 == 1) ? PTStyleClass.LIST_ITEM_ONE_BG : PTStyleClass.LIST_ITEM_TWO_BG);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetAlign("center");
        myCell.SetVAlign("top");
        myCell.SetWidth("20");
        myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
        // Add Delete Checkbox
        checkB = new HTMLInput(HTMLInputTypes.CHECKBOX, GetCBName(j), "");
        myCell.AddInnerHTMLElement(checkB);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
        myCell.SetHeight("20");
        myCell.SetWidth("95%");
        // This is a top Best Bet.
        if (j == 1)
        {
          HTMLImg iconTBB = SearchUIUtility.GetImage(this.m_asOwner, PTImageType.ICON_SEARCH_TOPBESTBET,
           m_asOwner.GetString(1866, ResourceConstants.PORTALBROWSINGMSGS),
           "16", "16");
          myCell.AddInnerHTMLElement(iconTBB);
        } else
        {
          HTMLImg iconSpacer = SearchUIUtility.GetImage(this.m_asOwner, PTImageType.SPACER,
           "", "16", "16");
          myCell.AddInnerHTMLElement(iconSpacer);
        }
        myCell.AddInnerHTMLString(XPHttpUtility.HtmlEncode(bbModel.GetTargetName(j)));
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
        myCell.SetHeight("20");
        myCell.SetWidth("5%");
        myCell.SetAlign("middle");
        myCell.SetVAlign("center");
        myCell.AddInnerHTMLString(j + "");
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
        myCell.SetAlign("right");
        myCell.SetVAlign("center");
        myCell.SetNoWrap(true);
        CreateButtons(myCell, j);
      }
      int maxTargets = bbModel.GetMaxTargets();
    }
    /*for (int j = numTargets + 1; j <= maxTargets; j++)
     {
     ////////////////////////////
     myRow = new HTMLTableRow();
     myTable.AddInnerHTMLElement(myRow);
     myRow.SetStyleClass((j % 2 == 1) ? PTStyleClass.LIST_ITEM_ONE_BG : PTStyleClass.LIST_ITEM_TWO_BG);
     
     myCell = new HTMLTableCell();
     myRow.AddInnerHTMLElement(myCell);
     
     myCell.SetAlign("center");
     myCell.SetVAlign("top");
     myCell.SetWidth("20");
     myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
     // Add Delete Checkbox
     //checkB = new HTMLInput(HTMLInputTypes.CHECKBOX,GetCBName(j), "");
     //myCell.AddInnerHTMLElement(checkB);
     
     myCell = new HTMLTableCell();
     myRow.AddInnerHTMLElement(myCell);
     myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
     myCell.SetHeight("21");
     myCell.SetWidth("95%");
     
     myCell.AddInnerHTMLString(m_asOwner.GetString(2464, ResourceConstants.PORTALADMINMSGS));
     
     myCell = new HTMLTableCell();
     myRow.AddInnerHTMLElement(myCell);
     myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
     myCell.SetHeight("20");
     myCell.SetWidth("5%");
     myCell.SetAlign("middle");
     myCell.SetVAlign("center");
     //myCell.AddInnerHTMLString(j + "");
     
     myCell = new HTMLTableCell();
     myRow.AddInnerHTMLElement(myCell);
     myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
     myCell.SetAlign("right");
     myCell.SetVAlign("center");
     myCell.SetNoWrap(true);
     }*/
     
    protected virtual String GetCBName(int id)
    {
      return DEL_TARGET_CB + XPConvert.ToString(id);
    }
    protected virtual void AddSearchBoxAndResults(HTMLElement container, IBestBetModelRO bbModel,
     IBaseSearchResultModelRO searchModel)
    {
      //int numTargets = bbModel.GetNumTargets();
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      myTable = new HTMLTable();
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellPadding("5");
      myTable.SetCellSpacing("0");
      container.AddInnerHTMLElement(myTable);
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetWidth("100%");
      myCell.SetHeight("25");
      myCell.SetVAlign("middle");
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      myCell.AddInnerHTMLString(m_asOwner.GetString(2395, ResourceConstants.PORTALADMINMSGS));
      HTMLElement spacer = SearchUIUtility.makeSpacer(this.m_asOwner, "5");
      container.AddInnerHTMLElement(spacer);
      myTable = new HTMLTable();
      container.AddInnerHTMLElement(myTable);
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellPadding("5");
      myTable.SetCellSpacing("0");
      // Show a row for a target that could be added
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.LIST_ITEM_ONE_BG);
      myCell = new HTMLTableCell();
      myCell.SetNoWrap(true);
      myRow.AddInnerHTMLElement(myCell);
      HTMLSpan mySpan = new HTMLSpan();
      myCell.AddInnerHTMLElement(mySpan);
      mySpan.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
      //_ui_SearchColon
      mySpan.AddInnerHTMLString(m_asOwner.GetString(2455, ResourceConstants.PORTALADMINMSGS));
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetNoWrap(true);
      myCell.SetWidth("100%");
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      // Show the search box
      HTMLElement sbView = m_asOwner.GetView(SearchBoxView.STR_MVC_CLASS_NAME).Display();
      myCell.AddInnerHTMLElement(sbView);
      // If there are already search results
      if (searchModel.IsInitialized())
      {
        myTable = new HTMLTable();
        myTable.SetWidth("100%");
        myTable.SetBorder("0");
        myTable.SetCellPadding("0");
        myTable.SetCellSpacing("0");
        container.AddInnerHTMLElement(myTable);
        // Create another row for results
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);
        // Add cell for results table
        myCell = new HTMLTableCell();
        myCell.SetColSpan("3");
        myRow.AddInnerHTMLElement(myCell);
        // Show the results in this cell
        HTMLElement resView = m_asOwner.GetView(SearchResultsView.STR_MVC_CLASS_NAME).Display();
        myCell.AddInnerHTMLElement(resView);
      }
    }
    /*
     }
     else
     {
     myTable = new HTMLTable();
     container.AddInnerHTMLElement(myTable);
     myTable.SetWidth("100%");
     myTable.SetBorder("0");
     myTable.SetCellPadding("5");
     myTable.SetCellSpacing("0");
     
     // Add a row saying we already have the max number of targets
     myRow = new HTMLTableRow();
     myTable.AddInnerHTMLElement(myRow);
     myRow.SetStyleClass(PTStyleClass.OBJECT_ACTIONBAR_BG);
     myCell = new HTMLTableCell();
     myCell.SetNoWrap(true);
     myCell.SetColSpan("4");
     myRow.AddInnerHTMLElement(myCell);
     HTMLSpan mySpan = new HTMLSpan();
     myCell.AddInnerHTMLElement(mySpan);
     mySpan.SetStyleClass(PTStyleClass.OBJECT_TEXT);
     
     mySpan.AddInnerHTMLString(m_asOwner.GetString(169,
     ResourceConstants.PORTALADMINMSGS));
     }
     */
     
    protected virtual HTMLElement CreateDeleteElement(PTImageType imageType,
     String altText, String width, String height, bool enabled)
    {
      HTMLImg image = SearchUIUtility.GetImage(this.m_asOwner, imageType,
       altText, width, height);
      if (!enabled)
      {
        return image;
      }
      HTMLAnchor retval = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
      // mj: removed 'Javascript:' from function name
      retval.SetOnClick("CheckTargetsAndDelete();");
      retval.SetPreComment(altText);
      retval.AddInnerHTMLElement(image);
      return retval;
    }
    public override HTMLScript DisplayJavascript()
    {
      try
      {
        XPStringBuilder sb = new XPStringBuilder("");
        sb.Append("\n\nfunction DoToggleAllDeletes() {\n");
        IBestBetModelRO bbModel = (IBestBetModelRO) m_model;
        int numTargets = bbModel.GetNumTargets();
        sb.Append("    var fElem = document.mainform;\n");
        sb.Append("    var feLen = fElem.elements.length;\n");
        sb.Append("    var checkVal = false;\n");
        sb.Append("    for (var i = 0; i < feLen; i++) {\n");
        sb.Append("        if (fElem.elements[i].name == \'" + DEL_ALLTARGET_CB + "\') {\n");
        sb.Append("            checkVal = fElem.elements[i].checked;\n");
        sb.Append("        }\n");
        sb.Append("    }\n");
        sb.Append("    for (var i = 0; i < feLen; i++) {\n");
        for (int j = 1;j <= numTargets;j++)
        {
          sb.Append("    " + ((j > 1) ? "else " : "") + "if (fElem.elements[i].name == \'" + DEL_TARGET_CB + XPConvert.ToString(j) + "\') {\n");
          sb.Append("        fElem.elements[i].checked = checkVal;\n");
          sb.Append("    }\n");
        }
        sb.Append("    }\n");
        sb.Append("    return true;\n");
        sb.Append("}\n");
        sb.Append("\n\nfunction CheckTargetsAndDelete() {\n");
        sb.Append("    var fElem = document.mainform;\n");
        sb.Append("    var feLen = fElem.elements.length;\n");
        sb.Append("    var bCanDelete = false;\n");
        sb.Append("    for (var i = 0; i < feLen; i++) {\n");
        for (int j = 1;j <= numTargets;j++)
        {
          sb.Append("    " + ((j > 1) ? "else " : "") + "if (fElem.elements[i].name == \'" + DEL_TARGET_CB + XPConvert.ToString(j) + "\') {\n");
          sb.Append("        if(fElem.elements[i].checked) bCanDelete = true;\n");
          sb.Append("    }\n");
        }
        sb.Append("    }\n");
        sb.Append("    if(bCanDelete) {\n");
        sb.Append("        postToSelf(" + BestBetRepostControl.POSTTOSELF_ACTION_DELTARGS + ");\n");
        sb.Append("        return true;\n");
        sb.Append("    }\n");
        sb.Append("    else\n");
        sb.Append("    {\n");
        sb.Append("        alert(\'" + m_asOwner.GetString(2577, ResourceConstants.PORTALADMINMSGS) + "\');\n");
        sb.Append("        return false;\n");
        sb.Append("    }\n");
        sb.Append("}\n");
        HTMLScript script = new HTMLScript("text/javaScript");
        script.AddInnerHTMLString(sb.ToString());
        return script;
      }
      catch (Exception e)
      {
        throw XPException.GetInstance(e);
      }
    }
  }
}