using System;
using com.plumtree.portalpages.common.search;
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.admin.editors.group
{
  /// <summary>
  /// This DP is a take on the SavSrchMain DP by MikeS.
  /// </summary>
  /// <author>AkashJ</author>
  public class GroupDynamicRulesDP : EditorDP
  {
    public const String STR_MVC_CLASS_NAME = "GroupDynamicRules";
    
    /// <summary>
    /// javascript name for function that alerts user of portlet cache 
    /// </summary>
    public const String UPDATE_ALERT_JS = "UpdateAlert";
    
    /// <summary>
    /// Displays some HTML after the body tag and before the form tag.
    /// For example this is used by pages using a DHTML calendar to display the div tag.
    /// The function is optional.  If it exists, it will be called by the framework, and
    /// the resulting HTMLElement will be added in the right place.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public override HTMLElementCollection DisplayHTMLBeforeForm()
    {
      return FilterView.GetPagePreFormContent(m_asOwner);
    }
    
    /// <summary>
    /// Returns the String to display on the left navigation menu,
    /// linking to that page.
    /// </summary>
    /// <returns>String</returns>
    public override String GetLeftNavString()
    {
      return (m_asOwner.GetString(2711, "ptmsgs_portaladminmsgs"));
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Implement this method to add HTML below the sub header.
    /// </summary>
    /// <returns>HTMLElement</returns>
    /// <throws>HTMLException</throws>
    protected override HTMLElement GetHTMLBelowSubHeader()
    {
      return HTMLConstructHelpers.GetSubHeader(m_asOwner.GetString(2712, "ptmsgs_portaladminmsgs"),
       this.m_asOwner);
    }
    
    /// <summary>
    /// Returns the HTML for the General Info page.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AFormDP#PageDisplay(HTMLTableRow)"></seealso>
    /// <returns>HTMLElement</returns>
    public override HTMLElement PageDisplay()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      // Insert super class HTML
      result.AddInnerHTMLElement(base.PageDisplay());
      HTMLElementCollection views = new HTMLElementCollection();
      views.AddInnerHTMLElement((m_asOwner.GetView(DynamicRulesView.STR_MVC_CLASS_NAME)).Display());
      views.AddInnerHTMLElement(EditorView.GetSpacerTable());
      views.AddInnerHTMLElement((m_asOwner.GetView(DynamicMembersPreviewView.STR_MVC_CLASS_NAME)).Display());
      result.AddInnerHTMLElement(CreateLeftNavBar(views));
      return result;
    }
    
    /// <summary>
    /// Display javascript for this display page
    /// </summary>
    /// <returns></returns>
    /// <throws>HTMLException</throws>
    protected override HTMLScriptCollection DisplayJavaScriptFromChild()
    {
      HTMLScriptCollection myScripts = base.DisplayJavaScriptFromChild();
      // ***CHANGE_HERE SELECTPTOBJECT*** Display JavaScripts for the Select PTObject list.
      myScripts.AddInnerHTMLElement(m_asOwner.GetView(DynamicRulesView.STR_MVC_CLASS_NAME).DisplayJavascript());
      myScripts.AddInnerHTMLElement(m_asOwner.GetView(DynamicMembersPreviewView.STR_MVC_CLASS_NAME).DisplayJavascript());
      String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE,
       GetOwner());
      myScripts.AddInnerHTMLElement(ConfigHelper.GetJSIncluderJSComponentInclude(ConfigHelper.JSCOMPONENT_JSDATEPICKER,
       GetOwner()));
      return myScripts;
    }
  }
}