using System;
// Framework Imports
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.admin.editors.job
{
  /// <summary>
  /// The main display page for the job editor
  /// </summary>
  /// <author>JohnH</author>
  public class MainDP : EditorDP
  {
    
    /// <summary>
    /// The unique identifier for this display page 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "JobMain";
    private const String ui_DatePickerFailed = "Unable to retrieve the date picker javascript construct.  Please validate that your image server is installed correctly.";
    
    /// <summary>
    /// Displays some HTML after the body tag and before the form tag.
    /// For example this is used by pages using a DHTML calendar to display the div tag.
    /// The function is optional.  If it exists, it will be called by the framework, and
    /// the resulting HTMLElement will be added in the right place.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public override HTMLElementCollection DisplayHTMLBeforeForm()
    {
      HTMLElementCollection myHTMLBeforeForm = null;
      try
      {
        //Display the DIV tag required for the Date picker control
        IJobModelRO ptROModel = (IJobModelRO) m_asOwner.GetModel(JobModel.STR_MVC_CLASS_NAME);
        myHTMLBeforeForm = ptROModel.GetDateTimePickerControlManager().GetPagePreFormContent();
        }
      catch (Exception e)
      {
        UIError myError = new UIError(UIErrorLevel.WARNING, UIErrorScope.CURRENT_USER,
         null, ui_DatePickerFailed, "");
        myError.SetActivitySpace(m_asOwner.GetName());
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        }
      return myHTMLBeforeForm;
    }
    
    /// <summary>
    /// Returns the HTML that is placed in the HEAD section of this HTML page.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public override HTMLElementCollection DisplayHTMLinHEAD()
    {
      String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE,
       GetOwner());
      HTMLElementCollection retVal = new HTMLElementCollection();
      try
      {
        IJobModelRO ptROModel = (IJobModelRO) m_asOwner.GetModel(JobModel.STR_MVC_CLASS_NAME);
        JSDateTimePickerMgr ptDatePickerMgr = ptROModel.GetDateTimePickerControlManager();
        retVal.AddInnerHTMLElement(ptDatePickerMgr.GetJSIncludes());
      }
      catch (Exception e)
      {
        UIError myError = new UIError(UIErrorLevel.WARNING, UIErrorScope.CURRENT_USER,
         null, ui_DatePickerFailed, "");
        myError.SetActivitySpace(m_asOwner.GetName());
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        }
      //Get access to the PTDate.js file for formatting date strings
      retVal.AddInnerHTMLElement(ConfigHelper.GetJSIncluderJSComponentInclude(ConfigHelper.JSCOMPONENT_JSUTIL,
       GetOwner()));
      return retVal;
    }
    
    /// <summary>
    /// Returns the String to display on the left navigation menu,
    /// linking to that page.
    /// </summary>
    /// <returns>String</returns>
    public override String GetLeftNavString()
    {
      return (m_asOwner.GetString(501, "ptmsgs_portaladminmsgs"));
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Implement this method to add HTML below the sub header.
    /// </summary>
    /// <returns>HTMLElement</returns>
    /// <throws>HTMLException</throws>
    protected override HTMLElement GetHTMLBelowSubHeader()
    {
      return HTMLConstructHelpers.GetSubHeader(m_asOwner.GetString(1681, "ptmsgs_portaladminmsgs"),
       this.m_asOwner);
    }
    
    /// <summary>
    /// Returns the HTML for the General Info page.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AFormDP#PageDisplay(HTMLTableRow)"></seealso>
    /// <returns>HTMLElement</returns>
    public override HTMLElement PageDisplay()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      // Insert super class HTML
      result.AddInnerHTMLElement(base.PageDisplay());
      HTMLElementCollection views = new HTMLElementCollection();
      //Warn the user if the job is currently locked (and the editor is in readonly mode)
      if (((IJobModelRO) m_asOwner.GetModel(m_asOwner.m_strModelName)).GetIsEditorReadOnly())
      {
        UIError myError = new UIError(UIErrorLevel.WARNING, UIErrorScope.CURRENT_USER,
         null, m_asOwner.GetString(1846, "ptmsgs_portaladminmsgs"), "");
        myError.SetActivitySpace(m_asOwner.GetName());
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        }
      if (!((IJobModelRO) m_asOwner.GetModel(m_asOwner.m_strModelName)).IsAddOperationDisabled())
      {
        // Display the Operations list.
        views.AddInnerHTMLElement(((EditorView) m_asOwner.GetView(JobEditorAS.m_strStorageSelectPTObject)).Display());
        views.AddInnerHTMLElement(EditorView.GetSpacerTable());
      }
      views.AddInnerHTMLElement(((EditorView) m_asOwner.GetView(ScheduleView.STR_MVC_CLASS_NAME)).Display());
      //views.AddInnerHTMLElement(EditorView.GetSpacerTable());
      views.AddInnerHTMLElement(((EditorView) m_asOwner.GetView(TimeoutView.STR_MVC_CLASS_NAME)).Display());
      views.AddInnerHTMLElement(EditorView.GetSpacerTable());
      views.AddInnerHTMLElement(((EditorView) m_asOwner.GetView(LoggingView.STR_MVC_CLASS_NAME)).Display());
      result.AddInnerHTMLElement(CreateLeftNavBar(views));
      return result;
    }
    
    /// <summary>
    /// Display the Javascripts for the Views.
    /// </summary>
    /// <returns>HTMLScriptCollection</returns>
    /// <throws>HTMLException</throws>
    protected override HTMLScriptCollection DisplayJavaScriptFromChild()
    {
      HTMLScriptCollection myScripts = base.DisplayJavaScriptFromChild();
      myScripts.AddInnerHTMLElement(m_asOwner.GetView(ScheduleView.STR_MVC_CLASS_NAME).DisplayJavascript());
      if (!((IJobModelRO) m_asOwner.GetModel(m_asOwner.m_strModelName)).IsAddOperationDisabled())
      {
        myScripts.AddInnerHTMLElement(m_asOwner.GetView(JobEditorAS.m_strStorageSelectPTObject).DisplayJavascript());
        }
      return myScripts;
    }
  }
}