using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.portalpages.common.editpage;
using com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes;
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory;
using com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity;
using com.plumtree.portaluiinfrastructure.layout;
using com.plumtree.portaluiinfrastructure.resultwrapper;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.tips;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.admin.editors.page
{
  /// <author>michaeld</author>
  public class PageEditorModel : ObjEditorModel,IPageEditorModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
     "com.plumtree.portalpages.admin.editors.page.PageEditorModel");
    
    /// <summary>
    /// Object Name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "PageEditorModel";
    private bool m_bIsDefaultProfileMyPage = false;
    private bool m_bIsMyPage = false;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new PageEditorModel();
    }
    
    /// <summary>
    /// Code executed after the object is stored.
    /// </summary>
    public override void DoTaskAfterStore()
    {
      int[][] annPortletOrder = ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).GetColumnOrder();
      for (int i = PortletColumnConstants.NARROW_ONE.ToInt();i <= PortletColumnConstants.NARROW_TWO.ToInt();
      i++)
      {
        ((IPTPage) m_ptObject).AssignPortlets(annPortletOrder[i], i);
      }
      int nFreeFormID = ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).FreeFormToBeRemove();
      if (nFreeFormID != -1)
      {
        try
        {
          ((IPTPage) m_ptObject).RemovePortlet(nFreeFormID);
        }
        catch (Exception e)
        {
          // SE: Bug 61272 - Guard against removing portlets that are not associated with the page.
          log.Warn("PageEditorModel::DoTaskAfterStore - An exception occurred when attempting to remove freeform gadget: " + e.Message);
          }
      }
      if (((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).GetHasFreeFormPortlet())
      {
        try
        {
          // SE: Bug 60903 - Fix for Vodafone and USDHHS. Check to see if free form portlet is from page template. Only add it if it 
          // is not from template. This was causing an issue with Community pages not properly inheriting the content canvas 
          // portlet.
          int nPageTemplateID = ((IPTPage) m_ptObject).GetPageTemplateID();
          int nFreeFormPortletID = ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).GetFreeFormPortletID();
          bool bIsFromTemplate = false;
          // Check if page inherits from page template
          if (((IPTPage) m_ptObject).GetInheritPageTemplate())
          {
            if (0 != nPageTemplateID)
            {
              // Get portlets from page template
              IPTPageTemplate ptPageTemplate = (IPTPageTemplate) m_Session.GetPageTemplates().Open(nPageTemplateID,
               false);
              ASQueryResultWrapper qrw = new ASQueryResultWrapper(ptPageTemplate.QueryGadgets());
              IASQueryResult ptqrPortlets = qrw;
              int nTemplatePortletID;
              if (null != ptqrPortlets)
              {
                for (int i = 0;i < ptqrPortlets.GetCount();i++)
                {
                  nTemplatePortletID = XPConvert.ToInteger(ptqrPortlets.GetFields(i,
                   PT_PROPIDS.PT_PROPID_OBJECTID));
                  if (nTemplatePortletID == nFreeFormPortletID)
                  {
                    bIsFromTemplate = true;
                    log.Debug("PageEditorModel::DoTaskAfterStore - freeform portlet ID: " + nFreeFormPortletID + " is from page template, omit adding free form portlet");
                    break;
                  }
                }
              }
            }
            if (!bIsFromTemplate)
            {
              ((IPTPage) m_ptObject).AddPortlet(nFreeFormPortletID, PortletColumnConstants.FREE_FORM.ToInt());
              log.Debug("PageEditorModel::DoTaskAfterStore - Added freeform portlet ID: " + nFreeFormPortletID);
              }
          } else
          {
            // The page is not inheriting from page template, just add the portlet in this case.
            ((IPTPage) m_ptObject).AddPortlet(nFreeFormPortletID, PortletColumnConstants.FREE_FORM.ToInt());
            }
        }
        // DCA - already added freeform gadget
        catch (Exception e)
        {
          // DCA - Don't really have to do anything
          log.Warn("PageEditorModel::DoTaskAfterStore - An exception occurred when adding freeform gadget: " + e.Message);
          }
      }
    }
    
    /// <summary>
    /// Initializes IPTProperty with the property type member  variable just
    /// before we store the object in the DB. Also sets 2 other values to
    /// default.
    /// </summary>
    public override void DoTaskBeforeStore()
    {
      // Get the settings back from the plug-in Model and sets them on the Object before the Framework stores it.
      if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
      {
      }
      // Set page type
      int nPageType = ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).GetPageType();
      // TODO-MJD : don't do this if we are inheriting page template
      if (nPageType != ((IPTPage) m_ptObject).GetPageType())
      {
        ((IPTPage) m_ptObject).SetPageType(nPageType);
      }
    }
    
    /// <summary>
    /// Override the StartLocNamesList, because we don't want to initialize that page until the page template
    /// has been set on the page.
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.editor.ObjEditorModel#StartLocNamesList()"></seealso>
    public override void StartLocNamesList()
    {
    }
    
    /// <summary>
    /// We will call this after the page template has been set on the page, to initialize the localized name page.
    /// </summary>
    public virtual void StartLocNamesList2()
    {
      base.StartLocNamesList();
    }
    
    /// <summary>
    /// Initializes the property type member variable with  the one from
    /// IPTProperty in Edit mode and the default  value in create mode.
    /// </summary>
    /// <param name="_htFormData"></param>
    public override void DoTaskOnStartEditor(XPHashtable _htFormData)
    {
      int nPageLayoutType;
      int nPageTemplateID;
      bool bInheritPageTemplate;
      IASQueryResult ptqrPortlets = null;
      bool bIgnorePortletFromPageTemplate = true;
      m_bIsMyPage = (0 > m_nFolderID) ? true : false;
      m_bIsDefaultProfileMyPage = false;
      int nDefaultProfileID = HTTPHelpers.GetArgumentAsInt(_htFormData, PTPages.PAGE_EDITOR_QS_INT_DEFAULT_PROFILE_ID,
       0);
      if (0 != nDefaultProfileID)
      {
        // TODO-MJD : need to add in PEI's for my pages here.
        m_bIsDefaultProfileMyPage = true;
        m_bIsMyPage = true;
        HTMLGenericElement e = new HTMLGenericElement(m_asOwner.GetString(150,
         "ptmsgs_tips"));
        ((TipsModel) m_asOwner.GetModel(TipsModel.STR_MVC_CLASS_NAME)).SetTip(e);
        }
      if (EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType())
      {
        // TODO-MJD : need to add in PEIs for community pages here
        nPageLayoutType = ((IPTPage) m_ptObject).GetPageType();
        nPageTemplateID = ((IPTPage) m_ptObject).GetPageTemplateID();
        bInheritPageTemplate = ((IPTPage) m_ptObject).GetInheritPageTemplate();
        ASQueryResultWrapper qrw = new ASQueryResultWrapper(((IPTPage) m_ptObject).QueryPortlets());
        ptqrPortlets = qrw;
        bIgnorePortletFromPageTemplate = false;
        ((EditorAS) m_asOwner).GetLeftMenu().HideLink(false, ObjectSecurityDP.STR_MVC_CLASS_NAME);
        if (m_bIsDefaultProfileMyPage)
        {
          ((EditorAS) m_asOwner).GetLeftMenu().HideLink(true, ObjectSecurityDP.STR_MVC_CLASS_NAME);
          ((EditorAS) m_asOwner).GetLeftMenu().HideLink(true, MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
          }
        if (0 > m_nFolderID)
        {
          if (m_Session.GetSessionInfo().GetCurrentMyPage(m_nObjectID).GetIsFirstPageInContainer())
          {
            // Set is home page
            ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).SetIsHomeMyPage(true);
            }
        }
      } else
      {
        // TODO-MJD : need to add in PEIs for community pages here
        // MJD - This is the page type.
        // BT: Use default portlet layout if portlet layouts are enabled.
        if (LayoutService.getChooserInstance().GetPortletLayoutsEnabled(m_asOwner))
        {
          // adaptive layout stores a negative value in the page type (negative value of a webservice layout id)
          nPageLayoutType = -PT_INTRINSICS.PT_WEBSERVICE_DEFAULT_PORTLET_LAYOUT;
          } else
        {
          nPageLayoutType = PageTypeConstants.TWO_COLUMN_NARROW_WIDE;
        }
        nPageTemplateID = HTTPHelpers.GetArgumentAsInt(_htFormData, PTPages.PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_OBJECT_ID,
         0);
        bInheritPageTemplate = HTTPHelpers.GetArgumentAsBoolean(_htFormData,
         PTPages.PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_INHERIT, (0 != nPageTemplateID));
        // SE: 60054 - Fix for Vodafone. Check if portlets are from page templates if we are
        // inheriting from a page template when creating a community page.
        if (bInheritPageTemplate)
        {
          bIgnorePortletFromPageTemplate = false;
        } else
        {
          bIgnorePortletFromPageTemplate = true;
        }
        ((IPTPage) m_ptObject).SetPageTemplateID(nPageTemplateID);
        if (0 != nPageTemplateID)
        {
          ((IPTPage) m_ptObject).SetDefaultsFromPageTemplate(nPageTemplateID);
          } else
        {
          ((IPTPage) m_ptObject).SetPageType(nPageLayoutType);
        }
        ((IPTPage) m_ptObject).SetInheritPageTemplate(bInheritPageTemplate);
        if (0 != nPageTemplateID)
        {
          // Get portlets from page template
          IPTPageTemplate ptPageTemplate = (IPTPageTemplate) m_Session.GetPageTemplates().Open(nPageTemplateID,
           false);
          ASQueryResultWrapper qrw = new ASQueryResultWrapper(ptPageTemplate.QueryGadgets());
          ptqrPortlets = qrw;
        }
        ((EditorAS) m_asOwner).GetLeftMenu().HideLink(true, ObjectSecurityDP.STR_MVC_CLASS_NAME);
        ((EditorAS) m_asOwner).GetLeftMenu().HideLink(true, MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
        if (0 > m_nFolderID)
        {
          if (0 == m_Session.GetSessionInfo().GetCurrentMyPages().RowCount())
          {
            // Set is home page
            ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).SetIsHomeMyPage(true);
            }
        }
      }
      // reupdate the names and properties page
      StartLocNamesList2();
      ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).SetOpenerClassID(PT_CLASSIDS.PT_PAGE_ID);
      ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).SetOpenerObjectID(m_nObjectID);
      if (0 > m_nFolderID)
      {
        ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).SetAllowFreeFormSection(false);
        } else
      {
        ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).SetAllowFreeFormSection(true);
        }
      ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).InitializePlugIn((IPTPage) m_ptObject,
       400, ptqrPortlets, bIgnorePortletFromPageTemplate, m_Session);
      ((PageTemplateListModel) m_asOwner.GetModel(PageEditorAS.PLUGIN_PAGE_TEMPLATE_LIST)).InitializePageTemplateList(m_Session,
       ((IPTPage) m_ptObject).GetPageTemplateID(), ((IPTPage) m_ptObject).GetInheritPageTemplate(),
       63);
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Calls the right save function according to the page name passed in
    /// argument.
    /// </summary>
    /// <param name="_sPageName"></param>
    /// <param name="_htFormData"></param>
    /// <returns>int Page status for server side validation</returns>
    public override int SavePage(String _sPageName, XPHashtable _htFormData)
    {
      int nPageStatus = base.SavePage(_sPageName, _htFormData);
      if (_sPageName.Equals(PageMainDP.STR_MVC_CLASS_NAME))
      {
        ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).SaveSection(_htFormData);
        }
      return nPageStatus;
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.page.IPageEditorModelRO#GetIsMyPage()"></seealso>
    public virtual bool GetIsMyPage()
    {
      return m_bIsMyPage;
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.page.IPageEditorModelRO#GetIsDefaultProfile()"></seealso>
    public virtual bool GetIsDefaultProfileMyPage()
    {
      return m_bIsDefaultProfileMyPage;
    }
    
    /// <summary>
    /// This method will be called when you come back to the main Editor from a sub Editor.
    /// </summary>
    public override void DoTaskOnComeBackFromSubEditor()
    {
      base.DoTaskOnComeBackFromSubEditor();
      ((PageLayoutPlugInModel) m_asOwner.GetModel(PageLayoutPlugInModel.STR_MVC_CLASS_NAME)).RepostNewPortlet();
      }
  }
}