using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.admin.editors.portlet;
using com.plumtree.portalpages.browsing.gateway;
using com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework;
using com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.marshalers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.tree;
namespace com.plumtree.portalpages.admin.editors.portlet.plugins
{
  /// <author>adolfoc</author>
  public class SelectWebServiceModel : EditorHelperModel,ISelectWebServiceModelRO,
  IPSubModel
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
     "com.plumtree.portalpages.admin.editors.portlet.plugins.SelectWebServiceModel");
    public const String STR_MVC_CLASS_NAME = "SelectWebServiceModel";
    
    /// <summary>
    /// This member defines a static string used to identify when m_htFormData
    /// has been initialized.
    /// </summary>
    public const String DEFAULTDATA = "Default Data";
    public const int NO_ACCESS_WS = -1;
    
    /// <summary>
    /// Our portlet - handed to us in InitPModel
    /// </summary>
    private IPTGadget m_ptPortlet = null;
    
    /// <summary>
    /// The following member holds a pointer to the Web Service object that
    /// this Portlet depends on. NOTE: The Web Service can change at any
    /// time and is NOT necessarily up-to-date. Do not use this member
    /// directly, but call GetWSObject() to get an up-to-date Web Service.
    /// </summary>
    private IPTWebService m_ptWebService = null;
    
    /// <summary>
    /// This member contains form data sent to this control (used for the Tree control, mostly).
    /// </summary>
    private XPHashtable m_htFormData;
    
    /// <summary>
    /// The boolean which tells our view whether or not it should display
    /// read-only. This value is forced to yes if the WebService is every null.
    /// This way the user gets a chance to reset it if it was null when the editor
    /// started.
    /// </summary>
    private bool m_bForceWritable = false;
    
    /// <summary>
    /// This member is used in determining whether or not the web service changed - whether
    /// or not we need to apprise our peers of the change.
    /// </summary>
    private int m_nOldWebServiceID = 0;
    
    /// <summary>
    /// This member is used to store the portlet class if the portlet is an intrinsic portlet.
    /// </summary>
    private String m_strIntrinsicPortletClass = null;
    public SelectWebServiceModel() : base()
    {
      m_htFormData = new XPHashtable();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SelectWebServiceModel();
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.portlet.IPSubModel#DoTaskBeforePStore()"></seealso>
    public virtual void DoTaskBeforePStore()
    {
      // DCA - check that the folder the web service is in isn't the page layouts folder
      if (m_ptWebService.GetAdminFolderID() == PT_INTRINSICS.PT_ADMIN_FOLDER_PAGE_LAYOUTS)
      {
        // DCA - this web service is a page or portlet layout, it should not be used as a portlet
        String strError = XPResourceManager.FormatMessage(m_asOwner.GetString(3221,
         "ptmsgs_portaladminmsgs"), m_ptWebService.GetName());
        UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
         null, strError, strError);
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        throw new XPException(strError);
      } else
      {
        // DCA - also check that none of the parent folders is the page layouts folder
        IPTAdminCatalog ptCatalog = ((IPTSession) m_asOwner.GetUserSession()).GetAdminCatalog();
        IPTAdminFolder ptFolder = ptCatalog.OpenAdminFolder(m_ptWebService.GetAdminFolderID(),
         false);
        Object[] parentIds = ptFolder.GetParentIDs();
        for (int i = 0;i < parentIds.Length;i++)
        {
          if (XPConvert.ToInteger(parentIds[i]) == PT_INTRINSICS.PT_ADMIN_FOLDER_PAGE_LAYOUTS)
          {
            // DCA - this web service is a page or portlet layout, it should not be used as a portlet
            String strError = XPResourceManager.FormatMessage(m_asOwner.GetString(3221,
             "ptmsgs_portaladminmsgs"), m_ptWebService.GetName());
            UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
             null, strError, strError);
            UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
             m_asOwner.GetApplication());
            throw new XPException(strError);
          }
        }
      }
      m_ptPortlet.SetWebServiceID(m_ptWebService.GetObjectID());
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.portlet.IPSubModel#DoTaskOnWSModeUpdate(int)"></seealso>
    public virtual void DoTaskOnPModeUpdate(int nNewWSMode)
    {
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.portlet.IPSubModel#DoTaskOnWebServiceUpdate(String,String)"></seealso>
    public virtual void DoTaskOnWebServiceUpdate(IPTWebService ptWebService)
    {
    }
    // AGC- This model initiates this change - it doesn't consume it.
    
    /// <seealso cref="com.plumtree.portalpagessample.form.ISampleFormModelRO#GetData(String)"></seealso>
    public virtual String GetData(String name)
    {
      String[] values = (String[]) m_htFormData.GetElement(name);
      if (values == null)
      {
        return DEFAULTDATA;
      }
      return values[0];
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Call this method to set the WebServiceObject to a new one.
    /// </summary>
    public virtual IPTWebService GetWSObject()
    {
      return m_ptWebService;
    }
    
    /// <summary>
    /// Returns the name value from m_strWebServiceTreeData.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetWebServiceName()
    {
      if (null == m_ptWebService)
      {
        return null;
      }
      return m_ptWebService.GetLocalizedName();
    }
    
    /// <summary>
    /// Returns the selected web service ID.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetWebServiceID()
    {
      return m_nOldWebServiceID;
    }
    
    /// <summary>
    /// Returns the path value from m_strWebServiceTreeData.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetWebServicePath()
    {
      if (null == m_ptWebService)
      {
        return null;
      }
      int nAdminFolderID = m_ptWebService.GetAdminFolderID();
      PortletModel mParentModel = (PortletModel) m_asOwner.GetModel(PortletModel.STR_MVC_CLASS_NAME);
      IPTSession ptSession = mParentModel.GetPTSession();
      IPTAdminCatalog ptAFCatalog = ptSession.GetAdminCatalog();
      IPTAdminFolder ptAFParent = ptAFCatalog.OpenAdminFolder(nAdminFolderID,
       false);
      String strPath = ptAFParent.GetPath();
      return strPath;
    }
    
    /// <summary>
    /// Return intrinsic portlet class
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetIntrinsicPortletClass()
    {
      return m_strIntrinsicPortletClass;
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.portlet.IPSubModel#InitPModel(IPTWebService)"></seealso>
    public virtual void InitPModel(IPTGadget ptPortlet)
    {
      m_ptPortlet = ptPortlet;
      PortletModel mParentModel = (PortletModel) m_asOwner.GetModel(PortletModel.STR_MVC_CLASS_NAME);
      int lWebServiceObjectID;
      m_bForceWritable = false;
      switch (mParentModel.GetPMode()){
      case PortletModel.PMODE_CREATE:
      m_bForceWritable = true;
      // AGC- We may have been passed in a WebServiceID by a popup tree when
      // we were created. If so, let's get it and select it.
      if (PT_CLASSIDS.PT_WEBSERVICE_ID == mParentModel.GetCreatorClassID())
      {
        lWebServiceObjectID = mParentModel.GetCreatorID();
        SetWSObject(lWebServiceObjectID);
      }
      break;
      case PortletModel.PMODE_PRECREATE:
      break;
      default:
      lWebServiceObjectID = ptPortlet.GetWebServiceID();
      try
      {
        IPTSession ptSession = mParentModel.GetPTSession();
        IPTWebService ptWebServiceToSet = (IPTWebService) ptSession.GetWebServices().Open(lWebServiceObjectID,
         false);
        m_ptWebService = ptWebServiceToSet;
        // Don't forget to update the Old Web service ID.
        m_nOldWebServiceID = lWebServiceObjectID;
      }
      catch (PTException e)
      {
        if (e.GetErrorCode() == -2147024891)
        {
          // user does not have access to the portlet's web service
          log.Warn(e, "SelectWebService::InitPModel User does not have access to Web Service.");
          m_ptWebService = null;
          m_nOldWebServiceID = NO_ACCESS_WS;
        } else
        {
          log.Error("SelectWebService::InitPModel Could not retrieve webservice from portlet. View will be non-read-only" + e.GetMessage());
          // AGC- We only force writable if (for some reason) the
          // WebService could not be found/set - which might happen if
          // we are imported from a broken object delivery package.
          m_bForceWritable = true;
        }
      }
      break;
      }
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.plugins.ISelectWebServiceModelRO#IsReadOnly()"></seealso>
    public virtual bool IsReadOnly()
    {
      // AGC- It may come about that our WebService is false because of some unpleasantness (like
      // a bogus import) We'll be nice and allow the user to fix it.
      if (null == m_ptWebService)
      {
        m_bForceWritable = true;
      }
      if (m_bForceWritable)
      {
        return false;
      }
      return true;
    }
    
    /// <summary>
    /// This method gets the selected items from the tree.
    /// </summary>
    public virtual void RefreshItemFromTree()
    {
      try
      {
        if (m_asOwner.GetSubSpaceExists(PortletEditorAS.WSTREE_SUBSPACE))
        {
          AActivitySpace asTree = m_asOwner.GetSubSpace(PortletEditorAS.WSTREE_SUBSPACE);
          ITreeModelRO treeModel = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
          XPArrayList itemList = treeModel.GetSelectedItems();
          if (itemList.GetSize() != 1)
          {
            log.Error("Tree selected wrong number of items.  Expecting 1, got " + itemList.GetSize());
            }
          TreeLeafNode item = (TreeLeafNode) itemList.GetElement(0);
          if (item == null)
          {
            log.Error("Tree selected item null");
          } else
          {
            SetWSObject(item.m_nObjectID);
          }
        }
      }
      catch (Exception e)
      {
        // This may simply be because the tree was cancelled with no node set.
        log.Warn(e, "Unknown Error RefreshItemFromTree.");
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.editor.EditorHelperModel#SaveSection(XPHashtable)"></seealso>
    public override int SaveSection(XPHashtable _htFormData)
    {
      if (!IsReadOnly())
      {
        // Store our form data for later use - especially for use in the tree control.
        int nSize = _htFormData.GetSize();
        IXPEnumerator xpKeys = _htFormData.GetKeys();
        for (int i = 0;i < nSize;i++)
        {
          xpKeys.MoveNext();
          m_htFormData.PutElement((String) xpKeys.GetCurrent(), (String[]) _htFormData.GetElement(xpKeys.GetCurrent()));
          }
        RefreshItemFromTree();
        if (null != m_ptWebService)
        {
          return RepostControl.PAGE_STATUS_VALID;
        } else
        {
          // if the WS is null just bc the user can't access it, then that is fine.
          if (m_nOldWebServiceID != NO_ACCESS_WS)
          {
            UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
             null, m_asOwner.GetString(594, "ptmsgs_portaladminmsgs"), m_asOwner.GetString(594,
             "ptmsgs_portaladminmsgs"));
            UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
             m_asOwner.GetApplication());
            return RepostControl.PAGE_STATUS_INVALID;
          }
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Call this method to set the WebServiceObject to a new one.
    /// Returns true if the update was successful AND the webservice was changed.
    /// Returns false if the update failed OR the new Webservice was identical to
    /// the existing web service.
    /// </summary>
    private bool SetWSObject(int nObjectID)
    {
      PortletModel mParentModel = (PortletModel) m_asOwner.GetModel(PortletModel.STR_MVC_CLASS_NAME);
      if (0 >= nObjectID)
      {
        m_ptWebService = null;
      }
      bool bUpdated = false;
      try
      {
        // Let's only make the change if necessary
        if (m_nOldWebServiceID != nObjectID)
        {
          // Oh my, it changed.
          IPTSession ptSession = mParentModel.GetPTSession();
          IPTWebService ptWebServiceToSet = (IPTWebService) ptSession.GetWebServices().Open(nObjectID,
           false);
          // The 0 in the following statement tells the object manager not to lock the object.
          String strProviderID = XPStringUtility.ToUpperCase(ptWebServiceToSet.GetProviderCLSID());
          if (((PortletModel) m_asOwner.GetModel(PortletModel.STR_MVC_CLASS_NAME)).GetEditorType() == EditorStartControl.EDITOR_START_FLAG_CREATE)
          {
            // MES- CR: Is reinitialization of Portlets supported?
            // AGC- Yes. Specifically allowed so that we can do this.
            m_ptPortlet.Initialize(strProviderID);
          }
          m_ptWebService = ptWebServiceToSet;
          // Update portlet class
          if (PortletModel.IGP_CLSID.Equals(strProviderID))
          {
            IXPPropertyBag ptProviderInfo = (IXPPropertyBag) m_ptWebService.GetProviderInfo();
            m_strIntrinsicPortletClass = PortletModelHelper.PBReadString(ptProviderInfo,
             AExtensibilityConstants.PTC_I_GADGET_CLASSID, "");
          }
          // Don't forget to update the Old Web service ID.
          m_nOldWebServiceID = nObjectID;
          bUpdated = true;
          // We must tell our peers of the change. It may have significant consequences.
          mParentModel.UpdatePSubModels();
        }
      }
      catch (Exception e)
      {
        log.Error(e, "PortletData::GetWSObject - Unable to change Web Service object. Continuing with existing Web Service (which may be null): " + e.Message);
        }
      return bUpdated;
    }
    
    /// <summary>
    /// Steps to generating an AdminPrefsURL:
    /// #0 Web Service Selected? If not, just return null.
    /// #1 Remote. Force all of the views to PreStore their data (the gateway needs an up-to-date portlet to work from).
    /// #2 Do we even have an AdminPrefsURL? (make sure we check if we are editing a template or not):
    /// x. Nope, return null:
    /// #3 Intrinsic or Non? (Check the PMODE)
    /// x. If intrinsic, construct URL from info stored in web service object.
    /// #4 Put the darned portlet on the Session for the Gateway to get to.
    /// #5 Ask the gateway to generate an AdminPrefsURL for us and return it.
    /// </summary>
    public virtual String GetAdminPrefsURL()
    {
      // Step 0, Do we even have a web service?
      PortletModel mParentModel = (PortletModel) m_asOwner.GetModel(PortletModel.STR_MVC_CLASS_NAME);
      IPTWebService ptWebService = mParentModel.GetWebService();
      if (null == ptWebService)
      {
        return null;
      }
      // Step 1. Force our peers to update the portlet object.
      try
      {
        mParentModel.PreStorePSubModels();
      }
      catch (Exception e)
      {
        log.Error(e, "PortletData::PreStorePSubModels() Unknown Error." + e.Message);
        }
      // Step 2. Do we have an AdminPrefsURL even, else return null.
      if (mParentModel.IsTemplate())
      {
        if (0 == (PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_ADMINTEMPLATE & ptWebService.GetSupportsPersonalization()))
        {
          return null;
        }
      } else
      {
        if (0 == (PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_ADMINISTRATOR & ptWebService.GetSupportsPersonalization()))
        {
          return null;
        }
      }
      // Step 3, Intrinsic?
      if (IsIntrinsicPortlet(ptWebService))
      {
        // x. Yes, intrinsic.
        IXPPropertyBag ptProviderInfo = (IXPPropertyBag) ptWebService.GetProviderInfo();
        m_strIntrinsicPortletClass = PortletModelHelper.PBReadString(ptProviderInfo,
         AExtensibilityConstants.PTC_I_GADGET_CLASSID, "");
        String strPrefsSpace;
        if (mParentModel.IsTemplate())
        {
          strPrefsSpace = PortletModelHelper.PBReadString(ptProviderInfo,
           AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_TEMPLT_SPACE, "");
          } else
        {
          strPrefsSpace = PortletModelHelper.PBReadString(ptProviderInfo,
           AExtensibilityConstants.PTC_I_GADGET_ADMINPREFS_SPACE, "");
        }
        String strSubSpaceKey = m_strIntrinsicPortletClass + IntrinsicPortletPrefsUtils.SUBSPACE_KEY_SUFFIX;
        // Construct URL to the admin prefs sub-editor
        ASURL myURL = ((EditorAS) m_asOwner).GoToSubEditor(strPrefsSpace,
         strSubSpaceKey, EditorStartControl.EDITOR_START_FLAG_MISC, null,
         null);
        return myURL.GetURLAsString();
      }
      // Step 4: Put the portlet on the Session for the Gateway
      m_asOwner.GetSubSession().SetAttribute(GatewayUtils.GetSavedPortletName(),
       m_ptPortlet);
      // Step 5: Ask the Gateway for an AdminPrefsURL of reasonable construction and return it.
      //JD- The URL is different for portlets and portlet templates.
      String strAdminPrefsURL = null;
      if (mParentModel.IsTemplate())
      {
        strAdminPrefsURL = GatewayHelpers.ConstructTemplateAdminPrefsLink(m_asOwner,
         true);
      } else
      {
        strAdminPrefsURL = GatewayHelpers.ConstructAdminPrefsLink(m_asOwner);
        }
      return strAdminPrefsURL;
    }
    
    /// <summary>
    /// Returns true if portlet is an intrinsic portlet
    /// </summary>
    /// <returns>True if portlet is intrinsic. False otherwise.</returns>
    private bool IsIntrinsicPortlet(IPTWebService ptWebService)
    {
      if (null == ptWebService)
      {
        return false;
      }
      if (0 == XPStringUtility.CompareIgnoreCase(PortletModel.IGP_CLSID, ptWebService.GetProviderCLSID()))
      {
        return true;
      }
      return false;
    }
  }
}