using System;
using com.plumtree.portalpages.admin.editors.webservice.plugins;
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.expandablelist;
namespace com.plumtree.portalpages.admin.editors.webservice
{
  /// <author>adolfoc</author>
  public class WebServiceRepostControl : ObjEditorRepostControl
  {
    public const String STR_MVC_CLASS_NAME = "WebServiceRepost";
    
    /// <summary>
    /// Repost that occurs when the Remote Server is chosen via Tree Section 
    /// </summary>
    public const int POSTTOSELF_ACTION_REMOTE_VIEW_CHOSEN = 66;
    
    /// <summary>
    /// Repost that occurs when the WebService Type is chosen via Radio Button 
    /// </summary>
    public const int POSTTOSELF_ACTION_WSTYPE_CHOSEN = 67;
    
    /// <summary>
    /// Repost that occurs when the WebService Type is removed (not via Tree Section)
    /// </summary>
    public const int POSTTOSELF_ACTION_REMOTE_VIEW_REMOVED = 68;
    
    /// <summary>
    /// Repost that occurs when the credential vault is removed 
    /// </summary>
    public const int POSTTOSELF_ACTION_CRED_VAULT_REMOVED = 69;
    
    /// <summary>
    /// Repost that occurs when the debug user is removed 
    /// </summary>
    public const int POSTTOSELF_ACTION_DEBUG_USER_REMOVED = 70;
    
    /// <summary>
    /// Repost that occurs when the debug portlet is removed 
    /// </summary>
    public const int POSTTOSELF_ACTION_DEBUG_PORTLET_REMOVED = 71;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new WebServiceRepostControl();
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Switch between the action modes and call
    /// the associated method for each action.
    /// </summary>
    /// <param name="_nAction"></param>
    protected override void PerformAction(int _nAction)
    {
      base.PerformAction(_nAction);
      // We need to call the PerformRepostAction on the plug-in control
      ((ExpListControl) m_asOwner.GetControl(WebServiceEditorAS.strStorage_UserPrefsListPluginView)).PerformRepostAction(_nAction,
       m_htFormData);
      // We need to call the PerformRepostAction on the plug-in control
      ((ExpListControl) m_asOwner.GetControl(WebServiceEditorAS.strStorage_CommunityPrefsListPluginView)).PerformRepostAction(_nAction,
       m_htFormData);
      // We need to call the PerformRepostAction on the plug-in control
      ((ExpListControl) m_asOwner.GetControl(WebServiceEditorAS.strStorage_SessionPrefsListPluginView)).PerformRepostAction(_nAction,
       m_htFormData);
      // We need to call the PerformRepostAction on the plug-in control
      ((ExpListControl) m_asOwner.GetControl(WebServiceEditorAS.strStorage_ActSecurityPluginView)).PerformRepostAction(_nAction,
       m_htFormData);
      // We need to call the PerformRepostAction on the plug-in control
      ((ExpListControl) m_asOwner.GetControl(WebServiceEditorAS.strStorage_GatewayPrefixPluginView)).PerformRepostAction(_nAction,
       m_htFormData);
      // We need to call the PerformRepostAction on the plug-in control
      ((ExpListControl) m_asOwner.GetControl(WebServiceEditorAS.strStorage_AddUserInfoPluginView)).PerformRepostAction(_nAction,
       m_htFormData);
      // aj: repost control for the form field login view
      ((ExpListControl) m_asOwner.GetControl(WebServiceEditorAS.strStorage_FormLoginFieldsPluginView)).PerformRepostAction(_nAction,
       m_htFormData);
      switch (_nAction){
      //AGC- The one view that affects all others is the Remote Server selection view.
      //if the Remote Server changes or is removed, all of the other views must be notified.		
      case POSTTOSELF_ACTION_REMOTE_VIEW_REMOVED:
      // As a favor to the Remote Server Model, we'll call the ClearRemoteServer method. To be properly
      // graceful, we should farm out this task to a control specifically associated with the remote
      // server plugin. But it's not really worth it for a single method.			
      ((RemoteServerPluginModel) m_asOwner.GetModel(WebServiceEditorAS.strStorage_RemoteServerPluginView)).ClearRemoteServer();
      goto case POSTTOSELF_ACTION_REMOTE_VIEW_CHOSEN;
      case POSTTOSELF_ACTION_REMOTE_VIEW_CHOSEN:
      // Save page is called before we get here - it may error that the package is not
      // up-to-date (or that the remote server's not selected). It shouldn't do that.
      UIErrorHandler.ClearErrors(m_asOwner.GetSubSession(), m_asOwner.GetApplication());
      goto case 
      // AGC- the following event occurs if the WSTYPE is changed - mid editor. Under normal circumstances
      // this should not happen. However, this can happen if the Choose Webservice Type view is enabled, which might
      // be done for debugging purposes.
      POSTTOSELF_ACTION_WSTYPE_CHOSEN;
      case POSTTOSELF_ACTION_WSTYPE_CHOSEN:
      // Refresh all of our views...
      ((WebServiceModel) m_asOwner.GetModel(WebServiceModel.STR_MVC_CLASS_NAME)).UpdateWSSubModels();
      break;
      case POSTTOSELF_ACTION_CRED_VAULT_REMOVED:
      // As a favor to the Remote Server Model, we'll call the ClearRemoteServer method. To be properly
      // graceful, we should farm out this task to a control specifically associated with the remote
      // server plugin. But it's not really worth it for a single method.			
      ((CredentialVaultPluginModel) m_asOwner.GetModel(WebServiceEditorAS.strStorage_CredentialVaultPluginView)).ClearLockbox();
      break;
      case POSTTOSELF_ACTION_DEBUG_USER_REMOVED:
      //Call the ClearUser method on the model.
      ((DebugUserPluginModel) m_asOwner.GetModel(WebServiceEditorAS.strStorage_DebugUserPluginView)).ClearUser();
      break;
      case POSTTOSELF_ACTION_DEBUG_PORTLET_REMOVED:
      //Call the ClearPortlet method on the model.
      ((DebugPortletPluginModel) m_asOwner.GetModel(WebServiceEditorAS.strStorage_DebugPortletPluginView)).ClearPortlet();
      break;
      }
    }
  }
}