using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portalpages.admin.editors.webservice;
using com.plumtree.portaluiinfrastructure.resultwrapper;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.form;
namespace com.plumtree.portalpages.admin.editors.webservice.plugins
{
  /// <author>adolfoc</author>
  public class AssocObjectsPluginModel : EditorHelperModel,IAssocObjectsPluginModelRO,
  IWSSubModel
  {
    public const String STR_MVC_CLASS_NAME = "AssocObjectsPlugin";
    
    /// <summary>
    /// The Web Service object being edited by our parent Web Service editor
    /// </summary>
    internal IPTWebService m_ptWebService = null;
    
    /// <summary>
    /// WSMode setting. We use this to determine which checkboxes to display
    /// and store.
    /// </summary>
    private int m_nWSMode;
    public AssocObjectsPluginModel() : base()
    {
      m_nWSMode = WebServiceModel.WSMODE_NONE;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new AssocObjectsPluginModel();
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskBeforeWSStore()"></seealso>
    public virtual void DoTaskBeforeWSStore()
    {
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnRemoteServerUpdate(String,String,String)"></seealso>
    public virtual void DoTaskOnRemoteServerUpdate(String strRemoteServerID,
     String strRemoteServerName, String strRemoteServerBaseURL)
    {
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnWSModeUpdate(int)"></seealso>
    public virtual void DoTaskOnWSModeUpdate(int nNewWSMode)
    {
      m_nWSMode = nNewWSMode;
    }
    
    /// <summary>
    /// Gets Objects Associated with the current object. Only works for Portlets and Web Services. 
    /// </summary>
    /// <param name="nCLASSID">The classid of the objects to return</param>
    /// <returns>IASQueryResult</returns>
    public virtual IASQueryResult GetAssocObjects(int nCLASSID)
    {
      WebServiceModel mParent = (WebServiceModel) m_asOwner.GetModel(WebServiceModel.STR_MVC_CLASS_NAME);
      if (EditorStartControl.EDITOR_START_FLAG_CREATE == mParent.GetEditorType())
      {
        return null;
      } else
      {
        IPTSession ptSession = mParent.GetPTSession();
        IPTObjectManager ptObjectManagers = (IPTObjectManager) ptSession.GetObjectManagers(nCLASSID);
        PTObjectManagerQueryWrapper omqr = new PTObjectManagerQueryWrapper(ptObjectManagers);
        IPTServerContext ptSCRemoteServer = (IPTServerContext) m_ptWebService.GetInterfaces("IPTServerContext");
        //MES-: So you're querying for all the objects with the given WebServiceID?  What if the
        //	classid passe in in nCLASSID doesn't correspond to an object that has PT_PROPID_GADGET_WEBSERVICEID?			
        // Query filter
        //AGC-: It fail. This method will only work for portlets and portlet templates. To expand this to work with
        //crawlers and other objects would require significant server changes (the addition of new queries). 
        Object[][] ptqf = new Object[3][];
        for (int j2c_index = 0;j2c_index < 3;j2c_index++)
        {
          ptqf[j2c_index] = new Object[1];
        }
        switch (nCLASSID){
        case PT_CLASSIDS.PT_GADGET_ID:
        case PT_CLASSIDS.PT_GADGETTEMPLATE_ID:
        ptqf[0][0] = PT_PROPIDS.PT_PROPID_GADGET_WEBSERVICEID;
        break;
        case PT_CLASSIDS.PT_AUTHSOURCE_ID:
        ptqf[0][0] = PT_PROPIDS.PT_PROPID_AUTHSOURCE_WEBSERVICEID;
        break;
        case PT_CLASSIDS.PT_DATASOURCE_ID:
        ptqf[0][0] = PT_PROPIDS.PT_PROPID_DS_WEBSERVICEID;
        break;
        case PT_CLASSIDS.PT_FEDERATEDPORTAL_ID:
        ptqf[0][0] = PT_PROPIDS.PT_PROPID_FEDERATEDPORTAL_WEBSERVICEID;
        break;
        case PT_CLASSIDS.PT_PROFILESOURCE_ID:
        ptqf[0][0] = PT_PROPIDS.PT_PROPID_PROFILESOURCE_WEBSERVICEID;
        break;
        }
        ptqf[1][0] = PT_FILTEROPS.PT_FILTEROP_EQ;
        ptqf[2][0] = ptSCRemoteServer.GetObjectID();
        omqr.Query(-1, -1, null, ptqf);
        return omqr;
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#GetValidationFunction()"></seealso>
    public virtual String GetValidationFunction()
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#InitWSModel()"></seealso>
    public virtual void InitWSModel(IPTWebService ptWebService)
    {
      m_ptWebService = ptWebService;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.editor.EditorHelperModel#SaveSection(XPHashtable)"></seealso>
    public override int SaveSection(XPHashtable _htFormData)
    {
      return RepostControl.PAGE_STATUS_VALID;
    }
  }
}