using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portalpages.admin.editors.webservice;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.form;
namespace com.plumtree.portalpages.admin.editors.webservice.plugins
{
  /// <author>adolfoc</author>
  public class SelectDevicesPluginModel : EditorHelperModel,ISelectDevicesPluginModelRO,
  IWSSubModel
  {
    public const String STR_MVC_CLASS_NAME = "SelectDevicesPlugin";
    
    /// <summary>
    /// The Web Service object being edited by our parent Web Service editor
    /// </summary>
    internal IPTWebService m_ptWebService = null;
    
    /// <summary>
    /// URL to the BaseURL from the Remote Server
    /// </summary>
    private String m_strBaseURL = "";
    private XPArrayList m_xpalAvailUIUniqueIDs = null;
    
    /// <summary>
    /// The array of selected Devices (by UniqueID)
    /// </summary>
    private XPArrayList m_xpalSelectedDevices;
    private XPHashtable m_xphAvailUIDescriptions = null;
    private XPHashtable m_xphAvailUINames = null;
    
    /// <summary>
    /// The array and hash tables that contain the available UserInterface data.
    /// These three members are initialized by the LazyInitAvailableUIs function.
    /// </summary>
    private bool m_bAvailUIsInitialized = false;
    public SelectDevicesPluginModel() : base()
    {
      m_xpalSelectedDevices = new XPArrayList();
      }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SelectDevicesPluginModel();
    }
    
    /// <summary>
    /// Returns whether or not a particular device is selected for this WebService.
    /// </summary>
    public virtual bool DeviceChecked(String strUnique)
    {
      return m_xpalSelectedDevices.Contains(strUnique);
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskBeforeWSStore()"></seealso>
    public virtual void DoTaskBeforeWSStore()
    {
      m_ptWebService.SetUserInterfaces(m_xpalSelectedDevices.ToArray());
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnRemoteServerUpdate(String,String,String)"></seealso>
    public virtual void DoTaskOnRemoteServerUpdate(String strRemoteServerID,
     String strRemoteServerName, String strRemoteServerBaseURL)
    {
      m_strBaseURL = strRemoteServerBaseURL;
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnWSModeUpdate(int)"></seealso>
    public virtual void DoTaskOnWSModeUpdate(int nNewWSMode)
    {
    }
    
    /// <summary>
    /// Returns an ArrayList of all of the available userinterface objectids.
    /// </summary>
    public virtual XPArrayList GetAvailableUIs()
    {
      LazyInitAvailableUIs();
      return m_xpalAvailUIUniqueIDs;
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.plugins.ISelectDevicesPluginModelRO#GetBaseURL()"></seealso>
    public virtual String GetBaseURL()
    {
      return m_strBaseURL;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Returns the Description of the interface with a particular Object ID (use GetAvailableUIs to
    /// get a list of valid objectids).
    /// </summary>
    public virtual String GetUIDescription(String strUnique)
    {
      LazyInitAvailableUIs();
      return (String) m_xphAvailUIDescriptions.GetElement(strUnique);
    }
    
    /// <summary>
    /// Returns the Name of the interface with a particular Object ID (use GetAvailableUIs to
    /// get a list of valid objectids).
    /// </summary>
    public virtual String GetUIName(String strUnique)
    {
      LazyInitAvailableUIs();
      return (String) m_xphAvailUINames.GetElement(strUnique);
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#GetValidationFunction()"></seealso>
    public virtual String GetValidationFunction()
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#InitWSModel()"></seealso>
    public virtual void InitWSModel(IPTWebService ptWebService)
    {
      m_ptWebService = ptWebService;
      m_xpalSelectedDevices.Clear();
      Object[] arDevices = (Object[]) ptWebService.GetUserInterfaces();
      if (null != arDevices)
      {
        for (int nIndex = 0;nIndex < arDevices.Length;nIndex++)
        {
          m_xpalSelectedDevices.Add(arDevices[nIndex]);
        }
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.editor.EditorHelperModel#SaveSection(XPHashtable)"></seealso>
    public override int SaveSection(XPHashtable _htFormData)
    {
      LazyInitAvailableUIs();
      m_xpalSelectedDevices.Clear();
      String[] data = (String[]) _htFormData.GetElement(SelectDevicesPluginView.HTTP_DEVICE_PREFIX);
      for (int nIndex = 0;nIndex < data.Length;nIndex++)
      {
        if ((null != data[nIndex]) && (0 < data[nIndex].Length))
        {
          m_xpalSelectedDevices.Add(data[nIndex]);
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    // Helper Functions
    
    /// <summary>
    /// Initializes the member variables that contain the available userinterface info.
    /// </summary>
    private void LazyInitAvailableUIs()
    {
      if (m_bAvailUIsInitialized)
      {
        return;
      }
      IPTSession ptSession = ((WebServiceModel) ((WebServiceEditorAS) m_asOwner).GetModel(WebServiceModel.STR_MVC_CLASS_NAME)).GetPTSession();
      //Future Feature: Should use ObjectManagerQueryResultWrapper here- will perform better.		
      IPTObjectManager ptOMUserInterfaces = ptSession.GetUserInterfaces();
      IPTQueryResult ptQRUserInterfaces = ptOMUserInterfaces.SimpleQuery(-1,
       PT_PROPIDS.PT_PROPID_NAME);
      int nNumUIs = 0;
      if (null != ptQRUserInterfaces)
      {
        nNumUIs = ptQRUserInterfaces.RowCount();
      }
      // Initalize our members
      m_xpalAvailUIUniqueIDs = new XPArrayList();
      m_xphAvailUINames = new XPHashtable();
      m_xphAvailUIDescriptions = new XPHashtable();
      // Now fill our members with da data.
      for (int nIndex = 0;nIndex < nNumUIs;nIndex++)
      {
        String strUniqueID = ptQRUserInterfaces.ItemAsString(nIndex, PT_PROPIDS.PT_PROPID_USERINTERFACE_UNIQUEID);
        m_xpalAvailUIUniqueIDs.Add(strUniqueID);
        m_xphAvailUINames.PutElement(strUniqueID, ptQRUserInterfaces.ItemAsString(nIndex,
         PT_PROPIDS.PT_PROPID_NAME));
        m_xphAvailUIDescriptions.PutElement(strUniqueID, ptQRUserInterfaces.ItemAsString(nIndex,
         PT_PROPIDS.PT_PROPID_DESCRIPTION));
      }
      m_bAvailUIsInitialized = true;
    }
  }
}