using System;
// Framework Imports
using com.plumtree.openfoundation.web;
using com.plumtree.portalpages.admin.ptutilities.diagnostic.dynamicloads.varpacks;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.form;
namespace com.plumtree.portalpages.admin.httpmemorydebugging
{
  /// <summary>
  /// This is the main repost control for the HTTPMemoryDebug AS.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class HTTPMemoryDebugRepostControl : RepostControl,IHTTPControl
  {
    private IXPRequest m_request = null;
    
    /// <summary>
    /// Repost action: Reload Var Pack 
    /// </summary>
    public const int POSTTOSELF_ACTION_RELOAD_VARPACK = 10;
    
    /// <summary>
    /// Repost action: View Var Pack 
    /// </summary>
    public const int POSTTOSELF_ACTION_VIEW_VARPACK = 11;
    
    /// <summary>
    /// Repost action: View Var Pack 
    /// </summary>
    public const int POSTTOSELF_ACTION_RELOAD_CURRENT_VIEWED_VARPACK = 12;
    
    /// <summary>
    /// Repost action: Clear cache 
    /// </summary>
    public const int POSTTOSELF_ACTION_CLEAR_CACHE = 13;
    
    /// <summary>
    /// Repost action: Clear memory 
    /// </summary>
    public const int POSTTOSELF_ACTION_CLEAR_MEMORY = 14;
    
    /// <summary>
    /// Repost action: Clear memory and caches 
    /// </summary>
    public const int POSTTOSELF_ACTION_CLEAR_MEMORY_AND_CACHES = 15;
    
    /// <summary>
    /// Repost action: Reload custom 
    /// </summary>
    public const int POSTTOSELF_ACTION_RELOAD_DYNAMIC_LOADS = 80;
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HTTPMemoryDebug";
    
    /// <summary>
    /// Switch between the action modes and call
    /// the associated method for each action.
    /// </summary>
    /// <param name="_nAction"></param>
    protected override void PerformAction(int _nAction)
    {
      base.PerformAction(_nAction);
      switch (_nAction){
      case POSTTOSELF_ACTION_RELOAD_VARPACK:
      ((HTTPMemoryDebugModel) m_model).ReloadVarPack();
      break;
      case POSTTOSELF_ACTION_VIEW_VARPACK:
      // JF- Nothing to do here, the SavePage method will get the ID 
      // of the var pack to display from the Query String.
      break;
      case POSTTOSELF_ACTION_RELOAD_CURRENT_VIEWED_VARPACK:
      ((HTTPMemoryDebugModel) m_model).ReloadCurrentViewedVarPack();
      Redirect myRedirect = new Redirect();
      myRedirect.SetLinkGetCachedSpace(m_asOwner);
      myRedirect.SetDisplayPage(VarPackContentDP.STR_MVC_CLASS_NAME);
      SetRedirect(myRedirect);
      break;
      case POSTTOSELF_ACTION_CLEAR_CACHE:
      ((HTTPMemoryDebugModel) m_model).ClearCache();
      break;
      case POSTTOSELF_ACTION_CLEAR_MEMORY:
      ((HTTPMemoryDebugModel) m_model).ClearMemory();
      break;
      case POSTTOSELF_ACTION_CLEAR_MEMORY_AND_CACHES:
      ((HTTPMemoryDebugModel) m_model).ClearMemoryAndCaches();
      break;
      case POSTTOSELF_ACTION_RELOAD_DYNAMIC_LOADS:
      ((HTTPMemoryDebugModel) m_model).ReloadDynamicLoads(m_request);
      break;
      }
      // null out member variables so we don't leak them.
      m_request = null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new HTTPMemoryDebugRepostControl();
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// SetXPRequest is called by the system before calling
    /// PerformAction.
    /// </summary>
    /// <param name="request">is the XPRequest object we're operating on</param>
    public virtual void SetHTTPItems(IXPRequest request, IWebData pageData)
    {
      m_request = request;
    }
  }
}