using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.admin.editors.xui;
using com.plumtree.portalpages.admin.ptutilities.migrationutility;
using com.plumtree.portalpages.common.areyousure;
using com.plumtree.portalpages.common.helpers;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.tempfile;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.uiinfrastructure.constants;
namespace com.plumtree.portalpages.admin.ptutilities.migrationutility.exp
{
  /// <author>jennifersu</author>
  public class MigrationUtilityExportModel : MigrationUtilityModel,IMigrationUtilityExportModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
     "com.plumtree.portalpages.admin.ptutilities.migrationutility.exp.MigrationUtilityExportModel");
    public const String STR_MVC_CLASS_NAME = "MigrationUtilityExportModel";
    
    /// <summary>
    /// The number of minutes we want our temp file to stick around.
    /// </summary>
    private static int TEMPFILE_MINUTES = 60;
    
    /// <summary>
    /// The maximum characters in the package name prior to the extension. (".pte" will be appended).
    /// </summary>
    private const int MAX_PACKAGE_NAME_LENGTH = 60;
    
    /// <summary>
    /// The name of the session key into which we will be storing the filename to our export package.
    /// </summary>
    public const String EXPORT_FILE_SESSION_KEY = "fdDownloadExportFile";
    
    /// <summary>
    /// The name of the session key into which we will be storing confirmation messages that the 
    /// AreYouSure popup can access and display.
    /// </summary>
    public const String AYS_SESSION_KEY = "aysExportMessages";
    
    /// <summary>
    /// Migration Package name
    /// </summary>
    private String m_strPackageName;
    
    /// <summary>
    /// Package description
    /// </summary>
    private String m_strPackageDescription;
    
    /// <summary>
    /// Package publisher
    /// </summary>
    private String m_strPublisherName;
    
    /// <summary>
    /// Export parent folders as objects if true, as references if false
    /// </summary>
    private const bool ExportParentFoldersDefault = false;
    private bool exportParentFolders = ExportParentFoldersDefault;
    
    /// <summary>
    /// Inits member variables
    /// </summary>
    /// <param name="parent"></param>
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
      m_strPackageName = "";
      m_strPackageDescription = "";
      m_strPublisherName = "";
      exportParentFolders = ExportParentFoldersDefault;
    }
    
    /// <summary>
    /// Implements some actions to perform on start.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>boolean Is true if the current user has the rights to access this Editor.</returns>
    public override bool CheckEditorAccessAndStart(XPHashtable _htFormData)
    {
      MigrationSelectForExportModel mySelectForExportModel = (MigrationSelectForExportModel) m_asOwner.GetModel(MigrationUtilityExportAS.strStorageSelectForExport);
      mySelectForExportModel.InitSFE(m_ptSession);
      StartXUIServicesSafely(XUIHelperMethods.XUI_MIGRATION_EXPORT_SERVICEID);
      return true;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new MigrationUtilityExportModel();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskBeforeStore()"></seealso>
    public override void DoTaskBeforeStore()
    {
      base.DoTaskBeforeStore();
      try
      {
        for (int indService = 0;indService < m_alMWSIDs.GetSize();indService++)
        {
          // mj: save xui pbag
          XUIHelperModel myXUIModel = (XUIHelperModel) m_asOwner.GetModel(XUIHelperModel.STR_MVC_CLASS_NAME + indService);
          IXPPropertyBag pbXUISettings = myXUIModel.GetFinalXUISettings();
          m_htPropBags.PutElement(m_alMWSIDs.GetElement(indService), pbXUISettings);
          }
      }
      catch (Exception e)
      {
        GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Providers"),
         STR_MVC_CLASS_NAME + "::DoTaskBeforeStore", e, true, UIErrorLevel.SERIOUS,
         m_asOwner.GetString(2, "ptmsgs_provUIMWS"));
      }
    }
    
    /// <summary>
    /// Implement some code to execute when the user clicks on "Finish".
    /// </summary>
    public override void Finish()
    {
      MigrationSelectForExportModel listModel = (MigrationSelectForExportModel) m_asOwner.GetModel(MigrationUtilityExportAS.strStorageSelectForExport);
      PTGrowableSortedArrayWrapper ptgsarItems = listModel.GetItems();
      if (m_alMWSIDs.GetSize() == 0)
      {
        if (ptgsarItems.GetCount() == 0)
        {
          String strError = m_asOwner.GetString(2775, "ptmsgs_portaladminmsgs");
          XPException e = new XPException(strError);
          GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Admin"),
           "MigrationUtilityExportModel::Finish", e, true, UIErrorLevel.FRIENDLY,
           strError);
        }
      }
      DoTaskBeforeStore();
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Calls the right save function according to the page name passed in argument.
    /// </summary>
    /// <param name="_sPageName"></param>
    /// <param name="_htFormData"></param>
    /// <returns>int Page status for server side validation</returns>
    public override int SavePage(String _sPageName, XPHashtable _htFormData)
    {
      int nPageStatus = RepostControl.PAGE_STATUS_VALID;
      if (_sPageName.Equals(MigrationExportMainDP.STR_MVC_CLASS_NAME))
      {
        m_strPackageName = ModelHelper.GetFormElementString(_htFormData, MigrationGeneralInfoView.HTMLINPUT_PACKAGE_NAME,
         "");
        m_strPackageDescription = ModelHelper.GetFormElementString(_htFormData,
         MigrationGeneralInfoView.HTMLINPUT_PACKAGE_DESCRIPTION, "");
        m_strPublisherName = ModelHelper.GetFormElementString(_htFormData,
         MigrationGeneralInfoView.HTMLINPUT_PUBLISHER_NAME, "");
      }
      // Save select for export
      if (_sPageName.Equals(MigrationExportPortalResourcesDP.STR_MVC_CLASS_NAME))
      {
        exportParentFolders = ModelHelper.GetFormElementBool(_htFormData,
         ExportSettingsView.HTMLCHECKBOX_EXPORT_PARENT_FOLDERS, ExportParentFoldersDefault);
        nPageStatus = ((MigrationSelectForExportModel) m_asOwner.GetModel(MigrationUtilityExportAS.strStorageSelectForExport)).SaveSection(_htFormData);
        }
      nPageStatus = base.SavePage(_sPageName, _htFormData);
      return nPageStatus;
    }
    
    /// <summary>
    /// Creates a new PTMigrationTask and spawns the new thread.
    /// </summary>
    public virtual void StartExport()
    {
      log.Info("Starting new migration task.");
      // Need to reset migration manager in case it's in Import mode
      // from loading existing packages
      ResetMigrationManager();
      // Temporarily write to this location
      String strExportFile = TempFile.CreateTempFile("odexport", FixPackageNameForFile(m_strPackageName),
       ".pte", TEMPFILE_MINUTES);
      // Initialize migration manager
      m_ptMigrationManager.InitializeForExport(strExportFile);
      // Set the export file path as the location for download in the session (to be used by
      // FileDownloadAS later).
      ISessionManager subSessionMgr = m_asOwner.GetSubSession();
      subSessionMgr.SetAttribute(EXPORT_FILE_SESSION_KEY, strExportFile);
      // Create a new PTMigrationTask and spawn new thread.
      MigrationSelectForExportModel listModel = (MigrationSelectForExportModel) m_asOwner.GetModel(MigrationUtilityExportAS.strStorageSelectForExport);
      PTGrowableSortedArrayWrapper listItems = listModel.GetItems();
      m_objMigrationTask = new PTExportMigrationTask(m_ptMigrationManager,
       m_ptSession, listItems, m_strPackageName, m_strPackageDescription,
       m_strPublisherName, exportParentFolders);
      ((PTExportMigrationTask) m_objMigrationTask).SetMWSPropBags(m_htPropBags);
      XPThread migrationThread = new XPThread(m_objMigrationTask);
      migrationThread.SetBackground(true);
      migrationThread.Start();
    }
    
    /// <summary>
    /// Perform action when user Adds Exisiting Package
    /// (load resources into list)
    /// </summary>
    public override void DoLoadPackage(XPHashtable _htFormData)
    {
      base.DoLoadPackage(_htFormData);
      MigrationSelectForExportModel listModel = (MigrationSelectForExportModel) m_asOwner.GetModel(MigrationUtilityExportAS.strStorageSelectForExport);
      listModel.LoadResources();
    }
    
    /// <summary>
    /// Clean up package name so that it will be a valid 
    /// file name.
    /// </summary>
    /// <param name="strIn"></param>
    /// <returns>String</returns>
    private String FixPackageNameForFile(String strIn)
    {
      String strDefault = "exportpackage";
      if (null == strIn)
      {
        return strDefault;
      }
      String strOut = GeneralHelpers.TruncateLongString(strIn, MAX_PACKAGE_NAME_LENGTH,
       false);
      strOut = XPStringUtility.Trim(strOut);
      strOut = GeneralHelpers.Replace(strOut, ".", "");
      strOut = GeneralHelpers.Replace(strOut, "\t", "");
      strOut = GeneralHelpers.Replace(strOut, "\r", "");
      strOut = GeneralHelpers.Replace(strOut, "\n", "");
      strOut = GeneralHelpers.Replace(strOut, "?", "");
      strOut = GeneralHelpers.Replace(strOut, "*", "");
      strOut = GeneralHelpers.Replace(strOut, "\\", "");
      strOut = GeneralHelpers.Replace(strOut, "/", "");
      strOut = GeneralHelpers.Replace(strOut, ":", "");
      strOut = GeneralHelpers.Replace(strOut, "\"", "");
      strOut = GeneralHelpers.Replace(strOut, "<", "");
      strOut = GeneralHelpers.Replace(strOut, ">", "");
      strOut = GeneralHelpers.Replace(strOut, "|", "");
      if (strOut.Length <= 0)
      {
        return strDefault;
      }
      return strOut;
    }
    
    /// <summary>
    /// Returns package name
    /// </summary>
    public virtual String GetPackageName()
    {
      return m_strPackageName;
    }
    
    /// <summary>
    /// Returns package description
    /// </summary>
    public virtual String GetPackageDescription()
    {
      return m_strPackageDescription;
    }
    
    /// <summary>
    /// Returns package publisher name
    /// </summary>
    public virtual String GetPublisher()
    {
      return m_strPublisherName;
    }
    
    /// <summary>
    /// When exporting dependencies, should the parent folders
    /// and their dependencies also be exported?
    /// </summary>
    /// <returns>
    /// true to export parent folder dependencies as
    /// objects and pull in their dependencies as well,
    /// false to only add references in the dependency map
    /// </returns>
    public virtual bool IsExportParentFoldersEnabled()
    {
      return exportParentFolders;
    }
    
    /// <summary>
    /// Get list of confirmation messages to 
    /// display in the AreYouSure popup
    /// </summary>
    /// <returns></returns>
    public virtual XPArrayList GetAYSMessages()
    {
      XPArrayList xpalMessages = new XPArrayList();
      String strMessage;
      strMessage = m_asOwner.GetString(3168, ResourceConstants.PORTALADMINMSGS);
      AYSMessage aysMessage = new AYSMessage(strMessage, AYSMessage.AYSTYPE_TOPLEVEL);
      xpalMessages.Add(aysMessage);
      return xpalMessages;
    }
    // mel: Fix for 46247, When a user exports objects from Collab or Publisher, no feedback is given that this was done. At least provide the UUID of the zip file in the final migration status.
    
    /// <returns>XPArrayList of String each of which represents the return value from an MWS ExportPackage call with info about the MWS. If no MWS's are present or none responded, returns an empty list.</returns>
    public virtual XPArrayList GetMWSMessages()
    {
      if (m_objMigrationTask == null || m_objMigrationTask.GetMWSMessages() == null)
      {
        return new XPArrayList();
      }
      return m_objMigrationTask.GetMWSMessages();
    }
  }
}