using System;
// Framework Imports
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.admin.editors.profilesource;
using com.plumtree.portaluiinfrastructure.ptmulticolumnlist;
using com.plumtree.portaluiinfrastructure.resultwrapper;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.portalpages.admin.ptutilities.userinfopropmap
{
  /// <summary>
  /// This class keeps an up-to-date state of all the settings.
  /// It contains the set and get methods for those settings.
  /// It contains the start, save and finish methods.
  /// This class is used by the Section class (to get the latest value of the settings
  /// to display) and by the Repost Control class (to perform the repost actions).
  /// </summary>
  /// <author>JohnH</author>
  public class UserInfoPropMapModel : EditorModel,IUserInfoPropMapModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
     "com.plumtree.portalpages.admin.ptutilities.userinfopropmap.UserInfoPropMapModel");
    
    /// <summary>
    /// The unique identifier for the class 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "UserInfoPropMapModel";
    //Constants
    
    /// <summary>
    /// The number of columns (other than the Plumtree object column) in the multi-column list 
    /// </summary>
    public const int MAPLIST_NUMCOLUMNS = 1;
    
    /// <summary>
    /// Zero-based list position for the PT object column 
    /// </summary>
    public const int MAPLIST_PTOBJECTPOSITION = 0;
    
    /// <summary>
    /// Server object: Plumtree Session 
    /// </summary>
    private IPTSession m_Session;
    //Member variables
    
    /// <summary>
    /// 2 dim array 0,0 = PropertyID, 1,0 = UserInfo element 
    /// </summary>
    private Object[][] m_arrPropMapInfo;
    
    /// <summary>
    /// The column to sort the multi-column list on 
    /// </summary>
    private int m_intSortColumn = 0;
    
    /// <summary>
    /// The sort direction 
    /// </summary>
    private int m_intSortDirection = 0;
    
    /// <summary>
    /// Implements some actions to perform on start.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>boolean Is true if the current user has the rights to access this Editor.</returns>
    public override bool CheckEditorAccessAndStart(XPHashtable _htFormData)
    {
      bool bReturn = (m_Session.GetSessionInfo().CheckForActivityRight(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_EDIT_USERINFO_MAP));
      if (bReturn)
      {
        StartUserInfoPropMapList();
        return true;
      } else
      {
        return false;
      }
    }
    
    /// <summary>
    /// Public default constructor
    /// </summary>
    public UserInfoPropMapModel() : base()
    {
      m_intSortColumn = PT_PROPIDS.PT_PROPID_NAME;
      m_intSortDirection = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new UserInfoPropMapModel();
    }
    
    /// <summary>
    /// Implement some code to execute when the user clicks on "Finish".
    /// </summary>
    public override void Finish()
    {
      //m_arrPropMapInfo
      //2 dim array 0,0 = PropertyID, 1,0 = UserInfo element
      try
      {
        if (null != m_arrPropMapInfo)
        {
          //Create a new, potentially larger array containing the
          //parsed out userinfo elements
          Object[][] myArray;
          int intCount = 0;
          //First, just get the proper size of the temporary array
          for (int i = 0;i < m_arrPropMapInfo[0].Length;i++)
          {
            Object[][] arrTemp = PTMultiColumnListModel.ParseFields((String) m_arrPropMapInfo[1][i]);
            intCount = intCount + arrTemp[0].Length;
          }
          myArray = new Object[2][];
          for (int j2c_index = 0;j2c_index < 2;j2c_index++)
          {
            myArray[j2c_index] = new Object[intCount];
          }
          //Now cycle through again, this time populating the temporary
          //array
          intCount = 0;
          for (int i = 0;i < m_arrPropMapInfo[0].Length;i++)
          {
            Object[][] arrTemp = PTMultiColumnListModel.ParseFields((String) m_arrPropMapInfo[1][i]);
            for (int j = 0;j < arrTemp[0].Length;j++)
            {
              //Set the property ID
              myArray[0][intCount] = m_arrPropMapInfo[0][i];
              //Set the userinfo string
              myArray[1][intCount] = arrTemp[0][j];
              //Update the counter
              intCount = intCount + 1;
            }
          }
          IPTProfileManager myProfileManager = (IPTProfileManager) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_PROFILE_MANAGER,
           true);
          if (null != m_arrPropMapInfo)
          {
            myProfileManager.SetUserInfoMap(myArray);
          }
        }
      }
      catch (Exception e)
      {
        GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Admin"),
         "UserInfoPropMapModel::Finish", e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(1584,
         "ptmsgs_portaladminmsgs"));
      }
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    // BEGIN ***************** helper methods *****************
    // END ***************** helper methods *****************
    
    /// <summary>
    /// Inits the Model with the IPTSession.
    /// </summary>
    /// <param name="parent"></param>
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
      // JF- Get the Session object from the associated AS.
      m_Session = (IPTSession) parent.GetUserSession();
    }
    
    /// <summary>
    /// Calls the right save function according to the page name passed in argument.
    /// </summary>
    /// <param name="_sPageName"></param>
    /// <param name="_htFormData"></param>
    /// <returns>int Page status for server side validation</returns>
    public override int SavePage(String _sPageName, XPHashtable _htFormData)
    {
      int nPageStatus = UserInfoPropMapRepostControl.PAGE_STATUS_VALID;
      // Save the items to be unlocked
      if (_sPageName.Equals(MainDP.STR_MVC_CLASS_NAME))
      {
        nPageStatus = SaveMainPage(_htFormData);
      }
      return nPageStatus;
    }
    
    /// <summary>
    /// Initializes the multi-column list with the standard intrinsic list of properties
    /// </summary>
    public virtual void StartUserInfoPropMapList()
    {
      UserInfoPropMapListModel myListModel = (UserInfoPropMapListModel) m_asOwner.GetModel(UserInfoPropMapAS.STR_MVC_STORAGE_NAME_PLUG_IN);
      //Set the number of columns.
      //Login form page
      myListModel.SetNumColumns(MAPLIST_NUMCOLUMNS);
      myListModel.SetIncludePTSelect(true);
      myListModel.SetPTObjectPosition(MAPLIST_PTOBJECTPOSITION);
      String[] arrPropMapSubTitles = new String[MAPLIST_NUMCOLUMNS];
      arrPropMapSubTitles[0] = m_asOwner.GetString(1579, "ptmsgs_portaladminmsgs");
      myListModel.InitializePTMultiColumnList(m_asOwner.GetString(1577, "ptmsgs_portaladminmsgs"),
       m_asOwner.GetString(1578, "ptmsgs_portaladminmsgs"), arrPropMapSubTitles,
       PTImageType.ICON_PROPERTY, m_asOwner.GetString(1580, "ptmsgs_portaladminmsgs"),
       new int[]{PT_CLASSIDS.PT_PROPERTY_ID}, 1);
      // Use the method below to set the value to populate a newly created input text.
      // (By default it will be a null array, so it MUST be initialized.
      //Login form page
      String[] arrPropMapDefaults = new String[MAPLIST_NUMCOLUMNS];
      //get just the user properties
      String userProperties = ProfileSourceModel.GetUserProperties(m_Session);
      arrPropMapDefaults[0] = "";
      myListModel.SetDefaultValue(arrPropMapDefaults);
      myListModel.SetDisplayObjects(userProperties);
      myListModel.SetRowIcon(PTImageType.ICON_PROPERTY);
      myListModel.SetTreeMultiSelect(true);
      myListModel.SetTreeSubTitle(m_asOwner.GetString(539, "ptmsgs_portaladminmsgs"));
      myListModel.SetTreeTitle(m_asOwner.GetString(538, "ptmsgs_portaladminmsgs"));
      //Set the message and instructions to display when the list is empty.
      myListModel.SetEmptyListMessages(m_asOwner.GetString(1581, "ptmsgs_portaladminmsgs"),
       m_asOwner.GetString(1582, "ptmsgs_portaladminmsgs"));
      //Get the name value pairs from the model in edit mode,
      //and populate them in here
      try
      {
        IPTProfileManager ptProfileManager = (IPTProfileManager) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_PROFILE_MANAGER,
         false);
        Object[][] arrMap;
        arrMap = (Object[][]) ptProfileManager.GetUserInfoMap();
        if (arrMap[0].Length > 0)
        {
          int intCount = 0;
          int intLastValue = 0;
          for (int k = 0;k < arrMap[0].Length;k++)
          {
            if (XPConvert.ToInteger(arrMap[0][k]) != intLastValue)
            {
              intCount++;
            }
            intLastValue = XPConvert.ToInteger(arrMap[0][k]);
          }
          if (intCount > 0)
          {
            Object[][] oRows = new Object[intCount][];
            for (int j2c_index = 0;j2c_index < intCount;j2c_index++)
            {
              oRows[j2c_index] = new Object[MAPLIST_NUMCOLUMNS + 4];
            }
            XPStringBuilder sb = new XPStringBuilder(200);
            //Dictionaries for the names/ids of the operations
            //Get all the possible classes that can be used as operations
            intLastValue = XPConvert.ToInteger(arrMap[0][0]);
            int intRowIndex = 0;
            String strLastValue = "";
            for (int i = 0;i < arrMap[0].Length;i++)
            {
              // get the name
              PTObjectManagerQueryWrapper omqrProperty = new PTObjectManagerQueryWrapper(m_Session.GetProperties());
              int intObjID = XPConvert.ToInteger(arrMap[0][i]);
              omqrProperty.QuerySingleObject(intObjID);
              if (intObjID != intLastValue)
              {
                // Set the class ID
                oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_CLASSID + MAPLIST_PTOBJECTPOSITION] = PT_CLASSIDS.PT_PROPERTY_ID;
                // Set the Object ID
                oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_OBJECTID + MAPLIST_PTOBJECTPOSITION] = intLastValue;
                // Set the Object name
                oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_STRING + MAPLIST_PTOBJECTPOSITION] = strLastValue;
                // Set the mapped UserInfo element
                if (MAPLIST_PTOBJECTPOSITION < MAPLIST_NUMCOLUMNS)
                {
                  oRows[intRowIndex][MAPLIST_PTOBJECTPOSITION + 4] = sb.ToString();
                  } else
                {
                  oRows[intRowIndex][MAPLIST_PTOBJECTPOSITION + 1] = sb.ToString();
                  }
                sb = new XPStringBuilder(200);
                sb.Append(arrMap[1][i]);
                intRowIndex++;
              } else
              {
                if (i > 0)
                {
                  sb.Append(", ").Append(arrMap[1][i]);
                } else
                {
                  sb.Append(arrMap[1][i]);
                }
              }
              intLastValue = intObjID;
              strLastValue = (String) omqrProperty.GetFields(0, PT_PROPIDS.PT_PROPID_NAME);
              }
            //Now pick up the last row
            // Set the class ID
            oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_CLASSID + MAPLIST_PTOBJECTPOSITION] = PT_CLASSIDS.PT_PROPERTY_ID;
            // Set the Object ID
            oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_OBJECTID + MAPLIST_PTOBJECTPOSITION] = intLastValue;
            // Set the Object name
            oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_STRING + MAPLIST_PTOBJECTPOSITION] = strLastValue;
            // Set the mapped UserInfo element
            if (MAPLIST_PTOBJECTPOSITION < MAPLIST_NUMCOLUMNS)
            {
              oRows[intRowIndex][MAPLIST_PTOBJECTPOSITION + 4] = sb.ToString();
              } else
            {
              oRows[intRowIndex][MAPLIST_PTOBJECTPOSITION + 1] = sb.ToString();
              }
            // Then add the array to the list
            myListModel.SetDefaultObjects(oRows);
          }
        }
      }
      catch (Exception e)
      {
        GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Admin"),
         "UserInfoPropMapModel::StartUserInfoPropMapList", e, true, UIErrorLevel.SERIOUS,
         m_asOwner.GetString(2426, "ptmsgs_portaladminmsgs"));
      }
    }
    
    /// <summary>
    /// Saves the data on the main page.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>int Page status for server side validation</returns>
    private int SaveMainPage(XPHashtable _htFormData)
    {
      int intPageStatus = UserInfoPropMapRepostControl.PAGE_STATUS_VALID;
      try
      {
        //First, save the existing section to the activity space
        ((UserInfoPropMapListModel) m_asOwner.GetModel(UserInfoPropMapAS.STR_MVC_STORAGE_NAME_PLUG_IN)).SaveSection(_htFormData);
        //Now retrieve the non Plumtree select elements
        Object[][] arrMappingInfo = ((UserInfoPropMapListModel) m_asOwner.GetModel(UserInfoPropMapAS.STR_MVC_STORAGE_NAME_PLUG_IN)).GetListValues();
        //m_arrPropMapInfo
        //2 dim array 0,0 = PropertyID, 1,0 = UserInfo element
        PTGrowableSortedArrayWrapper arrItems;
        arrItems = ((UserInfoPropMapListModel) m_asOwner.GetModel(UserInfoPropMapAS.STR_MVC_STORAGE_NAME_PLUG_IN)).GetItems();
        int intNumItems = arrItems.GetCount();
        if (intNumItems > 0)
        {
          m_arrPropMapInfo = new Object[2][];
          for (int j2c_index = 0;j2c_index < 2;j2c_index++)
          {
            m_arrPropMapInfo[j2c_index] = new Object[intNumItems];
          }
          XPHashtable htDupChecker = new XPHashtable();
          Object strMappedValue;
          for (int i = 0;i < intNumItems;i++)
          {
            //Validate first that there is not an empty string mapped
            if (((String) arrMappingInfo[0][i]).Equals(""))
            {
              //Raise an error and return invalid
              UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
               null, m_asOwner.GetString(2157, "ptmsgs_portaladminmsgs"),
               m_asOwner.GetString(2157, "ptmsgs_portaladminmsgs"));
              UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
               m_asOwner.GetApplication());
              return UserInfoPropMapRepostControl.PAGE_STATUS_INVALID;
            }
            //Now validate that the userinfo value hasn't yet been mapped to another property
            strMappedValue = arrMappingInfo[0][i];
            if (htDupChecker.ContainsKey(strMappedValue))
            {
              //Raise an error and return invalid
              UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
               null, m_asOwner.GetString(2425, "ptmsgs_portaladminmsgs"),
               m_asOwner.GetString(2425, "ptmsgs_portaladminmsgs"));
              UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
               m_asOwner.GetApplication());
              return UserInfoPropMapRepostControl.PAGE_STATUS_INVALID;
            }
            m_arrPropMapInfo[0][i] = arrItems.GetItem(i, PTMultiColumnListModel.EXPLIST_COL_OBJECTID + MAPLIST_PTOBJECTPOSITION);
            m_arrPropMapInfo[1][i] = XPStringUtility.Trim(XPConvert.ToString(strMappedValue));
            htDupChecker.PutElement(arrMappingInfo[0][i], arrMappingInfo[0][i]);
            }
        }
      }
      catch (Exception e)
      {
        intPageStatus = UserInfoPropMapRepostControl.PAGE_STATUS_INVALID;
        GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Admin"),
         "UserInfoPropMapModel::SaveMainPage", e, false, UIErrorLevel.SERIOUS,
         m_asOwner.GetString(1583, "ptmsgs_portaladminmsgs"));
      }
      return intPageStatus;
    }
  }
}
// END ***************** repost action methods *****************
