using System;
// Framework Imports
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.portalpages.admin.serversettings.general
{
  /// <author>$author$</author>
  public class SetServerSettingsAS : PTEditorAS
  {
    // ***COMMENT_1*** All the AS in Plumtree need to have a unique name.
    public const String STR_MVC_CLASS_NAME = "SetServerSettingsAS";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SetServerSettingsAS();
    }
    
    /// <summary>
    /// Returns the Editor title.
    /// </summary>
    /// <returns>String</returns>
    public override String GetEditorTitle()
    {
      return GetString(1346, "ptmsgs_portaladminmsgs");
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    // Strings to localize.
    
    /// <summary>
    /// Returns the class name of the Property Repost Control.
    /// </summary>
    /// <returns>String</returns>
    public override String GetRepostControlName()
    {
      return (SetServerSettingsRepostControl.STR_MVC_CLASS_NAME);
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.AActivitySpace#CheckBasicAccess(String,String,boolean)"></seealso>
    public override bool CheckBasicAccess(String strPage, String strControl,
     bool bSameSpace)
    {
      bool bInAdminGroup = PlumtreeHelpers.CheckCurrentUserIsAdmin((IPTSession) GetUserSession());
      if (!bInAdminGroup)
      {
        return false;
      }
      return base.CheckBasicAccess(strPage, strControl, bSameSpace);
    }
    
    /// <summary>
    /// Registers all the classes of the Property Activity Space.
    /// </summary>
    public override void Init()
    {
      m_strModelName = SetServerSettingsModel.STR_MVC_CLASS_NAME;
      // Warning: m_strModelName must be initialized before super.Init() is called.
      base.Init();
      IModel myModel = GetModel(SetServerSettingsModel.STR_MVC_CLASS_NAME);
      // Register DisplayPages
      // Main DP
      UserSettingsDP dp = new UserSettingsDP();
      dp.AddToValidationFunction("CheckNumericInputs");
      RegisterPage(dp);
      RegisterPage(new PortalURLsDP());
      // Register Views
      RegisterView(NewUserCreationView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(AccountLockingView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(NumericAuthenticationView.STR_MVC_CLASS_NAME, myModel);
      /* JHQ bug 25352: Doc Team says to remove this view because it is not
       * accurate.  I will not delete the class, since we may change our mind
       * in the future.  For now I'll just comment this out. */
       
      //		RegisterView(DefaultLanguageSettingsView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(UpdateLoginTokenKeyView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DebuggingModeView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(PortalURLsView.STR_MVC_CLASS_NAME, myModel);
      // Register Controls
      RegisterControl(SetServerSettingsRepostControl.STR_MVC_CLASS_NAME, myModel);
      // Set default page
      SetCurrentPage(UserSettingsDP.STR_MVC_CLASS_NAME);
      int nMenuOneID = m_mEditorMenu.StartNewSubMenu(GetString(1347, "ptmsgs_portaladminmsgs"));
      m_mEditorMenu.AddLinkToSubMenu(nMenuOneID, UserSettingsDP.STR_MVC_CLASS_NAME);
      m_mEditorMenu.AddLinkToSubMenu(nMenuOneID, PortalURLsDP.STR_MVC_CLASS_NAME);
      }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AActivitySpace#SetCurrentPage()"></seealso>
    /// <param name="IDisplayPage">the current Display Page</param>
    public override void SetCurrentPage(IDisplayPage page)
    {
      base.SetCurrentPage(page);
      String sPageName = page.GetName();
      if (UserSettingsDP.STR_MVC_CLASS_NAME.Equals(sPageName))
      {
        this.SetHelpTopic("PT_PortalServerManager_UserSettings");
      } else
      if (PortalURLsDP.STR_MVC_CLASS_NAME.Equals(sPageName))
      {
        this.SetHelpTopic("PT_PortalServerManager_PortalURLs");
      }
    }
  }
}