using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory;
using com.plumtree.portaluiinfrastructure.resultwrapper;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.admin.serversettings.globalaclsyncmap
{
  /// <summary>
  /// This class keeps an up-to-date state of all the settings.
  /// It contains the set and get methods for those settings.
  /// It contains the start, save and finish methods.
  /// This class is used by the Section class (to get the latest value of the settings
  /// to display) and by the Repost Control class (to perform the repost actions).
  /// </summary>
  /// <author>JohnH</author>
  /// <author>JenniferSu</author>
  public class GlobalACLSyncMapModel : EditorModel,IGlobalACLSyncMapModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
     "com.plumtree.portalpages.admin.serversettings.globalaclsyncmap.GlobalACLSyncMapModel");
    
    /// <summary>
    /// Unique identifier for the class 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "GlobalACLSyncMapModel";
    //Constants
    
    /// <summary>
    /// Number of columns in the multicolumn list 
    /// </summary>
    public const int MAPLIST_NUMCOLUMNS = 1;
    //Zero-based list position for the PT object column 
    
    /// <summary>
    /// The position of the plumtree object in the multicolumn list 
    /// </summary>
    public const int MAPLIST_PTOBJECTPOSITION = 0;
    
    /// <summary>
    /// Server object: Plumtree Session 
    /// </summary>
    private IPTSession m_Session;
    //Member variables
    //This is a 2 dim array 0,0=auth source name, 1,0=mapped domain names
    private Object[][] m_arrDomainMapInfo;
    //This is a 2 dim array 0,0=group name, 1,0=mapped external group
    private Object[][] m_arrGroupMapInfo;
    private int m_intSortColumn;
    private int m_intSortDirection;
    public GlobalACLSyncMapModel() : base()
    {
      m_intSortColumn = PT_PROPIDS.PT_PROPID_NAME;
      m_intSortDirection = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
    }
    
    /// <summary>
    /// Implements some actions to perform on start.
    /// </summary>
    /// <param name="_htFormData"></param>
    public override bool CheckEditorAccessAndStart(XPHashtable _htFormData)
    {
      if (!m_Session.GetSessionInfo().CheckForActivityRight(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_UTILITIES))
      {
        return false;
      }
      if (!m_Session.GetSessionInfo().CheckCurrentUserIsAdmin())
      {
        return false;
      }
      StartDomainMapList();
      StartGroupMapList();
      StartMigrationHistory();
      return true;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new GlobalACLSyncMapModel();
    }
    
    /// <summary>
    /// Implement some code to execute when the user clicks on "Finish".
    /// </summary>
    public override void Finish()
    {
      //m_arrDomainMapInfo
      //This is a 2 dim array 0,0=auth source Prefix, 1,0=mapped string
      try
      {
        if (null != m_arrDomainMapInfo)
        {
          if (null != m_arrDomainMapInfo)
          {
            IPTACLSyncMap mySyncMap = (IPTACLSyncMap) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_ACLSYNCMAP,
             false);
            //First, delete the existing entries in the map
            mySyncMap.ClearMappings(PT_ACLSYNC_MAPPINGTYPES.PT_ACLSYNCMAPPING_DOMAIN);
            for (int i = 0;i < m_arrDomainMapInfo[0].Length;i++)
            {
              String strPrefix = XPConvert.ToString(m_arrDomainMapInfo[0][i]);
              // special case for Plumtree intrinsic auth source
              if (strPrefix.Equals("Plumtree"))
              {
                strPrefix = "";
              }
              Object[] arrFields = null;
              String domain = (String) m_arrDomainMapInfo[1][i];
              if (!(domain == null || XPStringUtility.Trim(domain).Length == 0 || XPStringUtility.StartsWith(domain,
               CommonHTMLStrings.SPACE)))
              {
                arrFields = ParseFields(domain);
              }
              if (null != arrFields)
              {
                for (int j = 0;j < arrFields.Length;j++)
                {
                  String strNewDomain = (String) arrFields[j];
                  IPTQueryResult qr = mySyncMap.QueryMappingsForItem(PT_ACLSYNC_ITEMTYPES.PT_ACLSYNCITEM_AUTHSOURCECATEGORY,
                   strPrefix);
                  bool bExists = false;
                  for (int k = 0;k < qr.RowCount();k++)
                  {
                    String strCompare = qr.ItemAsString(k, PT_PROPIDS.PT_PROPID_ACLSYNC_FROMSTRING);
                    if (strCompare.Equals(strNewDomain))
                    {
                      bExists = true;
                    }
                  }
                  if (!bExists)
                  {
                    mySyncMap.AddMapping(PT_ACLSYNC_MAPPINGTYPES.PT_ACLSYNCMAPPING_DOMAIN,
                     strNewDomain, strPrefix);
                  }
                }
              }
            }
          }
        }
        // Set the migration status to approved if the checkbox was checked
        ((IMigrationHistoryModel) m_asOwner.GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME)).StoreMigrationStatus(PT_CLASSIDS.PT_GLOBALOBJECT_ID,
         PT_GLOBALOBJECTS.PT_GLOBAL_ACLSYNCMAP);
      }
      catch (Exception e)
      {
        log.Error("GlobalACLSyncMapModel::Finish Unable to set the ACL sync map domain information" + ". The following exception occured: " + e);
        throw new XPException(m_asOwner.GetString(1487, "ptmsgs_portaladminmsgs"),
         e);
      }
      //m_arrGroupMapInfo
      //This is a 2 dim array 0,0=group ID, 1,0=mapped external group name
      try
      {
        if (null != m_arrGroupMapInfo)
        {
          if (null != m_arrGroupMapInfo)
          {
            IPTACLSyncMap mySyncMap = (IPTACLSyncMap) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_ACLSYNCMAP,
             false);
            //First, delete the existing entries in the map
            mySyncMap.ClearMappings(PT_ACLSYNC_MAPPINGTYPES.PT_ACLSYNCMAPPING_GROUP);
            for (int i = 0;i < m_arrGroupMapInfo[0].Length;i++)
            {
              String strPTGroup = XPConvert.ToString(m_arrGroupMapInfo[0][i]);
              Object[] arrFields = null;
              String group = (String) m_arrGroupMapInfo[1][i];
              if (!(group == null || XPStringUtility.Trim(group).Length == 0 || XPStringUtility.StartsWith(group,
               CommonHTMLStrings.SPACE)))
              {
                arrFields = ParseFields(group);
              }
              if (null != arrFields)
              {
                for (int j = 0;j < arrFields.Length;j++)
                {
                  String strExternalGroup = (String) arrFields[j];
                  //Add the mapping
                  mySyncMap.AddMapping(PT_ACLSYNC_MAPPINGTYPES.PT_ACLSYNCMAPPING_GROUP,
                   strExternalGroup, strPTGroup);
                  log.Debug("GlobalACLSyncMapModel::Finish Adding group map from \'" + strPTGroup + "\' (internal) to \'" + strExternalGroup + "\' (external).");
                  }
              }
            }
          }
        }
      }
      catch (Exception e)
      {
        log.Error("GlobalACLSyncMapModel::Finish Unable to set the ACL sync map group information" + ". The following exception occured: " + e);
        throw new XPException(m_asOwner.GetString(1497, "ptmsgs_portaladminmsgs"),
         e);
      }
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    // END ***************** helper methods *****************
    
    /// <summary>
    /// Inits the Model with the IPTSession.
    /// </summary>
    /// <param name="parent"></param>
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
      // JF- Get the Session object from the associated AS.
      m_Session = (IPTSession) parent.GetUserSession();
    }
    // BEGIN ***************** helper methods *****************
    
    /// <summary>
    /// Parses the string of mapped fields and returns a two dimensional array
    /// in the format expected by the Plumtree server
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskOnStartEditor(XPHashtable)"></seealso>
    public virtual Object[] ParseFields(String strMappedFields)
    {
      Object[] myArray = null;
      int intTokenIndex = 0;
      XPStringTokenizer myTokenizer = new XPStringTokenizer(strMappedFields,
       ",");
      myArray = new Object[myTokenizer.GetTokenCount()];
      for (XPStringTokenizer i = myTokenizer;i.HasMoreTokens();)
      {
        String strToken = XPStringUtility.Trim(i.GetNextToken());
        myArray[intTokenIndex] = strToken;
        intTokenIndex++;
      }
      return myArray;
    }
    
    /// <summary>
    /// Calls the right save function according to the page name passed in argument.
    /// </summary>
    /// <param name="_sPageName"></param>
    /// <param name="_htFormData"></param>
    /// <returns>int Page status for server side validation</returns>
    public override int SavePage(String _sPageName, XPHashtable _htFormData)
    {
      int nPageStatus = GlobalACLSyncMapRepostControl.PAGE_STATUS_VALID;
      // Save the pages
      if (_sPageName.Equals(MainDP.STR_MVC_CLASS_NAME))
      {
        nPageStatus = SaveMainPage(_htFormData);
      }
      if (_sPageName.Equals(GroupMapDP.STR_MVC_CLASS_NAME))
      {
        nPageStatus = SaveGroupPage(_htFormData);
      }
      if (_sPageName.Equals(MigrationHistoryPluginDP.STR_MVC_CLASS_NAME))
      {
        nPageStatus = ((MigrationHistoryPluginModel) m_asOwner.GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME)).SaveSection(_htFormData);
        }
      return nPageStatus;
    }
    
    /// <summary>
    /// This method initializes the expandable list used as the basis for
    /// the domain ACL Sync map
    /// </summary>
    public virtual void StartDomainMapList()
    {
      GlobalACLSyncMapListModel myListModel = (GlobalACLSyncMapListModel) m_asOwner.GetModel(GlobalACLSyncMapAS.STR_MVC_STORAGE_NAME_DOMAIN_PLUG_IN);
      myListModel.SetMapType(GlobalACLSyncMapListModel.ACL_MAP_TYPE_DOMAIN);
      myListModel.SetNumColumns(MAPLIST_NUMCOLUMNS);
      myListModel.SetIncludePTSelect(true);
      myListModel.SetPTObjectPosition(MAPLIST_PTOBJECTPOSITION);
      String[] arrMapSubTitles = new String[MAPLIST_NUMCOLUMNS];
      //Domain Name
      arrMapSubTitles[0] = m_asOwner.GetString(1482, "ptmsgs_portaladminmsgs");
      myListModel.InitializePTMultiColumnList(m_asOwner.GetString(1480, "ptmsgs_portaladminmsgs"),
       m_asOwner.GetString(1481, "ptmsgs_portaladminmsgs"), arrMapSubTitles,
       PTImageType.ICON_AUTH_SOURCE, m_asOwner.GetString(2689, "ptmsgs_portaladminmsgs"),
       new int[]{PT_CLASSIDS.PT_AUTHSOURCE_ID}, 1);
      // Use the method below to set the value to populate a newly created input text.
      // (By default it will be a null array, so it MUST be initialized.
      //Login form page
      String[] arrMapDefaults = new String[MAPLIST_NUMCOLUMNS];
      arrMapDefaults[0] = "";
      myListModel.SetDefaultValue(arrMapDefaults);
      myListModel.SetTreeSubTitle(m_asOwner.GetString(2144, "ptmsgs_portaladminmsgs"));
      myListModel.SetTreeTitle(m_asOwner.GetString(2145, "ptmsgs_portaladminmsgs"));
      myListModel.SetTreeMultiSelect(true);
      //Set the message and instructions to display when the list is empty.
      myListModel.SetEmptyListMessages(m_asOwner.GetString(1484, "ptmsgs_portaladminmsgs"),
       m_asOwner.GetString(1485, "ptmsgs_portaladminmsgs"));
      //Get the name value pairs from the model in edit mode,
      //and populate them in here
      try
      {
        // Get all the auth source prefixes that are mapped
        IPTACLSyncMap ptSyncMap = (IPTACLSyncMap) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_ACLSYNCMAP,
         false);
        ASQueryResultWrapper ptqrCategories = new ASQueryResultWrapper(ptSyncMap.QueryMappedItems(PT_ACLSYNC_ITEMTYPES.PT_ACLSYNCITEM_AUTHSOURCECATEGORY));
        XPArrayList xpList = ASQueryResultWrapper.ConvertToArrayList(ptqrCategories,
         PT_PROPIDS.PT_PROPID_ACLSYNC_TOSTRING);
        // Populate the expandable list
        Object[][] oRows = new Object[xpList.GetSize()][];
        for (int j2c_index = 0;j2c_index < xpList.GetSize();j2c_index++)
        {
          oRows[j2c_index] = new Object[MAPLIST_NUMCOLUMNS + 4];
        }
        for (int i = 0;i < xpList.GetSize();i++)
        {
          XPArrayList lstDomains = new XPArrayList();
          String strCategory = (String) xpList.GetElement(i);
          ASQueryResultWrapper ptqrDomains = new ASQueryResultWrapper(ptSyncMap.QueryMappingsForItem(PT_ACLSYNC_ITEMTYPES.PT_ACLSYNCITEM_AUTHSOURCECATEGORY,
           strCategory));
          for (int j = 0;j < ptqrDomains.GetCount();j++)
          {
            lstDomains.Add(ptqrDomains.GetFields(j, PT_PROPIDS.PT_PROPID_ACLSYNC_FROMSTRING));
            }
          // Set the class ID
          oRows[i][GlobalACLSyncMapListModel.EXPLIST_COL_CLASSID + MAPLIST_PTOBJECTPOSITION] = PT_CLASSIDS.PT_AUTHSOURCE_ID;
          // Set the Object ID.  Bogus value since we don't care about the actual object.
          oRows[i][GlobalACLSyncMapListModel.EXPLIST_COL_OBJECTID + MAPLIST_PTOBJECTPOSITION] = -12345;
          if (null != strCategory && strCategory.Equals("_PT_"))
          {
            // Set the Object name
            oRows[i][GlobalACLSyncMapListModel.EXPLIST_COL_STRING + MAPLIST_PTOBJECTPOSITION] = "Plumtree";
            } else
          {
            // Set the Object name
            oRows[i][GlobalACLSyncMapListModel.EXPLIST_COL_STRING + MAPLIST_PTOBJECTPOSITION] = strCategory;
            }
          // Set the elements
          XPStringBuilder sbMappedItems = new XPStringBuilder(100);
          for (int j = 0;j < lstDomains.GetSize();j++)
          {
            sbMappedItems.Append(lstDomains.GetElement(j));
            if (j < (lstDomains.GetSize() - 1))
            {
              sbMappedItems.Append(",");
            }
          }
          if (1 == MAPLIST_PTOBJECTPOSITION)
          {
            oRows[i][1] = sbMappedItems.ToString();
          } else
          {
            oRows[i][4] = sbMappedItems.ToString();
          }
        }
        myListModel.SetDefaultObjects(oRows);
      }
      catch (Exception e)
      {
        log.Error("GlobalACLSyncMapModel::StartDomainMapList\tAn error occurred when trying populate the ACL Sync map select list.  Exception: " + e);
        throw new XPException(m_asOwner.GetString(1489, "ptmsgs_portaladminmsgs"),
         e);
      }
    }
    
    /// <summary>
    /// This method initializes the expandable list used as the basis for
    /// the group ACL Sync map
    /// </summary>
    public virtual void StartGroupMapList()
    {
      GlobalACLSyncMapListModel myListModel = (GlobalACLSyncMapListModel) m_asOwner.GetModel(GlobalACLSyncMapAS.STR_MVC_STORAGE_NAME_GROUP_PLUG_IN);
      myListModel.SetMapType(GlobalACLSyncMapListModel.ACL_MAP_TYPE_GROUP);
      myListModel.SetNumColumns(MAPLIST_NUMCOLUMNS);
      myListModel.SetIncludePTSelect(true);
      myListModel.SetPTObjectPosition(MAPLIST_PTOBJECTPOSITION);
      String[] arrMapSubTitles = new String[MAPLIST_NUMCOLUMNS];
      //External Group Name
      arrMapSubTitles[0] = m_asOwner.GetString(2691, "ptmsgs_portaladminmsgs");
      myListModel.InitializePTMultiColumnList(m_asOwner.GetString(1490, "ptmsgs_portaladminmsgs"),
       m_asOwner.GetString(2690, "ptmsgs_portaladminmsgs"), arrMapSubTitles,
       PTImageType.ICON_GROUP, m_asOwner.GetString(2689, "ptmsgs_portaladminmsgs"),
       new int[]{PT_CLASSIDS.PT_USERGROUP_ID}, 2);
      // Use the method below to set the value to populate a newly created input text.
      // (By default it will be a null array, so it MUST be initialized.
      String[] arrMapDefaults = new String[MAPLIST_NUMCOLUMNS];
      arrMapDefaults[0] = "";
      myListModel.SetDefaultValue(arrMapDefaults);
      myListModel.SetTreeMultiSelect(true);
      myListModel.SetTreeSubTitle(m_asOwner.GetString(2146, "ptmsgs_portaladminmsgs"));
      myListModel.SetTreeTitle(m_asOwner.GetString(2147, "ptmsgs_portaladminmsgs"));
      //Set the message and instructions to display when the list is empty.
      myListModel.SetEmptyListMessages(m_asOwner.GetString(1494, "ptmsgs_portaladminmsgs"),
       m_asOwner.GetString(1495, "ptmsgs_portaladminmsgs"));
      //Get the name value pairs from the model in edit mode,
      //and populate them in here
      try
      {
        // Get all the group mappings from the ACL Sync Map
        IPTACLSyncMap ptSyncMap = (IPTACLSyncMap) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_ACLSYNCMAP,
         false);
        ASQueryResultWrapper ptqrGroups = new ASQueryResultWrapper(ptSyncMap.QueryMappedItems(PT_ACLSYNC_ITEMTYPES.PT_ACLSYNCITEM_USERGROUP));
        XPArrayList xpList = ASQueryResultWrapper.ConvertToArrayList(ptqrGroups,
         PT_PROPIDS.PT_PROPID_ACLSYNC_TOSTRING);
        Object[][] oRows = new Object[xpList.GetSize()][];
        for (int j2c_index = 0;j2c_index < xpList.GetSize();j2c_index++)
        {
          oRows[j2c_index] = new Object[MAPLIST_NUMCOLUMNS + 4];
        }
        // TODO CR not used, commenting out.
        //String strObjName;
        for (int i = 0;i < xpList.GetSize();i++)
        {
          XPArrayList lstExternalGroups = new XPArrayList();
          String strPTGroup = (String) xpList.GetElement(i);
          ASQueryResultWrapper ptqrPTGroups = new ASQueryResultWrapper(ptSyncMap.QueryMappingsForItem(PT_ACLSYNC_ITEMTYPES.PT_ACLSYNCITEM_USERGROUP,
           strPTGroup));
          for (int j = 0;j < ptqrPTGroups.GetCount();j++)
          {
            lstExternalGroups.Add(ptqrPTGroups.GetFields(j, PT_PROPIDS.PT_PROPID_ACLSYNC_FROMSTRING));
            }
          // Set the class ID
          oRows[i][GlobalACLSyncMapListModel.EXPLIST_COL_CLASSID + MAPLIST_PTOBJECTPOSITION] = PT_CLASSIDS.PT_USERGROUP_ID;
          // Set the Object ID.  Bogus ID since we don't really care about the object
          oRows[i][GlobalACLSyncMapListModel.EXPLIST_COL_OBJECTID + MAPLIST_PTOBJECTPOSITION] = -12345;
          // Set the Object name
          oRows[i][GlobalACLSyncMapListModel.EXPLIST_COL_STRING + MAPLIST_PTOBJECTPOSITION] = strPTGroup;
          // Set the elements
          XPStringBuilder sbMappedItems = new XPStringBuilder(100);
          for (int j = 0;j < lstExternalGroups.GetSize();j++)
          {
            sbMappedItems.Append(lstExternalGroups.GetElement(j));
            if (j < (lstExternalGroups.GetSize() - 1))
            {
              sbMappedItems.Append(",");
            }
          }
          if (1 == MAPLIST_PTOBJECTPOSITION)
          {
            oRows[i][1] = sbMappedItems.ToString();
          } else
          {
            oRows[i][4] = sbMappedItems.ToString();
          }
        }
        // Then add the array to the list
        myListModel.SetDefaultObjects(oRows);
      }
      catch (Exception e)
      {
        log.Error("GlobalACLSyncMapModel::StartGroupMapList\tAn error occurred when trying populate the ACL Sync map group list.  Exception: " + e);
        throw new XPException(m_asOwner.GetString(1498, "ptmsgs_portaladminmsgs"),
         e);
      }
    }
    
    /// <summary>
    /// Helper method to start the migration history page.
    /// </summary>
    public virtual void StartMigrationHistory()
    {
      ((MigrationHistoryPluginModel) m_asOwner.GetModel(MigrationHistoryPluginModel.STR_MVC_CLASS_NAME)).InitializeHistory(PT_CLASSIDS.PT_GLOBALOBJECT_ID,
       PT_GLOBALOBJECTS.PT_GLOBAL_ACLSYNCMAP);
    }
    
    /// <summary>
    /// Saves the data on the group page.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>int Page status for server side validation</returns>
    private int SaveGroupPage(XPHashtable _htFormData)
    {
      try
      {
        //First, save the existing section to the activity space
        ((GlobalACLSyncMapListModel) m_asOwner.GetModel(GlobalACLSyncMapAS.STR_MVC_STORAGE_NAME_GROUP_PLUG_IN)).SaveSection(_htFormData);
        //Now retrieve the non Plumtree select elements
        Object[][] arrMappingInfo = ((GlobalACLSyncMapListModel) m_asOwner.GetModel(GlobalACLSyncMapAS.STR_MVC_STORAGE_NAME_GROUP_PLUG_IN)).GetListValues();
        //m_arrDomainMapInfo
        PTGrowableSortedArrayWrapper arrItems;
        arrItems = ((GlobalACLSyncMapListModel) m_asOwner.GetModel(GlobalACLSyncMapAS.STR_MVC_STORAGE_NAME_GROUP_PLUG_IN)).GetItems();
        int intNumItems = arrItems.GetCount();
        m_arrGroupMapInfo = new Object[2][];
        for (int j2c_index = 0;j2c_index < 2;j2c_index++)
        {
          m_arrGroupMapInfo[j2c_index] = new Object[intNumItems];
        }
        if (intNumItems > 0)
        {
          for (int i = 0;i < intNumItems;i++)
          {
            m_arrGroupMapInfo[0][i] = arrItems.GetItem(i, GlobalACLSyncMapListModel.EXPLIST_COL_STRING + MAPLIST_PTOBJECTPOSITION);
            String group = (String) arrMappingInfo[0][i];
            m_arrGroupMapInfo[1][i] = group;
          }
        }
      }
      catch (Exception e)
      {
        log.Error("GlobalACLSyncMapModel::SaveGroupPage " + m_asOwner.GetString(1496,
         "ptmsgs_portaladminmsgs") + "  " + "The following exception occured: " + e);
        throw new XPException(m_asOwner.GetString(1496, "ptmsgs_portaladminmsgs"),
         e);
      }
      return GlobalACLSyncMapRepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Saves the data on the main page.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>int Page status for server side validation</returns>
    private int SaveMainPage(XPHashtable _htFormData)
    {
      try
      {
        //First, save the existing section to the activity space
        ((GlobalACLSyncMapListModel) m_asOwner.GetModel(GlobalACLSyncMapAS.STR_MVC_STORAGE_NAME_DOMAIN_PLUG_IN)).SaveSection(_htFormData);
        //Now retrieve the non Plumtree select elements
        Object[][] arrMappingInfo = ((GlobalACLSyncMapListModel) m_asOwner.GetModel(GlobalACLSyncMapAS.STR_MVC_STORAGE_NAME_DOMAIN_PLUG_IN)).GetListValues();
        //m_arrDomainMapInfo
        PTGrowableSortedArrayWrapper arrItems;
        arrItems = ((GlobalACLSyncMapListModel) m_asOwner.GetModel(GlobalACLSyncMapAS.STR_MVC_STORAGE_NAME_DOMAIN_PLUG_IN)).GetItems();
        int intNumItems = arrItems.GetCount();
        m_arrDomainMapInfo = new Object[2][];
        for (int j2c_index = 0;j2c_index < 2;j2c_index++)
        {
          m_arrDomainMapInfo[j2c_index] = new Object[intNumItems];
        }
        if (intNumItems > 0)
        {
          for (int i = 0;i < intNumItems;i++)
          {
            m_arrDomainMapInfo[0][i] = arrItems.GetItem(i, GlobalACLSyncMapListModel.EXPLIST_COL_STRING + MAPLIST_PTOBJECTPOSITION);
            String domain = (String) arrMappingInfo[0][i];
            m_arrDomainMapInfo[1][i] = domain;
          }
        }
      }
      catch (Exception e)
      {
        log.Error("GlobalACLSyncMapModel::SaveMainPage " + m_asOwner.GetString(1486,
         "ptmsgs_portaladminmsgs") + "  " + "The following exception occured: " + e);
        throw new XPException(m_asOwner.GetString(1486, "ptmsgs_portaladminmsgs"),
         e);
      }
      return GlobalACLSyncMapRepostControl.PAGE_STATUS_VALID;
    }
  }
}
// END ***************** repost action methods *****************
