using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.common.helpers;
using com.plumtree.portaluiinfrastructure.classtypedescriptors;
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.portaluiinfrastructure.resultwrapper;
using com.plumtree.portaluiinfrastructure.search;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.search;
using com.plumtree.server.search.admin;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.pagecontrols.datepicker;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker;
using com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.jsutil;
namespace com.plumtree.portalpages.admin.serversettings.search
{
  /// <author>OlegS</author>
  public class SearchServerSettingsEditorModel : EditorModel,ISearchServerSettingsEditorModelRO,
  PropertyValidatorOwner
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
     "com.plumtree.portalpages.admin.serversettings.searchcluster.SearchServerSettingsEditorModel");
    public const String INDEXPREF_SAVEDIDS = "IndexingStatusSavedClassIds";
    public const String STR_MVC_CLASS_NAME = "SearchServerSettingsEditor";
    public const String DateTimePickerKey = "SSSE_DP1";
    private JSDateTimePickerMgr m_ptJSDateTimePickerMgr = null;
    private PageMgr m_ptPageMgr;
    private ADatePickerComponent ptJSDateTimePicker;
    internal IXPPropertyBag m_statusBag;
    internal IntegerProperty m_IntegerProperty = null;
    internal IntegerPropertyWithRange m_IntegerPropertyWithRange_1024_65535 = null;
    internal IntegerPropertyWithRange m_IntegerPropertyWithRange_1_1000000 = null;
    internal MandatoryProperty m_MandatoryProperty = null;
    internal OptionalProperty m_OptionalProperty = null;
    internal XPArrayList m_properties = null;
    //	PropertyValue m_failoverHost;
    //	PropertyValue m_failoverPort;
    internal PropertyValue m_fileSizePerBulkImport;
    internal PropertyValue m_indexingHost;
    internal PropertyValue m_indexingPort;
    internal PropertyValue m_maxCharsPerDocument;
    internal PropertyValue m_maxTermsForKeywordHighlighting;
    internal PropertyValue m_numCardsPerBulkImport;
    //	PropertyValue m_queryHost;
    //	PropertyValue m_queryPort;
    internal String m_strRepairRate;
    // member variables
    internal XPStringBuilder m_errors = null;
    // Indexing status
    private XPArrayList m_indexingStatusList = null;
    private int[] arAllClassIDs = null;
    private static readonly String PORTAL_ADMIN_MSGS = ResourceConstants.PORTALADMINMSGS;
    private static readonly String PORTAL_COMMON_MSGS = ResourceConstants.PORTALCOMMONMSGS;
    public SearchServerSettingsEditorModel() : base()
    {
      m_IntegerProperty = new IntegerProperty(this);
      m_IntegerPropertyWithRange_1024_65535 = new IntegerPropertyWithRange(this,
       1024, 65535);
      m_IntegerPropertyWithRange_1_1000000 = new IntegerPropertyWithRange(this,
       1, 1000000);
      m_MandatoryProperty = new MandatoryProperty(this);
      m_OptionalProperty = new OptionalProperty(this);
      m_properties = new XPArrayList();
      m_errors = new XPStringBuilder();
    }
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
      // This will initialize the manager.
      GetDateTimePickerControlManager();
      // initialize member variables with extracted string, which require m_asOwner set above
      //		m_failoverHost = new PropertyValue(m_properties,m_asOwner.GetString(2038, PORTAL_ADMIN_MSGS), null,PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_FAILOVER_QUERY_HOST_NAME,SearchServerSettingsEditorView.HTMLINPUT_FailoverHostName,new PropertyValidator[] { m_OptionalProperty });
      //
      //		m_failoverPort = new PropertyValue(m_properties,m_asOwner.GetString(2036, PORTAL_ADMIN_MSGS), null,PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_FAILOVER_QUERY_PORT_NUMBER,SearchServerSettingsEditorView.HTMLINPUT_FailoverPortNumber,new PropertyValidator[]{new OptionalProperty(this, m_failoverHost),m_IntegerPropertyWithRange_1024_65535});
      m_fileSizePerBulkImport = new PropertyValue(m_properties, m_asOwner.GetString(2045,
       PORTAL_ADMIN_MSGS), null, PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_BULK_IMPORT_BYTE_LIMIT,
       SearchServerSettingsEditorView.HTMLINPUT_FileSizePerBulkImport, new PropertyValidator[]{m_MandatoryProperty,
       m_IntegerPropertyWithRange_1_1000000});
      m_indexingHost = new PropertyValue(m_properties, m_asOwner.GetString(2035,
       PORTAL_ADMIN_MSGS), null, PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_CLUSTER_HOST_NAME,
       SearchServerSettingsEditorView.HTMLINPUT_IndexingHostName, new PropertyValidator[]{m_MandatoryProperty});
      m_indexingPort = new PropertyValue(m_properties, m_asOwner.GetString(2036,
       PORTAL_ADMIN_MSGS), null, PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_CLUSTER_HOST_PORT,
       SearchServerSettingsEditorView.HTMLINPUT_IndexingPortNumber, new PropertyValidator[]{m_MandatoryProperty,
       m_IntegerPropertyWithRange_1024_65535});
      m_maxCharsPerDocument = new PropertyValue(m_properties, m_asOwner.GetString(2043,
       PORTAL_ADMIN_MSGS), null, PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_ENCODED_DOCUMENT_BYTE_LIMIT,
       SearchServerSettingsEditorView.HTMLINPUT_MaxCharsPerDocument, new PropertyValidator[]{m_MandatoryProperty,
       m_IntegerPropertyWithRange_1_1000000});
      m_maxTermsForKeywordHighlighting = new PropertyValue(m_properties, m_asOwner.GetString(2652,
       PORTAL_ADMIN_MSGS), null, PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_MAX_NUM_KWICS,
       SearchServerSettingsEditorView.HTMLINPUT_MaxTermsForKeywordHighlighting,
       new PropertyValidator[]{m_MandatoryProperty, m_IntegerPropertyWithRange_1_1000000});
      //Note that nextRepairTime and numberOfDaysBetweenRepair are now handled outside the PropertyValue model.
      //m_nextRepairTime = new DateTimePropertyValue(m_properties,m_asOwner.GetString(2039, PORTAL_ADMIN_MSGS), null, 1,PT_SEARCHUPDATE_SETTINGS.PT_SUACONFIG_NEXTREPAIRDATE,ptJSDateTimePicker,new PropertyValidator[] { m_MandatoryProperty, new DateProperty(this) });
      m_numCardsPerBulkImport = new PropertyValue(m_properties, m_asOwner.GetString(2044,
       PORTAL_ADMIN_MSGS), null, PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_BULK_IMPORT_SIZE,
       SearchServerSettingsEditorView.HTMLINPUT_NumCardsPerBulkImport, new PropertyValidator[]{m_MandatoryProperty,
       m_IntegerPropertyWithRange_1_1000000});
    }
    // m_numberOfDaysBetweenRepair = new PropertyValue(m_properties,m_asOwner.GetString(2041, PORTAL_ADMIN_MSGS), null, 1,PT_SEARCHUPDATE_SETTINGS.PT_SUACONFIG_REPAIRRATE,SearchServerSettingsEditorView.HTMLINPUT_NumberOfDaysBetweenRepair,new PropertyValidator[]{m_MandatoryProperty, m_IntegerPropertyWithRange_1_1000000});
    //		m_queryHost = new PropertyValue(m_properties,m_asOwner.GetString(2037, PORTAL_ADMIN_MSGS), null,PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_QUERY_HOST_NAME,SearchServerSettingsEditorView.HTMLINPUT_QueryHostName,new PropertyValidator[] { m_MandatoryProperty });
    //
    //		m_queryPort	= new PropertyValue(m_properties,m_asOwner.GetString(2036, PORTAL_ADMIN_MSGS), null,PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_QUERY_PORT_NUMBER,SearchServerSettingsEditorView.HTMLINPUT_QueryPortNumber,new PropertyValidator[]{m_MandatoryProperty, m_IntegerPropertyWithRange_1024_65535});
    // interface PropertyValidatorOwner
    public virtual XPStringBuilder GetErrorString()
    {
      return m_errors;
    }
    //	/**
    //	 *
    //	 *
    //	 * @return
    //	 */
    //	public String GetFailoverHost()
    //	{
    //		return m_failoverHost.text();
    //	}
    //
    //	/**
    //	 *
    //	 *
    //	 * @return
    //	 */
    //	public String GetFailoverPort()
    //	{
    //		return m_failoverPort.text();
    //	}
    
    /// <returns></returns>
    public virtual String GetFileSizePerBulkImport()
    {
      return m_fileSizePerBulkImport.text();
    }
    // interface ISearchServerSettingsEditorModelRO
    public virtual String GetIndexingHost()
    {
      return m_indexingHost.text();
    }
    
    /// <returns></returns>
    public virtual String GetIndexingPort()
    {
      return m_indexingPort.text();
    }
    
    /// <returns></returns>
    public virtual String GetMaxCharsPerDocument()
    {
      return m_maxCharsPerDocument.text();
    }
    
    /// <returns></returns>
    public virtual String GetMaxTermsForKeywordHighlighting()
    {
      return m_maxTermsForKeywordHighlighting.text();
    }
    
    /// <returns></returns>
    public virtual JSDateTimePickerMgr GetDateTimePickerControlManager()
    {
      if (m_ptJSDateTimePickerMgr == null)
      {
        String strImageServerBaseURL = ImgSvrURLHelper.GetInstance().GetRootURL(m_asOwner.GetIsSecuredSpace());
        m_ptPageMgr = new PageMgr(m_asOwner.GetAccessStyle(), PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(m_asOwner),
         PTPersonalSettingsHelper.GetPersonalSettingValue_TimeZone(m_asOwner),
         strImageServerBaseURL, ConfigHelper.GetPortalImageServerConnectionURL(m_asOwner));
        JSRegistry ptJSRegistry = JSRegistry.GetRegistry(strImageServerBaseURL,
         ConfigHelper.GetPortalImageServerConnectionURL(m_asOwner));
        m_ptJSDateTimePickerMgr = new JSDateTimePickerMgr(ptJSRegistry, new JSDatePickerVersionMgr(),
         m_ptPageMgr);
        m_ptPageMgr.RegisterComponentMgr(m_ptJSDateTimePickerMgr);
      }
      return m_ptJSDateTimePickerMgr;
    }
    
    /// <returns></returns>
    public virtual String GetNumCardsPerBulkImport()
    {
      return m_numCardsPerBulkImport.text();
    }
    
    /// <returns></returns>
    public virtual String GetNumberOfDaysBetweenRepair()
    {
      return m_strRepairRate;
    }
    //	/**
    //	 *
    //	 *
    //	 * @return
    //	 */
    //	public String GetQueryHost()
    //	{
    //		return m_queryHost.text();
    //	}
    //
    //	/**
    //	 *
    //	 *
    //	 * @return
    //	 */
    //	public String GetQueryPort()
    //	{
    //		return m_queryPort.text();
    //	}
    
    /// <returns></returns>
    public virtual IXPPropertyBag GetStatusPropertyBag()
    {
      IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
      IPTSearchFactory searchFactory = ptSession.GetSearchFactory();
      IPTSAdmin searchAdmin = searchFactory.createAdmin();
      // What exceptions can this throw??? !!!
      m_statusBag = searchAdmin.retrieveStatus();
      return m_statusBag;
    }
    
    /// <returns></returns>
    public virtual IXPPropertyBag GetClusterStatusPropertyBag()
    {
      IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
      IPTSearchFactory searchFactory = ptSession.GetSearchFactory();
      IPTSAdmin searchAdmin = searchFactory.createAdmin();
      // What exceptions can this throw??? !!!
      m_statusBag = searchAdmin.retrieveClusterStatus();
      m_statusBag.Write("LAST_TRANSACTION_ID", searchAdmin.getLastSearchTransactionIDAsString());
      return m_statusBag;
    }
    public virtual String getLastSubmittedSearchTransactioID()
    {
      IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
      IPTSearchFactory searchFactory = ptSession.GetSearchFactory();
      IPTSAdmin searchAdmin = searchFactory.createAdmin();
      return searchAdmin.getLastSearchTransactionIDAsString();
    }
    // implementation
    public override bool CheckEditorAccessAndStart(XPHashtable _htFormData)
    {
      IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
      // User must be in the Administrators group.
      if (!PlumtreeHelpers.CheckCurrentUserIsAdmin(ptSession))
      {
        return false;
      }
      InitializePropertyValues();
      SetEnableApplyButton(true);
      return true;
    }
    public virtual void InitializePropertyValues()
    {
      IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
      IPTServerConfigSettings ptSC = (IPTServerConfigSettings) ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
       false);
      PropertySource src = new ServerConfigSettings(ptSC);
      traceDebug("Property Values at Start Editor: ");
      // Note that these get() calls are essential for proper
      // initialization of these PropertyValues from the database,
      // it's not just debugging.
      for (int i = 0;i < m_properties.GetSize();i++)
      {
        PropertyValue pv = (PropertyValue) m_properties.GetElement(i);
        pv.Get(src, PropertyValue.SRC_SERVER);
        traceDebug(i + ". " + pv.name() + " = " + pv.text());
      }
      // Note that the nextRepairDate and numberOfDaysBetweenRepair are no longer initialized above.
      XPDateTime repairDate = null;
      try
      {
        IPTSearchMaintenance ptSM = (IPTSearchMaintenance) ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_CATALOG_SEARCH_UPDATE_AGENT,
         false);
        repairDate = ptSM.GetSettingAsXPDateTime(PT_SEARCHUPDATE_SETTINGS.PT_SUACONFIG_NEXTREPAIRDATE);
        int repairRate = ptSM.GetSettingAsInt(PT_SEARCHUPDATE_SETTINGS.PT_SUACONFIG_REPAIRRATE);
        m_strRepairRate = XPConvert.ToString(repairRate);
      }
      catch (Exception e)
      {
        repairDate = new XPDateTime();
        m_strRepairRate = "";
      }
      CreateDatePicker(repairDate);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SearchServerSettingsEditorModel();
    }
    public override void Finish()
    {
      // Code now in DoApplyChanges().
      DoApplyChanges();
    }
    public virtual void DoApplyChanges()
    {
      IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
      IPTServerConfigSettings ptSC = (IPTServerConfigSettings) ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
       false);
      PropertySource src = new ServerConfigSettings(ptSC);
      traceDebug("Property Values at Finish: ");
      for (int i = 0;i < m_properties.GetSize();i++)
      {
        PropertyValue pv = (PropertyValue) m_properties.GetElement(i);
        traceDebug(i + ". " + pv.name() + " = " + pv.text());
        pv.Set(src, PropertyValue.SRC_SERVER);
        traceDebug("Set " + i + ". " + pv.name() + " = " + pv.text());
      }
      traceDebug("Property Listing complete ");
      SaveNextRepairDateAndRate(ptSession);
      SaveClassIds(m_indexingStatusList);
    }
    /*
     * Get the date from the date picker and store it in the IPTSearchMaintenance
     * collection.
     */
     
    protected virtual void SaveNextRepairDateAndRate(IPTSession ptSession)
    {
      IPTSearchMaintenance maint = (IPTSearchMaintenance) ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_CATALOG_SEARCH_UPDATE_AGENT,
       false);
      if (maint == null)
      {
        return;
      }
      maint.SetSetting(PT_SEARCHUPDATE_SETTINGS.PT_SUACONFIG_REPAIRRATE, XPConvert.ToInteger(m_strRepairRate));
      JSDateTimePicker picker = (JSDateTimePicker) GetDateTimePickerControlManager().GetComponent(DateTimePickerKey);
      if (picker != null)
      {
        String datetime = picker.GetDateTimeAsString();
        traceDebug("Attempting to set next repair time to " + datetime);
        XPDateTime time = picker.GetDateTimeAsDateTime();
        // If time is in the past, set it to now plus a minute.
        XPDateTime now = new XPDateTime();
        XPDateTime nowPlusAMinute = new XPDateTime(XPSystem.GetCurrentTimeMilliSec() + 60000);
        if (time.Before(now))
        {
          traceDebug("Next repair time is in the past. Setting to " + nowPlusAMinute.ToString());
          time = nowPlusAMinute;
          picker.Init("mainform", time, m_ptPageMgr, DateTimePickerKey);
        }
        maint.SetSetting(PT_SEARCHUPDATE_SETTINGS.PT_SUACONFIG_NEXTREPAIRDATE,
         time);
      }
    }
    /*
     * Update the date picker object from the form data.
     */
     
    protected virtual void updateRepairDateAndRateFromForm(XPHashtable formData)
    {
      JSDateTimePicker picker = (JSDateTimePicker) GetDateTimePickerControlManager().GetComponent(DateTimePickerKey);
      if (picker != null)
      {
        picker.StoreUserInput(formData);
      }
      int repairRate = 0;
      try
      {
        m_strRepairRate = ((String[]) formData.GetElement(SearchServerSettingsEditorView.HTMLINPUT_NumberOfDaysBetweenRepair))[0];
        m_strRepairRate = XPStringUtility.Trim(m_strRepairRate);
        repairRate = XPConvert.ToInteger(m_strRepairRate);
      }
      catch (Exception e)
      {
        m_errors.Append(" ");
        // "Number of days between repair"
        m_errors.Append(m_asOwner.GetString(2041, PORTAL_ADMIN_MSGS));
        m_errors.Append(". (");
        // "Value must be numeric."
        m_errors.Append(m_asOwner.GetString(57, PORTAL_COMMON_MSGS));
        m_errors.Append(")");
        return;
      }
      if ((repairRate <= 0) || (repairRate > 1000000))
      {
        m_errors.Append(" ");
        // "Number of days between repair"
        m_errors.Append(m_asOwner.GetString(2041, PORTAL_ADMIN_MSGS));
        m_errors.Append(". (");
        // "Value must be in range"
        m_errors.Append(m_asOwner.GetString(58, PORTAL_COMMON_MSGS));
        m_errors.Append(" [1, 1000000])");
      }
    }
    protected virtual void CreateDatePicker(XPDateTime repairDate)
    {
      if (repairDate.GetTimeInMilliSeconds() > 0)
      {
        traceDebug("Getting next repair time " + repairDate.GetTimeInMilliSeconds());
        ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");
        ptJSDateTimePicker.Init("mainform", repairDate, m_ptPageMgr, DateTimePickerKey);
        GetDateTimePickerControlManager().RegisterComponent(ptJSDateTimePicker);
        } else
      {
        ptJSDateTimePicker = (ADatePickerComponent) m_ptPageMgr.AddPageControl("DateTimePicker");
        ptJSDateTimePicker.Init("mainform", new XPDateTime(), m_ptPageMgr,
         DateTimePickerKey);
        GetDateTimePickerControlManager().RegisterComponent(ptJSDateTimePicker);
        }
    }
    
    /// <returns></returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <param name="_sPageName"></param>
    /// <param name="_htFormData"></param>
    /// <returns></returns>
    public override int SavePage(String _sPageName, XPHashtable _htFormData)
    {
      TraceHelper.Trace("SavePage(" + _sPageName + ")");
      TraceHelper.TraceQueryString(_htFormData);
      int result = RepostControl.PAGE_STATUS_VALID;
      if (_sPageName.Equals(SearchServerSettingsEditorDP.STR_MVC_CLASS_NAME))
      {
        result = SaveMainPage(_htFormData);
      }
      return result;
    }
    
    /// <returns></returns>
    public virtual AActivitySpace asOwner()
    {
      return m_asOwner;
    }
    
    /// <summary>
    /// Trace a message if debug tracing is enabled for the UI component.
    /// This method could be moved into the EditorModel base class, along
    /// with trace() and traceEnabled().
    /// </summary>
    protected virtual void traceDebug(String msg)
    {
      if (TraceHelper.Enabled())
      {
        TraceHelper.Trace(msg);
      }
    }
    public virtual void ActionOnRepost()
    {
    }
    
    /// <param name="_htFormData"></param>
    /// <returns></returns>
    protected virtual int SaveMainPage(XPHashtable _htFormData)
    {
      PropertySource src = new QueryString(_htFormData);
      int result = RepostControl.PAGE_STATUS_VALID;
      m_errors.SetLength(0);
      for (int i = 0;i < m_properties.GetSize();i++)
      {
        PropertyValue pv = (PropertyValue) m_properties.GetElement(i);
        pv.Get(src, PropertyValue.SRC_FORM);
        pv.validate();
      }
      updateRepairDateAndRateFromForm(_htFormData);
      if (m_errors.GetLength() > 0)
      {
        result = RepostControl.PAGE_STATUS_INVALID;
        String msg = m_asOwner.GetString(2046, PORTAL_ADMIN_MSGS) + m_errors.ToString();
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         null, msg, msg);
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        }
      return result;
    }
    public class IndexingCount
    {
      public const int ALL_OTHERS_ID = -1;
      internal int classId = 0;
      internal String className = null;
      internal int totalDatabaseCount = 0;
      internal int numberDirty = 0;
      internal int numberAwaitingDelete = 0;
      internal int numberInSearchIndex = 0;
      public virtual int GetClassId()
      {
        return classId;
      }
      public virtual String GetClassName()
      {
        return className;
      }
      public virtual int GetTotalDatabaseCount()
      {
        return totalDatabaseCount;
      }
      public virtual int GetNumberDirty()
      {
        return numberDirty;
      }
      public virtual int GetNumberAwaitingDelete()
      {
        return numberAwaitingDelete;
      }
      public virtual int GetNumberInSearchIndex()
      {
        return numberInSearchIndex;
      }
    }
    protected virtual void UpdateIndexingCount(IndexingCount ic, IXPPropertyBag[] bags)
    {
      if (ic.classId != -1)
      {
        ic.totalDatabaseCount = GetCountForClassID(bags[0], XPConvert.ToString(ic.classId));
        ic.numberInSearchIndex = GetCountSIClassID(ic.classId);
        ic.numberDirty = GetCountForClassID(bags[1], XPConvert.ToString(ic.classId));
        ic.numberAwaitingDelete = GetCountForClassID(bags[2], XPConvert.ToString(ic.classId));
        // For cards, add in the number for upload to the number Dirty.
        if (ic.classId == PT_CLASSIDS.PT_CATALOGCARD_ID)
        {
          ic.numberDirty+=GetCountForClassID(bags[3], XPConvert.ToString(ic.classId));
          }
      }
    }
    protected virtual void UpdateIndexingCountAllOthers(IndexingCount ic,
     IXPPropertyBag[] bags, XPArrayList icList)
    {
      ic.totalDatabaseCount = GetDBCountAllIDs(bags, icList);
      ic.numberInSearchIndex = GetSICountAllIDs(icList);
      ic.numberDirty = GetTotalAllClassID(bags[1], icList);
      ic.numberAwaitingDelete = GetTotalAllClassID(bags[2], icList);
    }
    protected virtual void LoadIndexingCount(IndexingCount ic, int classID,
     String className, IXPPropertyBag[] bags)
    {
      ic.classId = classID;
      ic.className = className;
      UpdateIndexingCount(ic, bags);
    }
    
    /// <summary>
    /// Called from the IndexingStatusView to gather the data
    /// </summary>
    /// <returns></returns>
    public virtual XPArrayList GetIndexingCounts()
    {
      ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();
      String strLang = PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(m_asOwner).GetLanguage();
      arAllClassIDs = ptClassTypeDescManager.GetOrderedListOfClassIDs(strLang);
      if (m_indexingStatusList == null)
      {
        InitIndexingStatusList();
      } else
      {
        UpdateStatusInfo(m_indexingStatusList);
      }
      return m_indexingStatusList;
    }
    protected virtual bool ListContainsClassId(int classId, XPArrayList indexCountList)
    {
      if (indexCountList == null)
      {
        return false;
      }
      IXPEnumerator xpEnum = indexCountList.GetEnumerator();
      while (xpEnum.MoveNext())
      {
        IndexingCount ic = (IndexingCount) xpEnum.GetCurrent();
        if (ic.classId == classId)
        {
          return true;
        }
      }
      return false;
    }
    protected virtual void UpdateStatusInfo(XPArrayList indexCountList)
    {
      IXPPropertyBag[] bags = GenerateSearchUpdateReport();
      IXPEnumerator xpEnum = indexCountList.GetEnumerator();
      IndexingCount allOthers = null;
      while (xpEnum.MoveNext())
      {
        IndexingCount ic = (IndexingCount) xpEnum.GetCurrent();
        if (ic.classId == IndexingCount.ALL_OTHERS_ID)
        {
          allOthers = ic;
        } else
        {
          UpdateIndexingCount(ic, bags);
        }
      }
      if (allOthers != null)
      {
        UpdateIndexingCountAllOthers(allOthers, bags, m_indexingStatusList);
        }
    }
    public virtual XPArrayList GetUndisplayedNamesAndTypes()
    {
      ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();
      String strLang = PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(m_asOwner).GetLanguage();
      int[] searchAbleIDs = FilterToSearchableIDs(arAllClassIDs);
      XPArrayList retval = new XPArrayList();
      for (int i = 0;i < searchAbleIDs.Length;i++)
      {
        if (!ListContainsClassId(searchAbleIDs[i], m_indexingStatusList))
        {
          IndexingCount ic = new IndexingCount();
          ic.classId = searchAbleIDs[i];
          ic.className = ptClassTypeDescManager.GetClassTypeLocalizedName(ic.classId,
           strLang);
          retval.Add(ic);
        }
      }
      return retval;
    }
    protected virtual void InsertInOrder(XPArrayList icArrayList, IndexingCount ic)
    {
      if (icArrayList.GetSize() == 0)
      {
        icArrayList.Add(ic);
        return;
      }
      // Special case of all others goes at end of list.
      if (ic.GetClassId() == IndexingCount.ALL_OTHERS_ID)
      {
        icArrayList.Add(ic);
        return;
      }
      // Insert an IndexingCount into the array in the order that the classids come back for the current
      // language.
      IXPEnumerator xpEnum = icArrayList.GetEnumerator();
      int index = 0;
      while (xpEnum.MoveNext())
      {
        IndexingCount listVal = (IndexingCount) xpEnum.GetCurrent();
        if (ComesAfter(listVal.GetClassId(), ic.GetClassId(), arAllClassIDs) || listVal.GetClassId() == IndexingCount.ALL_OTHERS_ID)
        {
          icArrayList.Insert(index, ic);
          return;
        }
        index++;
      }
    }
    protected virtual bool ComesAfter(int test, int id, int[] arrayOfInts)
    {
      int testIndex = -1;
      int idIndex = -1;
      for (int i = 0;i < arrayOfInts.Length;i++)
      {
        if (arrayOfInts[i] == test)
        {
          testIndex = i;
        }
        if (arrayOfInts[i] == id)
        {
          idIndex = i;
        }
      }
      return (testIndex > idIndex);
    }
    public virtual void AddTypeToIndexingStatus(int classid)
    {
      ClassTypeDescManager ctdMan = PTConfigHelper.GetPTClassTypeDescrManager();
      String strLang = PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(m_asOwner).GetLanguage();
      IXPPropertyBag[] bags = GenerateSearchUpdateReport();
      IndexingCount ic = new IndexingCount();
      // m_asOwner.GetString(2297, PORTAL_ADMIN_MSGS), bags);
      LoadIndexingCount(ic, classid, ctdMan.GetClassTypeLocalizedName(classid,
       strLang), bags);
      InsertInOrder(m_indexingStatusList, ic);
    }
    public virtual void DeleteTypeFromIndexingStatus(int classid)
    {
      int removeAt = -1;
      int index = 0;
      IXPEnumerator xpEnum = m_indexingStatusList.GetEnumerator();
      while (xpEnum.MoveNext())
      {
        SearchServerSettingsEditorModel.IndexingCount ic = (SearchServerSettingsEditorModel.IndexingCount) xpEnum.GetCurrent();
        if (ic.GetClassId() == classid)
        {
          removeAt = index;
          break;
        }
        index++;
      }
      if (removeAt >= 0)
      {
        m_indexingStatusList.RemoveAt(removeAt);
      }
    }
    public virtual void AddAllUnusedISTypes()
    {
      IXPPropertyBag[] bags = GenerateSearchUpdateReport();
      XPArrayList unused = GetUndisplayedNamesAndTypes();
      IXPEnumerator xpEnum = unused.GetEnumerator();
      while (xpEnum.MoveNext())
      {
        SearchServerSettingsEditorModel.IndexingCount ic = (SearchServerSettingsEditorModel.IndexingCount) xpEnum.GetCurrent();
        LoadIndexingCount(ic, ic.GetClassId(), ic.GetClassName(), bags);
        InsertInOrder(m_indexingStatusList, ic);
      }
    }
    protected virtual void InitIndexingStatusList()
    {
      m_indexingStatusList = new XPArrayList();
      ClassTypeDescManager ctdMan = PTConfigHelper.GetPTClassTypeDescrManager();
      String strLang = PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(m_asOwner).GetLanguage();
      IndexingCount allO = new IndexingCount();
      // -1 == all others
      allO.classId = IndexingCount.ALL_OTHERS_ID;
      // 2502 All Others
      allO.className = m_asOwner.GetString(2502, PORTAL_ADMIN_MSGS);
      InsertInOrder(m_indexingStatusList, allO);
      int[] classIds = GetSavedIds();
      IXPPropertyBag[] bags = GenerateSearchUpdateReport();
      for (int i = 0;i < classIds.Length;i++)
      {
        IndexingCount ic = new IndexingCount();
        LoadIndexingCount(ic, classIds[i], ctdMan.GetClassTypeLocalizedName(classIds[i],
         strLang), bags);
        InsertInOrder(m_indexingStatusList, ic);
      }
      UpdateIndexingCountAllOthers(allO, bags, m_indexingStatusList);
    }
    /*
     * Returns count of things in the database for a particular class id or 0 if not searchable.
     */
     
    public virtual int GetCountDBClassID(int classid)
    {
      if (!ClassIDSearchable(classid))
      {
        return 0;
      }
      int retval = 0;
      int nFields = PT_PROPIDS.PT_PROPID_OBJECTID;
      try
      {
        PTObjectManagerQueryWrapper PropMgr = new PTObjectManagerQueryWrapper(((IPTSession) m_asOwner.GetUserSession()).GetObjectManagers(classid));
        //ptqf);
        PropMgr.Query(nFields, -1, null, null);
        retval = ((IASQueryResult) PropMgr).GetCount();
      }
      catch (Exception e)
      {
      }
      // No object manager--need to look into this case
      return retval;
    }
    /*
     * Returns count of things in the search index for a particular class id or 0 if not searchable.
     */
     
    public virtual int GetCountSIClassID(int classid)
    {
      if (!ClassIDSearchable(classid))
      {
        return 0;
      }
      int[] idArray = new int[1];
      idArray[0] = classid;
      return GetCountSIClassID(idArray, null);
    }
    /*
     * Returns count of things in the search index for an array of class id.
     */
     
    public virtual int GetCountSIClassID(int[] classids, XPArrayList exceptThese)
    {
      try
      {
        IPTSearchRequest searchRequest = ((IPTSession) m_asOwner.GetUserSession()).GetSearchRequest();
        int[] filteredClassids = RemoveListIds(classids, exceptThese);
        if (filteredClassids.Length == 0)
        {
          return 0;
        }
        searchRequest.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES,
         filteredClassids);
        searchRequest.SetSettings(PT_SEARCH_SETTING.PT_SEARCHSETTING_APPS,
         XPConvert.ToInteger(PT_SEARCH_APPS.PT_SEARCH_APPS_PORTAL));
        IPTSearchQuery query = searchRequest.CreateBasicQuery("*", "PT1[1.0]");
        IPTSearchResponse response = searchRequest.Search(query);
        int retval = response.GetTotalMatches();
        return retval;
      }
      catch (SearchExecutionWarning e)
      {
        log.Error(e, "Caught WARNING in GetCountSIClassID.");
      }
      catch (SearchExecutionError e)
      {
        log.Error(e, "Caught ERROR in GetCountSIClassID.");
      }
      catch (SearchExecutionException e)
      {
      }
      // shouldn't happen; all instances should be an Error or Warning
      catch (Exception e)
      {
        log.Error(e, "Caught exception in GetCountSIClassID.");
      }
      return 0;
    }
    protected virtual IXPPropertyBag[] GenerateSearchUpdateReport()
    {
      IPTSearchMaintenance ptSearchMaintenance = (IPTSearchMaintenance) ((IPTSession) m_asOwner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_CATALOG_SEARCH_UPDATE_AGENT,
       false);
      IXPPropertyBag[] retval = new IXPPropertyBag[4];
      int[] arStatusType = {0, 1, 3, 4};
      /* This is what the above corresponds to
       String[] arStatusString = {
       "CLEAN",
       "DIRTY",
       "DELETE",
       "UPLOAD"
       };
       */
       
      for (int i = 0;i < arStatusType.Length;i++)
      {
        IXPPropertyBag pbag = ptSearchMaintenance.GetSearchUpdateStatus(arStatusType[i]);
        retval[i] = pbag;
      }
      return retval;
    }
    /*
     * Total the values in a given property bag across searchable classids (all should be searchable?)
     */
     
    public virtual int GetTotalAllClassID(IXPPropertyBag pbag, XPArrayList exceptTheseList)
    {
      IXPEnumerator xpEnum = pbag.GetEnumerator();
      int retval = 0;
      while (true == xpEnum.MoveNext())
      {
        String strClassID = (String) xpEnum.GetCurrent();
        int id = XPConvert.ToInteger(strClassID);
        if (ClassIDSearchable(id) && !ListContainsClassId(id, exceptTheseList))
        {
          retval+=pbag.ReadAsInt(strClassID);
        }
      }
      return retval;
    }
    /*
     * Returns count for a particular class id in the given bag
     */
     
    public virtual int GetCountForClassID(IXPPropertyBag pbag, String classid)
    {
      IXPEnumerator xpEnum = pbag.GetEnumerator();
      while (true == xpEnum.MoveNext())
      {
        String strClassID = (String) xpEnum.GetCurrent();
        if (XPStringUtility.EqualsIgnoreCase(classid, strClassID))
        {
          return pbag.ReadAsInt(strClassID);
        }
      }
      log.Debug(classid + " not found");
      return 0;
    }
    /*
     * Gets cound of items in db for all searchable classids.
     */
     
    protected virtual int GetDBCountAllIDs(IXPPropertyBag[] bags, XPArrayList exceptTheseList)
    {
      int retval = 0;
      int[] filteredArray = RemoveListIds(arAllClassIDs, exceptTheseList);
      for (int i = 0;i < filteredArray.Length;i++)
      {
        if (ClassIDSearchable(filteredArray[i]))
        {
          int val = GetCountForClassID(bags[0], XPConvert.ToString(filteredArray[i]));
          retval+=val;
        }
      }
      return retval;
    }
    /*
     * Get count in search index for all searchable classids.
     */
     
    protected virtual int GetSICountAllIDs(XPArrayList exceptThese)
    {
      int[] searchAbleIDs = FilterToSearchableIDs(arAllClassIDs);
      return GetCountSIClassID(searchAbleIDs, exceptThese);
    }
    //Returns true if a particular classid is searchable. Counts of things in the database are only
    //concerned with ids that this method returns true for. Note that this code is ripped from PTIndexer.cpp
    //TypeStringFromClassId and that exposing that method is the way this should be done.
    protected virtual bool ClassIDSearchable(int lClassID)
    {
      switch (lClassID){
      case PT_CLASSIDS.PT_USER_ID:
      case PT_CLASSIDS.PT_USERGROUP_ID:
      case PT_CLASSIDS.PT_AUTHSOURCE_ID:
      case PT_CLASSIDS.PT_PROFILESOURCE_ID:
      case PT_CLASSIDS.PT_CATALOGFOLDER_ID:
      case PT_CLASSIDS.PT_CATALOGCARD_ID:
      case PT_CLASSIDS.PT_ADMIN_FOLDER_ID:
      case PT_CLASSIDS.PT_FILTER_ID:
      case PT_CLASSIDS.PT_SAVEDSEARCH_ID:
      case PT_CLASSIDS.PT_DATASOURCE_ID:
      case PT_CLASSIDS.PT_PROPERTY_ID:
      case PT_CLASSIDS.PT_DOCUMENTTYPE_ID:
      case PT_CLASSIDS.PT_CRAWLER_ID:
      case PT_CLASSIDS.PT_GADGET_ID:
      case PT_CLASSIDS.PT_INVITATION_ID:
      case PT_CLASSIDS.PT_FEDERATEDPORTAL_ID:
      case PT_CLASSIDS.PT_GADGETCONTENTSERVER_ID:
      case PT_CLASSIDS.PT_COMMUNITYTEMPLATE_ID:
      case PT_CLASSIDS.PT_GADGETBUNDLE_ID:
      case PT_CLASSIDS.PT_PAGETEMPLATE_ID:
      case PT_CLASSIDS.PT_WEBSERVICE_ID:
      case PT_CLASSIDS.PT_EXTOPERATION_ID:
      case PT_CLASSIDS.PT_GADGETTEMPLATE_ID:
      case PT_CLASSIDS.PT_JOB_ID:
      case PT_CLASSIDS.PT_COMMUNITY_ID:
      return true;
      goto default;
      default:
      return false;
      break;
      }
    }
    /*
     * Removes non-searchable classids from idArray.
     */
     
    protected virtual int[] FilterToSearchableIDs(int[] idArray)
    {
      int searchableCount = 0;
      int[] flagArray = new int[idArray.Length];
      for (int i = 0;i < idArray.Length;i++)
      {
        if (ClassIDSearchable(idArray[i]))
        {
          flagArray[i] = 1;
          searchableCount++;
        }
      }
      int[] retval = new int[searchableCount];
      int j = 0;
      for (int i = 0;i < idArray.Length;i++)
      {
        if (flagArray[i] == 1)
        {
          retval[j++] = idArray[i];
        }
      }
      return retval;
    }
    protected virtual int[] RemoveListIds(int[] idArray, XPArrayList exceptTheseList)
    {
      if (exceptTheseList == null)
      {
        return idArray;
      }
      int useCount = 0;
      int[] flagArray = new int[idArray.Length];
      for (int i = 0;i < idArray.Length;i++)
      {
        if (!ListContainsClassId(idArray[i], exceptTheseList))
        {
          flagArray[i] = 1;
          useCount++;
        }
      }
      int[] retval = new int[useCount];
      int j = 0;
      for (int i = 0;i < idArray.Length;i++)
      {
        if (flagArray[i] == 1)
        {
          retval[j++] = idArray[i];
        }
      }
      return retval;
    }
    protected virtual int[] GetSavedIds()
    {
      try
      {
        Object[][] idArray = ((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo().LookupPreference(INDEXPREF_SAVEDIDS,
         0);
        if (idArray.Length > 0 && idArray[1] != null && idArray[1].Length == 1 && idArray[1][0] != null)
        {
          String ids = XPConvert.ToString(idArray[1][0]);
          //ids.split(",", -1);
          int[] retval = SplitIntoInts(ids, ",");
          log.Error("In GetSavedIds idArray is null");
          return retval;
        } else
        {
          int[] def = {17, 18, 33, 43};
          return def;
        }
      }
      catch (Exception e)
      {
        throw XPException.GetInstance(e);
      }
    }
    /*
     UIError err = new UIError(UIErrorLevel.SERIOUS,
     UIErrorScope.CURRENT_USER, e,
     
     m_asOwner.GetString(1036, "ptmsgs_portalbrowsingmsgs"),
     m_asOwner.GetString(1036, "ptmsgs_portalbrowsingmsgs"));
     UIErrorHandler.RaiseError(err, m_asOwner.GetSubSession(),
     m_asOwner.GetApplication());
     */
     
    protected virtual void SaveClassIds(XPArrayList classIdList)
    {
      if (classIdList == null)
      {
        return;
      }
      try
      {
        IXPEnumerator xpEnum = classIdList.GetEnumerator();
        String propVal = "";
        while (xpEnum.MoveNext())
        {
          IndexingCount ic = (IndexingCount) xpEnum.GetCurrent();
          if (ic.GetClassId() != IndexingCount.ALL_OTHERS_ID)
          {
            propVal+=ic.GetClassId() + ",";
          }
        }
        IPTSessionInfo sessionInfo = ((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo();
        sessionInfo.RemovePreference(INDEXPREF_SAVEDIDS, 0);
        sessionInfo.AddPreference(INDEXPREF_SAVEDIDS, propVal, 0);
      }
      catch (Exception e)
      {
        throw new XPException(e);
      }
    }
    /*
     UIError err = new UIError(UIErrorLevel.SERIOUS,
     UIErrorScope.CURRENT_USER, e,
     
     m_asOwner.GetString(1036, "ptmsgs_portalbrowsingmsgs"),
     m_asOwner.GetString(1036, "ptmsgs_portalbrowsingmsgs"));
     UIErrorHandler.RaiseError(err, m_asOwner.GetSubSession(),
     m_asOwner.GetApplication());
     */
     
    protected virtual int[] SplitIntoInts(String value, String delimiter)
    {
      log.Error("Trying to split: " + value);
      int i = 0;
      int[] vals = new int[100];
      int start = XPStringUtility.IndexOf(value, delimiter);
      String tempVal = value;
      while (start != -1)
      {
        String intstr = XPStringUtility.Substring(tempVal, 0, start);
        log.Error("Trying to convert: " + intstr);
        if (intstr != null && !intstr.Equals(""))
        {
          int candidate = XPConvert.ToInteger(intstr);
          if (ClassIDSearchable(candidate))
          {
            vals[i++] = candidate;
          }
          tempVal = XPStringUtility.Substring(tempVal, start + 1);
          start = XPStringUtility.IndexOf(tempVal, delimiter);
        } else
        {
          start = -1;
        }
      }
      int[] retval = new int[i];
      for (int k = 0;k < i;k++)
      {
        retval[k] = vals[k];
      }
      return retval;
    }
  }
}
//i is count of delimiter occurences
