using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.directory.abodelete;
using com.plumtree.portalpages.browsing.directory.browse;
using com.plumtree.portalpages.browsing.directory.browse_D;
using com.plumtree.portalpages.browsing.directory.createfolder;
using com.plumtree.portalpages.browsing.directory.documentsubmitsimple;
using com.plumtree.portalpages.browsing.directory.edit;
using com.plumtree.portalpages.browsing.directory.properties;
using com.plumtree.portalpages.browsing.directory.relatedresources;
using com.plumtree.portalpages.browsing.directory.rest;
using com.plumtree.portalpages.browsing.directory.senddoc;
using com.plumtree.portalpages.browsing.editors.document;
using com.plumtree.portalpages.common.createfolder;
using com.plumtree.portalpages.common.folder;
using com.plumtree.portalpages.common.objects;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.portaluiinfrastructure.editor.commonpages.keywords;
using com.plumtree.portaluiinfrastructure.opener;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.expandablelist;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.directory
{
  /// <author>StephenC</author>
  public class DirAS : PlumtreeAS
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.directory.DirAS");
    public const String DIRFOLDERVIEW = "DirFolderView";
    public const String DIRBANNERVIEW = "DirBannerView";
    public const String DIROBJECTVIEW = "DirObjectView";
    public const String DIRBREADCRUMBVIEW = "DirBreadcrumbView";
    public const String STR_MVC_CLASS_NAME = "Dir";
    public static readonly String JS_PARENT_SIMPLE_REPOST = "window.opener." + AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" + RepostControl.POSTTOSELF_ACTION_SIMPLEREPOST + ");\n";
    
    /// <summary>
    /// storage name for ClosePopUpDP when Editing Cards 
    /// </summary>
    public const String EDITCARDPROPERTIES_POPUP = "CardPropertiesPopUp";
    
    /// <summary>
    /// storage name for ClosePopUpDP when Clicking Through More Related Resources
    /// </summary>
    public const String MORE_RELATEDRESOURCES_POPUP = "MoreRelatedResourcesPopUp";
    
    /// <summary>
    /// storage name for ClosePopUpDP when Clicking Through More Docs Like This In Card Properties
    /// </summary>
    public const String CARDPROPERTIES_MOREDOCS_POPUP = "CardPropertiesMoreDocsPopUp";
    
    /// <summary>
    /// storage name for ClosePopUpDP after Advanced Card Submit
    /// </summary>
    public const String CARDSUBMIT_ADVANCED_CONFIM_POPUP = "CardSubmitAdvancedConfirmPopUp";
    
    /// <summary>
    /// Display Page name to handle the reloading of the directory after exiting from an editor *
    /// </summary>
    public const String DIR_EDITOR_FINISH = "DirEditorRedirect";
    
    /// <summary>
    /// MJD- Storage name for Keywords classes
    /// </summary>
    public const String STR_MVC_STORAGE_NAME_KEYWORDS = "DirKeywordsObjEditor";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new DirAS();
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.AActivitySpace#GetCurrentPage()"></seealso>
    public override IDisplayPage GetCurrentPage()
    {
      DirModel dirModel = (DirModel) GetModel(DirModel.STR_MVC_CLASS_NAME);
      int nDirMode = dirModel.GetDirectoryMode();
      switch (nDirMode){
      case DirConstants.BROWSE_MODE:
      {
        this.SetHelpTopic("PT_BrowseDirectory");
        return GetPage(DirBrowseDP.STR_MVC_CLASS_NAME);
        goto case DirConstants.EDIT_MODE;
      }
      case DirConstants.EDIT_MODE:
      {
        this.SetHelpTopic("PT_EditDirectory");
        return GetPage(DirEditDP.STR_MVC_CLASS_NAME);
        goto case DirConstants.DISABILITY_MODE;
      }
      case DirConstants.DISABILITY_MODE:
      {
        this.SetHelpTopic("PT_BrowseDirectory");
        return GetPage(DirBrowseDP_D.STR_MVC_CLASS_NAME);
        goto default;
      }
      default:
      {
        this.SetHelpTopic("PT_BrowseDirectory");
        return GetPage(DirBrowseDP.STR_MVC_CLASS_NAME);
        break;
      }
      }
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.AActivitySpace#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// This method handles the ClosePopUpDP and passes everything else to the base
    /// implementation.  It adds javascript to the ClosePopUpDP that sets the selected
    /// items on the opener window (if it's a remote tree) and calls a javascript
    /// function on the opener window.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AActivitySpace#GetPage(String)"></seealso>
    public override IDisplayPage GetPage(String pageName)
    {
      if (ClosePopUpDP.STR_MVC_CLASS_NAME.Equals(pageName))
      {
        // This is called when SimpleCardSubmitDP or CreateFolderDP is canceled.
        ClosePopUpDP result = (ClosePopUpDP) base.GetPage(pageName);
        try
        {
          HTMLScript myScript = new HTMLScript("text/javaScript");
          myScript.AddInnerHTMLString(JS_PARENT_SIMPLE_REPOST);
          result.SetScript(myScript);
        }
        catch (HTMLException he)
        {
          log.Error(he, "Error generating JS for ClosePopUpDP.");
        }
        return result;
      } else
      if (CARDSUBMIT_ADVANCED_CONFIM_POPUP.Equals(pageName))
      {
        DirModel dirModel = (DirModel) GetModel(DirModel.STR_MVC_CLASS_NAME);
        dirModel.LoadFoldersAndDocuments();
        DirEditDP result = (DirEditDP) base.GetPage(DirEditDP.STR_MVC_CLASS_NAME);
        int nCardID = dirModel.GetCardID();
        int nLoadedCardID = dirModel.GetLoadedCardID();
        if (nLoadedCardID != nCardID)
        {
          ASURL myURL = new ASURL(this);
          myURL.SetLinkGetCachedSpace(this);
          myURL.SetDisplayPage(DirCardSubmitDependenciesDP.STR_MVC_CLASS_NAME);
          // popup a new window
          XPStringBuilder sb = new XPStringBuilder(200);
          sb.Append("window.open(\'").Append(myURL.GetURLAsString()).Append("\', \'ConfirmDir\', \'location=no,menubar=no,height=575,width=600\');\n");
          result.SetScript(sb.ToString());
          dirModel.SetLoadedCardID(nCardID);
        }
        return result;
      } else
      if (DIR_EDITOR_FINISH.Equals(pageName))
      {
        DirModel dirModel = (DirModel) GetModel(DirModel.STR_MVC_CLASS_NAME);
        dirModel.LoadFoldersAndDocuments();
        switch (dirModel.GetDirectoryMode()){
        case DirConstants.BROWSE_MODE:
        {
          return base.GetPage(DirBrowseDP.STR_MVC_CLASS_NAME);
          goto case DirConstants.EDIT_MODE;
        }
        case DirConstants.EDIT_MODE:
        {
          this.SetHelpTopic("PT_EditDirectory");
          return base.GetPage(DirEditDP.STR_MVC_CLASS_NAME);
          goto case DirConstants.DISABILITY_MODE;
        }
        case DirConstants.DISABILITY_MODE:
        {
          this.SetHelpTopic("PT_BrowseDirectory");
          return base.GetPage(DirBrowseDP_D.STR_MVC_CLASS_NAME);
          goto default;
        }
        default:
        {
          return GetPage(DirBrowseDP.STR_MVC_CLASS_NAME);
          break;
        }
        }
      } else
      if (MORE_RELATEDRESOURCES_POPUP.Equals(pageName))
      {
        // This called when a user clicks through to a Related Resources posted in the More RR popup
        ClosePopUpDP result = (ClosePopUpDP) base.GetPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
        IDirModelRO model = (IDirModelRO) GetModel(DirModel.STR_MVC_CLASS_NAME);
        int nRRCategoryID = model.GetRRCategory();
        int nRRObjectID = model.GetRRObjectID();
        int nClassID;
        switch (nRRCategoryID){
        case PT_INTRINSICS.PT_RELATED_RESOURCE_COMMUNITIES:
        nClassID = PT_CLASSIDS.PT_COMMUNITY_ID;
        break;
        case PT_INTRINSICS.PT_RELATED_RESOURCE_FOLDERS:
        nClassID = PT_CLASSIDS.PT_CATALOGFOLDER_ID;
        break;
        case PT_INTRINSICS.PT_RELATED_RESOURCE_GADGETS:
        nClassID = PT_CLASSIDS.PT_GADGET_ID;
        break;
        case PT_INTRINSICS.PT_RELATED_RESOURCE_EXPERTS:
        case PT_INTRINSICS.PT_RELATED_RESOURCE_CONTENT_MANAGERS:
        nClassID = PT_CLASSIDS.PT_USER_ID;
        break;
        default:
        return base.GetPage(pageName);
        break;
        }
        String strURL = PTOpenerLinks.GetOpenerURLOpenObjID(nClassID, nRRObjectID,
         IPTOpenerConstants.OPENER_MODE_VIEW, this);
        try
        {
          String strAppBaseURL = ConfigHelper.GetApplicationBaseURL(this);
          HTMLScript myScript = new HTMLScript("text/javaScript");
          myScript.AddInnerHTMLString("window.opener.location=\'" + strAppBaseURL + strURL + "\';\n");
          result.SetScript(myScript);
        }
        catch (HTMLException he)
        {
          log.Error(he, "Error generating JS for ClosePopUpDP.");
        }
        return result;
      } else
      if (CARDPROPERTIES_MOREDOCS_POPUP.Equals(pageName))
      {
        // This is called when a user clicks on "More Docs Like This" property value
        ClosePopUpDP result = (ClosePopUpDP) base.GetPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
        IDirModelRO model = (IDirModelRO) GetModel(DirModel.STR_MVC_CLASS_NAME);
        int nMoreFolderID = model.GetMoreFolderID();
        int nDirMode = model.GetDirectoryMode();
        ASURL myURL = new ASURL(this);
        myURL.SetLinkGetCachedSpace(this);
        myURL.SetControl(OpenSubFolderControl.STR_MVC_CLASS_NAME);
        myURL.AddControlArgument(OpenDirFolderControl.m_SubFolderID, XPConvert.ToString(nMoreFolderID));
        myURL.AddControlArgument(DirConstants.QS_DIRECTORY_MODE, XPConvert.ToString(nDirMode));
        try
        {
          HTMLScript myScript = new HTMLScript("text/javaScript");
          myScript.AddInnerHTMLString("window.opener.location=\'" + myURL.GetURLAsString() + "\';\n");
          result.SetScript(myScript);
        }
        catch (HTMLException he)
        {
          log.Error(he, "Error generating JS for ClosePopUpDP.");
        }
        return result;
      } else
      if (EDITCARDPROPERTIES_POPUP.Equals(pageName))
      {
        // This is called when Edit button on Section Card Properties is clicked
        ClosePopUpDP result = (ClosePopUpDP) base.GetPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
        IDirModelRO model = (IDirModelRO) GetModel(DirModel.STR_MVC_CLASS_NAME);
        int nCardID = model.GetCardID();
        ASURL url = new ASURL(this);
        url.SetLinkCreateNewSpace(CardEditorAS.STR_MVC_CLASS_NAME, this);
        url.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
        url.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_EDIT));
        url.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_QS_OBJECT_ID,
         XPConvert.ToString(nCardID));
        url.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID, XPConvert.ToString(PT_CLASSIDS.PT_CATALOGCARD_ID));
        // We are in a frame, so we need to call parent, not window
        try
        {
          HTMLScript myScript = new HTMLScript("text/javaScript");
          /* bug 24681 JHQ: in IE5.5 window.opener.location is bugged!  Instead,
           * doing oWin = window.opener; oWin.location = url;
           */
           
          myScript.AddInnerHTMLString("var oWin = window.opener;" + "oWin.location=\'" + url.GetURLAsString() + "\';\n");
          result.SetScript(myScript);
        }
        catch (HTMLException he)
        {
          log.Error(he, "Error generating JS for ClosePopUpDP.");
        }
        return result;
      }
      return base.GetPage(pageName);
    }
    
    /// <summary>
    /// Returns the name of the repost control class.
    /// </summary>
    /// <seealso cref="com.plumtree.xpshared.form.IRepostAS#GetRepostControlName()"></seealso>
    /// <returns>String</returns>
    public override String GetRepostControlName()
    {
      return DirRepostControl.STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.AActivitySpace#Init()"></seealso>
    public override void Init()
    {
      base.Init();
      // Directory DisplayPages
      RegisterPage(new DirBrowseDP());
      RegisterPage(new DirEditDP());
      RegisterPage(new CreateFolderDP());
      AFormDP page = new DirCardSubmitSimpleDP();
      RegisterPage(page);
      RegisterPage(new DirCardSubmitDependenciesDP());
      RegisterPage(new DirCardPropertiesDP());
      RegisterPage(new RelatedResourcesMorePopupDP());
      RegisterPage(new SendDocPopUpDP());
      // Obj Mgr popup Pages -- delete warnings
      RegisterPage(new DirDeleteWarningsDP());
      RegisterPage(new DirABOLinksDP());
      RegisterPage(new ClosePopUpDP());
      // Directory Model
      RegisterModel(DirModel.STR_MVC_CLASS_NAME);
      IModel myModel = GetModel(DirModel.STR_MVC_CLASS_NAME);
      // This page uses the Dir Model (since it handles the View code as well),
      // so it needs to be registered after the model.
      RegisterPage(new DirRESTDP());
      // Directory Views
      RegisterView(DirBreadcrumbView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(FolderBannerView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(FolderView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(ObjectView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DocumentBannerView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(RelatedResourcesView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(RelatedResourcesMorePopupView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DirCreateFolderView.STR_MVC_CLASS_NAME, CreateFolderView.STR_VIEW_STORAGE_NAME,
       myModel);
      RegisterView(DirCardSubmitSimpleView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DirCardSubmitUploadingView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DirCardSubmitDoneView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DirCardPropertiesView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DirDeleteWarningsView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DirABOLinksView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DirCardSubmitDependenciesView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(SendDocPopUpView.STR_MVC_CLASS_NAME, myModel);
      // Directory Controls
      RegisterControl(SetStartItemControl.STR_MVC_CLASS_NAME, myModel);
      RegisterControl(SetLastItemControl.STR_MVC_CLASS_NAME, myModel);
      RegisterControl(DirRepostControl.STR_MVC_CLASS_NAME, myModel);
      RegisterControl(DirCheckDeleteControl.STR_MVC_CLASS_NAME, myModel);
      RegisterControl(DirRESTControl.STR_MVC_CLASS_NAME, myModel);
      // Register the opendirfolder control as opensubfolder control
      RegisterControl(OpenDirFolderControl.STR_MVC_CLASS_NAME, OpenSubFolderControl.STR_MVC_CLASS_NAME,
       myModel);
      // 508 pages and views
      RegisterPage(new DirBrowseDP_D());
      RegisterPage(new DirCardPropertiesDP_D());
      RegisterPage(new DirRelatedResourcesMoreDP_D());
      RegisterView(DirBrowseFooterView_D.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DirBrowseHeaderView_D.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DirCardPropertiesView_D.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DirRelatedResourcesView_D.STR_MVC_CLASS_NAME, myModel);
      RegisterView(DirRelatedResourcesMoreView_D.STR_MVC_CLASS_NAME, myModel);
      // Register Plug-In: Keywords
      RegisterModel(KeywordsModel.STR_MVC_CLASS_NAME, STR_MVC_STORAGE_NAME_KEYWORDS);
      RegisterView(DirKeywordsView.STR_MVC_CLASS_NAME, STR_MVC_STORAGE_NAME_KEYWORDS,
       GetModel(STR_MVC_STORAGE_NAME_KEYWORDS));
      RegisterControl(ExpListControl.STR_MVC_CLASS_NAME, STR_MVC_STORAGE_NAME_KEYWORDS,
       GetModel(STR_MVC_STORAGE_NAME_KEYWORDS));
      this.SetOverWriteSubSpaces(true);
    }
  }
}