using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.directory;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.directory.abodelete
{
  /// <author>MikeSample</author>
  public class DirDeleteWarningsView : IView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.directory.abodelete.DirDeleteWarningsView");
    
    /// <summary>
    /// object name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "DirDeleteWarnings";
    
    /// <summary>
    /// HTML Object names 
    /// </summary>
    public const String HTMLBUTTON_APPLY_NOW = "in_bu_ApplyNow";
    public const String HTMLBUTTON_APPLY_LATER = "in_bu_ApplyLater";
    public const String HTMLBUTTON_CANCEL = "in_bu_Cancel";
    
    /// <summary>
    /// javascript constants 
    /// </summary>
    public const String JAVASCRIPT_DELETE_NOW_ON_OPENER = "deleteNowOnOpener";
    public const String JAVASCRIPT_DELETE_LATER_ON_OPENER = "deleteLaterOnOpener";
    
    /// <summary>
    /// mvc objects 
    /// </summary>
    internal AActivitySpace m_asOwner;
    internal IDirModelRO m_asModel;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new DirDeleteWarningsView();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Display()"></seealso>
    public virtual HTMLElement Display()
    {
      HTMLElement result = new HTMLElementCollection();
      HTMLSpan mySpan;
      HTMLTable myTable;
      HTMLInput myInput;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      bool bFolders = m_asModel.CheckFoldersForDelete();
      try
      {
        myTable = new HTMLTable();
        result.AddInnerHTMLElement(myTable);
        myTable.SetBorder("0");
        myTable.SetWidth("100%");
        myTable.SetCellPadding("0");
        myTable.SetCellSpacing("10");
        myTable.SetStyleClass(PTStyleClass.DIALOGUE_BG);
        if (bFolders)
        {
          myRow = new HTMLTableRow();
          myTable.AddInnerHTMLElement(myRow);
          myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          mySpan = new HTMLSpan();
          myCell.AddInnerHTMLElement(mySpan);
          mySpan.SetStyleClass(PTStyleClass.ALERT_DIALOG_TEXT);
          mySpan.AddInnerHTMLString(m_asOwner.GetString(23, "ptmsgs_portalbrowsingmsgs"));
          }
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        mySpan = new HTMLSpan();
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.SetStyleClass(PTStyleClass.ALERT_DIALOG_TEXT);
        mySpan.AddInnerHTMLString(m_asOwner.GetString(22, "ptmsgs_portalbrowsingmsgs"));
        if (bFolders)
        {
          myRow = new HTMLTableRow();
          myTable.AddInnerHTMLElement(myRow);
          myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          mySpan = new HTMLSpan();
          myCell.AddInnerHTMLElement(mySpan);
          mySpan.SetStyleClass(PTStyleClass.ALERT_DIALOG_TEXT);
          mySpan.AddInnerHTMLString(m_asOwner.GetString(24, "ptmsgs_portalbrowsingmsgs"));
          }
        // Add buttons
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myCell.SetAlign(CommonHTMLStrings.MIDDLE);
        myRow.AddInnerHTMLElement(myCell);
        // OK button (Apply Now)
        myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_APPLY_NOW,
         "");
        myCell.AddInnerHTMLElement(myInput);
        myInput.SetValue(" " + m_asOwner.GetString(19, "ptmsgs_portalbrowsingmsgs") + " ");
        myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
        myInput.SetOnClick(JAVASCRIPT_DELETE_NOW_ON_OPENER + "(); return false;");
        if (bFolders)
        {
          // Apply Later button
          myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_APPLY_LATER,
           "");
          myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
          myCell.AddInnerHTMLElement(myInput);
          myInput.SetValue(m_asOwner.GetString(20, "ptmsgs_portalbrowsingmsgs"));
          myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
          myInput.SetOnClick(JAVASCRIPT_DELETE_LATER_ON_OPENER + "(); return false;");
          }
        // Cancel button
        myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_CANCEL,
         "");
        myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        myCell.AddInnerHTMLElement(myInput);
        myInput.SetValue(m_asOwner.GetString(21, "ptmsgs_portalbrowsingmsgs"));
        myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
        myInput.SetOnClick("window.close(); return false;");
        myTable.AddInnerHTMLElement(DirConstants.CreateSpacerRow(1, 175));
        result.AddInnerHTMLElement(new HTMLComment("End: Footer buttons table"));
        }
      catch (HTMLException he)
      {
        log.Error(he, "Unable to create warnings: " + he.Message);
        result = new HTMLGenericElement(m_asOwner.GetString(22, "ptmsgs_portalbrowsingmsgs"));
        }
      return result;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    public virtual HTMLScript DisplayJavascript()
    {
      // JavaScript to open the create folder popup
      HTMLScript myScript = new HTMLScript("text/javaScript");
      XPStringBuilder sb = new XPStringBuilder(50);
      // javascript functions
      sb = new XPStringBuilder(300);
      sb.Append("\n");
      sb.Append("function ").Append(JAVASCRIPT_DELETE_NOW_ON_OPENER).Append("()\n");
      sb.Append("{\n");
      sb.Append("  opener.").Append(FolderBannerView.JAVASCRIPT_SUBMIT_NO_REFRESH).Append("(").Append(DirRepostControl.POSTTOSELF_ACTION_DELETE).Append(");\n");
      sb.Append("  window.close();\n");
      sb.Append("}\n");
      sb.Append("\n");
      sb.Append("function ").Append(JAVASCRIPT_DELETE_LATER_ON_OPENER).Append("()\n");
      sb.Append("{\n");
      sb.Append("  opener.").Append(FolderBannerView.JAVASCRIPT_SUBMIT_NO_REFRESH).Append("(").Append(DirRepostControl.POSTTOSELF_ACTION_DELETE_LATER).Append(");\n");
      sb.Append("  window.close();\n");
      sb.Append("}\n");
      sb.Append("\n");
      try
      {
        // Add script
        myScript.AddInnerHTMLString(sb.ToString());
      }
      catch (HTMLException he)
      {
        log.Error(he, "Unable to create javascript: " + he.Message);
        return null;
      }
      return myScript;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO,AActivitySpace)"></seealso>
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      m_asOwner = parent;
      m_asModel = (IDirModelRO) model;
    }
  }
}