using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.directory;
using com.plumtree.portalpages.browsing.directory.properties;
using com.plumtree.portalpages.browsing.directory.relatedresources;
using com.plumtree.portalpages.browsing.directory.senddoc;
using com.plumtree.portalpages.common.folder;
using com.plumtree.portalpages.common.objects;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.portaluiinfrastructure.layout;
using com.plumtree.portaluiinfrastructure.search;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server.portalpagesperf;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.directory.browse
{
  /// <summary>
  /// Main display page for directory.
  /// </summary>
  /// <author>StephenC</author>
  public class DirBrowseDP : PlumtreeDP
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.directory.browse.DirBrowseDP");
    public const String STR_MVC_CLASS_NAME = "DirBrowse";
    private DirModel m_asModel;
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IDisplayPage#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.uiparts.PlumtreeDP#GetSubtitleForBanner()"></seealso>
    public override String GetSubtitleForBanner()
    {
      return "";
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.uiparts.PlumtreeDP#GetTitleForBanner()"></seealso>
    public override String GetTitleForBanner()
    {
      return m_asOwner.GetString(27, "ptmsgs_portalbrowsingmsgs");
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IDisplayPage#Init(AActivitySpace)"></seealso>
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
      SetAddMainForm(true);
    }
    // SCC: overriding AFormDP's implementation
    protected override bool GetRepostTypeIsPost()
    {
      return false;
    }
    
    /// <summary>
    /// helper display method for PlumtreeDP
    /// Fill in the body argument with HTML to display in the page.
    /// </summary>
    public override HTMLElement PageDisplay()
    {
      // Increment the performance counters only if info is on for commonlib	
      PerfCounters.INSTANCE.IncrementCounter(PORTALPAGESPERF_COUNTERS.PORTALPAGESPERF_DIRECTORY_PAGES);
      HTMLElement element = DisplayLayout(Layouts.KNOWLEDGE_DIRECTORY);
      if (null != element)
      {
        return element;
      }
      m_asModel = (DirModel) m_asOwner.GetModel(DirModel.STR_MVC_CLASS_NAME);
      ((ObjectView) m_asOwner.GetView(ObjectView.STR_MVC_CLASS_NAME)).SetObjectViewHelper(new BrowseDocViewHelper());
      HTMLTable table;
      HTMLTableRow row;
      HTMLTableCell cell;
      table = new HTMLTable();
      table.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      table.SetBorder(CommonHTMLStrings.ZERO);
      table.SetCellSpacing(CommonHTMLStrings.ZERO);
      table.SetCellPadding(CommonHTMLStrings.ZERO);
      row = new HTMLTableRow();
      table.AddInnerHTMLElement(row);
      cell = new HTMLTableCell();
      row.AddInnerHTMLElement(cell);
      //Breadcrumbs
      HTMLTable breadcrumbsTable = DirConstants.DisplayBreadcrumbTable(m_asOwner);
      if (null != breadcrumbsTable)
      {
        cell.AddInnerHTMLElement(breadcrumbsTable);
      }
      // Group Folder and Document stuff together to make related resources placement easier
      HTMLElementCollection foldersAndDocumentsGroup = new HTMLElementCollection();
      try
      {
        //Folder Actions Banner
        foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(FolderBannerView.STR_MVC_CLASS_NAME).Display());
        // Warnings about the search results(timeout, partial result, etc..)
        SearchExecutionWarning warning = m_asModel.GetWarning();
        if (null != warning)
        {
          //                cell.AddInnerHTMLElement(new HTMLBr());
          //                cell.AddInnerHTMLString(warning.getMessage());
          //                cell.AddInnerHTMLElement(new HTMLBr());
          HTMLSpan warn = new HTMLSpan();
          warn.AddInnerHTMLElement(new HTMLBr());
          warn.AddInnerHTMLEncodedString(warning.GetMessage());
          warn.AddInnerHTMLElement(new HTMLBr());
          warn.SetStyleClass(PTStyleClass.ALERT_BG);
          foldersAndDocumentsGroup.AddInnerHTMLElement(warn);
        }
        //Folders
        foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(FolderView.STR_MVC_CLASS_NAME).Display());
        // ADD extra spacer table
        foldersAndDocumentsGroup.AddInnerHTMLElement(DirConstants.CreateSpacerTable(15,
         1));
        //Documents
        if (!m_asModel.InRootFolder())
        {
          foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(DocumentBannerView.STR_MVC_CLASS_NAME).Display());
          foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(ObjectView.STR_MVC_CLASS_NAME).Display());
          HTMLTable tableDocLinks = DirConstants.DisplayDocumentLinksTable(m_asOwner,
           m_asOwner.GetString(1662, "ptmsgs_portalbrowsingmsgs"), m_asOwner.GetString(1663,
           "ptmsgs_portalbrowsingmsgs"), m_asOwner.GetString(1165, "ptmsgs_portalbrowsingmsgs"));
          if (null != tableDocLinks)
          {
            foldersAndDocumentsGroup.AddInnerHTMLElement(tableDocLinks);
          }
        }
        if (m_asModel.ShowRelatedResources())
        {
          int nRelatedResourceLocation = m_asModel.GetRelatedResourcesLocation();
          if (DirConstants.RELATED_RESOURCE_LOCATION_LEFT == nRelatedResourceLocation || DirConstants.RELATED_RESOURCE_LOCATION_RIGHT == nRelatedResourceLocation)
          {
            cell.SetColSpan(CommonHTMLStrings.THREE);
            row = new HTMLTableRow();
            table.AddInnerHTMLElement(row);
            if (nRelatedResourceLocation == DirConstants.RELATED_RESOURCE_LOCATION_LEFT)
            {
              //Related Resources on left
              cell = new HTMLTableCell();
              row.AddInnerHTMLElement(cell);
              cell.SetWidth("225");
              cell.SetVAlign(CommonHTMLStrings.TOP);
              cell.AddInnerHTMLElement(m_asOwner.GetView(RelatedResourcesView.STR_MVC_CLASS_NAME).Display());
              // add spacer
              cell = new HTMLTableCell();
              row.AddInnerHTMLElement(cell);
              cell.AddInnerHTMLElement(DirConstants.CreateSpacerImage(1, 10));
              //Directory on right
              cell = new HTMLTableCell();
              cell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
              cell.SetVAlign(CommonHTMLStrings.TOP);
              cell.AddInnerHTMLElement(foldersAndDocumentsGroup);
              row.AddInnerHTMLElement(cell);
            } else
            {
              //Directory on left
              cell = new HTMLTableCell();
              cell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
              cell.SetVAlign(CommonHTMLStrings.TOP);
              cell.AddInnerHTMLElement(foldersAndDocumentsGroup);
              row.AddInnerHTMLElement(cell);
              // add spacer
              cell = new HTMLTableCell();
              row.AddInnerHTMLElement(cell);
              cell.AddInnerHTMLElement(DirConstants.CreateSpacerImage(1, 10));
              //Related Resources on right
              cell = new HTMLTableCell();
              cell.SetWidth("225");
              cell.SetVAlign(CommonHTMLStrings.TOP);
              cell.AddInnerHTMLElement(m_asOwner.GetView(RelatedResourcesView.STR_MVC_CLASS_NAME).Display());
              row.AddInnerHTMLElement(cell);
            }
          } else
          if (DirConstants.RELATED_RESOURCE_LOCATION_TOP == nRelatedResourceLocation || DirConstants.RELATED_RESOURCE_LOCATION_BOTTOM == nRelatedResourceLocation)
          {
            row = new HTMLTableRow();
            table.AddInnerHTMLElement(row);
            // Add to the existing cell				
            if (nRelatedResourceLocation == DirConstants.RELATED_RESOURCE_LOCATION_TOP)
            {
              //Related Resources on top
              cell = new HTMLTableCell();
              row.AddInnerHTMLElement(cell);
              cell.AddInnerHTMLElement(m_asOwner.GetView(RelatedResourcesView.STR_MVC_CLASS_NAME).Display());
              //Directory on bottom
              row = new HTMLTableRow();
              table.AddInnerHTMLElement(row);
              cell = new HTMLTableCell();
              cell.AddInnerHTMLElement(foldersAndDocumentsGroup);
              row.AddInnerHTMLElement(cell);
            } else
            {
              //Directory on top
              cell = new HTMLTableCell();
              cell.AddInnerHTMLElement(foldersAndDocumentsGroup);
              row.AddInnerHTMLElement(cell);
              //Related Resources on bottom
              row = new HTMLTableRow();
              table.AddInnerHTMLElement(row);
              cell = new HTMLTableCell();
              row.AddInnerHTMLElement(cell);
              cell.AddInnerHTMLElement(m_asOwner.GetView(RelatedResourcesView.STR_MVC_CLASS_NAME).Display());
              }
          }
        } else
        {
          row = new HTMLTableRow();
          table.AddInnerHTMLElement(row);
          cell = new HTMLTableCell();
          cell.AddInnerHTMLElement(foldersAndDocumentsGroup);
          row.AddInnerHTMLElement(cell);
        }
      }
      catch (HTMLException he)
      {
        log.Error(he, "Error in DirBrowseDP::Display");
      }
      return table;
    }
    
    /// <summary>
    /// This is a helper method that all classes that inherit from AFormDP can override
    /// if they like.  The method provides a hook into the head so that DP's can add javascript.
    /// Simply override and return and HTMLScript.  Also, it is best to return the results of the
    /// DisplayJavaScript functions from the appropriate views.
    /// </summary>
    protected override HTMLScriptCollection DisplayJavaScriptFromChild()
    {
      HTMLScriptCollection scripts = new HTMLScriptCollection();
      HTMLScript myScript;
      myScript = m_asOwner.GetView(FolderBannerView.STR_MVC_CLASS_NAME).DisplayJavascript();
      if (null != myScript)
      {
        scripts.AddInnerHTMLElement(myScript);
      }
      myScript = m_asOwner.GetView(ObjectView.STR_MVC_CLASS_NAME).DisplayJavascript();
      if (null != myScript)
      {
        scripts.AddInnerHTMLElement(myScript);
      }
      myScript = m_asOwner.GetView(RelatedResourcesView.STR_MVC_CLASS_NAME).DisplayJavascript();
      if (null != myScript)
      {
        scripts.AddInnerHTMLElement(myScript);
      }
      myScript = m_asOwner.GetView(DocumentBannerView.STR_MVC_CLASS_NAME).DisplayJavascript();
      if (null != myScript)
      {
        scripts.AddInnerHTMLElement(myScript);
      }
      myScript = new HTMLScript("text/javaScript");
      XPStringBuilder sb = new XPStringBuilder(100);
      // Card Properties url
      ASURL myCardPropertiesURL = new ASURL(m_asOwner);
      myCardPropertiesURL.SetLinkGetCachedSpace(m_asOwner);
      myCardPropertiesURL.SetDisplayPage(DirCardPropertiesDP.STR_MVC_CLASS_NAME);
      myCardPropertiesURL.SetControl(DirRepostControl.STR_MVC_CLASS_NAME);
      // Send Doc Link url
      ASURL mySendDocURL = new ASURL(m_asOwner);
      mySendDocURL.SetLinkGetCachedSpace(m_asOwner);
      mySendDocURL.SetDisplayPage(SendDocPopUpDP.STR_MVC_CLASS_NAME);
      mySendDocURL.SetControl(DirRepostControl.STR_MVC_CLASS_NAME);
      try
      {
        sb = new XPStringBuilder(400);
        // Card Properties popup function
        sb.Append("function ").Append(DirConstants.JAVASCRIPT_CARDPROPERTIES_POPUP).Append("(cardID)\n");
        sb.Append("{\n");
        sb.Append("  window.open(\'").Append(myCardPropertiesURL.GetURLAsString()).Append("&").Append(DirConstants.QS_CARDID).Append("=\'+").Append("cardID, \'_CardProp1\', \'location=no,menubar=no,scrollbars=yes,height=550,width=600\');\n");
        sb.Append("}\n");
        // Send Link popup function
        sb.Append("function ").Append(DirConstants.JAVASCRIPT_SENDDOCLINK_POPUP).Append("(cardID)\n");
        sb.Append("{\n");
        sb.Append("  window.open(\'").Append(mySendDocURL.GetURLAsString()).Append("&").Append(DirConstants.QS_CARDID).Append("=\'+").Append("cardID, \'_SendDocLink1\', \'location=no,menubar=no,scrollbars=no,height=175,width=450\');\n");
        sb.Append("}\n");
        myScript.AddInnerHTMLString(sb.ToString());
      }
      catch (HTMLException he)
      {
        log.Error(he, "DirEditDP::DisplayJavascript The following error occured when trying to display the JavaScript.");
        throw new XPException(m_asOwner.GetString(29, "ptmsgs_portalbrowsingmsgs"),
         he);
      }
      scripts.AddInnerHTMLElement(myScript);
      return scripts;
    }
  }
}