using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.directory;
using com.plumtree.portalpages.browsing.directory.properties;
using com.plumtree.portalpages.common.folder;
using com.plumtree.portalpages.common.objects;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.directory.edit
{
  /// <author>StephenC</author>
  public class DirEditDP : PlumtreeDP
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.directory.edit.DirEditDP");
    public const String STR_MVC_CLASS_NAME = "DirEdit";
    
    /// <summary>
    /// HTML Input checkbox: Check All 
    /// </summary>
    public const String HTMLINPUT_CHECK_ALL_FOLDERS = "in_cb_checkAllFolders";
    public const String JAVASCRIPT_CHECK_ALL_FOLDERS = "checkAllFolders";
    private DirModel m_asModel;
    
    /// <summary>
    /// Javascript to call in the header 
    /// </summary>
    private String m_strScript = null;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.uiparts.PlumtreeDP#GetSubtitleForBanner()"></seealso>
    public override String GetSubtitleForBanner()
    {
      return "";
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.uiparts.PlumtreeDP#GetTitleForBanner()"></seealso>
    public override String GetTitleForBanner()
    {
      return m_asOwner.GetString(109, "ptmsgs_portalbrowsingmsgs");
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IDisplayPage#Init(AActivitySpace)"></seealso>
    public override void Init(AActivitySpace parent)
    {
      m_strScript = null;
      base.Init(parent);
      SetAddMainForm(true);
    }
    // SCC: overriding AFormDP's implementation
    protected override bool GetRepostTypeIsPost()
    {
      return false;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.common.AFormDP#PageDisplay()"></seealso>
    public override HTMLElement PageDisplay()
    {
      m_asModel = (DirModel) m_asOwner.GetModel(DirModel.STR_MVC_CLASS_NAME);
      ((ObjectView) m_asOwner.GetView(ObjectView.STR_MVC_CLASS_NAME)).SetObjectViewHelper(new EditDocViewHelper());
      HTMLTable table;
      HTMLTableRow row;
      HTMLTableCell cell;
      table = new HTMLTable();
      table.SetBorder(CommonHTMLStrings.ZERO);
      table.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      table.SetCellSpacing(CommonHTMLStrings.ZERO);
      table.SetCellPadding(CommonHTMLStrings.ZERO);
      row = new HTMLTableRow();
      table.AddInnerHTMLElement(row);
      // Group Folder and Document stuff together to make related resources placement easier
      HTMLElementCollection foldersAndDocumentsGroup = new HTMLElementCollection();
      cell = new HTMLTableCell();
      cell.AddInnerHTMLElement(foldersAndDocumentsGroup);
      row.AddInnerHTMLElement(cell);
      try
      {
        //Breadcrumbs
        HTMLTable breadcrumbsTable = DirConstants.DisplayBreadcrumbTable(m_asOwner);
        if (null != breadcrumbsTable)
        {
          foldersAndDocumentsGroup.AddInnerHTMLElement(breadcrumbsTable);
          }
        //Folder Actions Banner
        foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(FolderBannerView.STR_MVC_CLASS_NAME).Display());
        // Folder Header
        int nSubFolderCount = m_asModel.GetSubFolderCount(m_asModel.GetCurrentFolderLevel() + 1);
        if (0 != nSubFolderCount)
        {
          HTMLTable myTable = new HTMLTable();
          myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
          myTable.SetBorder(CommonHTMLStrings.ZERO);
          myTable.SetCellPadding("2");
          myTable.SetCellSpacing("2");
          foldersAndDocumentsGroup.AddInnerHTMLElement(myTable);
          // spacer row
          myTable.AddInnerHTMLElement(DirConstants.CreateSpacerRow(2, 5));
          HTMLTableRow myRow = new HTMLTableRow();
          myRow.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);
          myTable.AddInnerHTMLElement(myRow);
          HTMLTableCell myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          myCell.SetWidth("25");
          myCell.SetHeight("20");
          myCell.SetAlign(CommonHTMLStrings.CENTER);
          myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
          myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
          HTMLInput myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLINPUT_CHECK_ALL_FOLDERS,
           m_asOwner.GetString(110, "ptmsgs_portalbrowsingmsgs"));
          myCell.AddInnerHTMLElement(myInput);
          // mj: removed 'Javascript:' from function name
          myInput.SetOnClick(JAVASCRIPT_CHECK_ALL_FOLDERS + "(this, \'" + DirFolderViewHelper.QS_CHECKBOX + "\');");
          myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
          myCell.SetAlign(CommonHTMLStrings.LEFT);
          myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
          myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
          myCell.AddInnerHTMLString(m_asOwner.GetString(108, "ptmsgs_portalbrowsingmsgs"));
          // blank cell over folder edit icons
          myCell = new HTMLTableCell();
          myCell.AddInnerHTMLElement(DirConstants.CreateSpacerTable(1, 20));
          myRow.AddInnerHTMLElement(myCell);
        }
        // Folders
        foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(FolderView.STR_MVC_CLASS_NAME).Display());
        // ADD extra spacer table
        foldersAndDocumentsGroup.AddInnerHTMLElement(DirConstants.CreateSpacerTable(15,
         1));
        //Documents
        if (!m_asModel.InRootFolder())
        {
          foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(DocumentBannerView.STR_MVC_CLASS_NAME).Display());
          foldersAndDocumentsGroup.AddInnerHTMLElement(DirConstants.CreateSpacerTable(5,
           1));
          foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(ObjectView.STR_MVC_CLASS_NAME).Display());
          HTMLTable tableDocLinks = DirConstants.DisplayDocumentLinksTable(m_asOwner,
           m_asOwner.GetString(1662, "ptmsgs_portalbrowsingmsgs"), m_asOwner.GetString(1663,
           "ptmsgs_portalbrowsingmsgs"), m_asOwner.GetString(1213, "ptmsgs_portalbrowsingmsgs"));
          if (null != tableDocLinks)
          {
            foldersAndDocumentsGroup.AddInnerHTMLElement(tableDocLinks);
          }
        }
      }
      catch (HTMLException he)
      {
        log.Error(he, "Error in DirEditDP: " + he.Message);
      }
      return table;
    }
    
    /// <summary>
    /// This is a helper method that all classes that inherit from AFormDP can override
    /// if they like.  The method provides a hook into the head so that DP's can add javascript.
    /// Simply override and return and HTMLScript.  Also, it is best to return the results of the
    /// DisplayJavaScript functions from the appropriate views.
    /// </summary>
    protected override HTMLScriptCollection DisplayJavaScriptFromChild()
    {
      HTMLScriptCollection scripts = new HTMLScriptCollection();
      HTMLScript myScript;
      myScript = m_asOwner.GetView(FolderBannerView.STR_MVC_CLASS_NAME).DisplayJavascript();
      if (null != myScript)
      {
        scripts.AddInnerHTMLElement(myScript);
      }
      myScript = m_asOwner.GetView(ObjectView.STR_MVC_CLASS_NAME).DisplayJavascript();
      if (null != myScript)
      {
        scripts.AddInnerHTMLElement(myScript);
      }
      myScript = m_asOwner.GetView(DocumentBannerView.STR_MVC_CLASS_NAME).DisplayJavascript();
      if (null != myScript)
      {
        scripts.AddInnerHTMLElement(myScript);
      }
      // Card Properties script
      myScript = new HTMLScript("text/javaScript");
      XPStringBuilder sb = new XPStringBuilder(100);
      ASURL myCardPropertiesURL = new ASURL(m_asOwner);
      myCardPropertiesURL.SetLinkGetCachedSpace(m_asOwner);
      myCardPropertiesURL.SetDisplayPage(DirCardPropertiesDP.STR_MVC_CLASS_NAME);
      myCardPropertiesURL.SetControl(DirRepostControl.STR_MVC_CLASS_NAME);
      try
      {
        sb = new XPStringBuilder(400);
        // Open Card Properties popup function
        sb.Append("function ").Append(DirConstants.JAVASCRIPT_CARDPROPERTIES_POPUP).Append("(cardID)\n");
        sb.Append("{\n");
        sb.Append("  window.open(\'").Append(myCardPropertiesURL.GetURLAsString()).Append("&").Append(DirConstants.QS_CARDID).Append("=\'+").Append("cardID, \'_blank1\', \'location=no,menubar=no,scrollbars=yes,height=550,width=600\');\n");
        sb.Append("}\n");
        // JavaScript to check all the rows
        sb.Append("function ").Append(JAVASCRIPT_CHECK_ALL_FOLDERS).Append("(inputCBAll, cbName)\n").Append("{\n").Append("\t\tvar frm = document.").Append(EditorDP.HTMLFORM_NAME).Append(";\n").Append("\t\tvar len = frm.elements.length;\n").Append("\t\tvar nValue = inputCBAll.checked;\n").Append("\t\tvar i = 0;\n").Append("\t\tfor(i=0 ; i<len ; i++)\n").Append("\t\t{\n").Append("\t\t\tif (frm.elements[i].name == cbName)\n").Append("\t\t\t{\n").Append("\t\t\t\tfrm.elements[i].checked = nValue;\n").Append("\t\t\t}\n").Append("\t\t}\n").Append("}\n");
        myScript.AddInnerHTMLString(sb.ToString());
        // Add any custom scripts
        if (null != m_strScript)
        {
          myScript.AddInnerHTMLString(m_strScript);
          m_strScript = null;
        }
      }
      catch (HTMLException he)
      {
        log.Error(he, "DirEditDP::DisplayJavascript The following error occured when trying to display the JavaScript: " + he.Message);
        throw new XPException(m_asOwner.GetString(107, "ptmsgs_portalbrowsingmsgs"),
         he);
      }
      scripts.AddInnerHTMLElement(myScript);
      return scripts;
    }
    
    /// <summary>
    /// Set the script that will be added in the header. 
    /// </summary>
    /// <param name="script">the script to add before closing</param>
    public virtual void SetScript(String script)
    {
      m_strScript = script;
    }
  }
}