using System;
using com.plumtree.openfoundation.util;
using OpenLogService=com.plumtree.openlog.OpenLogService;
using OpenLogger=com.plumtree.openlog.OpenLogger;
using PageData=com.plumtree.portalpages.common.editpage.PageData;
using PageLayoutHeaderView=com.plumtree.portalpages.common.editpage.PageLayoutHeaderView;
using IPTURL=com.plumtree.portaluiinfrastructure.ptlink.IPTURL;
using PTDebugHelpers=com.plumtree.portaluiinfrastructure.statichelpers.PTDebugHelpers;
using IPTPage=com.plumtree.server.IPTPage;
using IPTSession=com.plumtree.server.IPTSession;
using PT_PROPIDS=com.plumtree.server.PT_PROPIDS;
using com.plumtree.uiinfrastructure.activityspace;
using ExpListModel=com.plumtree.uiinfrastructure.expandablelist.ExpListModel;
using TaskAPIUICommunity=com.plumtree.taskapi.portalui.TaskAPIUICommunity;
using com.plumtree.server;
using com.plumtree.server.impl.community;
namespace com.plumtree.portalpages.browsing.editors.community
{
  /// <summary>
  /// This control creates a single page in the community without having to go into the community editor
  /// </summary>
  /// <author>mattc</author>
  public class PageCreateDefaultControl : IControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.editors.communityPageCreateDefaultControl");
    
    /// <summary>
    /// Object Name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "PageCreateDefault";
    
    /// <summary>
    /// Associated Activity Space 
    /// </summary>
    public AActivitySpace m_asOwner;
    
    /// <summary>
    /// Associated Model 
    /// </summary>
    protected CommunityEditorModel m_model;
    public const String HTMLINPUT_COMMUNITY_ID = "commID";
    public const String HTMLINPUT_PAGE_TEMPLATE_ID = "templateID";
    public const String HTMLINPUT_INHERIT_TEMPLATE = "inherit";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new PageCreateDefaultControl();
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    public virtual void Init(IModel model, AActivitySpace space)
    {
      m_model = (CommunityEditorModel) model;
      m_asOwner = space;
    }
    
    /// <summary>
    /// Simply creates a new page and redirects the browser back to that page
    /// </summary>
    /// <param name="arguments">contains HTTP arguments from the post</param>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      // get current session
      IPTSession currSession = (IPTSession) m_asOwner.GetUserSession();
      int communityID = -1;
      bool bInheritPageTemplate = true;
      int nPageTemplateID = PT_INTRINSICS.PT_BLANK_PAGETEMPLATEID;
      String[] data;
      // Community ID
      data = (String[]) arguments.GetElement(HTMLINPUT_COMMUNITY_ID);
      if ((null == data) || (0 == data.Length))
      {
        // no communityID specified; 
        // TODO: redirect to error page
        return null;
      }
      communityID = XPConvert.ToInteger(data[0]);
      // Inherit Page Template
      data = (String[]) arguments.GetElement(HTMLINPUT_INHERIT_TEMPLATE);
      if ((null != data) && (0 != data.Length))
      {
        bInheritPageTemplate = XPConvert.ToBoolean(data[0]);
      }
      // Template ID
      data = (String[]) arguments.GetElement(HTMLINPUT_PAGE_TEMPLATE_ID);
      if ((null != data) && (0 != data.Length))
      {
        nPageTemplateID = XPConvert.ToInteger(data[0]);
      }
      // open the community to get the admin folder
      IPTCommunity community = (IPTCommunity) currSession.GetCommunities().Open(communityID,
       false);
      IPTPage ptPage = (IPTPage) (currSession).GetPages().Create(community.GetAdminFolderID());
      ptPage.SetPageTemplateID(nPageTemplateID);
      ptPage.SetInheritPageTemplate(bInheritPageTemplate);
      ptPage.SetDefaultsFromPageTemplate(nPageTemplateID);
      ptPage.SetNotDeletable(false);
      // Store page
      ptPage.Store();
      ptPage.UnlockObject();
      // redirect to the new page
      int nPageID = ptPage.GetObjectID();
      String strPageName = ptPage.GetName();
      IPTURL url = TaskAPIUICommunity.CreateCommunityPageCommonOpenerURL(m_asOwner,
       communityID, nPageID, strPageName);
      Redirect redirect = new Redirect();
      redirect = new Redirect();
      redirect.SetLinkToExternalURL(url.GetFullURL());
      redirect.SetIsHTTPRedirect(true);
      return redirect;
    }
  }
}