using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portalpages.admin.editors.property;
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.portaluiinfrastructure.expandablelist.properties;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tree;
using com.plumtree.server;
using com.plumtree.server.treeiterator;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.expandablelist;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.uiinfrastructure.tree;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.editors.dircommon.editdocumentproperty
{
  /// <summary>
  /// This is the implementation of the Expandable List for editing card properties
  /// </summary>
  /// <author>stephenc</author>
  public class EditCardPropView : ExpListPropEditView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.editors.dircommon.editdocumentproperty.EditCardPropView");
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "EditCardProp";
    public const String HTMLINPUT_SELECT_NEW_PROP = "in_se_NewProperty";
    public const String HTMLINPUT_SELECT_NEW_PROP_TAGID = "NewPropTagID";
    public const String JAVASCRIPT_NEWCARDPROP = "CreateNewCardProp";
    
    /// <summary>
    /// JavaScript function to open pick tree. 
    /// </summary>
    protected const String JAVASCRIPT_PICKTREE_OPEN = "pickTreeOpen";
    
    /// <summary>
    /// JavaScript function to get pick tree value. 
    /// </summary>
    protected const String JAVASCRIPT_PICKTREE_GETVALUE = "pickTreeGetValue";
    
    /// <summary>
    /// Prefix for picklist input 
    /// </summary>
    public const String HTMLINPUT_SELECT_PICKLIST_PREFIX = "in_se_PickList";
    
    /// <summary>
    /// TagID for picklist 
    /// </summary>
    public const String HTMLINPUT_SELECT_PICKLIST_TAGID = "PickListTagID";
    
    /// <summary>
    /// Used to set the style of the row 
    /// </summary>
    private PTStyleClass rowStyleClass;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new EditCardPropView();
    }
    
    /// <summary>
    /// This method returns the HTMLScript that encompasses the Javascript on the page.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    /// <returns>HTMLScript</returns>
    public override HTMLScript DisplayJavascript()
    {
      IEditCardPropModelRO iModel = (IEditCardPropModelRO) m_model;
      HTMLScript myScript = base.DisplayJavascript();
      XPStringBuilder sb;
      try
      {
        // The URL to open a pick tree
        ASURL asURLPickTree = GetPickTreeURL();
        // JavaScript fucntion to open pick tree
        sb = new XPStringBuilder(200);
        sb.Append("function ").Append(iModel.GetUniqueString(JAVASCRIPT_PICKTREE_OPEN)).Append("(varPropertyID, varRowIndex)\n");
        sb.Append("{\n");
        sb.Append("  document.").Append(AFormDP.HTMLFORM_NAME).Append(".").Append(iModel.GetUniqueString(HTMLHIDDEN_ROWINDEX)).Append(".value = varRowIndex;\n");
        sb.Append("  window.open(\'").Append(asURLPickTree.GetURLAsString());
        sb.Append("&").Append(ServerStartTreeControl.INIT_DATA).Append("=\' + varPropertyID");
        sb.Append(", \'PickTreeWindow\', \'").Append(TreeConstants.GetJavascriptTreeWindowAttributes()).Append("\');\n");
        sb.Append("}\n");
        myScript.AddInnerHTMLString(sb.ToString());
        // JavaScript function to get pick tree value on return
        sb = new XPStringBuilder(100);
        sb.Append("function ").Append(iModel.GetUniqueString(JAVASCRIPT_PICKTREE_GETVALUE)).Append("()\n");
        sb.Append("{\n");
        sb.Append("  ").Append(AFormDP.JAVASCRIPT_SUBMIT_FORM).Append("(").Append(iModel.GetUniqueInt(ExpListControl.POSTTOSELF_ACTION_MODIFYROW)).Append(");\n");
        sb.Append("}\n");
        myScript.AddInnerHTMLString(sb.ToString());
        // Create new object
        sb = new XPStringBuilder(700);
        sb.Append(" function " + JAVASCRIPT_NEWCARDPROP + "(varURL)");
        sb.Append(" {");
        sb.Append("    window.open(varURL,\'PopUpCreateMode\',\'scrollbars=yes,location=no,menubar=no,height=600,width=800\');\n");
        sb.Append(" }\n");
        myScript.AddInnerHTMLString(sb.ToString());
      }
      catch (HTMLException he)
      {
        log.Error(he, "EditCardPropView::DisplayJavascript The following error occured when trying to display the JavaScript: " + he.Message);
        }
      return myScript;
    }
    
    /// <summary>
    /// Returns HTML for pick list
    /// </summary>
    /// <param name="_nRowIndex">Row index</param>
    /// <param name="_strValue">Property value as string</param>
    /// <returns>HTML for pick list</returns>
    /// <throws>HTMLException</throws>
    protected virtual HTMLElementCollection GetHTMLForPickList(int _nRowIndex,
     String _strValue)
    {
      HTMLElementCollection myCollection = new HTMLElementCollection();
      IEditCardPropModelRO iModel = (IEditCardPropModelRO) m_model;
      if (iModel.CheckPickValuesAreValid(_nRowIndex))
      {
        HTMLSelect mySelect = new HTMLSelect(iModel.GetUniqueString(HTMLINPUT_SELECT_PICKLIST_PREFIX) + _nRowIndex,
         iModel.GetUniqueString(HTMLINPUT_SELECT_PICKLIST_TAGID) + _nRowIndex);
        myCollection.AddInnerHTMLElement(mySelect);
        // Add the first option, which is empty value
        HTMLOption optionEmpty = new HTMLOption("");
        mySelect.AddInnerHTMLElement(optionEmpty);
        mySelect.SetStyleClass(PTStyleClass.FORM_SELECT_TEXT);
        // Open the property ObjectManager in order to open property.
        Object[][] arPickListValues = iModel.GetPickListValues(iModel.GetRowPropertyID(_nRowIndex));
        bool bValueInList = false;
        // Add drop down
        if (null != arPickListValues)
        {
          for (int nOption = 0;nOption < arPickListValues[0].Length;nOption++)
          {
            String strPickListVal = (String) arPickListValues[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_NAME][nOption];
            String strEncodedString = XPHttpUtility.HtmlEncode(strPickListVal);
            HTMLOption optionPickListVal = new HTMLOption(strEncodedString);
            optionPickListVal.AddInnerHTMLString(strEncodedString);
            mySelect.AddInnerHTMLElement(optionPickListVal);
            if (strPickListVal.Equals(_strValue))
            {
              bValueInList = true;
            }
          }
        }
        if (null != _strValue && 0 < _strValue.Length)
        {
          // If the option doesn't exist as a valid pick list value, then display alert message.		
          if (!bValueInList)
          {
            // Display alert message indicating that value is no longer valid
            HTMLSpan mySpan = new HTMLSpan();
            myCollection.AddInnerHTMLElement(mySpan);
            mySpan.SetStyleClass(PTStyleClass.ALERT_REQUIRED_TEXT);
            mySpan.AddInnerHTMLEncodedString(XPResourceManager.FormatMessage(m_asOwner.GetString(469,
             "ptmsgs_portalinfrastructure"), _strValue));
          } else
          {
            // Set the selected value
            mySelect.SetValue(XPHttpUtility.HtmlEncode(_strValue));
          }
        }
      } else
      {
        myCollection.AddInnerHTMLEncodedString(_strValue);
        myCollection.AddInnerHTMLString(" ");
        HTMLSpan mySpan = new HTMLSpan();
        myCollection.AddInnerHTMLElement(mySpan);
        mySpan.SetStyleClass(PTStyleClass.ALERT_REQUIRED_TEXT);
        mySpan.AddInnerHTMLEncodedString(m_asOwner.GetString(471, "ptmsgs_portalinfrastructure"));
        }
      return myCollection;
    }
    
    /// <summary>
    /// Returns HTML for pick tree
    /// </summary>
    /// <param name="_nRowIndex">Row index</param>
    /// <param name="_strValue">Property value as string</param>
    /// <returns>HTML for pick tree</returns>
    /// <throws>Exception</throws>
    protected virtual HTMLElement GetHTMLForPickTree(int _nRowIndex, String _strValue)
    {
      IEditCardPropModelRO iModel = (IEditCardPropModelRO) m_model;
      HTMLAnchor myAnchor;
      HTMLImg myImage;
      PTConstruct myConstruct;
      XPStringBuilder sbJavascriptOpenPickTree;
      HTMLElementCollection result = new HTMLElementCollection();
      result.AddInnerHTMLEncodedString(_strValue);
      if (iModel.CheckPickValuesAreValid(_nRowIndex))
      {
        myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        result.AddInnerHTMLElement(myAnchor);
        myConstruct = UIConstructFactory.GetConstruct(PTImageType.ACTION_EDIT,
         true, this.m_asOwner);
        myImage = (HTMLImg) myConstruct.GetHTMLElement();
        myAnchor.AddInnerHTMLElement(myImage);
        myImage.SetAlt(m_asOwner.GetString(89, "ptmsgs_portalinfrastructure"));
        myImage.SetWidth("25");
        myImage.SetHeight("25");
        myImage.SetBorder(CommonHTMLStrings.ZERO);
        myImage.SetAlign(CommonHTMLStrings.ABSMIDDLE);
        sbJavascriptOpenPickTree = new XPStringBuilder(100);
        sbJavascriptOpenPickTree.Append(iModel.GetUniqueString(JAVASCRIPT_PICKTREE_OPEN)).Append("(");
        // Append property ID
        sbJavascriptOpenPickTree.Append(iModel.GetRowPropertyID(_nRowIndex));
        sbJavascriptOpenPickTree.Append(", ");
        // Append current row to modify
        sbJavascriptOpenPickTree.Append(_nRowIndex);
        sbJavascriptOpenPickTree.Append("); return false;");
        myAnchor.SetOnClick(sbJavascriptOpenPickTree.ToString());
      } else
      {
        result.AddInnerHTMLString(" ");
        HTMLSpan mySpan = new HTMLSpan();
        result.AddInnerHTMLElement(mySpan);
        mySpan.SetStyleClass(PTStyleClass.ALERT_REQUIRED_TEXT);
        mySpan.AddInnerHTMLEncodedString(m_asOwner.GetString(472, "ptmsgs_portalinfrastructure"));
        }
      return result;
    }
    
    /// <summary>
    /// Returns the HTML for the row _nRowIndex. Each row contains just an icon
    /// and input text.
    /// </summary>
    /// <param name="_saData"></param>
    /// <param name="_nRowIndex"></param>
    /// <returns>HTMLElement</returns>
    /// <throws>Exception</throws>
    public override HTMLElement GetHTMLForRow(IPTGrowableSortedArrayWrapperRO _saData,
     int _nRowIndex)
    {
      IEditCardPropModelRO iECPModel = (IEditCardPropModelRO) m_model;
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLTableCell myCell;
      result.AddInnerHTMLElement(GetHTMLForName(_saData, _nRowIndex));
      myCell = new HTMLTableCell();
      result.AddInnerHTMLElement(myCell);
      myCell.SetWidth("80%");
      myCell.SetHeight("30");
      myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
      myCell.SetNoWrap(true);
      myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      Object oValue = iECPModel.GetRowValue(_nRowIndex);
      int nPropertyType = iECPModel.GetRowType(_nRowIndex);
      String strValue = iECPModel.GetStringFromObjectValue(oValue, nPropertyType);
      // Determine what kind of form element to display
      int nPickType = iECPModel.GetPickType(_nRowIndex);
      switch (nPickType){
      case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_MANAGED_PICKLIST:
      case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_UNMANAGED_PICKLIST:
      // Get drop down menu for picklist
      myCell.AddInnerHTMLElement(GetHTMLForPickList(_nRowIndex, strValue));
      break;
      case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_PICKTREE:
      // Get picktree link
      myCell.AddInnerHTMLElement(GetHTMLForPickTree(_nRowIndex, strValue));
      break;
      default:
      myCell.AddInnerHTMLElement(GetHTMLElementForRow(_nRowIndex, oValue));
      break;
      }
      return result;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Empty method that sub classes can averride to add
    /// actions to the action bar (like a create object action).
    /// </summary>
    /// <param name="_myRow">Action bar row object.</param>
    /// <throws>Exception</throws>
    protected override void AddHTMLToActionBar(HTMLTableRow _myRow)
    {
      HTMLTableCell myCell;
      HTMLImg myImage;
      HTMLAnchor myAnchor;
      HTMLSpan mySpan;
      int nClassID = PT_CLASSIDS.PT_PROPERTY_ID;
      String sEditorASName = PropEditorAS.STR_MVC_CLASS_NAME;
      ASURL myURL = new ASURL(m_asOwner);
      myURL.SetLinkCreateNewSpace(sEditorASName, m_asOwner);
      myURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
      myURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_CREATE));
      myURL.AddControlArgument(EditorStartControl.QS_DELETE_ON_CANCEL, XPConvert.ToString(true));
      myURL.SetSubSpaceKey(sEditorASName);
      myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID, XPConvert.ToString(nClassID));
      myCell = new HTMLTableCell();
      _myRow.AddInnerHTMLElement(myCell);
      myCell.SetHeight("25");
      myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
      myCell.SetNoWrap(true);
      myCell.AddInnerHTMLString(SPACE);
      // mj: replaced the following line with a blank constructur and SetOnClick()
      // myAnchor = new HTMLAnchor(JAVASCRIPT_NEWCARDPROP +
      myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
      myAnchor.SetOnClick(JAVASCRIPT_NEWCARDPROP + "(\'" + myURL.GetURLAsString() + "\');");
      HTMLImg myNewImage = (HTMLImg) PTImageType.ACTION_NEW.GetHTMLElement();
      myNewImage.SetAlt("");
      myNewImage.SetAlign("absmiddle");
      myAnchor.AddInnerHTMLElement(myNewImage);
      // property icon
      myImage = (HTMLImg) PTImageType.ICON_PROPERTY.GetHTMLElement();
      myImage.SetAlt("");
      myImage.SetAlign("absmiddle");
      myAnchor.AddInnerHTMLElement(myImage);
      mySpan = new HTMLSpan();
      myAnchor.AddInnerHTMLElement(mySpan);
      mySpan.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);
      // Create label
      mySpan.AddInnerHTMLString(m_asOwner.GetString(350, "ptmsgs_portalbrowsingmsgs"));
      mySpan.AddInnerHTMLString(SPACE);
      mySpan.AddInnerHTMLString(SPACE);
      myCell.AddInnerHTMLElement(myAnchor);
      myCell.AddInnerHTMLString(SPACE);
    }
    
    /// <param name="_nRowIndex"></param>
    /// <returns></returns>
    /// <throws>HTMLException </throws>
    protected override HTMLTableCell GetHTMLForCheckBox(int _nRowIndex)
    {
      IEditCardPropModelRO iECPModel = (IEditCardPropModelRO) m_model;
      HTMLTableCell myCell;
      HTMLInput myInput;
      myCell = new HTMLTableCell();
      myCell.SetHeight("30");
      myCell.SetAlign("center");
      myCell.SetVAlign("middle");
      myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
      // checkbox
      myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, iECPModel.GetUniqueString(HTMLCHECKBOX_DELROW),
       iECPModel.GetDeleteIconAlt());
      myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
      myInput.SetValue(XPConvert.ToString(_nRowIndex));
      myCell.AddInnerHTMLElement(myInput);
      return myCell;
    }
    
    /// <param name="_saData"></param>
    /// <param name="_nRowIndex"></param>
    /// <returns></returns>
    /// <throws>HTMLException </throws>
    protected virtual HTMLTableCell GetHTMLForName(IPTGrowableSortedArrayWrapperRO _saData,
     int _nRowIndex)
    {
      IEditCardPropModelRO iECPModel = (IEditCardPropModelRO) m_model;
      HTMLTableCell myCell = new HTMLTableCell();
      myCell.SetWidth("22%");
      myCell.SetHeight("30");
      myCell.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
      String strName = iECPModel.GetRowName(_nRowIndex);
      int nStatus = iECPModel.GetRowStatus(_nRowIndex);
      // Display name
      if (EditCardPropModel.ROW_STATUS_ADDED_NEW == nStatus)
      {
        myCell.AddInnerHTMLElement(GetUnusedPropertiesHTMLSelect());
      } else
      {
        myCell.AddInnerHTMLString("<B>" + XPHttpUtility.HtmlEncode(strName) + "</B>");
        }
      return myCell;
    }
    
    /// <summary>
    /// Generates a list of properties to be added.  Only card props that have not been chosen are offered
    /// </summary>
    /// <returns></returns>
    /// <throws>HTMLException </throws>
    protected virtual HTMLSelect GetUnusedPropertiesHTMLSelect()
    {
      IEditCardPropModelRO iECPModel = (IEditCardPropModelRO) m_model;
      HTMLSelect mySelect = new HTMLSelect(HTMLINPUT_SELECT_NEW_PROP, HTMLINPUT_SELECT_NEW_PROP_TAGID);
      mySelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
      mySelect.SetOnChange(AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" + RepostControl.POSTTOSELF_ACTION_SIMPLEREPOST + ");");
      XPSortedDictionary xpUnusedProperties = iECPModel.GetUnusedProperties();
      IXPEnumerator xpEnum;
      IXPEnumerator xpKeys;
      xpEnum = xpUnusedProperties.GetElements();
      xpKeys = xpUnusedProperties.GetKeys();
      int nNewlySelectedPropertyID = iECPModel.GetNewlySelectedPropertyID();
      while (xpEnum.MoveNext() && xpKeys.MoveNext())
      {
        String strPropName = (String) xpKeys.GetCurrent();
        int nPropID = XPConvert.ToInteger(xpEnum.GetCurrent());
        HTMLOption optionVal = new HTMLOption(XPConvert.ToString(nPropID));
        if (nNewlySelectedPropertyID == nPropID)
        {
          optionVal.SetSelected(true);
        }
        mySelect.AddInnerHTMLElement(optionVal);
        optionVal.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strPropName));
        }
      return mySelect;
    }
    
    /// <summary>
    /// Helper method to set the style of a row.  This method is used by
    /// Display(). Overrides the ExpListView::SetExpListRowStyle().
    /// </summary>
    /// <param name="_myRow"></param>
    /// <param name="_nRowIndex"></param>
    /// <param name="_nLastVisibleRow"></param>
    /// <throws>HTMLException</throws>
    /// <seealso cref="com.plumtree.uiinfrastructure.expandablelist.ExpListView#SetExpListRowStyle(HTMLTableRow,int,int)"></seealso>
    protected virtual void SetRowStyle(HTMLTableRow _myRow, int _nRowIndex,
     int _nLastVisibleRow)
    {
      IEditCardPropModelRO iOPModel = (IEditCardPropModelRO) m_model;
      // Get current property ID
      int nPropertyID = iOPModel.GetRowPropertyID(_nRowIndex);
      // Get last visible property ID
      int nLastVisiblePropertyID = PT_INTRINSICS.PT_NULL_OBJECTID;
      if (-1 != _nLastVisibleRow)
      {
        nLastVisiblePropertyID = iOPModel.GetRowPropertyID(_nLastVisibleRow);
        }
      // Check if we have moved on to a new property
      if (nPropertyID != nLastVisiblePropertyID)
      {
        // Toggle style index if we have a new property
        if (PTStyleClass.LIST_ITEM_ONE_BG == rowStyleClass)
        {
          rowStyleClass = PTStyleClass.LIST_ITEM_TWO_BG;
        } else
        {
          rowStyleClass = PTStyleClass.LIST_ITEM_ONE_BG;
        }
      }
      _myRow.SetStyleClass(rowStyleClass);
    }
    
    /// <summary>
    /// Return ASURL for pick tree
    /// </summary>
    /// <returns>ASURL for pick tree</returns>
    protected virtual ASURL GetPickTreeURL()
    {
      IEditCardPropModelRO iModel = (IEditCardPropModelRO) m_model;
      // The URL to open a pick tree as a subspace.  We want to use the same pick tree each time.
      ASURL asURLPickTree = new ASURL(m_asOwner);
      asURLPickTree.SetLinkCreateNewSpace(ServerTreeAS.STR_MVC_CLASS_NAME,
       m_asOwner);
      asURLPickTree.SetSubSpaceKey(iModel.GetUniqueString(EditCardPropModel.SUBSPACE_KEY_PICKTREE));
      asURLPickTree.SetControl(ServerStartTreeControl.STR_MVC_CLASS_NAME);
      asURLPickTree.AddControlArgument(ServerStartTreeControl.INTERFACE_TYPE,
       XPConvert.ToString(ServerTreeModel.m_nPROP_PICK_TREE));
      asURLPickTree.AddControlArgument(AStartTreeControl.MULTI_SELECT, XPConvert.ToString(false));
      asURLPickTree.AddControlArgument(AStartTreeControl.SELECT_MODE, XPConvert.ToString(TreeConstants.N_LEAFSELECT));
      asURLPickTree.AddControlArgument(AStartTreeControl.SUBMIT_MODE, XPConvert.ToString(TreeConstants.N_SERVERSUBMIT));
      asURLPickTree.AddControlArgument(AStartTreeControl.PARENT_FORM_NAME,
       AFormDP.HTMLFORM_NAME);
      asURLPickTree.AddControlArgument(AStartTreeControl.PARENT_SUBMIT_CALL,
       iModel.GetUniqueString(JAVASCRIPT_PICKTREE_GETVALUE));
      asURLPickTree.AddControlArgument(AStartTreeControl.TITLE, m_asOwner.GetString(91,
       "ptmsgs_portalinfrastructure"));
      asURLPickTree.AddControlArgument(AStartTreeControl.ACTIONSUBTITLE, m_asOwner.GetString(90,
       "ptmsgs_portalinfrastructure"));
      asURLPickTree.AddControlArgument(AStartTreeControl.SHOW_ROOT, XPConvert.ToString(false));
      return asURLPickTree;
    }
  }
}