using System;
// Framework Imports
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.sitemap.sitemappage;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.editors.sitemap.sitemapfolderplugin
{
  /// <summary>
  /// This class displays the HTML for the web link selector attached to the Site
  /// Map folder plug-in. 
  /// PAI - WARNING!  Only user GetASOwner() in this file - never call m_asOwner.
  /// File: SiteMapCreateLinkView. java
  /// </summary>
  /// <author>John Hogan</author>
  public class SiteMapCreateWebLinkView : EditorView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.editors.sitemap.sitemapfolderplugin.SiteMapCreateWebLinkView");
    
    /// <summary>
    /// The unique identifier for this class 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "SiteMapCreateWebLinkView";
    // HTML Object names.
    
    /// <summary>
    /// HTML Input for the Link name 
    /// </summary>
    public const String HTMLINPUT_LINKNAME = "in_tx_LinkName";
    
    /// <summary>
    /// HTML Input for the link description 
    /// </summary>
    public const String HTMLINPUT_LINKDESC = "in_tx_LinkDesc";
    
    /// <summary>
    /// HTML Input for the Link URL 
    /// </summary>
    public const String HTMLINPUT_LINKURL = "in_tx_LinkURL";
    
    /// <summary>
    /// HTML Input for the name of the already created web link being edited 
    /// </summary>
    public const String HTMLINPUT_EDITLINKNAME = "in_tx_EditLinkName";
    
    /// <summary>
    /// HTML Input for the Description of the already created web link being edited 
    /// </summary>
    public const String HTMLINPUT_EDITLINKDESC = "in_tx_EditLinkDesc";
    
    /// <summary>
    /// HTML Input for the URL of the already created web link being edited 
    /// </summary>
    public const String HTMLINPUT_EDITLINKURL = "in_tx_EditLinkURL";
    
    /// <summary>
    /// Button for adding the link to the existing list 
    /// </summary>
    public const String HTMLBUTTON_REPOST = "in_bu_add";
    
    /// <summary>
    /// The OK button - closes the form and saves the changes 
    /// </summary>
    public const String HTMLBUTTON_OK = "in_bu_OK";
    
    /// <summary>
    /// The cancel button - closes but doesn't save 
    /// </summary>
    public const String HTMLBUTTON_CANCEL = "in_bu_Cancel";
    
    /// <summary>
    /// The check all checkbox 
    /// </summary>
    public const String HTMLCHECKBOX_CHECKALL = "in_cb_CheckAll";
    
    /// <summary>
    /// The check all checkbox 
    /// </summary>
    public const String HTMLCHECKBOX_WEBLINK = "in_cb_WebLink";
    
    /// <summary>
    /// Javascript to post changes and close the form 
    /// </summary>
    public const String JAVASCRIPT_POSTANDCLOSE = "postAndClose";
    
    /// <summary>
    /// Javascript to validate current and close the form 
    /// </summary>
    public const String JAVASCRIPT_VALIDATEBEFORECLOSE = "ValidateAndClose";
    
    /// <summary>
    /// Javascript to close the form without saving 
    /// </summary>
    public const String JAVASCRIPT_CANCEL = "closeForm";
    
    /// <summary>
    /// Web link javascript validation function 
    /// </summary>
    public const String JAVASCRIPT_VALIDWEBLINK = "ValidWebLink";
    
    /// <summary>
    /// Javascript to validate entry and move it to the selected list 
    /// </summary>
    public const String JAVASCRIPT_VALIDATEANDPOST = "ValidateAndPost";
    
    /// <summary>
    /// Javascript to check all the web links 
    /// </summary>
    public const String JAVASCRIPT_CHECKALL = "CheckAllLinks";
    
    /// <summary>
    /// Javascript to edit a given row 
    /// </summary>
    public const String JAVASCRIPT_EDITROW = "EditRow";
    
    /// <summary>
    /// Javascript to remove links from the list 
    /// </summary>
    public const String JAVASCRIPT_REMOVELINKS = "RemoveLinks";
    
    /// <summary>
    /// Javascript to validate the edit web link in the list of web links (as opposed to the create) 
    /// </summary>
    public const String JAVASCRIPT_VALIDATEEDIT = "ValidateEdit";
    private SiteMapAS m_asMyOwner;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SiteMapCreateWebLinkView();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO,AActivitySpace)"></seealso>
    public override void Init(IModelRO model, AActivitySpace parent)
    {
      if (parent.GetName().Equals(SiteMapAS.STR_MVC_CLASS_NAME))
      {
        m_model = model;
        m_asMyOwner = (SiteMapAS) parent;
        m_asOwner = null;
      } else
      {
        base.Init(model, parent);
      }
    }
    
    /// <summary>
    /// WARNING - you must use this function in this class - you should never use
    /// m_asOwner.  
    /// The reason for this function is that m_asOwner is cast to an EditorAS by the
    /// framework, because this is an EditorView.  However, we need this class in 
    /// non-editor places, so we override the Init (above), and use a different
    /// variable if we need to.  Calling GetASOwner() will give the appropriate space.
    /// </summary>
    /// <returns>AActivitySpace</returns>
    private AActivitySpace GetASOwner()
    {
      if (m_asOwner == null)
      {
        return m_asMyOwner;
      } else
      {
        return m_asOwner;
      }
    }
    
    /// <summary>
    /// Returns the HTML for the Section.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public override HTMLElement Display()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLTable myTable = null;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      try
      {
        result.AddInnerHTMLString("\n");
        result.SetPreComment("Begin: Display SiteMapCreateLinkView");
        result.AddInnerHTMLString("\n");
        myTable = new HTMLTable();
        result.AddInnerHTMLElement(myTable);
        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.SetBorder(CommonHTMLStrings.ZERO);
        myTable.SetCellPadding(CommonHTMLStrings.FIVE);
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        //JH- First row, first cell: The header
        myTable.AddInnerHTMLElement(GetHTMLForBanner());
        //A spacer row
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetHeight("5");
        //The subheader
        myTable.AddInnerHTMLElement(GetHTMLForSubHeader());
        //Spacer table
        result.AddInnerHTMLElement(EditorView.GetSpacerTable());
        //The link creation section
        result.AddInnerHTMLElement(GetHTMLForLinkCreation());
        //Spacer table
        result.AddInnerHTMLElement(EditorView.GetSpacerTable());
        //The Links list
        result.AddInnerHTMLElement(GetHTMLForLinkList());
        // 3rd Row end
        HTMLComment myComment = new HTMLComment("End: Display SiteMapCreateLinkView");
        result.AddInnerHTMLElement(myComment);
      }
      catch (Exception e)
      {
        log.Error(e, e.Message);
      }
      return result;
    }
    
    /// <summary>
    /// Returns the HTML for the banner
    /// </summary>
    /// <returns>HTMLTableRow</returns>
    protected virtual HTMLTableRow GetHTMLForBanner()
    {
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLInput myInput;
      HTMLH1 headerH1;
      myRow = new HTMLTableRow();
      myRow.SetStyleClass(PTStyleClass.BAN_HEADER_BG);
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      headerH1 = new HTMLH1();
      myCell.AddInnerHTMLElement(headerH1);
      headerH1.AddInnerHTMLString(GetASOwner().GetString(1489, "ptmsgs_portalbrowsingmsgs"));
      headerH1.SetStyleClass(PTStyleClass.BAN_HEADER);
      //Second cell = the close and cancel buttons
      myCell = new HTMLTableCell();
      myCell.SetWidth("300");
      myCell.SetAlign("right");
      myRow.AddInnerHTMLElement(myCell);
      // DCA - copied from EditorHeaderView so that buttons diplay next to each other instead of on top of
      HTMLTable mySubTable = new HTMLTable();
      mySubTable.SetBorder("0");
      mySubTable.SetCellPadding("5");
      mySubTable.SetCellSpacing("0");
      myCell.AddInnerHTMLElement(mySubTable);
      // button table 1st Row
      HTMLTableRow mySubRow = new HTMLTableRow();
      mySubTable.AddInnerHTMLElement(mySubRow);
      HTMLTableCell mySubCell = new HTMLTableCell();
      mySubRow.AddInnerHTMLElement(mySubCell);
      mySubCell.SetHeight("20");
      mySubCell.SetAlign("right");
      mySubCell.SetVAlign("middle");
      mySubCell.SetNoWrap(true);
      myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_OK, GetASOwner().GetString(62,
       "ptmsgs_portalbrowsingmsgs"));
      myInput.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
      myInput.SetValue(GetASOwner().GetString(81, "ptmsgs_portalbrowsingmsgs"));
      myInput.SetOnClick(JAVASCRIPT_POSTANDCLOSE + "();");
      mySubCell.AddInnerHTMLElement(myInput);
      mySubCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_CANCEL, GetASOwner().GetString(82,
       "ptmsgs_portalbrowsingmsgs"));
      myInput.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
      myInput.SetValue(GetASOwner().GetString(82, "ptmsgs_portalbrowsingmsgs"));
      myInput.SetOnClick(JAVASCRIPT_CANCEL + "();");
      mySubCell.AddInnerHTMLElement(myInput);
      return myRow;
    }
    
    /// <summary>
    /// Returns the HTML just below the banner
    /// </summary>
    /// <returns>HTMLTableRow</returns>
    protected virtual HTMLTableRow GetHTMLForSubHeader()
    {
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLTableCell helpCell;
      HTMLSpan mySpan = new HTMLSpan();
      // Row 1 - Cell 1 - Explanatory text
      myRow = new HTMLTableRow();
      myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
      myCell = new HTMLTableCell();
      myCell.SetWidth("100%");
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      myRow.AddInnerHTMLElement(myCell);
      myCell.AddInnerHTMLString(GetASOwner().GetString(1490, "ptmsgs_portalbrowsingmsgs"));
      // row 1 - cell 2 - help link
      helpCell = new HTMLTableCell();
      helpCell.SetWidth("100%");
      helpCell.SetNoWrap(true);
      helpCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      helpCell.SetAlign("right");
      mySpan.SetStyleClass(PTStyleClass.BAN_HELP_TEXT);
      mySpan.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      //	set the help link 
      GetASOwner().SetHelpTopic("PT_SelectLinks_WebPages");
      HTMLAnchor helpAnchor = HTMLConstructHelpers.GetHelpAnchor(GetASOwner());
      mySpan.AddInnerHTMLElement(helpAnchor);
      helpCell.AddInnerHTMLElement(mySpan);
      myRow.AddInnerHTMLElement(helpCell);
      return myRow;
    }
    
    /// <summary>
    /// Returns the HTML for the link selection form.  Links entered here can be
    /// moved to the list below by clicking on the button
    /// </summary>
    /// <returns>HTMLTable</returns>
    protected virtual HTMLTable GetHTMLForLinkCreation()
    {
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLInput myInput;
      HTMLSpan mySpan;
      myTable = new HTMLTable();
      myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      myTable.SetBorder(CommonHTMLStrings.ZERO);
      myTable.SetCellPadding(CommonHTMLStrings.FIVE);
      myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
      // 1st Row begin - the header row
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.OBJECT_HEADER_BG);
      myCell = new HTMLTableCell();
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      myCell.SetColSpan("2");
      myRow.AddInnerHTMLElement(myCell);
      mySpan = new HTMLSpan();
      myCell.AddInnerHTMLElement(mySpan);
      mySpan.SetStyleClass(PTStyleClass.OBJECT_HEADER);
      //"Properties"
      mySpan.AddInnerHTMLString(GetASOwner().GetString(927, "ptmsgs_portalbrowsingmsgs"));
      // 1st Row end
      //Second row begin - The link name and input box
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
      //First Cell - the link name
      myCell = new HTMLTableCell();
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      myRow.AddInnerHTMLElement(myCell);
      mySpan = new HTMLSpan();
      myCell.AddInnerHTMLElement(mySpan);
      mySpan.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
      //"Link Name"
      mySpan.AddInnerHTMLString(GetASOwner().GetString(913, "ptmsgs_portalbrowsingmsgs"));
      // 1st Row second cell = the Name input box
      myCell = new HTMLTableCell();
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      myCell.SetWidth("100%");
      myRow.AddInnerHTMLElement(myCell);
      myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_LINKNAME, "");
      myCell.AddInnerHTMLElement(myInput);
      myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
      //myInput.SetValue(GetASOwner().GetString(917,
      //		"ptmsgs_portalbrowsingmsgs"));
      myInput.SetMaxLength(63);
      myInput.SetSize(35);
      // 2nd Row end
      // 3rd Row begin
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
      //First cell 
      myCell = new HTMLTableCell();
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      myRow.AddInnerHTMLElement(myCell);
      mySpan = new HTMLSpan();
      myCell.AddInnerHTMLElement(mySpan);
      mySpan.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
      //"Description"
      mySpan.AddInnerHTMLString(GetASOwner().GetString(914, "ptmsgs_portalbrowsingmsgs"));
      // 3rd Row second cell - the description input box
      myCell = new HTMLTableCell();
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      myCell.SetWidth("100%");
      myRow.AddInnerHTMLElement(myCell);
      myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_LINKDESC, "");
      myCell.AddInnerHTMLElement(myInput);
      myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
      //myInput.SetValue(GetASOwner().GetString(918,
      //		"ptmsgs_portalbrowsingmsgs"));
      myInput.SetMaxLength(255);
      myInput.SetSize(35);
      // 4th Row begin
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
      //4th Row First cell
      myCell = new HTMLTableCell();
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      myRow.AddInnerHTMLElement(myCell);
      mySpan = new HTMLSpan();
      myCell.AddInnerHTMLElement(mySpan);
      mySpan.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
      //"Web Address (URL)"
      mySpan.AddInnerHTMLString(GetASOwner().GetString(915, "ptmsgs_portalbrowsingmsgs"));
      // 4th Row second cell - the URL input box
      myCell = new HTMLTableCell();
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      myCell.SetWidth("100%");
      myRow.AddInnerHTMLElement(myCell);
      myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_LINKURL, "");
      myCell.AddInnerHTMLElement(myInput);
      myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
      //myInput.SetValue("http://");
      myInput.SetMaxLength(255);
      myInput.SetSize(50);
      // 4th Row end
      // 5th Row begin
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      //5th row First cell - empty cell
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      //5th row Second cell - the Add button
      myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_REPOST, GetASOwner().GetString(919,
       "ptmsgs_portalbrowsingmsgs"));
      myCell.AddInnerHTMLElement(myInput);
      myInput.SetValue(GetASOwner().GetString(1903, "ptmsgs_portalbrowsingmsgs"));
      myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
      myInput.SetOnClick(JAVASCRIPT_VALIDATEANDPOST + "(); return false;");
      return myTable;
    }
    
    /// <summary>
    /// Returns the HTML for the Web link list section.
    /// </summary>
    /// <returns>HTMLTable</returns>
    protected virtual HTMLElementCollection GetHTMLForLinkList()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLSpan mySpan;
      HTMLAnchor myAnchor;
      PTConstruct myConstruct;
      myTable = new HTMLTable();
      result.AddInnerHTMLElement(myTable);
      myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      myTable.SetBorder(CommonHTMLStrings.ZERO);
      myTable.SetCellPadding(CommonHTMLStrings.FIVE);
      myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
      // 1st Row begin - the header row
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.OBJECT_HEADER_BG);
      myCell = new HTMLTableCell();
      myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      myCell.SetColSpan("2");
      myRow.AddInnerHTMLElement(myCell);
      mySpan = new HTMLSpan();
      myCell.AddInnerHTMLElement(mySpan);
      mySpan.SetStyleClass(PTStyleClass.OBJECT_HEADER);
      //"Properties"
      mySpan.AddInnerHTMLString(GetASOwner().GetString(894, "ptmsgs_portalbrowsingmsgs"));
      // 1st Row end
      HTMLElementCollection myCollection = new HTMLElementCollection();
      if (((ISiteMapFolderPlugInModelRO) m_model).GetWebLinkCount() > 0)
      {
        //Second row begin - the actionbar
        myConstruct = ConstructFactory.GetConstruct(PTImageType.ACTION_REMOVE);
        myAnchor = HTMLConstructHelpers.GetActionBarAnchor(0, (HTMLImg) myConstruct.GetHTMLElement(),
         GetASOwner().GetString(302, "ptmsgs_portalbrowsingmsgs"), GetASOwner().GetString(302,
         "ptmsgs_portalbrowsingmsgs"), JAVASCRIPT_REMOVELINKS + "(); return false;",
         GetASOwner());
        myCollection.AddInnerHTMLElement(myAnchor);
      } else
      {
        mySpan = new HTMLSpan();
        myCollection.AddInnerHTMLElement(mySpan);
      }
      result.AddInnerHTMLElement(HTMLConstructHelpers.GetActionBar(myCollection,
       "SiteMapCreateLinkView", GetASOwner()));
      //Short spacer table
      result.AddInnerHTMLElement(EditorView.GetShortSpacerTable());
      result.AddInnerHTMLElement(DisplayWebLinks());
      return result;
    }
    
    /// <summary>
    /// Returns the HTML for the Web Links list.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public virtual HTMLElement DisplayWebLinks()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLTable myTable = null;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLInput myInput;
      PTButtonType myType;
      PTConstruct myConstruct;
      HTMLAnchor myAnchor;
      HTMLImg myImage;
      String strUniqueID = "";
      bool bEditMode = false;
      try
      {
        String[][] webLinks = ((ISiteMapFolderPlugInModelRO) m_model).GetWebLinks();
        if (webLinks.Length != 0)
        {
          myTable = new HTMLTable();
          myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
          myTable.SetBorder(CommonHTMLStrings.ZERO);
          myTable.SetCellPadding(CommonHTMLStrings.ONE);
          myTable.SetCellSpacing(CommonHTMLStrings.ONE);
          myTable.SetStyleClass(PTStyleClass.LIST_TEXT);
          result.AddInnerHTMLElement(myTable);
          myRow = new HTMLTableRow();
          myRow.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);
          myTable.AddInnerHTMLElement(myRow);
          myCell = new HTMLTableCell();
          myCell.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
          myCell.SetWidth("25");
          myCell.SetAlign(CommonHTMLStrings.CENTER);
          myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
          myRow.AddInnerHTMLElement(myCell);
          myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLCHECKBOX_CHECKALL,
           GetASOwner().GetString(941, "ptmsgs_portalbrowsingmsgs"));
          myInput.SetOnClick(JAVASCRIPT_CHECKALL + "(); return true;");
          myCell.AddInnerHTMLElement(myInput);
          //Second Cell - name header
          myCell = new HTMLTableCell();
          myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
          myCell.AddInnerHTMLString(GetASOwner().GetString(896, "ptmsgs_portalbrowsingmsgs"));
          myRow.AddInnerHTMLElement(myCell);
          //Third Cell - URL header
          myCell = new HTMLTableCell();
          myCell.SetColSpan("2");
          myCell.SetWidth("100%");
          myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
          myCell.AddInnerHTMLString(GetASOwner().GetString(898, "ptmsgs_portalbrowsingmsgs"));
          myRow.AddInnerHTMLElement(myCell);
          for (int i = 0;i < webLinks.Length;i++)
          {
            if (webLinks[i][0] == null)
            {
              break;
            }
            myRow = new HTMLTableRow();
            if (0 == (i % 2))
            {
              myRow.SetStyleClass(PTStyleClass.LIST_ITEM_TWO_BG);
            } else
            {
              myRow.SetStyleClass(PTStyleClass.LIST_ITEM_ONE_BG);
            }
            myTable.AddInnerHTMLElement(myRow);
            myCell = new HTMLTableCell();
            myCell.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
            myCell.SetWidth("25");
            myCell.SetAlign(CommonHTMLStrings.CENTER);
            myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
            myRow.AddInnerHTMLElement(myCell);
            myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLCHECKBOX_WEBLINK,
             GetASOwner().GetString(895, "ptmsgs_portalbrowsingmsgs"));
            myInput.SetMaxLength(63);
            //Set the value to the this web link will hold in the webLinks array
            strUniqueID = webLinks[i][0];
            myInput.SetValue(webLinks[i][0]);
            myCell.AddInnerHTMLElement(myInput);
            if (strUniqueID.Equals(((ISiteMapFolderPlugInModelRO) m_model).GetEditedID()))
            {
              bEditMode = true;
            } else
            {
              bEditMode = false;
            }
            //New cell for the name
            myCell = new HTMLTableCell();
            myCell.SetNoWrap(true);
            myCell.SetWidth("30%");
            myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
            myRow.AddInnerHTMLElement(myCell);
            if (bEditMode)
            {
              myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_EDITLINKNAME,
               GetASOwner().GetString(896, "ptmsgs_portalbrowsingmsgs"));
              myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
              myInput.SetSize("30");
              myInput.SetValue(webLinks[i][1]);
              myCell.AddInnerHTMLElement(myInput);
            } else
            {
              //Add the name
              //myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
              //myAnchor.SetTarget(webLinks[i][3]);
              //DCA - htmlencoding incase url has a '>' char prematurely closing the anchor tag
              myAnchor = new HTMLAnchor(XPHttpUtility.HtmlEncode(webLinks[i][3]));
              myAnchor.SetTarget("_blank");
              myAnchor.AddInnerHTMLString(XPHttpUtility.HtmlEncode(webLinks[i][1]));
              myCell.AddInnerHTMLElement(myAnchor);
            }
            //Now for the description - just put it on a new line
            myCell.AddInnerHTMLElement(new HTMLBr());
            //Add the description
            if (bEditMode)
            {
              myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_EDITLINKDESC,
               GetASOwner().GetString(897, "ptmsgs_portalbrowsingmsgs"));
              myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
              myInput.SetSize("30");
              myInput.SetValue(webLinks[i][2]);
              myCell.AddInnerHTMLElement(myInput);
            } else
            {
              myCell.AddInnerHTMLString(XPHttpUtility.HtmlEncode(webLinks[i][2]));
              }
            //New cell for the URL
            myCell = new HTMLTableCell();
            myCell.SetNoWrap(true);
            myCell.SetWidth("100%");
            myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
            myRow.AddInnerHTMLElement(myCell);
            if (bEditMode)
            {
              myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_EDITLINKURL,
               GetASOwner().GetString(898, "ptmsgs_portalbrowsingmsgs"));
              myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
              myInput.SetSize("30");
              myInput.SetValue(webLinks[i][3]);
              myCell.AddInnerHTMLElement(myInput);
            } else
            {
              myCell.AddInnerHTMLString(XPHttpUtility.HtmlEncode(webLinks[i][3]));
              }
            //Last cell the edit button
            myCell = new HTMLTableCell();
            myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
            myRow.AddInnerHTMLElement(myCell);
            if (bEditMode)
            {
              myType = PTButtonType.BROWSE;
            } else
            {
              myType = PTButtonType.EDIT;
            }
            //	Get the construct, do update the strings.
            myConstruct = UIConstructFactory.GetConstruct(myType, true, this.GetASOwner());
            myConstruct.ReplaceNestedElementsWithHTMLString(myType.TITLE_ELEMENT_NAME,
             "");
            myAnchor = (HTMLAnchor) myConstruct.GetNestedElement(myType.ANCHOR_ELEMENT_NAME);
            //Implement the on click action to edit the field
            myAnchor.SetHRef(ConstructStaticHelpers.BLANK_URL);
            myImage = (HTMLImg) myConstruct.GetNestedElement(myType.IMG_ELEMENT_NAME);
            myImage.SetAlt(GetASOwner().GetString(876, "ptmsgs_portalbrowsingmsgs"));
            /*String strBooleanEditMode;
             if (bEditMode)
             {
             strBooleanEditMode = "true";
             }
             else
             {
             strBooleanEditMode = "false";
             }
             */
             
            //myAnchor.SetOnClick(JAVASCRIPT_EDITROW + "('" + webLinks[i][0] + "', " + strBooleanEditMode + "); return false;");
            myAnchor.SetOnClick(JAVASCRIPT_EDITROW + "(\'" + webLinks[i][0] + "\'" + "); return false;");
            myCell.AddInnerHTMLElement(myAnchor);
          }
        } else
        {
          //Display the empty list message
          myTable = new HTMLTable();
          result.AddInnerHTMLElement(myTable);
          myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
          myTable.SetBorder(CommonHTMLStrings.ZERO);
          myTable.SetCellPadding(CommonHTMLStrings.ZERO);
          myTable.SetCellSpacing(CommonHTMLStrings.TWO);
          myCell = new HTMLTableCell();
          myCell.SetColSpan(CommonHTMLStrings.TWO);
          myCell.SetStyleClass(PTStyleClass.ALERT_ERROR_TITLE);
          myCell.AddInnerHTMLString(GetASOwner().GetString(1497, "ptmsgs_portalbrowsingmsgs"));
          myRow = new HTMLTableRow();
          myRow.AddInnerHTMLElement(myCell);
          myTable.AddInnerHTMLElement(myRow);
          myCell = new HTMLTableCell();
          myCell.SetColSpan(CommonHTMLStrings.TWO);
          myCell.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);
          myCell.AddInnerHTMLString(GetASOwner().GetString(1498, "ptmsgs_portalbrowsingmsgs"));
          myRow = new HTMLTableRow();
          myRow.AddInnerHTMLElement(myCell);
          myTable.AddInnerHTMLElement(myRow);
        }
      }
      catch (Exception e)
      {
        log.Error(e, "Error Displaying WebLinks.");
      }
      return result;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    public override HTMLScript DisplayJavascript()
    {
      HTMLScript script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      XPStringBuilder sb = new XPStringBuilder(400);
      try
      {
        // JF- OnDone JavaScript function
        sb = new XPStringBuilder(400);
        sb.Append("function " + JAVASCRIPT_CANCEL + "()\n");
        sb.Append("{\n");
        sb.Append("\t\t" + AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" + SiteMapFolderPlugInControl.POSTTOSELF_ACTION_CANCEL + ");\n");
        if (!GetASOwner().GetName().Equals(SiteMapAS.STR_MVC_CLASS_NAME))
        {
          sb.Append("  window.opener.document." + AFormDP.HTMLFORM_NAME + ".submit();\n");
          }
        sb.Append("  window.close();\n");
        sb.Append("}\n");
        sb.Append("\n" + "function " + JAVASCRIPT_VALIDATEANDPOST + "() { \n");
        sb.Append(" \tif ( " + JAVASCRIPT_VALIDWEBLINK + "() )\n");
        sb.Append(" \t{ \n");
        sb.Append(" \t\t" + AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" + SiteMapFolderPlugInControl.POSTTOSELF_ACTION_ADDLINK + ");\n");
        sb.Append(" \t}\n");
        sb.Append("}\n\n");
        sb.Append("\n" + "function " + JAVASCRIPT_POSTANDCLOSE + "() { \n");
        sb.Append(" \tif ( " + JAVASCRIPT_VALIDATEBEFORECLOSE + "() )\n");
        sb.Append(" \t{ \n");
        sb.Append("\t" + AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" + SiteMapFolderPlugInControl.POSTTOSELF_ACTION_SAVELINKS + ");\n");
        sb.Append("\t\t}\n");
        sb.Append("}\n\n");
        sb.Append("\n" + "function " + JAVASCRIPT_REMOVELINKS + "() { \n");
        sb.Append("\t" + AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" + SiteMapFolderPlugInControl.POSTTOSELF_ACTION_REMOVELINKS + ");\n");
        sb.Append("}\n\n");
        String strJSUtilVersionPrefix = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSUTIL);
        sb.Append("\n" + "function " + JAVASCRIPT_VALIDATEBEFORECLOSE + "() { \n");
        sb.Append(" \tif ( " + strJSUtilVersionPrefix + "PTStringUtil.isAllWhitespace(document." + AFormDP.HTMLFORM_NAME + ".in_tx_LinkName.value) && " + strJSUtilVersionPrefix + "PTStringUtil.isAllWhitespace(document." + AFormDP.HTMLFORM_NAME + ".in_tx_LinkDesc.value) && " + strJSUtilVersionPrefix + "PTStringUtil.isAllWhitespace(document." + AFormDP.HTMLFORM_NAME + ".in_tx_LinkURL.value) )\n");
        sb.Append("\t\t{\n");
        sb.Append(" \t\treturn " + JAVASCRIPT_VALIDATEEDIT + "();\n");
        sb.Append("\t\t} else {\n");
        sb.Append(" \t\treturn (" + JAVASCRIPT_VALIDWEBLINK + "() && " + JAVASCRIPT_VALIDATEEDIT + "());\n");
        sb.Append("\t\t}\n");
        sb.Append("}\n\n");
        sb.Append("function " + JAVASCRIPT_VALIDWEBLINK + "()\n");
        sb.Append("{\n");
        sb.Append(" \tif ( " + strJSUtilVersionPrefix + "PTStringUtil.isAllWhitespace(document." + AFormDP.HTMLFORM_NAME + ".in_tx_LinkName.value) )\n");
        sb.Append("\t\t{\n");
        sb.Append(" \t\talert(\'" + GetASOwner().GetString(921, "ptmsgs_portalbrowsingmsgs") + "\');\n");
        sb.Append(" \t\treturn false;\n");
        sb.Append("\t\t}\n");
        sb.Append(" \tif ( !" + strJSUtilVersionPrefix + "PTStringUtil.isValidHTTPString(document." + AFormDP.HTMLFORM_NAME + ".in_tx_LinkURL.value) )\n");
        sb.Append("\t\t{\n");
        sb.Append(" \t\talert(\'" + GetASOwner().GetString(923, "ptmsgs_portalbrowsingmsgs") + "\');\n");
        sb.Append(" \t\treturn false;\n");
        sb.Append("\t\t}\n");
        sb.Append(" \treturn true;\n");
        sb.Append("}\n\n");
        script.AddInnerHTMLString(sb.ToString());
        sb = new XPStringBuilder(100);
        // JavaScript to check all the rows
        sb.Append("function " + JAVASCRIPT_CHECKALL + "()\n");
        sb.Append("{ \n");
        sb.Append("  var frm = document." + AFormDP.HTMLFORM_NAME + "; \n");
        sb.Append("  var len = frm.elements.length; \n");
        sb.Append("  var nValue = frm." + HTMLCHECKBOX_CHECKALL + ".checked; \n");
        sb.Append("  var i = 0; \n");
        sb.Append("  \n");
        sb.Append("  for(i=0 ; i<len ; i++) \n");
        sb.Append("  { \n");
        sb.Append("    if (frm.elements[i].name == \'" + HTMLCHECKBOX_WEBLINK + "\') \n");
        sb.Append("    { \n");
        sb.Append("       frm.elements[i].checked = nValue;\n");
        sb.Append("    } \n");
        sb.Append("  } \n");
        sb.Append("}\n");
        script.AddInnerHTMLString(sb.ToString());
        sb = new XPStringBuilder(300);
        //	sb.Append("function " + JAVASCRIPT_EDITROW + "(intID, bvalidateName)\n");
        sb.Append("function " + JAVASCRIPT_EDITROW + "(intID)\n");
        sb.Append("{ \n");
        //	sb.Append("  if (bvalidateName)\n");
        //	sb.Append("  if (true)\n");
        //	sb.Append("	 { \n");
        sb.Append("\t    if(!" + JAVASCRIPT_VALIDATEEDIT + "())\n");
        sb.Append("\t \t{ \n");
        sb.Append("\t \t    return false;\n");
        sb.Append("\t \t} \n");
        //	sb.Append("	 } \n");	
        sb.Append("  var frm = document." + AFormDP.HTMLFORM_NAME + "; \n");
        sb.Append("  if (frm.elements[\'" + HTMLCHECKBOX_WEBLINK + "\'].type == \'checkbox\')\n");
        sb.Append("\t { \n");
        sb.Append("      frm.elements[\'" + HTMLCHECKBOX_WEBLINK + "\'].checked=true;\n");
        sb.Append("      " + AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" + SiteMapFolderPlugInControl.POSTTOSELF_ACTION_EDITLINK + ");\n");
        sb.Append("\t } \n");
        sb.Append("\t else\n");
        sb.Append("\t { \n");
        sb.Append("\t \tfor( var i = 0; i < frm.elements[\'" + HTMLCHECKBOX_WEBLINK + "\'].length; i++) \n");
        sb.Append("\t \t{ \n");
        sb.Append("\t \t\tif(frm.elements[\'" + HTMLCHECKBOX_WEBLINK + "\'][i].value == intID) \n");
        sb.Append("\t \t\t{ \n");
        sb.Append("      \t\tfrm.elements[\'" + HTMLCHECKBOX_WEBLINK + "\'][i].checked=true;\n");
        sb.Append("\t \t\t} \n");
        sb.Append("\t \t\telse \n");
        sb.Append("\t \t\t{ \n");
        sb.Append("      \t\tfrm.elements[\'" + HTMLCHECKBOX_WEBLINK + "\'][i].checked=false;\n");
        sb.Append("\t \t\t} \n");
        sb.Append("\t \t} \n");
        sb.Append("      \t" + AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" + SiteMapFolderPlugInControl.POSTTOSELF_ACTION_EDITLINK + ");\n");
        sb.Append("\t } \n");
        sb.Append("}\n");
        script.AddInnerHTMLString(sb.ToString());
        sb = new XPStringBuilder(150);
        sb.Append("function " + JAVASCRIPT_VALIDATEEDIT + "()\n");
        sb.Append("{ \n");
        sb.Append(" \tif (document." + AFormDP.HTMLFORM_NAME + "." + HTMLINPUT_EDITLINKNAME + "&&" + strJSUtilVersionPrefix + "PTStringUtil.isAllWhitespace(document." + AFormDP.HTMLFORM_NAME + "." + HTMLINPUT_EDITLINKNAME + ".value))\n");
        sb.Append("\t \t{ \n");
        sb.Append(" \t\talert(\'" + GetASOwner().GetString(921, "ptmsgs_portalbrowsingmsgs") + "\');\n");
        sb.Append("\t \t\treturn false; \n");
        sb.Append("\t \t} \n");
        sb.Append(" \tif (document." + AFormDP.HTMLFORM_NAME + "." + HTMLINPUT_EDITLINKURL + "&&" + "!" + strJSUtilVersionPrefix + "PTStringUtil.isValidHTTPString(document." + AFormDP.HTMLFORM_NAME + "." + HTMLINPUT_EDITLINKURL + ".value))\n");
        sb.Append("\t \t{ \n");
        sb.Append(" \t\talert(\'" + GetASOwner().GetString(923, "ptmsgs_portalbrowsingmsgs") + "\');\n");
        sb.Append("\t \t\treturn false; \n");
        sb.Append("\t \t} \n");
        sb.Append("\t \treturn true; \n");
        sb.Append("}\n");
        script.AddInnerHTMLString(sb.ToString());
      }
      catch (HTMLException he)
      {
        log.Error(he, he.Message);
      }
      return script;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
  }
}