using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.gateway
{
  /// <author>daniilk</author>
  public class GatewayModel : IGatewayModel,IGatewayModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_GATEWAY),
     "com.plumtree.portalpages.browsing.gateway.GatewayModel");
    public const String STR_MVC_CLASS_NAME = "Gateway";
    
    /// <deprecated>Use STR_MVC_CLASS_NAME instead *</deprecated>
    public const String strModelName = "Gateway";
    private const int HOSTED_MODE_TAG_VALUE_NOT_SET = 0;
    private const int HOSTED_MODE_TAG_VALUE_HOSTED = 1;
    private const int HOSTED_MODE_TAG_VALUE_PLAIN = -1;
    private AActivitySpace m_asasParent;
    private IPTContent m_ptContent = null;
    private IPTSession m_ptSession = null;
    private String m_strReturnURIBody = null;
    private String m_strReturnURIHeaders = null;
    private bool m_bError = false;
    private bool m_bReturnURI = false;
    private int m_iReturnURIStatus = 200;
    private String m_strUserURL = "";
    private bool m_bControlCalled = false;
    private int m_iCommunityID = 0;
    private String m_strUIMode = GatewayHelpers.UI_STD;
    private HTMLElement m_displayElement = null;
    // private boolean m_bHostedModePTTagPresent = false;
    private int m_iHostedModeTagValue = HOSTED_MODE_TAG_VALUE_NOT_SET;
    private String m_strTitle = null;
    private String m_strSubtitle = null;
    private int m_iRefreshMode = PlumtreeExtensibility.PT_REFRESH_NONE;
    private int m_iErrorCode = 0;
    private String m_strErrorMessage = null;
    private String m_strErrorHTMLComment = null;
    private String m_strFileName = null;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()"></seealso>
    public virtual void CleanupTempData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()"></seealso>
    public virtual void CleanupAllData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new GatewayModel();
    }
    
    /// <summary>
    /// This method returns the body of the response from the remote server as an array of bytes.
    /// </summary>
    public virtual byte[] GetBinaryBody()
    {
      if (m_bReturnURI)
      {
        if (null != m_strReturnURIBody)
        {
          log.Debug("Hitting return-uri with body [" + m_strReturnURIBody + "]");
          return XPStringUtility.GetBytes(m_strReturnURIBody);
        } else
        {
          return null;
        }
      }
      try
      {
        return (byte[]) m_ptContent.GetBinaryBody();
      }
      catch (Exception e)
      {
        return null;
      }
    }
    
    /// <summary>
    /// This method returns the value of the content-type header from the HTTP response
    /// that came from the remote server
    /// </summary>
    public virtual String GetContentType()
    {
      try
      {
        return m_ptContent.GetMIMEType();
      }
      catch (Exception e)
      {
        return null;
      }
    }
    
    /// <returns></returns>
    public virtual String GetHeaders()
    {
      if (m_bReturnURI)
      {
        return m_strReturnURIHeaders;
      }
      try
      {
        String headers = m_ptContent.GetHeaders();
        if (m_strFileName != null && m_strFileName.Length > 0)
        {
          // DCA - add filename to header
          headers+="Content-Disposition: attachment; filename=" + m_strFileName + "\r\n";
          }
        return headers;
      }
      catch (Exception e)
      {
        return null;
      }
    }
    /*
     * Implementation of IGatewayModelRO interface methods.
     */
     
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return strModelName;
    }
    
    /// <returns></returns>
    public virtual IPTSession GetSession()
    {
      return m_ptSession;
    }
    
    /// <returns></returns>
    public virtual int GetStatus()
    {
      if (m_bReturnURI)
      {
        return m_iReturnURIStatus;
      }
      try
      {
        return m_ptContent.GetStatusCode();
      }
      catch (Exception e)
      {
        return 200;
      }
    }
    
    /// <summary>
    /// This method returns the body of the response from the remote server as string.
    /// </summary>
    public virtual Object[][] GetTextBody()
    {
      try
      {
        return (Object[][]) m_ptContent.GetBody();
      }
      catch (Exception e)
      {
        return null;
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)"></seealso>
    public virtual void Init(AActivitySpace parent)
    {
      m_asasParent = parent;
      m_ptSession = (IPTSession) parent.GetUserSession();
      m_ptContent = null;
      m_asasParent.SetIsInPlaceRefreshEnabled(true);
    }
    
    /// <returns></returns>
    public virtual bool IsErrorPage()
    {
      return m_bError;
    }
    
    /// <returns></returns>
    public virtual bool IsReturnPage()
    {
      return m_bReturnURI;
    }
    
    /// <returns></returns>
    public virtual bool IsText()
    {
      try
      {
        return m_ptContent.GetIsText();
      }
      catch (Exception e)
      {
        return false;
      }
    }
    public virtual void SetError(int iErrorCode, String strErrorMessage, String strErrorHTMLComment)
    {
      m_bError = true;
      m_iErrorCode = iErrorCode;
      m_strErrorMessage = strErrorMessage;
      m_strErrorHTMLComment = strErrorHTMLComment;
    }
    public virtual int GetErrorCode()
    {
      return m_iErrorCode;
    }
    public virtual String GetErrorMessage()
    {
      return m_strErrorMessage;
    }
    public virtual String GetErrorHTMLComment()
    {
      return m_strErrorHTMLComment;
    }
    
    /// <summary>
    /// This method is used to set gateway content on the model.
    /// </summary>
    public virtual void SetIPTContent(IPTContent ptContent)
    {
      m_ptContent = ptContent;
      m_bReturnURI = false;
      m_bError = false;
    }
    /*
     * Implementation of the methods defined on the IGatewayModel interface.
     */
     
    public virtual void SetReturnURIContent(int iStatusCode, String strHeaders,
     String strBody)
    {
      m_iReturnURIStatus = iStatusCode;
      m_strReturnURIHeaders = strHeaders;
      m_strReturnURIBody = strBody;
      m_bReturnURI = true;
      m_bError = false;
    }
    public virtual void SetUserURL(String strUserURL)
    {
      m_strUserURL = strUserURL;
    }
    public virtual String GetUserURL()
    {
      return m_strUserURL;
    }
    public virtual bool WasControlCalled()
    {
      return m_bControlCalled;
    }
    public virtual void SetControlCalled()
    {
      m_bControlCalled = true;
    }
    public virtual void SetCommunityID(int iCommunityID)
    {
      m_iCommunityID = iCommunityID;
    }
    public virtual int GetCommunityID()
    {
      return m_iCommunityID;
    }
    public virtual void SetUIMode(String strUIMode)
    {
      m_strUIMode = strUIMode;
    }
    public virtual String GetUIMode()
    {
      return m_strUIMode;
    }
    public virtual bool IsDeviceUIMode()
    {
      return (GatewayHelpers.UI_PDA.CompareTo(m_strUIMode) == 0 || GatewayHelpers.UI_WML.CompareTo(m_strUIMode) == 0);
      }
    public virtual int GetRefreshMode()
    {
      return m_iRefreshMode;
    }
    public virtual void SetRefreshMode(int iRefreshMode)
    {
      m_iRefreshMode = iRefreshMode;
    }
    public virtual void SetDisplayElement(HTMLElement elemDisplayString)
    {
      m_displayElement = elemDisplayString;
    }
    public virtual HTMLElement GetDisplayElement()
    {
      return m_displayElement;
    }
    public virtual bool GetIsHostedMode()
    {
      if (m_iHostedModeTagValue == HOSTED_MODE_TAG_VALUE_HOSTED)
      {
        return true;
      } else
      if (m_iHostedModeTagValue == HOSTED_MODE_TAG_VALUE_PLAIN)
      {
        return false;
      } else
      // ( m_iHostedModeTagValue == HOSTED_MODE_TAG_VALUE_NOT_SET ) 
      {
        return m_ptContent.IsHostedDisplayMode();
      }
    }
    // return m_bHostedModePTTagPresent || m_ptContent.IsHostedDisplayMode();
    public virtual String GetTitle()
    {
      return m_strTitle;
    }
    public virtual String GetSubtitle()
    {
      return m_strSubtitle;
    }
    public virtual void SetIsHostedModePTTagPresent(bool _bHostedMode)
    {
      m_iHostedModeTagValue = (_bHostedMode ? HOSTED_MODE_TAG_VALUE_HOSTED : HOSTED_MODE_TAG_VALUE_PLAIN);
      }
    public virtual void SetDisplayMode(int iMode)
    {
      m_iHostedModeTagValue = iMode;
    }
    public virtual void SetTitle(String strTitle)
    {
      m_strTitle = strTitle;
    }
    public virtual void SetSubtitle(String strSubtitle)
    {
      m_strSubtitle = strSubtitle;
    }
    public virtual void SetFileName(String strFileName)
    {
      m_strFileName = strFileName;
    }
  }
}