using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.portalpages.browsing.myportal.mypages;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.login;
namespace com.plumtree.portalpages.browsing.login
{
  /// <author>
  /// BenG
  /// This control executes immediately when the portal is hit.  This control
  /// is the default control for the Login activity space.  This control
  /// gets cookies sent, and attempts to log the user into the portal if the
  /// correct cookies are obtained.
  /// </author>
  public class DefaultLoginControl : IHTTPControl,ILoginControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.login.DefaultLoginControl");
    
    /// <summary>
    /// object name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "DefaultLoginControl";
    
    /// <summary>
    /// these need to be nulled out after execution 
    /// </summary>
    private IXPRequest m_xpRequest = null;
    private IWebData m_WebData = null;
    
    /// <summary>
    /// MVC Objects 
    /// </summary>
    private AActivitySpace m_asOwner;
    private LoginModel m_asModel;
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IControl#execute()"></seealso>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      log.Debug("CheckActionSecurityAndExecute()");
      // Check value of access style cookie and set it on the model.
      LoginControlHelpers.CheckAccessStyleCookie(m_xpRequest, this.m_asModel);
      bool bJustRedirect = false;
      String[] sValues = (String[]) arguments.GetElement(LoginHelper.PARAM_DONT_LOGIN_JUST_REDIRECT);
      if (sValues != null)
      {
        bJustRedirect = true;
      }
      IPTSession mySession = null;
      Redirect myRedirect = null;
      if (bJustRedirect)
      {
        mySession = (IPTSession) this.m_asOwner.GetUserSession();
        m_asModel.SetSession(mySession);
        myRedirect = GetRedirect(mySession);
      } else
      if (this.m_asModel.GetDoAttemptLogin())
      {
        myRedirect = m_asModel.attemptLogin(m_xpRequest, m_WebData);
        // If we have a new session, login was successful
        if (m_asModel.GetSession() != null)
        {
          log.Debug("Login Succeeded!");
          if (myRedirect != null)
          {
            myRedirect = GetRedirect(mySession);
          }
        }
      }
      // null out XPRequest & IWebData
      m_xpRequest = null;
      m_WebData = null;
      return myRedirect;
    }
    /////////////
    // IMVCObject
    /////////////
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new DefaultLoginControl();
    }
    /////////////////////////////
    // Methods from ILoginControl
    /////////////////////////////
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.ILoginControl#DoGetSession()"></seealso>
    public virtual bool DoGetSession()
    {
      // The interpreter should ask us for the session.
      return true;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IControl#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.ILoginControl#GetSession()"></seealso>
    public virtual Object GetSession()
    {
      IPTSession session = m_asModel.GetSession();
      if (session == null)
      {
        //	if we didn't get a new IPTSession, then
        //	we return the current session.  This prevents
        //	the guest user from logging out and logging back in.
        session = (IPTSession) this.m_asOwner.GetUserSession();
      }
      return session;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IControl#Init(IModelRW)"></seealso>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      m_asModel = (LoginModel) model;
      m_asOwner = space;
    }
    
    /// <param name="r"></param>
    /// <param name="webdata"></param>
    public virtual void SetHTTPItems(IXPRequest r, IWebData webdata)
    {
      m_xpRequest = r;
      m_WebData = webdata;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.ILoginControl#SetRedirectForUseAfterLogin(Redirect)"></seealso>
    public virtual void SetRedirectForUseAfterLogin(Redirect r)
    {
      this.m_asModel.SetRedirectAfterLogin(r);
    }
    
    /// <param name="mySession"></param>
    /// <returns></returns>
    private Redirect GetRedirect(IPTSession mySession)
    {
      // Get redirect if user was trying to access a specific page
      Redirect myRedirect = this.m_asModel.GetRedirectAfterLogin();
      // bjf -- special case for if the user was trying to hit the login page originally,
      // but they weren't logged in, so the portal took them to the login page, the user
      // logged in, and then the portal tries to redirect them to the page they were
      // originally trying to hit -- which is the login page!  pretty circular, but the point
      // is that, once a user has logged in, we do not ever want to immediately redirect
      // them to the login page.  So if the redirect is set to LoginAS, just clear it,
      // and then code below will get the SubPortalRedirectAfterLogin.
      // WilliamA -- the loop may involve external URL (302 redirect), e.g., 
      // SSO in security mode 1, so want to check the space name from the URL QS as well 
      // (in 6.0 common opener style: open=space&name=Login)
      if (myRedirect != null)
      {
        if (myRedirect.GetIsExternalURL())
        {
          String externalURL = myRedirect.GetExternalURL();
          if (null != externalURL)
          {
            if (-1 != XPStringUtility.IndexOf(externalURL, LoginControl.LOGIN_SPACE_COMMONOPENER_STYLE))
            {
              // This is a loop through external URL HTTP redirect, break it.					
              myRedirect = null;
              log.Debug("LoginControl:GetRedirect -- redirect is set to LoginAS (through a 302 redirect), but the user has already logged in.  Clear this, and redirect them to the SubportalRedirectAfterLogin instead.");
              }
          }
        } else
        {
          if (LoginAS.STR_MVC_CLASS_NAME.Equals(myRedirect.GetSpaceName()))
          {
            myRedirect = null;
            log.Debug("LoginControl:GetRedirect -- redirect is set to LoginAS, but the user has already logged in.  Clear this, and redirect them to the SubportalRedirectAfterLogin instead.");
            }
        }
      }
      if (null == myRedirect)
      {
        // Otherwise get sub-portal redirect
        // Put this in when subportals work.
        myRedirect = m_asModel.GetSubPortalRedirectAfterLogin();
        if (myRedirect == null)
        {
          // If that fails, just redirect to mypage.
          // redirect to myPage
          myRedirect = new Redirect();
          myRedirect.SetLinkCreateNewSpace(MyPageAS.STR_MVC_CLASS_NAME, m_asOwner);
          myRedirect.SetControl(SetPageControl.STR_MVC_CLASS_NAME);
          myRedirect.AddControlArgument(SetPageControl.STR_PAGE_ID, SetPageControl.N_HOME_PAGE_ID);
          }
      }
      // Make this use a true 302 redirect
      myRedirect.SetIsHTTPRedirect(true);
      return myRedirect;
    }
  }
}