using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.portalpages.browsing.myportal.mypages;
using com.plumtree.portalpages.common.login;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.uiinfrastructure.opener;
using com.plumtree.uiinfrastructure.opener.types.classtypes;
namespace com.plumtree.portalpages.browsing.login
{
  /// <summary>
  /// This class handles the control logic for login.
  /// </summary>
  /// <author>BenG</author>
  public class LoginControl : ILoginControl,IHTTPControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.login.LoginControl");
    
    /// <summary>
    /// object name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "Login";
    
    /// <summary>
    /// Query String parameters 
    /// </summary>
    public const String PARAM_CREATE_ACCT = "createacct";
    // open=space&name=Login
    
    /// <summary>
    /// Query string parameters to the Login space (Common Opener style) 
    /// </summary>
    public static readonly String LOGIN_SPACE_COMMONOPENER_STYLE = IOpenerConstants.QS_OPEN + LinkConstants.EQUALS + ASConstants.SPACENAME + LinkConstants.AMPERSAND + UIActivitySpaces.QS_ACTIVITY_SPACE_NAME + LinkConstants.EQUALS + LoginHandlers.LOGINSPACE;
    
    /// <summary>
    /// these need to be nulled out after execution. 
    /// </summary>
    private IWebData m_WebData = null;
    private IXPRequest m_xpRequest = null;
    
    /// <summary>
    /// MVC Objects 
    /// </summary>
    private AActivitySpace m_asOwner;
    private LoginModel m_asModel;
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IControl#execute()"></seealso>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      log.Debug("CheckActionSecurityAndExecute()");
      String[] sValues;
      // User name
      sValues = (String[]) arguments.GetElement(LoginHTML.PARAM_USERNAME);
      if (sValues != null)
      {
        log.Debug("userName: " + sValues[0]);
        m_asModel.SetUserName(sValues[0]);
      }
      // User Pass
      sValues = (String[]) arguments.GetElement(LoginHTML.PARAM_USERPASS);
      if (sValues != null)
      {
        m_asModel.SetUserPass(sValues[0]);
      }
      Redirect myRedirect = null;
      // Remember password
      sValues = (String[]) arguments.GetElement(LoginHTML.PARAM_REMEMBER);
      if (sValues != null)
      {
        m_asModel.SetRememberPassword(true);
      }
      // Create an account
      sValues = (String[]) arguments.GetElement(PARAM_CREATE_ACCT);
      if (sValues != null)
      {
        log.Info("create an account");
      }
      sValues = (String[]) arguments.GetElement(LoginHTML.PARAM_SHOWAUTHSOURCES);
      if (sValues != null)
      {
        String sTemp = sValues[0];
        if (sTemp != null)
        {
          if (XPConvert.ToBoolean(sTemp))
          {
            // This will be ignored if show auth sources mode is 3
            this.m_asModel.SetShowAuthSources(true);
          }
        }
      }
      // Do Login
      sValues = (String[]) arguments.GetElement(LoginHTML.PARAM_DOLOGIN);
      if (sValues != null)
      {
        log.Debug("doLogin: " + sValues[0]);
        bool bDoLogin;
        try
        {
          bDoLogin = XPConvert.ToBoolean(sValues[0]);
        }
        catch (Exception e)
        {
          log.Error(e, "Unexpected data for param DoLogin: " + sValues[0]);
          bDoLogin = false;
        }
        if (bDoLogin)
        {
          this.m_asModel.SetDoAttemptLogin(true);
        }
      }
      // Do Login 508
      sValues = (String[]) arguments.GetElement(LoginHTML.PARAM_DOLOGIN_508);
      if (sValues != null)
      {
        log.Debug("doLogin: " + sValues[0]);
        this.m_asModel.SetDoAttemptLogin(true);
      }
      // Authentication source
      sValues = (String[]) arguments.GetElement(LoginHTML.PARAM_AUTHSOURCE);
      if (sValues == null)
      {
        sValues = (String[]) arguments.GetElement(LoginHTML.PARAM_HIDDEN_AUTHSOURCE);
        }
      if (sValues != null)
      {
        log.Debug("authSource: " + sValues[0]);
        this.m_asModel.SetAuthSource(sValues[0]);
      }
      // Do Logout
      sValues = (String[]) arguments.GetElement(LoginHelper.PARAM_DOLOGOUT);
      if (sValues != null)
      {
        log.Debug("logging out!");
        // Bug 53270 - login and logout in the same cycle
        // Make sure if an internal Logout redirect is created (by OnAfterLogin)
        // We cancel any login attempt. Otherwise, after logout we re-login again
        // while displaying the login page for redirectAfterLogout.
        this.m_asModel.SetDoAttemptLogin(false);
        // Do logout
        this.m_asModel.SetDoAttemptLogout(true);
        this.m_asModel.SetRedirectAfterLogin(null);
        this.m_asModel.Logout(m_WebData);
        //PEI : LoginActions
        Redirect logoutRedirect;
        logoutRedirect = DoTasksBeforeLogout();
        if (logoutRedirect == null)
        {
          // DCA - if not redirecting anywhere, refresh to make sure the right
          // navigation scheme views are registered.
          logoutRedirect = new Redirect();
          // mj: check whether we're supposed to redirect to the default 
          // guest page when a user logs out.  if bJustRedirect is set to true,
          // we will redirect below instead of visiting the normal login page.
          if (IsRedirectOnLogoutEnabled())
          {
            // DCA - Check whether we want to use the Default Login Page or the Guest MyPage
            // (which is stored in the user's sandbox session) 
            bool bDefaultLogin = true;
            ISessionManager smSandbox = m_asOwner.GetSubSession();
            if (null != smSandbox)
            {
              String str = (String) smSandbox.GetAttribute(Authenticator.GUESTLOGIN_REDIRECTONLOGOUT);
              if (null != str)
              {
                bDefaultLogin = XPConvert.ToBoolean(str);
              }
            }
            if (bDefaultLogin)
            {
              logoutRedirect.SetLinkGetCachedSpace(m_asOwner);
            } else
            {
              logoutRedirect.SetLinkCreateNewSpace(MyPageAS.STR_MVC_CLASS_NAME,
               m_asOwner);
              logoutRedirect.SetControl(SetPageControl.STR_MVC_CLASS_NAME);
              logoutRedirect.AddControlArgument(SetPageControl.STR_PAGE_ID,
               SetPageControl.N_HOME_PAGE_ID);
            }
          } else
          {
            logoutRedirect.SetLinkGetCachedSpace(m_asOwner);
            logoutRedirect.AddControlArgument(LoginHelper.PARAM_DONT_LOGIN_JUST_REDIRECT,
             true);
          }
        }
        if (logoutRedirect != null)
        {
          myRedirect = logoutRedirect;
          m_xpRequest = null;
          m_WebData = null;
          return myRedirect;
        }
      }
      //PEI : OnLogout : END
      // Check for redirect after login URL
      Redirect r = this.m_asModel.GetRedirectAfterLogin();
      if (null == r)
      {
        // hidden input REDIRECT_AFTER_LOGIN_INPUT is used by login portlet and tag login portlet
        sValues = (String[]) arguments.GetElement(ASConstants.REDIRECT_AFTER_LOGIN_INPUT);
        if (sValues != null && sValues.Length > 0)
        {
          log.Debug("redirect after login: " + sValues[0]);
          r = new Redirect();
          r.SetLinkToExternalURL(sValues[0]);
          log.Info("Redirect after login QS param: {0}", sValues[0]);
          SetRedirectForUseAfterLogin(r);
        }
      }
      //	Here we go through the cookies and see if there's a
      //	access style cookie.  This is for the following case.  User
      //	sets access style to something, and then clicks the logout link.
      //	That user is taken to the login page, and the LoginControl executes,
      //	and must intercept this cookie to display the appropriate access
      //	style.  In the usual case, where a user just opens a browser and
      //	hits the login page, the default login control executes and
      //	intercepts cookies.
      // Check value of access style cookie and set it on the model.
      LoginControlHelpers.CheckAccessStyleCookie(m_xpRequest, this.m_asModel);
      bool bJustRedirect = false;
      sValues = (String[]) arguments.GetElement(LoginHelper.PARAM_DONT_LOGIN_JUST_REDIRECT);
      if (sValues != null)
      {
        bJustRedirect = true;
      }
      if (bJustRedirect)
      {
        myRedirect = GetRedirect((IPTSession) m_asOwner.GetUserSession());
        } else
      if (this.m_asModel.GetDoAttemptLogin())
      {
        // Log in
        myRedirect = this.m_asModel.attemptLogin(m_xpRequest, m_WebData);
        // A new session implies successful login
        if (m_asModel.GetSession() != null)
        {
          log.Debug("Login Succeeded!");
          // we want to use a new application (http) session after succesful (non-guest) login to avoid session fixation attacks
          // for bug #7824904 (oracle bug db). 
          // Set a flag to tell XPPage to use new application session on next request, we only want to do it after a succesful
          // login
          // We have to set a flag to abandon session on next request. Abandoning session in this request looses the session
          // in security mode 1 where a new session is created with secure session id cookie which is not accessible to a non
          // secure page after login redirect.
          if (!m_asModel.GetSession().GetSessionInfo().CheckCurrentUserIsGuest())
          {
            XPSession oldAppSession = ((XPRequest) m_xpRequest).GetSession();
            // only do it for java, dotnet uses a different approach by clearing session id cookie in javascript before
            // submitting login form (LoginHTML)
            if (!XPSystem.IsRunningInDotNet())
            {
              oldAppSession.SetAttribute(XPPage.USE_NEW_SESSION_FLAG, "");
              }
          }
          if (myRedirect == null)
          {
            myRedirect = GetRedirect(m_asModel.GetSession());
            //	clear this from the model
            m_asModel.SetRedirectAfterLogin(null);
          }
        }
      }
      // null out member variables so we don't leak them
      m_xpRequest = null;
      m_WebData = null;
      return myRedirect;
    }
    
    /// <summary>
    /// This method is used to determine if the guest user should
    /// be redirected to their default page or to the login page
    /// if a space is not specified.
    /// </summary>
    /// <param name="RequestData"></param>
    /// <returns>
    /// boolean true implies the guest should redirect to
    /// login if no space is specified.  Login is default.
    /// </returns>
    private bool IsRedirectOnLogoutEnabled()
    {
      // default value
      bool bResult = true;
      // then use the default setting in the x_config.xml
      ConfigVarPack vpConfig = (ConfigVarPack) m_asOwner.GetApplication().GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
      String strGuestRedirect = strGuestRedirect = vpConfig.GetConfigValueAsString(ConfigConstants.AUTHENTICATION_SECTION,
       "RedirectOnLogout", ConfigConstants.VALUE_ATTRIBUTE);
      if ((strGuestRedirect != null) && (!"".Equals(strGuestRedirect)))
      {
        if (XPConvert.ToInteger(strGuestRedirect) == 0)
        {
          bResult = false;
        }
      }
      return bResult;
    }
    /////////////
    // IMVCObject
    /////////////
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new LoginControl();
    }
    /////////////////////////////
    // Methods from ILoginControl
    /////////////////////////////
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.ILoginControl#DoGetSession()"></seealso>
    public virtual bool DoGetSession()
    {
      // The interpreter should ask us for the session.
      return true;
    }
    
    /// <summary>
    /// Call the Logout function in the Model
    /// </summary>
    /// <returns>Redirect - retrieve the correct Redirect object.</returns>
    public virtual Redirect DoTasksBeforeLogout()
    {
      return this.m_asModel.DoTasksBeforeLogout(m_xpRequest, m_WebData);
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IControl#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.ILoginControl#GetSession()"></seealso>
    public virtual Object GetSession()
    {
      IPTSession session = m_asModel.GetSession();
      if (session == null)
      {
        if (!this.m_asModel.GetDoAttemptLogout())
        {
          log.Debug("GetSession() - Session on the model is null, and the user is not logging out, recycling the session from our Activity Space.");
          //	if we didn't get a new IPTSession, then
          //	we return the current session.  This prevents
          //	the guest user from logging out and logging back in.
          session = (IPTSession) this.m_asOwner.GetUserSession();
        }
      }
      return session;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IControl#Init(IModelRW)"></seealso>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      m_asModel = (LoginModel) model;
      m_asOwner = space;
    }
    
    /// <seealso cref="IHTTPControl"></seealso>
    /// <param name="r"></param>
    /// <param name="pageData"></param>
    public virtual void SetHTTPItems(IXPRequest r, IWebData pageData)
    {
      m_xpRequest = r;
      m_WebData = pageData;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.ILoginControl#SetRedirectForUseAfterLogin(Redirect)"></seealso>
    public virtual void SetRedirectForUseAfterLogin(Redirect r)
    {
      this.m_asModel.SetRedirectAfterLogin(r);
    }
    
    /// <param name="mySession"></param>
    /// <returns></returns>
    private Redirect GetRedirect(IPTSession mySession)
    {
      // Get redirect if user was trying to access a specific page
      Redirect myRedirect = this.m_asModel.GetRedirectAfterLogin();
      // bjf -- special case for if the user was trying to hit the login page originally,
      // but they weren't logged in, so the portal took them to the login page, the user
      // logged in, and then the portal tries to redirect them to the page they were
      // originally trying to hit -- which is the login page!  pretty circular, but the point
      // is that, once a user has logged in, we do not ever want to immediately redirect
      // them to the login page.  So if the redirect is set to LoginAS, just clear it,
      // and then code below will get the SubPortalRedirectAfterLogin.
      // WilliamA -- the loop may involve external URL (302 redirect), e.g., 
      // SSO in security mode 1, so want to check the space name from the URL QS as well 
      // (in 6.0 common opener style: open=space&name=Login)
      if (myRedirect != null)
      {
        if (myRedirect.GetIsExternalURL())
        {
          String externalURL = myRedirect.GetExternalURL();
          if (null != externalURL)
          {
            if (-1 != XPStringUtility.IndexOf(externalURL, LOGIN_SPACE_COMMONOPENER_STYLE))
            {
              // This is a loop through external URL HTTP redirect, break it.					
              myRedirect = null;
              log.Debug("LoginControl:GetRedirect -- redirect is set to LoginAS (through a 302 redirect), but the user has already logged in.  Clear this, and redirect them to the SubportalRedirectAfterLogin instead.");
              }
          }
        } else
        {
          if (LoginAS.STR_MVC_CLASS_NAME.Equals(myRedirect.GetSpaceName()))
          {
            myRedirect = null;
            log.Debug("LoginControl:GetRedirect -- redirect is set to LoginAS, but the user has already logged in.  Clear this, and redirect them to the SubportalRedirectAfterLogin instead.");
            }
        }
      }
      if (null == myRedirect)
      {
        // Otherwise get sub-portal redirect
        // Put this in when subportals work.
        myRedirect = m_asModel.GetSubPortalRedirectAfterLogin();
        if (myRedirect == null)
        {
          // If that fails, just redirect to mypage.
          // redirect to myPage
          myRedirect = new Redirect();
          myRedirect.SetLinkCreateNewSpace(MyPageAS.STR_MVC_CLASS_NAME, m_asOwner);
          myRedirect.SetControl(SetPageControl.STR_MVC_CLASS_NAME);
          myRedirect.AddControlArgument(SetPageControl.STR_PAGE_ID, SetPageControl.N_HOME_PAGE_ID);
          }
      }
      // Make this use a true 302 redirect
      myRedirect.SetIsHTTPRedirect(true);
      return myRedirect;
    }
  }
}