using System;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.portalsettings
{
  /// <summary>
  /// Portal settings view management handler
  /// </summary>
  /// <author>DavidP</author>
  public class PortalSettingsView : IView
  {
    public const String STR_MVC_CLASS_NAME = "PortalSettings";
    private AActivitySpace m_asOwner;
    private IPortalSettingsModelRO m_asmModel;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new PortalSettingsView();
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.IView#Display()"></seealso>
    public virtual HTMLElement Display()
    {
      HTMLElement result = null;
      m_asmModel.InitSettings(m_asOwner);
      IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
      try
      {
        HTMLTable psTable = new HTMLTable();
        psTable.SetAlign("left");
        psTable.SetCellPadding("5");
        psTable.SetCellSpacing("0");
        psTable.SetBorder("0");
        psTable.SetWidth("100%");
        psTable.SetSummary(m_asOwner.GetString(1610, "ptmsgs_portalbrowsingmsgs"));
        HTMLTableRow headerRow = new HTMLTableRow();
        psTable.AddInnerHTMLElement(headerRow);
        HTMLTableCell headerCell = new HTMLTableCell();
        headerCell.SetAlign("left");
        headerCell.SetVAlign("top");
        headerRow.AddInnerHTMLElement(headerCell);
        headerRow.SetStyleClass(PTStyleClass.DIR_HEADER_BG);
        HTMLSpan headerSpan = new HTMLSpan();
        headerSpan.SetStyleClass(PTStyleClass.DIR_HEADER);
        headerSpan.AddInnerHTMLString(this.m_asOwner.GetString(1604, "ptmsgs_portalbrowsingmsgs"));
        headerCell.AddInnerHTMLElement(headerSpan);
        HTMLTableRow tRow = new HTMLTableRow();
        HTMLTableCell tCell = new HTMLTableCell();
        for (int i = 0;i < m_asmModel.GetNumSettingItems();i++)
        {
          String text = m_asmModel.GetSettingItemName(i);
          String desc = m_asmModel.GetSettingItemDescription(i);
          String strSettingURL = m_asmModel.GetSettingItemURL(i);
          int nActivityRight = m_asmModel.GetActivityRightRestriction(i);
          bool bHasRight = true;
          if (PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_NONE != nActivityRight)
          {
            bHasRight = ptSession.GetSessionInfo().CheckForActivityRight(nActivityRight);
            }
          if (bHasRight)
          {
            tRow = new HTMLTableRow();
            psTable.AddInnerHTMLElement(tRow);
            tCell = new HTMLTableCell();
            tRow.AddInnerHTMLElement(tCell);
            tCell.SetStyleClass(PTStyleClass.MENU_TEXT);
            HTMLAnchor tAnchor = new HTMLAnchor(strSettingURL);
            tCell.AddInnerHTMLElement(tAnchor);
            tAnchor.AddInnerHTMLString(text);
            tAnchor.SetTitle(text);
            tAnchor.SetName(text);
            // In 508 the Change Password link gets an access key treatment
            if (m_asOwner.GetAccessStyle() == AccessStyles.SECTION_508 && text.Equals(m_asOwner.GetString(1305,
             ResourceConstants.PORTALBROWSINGMSGS)))
            {
              tAnchor.SetTitle(m_asOwner.GetString(20, ResourceConstants.PORTAL508QUICKKEYMSGS));
              tAnchor.SetAccessKey(m_asOwner.GetString(7, ResourceConstants.PORTAL508QUICKKEYMSGS));
              }
            tCell.AddInnerHTMLString("<br>" + desc);
            AddBreakRow(psTable);
          }
        }
        result = psTable;
      }
      catch (HTMLException e)
      {
      }
      return result;
    }
    
    /// <summary>
    /// This method returns the HTMLScript that encompasses the Javascript on the page.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    /// <returns>HTMLScript</returns>
    public virtual HTMLScript DisplayJavascript()
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.IView#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// PortalSettings
    /// </summary>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.IView#Init(IModelRO,AActivitySpace)"></seealso>
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      m_asmModel = (IPortalSettingsModelRO) model;
      m_asOwner = parent;
    }
    
    /// <param name="table"></param>
    /// <throws>HTMLException </throws>
    protected virtual void AddBreakRow(HTMLTable table)
    {
      HTMLTableRow tRow = new HTMLTableRow();
      HTMLTableCell tCell = new HTMLTableCell();
      tRow.AddInnerHTMLElement(tCell);
      table.AddInnerHTMLElement(tRow);
      tCell.SetHeight("6");
    }
  }
}