using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.pei;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portalpages.browsing.portalsettings.password
{
  /// <author>
  /// DonH
  /// This saves your password
  /// </author>
  public class PasswordModel : EditorModel
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.portalsettings.password.PasswordModel");
    
    /// <summary>
    /// MVC object name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "ChangePassword";
    
    /// <summary>
    /// constants 
    /// </summary>
    public const int MAX_PASSWORD_LENGTH = 20;
    
    /// <summary>
    /// constants 
    /// </summary>
    public const int MIN_PASSWORD_LENGTH = 5;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new PasswordModel();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.editor.EditorModel#CheckEditorAccessAndStart(XPHashtable)"></seealso>
    public override bool CheckEditorAccessAndStart(XPHashtable _htFormData)
    {
      IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
      // Only allow password change if not guest
      if (LoginHelper.INSTANCE.GetIsGuestUser(ptSession))
      {
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         null, m_asOwner.GetString(1295, "ptmsgs_portalbrowsingmsgs"), "");
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        return false;
      }
      // Only allow password change if user is not synced to an auth source
      if (ptSession.GetUser().GetAuthenticationSourceID() != PT_INTRINSICS.PT_AUTHSOURCE_PLUMTREE)
      {
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         null, m_asOwner.GetString(1294, "ptmsgs_portalbrowsingmsgs"), "");
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        return false;
      }
      return true;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#Finish()"></seealso>
    public override void Finish()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String,XPHashtable)"></seealso>
    public override int SavePage(String _sPageName, XPHashtable _htFormData)
    {
      String[] sValues;
      // PasswordActions PEI
      // BrianH: All of the following code will load all of the available PasswordAction
      // PEIs and process through them. If they return a false string an error will 
      // appear and the saving of the password will stop.
      Object[] oa;
      Object o;
      IPasswordActions iActions;
      IApplication application = m_asOwner.GetApplication();
      oa = (Object[]) application.GetCachingManager().GetEntry("PasswordActions");
      if (oa != null)
      {
        String sErrorMsgs = "";
        for (int x = 0;x < oa.Length;x++)
        {
          o = oa[x];
          log.Debug("PasswordModel.SavePage - About to call OnChangePassword on an: " + o);
          if (o != null)
          {
            String sError;
            iActions = (IPasswordActions) o;
            sError = iActions.OnBeforeChangePassword(m_asOwner, (IPTSession) m_asOwner.GetUserSession(),
             _htFormData);
            if ((null != sError) && (sError.Length > 0))
            {
              sErrorMsgs+=sError + " ";
            }
          }
        }
        if ((null != sErrorMsgs) && (sErrorMsgs.Length > 0))
        {
          UIError myError = new UIError(UIErrorLevel.WARNING, UIErrorScope.CURRENT_USER,
           null, sErrorMsgs, sErrorMsgs);
          UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
          return RepostControl.PAGE_STATUS_INVALID;
        }
      }
      //PEI : OnChangePassword : END
      // Old password
      sValues = (String[]) _htFormData.GetElement(PasswordView.HTML_INPUT_OLD_PASSWORD);
      if (sValues == null)
      {
        log.Error("Could not find old password in form data.");
        return RepostControl.PAGE_STATUS_INVALID;
      }
      String strOldPassword = sValues[0];
      // New password
      sValues = (String[]) _htFormData.GetElement(PasswordView.HTML_INPUT_NEW_PASSWORD);
      if (sValues == null)
      {
        log.Error("Could not find new password in form data.");
        return RepostControl.PAGE_STATUS_INVALID;
      }
      String strNewPassword = sValues[0];
      // Confirm new password
      sValues = (String[]) _htFormData.GetElement(PasswordView.HTML_INPUT_CONFIRM_NEW_PASSWORD);
      if (sValues == null)
      {
        log.Error("Could not find confirm new password in form data.");
        return RepostControl.PAGE_STATUS_INVALID;
      }
      String strConfirmNewPassword = sValues[0];
      // Check if new passwords are the same
      if (!strNewPassword.Equals(strConfirmNewPassword))
      {
        log.Info("New passwords do not match.");
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         null, m_asOwner.GetString(1290, "ptmsgs_portalbrowsingmsgs"), "");
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        return RepostControl.PAGE_STATUS_INVALID;
      }
      // Check new password length
      if (strNewPassword.Length > MAX_PASSWORD_LENGTH)
      {
        log.Info("New passwords too long.");
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         null, m_asOwner.GetString(1291, "ptmsgs_portalbrowsingmsgs"), "");
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        return RepostControl.PAGE_STATUS_INVALID;
      }
      if (strNewPassword.Length < MIN_PASSWORD_LENGTH)
      {
        log.Info("New passwords too short.");
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         null, m_asOwner.GetString(1628, "ptmsgs_portalbrowsingmsgs"), "");
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        return RepostControl.PAGE_STATUS_INVALID;
      }
      IPTSession myRealIPTSession = (IPTSession) m_asOwner.GetUserSession();
      int nUserID = myRealIPTSession.GetSessionInfo().GetCurrentUserID();
      // Check old password
      try
      {
        IPTSession myTestIPTSession = PortalObjectsFactory.CreateSession();
        // null login token
        myTestIPTSession.Connect(nUserID, strOldPassword, null);
      }
      catch (Exception e)
      {
        log.Info(e, "Could not connect with old password.");
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         null, m_asOwner.GetString(1292, "ptmsgs_portalbrowsingmsgs"), "");
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        return RepostControl.PAGE_STATUS_INVALID;
      }
      //		Check old password and new password are not the same
      if (XPStringUtility.Compare(strNewPassword, strOldPassword) == 0)
      {
        log.Info("Old and new password cannot be the same.");
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         null, m_asOwner.GetString(1744, "ptmsgs_portalbrowsingmsgs"), "");
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        return RepostControl.PAGE_STATUS_INVALID;
      }
      // Change password
      // This should really be in Finish(), but Finish() does not allow
      // the same kind of error reporting as SavePage().
      try
      {
        myRealIPTSession.ChangePassword(strOldPassword, strNewPassword);
      }
      catch (Exception e)
      {
        log.Info(e, "Could not change password.");
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         null, m_asOwner.GetString(1293, "ptmsgs_portalbrowsingmsgs"), "");
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        return RepostControl.PAGE_STATUS_INVALID;
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
  }
}