using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.portalsettings.password
{
  /// <author>
  /// DonH
  /// This view displays the change password box.
  /// </author>
  public class PasswordView : EditorView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.portalsettings.password.PasswordView");
    
    /// <summary>
    /// MVC object name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "ChangePassword";
    
    /// <summary>
    /// HTML input constants 
    /// </summary>
    public const String HTML_INPUT_OLD_PASSWORD = "in_pw_OldPassword";
    public const String HTML_INPUT_NEW_PASSWORD = "in_pw_NewPassword";
    public const String HTML_INPUT_CONFIRM_NEW_PASSWORD = "in_pw_ConfirmNewPassword";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new PasswordView();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Display()"></seealso>
    public override HTMLElement Display()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLInput myInput;
      try
      {
        result.AddInnerHTMLElement(new HTMLComment("Begin: Change Password View"));
        // Add change password header
        result.AddInnerHTMLElement(base.GetHeaderView(m_asOwner.GetString(1306,
         "ptmsgs_portalbrowsingmsgs")));
        myTable = new HTMLTable();
        result.AddInnerHTMLElement(myTable);
        myTable.SetWidth("100%");
        myTable.SetBorder("0");
        myTable.SetCellPadding("5");
        myTable.SetCellSpacing("0");
        myTable.SetSummary(m_asOwner.GetString(1611, "ptmsgs_portalbrowsingmsgs"));
        // Old Password
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.LIST_ITEM_ONE_BG);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
        myCell.AddInnerHTMLString(m_asOwner.GetString(1296, "ptmsgs_portalbrowsingmsgs"));
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myInput = new HTMLInput(HTMLInputTypes.PASSWORD_TYPE, HTML_INPUT_OLD_PASSWORD,
         "");
        myCell.AddInnerHTMLElement(myInput);
        myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
        myInput.SetMaxLength(40);
        myInput.SetSize(40);
        // New Password
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.LIST_ITEM_ONE_BG);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
        myCell.AddInnerHTMLString(m_asOwner.GetString(1297, "ptmsgs_portalbrowsingmsgs"));
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myInput = new HTMLInput(HTMLInputTypes.PASSWORD_TYPE, HTML_INPUT_NEW_PASSWORD,
         "");
        myCell.AddInnerHTMLElement(myInput);
        myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
        myInput.SetMaxLength(40);
        myInput.SetSize(40);
        // Confirm New Password
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.LIST_ITEM_ONE_BG);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
        myCell.AddInnerHTMLString(m_asOwner.GetString(1298, "ptmsgs_portalbrowsingmsgs"));
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myInput = new HTMLInput(HTMLInputTypes.PASSWORD_TYPE, HTML_INPUT_CONFIRM_NEW_PASSWORD,
         "");
        myCell.AddInnerHTMLElement(myInput);
        myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
        myInput.SetMaxLength(40);
        myInput.SetSize(40);
        result.AddInnerHTMLElement(new HTMLComment("End: Change Password View"));
        }
      catch (HTMLException he)
      {
        log.Error(he, STR_MVC_CLASS_NAME + "::Display An error occured when trying to display the HTML.");
        }
      return result;
    }
    /* (non-Javadoc)
     * checks that the password is 5 chars or more
     * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
     */
     
    public override HTMLScript DisplayJavascript()
    {
      HTMLScript myScript;
      try
      {
        XPStringBuilder sb = new XPStringBuilder(300);
        myScript = new HTMLScript("text/javaScript");
        // The JavaScript below checks if the password is at least 5 chars
        sb.Append("function CheckPassword()\n");
        sb.Append("{\n");
        sb.Append("  var sizer = document." + EditorDP.HTMLFORM_NAME + "." + HTML_INPUT_NEW_PASSWORD + ".value;\n");
        sb.Append("  if (sizer.length < 5)\n");
        sb.Append("  {\n");
        // password cannot be less than 5 characters
        sb.Append("      alert(\"" + m_asOwner.GetString(1628, "ptmsgs_portalbrowsingmsgs") + "\");\n");
        sb.Append("      return false;\n");
        sb.Append("  }\n");
        sb.Append("  return true;\n");
        sb.Append("}\n");
        myScript.AddInnerHTMLString(sb.ToString());
      }
      catch (Exception e)
      {
        log.Error(e, "PasswordView::DisplayJavascript The following error occured when trying to display the JavaScript: " + e.Message);
        throw new XPException(m_asOwner.GetString(1826, "ptmsgs_portaladminmsgs"),
         e);
      }
      return myScript;
    }
  }
}