using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.search.results;
using com.plumtree.portalpages.pei;
using com.plumtree.portaluiinfrastructure.search;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portalpages.browsing.portalsettings.search
{
  /// <author>CraigS, based on JulieF template</author>
  public class SearchSettingsModel : EditorModel,ISearchSettingsModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.portalsettings.search.SearchSettingsModel");
    public const String STR_MVC_CLASS_NAME = "SearchSettings";
    public const int RETURNED_CARDS = 0;
    public const int RETURNED_FOLDERS = 1;
    public const int RETURNED_USERS = 2;
    public const int RETURNED_COMMUNITIES = 3;
    public const int RETURNED_PORTLETS = 4;
    public const int RETURNED_COLLAB = 5;
    public const int RETURNED_CONTENT = 6;
    public const int RETURNED_PAGES = 7;
    public const int RETURNED_OBJTYPE_ARRAY_SIZE = 8;
    private bool[] _returnedObjectTypes;
    private Object[][] _savedSearches;
    // Keep the list of saved searches, need to know which ones to delete
    private Object[][] _startingSavedSearches;
    private bool _restrictToDefaultLanguage;
    private int _pageSize;
    private IXPCollection _searchPEIs = null;
    private bool _loadedPEIs = false;
    private String m_strPreviousURL;
    private bool _collabIsInstalled = false;
    private String[][] _groupByOptions = null;
    private String _preferredGroupBy = null;
    
    /// <summary>
    /// Implements some actions to perform on start.
    /// </summary>
    /// <returns>boolean Is true if the current user has the rights to access this Editor.</returns>
    public override bool CheckEditorAccessAndStart(XPHashtable formData)
    {
      try
      {
        IPTSession session = (IPTSession) m_asOwner.GetUserSession();
        IPTSessionInfo sessionInfo = session.GetSessionInfo();
        _pageSize = GetIntSetting(sessionInfo, BannerStartSearchControl.SEARCHPREF_PAGESIZE,
         /*default=*/
        10);
        int limitFlag = GetIntSetting(sessionInfo, BannerStartSearchControl.SEARCHPREF_LIMITBYLOCALE,
         0);
        _restrictToDefaultLanguage = (limitFlag != 0);
        int objectTypeBitmask = GetIntSetting(sessionInfo, BannerStartSearchControl.SEARCHPREF_OBJECTTYPES,
         0);
        _returnedObjectTypes = new bool[RETURNED_OBJTYPE_ARRAY_SIZE];
        int bit = 1;
        for (int j = 0;j < RETURNED_OBJTYPE_ARRAY_SIZE;j++)
        {
          _returnedObjectTypes[j] = ((objectTypeBitmask & bit) == 0);
          bit*=2;
        }
        _savedSearches = (Object[][]) sessionInfo.LookupPreferences((String[]) null,
         SaveSearchControl.INTRINSIC_PORTLET_ID);
        IPTServerConfigSettings ptSC = (IPTServerConfigSettings) session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
         false);
        _collabIsInstalled = (0 != ptSC.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_SETTINGNAME_PLUMTREE_COLLAB_SERVER_INSTALLED));
        /* default= */
        _preferredGroupBy = GetStringSetting(sessionInfo, BannerStartSearchControl.SEARCHPREF_GROUPBY,
         null);
        // "last modified", "folder", "object type" always present
        int numPossibleGroupBy = 3;
        String[][] customGroupBy = ReorganizationViewHelper.GetCustomCategorizationFields();
        if (null != customGroupBy)
        {
          numPossibleGroupBy+=customGroupBy.Length;
        }
        _groupByOptions = new String[numPossibleGroupBy][];
        for (int j2c_index = 0;j2c_index < numPossibleGroupBy;j2c_index++)
        {
          _groupByOptions[j2c_index] = new String[2];
        }
        //!!!FIXME
        _groupByOptions[0][0] = "lastmodified";
        _groupByOptions[0][1] = m_asOwner.GetString(820, "ptmsgs_portalbrowsingmsgs");
        _groupByOptions[1][0] = SearchResultModel.GROUPBY_FOLDER;
        _groupByOptions[1][1] = m_asOwner.GetString(821, "ptmsgs_portalbrowsingmsgs");
        _groupByOptions[2][0] = SearchResultModel.GROUPBY_OBJECTTYPE;
        _groupByOptions[2][1] = m_asOwner.GetString(822, "ptmsgs_portalbrowsingmsgs");
        if (null != customGroupBy)
        {
          for (int j = 0;j < customGroupBy.Length;j++)
          {
            _groupByOptions[3 + j][0] = customGroupBy[j][0];
            _groupByOptions[3 + j][1] = customGroupBy[j][1];
          }
        }
        verifyValidityOfGroupBy();
      }
      catch (Exception e)
      {
        UIError err = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
         e, m_asOwner.GetString(1036, "ptmsgs_portalbrowsingmsgs"), m_asOwner.GetString(1036,
         "ptmsgs_portalbrowsingmsgs"));
        UIErrorHandler.RaiseError(err, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        }
      return true;
    }
    
    /// <returns></returns>
    public override Object Create()
    {
      return new SearchSettingsModel();
    }
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
      m_strPreviousURL = HTMLConstructHelpers.GetPreviousSpaceURL(m_asOwner);
      }
    
    /// <summary>
    /// Implements code to execute when the user clicks on "Finish".
    /// </summary>
    public override void Finish()
    {
      try
      {
        IPTSessionInfo sessionInfo = ((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo();
        sessionInfo.AddPreference(BannerStartSearchControl.SEARCHPREF_PAGESIZE,
         _pageSize, 0);
        if (_restrictToDefaultLanguage)
        {
          sessionInfo.AddPreference(BannerStartSearchControl.SEARCHPREF_LIMITBYLOCALE,
           1, 0);
        } else
        {
          sessionInfo.AddPreference(BannerStartSearchControl.SEARCHPREF_LIMITBYLOCALE,
           0, 0);
        }
        int objectTypeBitmask = 0;
        int bit = 1;
        for (int j = 0;j < RETURNED_OBJTYPE_ARRAY_SIZE;j++)
        {
          if (!_returnedObjectTypes[j])
          {
            objectTypeBitmask|=bit;
          }
          bit*=2;
        }
        sessionInfo.AddPreference(BannerStartSearchControl.SEARCHPREF_OBJECTTYPES,
         objectTypeBitmask, 0);
        // Save preferred group-by
        sessionInfo.RemovePreference(BannerStartSearchControl.SEARCHPREF_GROUPBY,
         0);
        if (null != _preferredGroupBy)
        {
          sessionInfo.AddPreference(BannerStartSearchControl.SEARCHPREF_GROUPBY,
           _preferredGroupBy, 0);
        }
        // Also remove the obsolete "sticky sorting" preference if it's present
        sessionInfo.RemovePreference(BannerStartSearchControl.SEARCHPREF_OBSOLETE_GROUPBY,
         0);
        // Save saved searches
        if (null != _startingSavedSearches)
        {
          String[] oldSavedSearchNames = new String[_startingSavedSearches[0].Length];
          for (int j = 0;j < _startingSavedSearches[0].Length;j++)
          {
            oldSavedSearchNames[j] = (String) (_startingSavedSearches[0][j]);
            }
          // Delete all the old saved searches, need to delete them one by one to update the prefscache
          sessionInfo.RemovePreferences(oldSavedSearchNames, SaveSearchControl.INTRINSIC_PORTLET_ID);
          }
        // Now save the new settings
        sessionInfo.AssignPreferences(_savedSearches, 
        // Don't need to clear old settings since we deleted them already.
        false, SaveSearchControl.INTRINSIC_PORTLET_ID);
        if (_savedSearches != null)
        {
          // JF- Load the new save search names in a String array so we can give them to the PEI 
          String[] aSearchNames = new String[_savedSearches[0].Length];
          for (int i = 0;i < _savedSearches[0].Length;i++)
          {
            if (null != _savedSearches[0][i])
            {
              aSearchNames[i] = (String) _savedSearches[0][i];
            } else
            {
              aSearchNames[i] = "";
            }
          }
          DoAfterModifyDeleteSSPEI(aSearchNames);
        }
        // Make sure to flush any previously cached preferences
        m_asOwner.GetSubSession().RemoveAttribute(BannerStartSearchControl.CACHED_SEARCHPREFS_KEY);
        }
      catch (Exception e)
      {
        UIError err = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
         e, m_asOwner.GetString(1037, "ptmsgs_portalbrowsingmsgs"), m_asOwner.GetString(1037,
         "ptmsgs_portalbrowsingmsgs"));
        UIErrorHandler.RaiseError(err, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        }
    }
    
    /// <returns></returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    //  ISearchSettingsModelRO implementation
    //
    public virtual int GetPageSize()
    {
      return _pageSize;
    }
    
    /// <returns></returns>
    public virtual bool[] GetReturnedObjectTypes()
    {
      return _returnedObjectTypes;
    }
    public virtual String GetPreferredGroupBy()
    {
      return _preferredGroupBy;
    }
    public virtual String[][] GetPossibleGroupBy()
    {
      return _groupByOptions;
    }
    
    /// <returns></returns>
    public virtual Object[][] GetSavedSearches()
    {
      return _savedSearches;
    }
    
    /// <returns></returns>
    public virtual String[] GetSpecificLanguagesReturned()
    {
      // Not implemented
      return null;
    }
    
    /// <returns></returns>
    public virtual bool IsRestrictedToDefaultLanguage()
    {
      return _restrictToDefaultLanguage;
    }
    
    /// <returns>true if Collaboration Server is installed on the local portal.</returns>
    public virtual bool IsCollabInstalled()
    {
      return _collabIsInstalled;
    }
    
    /// <param name="pageName"></param>
    /// <param name="formData"></param>
    /// <returns></returns>
    public override int SavePage(String pageName, XPHashtable formData)
    {
      // Clear any previous errors from this space
      {
        IXPEnumerator errors = UIErrorHandler.GetErrors(m_asOwner.GetSubSession(),
         m_asOwner.GetApplication(), m_asOwner, true, false);
        while (errors.MoveNext())
        {
          UIError pte = (UIError) errors.GetCurrent();
          UIErrorHandler.ClearError(pte, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
          }
      }
      if (pageName.Equals(SearchSettingsDP.STR_MVC_CLASS_NAME))
      {
        // Language settings
        String limitByLocale = GetFormElement(formData, SearchSettingsView.HTMLINPUT_LIMITBYLOCALE);
        _restrictToDefaultLanguage = (limitByLocale.Equals(SearchSettingsView.HTMLINPUT_LIMITBYLOCALE_YES));
        // Number of results per page
        String pageSize = GetFormElement(formData, SearchSettingsView.HTMLINPUT_PAGESIZE);
        if (pageSize.Equals("10"))
        {
          _pageSize = 10;
        } else
        if (pageSize.Equals("20"))
        {
          _pageSize = 20;
        } else
        if (pageSize.Equals("50"))
        {
          _pageSize = 50;
        } else
        if (pageSize.Equals("100"))
        {
          _pageSize = 100;
        } else
        {
          // Don't allow a user to hack in an extremely high setting
          log.Error("invalid setting for pagesize: " + pageSize);
          return RepostControl.PAGE_STATUS_INVALID;
        }
        // Preferred group-by
        _preferredGroupBy = GetFormElement(formData, SearchSettingsView.HTMLINPUT_GROUPBY);
        verifyValidityOfGroupBy();
        // Returned object types
        _returnedObjectTypes[RETURNED_CARDS] = formData.ContainsKey(SearchSettingsView.HTMLINPUT_RET_CARDS);
        _returnedObjectTypes[RETURNED_FOLDERS] = formData.ContainsKey(SearchSettingsView.HTMLINPUT_RET_FOLDERS);
        _returnedObjectTypes[RETURNED_USERS] = formData.ContainsKey(SearchSettingsView.HTMLINPUT_RET_USERS);
        _returnedObjectTypes[RETURNED_COMMUNITIES] = formData.ContainsKey(SearchSettingsView.HTMLINPUT_RET_COMMUNITIES);
        _returnedObjectTypes[RETURNED_PORTLETS] = formData.ContainsKey(SearchSettingsView.HTMLINPUT_RET_PORTLETS);
        _returnedObjectTypes[RETURNED_COLLAB] = formData.ContainsKey(SearchSettingsView.HTMLINPUT_RET_COLLAB);
        _returnedObjectTypes[RETURNED_CONTENT] = formData.ContainsKey(SearchSettingsView.HTMLINPUT_RET_CONTENT);
        _returnedObjectTypes[RETURNED_PAGES] = formData.ContainsKey(SearchSettingsView.HTMLINPUT_RET_PAGES);
        // Check that at least one type was selected
        bool choseAtLeastOne = false;
        for (int j = 0;j < RETURNED_OBJTYPE_ARRAY_SIZE;j++)
        {
          if (_returnedObjectTypes[j])
          {
            choseAtLeastOne = true;
            break;
          }
        }
        if (!choseAtLeastOne)
        {
          UIError err = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
           null, m_asOwner.GetString(1035, "ptmsgs_portalbrowsingmsgs"), m_asOwner.GetString(1035,
           "ptmsgs_portalbrowsingmsgs"));
          UIErrorHandler.RaiseError(err, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
          return RepostControl.PAGE_STATUS_INVALID;
        }
        // If Collab is not installed, at this point there is no way that the user could
        // have selected Collab on the page, since the Collab checkbox is suppressed.
        // So if we just saved the page at this point, the user's Collab preference would
        // be turned off.  So if an admin later installs Collab, the user will not get
        // any Collab search results unless he or she happens to come back to the Search
        // Preferences page and turns the preference back on.  This is not good..
        // Instead, always turn the preference ON if Collab is not installed.  This
        // is harmless (since it's ignored by PTSearch as long as Collab is not present)
        // and makes for a better default if the admin installs Collab later.
        if (!IsCollabInstalled())
        {
          _returnedObjectTypes[RETURNED_COLLAB] = true;
        }
        // Update saved searches list
        if ((null != _savedSearches) && (null != _savedSearches[0]) && (0 < _savedSearches[0].Length))
        {
          int ssCount = _savedSearches[0].Length;
          // Count the number of deleted searches
          for (int j = 0;j < _savedSearches[0].Length;j++)
          {
            if (formData.ContainsKey(SearchSettingsView.HTMLINPUT_DELETE_SS + j))
            {
              SavedSearchData ssd = new SavedSearchData((String) _savedSearches[0][j],
               (String) _savedSearches[1][j]);
              DoBeforeDeleteSSPEI(ssd);
              // Delete button was checked
              ssCount--;
            }
          }
          // If no saved searches left
          if (0 == ssCount)
          {
            // wipe out the array
            _startingSavedSearches = _savedSearches;
            _savedSearches = null;
          } else
          {
            // Allocate a replacement array
            Object[][] newSavedSearches = new Object[2][];
            for (int j2c_index = 0;j2c_index < 2;j2c_index++)
            {
              newSavedSearches[j2c_index] = new Object[ssCount];
            }
            int ssIdx = 0;
            bool duplicates = false;
            bool emptyStr = false;
            for (int j = 0;j < _savedSearches[0].Length;j++)
            {
              if (formData.ContainsKey(SearchSettingsView.HTMLINPUT_DELETE_SS + j))
              {
                // Delete button was checked
                continue;
              }
              // Use name from checkbox, which user might have changed
              String newName = ((String[]) (formData.GetElement(SearchSettingsView.HTMLINPUT_RENAME_SS + j)))[0];
              if (null == newName)
              {
                emptyStr = true;
                newName = "";
              }
              newName = XPStringUtility.Trim(newName);
              if (0 == newName.Length)
              {
                emptyStr = true;
              }
              newSavedSearches[0][ssIdx] = newName;
              // Check whether this name is a duplicate
              for (int k = 0;k < ssIdx;k++)
              {
                if (newName.Equals(((String) newSavedSearches[0][k])))
                {
                  duplicates = true;
                  // preserve old name
                  newSavedSearches[0][ssIdx] = _savedSearches[0][j];
                }
              }
              // Use same URL
              newSavedSearches[1][ssIdx] = _savedSearches[1][j];
              ssIdx++;
            }
            // Keep the list of searches that we started with.
            _startingSavedSearches = _savedSearches;
            // And replace the array
            _savedSearches = newSavedSearches;
            if (emptyStr)
            {
              UIError err = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
               m_asOwner.GetString(1861, "ptmsgs_portalbrowsingmsgs"), m_asOwner.GetString(1861,
               "ptmsgs_portalbrowsingmsgs"));
              UIErrorHandler.RaiseError(err, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
              return RepostControl.PAGE_STATUS_INVALID;
            }
            if (duplicates)
            {
              UIError err = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
               m_asOwner.GetString(1038, "ptmsgs_portalbrowsingmsgs"), m_asOwner.GetString(1038,
               "ptmsgs_portalbrowsingmsgs"));
              UIErrorHandler.RaiseError(err, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
              return RepostControl.PAGE_STATUS_INVALID;
            }
          }
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    public override Redirect GetRedirectAfterFinish()
    {
      Redirect myRedirect;
      if (m_asOwner.GetIsAccessStyleStandard())
      {
        myRedirect = base.GetRedirectAfterFinish();
      } else
      {
        myRedirect = new Redirect();
        myRedirect.SetIsErrorRedirect(false);
        myRedirect.SetLinkToExternalURL(m_strPreviousURL);
      }
      return myRedirect;
    }
    
    /// <param name="form"></param>
    /// <param name="name"></param>
    /// <returns></returns>
    private String GetFormElement(XPHashtable form, String name)
    {
      String[] data = (String[]) form.GetElement(name);
      if ((null == data) || (0 == data.Length))
      {
        log.Error("could not find element " + name + " in form");
        return "";
      }
      return data[0];
    }
    
    /// <summary>
    /// Helper method to verify that the preferred group-by is valid
    /// </summary>
    private void verifyValidityOfGroupBy()
    {
      if (null != _preferredGroupBy)
      {
        bool preferredGroupByIsValid = false;
        for (int j = 0;j < _groupByOptions.Length;j++)
        {
          if (_preferredGroupBy.Equals(_groupByOptions[j][0]))
          {
            preferredGroupByIsValid = true;
            break;
          }
        }
        if (!preferredGroupByIsValid)
        {
          // automatically default it back to "relevance"
          _preferredGroupBy = null;
        }
      }
    }
    
    /// <summary>
    /// Helper method to get an integer setting out of the database
    /// </summary>
    private int GetIntSetting(IPTSessionInfo sessionInfo, String settingName,
     int def)
    {
      int r = def;
      Object[][] setting = (Object[][]) sessionInfo.LookupPreference(settingName,
       0);
      if (null != setting)
      {
        Object o = setting[1][0];
        if (null != o)
        {
          r = XPConvert.ToInteger(o);
        }
      }
      return r;
    }
    
    /// <summary>
    /// Helper method to get a string setting out of the database
    /// </summary>
    private String GetStringSetting(IPTSessionInfo sessionInfo, String settingName,
     String def)
    {
      String r = def;
      Object[][] setting = (Object[][]) sessionInfo.LookupPreference(settingName,
       0);
      if (null != setting)
      {
        Object o = setting[1][0];
        if (null != o)
        {
          r = XPConvert.ToString(o);
        }
      }
      return r;
    }
    protected virtual void DoBeforeDeleteSSPEI(SavedSearchData ssd)
    {
      LoadSearchPEIs();
      // Call out to each PEI in turn
      if (null != _searchPEIs)
      {
        IXPEnumerator e = _searchPEIs.GetEnumerator();
        while (e.MoveNext())
        {
          ISearchSettingActions pei = (ISearchSettingActions) e.GetCurrent();
          pei.OnBeforeRemoveSavedSearch(m_asOwner, (IPTSession) m_asOwner.GetUserSession(),
           ssd);
        }
      }
    }
    protected virtual void DoAfterModifyDeleteSSPEI(String[] aSavedSearches)
    {
      LoadSearchPEIs();
      // Call out to each PEI in turn
      if (null != _searchPEIs)
      {
        IXPEnumerator e = _searchPEIs.GetEnumerator();
        while (e.MoveNext())
        {
          ISearchSettingActions pei = (ISearchSettingActions) e.GetCurrent();
          pei.OnAfterModifyOrRemoveSavedSearch(m_asOwner, (IPTSession) m_asOwner.GetUserSession(),
           aSavedSearches);
        }
      }
    }
    
    /// <summary>
    /// This function will pull in all of the available
    /// ISearchSettingActions PEIs.
    /// </summary>
    private void LoadSearchPEIs()
    {
      // if already loaded
      if (_loadedPEIs)
      {
        // don't do it again
        return;
      }
      _loadedPEIs = true;
      IApplication application = m_asOwner.GetApplication();
      Object[] actions = (Object[]) application.GetCachingManager().GetEntry("SearchSettingActions");
      if (null != actions)
      {
        _searchPEIs = new XPArrayList();
        for (int x = 0;x < actions.Length;x++)
        {
          Object oAction = actions[x];
          if ((null != oAction) && (oAction is ISearchSettingActions))
          {
            _searchPEIs.Add(oAction);
          } else
          {
            log.Warn("SearchSettingsModel.LoadBannerSearchPEIs: element " + XPConvert.ToString(x) + " is not an instance of ISearchSettingsActions");
            }
        }
      }
    }
  }
}