using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.portalpages.browsing.myportal.mypages;
using com.plumtree.portaluiinfrastructure.intrinsicportlets;
using com.plumtree.portaluiinfrastructure.resultwrapper;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.statichelpers;
namespace com.plumtree.portalpages.browsing.portletpreview
{
  /// <summary>
  /// File: PreviewPortletModel.java
  /// -----------------------------
  /// Model for entire preview/invitation (and invitation join) functionality.
  /// </summary>
  /// <author>NaveenP/DustinA</author>
  public class PreviewPortletModel : AClearModel,IRepostModel,IModel,IPreviewPortletModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
     "com.plumtree.portalpages.browsing.portletpreview.PreviewPortletModel");
    public const String STR_MVC_CLASS_NAME = "PreviewGadget";
    
    /// <summary>
    /// Constant: Preview Header Bar for Selection Page
    /// </summary>
    public const int SELECTIONPREVIEWHEADER = 1;
    
    /// <summary>
    /// Constant: Invite Header Bar 
    /// </summary>
    public const int INVITEHEADER = 2;
    
    /// <summary>
    /// Constant: Preview Header Bar for preview only (as in Portlet Editor)
    /// </summary>
    public const int PREVIEWHEADER = 3;
    
    /// <summary>
    /// Constant: Invite Header Bar 
    /// </summary>
    public const int SEARCHHEADER = 4;
    
    /// <summary>
    /// Whether to always use community preview mode 
    /// </summary>
    public bool m_bAlwaysCommunity;
    
    /// <summary>
    /// Portlet ID 
    /// </summary>
    public int m_nPortletID;
    
    /// <summary>
    /// Preview Status 
    /// </summary>
    public int m_nPreviewStatus;
    
    /// <summary>
    /// Is it an Error Page 
    /// </summary>
    public bool m_bErrorPage;
    
    /// <summary>
    /// Portlet Checked on selection page (for checkbox) 
    /// </summary>
    public bool m_bPortletChecked = false;
    
    /// <summary>
    /// Portlet Index on selection page (for checkbox) 
    /// </summary>
    public int m_nPortletIndex;
    
    /// <summary>
    /// Host Page URI 
    /// </summary>
    public String m_strHostPageURI = "url_not_set";
    public String m_strRawHeaders = "";
    
    /// <summary>
    /// Page ID 
    /// </summary>
    public int m_nPageID;
    
    /// <summary>
    /// Activity Space 
    /// </summary>
    public AActivitySpace m_asOwner = null;
    
    /// <summary>
    /// Session 
    /// </summary>
    public IPTSession m_ptSession;
    
    /// <summary>
    /// Localized Portlet Name *
    /// </summary>
    public String m_strPortletName = "";
    
    /// <summary>
    /// Localized Portlet Description *
    /// </summary>
    public String m_strPortletDesc = "";
    
    /// <summary>
    /// Portlet Alignment *
    /// </summary>
    public int m_nPortletAlignment = 0;
    
    /// <summary>
    /// Is Portlet Enabled *
    /// </summary>
    public bool m_bPortletEnabled = true;
    
    /// <summary>
    /// Disabled Message *
    /// </summary>
    public String m_strDisabledMessage = "";
    
    /// <summary>
    /// Add Portlet Javascript override *
    /// </summary>
    public String m_strJavascriptOverride = null;
    
    /// <summary>
    /// temp object 
    /// </summary>
    class PreviewPortletModelTempData
    {
      
      /// <summary>
      /// Portlet 
      /// </summary>
      public IPTGadget m_objPortlet;
      
      /// <summary>
      /// App Data State 
      /// </summary>
      public IPTState m_objAppDataState;
      
      /// <summary>
      /// Page Name 
      /// </summary>
      public String m_strPageName;
      
      /// <summary>
      /// XPHashtable for saving data 
      /// </summary>
      public XPHashtable m_htFormData = null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AClearModel#GetNewm_TempData()"></seealso>
    public override Object GetNewTempData()
    {
      PreviewPortletModelTempData tempData = new PreviewPortletModelTempData();
      tempData.m_htFormData = new XPHashtable();
      m_strHostPageURI = PTConfigHelper.GetApplicationBaseURL(m_asOwner);
      tempData.m_objAppDataState = AppDataObject.GetAppDataObject(0, m_nPageID,
       m_strHostPageURI, m_asOwner);
      // tempData.m_objAppDataState.Add(PlumtreeExtensibility.PT_HEADERS_KEY, m_strRawHeaders, 0, true);
      HttpUtils.SetReturnURI(m_strHostPageURI, m_asOwner.GetSubSession());
      if (UserHasReadAccess())
      {
        try
        {
          tempData.m_objPortlet = (IPTGadget) m_ptSession.GetGadgets().Open(m_nPortletID,
           false);
          // Fix to bug #28941, this is needed by other Views after the model is cleared
          m_strPortletName = tempData.m_objPortlet.GetLocalizedName();
          m_strPortletDesc = tempData.m_objPortlet.GetLocalizedDescription();
          m_nPortletAlignment = tempData.m_objPortlet.GetAlignment();
          IPTWebService ptWebService = (IPTWebService) m_ptSession.GetWebServices().Open(tempData.m_objPortlet.GetWebServiceID(),
           false);
          m_bPortletEnabled = ptWebService.GetEnabled();
          m_strDisabledMessage = ptWebService.GetDisabledMessage();
        }
        catch (Exception e)
        {
          log.Error(e, "Error opening portlet.");
          throw new XPException(m_asOwner.GetString(608, "ptmsgs_portalbrowsingmsgs"));
          }
      } else
      {
        throw new XPException(m_asOwner.GetString(614, "ptmsgs_portalbrowsingmsgs"));
        }
      return tempData;
    }
    public PreviewPortletModel()
    {
    }
    private PreviewPortletModelTempData GetMyTempData()
    {
      return (PreviewPortletModelTempData) GetTempData();
    }
    
    /// <summary>
    /// Add Portlet In Page - adds gadget to the specified user's page
    /// </summary>
    /// <param name="int">nPageID</param>
    public virtual void AddPortletInPage()
    {
      if (!IsPortletInPage())
      {
        IPTPage myPage = (IPTPage) m_ptSession.GetPages().Open(m_nPageID,
         false);
        // add the gadget to the correct column in the page
        int nPageType = myPage.GetPageType();
        int nColumnForPortlet = 0;
        if (IsNarrow())
        {
          nColumnForPortlet = PortletColumnConstants.NARROW_ONE.ToInt();
          if (nPageType == PageTypeConstants.ONE_COLUMN)
          {
            nColumnForPortlet = PortletColumnConstants.WIDE_ONE.ToInt();
          }
        } else
        {
          nColumnForPortlet = PortletColumnConstants.WIDE_ONE.ToInt();
        }
        myPage.AddPortlet(m_nPortletID, nColumnForPortlet);
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new PreviewPortletModel();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#Finish()"></seealso>
    public virtual void Finish()
    {
    }
    
    /// <seealso cref="com.plumtree.portalpages.browsing.portletpreview.IPreviewPortletModelRO#GetAddPortletJavaScriptOverride()"></seealso>
    public virtual String GetAddPortletJavaScriptOverride()
    {
      return m_strJavascriptOverride;
    }
    
    /// <summary>
    /// Get Default MyPage URL
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetDefaultMyPageURL()
    {
      ASURL urlmypage = new ASURL(m_asOwner);
      urlmypage.SetLinkGetSpaceIfCached(MyPageAS.STR_MVC_CLASS_NAME, m_asOwner);
      urlmypage.SetDisplayPage(MyPortalDP.STR_MVC_CLASS_NAME);
      urlmypage.SetControl(SetPageControl.STR_MVC_CLASS_NAME);
      urlmypage.AddControlArgument(SetPageControl.STR_PAGE_ID, 0);
      return urlmypage.GetURLAsString();
    }
    
    /// <summary>
    /// Get Portlet Alignment
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetPortletAlignment()
    {
      return GetMyTempData().m_objPortlet.GetAlignment();
    }
    
    /// <summary>
    /// Get Portlet HTML Text
    /// </summary>
    /// <returns>String</returns>
    public virtual Object[][] GetPortletHTMLText()
    {
      Object[][] HTMLText = null;
      try
      {
        HTMLText = (Object[][]) GetMyTempData().m_objPortlet.GetHTMLText(GetMyTempData().m_objAppDataState);
        }
      catch (Exception e)
      {
        //pkd extracted:     		throw new Exception(_ui_CouldNotDisplayGadget + e);	
        throw new XPException(m_asOwner.GetString(607, "ptmsgs_portalbrowsingmsgs") + e);
        }
      return HTMLText;
    }
    
    /// <seealso cref="com.plumtree.portalpages.browsing.portletpreview.IPreviewPortletModelRO#GetPortletID()"></seealso>
    public virtual int GetPortletID()
    {
      return m_nPortletID;
    }
    
    /// <seealso cref="com.plumtree.portalpages.browsing.portletpreview.IPreviewPortletModelRO#GetPortletIndex()"></seealso>
    public virtual int GetPortletIndex()
    {
      return m_nPortletIndex;
    }
    
    /// <summary>
    /// Get Portlet Name
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetPortletName()
    {
      //return GetMyTempData().m_objPortlet.GetLocalizedName();
      // Fix for #28941
      return m_strPortletName;
    }
    
    /// <summary>
    /// Get Portlet Description
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetPortletDescription()
    {
      //return GetMyTempData().m_objPortlet.GetLocalizedDescription();
      // Fix for #28941
      return m_strPortletDesc;
    }
    
    /// <summary>
    /// Is Portlet Enabled
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsPortletEnabled()
    {
      return m_bPortletEnabled;
    }
    
    /// <summary>
    /// Get Disabled Message if portlet is disabled
    /// </summary>
    /// <returns>boolean</returns>
    public virtual String GetPortletDisabledMessage()
    {
      return m_strDisabledMessage;
    }
    
    /// <summary>
    /// Get Preview Portlet URL
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetPortletPreviewURL()
    {
      ASURL urlPreview = new ASURL(m_asOwner);
      urlPreview.SetLinkGetCachedSpace(m_asOwner);
      urlPreview.SetDisplayPage(PreviewPortletDP.STR_MVC_CLASS_NAME);
      urlPreview.SetControl(PreviewPortletControl.STR_MVC_CLASS_NAME);
      urlPreview.AddControlArgument(PreviewPortletControl.m_strPortletParam,
       (XPConvert.ToString(m_nPortletID)));
      urlPreview.AddControlArgument(PreviewPortletControl.m_PreviewType, (XPConvert.ToString(PreviewPortletModel.INVITEHEADER)));
      return urlPreview.GetURLAsString();
    }
    
    /// <summary>
    /// Returns true if the community preview mode should be used even if the portlet is not community-only
    /// </summary>
    /// <returns>True if the community preview mode should be used even if the portlet is not community-only</returns>
    public virtual bool GetAlwaysUseCommunityPreview()
    {
      return m_bAlwaysCommunity;
    }
    
    /// <summary>
    /// Tells the model that community preview mode should always be used, even if the portlet is not community-only
    /// </summary>
    public virtual void SetAlwaysUseCommunityPreview(bool b)
    {
      m_bAlwaysCommunity = b;
    }
    
    /// <summary>
    /// Returns the gadget type
    /// </summary>
    /// <returns>Portlet type</returns>
    public virtual int GetPortletType()
    {
      return GetMyTempData().m_objPortlet.GetGadgetType();
    }
    
    /// <summary>
    /// Get MyPage URL
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetMyPageURL()
    {
      ASURL urlmypage = new ASURL(m_asOwner);
      urlmypage.SetLinkGetSpaceIfCached(MyPageAS.STR_MVC_CLASS_NAME, m_asOwner);
      urlmypage.SetDisplayPage(MyPortalDP.STR_MVC_CLASS_NAME);
      urlmypage.SetControl(SetPageControl.STR_MVC_CLASS_NAME);
      urlmypage.AddControlArgument(SetPageControl.STR_PAGE_ID, m_nPageID);
      return urlmypage.GetURLAsString();
    }
    
    /// <summary>
    /// Get My Pages
    /// </summary>
    /// <returns>boolean</returns>
    public virtual IASQueryResult GetMyPages()
    {
      IPTMyPages myPages = m_ptSession.GetMyPages();
      IPTQueryResult userPages = myPages.QueryPages(0);
      IASQueryResult asQR = null;
      try
      {
        asQR = new ASQueryResultWrapper(userPages);
      }
      catch (Exception e)
      {
        log.Error(e, e.Message);
      }
      return asQR;
    }
    
    /// <summary>
    /// Get Object Name
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.portalpages.browsing.portletpreview.IPreviewPortletModel#GetPageID()"></seealso>
    public virtual int GetPageID()
    {
      return m_nPageID;
    }
    
    /// <seealso cref="com.plumtree.portalpages.browsing.portletpreview.IPreviewPortletModel#GetPageID()"></seealso>
    public virtual String GetPageName()
    {
      return GetMyTempData().m_strPageName;
    }
    
    /// <summary>
    /// Get AS Parent
    /// </summary>
    /// <returns>AActivitySpace</returns>
    public virtual AActivitySpace GetParent()
    {
      return m_asOwner;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#GetRedirectAfterFinish()"></seealso>
    public virtual Redirect GetRedirectAfterFinish()
    {
      if (m_asOwner.GetAccessStyle() == AccessStyles.STANDARD)
      {
        return null;
      } else
      {
        Redirect redirect = new Redirect();
        redirect.SetLinkGetSpaceIfCached(MyPageAS.STR_MVC_CLASS_NAME, m_asOwner);
        return redirect;
      }
    }
    
    /// <seealso cref="com.plumtree.portalpages.browsing.portletpreview.IPreviewPortletModelRO#GetSession()"></seealso>
    public virtual IPTSession GetSession()
    {
      return m_ptSession;
    }
    
    /// <seealso cref="com.plumtree.portalpages.browsing.portletpreview.IPreviewPortletModelRO#GetUserID()"></seealso>
    public virtual int GetUserID()
    {
      return m_ptSession.GetSessionInfo().GetCurrentUserID();
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IModel#Init()"></seealso>
    public override void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
      m_ptSession = (IPTSession) m_asOwner.GetUserSession();
      m_bAlwaysCommunity = false;
    }
    
    /// <summary>
    /// IsErrorPage
    /// </summary>
    /// <param name="boolean"></param>
    public virtual bool IsErrorPage()
    {
      return m_bErrorPage;
    }
    
    /// <summary>
    /// Is Portlet Footer
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsFooter()
    {
      return (m_nPortletAlignment == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_FOOTER);
      }
    
    /// <summary>
    /// Is Portlet FreeForm
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsFreeForm()
    {
      return (m_nPortletAlignment == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_FREEFORM);
      }
    
    /// <summary>
    /// Check if Portlet is already selected
    /// </summary>
    public virtual bool IsPortletChecked()
    {
      return m_bPortletChecked;
    }
    
    /// <summary>
    /// Is Portlet In Page - checks to if the gadget is already in
    /// the user's page that's specified
    /// </summary>
    /// <param name="int">nPageID</param>
    public virtual bool IsPortletInPage()
    {
      IPTPage myPage = (IPTPage) m_ptSession.GetPages().Open(m_nPageID, false);
      //list of gadgets in the portal
      IPTQueryResult gadgetsInPage = myPage.QueryPortlets();
      //Variant Portlet;
      int intPortlet;
      bool bIsInPage = false;
      for (int i = 0;i < gadgetsInPage.RowCount();i++)
      {
        intPortlet = gadgetsInPage.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
        if (intPortlet == m_nPortletID)
        {
          bIsInPage = true;
          GetMyTempData().m_strPageName = myPage.GetName();
          break;
        }
      }
      return bIsInPage;
    }
    
    /// <summary>
    /// Is Portlet Invite Page
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsPortletInvitePage()
    {
      return (m_nPreviewStatus == INVITEHEADER);
    }
    
    /// <summary>
    /// Is Portlet Search Preview Page
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsPortletSearchPreviewPage()
    {
      return (m_nPreviewStatus == SEARCHHEADER);
    }
    
    /// <summary>
    /// Is Portlet Selection Preview Page
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsPortletSelectionPreviewPage()
    {
      return (m_nPreviewStatus == SELECTIONPREVIEWHEADER);
    }
    
    /// <summary>
    /// Is Portlet Header
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsHeader()
    {
      return (m_nPortletAlignment == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_HEADER);
      }
    
    /// <summary>
    /// Is Portlet Narrow
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsNarrow()
    {
      return (m_nPortletAlignment == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NARROW);
      }
    
    /// <summary>
    /// Is Portlet None
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsNone()
    {
      return (m_nPortletAlignment == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NONE);
      }
    
    /// <summary>
    /// Is Portlet Wide
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool IsWide()
    {
      return (m_nPortletAlignment == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE);
      }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String,XPHashtable)"></seealso>
    /// <returns>int Page status for server side validation</returns>
    public virtual int SavePage(String _sPageName, XPHashtable _htFormData)
    {
      PreviewPortletModelTempData tempData = (PreviewPortletModelTempData) GetTempData();
      int nSize = _htFormData.GetSize();
      IXPEnumerator xpKeys = _htFormData.GetKeys();
      IXPEnumerator xpElements = _htFormData.GetElements();
      for (int i = 0;i < nSize;i++)
      {
        xpKeys.MoveNext();
        xpElements.MoveNext();
        tempData.m_htFormData.PutElement((String) xpKeys.GetCurrent(), (String[]) xpElements.GetCurrent());
        }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Set the Add Portlet Javascript override.
    /// This javascript should be the name of the javascript function to be
    /// used in the onclick attribute of the add button.  The function should
    /// take 2 arguments: the portlet ID, and a reference to the popup window.
    /// (i.e. If the name is "addPortlet", then the function will look like
    /// this: "addPortlet(portletID, self)").  This function name can also
    /// include an optional window reference (i.e. "opener.addPortlet").
    /// Null means to use the default javascript.
    /// </summary>
    public virtual void SetAddPortletJavaScriptOverride(String _strOverride)
    {
      m_strJavascriptOverride = _strOverride;
    }
    
    /// <summary>
    /// SetErrorPage
    /// </summary>
    /// <param name="boolean"></param>
    public virtual void SetErrorPage(bool bSetErrorPage)
    {
      m_bErrorPage = bSetErrorPage;
    }
    
    /// <param name="sPortletChecked"></param>
    public virtual void SetPortletChecked(bool bPortletChecked)
    {
      m_bPortletChecked = bPortletChecked;
    }
    
    /// <summary>
    /// Set Portlet ID
    /// </summary>
    /// <param name="int">nPortletID</param>
    public virtual void SetPortletID(int nPortletID)
    {
      m_nPortletID = nPortletID;
      // DCA - calling GetMyTempData() to make sure nPortletID is valid
      GetMyTempData();
    }
    
    /// <param name="nPortletIndex"></param>
    public virtual void SetPortletIndex(int nPortletIndex)
    {
      m_nPortletIndex = nPortletIndex;
    }
    
    /// <summary>
    /// Set to use Invite Header
    /// </summary>
    public virtual void SetPortletToInvitePage()
    {
      m_nPreviewStatus = INVITEHEADER;
    }
    
    /// <summary>
    /// Set to use Preview Header
    /// </summary>
    public virtual void SetPortletToPreviewPage()
    {
      m_nPreviewStatus = PREVIEWHEADER;
    }
    
    /// <summary>
    /// Set to use Search Preview Header
    /// </summary>
    public virtual void SetPortletToSearchPreviewPage()
    {
      m_nPreviewStatus = SEARCHHEADER;
    }
    
    /// <summary>
    /// Set to use Selection Preview Header
    /// </summary>
    public virtual void SetPortletToSelectionPreviewPage()
    {
      m_nPreviewStatus = SELECTIONPREVIEWHEADER;
    }
    
    /// <summary>
    /// Sets the host page URI that is later used to initialize the AppDataObject
    /// </summary>
    /// <param name="strHostPageURI">strHostPageURI</param>
    public virtual void SetHostPageURI(String strHostPageURI)
    {
      m_strHostPageURI = strHostPageURI;
    }
    
    /// <summary>
    /// Set The Page ID
    /// </summary>
    /// <param name="int"></param>
    public virtual void SetPageID(int nPageID)
    {
      m_nPageID = nPageID;
    }
    
    /// <param name="strRawHeaders"></param>
    public virtual void SetRawHeaders(String strRawHeaders)
    {
      m_strRawHeaders = strRawHeaders;
    }
    
    /// <summary>
    /// Register model, view, and control for intrinsic portlet.  Initialize
    /// portlet with the portletID, the IPTMyPortal object, and the
    /// AppDataState object.
    /// </summary>
    /// <param name="_strPortletClass">Class identifier for portlet</param>
    /// <param name="_strStorageName">Storage name for portlet</param>
    /// <throws>
    /// XPException Exception thrown if model, view, or control is not
    /// registered successfully.
    /// </throws>
    public virtual void SetupIntrinsicPortlet(String _strPortletClass, String _strStorageName)
    {
      // Register Model
      if (!m_asOwner.GetModelExists(_strStorageName))
      {
        bool bModelRegistered = m_asOwner.RegisterModel(_strPortletClass,
         _strStorageName);
        if (!bModelRegistered)
        {
          log.Error("PreviewPortletModel::SetupIntrinsicPortlet. " + m_asOwner.GetString(605,
           "ptmsgs_portalbrowsingmsgs"));
          throw new XPException(m_asOwner.GetString(605, "ptmsgs_portalbrowsingmsgs"));
          }
      }
      IntrinsicPortletModel iPortletModel = (IntrinsicPortletModel) m_asOwner.GetModel(_strStorageName);
      // Register Section
      if (!m_asOwner.GetViewExists(_strStorageName))
      {
        bool bViewRegistered = m_asOwner.RegisterView(_strPortletClass, _strStorageName,
         iPortletModel);
        if (!bViewRegistered)
        {
          log.Error("PreviewPortletModel::SetupIntrinsicPortlet. " + m_asOwner.GetString(606,
           "ptmsgs_portalbrowsingmsgs"));
          throw new XPException(m_asOwner.GetString(606, "ptmsgs_portalbrowsingmsgs"));
          }
      }
      // Register Control
      if (iPortletModel.ControlExists())
      {
        if (!m_asOwner.GetControlExists(_strStorageName))
        {
          bool bControlRegistered = m_asOwner.RegisterControl(_strPortletClass,
           _strStorageName, iPortletModel);
          if (!bControlRegistered)
          {
            log.Error("PreviewPortletModel::SetupIntrinsicPortlet. " + m_asOwner.GetString(604,
             "ptmsgs_portalbrowsingmsgs"));
            throw new XPException(m_asOwner.GetString(604, "ptmsgs_portalbrowsingmsgs"));
            }
        }
      }
      PreviewPortletModelTempData tempData = GetMyTempData();
      // Initialize portlet model
      iPortletModel.InitPortlet(null, tempData.m_objAppDataState, tempData.m_objPortlet,
       m_nPortletID);
    }
    
    /// <seealso cref="com.plumtree.portalpages.browsing.portletpreview.IPreviewPortletModel#UserHasReadAccess()"></seealso>
    public virtual bool UserHasReadAccess()
    {
      return m_ptSession.GetGadgets().IsObjectAccessible(m_nPortletID, m_ptSession.GetSessionInfo().GetCurrentUserID(),
       PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ);
    }
    
    /// <seealso cref="com.plumtree.portalpages.browsing.portletpreview.IPreviewPortletModel#UserHasSelectAccess()"></seealso>
    public virtual bool UserHasSelectAccess()
    {
      return m_ptSession.GetGadgets().IsObjectAccessible(m_nPortletID, m_ptSession.GetSessionInfo().GetCurrentUserID(),
       PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);
    }
    
    /// <seealso cref="com.plumtree.portalpages.browsing.portletpreview.IPreviewPortletModelRO#UserHasMyPageAccess()"></seealso>
    public virtual bool UserHasMyPageAccess()
    {
      IPTSubPortalInfo subPortalInfo = m_ptSession.GetSessionInfo().GetCurrentUserCachedSubPortalInfo();
      int nBool = subPortalInfo.GetAccessType() & PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_MYPAGES_ACCESS;
      return nBool != 0;
    }
    
    /// <summary>
    /// Method GetAllCommunitiesUsingPortlet.  Gets a compound list suitable for
    /// URL constrcution for communities preview links.
    /// </summary>
    /// <returns>ASCompoundList</returns>
    public virtual XPArrayList GetAllCommunitiesUsingPortlet()
    {
      XPArrayList list = new XPArrayList();
      try
      {
        IPTCommunityManager ptCommOM = (IPTCommunityManager) m_ptSession.GetCommunities();
        if (ptCommOM == null)
        {
          return list;
        }
        IPTQueryResult ptQRCommunities = ptCommOM.QueryCommunitiesForGadget(m_nPortletID);
        if (ptQRCommunities == null)
        {
          return list;
        }
        // form list of objects to be used to form URLs to community preview (invite)
        for (int i = 0;i < ptQRCommunities.RowCount();i++)
        {
          int iCommID = ptQRCommunities.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
          String strCommName = ptQRCommunities.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
          list.Add(new CommunityInviteItem(iCommID, strCommName));
        }
      }
      catch (Exception e)
      {
        log.Error(e, "Exception getting Related Communities.");
      }
      return list;
    }
  }
}