using System;
// Plumtree Server Imports
using com.plumtree.openfoundation.util;
using com.plumtree.portalpages.admin.objmgr;
using com.plumtree.portalpages.common.documentlanguage;
using com.plumtree.portalpages.common.search;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.web;
//Put explicit import for Interpreter (to avoid ambiguity of using the deprecated class)
using Interpreter=com.plumtree.uiinfrastructure.interpreter.Interpreter;
namespace com.plumtree.portalpages.browsing.search.advanced
{
  /// <summary>
  /// ActivitySpace for advanced search.
  /// </summary>
  /// <author>CraigS</author>
  public class AdvancedSearchAS : PlumtreeAS,IFormAS
  {
    public const String STR_MVC_CLASS_NAME = "AdvancedSearch";
    
    /// <returns></returns>
    public override Object Create()
    {
      return new AdvancedSearchAS();
    }
    
    /// <returns></returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Returns the name of the repost control class.
    /// </summary>
    /// <returns>String</returns>
    public override String GetRepostControlName()
    {
      return AdvancedSearchRepostControl.STR_MVC_CLASS_NAME;
    }
    public override void Init()
    {
      base.Init();
      ISessionManager subsession = GetSubSession();
      BrowserType browser = (BrowserType) subsession.GetAttribute(Interpreter.BROWSERTYPE);
      bool bNS4 = browser.IsNetscape4();
      // In 508 mode, we actually want to use the "power search" model and view,
      // which use no javascript.
      if (GetAccessStyle() != AccessStyles.STANDARD || bNS4)
      {
        RegisterModel(PowerSearchModel.STR_MVC_CLASS_NAME);
        PowerSearchModel powerModel = (PowerSearchModel) GetModel(PowerSearchModel.STR_MVC_CLASS_NAME);
        /* propertyConstraints= */
        powerModel.Setup(new int[]{1, 2, 50, 3, 4, 101, 103, 106}, /* name */
        /* description */
        /* content */
        /* created */
        /* last modified */
        /* keywords */
        /* author */
        /* supportTextSearch= */
        true, /* supportedClassIDs= */
        new int[]{PT_CLASSIDS.PT_CATALOGCARD_ID, PT_CLASSIDS.PT_CATALOGFOLDER_ID,
         PT_CLASSIDS.PT_COMMUNITY_ID, PT_CLASSIDS.PT_GADGET_ID, PT_CLASSIDS.PT_USER_ID},
         /* makeClassIDsSelectable= */
        true, /* showAbstractTypeSelector= */
        false, /* showApplicationSelector= */
        true);
        RegisterControl(PowerSearchControl.STR_MVC_CLASS_NAME, powerModel);
        RegisterView(PowerSearchView.STR_MVC_CLASS_NAME, powerModel);
        RegisterPage(new PowerSearchDP());
        if (ObjMgrAS.STR_MVC_CLASS_NAME.Equals(GetParentSpaceName()))
        {
          // If we came from the ObjMgr page (admin), then we must be
          // doing an admin advanced search and need the appropriate
          // help.
          SetHelpTopic("PT_AdvancedAdministrativeSearch");
        } else
        {
          // Otherwise just use the normal advanced search help.
          SetHelpTopic("PT_AdvancedSearch");
        }
        return;
      }
      // Register and init model
      RegisterModel(AdvancedSearchModel.STR_MVC_CLASS_NAME);
      IAdvancedSearchModelRO model = (IAdvancedSearchModelRO) GetModel(AdvancedSearchModel.STR_MVC_CLASS_NAME);
      // Register DisplayPages
      RegisterPage(new AdvancedSearchDP());
      // Register views and give them access to the model
      RegisterView(ASFilterView.STR_MVC_CLASS_NAME, model);
      RegisterView(ASButtonView.STR_MVC_CLASS_NAME, model);
      RegisterView(ASFooterView.STR_MVC_CLASS_NAME, model);
      // Register controls
      RegisterControl(AdvancedSearchRepostControl.STR_MVC_CLASS_NAME, (IModel) model);
      RegisterControl(EditorStartControl.STR_MVC_CLASS_NAME, (IModel) model);
      // Register the language plugin model.
      RegisterModel(CardLanguagePlugInModel.STR_MVC_CLASS_NAME);
      IModel languageModel = GetModel(CardLanguagePlugInModel.STR_MVC_CLASS_NAME);
      // Register the language plugin views and controls.
      RegisterView(CardLanguagePlugInView.STR_MVC_CLASS_NAME, languageModel);
      RegisterControl(CardLanguagePlugInRepostControl.STR_MVC_CLASS_NAME,
       languageModel);
      // Register the number-of-results model.
      RegisterModel(SimpleSelectorModel.STR_MVC_CLASS_NAME);
      IModel nrModel = GetModel(SimpleSelectorModel.STR_MVC_CLASS_NAME);
      // Register the number-of-results plugin view.
      RegisterView(SimpleSelectorView.STR_MVC_CLASS_NAME, nrModel);
      // Register the folder selection plugin model.
      RegisterModel(FolderSelectorModel.STR_MVC_CLASS_NAME);
      IModel foldersModel = GetModel(FolderSelectorModel.STR_MVC_CLASS_NAME);
      // Register folder selection views and controls.
      RegisterView(ASFoldersView.STR_MVC_CLASS_NAME, foldersModel);
      RegisterControl(FolderSelectorRepostControl.STR_MVC_CLASS_NAME, foldersModel);
      // Register the type selection plugin model.
      RegisterModel(TypeSelectorModel.STR_MVC_CLASS_NAME);
      IModel typeModel = GetModel(TypeSelectorModel.STR_MVC_CLASS_NAME);
      // Register type selection views and controls.
      RegisterView(ASTypesView.STR_MVC_CLASS_NAME, typeModel);
      RegisterControl(TypeSelectorRepostControl.STR_MVC_CLASS_NAME, typeModel);
      if (ObjMgrAS.STR_MVC_CLASS_NAME.Equals(GetParentSpaceName()))
      {
        // If we came from the ObjMgr page (admin), then we must be
        // doing an admin advanced search and need the appropriate
        // help.
        SetHelpTopic("PT_AdvancedAdministrativeSearch");
      } else
      {
        // Otherwise just use the normal advanced search help.
        SetHelpTopic("PT_AdvancedSearch");
      }
    }
  }
}