using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portalpages.browsing.search.results;
using com.plumtree.portalpages.common.documentlanguage;
using com.plumtree.portalpages.common.search;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.portaluiinfrastructure.search;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.search.advanced
{
  /// <author>CraigS</author>
  public class AdvancedSearchDP : PlumtreeDP
  {
    public const String STR_MVC_CLASS_NAME = "AdvancedSearch";
    public static String TABLE_WIDTH = "95%";
    public const String NARROW_SPACER = "10";
    public const String WIDE_SPACER = "15";
    private const String PORTAL_BROWSING_MSGS = "ptmsgs_portalbrowsingmsgs";
    
    /// <summary>
    /// Displays some HTML after the body tag and before the form tag.
    /// For example this is used by pages using a DHTML calendar to display the div tag.
    /// The function is optional.  If it exists, it will be called by the framework, and
    /// the resulting HTMLElement will be added in the right place.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public override HTMLElementCollection DisplayHTMLBeforeForm()
    {
      return FilterView.GetPagePreFormContent(m_asOwner);
    }
    
    /// <returns></returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.uiparts.PlumtreeDP#GetSubtitleForBanner()"></seealso>
    public override String GetSubtitleForBanner()
    {
      return "";
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.uiparts.PlumtreeDP#GetTitleForBanner()"></seealso>
    public override String GetTitleForBanner()
    {
      return m_asOwner.GetString(748, PORTAL_BROWSING_MSGS);
    }
    
    /// <param name="parent"></param>
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
      SetAddMainForm(true);
    }
    
    /// <returns></returns>
    /// <throws>HTMLException </throws>
    public override HTMLElement PageDisplay()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      AddHiddenModeInput(result);
      result.AddInnerHTMLElement(makeSpacer(WIDE_SPACER, m_asOwner));
      // Query header outer table
      HTMLTable table = new HTMLTable();
      result.AddInnerHTMLElement(table);
      table.SetCellSpacing(CommonHTMLStrings.FIVE);
      table.SetCellPadding(CommonHTMLStrings.FIVE);
      table.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      HTMLTableRow row = new HTMLTableRow();
      row.SetStyleClass(PTStyleClass.DIR_HEADER_BG);
      table.AddInnerHTMLElement(row);
      HTMLTableCell cell = new HTMLTableCell();
      row.AddInnerHTMLElement(cell);
      cell.SetStyleClass(PTStyleClass.DIR_HEADER);
      // Query header inner table
      table = new HTMLTable();
      cell.AddInnerHTMLElement(table);
      table.SetCellSpacing(CommonHTMLStrings.ZERO);
      table.SetCellPadding(CommonHTMLStrings.ZERO);
      table.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      row = new HTMLTableRow();
      row.SetStyleClass(PTStyleClass.DIR_HEADER_BG);
      table.AddInnerHTMLElement(row);
      cell = new HTMLTableCell();
      row.AddInnerHTMLElement(cell);
      cell.SetStyleClass(PTStyleClass.DIR_TEXT);
      cell.SetHeight("25");
      cell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      cell.SetVAlign(CommonHTMLStrings.MIDDLE);
      cell.SetNoWrap(true);
      HTMLSpan span = new HTMLSpan();
      span.SetStyleClass(PTStyleClass.DIR_HEADER);
      cell.AddInnerHTMLElement(span);
      span.AddInnerHTMLString(m_asOwner.GetString(750, PORTAL_BROWSING_MSGS));
      cell = new HTMLTableCell();
      row.AddInnerHTMLElement(cell);
      cell.SetNoWrap(true);
      cell.SetAlign(CommonHTMLStrings.RIGHT);
      cell.SetWidth("300");
      cell.SetStyleClass(PTStyleClass.BAN_TEXT);
      cell.AddInnerHTMLElement(m_asOwner.GetView(ASButtonView.STR_MVC_CLASS_NAME).Display());
      cell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      cell.AddInnerHTMLElement(m_asOwner.GetView(ASFooterView.STR_MVC_CLASS_NAME).Display());
      // cell.AddInnerHTMLString("&nbsp;&nbsp;&nbsp;&nbsp;");
      cell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      //result.AddInnerHTMLElement(makeHelpText());
      // Query filter table
      table = new HTMLTable();
      result.AddInnerHTMLElement(table);
      //lll
      table.SetCellSpacing(CommonHTMLStrings.FIVE);
      table.SetCellPadding(CommonHTMLStrings.ZERO);
      table.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      // Settings table
      HTMLTable settingsTable = new HTMLTable();
      result.AddInnerHTMLElement(settingsTable);
      settingsTable.SetCellSpacing(CommonHTMLStrings.FIVE);
      settingsTable.SetCellPadding(CommonHTMLStrings.ZERO);
      settingsTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      // Single row for settings table
      HTMLTableRow settingsRow = new HTMLTableRow();
      settingsTable.AddInnerHTMLElement(settingsRow);
      // Cells for settings table
      HTMLTableCell leftSettingsCell = new HTMLTableCell();
      settingsRow.AddInnerHTMLElement(leftSettingsCell);
      leftSettingsCell.SetVAlign(CommonHTMLStrings.TOP);
      leftSettingsCell.SetWidth("74%");
      // Spacer cell between left and right side
      HTMLTableCell spacerCell = new HTMLTableCell();
      settingsRow.AddInnerHTMLElement(spacerCell);
      spacerCell.SetWidth(CommonHTMLStrings.ZERO);
      spacerCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      HTMLTableCell rightSettingsCell = new HTMLTableCell();
      settingsRow.AddInnerHTMLElement(rightSettingsCell);
      rightSettingsCell.SetVAlign(CommonHTMLStrings.TOP);
      leftSettingsCell.AddInnerHTMLElement(m_asOwner.GetView(ASFilterView.STR_MVC_CLASS_NAME).Display());
      leftSettingsCell.AddInnerHTMLElement(makeSpacer(NARROW_SPACER, m_asOwner));
      // CS: is this right?  it seems like it is deliberately misaligning the left and right cells?
      //	rightSettingsCell.AddInnerHTMLElement(makeSpacer("1", m_asOwner));
      leftSettingsCell.AddInnerHTMLElement(m_asOwner.GetView(ASFoldersView.STR_MVC_CLASS_NAME).Display());
      //leftSettingsCell.AddInnerHTMLElement(makeSpacer(NARROW_SPACER, m_asOwner));
      rightSettingsCell.AddInnerHTMLElement(m_asOwner.GetView(SimpleSelectorView.STR_MVC_CLASS_NAME).Display());
      rightSettingsCell.AddInnerHTMLElement(makeSpacer(NARROW_SPACER, m_asOwner));
      CardLanguagePlugInModel languageModel = (CardLanguagePlugInModel) m_asOwner.GetModel(CardLanguagePlugInModel.STR_MVC_CLASS_NAME);
      languageModel.SetInstructionLabel(m_asOwner.GetString(1664, PORTAL_BROWSING_MSGS));
      rightSettingsCell.AddInnerHTMLElement(((CardLanguagePlugInView) m_asOwner.GetView(CardLanguagePlugInView.STR_MVC_CLASS_NAME)).Display());
      rightSettingsCell.AddInnerHTMLElement(makeSpacer(NARROW_SPACER, m_asOwner));
      rightSettingsCell.AddInnerHTMLElement(m_asOwner.GetView(ASTypesView.STR_MVC_CLASS_NAME).Display());
      result.AddInnerHTMLElement(makeSpacer(WIDE_SPACER, m_asOwner));
      result.AddInnerHTMLElement(MakeSearchHelpText(m_asOwner));
      return result;
    }
    
    /// <param name="height"></param>
    /// <returns></returns>
    /// <throws>HTMLException </throws>
    public static HTMLElement makeSpacer(String height, AActivitySpace owner)
    {
      HTMLTable table = new HTMLTable();
      table.SetCellSpacing(CommonHTMLStrings.ZERO);
      table.SetCellPadding(CommonHTMLStrings.ZERO);
      table.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      HTMLTableRow row = new HTMLTableRow();
      table.AddInnerHTMLElement(row);
      HTMLTableCell cell = new HTMLTableCell();
      row.AddInnerHTMLElement(cell);
      HTMLImg img = null;
      try
      {
        img = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.SPACER,
         true, owner).GetHTMLElement();
      }
      catch (Exception e)
      {
        throw XPException.GetInstance(e);
      }
      cell.AddInnerHTMLElement(img);
      img.SetHeight(height);
      img.SetWidth(CommonHTMLStrings.ONE);
      img.SetBorder(CommonHTMLStrings.ZERO);
      img.SetAlign(CommonHTMLStrings.ABSMIDDLE);
      return table;
    }
    
    /// <returns></returns>
    /// <throws>HTMLException </throws>
    protected override HTMLScriptCollection DisplayJavaScriptFromChild()
    {
      HTMLScriptCollection scripts = base.DisplayJavaScriptFromChild();
      scripts.AddInnerHTMLElement(m_asOwner.GetView(ASFoldersView.STR_MVC_CLASS_NAME).DisplayJavascript());
      scripts.AddInnerHTMLElement(m_asOwner.GetView(ASTypesView.STR_MVC_CLASS_NAME).DisplayJavascript());
      scripts.AddInnerHTMLElement(m_asOwner.GetView(ASFilterView.STR_MVC_CLASS_NAME).DisplayJavascript());
      return scripts;
    }
    
    /// <returns></returns>
    /// <throws>HTMLException </throws>
    public static HTMLTable MakeSearchHelpText(AActivitySpace m_asOwner)
    {
      HTMLTable outerTable = new HTMLTable();
      outerTable.SetCellSpacing(CommonHTMLStrings.FIVE);
      outerTable.SetCellPadding(CommonHTMLStrings.FIVE);
      outerTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      HTMLTableRow row = new HTMLTableRow();
      row.SetStyleClass(PTStyleClass.DIR_CONTENT_BG);
      outerTable.AddInnerHTMLElement(row);
      HTMLTableCell cell = new HTMLTableCell();
      row.AddInnerHTMLElement(cell);
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      cell.AddInnerHTMLString(m_asOwner.GetString(751, PORTAL_BROWSING_MSGS));
      cell.AddInnerHTMLElement(new HTMLBr());
      cell.AddInnerHTMLElement(new HTMLBr());
      HTMLTable table = new HTMLTable();
      cell.AddInnerHTMLElement(table);
      table.SetCellSpacing(CommonHTMLStrings.TWO);
      table.SetCellPadding(CommonHTMLStrings.THREE);
      row = new HTMLTableRow();
      row.SetStyleClass(PTStyleClass.DIR_CONTENT_BG);
      table.AddInnerHTMLElement(row);
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(752, PORTAL_BROWSING_MSGS));
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(753, PORTAL_BROWSING_MSGS));
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(754, PORTAL_BROWSING_MSGS));
      row = new HTMLTableRow();
      row.SetStyleClass(PTStyleClass.DIR_CONTENT_BG);
      table.AddInnerHTMLElement(row);
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(755, PORTAL_BROWSING_MSGS));
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(756, PORTAL_BROWSING_MSGS));
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(757, PORTAL_BROWSING_MSGS));
      row = new HTMLTableRow();
      row.SetStyleClass(PTStyleClass.DIR_CONTENT_BG);
      table.AddInnerHTMLElement(row);
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(758, PORTAL_BROWSING_MSGS));
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(759, PORTAL_BROWSING_MSGS));
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(760, PORTAL_BROWSING_MSGS));
      row = new HTMLTableRow();
      row.SetStyleClass(PTStyleClass.DIR_CONTENT_BG);
      table.AddInnerHTMLElement(row);
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(761, PORTAL_BROWSING_MSGS));
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(762, PORTAL_BROWSING_MSGS));
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(763, PORTAL_BROWSING_MSGS));
      row = new HTMLTableRow();
      row.SetStyleClass(PTStyleClass.DIR_CONTENT_BG);
      table.AddInnerHTMLElement(row);
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(764, PORTAL_BROWSING_MSGS));
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(765, PORTAL_BROWSING_MSGS));
      cell = new HTMLTableCell();
      cell.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
      row.AddInnerHTMLElement(cell);
      cell.AddInnerHTMLString(m_asOwner.GetString(766, PORTAL_BROWSING_MSGS));
      return outerTable;
    }
    protected virtual void AddHiddenModeInput(HTMLElement result)
    {
      // The idea here is to pass along the mode we come in with, if it wasn't the default, BROWSE_MODE.
      if (!SearchPageModeHelper.UserHasEditModeAccess(m_asOwner))
      {
        return;
      }
      AdvancedSearchModel m = (AdvancedSearchModel) m_asOwner.GetModel(AdvancedSearchModel.STR_MVC_CLASS_NAME);
      String mode = m.GetMode();
      if (null != mode)
      {
        HTMLInput inp = new HTMLInput(HTMLInputTypes.HIDDEN, SearchResultModel.MODEL_MODE_INPUT,
         SearchResultModel.MODEL_MODE_INPUT);
        inp.SetValue(mode);
        result.AddInnerHTMLElement(inp);
      }
    }
    public static ASURL GetAdvancedSearchURL(AActivitySpace m_asOwner)
    {
      ASURL url = new ASURL(m_asOwner);
      url.SetLinkCreateNewSpace(AdvancedSearchAS.STR_MVC_CLASS_NAME, m_asOwner);
      url.SetControl("EditorStart");
      url.AddControlArgument("editorType", "10");
      url.AddInnerHTMLString(m_asOwner.GetString(1702, "ptmsgs_portalbrowsingmsgs"));
      return url;
    }
  }
}