using System;
using com.plumtree.portalpages.admin.objmgr;
using com.plumtree.portalpages.admin.objmgr.popups;
using com.plumtree.portalpages.admin.objsearch;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.portaluiinfrastructure.search;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.search.results
{
  /// <author>CraigS</author>
  public class SearchResultAS : PlumtreeAS
  {
    public const String STR_MVC_CLASS_NAME = "SearchResult";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SearchResultAS();
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.AActivitySpace#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.AActivitySpace#Init()"></seealso>
    public override void Init()
    {
      base.Init();
      IntegratedSearchAS.RegisterSearchClasses(this);
      // Get the search model that was registered by IntegratedSearchAS
      IModel model = GetModel(SearchResultModel.STR_MVC_CLASS_NAME);
      // Initialize helper class that holds info about whether we're in browse vs. edit mode
      SearchPageModeHelper pageMode = new SearchPageModeHelper();
      pageMode.Init((SearchResultModel) model, this);
      // Register helper model that handles logging to the Analytics Server
      RegisterModel(LoggingHelperModel.STR_MVC_CLASS_NAME);
      // Register controls that initiate a search
      RegisterControl(BannerStartSearchControl.STR_MVC_CLASS_NAME, model);
      RegisterControl(AdvancedStartSearchControl.STR_MVC_CLASS_NAME, model);
      // Register control that saves a search
      RegisterControl(SaveSearchControl.STR_MVC_CLASS_NAME, model);
      // Register "search again" control
      RegisterControl(SearchWithinResultsControl.STR_MVC_CLASS_NAME, model);
      // Register main DisplayPage
      SearchResultDP dp = new SearchResultDP();
      RegisterPage(dp);
      dp.SetPageModeHelper(pageMode);
      // Register views
      RegisterView(FollowupSearchView.STR_MVC_CLASS_NAME, model);
      RegisterView(SearchBannerView.STR_MVC_CLASS_NAME, model);
      RegisterView(OrganizationView.STR_MVC_CLASS_NAME, model);
      RegisterView(DrilldownView.STR_MVC_CLASS_NAME, model);
      RegisterView(SearchSummaryView.STR_MVC_CLASS_NAME, model);
      RegisterView(GroupedResultsView.STR_MVC_CLASS_NAME, model);
      RegisterView(SearchFooterView.STR_MVC_CLASS_NAME, model);
      RegisterPage(new ObjMgrDeleteWarningsDP());
      RegisterControl(SearchBrowseEditModeControl.STR_MVC_CLASS_NAME, model);
      // ObjMgr Model
      RegisterModel(AdminSearchModel.STR_MVC_CLASS_NAME);
      IModel myModel = (AdminSearchModel) GetModel(AdminSearchModel.STR_MVC_CLASS_NAME);
      ((IAdminSearchModelRO) myModel).SetSearchModel((ISearchResultModelRO) model);
      // ObjMgr Views
      RegisterView(ToolbarView.STR_MVC_CLASS_NAME, myModel);
      // Obj Mgr popup Views
      RegisterView(ObjMgrDeleteWarningsView.STR_MVC_CLASS_NAME, myModel);
      // ObjMgr Controls
      RegisterControl(AdminSearchRepostControl.STR_MVC_CLASS_NAME, myModel);
      // Obj Mgr popup Controls
      RegisterControl(CheckDeleteControl.STR_MVC_CLASS_NAME, myModel);
      // Set the view helper for the grouped results view
      GroupedResultsView grv = (GroupedResultsView) GetView(GroupedResultsView.STR_MVC_CLASS_NAME);
      if (GetAccessStyle() == AccessStyles.STANDARD)
      {
        grv.SetHelper(new GroupedResultsViewHelper(pageMode));
      } else
      {
        grv.SetHelper(new GroupedResults508ViewHelper(pageMode));
      }
      ((FollowupSearchView) GetView(FollowupSearchView.STR_MVC_CLASS_NAME)).SetPageModeHelper(pageMode);
      ((SearchSummaryView) GetView(SearchSummaryView.STR_MVC_CLASS_NAME)).SetPageModeHelper(pageMode);
      ((SearchBannerView) GetView(SearchBannerView.STR_MVC_CLASS_NAME)).SetPageModeHelper(pageMode);
      // Register popup "save this search" DisplayPage and view
      RegisterPage(new SaveSearchPopupDP());
      RegisterPage(new SaveSearch508DP());
      RegisterView(SaveSearchView.STR_MVC_CLASS_NAME, model);
      RegisterView(SaveSearch508View.STR_MVC_CLASS_NAME, model);
      this.SetHelpTopic("PT_SearchResults");
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.AActivitySpace#GetStyleSheetLinks()"></seealso>
    public override HTMLElementCollection GetStyleSheetLinks()
    {
      String settingLanguage = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE,
       this);
      SearchResultModel _model = (SearchResultModel) GetModel(SearchResultModel.STR_MVC_CLASS_NAME);
      if ((null == _model) || (!_model.IsInitialized()))
      {
        return base.GetStyleSheetLinks();
      }
      int[] nCommunityIDs = _model.GetCommunityRestrictions();
      HTMLElementCollection pCollection = null;
      if (nCommunityIDs == null || nCommunityIDs.Length == 0 || nCommunityIDs.Length > 1)
      {
        pCollection = base.GetStyleSheetLinks();
      } else
      {
        int nHeaderID = _model.GetHeaderID(nCommunityIDs[0]);
        String strStyleSheet = "";
        if (nHeaderID > 0)
        {
          strStyleSheet = _model.GetStyleSheet(nCommunityIDs[0]);
        }
        if (!("").Equals(strStyleSheet) && (strStyleSheet != null))
        {
          if (GetIsAccessStyleNonStandard())
          {
            strStyleSheet = PlumtreeHelpers.STYLE_PREFIX_508 + strStyleSheet;
            }
          PTStyleSheetLink ptStyleSheetLink = new PTStyleSheetLink(strStyleSheet,
           settingLanguage);
          pCollection = new HTMLElementCollection();
          try
          {
            pCollection.AddInnerHTMLElement(ptStyleSheetLink.GetLinkAsHTMLElement(ConfigHelper.GetStyleSheetsBaseURL(this)));
            }
          catch (HTMLException e)
          {
          }
          catch (Exception e)
          {
          }
        } else
        {
          // use the default subportal stylesheet
          pCollection = base.GetStyleSheetLinks();
        }
      }
      return pCollection;
    }
  }
}