using System;
using com.plumtree.portalpages.browsing.editors.sitemap.sitemapfolderplugin;
using com.plumtree.portalpages.browsing.objectselection;
using com.plumtree.portalpages.common.createfolder;
using com.plumtree.portalpages.common.folder;
using com.plumtree.portalpages.common.objects;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.common;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.form.pagination;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.browsing.sitemap.sitemappage
{
  /// <summary>
  /// File: SiteMapAS.java
  /// --------------------
  /// </summary>
  /// <author>NaveenP</author>
  public class SiteMapAS : PlumtreeAS
  {
    public const String STR_MVC_CLASS_NAME = "SiteMap";
    public const String s_strCommSelectionFolderView = "CommSelectionFolderView";
    public const String s_strMembersObjectView = "MembersObjectView";
    public const String s_strSubCommunitiesObjectView = "SubCommunitiesObjectView";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SiteMapAS();
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IActivitySpace#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <returns></returns>
    public override String GetRepostControlName()
    {
      return SiteMapRepostControl.STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IActivitySpace#Init()"></seealso>
    public override void Init()
    {
      base.Init();
      SetTrackCurrentPage(true);
      // SiteMap DisplayPages
      SiteMapDP smDP = new SiteMapDP();
      smDP.SetAddMainForm(true);
      RegisterPage(smDP);
      RegisterPage(new DisplayInPortletDP());
      RegisterPage(new CreateFolderDP());
      RegisterPage(new ClosePopUpDP());
      MembersDP membersDP = new MembersDP();
      membersDP.SetAddMainForm(true);
      RegisterPage(membersDP);
      SubCommunitiesDP subCommDP = new SubCommunitiesDP();
      RegisterPage(subCommDP);
      // SiteMap Model
      RegisterModel(SiteMapModel.STR_MVC_CLASS_NAME);
      RegisterModel(ObjectSelectionModel.STR_MVC_CLASS_NAME);
      IModel sitemapModel = GetModel(SiteMapModel.STR_MVC_CLASS_NAME);
      // SiteMap Views
      RegisterView(SiteMapBreadcrumbView.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterView(SiteMapBannerView.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterView(FolderView.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterView(ObjectView.STR_MVC_CLASS_NAME, sitemapModel);
      IObjectViewHelper objHelper = new SiteMapLinkViewHelper();
      ((ObjectView) GetView(ObjectView.STR_MVC_CLASS_NAME)).SetObjectViewHelper(objHelper);
      RegisterView(SiteMapCreateFolderView.STR_MVC_CLASS_NAME, CreateFolderView.STR_VIEW_STORAGE_NAME,
       sitemapModel);
      RegisterView(DisplayInPortletView.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterView(MemberAndSubCommunitiesView.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterView(MembersView.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterView(SubCommunitiesView.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterView(ObjectView.STR_MVC_CLASS_NAME, s_strMembersObjectView,
       sitemapModel);
      objHelper = new MembersViewHelper();
      ((ObjectView) GetView(s_strMembersObjectView)).SetObjectViewHelper(objHelper);
      RegisterView(ObjectView.STR_MVC_CLASS_NAME, s_strSubCommunitiesObjectView,
       sitemapModel);
      objHelper = new SubCommunitiesViewHelper();
      ((ObjectView) GetView(s_strSubCommunitiesObjectView)).SetObjectViewHelper(objHelper);
      // SiteMap Controls
      RegisterControl(SetSiteMapControl.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterControl(DisplayInPortletControl.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterControl(SiteMapRepostControl.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterControl(SetGroupControl.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterControl(DefaultSiteMapPageControl.STR_MVC_CLASS_NAME, "", sitemapModel);
      RegisterModel(SiteMapFolderPlugInModel.STR_MVC_CLASS_NAME);
      IModel mySiteMapPluginModel = GetModel(SiteMapFolderPlugInModel.STR_MVC_CLASS_NAME);
      //Register the display page
      RegisterPage(new SiteMapCreateWebLinkDP());
      //The Site map folder views
      RegisterView(SiteMapFolderPlugInView.STR_MVC_CLASS_NAME, mySiteMapPluginModel);
      RegisterView(SiteMapCreateWebLinkView.STR_MVC_CLASS_NAME, mySiteMapPluginModel);
      //The site map folder control
      RegisterControl(SiteMapFolderPlugInControl.STR_MVC_CLASS_NAME, mySiteMapPluginModel);
      // Pagination Control
      RegisterControl(PaginationControl.STR_MVC_CLASS_NAME, sitemapModel);
      RegisterControl(PassthruControl.STR_MVC_CLASS_NAME, sitemapModel);
      ((AActivitySpace) this).SetOverWriteSubSpaces(true);
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.AActivitySpace#GetStyleSheetLinks()"></seealso>
    public override HTMLElementCollection GetStyleSheetLinks()
    {
      String settingLanguage = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE,
       this);
      SiteMapModel myModel = (SiteMapModel) GetModel(SiteMapModel.STR_MVC_CLASS_NAME);
      int nHeaderID = myModel.GetHeaderID(myModel.GetCommunityID());
      String strStyleSheet = "";
      if (nHeaderID > 0)
      {
        strStyleSheet = myModel.GetStyleSheet(myModel.GetCommunityID());
      }
      HTMLElementCollection pCollection = null;
      if (!("").Equals(strStyleSheet) && (strStyleSheet != null))
      {
        if (GetIsAccessStyleNonStandard())
        {
          strStyleSheet = PlumtreeHelpers.STYLE_PREFIX_508 + strStyleSheet;
          }
        PTStyleSheetLink ptStyleSheetLink = new PTStyleSheetLink(strStyleSheet,
         settingLanguage);
        pCollection = new HTMLElementCollection();
        try
        {
          pCollection.AddInnerHTMLElement(ptStyleSheetLink.GetLinkAsHTMLElement(ConfigHelper.GetStyleSheetsBaseURL(this)));
          }
        catch (HTMLException e)
        {
        }
        catch (Exception e)
        {
        }
      } else
      {
        // use the default subportal stylesheet
        pCollection = base.GetStyleSheetLinks();
      }
      return pCollection;
    }
  }
}