using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portalpages.common.search;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.common.documentlanguage
{
  /// <author>
  /// StephenC
  /// This Section represents a section of the Main Settings page
  /// of the Card Editor and Card Submit Editor.
  /// This Section is printed out by CardEditorMainDP.and CardSubmitAdvancedMainDP
  /// </author>
  public class CardLanguagePlugInView : EditorLikeView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalpages.common.documentlanguage.CardLanguagePlugInView");
    public const String STR_MVC_CLASS_NAME = "CardLanguagePlugIn";
    // HTML Object names.
    public const String HTMLSELECT_CARDLANGUAGE = "in_se_CardLanguage";
    public const String HTMLBUTTON_DISPLAYLANGUAGES = "in_bu_DisplayLanguages";
    public const String s_AllLanguagesOptionCode = "xx";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new CardLanguagePlugInView();
    }
    
    /// <summary>
    /// Helper methods that returns the property type name
    /// corresponding to the property type ID passed in argument.
    /// </summary>
    /// <param name="_nPropType"></param>
    /// <returns>String</returns>
    public override HTMLElement Display()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLElementCollection optionCollection = new HTMLElementCollection();
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLSelect mySelect;
      HTMLOption myOption;
      HTMLSpan mySpan;
      try
      {
        String strViewTitle = ((ICardLanguagePlugInModelRO) m_model).GetViewTitle();
        String strInstructionLabel = ((ICardLanguagePlugInModelRO) m_model).GetInstructionLabel();
        HTMLComment myComment = new HTMLComment("Begin: Display CardLanguagePlugInView");
        result.AddInnerHTMLElement(myComment);
        result.AddInnerHTMLElement(base.GetHeaderView(strViewTitle, true));
        myTable = new HTMLTable();
        result.AddInnerHTMLElement(myTable);
        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.SetBorder(CommonHTMLStrings.ZERO);
        myTable.SetCellPadding(CommonHTMLStrings.FIVE);
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        // BEGIN Card Language Row
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetColSpan(CommonHTMLStrings.TWO);
        mySpan = new HTMLSpan();
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.SetStyleClass(PTStyleClass.OBJECT_TEXT);
        mySpan.AddInnerHTMLString(strInstructionLabel);
        mySpan.AddInnerHTMLString(SPACE);
        mySelect = new HTMLSelect(HTMLSELECT_CARDLANGUAGE, "e");
        mySelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
        myCell.AddInnerHTMLElement(mySelect);
        bool bIncludeAllLanguagesOption = ((ICardLanguagePlugInModelRO) m_model).IncludeAllLanguagesOption();
        if (true == bIncludeAllLanguagesOption)
        {
          myOption = new HTMLOption("" + s_AllLanguagesOptionCode + "");
          mySpan = new HTMLSpan();
          myOption.AddInnerHTMLElement(mySpan);
          mySpan.SetStyleClass(PTStyleClass.FORM_SELECT_TEXT);
          mySpan.AddInnerHTMLString(m_asOwner.GetString(30, "ptmsgs_portalcommonmsgs"));
          mySelect.AddInnerHTMLElement(myOption);
          myOption.SetSelected(true);
        }
        String strSelectedLanguageCode = ((ICardLanguagePlugInModelRO) m_model).GetCardLanguageCode();
        String[] arLanguageCodes;
        String[] arLanguageNames;
        arLanguageCodes = ((ICardLanguagePlugInModelRO) m_model).GetAllLanguageCodes();
        arLanguageNames = ((ICardLanguagePlugInModelRO) m_model).GetAllLanguageNames();
        int nLanguageCount = arLanguageCodes.Length;
        // Create an array list so that we can sort it
        XPArrayList xpLanguageNames = new XPArrayList();
        for (int i = 0;i < nLanguageCount;i++)
        {
          String strLanguageName = arLanguageNames[i];
          xpLanguageNames.Add(strLanguageName);
        }
        // Now build the dropdown list
        for (int i = 0;i < nLanguageCount;i++)
        {
          String strLanguageName = (String) xpLanguageNames.GetElement(i);
          int j = 0;
          for (;j < nLanguageCount;j++)
          {
            String strTempLanguageName = arLanguageNames[j];
            if (0 == XPStringUtility.Compare(strLanguageName, strTempLanguageName))
            {
              // found match
              break;
            }
          }
          String strLanguageCode = arLanguageCodes[j];
          // Create the option now that we have the code
          myOption = new HTMLOption("" + strLanguageCode + "");
          mySpan = new HTMLSpan();
          myOption.AddInnerHTMLElement(mySpan);
          mySpan.SetStyleClass(PTStyleClass.FORM_SELECT_TEXT);
          mySpan.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strLanguageName));
          //mySelect.AddInnerHTMLElement(myOption);
          optionCollection.AddInnerHTMLElement(myOption);
          if (0 == XPStringUtility.CompareIgnoreCase(strLanguageCode, strSelectedLanguageCode))
          {
            myOption.SetSelected(true);
          }
        }
        mySelect.AddInnerHTMLElement(optionCollection);
        myCell.AddInnerHTMLString(SPACE);
        myCell.AddInnerHTMLString(SPACE);
        // END Card Language Row
        myComment = new HTMLComment("End: Display CardLanguagePlugInView");
        result.AddInnerHTMLElement(myComment);
      }
      catch (HTMLException he)
      {
        log.Error(he, STR_MVC_CLASS_NAME + "::Display The following error occured when trying to display the HTML: " + he.Message);
        throw new XPException(m_asOwner.GetString(31, "ptmsgs_portalcommonmsgs"),
         he);
      }
      return result;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
  }
}