using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.portaluiinfrastructure.layout;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.common.editpage
{
  /// <author>MichaelD</author>
  public class PageLayoutAlternateView : EditorView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalpages.common.editpage.PageLayoutAlternateView");
    
    /// <summary>
    /// Object Name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "EditPageLayoutAlt";
    
    /// <summary>
    /// Height of the select boxes 
    /// </summary>
    private const int SELECT_BOX_SIZE = 20;
    
    /// <summary>
    /// Width of Layout Section 
    /// </summary>
    private const int LAYOUT_WIDTH = 700;
    
    /// <summary>
    /// Width of Button Column 
    /// </summary>
    private const int BUTTON_COL_WIDTH = 30;
    
    /// <summary>
    /// Action Button Width 
    /// </summary>
    private const String BUTTONS_WIDTH = "20";
    
    /// <summary>
    /// Action Button Height 
    /// </summary>
    private const String BUTTONS_HEIGHT = "20";
    
    /// <summary>
    /// Layout Column Prefix 
    /// </summary>
    private const String LAYOUT_COL_PREFIX = "in_se_LayoutColumn";
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
     */
     
    public override Object Create()
    {
      return new PageLayoutAlternateView();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
     */
     
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
     */
     
    public override HTMLElement Display()
    {
      if (PT_INTRINSICS.PT_BLANK_PAGETEMPLATEID == ((IPageLayoutModelRO) m_model).GetObjectID() && PT_CLASSIDS.PT_PAGETEMPLATE_ID == ((IPageLayoutModelRO) m_model).GetClassID())
      {
        return null;
      }
      HTMLElementCollection collection = new HTMLElementCollection();
      HTMLElement result = collection;
      int nPageType = ((PageLayoutPlugInModel) this.m_model).GetPageType();
      int nColumns = 0;
      int nStartColumn = 0;
      String strColZeroTitle;
      String strColOneTitle;
      String strColTwoTitle = m_asOwner.GetString(732, "ptmsgs_portalcommonmsgs");
      bool bHasContentCanvasSection = false;
      switch (nPageType){
      case PageTypeConstants.ONE_COLUMN_WITH_FREE_FORM_CONTENT:
      {
        bHasContentCanvasSection = true;
        goto case PageTypeConstants.ONE_COLUMN;
      }
      case PageTypeConstants.ONE_COLUMN:
      {
        nColumns = 1;
        nStartColumn = 1;
        strColZeroTitle = m_asOwner.GetString(733, "ptmsgs_portalcommonmsgs");
        strColOneTitle = "";
        break;
      }
      case PageTypeConstants.TWO_COLUMN_NARROW_WIDE_WFFC:
      case PageTypeConstants.TWO_COLUMN_WFFC_NARROW_WIDE:
      case PageTypeConstants.TWO_COLUMN_WFFC_WIDE_NARROW:
      case PageTypeConstants.TWO_COLUMN_WIDE_WFFC_NARROW:
      {
        bHasContentCanvasSection = true;
        goto case PageTypeConstants.TWO_COLUMN_NARROW_WIDE;
      }
      case PageTypeConstants.TWO_COLUMN_NARROW_WIDE:
      case PageTypeConstants.TWO_COLUMN_WIDE_NARROW:
      {
        nColumns = 2;
        strColZeroTitle = m_asOwner.GetString(732, "ptmsgs_portalcommonmsgs");
        strColOneTitle = m_asOwner.GetString(733, "ptmsgs_portalcommonmsgs");
        break;
      }
      case PageTypeConstants.THREE_COLUMN_NARROW_WIDE_WFF_NARROW:
      case PageTypeConstants.THREE_COLUMN_NARROW_WIDENARROW_WFFC:
      case PageTypeConstants.THREE_COLUMN_NARROWWIDE_WFFC_NARROW:
      case PageTypeConstants.THREE_COLUMN_WITH_FREE_FORM_CONTENT:
      {
        bHasContentCanvasSection = true;
        goto case PageTypeConstants.THREE_COLUMN;
      }
      case PageTypeConstants.THREE_COLUMN:
      {
        nColumns = 3;
        strColZeroTitle = m_asOwner.GetString(732, "ptmsgs_portalcommonmsgs");
        strColOneTitle = m_asOwner.GetString(733, "ptmsgs_portalcommonmsgs");
        break;
      }
      default:
      {
        // adaptive layout stores a negative value in the page type (negative value of a webservice layout id
        if (nPageType < 0)
        {
          // this covers the case if there is a layout id set AND portlet layouts are disabled ->
          // revert to a regular 3 column layout in the view code
          if (!LayoutService.getChooserInstance().GetPortletLayoutsEnabled(m_asOwner))
          {
            nColumns = 3;
            strColZeroTitle = m_asOwner.GetString(732, "ptmsgs_portalcommonmsgs");
            strColOneTitle = m_asOwner.GetString(733, "ptmsgs_portalcommonmsgs");
            } else
          {
            nColumns = 3;
            String strRegion = m_asOwner.GetString(800, "ptmsgs_portalcommonmsgs");
            strColTwoTitle = XPResourceManager.FormatMessage(strRegion, 3);
            strColZeroTitle = XPResourceManager.FormatMessage(strRegion, 1);
            strColOneTitle = XPResourceManager.FormatMessage(strRegion, 2);
            }
        } else
        {
          throw new XPIllegalStateException("Unknown page type: " + nPageType + " found on page with id: " + ((IPageLayoutModelRO) m_model).GetObjectID());
          }
        break;
      }
      }
      int nColumnWidth = (LAYOUT_WIDTH / nColumns);
      HTMLTable myTable = new HTMLTable();
      collection.AddInnerHTMLElement(myTable);
      myTable.SetWidth(XPConvert.ToString(LAYOUT_WIDTH));
      myTable.SetBorder(CommonHTMLStrings.ZERO);
      myTable.SetCellPadding(CommonHTMLStrings.ZERO);
      myTable.SetCellSpacing("2");
      myTable.SetAlign(CommonHTMLStrings.CENTER);
      HTMLTableRow myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      if (bHasContentCanvasSection)
      {
        HTMLTableCell myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetVAlign(CommonHTMLStrings.TOP);
        myCell.SetAlign(CommonHTMLStrings.CENTER);
        myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myCell.SetColSpan(XPConvert.ToString(nColumns));
        HTMLTable innerTable = new HTMLTable();
        myCell.AddInnerHTMLElement(innerTable);
        innerTable.SetBorder(CommonHTMLStrings.ZERO);
        innerTable.SetCellPadding("5");
        innerTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        innerTable.SetStyleClass(PTStyleClass.PAGELAYOUT_CONTENT_BG);
        innerTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        // Title Row
        HTMLTableRow innerRow = new HTMLTableRow();
        innerTable.AddInnerHTMLElement(innerRow);
        HTMLTableCell innerCell = new HTMLTableCell();
        innerRow.AddInnerHTMLElement(innerCell);
        innerCell.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
        innerCell.SetAlign(CommonHTMLStrings.CENTER);
        innerCell.AddInnerHTMLString(m_asOwner.GetString(734, "ptmsgs_portalcommonmsgs"));
        // Layout Row
        innerRow = new HTMLTableRow();
        innerTable.AddInnerHTMLElement(innerRow);
        innerCell = new HTMLTableCell();
        innerRow.AddInnerHTMLElement(innerCell);
        innerCell.SetVAlign(CommonHTMLStrings.TOP);
        innerCell.SetAlign(CommonHTMLStrings.CENTER);
        HTMLSpan mySpan = new HTMLSpan();
        innerCell.AddInnerHTMLElement(mySpan);
        mySpan.SetStyleClass(PTStyleClass.CUSTOMAPP_TEXT);
        if (((IPageLayoutModelRO) m_model).GetHasFreeFormPortlet())
        {
          mySpan.AddInnerHTMLEncodedString(((IPageLayoutModelRO) m_model).GetPortletName(-1));
          innerCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
          if (!((IPageLayoutModelRO) m_model).GetPortletIsFromPT(-1))
          {
            // remove button
            HTMLAnchor myAnchor = ConstructStaticHelpers.GetJavascriptAnchor("",
             PageLayoutHeaderView.JAVASCRIPT_DELETE_GADGET + "(\'" + ((IPageLayoutModelRO) m_model).GetPortletID(-1) + "\')");
            innerCell.AddInnerHTMLElement(myAnchor);
            myAnchor.SetTitle(m_asOwner.GetString(723, "ptmsgs_portalcommonmsgs"));
            PTConstruct myConstruct = UIConstructFactory.GetConstruct(PTImageType.ACTION_DELETE_BTN,
             true, this.m_asOwner);
            HTMLImg myImg = (HTMLImg) myConstruct.GetHTMLElement();
            myAnchor.AddInnerHTMLElement(myImg);
            myImg.SetWidth(BUTTONS_WIDTH);
            myImg.SetHeight(BUTTONS_HEIGHT);
            myImg.SetAlt(m_asOwner.GetString(723, "ptmsgs_portalcommonmsgs"));
            myImg.SetBorder(CommonHTMLStrings.ZERO);
          }
        } else
        {
          mySpan.AddInnerHTMLString(m_asOwner.GetString(735, "ptmsgs_portalcommonmsgs"));
          }
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
      }
      int i = 0;
      for (int nColumnIndex = nStartColumn;nColumnIndex < nColumns + nStartColumn;
      nColumnIndex++)
      {
        if (PageTypeConstants.TWO_COLUMN_WFFC_WIDE_NARROW == nPageType || PageTypeConstants.TWO_COLUMN_WIDE_NARROW == nPageType || PageTypeConstants.TWO_COLUMN_WIDE_WFFC_NARROW == nPageType)
        {
          if (0 == nColumnIndex)
          {
            i = 1;
          } else
          {
            i = 0;
          }
        } else
        {
          i = nColumnIndex;
        }
        // the select box
        HTMLTableCell myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetVAlign(CommonHTMLStrings.TOP);
        myCell.SetAlign(CommonHTMLStrings.RIGHT);
        myCell.SetWidth(XPConvert.ToString(nColumnWidth));
        HTMLTable innerTable = new HTMLTable();
        myCell.AddInnerHTMLElement(innerTable);
        innerTable.SetBorder(CommonHTMLStrings.ZERO);
        innerTable.SetCellPadding("5");
        innerTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        innerTable.SetStyleClass(PTStyleClass.PAGELAYOUT_COLUMN_BG);
        // Title Row
        HTMLTableRow innerRow = new HTMLTableRow();
        innerTable.AddInnerHTMLElement(innerRow);
        String strTitle;
        switch (i){
        case 0:
        {
          strTitle = strColZeroTitle;
          break;
        }
        case 1:
        {
          strTitle = strColOneTitle;
          break;
        }
        case 2:
        {
          strTitle = strColTwoTitle;
          break;
        }
        default:
        strTitle = "";
        break;
        }
        HTMLTableCell innerCell = new HTMLTableCell();
        innerRow.AddInnerHTMLElement(innerCell);
        innerCell.SetColSpan("2");
        innerCell.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
        innerCell.SetAlign(CommonHTMLStrings.CENTER);
        innerCell.AddInnerHTMLString(strTitle);
        // Layout Row
        innerRow = new HTMLTableRow();
        innerTable.AddInnerHTMLElement(innerRow);
        innerCell = new HTMLTableCell();
        innerRow.AddInnerHTMLElement(innerCell);
        innerCell.SetVAlign(CommonHTMLStrings.TOP);
        innerCell.SetAlign(CommonHTMLStrings.CENTER);
        innerCell.SetWidth(XPConvert.ToString(nColumnWidth - BUTTON_COL_WIDTH));
        HTMLSelect mySelect = new HTMLSelect(LAYOUT_COL_PREFIX + i, LAYOUT_COL_PREFIX + i);
        innerCell.AddInnerHTMLElement(mySelect);
        mySelect.SetSize(SELECT_BOX_SIZE);
        mySelect.SetOnChange("deselectOtherColumns(" + i + ")");
        BrowserType browser = (BrowserType) m_asOwner.GetSubSession().GetAttribute(Interpreter.BROWSERTYPE);
        if (!browser.IsNetscape4())
        {
          mySelect.SetStyle("width:" + (nColumnWidth - BUTTON_COL_WIDTH));
          } else
        {
          // sizer option for netscape4
          HTMLOption myOption = new HTMLOption("");
          mySelect.AddInnerHTMLElement(myOption);
          myOption.SetName("xxxxxxxxxxxxxxxxxxx");
        }
        // the positioning buttons
        innerCell = new HTMLTableCell();
        innerRow.AddInnerHTMLElement(innerCell);
        innerCell.SetVAlign(CommonHTMLStrings.TOP);
        innerCell.SetAlign(CommonHTMLStrings.LEFT);
        innerCell.SetWidth(XPConvert.ToString(BUTTON_COL_WIDTH));
        // up
        HTMLAnchor myAnchor = ConstructStaticHelpers.GetJavascriptAnchor("",
         "PortletPosition.movePortlet(" + i + ",\'up\')");
        innerCell.AddInnerHTMLElement(myAnchor);
        myAnchor.SetTitle(m_asOwner.GetString(719, "ptmsgs_portalcommonmsgs"));
        PTConstruct myConstruct = UIConstructFactory.GetConstruct(PTImageType.ACTION_ARROW_UP_BTN,
         true, this.m_asOwner);
        HTMLImg myImg = (HTMLImg) myConstruct.GetHTMLElement();
        myAnchor.AddInnerHTMLElement(myImg);
        myImg.SetWidth(BUTTONS_WIDTH);
        myImg.SetHeight(BUTTONS_HEIGHT);
        myImg.SetAlt(m_asOwner.GetString(719, "ptmsgs_portalcommonmsgs"));
        myImg.SetBorder(CommonHTMLStrings.ZERO);
        HTMLBr br = new HTMLBr();
        innerCell.AddInnerHTMLElement(br);
        String strMove;
        // right
        if (nColumnIndex < 2 && nColumns > 1 && nColumnIndex != (nColumns - 1))
        {
          if (i == nColumnIndex)
          {
            strMove = "right";
          } else
          {
            strMove = "left";
          }
          myAnchor = ConstructStaticHelpers.GetJavascriptAnchor("", "PortletPosition.movePortlet(" + i + ",\'" + strMove + "\')");
          innerCell.AddInnerHTMLElement(myAnchor);
          myAnchor.SetTitle(m_asOwner.GetString(722, "ptmsgs_portalcommonmsgs"));
          myConstruct = UIConstructFactory.GetConstruct(PTImageType.ACTION_ARROW_RIGHT_BTN,
           true, this.m_asOwner);
          myImg = (HTMLImg) myConstruct.GetHTMLElement();
          myAnchor.AddInnerHTMLElement(myImg);
          myImg.SetWidth(BUTTONS_WIDTH);
          myImg.SetHeight(BUTTONS_HEIGHT);
          myImg.SetAlt(m_asOwner.GetString(722, "ptmsgs_portalcommonmsgs"));
          myImg.SetBorder(CommonHTMLStrings.ZERO);
          innerCell.AddInnerHTMLElement(br);
        }
        // left
        if (nColumnIndex > 0 && nColumns > 1)
        {
          if (i == nColumnIndex)
          {
            strMove = "left";
          } else
          {
            strMove = "right";
          }
          myAnchor = ConstructStaticHelpers.GetJavascriptAnchor("", "PortletPosition.movePortlet(" + i + ",\'" + strMove + "\')");
          innerCell.AddInnerHTMLElement(myAnchor);
          myAnchor.SetTitle(m_asOwner.GetString(721, "ptmsgs_portalcommonmsgs"));
          myConstruct = UIConstructFactory.GetConstruct(PTImageType.ACTION_ARROW_LEFT_BTN,
           true, this.m_asOwner);
          myImg = (HTMLImg) myConstruct.GetHTMLElement();
          myAnchor.AddInnerHTMLElement(myImg);
          myImg.SetWidth(BUTTONS_WIDTH);
          myImg.SetHeight(BUTTONS_HEIGHT);
          myImg.SetAlt(m_asOwner.GetString(721, "ptmsgs_portalcommonmsgs"));
          myImg.SetBorder(CommonHTMLStrings.ZERO);
          innerCell.AddInnerHTMLElement(br);
        }
        // down
        myAnchor = ConstructStaticHelpers.GetJavascriptAnchor("", "PortletPosition.movePortlet(" + i + ",\'down\')");
        innerCell.AddInnerHTMLElement(myAnchor);
        myAnchor.SetTitle(m_asOwner.GetString(720, "ptmsgs_portalcommonmsgs"));
        myConstruct = UIConstructFactory.GetConstruct(PTImageType.ACTION_ARROW_DOWN_BTN,
         true, this.m_asOwner);
        myImg = (HTMLImg) myConstruct.GetHTMLElement();
        myAnchor.AddInnerHTMLElement(myImg);
        myImg.SetWidth(BUTTONS_WIDTH);
        myImg.SetHeight(BUTTONS_HEIGHT);
        myImg.SetAlt(m_asOwner.GetString(720, "ptmsgs_portalcommonmsgs"));
        myImg.SetBorder(CommonHTMLStrings.ZERO);
        innerCell.AddInnerHTMLElement(br);
        // delete
        myAnchor = ConstructStaticHelpers.GetJavascriptAnchor("", "PortletPosition.deletePortlet(" + i + ")");
        innerCell.AddInnerHTMLElement(myAnchor);
        myAnchor.SetTitle(m_asOwner.GetString(723, "ptmsgs_portalcommonmsgs"));
        myConstruct = UIConstructFactory.GetConstruct(PTImageType.ACTION_DELETE_BTN,
         true, this.m_asOwner);
        myImg = (HTMLImg) myConstruct.GetHTMLElement();
        myAnchor.AddInnerHTMLElement(myImg);
        myImg.SetWidth(BUTTONS_WIDTH);
        myImg.SetHeight(BUTTONS_HEIGHT);
        myImg.SetAlt(m_asOwner.GetString(723, "ptmsgs_portalcommonmsgs"));
        myImg.SetBorder(CommonHTMLStrings.ZERO);
      }
      XPStringBuilder sb = new XPStringBuilder(500);
      // Portlets
      String strAlignment = "";
      int nColumn;
      int nPortletID;
      String strPortletName;
      String strNameSuffix;
      bool bIsMandatory;
      int nRowID;
      int nPortletsInColumnZero = 0;
      int nPortletsInColumnOne = 0;
      int nPortletsInColumnTwo = 0;
      for (int nPortletIndex = 0;nPortletIndex < ((IPageLayoutModelRO) m_model).GetPortletCount();
      nPortletIndex++)
      {
        if (((IPageLayoutModelRO) m_model).GetPortletAlignment(nPortletIndex) == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NARROW)
        {
          strAlignment = "narrow";
          // Can't use CommonHTMLStrings.SPACE here because this gets passed to javascript.
          strNameSuffix = " " + m_asOwner.GetString(729, "ptmsgs_portalcommonmsgs");
          } else
        {
          strAlignment = "wide";
          // Can't use CommonHTMLStrings.SPACE here because this gets passed to javascript.
          strNameSuffix = " " + m_asOwner.GetString(730, "ptmsgs_portalcommonmsgs");
          }
        nColumn = ((IPageLayoutModelRO) m_model).GetPortletColumn(nPortletIndex);
        nPortletID = ((IPageLayoutModelRO) m_model).GetPortletID(nPortletIndex);
        strPortletName = GeneralHelpers.TruncateLongString(((IPageLayoutModelRO) m_model).GetPortletName(nPortletIndex),
         20, false);
        bIsMandatory = ((IPageLayoutModelRO) m_model).GetPortletIsFromPT(nPortletIndex);
        if (bIsMandatory)
        {
          strNameSuffix = " " + m_asOwner.GetString(731, "ptmsgs_portalcommonmsgs");
          }
        if (nColumn == PortletColumnConstants.NARROW_ONE.ToInt())
        {
          nRowID = nPortletsInColumnZero;
          nPortletsInColumnZero++;
        } else
        if (nColumn == PortletColumnConstants.WIDE_ONE.ToInt())
        {
          nRowID = nPortletsInColumnOne;
          nPortletsInColumnOne++;
        } else
        {
          nRowID = nPortletsInColumnTwo;
          nPortletsInColumnTwo++;
        }
        sb.Append("PortletPosition.addPortlet(" + nPortletID + ", \'" + JSHelper.JSEncodeString(strPortletName) + strNameSuffix + "\', \'" + strAlignment + "\', " + nColumn + ", " + nRowID + ", " + XPConvert.ToString(bIsMandatory) + ");\n");
        }
      sb.Append("redrawAllColumns();\n");
      HTMLScript myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      collection.AddInnerHTMLElement(myScript);
      try
      {
        myScript.AddInnerHTMLString(sb.ToString());
      }
      catch (HTMLException he)
      {
        log.Error(he, "PageListAlertnateView::Display - The following error occured when trying to display the JavaScript: " + he.Message);
        }
      return result;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
     */
     
    public virtual HTMLScriptCollection DisplayJavascriptCollection()
    {
      HTMLScriptCollection myScriptCollection = new HTMLScriptCollection();
      XPStringBuilder sb = new XPStringBuilder(500);
      // Initialization and configuration parameters
      // Page Layout Hidden Input Name
      sb.Append("var strPageLayoutTypeElementName =  \"").Append(PageLayoutHeaderView.HTMLINPUT_EDITPAGE_PAGELAYOUTTYPE).Append("\";\n").
      // Page Type
      Append("var intPageType\t\t= ").Append(((IPageLayoutModelRO) m_model).GetPageType()).Append(";\n").
      // Strings
      Append("var MSG_REMOVE_CONF\t\t= \"" + m_asOwner.GetString(724, "ptmsgs_portalcommonmsgs") + "\";\n").
      // Portlet Order Prefix
      Append("var PORTLET_ORDER_PREFIX\t\t= \"" + PageLayoutView.HTMLINPUT_EDITPAGE_GADGETORDER_PREFIX + "\";\n").
      // Layout Column Prefix
      Append("var LAYOUT_COL_PREFIX\t\t= \'" + LAYOUT_COL_PREFIX + "\';\n");
      HTMLScript myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      myScriptCollection.AddInnerHTMLElement(myScript);
      try
      {
        myScript.AddInnerHTMLString(sb.ToString());
      }
      catch (HTMLException he)
      {
        log.Error(he, "PageListAlertnateView::DisplayJavascript The following error occured when trying to display the JavaScript: " + he.Message);
        }
      myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      myScriptCollection.AddInnerHTMLElement(myScript);
      myScript.SetSrc(ConfigHelper.GetPortalImageServerJavaScriptURL(m_asOwner) + "alternatechangelayout.js");
      return myScriptCollection;
    }
  }
}