using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.admin.editors.pagetemplate;
using com.plumtree.portalpages.admin.editors.portlet;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.portalpages.browsing.objectselection;
using com.plumtree.portalpages.browsing.objectselection.portlets;
using LayoutService=com.plumtree.portaluiinfrastructure.layout.LayoutService;
using com.plumtree.portaluiinfrastructure.resultwrapper;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tree;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.tree;
namespace com.plumtree.portalpages.common.editpage
{
  /// <summary>
  /// This Model contains the state of the plug-in section.
  /// </summary>
  /// <author>MichaelD</author>
  public class PageLayoutPlugInModel : EditorHelperModel,IPageLayoutModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalpages.common.editpage.PageLayoutPlugInModel");
    
    /// <summary>
    /// Object Name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "PageLayoutPlugIn";
    
    /// <summary>
    /// Key name used by the Activity Space to store the add gadgets page. 
    /// </summary>
    public const String SUBSPACE_NAME_ADDGADGETS = "AddPortlets";
    
    /// <summary>
    /// Key name used by the Activity Space to store the add free form page. 
    /// </summary>
    public const String SUBSPACE_NAME_ADDFREEFORM = "FreeForm";
    
    /// <summary>
    /// Key name used by the Activity Space to store the select community template tree. 
    /// </summary>
    public const String SUBSPACE_NAME_SELECTLAYOUT = "SelectLayout";
    
    /// <summary>
    /// Sorted Array Index Column 
    /// </summary>
    public const int PAGELAYOUT_SORTEDARRAY_PROPID_INDEX = 0;
    
    /// <summary>
    /// Portlet Model Column 
    /// </summary>
    public const int PAGELAYOUT_COL_GADGET_MODEL = 1;
    
    /// <summary>
    /// Selected Portlet Column Count 
    /// </summary>
    public const int COL_GADGET_MAX = 4;
    
    /// <summary>
    /// Selected Portlet Index Column 
    /// </summary>
    public const int COL_GADGET_INDEX = 0;
    
    /// <summary>
    /// Selected Portlet Name Column 
    /// </summary>
    public const int COL_GADGET_NAME = 1;
    
    /// <summary>
    /// Selected Portlet Alignment Column 
    /// </summary>
    public const int COL_GADGET_ALIGNMENT = 2;
    
    /// <summary>
    /// Selected Portlet Query Index 
    /// </summary>
    public const int COL_GADGET_QUERY_INDEX = 3;
    
    /// <summary>
    /// Portlet Order in Columns 
    /// </summary>
    protected int[][] m_aanPortletOrder = null;
    
    /// <summary>
    /// IPTSession 
    /// </summary>
    private IPTSession m_ptSession;
    
    /// <summary>
    /// Portlets 
    /// </summary>
    private PTGrowableSortedArrayWrapper m_arrPortlets;
    
    /// <summary>
    /// Portlet Column Count 
    /// </summary>
    private PortletColumnCountConstants m_PortletColumnCount = null;
    
    /// <summary>
    /// Free Form Portlet 
    /// </summary>
    private PortletData m_gmFreeForm;
    
    /// <summary>
    /// Free Form to be removed
    /// </summary>
    private int m_nRemoveFreeForm = -1;
    
    /// <summary>
    /// Edit Page Name 
    /// </summary>
    private String m_strPageName;
    
    /// <summary>
    /// Space IDs 
    /// </summary>
    private XPHashtable m_htFormData;
    
    /// <summary>
    /// New Selected Objects 
    /// </summary>
    private XPSortedDictionary m_xpsdNewSelectedObjects;
    
    /// <summary>
    /// Old Selected Objects 
    /// </summary>
    private XPSortedDictionary m_xpsdOldSelectedObjects;
    private XPHashtable m_xpMandatoryPortlets;
    
    /// <summary>
    /// Index of first gadgets in each column 
    /// </summary>
    private int[] m_arFirstPortletIndex = null;
    
    /// <summary>
    /// Index of last gadgets in each column 
    /// </summary>
    private int[] m_arLastPortletIndex = null;
    
    /// <summary>
    /// Portlet ID to delete 
    /// </summary>
    private int m_nDeletePortletID;
    
    /// <summary>
    /// Mandatory Portlet Count 
    /// </summary>
    private int m_nMandPortletCount;
    
    /// <summary>
    /// Class ID 
    /// </summary>
    private int m_nClassID = 0;
    
    /// <summary>
    /// Object ID 
    /// </summary>
    private int m_nObjectID = 0;
    
    /// <summary>
    /// Folder ID this object is stored in 
    /// </summary>
    private int m_nFolderID;
    
    /// <summary>
    /// Is home my page 
    /// </summary>
    private bool m_bIsHomeMyPage = false;
    
    /// <summary>
    /// Allow Free Form Section 
    /// </summary>
    private bool m_bAllowFreeFormSection = true;
    
    /// <summary>
    /// Page Layout Type 
    /// </summary>
    private int m_nPageLayoutType;
    
    /// <summary>
    /// Page Template ID 
    /// </summary>
    private int m_nPageTemplateID;
    
    /// <summary>
    /// Inherit Page Template 
    /// </summary>
    private bool m_bInheritPageTemplate = false;
    
    /// <summary>
    /// Page Template Changed 
    /// </summary>
    private bool m_bPageTemplateChanged = false;
    
    /// <summary>
    /// Drag and Drop Layout Table Height 
    /// </summary>
    private int m_nTableHeight;
    
    /// <summary>
    /// Page has freeform portlet on startup *
    /// </summary>
    private bool m_bHasInitialFreeForm = false;
    public PageLayoutPlugInModel() : base()
    {
      m_PortletColumnCount = PortletColumnCountConstants.NONE;
      m_aanPortletOrder = new int[PortletColumnConstants.NARROW_TWO.ToInt() + 1][];
      }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new PageLayoutPlugInModel();
    }
    public virtual void DoTaskBeforeStore(IPTPage page)
    {
      if (!m_bInheritPageTemplate)
      {
        page.SetPageType(m_nPageLayoutType);
      }
    }
    public virtual void DoTaskAfterStore(IPTPage page)
    {
      for (int i = PortletColumnConstants.NARROW_ONE.ToInt();i <= PortletColumnConstants.NARROW_TWO.ToInt();
      i++)
      {
        page.AssignPortlets(m_aanPortletOrder[i], i);
      }
      if (m_nRemoveFreeForm != -1)
      {
        page.RemovePortlet(m_nRemoveFreeForm);
      }
      if ((null != m_gmFreeForm) && (0 < m_gmFreeForm.m_nPortletID) && !m_gmFreeForm.m_bFromPageTemplate)
      {
        try
        {
          page.AddPortlet(m_gmFreeForm.m_nPortletID, -1);
        }
        catch (Exception e)
        // DCA - already added freeform gadget
        {
        }
      }
    }
    // DCA - Don't really have to do anything
    public virtual void DoTaskBeforeStore(IPTPageTemplate pageTemplate)
    {
      pageTemplate.SetPageType(m_nPageLayoutType);
    }
    public virtual void DoTaskAfterStore(IPTPageTemplate pageTemplate)
    {
      for (int i = PortletColumnConstants.NARROW_ONE.ToInt();i <= PortletColumnConstants.NARROW_TWO.ToInt();
      i++)
      {
        pageTemplate.AssignGadgetsEx(m_aanPortletOrder[i], i);
      }
      if (m_nRemoveFreeForm != -1)
      {
        pageTemplate.RemoveGadget(m_nRemoveFreeForm);
      }
      if ((null != m_gmFreeForm) && (0 < m_gmFreeForm.m_nPortletID))
      {
        try
        {
          pageTemplate.AddGadgetEx(m_gmFreeForm.m_nPortletID, -1);
        }
        catch (Exception e)
        // DCA - already added freeform gadget
        {
        }
      }
    }
    // DCA - Don't really have to do anything
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetCanChangeLayout()"></seealso>
    public virtual bool GetCanChangeLayout()
    {
      if (m_bInheritPageTemplate && PT_INTRINSICS.PT_BLANK_PAGETEMPLATEID < m_nPageTemplateID)
      {
        return false;
      }
      return true;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetCanChooseFreeForm()"></seealso>
    public virtual bool GetCanChooseFreeForm()
    {
      if (PageTypeConstants.ONE_COLUMN_WITH_FREE_FORM_CONTENT <= m_nPageLayoutType)
      {
        // SE: Bug 61276 - Fix for Vodafone/USDHHS. Don't enable "Add Content Canvas" button if we already have a 
        // content canvas portlet on the page.
        if (null != m_gmFreeForm)
        {
          if (0 < m_gmFreeForm.m_nPortletID)
          {
            log.Debug("PageLayoutPlugInModel::GetCanChooseFreeForm - freeform portlet id: " + m_gmFreeForm.m_nPortletID + " exists on page");
            return false;
          }
        }
        if (PT_CLASSIDS.PT_PAGETEMPLATE_ID == m_nClassID)
        {
          if (PT_INTRINSICS.PT_BLANK_PAGETEMPLATEID == m_nPageTemplateID)
          {
            return false;
          }
          return true;
        }
        if ((null != m_gmFreeForm) && m_gmFreeForm.m_bFromPageTemplate)
        {
          return false;
        }
        return true;
      }
      return false;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetCanCreatePortlet()"></seealso>
    public virtual bool GetCanCreatePortlet()
    {
      if ((m_asOwner.GetName()).Equals(PageTemplateEditorAS.STR_MVC_CLASS_NAME) || (0 == m_nPageTemplateID))
      {
        return false;
      }
      IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
      return ptSession.GetSessionInfo().CheckForActivityRight(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_PORTLETS);
      }
    
    /// <summary>
    /// Method GetColumnOrder.
    /// Get the gadget order of the columns.
    /// </summary>
    /// <returns>int[][]</returns>
    public virtual int[][] GetColumnOrder()
    {
      return m_aanPortletOrder;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetPortletAlignment(int)"></seealso>
    public virtual int GetPortletAlignment(int nPortletIndex)
    {
      if (-1 == nPortletIndex)
      {
        return m_gmFreeForm.m_nPortletAlignment;
      }
      return ((PortletData) m_arrPortlets.GetItem(nPortletIndex, PAGELAYOUT_COL_GADGET_MODEL)).m_nPortletAlignment;
      }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetPortletColumn(int)"></seealso>
    public virtual int GetPortletColumn(int nPortletIndex)
    {
      if (-1 == nPortletIndex)
      {
        return m_gmFreeForm.m_nPortletColumn;
      }
      return ((PortletData) m_arrPortlets.GetItem(nPortletIndex, PAGELAYOUT_COL_GADGET_MODEL)).m_nPortletColumn;
      }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetPortletCount()"></seealso>
    public virtual int GetPortletCount()
    {
      return m_arrPortlets.GetCount();
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetPortletDescription(int)"></seealso>
    public virtual String GetPortletDescription(int nPortletIndex)
    {
      if (-1 == nPortletIndex)
      {
        return m_gmFreeForm.m_strPortletDescription;
      }
      return ((PortletData) m_arrPortlets.GetItem(nPortletIndex, PAGELAYOUT_COL_GADGET_MODEL)).m_strPortletDescription;
      }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetPortletID(int)"></seealso>
    public virtual int GetPortletID(int nPortletIndex)
    {
      if (-1 == nPortletIndex)
      {
        return m_gmFreeForm.m_nPortletID;
      }
      return ((PortletData) m_arrPortlets.GetItem(nPortletIndex, PAGELAYOUT_COL_GADGET_MODEL)).m_nPortletID;
      }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetPortletIsCommunityType(int)"></seealso>
    public virtual bool GetPortletIsCommunityType(int nPortletIndex)
    {
      if (-1 == nPortletIndex)
      {
        return m_gmFreeForm.m_bIsCommunityPortlet;
      }
      return ((PortletData) m_arrPortlets.GetItem(nPortletIndex, PAGELAYOUT_COL_GADGET_MODEL)).m_bIsCommunityPortlet;
      }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetPortletIsFromPT(int)"></seealso>
    public virtual bool GetPortletIsFromPT(int nPortletIndex)
    {
      if (-1 == nPortletIndex)
      {
        return m_gmFreeForm.m_bFromPageTemplate;
      }
      return ((PortletData) m_arrPortlets.GetItem(nPortletIndex, PAGELAYOUT_COL_GADGET_MODEL)).m_bFromPageTemplate;
      }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetPortletName(int)"></seealso>
    public virtual String GetPortletName(int nPortletIndex)
    {
      if (-1 == nPortletIndex)
      {
        return m_gmFreeForm.m_strPortletName;
      }
      return ((PortletData) m_arrPortlets.GetItem(nPortletIndex, PAGELAYOUT_COL_GADGET_MODEL)).m_strPortletName;
      }
    
    /// <summary>
    /// Method GetPortletsToHide.
    /// </summary>
    /// <returns>XPSortedDictionary</returns>
    public virtual XPSortedDictionary GetPortletsToHide()
    {
      XPSortedDictionary xpsd = new XPSortedDictionary();
      PortletData gmTemp;
      for (int i = 0;i < m_arrPortlets.GetCount();i++)
      {
        gmTemp = (PortletData) m_arrPortlets.GetItem(i, PAGELAYOUT_COL_GADGET_MODEL);
        if (gmTemp.m_bFromPageTemplate)
        {
          xpsd.PutElement(gmTemp.m_nPortletID, "");
        }
      }
      return xpsd;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetHasFreeFormPortlet()"></seealso>
    public virtual bool GetHasFreeFormPortlet()
    {
      if ((null != m_gmFreeForm) && (0 < m_gmFreeForm.m_nPortletID))
      {
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// DCA - Returns the freeform portlet's ID. Must call GetHasFreeFormPortlet() before calling this function.
    /// </summary>
    public virtual int GetFreeFormPortletID()
    {
      return m_gmFreeForm.m_nPortletID;
    }
    
    /// <summary>
    /// DCA - Returns the freeform portlet's ID if it has been removed. Returns -1 if no FreeForm to be remove.
    /// </summary>
    public virtual int FreeFormToBeRemove()
    {
      return m_nRemoveFreeForm;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetInputValue(String)"></seealso>
    public virtual String GetInputValue(String name)
    {
      String value = (String) m_htFormData.GetElement(name);
      if (value == null)
      {
        return "";
      }
      return value;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#IsFreeFormLayoutAllowed()"></seealso>
    public virtual bool IsFreeFormLayoutAllowed()
    {
      return m_bAllowFreeFormSection;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Method GetNewPortletID.
    /// Get the New Portlet's ID.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetNewPortletID()
    {
      int gadgetID = 0;
      PortMainPortletEditorAS portletAS = (PortMainPortletEditorAS) m_asOwner.GetSubSpace(PageLayoutHeaderView.SUBSPACE_NAME_CREATEPORTLET);
      if (portletAS != null)
      {
        PortletModel portletM = (PortletModel) portletAS.GetModel(PortletModel.STR_MVC_CLASS_NAME);
        gadgetID = portletM.GetObjectID();
        m_asOwner.DeleteSubSpace(PageLayoutHeaderView.SUBSPACE_NAME_CREATEPORTLET);
        }
      return gadgetID;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetOpenerClassID()"></seealso>
    public virtual int GetClassID()
    {
      return m_nClassID;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetOpenerCommunityID()"></seealso>
    public virtual int GetFolderID()
    {
      return m_nFolderID;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetOpenerObjectID()"></seealso>
    public virtual int GetObjectID()
    {
      return m_nObjectID;
    }
    
    /// <summary>
    /// Method GetPageModel.
    /// Get the page model of the page.
    /// </summary>
    /// <returns>PageData</returns>
    public virtual PageData GetPageModel()
    {
      PageData editPageModel = new PageData(m_nPageLayoutType, m_strPageName);
      editPageModel.m_nPortletCount = m_arrPortlets.GetCount();
      editPageModel.m_arrPortlets = m_arrPortlets;
      editPageModel.m_bPortletsSetUp = true;
      editPageModel.m_gmFreeForm = m_gmFreeForm;
      return editPageModel;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetPageType()"></seealso>
    public virtual int GetPageType()
    {
      return m_nPageLayoutType;
    }
    // ***CHANGE_HERE*** Put your set/get methods and other helper methods
    // BEGIN ***************** helper methods ***************** 
    
    /// <summary>
    /// Method GetSelectedPortlets.
    /// Gets the selected gadgets in a dictionary.
    /// </summary>
    /// <returns>XPSortedDictionary</returns>
    public virtual XPSortedDictionary GetSelectedPortlets()
    {
      return m_xpsdNewSelectedObjects;
    }
    
    /// <seealso cref="com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetTableHeight()"></seealso>
    public virtual int GetTableHeight()
    {
      return m_nTableHeight;
    }
    
    /// <summary>
    /// Method InitializePlugIn.
    /// To initialize this plug-in you need to set the DP name  and the default
    /// value of the plug-in setting. Called when creating a new page template or opening one to edit.
    /// Called when creating or editing a my page.
    /// </summary>
    /// <param name="_nPageLayoutType"></param>
    /// <param name="_nPageTemplateID"></param>
    /// <param name="_nTableHeight"></param>
    /// <param name="_ptqrPortlets"></param>
    /// <param name="_Session"></param>
    public virtual void InitializePlugIn(IPTPage ptPage, int _nTableHeight,
     IASQueryResult _ptqrPortlets, bool bIgnorePortletFromPageTemplate, IPTSession _Session)
    {
      // Setup hash map with names and values of form inputs
      // Currently this can only be used by the pages list view, 
      // but it could easily be extended for use by other views.
      m_htFormData = new XPHashtable();
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_ADDGADGETASID,
       "");
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_NEWGADGETASID,
       "");
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_ADDFREEFORMASID,
       "");
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_SELECTLAYOUTASID,
       "");
      m_nTableHeight = _nTableHeight;
      m_ptSession = _Session;
      m_nFolderID = ptPage.GetAdminFolderID();
      m_nPageTemplateID = ptPage.GetPageTemplateID();
      m_bInheritPageTemplate = ptPage.GetInheritPageTemplate();
      SetPageLayoutType(ptPage.GetPageType());
      // setup the gadgets
      SetupPortletsOnStart(_ptqrPortlets, bIgnorePortletFromPageTemplate);
      }
    
    /// <summary>
    /// Method InitializePlugIn.
    /// To initialize this plug-in you need to set the DP name  and the default
    /// value of the plug-in setting. Called when creating a new page template or opening one to edit.
    /// Called when creating or editing a my page.
    /// </summary>
    /// <param name="_nPageLayoutType"></param>
    /// <param name="_nPageTemplateID"></param>
    /// <param name="_nTableHeight"></param>
    /// <param name="_ptqrPortlets"></param>
    /// <param name="_Session"></param>
    public virtual void InitializePlugIn(IPTPageTemplate ptPageTemplate, int _nTableHeight,
     IASQueryResult _ptqrPortlets, IPTSession _Session)
    {
      // Setup hash map with names and values of form inputs
      // Currently this can only be used by the pages list view, 
      // but it could easily be extended for use by other views.
      m_htFormData = new XPHashtable();
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_ADDGADGETASID,
       "");
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_NEWGADGETASID,
       "");
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_ADDFREEFORMASID,
       "");
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_SELECTLAYOUTASID,
       "");
      m_nTableHeight = _nTableHeight;
      m_ptSession = _Session;
      m_nPageTemplateID = 0;
      m_bInheritPageTemplate = false;
      SetPageLayoutType(ptPageTemplate.GetPageType());
      // setup the gadgets
      SetupPortletsOnStart(_ptqrPortlets, true);
    }
    
    /// <summary>
    /// Method InitializePlugIn.
    /// To initialize this plug-in you need to set the DP name  and the default
    /// value of the plug-in setting. Called when creating or editing a community page.
    /// </summary>
    /// <param name="_myPageModel"></param>
    /// <param name="_myCommunity"></param>
    /// <param name="_nTableHeight"></param>
    /// <param name="_Session"></param>
    public virtual void InitializePlugIn(PageData _myPageModel, IPTCommunity _myCommunity,
     int _nTableHeight, IPTSession _Session)
    {
      // Setup hash map with names and values of form inputs
      // Currently this can only be used by the pages list view, 
      // but it could easily be extended for use by other views.
      m_htFormData = new XPHashtable();
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_ADDGADGETASID,
       "");
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_NEWGADGETASID,
       "");
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_ADDFREEFORMASID,
       "");
      m_htFormData.PutElement(PageLayoutHeaderView.HTMLINPUT_SELECTLAYOUTASID,
       "");
      m_nFolderID = _myCommunity.GetAdminFolderID();
      m_nClassID = PT_CLASSIDS.PT_PAGE_ID;
      m_nObjectID = _myPageModel.m_nPageID;
      m_bPageTemplateChanged = _myPageModel.m_bPageTemplateChanged;
      m_nTableHeight = _nTableHeight;
      m_ptSession = _Session;
      m_strPageName = _myPageModel.m_strPageName;
      m_nPageTemplateID = _myPageModel.m_nPageTemplateID;
      SetPageLayoutType(_myPageModel.m_nPageType);
      // setup the gadgets
      if (_myPageModel.m_bPortletsSetUp)
      {
        SetupPortletsOnStart(_myPageModel);
      } else
      {
        IASQueryResult ptqrPortlets = null;
        ASQueryResultWrapper qrw = null;
        if (0 == _myPageModel.m_nPageID || _myPageModel.m_bPageTemplateChanged)
        {
          IPTPageTemplate ptPageTemplate = (IPTPageTemplate) m_ptSession.GetPageTemplates().Open(_myPageModel.m_nPageTemplateID,
           false);
          qrw = new ASQueryResultWrapper(ptPageTemplate.QueryGadgets());
          ptqrPortlets = qrw;
          SetupPortletsOnStart(ptqrPortlets, true);
        } else
        {
          IPTPage ptPage = (IPTPage) _Session.GetPages().Open(_myPageModel.m_nPageID,
           false);
          qrw = new ASQueryResultWrapper(ptPage.QueryPortlets());
          ptqrPortlets = qrw;
          SetupPortletsOnStart(ptqrPortlets, false);
        }
      }
    }
    
    /// <summary>
    /// Method RepostAddFreeForm.
    /// Add a free form portlet
    /// </summary>
    public virtual void RepostAddFreeForm()
    {
      if (m_asOwner.GetSubSpaceExists(SUBSPACE_NAME_ADDFREEFORM))
      {
        TreeAS asTree = (TreeAS) m_asOwner.GetSubSpace(SUBSPACE_NAME_ADDFREEFORM);
        ITreeModelRO myTree = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
        XPArrayList mySelected = myTree.GetSelectedItems();
        // Set community template id
        int nFreeFormID = ((TreeLeafNode) mySelected.GetElement(0)).m_nObjectID;
        String nFreeFormName = ((TreeLeafNode) mySelected.GetElement(0)).GetName();
        if (null == m_gmFreeForm)
        {
          m_gmFreeForm = new PortletData();
        }
        m_gmFreeForm.m_nPortletID = nFreeFormID;
        m_gmFreeForm.m_bFromPageTemplate = false;
        m_gmFreeForm.m_nPortletColumn = PortletColumnConstants.FREE_FORM.ToInt();
        m_gmFreeForm.m_nPortletAlignment = PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_FREEFORM;
        m_gmFreeForm.m_strPortletName = nFreeFormName;
        // Have to query to get the description
        m_gmFreeForm.m_strPortletDescription = m_ptSession.GetGadgets().QuerySingleObject(nFreeFormID).ItemAsString(0,
         PT_PROPIDS.PT_PROPID_DESCRIPTION);
      }
    }
    
    /// <summary>
    /// Method RepostAddPortlets.
    /// Set the gadgets that were selected.
    /// </summary>
    public virtual void RepostAddPortlets()
    {
      if (m_asOwner.GetSubSpaceExists(SUBSPACE_NAME_ADDGADGETS))
      {
        PortletSelectionEditorAS asPortlet = (PortletSelectionEditorAS) m_asOwner.GetSubSpace(SUBSPACE_NAME_ADDGADGETS);
        ObjectSelectionModel myPortletSelection = (ObjectSelectionModel) asPortlet.GetModel(ObjectSelectionModel.STR_MVC_CLASS_NAME);
        XPSortedDictionary temp = myPortletSelection.GetSelectedObjectsForParent();
        // Parse dictionaries, separate into gadgets to add and to remove
        String[] aoPortletsToRemove = new String[m_xpsdNewSelectedObjects.GetSize()];
        int[] aTempRowsToRemove = new int[m_xpsdNewSelectedObjects.GetSize()];
        int iRemove = 0;
        IXPEnumerator xpeKeys = m_xpsdNewSelectedObjects.GetKeys();
        while (xpeKeys.MoveNext())
        {
          if (temp.ContainsKey(xpeKeys.GetCurrent()))
          {
            temp.RemoveElement(xpeKeys.GetCurrent());
          } else
          {
            aTempRowsToRemove[iRemove] = XPConvert.ToInteger(((Object[]) m_xpsdNewSelectedObjects.GetElement(xpeKeys.GetCurrent()))[COL_GADGET_INDEX]);
            aoPortletsToRemove[iRemove] = (String) xpeKeys.GetCurrent();
            iRemove++;
          }
        }
        // Remove gadgets from selected dictionary and from growable array
        int[] aRowsToRemove = new int[iRemove];
        for (int j = 0;j < iRemove;j++)
        {
          m_xpsdNewSelectedObjects.RemoveElement(aoPortletsToRemove[j]);
          aRowsToRemove[j] = aTempRowsToRemove[j];
        }
        if (0 < iRemove)
        {
          m_arrPortlets.RemoveRows(aRowsToRemove);
        }
        // Reset selected gadget's indexes
        for (int k = 0;k < m_arrPortlets.GetCount();k++)
        {
          ((Object[]) m_xpsdNewSelectedObjects.GetElement(XPConvert.ToString(((PortletData) m_arrPortlets.GetItem(k,
           PAGELAYOUT_COL_GADGET_MODEL)).m_nPortletID)))[COL_GADGET_INDEX] = k;
          }
        // Query for gadgets to add
        int[] aoSelectedPortlets = new int[temp.GetSize()];
        int i = 0;
        xpeKeys = temp.GetKeys();
        while (xpeKeys.MoveNext())
        {
          aoSelectedPortlets[i] = XPConvert.ToInteger((String) xpeKeys.GetCurrent());
          i++;
        }
        IASQueryResult ptqrPortlets = null;
        if (0 != aoSelectedPortlets.Length)
        {
          int nPropIDsToQuery = PT_PROPIDS.PT_PROPID_NAME | PT_PROPIDS.PT_PROPID_DESCRIPTION | PT_PROPIDS.PT_PROPID_GADGET_ALIGNMENT | PT_PROPIDS.PT_PROPID_OBJECTID | PT_PROPIDS.PT_PROPID_GADGET_GADGETTYPE;
          Object[][] ptqf = new Object[3][];
          for (int j2c_index = 0;j2c_index < 3;j2c_index++)
          {
            ptqf[j2c_index] = new Object[1];
          }
          ptqf[0][0] = PT_PROPIDS.PT_PROPID_OBJECTID;
          ptqf[1][0] = PT_FILTEROPS.PT_FILTEROP_IN;
          ptqf[2][0] = aoSelectedPortlets;
          PTObjectManagerQueryWrapper omqw = new PTObjectManagerQueryWrapper(m_ptSession.GetGadgets());
          omqw.Query(nPropIDsToQuery, -1, null, ptqf);
          ptqrPortlets = omqw;
        }
        // Add Portlets
        int nStartIndex = m_arrPortlets.GetCount();
        AddPortlets(ptqrPortlets, nStartIndex, false, true);
        PortletData myCurrentPortlet;
        for (int j = nStartIndex;j < m_arrPortlets.GetCount();j++)
        {
          myCurrentPortlet = ((PortletData) m_arrPortlets.GetItem(j, PAGELAYOUT_COL_GADGET_MODEL));
          if ((PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE == myCurrentPortlet.m_nPortletAlignment) || (PortletColumnCountConstants.ONE == m_PortletColumnCount))
          {
            myCurrentPortlet.m_nPortletColumn = PortletColumnConstants.WIDE_ONE.ToInt();
            } else
          {
            myCurrentPortlet.m_nPortletColumn = PortletColumnConstants.NARROW_ONE.ToInt();
            }
          SetFirstLastPortlets(myCurrentPortlet.m_nPortletColumn, j);
        }
      }
    }
    
    /// <summary>
    /// Method RepostDeletePortlet.
    /// Delete the gadget that were selected.
    /// </summary>
    public virtual void RepostDeletePortlet()
    {
      if ((null != m_gmFreeForm) && (m_gmFreeForm.m_nPortletID == m_nDeletePortletID))
      {
        // SE: 61272 - Only update portlet id to be removed if there was an initial freeform when editor started up.
        if (m_bHasInitialFreeForm && (-1 == m_nRemoveFreeForm))
        {
          // SE: Bug 61272 - Fix for Vodafone and UDSHHS. Only record the first time we are trying to remove the freeform 
          // portlet from the page editor. Or else it will attempt to remove a portlet from the page that doesn't exist in the DB. 
          // This issue can occur if the user attempts to change the content canvas portlet a couple of times before finally
          // saving. 
          // DCA - Let PageTemplate know freeform portlet has been removed
          m_nRemoveFreeForm = m_gmFreeForm.m_nPortletID;
        }
        log.Debug("PageLayoutPlugInModel::RepostDeletePortlet - Freeform portlet ID: " + m_nRemoveFreeForm + " marked for deletion");
        m_gmFreeForm = null;
      } else
      {
        int nDeleteIndex = XPConvert.ToInteger(((Object[]) m_xpsdNewSelectedObjects.GetElement(XPConvert.ToString(m_nDeletePortletID)))[COL_GADGET_INDEX]);
        m_arrPortlets.RemoveRow(nDeleteIndex);
        // Set the new index for the portlets in the list after the one we deleted.
        for (int i = nDeleteIndex;i < m_arrPortlets.GetCount();i++)
        {
          ((Object[]) m_xpsdNewSelectedObjects.GetElement(XPConvert.ToString(((PortletData) m_arrPortlets.GetItem(i,
           PAGELAYOUT_COL_GADGET_MODEL)).m_nPortletID)))[COL_GADGET_INDEX] = i;
          }
        m_xpsdNewSelectedObjects.RemoveElement(XPConvert.ToString(m_nDeletePortletID));
        }
      m_nDeletePortletID = 0;
    }
    
    /// <summary>
    /// Method RepostNewPortlet.
    /// Set the gadgets that were selected.
    /// </summary>
    public virtual void RepostNewPortlet()
    {
      IPTSession m_Session = (IPTSession) m_asOwner.GetUserSession();
      PTObjectManagerQueryWrapper omqr = new PTObjectManagerQueryWrapper(m_Session.GetGadgets());
      omqr.QuerySingleObject(GetNewPortletID());
      IASQueryResult ptqrPortlets = omqr;
      // DCA - check if portlet created is a free form 
      if (XPConvert.ToInteger(ptqrPortlets.GetFields(0, PT_PROPIDS.PT_PROPID_GADGET_ALIGNMENT)) == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_FREEFORM)
      {
        if (null == m_gmFreeForm)
        {
          m_gmFreeForm = new PortletData();
        }
        m_gmFreeForm.m_nPortletID = XPConvert.ToInteger(ptqrPortlets.GetFields(0,
         PT_PROPIDS.PT_PROPID_OBJECTID));
        m_gmFreeForm.m_bFromPageTemplate = false;
        m_gmFreeForm.m_nPortletColumn = PortletColumnConstants.FREE_FORM.ToInt();
        m_gmFreeForm.m_nPortletAlignment = PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_FREEFORM;
        m_gmFreeForm.m_strPortletName = XPConvert.ToString(ptqrPortlets.GetFields(0,
         PT_PROPIDS.PT_PROPID_NAME));
        m_gmFreeForm.m_strPortletDescription = XPConvert.ToString(ptqrPortlets.GetFields(0,
         PT_PROPIDS.PT_PROPID_DESCRIPTION));
        return;
      }
      // Add Portlets
      int nStartIndex = m_arrPortlets.GetCount();
      AddPortlets(ptqrPortlets, nStartIndex, false, true);
      PortletData myCurrentPortlet;
      for (int j = nStartIndex;j < m_arrPortlets.GetCount();j++)
      {
        myCurrentPortlet = ((PortletData) m_arrPortlets.GetItem(j, PAGELAYOUT_COL_GADGET_MODEL));
        if ((PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE == myCurrentPortlet.m_nPortletAlignment) || (PortletColumnCountConstants.ONE == m_PortletColumnCount))
        {
          myCurrentPortlet.m_nPortletColumn = PortletColumnConstants.WIDE_ONE.ToInt();
          } else
        {
          myCurrentPortlet.m_nPortletColumn = PortletColumnConstants.NARROW_ONE.ToInt();
          }
        SetFirstLastPortlets(myCurrentPortlet.m_nPortletColumn, j);
      }
    }
    
    /// <summary>
    /// Method SaveSection.
    /// Saves the data on the Main page. This method is called automatically by
    /// the Framework.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>int Status for server side validation</returns>
    public override int SaveSection(XPHashtable _htFormData)
    {
      String[] data;
      data = (String[]) _htFormData.GetElement(PageLayoutHeaderView.HTMLINPUT_EDITPAGE_PAGELAYOUTTYPE);
      // Set Page Type
      if ((null != data) && !(0 == data.Length))
      {
        if ((null != data[0]) && !("").Equals(data[0]))
        {
          SetPageLayoutType(XPConvert.ToInteger(data[0]));
        }
      } else
      {
        log.Warn("PageLayoutPlugInModel::SaveSection page layout type parameter was not in Query String.");
        }
      data = (String[]) _htFormData.GetElement(PageLayoutHeaderView.HTMLINPUT_DELETE_GADGETID);
      // Get Portlet To Delete
      if ((null != data) && !(0 == data.Length))
      {
        if ((null != data[0]) && !("").Equals(data[0]))
        {
          m_nDeletePortletID = XPConvert.ToInteger(data[0]);
        }
      } else
      {
        log.Info("PageLayoutPlugInModel::SaveSection delete parameter was not in Query String.");
        }
      // Get the gadget column orders
      SetColumnsOrder(_htFormData);
      String strKey;
      XPHashtable htTempData = new XPHashtable();
      IXPEnumerator xpKeys = m_htFormData.GetKeys();
      // Put values of inputs on main page into the form data object.
      // These inputs include the space ids for selecting a community
      // template, adding a new page, and one for each edit page. Also
      // the current edit page index is kept in here.
      while (xpKeys.MoveNext())
      {
        strKey = (String) xpKeys.GetCurrent();
        // && ("").equals((String) m_htFormData.GetElement(strKey))
        if (_htFormData.ContainsKey(strKey))
        {
          data = (String[]) _htFormData.GetElement(strKey);
          if ((null != data) && (0 != data.Length))
          {
            htTempData.PutElement(strKey, data[0]);
          }
        }
      }
      // Now copy it back to the core page model 
      IXPEnumerator xpKeys2 = htTempData.GetKeys();
      String data2;
      while (xpKeys2.MoveNext())
      {
        strKey = (String) xpKeys2.GetCurrent();
        if (_htFormData.ContainsKey(strKey))
        {
          data2 = (String) htTempData.GetElement(strKey);
          if ((null != data2) && (0 != data2.Length))
          {
            m_htFormData.PutElement(strKey, data2);
          }
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Method SetOpenerClassID.
    /// Sets the opener class ID.
    /// </summary>
    /// <param name="nOpenerClassID"></param>
    public virtual void SetOpenerClassID(int nOpenerClassID)
    {
      m_nClassID = nOpenerClassID;
    }
    
    /// <summary>
    /// Method SetOpenerObjectID.
    /// Sets the opener object ID.
    /// </summary>
    /// <param name="nOpenerObjectID"></param>
    public virtual void SetOpenerObjectID(int nOpenerObjectID)
    {
      m_nObjectID = nOpenerObjectID;
    }
    
    /// <summary>
    /// Method SetOpenerPageID.
    /// Sets the opener page ID.
    /// </summary>
    /// <param name="nOpenerPageID"></param>
    public virtual void SetOpenerPageID(int nOpenerPageID)
    {
      m_nClassID = PT_CLASSIDS.PT_PAGE_ID;
      m_nObjectID = nOpenerPageID;
    }
    
    /// <summary>
    /// Sets whether this is the users home my page or not.
    /// </summary>
    /// <param name="bIsHomeMyPage"></param>
    public virtual void SetIsHomeMyPage(bool bIsHomeMyPage)
    {
      m_bIsHomeMyPage = bIsHomeMyPage;
    }
    public virtual void SetAllowFreeFormSection(bool bAllowFreeFormSection)
    {
      m_bAllowFreeFormSection = bAllowFreeFormSection;
    }
    
    /// <summary>
    /// Method AddPortlets.
    /// Get gadget properties and set them up.
    /// </summary>
    /// <param name="ptqrPortlets"></param>
    /// <param name="nStartIndex"></param>
    /// <param name="_bExcludePageTempPortlets"></param>
    private void AddPortlets(IASQueryResult ptqrPortlets, int nStartIndex,
     bool _bIsMandPortlets, bool _bIgnorePageTempField)
    {
      int nPortletCount;
      // Portlet count
      if (null == ptqrPortlets)
      {
        nPortletCount = 0;
      } else
      {
        nPortletCount = ptqrPortlets.GetCount();
      }
      PortletData myPortletModel;
      // rob: this is an arraylist to temporarily store all rows we need to add to the growable sort array
      // we don't know how big the 2d array to pass into the growable sort array is at this time since there
      // might be duplicate portlets.
      XPArrayList listRows = new XPArrayList();
      Object[] aoSelectedInfo;
      int nPortletID;
      int nExcluded = 0;
      for (int i = 0;i < nPortletCount;i++)
      {
        nPortletID = XPConvert.ToInteger(ptqrPortlets.GetFields(i, PT_PROPIDS.PT_PROPID_OBJECTID));
        bool bExcluded = false;
        if (null != m_xpMandatoryPortlets && m_xpMandatoryPortlets.ContainsKey(XPConvert.ToString(nPortletID)))
        {
          bExcluded = true;
        }
        // SE: 60054 - Fix for Vodafone. Avoid adding duplicate portlets during creation of a new
        // community page that inherits from a page template.
        if (!bExcluded && !(m_xpsdNewSelectedObjects.ContainsKey(XPConvert.ToString(nPortletID))))
        {
          // Portlets names
          myPortletModel = new PortletData();
          myPortletModel.m_strPortletName = (String) ptqrPortlets.GetFields(i,
           PT_PROPIDS.PT_PROPID_NAME);
          if (!_bIsMandPortlets)
          {
            // Gadgets descriptions
            myPortletModel.m_strPortletDescription = (String) ptqrPortlets.GetFields(i,
             PT_PROPIDS.PT_PROPID_DESCRIPTION);
          } else
          {
            myPortletModel.m_strPortletDescription = "";
          }
          // Portlets IDs
          myPortletModel.m_nPortletID = nPortletID;
          // Portlets alignments
          myPortletModel.m_nPortletAlignment = XPConvert.ToInteger(ptqrPortlets.GetFields(i,
           PT_PROPIDS.PT_PROPID_GADGET_ALIGNMENT));
          // Portlets is of type community
          if (PT_GADGET_TYPES.PT_GADGETTYPE_COMMUNITY == XPConvert.ToInteger(ptqrPortlets.GetFields(i,
           PT_PROPIDS.PT_PROPID_GADGET_GADGETTYPE)))
          {
            myPortletModel.m_bIsCommunityPortlet = true;
          } else
          {
            myPortletModel.m_bIsCommunityPortlet = false;
          }
          if (!_bIgnorePageTempField)
          {
            if (1 == XPConvert.ToInteger(ptqrPortlets.GetFields(i, PT_PROPIDS.PT_PROPID_GADGET_FROMPAGETEMPLATE)))
            {
              myPortletModel.m_bFromPageTemplate = true;
            }
          }
          // Put gadget model in growable sorted array
          int propIdIndex = i + nStartIndex - nExcluded;
          listRows.Add(new Object[]{propIdIndex, myPortletModel});
          //m_arrPortlets.AddRow(arrRow);
          // New Selected Portlets
          aoSelectedInfo = new Object[COL_GADGET_MAX];
          aoSelectedInfo[COL_GADGET_INDEX] = propIdIndex;
          aoSelectedInfo[COL_GADGET_NAME] = myPortletModel.m_strPortletName;
          aoSelectedInfo[COL_GADGET_ALIGNMENT] = myPortletModel.m_nPortletAlignment;
          aoSelectedInfo[COL_GADGET_QUERY_INDEX] = i;
          m_xpsdNewSelectedObjects.PutElement(XPConvert.ToString(myPortletModel.m_nPortletID),
           aoSelectedInfo);
        } else
        {
          nExcluded++;
        }
      }
      // construct the 2d array to pass into the growablesortedarray from the list
      Object[][] arrRow = new Object[listRows.GetSize()][];
      for (int j2c_index = 0;j2c_index < listRows.GetSize();j2c_index++)
      {
        arrRow[j2c_index] = new Object[2];
      }
      for (int j = 0;j < listRows.GetSize();j++)
      {
        arrRow[j] = (Object[]) listRows.GetElement(j);
      }
      // fix for #51045: finally add new rows to the growablesortarray in one go instead of adding each individual
      // entry and resort after each add
      m_arrPortlets.AddRows(arrRow);
    }
    
    /// <summary>
    /// Method InitializeFirstLastPortletArray.
    /// Initializes the first and last gadget index arrays.
    /// </summary>
    private void InitializeFirstLastPortletArray()
    {
      m_arFirstPortletIndex = new int[PortletColumnCountConstants.THREE.ToInt()];
      m_arLastPortletIndex = new int[PortletColumnCountConstants.THREE.ToInt()];
      // initialize the first/last gadget with -1
      for (int i = 0;i < PortletColumnCountConstants.THREE.ToInt();i++)
      {
        m_arFirstPortletIndex[i] = -1;
        m_arLastPortletIndex[i] = -1;
      }
    }
    
    /// <summary>
    /// Method InitializeSortedArray.
    /// Initialize the growable sorted array.
    /// </summary>
    private void InitializeSortedArray()
    {
      m_arrPortlets = new PTGrowableSortedArrayWrapper();
      int[] arrColumns = new int[]{PAGELAYOUT_SORTEDARRAY_PROPID_INDEX, PAGELAYOUT_COL_GADGET_MODEL};
      int[][] arrSort = new int[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        arrSort[j2c_index] = new int[2];
      }
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID] = PAGELAYOUT_SORTEDARRAY_PROPID_INDEX;
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      m_arrPortlets.Initialize(arrColumns, null, arrSort, true);
    }
    
    /// <summary>
    /// Method SetColumnsOrder.
    /// Records the gadget column order for the given column.
    /// </summary>
    /// <param name="_htFormData"></param>
    private void SetColumnsOrder(XPHashtable _htFormData)
    {
      String[] data;
      int[] anPortletOrder = new int[0];
      Object[][] tempRows = new Object[m_arrPortlets.GetCount() - m_nMandPortletCount][];
      Object[] arrRow;
      String strPortletID;
      int nPortletID;
      Object[] aoSelectedInfo;
      int iOrder;
      int iNewIndex = 0;
      int iTempIndex = 0;
      Object[][] mandtempRows = null;
      if (IsHomeMyPage())
      {
        mandtempRows = new Object[m_nMandPortletCount][];
        int nColumnZeroMandatoryPortlets = 0;
        int nColumnOneMandatoryPortlets = 0;
        int nColumnTwoMandatoryPortlets = 0;
        PortletData gmTemp;
        for (int i = 0;i < m_arrPortlets.GetCount();i++)
        {
          gmTemp = ((PortletData) m_arrPortlets.GetItem(i, PAGELAYOUT_COL_GADGET_MODEL));
          if (gmTemp.m_bFromPageTemplate)
          {
            // Get column for mandatory portlet
            if (PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE == gmTemp.m_nPortletAlignment || PageTypeConstants.ONE_COLUMN == this.m_nPageLayoutType || PageTypeConstants.ONE_COLUMN_WITH_FREE_FORM_CONTENT == this.m_nPageLayoutType)
            {
              // Put wide mandatory gadgets in the wide column.
              gmTemp.m_nPortletColumn = PortletColumnConstants.WIDE_ONE.ToInt();
              nColumnOneMandatoryPortlets++;
            } else
            {
              // Put narrow mandatory gadgets in the narrow column with the least mandatory gadgets in it.
              if ((PortletColumnCountConstants.THREE == this.m_PortletColumnCount) && (nColumnTwoMandatoryPortlets < nColumnZeroMandatoryPortlets))
              {
                gmTemp.m_nPortletColumn = PortletColumnConstants.NARROW_TWO.ToInt();
                nColumnTwoMandatoryPortlets++;
              } else
              {
                gmTemp.m_nPortletColumn = PortletColumnConstants.NARROW_ONE.ToInt();
                nColumnZeroMandatoryPortlets++;
              }
            }
            arrRow = new Object[2];
            arrRow[PAGELAYOUT_SORTEDARRAY_PROPID_INDEX] = iNewIndex;
            arrRow[PAGELAYOUT_COL_GADGET_MODEL] = gmTemp;
            mandtempRows[iNewIndex] = arrRow;
            aoSelectedInfo = ((Object[]) m_xpsdNewSelectedObjects.GetElement(XPConvert.ToString(gmTemp.m_nPortletID)));
            aoSelectedInfo[COL_GADGET_INDEX] = arrRow[PAGELAYOUT_SORTEDARRAY_PROPID_INDEX];
            m_xpsdNewSelectedObjects.PutElement(XPConvert.ToString(gmTemp.m_nPortletID),
             aoSelectedInfo);
            SetFirstLastPortlets(gmTemp.m_nPortletColumn, iNewIndex);
            iNewIndex++;
          }
        }
      }
      for (int iColumn = PortletColumnConstants.NARROW_ONE.ToInt();iColumn <= PortletColumnConstants.NARROW_TWO.ToInt();
      iColumn++)
      {
        m_aanPortletOrder[iColumn] = new int[0];
        data = (String[]) _htFormData.GetElement(PageLayoutView.HTMLINPUT_EDITPAGE_GADGETORDER_PREFIX + XPConvert.ToString(iColumn));
        // Set order of column zero
        if ((null != data) && !(0 == data.Length))
        {
          if ((null != data[0]) && !("").Equals(data[0]))
          {
            // see DAP NOTE below
            int trueArraySize = 0;
            try
            {
              // the first token is null, but the tokenizer skips it
              XPStringTokenizer st = new XPStringTokenizer(data[0], ",");
              // DAP NOTE: we have to allocate *more* slots than we need
              // as this list includes the mandatory portlets also
              // At the bottom of this loop, we'll reallocate a smaller destination 
              // array and save that to preserve both order and size
              anPortletOrder = new int[st.GetTokenCount()];
              if (st.GetTokenCount() > 0)
              {
                iOrder = 0;
                while (st.HasMoreTokens())
                {
                  strPortletID = st.GetNextToken();
                  nPortletID = XPConvert.ToInteger(strPortletID);
                  if (!(IsHomeMyPage()) || !(m_xpMandatoryPortlets.ContainsKey(strPortletID)))
                  {
                    aoSelectedInfo = ((Object[]) m_xpsdNewSelectedObjects.GetElement(strPortletID));
                    arrRow = new Object[2];
                    arrRow[PAGELAYOUT_SORTEDARRAY_PROPID_INDEX] = iNewIndex;
                    arrRow[PAGELAYOUT_COL_GADGET_MODEL] = m_arrPortlets.GetItem(XPConvert.ToInteger(aoSelectedInfo[COL_GADGET_INDEX]),
                     PAGELAYOUT_COL_GADGET_MODEL);
                    if (!((PortletData) arrRow[PAGELAYOUT_COL_GADGET_MODEL]).m_bFromPageTemplate)
                    {
                      anPortletOrder[trueArraySize] = nPortletID;
                      trueArraySize++;
                    }
                    ((PortletData) arrRow[PAGELAYOUT_COL_GADGET_MODEL]).m_nPortletColumn = iColumn;
                    tempRows[iTempIndex] = arrRow;
                    aoSelectedInfo[COL_GADGET_INDEX] = arrRow[PAGELAYOUT_SORTEDARRAY_PROPID_INDEX];
                    m_xpsdNewSelectedObjects.PutElement(strPortletID, aoSelectedInfo);
                    SetFirstLastPortlets(iColumn, iNewIndex);
                    iNewIndex++;
                    iOrder++;
                    iTempIndex++;
                  }
                }
              }
            }
            catch (XPException e)
            {
              // probably no gadget in this column
              log.Warn("PageLayoutPlugInModel.SetColumnOrder: Exception when getting gadget order for the " + iColumn + " column -- probably are no gadgets in it.");
              }
            // DAP code to preserve the true final size of the order array
            // in light of mandatory gadgets present in column, we must reallocate
            // and copy over to a smaller array 
            if (trueArraySize > 0)
            {
              int[] anPortletOrderFinal = new int[trueArraySize];
              for (int i = 0;i < trueArraySize;++i)
              {
                anPortletOrderFinal[i] = anPortletOrder[i];
              }
              m_aanPortletOrder[iColumn] = anPortletOrderFinal;
            }
          }
        } else
        {
          log.Warn("PageLayoutPlugInModel::SaveSection page layout " + PageLayoutView.HTMLINPUT_EDITPAGE_GADGETORDER_PREFIX + XPConvert.ToString(iColumn) + " parameter was not in Query String.");
          }
      }
      m_arrPortlets.ClearArray();
      if (null != mandtempRows)
      {
        m_arrPortlets.AddRows(mandtempRows);
      }
      m_arrPortlets.AddRows(tempRows);
    }
    
    /// <summary>
    /// Method SetFirstLastPortlets.
    /// Records the first and last gadgets' indexes in an array.
    /// </summary>
    /// <param name="nPortletColumn"></param>
    /// <param name="nIndex"></param>
    private void SetFirstLastPortlets(int nPortletColumn, int nIndex)
    {
      // the first time we fill it in will be the first gadget
      if (m_arFirstPortletIndex[nPortletColumn] == -1)
      {
        m_arFirstPortletIndex[nPortletColumn] = nIndex;
      }
      // always put in the last gadget into the array
      m_arLastPortletIndex[nPortletColumn] = nIndex;
    }
    
    /// <summary>
    /// Method SetPageLayoutType.
    /// Sets the page layout type and the column count.
    /// </summary>
    /// <param name="nPageLayoutType"></param>
    private void SetPageLayoutType(int nPageLayoutType)
    {
      // Bug 64629: Fall back to using three column layout if portlet layout is disabled
      // and page layout type was set to adaptive layouts.
      if (!LayoutService.getChooserInstance().GetPortletLayoutsEnabled(m_asOwner) && nPageLayoutType < 0)
      {
        m_nPageLayoutType = PageTypeConstants.THREE_COLUMN;
      } else
      {
        m_nPageLayoutType = nPageLayoutType;
      }
      // Set m_nPortletColumnCount based upon m_nPageType
      switch (m_nPageLayoutType){
      case PageTypeConstants.ONE_COLUMN:
      case PageTypeConstants.ONE_COLUMN_WITH_FREE_FORM_CONTENT:
      m_PortletColumnCount = PortletColumnCountConstants.ONE;
      break;
      case PageTypeConstants.TWO_COLUMN_NARROW_WIDE:
      case PageTypeConstants.TWO_COLUMN_WIDE_NARROW:
      case PageTypeConstants.TWO_COLUMN_NARROW_WIDE_WFFC:
      case PageTypeConstants.TWO_COLUMN_WIDE_WFFC_NARROW:
      case PageTypeConstants.TWO_COLUMN_WFFC_NARROW_WIDE:
      case PageTypeConstants.TWO_COLUMN_WFFC_WIDE_NARROW:
      m_PortletColumnCount = PortletColumnCountConstants.TWO;
      break;
      case PageTypeConstants.THREE_COLUMN:
      case PageTypeConstants.THREE_COLUMN_WITH_FREE_FORM_CONTENT:
      case PageTypeConstants.THREE_COLUMN_NARROW_WIDENARROW_WFFC:
      case PageTypeConstants.THREE_COLUMN_NARROW_WIDE_WFF_NARROW:
      case PageTypeConstants.THREE_COLUMN_NARROWWIDE_WFFC_NARROW:
      m_PortletColumnCount = PortletColumnCountConstants.THREE;
      break;
      default:
      // adaptive layout, the same form field value is used for both page type and layout id,
      // layout ids are negative of the webservice id
      if (m_nPageLayoutType < 0)
      {
        // still want to display 3 columns in the layout editor 
        m_PortletColumnCount = PortletColumnCountConstants.THREE;
      } else
      {
        throw new XPIllegalArgumentException("Invalid page layout type: " + m_nPageLayoutType);
        }
      break;
      }
    }
    
    /// <summary>
    /// Method SetupPortletsOnStart.
    /// Pass in result from IPTMyPortal.QueryGadgets() to set up portlet layout.
    /// </summary>
    /// <param name="ptqrPortlets"></param>
    private void SetupPortletsOnStart(IASQueryResult ptqrPortlets, bool bIgnoreFromPageTemplate)
    {
      InitializeSortedArray();
      InitializeFirstLastPortletArray();
      m_gmFreeForm = null;
      m_xpsdNewSelectedObjects = new XPSortedDictionary();
      m_xpsdOldSelectedObjects = new XPSortedDictionary();
      PortletData gmTemp;
      m_nMandPortletCount = 0;
      m_xpMandatoryPortlets = new XPHashtable();
      if (IsHomeMyPage())
      {
        // Add mandatory portlet to home page edit layout
        ASQueryResultWrapper qrw = new ASQueryResultWrapper(m_ptSession.GetMyPages().QueryMandatoryGadgets());
        IASQueryResult ptqrMandPortlets = qrw;
        AddPortlets(ptqrMandPortlets, 0, true, true);
        m_nMandPortletCount = ptqrMandPortlets.GetCount();
        int nColumnZeroMandatoryPortlets = 0;
        int nColumnOneMandatoryPortlets = 0;
        int nColumnTwoMandatoryPortlets = 0;
        for (int i = 0;i < m_nMandPortletCount;i++)
        {
          gmTemp = ((PortletData) m_arrPortlets.GetItem(i, PAGELAYOUT_COL_GADGET_MODEL));
          m_xpMandatoryPortlets.PutElement(XPConvert.ToString(gmTemp.m_nPortletID),
           "");
          gmTemp.m_bFromPageTemplate = true;
          if (PT_GADGET_TYPES.PT_GADGETTYPE_COMMUNITY == XPConvert.ToInteger(ptqrMandPortlets.GetFields(i,
           PT_PROPIDS.PT_PROPID_GADGET_GADGETTYPE)))
          {
            gmTemp.m_bIsCommunityPortlet = true;
          } else
          {
            gmTemp.m_bIsCommunityPortlet = false;
          }
          // Get column for mandatory portlet
          if (PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE == gmTemp.m_nPortletAlignment || PageTypeConstants.ONE_COLUMN == this.m_nPageLayoutType || PageTypeConstants.ONE_COLUMN_WITH_FREE_FORM_CONTENT == this.m_nPageLayoutType)
          {
            // Put wide mandatory gadgets in the wide column.
            gmTemp.m_nPortletColumn = PortletColumnConstants.WIDE_ONE.ToInt();
            nColumnOneMandatoryPortlets++;
          } else
          {
            // Put narrow mandatory gadgets in the narrow column with the least mandatory gadgets in it.
            if ((PortletColumnCountConstants.THREE == this.m_PortletColumnCount) && (nColumnTwoMandatoryPortlets < nColumnZeroMandatoryPortlets))
            {
              gmTemp.m_nPortletColumn = PortletColumnConstants.NARROW_TWO.ToInt();
              nColumnTwoMandatoryPortlets++;
            } else
            {
              gmTemp.m_nPortletColumn = PortletColumnConstants.NARROW_ONE.ToInt();
              nColumnZeroMandatoryPortlets++;
            }
          }
          SetFirstLastPortlets(gmTemp.m_nPortletColumn, i);
        }
      }
      // If this is a not a community page, then ignore the From Page Template field.
      AddPortlets(ptqrPortlets, m_nMandPortletCount, false, bIgnoreFromPageTemplate);
      int nPortletColumn;
      int nAddedIndexForFreeForm = 0;
      int nPortletCount = m_arrPortlets.GetCount();
      int nQueryIndex;
      for (int i = 0;i + nAddedIndexForFreeForm < nPortletCount - m_nMandPortletCount;
      i++)
      {
        gmTemp = ((PortletData) m_arrPortlets.GetItem(i + m_nMandPortletCount,
         PAGELAYOUT_COL_GADGET_MODEL));
        Object oQueryIndex = ((Object[]) m_xpsdNewSelectedObjects.GetElement(XPConvert.ToString(gmTemp.m_nPortletID)))[COL_GADGET_QUERY_INDEX];
        nQueryIndex = XPConvert.ToInteger(oQueryIndex);
        nPortletColumn = XPConvert.ToInteger(ptqrPortlets.GetFields(nQueryIndex,
         PT_PROPIDS.PT_PROPID_GADGET_COLUMN));
        if (PortletColumnConstants.FREE_FORM.ToInt() == nPortletColumn)
        {
          // This is the free form portlet
          // Remove it from the array of other gadgets because it's special
          m_gmFreeForm = ((PortletData) m_arrPortlets.GetItem(i + m_nMandPortletCount,
           PAGELAYOUT_COL_GADGET_MODEL));
          m_gmFreeForm.m_nPortletColumn = nPortletColumn;
          if (bIgnoreFromPageTemplate)
          {
            m_gmFreeForm.m_bFromPageTemplate = false;
          } else
          {
            if (1 == XPConvert.ToInteger(ptqrPortlets.GetFields(nQueryIndex,
             PT_PROPIDS.PT_PROPID_GADGET_FROMPAGETEMPLATE)))
            {
              m_gmFreeForm.m_bFromPageTemplate = true;
            } else
            {
              m_gmFreeForm.m_bFromPageTemplate = false;
            }
          }
          if (PT_GADGET_TYPES.PT_GADGETTYPE_COMMUNITY == XPConvert.ToInteger(ptqrPortlets.GetFields(nQueryIndex,
           PT_PROPIDS.PT_PROPID_GADGET_GADGETTYPE)))
          {
            m_gmFreeForm.m_bIsCommunityPortlet = true;
          } else
          {
            m_gmFreeForm.m_bIsCommunityPortlet = false;
          }
          m_arrPortlets.RemoveRow(i + m_nMandPortletCount);
          // Since we remove it from the list the PTGrowableSortedArrayWrapper
          // has shifted all the portlets one index lower. So i needs to be decreased by one.
          i--;
          nAddedIndexForFreeForm++;
        } else
        {
          if (0 != nAddedIndexForFreeForm)
          {
            // update the new selected objects info, the index has changed
            ((Object[]) m_xpsdNewSelectedObjects.GetElement(XPConvert.ToString(gmTemp.m_nPortletID)))[COL_GADGET_INDEX] = i + m_nMandPortletCount;
            }
          if (bIgnoreFromPageTemplate)
          {
            gmTemp.m_bFromPageTemplate = false;
          } else
          {
            if (1 == XPConvert.ToInteger(ptqrPortlets.GetFields(nQueryIndex,
             PT_PROPIDS.PT_PROPID_GADGET_FROMPAGETEMPLATE)))
            {
              gmTemp.m_bFromPageTemplate = true;
            } else
            {
              gmTemp.m_bFromPageTemplate = false;
            }
          }
          if (PT_GADGET_TYPES.PT_GADGETTYPE_COMMUNITY == XPConvert.ToInteger(ptqrPortlets.GetFields(nQueryIndex,
           PT_PROPIDS.PT_PROPID_GADGET_GADGETTYPE)))
          {
            gmTemp.m_bIsCommunityPortlet = true;
          } else
          {
            gmTemp.m_bIsCommunityPortlet = false;
          }
          // Portlets columns
          gmTemp.m_nPortletColumn = nPortletColumn;
          SetFirstLastPortlets(nPortletColumn, i + m_nMandPortletCount);
          // Old Selected Portlets
          m_xpsdOldSelectedObjects.PutElement(XPConvert.ToString(gmTemp.m_nPortletID),
           "");
        }
      }
      // SE: Bug 61272 - Check if we have an initial freeform portlet.
      // We only want to remove portlet if there was one when the editor started up.
      if (null != m_gmFreeForm)
      {
        if (0 < m_gmFreeForm.m_nPortletID)
        {
          m_bHasInitialFreeForm = true;
        } else
        {
          m_bHasInitialFreeForm = false;
        }
      }
      m_nMandPortletCount = m_xpMandatoryPortlets.GetSize();
    }
    
    /// <summary>
    /// Method SetupPortletsOnStart.
    /// Pass in page model to set up portlet layout.
    /// </summary>
    /// <param name="pageModel"></param>
    private void SetupPortletsOnStart(PageData pageModel)
    {
      InitializeSortedArray();
      InitializeFirstLastPortletArray();
      m_xpsdNewSelectedObjects = new XPSortedDictionary();
      m_xpsdOldSelectedObjects = new XPSortedDictionary();
      m_xpMandatoryPortlets = new XPHashtable();
      if (pageModel.m_nPortletCount > 0)
      {
        for (int i = 0;i < pageModel.m_nPortletCount;i++)
        {
          PortletData temp = ((PortletData) pageModel.m_arrPortlets.GetItem(i,
           PAGELAYOUT_COL_GADGET_MODEL));
          // Put gadget model in growable sorted array
          Object[] arrRow = new Object[]{i, temp};
          m_arrPortlets.AddRow(arrRow);
          // New Selected Portlets
          Object[] aoSelectedInfo = new Object[COL_GADGET_MAX];
          aoSelectedInfo[COL_GADGET_INDEX] = arrRow[PAGELAYOUT_SORTEDARRAY_PROPID_INDEX];
          aoSelectedInfo[COL_GADGET_NAME] = temp.m_strPortletName;
          aoSelectedInfo[COL_GADGET_ALIGNMENT] = temp.m_nPortletAlignment;
          aoSelectedInfo[COL_GADGET_QUERY_INDEX] = i;
          m_xpsdNewSelectedObjects.PutElement(XPConvert.ToString(temp.m_nPortletID),
           aoSelectedInfo);
          SetFirstLastPortlets(temp.m_nPortletColumn, i);
          // Old Selected Portlets
          m_xpsdOldSelectedObjects.PutElement(XPConvert.ToString(temp.m_nPortletID),
           "");
        }
      }
      // Free Form portlet
      m_gmFreeForm = pageModel.m_gmFreeForm;
      // SE: Bug 61272 - Check if we have an initial freeform portlet.
      // We only want to remove portlet if there was one when the editor started up.
      if (null != m_gmFreeForm)
      {
        if (0 < m_gmFreeForm.m_nPortletID)
        {
          m_bHasInitialFreeForm = true;
        } else
        {
          m_bHasInitialFreeForm = false;
        }
      }
    }
    private bool IsHomeMyPage()
    {
      return m_bIsHomeMyPage;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portalpages.common.editpage.IPageLayoutModelRO#GetUseDragDropLayout()
     */
     
    public virtual bool GetUseDragDropLayout()
    {
      BrowserType browser = (BrowserType) m_asOwner.GetSubSession().GetAttribute(Interpreter.BROWSERTYPE);
      return browser.IsDragDropLayoutCompliant();
    }
  }
}