using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portalpages.common.search
{
  /// <author>
  /// CraigS
  /// This is an abstract superclass for a view that looks like an EditorView,
  /// i.e. has the same blue striped title across the top,
  /// but may be used in any ActivitySpace, not just EditorAS subclasses.
  /// </author>
  public abstract class EditorLikeView : IView
  {
    public const String SPACE = "&nbsp;";
    protected AActivitySpace m_asOwner;
    protected IModelRO m_model;
    
    /// <returns></returns>
    public abstract Object Create();
    
    /// <returns></returns>
    public abstract HTMLElement Display();
    
    /// <returns></returns>
    public abstract String GetName();
    
    /// <returns></returns>
    public virtual HTMLScript DisplayJavascript()
    {
      return null;
    }
    
    /// <param name="_sHeaderTitle"></param>
    /// <returns></returns>
    public virtual HTMLElement GetHeaderView(String _sHeaderTitle, bool addSpacer)
    {
      HTMLTable table = new HTMLTable();
      table.SetWidth("100%");
      table.SetCellSpacing("0");
      table.SetCellPadding("0");
      table.SetBorder("0");
      HTMLTableRow row = new HTMLTableRow();
      table.AddInnerHTMLElement(row);
      row.SetStyleClass(PTStyleClass.OBJECT_HEADER_BG);
      HTMLTableCell cell = new HTMLTableCell();
      row.AddInnerHTMLElement(cell);
      cell.SetWidth("100%");
      cell.SetHeight("20");
      cell.SetVAlign("middle");
      cell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      HTMLSpan span = new HTMLSpan();
      cell.AddInnerHTMLElement(span);
      span.AddInnerHTMLString(_sHeaderTitle);
      span.SetStyleClass(PTStyleClass.OBJECT_HEADER);
      HTMLElementCollection c = new HTMLElementCollection();
      c.AddInnerHTMLElement(table);
      if (addSpacer)
      {
        c.AddInnerHTMLElement(makeSpacer("2", this.m_asOwner));
      }
      return c;
    }
    private static HTMLElement makeSpacer(String height, AActivitySpace owner)
    {
      HTMLTable table = new HTMLTable();
      table.SetCellSpacing("0");
      table.SetCellPadding("0");
      table.SetWidth("100%");
      HTMLTableRow row = new HTMLTableRow();
      table.AddInnerHTMLElement(row);
      HTMLTableCell cell = new HTMLTableCell();
      row.AddInnerHTMLElement(cell);
      HTMLImg img = null;
      try
      {
        img = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.SPACER,
         true, owner).GetHTMLElement();
      }
      catch (Exception e)
      {
        throw XPException.GetInstance(e);
      }
      cell.AddInnerHTMLElement(img);
      img.SetHeight(height);
      img.SetWidth("1");
      img.SetBorder("0");
      img.SetAlign("absMiddle");
      return table;
    }
    
    /// <param name="model"></param>
    /// <param name="parent"></param>
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      m_model = model;
      m_asOwner = parent;
    }
  }
}