using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.filter;
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker;
using com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
using com.plumtree.xpshared.jsutil;
namespace com.plumtree.portalpages.common.search
{
  /// <author>CraigS, based on MikeS code</author>
  public abstract class FilterView : EditorLikeView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.portalpages.common.search.FilterView");
    private HTMLElement _textboxLabel = null;
    private String _addClauseLabel = null;
    private String _addConditionLabel = null;
    private String _propertyLabel = null;
    private String _removeClauseLabel = null;
    private String _removeConditionLabel = null;
    private String _viewTitle = "";
    private String _width = "100%";
    private bool _allowSubclauses = true;
    private bool _errorsNextToTextBoxes = false;
    private bool _putTextSearchAboveButtons = false;
    private bool _useJavascriptStatementButtons = true;
    private bool _useCheckboxLayout = false;
    private bool _addTestButton = false;
    private String _testButtonScript = "";
    private String _testButtonOnClickScript = "";
    // mbs- This is only used for finish validation of the filter editor,
    private bool m_bAtLeastOneErrorOccurred = false;
    // aj: this should default to true since this view is
    //     primarily used for search-related activities.  
    //     only the dynamic groups chooser should
    //     need to supress this box...
    private bool _showSearchBox = true;
    private int _repostAction;
    private int m_tablesize = 10;
    
    /// <summary>
    /// Display the HTML for the view
    /// </summary>
    /// <returns>HTMLElement</returns>
    public override HTMLElement Display()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      m_bAtLeastOneErrorOccurred = false;
      IFilterIteratorRO model = ((IFilterIteratorContainerRO) m_model).GetFilterIterator();
      // Get the user's locale and timezone
      String locale = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LOCALE,
       m_asOwner);
      String timezone = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.TIMEZONE,
       m_asOwner);
      XPLocale xpLocale = null;
      XPTimeZone xpTimeZone = null;
      if (null != locale)
      {
        xpLocale = new XPLocale(locale);
      } else
      {
        xpLocale = XPLocale.GetDefault();
      }
      if (null != timezone)
      {
        xpTimeZone = XPTimeZone.GetTimeZone(timezone);
      } else
      {
        xpTimeZone = XPTimeZone.GetCurrentTimeZone();
      }
      // Initialize the page controls and date/time picker
      String strImageServerBaseURL = ImgSvrURLHelper.GetInstance().GetRootURL(m_asOwner.GetIsSecuredSpace());
      PageMgr pageMgr = new PageMgr(m_asOwner.GetAccessStyle(), xpLocale,
       xpTimeZone, strImageServerBaseURL, ConfigHelper.GetPortalImageServerConnectionURL(m_asOwner));
      JSRegistry ptJSRegistry = JSRegistry.GetRegistry(strImageServerBaseURL,
       ConfigHelper.GetPortalImageServerConnectionURL(m_asOwner));
      JSDateTimePickerMgr dtPickerMgr = new JSDateTimePickerMgr(ptJSRegistry,
       new JSDatePickerVersionMgr(), pageMgr);
      pageMgr.RegisterComponentMgr(dtPickerMgr);
      try
      {
        result.AddInnerHTMLElement(dtPickerMgr.GetJSIncludes());
        if (null != _viewTitle)
        {
          result.AddInnerHTMLElement(base.GetHeaderView(_viewTitle, false));
          }
        //Display the filter banner and buttons
        HTMLElementCollection buttons = MakeButtons();
        bool shownButtonsYet = false;
        HTMLTable buttonTable = new HTMLTable();
        HTMLTableRow myRow = null;
        HTMLTableCell myCell = null;
        if (!_putTextSearchAboveButtons)
        {
          if (_useJavascriptStatementButtons)
          {
            // Add the action bar with the search filter actions
            result.AddInnerHTMLElement(HTMLConstructHelpers.GetActionBar(buttons,
             "FilterView", m_asOwner));
          } else
          {
            //TODO: this, it doesn't belong here.
            HTMLTable table = new HTMLTable();
            table.SetWidth("100%");
            table.SetBorder("0");
            table.SetCellPadding("0");
            table.SetCellSpacing("0");
            HTMLTableRow row = new HTMLTableRow();
            row.SetStyleClass(PTStyleClass.OBJECT_HEADER_BG);
            table.AddInnerHTMLElement(row);
            HTMLTableCell cell = new HTMLTableCell();
            row.AddInnerHTMLElement(cell);
            cell.SetWidth("100%");
            cell.SetHeight("20");
            cell.SetVAlign("middle");
            cell.SetStyleClass(PTStyleClass.OBJECT_HEADER);
            HTMLSpan span = new HTMLSpan();
            span.AddInnerHTMLString(m_asOwner.GetString(743, "ptmsgs_portalcommonmsgs"));
            cell.AddInnerHTMLElement(span);
            result.AddInnerHTMLElement(table);
            result.AddInnerHTMLElement(buttons);
          }
          shownButtonsYet = true;
        } else
        {
          // Build a table to hold the buttons (not in an actionbar)
          buttonTable.SetWidth(_width);
          buttonTable.SetBorder("0");
          buttonTable.SetCellPadding("5");
          buttonTable.SetCellSpacing("0");
          myRow = new HTMLTableRow();
          buttonTable.AddInnerHTMLElement(myRow);
          myRow.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);
          myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          HTMLBr br = new HTMLBr();
          myCell.AddInnerHTMLElement(br);
          myCell.AddInnerHTMLElement(buttons);
          br = new HTMLBr();
          myCell.AddInnerHTMLElement(br);
        }
        HTMLSpan errorSpan = new HTMLSpan();
        if (GetDoValidation() && (!_errorsNextToTextBoxes))
        {
          errorSpan.SetStyleClass(PTStyleClass.ALERT_ERROR_TEXT);
          result.AddInnerHTMLElement(errorSpan);
        }
        // aj: suppress the search box and "AND/OR" chooser
        if (_showSearchBox)
        {
          //Add a spacer table
          if (!_putTextSearchAboveButtons)
          {
            result.AddInnerHTMLElement(EditorView.GetShortSpacerTable());
            }
          String strLang = XPConvert.ToString(PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE,
           m_asOwner));
          if (_useJavascriptStatementButtons)
          {
            DynamicQueryTable.AddTopLevelSearchBoxAndOperator(model.GetTextSearchString(),
             model.GetTopLevelAllMustPass(), !_allowSubclauses, _textboxLabel,
             result, strLang);
          } else
          {
            RepostQueryTable.AddTopLevelSearchBoxAndOperator(model.GetTextSearchString(),
             model.GetTopLevelAllMustPass(), !_allowSubclauses, _textboxLabel,
             result, strLang, m_asOwner);
          }
        }
        // aj: originally the BuildFilterFromHashtable() function
        //     relied on the in_ra_topoperator radio to
        //     determine if it's actually processing the right
        //     form data.  since this can now be supressed
        //     we need to have a hidden element that will indicate
        //     we are on the right form data: in_he_thisIsAFilterForm
        HTMLInput he_isAFilterForm = new HTMLInput(HTMLInputTypes.HIDDEN,
         FilterForm.IS_FILTER_FORM, "");
        he_isAFilterForm.SetValue("true");
        result.AddInnerHTMLElement(he_isAFilterForm);
        int groupCount = 0;
        model.SetIteratorToStart();
        while (model.GetNextClausePosition())
        {
          bool Operator = model.GetCurrentClauseOperator();
          int offset = model.GetCurrentClauseDepth();
          // we have a clause, let's create a table for it
          groupCount++;
          IFilterQueryTable queryGroup = null;
          if (_useJavascriptStatementButtons)
          {
            queryGroup = new DynamicQueryTable(XPConvert.ToString(groupCount),
             offset, m_asOwner);
          } else
          {
            queryGroup = new RepostQueryTable(XPConvert.ToString(groupCount),
             offset, m_asOwner);
          }
          queryGroup.SetAllMustPass(Operator);
          queryGroup.SetTableSize(m_tablesize);
          // the table of queries must be seeded with a generic query used for dynamic expansion
          QueryCondition invisibleQuery = model.MakeQueryCondition();
          // default to name
          invisibleQuery.SetSelections(1, (int) (PT_FILTEROPS.PT_FILTEROP_CONTAINS),
           "");
          queryGroup.SetBlankQueryCondition(invisibleQuery);
          QueryCondition qc = invisibleQuery.MakeDuplicate();
          qc.SetVisible(true);
          // load the appropriate statements
          int itemcounter = 0;
          while (model.GetNextStatementAndSetQC(qc))
          {
            itemcounter++;
            QueryCondition next = qc.MakeDuplicate();
            bool bSetPropertyAdded = model.AddSetProperty(next);
            queryGroup.loadDefinedQueryCondition(next);
            if (!next.ContainsProperty(next.GetStatementProperty()))
            {
              SetError(errorSpan, XPResourceManager.FormatMessage(m_asOwner.GetString(792,
               "ptmsgs_portalcommonmsgs"), XPConvert.ToString(next.GetStatementProperty())),
               XPConvert.ToString(groupCount), XPConvert.ToString(itemcounter));
              }
            if (GetDoValidation())
            {
              String strWarning = next.ValidateSelections();
              if (strWarning.Length > 0)
              {
                SetError(errorSpan, strWarning, XPConvert.ToString(groupCount),
                 XPConvert.ToString(itemcounter));
              }
              if (bSetPropertyAdded)
              {
                SetError(errorSpan, m_asOwner.GetString(791, "ptmsgs_portalcommonmsgs"),
                 XPConvert.ToString(groupCount), XPConvert.ToString(itemcounter));
                }
            }
          }
          if (!_allowSubclauses)
          {
            // if subclauses are not allowed, it makes no sense to label the single
            // group "Grouping 1"
            //queryGroup.SetLabel(m_asOwner.GetString(200,
            //		"ptmsgs_portalcommonmsgs"));
            //queryGroup.SetSuppressCheckbox(true);
            //queryGroup.SetUseVerboseTerminology(true);
            queryGroup.SetHideGroupHeaderRow(true);
            if (null != _propertyLabel)
            {
              HTMLElementCollection he = new HTMLElementCollection();
              he.AddInnerHTMLString(_propertyLabel);
              if (_putTextSearchAboveButtons && (model.GetNumItemsInFirstClause() == 0))
              {
                he.AddInnerHTMLElement(buttons);
                shownButtonsYet = true;
              }
              queryGroup.SetExtraRowBelowGroupHeader(he);
            }
          }
          queryGroup.addTableToCollection(result, GetDoValidation() && _errorsNextToTextBoxes,
           dtPickerMgr, pageMgr);
        }
        // EDITOR_OBJECTID_UNKNOWN
        // we must have been opened for create
        if (0 == groupCount)
        {
          // Build the table....
          groupCount = 1;
          IFilterQueryTable querylist = null;
          if (_useJavascriptStatementButtons)
          {
            querylist = new DynamicQueryTable("1", 0, m_asOwner);
          } else
          {
            querylist = new RepostQueryTable("1", 0, m_asOwner);
          }
          querylist.SetAllMustPass(true);
          querylist.SetTableSize(m_tablesize);
          QueryCondition newQuery = model.MakeQueryCondition();
          newQuery.SetSelections(1, (int) (PT_FILTEROPS.PT_FILTEROP_CONTAINS),
           "");
          QueryCondition blankQuery = newQuery.MakeDuplicate();
          querylist.SetBlankQueryCondition(blankQuery);
          newQuery.SetVisible(true);
          //querylist.loadDefinedQueryCondition(newQuery);
          if (!_allowSubclauses)
          {
            // if subclauses are not allowed, it makes no sense to label the single
            // group "Grouping 1"
            //querylist.SetLabel(m_asOwner.GetString(200,
            //		"ptmsgs_portalcommonmsgs"));
            //querylist.SetSuppressCheckbox(true);
            //querylist.SetUseVerboseTerminology(true);
            querylist.SetHideGroupHeaderRow(true);
            if (null != _propertyLabel)
            {
              HTMLElementCollection he = new HTMLElementCollection();
              he.AddInnerHTMLString(_propertyLabel);
              if (_putTextSearchAboveButtons && (model.GetNumItemsInFirstClause() == 0))
              {
                he.AddInnerHTMLElement(buttons);
                shownButtonsYet = true;
              }
              querylist.SetExtraRowBelowGroupHeader(he);
            }
          }
          querylist.addTableToCollection(result, GetDoValidation() && _errorsNextToTextBoxes,
           dtPickerMgr, pageMgr);
        }
        if (!shownButtonsYet)
        {
          result.AddInnerHTMLElement(buttonTable);
        }
        /*the active group*/
        DynamicQueryTable.AddGlobalElements("1", groupCount, result);
      }
      catch (Exception e)
      {
        log.Error(e, "FilterView::Display An error occured when trying to display the HTML.");
        throw new XPException(m_asOwner.GetString(199, "ptmsgs_portalcommonmsgs"),
         e);
      }
      return result;
    }
    
    /// <returns></returns>
    public override HTMLScript DisplayJavascript()
    {
      HTMLScript script = null;
      try
      {
        String submitCommand = "\tpostToSelf(" + XPConvert.ToString(_repostAction) + ");\n";
        if (_useJavascriptStatementButtons)
        {
          script = (HTMLScript) DynamicQueryTable.GetJavaScript(submitCommand,
           m_asOwner);
        } else
        {
          script = (HTMLScript) RepostQueryTable.GetJavaScript(submitCommand);
          }
        if (_addTestButton && _testButtonScript != null)
        {
          script.AddInnerHTMLString(_testButtonScript);
        }
      }
      catch (Exception e)
      {
        log.Error(e, "FilterView::Display An error occured when trying to display the HTML.");
        throw new XPException(m_asOwner.GetString(199, "ptmsgs_portalcommonmsgs"),
         e);
      }
      return script;
    }
    private HTMLElementCollection MakeButtons()
    {
      IFilterIteratorRO model = ((IFilterIteratorContainerRO) m_model).GetFilterIterator();
      HTMLElementCollection result = new HTMLElementCollection();
      String strLang = XPConvert.ToString(PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE,
       m_asOwner));
      if (_useJavascriptStatementButtons)
      {
        result.AddInnerHTMLElement(DynamicQueryTable.GetAddConditionButton(_addConditionLabel,
         strLang, m_asOwner));
        result.AddInnerHTMLElement(DynamicQueryTable.GetRemoveLastConditionButton(_removeConditionLabel,
         strLang, m_asOwner));
        if (_allowSubclauses)
        {
          result.AddInnerHTMLElement(DynamicQueryTable.GetAddNewClauseButton(_addClauseLabel,
           strLang, m_asOwner));
          result.AddInnerHTMLElement(DynamicQueryTable.GetRemoveClauseButton(_removeClauseLabel,
           strLang, m_asOwner));
        }
        if (_addTestButton)
        {
          result.AddInnerHTMLElement(DynamicQueryTable.GetTestButton(m_asOwner.GetString(2600,
           "ptmsgs_portaladminmsgs"), _testButtonOnClickScript, m_asOwner));
          }
      } else
      {
        bool showAddStmtButton = true;
        bool showRemoveStmtButton = true;
        HTMLTable table = new HTMLTable();
        table.SetBorder("0");
        table.SetCellPadding("0");
        table.SetCellSpacing("0");
        table.SetWidth("100%");
        result.AddInnerHTMLElement(table);
        HTMLTableRow row = new HTMLTableRow();
        row.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);
        table.AddInnerHTMLElement(row);
        HTMLTableCell cell;
        // If we allow only one clause, turn off the add or remove statement
        // buttons when that one clause satisfies certain conditions
        if (!_allowSubclauses)
        {
          int numStmts = model.GetNumItemsInFirstClause();
          if (0 == numStmts)
          {
            showRemoveStmtButton = false;
          }
          if (m_tablesize == numStmts)
          {
            showAddStmtButton = false;
          }
        }
        if (showAddStmtButton)
        {
          cell = new HTMLTableCell();
          cell.SetHeight("25");
          cell.SetNoWrap(true);
          row.AddInnerHTMLElement(cell);
          cell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
          cell.AddInnerHTMLElement(RepostQueryTable.GetAddConditionButton(_addConditionLabel,
           strLang));
          cell = new HTMLTableCell();
          cell.SetHeight("25");
          cell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_DIVIDER);
          row.AddInnerHTMLElement(cell);
          PTConstruct construct = ConstructFactory.GetConstruct(PTImageType.SPACER,
           true);
          HTMLImg img = (HTMLImg) construct.GetHTMLElement();
          img.SetWidth(XPConvert.ToString(1));
          img.SetHeight("1");
          img.SetAlign(CommonHTMLStrings.ABSMIDDLE);
          cell.AddInnerHTMLElement(img);
        }
        if (showRemoveStmtButton)
        {
          cell = new HTMLTableCell();
          cell.SetHeight("25");
          cell.SetNoWrap(true);
          row.AddInnerHTMLElement(cell);
          cell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
          cell.AddInnerHTMLElement(RepostQueryTable.GetRemoveLastConditionButton(_removeConditionLabel,
           strLang));
          cell = new HTMLTableCell();
          cell.SetHeight("25");
          cell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_DIVIDER);
          row.AddInnerHTMLElement(cell);
          PTConstruct construct = ConstructFactory.GetConstruct(PTImageType.SPACER,
           true);
          HTMLImg img = (HTMLImg) construct.GetHTMLElement();
          img.SetWidth(XPConvert.ToString(1));
          img.SetHeight("1");
          img.SetAlign(CommonHTMLStrings.ABSMIDDLE);
          cell.AddInnerHTMLElement(img);
        }
        cell = new HTMLTableCell();
        cell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        cell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        row.AddInnerHTMLElement(cell);
        // Show the add/remove clause buttons if desired
        if (_allowSubclauses)
        {
          result.AddInnerHTMLElement(RepostQueryTable.GetAddNewClauseButton(_addClauseLabel,
           strLang));
          result.AddInnerHTMLElement(RepostQueryTable.GetRemoveClauseButton(_removeClauseLabel,
           strLang));
        }
      }
      return result;
    }
    public virtual bool QueryContainsValidationProblemsOnRender()
    {
      return m_bAtLeastOneErrorOccurred;
    }
    
    /// <summary>
    /// Subclasses may override to turn off the display of errors
    /// under certain circumstances
    /// </summary>
    protected virtual bool GetDoValidation()
    {
      return true;
    }
    
    /// <returns></returns>
    protected virtual bool GetUseJavascriptStatementButtons()
    {
      return _useJavascriptStatementButtons;
    }
    
    /// <param name="s"></param>
    protected virtual void SetAddClauseLabel(String s)
    {
      _addClauseLabel = s;
    }
    
    /// <param name="s"></param>
    protected virtual void SetAddConditionLabel(String s)
    {
      _addConditionLabel = s;
    }
    
    /// <param name="b"></param>
    protected virtual void SetAllowSubclauses(bool b)
    {
      _allowSubclauses = b;
    }
    
    /// <param name="b"></param>
    protected virtual void SetErrorsNextToTextBoxes(bool b)
    {
      _errorsNextToTextBoxes = b;
    }
    
    /// <param name="s"></param>
    protected virtual void SetPropertyLabel(String s)
    {
      _propertyLabel = s;
    }
    
    /// <param name="s"></param>
    protected virtual void SetRemoveClauseLabel(String s)
    {
      _removeClauseLabel = s;
    }
    
    /// <param name="s"></param>
    protected virtual void SetRemoveConditionLabel(String s)
    {
      _removeConditionLabel = s;
    }
    
    /// <param name="a"></param>
    protected virtual void SetRepostAction(int a)
    {
      _repostAction = a;
    }
    
    /// <param name="size"></param>
    protected virtual void SetShowSearchBox(bool b)
    {
      _showSearchBox = b;
    }
    
    /// <param name="size"></param>
    protected virtual void SetStatementCount(int size)
    {
      m_tablesize = size;
    }
    protected virtual void SetTextSearchAboveButtons()
    {
      _putTextSearchAboveButtons = true;
    }
    
    /// <param name="e"></param>
    protected virtual void SetTextboxLabel(HTMLElement e)
    {
      _textboxLabel = e;
    }
    public virtual void SetAddTestButton()
    {
      _addTestButton = true;
    }
    
    /// <param name="b"></param>
    protected virtual void SetUseJavascriptStatementButtons(bool b)
    {
      _useJavascriptStatementButtons = b;
    }
    
    /// <param name="s"></param>
    protected virtual void SetViewTitle(String s)
    {
      _viewTitle = s;
    }
    
    /// <param name="s"></param>
    protected virtual void SetWidth(String s)
    {
      _width = s;
    }
    public virtual void SetUseCheckboxLayout()
    {
      _useCheckboxLayout = true;
    }
    // Pass in the function definition to call when the test button is pressed.
    public virtual void SetTestButtonScript(String script)
    {
      _testButtonScript = script;
    }
    // Pass in the OnClick script for the test filter button.
    public virtual void SetTestButtonOnClickScript(String script)
    {
      _testButtonOnClickScript = script;
    }
    public static HTMLElementCollection GetPagePreFormContent(AActivitySpace asOwner)
    {
      JSDateTimePickerMgr mgr = FilterForm.MakeTempDateTimePickerMgr(asOwner);
      return mgr.GetPagePreFormContent();
    }
    private void SetError(HTMLElement _error, String _message, String _groupcnt,
     String _itemcnt)
    {
      m_bAtLeastOneErrorOccurred = true;
      if (!_errorsNextToTextBoxes)
      {
        if (_allowSubclauses)
        {
          _error.AddInnerHTMLString(XPResourceManager.FormatMessage(m_asOwner.GetString(201,
           "ptmsgs_portalcommonmsgs"), XPConvert.ToString(_groupcnt), XPConvert.ToString(_itemcnt),
           _message));
        } else
        {
          _error.AddInnerHTMLString(XPResourceManager.FormatMessage(m_asOwner.GetString(736,
           "ptmsgs_portalcommonmsgs"), XPConvert.ToString(_itemcnt), _message));
          }
        _error.AddInnerHTMLElement(new HTMLBr());
      }
    }
  }
}