/*
 */
 
using System;
using com.plumtree.openfoundation.reflect;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.editors.community;
using com.plumtree.portalpages.browsing.editors.editportletprefs;
using com.plumtree.portalpages.browsing.gateway;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.portalpages.browsing.myportal.communities;
using com.plumtree.portalpages.browsing.objectselection;
using com.plumtree.portalpages.browsing.objectselection.communities;
using com.plumtree.portalpages.browsing.objectselection.flyout;
using com.plumtree.portalpages.browsing.objectselection.portlets;
using com.plumtree.portalpages.browsing.sitemap.sitemappage;
using com.plumtree.portalpages.browsing.unsubscribe.communities;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes;
using com.plumtree.portaluiinfrastructure.opener;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.taskapi.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.taskapi.portalui
{
  /// <summary>
  /// Community related TaskAPI functions.
  /// </summary>
  public class TaskAPIUICommunity
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
     "com.plumtree.taskapi.portalui.TaskAPIUICommunity");
    
    /// <summary>
    /// Gets the name of the specified Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID of the Community to retrieve the name for.</param>
    /// <returns>	Name of the specified Community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static String GetCommunityName(AActivitySpace owner, int nCommunityID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      TaskAPIUICommon.ValidateObjectIDArgument(nCommunityID);
      IPTSession session = TaskAPIUICommon.GetPTSession(owner);
      String strCommName = TaskAPIServerCommunity.GetCommunityName(session,
       nCommunityID);
      return strCommName;
    }
    
    /// <summary>
    /// Gets the Subcommunities for the specified Community as an integer array.
    /// Subcommunites are Communities directly below a Community in the administrative folder hierarchy.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID for the Community to retrieve Subcommunities from.</param>
    /// <returns>	Array of integers with Object IDs of the Subcommunities in the specified Community. </returns>
    /// <throws>
    /// XPException
    /// 									If the Community could not be accessed, for example if the Object ID
    /// is not valid.
    /// </throws>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static int[] GetSubCommununityIDs(AActivitySpace owner, int nCommunityID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      TaskAPIUICommon.ValidateObjectIDArgument(nCommunityID);
      IPTSession session = TaskAPIUICommon.GetPTSession(owner);
      IPTQueryResult qrRelComms = TaskAPIServerCommunity.GetCommunitySubCommunities(session,
       nCommunityID);
      int[] arSubCommIDs = new int[qrRelComms.RowCount()];
      for (int i = 0;i < qrRelComms.RowCount();i++)
      {
        int nSubCommID = qrRelComms.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
        arSubCommIDs[i] = nSubCommID;
      }
      return arSubCommIDs;
    }
    
    /// <summary>
    /// Gets the Related Communities for the specified Community as an integer array.
    /// Related Communities are Communities in the same administrative folder.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID for the Community to retrieve Related Communities from.</param>
    /// <returns>	Array of integers with Object IDs of the Related Communities  in the specified Community. </returns>
    /// <throws>
    /// XPException
    /// 									If the Community could not be accessed, for example if the Object ID
    /// is not valid.
    /// </throws>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static int[] GetRelatedCommununityIDs(AActivitySpace owner, int nCommunityID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      TaskAPIUICommon.ValidateObjectIDArgument(nCommunityID);
      IPTSession session = TaskAPIUICommon.GetPTSession(owner);
      IPTQueryResult qrRelComms = TaskAPIServerCommunity.GetCommunityPeerCommIDs(session,
       nCommunityID);
      int[] arRelCommIDs = new int[qrRelComms.RowCount()];
      for (int i = 0;i < qrRelComms.RowCount();i++)
      {
        int nRelCommID = qrRelComms.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
        arRelCommIDs[i] = nRelCommID;
      }
      return arRelCommIDs;
    }
    
    /// <summary>
    /// Get Community Pages in the specified Community as a list of <c>IPTURL</c>s.
    /// The <c>PTURLConstants.KEY_CURRENTMARKER</c> flag is set on an URL entry if is leading to the current
    /// Community Page. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object Id of the Communitu to list pages of.</param>
    /// <returns>A list with Community Pages as <c>IPTURL</c> objects.</returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static XPArrayList GetCommunityPagesAsURLs(AActivitySpace owner,
     int nCommunityID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      TaskAPIUICommon.ValidateObjectIDArgument(nCommunityID);
      IPTSession session = TaskAPIUICommon.GetPTSession(owner);
      // get the community pages
      IPTQueryResult qrCommPages = TaskAPIServerCommunity.GetCommunityPages(session,
       nCommunityID);
      // create the arraylist to return
      XPArrayList list = new XPArrayList(qrCommPages.RowCount());
      for (int i = 0;i < qrCommPages.RowCount();i++)
      {
        // Get the pageid and page name from the queryresult
        int nPageID = qrCommPages.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
        String strPageName = qrCommPages.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
        IPTURL url = CreateCommunityPageCommonOpenerURL(owner, nCommunityID,
         nPageID, strPageName);
        // Add the link to the arraylist
        list.Add(url);
      }
      return list;
    }
    
    /// <summary>
    /// Gets to URL to the Community Knowledge Directory f0r the current Community.
    /// The <c>PTURLConstants.KEY_CURRENTMARKER</c> flag is set on the URL if the current page is a Community Knowledge
    /// Directory page.
    /// </summary>
    /// <param name="owner">Current Activityspace</param>
    /// <param name="nCommunityID">Object ID of the Community to get pages from.</param>
    /// <returns><c>IPTURL</c> with the URL to the Community Knowledge Directory for the specified Community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    /// <seealso cref="PTURLConstants"></seealso>
    public static IPTURL GetURLForCommunityKnowledgeDirectory(AActivitySpace owner,
     int nCommunityID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      // Don't want to display the knowledge directory or members folder for profile community
      if (nCommunityID <= TaskAPIUICommon.INVALID_OBJECTID || TaskAPIUICommunity.IsCurrentCommunityUserProfileCommunity(owner) || TaskAPIServerCommunity.GetCommunityInfoObject(ptSession,
       nCommunityID).GetSiteMapDisabled())
      {
        return null;
      }
      String strTitle = owner.GetString(1647, "ptmsgs_portalbrowsingmsgs");
      String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
      String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
      String strASName = SiteMapAS.STR_MVC_CLASS_NAME;
      int nUserID = TaskAPIUIUser.GetCurrentUserID(owner);
      CommonOpenerOpenASURL url = new CommonOpenerOpenASURL(strTitle, strApplBaseURL,
       strASName, nUserID);
      url.AddControlParameter(ASConstants.CONTROL, SetSiteMapControl.STR_MVC_CLASS_NAME);
      url.AddControlParameter(SetSiteMapControl.m_CommunityID, nCommunityID);
      url.AddControlParameter(SetSiteMapControl.m_Mode, SetSiteMapControl.BROWSE);
      if (TaskAPIUICommon.IsCurrentPageInPreviewMode(owner))
      {
        url.AddControlParameter(SetSiteMapControl.PREVIEW, "");
      }
      if (TaskAPIUICommon.IsCurrentActivitySpace(owner, SiteMapAS.STR_MVC_CLASS_NAME) && !TaskAPIUICommon.IsCurrentDisplayPage(owner,
       MembersDP.STR_MVC_CLASS_NAME))
      {
        // Show Community Knowledge Dir as label if in it except when in the members page,
        // The member will appear as selected instead.
        url.SetURLData(PTURLConstants.KEY_CURRENTMARKER, "");
      }
      // Set image
      url.SetURLImageType(PTImageType.ICON_KNOWLEDGE_DIRECTORY, strImageServerURL);
      return url;
    }
    
    /// <summary>
    /// Gets URLs to Related Communities for the specified Community in a list of <c>IPTURL</c>s.
    /// Related Communities are Communities in the same administrative folder.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID for the Community to retrieve Related Communities from.</param>
    /// <returns><c>XPArrayList</c> with <c>IPTURL</c>s to Related Communities or an empty <c>XPArrayList</c> if none exists.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static XPArrayList GetRelatedCommununitiesAsURLs(AActivitySpace owner,
     int nCommunityID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCommunityID))
      {
        return new XPArrayList();
      }
      IPTSession session = TaskAPIUICommon.GetPTSession(owner);
      // get the community pages
      IPTQueryResult qrRelComms = null;
      try
      {
        qrRelComms = TaskAPIServerCommunity.GetCommunityPeerCommIDs(session,
         nCommunityID);
      }
      catch (XPException e)
      {
        log.Warn(e, "Exception encountered while retrieving Related Communities IDs for Community: " + nCommunityID);
        return new XPArrayList();
      }
      // create the arraylist to return
      XPArrayList list = new XPArrayList(qrRelComms.RowCount());
      for (int i = 0;i < qrRelComms.RowCount();i++)
      {
        // Get the pageid and page name from the queryresult
        int nRelCommID = qrRelComms.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
        String strRelCommName = qrRelComms.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
        // Create a common opener url
        IPTURL url = CreateCommunityCommonOpenerURL(owner, nRelCommID, strRelCommName);
        // Add the link to the arraylist
        list.Add(url);
      }
      return list;
    }
    
    /// <summary>
    /// Gets the Community Name of the current Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	Community Name of the current community or <c>null</c> value if not currently on a Community Page. </returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 		If the provided <c>AActivityspace</c> is <c>null</c>.
    /// </throws>
    public static String GetCurrentCommunityName(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      int nCurrCommID = GetCurrentCommunityID(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCurrCommID))
      {
        return null;
      }
      String strCommName = TaskAPIServerCommunity.GetCommunityName(ptSession,
       nCurrCommID);
      return strCommName;
    }
    
    /// <summary>
    /// Gets the Community Page Name of the current Community Page. If current
    /// page is not a Community related page, i.e., Community portlet, Sitemap
    /// or Community Gatewaypage page, a null value is returned.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	Community Page Name of the current community pageor null value if not currently on a Community Page. </returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 		If the provided <c>AActivityspace</c> is <c>null</c>
    /// </throws>
    public static String GetCurrentCommunityPageName(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      String strPageName = null;
      if (TaskAPIUICommon.GetMyPortalPageModelExists(owner))
      {
        MyPortalModel ptModel = TaskAPIUICommon.GetMyPortalPageModel(owner);
        strPageName = ptModel.GetPageName();
      } else
      if (FlyoutPortletSelectionAS.STR_MVC_CLASS_NAME.Equals(TaskAPIUICommon.GetActivitySpaceName(owner)))
      {
        // rob: The flyout AS is always preceded by community or mypage AS, so call the same function with the parent space  
        strPageName = GetCurrentCommunityPageName(owner.GetParentSpace());
        }
      return strPageName;
    }
    
    /// <summary>
    /// Gets the Community ID of the current Community. If current page is not a Community page,
    /// i.e., Community portlet, Sitemap or Community Gatewaypage page, <c>TaskAPIUICommon.INVALID_OBJECTID</c> is returned.
    /// </summary>
    /// <param name="owner">Current Activityspace</param>
    /// <returns>	Community ID of the current community or <c>TaskAPIUICommon.INVALID_OBJECTID</c> if not in a Community.</returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 		If the provided <c>AActivityspace</c> is <c>null</c>
    /// </throws>
    public static int GetCurrentCommunityID(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      int nCommID = TaskAPIUICommon.INVALID_OBJECTID;
      if (TaskAPIUICommon.GetMyPortalPageModelExists(owner))
      {
        MyPortalModel portalMod = TaskAPIUICommon.GetMyPortalPageModel(owner);
        if (null != portalMod)
        {
          nCommID = portalMod.GetCommunityID();
        }
      } else
      if (TaskAPIUICommon.GetSiteMapPageModelExists(owner))
      {
        SiteMapModel sitemapMod = TaskAPIUICommon.GetSiteMapPageModel(owner);
        if (sitemapMod != null)
        {
          nCommID = sitemapMod.GetCommunityID();
        }
      } else
      if (TaskAPIUICommon.GetGatewayPageModelExists(owner))
      {
        // We are on a gatewayed page, community ID in the Gateway model
        IGatewayModelRO gatewayModel = (IGatewayModelRO) TaskAPIUICommon.GetGatewayPageModel(owner);
        if (gatewayModel != null)
        {
          nCommID = gatewayModel.GetCommunityID();
        }
      } else
      if (FlyoutPortletSelectionAS.STR_MVC_CLASS_NAME.Equals(TaskAPIUICommon.GetActivitySpaceName(owner)))
      {
        // rob: The flyout AS is always preceded by community or mypage AS, so call the same function with the parent space  
        nCommID = GetCurrentCommunityID(owner.GetParentSpace());
      }
      return nCommID;
    }
    
    /// <summary>
    /// Gets the current Community Page ID, If current page is not a Community page,
    /// the <c>TaskAPIUICommon.INVALID_OBJECTID</c> value is returned.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	Current Community Page ID or <c>TaskAPIUICommon.INVALID_OBJECTID</c> if not in on Community Page.</returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 		If the provided <c>AActivityspace</c> is <c>null</c>.
    /// </throws>
    public static int GetCurrentCommunityPageID(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      int nPageID = TaskAPIUICommon.INVALID_OBJECTID;
      if (!TaskAPIUICommunity.IsCurrentPageCommunityPage(owner))
      {
        return nPageID;
      }
      if (TaskAPIUICommon.GetMyPortalPageModelExists(owner))
      {
        MyPortalModel ptModel = TaskAPIUICommon.GetMyPortalPageModel(owner);
        nPageID = ptModel.GetPageID();
      } else
      if (FlyoutPortletSelectionAS.STR_MVC_CLASS_NAME.Equals(TaskAPIUICommon.GetActivitySpaceName(owner)))
      {
        // rob: The flyout AS is always preceded by community or mypage AS, so call the same function with the parent space  
        nPageID = GetCurrentCommunityPageID(owner.GetParentSpace());
      }
      return nPageID;
    }
    
    /// <summary>
    /// Gets the current Page as an object of type<c>IPTURL</c>. Returns  
    /// </summary>
    /// <param name="owner">Current Activityspace</param>
    /// <returns>IPTURL to the current community page or null if the current page is not a community page. </returns>
    /// <throws>XPIllegalArgumentException</throws>
    public static IPTURL GetCurrentPageAsURL(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTURL url = null;
      int commId = GetCurrentCommunityID(owner);
      int pageId = TaskAPIUICommon.INVALID_OBJECTID;
      String pageName = null;
      if (commId == TaskAPIUICommon.INVALID_OBJECTID)
      {
        pageId = TaskAPIUIMyPages.GetCurrentMyPageID(owner);
        pageName = TaskAPIUIMyPages.GetCurrentMyPageName(owner);
        url = TaskAPIUIMyPages.CreateMyPageCommonOpenerURL(owner, pageId,
         pageName);
      } else
      {
        pageId = GetCurrentCommunityPageID(owner);
        pageName = GetCurrentCommunityPageName(owner);
        url = CreateCommunityPageCommonOpenerURL(owner, commId, pageId, pageName);
        }
      return url;
    }
    
    /// <summary>
    /// Gets Subcommunities in the current Community as an <c>XPArrayList</c> with <c>IPTURL</c> entries.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// Array list with <c>IPTURL</c> of Subcommunities.  An empty list is returned if there are 
    /// no subcommunities exist or not in a Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static XPArrayList GetCurrentSubCommunitiesAsURLs(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      int nCommunityID = GetCurrentCommunityID(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCommunityID))
      {
        return new XPArrayList();
      }
      XPArrayList list = GetSubCommununitiesAsURLs(owner, nCommunityID);
      return list;
    }
    
    /// <summary>
    /// INTERNAL
    /// Helper function that performs the common checks for whether Community Actions should be displayed or not. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the Community Actions are displayable in the current context, or <c>false</c> if not.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    internal static bool IsCommunityActionsDisplayable(AActivitySpace owner)
    {
      if (IsCommunityFeatureEnabledInSubPortal(owner) && 
      //IsCurrentPageCommunityPage(owner) &&
      !TaskAPIUICommon.IsCurrentPageInPreviewMode(owner) && !TaskAPIUIUser.IsCurrentUserGuestUser(owner) && !TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(owner))
      {
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// Checks if the Community Feature is enabled in the current Experience Definition.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if Community feature is enabled in the current Experience Definintion, <c>false</c> otherwise. </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    internal static bool IsCommunityFeatureEnabledInSubPortal(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      return TaskAPIServerSubPortal.IsSubPortalFeatureEnabled(ptSession, PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_COMMUNITIES_ACCESS);
      }
    
    /// <summary>
    /// Gets the URL to the Unsubscribe from Communities Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Unsubscribe from Communities Editor, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetURLForUnsubscribeCommunitiesEditor(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        // DCA - we don't really need a community or my page AS specifically here
        // so just set to the activity space that was passed in
        communityAS = owner;
      }
      CommonOpenerOpenASURL url = null;
      if (IsCommunityFeatureEnabledInSubPortal(communityAS) && !TaskAPIUICommon.IsCurrentPageInPreviewMode(communityAS) && !TaskAPIUIUser.IsCurrentUserGuestUser(communityAS) || (TaskAPIUIUser.IsCurrentUserGuestUser(communityAS) && TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(communityAS)))
      {
        String strURLTitle = communityAS.GetString(638, "ptmsgs_portalcommonmsgs");
        // Use current AActivitySpace for the following calls
        String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
        String strASName = UnsubscribeCommunitiesAS.STR_MVC_CLASS_NAME;
        int nUserID = TaskAPIUIUser.GetCurrentUserID(communityAS);
        url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
         nUserID);
        url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
        url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
        url.AddControlParameter(ASConstants.PARENTSPACEID, communityAS.GetSpaceID());
        url.AddControlParameter(ASConstants.PARENTSPACENAME, communityAS.GetName());
        // Set image
        url.SetURLImageType(PTImageType.ACTION_COMMUNITIES_UNSUB, strImageServerURL);
        }
      return url;
    }
    
    /// <summary>
    /// Gets the URL to the Community Editor for the current Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Editor for the current Community, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetURLForCurrentCommunityEditor(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        return null;
      }
      CommonOpenerOpenObjIDURL url = null;
      if ((communityAS.GetAccessStyle() != AccessStyles.SECTION_508) && IsCommunityActionsDisplayable(communityAS))
      {
        if (IsCurrentPageCommunityPage(communityAS) && IsCurrentUserCommunityOwner(communityAS))
        {
          String strURLTitle = communityAS.GetString(308, "ptmsgs_portalcommonmsgs");
          int nCommunityID = TaskAPIUICommunity.GetCurrentCommunityID(communityAS);
          // Use current AActivitySpace for the following calls
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          url = new CommonOpenerOpenObjIDURL(strURLTitle, PT_CLASSIDS.PT_COMMUNITY_ID,
           nCommunityID, IPTOpenerConstants.OPENER_MODE_EDIT, owner);
          url.AddControlParameter(ASConstants.PARENTSPACEID, communityAS.GetSpaceID());
          url.AddControlParameter(ASConstants.PARENTSPACENAME, communityAS.GetName());
          // Set image
          url.SetURLImageType(PTImageType.ACTION_COMMUNITIES_EDIT, strImageServerURL);
          }
      }
      return url;
    }
    
    /// <summary>
    /// Gets an XPArrayList with a single entry for the URL for the current Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>XPArrayList</c> list containing a single URL to the current Community, 
    /// or <c>null</c> if the URL is not valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static XPArrayList GetURLForCurrentCommunity(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        return null;
      }
      // get the URL
      CommonOpenerOpenObjIDURL url = null;
      if ((communityAS.GetAccessStyle() != AccessStyles.SECTION_508) && IsCommunityActionsDisplayable(communityAS))
      {
        if (IsCurrentPageCommunityPage(communityAS) && IsCurrentUserCommunityOwner(communityAS))
        {
          String strURLTitle = GetCurrentCommunityName(communityAS);
          int nCommunityID = GetCurrentCommunityID(communityAS);
          // Use current AActivitySpace for the following calls
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          url = new CommonOpenerOpenObjIDURL(strURLTitle, PT_CLASSIDS.PT_COMMUNITY_ID,
           nCommunityID, IPTOpenerConstants.OPENER_MODE_VIEW, owner);
          url.AddControlParameter(ASConstants.PARENTSPACEID, communityAS.GetSpaceID());
          url.AddControlParameter(ASConstants.PARENTSPACENAME, communityAS.GetName());
          // Set image
          url.SetURLImageType(PTImageType.ICON_COMMUNITY, strImageServerURL);
          // Set metadata about this community and page on this URL object
          url.SetURLData("commname", strURLTitle);
          url.SetURLData("commid", XPConvert.ToString(nCommunityID));
          url.SetURLData("commpagename", GetCurrentCommunityPageName(owner));
          url.SetURLData("commpageid", XPConvert.ToString(GetCurrentCommunityPageID(owner)));
          }
      }
      // put the URL in the array
      XPArrayList list = null;
      if (url != null)
      {
        list = new XPArrayList(1);
        list.Add(url);
      }
      return list;
    }
    
    /// <summary>
    /// Gets the URL to the Community Add Portlets Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Community Add Portlets Editor, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetURLForAddCommunityPortletsEditor(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        return null;
      }
      CommonOpenerOpenASURL url = null;
      if (IsCommunityActionsDisplayable(communityAS))
      {
        if (IsCurrentPageCommunityPage(communityAS) && IsCurrentUserCommunityOwner(communityAS))
        {
          String strURLTitle = communityAS.GetString(307, "ptmsgs_portalcommonmsgs");
          // Use current AActivitySpace for the following calls
          String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          String strASName = PortletSelectionEditorAS.STR_MVC_CLASS_NAME;
          int nUserID = TaskAPIUIUser.GetCurrentUserID(communityAS);
          int nCommunityID = TaskAPIUICommunity.GetCurrentCommunityID(communityAS);
          int nPageID = TaskAPIUICommunity.GetCurrentCommunityPageID(communityAS);
          url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
           nUserID);
          url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
          url.AddControlParameter(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORCOMMUNITYID,
           nCommunityID);
          url.AddControlParameter(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTOROBJECTID,
           nPageID);
          url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
          url.AddControlParameter(ASConstants.PARENTSPACEID, communityAS.GetSpaceID());
          url.AddControlParameter(ASConstants.PARENTSPACENAME, communityAS.GetName());
          // Set image
          url.SetURLImageType(PTImageType.ACTION_PORTLET_ADD, strImageServerURL);
          }
      }
      return url;
    }
    
    /// <summary>
    /// Gets the URL to the Community Add Portlets Flyout.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Community Add Portlets Flyout, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetURLForAddCommunityPortletsFlyout(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      CommonOpenerOpenASURL url = null;
      if (IsCommunityActionsDisplayable(owner))
      {
        if (IsCurrentPageCommunityPage(owner) && IsCurrentUserCommunityOwner(owner))
        {
          String strURLTitle = owner.GetString(307, "ptmsgs_portalcommonmsgs");
          String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          String strASName = FlyoutPortletSelectionAS.STR_MVC_CLASS_NAME;
          int nUserID = TaskAPIUIUser.GetCurrentUserID(owner);
          int nCommunityID = TaskAPIUICommunity.GetCurrentCommunityID(owner);
          int nPageID = TaskAPIUICommunity.GetCurrentCommunityPageID(owner);
          url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
           nUserID);
          url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
          url.AddControlParameter(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORCOMMUNITYID,
           nCommunityID);
          url.AddControlParameter(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTOROBJECTID,
           nPageID);
          url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
          url.AddControlParameter(ASConstants.PARENTSPACEID, owner.GetSpaceID());
          url.AddControlParameter(ASConstants.PARENTSPACENAME, owner.GetName());
          // Set image
          url.SetURLImageType(PTImageType.ACTION_PORTLET_ADD, strImageServerURL);
          }
      }
      return url;
    }
    
    /// <summary>
    /// Gets the URL to the Create Community page in the Community Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to Create Community Page in the Community Editor, 
    /// or <c>null</c> if the URL is not valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetURLForCreateCommunityPage(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        return null;
      }
      CommonOpenerOpenObjIDURL url = null;
      if (IsCommunityActionsDisplayable(communityAS))
      {
        if (IsCurrentPageCommunityPage(communityAS) && IsCurrentUserCommunityOwner(communityAS) && communityAS.GetIsAccessStyleStandard())
        {
          String strURLTitle = communityAS.GetString(309, "ptmsgs_portalcommonmsgs");
          // Use current AActivitySpace for the following calls
          String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          int nUserID = TaskAPIUIUser.GetCurrentUserID(communityAS);
          int nCommunityID = TaskAPIUICommunity.GetCurrentCommunityID(communityAS);
          url = new CommonOpenerOpenObjIDURL(strURLTitle, strApplBaseURL,
           PT_CLASSIDS.PT_COMMUNITY_ID, nCommunityID, IPTOpenerConstants.OPENER_MODE_EDIT,
           nUserID);
          url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
          url.AddControlParameter(EditPortletPrefsAS.QS_PARAM_COMMUNITYID,
           nCommunityID);
          url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
          url.AddControlParameter(ASConstants.PARENTSPACEID, communityAS.GetSpaceID());
          url.AddControlParameter(ASConstants.PARENTSPACENAME, communityAS.GetName());
          // Set image
          url.SetURLImageType(PTImageType.ACTION_MYPAGE_NEW, strImageServerURL);
          }
      }
      return url;
    }
    
    /// <summary>
    /// Gets the URL to the Create Community page in the Community Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to Create Community Page in the Community Editor, 
    /// or <c>null</c> if the URL is not valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetURLForCreateCommunityPageOneClick(AActivitySpace owner,
     bool bInheritTemplate, int nPageTemplate)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        return null;
      }
      CommonOpenerOpenASURL url = null;
      if (IsCommunityActionsDisplayable(communityAS))
      {
        if (IsCurrentPageCommunityPage(communityAS) && IsCurrentUserCommunityOwner(communityAS) && communityAS.GetIsAccessStyleStandard())
        {
          String strURLTitle = communityAS.GetString(803, "ptmsgs_portalcommonmsgs");
          // Use current AActivitySpace for the following calls
          String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          int nUserID = TaskAPIUIUser.GetCurrentUserID(communityAS);
          int nCommunityID = TaskAPIUICommunity.GetCurrentCommunityID(communityAS);
          String strASName = CommunityEditorAS.STR_MVC_CLASS_NAME;
          if (nPageTemplate == -1)
          {
            nPageTemplate = PT_INTRINSICS.PT_BLANK_PAGETEMPLATEID;
          }
          url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
           nUserID);
          url.AddControlParameter(ASConstants.CONTROL, PageCreateDefaultControl.STR_MVC_CLASS_NAME);
          url.AddControlParameter(PageCreateDefaultControl.HTMLINPUT_COMMUNITY_ID,
           XPConvert.ToString(nCommunityID));
          url.AddControlParameter(PageCreateDefaultControl.HTMLINPUT_INHERIT_TEMPLATE,
           XPConvert.ToString(bInheritTemplate));
          url.AddControlParameter(PageCreateDefaultControl.HTMLINPUT_PAGE_TEMPLATE_ID,
           XPConvert.ToString(nPageTemplate));
          // Set image
          url.SetURLImageType(PTImageType.ACTION_MYPAGE_NEW, strImageServerURL);
          }
      }
      return url;
    }
    
    /// <summary>
    /// Gets the URL for the Join Community Editor where users add Communities to their My Communities list.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Join Communities Editor, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetURLForJoinCommunitiesEditor(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        // DCA - we don't really need a community or my page AS specifically here
        // so just set to the activity space that was passed in
        communityAS = owner;
      }
      CommonOpenerOpenASURL url = null;
      if ((IsCommunityFeatureEnabledInSubPortal(communityAS) && !TaskAPIUICommon.IsCurrentPageInPreviewMode(communityAS)) && !TaskAPIUIUser.IsCurrentUserGuestUser(communityAS) || (TaskAPIUIUser.IsCurrentUserGuestUser(communityAS) && TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(communityAS)))
      {
        String strURLTitle = communityAS.GetString(297, "ptmsgs_portalcommonmsgs");
        // Use current AActivitySpace for the following calls
        String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
        String strASName = CommunitySelectionEditorAS.STR_MVC_CLASS_NAME;
        int nUserID = TaskAPIUIUser.GetCurrentUserID(communityAS);
        url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
         nUserID);
        url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
        url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_CREATE);
        url.AddControlParameter(ASConstants.PARENTSPACEID, communityAS.GetSpaceID());
        url.AddControlParameter(ASConstants.PARENTSPACENAME, communityAS.GetName());
        // Set image
        url.SetURLImageType(PTImageType.ACTION_COMMUNITIES_JOIN, strImageServerURL);
        }
      return url;
    }
    
    /// <summary>
    /// Gets the URL for the Join Community Editor where users add Communities to their My Communities list.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Join Communities Editor, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetURLForJoinCommunitiesFlyout(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        // DCA - we don't really need a community or my page AS specifically here
        // so just set to the activity space that was passed in
        communityAS = owner;
      }
      CommonOpenerOpenASURL url = null;
      if ((IsCommunityFeatureEnabledInSubPortal(communityAS) && !TaskAPIUICommon.IsCurrentPageInPreviewMode(communityAS)) && !TaskAPIUIUser.IsCurrentUserGuestUser(communityAS) || (TaskAPIUIUser.IsCurrentUserGuestUser(communityAS) && TaskAPIUICommon.IsCurrentPageInEditDefaultProfileMode(communityAS)))
      {
        String strURLTitle = communityAS.GetString(297, "ptmsgs_portalcommonmsgs");
        // Use current AActivitySpace for the following calls
        String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
        String strASName = FlyoutCommunitySelectionAS.STR_MVC_CLASS_NAME;
        int nUserID = TaskAPIUIUser.GetCurrentUserID(communityAS);
        url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
         nUserID);
        url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
        url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_CREATE);
        url.AddControlParameter(ASConstants.PARENTSPACEID, communityAS.GetSpaceID());
        url.AddControlParameter(ASConstants.PARENTSPACENAME, communityAS.GetName());
        // Set image
        url.SetURLImageType(PTImageType.ACTION_COMMUNITIES_JOIN, strImageServerURL);
        }
      return url;
    }
    
    /// <summary>
    /// Gets the URL to the Community Edit Portlet Preferences Editor.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 		<c>IPTURL</c> with URL to the Community Edit Portlet Preferences Editor, or <c>null</c> if the URL is not
    /// valid in the current context.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetURLForEditPortletPreferencesEditor(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        return null;
      }
      CommonOpenerOpenASURL url = null;
      if (IsCommunityActionsDisplayable(communityAS))
      {
        if (IsCurrentPageCommunityPage(communityAS) && IsUserAbleToAccessCurrentCommunity(communityAS))
        {
          String strURLTitle = communityAS.GetString(306, "ptmsgs_portalcommonmsgs");
          // Use current AActivitySpace for the following calls
          String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          String strASName = EditPortletPrefsAS.STR_MVC_CLASS_NAME;
          int nUserID = TaskAPIUIUser.GetCurrentUserID(communityAS);
          int nCommunityID = TaskAPIUICommunity.GetCurrentCommunityID(communityAS);
          int nPageID = TaskAPIUICommunity.GetCurrentCommunityPageID(communityAS);
          url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
           nUserID);
          url.AddControlParameter(ASConstants.CONTROL, EditorStartControl.STR_MVC_CLASS_NAME);
          url.AddControlParameter(EditPortletPrefsAS.QS_PARAM_COMMUNITYID,
           nCommunityID);
          url.AddControlParameter(EditPortletPrefsAS.QS_PARAM_PAGEID, nPageID);
          url.AddControlParameter(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
          url.AddControlParameter(ASConstants.PARENTSPACEID, communityAS.GetSpaceID());
          url.AddControlParameter(ASConstants.PARENTSPACENAME, communityAS.GetName());
          // Set image
          url.SetURLImageType(PTImageType.ACTION_PORTLET_PREF, strImageServerURL);
          }
      }
      return url;
    }
    
    /// <summary>
    /// Checks if the current user is already subscribed to the current Community, i.e. if the current Community
    /// is already on the user's My Community list.
    /// </summary>
    /// <param name="owner">Current activityspace.</param>
    /// <returns>
    /// <c>true</c> if current Community is already subscribed by the user, <c>false</c> if not member, or on User profile community
    /// or  not or if not currently in a Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static bool IsUserMemberOfCurrentCommunity(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      int nCurCommID = GetCurrentCommunityID(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCurCommID))
      {
        return false;
      }
      if (nCurCommID == PT_INTRINSICS.PT_PROFILE_COMMUNITY)
      {
        return true;
      }
      IPTQueryResult qrMyComms = TaskAPIServerCommunity.GetMyCommunityMembership(ptSession,
       true);
      for (int i = 0;i < qrMyComms.RowCount();i++)
      {
        int nCommID = qrMyComms.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
        if (nCurCommID == nCommID)
        {
          return true;
        }
      }
      return false;
    }
    
    /// <summary>
    /// Checks if the current page is a Community Page.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current page is a Community Page, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static bool IsCurrentPageCommunityPage(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      bool bIsCommunityPage = false;
      String strASName = TaskAPIUICommon.GetActivitySpaceName(owner);
      if (CommunityPageAS.STR_MVC_CLASS_NAME.Equals(strASName) || SiteMapAS.STR_MVC_CLASS_NAME.Equals(strASName))
      {
        bIsCommunityPage = true;
      }
      if (FlyoutPortletSelectionAS.STR_MVC_CLASS_NAME.Equals(strASName))
      {
        IObjectSelectionModelRO model = (IObjectSelectionModelRO) TaskAPIUICommon.GetASModel(owner,
         ObjectSelectionModel.STR_MVC_CLASS_NAME);
        if (model.GetSelectorCommunityID() > 0)
        {
          bIsCommunityPage = true;
        }
      }
      return bIsCommunityPage;
    }
    
    /// <summary>
    /// Checks if the current Community is the User Profile Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current Community is the User Profile Community, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static bool IsCurrentCommunityUserProfileCommunity(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      int nCurCommID = GetCurrentCommunityID(owner);
      return (nCurCommID == PT_INTRINSICS.PT_PROFILE_COMMUNITY);
    }
    
    /// <summary>
    /// Checks if the current user has access (requires EDIT access) to use the Community Editor on the current
    /// Community Page.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current user has at least EDIT access to the current Community Page, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static bool IsCurrentUserAbleToEditCurrentCommunityPage(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      bool bAbleToEditCurrentCommunityPage = CheckUserAccessLevelToCurrentCommunityPage(owner,
       PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
      return bAbleToEditCurrentCommunityPage;
    }
    
    /// <summary>
    /// Checks if the current user is the owner of the current Community.
    /// The check only works on Community pages or Community Knowledge directory pages, not in Community editors.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current user has at least EDIT access to the current Community, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static bool IsCurrentUserCommunityOwner(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      bool bIsOwner = CheckUserAccessLevelToCurrentCommunity(owner, PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
      return bIsOwner;
    }
    
    /// <summary>
    /// Checks if the current user has at least SELECT access to the current Community.
    /// Only works on Community pages or Community Knowledge directory pages, not in Community Editors.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current user has at least SELECT access to the current Community, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static bool IsUserAbleToJoinCurrentCommunity(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      bool bCanJoin = TaskAPIUICommunity.CheckUserAccessLevelToCurrentCommunity(owner,
       PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);
      return bCanJoin;
    }
    
    /// <summary>
    /// Checks if the current user has at least READ access to the current Community.
    /// Only works on Community pages or Community Knowledge directory pages, not in Community editors.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current user has at least READ access to the current Community, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static bool IsUserAbleToAccessCurrentCommunity(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      bool bCanJoin = TaskAPIUICommunity.CheckUserAccessLevelToCurrentCommunity(owner,
       PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ);
      return bCanJoin;
    }
    
    /// <summary>
    /// Checks if the current page is a Myportal page, i.e. either a MyPage or Community Page.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	<c>true</c> if the current page is either a MyPage or Community Page, <c>false</c> otherwise.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static bool IsCurrentPageMyPortalPage(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      bool bMyPortalModelExists = TaskAPIUICommon.GetASModelExist(owner, MyPortalModel.STR_MVC_CLASS_NAME);
      return bMyPortalModelExists;
    }
    
    /// <summary>
    /// Gets Community Actions as an XPArrayList with IPTURL entries.
    /// Most Community actions only display when in an Community and only if a user has access 
    /// to perform the actions.  
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>	An <c>XPArrayList</c> with <c>IPTURL</c> entries with Community Actions. </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static XPArrayList GetCommunityActionsAsURLs(AActivitySpace owner)
    {
      return GetCommunityActionsAsURLs(owner, false, true, -1);
    }
    
    /// <summary>
    /// Gets Community Actions as an XPArrayList with IPTURL entries.
    /// Most Community actions only display when in an Community and only if a user has access 
    /// to perform the actions.  
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="includeOneClick">Flag to include the "one-click page creation" link</param>
    /// <returns>	An <c>XPArrayList</c> with <c>IPTURL</c> entries with Community Actions. </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static XPArrayList GetCommunityActionsAsURLs(AActivitySpace owner,
     bool bIncludeOneClick, bool bInheritTemplate, int nPageTemplate)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      XPArrayList listURL = new XPArrayList();
      IPTURL url = null;
      url = GetURLForJoinCurrentCommunityAction(owner);
      if (null != url)
      {
        listURL.Add(url);
      }
      url = GetURLForJoinCommunitiesEditor(owner);
      if (null != url)
      {
        listURL.Add(url);
      }
      url = GetURLForUnsubscribeCurrentCommunityAction(owner);
      if (null != url)
      {
        listURL.Add(url);
      }
      url = GetURLForUnsubscribeCommunitiesEditor(owner);
      if (null != url)
      {
        listURL.Add(url);
      }
      url = GetURLForAddCommunityPortletsEditor(owner);
      if (null != url)
      {
        listURL.Add(url);
      }
      url = GetURLForCurrentCommunityEditor(owner);
      if (null != url)
      {
        listURL.Add(url);
      }
      url = GetURLForCreateCommunityPage(owner);
      if (null != url)
      {
        listURL.Add(url);
      }
      if (bIncludeOneClick)
      {
        url = GetURLForCreateCommunityPageOneClick(owner, bInheritTemplate,
         nPageTemplate);
        if (null != url)
        {
          listURL.Add(url);
        }
      }
      url = GetURLForEditPortletPreferencesEditor(owner);
      if (null != url)
      {
        listURL.Add(url);
      }
      return listURL;
    }
    
    /// <summary>
    /// Helper function to create an URL to a specified Community. 
    /// The <c>PTURLConstants.KEY_CURRENTMARKER</c> flag is set on the URL if is leading to the current
    /// Community.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID of the Community to create an URL to.</param>
    /// <param name="strCommunityName">Name of the URL, normally the name of the Community.</param>
    /// <returns>	<c>IPTURL</c> with an URL to the specified Community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL CreateCommunityCommonOpenerURL(AActivitySpace owner,
     int nCommunityID, String strCommunityName)
    {
      // DCA - check if it's the user profile community and create a user profile URL instead
      if (PT_INTRINSICS.PT_PROFILE_COMMUNITY == nCommunityID)
      {
        int userId = TaskAPIUIUser.GetCurrentUserID(owner);
        String userName = TaskAPIUIUser.GetCurrentUserName(owner);
        return TaskAPIUICommunity.CreateUserProfilePageCommonOpenerURL(owner,
         userId, userName, strCommunityName);
      }
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
      IPTURL url = null;
      if (TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_COMMUNITY_ID))
      {
        url = new CommonOpenerFriendlyURL(strCommunityName, PT_CLASSIDS.PT_COMMUNITY_ID,
         nCommunityID, owner);
      } else
      {
        String strAppBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        int nUserID = TaskAPIUICommon.GetPTSession(owner).GetSessionInfo().GetCurrentUserID();
        url = new CommonOpenerOpenObjIDURL(strCommunityName, strAppBaseURL,
         PT_CLASSIDS.PT_COMMUNITY_ID, nCommunityID, IPTOpenerConstants.OPENER_MODE_VIEW,
         nUserID);
      }
      ((ACommonOpenerURL) url).SetURLImageType(PTImageType.ICON_COMMUNITY,
       strImageServerURL);
      if (nCommunityID == TaskAPIUICommunity.GetCurrentCommunityID(owner))
      {
        url.SetURLData(PTURLConstants.KEY_CURRENTMARKER, "");
      }
      return url;
    }
    
    /// <summary>
    /// Helper function to create an URL to a specified Community Page. 
    /// The <c>PTURLConstants.KEY_CURRENTMARKER</c> flag is set on the URL if is leading to the current
    /// Community Page. In case the page is a profile page, the link generated will be of the viewed profile,
    /// not the current user's.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID of the Community where the Page is located in.</param>
    /// <param name="nCommunityPageID">Object ID of the Page to create a URL to.</param>
    /// <param name="strCommunityPageName">Title of the URL, normally the name of the Page.</param>
    /// <returns><c>IPTURL</c> with an URL to the specified Community Page.</returns>
    /// <throws>
    /// XPIllegalArgumentException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL CreateCommunityPageCommonOpenerURL(AActivitySpace owner,
     int nCommunityID, int nCommunityPageID, String strCommunityPageName)
    {
      // DCA - check if it's the user profile community and create a user profile URL instead
      if (PT_INTRINSICS.PT_PROFILE_COMMUNITY == nCommunityID)
      {
        // DCA - we're getting the user profile Id here in case we are viewing someone's profile;
        // we want the other page's link to be the viewed profile not the user's
        int userId = TaskAPIUICommunity.GetUserProfileId(owner);
        // We don't have the profile name available, so just have it look it up from the cache.
        return TaskAPIUICommunity.CreateUserProfilePageCommonOpenerURL(owner,
         userId, null, strCommunityPageName, nCommunityPageID, strCommunityPageName);
        }
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
      IPTURL url = null;
      if (TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_COMMUNITY_ID))
      {
        if (nCommunityPageID == 0)
        {
          // go to community home page
          url = new CommonOpenerFriendlyURL(strCommunityPageName, PT_CLASSIDS.PT_COMMUNITY_ID,
           nCommunityID, owner);
        } else
        {
          url = new CommonOpenerFriendlyURL(strCommunityPageName, PT_CLASSIDS.PT_COMMUNITY_ID,
           nCommunityID, nCommunityPageID, owner);
        }
      } else
      {
        String strAppBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        int nUserID = TaskAPIUICommon.GetPTSession(owner).GetSessionInfo().GetCurrentUserID();
        url = new CommonOpenerOpenObjIDURL(strCommunityPageName, strAppBaseURL,
         PT_CLASSIDS.PT_COMMUNITY_ID, nCommunityID, IPTOpenerConstants.OPENER_MODE_VIEW,
         nUserID);
        url.AddControlParameter(PTCommunities.STR_PAGE_ID, nCommunityPageID);
        }
      //url.SetURLData(PTURLConstants.KEY_CLASSID, XPConvert.ToString(PT_CLASSIDS.PT_PAGE_ID));
      //url.SetURLData(PTURLConstants.KEY_OBJECTID, XPConvert.ToString(nCommunityPageID));
      ((ACommonOpenerURL) url).SetURLImageType(PTImageType.ICON_COMMUNITY,
       strImageServerURL);
      if (nCommunityID == TaskAPIUICommunity.GetCurrentCommunityID(owner) && nCommunityPageID == TaskAPIUICommunity.GetCurrentCommunityPageID(owner))
      {
        url.SetURLData(PTURLConstants.KEY_CURRENTMARKER, "");
      }
      return url;
    }
    private static int GetUserProfileId(AActivitySpace owner)
    {
      IPTSession session = TaskAPIUICommon.GetPTSession(owner);
      // Determine user name and ID - the user info object stores
      // data about the current profile being viewed - checking this
      // allows us to avoid a DB query.
      int nUserID = session.GetUserInfo().GetProfileUserID();
      // If we can't find the user ID, it probably means that the user profile
      // community hasn't been properly initialized and is displaying the
      // incorrect user data, so we're in a pretty bad shape.  This used to
      // happen when we were redirected here from an editor finishing and the
      // redirect URL was missing the profile ID, but now the default control
      // resets the profile ID, so this shouldn't happen anymore.
      // ID of 0 means not a user profile page
      // ID of -1 means view current user, but this should be replaced with
      // the actual user ID during community page init, and if this hasn't
      // happened yet, we're in trouble.
      if (nUserID <= 0)
      {
        log.Debug("Unable to find user profile ID when displaying the user profile community.  " + "The profile page does not appear to have been correctly initialized.");
        // Try to get the profile information (which user are we looking at?)
        // from the model.  This may or may not work.
        if (TaskAPIUICommon.GetMyPortalPageModelExists(owner))
        {
          MyPortalModel ptModel = TaskAPIUICommon.GetMyPortalPageModel(owner);
          nUserID = ptModel.GetProfileID();
        }
      }
      if (nUserID <= 0)
      {
        // DCA - if all else fails, set to the current user
        nUserID = TaskAPIUIUser.GetCurrentUserID(owner);
      }
      return nUserID;
    }
    
    /// <summary>
    /// Gets an URL to a UserProfile page for the specified user.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nProfileUserID">The Object ID of the user to open the User Profile page for.</param>
    /// <param name="sProfileUserName">The name of the user to open the User Profile page for.  Also used for the link title.</param>
    /// <returns>	<c>IPTURL</c> with the URL to the user profile page for the specified user.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner,
     int nProfileUserID, String sProfileUserName)
    {
      return TaskAPIUICommunity.CreateUserProfilePageCommonOpenerURL(owner,
       nProfileUserID, sProfileUserName, sProfileUserName, 0, null);
    }
    
    /// <summary>
    /// Gets an URL to a UserProfile page for the specified user.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nProfileUserID">The Object ID of the user to open the User Profile page for.</param>
    /// <param name="sProfileUserName">The name of the user to open the User Profile page for.</param>
    /// <param name="strURLTitle">The title of the URL.</param>
    /// <returns>	<c>IPTURL</c> with the URL to the user profile page for the specified user.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner,
     int nProfileUserID, String sProfileUserName, String strURLTitle)
    {
      return TaskAPIUICommunity.CreateUserProfilePageCommonOpenerURL(owner,
       nProfileUserID, sProfileUserName, strURLTitle, 0, null);
    }
    
    /// <summary>
    /// Gets an URL to a UserProfile page for the specified user.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nProfileUserID">The Object ID of the user to open the User Profile page for.</param>
    /// <param name="sProfileUserName">The name of the user to open the User Profile page for.</param>
    /// <param name="strURLTitle">The title of the URL.</param>
    /// <returns>	<c>IPTURL</c> with the URL to the user profile page for the specified user.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner,
     int nProfileUserID, String sProfileUserName, String strURLTitle, int nPageId,
     String strPageName)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
      IPTURL url = null;
      if (TaskAPIUICommon.UseFriendlyURLS(PT_CLASSIDS.PT_USER_ID))
      {
        url = new CommonOpenerFriendlyURL(strURLTitle, PT_CLASSIDS.PT_USER_ID,
         nProfileUserID, sProfileUserName, nPageId, strPageName, owner);
      } else
      {
        String strAppBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
        int nCurrUserID = TaskAPIUICommon.GetPTSession(owner).GetSessionInfo().GetCurrentUserID();
        int nUserProfileCommunityID = PT_INTRINSICS.PT_PROFILE_COMMUNITY;
        url = new CommonOpenerOpenObjIDURL(strURLTitle, strAppBaseURL, PT_CLASSIDS.PT_COMMUNITY_ID,
         nUserProfileCommunityID, IPTOpenerConstants.OPENER_MODE_VIEW, nCurrUserID);
        url.AddControlParameter(SetCommunityControl.PROFILEID, nProfileUserID);
        }
      ((ACommonOpenerURL) url).SetURLImageType(PTImageType.ICON_USER, strImageServerURL);
      if (PT_INTRINSICS.PT_PROFILE_COMMUNITY == TaskAPIUICommunity.GetCurrentCommunityID(owner) && (nPageId == 0 || nPageId == TaskAPIUICommunity.GetCurrentCommunityPageID(owner)))
      {
        url.SetURLData(PTURLConstants.KEY_CURRENTMARKER, "");
      }
      return url;
    }
    
    /// <summary>
    /// Check the if the current user has at least the accesslevel specified to the current
    /// community 
    /// </summary>
    /// <param name="AccessLevel">to check for. Example: PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ or PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT</param>
    /// <param name="nCommunityID">Object ID of the Community to check security of</param>
    /// <returns>boolean			True if the current user has at least the specified accesslevel, false if the user does not have access or there is no current community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    /// <seealso cref="PT_ACCESS_LEVELS"></seealso>
    public static bool CheckUserAccessLevelToCommunity(AActivitySpace owner,
     int nAccessLevel, int nCommunityID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      bool bHasAccess = false;
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCommunityID))
      {
        return bHasAccess;
      }
      IPTCommunityInfo ptCommInfo = TaskAPIServerCommunity.GetCommunityInfoObject(ptSession,
       nCommunityID);
      // mj: pretend that everyone has only read access to the profile community, 
      // so that the "edit" button will be hidden.
      //		if (ptCommInfo.GetObjectID() == PT_INTRINSICS.PT_PROFILE_COMMUNITY)
      //		{
      //			bHasAccess = (nAccessLevel <= PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ);
      //		}
      //		else
      //		{
      bHasAccess = (nAccessLevel <= ptCommInfo.GetAccessLevel());
      //		}
      return bHasAccess;
    }
    
    /// <summary>
    /// Check the if the current user has at least the accesslevel specified to the current
    /// community 
    /// </summary>
    /// <param name="AccessLevel">to check for. Example: PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ or PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT</param>
    /// <returns>boolean			True if the current user has at least the specified accesslevel, false if the user does not have access or there is no current community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    /// <seealso cref="PT_ACCESS_LEVELS"></seealso>
    public static bool CheckUserAccessLevelToCurrentCommunity(AActivitySpace owner,
     int nAccessLevel)
    {
      int nCurrCommID = TaskAPIUICommunity.GetCurrentCommunityID(owner);
      return CheckUserAccessLevelToCommunity(owner, nAccessLevel, nCurrCommID);
      }
    
    /// <summary>
    /// Check the if the current user has at least the accesslevel specified to the current
    /// community page
    /// </summary>
    /// <param name="AccessLevel">to check for. Example: PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ or PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT</param>
    /// <returns>boolean			True if the current user has at least the specified accesslevel, false if the user does not have access or there is no current community page.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    /// <seealso cref="PT_ACCESS_LEVELS"></seealso>
    public static bool CheckUserAccessLevelToCurrentCommunityPage(AActivitySpace owner,
     int nAccessLevel)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      bool bHasAccess = false;
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      int nCurrCommID = TaskAPIUICommunity.GetCurrentCommunityID(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCurrCommID))
      {
        return bHasAccess;
      }
      // mj: pretend that everyone has only read access to the profile community, 
      // so that the "edit" button will be hidden.
      if (TaskAPIUICommunity.IsCurrentCommunityUserProfileCommunity(owner))
      {
        bHasAccess = (nAccessLevel <= PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ);
        } else
      {
        int nCurrPageID = TaskAPIUICommunity.GetCurrentCommunityPageID(owner);
        if (!TaskAPIUICommon.IsValidObjectIDArgument(nCurrCommID))
        {
          return bHasAccess;
        }
        IPTPageInfo ptPageInfo = ((IPTPageManager) ptSession.GetPages()).CachedOpenPageInfo(nCurrPageID,
         false, false);
        bHasAccess = (nAccessLevel <= ptPageInfo.GetAccessLevel());
      }
      return bHasAccess;
    }
    
    /// <summary>
    /// Gets the Object ID of the parent Community in the specified Community.  (the Community where the current Community is located in the administrative hierarchy).
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID of the Community to get the parent Community from.</param>
    /// <returns>Object ID of current parent Community ID, or <c>0 /code>  if non-existant.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static int GetParentCommunityID(AActivitySpace owner, int nCommunityID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      TaskAPIUICommon.ValidateObjectIDArgument(nCommunityID);
      IPTSession session = TaskAPIUICommon.GetPTSession(owner);
      IPTQueryResult qrParentComm = TaskAPIServerCommunity.GetCommunityParentComm(session,
       nCommunityID);
      int nParentCommID = 0;
      if (qrParentComm.RowCount() == 1)
      {
        nParentCommID = qrParentComm.ItemAsInt(0, PT_PROPIDS.PT_PROPID_OBJECTID);
        }
      return nParentCommID;
    }
    
    /// <summary>
    /// Gets the Object ID of the parent Community in the current Community. 
    /// Parent Community is the Community where the current Community is located in the administrative hierarchy.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// Object ID of current parent Community ID, or <c>0</c> if not currently in a Community or
    /// the current Community does not have a parent Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static int GetCurrentParentCommunityID(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      int nCurrCommID = GetCurrentCommunityID(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCurrCommID))
      {
        return 0;
      }
      int id = GetParentCommunityID(owner, nCurrCommID);
      return id;
    }
    
    /// <summary>
    /// Gets Related Communities (a.k.a. peer Communities) of the current Community as an integer array.
    /// Related Communities are Communities located in one Community in the administrative hierarchy.  
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// Array with Object IDs of Related Community to the current Community.  
    /// Empty array if the current Community has no Related communities or if not in a Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static int[] GetCurrentRelatedCommununityIDs(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      int nCurrCommID = GetCurrentCommunityID(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCurrCommID))
      {
        return new int[0];
      }
      int[] ids = GetRelatedCommununityIDs(owner, nCurrCommID);
      return ids;
    }
    
    /// <summary>
    /// Gets the URL to Join Current Community Action.
    /// The URL goes to a page that subscribes the user to the current Community. This URL is only visible
    /// on Communities a user is not already member of.  
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns><c>IPTURL</c> to join current community or <c>null</c> if the current user is already a member or does not have access to join the current community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetURLForJoinCurrentCommunityAction(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      CommonOpenerOpenASURL url = null;
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        return null;
      }
      if (IsCommunityActionsDisplayable(communityAS))
      {
        if (!IsUserMemberOfCurrentCommunity(communityAS) && IsUserAbleToJoinCurrentCommunity(communityAS))
        {
          String strURLTitle = communityAS.GetString(653, "ptmsgs_portalbrowsingmsgs");
          // Use current AActivitySpace for the following calls
          String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          String strASName = CommunityPageAS.STR_MVC_CLASS_NAME;
          int nUserID = TaskAPIUIUser.GetCurrentUserID(communityAS);
          url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
           nUserID);
          url.AddControlParameter(ASConstants.CONTROL, JoinCurCommunityControl.STR_MVC_CLASS_NAME);
          url.AddControlParameter(SetCommunityControl.STR_COMMUNITY_ID, GetCurrentCommunityID(communityAS));
          // Set image
          url.SetURLImageType(PTImageType.ACTION_COMMUNITIES_JOIN, strImageServerURL);
          }
      }
      return url;
    }
    
    /// <summary>
    /// Gets the URL to Unsubscribe Current Community Action.
    /// The URL goes to a page that unsubscribes the user to the current Community. This URL is only visible
    /// on Communities a user is not already member of.  
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns><c>IPTURL</c> to unsubscribe current community or <c>null</c> if the current user is already not a member or does not have access to unsubscribe the current community.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetURLForUnsubscribeCurrentCommunityAction(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      CommonOpenerOpenASURL url = null;
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need to pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        return null;
      }
      if (IsCommunityActionsDisplayable(communityAS))
      {
        if (IsUserMemberOfCurrentCommunity(communityAS) && IsUserAbleToJoinCurrentCommunity(communityAS))
        {
          int nCurrentCommunityID = GetCurrentCommunityID(communityAS);
          // true means use cached data
          IPTQueryResult ptMemberships = TaskAPIServerCommunity.GetMyCommunityMembership(TaskAPIUICommon.GetPTSession(communityAS),
           true);
          // Check if the current community is mandatory.
          if (null != ptMemberships)
          {
            for (int i = 0;i < ptMemberships.RowCount();i++)
            {
              int nCommunityID = ptMemberships.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
              if (nCommunityID == nCurrentCommunityID)
              {
                int nMembershipType = ptMemberships.ItemAsInt(i, PT_PROPIDS.PT_PROPID_MEMBERSHIP_TYPE);
                if ((nMembershipType == PT_COMMUNITY_MEMBERSHIPTYPE.PT_COMMUNITYMEMBER_TAB) || (nMembershipType == PT_COMMUNITY_MEMBERSHIPTYPE.PT_COMMUNITYMEMBER_MANDATORY))
                {
                  // Can't unsubscribe from the current community if it's mandatory,
                  // so just return null.
                  return null;
                }
              }
            }
          }
          String strURLTitle = communityAS.GetString(1919, "ptmsgs_portalbrowsingmsgs");
          // Use current AActivitySpace for the following calls
          String strApplBaseURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(owner);
          String strImageServerURL = TaskAPIUIPortalConfig.GetImageServerBaseURL(owner);
          String strASName = CommunityPageAS.STR_MVC_CLASS_NAME;
          int nUserID = TaskAPIUIUser.GetCurrentUserID(communityAS);
          url = new CommonOpenerOpenASURL(strURLTitle, strApplBaseURL, strASName,
           nUserID);
          url.AddControlParameter(ASConstants.CONTROL, UnsubscribeCurCommunityControl.STR_MVC_CLASS_NAME);
          url.AddControlParameter(SetCommunityControl.STR_COMMUNITY_ID, GetCurrentCommunityID(communityAS));
          // Set image
          url.SetURLImageType(PTImageType.ACTION_COMMUNITIES_UNSUB, strImageServerURL);
          }
      }
      return url;
    }
    
    /// <summary>
    /// Gets the Subcommunities of the current Community as a integer array.
    /// Subcommunties are Communities located in another Community in the adminstrative hierarchy.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// An integer array with Object IDs to Subcommunity.  
    /// Returns an empty array if the current Community has no Subcommunities or not currently in a Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static int[] GetCurrentSubCommunityIDs(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      int nCurrCommID = GetCurrentCommunityID(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCurrCommID))
      {
        return new int[0];
      }
      int[] ids = GetSubCommununityIDs(owner, nCurrCommID);
      return ids;
    }
    
    /// <summary>
    /// Gets the Community Pages of the current Community as an <c>XPArraylist</c> with <c>IPTURL</c> entries.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>An <c>XPArrayList</c> with <c>IPTURL</c> to CommunityPages, or empty <c>XPArrayList</c> if the current page is not a Community Page or valid Page.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static XPArrayList GetCurrentCommunityPagesAsURLs(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      int nCurrCommID = 0;
      // This condition is to return the list of URLs when the Gateway Page is in
      // community and displays an outside page in hosted display mode. 
      if (XPStringUtility.EqualsIgnoreCase(owner.GetName(), GatewayAS.STR_MVC_CLASS_NAME))
      {
        nCurrCommID = GetCurrentCommunityID(owner);
        if (nCurrCommID > 0)
        {
          return GetCommunityPagesAsURLs(owner, nCurrCommID);
        }
      }
      // This will get the Parent activity space if the current space is not CommunityAS
      // We need pass in the correct space for the checks below.
      AActivitySpace communityAS = TaskAPIUICommon.GetMyPageOrCommunityAActivitySpace(owner);
      if (communityAS == null)
      {
        return new XPArrayList();
      }
      nCurrCommID = GetCurrentCommunityID(communityAS);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCurrCommID))
      {
        return new XPArrayList();
      }
      XPArrayList list = GetCommunityPagesAsURLs(owner, nCurrCommID);
      return list;
    }
    
    /// <summary>
    /// Gets URLs to Related Communities for for the current Community in a list of <c>IPTURL</c>s.
    /// Related Communities are Communities in the same administrative folder.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns><c>XPArrayList</c> with <c>IPTURL</c>s to Related Communities or an empty <c>XPArrayList</c> if none exists.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static XPArrayList GetCurrentRelatedCommunitiesAsURLs(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      int nCurrCommID = GetCurrentCommunityID(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCurrCommID))
      {
        return new XPArrayList();
      }
      XPArrayList list = GetRelatedCommununitiesAsURLs(owner, nCurrCommID);
      return list;
    }
    
    /// <summary>
    /// Gets the URL to the parent Community of the current Community. 
    /// Parent Community is the Community where the current Community is located in the administrative hierarchy.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// IPTURL to current Parent Community  or <c>null</c> if no parent Community exists, or if not currently
    /// in a Community.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetCurrentParentCommunityAsURL(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      int nCurrCommID = GetCurrentCommunityID(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCurrCommID))
      {
        return null;
      }
      IPTURL ptURL = GetParentCommunityAsURL(owner, nCurrCommID);
      return ptURL;
    }
    
    /// <summary>
    /// Gets the URL to the parent Community of the specified Community. 
    /// Parent Community is the Community where the current Community is located in the administrative hierarchy.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID"></param>
    /// <returns>
    ///  IPTURL to current Parent Community  or <c>null</c> if no parent Community exists, or if the 
    /// specified Community is invalid.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static IPTURL GetParentCommunityAsURL(AActivitySpace owner, int nCommunityID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCommunityID))
      {
        return null;
      }
      IPTSession session = TaskAPIUICommon.GetPTSession(owner);
      // get the parent community
      IPTQueryResult qrParentComm = null;
      try
      {
        qrParentComm = TaskAPIServerCommunity.GetCommunityParentComm(session,
         nCommunityID);
      }
      catch (XPException e)
      {
        log.Warn(e, "Exception encountered while retrieving Parent Community ID for Community: " + nCommunityID);
        return null;
      }
      IPTURL url = null;
      if (qrParentComm.RowCount() == 1)
      {
        // Get the pageid and page name from the queryresult
        int nParentCommID = qrParentComm.ItemAsInt(0, PT_PROPIDS.PT_PROPID_OBJECTID);
        String strParentCommName = qrParentComm.ItemAsString(0, PT_PROPIDS.PT_PROPID_NAME);
        // Create a common opener url
        url = CreateCommunityCommonOpenerURL(owner, nParentCommID, strParentCommName);
        }
      return url;
    }
    
    /// <summary>
    /// Gets the Subcommunities for the specified Community as a list with <c>IPTURL</c> entries.
    /// Subcommunities are Community located below another Community in the administrative hierarchy. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="nCommunityID">Object ID for the Community to get Subcommunities from.</param>
    /// <returns>
    /// 	An <c>XPArrayList</c> with <c>IPTURL</c> entries to Subcommunities, or
    /// empty array if the specified Community does no thave any Subcommunities or it is invalid.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static XPArrayList GetSubCommununitiesAsURLs(AActivitySpace owner,
     int nCommunityID)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      if (!TaskAPIUICommon.IsValidObjectIDArgument(nCommunityID))
      {
        return new XPArrayList();
      }
      IPTSession session = TaskAPIUICommon.GetPTSession(owner);
      // get the subcommunities 
      IPTQueryResult qrSubComms = TaskAPIServerCommunity.GetCommunitySubCommunities(session,
       nCommunityID);
      // create the arraylist to return
      XPArrayList list = new XPArrayList(qrSubComms.RowCount());
      for (int i = 0;i < qrSubComms.RowCount();i++)
      {
        // Get the pageid and page name from the queryresult
        int nSubCommID = qrSubComms.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
        String strSubCommName = qrSubComms.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
        // Create a common opener url
        IPTURL url = CreateCommunityCommonOpenerURL(owner, nSubCommID, strSubCommName);
        // Add the link to the arraylist
        list.Add(url);
      }
      return list;
    }
    
    /// <summary>
    /// Gets the default Community ID, the first Community in a user's My Community list.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <returns>
    /// 	Object ID of the first Community in the current user's My Community list, or <c>-1</c> if
    /// the user is not a member of any communities.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static int GetDefaultCommunityID(AActivitySpace owner)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTQueryResult ptCommMembership = null;
      ptCommMembership = TaskAPIServerCommunity.GetMyCommunityMembership(TaskAPIUICommon.GetPTSession(owner),
       true);
      if (ptCommMembership.RowCount() == 0)
      {
        return -1;
      } else
      {
        int nFirstCommunityID = ptCommMembership.ItemAsInt(0, PT_PROPIDS.PT_PROPID_OBJECTID);
        return nFirstCommunityID;
      }
    }
    
    /// <summary>
    /// Gets a user's My Communities list as an <c>XPArrayList</c> with <c>IPTURL</c> entries.
    /// The <c>PTURLConstants.KEY_CURRENTMARKER</c> flag is set on an URL entry if is leading to the current
    /// Community. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="bUseCachedData">
    /// Get the current user's mandatory Communities with Tabs from a cache that contains at 
    /// most 15 minutes old data if true. Will query the database for the mandatory communities if false.
    /// </param>
    /// <returns>
    /// 	<c>XPArrayList</c> with <c>IPTURL</c>s to the current user's My Communities ,
    /// or <c>null</c> if Communities features is disabled in the current Experience Definition.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static XPArrayList GetMyCommunitiesAsURLs(AActivitySpace owner,
     bool bUseCachedData)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      if (TaskAPIUICommon.IsCurrentPageInPreviewMode(owner))
      {
        return new XPArrayList();
      }
      IPTQueryResult qrMyComms = TaskAPIServerCommunity.GetMyCommunityMembership(ptSession,
       bUseCachedData);
      XPArrayList list = null;
      if (null != qrMyComms && IsCommunityFeatureEnabledInSubPortal(owner))
      {
        list = new XPArrayList(qrMyComms.RowCount());
        int COL_COMM_NAME = 0;
        int COL_COMM_ID = 1;
        int NUM_COLS = 2;
        int[] arrColumns = new int[]{COL_COMM_NAME, COL_COMM_ID};
        int[][] arrSort = new int[1][];
        for (int j2c_index = 0;j2c_index < 1;j2c_index++)
        {
          arrSort[j2c_index] = new int[2];
        }
        arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID] = COL_COMM_NAME;
        arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
        Object[][] arrData = new Object[NUM_COLS][];
        for (int j2c_index = 0;j2c_index < NUM_COLS;j2c_index++)
        {
          arrData[j2c_index] = new Object[qrMyComms.RowCount()];
        }
        for (int i = 0;i < qrMyComms.RowCount();i++)
        {
          arrData[COL_COMM_ID][i] = qrMyComms.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
          arrData[COL_COMM_NAME][i] = qrMyComms.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
          }
        PTSortedArrayWrapper saData = new PTSortedArrayWrapper();
        saData.Initialize(arrColumns, arrData, arrSort, true);
        for (int i = 0;i < saData.GetCount();i++)
        {
          // Don't think we need to show no link for current comm
          int nCommID = XPConvert.ToInteger(saData.GetItem(i, COL_COMM_ID));
          String strCommName = XPConvert.ToString(saData.GetItem(i, COL_COMM_NAME));
          IPTURL url = CreateCommunityCommonOpenerURL(owner, nCommID, strCommName);
          list.Add(url);
        }
      }
      return list;
    }
    
    /// <summary>
    /// Gets a user's Mandatory with Tab Communities as an <c>XPArrayList</c>  with <c>IPTURL</c> entries. 
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="bUseCachedData">
    /// Get the current user's mandatory Communities with Tabs from a cache that contains at most 15 minutes old data if true. Will query the database 
    /// for the mandatory communities if false.
    /// </param>
    /// <returns>
    /// <c>XPArrayList</c> with <c>IPTURL</c>s to the current user's mandatory Communities with Tabs,
    /// or <c>null</c> if Communities features is disabled in the current Experience Definition.
    /// </returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static XPArrayList GetMyMandatoryCommunitiesWithTabsAsURLs(AActivitySpace owner,
     bool bUseCachedData)
    {
      TaskAPIUICommon.ValidateActivitySpaceArgument(owner);
      IPTSession ptSession = TaskAPIUICommon.GetPTSession(owner);
      if (TaskAPIUICommon.IsCurrentPageInPreviewMode(owner))
      {
        return new XPArrayList();
      }
      IPTQueryResult ptMandComms = TaskAPIServerCommunity.GetMandatoryCommunitiesWithTabsForCurrentUser(ptSession,
       bUseCachedData);
      XPArrayList list = null;
      if (null != ptMandComms)
      {
        list = new XPArrayList(ptMandComms.RowCount());
        for (int i = 0;i < ptMandComms.RowCount();i++)
        {
          int nCommID = ptMandComms.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID);
          String strCommName = ptMandComms.ItemAsString(i, PT_PROPIDS.PT_PROPID_NAME);
          IPTURL url = CreateCommunityCommonOpenerURL(owner, nCommID, strCommName);
          list.Add(url);
        }
      }
      return list;
    }
    
    /// <summary>
    /// Gets the Community ID from an IPTURL with an URL to a Community or Community Page.
    /// </summary>
    /// <param name="ptURL"><c>IPTURL</c> to a Community or Community Page.</param>
    /// <returns>The Community ID, or <c>0</c> if none is found in the URL.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static int GetCommunityIDFromPTURL(IPTURL ptURL)
    {
      if (null == ptURL)
      {
        return 0;
      }
      int nCommID = 0;
      String strCommID = "";
      if (ptURL is CommonOpenerFriendlyURL)
      {
        // DCA - handle profile community properly
        if (XPConvert.ToInteger(ptURL.GetData(CommonOpenerFriendlyURL.KEY_CLASSID)) == PT_CLASSIDS.PT_USER_ID)
        {
          nCommID = PT_INTRINSICS.PT_PROFILE_COMMUNITY;
        } else
        {
          strCommID = ptURL.GetData(CommonOpenerFriendlyURL.KEY_FRIENDLYURL_OBJECTID);
          }
      } else
      if (ptURL is CommonOpenerOpenObjIDURL)
      {
        strCommID = ptURL.GetData(CommonOpenerOpenObjIDURL.KEY_COMMONOPENER_OBJECTID);
        } else
      {
        throw new XPException("Unsupported IPTURL type!");
      }
      if (null != strCommID && strCommID.Length > 0)
      {
        try
        {
          nCommID = XPConvert.ToInteger(strCommID);
        }
        catch (XPFormatException e)
        {
        }
      }
      // Don't do anything, we will just return the default page id
      return nCommID;
    }
    
    /// <summary>
    /// Gets the Community Page ID from an IPTURL with an URL to a Community Page.
    /// </summary>
    /// <param name="ptURL"><c>IPTURL</c> to a Community Page.</param>
    /// <returns>The Community Page ID, or <c>0</c> if none is found in the URL.</returns>
    /// <throws>
    /// XPIllegalAccessException
    /// 			If the user does not have sufficient access to the Community.
    /// </throws>
    public static int GetCommunityPageIDFromPTURL(IPTURL ptURL)
    {
      if (null == ptURL)
      {
        return 0;
      }
      int nPageID = 0;
      String strPageID = null;
      if (ptURL is CommonOpenerFriendlyURL)
      {
        strPageID = ptURL.GetData(CommonOpenerFriendlyURL.KEY_FRIENDLYURL_CHILDID);
        } else
      if (ptURL is CommonOpenerOpenObjIDURL)
      {
        strPageID = ptURL.GetData(PTCommunities.STR_PAGE_ID);
      } else
      {
        throw new XPException("Unsupported IPTURL type!");
      }
      if (null != strPageID)
      {
        try
        {
          nPageID = XPConvert.ToInteger(strPageID);
        }
        catch (XPFormatException e)
        {
        }
      }
      // Don't do anything, we will just return the default page id
      return nPageID;
    }
  }
}