using System;
using com.plumtree.openfoundation.web;
using com.plumtree.portalpages.common.uiparts;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taskapi.portalui
{
  /// <summary>
  /// HTML Related TaskAPI functions.
  /// </summary>
  public class TaskAPIUIHTML
  {
    
    /// <summary>
    /// Adds a spacer cell with a cell width of 4to the specified HTMLTableRow
    /// </summary>
    /// <param name="myTableRow">HTMLTableRow to add the spacer cell to</param>
    public static void CreateHorizontalSpacerCell(HTMLTableRow myTableRow)
    {
      NavigationCommonHelpers.CreateHorizontalSpacerCell(myTableRow);
    }
    
    /// <summary>
    /// Gets an HTML Image element for the specified Plumtree Image Construct.
    /// </summary>
    /// <param name="owner">Current Activityspace.</param>
    /// <param name="construct">A Plumtree Image Construct.</param>
    /// <returns>	HTML img element created from the Plumtree Image Construct object.</returns>
    public static HTMLImg GetImageConstruct(AActivitySpace owner, PTConstructType construct)
    {
      return (HTMLImg) UIConstructFactory.GetConstruct(construct, true, owner).GetHTMLElement();
      }
    
    /// <summary>
    /// Gets a blank spacer image element with the specified width and height.
    /// </summary>
    /// <param name="strWidth">Width value that goes directly into the width attribute in the HTML img element.</param>
    /// <param name="strHeight">Height value that goes directly into the width attribute in the HTML img element.</param>
    /// <returns>	HTMLImg element with a spacer image in the specified width and height. </returns>
    public static HTMLImg GetSpacerImage(String strWidth, String strHeight)
    {
      return PlumtreeHelpers.GetSpacerImage(strWidth, strHeight);
    }
    
    /// <summary>
    /// Returns an HTML-encoded version of the input string, or
    /// empty string if input is null. 
    /// </summary>
    /// <param name="strValue">a string of characters to encode for use as HTML text.</param>
    /// <returns>an encoded version of the input, or empty string if <c>null</c>.</returns>
    public static String HTMLEncode(String strValue)
    {
      return XPHttpUtility.HtmlEncode(strValue);
    }
    
    /// <summary>
    /// Returns a decoded version of the input string, 
    /// assuming it was a valid HTML encoding.
    /// </summary>
    /// <param name="strValue">the string to decode.</param>
    /// <returns>a decoded version of the input.</returns>
    /// <throws>XPNullPointerException if input is <c>null</c>.</throws>
    /// <throws>XPUnsupportedEncodingException if an invalid ampersand escape is found.</throws>
    public static String HTMLDecode(String strValue)
    {
      return XPHttpUtility.HtmlDecode(strValue);
    }
  }
}