package com.plumtree.portalpages.admin.approval.ptobject;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.portalpages.common.objectapproval.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.approval.*;
import com.plumtree.uiinfrastructure.tips.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 *
 *
 * @author $author$
 */
public class ObjectApprovalAS extends PTEditorAS
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.approval.ptobject.ObjectApprovalAS");



	public static final String STR_MVC_CLASS_NAME = "ObjectApproval";

	private static final String sClassName = "com.plumtree.portalpages.admin.approval.ptobject.ObjectApprovalAS";

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ObjectApprovalAS();
	}

	/**
	 * Returns the Editor title.
	 *
	 * @return String
	 */
	public String GetEditorTitle()
	{
		return (GetString(1201, ResourceConstants.PORTALADMINMSGS));
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}


	/**
	 * Returns the class name of the Property Repost Control.
	 *
	 * @return String
	 */
	public String GetRepostControlName()
	{
		return (ApprovalControl.STR_MVC_CLASS_NAME);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.AActivitySpace#CheckBasicAccess(String, String, boolean)
	 */
	public boolean CheckBasicAccess(String strPage, String strControl,
		boolean bSameSpace)
	{
		if (!((IPTSession) GetUserSession()).GetSessionInfo()
			.CheckForActivityRight(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_UTILITIES))
		{
			return false;
		}

		return super.CheckBasicAccess(strPage, strControl, bSameSpace);
	}

	/**
	 * Registers all the classes of the Property Activity Space.
	 *
	 */
	public void Init()
	{
		
			log.Debug("Init()");
		

		/**    Behold a most egregious hack.  The approval view was written to
		 *     display one or more of many possible types.  This architecture
		 *     clashes most badly with the editor framework.  The editor framework
		 *     expects multiple display pages to be registered, each page is
		 *     displayed when the user clicks the corresponding menu option.  In
		 *     order for the editor framework to generate appropriate menu options,
		 *     we must therefor register multiple instances of the same display page
		 *     class: ObjectApprovalDP.  The ObjectApprovalDP constructor then
		 *     takes a string describing which type is to be displayed, this type
		 *     is then passed on to the view display method.
		 *
		 *
		 *
		 */
		m_strModelName = ObjectApprovalModel.STR_MVC_CLASS_NAME;

		// Warning: m_strModelName must be initialized before super.Init() is called.
		super.Init();

		int nMenuOneID = m_mEditorMenu.StartNewSubMenu(GetString(1202,
					ResourceConstants.PORTALADMINMSGS));

		IModel myModel = GetModel(ObjectApprovalModel.STR_MVC_CLASS_NAME);

		ObjectApprovalModel oaModel = (ObjectApprovalModel) myModel;

		XPArrayList alTypes = oaModel.GetSupportedItemTypes();

		ApprovalItemType itDefault = oaModel.GetDefaultItemType();

		IXPEnumerator eTypes = alTypes.GetEnumerator();

		ApprovalItemType type   = null;
		ApprovalItemType firstTypeWithItems = null;
		String sTypeName;
		ObjectApprovalDP dpTemp = null;
		boolean bCurrentPageSet = false;

		//	iterate all of the supported types, and add links
		//	to the submenu.  Don't add links for those types
		//	containing no items to approve.  If the default type
		//	is one of the types added, then set that as the
		//	current page.
		while (eTypes.MoveNext())
		{
			type = (ApprovalItemType) eTypes.GetCurrent();

			
				log.Debug("Init() - About to see if there are any objects of type: " +
					type.GetName());
			

			if (oaModel.GetTypeCount(type) > 0)
			{

				if( firstTypeWithItems == null )
					firstTypeWithItems = type;

				sTypeName = type.GetName();

				
					log.Debug("Init() - Registering a page for type: " + sTypeName);
				

				dpTemp = new ObjectApprovalDP(sTypeName);

				RegisterPage(dpTemp, sTypeName);

				if (type == itDefault)
				{
					SetCurrentPage(dpTemp);

					if (type != null)
					{
						oaModel.SetCurrentType(type);
					}

					bCurrentPageSet = true;
				}

				m_mEditorMenu.AddLinkToSubMenu(nMenuOneID, sTypeName);
			}
		}

		//	if the current page has not been set, this is because
		//	the default type contains no items to approve.
		if (!bCurrentPageSet)
		{
			//	just set the current page to the last
			//	page registered.  This is arbitrary, change it if
			//	it doesn't suit you.
			if (dpTemp != null)
			{
				
					log.Debug("Init() - No items on the default type page, using: " +
						dpTemp.GetName());
				

				if (firstTypeWithItems != null)
				{
					oaModel.SetCurrentType(firstTypeWithItems);
				}

				bCurrentPageSet = true;
				SetCurrentPage(dpTemp);
			}
			else
			{
				//	no pages were registered, apparently there
				//	are no objects to approve




			}
		}

		//	this page is the page that gets loaded the first time.  This DP,
		//	when called upon to display itself, will determine which is the
		//	current type from the ObjectApprovalModel.
		dpTemp = new ObjectApprovalDP();
		RegisterPage(dpTemp);

		if( !bCurrentPageSet )
			SetCurrentPage( dpTemp );

		// Register Views
		RegisterView(ApprovalView.STR_MVC_CLASS_NAME, myModel);

		// Register Controls
		RegisterControl(ApprovalControl.STR_MVC_CLASS_NAME, myModel);

		// END Sub menu "General Settings"
		//	TIPS
		TipsModel tm = (TipsModel) GetModel(TipsModel.STR_MVC_CLASS_NAME);

		if (tm != null)
		{
			try
			{

				tm.SetGeneratingClass( sClassName );



				int iTitleId = 3;
				int iTipId = 4;
				int iSecondaryTipId = 5;

				tm.SetTipTitle( this.GetString( iTitleId, ResourceConstants.TIPS) );

				HTMLGenericElement e = new HTMLGenericElement( this.GetString( iTipId, ResourceConstants.TIPS) );

				tm.SetTip( e );

				String sSecondary = this.GetString( iSecondaryTipId, ResourceConstants.TIPS);

				if( !("none_do_not_translate".equals( sSecondary )) )
				{

					e = new HTMLGenericElement( sSecondary );

					tm.SetSecondarTip( e );
				}



			}
			catch (Exception ex)
			{
				
					log.Error(ex, "Couldn't generate tip.");
				
			}
		}
		 //	Got the tips model

		 this.SetHelpTopic( "PT_ApproveObjectsforMigration" );
	}
}

