package com.plumtree.portalpages.admin.editors.adminfolder;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.portaluiinfrastructure.securitymatrix.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This Section represents a section of the Advanced Settings page
 * of the Admin Folder Editor.
 *
 * @author RickyW
 */
public class AdminFolderAdvView extends EditorView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.adminfolder.AdminFolderAdvView");

	/** class name **/
	public static final String STR_MVC_CLASS_NAME = "AdminFolderAdvView";


	/** html form constants */
	public static final String HTMLSELECT_EDITPROFILE   = "in_se_ProfileType";


	public static final String HTMLBUTTON_EDITPROFILE   = "in_bu_EditProfile";


	public static final String HTMLSELECT_PROFILESELECT = "in_se_ProfileSelect";

	/** javascript function constants */
	public static final String JAVASCRIPT_SUBMIT_BY_POST = "submitNoRefresh";

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new AdminFolderAdvView();
	}

	/**
	 * Returns the HTML for the Section.
	 *
	 * @return HTMLElement
	 */
	public HTMLElement Display()
	{
		HTMLElementCollection result = new HTMLElementCollection();
		HTMLTable myTable			 = new HTMLTable();
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLSelect mySelect;
		HTMLOption myOption;
		HTMLInput myInput;
		HTMLSpan mySpan;

		try
		{
			result.AddInnerHTMLString( "\n" );

			HTMLComment myComment = new HTMLComment( "Begin: Display AdminFolderMainView");
			result.AddInnerHTMLElement( myComment );



			// add view title
			result.AddInnerHTMLElement(super.GetHeaderView(m_asOwner.GetString(
						10, ResourceConstants.PORTALADMINMSGS)));

			// create table for rest of info
			myTable = new HTMLTable();
			result.AddInnerHTMLElement(myTable);
			myTable.SetWidth("100%");
			myTable.SetBorder("0");
			myTable.SetCellPadding("5");
			myTable.SetCellSpacing("0");

			// BEGIN ROW 1
			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);
			myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);

			// display info text
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);
			mySpan = new HTMLSpan();
			myCell.AddInnerHTMLElement(mySpan);
			mySpan.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);

			mySpan.AddInnerHTMLString(m_asOwner.GetString(11,
					ResourceConstants.PORTALADMINMSGS));

			// set up subportal dropdown
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);
			myCell.SetNoWrap(true);
			mySelect = new HTMLSelect(HTMLSELECT_PROFILESELECT,
					"myProfileSelect");
			// set style of dropdown - bug 13441
			mySelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
			myCell.AddInnerHTMLElement(mySelect);

			// repost when dropdown changes so we can save
			mySelect.SetOnChange(AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" +
				AdminFolderRepostControl.POSTTOSELF_ACTION_CHANGE_DROPDOWN +
				");");

			// retrieve selected subportal. if none selected, value will be 0
			// (won't ever be 1 because we can't edit community folders)
			String mySelectedSubPortal = ((ISecurityMatrixModelRO) m_model).GetSelectedSubPortalID();

			// always have a none option
			myOption     = new HTMLOption(((ISecurityMatrixModelRO) m_model).GetRegularFolderID());
			mySpan		 = new HTMLSpan();
			myOption.AddInnerHTMLElement(mySpan);
			mySpan.SetStyleClass(PTStyleClass.FORM_SELECT_TEXT);
			mySpan.AddInnerHTMLString(m_asOwner.GetString(2367,
				ResourceConstants.PORTALADMINMSGS));
			mySelect.AddInnerHTMLElement(myOption);

			// make this the selected option if no subportal selected
			if (XPStringUtility.EqualsIgnoreCase(mySelectedSubPortal,((ISecurityMatrixModelRO) m_model).GetRegularFolderID()))
			{
				myOption.SetSelected(true);
			}

			// add dynamically loaded options
			for (int ii = 0;
					ii < ((ISecurityMatrixModelRO) m_model).GetNumSubPortals();
					ii++)
			{
				myOption     = new HTMLOption(((ISecurityMatrixModelRO) m_model).GetSubPortalID(
							ii));
				mySpan		 = new HTMLSpan();
				myOption.AddInnerHTMLElement(mySpan);
				mySpan.SetStyleClass(PTStyleClass.FORM_SELECT_TEXT);
				mySpan.AddInnerHTMLEncodedString(((ISecurityMatrixModelRO) m_model).GetSubPortalName(
						ii));
				mySelect.AddInnerHTMLElement(myOption);

				// make this subportal the selected option
				if (XPStringUtility.EqualsIgnoreCase(mySelectedSubPortal,((ISecurityMatrixModelRO) m_model).GetSubPortalID(ii)))
				{
					myOption.SetSelected(true);
				}
			}

			// display button for subportal profile editor
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myInput = new HTMLInput(HTMLInputTypes.BUTTON,
					HTMLBUTTON_EDITPROFILE,
					m_asOwner.GetString(13, ResourceConstants.PORTALADMINMSGS));
			myCell.AddInnerHTMLElement(myInput);

			myInput.SetValue(m_asOwner.GetString(12, ResourceConstants.PORTALADMINMSGS));
			myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);

			// disable button if no subportal is selected
			// htmlelements currently doesn't support this
			// but it will in the next release.
			if (mySelectedSubPortal.compareTo(
						((ISecurityMatrixModelRO) m_model).GetRegularFolderID()) == 0)
			{
				myInput.SetDisabled(true);
			}

			// associate button with the subportal editor's URL we want to open to
			ASURL myURL = ((ISecurityMatrixModelRO) m_model).GetEditObjectURL(PT_CLASSIDS.PT_SUBPORTAL_ID,
					XPConvert.ToInteger(mySelectedSubPortal));
			myInput.SetOnClick("OpenSubPortalEditor('" +	// mj: removed 'Javascript:' from function name
				myURL.GetURLAsString() + "', '" + mySelectedSubPortal + "');");

			// WE ARE FINISHED
			result.AddInnerHTMLString( "\n" );

			myComment = new HTMLComment( "End: Display AdminFolderAdvView");
			result.AddInnerHTMLElement( myComment );


		}
		catch (Exception e)
		{
			// Update the error messages (log and exception).

			log.Error(e, "AdminFolderAdvView::Display An error occured when trying to display the HTML.");

			throw new XPException(m_asOwner.GetString(14,
					ResourceConstants.PORTALADMINMSGS), e);
		}

		return result;
	}

	/**
	 * Returns Java Script for that Section.  Note that this method has to be
	 * called from any Display Pages that uses this Section.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 *
	 * @return HTMLScript
	 */
	public HTMLScript DisplayJavascript()
	{
		HTMLScript myScript;

		try
		{
			XPStringBuilder sb = new XPStringBuilder(500);
			myScript = new HTMLScript("text/javaScript");

			// add javascript functions here
			// open a subportal profile for editing
			sb.Append("\n");
			sb.Append(" function OpenSubPortalEditor(varURL, varIndex)");
			sb.Append(" {\n");
			sb.Append(
				"    window.open(varURL,'PopUpEditMode' + varIndex,'location=no,menubar=no,resizable=yes,scrollbars=yes,height=600,width=875');\n");
			sb.Append(" }\n");

			// reposts with a given control
			sb.Append("\n");
			sb.Append("function ").Append(JAVASCRIPT_SUBMIT_BY_POST).Append("(varID)\n");
			sb.Append("{\n");
			sb.Append("  document.").Append(AFormDP.HTMLFORM_NAME).Append(".method='POST';\n");
			sb.Append("  ").Append(AFormDP.JAVASCRIPT_SUBMIT_FORM).Append("(varID);\n");
			sb.Append("}\n");

			myScript.AddInnerHTMLString(sb.ToString());
		}
		catch (Exception e)
		{
			
				log.Error("AdminFolderMainView::DisplayJavascript The following error occured when trying to display the JavaScript: " +
					e.getMessage());
			

			throw new XPException(m_asOwner.GetString(15,
					ResourceConstants.PORTALADMINMSGS), e);
		}

		return myScript;
	}

	/** logger setup */
	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
}

