package com.plumtree.portalpages.admin.editors.group;

import com.plumtree.portalpages.common.search.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This DP is a take on the SavSrchMain DP by MikeS.
 * 
 * @author AkashJ
 *
 */
public class GroupDynamicRulesDP extends EditorDP
{

	public static final String STR_MVC_CLASS_NAME = "GroupDynamicRules";

	/** javascript name for function that alerts user of portlet cache */
	public static final String UPDATE_ALERT_JS = "UpdateAlert";

	/**
	* Displays some HTML after the body tag and before the form tag.
	* For example this is used by pages using a DHTML calendar to display the div tag.
	* The function is optional.  If it exists, it will be called by the framework, and
	* the resulting HTMLElement will be added in the right place.
	*
	* @return HTMLElement
	*/
	public HTMLElementCollection DisplayHTMLBeforeForm()

	{
		return FilterView.GetPagePreFormContent(m_asOwner);
	}

	/**
	 * Returns the String to display on the left navigation menu,
	 * linking to that page.
	 *
	 * @return String
	 */
	public String GetLeftNavString()
	{
		return (m_asOwner.GetString(2711, "ptmsgs_portaladminmsgs"));
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	* Implement this method to add HTML below the sub header.
	*
	* @return HTMLElement
	*
	* @throws HTMLException
	*/
	protected HTMLElement GetHTMLBelowSubHeader() {
		return HTMLConstructHelpers.GetSubHeader(
			m_asOwner.GetString(2712, "ptmsgs_portaladminmsgs"),
			this.m_asOwner);
	}

	/**
	 * Returns the HTML for the General Info page.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.AFormDP#PageDisplay(HTMLTableRow)
	 *
	 * @return HTMLElement
	 */
	public HTMLElement PageDisplay()

	{
		HTMLElementCollection result = new HTMLElementCollection();

		// Insert super class HTML
		result.AddInnerHTMLElement(super.PageDisplay());

		HTMLElementCollection views = new HTMLElementCollection();

		views.AddInnerHTMLElement((m_asOwner.GetView(
				DynamicRulesView.STR_MVC_CLASS_NAME)).Display());
		
		views.AddInnerHTMLElement(EditorView.GetSpacerTable());
		
		views.AddInnerHTMLElement((m_asOwner.GetView(
				DynamicMembersPreviewView.STR_MVC_CLASS_NAME)).Display());

		result.AddInnerHTMLElement(CreateLeftNavBar(views));

		return result;
	}

	/**
	 * Display javascript for this display page
	 *
	 * @return
	 *
	 * @throws HTMLException
	 */
	protected HTMLScriptCollection DisplayJavaScriptFromChild()

	{
		HTMLScriptCollection myScripts = super.DisplayJavaScriptFromChild();

		// ***CHANGE_HERE SELECTPTOBJECT*** Display JavaScripts for the Select PTObject list.
		myScripts.AddInnerHTMLElement(m_asOwner.GetView(
				DynamicRulesView.STR_MVC_CLASS_NAME).DisplayJavascript());
		
		myScripts.AddInnerHTMLElement(m_asOwner.GetView(
				DynamicMembersPreviewView.STR_MVC_CLASS_NAME).DisplayJavascript());

		String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE,
				GetOwner());
		
		myScripts.AddInnerHTMLElement( ConfigHelper.GetJSIncluderJSComponentInclude(ConfigHelper.JSCOMPONENT_JSDATEPICKER, GetOwner()) );
		
		return myScripts;
	}
}

