package com.plumtree.portalpages.admin.editors.job;


// Framework Imports
import com.plumtree.portalpages.common.joblogs.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class extends EditorDP and displays the History
 * page of the Job Editor.
 *
 */
public class HistoryDP extends EditorDP
{
	/** Unique identifier for the class */
	public static final String STR_MVC_CLASS_NAME = "JobHistory";

	/**
	 * Displays some HTML after the body tag and before the form tag.
	 * For example this is used by pages using a DHTML calendar to display the div tag.
	 * The function is optional.  If it exists, it will be called by the framework, and
	 * the resulting HTMLElement will be added in the right place.
	 *
	 * @return HTMLElement
	 */
	public HTMLElementCollection DisplayHTMLBeforeForm()
	{
		HTMLElementCollection myHTMLBeforeForm = new HTMLElementCollection();

		//Display the DIV tag required for the Date picker control
		IJobLogModelRO ptROModel = (IJobLogModelRO) m_asOwner.GetModel(JobLogModel.STR_MVC_CLASS_NAME);
		myHTMLBeforeForm = ptROModel.GetDatePickerControlManager().GetPagePreFormContent();

		return myHTMLBeforeForm;
	}

	/**
	 * Returns the HTML that is placed in the HEAD section of this HTML page.
	 *
	 * @return HTMLElementCollection
	 */
	public HTMLElementCollection DisplayHTMLinHEAD()

	{
		HTMLElementCollection retVal = new HTMLElementCollection();

		IJobLogModelRO ptROModel = (IJobLogModelRO) m_asOwner.GetModel(JobLogModel.STR_MVC_CLASS_NAME);
		JSDatePickerMgr ptDatePickerMgr = ptROModel.GetDatePickerControlManager();
		retVal.AddInnerHTMLElement(ptDatePickerMgr.GetJSIncludes());

		return retVal;
	}

	/**
	 * Returns the String to display on the left navigation menu,
	 * linking to that page.
	 *
	 * @return String
	 */
	public String GetLeftNavString()
	{
		return (m_asOwner.GetString(457, "ptmsgs_portaladminmsgs"));
	}

	
	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the HTML for the Job History page.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.AFormDP#PageDisplay(HTMLTableRow)
	 *
	 * @return HTMLElement
	 */
	public HTMLElement PageDisplay()

	{
		HTMLElementCollection result = new HTMLElementCollection();

		// Insert super class HTML
		result.AddInnerHTMLElement(super.PageDisplay());

		HTMLElementCollection views = new HTMLElementCollection();

		views.AddInnerHTMLElement(m_asOwner.GetView(
				HistoryView.STR_MVC_CLASS_NAME).Display());

		//Add menu bar construct here, along with job list
		result.AddInnerHTMLElement(CreateLeftNavBar(views));

		return result;
	}

	/**
	 * Display the Javascripts for the Views.
	 *
	 * @return HTMLScriptCollection
	 * @throws HTMLException
	 */
	protected HTMLScriptCollection DisplayJavaScriptFromChild()

	{
		HTMLScriptCollection myScripts = super.DisplayJavaScriptFromChild();

		myScripts.AddInnerHTMLElement(m_asOwner.GetView(
				HistoryView.STR_MVC_CLASS_NAME).DisplayJavascript());

		return myScripts;
	}
}

