package com.plumtree.portalpages.admin.editors.lockbox;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.editors.property.*;
import com.plumtree.portalpages.admin.ptutilities.credentialvaultmanager.*;
import com.plumtree.portalpages.common.helpers.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.marshalers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;

/**
 * 
 * @author jennifersu
 *
 */
public class LockboxEditorModel extends ObjEditorModel
	implements ILockboxEditorModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.lockbox.LockboxEditorModel");

	
	public static final String STR_MVC_CLASS_NAME = "LockboxEditorModel";
	protected boolean m_bNewLockbox = true;
	protected int m_nRow				= -1;
	protected int m_nUsernamePropID = -1;
	protected int m_nPasswordPropID = -1;
	protected String m_strUsernamePropName = null;
	protected String m_strPasswordPropName = null;
	
	private static final int PROPERTY_USERNAME = 0;
	private static final int PROPERTY_PASSWORD = 1;
	
	/**
	 * Code executed when the Editor is starting.
	 *
	 * @return boolean Is true if the current user has the rights to access this Editor.
	 */
	public boolean CheckEditorAccessAndStart(XPHashtable _htFormData)
	{
		m_bDisableSecurity = true;

		boolean bAccess = super.CheckEditorAccessAndStart(_htFormData);
		 
		if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
		{
			m_ptObject.SetName(m_asOwner.GetString(2903, "ptmsgs_portaladminmsgs"));
		}
		else
		{
			Object[] arPropIDs = ((IPTProfileSection)m_ptObject).GetChildProperties();
			if (arPropIDs.length >= 2)
			{
				m_nUsernamePropID = XPConvert.ToInteger(arPropIDs[0]);
				m_nPasswordPropID = XPConvert.ToInteger(arPropIDs[1]);
				
				IPTObjectManager propManager = m_Session.GetProperties();
				IPTProperty ptUsernameProp = (IPTProperty)propManager.Open(m_nUsernamePropID, false);
				m_strUsernamePropName = ptUsernameProp.GetName();
				IPTProperty ptPasswordProp = (IPTProperty)propManager.Open(m_nPasswordPropID, false);
				m_strPasswordPropName = ptPasswordProp.GetName();
			}
		}

		 return bAccess;
	}
	
	
	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskOnStartEditor(com.plumtree.openfoundation.util.XPHashtable)
	 */
	public void DoTaskOnStartEditor(XPHashtable _htFormData)
	{
		if (EditorStartControl.EDITOR_START_FLAG_EDIT != GetEditorType())
		{
			((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
				MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
		}
		else
		{
			m_bNewLockbox = false;
		}

		String[] data;
		data = (String[]) _htFormData.GetElement(LockboxListModel.LOCKBOX_ROW);
		if (data != null)
		{
			String strRow = data[0];
			m_nRow = XPConvert.ToInteger(strRow);
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new LockboxEditorModel();
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.admin.editors.lockbox.ILockboxEditorModelRO#IsNewLockbox()
	 */
	public boolean IsNewLockbox()
	{
		return m_bNewLockbox;
	}
	
	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.admin.editors.lockbox.ILockboxEditorModelRO#GetRow()
	 */
	public int GetRow()
	{
		return m_nRow;
	}
		
	/**
	 * 
	 *
	 */
	public void ResetInputs()
	{
		m_nRow = -1;
		m_bNewLockbox = true;
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Adding a new Lockbox object requires a number of steps under the covers:
	 * 1.	Create a new Profile Section object.
	 * 2.	Create two new Property objects (user name and password).
	 * 3.	Add these Properties to the Profile Section object.
	 * 4.	Map these Properties to the User object in the Global Object Properties map.
	 * 5.	Add these Properties to the User Info Property Map.
	 */
	public void DoTaskBeforeStore()
	{

		// (only in CREATE mode)
		if (EditorStartControl.EDITOR_START_FLAG_EDIT != GetEditorType())
		{
			// Create username and password properties
			IPTProperty usernameProp = CreateProperty(PROPERTY_USERNAME);
			IPTProperty passwordProp = CreateProperty(PROPERTY_PASSWORD);
			
			m_nUsernamePropID = ((IPTServerContext)usernameProp).GetObjectID();
			m_nPasswordPropID = ((IPTServerContext)passwordProp).GetObjectID();
			
			// Add properties to Global Object Properties map
			IPTObjectPropertyMap ptOPM     = (IPTObjectPropertyMap) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_OBJECTPROPERTYMAP,
					true);
			IPTQueryResult ptQR = ptOPM.QueryClassPropertyMappings(PT_CLASSIDS.PT_USER_ID, PT_PROPIDS.PT_PROPID_OBJECTID);
			int[] arPropIDs = new int[ptQR.RowCount() + 2];
			
			int nRow;
			for (nRow = 0; nRow < ptQR.RowCount(); nRow++)
			{
				int nPropertyID = ptQR.ItemAsInt(nRow, PT_PROPIDS.PT_PROPID_OBJECTID);
				arPropIDs[nRow] = nPropertyID;
			}
			arPropIDs[nRow] = m_nUsernamePropID;
			arPropIDs[nRow + 1] = m_nPasswordPropID;
			
			ptOPM.SetClassPropertyMappings(PT_CLASSIDS.PT_USER_ID, arPropIDs);
			
			// Add properties to User Info Property Map
			IPTProfileManager ptProfileManager = (IPTProfileManager) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_PROFILE_MANAGER,
					true);
			Object[][] arrMap = ptProfileManager.GetUserInfoMap();
			Object[][] arrNewMap = new Object[2][arrMap[0].length + 2];
			nRow = 0;
			for (nRow = 0; nRow < arrMap[0].length; nRow++)
			{
				arrNewMap[0][nRow] = arrMap[0][nRow];
				arrNewMap[1][nRow] = arrMap[1][nRow];
			}
			
			IPTMigrationManager ptMigrationManager = (IPTMigrationManager) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_MIGRATION_MANAGER, false);
			arrNewMap[0][nRow] = new Integer(m_nUsernamePropID);
			arrNewMap[1][nRow] = ((IPTMigratable)usernameProp).GetObjectUUID();
			arrNewMap[0][nRow + 1] = new Integer(m_nPasswordPropID);
			arrNewMap[1][nRow + 1] = ((IPTMigratable)passwordProp).GetObjectUUID();

			ptProfileManager.SetUserInfoMap(arrNewMap);
			
			// Add properties to PTProfileSection
			Object[] arProps = new Object[2];
			arProps[0] = new Integer(m_nUsernamePropID);
			arProps[1] = new Integer(m_nPasswordPropID);
			
			((IPTProfileSection)m_ptObject).SetChildProperties(arProps);
		}
		
	}
	
	/**
	 * Creates a property object and stores in DB.
	 * @param nPropertyType
	 * @return int property ID
	 */
	private IPTProperty CreateProperty(int nPropertyType)
	{
		IPTProperty ptProperty = (IPTProperty) m_Session.GetObjectManagers(PT_CLASSIDS.PT_PROPERTY_ID).Create(m_nFolderID);
		if (nPropertyType == PROPERTY_USERNAME)
		{
			ptProperty.Initialize(PT_PROPERTY_TYPES.PT_PROPTYPE_STRING);
			ptProperty.SetName(m_strUsernamePropName);
		}
		else
		{
			ptProperty.Initialize(PT_PROPERTY_TYPES.PT_PROPTYPE_ENCRYPTED);
			ptProperty.SetName(m_strPasswordPropName);			
		}
		ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_HREF, 0);
		ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_REFCLASS, -1);
		ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_ISCARDPROP, 0);
		ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_VISIBLE, 1);
		ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_READONLYVALUE, 0);
		ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_MANDATORY, 0);
		ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_SEARCHABLE, 0);
		ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_MULTIVALUED, 0);
		ptProperty.SetCharacteristics(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_PICKTYPE, PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_NONE);


		// JF- Store the Object in the DB.
		IPTServerContext ptSC = (IPTServerContext) ptProperty.GetInterfaces("IPTServerContext");
					
		int nPropertyID;
		
		try
		{
			ptSC.Store();
			
			nPropertyID = ptSC.GetObjectID();
		}
		catch (Exception e)
		{
			log.Error(e, STR_MVC_CLASS_NAME + "::CreateProperty Unable to store the Object.");

			String sMessage = m_asOwner.GetString(158, "ptmsgs_portalinfrastructure" );
			// JF- The exception message should not be extracted.
			String sExceptionMessage = "The Portal was unable to store the Object.";
				
			if( e instanceof PTException )
			{
				PTException ptex = (PTException) e;
				
				int iCode = ptex.GetErrorCode();
					
				if((iCode == PT_RESULTCODES.PT_E_INSUFFICIENTROLEACCESS) 
					|| (iCode == PT_RESULTCODES.PT_E_ACCESSDENIED)
					|| (iCode == PT_RESULTCODES.PT_E_CANNOTEDITREADONLY))
				{
					sMessage = m_asOwner.GetString(441, "ptmsgs_portalinfrastructure");
					sExceptionMessage = "The current user does not have sufficient rights to store this Object";
				}
				else if (iCode == PT_RESULTCODES.PT_E_OBJECTLOCKED)
				{
					sMessage = m_asOwner.GetString(442, "ptmsgs_portalinfrastructure");
					sExceptionMessage = "This Object is already locked. It cannot be stored.";
				}
				else if (iCode == PT_RESULTCODES.PT_E_OBJECTNOTLOCKED)
				{
					sMessage = m_asOwner.GetString(443, "ptmsgs_portalinfrastructure");
					sExceptionMessage = "Unable to lock this Object. It cannot be stored.";
				}
										
				UIError myError = new UIError(UIErrorLevel.SERIOUS,
					UIErrorScope.CURRENT_USER, null, 
					sMessage, null);
				UIErrorHandler.RaiseError(myError,
					m_asOwner.GetSubSession(), m_asOwner.GetApplication());
			}

			throw new XPException(sExceptionMessage, e);
		}
		finally
		{
			ptSC.UnlockObject();
		}
		
		return ptProperty;			
	}
	
	/**
	 * That method is called by the EditorFinishControl.  It takes care of
	 * storing the IPTObject in the DB  when the user finishes the Editor.
	 *
	 * @throws XPException
	 */
	public void Finish()
	{
		DoTaskBeforeStore();

		((LocNamesModel) m_asOwner.GetModel(LocNamesModel.STR_MVC_CLASS_NAME)).Finish();

		StoreObject();

		SaveMigrationStatus();
		
		DoTaskAfterStore();

		// JF- The Object has been stored. We are now in Edit mode.
		if (EditorStartControl.EDITOR_START_FLAG_CREATE == m_nEditorType)
		{
			m_nEditorType = EditorStartControl.EDITOR_START_FLAG_EDIT;
		}
	}
	
	/**
	 * Calls the right save function according to the page name passed in
	 * argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 *
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		int nPageStatus = super.SavePage(_sPageName, _htFormData);

		if (_sPageName.equals(LockboxEditorLocNamesDP.STR_MVC_CLASS_NAME))
		{
			nPageStatus = ((LocNamesModel) m_asOwner.GetModel(LocNamesModel.STR_MVC_CLASS_NAME)).SaveSection(_htFormData);
			if (nPageStatus == RepostControl.PAGE_STATUS_VALID && EditorStartControl.EDITOR_START_FLAG_CREATE == m_nEditorType)
			{
				m_strUsernamePropName = XPStringUtility.Trim(ModelHelper.GetFormElementString(_htFormData, LockboxEditorPropertiesView.HTMLINPUT_USERNAME_NAME, "Username"));
				if (m_strUsernamePropName.equals(""))
				{
					m_strUsernamePropName = "Username";
				}
				m_strPasswordPropName = XPStringUtility.Trim(ModelHelper.GetFormElementString(_htFormData, LockboxEditorPropertiesView.HTMLINPUT_PASSWORD_NAME, "Password"));
				if (m_strPasswordPropName.equals(""))
				{
					m_strPasswordPropName = "Password";
				}
			}
		}
		return nPageStatus;
	}
	
	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.admin.editors.lockbox.ILockboxEditorModelRO#GetUsernamePropID()
	 */
	public int GetUsernamePropID()
	{
		return m_nUsernamePropID;
	}
	
	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.admin.editors.lockbox.ILockboxEditorModelRO#GetPasswordPropID()
	 */
	public int GetPasswordPropID()
	{
		return m_nPasswordPropID;
	}
	
	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.admin.editors.lockbox.ILockboxEditorModelRO#GetUsernamePropName()
	 */
	public String GetUsernamePropName()
	{
		return m_strUsernamePropName;
	}
	
	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.admin.editors.lockbox.ILockboxEditorModelRO#GetPasswordPropName()
	 */
	public String GetPasswordPropName()
	{
		return m_strPasswordPropName;
	}
	
	/**
	 * Returns the URL to edit the object.
	 *
	 * @param _nClassID
	 * @param _nObjectID
	 * @param _sEditorASName
	 *
	 * @return ASURL URL to the Editor
	 */
	public ASURL GetEditPropertyURL(int _nPropertyID)
	{
		ASURL myURL = new ASURL(m_asOwner);

		String sEditorASName = PropEditorAS.STR_MVC_CLASS_NAME;

		myURL.SetLinkCreateNewSpace(sEditorASName, m_asOwner);
		myURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
		myURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE,
			XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_EDIT));
		myURL.SetSubSpaceKey(sEditorASName);

		myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID, XPConvert.ToString(PT_CLASSIDS.PT_PROPERTY_ID));
		myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_QS_OBJECT_ID, XPConvert.ToString(_nPropertyID));

		return myURL;
	}

	/**
	 * This method will be called when you come back to the main Editor from a sub Editor.
	 *
	 */
	public void DoTaskOnComeBackFromSubEditor()
	{
		PropEditorAS myPropEditorAS;
		PropModel myPropModel;
		int nLockboxID;

		String strSubEditorASName = PropEditorAS.STR_MVC_CLASS_NAME;

		if (m_asOwner.GetSubSpaceExists(strSubEditorASName))
		{
			myPropEditorAS = (PropEditorAS) m_asOwner.GetSubSpace(strSubEditorASName);
			myPropModel = (PropModel) myPropEditorAS.GetModel(myPropEditorAS.m_strModelName);
			if (myPropModel.GetObjectID() == m_nUsernamePropID)
			{
				m_strUsernamePropName = myPropModel.GetObjectName();
			}
			else if (myPropModel.GetObjectID() == m_nPasswordPropID)
			{
				m_strPasswordPropName = myPropModel.GetObjectName();
			}
		}
	}

}

