package com.plumtree.portalpages.admin.editors.portlet;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.editors.portlet.plugins.*;
import com.plumtree.portalpages.browsing.objectselection.*;
import com.plumtree.portalpages.browsing.objectselection.portlets.*;
import com.plumtree.portalpages.browsing.portletpreview.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.*;
import com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author adolfoc
 */
public class PortletModel extends ObjEditorModel implements IPortletModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.portlet.PortletModel");

	public static final String STR_MVC_CLASS_NAME = "Portlet";

	/**
	 * These static variables are used in determining what UI to display and how to initialize
	 * the portlet object.
	 */
	public static final String HTTPGP_CLSID			  = PT_PROVIDER_CLSIDS.PT_REMOTEPORTLET_PROVIDER;
	public static final String IGP_CLSID			  = PT_PROVIDER_CLSIDS.PT_INTRINSICPORTLET_PROVIDER;
	public static final String PORTLET_CLSID		  = "{C73FD37D-3A81-43f0-9463-C55AF23236BE}";
	public static final String PORTLET_TEMPLATE_CLSID = "{B940496F-E658-4ba0-A111-E87AC40BB6BD}";

	/**
	 * The following static members define the possible modes this editor can be in.
	 */
	public static final int PMODE_INVALID			  = -1; // An invalid mode. It should never be returned unless this editor is seriously messed up.
	public static final int PMODE_PRECREATE			  = 0; // The mode the editor starts up when started in CREATE_MODE.
	public static final int PMODE_CREATE			  = 1; // The mode used when the portlet is being created from scratch.
	public static final int PMODE_CREATE_FROMTEMPLATE = 2; // The mode used when the portlet is being created from a portlet template.
	public static final int PMODE_EDIT = 3; // The mode used when we are editing a portlet.

	/** Query String to determine if freeform templates should be shown */
	public static final String QS_SHOWFREEFORM = "ShowFreeForm";
	
	/** Query String to determine if freeform templates should be shown */
	public static final String QS_SHOWHEADERFOOTER = "ShowHeaderFooter";
	
	/** Web Service ID for intrinsic Community Links portlet **/
	public static final int WSID_COMMUNITYLINKS = 2;

	/**
	 * This member maintains a registry of page names that extend
	 * PPluginDP. We must remember to enable and disable these pages
	 * when the appropriate updates occur.
	 */
	protected XPArrayList m_xalPPluginDPs;

	/**
	 * This member maintains a registry of Plugin model keys that support
	 * the IWSSubModel interface. We must remember to exercise the IWSSubModel
	 * interface on them when the appropriate updates occur.
	 */
	protected XPArrayList m_xalPSubModels;

	/**
	 * This member contains wether the portlet being created is for a community (it is
	 * not always present). It is used when creating a new portlet from a
	 * template. It is false if no value was passed in.
	 */
	private boolean m_bForCommunity = false;

	/**
	 * This member holds the CLASSID of the object we will be created from. It
	 * can either be the classid for WebServices or for Portlet Templates.
	 */
	private int m_nCreatorClassID = -1;
	/**
	 * This member holds the object ID of the object we will be created from. It
	 * is used with m_nCreatorClassID to initialize ourselves either from the
	 * tree popup (Admin view) or the PSelectTemplate page.
	 */
	private int m_nCreatorID = -1;

	/**
	 * This member defines the current mode of the PortletEditor (as per the above enumeration);
	 * When this member is changed, we must send an update to all of our pages and views.
	 */
	private int m_nPMode;

	public PortletModel()
	{
		super();
		m_xalPPluginDPs = new XPArrayList();
		m_xalPSubModels = new XPArrayList();
		m_nPMode = PMODE_INVALID;
	}

	public void AddSendToOthersButtonToHelpBanner(PTConstruct myHelpBanner)
	{
		try
		{
			IPTGadget ptPortlet = (IPTGadget) GetObject();

			if ((EditorStartControl.EDITOR_START_FLAG_EDIT == GetEditorType()) &&
					(GetPortletType() == PT_GADGET_TYPES.PT_GADGETTYPE_PERSONAL) &&

				((ptPortlet.GetAlignment() == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NARROW) ||
					(ptPortlet.GetAlignment() == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE)))
			{
				// Send to Others Link
				PTConstruct buttonConstruct = UIConstructFactory.GetConstruct(PTButtonType.SEND_TO_OTHERS,
						true, this.m_asOwner);

				ElementName eName = PTButtonType.SEND_TO_OTHERS.GetElementName(PTButtonType.ELEMENT_NAME_ID_TITLE);
				// "Send To Others"
				buttonConstruct.ReplaceNestedElementsWithHTMLString(eName,
					m_asOwner.GetString(640, "ptmsgs_portalcommonmsgs"));
					
				ConstructFactory.SetAltOnButtonImages(buttonConstruct, 
					m_asOwner.GetString(640, "ptmsgs_portalcommonmsgs"));

				HTMLAnchor myAnchor		    = (HTMLAnchor) buttonConstruct.GetNestedElement(PTButtonType.SEND_TO_OTHERS.ANCHOR_ELEMENT_NAME);

				ASURL asurl = new ASURL(m_asOwner);
				asurl.SetLinkGetSpaceIfCached(PortletSelectionEditorAS.STR_MVC_CLASS_NAME, m_asOwner);
				asurl.SetControl(InvitationPopupControl.STR_MVC_CLASS_NAME);
				asurl.SetDisplayPage(PortletInvitationPopupDP.STR_MVC_CLASS_NAME);
				asurl.AddControlArgument(InvitationPopupControl.strTemporaryInvitationID,
					XPConvert.ToString(GetObjectID()));
				asurl.AddControlArgument(InvitationPopupControl.strTemporaryInvitationName,
					m_ptObject.GetLocalizedName());
				
				String strJSUtilVersionPrefix = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSUTIL);
				myAnchor.SetHRef(ConstructStaticHelpers.BLANK_URL);
				myAnchor.SetOnClick(strJSUtilVersionPrefix+"PTWindowUtil.openWindow('" +
					asurl.GetURLAsString() + "', 'SendToOthers', '250','600'); return false;");

				//myAnchor.AddInnerHTMLString(_ui_SendToOthers);
				myHelpBanner.AddElementToNestedElement(PTBannerType.BANNER_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
					myAnchor);
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "PortletEditorAS::GetHelpBanner The following error occured when trying to get the help banner: " +
					e.getMessage());
			
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new PortletModel();
	}

	/**
	 * Calculates whether or not the "mandatory" column should be displayed on
	 * the security tab.  We should display the mandatory column if the gadget
	 * is not a template AND it is a narrow or wide portlet.
	 *
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String, XPHashtable)
	 */
	public void DetermineWhetherToDisplayMandatoryColumnOnSecurityPage()
	{
		int nAlignment  = GetPortletSize();
		int nPortletType = GetPortletType();

		if (((nAlignment == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NARROW) ||
				(nAlignment == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE)) &&
				(nPortletType == PT_GADGET_TYPES.PT_GADGETTYPE_PERSONAL))
		{
			ObjectSecurityModel modelSecurity = (ObjectSecurityModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB);
			modelSecurity.SetSupportMandatoryColumn(ObjectSecurityModel.MANDCOL_REGULAR);

			PriorityDropDownModel modelPriority = (PriorityDropDownModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN);
			modelPriority.SetSupportMandatoryColumn(true);
		}
		else
		{
			ObjectSecurityModel modelSecurity = (ObjectSecurityModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB);
			modelSecurity.SetSupportMandatoryColumn(ObjectSecurityModel.MANDCOL_NONE);

			PriorityDropDownModel modelPriority = (PriorityDropDownModel) m_asOwner.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_PRIORITY_DROP_DOWN);
			modelPriority.SetSupportMandatoryColumn(false);
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskBeforeStore()
	 */
	public void DoTaskBeforeStore()
	{
		super.DoTaskBeforeStore();

		PreStorePSubModels();

		return;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskOnStartEditor(_htFormData)
	 */

	//MES-CR: I don't really get this- if the editor is just starting, how would we
	//	have form data?
	//AGC-CR: Data passed into the editor on start.
	public void DoTaskOnStartEditor(XPHashtable _htFormData)
	{
		super.DoTaskOnStartEditor(_htFormData);

		try
		{
			switch (GetEditorType())
			{
			case EditorStartControl.EDITOR_START_FLAG_EDIT:
				m_nPMode = PMODE_EDIT;

				UpdatePages();
				InitPSubModels();
				UpdatePSubModels();

				DetermineWhetherToDisplayMandatoryColumnOnSecurityPage();

				break;

			case EditorStartControl.EDITOR_START_FLAG_CREATE:

				// AGC- This initialization is temporary, just to avoid spurious
				// errors. Since the object is likely to be replaced, we'll be
				// reinitializing it later. At that point, we'll know what
				// WebService to use (and thus what provider CLSID). Whee.
				((IPTGadget) m_ptObject).Initialize(HTTPGP_CLSID);

				// AGC- We determine what mode we're in depending on whether we're
				// a template or not - which is in turn determined by the name of the
				// Activity space that we belong to.
				if (0 == PortTempPortletEditorAS.STR_MVC_CLASS_NAME.compareTo(
							m_asOwner.GetName()))
				{
					m_nPMode = PMODE_CREATE;

					// AGC- This may not strictly be necessary, but it's a good idea.
					// NOTE: This must be done through the IPTGadgetTemplate interface or
					// there will be an error.
					((IPTGadgetTemplate) m_ptObject).SetGadgetType(PT_GADGET_TYPES.PT_GADGETTYPE_TEMPLATE);
				}
				else if ((m_nCreatorClassID <= 0) || (m_nCreatorID <= 0))
				{
					m_nPMode = PMODE_PRECREATE;
					m_asOwner.SetCurrentPage(m_asOwner.GetPage(PreCreateDP.STR_MVC_CLASS_NAME));
					if (!m_bForCommunity)
					{
						// AGC- This is an error case. Either the CreatorID/CLASSID
						// must be set or this must be for a community. We should note the error, but continue in PRECREATE mode (the
						// best we're gonna get).=
						
							log.Error("PortletData::DoTaskOnStartEditor() Either you must use this editor for a Community Portlet OR (not exclusive) CREATORID AND CREATORCLASSID must be set in order to use this editor.");
						

					}
				}
				else
				{
					// AGC- The Creator ID and CLASSID were passed in. We must create ourselves.
					if (m_nCreatorClassID == PT_CLASSIDS.PT_WEBSERVICE_ID)
					{
						// If the classID is the WebService ID, we can do nothing ( create as usual). The
						// WebService selection view on the MainDP will collect the object ID from m_nCreatorID.
						m_nPMode = PMODE_CREATE;
					}
					else if (m_nCreatorClassID == PT_CLASSIDS.PT_GADGETTEMPLATE_ID)
					{
						// If the classID is the Portlet Template ID, we must recreate our m_ptObject based on
						// template (much like PChooseModel or PSelectTemplateModel do).
						SetEditedObject(CreateObjectFromTemplate(m_nCreatorID));
						m_nPMode = PortletModel.PMODE_CREATE_FROMTEMPLATE;
					} else
					{
						// AGC- Something is seriously wrong. This editor only knows how to
						// create itself from an existing WebService or PortletTemplate.
						// Best we can do is try to go to PRE_CREATE mode and pray for the best.
						m_nPMode = PMODE_CREATE;

						
							log.Error("PortletData::DoTaskOnStartEditor() CREATORCLASSID is not WebService or Portlet Template! Going to PreCreateMode");
						

					}



				}

				UpdatePages();
				InitPSubModels();
				UpdatePSubModels();

				// this must be called AFTER we set m_intPortletSize and m_intGadgetType
				DetermineWhetherToDisplayMandatoryColumnOnSecurityPage();

				((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
					MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);

				break;

			default:
				m_nPMode = PMODE_INVALID;

				
					log.Warn("PortletData::DoTaskOnStartEditor: Invalid start flag passed into editor. Setting PMODE INVALID!");
				

				break;
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "PortletData::DoTaskOnStartEditor - Unable to initialize editor from server: " +
					e.getMessage());
			
		}
	}

	public boolean IsForCommunity()
	{
		return m_bForCommunity;
	}

	public int GetFolderID()
	{
		return m_nFolderID;
	}

	// Helper Functions
	public int GetPortletType()
	{
		PortletSettingsModel mPortletSettings = (PortletSettingsModel) m_asOwner.GetModel(PortletEditorAS.Storage_PortletSettingsView);

		return mPortletSettings.GetPortletType();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the nPMode.
	 * @return int
	 */
	public int GetPMode()
	{
		return m_nPMode;
	}

	public int GetPortletSize()
	{
		PortletSettingsModel mPortletSettings = (PortletSettingsModel) m_asOwner.GetModel(PortletEditorAS.Storage_PortletSettingsView);

		return mPortletSettings.GetPortletSize();
	}

	public String GetProviderCLSID()
	{
		SelectWebServiceModel mSelectWebService = (SelectWebServiceModel) m_asOwner.GetModel(PortletEditorAS.Storage_SelectWebServiceView);

		return XPStringUtility.ToUpperCase(mSelectWebService.GetWSObject().GetProviderCLSID());
	}

	public IPTWebService GetWebService()
	{
		SelectWebServiceModel mSelectWebService = (SelectWebServiceModel) m_asOwner.GetModel(PortletEditorAS.Storage_SelectWebServiceView);

		return mSelectWebService.GetWSObject();
	}

	public boolean IsTemplate()
	{
		PortletSettingsModel mPortletSettings = (PortletSettingsModel) m_asOwner.GetModel(PortletEditorAS.Storage_PortletSettingsView);

		return (PT_GADGET_TYPES.PT_GADGETTYPE_TEMPLATE == mPortletSettings.GetPortletType());
	}


	public boolean IsMandatory()
	{
		// Templates cannot be mandatory.
		if (IsTemplate())
		{
			return false;
		}

		// reopen the object (this is called after the object has been stored)
		IPTGadget ptPortlet = (IPTGadget) m_ptObject;
		IPTQueryResult ptQR = ptPortlet.QueryMandatoryGroups();
		return(null != ptQR && 0 < ptQR.RowCount());
	}

	public void PreStorePSubModels()
	{
		// AGC- Only store those views that are active in our current mode!
		IXPEnumerator xpEnumPPluginDPs = m_xalPPluginDPs.GetEnumerator();

		// Walk through all of the pages
		while (xpEnumPPluginDPs.MoveNext())
		{
			PPluginDP pPluginDPCurrent     = (PPluginDP) m_asOwner.GetPage((String) xpEnumPPluginDPs.GetCurrent());
			XPArrayList xpalModelsCurrent  = pPluginDPCurrent.GetOrderedViewList(m_nPMode);
			IXPEnumerator xpEnumPSubModels = xpalModelsCurrent.GetEnumerator();

			// Walk through each view on each page
			while (xpEnumPSubModels.MoveNext())
			{
				String strSubModelKeyCurrent = "";
				strSubModelKeyCurrent = (String) xpEnumPSubModels.GetCurrent();
				IPSubModel wsSubModelCurrent = (IPSubModel) m_asOwner.GetModel(strSubModelKeyCurrent);
				// Tell the buggers to prepare for the end times
				wsSubModelCurrent.DoTaskBeforePStore();

			}
		}
	}

	// ***************************************** SubModel Management ********************************8
	// ************************************ IPSubModel methods ***************************************

	/**
	 * This method is used to register a new PPluginDP with this WebService main model. This is done
	 * so that plugin pages can be hidden and shown automatically depending on the PMODE.
	 */
	public void RegisterPPage(String strDPName)
	{
		try
		{
			IDisplayPage myIDP = m_asOwner.GetPage(strDPName);
			PPluginDP myPPDP   = null;

			try
			{
				myPPDP = (PPluginDP) myIDP;
				myPPDP.IsEnabled(m_nPMode, "");
			}
			catch (Exception e)
			{
				
					log.Error(e, "PortletData::RegisterPPage(\"" + strDPName +
						"\") - Attempted to register page that doesn't extend PPluginDP. Error:" +
						e.getMessage());
				
			}

			if (null == myPPDP)
			{
				log.Error("PortletData::RegisterPPage(\"" + strDPName +
					"\") - Attempted to register null PPluginDP. Error:");
			}
			else
			{
				m_xalPPluginDPs.Add(strDPName);
			}
		}
		catch (Exception e)
		{
			
				log.Error("PortletData::RegisterPPage(\"" + strDPName +
					"\") - Could not register page. Error:" + e.getMessage());
			
		}
	}

	/**
	 * This method is used to register a new submodel with this WebService main model. This is done
	 * so that submodels can have access to the Remote Server base URL which is necessary for them
	 * to display. All submodels that are registered must support the IPSubModel interface.
	 */
	public void RegisterPSubModel(String strPluginModelStorageName)
	{
		try
		{
			IModel imSubModel     = m_asOwner.GetModel(strPluginModelStorageName);
			IPSubModel wsSubModel = null;

			try
			{
				wsSubModel = (IPSubModel) imSubModel;
			}
			catch (Exception e)
			{
				
					log.Error(e, "PortletData::RegisterPSubModel(\"" +
						strPluginModelStorageName +
						"\") - Attempted to register model that doesn't support IPSubModel. Error:" +
						e.getMessage());
				
			}

			if (null == wsSubModel)
			{
				log.Error("PortletData::RegisterPSubModel(\"" +
					strPluginModelStorageName +
					"\") - Attempted to register null IPSubModel. Error:");
			}
			else
			{
				m_xalPSubModels.Add(strPluginModelStorageName);
			}
		}
		catch (Exception e)
		{
			
				log.Error("PortletData::RegisterPSubModel(\"" +
					strPluginModelStorageName +
					"\") - Could not register model. Error:" + e.getMessage());
			
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String, XPHashtable)
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		int nPageStatus = super.SavePage(_sPageName, _htFormData);

		IDisplayPage myDP     = m_asOwner.GetPage(_sPageName);
		PPluginDP myPortletDP = null;

		try
		{
			//AGC - try to cast to
			myPortletDP = (PPluginDP) myDP;
		}
		catch (Exception e)
		{
			// Didn't work. It's someone else's page. Just return.
			return nPageStatus;
		}

		if (null == myPortletDP) // Just double checking.
		{
			return nPageStatus;
		}

		XPArrayList xpalViews		  = myPortletDP.GetOrderedViewList(GetPMode());
		IXPEnumerator xpEnumSubModels = xpalViews.GetEnumerator();

		while (xpEnumSubModels.MoveNext())
		{
			String strSubModelKeyCurrent = (String) xpEnumSubModels.GetCurrent();

			try
			{
				IPSubModel pSubModelCurrent = (IPSubModel) m_asOwner.GetModel(strSubModelKeyCurrent);

				int nLocalStatus = ((EditorHelperModel) pSubModelCurrent).SaveSection(_htFormData);

				if (EditorRepostControl.PAGE_STATUS_VALID != nLocalStatus)
				{
					nPageStatus = nLocalStatus;
				}
			}
			catch (Exception e)
			{
				
					log.Error(GetName() + "::SavePage(" + _sPageName +
						") The following error occured when trying to Save the Section(" +
						strSubModelKeyCurrent + "): " + e.getMessage());
				
			}
		}

		// this must be called AFTER we set m_intPortletSize and m_intGadgetType
		DetermineWhetherToDisplayMandatoryColumnOnSecurityPage();

		return nPageStatus;
	}

	// *************************************** Persistence ******************************************8

	/**
	 * Reset the portlet object we are editing to this new one. This will require some special processing
	 * as some of our framework needs to be updated along with our own submodels.
	 * Note* This call can only be called from CREATE mode and will do nothing if called in any other mode.
	 */
	public void SetEditedObject(IPTGadget ptPortlet)
	{
		if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
		{
			m_ptObject = (IPTObject) ptPortlet;
		}
		else
		{
			return;
		}

		// We must reinitialize the framework. These calls are made by our super class in StartEditor. If
		// the super class changes, this section should be updated.
		// Init common pages
		super.StartLocNamesList();
		super.StartSecurityList();
		super.StartObjectProp();

		// Perform Editor specific tasks. This will also reinitialize our submodels for us.
		// These calls are similar to those made in DoTaskOnStartEditor.
		UpdatePages();
		InitPSubModels();
		UpdatePSubModels();

		DetermineWhetherToDisplayMandatoryColumnOnSecurityPage();

		// We need not bother calling Update on our models. We know that the PMODE will be probably
		// be changed soon and that will do it for us.
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.editor.ObjEditorModel#SetMemberVariablesFromQS(XPHashtable)
	 */
	public void SetMemberVariablesFromQS(XPHashtable _htFormData)
	{
		super.SetMemberVariablesFromQS(_htFormData);

		// Community ID
		m_bForCommunity = PortletModelHelper.GetFormElementBool(_htFormData, PTPortlets.PORTLET_EDITOR_QS_BOOL_FOR_COMMUNITY, false);
		m_nCreatorClassID = PortletModelHelper.GetFormElementInt(_htFormData, PTPortlets.PORTLET_EDITOR_QS_INT_CREATOR_CLASSID, -1);
		m_nCreatorID = PortletModelHelper.GetFormElementInt(_htFormData, PTPortlets.PORTLET_EDITOR_QS_INT_CREATOR_ID, -1);
	}

	/**
	 * Sets the nPMode.
	 * @param nPMode The nPMode to set
	 */
	public void SetPMode(int nPMode)
	{
		m_nPMode = nPMode;

		// AGC- Don't forget to hide/show all of our pages based on the new information...
		UpdatePages();

		// AGC- The views should be told as well... just in case they care...
		UpdatePSubModels();
	}

	public void UpdatePSubModels()
	{
		try
		{
			IXPEnumerator xpEnumPSubModels = m_xalPSubModels.GetEnumerator();

			while (xpEnumPSubModels.MoveNext())
			{
				String strSubModelKeyCurrent = "";

				try
				{
					strSubModelKeyCurrent = (String) xpEnumPSubModels.GetCurrent();

					IPSubModel pSubModelCurrent = (IPSubModel) m_asOwner.GetModel(strSubModelKeyCurrent);
					pSubModelCurrent.DoTaskOnWebServiceUpdate(GetWebService());
					pSubModelCurrent.DoTaskOnPModeUpdate(GetPMode());
				}
				catch (Exception e)
				{
					
						log.Error("PortletData::UpdatePSubModels() Unknown Error updating PSubModel (" +
							strSubModelKeyCurrent + ")." + e.getMessage());
					
				}
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "PortletData::UpdatePSubModels() Unknown Error." +
					e.getMessage());
			
		}
	}

	public void UpdatePages()
	{
		// Automatically hide empty pages and show pages with information
		try
		{
			// Hide and show the appropriate frameworks pages.
			// Hide the security page in create mode
			if (IsForCommunity())
			{
				switch (m_nPMode)
				{


				case PMODE_PRECREATE:
				case -1://PMODE_INVALID:
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
						ObjectSecurityDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
						LocNamesDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
						MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);

					break;

				case PMODE_CREATE_FROMTEMPLATE:
				case PMODE_CREATE:
					// When creating from the community page, you shouldn't
					// show the security editor - the parent community editor
					// will reset the security (security inherited from the community)
					// upon store. Any changes you would make in the security editor
					// would be lost anyway.
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
						ObjectSecurityDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
						LocNamesDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
						MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);

					break;

				case PMODE_EDIT:
				default:
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
						ObjectSecurityDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
						LocNamesDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
						MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
				}

			} else
			{
				switch (m_nPMode)
				{
				case PMODE_PRECREATE:

				case -1://PMODE_INVALID:
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
						ObjectSecurityDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
						LocNamesDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
						MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);

					break;

				case PMODE_CREATE_FROMTEMPLATE:
				case PMODE_CREATE:
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
						ObjectSecurityDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
						LocNamesDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
						MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
	
					break;

				case PMODE_EDIT:
				default:
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
						ObjectSecurityDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
						LocNamesDP.STR_MVC_CLASS_NAME);
					((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
						MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
				}
			}

			((EditorAS) m_asOwner).GetLeftMenu().HideSubMenu((m_nPMode == PMODE_PRECREATE), 0);
			((EditorAS) m_asOwner).GetLeftMenu().HideSubMenu((m_nPMode == PMODE_PRECREATE), 1);

			// Hide and show our plugin pages and views.
			IXPEnumerator xpEnumPPluginDPs = m_xalPPluginDPs.GetEnumerator();

			while (xpEnumPPluginDPs.MoveNext())
			{
				try
				{
					String strDPNameCurrent    = (String) xpEnumPPluginDPs.GetCurrent();
					PPluginDP pPluginDPCurrent = (PPluginDP) m_asOwner.GetPage(strDPNameCurrent);
					XPArrayList xpalViews	   = pPluginDPCurrent.GetOrderedViewList(m_nPMode);
					int nNumViews			   = xpalViews.GetSize();

					if (0 >= nNumViews)
					{
						((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
							strDPNameCurrent);
					}
					else
					{
						((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
							strDPNameCurrent);
					}
				}
				catch (Exception e)
				{
					
						log.Error(e, "PortletData::UpdatePSubModels() Unknown Error updating PSubModel." +
							e.getMessage());
					
				}
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "PortletData::UpdatePSubModels() Unknown Error." +
					e.getMessage());
			
		}
	}

	private void InitPSubModels()
	{
		try
		{
			IXPEnumerator xpEnumPSubModels = m_xalPSubModels.GetEnumerator();

			while (xpEnumPSubModels.MoveNext())
			{
				String strSubModelKeyCurrent = "";

				try
				{
					strSubModelKeyCurrent = (String) xpEnumPSubModels.GetCurrent();

					IPSubModel pSubModelCurrent = (IPSubModel) m_asOwner.GetModel(strSubModelKeyCurrent);
					pSubModelCurrent.InitPModel((IPTGadget) m_ptObject);
				}
				catch (Exception e)
				{
					
						log.Error("PortletData::InitPSubModels() Unknown Error initializing PSubModel (" +
							strSubModelKeyCurrent + ")." + e.getMessage());
					
				}
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "PortletData::InitPSubModels() Unknown Error." +
					e.getMessage());
			
		}
	}

	/**
	 * If this editor is invoked from a template - this function is used to create the m_ptObject from a template.
	 */
	public IPTGadget CreateObjectFromTemplate(int nObjectIDOfTemplate)
	{
		try
		{
			IPTSession ptSession = GetPTSession();

			IPTGadgetTemplate ptTemplateToClone = ((IPTGadgetTemplate) ptSession.GetGadgetTemplates()
				.Open(nObjectIDOfTemplate, false));

			m_nPMode = PMODE_CREATE_FROMTEMPLATE;
			m_nCreatorClassID = PT_CLASSIDS.PT_WEBSERVICE_ID;
			m_nCreatorID = ptTemplateToClone.GetWebServiceID();
			
			IPTGadget ptPortlet = ptTemplateToClone.CreateGadgetInstance(ptSession,
				GetFolderID(), IsForCommunity() ? 1 : 0);
			ptPortlet.SetName(XPResourceManager.FormatMessage(m_asOwner.GetString(3152, "ptmsgs_portaladminmsgs"), 
			        ptTemplateToClone.GetLocalizedName()));
			return ptPortlet;
		}
		catch (Exception e)
		{
			
				log.Error(e, "PChooseModel::CreateObjectFromTemplate(" +
					XPConvert.ToString(nObjectIDOfTemplate) +
					") Unknown error, cannot create templated object. Error: " +
					e.getMessage());
			
		}

		return null;
	}

	/**
	 * Returns the creatorClassID.
	 * @return int
	 */
	public int GetCreatorClassID() {
		return m_nCreatorClassID;
	}

	/**
	 * Returns the creatorID.
	 * @return int
	 */
	public int GetCreatorID() {
		return m_nCreatorID;
	}


	/**
	 * This method will be called when you come back to the main Editor from a sub Editor.
	 *
	 */
	public void DoTaskOnComeBackFromSubEditor()
	{
		super.DoTaskOnComeBackFromSubEditor();

		SelectWebServiceModel mSelectWebService = (SelectWebServiceModel) m_asOwner.GetModel(PortletEditorAS.Storage_SelectWebServiceView);

		String strIntrinsicPortletClass = mSelectWebService.GetIntrinsicPortletClass();

		String strSubSpaceKey = strIntrinsicPortletClass + IntrinsicPortletPrefsUtils.SUBSPACE_KEY_SUFFIX;

		if (m_asOwner.GetSubSpaceExists(strSubSpaceKey))
		{
			m_asOwner.DeleteSubSpace(strSubSpaceKey);
		}
	}
	
	/**
	 * Helper method to check the current user rights to this Object Editor.
	 *
	 * @param _htFormData Form data
	 * @return boolean
	 */
	protected boolean CheckEditorAccess(XPHashtable _htFormData)
	{
		String[] values = (String[]) _htFormData.GetElement(QS_SHOWFREEFORM);

		if (values != null)
		{
			((PSelectTemplateModel) m_asOwner.GetModel(PortletEditorAS.Storage_PSelectTemplateView)).SetShowFreeForm(true);
		}
		
		values = (String[]) _htFormData.GetElement(QS_SHOWHEADERFOOTER);

		if (values != null)
		{
			((PSelectTemplateModel) m_asOwner.GetModel(PortletEditorAS.Storage_PSelectTemplateView)).SetShowHeaderAndFooter(true);
		}
		
		return super.CheckEditorAccess(_htFormData);	
	}
	
}

