package com.plumtree.portalpages.admin.editors.savedsearch;


// Framework Imports
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.portalpages.common.objects.sort.*;
import com.plumtree.portalpages.common.search.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.tree.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
* @author MikeS
 *
 */
public class SavSrchMainDP extends EditorDP
{

	public static final String STR_MVC_CLASS_NAME = "SrchMain";

	/** javascript name for function that alerts user of portlet cache */
	public static final String UPDATE_ALERT_JS = "UpdateAlert";

	public static final String strCardLanguagePlugInKey = "CardEditorCardLanguagePlugIn";

	/** Checkbox name for the folders */
	public static final String HTMLCHECKBOX_CATALOGFOLDER    = "in_cb_CatalogFolder"; // +group, value is folder id or -1 for all

	/** Checkbox name for the folders */
	public static final String HTMLCHECKBOX_ADMINFOLDER    = "in_cb_AdminFolder"; // +group, value is folder id or -1 for all

	/** Checkbox to be displayed in the subheader of the Add Folders toolabar */
	public static final String HTMLCHECKBOX_CHECKALL = "in_cb_CheckAll";

	/** Hidden HTML field for the catalog activity space id */
	public static final String HTMLHIDDEN_CATSPACEID	    = "in_hi_CatalogFolderSpaceID";

	/** Hidden HTML field for the Admin folder activity space id */
	public static final String HTMLHIDDEN_ADMSPACEID	    = "in_hi_AdminFolderSpaceID";

	/** Hidden HTML for the tree repost action */
	public static final String HTMLHIDDEN_TREEREPOST	    = "in_hi_CatalogFolderRepostAction";

	/** Catalog folder open javascript function name */
	public static final String JAVASCRIPT_CATALOGFOLDERTREE = "openRemoteCatalogFolderTree";

	/** Admin folder open javascript function name */
	public static final String JAVASCRIPT_ADMINFOLDERTREE = "openRemoteAdminFolderTree";
	
	/** Hidden HTML field for the catalog folder selected*/
	public static final String HTMLHIDDEN_FOLDERSELECTED    = "in_hi_CatalogFolderSelected";

	
	protected PTStyleClass m_styleListItem = null;

	
	/**
	* Displays some HTML after the body tag and before the form tag.
	* For example this is used by pages using a DHTML calendar to display the div tag.
	* The function is optional.  If it exists, it will be called by the framework, and
	* the resulting HTMLElement will be added in the right place.
	*
	* @return HTMLElement
	*/
	public HTMLElementCollection DisplayHTMLBeforeForm()

	{
		return FilterView.GetPagePreFormContent(m_asOwner);
	}

	/**
	 * Returns the String to display on the left navigation menu,
	 * linking to that page.
	 *
	 * @return String
	 */
	public String GetLeftNavString()
	{
		return (m_asOwner.GetString(771, "ptmsgs_portaladminmsgs"));
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	* Implement this method to add HTML below the sub header.
	*
	* @return HTMLElement
	*
	* @throws HTMLException
	*/
	protected HTMLElement GetHTMLBelowSubHeader() {
		EditorModel model = (EditorModel)m_asOwner.GetModel(SavedSearchModel.STR_MVC_CLASS_NAME);
		if (EditorStartControl.EDITOR_START_FLAG_CREATE == model.GetEditorType())
		{
			return HTMLConstructHelpers.GetSubHeader(
					m_asOwner.GetString(3187, "ptmsgs_portaladminmsgs"),
					this.m_asOwner);
		}
		else
		{
			return HTMLConstructHelpers.GetSubHeader(
					m_asOwner.GetString(1686, "ptmsgs_portaladminmsgs"),
					this.m_asOwner);
		}
	}

	/**
	 * Returns the HTML for the General Info page.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.AFormDP#PageDisplay(HTMLTableRow)
	 *
	 * @return HTMLElement
	 */
	public HTMLElement PageDisplay()

	{
		HTMLElementCollection result = new HTMLElementCollection();
		HTMLElementCollection views = new HTMLElementCollection();

		// Insert super class HTML
		result.AddInnerHTMLElement(super.PageDisplay());
		
		/* filter view */
		views.AddInnerHTMLElement((m_asOwner.GetView(
				SavSrchFilterView.STR_MVC_CLASS_NAME)).Display());

		/* language view */
		views.AddInnerHTMLElement(EditorView.GetSpacerTable());
		views.AddInnerHTMLElement((m_asOwner.GetView(strCardLanguagePlugInKey)).Display());

		/* query range view */
		views.AddInnerHTMLElement(EditorView.GetSpacerTable());
		views.AddInnerHTMLElement((m_asOwner.GetView(SnapshotQueryRangeView.STR_MVC_CLASS_NAME)).Display());

		/* the following views will be shown depending on what repositories are selected */
		int nRepositories = ((ISavedSearchModelRO)m_asOwner.GetModel(SavedSearchModel.STR_MVC_CLASS_NAME)).GetRepositories();
		
		/* kd repository constraints view */
		if ( 0 != (nRepositories&SavedSearchModel.REPOSITORY_TYPES_KD) )
		{
			views.AddInnerHTMLElement(EditorView.GetSpacerTable());
			views.AddInnerHTMLElement((m_asOwner.GetView(SnapshotKDConstraintsView.STR_MVC_CLASS_NAME)).Display());
		}
		
		/* ad repository constraints view */
		if ( 0 != (nRepositories&SavedSearchModel.REPOSITORY_TYPES_ADMIN) )
		{
			views.AddInnerHTMLElement(EditorView.GetSpacerTable());
			views.AddInnerHTMLElement((m_asOwner.GetView(SnapshotADConstraintsView.STR_MVC_CLASS_NAME)).Display());
		}

		//		views.AddInnerHTMLElement(EditorView.GetSpacerTable());
//		views.AddInnerHTMLElement((m_asOwner.GetView(
//				SearchConstraintsView.STR_MVC_CLASS_NAME)).Display());

		/* collab repository constraints view */
		if ( 0 != (nRepositories&SavedSearchModel.REPOSITORY_TYPES_COLLAB) )
		{
			views.AddInnerHTMLElement(EditorView.GetSpacerTable());
			views.AddInnerHTMLElement((m_asOwner.GetView(
			        SavedSearchEditorAS.STR_MVC_SELECT_COLLAB_PROJECT)).Display());
		}
		
		/* content repository constraints view */
		if ( 0 != (nRepositories&SavedSearchModel.REPOSITORY_TYPES_CONTENT) )
		{
			views.AddInnerHTMLElement(EditorView.GetSpacerTable());
			views.AddInnerHTMLElement((m_asOwner.GetView(
			        SavedSearchEditorAS.STR_MVC_SELECT_CONTENT)).Display());
		}
		
		result.AddInnerHTMLElement(CreateLeftNavBar(views));

		/* needed for trees */
		DisplayHiddenInputs(result);
		
		return result;
	}

	/**
	 * Display javascript for this display page
	 *
	 * @return
	 *
	 * @throws HTMLException
	 */
	protected HTMLScriptCollection DisplayJavaScriptFromChild()

	{
		HTMLScriptCollection myScripts = super.DisplayJavaScriptFromChild();

		/* add my tree javascripts */
		myScripts.AddInnerHTMLElement(GetTreeJavascripts());
		
		
		myScripts.AddInnerHTMLElement(m_asOwner.GetView(
				SavSrchFilterView.STR_MVC_CLASS_NAME).DisplayJavascript());
		
		myScripts.AddInnerHTMLElement(m_asOwner.GetView(
				SnapshotQueryRangeView.STR_MVC_CLASS_NAME).DisplayJavascript());

		/* the following views will be shown depending on what repositories are selected */
		int nRepositories = ((ISavedSearchModelRO)m_asOwner.
		        GetModel(SavedSearchModel.STR_MVC_CLASS_NAME)).GetRepositories();
		
		/* kd repository constraints view */
		if ( 0 != (nRepositories&SavedSearchModel.REPOSITORY_TYPES_KD) )
		{
		    myScripts.AddInnerHTMLElement(m_asOwner.GetView(
		            SnapshotKDConstraintsView.STR_MVC_CLASS_NAME).DisplayJavascript());
		}
		
		/* ad repository constraints view */
		if ( 0 != (nRepositories&SavedSearchModel.REPOSITORY_TYPES_ADMIN) )
		{
			myScripts.AddInnerHTMLElement(m_asOwner.GetView(
			        SnapshotADConstraintsView.STR_MVC_CLASS_NAME).DisplayJavascript());
		}

		/* collab repository constraints view */
		if ( 0 != (nRepositories&SavedSearchModel.REPOSITORY_TYPES_COLLAB) )
		{
			myScripts.AddInnerHTMLElement(m_asOwner.GetView(
			        SavedSearchEditorAS.STR_MVC_SELECT_COLLAB_PROJECT).DisplayJavascript());
		}
		
		/* content repository constraints view */
		if ( 0 != (nRepositories&SavedSearchModel.REPOSITORY_TYPES_CONTENT) )
		{
			myScripts.AddInnerHTMLElement(m_asOwner.GetView(
			        SavedSearchEditorAS.STR_MVC_SELECT_CONTENT).DisplayJavascript());
		}
		
//		myScripts.AddInnerHTMLElement(m_asOwner.GetView(
//				SearchConstraintsView.STR_MVC_CLASS_NAME).DisplayJavascript());

		
		String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE,
				GetOwner());

		myScripts.AddInnerHTMLElement(ConfigHelper.GetJSIncluderJSComponentInclude(ConfigHelper.JSCOMPONENT_JSDATEPICKER, GetOwner()) );

		/* if we are in edit (not create) mode, then alert the user that any
		 * changes will not be seen in portlets using this query until their
		 * cache is cleared. */
		IModel model = m_asOwner.GetModel(SavedSearchModel.STR_MVC_CLASS_NAME);
		XPStringBuilder sb = new XPStringBuilder(200);
		sb.Append("function "+UPDATE_ALERT_JS+"()\n");
		sb.Append("{\n");
		sb.Append("  /* This function will contain an Alert() message if the\n");
		sb.Append("   * editor is in edit mode. Otherwise, it just returns true.\n");
		sb.Append("   */\n");
		if (((SavedSearchModel)model).GetObjectID() 
			!= SavedSearchModel.EDITOR_OBJECTID_UNKNOWN)
		{
			sb.Append("  alert(\""+m_asOwner.GetString(2576, "ptmsgs_portaladminmsgs")+"\");\n");
		}
		sb.Append("  return true;\n");
		sb.Append("}\n");
		
		HTMLScript script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
		script.AddInnerHTMLString(sb.ToString());
		myScripts.AddInnerHTMLElement(script);
		
		return myScripts;
	}
	
	private HTMLElement GetTreeJavascripts()
	{
		HTMLScript script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);

		String strFormName = AFormDP.HTMLFORM_NAME;

		// The URL to open a new remote catalog folder tree
		ASURL newCatalogFolderTree = new ASURL(m_asOwner);
		newCatalogFolderTree.SetLinkCreateNewSpace(TreeAS.STR_MVC_CLASS_NAME,
			m_asOwner);

		newCatalogFolderTree.SetSubSpaceKey("CatFolderTree");

		newCatalogFolderTree.SetControl(StartTreeControl.STR_MVC_CLASS_NAME);

		newCatalogFolderTree.AddControlArgument(StartTreeControl.TITLE,
			m_asOwner.GetString(782, "ptmsgs_portaladminmsgs"));

		newCatalogFolderTree.AddControlArgument(StartTreeControl.ACTIONSUBTITLE,
			" " + m_asOwner.GetString(784, "ptmsgs_portaladminmsgs"));
		newCatalogFolderTree.AddControlArgument(StartTreeControl.CLASS_IDS_TO_DISPLAY,
			XPConvert.ToString(PT_CLASSIDS.PT_CATALOGFOLDER_ID));
		newCatalogFolderTree.AddControlArgument(StartTreeControl.ROOT_ID,
			XPConvert.ToString(PT_INTRINSICS.PT_FOLDER_ROOTFOLDER));
		newCatalogFolderTree.AddControlArgument(StartTreeControl.SHOW_ROOT,
			XPConvert.ToString(true));
		newCatalogFolderTree.AddControlArgument(StartTreeControl.MULTI_SELECT,
			XPConvert.ToString(true));
		newCatalogFolderTree.AddControlArgument(StartTreeControl.SELECT_MODE,
			XPConvert.ToString(TreeConstants.N_COMPOSITESELECT));
		newCatalogFolderTree.AddControlArgument(StartTreeControl.SUBMIT_MODE,
			XPConvert.ToString(TreeConstants.N_SERVERSUBMIT));
		newCatalogFolderTree.AddControlArgument(StartTreeControl.PARENT_FORM_NAME,
			strFormName);
		newCatalogFolderTree.AddControlArgument(StartTreeControl.PARENT_FORM_SPACE_ID,
			HTMLHIDDEN_CATSPACEID);
		newCatalogFolderTree.AddControlArgument(StartTreeControl.PARENT_SUBMIT_CALL,
			"treeSubmit");
		
		/* get the selected folders */
		XPArrayList xpFolders = GetModel().GetSelectedFolderIDs(PT_CLASSIDS.PT_CATALOGFOLDER_ID, true);
		XPStringBuilder sbFolders = new XPStringBuilder(300);
		for(int i=0; i<xpFolders.GetSize(); i++)
		{
			sbFolders
				.Append(PT_CLASSIDS.PT_CATALOGFOLDER_ID)
				.Append(",")
				.Append(XPConvert.ToString(xpFolders.GetElement(i)));

			if (i < xpFolders.GetSize()-1)
			{
				sbFolders.Append(",");
			}
		}

		newCatalogFolderTree.AddControlArgument(
			StartTreeControl.SELECTED_ITEMS,
			sbFolders.ToString());

		
		ASURL newAdminFolderTree = new ASURL(m_asOwner);
		newAdminFolderTree.SetLinkCreateNewSpace(TreeAS.STR_MVC_CLASS_NAME,
			m_asOwner);

		newAdminFolderTree.SetSubSpaceKey("AdminFolderTree");

		newAdminFolderTree.SetControl(StartTreeControl.STR_MVC_CLASS_NAME);

		newAdminFolderTree.AddControlArgument(StartTreeControl.TITLE,
			m_asOwner.GetString(2140, "ptmsgs_portaladminmsgs"));

		newAdminFolderTree.AddControlArgument(StartTreeControl.ACTIONSUBTITLE,
			" " + m_asOwner.GetString(789, "ptmsgs_portaladminmsgs"));

		//			newAdminFolderTree.AddControlArgument(StartTreeControl.CLASS_IDS_TO_DISPLAY, XPConvert.ToString(PT_CLASSIDS.PT_ADMIN_FOLDER_ID));
		newAdminFolderTree.AddControlArgument(StartTreeControl.ROOT_ID,
			XPConvert.ToString(PT_INTRINSICS.PT_FOLDER_ROOTFOLDER));
		newAdminFolderTree.AddControlArgument(StartTreeControl.SHOW_ROOT,
			XPConvert.ToString(true));
		newAdminFolderTree.AddControlArgument(StartTreeControl.MULTI_SELECT,
			XPConvert.ToString(true));
		newAdminFolderTree.AddControlArgument(StartTreeControl.SELECT_MODE,
			XPConvert.ToString(TreeConstants.N_COMPOSITESELECT));
		newAdminFolderTree.AddControlArgument(StartTreeControl.SUBMIT_MODE,
			XPConvert.ToString(TreeConstants.N_SERVERSUBMIT));
		newAdminFolderTree.AddControlArgument(StartTreeControl.PARENT_FORM_NAME,
			strFormName);
		newAdminFolderTree.AddControlArgument(StartTreeControl.PARENT_FORM_SPACE_ID,
			HTMLHIDDEN_ADMSPACEID);
		newAdminFolderTree.AddControlArgument(StartTreeControl.PARENT_SUBMIT_CALL,
			"treeSubmit");

		/* get the selected folders */				
		xpFolders = GetModel().GetSelectedFolderIDs(PT_CLASSIDS.PT_ADMIN_FOLDER_ID, true);
		sbFolders = new XPStringBuilder(300);
		for(int i=0; i<xpFolders.GetSize(); i++)
		{
			sbFolders
				.Append(PT_CLASSIDS.PT_ADMIN_FOLDER_ID)
				.Append(",")
				.Append(XPConvert.ToString(xpFolders.GetElement(i)));
			
			if (i < xpFolders.GetSize()-1)
			{
				sbFolders.Append(",");
			}
		}

		newAdminFolderTree.AddControlArgument(
			StartTreeControl.SELECTED_ITEMS,
			sbFolders.ToString());

		
		// The url to open a previously opened remote catalog folder tree
		// This needs to have the SpaceID added to it.
		ASURL oldCatalogFolderTree = new ASURL(m_asOwner);
		oldCatalogFolderTree.SetLinkCreateNewSpace(TreeAS.STR_MVC_CLASS_NAME,
			m_asOwner);
		oldCatalogFolderTree.IsSubSpace();

		// The url to open a previously opened remote admin folder tree
		// This needs to have the SpaceID added to it.
		ASURL oldAdminFolderTree = new ASURL(m_asOwner);
		oldAdminFolderTree.SetLinkCreateNewSpace(TreeAS.STR_MVC_CLASS_NAME,
			m_asOwner);
		oldAdminFolderTree.IsSubSpace();

		String cat    = newCatalogFolderTree.GetURLAsString();
		String oldcat = oldCatalogFolderTree.GetURLAsString();
		String adm    = newAdminFolderTree.GetURLAsString();
		String oldadm = oldAdminFolderTree.GetURLAsString();

		// Open remote catalog folder tree function
		XPStringBuilder sb = new XPStringBuilder(200);
		sb.Append("function " + JAVASCRIPT_CATALOGFOLDERTREE +
			"(catspaceid, repostaction)\n");
		sb.Append("{\n");
		sb.Append("  	document." + AFormDP.HTMLFORM_NAME + "." +
			HTMLHIDDEN_TREEREPOST + ".value = repostaction;\n");
		sb.Append("  	if (catspaceid.value == '')\n");
		sb.Append("    		window.open('" + cat +
			"', '_blank1', '")
		.Append(TreeConstants.GetJavascriptTreeWindowAttributes()).Append("');\n");
		sb.Append("  	else\n");
		sb.Append("    		window.open('" + oldcat + "&" +
			ASConstants.SPACEID +
			"=' + catspaceid.value, '_blank1', '")
		.Append(TreeConstants.GetJavascriptTreeWindowAttributes()).Append("');\n");
		sb.Append("}\n");
		script.AddInnerHTMLString(sb.ToString());

		// Open remote admin folder tree function
		sb = new XPStringBuilder(200);
		sb.Append("function " + JAVASCRIPT_ADMINFOLDERTREE +
			"(admspaceid, repostaction)\n");
		sb.Append("{\n");
		sb.Append("  	document." + AFormDP.HTMLFORM_NAME + "." +
			HTMLHIDDEN_TREEREPOST + ".value = repostaction;\n");
		sb.Append("  	if (admspaceid.value == '')\n");
		sb.Append("    		window.open('" + adm +
			"', '_blank1', '")
		.Append(TreeConstants.GetJavascriptTreeWindowAttributes()).Append("');\n");
		sb.Append("  	else\n");
		sb.Append("    		window.open('" + oldadm + "&" +
			ASConstants.SPACEID +
			"=' + admspaceid.value, '_blank1', '")
		.Append(TreeConstants.GetJavascriptTreeWindowAttributes()).Append("');\n");
		sb.Append("}\n");
		script.AddInnerHTMLString(sb.ToString());

		sb = new XPStringBuilder(300);

		// JavaScript to check all the rows
		/* JHQ bug 22723 fix: because we are using HTMLCHECKBOX_CHECKALL 
		 * for bot admin and catalog folders, we need to pass in the 
		 * correct one, thus passing in the current checkbox being activated.
		 */
		sb.Append("function CheckAllFolderRows(elem, strControlName)\n");
		sb.Append("{ \n");
		sb.Append("  var frm = document." + AFormDP.HTMLFORM_NAME +
			"; \n");
		sb.Append("  var len = frm.elements.length; \n");
		sb.Append("  var nValue = elem.checked; \n");
		sb.Append("  var i = 0; \n");
		sb.Append("  \n");
		sb.Append("  for(i=0 ; i<len ; i++) \n");
		sb.Append("  { \n");
		sb.Append("    if (frm.elements[i].name == strControlName) \n");
		sb.Append("    { \n");
		sb.Append("       frm.elements[i].checked = nValue;\n");
		sb.Append("    } \n");
		sb.Append("  } \n");
		sb.Append("}\n");

		script.AddInnerHTMLString(sb.ToString());

		sb.Append("\n// disable back button navigation to avoid losing presentation data\n");
		sb.Append("window.history.forward(1);\n");

		script.AddInnerHTMLString(sb.ToString());

		// Submit function
		sb.SetLength(0);
		sb.Append("function treeSubmit()\n");
		sb.Append("{\n");
		sb.Append("  " + AFormDP.JAVASCRIPT_SUBMIT_FORM + "(document." + AFormDP.HTMLFORM_NAME + "." +
			HTMLHIDDEN_TREEREPOST + ".value);\n");
		sb.Append("}\n");

		script.AddInnerHTMLString(sb.ToString());
		
		return script;
	}

	/**
	 * Display the action bar for the folders
	 *
	 * @param result
	 *
	 * @throws HTMLException
	 */
	private void DisplayActionBar(HTMLElement element, int type)

	{
		HTMLAnchor myAnchor;
		HTMLImg myImg;
		HTMLElementCollection collActions = new HTMLElementCollection();

		//Get the delete button
		myImg = GetImage(PTImageType.ACTION_REMOVE,
				m_asOwner.GetString(783, "ptmsgs_portaladminmsgs"), "25", "25");

		myAnchor = HTMLConstructHelpers.GetActionBarAnchor(0, myImg, m_asOwner.GetString(783, "ptmsgs_portaladminmsgs"), "", AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" +
								(1000 + SavedSearchRepostControl.ACTION_DEL) + "); return false;", m_asOwner);

		collActions.AddInnerHTMLElement(myAnchor);

		//Get the Add Catalog Folder button
		if (type == PT_CLASSIDS.PT_CATALOGFOLDER_ID)
		{
			PTConstruct myConstruct = ConstructFactory.GetConstruct(PTImageType.ICON_FOLDER);
			myImg = (HTMLImg) myConstruct.GetHTMLElement();
			XPStringBuilder sb = new XPStringBuilder(200);
			sb.Append(JAVASCRIPT_CATALOGFOLDERTREE + "(document." +
				AFormDP.HTMLFORM_NAME + "." + HTMLHIDDEN_CATSPACEID +
				"," + XPConvert.ToString(1000 + SavedSearchRepostControl.ACTION_ADDCATALOGFOLDER) + ");");
			myAnchor = HTMLConstructHelpers.GetActionBarAnchor(1, myImg, m_asOwner.GetString(782, "ptmsgs_portaladminmsgs"), m_asOwner.GetString(782, "ptmsgs_portaladminmsgs"), sb.ToString(), m_asOwner);
			collActions.AddInnerHTMLElement(myAnchor);
		}
		if (type == PT_CLASSIDS.PT_ADMIN_FOLDER_ID)
		{
			//Get the Add Admin folder button
		    PTConstruct myConstruct = ConstructFactory.GetConstruct(PTImageType.ICON_FOLDER);
			myImg = (HTMLImg) myConstruct.GetHTMLElement();
			//Create an Anchor that will call the open the admin folder tree javascript
			XPStringBuilder sb = new XPStringBuilder(200);
			sb.Append(JAVASCRIPT_ADMINFOLDERTREE + "(document." +
				AFormDP.HTMLFORM_NAME + "." + HTMLHIDDEN_ADMSPACEID +
				"," + XPConvert.ToString(1000 + SavedSearchRepostControl.ACTION_ADDADMINFOLDER) + ");");
			myAnchor = HTMLConstructHelpers.GetActionBarAnchor(1, myImg, m_asOwner.GetString(2140, "ptmsgs_portaladminmsgs"), m_asOwner.GetString(2140, "ptmsgs_portaladminmsgs"), sb.ToString(), m_asOwner);
			collActions.AddInnerHTMLElement(myAnchor);
		}
		
//		// The recursive checkbox
//		HTMLElementCollection myInternalColl = new HTMLElementCollection();
//
//		HTMLInput recInput = new HTMLInput(HTMLInputTypes.CHECKBOX,
//				HTMLCHECKBOX_RECURSIVE, "");
//		
//		recInput.SetChecked(GetModel().GetApplyRecursive());
//		recInput.SetValue(XPConvert.ToString(true));
//		recInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
//
//		myInternalColl.AddInnerHTMLElement(recInput);

//		//The text for the recursive button
//		HTMLSpan span = new HTMLSpan();
//		span.SetStyleClass(PTStyleClass.LIST_TEXT);
//		span.AddInnerHTMLString(m_asOwner.GetString(790,
//				"ptmsgs_portaladminmsgs"));
//
//		myInternalColl.AddInnerHTMLElement(span);
//
//		collActions.AddInnerHTMLElement(myInternalColl);

		//Now, get the banner
		element.AddInnerHTMLElement(HTMLConstructHelpers.GetActionBar(collActions, "SavSrchSettingsView", m_asOwner));

	}

	/**
	 * Display the hidden inputs
	 *
	 * @param result The HTMLElementCollection being manipulated
	 *
	 * @throws HTMLException
	 */
	private void DisplayHiddenInputs(HTMLElementCollection result)

	{
		HTMLInput myInput;

		// Hidden input for the Space ID for the tree.  That way if you hit the folder
		// tree link twice, it will open the same tree both times.
		// olegs: cool but dangerous; if you restart server you get a blank tree window
		// due to null root pointer in TreeCompositeIterator.<init>
		// one for the document directory
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN, HTMLHIDDEN_CATSPACEID, "");
		result.AddInnerHTMLElement(myInput);
		myInput.SetValue((GetModel().GetLastCatTreeSpaceId() == null) ? ""
																	  : GetModel()
			.GetLastCatTreeSpaceId());

		// and one for the admin directory
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN, HTMLHIDDEN_ADMSPACEID, "");
		result.AddInnerHTMLElement(myInput);
		myInput.SetValue((GetModel().GetLastAdmTreeSpaceId() == null) ? ""
																	  : GetModel()
			.GetLastAdmTreeSpaceId());

		/* for old javascript submit only ??? javascript submitToParent() fails without it*/

		// hidden input to hold folders selected in tree
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
				HTMLHIDDEN_FOLDERSELECTED, "");
		result.AddInnerHTMLElement(myInput);
		myInput.SetValue("");

		// hidden input to hold repost action on tree submit
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN, HTMLHIDDEN_TREEREPOST, "");
		result.AddInnerHTMLElement(myInput);
		myInput.SetValue("");
	}

	/**
	 * Displays a list of selected folders, admin or catalog
	 *
	 * @param foldercell The cell to display the folders in
	 * @param type The type of folder to display
	 *
	 * @throws HTMLException
	 */
	public void DisplayFolderList(HTMLTableCell foldercell, int type)

	{
		foldercell.SetVAlign("top");

		/* action bar */
		DisplayActionBar(foldercell, type);

		HTMLTable myTable;
		myTable = new HTMLTable();
		foldercell.AddInnerHTMLElement(myTable);
		myTable.SetWidth("100%");
		myTable.SetBorder("0");
		myTable.SetCellPadding("0");
		myTable.SetCellSpacing("2");

		// header
		DisplayFolderListHeader(myTable, type);

		// items
		m_styleListItem = null;

		XPArrayList folderIDs;

		if (type == PT_CLASSIDS.PT_CATALOGFOLDER_ID)
		{
			folderIDs = GetModel().GetCatFolderIDs();
		}
		else
		{
			folderIDs = GetModel().GetAdmFolderIDs();
		}

		PTObjectIdNamePair[] folderIdNamePairs = SortObjectByName(folderIDs,
				type, GetModel().GetSortAscending());

		for (int i = 0;
				(folderIdNamePairs != null) && (i < folderIdNamePairs.length);
				i++)
		{
			int folderId	  = folderIdNamePairs[i].id;
			if (folderIdNamePairs[i].name == null || folderIdNamePairs[i].path == null)
			{
				String[] strItems = GetFolderNameAndPath(folderId, type);
				folderIdNamePairs[i].name = strItems[0];
				folderIdNamePairs[i].path = strItems[1];
			}

			DisplayFolderListItem(myTable, folderId, folderIdNamePairs[i].name, folderIdNamePairs[i].path, type);
		}
	}
	
	/**
	 * Display the header row for a list of folders
	 *
	 * @param myTable The table to display the header in
	 * @param classid The type of folder
	 *
	 * @throws HTMLException
	 */
	protected void DisplayFolderListHeader(HTMLTable myTable, int classid)

	{
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLAnchor myAnchor;
		HTMLInput myInput;
		HTMLImg myImg;
		myRow = new HTMLTableRow();
		myRow.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);
		myTable.AddInnerHTMLElement(myRow);

		//Add the "check all" checkbox
		myCell = new HTMLTableCell();
		myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLCHECKBOX_CHECKALL, m_asOwner.GetString(2274, "ptmsgs_portaladminmsgs"));
		myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
		
		/* JHQ bug 22723 fix: name of checkbox element needs to be in quote,
		 * and because we are using HTMLCHECKBOX_CHECKALL for bot admin and 
		 * catalog folders, we need to pass in the correct one, thus passing in 
		 * this.
		 */
		if (PT_CLASSIDS.PT_CATALOGFOLDER_ID == classid)
		{
			myInput.SetOnClick("CheckAllFolderRows(this,'" + HTMLCHECKBOX_CATALOGFOLDER + "');");
		} else
		{
			myInput.SetOnClick("CheckAllFolderRows(this,'" + HTMLCHECKBOX_ADMINFOLDER + "');");
		}
		myCell.AddInnerHTMLElement(myInput);
		myRow.AddInnerHTMLElement(myCell);

		myCell = new HTMLTableCell();
		myCell.SetWidth("100%");
		myCell.SetHeight("20");

		myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
		myRow.AddInnerHTMLElement(myCell);

		myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
		myAnchor.SetOnClick(AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" +
						(1000 + SavedSearchRepostControl.ACTION_SORT) + ");");

		if (PT_CLASSIDS.PT_CATALOGFOLDER_ID == classid)
		{
			myAnchor.AddInnerHTMLString(m_asOwner.GetString(780,
					"ptmsgs_portaladminmsgs"));
		}
		else
		{
			myAnchor.AddInnerHTMLString(m_asOwner.GetString(781,
					"ptmsgs_portaladminmsgs"));
		}

		myAnchor.SetStyleClass(PTStyleClass.LIST_TEXT);

		myCell.AddInnerHTMLElement(myAnchor);

		myImg = GetImage(GetModel().GetSortAscending()
				? PTImageType.ICON_ARROW_DESCEND : PTImageType.ICON_ARROW_ASCEND,
				m_asOwner.GetString(787, "ptmsgs_portaladminmsgs"), "16", "16");
		myImg.SetAlign("absMiddle");
		myAnchor.AddInnerHTMLElement(myImg);
	}

	/**
	 * Display a single item in a list of folders
	 *
	 * @param myTable The table being manipulated
	 * @param id The id of the folder
	 * @param name The name of the folder
	 * @param intType = The classid of the folder type
	 *
	 * @throws HTMLException
	 */
	protected void DisplayFolderListItem(HTMLTable myTable, int id, String name, String path, int intType)

	{
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLInput myInput;
		HTMLSpan mySpan;
		HTMLImg myImg;
		myRow = new HTMLTableRow();

		m_styleListItem = ((m_styleListItem == null) || (m_styleListItem == PTStyleClass.LIST_ITEM_ONE_BG)) ? PTStyleClass.LIST_ITEM_TWO_BG : PTStyleClass.LIST_ITEM_ONE_BG;

		myRow.SetStyleClass(m_styleListItem);
		myTable.AddInnerHTMLElement(myRow);
		myCell = new HTMLTableCell();
		myCell.SetWidth("20");
		myCell.SetVAlign("center");
		myCell.SetAlign("middle");
		myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
		myRow.AddInnerHTMLElement(myCell);

		if (intType == PT_CLASSIDS.PT_ADMIN_FOLDER_ID)
		{
			myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLCHECKBOX_ADMINFOLDER,
					m_asOwner.GetString(786, "ptmsgs_portaladminmsgs") + " \"" +
					name + "\"");
		}
		else
		{
			myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLCHECKBOX_CATALOGFOLDER,
					m_asOwner.GetString(786, "ptmsgs_portaladminmsgs") + " \"" +
					name + "\"");
		}
		myInput.SetChecked(false);
		myInput.SetValue(XPConvert.ToString(id));
		myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
		myCell.AddInnerHTMLElement(myInput);
		myCell = new HTMLTableCell();
		myCell.SetWidth("100%");
		myCell.SetHeight("20");
		myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
		myCell.SetNoWrap(true);
		myRow.AddInnerHTMLElement(myCell);

		mySpan = new HTMLSpan();
		mySpan.SetStyleClass(PTStyleClass.OBJECT_TEXT);

		myImg = GetImage(PTImageType.ICON_FOLDER,
				m_asOwner.GetString(784, "ptmsgs_portaladminmsgs"), "18", "20");
		myImg.SetAlign("absMiddle");
		mySpan.AddInnerHTMLElement(myImg);
		mySpan.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		mySpan.AddInnerHTMLEncodedString(name);
		mySpan.SetTitle(XPHttpUtility.HtmlEncode(path));
		myCell.AddInnerHTMLElement(mySpan);
	}
	
	/**
	 * Returns the read-only model
	 *
	 * @return ISavedSearchModelRO
	 */
	protected ISavedSearchModelRO GetModel()
	{
		return (ISavedSearchModelRO) m_asOwner.GetModel(SavedSearchModel.STR_MVC_CLASS_NAME);
	}

	/**
	 * Helper function for retrieving images from the construct factory
	 *
	 * @param type PTImageType
	 * @param alt String
	 * @param w width - String
	 * @param h height - String
	 *
	 * @return HTMLImg
	 */
	public HTMLImg GetImage(PTImageType type, String alt, String w, String h)
	{
//		try
//		{
			PTConstruct myConstruct = UIConstructFactory.GetConstruct(type,
					true, this.m_asOwner);
			HTMLImg myImg		    = (HTMLImg) myConstruct.GetHTMLElement();
			myImg.SetAlt(alt);
			myImg.SetBorder("0");
			myImg.SetWidth(w);
			myImg.SetHeight(h);

			return myImg;
//		}
//		catch (Exception e)
//		{
//			log.Error(e, "SmartSortEditorMainView.DisplayJavascript couldn't add javascript to the HTMLScript element: " +
//				e.getMessage());
//
//			throw new XPException(m_asOwner.GetString(779,
//					"ptmsgs_portaladminmsgs") + " (Image)", e);
//		}
	}
	
	/**
	 * Helper function for sorting
	 *
	 * @param ids ID of folders being manipulated
	 * @param type The type of folder
	 * @param ascending Ascending or descending
	 *
	 * @return PTObjectIDNamePair[]
	 */
	protected PTObjectIdNamePair[] SortObjectByName(XPArrayList ids, int type,
		boolean ascending)
	{
		if (ids == null || 0 == ids.GetSize())
		{
			return null;
		}

		PTFolderObjectNameGetter nameGetter = new PTFolderObjectNameGetter(type);

		XPArrayList pairs = new XPArrayList(ids.GetSize());

		for (int i = 0; i < ids.GetSize(); i++)
		{
			int folderId = XPConvert.ToInteger(ids.GetElement(i));
			PTObjectIdNamePair nextItem = new PTObjectIdNamePair(folderId);
			String[] folderInfo = nameGetter.GetItem(folderId, (IPTSession)m_asOwner.GetUserSession());
			if (null != folderInfo)
			{
				if (folderInfo[0].equals(""))
				{
					folderInfo[0] = m_asOwner.GetString(111, "ptmsgs_portaladminmsgs");
					folderInfo[1] = "";
				}
				nextItem.name = folderInfo[0];
				nextItem.path = folderInfo[1];
			}

			pairs.Add(nextItem);
		}

		XPCollections.Sort(pairs,
			new PTObjectNameComparator(
				(IPTObjectNameGetter) new PTFolderObjectNameGetter(type),
				(IPTSession) m_asOwner.GetUserSession(), ascending));

		PTObjectIdNamePair[] result = new PTObjectIdNamePair[ids.GetSize()];

		return (PTObjectIdNamePair[]) pairs.ToArray(result);

		// we don't change source array, but if we wantewd to:
		// for(int i=0; i<ids.length; i++)	ids[i]=new Integer(pairs[i].id);
	}
	
	/**
	 * Get the folder name based on ID and type
	 *
	 * @param id The ID of the folder
	 * @param type The type of the folder
	 *
	 * @return
	 */
	protected String[] GetFolderNameAndPath(int id, int type)
	{
		IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();

		try
		{
			if (PT_CLASSIDS.PT_ADMIN_FOLDER_ID == type)
			{
				IPTAdminFolder ptFolder = ptSession.GetAdminCatalog()
					.OpenAdminFolder(id, false);

				String[] strItems = new String[2];
				strItems[0] = ptFolder.GetLocalizedName();
				strItems[1] = ptFolder.GetPath();
				return strItems;
			}
			else
			{
				IPTFolder ptFolder = ptSession.GetCatalog().OpenFolder(id, false);

				String[] strItems = new String[2];
				strItems[0] = ptFolder.GetLocalizedName();
				strItems[1] = ptFolder.GetPath();
				return strItems;
			}
		}
		catch(Exception e)
		{
			String[] strFolderInfo = new String[2];
			strFolderInfo[0] = m_asOwner.GetString(111, "ptmsgs_portaladminmsgs");
			strFolderInfo[1] = e.getMessage();
			return strFolderInfo;
		}

	}
}