/**
 * Created on Dec 6, 2002
 * by adolfoc
 */
package com.plumtree.portalpages.admin.editors.webservice;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author adolfoc
 *
 * This class is used to automatically identify and display
 * the appropriate views based on the list of views in the WebServiceModel
 * This should help avoid mistakes with forgetting to display javascript and
 * missing spacers, etc.
 */
public abstract class WSPluginDP extends EditorDP
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.webservice.WSPluginDP");

	/**
	 * Returns the name of the Help Topic to be used for this
	 * page.
	 * @return
	 */
	public abstract String GetPluginHelpTopic();

	/**
	 * Returns a list of all of the possible views this page can show.
	 * (Used to generate javascript for each view).
	 */
	public abstract XPArrayList GetAllPossibleViews();

	/**
	 * Returns an ordered list of the views for this page, in this mode.
	 */
	public abstract XPArrayList GetOrderedViewList(int nWSMode);

	public void InitPluginDP()
	{
		//AGC - we must set the validation functions from our submodels.
		WebServiceModel myModel = (WebServiceModel) m_asOwner.GetModel(WebServiceModel.STR_MVC_CLASS_NAME);
		XPArrayList xpalWSViews = GetOrderedViewList(myModel.GetWSMode());

		IXPEnumerator xpEnumWSSubModels = xpalWSViews.GetEnumerator();

		while (xpEnumWSSubModels.MoveNext())
		{
			String strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();

			try
			{
				IWSSubModel wsSubModelCurrent = (IWSSubModel) m_asOwner.GetModel(strSubModelKeyCurrent);

				String strValidationFunc = wsSubModelCurrent.GetValidationFunction();

				if ((null != strValidationFunc) &&
						(strValidationFunc.trim().length() > 0))
				{
					AddToValidationFunction(strValidationFunc);
				}
			}
			catch (Exception e)
			{
				log.Error(e,GetName() +
					"::PageDisplay() The following error occured when trying to display the Javascript for Section (" +
					strSubModelKeyCurrent + ").");

				throw new XPException(m_asOwner.GetString(1197,
						"ptmsgs_portaladminmsgs"), e);
			}
		}
	}

	/**
	 * Determines whether a specific view is enabled in the suggested mode.
	 */
	public boolean IsEnabled(int nWSMode, String strViewName)
	{
		XPArrayList xpalWSViews = GetOrderedViewList(nWSMode);

		IXPEnumerator xpEnumWSSubModels = xpalWSViews.GetEnumerator();

		while (xpEnumWSSubModels.MoveNext())
		{
			String strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();

			if (0 == strSubModelKeyCurrent.compareTo(strViewName))
			{
				return true;
			}
		}

		return false;
	}

	
	/**
	 *
	 * @see com.plumtree.uiinfrastructure.form.AFormDP#PageDisplay()
	 *
	 * @return HTMLElement
	 */
	public HTMLElement PageDisplay()

	{
		HTMLElementCollection result = new HTMLElementCollection();

		// Insert super class HTML
		result.AddInnerHTMLElement(super.PageDisplay());

		HTMLElementCollection views = new HTMLElementCollection();

		WebServiceModel myModel = (WebServiceModel) m_asOwner.GetModel(WebServiceModel.STR_MVC_CLASS_NAME);
		XPArrayList xpalWSViews = GetOrderedViewList(myModel.GetWSMode());

		IXPEnumerator xpEnumWSSubModels = xpalWSViews.GetEnumerator();

		while (xpEnumWSSubModels.MoveNext())
		{
			String strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();

			try
			{
				EditorView wsSubViewCurrent = (EditorView) m_asOwner.GetView(strSubModelKeyCurrent);
				views.AddInnerHTMLElement(wsSubViewCurrent.Display());
				views.AddInnerHTMLElement(EditorView.GetSpacerTable());
			}
			catch (Exception e)
			{
				log.Error(e, GetName() +
					"::PageDisplay() The following error occurred when trying to display the HTML for Section (" +
					strSubModelKeyCurrent + ").");

				throw new XPException(m_asOwner.GetString(1197,
						"ptmsgs_portaladminmsgs"), e);
			}
		}

		result.AddInnerHTMLElement(CreateLeftNavBar(views));

		return result;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.AFormDP#DisplayJavaScriptFromChild()
	 */
	protected HTMLScriptCollection DisplayJavaScriptFromChild()

	{
		HTMLScriptCollection result = super.DisplayJavaScriptFromChild();

		if (null == result)
		{
			result = new HTMLScriptCollection();
		}

		// AGC- Note, we actually show all of our javascript all of the time. This is because
		// the Page-level validation function is determined at editor start-up. If we
		// don't supply all of the possible validation functions, we will cause javascript errors
		// if the WSMODE is ever changed. In order to avoid validating components that don't show,
		// we rely on the views returning blank/alwaystrue validation when not in the appropriate
		// mode.
		XPArrayList xpalWSViews = GetAllPossibleViews();

		IXPEnumerator xpEnumWSSubModels = xpalWSViews.GetEnumerator();

		while (xpEnumWSSubModels.MoveNext())
		{
			String strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();

			try
			{
				EditorView wsSubViewCurrent = (EditorView) m_asOwner.GetView(strSubModelKeyCurrent);
				HTMLScript scriptCurrent    = wsSubViewCurrent.DisplayJavascript();

				if (null != scriptCurrent)
				{
					result.AddInnerHTMLElement(scriptCurrent);
				}
			}
			catch (Exception e)
			{
				log.Error(e, GetName() +
					"::PageDisplay() The following error occured when trying to display the Javascript for Section (" +
					strSubModelKeyCurrent + ").");
				

				throw new XPException(m_asOwner.GetString(1197,
						"ptmsgs_portaladminmsgs"), e);
			}
		}

		return result;
	}
}

