package com.plumtree.portalpages.admin.editors.webservice;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.editors.webservice.plugins.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.expandablelist.*;

/**
 * @author adolfoc
 *
 */
public class WebServiceEditorAS extends ObjEditorAS
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.webservice.WebServiceEditorAS");

	/** The Object Name of the Remote Server Editor */
	public static final String STR_MVC_CLASS_NAME = "WebServiceEditor";


	//MES-: Why aren't these constants defined in the class to which they are relevant?
	//AGC-Response: They are. The nature of plugins is that they should have keys that are unique within the
	//       activity space they are used in. It's not good enough for each chunk of code to have it's own unique name. 
	// 		 Why? Because plugins can be used more than once within an activity space. For instance, the Prefs plugin keys 
	//       below all refer to the same actual plugin. They must be keyed differently to avoid collision.
	// AGC- IMPORTANT NOTE: PLEASE READ IF YOU WANT TO UNDERSTAND THE PLUGINDP AND ASSOCIATED CODE!
	// 
	// An important part of the automated plugin placement and hiding code is that plugin views share 
	// a storage key with their associated models. This framework relies on being to access the related
	// model and view with the same key! If you intend to add any plugin views and models to this framework,
	// keep this in mind. 

	/** Plugin key for ActSecurityPluginView*/
	public static final String strStorage_ActSecurityPluginView = "STOR_ActSecurityPluginView";

	/** Plugin key for AddSettingsPluginView */
	public static final String strStorage_AddSettingsPluginView = "STOR_AddSettingsPluginView";

	/** Plugin key for AddSettingsPluginView */
	public static final String strStorage_AddUserInfoPluginView = "STOR_AddUserInfoPluginView";

	/** Plugin key for AdminPrefsPluginView*/
	public static final String strStorage_AdminPrefsPluginView = "STOR_AdminPrefsPluginView";

	/** Plugin key for AdvSettingsPluginView*/
	public static final String strStorage_AdvSettingsPluginView = "STOR_AdvSettingsPluginView";

	/** Plugin key for AssocObjectsPluginView*/
	public static final String strStorage_AssocObjectsPluginView = "STOR_AssocObjectsPluginView";

	/** Plugin key for AWSSettingsPluginView*/
	public static final String strStorage_AWSSettingsPluginView = "STOR_AWSSettingsPluginView";

	/** Plugin key for PWSSettingsPluginView*/
	public static final String strStorage_PWSSettingsPluginView = "STOR_PWSSettingsPluginView";

	/** Plugin key for MWSSettingsPluginView*/
	public static final String strStorage_MWSSettingsPluginView = "STOR_MWSSettingsPluginView";

	/** Plugin key for BasicAuthPluginView */
	public static final String strStorage_BasicAuthPluginView = "STOR_BasicAuthPluginView";

	/** Plugin key for ChooseWSTypePluginView */
	public static final String strStorage_ChooseWSTypePluginView = "STOR_ChooseWSTypePluginView";

	/** Plugin key for CommunityPrefsListPluginView*/
	public static final String strStorage_CommunityPrefsListPluginView = "STOR_CommunityPrefsListPluginView";

	/** Plugin key for CommunityPrefsPluginView*/
	public static final String strStorage_CommunityPrefsPluginView = "STOR_CommunityPrefsPluginView";

	/** Plugin key for CredentialVaultPluginView*/
	public static final String strStorage_CredentialVaultPluginView = "STOR_CredentialVaultPluginView";
	
	/** Plugin key for CWSSettingsPluginView*/
	public static final String strStorage_CWSSettingsPluginView = "STOR_CWSSettingsPluginView";
	
	/** Plugin key for FormLoginPluginView*/
	public static final String strStorage_FormLoginPluginView = "STOR_FormLoginPluginView";
	
	/** Plugin key for FormLoginFieldsPluginView*/
	public static final String strStorage_FormLoginFieldsPluginView = "STOR_FormLoginFieldsPluginView";
	
	/** Plugin key for GatewayCachingPluginView*/
	public static final String strStorage_GatewayCachingPluginView = "STOR_GatewayCachingPluginView";

	/** Plugin key for GatewayPrefixPluginView*/
	public static final String strStorage_GatewayPrefixPluginView = "STOR_GatewayPrefixPluginView";

	/** Plugin key for GatewaySettingsPluginView*/
	public static final String strStorage_GatewaySettingsPluginView = "STOR_GatewaySettingsPluginView";
	
	/** Plugin key for DebugConfigPluginView*/
	public static final String strStorage_DebugConfigPluginView = "STOR_DebugConfigPluginView";	

	/** Plugin key for DebugUserPluginView*/
	public static final String strStorage_DebugUserPluginView = "STOR_DebugUserPluginView";	
	
	/** Plugin key for DebugPortletPluginView*/
	public static final String strStorage_DebugPortletPluginView = "STOR_DebugPortletPluginView";	

	/** Plugin key for HelpURLPluginView */
	public static final String strStorage_HelpURLPluginView = "STOR_HelpURLPluginView";

	/** Plugin key for HTTPSettingsPluginView */
	public static final String strStorage_HTTPSettingsPluginView = "STOR_HTTPSettingsPluginView";

	/** Plugin key for RemoteConfigPluginView*/
	public static final String strStorage_RemoteConfigPluginView = "STOR_RemoteConfigPluginView";

	/** Plugin key for RemoteMigrationPluginView*/
	public static final String strStorage_RemoteMigrationPluginView = "STOR_RemoteMigrationPluginView";

	/** Plugin key for RemoteServerPluginView*/
	public static final String strStorage_RemoteServerPluginView = "STOR_RemoteServerPluginView";

	/** Plugin key for SelectDevicesPluginView*/
	public static final String strStorage_SelectDevicesPluginView = "STOR_SelectDevicesPluginView";

	/** Plugin key for SOAPEncodingPluginView*/
	public static final String strStorage_SOAPEncodingPluginView = "STOR_SOAPEncodingPluginView";

	/** Plugin key for SummaryURLPluginView*/
	public static final String strStorage_SummaryURLPluginView = "STOR_SummaryURLPluginView";

	/** Plugin key for SWSSettingsPluginView*/
	public static final String strStorage_SWSSettingsPluginView = "STOR_SWSSettingsPluginView";

	/** Plugin key for UserInfoPluginView*/
	public static final String strStorage_UserInfoPluginView = "STOR_UserInfoPluginView";

	/** Plugin key for UserPrefsListPluginView*/
	public static final String strStorage_UserPrefsListPluginView = "STOR_UserPrefsListPluginView";

	/** Plugin key for UserPrefsPluginView*/
	public static final String strStorage_UserPrefsPluginView = "STOR_UserPrefsPluginView";

	/** Plugin key for SessionPrefsListPluginView*/
	public static final String strStorage_SessionPrefsListPluginView = "STOR_SessionPrefsListPluginView";

	/** Plugin key for SessionPrefsPluginView*/
	public static final String strStorage_SessionPrefsPluginView = "STOR_SessionPrefsPluginView";

	/** Plugin key for XUIURLPluginView */
	public static final String strStorage_XUIURLPluginView = "STOR_XUIURLPluginView";

	/** Plugin key for IntrinsicPortletWSSettingsPluginView */
	public static final String strStorage_IntrinsicPortletWSSettingsPluginView =
		"STOR_IntrinsicPortletWSSettingsPluginView";

	/** Plugin key for IntrinsicAdminPrefsPluginView*/
	public static final String strStorage_IntrinsicAdminPrefsPluginView = "STOR_IntrinsicAdminPrefsPluginView";

	/** Plugin key for IntrinsicUserPrefsPluginView*/
	public static final String strStorage_IntrinsicUserPrefsPluginView = "STOR_IntrinsicUserPrefsPluginView";

	/** Plugin key for IntrinsicCommunityPrefsPluginView*/
	public static final String strStorage_IntrinsicCommunityPrefsPluginView = "STOR_IntrinsicCommunityPrefsPluginView";

	/** Plugin key for IntrinsicHelpSpacePluginView */
	public static final String strStorage_IntrinsicHelpSpacePluginView = "STOR_IntrinsicHelpSpacePluginView";

	/** Plugin key for IntrinsicProfilePluginView */
	public static final String strStorage_IntrinsicProfilePluginView = "STOR_IntrinsicProfilePluginView";

	/** sub-space key for tree*/
	public static final String strTREE_SUBSPACE = "WSTreeSubSpace";

	/** sub-space key for second tree*/
	public static final String strTREE_SECOND_SUBSPACE = "WSTreeSecondSubSpace";
	
	/** sub-space key for third tree - CV LockBox Tree */
	public static final String strTREE_LOCKBOX_SUBSPACE = "WSTreeLockBoxSubSpace";
	
	/** sub-space key for fourth tree - Debug User Tree */
	public static final String strTREE_DEBUGUSER_SUBSPACE = "WSTreeDebugUserSubSpace";

	
	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new WebServiceEditorAS();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.editor#GetDependenciesViewName()
	 *
	 */
	public String GetDependenciesViewName()
	{
		return (WSDependenciesView.STR_MVC_CLASS_NAME);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorAS#GetEditorTitle()
	 */
	public String GetEditorTitle()
	{
		WebServiceModel myWSModel=(WebServiceModel)GetModel(m_strModelName);
		String format=null;
		
		if (EditorStartControl.EDITOR_START_FLAG_CREATE == ((EditorModel) GetModel(
					m_strModelName)).GetEditorType())
		{
			format=GetString(2623, "ptmsgs_portaladminmsgs");
			return XPResourceManager.FormatMessage(format,myWSModel.GetWSObjectTitle());
		}
		else
		{
			format=GetString(2624, "ptmsgs_portaladminmsgs");
			return XPResourceManager.FormatMessage(format,myWSModel.GetWSObjectTitle(), GetObjectName());
		}
		
		
		
	}
	
	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostAS#GetRepostControlName()
	 */
	public String GetRepostControlName()
	{
		return (WebServiceRepostControl.STR_MVC_CLASS_NAME);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.AActivitySpace#Init()
	 */
	public void Init()
	{
		// Get and initialize my data model.
		m_strModelName = WebServiceModel.STR_MVC_CLASS_NAME;

		super.Init();

		IModel myModel			  = GetModel(m_strModelName);
		WebServiceModel myWSModel = (WebServiceModel) myModel;

		// Register my main control
		RegisterControl(WebServiceRepostControl.STR_MVC_CLASS_NAME, myModel);

		RegisterView(WSDependenciesView.STR_MVC_CLASS_NAME, myModel);

		// Register Plugins
		RegisterModel(ActSecurityPluginModel.STR_MVC_CLASS_NAME,
			strStorage_ActSecurityPluginView);
		RegisterView(ActSecurityPluginView.STR_MVC_CLASS_NAME,
			strStorage_ActSecurityPluginView,
			GetModel(strStorage_ActSecurityPluginView));
		RegisterControl(ExpListControl.STR_MVC_CLASS_NAME,
			strStorage_ActSecurityPluginView,
			GetModel(strStorage_ActSecurityPluginView));
		myWSModel.RegisterWSSubModel(strStorage_ActSecurityPluginView);

		RegisterModel(AddSettingsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_AddSettingsPluginView);
		RegisterView(AddSettingsPluginView.STR_MVC_CLASS_NAME,
			strStorage_AddSettingsPluginView,
			GetModel(strStorage_AddSettingsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_AddSettingsPluginView);

		RegisterModel(AddUserInfoPluginModel.STR_MVC_CLASS_NAME,
			strStorage_AddUserInfoPluginView);
		RegisterView(AddUserInfoPluginView.STR_MVC_CLASS_NAME,
			strStorage_AddUserInfoPluginView,
			GetModel(strStorage_AddUserInfoPluginView));
		RegisterControl(ExpListControl.STR_MVC_CLASS_NAME,
			strStorage_AddUserInfoPluginView,
			GetModel(strStorage_AddUserInfoPluginView));
		myWSModel.RegisterWSSubModel(strStorage_AddUserInfoPluginView);

		RegisterModel(UserPrefsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_AdminPrefsPluginView);
		RegisterView(UserPrefsPluginView.STR_MVC_CLASS_NAME,
			strStorage_AdminPrefsPluginView,
			GetModel(strStorage_AdminPrefsPluginView));
		((UserPrefsPluginModel) GetModel(strStorage_AdminPrefsPluginView)).SetPrefMode(UserPrefsPluginModel.UPPM_PREFMODE_ADMINPREFS);
		myWSModel.RegisterWSSubModel(strStorage_AdminPrefsPluginView);

		RegisterModel(IntrinsicUserPrefsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicAdminPrefsPluginView);
		RegisterView(IntrinsicUserPrefsPluginView.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicAdminPrefsPluginView,
			GetModel(strStorage_IntrinsicAdminPrefsPluginView));
		((IntrinsicUserPrefsPluginModel) GetModel(strStorage_IntrinsicAdminPrefsPluginView)).SetPrefMode(IntrinsicUserPrefsPluginModel.UPPM_PREFMODE_ADMINPREFS);
		myWSModel.RegisterWSSubModel(strStorage_IntrinsicAdminPrefsPluginView);

		RegisterModel(AdvSettingsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_AdvSettingsPluginView);
		RegisterView(AdvSettingsPluginView.STR_MVC_CLASS_NAME,
			strStorage_AdvSettingsPluginView,
			GetModel(strStorage_AdvSettingsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_AdvSettingsPluginView);

		RegisterModel(AssocObjectsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_AssocObjectsPluginView);
		RegisterView(AssocObjectsPluginView.STR_MVC_CLASS_NAME,
			strStorage_AssocObjectsPluginView,
			GetModel(strStorage_AssocObjectsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_AssocObjectsPluginView);

		RegisterModel(AWSSettingsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_AWSSettingsPluginView);
		RegisterView(AWSSettingsPluginView.STR_MVC_CLASS_NAME,
			strStorage_AWSSettingsPluginView,
			GetModel(strStorage_AWSSettingsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_AWSSettingsPluginView);

		RegisterModel(BasicAuthPluginModel.STR_MVC_CLASS_NAME,
			strStorage_BasicAuthPluginView);
		RegisterView(BasicAuthPluginView.STR_MVC_CLASS_NAME,
			strStorage_BasicAuthPluginView,
			GetModel(strStorage_BasicAuthPluginView));
		myWSModel.RegisterWSSubModel(strStorage_BasicAuthPluginView);
		
		RegisterModel(FormLoginPluginModel.STR_MVC_CLASS_NAME,
			strStorage_FormLoginPluginView);
		RegisterView(FormLoginPluginView.STR_MVC_CLASS_NAME,
			strStorage_FormLoginPluginView,
			GetModel(strStorage_FormLoginPluginView));
		myWSModel.RegisterWSSubModel(strStorage_FormLoginPluginView);

		RegisterModel(FormLoginFieldsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_FormLoginFieldsPluginView);
		RegisterView(FormLoginFieldsPluginView.STR_MVC_CLASS_NAME,
			strStorage_FormLoginFieldsPluginView,
			GetModel(strStorage_FormLoginFieldsPluginView));
		RegisterControl(ExpListControl.STR_MVC_CLASS_NAME,
			strStorage_FormLoginFieldsPluginView,
			GetModel(strStorage_FormLoginFieldsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_FormLoginFieldsPluginView);

		RegisterModel(PrefsListPluginModel.STR_MVC_CLASS_NAME,
			strStorage_CommunityPrefsListPluginView);
		RegisterView(PrefsListPluginView.STR_MVC_CLASS_NAME,
			strStorage_CommunityPrefsListPluginView,
			GetModel(strStorage_CommunityPrefsListPluginView));
		RegisterControl(ExpListControl.STR_MVC_CLASS_NAME,
			strStorage_CommunityPrefsListPluginView,
			GetModel(strStorage_CommunityPrefsListPluginView));
		((PrefsListPluginModel) GetModel(strStorage_CommunityPrefsListPluginView)).SetPrefMode(PrefsListPluginModel.PLPM_PREFMODE_COMMUNITYPREFS);
		myWSModel.RegisterWSSubModel(strStorage_CommunityPrefsListPluginView);

		RegisterModel(UserPrefsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_CommunityPrefsPluginView);
		RegisterView(UserPrefsPluginView.STR_MVC_CLASS_NAME,
			strStorage_CommunityPrefsPluginView,
			GetModel(strStorage_CommunityPrefsPluginView));
		((UserPrefsPluginModel) GetModel(strStorage_CommunityPrefsPluginView)).SetPrefMode(UserPrefsPluginModel.UPPM_PREFMODE_COMMUNITYPREFS);
		myWSModel.RegisterWSSubModel(strStorage_CommunityPrefsPluginView);

		RegisterModel(PrefsListPluginModel.STR_MVC_CLASS_NAME,
				strStorage_SessionPrefsListPluginView);
			RegisterView(PrefsListPluginView.STR_MVC_CLASS_NAME,
				strStorage_SessionPrefsListPluginView,
				GetModel(strStorage_SessionPrefsListPluginView));
			RegisterControl(ExpListControl.STR_MVC_CLASS_NAME,
				strStorage_SessionPrefsListPluginView,
				GetModel(strStorage_SessionPrefsListPluginView));
			((PrefsListPluginModel) GetModel(strStorage_SessionPrefsListPluginView)).SetPrefMode(PrefsListPluginModel.PLPM_PREFMODE_SESSIONPREFS);
			myWSModel.RegisterWSSubModel(strStorage_SessionPrefsListPluginView);

		RegisterModel(UserPrefsPluginModel.STR_MVC_CLASS_NAME,
				strStorage_SessionPrefsPluginView);
			RegisterView(UserPrefsPluginView.STR_MVC_CLASS_NAME,
				strStorage_SessionPrefsPluginView,
				GetModel(strStorage_SessionPrefsPluginView));
			((UserPrefsPluginModel) GetModel(strStorage_SessionPrefsPluginView)).SetPrefMode(UserPrefsPluginModel.UPPM_PREFMODE_SESSIONPREFS);
			myWSModel.RegisterWSSubModel(strStorage_SessionPrefsPluginView);	

		RegisterModel(IntrinsicUserPrefsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicCommunityPrefsPluginView);
		RegisterView(IntrinsicUserPrefsPluginView.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicCommunityPrefsPluginView,
			GetModel(strStorage_IntrinsicCommunityPrefsPluginView));
		((IntrinsicUserPrefsPluginModel) GetModel(strStorage_IntrinsicCommunityPrefsPluginView)).SetPrefMode(IntrinsicUserPrefsPluginModel.UPPM_PREFMODE_COMMUNITYPREFS);
		myWSModel.RegisterWSSubModel(strStorage_IntrinsicCommunityPrefsPluginView);

		RegisterModel(CWSSettingsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_CWSSettingsPluginView);
		RegisterView(CWSSettingsPluginView.STR_MVC_CLASS_NAME,
			strStorage_CWSSettingsPluginView,
			GetModel(strStorage_CWSSettingsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_CWSSettingsPluginView);

		RegisterModel(ChooseWSTypePluginModel.STR_MVC_CLASS_NAME,
			strStorage_ChooseWSTypePluginView);
		RegisterView(ChooseWSTypePluginView.STR_MVC_CLASS_NAME,
			strStorage_ChooseWSTypePluginView,
			GetModel(strStorage_ChooseWSTypePluginView));
		myWSModel.RegisterWSSubModel(strStorage_ChooseWSTypePluginView);

		RegisterModel(GatewayCachingPluginModel.STR_MVC_CLASS_NAME,
			strStorage_GatewayCachingPluginView);
		RegisterView(GatewayCachingPluginView.STR_MVC_CLASS_NAME,
			strStorage_GatewayCachingPluginView,
			GetModel(strStorage_GatewayCachingPluginView));
		myWSModel.RegisterWSSubModel(strStorage_GatewayCachingPluginView);

		RegisterModel(GatewayPrefixPluginModel.STR_MVC_CLASS_NAME,
			strStorage_GatewayPrefixPluginView);
		RegisterView(GatewayPrefixPluginView.STR_MVC_CLASS_NAME,
			strStorage_GatewayPrefixPluginView,
			GetModel(strStorage_GatewayPrefixPluginView));
		RegisterControl(ExpListControl.STR_MVC_CLASS_NAME,
			strStorage_GatewayPrefixPluginView,
			GetModel(strStorage_GatewayPrefixPluginView));
		myWSModel.RegisterWSSubModel(strStorage_GatewayPrefixPluginView);

		RegisterModel(GatewaySettingsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_GatewaySettingsPluginView);
		RegisterView(GatewaySettingsPluginView.STR_MVC_CLASS_NAME,
			strStorage_GatewaySettingsPluginView,
			GetModel(strStorage_GatewaySettingsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_GatewaySettingsPluginView);

		RegisterModel(CredentialVaultPluginModel.STR_MVC_CLASS_NAME,
			strStorage_CredentialVaultPluginView);
		RegisterView(CredentialVaultPluginView.STR_MVC_CLASS_NAME,
			strStorage_CredentialVaultPluginView,
			GetModel(strStorage_CredentialVaultPluginView));
		myWSModel.RegisterWSSubModel(strStorage_CredentialVaultPluginView);
		
		RegisterModel(DebugConfigPluginModel.STR_MVC_CLASS_NAME,
			strStorage_DebugConfigPluginView);
		RegisterView(DebugConfigPluginView.STR_MVC_CLASS_NAME,
			strStorage_DebugConfigPluginView,
			GetModel(strStorage_DebugConfigPluginView));
		myWSModel.RegisterWSSubModel(strStorage_DebugConfigPluginView);		

		RegisterModel(DebugUserPluginModel.STR_MVC_CLASS_NAME,
			strStorage_DebugUserPluginView);
		RegisterView(DebugUserPluginView.STR_MVC_CLASS_NAME,
			strStorage_DebugUserPluginView,
			GetModel(strStorage_DebugUserPluginView));
		myWSModel.RegisterWSSubModel(strStorage_DebugUserPluginView);		
		
		RegisterModel(DebugPortletPluginModel.STR_MVC_CLASS_NAME,
			strStorage_DebugPortletPluginView);
		RegisterView(DebugPortletPluginView.STR_MVC_CLASS_NAME,
			strStorage_DebugPortletPluginView,
			GetModel(strStorage_DebugPortletPluginView));
		myWSModel.RegisterWSSubModel(strStorage_DebugPortletPluginView);	
		
		RegisterModel(HelpURLPluginModel.STR_MVC_CLASS_NAME,
			strStorage_HelpURLPluginView);
		RegisterView(HelpURLPluginView.STR_MVC_CLASS_NAME,
			strStorage_HelpURLPluginView,
			GetModel(strStorage_HelpURLPluginView));
		myWSModel.RegisterWSSubModel(strStorage_HelpURLPluginView);

		RegisterModel(IntrinsicHelpSpacePluginModel.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicHelpSpacePluginView);
		RegisterView(IntrinsicHelpSpacePluginView.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicHelpSpacePluginView,
			GetModel(strStorage_IntrinsicHelpSpacePluginView));
		myWSModel.RegisterWSSubModel(strStorage_IntrinsicHelpSpacePluginView);

		RegisterModel(HTTPSettingsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_HTTPSettingsPluginView);
		RegisterView(HTTPSettingsPluginView.STR_MVC_CLASS_NAME,
			strStorage_HTTPSettingsPluginView,
			GetModel(strStorage_HTTPSettingsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_HTTPSettingsPluginView);

		RegisterModel(PWSSettingsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_PWSSettingsPluginView);
		RegisterView(PWSSettingsPluginView.STR_MVC_CLASS_NAME,
			strStorage_PWSSettingsPluginView,
			GetModel(strStorage_PWSSettingsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_PWSSettingsPluginView);

		RegisterModel(RemoteConfigPluginModel.STR_MVC_CLASS_NAME,
			strStorage_RemoteConfigPluginView);
		RegisterView(RemoteConfigPluginView.STR_MVC_CLASS_NAME,
			strStorage_RemoteConfigPluginView,
			GetModel(strStorage_RemoteConfigPluginView));
		myWSModel.RegisterWSSubModel(strStorage_RemoteConfigPluginView);

		RegisterModel(RemoteMigrationPluginModel.STR_MVC_CLASS_NAME,
			strStorage_RemoteMigrationPluginView);
		RegisterView(RemoteMigrationPluginView.STR_MVC_CLASS_NAME,
			strStorage_RemoteMigrationPluginView,
			GetModel(strStorage_RemoteMigrationPluginView));
		myWSModel.RegisterWSSubModel(strStorage_RemoteMigrationPluginView);

		RegisterModel(RemoteServerPluginModel.STR_MVC_CLASS_NAME,
			strStorage_RemoteServerPluginView);
		RegisterView(RemoteServerPluginView.STR_MVC_CLASS_NAME,
			strStorage_RemoteServerPluginView,
			GetModel(strStorage_RemoteServerPluginView));
		myWSModel.RegisterWSSubModel(strStorage_RemoteServerPluginView);

		RegisterModel(SelectDevicesPluginModel.STR_MVC_CLASS_NAME,
			strStorage_SelectDevicesPluginView);
		RegisterView(SelectDevicesPluginView.STR_MVC_CLASS_NAME,
			strStorage_SelectDevicesPluginView,
			GetModel(strStorage_SelectDevicesPluginView));
		myWSModel.RegisterWSSubModel(strStorage_SelectDevicesPluginView);

		RegisterModel(SOAPEncodingPluginModel.STR_MVC_CLASS_NAME,
			strStorage_SOAPEncodingPluginView);
		RegisterView(SOAPEncodingPluginView.STR_MVC_CLASS_NAME,
			strStorage_SOAPEncodingPluginView,
			GetModel(strStorage_SOAPEncodingPluginView));
		myWSModel.RegisterWSSubModel(strStorage_SOAPEncodingPluginView);


		RegisterModel(SummaryURLPluginModel.STR_MVC_CLASS_NAME,
			strStorage_SummaryURLPluginView);
		RegisterView(SummaryURLPluginView.STR_MVC_CLASS_NAME,
			strStorage_SummaryURLPluginView,
			GetModel(strStorage_SummaryURLPluginView));
		myWSModel.RegisterWSSubModel(strStorage_SummaryURLPluginView);

		RegisterModel(SWSSettingsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_SWSSettingsPluginView);
		RegisterView(SWSSettingsPluginView.STR_MVC_CLASS_NAME,
			strStorage_SWSSettingsPluginView,
			GetModel(strStorage_SWSSettingsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_SWSSettingsPluginView);

		RegisterModel(UserInfoPluginModel.STR_MVC_CLASS_NAME,
			strStorage_UserInfoPluginView);
		RegisterView(UserInfoPluginView.STR_MVC_CLASS_NAME,
			strStorage_UserInfoPluginView,
			GetModel(strStorage_UserInfoPluginView));
		myWSModel.RegisterWSSubModel(strStorage_UserInfoPluginView);

		RegisterModel(PrefsListPluginModel.STR_MVC_CLASS_NAME,
			strStorage_UserPrefsListPluginView);
		RegisterView(PrefsListPluginView.STR_MVC_CLASS_NAME,
			strStorage_UserPrefsListPluginView,
			GetModel(strStorage_UserPrefsListPluginView));
		RegisterControl(ExpListControl.STR_MVC_CLASS_NAME,
			strStorage_UserPrefsListPluginView,
			GetModel(strStorage_UserPrefsListPluginView));
		((PrefsListPluginModel) GetModel(strStorage_UserPrefsListPluginView)).SetPrefMode(PrefsListPluginModel.PLPM_PREFMODE_USERPREFS);
		myWSModel.RegisterWSSubModel(strStorage_UserPrefsListPluginView);

		RegisterModel(UserPrefsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_UserPrefsPluginView);
		RegisterView(UserPrefsPluginView.STR_MVC_CLASS_NAME,
			strStorage_UserPrefsPluginView,
			GetModel(strStorage_UserPrefsPluginView));
		((UserPrefsPluginModel) GetModel(strStorage_UserPrefsPluginView)).SetPrefMode(UserPrefsPluginModel.UPPM_PREFMODE_USERPREFS);
		myWSModel.RegisterWSSubModel(strStorage_UserPrefsPluginView);

		RegisterModel(IntrinsicUserPrefsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicUserPrefsPluginView);
		RegisterView(IntrinsicUserPrefsPluginView.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicUserPrefsPluginView,
			GetModel(strStorage_IntrinsicUserPrefsPluginView));
		((IntrinsicUserPrefsPluginModel) GetModel(strStorage_IntrinsicUserPrefsPluginView)).SetPrefMode(IntrinsicUserPrefsPluginModel.UPPM_PREFMODE_USERPREFS);
		myWSModel.RegisterWSSubModel(strStorage_IntrinsicUserPrefsPluginView);

		RegisterModel(XUIURLPluginModel.STR_MVC_CLASS_NAME,
			strStorage_XUIURLPluginView);
		RegisterView(XUIURLPluginView.STR_MVC_CLASS_NAME,
			strStorage_XUIURLPluginView,
			GetModel(strStorage_XUIURLPluginView));
		myWSModel.RegisterWSSubModel(strStorage_XUIURLPluginView);

		RegisterModel(IntrinsicPortletWSSettingsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicPortletWSSettingsPluginView);
		RegisterView(IntrinsicPortletWSSettingsPluginView.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicPortletWSSettingsPluginView,
			GetModel(strStorage_IntrinsicPortletWSSettingsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_IntrinsicPortletWSSettingsPluginView);

		RegisterModel(IntrinsicProfilePluginModel.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicProfilePluginView);
		RegisterView(IntrinsicProfilePluginView.STR_MVC_CLASS_NAME,
			strStorage_IntrinsicProfilePluginView,
			GetModel(strStorage_IntrinsicProfilePluginView));
		myWSModel.RegisterWSSubModel(strStorage_IntrinsicProfilePluginView);

		RegisterModel(MWSSettingsPluginModel.STR_MVC_CLASS_NAME,
			strStorage_MWSSettingsPluginView);
		RegisterView(MWSSettingsPluginView.STR_MVC_CLASS_NAME,
			strStorage_MWSSettingsPluginView,
			GetModel(strStorage_MWSSettingsPluginView));
		myWSModel.RegisterWSSubModel(strStorage_MWSSettingsPluginView);

		// Register DisplayPages
		// AGC- Note - this must occur AFTER the initialization of the WSSubModels.
		RegisterPage(new MainDP());
		myWSModel.RegisterWSPage(MainDP.STR_MVC_CLASS_NAME);

		RegisterPage(new HTTPConfigDP());
		myWSModel.RegisterWSPage(HTTPConfigDP.STR_MVC_CLASS_NAME);

		RegisterPage(new AdvSettingsDP());
		myWSModel.RegisterWSPage(AdvSettingsDP.STR_MVC_CLASS_NAME);

		RegisterPage(new AdvURLSettingsDP());
		myWSModel.RegisterWSPage(AdvURLSettingsDP.STR_MVC_CLASS_NAME);

		RegisterPage(new AdvSpaceSettingsDP());
		myWSModel.RegisterWSPage(AdvSpaceSettingsDP.STR_MVC_CLASS_NAME);

		RegisterPage(new UserInfoDP());
		myWSModel.RegisterWSPage(UserInfoDP.STR_MVC_CLASS_NAME);

		RegisterPage(new PrefsDP());
		myWSModel.RegisterWSPage(PrefsDP.STR_MVC_CLASS_NAME);

		RegisterPage(new DebugDP());
		myWSModel.RegisterWSPage(DebugDP.STR_MVC_CLASS_NAME);

		RegisterPage(new DevicesDP());
		myWSModel.RegisterWSPage(DevicesDP.STR_MVC_CLASS_NAME);
		
		RegisterPage(new AssocObjectsDP());
		myWSModel.RegisterWSPage(AssocObjectsDP.STR_MVC_CLASS_NAME);

		RegisterPage(new AuthenticationDP());
		myWSModel.RegisterWSPage(AuthenticationDP.STR_MVC_CLASS_NAME);
		
		// Default page
		SetCurrentPage(MainDP.STR_MVC_CLASS_NAME);

		//Sets left nav menu
		// Sub menu
		int nMenuOneID = m_mEditorMenu.StartNewSubMenu(GetString(1194,
					"ptmsgs_portaladminmsgs"));
		m_mEditorMenu.AddLinkToSubMenu(nMenuOneID, MainDP.STR_MVC_CLASS_NAME);
		m_mEditorMenu.AddLinkToSubMenu(nMenuOneID,
			HTTPConfigDP.STR_MVC_CLASS_NAME);
		m_mEditorMenu.AddLinkToSubMenu(nMenuOneID,
			AdvURLSettingsDP.STR_MVC_CLASS_NAME);
		m_mEditorMenu.AddLinkToSubMenu(nMenuOneID,
			AdvSpaceSettingsDP.STR_MVC_CLASS_NAME);
		m_mEditorMenu.AddLinkToSubMenu(nMenuOneID,
			AdvSettingsDP.STR_MVC_CLASS_NAME);
		m_mEditorMenu.AddLinkToSubMenu(nMenuOneID,
				AuthenticationDP.STR_MVC_CLASS_NAME);
		m_mEditorMenu.AddLinkToSubMenu(nMenuOneID, PrefsDP.STR_MVC_CLASS_NAME);
		m_mEditorMenu.AddLinkToSubMenu(nMenuOneID, UserInfoDP.STR_MVC_CLASS_NAME);
		m_mEditorMenu.AddLinkToSubMenu(nMenuOneID, DebugDP.STR_MVC_CLASS_NAME);
		m_mEditorMenu.AddLinkToSubMenu(nMenuOneID, DevicesDP.STR_MVC_CLASS_NAME);
		m_mEditorMenu.AddLinkToSubMenu(nMenuOneID,
			AssocObjectsDP.STR_MVC_CLASS_NAME);
					
		// BEGIN Sub menu "Standard Settings"

		int nMenuTwoID = m_mEditorMenu.StartNewSubMenu(GetString(1940, "ptmsgs_portaladminmsgs"));

		//Properties and Names
		m_mEditorMenu.AddLinkToSubMenu(nMenuTwoID, LocNamesDP.STR_MVC_CLASS_NAME);
		//Security
		m_mEditorMenu.AddLinkToSubMenu(nMenuTwoID, ObjectSecurityDP.STR_MVC_CLASS_NAME);
		//Migration History
		m_mEditorMenu.AddLinkToSubMenu(nMenuTwoID, MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);

		// END Sub menu "Standard Settings"

	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.AActivitySpace#SetCurrentPage()
	 *
	 * @param IDisplayPage the current Display Page
	 */
	public void SetCurrentPage(IDisplayPage page)

	{
		super.SetCurrentPage( page );
		
		// Get the PluginDP's to find their help topic.
		// Note the cast will fail (correctly), if the page is
		// not a Plugin Page DP.
		try
		{
			WSPluginDP wspluginPage = (WSPluginDP)page;
			String strTopic = wspluginPage.GetPluginHelpTopic();
			if ((null != strTopic) && (strTopic.length() > 0))
			{
				SetHelpTopic(strTopic); 
			}						
		} catch (Exception e)
		{
			//AGC- This is not a bug. Non Plugin
			
				log.Debug(e, "Couldn't get help from PluginPage - this page is likely not a WSPluginDP.");
						
		}
	}

}

