package com.plumtree.portalpages.admin.editors.webservice;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.editors.webservice.plugins.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.migrationhistory.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;

/**
 * @author adolfoc
 */
public class WebServiceModel extends ObjEditorModel
	implements IWebServiceModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.webservice.WebServiceModel");

	/** Object Name */
	public static final String STR_MVC_CLASS_NAME = "WebService";

	/**
	 * The additional display modes this WebService Editor can be in. This mostly depends
	 * on the Web Service Type.
	 */
	public static final int WSMODE_NONE				 = 0; //The WSType hasn't been chosen yet. // In the current UI this is not used, but it could be revived in the future.
	public static final int WSMODE_PORTLET			 = 2; //Portlet Mode
	public static final int WSMODE_INTRINSIC_PORTLET = 3; // Intrinsic Portlet Mode
	public static final int WSMODE_AWS = 4; //AuthSourceWS Mode
	public static final int WSMODE_CWS = 5; //CrawlerWS Mode
	public static final int WSMODE_SWS = 6; //SearchWS Mode 
	public static final int WSMODE_PWS = 7; //ProfileWS Mode
	public static final int WSMODE_MWS = 8; //MigrationWS Mode
	
	/**
	 * The remote server ID passed into this editor. - Used in create mode to
	 * pre-assign us a Remote server object.
	 */
	private int m_intRemoteServerIDPassedIn = 0;	
	
	/**
	 * This member maintains a registry of page names that extend
	 * WSPluginDP. We must remember to enable and disable these pages
	 * when the appropriate updates occur.
	 */
	XPArrayList m_xalWSPluginDPs;

	/** Logging object and booleans */
	/**
	 * This member maintains a registry of Plugin model keys that support
	 * the IWSSubModel interface. We must remember to exercise the IWSSubModel
	 * interface on them when the appropriate updates occur.
	 */
	XPArrayList m_xalWSSubModels;

	/**
	 * This member defines the current mode of the WSEditor (as per the above enumeration);
	 * When this member is changed, we must send an update to all of our pages and views.
	 */
	private int m_nWSMode;

	public WebServiceModel()
	{
		super();
		m_xalWSPluginDPs = new XPArrayList();
		m_xalWSSubModels = new XPArrayList();
		m_nWSMode = WSMODE_NONE;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new WebServiceModel();
	}

	// **************************************** Persistence ******************************************

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskBeforeStore()
	 */
	public void DoTaskBeforeStore()
	{
		super.DoTaskBeforeStore();

		// Make sure that the provider info property bag set to the correct format.
		IPTWebService ptWebService    = (IPTWebService) m_ptObject;
		IXPPropertyBag ptProviderInfo = (IXPPropertyBag) ptWebService.GetProviderInfo();

		switch (GetWSMode())
		{
		case WSMODE_INTRINSIC_PORTLET:
			WSModelHelper.PBWrite(ptProviderInfo,
				AExtensibilityConstants.PT_PROPBAG_FORMAT,
				AExtensibilityConstants.PT_PROPBAG_FORMAT_INTRINSICGADGET_INIT);

		default:
			WSModelHelper.PBWrite(ptProviderInfo,
				AExtensibilityConstants.PT_PROPBAG_FORMAT,
				AExtensibilityConstants.PT_PROPBAG_FORMAT_HTTPGADGET_INIT);
		}

		// All of our sub models store their own data.... yippee!
		PreStoreWSSubModels();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskOnStartEditor(_htFormData)
	 */
	public void DoTaskOnStartEditor(XPHashtable _htFormData)
	{
		super.DoTaskOnStartEditor(_htFormData);

		try
		{
			String strWSType = WSModelHelper.WSTYPE_HTTPGP;

			// In create mode, we set our editor mode based on the 
			// activitiy space name that created us.
			if (EditorStartControl.EDITOR_START_FLAG_CREATE == GetEditorType())
			{
				//AGC- Let's choose our mode based on the name of our ActivitySpace.
				if (0 == AuthWSWebServiceEditorAS.STR_MVC_CLASS_NAME.compareTo(
							m_asOwner.GetName()))
				{
					m_nWSMode     = WSMODE_AWS;
					strWSType     = WSModelHelper.WSTYPE_AWS;
				}
				else if (0 == CrawlWSWebServiceEditorAS.STR_MVC_CLASS_NAME.compareTo(
							m_asOwner.GetName()))
				{
					m_nWSMode     = WSMODE_CWS;
					strWSType     = WSModelHelper.WSTYPE_CWS;
				}
				else if (0 == IntrPortWSWebServiceEditorAS.STR_MVC_CLASS_NAME.compareTo(
							m_asOwner.GetName()))
				{
					m_nWSMode     = WSMODE_INTRINSIC_PORTLET;
					strWSType     = WSModelHelper.WSTYPE_IGP;
				}
				else if (0 == ProfileWSWebServiceEditorAS.STR_MVC_CLASS_NAME.compareTo(
							m_asOwner.GetName()))
				{
					m_nWSMode     = WSMODE_PWS;
					strWSType     = WSModelHelper.WSTYPE_PWS;
				}
				else if (0 == RemPortWSWebServiceEditorAS.STR_MVC_CLASS_NAME.compareTo(
							m_asOwner.GetName()))
				{
					m_nWSMode     = WSMODE_PORTLET;
					strWSType     = WSModelHelper.WSTYPE_HTTPGP;
				}
				else if (0 == SearchWSWebServiceEditorAS.STR_MVC_CLASS_NAME.compareTo(
							m_asOwner.GetName()))
				{
					m_nWSMode     = WSMODE_SWS;
					strWSType     = WSModelHelper.WSTYPE_SWS;
				}
				else if (0 == MWSWebServiceEditorAS.STR_MVC_CLASS_NAME.compareTo(
							m_asOwner.GetName()))
				{
					m_nWSMode     = WSMODE_MWS;
					strWSType     = WSModelHelper.WSTYPE_MWS;
				}

				IPTWebService ptWebService = (IPTWebService) m_ptObject;
				ptWebService.SetProviderCLSID(strWSType);
				
				if (m_intRemoteServerIDPassedIn > 0)
				{
					//AGC- Attempt to set the Remote Server ID that was passed to us.
					ptWebService.SetRemoteServerID(m_intRemoteServerIDPassedIn);
				}
			}
			else
			{
				IPTWebService ptWebService = (IPTWebService) m_ptObject;
				String strProviderID	   = ptWebService.GetProviderCLSID();

				if (0 == strProviderID.compareTo(WSModelHelper.WSTYPE_HTTPGP))
				{
					m_nWSMode = WebServiceModel.WSMODE_PORTLET;
				}
				else if (0 == strProviderID.compareTo(WSModelHelper.WSTYPE_IGP))
				{
					m_nWSMode = WebServiceModel.WSMODE_INTRINSIC_PORTLET;
				}
				else if (0 == strProviderID.compareTo(WSModelHelper.WSTYPE_AWS))
				{
					m_nWSMode = WebServiceModel.WSMODE_AWS;
				}
				else if (0 == strProviderID.compareTo(WSModelHelper.WSTYPE_PWS))
				{
					m_nWSMode = WebServiceModel.WSMODE_PWS;
				}
				else if (0 == strProviderID.compareTo(WSModelHelper.WSTYPE_CWS))
				{
					m_nWSMode = WebServiceModel.WSMODE_CWS;
				}
				else if (0 == strProviderID.compareTo(WSModelHelper.WSTYPE_SWS))
				{
					m_nWSMode = WebServiceModel.WSMODE_SWS;
				}
				else if (0 == strProviderID.compareTo(WSModelHelper.WSTYPE_MWS))
				{
					m_nWSMode = WebServiceModel.WSMODE_MWS;
				}
			}

			if (EditorStartControl.EDITOR_START_FLAG_EDIT != GetEditorType())
			{
				((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
					MigrationHistoryPluginDP.STR_MVC_CLASS_NAME);
			}

			UpdatePages();
			InitWSSubModels();
			UpdateWSSubModels();
		}
		catch (Exception e)
		{
			
				log.Error(e, "WebServiceModel::DoTaskOnStartEditor - Unable to initialize editor from server: " +
					e.getMessage());
			
		}
	}

	// ************************************ Basic Model Settings *************************************

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the nWSMode.
	 * @return int
	 */
	public int GetWSMode()
	{
		return m_nWSMode;
	}
	
	public String GetWSObjectTitle() {
		String s=null;
		switch(GetWSMode()) {
		case WSMODE_PORTLET: s=m_asOwner.GetString(2375,"ptmsgs_portaladminmsgs"); break; // Remote Portlet
		case WSMODE_INTRINSIC_PORTLET: s=m_asOwner.GetString(2371,"ptmsgs_portaladminmsgs"); break; // Intrinsic Portlet
		case WSMODE_AWS: s=m_asOwner.GetString(2374,"ptmsgs_portaladminmsgs"); break; // Authentication
		case WSMODE_CWS: s=m_asOwner.GetString(2376,"ptmsgs_portaladminmsgs"); break; // Crawler
		case WSMODE_SWS: s=m_asOwner.GetString(2372,"ptmsgs_portaladminmsgs"); break; // Search
		case WSMODE_PWS: s=m_asOwner.GetString(2373,"ptmsgs_portaladminmsgs"); break; // Profile
		case WSMODE_MWS: s=m_asOwner.GetString(0, "ptmsgs_provUIMWS"); break; // Migration
		default: s=m_asOwner.GetString(1544,"ptmsgs_portaladminmsgs"); break; // Unknown
		}
		return s;
	}
	
	public String GetWSObjectMiniHelp() {
			String s=null;
			switch(GetWSMode()) {
			case WSMODE_PORTLET: s=m_asOwner.GetString(2382,"ptmsgs_portaladminmsgs"); break; // Specify the Remote Server that will host this Web Service.
			case WSMODE_INTRINSIC_PORTLET: s=m_asOwner.GetString(2378,"ptmsgs_portaladminmsgs"); break; // Specify the Portlet Class Identifier for this Web Service.
			case WSMODE_AWS: s=m_asOwner.GetString(2381,"ptmsgs_portaladminmsgs"); break; // Specify the Remote Server that will host this Web Service and any additional settings required.
			case WSMODE_CWS: s=m_asOwner.GetString(2383,"ptmsgs_portaladminmsgs"); break; // Specify the Container and Document URL settings.
			case WSMODE_SWS: s=m_asOwner.GetString(2379,"ptmsgs_portaladminmsgs"); break; // Specify the Remote Server that will host this Web Service.
			case WSMODE_PWS: s=m_asOwner.GetString(2380,"ptmsgs_portaladminmsgs"); break; // Specify the Remote Server that will host this Web Service.
			case WSMODE_MWS: s=m_asOwner.GetString(2381,"ptmsgs_portaladminmsgs"); break; // Specify the Remote Server that will host this Web Service.
			}
			return s;
		}

	// ************************************ Getters and Setters *************************************

	/**
	 * Returns the arraylist of views that are registered IWSSubModels...
	 */
	public XPArrayList GetWSViews()
	{
		return m_xalWSSubModels;
	}

	// ************************************ IWSSubModel methods ***************************************

	/**
	 * This method is used to register a new WSPluginDP with this WebService main model. This is done
	 * so that plugin pages can be hidden and shown automatically depending on the WSMODE.
	 */
	public void RegisterWSPage(String strDPName)
	{
		try
		{
			IDisplayPage myIDP = m_asOwner.GetPage(strDPName);
			WSPluginDP myWSPDP = null;

			try
			{
				myWSPDP = (WSPluginDP) myIDP;
				myWSPDP.InitPluginDP();
			}
			catch (Exception e)
			{
				
					log.Error(e, "WebServiceModel::RegisterWSPage(\"" + strDPName +
						"\") - Attempted to register page that doesn't extend WSPluginDP. Error:" +
						e.getMessage());
				
			}

			if (null == myWSPDP)
			{
				log.Error("WebServiceModel::RegisterWSPage(\"" + strDPName +
					"\") - Attempted to register null WSPluginDP. Error:");
			}
			else
			{
				m_xalWSPluginDPs.Add(strDPName);
			}
		}
		catch (Exception e)
		{
			
				log.Error("WebServiceModel::RegisterWSPage(\"" + strDPName +
					"\") - Could not register page. Error:" + e.getMessage());
			
		}
	}

	/**
	 * This method is used to register a new submodel with this WebService main model. This is done
	 * so that submodels can have access to the Remote Server base URL which is necessary for them
	 * to display. All submodels that are registered must support the IWSSubModel interface.
	 */
	public void RegisterWSSubModel(String strPluginModelStorageName)
	{
		try
		{
			IModel imSubModel	   = m_asOwner.GetModel(strPluginModelStorageName);
			IWSSubModel wsSubModel = null;

			try
			{
				wsSubModel = (IWSSubModel) imSubModel;
			}
			catch (Exception e)
			{
				
					log.Error(e, "WebServiceModel::RegisterWSSubModel(\"" +
						strPluginModelStorageName +
						"\") - Attempted to register model that doesn't support IWSSubModel. Error:" +
						e.getMessage());
				
			}

			if (null == wsSubModel)
			{
				log.Error("WebServiceModel::RegisterWSSubModel(\"" +
					strPluginModelStorageName +
					"\") - Attempted to register null IWSSubModel. Error:");
			}
			else
			{
				m_xalWSSubModels.Add(strPluginModelStorageName);
			}
		}
		catch (Exception e)
		{
			
				log.Error("WebServiceModel::RegisterWSSubModel(\"" +
					strPluginModelStorageName +
					"\") - Could not register model. Error:" + e.getMessage());
			
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String,
	 *      XPHashtable)
	 *
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		int nPageStatus = super.SavePage(_sPageName, _htFormData);

		//MES- CR: If the status is now invalid, can we return without going through the models?
		//AGC- CR-Response: Note, we still need to update the local variables with any changed information.
		//        It would be very strange if users found that all of the settings they changed on a page
		//        revert to their original values if any single setting is incorrect.    	
		IDisplayPage myDP = m_asOwner.GetPage(_sPageName);
		WSPluginDP myWSDP = null;

		try
		{
			//AGC - try to cast to WSPluginDP
			myWSDP = (WSPluginDP) myDP;
		}
		catch (Exception e)
		{
			// Didn't work. It's someone else's page. Just return.
			return nPageStatus;
		}

		if (null == myWSDP) // Just double checking.
		{
			return nPageStatus;
		}

		//AGC- CR-Response: Note, we assume that the views and their associated Models have the same storage key
		//     (this is a necessary part of the automated view-model framework). If the following code
		//     looks strange (using views to enumerate models) that's why. 
		XPArrayList xpalWSViews		    = myWSDP.GetOrderedViewList(GetWSMode());
		IXPEnumerator xpEnumWSSubModels = xpalWSViews.GetEnumerator();

		while (xpEnumWSSubModels.MoveNext())
		{
			String strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();

			try
			{
				IWSSubModel wsSubModelCurrent = (IWSSubModel) m_asOwner.GetModel(strSubModelKeyCurrent);

				int nLocalStatus = ((EditorHelperModel) wsSubModelCurrent).SaveSection(_htFormData);

				if (EditorRepostControl.PAGE_STATUS_VALID != nLocalStatus)
				{
					nPageStatus = nLocalStatus;
				}
			}
			catch (Exception e)
			{
				
					log.Error(GetName() + "::SavePage(" + _sPageName +
						") The following error occured when trying to Save the Section(" +
						strSubModelKeyCurrent + "): " + e.getMessage());
				
			}
		}

		return nPageStatus;
	}

	/**
	 * Sets the nWSMode.
	 * @param nWSMode The nWSMode to set
	 */
	public void SetWSMode(int nWSMode)
	{
		m_nWSMode = nWSMode;

		// AGC- Don't forget to hide/show all of our pages based on the new information...
		UpdatePages();

		// AGC- The views should be told as well... just in case they care...
		UpdateWSSubModels();
	}

	public void UpdateWSSubModels()
	{
		try
		{
			IXPEnumerator xpEnumWSSubModels = m_xalWSSubModels.GetEnumerator();

			while (xpEnumWSSubModels.MoveNext())
			{
				String strSubModelKeyCurrent = "";

				try
				{
					strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();

					IWSSubModel wsSubModelCurrent = (IWSSubModel) m_asOwner.GetModel(strSubModelKeyCurrent);
					wsSubModelCurrent.DoTaskOnRemoteServerUpdate(XPConvert.ToString(
							GetRemoteServerObjectID()), GetRemoteServerName(),
						GetRemoteServerBaseURL());
					wsSubModelCurrent.DoTaskOnWSModeUpdate(GetWSMode());
				}
				catch (Exception e)
				{
					
						log.Error("WebServiceModel::UpdateWSSubModels() Unknown Error updating WSSubModel (" +
							strSubModelKeyCurrent + ")." + e.getMessage());
					
				}
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "WebServiceModel::UpdateWSSubModels() Unknown Error." +
					e.getMessage());
			
		}
	}

	private String GetRemoteServerBaseURL()
	{
		return ((RemoteServerPluginModel) m_asOwner.GetModel(WebServiceEditorAS.strStorage_RemoteServerPluginView)).GetRemoteServerBaseURL();
	}

	private String GetRemoteServerName()
	{
		return ((RemoteServerPluginModel) m_asOwner.GetModel(WebServiceEditorAS.strStorage_RemoteServerPluginView)).GetRemoteServerName();
	}

	private long GetRemoteServerObjectID()
	{
		return ((RemoteServerPluginModel) m_asOwner.GetModel(WebServiceEditorAS.strStorage_RemoteServerPluginView)).GetRemoteServerObjectID();
	}

	private void InitWSSubModels()
	{
		try
		{
			IXPEnumerator xpEnumWSSubModels = m_xalWSSubModels.GetEnumerator();

			while (xpEnumWSSubModels.MoveNext())
			{
				String strSubModelKeyCurrent = "";

				try
				{
					strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();

					IWSSubModel wsSubModelCurrent = (IWSSubModel) m_asOwner.GetModel(strSubModelKeyCurrent);
					wsSubModelCurrent.InitWSModel((IPTWebService) m_ptObject);
				}
				catch (Exception e)
				{
					
						log.Error("WebServiceModel::InitWSSubModels() Unknown Error initializing WSSubModel (" +
							strSubModelKeyCurrent + ")." + e.getMessage());
					
				}
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "WebServiceModel::InitWSSubModels() Unknown Error." +
					e.getMessage());
			
		}
	}

	private void PreStoreWSSubModels()
	{
		try
		{
			// AGC- Only store those views that are active in our current mode!
			IXPEnumerator xpEnumWSPluginDPs = m_xalWSPluginDPs.GetEnumerator();

			// Walk through all of the pages
			while (xpEnumWSPluginDPs.MoveNext())
			{
				WSPluginDP wsPluginDPCurrent    = (WSPluginDP) m_asOwner.GetPage((String) xpEnumWSPluginDPs.GetCurrent());
				XPArrayList xpalModelsCurrent   = wsPluginDPCurrent.GetOrderedViewList(m_nWSMode);
				IXPEnumerator xpEnumWSSubModels = xpalModelsCurrent.GetEnumerator();

				// Walk through each view on each page
				while (xpEnumWSSubModels.MoveNext())
				{
					String strSubModelKeyCurrent = "";

					try
					{
						strSubModelKeyCurrent = (String) xpEnumWSSubModels.GetCurrent();

						IWSSubModel wsSubModelCurrent = (IWSSubModel) m_asOwner.GetModel(strSubModelKeyCurrent);

						// Tell the buggers to prepare for the end times
						wsSubModelCurrent.DoTaskBeforeWSStore();
					}
					catch (Exception e)
					{
						
							log.Error("WebServiceModel::PreStoreWSSubModels() Unknown Error updating WSSubModel (" +
								strSubModelKeyCurrent + ")." + e.getMessage());
						
					}
				}
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "WebServiceModel::PreStoreWSSubModels() Unknown Error." +
					e.getMessage());
			
		}
	}

	private void UpdatePages()
	{
		// Automatically hide empty pages and show pages with information
		try
		{
			IXPEnumerator xpEnumWSPluginDPs = m_xalWSPluginDPs.GetEnumerator();

			while (xpEnumWSPluginDPs.MoveNext())
			{
				try
				{
					String strDPNameCurrent		 = (String) xpEnumWSPluginDPs.GetCurrent();
					WSPluginDP wsPluginDPCurrent = (WSPluginDP) m_asOwner.GetPage(strDPNameCurrent);
					XPArrayList xpalViews		 = wsPluginDPCurrent.GetOrderedViewList(m_nWSMode);
					int nNumViews				 = xpalViews.GetSize();

					if (0 >= nNumViews)
					{
						((EditorAS) m_asOwner).GetLeftMenu().HideLink(true,
							strDPNameCurrent);
					}
					else
					{
						((EditorAS) m_asOwner).GetLeftMenu().HideLink(false,
							strDPNameCurrent);
					}
				}
				catch (Exception e)
				{
					
						log.Error(e, "WebServiceModel::UpdateWSSubModels() Unknown Error updating WSSubModel." +
							e.getMessage());
					
				}
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "WebServiceModel::UpdateWSSubModels() Unknown Error." +
					e.getMessage());
			
		}
	}
	
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.editor.ObjEditorModel#SetMemberVariablesFromQS(com.plumtree.openfoundation.util.XPHashtable)
	 */
	public void SetMemberVariablesFromQS(XPHashtable _htFormData) {
		super.SetMemberVariablesFromQS(_htFormData);
		
		m_intRemoteServerIDPassedIn = WSModelHelper.GetFormElementInt(_htFormData, PTWebServices.QS_WS_REMOTE_SERVER, m_intRemoteServerIDPassedIn);
	}

}

