/**
 * Created on Oct 30, 2002
 *
 * To change this generated comment edit the template variable "filecomment":

 * To enable and disable the creation of file comments go to

 */
package com.plumtree.portalpages.admin.editors.webservice.plugins;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.admin.editors.webservice.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;

/**
 * @author adolfoc
 *
 */
public class AdvSettingsPluginModel extends EditorHelperModel
	implements IAdvSettingsPluginModelRO, IWSSubModel
{
	
	public static final String STR_MVC_CLASS_NAME = "AdvSettingsPlugin";

	/**
	 * The Web Service object being edited by our parent Web Service editor
	 */
	IPTWebService m_ptWebService = null;

	/**
	 * Whether or not this CWS (CWS only) supports file path submission.
	 */
	private boolean m_bCardSubmitFilePath = false;

	/**
	 * Whether or not this CWS (CWS only) supports Remote UI submission.
	 */
	private boolean m_bCardSubmitRemoteUI = false;

	/**
	 * Whether or not this CWS (CWS only) supports Remote UI submission.
	 */
	private boolean m_bSupportsDocUpload = false;
	
	/**
	 * Whether or not to send the Community ACL to the portlet.
	 */
	private boolean m_bSendCommunityACL = false;

	/**
	 * Whether or not to send the Login Token to the portlet.
	 */
	private boolean m_bSendLoginToken = false;

	/**
	 * Whether or not to send the PageID to the portlet.
	 */
	private boolean m_bSendPageID = false;

	/**
	 * Whether or not to send the Timezone to the portlet.
	 */
	private boolean m_bSendTimezone = true;


	/**
	 * Whether or not to send the Locale to the portlet.
	 */
	private boolean m_bSendLocale = true;

	/**
	 * Whether or not this web service supports mirroring.
	 */
	private boolean m_bSupportsMirroring = false;

	/**
	 * Whether or not this web service supports ACL Import.
	 */
	private boolean m_bSupportsACLImport = false;
	

	/**
	 * Token duration (in minutes) for the Login Token
	 */
	private int m_nTokenTimeout = 5; // A reasonable timeout I assume...

	/**
	 * Whether or not portlets created from this web service should show refresh button.
	 */
	private boolean m_bRefreshButton = false;	

	/**
	 * Whether or not portlets created from this web service should share sessions with other portlets
	 * on the same remote server.
	 */
	private boolean m_bSharedSession = false;	
	
	/**
	 * Whether or not to send subportalID to portlet.
	 */
	private boolean m_bSendSubportalID = false;
	
	/**
	 * Whether or not to send portlet aligntment to portlet.
	 */
	private boolean m_bSendAlignment = false;
	
	/**
	 * WSMode setting. We use this to determine which checkboxes to display
	 * and store.
	 */
	private int m_nWSMode;

	public AdvSettingsPluginModel()
	{
		super();
		m_nWSMode = WebServiceModel.WSMODE_NONE;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new AdvSettingsPluginModel();
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskBeforeWSStore()
	 */
	public void DoTaskBeforeWSStore()
	{
		IXPPropertyBag ptProviderInfo = (IXPPropertyBag) m_ptWebService.GetProviderInfo();

		ptProviderInfo.Write(AExtensibilityConstants.PTC_HTTPGADGET_LOGINTOKENVALID,
			m_nTokenTimeout);

		WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_HTTPGADGET_SENDPAGEID, m_bSendPageID);
		WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_HTTPGADGET_SENDLOGINTOKEN, m_bSendLoginToken);
		WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_HTTPGADGET_SENDCOMMUNITYACL, m_bSendCommunityACL);
		WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_HTTPGADGET_SENDTIMEZONE, m_bSendTimezone);
		WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_HTTPCWS_SUPPORTSACLIMPORT, m_bSupportsACLImport);
		WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_HTTPCWS_SUPPORTSMIRRORING, m_bSupportsMirroring);
		
		int nCardSubmitType = 0;

		if (m_bCardSubmitFilePath)
		{
			nCardSubmitType |= PT_WEBSERVICE_CARDSUBMIT_TYPES.PT_WEBSERVICE_CARDSUBMIT_SIMPLE;
		}

		if (m_bCardSubmitRemoteUI)
		{
			nCardSubmitType |= PT_WEBSERVICE_CARDSUBMIT_TYPES.PT_WEBSERVICE_CARDSUBMIT_ADVANCED;
		}

		if (m_bSupportsDocUpload)
		{
			nCardSubmitType |= PT_WEBSERVICE_CARDSUBMIT_TYPES.PT_WEBSERVICE_CARDSUBMIT_UPLOAD;
		}
		
		// AGC- add or remove the locale string from the list of User prefs. But ONLY IN SWS MODE
		// ONLY SWS MODE!!!
		WebServiceModel mParent = (WebServiceModel)m_asOwner.GetModel(WebServiceModel.STR_MVC_CLASS_NAME);
		if (WebServiceModel.WSMODE_SWS == mParent.GetWSMode())
		{ 
			XPArrayList xpalUserPrefs = null;
			try {
				xpalUserPrefs = WSModelHelper.ObjectArrayToXPArray(ptProviderInfo.ReadAsObject1DArray(AExtensibilityConstants.PT_PROPBAG_HTTPGADGET_USERPREFS));
			} catch (Exception e)
			{
				//AGC- This likely means that this has never been set in the propertybag.
				// That's fine.
			}
			
			if (null == xpalUserPrefs)
			{
				xpalUserPrefs = new XPArrayList();
			}
			
			if (!m_bSendLocale)
			{
				int nLocaleIndex = xpalUserPrefs.IndexOf(AExtensibilityConstants.PTC_STRLOCALE); 
				if ((nLocaleIndex >= 0) && (nLocaleIndex < xpalUserPrefs.GetSize()))
				{
					xpalUserPrefs.RemoveAt(nLocaleIndex);
				}
				
			} else
			{
				if (!xpalUserPrefs.Contains(AExtensibilityConstants.PTC_STRLOCALE))
				{
					xpalUserPrefs.Add(AExtensibilityConstants.PTC_STRLOCALE);
				}
			}
			
			WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PT_PROPBAG_HTTPGADGET_USERPREFS, xpalUserPrefs);
		}
		
		m_ptWebService.SetCardSubmitType(nCardSubmitType);
		
		// Set refresh button.
		int nSupportsPersonalization = m_ptWebService.GetSupportsPersonalization();

		if (m_bRefreshButton)
		{
			nSupportsPersonalization = nSupportsPersonalization |
				PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_REFRESH;
		}
		else
		{
			nSupportsPersonalization = nSupportsPersonalization &
				~PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_REFRESH;
		}

		m_ptWebService.SetSupportsPersonalization(nSupportsPersonalization);

		WSModelHelper.PBWrite(ptProviderInfo, PlumtreeExtensibility.PT_PROPBAG_HTTP_SESSION_TYPE, m_bSharedSession);
		
		WSModelHelper.PBWrite(ptProviderInfo, PlumtreeExtensibility.PT_PROPBAG_HTTPGADGET_SENDSUBPORTALID, m_bSendSubportalID);

		WSModelHelper.PBWrite(ptProviderInfo, PlumtreeExtensibility.PT_PROPBAG_HTTPGADGET_SENDALIGNMENT, m_bSendAlignment);
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnRemoteServerUpdate(String, String, String)
	 */
	public void DoTaskOnRemoteServerUpdate(String strRemoteServerID,
		String strRemoteServerName, String strRemoteServerBaseURL)
	{
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnWSModeUpdate(int)
	 */
	public void DoTaskOnWSModeUpdate(int nNewWSMode)
	{
		m_nWSMode = nNewWSMode;
	}

	/**
	 * Whether or not to support card submission via file path (CWS Only).
	 * @returns boolean
	 */
	public boolean GetCardSubmitFilePath()
	{
		return m_bCardSubmitFilePath;
	}

	/**
	 * Whether or not to support card submission via remote UI (CWS Only).
	 * @returns boolean
	 */
	public boolean GetCardSubmitRemoteUI()
	{
		return m_bCardSubmitRemoteUI;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Whether or not to send a community ACL to the portlet.
	 * @returns boolean
	 */
	public boolean GetSendCommunityACL()
	{
		return m_bSendCommunityACL;
	}

	/**
	 * Whether or not to send a login token to the portlet.
	 * @returns boolean
	 */
	public boolean GetSendLoginToken()
	{
		return m_bSendLoginToken;
	}

	/**
	 * Whether or not to send a pageid to the portlet.
	 * @returns boolean
	 */
	public boolean GetSendPageID()
	{
		return m_bSendPageID;
	}

	/**
	 * Whether or not to send a timezone to the portlet.
	 * @returns boolean
	 */
	public boolean GetSendTimezone()
	{
		return m_bSendTimezone;
	}


	/**
	 * Whether or not to send a locale to the portlet.
	 * @returns boolean
	 */
	public boolean GetSendLocale()
	{
		return m_bSendLocale;
	}

	/**
	 * Whether or not this web service supports folder mirroring
	 * @returns boolean
	 */
	public boolean GetSupportsMirroring()
	{
		return m_bSupportsMirroring;
	}


	/**
	 * Whether or not this web service supports Document Upload
	 * @returns boolean
	 */
	public boolean GetSupportsUpload()
	{
		return m_bSupportsDocUpload;
	}


	/**
	 * Whether or not this web service supports ACL Import
	 * @returns boolean
	 */
	public boolean GetSupportsACLImport()
	{
		return m_bSupportsACLImport;
	}

	/**
	 * The duration of the timeout token (in minutes)
	 * @returns int
	 */
	public int GetTokenTimeout()
	{
		return m_nTokenTimeout;
	}

	/**
	 * Whether or not portlets from this web service should show the refresh button
	 * @returns boolean
	 */
	public boolean GetRefreshButton()
	{
		return m_bRefreshButton;
	}
	
	/**
	 * Whether each portlet has its own HTTP session, or shares sessions with other portlets on the same remote server
	 * @return int
	 */
	public boolean GetSharedSession()
	{
		return m_bSharedSession;
	}
	
	/**
	 * Whether to send subportal id to portlets.
	 * @returns boolean
	 */
	public boolean GetSendSubportalID()
	{
		return m_bSendSubportalID;
	}
	
	/**
	 * Whether to send alignment to portlets.
	 * @returns boolean
	 */
	public boolean GetSendAlignment()
	{
		return m_bSendAlignment;
	}
	
	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#GetValidationFunction()
	 */
	public String GetValidationFunction()
	{
		return null;
	}

	/**
	 * Returns our current WS display mode from the WebServiceModel.WSMODE enumeration.
	 * @returns int
	 */
	public int GetWSMode()
	{
		return m_nWSMode;
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#InitWSModel()
	 */
	public void InitWSModel(IPTWebService ptWebService)
	{
		m_ptWebService = ptWebService;

		IXPPropertyBag ptProviderInfo = (IXPPropertyBag) m_ptWebService.GetProviderInfo();
		
		m_nTokenTimeout = WSModelHelper.PBReadInt(ptProviderInfo, AExtensibilityConstants.PTC_HTTPGADGET_LOGINTOKENVALID, m_nTokenTimeout);
		m_bSendPageID = WSModelHelper.PBReadBool(ptProviderInfo, AExtensibilityConstants.PTC_HTTPGADGET_SENDPAGEID, m_bSendPageID);

		m_bSendLoginToken = WSModelHelper.PBReadBool(ptProviderInfo, AExtensibilityConstants.PTC_HTTPGADGET_SENDLOGINTOKEN, m_bSendLoginToken);
		m_bSendCommunityACL = WSModelHelper.PBReadBool(ptProviderInfo, AExtensibilityConstants.PTC_HTTPGADGET_SENDCOMMUNITYACL, m_bSendCommunityACL);

		m_bSupportsACLImport = WSModelHelper.PBReadBool(ptProviderInfo, AExtensibilityConstants.PTC_HTTPCWS_SUPPORTSACLIMPORT, m_bSupportsACLImport);
		m_bSupportsMirroring = WSModelHelper.PBReadBool(ptProviderInfo, AExtensibilityConstants.PTC_HTTPCWS_SUPPORTSMIRRORING, m_bSupportsMirroring);


		int nCardSubmitType     = ptWebService.GetCardSubmitType();
		m_bCardSubmitFilePath   = (PT_WEBSERVICE_CARDSUBMIT_TYPES.PT_WEBSERVICE_CARDSUBMIT_SIMPLE == (PT_WEBSERVICE_CARDSUBMIT_TYPES.PT_WEBSERVICE_CARDSUBMIT_SIMPLE &
			nCardSubmitType));
		m_bCardSubmitRemoteUI   = (PT_WEBSERVICE_CARDSUBMIT_TYPES.PT_WEBSERVICE_CARDSUBMIT_ADVANCED == (PT_WEBSERVICE_CARDSUBMIT_TYPES.PT_WEBSERVICE_CARDSUBMIT_ADVANCED &
			nCardSubmitType));
		m_bSupportsDocUpload   = (PT_WEBSERVICE_CARDSUBMIT_TYPES.PT_WEBSERVICE_CARDSUBMIT_UPLOAD == (PT_WEBSERVICE_CARDSUBMIT_TYPES.PT_WEBSERVICE_CARDSUBMIT_UPLOAD &
			nCardSubmitType));

		//MES- CR: This comment would be clearer if you pointed out that SendTimezone was a new feature for 5.0
		//AGC- RCR: Why should I, when you are so eloquently doing so for me?
		// AGC- This case is a little different. If we are reading an upgraded object, it won't have this item set. We want it to be set
		// by default... but not for newly created objects. We are assuming that if we are in edit mode and the setting isn't set, we are
		// editing an upgraded object. We are assuming that if the value is missing from the property bag, the propertybag will return either
		// null or the empty string.
		// Default if the setting is incorrect or missing from the propertybag.
		if (EditorStartControl.EDITOR_START_FLAG_EDIT == ((WebServiceModel) m_asOwner.GetModel(
					WebServiceModel.STR_MVC_CLASS_NAME)).GetEditorType())
		{
			m_bSendTimezone = true;
		}
		else
		{
			m_bSendTimezone = false;
		}
		
		m_bSendTimezone = WSModelHelper.PBReadBool(ptProviderInfo, AExtensibilityConstants.PTC_HTTPGADGET_SENDTIMEZONE, m_bSendTimezone);

		XPArrayList xpalUserPrefs = null;
		try {
			xpalUserPrefs = WSModelHelper.ObjectArrayToXPArray(ptProviderInfo.ReadAsObject1DArray(AExtensibilityConstants.PT_PROPBAG_HTTPGADGET_USERPREFS));
		} catch (Exception e)
		{
			//AGC- This likely means that this has never been set in the propertybag.
			// That's fine. Note that this means we default to TRUE (intended).
		}
		
		if (null != xpalUserPrefs)
		{
			if (!xpalUserPrefs.Contains(AExtensibilityConstants.PTC_STRLOCALE))
			{
				m_bSendLocale = false;
			}
		}
		
		m_bRefreshButton = ((m_ptWebService.GetSupportsPersonalization() & PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_REFRESH) > 0);
		m_bSharedSession = WSModelHelper.PBReadBool(ptProviderInfo, PlumtreeExtensibility.PT_PROPBAG_HTTP_SESSION_TYPE, m_bSharedSession);
		m_bSendSubportalID = WSModelHelper.PBReadBool(ptProviderInfo, PlumtreeExtensibility.PT_PROPBAG_HTTPGADGET_SENDSUBPORTALID, m_bSendSubportalID);
		m_bSendAlignment = WSModelHelper.PBReadBool(ptProviderInfo, PlumtreeExtensibility.PT_PROPBAG_HTTPGADGET_SENDALIGNMENT, m_bSendAlignment);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorHelperModel#SaveSection(XPHashtable)
	 */
	public int SaveSection(XPHashtable _htFormData)
	{
				
		SetTokenTimeout(WSModelHelper.GetFormElementInt(_htFormData, AdvSettingsPluginView.HTMLINPUT_LOGINTOKEN_TIMEOUT, GetTokenTimeout()));
		
		//AGC- Because these are checkboxes, we want to make sure that if they are not set, we return false - hence the false in thefollowing statements.
		SetSendPageID(WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_SEND_PAGEID, false));
		m_bSendLoginToken = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_SEND_LOGINTOKEN, false);
		m_bSendCommunityACL = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_SEND_COMMUNITYACL, false);
		m_bSendTimezone = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_SEND_TIMEZONE, false);
		
		m_bCardSubmitFilePath = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_CARDSUBMIT_FILEPATH, false);
		m_bCardSubmitRemoteUI = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_CARDSUBMIT_REMOTEUI, false);
		m_bSupportsDocUpload = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_CARDSUBMIT_UPLOAD, false);
		m_bSendLocale = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_SEND_LOCALE, false);
		
		m_bSupportsACLImport = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_SUPPORTSACLIMPORT, false);
		m_bSupportsMirroring = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_SUPPORTSMIRRORING, false);
		m_bRefreshButton = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_REFRESH_BUTTON, false);
		m_bSharedSession = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_SHARED_SESSION, false);
		m_bSendSubportalID = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_SEND_SUBPORTALID, false);
		m_bSendAlignment = WSModelHelper.GetFormElementBool(_htFormData, AdvSettingsPluginView.HTMLCHECKBOX_SEND_ALIGNMENT, false);


		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Whether or not to send a pageid to the portlet.
	 */
	public void SetSendPageID(boolean bSendPageID)
	{
		m_bSendPageID = bSendPageID;
	}

	/**
	 * The duration of the timeout token (in minutes)
	 */
	public void SetTokenTimeout(int nTokenTimeout)
	{
		m_nTokenTimeout = nTokenTimeout;
	}
}

