package com.plumtree.portalpages.admin.editors.webservice.plugins;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.editors.webservice.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author adolfoc
 *
 */
public class AdvSettingsPluginView extends EditorView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.webservice.plugins.AdvSettingsPluginView");

	public static final String STR_MVC_CLASS_NAME = "AdvSettingsPluginView";

	public static final String HTMLCHECKBOX_SEND_PAGEID		    = "in_cb_SendPageID";
	public static final String HTMLCHECKBOX_SEND_LOGINTOKEN     = "in_cb_SendLoginToken";
	public static final String HTMLCHECKBOX_SEND_LOCALE     = "in_cb_SendLocale";
	public static final String HTMLCHECKBOX_SEND_COMMUNITYACL   = "in_cb_SendCommACL";
	public static final String HTMLCHECKBOX_SEND_TIMEZONE	    = "in_cb_SendTimezone";
	public static final String HTMLCHECKBOX_CARDSUBMIT_FILEPATH = "in_cb_CardSubmitFilePath";
	public static final String HTMLCHECKBOX_CARDSUBMIT_REMOTEUI = "in_cb_CardSubmitRemoteUI";
	public static final String HTMLCHECKBOX_SUPPORTSMIRRORING = "in_cb_SupportMirror";
	public static final String HTMLCHECKBOX_SUPPORTSACLIMPORT = "in_cb_SupportACLImport";
	public static final String HTMLINPUT_LOGINTOKEN_TIMEOUT = "in_tx_LoginTimeout";
	public static final String HTMLCHECKBOX_CARDSUBMIT_UPLOAD = "in_cb_CardSubmitUpload";
	public static final String HTMLCHECKBOX_REFRESH_BUTTON    = "in_cb_RefreshButton";
	public static final String HTMLCHECKBOX_SHARED_SESSION    = "in_cb_SharedSession";
	public static final String HTMLCHECKBOX_SEND_SUBPORTALID  = "in_cb_SendSubportalID";
	public static final String HTMLCHECKBOX_SEND_ALIGNMENT 	  = "in_cb_SendAlingment";

	/**
	 * Our parent Activity space.
	 */
	private AActivitySpace m_asasParent;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new AdvSettingsPluginView();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
	 */
	public HTMLElement Display()
	{
		HTMLElementCollection result = new HTMLElementCollection();
		HTMLTable myTable;
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLSpan mySpan;
		HTMLInput myInput;

		try
		{
			result.AddInnerHTMLElement(super.GetHeaderView(m_asOwner.GetString(
						1033, "ptmsgs_portaladminmsgs")));

			myTable = WSViewHelper.AddTable(result);

			// 1st row begins (Top Margin)
			WSViewHelper.AddEmptyRow(myTable, "5");

			// 1st row ends
			// 2nd row
			IAdvSettingsPluginModelRO modelRO = (IAdvSettingsPluginModelRO) m_model;
			switch (modelRO.GetWSMode())
			{
			case WebServiceModel.WSMODE_PORTLET:

				DrawCheckBox(myTable, HTMLCHECKBOX_SEND_COMMUNITYACL, "1",
					m_asOwner.GetString(1036, "ptmsgs_portaladminmsgs"),
					modelRO.GetSendCommunityACL());

				DrawCheckBox(myTable, HTMLCHECKBOX_SEND_PAGEID, "1",
					m_asOwner.GetString(1034, "ptmsgs_portaladminmsgs"),
					modelRO.GetSendPageID());

				break;

			case WebServiceModel.WSMODE_CWS:

				DrawCheckBox(myTable, HTMLCHECKBOX_CARDSUBMIT_FILEPATH, "1",
					m_asOwner.GetString(1038, "ptmsgs_portaladminmsgs"),
					modelRO.GetCardSubmitFilePath());

				DrawCheckBox(myTable, HTMLCHECKBOX_CARDSUBMIT_REMOTEUI, "1",
					m_asOwner.GetString(1039, "ptmsgs_portaladminmsgs"),
					modelRO.GetCardSubmitRemoteUI());

				DrawCheckBox(myTable, HTMLCHECKBOX_CARDSUBMIT_UPLOAD, "1",
					m_asOwner.GetString(2626, "ptmsgs_portaladminmsgs"),
					modelRO.GetSupportsUpload());
									
				DrawCheckBox(myTable, HTMLCHECKBOX_SUPPORTSMIRRORING, "1",
					m_asOwner.GetString(1913, "ptmsgs_portaladminmsgs"),
					modelRO.GetSupportsMirroring());

				DrawCheckBox(myTable, HTMLCHECKBOX_SUPPORTSACLIMPORT, "1",
					m_asOwner.GetString(1914, "ptmsgs_portaladminmsgs"),
					modelRO.GetSupportsACLImport());

				break;
			case WebServiceModel.WSMODE_SWS:
				DrawCheckBox(myTable, HTMLCHECKBOX_SEND_LOCALE, "1",
					m_asOwner.GetString(1912, "ptmsgs_portaladminmsgs"),
					modelRO.GetSendLocale());
				break;
			case WebServiceModel.WSMODE_AWS:
			case WebServiceModel.WSMODE_NONE:default:
			}

			DrawCheckBox(myTable, HTMLCHECKBOX_SEND_TIMEZONE, "1",
				m_asOwner.GetString(1037, "ptmsgs_portaladminmsgs"),
				modelRO.GetSendTimezone());

			DrawCheckBox(myTable, HTMLCHECKBOX_SEND_LOGINTOKEN, "1",
				m_asOwner.GetString(1035, "ptmsgs_portaladminmsgs"),
				modelRO.GetSendLoginToken());

			// Show the Login Token in minutes.
			myRow = WSViewHelper.AddRow(myTable);

			// Left Margin
			WSViewHelper.AddEmptyCell(myRow, "1");
			myCell = WSViewHelper.AddEmptyCell(myRow, "1");
			myCell.SetWidth("30");

			// The label.
			myCell     = WSViewHelper.AddCell(myRow, "1");
			mySpan     = WSViewHelper.AddSpanToCell(myCell);

			// AGC- All of these spaces are just for an indent.
			mySpan.AddInnerHTMLString(XPHttpUtility.HtmlEncode(
					m_asOwner.GetString(1040, "ptmsgs_portaladminmsgs")) +
				SPACE + SPACE);

			// The actual input			
			myInput = new HTMLInput(HTMLInputTypes.TEXT,
					HTMLINPUT_LOGINTOKEN_TIMEOUT, "");

			//This is a number input box, so set the size = 3
			myInput.SetSize(3);
			myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);

			String strSetDuration = XPConvert.ToString(modelRO.GetTokenTimeout());

			if ((null != strSetDuration) && (0 != strSetDuration.length()))
			{
				myInput.SetValue(strSetDuration);
			}

			mySpan.AddInnerHTMLElement(myInput);

			// Add the refresh button and shared session checkboxes at the end, only if it's a portlet -jas
			if(modelRO.GetWSMode() == WebServiceModel.WSMODE_PORTLET)
			{
				DrawCheckBox(myTable, HTMLCHECKBOX_REFRESH_BUTTON, "1",
						m_asOwner.GetString(2732, "ptmsgs_portaladminmsgs"),
						modelRO.GetRefreshButton());

				DrawCheckBox(myTable, HTMLCHECKBOX_SHARED_SESSION, "1",
						m_asOwner.GetString(2733, "ptmsgs_portaladminmsgs"), modelRO.GetSharedSession());
			}
			
			// Add the send subportal ID checkbox. This is for all types of web services.
			DrawCheckBox(myTable, HTMLCHECKBOX_SEND_SUBPORTALID, "1",
					m_asOwner.GetString(2832, "ptmsgs_portaladminmsgs"), modelRO.GetSendSubportalID());			
			
			// Add the portlet alignment checkbox. This is only for portlets.
			if(modelRO.GetWSMode() == WebServiceModel.WSMODE_PORTLET)
			{
				DrawCheckBox(myTable, HTMLCHECKBOX_SEND_ALIGNMENT, "1",
						m_asOwner.GetString(2833, "ptmsgs_portaladminmsgs"),
						modelRO.GetSendAlignment());
			}
			
			// Right Margin
			WSViewHelper.AddSpacerCell(myRow, "1");
			WSViewHelper.AddEmptyCell(myRow, "1");

			// last row (spacer row)
			WSViewHelper.AddEmptyRow(myTable, "5");
		}
		catch (HTMLException e)
		{
			
				log.Error(e, STR_MVC_CLASS_NAME +
					"::Display The following error occured when trying to display the HTML: " +
					e.getMessage());
			

			throw new XPException(m_asOwner.GetString(1032,
					"ptmsgs_portaladminmsgs"), e);
		}

		return result;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		HTMLScript result = new HTMLScript("text/javaScript");
		return result;
	}

	/**
	* @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	*/
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO, AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		super.Init(model, parent);
		m_asasParent = parent;
	}

	// Helper Function:
	private void DrawCheckBox(HTMLTable myTable, String strHTTP,
		String strValue, String strUI, boolean bChecked)
	{
		WSViewHelper.AddSimpleCheckbox(myTable, 5, strUI, strHTTP, strValue, bChecked, "");
	}
}

