package com.plumtree.portalpages.admin.editors.webservice.plugins;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.admin.editors.webservice.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;

/**
 * @author JaneD
 *
 */
public class DebugConfigPluginModel extends EditorHelperModel
	implements IDebugConfigPluginModelRO, IWSSubModel
{
	/** This object's name */
	public static final String STR_MVC_CLASS_NAME = "WSDebugConfigPluginModel";


	/** Is http request tracing enabled? */
	private boolean m_bHttpRequestTracingOn = false;
	
	/** Is http response tracing enabled? */
	private boolean m_bHttpResponseTracingOn = false;

	
	/**
	 * The Web Service object being edited by our parent Web Service editor
	 */
	IPTWebService m_ptWebService = null;

	
	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new DebugConfigPluginModel();
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskBeforeWSStore()
	 */
	public void DoTaskBeforeWSStore()
	{
		IXPPropertyBag ptProviderInfo = (IXPPropertyBag) m_ptWebService.GetProviderInfo();

		WSModelHelper.PBWrite(ptProviderInfo, PlumtreeExtensibility.PT_HTTP_REQUEST_TRACE_ENABLED, GetHTTPRequestTracingOn());
		WSModelHelper.PBWrite(ptProviderInfo, PlumtreeExtensibility.PT_HTTP_RESPONSE_TRACE_ENABLED, GetHTTPResponseTracingOn());
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnRemoteServerUpdate(String, String, String)
	 */
	public void DoTaskOnRemoteServerUpdate(String strRemoteServerID,
		String strRemoteServerName, String strRemoteServerBaseURL)
	{
		// do nothing
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnWSModeUpdate(int)
	 */
	public void DoTaskOnWSModeUpdate(int nNewWSMode)
	{
		//do nothing
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}


	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#GetValidationFunction()
	 */
	public String GetValidationFunction()
	{
		//do nothing
		return null;
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#InitWSModel()
	 */
	public void InitWSModel(IPTWebService ptWebService)
	{
		m_ptWebService = ptWebService;

		IXPPropertyBag ptProviderInfo = (IXPPropertyBag) ptWebService.GetProviderInfo();
		
		m_bHttpRequestTracingOn = WSModelHelper.PBReadBool(ptProviderInfo, PlumtreeExtensibility.PT_HTTP_REQUEST_TRACE_ENABLED, m_bHttpRequestTracingOn);
		m_bHttpResponseTracingOn = WSModelHelper.PBReadBool(ptProviderInfo, PlumtreeExtensibility.PT_HTTP_RESPONSE_TRACE_ENABLED, m_bHttpResponseTracingOn);
		
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorHelperModel#SaveSection(XPHashtable)
	 */
	public int SaveSection(XPHashtable _htFormData)
	{
		m_bHttpRequestTracingOn = WSModelHelper.GetFormElementBool(_htFormData, DebugConfigPluginView.HTMLRADIO_HTTPREQUEST, false);
		m_bHttpResponseTracingOn = WSModelHelper.GetFormElementBool(_htFormData, DebugConfigPluginView.HTMLRADIO_HTTPRESPONSE, false);

		return RepostControl.PAGE_STATUS_VALID;
	}
	
	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.plugins.IDebugConfigPluginModelRO#GetHTTPRequestTracingOn()
	 */
	public boolean GetHTTPRequestTracingOn()
	{
		return m_bHttpRequestTracingOn;
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.plugins.IDebugConfigPluginModelRO#GetHTTPResponseTracingOn()
	 */
	public boolean GetHTTPResponseTracingOn()
	{
		return m_bHttpResponseTracingOn;
	}
	
}

