/**
 * Created on Oct 29, 2002
 *
 * To change this generated comment edit the template variable "filecomment":

 * To enable and disable the creation of file comments go to

 */
package com.plumtree.portalpages.admin.editors.webservice.plugins;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.admin.editors.webservice.*;
import com.plumtree.portalpages.common.helpers.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.providerframework.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * @author adolfoc
 *
 */
public class HTTPSettingsPluginModel extends EditorHelperModel
	implements IHTTPSettingsPluginModelRO, IWSSubModel
{
	/** This object's name */
	public static final String STR_MVC_CLASS_NAME = "WSHTTPSettingsPluginModel";

	/**
	 * This mask is used by our view to send time data to us in units that we
	 * support.
	 */
	public static final int NMASK_SUPPORTED_UNITS = PTDurationCombo.DISPLAYMINUTES |
		PTDurationCombo.DISPLAYSECONDS;

	/**
	 * The Web Service object being edited by our parent Web Service editor
	 */
	IPTWebService m_ptWebService = null;

	/**
	 * We might get the Remote Server Base URL from
	 * our parent model if it is a WebService Model. It is stored here.
	 */
	private String m_strBaseURL = "";

	/** URL to this Web Service (To be combined with the BaseURL from the Remote Server)
	 * (RemoteServerPluginView) */
	private String m_strPortletURL = "";

	/**
	 * Timeout information.
	 */
	protected PTDuration m_ptdPortletTimeout;
	protected PTDuration m_ptdGatewayTimeout;

	public HTTPSettingsPluginModel()
	{
		super();
		m_ptdPortletTimeout = new PTDuration(5000, NMASK_SUPPORTED_UNITS);
		m_ptdGatewayTimeout = new PTDuration(20000, NMASK_SUPPORTED_UNITS);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new HTTPSettingsPluginModel();
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskBeforeWSStore()
	 */
	public void DoTaskBeforeWSStore()
	{
		IXPPropertyBag ptProviderInfo = (IXPPropertyBag) m_ptWebService.GetProviderInfo();

		WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PT_PROPBAG_FORMAT, AExtensibilityConstants.PT_PROPBAG_FORMAT_HTTPGADGET_INIT);
		WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PT_PROPBAG_HTTPGADGET_URL, GetPortletURL());
		WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PT_PROPBAG_HTTPGADGET_TIMEOUT, m_ptdPortletTimeout.GetMilliseconds());
		WSModelHelper.PBWrite(ptProviderInfo, AExtensibilityConstants.PTC_HTTPGADGET_GATEWAYTIMEOUT, m_ptdGatewayTimeout.GetMilliseconds());
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnRemoteServerUpdate(String, String, String)
	 */
	public void DoTaskOnRemoteServerUpdate(String strRemoteServerID,
		String strRemoteServerName, String strRemoteServerBaseURL)
	{
		m_strBaseURL = strRemoteServerBaseURL;
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#DoTaskOnWSModeUpdate(int)
	 */
	public void DoTaskOnWSModeUpdate(int nNewWSMode)
	{
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.plugins.IHTTPSettingsPluginModelRO#GetBaseURL()
	 */
	public String GetBaseURL()
	{
		return m_strBaseURL;
	}


	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the m_strWebServiceURL. Value must be WS_ENABLED or WS_DISABLED
	 *
	 * @return String
	 */
	public String GetPortletURL()
	{
		return m_strPortletURL;
	}


	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#GetValidationFunction()
	 */
	public String GetValidationFunction()
	{
		return HTTPSettingsPluginView.VALIDATION_FUNCTION_NAME;
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.IWSSubModel#InitWSModel()
	 */
	public void InitWSModel(IPTWebService ptWebService)
	{
		m_ptWebService = ptWebService;

		IXPPropertyBag ptProviderInfo = (IXPPropertyBag) ptWebService.GetProviderInfo();
		
		SetPortletURL(WSModelHelper.PBReadString(ptProviderInfo, AExtensibilityConstants.PT_PROPBAG_HTTPGADGET_URL, GetPortletURL()));
		
		
		m_ptdPortletTimeout = new PTDuration(WSModelHelper.PBReadInt(
					ptProviderInfo,
					AExtensibilityConstants.PT_PROPBAG_HTTPGADGET_TIMEOUT,
					m_ptdPortletTimeout.GetMilliseconds()), NMASK_SUPPORTED_UNITS);

		m_ptdGatewayTimeout = new PTDuration(WSModelHelper.PBReadInt(
					ptProviderInfo,
					AExtensibilityConstants.PTC_HTTPGADGET_GATEWAYTIMEOUT,
					m_ptdGatewayTimeout.GetMilliseconds()), NMASK_SUPPORTED_UNITS);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorHelperModel#SaveSection(XPHashtable)
	 */
	public int SaveSection(XPHashtable _htFormData)
	{


		SetPortletURL(WSModelHelper.GetFormElementString(_htFormData, HTTPSettingsPluginView.HTMLINPUT_PORTLET_URL, GetPortletURL()));


		m_ptdPortletTimeout = WSModelHelper.GetFormElementDuration(_htFormData,
				HTTPSettingsPluginView.HTMLINPUT_TIMEOUT_DURATION,
				HTTPSettingsPluginView.HTMLSELECT_TIMEOUT_UNITS, m_ptdPortletTimeout);

		m_ptdGatewayTimeout = WSModelHelper.GetFormElementDuration(_htFormData,
				HTTPSettingsPluginView.HTMLINPUT_TIMEOUT_GATEWAYDURATION,
				HTTPSettingsPluginView.HTMLSELECT_TIMEOUT_GATEWAYUNITS, m_ptdGatewayTimeout);

		int nValid = RepostControl.PAGE_STATUS_VALID;


		if ((null == m_strBaseURL) || (m_strBaseURL.length() <= 0))
		{
			nValid = WSModelHelper.ValidateIsURLString(m_strPortletURL, m_asOwner, m_asOwner.GetString(1925, "ptmsgs_portaladminmsgs"), nValid);
		}
		else
		{
			nValid = WSModelHelper.ValidateNonEmptyString(m_strPortletURL, m_asOwner, m_asOwner.GetString(1093, "ptmsgs_portaladminmsgs"), nValid);
		}

		nValid = WSModelHelper.ValidateDurationRange(m_ptdPortletTimeout,
				m_asOwner, m_asOwner.GetString(1926, "ptmsgs_portaladminmsgs"), nValid, 1000,
				1000 * 60 * 90);


		nValid = WSModelHelper.ValidateDurationRange(m_ptdGatewayTimeout,
				m_asOwner, m_asOwner.GetString(1927, "ptmsgs_portaladminmsgs"), nValid, 1000,
				1000 * 60 * 90);

		return nValid;
	}


	/**
	 * Sets the m_strWebServiceURL.
	 *
	 * @param strWebServiceURL The m_strWebServiceURL to set
	 */
	public void SetPortletURL(String strWebServiceURL)
	{
		this.m_strPortletURL = strWebServiceURL;
	}
	
	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.plugins.ICWSSettingsPluginModelRO#GetGatewayDuration()
	 */
	public PTDuration GetGatewayDuration() {
		return m_ptdGatewayTimeout;
	}

	/**
	 * @see com.plumtree.portalpages.admin.editors.webservice.plugins.ICWSSettingsPluginModelRO#GetSOAPDuration()
	 */
	public PTDuration GetPortletDuration() {
		return m_ptdPortletTimeout;
	}
	
}

