package com.plumtree.portalpages.admin.editors.xui;


// Plumtree Server
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.gateway.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.xui.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This is the Helper Model class for the XUI page.
 *
 * @author Julie Fournier
 */
public class XUIHelperModel extends EditorHelperModel
	implements IXUIHelperModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.editors.xui.XUIHelperModel");

	/**
	 * Class Name for the Activity Space.
	 */
	public static final String STR_MVC_CLASS_NAME = "XUIHelper";

	/**
	 * Prefix for the XUI input param names.
  	 */
	public static final String XUI_INPUT_PREFIX = "XUI_";

	/**
	 * Integer type of value of the input.
	 */
	public static final String XUI_INPUT_TYPE_INT = "integer";

	/**
	 * Boolean type of value of the input.
	 */
	public static final String XUI_INPUT_TYPE_BOOLEAN = "boolean";

	/**
	 * XUI page not initialized.
	 */
	public static final boolean XUI_NOT_INITIALIZED = false;

	/**
	 * XUI page valid.
	 */
	public static final boolean XUI_PAGE_VALID = true;

	/**
	 * Property Bag of the Objects being created or edited.
	 * It contains all the settings set on the XUI pages.
	 */
	private IXPPropertyBag m_pbXUISettings;

	/**
	 * XUI Service object from the remote server.
	 */
	private IService m_xuiService;
	
	/**
	 * Index of XUI service (support for multiple XUI services
	 * per editor).  Starts at 0.
	 */
	private int m_nServiceIndex;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new XUIHelperModel();
	}

	/**
	 * Finalizes and returns the property bag.
	 * Needs to be called at the end of the wizard, before we store the
	 * property bag in the object.
	 *
	 * @return IXPPropertyBag
	 */
	public IXPPropertyBag GetFinalXUISettings()
	{
		// Bug 44211 - This new validation code added to Martok caused some issues, backing it out for now.
		//	Revisit in a future release. -jas
//		for (int indPage = 1; indPage <= m_xuiService.GetPageCount(); indPage++)
//		{
//			if (!m_xuiService.Validate(indPage, m_pbXUISettings))
//			{
//				UIError myError = new UIError(UIErrorLevel.SERIOUS,
//						UIErrorScope.CURRENT_USER, null, null,
//						null);
//							
//				HTMLElementCollection myHTML = new HTMLElementCollection();
//				HTMLSpan mySpan;
//				HTMLAnchor myAnchor = null;
//				
//				mySpan = new HTMLSpan();
//				myHTML.AddInnerHTMLElement(mySpan);
//				mySpan.SetStyleClass(PTStyleClass.ALERT_SUBTITLE);
//				mySpan.AddInnerHTMLString(m_asOwner.GetString(2635, "ptmsgs_portaladminmsgs"));
//				
//				String sPageName;
//				EditorDP dpPage;
//				
//				if (m_nServiceIndex == 0)
//				{
//					sPageName = "XUI" + indPage;
//				}
//				else
//				{
//					sPageName = "XUI" + m_nServiceIndex + "_" + indPage;
//				}
//
//				dpPage = (EditorDP) m_asOwner.GetPage(sPageName);
//					
//				myHTML.AddInnerHTMLElement(new HTMLBr());
//				myHTML.AddInnerHTMLString("&bull;");
//				myHTML.AddInnerHTMLString(CommonHTMLStrings.SPACE);
//				myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
//				myAnchor.SetOnClick("goToPage('" + m_asOwner.GetName() +
//					"', '" + sPageName + "',''); return false;");
//				myAnchor.AddInnerHTMLString(XPHttpUtility.HtmlEncode(dpPage.GetLeftNavString()));
//				myHTML.AddInnerHTMLElement(myAnchor);
//					
//				myError.SetHTMLMessage(myHTML);			
//				
//				UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), 
//					m_asOwner.GetApplication());
//
//			}
//		}
		
		m_xuiService.Finalize(m_pbXUISettings);

		return (m_pbXUISettings);
	}

	/**
	 * Returns the HTML for the XUI page.
	 *
	 * @param _nXUIPageID ID of the page to return the HTML for.
	 *
	 * @return String
	 */
	public String GetHTMLForXUIPage(int _nXUIPageID)
	{
		String sHTML;

		Object[][] xuiParam = new Object[2][1];
		xuiParam[0][0]     = "gp_sRelativeImageServerBaseURL";
		xuiParam[1][0]     = ImgSvrURLHelper.GetInstance().GetRootURL(m_asOwner.GetIsSecuredSpace());

		// aj: IServiceContent contains either:
		//	a) a String that represents a transformed SCI page
		//	b) a markup fragment array that needs to be passed through the taglib stuff
		IServiceContent pXUIContent = m_xuiService.GetContent(_nXUIPageID, m_pbXUISettings, xuiParam);
		if(pXUIContent.ContentIsMarkupArray())
		{
			HTMLDiv myDiv = new HTMLDiv();
			IPTState objAppDataObject = AppDataObject.GetAppDataObject(0, 0, PTConfigHelper.GetApplicationBaseURL(m_asOwner), m_asOwner);
			Object[][] arrMarkup = pXUIContent.GetMarkupFragment(objAppDataObject);
			myDiv.AddInnerHTMLElement(GatewayUtils.AddMarkupToSingleElement(m_asOwner, arrMarkup));
			sHTML = myDiv.GetDisplayString();
		} else {
			// aj: simple XUI content, ah the good old days
			sHTML = pXUIContent.GetXUIPage();
		}

		return sHTML;
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the XUI page instructions.
	 *
	 * @param _nXUIPageID ID of the page to return the instructions for.
	 *
	 * @return String
	 */
	public String GetPageInstructions(int _nXUIPageID)
	{
		return m_xuiService.GetWizardPageInstructions(_nXUIPageID);
	}

	/**
	 * Returns the XUI page title.
	 *
	 * @param _nXUIPageID ID of the page to return the title for.
	 *
	 * @return String
	 */
	public String GetPageTitle(int _nXUIPageID)
	{
		return m_xuiService.GetWizardPageTitle(_nXUIPageID);
	}

	/**
	 * Returns the XUI page help page name.
	 *
	 * @param _nXUIPageID ID of the page to return the help for.
	 *
	 * @return String
	 */
	public String GetHelpFileName(int _nXUIPageID)
	{
		if (null == m_xuiService) {
			return "PT_Submit_RemoteDocument_MainSettings";
		} else {
			return m_xuiService.GetWizardPageHelp(_nXUIPageID);
		}
	}

	/**
	 * Inits the XUI helper model.
	 *
	 * @param _pbXUISettings PropertyBag to store the XUI settings
	 * @param _xuiService XUI Service on the remote server
	 */
	public void InitializeXUIHelperModel(IXPPropertyBag _pbXUISettings,
		IService _xuiService)
	{
		m_pbXUISettings     = _pbXUISettings;
		m_xuiService	    = _xuiService;
		m_nServiceIndex		= 0;

		if (XUI_NOT_INITIALIZED == m_xuiService.GetInitialized())
		{
			m_xuiService.Initialize(m_pbXUISettings,
				XUIHelperMethods.XUI_TIMEOUT_IN_SECONDS);
		}
	}

	/**
	 * Calls the right save function according to the page name passed in argument.
	 *
	 * @param _htFormData
	 * @return int Page status for server side validation
	 */
	public int SaveSection(XPHashtable _htFormData)
	{
		int nXUIPageID = -1;

		if (_htFormData.ContainsKey(XUIView.HTMLHIDDEN_XUI_PAGEID))
		{
			// JF- Retrieve the ID of the page that has just been posted.
			String[] data = (String[]) _htFormData.GetElement(XUIView.HTMLHIDDEN_XUI_PAGEID);

			try
			{
				nXUIPageID = XPConvert.ToInteger(data[0]);


				// JF- Save the settings posted by the form.
				UpdateXUISettings(_htFormData);

				int nRepostAction = RepostControl.POSTTOSELF_ACTION_UNDEFINED;

				try
				{
					// JF- Get the Repost control and the repost action
					EditorRepostControl myControl = (EditorRepostControl) m_asOwner.GetControl(((AFormAS) m_asOwner).
						GetRepostControlName());
					nRepostAction = myControl.GetRepostAction();
				}
				catch (Exception e)
				{
					
						log.Error("XUIHelperModel::SaveSection The following exception " +
							"happened when trying to retrieve the Editor Repost Control:" + e);
					
				}

				if ((RepostControl.POSTTOSELF_ACTION_GOTOPAGE == nRepostAction) ||
  					(RepostControl.POSTTOSELF_ACTION_FINISH == nRepostAction) ||
  					(RepostControl.POSTTOSELF_ACTION_APPLY_CHANGES == nRepostAction))
  				{
					if (XUI_PAGE_VALID == m_xuiService.Validate(nXUIPageID,
							m_pbXUISettings))
					{
						return (RepostControl.PAGE_STATUS_VALID);
					}
					else
					{
						return (RepostControl.PAGE_STATUS_INVALID);
					}
				}
			}
			catch (Exception e)
			{
				
					log.Error("XUIHelperModel::SaveSection Unable to save the XUI page with ID: " +
						data + ". The following exception happened: " + e);
				
				
				return (RepostControl.PAGE_STATUS_INVALID);
			}
		}

		return (RepostControl.PAGE_STATUS_VALID);
	}

	/**
	 * Saves the XUI settings posted by the form into the PropertyBag.
	 *
	 * @param _htFormData
	 */
	private void UpdateXUISettings(XPHashtable _htFormData)
	{
		String sKey;
		String[] aData;

		int nNextSeparatorIndex;
		String sXUIInputType;
		String sXUIInputName;

		IXPEnumerator keys = _htFormData.GetKeys();

		while (keys.MoveNext())
		{
			sKey = (String) keys.GetCurrent();

			if (sKey.startsWith(XUI_INPUT_PREFIX))
			{
				try
				{
					aData     = (String[]) _htFormData.GetElement(sKey);

					nNextSeparatorIndex     = sKey.indexOf("_",
							XUI_INPUT_PREFIX.length());
					sXUIInputType		    = sKey.substring(XUI_INPUT_PREFIX.length(),
							nNextSeparatorIndex);
					sXUIInputName		    = sKey.substring(nNextSeparatorIndex +
							1);

					if (XPStringUtility.EqualsIgnoreCase(XUI_INPUT_TYPE_INT,sXUIInputType))
					{
						if ((null != aData[0]) && (0 != aData[0].length()))
						{
							m_pbXUISettings.Write(sXUIInputName,
								XPConvert.ToInteger(aData[0]));
						}
					}
					else if (XPStringUtility.EqualsIgnoreCase(XUI_INPUT_TYPE_BOOLEAN,sXUIInputType))
					{
						if (_htFormData.ContainsKey("TRUE_" + sXUIInputName))
						{
							aData = (String[]) _htFormData.GetElement("TRUE_" +
									sXUIInputName);
							m_pbXUISettings.Write(sXUIInputName,
								XPConvert.ToInteger(aData[0]));
						}
						else
						{
							m_pbXUISettings.Write(sXUIInputName,
								XPConvert.ToInteger(aData[0]));
						}
					}
					else
					{
						m_pbXUISettings.Write(sXUIInputName, aData[0]);
					}
				}
				catch (Exception e)
				{
					
						log.Error("XUIHelperModel::UpdateXUISettings The following exception " +
							"happened when trying to set the data with key: " +
							sKey + "." + e);
					
				}
			}
		}
	}
	
	/**
	 * If there are multiple XUI services for the editor,
	 * set the index for this model.
	 * @param _nIndex
	 */
	public void SetServiceIndex(int _nIndex)
	{
		m_nServiceIndex = _nIndex;
	}
}

