package com.plumtree.portalpages.admin.httpmemorydebugging;

// Framework Imports
import com.plumtree.openfoundation.web.*;
import com.plumtree.portalpages.admin.ptutilities.diagnostic.dynamicloads.varpacks.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;


/**
 * This is the main repost control for the HTTPMemoryDebug AS.
 *
 * @author Julie Fournier
 */
public class HTTPMemoryDebugRepostControl extends RepostControl implements IHTTPControl
{
	private IXPRequest m_request = null;

	/** Repost action: Reload Var Pack */
	public final static int POSTTOSELF_ACTION_RELOAD_VARPACK = 10;
	
	/** Repost action: View Var Pack */
	public final static int POSTTOSELF_ACTION_VIEW_VARPACK = 11;	
	
	/** Repost action: View Var Pack */
	public final static int POSTTOSELF_ACTION_RELOAD_CURRENT_VIEWED_VARPACK = 12;
	
	/** Repost action: Clear cache */
	public final static int POSTTOSELF_ACTION_CLEAR_CACHE = 13;
	
	/** Repost action: Clear memory */
	public final static int POSTTOSELF_ACTION_CLEAR_MEMORY = 14;
		
	/** Repost action: Clear memory and caches */
	public final static int POSTTOSELF_ACTION_CLEAR_MEMORY_AND_CACHES = 15;						
	
	/** Repost action: Reload custom */
	public final static int POSTTOSELF_ACTION_RELOAD_DYNAMIC_LOADS = 80;
	
	/** Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "HTTPMemoryDebug";

	/**
	 * Switch between the action modes and call
	 * the associated method for each action.
	 *
	 * @param _nAction
	 */
	protected void PerformAction(int _nAction)
	{
		super.PerformAction(_nAction);

		switch (_nAction)
		{
			case POSTTOSELF_ACTION_RELOAD_VARPACK:
			((HTTPMemoryDebugModel) m_model).ReloadVarPack();
			break;
			
			case POSTTOSELF_ACTION_VIEW_VARPACK:
			// JF- Nothing to do here, the SavePage method will get the ID 
			// of the var pack to display from the Query String.
			break;
			
			case POSTTOSELF_ACTION_RELOAD_CURRENT_VIEWED_VARPACK:
			((HTTPMemoryDebugModel) m_model).ReloadCurrentViewedVarPack();
			
			Redirect myRedirect = new Redirect();
			myRedirect.SetLinkGetCachedSpace(m_asOwner);
			myRedirect.SetDisplayPage(VarPackContentDP.STR_MVC_CLASS_NAME);
			
			SetRedirect(myRedirect);
			break;
			
			case POSTTOSELF_ACTION_CLEAR_CACHE:
			((HTTPMemoryDebugModel) m_model).ClearCache();
			break;
			
			case POSTTOSELF_ACTION_CLEAR_MEMORY:
			((HTTPMemoryDebugModel) m_model).ClearMemory();
			break;
			
			case POSTTOSELF_ACTION_CLEAR_MEMORY_AND_CACHES:
			((HTTPMemoryDebugModel) m_model).ClearMemoryAndCaches();
			break;

			case POSTTOSELF_ACTION_RELOAD_DYNAMIC_LOADS:
			((HTTPMemoryDebugModel) m_model).ReloadDynamicLoads(m_request);
			break;
		}
		
		// null out member variables so we don't leak them.
		m_request = null;
	}
	
	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new HTTPMemoryDebugRepostControl();
	}

	/**
	 * Returns the Class name.
	 * 
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
	
	/**
	 * SetXPRequest is called by the system before calling
	 * PerformAction.
	 *
	 * @param request is the XPRequest object we're operating on
	 */
	public void SetHTTPItems(IXPRequest request, IWebData pageData)
	{
		m_request = request;
	}
}

