package com.plumtree.portalpages.admin.ptutilities.jobfolders;


// Framework Imports
import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.expandablelist.selectptobject.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.expandablelist.*;

/**
 * This is the implementation of the Expandable List
 * framework for the Growable Array.
 * The Growable Array is a list of input texts.
 * On add, a new input text populated with the default value
 * will be added at the bottom of the list.
 *
 * @author SudhaS
 */
public class JobFoldersListModel extends SelectPTObjectModel
	implements IJobFoldersListModelRO
{
	
	public static final String STR_MVC_CLASS_NAME = "JobFoldersList";
	private int[] m_aRowsToDelete;
	
	public static final int JOBS_COL_MAX = 3;
	
	public static final int JOBS_COL_NAME = 0;
	
	public static final int JOBS_COL_NEXTRUN = 1;
	
	public static final int JOBS_COL_ID = 2;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new JobFoldersListModel();
	}

	/**
	 * 
	 *
	 * @param _htFormData 
	 */
	public void DeleteRowsOnRepost(XPHashtable _htFormData)
	{
		int[] aRowsToDelete;

		if (!m_bIsReadOnly)
		{
			String sDataKey = GetUniqueString(ExpListView.HTMLCHECKBOX_DELROW);

			// JF- We get back the list of index of rows to delete.
			if (_htFormData.ContainsKey(sDataKey))
			{
				aRowsToDelete = GetArrayOfObjToDelete(_htFormData);

				// JF- We can only delete rows if the array is not empty
				if (m_saData.GetCount() > 0)
				{
					m_aRowsToDelete = new int[aRowsToDelete.length];

					for (int i = 0; i < aRowsToDelete.length; i++)
					{
						m_aRowsToDelete[i] = XPConvert.ToInteger(m_saData.GetItem(aRowsToDelete[i],
								EXPLIST_COL_OBJECTID));
					}

					m_saData.RemoveRows(aRowsToDelete);
				}
			}
		}
	}

	/**
	 * 
	 *
	 * @param _nArrIndex 
	 * @param _nObjID 
	 *
	 * @return 
	 */
	public Object[][] GetJobsByFolder(int _nArrIndex, int _nObjID)
	{
		IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
		//IPTScheduler ptScheduler = ptSession.GetScheduler();
		
		//ASQueryResultWrapper myQueryResult = null;

		IPTQueryResult ptqrJobsByFolder = ptSession.GetAdminCatalog()
			.GetObjectManager(PT_CLASSIDS.PT_JOB_ID).Query(-1, _nObjID,
				PT_PROPIDS.PT_PROPID_NAME, 0, -1,
				null);

		//ptqrJobHistoriesByFolder = (IPTQueryResult) ptScheduler.QueryJobAssignmentsByFolder(_nObjID);
		Object[][] oJobsArray = new Object[ptqrJobsByFolder.RowCount()][JOBS_COL_MAX];

		for (int i = 0; i < ptqrJobsByFolder.RowCount(); i++)
		{
			oJobsArray[i][JOBS_COL_NAME] = ptqrJobsByFolder.ItemAsString(i,
					PT_PROPIDS.PT_PROPID_NAME);
			
			XPDateTime dtNextRunTime = new XPDateTime(0);
			try
			{
				dtNextRunTime = ptqrJobsByFolder.ItemAsXPDateTime(i, PT_PROPIDS.PT_PROPID_JOB_NEXTRUNTIME);

        		XPDateFormat dtFmt = PTPersonalSettingsHelper.GetPersonalDateTimeFormat(m_asOwner,
                                                            /* use portal server time zone */  false);
				oJobsArray[i][JOBS_COL_NEXTRUN] = (String) dtFmt.Format(dtNextRunTime);
			}
			catch (Exception e) // TODO: Catching this is a hack!
			{
			   // TODO: This message is not really the right message to print for unscheduled jobs... 
			   oJobsArray[i][JOBS_COL_NEXTRUN] = m_asOwner.GetString(2418, "ptmsgs_portaladminmsgs");
			}
			
			oJobsArray[i][JOBS_COL_ID] = ptqrJobsByFolder.ItemAsObject(i,PT_PROPIDS.PT_PROPID_OBJECTID);			
		}

		return oJobsArray;
	}
	


	/**
	 * 
	 *
	 * @return 
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public int[] GetRowsToDelete()
	{
		return m_aRowsToDelete;
	}
}

