package com.plumtree.portalpages.admin.ptutilities.migrationutility;

import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openkernel.config.*;
import com.plumtree.openkernel.exceptions.*;
import com.plumtree.openkernel.factory.*;
import com.plumtree.openkernel.http.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.tempfile.*;
import com.plumtree.xpshared.config.ConfigPathResolver;

/**
 * 
 * @author jennifersu
 *
 */
public class MigrationLoadPackageModel extends EditorModel implements IMigrationLoadPackageModelRO
{
	
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.ptutilities.migrationutility.MigrationLoadPackageModel");

	public static final String STR_MVC_CLASS_NAME = "MigrationLoadPackageModel";
	
	/** Storage key for MigrationLoadPackageModel */
	public static final String strStorageLoadPackage = "STOR_LoadPackage";

	public static final String PACKAGE_NAME = "Package_Name";
	public static final String PACKAGE_DESCRIPTION = "Package_Description";
	public static final String PUBLISHER_NAME = "Publisher_Name";

	/**
	 * The number of minutes we want our temp file to stick around.
	 */
	private static int TEMPFILE_MINUTES = 20;

	/**
	 * The filename (path) of the currently loaded package.
	 */
	protected String m_strPackagePath;

	/**
	 * The filename without the path of the currently loaded package.
	 */
	private String m_strFileName;
	
	/**
	 * Whether the package was loaded from a web location
	 */
	private boolean m_bPackageIsWebLocation;
	
	/**
	 * If the package was loaded from a web location, the URL.
	 */
	private String m_strWebPath;
	
	/**
	 * Whether to pass basic auth credentials to retrieve package from URL.
	 */
	private boolean m_bBasicAuth;
	
	/**
	 * Basic Auth username
	 */
	private String m_strUsername;
	
	/**
	 * Basic Auth password
	 */
	private String m_strPassword;
	
	/**
	 * True if a package has been successfully loaded
	 */
	private boolean m_bPackageLoaded;
	
	/**
	 * Main model for the activity space (import or export)
	 */
	private MigrationUtilityModel m_mainModel;

	/**
	 * Empties all of our members and sets our package data to null.
	 *
	 */
	protected void ClearPackage()
	{
		m_strPackagePath = "";
		m_strFileName = null;
		m_bPackageIsWebLocation = false;
		m_strWebPath = "";
		m_bBasicAuth = false;
		m_strUsername = "";
		m_strPassword = "";
		m_bPackageLoaded = false;
		m_mainModel.ResetMigrationManager();
	}
	
	/**
	 * Inits member variables
	 *
	 * @param parent
	 */
	public void Init(AActivitySpace parent)
	{
		super.Init(parent);
		m_bBasicAuth = false;
		m_strUsername = "";
		m_strPassword = "";
		m_bPackageIsWebLocation = false;
		m_strFileName = "";
		m_strPackagePath = "";
		m_strWebPath = "";
		m_mainModel = (MigrationUtilityModel)m_asOwner.GetModel(((EditorAS)m_asOwner).m_strModelName);
		m_bPackageLoaded = false;
	}

	/**
	 * Implements some actions to perform on start.
	 *
	 * @param _htFormData
	 *
	 * @return boolean Is true if the current user has the rights to access this Editor.
	 */
	public boolean CheckEditorAccessAndStart(XPHashtable _htFormData)
	{
		return true;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new MigrationLoadPackageModel();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskBeforeStore()
	 */
	public void DoTaskBeforeStore()
	{
		super.DoTaskBeforeStore();
	}

	/**
	 * Implement some code to execute when the user clicks on "Finish".
	 *
	 */
	public void Finish()
	{
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Calls the right save function according to the page name passed in argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		int nPageStatus = RepostControl.PAGE_STATUS_VALID;

		return nPageStatus;
	}
	
	/**
	 * 
	 * @param xpRUpload
	 * @param _htFormData
	 */
	public void LoadPackage(XPHashtable _htFormData)
	{
		ClearPackage();
		
		// Create a temp directory to hold the new package data.
		XPFile xpfImport = new XPFile(TempFile.CreateTempFile("odimport", "importpackage", ".pte", TEMPFILE_MINUTES));
					
		// Determine if we are loading from URL or UNC path
		String[] data;
		data = (String[])_htFormData.GetElement(MigrationLoadPackageView.HTMLRADIO_PACKAGE_TYPE);
		if (((null == data) || (0 == data.length) || (null == data[0])))
		{
			log.Error(STR_MVC_CLASS_NAME + "::LoadPackage The package type is invalid: " + data);
			throw new XPException("blahblah");
		}

		if (data[0].equals("web"))
		{
			m_bPackageIsWebLocation = true;
		}
		else
		{
			m_bPackageIsWebLocation = false;
		}
		
		if (!m_bPackageIsWebLocation)
		{
			LoadPackageFromFile(xpfImport);
		}
		else
		{
			// Extract URL and basic auth info before attempting to load package.
			data = (String[])_htFormData.GetElement(MigrationLoadPackageView.HTMLTEXT_PACKAGE_WEB);
			if ((null == data) || (0 == data.length) || (null == data[0]))
			{
				log.Error(STR_MVC_CLASS_NAME + "::LoadPackage The package URL is invalid: " + data);
				throw new XPException(STR_MVC_CLASS_NAME + "::LoadPackage The package URL is invalid: " + data);
			}
			m_strWebPath = data[0];
			log.Info("Retrieving migration package from: " + m_strWebPath);				

			data = (String[])_htFormData.GetElement(MigrationLoadPackageView.HTMLCHECKBOX_BASICAUTH);
			if ((null == data) || (0 == data.length) || (null == data[0]))
			{
				m_bBasicAuth = false;
			}
			else
			{
				m_bBasicAuth = true;
			}

			if (m_bBasicAuth)
			{
				data = (String[])_htFormData.GetElement(MigrationLoadPackageView.HTMLINPUT_USERNAME);
				if ((null == data) || (0 == data.length) || (null == data[0]))
				{
					log.Error(STR_MVC_CLASS_NAME + "::LoadPackage The basic authusername is invalid: " + data);
					throw new XPException("blahblah");
				}
				m_strUsername = data[0];				

				data = (String[])_htFormData.GetElement(MigrationLoadPackageView.HTMLINPUT_PASSWORD);
				if ((null == data) || (0 == data.length) || (null == data[0]))
				{
					log.Error(STR_MVC_CLASS_NAME + "::LoadPackage The basic password is invalid: " + data);
					throw new XPException("blahblah");
				}				
				m_strPassword = data[0];				
			}

			LoadPackageFromWeb(xpfImport);
		}
	}

	/**
	 * Loads a package file from a known location and updates our
	 * member variables accordingly. If the initialization fails or the
	 * path is the same as the previous path, nothing happens.
	 */
	private void LoadPackageFromFile(XPFile _xpfImport)
	{
		String strFilePath = "";
		
		try
		{
			// Upload to the directory we created the temp file into (unfortunately, the uploader will not
			// preserve the tempfile filename, so we must just a temp createfile to reserve a directory for us.
			// It hardly matters, the empty temp file will be removed when the real one is removed.
			IXPRequest _xpRUpload = m_asOwner.GetCurrentHTTPRequest();
			_xpRUpload.Save(_xpfImport.GetParent().GetFullPath());

			// Path that the file was uploaded to
			strFilePath = ((XPRequest) _xpRUpload).GetCompletePath();
			
			// Path that the file was uploaded from
			m_strPackagePath = ((XPRequest) _xpRUpload).GetRemoteFilePath();

			if ((null == strFilePath) || (0 >= strFilePath.length()))
			{
				throw new XPException(STR_MVC_CLASS_NAME + "::LoadPackage() : Filepath null or empty");
			}

			if (0 == m_strPackagePath.compareTo(strFilePath))
			{
				// We've already loaded this package - why do it again?
				return;
			}

			m_mainModel.GetMigrationManager().InitializeForImport(strFilePath);
			m_strPackagePath = strFilePath;
			m_bPackageLoaded = true;
			log.Info("Loaded migration package into: " + m_strPackagePath);
		}
		catch (Exception e)
		{
			
			log.Error(e, STR_MVC_CLASS_NAME + "::LoadPackageFromFile(...) Unknown error loading package. e: " + e.getMessage());
			

			m_strPackagePath = "";

			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
					UIErrorScope.CURRENT_USER,
					m_asOwner.GetString(510, "ptmsgs_portalbrowsingmsgs"),
					m_asOwner.GetString(510, "ptmsgs_portalbrowsingmsgs"));
			UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
		}
	}
	
	/**
	 * Loads a package file from a known location and updates our
	 * member variables accordingly. If the initialization fails or the
	 * path is the same as the previous path, nothing happens.
	 */
	private void LoadPackageFromWeb(XPFile _xpfImport)
	{
		String strFilePath = "";
		
		try
		{
			// Upload to the directory we created the temp file into (unfortunately, the uploader will not
			// preserve the tempfile filename, so we must just a temp createfile to reserve a directory for us.
			// It hardly matters, the empty temp file will be removed when the real one is removed.

			m_strFileName = m_strWebPath.substring(m_strWebPath.lastIndexOf("/") + 1);
			GetURLContent(_xpfImport);
			strFilePath = _xpfImport.GetParent().GetFullPath() + "\\" + m_strFileName;
			
			if ((null == strFilePath) || (0 >= strFilePath.length()))
			{
				throw new XPException(STR_MVC_CLASS_NAME + "::LoadPackageFromWeb() : Filepath null or empty");
			}

			m_mainModel.GetMigrationManager().InitializeForImport(strFilePath);
			m_strPackagePath = strFilePath;
			m_bPackageLoaded = true;
			log.Info("Loaded migration package into: " + m_strPackagePath);
		}
		catch (Exception e)
		{
			m_strPackagePath = "";

			log.Error(e, STR_MVC_CLASS_NAME + "::LoadPackageFromWeb(...) Unknown error loading package. e: " + e.getMessage());			
			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
					UIErrorScope.CURRENT_USER,
					m_asOwner.GetString(510, "ptmsgs_portalbrowsingmsgs"),
					m_asOwner.GetString(510, "ptmsgs_portalbrowsingmsgs"));
			UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
		}
	}

	/**
	 * Sends HTTP request to m_strWebPath and writes response
	 * to _xpfImport location.
	 * @param _xpfImport
	 */
	private void GetURLContent(XPFile _xpfImport)
	{
		IOKHttpResponse response = SendRequest();
		if (response.GetResponseCode() == OKHttpResponseCode.OK)
		{
			log.Info(STR_MVC_CLASS_NAME + "::GetURLContent() Sent request to: " + m_strWebPath);
		}
		int intResponseCode = response.GetResponseCode();
		if ( intResponseCode != OKHttpResponseCode.OK)
		{
			log.Error(STR_MVC_CLASS_NAME + "::GetURLContent() Received response code of: " + intResponseCode);
		}
				 
		response.WriteBodyToFile(_xpfImport.GetParent().GetFullPath() + "\\" + m_strFileName); 
	}	

	 /**
	  * Uses OpenHttp to send request to m_strWebPath.
	  * Sends appropriate Basic Authentication credentials.
	  * @return IHttpResponse
	  */
	private IOKHttpResponse SendRequest()
	{
		IOKHttpResponse response = null;
			  
		try
		{
		    String sApplicationName = "portal";
		    
	        //Initialize OpenKernel, telling OpenConfig where to get it's stuff
		    String strServerConfigDir = ConfigPathResolver.GetOpenConfigPath();
	    	IOKContext configContext = OKConfigFactory.createInstance(strServerConfigDir, sApplicationName);
			PortalObjectsFactory.Init(configContext);

			IOKHttpService okservice = OKHttpFactory.getInstance(configContext);
				   
			//  OpenKernel.GetHttpService().CreateUrl(m_strWebPath);
			IOKUrl url = okservice.CreateUrl(m_strWebPath); 
			//IHttpSession session = OpenKernel.GetHttpService().CreateHttpSession();
			IOKHttpSession session = okservice.CreateHttpSession();
				   
			IOKHttpRequest request = session.CreateRequest(url);
			IOKRequestWaiter waiter = okservice.GetWaiterObject();
			
			if (m_bBasicAuth)
			{
				request.SetHttpAuthentication(m_strUsername, m_strPassword);
			}
				 
			//could do timeout here...
			request.SetTimeout(15000);
			waiter.AddRequest( request );
			request.Submit();
			waiter.Wait( 15000 );
			
			if ( OKHttpRequestState.DONE == request.GetRequestState() )
			{
				response = request.GetResponse();
			}
			else if (OKHttpRequestState.CANCELED == request.GetRequestState())
			{
				log.Error(STR_MVC_CLASS_NAME + " ::SendRequest() Request timed out.");
			}
			else if (OKHttpRequestState.ERROR == request.GetRequestState())
			{
				OKHttpException e = request.GetErrorInfo();
				if (e != null)
				{
					throw e;					
				}
			}
				 
		}
		catch (Exception e)
		{
			log.Error(e, STR_MVC_CLASS_NAME + " ::SendRequest() Unknown error sending request to " + m_strWebPath + ". e: " + e.getMessage());
			throw XPException.GetInstance(e);
		}
			  
		return response;
	}

	/**
	 * Returns filename (no path) of loaded package.
	 * @return String
	 */
	public String GetPackageFileName()
	{
		return m_strPackagePath;
	}

	/**
	 * True if a package has been successfully loaded.
	 * @return boolean
	 */
	public boolean IsPackageLoaded()
	{
		return m_bPackageLoaded;
	}
	
	/**
	 * Get package name
	 * @return
	 */
	public String GetPackageName()
	{
		if (!m_bPackageLoaded){
			return "";
		}
		
		try
		{
			return m_mainModel.GetMigrationManager().ReadUserDataItemFromPackage(PACKAGE_NAME);
		}
		catch (Exception e)
		{
			return "";
		}
	}

	/**
	 * Get package description
	 */
	public String GetPackageDescription()
	{
		if (!m_bPackageLoaded)
		{
			return "";
		}

		try
		{
			return m_mainModel.GetMigrationManager().ReadUserDataItemFromPackage(PACKAGE_DESCRIPTION);
		}
		catch (Exception e)
		{
			return "";
		}
	}

	/**
	 * Get package publisher
	 */
	public String GetPublisherName()
	{
		if (!m_bPackageLoaded){
			return "";
		}

		try
		{
			return m_mainModel.GetMigrationManager().ReadUserDataItemFromPackage(PUBLISHER_NAME);
		}
		catch (Exception e)
		{
			return "";
		}
	}
}

