package com.plumtree.portalpages.admin.ptutilities.migrationutility.exp;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.ptutilities.migrationutility.*;
import com.plumtree.portaluiinfrastructure.expandablelist.selectptobject.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tree.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * 
 * @author jennifersu
 *
 */
public class MigrationSelectForExportModel extends SelectPTObjectModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.ptutilities.migrationutility.exp.MigrationSelectForExportModel");

	public static final String STR_MVC_CLASS_NAME = "MigrationSelectForExportModel";

	/**
	 * Number total of columns
	 */
	public static final int EXPLIST_COL_MAX = 9;

	/**
	 * This column contains the Object Class IDs.
	 */
	public static final int EXPLIST_COL_FOLDER_CONTENTS = 4;

	/**
	 * This column contains the Object Class IDs.
	 */
	public static final int EXPLIST_COL_DEPENDENCIES = 5;

	/**
	 * This column contains the Object Class IDs.
	 */
	public static final int EXPLIST_COL_TYPE = 6;
	
	/**
	 * This column contains the migration submitter.
	 */
	public static final int EXPLIST_COL_SUBMITTED_BY = 7;
	
	/**
	 * This column contains the description
	 */
	public static final int EXPLIST_COL_DESCRIPTION = 8;

	/**
	 * Column widths for the various columns.
	 */
	public static final String ICON_COLUMN_WIDTH = "20";
	public static final String NAME_COLUMN_WIDTH = "60%";
	public static final String SUBMITTED_BY_COLUMN_WIDTH = "10%";
	public static final String CONTENTS_COLUMN_WIDTH = "15%";
	public static final String DEPENDENCIES_COLUMN_WIDTH = "15%";

	/**
	 * Hashtable to keep track of which objects have already
	 * been added to the list of objects to migrate.
	 * key=classid,objectid; value=name string.
	 */
	protected XPHashtable m_htObjsToMigrate;

	/**
	 * Tree Space ID. The first time the ID is empty and a new AS is opened,
	 * but after that, we save the ID and the same tree AS is reopened each time. */
	protected String m_sAdminTreeSpaceID = "";

	/**
	 * Tree Space ID. The first time the ID is empty and a new AS is opened,
	 * but after that, we save the ID and the same tree AS is reopened each time. */
	protected String m_sDirTreeSpaceID = "";

	/**
	 * Tree Space ID. The first time the ID is empty and a new AS is opened,
	 * but after that, we save the ID and the same tree AS is reopened each time. */
	protected String m_sGlobalTreeSpaceID = "";
	
	/**
	 * Current username.  Used for the SubmittedBy column.
	 */
	protected String m_sCurrentUser = "";
	
	/** 
	 * HTML dropdown value for filtering on all.
	 */
	public static final int TYPE_ADMIN_OBJ = 0;
	
	/**
	 * HTML dropdown value for filtering on admin disabled.
	 */
	public static final int TYPE_DIRECTORY = 1;

	/**
	 * HTML dropdown value for filtering on agent disabled.
	 */
	public static final int TYPE_GLOBAL = 2;

	/**
	 * Key name used by the Activity Space to store the tree. */
	public static final String SUBSPACE_KEY_ADMINTREE = "AdminObjTree";

	/**
	 * Key name used by the Activity Space to store the tree. */
	public static final String SUBSPACE_KEY_DIRTREE = "DirTree";
	
	/**
	 * Key name used by the Activity Space to store the tree. */
	public static final String SUBSPACE_KEY_GLOBALTREE = "GlobalObjTree";
	
	/**
	 * Inits member variables
	 *
	 */
	public MigrationSelectForExportModel()
	{
		super();
		
		// Want to be able to select leaf nodes and folders
		m_treeLeafType = TreeConstants.N_LEAFCOMPOSITESELECT;
		m_htObjsToMigrate = new XPHashtable();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new MigrationSelectForExportModel();
	}


	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.expandablelist.selectptobject.ISelectPTObjectModelRO#GetTreeMultiSelect()
	 */
	public String GetTreeMultiSelect()
	{
		return XPConvert.ToString(true);
	}
	
	/**
	 * "Resources to Export"
	 */
	public String GetTreeTitle() 
	{
		return m_asOwner.GetString(479,"ptmsgs_portalbrowsingmsgs");
	}
	
	/**
	 * Initializes and starts the list of 
	 * objects to export.
	 */
	public void InitSFE(IPTSession ptSession)
	{
		m_sCurrentUser = ((IPTSession)m_asOwner.GetUserSession()).GetSessionInfo().GetCurrentUserName();

		InitializeSelectForExport(m_asOwner.GetString(480, "ptmsgs_portalbrowsingmsgs"),		
			m_asOwner.GetString(481, "ptmsgs_portalbrowsingmsgs"),
			PTImageType.ACTION_ADD,
			m_asOwner.GetString(482, "ptmsgs_portalbrowsingmsgs"),
			PT_MIGRATION_CLASSES.SUPPORTED_CLASSES, 999);

		SetEmptyListMessages(m_asOwner.GetString(483, "ptmsgs_portalbrowsingmsgs"),
			m_asOwner.GetString(1894, "ptmsgs_portalbrowsingmsgs"));
			
		
		// Can only export objects that the user has at least
		// EDIT rights to.
		SetAccessLevel(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT);
		
		SetAddIcon(PTImageType.ICON_RESOURCE);
	}
	
	/**
	 * Initializes the expandable list.
	 *
	 * @param _sSectionTitle Main title.
	 * @param _sSectionSubTitle Sub title
	 * @param _imgAddIcon Icon for the add action.
	 * @param _sAddLabel Text to display next to the add icon.
	 * @param _arrObjectClassIDs Class ID of the Objects displayed on the list.
	 * @param _nListKey In case you have multiple Expandable Lists on
	 * the same DP, they should have a different key (otherwise, just set it to 1).
	 */
	public void InitializeSelectForExport(String _sSectionTitle,
		String _sSectionSubTitle, PTImageType _imgAddIcon, String _sAddLabel,
		int[] _arrObjectClassIDs, int _nListKey)
	{
		Object[][] arrExpListColInfo = new Object[4][5];
		arrExpListColInfo[0][EXPLIST_COLUMN_ID]			    = new Integer(EXPLIST_COL_STRING);
		arrExpListColInfo[0][EXPLIST_COLUMN_TITLE]		    = _sSectionSubTitle;
		arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH]		    = NAME_COLUMN_WIDTH;
		arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN]	    = "2";
		arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT]     = Boolean.TRUE;

		arrExpListColInfo[1][EXPLIST_COLUMN_ID]			    = new Integer(EXPLIST_COL_SUBMITTED_BY);
		arrExpListColInfo[1][EXPLIST_COLUMN_TITLE]		    = m_asOwner.GetString(26, "ptmsgs_infrastructure");
		arrExpListColInfo[1][EXPLIST_COLUMN_WIDTH]		    = SUBMITTED_BY_COLUMN_WIDTH;
		arrExpListColInfo[1][EXPLIST_COLUMN_COLSPAN]	    = "1";
		arrExpListColInfo[1][EXPLIST_COLUMN_ENABLESORT]     = Boolean.FALSE;

		arrExpListColInfo[2][EXPLIST_COLUMN_ID]			    = new Integer(EXPLIST_COL_FOLDER_CONTENTS);
        arrExpListColInfo[2][EXPLIST_COLUMN_TITLE]		    = m_asOwner.GetString(2790, "ptmsgs_portaladminmsgs");
		arrExpListColInfo[2][EXPLIST_COLUMN_WIDTH]		    = CONTENTS_COLUMN_WIDTH;
		arrExpListColInfo[2][EXPLIST_COLUMN_COLSPAN]	    = "1";
		arrExpListColInfo[2][EXPLIST_COLUMN_ENABLESORT]     = Boolean.FALSE;

		arrExpListColInfo[3][EXPLIST_COLUMN_ID]			    = new Integer(EXPLIST_COL_DEPENDENCIES);
        arrExpListColInfo[3][EXPLIST_COLUMN_TITLE]		    = m_asOwner.GetString(2791, "ptmsgs_portaladminmsgs");
		arrExpListColInfo[3][EXPLIST_COLUMN_WIDTH]		    = DEPENDENCIES_COLUMN_WIDTH;
		arrExpListColInfo[3][EXPLIST_COLUMN_COLSPAN]	    = "1";
		arrExpListColInfo[3][EXPLIST_COLUMN_ENABLESORT]     = Boolean.FALSE;

		m_arrObjectClassIDs = _arrObjectClassIDs;

		super.InitializeExpandableList(_sSectionTitle, arrExpListColInfo,
			_imgAddIcon, _sAddLabel, _nListKey);
	}

	/**
	 * Initalizes the list (PTGrowableSortedArrayWrapper)
	 * with a column mapping and an array of sort.
	 * The growable array will always be sorted by the 1st
	 * column ascending.
	 *
	 * @param _saData
	 */
	public void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
	{
		// JF- Column mapping
		int[] arrColumns = new int[]
			{
				EXPLIST_SORTEDARRAY_PROPID_INDEX, EXPLIST_COL_CLASSID,
				EXPLIST_COL_OBJECTID, EXPLIST_COL_STRING, EXPLIST_COL_FOLDER_CONTENTS, 
				EXPLIST_COL_DEPENDENCIES, EXPLIST_COL_TYPE, EXPLIST_COL_SUBMITTED_BY, EXPLIST_COL_DESCRIPTION
			};

		int[][] arrSort = new int[2][2];
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_COL_CLASSID;
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
		arrSort[1][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_COL_STRING;
		arrSort[1][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;

		// JF- We don't initalize the list itself (null below)
		// At first it's empty and we will be initalized if needed using the 
		// AddRowsToList method.
		_saData.Initialize(arrColumns, null, arrSort, true,
			EXPLIST_DEFAULT_INITIAL_SIZE);
	}

	/**
	 * Loads the list with objects that get uploaded from the
	 * Add Existing Package feature.
	 * 
	 */
	public void LoadResources()
	{
		IMigrationUtilityModelRO mainModel = (IMigrationUtilityModelRO)m_asOwner.GetModel(((EditorAS)m_asOwner).m_strModelName);
		XPArrayList[] alResources = mainModel.GetAllObjectsInPackage();		
		
		// Iterate through each object in the package.  If the UUID is already
		// detected in the current list of objects to export, skip it.  
		// If the UUID can be found
		// in the local DB, that means the object exists and is available to be
		// exported.  If that's the case, go ahead and add it to the list of
		// resources to be exported.  If it can't be found, then it cannot be
		// exported, so we skip over it and log a warning message.
		IPTMigrationManager migMgr = mainModel.GetMigrationManager();
		for (int indClass = 0; indClass < alResources.length; indClass++)
		{
			XPArrayList xpalClassObjs = alResources[indClass];
			
			int nClassID = PT_MIGRATION_CLASSES.SUPPORTED_CLASSES[indClass];
			
			for (int indObj = 0; indObj < xpalClassObjs.GetSize(); indObj++)
			{
				IPTMigrationElement ptME = (IPTMigrationElement)xpalClassObjs.GetElement(indObj);
				
				int nObjectID = migMgr.ResolveObject(ptME, true);
				if (nObjectID != PT_INTRINSICS.PT_NULL_OBJECTID)
				{				
					String strUniqueString = nClassID + "," + nObjectID;
					if (null != m_htObjsToMigrate.GetElement(strUniqueString))
					{
						log.Debug(ptME.GetName() + " already added to package. Skipping...");
						continue;
					} 
					else
					{
						m_htObjsToMigrate.PutElement(strUniqueString, ptME.GetName());
                        
                        if (ptME.GetClassID() == PT_CLASSIDS.PT_EXPRESSION_ID && ptME.ReadSimpleElementAsInt("TOPICPOS") == 0) {
                            // DCA - subexpressions are not shown on the list
                            continue;
                        } else {
    						Object[] aRowToAdd = new Object[EXPLIST_COL_MAX];
    						aRowToAdd[EXPLIST_COL_CLASSID] = new Integer(nClassID);
    						aRowToAdd[EXPLIST_COL_OBJECTID] = new Integer(nObjectID);
    						aRowToAdd[EXPLIST_COL_STRING] = ptME.GetName();
    						aRowToAdd[EXPLIST_COL_FOLDER_CONTENTS] = new Boolean(false);
    						aRowToAdd[EXPLIST_COL_DEPENDENCIES] = new Boolean(false);
    						if (IsAdminObject(nClassID))
    						{
    							aRowToAdd[EXPLIST_COL_TYPE] = new Integer(TYPE_ADMIN_OBJ);
    						} else if (IsDirObject(nClassID))
    						{
    							aRowToAdd[EXPLIST_COL_TYPE] = new Integer(TYPE_DIRECTORY);
    						}
    						else
    						{
    							aRowToAdd[EXPLIST_COL_TYPE] = new Integer(TYPE_GLOBAL);							
    						}
    						aRowToAdd[EXPLIST_COL_SUBMITTED_BY] = m_sCurrentUser;			
    						String strDesc;
    						try
    						{
    							strDesc = ptME.GetDescription();						
    						}
    						catch (Exception e)
    						{
    							// This object probably doesn't support description
    							// (Global maps, etc)
    							strDesc = "";
    						}
    						aRowToAdd[EXPLIST_COL_DESCRIPTION] = strDesc;
    						
    						GetItems().AddRow(aRowToAdd);
    						log.Info("Found object to add to export package: " + ptME.GetName());
                        }
					}
				}
				else
				{
					log.Warn("Could not find object to add to export package: " + ptME.GetName() + ". Skipping...");
				}
			}
		}

	}

	/**
	 * @see com.plumtree.uiinfrastructure.editor.EditorHelperModel#SaveSection(XPHashtable)
	 */
	public int SaveSection(XPHashtable _htFormData)
	{
		int nValid = super.SaveSection(_htFormData);
		
		for (int indRow = 0; indRow < GetItems().GetCount(); indRow++)
		{
			GetItems().SetItem(indRow, EXPLIST_COL_FOLDER_CONTENTS, new Boolean(false));
			GetItems().SetItem(indRow, EXPLIST_COL_DEPENDENCIES, new Boolean(false));
		}
		
		String sDataKey     = GetUniqueString(MigrationSelectForExportView.HTMLCHECKBOX_FOLDERCONTENTS);
		String[] aDataValue;
		int intRow;

		if (_htFormData.ContainsKey(sDataKey))
		{
			aDataValue = (String[]) _htFormData.GetElement(sDataKey);

			if ((null != aDataValue) && !(0 == aDataValue.length))
			{
				for (int indValue = 0; indValue < aDataValue.length; indValue++)
				{
					intRow = XPConvert.ToInteger(aDataValue[indValue]);
					GetItems().SetItem(intRow, EXPLIST_COL_FOLDER_CONTENTS, new Boolean(true));
				}
			}
		}

		sDataKey     = GetUniqueString(MigrationSelectForExportView.HTMLCHECKBOX_DEPENDENCIES);
		if (_htFormData.ContainsKey(sDataKey))
		{
			aDataValue = (String[]) _htFormData.GetElement(sDataKey);

			if ((null != aDataValue) && !(0 == aDataValue.length))
			{
				for (int indValue = 0; indValue < aDataValue.length; indValue++)
				{
					intRow = XPConvert.ToInteger(aDataValue[indValue]);
					GetItems().SetItem(intRow, EXPLIST_COL_DEPENDENCIES, new Boolean(true));
				}
			}
		}

		return nValid; 
	}
	
	/**
	 * Saves the data for columns other than the plumtree object select column
	 * on the row _nRowIndex.
	 *
	 * @param _saData List to update the data on row _nRowIndex
	 * @param _nRowIndex Index of the row to save.
	 * @param _htFormData Data posted by the form.
	 * @return int Status for server side validation.
	 */
	protected int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
		XPHashtable _htFormData)
	{
		return RepostControl.PAGE_STATUS_VALID;
	}

    // mel: Fix for bug# 45542
    /**
     * @return The TreeAS corresponding to the subSpaceKey or null if none
     */
    private TreeAS SafeGetSubSpace(String subSpaceKey)
    {
        if(subSpaceKey == null)
        {
            return null;
        }
        String uniqueString = GetUniqueString(subSpaceKey);
        TreeAS asTree = null;
        if(m_asOwner.GetSubSpaceExists(uniqueString))
        {
            asTree = (TreeAS) m_asOwner.GetSubSpace(uniqueString);
        }
        return asTree;
    }

	/**
	 * Clear the list and add all the values from the tree.
	 *
	 * @param _htFormData Form data
	 */
	public void AddRowsOnRepost(XPHashtable _htFormData)
	{
		if (!m_bIsReadOnly)
		{
			SaveTreeID(_htFormData);

            // TODO The subspace names and "TYPE_" should be merged since they represent the same data, a pseudo-enum would do nicely
            AddTreeSelectionToList(SUBSPACE_KEY_ADMINTREE, TYPE_ADMIN_OBJ);
            AddTreeSelectionToList(SUBSPACE_KEY_DIRTREE, TYPE_DIRECTORY);
            AddTreeSelectionToList(SUBSPACE_KEY_GLOBALTREE, TYPE_GLOBAL);
		}
	}

    private void AddTreeSelectionToList(String subspaceTreeKey, int objectType)
    {
        TreeAS asTree = SafeGetSubSpace(subspaceTreeKey);
        if(asTree == null)
        {
            // If the tree picker for this subspace (admin,dir,global)
            // has not been initialized yet, then we can't be selecting
            // items in it.
            return;
        }
        ITreeModelRO myTree    = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
        XPArrayList mySelected = myTree.GetSelectedItems();
        Object[] oNewRow;
        int nClassID;
        int nObjectID;
        int nIndex;

        Object[][] arrIndexMapping = CreateIndexMapping(GetItems());

        for (int i = 0; i < mySelected.GetSize(); i++)
        {
            nClassID	  = ((TreeLeafNode) mySelected.GetElement(i)).m_nClassID;
            nObjectID     = ((TreeLeafNode) mySelected.GetElement(i)).m_nObjectID;
            nIndex		  = GetIndex(arrIndexMapping, nClassID,
                                       nObjectID);

            if (-1 == nIndex)
            {
                nIndex = m_nNextIndex;
                m_nNextIndex++;
            }

            oNewRow										  = new Object[EXPLIST_COL_MAX];
            oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX]     = new Integer(nIndex);
            oNewRow[EXPLIST_COL_CLASSID]				  = new Integer(nClassID);
            oNewRow[EXPLIST_COL_OBJECTID]				  = new Integer(nObjectID);

            // For PT_CLASSIDS.PT_CATALOGFOLDER_ID == nClassID)
            // we could use
            // oNewRow[EXPLIST_COL_STRING] = ((TreeLeafNode) mySelected.GetElement(i)).m_strLocation;
            // to get a path but that looks bad.
            oNewRow[EXPLIST_COL_STRING] = ((TreeLeafNode) mySelected.GetElement(i)).GetName();

            oNewRow[EXPLIST_COL_FOLDER_CONTENTS] = new Boolean(false);
            oNewRow[EXPLIST_COL_DEPENDENCIES] = new Boolean(false);
            oNewRow[EXPLIST_COL_TYPE] = new Integer(objectType);
            oNewRow[EXPLIST_COL_SUBMITTED_BY] = m_sCurrentUser;

            AddRowToList(oNewRow, nClassID, nObjectID);
        }
    }

    private void AddRowToList(Object[] rowObjectArray, int classID, int objectID)
    {
        String strUniqueString = classID + "," + objectID;
        if(!m_htObjsToMigrate.ContainsKey(strUniqueString))
        {
            m_htObjsToMigrate.PutElement(strUniqueString, rowObjectArray[EXPLIST_COL_STRING]);
            GetItems().AddRow(rowObjectArray);
        }
    }

    private void RemoveRowFromList(int classID, int objectID)
    {
        String strUniqueString = classID + "," + objectID;
        m_htObjsToMigrate.RemoveElement(strUniqueString);


        // mel: Fix for 42066, ensure the objects are removed from the selection
        // otherwise we'll just re-add them later.

        // mel: This remove method has no effect if the object does not
        // exist or is not selected.
        // When the remove button is clicked, we don't know which tree it applies to,
        // so we have to remove from each of the trees and rely on
        // RemoveItemFromTreeSelection ignoring remove on non-existent objects.
        RemoveItemFromTreeSelection(SUBSPACE_KEY_ADMINTREE, classID, objectID);
        RemoveItemFromTreeSelection(SUBSPACE_KEY_DIRTREE, classID, objectID);
        RemoveItemFromTreeSelection(SUBSPACE_KEY_GLOBALTREE, classID, objectID);
    }

    private void RemoveItemFromTreeSelection(String subspaceTreeKey, int classID, int objectID)
    {
        TreeAS asTree = SafeGetSubSpace(subspaceTreeKey);
        if (null != asTree)
        {
            ITreeModel myTree = (ITreeModel) asTree.GetModel(ATreeAS.TREEMODEL);
            myTree.RemoveFromSelection(classID, objectID);
        }
    }

    // This override is a quick way to see where super.m_saData and super.GetItems are being used.
//    public PTGrowableSortedArrayWrapper GetItems()
//    {
//        return super.GetItems();
//    }


	/**
	 * Gets the array of checked rows and delete  them from the list.
	 *
	 * @param _htFormData
	 */
	public void DeleteRowsOnRepost(XPHashtable _htFormData)
	{
		if (!m_bIsReadOnly)
		{
			String sDataKey = GetUniqueString(ExpListView.HTMLCHECKBOX_DELROW);

			// JF- We get back the list of index of rows to delete.
			if (_htFormData.ContainsKey(sDataKey))
			{
				int[] aRowsToDelete = GetArrayOfObjToDelete(_htFormData);

				// JF- We can only delete rows if the array is not empty
				if (GetItems().GetCount() > 0)
				{
					for (int indRow = 0; indRow < aRowsToDelete.length; indRow++)
					{
						int nClassID = XPConvert.ToInteger(GetItems().GetItem(aRowsToDelete[indRow], EXPLIST_COL_CLASSID));
						int nObjectID = XPConvert.ToInteger(GetItems().GetItem(aRowsToDelete[indRow], EXPLIST_COL_OBJECTID));
                        RemoveRowFromList(nClassID, nObjectID);
					}
                    // mel: Must remove all at once. Removing one at a time will
                    // cause indices to become invalid and incorrect items removed.
                    GetItems().RemoveRows(aRowsToDelete);
				}
			}
		}
	}

	
	/**
	 * Adds all approved objects to list of resources to export.
	 *
	 */
	public void AddAllApproved()
	{
		//TODO: The queries here could probably be optimized...
			
		IMigrationUtilityModelRO mainModel = (IMigrationUtilityModelRO)m_asOwner.GetModel(((EditorAS)m_asOwner).m_strModelName);
		IPTMigrationManager migMgr = mainModel.GetMigrationManager();
		Object[] aRowToAdd;

		// Query the number of approved objects by class
		IPTQueryResult ptQRCount = migMgr.QueryObjectsByStatusCount(PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVED, null);

		// Find the last row index so we can assign
		// row indices correctly
		int nRowIndex = GetItems().GetCount();
		
		// Loop through each class and add objects
		for (int indClass = 0; indClass < ptQRCount.RowCount(); indClass++)
		{
			int nClassID    = ptQRCount.ItemAsInt(indClass, PT_PROPIDS.PT_PROPID_CLASSID);
			int nNumObjects = ptQRCount.ItemAsInt(indClass, PT_PROPIDS.PT_PROPID_OBJECTCOUNT);

			// Only query the DB if necessary
			if (nNumObjects > 0)
			{
				// Query all approved objects of the current class
				IPTQueryResult ptQR = migMgr.QueryObjectsByStatus(nClassID, PT_MIGRATION_STATUS_TYPES.PT_MIGRATION_STATUS_APPROVED, 0, -1, null);

				for (int indObj = 0; indObj < ptQR.RowCount(); indObj++)
				{
					int nObjectID         = ptQR.ItemAsInt(indObj, PT_PROPIDS.PT_PROPID_OBJECTID);
					String strName = ptQR.ItemAsString(indObj, PT_PROPIDS.PT_PROPID_NAME);	
		            String strSubmittedBy = ptQR.ItemAsString(indObj, PT_PROPIDS.PT_PROPID_MIGRATION_REQUESTOR);
		            
					// Don't load duplicates
					String strUniqueString = nClassID + "," + nObjectID;
					if (null != m_htObjsToMigrate.GetElement(strUniqueString))
					{
						log.Debug(strName + " already added to package. Skipping...");
						continue;
					}
					else
					{
						m_htObjsToMigrate.PutElement(strUniqueString, strName);
						aRowToAdd = new Object[EXPLIST_COL_MAX];
						aRowToAdd[EXPLIST_SORTEDARRAY_PROPID_INDEX] = new Integer(nRowIndex);
						aRowToAdd[EXPLIST_COL_CLASSID] = new Integer(nClassID);
						aRowToAdd[EXPLIST_COL_OBJECTID] = new Integer(nObjectID);
						aRowToAdd[EXPLIST_COL_STRING] = strName;
						aRowToAdd[EXPLIST_COL_FOLDER_CONTENTS] = new Boolean(false);
						aRowToAdd[EXPLIST_COL_DEPENDENCIES] = new Boolean(false);
						if (IsAdminObject(nClassID))
						{
							aRowToAdd[EXPLIST_COL_TYPE] = new Integer(TYPE_ADMIN_OBJ);
						} else if (IsDirObject(nClassID))
						{
							aRowToAdd[EXPLIST_COL_TYPE] = new Integer(TYPE_DIRECTORY);
						}
						else
						{
							aRowToAdd[EXPLIST_COL_TYPE] = new Integer(TYPE_GLOBAL);							
						}
						aRowToAdd[EXPLIST_COL_SUBMITTED_BY] = strSubmittedBy;

						GetItems().AddRow(aRowToAdd);
						nRowIndex++;

						log.Info("Adding " + strName + ", ObjectID " + nObjectID);
					}
				}
			}
		}
	}

    // mel: Fix for 45544, error: No object manager for this class (6) (PT_CLASSIDS.PT_ACTIVITY_ID).
    // Note that activities are handled via the GlobalObjsTreeModel, see GetGlobalTreeSpaceID
    //
    // mel: Fix for 45812, agents (intrinsic jobs) should not be exported, e.g. Card Refresh Agent
    // Excluding all global objects is acceptable because AdminTreeModel,
    // when displaying PT_GLOBALOBJECT_ID, only
    // displays PT_GLOBALOBJECTS.PT_GLOBAL_SCHEDULED_AGENTS and no others.
    private boolean IsExcludedAdminObjectClassID(int migrationClassID)
    {
        // Don't want to dislpay PTGuestLogin (PT_CLASSIDS.PT_GUESTLOGIN_ID)
        if(migrationClassID == PT_CLASSIDS.PT_ACTIVITY_ID
           || migrationClassID == PT_CLASSIDS.PT_GUESTLOGIN_ID
           || migrationClassID == PT_CLASSIDS.PT_GLOBALOBJECT_ID)
        {
            return true;
        }
        return false;
    }

	/**
	 * Returns String of Class ID of the Objects
	 * displayed in the tree.
	 * E.g.: "2,45,3"
	 *
	 * This is the format used by the tree.
	 *
	 * @return String
	 */
	public String GetAdminObjectClassIDs()
	{
		String sClassIDs = "";

		// JF- We loop through the array of IDs and add each ID 
		// to the String.
		for (int i = 0; i < PT_MIGRATION_CLASSES.ADMIN_OBJ_CLASSES.length; i++)
		{
            int migrationClassID = PT_MIGRATION_CLASSES.ADMIN_OBJ_CLASSES[i];
            if(!IsExcludedAdminObjectClassID(migrationClassID))
            {
                if (0 != sClassIDs.length())
                {
                    sClassIDs += ",";
                }
		    	sClassIDs += (XPConvert.ToString(migrationClassID));
            }
		}

		return sClassIDs;
	}

	/**
	 * Returns String of Class ID of the Objects
	 * displayed in the tree.
	 * E.g.: "2,45,3"
	 *
	 * This is the format used by the tree.
	 *
	 * @return String
	 */
	public String GetDirectoryClassIDs()
	{
		String sClassIDs = "";

		// JF- We loop through the array of IDs and add each ID 
		// to the String.
		for (int i = 0; i < PT_MIGRATION_CLASSES.KD_CLASSES.length; i++)
		{
			if (0 != sClassIDs.length())
			{
				sClassIDs += ",";
			}

			sClassIDs += (XPConvert.ToString(PT_MIGRATION_CLASSES.KD_CLASSES[i]));
		}

		return sClassIDs;
	}
	
	/**
	 * Helper method to save the Tree ID on repost.
	 *
	 * @param _htFormData Form data
	 */
	protected void SaveTreeID(XPHashtable _htFormData)
	{
		super.SaveTreeID(_htFormData);
		
		// JF- We save the tree ID only if we don't already have it.
		if (("").equals(m_sAdminTreeSpaceID) &&
				_htFormData.ContainsKey(GetUniqueString(
						MigrationSelectForExportView.HTMLINPUT_ADMIN_TREEASID)))
		{
			String[] data = (String[]) _htFormData.GetElement(GetUniqueString(
				MigrationSelectForExportView.HTMLINPUT_ADMIN_TREEASID));

			if ((null != data) && !(0 == data.length))
			{
				m_sAdminTreeSpaceID = data[0];
			}
		}

		if (("").equals(m_sDirTreeSpaceID) &&
				_htFormData.ContainsKey(GetUniqueString(
						MigrationSelectForExportView.HTMLINPUT_DIR_TREEASID)))
		{
			String[] data = (String[]) _htFormData.GetElement(GetUniqueString(
				MigrationSelectForExportView.HTMLINPUT_DIR_TREEASID));

			if ((null != data) && !(0 == data.length))
			{
				m_sDirTreeSpaceID = data[0];
			}
		}

		if (("").equals(m_sGlobalTreeSpaceID) &&
				_htFormData.ContainsKey(GetUniqueString(
						MigrationSelectForExportView.HTMLINPUT_GLOBAL_TREEASID)))
		{
			String[] data = (String[]) _htFormData.GetElement(GetUniqueString(
				MigrationSelectForExportView.HTMLINPUT_GLOBAL_TREEASID));

			if ((null != data) && !(0 == data.length))
			{
				m_sDirTreeSpaceID = data[0];
			}
		}
	}
	
	/**
	 * Returns tree AS ID.
	 *
	 * @return String
	 */
	public String GetAdminTreeSpaceID()
	{
		return m_sAdminTreeSpaceID;
	}
	
	/**
	 * Returns tree AS ID.
	 *
	 * @return String
	 */
	public String GetDirectoryTreeSpaceID()
	{
		return m_sDirTreeSpaceID;
	}

	/**
	 * Returns tree AS ID.
	 *
	 * @return String
	 */
	public String GetGlobalTreeSpaceID()
	{
		return m_sGlobalTreeSpaceID;
	}
	
	public boolean IsAdminObject(int _intClassID)
	{
		if (IsDirObject(_intClassID))
		{
			return false;
		} else if (IsGlobalObject(_intClassID))
		{
			return false;
		}
		return true;
	}
    
	public boolean IsDirObject(int _intClassID)
	{ 
		for (int indClass = 0; indClass < PT_MIGRATION_CLASSES.KD_CLASSES.length; indClass++)
		{
			if (_intClassID == PT_MIGRATION_CLASSES.KD_CLASSES[indClass])
			{
				return true;
			}
		}
		
		return false;
	}
	
	public boolean IsGlobalObject(int _intClassID)
	{
		switch (_intClassID)
		{
			case PT_CLASSIDS.PT_GLOBALOBJECT_ID :
			case PT_CLASSIDS.PT_ACTIVITY_ID :
			case PT_CLASSIDS.PT_EXPRESSION_ID :
				return true;
			default:
				return false;
		}
	}
}

