package com.plumtree.portalpages.admin.ptutilities.userinfopropmap;


// Framework Imports
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.editors.profilesource.*;
import com.plumtree.portaluiinfrastructure.ptmulticolumnlist.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;


/**
 * This class keeps an up-to-date state of all the settings.
 *
 * It contains the set and get methods for those settings.
 *
 * It contains the start, save and finish methods.
 *
 * This class is used by the Section class (to get the latest value of the settings
 * to display) and by the Repost Control class (to perform the repost actions).
 *
 * @author JohnH
 */
public class UserInfoPropMapModel extends EditorModel
	implements IUserInfoPropMapModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.ptutilities.userinfopropmap.UserInfoPropMapModel");

	/** The unique identifier for the class */
	public static final String STR_MVC_CLASS_NAME = "UserInfoPropMapModel";

	//Constants
	/** The number of columns (other than the Plumtree object column) in the multi-column list */
	public static final int MAPLIST_NUMCOLUMNS		 = 1;

	/** Zero-based list position for the PT object column */
	public static final int MAPLIST_PTOBJECTPOSITION = 0; 

	/** Server object: Plumtree Session */
	private IPTSession m_Session;

	//Member variables
	/** 2 dim array 0,0 = PropertyID, 1,0 = UserInfo element */
	private Object[][] m_arrPropMapInfo;
	/** The column to sort the multi-column list on */
	private int m_intSortColumn						 = 0;
	/** The sort direction */
	private int m_intSortDirection					 = 0;

	/**
	 * Implements some actions to perform on start.
	 *
	 * @param _htFormData
	 *
	 * @return boolean Is true if the current user has the rights to access this Editor.
	 */
	public boolean CheckEditorAccessAndStart(XPHashtable _htFormData)
	{
		boolean bReturn = (m_Session.GetSessionInfo().CheckForActivityRight(
					PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_EDIT_USERINFO_MAP));

		if (bReturn)
		{
			StartUserInfoPropMapList();

			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Public default constructor
	 */
	public UserInfoPropMapModel()
	{
		super();
		m_intSortColumn						 = PT_PROPIDS.PT_PROPID_NAME;
		m_intSortDirection					 = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new UserInfoPropMapModel();
	}

	/**
	 * Implement some code to execute when the user clicks on "Finish".
	 *
	 */
	public void Finish()
	{
		//m_arrPropMapInfo
		//2 dim array 0,0 = PropertyID, 1,0 = UserInfo element
		try
		{
			if (null != m_arrPropMapInfo)
			{
				//Create a new, potentially larger array containing the
				//parsed out userinfo elements
				Object[][] myArray;
				int intCount = 0;

				//First, just get the proper size of the temporary array
				for (int i = 0; i < m_arrPropMapInfo[0].length; i++)
				{
					Object[][] arrTemp = PTMultiColumnListModel.ParseFields((String) m_arrPropMapInfo[1][i]);
					intCount = intCount + arrTemp[0].length;
				}

				myArray     = new Object[2][intCount];

				//Now cycle through again, this time populating the temporary
				//array
				intCount = 0;

				for (int i = 0; i < m_arrPropMapInfo[0].length; i++)
				{
					Object[][] arrTemp = PTMultiColumnListModel.ParseFields((String) m_arrPropMapInfo[1][i]);

					for (int j = 0; j < arrTemp[0].length; j++)
					{
						myArray[0][intCount]     = m_arrPropMapInfo[0][i]; //Set the property ID
						myArray[1][intCount]     = arrTemp[0][j]; //Set the userinfo string
						intCount				 = intCount + 1; //Update the counter
					}
				}

				IPTProfileManager myProfileManager = (IPTProfileManager) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_PROFILE_MANAGER,
						true);

				if (null != m_arrPropMapInfo)
				{
					myProfileManager.SetUserInfoMap(myArray);
				}
			}
		}
		catch (Exception e)
		{
			GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Admin"), "UserInfoPropMapModel::Finish", 
				e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(1584,"ptmsgs_portaladminmsgs"));
		}
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	// BEGIN ***************** helper methods *****************
	// END ***************** helper methods *****************

	/**
	 * Inits the Model with the IPTSession.
	 *
	 * @param parent
	 */
	public void Init(AActivitySpace parent)
	{
		super.Init(parent);

		// JF- Get the Session object from the associated AS.
		m_Session = (IPTSession) parent.GetUserSession();
	}

	/**
	 * Calls the right save function according to the page name passed in argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		int nPageStatus = UserInfoPropMapRepostControl.PAGE_STATUS_VALID;

		// Save the items to be unlocked
		if (_sPageName.equals(MainDP.STR_MVC_CLASS_NAME))
		{
			nPageStatus = SaveMainPage(_htFormData);
		}

		return nPageStatus;
	}

	/**
	 * Initializes the multi-column list with the standard intrinsic list of properties
	 */
	public void StartUserInfoPropMapList()
	{
		UserInfoPropMapListModel myListModel = (UserInfoPropMapListModel) m_asOwner.GetModel(UserInfoPropMapAS.STR_MVC_STORAGE_NAME_PLUG_IN);

		//Set the number of columns.
		//Login form page
		myListModel.SetNumColumns(MAPLIST_NUMCOLUMNS);
		myListModel.SetIncludePTSelect(true);
		myListModel.SetPTObjectPosition(MAPLIST_PTOBJECTPOSITION);

		String[] arrPropMapSubTitles = new String[MAPLIST_NUMCOLUMNS];

		arrPropMapSubTitles[0] = m_asOwner.GetString(1579,
				"ptmsgs_portaladminmsgs");

		myListModel.InitializePTMultiColumnList(m_asOwner.GetString(1577,
				"ptmsgs_portaladminmsgs"),
			
		m_asOwner.GetString(1578, "ptmsgs_portaladminmsgs"),
			arrPropMapSubTitles, PTImageType.ICON_PROPERTY,
			
		m_asOwner.GetString(1580, "ptmsgs_portaladminmsgs"),
			new int[] { PT_CLASSIDS.PT_PROPERTY_ID }, 1);

		// Use the method below to set the value to populate a newly created input text.
		// (By default it will be a null array, so it MUST be initialized.
		//Login form page
		String[] arrPropMapDefaults = new String[MAPLIST_NUMCOLUMNS];

		//get just the user properties
		String userProperties = ProfileSourceModel.GetUserProperties(m_Session);
		
		arrPropMapDefaults[0] = "";
		myListModel.SetDefaultValue(arrPropMapDefaults);
		myListModel.SetDisplayObjects(userProperties);
		myListModel.SetRowIcon(PTImageType.ICON_PROPERTY);
		myListModel.SetTreeMultiSelect(true);
		myListModel.SetTreeSubTitle(m_asOwner.GetString(539,
				"ptmsgs_portaladminmsgs"));
		myListModel.SetTreeTitle(m_asOwner.GetString(538,
				"ptmsgs_portaladminmsgs"));

		//Set the message and instructions to display when the list is empty.
		myListModel.SetEmptyListMessages(m_asOwner.GetString(1581,
				"ptmsgs_portaladminmsgs"),
			m_asOwner.GetString(1582, "ptmsgs_portaladminmsgs"));

		//Get the name value pairs from the model in edit mode,
		//and populate them in here
		try
		{
			IPTProfileManager ptProfileManager = (IPTProfileManager) m_Session.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_PROFILE_MANAGER,
					false);
			Object[][] arrMap;
			arrMap = (Object[][]) ptProfileManager.GetUserInfoMap();

			if (arrMap[0].length > 0)
			{
				int intCount     = 0;
				int intLastValue = 0;

				for (int k = 0; k < arrMap[0].length; k++)
				{
					if (XPConvert.ToInteger(arrMap[0][k]) != intLastValue)
					{
						intCount++;
					}

					intLastValue = XPConvert.ToInteger(arrMap[0][k]);
				}

				if (intCount > 0)
				{
					Object[][] oRows   = new Object[intCount][MAPLIST_NUMCOLUMNS +
						4];
					XPStringBuilder sb = new XPStringBuilder(200);

					//Dictionaries for the names/ids of the operations
					//Get all the possible classes that can be used as operations
					intLastValue = XPConvert.ToInteger(arrMap[0][0]);

					int intRowIndex     = 0;
					String strLastValue = "";

					for (int i = 0; i < arrMap[0].length; i++)
					{
						// get the name
						PTObjectManagerQueryWrapper omqrProperty = new PTObjectManagerQueryWrapper(m_Session.GetProperties());
						int intObjID						 =  XPConvert.ToInteger(arrMap[0][i]);
						omqrProperty.QuerySingleObject(intObjID);

						if (intObjID != intLastValue)
						{
							// Set the class ID
							oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_CLASSID +
							MAPLIST_PTOBJECTPOSITION]																	  = new Integer(PT_CLASSIDS.PT_PROPERTY_ID);

							// Set the Object ID
							oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_OBJECTID +
							MAPLIST_PTOBJECTPOSITION]																	   = new Integer(intLastValue);

							// Set the Object name
							oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_STRING +
							MAPLIST_PTOBJECTPOSITION]																	 = strLastValue;

							// Set the mapped UserInfo element
							if (MAPLIST_PTOBJECTPOSITION < MAPLIST_NUMCOLUMNS)
							{
								oRows[intRowIndex][MAPLIST_PTOBJECTPOSITION + 4]     = sb.ToString();
							}
							else
							{
								oRows[intRowIndex][MAPLIST_PTOBJECTPOSITION + 1]     = sb.ToString();
							}

							sb = new XPStringBuilder(200);
							sb.Append(arrMap[1][i]);
							intRowIndex++;
						}
						else
						{
							if (i > 0)
							{
								sb.Append(", ").Append(arrMap[1][i]);
							}
							else
							{
								sb.Append(arrMap[1][i]);
							}
						}

						intLastValue     = intObjID;
						strLastValue     = (String) omqrProperty.GetFields(0,
								PT_PROPIDS.PT_PROPID_NAME);
					}

					//Now pick up the last row
					// Set the class ID
					oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_CLASSID +
					MAPLIST_PTOBJECTPOSITION]																	  = new Integer(PT_CLASSIDS.PT_PROPERTY_ID);

					// Set the Object ID
					oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_OBJECTID +
					MAPLIST_PTOBJECTPOSITION]																	   = new Integer(intLastValue);

					// Set the Object name
					oRows[intRowIndex][PTMultiColumnListModel.EXPLIST_COL_STRING +
					MAPLIST_PTOBJECTPOSITION]																	 = strLastValue;

					// Set the mapped UserInfo element
					if (MAPLIST_PTOBJECTPOSITION < MAPLIST_NUMCOLUMNS)
					{
						oRows[intRowIndex][MAPLIST_PTOBJECTPOSITION + 4]     = sb.ToString();
					}
					else
					{
						oRows[intRowIndex][MAPLIST_PTOBJECTPOSITION + 1]     = sb.ToString();
					}

					// Then add the array to the list
					myListModel.SetDefaultObjects(oRows);
				}
			}
		}
		catch (Exception e)
		{
			GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Admin"), "UserInfoPropMapModel::StartUserInfoPropMapList", 
				e, true, UIErrorLevel.SERIOUS, m_asOwner.GetString(2426,"ptmsgs_portaladminmsgs"));
		}
	}

	/**
	 * Saves the data on the main page.
	 *
	 * @param _htFormData
	 * @return int Page status for server side validation
	 */
	private int SaveMainPage(XPHashtable _htFormData)
	{
		int intPageStatus = UserInfoPropMapRepostControl.PAGE_STATUS_VALID;
		try
		{
			//First, save the existing section to the activity space
			((UserInfoPropMapListModel) m_asOwner.GetModel(UserInfoPropMapAS.STR_MVC_STORAGE_NAME_PLUG_IN)).SaveSection(_htFormData);

			//Now retrieve the non Plumtree select elements
			Object[][] arrMappingInfo = ((UserInfoPropMapListModel) m_asOwner.GetModel(UserInfoPropMapAS.STR_MVC_STORAGE_NAME_PLUG_IN)).GetListValues();

			//m_arrPropMapInfo
			//2 dim array 0,0 = PropertyID, 1,0 = UserInfo element
			PTGrowableSortedArrayWrapper arrItems;
			arrItems = ((UserInfoPropMapListModel) m_asOwner.GetModel(UserInfoPropMapAS.STR_MVC_STORAGE_NAME_PLUG_IN)).GetItems();

			int intNumItems = arrItems.GetCount();

			if (intNumItems > 0)
			{
				m_arrPropMapInfo = new Object[2][intNumItems];
				XPHashtable htDupChecker = new XPHashtable();
				Object strMappedValue;
				
				for (int i = 0; i < intNumItems; i++)
				{
					//Validate first that there is not an empty string mapped
					if ( ((String) arrMappingInfo[0][i]).equals("") )
					{
						//Raise an error and return invalid
						UIError myError = new UIError(UIErrorLevel.FRIENDLY,

						UIErrorScope.CURRENT_USER,
							null,
							m_asOwner.GetString(2157, "ptmsgs_portaladminmsgs"),
							m_asOwner.GetString(2157, "ptmsgs_portaladminmsgs"));
						UIErrorHandler.RaiseError(
							myError,
							m_asOwner.GetSubSession(),
							m_asOwner.GetApplication());

						return UserInfoPropMapRepostControl.PAGE_STATUS_INVALID;
					}
					
					//Now validate that the userinfo value hasn't yet been mapped to another property
					strMappedValue = arrMappingInfo[0][i];
					
					if (htDupChecker.ContainsKey(strMappedValue))
					{
						//Raise an error and return invalid
						UIError myError = new UIError(UIErrorLevel.FRIENDLY,

						UIErrorScope.CURRENT_USER,
							null,
							m_asOwner.GetString(2425, "ptmsgs_portaladminmsgs"),
							m_asOwner.GetString(2425, "ptmsgs_portaladminmsgs"));
						UIErrorHandler.RaiseError(
							myError,
							m_asOwner.GetSubSession(),
							m_asOwner.GetApplication());

						return UserInfoPropMapRepostControl.PAGE_STATUS_INVALID;
					}
					
					m_arrPropMapInfo[0][i]     = arrItems.GetItem(i,
							PTMultiColumnListModel.EXPLIST_COL_OBJECTID +
							MAPLIST_PTOBJECTPOSITION);
					m_arrPropMapInfo[1][i]     = XPConvert.ToString(strMappedValue).trim();
					
					htDupChecker.PutElement(arrMappingInfo[0][i], arrMappingInfo[0][i]);
				}
			}
		}
		catch (Exception e)
		{
			intPageStatus = UserInfoPropMapRepostControl.PAGE_STATUS_INVALID;

			GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Admin"), "UserInfoPropMapModel::SaveMainPage", 
				e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(1583,"ptmsgs_portaladminmsgs"));
		}

		return intPageStatus;
	}

	// END ***************** repost action methods *****************
}


