package com.plumtree.portalpages.admin.serversettings.objectpropertymap.plugin;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tree.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This is the implementation of the Expandable List for object properties map.
 *
 * @author josephc
 */
public class ObjPropMapPluginModel extends ExpListModel
	implements IObjPropMapPluginModelRO
{
	/** Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "ObjPropMapPlugin";

	/** Number total of columns */
	public static final int EXPLIST_COL_MAX = 3;

	/** Column for class ID */
	public static final int EXPLIST_COL_CLASSID = 0;

	/** Column for class name */
	public static final int EXPLIST_COL_CLASS_NAME = 1;

	/** Column for mapped properties */
	public static final int EXPLIST_COL_MAPPED_PROPERTIES = 2;

	/** Subspace key for property tree */
	public static final String SUBSPACE_KEY_PROPERTYTREE = "ObjPropMapSelectPropertySubSpace";

	/** Property info column for name */
	public static int PROPERTY_INFO_NAME = 0;
	
	/** Property info column for description */
	public static int PROPERTY_INFO_DESC = 1;

	/** Class info column for class ID */
	private static int CLASS_INFO_COLUMN_CLASSID = 0;

	/** Class info column for class name */
	private static int CLASS_INFO_COLUMN_NAME = 1;

	/** Property object manager */
	protected IPTObjectManager m_ptPropertyManager;

	/** Object property map */
	protected IPTObjectPropertyMap m_ptOPM;

	/** IPTSession */
	protected IPTSession m_ptSession;

	/** This is used on add to populate the input text of the newly added row. */
	protected Object m_oDefaultValue;

	/** Table of class mappings */
	protected XPHashtable m_htClassMappings;

	/** Table of property names and descrptions */
	protected XPHashtable m_htPropertyInfo;

	/**
	 * @see com.plumtree.uiinfrastructure.expandablelist.ExpListModel#AddRowsOnRepost(XPHashtable)
	 */
	public void AddRowsOnRepost(XPHashtable _htFormData)
	{
		TreeAS asTree = (TreeAS) m_asOwner.GetSubSpace(GetUniqueString(
					SUBSPACE_KEY_PROPERTYTREE));

		if (null != asTree)
		{
			ITreeModelRO myTree    = (ITreeModelRO) asTree.GetModel(ATreeAS.TREEMODEL);
			XPArrayList mySelected = myTree.GetSelectedItems();

			String sRowKey = GetUniqueString(ExpListView.HTMLHIDDEN_ROWINDEX);

			int nRowIndex = -1;

			if (_htFormData.ContainsKey(sRowKey))
			{
				String[] aRowValue = (String[]) _htFormData.GetElement(sRowKey);

				if ((null != aRowValue) && (aRowValue.length > 0))
				{
					nRowIndex = XPConvert.ToInteger(aRowValue[0]);
				}
			}

			if (-1 != nRowIndex)
			{
				AddPropertiesToRow(nRowIndex, mySelected);
			}
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ObjPropMapPluginModel();
	}

	/**
	 * Returns the class icon
	 *
	 * @param _nRowIndex Row index
	 *
	 * @return Icon for class
	 *
	 * @see com.plumtree.portalpages.admin.editors.objectpropertymap.plugin.IObjPropMapPluginModelRO#GetClassIcon(int)
	 */
	public HTMLImg GetClassIcon(int _nRowIndex)
	{
		// Get the list of all the classes in the system
		ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();

		int nClassID = XPConvert.ToInteger(m_saData.GetItem(_nRowIndex,
				ObjPropMapPluginModel.EXPLIST_COL_CLASSID));
		
		String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE, m_asOwner);

		return ptClassTypeDescManager.GetDefaultIconForClassType(nClassID, strLangID);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns array of property names and descriptions.
	 *
	 * @param _nRowIndex Row index
	 *
	 * @return Array of property names and descriptions
	 *
	 * @see com.plumtree.portalpages.admin.editors.objectpropertymap.plugin.IObjPropMapPluginModelRO#GetPropertyNames(int)
	 */
	public String[][] GetPropertyInfo(int _nRowIndex)
	{
		Object nClassID = m_saData.GetItem(_nRowIndex,
				ObjPropMapPluginModel.EXPLIST_COL_CLASSID);

		XPArrayList alistProperties = (XPArrayList) m_htClassMappings.GetElement(nClassID);

		String[][] arPropertyInfo = new String[alistProperties.GetSize()][2];

		for (int i = 0; i < alistProperties.GetSize(); i++)
		{
			Object nPropertyID = alistProperties.GetElement(i);

			// Try to get cached name of property.  If not found, then open property and cache it.
			if (!m_htPropertyInfo.ContainsKey(nPropertyID))
			{
				IPTProperty ptProperty = (IPTProperty) m_ptPropertyManager.Open(XPConvert.ToInteger(nPropertyID), false);
				arPropertyInfo[i][PROPERTY_INFO_NAME] = ptProperty.GetLocalizedName();
				arPropertyInfo[i][PROPERTY_INFO_DESC] = ptProperty.GetDescription();
				
				// Cache the row
				m_htPropertyInfo.PutElement(nPropertyID, arPropertyInfo[i]);
			}
			else
			{
				arPropertyInfo[i] = (String[]) m_htPropertyInfo.GetElement(nPropertyID);
			}
		}

		return arPropertyInfo;
	}

	/**
	 * Returns a String of Class IDs and Object IDs of the Objects  to select
	 * in the tree. E.g.: "propertyClassID,ObjID1,propertyClassID,objID2".  In
	 * our case, propertyClassID is the classID for properties.
	 *
	 * @param _nRowIndex Row index
	 *
	 * @return String
	 *
	 * @see com.plumtree.portalpages.admin.editors.objectpropertymap.plugin.IObjPropMapPluginModelRO#GetSelectedItems(int)
	 */
	public String GetSelectedItems(int _nRowIndex)
	{
		XPStringBuilder sb = new XPStringBuilder(100);

		Object nClassID		    = m_saData.GetItem(_nRowIndex,
				ObjPropMapPluginModel.EXPLIST_COL_CLASSID);
		XPArrayList alistProperties = (XPArrayList) m_htClassMappings.GetElement(nClassID);

		// RW - loop through array and add object id to string
		for (int i = 0; i < alistProperties.GetSize(); i++)
		{
			int nPropertyID = XPConvert.ToInteger(alistProperties.GetElement(i));

			// JS- Add the separator
			if (0 != i)
			{
				sb.Append(",");
			}

			sb.Append(PT_CLASSIDS.PT_PROPERTY_ID).Append(",").Append(nPropertyID);
		}

		return sb.ToString();
	}

	/**
	 * Initialize object properties map. Initialize expandable list and set the
	 * row icon.
	 *
	 * @param _ptSession IPTSession
	 * @param _sSectionTitle Category title
	 * @param _nListKey List key
	 *
	 * @return True if initialized successfully. False otherwise.
	 */
	public boolean InitializeObjectPropertiesMap(IPTSession _ptSession,
		String _sSectionTitle, int _nListKey)
	{
		m_ptSession     = _ptSession;

		m_ptOPM     = (IPTObjectPropertyMap) m_ptSession.OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_OBJECTPROPERTYMAP,
				false);

		m_ptPropertyManager = m_ptSession.GetProperties();

		// Initialize expandable list
		InitExpandableList(_sSectionTitle, _nListKey);

		return true;
	}

	/**
	 * Initalizes the list (PTGrowableSortedArrayWrapper)  with a column
	 * mapping and an array of sort. The growable array will be sorted by the
	 * 2 columns, first the name and second classID, ascending
	 *
	 * @param _saData
	 *
	 * @see com.plumtree.uiinfrastructure.expandablelist.ExpListModel#InitializeSortedArray(PTGrowableSortedArrayWrapper)
	 */
	public void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
	{
		int[] arrColumns = new int[] { EXPLIST_COL_CLASSID, EXPLIST_COL_CLASS_NAME };

		// The growable array will be sorted by 2 columns, first the class name and classID
		int[][] arrSort = new int[2][2];
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_COL_CLASS_NAME;
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
		arrSort[1][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = EXPLIST_COL_CLASSID;
		arrSort[1][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;

		Object[][] arClassInfo										  = null;

		// Get the list of all the classes in the system
		ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();
		String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE, m_asOwner);

		int[] arAllClassIDs		  = ptClassTypeDescManager.GetOrderedListOfClassIDs(
				PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(m_asOwner).GetLanguage());
		XPArrayList alistClassIDs = new XPArrayList();

		// Filter out all of the classes that cannot be used for extended object data.
		if (null != arAllClassIDs)
		{
			for (int i = 0; i < arAllClassIDs.length; i++)
			{
				int nClassID = arAllClassIDs[i];

				if (ptClassTypeDescManager.AppearsInGlobalObjPropMap(nClassID))
				{
					alistClassIDs.Add(new Integer(nClassID));
				}
			}
		}

		// Construct class info array in order to initialize the sorted array
		if (!alistClassIDs.IsEmpty())
		{
			arClassInfo = new Object[2][alistClassIDs.GetSize()];

			for (int i = 0; i < alistClassIDs.GetSize(); i++)
			{
				arClassInfo[CLASS_INFO_COLUMN_CLASSID][i] = alistClassIDs.GetElement(i);

				int nClassID = XPConvert.ToInteger(alistClassIDs.GetElement(i));
				arClassInfo[CLASS_INFO_COLUMN_NAME][i] = ptClassTypeDescManager.GetClassTypeLocalizedName(nClassID, strLangID);
			}

			// Load mappings into hashtable
			LoadClassMappings(alistClassIDs);
		}

		// Initialize the sorted array
		_saData.Initialize(arrColumns, arClassInfo, arrSort, true);
	}

	/**
	 * Sets a new default value.
	 *
	 * @param _oDefaultValue New value
	 */
	public void SetDefaultValue(Object _oDefaultValue)
	{
		m_oDefaultValue = _oDefaultValue;
	}

	/**
	 * Stores the property mapping changes to the property object.
	 */
	public void StorePropertyMappings()
	{
		// Iterate through the different classes
		for (int nRowIndex = 0; nRowIndex < m_saData.GetCount(); nRowIndex++)
		{
			Object nClassID = m_saData.GetItem(nRowIndex,
					EXPLIST_COL_CLASSID);

			// For each class get the list of properties
			XPArrayList alistProperties = (XPArrayList) m_htClassMappings.GetElement(nClassID);

			int[] arPropertyIDs = new int[alistProperties.GetSize()];

			for (int i = 0; i < arPropertyIDs.length; i++)
			{
				int nPropertyID = XPConvert.ToInteger(alistProperties.GetElement(i));
				arPropertyIDs[i] = nPropertyID;
			}

			m_ptOPM.SetClassPropertyMappings(XPConvert.ToInteger(nClassID), arPropertyIDs);
		}
	}

	/**
	 * Adds new properties to the row.
	 *
	 * @param _nRowIndex Row index
	 * @param _alistSelectedProperties List of selected properties
	 */
	protected void AddPropertiesToRow(int _nRowIndex,
		XPArrayList _alistSelectedProperties)
	{
		// Get classID
		Object nClassID = m_saData.GetItem(_nRowIndex,
				EXPLIST_COL_CLASSID);

		XPArrayList alistProperties = (XPArrayList) m_htClassMappings.GetElement(nClassID);

		alistProperties.Clear();

		for (int i = 0; i < _alistSelectedProperties.GetSize(); i++)
		{
			int nPropertyID = ((TreeLeafNode) _alistSelectedProperties.GetElement(i)).m_nObjectID;

			alistProperties.Add(new Integer(nPropertyID));
		}
	}

	/**
	 * Saves the data on the row _nRowIndex.
	 *
	 * @param _saData List to update the data on row _nRowIndex
	 * @param _nRowIndex Index of the row to save.
	 * @param _htFormData Data posted by the form.
	 *
	 * @return int Status for server side validation.
	 */
	protected int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
		XPHashtable _htFormData)
	{
		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Initializes expandable list.
	 *
	 * @param _sSectionTitle Category title
	 * @param _nListKey List key
	 */
	private void InitExpandableList(String _sSectionTitle, int _nListKey)
	{
		Object[][] arrExpListColInfo = new Object[2][5];

		// The first column will be the class name and will be sortable
		arrExpListColInfo[0][EXPLIST_COLUMN_ID]     = new Integer(EXPLIST_COL_CLASS_NAME);

		arrExpListColInfo[0][EXPLIST_COLUMN_TITLE]		    = m_asOwner.GetString(535,
				"ptmsgs_portaladminmsgs");
		arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH]		    = "20%";
		arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN]	    = "2";
		arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT]     = Boolean.TRUE;

		// The second column will be the properties mapped to the class and will not be sortable
		arrExpListColInfo[1][EXPLIST_COLUMN_ID]     = new Integer(EXPLIST_COL_MAPPED_PROPERTIES);

		arrExpListColInfo[1][EXPLIST_COLUMN_TITLE]		    = m_asOwner.GetString(536,
				"ptmsgs_portaladminmsgs");
		arrExpListColInfo[1][EXPLIST_COLUMN_WIDTH]		    = "80%";
		arrExpListColInfo[1][EXPLIST_COLUMN_COLSPAN]	    = "2";
		arrExpListColInfo[1][EXPLIST_COLUMN_ENABLESORT]     = Boolean.FALSE;

		super.InitializeReadOnlyExpandableList(_sSectionTitle,
			arrExpListColInfo, _nListKey);
	}

	/**
	 * Load class mappings into table.
	 *
	 * @param alistClassIDs List of class ID's
	 */
	private void LoadClassMappings(XPArrayList alistClassIDs)
	{
		// Create hash table of class ID's and properties
		m_htClassMappings     = new XPHashtable();

		// Create hash table of property names
		m_htPropertyInfo = new XPHashtable();

		IXPEnumerator enumClasses = alistClassIDs.GetEnumerator();

		while (enumClasses.MoveNext())
		{
			Object nClassID = enumClasses.GetCurrent();

			// Add array list to hash table
			m_htClassMappings.PutElement(nClassID, new XPArrayList());
		}

		// TODOCR JIC - Are we supposed to use a wrapper class isntead of IPTQueryResult?
		IPTQueryResult ptQR = m_ptOPM.QueryAllClassPropertyMappings(PT_PROPIDS.PT_PROPID_ALL);

		for (int nRow = 0; nRow < ptQR.RowCount(); nRow++)
		{
			Object nClassID    = new Integer (ptQR.ItemAsInt(nRow,
					PT_PROPIDS.PT_PROPID_CLASSID));
			Object nPropertyID = new Integer (ptQR.ItemAsInt(nRow,
					PT_PROPIDS.PT_PROPID_OBJECTID));

			if (m_htClassMappings.ContainsKey(nClassID))
			{
				XPArrayList alistProperties = (XPArrayList) m_htClassMappings.GetElement(nClassID);
				alistProperties.Add(nPropertyID);

				if (!m_htPropertyInfo.ContainsKey(nPropertyID))
				{
					String strPropertyName = ptQR.ItemAsString(nRow, PT_PROPIDS.PT_PROPID_NAME);
					String strPropertyDesc = ptQR.ItemAsString(nRow, PT_PROPIDS.PT_PROPID_DESCRIPTION);
					
					String[] propertyInfoRow = new String[2];
					propertyInfoRow[PROPERTY_INFO_NAME] = strPropertyName;
					propertyInfoRow[PROPERTY_INFO_DESC] = strPropertyDesc;
					
					m_htPropertyInfo.PutElement(nPropertyID, propertyInfoRow);
				}
			}
		}
	}
}

