package com.plumtree.portalpages.admin.serversettings.search;

import com.plumtree.openfoundation.internationalization.XPLocale;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.helpers.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.search.*;
import com.plumtree.server.search.admin.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;

/**
 * @author SamirM
 */
public class SearchServerTuningEditorModel extends EditorModel
	implements ISearchServerTuningEditorModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADMIN),
		"com.plumtree.portalpages.admin.serversettings.search.SearchServerTuningEditorModel");


	public static final String STR_MVC_CLASS_NAME = "SearchServerTuningEditorModel";
	
	IXPPropertyBag m_statusBag;

	// member variables
	XPStringBuilder m_errors = null;
	
	private AliasElement[] _aliasElements = null;
	private String _alias = null;
	
	private boolean _thesaurusOn = false;
	private boolean _spellCheckOn = false;
	
	private IPTSearchBestBetMaintenance m_bestBetModel; // Best Bet Model - initialized by InitBestBetsList()

	public SearchServerTuningEditorModel()
	{
		super();
		m_errors = new XPStringBuilder();

	}

	public void Init(AActivitySpace parent)
	{
		super.Init(parent);
	}

	protected void InitSpellCheckThesaurus()
	{
		IPTServerConfigSettings ptSCS = (IPTServerConfigSettings) ((IPTSession)m_asOwner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
				false);
		_thesaurusOn = (ptSCS.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_ALLOW_THESAURUS) == 1);
		_spellCheckOn = (ptSCS.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_ALLOW_SPELLCHECK) == 1);
	}

	// interface PropertyValidatorOwner
	public XPStringBuilder GetErrorString()
	{
		return m_errors;
	}

	public boolean CheckEditorAccessAndStart(XPHashtable _htFormData)
	{
		IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
		boolean isAdmin = PlumtreeHelpers.CheckCurrentUserIsAdmin(ptSession);
		boolean hasRight = ptSession.GetSessionInfo().CheckForActivityRight(PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_ACCESS_SEARCH_SERVER_TUNING);

		if(!(isAdmin || hasRight))
		{
			log.Debug("CheckEditorAccessAndStart returning false");

			return false;
		}	

		InitBestBetsList();
		
		InitSpellCheckThesaurus();

		SetEnableApplyButton(true);

		return true;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new SearchServerTuningEditorModel();
	}


	/**
	 *
	 */
	public void Finish()
	{
    	DoApplyChanges(null);
	}

  	public void DoApplyChanges(XPHashtable lastUserVals)
	{
		SaveBestBetsChanges();
		if(lastUserVals != null)
			LoadLastUserVals(lastUserVals);
		if(CheckBannerAliasAndDelocalizeWeights())
			SaveFieldAliasToServer();
		SaveThesaurusSpellToServer();
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 *
	 *
	 * @param _sPageName
	 * @param _htFormData
	 *
	 * @return
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		TraceHelper.Trace("SavePage(" + _sPageName + ")");
		TraceHelper.TraceQueryString(_htFormData);

		int result = RepostControl.PAGE_STATUS_VALID;

		if (_sPageName.equals(SearchServerTuningEditorDP.STR_MVC_CLASS_NAME))
		{
			result = SaveMainPage(_htFormData);
		}
		else if (_sPageName.equals(ThesaurusSettingsDP.STR_MVC_CLASS_NAME))
			SaveThesaurusFromForm(_htFormData);
		else if (_sPageName.equals(BannerFieldAliasDP.STR_MVC_CLASS_NAME))
			result = SaveBannerAlias(_htFormData);

		return result;
	}

	/***** START - BEST BET RELATED METHODS *******/
	/* START - ISearchServerTuningEditorModelRO implementation */
	private String m_searchString;
	private int m_nNumBBsAskedFor = 10;
	private int m_nBBStartCount = 1;
	
	public String GetBestBetSearchString()
	{
		return m_searchString;
	}
	
	public boolean GetIsShowAllBestBets()
	{
		return m_bestBetModel.GetShowingAll();
	}

	public int GetNumBestBetsAskedFor()
	{
		return m_nNumBBsAskedFor;
	}

	public int GetNumBestBetsOnPage()
	{
		return m_bestBetModel.GetDisplayedCount();
	}
	
	public int GetBestBetStartCount()
	{	
		return m_nBBStartCount;	
	}
	public boolean GetShowPreviousBestBet()
	{
		return (GetBestBetStartCount() > 1);	
	}
	public boolean GetShowNextBestBet()
	{
		return (GetTotalBestBetCount() >= (GetBestBetStartCount()+GetNumBestBetsOnPage()));
	}
	public int GetTotalBestBetCount()
	{
		return m_bestBetModel.GetTotalCount();
	}
	
	public int GetNumDeletedBestBets()
	{
		return m_bestBetModel.GetDeletedCount();
	}

	public int GetNumAddedBestBets()
	{
		return m_bestBetModel.GetAddedCount();
	}

	// Make sure to call CheckDuplicateBestBetQuery before calling this.
	public void UpdateBestBetFromSubEditor(String originalTrigger, boolean bModifiedBBWasFromDB, String strTrigger, String setup, IXPList arrTargets /* IPTBestBetTarget */)
	{
		m_bestBetModel.UpdateBestBet(originalTrigger, bModifiedBBWasFromDB, strTrigger, setup, arrTargets);
		m_bestBetModel.RefreshDisplayList();
	}

	public String GetBestBetTrigger(int i)
	{
		return m_bestBetModel.GetDisplayedItemTrigger(i);
	}
	
	public String GetBestBetSetupString(int i)
	{
		return m_bestBetModel.GetDisplayedItemSetupString(i);
	}

	public IXPList GetBestBetTargets(int i)
	{
		return m_bestBetModel.GetDisplayedItemTargets(i);
	}

	public boolean HasBestBetBeenDeleted(int i)
	{
		return m_bestBetModel.HasDisplayedItemBeenDeleted(i);
	}

	public boolean HasBestBetStringBeenDeleted(String trigger)
	{
		return m_bestBetModel.HasBestBetBeenDeleted(trigger);
	}

	public boolean HasBestBetBeenNewlyAdded(int i)
	{
		return m_bestBetModel.HasDisplayedItemBeenNewlyAdded(i);
	}

	public boolean CheckDuplicateBestBetQuery(String originalTrigger, String strTrigger, boolean bNewlyAdded)
	{
		return m_bestBetModel.CanAddBestBet(originalTrigger, strTrigger, bNewlyAdded);
	}
	/* END - IMPLEMENTATION OF ISearchServerTuningEditorModelRO */

	public void SetNumBestBetsAskedFor(int n)
	{
		m_nNumBBsAskedFor = n;
		m_bestBetModel.SetNumResultsAskedFor(n);
	}

	public void SetBestBetStartCount(int n)
	{
		m_nBBStartCount = n;
		m_bestBetModel.SetStartCount(n);
	}

	public void DeleteBestBets(int ids[]) // Assumption is that list of ids are in ascending order.
	{
		m_bestBetModel.DeleteFromDisplayedList(ids);
		m_bestBetModel.RefreshDisplayList();
	}

	
	public void SearchBestBets(String str)
	{
	   m_searchString = str;
	   m_bestBetModel.SetShowingAll(false);
	   m_bestBetModel.SetSearchString(str);
	   SetBestBetStartCount(1);
	   m_bestBetModel.RefreshDisplayList();
	}

	public void ShowAllBestBets()
	{
		m_bestBetModel.SetShowingAll(true);
		m_bestBetModel.SetSearchString(null);
	  m_searchString = "";
	  SetBestBetStartCount(1);
	  m_bestBetModel.RefreshDisplayList();
	}

	public void PaginateBestBets(int startCount, int numResultsPerPage)
	{
		SetBestBetStartCount(startCount);
		SetNumBestBetsAskedFor(numResultsPerPage);
		m_bestBetModel.RefreshDisplayList();
	}
	
	/***** END - BEST BET RELATED METHODS *******/
	
	protected int SaveBannerAlias(XPHashtable _htFormData)
	{
		int result = RepostControl.PAGE_STATUS_VALID;
		
		String[] numVals = (String[]) _htFormData.GetElement(BannerFieldAliasView.IN_HI_NUM_FIELDS);
		
		if(numVals == null || numVals.length != 1)
		{
			return RaiseBannerAliasError(m_asOwner.GetString(2567, "ptmsgs_portaladminmsgs"));
		}	
		int number = XPConvert.ToInteger(numVals[0]);
		String alias = "";
		AliasElement[] newAlias = new AliasElement[number];
		String id;
		for(int i = 0; i < number; i++)
		{
			if(i > 0)
				alias += ",";

			String idInputName = BannerFieldAliasView.IN_SE_NAME + i;
			String[] idVals = (String[]) _htFormData.GetElement(idInputName);
			id = idVals[0];

			String weightInputName = BannerFieldAliasView.IN_WEIGHT_INPUT + i;
			String[] weightVals = (String[]) _htFormData.GetElement(weightInputName);
			String newPropVal = "PT" + id + "[";
			if(alias.indexOf(newPropVal) != -1)
				return RaiseBannerAliasError(m_asOwner.GetString(2568, "ptmsgs_portaladminmsgs"));
			try
			{
				float weight = XPConvert.ToFloat(DeLocalizeFloat(weightVals[0]));
				if(weight <= 0)
					return RaiseBannerAliasError(m_asOwner.GetString(2569, "ptmsgs_portaladminmsgs"));
			}
			catch(Exception e)
			{
				return RaiseBannerAliasError(m_asOwner.GetString(2570, "ptmsgs_portaladminmsgs"));				
			}
			alias += newPropVal + DeLocalizeFloat(weightVals[0]) + "]";
			newAlias[i] = new AliasElement(id, DeLocalizeFloat(weightVals[0]));
		}
		_aliasElements = newAlias;
		_alias = alias;
		return result;
	}
	
	protected boolean CheckBannerAliasAndDelocalizeWeights()
	{
		if(_aliasElements == null)
			return false;
		for(int i = 0; i < _aliasElements.length; i++)
		{
			
			for(int j = 1; j < _aliasElements.length; j++)
			{
				if(i != j)
				{
					if(_aliasElements[i].propID == _aliasElements[j].propID)
					{
						RaiseBannerAliasError(m_asOwner.GetString(2568, "ptmsgs_portaladminmsgs"));
						return false;
					}
				}
            }
			try
			{
				float weight = XPConvert.ToFloat(DeLocalizeFloat(_aliasElements[i].weightVal));
				if(weight <= 0)
				{
					RaiseBannerAliasError(m_asOwner.GetString(2569, "ptmsgs_portaladminmsgs"));
					return false;
				}
				// Replace the existing weight val in memory with the "delocalized" version.
				_aliasElements[i].weightVal = XPConvert.ToString(weight);
			}
			catch(Exception e)
			{
				RaiseBannerAliasError(m_asOwner.GetString(2570, "ptmsgs_portaladminmsgs"));		
				return false;		
			}
		}
        
        // OK, it all looks ok right now, but recalculate the percentages, and make sure
        // that none of them round down to 0.
        RecalculatePercentages(_aliasElements);
        for(int i = 0; i < _aliasElements.length; i++)
        {
            if(XPConvert.ToFloat(_aliasElements[i].percentage) <= 0)
            {
                RaiseBannerAliasError(m_asOwner.GetString(2569, "ptmsgs_portaladminmsgs"));
                return false;
            }
        }
        
		return true;
	}

	public void AddAliasTerm(XPHashtable lastUserVals)
	{
		AliasElement[] newAlias = new AliasElement[_aliasElements.length + 1];
		for(int i = 0; i < _aliasElements.length; i++)
		{
			newAlias[i] = _aliasElements[i];
		}
				// This default is "Name" "1.0", which seems as good as any.
		newAlias[_aliasElements.length] = new AliasElement("1","1.0");
		_aliasElements = newAlias;
		LoadLastUserVals(lastUserVals);
	}
	
	public boolean OkToDeleteThisManyAliasTerms(int number)
	{
			// We can't delete all the terms--at least one must remain.
		return (number < _aliasElements.length);
	}
	
	public void DeleteTheseAliasTerms(int[] indices, XPHashtable lastUserVals)
	{
		LoadLastUserVals(lastUserVals);
		int newNumber = _aliasElements.length - indices.length;
		AliasElement[] newAlias = new AliasElement[newNumber];
		int count = 0;
		for(int j = 0; j < indices.length; j++)
		{
			_aliasElements[indices[j]] = null;
		}

		for(int i = 0; i < _aliasElements.length; i++)
		{
			if(_aliasElements[i] != null)
				newAlias[count++] = _aliasElements[i];
		}
		_aliasElements = newAlias;
		
	}
	
	public void LoadLastUserVals(XPHashtable lastUserVals)
	{
		if(_aliasElements == null)
			return;
		for(int i = 0; i < _aliasElements.length; i++)
		{
			String weightName = BannerFieldAliasView.IN_WEIGHT_INPUT + i;
			String[] vals = (String[])lastUserVals.GetElement(weightName);
			if(vals != null)
				_aliasElements[i].weightVal = vals[0];

			String propIdName = BannerFieldAliasView.IN_SE_NAME + i;
			vals = (String[])lastUserVals.GetElement(propIdName);
			try
			{
				if(vals != null)
					_aliasElements[i].propID = XPConvert.ToInteger(vals[0]);
			}
			catch(Exception e)
			{
			}
		}
	}
	
	protected void BuildAliasFromElements()
	{
		String alias = "";
		for(int i = 0; i < _aliasElements.length; i++)
		{
			if(i > 0)
				alias += ",";
			alias += "PT" + _aliasElements[i].propID + "[" + _aliasElements[i].weightVal + "]";		
		}
		_alias = alias;
	}
	
	protected void SaveFieldAliasToServer()
	{
		BuildAliasFromElements();
		if(_alias == null || _alias.equals(""))
			return;
		IPTServerConfigSettings ptSCS = (IPTServerConfigSettings) ((IPTSession)m_asOwner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
				false);
		ptSCS.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_DEFAULT_FIELD_ALIAS, _alias);
		
		if(log.IsDebugEnabled())
			log.Debug("Saving Banner Alias = " + _alias);
	}
	
	protected int RaiseBannerAliasError(String message)
	{
		UIError err = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,message,"");
		err.SetActivitySpace(m_asOwner.GetName());
		err.SetDisplayOnSeparatePage(false);
		err.SetClearWithActivitySpaceChange(true);
		err.SetImmediateClear(true);
		
		UIErrorHandler.RaiseError(err,m_asOwner.GetSubSession(), m_asOwner.GetApplication());

		return RepostControl.PAGE_STATUS_INVALID;
	}
	
	/**
	 *
	 *
	 * @return
	 */
	public AActivitySpace asOwner()
	{
		return m_asOwner;
	}
    
	/**
	 * Trace a message if debug tracing is enabled for the UI component.
	 * This method could be moved into the EditorModel base class, along
	 * with trace() and traceEnabled().
	 */
	protected void TraceDebug(String msg)
	{
		if (TraceHelper.Enabled())
		{
			TraceHelper.Trace(msg);
		}
	}

	/**
	 *
	 */
	public void ActionOnRepost()
	{
	}

	/**
	 *
	 *
	 * @param _htFormData
	 *
	 * @return
	 */
	protected int SaveMainPage(XPHashtable _htFormData)
	{
		int result		   = RepostControl.PAGE_STATUS_VALID;

		return result;
	}

	protected void SaveThesaurusFromForm(XPHashtable _htFormData)
	{

		// Recover the checkbox element for turning spell check on, if any
		String[] vals = (String[])_htFormData.GetElement(ThesaurusSettingsView.APPLY_SPELL_CB);
		boolean spellCheckOn = false;
		if(vals != null && vals.length == 1)
		{	
			spellCheckOn = true;	
			if(log.IsDebugEnabled())	 
				log.Debug("SaveThesaurusFromForm: spell check on");
		 
		}
		else if(log.IsDebugEnabled())
			log.Error("SaveThesaurusFromForm: spell check off");
		
		SetSpellCheckOn(spellCheckOn);

		// Recover the thesaurus element
		vals = (String[])_htFormData.GetElement(ThesaurusSettingsView.APPLY_THESAURUS_CB);
		boolean thesaurusOn = false;
		if(vals != null && vals.length == 1)
		{	
			thesaurusOn = true;		 
			if(log.IsDebugEnabled())	 
				log.Debug("SaveThesaurusFromForm: thesaurus on");		 
		}
		else if(log.IsDebugEnabled())	 
			log.Debug("SaveThesaurusFromForm: thesaurus off");		 

		SetThesaurusOn(thesaurusOn);

	}
	
	protected void SaveThesaurusSpellToServer()
	{
		int spellOn = GetSpellCheckOn() ? 1 : 0;
		int thesOn = GetThesaurusOn() ? 1 : 0;
		
		IPTServerConfigSettings ptSCS = (IPTServerConfigSettings) ((IPTSession)m_asOwner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
				false);
		
		ptSCS.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_ALLOW_SPELLCHECK, spellOn);
		
		if(log.IsDebugEnabled())	 
			log.Debug("SaveThesaurusSpellToServer: spellcheck setting " + spellOn);		 
	
		ptSCS.SetSetting(PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_ALLOW_THESAURUS, thesOn);
		
		if(log.IsDebugEnabled())	 
			log.Debug("SaveThesaurusSpellToServer: thesaurus setting " + thesOn);		 

	}
	
	// Best Bets interface
	private void InitBestBetsList()
	{
		try
		{
			IPTSession session				    = (IPTSession) m_asOwner.GetUserSession();
                        IPTSearchFactory sf = session.GetSearchFactory();
                        IPTSAdmin searchAdmin = sf.createAdmin();
			m_bestBetModel = searchAdmin.createBestBetMaintenance();
			
			m_bestBetModel.SetStartCount(1);
			m_bestBetModel.RefreshDisplayList();
		}
		catch (Exception e)
		{
			XPException i18n = SearchUIUtility.GetI18NException(e,m_asOwner);
			UIError err = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,i18n, i18n.GetMessage(),"");
			err.SetActivitySpace(m_asOwner.GetName());
			err.SetDisplayOnSeparatePage(false);
			err.SetHTMLMessage(((SearchExecutionError)i18n).GetComment());
			err.SetClearWithActivitySpaceChange(true);
			err.SetImmediateClear(true);

			UIErrorHandler.RaiseError(err,m_asOwner.GetSubSession(), m_asOwner.GetApplication());
		}
	}

	/**
	 *
	 */
	private void SaveBestBetsChanges()
	{
		try
		{
			m_bestBetModel.ApplyChanges();
		}
		catch (Exception e)
		{
			XPException i18n = SearchUIUtility.GetI18NException(e,m_asOwner);
			UIError err = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,i18n, i18n.GetMessage(),"");
			err.SetActivitySpace(m_asOwner.GetName());
			err.SetDisplayOnSeparatePage(false);
			err.SetHTMLMessage(((SearchExecutionError)i18n).GetComment());
			err.SetClearWithActivitySpaceChange(true);
			err.SetImmediateClear(true);

			UIErrorHandler.RaiseError(err,m_asOwner.GetSubSession(), m_asOwner.GetApplication());

		}
	}
	protected String GetFieldAliasFromServer()
	{
		IPTServerConfigSettings ptSCS = (IPTServerConfigSettings) ((IPTSession)m_asOwner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
				false);
		_alias = ptSCS.GetSettingAsString(PT_SERVER_CONFIG_SETTINGNAME.PT_SEARCH_DEFAULT_FIELD_ALIAS);
		
		return _alias;
	}
	
	protected AliasElement[] ParseAlias(String alias)
	{
		if(alias == null || alias.equals(""))
			return new AliasElement[0];
		String delim = ",";

		int j= 0;
		XPStringTokenizer tokenizer = new XPStringTokenizer(alias, delim);
		String[] ellsVals = new String[tokenizer.GetTokenCount()];
		while(tokenizer.HasMoreElements())
		{
			ellsVals[j++] = tokenizer.GetNextElement();
		}
		
		AliasElement[] retval = new AliasElement[ellsVals.length];
		for(int i = 0; i < ellsVals.length; i++)
		{
			int openBrace = ellsVals[i].indexOf("[");
			String propIDVal = ellsVals[i].substring(2,openBrace);
			int closeBrace = ellsVals[i].indexOf("]");
			String weight = ellsVals[i].substring(openBrace + 1, closeBrace);
			AliasElement el = new AliasElement();
			try
			{
				el.propID = XPConvert.ToInteger(propIDVal);
			}
			catch(Exception e)
			{
				el.propID = -1;
			}
			el.weightVal = weight;
			try
			{
				XPConvert.ToFloat(weight);
			}
			catch(Exception e)
			{
				el.propID = -1;
			}
			retval[i] = el;		
		}
		RecalculatePercentages(retval);
		return retval;
	}
	
	public void RestoreDefaultFieldAlias()
	{
		_alias = "PT1[2.0],PT2[0.1],PT50[0.1]";
		_aliasElements = new AliasElement[3];
		_aliasElements[0] = new AliasElement("1","2.0");
		_aliasElements[1] = new AliasElement("2","0.1");
		_aliasElements[2] = new AliasElement("50","0.1");
	}
	
	public Object[] GetCurrentAliasElements()
	{
		if(_aliasElements == null)
			_aliasElements = ParseAlias(GetFieldAliasFromServer());
		return _aliasElements;
	}
	
	protected void RecalculatePercentages(AliasElement[] elements)
	{
		float totalWeight = 0;
		for(int i = 0; i < elements.length; i++)
		{
			try
			{
				totalWeight += XPConvert.ToFloat(elements[i].weightVal);
			}
			catch(Exception e)
			{
			}
		}
		for(int i = 0; i < elements.length; i++)
		{
			try
			{
				String temp = XPConvert.ToString((XPConvert.ToFloat(elements[i].weightVal)/totalWeight) * 100.0 + 0.005);
				int decimalpt = temp.indexOf(".");
				elements[i].percentage = temp.substring(0,decimalpt + 3);
			}
			catch(Exception e)
			{
				elements[i].percentage = "0";
			}
		}
	}

	public static class AliasElement
	{
		int propID;
		String weightVal;
		String percentage;
		public AliasElement(String prop, String weight)
		{
			try
			{
				propID = XPConvert.ToInteger(prop);
			}
			catch(Exception e)
			{
				propID = -1;
			}
			weightVal = weight;
		}
		public AliasElement()
		{
			propID = -1;
		}

	}

	public boolean GetThesaurusOn()
	{
		return _thesaurusOn;
	}
	
	public void SetThesaurusOn(boolean tf)
	{
		_thesaurusOn = tf;
	}
	
	public boolean GetSpellCheckOn()
	{
		return _spellCheckOn;
	}

	public void SetSpellCheckOn(boolean tf)
	{
		_spellCheckOn = tf;
	}
	
    /**
     * GetLocalizedDecimalSeperator - Returns the locale specific decimal seperator.
     *                                This method is public since it is used by the view.
	 * Helper method that deals with localization of floating point #s in the UI.
     * @return
     */
    public char GetLocalizedDecimalSeperator()
    {
        double d = 0.1;
        String s = XPFormat.Format(new XPLocale(m_asOwner.GetLocale()), d);
        if(s.length() != 3) // something wierd is going on...
        {
           return '.';
        }
        else
        {
           return s.charAt(1);
        }
    }

    /**
     * LocalizeFloat - Converts internal string representation of floating point
     *                 number into the locale specific string representation.
     *                 This method is public since it is used by the view.
     * 
	 * Helper method that deals with localization of floating point #s in the UI.
     * @param strFloatVal
     * @return
     */
    public String LocalizeFloat(String strFloatVal)
    {
        if(strFloatVal == null || strFloatVal.length() == 0)
            return "";
        double d = 0.0;
        try
        {
            d = XPConvert.ToDouble(strFloatVal);
        }
        catch(XPException e)
        {
            return "";
        }
        return XPFormat.Format(new XPLocale(m_asOwner.GetLocale()), d);
    }

    /**
	 * DeLocalizeFloat - Converts a localized string representation of a floating point
	 *                   number into the internal string representation (i.e. replaces
	 *                   locale specific decimal seperator with a period '.')
	 * 
	 * Helper method that deals with localization of floating point #s in the UI.
	 * @param strFloatVal
	 * @return
	 */
    private String DeLocalizeFloat(String strFloatVal)
    {
        char cSep = GetLocalizedDecimalSeperator();
        String strSep = "" + cSep;
        int nPosComma = strFloatVal.indexOf(strSep);
        if(nPosComma >= 0)
        {
            strFloatVal = strFloatVal.replace(cSep,'.');
        }
        return strFloatVal;
    }	
}

