package com.plumtree.portalpages.browsing.directory.edit;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portalpages.browsing.directory.properties.*;
import com.plumtree.portalpages.common.folder.*;
import com.plumtree.portalpages.common.objects.*;
import com.plumtree.portalpages.common.uiparts.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author StephenC
 *
 */
public class DirEditDP extends PlumtreeDP
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.directory.edit.DirEditDP");

	
	public static final String STR_MVC_CLASS_NAME = "DirEdit";

	/** HTML Input checkbox: Check All */
	public final String HTMLINPUT_CHECK_ALL_FOLDERS = "in_cb_checkAllFolders";

	
	public final String JAVASCRIPT_CHECK_ALL_FOLDERS = "checkAllFolders";

	private DirModel m_asModel;

	/** Javascript to call in the header */
	private String m_strScript = null;


	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.portalpages.common.uiparts.PlumtreeDP#GetSubtitleForBanner()
	 */
	public String GetSubtitleForBanner()
	{
		return "";
	}

	/**
	 * @see com.plumtree.portalpages.common.uiparts.PlumtreeDP#GetTitleForBanner()
	 */
	public String GetTitleForBanner()
	{
		return m_asOwner.GetString(109, "ptmsgs_portalbrowsingmsgs");
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IDisplayPage#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		m_strScript = null;
		
		super.Init(parent);
		SetAddMainForm(true);
		
	}

	// SCC: overriding AFormDP's implementation
	protected boolean GetRepostTypeIsPost()
	{
		return false;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.common.AFormDP#PageDisplay()
	 */
	public HTMLElement PageDisplay()
	{
		
		m_asModel = (DirModel) m_asOwner.GetModel(DirModel.STR_MVC_CLASS_NAME);

		((ObjectView) m_asOwner.GetView(ObjectView.STR_MVC_CLASS_NAME)).SetObjectViewHelper(new EditDocViewHelper());

		HTMLTable table;
		HTMLTableRow row;
		HTMLTableCell cell;

		table = new HTMLTable();
		table.SetBorder(CommonHTMLStrings.ZERO);
		table.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		table.SetCellSpacing(CommonHTMLStrings.ZERO);
		table.SetCellPadding(CommonHTMLStrings.ZERO);
		row = new HTMLTableRow();
		table.AddInnerHTMLElement(row);

		// Group Folder and Document stuff together to make related resources placement easier
		HTMLElementCollection foldersAndDocumentsGroup = new HTMLElementCollection();
		cell = new HTMLTableCell();
		cell.AddInnerHTMLElement(foldersAndDocumentsGroup);
		row.AddInnerHTMLElement(cell);

		try
		{
			//Breadcrumbs
			HTMLTable breadcrumbsTable = DirConstants.DisplayBreadcrumbTable(m_asOwner);

			if (null != breadcrumbsTable)
			{
				foldersAndDocumentsGroup.AddInnerHTMLElement(breadcrumbsTable);
			}

			//Folder Actions Banner
			foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(
					FolderBannerView.STR_MVC_CLASS_NAME).Display());

			// Folder Header
			int nSubFolderCount = m_asModel.GetSubFolderCount(m_asModel.GetCurrentFolderLevel() +
					1);

			if (0 != nSubFolderCount)
			{
				HTMLTable myTable = new HTMLTable();
				myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
				myTable.SetBorder(CommonHTMLStrings.ZERO);
				myTable.SetCellPadding("2");
				myTable.SetCellSpacing("2");
				foldersAndDocumentsGroup.AddInnerHTMLElement(myTable);

				// spacer row
				myTable.AddInnerHTMLElement(DirConstants.CreateSpacerRow(2, 5));

				HTMLTableRow myRow = new HTMLTableRow();
				myRow.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);
				myTable.AddInnerHTMLElement(myRow);

				HTMLTableCell myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);
				myCell.SetWidth("25");
				myCell.SetHeight("20");
				myCell.SetAlign(CommonHTMLStrings.CENTER);
				myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
				myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

				HTMLInput myInput = new HTMLInput(HTMLInputTypes.CHECKBOX,
						HTMLINPUT_CHECK_ALL_FOLDERS,
						m_asOwner.GetString(110, "ptmsgs_portalbrowsingmsgs"));
				myCell.AddInnerHTMLElement(myInput);
				myInput.SetOnClick(	// mj: removed 'Javascript:' from function name
					JAVASCRIPT_CHECK_ALL_FOLDERS + "(this, '" +
					DirFolderViewHelper.QS_CHECKBOX + "');");

				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);
				myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
				myCell.SetAlign(CommonHTMLStrings.LEFT);
				myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
				myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);

				myCell.AddInnerHTMLString(m_asOwner.GetString(108,
						"ptmsgs_portalbrowsingmsgs"));

				// blank cell over folder edit icons
				myCell = new HTMLTableCell();
				myCell.AddInnerHTMLElement(DirConstants.CreateSpacerTable(1,20));
				myRow.AddInnerHTMLElement(myCell);
			}

			// Folders
			foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(
					FolderView.STR_MVC_CLASS_NAME).Display());

			// ADD extra spacer table
			foldersAndDocumentsGroup.AddInnerHTMLElement(DirConstants.CreateSpacerTable(15,1));

			//Documents
			if (!m_asModel.InRootFolder())
			{
				foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(
						DocumentBannerView.STR_MVC_CLASS_NAME).Display());

				foldersAndDocumentsGroup.AddInnerHTMLElement(DirConstants.CreateSpacerTable(5,1));

				foldersAndDocumentsGroup.AddInnerHTMLElement(m_asOwner.GetView(
						ObjectView.STR_MVC_CLASS_NAME).Display());

				HTMLTable tableDocLinks = DirConstants.DisplayDocumentLinksTable(m_asOwner,
						m_asOwner.GetString(1662, "ptmsgs_portalbrowsingmsgs"), 
						m_asOwner.GetString(1663, "ptmsgs_portalbrowsingmsgs"), 
						m_asOwner.GetString(1213, "ptmsgs_portalbrowsingmsgs"));

				if (null != tableDocLinks)
				{
					foldersAndDocumentsGroup.AddInnerHTMLElement(tableDocLinks);
				}
			}
		}
		catch (HTMLException he)
		{
			
				log.Error(he, "Error in DirEditDP: " + he.getMessage());
			
		}

		return table;
	}

	/**
	 * This is a helper method that all classes that inherit from AFormDP can override
	 * if they like.  The method provides a hook into the head so that DP's can add javascript.
	 * Simply override and return and HTMLScript.  Also, it is best to return the results of the
	 * DisplayJavaScript functions from the appropriate views.
	 */
	protected HTMLScriptCollection DisplayJavaScriptFromChild()

	{
		HTMLScriptCollection scripts = new HTMLScriptCollection();

		HTMLScript myScript;
		myScript = m_asOwner.GetView(FolderBannerView.STR_MVC_CLASS_NAME)
			.DisplayJavascript();

		if (null != myScript)
		{
			scripts.AddInnerHTMLElement(myScript);
		}

		myScript = m_asOwner.GetView(ObjectView.STR_MVC_CLASS_NAME)
			.DisplayJavascript();

		if (null != myScript)
		{
			scripts.AddInnerHTMLElement(myScript);
		}

		myScript = m_asOwner.GetView(DocumentBannerView.STR_MVC_CLASS_NAME)
			.DisplayJavascript();

		if (null != myScript)
		{
			scripts.AddInnerHTMLElement(myScript);
		}

		// Card Properties script
		myScript = new HTMLScript("text/javaScript");

		XPStringBuilder sb = new XPStringBuilder(100);

		ASURL myCardPropertiesURL = new ASURL(m_asOwner);
		myCardPropertiesURL.SetLinkGetCachedSpace(m_asOwner);
		myCardPropertiesURL.SetDisplayPage(DirCardPropertiesDP.STR_MVC_CLASS_NAME);
		myCardPropertiesURL.SetControl(DirRepostControl.STR_MVC_CLASS_NAME);

		try
		{
			sb = new XPStringBuilder(400);

			// Open Card Properties popup function
			sb.Append("function ")
			.Append(DirConstants.JAVASCRIPT_CARDPROPERTIES_POPUP).Append("(cardID)\n");
			sb.Append("{\n");
			sb.Append("  window.open('")
			.Append(myCardPropertiesURL.GetURLAsString()).Append("&")
			.Append(DirConstants.QS_CARDID).Append("='+").Append("cardID, '_blank1', 'location=no,menubar=no,scrollbars=yes,height=550,width=600');\n");
			sb.Append("}\n");

			// JavaScript to check all the rows
			sb.Append("function ").Append(JAVASCRIPT_CHECK_ALL_FOLDERS)
			.Append("(inputCBAll, cbName)\n").Append("{\n")
			.Append("		var frm = document.").Append(EditorDP.HTMLFORM_NAME)
			.Append(";\n").Append("		var len = frm.elements.length;\n")
			.Append("		var nValue = inputCBAll.checked;\n")
			.Append("		var i = 0;\n").Append("		for(i=0 ; i<len ; i++)\n")
			.Append("		{\n").Append("			if (frm.elements[i].name == cbName)\n")
			.Append("			{\n").Append("				frm.elements[i].checked = nValue;\n")
			.Append("			}\n").Append("		}\n").Append("}\n");

			myScript.AddInnerHTMLString(sb.ToString());
			
			// Add any custom scripts
			if(null != m_strScript) 
			{
				myScript.AddInnerHTMLString(m_strScript);
				m_strScript = null;
			}
		}
		catch (HTMLException he)
		{
			
				log.Error(he, "DirEditDP::DisplayJavascript The following error occured when trying to display the JavaScript: " +
					he.getMessage());
			

			throw new XPException(m_asOwner.GetString(107,
					"ptmsgs_portalbrowsingmsgs"), he);
		}

		scripts.AddInnerHTMLElement(myScript);

		return scripts;
	}
	
	/**
	 * Set the script that will be added in the header. 
	 *
	 * @param script the script to add before closing
	 */
	public void SetScript(String script)
	{
		m_strScript = script;
	}
}

