package com.plumtree.portalpages.browsing.directory.relatedresources;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portalpages.common.folder.*;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author StephenC
 */
public class RelatedResourcesMorePopupView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.directory.relatedresources.RelatedResourcesMorePopupView");

	
	public static final String STR_MVC_CLASS_NAME = "RelatedResourcesMorePopupView";

	public static final String HTMLBUTTON_EDITOR_OKAY = "in_bu_Okay";
	private AActivitySpace m_asOwner;
	private IModelRO m_model;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new RelatedResourcesMorePopupView();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
	 */
	public HTMLElement Display()
	{
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLSpan mySpan;
		HTMLInput myInput;
		HTMLH1 headerH1;
		
		HTMLTable mainTable = new HTMLTable();
		mainTable.SetBorder("0");
		mainTable.SetWidth("100%");
		mainTable.SetCellPadding("0");
		mainTable.SetCellSpacing("0");

		HTMLTable sectionTable;
		HTMLTable innerTable;

		try
		{
			int nCategoryID		    = ((IDirModelRO) m_model).GetRRCategory();
			int nCategoryClass = -1;
			switch (nCategoryID)
			{
				case PT_INTRINSICS.PT_RELATED_RESOURCE_COMMUNITIES:
					nCategoryClass = PT_CLASSIDS.PT_COMMUNITY_ID;
					break;
					
				case PT_INTRINSICS.PT_RELATED_RESOURCE_FOLDERS:
					nCategoryClass = PT_CLASSIDS.PT_CATALOGFOLDER_ID;
					break;
	
				case PT_INTRINSICS.PT_RELATED_RESOURCE_GADGETS:
					nCategoryClass = PT_CLASSIDS.PT_GADGET_ID;
					break;
	
				case PT_INTRINSICS.PT_RELATED_RESOURCE_EXPERTS:
				case PT_INTRINSICS.PT_RELATED_RESOURCE_CONTENT_MANAGERS:
					nCategoryClass = PT_CLASSIDS.PT_USER_ID;
					break;
			}

			myRow				    = new HTMLTableRow();
			mainTable.AddInnerHTMLElement(myRow);
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			sectionTable = new HTMLTable();
			sectionTable.SetBorder("0");
			sectionTable.SetWidth("100%");
			sectionTable.SetCellPadding("5");
			sectionTable.SetCellSpacing("0");
			myCell.AddInnerHTMLElement(sectionTable);

			// BEGIN Banner and Edit and Close buttons
			myRow = new HTMLTableRow();
			sectionTable.AddInnerHTMLElement(myRow);
			myRow.SetStyleClass(PTStyleClass.BAN_HEADER_BG);

			// title
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);
			headerH1 = new HTMLH1();
			myCell.AddInnerHTMLElement(headerH1);
			myCell.SetHeight("30");
			headerH1.AddInnerHTMLString(m_asOwner.GetString(127,
				"ptmsgs_portalbrowsingmsgs"));
			headerH1.SetStyleClass(PTStyleClass.BAN_HEADER);

			// buttons
			myCell = new HTMLTableCell();
			myCell.SetAlign("right");
			myCell.SetWidth("300");
			myRow.AddInnerHTMLElement(myCell);

			// OK button
			myInput = new HTMLInput(HTMLInputTypes.BUTTON,
					HTMLBUTTON_EDITOR_OKAY, "");
			myCell.AddInnerHTMLElement(myInput);


			myInput.SetValue(m_asOwner.GetString(128, "ptmsgs_portalbrowsingmsgs"));
			myInput.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
			myInput.SetOnClick("self.close();");	// mj: removed 'Javascript:' from function name

			myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
			myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);

			// BEGIN Banner and Edit and Close buttons
			// Add extra spacer row
			mainTable.AddInnerHTMLElement(DirConstants.CreateSpacerRow(1, 5));

			myRow = new HTMLTableRow();
			mainTable.AddInnerHTMLElement(myRow);
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			sectionTable = new HTMLTable();
			sectionTable.SetBorder("0");
			sectionTable.SetWidth("100%");
			sectionTable.SetCellPadding("5");
			sectionTable.SetCellSpacing("0");
			myCell.AddInnerHTMLElement(sectionTable);

			// BEGIN Title Text ROW
			myRow = new HTMLTableRow();
			sectionTable.AddInnerHTMLElement(myRow);
			myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);

			// title
			String strFolderName = ((IFolderModelRO) m_model).GetCurrentFolderName();

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);
			mySpan = new HTMLSpan();
			myCell.AddInnerHTMLElement(mySpan);
			mySpan.SetStyleClass(PTStyleClass.OBJECT_TEXT); 
			mySpan.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strFolderName));

			// END Title Text ROW
			// Add extra spacer row
			mainTable.AddInnerHTMLElement(DirConstants.CreateSpacerRow(1, 5));

			myRow = new HTMLTableRow();
			mainTable.AddInnerHTMLElement(myRow);
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			sectionTable = new HTMLTable();
			sectionTable.SetBorder("0");
			sectionTable.SetWidth("100%");
			sectionTable.SetCellPadding("0");
			sectionTable.SetCellSpacing("5");
			myCell.AddInnerHTMLElement(sectionTable);

			myRow = new HTMLTableRow();
			sectionTable.AddInnerHTMLElement(myRow);
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			innerTable = new HTMLTable();
			innerTable.SetBorder("0");
			innerTable.SetWidth("100%");
			innerTable.SetCellPadding("2");
			innerTable.SetCellSpacing("2");
			myCell.AddInnerHTMLElement(innerTable);

			// BEGIN Header ROW
			String strCategoryName = ((IDirModelRO) m_model).GetRelatedResourceString(nCategoryID);

			myRow = new HTMLTableRow();
			myRow.SetStyleClass(PTStyleClass.OBJECT_HEADER_BG);
			innerTable.AddInnerHTMLElement(myRow);
			myCell = new HTMLTableCell();
			myCell.SetColSpan("3");
			myRow.AddInnerHTMLElement(myCell);
			mySpan = new HTMLSpan();
			myCell.AddInnerHTMLElement(mySpan);
			mySpan.SetStyleClass(PTStyleClass.OBJECT_HEADER);
			mySpan.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strCategoryName));

			// END Header ROW
			// BEGIN Column Headers ROW
			myRow = new HTMLTableRow();
			myRow.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);
			innerTable.AddInnerHTMLElement(myRow);

			myCell = new HTMLTableCell();
			myCell.SetWidth("50%");
			myRow.AddInnerHTMLElement(myCell);
			mySpan = new HTMLSpan();
			myCell.AddInnerHTMLElement(mySpan);
			mySpan.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);


			mySpan.AddInnerHTMLString(m_asOwner.GetString(130,
					"ptmsgs_portalbrowsingmsgs"));

			mySpan = new HTMLSpan();
			myCell.AddInnerHTMLElement(mySpan);
			mySpan.SetStyleClass(PTStyleClass.LIST_TEXT);


			mySpan.AddInnerHTMLString(m_asOwner.GetString(131,
					"ptmsgs_portalbrowsingmsgs"));

			myCell = new HTMLTableCell();
			myCell.SetWidth("25%");
			myRow.AddInnerHTMLElement(myCell);
			mySpan = new HTMLSpan();
			myCell.AddInnerHTMLElement(mySpan);
			mySpan.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);


			mySpan.AddInnerHTMLString(m_asOwner.GetString(133,
					"ptmsgs_portalbrowsingmsgs"));

			// END Column Headers ROW
			String strName;
			String strDescription;
			int nObjectID;
			String strModifiedDate;

			Object[][] arRRMoreInfo = ((IDirModelRO) m_model).GetMoreRelatedResources(nCategoryID);
			int nNumRRs = arRRMoreInfo.length;

			boolean bOdd = false;

			for (int i = 0; i < nNumRRs; i++)
			{
				nObjectID		    = XPConvert.ToInteger(arRRMoreInfo[i][0]);
				strName			    = (String) arRRMoreInfo[i][1];
				strDescription	    = (String) arRRMoreInfo[i][2];
				strModifiedDate     = (String) arRRMoreInfo[i][3];

				myRow = new HTMLTableRow();
				innerTable.AddInnerHTMLElement(myRow);
				myRow.SetStyleClass(bOdd ? PTStyleClass.LIST_ITEM_ONE_BG
										 : PTStyleClass.LIST_ITEM_TWO_BG);

				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);
				mySpan = new HTMLSpan();
				myCell.AddInnerHTMLElement(mySpan);
				mySpan.SetStyleClass(PTStyleClass.LIST_SUBTITLE);

				// mj: replaced the following line with a blank constructur and SetOnClick()
				// HTMLAnchor myAnchor = new HTMLAnchor(
				HTMLAnchor myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);

				if(PT_INTRINSICS.PT_RELATED_RESOURCE_GADGETS == nCategoryID)
				{
					String strGadgetURL = PTOpenerLinks.GetJSOpenInNewWindow(
						PTOpenerLinks.GetOpenerJSURLOpenObjID(PT_CLASSIDS.PT_GADGET_ID, nObjectID,
							null, IPTOpenerConstants.OPENER_MODE_VIEW),
						PTOpenerLinks.GetWindowName("" + PT_CLASSIDS.PT_GADGET_ID + "_" + nObjectID),
					    	"500", "500", true);
					myAnchor.SetOnClick(strGadgetURL + "; window.close(); return false;");
				}
				else
				{
					myAnchor.SetOnClick(
							DirConstants.JAVASCRIPT_MORERR_CLICKTHROUGH + "('" +
							PTOpenerLinks.GetOpenerURLOpenObjID(nCategoryClass, nObjectID, IPTOpenerConstants.OPENER_MODE_VIEW, m_asOwner) 
							+ "'); return false;");
				}

				myAnchor.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strName));

				mySpan.AddInnerHTMLElement(myAnchor);

				myCell.AddInnerHTMLElement(new HTMLBr());

				mySpan = new HTMLSpan();
				myCell.AddInnerHTMLElement(mySpan);
				mySpan.SetStyleClass(PTStyleClass.LIST_TEXT);
				mySpan.AddInnerHTMLString(XPHttpUtility.HtmlEncode(
						strDescription));

				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);
				mySpan = new HTMLSpan();
				myCell.AddInnerHTMLElement(mySpan);
				mySpan.SetStyleClass(PTStyleClass.LIST_TEXT);
				mySpan.AddInnerHTMLString(XPHttpUtility.HtmlEncode(
						strModifiedDate));

				bOdd = !bOdd;
			}
		}
		catch (HTMLException he)
		{
			
				log.Error(he, STR_MVC_CLASS_NAME +
					"::Display The following error occured when trying to display the HTML: " +
					he.getMessage());
			


			throw new XPException(m_asOwner.GetString(129,
					"ptmsgs_portalbrowsingmsgs"), he);
		}

		return mainTable;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		HTMLScript myScript = new HTMLScript("text/javaScript");
		XPStringBuilder sb  = new XPStringBuilder(100);

//		ASURL myMoreRRURL = new ASURL(m_asOwner);
//		myMoreRRURL.SetLinkGetCachedSpace(m_asOwner);
//		myMoreRRURL.SetControl(DirRepostControl.STR_MVC_CLASS_NAME);
		
		try
		{
			// Open create folder popup function
			sb = new XPStringBuilder(200);
			sb.Append("function ")
			.Append(DirConstants.JAVASCRIPT_MORERR_CLICKTHROUGH).Append("(rrUrl)\n");
			sb.Append("{\n");
			sb.Append("  window.opener.location=rrUrl;window.close();\n");
//			.Append(myMoreRRURL.GetURLAsString())
//			.Append("&").Append(DirConstants.QS_RRCATEGORY).Append("='+")
//			.Append("rrCategoryID").Append("+'&")
//			.Append(DirConstants.QS_RROBJECTID).Append("='+").Append("rrObjectID;\n");
			sb.Append("}\n");

			myScript.AddInnerHTMLString(sb.ToString());
		}
		catch (HTMLException he)
		{
			
				log.Error(he, "RelatedResourcesMorePopupView::DisplayJavascript The following error occured when trying to display the JavaScript: " +
					he.getMessage());
			


			throw new XPException(m_asOwner.GetString(134,
					"ptmsgs_portalbrowsingmsgs"), he);
		}

		return myScript;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO, AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		m_model		  = model;
		m_asOwner     = parent;
	}
}

