package com.plumtree.portalpages.browsing.directory.relatedresources;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author StephenC
 *
 * List the Related Resources based on the categories to show.
 */
public class RelatedResourcesView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.directory.relatedresources.RelatedResourcesView");

	
	public static final String STR_MVC_CLASS_NAME = "RelatedResources";

	private static final int RELATED_RESOURCE_DISPLAY_LIMIT = 3;
	private AActivitySpace m_asOwner;
	private IDirModelRO m_asModel;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new RelatedResourcesView();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
	 */
	public HTMLElement Display()
	{
		HTMLTable sectionTable;
		HTMLTableRow myRow;
		HTMLTableCell myCell;

		HTMLTable mainTable = new HTMLTable();
		mainTable.SetBorder("0");
		mainTable.SetWidth("100%");
		mainTable.SetCellPadding("0");
		mainTable.SetCellSpacing("0");

		try
		{
			int[] arrRelatedResourceIDs = m_asModel.GetRelatedResourceIDs();

			if (null == arrRelatedResourceIDs)
			{
				return mainTable;
			}

			for (int i = 0; i < arrRelatedResourceIDs.length; i++)
			{
				int nRRCategoryID   = arrRelatedResourceIDs[i];
				IASQueryResult asqr = m_asModel.GetRelatedResource(nRRCategoryID);
				int nNumRRToDisplay = 0;

				boolean bHasSelfSelectExpertRightOnFolder = m_asModel.HasSelfSelectExpertRightOnThisFolder();
				boolean bRelatedExperts					  = arrRelatedResourceIDs[i] == PT_INTRINSICS.PT_RELATED_RESOURCE_EXPERTS;
				boolean bIsRelatedExpert				  = m_asModel.IsRelatedExpert();
				boolean bShowMore						  = false;

				int nAccessType = m_asModel.GetSubPortalAccessType();

				// skip Related Communities if the current user does not have access to Communities
				if((arrRelatedResourceIDs[i] == PT_INTRINSICS.PT_RELATED_RESOURCE_COMMUNITIES) && 
					((nAccessType & PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_COMMUNITIES_ACCESS) == 0))
				{
					continue;
				}

				// skip Related Portlets if the current user does not have access to My Pages
				if((arrRelatedResourceIDs[i] == PT_INTRINSICS.PT_RELATED_RESOURCE_GADGETS) && 
					((nAccessType & PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_MYPAGES_ACCESS) == 0) &&
					((nAccessType & PT_SUBPORTAL_ACCESS_TYPES.PT_SUBPORTAL_COMMUNITIES_ACCESS) == 0))
				{
					continue;
				}

				
				if ((null != asqr) ||
						((true == bHasSelfSelectExpertRightOnFolder) &&
						(true == bRelatedExperts)))
				{
					if (null == asqr)
					{
						nNumRRToDisplay = 0;
					}
					else
					{
						nNumRRToDisplay = asqr.GetCount();
					}

					boolean bDisplaySelfSelectBanner = (true == bRelatedExperts) &&
						(true == bHasSelfSelectExpertRightOnFolder) &&
						(false == bIsRelatedExpert);

					if ((nNumRRToDisplay != 0) || bDisplaySelfSelectBanner)
					{ // always display users banner (to allow self-selecting experts feature)

						//	Get the banner, set the title
						try
						{
							myRow = new HTMLTableRow();
							mainTable.AddInnerHTMLElement(myRow);
							myCell = new HTMLTableCell();
							myCell.SetWidth("100%");
							myRow.AddInnerHTMLElement(myCell);

							sectionTable = new HTMLTable();
							sectionTable.SetBorder("0");
							sectionTable.SetWidth("100%");
							sectionTable.SetCellPadding("0");
							sectionTable.SetCellSpacing("0");
							myCell.AddInnerHTMLElement(sectionTable);

							myRow = new HTMLTableRow();
							sectionTable.AddInnerHTMLElement(myRow);
							myRow.SetStyleClass(PTStyleClass.MENU_HEADER_BG);

							myCell = new HTMLTableCell();
							myRow.AddInnerHTMLElement(myCell);
							myCell.SetWidth("100%");
							myCell.SetHeight("20");

							String strBannerTitle = m_asModel.GetRelatedResourceString(arrRelatedResourceIDs[i]);
							PTConstruct construct = UIConstructFactory.GetConstruct(PTBannerType.MENU);
							construct.ReplaceNestedElementsWithHTMLString(PTBannerType.MENU.GetElementName(
									PTBannerType.ELEMENT_NAME_ID_TITLE),
								strBannerTitle);
							myCell.AddInnerHTMLElement(construct.GetHTMLElement());

							// add Sub-banner for Self-Selecting Experts
							if (true == bDisplaySelfSelectBanner)
							{
								myRow = new HTMLTableRow();
								mainTable.AddInnerHTMLElement(myRow);
								myCell = new HTMLTableCell();
								myRow.AddInnerHTMLElement(myCell);

								sectionTable = new HTMLTable();
								sectionTable.SetBorder("0");
								sectionTable.SetWidth("100%");
								sectionTable.SetCellPadding("0");
								sectionTable.SetCellSpacing("0");
								myCell.AddInnerHTMLElement(sectionTable);

								myRow = new HTMLTableRow();
								sectionTable.AddInnerHTMLElement(myRow);
								myRow.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_BG);

								myCell = new HTMLTableCell();
								myRow.AddInnerHTMLElement(myCell);
								myCell.SetHeight("25");

								// Create banner
								PTConstruct bannerConstruct = UIConstructFactory.GetConstruct(PTBannerType.DIRECTORY_ACTION_BAR); 
								myCell.AddInnerHTMLElement(bannerConstruct.GetHTMLElement());

								// Add Self-Select Button 
								PTConstruct buttonAddExpertConstruct = UIConstructFactory.GetConstruct(PTButtonType.ADD_ME_ACTIONBAR,
										true, this.m_asOwner); 


								buttonAddExpertConstruct.ReplaceNestedElementsWithHTMLString(PTButtonType.ADD_ME_ACTIONBAR.TITLE_ELEMENT_NAME,
									m_asOwner.GetString(136,
										"ptmsgs_portalbrowsingmsgs"));

								HTMLAnchor myAnchor = (HTMLAnchor) buttonAddExpertConstruct.GetNestedElement(PTButtonType.ADD_ME_ACTIONBAR.ANCHOR_ELEMENT_NAME);
								myAnchor.SetHRef(ConstructStaticHelpers.BLANK_URL);
								myAnchor.SetOnClick(AFormDP.JAVASCRIPT_SUBMIT_FORM +
									"(" +
									DirRepostControl.POSTTOSELF_ACTION_SELFSELECTEXPERTS +
									"); return false;");

								bannerConstruct.AddElementToNestedElement(PTBannerType.DIRECTORY_ACTION_BAR.EXTENDABLE_CELL_ELEMENT_NAME,
									buttonAddExpertConstruct.GetHTMLElement());
							}
						}
						catch (Exception e)
						{
									
								log.Error(e, "Error in RelatedResourcesViews::Display: ");
							
						}
					}

					if (nNumRRToDisplay > RELATED_RESOURCE_DISPLAY_LIMIT)
					{
						nNumRRToDisplay     = RELATED_RESOURCE_DISPLAY_LIMIT;
						bShowMore		    = true;
					}
				}

				if (0 < nNumRRToDisplay)
				{
					myRow = new HTMLTableRow();
					mainTable.AddInnerHTMLElement(myRow);
					myCell = new HTMLTableCell();
					myRow.AddInnerHTMLElement(myCell);

					sectionTable = new HTMLTable();
					sectionTable.SetBorder("0");
					sectionTable.SetWidth("100%");
					sectionTable.SetCellPadding("0");
					sectionTable.SetCellSpacing("0");
					myCell.AddInnerHTMLElement(sectionTable);

					// Display values
					for (int j = 0; j < nNumRRToDisplay; j++)
					{
						String strValue = (String) asqr.GetFields(j,
								PT_PROPIDS.PT_PROPID_NAME);
						int nObjectID   = XPConvert.ToInteger(asqr.GetFields(j,
								PT_PROPIDS.PT_PROPID_OBJECTID));

						myRow = new HTMLTableRow();
						sectionTable.AddInnerHTMLElement(myRow);
						myRow.SetStyleClass(PTStyleClass.MENU_TEXT);

						myCell = new HTMLTableCell();
						myRow.AddInnerHTMLElement(myCell);
						myCell.SetHeight("20");

						String strJSURL = "";
						HTMLImg myImg = null;

						switch (nRRCategoryID)
						{
						case PT_INTRINSICS.PT_RELATED_RESOURCE_COMMUNITIES:
							strJSURL = PTOpenerLinks.GetJSOpenInSameWindow(
								PTOpenerLinks.GetOpenerJSURLOpenObjID(PT_CLASSIDS.PT_COMMUNITY_ID,
									nObjectID, null, IPTOpenerConstants.OPENER_MODE_VIEW));							
							myImg = (HTMLImg) PTImageType.ICON_COMMUNITY.GetHTMLElement();
							break;

						case PT_INTRINSICS.PT_RELATED_RESOURCE_FOLDERS:
							strJSURL = PTOpenerLinks.GetJSOpenInSameWindow(
								PTOpenerLinks.GetOpenerJSURLOpenObjID(PT_CLASSIDS.PT_CATALOGFOLDER_ID, 
							        nObjectID, null, IPTOpenerConstants.OPENER_MODE_VIEW));
							myImg = (HTMLImg) PTImageType.ICON_FOLDER.GetHTMLElement();
							break;

						case PT_INTRINSICS.PT_RELATED_RESOURCE_GADGETS:
							// SCC I am forcing this to be wide.  It is too expensive to query the database just to retrieve this preference.
							strJSURL = PTOpenerLinks.GetJSOpenInNewWindow(
								PTOpenerLinks.GetOpenerJSURLOpenObjID(PT_CLASSIDS.PT_GADGET_ID,
									nObjectID, null, IPTOpenerConstants.OPENER_MODE_VIEW),
								PTOpenerLinks.GetWindowName("" + PT_CLASSIDS.PT_GADGET_ID + "_" + nObjectID),
							    	"500", "500", true);
							myImg = (HTMLImg) PTImageType.ICON_PORTLET.GetHTMLElement();
							break;

						case PT_INTRINSICS.PT_RELATED_RESOURCE_EXPERTS:
						case PT_INTRINSICS.PT_RELATED_RESOURCE_CONTENT_MANAGERS:
							strJSURL = PTOpenerLinks.GetJSOpenInSameWindow(
								PTOpenerLinks.GetOpenerJSURLOpenObjID(PT_CLASSIDS.PT_USER_ID,
									nObjectID, null, IPTOpenerConstants.OPENER_MODE_VIEW));
							myImg = (HTMLImg) PTImageType.ICON_USER.GetHTMLElement();
							break;
						}

						HTMLAnchor myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
						myAnchor.SetOnClick(strJSURL + "; return false;");
						myAnchor.AddInnerHTMLElement(myImg);
						myAnchor.AddInnerHTMLString(CommonHTMLStrings.SPACE);
						myAnchor.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strValue));
						myCell.AddInnerHTMLElement(myAnchor);
					}
					 // for

					// Display the More... link				
					if (bShowMore)
					{
						myRow = new HTMLTableRow();
						sectionTable.AddInnerHTMLElement(myRow);
						myRow.SetStyleClass(PTStyleClass.MENU_TEXT);

						myCell = new HTMLTableCell();
						myRow.AddInnerHTMLElement(myCell);
						myCell.SetHeight("20");
						myCell.SetAlign("right");

						// mj: replaced the following line with a blank constructur and SetOnClick()
						// HTMLAnchor myAnchor = new HTMLAnchor(
						HTMLAnchor myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
						myAnchor.SetOnClick(
								DirConstants.JAVASCRIPT_MORERR_POPUP + "(" +
								nRRCategoryID + "," +
								m_asModel.GetCurrentFolderID() + ");");


						myAnchor.AddInnerHTMLString(m_asOwner.GetString(135,
								"ptmsgs_portalbrowsingmsgs"));
						myCell.AddInnerHTMLElement(myAnchor);
					}
				} 
				else if(PT_INTRINSICS.PT_RELATED_RESOURCE_EXPERTS == nRRCategoryID && bHasSelfSelectExpertRightOnFolder) 
				{
					// If there are no experts then show a message.
					myRow = new HTMLTableRow();
					mainTable.AddInnerHTMLElement(myRow);
					myCell = new HTMLTableCell();
					myRow.AddInnerHTMLElement(myCell);

					sectionTable = new HTMLTable();
					sectionTable.SetBorder("0");
					sectionTable.SetWidth("100%");
					sectionTable.SetCellPadding("0");
					sectionTable.SetCellSpacing("0");
					myCell.AddInnerHTMLElement(sectionTable);
					
					// No Experts Message
					myRow = new HTMLTableRow();
					sectionTable.AddInnerHTMLElement(myRow);
					myRow.SetStyleClass(PTStyleClass.MENU_TEXT);

					myCell = new HTMLTableCell();
					myRow.AddInnerHTMLElement(myCell);
					myCell.SetHeight("20");

					myCell.AddInnerHTMLString(m_asOwner.GetString(1217, "ptmsgs_portalbrowsingmsgs"));

					// Add me Link
					myRow = new HTMLTableRow();
					sectionTable.AddInnerHTMLElement(myRow);
					myRow.SetStyleClass(PTStyleClass.MENU_TEXT);

					myCell = new HTMLTableCell();
					myRow.AddInnerHTMLElement(myCell);
					myCell.SetHeight("20");

					HTMLAnchor myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);

					myAnchor.AddInnerHTMLString(m_asOwner.GetString(1218, "ptmsgs_portalbrowsingmsgs"));
					myAnchor.SetHRef(ConstructStaticHelpers.BLANK_URL);
					myAnchor.SetOnClick(AFormDP.JAVASCRIPT_SUBMIT_FORM +
						"(" +
						DirRepostControl.POSTTOSELF_ACTION_SELFSELECTEXPERTS +
						"); return false;");

					myCell.AddInnerHTMLElement(myAnchor);
				}
				else
				{
					continue;
				}

				myRow = new HTMLTableRow();
				mainTable.AddInnerHTMLElement(myRow);
				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);
				myCell.AddInnerHTMLElement(DirConstants.CreateSpacerTable(10,225));
			}
		}
		catch (Exception e)
		{
								
				log.Error(e, "Error in SetServerSettingsModel::Finish: ");
			
		}

		return mainTable;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		HTMLScript myScript = new HTMLScript("text/javaScript");
		XPStringBuilder sb  = new XPStringBuilder(100);

		ASURL myMoreRRURL = new ASURL(m_asOwner);
		myMoreRRURL.SetLinkGetCachedSpace(m_asOwner);
		myMoreRRURL.SetControl(DirRepostControl.STR_MVC_CLASS_NAME);
		myMoreRRURL.SetDisplayPage(RelatedResourcesMorePopupDP.STR_MVC_CLASS_NAME);

		try
		{
			// Open create folder popup function
			sb = new XPStringBuilder(200);
			sb.Append("function ").Append(DirConstants.JAVASCRIPT_MORERR_POPUP)
			.Append("(rrCategoryID, rrFolderID)\n");
			sb.Append("{\n");
			sb.Append("  window.open('").Append(myMoreRRURL.GetURLAsString())
			.Append("&").Append(DirConstants.QS_RRCATEGORY).Append("='+")
			.Append("rrCategoryID").Append("+'&")
			.Append(OpenDirFolderControl.m_SubFolderID).Append("='+").Append("rrFolderID, '_blank1', 'location=no,menubar=no,scrollbars=yes,height=550,width=600');\n");
			sb.Append("}\n");

			myScript.AddInnerHTMLString(sb.ToString());
		}
		catch (HTMLException he)
		{
			
				log.Error(he, "DirEditDP::DisplayJavascript The following error occured when trying to display the JavaScript: " +
					he.getMessage());
			


			throw new XPException(m_asOwner.GetString(138,
					"ptmsgs_portalbrowsingmsgs"), he);
		}

		return myScript;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO, AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		m_asModel     = (IDirModelRO) model;
		m_asOwner     = parent;
	}
}

