package com.plumtree.portalpages.browsing.editors.community;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.OpenLogService;
import com.plumtree.openlog.OpenLogger;
import com.plumtree.portalpages.common.editpage.PageData;
import com.plumtree.portalpages.common.editpage.PageLayoutHeaderView;
import com.plumtree.portaluiinfrastructure.ptlink.IPTURL;
import com.plumtree.portaluiinfrastructure.statichelpers.PTDebugHelpers;
import com.plumtree.server.IPTPage;
import com.plumtree.server.IPTSession;
import com.plumtree.server.PT_PROPIDS;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.expandablelist.ExpListModel;
import com.plumtree.taskapi.portalui.TaskAPIUICommunity;
import com.plumtree.server.*;
import com.plumtree.server.impl.community.*;

/**
 * This control creates a single page in the community without having to go into the community editor
 * @author mattc
 *
 */
public class PageCreateDefaultControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
			"com.plumtree.portalpages.browsing.editors.communityPageCreateDefaultControl");

	/** Object Name */
	public static final String STR_MVC_CLASS_NAME = "PageCreateDefault";

	/** Associated Activity Space */
	public AActivitySpace m_asOwner;

	/** Associated Model */
	protected CommunityEditorModel m_model;

	public static final String HTMLINPUT_COMMUNITY_ID = "commID";
	public static final String HTMLINPUT_PAGE_TEMPLATE_ID = "templateID";
	public static final String HTMLINPUT_INHERIT_TEMPLATE = "inherit";

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new PageCreateDefaultControl();
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	public void Init(IModel model, AActivitySpace space)
	{
		m_model = (CommunityEditorModel)model;
		m_asOwner = space;
	}	
	
	/**
	 * Simply creates a new page and redirects the browser back to that page
	 *
	 * @param arguments contains HTTP arguments from the post
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		// get current session
		IPTSession currSession = (IPTSession)m_asOwner.GetUserSession();
		int communityID = -1;
		boolean bInheritPageTemplate = true;
		int nPageTemplateID = PT_INTRINSICS.PT_BLANK_PAGETEMPLATEID;

		String[] data;
		
		// Community ID
		data = (String[]) arguments.GetElement(HTMLINPUT_COMMUNITY_ID);
		if ((null == data) || (0 == data.length))
		{
			// no communityID specified; 
			// TODO: redirect to error page
			return null;
		}
		communityID = XPConvert.ToInteger(data[0]);

		// Inherit Page Template
		data = (String[]) arguments.GetElement(HTMLINPUT_INHERIT_TEMPLATE);
		if ((null != data) && (0 != data.length))
			bInheritPageTemplate = XPConvert.ToBoolean(data[0]);

		// Template ID
		data = (String[]) arguments.GetElement(HTMLINPUT_PAGE_TEMPLATE_ID);
		if ((null != data) && (0 != data.length))
			nPageTemplateID = XPConvert.ToInteger(data[0]);

		// open the community to get the admin folder
		IPTCommunity community = (IPTCommunity)currSession.GetCommunities().Open(communityID, false);

		IPTPage ptPage = (IPTPage)(currSession).GetPages().Create(community.GetAdminFolderID());
		ptPage.SetPageTemplateID(nPageTemplateID);
		ptPage.SetInheritPageTemplate(bInheritPageTemplate);
		ptPage.SetDefaultsFromPageTemplate(nPageTemplateID);
		ptPage.SetNotDeletable(false);
		
		// Store page
		ptPage.Store();
		ptPage.UnlockObject();
		
		// redirect to the new page
	 	int nPageID = ptPage.GetObjectID();
	 	String strPageName  = ptPage.GetName();
	 	
	 	IPTURL url = TaskAPIUICommunity.CreateCommunityPageCommonOpenerURL(m_asOwner, communityID, nPageID, strPageName);
	 	Redirect redirect = new Redirect();	 	
		redirect = new Redirect();
		redirect.SetLinkToExternalURL(url.GetFullURL());
		redirect.SetIsHTTPRedirect(true);
		return redirect;
	}
}

