package com.plumtree.portalpages.browsing.editors.dircommon.editdocumentproperty;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.admin.editors.property.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.expandablelist.properties.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tree.*;
import com.plumtree.server.*;
import com.plumtree.server.treeiterator.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.expandablelist.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This is the implementation of the Expandable List for editing card properties
 *
 * @author stephenc
 */
public class EditCardPropView extends ExpListPropEditView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.editors.dircommon.editdocumentproperty.EditCardPropView");

	/** Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "EditCardProp";

	
	public static final String HTMLINPUT_SELECT_NEW_PROP	   = "in_se_NewProperty";

	
	public static final String HTMLINPUT_SELECT_NEW_PROP_TAGID = "NewPropTagID";

	public static final String JAVASCRIPT_NEWCARDPROP = "CreateNewCardProp";
	
	/** JavaScript function to open pick tree. */
	protected static final String JAVASCRIPT_PICKTREE_OPEN = "pickTreeOpen";

	/** JavaScript function to get pick tree value. */
	protected static final String JAVASCRIPT_PICKTREE_GETVALUE = "pickTreeGetValue";
	
	/** Prefix for picklist input */
	public static final String HTMLINPUT_SELECT_PICKLIST_PREFIX = "in_se_PickList";

	/** TagID for picklist */
	public static final String HTMLINPUT_SELECT_PICKLIST_TAGID = "PickListTagID";
	
	/** Used to set the style of the row */
	private PTStyleClass rowStyleClass;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new EditCardPropView();
	}

	/**
	 * This method returns the HTMLScript that encompasses the Javascript on the page.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 *
	 * @return HTMLScript
	 */
	public HTMLScript DisplayJavascript()
	{
		IEditCardPropModelRO iModel = (IEditCardPropModelRO)m_model;
		HTMLScript myScript = super.DisplayJavascript();

		XPStringBuilder sb;

		try
		{
			// The URL to open a pick tree
			ASURL asURLPickTree = GetPickTreeURL();

			// JavaScript fucntion to open pick tree
			sb = new XPStringBuilder(200);
			sb.Append("function ").Append(iModel.GetUniqueString(JAVASCRIPT_PICKTREE_OPEN)).Append("(varPropertyID, varRowIndex)\n");
			sb.Append("{\n");
			sb.Append("  document.").Append(AFormDP.HTMLFORM_NAME).Append(".").Append(iModel.GetUniqueString(HTMLHIDDEN_ROWINDEX)).Append(".value = varRowIndex;\n");
			sb.Append("  window.open('").Append(asURLPickTree.GetURLAsString());
			sb.Append("&").Append(ServerStartTreeControl.INIT_DATA).Append("=' + varPropertyID");
			sb.Append(", 'PickTreeWindow', '")
			.Append(TreeConstants.GetJavascriptTreeWindowAttributes()).Append("');\n");
			sb.Append("}\n");
			myScript.AddInnerHTMLString(sb.ToString());
			
			// JavaScript function to get pick tree value on return
			sb = new XPStringBuilder(100);
			sb.Append("function ").Append(iModel.GetUniqueString(JAVASCRIPT_PICKTREE_GETVALUE)).Append("()\n");
			sb.Append("{\n");
			sb.Append("  ").Append(AFormDP.JAVASCRIPT_SUBMIT_FORM).Append("(").Append(iModel.GetUniqueInt(ExpListControl.POSTTOSELF_ACTION_MODIFYROW)).Append(");\n");
			sb.Append("}\n");
			myScript.AddInnerHTMLString(sb.ToString());
			
			// Create new object
			sb = new XPStringBuilder(700);
			sb.Append(" function " + JAVASCRIPT_NEWCARDPROP + "(varURL)");
			sb.Append(" {");
			sb.Append(
				"    window.open(varURL,'PopUpCreateMode','scrollbars=yes,location=no,menubar=no,height=600,width=800');\n");
			sb.Append(" }\n");
			
			myScript.AddInnerHTMLString(sb.ToString());
		}
		catch (HTMLException he)
		{
			
				log.Error(he, "EditCardPropView::DisplayJavascript The following error occured when trying to display the JavaScript: " +
					he.getMessage());
			
		}

		return myScript;
	}
	
	/**
	 * Returns HTML for pick list
	 *
	 * @param _nRowIndex Row index
	 * @param _strValue Property value as string
	 *
	 * @return HTML for pick list
	 *
	 * @throws HTMLException
	 */
	protected HTMLElementCollection GetHTMLForPickList(int _nRowIndex, String _strValue)
	{
		HTMLElementCollection myCollection = new HTMLElementCollection();
		
		IEditCardPropModelRO iModel = (IEditCardPropModelRO) m_model;

		if (iModel.CheckPickValuesAreValid(_nRowIndex))
		{
			HTMLSelect mySelect = new HTMLSelect(iModel.GetUniqueString(HTMLINPUT_SELECT_PICKLIST_PREFIX) + _nRowIndex, iModel.GetUniqueString(HTMLINPUT_SELECT_PICKLIST_TAGID) + _nRowIndex);
			myCollection.AddInnerHTMLElement(mySelect);
	
			// Add the first option, which is empty value
			HTMLOption optionEmpty = new HTMLOption("");
			mySelect.AddInnerHTMLElement(optionEmpty);
			mySelect.SetStyleClass(PTStyleClass.FORM_SELECT_TEXT);
	
			// Open the property ObjectManager in order to open property.
			Object[][] arPickListValues = iModel.GetPickListValues(iModel.GetRowPropertyID(_nRowIndex));
	
			boolean bValueInList = false;
			
			// Add drop down
			if (null != arPickListValues)
			{
				for (int nOption = 0; nOption < arPickListValues[0].length;
						nOption++)
				{
					String strPickListVal = (String) arPickListValues[PT_TREE_ITERATOR_CHILD_INFO.PT_TREE_ITERATOR_CHILD_INFO_NAME][nOption];
									
					String strEncodedString = XPHttpUtility.HtmlEncode(strPickListVal);
					HTMLOption optionPickListVal = new HTMLOption(strEncodedString);
					optionPickListVal.AddInnerHTMLString(strEncodedString);
	
					mySelect.AddInnerHTMLElement(optionPickListVal);
					
					if (strPickListVal.equals(_strValue))
					{
						bValueInList = true;
					}
				}
			}
	
			if (null != _strValue && 0 < _strValue.length())
			{
				// If the option doesn't exist as a valid pick list value, then display alert message.		
				if (!bValueInList)
				{
					// Display alert message indicating that value is no longer valid
					HTMLSpan mySpan = new HTMLSpan();
					myCollection.AddInnerHTMLElement(mySpan);
					mySpan.SetStyleClass(PTStyleClass.ALERT_REQUIRED_TEXT);
					mySpan.AddInnerHTMLEncodedString(XPResourceManager.FormatMessage(m_asOwner.GetString(469, "ptmsgs_portalinfrastructure"), _strValue));
				}
				else
				{
					// Set the selected value
					mySelect.SetValue(XPHttpUtility.HtmlEncode(_strValue));
				}
			}
		}
		else
		{
			myCollection.AddInnerHTMLEncodedString(_strValue);
			myCollection.AddInnerHTMLString(" ");
			
			HTMLSpan mySpan = new HTMLSpan();
			myCollection.AddInnerHTMLElement(mySpan);
			mySpan.SetStyleClass(PTStyleClass.ALERT_REQUIRED_TEXT);
			mySpan.AddInnerHTMLEncodedString(m_asOwner.GetString(471, "ptmsgs_portalinfrastructure"));
		}
		return myCollection;
	}

	/**
	 * Returns HTML for pick tree
	 *
	 * @param _nRowIndex Row index
	 * @param _strValue Property value as string
	 *
	 * @return HTML for pick tree
	 *
	 * @throws Exception
	 */
	protected HTMLElement GetHTMLForPickTree(int _nRowIndex, String _strValue)
	{
		IEditCardPropModelRO iModel = (IEditCardPropModelRO) m_model;

		HTMLAnchor myAnchor;
		HTMLImg myImage;
		PTConstruct myConstruct;
		XPStringBuilder sbJavascriptOpenPickTree;
		HTMLElementCollection result = new HTMLElementCollection();

		result.AddInnerHTMLEncodedString(_strValue);

		if (iModel.CheckPickValuesAreValid(_nRowIndex))
		{
			myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
			result.AddInnerHTMLElement(myAnchor);
			myConstruct     = UIConstructFactory.GetConstruct(PTImageType.ACTION_EDIT, true, this.m_asOwner);
			myImage = (HTMLImg) myConstruct.GetHTMLElement();
			myAnchor.AddInnerHTMLElement(myImage);
	
			myImage.SetAlt(m_asOwner.GetString(89, "ptmsgs_portalinfrastructure"));
			myImage.SetWidth("25");
			myImage.SetHeight("25");
			myImage.SetBorder(CommonHTMLStrings.ZERO);
			myImage.SetAlign(CommonHTMLStrings.ABSMIDDLE);
	
			sbJavascriptOpenPickTree = new XPStringBuilder(100);
			sbJavascriptOpenPickTree.Append(iModel.GetUniqueString(JAVASCRIPT_PICKTREE_OPEN)).Append("(");
			sbJavascriptOpenPickTree.Append(iModel.GetRowPropertyID(_nRowIndex)); // Append property ID
			sbJavascriptOpenPickTree.Append(", ");
			sbJavascriptOpenPickTree.Append(_nRowIndex); // Append current row to modify
			sbJavascriptOpenPickTree.Append("); return false;");
			myAnchor.SetOnClick(sbJavascriptOpenPickTree.ToString());
		}
		else
		{
			result.AddInnerHTMLString(" ");
			
			HTMLSpan mySpan = new HTMLSpan();
			result.AddInnerHTMLElement(mySpan);
			mySpan.SetStyleClass(PTStyleClass.ALERT_REQUIRED_TEXT);
			
			mySpan.AddInnerHTMLEncodedString(m_asOwner.GetString(472, "ptmsgs_portalinfrastructure"));
		}

		return result;
	}

	
	/**
	 * Returns the HTML for the row _nRowIndex. Each row contains just an icon
	 * and input text.
	 *
	 * @param _saData
	 * @param _nRowIndex
	 *
	 * @return HTMLElement
	 *
	 * @throws Exception
	 */
	public HTMLElement GetHTMLForRow(IPTGrowableSortedArrayWrapperRO _saData,
		int _nRowIndex)
	{
		IEditCardPropModelRO iECPModel = (IEditCardPropModelRO) m_model;

		HTMLElementCollection result = new HTMLElementCollection();
		HTMLTableCell myCell;

		result.AddInnerHTMLElement(GetHTMLForName(_saData, _nRowIndex));

		myCell = new HTMLTableCell();
		result.AddInnerHTMLElement(myCell);
		myCell.SetWidth("80%");
		myCell.SetHeight("30");
		myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
		myCell.SetNoWrap(true);
		myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);

		Object oValue = iECPModel.GetRowValue(_nRowIndex);
		int nPropertyType = iECPModel.GetRowType(_nRowIndex);
		String strValue = iECPModel.GetStringFromObjectValue(oValue, nPropertyType);
		
		// Determine what kind of form element to display
		int nPickType = iECPModel.GetPickType(_nRowIndex);
		
		switch (nPickType)
		{
		case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_MANAGED_PICKLIST:
		case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_UNMANAGED_PICKLIST:

			// Get drop down menu for picklist
			myCell.AddInnerHTMLElement(GetHTMLForPickList(_nRowIndex, strValue));

			break;

		case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_PICKTREE:

			// Get picktree link
			myCell.AddInnerHTMLElement(GetHTMLForPickTree(_nRowIndex, strValue));

			break;

		default:
			myCell.AddInnerHTMLElement(GetHTMLElementForRow(_nRowIndex, oValue));
			break;
		}
		
		return result;
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Empty method that sub classes can averride to add
	 * actions to the action bar (like a create object action).
	 *
	 * @param _myRow Action bar row object.
	 * @throws Exception
	 */
	protected void AddHTMLToActionBar(HTMLTableRow _myRow)
	{
		HTMLTableCell myCell;
		HTMLImg myImage;
		HTMLAnchor myAnchor;
		HTMLSpan mySpan;

		int nClassID		 = PT_CLASSIDS.PT_PROPERTY_ID;
		String sEditorASName = PropEditorAS.STR_MVC_CLASS_NAME;

		ASURL myURL = new ASURL(m_asOwner);

		myURL.SetLinkCreateNewSpace(sEditorASName, m_asOwner);
		myURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
		myURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE,
			XPConvert.ToString(EditorStartControl.EDITOR_START_FLAG_CREATE));
		myURL.AddControlArgument(EditorStartControl.QS_DELETE_ON_CANCEL,
			XPConvert.ToString(true));
		myURL.SetSubSpaceKey(sEditorASName);
		myURL.AddControlArgument(ObjEditorModel.EDITOR_QS_INT_CLASS_ID,
			XPConvert.ToString(nClassID));

		myCell = new HTMLTableCell();
		_myRow.AddInnerHTMLElement(myCell);
		myCell.SetHeight("25");
		myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
		myCell.SetNoWrap(true);
		myCell.AddInnerHTMLString(SPACE);

		// mj: replaced the following line with a blank constructur and SetOnClick()
		// myAnchor = new HTMLAnchor(JAVASCRIPT_NEWCARDPROP +
		myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
		myAnchor.SetOnClick(JAVASCRIPT_NEWCARDPROP +
				"('" + myURL.GetURLAsString() + "');");
		HTMLImg myNewImage = (HTMLImg) PTImageType.ACTION_NEW.GetHTMLElement();
		myNewImage.SetAlt("");
		myNewImage.SetAlign("absmiddle");

		myAnchor.AddInnerHTMLElement(myNewImage);

		// property icon
		myImage = (HTMLImg) PTImageType.ICON_PROPERTY.GetHTMLElement();
		myImage.SetAlt("");
		myImage.SetAlign("absmiddle");

		myAnchor.AddInnerHTMLElement(myImage);
		mySpan = new HTMLSpan();
		myAnchor.AddInnerHTMLElement(mySpan);
		mySpan.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);

		// Create label
		mySpan.AddInnerHTMLString(m_asOwner.GetString(350,
				"ptmsgs_portalbrowsingmsgs"));
		mySpan.AddInnerHTMLString(SPACE);
		mySpan.AddInnerHTMLString(SPACE);
		myCell.AddInnerHTMLElement(myAnchor);
		myCell.AddInnerHTMLString(SPACE);
	}

	/**
	 * 
	 *
	 * @param _nRowIndex 
	 *
	 * @return 
	 *
	 * @throws HTMLException 
	 */
	protected HTMLTableCell GetHTMLForCheckBox(int _nRowIndex)

	{
		IEditCardPropModelRO iECPModel = (IEditCardPropModelRO) m_model;

		HTMLTableCell myCell;
		HTMLInput myInput;

		myCell = new HTMLTableCell();
		myCell.SetHeight("30");
		myCell.SetAlign("center");
		myCell.SetVAlign("middle");
		myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

		// checkbox
		myInput = new HTMLInput(HTMLInputTypes.CHECKBOX,
				iECPModel.GetUniqueString(HTMLCHECKBOX_DELROW),
				iECPModel.GetDeleteIconAlt());
		myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
		myInput.SetValue(XPConvert.ToString(_nRowIndex));
		myCell.AddInnerHTMLElement(myInput);

		return myCell;
	}

	
	/**
	 * 
	 *
	 * @param _saData 
	 * @param _nRowIndex 
	 *
	 * @return 
	 *
	 * @throws HTMLException 
	 */
	protected HTMLTableCell GetHTMLForName(
		IPTGrowableSortedArrayWrapperRO _saData, int _nRowIndex)

	{
		IEditCardPropModelRO iECPModel = (IEditCardPropModelRO) m_model;

		HTMLTableCell myCell = new HTMLTableCell();
		myCell.SetWidth("22%");
		myCell.SetHeight("30");
		myCell.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);

		String strName = iECPModel.GetRowName(_nRowIndex);
		int nStatus    = iECPModel.GetRowStatus(_nRowIndex);

		// Display name
		if (EditCardPropModel.ROW_STATUS_ADDED_NEW == nStatus)
		{
			myCell.AddInnerHTMLElement(GetUnusedPropertiesHTMLSelect());
		}
		else
		{
			myCell.AddInnerHTMLString("<B>" +
				XPHttpUtility.HtmlEncode(strName) + "</B>");
		}

		return myCell;
	}

	/**
	 * Generates a list of properties to be added.  Only card props that have not been chosen are offered
	 *
	 * @return 
	 *
	 * @throws HTMLException 
	 */
	protected HTMLSelect GetUnusedPropertiesHTMLSelect()

	{
		IEditCardPropModelRO iECPModel = (IEditCardPropModelRO) m_model;

		HTMLSelect mySelect = new HTMLSelect(HTMLINPUT_SELECT_NEW_PROP,
				HTMLINPUT_SELECT_NEW_PROP_TAGID);
		mySelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
		mySelect.SetOnChange(AFormDP.JAVASCRIPT_SUBMIT_FORM + "(" +
			RepostControl.POSTTOSELF_ACTION_SIMPLEREPOST + ");");

		XPSortedDictionary xpUnusedProperties = iECPModel.GetUnusedProperties();

		IXPEnumerator xpEnum;
		IXPEnumerator xpKeys;

		xpEnum     = xpUnusedProperties.GetElements();
		xpKeys     = xpUnusedProperties.GetKeys();

		int nNewlySelectedPropertyID = iECPModel.GetNewlySelectedPropertyID();

		while (xpEnum.MoveNext() && xpKeys.MoveNext())
		{
			String strPropName = (String) xpKeys.GetCurrent();
			int nPropID		   = XPConvert.ToInteger(xpEnum.GetCurrent());

			HTMLOption optionVal = new HTMLOption(XPConvert.ToString(nPropID));

			if (nNewlySelectedPropertyID == nPropID)
			{
				optionVal.SetSelected(true);
			}

			mySelect.AddInnerHTMLElement(optionVal);
			optionVal.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strPropName));
		}

		return mySelect;
	}

	/**
	  * Helper method to set the style of a row.  This method is used by
	  * Display(). Overrides the ExpListView::SetExpListRowStyle().
	  *
	  * @param _myRow
	  * @param _nRowIndex
	  * @param _nLastVisibleRow
	  *
	  * @throws HTMLException
	  *
	  * @see com.plumtree.uiinfrastructure.expandablelist.ExpListView#SetExpListRowStyle(HTMLTableRow,
	  *      int, int)
	 */
	protected void SetRowStyle(HTMLTableRow _myRow, int _nRowIndex,
		int _nLastVisibleRow)

	{
		IEditCardPropModelRO iOPModel = (IEditCardPropModelRO) m_model;

		// Get current property ID
		int nPropertyID = iOPModel.GetRowPropertyID(_nRowIndex);

		// Get last visible property ID
		int nLastVisiblePropertyID = PT_INTRINSICS.PT_NULL_OBJECTID;

		if (-1 != _nLastVisibleRow)
		{
			nLastVisiblePropertyID = iOPModel.GetRowPropertyID(_nLastVisibleRow);
		}

		// Check if we have moved on to a new property
		if (nPropertyID != nLastVisiblePropertyID)
		{
			// Toggle style index if we have a new property
			if (PTStyleClass.LIST_ITEM_ONE_BG == rowStyleClass)
			{
				rowStyleClass = PTStyleClass.LIST_ITEM_TWO_BG;
			}
			else
			{
				rowStyleClass = PTStyleClass.LIST_ITEM_ONE_BG;
			}
		}

		_myRow.SetStyleClass(rowStyleClass);
	}
	
	
	/**
	 * Return ASURL for pick tree
	 *
	 * @return ASURL for pick tree
	 */
	protected ASURL GetPickTreeURL()
	{
		IEditCardPropModelRO iModel = (IEditCardPropModelRO) m_model;

		// The URL to open a pick tree as a subspace.  We want to use the same pick tree each time.
		ASURL asURLPickTree = new ASURL(m_asOwner);

		asURLPickTree.SetLinkCreateNewSpace(ServerTreeAS.STR_MVC_CLASS_NAME, m_asOwner);

		asURLPickTree.SetSubSpaceKey(iModel.GetUniqueString(EditCardPropModel.SUBSPACE_KEY_PICKTREE));
		asURLPickTree.SetControl(ServerStartTreeControl.STR_MVC_CLASS_NAME);
		asURLPickTree.AddControlArgument(ServerStartTreeControl.INTERFACE_TYPE, XPConvert.ToString(ServerTreeModel.m_nPROP_PICK_TREE));
		asURLPickTree.AddControlArgument(AStartTreeControl.MULTI_SELECT, XPConvert.ToString(false));
		asURLPickTree.AddControlArgument(AStartTreeControl.SELECT_MODE, XPConvert.ToString(TreeConstants.N_LEAFSELECT));
		asURLPickTree.AddControlArgument(AStartTreeControl.SUBMIT_MODE, XPConvert.ToString(TreeConstants.N_SERVERSUBMIT));
		asURLPickTree.AddControlArgument(AStartTreeControl.PARENT_FORM_NAME, AFormDP.HTMLFORM_NAME);
		asURLPickTree.AddControlArgument(AStartTreeControl.PARENT_SUBMIT_CALL, iModel.GetUniqueString(JAVASCRIPT_PICKTREE_GETVALUE));
		asURLPickTree.AddControlArgument(AStartTreeControl.TITLE, m_asOwner.GetString(91, "ptmsgs_portalinfrastructure"));
		asURLPickTree.AddControlArgument(AStartTreeControl.ACTIONSUBTITLE, m_asOwner.GetString(90, "ptmsgs_portalinfrastructure"));
		asURLPickTree.AddControlArgument(AStartTreeControl.SHOW_ROOT, XPConvert.ToString(false));

		return asURLPickTree;
	}
}

