package com.plumtree.portalpages.browsing.editors.multipledocumentlinksettings;


// Framework Imports
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.portalpages.common.linksettings.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This is class is used to print out together the Views for
 * the General Info page of the MultipleCardLinkSettings Editor.
 *
 * @author StephenC
 */
public class MultipleCardLinkSettingsMainDP extends EditorDP
{
	
	public static final String STR_MVC_CLASS_NAME = "MultipleCardLinkSettings";

	/**
	 * Displays some HTML after the body tag and before the form tag.
	 * For example this is used by pages using a DHTML calendar to display the div tag.
	 * The function is optional.  If it exists, it will be called by the framework, and
	 * the resulting HTMLElement will be added in the right place.
	 *
	 * @return HTMLElement
	 */
	public HTMLElementCollection DisplayHTMLBeforeForm()
	{
		HTMLElementCollection myHTMLBeforeForm = null;

		try
		{
			//Display the DIV tag required for the Date picker control
			ILinkSettingsPlugInModelRO ptROModel = (ILinkSettingsPlugInModelRO) m_asOwner.GetModel(MultipleCardLinkSettingsAS.strLinkSettingsPlugInKey);
			myHTMLBeforeForm = ptROModel.GetDateTimePickerControlManager().GetPagePreFormContent();
		}
		catch (Exception e)
		{
			UIError myError = new UIError(UIErrorLevel.WARNING,
						UIErrorScope.CURRENT_USER,
						null,
						m_asOwner.GetString(697, "ptmsgs_portalcommonmsgs"),
						"");

			myError.SetActivitySpace(m_asOwner.GetName());

			UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),m_asOwner.GetApplication());
		}

		return myHTMLBeforeForm;
	}

	/**
	 * Returns the HTML that is placed in the HEAD section of this HTML page.
	 *
	 * @return HTMLElement
	 */
	public HTMLElementCollection DisplayHTMLinHEAD()
	{
		String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE,
				GetOwner());

		HTMLElementCollection retVal = new HTMLElementCollection();

		try
		{
			ILinkSettingsPlugInModelRO ptROModel = (ILinkSettingsPlugInModelRO) m_asOwner.GetModel(MultipleCardLinkSettingsAS.strLinkSettingsPlugInKey);
			JSDateTimePickerMgr ptDatePickerMgr = ptROModel.GetDateTimePickerControlManager();
			retVal.AddInnerHTMLElement(ptDatePickerMgr.GetJSIncludes());
		}
		catch (Exception e)
		{
			UIError myError = new UIError(UIErrorLevel.WARNING,
						UIErrorScope.CURRENT_USER,
						null,
						m_asOwner.GetString(697, "ptmsgs_portalcommonmsgs"),
						"");

			myError.SetActivitySpace(m_asOwner.GetName());

			UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),m_asOwner.GetApplication());
		}
		
		//Get access to the PTDate.js file for formatting date strings
		
		retVal.AddInnerHTMLElement(ConfigHelper.GetJSIncluderJSComponentInclude(ConfigHelper.JSCOMPONENT_JSUTIL, GetOwner()));
		return retVal;
	}


	/**
	 * Returns the String to display on the left navigation menu,
	 * linking to that page.
	 *
	 * @return String
	 */
	public String GetLeftNavString()
	{
		return (m_asOwner.GetString(466, "ptmsgs_portalbrowsingmsgs"));
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the HTML for the General Info page.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.AFormDP#PageDisplay(HTMLTableRow)
	 *
	 * @return HTMLElement
	 * @throws HTMLException
	 */
	public HTMLElement PageDisplay()
	{
		HTMLElementCollection result = new HTMLElementCollection();

		// Insert super class HTML
		result.AddInnerHTMLElement(super.PageDisplay());

		HTMLElementCollection views = new HTMLElementCollection();

		views.AddInnerHTMLElement(m_asOwner.GetView(
				MultipleCardLinkSettingsAS.strLinkSettingsPlugInKey).Display());

		result.AddInnerHTMLElement(CreateLeftNavBar(views));

		return result;
	}

	/**
	 * Display the Javascripts for the Views.
	 *
	 * @return HTMLScriptCollection
	 * @throws HTMLException
	 */
	protected HTMLScriptCollection DisplayJavaScriptFromChild()
	{
		HTMLScriptCollection myScripts = super.DisplayJavaScriptFromChild();

		myScripts.AddInnerHTMLElement(m_asOwner.GetView(
				MultipleCardLinkSettingsAS.strLinkSettingsPlugInKey)
			.DisplayJavascript());

		return myScripts;
	}

	/**
	 * 
	 *
	 * @return 
	 *
	 * @throws HTMLException 
	 * @throws XPException 
	 */
	protected HTMLElement GetHTMLBelowSubHeader()
	{
		IMultipleCardLinkSettingsModelRO asModel = (IMultipleCardLinkSettingsModelRO) m_asOwner.GetModel(MultipleCardLinkSettingsModel.STR_MVC_CLASS_NAME);

		String strCardNames = asModel.GetCardNames();

		return HTMLConstructHelpers.GetSubHeader(
			XPHttpUtility.HtmlEncode(strCardNames),
			m_asOwner);
	}
}

