package com.plumtree.portalpages.browsing.editors.smartsort;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.helpers.*;
import com.plumtree.portalpages.common.objects.sort.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This Section contains the HTML for the Smart Sort Editor.
 * This Section is printed out by SmartSortEditorMainDP.
 *
 * @author OlegS
 */
public class SmartSortEditorMainView extends EditorView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.editors.smartsort.SmartSortEditorMainView");

	
	public static final String STR_MVC_CLASS_NAME = "SmartSortEditorMain";

	
	public static final String HTMLCHECKBOX_FOLDER     = "in_cb_folder_"; // +group, value is folder id or -1 for all

	
	public static final String HTMLCHECKBOX_RECOURSIVE = "in_cb_recoursive";

	
	public static final String HTMLCHECKBOX_MOVE    = "in_cb_move";

	
	public static final String HTMLCHECKBOX_APPROVE = "in_cb_approve";

	
	public static final String HTMLCHECKBOX_FILTER = "in_cb_filter";

	
	public static final String HTMLHIDDEN_SPACEID		    = "in_hi_CatalogFolderSpaceID";

	
	public static final String HTMLHIDDEN_TREEREPOST	    = "in_hi_CatalogFolderRepostAction";

	public static final String HTMLHIDDEN_CHECKALL_CNT     = "in_hi_checkall_"; // +group

	public static final String JAVASCRIPT_CATALOGFOLDERTREE = "openCatalogFolderTree";
	
	public static final String JAVASCRIPT_CHECKALLFOLDERS 	= "checkAllFolderRows";
	
	
	/** A name for the sub activity space for the admin folder tree */
	public static final String DESTINATION_FOLDERTREESPACE = "FolderTreeDestination";
	public static final String SOURCE_FOLDERTREESPACE = "FolderTreeSource";

	protected static Spacer spacer15 = new Spacer("15");

	// member variables
	protected PTStyleClass m_styleListItem = null;

	/**
	 * 
	 *
	 * @param type 
	 * @param alt 
	 * @param w 
	 * @param h 
	 *
	 * @return 
	 */
	public HTMLImg GetImage(PTImageType type, String alt, String w, String h)
	{
		try
		{
			PTConstruct myConstruct = UIConstructFactory.GetConstruct(type,
					true, this.m_asOwner);
			HTMLImg myImg		    = (HTMLImg) myConstruct.GetHTMLElement();
			myImg.SetAlt(alt);
			myImg.SetBorder("0");
			myImg.SetWidth(w);
			myImg.SetHeight(h);

			return myImg;
		}
		catch (Exception e)
		{
			log.Error(e, "SmartSortEditorMainView.GetImage couldn't get image: " +
				e.getMessage());


			throw new XPException(m_asOwner.GetString(562,
					"ptmsgs_portalbrowsingmsgs") + " (Image)", e);
		}
	}

	// Strings to localize.

	/** Localized String: Source Folders */



	/** Localized String: Destination Folders */



	/** Localized String: Delete Selected Item */



	/** Localized String: Add Folder */



	/** Localized String: Folder Name */



	/** Localized String: Unable to display the HTML for SmartSortEditorMainView (Display). */



	/** Localized String: Check box for all folders */



	/** Localized String: Check box for folder */



	/** Localized String: Folder */



	/** Localized String: Sort */



	/** Localized String: Sort content from all the subfolders of the selected source Folders. */



	/** Localized String: Delete sorted cards from the source folders. */



	/** Localized String: Sorted documents are automatically approved. */



	/** Localized String: Sorted documents must pass the Filters used by Destinations Folders. */



	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.
	 * IManagedObject#Create()
	 */
	public Object Create()
	{
		return new SmartSortEditorMainView();
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public HTMLElement Display()
	{
		try
		{
			HTMLElementCollection result = new HTMLElementCollection();
			HTMLComment myComment = new HTMLComment( "Begin: Display SmartSortEditorMainView");
			result.AddInnerHTMLElement(myComment);
			DisplayHiddenInputs(result);

			// source folders
			int group = SmartSortEditorModel.SOURCE_FOLDER_GROUP;


			DisplayObjectHeader(result,
				m_asOwner.GetString(557, "ptmsgs_portalbrowsingmsgs"));
			DisplayActionBar(result, group);
			DisplayFolderList(result, group);
			spacer15.display(result);
			DisplaySrcOptions(result);
			DisplayBreak(result);

			// destination folders
			group = SmartSortEditorModel.DESTINATION_FOLDER_GROUP;


			DisplayObjectHeader(result,
				m_asOwner.GetString(558, "ptmsgs_portalbrowsingmsgs"));
			DisplayActionBar(result, group);
			DisplayFolderList(result, group);
			spacer15.display(result);
			DisplayDstOptions(result);
			myComment = new HTMLComment( "End: Display SmartSortEditorMainView");
			result.AddInnerHTMLElement(myComment);

			return result;
		}
		catch (Exception e)
		{
			log.Error("SmartSortEditorMainView::Display The following error occured when trying to display the HTML: " +
				e /*.getMessage()*/    );


			throw new XPException(m_asOwner.GetString(562,
					"ptmsgs_portalbrowsingmsgs"), e);
		}
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public HTMLScript DisplayJavascript()
	{
		HTMLScript myScript = new HTMLScript("text/javaScript");
		try
		{
			myScript.AddInnerHTMLString(TreeHelper.makeTreeJavascript(
					m_asOwner, DESTINATION_FOLDERTREESPACE,
					HTMLHIDDEN_SPACEID,
					HTMLHIDDEN_TREEREPOST,
					m_asOwner.GetString(560, "ptmsgs_portalbrowsingmsgs"),
					m_asOwner.GetString(565, "ptmsgs_portalbrowsingmsgs"),
					PT_CLASSIDS.PT_CATALOGFOLDER_ID,
					PT_INTRINSICS.PT_FOLDER_ROOTFOLDER,
					TreeConstants.N_COMPOSITESELECT, true, true, null, GetModel().makeListOfDestinationFoldersAlreadyShown()));
			
			myScript.AddInnerHTMLString(TreeHelper.makeTreeJavascript(
					m_asOwner, SOURCE_FOLDERTREESPACE,
					HTMLHIDDEN_SPACEID,
					HTMLHIDDEN_TREEREPOST,
					m_asOwner.GetString(560, "ptmsgs_portalbrowsingmsgs"),
					m_asOwner.GetString(565, "ptmsgs_portalbrowsingmsgs"),
					PT_CLASSIDS.PT_CATALOGFOLDER_ID,
					PT_INTRINSICS.PT_FOLDER_ROOTFOLDER,
					TreeConstants.N_COMPOSITESELECT, true, true, null, GetModel().makeListOfSourceFoldersAlreadyShown()));
			
			
			XPStringBuilder sb = new XPStringBuilder(300);

			// JavaScript to check all the rows
			sb.Append("function "+JAVASCRIPT_CHECKALLFOLDERS+"(item, group)\n");
			sb.Append("{ \n");
			sb.Append("  var checkBoxes = document.getElementsByName(item.name); \n");
			sb.Append("  var counter = document.getElementById('"+HTMLHIDDEN_CHECKALL_CNT+"'+group); \n");
			sb.Append("\n");
			sb.Append("	 if (-1 == counter.value)	// first time, initialize\n");
			sb.Append("	 {\n");
			sb.Append("	   counter.value=0;\n");
			sb.Append("	   for(i = 0; i < checkBoxes.length; i++)\n");
			sb.Append("	   {\n");
			sb.Append("		 if(checkBoxes[i].checked)\n");
			sb.Append("		   counter.value++;\n");
			sb.Append("	   }\n");
			sb.Append("	 }\n");
			sb.Append("\n");
			sb.Append("  if (-1==item.value) /*checkall*/ \n");
			sb.Append("  { \n");
			sb.Append("    for(i=0 ; i<checkBoxes.length; i++) \n");
			sb.Append("    { \n");
			sb.Append("      checkBoxes[i].checked = item.checked;\n");
			sb.Append("    } \n");
			sb.Append("\n");
			sb.Append("    if(item.checked) \n");
			sb.Append("    { \n");
			sb.Append("    	 counter.value = checkBoxes.length;\n");
			sb.Append("    }\n");
			sb.Append("    else\n");
			sb.Append("    {\n");
			sb.Append("      counter.value = 1;\n");
			sb.Append("    }\n");
			sb.Append("  } \n");
			sb.Append("  else\n");
			sb.Append("  {\n");
			sb.Append("    if(item.checked)\n");
			sb.Append("      counter.value++;\n");
			sb.Append("    else\n");
			sb.Append("      counter.value--;\n");
			sb.Append("  }\n");
			sb.Append("\n");
			sb.Append("document.getElementById('"+HTMLCHECKBOX_FOLDER+"'+group+'all').checked = (counter.value==checkBoxes.length);\n");
			sb.Append("\n");
			sb.Append("return true;");
			sb.Append("}\n");
			myScript.AddInnerHTMLString(sb.ToString());
		}
		catch (Exception e)
		{
			log.Error(e, "SmartSortEditorMainView.DisplayJavascript couldn't add javascript to the HTMLScript element: " +
				e.getMessage());


			throw new XPException(m_asOwner.GetString(562,
					"ptmsgs_portalbrowsingmsgs") + " (Javascript)", e);
		}

		return myScript;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * 
	 *
	 * @param result 
	 * @param group 
	 *
	 * @throws HTMLException 
	 */
	protected void DisplayActionBar(HTMLElementCollection result, int group)

	{
		HTMLTable myTable;
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLSpan mySpan;
		HTMLAnchor myAnchor;
		HTMLImg myImg;
		HTMLComment myComment = new HTMLComment( "Begin: DisplayActionBar SmartSortEditorMainView");
		result.AddInnerHTMLElement(myComment);
		
		myTable = new HTMLTable();
		result.AddInnerHTMLElement(myTable);
		myTable.SetWidth("100%");
		myTable.SetBorder("0");
		myTable.SetCellPadding("0");
		myTable.SetCellSpacing("0");
		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);
		myRow.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);

		// add folder		
		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);
		myCell.SetHeight("25");
		myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
		myCell.SetNoWrap(true);
		String strLang = PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(m_asOwner).GetLanguage();
		if (SmartSortEditorModel.SOURCE_FOLDER_GROUP == group) {
		myAnchor = TreeHelper.makeTreeAnchor(SOURCE_FOLDERTREESPACE,
				HTMLHIDDEN_SPACEID,
				SmartSortEditorRepostControl.ACTION_ADD + SmartSortEditorRepostControl.GROUP_SHIFT*group, strLang);
		} else {
			myAnchor = TreeHelper.makeTreeAnchor(DESTINATION_FOLDERTREESPACE,
					HTMLHIDDEN_SPACEID,
					SmartSortEditorRepostControl.ACTION_ADD + SmartSortEditorRepostControl.GROUP_SHIFT*group, strLang);
		}
		myCell.AddInnerHTMLElement(myAnchor);

		myImg = GetImage(PTImageType.ACTION_ADD,
				m_asOwner.GetString(560, "ptmsgs_portalbrowsingmsgs"), "6", "25");
		myImg.SetAlign("absMiddle");
		myAnchor.AddInnerHTMLElement(myImg);
		myImg = GetImage(PTImageType.ICON_FOLDER,
				m_asOwner.GetString(565, "ptmsgs_portalbrowsingmsgs"), "18", "20");
		myImg.SetAlign("absMiddle");
		myAnchor.AddInnerHTMLElement(myImg);
		myAnchor.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		
		mySpan = new HTMLSpan();
		myAnchor.AddInnerHTMLElement(mySpan);
		mySpan.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);


		mySpan.AddInnerHTMLString(m_asOwner.GetString(560,
				"ptmsgs_portalbrowsingmsgs") + "&nbsp;" + "&nbsp;");

		// divider
		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);
		myCell.SetHeight("25");
		myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_DIVIDER);
		myImg = GetImage(PTImageType.SPACER, "", "1", "1");
		myCell.AddInnerHTMLElement(myImg);

		// delete folder	
		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);
		myCell.SetHeight("25");
		myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
		myCell.SetNoWrap(true);
		// mj: replaced the following line with a blank constructur and SetOnClick()
		// myAnchor = new HTMLAnchor("postToSelf(" +
		myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
		myAnchor.SetOnClick("postToSelf(" +
				((SmartSortEditorRepostControl.GROUP_SHIFT * group) + SmartSortEditorRepostControl.ACTION_DEL) +
				"); return false;");
		myCell.AddInnerHTMLElement(myAnchor);
		myAnchor.AddInnerHTMLString(CommonHTMLStrings.SPACE);


		myImg = GetImage(PTImageType.ACTION_DELETE,
				m_asOwner.GetString(559, "ptmsgs_portalbrowsingmsgs"), "25",
				"25");
		myImg.SetAlign("absMiddle");
		myAnchor.AddInnerHTMLElement(myImg);

		// divider
		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);
		myCell.SetHeight("25");
		myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_DIVIDER);
		myImg = GetImage(PTImageType.SPACER, "", "1", "1");
		myCell.AddInnerHTMLElement(myImg);

		// last
		myCell = new HTMLTableCell();
		myCell.SetHeight("25");
		myCell.SetWidth("100%");
		myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		myRow.AddInnerHTMLElement(myCell);
		myComment = new HTMLComment( "End: DisplayActionBar SmartSortEditorMainView");
		result.AddInnerHTMLElement(myComment);
		
	}

	/**
	 * 
	 *
	 * @param result 
	 *
	 * @throws HTMLException 
	 */
	protected void DisplayBreak(HTMLElementCollection result)

	{
		result.AddInnerHTMLElement(new HTMLBr());
	}

	/**
	 * 
	 *
	 * @param result 
	 *
	 * @throws HTMLException 
	 */
	protected void DisplayDstOptions(HTMLElementCollection result)

	{
		HTMLTable myTable;
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLInput myInput;
		HTMLComment myComment = new HTMLComment( "Begin: DisplayDstOptions SmartSortEditorMainView");
		result.AddInnerHTMLElement(myComment);
		myTable = new HTMLTable();
		result.AddInnerHTMLElement(myTable);
		myTable.SetWidth("100%");
		myTable.SetBorder("0");
		myTable.SetCellPadding("5");
		myTable.SetCellSpacing("0");
		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);
		myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
		
		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);
		myCell.SetWidth("100%");
		myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
		myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLCHECKBOX_APPROVE,
				"");
		myInput.SetChecked(GetModel().GetSetting(PT_TAXONOMIST_SETTINGS.PT_TAXONOMIST_ADD_CARDS_TO_FOLDERS_APPROVED));
		myInput.SetValue("1");
		myCell.AddInnerHTMLElement(myInput);


		myCell.AddInnerHTMLString(m_asOwner.GetString(569,
				"ptmsgs_portalbrowsingmsgs"));
		myCell.AddInnerHTMLElement(new HTMLBr());
		myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLCHECKBOX_FILTER, "");
		myInput.SetChecked(!GetModel().GetSetting(PT_TAXONOMIST_SETTINGS.PT_TAXONOMIST_ONLY_APPLY_SUBFOLDER_FILTERS)); // checkbox displays inverted setting
		myInput.SetValue("1");
		myCell.AddInnerHTMLElement(myInput);


		myCell.AddInnerHTMLString(m_asOwner.GetString(570,
				"ptmsgs_portalbrowsingmsgs"));
		myComment = new HTMLComment( "End: DisplayDstOptions SmartSortEditorMainView");
		result.AddInnerHTMLElement(myComment);
	}

	/**
	 * 
	 *
	 * @param result 
	 * @param group 
	 *
	 * @throws HTMLException 
	 */
	protected void DisplayFolderList(HTMLElementCollection result, int group)

	{
		HTMLTable myTable;
		HTMLComment myComment = new HTMLComment( "Begin: DisplayFolderList SmartSortEditorMainView");
		result.AddInnerHTMLElement(myComment);
		myTable = new HTMLTable();
		result.AddInnerHTMLElement(myTable);
		myTable.SetWidth("100%");
		myTable.SetBorder("0");
		myTable.SetCellPadding("0");
		myTable.SetCellSpacing("2");

		// items
		Object[] folderIDs					   = GetModel().GetFolderIDs(group);
		PTObjectIdNamePair[] folderIdNamePairs = SortObjectByName(folderIDs,
				GetModel().GetSortAscending(group));

		if ((folderIdNamePairs == null) || (folderIdNamePairs.length == 0))
		{
			DisplayRowForEmptyList(myTable);
		}
		else
		{
			m_styleListItem = null;
			DisplayFolderListHeader(myTable, group);

			for (int i = 0; i < folderIdNamePairs.length; i++)
			{
				int folderId	  = folderIdNamePairs[i].id;
				String folderName = (folderIdNamePairs[i].name == null)
					? GetFolderName(folderId) : folderIdNamePairs[i].name;
				DisplayFolderListItem(myTable, group, folderId, folderName);
			}
		}

		myComment = new HTMLComment( "End: DisplayFolderList SmartSortEditorMainView");
		result.AddInnerHTMLElement(myComment);
	}

	/**
	 * 
	 *
	 * @param myTable 
	 * @param group 
	 *
	 * @throws HTMLException 
	 */
	protected void DisplayFolderListHeader(HTMLTable myTable, int group)

	{
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLInput myInput;
		HTMLAnchor myAnchor;
		HTMLImg myImg;
		myRow = new HTMLTableRow();
		myRow.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);
		myTable.AddInnerHTMLElement(myRow);
		myCell = new HTMLTableCell();
		myCell.SetWidth("20");
		myCell.SetVAlign("center");
		myCell.SetAlign("middle");
		myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
		myRow.AddInnerHTMLElement(myCell);


		String sInputName = HTMLHIDDEN_CHECKALL_CNT + XPConvert.ToString(group); 
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN, sInputName, "");
		myInput.SetID(sInputName);
		myInput.SetValue("-1");
		myCell.AddInnerHTMLElement(myInput);
		
		sInputName = HTMLCHECKBOX_FOLDER + XPConvert.ToString(group); 
		myInput = new HTMLInput(HTMLInputTypes.CHECKBOX,
				sInputName,
				m_asOwner.GetString(563, "ptmsgs_portalbrowsingmsgs"));
		myInput.SetID(sInputName+"all");
		myInput.SetValue("-1"); // all
		myInput.SetOnClick(JAVASCRIPT_CHECKALLFOLDERS+"(this, '"+XPConvert.ToString(group)+"');"); // checkall
		myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
		myCell.AddInnerHTMLElement(myInput);

		myCell = new HTMLTableCell();
		myCell.SetWidth("100%");
		myCell.SetHeight("20");
		myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
		myRow.AddInnerHTMLElement(myCell);
		// mj: replaced the following line with a blank constructur and SetOnClick()
		// myAnchor = new HTMLAnchor("postToSelf(" +
		myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
		myAnchor.SetOnClick("postToSelf(" +
				((SmartSortEditorRepostControl.GROUP_SHIFT * group) + SmartSortEditorRepostControl.ACTION_SORT) +
				"); return false;");


		myAnchor.AddInnerHTMLString(m_asOwner.GetString(561,
				"ptmsgs_portalbrowsingmsgs"));
		myCell.AddInnerHTMLElement(myAnchor);


		myImg = GetImage(GetModel().GetSortAscending(group)
				? PTImageType.ICON_ARROW_DESCEND : PTImageType.ICON_ARROW_ASCEND,
				m_asOwner.GetString(566, "ptmsgs_portalbrowsingmsgs"), "11",
				"20");
		myImg.SetAlign("absMiddle");
		myAnchor.AddInnerHTMLElement(myImg);
	}

	/**
	 * 
	 *
	 * @param myTable 
	 * @param group 
	 * @param id 
	 * @param name 
	 *
	 * @throws HTMLException 
	 */
	protected void DisplayFolderListItem(HTMLTable myTable, int group, int id, String name)

	{
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLInput myInput;
		HTMLImg myImg;
		myRow = new HTMLTableRow();

		m_styleListItem = ((m_styleListItem == null) || (m_styleListItem == PTStyleClass.LIST_ITEM_ONE_BG)) ? PTStyleClass.LIST_ITEM_TWO_BG : PTStyleClass.LIST_ITEM_ONE_BG;

		myRow.SetStyleClass(m_styleListItem);
		myTable.AddInnerHTMLElement(myRow);
		myCell = new HTMLTableCell();
		myCell.SetWidth("20");
		myCell.SetVAlign("center");
		myCell.SetAlign("middle");
		myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
		myRow.AddInnerHTMLElement(myCell);


		myInput = new HTMLInput(HTMLInputTypes.CHECKBOX,
				HTMLCHECKBOX_FOLDER + XPConvert.ToString(group),
				m_asOwner.GetString(564, "ptmsgs_portalbrowsingmsgs") + " \"" +
				name + "\"");
		myInput.SetChecked(false);
		myInput.SetValue(XPConvert.ToString(id));
		myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
		myInput.SetOnClick(JAVASCRIPT_CHECKALLFOLDERS+"(this, '"+ XPConvert.ToString(group) +"');");
		myCell.AddInnerHTMLElement(myInput);
		myCell = new HTMLTableCell();
		myCell.SetWidth("100%");
		myCell.SetHeight("20");
		myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
		myCell.SetNoWrap(true);
		myRow.AddInnerHTMLElement(myCell);
		// mj: replaced the following line with a blank constructur and SetOnClick()
		// myAnchor     = new HTMLAnchor("postToSelf(" +
		
		myImg = GetImage(PTImageType.ICON_FOLDER,
				m_asOwner.GetString(565, "ptmsgs_portalbrowsingmsgs"), "18",
				"20");
		myImg.SetAlign("absMiddle");
		myCell.AddInnerHTMLElement(myImg);
		myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		myCell.AddInnerHTMLString(XPHttpUtility.HtmlEncode(name));
		

	}

	/**
	 * 
	 *
	 * @param result 
	 *
	 * @throws HTMLException 
	 */
	protected void DisplayHiddenInputs(HTMLElementCollection result)

	{
		HTMLInput myInput;

		// Hidden input for the Space ID for the tree.  That way if you hit the folder
		// tree link twice, it will open the same tree both times.
		// olegs: cool but dangerous; if you restart server you get a blank tree window
		// due to null root pointer in TreeCompositeIterator.<init>
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN, HTMLHIDDEN_SPACEID, "");
		result.AddInnerHTMLElement(myInput);
		myInput.SetValue((GetModel().GetLastTreeSpaceId() == null) ? "" : GetModel().GetLastTreeSpaceId());

		// hidden input to hold repost action on tree submit
		myInput = new HTMLInput(HTMLInputTypes.HIDDEN, HTMLHIDDEN_TREEREPOST, "");
		result.AddInnerHTMLElement(myInput);
		myInput.SetValue("");
	}

	/**
	 * 
	 *
	 * @param result 
	 * @param name 
	 *
	 * @throws HTMLException 
	 */
	protected void DisplayObjectHeader(HTMLElementCollection result, String name)

	{
		HTMLTable myTable;
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLSpan mySpan;
		HTMLComment myComment = new HTMLComment( "Begin: DisplayObjectHeader SmartSortEditorMainView");
		result.AddInnerHTMLElement(myComment);
		myTable = new HTMLTable();
		result.AddInnerHTMLElement(myTable);
		myTable.SetWidth("100%");
		myTable.SetBorder("0");
		myTable.SetCellPadding("0");
		myTable.SetCellSpacing("0");
		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);
		myRow.SetStyleClass(PTStyleClass.OBJECT_HEADER_BG);
		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);
		myCell.SetWidth("100%");
		myCell.SetHeight("20");
		myCell.SetVAlign("center");
		myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
		mySpan = new HTMLSpan();
		myCell.AddInnerHTMLElement(mySpan);
		mySpan.SetStyleClass(PTStyleClass.OBJECT_HEADER);
		mySpan.AddInnerHTMLString(name);
		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);
		myCell.SetWidth("10");
		myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		myComment = new HTMLComment( "End: DisplayObjectHeader SmartSortEditorMainView");
		result.AddInnerHTMLElement(myComment);
	}

	/**
	 * 
	 *
	 * @param myTable 
	 *
	 * @throws HTMLException 
	 */
	protected void DisplayRowForEmptyList(HTMLTable myTable)

	{
		HTMLTableRow myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);

		HTMLTableCell myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);
		myCell.SetWidth("100%");
		myCell.SetStyleClass(PTStyleClass.TIPS_TEXT);

		HTMLP myParagraph=new HTMLP();
		myCell.AddInnerHTMLElement(myParagraph);

		HTMLSpan mySpan = new HTMLSpan();
		myParagraph.AddInnerHTMLElement(mySpan);
		mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_TITLE);
		mySpan.AddInnerHTMLString(GetModel().GetEmptyListMainMessage());
		myParagraph.AddInnerHTMLElement(new HTMLBr());
		myParagraph.AddInnerHTMLString(GetModel().GetEmptyListInstructions());
	}

	/**
	 * 
	 *
	 * @param result 
	 *
	 * @throws HTMLException 
	 */
	protected void DisplaySrcOptions(HTMLElementCollection result)

	{
		HTMLTable myTable;
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLInput myInput;
		HTMLComment myComment = new HTMLComment( "Begin: DisplaySrcOptions SmartSortEditorMainView");
		result.AddInnerHTMLElement(myComment);
		myTable = new HTMLTable();
		result.AddInnerHTMLElement(myTable);
		myTable.SetWidth("100%");
		myTable.SetBorder("0");
		myTable.SetCellPadding("5");
		myTable.SetCellSpacing("0");
		myRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(myRow);
		myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
		
		myCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(myCell);
		myCell.SetWidth("100%");
		myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
		myInput = new HTMLInput(HTMLInputTypes.CHECKBOX,
				HTMLCHECKBOX_RECOURSIVE, "");
		myInput.SetChecked(GetModel().GetSetting(PT_TAXONOMIST_SETTINGS.PT_TAXONOMIST_TAXONOMIZE_FOLDER_BRANCH));
		myInput.SetValue("1");
		myCell.AddInnerHTMLElement(myInput);


		myCell.AddInnerHTMLString(m_asOwner.GetString(567,
				"ptmsgs_portalbrowsingmsgs"));
		myCell.AddInnerHTMLElement(new HTMLBr());
		myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, HTMLCHECKBOX_MOVE, "");
		myInput.SetChecked(GetModel().GetSetting(PT_TAXONOMIST_SETTINGS.PT_TAXONOMIST_REMOVE_CARDS_FROM_PRESENT_LOCATION));
		myInput.SetValue("1");
		myCell.AddInnerHTMLElement(myInput);


		myCell.AddInnerHTMLString(m_asOwner.GetString(568,
				"ptmsgs_portalbrowsingmsgs"));
		myComment = new HTMLComment( "End: DisplaySrcOptions SmartSortEditorMainView");
		result.AddInnerHTMLElement(myComment);
	}

	/**
	 * 
	 *
	 * @param id 
	 *
	 * @return 
	 */
	protected String GetFolderName(int id)
	{
		IPTSession ptSession = (IPTSession) m_asOwner.GetUserSession();
		IPTFolder ptFolder   = ptSession.GetCatalog().OpenFolder(id, false);

		return ptFolder.GetLocalizedName();
	}

	/**
	 * 
	 *
	 * @return 
	 */
	protected ISmartSortEditorModelRO GetModel()
	{
		return (ISmartSortEditorModelRO) m_model;
	}

	/**
	 * 
	 *
	 * @param ids 
	 * @param ascending 
	 *
	 * @return 
	 */
	protected PTObjectIdNamePair[] SortObjectByName(Object[] ids, boolean ascending)
	{
		if (ids == null)
			return null;

		XPArrayList pairs = new XPArrayList(ids.length);

		for (int i = 0; i < ids.length; i++)
		{
			int folderId = XPConvert.ToInteger(ids[i]);
			pairs.Add(new PTObjectIdNamePair(folderId));
		}

		XPCollections.Sort(pairs,
			new PTObjectNameComparator(
				(IPTObjectNameGetter) new PTFolderObjectNameGetter(),
				(IPTSession) m_asOwner.GetUserSession(), ascending));

		// (fails on .NET, replaced below) return (PTObjectIdNamePair[]) pairs.ToArray(result);
		PTObjectIdNamePair[] result = new PTObjectIdNamePair[ids.length];
		for(int i=0; i<ids.length; i++) 
			result[i]=(PTObjectIdNamePair)pairs.GetElement(i);
		return result;

		// we don't change source array, but if we wantewd to:
		// for(int i=0; i<ids.length; i++)	ids[i]=new Integer(pairs[i].id);
	}
}

