package com.plumtree.portalpages.browsing.gateway;

import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.portalpagesperf.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author daniilk
 *
 */
public class GatewayBinaryDP implements IDisplayPage
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.gateway.GatewayBinaryDP");

	
	public static String STR_MVC_CLASS_NAME = "GatewayBinary";
	private AActivitySpace m_asOwner	    = null;

	/**
	 * 
	 *
	 * @param webData 
	 */
	public void Display(IWebData webData)
	{
		IGatewayModelRO gwModel = (IGatewayModelRO) m_asOwner.GetModel(GatewayModel.strModelName);
		String strRawHeaders    = gwModel.GetHeaders();
		String strContentType   = gwModel.GetContentType();
		int iStatus			    = gwModel.GetStatus();
		boolean bIsText		    = gwModel.IsText();
		
		
			log.Debug("Display(): " + "Setting HTTP status on the response to [" +
				iStatus + "]");
		

		webData.SetStatus(iStatus);

		
			if (304 == iStatus)
			{
				log.Info("Display(): " +
					"Returning [304 Not Modified] response to the browser");
			}
		

		
			log.Debug("Display(): " + "Setting headers on WebData object \r\n[" +
				strRawHeaders + "]");
		

		if (null != strRawHeaders)
		{
			GatewayUtils.AddHeadersToResponse(strRawHeaders, webData, !bIsText);
		}

		
			log.Debug("Display(): " + "Content-Type is [" + strContentType + "]");
		
		
		// increment perfcounter
		PerfCounters.INSTANCE.IncrementCounter( PORTALPAGESPERF_COUNTERS.PORTALPAGESPERF_CLICKTHROUGHS);

		/* if the content type is text */
		if (bIsText)
		{			
			log.Debug("Display(): treating gateway response as text.");

			HTMLElement result = gwModel.GetDisplayElement();

			String strHTML = null;
			if (null != result)
			{
				strHTML = result.GetDisplayString();
			}
			
			if (null != strHTML)
			{
				webData.SetData(strHTML);
			}
			else
			{
				webData.SetData(
					"<B>" + m_asOwner.GetString(623, "ptmsgs_portalbrowsingmsgs") + "</B>\r\n"
					// the following line can stay non-localized, since it's going to be in the HTML comments
					// and will not be displayed in the browser.
					+ "<!-- GetTextBody call returned NULL. There was a problem accessing gateway content. -->");				
			}
		}
		else /* if the content type is binary */
		{			
			log.Debug("Display(): treating gateway response as binary.");

			byte[] arrBinaryBody = gwModel.GetBinaryBody();

			if (null != arrBinaryBody)
			{
				webData.SetData(arrBinaryBody);
			}
		}

		return;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public AActivitySpace GetOwner()
	{
		return m_asOwner;
	}

	/**
	 * 
	 *
	 * @param parent 
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner = parent;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.AFormDP#PageDisplay()
	 */
	public HTMLElement PageDisplay()
	{
		return null;
	}
}

