package com.plumtree.portalpages.browsing.gateway;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author daniilk
 *
 */
public class GatewayModel implements IGatewayModel, IGatewayModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_GATEWAY),
		"com.plumtree.portalpages.browsing.gateway.GatewayModel");

	public static final String STR_MVC_CLASS_NAME = "Gateway";
	
	/** @deprecated Use STR_MVC_CLASS_NAME instead **/
	public static final String strModelName = "Gateway";

	private static final int HOSTED_MODE_TAG_VALUE_NOT_SET = 0;
	private static final int HOSTED_MODE_TAG_VALUE_HOSTED = 1;
	private static final int HOSTED_MODE_TAG_VALUE_PLAIN = -1;
	
	private AActivitySpace m_asasParent;
	private IPTContent m_ptContent = null;
	private IPTSession m_ptSession = null;
	private String m_strReturnURIBody    = null;
	private String m_strReturnURIHeaders = null;
	private boolean m_bError     = false;
	private boolean m_bReturnURI = false;
	private int m_iReturnURIStatus = 200;
	private String m_strUserURL = "";
	private boolean m_bControlCalled = false;
	private int m_iCommunityID = 0;
	private String m_strUIMode = GatewayHelpers.UI_STD;
	private HTMLElement m_displayElement = null;
	// private boolean m_bHostedModePTTagPresent = false;
	private int m_iHostedModeTagValue = HOSTED_MODE_TAG_VALUE_NOT_SET;
	private String m_strTitle = null;
	private String m_strSubtitle = null;
	private int m_iRefreshMode = PlumtreeExtensibility.PT_REFRESH_NONE;
	private int m_iErrorCode = 0;
	private String m_strErrorMessage = null;
	private String m_strErrorHTMLComment = null;
	private String m_strFileName = null;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new GatewayModel();
	}

	/**
	 * This method returns the body of the response from the remote server as an array of bytes.
	 */
	public byte[] GetBinaryBody()
	{
		if (m_bReturnURI)
		{
			if (null != m_strReturnURIBody)
			{
				log.Debug("Hitting return-uri with body [" + m_strReturnURIBody + "]");
				return XPStringUtility.GetBytes(m_strReturnURIBody);
			}
			else
			{
				return null;
			}
		}

		try
		{
			return (byte[]) m_ptContent.GetBinaryBody();
		}
		catch (Exception e)
		{
			return null;
		}
	}

	/**
	 * This method returns the value of the content-type header from the HTTP response
	 * that came from the remote server
	 */
	public String GetContentType()
	{
		try
		{
			return m_ptContent.GetMIMEType();
		}
		catch (Exception e)
		{
			return null;
		}
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public String GetHeaders()
	{
		if (m_bReturnURI)
		{
			return m_strReturnURIHeaders;
		}

		try
		{
			String headers = m_ptContent.GetHeaders();
			if (m_strFileName != null && m_strFileName.length() > 0) {
				// DCA - add filename to header
				headers += "Content-Disposition: attachment; filename=" + m_strFileName + "\r\n";
			}
			return headers;
		}
		catch (Exception e)
		{
			return null;
		}
	}

	/*
	 * Implementation of IGatewayModelRO interface methods.
	 */

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return strModelName;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public IPTSession GetSession()
	{
		return m_ptSession;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public int GetStatus()
	{
		if (m_bReturnURI)
		{
			return m_iReturnURIStatus;
		}

		try
		{
			return m_ptContent.GetStatusCode();
		}
		catch (Exception e)
		{
			return 200;
		}
	}

	/**
	 * This method returns the body of the response from the remote server as string.
	 */
	public Object[][] GetTextBody()
	{
		try
		{
			return (Object[][]) m_ptContent.GetBody();
		}
		catch (Exception e)
		{
			return null;
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		m_asasParent     = parent;
		m_ptSession		 = (IPTSession) parent.GetUserSession();
		m_ptContent		 = null;
		m_asasParent.SetIsInPlaceRefreshEnabled(true);
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public boolean IsErrorPage()
	{
		return m_bError;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public boolean IsReturnPage()
	{
		return m_bReturnURI;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public boolean IsText()
	{
		try
		{
			return m_ptContent.GetIsText();
		}
		catch (Exception e)
		{
			return false;
		}
	}

	/**
	 * 
	 */
	public void SetError(int iErrorCode, String strErrorMessage, String strErrorHTMLComment)
	{
		m_bError = true;
		m_iErrorCode = iErrorCode;
		m_strErrorMessage = strErrorMessage;
		m_strErrorHTMLComment = strErrorHTMLComment;
	}
	public int GetErrorCode()
	{
		return m_iErrorCode;
	}
	
	public String GetErrorMessage()
	{
		return m_strErrorMessage;
	}
	
	public String GetErrorHTMLComment()
	{
		return m_strErrorHTMLComment;
	}
	
	
	/**
	 * This method is used to set gateway content on the model.
	 */
	public void SetIPTContent(IPTContent ptContent)
	{
		m_ptContent		 = ptContent;
		m_bReturnURI     = false;
		m_bError		 = false;
	}

	/*
	 * Implementation of the methods defined on the IGatewayModel interface.
	 */
	public void SetReturnURIContent(int iStatusCode, String strHeaders,
		String strBody)
	{
		m_iReturnURIStatus		  = iStatusCode;
		m_strReturnURIHeaders     = strHeaders;
		m_strReturnURIBody		  = strBody;
		m_bReturnURI			  = true;
		m_bError				  = false;
	}
	
	public void SetUserURL(String strUserURL)
	{
		m_strUserURL = strUserURL;
	}

	public String GetUserURL()
	{
		return m_strUserURL;
	}
	
	public boolean WasControlCalled()
	{
		return m_bControlCalled;
	}
	
	public void SetControlCalled()
	{
		m_bControlCalled = true;
	}
	
	public void SetCommunityID(int iCommunityID)
	{
		m_iCommunityID = iCommunityID;
	}
	
	public int GetCommunityID()
	{
		return m_iCommunityID;
	}
	
	public void SetUIMode(String strUIMode)
	{
		m_strUIMode = strUIMode;
	}
	
	public String GetUIMode()
	{
		return m_strUIMode;
	}
	
	public boolean IsDeviceUIMode()
	{
		return (GatewayHelpers.UI_PDA.compareTo(m_strUIMode) == 0
			 || GatewayHelpers.UI_WML.compareTo(m_strUIMode) == 0);
	}
	
	public int GetRefreshMode()
	{
		return m_iRefreshMode;
	}
	
	public void SetRefreshMode(int iRefreshMode)
	{
		m_iRefreshMode = iRefreshMode;
	}

	public void SetDisplayElement(HTMLElement elemDisplayString)
	{
		m_displayElement = elemDisplayString;
	}
	
	public HTMLElement GetDisplayElement()
	{
		return m_displayElement;
	}

	public boolean GetIsHostedMode()
	{
		if ( m_iHostedModeTagValue == HOSTED_MODE_TAG_VALUE_HOSTED )
		{
			return true;
		}
		else if ( m_iHostedModeTagValue == HOSTED_MODE_TAG_VALUE_PLAIN )
		{
			return false;
		}
		else // ( m_iHostedModeTagValue == HOSTED_MODE_TAG_VALUE_NOT_SET ) 
		{
			return m_ptContent.IsHostedDisplayMode();
		}
		// return m_bHostedModePTTagPresent || m_ptContent.IsHostedDisplayMode();
	}

	public String GetTitle()
	{
		return m_strTitle;
	}

	public String GetSubtitle()
	{
		return m_strSubtitle;
	}

	public void SetIsHostedModePTTagPresent(boolean _bHostedMode)
	{
		m_iHostedModeTagValue = 
			( _bHostedMode ? HOSTED_MODE_TAG_VALUE_HOSTED : HOSTED_MODE_TAG_VALUE_PLAIN );
	}

	public void SetDisplayMode(int iMode)
	{
		m_iHostedModeTagValue = iMode;
	}

	public void SetTitle(String strTitle)
	{
		m_strTitle = strTitle;
	}

	public void SetSubtitle(String strSubtitle)
	{
		m_strSubtitle = strSubtitle;
	}

	public void SetFileName(String strFileName) {
		m_strFileName = strFileName;
	}
}