package com.plumtree.portalpages.browsing.gateway;

import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.PT_RESULTCODES;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.openfoundation.util.*;

/**
 * @author daniilk
 *
 */
public class GatewayRefreshErrorDP implements IDisplayPage
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.gateway.GatewayRefreshErrorDP");

	
	public static String STR_MVC_CLASS_NAME = "GatewayRefreshError";
	private AActivitySpace m_asOwner	    = null;
	private static String strHeaderName 	= "PT-HTTPResponse-Type";
	private static String strTimeoutHeaderValue = "PORTLET_TIMEOUT";
	private static String strPortletErrorHeaderValue = "PORTLET_ERROR";
	private static String strPortletXMLError = "<?xml version=\"1.0\"?><error>" +
			"<field><name>Error Code</name><value><![CDATA[{0}]]></value></field>" + 
			"<field><name>Error Message</name><value><![CDATA[{1}]]></value></field>" +
			"<field><name>Extended Error Information</name><value><![CDATA[{2}]]]></value></field>" +
			"</error>";
	private static final String strContentTypeHeaderName = "Content-Type";
	private static final String strTextXML = "text/xml";


	/**
	 * 
	 *
	 * @param webData 
	 */
	public void Display(IWebData webData)
	{
		IGatewayModelRO gwModel = (IGatewayModelRO) m_asOwner.GetModel(GatewayModel.strModelName);
		
		String[] strArgs = new String[3];
		strArgs[0] = XPConvert.ToString(gwModel.GetErrorCode());
		strArgs[1] = gwModel.GetErrorMessage();
		strArgs[2] = gwModel.GetErrorHTMLComment();
		String strErrorBody = XPFormat.Format(strPortletXMLError, strArgs);
		webData.SetData(strErrorBody);
		
		if (gwModel.GetErrorCode() == PT_RESULTCODES.PT_E_GADGET_TIMEOUT)
			webData.SetHeader(strHeaderName, strTimeoutHeaderValue);
		else
			webData.SetHeader(strHeaderName, strPortletErrorHeaderValue);
		
		webData.SetHeader(strContentTypeHeaderName, strTextXML);

		return;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public AActivitySpace GetOwner()
	{
		return m_asOwner;
	}

	/**
	 * 
	 *
	 * @param parent 
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner = parent;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.AFormDP#PageDisplay()
	 */
	public HTMLElement PageDisplay()
	{
		return null;
	}
}

