package com.plumtree.portalpages.browsing.login;

import com.plumtree.openfoundation.util.XPSystem;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.login.createaccount.*;
import com.plumtree.portalpages.common.login.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.taskapi.portalui.TaskAPIUIPortalConfig;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.web.ISessionManager;
import com.plumtree.xpshared.htmlelements.*;


/**
 * This class displays the login form for the login space.
 * 
 * @author BenG
 */
public class LoginView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.login.LoginView");

	public static final String STR_MVC_CLASS_NAME = "LoginView";

	
	public final static String sFormName = "lform";
	private AActivitySpace m_asOwner;
	private ILoginModelRO m_asmLoginModelRO;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new LoginView();
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#display()
	 */
	public HTMLElement Display()
	{
		HTMLForm myForm = null;
		HTMLTable myTable;
		HTMLElement myCreateLink = null;

		boolean b508 = false;

		if (this.m_asmLoginModelRO.GetAccessStyle() != AccessStyles.STANDARD.GetAccessID())
		{
			b508 = true;
		}

		try
		{
			myForm = new HTMLForm("loginFormID");

			// Bug fix 9351913 - REDIRECT TO REQUESTED PAGE AFTER LOGIN FAILS WITH ADAPTIVE LAYOUTS DISABLED
			// in .net version of the portal, the redirect to the requested page after login fails in classic mode always redirected the user
			// to their home page b/c the redirect object was null.  Adding the code here (which is the same code in the LoginFormTag) to
			// keep the redirect alive by adding it as a hidden input element.

			if (XPSystem.IsRunningInDotNet()) {

				//check if there is a redirect in the login
				ISessionManager subSession = m_asOwner.GetSubSession();			
				String redirectAfterLoginURL = null;
				if (null != subSession) 
				{
					redirectAfterLoginURL = (String)subSession.GetAttribute("PT_" + ASConstants.REDIRECT_AFTER_LOGIN);
				}
			
				// We also check if redirect-after-login is set on LoginModel
				//this will be used if it is not set on subsession.
				if (null == redirectAfterLoginURL || redirectAfterLoginURL.length()==0)
				{
					if ( m_asOwner.GetModelExists(LoginModel.STR_MVC_CLASS_NAME)){			
						LoginModel model = (LoginModel)m_asOwner.GetModel(LoginModel.STR_MVC_CLASS_NAME);

						Redirect r = model.GetRedirectAfterLogin();
						//Get the url from the Redirect value if it is not null and add
						// it to hidden input.
						if(null != r){
							redirectAfterLoginURL = TaskAPIUIPortalConfig.GetApplicationBaseURL(m_asOwner) + r.GetQueryString();

							//Bug fix 9351913 - Requested page after login fails with adaptive layouts disabled
							// If you set AllowGuestUser=0, bug fix 9351913 will fail for adaptive layouts with a friendly URL redirect.
	
							//a friendly url redirect will always be an externalURL, so the below check will always be true, so this fix
							//is applied to friendly URLS only when adaptive layout mode is enabled.
							if ( r.GetIsExternalURL() )
							{
								redirectAfterLoginURL = r.GetExternalURL();
							}
						}
					}
				}
			
				if (null != redirectAfterLoginURL && redirectAfterLoginURL.length()>0)
				{
					// Sets the hidden input, so when the login is successful the user
					// will be redirected to the proper page requested in the first place.
					HTMLInput myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
						ASConstants.REDIRECT_AFTER_LOGIN_INPUT, 
						ASConstants.REDIRECT_AFTER_LOGIN_INPUT,
						"This is an input");
					myInput.SetValue(redirectAfterLoginURL);
					myForm.AddInnerHTMLElement(myInput);
				}
			}

			//show authsource dropdown?
			boolean bShowAuthsourceDropdown = m_asmLoginModelRO.GetShowAuthSources();

			boolean bAllowAuthSourceDropdownDisplay = true;
			int nShowAuthSourceMode = m_asmLoginModelRO.GetAllowDefaultAuthSourceMode();
			if (nShowAuthSourceMode == 3)
			{
				// Mode 3 means never show auth source dropdown
				bAllowAuthSourceDropdownDisplay = false;
				bShowAuthsourceDropdown = false;
			}
			

			myTable = LoginHTML.GetLoginDirections(m_asOwner, "400", bShowAuthsourceDropdown);
			myForm.AddInnerHTMLElement(myTable);

			// Only create this link if we are actually going to use it.
			if( PTLoginHelper.GetAllowUserCreation( m_asOwner, m_asOwner.GetApplication() ) )
			{
				// CREATE ACCOUNT LINK
				//   This button can just be a redirect because the form
				//   needn't be submitted.
				ASURL myHRef = new ASURL(m_asOwner);
	
				myHRef.SetLinkGetSpaceIfCached(CreateAccountAS.STR_MVC_CLASS_NAME,
					this.m_asOwner);
	
				myHRef.SetControl(CreateAccountControl.STR_MVC_CLASS_NAME);
	
				// Create an account
				myHRef.AddInnerHTMLString(m_asOwner.GetString(637,
						"ptmsgs_portalbrowsingmsgs"));
	
				// Create an account
				((HTMLAnchor) myHRef.GetURLAsHTMLElement()).SetTitle(
					m_asOwner.GetString(1459, "ptmsgs_portalbrowsingmsgs"));

				myCreateLink = myHRef.GetURLAsHTMLElement();
			}
			
			String sUniquifier = "";

			LoginHTML.MakeLoginForm(myForm, sFormName, m_asOwner.GetName(), 
				m_asOwner.GetSpaceID(), LoginControl.STR_MVC_CLASS_NAME, 
				m_asOwner, b508, sUniquifier, myCreateLink,  bShowAuthsourceDropdown,
				bAllowAuthSourceDropdownDisplay);

			if (!b508)
			{
				//	doOnLoad method sets the default focus.
				HTMLScript myScript = (HTMLScript) myForm.InsertInnerHTMLElement(new HTMLScript(HTMLScript.TYPE_JAVASCRIPT));
				myScript.AddInnerHTMLString("doOnLoad" + sUniquifier + "();\n");
			}
		}
		catch (HTMLException he)
		{
			log.Error(he, "Unable to display Login view HTML.");
		}
		catch (Exception ex)
		{
			log.Error(ex, "Unable to display Login view.");
		}

		return myForm;
	}

	/**
	 * This method returns the HTMLScript that encompasses the Javascript on the page.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 *
	 * @return HTMLScript
	 */
	public HTMLScript DisplayJavascript()
	{
		boolean b508 = false;
		if ( AccessStyles.SECTION_508 == m_asOwner.GetAccessStyle() )
		{
			b508 = true;
		}
		
		HTMLScript myScript = LoginHTML.GetLoginJScript(b508, LoginView.sFormName, "");
		
		return myScript;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#Init(IModel, AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		m_asmLoginModelRO     = (ILoginModelRO) model;
		m_asOwner	   = parent;
	}
}

