package com.plumtree.portalpages.browsing.login.createaccount;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Create Account web page, allows user to enter desired username, password
 * Updated to support intrinsic invite object
 *
 * @author BenG
 * @author DavidP
 */
public class CreateAccountView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.login.createaccount.CreateAccountView");

	
	public static final String STR_MVC_CLASS_NAME = "CreateAccountView";

	public static final String HTMLBUTTON_CREATEACCOUNT = "in_bu_CreateAccount";
	
	public static final String PARAM_DOCREATE_508 = "in_su_CreateAccount_508";
	
	/** Javascript function: doCreateAccountS */
	public static final String STR_JS_CREATE_ACCOUNT = "doCreateAccount";
	
	final static String sFormName = "lform";

	
	private AActivitySpace m_asOwner;

	
	private CreateAccountModel m_asmModel;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new CreateAccountView();
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#display()
	 */
	public HTMLElement Display()
	{
		
		IPTServerConfigSettings config = (IPTServerConfigSettings) ((IPTSession)m_asOwner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS,
				true);

		boolean allowAccountCreation = (config.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_SETTINGNAME_ALLOW_NEW_USER_CREATION_TO_ALL) == 1);
		// If allow account creation is off, send an error unless we have a possible invite code.
		String inviteCode = m_asmModel.GetInviteCode();
		if(! allowAccountCreation && (inviteCode == null || inviteCode.equals("")))
		{
			String msg	=  m_asOwner.GetString(1711,"ptmsgs_portalbrowsingmsgs");
			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
					UIErrorScope.CURRENT_USER, null, msg, msg);
			UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
				m_asOwner.GetApplication());

			throw new XPException(msg);
		}
	

		HTMLForm myForm			   = null;
		HTMLElementCollection eOut = new HTMLElementCollection();

		HTMLTable myTable = new HTMLTable();

		myTable.SetCellPadding("2");
		myTable.SetCellSpacing(CommonHTMLStrings.ZERO);

		myTable.SetAlign(CommonHTMLStrings.CENTER);
		myTable.SetWidth("400");

		HTMLTableRow myRow;
		HTMLTableCell myCell;

		HTMLInput myInput;

		try
		{
			myForm = new HTMLForm("createAccountFormID");

			eOut.AddInnerHTMLElement(myForm);

			myForm.SetAction(ConfigHelper.GetApplicationBaseURLToAS(m_asOwner.GetCurrentNormalizedBaseURL(), 
				m_asOwner.GetName(), m_asOwner.IsSubSpace()));
			myForm.SetMethod(CommonHTMLStrings.POST);
			myForm.SetName(sFormName);

			// Sets the space name for the event to be passed to the server
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ASConstants.SPACENAME_INPUT, ASConstants.SPACENAME_INPUT,
					"This is an input");
			myInput.SetValue(m_asOwner.GetName());
			myForm.AddInnerHTMLElement(myInput);

			// Sets the space ID for the event to be passed to the server
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ASConstants.SPACEID_INPUT, ASConstants.SPACEID_INPUT,
					"This is an input");
			myInput.SetValue(m_asOwner.GetSpaceID());
			myForm.AddInnerHTMLElement(myInput);

			// Sets the control name for the event to be passed to the server
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ASConstants.CONTROL_INPUT, ASConstants.CONTROL_INPUT,
					"This is an input");
			myInput.SetValue(CreateAccountControl.STR_MVC_CLASS_NAME);
			myForm.AddInnerHTMLElement(myInput);

			// Sets the create account invite parameter
			// or uses the passed in invite token
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
				CreateAccountControl.CONTROLARGUMENT_INVCODE,
				CreateAccountControl.CONTROLARGUMENT_INVCODE,
					"This is an input");
			if ((m_asmModel.GetInviteCode() != null) && (m_asmModel.GetInviteCode().length() > 0))					
			{					
				myInput.SetValue(m_asmModel.GetInviteCode());
			}
			else
			{					
				myInput.SetValue(m_asmModel.GetCreateUserInviteToken());
			}
			myForm.AddInnerHTMLElement(myInput);

			// Sets the dologin parameter
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					CreateAccountControl.PARAM_DOCREATEACCOUNT,
					CreateAccountControl.PARAM_DOCREATEACCOUNT,
					"This is an input");
			myInput.SetValue(XPConvert.ToString(false));
			myForm.AddInnerHTMLElement(myInput);

			myForm.AddInnerHTMLElement(myTable);

			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetStyleClass(PTStyleClass.OBJECT_TITLE);

			myCell.AddInnerHTMLString(m_asOwner.GetString(1,
					"ptmsgs_portalbrowsingmsgs"));

			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);

			myCell.AddInnerHTMLString(m_asOwner.GetString(2,
					"ptmsgs_portalbrowsingmsgs"));

			myTable = new HTMLTable();

			myForm.AddInnerHTMLElement(myTable);

			myTable.SetCellPadding("2");
			myTable.SetCellSpacing(CommonHTMLStrings.ZERO);

			myTable.SetAlign(CommonHTMLStrings.CENTER);
			myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);

			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetColSpan("2");
			myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);

			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("40%");
			myCell.SetAlign(CommonHTMLStrings.RIGHT);
			myCell.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);

			myCell.AddInnerHTMLString(m_asOwner.GetString(3,
					"ptmsgs_portalbrowsingmsgs"));

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("60%");
			myCell.SetAlign(CommonHTMLStrings.LEFT);
			myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);

			myInput = new HTMLInput(HTMLInputTypes.TEXT,
				CreateAccountControl.PARAM_USERNAME, "id",
				m_asOwner.GetString(1796, "ptmsgs_portalbrowsingmsgs"));

			myInput.SetSize("30");
			myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
			if(m_asOwner.GetAccessStyle() == AccessStyles.STANDARD)
			{
				myInput.SetOnKeyPress("return " + PlumtreeHelpers.STR_JS_FUNC_NAME_EXECUTE_VIA_ENTER + "(event);");
			}

			myCell.AddInnerHTMLElement(myInput);

			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myCell = new HTMLTableCell();
			myCell.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);

			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("40%");
			myCell.SetAlign(CommonHTMLStrings.RIGHT);

			myCell.AddInnerHTMLString(m_asOwner.GetString(4,
					"ptmsgs_portalbrowsingmsgs"));

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("60%");
			myCell.SetAlign(CommonHTMLStrings.LEFT);
			myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);

			myInput = new HTMLInput(HTMLInputTypes.PASSWORD,
				CreateAccountControl.PARAM_USERPASS, "id", 
				m_asOwner.GetString(1797, "ptmsgs_portalbrowsingmsgs"));
				
			myInput.SetSize("30");
			myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
			if(m_asOwner.GetAccessStyle() == AccessStyles.STANDARD)
			{
				myInput.SetOnKeyPress("return " + PlumtreeHelpers.STR_JS_FUNC_NAME_EXECUTE_VIA_ENTER + "(event);");
			}

			myCell.AddInnerHTMLElement(myInput);

			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myCell = new HTMLTableCell();
			myCell.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);

			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("40%");
			myCell.SetAlign(CommonHTMLStrings.RIGHT);

			myCell.AddInnerHTMLString(m_asOwner.GetString(5,
					"ptmsgs_portalbrowsingmsgs"));

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("60%");
			myCell.SetAlign(CommonHTMLStrings.LEFT);
			myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);

			myInput = new HTMLInput(HTMLInputTypes.PASSWORD,
				CreateAccountControl.PARAM_CONFIRM_USERPASS, "id", 
				m_asOwner.GetString(1798, "ptmsgs_portalbrowsingmsgs"));

			myInput.SetSize("30");
			myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
			if(m_asOwner.GetAccessStyle() == AccessStyles.STANDARD)
			{
				myInput.SetOnKeyPress("return " + PlumtreeHelpers.STR_JS_FUNC_NAME_EXECUTE_VIA_ENTER + "(event);");
			}

			myCell.AddInnerHTMLElement(myInput);

			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);
			
			myCell = new HTMLTableCell();
			myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
			myCell.SetAlign(CommonHTMLStrings.LEFT);

			myRow.AddInnerHTMLElement(myCell);

			//Create Account button
			HTMLInput myLoginInput;
			if(m_asOwner.GetAccessStyle() == AccessStyles.STANDARD)
			{
				myLoginInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_CREATEACCOUNT, 
					m_asOwner.GetString(0, "ptmsgs_portalbrowsingmsgs"));
				myLoginInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);				
				myLoginInput.SetOnClick(STR_JS_CREATE_ACCOUNT + "(); return false;");
			} 
			else 
			{
				myLoginInput = new HTMLInput(HTMLInputTypes.SUBMIT_TYPE, PARAM_DOCREATE_508, 
					m_asOwner.GetString(0, "ptmsgs_portalbrowsingmsgs"));				
				myLoginInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);				
			}	
			myLoginInput.SetValue(m_asOwner.GetString(0, "ptmsgs_portalbrowsingmsgs"));

			myCell.AddInnerHTMLElement(myLoginInput);
		}
		catch (HTMLException he)
		{
			
				log.Error(he, "Error displaying Create Account HTML: ");
			
		}
		catch (Exception ex)
		{
			
				log.Error(ex, "Error in Create Account: ");
			
		}

		return eOut;
	}

	/**
	 * This method returns the HTMLScript that encompasses the Javascript on
	 * the page.
	 *
	 * @return HTMLScript
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		HTMLScript myScript = PlumtreeHelpers.GetExecuteViaEnterJScript(STR_JS_CREATE_ACCOUNT);
		
		XPStringBuilder sb  = new XPStringBuilder();
		
		sb.Append("function ").Append(STR_JS_CREATE_ACCOUNT).Append("()\n");
		sb.Append("{\n");

		sb.Append("	if ((typeof bCreating != 'undefined') && (bCreating))\n");
		sb.Append("	{\n");
		sb.Append("		return;\n");
		sb.Append("	}\n");
		sb.Append("	var b;\n");
		sb.Append("	b = CheckNames();\n");
		sb.Append("	if( b )\n");
		sb.Append("	{\n");
		sb.Append("		b = CheckMinLength();\n");
		sb.Append("		if( b )\n");
		sb.Append("		{\n");
		sb.Append("			b = CheckNotWhitespace();\n");
		sb.Append("		}\n");
		sb.Append("	}\n");
		sb.Append("	if( b )\n");
		sb.Append("	{\n");
		sb.Append("		bCreating = true;\n");
		sb.Append("		var doInp = document.").Append(CreateAccountView.sFormName).Append(".").Append(CreateAccountControl.PARAM_DOCREATEACCOUNT).Append(";\n");
		sb.Append("		doInp.value = \"").Append(XPConvert.ToString(true)).Append("\";\n");
		sb.Append("		document.").Append(CreateAccountView.sFormName).Append(".submit();\n");
		sb.Append("		return false;\n");
		sb.Append("	}\n");
		sb.Append("	return false;\n");
		sb.Append("}\n");

		// The JavaScript bellow checks if the Property Name is empty.
		sb.Append("function CheckNames()\n");
		sb.Append("{\n");
		sb.Append("  if (document." + CreateAccountView.sFormName + "." +
			CreateAccountControl.PARAM_USERPASS + ".value != document." +
			CreateAccountView.sFormName + "." +
			CreateAccountControl.PARAM_CONFIRM_USERPASS + ".value)\n");
		sb.Append("  {\n");
		sb.Append("      alert(\"" + m_asOwner.GetString(2171,"ptmsgs_portaladminmsgs") + "\");\n");
		sb.Append("      return false;\n");
		sb.Append("  }\n");
		sb.Append("  return true;\n");
		sb.Append("}\n");

		sb.Append("function CheckMinLength()\n");
		sb.Append("{\n");
		sb.Append("  if (document." + CreateAccountView.sFormName + "." +
			CreateAccountControl.PARAM_USERPASS + ".value.length < 5)\n");
		sb.Append("  {\n");
		sb.Append("      alert(\"" + m_asOwner.GetString(2172,"ptmsgs_portaladminmsgs") + "\");\n");
		sb.Append("      return false;\n");
		sb.Append("  }\n");
		sb.Append("  return true;\n");
		sb.Append("}\n");
		
		String strJSUtilVersionPrefix = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSUTIL);
		
		sb.Append("function CheckNotWhitespace()\n");
		sb.Append("{\n");
		sb.Append("  if ("+strJSUtilVersionPrefix+"PTStringUtil.isAllWhitespace(document." + CreateAccountView.sFormName + "." +
			CreateAccountControl.PARAM_USERPASS + ".value))\n");
		sb.Append("  {\n");
		sb.Append("      alert(\"" +
			m_asOwner.GetString(2173,"ptmsgs_portaladminmsgs") + "\");\n");
		sb.Append("      return false;\n");
		sb.Append("  }\n");
		
		sb.Append("  if ("+strJSUtilVersionPrefix+"PTStringUtil.isAllWhitespace(document." + CreateAccountView.sFormName + "." +
			CreateAccountControl.PARAM_USERNAME + ".value))\n");
		sb.Append("  {\n");
		sb.Append("      alert(\"" +
			m_asOwner.GetString(2483,"ptmsgs_portaladminmsgs") + "\");\n");
		sb.Append("      return false;\n");
		sb.Append("  }\n");
		sb.Append("  return true;\n");
		sb.Append("}\n");
		
		myScript.AddInnerHTMLString(sb.ToString());
		
		return myScript;
	}

	
	/**
	 * @see com.plumtree.xpshared.activityspace.IView#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#Init(IModel,
	 *      AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		m_asmModel     = (CreateAccountModel) model;
		m_asOwner	   = parent;
	}
}

