package com.plumtree.portalpages.browsing.myportal;


import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.gateway.*;
import com.plumtree.portalpages.browsing.objectselection.*;
import com.plumtree.portalpages.browsing.objectselection.portlets.*;
import com.plumtree.portalpages.browsing.portalsettings.displayoptions.*;
import com.plumtree.portalpages.common.uiparts.*;
import com.plumtree.portaluiinfrastructure.htmlconstructs.*;
import com.plumtree.portaluiinfrastructure.intrinsicportlets.*;
import com.plumtree.portaluiinfrastructure.layout.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.marshalers.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.activityspace.utils.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.utils.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * CUSTOMIZABLE - If you want to customize the my page and community page layouts, subclass this
 * class with a new class and customize the code in that new file. Do not alter the code this
 * class at all. You only need to override the methods that you want to customize. Your new class 
 * will override MyPortalContentView through Dynamic Discovery. Please read the section on 
 * Dynamic Discovery in the UI Guide.
 * 
 * The variables and methods in MyPortalContentView that you can customize in your new class are
 * bounded by the comments BEGIN CUSTOMIZABLE CODE and END CUSTOMIZABLE CODE. Each customizable 
 * variable and method is also marked as such in its javadoc comments.
 * 
 * The structure of how MyPortalContentView displays a mypage or community page is as follows.
 * Display determines whether to call OneColumn, TwoColumns, ThreeColumns based upon whether the
 * PageType of a page is a one, two, or three column layout.
 * OneColumn, TwoColumns, and ThreeColumns determine whether or not there is a content canvas area
 * to be displayed, where that content canvas area gets displayed, and they call PageColumnTable
 * for each column in the layout. If the content canvas area spans more than one portlet column, 
 * then it gets a call to PageColumnTable for itself.
 * PageColumnTable determines whether it needs to display a content canvas area above the portlets
 * in the column it is displaying, and makes a call to PortletContent for that content canvas portlet.
 * Then PageColumnTable makes a call to PortletHeader and PortletContent for each portlet in the 
 * column.
 * PortletContent returns the content of the portlet, it should not under normal circumstances need to
 * be customized.
 * PortletHeader delegates the display of the portlet header to 
 * com.plumtree.portaluiinfrastructure.htmlconstructs.PTPortletTitleBar. If you want to customize the
 * portlet header, make your own version of PTPortletTitleBar (with a different name) and have
 * PortletHeader use that instead.
 * PortletHeader calls GetPortletActions and passes the return value to PTPortletTitleBar.
 * GetPortletActions puts the list of buttons to be displayed in the portlet header (other than the 
 * help button) into an array and returns that array. You can add more buttons to this array, or
 * you can remove some of the default buttons.
 * GetLayoutTable, GetNarrowColumn, GetWideColumn, GetPortletVerticalPadding, 
 * GetPortletVerticalPaddingRow, and GetPortletHorizontalPadding are the glue used to bind the portlets
 * together and in most circumstances should not need to be customized.
 * 
 *
 * @author MichaelD
 */
public class MyPortalContentView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.myportal.MyPortalContentView");
	
	// BEGIN CUSTOMIZABLE CODE
	
	/** 
	 * CUSTOMIZABLE - The width of a narrow column.
	 */
	public static final int N_NARROW_COL_WIDTH = 250;
	 
	/** 
	 * CUSTOMIZABLE - The width of the padding between portlet columns and between portlets.
	 */
	public static final int N_PORTLET_PADDING = 10;
	
	/**
	 * These strings are used when composing the refresh link for the portlet.
	 */
	public static final String strRefreshJSFunctionBegin = "pt_";
	public static final String strRefreshJSFunctionEnd = ".refresh(); return false;";
	
	protected HTMLScript m_prefsDropDownInitializers = null;
	/**
	 * These strings are used when composing links for showing additional error information.
	 */
	public static final String strMoreInfoDiv = "pt-error-more-info-";
	public static final String strExpandedDiv = "pt-error-info-expanded-";
	public static final String strToggleJSFunctionBegin = "PTDOMUtil.toggleVisibility('";
	public static final String strToggleJSFunctionEnd = "');";
	public static final String strReturnFalse = "return false;";
	public static final String strDisplayNone = "display:none";
	public static final String strDisplayBlock = "display:block";
	
	
	/**
	 * CUSTOMIZABLE
	 * The Display() method's main function is to take the PageType of the page
	 * and determine whether the page has a one, two, or three column layout.
	 * Although the page types THREE_COLUMN_NARROW_WIDENARROW_WFFC and 
	 * THREE_COLUMN_NARROWWIDE_WFFC_NARROW are three column layout, we use the
	 * TwoColumns() method and create a nested table inside one of the columns
	 * that has two columns as well. TwoColumns() method uses the nPageType passed
	 * into it to determine this.
	 */
	public HTMLElement Display()
	{
		
			log.Info("MyPortalContentView.display: Display browsing page content");
		
			
		HTMLElement result = null;
		// Reset this every time we display
		m_prefsDropDownInitializers = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
		
		// Get layout type
		int nPageType = m_asModel.GetPageType();

		// Get access types
		try
		{
			HTMLElementCollection pageContent = new HTMLElementCollection();

			// Set up layout
			switch (nPageType)
			{
				case PageTypeConstants.ONE_COLUMN:
				case PageTypeConstants.ONE_COLUMN_WITH_FREE_FORM_CONTENT:
					pageContent.AddInnerHTMLElement(OneColumn(nPageType));
					break;


				case PageTypeConstants.TWO_COLUMN_NARROW_WIDE:
				case PageTypeConstants.TWO_COLUMN_WIDE_NARROW:
				case PageTypeConstants.TWO_COLUMN_NARROW_WIDE_WFFC:
				case PageTypeConstants.TWO_COLUMN_WIDE_WFFC_NARROW:
				case PageTypeConstants.TWO_COLUMN_WFFC_NARROW_WIDE:
				case PageTypeConstants.TWO_COLUMN_WFFC_WIDE_NARROW:
				case PageTypeConstants.THREE_COLUMN_NARROW_WIDENARROW_WFFC:
					pageContent.AddInnerHTMLElement(TwoColumns(nPageType,
							PortletColumnConstants.NARROW_ONE, true));
					break;


				case PageTypeConstants.THREE_COLUMN:
				case PageTypeConstants.THREE_COLUMN_NARROW_WIDE_WFF_NARROW:
				case PageTypeConstants.THREE_COLUMN_WITH_FREE_FORM_CONTENT:
					pageContent.AddInnerHTMLElement(ThreeColumns(nPageType));
					break;


				case PageTypeConstants.THREE_COLUMN_NARROWWIDE_WFFC_NARROW:
					pageContent.AddInnerHTMLElement(TwoColumns(nPageType,
							PortletColumnConstants.NARROW_TWO, true));
					break;


				default:
					// adaptive layout stores a negative value in the page type (negative value of a webservice layout id
					if ( nPageType < 0 ) {
						// this covers the case if there is a layout id set AND portlet layouts are disabled ->
						// revert to a regular 3 column layout
						// @TODO update to change for portlet layout mode disabled once that setting is added
						if ( !LayoutService.getChooserInstance().GetPortletLayoutsEnabled(m_asOwner) ) 
						{
							log.Info("Portlet page with id: "+m_asModel.GetPageID()+" uses adaptive portlet layout but that feature is disabled in portalconfig.xml. "+
									"Reverting to 3 column mode.");
							
							nPageType = PageTypeConstants.THREE_COLUMN_WITH_FREE_FORM_CONTENT;
							pageContent.AddInnerHTMLElement(ThreeColumns(nPageType));
						}
						
					} else {
						pageContent.AddInnerHTMLString("Page Type not found.");
						log.Warn("Page type: "+nPageType+" not recognized.");
					}
			}
			
			result = pageContent;
			
			if ( m_asOwner.GetAccessStyle() == AccessStyles.STANDARD)
			{
				// include this code for backwards compatibility, even if page layout is enabled
				
				String strImg = PTConfigHelper.GetImageServerBaseURL(m_asOwner)+PTImageType.ACTION_PORTLET_MENU.GetFileName();
				String strMenuImgAlt = m_asOwner.GetString(1839, "ptmsgs_portalbrowsingmsgs");
				
				String defaultStyleSheet = ConfigHelper.GetImageServerBaseURL(m_asOwner) + "common/private/js/jsportalmenus/" + 
						ConfigHelper.GetJSIncluderComponentVersion(ConfigHelper.JSCOMPONENT_JSPORTALMENU) + "/styles/css/PTPMMenu.css";
				
				
				HTMLScript createMenuFunction = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
				createMenuFunction.AddInnerHTMLString(
				  // don't do anything, assume jsportalmenus aren't on the page 
				  // css menus should be used instead
				  "try { PTPMMenu.defaultCssFile = \""+defaultStyleSheet+"\"; } catch (err) { ; } \n");
				
				createMenuFunction.AddInnerHTMLString(
						"function createPrefsDropDownMenu(linksArray, strDivID)\n"+
						"{ \n"+
						"	var menu;\n"+
						// dont do anything here, assume in adaptive layout mode where css menus are used
						"   try {  menu = new PTPMMenuContainer(); } catch (err) { return; }  \n"+ 
						"	// Add each entry to the PTPMMenuContainer\n"+
						"	for ( j = 0; j < linksArray.length; j++ )\n "+
						"	{\n"+
						"		var strImgSrc = linksArray[j][0];\n"+
						"		var strImgWidth = linksArray[j][1];\n"+
						"		var strTitle = linksArray[j][2];\n"+
						"		var strURL   = linksArray[j][3];\n"+
						"		var strSuppressPopup   = linksArray[j][4];\n"+
						"		menu.menuItems[j] = new PTPMSimpleMenuItem();\n"+
						"		// Set title\n"+
						"		menu.menuItems[j].text = strTitle;\n"+
						"		// Set Image\n"+
						"		menu.menuItems[j].image	= new PTPMImage();\n"+
						"		menu.menuItems[j].image.imgSrc = strImgSrc;\n"+
						"		menu.menuItems[j].image.imgWidth = strImgWidth;\n"+
						"		// Set action\n"+
						"		menu.menuItems[j].action = new PTPMJavaScriptAction();\n"+
						( (m_asOwner.GetAccessStyle() == AccessStyles.STANDARD) ?
								"       if ( strSuppressPopup ==  '0' ) {\n"+
								"		  menu.menuItems[j].action.js = 'OpenPortletPrefsWindow( \"'+strURL+'\")';\n"+
								"       } else {\n"+
								"		  menu.menuItems[j].action.js = 'window.location = \"'+strURL+'\"'; } \n"
							:
								"		menu.menuItems[j].action.js = 'window.location = \"'+strURL+'\"';\n"
						) +
						
						"	}\n"+
						
						"	var buttonText	=\"<img src='"+strImg+"' align='absmiddle' height='13' width='10' border='0' alt='"+strMenuImgAlt+"' >\" \n"+
						"	PTPMSelectMenu.init(menu, strDivID, buttonText);\n"+
						"}	\n"
				);
				result.AddInnerHTMLElement(createMenuFunction);
				
				result.AddInnerHTMLElement(m_prefsDropDownInitializers);
			}
			
		}
		catch (Exception e)
		{
			result = new HTMLGenericElement("Error creating page content: " +
					e.getMessage());
		}

		return result;
	}

	/**
	 * CUSTOMIZABLE
	 *
	 * @param nPageType 
	 *
	 * @return and HTMLTable that is the page layout of a one column page.
	 *
	 * @throws HTMLException 
	 */
	protected HTMLTable OneColumn(int nPageType)
		throws HTMLException
	{
		boolean bColumnOneContainsFreeFormContent = (PageTypeConstants.ONE_COLUMN_WITH_FREE_FORM_CONTENT == nPageType)
			? true : false;	
        ((PlumtreeAS) m_asOwner).SetStrLayoutStyleClass("layoutOneColumn");
        
		HTMLTable table = GetLayoutTable(m_asOwner.GetString(1807, "ptmsgs_portalbrowsingmsgs"));
        
		HTMLTableRow row = new HTMLTableRow();
		table.AddInnerHTMLElement(row);
        
		HTMLTableCell cell = GetWideColumn();
        cell.SetStyleClass("columnOne");
        cell.SetID("columnOne");
		row.AddInnerHTMLElement(cell);
		cell.AddInnerHTMLElement(PageColumnTable(PortletColumnConstants.WIDE_ONE.ToInt(),
			bColumnOneContainsFreeFormContent));
			
		return table;
	}

	/**
	 * CUSTOMIZABLE
	 *
	 * @param nPageType 
	 *
	 * @return a HTMLTable that is the page layout of a three column page.
	 *
	 * @throws HTMLException 
	 */
	protected HTMLTable ThreeColumns(int _nPageType)
		throws HTMLException
	{
		boolean bColumnOneContainsFreeFormContent = (PageTypeConstants.THREE_COLUMN_NARROW_WIDE_WFF_NARROW == _nPageType)
			? true : false;
        ((PlumtreeAS) m_asOwner).SetStrLayoutStyleClass("layoutThreeColumn");
        
		HTMLTable table = GetLayoutTable(m_asOwner.GetString(1807, "ptmsgs_portalbrowsingmsgs"));

		HTMLTableRow row = new HTMLTableRow();
		table.AddInnerHTMLElement(row);
		
		if (PageTypeConstants.THREE_COLUMN_WITH_FREE_FORM_CONTENT == _nPageType)
		{
			// Use the current row for the Content Canvas area and create a new one for the portlet columns
			HTMLTableCell cellContentCanvas = GetWideColumn();
			cellContentCanvas.SetID("contentCanvas");
			cellContentCanvas.SetStyleClass("contentCanvas");
			cellContentCanvas.SetColSpan("5");
			row.AddInnerHTMLElement(cellContentCanvas);
			cellContentCanvas.AddInnerHTMLElement(PageColumnTable(-1, true));

			table.AddInnerHTMLElement(GetPortletVerticalPaddingRow("5"));

			row = new HTMLTableRow();
			table.AddInnerHTMLElement(row);
		}
		
		HTMLTableCell cellLeftColumn   = GetNarrowColumn();
        cellLeftColumn.SetStyleClass("columnOne");
        cellLeftColumn.SetID("columnOne");
		row.AddInnerHTMLElement(cellLeftColumn);
		row.AddInnerHTMLElement(GetPortletHorizontalPaddingCell("0"));
		HTMLTableCell cellMiddleColumn = GetWideColumn();
        cellMiddleColumn.SetStyleClass("columnTwo");
        cellMiddleColumn.SetID("columnTwo");
		row.AddInnerHTMLElement(cellMiddleColumn);
		row.AddInnerHTMLElement(GetPortletHorizontalPaddingCell("1"));
		HTMLTableCell cellRightColumn  = GetNarrowColumn();
        cellRightColumn.SetStyleClass("columnThree");
        cellRightColumn.SetID("columnThree");
		row.AddInnerHTMLElement(cellRightColumn);

		try
		{
			cellLeftColumn.AddInnerHTMLElement(PageColumnTable(PortletColumnConstants.NARROW_ONE.ToInt(),
					false));
			cellMiddleColumn.AddInnerHTMLElement(PageColumnTable(PortletColumnConstants.WIDE_ONE.ToInt(),
					bColumnOneContainsFreeFormContent));
			cellRightColumn.AddInnerHTMLElement(PageColumnTable(PortletColumnConstants.NARROW_TWO.ToInt(),
					false));
		}
		catch (HTMLException e)
		{
			throw e;
		}

		return table;
	}

	/**
	 * CUSTOMIZABLE
	 *
	 * @param nPageType 
	 *
	 * @return a HTMLTable that is the layout of a two column page, or the outer and inner tables of
	 * the THREE_COLUMN_NARROW_WIDENARROW_WFFC and THREE_COLUMN_NARROWWIDE_WFFC_NARROW layouts.
	 *
	 * @throws HTMLException 
	 */
	protected HTMLTable TwoColumns(int _nPageType,
		PortletColumnConstants _narrowColumnToDisplay, boolean _bIsOuterTwoColumns)
		throws HTMLException
	{
		boolean bColumnOneContainsFreeFormContent;

		if ((PageTypeConstants.TWO_COLUMN_NARROW_WIDE_WFFC == _nPageType) ||
				(PageTypeConstants.TWO_COLUMN_WIDE_WFFC_NARROW == _nPageType))
		{
			bColumnOneContainsFreeFormContent = true;
		}
		else
		{
			bColumnOneContainsFreeFormContent = false;
		}
		
		HTMLTable table = GetLayoutTable(m_asOwner.GetString(1807, "ptmsgs_portalbrowsingmsgs"));
		HTMLTableRow row = new HTMLTableRow();
		table.AddInnerHTMLElement(row);
		
		if (PageTypeConstants.TWO_COLUMN_WFFC_NARROW_WIDE == _nPageType || PageTypeConstants.TWO_COLUMN_WFFC_WIDE_NARROW == _nPageType)
		{
			// Use the current row for the Content Canvas area and create a new one for the portlet columns
			HTMLTableCell cellContentCanvas = GetWideColumn();
			cellContentCanvas.SetID("contentCanvas");
			cellContentCanvas.SetStyleClass("contentCanvas");
			cellContentCanvas.SetColSpan("3");
			row.AddInnerHTMLElement(cellContentCanvas);
			cellContentCanvas.AddInnerHTMLElement(PageColumnTable(-1, true));

			table.AddInnerHTMLElement(GetPortletVerticalPaddingRow("3"));

			row = new HTMLTableRow();
			table.AddInnerHTMLElement(row);
		}

		HTMLTableCell cellNarrow  = GetNarrowColumn();
		HTMLTableCell cellWide    = GetWideColumn();
		// Add cells to row
		if ((_nPageType == PageTypeConstants.TWO_COLUMN_NARROW_WIDE) ||
				(_nPageType == PageTypeConstants.TWO_COLUMN_WFFC_NARROW_WIDE) ||
				(_nPageType == PageTypeConstants.TWO_COLUMN_NARROW_WIDE_WFFC) ||
				((_nPageType == PageTypeConstants.THREE_COLUMN_NARROW_WIDENARROW_WFFC) &&
				(_narrowColumnToDisplay.ToInt() == PortletColumnConstants.NARROW_ONE.ToInt())) ||
				((_nPageType == PageTypeConstants.THREE_COLUMN_NARROWWIDE_WFFC_NARROW) &&
				(_narrowColumnToDisplay.ToInt() == PortletColumnConstants.NARROW_ONE.ToInt())))
		{
			row.AddInnerHTMLElement(cellNarrow);
			row.AddInnerHTMLElement(GetPortletHorizontalPaddingCell("0"));
			row.AddInnerHTMLElement(cellWide);
            if (_bIsOuterTwoColumns) {
                ((PlumtreeAS) m_asOwner).SetStrLayoutStyleClass("layoutTwoColumnNarrowWide");
                cellNarrow.SetStyleClass("columnOne");
                cellWide.SetStyleClass("columnTwo");
                cellNarrow.SetID("columnOne");
                cellWide.SetID("columnTwo");
            } else {
                cellNarrow.SetStyleClass("columnTwo");
                cellWide.SetStyleClass("columnThree");
                cellNarrow.SetID("columnTwo");
                cellWide.SetID("columnThree");
            }
		}
		else
		{
			row.AddInnerHTMLElement(cellWide);
			row.AddInnerHTMLElement(GetPortletHorizontalPaddingCell("0"));
			row.AddInnerHTMLElement(cellNarrow);
            if (_bIsOuterTwoColumns) {
                ((PlumtreeAS) m_asOwner).SetStrLayoutStyleClass("layoutTwoColumnWideNarrow");
                cellWide.SetStyleClass("columnOne");
                cellNarrow.SetStyleClass("columnTwo");
                cellWide.SetID("columnOne");
                cellNarrow.SetID("columnTwo");
            } else {
                cellWide.SetStyleClass("columnTwo");
                cellNarrow.SetStyleClass("columnThree"); 
                cellWide.SetID("columnTwo");
                cellNarrow.SetID("columnThree"); 
            }
		}

		try
		{
			cellNarrow.AddInnerHTMLElement(PageColumnTable(
					_narrowColumnToDisplay.ToInt(), false));

			if ((_nPageType == PageTypeConstants.THREE_COLUMN_NARROW_WIDENARROW_WFFC) &&
					(_narrowColumnToDisplay.ToInt() == PortletColumnConstants.NARROW_ONE.ToInt()))
			{
                ((PlumtreeAS) m_asOwner).SetStrLayoutStyleClass("layoutThreeColumn");
                cellWide.SetStyleClass((StyleClass) null);
                
				cellWide.AddInnerHTMLElement(PageColumnTable(-1, true));
				cellWide.AddInnerHTMLElement(GetPortletVerticalPadding());
				cellWide.AddInnerHTMLElement(TwoColumns(_nPageType,
						PortletColumnConstants.NARROW_TWO, false));
                
			}
			else if ((_nPageType == PageTypeConstants.THREE_COLUMN_NARROWWIDE_WFFC_NARROW) &&
					(_narrowColumnToDisplay.ToInt() == PortletColumnConstants.NARROW_TWO.ToInt()))
			{
                ((PlumtreeAS) m_asOwner).SetStrLayoutStyleClass("layoutThreeColumn");
                cellWide.SetStyleClass((StyleClass) null);
                
				cellWide.AddInnerHTMLElement(PageColumnTable(-1, true));
				cellWide.AddInnerHTMLElement(GetPortletVerticalPadding());
				cellWide.AddInnerHTMLElement(TwoColumns(_nPageType,
						PortletColumnConstants.NARROW_ONE, false));
			}
			else
			{
				cellWide.AddInnerHTMLElement(PageColumnTable(
						PortletColumnConstants.WIDE_ONE.ToInt(),
						bColumnOneContainsFreeFormContent));
			}
		}
		catch (HTMLException e)
		{
			throw e;
		}

		return table;
	}
	
	/**
	 * CUSTOMIZABLE
	 * PortletContent must always be called before PortletHeader for a portlet.
	 * The portlet header might come from the portlet server, so we need to
	 * get the portlet content before getting the header.
	 * 
	 * Returns a page column for the MyPage.  Structure is: column 0 = narrow,
	 * left; column 1 = wide, center; column 2 = narrow, right.
	 *
	 * @param nColumnID 
	 * @param bContainsFreeFormContent 
	 *
	 * @return HTMLTable
	 *
	 * @throws HTMLException 
	 */
	protected HTMLElement PageColumnTable(int _nColumnID,
		boolean _bContainsFreeFormContent)
		throws HTMLException
	{
		HTMLElementCollection collection = new HTMLElementCollection();
		HTMLElement result = collection;
		
		// Keep this unlocalized. It is for IEControl tests.
		HTMLComment comment = new HTMLComment("Begin Portlet Column " + _nColumnID);
		collection.AddInnerHTMLElement(comment);

		// Portlet Content
		HTMLTable tablePortletContent;

		// Padding between portlets.
		HTMLTable tablePortletPadding = GetPortletVerticalPadding();

		if (_bContainsFreeFormContent &&
				(-1 != m_asModel.GetFirstPortletIndexInColumn(MyPortalModel.N_CONTENTCANVAS_COLUMN)))
		{
			// Display free form content
			int nFreeFormIndex = m_asModel.GetFirstPortletIndexInColumn(MyPortalModel.N_CONTENTCANVAS_COLUMN);

			// Portlet content.
			collection.AddInnerHTMLElement(PortletContent(nFreeFormIndex, -1));

			if (-1 != _nColumnID)
			{
				// Portlet Padding
				collection.AddInnerHTMLElement(tablePortletPadding);
			}
		}

		if (-1 != _nColumnID)
		{
			// Is the column empty?
			if (-1 == m_asModel.GetFirstPortletIndexInColumn(_nColumnID))
			{
				HTMLSpan span = new HTMLSpan();
				collection.AddInnerHTMLElement(span);
				span.SetStyleClass(PTStyleClass.ALERT_TEXT);

				span.AddInnerHTMLString(m_asOwner.GetString(671,
						"ptmsgs_portalbrowsingmsgs"));

				// only show the add portlets link if this is a personal page, or it's a community
				// and the user has edit access
				boolean bShowAddPortletsLink = false;

				NavigationModel navModel = ((NavigationModel)m_asOwner.GetModel(NavigationModel.STR_MVC_CLASS_NAME));
				if (!((navModel.IsGuestSession() && !navModel.IsDefaultProfile()) || navModel.IsPreviewPage()))
				{
					if (!m_asModel.GetIsCommunityPage())
					{
						bShowAddPortletsLink = true;
					}
					else if (TaskAPIUICommunity.IsCurrentUserAbleToEditCurrentCommunityPage(m_asOwner))
					{
						bShowAddPortletsLink = true;
					}
				}

				if (bShowAddPortletsLink)
				{
					ASURL asurlPortletSelection = new ASURL(m_asOwner);
					asurlPortletSelection.SetLinkCreateNewSpace(PortletSelectionEditorAS.STR_MVC_CLASS_NAME,
						m_asOwner);
					asurlPortletSelection.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
					asurlPortletSelection.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE,
						XPConvert.ToString(
							EditorStartControl.EDITOR_START_FLAG_MISC));

					asurlPortletSelection.AddControlArgument(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORPAGEID,
						XPConvert.ToString(m_asModel.GetPageID()));

					if (m_asModel.GetIsCommunityPage())
					{
						asurlPortletSelection.AddControlArgument(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORCOMMUNITYID,
							XPConvert.ToString(m_asModel.GetCommunityID()));
						asurlPortletSelection.AddInnerHTMLString(m_asOwner.GetString(
								670, "ptmsgs_portalbrowsingmsgs"));
					}
					else
					{
						asurlPortletSelection.AddInnerHTMLString(m_asOwner.GetString(
								669, "ptmsgs_portalbrowsingmsgs"));
					}
					
					HTMLAnchor myAnchor = (HTMLAnchor)asurlPortletSelection.GetURLAsHTMLElement();
					myAnchor.SetTitle(m_asOwner.GetString(300,"ptmsgs_portalcommonmsgs"));
					span.AddInnerHTMLElement(myAnchor);
				}
			}
			else
			{
				// Display all gadgets in column
				for (int i = 0; i < m_asModel.GetPortletCount(); i++)
				{
					if (_nColumnID == m_asModel.GetPortletColumnFromIndex(i))
					{
						collection.AddInnerHTMLElement( DisplaySinglePortlet(_nColumnID, i) );
					}
				}
			}

			// This spacer fixes a bunch(!!!) of netscape problems. It is necessarry to Get ns to display 
			// correctly and keep an extra level of table nesting out of the product		
			HTMLTable tableColumnFormatter		= GetLayoutTable(m_asOwner.GetString(1808, "ptmsgs_portalbrowsingmsgs"));
			HTMLTableRow rowColumnFormatter		= new HTMLTableRow();
			tableColumnFormatter.AddInnerHTMLElement(rowColumnFormatter);
			HTMLTableCell cellColumnFormatter	= new HTMLTableCell();
			cellColumnFormatter.SetID("portletColumnFormatter_" + _nColumnID);
			rowColumnFormatter.AddInnerHTMLElement(cellColumnFormatter);

			try
			{
				PTConstruct myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER,
						true, m_asOwner);

				HTMLImg img = (HTMLImg) myConstruct.GetHTMLElement();
				img.SetID("portletColumnSpacer_" + _nColumnID);
				cellColumnFormatter.AddInnerHTMLElement(img);
				img.SetAlt("");
				img.SetWidth(STR_NARROW_COL_WIDTH);
				img.SetHeight(CommonHTMLStrings.ONE);
				img.SetAlign(CommonHTMLStrings.ABSMIDDLE);
				img.SetBorder(CommonHTMLStrings.ZERO);
			}
			catch (Exception e)
			{
				
					log.Error(e, "Spacer image construct failed: " + e.getMessage());
				
			}

			collection.AddInnerHTMLElement(tableColumnFormatter);
		}
		
		// Keep this unlocalized. It is for IEControl tests.
		comment = new HTMLComment("End Portlet Column " + _nColumnID);
		collection.AddInnerHTMLElement(comment);

		return result;
	}

	/**
	 * CUSTOMIZABLE
	 *
	 * @param nColumnID			The ID of the column this portlet is currently in.
	 * @param nPortletIndex 
	 * 
	 * @return a HTMLElementCollection containing the entire portlet including the
	 * portlet header and content.
	 */
	public HTMLElementCollection DisplaySinglePortlet(int nColumnID,  int portletIndex) 
	{
		HTMLElementCollection collection = new HTMLElementCollection();
		// Portlet content must be called before gadget header -- header
		// might come from the gadget server, so that's why we need to
		// get the gadget content before getting the header.
		HTMLDiv innerPortletContent = PortletContent(portletIndex, nColumnID);
		
		HTMLTable portletTable = new HTMLTable();
		portletTable.SetStyleClass(PTStyleClass.PORTLET_BASE);
		portletTable.SetID(PTStyleID.GetIDStringForPortlet(m_asModel.GetPortletIDFromIndex(portletIndex)));
		portletTable.SetCellPadding("0");
		portletTable.SetCellSpacing("0");
		portletTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);

		if (m_asModel.GetPortletHasTitleBar(portletIndex) || m_asModel.GetDisplayPortletHeaderDueToError(portletIndex))
		{
			portletTable.AddInnerHTMLElement(PortletHeader(portletIndex, nColumnID));
		}
		
		HTMLTableRow row = (HTMLTableRow) portletTable.InsertInnerHTMLElement(new HTMLTableRow());
		
		HTMLTableCell cell = (HTMLTableCell) row.InsertInnerHTMLElement(new HTMLTableCell());
		cell.SetColSpan("2");
		cell.SetStyleClass(PTStyleClass.PORTLET_BODY);

		//Add the portlet content here always, even if the portlet is minimized,
		//there still may be JavaScript that needs to be inserted.
		cell.AddInnerHTMLElement(innerPortletContent);

		collection.AddInnerHTMLElement(portletTable);
		return collection;
	}

	/**
	 * CUSTOMIZABLE
	 *
	 * @param nColumnID			The ID of the column this portlet is currently in.
	 * @param nPortletIndex 
	 * 
	 * @return a HTMLElementCollection containing the portlet body/content.
	 */
	public HTMLElementCollection DisplaySinglePortletContent(int nColumnID, int portletIndex) 
	{
		HTMLElementCollection collection = new HTMLElementCollection();
		HTMLDiv innerPortletContent = PortletContent(portletIndex, nColumnID);

		collection.AddInnerHTMLElement(innerPortletContent);
		return collection;
	}
	
	/**
	 * CUSTOMIZABLE
	 *
	 * @param nPortletIndex 
	 * @param nColumnID			The ID of the column this portlet is currently in.
	 *
	 * @return a HTMLTableRow containing the portlet header.
	 *
	 * @throws HTMLException 
	 */
	protected HTMLTableRow PortletHeader(int nPortletIndex, int nColumnID)
		throws HTMLException
	{
		HTMLTableRow row = new HTMLTableRow();
		row.AddInnerHTMLElement(GetPortletNameCell(nPortletIndex, nColumnID));
		row.AddInnerHTMLElement(GetPortletButtonsCell(nPortletIndex));

		if ( log.IsDebugEnabled() ) {
			log.Debug("Portlet " + m_asModel.GetPortletIDFromIndex(nPortletIndex) + " Title bar.");
		}

		return row;
	}
	
	/**
	 * CUSTOMIZABLE
	 * @param nPortletIndex 
	 * @return A HTMLTableCell containing all the action buttons to be displayed in the portlet header.
	 */
	protected HTMLTableCell GetPortletButtonsCell(int nPortletIndex)
	{
		HTMLTableCell cell	    = new HTMLTableCell();

		cell.SetStyleClass(PTStyleClass.PLATPORTLET_HEADER_BG + " " + PTStyleClass.CUSTOMAPP_TEXT + " " + PTStyleClass.PORTLET_BUTTONS);
		cell.SetAlign(CommonHTMLStrings.RIGHT);
		cell.SetVAlign(CommonHTMLStrings.MIDDLE);
		cell.SetNoWrap(true);
		cell.SetColSpan(null);

		cell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		
		// Get the buttons for the portlet header if enabled/available.
		cell.AddInnerHTMLElement(GetPortletRefreshButtonContent(nPortletIndex));
		cell.AddInnerHTMLElement(GetPortletHelpButtonContent(nPortletIndex));
		cell.AddInnerHTMLElement(GetPortletPreferenceButtonContent(nPortletIndex));
		cell.AddInnerHTMLElement(GetPortletCollapseExpandButtonContent(nPortletIndex));
		cell.AddInnerHTMLElement(GetPortletRemoveButtonContent(nPortletIndex));
	
		return cell;
	}
	
	/**
	 * CUSTOMIZABLE
	 *
	 * @param nPortletIndex 
	 * @return a HTMLElementCollection containing the refresh button for the portlet header.
	 */
	public HTMLElementCollection GetPortletRefreshButtonContent(int nPortletIndex)
	{
		PTConstruct myConstruct = null;
		HTMLElementCollection collection = new HTMLElementCollection();

		// Put the refresh button in the title bar first
		// If we put in the refresh button, we should also put in a spacer cell afterwards
		try
		{
			// Refresh not supported in 508 mode, so we don't show refresh button at all
			if (m_asModel.GetPortletHasRefreshButton(nPortletIndex) &&
					AccessStyles.SECTION_508 != m_asOwner.GetAccessStyle())
			{
				//Add icon
				myConstruct = UIConstructFactory.GetConstruct(PTImageType.ACTION_PORTLET_REFRESH);

				HTMLImg myImage;
				myImage = (HTMLImg) myConstruct.GetHTMLElement();

				myImage.SetAlt(XPResourceManager.FormatMessage(m_asOwner.GetString(1814, 
						"ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex))));
				myImage.SetBorder(CommonHTMLStrings.ZERO);
				myImage.SetAlign(CommonHTMLStrings.ABSMIDDLE);

				HTMLAnchor myAnchor;
					myAnchor = new HTMLAnchor(CommonHTMLStrings.POUND_SYMBOL);
					myAnchor.SetOnClick(GetRefreshJS(m_asModel.GetPortletIDFromIndex(nPortletIndex)));
				myAnchor.SetTitle(XPResourceManager.FormatMessage(m_asOwner.GetString(1814, 
						"ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex))));
				myAnchor.AddInnerHTMLElement(myImage);
				

				// Put the icon and link inside a span, with class = PORTLET_REFRESH_BUTTON
				HTMLSpan span = new HTMLSpan();
				span.SetStyleClass(PTStyleClass.PORTLET_REFRESH_BUTTON);
				span.AddInnerHTMLElement(myAnchor);
				
				collection.AddInnerHTMLElement(span);
				
				//Now, add the spacer
				collection.AddInnerHTMLElement(PlumtreeHelpers.GetSpacerImage("2", "1"));
				collection.AddInnerHTMLString(CommonHTMLStrings.LINE_BREAK);
			}
		}
		catch (Exception e)
		{
			log.Error(e, "Failed getting portlet refresh button.");
		}
		
		return collection;
	}
	
	


	/**
	 * CUSTOMIZABLE
	 *
	 * @param nPortletIndex 
	 * @return a HTMLElementCollection containing the help button for the portlet header.
	 */
	public HTMLElementCollection GetPortletHelpButtonContent(int nPortletIndex)
	{
		PTConstruct myConstruct = null;
		HTMLElementCollection collection = new HTMLElementCollection();
		
		// Put help button in title bar
		// If we put in the help button, we should also put in a spacer cell afterwards
		try
		{
			if (m_asModel.GetPortletHasHelpURL(nPortletIndex))
			{
				// Add icon
				myConstruct = UIConstructFactory.GetConstruct(PTImageType.ACTION_PORTLET_HELP);

				HTMLImg myImage;
				myImage = (HTMLImg) myConstruct.GetHTMLElement();

				myImage.SetAlt(XPResourceManager.FormatMessage(m_asOwner.GetString(1793,
						"ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex))));
				myImage.SetBorder(CommonHTMLStrings.ZERO);
				myImage.SetAlign(CommonHTMLStrings.ABSMIDDLE);

				HTMLAnchor myAnchor;
				if (AccessStyles.STANDARD == m_asOwner.GetAccessStyle())
				{
					String strJSUtilVersionPrefix = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSUTIL);
					myAnchor = new HTMLAnchor(CommonHTMLStrings.POUND_SYMBOL);
					myAnchor.SetOnClick(strJSUtilVersionPrefix+"PTWindowUtil.openHelpWindow('" + m_asModel.GetPortletHelpURL(nPortletIndex) + "', '600', '600', true);return false;");
				}
				else
				{
					myAnchor = new HTMLAnchor(m_asModel.GetPortletHelpURL(nPortletIndex));
				}
				myAnchor.SetTitle(XPResourceManager.FormatMessage(m_asOwner.GetString(1793,
					"ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex))));
				myAnchor.AddInnerHTMLElement(myImage);
				
				// Put the icon and link inside a span, with class = PORTLET_HELP_BUTTON
				HTMLSpan span = new HTMLSpan();
				span.SetStyleClass(PTStyleClass.PORTLET_HELP_BUTTON);
				span.AddInnerHTMLElement(myAnchor);
				
				collection.AddInnerHTMLElement(span);
				
				//Now, add the spacer
				collection.AddInnerHTMLElement(PlumtreeHelpers.GetSpacerImage("2", "1"));
				collection.AddInnerHTMLString(CommonHTMLStrings.LINE_BREAK);
			}
		}
		catch (Exception e)
		{
			log.Error(e, "Failed getting gadget help button.");
		}

		return collection;
	}

	/**
	 * CUSTOMIZABLE
	 *
	 * @param nPortletIndex 
	 * @return a HTMLElementCollection containing the preference button for the portlet header.
	 */
	public HTMLElementCollection GetPortletPreferenceButtonContent(int nPortletIndex)
	{
		HTMLElementCollection collection = new HTMLElementCollection();

		// Put preference action button in title bar
		// After each button, put in a spacer cell
		try
		{
			HTMLSpan[] preferenceActions = GetPortletPreferenceAction(nPortletIndex);
			
			if (preferenceActions != null)
			{
				for (int i=0; i< preferenceActions.length; i++) {
					collection.AddInnerHTMLElement(preferenceActions[i]);
					collection.AddInnerHTMLElement(PlumtreeHelpers.GetSpacerImage("2", "1"));
					collection.AddInnerHTMLString(CommonHTMLStrings.LINE_BREAK);
				}
			}
		}
		catch (Exception e)
		{
			log.Error(e, "Failed getting preference gadget button.");
		}

		return collection;
	}	
	
	/**
	 * CUSTOMIZABLE
	 *
	 * @param nPortletIndex 
	 * @return a HTMLElementCollection containing the collapse/expand button for the portlet header.
	 */
	public HTMLElementCollection GetPortletCollapseExpandButtonContent(int nPortletIndex)
	{
		return GetPortletCollapseExpandButtonContent(nPortletIndex, null);
	}
	
	public HTMLElementCollection GetPortletCollapseExpandButtonContent(int nPortletIndex, String strOnclickFunction)
	{
		HTMLElementCollection collection = new HTMLElementCollection();

		// Put collapse/expand action button in title bar
		// After each button, put in a spacer cell
		try
		{
			HTMLSpan collapseExpandAction = GetPortletCollapseExpandAction(nPortletIndex, strOnclickFunction);
			
			if (collapseExpandAction != null)
			{
				collection.AddInnerHTMLElement(collapseExpandAction);
				collection.AddInnerHTMLElement(PlumtreeHelpers.GetSpacerImage("2", "1"));
				collection.AddInnerHTMLString(CommonHTMLStrings.LINE_BREAK);
			}
		}
		catch (Exception e)
		{
			log.Error(e, "Failed getting collapse/expand gadget button.");
		}

		return collection;
	}

	public HTMLElementCollection GetPortletRemoveButtonContent(int nPortletIndex)
	{
		HTMLElementCollection collection = new HTMLElementCollection();

		// Put Remove action button in title bar
		// After each button, put in a spacer cell
		try
		{
			HTMLSpan RemoveAction = GetPortletRemoveAction(nPortletIndex);
			
			if (RemoveAction != null)
			{
				collection.AddInnerHTMLElement(RemoveAction);
				collection.AddInnerHTMLElement(PlumtreeHelpers.GetSpacerImage("2", "1"));
				collection.AddInnerHTMLString(CommonHTMLStrings.LINE_BREAK);
			}
		}
		catch (Exception e)
		{
			log.Error(e, "Failed getting Remove button.");
		}

		return collection;
	}
	/**
	 * CUSTOMIZABLE
	 * @param nPortletIndex 
	 * @param nColumnID			The ID of the column this portlet is currently in.
	 * @return a HTMLTableCell containing the portlet's name.
	 */
	protected HTMLTableCell GetPortletNameCell(int nPortletIndex, int nColumnID)
	{
		// Put gadget name in title bar
		HTMLTableCell cell = new HTMLTableCell();

		cell.SetStyleClass(PTStyleClass.PLATPORTLET_HEADER_BG + " " + PTStyleClass.CUSTOMAPP_TEXT + " " + PTStyleClass.PORTLET_TITLE);
		cell.SetVAlign(CommonHTMLStrings.MIDDLE);
		cell.SetNoWrap(false);
		cell.SetColSpan(null);

		if (AccessStyles.STANDARD == m_asOwner.GetAccessStyle())
		{
			HTMLSpan span = new HTMLSpan();
			cell.AddInnerHTMLElement(span);
	
			if (1 == nColumnID)
			{
				span.SetStyleClass(PTStyleClass.PLATPORTLET_WIDE_HEADER);
			}
			else
			{
				span.SetStyleClass(PTStyleClass.PLATPORTLET_NARROW_HEADER);
			}

			span.AddInnerHTMLEncodedString(m_asModel.GetPortletNameFromIndex(nPortletIndex));
		}
		else
		{
			HTMLH4 h4 = new HTMLH4();
			cell.SetHeight("18");
			cell.AddInnerHTMLElement(h4);

			if (1 == nColumnID)
			{
				h4.SetStyleClass(PTStyleClass.PLATPORTLET_WIDE_HEADER);
			}
			else
			{
				h4.SetStyleClass(PTStyleClass.PLATPORTLET_NARROW_HEADER);
			}
			
			h4.AddInnerHTMLEncodedString(m_asModel.GetPortletNameFromIndex(nPortletIndex));
			Add508AccessKeyAnchor(m_asModel.GetPortletNameFromIndex(nPortletIndex), nPortletIndex, cell);
		}
		
		return cell;
	}
	
	/**
	 * Given a cell and an int (a portlet index) adds HTML for an access 
	 * anchor corresponding to that int, the title is set to the portlet name.
	 * 
	 * @param strPortletName
	 * @param nPortletIndex
	 * @param cell
	 */
	protected void Add508AccessKeyAnchor(String strPortletName, int nPortletIndex, HTMLTableCell cell)
	{
		String strPortletIndex = XPConvert.ToString(nPortletIndex);			
		HTMLAnchor tAnchor = new HTMLAnchor("");			
		tAnchor.SetName(strPortletIndex);
		tAnchor.SetTitle(XPHttpUtility.HtmlEncode(strPortletName));
		cell.AddInnerHTMLElement(tAnchor);
	}
	
	/**
	 * Obtain the Preference Button from the portlet header given the 
	 * portlet index.
	 * 
	 * @param nPortletIndex
	 * @return Preference HTML button. NOTE: May contain null elements
	 */
	protected HTMLSpan[] GetPortletPreferenceAction(int nPortletIndex)
	{
		// TODO CR DLH: This code is very very confusing and should probably be rewritten,
		// or at least well commented.  The easiest thing to do for now is to just step
		// through the code in a debugger to see what's happening, because it's
		// almost impossible to figure it out from reading it.

		PTConstruct myConstruct 	= null;
		HTMLImg preferenceImage 	= null;
		HTMLAnchor preferenceAnchor = null;
		HTMLSpan[] arPreferenceSpan	= null;

		IXPList arPrefActions = m_asModel.GetPortletPreferenceActionsData(nPortletIndex);
		
		/*
		 * Page preference button and dropdown twizzle 
		 */
		if ( arPrefActions.GetSize() > 0 )
		{
			boolean bUsePopup = (m_asOwner.GetAccessStyle() == AccessStyles.STANDARD);
			String strPrefsButtonURL = null;
			String strPrefsButtonTitle = null;
			String strPrefsType = null;

			// Export this as a xml setting
			boolean bUseDropDownMenus = true;
			
			// 508 accessibility mode
			if  (m_asOwner.GetAccessStyle() != AccessStyles.STANDARD )
			{
				bUseDropDownMenus = false;
			}
			
			JS2DArrayBuilder arMenuEntries = new JS2DArrayBuilder(5);
			for (int i=0; i<arPrefActions.GetSize(); i++) {
				IPTURL oPrefURL = (IPTURL) arPrefActions.GetElement(i);
				if (0 == i) {
					strPrefsButtonURL = oPrefURL.GetFullURL();
					strPrefsButtonTitle = oPrefURL.GetData(PTURLConstants.KEY_TITLE);
					strPrefsType = oPrefURL.GetData(MyPortalModel.PORTLET_PREF_TYPE);
				}
				// Add to dropdown menu
				arMenuEntries.AddNewElement();
				arMenuEntries.SetColumnStringValue(0, oPrefURL.GetData(PTURLConstants.KEY_ICONIMG));
				arMenuEntries.SetColumnStringValue(1, "16");
				arMenuEntries.SetColumnStringValue(2, oPrefURL.GetData(PTURLConstants.KEY_TITLE));
				arMenuEntries.SetColumnStringValue(3, oPrefURL.GetFullURL());
				if (MyPortalModel.ADMIN_PREF_TYPE.equals(oPrefURL.GetData(MyPortalModel.PORTLET_PREF_TYPE))) {
					arMenuEntries.SetColumnStringValue(4, "1");
				} else {
					arMenuEntries.SetColumnStringValue(4, "0");
				}
			}
			
			// Dropdown menu initialization
			int nCurPortletID = m_asModel.GetPortletIDFromIndex(nPortletIndex);
			String strMenuHolderID = "Portlet_"+nCurPortletID+"_MenuHolder";
			BrowserType bt = new BrowserType(m_asOwner.GetCurrentHTTPRequest());
			
			boolean bShowMenu = arMenuEntries.GetSize() > 1 && bUseDropDownMenus && 
							  !( bt.IsMac() && (bt.IsIE() || bt.IsSafari()) );
			
			// DCA - only show dropdown menu when we're in classic mode
			int nLayoutID = -1;
			NavigationModel navModel = (NavigationModel)m_asOwner.GetModel(NavigationModel.STR_MVC_CLASS_NAME);						
			if ((navModel != null) && (navModel.IsCommunityPage()) && (navModel.GetCurCommunityID() == PT_INTRINSICS.PT_PROFILE_COMMUNITY))
				nLayoutID = LayoutService.getChooserInstance().GetLayoutWebService(Layouts.PROFILE_PAGE, m_asOwner);
			else
				nLayoutID = LayoutService.getChooserInstance().GetLayoutWebService(Layouts.BASE_PAGE, m_asOwner);
			bShowMenu = bShowMenu && (nLayoutID < 0);
			
			// don't create a dropdown menu unless we have more than one entry
			if (  bShowMenu ) 
			{
				// create new dropdown menu
				JSFunctionCallBuilder createMenu = new JSFunctionCallBuilder("createPrefsDropDownMenu");
				// pass in the array directly as a method arugment 
				createMenu.AddVariableArgument(arMenuEntries.ToString());
				// div id to insert the menu into
				createMenu.AddStringValueArgument(strMenuHolderID);
				if ( null == m_prefsDropDownInitializers )
				{
					m_prefsDropDownInitializers = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
				}
				m_prefsDropDownInitializers.AddInnerHTMLString(createMenu.ToString()+";\n");
			}
			
			// Create edit prefs button anchor
			myConstruct     = UIConstructFactory.GetConstruct(PTImageType.ACTION_PORTLET_EDIT, true, m_asOwner);
			preferenceImage     = (HTMLImg) myConstruct.GetHTMLElement();
			preferenceImage.SetAlt(strPrefsButtonTitle);
			preferenceImage.SetBorder(CommonHTMLStrings.ZERO);
			preferenceImage.SetAlign(CommonHTMLStrings.ABSMIDDLE);
			
			if ( bShowMenu )
			{
				// Don't want a popup for the admin prefs page since it goes to the portlet editor
				if ( bUsePopup && !MyPortalModel.ADMIN_PREF_TYPE.equals(strPrefsType))
				{
					preferenceAnchor = new HTMLAnchor(CommonHTMLStrings.POUND_SYMBOL);
					preferenceAnchor.SetOnClick("OpenPortletPrefsWindow('"+strPrefsButtonURL+"'); return false;");
				} else {

					preferenceAnchor = new HTMLAnchor(strPrefsButtonURL);
				}
				preferenceAnchor.SetTitle(strPrefsButtonTitle);
				preferenceAnchor.AddInnerHTMLElement(preferenceImage);
				
				// Create outer table for button and menu twizzle
				HTMLTable table = new HTMLTable();
				table.SetBorder("0");
				table.SetCellPadding("0");
				table.SetCellSpacing("0");
				table.SetStyle("display:inline; vertical-align:bottom; line-height: 0.6;");
				
				
				HTMLTableRow row = new HTMLTableRow();
				table.AddInnerHTMLElement(row);
				HTMLTableCell cell = new HTMLTableCell();
				cell.SetStyle("padding: 0px 0px 1px 0px;");
				// Add button
				row.AddInnerHTMLElement(cell);
				cell.AddInnerHTMLElement(preferenceAnchor);
				// Add container for dropdown twizzle
				cell = new HTMLTableCell();
				row.AddInnerHTMLElement(cell);
				cell.SetID(strMenuHolderID);
				
				HTMLSpan prefSpan = new HTMLSpan();
				prefSpan.SetStyleClass(PTStyleClass.PORTLET_PREFS_BUTTON);
				prefSpan.AddInnerHTMLElement(table);
				
				arPreferenceSpan	= new HTMLSpan[1];
				arPreferenceSpan[0] = prefSpan;	
			}
			else 
			{
				// Add each pref as a span
				arPreferenceSpan	= new HTMLSpan[arPrefActions.GetSize()];
				for (int i=0; i<arPrefActions.GetSize(); i++) {
					IPTURL oPrefURL = (IPTURL) arPrefActions.GetElement(i);
					strPrefsButtonURL = oPrefURL.GetFullURL();
					strPrefsButtonTitle = oPrefURL.GetData(PTURLConstants.KEY_TITLE);
					strPrefsType = oPrefURL.GetData(MyPortalModel.PORTLET_PREF_TYPE);
					
					// Don't want a popup for the admin prefs page since it goes to the portlet editor
					if ( bUsePopup && !MyPortalModel.ADMIN_PREF_TYPE.equals(strPrefsType))
					{
						preferenceAnchor = new HTMLAnchor(CommonHTMLStrings.POUND_SYMBOL);
						preferenceAnchor.SetOnClick("OpenPortletPrefsWindow('"+strPrefsButtonURL+"'); return false;");
					} else {
						preferenceAnchor = new HTMLAnchor(strPrefsButtonURL);
					}
					preferenceAnchor.SetTitle(strPrefsButtonTitle);

					// Create edit prefs button anchor - not sure why this is done above and wasn't done here,
					// but since each link has a different alt string, this needs to be created each time.
					myConstruct     = UIConstructFactory.GetConstruct(PTImageType.ACTION_PORTLET_EDIT, true, m_asOwner);
					preferenceImage     = (HTMLImg) myConstruct.GetHTMLElement();
					preferenceImage.SetAlt(strPrefsButtonTitle);
					preferenceImage.SetBorder(CommonHTMLStrings.ZERO);
					preferenceImage.SetAlign(CommonHTMLStrings.ABSMIDDLE);
					
					preferenceAnchor.AddInnerHTMLElement(preferenceImage);
					
					HTMLSpan prefSpan = new HTMLSpan();
					prefSpan.SetStyleClass(PTStyleClass.PORTLET_PREFS_BUTTON);
					prefSpan.AddInnerHTMLElement(preferenceAnchor);	
					
					arPreferenceSpan[i] = prefSpan;	
				}
				
			}
		}

		return arPreferenceSpan;
	}
	
	/**
	 * Obtain the collapse button from the portlet header given the portlet index.
	 * 
	 * @param nPortletIndex
	 * @param strOnclickFunction - name of javascript function to call if there is one, null otherwise
	 * @return Collapse/Expand HTML button. NOTE: May contain null elements
	 */
	protected HTMLSpan GetPortletCollapseExpandAction(int nPortletIndex, String strOnclickFunction)
	{
		HTMLAnchor collapseAnchor = null;
		HTMLSpan collapseSpan	= null;
		
		// Collapse/Expand link
		if (!m_asModel.GetPortletIsMandatory(nPortletIndex) && !MyPortalModel.IsGuestSession(m_asOwner))
		{
			// Only Add if portlet is not mandatory
			if (m_asModel.GetPortletCollapsed(nPortletIndex))
			{
				// Get Expand Anchor
				collapseAnchor = GetCollapseOrExpandAnchor(nPortletIndex,
					CollapsePortletControl.STR_EXPAND_CONTROL,
					PTImageType.ACTION_PORTLET_EXPAND,
					m_asOwner.GetString(1802, "ptmsgs_portalbrowsingmsgs")
				);
			}
			else if ((!m_asModel.GetPortletCollapsed(nPortletIndex))
					&& (m_asModel.GetCommunityID() != PT_INTRINSICS.PT_PROFILE_COMMUNITY))
			{
				// Get Collapse Anchor, but only if the page is not the profile community
				// note: if a portlet is already collapsed somehow, we still allow the expand
				//       anchor to be displayed, even if it's the profile community
				collapseAnchor = GetCollapseOrExpandAnchor(nPortletIndex,
					CollapsePortletControl.STR_COLLAPSE_CONTROL,
					PTImageType.ACTION_PORTLET_COLLAPSE,
					m_asOwner.GetString(1803, "ptmsgs_portalbrowsingmsgs")
				);
			}
			
			if (null != collapseAnchor && null != strOnclickFunction) {
				// DCA - change Anchor from having an href to an onclick
				String strCollapseURL = collapseAnchor.GetHRef();
				collapseAnchor.SetHRef(CommonHTMLStrings.POUND_SYMBOL);
				collapseAnchor.SetOnClick("return " + strOnclickFunction + "('" + strCollapseURL + "');");
			}
			
			collapseSpan = new HTMLSpan();
			collapseSpan.SetStyleClass(PTStyleClass.PORTLET_COLLAPSE_BUTTON);
			collapseSpan.AddInnerHTMLElement(collapseAnchor);
		}

		return collapseSpan;
	}

	protected HTMLSpan GetPortletRemoveAction(int nPortletIndex)
	{
		// For Community Pages verifying If user is not Community Owner	
		boolean bIsCommunityPage = m_asModel.GetCommunityID() > 0;		
		boolean	bHasCommPrivilage = (PT_GADGETPERSONALIZATION_ROLES.PT_GADGETPERSONALIZATION_COMMUNITYOWNER ) > 0 &&
									TaskAPIUICommunity.IsCurrentUserCommunityOwner(m_asOwner);
					
		if(bIsCommunityPage && !bHasCommPrivilage)
		{	// If user not community owner Remove Button is not displayed
			return null;
		} 
		
		int nPageId = m_asModel.GetPageID();
		// don't check last modified date, opening this page should have already updated the cache entry if necessary 
		IPTPageInfo page = ((IPTPageManager) m_asModel.GetSession().GetPages()).CachedOpenPageInfo(nPageId, !bIsCommunityPage, false);
		boolean bPageInherited = page.GetInheritPageTemplate();
		// If current Page is Inherites a Template, Then Remove Button is not displayed
		// User can only make changes to Template.
		if(bPageInherited)
		{
			return null;
		}
		
		HTMLAnchor removeAnchor = null;
		
		//		 Remove link
		if (!m_asModel.GetPortletIsMandatory(nPortletIndex) && !MyPortalModel.IsGuestSession(m_asOwner) )
		{
			// Only Add if portlet is not mandatory
			removeAnchor = GetRemoveAnchor(nPortletIndex,
				AddRemovePortletControl.STR_MVC_CLASS_NAME,
				PTImageType.ACTION_PORTLET_REMOVE,
				m_asOwner.GetString(1906, "ptmsgs_portalbrowsingmsgs")
			);
	
		}
				
		HTMLSpan removeSpan = new HTMLSpan();
		removeSpan.SetStyleClass(PTStyleClass.PORTLET_REMOVE_BUTTON);
		removeSpan.AddInnerHTMLElement(removeAnchor);

		return removeSpan;
	}
	/**
	 * @param nPortletIndex
	 * @param strCollapseOrExpandControl
	 * @param imgType
	 * @param strTitle
	 * @return anchor for collapsing or expanding the portlet
	 */
	protected HTMLAnchor GetCollapseOrExpandAnchor(int nPortletIndex, String strControl, PTImageType imgType, String strTitle)
	{
		// Get Collapse or Expand Image
		PTConstruct myConstruct     = UIConstructFactory.GetConstruct(imgType);
		HTMLImg actionImage     = (HTMLImg) myConstruct.GetHTMLElement();
		actionImage.SetAlt(XPResourceManager.FormatMessage(strTitle, XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex))));
		actionImage.SetBorder(CommonHTMLStrings.ZERO);
		actionImage.SetAlign(CommonHTMLStrings.ABSMIDDLE);

		// Build URL for Collapse Or Expand
		ASURL collURL = m_asModel.GetPageASURL();
		collURL.AddControlArgument(CollapsePortletControl.STR_PORTLET_TO_COLLAPSE_EXPAND, m_asModel.GetPortletIDFromIndex(nPortletIndex));
		collURL.SetControl(strControl);
		
		// Get Anchor for Collapse or Expand
		HTMLAnchor actionAnchor = (HTMLAnchor)collURL.GetURLAsHTMLElement();
		actionAnchor.SetTitle(XPResourceManager.FormatMessage(strTitle, XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex))));
		actionAnchor.AddInnerHTMLElement(actionImage);

		return actionAnchor;
	}
	
	/**
	 * 
	 * @param nPortletIndex
	 * @param strControl
	 * @param imgType
	 * @param strTitle
	 * @return anchor for Removing Portlet.
	 */
	protected HTMLAnchor GetRemoveAnchor (int nPortletIndex, String strControl, PTImageType imgType, String strTitle)
	{
		// Get Remove Image
		PTConstruct myConstruct     = UIConstructFactory.GetConstruct(imgType);
		HTMLImg actionImage     = (HTMLImg) myConstruct.GetHTMLElement();
		actionImage.SetAlt(XPResourceManager.FormatMessage(strTitle, XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex))));
		actionImage.SetBorder(CommonHTMLStrings.ZERO);
		actionImage.SetAlign(CommonHTMLStrings.ABSMIDDLE);
		
		// Build URL for Remove
		ASURL collURL = m_asModel.GetPageASURL();
		collURL.AddControlArgument(AddRemovePortletControl.STR_PORTLET_TO_REMOVE, m_asModel.GetPortletIDFromIndex(nPortletIndex));
		collURL.SetControl(strControl);
		
		// Get Anchor for Remove
		HTMLAnchor actionAnchor = (HTMLAnchor)collURL.GetURLAsHTMLElement();
		actionAnchor.SetOnClick("RemovePortlet('" + collURL.GetURLAsString() + "', "+ m_asModel.GetPortletIDFromIndex(nPortletIndex) +"); return false;");
		actionAnchor.SetHRef("#");
		actionAnchor.SetTitle(XPResourceManager.FormatMessage(strTitle, XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex))));
		actionAnchor.AddInnerHTMLElement(actionImage);
		
		return actionAnchor;
	}

	/**
	 * CUSTOMIZABLE
	 * Method GetLayoutTable.
	 * 
	 * @param _strTableSummary	Summary of the table.
	 * @return HTMLTable
	 */
	public HTMLTable GetLayoutTable(String _strTableSummary)
	{
		HTMLTable table = new HTMLTable();
		table.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		table.SetBorder(CommonHTMLStrings.ZERO);
		table.SetCellPadding(CommonHTMLStrings.ZERO);
		table.SetCellSpacing(CommonHTMLStrings.ZERO);
		table.SetSummary(_strTableSummary);
		
		return table;	
	}

	/**
	 * CUSTOMIZABLE
	 * Method GetNarrowColumn.
	 * 
	 * @return HTMLTableCell - has no width
	 */
	protected HTMLTableCell GetNarrowColumn()
	{
		HTMLTableCell cell = new HTMLTableCell();
		cell.SetAlign(CommonHTMLStrings.CENTER);
		cell.SetVAlign(CommonHTMLStrings.TOP);
		
		return cell;
	}

	/**
	 * CUSTOMIZABLE
	 * Method GetWideColumn.
	 * 
	 * @return HTMLTableCell - has 100% width
	 */
	protected HTMLTableCell GetWideColumn()
	{
		HTMLTableCell cell = new HTMLTableCell();
		cell.SetAlign(CommonHTMLStrings.CENTER);
		cell.SetVAlign(CommonHTMLStrings.TOP);
		cell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		
		return cell;
	}

	/**
	 * CUSTOMIZABLE
	 * Method GetPortletVerticalPadding.
	 * Gets a table with a spacer image to put padding 
	 * between each portlet from top to bottom.
	 * 
	 * @return HTMLTable
	 */
	public HTMLTable GetPortletVerticalPadding()
	{
		HTMLTable table = GetLayoutTable(m_asOwner.GetString(1809, "ptmsgs_portalbrowsingmsgs"));
		table.AddInnerHTMLElement(GetPortletVerticalPaddingRow(CommonHTMLStrings.ONE));
		
		return table;
	}

	/**
	 * CUSTOMIZABLE
	 * Method GetPortletVerticalPadding.
	 * Gets a row with a spacer image to put padding 
	 * between each portlet from top to bottom.
	 * 
	 * @param _colSpan
	 */
	protected HTMLTableRow GetPortletVerticalPaddingRow(String _colSpan) {
		HTMLTableRow row = new HTMLTableRow();
		HTMLTableCell cell = new HTMLTableCell();
		cell.SetID("portletVerticalPaddingRow" + _colSpan);
		row.AddInnerHTMLElement(cell);
		
		try
		{
			PTConstruct myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER,
					true, m_asOwner);
		
			HTMLImg img = (HTMLImg) myConstruct.GetHTMLElement();
			cell.AddInnerHTMLElement(img);
			img.SetAlt("");
			img.SetWidth(CommonHTMLStrings.ONE);
			img.SetHeight(STR_PORTLET_PADDING);
			img.SetAlign(CommonHTMLStrings.ABSMIDDLE);
			img.SetBorder(CommonHTMLStrings.ZERO);
		}
		catch (Exception e)
		{
			log.Error(e, "Spacer image construct failed: " + e.getMessage());

			cell.SetHeight(STR_PORTLET_PADDING);
			cell.AddInnerHTMLElement(new HTMLBr());
		}
		
		return row;
	}
	
	/**
	 * CUSTOMIZABLE
	 * Method GetPortletHorizontalPaddingCell.
	 * Returns a cell with a spacer image in it to put padding between portlet columns.
	 */
	protected HTMLTableCell GetPortletHorizontalPaddingCell()
	{
		return GetPortletHorizontalPaddingCell("0");
	}

	/**
	 * CUSTOMIZABLE
	 * Method GetPortletHorizontalPaddingCell.
	 * Returns a cell with a spacer image in it to put padding between portlet columns.
	 * 
	 * @return HTMLTableCell
	 */
	protected HTMLTableCell GetPortletHorizontalPaddingCell(String _strColumnID)
	{
		HTMLTableCell cell = new HTMLTableCell();
		cell.SetID("horizontalPaddingCell" + _strColumnID);
		try
		{
			PTConstruct myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER,
					true, m_asOwner);

			HTMLImg img = (HTMLImg) myConstruct.GetHTMLElement();
			cell.AddInnerHTMLElement(img);
			img.SetAlt("");
			img.SetWidth(STR_PORTLET_PADDING);
			img.SetHeight(CommonHTMLStrings.ONE);
			img.SetAlign(CommonHTMLStrings.ABSMIDDLE);
			img.SetBorder(CommonHTMLStrings.ZERO);
		}
		catch (Exception e)
		{
			log.Error(e, "Spacer image construct failed: " + e.getMessage());

			cell.SetWidth(STR_PORTLET_PADDING);
			cell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		}

		return cell;		
	}

	/**
	 * CUSTOMIZABLE
	 * Method AddPTConstantsToPageScript.
	 * Adds the PTConstants JavaScript object to the portal page,
	 * Initializing the values properly.  The script argument
	 * is a reference to the script object being passed in from
	 * the DisplayJavascript method.
	 * 
	 * @return HTMLTableCell
	 */
	protected void AddPTConstantsToPageScript(HTMLScript script)
	{
		XPStringBuilder sb = new XPStringBuilder("// Define PTPortalContext for CSAPI\n\n");
		
		sb.Append("PTPortalContext = new Object();\n");

		final String m_sessionPrefsURL = ConfigHelper.GetApplicationBaseURL(m_asOwner) + 
									"space=SessionPrefs&control=SessionPrefs&action=";
		final String m_getPrefsURL = JSHelper.escapeJS(m_sessionPrefsURL + "getprefs");
		final String m_setPrefsURL = JSHelper.escapeJS(m_sessionPrefsURL + "setprefs");

		sb.Append("PTPortalContext.GET_SESSION_PREFS_URL = '" + m_getPrefsURL + "';\n");
		sb.Append("PTPortalContext.SET_SESSION_PREFS_URL = '" + m_setPrefsURL + "';\n");

		sb.Append("\n");
		
		String locale = m_asOwner.GetLocale();
		sb.Append("PTPortalContext.USER_LOCALE = '" + locale + "';\n");
		
		String loginName = ((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo().GetCurrentUserLoginName();
		sb.Append("PTPortalContext.USER_LOGIN_NAME = '" + JSHelper.escapeJS(loginName) + "';\n");
		
		sb.Append("\n");
		
		String pageType = (m_asModel.GetIsCommunityPage()) ? "PTPortalPage.TYPE_COMMUNITY_PAGE" : "PTPortalPage.TYPE_MY_PAGE";
		sb.Append("PTPortalContext.PAGE_TYPE = " + pageType + ";\n");

		sb.Append("\n");
		
		script.AddInnerHTMLString(sb.toString());
	}

	// END CUSTOMIZABLE CODE
	

	/** Object Name */
	public static final String STR_MVC_CLASS_NAME = "MyPortalContent";

	/** Narrow Column Width - String*/
	public static final String STR_NARROW_COL_WIDTH = XPConvert.ToString(N_NARROW_COL_WIDTH);
	
	/** Padding between Portlet Columns and between portlets - String */
	public static final String STR_PORTLET_PADDING = XPConvert.ToString(N_PORTLET_PADDING);
   
	/** Parent Activity Space */
	private AActivitySpace m_asOwner;

	/** Read Only Model for MyPortal */
	private IMyPortalModelRO m_asModel;

	
	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new MyPortalContentView();
	}

	/**
	 * This method returns the HTMLScript that encompasses the Javascript on
	 * the page.
	 *
	 * @return HTMLScript
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		HTMLScript myScript = new HTMLScript("text/javascript");

		// adds definition for PTPortalContext
		AddPTConstantsToPageScript(myScript);

		myScript.AddInnerHTMLString("function OpenPortletPrefsWindow(strURL) { window.open(strURL,'PortletPrefsWindow',"
						+ "'width=700,height=500,resizable=1,status=0,toolbar=0,menubar=0,scrollbars=1,location=0');   }\n");
		
		// add function for confirmation handling of single-click MyPage delete
		myScript.AddInnerHTMLString("\nvar MSG_REMOVE_PAGE_CONF = \"" + 
				m_asOwner.GetString(1935, "ptmsgs_portalbrowsingmsgs") +
				"\";\n");
		myScript.AddInnerHTMLString("function DeletePage(url) {\n   if (confirm(MSG_REMOVE_PAGE_CONF)) {\n" +
				"      window.location = url;\n" +
				"   }\n}\n");

		// add function for confirmation handling of single-click portlet remove
		myScript.AddInnerHTMLString("\nvar MSG_REMOVE_PORTLET_CONF = \"" + 
				m_asOwner.GetString(1937, "ptmsgs_portalbrowsingmsgs") +
				"\";\n");
		myScript.AddInnerHTMLString("function RemovePortlet(url, id) {\n   if (confirm(MSG_REMOVE_PORTLET_CONF)) {\n" +
				"      if (typeof handleRemovePortletResponse != 'function') {\n" +
				"            window.location = url;\n" +
				"      } else {\n" +
				"            url = url + '&" + ASConstants.PAGE + "=" + MyPortalPartialPortletsDP.STR_MVC_CLASS_NAME + "';\n"+
				"            var request = new PTHTTPGETRequest(url, handleRemovePortletResponse);\n            request.invoke();\n" +
				"            swapPortlet(id);" +
				"      }\n" +
				"   }\n}\n");
		
		int communityId = m_asModel.GetCommunityID();
		int pageId = m_asModel.GetPageID();
		String pageName = JSHelper.JSEncodeString( m_asModel.GetPageName() ); 
		// used by drag and drop code
		myScript.AddInnerHTMLString("PTPortalContext.commId = " + communityId + "; \n");
		myScript.AddInnerHTMLString("PTPortalContext.pageId = " + pageId+ "; \n");
		myScript.AddInnerHTMLString("PTPortalContext.pageName = '" + XPHttpUtility.UrlEncode(pageName, "UTF-8") + "'; \n");

		return myScript;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#Init(IModel,
	 *      AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		m_asModel     = (IMyPortalModelRO) model;
		m_asOwner     = parent;
	}

	/**
	 * // CR ROB: This file is customizable by users so add in useful descriptions of method and arguments
	 * 
	 * @param nPortletIndex 
	 * @param nColumnID NOT USED
	 *
	 * @return an HTMLTable containing the portlet content or an error message telling why the portlet content was not displayed.
	 *
	 * @throws HTMLException 
	 */
	public HTMLDiv PortletContent(int nPortletIndex, int nColumnID) throws HTMLException
	{
		HTMLDiv div	   = new HTMLDiv();
		div.SetID(PlumtreeExtensibility.STR_PT_PORTLET_CONTENT + XPConvert.ToString(m_asModel.GetPortletIDFromIndex(nPortletIndex)));
		
		Object[][] objArrMarkup = null;

		if (!m_asModel.GetPortletEnabled(nPortletIndex))
		{
			// Portlet is disabled
			div.SetStyleClass(PTStyleClass.ALERT_TEXT);
			boolean bCollapsed = m_asModel.GetPortletCollapsed(nPortletIndex);

			if (!bCollapsed)
			{
				div.AddInnerHTMLString(m_asModel.GetPortletDisabledMessage(nPortletIndex));
			}

			// If there is a refresh button, we need to put in the PTPortlet constructor.
			DisplayCollapsedOrErrorContent(nPortletIndex, div, bCollapsed);
			
			return div;
		}
		else if ((m_asOwner.GetAccessStyle() == AccessStyles.SECTION_508) && (!m_asModel.GetPortletSupports508(nPortletIndex)))
		{
			// show no-508-support message
			div.SetStyleClass(PTStyleClass.ALERT_TEXT);
			div.AddInnerHTMLString(m_asOwner.GetString(1266, "ptmsgs_portalbrowsingmsgs"));

			return div;
		}
		else
		{
			try
			{
				objArrMarkup = m_asModel.GetPortletHTMLTextFromIndex(nPortletIndex);
			}
			catch (Throwable ex)
			{
				
				if( ex instanceof PTException )
				{
					PTException ptex = (PTException) ex;

					div.SetStyleClass(PTStyleClass.ALERT_TEXT);
				
					String strErrorComment = ptex.getMessage();
					String strErrorDescription;
					Object[] aoRecommenedActions;
					
					// If there is a refresh button, we need to put in the PTPortlet constructor.
					DisplayCollapsedOrErrorContent(nPortletIndex, div, false);

					switch ( ptex.GetErrorCode() )
					{
						case PT_RESULTCODES.PT_E_GADGET_TIMEOUT:
						{
							strErrorDescription = XPResourceManager.FormatMessage(m_asOwner.GetString(1526, "ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex)));
							aoRecommenedActions = new Object[3];
							ASURL myASURL = new ASURL(m_asOwner);
							myASURL.SetLinkCreateNewSpace(DisplayOptAS.STR_MVC_CLASS_NAME, m_asOwner);
							myASURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
							myASURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
							myASURL.AddInnerHTMLString(m_asOwner.GetString(1597, "ptmsgs_portalbrowsingmsgs"));
							aoRecommenedActions[0] = myASURL.GetURLAsHTMLElement();
							aoRecommenedActions[1] = m_asOwner.GetString(1594, "ptmsgs_portalbrowsingmsgs");
							aoRecommenedActions[2] = m_asOwner.GetString(1595, "ptmsgs_portalbrowsingmsgs");
							break;
						}
						
						default:
						{
							strErrorDescription = XPResourceManager.FormatMessage(m_asOwner.GetString(1527, "ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex)));
							aoRecommenedActions = new Object[2];
							aoRecommenedActions[0] = m_asOwner.GetString(1596, "ptmsgs_portalbrowsingmsgs");
							aoRecommenedActions[1] = m_asOwner.GetString(1595, "ptmsgs_portalbrowsingmsgs");
						}
					}
					// always display the gadget header if an error occurred.
					m_asModel.SetDisplayPortletHeaderDueToError(nPortletIndex, true);
									
					DisplayPortletError(div, strErrorDescription, XPHttpUtility.HtmlEncode(strErrorComment), aoRecommenedActions, nPortletIndex, null);
	
					return div;
					
				}
				else
				{
					String strErrorComment = ex.getMessage();
					String strErrorDescription = XPResourceManager.FormatMessage(m_asOwner.GetString(1527, "ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex)));
					Object[] aoRecommenedActions = new Object[2];
					aoRecommenedActions[0] = m_asOwner.GetString(1596, "ptmsgs_portalbrowsingmsgs");
					aoRecommenedActions[1] = m_asOwner.GetString(1595, "ptmsgs_portalbrowsingmsgs");

					DisplayPortletError(div, strErrorDescription, strErrorComment, aoRecommenedActions, nPortletIndex, null);
	
					return div;
				}
				
			}

		}
		
		if (GatewayUtils.IsErrorContentArray(objArrMarkup))
		{
			DisplayCollapsedOrErrorContent(nPortletIndex, div, true);

			int iErrorCode = GatewayUtils.GetErrorCode(objArrMarkup);
			String strErrorDescription = null;
			String strErrorComment = GatewayUtils.GetErrorMessage(objArrMarkup);
			Object[] aoRecommenedActions = new Object[2];
			
			if (iErrorCode == PT_RESULTCODES.PT_E_GADGET_TIMEOUT)
			{
				strErrorDescription = XPResourceManager.FormatMessage(m_asOwner.GetString(1526, "ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex)));
				ASURL myASURL = new ASURL(m_asOwner);
				myASURL.SetLinkCreateNewSpace(DisplayOptAS.STR_MVC_CLASS_NAME, m_asOwner);
				myASURL.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
				myASURL.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
				myASURL.AddInnerHTMLString(m_asOwner.GetString(1597, "ptmsgs_portalbrowsingmsgs"));
				aoRecommenedActions[0] = myASURL.GetURLAsHTMLElement();
				aoRecommenedActions[1] = m_asOwner.GetString(1595, "ptmsgs_portalbrowsingmsgs");
			}
			else if (iErrorCode == PT_RESULTCODES.PT_E_GADGET_HTTP_ERROR_CODE)
			{
				strErrorDescription = XPResourceManager.FormatMessage(m_asOwner.GetString(1879, "ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex)));
				aoRecommenedActions[0] = m_asOwner.GetString(1596, "ptmsgs_portalbrowsingmsgs");
				aoRecommenedActions[1] = m_asOwner.GetString(1595, "ptmsgs_portalbrowsingmsgs");
			}
			else if (iErrorCode == PT_RESULTCODES.PT_E_GADGET_REDIRECT_OUTSIDE_GATEWAY)
			{
				strErrorDescription = XPResourceManager.FormatMessage(m_asOwner.GetString(1880, "ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex)));
				aoRecommenedActions[0] = m_asOwner.GetString(1596, "ptmsgs_portalbrowsingmsgs");
				aoRecommenedActions[1] = m_asOwner.GetString(1595, "ptmsgs_portalbrowsingmsgs");
			}
			else 
			{
				strErrorDescription = XPResourceManager.FormatMessage(m_asOwner.GetString(1527, "ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex)));
				aoRecommenedActions[0] = m_asOwner.GetString(1596, "ptmsgs_portalbrowsingmsgs");
				aoRecommenedActions[1] = m_asOwner.GetString(1595, "ptmsgs_portalbrowsingmsgs");
			}
			
			
			m_asModel.SetDisplayPortletHeaderDueToError(nPortletIndex, true);
			
			DisplayPortletError(div, strErrorDescription, strErrorComment, aoRecommenedActions, nPortletIndex, objArrMarkup);

			return div;
			
		}
		div.SetStyleClass(PTStyleClass.CUSTOMAPP_TEXT);

		// Handle intrinsic portlets
		if (IntrinsicPortletModel.IsIntrinsicPortlet(objArrMarkup))
		{
			try
			{
				// Get HTML for intrinsic portlet
				div.AddInnerHTMLElement(GetIntrinsicPortletContent(
						nPortletIndex, objArrMarkup));
				
				//Now, we need to add the PTPortlet constructor.
				if ((objArrMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT].length == 2)
						&& (XPConvert.ToInteger(objArrMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_TYPE][1]) == 
							PT_MARKUP_FRAGMENT_TYPES.PT_MARKUP_FRAGMENT_TYPE_FINISHED))
				{
					// Just add the markup - the first entry in the array relating to Intrinsic provider stuff will be ignored.
					// NOTE: the shared JS needs to be displayed before all portlets
					div.AddInnerHTMLElement(GatewayUtils.AddMarkupToElement(m_asOwner, objArrMarkup));	
				}
						
						
			}
			catch (Exception ex)
			{
				div.SetStyleClass(PTStyleClass.ALERT_TEXT);
									
				String strErrorComment = ex.getMessage();
				String strErrorDescription = XPResourceManager.FormatMessage(m_asOwner.GetString(1527, "ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(m_asModel.GetPortletNameFromIndex(nPortletIndex)));
				Object[] aoRecommenedActions = new Object[2];
				aoRecommenedActions[0] = m_asOwner.GetString(1596, "ptmsgs_portalbrowsingmsgs");
				aoRecommenedActions[1] = m_asOwner.GetString(1595, "ptmsgs_portalbrowsingmsgs");

				DisplayPortletError(div, strErrorDescription, strErrorComment, aoRecommenedActions, nPortletIndex, null);

				return div;									
			}
		}
		else
		{
			MyPortalDP mDP = (MyPortalDP)m_asOwner.GetPage(MyPortalDP.STR_MVC_CLASS_NAME);
			
			// invoke the "UI transformer" function -jas
			// NOTE: the shared JS needs to be displayed before all portlets
			div.AddInnerHTMLElement(GatewayUtils.AddMarkupToElement(m_asOwner, objArrMarkup));
		}

		return div;
	}

	/**
	 * Retrieves HTML for intrinsic portlet.
	 *
	 * @param nPortletIndex 
	 * @param objArrMarkup 
	 *
	 * @return 
	 */
	private HTMLElement GetIntrinsicPortletContent(int nPortletIndex,
		Object[][] objArrMarkup)
	{
		String strPortletClass = (String) objArrMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][0];
		int nPortletID		   = m_asModel.GetPortletIDFromIndex(nPortletIndex);
		String strStorageName  = strPortletClass + nPortletID;

		// BT: Bug 64737 - Moved SetupIntrinsicPortlet call to MyPortalModel::InitializePortletContent
		// We shouldn't be initializing anything in the view.
		// We are deliberately breaking the rules for models and views here.  Cast RO interface
		// back to model and call setup method from model. -jic
		/*
		MyPortalModel myPortalModel = (MyPortalModel) m_asModel;
		myPortalModel.SetupIntrinsicPortlet(strPortletClass, strStorageName,
			nPortletIndex);
		*/

		IntrinsicPortletView iPortletView = (IntrinsicPortletView) m_asOwner.GetView(strStorageName);

		return iPortletView.Display();
	}
	
	private void DisplayPortletError(HTMLElement _element, String _strErrorDescription, String _strErrorComment, 
									 Object[] _aoRecommendedActions, int _iPortletIndex, Object[][] objArrMarkup)
	{
		HTMLBr br = new HTMLBr();
		
		HTMLSpan mySpan = new HTMLSpan();
		_element.AddInnerHTMLElement(mySpan);
		mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_TITLE);
		try {
			PTConstruct myConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_ERROR);
			HTMLImg img = (HTMLImg) myConstruct.GetHTMLElement();

			img.SetAlt(m_asOwner.GetString(1592, "ptmsgs_portalbrowsingmsgs"));
				
			mySpan.AddInnerHTMLElement(img);
		}
		catch (Exception e) {
			log.Error(e, "DisplayPortletError: Failed to create error image. " + e.getMessage());
		}
		
		mySpan.AddInnerHTMLString(CommonHTMLStrings.SPACE + m_asOwner.GetString(1592, "ptmsgs_portalbrowsingmsgs"));
		
		_element.AddInnerHTMLElement(br);
				
		mySpan = new HTMLSpan();
		_element.AddInnerHTMLElement(mySpan);
		mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_DESCRIPTION);
		mySpan.AddInnerHTMLString(_strErrorDescription);
		
		
		HTMLP midPara = new HTMLP();
		_element.AddInnerHTMLElement(midPara);
		mySpan = new HTMLSpan();
		midPara.AddInnerHTMLElement(mySpan);
		mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_SUBTITLE);
		mySpan.AddInnerHTMLString(m_asOwner.GetString(1593, "ptmsgs_portalbrowsingmsgs"));

		//Get all the info for the refresh link
		int iPortletID = m_asModel.GetPortletIDFromIndex(_iPortletIndex);
		String strPortletName = m_asModel.GetPortletNameFromIndex(_iPortletIndex);
		String strRefreshText = XPResourceManager.FormatMessage(m_asOwner.GetString(1838, "ptmsgs_portalbrowsingmsgs"), XPHttpUtility.HtmlEncode(strPortletName));
		HTMLAnchor refreshLink = new HTMLAnchor(CommonHTMLStrings.POUND_SYMBOL);
		refreshLink.SetOnClick(GetRefreshJS(iPortletID));
		refreshLink.AddInnerHTMLString(strRefreshText);
		refreshLink.SetTitle(strRefreshText);
		
		//Add refresh link.
		midPara.AddInnerHTMLElement(br);
		midPara.AddInnerHTMLString("&bull;" + CommonHTMLStrings.SPACE);
		midPara.AddInnerHTMLElement(refreshLink);
		
		//Add recommended actions
		for (int i = 0; i < _aoRecommendedActions.length; i++)
		{
			midPara.AddInnerHTMLElement(br);
			midPara.AddInnerHTMLString("&bull;" + CommonHTMLStrings.SPACE);
			if (null !=_aoRecommendedActions[i])
			{
				if (_aoRecommendedActions[i] instanceof String)
				{
					midPara.AddInnerHTMLString(XPConvert.ToString(_aoRecommendedActions[i]));
				}
				else
				{
					midPara.AddInnerHTMLElement((HTMLElement)_aoRecommendedActions[i]);
				}
			}
		}

		midPara.AddInnerHTMLElement(br);
		HTMLComment commentError = new HTMLComment(_strErrorComment);
		_element.AddInnerHTMLElement(commentError);
		
		//Now, get the config setting for whether we should print extra debugging information.
		IPTServerConfigSettings config = (IPTServerConfigSettings) ((IPTSession) m_asOwner.GetUserSession()).
		OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_SERVERCONFIGSETTINGS, false);

		// now, print out the extra debugging info.
		if (objArrMarkup != null && GatewayUtils.IsErrorContentArray(objArrMarkup)
				&& (config.GetSettingAsInt(PT_SERVER_CONFIG_SETTINGNAME.PT_SETTINGNAME_ENABLE_DEBUG_PORTLET_ERRORS_MODE) == 1))
		{
			boolean bResponseReceived = GatewayUtils.ResponseReceivedForErrors(objArrMarkup);
			boolean bTimeoutInfoReceived = GatewayUtils.TimeoutErrorReceived(objArrMarkup);
			boolean bExceptionStackTraceReceived = GatewayUtils.ExceptionStackReceived(objArrMarkup);
			
			//Initialize the strings, we have to do this to get rid of compiler warnings
			String strResponseStatus, strResponseBody, strPortletTimeout, strEffectiveTimeout, strExceptionStackTrace;
			strResponseStatus = null;
			strResponseBody = null;
			strPortletTimeout = null;
			strEffectiveTimeout = null;
			strExceptionStackTrace = null;
			int iPageTimeout = 0;
			
			
			if (bResponseReceived)
			{
				strResponseStatus = GatewayUtils.GetResponseStatus(objArrMarkup);
				strResponseBody = GatewayUtils.GetResponseBody(objArrMarkup);
			}
			
			if (bTimeoutInfoReceived)
			{
				strPortletTimeout = GatewayUtils.GetPortletTimeout(objArrMarkup);
				strEffectiveTimeout = GatewayUtils.GetEffectivePortletTimeout(objArrMarkup);
				
                iPageTimeout = 0;
                try
                {
                	Object objSettingValue = PTPersonalSettingsHelper.GetPersonalSettingValue(
                							PersonalSettingNames.PORTLET_TIMEOUT, m_asOwner);
                    if (null != objSettingValue)
                    {
                    	iPageTimeout = XPConvert.ToInteger(objSettingValue);
                    }
                }
                catch (XPException xpe)
                {
                     //JD- Ignore errors on reading the setting- it's probably not set
                }
				// Gotta transfer into seconds from milliseconds
				if (iPageTimeout > 0)
				{
					iPageTimeout = iPageTimeout/1000;
				}
			}
			
			if (bExceptionStackTraceReceived)
			{
				strExceptionStackTrace = GatewayUtils.GetExcepionStackTrace(objArrMarkup);
			}
		
			HTMLDiv moreInfoDiv = new HTMLDiv();
			String strMoreInfoDivID = strMoreInfoDiv + XPConvert.ToString(iPortletID);
			String strExpandedDivID = strExpandedDiv + XPConvert.ToString(iPortletID);
			moreInfoDiv.SetID(strMoreInfoDivID);
			moreInfoDiv.SetStyle(strDisplayBlock);
			
			HTMLAnchor moreInfoLink = new HTMLAnchor(CommonHTMLStrings.POUND_SYMBOL);
			moreInfoLink.SetOnClick(GetToggleJS(strMoreInfoDivID) + GetToggleJS(strExpandedDivID) + strReturnFalse);
			moreInfoLink.SetTitle(m_asOwner.GetString(1850, "ptmsgs_portalbrowsingmsgs"));
			moreInfoLink.AddInnerHTMLString(m_asOwner.GetString(1849, "ptmsgs_portalbrowsingmsgs"));
			moreInfoLink.AddInnerHTMLElement(br);
			
			moreInfoDiv.AddInnerHTMLElement(moreInfoLink);
			_element.AddInnerHTMLElement(moreInfoDiv);
			
			HTMLDiv expandedInfoDiv = new HTMLDiv();
			expandedInfoDiv.SetStyle(strDisplayNone);
			expandedInfoDiv.SetID(strExpandedDivID);
			if (bResponseReceived)
			{
				expandedInfoDiv.AddInnerHTMLElement(br);
				expandedInfoDiv.AddInnerHTMLString("&bull;" + CommonHTMLStrings.SPACE);
				expandedInfoDiv.AddInnerHTMLString(m_asOwner.GetString(1854, "ptmsgs_portalbrowsingmsgs") + CommonHTMLStrings.SPACE + strResponseStatus);
				expandedInfoDiv.AddInnerHTMLElement(br);
				expandedInfoDiv.AddInnerHTMLString("&bull;" + CommonHTMLStrings.SPACE);
				expandedInfoDiv.AddInnerHTMLString(m_asOwner.GetString(1856, "ptmsgs_portalbrowsingmsgs") + strResponseBody);
			}
			
			if (bTimeoutInfoReceived)
			{
				expandedInfoDiv.AddInnerHTMLElement(br);
				expandedInfoDiv.AddInnerHTMLString("&bull;" + CommonHTMLStrings.SPACE);
				expandedInfoDiv.AddInnerHTMLString(XPResourceManager.FormatMessage(
						m_asOwner.GetString(1853, "ptmsgs_portalbrowsingmsgs"), strPortletTimeout));
				expandedInfoDiv.AddInnerHTMLElement(br);
				expandedInfoDiv.AddInnerHTMLString("&bull;" + CommonHTMLStrings.SPACE);
				String strPageTimeoutValue;
				if (iPageTimeout > 0)
				{
					strPageTimeoutValue = XPResourceManager.FormatMessage(m_asOwner.GetString(1873, "ptmsgs_portalbrowsingmsgs"),
							XPConvert.ToString(iPageTimeout));
				}
				else
				{
					strPageTimeoutValue = m_asOwner.GetString(1874, "ptmsgs_portalbrowsingmsgs");
				}
				expandedInfoDiv.AddInnerHTMLString(XPResourceManager.FormatMessage
						(m_asOwner.GetString(1855, "ptmsgs_portalbrowsingmsgs"), strPageTimeoutValue));
				expandedInfoDiv.AddInnerHTMLElement(br);
				expandedInfoDiv.AddInnerHTMLString("&bull;" + CommonHTMLStrings.SPACE);
				expandedInfoDiv.AddInnerHTMLString(XPResourceManager.FormatMessage
						(m_asOwner.GetString(1875, "ptmsgs_portalbrowsingmsgs"), strEffectiveTimeout));
				
			}
			
			if (bExceptionStackTraceReceived)
			{
				expandedInfoDiv.AddInnerHTMLElement(br);
				expandedInfoDiv.AddInnerHTMLString("&bull;" + CommonHTMLStrings.SPACE);
				expandedInfoDiv.AddInnerHTMLString(m_asOwner.GetString(1876, "ptmsgs_portalbrowsingmsgs") + CommonHTMLStrings.SPACE + strExceptionStackTrace);
			}

			HTMLAnchor hideInfoLink = new HTMLAnchor(CommonHTMLStrings.POUND_SYMBOL);
			hideInfoLink.SetOnClick(GetToggleJS(strMoreInfoDivID) + GetToggleJS(strExpandedDivID) + strReturnFalse);
			hideInfoLink.SetTitle(m_asOwner.GetString(1852, "ptmsgs_portalbrowsingmsgs"));
			hideInfoLink.AddInnerHTMLString(m_asOwner.GetString(1851, "ptmsgs_portalbrowsingmsgs"));
			expandedInfoDiv.AddInnerHTMLElement(br);
			expandedInfoDiv.AddInnerHTMLElement(br);
			expandedInfoDiv.AddInnerHTMLElement(hideInfoLink);
			_element.AddInnerHTMLElement(expandedInfoDiv);
			
		}
	}
	
	private void DisplayCollapsedOrErrorContent(int nPortletIndex, HTMLElement element, boolean bSkipRefreshCheck)
	{
		// If there is a refresh button, we need to put in the PTPortlet constructor.
		// If bSkipRefreshCheck is true, that means we don't need to worry about checking whether this portlet has a refresh button.
		if ((bSkipRefreshCheck || m_asModel.GetPortletHasRefreshButton(nPortletIndex))
				&& (m_asOwner.GetAccessStyle() != AccessStyles.SECTION_508))
		{
			Object[][] objArrMarkup = null;

			try
			{
				objArrMarkup = m_asModel.GetCollapsedOrErrorContent(nPortletIndex);	
			}
			catch(Exception ex)
			{
				//We can ignore this exception
				objArrMarkup = null;
			}
			
			if (objArrMarkup != null)
			{
				// invoke the "UI transformer" function -jas
				// NOTE: the shared JS needs to be displayed before all portlets
				element.AddInnerHTMLElement(GatewayUtils.AddMarkupToElement(m_asOwner, objArrMarkup));
			}
		}
	}
	
	/**
	 * 
	 * @param iPortletID - ID of the portlet on the Page
	 * @return JavaScript code that refreshes this particular portlet
	 */
	private String GetRefreshJS(int iPortletID)
	{
		return strRefreshJSFunctionBegin + XPConvert.ToString(iPortletID) + strRefreshJSFunctionEnd;
	}
	
	private String GetToggleJS(String strID)
	{
		return strToggleJSFunctionBegin + strID + strToggleJSFunctionEnd;		
	}
}


