package com.plumtree.portalpages.browsing.objectselection;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.objectselection.portlets.*;
import com.plumtree.portalpages.common.folder.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * 
 *
 * @author michaeld
 */
public class ObjectSelectionModel extends EditorModel implements IEditorModelRO,
	IFolderModel, IFolderModelRO, IObjectSelectionModel, IObjectSelectionModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.objectselection.ObjectSelectionModel");

	/** Object Name */
	public static final String STR_MVC_CLASS_NAME = "ObjectSelection";

	/** Dirty dictionary */
	public static final String DIRTY = "Dirty";

	/** HTML Input Hidden Name */
	public static final String HTMLINPUT_HIDDEN_NAME = "in_hi_Name_";

	/** HTML Input Hidden Alignment */
	public static final String HTMLINPUT_HIDDEN_ALIGNMENT = "in_hi_Alignment_";
	
	/** HTML Input Hidden Object ID */
	public static final String HTMLINPUT_HIDDEN_ID = "in_hi_ID_";

	/** Default maximum objects that can be displayed in a flat selection view. */
	public static final int MAX_OBJECTS_FOR_FLAT_VIEW = 50;

	/**
	 * Default number of objects that can be displayed in the search results if
	 * the number of search columns is specified.
	 **/
	public static final int DEFAULT_OBJECTS_FOR_SEARCH_COLUMN_RESULTS = 10;

	/** Column for Selected info item's name */
	public static final int COL_SELECTED_NAME = 0;

	/** Column count for selected object */
	public static final int COL_OBJECT_MAX = 2;

	/** Column for selected object's id */
	public static final int COL_OBJECT_ID = 0;

	/** Column for selected object's info */
	public static final int COL_OBJECT_INFO = 1;

	/** Stores common class used to display one folder */
	public IFolderViewHelper m_folderViewHelper = null;

	/** Selection Model Helper */
	private IObjectSelectionModelHelper m_selectionModelHelper;

	/** Admin Catalog */
	private IPTAdminCatalog m_ptCatalog;

	/** Current Admin Folder */
	private IPTAdminFolder m_ptCurrentFolder;

	/** IPTSession */
	private IPTSession m_ptSession;

	/** Current Folder Name */
	private String m_sCurrentFolderName;

	/** Object properties to remove or add in 508 mode. */
	private String m_str508ObjectID = "";
	private String m_str508ObjectName = "";
	private String m_str508ObjectAlignment = "";

	/** Parent Form Name */
	private String m_strParentFormName = "";

	/** Parent Form Selected Items */
	private String m_strParentFormSelectedItems = "";

	/** Parent Form Space ID */
	private String m_strParentFormSpaceID = "";

	/** Parent Form Submit Function */
	private String m_strParentFormSubmitFunction = "";

	/** Temporary Invitation Name */
	private String m_strTemporaryInvitationName = "";

	/** Form data: Input - Value */
	private XPHashtable m_htFormData;

	/** New Selected Objects */
	private XPSortedDictionary m_xpsdNewSelectedObjects;

	/** Old Selected Objects */
	private XPSortedDictionary m_xpsdOldSelectedObjects;

	/** Temporary Selected Objects: Section removes from this as it shows selected items. */
	private XPSortedDictionary m_xpsdVisibleSelectedObjects;

	/**
	 * Class IDs
	 * NOTE: Portlets inside a portlet bundle are the negative of the portlet class ID.
	 */
	private int[] m_anClassID;

	/** Prop IDs to Query */
	private int[] m_anPropIDsToQuery;

	/** Object Manager */
	private IPTObjectManager[] m_aptObjectManager;
	
	/** Subfolders Query Result */
	private ASQueryResultWrapper m_qrSubFolders;

	/** Objects Query Result */
	private IASQueryResult[] m_aqrObjects;

	/** Strings for headers */
	private String[][] m_astrHeaders;

	/**
	 * Sorted dictionary of objects IDs for objects that should not be shown as
	 * selectable. (Example: mandatory gadgets for a user's mypage.
	 */
	private XPSortedDictionary[] m_xpDoNotShowObjectID;

	/** Display search results */
	private boolean m_bDisplaySearchResults;

	/** Is multiple select? */
	private boolean m_bMultipleSelect = true;

	/** Has model been initialized yet? */
	private boolean m_bInitialized = false;

	/** Search initialized */
	private boolean m_bSearchInitialized;
	
	/** Search Setting Collection */
	protected SearchSettingCollection m_ssc;

	/** Search initialized */
	private boolean m_bShowBanner = true;

	/** Show Checkbox for folders? */
	private boolean m_bShowCheckbox = false;

	/** Bundle ID Selected */
	private int m_nBundleIDSelected;

	/** Current Folder ID */
	private int m_nCurrentFolderID = -1;

	/** First Item Index */
	private int m_nFirstItemIndex = 0;

	/** Last Item Index */
	private int m_nLastItemIndex = 3;

	/** Object Section Index */
	private int m_nObjectViewIndex;

	/** Number of object views */
	private int m_nObjectViewMax;

	/** Folder ID to open */
	private int m_nOpenFolderID;

	/** Selector Class ID */
	private int m_nSelectorClassID = 0;

	/** Selector Community Folder ID */
	private int m_nSelectorCommunityFolderID = 0;
	
	/** Object Sort Property ID, default is to display the last modified objects first  **/
	private int m_nSortPropertyId = -PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED;

	/**
	 * The max number of results columns - used to make sure the number of results
	 * is divisible by the number of columns for search and browse (when there are
	 * no subfolders).
	 */
	private int m_nMaxResultColumns = 1;

	/** Comma separate string of additional search result property ids **/
	private int[] m_searchResultReturnPropIds = new int[0];
	
	/** Selector Object ID */
	private int m_nSelectorObjectID = 0;

	/** Selector Page ID */
	private int m_nSelectorPageID = 0;
	
	/** Selector Community ID */
	private int m_nSelectorCommunityID = 0;
	
	/** Is the home my page */
	private boolean m_bIsHomeMyPage = false;

	/** Temporary Invitation ID */
	private int m_nTemporaryInvitationID = 0;

	protected int m_nFirstDisplayableItem = 1;
	protected int m_nNumberOfItemsPerPage = 10;

	public ObjectSelectionModel()
	{
		super();
		m_folderViewHelper = new ObjSelFolderViewHelper();
	}
	
	/**
	 * Implements some actions to perform on start.
	 *
	 * @param _htFormData	Request data
	 *
	 * @return boolean Is true if the current user has the rights to access this Editor.
	 */
	public boolean CheckEditorAccessAndStart(XPHashtable _htFormData)
	{
		m_bInitialized = false;
		m_bDisplaySearchResults = m_selectionModelHelper.GetStartInSearch();
		
		m_ptSession = (IPTSession) m_asOwner.GetUserSession();
		SetValuesFromQueryString(_htFormData);

		// Setup hash map with names and values of form inputs
		// Currently this is only used by the gadget selection page for bundles, 
		// but it could easily be extended for use by other views.
		m_htFormData = new XPHashtable();
		m_selectionModelHelper.InitializeFormInputs(m_htFormData);

		// Get the total number of object views from the model helper.
		m_nObjectViewMax     	= m_selectionModelHelper.GetNumObjectViews();

		m_anClassID				= new int[m_nObjectViewMax];
		m_astrHeaders			= new String[m_nObjectViewMax][];
		m_aqrObjects			= new IASQueryResult[m_nObjectViewMax];
		m_aptObjectManager		= new IPTObjectManager[m_nObjectViewMax];
		m_anPropIDsToQuery		= new int[m_nObjectViewMax];
		m_xpDoNotShowObjectID   = new XPSortedDictionary[m_nObjectViewMax];

		// Initialize the object view index to 0, this can only be changed 
		// by calling SetObjectViewIndex from the display page.
		m_nObjectViewIndex = 0;

		// Get the admin catalog unless this is displaying the portlets inside a portlet bundle.
		if (m_anClassID[0] != -PT_CLASSIDS.PT_GADGET_ID)
		{
			m_ptCatalog = m_ptSession.GetAdminCatalog();
		}

		for (int i = 0; i < m_nObjectViewMax; i++)
		{
			// Get Class ID        
			m_anClassID[i]     = m_selectionModelHelper.GetObjectType(i);
			// Get headers
			m_astrHeaders[i] = m_selectionModelHelper.GetHeaders(i);
		}
		
		InitializeSearchSettingCollection();
	
		SetupObjectSelection();

		// rob: Moved this to after the queryresult is set in case GetSelectedObjects wants to use 
		// the queryresult so don't have to query twice.
		m_xpsdOldSelectedObjects		 = new XPSortedDictionary();
		m_xpsdNewSelectedObjects		 = new XPSortedDictionary();
		m_xpsdVisibleSelectedObjects     = new XPSortedDictionary();

		RefreshSelectedObjects();

		// Get object IDs not to show
		m_xpDoNotShowObjectID[0] = m_selectionModelHelper.GetDoNotShowObjectIDs(this, 0);
				
		return true;
	}

	/**
	 * Method SetupObjectSelection.
	 */
	private void SetupObjectSelection()
	{
		if (m_bDisplaySearchResults)
		{
			// if we DO start the object selection in a search result, we need to initialize the search model
			ISearchResultModel srm = 
			  (ISearchResultModel) m_asOwner.GetModel(SearchResultModel.STR_MVC_CLASS_NAME);
				
			m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_ADMINFOLDERS, new int[] { PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER });
			
			try
			{
				srm.Search("*", null, null, m_ssc);
			}
			catch (Exception e)
			{
				log.Warn(e, "CheckEditorAccessAndStart: " + e.getMessage());				
				m_bDisplaySearchResults = false;
			}
		}
		
		if (!m_bDisplaySearchResults)
		{
			// if we don't start the object selection in a search result
			if (m_anClassID[0] == -PT_CLASSIDS.PT_GADGET_ID)
			{
				// Portlets from Portlet Bundle
				IPTGadgetBundle ptgb = (IPTGadgetBundle) m_ptSession.GetGadgetBundles()
					.Open(m_nSelectorObjectID, false);
				m_sCurrentFolderName = ptgb.GetLocalizedName();
		
				// If for my page set gadget type filter to personal
				Object[] aoGadgetTypes;
		
				if ((0 == m_nSelectorCommunityFolderID) && (0 != m_nSelectorPageID))
				{
					aoGadgetTypes = new Object[]
						{
							new Integer(PT_GADGET_TYPES.PT_GADGETTYPE_PERSONAL)
						};
				}
				else
				{
					aoGadgetTypes = new Object[0];
				}
		
				ASQueryResultWrapper qrw = new ASQueryResultWrapper(ptgb.QueryGadgets(
							aoGadgetTypes));
				m_aqrObjects[0] = qrw;
			}
			else
			{
				OpenSubFolder(PT_INTRINSICS.PT_ADMIN_FOLDER_ROOTFOLDER);
			}
		}
	}
	
	/**
	 * Method RefreshObjectSelection.
	 */
	public void RefreshObjectSelection()
	{
		m_bDisplaySearchResults = m_selectionModelHelper.GetStartInSearch();

		// Setup hash map with names and values of form inputs
		// Currently this is only used by the gadget selection page for bundles, 
		// but it could easily be extended for use by other views.
		m_htFormData = new XPHashtable();
		m_selectionModelHelper.InitializeFormInputs(m_htFormData);

		SetupObjectSelection();

		RefreshSelectedObjects();
	}
	
	/**
	 * Method RefreshSelectedObjects.
	 * Refreshes the selected objects.
	 */
	public void RefreshSelectedObjects()
	{
		m_xpsdOldSelectedObjects.Clear();
		m_xpsdNewSelectedObjects.Clear();
		m_xpsdVisibleSelectedObjects.Clear();
		
		// Get Selected Objects
		// rob- Should this function be called GetInitialSelectObjects or InitialSelectedObjects instead???
		Object[][] aoSelectedObjects = m_selectionModelHelper.GetSelectedObjects(this);

		if (null != aoSelectedObjects)
		{
			for (int i = 0; i < aoSelectedObjects.length; i++)
			{
				m_xpsdOldSelectedObjects.PutElement(XPConvert.ToString(aoSelectedObjects[i][COL_OBJECT_ID]),
					aoSelectedObjects[i][COL_OBJECT_ID]);
				m_xpsdNewSelectedObjects.PutElement(XPConvert.ToString(aoSelectedObjects[i][COL_OBJECT_ID]),
					aoSelectedObjects[i][COL_OBJECT_INFO]);
			}
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ObjectSelectionModel();
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#DoNotShowObject(int)
	 */
	public boolean DoNotShowObject(int nIndex)
	{
		try
		{
			if ((m_xpDoNotShowObjectID[m_nObjectViewIndex] != null) &&
					(m_xpDoNotShowObjectID[m_nObjectViewIndex].GetSize() != 0) &&
					m_xpDoNotShowObjectID[m_nObjectViewIndex].ContainsKey(m_aqrObjects[m_nObjectViewIndex].GetFields(
						nIndex, PT_PROPIDS.PT_PROPID_OBJECTID)))
			{
				return true;
			}
		}
		catch (Exception e)
		{
			log.Warn(e, "DoNotShowObject: " + e.getMessage());
		}

		return false;
	}
	
	/**
	 * 
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#DoNotShowObjectWithID(int)
	 */
	public boolean DoNotShowObjectWithID(int nObjectID)
	{
		try
		{
			if ((m_xpDoNotShowObjectID[m_nObjectViewIndex] != null) &&
					(m_xpDoNotShowObjectID[m_nObjectViewIndex].GetSize() != 0) &&
					m_xpDoNotShowObjectID[m_nObjectViewIndex].ContainsKey(new Integer(nObjectID)))
			{
				return true;
			}
		}
		catch (Exception e)
		{
			log.Warn(e, "DoNotShowObject: " + e.getMessage());
		}

		return false;
	}

	/**
	 * 
	 * @see com.plumtree.uiinfrastructure.editor.EditorModel#DoTaskOnComeBackFromSubEditor()
	 * 
	 * This method will be called when you come back to the main Editor from a sub Editor.
	 */
	public void DoTaskOnComeBackFromSubEditor()
	{
		OpenSubFolder(m_nOpenFolderID);
	}

	/**
	 * Implement some code to execute when the user clicks on "Finish".
	 */
	public void Finish()
	{
		m_selectionModelHelper.SetSelectedObjects(this,
			m_xpsdOldSelectedObjects, m_xpsdNewSelectedObjects);
	}

	/**
	 * Get the activity space.
	 *
	 * @return AActivitySpace space
	 */
	public AActivitySpace GetActivitySpace()
	{
		return m_asOwner;
	}

	/////////////////////////////////////////////
	// Read only methods from IFolderModelRO
	/////////////////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetCurrentFolderID()
	 */
	public int GetCurrentFolderID()
	{
		return m_nCurrentFolderID;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetCurrentFolderLevel()
	 */
	public int GetCurrentFolderLevel()
	{
		return m_ptCurrentFolder.GetFolderLevel();
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetCurrentFolderName()
	 */
	public String GetCurrentFolderName()
	{
		return m_sCurrentFolderName;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IPaginationModelRO#GetCurrentLowerObjectIndex()
	 */
	public int GetCurrentLowerObjectIndex()
	{
		// This should never be called.
		return -1;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IPaginationModelRO#GetCurrentUpperObjectIndex()
	 */
	public int GetCurrentUpperObjectIndex()
	{
		// This should never be called.
		return -1;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetDisplayPaginationHiddenInputs()
	 */
	public boolean GetDisplayPaginationHiddenInputs()
	{
		// This view does not use pagination
		return false;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetDisplaySearch()
	 */
	public boolean GetDisplaySearch()
	{
		return m_bDisplaySearchResults;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetDisplaySortHiddenInputs()
	 */
	public boolean GetDisplaySortHiddenInputs()
	{
		// This view is not sortable.
		return false;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFirstItemIndex()
	 */
	public int GetFirstFolderIndex()
	{
		return m_nFirstItemIndex;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetNumColumns()
	 */
	public int GetFolderColumnCount()
	{
		return 2;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFolderObjectCount()
	 */
	public IASQueryResult GetFolderObjectCount()
	{
		ASQueryResultWrapper asqr = null;

		try
		{
			asqr = new ASQueryResultWrapper(m_ptCurrentFolder.QueryObjectsCount(false));
		}
		catch (Exception e)
		{
			
				log.Error("GetFolderObjectCount: " + e.getMessage());
			
		}

		return asqr;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFolderQueryResult()
	 */
	public IASQueryResult GetFolderQueryResult()
	{
		return m_qrSubFolders;
	}

	/**
	 * Retrieves common class used to display one folder
	 *
	 * @return 
	 */
	public IFolderViewHelper GetFolderViewHelper()
	{
		return m_folderViewHelper;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetHeaderCount()
	 */
	public int GetHeaderCount()
	{
		if ((null == m_astrHeaders) ||
				(null == m_astrHeaders[m_nObjectViewIndex]))
		{
			return 0;
		}

		return m_astrHeaders[m_nObjectViewIndex].length;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetHeaderHasCheckbox()
	 */
	public boolean GetHeaderHasCheckbox()
	{
		return true;
	}

	/////////////////////////////////////////
	// Read only methods from IObjectModelRO
	/////////////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetHeaderRow()
	 */
	public HTMLTableRow GetHeaderRow()
	{
		return null;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetHeaderText(int)
	 */
	public String GetHeaderText(int nHeaderIndex)
	{
		return m_astrHeaders[m_nObjectViewIndex][nHeaderIndex];
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetInputValue(String)
	 */
	public String GetInputValue(String strInput)
	{
		String value = (String) m_htFormData.GetElement(strInput);

		if (value == null)
		{
			return "";
		}

		return value;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetIsCollapsable()
	 */
	public boolean GetIsCollapsable()
	{
		return false;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetShowCheckbox()
	 */
	public boolean GetIsFolderSelected(int nFolderID)
	{
		return false;
	}

	////////////
	////////////

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetIsObjectSelected(int)
	 */
	public boolean GetIsObjectSelected(int nObjectID)
	{
		boolean bReturn = m_xpsdNewSelectedObjects.ContainsKey(XPConvert.ToString(
					nObjectID));

		if (m_xpsdVisibleSelectedObjects.ContainsKey(DIRTY))
		{
			m_xpsdVisibleSelectedObjects = new XPSortedDictionary();
		}

		if (bReturn)
		{
			m_xpsdVisibleSelectedObjects.PutElement(XPConvert.ToString(nObjectID),
				"");
		}

		return bReturn;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetLastItemIndex()
	 */
	public int GetLastFolderIndex()
	{
		return m_nLastItemIndex;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetMultipleSelect()
	 */
	public boolean GetMultipleSelect()
	{
		return m_bMultipleSelect;
	}

	///////////////////////////////////////
	// Read only methods from IModelRO
	///////////////////////////////////////

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IModelRO#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	///////////////////////////////////////////
	// Read only method from IPaginationModelRO
	///////////////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.common.objects.IPaginationModelRO#GetNumObjects()
	 */
	public int GetNumObjects()
	{
		// This should never be called.
		return -1;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetNumSubSubFolders()
	 */
	public int GetNumSubSubFolders()
	{
		return 0;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetNumSubSubFolders()
	 */
	public int GetNumSubSubFoldersToDisplay()
	{
		return 0;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetObjectBannerTitle()
	 */
	public String GetObjectBannerTitle()
	{
		return m_selectionModelHelper.GetObjectBannerTitle(m_sCurrentFolderName,
			m_nObjectViewIndex);
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetColumnCount()
	 */
	public int GetObjectColumnCount()
	{
		return 1;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetObjectCount()
	 */
	public int GetObjectCount()
	{
		if (null == m_aqrObjects[m_nObjectViewIndex])
		{
			return 0;
		}

		return m_aqrObjects[m_nObjectViewIndex].GetCount();
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetObjectFields(int,
	 *      int)
	 */
	public Object GetObjectFields(int nIndex, int ptPropID)
	{
		return m_aqrObjects[m_nObjectViewIndex].GetFields(nIndex, ptPropID);
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.
	 * IObjectModelRO#GetObjectType()
	 */
	public int GetObjectType()
	{
		return m_anClassID[m_nObjectViewIndex];
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetObjectSubType()
	 */
	public int GetObjectSubType(int nIndex) 
	{
		return -1;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetObjectViewIsCollapsed()
	 */
	public boolean GetObjectViewIsCollapsed()
	{
		return false;
	}

	/**
	 * Get the session
	 *
	 * @return IPTSession ptSession
	 */
	public IPTSession GetPTSession()
	{
		return m_ptSession;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetParentFolderID()
	 */
	public int GetParentFolderID()
	{
		return m_ptCurrentFolder.GetParentFolderID();
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetParentFolderIDs()
	 */
	public Object[] GetParentFolderIDs()
	{
		return m_ptCurrentFolder.GetParentIDs();
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetParentFormName()
	 */
	public String GetParentFormName()
	{
		return m_strParentFormName;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetParentFormSelectedItems()
	 */
	public String GetParentFormSelectedItems()
	{
		return m_strParentFormSelectedItems;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetParentFormSpaceID()
	 */
	public String GetParentFormSpaceID()
	{
		return m_strParentFormSpaceID;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetParentFormSubmitFunction()
	 */
	public String GetParentFormSubmitFunction()
	{
		return m_strParentFormSubmitFunction;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetPath()
	 */
	public String GetPath()
	{
		return m_ptCurrentFolder.GetPath();
	}

	// END ***************** repost action methods ***************** 

	/**
	 * Returns the Obj Manager Redirect object  (this is were we need to go at
	 * the end of the Editor).
	 *
	 * @return Redirect
	 */
	public Redirect GetRedirectAfterFinish()
	{
		// ***CHANGE_HERE*** Replace by the name of the AS you want to 
		// redirect to on finish.
		Redirect redirect = this.m_selectionModelHelper.GetRedirectAfterFinish(this);

		if (null != redirect)
		{
			return redirect;
		}
		return super.GetRedirectAfterFinish();
	}

	/**
	 * Gets the selected objects from this object selector.
	 *
	 * @return XPSortedDictionary
	 */
	public XPSortedDictionary GetRemovedObjectsForParent()
	{
		m_xpsdVisibleSelectedObjects.PutElement(DIRTY, "");

		return m_xpsdVisibleSelectedObjects;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetSearchStarted()
	 */
	public boolean GetSearchStarted()
	{
		return m_bSearchInitialized;
	}

	////////////////////////////////////////////////////
	// Read only methods from IObjectSelectionModelRO
	////////////////////////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetSelectedNamesAsString()
	 */
	public String GetSelectedNamesAsString()
	{
		XPStringBuilder sb = new XPStringBuilder(4 * m_xpsdNewSelectedObjects.GetSize());

		if (0 == m_xpsdNewSelectedObjects.GetSize())
		{
			sb.Append(m_asOwner.GetString(745, "ptmsgs_portalbrowsingmsgs"));
		}
		else
		{
			IXPEnumerator xpenum = m_xpsdNewSelectedObjects.GetKeys();
			int i				 = 0;

			while (xpenum.MoveNext())
			{
				if (0 != i)
				{
					sb.Append(", ");
				}

				sb.Append((String) ((Object[]) m_xpsdNewSelectedObjects.GetElement(
						xpenum.GetCurrent()))[COL_SELECTED_NAME]);

				i++;
			}
		}

		return sb.ToString();
	}

	/**
	 * Gets the selected objects from this object selector.
	 *
	 * @return XPSortedDictionary
	 */
	public XPSortedDictionary GetSelectedObjectsForParent()
	{
		return m_xpsdNewSelectedObjects;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public IObjectSelectionModelHelper GetSelectionModelHelper()
	{
		return m_selectionModelHelper;
	}

	/**
	 * Get the selector's class ID.
	 *
	 * @return int nSelectorClassID
	 */
	public int GetSelectorClassID()
	{
		return m_nSelectorClassID;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetSelectorCommunityFolderID()
	 */
	public int GetSelectorCommunityFolderID()
	{
		return m_nSelectorCommunityFolderID;
	}

	/**
	 * Get the selector's object ID.
	 *
	 * @return int nSelectorObjectID
	 */
	public int GetSelectorObjectID()
	{
		return m_nSelectorObjectID;
	}

	/**
	 * Get the selector's page ID.
	 *
	 * @return int nSelectorPageID
	 */
	public int GetSelectorPageID()
	{
		return m_nSelectorPageID;
	}
	
	/**
	 * Get the selector's community ID.
	 * @return int
	 */
	public int GetSelectorCommunityID()
	{
		return m_nSelectorCommunityID;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetShowCheckbox()
	 */
	public boolean GetShowCheckbox()
	{
		return m_bShowCheckbox;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetShowEditIcon()
	 */
	public boolean GetShowEditIcon()
	{
		return false;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetShowBanner()
	 */
	public boolean GetShowFolderBanner()
	{
		return m_bShowBanner;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetShowPagination()
	 */
	public boolean GetShowPagination()
	{
		// This view does not use pagination.
		return false;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetSortDirection()
	 */
	public int GetSortDirection()
	{
		// This should never be called.
		return -1;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetSortable()
	 */
	public boolean GetSortable()
	{
		// This view is not sortable.
		return false;
	}

	//////////////////////////////////////
	// Read only methods from ISortModelRO
	//////////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#GetSortedHeader()
	 */
	public int GetSortedHeader()
	{
		// This should never be called.
		return -1;
	}

	// END ***************** helper methods ***************** 

	/**
	 * Get the SpaceID of the parent Activity Space
	 *
	 * @return int m_asOwner.GetSpaceID();
	 */
	public String GetSpaceID()
	{
		return m_asOwner.GetSpaceID();
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFolderCount()
	 * Get all subfolders in query and disregard the folder level
	 */
	public int GetSubFolderCount(int nFolderLevel)
	{
		return GetSubFolderCount();
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetFolderCount()
	 * Get the number of folders at all levels in the hierachy
	 */
	public int GetSubFolderCount()
	{
		return m_qrSubFolders.GetCount();
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#GetSubSubFolderInfo(int,
	 *      int)
	 */
	public XPArrayList GetSubSubFolderInfo(int nSubFolderIndex, int nInfoType)
	{
		return null;
	}

	/**
	 * Temporary value, filled in by invitation controls, that gives the ID of
	 * the object to the invitation view.
	 *
	 * @returns strTemporaryInvitationID 
	 */
	public int GetTemporaryInvitationID()
	{
		return m_nTemporaryInvitationID;
	}

	/**
	 * Temporary value, filled in by invitation controls, that gives the name
	 * of the object to the invitation view.
	 *
	 * @returns strTemporaryInvitationName Name to display in invitation popup.
	 */
	public String GetTemporaryInvitationName()
	{
		return m_strTemporaryInvitationName;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModelRO#InRootFolder()
	 */
	public boolean InRootFolder()
	{
		return (m_nCurrentFolderID == 1);
	}

	/////////////////////////////////////////
	// Control methods from IFolderModel
	/////////////////////////////////////////

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModel#OpenSubFolder(int)
	 */
	public void OpenSubFolder(int _nFolderID)
	{
		m_bDisplaySearchResults = false;

		m_ptCurrentFolder		 = GetAdminFolder(_nFolderID);
		m_nCurrentFolderID		 = _nFolderID;
		m_sCurrentFolderName     = m_ptCurrentFolder.GetLocalizedName();

		InitializeSubFolders();

		// Get Objects
		for (int i = 0; i < m_nObjectViewMax; i++)
		{
			InitializeObjects(i, _nFolderID);
		}
	}

	/**
	 * 
	 */
	public void RepostAddObject()
	{
		Object[] aoSelectedInfo;

		if (PT_CLASSIDS.PT_GADGET_ID == m_anClassID[0])
		{
			aoSelectedInfo = new Object[PortletSelectionModelHelper.COL_GADGET_MAX];

			String strAlignment = m_str508ObjectAlignment;

			if (("").equals(strAlignment))
			{
				strAlignment = "0";
			}

			aoSelectedInfo[PortletSelectionModelHelper.COL_GADGET_ALIGNMENT] = new Integer(XPConvert.ToInteger(strAlignment));
		}
		else
		{
			aoSelectedInfo = new Object[1];
		}

		aoSelectedInfo[COL_SELECTED_NAME] = m_str508ObjectName;

		m_xpsdNewSelectedObjects.PutElement(m_str508ObjectID, aoSelectedInfo);

		m_str508ObjectID = "";
		m_str508ObjectName = "";
		m_str508ObjectAlignment = "";
	}

	/**
	 * Method RepostAddPortletsFromBundle.
	 * Add and remove portlets from bundle.
	 */
	public void RepostAddPortletsFromBundle()
	{
		String strCurrentBundleID = (String) m_htFormData.GetElement(BundleSelectionViewHelper.HTMLINPUT_CURRENT_BUNDLE);

		String strSubspaceName = PortletSelectionModelHelper.SUBSPACE_NAME_PORTLETSINBUNDLE +
			strCurrentBundleID;

		if (m_asOwner.GetSubSpaceExists(strSubspaceName))
		{
			PortletInBundleSelectionAS asSelection = (PortletInBundleSelectionAS) m_asOwner.GetSubSpace(strSubspaceName);

			ObjectSelectionModel mySelectionModel = (ObjectSelectionModel) asSelection.GetModel(ObjectSelectionModel.STR_MVC_CLASS_NAME);

			// Remove gadgets from new selected sorted dictionary
			XPSortedDictionary xpsdRemoveGadgets = mySelectionModel.GetRemovedObjectsForParent();
			IXPEnumerator xpeKeys				 = xpsdRemoveGadgets.GetKeys();

			while (xpeKeys.MoveNext())
			{
				if (!(DIRTY).equals(xpeKeys.GetCurrent()))
				{
					m_xpsdNewSelectedObjects.RemoveElement(xpeKeys.GetCurrent());
				}
			}

			// Add gadgets to new selected sorted dictionary
			XPSortedDictionary xpsdAddGadgets = mySelectionModel.GetSelectedObjectsForParent();
			xpeKeys = xpsdAddGadgets.GetKeys();

			while (xpeKeys.MoveNext())
			{
				m_xpsdNewSelectedObjects.PutElement(xpeKeys.GetCurrent(),
					xpsdAddGadgets.GetElement(xpeKeys.GetCurrent()));
			}
		}
	}

	/**
	 * Method RepostOpenFolder.
	 * Opens a folder on repost.
	 */
	public void RepostOpenFolder()
	{
		OpenSubFolder(m_nOpenFolderID);
	}

	/**
	 * 
	 */
	public void RepostRemoveObject()
	{
		m_xpsdNewSelectedObjects.RemoveElement(m_str508ObjectID);

		m_str508ObjectID = "";
		m_str508ObjectName = "";
		m_str508ObjectAlignment = "";
	}

	/**
	 * Method RepostSelectPortletBundle.
	 * Add all portlets from the portlet bundle to the list of selected portlets.
	 */
	public void RepostSelectPortletBundle()
	{
		// If for my page set gadget type filter to personal
		Object[] aoGadgetTypes;

		if ((-1 == m_nSelectorCommunityFolderID) && (0 != m_nSelectorPageID))
		{
			aoGadgetTypes = new Object[]
				{
					new Integer(PT_GADGET_TYPES.PT_GADGETTYPE_PERSONAL)
				};
		}
		else
		{
			aoGadgetTypes = new Object[0];
		}

		IASQueryResult asqr		 = null;
		ASQueryResultWrapper qrw = new ASQueryResultWrapper(((IPTGadgetBundle) m_ptSession.GetGadgetBundles()
				.Open(m_nBundleIDSelected, false)).QueryGadgets(aoGadgetTypes));
		asqr					 = qrw;

		Object[] aoSelectedInfo;
		String strObjectID;

		for (int i = 0; i < asqr.GetCount(); i++)
		{
			if ( PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT <= 
				XPConvert.ToInteger(asqr.GetFields(i, PT_PROPIDS.PT_PROPID_ACCESSLEVEL)) )
			{
				strObjectID							  = XPConvert.ToString(asqr.GetFields(i,
						PT_PROPIDS.PT_PROPID_OBJECTID));
				aoSelectedInfo						  = m_selectionModelHelper.GetSelectedInfo(i,
						asqr);
				aoSelectedInfo[COL_SELECTED_NAME]     = asqr.GetFields(i,
						PT_PROPIDS.PT_PROPID_NAME);
				m_xpsdNewSelectedObjects.PutElement(strObjectID, aoSelectedInfo);
			}
		}

		m_nBundleIDSelected = 0;
	}

	// END ***************** save DP methods ***************** 
	// ***CHANGE_HERE*** Implement the repost action methods that you 
	//      will call from your Repost Control.
	// BEGIN ***************** repost action methods ***************** 

	/**
	 * Modifies the object name on repost.
	 */
	public void RepostToSearch()
	{
	}

	/**
	 * Calls the right save function according to the page name passed in
	 * argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		
		// if the search is not empty but then default to relevance 
		// as the sort order
		String myQuery = GetParameter(_htFormData, "in_tx_query");
		if (null != myQuery && (!myQuery.equals("")))
		{
			String[] mySort = new String[] {""};
		    _htFormData.PutElement("in_se_OrganizationMenu", mySort);
		    String[] myOrderBy = new String[] {"0"};
		    _htFormData.PutElement("in_hi_req_orderby", myOrderBy);
		}
	    
	    if (m_selectionModelHelper.GetShouldSavePage())
		{
			SaveMainPage(_htFormData);
		}

		String strKey;
		String[] data;
		XPHashtable htTempData = new XPHashtable();

		IXPEnumerator xpKeys = m_htFormData.GetKeys();
		
		// Put values of inputs on main page into the form data object.
		// These inputs include the space ids for selecting a community
		// template, adding a new page, and one for each edit page. Also
		// the current edit page index is kept in here.
		while (xpKeys.MoveNext())
		{
			strKey = (String) xpKeys.GetCurrent();

			if (_htFormData.ContainsKey(strKey))
			{
				data = (String[]) _htFormData.GetElement(strKey);

				if ((null != data) && (0 != data.length))
				{
					htTempData.PutElement(strKey, data[0]);
				}
			}
		}
		
		// Now copy it back to the core page model 
		IXPEnumerator xpKeys2 = htTempData.GetKeys();
 		String data2;
		while (xpKeys2.MoveNext())
		{
			strKey = (String) xpKeys2.GetCurrent();
   
			if (_htFormData.ContainsKey(strKey))
			{
 				data2 = (String) htTempData.GetElement(strKey);
  
 				if ((null != data2) && (0 != data2.length()))
				{
 					m_htFormData.PutElement(strKey, data2);
				}
			}
		}

		return RepostControl.PAGE_STATUS_VALID;
	}
	
	/**
	 * Given an XPHashtable of request parameters return the first string
	 * from the String array returned by getting the element whose key is
	 * _sKey.
	 * 
	 * @param _hashTable XPHashtable of request parameters
	 * @param _sKey String key to an element in _hashTable 
	 * 
	 * @return String first element of the String array identified by _sKey
	 */
	private String GetParameter(XPHashtable _hashTable, String _sKey)
	{
	    String[] array = (String[])_hashTable.GetElement(_sKey);
	    if (null != array)
	    {
	        return array[0];
	    }
	    else
	    {
	        return null;
	    }
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModel#SetDisplaySearch(boolean)
	 */
	public void SetDisplaySearch(boolean _bDisplaySearch)
	{
		m_bDisplaySearchResults = _bDisplaySearch;
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModel#SetFirstItemIndex(int)
	 */
	public void SetFirstItemIndex(int nFirstItemIndex)
	{
		m_nFirstItemIndex = nFirstItemIndex;
	}

	/**
	 * 
	 *
	 * @param strHost 
	 */
	public void SetHost(String strHost)
	{
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModel#SetLastItemIndex(int)
	 */
	public void SetLastItemIndex(int nLastItemIndex)
	{
		m_nLastItemIndex = nLastItemIndex;
	}

	/**
	 * 
	 *
	 * @param bNeverFlatView 
	 */
	public void SetNeverFlatView(boolean bNeverFlatView)
	{
	}

	/**
	 * @see com.plumtree.portalpages.common.folder.IFolderModel#SetNumSubSubFolders(int)
	 */
	public void SetNumSubSubFolders(int nSubSubFolders)
	{
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModel#SetObjectViewIndex(int)
	 */
	public void SetObjectViewIndex(int nObjectViewIndex)
	{
		m_nObjectViewIndex = nObjectViewIndex;
	}

	/**
	 * Sets the Selection Model Helper for the Selection Model to use. This
	 * must be called after Init is called.
	 *
	 * @param selectionModelHelper selectionModelHelper
	 */
	public void SetSelectionModelHelper(
		IObjectSelectionModelHelper selectionModelHelper)
	{
		m_selectionModelHelper = selectionModelHelper;
	}

	/**
	 *
	 * @param bShowBanner
	 */
	public void SetShowFolderBanner(boolean bShowBanner)
	{
		m_bShowBanner = bShowBanner;
	}

	/**
	 * Temporary value, filled in by invitation controls, that gives the ID of
	 * the object to the invitation view.
	 *
	 * @param strTemporaryInvitationID 
	 */
	public void SetTemporaryInvitationID(int nTemporaryInvitationID)
	{
		m_nTemporaryInvitationID = nTemporaryInvitationID;
	}

	/**
	 * Temporary value, filled in by invitation controls, that gives the name
	 *  of  the object to the invitation view.
	 *
	 * @param strTemporaryInvitationName Name to display in invitation popup.
	 */
	public void SetTemporaryInvitationName(String strTemporaryInvitationName)
	{
		m_strTemporaryInvitationName = strTemporaryInvitationName;
	}

	/**
	 * @see com.plumtree.portalpages.common.objects.IObjectModelRO#ShowObjectBanner()
	 */
	public boolean ShowObjectBanner()
	{
		return true;
	}

	// BEGIN ***************** save DP methods ***************** 

	/**
	 * Saves the data on the Main page.
	 *
	 * @param _htFormData
	 */
	protected void SaveMainPage(XPHashtable _htFormData)
	{
		// Get open folder id if there is one
		if (_htFormData.ContainsKey(
					ObjSelFolderViewHelper.HTMLINPUT_OPENFOLDERID))
		{
			String[] openFolderIDdata = (String[]) _htFormData.GetElement(ObjSelFolderViewHelper.HTMLINPUT_OPENFOLDERID);

			if ((null != openFolderIDdata) && (0 < openFolderIDdata.length))
			{
				if ((null != openFolderIDdata[0]) &&
						!("").equals(openFolderIDdata[0]))
				{
					m_nOpenFolderID = XPConvert.ToInteger(openFolderIDdata[0]);
				}
			}
		}

		// Get bundle id selected if there is one
		if (_htFormData.ContainsKey(
					BundleSelectionViewHelper.HTMLINPUT_SELECT_BUNDLE))
		{
			String[] bundleIDdata = (String[]) _htFormData.GetElement(BundleSelectionViewHelper.HTMLINPUT_SELECT_BUNDLE);

			if ((null != bundleIDdata) && (0 < bundleIDdata.length))
			{
				if ((null != bundleIDdata[0]) && !("").equals(bundleIDdata[0]))
				{
					m_nBundleIDSelected = XPConvert.ToInteger(bundleIDdata[0]);
				}
			}
		}

		// Saving setting1
		String[] astrObjectInputs = null;

		if (_htFormData.ContainsKey(m_selectionModelHelper.GetObjectInputName(0)))
		{
			astrObjectInputs = (String[]) _htFormData.GetElement(m_selectionModelHelper.GetObjectInputName(
						0));

			if (AccessStyles.STANDARD != m_asOwner.GetAccessStyle() && 0 < astrObjectInputs.length)
			{
				m_str508ObjectID = astrObjectInputs[0];
				
				if (null == m_aptObjectManager[0])
				{
					m_aptObjectManager[0] = m_selectionModelHelper.GetObjectMan(m_ptSession, 0);
				}
				
				// Query Single Object
				IPTQueryResult ptQR = m_aptObjectManager[0].QuerySingleObject(XPConvert.ToInteger(m_str508ObjectID));
				
				// Set name
				m_str508ObjectName = ptQR.ItemAsString(0, PT_PROPIDS.PT_PROPID_NAME);

				if (PT_CLASSIDS.PT_GADGET_ID == m_anClassID[0])
				{
					// Set alignment
					m_str508ObjectAlignment = XPConvert.ToString(PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE);
				}
			}
		}

		if (null == astrObjectInputs)
		{
			astrObjectInputs = new String[0];
		}

		if (AccessStyles.STANDARD == m_asOwner.GetAccessStyle())
		{
			int nIndex;
			String strObjectID;
			String strAlignment;
			String strName;

			if(PT_CLASSIDS.PT_GADGET_ID == m_anClassID[0] || PT_CLASSIDS.PT_COMMUNITY_ID == m_anClassID[0])
			{
				IXPEnumerator doNotShowObjects = m_xpDoNotShowObjectID[m_nObjectViewIndex].GetKeys();
				
				while(doNotShowObjects.MoveNext())
				{
					strObjectID = XPConvert.ToString(doNotShowObjects.GetCurrent());
													
					m_xpsdVisibleSelectedObjects.RemoveElement(strObjectID);
					
				}
			}
			
			for (int i = 0; i < astrObjectInputs.length; i++)
			{
				nIndex = XPConvert.ToInteger(astrObjectInputs[i]);

				strObjectID = ((String[]) _htFormData.GetElement(HTMLINPUT_HIDDEN_ID + nIndex))[0];
				if (PT_CLASSIDS.PT_GADGET_ID == m_anClassID[0])
				{
					strAlignment = ((String[]) _htFormData.GetElement(HTMLINPUT_HIDDEN_ALIGNMENT + nIndex))[0];
				} else {
					strAlignment = "";
				}
				strName = ((String[]) _htFormData.GetElement(HTMLINPUT_HIDDEN_NAME + nIndex))[0];
				
				AddNewlySelected(strObjectID, strAlignment, strName);
			}

			IXPEnumerator xpeKeys = m_xpsdVisibleSelectedObjects.GetKeys();

			while (xpeKeys.MoveNext())
			{
				m_xpsdNewSelectedObjects.RemoveElement(xpeKeys.GetCurrent());
			}
		}

		// If we are not viewing portlets inside a portlet bundle
		if (-PT_CLASSIDS.PT_GADGET_ID != m_anClassID[0])
		{
			m_xpsdVisibleSelectedObjects = new XPSortedDictionary();
		}

		if (_htFormData.ContainsKey(
					ObjSelSearchFormView.HTMLINPUT_VIEWTYPE))
		{
			// Are we on the search results view?
			String[] sValues = (String[]) _htFormData.GetElement(ObjSelSearchFormView.HTMLINPUT_VIEWTYPE);

			if ((sValues != null) && (sValues[0] != null))
			{
				boolean bSearch = m_bDisplaySearchResults;

				try
				{
					bSearch = XPConvert.ToBoolean(sValues[0]);
				}
				catch (Exception e)
				{
					
						log.Error("Unable to convert view type to boolean: " +
							sValues[0]);
					
				}

				if (bSearch != m_bDisplaySearchResults)
				{
					// Need to switch the view and refresh the header
					m_bDisplaySearchResults     = bSearch;

					// If we're switching views (and we started in tree), then
					// search must be initialized
					m_bSearchInitialized = true;
				}
			}
		}
	}

	/**
	 * Adds a newly selected object to the list so that the objects can be added permanently
	 * when the user clicks on finish.
	 *
	 * @param _strObjectID, _strAlignment, _strName
	 */
	public void AddNewlySelected(String _strObjectID, String _strAlignment, String _strName)
	{
		Object[] aoSelectedInfo;
			
		if (m_xpsdVisibleSelectedObjects.ContainsKey(_strObjectID))
		{
			m_xpsdVisibleSelectedObjects.RemoveElement(_strObjectID);
		}
		else
		{
			// If this is a portlet, or a portlet inside a portlet bundle, add the alignment info.
			if ((PT_CLASSIDS.PT_GADGET_ID == m_anClassID[0]) ||
				(-PT_CLASSIDS.PT_GADGET_ID == m_anClassID[0]))
			{
				aoSelectedInfo = new Object[PortletSelectionModelHelper.COL_GADGET_MAX];

				if (("").equals(_strAlignment))
				{
					_strAlignment = "0";
				}

				aoSelectedInfo[PortletSelectionModelHelper.COL_GADGET_ALIGNMENT] = new Integer(XPConvert.ToInteger(_strAlignment));
			}
			else
			{
				aoSelectedInfo = new Object[1];
			}

			aoSelectedInfo[COL_SELECTED_NAME] = XPHttpUtility.HtmlDecode(_strName);

			m_xpsdNewSelectedObjects.PutElement(_strObjectID,
				aoSelectedInfo);
		}
	}
	
	/**
	 * Removes an object from the newselected list so that it will not get added when the user
	 * clicks on finish.
	 *
	 * @param _strObjectID
	 */
	public void RemoveObject(String _strObjectID)
	{
		m_xpsdNewSelectedObjects.RemoveElement(_strObjectID);
	}
	
	// ********** private helper functions

	/**
	 * Method GetQueryFilter.
	 * Gets the query filter for the object type at the given index.
	 * @param _nObjectIndex
	 * @return Object[][]
	 */
	private Object[][] GetQueryFilter(int _nObjectIndex)
	{
		Object[][] ptqf = m_selectionModelHelper.GetQueryFilter(this, _nObjectIndex);
		ptqf[0][0]     = new Integer(PT_PROPIDS.PT_PROPID_ACCESSLEVEL);
		ptqf[1][0]     = new Integer(PT_FILTEROPS.PT_FILTEROP_GTE);
		ptqf[2][0]     = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);
		
		return ptqf;
	}
	
	private IPTAdminFolder GetAdminFolder(int _nFolderID)
	{
		IPTAdminFolder temp = null;

		try
		{
			temp = m_ptCatalog.OpenAdminFolder(_nFolderID, false);
		}
		catch (Exception e)
		{
			
			log.Error(e, "ObjectSelectionModel::GetAdminFolder An exception occured when trying to open the folder with id : " +
				_nFolderID + ". ");
		}
		
		return temp;
	}

	/**
	 * Sets up the initial configuration of the search for portlets or comunities.
	 * Sets the search sort order and grouping by last modified.
	 */
	protected void InitializeSearchSettingCollection()
	{
		m_ssc = new SearchSettingCollection();
		
		// rob: default search property 
		m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY, m_nSortPropertyId);
		
		// rob: we also want to make sure that we are getting the property we are ordering by in the search response
		// it could be searching on a custom object property. 
		m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_RET_PROPS, m_searchResultReturnPropIds);

		if (m_nMaxResultColumns > 1)
		{
			// don: Make sure the number of results is divisible by the number of search columns
			// being displayed.
			int nItemsToDisplay = DEFAULT_OBJECTS_FOR_SEARCH_COLUMN_RESULTS -
				(DEFAULT_OBJECTS_FOR_SEARCH_COLUMN_RESULTS % m_nMaxResultColumns);
			m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_MAXRESULTS, new Integer(nItemsToDisplay));
		}

		m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_APPS, new Integer(PT_SEARCH_APPS.PT_SEARCH_APPS_PORTAL));
		
		m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES, m_anClassID);
		m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_ACCESS_LEVEL,
				new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT));
		if (PT_CLASSIDS.PT_GADGET_ID == m_anClassID[0])
		{
			m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_GADGET_FIELDS, Boolean.TRUE);
	
			// Portlet stuff (templates, wide, etc...)
			int[] anGadgetValues;
	
			if (!IsMyPage())
			{
				anGadgetValues		  = new int[2];
				anGadgetValues[1]     = PT_GADGET_TYPES.PT_GADGETTYPE_COMMUNITY;
			}
			else
			{
				anGadgetValues = new int[1];
			}
	
			anGadgetValues[0] = PT_GADGET_TYPES.PT_GADGETTYPE_PERSONAL;
	
			m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_GADGET_TYPE,
					anGadgetValues);
			m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_GADGET_ALIGNMENT,
					new int[]
					{
						PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NARROW,
						PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE
					});
			m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_GADGET_SEARCH_TYPE, SearchSettingCollection.GADGET_SEARCH_TYPE_NONCOMMUNITYFOLDER);
		}
		m_ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_INCLUDE_SUBFOLDERS, Boolean.TRUE);
		
	}
	
	/**
	 * Method InitializeObjects.
	 * @param _nObjectIndex
	 * @param _nFolderID
	 */
	private void InitializeObjects(int _nObjectIndex, int _nFolderID)
	{
		// Object Query Filter
		Object[][] ptqfObjects = GetQueryFilter(_nObjectIndex);

		if (m_anClassID[_nObjectIndex] <= 0)
		{
			// rob- Get the object in this folder (if any)
			m_aqrObjects[_nObjectIndex] = m_selectionModelHelper.GetObjects(m_ptSession,
					m_ptCurrentFolder);
			m_bInitialized = true;
		}
		else
		{
			// NOTE: This used to only happen if we weren't initialized, but since
			// we do some lazy initialization now, just check if they're null
			// and initialize them if they're empty.

			// if the model hasn't been initialized yet, store the object manager and prop ids to query.
			if (null == m_aptObjectManager[_nObjectIndex])
			{
				m_aptObjectManager[_nObjectIndex]     = m_selectionModelHelper.GetObjectMan(m_ptSession,
				_nObjectIndex);

				m_anPropIDsToQuery[_nObjectIndex]     = m_selectionModelHelper.GetPropIDsToQuery(_nObjectIndex);

				if (m_nObjectViewMax-1 == _nObjectIndex)
				{
					m_bInitialized = true; 
				}
			}

			if (1 != m_nCurrentFolderID && m_aptObjectManager[_nObjectIndex] != null)
			{
				// Query for the objects in this folder.
				PTObjectManagerQueryWrapper omqw = new PTObjectManagerQueryWrapper(m_aptObjectManager[_nObjectIndex]);
				omqw.Query(m_anPropIDsToQuery[_nObjectIndex], m_nCurrentFolderID, null,
					ptqfObjects);
				m_aqrObjects[_nObjectIndex] = omqw;

				// This is currently only used by the bundles in the gadget selection page for
				// their activity space ids.
				m_selectionModelHelper.AddFormInputsForEachObject(m_htFormData,
					m_aqrObjects[_nObjectIndex], _nObjectIndex);
			} else
			{
				// If we go into a subfolder and then come back to the main admin folder,
				// we should reset the queries.
				m_aqrObjects[_nObjectIndex] = null;
			}
		}

		// Reset pagination, if it's being used.
		m_nFirstDisplayableItem = 1;
	}
	
	private void InitializeSubFolders()
	{
		// Get Subfolders
		Object[][] ptqfSubFolders = null;
		if (PT_CLASSIDS.PT_GADGET_ID == m_anClassID[0])
		{
			ptqfSubFolders = new Object[3][1];
			ptqfSubFolders[0][0] = new Integer(PT_PROPIDS.PT_PROPID_FOLDER_FOLDERTYPE);
			ptqfSubFolders[1][0] = new Integer(PT_FILTEROPS.PT_FILTEROP_NE);
			ptqfSubFolders[2][0] = new Integer(PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER);
		}

		try
		{
			m_qrSubFolders = new ASQueryResultWrapper(m_ptCurrentFolder.QuerySubfolders(PT_PROPIDS.PT_PROPID_OBJECTID |
				PT_PROPIDS.PT_PROPID_NAME | PT_PROPIDS.PT_PROPID_FOLDER_FOLDERTYPE | PT_PROPIDS.PT_PROPID_DESCRIPTION, 0,
				new Integer(PT_PROPIDS.PT_PROPID_NAME), 0, -1, ptqfSubFolders));
		}
		catch (Exception e)
		{
			log.Error(e, "InitializeObjects - Get Subfolders: " + e.getMessage());
		}
	}

	/**
	 * Set values of member variable from Query String
	 *
	 * @param XPHashTable _htFormData
	 */
	private void SetValuesFromQueryString(XPHashtable _htFormData)
	{
		// Set selector variables
		if (_htFormData.ContainsKey(
					ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORPAGEID))
		{
			m_nSelectorPageID = XPConvert.ToInteger(((String[]) _htFormData.GetElement(
						ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORPAGEID))[0]);
		}
		
		if (_htFormData.ContainsKey(
					ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORCOMMUNITYID))
		{
			m_nSelectorCommunityID = XPConvert.ToInteger(((String[]) _htFormData.GetElement(
						ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORCOMMUNITYID))[0]);
		}

		if (_htFormData.ContainsKey(
					ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORCLASSID))
		{
			m_nSelectorClassID = XPConvert.ToInteger(((String[]) _htFormData.GetElement(
						ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORCLASSID))[0]);
		}

		if (_htFormData.ContainsKey(
					ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTOROBJECTID))
		{
			m_nSelectorObjectID = XPConvert.ToInteger(((String[]) _htFormData.GetElement(
						ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTOROBJECTID))[0]);
		}

		if (_htFormData.ContainsKey(
					ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORFOLDERID))
		{
			m_nSelectorCommunityFolderID = XPConvert.ToInteger(((String[]) _htFormData.GetElement(
						ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SELECTORFOLDERID))[0]);
		}

		if (_htFormData.ContainsKey(ObjSelQueryArguments.MAX_RESULT_COLUMNS))
		{
			m_nMaxResultColumns = XPConvert.ToInteger(((String[]) _htFormData.GetElement(
				ObjSelQueryArguments.MAX_RESULT_COLUMNS))[0]);
		}

		// order by prop id
		if (_htFormData.ContainsKey(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SORTPROPERTYID))
		{
			m_nSortPropertyId = XPConvert.ToInteger(((String[]) 
					_htFormData.GetElement( ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SORTPROPERTYID))[0]);
		}
		// prop ids to return 
		if ( _htFormData.ContainsKey(ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SEARCH_RETURNPROPIDS) ) {
			String strPropIds = XPConvert.ToString( 
					((String[]) _htFormData.GetElement( ObjSelQueryArguments.HTMLINPUT_OBJECTSELECTION_SEARCH_RETURNPROPIDS))[0] );
			// convert comma separated string to int array 
			XPStringTokenizer list = new XPStringTokenizer(strPropIds, ",", false);
			XPArrayList tempList = new XPArrayList();
			while ( list.HasMoreTokens() ) {
				try { // store the parsed prop ids in a list first, in case there are any bad entries, don't know the final size of array
					tempList.Add(list.GetNextToken());   
				} catch (XPException e) { ; /** skip any entries that are not integers **/ }
			}
			m_searchResultReturnPropIds = new int[tempList.GetSize()];
			for ( int i = 0; i < tempList.GetSize(); i++ ) { // now we know how many good entries we have
				int val = XPConvert.ToInteger(tempList.GetElement(i));
				// store the absolute integer value, prop ids can be negative for descending sort order
				m_searchResultReturnPropIds[i] = (val < 0) ? -val : val;
			}
		}
		
		// Page ID is deprecated, and object ID should be used instead.  However, some
		// code still uses page ID, and some object ID.  Therefore, if one is set, we should
		// set the other to it.
		if ((m_nSelectorPageID == 0) && (m_nSelectorObjectID != 0))
		{
			m_nSelectorPageID = m_nSelectorObjectID;
		} else if ((m_nSelectorObjectID == 0) && (m_nSelectorPageID != 0))
		{
			m_nSelectorObjectID = m_nSelectorPageID;			
		}

		if ( 0 == m_nSelectorCommunityFolderID && PT_CLASSIDS.PT_PAGETEMPLATE_ID != m_nSelectorClassID )
		{
			IPTQueryResult ptqrPage = m_ptSession.GetPages().QuerySingleObject(m_nSelectorObjectID);
			if (null != ptqrPage && 0 < ptqrPage.RowCount())
			{
				m_nSelectorCommunityFolderID = ptqrPage.ItemAsInt(0, PT_PROPIDS.PT_PROPID_FOLDERID);
			}
		}

		if (IsMyPage())
		{
			int nPageID = m_nSelectorObjectID;
			
			if (0!=nPageID)
			{			
				try
				{
					m_bIsHomeMyPage = ((IPTPageManager)m_ptSession.GetPages()).CachedOpenPageInfo(nPageID, true, true).GetIsFirstPageInContainer();
				}
				catch (XPException xpe)
				{
					IPTQueryResult ptqrPage2 = m_ptSession.GetMyPages().QueryPages(0);
					m_bIsHomeMyPage = (0 == ptqrPage2.RowCount());
				}
			}
		}

		// Set Parent Form Name Value
		if (_htFormData.ContainsKey(
					ObjectSelectionRepostControl.strPARENT_FORM_NAME))
		{
			m_strParentFormName = ((String[]) _htFormData.GetElement(ObjectSelectionRepostControl.strPARENT_FORM_NAME))[0];
		}

		// Set Parent Form Space ID Value
		if (_htFormData.ContainsKey(
					ObjectSelectionRepostControl.strPARENT_FORM_SPACE_ID))
		{
			m_strParentFormSpaceID = ((String[]) _htFormData.GetElement(ObjectSelectionRepostControl.strPARENT_FORM_SPACE_ID))[0];
		}

		// Set Parent Form Submit Function Value
		if (_htFormData.ContainsKey(
					ObjectSelectionRepostControl.strPARENT_SUBMIT_CALL))
		{
			m_strParentFormSubmitFunction = ((String[]) _htFormData.GetElement(ObjectSelectionRepostControl.strPARENT_SUBMIT_CALL))[0];
		}
	}
	
	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetSearchSettingCollection()
	 */
	public SearchSettingCollection GetSearchSettingCollection()
	{
		return m_ssc;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetBreadCrumbPath()
	 */
	public IASQueryResult GetBreadCrumbPath()
	{
		return new ASQueryResultWrapper(m_ptCurrentFolder.GetPathAsQueryResult());
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#IsHomeMyPage()
	 */
	public boolean IsHomeMyPage()
	{
		return m_bIsHomeMyPage;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#IsCommunityPage()
	 */
	public boolean IsCommunityPage()
	{
		return (0 < m_nSelectorCommunityFolderID &&
			PT_CLASSIDS.PT_PAGETEMPLATE_ID != m_nSelectorClassID);
	}
	
	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#IsMyPage()
	 */
	public boolean IsMyPage()
	{
		return (0 >= m_nSelectorCommunityFolderID &&
			PT_CLASSIDS.PT_PAGETEMPLATE_ID != m_nSelectorClassID);
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#IsPageTemplate()
	 */
	public boolean IsPageTemplate()
	{
		return (PT_CLASSIDS.PT_PAGETEMPLATE_ID == m_nSelectorClassID);
	}

	public void SetSortPropertyId(int sortPropertyId) 
	{
		log.Debug("Object selection search result sort property id set to: "+sortPropertyId);
		m_nSortPropertyId = sortPropertyId;
	}
	
	public int GetSortPropertyId() 
	{
		return m_nSortPropertyId;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModel#SetFirstDisplayableItem(int)
	 */
	public void SetFirstDisplayableItem(int _nFirstItem)
	{
		m_nFirstDisplayableItem = _nFirstItem;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModel#SetItemsPerPage(int)
	 */
	public void SetItemsPerPage(int _nItemsPerPage)
	{
		m_nNumberOfItemsPerPage = _nItemsPerPage;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetFirstDisplayableItem()
	 */
	public int GetFirstDisplayableItem()
	{
		return m_nFirstDisplayableItem;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetItemsPerPage()
	 */
	public int GetItemsPerPage()
	{
		// This method isn't used for search.

		// Assume that the number is fine if we are using one of the columns to display
		// subfolders (by default we will have 2 columns left, which should be divisible
		// evenly by the default of 10).
		if (GetSubFolderCount() == 0)
		{
			// If we're displaying only portlets and no folders (in browse mode),
			// then we will be using the folder column for results, so make sure
			// that the number of items per page divides evenly by the maximum number of columns.
			return m_nNumberOfItemsPerPage - (m_nNumberOfItemsPerPage % m_nMaxResultColumns);
		}

		return m_nNumberOfItemsPerPage;
	}

	/**
	 * @see com.plumtree.portalpages.browsing.objectselection.IObjectSelectionModelRO#GetTotalItems()
	 */
	public int GetTotalItems()
	{
		int nTotalItems = 0;
		for (int i = 0; i < m_aqrObjects.length; i++)
		{
			// These can be null in the root directory, which has no objects.
			if (m_aqrObjects[i] != null)
			{
				nTotalItems += m_aqrObjects[i].GetCount();
			}
		}

		return nTotalItems;
	}
}
