package com.plumtree.portalpages.browsing.portletpreview;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.gateway.*;
import com.plumtree.portalpages.browsing.myportal.communities.*;
import com.plumtree.portalpages.browsing.objectselection.portlets.*;
import com.plumtree.portaluiinfrastructure.intrinsicportlets.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * File: PreviewPortletView.java
 * ----------------------------
 * Section for Previewing Portlet.  This view displays the
 * preview bar and then calls into the IPTGadget.GetHTMLText().
 * The preview bar is dependent on what type of preview or
 * invitation is desired.  These different cases also require different
 * JavaScript for the various uses.
 *
 * @author NaveenP/DustinA
 */
public class PreviewPortletView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.portletpreview.PreviewPortletView");

	
	public static final String STR_MVC_CLASS_NAME = "PreviewPortlet";

	/** Activity Space */
	private AActivitySpace m_asOwner;

	/** model */
	private IPreviewPortletModelRO m_asModel;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new PreviewPortletView();
	}

	/** Logging */
	
	public static String HTMLINPUT_ADD 			= "in_bu_add";
	public static String HTMLINPUT_CLOSE 			= "in_bu_close";
	public static String HTMLINPUT_DESCRIPTION	= "in_bu_description";
	public static String HTMLSELECT_PAGEMENU		= "in_se_pageMenu";
	

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#display()
	 */
	public HTMLElement Display()
	{
		HTMLTable mainTable = new HTMLTable();
		HTMLTableRow row1   = new HTMLTableRow();
		HTMLTableCell cell1 = new HTMLTableCell();

		mainTable.SetBorder("0");
		mainTable.SetCellPadding("4");
		mainTable.SetCellSpacing("0");
		mainTable.SetWidth("100%");

		try
		{
			cell1.AddInnerHTMLElement(DisplayBar());

			row1.AddInnerHTMLElement(cell1);

			mainTable.AddInnerHTMLElement(row1);
		}
		catch (Exception e)
		{
			log.Error(e, "PreviewPortletView.Display()");

			HTMLTable errorTable    = new HTMLTable();
			HTMLTableRow errorRow   = new HTMLTableRow();
			HTMLTableCell errorCell = new HTMLTableCell();
			HTMLBr errorBr		    = new HTMLBr();

			try
			{
				errorTable.AddInnerHTMLElement(errorRow);
				errorRow.AddInnerHTMLElement(errorCell);

				errorCell.AddInnerHTMLString(m_asOwner.GetString(622,
						"ptmsgs_portalbrowsingmsgs"));
				errorCell.AddInnerHTMLElement(errorBr);
				errorCell.AddInnerHTMLString(e.getMessage());
			}
			catch (HTMLException he)
			{
				log.Error(he, "Error displaying error.");
			}

			return errorTable;
		}

		return mainTable;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#display()
	 */
	public HTMLElement DisplayPortlet()
	{
		HTMLTable mainTable = new HTMLTable();
		HTMLTableRow row   = new HTMLTableRow();
		HTMLTableCell cell = new HTMLTableCell();

		mainTable.SetBorder("0");
		mainTable.SetCellPadding("4");
		mainTable.SetCellSpacing("0");
		mainTable.SetWidth("100%");

		try
		{
			cell.AddInnerHTMLElement(showPreview());

			row.AddInnerHTMLElement(cell);

			mainTable.AddInnerHTMLElement(row);
		}
		catch (Exception e)
		{
			log.Error(e, "PreviewPortletView.Display()");

			HTMLTable errorTable    = new HTMLTable();
			HTMLTableRow errorRow   = new HTMLTableRow();
			HTMLTableCell errorCell = new HTMLTableCell();
			HTMLBr errorBr		    = new HTMLBr();

			try
			{
				errorTable.AddInnerHTMLElement(errorRow);
				errorRow.AddInnerHTMLElement(errorCell);

				errorCell.AddInnerHTMLString(m_asOwner.GetString(622,
						"ptmsgs_portalbrowsingmsgs"));
				errorCell.AddInnerHTMLElement(errorBr);
				errorCell.AddInnerHTMLString(e.getMessage());
			}
			catch (HTMLException he)
			{
				log.Error(he, "Error displaying error.");
			}

			return errorTable;
		}

		return mainTable;
	}


	/**
	 * Display JavaScript
	 *
	 * @return HTMLScript
	 */
	public HTMLScript DisplayJavascript()
	{
		HTMLScript script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);

		if (m_asModel.UserHasSelectAccess())
		{
			if (m_asModel.IsPortletSelectionPreviewPage())
			{
				// We only need to display this javascript for the default case.
				// If we are overriding the javascript method, we don't need this.
				if (m_asModel.GetAddPortletJavaScriptOverride() == null)
				{
					script.SetSrc(ConfigHelper.GetPortalImageServerJavaScriptURL(m_asOwner) +
						"updateParent.js");
				}
			}
			else
			{
				log.Info("No JavaScript for Portlet Preview or Invitation.");
			}
		}
		else
		{
			try
			{
				script.AddInnerHTMLString("alert('" +
					m_asOwner.GetString(615, "ptmsgs_portalbrowsingmsgs") +
					"')");
			}
			catch (HTMLException he)
			{
				log.Error(he, "Error displaying javascript.");
			}
		}

		return script;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#Init(IModel,
	 *      AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		m_asModel     = (IPreviewPortletModelRO) model;
		m_asOwner     = parent;
	}

	/**
	 * Method DisplayBar - returns an HTMLTable containing the preview bar
	 *
	 * @return HTMLTable
	 *
	 * @throws HTMLException
	 */
	private HTMLTable DisplayBar()

	{
		HTMLTable table = new HTMLTable();
 
		// title row
		HTMLTableRow titleRow = new HTMLTableRow();
		table.AddInnerHTMLElement(titleRow);
		HTMLTableCell titleCell = new HTMLTableCell();
		titleRow.AddInnerHTMLElement(titleCell);
		
		titleCell.SetAutoFit(true);
		titleCell.SetAlign(CommonHTMLStrings.LEFT);
		titleCell.SetStyleClass(PTStyleClass.BAN_HEADER);
		titleCell.AddInnerHTMLString(m_asOwner.GetString(1617, "ptmsgs_portalbrowsingmsgs") + CommonHTMLStrings.SPACE + XPHttpUtility.HtmlEncode(m_asModel.GetPortletName()));

		//buttons
		HTMLInput add	    = null;
		HTMLSelect pageMenu = null;
		HTMLInput close;
		HTMLInput Description;

		int nAlignment = m_asModel.GetPortletAlignment();
		if (m_asModel.UserHasSelectAccess() && 
				!(PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_HEADER == nAlignment || 
				PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_FOOTER == nAlignment ||
				PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_FREEFORM == nAlignment) && 
				(m_asModel.IsPortletInvitePage() ||
				 m_asModel.IsPortletSearchPreviewPage()) && m_asModel.UserHasMyPageAccess())
		{
			// for invite or search preview, we show a drop-down of the user's pages.
			// select box for users pages
			pageMenu = new HTMLSelect(HTMLSELECT_PAGEMENU, "pageMenu");
			
			CreatePageMenu(pageMenu);
			
		} else if  (m_asModel.UserHasSelectAccess() && m_asModel.IsPortletSelectionPreviewPage()) {
			// for preview page, we just toss in an add button
			add			  = new HTMLInput(1, HTMLINPUT_ADD,
								"add",
								m_asOwner.GetString(610, "ptmsgs_portalbrowsingmsgs"));

			CreateAddButton(add);					
		}	

		close     = new HTMLInput(1, HTMLINPUT_CLOSE,
				"close",
				m_asOwner.GetString(612, "ptmsgs_portalbrowsingmsgs"));
		String cancelString = "self.close()";
		close.SetValue(m_asOwner.GetString(1333, "ptmsgs_portalbrowsingmsgs"));
		close.SetOnClick(cancelString);
		close.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);

		//Specifying the mainTable attributes
		//class="boxBrowserGray" width="100%" height="40" border="0" cellpadding="4" cellspacing="0" bgcolor="#D6D3CE"
		table.SetStyleClass(PTStyleClass.BAN_HEADER_BG);
		table.SetBorder("0");
		table.SetCellPadding("4");
		table.SetCellSpacing("0");
		table.SetWidth("100%");
		table.SetHeight("40");
		//table.SetBGColor("#D6D3CE");

		HTMLTableRow mainRow    = new HTMLTableRow();
		table.AddInnerHTMLElement(mainRow);
		
		HTMLTableCell cellAlreadyHasPortlet = new HTMLTableCell();
		//HTMLTableCell cellClose= new HTMLTableCell();
		mainRow.SetStyleClass(PTStyleClass.ACTIONBAR_BAN_BG);
		
		boolean bAlreadyHasPortlet = false;
		if (m_asModel.GetPortletAlignment() == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NARROW ||
			m_asModel.GetPortletAlignment() == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE)
		{
			if (m_asModel.IsPortletSelectionPreviewPage() ||
					m_asModel.IsPortletInvitePage() ||
					m_asModel.IsPortletSearchPreviewPage())
			{
				if (!m_asModel.IsPortletChecked())
				{

				}
				else
				{
					cellAlreadyHasPortlet.SetAlign(CommonHTMLStrings.LEFT);
					cellAlreadyHasPortlet.SetStyleClass(PTStyleClass.ACTIONBAR_BAN_TEXT);
					cellAlreadyHasPortlet.AddInnerHTMLString(CommonHTMLStrings.SPACE);
					cellAlreadyHasPortlet.AddInnerHTMLString(m_asOwner.GetString(619,
							"ptmsgs_portalbrowsingmsgs"));
					bAlreadyHasPortlet = true;				
				}
			}


			if (m_asModel.GetPortletAlignment() == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NARROW)
			{ 
				log.Debug("Displaying Preview Narrow Portlet");
			} else {
				log.Debug("Displaying Preview Wide Portlet");	
			}			
		}
		else
		{
 
			log.Error("Error Displaying Preview Portlet: Alignment must be wide or narrow");
		}

		HTMLTableCell cellButtons = new HTMLTableCell();
		mainRow.AddInnerHTMLElement(cellButtons);
		cellButtons.SetAlign(CommonHTMLStrings.LEFT);
		cellButtons.SetAutoFit(true);

		if (!bAlreadyHasPortlet)
		{
			// bjf -- in this case, put add to page in mainRow.
			if (m_asModel.IsPortletInvitePage() ||
				m_asModel.IsPortletSearchPreviewPage())
			{
				cellButtons.AddInnerHTMLElement(pageMenu);
				// the drop-down looks better with two spaces after it...
				cellButtons.AddInnerHTMLString(CommonHTMLStrings.SPACE);
				cellButtons.AddInnerHTMLString(CommonHTMLStrings.SPACE);
			} else if (m_asModel.IsPortletSelectionPreviewPage()) {
				cellButtons.AddInnerHTMLElement(add);
				// just one space after add button.
				cellButtons.AddInnerHTMLString(CommonHTMLStrings.SPACE);
			}
		}

		Description = new HTMLInput(1, HTMLINPUT_DESCRIPTION, "description", m_asOwner.GetString(1331, "ptmsgs_portalbrowsingmsgs"));
		cellButtons.AddInnerHTMLElement(Description);
		String formatLabel;
		String label = m_asOwner.GetString(1332, "ptmsgs_portalbrowsingmsgs");
		if (m_asModel.GetPortletAlignment() == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NARROW) {
			formatLabel= GeneralHelpers.TruncateLongString(label, 23, false);
		} else {
			formatLabel= GeneralHelpers.TruncateLongString(label, 23, false);
		}
		 
		Description.SetValue(formatLabel);
		if (label.length() != formatLabel.length()) {
			Description.SetTitle(label);
		}
		
		String strJSUtilVersionPrefix = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSUTIL);
		Description.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
		ASURL descUrl = new ASURL(m_asOwner);
		descUrl.SetLinkGetCachedSpace(m_asOwner);
		descUrl.SetDisplayPage(PreviewPortletDescPopupDP.STR_MVC_CLASS_NAME);
		Description.SetOnClick( strJSUtilVersionPrefix+"PTWindowUtil.openWindow('" + descUrl.GetURLAsString() + "','PUWEBDIALOG',200,450,false)" );	// mj: removed 'JavaScript:' from function name

		// now add the close input
		cellButtons.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		cellButtons.AddInnerHTMLElement(close);

		// if they already have the portlet, add a row that lets them know this.
		if (bAlreadyHasPortlet)
		{
			//add a new row that will contain AddToPage
			mainRow = new HTMLTableRow();
			table.AddInnerHTMLElement(mainRow);
			mainRow.SetStyleClass(PTStyleClass.ACTIONBAR_BAN_BG);

			// bjf -- add cellAlreadyHasPortlet
			cellAlreadyHasPortlet.SetAutoFit(true);
			mainRow.AddInnerHTMLElement(cellAlreadyHasPortlet);
		}

		return table;
	}

	/**
	 * Creates the appropriate add button and pageMenu if needed
	 *
	 * @param add, pageMenu
	 */
	private void CreateAddButton(HTMLInput add)
	{
		String addString;

		if (m_asModel.GetAddPortletJavaScriptOverride() != null)
		{
			addString = m_asModel.GetAddPortletJavaScriptOverride() +
				"(" + m_asModel.GetPortletID() + ", self)";
		} else
		{
			// set add button for selection preview to use updateParent (which checks the checkbox)
			addString = "updateParent(" + // mj: removed 'JavaScript:' from function name
				m_asModel.GetPortletIndex() + ",'" + EditorDP.HTMLFORM_NAME +
				"." +
				PortletSelectionViewHelper.HTMLINPUT_SELECTION_OBJECT +
				"','self')";
		}
 		
 		String formatLabel;
 		String label = m_asOwner.GetString(611, "ptmsgs_portalbrowsingmsgs");
		if (m_asModel.GetPortletAlignment() == PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NARROW) {
 			formatLabel = GeneralHelpers.TruncateLongString(label, 23, false);
		} else {
			formatLabel = GeneralHelpers.TruncateLongString(label, 23, false);
		}
		add.SetValue(formatLabel);
		if (label.length() != formatLabel.length()) {
			add.SetTitle(label);
		}
		add.SetOnClick(addString);
		add.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
	}

	/**
	 * Creates the appropriate add button and pageMenu if needed
	 * 
	 * @param add, pageMenu
	 */
	private void CreatePageMenu(HTMLSelect pageMenu)
	{
		String addString;
		
		// set add button to call JS that will postToSelf with the AddPortletControl
		addString     = "postToSelf(2);";	// mj: removed 'JavaScript:' from function name
 
			pageMenu.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
			pageMenu.SetOnChange("if (selectedIndex > 0) " + addString);

		// Defult option
		HTMLOption defaultOption = new HTMLOption("");
			pageMenu.AddInnerHTMLElement(defaultOption);
 
		defaultOption.AddInnerHTMLString(m_asOwner.GetString(1618,
				"ptmsgs_portalbrowsingmsgs"));
		defaultOption.SetValue("0");
		defaultOption.SetSelected(true);
 
		IASQueryResult userPages = m_asModel.GetMyPages();
		String strPageName;
		int nPageID;

		for (int i = 0; i < userPages.GetCount(); i++)
		{
			nPageID		    = XPConvert.ToInteger(userPages.GetFields(i,
					PT_PROPIDS.PT_PROPID_OBJECTID));
			strPageName     = (String) userPages.GetFields(i,
					PT_PROPIDS.PT_PROPID_NAME);

			String optionValue = XPConvert.ToString(nPageID);
			HTMLOption option = new HTMLOption(optionValue);
			option.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strPageName));

			pageMenu.AddInnerHTMLElement(option);
		}
	}

	/**
	 * Retrieves HTML for intrinsic portlet.  If the model, view, and control
	 * for the portlet
	 *
	 * @param objArrMarkup 
	 *
	 * @return 
	 */
	private HTMLElement GetIntrinsicPortletContent(Object[][] objArrMarkup)
	{
		String strPortletClass = (String) objArrMarkup[PT_MARKUP_ARRAY_COLS.PT_MARKUP_ARRAY_COLS_FRAGMENT][0];
		int nPortletID		   = m_asModel.GetPortletID();
		String strStorageName  = strPortletClass + nPortletID;

		// We are deliberately breaking the rules for models and views here.  Cast RO interface
		// back to model and call setup method from model. -jic
		PreviewPortletModel myModel = (PreviewPortletModel) m_asModel;
		myModel.SetupIntrinsicPortlet(strPortletClass, strStorageName);

		IntrinsicPortletView iPortletView = (IntrinsicPortletView) m_asOwner.GetView(strStorageName);

		return iPortletView.Display();
	}

	/**
	 * Method showPreview - returns an HTMLTable containing a preview of the
	 * gadget
	 *
	 * @return HTMLTable
	 *
	 * @throws HTMLException
	 */
	private HTMLTable showPreview()

	{
		//gadgetTable components
		HTMLTable gadgetTable	    = new HTMLTable();
		HTMLTableRow gadgetRow	    = new HTMLTableRow();
		HTMLTableCell gadgetRowCell = new HTMLTableCell();

		if (m_asModel.UserHasReadAccess())
		{
			// Check the gadget type
			if (PT_GADGET_TYPES.PT_GADGETTYPE_COMMUNITY != m_asModel.GetPortletType())
			{
				//Specifying the gadgetTable attributes
				//align="center" cellpadding="2" cellspacing="0" border="0" width=?
				gadgetTable.SetBorder("0");
				gadgetTable.SetCellPadding("2");
				gadgetTable.SetCellSpacing("0");
				gadgetTable.SetAlign("center");
				gadgetTable.SetWidth("100%");
				
				// DCA - check that portlet is not disabled
				if (!m_asModel.IsPortletEnabled()) {
				    gadgetRowCell.AddInnerHTMLString(m_asModel.GetPortletDisabledMessage());
				    PreviewCommunityView.addRowCell(gadgetTable, gadgetRow,gadgetRowCell);
				    return gadgetTable;
				}
				
				Object[][] objArrMarkup = null;

				try
				{
					objArrMarkup = m_asModel.GetPortletHTMLText();
				}
				catch (Exception e)
				{
					log.Error(e, "Error Getting Portlet Content");

					//pkd extracted: 					throw new Exception(_ui_ErrorGettingContent);	
					throw new XPException(m_asOwner.GetString(620,
							"ptmsgs_portalbrowsingmsgs"));
				}

				// Handle intrinsic portlets
				if (IntrinsicPortletModel.IsIntrinsicPortlet(objArrMarkup))
				{
					// Get HTML for intrinsic portlet
					try
					{
						gadgetRowCell.AddInnerHTMLElement(GetIntrinsicPortletContent(
								objArrMarkup));
					}
					catch (Exception e)
					{
						log.Error(e, "Error Getting Intrinsic Portlet Content");

						//pkd extracted: 		        		throw new Exception(_ui_ErrorGettingContentIntrinsic);
						throw new XPException(m_asOwner.GetString(621,
								"ptmsgs_portalbrowsingmsgs"));
					}
				}
				else
				{
					// invoke the "UI transformer" function -jas
					gadgetRowCell.AddInnerHTMLElement(GatewayUtils.AddMarkupToSingleElement(m_asOwner, objArrMarkup));
				}

				PreviewCommunityView.addRowCell(gadgetTable, gadgetRow,
					gadgetRowCell);
			}
			else
			{
				// Preview functionaliy not supported for community portlets
				gadgetRowCell.AddInnerHTMLString(m_asOwner.GetString(616,
						"ptmsgs_portalbrowsingmsgs"));
				PreviewCommunityView.addRowCell(gadgetTable, gadgetRow,
					gadgetRowCell);
			}
		}
		else
		{
			gadgetRowCell.AddInnerHTMLString(m_asOwner.GetString(614,
					"ptmsgs_portalbrowsingmsgs"));
			PreviewCommunityView.addRowCell(gadgetTable, gadgetRow,
				gadgetRowCell);
		}

		return gadgetTable;
	}
}

