package com.plumtree.portalpages.browsing.search.results;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.portalsettings.search.*;
import com.plumtree.portalpages.pei.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * @author CraigS
 */
public class SaveSearchControl extends SearchResultControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.search.results.SaveSearchControl");

	
	public static final String STR_MVC_CLASS_NAME = "savesearch";

	
	public static final String NAME_DATA		 = SaveSearchView.HI_TEXTBOX;

	
	public static final int INTRINSIC_PORTLET_ID = 4;

	
	public static final String HI_508_SAVE   = SaveSearch508View.HI_508_SAVE;

	
	public static final String HI_508_CANCEL = SaveSearch508View.HI_508_CANCEL;

	
	public static final int ERRORCODE_NO_NAME_ENTERED = 1;

	
	public static final int ERRORCODE_ALREADY_EXISTS  = 2;

	
	public static final int ERRORCODE_GENERAL_FAILURE = 3;

	private IXPCollection _searchPEIs = null;
	
	private boolean _loadedPEIs = false;

	/**
	 * 
	 *
	 * @return 
	 */
	public Object Create()
	{
		return new SaveSearchControl();
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * 
	 *
	 * @param args 
	 * @param key 
	 *
	 * @return 
	 *
	 * @throws SearchExecutionException 
	 */
	protected Redirect ExecuteSearchControl(XPHashtable args, String key)

	{
		// Check whether we're in 508 mode and "cancel" was pressed
		if (is508())
		{
			if (args.ContainsKey(HI_508_CANCEL))
			{
				// return a redirect to the main results page without saving
				Redirect r = new Redirect();
				r.SetLinkGetCachedSpace(_parent);
				r.SetDisplayPage(SearchResultDP.STR_MVC_CLASS_NAME);

				return r;
			}
		}

		String[] values = (String[]) args.GetElement(NAME_DATA);

		if (null == values)
		{
			
				log.Error("could not find name data in query string.");
			

			return null;
		}

		String name = values[0];

		if (null == name)
		{
			_model.SetSavedSearchErrorCode(ERRORCODE_NO_NAME_ENTERED);

			return null;
		}

		name = name.trim();

		if ((null == name) || (0 == name.trim().length()))
		{
			_model.SetSavedSearchErrorCode(ERRORCODE_NO_NAME_ENTERED);

			return null;
		}

		// Make an URL that will reconstruct the model.  Note: do NOT attach the parent space and ID,
		// as they will not be valid when this is clicked
		XPStringBuilder url = new XPStringBuilder(200);
		url.Append(ASConstants.SPACENAME)
		.Append("=").Append(SearchResultAS.STR_MVC_CLASS_NAME).Append("&")
		.Append(ASConstants.CONTROL).Append("=").Append(NullSearchControl.STR_MVC_CLASS_NAME);

		// Get arguments needed to reconstruct the model.
		// NOTE: Ignore any order-by, page number, or group-by settings,
		// we want the model to be reconstructed as if the search were
		// freshly run.
		XPHashtable reconstructArgs = ((ISearchResultModelRO) _model).GetReconstructionArgs(false);

		IXPEnumerator keys = reconstructArgs.GetKeys();

		while (keys.MoveNext())
		{
			String rKey		 = (String) keys.GetCurrent();
			String[] rValues = (String[]) reconstructArgs.GetElement(rKey);

			for (int j = 0; j < rValues.length; j++)
			{
				url.Append("&").Append(rKey).Append("=").Append(XPHttpUtility.UrlEncode(
						rValues[j], "utf-8"));
			}
		}


		try
		{
			SavedSearchData ssd = new SavedSearchData(name, url.ToString());
			
			DoBeforeSavePEI(ssd, true);
			// Check whether another saved search exists with this name
			Object[][] existing = (Object[][]) ((IPTSession) _parent.GetUserSession()).GetSessionInfo()
				.LookupPreferences((String[])null, INTRINSIC_PORTLET_ID);

			if ((null != existing) && (null != existing[0]))
			{
				for (int j = 0; j < existing[0].length; j++)
				{
					if (null != existing[0][j])
					{
						if (ssd.GetName().equals((String) (existing[0][j])))
						{
							_model.SetSavedSearchErrorCode(ERRORCODE_ALREADY_EXISTS);

							return null;
						}
					}
				}
			}
			// Save the name and URL
			Object[][] setting = new Object[2][1];
			setting[0][0]     = ssd.GetName();
			setting[1][0]     = ssd.GetURLQueryString();

			((IPTSession) _parent.GetUserSession()).GetSessionInfo().AssignPreferences(setting, false, INTRINSIC_PORTLET_ID);
			
			DoBeforeSavePEI(ssd, false);
		}
		catch (Exception e)
		{
			_model.SetSavedSearchErrorCode(ERRORCODE_GENERAL_FAILURE);

			return null;
		}

		// Tell the model that it has been saved
		_model.SetSavedSearchName(name);
		_model.SetSavedSearchErrorCode(0);

		if (is508())
		{
			// In 508 mode, return a redirect to the actual results page
			Redirect r = new Redirect();
			r.SetLinkGetCachedSpace(_parent);
			r.SetDisplayPage(SearchResultDP.STR_MVC_CLASS_NAME);

			return r;
		}
		else
		{
			// In non-508, simply redisplay the page, which will emit a window.close()
			// on the popup and then redisplay the parent
			return null;
		}
	}

	/**
	 * 
	 *
	 * @return 
	 */
	protected boolean is508()
	{
		return (_parent.GetAccessStyle() != AccessStyles.STANDARD);
	}
				
	protected void DoBeforeSavePEI(SavedSearchData ssd, boolean bIsBefore) {
		LoadSearchSettingsPEIs();
		// Call out to each PEI in turn
		if (null != _searchPEIs) {
			IXPEnumerator e = _searchPEIs.GetEnumerator();
			while (e.MoveNext()) {
				ISearchSettingActions pei =
					(ISearchSettingActions) e.GetCurrent();
					
				if (bIsBefore)
				{
					pei.OnBeforeSaveSearch(
						_parent,
						(IPTSession) _parent.GetUserSession(),
						ssd);
				}
				else
				{
					pei.OnAfterSaveSearch(
						_parent,
						(IPTSession) _parent.GetUserSession(),
						ssd);
				}
			}
		}

	}
	/**
	 * This function will pull in all of the available
	 * ISearchSettingActions PEIs.
	 */
	private void LoadSearchSettingsPEIs() 
	{
		// if already loaded
		if (_loadedPEIs) {
			// don't do it again
			return;
		}
	
		_loadedPEIs = true;
	
		IApplication application = _parent.GetApplication();
		Object[] actions =
			(Object[]) application.GetCachingManager().GetEntry(
				"SearchSettingActions");
	
		if (null != actions) {
	
			_searchPEIs = new XPArrayList();
	
			for (int x = 0; x < actions.length; x++) {
				Object oAction = actions[x];
	
				if ((null != oAction)
					&& (oAction instanceof ISearchSettingActions)) {
						_searchPEIs.Add(oAction);
				} else {
					log.Warn("SaveSearchControl.LoadBannerSearchPEIs: element "
						+ XPConvert.ToString(x)
						+ " is not an instance of ISearchSettingActions");
				}
			}
		}
	}

}

