package com.plumtree.portalpages.browsing.search.results;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portalpages.admin.objsearch.*;
import com.plumtree.portalpages.common.uiparts.*;
import com.plumtree.portaluiinfrastructure.layout.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author CraigS
 *
 */
public class SearchResultDP extends PlumtreeDP
{

	public static final String STR_MVC_CLASS_NAME = "SearchResult";
	public static final String HTML_FORM_NAME = "mainform";
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.IDisplayPage#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	protected SearchPageModeHelper _pageMode = null;
	public void SetPageModeHelper(SearchPageModeHelper pageMode)
	{
		_pageMode = pageMode;
	}

	/**
	 * @see com.plumtree.portalpages.common.uiparts.PlumtreeDP#GetSubtitleForBanner()
	 */
	public String GetSubtitleForBanner()
	{
		return "";
	}

	/**
	 * @see com.plumtree.portalpages.common.uiparts.PlumtreeDP#GetTitleForBanner()
	 */
	public String GetTitleForBanner()
	{
		return m_asOwner.GetString(845, "ptmsgs_portalbrowsingmsgs");
	}

	/**
	 * @see com.plumtree.portalpages.common.uiparts.PlumtreeDP#PageDisplay()
	 */
	public HTMLElement PageDisplay()
	{
		boolean is508orLB = Is508OrLB();

		// SM: If we are not in 508 mode or in Low bandwidth mode (i.e. we are in standard mode),
		// try to use the XML page layout to display this page.
		//
		// Also, only use XML based page layout for Browse mode.  For Edit or Admin mode,
		// use the classic layout - this simplifies the amount of work needed for the
		// results tag.
		//
		// If page layout is not enabled, the DisplayLayout method will return null.
		//
		if(is508orLB == false && _pageMode.IsBrowseMode())
		{
			SearchResultModel model = (SearchResultModel) m_asOwner.GetModel(SearchResultModel.STR_MVC_CLASS_NAME);
			SearchExecutionError error = model.GetError();
			// Don't use Page Layout if there was a search error.
			if(error == null)
			{
				HTMLElement element = DisplayLayout(Layouts.SEARCH_RESULTS);
				if (null != element) {
					return element;
				}
			}
		}

		HTMLElementCollection group = new HTMLElementCollection();

		try
		{
			// Enclose all views in a single table cell.
			HTMLTable table = new HTMLTable();
			group.AddInnerHTMLElement(table);

			table.SetWidth("100%");
			table.SetCellSpacing("0");
			table.SetCellPadding("7");

			HTMLTableRow row = new HTMLTableRow();
			table.AddInnerHTMLElement(row);

			HTMLTableCell cell = new HTMLTableCell();
			row.AddInnerHTMLElement(cell);

			if (is508orLB)
			{
				// In 508/LB mode (only):
				// If the search had an error that is preventing any results from being
				// displayed, show a "Go Back" link to let the user (possibly) fix the
				// problem.
				// See bug 35808.  If this works well for 508, we should remove the
				// conditional and use this link in standard UI too.. but it's not long
				// before a release and I don't want to make such a visible change.

				SearchResultModel _model =
					(SearchResultModel) m_asOwner.GetModel(SearchResultModel.STR_MVC_CLASS_NAME);

				if ((null == _model) ||    // no model
				    ((null != _model) && (! _model.IsInitialized())) ||   // model, but never init
				    ((null != _model) && (_model.IsInitialized()) && (null != _model.GetError())))    // init, but with error
				{
					HTMLAnchor goBack = new HTMLAnchor(HTMLConstructHelpers.GetPreviousSpaceURL(m_asOwner));

					goBack.AddInnerHTMLString(m_asOwner.GetString(47, "ptmsgs_portalbrowsingmsgs"));
					goBack.SetTitle(m_asOwner.GetString(47, "ptmsgs_portalbrowsingmsgs"));

					cell.SetStyleClass(PTStyleClass.LIST_TEXT);
					cell.AddInnerHTMLElement(goBack);

					// Make a new row and cell for the rest of the page
					row = new HTMLTableRow();
					table.AddInnerHTMLElement(row);
					cell = new HTMLTableCell();
					row.AddInnerHTMLElement(cell);

				}  // end if model contained an error
			}  // end if 508 version of page

			HTMLElement viewElement;

			// "Search again:"
			viewElement = m_asOwner.GetView(FollowupSearchView.STR_MVC_CLASS_NAME)
				.Display();

			if (null != viewElement)
			{
				cell.AddInnerHTMLElement(viewElement);

				// Not needed.  Browser adds space after a form on its own.
				//cell.AddInnerHTMLElement(makeSpacer("15"));
			}

			// "42 results searching for 'kumquat'"
			// In non-508/LB mode, this is displayed lower on the page (see below).
			if (is508orLB) {
				viewElement = m_asOwner.GetView(SearchSummaryView.STR_MVC_CLASS_NAME).Display();

				if (null != viewElement)
				{
					cell.AddInnerHTMLElement(viewElement);
					cell.AddInnerHTMLElement(makeSpacer("15"));
				}
			}

			HTMLElement containerForGroupedResults = null;

			if (is508orLB) {
				// In 508/LB mode, show the drilldown toolbar next
				viewElement = m_asOwner.GetView(DrilldownView.STR_MVC_CLASS_NAME).Display();
				if (null != viewElement) {
					cell.AddInnerHTMLElement(viewElement);
				}

				containerForGroupedResults = cell;
			} else {
				// Non-508 mode:
				// This is a workaround for quirky browser behavior in which a large vertical
				// space is inserted after each form.  Instead of simply having OrganizationView
				// return a form, we get the form first and then insert everything else inside of
				// it.
				OrganizationView orgView = (OrganizationView) m_asOwner.GetView(OrganizationView.STR_MVC_CLASS_NAME);

				HTMLForm orgForm = new HTMLForm(OrganizationView.REORGANIZATION_FORM_NAME);
				orgForm.SetMethod("get");
				orgForm.SetName(OrganizationView.REORGANIZATION_FORM_NAME);

				orgView.addTargetsToOrganizationForm(orgForm);

				cell.AddInnerHTMLElement(orgForm);

				// Note that in non-508 mode, several of these views are displayed as part of GroupedResultsView
				// "Search Results"
				viewElement = m_asOwner.GetView(SearchBannerView.STR_MVC_CLASS_NAME).Display();
				if (null != viewElement)
				{
					orgForm.AddInnerHTMLElement(viewElement);
				}

				viewElement = m_asOwner.GetView(OrganizationView.STR_MVC_CLASS_NAME).Display();
				if (null != viewElement) {
					orgForm.AddInnerHTMLElement(viewElement);
					orgForm.AddInnerHTMLElement(makeSpacer("1"));
				}

				// "42 results searching for 'kumquat'"
				// In 508/LB mode, this is displayed farther up on the page (see above).
				viewElement = m_asOwner.GetView(SearchSummaryView.STR_MVC_CLASS_NAME).Display();
				if (null != viewElement)
				{
					orgForm.AddInnerHTMLElement(viewElement);
				}

				containerForGroupedResults = orgForm;
			}

			// Non-508 browse mode: rest of page goes within organization form, to avoid extra space (see above comments)
			// 508 browse mode: rest of page stands on its own, not within a form
			// Non-508 non-browse mode: rest of page goes within form used for toolbar; organization form stands alone
			// 508 non-browse mode: same as non-508 non-browse mode

			ISearchResultModelRO model = ((ISearchResultModelRO)m_asOwner.GetModel(SearchResultModel.STR_MVC_CLASS_NAME));
			//If we are in EditMode or AdminObjectMode then we need to have the toolbar in place.
			if(! _pageMode.IsBrowseMode())
			{
				HTMLForm moveForm = new HTMLForm(AdminSearchDP.HTML_FORM_NAME);
				moveForm.SetMethod("post");
				moveForm.SetName(AdminSearchDP.HTML_FORM_NAME);
				cell.AddInnerHTMLElement(moveForm);

				ToolbarView tbv = (ToolbarView)m_asOwner.GetView(ToolbarView.STR_MVC_CLASS_NAME);
				tbv.addTargetsToActionBarForm(moveForm);
	
				tbv.SetFormName(AdminSearchDP.HTML_FORM_NAME);
				moveForm.AddInnerHTMLElement(tbv.Display());
				
				containerForGroupedResults = moveForm;
			}

			viewElement = m_asOwner.GetView(GroupedResultsView.STR_MVC_CLASS_NAME).Display();
			if(null != viewElement)
			{
				containerForGroupedResults.AddInnerHTMLElement(viewElement);
			}

			// Pagination footer
			viewElement = m_asOwner.GetView(SearchFooterView.STR_MVC_CLASS_NAME)
				.Display();

			if (null != viewElement)
			{
				containerForGroupedResults.AddInnerHTMLElement(makeSpacer("1"));
				containerForGroupedResults.AddInnerHTMLElement(viewElement);
			}
		}
		catch (HTMLException he)
		{
			group.AddInnerHTMLElement(new HTMLGenericElement(he.getMessage()));
		}

		return group;
	}

	/**
	 *  Helper method to make a spacer table
	 */
	private HTMLTable makeSpacer(String height)

	{
		HTMLTable table = new HTMLTable();

		HTMLTableRow row = new HTMLTableRow();
		table.AddInnerHTMLElement(row);

		HTMLTableCell cell = new HTMLTableCell();
		row.AddInnerHTMLElement(cell);

		HTMLImg spacer = null;

		try
		{
			spacer = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.SPACER,
					true, m_asOwner).GetHTMLElement();
		}
		catch (Exception e)
		{
			throw XPException.GetInstance(e);
		}

		if (null == spacer)
		{
			throw new HTMLException();
		}

		cell.AddInnerHTMLElement(spacer);
		spacer.SetWidth("1");
		spacer.SetHeight(height);
		spacer.SetAlign("absmiddle");
		spacer.SetBorder("0");
		spacer.SetAlt("");

		return table;
	}

	protected HTMLScriptCollection DisplayJavaScriptFromChild()
	{
		HTMLScriptCollection c = null;
		HTMLScript s = m_asOwner.GetView(GroupedResultsView.STR_MVC_CLASS_NAME)
                  .DisplayJavascript();
        if (null != s)
        {
          c = new HTMLScriptCollection();
          c.AddInnerHTMLElement(s);
        }
        
		boolean notBrowseMode = ! (_pageMode.IsBrowseMode());

        if(notBrowseMode)
        {
        	ToolbarView tbv = (ToolbarView)m_asOwner.GetView(ToolbarView.STR_MVC_CLASS_NAME);
    		tbv.SetFormName(HTML_FORM_NAME);
			s = tbv.DisplayJavascript();
			if(s != null)
			{				
				if(c == null)
					c = new HTMLScriptCollection();
				c.AddInnerHTMLElement(s);
			}
		}      
        return c;
    }
        
	private boolean Is508OrLB()
	{
		return (m_asOwner.GetAccessStyle() != AccessStyles.STANDARD);
	}

}

