/*
 * Created on Mar 25, 2003
 */
package com.plumtree.portalpages.browsing.sitemap.sitemappage;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.common.uiparts.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;

/**
 * Handles fallback case where there's no control available.  This can occur
 * on various fallthrough conditions such as ClearErrorsAndReturn().
 *
 * @author Naveenp
 */
public class DefaultSiteMapPageControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.portalpages.browsing.sitemap.sitemappage.DefaultSiteMapPageControl");

	
	public static final String STR_MVC_CLASS_NAME = "DefaultSiteMapPage";

	/** Space */
	private AActivitySpace m_asOwner;

	/** Model */
	private SiteMapModel m_asModel;

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#CheckActionSecurityAndExecute(com.plumtree.openfoundation.util.XPHashtable)
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		return RedirectToSiteMapPage(arguments);
	}

	/**
	 * Redirects to a community page.
	 *
	 * @param arguments
	 * @return Redirect
	 */
	public Redirect RedirectToSiteMapPage(XPHashtable arguments)
	{
		Redirect redirect = null;
		int nCommunityID = NavigationModel.NO_CURRENT_COMMID;
		int nFolderID = 0, nGroupID = 0;

		nCommunityID = m_asModel.GetCommunityID();
		nFolderID = m_asModel.GetCurrentFolderID();
		nGroupID = m_asModel.GetCurrentGroupID();

		try {
			m_asModel.SetCommunityID(nCommunityID);
			if(m_asOwner.GetCurrentPage().GetName().equals(MembersDP.STR_MVC_CLASS_NAME))
			{
				if(nGroupID != 0) {
					m_asModel.SetGroupID(nGroupID);
				}
			} 
			else
			{
				m_asModel.OpenSubFolder(nFolderID);
				if(m_asModel.IsEditMode()) {
					m_asModel.SetEditMode();
				} else {
					m_asModel.SetBrowseMode();
				}
			}

		} catch(Exception e) {
		
			log.Error(e, "Server Error");

			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
					UIErrorScope.CURRENT_USER, null, e.getMessage(),
					e.getMessage());
			UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
				m_asOwner.GetApplication());
		}

		return redirect;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#Init(com.plumtree.uiinfrastructure.activityspace.IModel, com.plumtree.uiinfrastructure.activityspace.AActivitySpace)
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		m_asModel     = (SiteMapModel) model;
		m_asOwner     = space;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new DefaultSiteMapPageControl();

	}
}

