package com.plumtree.portalpages.common.helpers;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.portaluiinfrastructure.resultwrapper.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author adolfoc
 * 
 * This class was built for use in generating small collections of HTML for editors
 * that were not appropriate for HTMLConstructs (mostly because they are used in 
 * table generation).
 */
public class ViewHelper {
	
	public static final String SPACE			  = "&nbsp;";
	public static final int INPUTSIZE_PREFIXED    = 40;
	public static final int INPUTSIZE_NONPREFIXED = 63;
	public static final int TEXTAREA_COLUMNS = 50;
	public static final int TEXTAREA_ROWS = 4;
	public static final int INPUTMAXLENGTH_SHORT = 255;
	public static final int INPUTMAXLENGTH_LONG = 511;

	/**
	 * Adds a cell with default view styling to myRow.
	 * The cell will span colspan columns.
	 *
	 * @param myRow
	 * @param colspan
	 * @return HTMLTableCell
	 * @throws HTMLException
	 */
	public static HTMLTableCell AddCell(HTMLTableRow myRow, String colspan)

	{
		HTMLTableCell resultCell;
		resultCell = new HTMLTableCell();
		myRow.AddInnerHTMLElement(resultCell);
		resultCell.SetNoWrap(true);
		resultCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
		resultCell.SetVAlign(CommonHTMLStrings.TOP);
		
		if (null == colspan) {
			resultCell.SetAutoFit(true);
		} else 
		{
			resultCell.SetColSpan(colspan);
		}		
		resultCell.SetHeight(CommonHTMLStrings.THREE);

		return resultCell;
	}

	/**
	 * @param myRow 
	 * @param colspan 
	 * @param sDefaultValue 
	 * @param sHTMLID 
	 *
	 * @return 
	 *
	 * @throws HTMLException 
	 */
	public static HTMLTableCell AddEditBoxCell(HTMLTableRow myRow,
		String colspan, String sDefaultValue, String sHTMLID)

	{
		HTMLTableCell myCell = AddCell(myRow, colspan);
		myCell.SetNoWrap(true);

		HTMLSpan mySpan = AddSpanToCell(myCell);
		mySpan.AddInnerHTMLElement(MakeSmallInput(INPUTSIZE_PREFIXED,
				sDefaultValue, sHTMLID, HTMLInputTypes.TEXT));

		return myCell;
	}
	
	public static HTMLTableCell AddEditBoxCell(HTMLTableRow myRow,
			String colspan, String sDefaultValue, String sHTMLID,int width)

		{
			return AddEditBoxCell(myRow, colspan, sDefaultValue, sHTMLID, width, false, null);
		}
	
	public static HTMLTableCell AddEditBoxCell(HTMLTableRow myRow,
			String colspan, String sDefaultValue, String sHTMLID,int width, 
			boolean bIsInputDisabled, String sFormID)

		{
			HTMLTableCell myCell = AddCell(myRow, colspan);
			myCell.SetNoWrap(true);

			HTMLSpan mySpan = AddSpanToCell(myCell);
			mySpan.AddInnerHTMLElement(MakeSmallInput(width,
					sDefaultValue, sHTMLID, HTMLInputTypes.TEXT, 
					bIsInputDisabled, sFormID, INPUTMAXLENGTH_SHORT));

			return myCell;
		}

	/**
	 * Adds an empty cell with default view styling to myRow. The cell will span
	 * colspan columns.
	 *
	 * @param myRow
	 * @param colspan
	 * @return HTMLTableCell
	 * @throws HTMLException
	 */
	public static HTMLTableCell AddEmptyCell(HTMLTableRow myRow, String colspan)

	{
		HTMLTableCell resultCell;
		resultCell = AddCell(myRow, colspan);
		resultCell.AddInnerHTMLString(SPACE + SPACE);

		return resultCell;
	}

	/**
	 * Adds an empty row with default view styling to myTable.
	 * The empty row will span colspan columns.
	 *
	 * @param myTable
	 * @param colspan
	 * @return HTMLTableRow
	 * @throws HTMLException
	 */
	public static HTMLTableRow AddEmptyRow(HTMLTable myTable, String colspan)

	{
		HTMLTableRow resultRow;
		resultRow = AddRow(myTable);
		resultRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
		
		AddEmptyCell(resultRow, colspan);

		return resultRow;
	}

	/**
	 * Adds a Cell with a single object icon in it.
	 *
	 * @param myRow
	 * @param colspan
	 * @param nCLASSID
	 * @param strProviderCLSID
	 * @return HTMLTableCell
	 * @throws HTMLException
	 */
	public static HTMLTableCell AddIconCell(HTMLTableRow myRow, String colspan,
		int nCLASSID, String strProviderCLSID, String _strLangID)

	{
		HTMLTableCell resultCell = AddCell(myRow, colspan);
		resultCell.SetVAlign(CommonHTMLStrings.CENTER);
		resultCell.SetWidth("20");

		HTMLSpan mySpan = AddSpanToCell(resultCell);
		
		if (0 < nCLASSID) {
			// Add Class description
			ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();
	
			// AGC- The english name is used here only as a reference to get icons with. It should not be printed.
			String strProvEnglishName = ptClassTypeDescManager.GetProviderEnglishNameGivenCLSID(strProviderCLSID);
			HTMLImg myImage			  = ptClassTypeDescManager.GetDefaultIconForClassType(nCLASSID,
					_strLangID, strProvEnglishName);
	
			// Icon
			mySpan.AddInnerHTMLElement(myImage);
		} else {
			mySpan.AddInnerHTMLString(SPACE);
		}

		return resultCell;
	}

	/**
	 * Adds a Cell with a single Type string (Web Service, etc.) in it.
	 *
	 * @param myRow
	 * @param colspan
	 * @param nCLASSID
	 * @param strProviderCLSID
	 * @return HTMLTableCell
	 * @throws HTMLException
	 */
	public static HTMLTableCell AddTypeCell(HTMLTableRow myRow, String colspan,
		int nCLASSID, String strProviderCLSID, String _strLangID)

	{
		HTMLTableCell resultCell = AddCell(myRow, colspan);
		

		HTMLSpan mySpan = AddSpanToCell(resultCell);

		// Add Class description
		ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();

		// AGC- The english name is used here only as a reference to get icons with. It should not be printed.
		String strProvEnglishName = "";
		String strType = "";
		try {
			strProvEnglishName  = ptClassTypeDescManager.GetProviderEnglishNameGivenCLSID(strProviderCLSID);
			strType			    = ptClassTypeDescManager.GetClassTypeLocalizedName(nCLASSID, _strLangID, strProvEnglishName);
		} catch (Exception e)
		{
			// AGC- it is very possible that this item either doesn't have a provEnglishName or doesn't have a
			// provider-specific name (like portlets).
			strType			  = ptClassTypeDescManager.GetClassTypeLocalizedName(nCLASSID, _strLangID);
		}

		// Icon
		mySpan.AddInnerHTMLEncodedString(strType);

		return resultCell;
	}


	/**
	 * @param myRow 
	 * @param colspan 
	 *
	 * @return 
	 *
	 * @throws HTMLException 
	 */
	public static HTMLTableCell AddIndentCell(HTMLTableRow myRow, String colspan)

	{
		HTMLTableCell resultCell = AddEmptyCell(myRow, colspan);
		resultCell.SetWidth("30");

		return resultCell;
	}

	/**
	 * Returns a Cell suitable for long text descriptions. (no wrapping,
	 * bolded, etc)
	 *
	 * @param myRow
	 * @param colspan
	 * @param strText
	 * @return HTMLTableCell
	 * @throws HTMLException
	 */
	public static HTMLTableCell AddLabelCell(HTMLTableRow myRow,
		String colspan, String strText)

	{
		HTMLTableCell myCell = AddCell(myRow, colspan);
		myCell.SetNoWrap(true);

		HTMLSpan mySpan = AddSpanToCell(myCell);
		myCell.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
		mySpan.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
		
		mySpan.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strText) + SPACE);
		return myCell;
	}

	/**
	 * Adds a cell to a row containing a bolded label followed on a separate
	 * line by text. The cell will wrap.
	 * 
	 * @param myRow
	 * @param colspan
	 * @param strLabel
	 * @param strText
	 * @throws HTMLException
	 */
	public static HTMLTableCell AddLabelAndTextCell(HTMLTableRow myRow, String colspan, String strLabel, String strText)

	{

		HTMLTableCell myCell = AddLabelCell(myRow, colspan, strLabel);
		myCell.SetNoWrap(false);
		
		if ((null != strText) && (strText.length() > 0))
		{
			HTMLSpan mySpan = AddSpanToCell(myCell);
			mySpan.AddInnerHTMLElement(new HTMLBr());
			mySpan.AddInnerHTMLEncodedString(strText);
		}
		
		return myCell;
	}


	/**
	 * Returns a Cell suitable for long text descriptions. (no wrapping,
	 *  etc).  The only difference from AddLabelCell is that the label is not
	 * bolded.
	 *
	 * @param myRow
	 * @param colspan
	 * @param strText
	 * @return HTMLTableCell
	 * @throws HTMLException
	 */
	public static HTMLTableCell AddSubLabelCell(HTMLTableRow myRow,
		String colspan, String strText)

	{
		HTMLTableCell myCell = AddCell(myRow, colspan);
		myCell.SetNoWrap(true);

		HTMLSpan mySpan = AddSpanToCell(myCell);
		mySpan.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strText) + SPACE);		

		return myCell;
	}

	/**
	 * Adds a simple cell containing an image - centered with no border.
	 * @param myRow
	 * @param colspan
	 * @param asOwner
	 * @param imgType
	 * @return HTMLTableCell
	 * @throws HTMLException
	 */
	public static HTMLTableCell AddImageCell(HTMLTableRow myRow,
		String colspan, AActivitySpace asOwner, PTImageType imgType)

	{
		HTMLTableCell myCell = AddCell(myRow, colspan);
		myCell.SetAlign(CommonHTMLStrings.MIDDLE);
		
		HTMLImg htmlImg = null;

		htmlImg = (HTMLImg) UIConstructFactory.GetConstruct(imgType, true,
				asOwner).GetHTMLElement();

		htmlImg.SetAlign(CommonHTMLStrings.ABSMIDDLE);
		htmlImg.SetBorder(CommonHTMLStrings.ZERO);

		myCell.AddInnerHTMLElement(htmlImg);
		return myCell;		
	}

	/**
	 * Uses nColCount to determine if there is enough space for margins and a
	 * spacer cell. If so, adds a left margin and returns the count of middle
	 * columns (non-margin cells) left for adding inputs.
	 *
	 * @param myRow
	 * @param nColCount
	 * @return int
	 * @throws HTMLException
	 */
	public static int AddLeftMargin(HTMLTableRow myRow, int nColCount)

	{
		int nMiddleColumns = nColCount;

		if (nColCount >= 3)
		{
			// Add A Margin
			AddEmptyCell(myRow, CommonHTMLStrings.ONE);

			// Return Column count minus the margins.
			nMiddleColumns = nColCount - 2;
		}

		if (nColCount >= 5)
		{
			// Return Column count minus space for a spacer cell.
			nMiddleColumns = nMiddleColumns - 1;
		}

		return nMiddleColumns;
	}

	/**
	 * @param myRow 
	 * @param colspan 
	 * @param sDefaultValue 
	 * @param sHTMLID 
	 *
	 * @return 
	 *
	 * @throws HTMLException 
	 */
	public static HTMLTableCell AddPasswordBoxCell(HTMLTableRow myRow,
		String colspan, String sDefaultValue, String sHTMLID)

	{
		HTMLTableCell myCell = AddCell(myRow, colspan);
		myCell.SetNoWrap(true);

		HTMLSpan mySpan = AddSpanToCell(myCell);
		mySpan.AddInnerHTMLElement(MakeSmallInput(INPUTSIZE_PREFIXED,
				sDefaultValue, sHTMLID, HTMLInputTypes.PASSWORD));

		return myCell;
	}
	
	public static HTMLTableCell AddPasswordBoxCell(HTMLTableRow myRow,
			String colspan, String sDefaultValue, String sHTMLID,int width)

		{
			HTMLTableCell myCell = AddCell(myRow, colspan);
			myCell.SetNoWrap(true);

			HTMLSpan mySpan = AddSpanToCell(myCell);
			mySpan.AddInnerHTMLElement(MakeSmallInput(width,
					sDefaultValue, sHTMLID, HTMLInputTypes.PASSWORD));

			return myCell;
		}

	/**
	 * Uses nColCount to determine if there is enough space for margins and a
	 * spacer cell. If so, adds a right margin and possibly, right spacer.
	 * Assumes that the results of AddLeftMargin were respected with regards to
	 * remaining column count.
	 *
	 * @param myRow
	 * @param nColCount
	 * @throws HTMLException
	 */
	public static void AddRightMargin(HTMLTableRow myRow, int nColCount)

	{
		if (nColCount >= 5)
		{
			// Add A Spacer Cell
			AddSpacerCell(myRow, CommonHTMLStrings.ONE);
		}

		if (nColCount >= 3)
		{
			// Add A Margin
			AddEmptyCell(myRow, CommonHTMLStrings.ONE);
		}
	}

	/**
	 * Adds a row with default view styling to myTable.
	 *
	 * @param myTable
	 * @return HTMLTableRow
	 * @throws HTMLException
	 */
	public static HTMLTableRow AddRow(HTMLTable myTable)

	{
		HTMLTableRow resultRow;
		resultRow = new HTMLTableRow();
		myTable.AddInnerHTMLElement(resultRow);

		return resultRow;
	}

	/**
	 * Adds a label + PTDurationCombo combination to myTable (spanning a single
	 * row).
	 * nColCount is the column count of the table, and ptdDefault contains the
	 * default information used to render the PTDurationCombo.
	 *
	 * @param myTable
	 * @param nColCount
	 * @param strLabel
	 * @param strHTMLTimeoutInput
	 * @param strHTMLTimeoutUnitsSelect
	 * @param ptdIn
	 * @throws Exception
	 */
	public static void AddSimpleDurationCombo(HTMLTable myTable, int nColCount,
		String strLabel, String strHTMLTimeoutInput,
		String strHTMLTimeoutUnitsSelect, PTDuration ptdIn, AActivitySpace asOwner)

	{
		HTMLTableRow myRow;
		HTMLTableCell myCell;

		myRow = ViewHelper.AddRow(myTable);

		int nMiddleColumns = AddLeftMargin(myRow, nColCount);

		// Label		
		myCell     = AddLabelCell(myRow, CommonHTMLStrings.ONE, strLabel);		

		// Duration Combo
		if (nMiddleColumns > 1)
		{
			// Put the label and duration combo in different cells.
			myCell = ViewHelper.AddCell(myRow,
					XPConvert.ToString(nMiddleColumns - 1));
			myCell.SetAlign(CommonHTMLStrings.LEFT);
		}
		

		////////// Timeout duration construct.		
		myCell.AddInnerHTMLElement(PTDurationCombo.GetDurationCombo(
				strHTMLTimeoutInput, strHTMLTimeoutUnitsSelect,
				ptdIn.GetSupportedUnitsMask(),
				XPConvert.ToString(ptdIn.GetDurationInBestUnits()),
				ptdIn.GetBestDurationUnit(), asOwner));

		AddRightMargin(myRow, nColCount);
	}

	/**
	 * Adds a label + edit box combination spanning two rows to myTable.
	 * nColCount is the number of columns in the table (margins will be added if
	 * nColCount >= 3). strPrefix is any text that can be prepended to the input
	 * box (like the base URL for an gateway input box). sDefaultValue is the
	 * value in the input box. sHTMLID is the name of the input box form
	 * element.
	 *
	 * @param myTable
	 * @param nColCount
	 * @param strLabel
	 * @param strPrefix
	 * @param sDefaultValue
	 * @param sHTMLID
	 * @throws HTMLException
	 */
	public static void AddSimpleEditBox(HTMLTable myTable, int nColCount,
		String strLabel, String strPrefix, String sDefaultValue, String sHTMLID)

	{
		HTMLTableRow myRow;
		HTMLTableCell myCell = null;

		// Label Row
		if ((null != strLabel) && (strLabel.trim().length() > 0))
		{
			AddSimpleLabel(myTable, nColCount, strLabel);
		}

		myRow = AddRow(myTable);

		int nMiddleColumns = AddLeftMargin(myRow, nColCount);

		int nInputSize = INPUTSIZE_PREFIXED;

		if ((null != strPrefix) && (strPrefix.trim().length() > 0))
		{
			myCell = AddSubLabelCell(myRow, XPConvert.ToString(nMiddleColumns),
					strPrefix);
		}
		else
		{
			nInputSize     = INPUTSIZE_NONPREFIXED;
			myCell		   = AddCell(myRow, XPConvert.ToString(nMiddleColumns));
		}

		myCell.AddInnerHTMLElement(MakeSmallInput(nInputSize, sDefaultValue,
				sHTMLID, HTMLInputTypes.TEXT, false, null, INPUTMAXLENGTH_LONG));

		AddRightMargin(myRow, nColCount);
	}

	/**
	 * Adds a simple Label - full width of the table (adding margins)
	 *
	 * @param myTable
	 * @param nColCount
	 * @param strLabel
	 * @throws HTMLException
	 */
	public static void AddSimpleLabel(HTMLTable myTable, int nColCount,
		String strLabel)

	{
		HTMLTableRow myRow = AddRow(myTable);

		int nMiddleColumns = AddLeftMargin(myRow, nColCount);

		AddLabelCell(myRow, XPConvert.ToString(nMiddleColumns), strLabel);

		AddRightMargin(myRow, nColCount);
	}
	
	/**
	 * Adds a row with a simple label: value listing. Respects the number of columns required as passed in nColCount.
	 * nColCount must be 2 or greater.
	 */
	public static void AddSimpleNameValue(HTMLTable myTable, int nColCount,
		String strLabel, String sDefaultValue, String strLang)

	{
		HTMLTableRow myRow = AddRow(myTable);
		
		int nMiddleColumns = AddLeftMargin(myRow, nColCount);
		
		AddLabelCell(myRow, CommonHTMLStrings.ONE, strLabel);
		
		int nRemainingColumns = nMiddleColumns - 1;
		if (nRemainingColumns <= 0)
		{
			nRemainingColumns = 1;
		}
		
		AddTextCell(myRow, XPConvert.ToString(nRemainingColumns), sDefaultValue);
		AddRightMargin(myRow, nColCount);
	}

	/**
	 * Adds a row to myTable containing a simple radio selection.
	 *
	 * @param myTable 
	 * @param nColCount 
	 * @param strLabel 
	 * @param strHTMLID 
	 * @param strValue 
	 * @param bChecked
	 * @param strOnChange
	 *
	 * @throws HTMLException 
	 */
	public static HTMLTableRow AddSimpleRadio(HTMLTable myTable, int nColCount,
		String strLabel, String strHTMLID, String strValue, boolean bChecked,
		String strOnChange)

	{
		return AddSimpleRadioCheckbox(myTable, nColCount, strLabel, strHTMLID, strValue, bChecked, strOnChange, true);
	}

	/**
	 * Adds a row to myTable containing a simple checkbox selection.
	 *
	 * @param myTable 
	 * @param nColCount 
	 * @param strLabel 
	 * @param strHTMLID 
	 * @param strValue 
	 * @param bChecked
	 * @param strOnChange
	 *
	 * @throws HTMLException 
	 */
	public static HTMLTableRow AddSimpleCheckbox(HTMLTable myTable, int nColCount,
		String strLabel, String strHTMLID, String strValue, boolean bChecked,
		String strOnChange)

	{
		return AddSimpleRadioCheckbox(myTable, nColCount, strLabel, strHTMLID, strValue, bChecked, strOnChange, false);
	}


	/**
	 * Adds a row to myTable containing a simple radio (or checkbox) selection.
	 *
	 * @param myTable 
	 * @param nColCount 
	 * @param strLabel 
	 * @param strHTMLID 
	 * @param strValue 
	 * @param bChecked
	 * @param strOnChange
	 *
	 * @throws HTMLException 
	 */
	private static HTMLTableRow AddSimpleRadioCheckbox(HTMLTable myTable, int nColCount,
		String strLabel, String strHTMLID, String strValue, boolean bChecked,
		String strOnChange, boolean bRadioCheckbox)

	{
		boolean disableIndentCell=false;
		if(nColCount<0) { // olegs - hack to allow proper positioning
			nColCount=-nColCount;
			disableIndentCell=true;
		}
		
		HTMLTableRow myRow = AddRow(myTable);

		int nMiddleColumns = AddLeftMargin(myRow, nColCount);

		if (nMiddleColumns > 2 && !disableIndentCell)
		{
			// Add an indent.
			AddIndentCell(myRow, CommonHTMLStrings.ONE);
			nMiddleColumns -= 1;
		}

		HTMLTableCell myCell = AddCell(myRow, XPConvert.ToString(nMiddleColumns));
		HTMLSpan mySpan		 = AddSpanToCell(myCell);

		mySpan.AddInnerHTMLElement(MakeSimpleRadioCheck(strLabel, strHTMLID, strValue, bChecked, strOnChange, bRadioCheckbox));

		AddRightMargin(myRow, nColCount);
		return myRow;
	}

	public static HTMLElement MakeHiddenInput(String strHTMLID, String strvalue)
	{
		HTMLInput myInput = new HTMLInput(HTMLInputTypes.HIDDEN, strHTMLID, "");
		myInput.SetValue(strvalue);
		return myInput;
	}
	/**
	 * Creates a simple radio selection and returns it.
	 * @param strLabel
	 * @param strHTMLID
	 * @param strValue
	 * @param bChecked
	 * @param strOnChange
	 * @return HTMLElement
	 * @throws HTMLException
	 */
	public static HTMLElement MakeSimpleRadio(String strLabel, String strHTMLID, String strValue, boolean bChecked, String strOnChange) 
	{
		return MakeSimpleRadioCheck(strLabel, strHTMLID, strValue, bChecked, strOnChange, true);
	}

	/**
	 * Creates a simple checkbox and returns it.
	 * @param strLabel
	 * @param strHTMLID
	 * @param strValue
	 * @param bChecked
	 * @param strOnChange
	 * @return HTMLElement
	 * @throws HTMLException
	 */
	public static HTMLElement MakeSimpleCheckbox(String strLabel, String strHTMLID, String strValue, boolean bChecked, String strOnChange)
	{
		return MakeSimpleRadioCheck(strLabel, strHTMLID, strValue, bChecked, strOnChange, false);
	}
	
	/**
	 * Creates a simple radio (or checkbox) selection and returns it.
	 * @param strLabel
	 * @param strHTMLID
	 * @param strValue
	 * @param bChecked
	 * @param strOnChange
	 * @return HTMLElement
	 * @throws HTMLException
	 */
	private static HTMLElement MakeSimpleRadioCheck(String strLabel, String strHTMLID, String strValue, boolean bChecked, String strOnChange, boolean bRadioCheckbox) 
	{
		HTMLElementCollection result = new HTMLElementCollection();
		
		HTMLInput myInput;
		if (bRadioCheckbox)
		{	
			myInput = new HTMLInput(HTMLInputTypes.RADIO, strHTMLID, "");
		} else		
		{
			myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, strHTMLID, "");
		}
		result.AddInnerHTMLElement(myInput);
		
		myInput.SetStyleClass(PTStyleClass.FORM_RADIO);
		myInput.SetValue(strValue);
		myInput.SetChecked(bChecked);

		if ((null != strOnChange) && (strOnChange.length() > 0))
		{
			myInput.SetOnClick(strOnChange);
		}
		
		result.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strLabel));
		
		return result;
	}



	/**
	 * Adds a row to myTable containing a simple radio selection.
	 *
	 * @param myTable 
	 * @param nColCount 
	 * @param strLabel 
	 * @param strHTMLID 
	 * @param strValue 
	 * @param bChecked 
	 *
	 * @throws HTMLException 
	 */
	public static HTMLTableRow AddSimpleRadio(HTMLTable myTable, int nColCount,
		String strLabel, String strHTMLID, String strValue, boolean bChecked)
		
	{
		return AddSimpleRadio(myTable, nColCount, strLabel, strHTMLID, strValue, bChecked, "");
	}

	/**
	 * Adds a simple header (no sorting or whatnot) to an existing table.
	 *
	 * @param myTable
	 * @param arsHeaders
	 * @throws Exception
	 */
	public static void AddSimpleTableHeader(HTMLTable myTable,
		String[] arsHeaders)
		throws Exception
	{
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLSpan mySpan;
		HTMLB myB;

		myRow = ViewHelper.AddRow(myTable);
		myRow.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);

		for (int nHeaderIndex = 0; nHeaderIndex < arsHeaders.length;
				nHeaderIndex++)
		{
			myCell = ViewHelper.AddCell(myRow, CommonHTMLStrings.ONE);
			myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
			mySpan = ViewHelper.AddSpanToCell(myCell);
			myB = new HTMLB();
			mySpan.AddInnerHTMLElement(myB);
			myB.AddInnerHTMLString(XPHttpUtility.HtmlEncode(
					arsHeaders[nHeaderIndex]) + SPACE);

			// AGC- Make the last cell width some large percentage so the table cells
			// are left justified.
			if (nHeaderIndex == (arsHeaders.length - 1))
			{
				myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
			}
		}
	}

	/**
	 * Adds a cell with default view styling to myRow. The cell will span
	 * colspan columns. The cell has a width of 100% (can be used to force other
	 * cells in the row towards the left margin).
	 *
	 * @param myRow
	 * @param colspan
	 * @return HTMLTableCell
	 * @throws HTMLException
	 */
	public static HTMLTableCell AddSpacerCell(HTMLTableRow myRow, String colspan)
		
	{
		HTMLTableCell resultCell = AddEmptyCell(myRow, colspan);
		resultCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);

		return resultCell;
	}

	/**
	 * Adds a span with default view styling to myCell.
	 *
	 * @param myCell
	 * @return HTMLSpan
	 * @throws HTMLException
	 */
	public static HTMLSpan AddSpanToCell(HTMLTableCell myCell)
		
	{
		HTMLSpan resultSpan;
		resultSpan = new HTMLSpan();
		myCell.AddInnerHTMLElement(resultSpan);
		resultSpan.SetStyleClass(PTStyleClass.OBJECT_TEXT);

		return resultSpan;
	}

	/**
	 * Adds a table with default view styling to HTMLCollection result.
	 *
	 * @param result
	 * @return HTMLTable
	 * @throws HTMLException
	 */
	public static HTMLTable AddTable(HTMLElementCollection result)
		
	{
		HTMLTable myTable = MakeTable();
		result.AddInnerHTMLElement(myTable);

		return myTable;
	}

	/**
	 * Adds a table with default list view styling to HTMLCollection result.
	 *
	 * @param result
	 * @return HTMLTable
	 * @throws HTMLException
	 */
	public static HTMLTable AddListTable(HTMLElementCollection result)
		
	{
		HTMLTable myTable = MakeListTable();
		result.AddInnerHTMLElement(myTable);

		return myTable;
	}

	/**
	 * Returns a table with the default view styling.
	 * 
	 * @return HTMLTable
	 * @throws HTMLException
	 */
	public static HTMLTable MakeTable() 
	{
		HTMLTable myTable = new HTMLTable();		
		myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		myTable.SetBorder(CommonHTMLStrings.ZERO);
		myTable.SetCellPadding(CommonHTMLStrings.ONE);
		myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
		myTable.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);		
		return myTable;
	}	

	/**
	 * Returns a table with the default view styling.
	 * 
	 * @return HTMLTable
	 * @throws HTMLException
	 */
	public static HTMLTable MakeListTable() 
	{
		HTMLTable myTable = new HTMLTable();		
		myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		myTable.SetBorder(CommonHTMLStrings.ZERO);
		myTable.SetCellPadding(CommonHTMLStrings.THREE);
		myTable.SetCellSpacing(CommonHTMLStrings.THREE);
		myTable.SetStyleClass(PTStyleClass.LIST_ITEM_TWO_BG);
		return myTable;
	}	

	/**
	 * @param myRow 
	 * @param colspan 
	 * @param sDefaultValue 
	 * @param sHTMLID 
	 *
	 * @return 
	 *
	 * @throws HTMLException 
	 */
	public static HTMLTableCell AddTextAreaCell(HTMLTableRow myRow,
		String colspan, String sDefaultValue, String sHTMLID)
		
	{
		HTMLTableCell myCell = AddCell(myRow, colspan);
		myCell.SetNoWrap(true);

		HTMLSpan mySpan = AddSpanToCell(myCell);
		mySpan.AddInnerHTMLElement(MakeTextArea(TEXTAREA_COLUMNS,
				TEXTAREA_ROWS, sDefaultValue, sHTMLID));

		return myCell;
	}

	/**
	 * Returns a Cell suitable for long text descriptions. (wrapping, 100%, etc)
	 *
	 * @param myRow
	 * @param colspan
	 * @param strText
	 * @return HTMLTableCell
	 * @throws HTMLException
	 */
	public static HTMLTableCell AddTextCell(HTMLTableRow myRow, String colspan,
		String strText) 
	{
		HTMLTableCell myCell = AddCell(myRow, colspan);
		myCell.SetNoWrap(false);
		myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);

		HTMLSpan mySpan = AddSpanToCell(myCell);
		mySpan.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strText));

		return myCell;
	}

	/**
	 * Returns the String path to the folder with ID nAdminFolderID
	 */
	public static String GetAdminFolderPath(IPTSession ptSession,
		int nAdminFolderID)
	{
		IPTAdminCatalog ptAFCatalog = ptSession.GetAdminCatalog();

		IPTAdminFolder ptAFParent = ptAFCatalog.OpenAdminFolder(nAdminFolderID, false);
		String strPath			  = ptAFParent.GetPath();

		return strPath;
	}

	/**
	 * Retrieves an int value from a Query Result.
	 *
	 * @param ptQR
	 * @param nRowIndex
	 * @param nPropID
	 * @return int
	 */
	public static int GetQRValueAsInt(IASQueryResult ptQR, int nRowIndex,
		int nPropID)
	{
		return XPConvert.ToInteger(ptQR.GetFields(nRowIndex, nPropID));
	}

	/**
	 * Retrieves a String value from a QueryResult.
	 *
	 * @param ptQR
	 * @param nRowIndex
	 * @param nPropID
	 * @return String
	 */
	public static String GetQRValueAsString(IASQueryResult ptQR, int nRowIndex,
		int nPropID)
	{
		return XPConvert.ToString(ptQR.GetFields(nRowIndex, nPropID));
	}

	/**
	 * 
	 * Makes a simple link that creates an object in the parent window - passing
	 * in the added arguments.
	 * 
	 * @param nCLASSID
	 * @param strProviderCLSID
	 * @param nFolderID
	 * @param strLink
	 * @param asOwner
	 * @param strArgs
	 * @param bCloseWindowOnExit
	 * @return HTMLElement
	 * @throws HTMLException
	 */
	public static HTMLElement MakeCreateObjectLink(int nCLASSID,
		String strProviderCLSID, int nFolderID, String strLink,
		AActivitySpace asOwner, String strArgs, boolean bCloseWindowOnExit)
		
	{
		HTMLAnchor myAnchor = new HTMLAnchor("#");

		String strURL = "";

		if ((null != strProviderCLSID) && (0 < strProviderCLSID.length()))
		{
			ClassTypeDescManager ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();
			String strProvEnglishName = ptClassTypeDescManager.GetProviderEnglishNameGivenCLSID(strProviderCLSID);

			// explicitly return to the space before this space.
			strURL = PTOpenerLinks.GetOpenerURLCreateExt(nCLASSID, nFolderID, strProvEnglishName, -1, -1, 
			    asOwner, asOwner.GetParentSpaceName(), asOwner.GetParentSpaceID(), null);
		}
		else
		{
			// explicitly return to the space before this space.
			strURL = PTOpenerLinks.GetOpenerURLCreate(nCLASSID, nFolderID, asOwner,
			    asOwner.GetParentSpaceName(), asOwner.GetParentSpaceID(), null);		
		}

		//AGC- Add the args.
		if (null != strArgs)
		{
			strURL = XPStringUtility.ForceEndsWith(strURL, "&");
			if (strArgs.startsWith("&"))
			{
				strArgs = strArgs.substring(1);
			}

			strURL += strArgs;
		}

		XPStringBuilder sb = new XPStringBuilder(200);
		
		
		// TODO CR - MJD : document.location was deprecated in Javascript 1.1, use window.opener.location instead		
		sb.Append("window.opener.document.location = '");
		sb.Append(strURL + "';");

		if (bCloseWindowOnExit)
		{
			sb.Append("window.close();");
		}
		
		sb.Append(" return false;");

		myAnchor.SetHRef("");
		myAnchor.SetOnClick(sb.ToString());
		myAnchor.AddInnerHTMLString(XPHttpUtility.HtmlEncode(strLink));

		return myAnchor;
	}


	/**
	 * Makes a simple button with text strText and onClick handler strOnClick.
	 * Uses the default style for an editor button.
	 *
	 * @param strText
	 * @param strOnClick
	 * @return HTMLElement
	 * @throws HTMLException
	 */
	public static HTMLElement MakeSimpleButton(String strText, String strOnClick)
		
	{
		return ViewHelper.MakeComplexValueButton(strText,
			PTStyleClass.OBJECT_TEXT, PTStyleClass.FORM_BTN_TEXT, strOnClick,
			"in_bu_NULL");
	}

	/**
	 * Makes a simple button with text strText and onClick handler strOnClick.
	 * strHTML will be the button name.
	 * Uses the default style for an editor button.
	 *
	 * @param strText
	 * @param strOnClick
	 * @param strHTML
	 * @return HTMLElement
	 * @throws HTMLException
	 */
	public static HTMLElement MakeSimpleButton(String strText,
		String strOnClick, String strHTML)
		
	{
		return ViewHelper.MakeComplexValueButton(strText,
			PTStyleClass.OBJECT_TEXT, PTStyleClass.FORM_BTN_TEXT, strOnClick,
			strHTML);
	}
	
	/**
	 * Sets the cell color based on bRowColor. Useful for alternating
	 * row colors in table lists.
	 *
	 * @param myCell
	 * @param bRowColor
	 */
	public static void SetRowColor(HTMLTableCell myCell, boolean bRowColor)
		
	{
		if (bRowColor)
		{
			myCell.SetStyleClass(PTStyleClass.LIST_ITEM_TWO_BG);
		} else
		{
			myCell.SetStyleClass(PTStyleClass.LIST_ITEM_ONE_BG);
		}
	}

	/**
	 * Returns JS for validating that a numeric input is, indeed numeric (and
	 * positive)
	 *
	 * @param strFormName
	 * @param strInputName
	 * @param strError
	 * @return String
	 */
	public static String ValidateNumericInputJS(String strFormName,
		String strInputName, String strError)
	{
		XPStringBuilder sb = new XPStringBuilder(300);

		String strJSUtilVersionPrefix = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSUTIL);
		
		sb.Append(" 	if ( !"+strJSUtilVersionPrefix+"PTNumberUtil.isPositiveInteger(document.").Append(strFormName)
		.Append("." + strInputName + ".value) )\n");
		sb.Append("		{\n");
		sb.Append(" 		alert('").Append(strError).Append("');\n");
		sb.Append(" 		return false;\n");
		sb.Append("		}\n");

		return sb.ToString();
	}

	/**
	 * Makes a simple button with text strText and onClick handler strOnClick.
	 * strHTML will be the button name.
	 * stroutterSpanStyle and titleSpanStyle modify the button styles.
	 *
	 * @param sValue
	 * @param outterSpanStyle
	 * @param titleSpanStyle
	 * @param strOnClick
	 * @param strHTML
	 * @return HTMLElement
	 * @throws HTMLException
	 */
	private static HTMLElement MakeComplexValueButton(String sValue,
		PTStyleClass outterSpanStyle, PTStyleClass titleSpanStyle,
		String strOnClick, String strHTML)
		
	{
		HTMLElementCollection result = new HTMLElementCollection();
		HTMLSpan mySpan;
		HTMLInput myInput;

		mySpan = new HTMLSpan();
		result.AddInnerHTMLElement(mySpan);

		mySpan.SetPreComment("begin " + sValue + " button");
		mySpan.SetPostComment("end " + sValue + " button");

		mySpan.SetStyleClass(outterSpanStyle);

		myInput = new HTMLInput(HTMLInputTypes.BUTTON, strHTML, "");
		myInput.SetName(strHTML);

		mySpan.AddInnerHTMLElement(myInput);

		if (null != strOnClick)
		{
			myInput.SetOnClick(strOnClick);
		}

		if (sValue != null)
		{
			myInput.SetValue(sValue);

			if (titleSpanStyle != null)
			{
				myInput.SetStyleClass(titleSpanStyle);
			}
		}

		return result;
	}

	/**
	 * Creates a simple Input element.
	 *
	 * @param myRow
	 * @param colspan
	 * @param strPrefix
	 * @param sDefaultValue
	 * @param sHTMLID
	 * @throws HTMLException
	 */
	private static HTMLElement MakeSmallInput(int nInputSize,
		String sDefaultValue, String sHTMLID, int nInputType)
		
	{
		return MakeSmallInput(nInputSize, sDefaultValue, sHTMLID, nInputType, false, null, INPUTMAXLENGTH_SHORT);
	}
	
	private static HTMLElement MakeSmallInput(int nInputSize,
			String sDefaultValue, String sHTMLID, int nInputType, boolean bInputIsDisabled, String sFormID,
			int nInputMaxLength)
			
		{
			// Input Prefix/Label
			HTMLSpan mySpan = new HTMLSpan();
			mySpan.SetStyleClass(PTStyleClass.OBJECT_TEXT);

			//// Input itself
			HTMLInput myInput = new HTMLInput(nInputType, sHTMLID, "");

			mySpan.AddInnerHTMLElement(myInput);
			myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
			myInput.SetMaxLength(nInputMaxLength); 
			myInput.SetSize(nInputSize);
			
			// aj: if the form id is specified, use it - not sure why we don't just use the
			//     HTML Id, but thats a potentially very large change this late in the game...
			if(sFormID != null)
			{
				myInput.SetID(sFormID);
			}
			
			// aj: set the disabled status...
			myInput.SetDisabled(bInputIsDisabled);

			myInput.SetValue(sDefaultValue);

			return mySpan;
		}

	/**
	 * Creates a simple TextArea element
	 *
	 * @param nColumns
	 * @param nRows
	 * @param sDefaultValue
	 * @param sHTMLID
	 * @return HTMLElement
	 * @throws HTMLException
	 */
	private static HTMLElement MakeTextArea(int nColumns, int nRows,
		String sDefaultValue, String sHTMLID)
		
	{
		// Input Prefix/Label
		HTMLSpan mySpan = new HTMLSpan();
		mySpan.SetStyleClass(PTStyleClass.OBJECT_TEXT);

		//// Input itself
		HTMLTextArea myTextArea = new HTMLTextArea(XPConvert.ToString(nColumns),
				XPConvert.ToString(nRows));
		myTextArea.SetName(sHTMLID);
		myTextArea.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
		mySpan.AddInnerHTMLElement(myTextArea);
		myTextArea.AddInnerHTMLString(XPHttpUtility.HtmlEncode(sDefaultValue));

		return mySpan;
	}
}

