package com.plumtree.portalpages.common.search;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.tree.*;

/**
 *  @author CraigS, based on MikeS code
 */
public class FolderSelectorModel extends EditorHelperModel
	implements IFolderSelectorModelRO, IFolderSelectorModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
		"com.plumtree.portalpages.common.search.FolderSelectorModel");

	
	public static final String STR_MVC_CLASS_NAME = "FolderSelector";
	private String _lastAdmTreeSpaceId;
	private String _lastCatTreeSpaceId;
	private XPArrayList _admSelections;
	private XPArrayList _catSelections;
	private boolean _includeSubfolders = true;
	private boolean _multiSelectMode;

	/**
	 * 
	 *
	 * @param isAdmin 
	 * @param tree 
	 */
	public void AddFoldersFromTreeSelections(boolean isAdmin, ITreeModel tree)
	{
		XPArrayList treeSelections = tree.GetSelectedItems();
		XPArrayList tempList = new XPArrayList();

		IXPEnumerator e = treeSelections.GetEnumerator();

		while (e.MoveNext())
		{
			TreeLeafNode newNode = (TreeLeafNode) e.GetCurrent();
			tempList.Add(newNode);			
		}
		if(isAdmin)
			_admSelections = tempList;
		else
			_catSelections = tempList;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public Object Create()
	{
		return new FolderSelectorModel();
	}

	/**
	 * 
	 *
	 * @param isAdmin 
	 * @param folderIDs 
	 */
	public void DeleteFolders(boolean isAdmin, int[] folderIDs)
	{
		XPArrayList folderList = (isAdmin ? _admSelections : _catSelections);

		for (int j = 0; j < folderIDs.length; j++)
		{
			for (int k = 0; k < folderList.GetSize(); k++)
			{
				TreeLeafNode node = (TreeLeafNode) folderList.GetElement(k);

				if (node.m_nObjectID == folderIDs[j])
				{
					folderList.RemoveAt(k);

					break;
				}
			}
		}
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public boolean GetIncludeSubfolders()
	{
		return _includeSubfolders;
	}

	/**
	 * 
	 *
	 * @param isAdmin 
	 *
	 * @return 
	 */
	public String GetLastTreeSpaceId(boolean isAdmin)
	{
		if (_multiSelectMode)
		{
			// In multiselect mode, reuse the same tree
			if (isAdmin)
			{
				return _lastAdmTreeSpaceId;
			}
			else
			{
				return _lastCatTreeSpaceId;
			}
		}
		else
		{
			// For single-select mode, make a new tree every time.  On
			// the second and later tree calls, this avoids a confusing "You
			// have selected:" bar appearing with the previous selection.
			return null;
		}
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public boolean GetMultiSelectMode()
	{
		return _multiSelectMode;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * 
	 *
	 * @param isAdmin 
	 *
	 * @return 
	 */
	public IXPCollection GetSelections(boolean isAdmin)
	{
		if (isAdmin)
		{
			return _admSelections;
		}
		else
		{
			return _catSelections;
		}
	}

	/**
	 * 
	 *
	 * @param multiSelectMode 
	 */
	public void InitializePlugIn(boolean multiSelectMode)
	{
		_catSelections		    = new XPArrayList();
		_admSelections		    = new XPArrayList();
		_lastCatTreeSpaceId     = null;
		_lastAdmTreeSpaceId     = null;

		_multiSelectMode = multiSelectMode;
	}

	/**
	 * 
	 *
	 * @param formData 
	 *
	 * @return 
	 */
	public int SaveSection(XPHashtable formData)
	{
		_includeSubfolders = formData.ContainsKey(FolderSelectorView.HTMLCHECKBOX_RECURSIVE);

		String[] data = (String[]) formData.GetElement(FolderSelectorView.HTMLHIDDEN_CATSPACEID);

		if ((null != data) && (data.length > 0) && (data[0].length() > 0))
		{
			_lastCatTreeSpaceId = data[0];
		}

		data = (String[]) formData.GetElement(FolderSelectorView.HTMLHIDDEN_ADMSPACEID);

		if ((null != data) && (data.length > 0) && (data[0].length() > 0))
		{
			_lastAdmTreeSpaceId = data[0];
		}

		return RepostControl.PAGE_STATUS_VALID;
	}
	
	public void InitFromFormData(XPHashtable formData)
	{
		if(formData == null)
			return;
		IPTSession _session    = (IPTSession) m_asOwner.GetUserSession();

		String[] subfolders = (String[]) formData.GetElement(StartSearchControl.NON_INPUT_PREFIX +
						SearchSettingCollection.PT_SEARCHSETTING_INCLUDE_SUBFOLDERS_S);
								
		if( (null != subfolders) && subfolders.length > 0)
		{
			try
			{
				_includeSubfolders = (XPConvert.ToInteger(subfolders[0]) != 0);
			}
			catch(Exception e)
			{
				log.Error(e, "FolderSelectorModel.InitFromFormData got exception.");
			}
		}
					
		String[] folders = (String[]) formData.GetElement(StartSearchControl.REQUIRED_CONSTRAINT_PREFIX +
								SearchSettingCollection.PT_SEARCHSETTING_ADMINFOLDERS_S);
		IPTObject ptObj = null;

		if ((null != folders) && (folders.length > 0))
		{
			int id = 0;
			for (int j = 0; j < folders.length; j++)
			{
					try
					{
						id = XPConvert.ToInteger(folders[j]);
						
						TreeLeafNode newNode = new TreeLeafNode(1);
						newNode.m_nObjectID = id;
						try
						{
						
							ptObj = _session.GetAdminCatalog().OpenAdminFolder(id, false);
							if (null != ptObj)
							{
								String name = ptObj.GetLocalizedName();
	
								if (null != name)
								{
									newNode.SetName(name);
								}
								else
									newNode.SetName("NULL");
							}

						}
						catch(Exception e)
						{
							newNode.SetName("NULL");
						}
						_admSelections.Add(newNode);

					}
					catch (Exception e)
					{
						throw XPException.GetInstance(e);
					}
			}
		}
		folders = (String[]) formData.GetElement(StartSearchControl.REQUIRED_CONSTRAINT_PREFIX +
							SearchSettingCollection.PT_SEARCHSETTING_DDFOLDERS_S);

		if ((null != folders) && (folders.length > 0))
		{
			int id = 0;
			for (int j = 0; j < folders.length; j++)
			{
				try
				{
					id = XPConvert.ToInteger(folders[j]);
					TreeLeafNode newNode = new TreeLeafNode(1);
					newNode.m_nObjectID = id;
					try
					{
						
						ptObj = _session.GetCatalog().OpenFolder(id, false);
						if (null != ptObj)
						{
							String name = ptObj.GetLocalizedName();
	
							if (null != name)
							{
								newNode.SetName(name);
							}
							else
								newNode.SetName("NULL");
						}

					}
					catch(Exception e)
					{
						newNode.SetName("NULL");
					}

					_catSelections.Add(newNode);
				}
				catch (Exception e)
				{
					throw XPException.GetInstance(e);
				}

			}
		}
				
	}
	
	/*
	 * getFolderSelections returns a comma-separated list of classid,objectid pairs. It is
	 * used to initialize the tree view.
	 * @see com.plumtree.portalpages.common.search.IFolderSelectorModelRO#getFolderSelections(boolean)
	 */
	public String GetFolderSelections(boolean isAdmin)
	{
		XPArrayList folderList = (isAdmin ? _admSelections : _catSelections);
		String retval = "";
		for (int j = 0; j < folderList.GetSize(); j++)
		{
			for (int k = 0; k < folderList.GetSize(); k++)
			{
				TreeLeafNode node = (TreeLeafNode) folderList.GetElement(k);

				// Add classid,
				retval = commaAdd(retval,(isAdmin) ? "20" : "17");
				// Add objectid,
				retval = commaAdd(retval, XPConvert.ToString(node.m_nObjectID));
			}
		}
		return retval;
	}
	
	private String commaAdd(String initial, String addend)
	{
		if(initial != null && ! initial.equals("") )
			return initial + "," + addend;
		else
			return addend;
	}
}

