package com.plumtree.portalpages.common.search;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.filter.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;
/**
 * @author MikeS, modified by CraigS
 *
 *   Similar to DynamicQueryTable, but avoids use of javascript to
 *   hide or reveal individual statement rows.
 *
 *   A total of five calls will generate all of the necessary
 *   elements:
 *
 *   1) SetBlankQueryCondition(QueryCondition)  // defines the row template
 *   2) generateTable (numberOfRows)    // gets a table of hidden rows
 *   3) getJavaScriptFunctions() // gets necessary script elements
 *   4) getAddConditionButton(strAdd) // button to reveal a row
 *   5) getRemoveLastConditionButton(strRemove)  // button to hide last row
 *
 *   The "Add Condition" and "Remove Last Condition" buttons
 *   must be retrieved with separate calls.
 */
public class RepostQueryTable implements IFilterQueryTable
{
	private static final String IN_CB_TOGGLEALL = "in_cb_ToggleAllRQT";
	private static final String PORTALCOMMONMSGS = ResourceConstants.PORTALCOMMONMSGS;
	private static final String EMPTY = "";
	private AActivitySpace m_asOwner;
	private QueryCondition _blankQC					  = null;
	private HTMLElement _extraRowBelowGroupHeader = null;
	private boolean _hideGroupHeader = false;
	private String _label;
	private String _name;
	private XPArrayList _definedRows		 = null;
	private boolean _andOperator		   = true;
	private boolean _suppressCheckbox	   = false;
	private boolean _useVerboseTerminology = false;
	private int _depth   = 0;
	private int _maxRows = 0;

	/**
	 * Creates a new RepostQueryTable object.
	 *
	 * @param strName 
	 * @param Depth 
	 * @param owner 
	 */
	public RepostQueryTable(String strName, int Depth, AActivitySpace owner)
	{
		m_asOwner     = owner;
		_depth		  = Depth;
		_name		  = strName;
		_maxRows	  = 0;
		_label		  = null;
	}

	/**
	 * 
	 *
	 * @param bOperator 
	 */
	public void SetAllMustPass(boolean bOperator)
	{
		_andOperator = bOperator;
	}

	/**
	 * 
	 *
	 * @param qc 
	 */
	public void SetBlankQueryCondition(QueryCondition qc)
	{
		qc.SetVisible(false);
    qc.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);
		_blankQC = qc;
	}

	/**
	 * 
	 *
	 * @param s 
	 */
	public void SetExtraRowBelowGroupHeader(String s)
	{
		_extraRowBelowGroupHeader = new HTMLGenericElement(s);
	}

	/**
	 * 
	 *
	 * @param s 
	 */
	public void SetExtraRowBelowGroupHeader(HTMLElement e)
	{
		_extraRowBelowGroupHeader = e;
	}

	public void SetHideGroupHeaderRow(boolean hide) {
		_hideGroupHeader = hide;
	}

	/**
	 * 
	 *
	 * @param label 
	 */
	public void SetLabel(String label)
	{
		_label = label;
	}

	/**
	 * 
	 *
	 * @param b 
	 */
	public void SetSuppressCheckbox(boolean b)
	{
		_suppressCheckbox = b;
	}

	/**
	 * 
	 *
	 * @param nRows 
	 */
	public void SetTableSize(int nRows)
	{
		if (nRows <= 0)
		{
			throw new XPException("Row count must be greater than zero");
		}
		else if (_maxRows != 0)
		{
			throw new XPException("Row count may not be set more than once");
		}
		else
		{
			_maxRows = nRows;
		}
	}

	/**
	 * 
	 *
	 * @param b 
	 */
	public void SetUseVerboseTerminology(boolean b)
	{
		_useVerboseTerminology = b;
	}

	/**
	 * 
	 *
	 * @param strActiveClause 
	 * @param groupCount 
	 * @param result 
	 *
	 * @throws Exception 
	 */
	public static void AddGlobalElements(String strActiveClause,
		int groupCount, HTMLElementCollection result)

	{
		HTMLInput activeClause = new HTMLInput(HTMLInputTypes.HIDDEN,
				FilterForm.ACTIVE_CLAUSE, EMPTY);
		activeClause.SetID("activeclause");
		activeClause.SetValue(strActiveClause);
		result.AddInnerHTMLElement(activeClause);

		HTMLInput action = new HTMLInput(HTMLInputTypes.HIDDEN,
				FilterForm.ACTION, EMPTY);
		action.SetID("groupaction");
		action.SetValue(EMPTY);
		result.AddInnerHTMLElement(action);

		HTMLInput totalGroups = new HTMLInput(HTMLInputTypes.HIDDEN,
				FilterForm.TOTAL_GROUPS, EMPTY);
		totalGroups.SetID("totalgroups");
		totalGroups.SetValue(XPConvert.ToString(groupCount));
		result.AddInnerHTMLElement(totalGroups);
	}

	/**
	 * 
	 *
	 * @param searchstring 
	 * @param AllMustPass 
	 * @param result 
	 *
	 * @throws Exception 
	 */
	public static void AddTopLevelSearchBoxAndOperator(String searchstring,
		boolean AllMustPass, HTMLElementCollection result, String strLang, AActivitySpace asOwner)

	{
		AddTopLevelSearchBoxAndOperator(searchstring, AllMustPass, false, null,
			result, strLang, asOwner);
	}

	/**
	 * 
	 *
	 * @param searchstring 
	 * @param AllMustPass 
	 * @param useVerboseTerminology 
	 * @param textboxLabel 
	 * @param result 
	 *
	 * @throws Exception 
	 */
	public static void AddTopLevelSearchBoxAndOperator(String searchstring,
		boolean AllMustPass, boolean useVerboseTerminology,
		HTMLElement textboxLabel, HTMLElementCollection result, String strLang, AActivitySpace asOwner)

	{
		HTMLTable myTable = new HTMLTable();
		result.AddInnerHTMLElement(myTable);

		myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		myTable.SetBorder(CommonHTMLStrings.ZERO);
		myTable.SetCellPadding(CommonHTMLStrings.FIVE);
		myTable.SetCellSpacing(CommonHTMLStrings.ZERO);

		HTMLTableRow textSearchRow = new HTMLTableRow();
		HTMLTableRow operatorRow   = new HTMLTableRow();
		
		myTable.AddInnerHTMLElement(textSearchRow);
		myTable.AddInnerHTMLElement(operatorRow);
		textSearchRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
		operatorRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);

		HTMLTableCell textCell = new HTMLTableCell();
		textCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
		textCell.SetWidth("50%");

		HTMLTableCell opCell   = new HTMLTableCell();
		opCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
		opCell.SetWidth("50%");

		HTMLTableCell textCellLabel = new HTMLTableCell();
		textCellLabel.SetStyleClass(PTStyleClass.OBJECT_TEXT);
		textCellLabel.SetWidth("50%");
		textCellLabel.SetNoWrap(true);
		HTMLSpan span = new HTMLSpan();
		span.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
		textCellLabel.AddInnerHTMLElement(span);
		span.AddInnerHTMLString(asOwner.GetString(737, PORTALCOMMONMSGS));

		HTMLTableCell opCellLabel = new HTMLTableCell();
		opCellLabel.SetStyleClass(PTStyleClass.OBJECT_TEXT);
		opCellLabel.SetWidth("50%");
		opCellLabel.SetNoWrap(true);
		
		textSearchRow.AddInnerHTMLElement(textCellLabel);
		//textSearchRow.AddInnerHTMLElement(textCell);
		textSearchRow.AddInnerHTMLElement(opCellLabel);
		
		span = new HTMLSpan();
		span.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
		opCellLabel.AddInnerHTMLElement(span);
		span.AddInnerHTMLString(asOwner.GetString(738, PORTALCOMMONMSGS));
		operatorRow.AddInnerHTMLElement(textCell);
		//operatorRow.AddInnerHTMLElement(opCellLabel);
		operatorRow.AddInnerHTMLElement(opCell);


		HTMLInput input = new HTMLInput(HTMLInputTypes.TEXT_TYPE,
				FilterForm.FULL_TEXT, EMPTY);
		input.SetValue(searchstring);
		input.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
		input.SetSize(55);

		IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
		XPResourceManager rm     = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");

/*		if (null != textboxLabel)
		{
			textCell.AddInnerHTMLElement(textboxLabel);
		}
		else
		{
			textCell.AddInnerHTMLString(rm.GetString(614, PORTALCOMMONMSGS, strLang));
		}
*/
		textCell.AddInnerHTMLElement(input);

		HTMLInput andRB = new HTMLInput(HTMLInputTypes.RADIO,
				FilterForm.TOP_OPERATOR, EMPTY);
		andRB.SetValue("and");
		andRB.SetChecked(AllMustPass);
		//opCell.SetStyleClass(PTStyleClass.MENU_TEXT);
		opCell.AddInnerHTMLElement(andRB);

		if (useVerboseTerminology)
		{
			opCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
			opCell.AddInnerHTMLString(rm.GetString(619, PORTALCOMMONMSGS, strLang));
			opCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
			opCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		}
		else
		{
			opCell.AddInnerHTMLString(rm.GetString(616, PORTALCOMMONMSGS, strLang));
		}

		HTMLInput orRB = new HTMLInput(HTMLInputTypes.RADIO,
				FilterForm.TOP_OPERATOR, EMPTY);
		orRB.SetValue("or");
		orRB.SetChecked(!AllMustPass);
		opCell.AddInnerHTMLElement(orRB);

		if (useVerboseTerminology)
		{
			opCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
			opCell.AddInnerHTMLString(rm.GetString(618, PORTALCOMMONMSGS, strLang));
		}
		else
		{
			opCell.AddInnerHTMLString(rm.GetString(617, PORTALCOMMONMSGS, strLang));
		}
	}

	/**
	 * 
	 *
	 * @param strLabel 
	 *
	 * @return 
	 *
	 * @throws Exception 
	 */
	public static HTMLElement GetAddConditionButton(String strLabel, String strLang)

	{
		if ((null == strLabel) || (0 == strLabel.length()))
		{
			IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
			XPResourceManager rm = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");

			strLabel = rm.GetString(625, PORTALCOMMONMSGS, strLang);
		}

		HTMLAnchor a = new HTMLAnchor(CommonHTMLStrings.POUND_SYMBOL);
		a.SetOnClick("adjustRow(getActiveClause(), true); return false;");
		a.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		PTConstruct construct = ConstructFactory.GetConstruct(PTImageType.ACTION_ADD,
				true);
		HTMLImg img		   = (HTMLImg) construct.GetHTMLElement();
		img.SetWidth(XPConvert.ToString(6));
		img.SetHeight("25");
		img.SetAlign(CommonHTMLStrings.ABSMIDDLE);
		a.AddInnerHTMLElement(img);

		construct = ConstructFactory.GetConstruct(PTImageType.ICON_DOCUMENT,
						true);
		img		   = (HTMLImg) construct.GetHTMLElement();
		img.SetWidth(XPConvert.ToString(14));
		img.SetHeight("20");
		img.SetAlign(CommonHTMLStrings.ABSMIDDLE);
		a.AddInnerHTMLElement(img);

		HTMLSpan span = new HTMLSpan();
		span.AddInnerHTMLString(strLabel);
		span.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);
		a.AddInnerHTMLElement(span);
		
		a.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		return a;
	}

	/**
	 * 
	 *
	 * @param strLabel 
	 *
	 * @return 
	 *
	 * @throws Exception 
	 */
	public static HTMLElement GetAddNewClauseButton(String strLabel, String strLang)

	{
		return null;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public boolean GetAllMustPass()
	{
		return _andOperator;
	}

	/**
	 * 
	 *
	 * @param iIndex 
	 *
	 * @return 
	 */
	public QueryCondition GetDefinedQueryCondition(int iIndex)
	{
		if ((iIndex + 1) > _definedRows.GetSize())
		{
			throw new XPException("Requested condition is not in list: " +
				(iIndex));
		}

		return (QueryCondition) (_definedRows.GetElement(iIndex));
	}

	/**
	 * 
	 *
	 * @param submitString 
	 *
	 * @return 
	 *
	 * @throws Exception 
	 */
	public static HTMLElement GetJavaScript(String submitString)

	{
		XPStringBuilder sb = new XPStringBuilder();
		HTMLScript script  = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);

		sb.Append(
			"\nfunction ToggleStatementInput(TurnOff, GroupID, StatementID)\n");
		sb.Append("{\n");
		sb.Append("\n");
		sb.Append(" var dispValue = (0 == TurnOff) ? '' : 'none';\n");
		sb.Append(
			" var compVal = document.getElementById(GroupID + '_divCompareval_' + StatementID);\n");
		sb.Append(" compVal.style.display = dispValue;\n");
		sb.Append("}\n");

		sb.Append("function ToggleDateInput(TurnOff, GroupID, StatementID)\n");
		sb.Append("{\n");
		sb.Append("   var dispValue = (4 == TurnOff) ? '' : 'none';\n");
		sb.Append("	  var compVal = document.getElementById(GroupID + '_divRelative_' + StatementID);\n");
		sb.Append("	  compVal.style.display = dispValue;\n");
		sb.Append("\n");
		sb.Append("   dispValue = (4 == TurnOff) ? 'none' : '';\n");
		sb.Append("	  compVal = document.getElementById(GroupID + '_divDateChooser_' + StatementID);\n");
		sb.Append("	  compVal.style.display = dispValue;\n");
		sb.Append("}\n");
		sb.Append("\n");

		sb.Append("\nfunction AdjustOperator(GroupID, RowNumber, val)\n");
		sb.Append("{\n");
		sb.Append("\n");
		sb.Append(" var valPair = val.split(\"|\");\n");
		sb.Append(
			" var CompVal = document.getElementById(GroupID + '_divCompareval_' + RowNumber);\n");
		sb.Append(" CompVal.style.display = '';\n");
		sb.Append(
			" var DateChooser = document.getElementById(GroupID + '_divDateChooser_' + RowNumber);\n");
		sb.Append(" DateChooser.style.display = 'none';\n");
		sb.Append(" var Type = 'Text';\n");
		sb.Append(" if (valPair[1]=='3' || valPair[1]=='5')\n");
		sb.Append("  Type = 'Number';\n");
		sb.Append(" if (valPair[1]=='7')\n");
		sb.Append(" {\n");
		// Not until using jitDatePicker sb.Append("  eval('fxn' + GroupID + '_divDateChooser_' + RowNumber + '();');\n");
		sb.Append("  DateChooser.style.display = '';\n");
		sb.Append("  CompVal.style.display = 'none';\n");
		sb.Append("  Type = 'Date';\n");
		sb.Append(" }\n");
		sb.Append("\n");

		// First shut all of them off, then turn on the appropriate operator drop down
		sb.Append(
			" var OpDiv = document.getElementById(GroupID + '_divText_' + RowNumber);\n");
		sb.Append(" OpDiv.style.display = 'none';\n");
		sb.Append(
			" OpDiv = document.getElementById(GroupID + '_divNumber_' + RowNumber);\n");
		sb.Append(" OpDiv.style.display = 'none';\n");
		sb.Append(
			" OpDiv = document.getElementById(GroupID + '_divDate_' + RowNumber);\n");
		sb.Append(" OpDiv.style.display = 'none';\n");
		sb.Append(
			"	OpDiv = document.getElementById(GroupID + '_divRelative_' + RowNumber);\n");
		sb.Append("	OpDiv.style.display = 'none';\n");
		sb.Append(
		"	// If this is the Member Of property (73), we don't want to display anything.\n");
	sb.Append(
		"	// Otherwise display the appropriate operator drop down.\n");
	sb.Append(
		"	if(valPair[1]!='73')\n");
	sb.Append(
		"	{\n");
		sb.Append(
			" OpDiv = document.getElementById(GroupID + '_div' + Type + '_' + RowNumber);\n");
		sb.Append(" OpDiv.style.display = '';\n");
		sb.Append(
		"	}\n");
		sb.Append("}\n");

		sb.Append("\nfunction adjustRow(GroupID, bAdd)\n");
		sb.Append("{\n");
		sb.Append(" var action = document.getElementById('groupaction');\n");
		sb.Append(" if (bAdd)\n");
		sb.Append("  action.value = 'ADDSTMT';\n");
		sb.Append(" else\n");
		sb.Append("  action.value = 'REMOVECHECKEDSTMTS';\n");
		sb.Append(submitString);
		sb.Append("}\n");

		sb.Append("function getActiveClause()\n");
		sb.Append("{\n");
		sb.Append(" var clause = document.getElementById('activeclause');\n");
		sb.Append(" var groupid = clause.value;\n");
		sb.Append(" return groupid;\n");
		sb.Append("}\n");

		sb.Append("\nfunction setActiveClause(NewGroupNum, IsChecked)\n");
		sb.Append("{\n");
		sb.Append(" if (!IsChecked)\n");
		sb.Append(" { // it doesn't make sense to uncheck a group\n");
		sb.Append(
			"  var ThisCheckBox = document.getElementById(NewGroupNum+'_checked');\n");
		sb.Append("  ThisCheckBox.checked = true;\n");
		sb.Append("  return;\n");
		sb.Append(" }\n");
		sb.Append("\n");
		sb.Append(" var clause = document.getElementById('activeclause');\n");
		sb.Append(" var oldfocus = clause.value;\n");
		sb.Append(
			" var vChecked = document.getElementById(oldfocus+'_checked');\n");
		sb.Append(" clause.value = NewGroupNum;\n");
		sb.Append(" if (!vChecked.checked)\n");
		sb.Append("  return;\n");
		sb.Append(" vChecked.checked = false;\n");
		sb.Append("}\n");

		sb.Append("\nfunction adjustGroups(bAdd)\n");
		sb.Append("{\n");
		sb.Append(" var action = document.getElementById('groupaction');\n");
		sb.Append(" if (bAdd)\n");
		sb.Append("  action.value = 'ADD';\n");
		sb.Append(" else\n");
		sb.Append("  action.value = 'REMOVE';\n");
		sb.Append(submitString);
		sb.Append("}\n");
		sb.Append("function DoToggleAllRQT()\n");
		sb.Append("{\n");			
		sb.Append("    var fElem = document.mainform;\n");
		sb.Append("    var feLen = fElem.elements.length;\n");
		sb.Append("    var checkVal = false;\n");
		sb.Append("    for (var i = 0; i < feLen; i++) {\n");
		sb.Append("        if (fElem.elements[i].name == '" + IN_CB_TOGGLEALL + "') {\n");
		sb.Append("            checkVal = fElem.elements[i].checked;\n");
		sb.Append("        }\n");
		sb.Append("    }\n");
		sb.Append("    for (var i = 0; i < feLen; i++) {\n");
		sb.Append("    	   if (fElem.elements[i].name.indexOf('" + FilterForm.REMOVE_STMT_CHECKBOX + "') != -1) {\n");
		sb.Append("            fElem.elements[i].checked = checkVal;\n");
		sb.Append("        }\n");
		sb.Append("    }\n");
		sb.Append("    return true;\n"); 
		sb.Append("}\n"); 

		script.AddInnerHTMLString(sb.ToString());

		return script;
	}

	/**
	 * 
	 *
	 * @param strLabel 
	 *
	 * @return 
	 *
	 * @throws Exception 
	 */
	public static HTMLElement GetRemoveClauseButton(String strLabel, String strLang)

	{
		return null;

	}

	/**
	 * 
	 *
	 * @param strLabel 
	 *
	 * @return 
	 *
	 * @throws Exception 
	 */
	public static HTMLElement GetRemoveLastConditionButton(String strLabel, String strLang)

	{
		HTMLAnchor a = new HTMLAnchor(CommonHTMLStrings.POUND_SYMBOL);
		a.SetOnClick("adjustRow(getActiveClause(), false); return false;");
		a.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		PTConstruct construct = ConstructFactory.GetConstruct(PTImageType.ACTION_REMOVE,
				true);
		HTMLImg img		   = (HTMLImg) construct.GetHTMLElement();
		img.SetWidth(XPConvert.ToString(25));
		img.SetHeight("25");
		img.SetAlign(CommonHTMLStrings.ABSMIDDLE);
		a.AddInnerHTMLElement(img);

		return a;

	}

	/**
	 * 
	 *
	 * @param result 
	 * @param showErrorsNextToInputs 
	 */
	public void addTableToCollection(HTMLElementCollection result,
                                   boolean showErrorsNextToInputs,
                                   JSDateTimePickerMgr jsDateTimePickerMgr, PageMgr _ptPageMgr)
	{
		// this spacer table is used for indenting clauses
		if (_blankQC == null)
		{
			throw new XPException(
				"Blank query condition must be set before generating table");
		}

		if (_maxRows == 0)
		{
			throw new XPException(
				"Table size must be set before generating table.");
		}

		HTMLTable spacerTable = new HTMLTable();
		spacerTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		spacerTable.SetBorder(CommonHTMLStrings.ZERO);
		spacerTable.SetCellPadding(CommonHTMLStrings.ZERO);
		spacerTable.SetCellSpacing(CommonHTMLStrings.ZERO);

		HTMLTableRow spacerRow   = new HTMLTableRow();
		HTMLTableCell spacerCell = new HTMLTableCell();
		HTMLTableCell clauseCell = new HTMLTableCell();

		HTMLTable myTable = new HTMLTable(); // the inner table

		if (_definedRows == null)
		{
			_definedRows = new XPArrayList();
		}

		try
		{
			if (_depth > 0)
			{ 

				PTConstruct construct = ConstructFactory.GetConstruct(PTImageType.SPACER,
						true);
				HTMLImg img		   = (HTMLImg) construct.GetHTMLElement();
				img.SetWidth(XPConvert.ToString(_depth * 50));
				spacerCell.AddInnerHTMLElement(img);
				spacerRow.AddInnerHTMLElement(spacerCell);
			}
			if( _definedRows.GetSize() > 0)
				result.AddInnerHTMLElement(SearchUIUtility.makeSpacer(m_asOwner, CommonHTMLStrings.FIVE));
			// ATTACH SPACER TABLE TO RESULT
			result.AddInnerHTMLElement(spacerTable);

			// ATTACH ROW TO SPACER TABLE
			spacerTable.AddInnerHTMLElement(spacerRow);

			// ATTACH CELL TO ROW
			spacerRow.AddInnerHTMLElement(clauseCell);

			// ATTACH INNER TABLE TO CELL
			clauseCell.AddInnerHTMLElement(myTable);

			myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT); //was 600
			myTable.SetBorder(CommonHTMLStrings.ZERO);
			myTable.SetCellPadding(CommonHTMLStrings.TWO);
			myTable.SetCellSpacing(CommonHTMLStrings.TWO);
			if (null != _extraRowBelowGroupHeader)
			{
				HTMLTableRow extraRow = new HTMLTableRow();
				extraRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
				myTable.AddInnerHTMLElement(extraRow);

				HTMLTableCell extraCell = new HTMLTableCell();
				extraCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
				extraCell.SetHeight("35");
				extraCell.SetVAlign(CommonHTMLStrings.CENTER);
				extraRow.AddInnerHTMLElement(extraCell);
				extraCell.SetColSpan(CommonHTMLStrings.THREE);
				extraCell.AddInnerHTMLElement(_extraRowBelowGroupHeader);
			}

			int tableSize = _definedRows.GetSize();
			if(tableSize > 0 && !_hideGroupHeader)
			{
				myTable.AddInnerHTMLElement(GetHeaderRow());
			}

			// add the actual property constraint rows
			QueryCondition qc;

			for (int i = 0; i < tableSize; i++)
			{
				qc = (QueryCondition) _definedRows.GetElement(i);
				qc.SetVisible(true);
				if(i % 2 == 1)
					qc.SetStyleClass(PTStyleClass.LIST_ITEM_TWO_BG);
				else
					qc.SetStyleClass(PTStyleClass.LIST_ITEM_ONE_BG);
				qc.PadCheckBox(false);
				myTable.AddInnerHTMLElement(qc.GetTableRow(_name,
						XPConvert.ToString(i + 1), showErrorsNextToInputs, 
            /* showRemovalCheckbox= true*/!_suppressCheckbox,
               jsDateTimePickerMgr, _ptPageMgr,
            /* jitDatePicker */ false));
			}

			HTMLInput maxrows = new HTMLInput(HTMLInputTypes.HIDDEN,
					FilterForm.MAX_STMTS + _name, EMPTY);
			maxrows.SetID(_name + "_maxrows");
			maxrows.SetValue(XPConvert.ToString(_maxRows));
			clauseCell.AddInnerHTMLElement(maxrows);

			HTMLInput revealed = new HTMLInput(HTMLInputTypes.HIDDEN,
				FilterForm.REVEALED + _name, EMPTY);
			revealed.SetID(_name + "_revealed");
			revealed.SetValue(XPConvert.ToString(tableSize));
			clauseCell.AddInnerHTMLElement(revealed);

			HTMLInput depth = new HTMLInput(HTMLInputTypes.HIDDEN,
					FilterForm.DEPTH_PREFIX + _name, EMPTY);
			depth.SetID(_name + "_depth");
			depth.SetValue(XPConvert.ToString(_depth));
			clauseCell.AddInnerHTMLElement(depth);
		}
		catch (Exception e)
		{
			throw XPException.GetInstance(e);
		}
	}
	public HTMLTableRow GetHeaderRow()
	{
		HTMLTableRow retval = new HTMLTableRow();
		retval.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);
		HTMLTableCell cell;
		
		if (!_suppressCheckbox)
		{
			cell = new HTMLTableCell();
			retval.AddInnerHTMLElement(cell);
			cell.SetWidth("20");
			cell.SetHeight("20");
			cell.SetAlign(CommonHTMLStrings.CENTER);
			cell.SetVAlign(CommonHTMLStrings.TOP);
			cell.SetStyleClass(PTStyleClass.LIST_TEXT);

			HTMLInput checkBox = new HTMLInput(HTMLInputTypes.CHECKBOX, IN_CB_TOGGLEALL,"");
			checkBox.SetOnClick("DoToggleAllRQT()");
			cell.AddInnerHTMLElement(checkBox);
		}
		
		cell = new HTMLTableCell();
		retval.AddInnerHTMLElement(cell);
		cell.SetHeight("20");
		cell.SetWidth("250");
		cell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
		cell.AddInnerHTMLString(m_asOwner.GetString(740, PORTALCOMMONMSGS));//Property
		
		cell = new HTMLTableCell();
		retval.AddInnerHTMLElement(cell);
		cell.SetHeight("20");
		cell.SetWidth("250");
		cell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
		cell.AddInnerHTMLString(m_asOwner.GetString(741, PORTALCOMMONMSGS));//Condition

		cell = new HTMLTableCell();
		retval.AddInnerHTMLElement(cell);
		cell.SetHeight("20");
		cell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		cell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
		cell.AddInnerHTMLString(m_asOwner.GetString(742, PORTALCOMMONMSGS));//Value

		return retval;
	}
	
	/**
	 * 
	 *
	 * @param qc 
	 */
	public void loadDefinedQueryCondition(QueryCondition qc)
	{
		if (_definedRows == null)
		{
			_definedRows = new XPArrayList();
		}

		_definedRows.Add(qc);
	}

	/**
	 * 
	 *
	 * @param iIndex 
	 */
	public void removeDefinedQueryCondition(int iIndex)
	{
		if ((iIndex + 1) > _definedRows.GetSize())
		{
			throw new XPException("Requested condition is not in list: " +
				(iIndex));
		}

		_definedRows.RemoveAt(iIndex);
	}
}

