package com.plumtree.portalpages.common.search;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author CraigS
 */
public class SimpleSelectorView extends EditorLikeView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_COMMON),
		"com.plumtree.portalpages.common.search.SimpleSelectorView");

	
	public static final String STR_MVC_CLASS_NAME = "SimpleSelector";

	private ISimpleSelectorModelRO _model = null;

	/**
	 * 
	 *
	 * @return 
	 */
	public Object Create()
	{
		return new SimpleSelectorView();
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public HTMLElement Display()
	{
		_model = (ISimpleSelectorModelRO) m_model;

		HTMLElementCollection result = new HTMLElementCollection();

		try
		{
			result.AddInnerHTMLElement(super.GetHeaderView(_model.GetTitle(), true));

			HTMLTable table = new HTMLTable();
			table.SetCellSpacing("0");
			table.SetCellPadding("5");
			table.SetWidth("100%");
			result.AddInnerHTMLElement(table);

			HTMLTableRow row = new HTMLTableRow();
			table.AddInnerHTMLElement(row);
			row.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);

			HTMLTableCell cell = new HTMLTableCell();
			cell.SetWidth("100%");
			cell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
			row.AddInnerHTMLElement(cell);

			HTMLSpan span = new HTMLSpan();
			span.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);
			span.AddInnerHTMLString(_model.GetPromptText());
			cell.AddInnerHTMLElement(span);
			
			HTMLSelect select = new HTMLSelect(_model.GetElementName(), "");
			select.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
			cell.AddInnerHTMLString("&nbsp;&nbsp;");
			cell.AddInnerHTMLElement(select);

			String[] options = _model.GetPossibleSelections();

			for (int j = 0; j < options.length; j++)
			{
				HTMLOption option = new HTMLOption(XPConvert.ToString(j));
				option.AddInnerHTMLString(XPHttpUtility.HtmlEncode(options[j]));
				option.SetSelected(j == _model.GetCurrentSelection());
				select.AddInnerHTMLElement(option);
			}

			cell.AddInnerHTMLElement(new HTMLBr());
		}
		catch (Exception e)
		{
			
				log.Error(e, "SimpleSelectorView.Display error.");
			

			throw new XPException(m_asOwner.GetString(227,
					"ptmsgs_portalcommonmsgs"), e);
		}

		return result;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
}

