package com.plumtree.portalpages.pei;

import com.plumtree.openfoundation.util.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;


/**
 * This interface was created to allow developers the ability to create
 * functions that can be dynamically discovered by the Plumtree application
 * for the purposes of performing some functionality when the user password is 
 * changed.
 * <br><br>
 * NOTE: Should this interface be implemented more than once within the
 * Dynamic Discovery process, the implementations will be processed in the
 * order that they are displayed within the representative XML file. All
 * implementations will be called.
 * <br><br>
 * NOTE: In the case where multiple implementations return a String object, all 
 * the valid Strings (not null and not empty) will be appended together 
 * (separated by a blank space) and the final String will be the error message 
 * displayed on the page.
 */
public interface IPasswordActions
{
    /**
     * This interface will allow for the implementation of functions that will
     * occur when a user tries to change their password.
     * <br><br>
     * Warning: Values in _htFormData are String Arrays (String[]).<br>
     * e.g.: <br>
     * String[] arrVals = (String[]) _htFormData.GetElement(sKey);<br>
     * String sVal = arrVals[0];
     *
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _ptUserSession - Plumtree session object for the current user
     * @param _htFormData - This parameter refers to the form data that has
     *        been entered by the user during the changing of a password.
     *
     * @return String - This will be a string containing a error message if the
     *         data was incorrect or if the process should not continue
     *         otherwise it will be null or "".
     */
    public String OnBeforeChangePassword(AActivitySpace _asCurrentSpace,
                                         IPTSession _ptUserSession,
                                         XPHashtable _htFormData);
}

