package com.plumtree.portalpages.portlets.savedsearch;

import com.plumtree.openfoundation.internationalization.XPLocale;
import com.plumtree.openfoundation.internationalization.XPTimeZone;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.browsing.myportal.*;
import com.plumtree.portalpages.browsing.portletpreview.*;
import com.plumtree.portalpages.common.constants.*;
import com.plumtree.portalpages.common.search.*;
import com.plumtree.portaluiinfrastructure.filter.*;
import com.plumtree.portaluiinfrastructure.intrinsicportlets.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.search.exceptions.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.*;
import com.plumtree.uiinfrastructure.pagination.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 *
 *
 * @author mikesample
 */
public class SavedSearchPortletView extends IntrinsicPortletView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_PORTLETS),
		"com.plumtree.portalpages.portlets.savedsearch.SavedSearchPortletView");

	/**
	 * Class name for the view. For intrinsic portlets, the view must have the
	 * same class name as the model.
	 */
	public static final String STR_MVC_CLASS_NAME = SavedSearchPortletModel.STR_MVC_CLASS_NAME;

	
	public static final String HTMLINPUT_VALUE = "in_tx_Value";

	
	public static final String HTMLHIDDENINPUT_MAILID = "in_hi_mailID";

	
	public static final String HTMLHIDDENINPUT_RESULTPAGE = "in_hi_resultPage";

	
	public static final String HTMLHIDDENINPUT_SORTID = "in_hi_sortID";

	
	public static final String HTMLINPUT_SUBSCRIBE = "in_bu_subscribe";

	
	public static final String HTMLINPUT_SEARCH = "in_bu_search";

	
	public static final String HTMLHIDDENINPUT_PAGINATE_FIRST = "in_hi_first";
	
	
	public static final String HTMLHIDDENINPUT_PAGINATE_NUM = "in_hi_num";

	
	public static final String JAVASCRIPT_RESULTPAGE = "ResultPage";

	
	public static final String JAVASCRIPT_SEARCH = "Search";
	
	
	public static final String JAVASCRIPT_CLEAR_SEARCH = "ClearSearch";

	
	public static final String JAVASCRIPT_SORT = "Sort";

	
	public static final String JAVASCRIPT_PAGINATE = "Paginate";

	
//	public static final String JAVASCRIPT_REFRESH = "refresh";

	
	public static final String ITEMSIZES = "20";

	
//	public static final String REFRESH_ID = "refresharea";


	private boolean _preferPopupDocs = false;
	
	/**
	 * Creates a new SavedSearchPortletView object.
	 */
	public SavedSearchPortletView()
	{
		m_bAddFormToPortlet     = true;
		m_bMultiPart		    = false;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new SavedSearchPortletView();
	}

	/**
	 * This method returns the HTMLScript that encompasses the Javascript on
	 * the page.
	 *
	 * @return HTMLScript
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		HTMLScript myScript = super.DisplayJavascript();

		XPStringBuilder sb;

		int nID = ((ISavedSearchPortletModelRO) m_model).GetPortletID();
		String strPortletJavaScriptSubmitForm = IntrinsicPortletView.INTRINSIC_PORTLET_JAVASCRIPT_SUBMIT_FORM + nID;

		try
		{
			// JF- OnDone JavaScript function
			sb = new XPStringBuilder();
			sb.Append("function collabPopUp").Append(nID).Append
				("(url) { var x = String((screen.width - 800) / 2);    var y = String((screen.height - 550) / 2);    var dummy = window.open(url+'&popUp=true','appViewWindow','status=no,toolbar=no,location=no,menu=no,resizable=yes,scrollbars=yes,width=800,left='+x+',height=550,top='+y);    return false;  }\n");
			sb.Append("function contentPopUp").Append(nID).Append
				("(url) { var dummy = window.open(url,'contentPopupWindow','status=no,toolbar=no,location=no,menu=no,resizable=yes,scrollbars=yes,width=600,height=550'); return false;  }\n");
			sb.Append("\nfunction ").Append(JAVASCRIPT_RESULTPAGE).Append(nID).Append("()\n");
			sb.Append("{\n");
			sb.Append("\n  var respage = document.getElementById('").Append(HTMLHIDDENINPUT_RESULTPAGE).Append(nID).Append("');\n");
			sb.Append("  respage.value = 1;\n");
			sb.Append("  ").Append(strPortletJavaScriptSubmitForm).Append("(").Append(SavedSearchPortletControl.POSTTOSELF_ACTION_MORE_LINK).Append(nID).Append(");\n");
			sb.Append("}\n");

			/* parametric search submit */
			sb.Append("\nfunction ").Append(JAVASCRIPT_SEARCH).Append(nID).Append("()\n");
			sb.Append("{\n");
			sb.Append("  ").Append(strPortletJavaScriptSubmitForm).Append("(").Append(SavedSearchPortletControl.POSTTOSELF_ACTION_PARAMETRIC_SEARCH).Append(nID).Append(");\n");
			sb.Append("}\n");

			sb.Append("\nfunction ").Append(JAVASCRIPT_CLEAR_SEARCH).Append(nID).Append("()\n");
			sb.Append("{\n");
			sb.Append("  ").Append(strPortletJavaScriptSubmitForm).Append("(").Append(SavedSearchPortletControl.POSTTOSELF_ACTION_CLEAR_PARAMETRIC_SEARCH).Append(nID).Append(");\n");
			sb.Append("}\n");

			sb.Append("\nfunction ").Append(JAVASCRIPT_SORT).Append(nID).Append("(id)\n");
			sb.Append("{\n");
			sb.Append("  var sortid = document.getElementById('").Append(HTMLHIDDENINPUT_SORTID).Append(nID).Append("');\n");
			sb.Append("   if ((Math.abs(sortid.value)==id) && (sortid.value > 0))\n");
			sb.Append("   {\n");
			sb.Append("      sortid.value = -id;\n");
			sb.Append("   }\n");
			sb.Append("   else\n");
			sb.Append("   {\n");
			sb.Append("      sortid.value = id;\n");
			sb.Append("   }\n");
			sb.Append("  ").Append(strPortletJavaScriptSubmitForm).Append("(").Append(SavedSearchPortletControl.POSTTOSELF_ACTION_SORT).Append(nID).Append(");\n");
			sb.Append("}\n");

			sb.Append("\nfunction ").Append(JAVASCRIPT_PAGINATE).Append(nID).Append("(_first, _num)\n");
			sb.Append("{\n");
			sb.Append("  var num = document.getElementById('").Append(HTMLHIDDENINPUT_PAGINATE_NUM).Append(nID).Append("');\n");
			sb.Append("  num.value = _num;\n");
			sb.Append("  \n");
			sb.Append("  var first = document.getElementById('").Append(HTMLHIDDENINPUT_PAGINATE_FIRST).Append(nID).Append("');\n");
			sb.Append("  first.value = _first;\n");
			sb.Append("  \n");
			sb.Append("  ").Append(strPortletJavaScriptSubmitForm).Append("(").Append(SavedSearchPortletControl.POSTTOSELF_ACTION_PAGINATE).Append(nID).Append(");\n");
//			sb.Append("  ").Append(JAVASCRIPT_REFRESH).Append(nID).Append("();\n");
//			sb.Append("  ").Append("SendPOSTOnSpecificFormDP").Append(nID)
//				.Append("(").Append("m_asOwner.GetCurrentNormalizedBaseURL()&refresh=true;&portlet=").Append(nID).Append(", refreshElementID, RepostAction);")
//				.Append("\n");
//			sb.Append("pt-portlet-").Append(nID).Append(".refresh(); return false;");
			sb.Append("}\n");

//			if (model.m_bNeedRefresh)
//			{
//				HTMLScript inplaceRefreshScript = new HTMLScript("text/javaScript");
		
//				XPStringBuilder sb = new XPStringBuilder(100);
		
//				sb.Append("\nfunction ").Append(JAVASCRIPT_REFRESH).Append(nID).Append("() \n{\n");
//				sb.Append("  ").Append("SendPOSTOnSpecificFormDP").Append(nID).Append("('")
////				        InPlaceRefreshHelper.GetJSSendGETOnAFormDP(
//					        .Append(m_asOwner.GetCurrentNormalizedBaseURL()).Append("','")
//					        .Append("pt-portlet-content-"+nID).Append("','") 
//					        .Append(RepostControl.POSTTOSELF_ACTION_IN_PLACE_REFRESH).Append("');").Append("\n");
//				sb.Append("}\n");
//				sb.Append("\n");
//				sb.Append("document.PCC.RegisterForWindowEvent(\"onload\", \"")
//					.Append(JAVASCRIPT_REFRESH).Append(nID).Append("();\");\n");
		
//				inplaceRefreshScript.AddInnerHTMLString(sb.ToString());
		
//				myScripts.AddInnerHTMLElement(inplaceRefreshScript);
//			}
				
//			sb.Append("\nfunction SendPOSTOnSpecificFormDP").Append(nID).Append("(url, refreshElementID, repostAction)\n");
//			sb.Append("{\n");
//			sb.Append("   var myForm = document.IntrinsicPortletForm").Append(nID).Append(";\n");
//			sb.Append("   var myRepostAction = (repostAction) ? repostAction : PTInPlaceRefresh.POSTTOSELF_ACTION_IN_PLACE_REFRESH;\n");
//			sb.Append("   myForm[PTInPlaceRefresh.HTMLINPUT_POSTTOSELF].value = repostAction;\n");
//			sb.Append("   var myArgs = new Object();\n");
//			sb.Append("   myArgs[PTInPlaceRefresh.ARG_REFRESH_ELEMENT_ID] = refreshElementID;\n");
//			sb.Append("   PTInPlaceRefresh.SendPOST(url, myForm, PTInPlaceRefresh.HandleResponseReplaceInnerHTML, myArgs, repostAction);\n");
//			sb.Append("}\n");
			
			
			myScript.AddInnerHTMLString(sb.ToString());
			

			/* needed for query table */
			JSDateTimePickerMgr jsDTMgr =  ((ISavedSearchPortletModelRO)m_model).GetDatePickerMgr();
			String submitCommand = strPortletJavaScriptSubmitForm +"(" + XPConvert.ToString(IntrinsicPortletsRepostControl.POSTTOSELF_ACTION_SIMPLEREPOST) + ");\n";
			myScript.AddInnerHTMLString(DynamicQueryTable.GetJavaScript(XPConvert.ToString(jsDTMgr.GetPreFormDivID()),submitCommand,m_asOwner).GetInnerHTML());
		}
		catch (HTMLException he)
		{
			log.Warn(he, "An error occured when trying to display the JavaScript in Search portlet.");
		}

		return myScript;
	}
	
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
     */
    public HTMLElement Display() 
    {
	    /* our parent html collection */
		HTMLElementCollection result = new HTMLElementCollection();

		JSDateTimePickerMgr dtPickerMgr =  ((ISavedSearchPortletModelRO) m_model).GetDatePickerMgr();
		int nPortletID = ((ISavedSearchPortletModelRO) m_model).GetPortletID();
		result.AddInnerHTMLElement( dtPickerMgr.GetPagePreFormContent("PID"+nPortletID) );
		
		/* In place refresh div */
//		HTMLDiv refreshDiv = new HTMLDiv();
//		refreshDiv.SetID(REFRESH_ID+((ISavedSearchPortletModelRO)m_model).GetPortletID());
//		result.AddInnerHTMLElement(refreshDiv);
//		refreshDiv.AddInnerHTMLElement(mainTable);

		/* now add our parent display */
		result.AddInnerHTMLElement(super.Display());
		
        return result;
    }
    
	/**
	 * 
	 *
	 * @return 
	 */
	public HTMLElement DisplayPortlet()
	{
	    /* our parent html collection */
		HTMLElementCollection result = new HTMLElementCollection();

	    /* get our model and other variables here for convience */
		ISavedSearchPortletModelRO iModel = (ISavedSearchPortletModelRO) m_model;
		int iPortletID = iModel.GetPortletID();
		_preferPopupDocs = GetPopupSetting();
		
		/* try to get necessary snapshot query objects from model here */
	    IPTSearchResponse sr = null;
		int srCnt = 0;
		boolean bSubscribed = false;
		boolean bSubscriptionEnabled = false;
		boolean bIsParameterized = false;
		boolean bHasParameterizedResults = false;
		String sErrorMsg = "";
		
		try
		{
		    /* get the results */
		    try
		    {
		        sr = GetSnapshotQueryResponse();
	        }
	        catch (XPException e)
	        {
				SearchExecutionException eprime = SearchUIUtility.GetI18NException(e, m_asOwner);
				if (eprime instanceof QueryParseException)
				{
				    sErrorMsg = m_asOwner.GetString(75, ResourceConstants.PORTLETSMSGS);
				}
				else if (!SearchUIUtility.ExceptionHiddenInPageSource(eprime))
				{
					sErrorMsg = eprime.GetMessage();
				}
				else
				{
				    throw e;
				}
	        }

		    if (null!=sr)
		    {
		        srCnt = sr.GetResultsReturned();
		    }
		    
		    /* get subscription options */
			bSubscriptionEnabled = iModel.IsSubscriptionEnabled();
			if (bSubscriptionEnabled)
			{
				bSubscribed = iModel.IsUserSubscribed();
			}
			
			/* now get our parameteric (search) options */
			bIsParameterized = iModel.IsParameterized();
			if (bIsParameterized)
			{
			    bHasParameterizedResults = iModel.HasParametericResults();
			}
		}
		catch (Exception e)
		{
			HTMLSpan errspan = new HTMLSpan();
			errspan.SetStyleClass(PTStyleClass.LIST_TEXT);

			errspan.AddInnerHTMLString(m_asOwner.GetString(564, "ptmsgs_portalcommonmsgs"));
			result.AddInnerHTMLElement(errspan);

			return result;
		}

		
		/* lets try to build our UI */
		try
		{
		    /* did the original results return 0? */
			if ((0 == srCnt) && !bHasParameterizedResults)
			{
				HTMLSpan errspan = new HTMLSpan();
				errspan.SetStyleClass(PTStyleClass.LIST_TEXT);

				errspan.AddInnerHTMLString(m_asOwner.GetString(563, "ptmsgs_portalcommonmsgs"));
				result.AddInnerHTMLElement(errspan);

				return result;
			}

			/* ok, either we have results or our parameterized results didn't return anything
			 * in either case, build our table, show the search form and then either show the 
			 * results or inform the user that her parameterized results didn't return anything
			 * this will allow the user a chance to do a new search or reset to the original results.
			 */

		    /* our main table */
			HTMLTable mainTable = new HTMLTable();
			result.AddInnerHTMLElement(mainTable);
			mainTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
			mainTable.SetBorder(CommonHTMLStrings.ZERO);
			mainTable.SetCellPadding(CommonHTMLStrings.FIVE);
			mainTable.SetCellSpacing(CommonHTMLStrings.ZERO);

			/* if we support parameterized search, then display the search form */
			if (bIsParameterized)
			{
			    AddParametericHTMLRow(mainTable);
			}

			/* did the user's parameteric search not return anything? */
			if ((0 == srCnt) && bHasParameterizedResults)
			{
				HTMLSpan errspan = new HTMLSpan();
				result.AddInnerHTMLElement(errspan);
				errspan.SetStyleClass(PTStyleClass.LIST_TEXT);
				
				if (sErrorMsg.length()>0)
				{
				    errspan.AddInnerHTMLString(sErrorMsg);
				    errspan.AddInnerHTMLElement(new HTMLBr());
				}
				else
				{
				    errspan.AddInnerHTMLString(m_asOwner.GetString(563, "ptmsgs_portalcommonmsgs"));
				}

				errspan.AddInnerHTMLElement(new HTMLBr());

				HTMLAnchor resetLink = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
				errspan.AddInnerHTMLElement(resetLink);
				resetLink.SetStyleClass(PTStyleClass.LIST_PAGINATION_TEXT);
				resetLink.SetOnClick(JAVASCRIPT_CLEAR_SEARCH + iPortletID + "(); return false;");
				resetLink.AddInnerHTMLString(m_asOwner.GetString(71, ResourceConstants.PORTLETSMSGS));

				return result;
			}

			
			/* ok, we have results, let's show them! */

			
			// we use the image generation stuff from search
			SearchResultModel srm = new SearchResultModel();
			srm.ExternalLightInitialization(sr, m_asOwner);

			/* use this for tab spaces */
			String tab = "&nbsp;&nbsp;&nbsp";

			
			/* some hidden elements needed for form posts */
			String sHiddenValue = HTMLHIDDENINPUT_RESULTPAGE + iPortletID;
			HTMLInput hidden = new HTMLInput(HTMLInputTypes.HIDDEN, sHiddenValue, "");
			result.AddInnerHTMLElement(hidden);
			hidden.SetValue("0");
			hidden.SetID(sHiddenValue);
			
			sHiddenValue = HTMLHIDDENINPUT_SORTID + iPortletID;
			hidden = new HTMLInput(HTMLInputTypes.HIDDEN, sHiddenValue, "");
			result.AddInnerHTMLElement(hidden);
			hidden.SetValue(XPConvert.ToString(((ISavedSearchPortletModelRO)m_model).GetSortOrder()));
			hidden.SetID(sHiddenValue);

			/* for pagination */
			sHiddenValue = HTMLHIDDENINPUT_PAGINATE_FIRST + iPortletID;
			hidden = new HTMLInput(HTMLInputTypes.HIDDEN, sHiddenValue, "");
			result.AddInnerHTMLElement(hidden);
			hidden.SetValue("");
			hidden.SetID(sHiddenValue);
			
			sHiddenValue = HTMLHIDDENINPUT_PAGINATE_NUM + iPortletID;
			hidden = new HTMLInput(HTMLInputTypes.HIDDEN, sHiddenValue, "");
			result.AddInnerHTMLElement(hidden);
			hidden.SetValue("");
			hidden.SetID(sHiddenValue);
			
			mainTable.AddInnerHTMLElement(GetSpacerRow(PTStyleClass.OBJECT_TEXT, "1"));

			
		    /* main table row & cell */
		    HTMLTableRow mainTableRow = new HTMLTableRow();
			mainTable.AddInnerHTMLElement(mainTableRow);
			HTMLTableCell mainTableCell = new HTMLTableCell();
			mainTableRow.AddInnerHTMLElement(mainTableCell);
			
			/* innerTable with cells for sortable fields and rows for non-sortable fields */
			HTMLTable innerTable = new HTMLTable();
			mainTableCell.AddInnerHTMLElement(innerTable);
			innerTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
			innerTable.SetBorder(CommonHTMLStrings.ZERO);
			innerTable.SetCellPadding(CommonHTMLStrings.ONE);
			innerTable.SetCellSpacing(CommonHTMLStrings.ZERO);

			/* first row is our sort/title row */
		    HTMLTableRow sortRow = new HTMLTableRow();
			sortRow.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);
			HTMLTableCell spacerCell = new HTMLTableCell();
			sortRow.AddInnerHTMLElement(spacerCell);
			spacerCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
			
			/* loop thru the returned properties and add the prop name to the header row */
			Object[][] arRetPropInfo = iModel.GetReturnedPropertyInfo();
			if (0==arRetPropInfo.length)
			{
				HTMLSpan errspan = new HTMLSpan();
				result.AddInnerHTMLElement(errspan);
				
				errspan.SetStyleClass(PTStyleClass.LIST_TEXT);
				errspan.AddInnerHTMLString(m_asOwner.GetString(69, ResourceConstants.PORTLETSMSGS));

				return errspan;
			}
			
			int nColCnt = 0; /* how many columns each row should contain */
//			boolean bHasPropsToDisplay = false;
			
			for (int i=0; i<arRetPropInfo.length; i++)
		    {
			    /* this occurs when a property does not exist or is not accessible */
			    if (null==arRetPropInfo[i][SavedSearchPortletModel.PROPINFO_INDEX_ID])
			    {
			        continue; 
			    }

			    String id = XPConvert.ToString(arRetPropInfo[i][SavedSearchPortletModel.PROPINFO_INDEX_ID]);
			    String propname = XPConvert.ToString(arRetPropInfo[i][SavedSearchPortletModel.PROPINFO_INDEX_NAME]);
			    int type = XPConvert.ToInteger(arRetPropInfo[i][SavedSearchPortletModel.PROPINFO_INDEX_TYPE]);
			    String sRowOrCol = XPConvert.ToString(arRetPropInfo[i][SavedSearchPortletModel.PROPINFO_INDEX_ORIENT]);
			    
			    if (XPStringUtility.EqualsIgnoreCase(SavedSearchPortletModel.PROPINFO_ORIENT_COL, sRowOrCol))
			    {
			        HTMLTableCell titleCell = new HTMLTableCell();
			        sortRow.AddInnerHTMLElement(titleCell);
			        titleCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);

			        if (IsSortable(id,type))
			        {
						/* sorting anchor */
						HTMLAnchor sortAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL); 
						titleCell.AddInnerHTMLElement(sortAnchor);
//	REMOVING THE SPECIAL TREATMENT OF NAME SINCE WE CAN NOW DO THE TRUE LEXICOGRAPHIC SORTING					
//						/* name field is *special*. we don't actually sort by the name property, 
//						 * but by the nameindex property */
//						if (PT_INTRINSICS.PT_PROPERTY_OBJECTNAME == XPConvert.ToInteger(id))
//						{
//						    sortAnchor.SetOnClick(JAVASCRIPT_SORT + iPortletID + "('" + PT_INTRINSICS.PT_PROPERTY_NAMEINDEX + "'); return false;");
//						}
//						else
//						{
						    sortAnchor.SetOnClick(JAVASCRIPT_SORT + iPortletID + "('" + id + "'); return false;");
//						}
						
						sortAnchor.SetTitle(propname);
						sortAnchor.AddInnerHTMLEncodedString(propname);
			        }
			        else
			        {
			            titleCell.AddInnerHTMLEncodedString(propname);
			        }
			        
			        nColCnt++;
			    }
			    
//			    bHasPropsToDisplay = true;
		    }
			
			/* if we cannot display any of the properties, then display an error message.
			 * 
			 * NOTE: i am not sure if we should display an error here or not. 
			 *  we could just let this condition pass, in which case results will still
			 *  be displayed with a bunch of rows listed with icons but no text (which looks very similiar 
			 *  to a portlet that has properties but no values for any of them).  for now, i will let this
			 *  condition pass, but i might change my mind (or have my mind changed for me) later. */
//			if (!bHasPropsToDisplay)
//			{
//				HTMLSpan errspan = new HTMLSpan();
//				errspan.SetStyleClass(PTStyleClass.LIST_TEXT);
//
//				errspan.AddInnerHTMLString(m_asOwner.GetString(564, "ptmsgs_portalcommonmsgs"));
//				
//				result = new HTMLElementCollection();				
//				result.AddInnerHTMLElement(errspan);
//
//				log.Error("There are no display properties available for this user.");
//				return result;
//			}

			/* if we have columns (i.e. props), then add the sort row to the table */
			if (nColCnt>0)
			{
				innerTable.AddInnerHTMLElement(sortRow);
			}
			
			/* we'll use this colcnt to know how many columns each row should contain */
			String colcnt = XPConvert.ToString(nColCnt+1); /* plus a cell for icon/row title */
			
			/* we'll use this to format our date values */
			XPLocale xpLocale = PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(m_asOwner);
			XPTimeZone xpTimeZone = PTPersonalSettingsHelper.GetPersonalSettingValue_TimeZone(m_asOwner);

			XPDateFormat dateFormat = XPDateFormat.GetDateTimeInstance(
				XPDateFormat.SHORT, XPDateFormat.SHORT, 
				xpLocale, xpTimeZone);

			/* loop thru each item result and print out the item's properties. */ 
			for (int i = 0; i < srCnt; i++)
			{
			    /* new row for new item */
			    HTMLTableRow itemRow = new HTMLTableRow();
				innerTable.AddInnerHTMLElement(itemRow);

				/* alternate colors */
				PTStyleClass style = (0 == (i % 2))? PTStyleClass.LIST_ITEM_TWO_BG : PTStyleClass.LIST_ITEM_ONE_BG; 
				itemRow.SetStyleClass(style);

				/* image cell for object type */
				HTMLTableCell imageCell = new HTMLTableCell();
				itemRow.AddInnerHTMLElement(imageCell);
				imageCell.SetHeight(ITEMSIZES);
				imageCell.SetWidth(ITEMSIZES);
				imageCell.SetAlign(CommonHTMLStrings.CENTER);
				imageCell.SetVAlign(CommonHTMLStrings.CENTER);
				imageCell.SetStyleClass(PTStyleClass.LIST_TEXT);
				
				/* image icon & anchor */
				HTMLAnchor iconAnchor = CreateSnapshotClickThruURL(srm, i, iPortletID);
				imageCell.AddInnerHTMLElement(iconAnchor);
			    HTMLImg icon = (HTMLImg) srm.GetResultIcon(i);
				iconAnchor.AddInnerHTMLElement((HTMLElement) icon);
				icon.SetAlign(CommonHTMLStrings.CENTER);
				icon.SetBorder(CommonHTMLStrings.ZERO);
			    
				/* if colcnt is 0 then we have all rows, but we need atleast one more col for the row value
				 * i.e. description: value */
				if (nColCnt==0) 
				{
			        HTMLTableCell spaceCell = new HTMLTableCell();
					itemRow.AddInnerHTMLElement(spaceCell);
					spaceCell.SetStyleClass(PTStyleClass.LIST_TEXT);
			        spaceCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
				 }

				/* loop this item's properties and display thier values */ 
			    for (int j=0; j<arRetPropInfo.length; j++)
			    {
				    /* this occurs when a property does not exist or is not accessible */
				    if (null==arRetPropInfo[j][SavedSearchPortletModel.PROPINFO_INDEX_ID])
				    {
				        continue; 
				    }

				    int id = XPConvert.ToInteger(arRetPropInfo[j][SavedSearchPortletModel.PROPINFO_INDEX_ID]);
				    int type = XPConvert.ToInteger(arRetPropInfo[j][SavedSearchPortletModel.PROPINFO_INDEX_TYPE]);
				    String sRowOrCol = XPConvert.ToString(arRetPropInfo[j][SavedSearchPortletModel.PROPINFO_INDEX_ORIENT]);

				    String value = null;
				    
				    /* if this is date try to get string format of it */
				    if (type==PT_PROPERTY_TYPES.PT_PROPTYPE_DATE)
				    {
				        value = GetDateTimeValueAsString(sr, i, id, dateFormat);
				    }
					/* props are actually floats, not doubles */
					else if (type == PT_PROPERTY_TYPES.PT_PROPTYPE_DOUBLE)
					{
					    Object oValue = sr.GetFieldsAsObject(i, XPConvert.ToInteger(id));
					    value = (null!=oValue)? 
					            XPConvert.ToString(XPConvert.ToFloat(oValue)) :
					            null;
					}
				    else /* else, just treat it as a string */
				    {
					    value = XPConvert.ToString(sr.GetFieldsAsObject(i, XPConvert.ToInteger(id)));
				    }

				    /* if we couldn't get a value or it didn't exist, just display a blank */
				    if (null == value)
					{
					    value = CommonHTMLStrings.SPACE;
					}
					else /* we have a value so html encode it */
					{
					    value = XPHttpUtility.HtmlEncode(value);
					}
					
				    /* we have a value (or blank).  it will be displayed either as a cell to
				     * the topmost row, or a new row under the "cell" fields */
			        HTMLTableCell valueCell = new HTMLTableCell();
					valueCell.SetStyleClass(PTStyleClass.LIST_TEXT);
				    if (XPStringUtility.EqualsIgnoreCase(SavedSearchPortletModel.PROPINFO_ORIENT_COL, sRowOrCol)/*IsSortable(id,type)*/) /* will be added as a cell */
				    {
				        if (id==PT_INTRINSICS.PT_PROPERTY_OBJECTNAME) /* TODO: what if name is not a displayable field? name field is *special*. it's the anchor */
				        {
							/* name cell */
					        itemRow.AddInnerHTMLElement(valueCell);
					        	
							/* name anchor */
							HTMLAnchor nameAnchor = CreateSnapshotClickThruURL(srm, i, iPortletID);
							valueCell.AddInnerHTMLElement(nameAnchor);
							nameAnchor.SetTitle(value);
							nameAnchor.AddInnerHTMLString(value);
				        }
				        else
				        {
							itemRow.AddInnerHTMLElement(valueCell);
					        valueCell.AddInnerHTMLString(value);
				        }
				    }
				    else /* will be added as a new row */
				    {
					    HTMLTableRow nosortRow = new HTMLTableRow();
						innerTable.AddInnerHTMLElement(nosortRow);
						nosortRow.SetStyleClass(style);
						
						HTMLTableCell titleCell = new HTMLTableCell();
						nosortRow.AddInnerHTMLElement(titleCell);
						titleCell.SetStyleClass(PTStyleClass.LIST_ITALIC_TEXT);
					    String propname = XPConvert.ToString(arRetPropInfo[j][SavedSearchPortletModel.PROPINFO_INDEX_NAME]);
					    titleCell.AddInnerHTMLEncodedString(propname+":");
						
				        if (id==PT_INTRINSICS.PT_PROPERTY_OBJECTNAME) /* TODO: what if name is not a displayable field? name field is *special*. it's the anchor */
				        {
							/* name cell */
				            nosortRow.AddInnerHTMLElement(valueCell);
					        	
							/* name anchor */
							HTMLAnchor nameAnchor = CreateSnapshotClickThruURL(srm, i, iPortletID);
							valueCell.AddInnerHTMLElement(nameAnchor);
							nameAnchor.SetTitle(value);
							nameAnchor.AddInnerHTMLString(value);
				        }
				        else
				        {
					        nosortRow.AddInnerHTMLElement(valueCell);
					        valueCell.SetColSpan(colcnt);
					        valueCell.AddInnerHTMLString(value);
				        }
				    }
			    }
			}

			/* this last row contains the subscription button, clear link, & more link */
			HTMLTableRow lastRow = new HTMLTableRow();
			mainTable.AddInnerHTMLElement(lastRow);
			lastRow.SetStyleClass(PTStyleClass.LIST_ITEM_TWO_BG);

			HTMLTableCell lastCell = new HTMLTableCell();
			lastRow.AddInnerHTMLElement(lastCell);
			lastCell.SetColSpan(colcnt);
			lastCell.SetStyleClass(PTStyleClass.LIST_TEXT);
			lastCell.SetAlign("right");
			lastCell.SetHeight(ITEMSIZES);

			/* subscription stuff */
			if (bSubscriptionEnabled)
			{
				// after the table, we want to provide as subscription link...
				String strPortletJavaScriptSubmitForm = IntrinsicPortletView.INTRINSIC_PORTLET_JAVASCRIPT_SUBMIT_FORM + iPortletID;

				HTMLInput myButton = new HTMLInput(HTMLInputTypes.BUTTON, HTMLINPUT_SUBSCRIBE + iPortletID, m_asOwner.GetString(560, "ptmsgs_portalcommonmsgs"));
				lastCell.AddInnerHTMLElement(myButton);
				myButton.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
				myButton.SetOnClick(strPortletJavaScriptSubmitForm + "(" + SavedSearchPortletControl.POSTTOSELF_ACTION_CHANGE_SUBSCRIPTION + iPortletID + ");");

				/* get the respost value depending on whether the use has subscribed yet */
				String strRepostValue = bSubscribed? 
				        XPConvert.ToString(SavedSearchPortletControl.POSTTOSELF_ACTION_UNSUBSCRIBE) :
				        XPConvert.ToString(SavedSearchPortletControl.POSTTOSELF_ACTION_SUBSCRIBE);
				 
				/* ditto the text the display */
				myButton.SetValue(m_asOwner.GetString(bSubscribed?556:555, "ptmsgs_portalcommonmsgs"));

				/* we need this to if are trying to subscribe or unsubscribe */
				HTMLInput MailID = new HTMLInput(HTMLInputTypes.HIDDEN, HTMLHIDDENINPUT_MAILID + iPortletID, "");
				MailID.SetValue(strRepostValue);
				result.AddInnerHTMLElement(MailID);

				/* give us a little breathing room, huh? */
				lastCell.AddInnerHTMLString(tab);
			}

			/* Reset link stuff */
			if (iModel.HasParametericResults())
			{
				HTMLAnchor resetLink = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
			    lastCell.AddInnerHTMLElement(resetLink);
				resetLink.SetStyleClass(PTStyleClass.LIST_PAGINATION_TEXT);
				resetLink.SetOnClick(JAVASCRIPT_CLEAR_SEARCH + iPortletID + "(); return false;");
				resetLink.AddInnerHTMLString(m_asOwner.GetString(70, ResourceConstants.PORTLETSMSGS));
			    lastCell.AddInnerHTMLString(tab);
			}
			
			/* show the more link if have cached results, otherwise, use pagination */
			/* bug 30485: don't show more... link in preview */
			if (!m_asOwner.GetName().equals(PreviewPortletAS.STR_MVC_CLASS_NAME))
			{
				/* BT: Bug 54598 - Get the page size and actual result size so we can
				 * we can determine whether we need to display the more link. We don't
				 * need to display the "more..." link or pagination if results can fit
				 * all on the same page.
				 */
				int nPageSize   = ((IPaginationRO) m_model).GetItemsPerPage();
				int nTotMatches = ((ISavedSearchPortletModelRO) m_model).GetTotalMatches();
				
				if (nTotMatches >= nPageSize)
				{
					if (!iModel.HasParametericResults())
					{
					    HTMLAnchor myAnchor; 
					 
					    /* if pagination is enabled, then the more link will conduct a 
					     * followup with the original qry which will "turn" on parameteric results
					     * and pagination */
						if (iModel.IsPaginationEnabled())
						{
							myAnchor = MakeURL(m_asOwner.GetString(562, "ptmsgs_portalcommonmsgs"), 1, 0);
						}
						else
						{
						    /* more link to adv search */
							myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
							myAnchor.SetOnClick(JAVASCRIPT_RESULTPAGE + iPortletID + "(); return false;");
							myAnchor.AddInnerHTMLString(m_asOwner.GetString(562, "ptmsgs_portalcommonmsgs"));
					    }
						
						lastCell.AddInnerHTMLElement(myAnchor);
						myAnchor.SetStyleClass(PTStyleClass.LIST_PAGINATION_TEXT);
					}
					else
					{
					    lastCell.AddInnerHTMLElement(DisplayPagination());
					}
				}
			}
		}
		catch (Exception e)
		{
			log.Error(e, "SavedSearchPortletView::Display. " + m_asOwner.GetString(558, "ptmsgs_portalcommonmsgs"));
			throw new XPException(m_asOwner.GetString(558, "ptmsgs_portalcommonmsgs"), e);
		}

		return result;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.intrinsicportlets.IntrinsicPortletView#DisplayPortletWML()
	 */
	public String DisplayPortletWML()
	{
		return null;
	}

	private HTMLTableRow GetSpacerRow(PTStyleClass _style, String _colspan)
	{
	    HTMLTableRow spacerRow = new HTMLTableRow();
		spacerRow.SetStyleClass(_style);
		
	    HTMLTableCell spacerCell = new HTMLTableCell();
		spacerRow.AddInnerHTMLElement(spacerCell);
		spacerCell.SetHeight("10");
		spacerCell.SetColSpan(_colspan);
		
		return spacerRow;
	}
	
	private IPTSearchResponse GetSnapshotQueryResponse()
	{
	    IPTSearchResponse sr = null;
	    
	    if (((ISavedSearchPortletModelRO) m_model).HasParametericResults())
	    {
            sr = ((ISavedSearchPortletModelRO) m_model).GetFollowupResponse();
	    }
	    else
	    {
	        sr = ((ISavedSearchPortletModelRO) m_model).GetSearchResponseObject();
	    }
	    
	    return sr;
	}
	
	private void AddParametericHTMLRow(HTMLTable _parentTable)
	{
	    /* row to hold search form */
	    HTMLTableRow searchRow = new HTMLTableRow();
	    _parentTable.AddInnerHTMLElement(searchRow);
	    HTMLTableCell searchCell = new HTMLTableCell();
		searchRow.AddInnerHTMLElement(searchCell);
		searchCell.SetColSpan("2");
		searchRow.SetStyleClass(PTStyleClass.CUSTOMAPP_LT_GRAY_BG);
	    searchCell.SetStyleClass(PTStyleClass.CUSTOMAPP_BORDER);

	    HTMLDiv myDiv = new HTMLDiv();
	    searchCell.AddInnerHTMLElement(myDiv);
	    myDiv.SetStyleClass(PTStyleClass.CUSTOMAPP_SUBHEAD);
	    myDiv.AddInnerHTMLEncodedString(m_asOwner.GetString(77, ResourceConstants.PORTLETSMSGS));

	    /* create a collection that we will add the filterform to */
	    HTMLElementCollection parametericResult = new HTMLElementCollection();
	    searchCell.AddInnerHTMLElement(parametericResult);

	    
	    /* get our model & portletid for convience here */
	    ISavedSearchPortletModelRO iModel = (ISavedSearchPortletModelRO) m_model;
		int iPortletID = iModel.GetPortletID();

		
	    /* add the datepicker js */
	    parametericResult.AddInnerHTMLElement(iModel.GetDatePickerMgr().GetJSIncludes());

		JSDateTimePickerMgr jsDTMgr =  iModel.GetDatePickerMgr();
		String jsdtDivID = jsDTMgr.GetPreFormDivID();
	    
	    //  FilterForm.BuildFilterFromForm() function relies on in_he_thisIsAFilterForm
	    //  to determine if it's actually processing the right form data.  
	    HTMLInput he_FilterFormSettings = new HTMLInput(HTMLInputTypes.HIDDEN, FilterForm.IS_FILTER_FORM+jsDTMgr.GetPreFormDivID(), "");
	    he_FilterFormSettings.SetValue("true");
		searchCell.AddInnerHTMLElement(he_FilterFormSettings);
		

		/* create query (search) table */
		IFilterQueryTable qryTable = new DynamicQueryTable("1", 0, m_asOwner);
		qryTable.SetAllMustPass(true);
		qryTable.SetTableSize(1); /* should be number of parameteric fields we have */

		/* add hidden elements needed for form to function */
		DynamicQueryTable.AddGlobalElements(jsdtDivID, "1" /*the active group*/, 1, parametericResult);

		/* need this for querycondition html */
		String strPortletFormName = IntrinsicPortletView.INTRINSIC_PORTLET_HTMLFORM_NAME + iPortletID;

		/* get a filter iterator with all our parameteric fields 
		 * and then add it's query condition to our html results */
		FilterIterator filter = iModel.GetParametericFilterIterator();

		QueryCondition myQryCondition = filter.MakeQueryCondition();
		myQryCondition.SetFormName(strPortletFormName);
		myQryCondition.SetDivID(jsdtDivID); /* see bug 41702 */
		
		/* query table always needs to have one blank condition */
		QueryCondition dummy = myQryCondition.MakeDuplicate();
		qryTable.SetBlankQueryCondition(dummy);
		
		/* the follow code would print out one input row per property
		 * so that we have a more traditional search box. however, the 
		 * filter UI defaults the datetime field and blanks are also 
		 * submitted.  We can have FilterForm ignore the blanks, but 
		 * the dates are still an issue.  Hence, we will keep the 
		 * search from as a dynamic one-row search form. */
		Object[][] arPropInfo = myQryCondition.GetPropInfo();
//		for (int i=0; i<arPropInfo.length; i++)
//		{
//		    QueryCondition qcSingle = new QueryCondition(m_asOwner);
//		    
//		    qcSingle.AddProperty(
//		            XPConvert.ToString(arPropInfo[i][QueryCondition.IDX_PROPINFO_NAME]), 
//		            XPConvert.ToInteger(arPropInfo[i][QueryCondition.IDX_PROPINFO_ID]), 
//		            XPConvert.ToInteger(arPropInfo[i][QueryCondition.IDX_PROPINFO_TYPE]));
//		    
//		    /* hack to prepopulate each property row. the filterops and value are inconsequential */
//		    qcSingle.SetSelections(
//		            XPConvert.ToInteger(arPropInfo[i][QueryCondition.IDX_PROPINFO_ID]),
//		            PT_FILTEROPS.PT_FILTEROP_CONTAINS,"");
//
//		    qcSingle.SetFormName(strPortletFormName);
////		    qcSingle.SetDivID("dpDiv"+iModel.GetPortletID());
//
//			qryTable.loadDefinedQueryCondition(qcSingle);
//		}
		qryTable.loadDefinedQueryCondition(myQryCondition);
	    /* hack to prepopulate each property row. the filterops and value are inconsequential */
		myQryCondition.SetSelections(
		        XPConvert.ToInteger(arPropInfo[0][QueryCondition.IDX_PROPINFO_ID]),
	            PT_FILTEROPS.PT_FILTEROP_CONTAINS,"");

		/* another hack: FilterForm expects to see group operator.  We don't have
		 * groups in this ui (it's a search from, not a filter per say */
		HTMLDiv grpOperatorDiv = new HTMLDiv();
		parametericResult.AddInnerHTMLElement(grpOperatorDiv);

		HTMLInput andRB = new HTMLInput(HTMLInputTypes.RADIO, "in_ra_groupoperator_"+jsdtDivID+"1", "1");
		grpOperatorDiv.AddInnerHTMLElement(andRB);
		andRB.SetID(1 + "_and");
		andRB.SetValue("and");
		andRB.SetChecked(true);
		grpOperatorDiv.SetStyle("display: none;");
		
		
		/* make our table look less-filter UI-ish */
		qryTable.SetHideGroupHeaderRow(true);
		qryTable.SetSuppressCheckbox(true);

		// add the table to our collection
		((DynamicQueryTable)qryTable).addTableToCollection(
		        jsdtDivID,
		        parametericResult, 
		        false, 
		        iModel.GetDatePickerMgr(), 
		        iModel.GetPageMgr(),
		        false);

		/* add search button */
		HTMLInput bnSearchBtn = new HTMLInput(HTMLInputTypes.BUTTON, HTMLINPUT_SEARCH+iPortletID, m_asOwner.GetString(74, ResourceConstants.PORTLETSMSGS));
		searchCell.AddInnerHTMLElement(new HTMLBr());
		searchCell.AddInnerHTMLElement(bnSearchBtn);
		bnSearchBtn.SetValue(m_asOwner.GetString(74, ResourceConstants.PORTLETSMSGS));
		bnSearchBtn.SetOnClick(JAVASCRIPT_SEARCH+iPortletID+"()");
		bnSearchBtn.SetStyleClass(PTStyleClass.OBJECT_BTN);
	}
	
	private boolean GetPopupSetting()
	{
	    boolean popup = false;
	    
		IPTSessionInfo info = ((IPTSession) m_asOwner.GetUserSession()).GetSessionInfo();
		Object[][] setting = (Object[][]) info.LookupPreference(PersonalSettingNames.OPEN_DOC_NEW_WINDOW, 0);

		if ((null != setting) && (null != setting[1][0])) 
		{
		    popup = XPConvert.ToString(setting[1][0]).equals("1");
		}
		
		return popup;		
	}
	
	private boolean IsSortable(String id, int type)
	{
	    return 
	    	(PT_PROPERTY_TYPES.PT_PROPTYPE_DATE == type) || (PT_PROPERTY_TYPES.PT_PROPTYPE_DOUBLE == type) ||
	        (PT_PROPERTY_TYPES.PT_PROPTYPE_LONG == type) || id.equals("1"); // name too!
	}

	private HTMLAnchor CreateSnapshotClickThruURL(SearchResultModel srm, int resultNum, int PortletID)
	{

	    String app  = srm.GetResultApp(resultNum);
		String strID = XPConvert.ToString(PortletID);
	    // Make a clickthru URL.
	
	    HTMLAnchor clickthru = null;
	
	    if (app.equals(SearchResultModel.TEMPLATING_APP)) {
	      // Special case: for Community Templating items, clickthrough should go to
	      // a popup with a list of communities on which the associated portlet is
	      // used, as in portlet preview.
	
              int[] portletIDs = srm.GetResultContentPortletIDs(resultNum);

              int portletId = 0;

              // "Templating" items were only indexed by pre-6.0 versions of Content Server
              // (although they could potentially still be in a search index), which would
              // associate only one portlet with each item.  So, we're basically guaranteed
              // to have exactly one item in the portletIDs array, but we might as well
              // be safe:
              if ((null != portletIDs) && (portletIDs.length > 0)) {
                portletId = portletIDs[0];
              }
	
	      ASURL templatingURL = new ASURL(m_asOwner);
	      templatingURL.SetLinkCreateNewSpace(PreviewPortletAS.STR_MVC_CLASS_NAME,
	                                          m_asOwner);
	      templatingURL.SetDisplayPage(InvitePortletJSDP.STR_MVC_CLASS_NAME);
	      templatingURL.SetControl(PreviewPortletControl.STR_MVC_CLASS_NAME);
	      templatingURL.AddControlArgument(PreviewPortletControl.m_PreviewType,
	          PreviewPortletModel.SEARCHHEADER);
	      templatingURL.AddControlArgument(PreviewPortletControl.m_strPortletParam,
	          XPConvert.ToString(portletId));
	      templatingURL.AddControlArgument(PreviewPortletControl.m_strAlwaysCommunity, "y");
	
	      clickthru = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
	      clickthru.SetOnClick("contentPopUp" + strID + "('" +	// mj: removed 'javascript:' from function name
	                           templatingURL.GetURLAsString() + "'); return false;");
	    } else {
	      // Otherwise, just use the standard clickthrough
	      clickthru = srm.GetResultClickthroughURL(resultNum, true);      
	    }
	
	    String absType = srm.GetResultAbsType(resultNum);
	
	    // Collab documents always appear in a new window.
	    // Other docs appear in a popup if a preference is set (we've already
	    // initialized _preferPopupDocs with the value of this pref).
	
	    // If it's a document
	    if ((null != absType) && 
	        (absType.equals(SearchResultModel.DOCUMENT_ABSTYPE)))
	    {
	
			  if (_preferPopupDocs ||
			      app.equals(SearchResultModel.COLLAB_APP))
			  {
			
			    // set target=_blank
				clickthru.SetTarget("_blank");
			  }
	    }
	    else
	    {
	      if (app.equals(SearchResultModel.COLLAB_APP)) {
	
	        // For non-document Collab items, use javascript to open a popup
	
	        // Get the original gatewayed href
	        String clickthruURL = clickthru.GetHRef();
	        clickthru.SetHRef(ConstructStaticHelpers.BLANK_URL);
	        clickthru.SetOnClick("collabPopUp" + strID + "('" + clickthruURL + "'); return false;");	// mj: removed 'javascript:' from function name
	      }
	    }
	
	    return clickthru;
	}
	
	/**
	 *  Returns an HTMLElement for pagination of the results table.
	 */
	private HTMLElement DisplayPagination()
	{
		HTMLElementCollection group = new HTMLElementCollection();
		HTMLSpan span = new HTMLSpan();
		span.SetStyleClass(PTStyleClass.OBJECTSEARCH_TEXT);
		group.AddInnerHTMLElement(span);

		final int PAGES_TO_DISPLAY = 10;

		try
		{
			String sortOrder = ((IPaginationRO) m_model).GetSortOrder();

			int currFirst  = ((IPaginationRO) m_model).GetFirstDisplayableItem();
			int totResults = ((IPaginationRO) m_model).GetTotalItems();
			int pageSize   = ((IPaginationRO) m_model).GetItemsPerPage();

			

			boolean shownOneItem = false;

			// If there is only one page
			if (totResults <= pageSize)
			{
				// Don't bother
				return group;
			}

			// Add a space at the beginning so we don't run into a cell border
			span.AddInnerHTMLString(CommonHTMLStrings.SPACE);

			// First, check whether we can display a "previous" link.
			int prevFirst = 0;

			if (currFirst > pageSize)
			{
				// we're at least one page beyond the beginning;
				// we can back up one page
				prevFirst = currFirst - pageSize;
			}
			else if (currFirst > 1)
			{
				// odd case: we're partway thru the first page.
				// how did we get here?
				// display a link to go back to the first result
				prevFirst = 1;
			}

			// If we can display a "previous" link
			if (0 < prevFirst)
			{
				// Display "prev" link
				span.AddInnerHTMLElement(
				        MakeURL(m_asOwner.GetString(207, "ptmsgs_portalinfrastructure"), prevFirst, pageSize));
			}
			else
			{
				// Display a nonclickable link
				span.AddInnerHTMLString(m_asOwner.GetString(207, "ptmsgs_portalinfrastructure"));
			}

			shownOneItem = true;

			int currPage = (currFirst - 1) / pageSize;

			int firstPageToDisplay = (currPage / PAGES_TO_DISPLAY); // integer divide!
			firstPageToDisplay = (firstPageToDisplay * PAGES_TO_DISPLAY) - 1;

			int lastPageToDisplay = firstPageToDisplay + PAGES_TO_DISPLAY + 1;

			for (int page = firstPageToDisplay; page <= lastPageToDisplay; page++)
			{
				if (page < 0)
				{
					continue;
				}

				int firstResultInPage = (page * pageSize) + 1;
				int lastResultInPage  = (firstResultInPage + pageSize) - 1;

				// If we're over the total number of results
				if (firstResultInPage > totResults)
				{
					// we're all done
					break;
				}

				// If this is the last page
				if (lastResultInPage > totResults)
				{
					// show the exact number of results we'll be displaying
					lastResultInPage = totResults;
				}

				String linkText = "...";

				// If we're not showing the first or last link
				if ((page > firstPageToDisplay) && (page < lastPageToDisplay))
				{
					// show the actual result range
					linkText = XPConvert.ToString(firstResultInPage) + "-" +
						XPConvert.ToString(lastResultInPage);
				}

				if (shownOneItem)
				{
					span.AddInnerHTMLString("&nbsp;|&nbsp;");
				}

				shownOneItem = true;

				// If this isn't the current page
				if (page != currPage)
				{
					// make a link
					span.AddInnerHTMLElement(MakeURL(linkText, firstResultInPage, pageSize));
				}
				else
				{
					// else make a nonclickable thing
					span.AddInnerHTMLString(linkText);
				}
			}

			// Finally, handle the "next" link
			// Calculate start of next page
			int nextFirst = currFirst + pageSize;

			// Check that "next" won't go beyond the end of
			// the result set
			if (nextFirst > totResults)
			{
				nextFirst = 0;
			}

			if (shownOneItem)
			{
				span.AddInnerHTMLString("&nbsp;|&nbsp;");
			}

			shownOneItem = true;

			// if displaying "next" link
			if (0 != nextFirst)
			{
				int numOnNextPage = totResults - nextFirst + 1;

				if (numOnNextPage > pageSize)
				{
					numOnNextPage = pageSize;
				}

				span.AddInnerHTMLElement(
				        MakeURL(m_asOwner.GetString(208, "ptmsgs_portalinfrastructure"), nextFirst, pageSize));
			}
			else
			{
				// Show a nonclickable "Next"
				span.AddInnerHTMLString(m_asOwner.GetString(208, "ptmsgs_portalinfrastructure"));
			}

			// Finally, add a space on the end
			span.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		}
		catch (Exception e)
		{
			throw XPException.GetInstance(e);
		}

		return group;
	}

	/**
	 *  Factory method to make a pagination link for some search results.
	 *   @param model     Current batch of search results
	 *   @param parent    ActivitySpace we're living in
	 *   @param _urlText   Desired text within URL
	 *   @param orderBy   Sort order within which newFirst is to be interpreted
	 *   @param _newFirst  Desired first result on new page
	 *   @param _pageSize  Desired number of results on page
	 */
	private HTMLAnchor MakeURL(String _urlText, int _newFirst, int _pageSize)
	{
	    XPStringBuilder sbUrl = new XPStringBuilder(300);
	    sbUrl.Append(JAVASCRIPT_PAGINATE).Append(((ISavedSearchPortletModelRO) m_model).GetPortletID())
	    	.Append("('"+_newFirst+"','"+_pageSize+"'); return false;");
	    
		HTMLAnchor urlAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL); 
	    urlAnchor.SetOnClick(sbUrl.ToString());
	
		urlAnchor.SetTitle(_urlText);
		urlAnchor.AddInnerHTMLString(_urlText);

	    return urlAnchor;
	}

	private String GetDateTimeValueAsString(IPTSearchResponse _sr, int _index, int _id, XPDateFormat _dateFormat)
	{
    	XPDateTime date = null;
    	if (_id == PT_INTRINSICS.PT_PROPERTY_OBJECTCREATED ||
    	    _id == PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED)
    	{
			try
			{
			    date = SearchResultModel.ConvertSearchDateTimeToXPDateTime(
			            _sr.GetFieldsAsDouble(_index, _id));
	    	}
			catch (Exception e)
			{
				// mbs- the GetFieldsAsDouble call seems to blow up in the wrapper
				// layer if there is no valid value....this is a workaround for Nitro
			}
    	}
    	else
    	{
    	    date = _sr.GetFieldsAsXPDateTime(_index, _id);
    	}
    	
		return (null==date)? "" : _dateFormat.Format(date);
	}
	
	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
}
