package com.plumtree.portalpages.portlets.userprofile.infrastructure;

import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * 
 *
 * @author dand This is a helper class designed to help draw some common
 *         elements for portlets
 */
public class HTMLHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_PORTLETS),
		"com.plumtree.portalpages.portlets.userprofile.infrastructure.HTMLHelper");

	
	protected static final String PTOPENERLINK_START = "<pt:openerLink xmlns:pt='http://www.plumtree.com/xmlschemas/ptui/'";

	
	protected static final String PTOPENERLINK_END = "</pt:openerLink>";

	
	protected static final String OBJECTID_TAG = " pt:objectID=";

	
	protected static final String CLASSID_TAG = " pt:classID=";

	
	protected static final String MODE_TAG = " pt:mode='2'";

	
	protected static final String SINGLEQUOTE = "'";

	/** string for a blank URL */
	public static final String BLANK_URL = "#";

	/**
	 * This method will return a standard introspective portlet cell based on a
	 * Plumtree ASQueryResultWrapper object.  It should get the name,
	 * description, and create a common opener java script function based on
	 * the object class id and object id.
	 *
	 * @param searchResponse  THe search result object containing all the data
	 * @param index the index row of the search result
	 * @param Classid 
	 *
	 * @return ProfileCell  a introspective ProfileCell that will be displayed
	 *
	 * @throws HTMLException 
	 */
	public static ProfileCell CreateProfileRowFromPTResults(AActivitySpace aspace, IPTQueryResult searchResponse, int index, int Classid)
		throws HTMLException
	{
		String sName;
		int Objectid;
		String sDescr;

		// create the text cell based on fodler name and description
		sName	   = searchResponse.ItemAsString(index,
				PT_PROPIDS.PT_PROPID_NAME);
		sDescr     = searchResponse.ItemAsString(index,
				PT_PROPIDS.PT_PROPID_DESCRIPTION);

		// create the ptopener link using pt opener tag - previously it was using the common opener
		// function but this seems to be better especially for 508. 
		// get the object and class ids
		Objectid     = searchResponse.ItemAsInt(index,
				PT_PROPIDS.PT_PROPID_OBJECTID);
		HTMLAnchor openerLink = HTMLHelper.GetObjectOpenerTag(aspace, sName, Classid, Objectid);
		
		logData(sName, sDescr, Classid, Objectid);

		return new ProfileCell(sName, sDescr, sName, openerLink, false);
	}

	/**
	 * This method will return a standard introspective portlet cell based on a
	 * Plumtree IPTSearchResponse object.  It should get the name,
	 * description, and create a common opener java script function based on
	 * the object class id and object id.
	 *
	 * @param searchResponse  THe search result object containing all the data
	 * @param index the index row of the search result
	 *
	 * @return ProfileCell  a introspective ProfileCell that will be displayed
	 *
	 * @throws HTMLException 
	 */
	public static ProfileCell CreateProfileRowFromPTResults(AActivitySpace aspace, IPTSearchResponse searchResponse, int index)
		throws HTMLException
	{
		String sName;
		int Classid;
		int Objectid;
		String sDescr;

		// create the text cell based on fodler name and description
		sName     = searchResponse.GetFieldsAsString(index,
				PT_INTRINSICS.PT_PROPERTY_OBJECTNAME);
		sDescr     = searchResponse.GetFieldsAsString(index,
				PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY);

		// create the ptopener link using pt opener tag - previously it was using the common opener
		// function but this seems to be better especially for 508.
		// get the object and class ids
		Classid     = searchResponse.GetFieldsAsInt(index,
				PT_INTRINSICS.PT_PROPERTY_CLASSID);
		Objectid     = searchResponse.GetFieldsAsInt(index,
				PT_INTRINSICS.PT_PROPERTY_OBJECTID);
		HTMLAnchor openerLink = HTMLHelper.GetObjectOpenerTag(aspace, sName, Classid, Objectid);

		logData(sName, sDescr, Classid, Objectid);

		return new ProfileCell(sName, sDescr, sName, openerLink, false);
	}

	/**
	 * Returns the header for the Section.
	 *
	 * @param _sHeaderTitle Title you want to display in the header.
	 *
	 * @return HTMLTable
	 */
	public static HTMLElement GetHeaderView(String _sHeaderTitle)
	{
		PTConstruct myViewBanner = null;

		try
		{
			// JF- Get the construct for the Section banner
			myViewBanner = ConstructFactory.GetConstruct(PTBannerType.OBJECT);
			myViewBanner.ReplaceNestedElementsWithHTMLString(PTBannerType.OBJECT.TITLE_ELEMENT_NAME,
				_sHeaderTitle);
			myViewBanner.ReplaceNestedElementsWithHTMLString(PTBannerType.OBJECT.SUBTITLE_ELEMENT_NAME,
				"");
		}
		catch (Exception e)
		{
			
				log.Error(e, "EditorView::GetHeaderView The following error occured when trying to display the HTML: " +
					e.getMessage());
			

			// DRD - not going to throw an exception
		}

		return myViewBanner.GetHTMLElement();
	}

	/**
	 * Returns a pttag that sould create an appropriate URL to open a plumtree
	 * object
	 *
	 * @param name 
	 * @param classid  the classid of the plumtree object
	 * @param objectid the Plumtree object id
	 *
	 * @return the javascript function that opens a common plumtree object
	 *
	 * @throws HTMLException 
	 */
	public static HTMLAnchor GetObjectOpenerTag(AActivitySpace aspace, String name, int classid, int objectid)
		throws HTMLException
	{
		String strAppBaseURL = ConfigHelper.GetApplicationBaseURL(aspace);
		int nCurrentUserID = ((IPTSession) aspace.GetUserSession()).GetSessionInfo().GetCurrentUserID();
		String strURL = PTOpenerLinks.GetOpenerURLOpenObjID(classid, objectid, IPTOpenerConstants.OPENER_MODE_VIEW, aspace);
		HTMLAnchor openerLink = new HTMLAnchor(strURL);
		return openerLink;
	}

	/**
	 * Returns a short empty table. This is the one you should use in your
	 * Section, just below the Section header (the one returned by GetHeaderView)
	 *
	 * @return HTMLTable
	 */
	public static HTMLTable GetShortSpacerTable()
	{
		HTMLTable mySpacerTable = new HTMLTable();
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLImg myImage;
		PTConstruct myConstruct;

		try
		{
			mySpacerTable.SetWidth("100%");
			mySpacerTable.SetBorder("0");
			mySpacerTable.SetCellPadding("0");
			mySpacerTable.SetCellSpacing("0");

			// 1st Row begin
			myRow = new HTMLTableRow();
			mySpacerTable.AddInnerHTMLElement(myRow);

			// Empty Cell
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);
			myConstruct     = ConstructFactory.GetConstruct(PTImageType.SPACER);
			myImage		    = (HTMLImg) myConstruct.GetHTMLElement();
			myImage.SetAlt("");
			myImage.SetWidth("1");
			myImage.SetHeight("5");
			myImage.SetBorder("0");
			myCell.AddInnerHTMLElement(myImage);
		}
		catch (Exception e)
		{
			
				log.Error(e, "EditorView::GetShortSpacerTable The following error occured when trying to display the HTML: " +
					e.getMessage());
			

			// JF- no need to throw an exception, this is not critical.
		}

		return mySpacerTable;
	}

	/**
	 * Returns an empty table. This is the one used to separate each section in
	 * any Editor page. Note that this is a static method so you can also call
	 * it from any DP.
	 *
	 * @return HTMLTable
	 */
	public static HTMLTable GetSpacerTable()
	{
		HTMLTable mySpacerTable = new HTMLTable();
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLImg myImage;
		PTConstruct myConstruct;

		try
		{
			mySpacerTable.SetWidth("100%");
			mySpacerTable.SetBorder("0");
			mySpacerTable.SetCellPadding("0");
			mySpacerTable.SetCellSpacing("0");

			// 1st Row begin
			myRow = new HTMLTableRow();
			mySpacerTable.AddInnerHTMLElement(myRow);

			// Empty Cell
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);
			myConstruct     = ConstructFactory.GetConstruct(PTImageType.SPACER);
			myImage		    = (HTMLImg) myConstruct.GetHTMLElement();
			myImage.SetAlt("");
			myImage.SetWidth("1");
			myImage.SetHeight("15");
			myImage.SetBorder("0");
			myCell.AddInnerHTMLElement(myImage);
		}
		catch (Exception e)
		{
			
				log.Error(e, "EditorView::GetSpacerTable The following error occured when trying to display the HTML: " +
					e.getMessage());
			

			// JF- no need to throw an exception, this is not critical.
		}

		return mySpacerTable;
	}

	/**
	 * Log the data from the search response
	 *
	 * @param Name
	 * @param Descr
	 * @param Classid
	 * @param Objectid
	 */
	private static void logData(String Name, String Descr, int Classid,
		int Objectid)
	{
		if ( log.IsDebugEnabled() )
			log.Debug("Name: " + Name+" Description: " + Descr+" Class Id: " + Classid+" Object Id: " + Objectid);
	}
}

