package com.plumtree.portalpages.rest.user;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.rest.*;
import com.plumtree.portalpages.rest.utils.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * Display the JSON/XML result of the User REST API.
 * 
 * NOTE: Theoretically this should return the state of the User object,
 * but for now it just returns the update part of the state.
 */
public class UserRESTDP implements IDisplayPage
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_REST),
		"com.plumtree.portalpages.rest.user.UserRESTDP");

	public static final String STR_MVC_CLASS_NAME = "UserREST";

	protected IUserRestModelRO m_model;

	public void Display(IWebData webData)
	{
		// Comment filtering is often used to thwart JSON security attacks.
		// The JSON data should be wrapped in /* */ for any JSON created on
		// the server side.
		// In addition, all JSON output should be wrapped in a plain { } block
		// so that it can't be used directly in a script src tag.
		try
		{
			// This will turn off HTML performance comments, allowing us to output
			// a valid JSON response.  Note that adding comment filtering allows us
			// to add the comments after the comment and Dojo will ignore them.  We
			// should still disable them anyways, in case someone wants to use a
			// library other than Dojo.
			webData.SetIsHTMLText(false);

			// Say that this data is comment-filtered JSON.
			webData.SetHeader(HTTPHeaderConstants.CONTENT_TYPE, "text/json-comment-filtered");

			// This currently only supports JSON output

			IRESTError error = m_model.GetError();
			if (error != null)
			{
				// This handles outputting the error to the webdata object.
				new RESTHelper().OutputError(error, webData);
			} else
			{
				// Display user community subscription results

				// Really rough estimate of smallest output
				XPStringBuilder sbJSON = new XPStringBuilder(130);

				// We don't need to escape the IDs, since those are ints.
				int nJoinedCommunityID = m_model.GetJoinedCommunityID();
				int nLeftCommunityID = m_model.GetLeftCommunityID();

				sbJSON
				.Append("/*{\n")
				.Append("  \"").Append(UserRESTConstants.USER_FORMAT_NAME).Append("\": {\n");

				// We need to add a comma after the first entry if we have both items.
				if ((nJoinedCommunityID != -1) && (nLeftCommunityID != -1))
				{
					sbJSON.Append("    \"").Append(UserRESTConstants.USER_JOIN_COMMUNITIES).Append("\": \"").Append(nJoinedCommunityID).Append("\",\n");					
					sbJSON.Append("    \"").Append(UserRESTConstants.USER_LEAVE_COMMUNITIES).Append("\": \"").Append(nLeftCommunityID).Append("\"\n");					
				} else
				{
					if (nJoinedCommunityID != -1)
					{
						sbJSON.Append("    \"").Append(UserRESTConstants.USER_JOIN_COMMUNITIES).Append("\": \"").Append(nJoinedCommunityID).Append("\"\n");
					}
					if (nLeftCommunityID != -1)
					{
						sbJSON.Append("    \"").Append(UserRESTConstants.USER_LEAVE_COMMUNITIES).Append("\": \"").Append(nLeftCommunityID).Append("\"\n");					
					}					
				}
				sbJSON
				.Append("  }\n")
				.Append("}*/");

				// Successful HTTP Status Code (201 is only for created).
				webData.SetStatus(200);
				webData.SetData(sbJSON.ToString());
			}
		} catch (Exception e)
		{
			log.Error(e, "Unable to display JSON output in User REST API.");

			// This handles outputting the error to the webdata object.
			new RESTHelper().OutputError(CommonRestErrors.GENERAL, webData);
		}
	}

	public void Init(AActivitySpace parent)
	{
		m_model = (IUserRestModelRO) parent.GetModel(UserRESTModel.STR_MVC_CLASS_NAME);
	}

	public String GetName() {
		return STR_MVC_CLASS_NAME;
	}

	public AActivitySpace GetOwner() {
		// Not used.
		return null;
	}
}