package com.plumtree.portalpages.rest.user;

import com.plumtree.portalpages.rest.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * This model manages the user REST API logic.
 */
public class UserRESTModel implements IModel, IUserRestModel
{
	public static final String STR_MVC_CLASS_NAME = "UserREST";

	protected AActivitySpace m_asOwner;

	protected int m_nJoinedCommunityID = -1;
	protected int m_nLeftCommunityID = -1;

	protected IRESTError m_error = null;

	public void Init(AActivitySpace parent) {
		m_asOwner = parent;
	}

	public String GetName() {
		return STR_MVC_CLASS_NAME;
	}

	public Object Create() {
		return new UserRESTModel();
	}

	public void SetError(IRESTError _error) {
		m_error = _error;
	}

	public IRESTError GetError() {
		return m_error;
	}

	public void CleanupAllData() {
		m_error = null;
	}

	public void CleanupTempData() {
		m_error = null;
	}

	public void JoinCommunity(int _nCommunityID)
	{
		ITaskAPIUICommunity task = TaskAPIUICommunityService.GetInstance();

		// NOTE: If this method returns false, it means we already joined
		// this community.
		task.JoinCommunity(m_asOwner, _nCommunityID);
		m_nJoinedCommunityID = _nCommunityID;
	}

	public void LeaveCommunity(int _nCommunityID)
	{
		ITaskAPIUICommunity task = TaskAPIUICommunityService.GetInstance();

		// NOTE: If this method returns false, it means we were not part of
		// this community.
		task.LeaveCommunity(m_asOwner, _nCommunityID);
		m_nLeftCommunityID = _nCommunityID;
	}

	public int GetJoinedCommunityID() {
		return m_nJoinedCommunityID;
	}

	public int GetLeftCommunityID() {
		return m_nLeftCommunityID;
	}
}