package com.plumtree.taskapi.portalui;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portalpages.pei.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * This class manages instances of the ITaskAPIUICommunity interface
 * for both production and testing purposes.
 * 
 * It is a replacement for the TaskAPIUICommunity class, since that
 * class uses static methods and cannot be mocked for testing.
 */
public class TaskAPIUICommunityService implements ITaskAPIUICommunity
{
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_BROWSING),
		"com.plumtree.taskapi.portalui.TaskAPIUICommunityService");

	private static ITaskAPIUICommunity m_DEBUG_INSTANCE = null;

	/**
	 * @return An instance of the ITaskAPIUIDirectory interface.
	 */
	public static ITaskAPIUICommunity GetInstance()
	{
		if (m_DEBUG_INSTANCE != null)
		{
			return m_DEBUG_INSTANCE;
		}
	
		return new TaskAPIUICommunityService();
	}

	/**
	 * This method sets the debug instance of the ITaskAPIUIDirectory interface
	 * for use as a mock object.  This should never be used in a production
	 * environment.
	 * 
	 * This should be reset to null at the end of testing to ensure other code
	 * does not get an old mock instance.
	 * 
	 * @param _instance The mock instance of the ITaskAPIUIDirectory interface.
	 * When the mock instance is done being used it should be reset to null by
	 * calling this method again with a null argument.
	 */
	public static void SetMockInstanceForDebug(ITaskAPIUICommunity _instance)
	{
		m_DEBUG_INSTANCE = _instance;
	}

	/**
	 * Private constructor to force people to use the GetInstance() method..
	 */
	private TaskAPIUICommunityService() {}

	public boolean CheckUserAccessLevelToCommunity(AActivitySpace owner,
			int nAccessLevel, int nCommunityID) {
		return TaskAPIUICommunity.CheckUserAccessLevelToCommunity(owner, nAccessLevel, nCommunityID);
	}

	public boolean CheckUserAccessLevelToCurrentCommunity(AActivitySpace owner,
			int nAccessLevel) {
		return TaskAPIUICommunity.CheckUserAccessLevelToCurrentCommunity(owner, nAccessLevel);
	}

	public boolean CheckUserAccessLevelToCurrentCommunityPage(
			AActivitySpace owner, int nAccessLevel) {
		return TaskAPIUICommunity.CheckUserAccessLevelToCurrentCommunityPage(owner, nAccessLevel);
	}

	public IPTURL CreateCommunityCommonOpenerURL(AActivitySpace owner,
			int nCommunityID, String strCommunityName) {
		return TaskAPIUICommunity.CreateCommunityCommonOpenerURL(owner, nCommunityID, strCommunityName);
	}

	public IPTURL CreateCommunityPageCommonOpenerURL(AActivitySpace owner,
			int nCommunityID, int nCommunityPageID, String strCommunityPageName) {
		return TaskAPIUICommunity.CreateCommunityPageCommonOpenerURL(owner, nCommunityID, nCommunityPageID, strCommunityPageName);
	}

	public IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner,
			int nProfileUserID, String sProfileUserName) {
		return TaskAPIUICommunity.CreateUserProfilePageCommonOpenerURL(owner, nProfileUserID, sProfileUserName);
	}

	public IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner,
			int nProfileUserID, String sProfileUserName, String strURLTitle) {
		return TaskAPIUICommunity.CreateUserProfilePageCommonOpenerURL(owner, nProfileUserID, sProfileUserName, strURLTitle);
	}

	public IPTURL CreateUserProfilePageCommonOpenerURL(AActivitySpace owner,
			int nProfileUserID, String sProfileUserName, String strURLTitle,
			int nPageId, String strPageName) {
		return TaskAPIUICommunity.CreateUserProfilePageCommonOpenerURL(owner, nProfileUserID, sProfileUserName, strURLTitle, nPageId, strPageName);
	}

	public XPArrayList GetCommunityActionsAsURLs(AActivitySpace owner) {
		return TaskAPIUICommunity.GetCommunityActionsAsURLs(owner);
	}

	public XPArrayList GetCommunityActionsAsURLs(AActivitySpace owner,
			boolean bIncludeOneClick, boolean bInheritTemplate,
			int nPageTemplate) {
		return TaskAPIUICommunity.GetCommunityActionsAsURLs(owner, bIncludeOneClick, bInheritTemplate, nPageTemplate);
	}

	public int GetCommunityIDFromPTURL(IPTURL ptURL) {
		return TaskAPIUICommunity.GetCommunityIDFromPTURL(ptURL);
	}

	public String GetCommunityName(AActivitySpace owner, int nCommunityID) {
		return TaskAPIUICommunity.GetCommunityName(owner, nCommunityID);
	}

	public int GetCommunityPageIDFromPTURL(IPTURL ptURL) {
		return TaskAPIUICommunity.GetCommunityPageIDFromPTURL(ptURL);
	}

	public XPArrayList GetCommunityPagesAsURLs(AActivitySpace owner,
			int nCommunityID) {
		return TaskAPIUICommunity.GetCommunityPagesAsURLs(owner, nCommunityID);
	}

	public int GetCurrentCommunityID(AActivitySpace owner) {
		return TaskAPIUICommunity.GetCurrentCommunityID(owner);
	}

	public String GetCurrentCommunityName(AActivitySpace owner)
			throws XPIllegalArgumentException {
		return TaskAPIUICommunity.GetCurrentCommunityName(owner);
	}

	public int GetCurrentCommunityPageID(AActivitySpace owner)
			throws XPIllegalArgumentException {
		return TaskAPIUICommunity.GetCurrentCommunityPageID(owner);
	}

	public String GetCurrentCommunityPageName(AActivitySpace owner)
			throws XPIllegalArgumentException {
		return TaskAPIUICommunity.GetCurrentCommunityPageName(owner);
	}

	public XPArrayList GetCurrentCommunityPagesAsURLs(AActivitySpace owner) {
		return TaskAPIUICommunity.GetCurrentCommunityPagesAsURLs(owner);
	}

	public IPTURL GetCurrentPageAsURL(AActivitySpace owner)
			throws XPIllegalArgumentException {
		return TaskAPIUICommunity.GetCurrentPageAsURL(owner);
	}

	public IPTURL GetCurrentParentCommunityAsURL(AActivitySpace owner) {
		return TaskAPIUICommunity.GetCurrentParentCommunityAsURL(owner);
	}

	public int GetCurrentParentCommunityID(AActivitySpace owner) {
		return TaskAPIUICommunity.GetCurrentParentCommunityID(owner);
	}

	public XPArrayList GetCurrentRelatedCommunitiesAsURLs(AActivitySpace owner) {
		return TaskAPIUICommunity.GetCurrentRelatedCommunitiesAsURLs(owner);
	}

	public int[] GetCurrentRelatedCommununityIDs(AActivitySpace owner) {
		return TaskAPIUICommunity.GetCurrentRelatedCommununityIDs(owner);
	}

	public XPArrayList GetCurrentSubCommunitiesAsURLs(AActivitySpace owner) {
		return TaskAPIUICommunity.GetCurrentSubCommunitiesAsURLs(owner);
	}

	public int[] GetCurrentSubCommunityIDs(AActivitySpace owner) {
		return TaskAPIUICommunity.GetCurrentSubCommunityIDs(owner);
	}

	public int GetDefaultCommunityID(AActivitySpace owner) {
		return TaskAPIUICommunity.GetDefaultCommunityID(owner);
	}

	public XPArrayList GetMyCommunitiesAsURLs(AActivitySpace owner,
			boolean bUseCachedData) {
		return TaskAPIUICommunity.GetMyCommunitiesAsURLs(owner, bUseCachedData);
	}

	public XPArrayList GetMyMandatoryCommunitiesWithTabsAsURLs(
			AActivitySpace owner, boolean bUseCachedData) {
		return TaskAPIUICommunity.GetMyMandatoryCommunitiesWithTabsAsURLs(owner, bUseCachedData);
	}

	public IPTURL GetParentCommunityAsURL(AActivitySpace owner, int nCommunityID) {
		return TaskAPIUICommunity.GetParentCommunityAsURL(owner, nCommunityID);
	}

	public int GetParentCommunityID(AActivitySpace owner, int nCommunityID) {
		return TaskAPIUICommunity.GetParentCommunityID(owner, nCommunityID);
	}

	public XPArrayList GetRelatedCommununitiesAsURLs(AActivitySpace owner,
			int nCommunityID) {
		return TaskAPIUICommunity.GetRelatedCommununitiesAsURLs(owner, nCommunityID);
	}

	public int[] GetRelatedCommununityIDs(AActivitySpace owner, int nCommunityID) {
		return TaskAPIUICommunity.GetRelatedCommununityIDs(owner, nCommunityID);
	}

	public XPArrayList GetSubCommununitiesAsURLs(AActivitySpace owner,
			int nCommunityID) {
		return TaskAPIUICommunity.GetSubCommununitiesAsURLs(owner, nCommunityID);
	}

	public int[] GetSubCommununityIDs(AActivitySpace owner, int nCommunityID) {
		return TaskAPIUICommunity.GetSubCommununityIDs(owner, nCommunityID);
	}

	public IPTURL GetURLForAddCommunityPortletsEditor(AActivitySpace owner) {
		return TaskAPIUICommunity.GetURLForAddCommunityPortletsEditor(owner);
	}

	public IPTURL GetURLForAddCommunityPortletsFlyout(AActivitySpace owner) {
		return TaskAPIUICommunity.GetURLForAddCommunityPortletsFlyout(owner);
	}

	public IPTURL GetURLForCommunityKnowledgeDirectory(AActivitySpace owner,
			int nCommunityID) {
		return TaskAPIUICommunity.GetURLForCommunityKnowledgeDirectory(owner, nCommunityID);
	}

	public IPTURL GetURLForCreateCommunityPage(AActivitySpace owner) {
		return TaskAPIUICommunity.GetURLForCreateCommunityPage(owner);
	}

	public IPTURL GetURLForCreateCommunityPageOneClick(AActivitySpace owner,
			boolean bInheritTemplate, int nPageTemplate) {
		return TaskAPIUICommunity.GetURLForCreateCommunityPageOneClick(owner, bInheritTemplate, nPageTemplate);
	}

	public XPArrayList GetURLForCurrentCommunity(AActivitySpace owner) {
		return TaskAPIUICommunity.GetURLForCurrentCommunity(owner);
	}

	public IPTURL GetURLForCurrentCommunityEditor(AActivitySpace owner) {
		return TaskAPIUICommunity.GetURLForCurrentCommunityEditor(owner);
	}

	public IPTURL GetURLForEditPortletPreferencesEditor(AActivitySpace owner) {
		return TaskAPIUICommunity.GetURLForEditPortletPreferencesEditor(owner);
	}

	public IPTURL GetURLForJoinCommunitiesEditor(AActivitySpace owner) {
		return TaskAPIUICommunity.GetURLForJoinCommunitiesEditor(owner);
	}

	public IPTURL GetURLForJoinCurrentCommunityAction(AActivitySpace owner) {
		return TaskAPIUICommunity.GetURLForJoinCurrentCommunityAction(owner);
	}

	public IPTURL GetURLForUnsubscribeCommunitiesEditor(AActivitySpace owner) {
		return TaskAPIUICommunity.GetURLForUnsubscribeCommunitiesEditor(owner);
	}

	public IPTURL GetURLForUnsubscribeCurrentCommunityAction(
			AActivitySpace owner) {
		return TaskAPIUICommunity.GetURLForUnsubscribeCurrentCommunityAction(owner);
	}

	public boolean IsCurrentCommunityUserProfileCommunity(AActivitySpace owner) {
		return TaskAPIUICommunity.IsCurrentCommunityUserProfileCommunity(owner);
	}

	public boolean IsCurrentPageCommunityPage(AActivitySpace owner) {
		return TaskAPIUICommunity.IsCurrentPageCommunityPage(owner);
	}

	public boolean IsCurrentPageMyPortalPage(AActivitySpace owner) {
		return TaskAPIUICommunity.IsCurrentPageMyPortalPage(owner);
	}

	public boolean IsCurrentUserAbleToEditCurrentCommunityPage(
			AActivitySpace owner) {
		return TaskAPIUICommunity.IsCurrentUserAbleToEditCurrentCommunityPage(owner);
	}

	public boolean IsCurrentUserCommunityOwner(AActivitySpace owner) {
		return TaskAPIUICommunity.IsCurrentUserCommunityOwner(owner);
	}

	public boolean IsUserAbleToAccessCurrentCommunity(AActivitySpace owner) {
		return TaskAPIUICommunity.IsUserAbleToAccessCurrentCommunity(owner);
	}

	public boolean IsUserAbleToJoinCurrentCommunity(AActivitySpace owner) {
		return TaskAPIUICommunity.IsUserAbleToJoinCurrentCommunity(owner);
	}

	public boolean IsUserMemberOfCurrentCommunity(AActivitySpace owner) {
		return TaskAPIUICommunity.IsUserMemberOfCurrentCommunity(owner);
	}

	/**
	 * @see com.plumtree.taskapi.portalui.ITaskAPIUICommunity.IsCommunityMember(AActivitySpace, int)
	 */
	public boolean IsCommunityMember(AActivitySpace _asOwner, int _nCommunityID)
	{
		IPTSession userSession = (IPTSession) _asOwner.GetUserSession();
		IPTQueryResult membershipList = userSession.GetSessionInfo().GetCurrentCommunityMembership();

		// go through user's list of communities joined
		for (int i = 0; i < membershipList.RowCount(); i++)
		{
			// if community specified by commID is in the list
			if (membershipList.ItemAsInt(i, PT_PROPIDS.PT_PROPID_OBJECTID) == _nCommunityID)
			{
				// user is already a member of community
				return true;
			}
		}

		return false;
	}

	/**
	 * @see com.plumtree.taskapi.portalui.ITaskAPIUICommunity.JoinCommunity(AActivitySpace, int)
	 */
	public boolean JoinCommunity(AActivitySpace _asOwner, int _nCommunityID)
	{
		// Joining a community will fail if the user is already a member, so check
		// membership first.
		if (IsCommunityMember(_asOwner, _nCommunityID))
		{
			// User is already a member of the community, return false.
			return false;
		}

		try
		{
			IPTSession userSession = (IPTSession) _asOwner.GetUserSession();
			((IPTCommunityManager)userSession.GetCommunities()).JoinCommunity(_nCommunityID,
				PT_COMMUNITY_MEMBERSHIPTYPE.PT_COMMUNITYMEMBER_FREE);
		} catch (Exception ex)
		{
			log.Error(ex, "Unable to join community " + _nCommunityID);
			throw XPException.GetInstance(ex);
		}

		try
		{
			//PEI : CommunityActions
			// MJD: All of the following code will load all of the available CommunityAction
			// PEIs and process through them.
			IApplication application = _asOwner.GetApplication();
			Object[] oa = (Object[]) application.GetCachingManager().GetEntry("CommunityActions");

			if (oa != null)
			{
				for (int iCommAction = 0; iCommAction < oa.length; iCommAction++)
				{
					Object o = oa[iCommAction];

					if(o != null)
					{
						ICommunityActions iActions = (ICommunityActions) o;
						iActions.OnAfterUserJoinsCommunity(_asOwner, 
							(IPTSession) _asOwner.GetUserSession(), new int[]{_nCommunityID});
					}
				}
			}
			//PEI : CommunityActions : END
		}
		catch (Exception ex)
		{
			// Ignore errors in the PEI
			log.Error(ex, "OnAfterUserJoinsCommunity PEI failed.");
		}

		// User has successfully joined the community.
		return true;
	}

	/**
	 * @see com.plumtree.taskapi.portalui.ITaskAPIUICommunity.LeaveCommunity(AActivitySpace, int)
	 */
	public boolean LeaveCommunity(AActivitySpace _asOwner, int _nCommunityID)
	{
		// Leaving a community will fail if the user is not a member, so check
		// membership first.
		if (!IsCommunityMember(_asOwner, _nCommunityID))
		{
			// User is not a member of the community, return false.
			return false;
		}

		IPTSession userSession = (IPTSession) _asOwner.GetUserSession();
		IPTCommunityManager myCommunities = (IPTCommunityManager)userSession.GetCommunities();

		try
		{
			myCommunities.QuitCommunity(_nCommunityID);
		} catch (Exception ex)
		{
			log.Error(ex, "Unable to leave community " + _nCommunityID);
			throw XPException.GetInstance(ex);
		}

		try
		{
			//PEI : CommunityActions
			// MJD: All of the following code will load all of the available CommunityAction
			// PEIs and process through them.
			IApplication application = _asOwner.GetApplication();
			Object[] oa = (Object[]) application.GetCachingManager().GetEntry("CommunityActions");
				
			if (oa != null)
			{
				for (int iCommAction = 0; iCommAction < oa.length; iCommAction++)
				{
					Object o = oa[iCommAction];

					if(o != null)
					{
						ICommunityActions iActions = (ICommunityActions) o;
						iActions.OnAfterUserJoinsCommunity(_asOwner, 
						(IPTSession) _asOwner.GetUserSession(), new int[]{_nCommunityID});
					}
				}
			}
			//PEI : CommunityActions : END
		}
		catch (Exception ex)
		{
			log.Error(ex, "OnAfterUserQuitsCommunity PEI failed.");
		}

		// User has successfully left the community.
		return true;
	}
}