package com.plumtree.taskapi.portalui;

import com.plumtree.openfoundation.web.*;
import com.plumtree.portalpages.common.uiparts.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * HTML Related TaskAPI functions.
 * 
 */
public class TaskAPIUIHTML
{

	/**
	 * Adds a spacer cell with a cell width of 4to the specified HTMLTableRow
	 * 
	 * @param myTableRow			HTMLTableRow to add the spacer cell to
	 */
	public static void CreateHorizontalSpacerCell(HTMLTableRow myTableRow)
	{
		NavigationCommonHelpers.CreateHorizontalSpacerCell(myTableRow);
	}

	/**
	 * Gets an HTML Image element for the specified Plumtree Image Construct.
	 * 
	 * @param owner		Current Activityspace.
	 * @param construct	A Plumtree Image Construct.
	 * @return	HTML img element created from the Plumtree Image Construct object.
	 */
	public static  HTMLImg GetImageConstruct(AActivitySpace owner,  PTConstructType construct)
	{
		return (HTMLImg)UIConstructFactory.GetConstruct(construct, true, owner).GetHTMLElement(); 
				
	}
	
	/**
	 * Gets a blank spacer image element with the specified width and height.
	 * 
	 * @param strWidth	Width value that goes directly into the width attribute in the HTML img element. 
	 * @param strHeight Height value that goes directly into the width attribute in the HTML img element.
	 * @return	HTMLImg element with a spacer image in the specified width and height. 
	 */
	public static HTMLImg GetSpacerImage(String strWidth, String strHeight)
	{
		return PlumtreeHelpers.GetSpacerImage(strWidth, strHeight);
		
	}

	/**
	 * Returns an HTML-encoded version of the input string, or
	 * empty string if input is null. 
	 * @param strValue a string of characters to encode for use as HTML text.
	 * @return an encoded version of the input, or empty string if <code>null</code>.
	 */
	public static String HTMLEncode(String strValue)
	{
		return XPHttpUtility.HtmlEncode(strValue);
	}
	
	/**
	 * Returns a decoded version of the input string, 
	 * assuming it was a valid HTML encoding.
	 * 
	 * @param strValue the string to decode.
	 * @return a decoded version of the input.
	 * @throws XPNullPointerException if input is <code>null</code>.
	 * @throws XPUnsupportedEncodingException if an invalid ampersand escape is found.
	 *  
	 */
	public static String HTMLDecode(String strValue)
	{
		return XPHttpUtility.HtmlDecode(strValue);
	}
}
