using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.classtypedescriptors;
using com.plumtree.portaluiinfrastructure.friendlyurl;
using com.plumtree.portaluiinfrastructure.opener;
using com.plumtree.portaluiinfrastructure.opener.plugins;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.marshalers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes
{
  /// <summary>
  /// Common Opener Mode Description:
  /// 1) Create - Show Error (PTCommunityPages).
  /// 2) Edit - Redirect to View (PTCommunityPages).
  /// 3) View - Goes to the community page (PTCommunityPages).
  /// 4) Meta - Redirect to View (PTCommunityPages).
  /// </summary>
  /// <author>Naveen Pai</author>
  public class PTPages : APTNonExtClassTypeDesc
  {
    /*
     * Setting two fields here as follows provides support for Admin UI->Release Locks
     * The Left Navigation Menu string will be loaded from "ptmsgs_portaladminmsgs" file using objectLockManagerMsgID
     * If it should be loaded from a different file, set objectLockManagerMsgFileName accordingly
     */
     
    public PTPages() : base()
    {
      hasObjectLockManager = true;
      objectLockManagerMsgID = 2785;
    }
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes.PTCommunityPages");
    public const String MY_PAGE_AS_NAME = "MyPage";
    public const String COMM_PAGE_AS_NAME = "CommunityPage";
    public const String MY_PAGE_CONTROL_NAME = "SetPage";
    public const String COMM_PAGE_CONTROL_NAME = "SetCommunity";
    public const String STR_PAGE_ID = "PageID";
    public const String STR_COMM_ID = "CommunityID";
    public const String EDITPAGE_AS_NAME = "EditMyPortalPage";
    public const String EDITPAGE_CONTROL_NAME = "EditorStart";
    public const String STR_OPENERPAGE_ID = "OpenerPageID";
    public const String STR_OPENERCONTAINER_ID = "OpenerCommunityID";
    public const String STR_EDITORTYPE_ID = "editorType";
    public const String PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_OBJECT_ID = "ptID";
    public const String PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_INHERIT = "inherit";
    public const String PAGE_EDITOR_QS_INT_DEFAULT_PROFILE_ID = "dpID";
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#AppearsInMainCreateNewClassMenu()
     */
     
    public override bool AppearsInMainCreateNewClassMenu()
    {
      return true;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.APTClassTypeDesc#GetCreateActivityID()
     */
     
    public override int GetCreateActivityID()
    {
      return PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_NONE;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.APTNonExtClassTypeDesc#GetASNameForEditor()
     */
     
    public override String GetASNameForEditor()
    {
      return "PageEditor";
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetClassID()
     */
     
    public override int GetClassID()
    {
      return PT_CLASSIDS.PT_PAGE_ID;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetClassTypeLocalizedName(java.lang.String)
     */
     
    public override String GetClassTypeLocalizedName(String _strLangID)
    {
      IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
      XPResourceManager rm = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");
      return rm.GetString(365, "ptmsgs_portalinfrastructure", _strLangID);
      }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetDefaultIcon()
     */
     
    public override HTMLImg GetDefaultIcon()
    {
      try
      {
        // we re-use the my page icon.
        HTMLImg imgIcon = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.ICON_MYPAGE).GetHTMLElement();
        return imgIcon;
      }
      catch (Exception e)
      {
        return null;
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetRedirectViewObject(com.plumtree.openfoundation.util.XPHashtable, com.plumtree.uiinfrastructure.activityspace.AActivitySpace, com.plumtree.server.IPTSession)
     */
     
    public override Redirect GetRedirectViewObject(XPHashtable _htArguments,
     AActivitySpace _asParent, Object _userSession)
    {
      int nPageID = HTTPHelpers.GetArgumentAsInt(_htArguments, IPTOpenerConstants.QS_OBJECTID,
       HTTPHelpers.GetArgumentAsInt(_htArguments, IPTOpenerConstants.STR_QS_OBJECTID,
       -1));
      IPTPageInfo pageInfo = null;
      try
      {
        pageInfo = ((IPTPageManager) ((IPTSession) _userSession).GetPages()).CachedOpenPageInfo(nPageID,
         false, false);
      }
      catch (XPException ex)
      {
        try
        {
          // it wasn't a community page they had read acl on, maybe it is a my page
          pageInfo = ((IPTSession) _userSession).GetSessionInfo().GetCurrentMyPage(nPageID);
          }
        catch (XPException ex2)
        {
        }
      }
      // either it was another user's my page, or a community page the user did not have read acl on
      // send them to their home my page
      // Create the Redirect Object
      Redirect resultingRedirect = new Redirect();
      // Set the space
      if (null == pageInfo || PT_CLASSIDS.PT_MYPAGESCONTAINER_ID == pageInfo.GetContainerClassID())
      {
        resultingRedirect.SetLinkCreateNewSpace(MY_PAGE_AS_NAME, _asParent);
        resultingRedirect.SetControl(MY_PAGE_CONTROL_NAME);
      } else
      {
        resultingRedirect.SetLinkCreateNewSpace(COMM_PAGE_AS_NAME, _asParent);
        resultingRedirect.SetControl(COMM_PAGE_CONTROL_NAME);
        resultingRedirect.AddControlArgument(STR_COMM_ID, XPConvert.ToString(pageInfo.GetContainerObjectID()));
        }
      if (null == pageInfo)
      {
        resultingRedirect.AddControlArgument(STR_PAGE_ID, XPConvert.ToString(0));
        } else
      {
        resultingRedirect.AddControlArgument(STR_PAGE_ID, XPConvert.ToString(nPageID));
        }
      // no parent space or parent ID
      resultingRedirect.SetStripParentSpace(true);
      return resultingRedirect;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetRelatedClassTypeIDs()
     */
     
    public override int[] GetRelatedClassTypeIDs()
    {
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#HasGlobalClassTypeMap()
     */
     
    public override bool HasGlobalClassTypeMap()
    {
      return false;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.classtypedescriptors.IClassTypeDesc#GetRedirectCreateNew(com.plumtree.openfoundation.util.XPHashtable,com.plumtree.uiinfrastructure.activityspace.AActivitySpace,com.plumtree.server.IPTSession)"></seealso>
    public override Redirect GetRedirectCreateNew(XPHashtable _htArguments,
     AActivitySpace _asParent, Object _userSession)
    {
      Redirect result = base.GetRedirectCreateNew(_htArguments, _asParent,
       _userSession);
      int nDefaultProfileID = HTTPHelpers.GetArgumentAsInt(_htArguments, PAGE_EDITOR_QS_INT_DEFAULT_PROFILE_ID,
       0);
      if (0 != nDefaultProfileID)
      {
        result.AddControlArgument(PAGE_EDITOR_QS_INT_DEFAULT_PROFILE_ID, nDefaultProfileID);
        }
      int nPageTemplateID = HTTPHelpers.GetArgumentAsInt(_htArguments, PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_OBJECT_ID,
       0);
      if (0 != nPageTemplateID)
      {
        result.AddControlArgument(PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_OBJECT_ID,
         nPageTemplateID);
        result.AddControlArgument(PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_INHERIT,
         HTTPHelpers.GetArgumentAsBoolean(_htArguments, PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_INHERIT,
         (0 != nPageTemplateID)));
      }
      return result;
    }
    
    /// <seealso cref="IPTClassTypeDesc#ConstructFriendlyURL(String,int,String)">
    /// Generates a friendly Url for Community page
    /// In the worst case, this method will perform two cache lookups, and two db queries
    /// </seealso>
    public override String ConstructFriendlyURL(FriendlyURL url, AActivitySpace space)
    {
      // name of page
      String objectName = url.GetOjectName();
      // page id
      int objectId = url.GetObjectId();
      if (("".Equals(objectName) || objectName == null) && objectId <= 0)
      {
        throw new PTException("Both object name and id are missing. At leaset one needs to be specified to create a friendly Url.",
         PT_RESULTCODES.PT_E_INVALIDARGUMENT);
      }
      IPTSession session = (IPTSession) space.GetUserSession();
      IPTPageInfo curPage = ((IPTPageManager) session.GetPages()).CachedOpenPageInfo(objectId,
       false, false);
      if (objectName == null)
      {
        objectName = curPage.GetName();
      }
      XPStringBuilder buf = new XPStringBuilder(200);
      int commId = curPage.GetContainerObjectID();
      if (commId < 1)
      {
        // DCA - we're not dealing with a community page, return null
        return null;
      }
      IPTCommunityInfo comm = ((IPTCommunityManager) session.GetCommunities()).CachedOpenCommunityInfo(commId,
       false);
      // @TODO how to handle errors here?
      String commName = comm.GetName();
      String commKeyword = ((IFriendlyURLManager) PTConfigHelper.GetTypeDescrManager()).LookupFriendlyURLKeyFromClassId(PT_CLASSIDS.PT_COMMUNITY_ID + "");
      if (commKeyword == null)
      {
        throw new PTException("No Friendly URL keyword was found for Communities. " + "Verify that a mapping entry exists in FriendlyURL.xml.",
         PT_RESULTCODES.PT_E_FAIL);
      }
      buf.Append("/").Append(commKeyword);
      // Add the community name which has to come before the community page name
      commName = FriendlyURL.NormalizeObjectName(commName);
      // DCA - we're encoding in case it has non-ASCII chars
      commName = XPHttpUtility.UrlEncode(commName, LinkConstants.UTF_8_ENCODING);
      buf.Append("/").Append(commName);
      if (FriendlyURLPlugin.UseObjectId())
      {
        buf.Append("/").Append(commId);
      }
      // now add the child name, which is the page name
      objectName = FriendlyURL.NormalizeObjectName(objectName);
      // DCA - we're encoding in case it has non-ASCII chars
      objectName = XPHttpUtility.UrlEncode(objectName, LinkConstants.UTF_8_ENCODING);
      buf.Append("/").Append(objectName);
      return buf.ToString();
    }
  }
}