using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.application.varpacks;
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.server.marshalers;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.expandablelist;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames
{
  /// <summary>
  /// This is the implementation of the Expandable List  framework for the
  /// Growable Array. The Growable Array is a list of input texts.  On add, a new
  /// input text populated with the default value  will be added at the bottom of
  /// the list.
  /// </summary>
  /// <author>MichaelS</author>
  public class LocNamesModel : ExpListModel,ILocNamesModel
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.LocNamesModel");
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "LocNames";
    
    /// <summary>
    /// MES- Constants for client side validation 
    /// </summary>
    public const int CLIENT_VALIDATION_MAX_NAME = 255;
    public const int CLIENT_VALIDATION_MAX_DESCRIPTION = 255;
    
    /// <summary>
    /// First column: an index to keep the list sorted.
    /// </summary>
    public const int LOCNAMEMODEL_COL_INDEX = 0;
    
    /// <summary>
    /// Second column: name. 
    /// </summary>
    public const int LOCNAMEMODEL_COL_NAME = 1;
    
    /// <summary>
    /// Third column: description. 
    /// </summary>
    public const int LOCNAMEMODEL_COL_DESCRIPTION = 2;
    
    /// <summary>
    /// Fourth column: language. 
    /// </summary>
    public const int LOCNAMEMODEL_COL_LANG = 3;
    
    /// <summary>
    /// Fifth column: language name. 
    /// </summary>
    public const int LOCNAMEMODEL_COL_LANGNAME = 4;
    
    /// <summary>
    /// Number total of columns 
    /// </summary>
    public const int LOCNAMEMODEL_COL_MAX = 5;
    
    /// <summary>
    /// MES- Constants we use to get data out of x_config.xml 
    /// </summary>
    private const String MAND_OBJ_LANG_VAR_SECTION = "International";
    private const String MAND_OBJ_LANG_VAR_NAME = "MandatoryObjectLanguage";
    
    /// <summary>
    /// Icon to display on each row, just before the input text. 
    /// </summary>
    public PTImageType m_imgRowIcon;
    
    /// <summary>
    /// MES- Does the portal have a mandatory language?  See comments below. 
    /// </summary>
    public bool m_bHasMandatoryLang = false;
    
    /// <summary>
    /// MES- Does the object support localization at all? 
    /// </summary>
    public bool m_bSupportsLocalization = false;
    
    /// <summary>
    /// RWW - Allow someone to set a different max length
    /// for name. For example, AdminFolders need to have a
    /// max lenght of 63 instead of the default 255 for 
    /// community naming purposes.
    /// </summary>
    public int m_iClientValidationMaxNameOverride;
    
    /// <summary>
    /// RWW - Sse the max name override value if this
    /// is true, use the default value if this is false.
    /// </summary>
    public bool m_bUseClientValidationMaxNameOverride;
    
    /// <summary>
    /// MES- This is a sort of "temp" variable used when a popup edits a
    /// name and description.  It's populated by the RepostControl, and read
    /// by LocNamesPopupView and this class 
    /// </summary>
    public int m_iRowToEdit;
    
    /// <summary>
    /// The object we are editing 
    /// </summary>
    private IPTLocalizable m_objEditing;
    
    /// <summary>
    /// The language the user is currently in
    /// </summary>
    private String m_sUserLang = null;
    
    /// <summary>
    /// Label for Name Field 
    /// </summary>
    private String m_strNameLabel = "";
    
    /// <summary>
    /// Boolean: Do not display or set Descriptions 
    /// </summary>
    private bool m_bNoDescriptions = false;
    
    /// <summary>
    /// Pop Up Display Page Name 
    /// </summary>
    private String m_strPopUpDisplayPageName = LocNamesPopUpDP.STR_MVC_CLASS_NAME;
    
    /// <summary>
    /// RWW - Returns whether we should use use the
    /// override or default value for name field.
    /// </summary>
    public virtual bool IsUsingMaxNameOverride()
    {
      return m_bUseClientValidationMaxNameOverride;
    }
    
    /// <summary>
    /// RWW - Returns the override value for the max length
    /// for the name field (for client validation).
    /// </summary>
    /// <returns>Max length for name field.</returns>
    public virtual int GetMaxNameOverride()
    {
      return m_iClientValidationMaxNameOverride;
    }
    
    /// <summary>
    /// Adds one row at the bottom of the list and  sets the following values:
    /// First column: m_nNextIndex Second column: m_sDefaultValue
    /// </summary>
    /// <param name="_htFormData">Form data</param>
    public override void AddRowsOnRepost(XPHashtable _htFormData)
    {
      //MES- Get the data that was posted to us
      String sNewName = ((String[]) _htFormData.GetElement(GetUniqueString(LocNamesView.HTML_INPUT_NEW_NAME)))[0];
      String sNewDescription = ((String[]) _htFormData.GetElement(GetUniqueString(LocNamesView.HTML_INPUT_NEW_DESCRIPTION)))[0];
      String sNewLang = ((String[]) _htFormData.GetElement(GetUniqueString(LocNamesView.HTML_INPUT_NEW_LANG)))[0];
      String sNewLangName = m_asOwner.GetResourceManager().GetDisplayLocaleFromLocaleID(sNewLang,
       m_sUserLang);
      //MES- Are we adding a new row, or editing an existing row?
      //	m_iRowToEdit was set by LocNamesRepostControl, and is the
      //	row number or LocNamesView.EDIT_ROW_NEWROW
      if (LocNamesView.EDIT_ROW_NEWROW != m_iRowToEdit)
      {
        //MES- Editing an existing row
        m_saData.SetItem(m_iRowToEdit, LOCNAMEMODEL_COL_NAME, sNewName);
        m_saData.SetItem(m_iRowToEdit, LOCNAMEMODEL_COL_DESCRIPTION, sNewDescription);
        m_saData.SetItem(m_iRowToEdit, LOCNAMEMODEL_COL_LANG, sNewLang);
        m_saData.SetItem(m_iRowToEdit, LOCNAMEMODEL_COL_LANGNAME, sNewLangName);
        } else
      {
        //MES- Adding a new row
        int iRows = m_saData.GetCount();
        Object[] oRow = new Object[LOCNAMEMODEL_COL_MAX];
        oRow[LOCNAMEMODEL_COL_INDEX] = iRows;
        oRow[LOCNAMEMODEL_COL_NAME] = sNewName;
        oRow[LOCNAMEMODEL_COL_DESCRIPTION] = sNewDescription;
        oRow[LOCNAMEMODEL_COL_LANG] = sNewLang;
        oRow[LOCNAMEMODEL_COL_LANGNAME] = sNewLangName;
        //MES- Add the row
        m_saData.AddRow(oRow);
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new LocNamesModel();
    }
    
    /// <summary>
    /// MES- Finish is called by the ObjEditorModel when the object is being
    /// stored.  We should store the data from the model to the IPTObject here.
    /// </summary>
    public virtual void Finish()
    {
      //MES- Add the localized names and descriptions, if we're supporting that
      if (m_objEditing.GetIsLocalized() && m_bSupportsLocalization)
      {
        //MES- The localized strings arein m_saData.  We need to 
        //	put it into the object.
        int iRows = m_saData.GetCount();
        Object[][] arrNames = new Object[PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_MAX_NUM_COLUMNS][];
        for (int j2c_index = 0;j2c_index < PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_MAX_NUM_COLUMNS;
        j2c_index++)
        {
          arrNames[j2c_index] = new Object[iRows];
        }
        Object[][] arrDescriptions = new Object[PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_MAX_NUM_COLUMNS][];
        for (int j2c_index = 0;j2c_index < PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_MAX_NUM_COLUMNS;
        j2c_index++)
        {
          arrDescriptions[j2c_index] = new Object[iRows];
        }
        //MES- Run through the items, adding them to the arrays
        for (int iRow = 0;iRow < iRows;++iRow)
        {
          String sName = (String) m_saData.GetItem(iRow, LOCNAMEMODEL_COL_NAME);
          String sDescription = (String) m_saData.GetItem(iRow, LOCNAMEMODEL_COL_DESCRIPTION);
          String sLang = (String) m_saData.GetItem(iRow, LOCNAMEMODEL_COL_LANG);
          arrNames[PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_LANG][iRow] = sLang;
          arrNames[PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_STRING][iRow] = sName;
          arrDescriptions[PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_LANG][iRow] = sLang;
          arrDescriptions[PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_STRING][iRow] = sDescription;
          }
        //MES- Set the localized names and descriptions
        m_objEditing.SetLocalizedNames(arrNames);
        m_objEditing.SetLocalizedDescriptions(arrDescriptions);
      }
    }
    
    /// <returns>Name label</returns>
    public virtual String GetNameLabel()
    {
      if (null == m_strNameLabel || "".Equals(m_strNameLabel))
      {
        return m_asOwner.GetString(63, "ptmsgs_portalinfrastructure");
      }
      return m_strNameLabel;
    }
    
    /// <summary>
    /// Call this method if the default name label is not appropriate.
    /// </summary>
    /// <param name="sNameLabel"></param>
    public virtual void SetNameLabel(String strNameLabel)
    {
      m_strNameLabel = strNameLabel;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.ILocNamesModelRO#GetNoDescriptions()"></seealso>
    public virtual bool IsNoDescriptions()
    {
      return m_bNoDescriptions;
    }
    
    /// <summary>
    /// If this method is not called, then it will default to using descriptions unless the Object does not support it.
    /// </summary>
    /// <param name="bNoDescriptions"></param>
    public virtual void SetIsNoDescriptions(bool bNoDescriptions)
    {
      m_bNoDescriptions = bNoDescriptions;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.ILocNamesModelRO#GetPopUpDisplayPageName()"></seealso>
    public virtual String GetPopUpDisplayPageName()
    {
      return m_strPopUpDisplayPageName;
    }
    
    /// <summary>
    /// Call this method if you want the popup display page to be something other than LocNamesPopUpDP.STR_MVC_CLASS_NAME
    /// </summary>
    /// <param name="sPopUpDisplayPageName"></param>
    public virtual void SetPopUpDisplayPageName(String strPopUpDisplayPageName)
    {
      m_strPopUpDisplayPageName = strPopUpDisplayPageName;
    }
    
    /// <summary>
    /// MES- GetIsLocalized is used by controls to find out if the object
    /// is localized.
    /// </summary>
    /// <returns>a boolean indicating if the object is localized</returns>
    public virtual bool GetIsLocalized()
    {
      return m_objEditing.GetIsLocalized();
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// MES- GetPrimaryDesc is used by controls to find out the primary
    /// description for the object we're editing
    /// </summary>
    /// <returns>the primary name of the object</returns>
    public virtual String GetPrimaryDesc()
    {
      return m_objEditing.GetDescription();
    }
    
    /// <summary>
    /// MES- GetPrimaryLang is used by controls to find out the primary
    /// language for the object we're editing
    /// </summary>
    /// <returns>the identifier of the primary language</returns>
    public virtual String GetPrimaryLang()
    {
      return m_objEditing.GetPrimaryLang();
    }
    
    /// <summary>
    /// MES- GetPrimaryName is used by controls to find out the primary
    /// name for the object we're editing
    /// </summary>
    /// <returns>the primary name of the object</returns>
    public virtual String GetPrimaryName()
    {
      return m_objEditing.GetName();
    }
    
    /// <summary>
    /// Does the portal have a mandatory language?
    /// </summary>
    /// <returns></returns>
    public virtual bool HasMandatoryLang()
    {
      return m_bHasMandatoryLang;
    }
    
    /// <summary>
    /// Does the object support localization at all?
    /// </summary>
    /// <returns></returns>
    public virtual bool SupportsLocalization()
    {
      return m_bSupportsLocalization;
    }
    
    /// <returns></returns>
    public virtual int GetRowToEdit()
    {
      return m_iRowToEdit;
    }
    
    /// <summary>
    /// Returns the icon to display on each row before the  input text. If the
    /// value is null, no icon is displayed.
    /// </summary>
    /// <returns>PTImageType</returns>
    public virtual PTImageType GetRowIcon()
    {
      return m_imgRowIcon;
    }
    
    /// <summary>
    /// Initializes the expandable list.
    /// </summary>
    /// <param name="objToEdit">is the IPTObject to be edited</param>
    public virtual void InitializeNameDescEditor(IPTLocalizable objToEdit,
     int _nListKey)
    {
      Object[][] arrExpListColInfo = new Object[2][];
      for (int j2c_index = 0;j2c_index < 2;j2c_index++)
      {
        arrExpListColInfo[j2c_index] = new Object[EXPLIST_COLUMN_INVALID];
        }
      arrExpListColInfo[0][EXPLIST_COLUMN_ID] = LOCNAMEMODEL_COL_NAME;
      arrExpListColInfo[0][EXPLIST_COLUMN_TITLE] = m_asOwner.GetString(49,
       "ptmsgs_portalinfrastructure");
      arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH] = "80%";
      arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "1";
      arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT] = true;
      arrExpListColInfo[1][EXPLIST_COLUMN_ID] = LOCNAMEMODEL_COL_LANGNAME;
      arrExpListColInfo[1][EXPLIST_COLUMN_TITLE] = m_asOwner.GetString(48,
       "ptmsgs_portalinfrastructure");
      arrExpListColInfo[1][EXPLIST_COLUMN_WIDTH] = "20%";
      arrExpListColInfo[1][EXPLIST_COLUMN_COLSPAN] = "1";
      arrExpListColInfo[1][EXPLIST_COLUMN_ENABLESORT] = true;
      //MES- Record the object we're editing
      m_objEditing = objToEdit;
      //MES- Get info from x_config.xml about whether we support "mandatory" primary langs
      //	Some customers may wish to enforce that ALL objects MUST have a
      //	particular primary language (e.g. Ford may decide that 80% of their
      //	workforce is English speaking, and all objects should have an English name.)
      //	To do this, they'd put the locale they want to be mandatory into the
      //	MandatoryObjectLanguage key in x_config.xml.  We should respect that item if
      //	it's there by not allowing the client to choose primary language 
      String sMandatoryLang = ((PTConfigVarPack) m_asOwner.GetApplication().GetVarPackManager().GetVariablePackage(PTConfigVarPack.VARPACK_ID)).GetConfigValueAsString(MAND_OBJ_LANG_VAR_SECTION,
       MAND_OBJ_LANG_VAR_NAME, "value");
      if (null == sMandatoryLang)
      {
        sMandatoryLang = "";
      }
      //MES- Make sure that if the object gets stored, the primary lang will
      //	be the mandatory lang.
      m_bHasMandatoryLang = false;
      if (!("").Equals(sMandatoryLang))
      {
        m_objEditing.SetPrimaryLang(sMandatoryLang);
        m_bHasMandatoryLang = true;
      }
      //MES- Find out if the object supports localization at all
      m_bSupportsLocalization = m_objEditing.GetSupportsLocalization();
      /*MES- List Key*/
      base.InitializeExpandableList(m_asOwner.GetString(50, "ptmsgs_portalinfrastructure"),
       arrExpListColInfo, PTImageType.ACTION_ADD, m_asOwner.GetString(47,
       "ptmsgs_portalinfrastructure"), _nListKey);
    }
    
    /// <summary>
    /// Initalizes the list (PTGrowableSortedArrayWrapper)  with a column
    /// mapping and an array of sort. The growable array will always be sorted
    /// by the 1st  column ascending.
    /// </summary>
    /// <param name="_saData"></param>
    public override void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
    {
      // JF- Column mapping
      int[] arrColumns = new int[]{LOCNAMEMODEL_COL_INDEX, LOCNAMEMODEL_COL_NAME,
       LOCNAMEMODEL_COL_DESCRIPTION, LOCNAMEMODEL_COL_LANG, LOCNAMEMODEL_COL_LANGNAME};
      // JF- Sort array
      // The growable array will be sorted by the 1st column.
      int[][] arrSort = new int[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        arrSort[j2c_index] = new int[2];
      }
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID] = LOCNAMEMODEL_COL_INDEX;
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      //MES- Only initialize if we actually SUPPORT localization
      if (m_bSupportsLocalization)
      {
        //MES- Convert the data from the form in the object into the form in the
        //	list.  The list contains Name, Description, Language, and Default (the
        //	Default flag indicates if that row is the primary name, desc, and lang.)
        Object[][] arrLocalizedNames = (Object[][]) m_objEditing.GetLocalizedNames();
        Object[][] arrLocalizedDescriptions = (Object[][]) m_objEditing.GetLocalizedDescriptions();
        //MES- Put the arrays into hash tables- we need to correlate the name and the description
        //	for a given language, and hashtables are a convenient way to do that
        XPHashtable hashLangToName = new XPHashtable();
        XPHashtable hashLangToDescription = new XPHashtable();
        AddLocalizedArraysToMap(arrLocalizedNames, hashLangToName, hashLangToDescription);
        AddLocalizedArraysToMap(arrLocalizedDescriptions, hashLangToDescription,
         hashLangToName);
        //MES- Create the data array
        Object[][] arrAllData = new Object[LOCNAMEMODEL_COL_MAX][];
        for (int j2c_index = 0;j2c_index < LOCNAMEMODEL_COL_MAX;j2c_index++)
        {
          arrAllData[j2c_index] = new Object[hashLangToName.GetSize()];
        }
        //MES- Add the localized strings
        String sName;
        String sDescription;
        String sLang;
        IXPEnumerator enumLangs = hashLangToName.GetKeys();
        enumLangs.MoveNext();
        m_sUserLang = PTPersonalSettingsHelper.GetPersonalSettingValue_Locale(m_asOwner).GetLanguage();
        for (int iRowIndex = 0;iRowIndex < hashLangToName.GetSize();++iRowIndex)
        {
          sLang = (String) enumLangs.GetCurrent();
          sName = (String) hashLangToName.GetElement(sLang);
          sDescription = (String) hashLangToDescription.GetElement(sLang);
          arrAllData[LOCNAMEMODEL_COL_INDEX][iRowIndex] = iRowIndex + 1;
          arrAllData[LOCNAMEMODEL_COL_NAME][iRowIndex] = sName;
          arrAllData[LOCNAMEMODEL_COL_DESCRIPTION][iRowIndex] = sDescription;
          arrAllData[LOCNAMEMODEL_COL_LANG][iRowIndex] = sLang;
          // DCA - storing localized language name so that we can sort by it
          arrAllData[LOCNAMEMODEL_COL_LANGNAME][iRowIndex] = m_asOwner.GetResourceManager().GetDisplayLocaleFromLocaleID(sLang,
           m_sUserLang);
          enumLangs.MoveNext();
        }
        //MES- We set up the data, initialize the list.
        _saData.Initialize(arrColumns, arrAllData, arrSort, false);
      } else
      {
        //MES- We don't support localization.  Set up the array with
        //	NO rows, so that it's at least initialized correctly.
        Object[][] arrEmptyData = new Object[LOCNAMEMODEL_COL_MAX][];
        for (int j2c_index = 0;j2c_index < LOCNAMEMODEL_COL_MAX;j2c_index++)
        {
          arrEmptyData[j2c_index] = new Object[0];
        }
        _saData.Initialize(arrColumns, arrEmptyData, arrSort, false);
      }
    }
    
    /// <summary>
    /// Saves the data when a repost occurs. Editors using this framework need
    /// to call that method from  the savePage method of the main Model.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <returns>int Status for server side validation</returns>
    public override int SaveSection(XPHashtable _htFormData)
    {
      int intStatus = base.SaveSection(_htFormData);
      try
      {
        //MES- Store the primary name, primary desc, primary lang,
        //	and IsLocalized info to the object.  Normally, 
        //	in the MVC model we'd just store this to member variables
        //	and store it to the object in our Finish method.  We can't
        //	do that, because other pages need to know the "new" name
        //	of the object before Finish has been called, and they 
        //	don't know how to get to this model.  It's a flaw in the
        //	architecture that we work around by editing the actual
        //	object- the object itself becomes the data sharing mechanism.
        String sPrimaryName = ((String[]) _htFormData.GetElement(GetUniqueString(LocNamesPrimaryNameView.HTMLINPUT_NAME)))[0];
        m_objEditing.SetName(sPrimaryName);
        //MES- Only handle description if this object type supports setting the desc
        if (!IsNoDescriptions())
        {
          String sPrimaryDescription = ((String[]) _htFormData.GetElement(GetUniqueString(LocNamesPrimaryNameView.HTMLINPUT_DESCRIPTION)))[0];
          m_objEditing.SetDescription(sPrimaryDescription);
        }
        //MES- Store the primary language.  Sometimes we post and store every time
        //	the primary language changes, but sometimes we don't (we do when it could
        //	affect the choices for localized languages, but if we're not showing
        //	localized languages, then changing the language doesn't affect the display,
        //	so we don't have to post.)
        //	NOTE: If there's a MANDATORY language, then the Primary Lang input won't be 
        //	displayed, and therefore won't be posted to us.  In this case, we should NOT
        //	try to read the form value, as it doesn't exist...
        if (!m_bHasMandatoryLang)
        {
          SetPrimaryLang(((String[]) _htFormData.GetElement(GetUniqueString(LocNamesPrimaryNameView.PRIMARY_LANG_DROPDOWN)))[0]);
          }
      }
      catch (Exception e)
      {
        //Something is not allowed
        intStatus = RepostControl.PAGE_STATUS_INVALID;
        if ((e is PTException) && (((PTException) e).GetErrorCode() == -2147024809))
        {
          PlumtreeHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Admin"),
           "LocNamesModel::SaveSection", e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(415,
           "ptmsgs_portalinfrastructure"));
        } else
        {
          PlumtreeHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("Portal_Admin"),
           "LocNamesModel::SaveSection", e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(416,
           "ptmsgs_portalinfrastructure"));
        }
      }
      //MES- Let the superclass do it's thing
      return intStatus;
    }
    
    /// <summary>
    /// MES- SetIsLocalized is used by controls to set the IsLocalized flag on
    /// the object we're editing.  See comments in SaveSection
    /// </summary>
    /// <param name="bIsLocalized">indicates if the object supports localization</param>
    /// <returns>void</returns>
    public virtual void SetIsLocalized(bool bIsLocalized)
    {
      m_objEditing.SetIsLocalized(bIsLocalized);
    }
    
    /// <summary>
    /// MES- SetPrimaryLang is used by controls to set the primary language on
    /// the object we're editing.  See comments in SaveSection
    /// </summary>
    /// <param name="sPrimaryLang">is the identifier of the new primary language</param>
    /// <returns>void</returns>
    public virtual void SetPrimaryLang(String sPrimaryLang)
    {
      //MES- Store the new primary language into the model
      m_objEditing.SetPrimaryLang(sPrimaryLang);
      //MES- And delete any localized string that's in that language
      RemoveLang(sPrimaryLang);
    }
    
    /// <param name="nRowToEdit"></param>
    public virtual void SetRowToEdit(int nRowToEdit)
    {
      m_iRowToEdit = nRowToEdit;
    }
    
    /// <summary>
    /// Sets the icon to display on each row before the  input text. By default,
    /// no icon is displayed.
    /// </summary>
    /// <param name="_imgRowIcon"></param>
    public virtual void SetRowIcon(PTImageType _imgRowIcon)
    {
      m_imgRowIcon = _imgRowIcon;
    }
    
    /// <summary>
    /// Saves the data on the row _nRowIndex.
    /// </summary>
    /// <param name="_saData">List to update the data on row _nRowIndex</param>
    /// <param name="_nRowIndex">Index of the row to save.</param>
    /// <param name="_htFormData">Data posted by the form.</param>
    /// <returns>int Status for server side validation.</returns>
    protected override int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
     XPHashtable _htFormData)
    {
      //MES- Nothing to do to save a row- it's handled in SaveSection
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// MES- AddLocalizedArraysToMap is a helper function called by InitializeSortedArray.
    /// It is used to correlate entries in the Object.GetLocalizedNames and
    /// Object.GetLocalizedDescriptions calls.  One of the arrays is taken in, and it's
    /// put into BOTH HashMaps.  The actual value is put into the "primary" hashmap, and
    /// a blank placeholder is placed into the secondary hashmap.  This way, the hashmaps
    /// are always in sync.
    /// </summary>
    /// <param name="arrLocalizedStrings">is the array of strings to add</param>
    /// <param name="hashPrimary">is the table to put the strings into (or update in place)</param>
    /// <param name="hashSecondary">is the table to put the placeholders into</param>
    private void AddLocalizedArraysToMap(Object[][] arrLocalizedStrings, XPHashtable hashPrimary,
     XPHashtable hashSecondary)
    {
      //MES- Do we have any strings to add?
      if (null != arrLocalizedStrings)
      {
        //MES- Run through them
        for (int iStringIndex = 0;iStringIndex < arrLocalizedStrings[0].Length;
        ++iStringIndex)
        {
          String sLang = (String) arrLocalizedStrings[PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_LANG][iStringIndex];
          String sLocalizedString = (String) arrLocalizedStrings[PT_LOCALIZED_STRINGS_INDICES.PT_LSINDEX_STRING][iStringIndex];
          //MES- Just put it into the primary map, we know what the value's supposed to be
          hashPrimary.PutElement(sLang, sLocalizedString);
          //MES- Only put a placeholder into the secondary map if there isn't
          //	already an entry- we wouldn't want to overwrite the "real" value.
          if (!hashSecondary.ContainsKey(sLang))
          {
            hashSecondary.PutElement(sLang, "");
          }
        }
      }
    }
    
    /// <summary>
    /// MES- RemoveLang is used to remove an entry from the m_saData array.
    /// It removes the first row that's associated with the language passed
    /// in.  This is used when the user switches the primary language of the
    /// object- no localized language should match the primary language, so
    /// any localization matching the new primary language should be deleted.
    /// </summary>
    /// <param name="sLangToRemove">is the language to remove</param>
    private void RemoveLang(String sLangToRemove)
    {
      //MES- Run through the array, looking for an entry with that lang.
      //	As there should be at most one, we can quit as soon as we find one!
      int iNumRows = m_saData.GetCount();
      for (int iIndex = 0;iIndex < iNumRows;++iIndex)
      {
        String sRowLang = (String) m_saData.GetItem(iIndex, LOCNAMEMODEL_COL_LANG);
        if (0 == XPStringUtility.CompareIgnoreCase(sRowLang, sLangToRemove))
        {
          m_saData.RemoveRow(iIndex);
          return;
        }
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.ILocNamesModelRO#GetObjectID()
     */
     
    public virtual int GetObjectID()
    {
      IPTServerContext pSC = null;
      try
      {
        pSC = (IPTServerContext) m_objEditing;
      }
      catch (Exception e)
      {
        // if the object does not support IPTServerContext, return 0 -jas
        return 0;
      }
      return pSC.GetObjectID();
    }
    /* (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.ILocNamesModelRO#GetObjectUUID()
     */
     
    public virtual String GetObjectUUID()
    {
      IPTMigratable pM = null;
      try
      {
        pM = (IPTMigratable) m_objEditing;
      }
      catch (Exception e)
      {
        // if the object does not support IPTMigratable, return null -jas
        return null;
      }
      return pM.GetObjectUUID();
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.editor.commonpages.localizednames.ILocNamesModelRO#GetClassID()"></seealso>
    public virtual int GetClassID()
    {
      IPTObject pObj = null;
      try
      {
        pObj = (IPTObject) m_objEditing;
      }
      catch (Exception e)
      {
        // if the object does not support IPTObject, return 0 -jas
        return 0;
      }
      return pObj.GetClassID();
    }
  }
}