using System;
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.form;
namespace com.plumtree.portaluiinfrastructure.editor.saveobject
{
  /// <summary>
  /// This is the AS class for the save object pop-up window.
  /// The save object window will pop-up at the end of each
  /// object Editor in create mode.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class SaveObjectAS : AFormAS
  {
    
    /// <summary>
    /// Class name for the activity space
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "SaveObjectPopUp";
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.activityspace.AActivitySpace#CheckBasicAccess(String,String,boolean)"></seealso>
    public override bool CheckBasicAccess(String strPage, String strControl,
     bool bSameSpace)
    {
      // JF- This is a sub AS and the security Access has been checked by the Object Editor.
      // If somebody is trying to access this AS directly, it will fail.
      return true;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SaveObjectAS();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Returns the name of the repost control class.
    /// E.g.: PropRepostControl.GetName()
    /// </summary>
    /// <seealso cref="com.plumtree.xpshared.form.IRepostAS#GetRepostControlName()"></seealso>
    /// <returns>String</returns>
    public override String GetRepostControlName()
    {
      return SaveObjectRepostControl.STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IActivitySpace#Init()"></seealso>
    public override void Init()
    {
      base.Init();
      // Register Main Model
      RegisterModel(SaveObjectModel.STR_MVC_CLASS_NAME);
      IModel myModel = GetModel(SaveObjectModel.STR_MVC_CLASS_NAME);
      // Register Display Pages
      RegisterPage(new SaveObjectMainFrameDP());
      RegisterPage(new SaveObjectTopFrameDP());
      RegisterPage(new SaveObjectBottomFrameDP());
      RegisterPage(new SaveObjectRedirectDP());
      RegisterPage(new SaveObjectCantChangeFolderDP());
      RegisterPage(new SaveObjectRecentFoldersDP());
      RegisterPage(new SaveObjectFoldersFrameDP());
      RegisterPage(new SaveObjectGoToSecurityTabDP());
      RegisterPage(new SaveObjectClosePopUpDP());
      // Register Views
      RegisterView(SaveObjectNameDescView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(SaveObjectMenuView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(SaveObjectCantChangeFolderView.STR_MVC_CLASS_NAME, myModel);
      RegisterView(SaveObjectRecentFoldersView.STR_MVC_CLASS_NAME, myModel);
      // Register Controls
      RegisterControl(SaveObjectStartControl.STR_MVC_CLASS_NAME, myModel);
      RegisterControl(SaveObjectRepostControl.STR_MVC_CLASS_NAME, myModel);
      // JF- The Security Page should not display the header and the left navigation.
      AActivitySpace spaceParent = GetParentSpace();
      if (null != spaceParent && this is SaveObjectAS)
      {
        ((ObjectSecurityModel) spaceParent.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).SetShowHeaderAndLeftNav(false);
        }
    }
  }
}