using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.common;
namespace com.plumtree.portaluiinfrastructure.editor.saveobject
{
  /// <summary>
  /// This is the main Display Page for the save object pop-up window.
  /// It defines the 3 frames: top one with the tabs, body one with
  /// the tree or the security and the bottom one with the name and description.
  /// By default the body frame will display the folder tree.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class SaveObjectMainFrameDP : FrameSetDP
  {
    
    /// <summary>
    /// Frameset object name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "SaveObjectPopUpFrameMain";
    
    /// <summary>
    /// in pixels 
    /// </summary>
    public const int DEFAULT_BOTTOM_SIZE = 145;
    
    /// <summary>
    /// The size of the bottom frame for various languages. 
    /// </summary>
    public static readonly Object[][] BOTTOM_SIZES = {new Object[]{"ja", "zh",
     "ko"}, new Object[]{"160", "155", "150"}};
    
    /// <summary>
    /// the indices for the bottom frame size array. 
    /// </summary>
    public const int BOTTOM_SIZES_LANG = 0;
    public const int BOTTOM_SIZES_SIZE = 1;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.common.FrameSetDP#GetFrameScrollBody()"></seealso>
    public override bool GetFrameScrollBody()
    {
      return true;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.common.FrameSetDP#GetFrameScrollBottom()"></seealso>
    public override bool GetFrameScrollBottom()
    {
      return false;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.common.FrameSetDP#GetFrameSetAttributes()"></seealso>
    public override String GetFrameSetAttributes()
    {
      String strLang = m_asOwner.GetLanguage();
      int nBottomSize = DEFAULT_BOTTOM_SIZE;
      for (int i = 0;i < BOTTOM_SIZES[BOTTOM_SIZES_LANG].Length;i++)
      {
        if (0 == XPStringUtility.CompareIgnoreCase(strLang, (String) BOTTOM_SIZES[BOTTOM_SIZES_LANG][i]))
        {
          nBottomSize = XPConvert.ToInteger(BOTTOM_SIZES[BOTTOM_SIZES_SIZE][i]);
          }
      }
      return "60px,80%," + nBottomSize + "px";
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.common.FrameSetDP#GetFrameSetTitle()"></seealso>
    public override String GetFrameSetTitle()
    {
      ISaveObjectModelRO myModel = (ISaveObjectModelRO) m_asOwner.GetModel(SaveObjectModel.STR_MVC_CLASS_NAME);
      return myModel.GetTitle();
    }
    
    /// <summary>
    /// Returns the URL to open the body frame.
    /// </summary>
    /// <returns>ASURL</returns>
    public override ASURL GetFrameURLBody()
    {
      // Display the folder tree
      ASURL myBodyFrame = new ASURL(m_asOwner);
      ISaveObjectModelRO myModel = (ISaveObjectModelRO) m_asOwner.GetModel(SaveObjectModel.STR_MVC_CLASS_NAME);
      int nCurrentTab = myModel.GetCurrentTab();
      if (SaveObjectModel.SAVEOBJECT_TAB_FOLDERTREE == nCurrentTab)
      {
        if (myModel.ForceSaveInFolder())
        {
          myBodyFrame.SetLinkGetCachedSpace(m_asOwner);
          myBodyFrame.SetDisplayPage(SaveObjectCantChangeFolderDP.STR_MVC_CLASS_NAME);
          } else
        {
          myBodyFrame.SetLinkGetCachedSpace(m_asOwner);
          myBodyFrame.SetDisplayPage(SaveObjectFoldersFrameDP.STR_MVC_CLASS_NAME);
          }
      } else
      if (SaveObjectModel.SAVEOBJECT_TAB_SECURITY == nCurrentTab)
      {
        // JF- We go to the security page.
        myBodyFrame.SetLinkGetCachedSpace(m_asOwner.GetParentSpace());
        myBodyFrame.SetDisplayPage(ObjectSecurityDP.STR_MVC_CLASS_NAME);
      }
      return myBodyFrame;
    }
    
    /// <summary>
    /// Returns the URL to go to the Bottom Frame (name and description).
    /// </summary>
    /// <returns>ASURL</returns>
    public override ASURL GetFrameURLBottom()
    {
      ASURL result = new ASURL(m_asOwner);
      result.SetLinkGetCachedSpace(m_asOwner);
      result.SetDisplayPage(SaveObjectBottomFrameDP.STR_MVC_CLASS_NAME);
      return result;
    }
    
    /// <summary>
    /// Returns the URL to go to the Top Frame (tabs).
    /// </summary>
    /// <returns>ASURL</returns>
    public override ASURL GetFrameURLTop()
    {
      ASURL result = new ASURL(m_asOwner);
      result.SetLinkGetCachedSpace(m_asOwner);
      result.SetDisplayPage(SaveObjectTopFrameDP.STR_MVC_CLASS_NAME);
      return result;
    }
  }
}