using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
namespace com.plumtree.portaluiinfrastructure.friendlyurl
{
  /// <summary>
  /// This class handles friendly url requests for Search in ALI Portal.
  /// Currently it only supports banner search.
  /// </summary>
  /// <author>Ravi Tunuguntla</author>
  public class SearchFriendlyURLParser : FriendlyURLParser
  {
    
    /// <summary>
    /// Friendly URL Search query string parameter name.
    /// </summary>
    public const String SEARCH_QS = "q";
    
    /// <summary>
    /// Friendly URL Search sort string parameter name.
    /// </summary>
    public const String SORT_QS = "sort";
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.friendlyurl.SearchFriendlyURLParser");
    
    /// <summary>
    /// Public constructor that is initialized to parse friendly URL with search keyword.
    /// </summary>
    /// <param name="strKeyword">This is friendly URL for search keyword defined in friendlyURLs.xml</param>
    /// <param name="strClassId">This is the classid of friendly URL for search defined in FriendlyURLs.xml and FriendlyURLConstants.</param>
    /// <param name="stURLTokens">
    /// The tokens here are resource type, 
    /// resource name to identify in which resource the search has to take place. This is to support rest style url syntax.
    /// if no resources are defined in url searching will be done in all the resources.
    /// </param>
    public SearchFriendlyURLParser(String strKeyword, String strClassId, XPStringTokenizer stURLTokens)
    {
      m_strKeyword = strKeyword;
      m_strClassId = strClassId;
      // RT - This is where we parse the resource type and resource name in the URL
      // if any of these are empty then we search all the resources. This is not supported
      // yet. Right now we support searching all the resources.
      try
      {
        if (stURLTokens.HasMoreTokens())
        {
          // RT - this has to be the resource name
          String strObjName = stURLTokens.GetNextToken();
          m_strObjectName = URLDecode(strObjName);
        }
        if (stURLTokens.HasMoreTokens())
        {
          // RT - this has to be resource id.
          m_nObjectId = XPConvert.ToInteger(stURLTokens.GetNextToken());
        }
      }
      catch (Exception e)
      {
        log.Warn(e, "SearchFriendlyURLParser: Could not parse object name or Object ID properly.");
        }
    }
  }
}