using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs
{
  /// <summary>
  /// This is the base intrinsic portlet preferences model.  All intrinsic portlet
  /// models should extend this base class.  This model extends the editor model.
  /// </summary>
  /// <author>josephc</author>
  public abstract class IntrinsicPortletPrefsEditorModel : EditorModel,IEditorModelRO,
  IIntrinsicPortletPrefsModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IntrinsicPortletPrefsEditorModel");
    
    /// <summary>
    /// Setting type for user portlet preferences 
    /// </summary>
    public const int SETTINGSTYPE_PORTLET = 1;
    
    /// <summary>
    /// Setting type for admin preferences 
    /// </summary>
    public const int SETTINGSTYPE_ADMIN = 2;
    
    /// <summary>
    /// Setting type for community preferences 
    /// </summary>
    public const int SETTINGSTYPE_COMMUNITY = 3;
    
    /// <summary>
    /// Setting type for community portlet preferences 
    /// </summary>
    public const int SETTINGSTYPE_COMMUNITYPORTLET = 4;
    
    /// <summary>
    /// Setting type for user preferences 
    /// </summary>
    public const int SETTINGSTYPE_USER = 5;
    
    /// <summary>
    /// Preference type for admin preferences 
    /// </summary>
    public const int PREF_TYPE_ADMIN = 1;
    
    /// <summary>
    /// Preference type for personal preferences 
    /// </summary>
    public const int PREF_TYPE_PERSONAL = 2;
    
    /// <summary>
    /// Preference type for community preferences 
    /// </summary>
    public const int PREF_TYPE_COMMUNITY = 3;
    
    /// <summary>
    /// IPTGadget 
    /// </summary>
    protected IPTGadget m_ptPortlet;
    
    /// <summary>
    /// IPTSession 
    /// </summary>
    protected IPTSession m_ptSession;
    
    /// <summary>
    /// CommunityID 
    /// </summary>
    protected int m_nCommunityID = 0;
    
    /// <summary>
    /// PageID 
    /// </summary>
    protected int m_nPageID;
    
    /// <summary>
    /// PortletID 
    /// </summary>
    protected int m_nPortletID;
    
    /// <summary>
    /// Preference type 
    /// </summary>
    protected int m_nPrefType;
    protected String m_strPreviousURL;
    
    /// <summary>
    /// That method is called by the EditorStartControl.
    /// </summary>
    /// <param name="_htFormData"></param>
    /// <seealso cref="com.plumtree.uiinfrastructure.editor.EditorModel#StartEditor(XPHashtable)"></seealso>
    public override bool CheckEditorAccessAndStart(XPHashtable _htFormData)
    {
      // Set pageID, communityID, and portletID
      SetMemberVariablesFromQS(_htFormData);
      // Initialize the portlet
      InitializePortlet(_htFormData);
      // Perform Editor specific tasks
      DoTaskOnStartEditor(_htFormData);
      return true;
    }
    
    /// <returns></returns>
    public override int GetEditorType()
    {
      return base.GetEditorType();
    }
    
    /// <summary>
    /// Returns the DeleteOnCancel flag.
    /// For sub Editors only:
    /// this flag will determine if the Editor
    /// should be deleted from the list of sub Editors
    /// on cancel.
    /// </summary>
    /// <returns>boolean</returns>
    public override bool GetHideLeftNav()
    {
      return base.GetHideLeftNav();
    }
    
    /// <summary>
    /// Enables or disables the apply button next to the finish button.
    /// </summary>
    /// <param name="_bEnableApplyButton"></param>
    public override void SetEnableApplyButton(bool _bEnableApplyButton)
    {
      base.SetEnableApplyButton(_bEnableApplyButton);
    }
    
    /// <summary>
    /// Returns true if the apply button should be displayed.
    /// </summary>
    /// <returns>boolean</returns>
    public override bool GetEnableApplyButton()
    {
      return base.GetEnableApplyButton();
    }
    
    /// <summary>
    /// Returns true if the current user only has read access to this Editor.
    /// (can be true in Edit mode only)
    /// </summary>
    /// <returns>boolean</returns>
    public override bool GetIsEditorReadOnly()
    {
      return base.GetIsEditorReadOnly();
    }
    
    /// <summary>
    /// Get portlet name
    /// </summary>
    /// <returns>Portlet name</returns>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetPortletName()"></seealso>
    public virtual String GetPortletName()
    //JD- TODOCR 1/16/03
    //Remote gadgets have functionality where the portlet writer can set the title of the portlet.
    //This is done using CSP Protocol.  Not sure if you want to offer the same capability to 
    //intrinsic portlet writers.
    {
      return m_ptPortlet.GetLocalizedName();
    }
    
    /// <summary>
    /// Returns the type of preference that is being requested.
    /// </summary>
    /// <returns>Preference type</returns>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetPrefType()"></seealso>
    public virtual int GetPrefType()
    {
      return m_nPrefType;
    }
    
    /// <summary>
    /// Returns the Redirect object  (this is were we need to go at the end of
    /// the Editor). For prefs pages this just closes the popup window since
    /// the prefs page should be in a popup.
    /// </summary>
    /// <returns>Redirect</returns>
    public override Redirect GetRedirectAfterFinish()
    {
      Redirect myRedirect = new Redirect();
      if (m_asOwner.GetIsAccessStyleStandard())
      {
        HTMLScript script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
        try
        {
          script.AddInnerHTMLString("window.close();");
        }
        catch (HTMLException he)
        {
          log.Error(he, "EditorRepostControl::SetRedirectToCloseSubEditor An error occured when trying to display the JavaScript.");
          }
        ClosePopUpDP myClosePopUp = (ClosePopUpDP) m_asOwner.GetPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
        myClosePopUp.SetScript(script);
        myRedirect.SetLinkGetCachedSpace(m_asOwner);
        myRedirect.SetDisplayPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
      } else
      {
        myRedirect.SetIsErrorRedirect(false);
        myRedirect.SetLinkToExternalURL(m_strPreviousURL);
      }
      return myRedirect;
    }
    
    /// <summary>
    /// Returns a single setting value, based on the setting type specified.
    /// </summary>
    /// <param name="nSettingType">Setting type</param>
    /// <param name="strName">Setting name</param>
    /// <returns>Setting value</returns>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetSettingValue(int,String)"></seealso>
    public virtual Object GetSettingValue(int nSettingType, String strName)
    {
      String[] arFilter = new String[]{strName};
      Object[][] arValues;
      Object oValue = null;
      arValues = GetSettingValues(nSettingType, arFilter);
      if ((null != arValues) && (arValues.Length > 0))
      {
        oValue = arValues[1][0];
      }
      return oValue;
    }
    //JD- TODOCR 1/16/03
    //There are helper functions to get user preferences in LoginHelper.  I'm not sure whether
    //or not they should be used here... but either way there should be a comment specifying 
    //what the decision is, and why was it made.
    //JD- TODOCR 1/16/03
    // Perhaps it's worth doing a sanity check that if community or community-gadget prefs
    // are set, it's happenning on a community.  This applies to all functions that set,
    // read, or remove a pref.
    //JD- TODOCR 1/16/03
    // How are you handling security for admin prefs? What happens if someone who does not
    // have edit rights on the Portlet object tries to set admin prefs?
    
    /// <summary>
    /// Returns all settings, based on the setting type and filter specified.
    /// </summary>
    /// <param name="nSettingType">Setting type</param>
    /// <param name="arFilter">Filter for settings</param>
    /// <returns>All settings</returns>
    /// <seealso cref="com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetSettingValues(int,Object[])"></seealso>
    public virtual Object[][] GetSettingValues(int nSettingType, String[] arFilter)
    {
      Object[][] arValues;
      switch (nSettingType){
      case SETTINGSTYPE_PORTLET:
      if (PREF_TYPE_COMMUNITY == m_nPrefType)
      {
        arValues = (Object[][]) ((IPTCommunityManager) m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nCommunityID,
         true).LookupPreferences(arFilter, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_CURRENT_USER);
        } else
      {
        arValues = (Object[][]) m_ptSession.GetSessionInfo().LookupPreferences(arFilter,
         m_nPortletID);
      }
      //arValues = (Object[][]) m_ptMyPortal.LookupGadgetSettings(m_nPortletID, arFilter);
      break;
      case SETTINGSTYPE_ADMIN:
      {
        IPTAdminSettings ptAdminSettings = (IPTAdminSettings) m_ptPortlet.GetInterfaces("IPTAdminSettings");
        arValues = (Object[][]) ptAdminSettings.LookupAdminGadgetSettings(arFilter);
        break;
      }
      case SETTINGSTYPE_COMMUNITY:
      arValues = (Object[][]) ((IPTCommunityManager) m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nCommunityID,
       true).LookupPreferences(arFilter, 0, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
      //arValues = (Object[][]) m_ptMyPortal.LookupCommunitySettings(arFilter);
      break;
      case SETTINGSTYPE_COMMUNITYPORTLET:
      arValues = (Object[][]) ((IPTCommunityManager) m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nCommunityID,
       true).LookupPreferences(arFilter, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
      //arValues = (Object[][]) m_ptMyPortal.LookupCommunityGadgetSettings(m_nPortletID, arFilter);
      break;
      case SETTINGSTYPE_USER:
      arValues = (Object[][]) m_ptSession.GetSessionInfo().LookupPreferences(arFilter,
       0);
      //arValues = (Object[][]) m_ptMyPortal.LookupPersonalSettings(arFilter);
      break;
      default:
      return null;
      break;
      }
      return arValues;
    }
    
    /// <summary>
    /// Inits the Model with the IPTSession.
    /// </summary>
    /// <param name="parent"></param>
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
      // JF- Get the Session object from the associated AS.
      m_ptSession = (IPTSession) parent.GetUserSession();
      m_strPreviousURL = HTMLConstructHelpers.GetPreviousSpaceURL(m_asOwner);
      }
    
    /// <summary>
    /// Removes a setting value, depending on the specified setting type.
    /// </summary>
    /// <param name="nSettingType">Setting type</param>
    /// <param name="strName">Setting name</param>
    public virtual void RemoveSettingValue(int nSettingType, String strName)
    {
      switch (nSettingType){
      case SETTINGSTYPE_PORTLET:
      if (PREF_TYPE_COMMUNITY == m_nPrefType)
      {
        ((IPTCommunity) m_ptSession.GetCommunities().Open(m_nCommunityID,
         false)).RemovePreference(strName, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_CURRENT_USER);
        } else
      {
        m_ptSession.GetSessionInfo().RemovePreference(strName, m_nPortletID);
        }
      //m_ptMyPortal.RemoveGadgetSetting(m_nPortletID, strName);
      break;
      case SETTINGSTYPE_ADMIN:
      {
        IPTAdminSettings ptAdminSettings = (IPTAdminSettings) m_ptPortlet.GetInterfaces("IPTAdminSettings");
        ptAdminSettings.RemoveAdminGadgetSetting(strName);
        break;
      }
      case SETTINGSTYPE_COMMUNITY:
      ((IPTCommunity) m_ptSession.GetCommunities().Open(m_nCommunityID, false)).RemovePreference(strName,
       0, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
      //m_ptMyPortal.RemoveCommunitySetting(strName);
      break;
      case SETTINGSTYPE_COMMUNITYPORTLET:
      ((IPTCommunity) m_ptSession.GetCommunities().Open(m_nCommunityID, false)).RemovePreference(strName,
       m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
      //m_ptMyPortal.RemoveCommunityGadgetSetting(m_nPortletID, strName);
      break;
      case SETTINGSTYPE_USER:
      m_ptSession.GetSessionInfo().RemovePreference(strName, 0);
      //m_ptMyPortal.RemovePersonalSetting(strName);
      break;
      default:
      break;
      }
    }
    
    /// <summary>
    /// Helper method for the StartEditor method. Fill out the member variables
    /// with data from QS.
    /// </summary>
    /// <param name="_htFormData"></param>
    public virtual void SetMemberVariablesFromQS(XPHashtable _htFormData)
    {
      // Set the page ID if it exists
      String[] data = (String[]) _htFormData.GetElement(IntrinsicPortletPrefsUtils.PAGEID);
      if ((null != data) && (0 != data.Length) && (data[0] != null))
      {
        m_nPageID = XPConvert.ToInteger(data[0]);
        log.Debug("IntrinsicPortletPrefsEditorModel::SetMemberVariablesFromQS PageID: " + data[0]);
        }
      // Set the community ID if it exists
      data = (String[]) _htFormData.GetElement(IntrinsicPortletPrefsUtils.COMMUNITYID);
      if ((null != data) && (0 != data.Length) && (data[0] != null))
      {
        m_nCommunityID = XPConvert.ToInteger(data[0]);
        log.Debug("IntrinsicPortletPrefsEditorModel::SetMemberVariablesFromQS CommunityID: " + data[0]);
        }
      // Set the portlet ID if it exists
      data = (String[]) _htFormData.GetElement(IntrinsicPortletPrefsUtils.PORTLETID);
      if ((null != data) && (0 != data.Length) && (data[0] != null))
      {
        m_nPortletID = XPConvert.ToInteger(data[0]);
        log.Debug("IntrinsicPortletPrefsEditorModel::SetMemberVariablesFromQS PortletID: " + data[0]);
        }
    }
    
    /// <summary>
    /// Sets the appropriate setting value, based on the setting type.
    /// </summary>
    /// <param name="nSettingType">Setting type</param>
    /// <param name="strName">Setting name</param>
    /// <param name="oValue">Setting value</param>
    public virtual void SetSettingValue(int nSettingType, String strName,
     Object oValue)
    {
      Object[][] arValues;
      switch (nSettingType){
      case SETTINGSTYPE_PORTLET:
      arValues = new Object[2][];
      for (int j2c_index = 0;j2c_index < 2;j2c_index++)
      {
        arValues[j2c_index] = new Object[1];
      }
      arValues[0][0] = strName;
      arValues[1][0] = oValue;
      if (PREF_TYPE_COMMUNITY == m_nPrefType)
      {
        ((IPTCommunity) m_ptSession.GetCommunities().Open(m_nCommunityID,
         false)).AssignPreferences(arValues, false, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_CURRENT_USER);
        } else
      {
        m_ptSession.GetSessionInfo().AssignPreferences(arValues, false, m_nPortletID);
        }
      //m_ptMyPortal.AssignGadgetSettingsEx(m_nPortletID, arValues, false);
      break;
      case SETTINGSTYPE_ADMIN:
      {
        IPTAdminSettings ptAdminSettings = (IPTAdminSettings) m_ptPortlet.GetInterfaces("IPTAdminSettings");
        ptAdminSettings.AddAdminGadgetSetting(strName, oValue);
        break;
      }
      case SETTINGSTYPE_COMMUNITY:
      ((IPTCommunity) m_ptSession.GetCommunities().Open(m_nCommunityID, false)).AddPreference(strName,
       oValue, 0, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
      //m_ptMyPortal.AddCommunitySetting(strName, oValue);
      break;
      case SETTINGSTYPE_COMMUNITYPORTLET:
      arValues = new Object[2][];
      for (int j2c_index = 0;j2c_index < 2;j2c_index++)
      {
        arValues[j2c_index] = new Object[1];
      }
      arValues[0][0] = strName;
      arValues[1][0] = oValue;
      ((IPTCommunity) m_ptSession.GetCommunities().Open(m_nCommunityID, false)).AssignPreferences(arValues,
       false, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
      //m_ptMyPortal.AssignCommunityGadgetSettingsEx(m_nPortletID, arValues, false);
      break;
      case SETTINGSTYPE_USER:
      m_ptSession.GetSessionInfo().AddPreference(strName, oValue, 0);
      //m_ptMyPortal.AddPersonalSetting(strName, oValue);
      break;
      default:
      break;
      }
    }
    public virtual int GetCommunityId()
    {
      return m_nCommunityID;
    }
    
    /// <summary>
    /// If this model is started as a sub-editor from the Portlet editor's model, then
    /// get the portlet object from the editor model using this method.
    /// </summary>
    /// <param name="asParent">Parent space of editor when started as a subeditor</param>
    /// <returns>Portlet object from the PortletEditor's model</returns>
    private IPTGadget GetPortletFromSpace(AActivitySpace asParent)
    {
      // Use "Portlet" here instead of using PortletModel.STR_MVC_CLASS_NAME because
      // we don't want to introduce dependency on portal pages.
      // The "correct" way might be to iterate through all the models from the parent space,
      // looking for an ObjEditorModel, which would be our PortletModel.
      /*PortletModel.STR_MVC_CLASS_NAME*/
      ObjEditorModel oModel = (ObjEditorModel) asParent.GetModel("Portlet");
      return (IPTGadget) oModel.GetObject();
    }
    
    /// <summary>
    /// Initialize the member variables of the model.  If the pageID is 0, then
    /// we are editing admin pref, and so get the IPTMyPortal object from the
    /// parent space and don't create an AppDataState object.
    /// </summary>
    /// <param name="_htFormData">Table containing form data</param>
    protected virtual void InitializePortlet(XPHashtable _htFormData)
    {
      AActivitySpace asParent = m_asOwner.GetParentSpace();
      try
      {
        // Check if we're coming from Portlet Prefs editor or Portlet editor
        if (null != asParent)
        {
          // We're coming from Portlet editor.
          m_ptPortlet = GetPortletFromSpace(asParent);
          if (0 == m_nPortletID)
          {
            IPTServerContext ptSC = (IPTServerContext) m_ptPortlet.GetInterfaces("IPTServerContext");
            m_nPortletID = ptSC.GetObjectID();
          }
          m_nPrefType = PREF_TYPE_ADMIN;
        } else
        {
          // We're coming from Portlet Prefs editor
          if (0 < m_nCommunityID)
          {
            m_nPrefType = PREF_TYPE_COMMUNITY;
          } else
          {
            m_nPrefType = PREF_TYPE_PERSONAL;
          }
          // Get the portlet object
          m_ptPortlet = (IPTGadget) m_ptSession.GetGadgets().Open(m_nPortletID,
           false);
        }
      }
      catch (Exception e)
      {
        log.Error(e, "Error while initialize portlet preferences editor.");
        throw XPException.GetInstance(e);
      }
    }
  }
}