using System;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.portaluiinfrastructure.layout
{
  /// <summary>
  /// This class is responsible for determining which Layout Web Service
  /// to use for each page.
  /// </summary>
  public interface ILayoutChooser
  {
    
    /// <summary>
    /// Calculate the web service ID to use for the current page
    /// from the current Experience Definition.
    /// </summary>
    /// <param name="Layouts">the layout to used to calculate the layout Web Service.</param>
    /// <param name="AActivitySpace">the current Activity Space.</param>
    /// <returns>
    /// int the Web Service ID to use for this page.
    /// -1 means layout is not enabled for this page at this time. 
    /// </returns>
    int GetLayoutWebService(Layouts _layout, AActivitySpace _asOwner);
    
    /// <summary>
    /// Get the web services that are available for the requested layout.
    /// These are stored in a special folder (one folder per layout) in
    /// the Admin Directory.
    /// </summary>
    /// <param name="_layout">the requested layout</param>
    /// <param name="AActivitySpace">the current Activity Space.</param>
    /// <returns>
    /// IPTQueryResult The list of available web services for this
    /// layout with name, description, and object ID information.
    /// </returns>
    IPTQueryResult GetAvailableWebServices(Layouts _layout, AActivitySpace _asOwner);
    
    /// <returns>whether or not page layouts are enabled globally.</returns>
    bool GetPageLayoutsEnabled(AActivitySpace _asOwner);
    
    /// <returns>whether or not portlet layouts are enabled globally.</returns>
    bool GetPortletLayoutsEnabled(AActivitySpace _asOwner);
  }
}