/*
 * Created on Mar 27, 2003
 */
 
using System;
using com.plumtree.openfoundation.util;
namespace com.plumtree.portaluiinfrastructure.navtype
{
  /// <summary>
  /// Helper class to assist in the enumeration of navigation 
  /// feature types
  /// </summary>
  /// <author>DavidP</author>
  public class NavFeatureHelper
  {
    private static XPArrayList navFeatureArray = null;
    public static readonly NavFeatureHelper INSTANCE = new NavFeatureHelper();
    
    /// <summary>
    /// Constructs helper class with accessors for array of nav types.
    /// </summary>
    public NavFeatureHelper()
    {
      navFeatureArray = new XPArrayList();
      for (int i = 0;i < NavFeatureEnum.m_navFeatureEnums.Length;++i)
      {
        navFeatureArray.Add(NavFeatureEnum.m_navFeatureEnums[i]);
      }
    }
    
    /// <summary>
    /// Enumerates the possible nav types available.
    /// </summary>
    /// <returns>IXPEnumerator</returns>
    public virtual IXPEnumerator GetCategoryTypes()
    {
      IXPEnumerator eKeys = null;
      if (navFeatureArray != null && navFeatureArray.GetSize() > 0)
      {
        eKeys = navFeatureArray.GetEnumerator();
      }
      return eKeys;
    }
    
    /// <summary>
    /// Enumerates the possible nav types available.
    /// </summary>
    /// <returns>XPArrayList</returns>
    public virtual XPArrayList GetCategoryTypeArray()
    {
      return navFeatureArray;
    }
    
    /// <summary>
    /// Returns the number of nav types.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetNumCategoryTypes()
    {
      if (navFeatureArray != null)
      {
        return navFeatureArray.GetSize();
      }
      return 0;
    }
    
    /// <summary>
    /// Get the name if the view class from pluggable navigation to be registered 
    /// in the AActivitySpace and called by DisplayPage to render that area of 
    /// the page
    /// </summary>
    /// <param name="plugNav"></param>
    /// <param name="pageArea"></param>
    /// <returns>String</returns>
    public virtual bool GetNavFeatureEnabled(INavTypes plugNav, NavFeatureEnum featureEnabled)
    {
      return plugNav.IsFeatureEnabled(featureEnabled);
    }
  }
}