using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.server;
namespace com.plumtree.portaluiinfrastructure.profile
{
  /// <author>
  /// jennifersu
  /// Helper class that describes a Property in the User Profile
  /// Layout.
  /// </author>
  public sealed class Property
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("Portal_UserProfile"),
     "com.plumtree.portaluiinfrastructure.profile.Property");
    private Object m_value;
    private String m_name;
    private Object[][] m_arrPickList = null;
    private int m_id;
    private int m_picktype;
    private int m_type;
    private int m_isHREF;
    private int m_isMandatory;
    private int m_isMultiValued;
    private int m_isReadOnly;
    private int m_isVisible;
    private int m_refClassID;
    private String m_refName;
    
    /// <param name="_id"></param>
    /// <param name="_name"></param>
    /// <param name="_type"></param>
    /// <param name="_value"></param>
    /// <param name="_picktype"></param>
    /// <param name="_isHREF"></param>
    /// <param name="_isMandatory"></param>
    /// <param name="_isMultiValued"></param>
    /// <param name="_isReadOnly"></param>
    /// <param name="_isVisible"></param>
    /// <param name="_refClassID"></param>
    /// <param name="_arrPickList"></param>
    public Property(int _id, String _name, int _type, Object _value, int _picktype,
     int _isHREF, int _isMandatory, int _isMultiValued, int _isReadOnly, int _isVisible,
     int _refClassID, Object[][] _arrPickList)
    {
      m_id = XPConvert.ToInteger(_id);
      m_name = (new XPStringBuilder(_name)).ToString();
      m_type = XPConvert.ToInteger(_type);
      m_value = _value;
      m_picktype = XPConvert.ToInteger(_picktype);
      m_isHREF = XPConvert.ToInteger(_isHREF);
      m_isMandatory = XPConvert.ToInteger(_isMandatory);
      m_isMultiValued = XPConvert.ToInteger(_isMultiValued);
      m_isReadOnly = XPConvert.ToInteger(_isReadOnly);
      m_isVisible = XPConvert.ToInteger(_isVisible);
      m_refClassID = XPConvert.ToInteger(_refClassID);
      m_refName = null;
      m_arrPickList = _arrPickList;
    }
    
    /// <summary>
    /// Gets the ID of this Property object.
    /// </summary>
    /// <returns>The id of this Property object.</returns>
    public int GetID()
    {
      return m_id;
    }
    
    /// <summary>
    /// Gets the name of this Property object.
    /// </summary>
    /// <returns>The name of this Property object.</returns>
    public String GetName()
    {
      return m_name;
    }
    
    /// <summary>
    /// Get the picklist values from the property object
    /// </summary>
    /// <returns>
    /// Object[][] an object array of the picklist values. If there is
    /// nothing it returns null
    /// </returns>
    public Object[][] GetPickListValues()
    {
      throw new XPException("This method has been desupported.  Use IPTProperty.GetCharacteristicAsObject2DArray(PT_PROPERTY_CHARACTERISTICS.PT_PROPCHAR_PICKTABLECOLS.");
      }
    
    /// <summary>
    /// Gets the pick type of this Property object
    /// </summary>
    /// <returns>int The pick type for this object property</returns>
    public int GetPickType()
    {
      return m_picktype;
    }
    
    /// <summary>
    /// Gets the type of this Property object.
    /// </summary>
    /// <returns>
    /// The type of this Property object. Will be one of
    /// PT_PROPERTY_TYPES.
    /// </returns>
    public int GetType()
    {
      return m_type;
    }
    
    /// <summary>
    /// Returns the value of this Property object.
    /// </summary>
    /// <returns>The value of this Property object.</returns>
    public Object GetValue()
    {
      return m_value;
    }
    
    /// <returns>true if the Property is treated as an HREF</returns>
    public bool IsHREF()
    {
      return XPConvert.ToBoolean(m_isHREF);
    }
    
    /// <returntrue>if the Property is mandatory</returntrue>
    public bool IsMandatory()
    {
      return XPConvert.ToBoolean(m_isMandatory);
    }
    
    /// <returns>true if the Property supports multiple values</returns>
    public bool IsMultiValued()
    {
      return XPConvert.ToBoolean(m_isMultiValued);
    }
    
    /// <returns>true if the Property is read-only</returns>
    public bool IsReadOnly()
    {
      return XPConvert.ToBoolean(m_isReadOnly);
    }
    
    /// <returns>true if the Property is visible in the UI</returns>
    public bool IsVisible()
    {
      return XPConvert.ToBoolean(m_isVisible);
    }
    
    /// <param name="iPickType"></param>
    /// <returns>true if the Property is a pick list</returns>
    private bool HasPickList(int iPickType)
    {
      switch (iPickType){
      case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_MANAGED_PICKLIST:
      case PT_PICKTABLE_TYPES.PT_PICKTABLE_TYPE_UNMANAGED_PICKLIST:
      return true;
      goto default;
      default:
      return false;
      break;
      }
    }
    
    /// <param name="refName"></param>
    public void SetRefName(String refName)
    {
      m_refName = refName;
    }
    
    /// <returns>class ID of the reference type</returns>
    public int GetRefClassID()
    {
      return m_refClassID;
    }
    
    /// <returns>name of the reference value</returns>
    public String GetRefName()
    {
      return m_refName;
    }
    
    /// <param name="_value"></param>
    public void SetValue(Object _value)
    {
      m_value = _value;
    }
  }
}