using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.search.exceptions;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
using com.plumtree.server.search.exceptions;
namespace com.plumtree.portaluiinfrastructure.search
{
  /// <summary>
  /// Some common stuff for the Search UI.
  /// </summary>
  /// <author>MatthewF</author>
  public class SearchUIUtility
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.search.SearchUIUtility");
    public const String COM_EMPTY_SEARCH_STRING = "Empty search string";
    public const String COM_EMPTY_QUERY = "Empty query";
    public const String COM_CANT_CONNECT = "Unable to connect to search server";
    public const String COM_EMPTY_SERVER_LIST = "Working server list is empty";
    public const String COM_NO_ACTIVE = " no active ";
    public const String COM_MIDSTRING_WILDCARD = "Midstring wildcard";
    public const String COM_INAPPROPRIATE_OP = "inappropriate operator";
    public const String COM_BAD_COMPARISON_VALUE = "incompatible comparison value";
    public const String COM_SOCKET_TIMEOUT = "SOCKET_TIMEOUT";
    public const String COM_TWO_OPERATORS = "Query contains two operators in a row";
    public const String COM_ENDS_OPERATOR = "Query ends with an operator";
    public const String COM_TOO_MANY_PARENS = "too many right";
    public const String COM_NO_CONSTRAINTS = "had no constraints";
    public static HTMLImg GetImage(AActivitySpace space, PTImageType imageType,
     String altText, String width, String height)
    {
      PTConstruct iconButton = null;
      try
      {
        iconButton = UIConstructFactory.GetConstruct(imageType, true, space);
        }
      catch (Exception e)
      {
      }
      //PTDebug.Trace message here.
      if (null != iconButton)
      {
        HTMLImg buttonImg = (HTMLImg) iconButton.GetHTMLElement();
        buttonImg.SetAlt(altText);
        buttonImg.SetWidth(width);
        buttonImg.SetHeight(height);
        return buttonImg;
      }
      return null;
    }
    
    /// <param name="height"></param>
    /// <returns></returns>
    /// <throws>HTMLException </throws>
    public static HTMLElement makeSpacer(AActivitySpace activitySpace, String height)
    {
      HTMLTable table = new HTMLTable();
      table.SetCellSpacing("0");
      table.SetCellPadding("0");
      table.SetWidth("100%");
      HTMLTableRow row = new HTMLTableRow();
      table.AddInnerHTMLElement(row);
      HTMLTableCell cell = new HTMLTableCell();
      row.AddInnerHTMLElement(cell);
      HTMLImg img = null;
      try
      {
        img = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.SPACER,
         true, activitySpace).GetHTMLElement();
      }
      catch (Exception e)
      {
        // error to spy.
        throw new HTMLException(e.Message);
      }
      cell.AddInnerHTMLElement(img);
      img.SetHeight(height);
      img.SetWidth("1");
      img.SetBorder("0");
      img.SetAlign("absMiddle");
      return table;
    }
    
    /// <summary>
    /// Helper method to translate and rethrow a COM exception as
    /// a SearchExecutionException
    /// </summary>
    public static void RethrowCOMExecutionException(Exception e, AActivitySpace m_asOwner)
    {
      // log the error
      log.Error(e, "Rethrowing search com exception.");
      SearchExecutionException exception = GetI18NException(e, m_asOwner);
      throw exception;
    }
    
    /// <summary>
    /// Helper method to translate  a COM exception into an internationalized
    /// SearchExecutionException
    /// </summary>
    public static SearchExecutionException GetI18NException(Exception e, AActivitySpace m_asOwner)
    {
      // First, check whether the exception is a known type
      if (e is PTSEmptyQueryException)
      {
        return new EmptyQueryException(m_asOwner);
      } else
      if (e is PTSUnsupportedWildcardException)
      {
        return new InsufficientWildcardCharactersException(m_asOwner);
      } else
      if (e is PTSAdjacentOperatorsException)
      {
        return new TwoOperatorsException(m_asOwner);
      } else
      if (e is PTSEndsInOperatorException)
      {
        return new EndsInOperatorsException(m_asOwner);
      } else
      if (e is PTSInappropriateOperatorException)
      {
        return new InappropriateOperatorException(m_asOwner);
      } else
      if (e is PTSUnbalancedParenthesesException)
      {
        return new TooManyRightParensException(m_asOwner);
      }
      // Distinguish other types by parsing the exception message
      String message = e.Message;
      if (-1 != XPStringUtility.IndexOf(message, COM_EMPTY_SEARCH_STRING))
      {
        return new EmptyQueryException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_EMPTY_QUERY))
      {
        return new EmptyQueryException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_CANT_CONNECT))
      {
        return new NoSearchServerException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_EMPTY_SERVER_LIST))
      {
        return new NoSearchServerException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_NO_ACTIVE))
      {
        return new NoSearchServerException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_MIDSTRING_WILDCARD))
      {
        return new InsufficientWildcardCharactersException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_INAPPROPRIATE_OP))
      {
        return new InappropriateOperatorException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_BAD_COMPARISON_VALUE))
      {
        return new QueryParseException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_TWO_OPERATORS))
      {
        return new TwoOperatorsException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_SOCKET_TIMEOUT))
      {
        return new SocketTimeoutException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_ENDS_OPERATOR))
      {
        return new EndsInOperatorsException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_TOO_MANY_PARENS))
      {
        return new TooManyRightParensException(m_asOwner);
      } else
      if (-1 != XPStringUtility.IndexOf(message, COM_NO_CONSTRAINTS))
      {
        return new EmptyQueryException(m_asOwner);
      } else
      {
        String msg = m_asOwner.GetString(344, "ptmsgs_portalinfrastructure");
        XPException innerException = XPException.GetInstance(e);
        return new Error6321Exception(msg, new XPException(innerException));
        }
    }
    public static bool ExceptionHiddenInPageSource(SearchExecutionException exception)
    {
      if (exception == null)
      {
        return false;
      }
      return (exception is Error6321Exception);
    }
  }
}