using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.sso;
using com.plumtree.portaluiinfrastructure.statichelpers;
namespace com.plumtree.portaluiinfrastructure.sso.integrations
{
  /// <summary>
  /// Simple class used to test SSO.
  /// </summary>
  /// <author>Mike Jahr</author>
  public class SSOTest : ISSOIntegration
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.sso.integrations.SSOTest");
    
    /// <summary>
    /// Creates a new SSOTest object.
    /// </summary>
    public SSOTest()
    {
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#GetLoginInfo()"></seealso>
    public virtual SSOLoginInfo GetLoginInfo(IXPRequest request)
    {
      String strUserName = request.GetHeader("PT_USERNAME");
      if (strUserName == null || strUserName.Length == 0)
      {
        strUserName = "SSOTestUser";
      }
      return new SSOLoginInfo(request.GetHeader("PT_DOMAIN"), strUserName,
       request.GetHeader("PT_PASSWORD"));
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#GetSSOProductName()"></seealso>
    public virtual String GetSSOProductName()
    {
      return "Plumtree Test SSO";
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#GetSecureCookies()"></seealso>
    public virtual String[] GetSecureCookies()
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#GetSecureHeaders()"></seealso>
    public virtual String[] GetSecureHeaders()
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#Initialize()"></seealso>
    public virtual bool Initialize(XPHashtable mapSettings)
    {
      return true;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.sso.ISSOIntegration#Logout()"></seealso>
    public virtual bool OnLogout(IXPResponse response, String sReturnURL)
    {
      String sURL = "http://qweb2/gsroot/sso/redirect.asp?url=" + XPHttpUtility.UrlEncode(sReturnURL,
       "UTF-8");
      response.Redirect(sURL);
      log.Info("SSOTest is redirecting to " + sURL);
      return true;
    }
  }
}