using System;
using com.plumtree.openfoundation.io;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openfoundation.xml.dom;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.classtypedescriptors;
using com.plumtree.portaluiinfrastructure.friendlyurl;
using com.plumtree.portaluiinfrastructure.helpers;
using com.plumtree.portaluiinfrastructure.login;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.portaluiinfrastructure.statichelpers
{
  /// <author>JaneD</author>
  public class AppDataObject
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.statichelpers.AppDataObject");
    
    /// <summary>
    /// bjf -- this must match PT_APPDATASTATEOBJECT_COLLAPSEDGADGETS in the server 
    /// </summary>
    public const String PT_COLLAPSEDGADGETS = "CollapsedGadgets";
    public const String m_strDefaultHeaders = "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, " + "application/vnd.ms-powerpoint, application/vnd.ms-excel, application/msword, */*\r\n" + "Accept-Language: en-us\r\n" + "Accept-Encoding: gzip, deflate\r\n" + "User-Agent: Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.0.3705)\r\n" + "Connection: Keep-Alive\r\n";
    
    /// <summary>
    /// Alternate Stylesheet that will be used when LayoutMode is enabled, A configuration will also
    /// be used to determine language specific user configured stylesheet.
    /// </summary>
    private static String PT_LAYOUT_STYLESHEET = "portal/private/css/mainstyle.css";
    
    /// <summary>
    /// configuration file to determine language specific user configured stylesheet. 
    /// </summary>
    private static String PT_LANGUAGE_FILE = "CustomStyles.xml";
    
    /// <summary>
    /// Pattern that contains language and style settings as its leaf nodes 
    /// </summary>
    private static String PT_STYLE_PATTERN = "StyleSettings";
    
    /// <summary>
    /// Language Key in the settings file 
    /// </summary>
    private static String PT_LANGUAGE_KEY = "language";
    
    /// <summary>
    /// Node name that holds reference to ths stylesheet for a given language 
    /// </summary>
    private static String PT_STYLE_KEY = "styles";
    private static XPHashtable customSheetList;
    //JD- Default values for AppData.
    public static Object[][] m_arrAppData = {new Object[]{PlumtreeExtensibility.PT_CAN_SET_ADMIN_KEY,
     0}, new Object[]{PlumtreeExtensibility.PT_CAN_SET_PERSONAL_KEY, 1}, new Object[]{PlumtreeExtensibility.PT_CAN_SET_COMMUNITY_KEY,
     0}, new Object[]{PlumtreeExtensibility.PT_HTTP_METHOD_KEY, "GET"}, new Object[]{PlumtreeExtensibility.PT_PAGE_ID_KEY,
     0}, new Object[]{PlumtreeExtensibility.PT_COMMUNITY_ID_KEY, 0}, new Object[]{PlumtreeExtensibility.PT_STYLESHEET_URI_KEY,
     PTConfigHelper.GetImageServerBaseURL(false) + "common/public/css/mainstyle-en.css"},
     new Object[]{PlumtreeExtensibility.PT_RETURN_URI_KEY, "http://hostpageuriwillbehere/"},
     new Object[]{PlumtreeExtensibility.PT_GATEWAY_URI_KEY, "http://hostpageuriwillbehere/"},
     new Object[]{PlumtreeExtensibility.PT_SECURE_GATEWAY_URI_KEY, "http://hostpageuriwillbehere/"},
     new Object[]{PlumtreeExtensibility.PT_HOST_PAGE_URI_KEY, "http://hostpageuriwillbehere/"},
     new Object[]{PlumtreeExtensibility.PT_IMAGESERVER_URI_KEY, "http://imageserverurlnotset"},
     new Object[]{
    // The real URL will be set in the GetAppDataObject method below
    PlumtreeExtensibility.PT_GATEWAY_VERSION_KEY, "2.5"}, new Object[]{PlumtreeExtensibility.PT_CLASS_ID_KEY,
     PT_CLASSIDS.PT_GADGET_ID}, new Object[]{PlumtreeExtensibility.PT_HEADERS_KEY,
     m_strDefaultHeaders}, new Object[]{PlumtreeExtensibility.PT_CHARSET_KEY,
     LinkConstants.UTF_8_ENCODING}, new Object[]{PlumtreeExtensibility.PT_LOCALE_ID_KEY,
     "en-us"}, new Object[]{PlumtreeExtensibility.PT_TIMEZONE_KEY, "America/Los_Angeles"},
     new Object[]{PlumtreeExtensibility.PT_REFRESH_URL_KEY, "http://refreshurlwillbehere"},
     new Object[]{PlumtreeExtensibility.PT_REFRESH_TYPE_KEY, PlumtreeExtensibility.PT_REFRESH_NONE},
     new Object[]{PlumtreeExtensibility.PT_508_MODE_KEY, 0}};
    // Returns the app data state object. This function is used by everyone but the gateway.
    public static IPTState GetAppDataObject(int iCommunityID, int iPageID,
     String strHostPageURI, AActivitySpace asOwner)
    {
      return GetAppDataObject(iCommunityID, iPageID, 0, strHostPageURI, asOwner);
      }
    // Returns the app data state object. This function is used by everyone but the gateway.
    public static IPTState GetAppDataObject(int iCommunityID, int iPageID,
     String strHostPageURI, AActivitySpace asOwner, bool bResolveSecureBase)
    {
      return GetAppDataObject(iCommunityID, iPageID, 0, strHostPageURI, asOwner,
       bResolveSecureBase);
    }
    
    /// <summary>
    /// Return GetAppDataObject(int iCommunityID, int iPageID, int iGWParams,
    /// String strHostPageURI, AActivitySpace asOwner,
    /// boolean bResolveSecureBase)
    /// with the default value of bResolveSecureBase set to true.
    /// </summary>
    /// <param name="iCommunityID"></param>
    /// <param name="iPageID"></param>
    /// <param name="iGWParams"></param>
    /// <param name="strHostPageURI"></param>
    /// <param name="asOwner"></param>
    /// <returns></returns>
    public static IPTState GetAppDataObject(int iCommunityID, int iPageID,
     int iGWParams, String strHostPageURI, AActivitySpace asOwner)
    {
      return GetAppDataObject(iCommunityID, iPageID, iGWParams, strHostPageURI,
       asOwner, true);
    }
    
    /// <summary>
    /// Returns the app data state object. This function is used by the gateway directly.
    /// The bResolveSecureBase specifies that in security mode 3 the method should resolve
    /// strSecureAppBaseURL by passing it to PTConfigHelper.GetApplicationBaseURL or not
    /// </summary>
    /// <param name="iCommunityID"></param>
    /// <param name="iPageID"></param>
    /// <param name="iGWParams"></param>
    /// <param name="strHostPageURI"></param>
    /// <param name="asOwner"></param>
    /// <param name="bResolveSecureBase"></param>
    /// <returns></returns>
    public static IPTState GetAppDataObject(int iCommunityID, int iPageID,
     int iGWParams, String strHostPageURI, AActivitySpace asOwner, bool bResolveSecureBase)
    {
      IPTSession ptSession = (IPTSession) asOwner.GetUserSession();
      IPTState objAppDataObject = PortalObjectsFactory.CreateState();
      bool bAccessedThroughHTTPS = asOwner.GetIsSecuredSpace();
      /* Modes 2 and 3 */
      bool bIsMode2 = PTConfigHelper.GetSecurityMode() > 1;
      bool bIsMode0 = PTConfigHelper.GetSecurityMode() == 0;
      String strNormalizedBaseURL = PTConfigHelper.GetNormalizedBaseURL(strHostPageURI);
      String strAppBaseURL = PTConfigHelper.GetApplicationBaseURL(strNormalizedBaseURL,
       false);
      String strSecureAppBaseURL;
      String strLanguageStyle = PT_LAYOUT_STYLESHEET;
      if (PTConfigHelper.GetSecurityMode() == 3)
      {
        strSecureAppBaseURL = PTConfigHelper.GetApplicationBaseURL(strNormalizedBaseURL,
         bResolveSecureBase);
      } else
      {
        strSecureAppBaseURL = PTConfigHelper.GetApplicationBaseURL(strNormalizedBaseURL,
         true);
      }
      int iUserID = 0;
      try
      {
        int iPutUserIDInURL = PTConfigHelper.GetConfigVariableAsInt("Gateway",
         "PutUserIDInURL", "value", asOwner);
        if (iPutUserIDInURL == 1)
        {
          iUserID = ptSession.GetSessionInfo().GetCurrentUserID();
        }
      }
      catch (Exception e)
      {
        // if the setting is not in the config file, 
        // then assume that we should not put user id into URL 
        iUserID = 0;
      }
      /* GetApplicationBaseURL returns URL with "?" appended. Since it is too late to change it and
       * bugging Julie is not part of my plans for the nearest future i will just fix the URL here.
       * In case someone will ever decide to fix it, we will take an intelligent approach and search
       * for "?" first, instead of blindly removing the last character.
       * 		-- Moose
       */
       
      if (null != strAppBaseURL)
      {
        int iQuestionMarkLocation = XPStringUtility.IndexOf(strAppBaseURL,
         "?");
        if (-1 != iQuestionMarkLocation)
        {
          strAppBaseURL = XPStringUtility.Substring(strAppBaseURL, 0, iQuestionMarkLocation);
          }
      }
      // do the same thing for the secure URL
      if (null != strSecureAppBaseURL)
      {
        int iQuestionMarkLocation = XPStringUtility.IndexOf(strSecureAppBaseURL,
         "?");
        if (-1 != iQuestionMarkLocation)
        {
          strSecureAppBaseURL = XPStringUtility.Substring(strSecureAppBaseURL,
           0, iQuestionMarkLocation);
        }
      }
      /* if the community ID that is passed is negative, then just set it to 0 */
      iCommunityID = iCommunityID > 0 ? iCommunityID : 0;
      for (int i = 0;i < m_arrAppData.Length;i++)
      {
        objAppDataObject.Add((String) m_arrAppData[i][0], m_arrAppData[i][1],
         0, true);
      }
      // WilliamA - multiple-guest-user support.
      // -- if (iUserID != 2)
      if (!LoginHelper.INSTANCE.GetIsGuestUser(ptSession))
      {
        objAppDataObject.Add(PlumtreeExtensibility.PT_CAN_SET_PERSONAL_KEY,
         PlumtreeExtensibility.PT_CAN_SET, 0, true);
      } else
      {
        // This is stored on the persistent subsession so that it doesn't get cleared
        // during logoff.
        if (PlumtreeHelpers.IsEditingDefaultProfile(asOwner))
        {
          //Editing the guest session, should be able to set Can-Set values
          objAppDataObject.Add(PlumtreeExtensibility.PT_CAN_SET_PERSONAL_KEY,
           PlumtreeExtensibility.PT_CAN_SET, 0, true);
        } else
        {
          //regular guest viewing the page
          objAppDataObject.Add(PlumtreeExtensibility.PT_CAN_SET_PERSONAL_KEY,
           PlumtreeExtensibility.PT_CAN_NOT_SET, 0, true);
        }
      }
      objAppDataObject.Add((String) PlumtreeExtensibility.PT_COMMUNITY_ID_KEY,
       iCommunityID, 0, true);
      objAppDataObject.Add((String) PlumtreeExtensibility.PT_PAGE_ID_KEY,
       iPageID, 0, true);
      objAppDataObject.Add((String) PlumtreeExtensibility.PT_SESSION_KEY,
       ptSession, 0, true);
      // this is ok if we're not in mode 3
      if (PTConfigHelper.GetSecurityMode() != 3)
      {
        objAppDataObject.Add((String) PlumtreeExtensibility.PT_HOST_PAGE_URI_KEY,
         HttpUtils.GetSafeURL(HttpUtils.GetNormalizedURL(strHostPageURI, bIsMode2 || bAccessedThroughHTTPS)),
         0, true);
      } else
      // in mode 3 it isn't necessary to normalize this url
      {
        objAppDataObject.Add((String) PlumtreeExtensibility.PT_HOST_PAGE_URI_KEY,
         HttpUtils.GetSafeURL(strHostPageURI), 0, true);
      }
      objAppDataObject.Add((String) PlumtreeExtensibility.PT_SSOINFO_ON_STATE,
       asOwner.GetSubSession().GetAttribute(PlumtreeExtensibility.PT_SSOINFO_ON_STATE),
       0, true);
      objAppDataObject.Add(PlumtreeExtensibility.PT_TIMEZONE_KEY, PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.TIMEZONE,
       asOwner), 0, true);
      objAppDataObject.Add(PlumtreeExtensibility.PT_LOCALE_ID_KEY, PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LOCALE,
       asOwner), 0, true);
      String baseAppUrl = (bIsMode2 || bAccessedThroughHTTPS) ? strSecureAppBaseURL : strAppBaseURL;
      String frGWkey = GatewayHelpers.GetFriendlyGatewayURLKeyword();
      if (frGWkey != null)
      {
        // use friendly gateway urls, also needed by the server code to generate friendly gateway urls
        objAppDataObject.Add(PlumtreeExtensibility.PT_GATEWAYFRIENDLY_KEYWORD_KEY,
         frGWkey, 0, true);
      }
      String returnUri = GatewayHelpers.ConstructGatewayURL(baseAppUrl, 0,
       0, 0, (iGWParams | GatewayHelpers.RETURNFLAG), PT_CLASSIDS.PT_GADGET_ID,
       0, "a?b=c&");
      objAppDataObject.Add(PlumtreeExtensibility.PT_RETURN_URI_KEY, returnUri,
       0, true);
      // + GatewayHelpers.GATEWAY_URL_PREFIX + (iGWParams | GatewayHelpers.RETURNFLAG) + "_0_0_0_43/a?b=c&", 
      // setting partial gateway uris, the server will use these to create complete uri
      objAppDataObject.Add(PlumtreeExtensibility.PT_GATEWAY_URI_KEY, GatewayHelpers.ConstructPartialGatewayURL(baseAppUrl,
       iGWParams, iUserID), 0, true);
      objAppDataObject.Add(PlumtreeExtensibility.PT_SECURE_GATEWAY_URI_KEY,
       GatewayHelpers.ConstructPartialGatewayURL((!bIsMode0 || bAccessedThroughHTTPS) ? strSecureAppBaseURL : strAppBaseURL,
       iGWParams, iUserID), 0, true);
      objAppDataObject.Add(PlumtreeExtensibility.PT_REFRESH_URL_KEY, GatewayHelpers.ConstructPartialGatewayURL(baseAppUrl,
       GatewayHelpers.PORTLETCONTENT, iUserID), 0, true);
      // setting partial friendly gateway uris, the server will use these to create complete uri 
      if (frGWkey != null)
      {
        objAppDataObject.Add(PlumtreeExtensibility.PT_FRIENDLYGATEWAY_URI_KEY,
         GatewayHelpers.ConstructPartialFriendlyGatewayURLKeyword(baseAppUrl,
         iGWParams, iUserID), 0, true);
        objAppDataObject.Add(PlumtreeExtensibility.PT_FRIENDLYSECURE_GATEWAY_URI_KEY,
         GatewayHelpers.ConstructPartialFriendlyGatewayURLKeyword((!bIsMode0 || bAccessedThroughHTTPS) ? strSecureAppBaseURL : strAppBaseURL,
         iGWParams, iUserID), 0, true);
        objAppDataObject.Add(PlumtreeExtensibility.PT_FRIENDLYREFRESH_URL_KEY,
         GatewayHelpers.ConstructPartialFriendlyGatewayURLKeyword(baseAppUrl,
         GatewayHelpers.PORTLETCONTENT, iUserID), 0, true);
      }
      objAppDataObject.Add(PlumtreeExtensibility.PT_PORTLET_TIMEOUT_KEY, PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.PORTLET_TIMEOUT,
       asOwner), 0, true);
      // This handles the conversion from relative to absolute for us.
      String strImgSvrRootURL = PTConfigService.getPTInstance().GetAbsoluteImageServerRootURL(asOwner,
       strHostPageURI);
      objAppDataObject.Add(PlumtreeExtensibility.PT_IMAGESERVER_URI_KEY, strImgSvrRootURL,
       0, true);
      //					+ GatewayHelpers.GATEWAY_URL_PREFIX
      //					+ GatewayHelpers.PORTLETCONTENT + "_" + iUserID + "_", 
      // if we are in low banwidth mode, we should put that on the app data.
      // if we are NOT in low bandwidth mode, no need to put anything on app data
      // since it already defaults to NOT-low bandwidth mode.
      if (asOwner.GetAccessStyle() == AccessStyles.SECTION_508)
      {
        objAppDataObject.Add(PlumtreeExtensibility.PT_508_MODE_KEY, 1, 0,
         true);
      }
      IXPRequest requestCurrent = asOwner.GetCurrentHTTPRequest();
      if (null != requestCurrent)
      {
        objAppDataObject.Add(PlumtreeExtensibility.PT_HEADERS_KEY, HttpUtils.GetRawHeaders(requestCurrent),
         0, true);
      }
      // Bug 66066 - Add legacy stylesheet URL to AppDataObject for use with legacy apps.
      // For example, non-hosted gatewayed pages will need the old stylesheets. CSP requests
      // will always use legacy stylesheets so it will be used for the PT-Stylesheet-URI in the
      // CSP header.
      String strLegacyStyleSheetURL = GetStyleSheetURL(iCommunityID, ptSession,
       asOwner);
      // if the style sheet URL is relative, calculate based on user's base url pref
      if (!XPStringUtility.StartsWith(strLegacyStyleSheetURL, "http://") && !XPStringUtility.StartsWith(strLegacyStyleSheetURL,
       "https://"))
      {
        // it's relative, so look up the user's base url pref
        String domainBaseURL = XPConvert.ToString(PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.BASE_URL,
         asOwner.GetSubSession(), asOwner.GetApplication(), ptSession));
        if (null == domainBaseURL || domainBaseURL.Length < 1)
        {
          int indProtocol = XPStringUtility.IndexOf(strAppBaseURL, "://") + 3;
          domainBaseURL = XPStringUtility.Substring(strAppBaseURL, 0, XPStringUtility.IndexOf(strAppBaseURL,
           "/", indProtocol));
        }
        XPStringBuilder sbStyleSheetURL = new XPStringBuilder();
        sbStyleSheetURL.Append(domainBaseURL);
        // make sure there is always a single slash between the strBaseURL
        // and rest of style sheet URL.
        if (!XPStringUtility.StartsWith(strLegacyStyleSheetURL, "/"))
        {
          sbStyleSheetURL.Append("/");
        }
        sbStyleSheetURL.Append(strLegacyStyleSheetURL);
        strLegacyStyleSheetURL = sbStyleSheetURL.ToString();
      }
      //	 Checking if LayoutMode is enabled and replacing with Layout Specific Stylesheet if the mode is enabled.
      String strStyleSheetURL = null;
      if (GetLayoutsEnabled(asOwner))
      {
        //String spaceName = asOwner.GetModel("Gateway").GetName();
        String settingLanguage = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE,
         asOwner);
        // Checking if user specified different style sheet for this language when the layout mode is enabled
        // in CustomStyles.xml and replacing PT_LAYOUT_STYLESHEET with it.
        if (GetCustomStyleSheet(settingLanguage) != null)
        {
          // strLanguageStyle by default refers to mainstyle.css
          strLanguageStyle = GetCustomStyleSheet(settingLanguage);
        }
        strStyleSheetURL = PTConfigService.getPTInstance().GetAbsoluteImageServerBaseURL(asOwner,
         strHostPageURI) + strLanguageStyle;
      } else
      {
        // LayoutMode is not enabled, use the legacy stylesheets.
        strStyleSheetURL = strLegacyStyleSheetURL;
      }
      log.Debug("Passing absolute style sheet URL in AppDataObject: " + strStyleSheetURL);
      objAppDataObject.Add(PlumtreeExtensibility.PT_STYLESHEET_URI_KEY, strStyleSheetURL,
       0, true);
      log.Debug("Passing absolute legacy style sheet URL in AppDataObject: " + strLegacyStyleSheetURL);
      objAppDataObject.Add(PlumtreeExtensibility.PT_LEGACY_STYLESHEET_URI_KEY,
       strLegacyStyleSheetURL, 0, true);
      //set the query string on UserInfo
      ptSession.GetUserInfo().SetHostPageQueryString(asOwner.GetCurrentHTTPRequest().GetQueryString());
      log.Debug("Finished setting up AppDataObject. Here are the results.");
      log.Debug("PT_RETURN_URI_KEY: [" + objAppDataObject.GetAsString(PlumtreeExtensibility.PT_RETURN_URI_KEY) + "]");
      log.Debug("PT_SECURE_GATEWAY_URI_KEY: [" + objAppDataObject.GetAsString(PlumtreeExtensibility.PT_SECURE_GATEWAY_URI_KEY) + "]");
      log.Debug("PT_GATEWAY_URI_KEY: [" + objAppDataObject.GetAsString(PlumtreeExtensibility.PT_GATEWAY_URI_KEY) + "]");
      log.Debug("PT_HOST_PAGE_URI_KEY: [" + objAppDataObject.GetAsString(PlumtreeExtensibility.PT_HOST_PAGE_URI_KEY) + "]");
      log.Debug("PT_HEADERS_KEY: [" + objAppDataObject.GetAsString(PlumtreeExtensibility.PT_HEADERS_KEY) + "]");
      return objAppDataObject;
    }
    
    /// <summary>
    /// Calculate the style sheet for this gatewayed page.
    /// If this page is not coming from a community, then we use the subportal's
    /// style sheet.
    /// If it is coming from a community, then we first check to see if there is
    /// a community style sheet; if there is not, then we default to the
    /// subportal.
    /// </summary>
    public static PTStyleSheetLink GetStyleSheet(int iCommunityID, IPTSession ptSession,
     AActivitySpace asOwner)
    {
      String settingLanguage = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE,
       asOwner);
      String strStyleSheet = "";
      if (iCommunityID > 0)
      {
        IPTCommunityManager ptCommunityManager = (IPTCommunityManager) ptSession.GetCommunities();
        IPTCommunityInfo ptCommunityInfo = (IPTCommunityInfo) ptCommunityManager.CachedOpenCommunityInfo(iCommunityID,
         false);
        if (ptCommunityInfo != null)
        {
          int nHeaderID = ptCommunityInfo.GetHeaderID();
          if (nHeaderID > 0)
          {
            strStyleSheet = ptCommunityInfo.GetStyleSheet();
          }
        }
      }
      PTStyleSheetLink ptStyleSheetLink;
      if (!("").Equals(strStyleSheet) && (strStyleSheet != null))
      {
        if (asOwner.GetIsAccessStyleNonStandard())
        {
          strStyleSheet = PlumtreeHelpers.STYLE_PREFIX_508 + strStyleSheet;
          }
        ptStyleSheetLink = new PTStyleSheetLink(strStyleSheet, settingLanguage);
        } else
      {
        // TODOCR: This should just return the whole collection of style sheet links
        // use the default subportal stylesheet
        HTMLElementCollection pCollection = LoginHelper.INSTANCE.GetStyleSheetLinks(ptSession,
         asOwner);
        XPArrayList styleList = pCollection.GetInnerHTMLElementsAsArrayList(false);
        ptStyleSheetLink = new PTStyleSheetLink((HTMLLink) styleList.GetElement(0));
        }
      return ptStyleSheetLink;
    }
    
    /// <summary>
    /// Same as above, but returns just the URL -jas
    /// </summary>
    private static String GetStyleSheetURL(int iCommunityID, IPTSession ptSession,
     AActivitySpace asOwner)
    {
      PTStyleSheetLink ptStyleSheetLink = GetStyleSheet(iCommunityID, ptSession,
       asOwner);
      return ptStyleSheetLink.GetStyleSheetAsString(ConfigHelper.GetStyleSheetsBaseURL(asOwner));
      }
    
    /// <param name="_space"></param>
    /// <returns>If the Layout Mode is enabled/disabled</returns>
    private static bool GetLayoutsEnabled(AActivitySpace _space)
    {
      try
      {
        IPTSubPortalInfo subPortal = (((IPTSession) _space.GetUserSession()).GetSessionInfo().GetCurrentUserCachedSubPortalInfo());
        bool bLayoutsEnabled = subPortal.GetLayoutEnabled();
        return bLayoutsEnabled;
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to get the SubPortalInfo object to check for layout mode.");
        throw XPException.GetInstance(e);
      }
    }
    
    /// <param name="strLangKey">The Language of the user</param>
    /// <returns>The URI Suffix to the language specific custom css or null if nothing is specified for this language. </returns>
    private static String GetCustomStyleSheet(String strLangKey)
    {
      if (customSheetList == null)
      {
        customSheetList = GetCustomSheetList();
      }
      if (customSheetList.GetElement(strLangKey) != null)
      {
        return (String) customSheetList.GetElement(strLangKey);
      } else
      {
        return null;
      }
    }
    
    /// <summary>
    /// Returns the list of Language, Stylesheet Key value pairs.
    /// </summary>
    /// <returns>htKeyToValLookup Language, Stylesheet Key value pairs.</returns>
    private static XPHashtable GetCustomSheetList()
    {
      String strSettingsPath = AppWarmUp.GetSettingValueAsString(AppWarmUp.KEY_PATH_PORTAL_SETTINGS);
      XPHashtable htKeyToValLookup = new XPHashtable();
      try
      {
        String strStylesFile = GeneralHelpers.FormatPath(strSettingsPath + PT_LANGUAGE_FILE,
         AppWarmUp.PATH_SEPARATOR, false);
        XPDocument document = XPBuilder.LoadDocument(strStylesFile);
        //	navigate to get the pattern for detecting Style Settings.
        XPNodeList nlPattern = document.GetElementsByTagName(PT_STYLE_PATTERN);
        XPNode node = nlPattern.Item(0);
        XPNodeList nlKeys = document.GetElementsByTagName(PT_LANGUAGE_KEY);
        XPNodeList nlVals = document.GetElementsByTagName(PT_STYLE_KEY);
        for (int i = 0;i < nlKeys.GetLength();i++)
        {
          node = nlKeys.Item(i);
          String key = node.GetFirstChild().GetNodeValue();
          node = nlVals.Item(i);
          String value = node.GetFirstChild().GetNodeValue();
          htKeyToValLookup.PutElement(key, value);
          log.Debug("cssMapping:Language Styles mapping found: " + key + " -> " + value);
          }
      }
      catch (XPFileNotFoundException e)
      {
        log.Info("Unable to create the CustomStylesheet List: " + e.GetMessage());
        }
      catch (Exception e)
      {
        log.Error(e, "Failed to create the custom stylesheet list.");
      }
      return htKeyToValLookup;
    }
  }
}