/*
 * Created on Apr 30, 2003
 */
 
using System;
//import com.plumtree.openkernel.counters.*;
using com.plumtree.openlog;
using com.plumtree.server.impl.core;
using com.plumtree.server.portalpagesperf;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.portaluiinfrastructure.statichelpers
{
  /// <summary>
  /// Updates the performance counters available through PortalPagesPerf.  
  /// Also handles the case when the DLL is not present and disables
  /// future requests.  See com.plumtree.server.portalpagesperf.PORTALPAGESPERF_COUNTERS 
  /// for a list of available counters.
  /// </summary>
  /// <author>DavidP</author>
  public class PTPerfCounters : IPerfCounterHelper
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.statichelpers.PTPerfCounters");
    private static bool s_bInitialized = false;
    private static bool s_bHosed = false;
    private static int OPEN_SESSIONS = 0;
    // Singleton perf counter object; avoids multiple loaded DLL's	
    private static IPortalPagesReporter perfInstance = null;
    
    /// <summary>
    /// Initialize the performance counter object. If there is an exception, shuts down all future 
    /// logging for this instance.
    /// </summary>
    public virtual void Init()
    {
    }
    /*
     // TODOCR: DonH - is this actually called anywhere?
     if  (!s_bInitialized && !s_bHosed)
     {
     try
     {
     IOKCounterSystem ocSys = InternalSession.GetCounterService();
     PortalPagesReporter reporter = new PortalPagesReporter();
     perfInstance = reporter;
     IOKCounterCategoryInstance counterReporter = reporter;
     if (null != ocSys) {
     ocSys.RegisterInstance(counterReporter);
     } else {
     log.Warn("PTPerfCounters: did not find an opencounters system to register the portalpages reporter with. Counters were not registered.");
     }
     
     }
     catch( Exception e )
     {
     s_bHosed = true;
     
     log.Error(e, "PTPerfCounters: error creating perf counter object. Counters are now disabled.");
     
     }
     s_bInitialized = true;
     
     log.Info("PTPerfCounters: PortalPagesPerf performance counters are initialized.");
     
     }
     */
     
    // TODOCR: DonH - Should probably log an error if someone tries to init twice.
    
    /// <summary>
    /// Increment the performance counter.  If there is an exception, shuts down all future 
    /// logging for this instance.  The passed integer must be of the enum
    /// PORTALPAGESPERF_COUNTERS.
    /// </summary>
    /// <param name="counterVal"></param>
    public virtual void IncrementCounter(int counterVal)
    {
      if (GetSessionCounterID() == counterVal)
      {
        OPEN_SESSIONS++;
      }
      if (s_bInitialized && !s_bHosed)
      {
        try
        {
          perfInstance.IncrementCounter(counterVal);
        }
        catch (Exception e)
        {
          s_bHosed = true;
          log.Error(e, "Plumtree Helpers - error incrementing perf counter. Counters are now disabled.");
          }
      }
    }
    
    /// <summary>
    /// Decrement the performance counter.  If there is an exception, shuts down all future 
    /// logging for this instance.  The passed integer must be of the enum
    /// PORTALPAGESPERF_COUNTERS.
    /// </summary>
    /// <param name="counterVal"></param>
    public virtual void DecrementCounter(int counterVal)
    {
      if (GetSessionCounterID() == counterVal)
      {
        OPEN_SESSIONS--;
      }
      if (s_bInitialized && !s_bHosed)
      {
        try
        {
          perfInstance.DecrementCounter(counterVal);
        }
        catch (Exception e)
        {
          s_bHosed = true;
          log.Error(e, "Plumtree Helpers - error decrementing perf counter. Counters are now disabled.");
          }
      }
    }
    
    /// <summary>
    /// Checks that the perf counter interface is initialized
    /// </summary>
    public virtual bool IsInitialized()
    {
      return s_bInitialized;
    }
    
    /// <summary>
    /// Checks that the perf counter interface is not disabled
    /// This occurs if an exception is thrown
    /// </summary>
    public virtual bool IsDisabled()
    {
      return s_bHosed;
    }
    
    /// <summary>
    /// Gets the integer for the counter for session total
    /// </summary>
    public virtual int GetSessionCounterID()
    {
      return PORTALPAGESPERF_COUNTERS.PORTALPAGESPERF_TOTAL_OPENSESSIONS;
      }
    public virtual int GetOpenSessions()
    {
      return OPEN_SESSIONS;
    }
  }
}