using System;
// Plumtree Server
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.common.constants;
using com.plumtree.portaluiinfrastructure.editor;
using com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity;
using com.plumtree.portaluiinfrastructure.expandablelist.selectptobject;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.server;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.portaluiinfrastructure.subscribematrix
{
  /// <summary>
  /// This is the implementation of the Expandable List
  /// framework for the the Select PTObject list.
  /// The Select PTObject is a list of objects.
  /// New objects can be added to the list by selecting
  /// them from the associated Object tree.
  /// The list is by default sorted by name ascending.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class MultiObjectsSubscribeModel : ObjectSecurityModel,IMultiObjectsSubscribeModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
     "com.plumtree.portaluiinfrastructure.subscribematrix.MultiObjectsSubscribeModel");
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "MultiObjectsSubscribe";
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String QS_MULTI_OBJ_SUBSCRIBE_IDS = "objectAndClassIDs";
    
    /// <summary>
    /// Subscribe column 
    /// </summary>
    public const int EXPLIST_COL_SUBSCRIBE = 6;
    
    /// <summary>
    /// Object img 
    /// </summary>
    public const int EXPLIST_COLUMN_IMG = 5;
    
    /// <summary>
    /// Array of objects we are editing the security for. 
    /// </summary>
    protected XPArrayList m_arrObjects;
    
    /// <summary>
    /// Array of column info. 
    /// </summary>
    protected int[] m_arrColumns;
    
    /// <summary>
    /// Flag to know if we are Editing the Security on Admin Folders.
    /// If we are, we also need to display the ABO pop-up when the user clicks on Finish.
    /// </summary>
    private bool m_bEditAdminFolderSecurity = false;
    
    /// <summary>
    /// Array of Object IDs.
    /// </summary>
    private int[] m_arrObjectIDs;
    
    /// <summary>
    /// Bulk Subscribe Job ID
    /// </summary>
    private int m_nJobID;
    
    /// <summary>
    /// Bulk Subscribe Job Name
    /// </summary>
    private String m_sJobName;
    
    /// <summary>
    /// Bulk Subscribe Job Folder Name
    /// </summary>
    private String m_sFolderJobCreatedInName;
    private XPArrayList m_xpalSubscriptions;
    private bool m_bHaveSubscriptionsBeenMade;
    
    /// <summary>
    /// Helper method that adds a row to the list.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <param name="_sName"></param>
    protected override void AddACLToList(int _nClassID, int _nObjectID, String _sName)
    {
      Object[] oNewRow = new Object[m_arrColumns.Length];
      oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX] = m_nNextIndex;
      m_nNextIndex++;
      oNewRow[EXPLIST_COL_CLASSID] = _nClassID;
      oNewRow[EXPLIST_COL_OBJECTID] = _nObjectID;
      oNewRow[EXPLIST_COL_STRING] = _sName;
      // JF- Defaut selected access level: read
      oNewRow[EXPLIST_COL_ACCESSLEVEL_ID] = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT;
      oNewRow[EXPLIST_COL_ACCESSLEVEL_STRING] = GetAccessLevelString(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);
      // JF- By default the user has read/write access to the row
      oNewRow[EXPLIST_COL_ROWACCESS] = PT_ACCESS_LIST_ENTRY_READWRITE;
      // JF- By default the object is not mandatory
      oNewRow[EXPLIST_COL_MANDATORY] = MANDATORY_FALSE;
      // JF- By default there is no tree ID
      oNewRow[EXPLIST_COL_GROUPTREEASID] = "";
      int k = EXPLIST_COL_MAX;
      while (k < m_arrColumns.Length)
      {
        oNewRow[k] = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT;
        oNewRow[k + 1] = GetAccessLevelString(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);
        oNewRow[k + 2] = "0";
        k = k + 3;
      }
      base.GetItems().AddRow(oNewRow);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new MultiObjectsSubscribeModel();
    }
    
    /// <summary>
    /// Saves the security settings into the IPTObject.
    /// </summary>
    public override void Finish()
    {
      IPTBulkSubscriber subscriber = m_ptSession.GetBulkSubscriber();
      for (int i = 0;i < m_arrObjects.GetSize();i++)
      {
        StoreSubscriptionsToObject((IPTObject) m_arrObjects.GetElement(i),
         subscriber);
      }
      if (m_bHaveSubscriptionsBeenMade)
      {
        m_nJobID = subscriber.ScheduleImmediateJob();
        try
        {
          // get the job's localized name
          IPTJob myTempJob = (IPTJob) m_ptSession.GetObjectManagers(PT_CLASSIDS.PT_JOB_ID).Open(m_nJobID,
           false);
          m_sJobName = myTempJob.GetLocalizedName();
          // get the folder that the job was created in's localized name
          IPTAdminFolder myTempAdminFolder = (IPTAdminFolder) m_ptSession.GetAdminCatalog().OpenAdminFolder(PT_INTRINSICS.PT_ADMIN_FOLDER_INTRINSICOPS,
           false);
          m_sFolderJobCreatedInName = myTempAdminFolder.GetLocalizedName();
          }
        catch (Exception ex)
        {
          log.Error(m_asOwner.GetString(1701, ResourceConstants.PORTALADMINMSGS) + PT_CLASSIDS.PT_ADMIN_FOLDER_ID);
          }
      }
    }
    public virtual int GetJobID()
    {
      return m_nJobID;
    }
    public virtual String GetJobName()
    {
      return m_sJobName;
    }
    public virtual String GetJobFolderName()
    {
      return m_sFolderJobCreatedInName;
    }
    
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <returns>int</returns>
    public virtual int GetACLIndex(int _nClassID, int _nObjectID)
    {
      int nIndex = -1;
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        if ((_nClassID == XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID))) && (_nObjectID == XPConvert.ToInteger(m_saData.GetItem(i,
         EXPLIST_COL_OBJECTID))))
        {
          nIndex = i;
          break;
        }
      }
      return nIndex;
    }
    
    /// <summary>
    /// Returns a flag to know whether any subscriptions have been made.
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool HaveSubscriptionsBeenMade()
    {
      return m_bHaveSubscriptionsBeenMade;
    }
    
    /// <summary>
    /// Tells whether the row identified by the class ID, the object ID
    /// and the Access Level ID in argument is in the Object ACL or no.
    /// </summary>
    /// <param name="_nClassID"></param>
    /// <param name="_nObjectID"></param>
    /// <param name="_nAccessLevel"></param>
    /// <param name="_ptACL"></param>
    /// <returns>boolean</returns>
    protected virtual bool IsMatrixRowInObjectACL(int _nClassID, int _nObjectID,
     int _nAccessLevel, IPTAccessList _ptACL)
    {
      IPTAccessListEntry ptACLEntry;
      for (int i = 0;i < _ptACL.GetCount();i++)
      {
        ptACLEntry = _ptACL.GetItem(i);
        if ((_nClassID == ptACLEntry.GetClassID()) && (_nObjectID == ptACLEntry.GetObjectID()) && (_nAccessLevel == ptACLEntry.GetAccessLevel()))
        {
          return true;
        }
      }
      return false;
    }
    
    /// <returns>the current IPTAccessList of the folder whose security is being edited</returns>
    public virtual IPTAccessList GetCurrentFolderACL()
    {
      // can only edit one admin folder at a time so the first
      // element of the object array will be the folder we want 
      IPTAdminFolder ptFolder = (IPTAdminFolder) ((MultiObjectsSubscribeModel) m_asOwner.GetModel(SubscribeMatrixEditorAS.STR_MVC_STORAGE_NAME_MULTI_SUBSCRIBE)).GetObjectArray().GetElement(0);
      IPTAdminFolder ptNewFolder = m_ptSession.GetAdminCatalog().OpenAdminFolder(((IPTServerContext) ptFolder.GetInterfaces("IPTServerContext")).GetObjectID(),
       false);
      // lock the folder for edit								
      IPTServerContext ptSC = (IPTServerContext) ptNewFolder.GetInterfaces("IPTServerContext");
      return ptSC.GetACL();
    }
    
    /// <summary>
    /// Returns the array of Object IDs.
    /// </summary>
    /// <returns>int[]</returns>
    public virtual int[] GetObjectIDs()
    {
      return m_arrObjectIDs;
    }
    
    /// <summary>
    /// Returns the array of column info.
    /// </summary>
    /// <returns>int[]</returns>
    public virtual int[] GetColumns()
    {
      return m_arrColumns;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Returns the array of Objects.
    /// </summary>
    /// <returns>XPArrayList</returns>
    public virtual XPArrayList GetObjectArray()
    {
      return m_arrObjects;
    }
    
    /// <summary>
    /// Initializes the expandable list.
    /// </summary>
    /// <param name="_ptObject">Object being created or edited</param>
    /// <param name="_ptSession">User session</param>
    /// <returns>boolean Flag to know if the initialization was successful or no.</returns>
    public virtual bool InitializeMultiObjsSubscribePage(XPHashtable _htFormData,
     IPTSession _ptSession)
    {
      m_ptSession = _ptSession;
      m_xpalSubscriptions = new XPArrayList();
      m_bHaveSubscriptionsBeenMade = false;
      m_nJobID = 0;
      m_sJobName = "";
      m_sFolderJobCreatedInName = "";
      DisableAddButton();
      DisableCheckBoxes();
      String[] data;
      if (!_htFormData.ContainsKey(QS_MULTI_OBJ_SUBSCRIBE_IDS))
      {
        log.Error("The objectAndClassIDs parameter was not in the Query String.");
        throw new XPException("MultiObjectsSubscribeModel::InitializeMultiObjsSecurityPage Unable to start the Subscribe matrix.");
        }
      data = (String[]) _htFormData.GetElement(QS_MULTI_OBJ_SUBSCRIBE_IDS);
      if ((null == data) || (0 == data.Length) || (null == data[0]))
      {
        log.Error("The objectAndClassIDs parameter was null or empty.");
        throw new XPException("MultiObjectsSubscribeModel::InitializeMultiObjsSecurityPage Unable to start the Subscribe matrix.");
        }
      m_arrObjects = new XPArrayList();
      String sIDs = data[0];
      int nClassID;
      int nObjectID;
      bool bNonFailureErrorSet = false;
      while (0 != sIDs.Length)
      {
        // JF- Get class ID
        nClassID = XPConvert.ToInteger(XPStringUtility.Substring(sIDs, 0,
         XPStringUtility.IndexOf(sIDs, ",")));
        sIDs = XPStringUtility.Substring(sIDs, XPStringUtility.IndexOf(sIDs,
         ",") + 1);
        if (-1 != XPStringUtility.IndexOf(sIDs, ","))
        {
          // JF- Get object ID
          nObjectID = XPConvert.ToInteger(XPStringUtility.Substring(sIDs,
           0, XPStringUtility.IndexOf(sIDs, ",")));
          sIDs = XPStringUtility.Substring(sIDs, XPStringUtility.IndexOf(sIDs,
           ",") + 1);
        } else
        {
          // JF- Get object ID
          nObjectID = XPConvert.ToInteger(sIDs);
          sIDs = "";
        }
        // JF- Open the Object and add it to the array
        if (PT_CLASSIDS.PT_ADMIN_FOLDER_ID == nClassID)
        {
          IPTAdminFolder ptAdminFolder = m_ptSession.GetAdminCatalog().OpenAdminFolder(nObjectID,
           false);
          if (PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER == ptAdminFolder.GetFolderType())
          {
            IPTQueryResult qrComm = m_ptSession.GetCommunities().Query(PT_PROPIDS.PT_PROPID_OBJECTID,
             nObjectID, PT_PROPIDS.PT_PROPID_OBJECTID, 0, 1, null);
            m_arrObjects.Add(m_ptSession.GetCommunities().Open(qrComm.ItemAsInt(0,
             PT_PROPIDS.PT_PROPID_OBJECTID), false));
            m_bEditAdminFolderSecurity = true;
            if (!_ptSession.GetCommunities().IsObjectAccessible(qrComm.ItemAsInt(0,
             PT_PROPIDS.PT_PROPID_OBJECTID), _ptSession.GetSessionInfo().GetCurrentUserID(),
             PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT))
            {
              //Raise the error that the matrix could not be started
              UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
               null, m_asOwner.GetString(514, "ptmsgs_portalinfrastructure"),
               null);
              UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
               m_asOwner.GetApplication());
              return false;
            }
          } else
          {
            log.Warn("InitializeMultiObjsSubscribePage Ignoring Folder with ID: " + nObjectID + ". (it is not a Community Folder).");
            }
        } else
        if (PT_CLASSIDS.PT_COMMUNITY_ID == nClassID || PT_CLASSIDS.PT_GADGET_ID == nClassID)
        {
          try
          {
            IPTObject ptObject = (IPTObject) m_ptSession.GetObjectManagers(nClassID).Open(nObjectID,
             false);
            if (PT_CLASSIDS.PT_GADGET_ID == nClassID)
            {
              if (PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NARROW == ((IPTGadget) ptObject).GetAlignment() || PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE == ((IPTGadget) ptObject).GetAlignment())
              {
                m_arrObjects.Add(ptObject);
              } else
              {
                //Raise the error that the portlet was a footer, header, or content canvas portlet.
                UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
                 null, XPFormat.Format(m_asOwner.GetString(542, "ptmsgs_portalinfrastructure"),
                 new Object[]{ptObject.GetName()}), null);
                UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
                 m_asOwner.GetApplication());
                bNonFailureErrorSet = true;
              }
            } else
            {
              m_arrObjects.Add(ptObject);
            }
            if (!_ptSession.GetObjectManagers(nClassID).IsObjectAccessible(nObjectID,
             _ptSession.GetSessionInfo().GetCurrentUserID(), PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT))
            {
              //Raise the error that the matrix could not be started
              UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
               null, m_asOwner.GetString(515, "ptmsgs_portalinfrastructure"),
               null);
              UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(),
               m_asOwner.GetApplication());
              return false;
            }
          }
          catch (Exception e)
          {
            log.Warn("InitializeMultiObjsSecurityPage Unable to open object with class ID: " + nClassID + " and object ID: " + nObjectID + ". The following exception was thrown: " + e);
            }
        } else
        {
          log.Warn("InitializeMultiObjsSecurityPage Ignoring object with class ID: " + nClassID + " and object ID: " + nObjectID + ". (Can only subscribe to Communities and Portlets).");
          }
      }
      if (0 < m_arrObjects.GetSize())
      {
        m_arrObjectClassIDs = new int[]{PT_CLASSIDS.PT_USERGROUP_ID};
        // JF- Set the icon to display on each row.
        SetRowIconByClassID(PT_CLASSIDS.PT_USERGROUP_ID, PTImageType.ICON_GROUP);
        base.InitializeExpandableList(m_asOwner.GetString(518, "ptmsgs_portalinfrastructure"),
         CreateColumnInfos(), PTImageType.ICON_GROUP, "", 11);
        for (int i = 0;i < m_arrObjects.GetSize();i++)
        {
          IPTObject ptObject = (IPTObject) m_arrObjects.GetElement(i);
          InitListWithACLFromObject(ptObject);
        }
      } else
      if (!bNonFailureErrorSet)
      {
        String sMessage = m_asOwner.GetString(513, "ptmsgs_portalinfrastructure");
        //Raise the error that the matrix could not be started
        UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
         null, sMessage, null);
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        throw new XPException("MultiObjectsSubscribeModel::InitializeMultiObjsSubscribePage Array of Objects is empty.");
        }
      return true;
    }
    
    /// <summary>
    /// Creates the array of clolumn info.
    /// </summary>
    /// <returns>Object[][]</returns>
    protected virtual Object[][] CreateColumnInfos()
    {
      Object[][] arrExpListColInfo;
      // JF- Column mapping
      m_arrColumns = new int[EXPLIST_COL_MAX + (3 * m_arrObjects.GetSize())];
      m_arrColumns[0] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
      m_arrColumns[1] = EXPLIST_COL_CLASSID;
      m_arrColumns[2] = EXPLIST_COL_OBJECTID;
      m_arrColumns[3] = EXPLIST_COL_STRING;
      m_arrColumns[4] = EXPLIST_COL_ACCESSLEVEL_ID;
      m_arrColumns[5] = EXPLIST_COL_ACCESSLEVEL_STRING;
      m_arrColumns[6] = EXPLIST_COL_ROWACCESS;
      m_arrColumns[7] = EXPLIST_COL_MANDATORY;
      m_arrColumns[8] = EXPLIST_COL_GROUPTREEASID;
      // JF- Create the the 2D array to give column information to the super class.
      arrExpListColInfo = new Object[m_arrObjects.GetSize() + 2][];
      for (int j2c_index = 0;j2c_index < m_arrObjects.GetSize() + 2;j2c_index++)
      {
        arrExpListColInfo[j2c_index] = new Object[6];
      }
      // First column: object name
      arrExpListColInfo[0][EXPLIST_COLUMN_ID] = EXPLIST_COL_STRING;
      arrExpListColInfo[0][EXPLIST_COLUMN_TITLE] = m_asOwner.GetString(523,
       "ptmsgs_portalinfrastructure");
      arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH] = "40%";
      arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN] = "1";
      arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT] = true;
      arrExpListColInfo[0][EXPLIST_COLUMN_IMG] = null;
      int k = EXPLIST_COL_MAX;
      int nCount = m_arrObjects.GetSize();
      m_arrObjectIDs = new int[nCount];
      for (int i = 0;i < nCount;i++)
      {
        IPTObject ptObject = (IPTObject) m_arrObjects.GetElement(i);
        IPTServerContext ptsc = (IPTServerContext) ptObject.GetInterfaces("IPTServerContext");
        String sObjectKey = XPConvert.ToString(ptObject.GetClassID()) + XPConvert.ToString(ptsc.GetObjectID());
        m_arrObjectIDs[i] = ptsc.GetObjectID();
        // Rights column
        arrExpListColInfo[i + 1][EXPLIST_COLUMN_ID] = EXPLIST_COL_ACCESSLEVEL_STRING + sObjectKey;
        arrExpListColInfo[i + 1][EXPLIST_COLUMN_TITLE] = XPHttpUtility.HtmlEncode(GeneralHelpers.TruncateLongString(ptObject.GetLocalizedName(),
         ObjEditorModel.OBJECT_NAME_TRUNCATE, false));
        arrExpListColInfo[i + 1][EXPLIST_COLUMN_WIDTH] = "15%";
        arrExpListColInfo[i + 1][EXPLIST_COLUMN_COLSPAN] = "1";
        arrExpListColInfo[i + 1][EXPLIST_COLUMN_ENABLESORT] = false;
        PTImageType img = null;
        if (PT_CLASSIDS.PT_COMMUNITY_ID == ptObject.GetClassID())
        {
          img = PTImageType.ICON_COMMUNITY;
        } else
        if (PT_CLASSIDS.PT_GADGET_ID == ptObject.GetClassID())
        {
          img = PTImageType.ICON_PORTLET;
        }
        arrExpListColInfo[i + 1][EXPLIST_COLUMN_IMG] = img;
        m_arrColumns[k] = XPConvert.ToInteger(EXPLIST_COL_ACCESSLEVEL_ID + sObjectKey);
        m_arrColumns[k + 1] = XPConvert.ToInteger(EXPLIST_COL_ACCESSLEVEL_STRING + sObjectKey);
        m_arrColumns[k + 2] = XPConvert.ToInteger(EXPLIST_COL_SUBSCRIBE + sObjectKey);
        k = k + 3;
      }
      arrExpListColInfo[nCount + 1][EXPLIST_COLUMN_ID] = -1;
      arrExpListColInfo[nCount + 1][EXPLIST_COLUMN_TITLE] = m_asOwner.GetString(360,
       "ptmsgs_portalinfrastructure");
      arrExpListColInfo[nCount + 1][EXPLIST_COLUMN_WIDTH] = "15%";
      arrExpListColInfo[nCount + 1][EXPLIST_COLUMN_COLSPAN] = "1";
      arrExpListColInfo[nCount + 1][EXPLIST_COLUMN_ENABLESORT] = false;
      return arrExpListColInfo;
    }
    
    /// <summary>
    /// Helper method to get the sort.
    /// </summary>
    /// <returns>int[]</returns>
    protected override int[] GetArrayOfColumns()
    {
      return m_arrColumns;
    }
    
    /// <summary>
    /// Initializes the list with the ACL from the PTObject.
    /// </summary>
    /// <param name="_ptObject">Object being created or edited</param>
    protected virtual void InitListWithACLFromObject(IPTObject _ptObject)
    {
      IPTServerContext ptscObject = (IPTServerContext) _ptObject.GetInterfaces("IPTServerContext");
      IPTAccessListEntry ptACLEntry;
      Object[] oNewRow;
      m_arrLowAccessObjects = new XPArrayList();
      m_arrNoContentTree = new XPArrayList();
      // JF- Keep track of the items we are hiding in the tree
      XPArrayList arrTreeHiddenItems = new XPArrayList();
      SetWrapColumnHeaders(false);
      IPTAccessList ptACL = ptscObject.GetACL();
      String sObjectKey = XPConvert.ToString(_ptObject.GetClassID()) + XPConvert.ToString(ptscObject.GetObjectID());
      int nCurrentObjectColAccessLevelID = XPConvert.ToInteger(EXPLIST_COL_ACCESSLEVEL_ID + sObjectKey);
      int nCurrentObjectColAccessLevelString = XPConvert.ToInteger(EXPLIST_COL_ACCESSLEVEL_STRING + sObjectKey);
      int nCurrentObjectColRowAccess = XPConvert.ToInteger(EXPLIST_COL_SUBSCRIBE + sObjectKey);
      for (int i = 0;i < ptACL.GetCount();i++)
      {
        ptACLEntry = ptACL.GetItem(i);
        int nACLClassID = ptACLEntry.GetClassID();
        // only allow bulk subscription of groups
        if (PT_CLASSIDS.PT_USERGROUP_ID == nACLClassID)
        {
          int nACLObjectID = ptACLEntry.GetObjectID();
          if (m_ptSession.GetObjectManagers(PT_CLASSIDS.PT_USERGROUP_ID).IsObjectAccessible(nACLObjectID,
           m_ptSession.GetSessionInfo().GetCurrentUserID(), PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT))
          {
            int nACLIndex = GetACLIndex(nACLClassID, nACLObjectID);
            if (-1 == nACLIndex)
            {
              oNewRow = new Object[m_arrColumns.Length];
              oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX] = m_nNextIndex;
              m_nNextIndex++;
              oNewRow[EXPLIST_COL_CLASSID] = nACLClassID;
              oNewRow[EXPLIST_COL_OBJECTID] = nACLObjectID;
              oNewRow[EXPLIST_COL_STRING] = ptACLEntry.GetName();
              oNewRow[EXPLIST_COL_ROWACCESS] = ptACLEntry.GetReadOnly();
              oNewRow[EXPLIST_COL_GROUPTREEASID] = "";
              int k = EXPLIST_COL_MAX;
              while (k < m_arrColumns.Length)
              {
                if ((nCurrentObjectColAccessLevelID == m_arrColumns[k]) && (nCurrentObjectColAccessLevelString == m_arrColumns[k + 1]) && (nCurrentObjectColRowAccess == m_arrColumns[k + 2]))
                {
                  oNewRow[k] = ptACLEntry.GetAccessLevel();
                  oNewRow[k + 1] = GetAccessLevelString(ptACLEntry.GetAccessLevel());
                  oNewRow[k + 2] = "N";
                } else
                {
                  oNewRow[k] = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_NONE;
                  oNewRow[k + 1] = GetAccessLevelString(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_NONE);
                  oNewRow[k + 2] = "N";
                }
                k = k + 3;
              }
              base.GetItems().AddRow(oNewRow);
            } else
            {
              m_saData.SetItem(nACLIndex, nCurrentObjectColAccessLevelID,
               ptACLEntry.GetAccessLevel());
              m_saData.SetItem(nACLIndex, nCurrentObjectColAccessLevelString,
               GetAccessLevelString(ptACLEntry.GetAccessLevel()));
              m_saData.SetItem(nACLIndex, nCurrentObjectColRowAccess, "N");
              if (PT_ACCESS_LIST_ENTRY_READONLY == ptACLEntry.GetReadOnly())
              {
                m_saData.SetItem(nACLIndex, EXPLIST_COL_ROWACCESS, PT_ACCESS_LIST_ENTRY_READONLY);
                }
            }
          }
        }
      }
    }
    
    /// <summary>
    /// Saves the access level for the row _nRowIndex.
    /// </summary>
    /// <param name="_saData">List to update the data on row _nRowIndex</param>
    /// <param name="_nRowIndex">Index of the row to save.</param>
    /// <param name="_htFormData">Data posted by the form.</param>
    /// <returns>int Status for server side validation.</returns>
    protected override int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
     XPHashtable _htFormData)
    {
      // JF- We need to save the privilege drop-downs.
      if (!m_bIsReadOnly)
      {
        int k = EXPLIST_COL_MAX;
        while (k < m_arrColumns.Length)
        {
          // JF- We are building the select input name the same way we did 
          // in ObjectSecurityView when we displayed it.
          String sInputSelectName = GetUniqueString(MultiObjectsSubscribeView.HTMLCHECKBOX_SUBSCRIBE) + _nRowIndex + k;
          String[] data = (String[]) _htFormData.GetElement(sInputSelectName);
          if ((data != null) && (0 != data.Length) && (null != data[0]) && ("1").Equals(data[0]))
          {
            _saData.SetItem(_nRowIndex, m_arrColumns[k + 2], "Y");
          }
          k = k + 3;
        }
      }
      int nObjectID = XPConvert.ToInteger(_saData.GetItem(_nRowIndex, EXPLIST_COL_OBJECTID));
      String sGroupTreeASID = (String) _saData.GetItem(_nRowIndex, EXPLIST_COL_GROUPTREEASID);
      // JF- We save the Group tree ID only if we don't already have it.
      if (("").Equals(sGroupTreeASID) && _htFormData.ContainsKey(GetUniqueString(ObjectSecurityView.HTMLINPUT_GROUPTREEASID) + nObjectID))
      {
        String[] data = (String[]) _htFormData.GetElement(GetUniqueString(ObjectSecurityView.HTMLINPUT_GROUPTREEASID) + nObjectID);
        if ((null != data) && !(0 == data.Length))
        {
          _saData.SetItem(_nRowIndex, EXPLIST_COL_GROUPTREEASID, data[0]);
          }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Saves the security settings into the IPTObject.
    /// </summary>
    protected virtual void StoreSubscriptionsToObject(IPTObject _ptObject,
     IPTBulkSubscriber _ptSubscriber)
    {
      IPTServerContext ptscObject = (IPTServerContext) _ptObject.GetInterfaces("IPTServerContext");
      int nClassID;
      int nObjectID;
      String strSubscribe;
      XPArrayList xpalGroupIDs = new XPArrayList();
      String sObjectKey = XPConvert.ToString(_ptObject.GetClassID()) + XPConvert.ToString(ptscObject.GetObjectID());
      // JF- Then we go through all the users and groups from the list and we add 
      // them to the Object ACL (except the read only ones.)		
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        strSubscribe = (String) m_saData.GetItem(i, XPConvert.ToInteger(EXPLIST_COL_SUBSCRIBE + sObjectKey));
        nClassID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID));
        nObjectID = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_OBJECTID));
        if (("Y").Equals(strSubscribe) && PT_CLASSIDS.PT_USERGROUP_ID == nClassID)
        {
          xpalGroupIDs.Add(nObjectID);
        }
      }
      if (0 < xpalGroupIDs.GetSize())
      {
        m_bHaveSubscriptionsBeenMade = true;
        int[] anGroupIDs = new int[xpalGroupIDs.GetSize()];
        for (int i = 0;i < xpalGroupIDs.GetSize();i++)
        {
          anGroupIDs[i] = XPConvert.ToInteger(xpalGroupIDs.GetElement(i));
          }
        if (PT_CLASSIDS.PT_COMMUNITY_ID == _ptObject.GetClassID() || PT_CLASSIDS.PT_GADGET_ID == _ptObject.GetClassID())
        {
          _ptSubscriber.SetBulkSubscription(_ptObject.GetClassID(), ptscObject.GetObjectID(),
           anGroupIDs);
        }
      }
    }
  }
}