using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
namespace com.plumtree.portaluiinfrastructure.tags.data
{
  /// <summary>
  /// This class is wrapper around XPHashtable that allows it
  /// to be used as a data collection in Tag memory.
  /// </summary>
  public class HashtableDataObject : IDataObject
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     "com.plumtree.portaluiinfrastructure.tags.data.HashtableDataObject");
    
    /// <summary>
    /// The underlying hashtable object.
    /// </summary>
    protected readonly XPHashtable m_hashtable;
    
    /// <summary>
    /// Constructor for the Hashtable wrapper data object that
    /// takes a pre-constructed hashtable.
    /// </summary>
    /// <param name="_hashtable">A pre-constructed hashtable.</param>
    public HashtableDataObject(XPHashtable _hashtable)
    {
      m_hashtable = _hashtable;
    }
    
    /// <summary>
    /// Constructor for the Hashtable wrapper data object.
    /// </summary>
    public HashtableDataObject()
    {
      m_hashtable = new XPHashtable();
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.data.IDataObject#GetData(String)"></seealso>
    public virtual String GetData(String _strKey)
    {
      String strResult;
      try
      {
        strResult = (String) m_hashtable.GetElement(_strKey);
      }
      catch (Exception e)
      {
        log.Warn("Attempt to access data with key " + _strKey + " failed.  No such data exist.");
        strResult = null;
      }
      return strResult;
    }
    
    /// <summary>
    /// Sets a name value pair on the hashtable.
    /// </summary>
    /// <param name="_strKey">The name used to store the value.</param>
    /// <param name="_strValue">The value to be stored.</param>
    public virtual void SetData(String _strKey, String _strValue)
    {
      m_hashtable.PutElement(_strKey, _strValue);
    }
    
    /// <summary>
    /// Get the underlying hashtable object.
    /// </summary>
    /// <returns>XPHashtable The underlying hashtable object.</returns>
    public virtual XPHashtable GetHashTable()
    {
      return m_hashtable;
    }
  }
}