using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.friendlyurl;
using com.plumtree.server;
using junit.framework;
namespace com.plumtree.portaluiinfrastructure.test.friendlyurl
{
  public class FriendlyURLParserManagerTest : TestCase
  {
    protected IFriendlyURLParserManager testObject;
    protected String commKeyword = "community";
    protected String mypageKeyword = "mypages";
    protected String APIKeyword = "api";
    public FriendlyURLParserManagerTest(String name) : base(name)
    {
    }
    
    /// <summary>
    /// Setup if any is required.
    /// </summary>
    public override void setUp()
    {
      base.setUp();
      XPHashtable htKeyToClassIdLookup = new XPHashtable();
      htKeyToClassIdLookup.PutElement(commKeyword, XPConvert.ToString(PT_CLASSIDS.PT_COMMUNITY_ID));
      htKeyToClassIdLookup.PutElement(mypageKeyword, XPConvert.ToString(PT_CLASSIDS.PT_MYPAGESCONTAINER_ID));
      htKeyToClassIdLookup.PutElement(APIKeyword, XPConvert.ToString(FriendlyURLConstants.REST_API_ID));
      testObject = FriendlyURLParserManager.getInstance();
      testObject.SetKeyToClassIdLookupTable(htKeyToClassIdLookup);
    }
    
    /// <seealso cref="TestCase#tearDown()"></seealso>
    public override void tearDown()
    {
      base.tearDown();
      testObject = null;
    }
    public virtual void testGetParserCommunity()
    {
      XPStringTokenizer stURLTokens = new XPStringTokenizer(commKeyword + "/community_name/",
       "/", false);
      IFriendlyURLParser fURLParser = testObject.GetParser(stURLTokens);
      Assert.assertEquals("Wrong FriendlyURLParser was returned.", typeof(CommunityFriendlyURLParser),
       fURLParser.GetType());
    }
    public virtual void testGetParserMyPages()
    {
      XPStringTokenizer stURLTokens = new XPStringTokenizer(mypageKeyword + "/community_name/",
       "/", false);
      IFriendlyURLParser fURLParser = testObject.GetParser(stURLTokens);
      Assert.assertEquals("Wrong FriendlyURLParser was returned.", typeof(MyPageFriendlyURLParser),
       fURLParser.GetType());
    }
    public virtual void testGetParserAPI()
    {
      XPStringTokenizer stURLTokens = new XPStringTokenizer(APIKeyword + "/v1/kd/123",
       "/", false);
      IFriendlyURLParser fURLParser = testObject.GetParser(stURLTokens);
      Assert.assertEquals("Wrong FriendlyURLParser was returned.", typeof(APIFriendlyURLParser),
       fURLParser.GetType());
    }
  }
}