package com.plumtree.portaluiinfrastructure.application.varpacks;

import com.plumtree.openkernel.config.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.application.AppWarmUp;
import com.plumtree.uiinfrastructure.web.*;

public class PTAnalyticsVarPack extends BaseVarPack {

	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.application.varpacks.PTAnalyticsVarPack");

	/** The string ID of this variable package. */
	public static final String VARPACK_ID = "PTAnalyticsVarPack";
	
	// OpenConfig component type
	public static final String ANALYTICS_COMMUNICATION_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/analytics-communication";

	private XPHashtable m_mapSettings;
	private boolean m_bLoadSucceeded = false;

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()
	 */
	public String GetVarPackID() {
		return VARPACK_ID;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackKeys()
	 */
	public IXPEnumerator GetVarPackKeys() {
		return m_mapSettings.GetKeys();
	}
	
	/* (non-Javadoc)
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Load()
	 */
	protected void Load() {
		try {
			m_mapSettings = LoadSettings();
			m_bLoadSucceeded = true;
		}
		catch (Exception e) {
			log.Warn("Loading of Analytics settings failed.");			
		}
	}

	/* (non-Javadoc)
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Reload()
	 */
	protected void Reload() {
        m_bLoadSucceeded = false;
        
        Load();
	}

	/* (non-Javadoc)
	 * You cannot expire the content. You can only reload it. This method does nothing.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#ExpireContent()
	 */
	protected void ExpireContent() {
	}

	/* (non-Javadoc)
	 * @see com.plumtree.xpsupport.web.BaseVarPack#GetValue(java.lang.String)
	 */
	protected Object GetValue(String key) {
		if (null != m_mapSettings) {
			return m_mapSettings.GetElement(key);
		} else {
			return null;
		}
	}
	
	private XPHashtable LoadSettings() {
		XPHashtable mapReturn = new XPHashtable();
		
		// Use OpenConfig to read the settings
		IOKContext configContext = AppWarmUp.GetOKConfigContext();
		IOKComponent component = null;
		
		component = configContext.getComponentByType(ANALYTICS_COMMUNICATION_COMPONENT_TYPE);

		if (null == component)
		{
			log.Warn("Loading of Analytics Communication setting component failed.");
			return null;
		}			
			
		IOKSetting[] settings = component.querySettings();
		for(int j=0;j<settings.length;j++)
		{
			String strName = settings[j].getName();
			String strValue = null;
			
			//log.Info("Setting Loaded: " + strName);		        
			//log.Info("Setting type: " + settings[j].getDataType());
		        
			int nSettingType = settings[j].getDataType();
			switch(nSettingType)
			{
				case OKConfigDataTypes.TYPE_BOOLEAN:
					strValue = XPConvert.ToString(settings[j].getBooleanValue());
					break;
				case OKConfigDataTypes.TYPE_STRING:
					strValue = settings[j].getStringValue();
					break;
				case OKConfigDataTypes.TYPE_INTEGER:
					strValue = XPConvert.ToString(settings[j].getIntValue());
					break;
				case OKConfigDataTypes.TYPE_FLOAT:
					strValue = XPConvert.ToString(settings[j].getFloatValue());
					break;
				case OKConfigDataTypes.TYPE_DATE:
					strValue = XPConvert.ToString(settings[j].getDateValue());
					break;
				default:
					throw new XPException("Unknown data type:" + nSettingType);
			}

			//log.Info("Setting value: " + strValue);
				
			if ((null != strName) && (null != strValue))
			{			
				mapReturn.PutElement(strName, strValue);
			}			
		}
		
		return mapReturn;
	}
	
	public String GetConfigValueAsString(String key)
	{
		return (String)GetValue(key);
	}
	
	public int GetConfigValueAsInt(String key)
	{
		String strValue = (String)GetValue(key);
	
		if (null == strValue) {
			return -1;
		} else {
			return XPConvert.ToInteger(strValue);
		}
	}
	
	public boolean GetConfigValueAsBoolean(String key)
	{
		String strValue = (String)GetValue(key);
		
		if (null == strValue) {
			return false;
		} else {
			return XPConvert.ToBoolean(strValue);
		}
	}
}
