package com.plumtree.portaluiinfrastructure.application.varpacks;

import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.sso.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.openkernel.config.*;

/**
 * This VarPack contains the Plumtree Application data.
 * 
 * This extends the base config var pack, and adds extra data
 * from the ptconfig.xml file.
 *
 * @author John Osborne
 */
public class PTConfigVarPack extends ConfigVarPack
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.application.varpacks.PTConfigVarPack");

	// OpenConfig component types
	public static final String PORTAL_DOCUMENTS_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/documents"; 
	public static final String PORTAL_CRAWLERS_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/crawlers";
	public static final String PORTAL_SEARCH_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/search";
	public static final String PORTAL_STYLE_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/style";
	public static final String PORTAL_COMMUNITIES_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/communities";
	public static final String PORTAL_ADMINISTRATION_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/administration";
	public static final String PORTAL_FAILOVER_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/failover";
	public static final String PORTAL_INVITATIONS_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/invitations";
	public static final String PORTAL_GATEWAY_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/gateway";

	/** mj: sso login page relative URL. E.g.: SSOServlet */
	private String m_sSSORelativeURL;

    /** mj: sso login page base URL. E.g.: /portal/SSOServlet */
	private String m_sSSOBaseURL = "";

	/** The XML Document that gets loaded. */
	//private XPDocument m_xmlPTDoc;

	/**
	 * Constructor for PTConfigVarPack.
	 *
	 * @param __strPlatform The current platform "Java" or "DotNet")
	 */
	public PTConfigVarPack(String _strPlatform)
	{
		super(_strPlatform);
	}

	/**
	 * JF- Returns the SSO Base URL. E.g.: /portal/SSOServlet
	 *
	 * @return String
	 */
	public String GetSSOBaseURL()
	{
		return m_sSSOBaseURL;
	}

	/**
	 * JF- Returns the SSO relative URL. E.g.: SSOServlet
	 *
	 * @return String
	 */
	public String GetSSORelativeURL()
	{
		return m_sSSORelativeURL;
	}

  	/**
  	 * Returns an xml element consisting of all the XML in the  specified
  	 * section and subsection. Returns null if no such section or subsection
  	 * exists.
  	 * 
	 * It first tries to get the element from the base class file, and then
	 * from the PTConfig.xml file.
	 * 
	 * WARNING: if the element exists in both the base x_config.xml file and
	 * the PTconfig.xml file, only the element from the base x_config.xml
	 * file will be returned.
  	 * 
  	 * @param _strSection 
  	 * @param _strSubSection 
  	 *
  	 * @return 
  	 */
  	public XPElement GetXMLElement(String _strSection, String _strSubSection)
  	{
		
			log.Error("Accessing XML document directly no longer works.");
		
  		return null;
  	}

	/**
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()
	 */
	public String GetVarPackID()
	{
		return VARPACK_ID;
	}

	/**
	 * This method loads the PTconfig.xml file into memory.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Load()
	 */
	protected void Load()
	{
		super.Load();
		
		/*
		String strFileSeparator = XPFileSystemObject.GetPathSeparator();

		
			log.Info("Loading PTConfig XML document.");
		

		XPDocument xmlPTDoc = XPBuilder.LoadDocument(GetConfigPath() + strFileSeparator + "PTconfig.xml");
		*/

		BuildSSOBaseURL();
		LoadSettingsIntoHashtable();

        // mj: init sso. if this is a reload, StaticInitSSO will be called later.
        SSOIntegrator.StaticInitConfig(this);
        PTLoginHelper.StaticInit(this);
		
		log.Info("Finished loading portal config XML document.");		
	}

	/**
	 * Helper method to load a number of component settings from portalconfig.xml 
	 * and put them in a XPHashtable.
	 * This Hashtable will be used for the Memory Debug page where we need to be able
	 * to iterate through the keys and display all the name/value pairs.
	 */
	private void LoadSettingsIntoHashtable()
	{
		// Use OpenConfig to read the settings
		IOKContext configContext = AppWarmUp.GetOKConfigContext();
		IOKComponent component = null;
		
		component = configContext.getComponentByType(PORTAL_DOCUMENTS_COMPONENT_TYPE);
		LoadComponentSettings(component, PTConfigConstants.DOCUMENTS_SECTION);		

		component = configContext.getComponentByType(PORTAL_CRAWLERS_COMPONENT_TYPE);
		LoadComponentSettings(component, PTConfigConstants.CRAWLERS_SECTION);

		component = configContext.getComponentByType(PORTAL_SEARCH_COMPONENT_TYPE);
		LoadComponentSettings(component, PTConfigConstants.SEARCH_SECTION);

		component = configContext.getComponentByType(PORTAL_STYLE_COMPONENT_TYPE);
		LoadComponentSettings(component, PTConfigConstants.STYLE_SECTION);

		component = configContext.getComponentByType(PORTAL_COMMUNITIES_COMPONENT_TYPE);
		LoadComponentSettings(component, PTConfigConstants.COMMUNITIES_SECTION);

		component = configContext.getComponentByType(PORTAL_ADMINISTRATION_COMPONENT_TYPE);
		LoadComponentSettings(component, PTConfigConstants.ADMINISTRATION_SECTION);

		component = configContext.getComponentByType(PORTAL_FAILOVER_COMPONENT_TYPE);
		LoadComponentSettings(component, PTConfigConstants.FAILOVER_SECTION);
		
		component = configContext.getComponentByType(PORTAL_INVITATIONS_COMPONENT_TYPE);
		LoadComponentSettings(component, PTConfigConstants.INVITATIONS_SECTION);

		component = configContext.getComponentByType(PORTAL_GATEWAY_COMPONENT_TYPE);
		LoadComponentSettings(component, PTConfigConstants.GATEWAY_SECTION);		
	}
	
	/**
	 * Helper method to load the settings from PTconfig.xml and put them in a XPHashtable.
	 * This Hashtable will be used for the Memory Debug page where we need to be able 
	 * to iterate through the keys and display all the name/value pairs.
	 * 
	 * @deprecated Use LoadSettingsIntoHashtable() instead that uses OpenConfig API
	 */
	private void LoadSettingsIntoHashtable(XPDocument _xmlPTDoc)
	{
		// JF- PTconfig.xml is two levels deep. 
		// We need to iterate through both levels before we can actually get the name/value pairs.
		//		TODOCR RM 05/05/2003
		// Replace the following if block and .Item(0) call with
		// XPNode nTopNode = xmlDoc.GetDocumentElement() because
	    // GetChildNodes() on <?xml> tag will return 0 child Nodes.
	    
	    XPElement rootNode = _xmlPTDoc.GetDocumentElement();
		
		
			log.Info("The top root node of x_config.xml is " + rootNode.GetNodeName());
		
	    
	    
		XPNodeList nlFirstLevel = rootNode.GetChildNodes();
		XPNodeList nlSecondLevel;
		String sName, sValue, sFirstLevelName, sKey;
		
		for (int i = 0; i < nlFirstLevel.GetLength(); i ++)
		{
			// JF- Make sure this is a valid node and not a comment
			if (!nlFirstLevel.Item(i).GetNodeName().startsWith("#"))
			{
				nlSecondLevel = nlFirstLevel.Item(i).GetChildNodes();
				sFirstLevelName = nlFirstLevel.Item(i).GetNodeName();
				
				for (int k = 0; k < nlSecondLevel.GetLength(); k ++)
				{
					sName = nlSecondLevel.Item(k).GetNodeName();
				
					// JF- Make sure this is a valid node and not a comment
					if (!sName.startsWith("#"))
					{
						sValue = ((XPElement) nlSecondLevel.Item(k)).GetAttributeNode("value").GetNodeValue();
						sKey = sFirstLevelName + sName;
						m_htConfigXMLSettings.PutElement(sKey, sValue);	
					}
				}	
			}	
		}
	}

	/**
	 * This method reloads the PTconfig.xml file into memory. It clears the
	 * current data rather than delegating to Load().
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Reload()
	 */
	protected void Reload()
	{
		super.Reload();

		Load();

        try
        {
            // mj: call StaticInitSSO in case SSO settings have changed.
            IApplication application = ApplicationManager.GetInstance()
                .GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
            PTSSOVarPack vpSSOConfig     = (PTSSOVarPack) application.GetVarPackManager()
                .GetVariablePackage(PTSSOVarPack.VARPACK_ID);
                
            SSOIntegrator.StaticInitSSO(vpSSOConfig);
        }
        catch (Exception e)
        {
            
                log.Error(e, "Unable to initialize SSO.");
            
        }
	}

	/**
	 * Helper method to build the SSO base URL based on the values from PTconfig.xml.
	 *
	 */
	private void BuildSSOBaseURL()
	{
		m_sSSORelativeURL = GetConfigValueAsString("SystemProperties",
			"SSOServletName", "value");
		
		m_sSSOBaseURL = GetConfigValueAsString("SystemProperties",
				"SSOVirtualDirectoryPath", "value");

		if (!m_sSSOBaseURL.endsWith("/"))
		{
			m_sSSOBaseURL += "/";
		}

		m_sSSOBaseURL += GetConfigValueAsString("SystemProperties",
			"SSOServletName", "value");

		if (m_sSSOBaseURL.endsWith("?"))
		{
			m_sSSOBaseURL.substring(0, m_sSSOBaseURL.length() - 1);
		}

		
			log.Info("Config value: SSO Base URL has been set to: " + m_sSSOBaseURL);
		
	}
}

