/*
 * Created on Feb 13, 2003
 *
 */
package com.plumtree.portaluiinfrastructure.application.varpacks;

import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.sso.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.openkernel.config.*;

/**
 * VarPack for sso.xml, a file used to customize various SSO integrations.
 * It reads in all settings in the file and stores them in a map, which
 * is available via the GetVendorSettings method.
 * 
 * @author Mike Jahr
 */
public class PTSSOVarPack extends BaseVarPack
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.application.varpacks.PTSSOVarPack");

	/** The string ID of this variable package. */
	public static final String VARPACK_ID = "PTSSOVarPack";

	// OpenConfig component type
	public static final String PORTAL_SSOVENDOR_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/ssovendor";

	private XPHashtable m_mapVendorSettings;
	private boolean m_bLoadSucceeded = false;

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackID()
	 */
	public String GetVarPackID()
	{
		return VARPACK_ID;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.web.IVarPack#GetVarPackKeys()
	 */
	public IXPEnumerator GetVarPackKeys()
	{
		return m_mapVendorSettings.GetKeys();
	}

	/**
	 * Returns an <code>XPHashtable</code> from <code>String</code> to
	 * <code>XPArrayList</code>, containing all the settings in
	 * the <code>sso.xml</code> config file.
	 *
	 * @return an XPHashtable of vendor settings
	 */
	public XPHashtable GetVendorSettings()
	{
		if (!m_bLoadSucceeded &&
				log.IsErrorEnabled())
		{
			log.Error("Loading of SSO vendor settings failed.");
		}

		return m_mapVendorSettings;
	}

	/* (non-Javadoc)
	 * You cannot expire the content. You can only reload it. This method does nothing.
	 *
	 * @see com.plumtree.xpsupport.web.BaseVarPack#ExpireContent()
	 */
	protected void ExpireContent()
	{
	}

	/* (non-Javadoc)
	 * @see com.plumtree.xpsupport.web.BaseVarPack#GetValue(java.lang.String)
	 */
	protected Object GetValue(String arg0)
	{
		return m_mapVendorSettings.GetElement(arg0);
	}

	/* (non-Javadoc)
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Load()
	 */
	protected void Load()
	{
		String strFileSeparator = XPFileSystemObject.GetPathSeparator();
		
		try
		{
			/*
			XPDocument xmlDoc = XPBuilder.LoadDocument(GetConfigPath() + strFileSeparator + "sso.xml");

			XPElement xmlRoot = xmlDoc.GetDocumentElement();
			
			m_mapVendorSettings     = LoadSettings(xmlRoot, "value");
			*/

			m_mapVendorSettings     = LoadSettings();
			m_bLoadSucceeded	    = true;
            
            // mj: re-init sso in case settings changed.
            SSOIntegrator.StaticInitSSO(this);
		}
		catch (Exception e)
		{
			
				log.Warn("Loading of SSO vendor settings failed.");
			
		}
	}

	/* (non-Javadoc)
	 * @see com.plumtree.xpsupport.web.BaseVarPack#Reload()
	 */
	protected void Reload()
	{
        m_bLoadSucceeded = false;
        m_mapVendorSettings = null;
        
        Load();
	}

	/**
	 * This loads all settings from the given section.  Returns an XPHashtable containing
	 * strings mapped to XPArrayLists.
	 */
	private static XPHashtable LoadSettings()
	{
		log.Info("LoadSettings() called.");
		
		XPHashtable mapReturn    = new XPHashtable();
		
		// Use OpenConfig to read the settings
		IOKContext configContext = AppWarmUp.GetOKConfigContext();
		IOKComponent component = null;
		
		component = configContext.getComponentByType(PORTAL_SSOVENDOR_COMPONENT_TYPE);

		if (null == component)
		{
			log.Warn("Cannot retrieve component type: {0}", PORTAL_SSOVENDOR_COMPONENT_TYPE);
			return mapReturn;
		}			
			
		IOKSetting[] settings = component.querySettings();
		for (int j=0;j<settings.length;j++)
		{
			String strName = settings[j].getName();
			String strValue = null;
			
			//log.Info("Setting Loaded: " + strName);		        
			//log.Info("Setting type: " + settings[j].getDataType());
		        
			int nSettingType = settings[j].getDataType();
			switch(nSettingType)
			{
				case OKConfigDataTypes.TYPE_BOOLEAN:
					strValue = XPConvert.ToString(settings[j].getBooleanValue());
					break;
				case OKConfigDataTypes.TYPE_STRING:
					strValue = settings[j].getStringValue();
					break;
				case OKConfigDataTypes.TYPE_INTEGER:
					strValue = XPConvert.ToString(settings[j].getIntValue());
					break;
				case OKConfigDataTypes.TYPE_FLOAT:
					strValue = XPConvert.ToString(settings[j].getFloatValue());
					break;
				case OKConfigDataTypes.TYPE_DATE:
					strValue = XPConvert.ToString(settings[j].getDateValue());
					break;
				default:
					throw new XPException("Unknown data type:" + nSettingType);
			}

			//log.Info("Setting value: " + strValue);
				
			if ((null != strName) && (null != strValue))
			{			
				// mj: if this setting is already in the map, we should append the 
				//     value to the existing list
				XPArrayList list = (XPArrayList) mapReturn.GetElement(strName);

				if (list == null)
				{
					list = new XPArrayList();
				}

				XPStringTokenizer strTokenizer = new XPStringTokenizer(strValue, ";");
				String strToken;
				while (strTokenizer.HasMoreTokens())
				{
					strToken = XPStringUtility.Trim(strTokenizer.GetNextToken());
					list.Add(strToken);
				}

				mapReturn.PutElement(strName, list);
			} 
			else 
			{
				log.Error("LoadSettings() failed to load {0} with value {1}", strName, strValue);
			}
		}
		
		return mapReturn;
	}
	
	/**
	 * This loads all settings from the given section.  Returns an XPHashtable containing
	 * strings mapped to XPArrayLists.
	 * 
	 * @deprecated User LoadSettings() instead, which uses OpenConfig API to load the XML.
	 */
	private static XPHashtable LoadSettings(XPElement xmlRoot, String sAttribute)
	{
		
			log.Info("LoadSettings() called.");
		

		XPNodeList xmlList = null;
		XPHashtable mapReturn    = new XPHashtable();

		try
		{
			// mj: get the section
			xmlList = xmlRoot.GetChildNodes();

			// mj: iterate over members of the section
			int nLength = xmlList.GetLength();

			for (int i = 0; i < nLength; i++)
			{
				XPNode xmlNode	    = null;
				XPNode xmlAttribute = null;
				String sName	    = null;
				String sValue	    = null;

				// mj: we need to skip all the random text nodes. 
				//     we only care about the XPElements.
				xmlNode = xmlList.Item(i);

				if (!(xmlNode instanceof XPElement))
				{
					continue;
				}

				try
				{
					sName			 = xmlNode.GetNodeName();
					xmlAttribute     = xmlNode.GetAttributes().GetNamedItem(sAttribute);
					sValue			 = xmlAttribute.GetNodeValue();

					
						log.Info("LoadSettings() is adding the setting \"" + sName +
							"\", value \"" + sValue + "\".");
					

					// mj: if this setting is already in the map, we should append the 
					//     value to the existing list
					XPArrayList list = (XPArrayList) mapReturn.GetElement(sName);

					if (list == null)
					{
						list = new XPArrayList();
					}

					list.Add(sValue);

					mapReturn.PutElement(sName, list);
				}
				catch (Exception e)
				{
					
						if ((sName != null) && (xmlAttribute == null))
						{
							log.Error(e, "LoadSettings() failed to find the attribute " +
								sAttribute + " for the setting " + sName);
						}
						else if (sName != null)
						{
							log.Error(e, "LoadSection() failed for the setting " +
								sName);
						}
						else
						{
							log.Error(e, "LoadSection() failed for the node " + xmlNode);
						}
					
				}
			}
		}
		catch (Exception e)
		{
			
				if (xmlList == null)
				{
					log.Error(e, "LoadSettings() failed to find any settings.");
				}
				else
				{
					log.Error(e, "LoadSection() failed for the attribute " + sAttribute);
				}
			
		}

		return mapReturn;
	}
}

