package com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.friendlyurl.*;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.portaluiinfrastructure.opener.plugins.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.marshalers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Common Opener Mode Description:
 * 1) Create - Show Error (PTCommunityPages).
 * 2) Edit - Redirect to View (PTCommunityPages).
 * 3) View - Goes to the community page (PTCommunityPages).
 * 4) Meta - Redirect to View (PTCommunityPages).
 * 
 * @author Naveen Pai
 */
public class PTPages extends APTNonExtClassTypeDesc
{
    /*
     * Setting two fields here as follows provides support for Admin UI->Release Locks
     * The Left Navigation Menu string will be loaded from "ptmsgs_portaladminmsgs" file using objectLockManagerMsgID
     * If it should be loaded from a different file, set objectLockManagerMsgFileName accordingly
     */ 
    public PTPages() {
        super();
        hasObjectLockManager = true;
        objectLockManagerMsgID   = 2785;
    }

	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes.PTCommunityPages");

	public static final String MY_PAGE_AS_NAME = "MyPage";
	public static final String COMM_PAGE_AS_NAME = "CommunityPage";
	public static final String MY_PAGE_CONTROL_NAME = "SetPage";
	public static final String COMM_PAGE_CONTROL_NAME = "SetCommunity";
	public static final String STR_PAGE_ID = "PageID";
	public static final String STR_COMM_ID = "CommunityID";
	public static final String EDITPAGE_AS_NAME = "EditMyPortalPage";
	public static final String EDITPAGE_CONTROL_NAME = "EditorStart";
	public static final String STR_OPENERPAGE_ID = "OpenerPageID";
	public static final String STR_OPENERCONTAINER_ID = "OpenerCommunityID";
	public static final String STR_EDITORTYPE_ID = "editorType";
	public static final String PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_OBJECT_ID = "ptID";
	public static final String PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_INHERIT = "inherit";
	public static final String PAGE_EDITOR_QS_INT_DEFAULT_PROFILE_ID = "dpID";

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#AppearsInMainCreateNewClassMenu()
	 */
	public boolean AppearsInMainCreateNewClassMenu()
	{
		return true;
	}
	
	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.APTClassTypeDesc#GetCreateActivityID()
	 */
	public int GetCreateActivityID()
	{
		return PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_NONE;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.APTNonExtClassTypeDesc#GetASNameForEditor()
	 */
	public String GetASNameForEditor()
	{
		return "PageEditor";
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetClassID()
	 */
	public int GetClassID()
	{
		return PT_CLASSIDS.PT_PAGE_ID;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetClassTypeLocalizedName(java.lang.String)
	 */
	public String GetClassTypeLocalizedName(String _strLangID)
	{
		IApplication application = ApplicationManager.GetInstance()
			.GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
		XPResourceManager rm = (XPResourceManager) application.GetCachingManager()
			.GetEntry("ResourceManager");

		return rm.GetString(365, "ptmsgs_portalinfrastructure", _strLangID);
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetDefaultIcon()
	 */
	public HTMLImg GetDefaultIcon()
	{
		try
		{
			// we re-use the my page icon.
			HTMLImg imgIcon = (HTMLImg) UIConstructFactory.GetConstruct(
				PTImageType.ICON_MYPAGE).GetHTMLElement();

			return imgIcon;
		}
		catch (Exception e)
		{
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetRedirectViewObject(com.plumtree.openfoundation.util.XPHashtable, com.plumtree.uiinfrastructure.activityspace.AActivitySpace, com.plumtree.server.IPTSession)
	 */
	public Redirect GetRedirectViewObject(XPHashtable _htArguments, AActivitySpace _asParent, Object _userSession)
	{
		int nPageID = HTTPHelpers.GetArgumentAsInt(_htArguments, IPTOpenerConstants.QS_OBJECTID, 
			HTTPHelpers.GetArgumentAsInt(_htArguments, IPTOpenerConstants.STR_QS_OBJECTID, -1));
			
		IPTPageInfo pageInfo = null;
		try
		{
			pageInfo = ((IPTPageManager)((IPTSession)_userSession).GetPages()).CachedOpenPageInfo(nPageID, false, false);
		}
		catch (XPException ex)
		{
			try
			{
				// it wasn't a community page they had read acl on, maybe it is a my page
				pageInfo = ((IPTSession)_userSession).GetSessionInfo().GetCurrentMyPage(nPageID);
			}
			catch (XPException ex2)
			{
				// either it was another user's my page, or a community page the user did not have read acl on
				// send them to their home my page
			}
		}

		// Create the Redirect Object
		Redirect resultingRedirect = new Redirect();

		// Set the space
		if ( null == pageInfo || PT_CLASSIDS.PT_MYPAGESCONTAINER_ID == pageInfo.GetContainerClassID() )
		{
			resultingRedirect.SetLinkCreateNewSpace(MY_PAGE_AS_NAME, _asParent);
			resultingRedirect.SetControl(MY_PAGE_CONTROL_NAME);
		}
		else
		{
			resultingRedirect.SetLinkCreateNewSpace(COMM_PAGE_AS_NAME, _asParent);
			resultingRedirect.SetControl(COMM_PAGE_CONTROL_NAME);
			resultingRedirect.AddControlArgument(STR_COMM_ID,
				XPConvert.ToString(pageInfo.GetContainerObjectID()));
		}
		if ( null == pageInfo )
		{
			resultingRedirect.AddControlArgument(STR_PAGE_ID,
				XPConvert.ToString(0));
		}
		else
		{
			resultingRedirect.AddControlArgument(STR_PAGE_ID,
				XPConvert.ToString(nPageID));
		}
		
		// no parent space or parent ID
		resultingRedirect.SetStripParentSpace(true);

		return resultingRedirect;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetRelatedClassTypeIDs()
	 */
	public int[] GetRelatedClassTypeIDs()
	{
		return null;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#HasGlobalClassTypeMap()
	 */
	public boolean HasGlobalClassTypeMap()
	{
		return false;
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IClassTypeDesc#GetRedirectCreateNew(com.plumtree.openfoundation.util.XPHashtable, com.plumtree.uiinfrastructure.activityspace.AActivitySpace, com.plumtree.server.IPTSession)
	 */
	public Redirect GetRedirectCreateNew(XPHashtable _htArguments, AActivitySpace _asParent, Object _userSession)
	{
		Redirect result = super.GetRedirectCreateNew(_htArguments, _asParent, _userSession);
		
		int nDefaultProfileID = HTTPHelpers.GetArgumentAsInt(_htArguments, PAGE_EDITOR_QS_INT_DEFAULT_PROFILE_ID, 0);
		if (0 != nDefaultProfileID)
		{
			result.AddControlArgument(PAGE_EDITOR_QS_INT_DEFAULT_PROFILE_ID, nDefaultProfileID);
		}
		
		int nPageTemplateID = HTTPHelpers.GetArgumentAsInt(_htArguments, PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_OBJECT_ID, 0);
		if (0 != nPageTemplateID)
		{
			result.AddControlArgument(PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_OBJECT_ID, nPageTemplateID);
			result.AddControlArgument(PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_INHERIT, HTTPHelpers.GetArgumentAsBoolean(_htArguments, PAGE_EDITOR_QS_INT_PAGE_TEMPLATE_INHERIT, (0 != nPageTemplateID)));
		}
		
		return result;
	}
	
	/**
	 * @see IPTClassTypeDesc#ConstructFriendlyURL(String, int, String) 
	 * 
	 * Generates a friendly Url for Community page
	 * 
	 * In the worst case, this method will perform two cache lookups, and two db queries
	 * 
	 */
	public String ConstructFriendlyURL(FriendlyURL url, AActivitySpace space)
	{
		// name of page
		String objectName = url.GetOjectName();
		// page id
		int objectId = url.GetObjectId();
		
		if ( ("".equals(objectName) || objectName == null) && objectId <= 0 ) { 
			throw new PTException("Both object name and id are missing. At leaset one needs to be specified to create a friendly Url.", PT_RESULTCODES.PT_E_INVALIDARGUMENT); }
		
		IPTSession session = (IPTSession)space.GetUserSession();
		IPTPageInfo curPage = ((IPTPageManager)session.GetPages()).CachedOpenPageInfo(objectId, false, false);
		if ( objectName == null )
		{
			objectName = curPage.GetName();
		}
		
		XPStringBuilder buf = new XPStringBuilder(200);
		
		int commId = curPage.GetContainerObjectID();
		if (commId < 1) {
			// DCA - we're not dealing with a community page, return null
			return null;
		}
		IPTCommunityInfo comm = ((IPTCommunityManager)session.GetCommunities()).CachedOpenCommunityInfo(commId, false);
		// @TODO how to handle errors here?
		String commName = comm.GetName();
		
				
		String commKeyword = 
			((IFriendlyURLManager)PTConfigHelper.GetTypeDescrManager()).LookupFriendlyURLKeyFromClassId(PT_CLASSIDS.PT_COMMUNITY_ID+"");
		if ( commKeyword == null ) {
			throw new PTException("No Friendly URL keyword was found for Communities. "+
					"Verify that a mapping entry exists in FriendlyURL.xml.", PT_RESULTCODES.PT_E_FAIL);
		}
		buf.Append("/").Append(commKeyword);
		
		// Add the community name which has to come before the community page name
		commName = FriendlyURL.NormalizeObjectName(commName);
		// DCA - we're encoding in case it has non-ASCII chars
		commName = XPHttpUtility.UrlEncode(commName, LinkConstants.UTF_8_ENCODING);
		buf.Append("/").Append(commName);
		
		if ( FriendlyURLPlugin.UseObjectId() ) {
			buf.Append("/").Append(commId);
		}
		
		// now add the child name, which is the page name
		objectName = FriendlyURL.NormalizeObjectName(objectName);
		// DCA - we're encoding in case it has non-ASCII chars
		objectName = XPHttpUtility.UrlEncode(objectName, LinkConstants.UTF_8_ENCODING);
		buf.Append("/").Append(objectName);
		
		return buf.ToString();
	}
}

