package com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.XPHttpUtility;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.friendlyurl.FriendlyURL;
import com.plumtree.portaluiinfrastructure.friendlyurl.IFriendlyURLParser;
import com.plumtree.portaluiinfrastructure.opener.*;
import com.plumtree.portaluiinfrastructure.opener.plugins.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.marshalers.PTException;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.AppWarmUp;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Common Opener Mode Description:
 * 1) Create - User editor in create mode (APTNonExtClassTypeDesc).
 * 2) Edit - User editor in edit mode (APTNonExtClassTypeDesc).
 * 3) View - View user profile page (PTUsers).
 * 4) Meta - View object properties page (APTClassTypeDesc).
 * 
 * @author John Osborne
 */
public class PTUsers extends APTNonExtClassTypeDesc
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_FRIENDLYURL),
		"com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes.PTUsers");

	public static final String COMMUNITY_AS_NAME = "CommunityPage";	
	public static final String COMMUNITY_CONTROL_NAME = "SetCommunity";	

	public static final String SEARCH_AS_NAME = "SearchResult";
	public static final String SEARCH_CONTROL_NAME = "advancedstart";
	public static final String SEARCH_USER_QUERY   = "in_tx_query";
	public static final String SEARCH_REQD_OBJECT_TYPE = "in_hi_req_objtype";

	/*
     * Setting two fields here as follows provides support for Admin UI->Release Locks
     * The Left Navigation Menu string will be loaded from "ptmsgs_portaladminmsgs" file using objectLockManagerMsgID
     * If it should be loaded from a different file, set objectLockManagerMsgFileName accordingly
     */ 
    public PTUsers() {
        super();
        hasObjectLockManager = true;
        objectLockManagerMsgID   = 1568;
    }

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#AppearsInMainCreateNewClassMenu()
	 */
	public boolean AppearsInMainCreateNewClassMenu()
	{
		return true;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.APTNonExtClassTypeDesc#GetASNameForEditor()
	 */
	public String GetASNameForEditor()
	{
		return "UserEditor";
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetClassID()
	 */
	public int GetClassID()
	{
		return PT_CLASSIDS.PT_USER_ID;
	}
	
	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetClassTypeLocalizedName(java.lang.String)
	 */
	public String GetClassTypeLocalizedName(String _strLangID)
	{
		IApplication application = ApplicationManager.GetInstance()
			.GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
		XPResourceManager rm = (XPResourceManager) application.GetCachingManager()
			.GetEntry("ResourceManager");

		return rm.GetString(30, "ptmsgs_portalinfrastructure", _strLangID);
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.APTClassTypeDesc#GetCreateActivityID()
	 */
	public int GetCreateActivityID()
	{
		return PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_USERS;	
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetDefaultIcon()
	 */
	public HTMLImg GetDefaultIcon()
	{
		try
		{
			HTMLImg imgIcon = (HTMLImg) UIConstructFactory.GetConstruct(
				PTImageType.ICON_USER).GetHTMLElement();

			return imgIcon;
		}
		catch (Exception e)
		{
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetRedirectViewObject(com.plumtree.openfoundation.util.XPHashtable, com.plumtree.uiinfrastructure.activityspace.AActivitySpace, com.plumtree.server.IPTSession)
	 */
	public Redirect GetRedirectViewObject(XPHashtable _htArguments,
		AActivitySpace _asParent, Object _userSession)
	{
		// Create the Redirect Object
		Redirect resultingRedirect = new Redirect();

		resultingRedirect.SetLinkCreateNewSpace("CommunityPage", _asParent);

		// Spoof the sub-space key
		String strSubSpaceKey = HTTPHelpers.GetArgumentAsString(_htArguments, ASConstants.SUBSPACE,
			HTTPHelpers.GetArgumentAsString(_htArguments, ASConstants.SUBSPACE_INPUT, ""));

		if (!strSubSpaceKey.equals(""))
		{
			resultingRedirect.SetSubSpaceKey(strSubSpaceKey);
		}

		resultingRedirect.SetControl("SetCommunity");
		resultingRedirect.AddControlArgument("CommunityID", PT_INTRINSICS.PT_PROFILE_COMMUNITY);
		// page ID 0 means the default page, which is general information unless they change the page.
		resultingRedirect.AddControlArgument("PageID", 0);

		int nProfileID = HTTPHelpers.GetArgumentAsInt(_htArguments, IPTOpenerConstants.QS_OBJECTID, 
			HTTPHelpers.GetArgumentAsInt(_htArguments, IPTOpenerConstants.STR_QS_OBJECTID, -1));
		resultingRedirect.AddControlArgument("ProfileID", nProfileID);

		return resultingRedirect;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetRedirectViewObject()
	 */
	public Redirect GetRedirectViewObject(IFriendlyURLParser _fURLParser, AActivitySpace _asParent, Object _userSession)
	{
		if (null == _fURLParser || null == _asParent || null == _userSession) {
			throw new PTException("Parameters cannot be null.", PT_RESULTCODES.PT_E_INVALIDARGUMENT);
		}
		// Create the Redirect Object
		Redirect resultingRedirect = new Redirect();
		
		String strUserName = _fURLParser.GetObjectName(true);
		int nUserId = _fURLParser.GetObjectId();
		// By default this will be 0.
		int nPageId = _fURLParser.GetChildId();

		// look up the userID if just a userName was specified
		int[] arrUserIds = null;
		if (nUserId < 1) {
			IPTObjectManager ptUserManager = (IPTObjectManager) ((IPTSession)_userSession).GetUsers();
			arrUserIds = ptUserManager.LookupObjectIdFromName(strUserName);
			if (arrUserIds.length == 1) {
				nUserId = arrUserIds[0];
			}
		}

		// Lookup the pageID if just a pageName was specified. -1 means no page ID was specified
		if (nPageId == -1)
		{
			String strPageName = _fURLParser.GetChildName(true);
			if (strPageName != null)
			{
				IPTObjectManager ptCommunityManager = (IPTObjectManager) ((IPTSession) _userSession).GetCommunities();

				// map page name to page Id
				try {
					nPageId = ((IPTCommunityManager) ptCommunityManager).LookupPageIdFromCommunityPageName(PT_INTRINSICS.PT_PROFILE_COMMUNITY, strPageName);
				} catch (XPException e) {
					// revert to profile home page
					nPageId = 0;
					log.Info(e, "Could not lookup user profile page with name: " + strPageName +
						". Redirecting to user profile home page.");
				}				
			} else
			{
				// revert to default if we don't have a valid page name,
				// although this should never be the case.
				nPageId = 0;
				log.Info("Could not find user profile page with name: " + strPageName +
					". Redirecting to user profile home page.");
			}
		}

		if (nUserId > 0) {
			// nUserId > 0 means there was a single user with that name
			// Use cached version if available in case it's a repost
			resultingRedirect.SetLinkGetSpaceIfCached(COMMUNITY_AS_NAME, _asParent);
			resultingRedirect.SetControl(COMMUNITY_CONTROL_NAME);
			resultingRedirect.AddControlArgument("CommunityID", PT_INTRINSICS.PT_PROFILE_COMMUNITY);
			// page ID 0 means the default page, which is general information unless they change the page.
			resultingRedirect.AddControlArgument("PageID", nPageId);
			resultingRedirect.AddControlArgument("ProfileID", nUserId);
		}
		 else {
			// nUserId <= 0 means there was more than one user w/ the same name
			// redirect to a search results querying for the user name
			resultingRedirect = new Redirect();
			resultingRedirect.SetLinkCreateNewSpace(SEARCH_AS_NAME, _asParent);
			resultingRedirect.SetControl(SEARCH_CONTROL_NAME);
			resultingRedirect.AddControlArgument(SEARCH_USER_QUERY, strUserName);
			resultingRedirect.AddControlArgument(SEARCH_REQD_OBJECT_TYPE, PT_CLASSIDS.PT_USER_ID);
			// This currently ignores the requested page if it needs to redirect to search.
			// TODO: Does this need to handle the requested page, or can it just go to
			// the home page?
		}		
		return resultingRedirect;
	}

	/**
	 * @see IPTClassTypeDesc#ConstructFriendlyURL(String, int)
	 * 
	 * Generates a friendly Url for user
	 */	
	public String ConstructFriendlyURL(FriendlyURL url, AActivitySpace space)
	{		
		String userName = url.GetOjectName();
		int userId = url.GetObjectId();

		XPStringBuilder buf = new XPStringBuilder(200);		
		if ( ("".equals(userName) || userName == null) && userId < 1 ) { 
			throw new PTException("Both user name and id are missing. At least a name needs to be specified to create a friendly Url.", PT_RESULTCODES.PT_E_INVALIDARGUMENT); }
		
		IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
		IFriendlyURLManager ptFriendlyURLMgr = (IFriendlyURLManager) application.GetCachingManager().GetEntry(AppWarmUp.KEY_CACHEMAN_CLASS_TYPE_MANAGER);
		String userKeyword = ptFriendlyURLMgr.LookupFriendlyURLKeyFromClassId(PT_CLASSIDS.PT_USER_ID+"");

		// user keyword part of URL
		if ( userKeyword == null ) {
			throw new PTException("No Friendly URL keyword was found for users. "+
					"Verify that a mapping entry exists in FriendlyURL.xml.", PT_RESULTCODES.PT_E_FAIL);
		}
		buf.Append("/").Append(userKeyword);
		
		// need to lookup object name from object id.
		if ( ("".equals(userName) || userName == null) ) {
			try {
				IPTProfileManager pm = (IPTProfileManager) ((IPTSession)space.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_PROFILE_MANAGER, false);
				Object[][] arResultUserName = (Object[][]) pm.GetUserName(userId);
				userName = (String)arResultUserName[0][0];
			} catch (PTException e) {
				throw new PTException(e, 
						"Invalid user id provided: " + userId + ". The user does not exist or you do not have access to the object.", 
						PT_RESULTCODES.PT_E_INVALIDARGUMENT);
			}
		}
		
		// user name part of URL 
		String normUserName = FriendlyURL.NormalizeObjectName(userName);
		// encode value in case it has non-ASCII chars
		normUserName = XPHttpUtility.UrlEncode(normUserName, LinkConstants.UTF_8_ENCODING);
		buf.Append("/").Append(normUserName);		
		if(userName == null ){
			throw new PTException("FriendlyURL: User " + userId + " does not exist!",  PT_RESULTCODES.PT_E_FAIL);
		}
		
		// user ID part of URL
		if ( FriendlyURLPlugin.UseObjectId() ) {
			buf.Append("/").Append(userId);
		}

		String pageName = url.GetChildName();
		int pageId = url.GetChildId();

		// Add page name/ID if available.
		if ( pageId != 0) 
		{
			// we don't have page name so look it up from the id
			if ( pageName == null ) {
				IPTPageInfo page = null;
				IPTSession session = (IPTSession)space.GetUserSession();
				try {
					page = ((IPTPageManager)session.GetPages()).CachedOpenPageInfo(pageId, false, false);
				} catch (PTException e) {
					throw new PTException(e, 
							"Invalid page id provided: "+pageId+ " . The user profile page does not exist or the user does not have access to it.", 
							PT_RESULTCODES.PT_E_INVALIDARGUMENT);
				}
				pageName = page.GetName();
			}

			// normalize page name
			pageName = FriendlyURL.NormalizeObjectName(pageName);
			// DCA - we're encoding in case it has non-ASCII chars
			pageName = XPHttpUtility.UrlEncode(pageName, LinkConstants.UTF_8_ENCODING);
			buf.Append("/").Append(pageName);
			// always add page id too to disambiguate page names
			if ( FriendlyURLPlugin.UseObjectId() ) {
				buf.Append("/").Append(pageId);
			}
		}

		return buf.ToString();
	}	
	
	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetRelatedClassTypeIDs()
	 */
	public int[] GetRelatedClassTypeIDs()
	{
		return null;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#HasGlobalClassTypeMap()
	 */
	public boolean HasGlobalClassTypeMap()
	{
		return false;
	}
}