package com.plumtree.portaluiinfrastructure.classtypedescriptors.classtypes;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Common Opener Mode Description:
 * 1) Create - Web services editor in create mode (PTWebServices).
 * 2) Edit - Web services editor in edit mode (APTExtClassTypeDesc).
 * 3) View - Redirect to Edit (APTClassTypeDesc).
 * 4) Meta - View object properties page (APTClassTypeDesc).
 *
 * @author John Osborne
 */
public class PTWebServices extends APTExtClassTypeDesc
{
    /*
     * Setting two fields here as follows provides support for Admin UI->Release Locks
     * The Left Navigation Menu string will be loaded from "ptmsgs_portaladminmsgs" file using objectLockManagerMsgID
     * If it should be loaded from a different file, set objectLockManagerMsgFileName accordingly
     */ 
    public PTWebServices() {
        super();
        hasObjectLockManager = true;
        objectLockManagerMsgID   = 2003;
    }

	public static final String QS_WS_REMOTE_SERVER = "remoteServer";

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#AppearsInMainCreateNewClassMenu()
	 */
	public boolean AppearsInMainCreateNewClassMenu()
	{
		return true;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.APTExtClassTypeDesc#GetASNameForEditor(java.lang.String)
	 */
	public String GetASNameForEditor(String _strProvEnglishName)
	{
		String strBaseEditorASName = "WebServiceEditor";

		return _strProvEnglishName + strBaseEditorASName;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetClassID()
	 */
	public int GetClassID()
	{
		return PT_CLASSIDS.PT_WEBSERVICE_ID;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.APTClassTypeDesc#GetCreateActivityID()
	 */
	public int GetCreateActivityID()
	{
		return PT_ACTIVITY_RIGHTS.PT_ACTIVITY_RIGHT_CREATE_WEBSERVICE_INFRASTRUCTURE;	
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetClassTypeLocalizedName(java.lang.String)
	 */
	public String GetClassTypeLocalizedName(String _strLangID)
	{
		IApplication application = ApplicationManager.GetInstance()
			.GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
		XPResourceManager rm     = (XPResourceManager) application.GetCachingManager()
			.GetEntry("ResourceManager");

		return rm.GetString(31, "ptmsgs_portalinfrastructure", _strLangID);
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.APTExtClassTypeDesc#GetCLSIDForExistingObject(int, com.plumtree.server.IPTSession)
	 */
	public final String GetCLSIDForExistingObject(int _nObjectID,
		IPTSession _ptSession)
	{
		IPTWebService ptWebService = (IPTWebService) _ptSession.GetWebServices()
			.Open(_nObjectID, false);
		String sProviderCLSID = ptWebService.GetProviderCLSID();

		return sProviderCLSID;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetDefaultIcon()
	 */
	public HTMLImg GetDefaultIcon()
	{
		try
		{
			HTMLImg imgIcon = (HTMLImg) UIConstructFactory.GetConstruct(
				PTImageType.ICON_WEB_SERVICE).GetHTMLElement();

			return imgIcon;
		}
		catch (Exception e)
		{
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetRedirectCreateNew(com.plumtree.openfoundation.util.XPHashtable, com.plumtree.uiinfrastructure.activityspace.AActivitySpace, com.plumtree.server.IPTSession)
	 */
	public Redirect GetRedirectCreateNew(XPHashtable _htArguments,
		AActivitySpace _asParent, Object _userSession) 
	{
		Redirect resultingRedirect = new Redirect();

		resultingRedirect = super.GetRedirectCreateNew(_htArguments, _asParent, _userSession);
					
		// Pass on the Remote Server ID, if it exists
		String strRemoteServerID = HTTPHelpers.GetArgumentAsString(_htArguments,
				QS_WS_REMOTE_SERVER, "");
		resultingRedirect.AddControlArgument(QS_WS_REMOTE_SERVER,
			strRemoteServerID);

		return resultingRedirect;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetRelatedClassTypeIDs()
	 */
	public int[] GetRelatedClassTypeIDs()
	{
		return null;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#GetRelatedClassTypeIDs()
	 */
	public int[] GetDependentClassTypeIDs()
	{
		// Technically, only certain kinds of web services can have
		// certain kinds of dependent objects, but it doesn't hurt
		// to try to delete all of these objects before any Web
		// Service.
		return new int[] {
			PT_CLASSIDS.PT_GADGET_ID,
			PT_CLASSIDS.PT_GADGETTEMPLATE_ID,
			PT_CLASSIDS.PT_DATASOURCE_ID,
			PT_CLASSIDS.PT_AUTHSOURCE_ID,
			PT_CLASSIDS.PT_PROFILESOURCE_ID,
			PT_CLASSIDS.PT_FEDERATEDPORTAL_ID };
	}

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.classframework.IPTClassTypeDesc#HasGlobalClassTypeMap()
	 */
	public boolean HasGlobalClassTypeMap()
	{
		return false;
	}
}

