package com.plumtree.portaluiinfrastructure.editor.saveobject;

import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.*;

/**
 * This is the AS class for the save object pop-up window.
 * The save object window will pop-up at the end of each
 * object Editor in create mode.
 *
 * @author Julie Fournier
 */
public class SaveObjectAS extends AFormAS
{
	/**
	 * Class name for the activity space
	 */
	public static final String STR_MVC_CLASS_NAME = "SaveObjectPopUp";

	/**
	 * @see com.plumtree.portaluiinfrastructure.activityspace.AActivitySpace#CheckBasicAccess(String, String, boolean)
	 */
	public boolean CheckBasicAccess(String strPage, String strControl,
		boolean bSameSpace)
	{
		// JF- This is a sub AS and the security Access has been checked by the Object Editor.
		// If somebody is trying to access this AS directly, it will fail.
		return true;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new SaveObjectAS();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the name of the repost control class.
	 * E.g.: PropRepostControl.GetName()
	 *
	 * @see com.plumtree.xpshared.form.IRepostAS#GetRepostControlName()
	 *
	 * @return String
	 */
	public String GetRepostControlName()
	{
		return SaveObjectRepostControl.STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IActivitySpace#Init()
	 */
	public void Init()
	{
		super.Init();

		// Register Main Model
		RegisterModel(SaveObjectModel.STR_MVC_CLASS_NAME);

		IModel myModel = GetModel(SaveObjectModel.STR_MVC_CLASS_NAME);

		// Register Display Pages
		RegisterPage(new SaveObjectMainFrameDP());
		RegisterPage(new SaveObjectTopFrameDP());
		RegisterPage(new SaveObjectBottomFrameDP());
		RegisterPage(new SaveObjectRedirectDP());
		RegisterPage(new SaveObjectCantChangeFolderDP());
		RegisterPage(new SaveObjectRecentFoldersDP());
		RegisterPage(new SaveObjectFoldersFrameDP());
		RegisterPage(new SaveObjectGoToSecurityTabDP());
		RegisterPage(new SaveObjectClosePopUpDP());

		// Register Views
		RegisterView(SaveObjectNameDescView.STR_MVC_CLASS_NAME, myModel);
		RegisterView(SaveObjectMenuView.STR_MVC_CLASS_NAME, myModel);
		RegisterView(SaveObjectCantChangeFolderView.STR_MVC_CLASS_NAME, myModel);
		RegisterView(SaveObjectRecentFoldersView.STR_MVC_CLASS_NAME, myModel);

		// Register Controls
		RegisterControl(SaveObjectStartControl.STR_MVC_CLASS_NAME, myModel);
		RegisterControl(SaveObjectRepostControl.STR_MVC_CLASS_NAME, myModel);

		// JF- The Security Page should not display the header and the left navigation.
		AActivitySpace spaceParent = GetParentSpace();
		if (null != spaceParent && this instanceof SaveObjectAS)
		{
			((ObjectSecurityModel) spaceParent.GetModel(ObjEditorAS.STR_MVC_STORAGE_NAME_SECURITY_TAB)).SetShowHeaderAndLeftNav(false);
		}
	}
}

