package com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This is the base intrinsic portlet preferences model.  All intrinsic portlet
 * models should extend this base class.  This model extends the editor model.
 *
 * @author josephc
 */
public abstract class IntrinsicPortletPrefsEditorModel extends EditorModel implements IEditorModelRO, IIntrinsicPortletPrefsModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IntrinsicPortletPrefsEditorModel");

	/** Setting type for user portlet preferences */
	public static final int SETTINGSTYPE_PORTLET = 1;

	/** Setting type for admin preferences */
	public static final int SETTINGSTYPE_ADMIN = 2;

	/** Setting type for community preferences */
	public static final int SETTINGSTYPE_COMMUNITY = 3;

	/** Setting type for community portlet preferences */
	public static final int SETTINGSTYPE_COMMUNITYPORTLET = 4;

	/** Setting type for user preferences */
	public static final int SETTINGSTYPE_USER = 5;

	/** Preference type for admin preferences */
	public static final int PREF_TYPE_ADMIN = 1;

	/** Preference type for personal preferences */
	public static final int PREF_TYPE_PERSONAL = 2;

	/** Preference type for community preferences */
	public static final int PREF_TYPE_COMMUNITY = 3;

	/** IPTGadget */
	protected IPTGadget m_ptPortlet;

	/** IPTSession */
	protected IPTSession m_ptSession;

	/** CommunityID */
	protected int m_nCommunityID = 0;

	/** PageID */
	protected int m_nPageID;

	/** PortletID */
	protected int m_nPortletID;

	/** Preference type */
	protected int m_nPrefType;
    
    protected String m_strPreviousURL;

	/**
	 * That method is called by the EditorStartControl.
	 *
	 * @param _htFormData
	 *
	 * @see com.plumtree.uiinfrastructure.editor.EditorModel#StartEditor(XPHashtable)
	 */
	public boolean CheckEditorAccessAndStart(XPHashtable _htFormData)
	{
		// Set pageID, communityID, and portletID
		SetMemberVariablesFromQS(_htFormData);

		// Initialize the portlet
		InitializePortlet(_htFormData);

		// Perform Editor specific tasks
		DoTaskOnStartEditor(_htFormData);

		return true;
	}

	/**
	 * 
	 *
	 * @return 
	 */
	public int GetEditorType()
	{
		return super.GetEditorType();
	}

	/**
	 * Returns the DeleteOnCancel flag.
	 * For sub Editors only:
	 * this flag will determine if the Editor
	 * should be deleted from the list of sub Editors
	 * on cancel.
	 *
	 * @return boolean
	 */
	public boolean GetHideLeftNav()
	{
		return super.GetHideLeftNav();
	}

	/**
	 * Enables or disables the apply button next to the finish button.
	 *
	 * @param _bEnableApplyButton
	 */
	public void SetEnableApplyButton(boolean _bEnableApplyButton)
	{
		super.SetEnableApplyButton(_bEnableApplyButton);
	}

	/**
	 * Returns true if the apply button should be displayed.
	 *
	 * @return boolean
	 */
	public boolean GetEnableApplyButton()
	{
		return super.GetEnableApplyButton();
	}

	/**
	 * Returns true if the current user only has read access to this Editor.
	 * (can be true in Edit mode only)
	 *
	 * @return boolean
	 */
	public boolean GetIsEditorReadOnly()
	{
		return super.GetIsEditorReadOnly();
	}

	/**
	 * Get portlet name
	 *
	 * @return Portlet name
	 *
	 * @see com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetPortletName()
	 */

	//JD- TODOCR 1/16/03
	//Remote gadgets have functionality where the portlet writer can set the title of the portlet.
	//This is done using CSP Protocol.  Not sure if you want to offer the same capability to 
	//intrinsic portlet writers.
	public String GetPortletName()
	{
		return m_ptPortlet.GetLocalizedName();
	}

	/**
	 * Returns the type of preference that is being requested.
	 *
	 * @return Preference type
	 *
	 * @see com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetPrefType()
	 */
	public int GetPrefType()
	{
		return m_nPrefType;
	}

	/**
	 * Returns the Redirect object  (this is were we need to go at the end of
	 * the Editor). For prefs pages this just closes the popup window since
	 * the prefs page should be in a popup.
	 *
	 * @return Redirect
	 */
	public Redirect GetRedirectAfterFinish()
	{
        Redirect myRedirect = new Redirect();
        if (m_asOwner.GetIsAccessStyleStandard()) {
    		HTMLScript script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
    
    		try
    		{
    			script.AddInnerHTMLString("window.close();");
    		}
    		catch (HTMLException he)
    		{
    			log.Error(he, "EditorRepostControl::SetRedirectToCloseSubEditor An error occured when trying to display the JavaScript.");
    		}
    
    		ClosePopUpDP myClosePopUp = (ClosePopUpDP) m_asOwner.GetPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
    		myClosePopUp.SetScript(script);
    
    		myRedirect.SetLinkGetCachedSpace(m_asOwner);
    		myRedirect.SetDisplayPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
        } else {
            myRedirect.SetIsErrorRedirect(false);
            myRedirect.SetLinkToExternalURL(m_strPreviousURL);
        }

		return myRedirect;
	}

	/**
	 * Returns a single setting value, based on the setting type specified.
	 *
	 * @param nSettingType Setting type
	 * @param strName Setting name
	 *
	 * @return Setting value
	 *
	 * @see com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetSettingValue(int,
	 *      String)
	 */
	public Object GetSettingValue(int nSettingType, String strName)
	{
		String[] arFilter = new String[] { strName };
		Object[][] arValues;
		Object oValue = null;

		arValues = GetSettingValues(nSettingType, arFilter);

		if ((null != arValues) && (arValues.length > 0))
		{
			oValue = arValues[1][0];
		}

		return oValue;
	}

	//JD- TODOCR 1/16/03
	//There are helper functions to get user preferences in LoginHelper.  I'm not sure whether
	//or not they should be used here... but either way there should be a comment specifying 
	//what the decision is, and why was it made.
	//JD- TODOCR 1/16/03
	// Perhaps it's worth doing a sanity check that if community or community-gadget prefs
	// are set, it's happenning on a community.  This applies to all functions that set,
	// read, or remove a pref.
	//JD- TODOCR 1/16/03
	// How are you handling security for admin prefs? What happens if someone who does not
	// have edit rights on the Portlet object tries to set admin prefs?

	/**
	 * Returns all settings, based on the setting type and filter specified.
	 *
	 * @param nSettingType Setting type
	 * @param arFilter Filter for settings
	 *
	 * @return All settings
	 *
	 * @see com.plumtree.portaluiinfrastructure.intrinsicportlets.prefs.IIntrinsicPortletPrefsModelRO#GetSettingValues(int,
	 *      Object[])
	 */
	public Object[][] GetSettingValues(int nSettingType, String[] arFilter)
	{
		Object[][] arValues;

		switch (nSettingType)
		{
		case SETTINGSTYPE_PORTLET:
			if (PREF_TYPE_COMMUNITY == m_nPrefType)
			{
				arValues = (Object[][]) ((IPTCommunityManager)m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nCommunityID, true)
					.LookupPreferences(arFilter, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_CURRENT_USER);
			}
			else
			{
				arValues = (Object[][]) m_ptSession.GetSessionInfo().LookupPreferences(arFilter, m_nPortletID);
			}
			//arValues = (Object[][]) m_ptMyPortal.LookupGadgetSettings(m_nPortletID, arFilter);

			break;

		case SETTINGSTYPE_ADMIN:
		{
			IPTAdminSettings ptAdminSettings = (IPTAdminSettings) m_ptPortlet.GetInterfaces("IPTAdminSettings");
			arValues = (Object[][]) ptAdminSettings.LookupAdminGadgetSettings(arFilter);
		}

		break;

		case SETTINGSTYPE_COMMUNITY:
			
			arValues = (Object[][]) ((IPTCommunityManager)m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nCommunityID, true)
				.LookupPreferences(arFilter, 0, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
			//arValues = (Object[][]) m_ptMyPortal.LookupCommunitySettings(arFilter);

			break;

		case SETTINGSTYPE_COMMUNITYPORTLET:
			arValues = (Object[][]) ((IPTCommunityManager)m_ptSession.GetCommunities()).CachedOpenCommunityInfo(m_nCommunityID, true)
				.LookupPreferences(arFilter, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
			//arValues = (Object[][]) m_ptMyPortal.LookupCommunityGadgetSettings(m_nPortletID, arFilter);

			break;

		case SETTINGSTYPE_USER:
			arValues = (Object[][]) m_ptSession.GetSessionInfo().LookupPreferences(arFilter, 0);
			//arValues = (Object[][]) m_ptMyPortal.LookupPersonalSettings(arFilter);

			break;

		default:
			return null;
		}

		return arValues;
	}

	/**
	 * Inits the Model with the IPTSession.
	 *
	 * @param parent
	 */
	public void Init(AActivitySpace parent)
	{
		super.Init(parent);

		// JF- Get the Session object from the associated AS.
		m_ptSession = (IPTSession) parent.GetUserSession();
        m_strPreviousURL = HTMLConstructHelpers.GetPreviousSpaceURL(m_asOwner);
	}

	/**
	 * Removes a setting value, depending on the specified setting type.
	 *
	 * @param nSettingType Setting type
	 * @param strName Setting name
	 */
	public void RemoveSettingValue(int nSettingType, String strName)
	{
		switch (nSettingType)
		{
		case SETTINGSTYPE_PORTLET:
			if (PREF_TYPE_COMMUNITY == m_nPrefType)
			{
				((IPTCommunity)m_ptSession.GetCommunities().Open(m_nCommunityID, false))
					.RemovePreference(strName, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_CURRENT_USER);
			}
			else
			{
				m_ptSession.GetSessionInfo().RemovePreference(strName, m_nPortletID);
			}
			//m_ptMyPortal.RemoveGadgetSetting(m_nPortletID, strName);

			break;

		case SETTINGSTYPE_ADMIN:
		{
			IPTAdminSettings ptAdminSettings = (IPTAdminSettings) m_ptPortlet.GetInterfaces("IPTAdminSettings");

			ptAdminSettings.RemoveAdminGadgetSetting(strName);
		}

		break;

		case SETTINGSTYPE_COMMUNITY:
			((IPTCommunity)m_ptSession.GetCommunities().Open(m_nCommunityID, false))
				.RemovePreference(strName, 0, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
			//m_ptMyPortal.RemoveCommunitySetting(strName);

			break;

		case SETTINGSTYPE_COMMUNITYPORTLET:
			((IPTCommunity)m_ptSession.GetCommunities().Open(m_nCommunityID, false))
				.RemovePreference(strName, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
			//m_ptMyPortal.RemoveCommunityGadgetSetting(m_nPortletID, strName);

			break;

		case SETTINGSTYPE_USER:
			m_ptSession.GetSessionInfo().RemovePreference(strName, 0);
			//m_ptMyPortal.RemovePersonalSetting(strName);

			break;

		default:
			break;
		}
	}

	/**
	 * Helper method for the StartEditor method. Fill out the member variables
	 * with data from QS.
	 *
	 * @param _htFormData
	 */
	public void SetMemberVariablesFromQS(XPHashtable _htFormData)
	{
		// Set the page ID if it exists
		String[] data = (String[]) _htFormData.GetElement(IntrinsicPortletPrefsUtils.PAGEID);

		if ((null != data) && (0 != data.length) && (data[0] != null))
		{
			m_nPageID = XPConvert.ToInteger(data[0]);

			
				log.Debug("IntrinsicPortletPrefsEditorModel::SetMemberVariablesFromQS PageID: " + data[0]);
			
		}

		// Set the community ID if it exists
		data = (String[]) _htFormData.GetElement(IntrinsicPortletPrefsUtils.COMMUNITYID);

		if ((null != data) && (0 != data.length) && (data[0] != null))
		{
			m_nCommunityID = XPConvert.ToInteger(data[0]);

			
				log.Debug("IntrinsicPortletPrefsEditorModel::SetMemberVariablesFromQS CommunityID: " + data[0]);
			
		}

		// Set the portlet ID if it exists
		data = (String[]) _htFormData.GetElement(IntrinsicPortletPrefsUtils.PORTLETID);

		if ((null != data) && (0 != data.length) && (data[0] != null))
		{
			m_nPortletID = XPConvert.ToInteger(data[0]);

			
				log.Debug("IntrinsicPortletPrefsEditorModel::SetMemberVariablesFromQS PortletID: " + data[0]);
			
		}
	}

	/**
	 * Sets the appropriate setting value, based on the setting type.
	 *
	 * @param nSettingType Setting type
	 * @param strName Setting name
	 * @param oValue Setting value
	 */
	public void SetSettingValue(int nSettingType, String strName, Object oValue)
	{
		Object[][] arValues;

		switch (nSettingType)
		{
		case SETTINGSTYPE_PORTLET:
			arValues = new Object[2][1];
			arValues[0][0] = strName;
			arValues[1][0] = oValue;

			if (PREF_TYPE_COMMUNITY == m_nPrefType)
			{
				((IPTCommunity)m_ptSession.GetCommunities().Open(m_nCommunityID, false))
					.AssignPreferences(arValues, false, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_CURRENT_USER);
			}
			else
			{
				m_ptSession.GetSessionInfo().AssignPreferences(arValues, false, m_nPortletID);
			}
			//m_ptMyPortal.AssignGadgetSettingsEx(m_nPortletID, arValues, false);

			break;

		case SETTINGSTYPE_ADMIN:
		{
			IPTAdminSettings ptAdminSettings = (IPTAdminSettings) m_ptPortlet.GetInterfaces("IPTAdminSettings");
			ptAdminSettings.AddAdminGadgetSetting(strName, oValue);
		}

		break;

		case SETTINGSTYPE_COMMUNITY:
			((IPTCommunity)m_ptSession.GetCommunities().Open(m_nCommunityID, false))
				.AddPreference(strName, oValue, 0, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
			//m_ptMyPortal.AddCommunitySetting(strName, oValue);

			break;

		case SETTINGSTYPE_COMMUNITYPORTLET:
			arValues = new Object[2][1];
			arValues[0][0] = strName;
			arValues[1][0] = oValue;

			((IPTCommunity)m_ptSession.GetCommunities().Open(m_nCommunityID, false))
				.AssignPreferences(arValues, false, m_nPortletID, PT_PREF_TYPES.PT_PREFTYPE_ALL_USERS);
			//m_ptMyPortal.AssignCommunityGadgetSettingsEx(m_nPortletID, arValues, false);

			break;

		case SETTINGSTYPE_USER:
			m_ptSession.GetSessionInfo().AddPreference(strName, oValue, 0);
			//m_ptMyPortal.AddPersonalSetting(strName, oValue);

			break;

		default:
			break;
		}
	}
	
	public int GetCommunityId() {
		return m_nCommunityID;
	}

	/**
	 * If this model is started as a sub-editor from the Portlet editor's model, then
	 * get the portlet object from the editor model using this method.
	 *
	 * @param asParent Parent space of editor when started as a subeditor
	 *
	 * @return Portlet object from the PortletEditor's model
	 */
	private IPTGadget GetPortletFromSpace(AActivitySpace asParent)
	{
		// Use "Portlet" here instead of using PortletModel.STR_MVC_CLASS_NAME because
		// we don't want to introduce dependency on portal pages.
		// The "correct" way might be to iterate through all the models from the parent space,
		// looking for an ObjEditorModel, which would be our PortletModel.
		ObjEditorModel oModel = (ObjEditorModel) asParent.GetModel("Portlet" /*PortletModel.STR_MVC_CLASS_NAME*/);
		return (IPTGadget) oModel.GetObject();
	}

	/**
	 * Initialize the member variables of the model.  If the pageID is 0, then
	 * we are editing admin pref, and so get the IPTMyPortal object from the
	 * parent space and don't create an AppDataState object.
	 *
	 * @param _htFormData Table containing form data
	 */
	protected void InitializePortlet(XPHashtable _htFormData)
	{
		AActivitySpace asParent = m_asOwner.GetParentSpace();

		try
		{
			// Check if we're coming from Portlet Prefs editor or Portlet editor
			if (null != asParent)
			{
				// We're coming from Portlet editor.
				m_ptPortlet = GetPortletFromSpace(asParent);

				if (0 == m_nPortletID)
				{
					IPTServerContext ptSC = (IPTServerContext) m_ptPortlet.GetInterfaces("IPTServerContext");
					m_nPortletID = ptSC.GetObjectID();
				}

				m_nPrefType = PREF_TYPE_ADMIN;
			}
			else
			{
				// We're coming from Portlet Prefs editor

				if (0 < m_nCommunityID)
				{
					m_nPrefType = PREF_TYPE_COMMUNITY;
				}
				else
				{
					m_nPrefType = PREF_TYPE_PERSONAL;
				}

				// Get the portlet object
				m_ptPortlet = (IPTGadget) m_ptSession.GetGadgets().Open(m_nPortletID, false);
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "Error while initialize portlet preferences editor.");
			

			throw XPException.GetInstance(e);
		}
	}
}

