package com.plumtree.portaluiinfrastructure.opener.plugins;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.friendlyurl.*;
import com.plumtree.portaluiinfrastructure.search.PaginationControl;
import com.plumtree.portaluiinfrastructure.search.ReorganizationControl;
import com.plumtree.portaluiinfrastructure.search.SearchResultControl;
import com.plumtree.portaluiinfrastructure.search.SearchResultModel;
import com.plumtree.portaluiinfrastructure.search.exceptions.EmptyQueryException;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.server.marshalers.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.interpreter.Interpreter;
import com.plumtree.uiinfrastructure.opener.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * This plugin enables the Common Opener to open objects using
 * friendly URLs.
 * 
 * @author DustinA
 * 
 * CR rob - shouldnt you make a varpack to load the config file instead? Seems overly complicated
 * to parse your own xml file
 * 
 */
public class FriendlyURLPlugin implements IOpenerPlugins 
{
	// node for the url detection pattern to check for friendly urls
	public static final String NODENAME_DETECTIONPATTERN = "DetectionPattern";
	//
	public static final String NODENAME_DISABLEOBJECTIDS = "DisableObjectId";

	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
			"com.plumtree.portaluiinfrastructure.opener.plugins.FriendlyURLPlugin");
    
	private static final String SLASH = "/";
	private static String URI_PATTERN = null;
	private static boolean DISABLE_OBJECTIDS = false; 
	
    public FriendlyURLPlugin() {
    	// DCA - load settings from config file
    	if (null == URI_PATTERN) {
    		// DCA - create document object to access config
			String strSettingsPath = AppWarmUp.GetSettingValueAsString(AppWarmUp.KEY_PATH_PORTAL_SETTINGS);
			String strFriendlyURLsXMLFile = GeneralHelpers.FormatPath(strSettingsPath +
					"FriendlyURLs.xml", AppWarmUp.PATH_SEPARATOR, false);
			XPDocument document = XPBuilder.LoadDocument(strFriendlyURLsXMLFile);
			log.Debug("FriendlyURLPlugin:FriendlyURLPlugin Retrieving configuration from " + strFriendlyURLsXMLFile);
			SetUpConfiguration(document);
    	}
	}
    
    public static void SetUpConfiguration(XPDocument document) { 
    	//DCA - navigate to get the pattern for detecting friendly URLs
		XPNodeList nlPattern = document.GetElementsByTagName(NODENAME_DETECTIONPATTERN);
		XPNode node = nlPattern.Item(0);
		if ( null != node && node.GetFirstChild() != null) {
			URI_PATTERN = node.GetFirstChild().GetNodeValue();
		} else {
			throw new XPException("No setting for Url detection pattern for friendly Urls found in FriendlyURLs.xml. Unable to continue.");
		}
		log.Info("FriendlyURLPlugin:FriendlyURLPlugin Pattern to detect friendly URLs: " + URI_PATTERN);
		
		XPNodeList disObjId = document.GetElementsByTagName(NODENAME_DISABLEOBJECTIDS);
		node = disObjId.Item(0);
		if ( null != node && node.GetFirstChild() != null) {
			DISABLE_OBJECTIDS = XPConvert.ToBoolean( node.GetFirstChild().GetNodeValue() );
		} else {
			log.Info("No setting for disabling Object Ids in friendly Urls found in FriendlyURLs.xml. Disabling Object Ids by default.");
		}
		
		
		// DCA - add all key to class mappings into the table
		XPHashtable htKeyToClassIdLookup = new XPHashtable();
		XPHashtable htClassIdToKeyLookup = new XPHashtable();
		XPNodeList nlKeys = document.GetElementsByTagName("key");
		XPNodeList nlClassIds = document.GetElementsByTagName("classId");
		for (int i=0; i< nlKeys.GetLength(); i++) {
			node = nlKeys.Item(i);
			String key = node.GetFirstChild().GetNodeValue();
			node = nlClassIds.Item(i);
			String value = node.GetFirstChild().GetNodeValue();
			htKeyToClassIdLookup.PutElement(key, value);
			htClassIdToKeyLookup.PutElement(value, key);
			log.Debug("FriendlyURLPlugin:FriendlyURLPlugin Friendly URL mapping found: " + key + " -> " + value);
		}
		
		IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
		application.GetCachingManager().SetEntry("FriendlyURLKeyFromClassIdLookupTable", htClassIdToKeyLookup);
		
		IFriendlyURLParserManager fURLParserManager = FriendlyURLParserManager.getInstance();
		fURLParserManager.SetKeyToClassIdLookupTable(htKeyToClassIdLookup);
    }

	/**
	 * @see com.plumtree.portaluiinfrastructure.opener.IOpenerPlugins#ProcessPlugIn(RedirectData, AActivitySpace, Object)
	 * 
	 * ** CR rob - highlevel description of what is going on in this method would be useful
	 * 
	 */
	public RedirectData ProcessPlugIn(RedirectData _redirectData, AActivitySpace _asCurrentSpace, Object _userSession)
	{
		// DCA - check if the request URI contains the pattern "server.pt/"
		String strURI = _asCurrentSpace.GetCurrentHTTPRequest().GetRequestURI();
		int index = XPStringUtility.IndexOf(strURI, URI_PATTERN);
		if (index > -1) {
			String strError = null;
			// ** CR rob - do you really know this is a friendly url from just this check? What happens with invalid urls?
			// DCA - Invalid URLs will cause an exception and will be handled in the catch clause
			log.Debug("FriendlyURLPlugin:ProcessPlugIn Request is in Friendly URL format.");
			try
			{
				// DCA - grab everything after URI_PATTERN
				String strFriendlyURL = XPStringUtility.Substring(strURI, index + URI_PATTERN.length());
				// ** CR rob - this string tokenizer is really messy and you pass it around between a few methods
				// ** creating a class with methods like, has page name, has page id, has community name etc, would be
				// ** much cleaner
				// DCA - at this point, we don't really know if the friendly URL is to a community page, a my page, a search result page, a document page, etc.
				// I don't think we want a class that would have a method for every possible part of a friendly URL
				XPStringTokenizer stURLTokens = new XPStringTokenizer(strFriendlyURL, SLASH, false);

				if (stURLTokens.HasMoreTokens()) {
					IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
					IFriendlyURLParserManager fURLParserManager = FriendlyURLParserManager.getInstance();
					IFriendlyURLParser fURLParser = fURLParserManager.GetParser(stURLTokens);
					
					boolean isHandled = HandleNonObjectFriendlyURLs(fURLParser, _redirectData, _asCurrentSpace);
					if (isHandled) {
						return _redirectData;
					}
					
					// DCA - check that we found a class type descriptor id and 
					// that the URL has at least the object name in it
					if (null != fURLParser.GetClassId() && null != fURLParser.GetObjectName(false)) {
						log.Debug("FriendlyURLPlugin:ProcessPlugIn Calling ClassTypeDescManager to process friendly URL.");
						// DCA - use class Id and the rest of the URL tokens to get the correct redirect
						IFriendlyURLManager ptFriendlyURLMgr = (IFriendlyURLManager) application.GetCachingManager().GetEntry(AppWarmUp.KEY_CACHEMAN_CLASS_TYPE_MANAGER);
						
						Redirect redirect = ptFriendlyURLMgr.GetFriendlyViewOpenerRedirect(fURLParser, _asCurrentSpace, _userSession);
						_redirectData.SetRedirect(redirect);
						_redirectData.SetHaltPlugins(true);
					} else {
						
						if (null == fURLParser.GetClassId()) {
							// DCA - no keyword mapping found
							strError = _asCurrentSpace.GetString(546, "ptmsgs_portalinfrastructure");
							strError = XPResourceManager.FormatMessage(strError, fURLParser.GetKeyword());
						} else {
							// DCA - incomplete URL
							strError = _asCurrentSpace.GetString(547, "ptmsgs_portalinfrastructure");
						}
					}
				} else {
					//  DCA - incomplete URL
					strError = _asCurrentSpace.GetString(547, "ptmsgs_portalinfrastructure");
				}
				if (null != strError) {
					// DCA - we encountered an error
					UIError myError = new UIError(UIErrorLevel.FRIENDLY,
							UIErrorScope.CURRENT_USER, null,
							strError, strError);
					myError.SetActivitySpace(Interpreter.DEFAULTERRORSPACE);
					myError.SetClearWithActivitySpaceChange(true);

					UIErrorHandler.RaiseError(myError, _asCurrentSpace.GetSubSession(), _asCurrentSpace.GetApplication());
					throw new XPException("An error occured while trying to parse the request. Check that the URL is valid.");
				}
			}
			catch (Exception e)
			{
				// generic error
				String error 		   = _asCurrentSpace.GetString(681, "ptmsgs_portalbrowsingmsgs");
				String extendedError   = _asCurrentSpace.GetString(682, "ptmsgs_portalbrowsingmsgs");
				// check if we need to set more specific errors
				if ( e instanceof PTException) {
					if ( ((PTException)e).GetErrorCode() == PT_RESULTCODES.PT_E_SEARCH_SERVER_CONNECTION ) {
						error 		  = _asCurrentSpace.GetString(1967, "ptmsgs_portalbrowsingmsgs");
						extendedError = _asCurrentSpace.GetString(1968, "ptmsgs_portalbrowsingmsgs");
					}
				}
				
				UIError myError = new UIError(UIErrorLevel.FRIENDLY,
						UIErrorScope.CURRENT_USER, e,
						error,
						extendedError);
				myError.SetActivitySpace(BaseOpenerAS.STR_MVC_CLASS_NAME);
				myError.SetClearWithActivitySpaceChange(true);

				UIErrorHandler.RaiseError(myError, _asCurrentSpace.GetSubSession(), _asCurrentSpace.GetApplication());
				// DCA - there is also a System.UnauthorizedAccessException in dotNet
				throw new com.plumtree.uiinfrastructure.interpreter.UnauthorizedAccessException();
			}
			
			// DCA - handle query strings (example: sorting, pagination)
			IXPEnumerator xpQueryParams = _asCurrentSpace.GetCurrentHTTPRequest().GetParameterNames();
			while (xpQueryParams.MoveNext()) {
				Object paramName = xpQueryParams.GetCurrent();
				String paramValue = _asCurrentSpace.GetCurrentHTTPRequest().GetParameterValue((String) paramName);
				_redirectData.GetRedirect().AddControlArgument((String) paramName, paramValue);
			}
		} 
		// not a Friendly URL
		else
		{
		    log.Debug("FriendlyURLPlugin:ProcessPlugIn Request is not in Friendly URL format.");
		}
		
		return _redirectData;
	}
	
	/**
	 * DCA - Current way of handling non object friendly URLs. If this if/else statements get too long, we should
	 * figure out a different way of doing this.
	 * @param _fURLParser
	 * @param _redirectData
	 * @param _asCurrentSpace
	 * @return
	 */
	public boolean HandleNonObjectFriendlyURLs(IFriendlyURLParser _fURLParser, RedirectData _redirectData, AActivitySpace _asCurrentSpace) {
		Redirect redirect = new Redirect();
		if (FriendlyURLConstants.DEBUG_UI_CLASS_ID.equals(_fURLParser.GetClassId())) {
			// DCA - redirect to the debug layout space		
			redirect.SetLinkCreateNewSpace("DebugLayout", _asCurrentSpace);
		} else if (FriendlyURLConstants.DEBUG_MEMORY_CLASS_ID.equals(_fURLParser.GetClassId())) {
			// DCA - redirect to the memory debug space
			redirect.SetLinkCreateNewSpace("MemoryDebug", _asCurrentSpace);
		} else if (FriendlyURLConstants.DEBUG_PORTAL_CLASS_ID.equals(_fURLParser.GetClassId())) {
			// DCA - redirect to the diagnostic page space
			redirect.SetLinkCreateNewSpace("diagnostic", _asCurrentSpace);
			redirect.SetControl(EditorStartControl.STR_MVC_CLASS_NAME);
			redirect.AddControlArgument(EditorStartControl.QS_EDITOR_TYPE, EditorStartControl.EDITOR_START_FLAG_MISC);
		} else if (FriendlyURLConstants.DEBUG_CLASS_ID.equals(_fURLParser.GetClassId())) {
			// DCA - redirect to the debug space
			redirect.SetLinkCreateNewSpace("Debug", _asCurrentSpace);
		} else if (FriendlyURLConstants.REST_API_ID_STRING.equals(_fURLParser.GetClassId())) {
			// Some APIs expect the child ID to be an integer, some a string.  We just let
			// the API controls handle parsing it.
			if (APIFriendlyURLParser.API_TYPE_KD.equals(_fURLParser.GetChildName(false)))
			{
				// redirect to the Dir Rest Control
				redirect.SetLinkCreateNewSpace("Dir", _asCurrentSpace);
				redirect.SetControl("DirREST");
				redirect.AddControlArgument("folderID", ((APIFriendlyURLParser) _fURLParser).GetChildIDString());
			} else if (APIFriendlyURLParser.API_TYPE_USER.equals(_fURLParser.GetChildName(false)))
			{
				// redirect to the User Rest Control
				redirect.SetLinkCreateNewSpace("UserREST", _asCurrentSpace);
				redirect.SetControl("UserREST");
				redirect.AddControlArgument("user", ((APIFriendlyURLParser) _fURLParser).GetChildIDString());
			} else
			{
				log.Error("Unknown Rest API type " + _fURLParser.GetChildName(false) +
					". Was expecting " + APIFriendlyURLParser.API_TYPE_KD + " or " +
					APIFriendlyURLParser.API_TYPE_USER + ".");
				return false;
			}
		} else if (FriendlyURLConstants.FRIENDLY_SEARCH_URL.equals(_fURLParser.GetClassId())) {
			// RT - redirect to the search page space
			redirect.SetLinkCreateNewSpace(SearchResultModel.STR_MVC_CLASS_NAME, _asCurrentSpace);
			redirect.SetControl("bannerstart");	
			
			XPHashtable qsArgs = _redirectData.GetQSArguments();
			
			IXPEnumerator qsEnum = qsArgs.GetKeys();
			
			// This is boolean flag to verify if url has search string or not.
			boolean bQueryStr = false;
			
			// Adding all query parameters to the BannerStartSearchControl
			// before setting the Redirect.
			while (qsEnum.MoveNext())
			{
				String currKey = (String) qsEnum.GetCurrent();
				// RT - Adding search string as control argument. 
				if(XPStringUtility.EqualsIgnoreCase(currKey, SearchFriendlyURLParser.SEARCH_QS))
				{					
					String strQueryVal = ((String[]) qsArgs.GetElement(SearchFriendlyURLParser.SEARCH_QS))[0];
					redirect.AddControlArgument(SearchResultControl.USER_QUERY, strQueryVal );
					// Setting the flag that search string is added as control argument.
					bQueryStr = true;
				} else {
					if(XPStringUtility.EqualsIgnoreCase(currKey, SearchResultControl.USER_QUERY))
					{
						bQueryStr = true;
					}
					String[] strQueryVal = ((String[]) qsArgs.GetElement(currKey));
					redirect.AddControlArgument(currKey, strQueryVal );
				}
			}
			// If there is no search string parameter and value in the url then 
			// we can not proceed.
			if(!bQueryStr)
			{
				throw new EmptyQueryException(_asCurrentSpace);
			}		
			//TODO: Make object ID into Friendly URL and add it to control argument
				
		} else {
			
			return false;
		}
		_redirectData.SetRedirect(redirect);
		_redirectData.SetHaltPlugins(true);
		return true;
	}

	/**
	 * Method to access the URI Pattern set.
	 * @return URI Pattern set
	 */
	public String GetURIPattern() {
		if (null == URI_PATTERN) {
			throw new XPException("The FriendlyURLPlugin class was not initialized properly.");
		}
		return URI_PATTERN;
	}
	
	/**
	 * This method uninitializes the FriendlyURLPlugin class. This should only be used in testing.
	 *
	 */
	public static void UnInit() {
		URI_PATTERN = null;
	}
	
	/**
	 * Checks if generated friendly URLs should have object ids in them
	 * @return
	 */
	public static boolean UseObjectId()
	{
		return !DISABLE_OBJECTIDS;
	}
}
