/*
 * Created on Apr 30, 2003
 */
package com.plumtree.portaluiinfrastructure.statichelpers;

//import com.plumtree.openkernel.counters.*;
import com.plumtree.openlog.*;
import com.plumtree.server.impl.core.*;
import com.plumtree.server.portalpagesperf.*;
import com.plumtree.uiinfrastructure.web.*;


/**
 * Updates the performance counters available through PortalPagesPerf.  
 * Also handles the case when the DLL is not present and disables
 * future requests.  See com.plumtree.server.portalpagesperf.PORTALPAGESPERF_COUNTERS 
 * for a list of available counters.
 * 
 * @author DavidP
 */
public class PTPerfCounters implements IPerfCounterHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.statichelpers.PTPerfCounters");

	private static boolean s_bInitialized = false;
	private static boolean s_bHosed = false;
	
    private static int OPEN_SESSIONS = 0;
    
	// Singleton perf counter object; avoids multiple loaded DLL's	
	private static IPortalPagesReporter perfInstance = null;
	
	/**
	 * Initialize the performance counter object. If there is an exception, shuts down all future 
	 * logging for this instance.
	 */
	public void Init()
	{
		/*
		// TODOCR: DonH - is this actually called anywhere?
		if  (!s_bInitialized && !s_bHosed)
		{
			try
			{
				IOKCounterSystem ocSys = InternalSession.GetCounterService();
				PortalPagesReporter reporter = new PortalPagesReporter();
				perfInstance = reporter;
				IOKCounterCategoryInstance counterReporter = reporter;
				if (null != ocSys) {
					ocSys.RegisterInstance(counterReporter);
				} else {
					log.Warn("PTPerfCounters: did not find an opencounters system to register the portalpages reporter with. Counters were not registered.");
				}

			}
			catch( Exception e )
			{
				s_bHosed = true;
				
					log.Error(e, "PTPerfCounters: error creating perf counter object. Counters are now disabled.");
								
			}
			s_bInitialized = true;
			
				log.Info("PTPerfCounters: PortalPagesPerf performance counters are initialized.");
							
		}
		*/
		// TODOCR: DonH - Should probably log an error if someone tries to init twice.
	}

	/**
	 * Increment the performance counter.  If there is an exception, shuts down all future 
	 * logging for this instance.  The passed integer must be of the enum
	 * PORTALPAGESPERF_COUNTERS.
	 * 
	 * @param counterVal
	 */
	public void IncrementCounter( int counterVal )
	{
        if (GetSessionCounterID() == counterVal) 
        {
            OPEN_SESSIONS++;
        }
		if  (s_bInitialized && !s_bHosed)
		{
			try
			{
				perfInstance.IncrementCounter( counterVal );
			}
			catch( Exception e )
			{
				s_bHosed = true;
				
					log.Error(e, "Plumtree Helpers - error incrementing perf counter. Counters are now disabled.");
								
			}
		}
	}

	/**
	 * Decrement the performance counter.  If there is an exception, shuts down all future 
	 * logging for this instance.  The passed integer must be of the enum
	 * PORTALPAGESPERF_COUNTERS.
	 * 
	 * @param counterVal
	 */
	public void DecrementCounter( int counterVal )
	{
        if (GetSessionCounterID() == counterVal) 
        {
            OPEN_SESSIONS--;
        }
		if  (s_bInitialized && !s_bHosed)
		{
			try
			{
				perfInstance.DecrementCounter( counterVal );
			}
			catch( Exception e )
			{
				s_bHosed = true;
				
					log.Error(e, "Plumtree Helpers - error decrementing perf counter. Counters are now disabled.");
								
			}
		}
	}
	
	/**
	 * Checks that the perf counter interface is initialized
	 */
	public boolean IsInitialized()
	{
		return s_bInitialized;
	}

	/**
	 * Checks that the perf counter interface is not disabled
	 * This occurs if an exception is thrown
	 */
	public boolean IsDisabled()
	{
		return s_bHosed;	
	}

	/**
	 * Gets the integer for the counter for session total
	 */
	public int GetSessionCounterID()
	{
		return PORTALPAGESPERF_COUNTERS.PORTALPAGESPERF_TOTAL_OPENSESSIONS;
	}
    
    public int GetOpenSessions() 
    {
        return OPEN_SESSIONS;
    }
}

