package com.plumtree.portaluiinfrastructure.subscribematrix;


// Plumtree Server
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.common.constants.*;
import com.plumtree.portaluiinfrastructure.editor.*;
import com.plumtree.portaluiinfrastructure.editor.commonpages.objectsecurity.*;
import com.plumtree.portaluiinfrastructure.expandablelist.selectptobject.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * This is the implementation of the Expandable List
 * framework for the the Select PTObject list.
 * The Select PTObject is a list of objects.
 * New objects can be added to the list by selecting
 * them from the associated Object tree.
 * The list is by default sorted by name ascending.
 *
 * @author Julie Fournier
 */
public class MultiObjectsSubscribeModel extends ObjectSecurityModel
	implements IMultiObjectsSubscribeModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.subscribematrix.MultiObjectsSubscribeModel");

	/** Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "MultiObjectsSubscribe";

	/** Class name for the activity space */
	public static final String QS_MULTI_OBJ_SUBSCRIBE_IDS = "objectAndClassIDs";
	
	/** Subscribe column */
	public static final int EXPLIST_COL_SUBSCRIBE = 6;
	
	/** Object img */
	public static final int EXPLIST_COLUMN_IMG = 5;

	/** Array of objects we are editing the security for. */
	protected XPArrayList m_arrObjects;

	/** Array of column info. */
	protected int[] m_arrColumns;
	
	/** 
	 * Flag to know if we are Editing the Security on Admin Folders.
	 * If we are, we also need to display the ABO pop-up when the user clicks on Finish.
	 */
	private boolean m_bEditAdminFolderSecurity = false;
	
	/**
	 * Array of Object IDs.
	 */
	private int[] m_arrObjectIDs;
	
	/**
	 * Bulk Subscribe Job ID
	 */
	private int m_nJobID;
	
	/**
	 * Bulk Subscribe Job Name
	 */
	private String m_sJobName;
	
	/**
	 * Bulk Subscribe Job Folder Name
	 */
	private String m_sFolderJobCreatedInName;
	
	private XPArrayList m_xpalSubscriptions;
	
	private boolean m_bHaveSubscriptionsBeenMade;

	/**
	 * Helper method that adds a row to the list.
	 * 
	 * @param _nClassID
	 * @param _nObjectID
	 * @param _sName
	 */
	protected void AddACLToList(int _nClassID, int _nObjectID, String _sName)
	{
		Object[] oNewRow = new Object[m_arrColumns.length];
		
		oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX]     = new Integer(m_nNextIndex);
		m_nNextIndex++;
		oNewRow[EXPLIST_COL_CLASSID]	  = new Integer(_nClassID);
		oNewRow[EXPLIST_COL_OBJECTID]     = new Integer(_nObjectID);
		oNewRow[EXPLIST_COL_STRING]		  = _sName;
	
		// JF- Defaut selected access level: read
		oNewRow[EXPLIST_COL_ACCESSLEVEL_ID]		    = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);
		oNewRow[EXPLIST_COL_ACCESSLEVEL_STRING]     = GetAccessLevelString(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);
	
		// JF- By default the user has read/write access to the row
		oNewRow[EXPLIST_COL_ROWACCESS]     = new Boolean(PT_ACCESS_LIST_ENTRY_READWRITE);
	
		// JF- By default the object is not mandatory
		oNewRow[EXPLIST_COL_MANDATORY]     = new Integer(MANDATORY_FALSE);
	
		// JF- By default there is no tree ID
		oNewRow[EXPLIST_COL_GROUPTREEASID] = "";
		
		int k = EXPLIST_COL_MAX;

		while (k < m_arrColumns.length)
		{
			oNewRow[k]		   = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);
			oNewRow[k + 1]     = GetAccessLevelString(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT);
			oNewRow[k + 2]     = "0";
			k				   = k + 3;
		}
	
		super.GetItems().AddRow(oNewRow);
	}
	
	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new MultiObjectsSubscribeModel();
	}

	/**
	 * Saves the security settings into the IPTObject.
	 *
	 */
	public void Finish()
	{
		IPTBulkSubscriber subscriber = m_ptSession.GetBulkSubscriber();
		
		for (int i = 0; i < m_arrObjects.GetSize(); i++)
		{
			StoreSubscriptionsToObject((IPTObject) m_arrObjects.GetElement(i), subscriber);
		}
		
		if ( m_bHaveSubscriptionsBeenMade )
		{
			m_nJobID = subscriber.ScheduleImmediateJob();

			try
			{
				// get the job's localized name
				IPTJob myTempJob = (IPTJob) m_ptSession.
					GetObjectManagers(PT_CLASSIDS.PT_JOB_ID).
					Open(m_nJobID, false);
				
				m_sJobName = myTempJob.GetLocalizedName();
	
				// get the folder that the job was created in's localized name
				IPTAdminFolder myTempAdminFolder = (IPTAdminFolder) m_ptSession.GetAdminCatalog().
					OpenAdminFolder(PT_INTRINSICS.PT_ADMIN_FOLDER_INTRINSICOPS, false);
				m_sFolderJobCreatedInName = myTempAdminFolder.GetLocalizedName();
			}
			catch (Exception ex)
			{
				log.Error(m_asOwner.GetString(1701, ResourceConstants.PORTALADMINMSGS) + 
					PT_CLASSIDS.PT_ADMIN_FOLDER_ID);
			}			
		}
	}
	
	public int GetJobID()
	{
		return m_nJobID;
	}
	
	public String GetJobName()
	{
		return m_sJobName;
	}
	
	public String GetJobFolderName()
	{
		return m_sFolderJobCreatedInName;
	}

	/**
	 *
	 *
	 * @param _nClassID
	 * @param _nObjectID
	 *
	 * @return int
	 */
	public int GetACLIndex(int _nClassID, int _nObjectID)
	{
		int nIndex = -1;

		for (int i = 0; i < m_saData.GetCount(); i++)
		{
			if ((_nClassID == XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID))) &&
				(_nObjectID == XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_OBJECTID))))
			{
				nIndex = i;

				break;
			}
		}

		return nIndex;
	}
	
	/**
	 * Returns a flag to know whether any subscriptions have been made.
	 * 
	 * @return boolean
	 */
	public boolean HaveSubscriptionsBeenMade()
	{
		return m_bHaveSubscriptionsBeenMade;
	}
	
	/**
	 * Tells whether the row identified by the class ID, the object ID
	 * and the Access Level ID in argument is in the Object ACL or no.
	 *
	 * @param _nClassID
	 * @param _nObjectID
	 * @param _nAccessLevel
	 * @param _ptACL
	 *
	 * @return boolean
	 */
	protected boolean IsMatrixRowInObjectACL(int _nClassID, int _nObjectID,
		int _nAccessLevel, IPTAccessList _ptACL)
	{
		IPTAccessListEntry ptACLEntry;
			
		for (int i = 0; i < _ptACL.GetCount(); i++)
		{
			ptACLEntry = _ptACL.GetItem(i);

			if ((_nClassID == ptACLEntry.GetClassID()) &&
					(_nObjectID == ptACLEntry.GetObjectID()) &&
					(_nAccessLevel == ptACLEntry.GetAccessLevel()))
			{
				return true;
			}
		}

		return false;
	}
	
	/**
	 * 
	 * @return the current IPTAccessList of the folder whose security is being edited
	 */
	public IPTAccessList GetCurrentFolderACL()
	{
		// can only edit one admin folder at a time so the first
		// element of the object array will be the folder we want 
		IPTAdminFolder ptFolder = (IPTAdminFolder)((MultiObjectsSubscribeModel) m_asOwner.
			GetModel(SubscribeMatrixEditorAS.STR_MVC_STORAGE_NAME_MULTI_SUBSCRIBE)).
			GetObjectArray().GetElement(0);

		IPTAdminFolder ptNewFolder = m_ptSession.GetAdminCatalog()
			.OpenAdminFolder(((IPTServerContext)ptFolder.GetInterfaces("IPTServerContext")).GetObjectID(), false);

		// lock the folder for edit								
		IPTServerContext ptSC = (IPTServerContext) ptNewFolder.GetInterfaces("IPTServerContext");
		
		return ptSC.GetACL();
	}

	/**
	 * Returns the array of Object IDs.
	 *
	 * @return int[]
	 */	
	public int[] GetObjectIDs()
	{	
		return m_arrObjectIDs;
	}

	/**
	 * Returns the array of column info.
	 *
	 * @return int[]
	 */
	public int[] GetColumns()
	{
		return m_arrColumns;
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the array of Objects.
	 *
	 * @return XPArrayList
	 */
	public XPArrayList GetObjectArray()
	{
		return m_arrObjects;
	}

	/**
	 * Initializes the expandable list.
	 *
	 * @param _ptObject Object being created or edited
	 * @param _ptSession User session
	 *
	 * @return boolean Flag to know if the initialization was successful or no.
	 */
	public boolean InitializeMultiObjsSubscribePage(XPHashtable _htFormData,
		IPTSession _ptSession)
	{
		m_ptSession = _ptSession;
		m_xpalSubscriptions = new XPArrayList();
		m_bHaveSubscriptionsBeenMade = false;
		m_nJobID = 0;
		m_sJobName = "";
		m_sFolderJobCreatedInName = "";
		
		DisableAddButton();
		DisableCheckBoxes();
		
		String[] data;

		if (!_htFormData.ContainsKey(QS_MULTI_OBJ_SUBSCRIBE_IDS))
		{
			
				log.Error("The objectAndClassIDs parameter was not in the Query String.");
			

			throw new XPException(
				"MultiObjectsSubscribeModel::InitializeMultiObjsSecurityPage Unable to start the Subscribe matrix.");
		}

		data = (String[]) _htFormData.GetElement(QS_MULTI_OBJ_SUBSCRIBE_IDS);

		if ((null == data) || (0 == data.length) || (null == data[0]))
		{
			
				log.Error("The objectAndClassIDs parameter was null or empty.");
			

			throw new XPException(
				"MultiObjectsSubscribeModel::InitializeMultiObjsSecurityPage Unable to start the Subscribe matrix.");
		}

		m_arrObjects = new XPArrayList();

		String sIDs   = data[0];
		int nClassID;
		int nObjectID;
		
		boolean bNonFailureErrorSet = false;

		while (0 != sIDs.length())
		{
			// JF- Get class ID
			nClassID     = XPConvert.ToInteger(sIDs.substring(0, sIDs.indexOf(",")));
			sIDs		 = sIDs.substring(sIDs.indexOf(",") + 1);

			if (-1 != sIDs.indexOf(","))
			{
				// JF- Get object ID
				nObjectID     = XPConvert.ToInteger(sIDs.substring(0,
							sIDs.indexOf(",")));
				sIDs		  = sIDs.substring(sIDs.indexOf(",") + 1);
			}
			else
			{
				// JF- Get object ID
				nObjectID     = XPConvert.ToInteger(sIDs);
				sIDs		  = "";
			}

			// JF- Open the Object and add it to the array
			if (PT_CLASSIDS.PT_ADMIN_FOLDER_ID == nClassID)
			{
				IPTAdminFolder ptAdminFolder = m_ptSession.GetAdminCatalog().OpenAdminFolder(nObjectID, false);
				
				if (PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER == ptAdminFolder.GetFolderType())
				{
					IPTQueryResult qrComm = m_ptSession.GetCommunities().Query(PT_PROPIDS.PT_PROPID_OBJECTID, nObjectID, PT_PROPIDS.PT_PROPID_OBJECTID, 0, 1, null);
					m_arrObjects.Add(m_ptSession.GetCommunities().Open(qrComm.ItemAsInt(0, PT_PROPIDS.PT_PROPID_OBJECTID),false));
					m_bEditAdminFolderSecurity = true;
					
					if (!_ptSession.GetCommunities()
						.IsObjectAccessible(qrComm.ItemAsInt(0, PT_PROPIDS.PT_PROPID_OBJECTID), _ptSession.GetSessionInfo().GetCurrentUserID(),
							PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT))
					{
						//Raise the error that the matrix could not be started
						UIError myError = new UIError(UIErrorLevel.SERIOUS,
							UIErrorScope.CURRENT_USER, null, 
							m_asOwner.GetString(514, "ptmsgs_portalinfrastructure"), null);
						UIErrorHandler.RaiseError(myError,
							m_asOwner.GetSubSession(), m_asOwner.GetApplication());
								
						return false;			
					}
				}
				else
				{
					log.Warn("InitializeMultiObjsSubscribePage Ignoring Folder with ID: " +
						nObjectID + ". (it is not a Community Folder).");	
				}
			}
			else if (PT_CLASSIDS.PT_COMMUNITY_ID == nClassID || PT_CLASSIDS.PT_GADGET_ID == nClassID)
			{
				try
				{
					IPTObject ptObject = (IPTObject)m_ptSession.GetObjectManagers(nClassID).Open(nObjectID, false);
					
					if ( PT_CLASSIDS.PT_GADGET_ID == nClassID )
					{
						if ( PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_NARROW == ((IPTGadget)ptObject).GetAlignment() ||
							PT_GADGET_ALIGNMENTS.PT_GADGETALIGN_WIDE == ((IPTGadget)ptObject).GetAlignment() )
						{
							m_arrObjects.Add(ptObject);	
						}
						else
						{
							//Raise the error that the portlet was a footer, header, or content canvas portlet.
							UIError myError = new UIError(UIErrorLevel.SERIOUS,
								UIErrorScope.CURRENT_USER, null, 
								XPFormat.Format(m_asOwner.GetString(542, "ptmsgs_portalinfrastructure"), new Object[]{ptObject.GetName()}), null);
							UIErrorHandler.RaiseError(myError,
								m_asOwner.GetSubSession(), m_asOwner.GetApplication());
								
							bNonFailureErrorSet = true;
						}
					}
					else
					{
						m_arrObjects.Add(ptObject);	
					}
						
					if (!_ptSession.GetObjectManagers(nClassID)
						.IsObjectAccessible(nObjectID, _ptSession.GetSessionInfo().GetCurrentUserID(),
							PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT))
					{
						//Raise the error that the matrix could not be started
						UIError myError = new UIError(UIErrorLevel.SERIOUS,
							UIErrorScope.CURRENT_USER, null, 
							m_asOwner.GetString(515, "ptmsgs_portalinfrastructure"), null);
						UIErrorHandler.RaiseError(myError,
							m_asOwner.GetSubSession(), m_asOwner.GetApplication());
							
						return false;			
					}
				}
				catch (Exception e)
				{
					log.Warn("InitializeMultiObjsSecurityPage Unable to open object with class ID: " +
						nClassID + " and object ID: " + nObjectID +
						". The following exception was thrown: " + e);
				}
				
				
			}
			else
			{
				log.Warn("InitializeMultiObjsSecurityPage Ignoring object with class ID: " +
					nClassID + " and object ID: " +
					nObjectID + ". (Can only subscribe to Communities and Portlets).");
			}
		}

		if (0 < m_arrObjects.GetSize())
		{
			m_arrObjectClassIDs = new int[]{ PT_CLASSIDS.PT_USERGROUP_ID };
			
			// JF- Set the icon to display on each row.
			SetRowIconByClassID(PT_CLASSIDS.PT_USERGROUP_ID,
				PTImageType.ICON_GROUP);

			super.InitializeExpandableList(m_asOwner.GetString(518,
					"ptmsgs_portalinfrastructure"), CreateColumnInfos(),
				PTImageType.ICON_GROUP,
				"", 11);

			for (int i = 0; i < m_arrObjects.GetSize(); i++)
			{
				IPTObject ptObject = (IPTObject) m_arrObjects.GetElement(i);
				InitListWithACLFromObject(ptObject);
			}
		}
		else if (!bNonFailureErrorSet)
		{
			String sMessage =  m_asOwner.GetString(513, "ptmsgs_portalinfrastructure");
			
			//Raise the error that the matrix could not be started
			UIError myError = new UIError(UIErrorLevel.SERIOUS,
				UIErrorScope.CURRENT_USER, null, sMessage, null);
			UIErrorHandler.RaiseError(myError,
				m_asOwner.GetSubSession(), m_asOwner.GetApplication());
						
			throw new XPException(
				"MultiObjectsSubscribeModel::InitializeMultiObjsSubscribePage Array of Objects is empty.");
		}

		return true;
	}

	/**
	 * Creates the array of clolumn info.
	 *
	 * @return Object[][]
	 */
	protected Object[][] CreateColumnInfos()
	{
		Object[][] arrExpListColInfo;

		// JF- Column mapping
		m_arrColumns	    = new int[EXPLIST_COL_MAX +
			(3 * m_arrObjects.GetSize())];
		m_arrColumns[0]     = EXPLIST_SORTEDARRAY_PROPID_INDEX;
		m_arrColumns[1]     = EXPLIST_COL_CLASSID;
		m_arrColumns[2]     = EXPLIST_COL_OBJECTID;
		m_arrColumns[3]     = EXPLIST_COL_STRING;
		m_arrColumns[4]     = EXPLIST_COL_ACCESSLEVEL_ID;
		m_arrColumns[5]     = EXPLIST_COL_ACCESSLEVEL_STRING;
		m_arrColumns[6]     = EXPLIST_COL_ROWACCESS;
		m_arrColumns[7]     = EXPLIST_COL_MANDATORY;
		m_arrColumns[8]     = EXPLIST_COL_GROUPTREEASID;

		// JF- Create the the 2D array to give column information to the super class.
		arrExpListColInfo     = new Object[m_arrObjects.GetSize() + 2][6];

		// First column: object name
		arrExpListColInfo[0][EXPLIST_COLUMN_ID]     = new Integer(EXPLIST_COL_STRING);

		arrExpListColInfo[0][EXPLIST_COLUMN_TITLE]		    = m_asOwner.GetString(523,
				"ptmsgs_portalinfrastructure");
		arrExpListColInfo[0][EXPLIST_COLUMN_WIDTH]		    = "40%";
		arrExpListColInfo[0][EXPLIST_COLUMN_COLSPAN]	    = "1";
		arrExpListColInfo[0][EXPLIST_COLUMN_ENABLESORT]     = Boolean.TRUE;
		arrExpListColInfo[0][EXPLIST_COLUMN_IMG]     		= null;

		int k											    = EXPLIST_COL_MAX;
		
		int nCount = m_arrObjects.GetSize();
		m_arrObjectIDs = new int[nCount];

		for (int i = 0; i < nCount; i++)
		{
			IPTObject ptObject    = (IPTObject) m_arrObjects.GetElement(i);
			IPTServerContext ptsc = (IPTServerContext) ptObject.GetInterfaces(
					"IPTServerContext");
			String sObjectKey     = XPConvert.ToString(ptObject.GetClassID()) +
				XPConvert.ToString(ptsc.GetObjectID());
			m_arrObjectIDs[i] = ptsc.GetObjectID();

			// Rights column
			arrExpListColInfo[i + 1][EXPLIST_COLUMN_ID]			    = new Integer(EXPLIST_COL_ACCESSLEVEL_STRING +
					sObjectKey);
			arrExpListColInfo[i + 1][EXPLIST_COLUMN_TITLE]		    = 
				XPHttpUtility.HtmlEncode(GeneralHelpers.TruncateLongString(ptObject.GetLocalizedName(), ObjEditorModel.OBJECT_NAME_TRUNCATE, false));
			arrExpListColInfo[i + 1][EXPLIST_COLUMN_WIDTH]		    = "15%";
			arrExpListColInfo[i + 1][EXPLIST_COLUMN_COLSPAN]	    = "1";
			arrExpListColInfo[i + 1][EXPLIST_COLUMN_ENABLESORT]     = Boolean.FALSE;
			PTImageType img = null;
			if ( PT_CLASSIDS.PT_COMMUNITY_ID == ptObject.GetClassID() )
			{
				img = PTImageType.ICON_COMMUNITY;
			}
			else if ( PT_CLASSIDS.PT_GADGET_ID == ptObject.GetClassID() )
			{
				img = PTImageType.ICON_PORTLET;
			}
			arrExpListColInfo[i + 1][EXPLIST_COLUMN_IMG]     		= img;

			m_arrColumns[k]		    = XPConvert.ToInteger(EXPLIST_COL_ACCESSLEVEL_ID +
					sObjectKey);
			m_arrColumns[k + 1]     = XPConvert.ToInteger(EXPLIST_COL_ACCESSLEVEL_STRING +
					sObjectKey);
			m_arrColumns[k + 2]     = XPConvert.ToInteger(EXPLIST_COL_SUBSCRIBE +
					sObjectKey);
			k					    = k + 3;
		}
		
		arrExpListColInfo[nCount + 1][EXPLIST_COLUMN_ID] = new Integer(-1);
		arrExpListColInfo[nCount + 1][EXPLIST_COLUMN_TITLE] = m_asOwner.GetString(360,"ptmsgs_portalinfrastructure");
		arrExpListColInfo[nCount + 1][EXPLIST_COLUMN_WIDTH]	= "15%";
		arrExpListColInfo[nCount + 1][EXPLIST_COLUMN_COLSPAN] = "1";
		arrExpListColInfo[nCount + 1][EXPLIST_COLUMN_ENABLESORT] = Boolean.FALSE;

		return arrExpListColInfo;
	}

	/**
	 * Helper method to get the sort.
	 *
	 * @return int[]
	 */
	protected int[] GetArrayOfColumns()
	{
		return m_arrColumns;
	}

	/**
	 * Initializes the list with the ACL from the PTObject.
	 *
	 * @param _ptObject Object being created or edited
	 */
	protected void InitListWithACLFromObject(IPTObject _ptObject)
	{
		IPTServerContext ptscObject   = (IPTServerContext) _ptObject.GetInterfaces(
				"IPTServerContext");
		IPTAccessListEntry ptACLEntry;
		Object[] oNewRow;
		
		m_arrLowAccessObjects = new XPArrayList();
		m_arrNoContentTree = new XPArrayList();
		// JF- Keep track of the items we are hiding in the tree
		XPArrayList arrTreeHiddenItems = new XPArrayList(); 
		
		SetWrapColumnHeaders(false);

		IPTAccessList ptACL = ptscObject.GetACL();

		String sObjectKey					   = XPConvert.ToString(_ptObject.GetClassID()) +
			XPConvert.ToString(ptscObject.GetObjectID());
		int nCurrentObjectColAccessLevelID     = XPConvert.ToInteger(EXPLIST_COL_ACCESSLEVEL_ID +
				sObjectKey);
		int nCurrentObjectColAccessLevelString = XPConvert.ToInteger(EXPLIST_COL_ACCESSLEVEL_STRING +
				sObjectKey);
		int nCurrentObjectColRowAccess		   = XPConvert.ToInteger(EXPLIST_COL_SUBSCRIBE +
				sObjectKey);

		for (int i = 0; i < ptACL.GetCount(); i++)
		{
			ptACLEntry = ptACL.GetItem(i);
			
			int nACLClassID = ptACLEntry.GetClassID();
			
			// only allow bulk subscription of groups
			if (PT_CLASSIDS.PT_USERGROUP_ID == nACLClassID)
			{
				int nACLObjectID = ptACLEntry.GetObjectID(); 
				
				if ( m_ptSession.GetObjectManagers(PT_CLASSIDS.PT_USERGROUP_ID)
						.IsObjectAccessible(nACLObjectID, m_ptSession.GetSessionInfo().GetCurrentUserID(),
							PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT) )
				{
					int nACLIndex = GetACLIndex(nACLClassID,
						nACLObjectID);
		
					if (-1 == nACLIndex)
					{
						oNewRow										  = new Object[m_arrColumns.length];
						oNewRow[EXPLIST_SORTEDARRAY_PROPID_INDEX]     = new Integer(m_nNextIndex);
						m_nNextIndex++;
						oNewRow[EXPLIST_COL_CLASSID]		   = new Integer(nACLClassID);
						oNewRow[EXPLIST_COL_OBJECTID]		   = new Integer(nACLObjectID);
						oNewRow[EXPLIST_COL_STRING]			   = ptACLEntry.GetName();
						oNewRow[EXPLIST_COL_ROWACCESS]		   = new Boolean(ptACLEntry.GetReadOnly());
						oNewRow[EXPLIST_COL_GROUPTREEASID]     = "";
		
						int k								   = EXPLIST_COL_MAX;
		
						while (k < m_arrColumns.length)
						{
							if ((nCurrentObjectColAccessLevelID == m_arrColumns[k]) &&
									(nCurrentObjectColAccessLevelString == m_arrColumns[k +
									1]) &&
									(nCurrentObjectColRowAccess == m_arrColumns[k + 2]))
							{
								oNewRow[k]		   = new Integer(ptACLEntry.GetAccessLevel());
								oNewRow[k + 1]     = GetAccessLevelString(ptACLEntry.GetAccessLevel());
								oNewRow[k + 2]     = "N";
							}
							else
							{
								oNewRow[k]		   = new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_NONE);
								oNewRow[k + 1]     = GetAccessLevelString(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_NONE);
								oNewRow[k + 2]     = "N";
							}
		
							k = k + 3;
						}
		
						super.GetItems().AddRow(oNewRow);
					}
					else
					{
						m_saData.SetItem(nACLIndex, nCurrentObjectColAccessLevelID,
							new Integer(ptACLEntry.GetAccessLevel()));
						m_saData.SetItem(nACLIndex, nCurrentObjectColAccessLevelString,
							GetAccessLevelString(ptACLEntry.GetAccessLevel()));
						m_saData.SetItem(nACLIndex, nCurrentObjectColRowAccess,
							"N");
		
						if (PT_ACCESS_LIST_ENTRY_READONLY == ptACLEntry.GetReadOnly())
						{
							m_saData.SetItem(nACLIndex, EXPLIST_COL_ROWACCESS,
								new Boolean(PT_ACCESS_LIST_ENTRY_READONLY));
						}
					}
				}
			}
		}
	}

	/**
	 * Saves the access level for the row _nRowIndex.
	 *
	 * @param _saData List to update the data on row _nRowIndex
	 * @param _nRowIndex Index of the row to save.
	 * @param _htFormData Data posted by the form.
	 * @return int Status for server side validation.
	 */
	protected int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
		XPHashtable _htFormData)
	{
		// JF- We need to save the privilege drop-downs.
		if (!m_bIsReadOnly)
		{
			int k = EXPLIST_COL_MAX;

			while (k < m_arrColumns.length)
			{
				// JF- We are building the select input name the same way we did 
				// in ObjectSecurityView when we displayed it.
				String sInputSelectName = GetUniqueString(MultiObjectsSubscribeView.HTMLCHECKBOX_SUBSCRIBE) +
					_nRowIndex + k;

				String[] data = (String[]) _htFormData.GetElement(sInputSelectName);

				if ((data != null) && (0 != data.length) &&
						(null != data[0]) && ("1").equals(data[0]))
				{
					_saData.SetItem(_nRowIndex, m_arrColumns[k+2], "Y");
				}

				k = k + 3;
			}
		}

		int nObjectID		  = XPConvert.ToInteger(_saData.GetItem(_nRowIndex,
				EXPLIST_COL_OBJECTID));
		String sGroupTreeASID = (String) _saData.GetItem(_nRowIndex,
				EXPLIST_COL_GROUPTREEASID);

		// JF- We save the Group tree ID only if we don't already have it.
		if (("").equals(sGroupTreeASID) &&
				_htFormData.ContainsKey(GetUniqueString(
						ObjectSecurityView.HTMLINPUT_GROUPTREEASID) +
					nObjectID))
		{
			String[] data = (String[]) _htFormData.GetElement(GetUniqueString(
						ObjectSecurityView.HTMLINPUT_GROUPTREEASID) +
					nObjectID);

			if ((null != data) && !(0 == data.length))
			{
				_saData.SetItem(_nRowIndex, EXPLIST_COL_GROUPTREEASID, data[0]);
			}
		}

		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Saves the security settings into the IPTObject.
	 *
	 */
	protected void StoreSubscriptionsToObject(IPTObject _ptObject, IPTBulkSubscriber _ptSubscriber)
	{
		IPTServerContext ptscObject = (IPTServerContext) _ptObject.GetInterfaces(
				"IPTServerContext");
		int nClassID;
		int nObjectID;
		String strSubscribe;
		XPArrayList xpalGroupIDs = new XPArrayList();

		String sObjectKey = XPConvert.ToString(_ptObject.GetClassID()) +
			XPConvert.ToString(ptscObject.GetObjectID());

		// JF- Then we go through all the users and groups from the list and we add 
		// them to the Object ACL (except the read only ones.)		
		for (int i = 0; i < m_saData.GetCount(); i++)
		{
			strSubscribe   = (String)m_saData.GetItem(i,
								XPConvert.ToInteger(EXPLIST_COL_SUBSCRIBE + sObjectKey));
			nClassID	   = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_CLASSID));
			nObjectID	   = XPConvert.ToInteger(m_saData.GetItem(i, EXPLIST_COL_OBJECTID));

			if (("Y").equals(strSubscribe) && PT_CLASSIDS.PT_USERGROUP_ID == nClassID)
			{
				xpalGroupIDs.Add(new Integer(nObjectID));
			}
		}
		
		if (0 < xpalGroupIDs.GetSize())
		{
			m_bHaveSubscriptionsBeenMade = true;

			int[] anGroupIDs = new int[xpalGroupIDs.GetSize()];
			for (int i = 0; i < xpalGroupIDs.GetSize(); i++)
			{
				anGroupIDs[i] = XPConvert.ToInteger(xpalGroupIDs.GetElement(i));
			}

			if (PT_CLASSIDS.PT_COMMUNITY_ID == _ptObject.GetClassID() || PT_CLASSIDS.PT_GADGET_ID == _ptObject.GetClassID())
			{
				_ptSubscriber.SetBulkSubscription(_ptObject.GetClassID(), ptscObject.GetObjectID(), anGroupIDs);
			}
		}
	}
}

