package com.plumtree.portaluiinfrastructure.tags.data;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;

/**
 * This class is wrapper around XPHashtable that allows it
 * to be used as a data collection in Tag memory.
 */
public class HashtableDataObject implements IDataObject
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		"com.plumtree.portaluiinfrastructure.tags.data.HashtableDataObject");

	/**
	 * The underlying hashtable object.
	 */
	protected final XPHashtable m_hashtable;

	/**
	 * Constructor for the Hashtable wrapper data object that
	 * takes a pre-constructed hashtable.
	 * 
	 * @param _hashtable A pre-constructed hashtable.
	 */
	public HashtableDataObject(XPHashtable _hashtable)
	{
		m_hashtable = _hashtable;
	}

	/**
	 * Constructor for the Hashtable wrapper data object.
	 */
	public HashtableDataObject()
	{
		m_hashtable = new XPHashtable();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.data.IDataObject#GetData(String)
	 */
	public String GetData(String _strKey)
	{
		String strResult;

		try
		{
			strResult = (String) m_hashtable.GetElement(_strKey);
		} catch (Exception e)
		{
			log.Warn("Attempt to access data with key " + _strKey +
				" failed.  No such data exist.");

			strResult = null;
		}

		return strResult;
	}

	/**
	 * Sets a name value pair on the hashtable.
	 * 
	 * @param _strKey The name used to store the value.
	 * @param _strValue The value to be stored.
	 */
	public void SetData(String _strKey, String _strValue)
	{
		m_hashtable.PutElement(_strKey, _strValue);
	}

	/**
	 * Get the underlying hashtable object.
	 * 
	 * @return XPHashtable The underlying hashtable object.
	 */
	public XPHashtable GetHashTable()
	{
		return m_hashtable;
	}
}
