package com.plumtree.portaluiinfrastructure.tree;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.classtypedescriptors.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.common.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.tree.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author Don Hayler
 *
 * This class implements some of the base code for the admin and browsing trees.
 */
public abstract class AObjectTreeModel extends ATreeModel
	implements IObjectTreeModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_UI_INFRASTRUCTURE),
		"com.plumtree.portaluiinfrastructure.tree.AObjectTreeModel");

	/** Community Modes -- NO is default */
	public static final int NOCOMMUNITIES  = 1;
	/** THISCOMMUNITY implies the requested community plus all of it's parent communities */
	public static final int THISCOMMUNITY  = 2;
	public static final int ALLCOMMUNITIES = 3;

	/** class type descriptor */
	protected static ClassTypeDescManager m_ptClassTypeDescManager = null;

	/** access level query filter */
	protected Object[][] m_ptqf = null;

	/**
	 * read only access level query filter.
	 * This is used to query for browsable but not selectable admin folders.
	 * Not used if null.
	 */
	protected Object[][] m_ptFolderSelectqf = null;

	/** internal data */
	protected boolean m_bDisplayingSearchResults = false;
	protected boolean m_bSearchInitialized		 = false;
	protected int m_nAccessLevel;

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetSecurityLevel()
	 */
	public int GetAccessLevel()
	{
		return m_nAccessLevel;
	}

	///////////////////////////////////////
	// Read Methods from IObjectTreeModelRO
	///////////////////////////////////////

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetDisplayTreeOrSearch()
	 */
	public boolean GetDisplayingSearchResults()
	{
		return m_bDisplayingSearchResults;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetPortletAlignment()
	 */
	public int[] GetPortletAlignment()
	{
		return new int[0];
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetPortletWebServiceID()
	 */
	public int[] GetPortletWebServiceID()
	{
		return new int[0];
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetPortletType()
	 */
	public int[] GetPortletType()
	{
		return new int[0];
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetUserTypeFilter()
	 */
	public int[] GetUserTypeFilter()
	{
		return new int[0];
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetFilterCLSID()
	 */
	public String[] GetFilterCLSID()
	{
		return new String[0];
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetFilterCLSIDPropID()
	 */
	public int GetFilterCLSIDPropID()
	{
		return 0;
	}

	/**
	 * This method returns the class ID to be filtered.
	 *
	 * @return the class ID to apply the above filter to
	 */
	public int GetFilterClassID()
	{
		return 0;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetItemSelected(int, int)
	 */
	public boolean GetItemSelected(int nClassID, int nObjectID)
	{
		TreeLeafNode item = (TreeLeafNode) m_xpmSelectedItems.GetElement(nClassID +
				"," + nObjectID);

		if (item != null)
		{
			if (item.m_bSelected != true)
			{
				
					log.Error("Broken invariant: item in selected item list is not selected (Class ID: " +
						nClassID + ", Object ID: " + nObjectID + ").");
				
			}

			return true;
		}

		return false;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetSearchStarted()
	 */
	public boolean GetSearchStarted()
	{
		return m_bSearchInitialized;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetSearchEnabled()
	 */
	public boolean GetSearchEnabled()
	{
		return true;
	}

	/////////////////
	// IModel methods
	/////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		m_ptClassTypeDescManager = PTConfigHelper.GetPTClassTypeDescrManager();
		super.Init(parent);
	}

	/**
	 * 
	 */
	public void ResetTree()
	{
		super.ResetTree();
		m_bDisplayingSearchResults     = false;
		m_bSearchInitialized		   = false;
		m_nAccessLevel			   	   = PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT;
		m_ptqf						   = PlumtreeHelpers.GetQueryFilter(PT_PROPIDS.PT_PROPID_ACCESSLEVEL,
			PT_FILTEROPS.PT_FILTEROP_GTE, new Integer(m_nAccessLevel));
		m_ptFolderSelectqf			   = CalculateFolderSelectQueryFilter(m_nAccessLevel);
		m_strOnClickFunction		   = "parent." + FrameSetDP.BOTTOM_FRAME + "." +
										TreeSelectView.JAVASCRIPT_SELECT;
	}

	//////////////////////////////////
	// Write Methods from IRepostModel
	//////////////////////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String, XPHashtable)
	 *
	 * @return int Page status for server side validation
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData)
	{
		String[] sValues;

		sValues = (String[]) _htFormData.GetElement(TreeWithSearchDP.HTMLINPUT_DISPLAYING_SEARCH);

		if ((sValues != null) && (sValues[0] != null))
		{
			boolean bSearch = m_bDisplayingSearchResults;

			try
			{
				bSearch = XPConvert.ToBoolean(sValues[0]);
			}
			catch (Exception e)
			{
				
					log.Error("Unable to convert view type to boolean: " +
						sValues[0]);
				
			}

			if (bSearch != m_bDisplayingSearchResults)
			{
				// Need to switch the view and refresh the header
				m_bDisplayingSearchResults     = bSearch;

				// If we're switching views (and we started in tree), then
				// search must already be initialized
				m_bSearchInitialized = true;
			}

			// Get pre-selected nodes (search only, not tree)
			String[] strTemp = (String[]) _htFormData.GetElement(TreeSearchView.HTMLHIDDEN_SELECTEDIDS);

			if (strTemp != null)
			{
				// Search was reposted, handle it
				String strSelectedInputName = null;

				if (_htFormData.GetElement(TreeView.HTMLBUTTON_RADIO) != null)
				{
					strSelectedInputName = TreeView.HTMLBUTTON_RADIO;
				}
				else if (_htFormData.GetElement(TreeView.HTMLBUTTON_CHECKBOX) != null)
				{
					strSelectedInputName = TreeView.HTMLBUTTON_CHECKBOX;
				}

				XPHashtable xpmTempSelected = new XPHashtable();

				int[] nPreSelectedIndexes	   = GeneralHelpers.ConvertToInts(strTemp[0]);
				String[] strSelectedItemsIndex = (String[]) _htFormData.GetElement(strSelectedInputName);

				int nSearchIndex;
				int nClassID;
				int nObjectID;
				SearchResultModel searchModel = (SearchResultModel) m_asOwner.GetModel(SearchResultModel.STR_MVC_CLASS_NAME);
				TreeLeafNode selectedNode;

				if (strSelectedItemsIndex != null)
				{
					// Find and set the new selected element
					for (int i = 0; i < strSelectedItemsIndex.length; i++)
					{
						try
						{
							try
							{
								nSearchIndex = XPConvert.ToInteger(strSelectedItemsIndex[i]);
							}
							catch (Exception e)
							{
								throw XPException.GetInstance(e);
							}
						}
						catch (XPFormatException fe)
						{
							
								log.Error("Save Page: ID of selected item in invalid format: " +
									strSelectedItemsIndex[i]);
								log.Error(fe, "Format error.");
							

							nSearchIndex = -1;
						}

						if (nSearchIndex != -1)
						{
							nClassID	  = searchModel.GetResultClassID(nSearchIndex);
							nObjectID     = searchModel.GetResultObjectID(nSearchIndex);

							selectedNode = (TreeLeafNode) m_xpmSelectedItems.GetElement(nClassID +
									"," + nObjectID);

							if (selectedNode == null)
							{
								selectedNode = m_Root.GetNode(nClassID,
										nObjectID);

								if (selectedNode == null)
								{
									if ((nClassID == PT_CLASSIDS.PT_ADMIN_FOLDER_ID) ||
											(nClassID == PT_CLASSIDS.PT_USERGROUP_ID))
									{
										selectedNode = new TreeCompositeNode(-1);
									}
									else
									{
										selectedNode = new TreeLeafNode(-1);
									}

									selectedNode.m_nClassID		 = nClassID;
									selectedNode.m_nObjectID     = nObjectID;
									selectedNode.SetName(searchModel.GetResultName(nSearchIndex));
								}

								selectedNode.m_bSelected = true;
								m_xpmSelectedItems.PutElement(nClassID + "," +
									nObjectID, selectedNode);
							}

							xpmTempSelected.PutElement(nClassID + "," +
								nObjectID, selectedNode);
						}
					}
				}

				TreeLeafNode curNode;

				// Find items that have been un-selected and remove them from the list.
				for (int i = 0; i < nPreSelectedIndexes.length; i++)
				{
					nClassID	  = searchModel.GetResultClassID(nPreSelectedIndexes[i]);
					nObjectID     = searchModel.GetResultObjectID(nPreSelectedIndexes[i]);

					if (!xpmTempSelected.ContainsKey(nClassID + "," +
								nObjectID))
					{
						// Item was un-selected, remove it
						curNode = (TreeLeafNode) m_xpmSelectedItems.RemoveElement(nClassID +
								"," + nObjectID);

						if (curNode != null)
						{
							// make sure we unselect it so it doesn't show up selected in the tree
							curNode.m_bSelected = false;
						}
					}
				}

				IXPEnumerator temp = xpmTempSelected.GetElements();

				// Clear the selected list if we have a selection in single select mode
				if ((!m_bMultiSelect) && (xpmTempSelected.GetSize() > 0))
				{
					m_xpmSelectedItems.Clear();

					// Add the single selected item
					temp.MoveNext();
					curNode = (TreeLeafNode) temp.GetCurrent();
					m_xpmSelectedItems.PutElement(curNode.m_nClassID + "," +
						curNode.m_nObjectID, curNode);
				}
				else
				{
					// Add all selected items to the list
					while (temp.MoveNext())
					{
						curNode = (TreeLeafNode) temp.GetCurrent();
						m_xpmSelectedItems.PutElement(curNode.m_nClassID + "," +
							curNode.m_nObjectID, curNode);
					}
				}
			}
		}
		else
		{
			
				log.Error("Unable to find view type in Query String.");
			
		}

		return super.SavePage(_sPageName, _htFormData);
	}

	//////////////////////////////////////
	// Write Methods from IObjectTreeModel
	//////////////////////////////////////

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SetAccessLevel(int)
	 */
	public void SetAccessLevel(int nLevel)
	{
		if ((nLevel != PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_ADMIN) &&
				(nLevel != PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_EDIT) &&
				(nLevel != PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_SELECT) &&
				(nLevel != PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ))
		{
			
				log.Error("SetAccessLevel called with invalid access level (should be admin, edit, select, or read): " +
					nLevel);
			

			return;
		}

		m_nAccessLevel		= nLevel;
		m_ptqf				= PlumtreeHelpers.GetQueryFilter(PT_PROPIDS.PT_PROPID_ACCESSLEVEL,
			PT_FILTEROPS.PT_FILTEROP_GTE, new Integer(m_nAccessLevel));
		m_ptFolderSelectqf	= CalculateFolderSelectQueryFilter(m_nAccessLevel);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SelectNode(int, boolean)
	 */
	public void SelectNode(int nID, boolean bSelect)
	{
		TreeLeafNode selectedNode = null;

		if (m_bDisplayingSearchResults)
		{
			SearchResultModel searchModel = (SearchResultModel) m_asOwner.GetModel(SearchResultModel.STR_MVC_CLASS_NAME);

			if (nID != -1)
			{
				int nClassID	  = searchModel.GetResultClassID(nID);
				int nObjectID     = searchModel.GetResultObjectID(nID);

				selectedNode = (TreeLeafNode) m_xpmSelectedItems.GetElement(nClassID +
						"," + nObjectID);

				if (selectedNode == null)
				{
					selectedNode = m_Root.GetNode(nClassID,
							nObjectID);
	
					// Only create a new node if we are selecting it.
					// if we are deselecting, it should already exist.
					if ((selectedNode == null) && (bSelect))
					{
						if ((nClassID == PT_CLASSIDS.PT_ADMIN_FOLDER_ID) ||
								(nClassID == PT_CLASSIDS.PT_USERGROUP_ID))
						{
							selectedNode = new TreeCompositeNode(-1);
						}
						else
						{
							selectedNode = new TreeLeafNode(-1);
						}
	
						selectedNode.m_nClassID		 = nClassID;
						selectedNode.m_nObjectID     = nObjectID;
						selectedNode.SetName(searchModel.GetResultName(nID));
					}
				}
			}
		} else
		{
			selectedNode = m_Root.GetNode(nID);
		}

		if (selectedNode == null)
		{
			
				log.Error("Unable to get tree node to de/select.  Ignoring");
			

			return;
		}

		selectedNode.m_bSelected = bSelect;

		if (!m_bMultiSelect)
		{
			// Clear the previously selected items before removing them
			// from the list.
			IXPEnumerator temp = m_xpmSelectedItems.GetElements();
	
			while (temp.MoveNext())
			{
				TreeLeafNode curNode = (TreeLeafNode) temp.GetCurrent();
				curNode.m_bSelected = false;
			}

			m_xpmSelectedItems.Clear();
		}

		if (bSelect)
		{
			m_xpmSelectedItems.PutElement(selectedNode.m_nClassID + "," + selectedNode.m_nObjectID,
				selectedNode);
		} else
		{
			m_xpmSelectedItems.RemoveElement(selectedNode.m_nClassID + "," + selectedNode.m_nObjectID);
		}
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#ResetToBrowsingMode()
	 */
	public void ResetToBrowsingMode()
	{
		m_bDisplayingSearchResults = false;
	}

	/**
	 * 
	 *
	 * @param node 
	 */
	public static void SetCompositeImage(TreeCompositeNode node, AActivitySpace asOwner)
	{
		// folders are taken care of by default, so ignore them for now
		if ((node.m_nClassID != PT_CLASSIDS.PT_ADMIN_FOLDER_ID) &&
				(node.m_nClassID != PT_CLASSIDS.PT_ADMIN_CATALOG_ID))
		{
			if (m_ptClassTypeDescManager != null)
			{
				// Are there open images for things other than folders?
				String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE, asOwner);
				node.m_OpenImage = m_ptClassTypeDescManager.GetDefaultIconForClassType(node.m_nClassID, strLangID);
				if (node.m_OpenImage != null)
				{
					node.m_OpenImage.SetAlt(node.GetEncodedName());
				} else
				{
					
						log.Error("Unable to get open image for composite, can't set alt string on node " + node.GetEncodedName() +
							" with class ID " + node.m_nClassID);
					
				}
			}
		}

		SetLeafImage(node, asOwner);
	}

	/**
	 * 
	 *
	 * @param node 
	 * @param nFolderType 
	 */
	public static void SetCompositeImage(TreeCompositeNode node, int nFolderType, AActivitySpace asOwner)
	{
		if ((node.m_nClassID == PT_CLASSIDS.PT_ADMIN_FOLDER_ID) &&
				(nFolderType != PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_REGULARFOLDER))
		{
			PTConstruct myOpenConstruct;
			PTConstruct myClosedConstruct;

			if (nFolderType == PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_COMMUNITYFOLDER)
			{
				myOpenConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_FOLDER_COMMUNITY_OPEN);
				myClosedConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_FOLDER_COMMUNITY);
			}
			else {
				// Everything else is a sub-portal.  The type will be the ID of the sub-portal object,
				// therefore we can't just check against PT_ADMIN_FOLDER_TYPES.PT_ADMIN_FOLDER_TYPE_DEFAULTSUBPORTALFOLDER.
				// (we already checked for regular folders above)
				myOpenConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_FOLDER_SUBPORTAL_OPEN);
				myClosedConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_FOLDER_SUBPORTAL);
			}

			node.m_OpenImage = (HTMLImg) myOpenConstruct.GetHTMLElement();
			if (node.m_OpenImage != null)
			{
				node.m_OpenImage.SetAlt(node.GetEncodedName());
			} else
			{
				
					log.Error("Unable to get open image for composite, can't set alt string on node " + node.GetEncodedName() +
						" with class ID " + node.m_nClassID);
				
			}

			node.m_Image = (HTMLImg) myClosedConstruct.GetHTMLElement();
			if (node.m_Image != null)
			{
				node.m_Image.SetAlt(node.GetEncodedName());
			} else
			{
				
					log.Error("Unable to get image for composite, can't set alt string on node " + node.GetEncodedName() +
						" with class ID " + node.m_nClassID);
				
			}
		}
		else
		{
			// Pass non-folders and regular folders through to the normal method
			SetCompositeImage(node, asOwner);
		}
	}

	/**
	 * 
	 *
	 * @param node 
	 */
	public static void SetLeafImage(TreeLeafNode node, AActivitySpace asOwner)
	{
		if (m_ptClassTypeDescManager != null)
		{
			String strLangID = (String) PTPersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingNames.LANGUAGE, asOwner);
			node.m_Image = m_ptClassTypeDescManager.GetDefaultIconForClassType(node.m_nClassID, strLangID);
			if (node.m_Image != null)
			{
				node.m_Image.SetAlt(node.GetEncodedName());
			} else
			{
				
					log.Error("Unable to get image for leaf, can't set alt string on node " + node.GetEncodedName() +
						" with class ID " + node.m_nClassID);
				
			}
		}
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetDisplayedClassIDs()
	 */
	abstract public int[] GetDisplayedClassIDs();

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetFolderClassID()
	 */
	abstract public int GetFolderClassID();

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModelRO#GetItemDisplayed(int, int)
	 */
	abstract public boolean GetItemDisplayed(int nClassID, int nObjectID);

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	abstract public String GetName();

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetClassIDsToDisplay(int[])
	 */
	abstract public void SetClassIDsToDisplay(int[] nIDs);

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetObjectIDsToHide(int[])
	 */
	abstract public void SetObjectIDsToHide(int[] nIDs);

	/**
	 * @see com.plumtree.uiinfrastructure.tree.ITreeModel#SetObjectIDsToSelect(int[])
	 */
	abstract public void SetObjectIDsToSelect(int[] nIDs);

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SetQueryFilter(int, int, int[])
	 */
	abstract public void SetQueryFilter(int nClassID, int nPropID,
		int[] nValues);

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SetQueryFilter(int, int, String[])
	 */
	abstract public void SetQueryFilter(int nClassID, int nPropID,
		String[] sValues);

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SetCommunityMode(int, int)
	 */
	abstract public void SetCommunityMode(int nMode, int nFolderID);

	///////////////////
	// Abstract methods
	///////////////////

	/**
	 * @see com.plumtree.portaluiinfrastructure.tree.IObjectTreeModel#SetRootFolderID(int)
	 */
	abstract public void SetRootFolderID(int nID);

	/**
	 * This method calculates the read only query filter.
	 * 
	 * @param int the access level required by the tree
	 * @return the qf.
	 */
	abstract protected Object[][] CalculateFolderSelectQueryFilter(int nAccessLevel);

	/////////////////
	// Helper methods
	/////////////////

	/**
	 * This needs it's own implementation because of users.
	 */
	protected TreeLeafNode GetLeafIfAlreadySelected(String strKey)
	{
		TreeLeafNode myLeaf = (TreeLeafNode) m_xpmSelectedItems.GetElement(strKey);

		if (myLeaf == null)
		{
			return null;
		}

		if ((myLeaf.m_nTreeNodeID != -1) &&
				(myLeaf.m_nClassID != PT_CLASSIDS.PT_USER_ID))
		{
			// Users are an exception because they can exist in multiple groups.
			
				log.Warn("attempting to create a new tree node that is already selected.  Key " +
					strKey);
			
		}
		else
		{
			// Should this copy everything?  expanded, etc...?
			TreeLeafNode temp = new TreeLeafNode(GetNextUniqueID());
			temp.SetNameAndDescription(myLeaf);
			temp.m_nObjectID	   = myLeaf.m_nObjectID;
			temp.m_nClassID		   = myLeaf.m_nClassID;
			temp.m_strLocation     = myLeaf.m_strLocation;
			temp.m_bSelected	   = myLeaf.m_bSelected;

			return temp;
		}

		return null;
	}
}

