package com.plumtree.portaluiinfrastructure.mock.search;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Mock version of the search result model, for use in testing the search results
 * page layout tags.
 * 
 * @author smahendr
 */
public class MockSearchResultModel implements ISearchResultModelRO, ISearchResultModel  
{
	private int numDisplayableItems = 0;
	private int firstDisplayableItem = 0;

	private String currentKey = null;
	private String searchString = null;
	private boolean bAdvancedSearch = false;
	private boolean bIncludeSubfolders = true;
	private XPHashtable hashReconstructionArgs = null;
	private int[] arrPortletRestrictions = null;
	private SearchSettingCollection removableSettings = null;
	private int nTotalItems = 0;
	private int nDrillDownDepth = 0;
	
	private XPArrayList resultsList;
	
	/********* Methods used to set up mock data **********/
	
	public void SetNumDisplayableItems(int numDisplayableItems) {
		this.numDisplayableItems = numDisplayableItems;
	}

	public void SetFirstDisplayableItem(int firstDisplayableItem) {
		this.firstDisplayableItem = firstDisplayableItem;
	}

	public void SetResultsList(XPArrayList resultsList) {
		this.resultsList = resultsList;
	}

	public void SetCurrentKey (String currentKey) {
		this.currentKey = currentKey;
	}

	public void SetSearchString (String searchString) {
		this.searchString = searchString;
	}
	
	public void SetAdvancedSearch (boolean bAdvancedSearch) {
		this.bAdvancedSearch = bAdvancedSearch;
	}

	public void SetIncludeSubfolders (boolean bIncludeSubfolders) {
		this.bIncludeSubfolders = bIncludeSubfolders;
	}
	
	public void SetReconstructionArgs(XPHashtable hashReconstructionArgs) {
		this.hashReconstructionArgs = hashReconstructionArgs;
	}
	
	public void SetRemovableSettings(SearchSettingCollection removableSettings) throws SearchModelException {
		this.removableSettings = removableSettings;
	}

	public void SetPortletRestrictions(int[] arrPortletRestrictions)
	{
		this.arrPortletRestrictions = arrPortletRestrictions;
	}
	
	public void SetTotalItems(int nTotalItems)
	{
		this.nTotalItems = nTotalItems;
	}

	/********************************************************************
	 * Methods used to retrieve Mock data.  These should match the methods
	 * called in the data tags.
	 ********************************************************************/

	/**
	 * Return the mock number of displayable items set.
	 */
	public int GetNumDisplayableItems()
	{
		return numDisplayableItems;
	}
	
	public int GetFirstDisplayableItem()
	{
		return firstDisplayableItem;
	}

	public String GetCurrentKey() throws SearchModelException {
		return currentKey;
	}

	public String GetSearchString() {
		return searchString;
	}
	
	public boolean isAdvancedSearch() {
		return bAdvancedSearch;
	}

	public boolean GetIncludeSubfolders() {
		return bIncludeSubfolders;
	}
	
	public XPHashtable GetReconstructionArgs() throws SearchModelException {
	    return GetReconstructionArgs(true);
	}

	public XPHashtable GetReconstructionArgs(boolean pageContext)
			throws SearchModelException {
		return GetReconstructionArgs(SearchResultControl.RECONSTRUCTION_ARGS_PREFIX, pageContext);
	}

	public XPHashtable GetReconstructionArgs(String prefix,
			boolean wantPageContext) throws SearchModelException
	{
		return hashReconstructionArgs;
	}

	public SearchSettingCollection GetRemovableSettings() throws SearchModelException {
		return removableSettings;
	}

	public int[] GetPortletRestrictions() {
		return arrPortletRestrictions;
	}
	
	public int GetTotalItems() {
		return nTotalItems;
	}

	/********* Methods used to retrieve mock result data **********/
	
	public String GetResultName(int rank) {
		return ((MockSearchResult)resultsList.GetElement(rank)).getName();
	}
	
	public HTMLAnchor GetResultClickthroughURL(int rank, boolean allowPopup) throws SearchModelException
	{
		return GetResultClickthroughURL(rank, allowPopup, false, null);
	}
	
	public HTMLAnchor GetResultClickthroughURL(int rank, boolean allowPopup,
		boolean editMode) throws SearchModelException
	{
		return GetResultClickthroughURL(rank, allowPopup, editMode, null);
	}
	
	public HTMLAnchor GetResultClickthroughURL(int rank, boolean allowPopup,
		boolean editMode, String extraParameters) throws SearchModelException
	{
		return ((MockSearchResult)resultsList.GetElement(rank)).getClickthroughURL();
	}

	public HTMLAnchor GetResultPropertiesURL(int rank, boolean allowPopup) throws SearchModelException
	{
		return ((MockSearchResult)resultsList.GetElement(rank)).getPropertiesURL();
	}

	public boolean isResultBestBet(int rank) {
		return ((MockSearchResult)resultsList.GetElement(rank)).isBestBet();
	}

	public String GetResultApp(int rank) {
		return ((MockSearchResult)resultsList.GetElement(rank)).getApp();
	}

	public int GetResultClassID(int rank) {
		return ((MockSearchResult)resultsList.GetElement(rank)).getClassID();
	}

	public int GetResultObjectID(int rank) {
		return ((MockSearchResult)resultsList.GetElement(rank)).getObjectID();
	}

	public int GetResultFolderID(int rank) {
		return ((MockSearchResult)resultsList.GetElement(rank)).getFolderID();
	}

	public Object GetResultIcon(int rank) {
		return ((MockSearchResult)resultsList.GetElement(rank)).getIcon();
	}

	public XPDateTime GetResultLastModified(int rank) {
		return ((MockSearchResult)resultsList.GetElement(rank)).getLastModified();
	}
	
	public String GetKWIC(int rank) {
		return ((MockSearchResult)resultsList.GetElement(rank)).getKwic();
	}

	public String GetResultContentLastPubUser(int rank) throws SearchModelException {
		return ((MockSearchResult)resultsList.GetElement(rank)).getContentLastPubUser();
	}

	public int[] GetResultContentPortletIDs(int rank) throws SearchModelException {
		return ((MockSearchResult)resultsList.GetElement(rank)).getContentPortletIDs();
	}

	public XPDateFormat GetDateFormat(AActivitySpace aspace)
	{
	  XPLocale xpLocale = null;
	  XPTimeZone xpTimeZone = null;
	  xpLocale = XPLocale.GetDefault();
	  xpTimeZone = XPTimeZone.GetCurrentTimeZone();
	  return XPDateFormat.GetDateTimeInstance(XPDateFormat.SHORT, XPDateFormat.SHORT, xpLocale, xpTimeZone);
	}

	/************* Unimplemented methods **********************/
	
	public String GetBreadcrumbKey(int step) throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetBreadcrumbLabel(int step) throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public int GetCorrectionFrequency(int misspellingIndex, int correctionIndex)
			throws SearchModelException {
		// TODO Auto-generated method stub
		return 0;
	}

	public int GetDrilldownDepth() throws SearchModelException {
		return nDrillDownDepth;
	}

	public void SetDrilldownDepth(int _nDepth) {
		nDrillDownDepth = _nDepth;
	}

	public Object GetGroupIcon(int group) throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetGroupName(int group) throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetLastGroupByField() throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetMisspelledToken(int misspellingIndex)
			throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public int GetNumCorrections(int misspellingIndex)
			throws SearchModelException {
		// TODO Auto-generated method stub
		return 0;
	}

	public int GetNumMisspelledTokens() throws SearchModelException {
		// TODO Auto-generated method stub
		return 0;
	}

	public int GetNumResultGroups() throws SearchModelException {
		// TODO Auto-generated method stub
		return 0;
	}

	public String GetPageMode() {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetResultBestBetKey(int rank) throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetResultCollabProjectName(int rank)
			throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetResultCollabProjectURL(int rank)
			throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public HTMLAnchor GetResultEditLink(int rank, boolean allowPopup)
			throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public int GetResultRank(int groupNum, int resultNum)
			throws SearchModelException {
		// TODO Auto-generated method stub
		return 0;
	}

	public int GetReturnedResultsInGroup(int group) throws SearchModelException {
		// TODO Auto-generated method stub
		return 0;
	}

	public XPHashtable GetSearchAgainArgs(boolean includeOptional)
			throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetSpellingCorrection(int misspellingIndex,
			int correctionIndex) throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetTotalResultsInGroup(int group) throws SearchModelException {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean hasDrilledDownIntoSingleCategory(String orgType)
			throws SearchModelException {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isEntireGroupReturned(int group) throws SearchModelException {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean wasSpellCorrected() throws SearchModelException {
		// TODO Auto-generated method stub
		return false;
	}

	public SearchExecutionError GetError() {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetResultAbsType(int rank) {
		// TODO Auto-generated method stub
		return null;
	}

	public XPDateTime GetResultCreated(int rank) {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetResultEODField(int rank, int propertyId) {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetResultPath(int rank) {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetResultPortletField(int rank, int fieldId) {
		// TODO Auto-generated method stub
		return null;
	}

	public int GetResultSize(int rank) {
		// TODO Auto-generated method stub
		return 0;
	}

	public String GetResultSubType(int rank) {
		// TODO Auto-generated method stub
		return null;
	}

	public SearchExecutionWarning GetWarning() {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean IsInitialized() {
		// TODO Auto-generated method stub
		return false;
	}

	public String GetName() {
		// TODO Auto-generated method stub
		return null;
	}

	public int GetItemsPerPage() {
		// TODO Auto-generated method stub
		return 10;
	}

	public String GetSortOrder() {
		// TODO Auto-generated method stub
		return null;
	}

	public int GetAbstractTypeRestrictions() {
		// TODO Auto-generated method stub
		return 0;
	}

	public int[] GetAdminFolderRestrictions() {
		// TODO Auto-generated method stub
		return null;
	}

	public int GetApplicationRestrictions() {
		// TODO Auto-generated method stub
		return 0;
	}

	public int[] GetCollabProjectRestrictions() {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetCommunityNameForID(int id) {
		// TODO Auto-generated method stub
		return null;
	}

	public int[] GetCommunityRestrictions() {
		// TODO Auto-generated method stub
		return null;
	}

	public int[] GetDocFolderRestrictions() {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetFolderNameForID(int id, boolean isAdmin) {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetLanguageRestriction() {
		// TODO Auto-generated method stub
		return null;
	}

	public int[] GetObjectTypeRestrictions() {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetPortletNameForID(int id) {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetASConstraintOperator(int condition) {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetASConstraintProperty(int condition) {
		// TODO Auto-generated method stub
		return null;
	}

	public String GetASConstraintValue(int condition) {
		// TODO Auto-generated method stub
		return null;
	}

	public int GetASNumPropertyConstraints() {
		// TODO Auto-generated method stub
		return 0;
	}

	public boolean GetASPropertyClauseOperator() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean GetASTopLevelOperator() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean IsASTooComplexToSummarize() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean GetHasBeenSaved() {
		// TODO Auto-generated method stub
		return false;
	}

	public int GetSavedSearchErrorCode() {
		// TODO Auto-generated method stub
		return 0;
	}

	public String GetSavedSearchName() {
		// TODO Auto-generated method stub
		return null;
	}

	public void Drilldown(String key, String groupByField, int category) {
		// TODO Auto-generated method stub
		
	}

	public void RefreshResults(String keyOnPage) {
		// TODO Auto-generated method stub
		
	}

	public void Reorganize(String key, String groupByField) {
		// TODO Auto-generated method stub
		
	}

	public void RestoreFromBreadcrumb(String keyOfPageWhereLinkAppears,
			String keyToRestore) {
		// TODO Auto-generated method stub
		
	}

	public void Search(String searchString, String basicFields,
			IPTFilter advancedFilter, SearchSettingCollection settings) {
		// TODO Auto-generated method stub
		
	}

	public void Search(String searchString, String basicFields,
			IPTFilter advancedFilter, SearchSettingCollection allSettings,
			SearchSettingCollection removableSettings) {
		// TODO Auto-generated method stub
		
	}

	public void SearchWithinResults(String key, String additionalQuery) {
		// TODO Auto-generated method stub
		
	}

	public void SetError(SearchExecutionError e) {
		// TODO Auto-generated method stub
		
	}

	public void SetPageMode(String pageMode) {
		// TODO Auto-generated method stub
		
	}

	public void SetWarning(SearchExecutionWarning e) {
		// TODO Auto-generated method stub
		
	}

	public void CleanupAllData() {
		// TODO Auto-generated method stub
		
	}

	public void CleanupTempData() {
		// TODO Auto-generated method stub
		
	}

	public void Init(AActivitySpace arg0) {
		// TODO Auto-generated method stub
		
	}

	public Object Create() {
		// TODO Auto-generated method stub
		return null;
	}

	public void GoToPage(String arg0, int arg1, int arg2, Object arg3) {
		// TODO Auto-generated method stub
		
	}

	public void SetSavedSearchErrorCode(int code) {
		// TODO Auto-generated method stub
		
	}

	public void SetSavedSearchName(String name) {
		// TODO Auto-generated method stub
		
	}

	public String GetResultStyleClass(String subType) {
		// TODO Auto-generated method stub
		return null;
	}

	public void Filter(String key, int filterId) {
		// TODO Auto-generated method stub
		
	}

	public boolean CollabInstalled() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean ContentInstalled() {
		// TODO Auto-generated method stub
		return false;
	}

	public void ChangeSortOrder(String sortOrder, Object additionalInfo) {
		// TODO Auto-generated method stub
		
	}

	public int GetSelectedTypeFilter() {
		// TODO Auto-generated method stub
		return 0;
	}
}
