package com.plumtree.portaluiinfrastructure.test.opener.plugins;

import junit.framework.*;

import com.plumtree.openfoundation.mockobjects.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.openfoundation.xml.dom.*;
import com.plumtree.portaluiinfrastructure.friendlyurl.*;
import com.plumtree.portaluiinfrastructure.mock.friendlyurl.*;
import com.plumtree.portaluiinfrastructure.mock.tags.*;
import com.plumtree.portaluiinfrastructure.opener.plugins.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.opener.*;
import com.plumtree.uiinfrastructure.web.*;

public class FriendlyURLPluginTest extends TestCase { 

	private XPDocument document = null;
	private String URIPattern = "portal/server.pt/";
	private String key = "community";
	private String classId = "512";
	private RedirectData redirectData = null;
	private BaseOpenerAS asCurrentSpace = null;
	private Object userSession = null;
	
	/**
	 */
	public FriendlyURLPluginTest(String name)
   	{
		super(name);
   	}
   
   	/**
	 * Setup if any is required.
	 */
   	public void setUp() throws Exception
   	{
		super.setUp();
		// DCA - set up config document
		document = new XPDocument();
		XPNode textNode = document.CreateTextNode(URIPattern);
		XPNode elemNode = document.CreateElement("DetectionPattern");
		elemNode.AppendChild(textNode);
		XPNode settingsNode = document.CreateElement("FriendlyURLSettings");
		settingsNode.AppendChild(elemNode);
		document.AppendChild(settingsNode);
		
		// DCA - set up mapping
		textNode = document.CreateTextNode(key);
		elemNode = document.CreateElement("key");
		elemNode.AppendChild(textNode);
		XPNode parentNode = document.CreateElement("FriendlyURLMapping");
		parentNode.AppendChild(elemNode);
		textNode = document.CreateTextNode(classId);
		elemNode = document.CreateElement("classId");
		elemNode.AppendChild(textNode);
		parentNode.AppendChild(elemNode);
		settingsNode.AppendChild(parentNode);
		
		IApplication application = new Application("","");
		ApplicationManager.SetTestInstance(new MockApplicationManager());
		((MockApplicationManager) ApplicationManager.GetInstance()).SetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString(), application);
		application.GetCachingManager().SetEntry(AppWarmUp.KEY_CACHEMAN_CLASS_TYPE_MANAGER, new MockClassTypeDescManager());
   	}
   	
   	/**
	 * @see TestCase#tearDown()
	 */
	public void tearDown()  
	{
		document = null;
		FriendlyURLPlugin.UnInit();
		redirectData = null;
		asCurrentSpace = null;
		userSession = null;
	}
	
   	public void testSetUpConfiguration() 
   	{
   		FriendlyURLPlugin.SetUpConfiguration(document);
   		FriendlyURLPlugin testObject = new FriendlyURLPlugin();
   		
   		String pattern = testObject.GetURIPattern();
   		Assert.assertEquals("URI pattern does not match what was set in the variable.", pattern, URIPattern);
   	}
   	
   	public void testProcessPlugIn() 
   	{
   		// DCA - need to call this to set up config
   		FriendlyURLPlugin.SetUpConfiguration(document);
   		FriendlyURLPlugin testObject = new FriendlyURLPlugin();
   		
   		redirectData = new RedirectData();
   		asCurrentSpace = new BaseOpenerAS();
   		IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
   		asCurrentSpace.SetApplication(application);
	
		ConfigVarPack varPack;
		if (XPSystem.IsRunningInDotNet()) {
			varPack = new MockConfigVarPack(ConfigVarPack.APP_PLATFORM_DOT_NET);
		} else {
			varPack = new MockConfigVarPack(ConfigVarPack.APP_PLATFORM_JAVA);
		}
		asCurrentSpace.GetApplication().GetVarPackManager().AddVariablePackage(varPack);
		userSession = new Object();
		IXPRequest request = null;
		
		boolean failed = false;
   		try {
   			request = XPRequestSimulator.CreateGET("http://localhost/portal/server.pt/community/");
   			asCurrentSpace.SetCurrentHTTPRequestResponseObjects(request, null);
   			testObject.ProcessPlugIn(redirectData, asCurrentSpace, userSession);
   		} catch (XPException e) {
   			failed = true;
   		}
   		Assert.assertTrue("Friendly URL w/o a community name should fail.", failed);
   		
   		failed = false;
   		try {
   			request = XPRequestSimulator.CreateGET("http://localhost/portal/server.pt/fakeKey/my_community");
   			asCurrentSpace.SetCurrentHTTPRequestResponseObjects(request, null);
   			testObject.ProcessPlugIn(redirectData, asCurrentSpace, userSession);
   		} catch (XPException e) {
   			failed = true;
   		}
   		Assert.assertTrue("Friendly URL with a bad key should fail.", failed);
   		
   		request = XPRequestSimulator.CreateGET("http://localhost/portal/server.pt/community/my_community");
		asCurrentSpace.SetCurrentHTTPRequestResponseObjects(request, null);
		testObject.ProcessPlugIn(redirectData, asCurrentSpace, userSession);
   	}

   	public void testDirRESTAPIRedirect()
   	{
   		// DCA - need to call this to set up config
   		FriendlyURLPlugin.SetUpConfiguration(document);
   		FriendlyURLPlugin testObject = new FriendlyURLPlugin();

   		APIFriendlyURLParser parser = new APIFriendlyURLParser("api", FriendlyURLConstants.REST_API_ID_STRING,
   			new XPStringTokenizer("v1/kd/123", "/"));
   		RedirectData data = new RedirectData();
   		boolean bResult = testObject.HandleNonObjectFriendlyURLs(parser, data, new MockEnvironment());

   		Assert.assertTrue("Dir REST API URl should be handled.", bResult);

		Redirect result = data.GetRedirect();
		Assert.assertEquals("Dir REST API redirected to wrong space.", "Dir", result.GetSpaceName());
		Assert.assertEquals("Dir REST API redirected to wrong control.", "DirREST", result.GetControl());
		Assert.assertEquals("Dir REST API included wrong folder ID.", "123", result.GetControlArgument("folderID"));
   	}

   	public void testUserRESTAPIRedirect()
   	{
   		// DCA - need to call this to set up config
   		FriendlyURLPlugin.SetUpConfiguration(document);
   		FriendlyURLPlugin testObject = new FriendlyURLPlugin();

   		APIFriendlyURLParser parser = new APIFriendlyURLParser("api", FriendlyURLConstants.REST_API_ID_STRING,
   			new XPStringTokenizer("v1/user/current", "/"));
   		RedirectData data = new RedirectData();
   		boolean bResult = testObject.HandleNonObjectFriendlyURLs(parser, data, new MockEnvironment());

   		Assert.assertTrue("User REST API URl should be handled.", bResult);

		Redirect result = data.GetRedirect();
		Assert.assertEquals("User REST API redirected to wrong space.", "UserREST", result.GetSpaceName());
		Assert.assertEquals("User REST API redirected to wrong control.", "UserREST", result.GetControl());
		Assert.assertEquals("User REST API included wrong user ID.", "current", result.GetControlArgument("user"));
   	}
}