/*
 * Created on Jun 8, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.data;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.uiinfrastructure.constants;
namespace com.plumtree.taglib.ptdata
{
  /// <summary>
  /// Shared data and definitions for PTURL DO Data tags
  /// </summary>
  public abstract class APTURLDODataTag : ATag
  {
    public const String COMMON_PRETAGMETADATA_DESC = "Sets PTURL DO data on the PORTLET REQUEST scope with ";
    public static readonly String COMMON_POSTTAGMETADATA_DESC = "<![CDATA[<p>]]>" + "This URL contains the following standard URL components: " + "<![CDATA[<code>]]>" + PTURLConstants.KEY_TITLE + ", " + PTURLConstants.KEY_FULLURL + ", " + PTURLConstants.KEY_URI + ", " + PTURLConstants.KEY_ICONIMG + ", " + PTURLConstants.KEY_ICONIMGHEIGHT + ", " + PTURLConstants.KEY_ICONIMGWIDTH + " and " + PTURLConstants.KEY_PARAMS + "<![CDATA[</code>]]>. <![CDATA[<br>]]>" + "In addition, the name of each parameter in the query string of the URL in the link is also " + "a valid component key. So param1 could be used as a key to retrieve the value of the query string parameter " + "<![CDATA[<code>]]>param1=value1<![CDATA[</code>]]>. For instance, if the url is " + "<![CDATA[http://...server.pt?open=518&objID=58411...]]> you can get the objid data as a name/value pair stored under objid (all lower case).<![CDATA[<br>]]>";
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly RequiredTagAttribute ATTRIB_ID;
    public static readonly OptionalTagAttribute ATTRIB_SCOPE;
    public static readonly OptionalTagAttribute ATTRIB_IMG;
    static APTURLDODataTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      ATTRIB_ID = new RequiredTagAttribute("id", "Name of the variable in memory where the PTURL DO data created by this tag is stored. " + "Specify this ID in the data attribute (most commonly the DATAVAR attribute) for any display tag consuming PTURL DO data. " + "Multiple data tags can share the same ID, in which case they are added to a list in the order they were processed. ",
       AttributeType.STRING);
      ATTRIB_SCOPE = new OptionalTagAttribute("scope", "The scope used to store the data. See Tagdoc for the logic tag library for a list of all available scopes. ",
       AttributeType.STRING, Scope.PORTLET_REQUEST.ToString());
      ATTRIB_IMG = new OptionalTagAttribute("img", "This scope used to store the data. See Tagdoc for the logic tag " + "library for a list of all available scopes. ",
       AttributeType.STRING, null);
    }
    
    /// <summary>
    /// PTURL Setter Helper class *
    /// </summary>
    internal TagDataSetterPTURLDOData m_URLSetter = new TagDataSetterPTURLDOData();
    
    /// <param name="xpList"></param>
    public virtual void SetPTURLDataCollection(IXPList xpList)
    {
      Scope tagScope = Scope.GetScope(GetTagAttributeAsString(ATTRIB_SCOPE));
      m_URLSetter.SetScope(tagScope);
      String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
      m_URLSetter.SetDataCollectionOnTagScope(GetState(), strDataReference,
       xpList);
    }
    
    /// <param name="urlEntry"></param>
    public virtual void SetPTURLDataEntry(IPTURL urlEntry)
    {
      Scope tagScope = Scope.GetScope(GetTagAttributeAsString(ATTRIB_SCOPE));
      m_URLSetter.SetScope(tagScope);
      String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
      String strImg = GetTagAttributeAsString(ATTRIB_IMG);
      if (null != strImg && urlEntry != null)
      {
        urlEntry.SetURLData(PTURLConstants.KEY_ICONIMG, strImg);
      }
      m_URLSetter.SetDataEntryOnTagScope(GetState(), strDataReference, urlEntry);
      }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(com.plumtree.uiinfrastructure.constants.AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles style)
    {
      // This tag does not display any JavaScript, and therefore
      // supports all access styles.
      return true;
    }
  }
}