/*
 * Created on Dec 5, 2007
 */
 
using System;
using XPArrayList=com.plumtree.openfoundation.util.XPArrayList;
using com.plumtree.portaluiinfrastructure.statichelpers;
using ATag=com.plumtree.portaluiinfrastructure.tags.ATag;
using Scope=com.plumtree.portaluiinfrastructure.tags.Scope;
using HashtableDataObject=com.plumtree.portaluiinfrastructure.tags.data.HashtableDataObject;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.server;
using com.plumtree.taskapi.portalui;
using com.plumtree.uiinfrastructure.uitasks;
using com.plumtree.xpshared.htmlelements;
using com.plumtree.uiinfrastructure.activityspace;
using AccessStyles=com.plumtree.uiinfrastructure.constants.AccessStyles;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
namespace com.plumtree.taglib.ptdata
{
  /// <summary>
  /// Implementation of the CurrentProfileIDDataTag Data Tag
  /// </summary>
  public class CurrentProfileDataTag : ATag
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     typeof(CurrentProfileDataTag));
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute KEY;
    public static readonly OptionalTagAttribute ATTRIB_SCOPE;
    static CurrentProfileDataTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("currprofiledata", "Returns information about the current User Profile (on profile pages). Returns nothing if user is not on a profile page. " + "Note this tag displays information about the user who\'s profile is being viewed; not the logged-in user.");
      KEY = new RequiredTagAttribute("key", "The name of the variable the profile data collection is stored.");
      ATTRIB_SCOPE = new OptionalTagAttribute("scope", "The scope used to store the data. See Tagdoc for the logic tag library for a list of all available scopes. ",
       AttributeType.STRING, Scope.PORTLET_REQUEST.ToString());
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()"></seealso>
    public override HTMLElement DisplayTag()
    {
      Scope tagScope = Scope.GetScope(GetTagAttributeAsString(ATTRIB_SCOPE));
      IEnvironment env = GetEnvironment();
      AActivitySpace owner = TaskAPIUICommon.GetEnvTypeObject(env);
      int profileID = -1;
      // make sure tag is being used on the profile community only
      if (TaskAPIUICommunity.IsCurrentCommunityUserProfileCommunity(owner) && (owner.GetModelExists(MyPortalModel.STR_MVC_CLASS_NAME)))
      {
        // get profile id
        MyPortalModel model = (MyPortalModel) owner.GetModel(MyPortalModel.STR_MVC_CLASS_NAME);
        profileID = model.GetProfileID();
        IPTProfileManager m_ptProfileManager = null;
        try
        {
          // open the profile manager
          m_ptProfileManager = (IPTProfileManager) ((IPTSession) owner.GetUserSession()).OpenGlobalObject(PT_GLOBALOBJECTS.PT_GLOBAL_PROFILE_MANAGER,
           false);
          IPTUserInfo userInfo = m_ptProfileManager.GetUserInfo(profileID);
          IXPList profileData = new XPArrayList();
          // load the properties into a dataObject
          HashtableDataObject profileDO = new HashtableDataObject();
          String[] emptyArray = null;
          Object[][] property = userInfo.LookupUserInfoSettings(emptyArray);
          if (null != property)
          {
            int numProperties = property[0].Length;
            for (int i = 0;i < numProperties;i++)
            {
              profileDO.SetData(XPStringUtility.ToLowerCase((String) property[0][i]),
               safeString((String) property[1][i]));
            }
          }
          profileDO.SetData("id", "" + profileID);
          // add the dataObject to the arrayList
          profileData.Add(profileDO);
          SetStateSharedVariable(GetTagAttributeAsString(KEY), profileDO,
           tagScope, false);
        }
        catch (Exception e)
        {
          log.Info(e, "Tag currprofiledata was unable to retrieve User Profile data from the current page. " + "No data was set to datavar: \'{0}\' in scope: \'{1}\'",
           GetTagAttributeAsString(KEY), tagScope);
          return null;
        }
      }
      log.Info("Tag currprofiledata was not used on a User Profile Page. " + "No data was set to datavar: \'{0}\' in scope: \'{1}\'",
       GetTagAttributeAsString(KEY), tagScope);
      return null;
    }
    // avoid setting null values
    private String safeString(String in_text)
    {
      return (in_text == null) ? "" : in_text;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#Create()"></seealso>
    public override ATag Create()
    {
      return new CurrentProfileDataTag();
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(com.plumtree.uiinfrastructure.constants.AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles style)
    {
      // This tag does not display any JavaScript, and therefore
      // supports all access styles.
      return true;
    }
  }
}