using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.helpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.ptdata
{
  /// <summary>
  /// This tag will display a link back to the current page or to
  /// the base URL of the current page (without the ?).  The HTTP
  /// entry point portion of the link can be modified.
  /// </summary>
  public class PageLinkTag : ATag
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     "com.plumtree.taglib.ptdata.PageLinkTag");
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly OptionalTagAttribute KEY;
    public static readonly OptionalTagAttribute SCOPE;
    public static readonly OptionalTagAttribute ENTRY_POINT;
    public static readonly OptionalTagAttribute USE_BASE_URL;
    static PageLinkTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("pagelink", "This tag will display a link back to the current page or to the base URL of the " + "current page (without the ?).  The HTTP entry point portion of the link can be modified.",
       ATag.EncodeForTagDoc("<pt:ptdata.pagelink pt:usebaseurl=\"true\"/><br>" + "<pt:logic.value pt:value=\"$pagelink\"/><br>" + "or<br>" + "<pt:ptdata.pagelink pt:entrypoint=\"/portal/test.pt\"/><br>" + "<pt:logic.value pt:value=\"$pagelink\"/>"));
      KEY = new OptionalTagAttribute("key", "The key used to store the variable in memory.",
       AttributeType.STRING, "pagelink");
      SCOPE = new OptionalTagAttribute("scope", "The scope used to store the variable.  See the Logic Tag Library " + "description for more details about valid scope arguments.",
       AttributeType.STRING, Scope.PORTLET_REQUEST.ToString());
      ENTRY_POINT = new OptionalTagAttribute("entrypoint", "The new HTTP entry point of the URL that will " + "replace the main HTTP entry point (i.e. /portal/server.pt).",
       AttributeType.STRING, "");
      USE_BASE_URL = new OptionalTagAttribute("usebaseurl", "Use the normalized base URL of the current " + "page, rather than the full URL.  This will cause the query string, and any portion of the " + "URL after the entry point (i.e. friendly URL sections) to be removed.",
       AttributeType.BOOLEAN, "false");
    }
    public override HTMLElement DisplayTag()
    {
      String strURL = GetEnvironment().GetCurrentRequestURL();
      if (GetTagAttributeAsBoolean(USE_BASE_URL))
      {
        // Use the base URL instead of the full URL.
        strURL = GetEnvironment().GetCurrentNormalizedBaseURL();
        if (XPStringUtility.EndsWith(strURL, "?"))
        {
          strURL = XPStringUtility.Substring(strURL, 0, strURL.Length - 1);
          }
      }
      String strNewEntryPoint = GetTagAttributeAsString(ENTRY_POINT);
      // If we have a new entry point, modify the URL.
      if ((strNewEntryPoint != null) && (XPStringUtility.Compare("", strNewEntryPoint) != 0))
      {
        String strCurrentEntryPoint = ConfigService.getInstance().GetHTTPEntryPoint();
        int nMethodIndex = XPStringUtility.IndexOf(strURL, "//");
        int nHostIndex = XPStringUtility.IndexOf(strURL, "/", nMethodIndex + 2);
        int nEntryPointIndex = XPStringUtility.IndexOf(strURL, strCurrentEntryPoint,
         nHostIndex + 1);
        // Replace the old entry point with the new one
        strURL = XPStringUtility.Substring(strURL, 0, nHostIndex) + strNewEntryPoint + XPStringUtility.Substring(strURL,
         nEntryPointIndex + strCurrentEntryPoint.Length, strURL.Length);
      }
      Scope scope = Scope.GetScope(GetTagAttributeAsString(SCOPE));
      if (null == scope)
      {
        log.Warn("Tag variable unable to convert scope attribute {0} to a valid scope with key \'{1}\'",
         GetTagAttributeAsString(SCOPE), GetTagAttributeAsString(KEY));
        scope = Scope.PORTLET_REQUEST;
      }
      SetStateSharedVariable(GetTagAttributeAsString(KEY), strURL, scope,
       false);
      return null;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#GetTagType()"></seealso>
    public override TagType GetTagType()
    {
      return TagType.NO_BODY;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles _style)
    {
      // This tag does not display any content by itself, and therefore
      // supports all access styles.
      return true;
    }
    public override ATag Create()
    {
      return new PageLinkTag();
    }
  }
}