using System;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.data;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.taskapi.portalui;
using com.plumtree.uiinfrastructure.uitasks;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.ptdata
{
  /// <summary>
  /// Implementation of the ParentCommunity PTURL DO Data Tag
  /// </summary>
  public class ParentCommunityDataTag : APTURLDODataTag
  {
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute ATTRIB_COMMUNITYID;
    static ParentCommunityDataTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("parentcommunitydata", COMMON_PRETAGMETADATA_DESC + "URL to Parent Community of a " + "specified Community (The Community above the specified Community in the " + "administrative hierarchy. No URL is set if the user " + "does not have access to the Parent Community). " + COMMON_POSTTAGMETADATA_DESC + "The URL will also set the " + PTURLConstants.KEY_CURRENTMARKER + " DO data component on a Community URL " + "if currently in that Community. ");
      ATTRIB_COMMUNITYID = new RequiredTagAttribute("commid", "The Object ID of the Community to retrieve the Parent Communities from.",
       AttributeType.INT);
    }
    internal TagDataSetterPTURLDOData m_URLSetter = new TagDataSetterPTURLDOData();
    public override HTMLElement DisplayTag()
    {
      IEnvironment env = GetEnvironment();
      int nCommunityID = GetTagAttributeAsInt(ATTRIB_COMMUNITYID);
      IPTURL url = TaskAPIUICommunity.GetParentCommunityAsURL(TaskAPIUICommon.GetEnvTypeObject(env),
       nCommunityID);
      SetPTURLDataEntry(url);
      return null;
    }
    public override ATag Create()
    {
      return new ParentCommunityDataTag();
    }
  }
}