/*
 * Created on Jun 8, 2005
 */
package com.plumtree.taglib.ptdata;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.constants.*;

/**
 * Shared data and definitions for PTURL DO Data tags
 * 
 */
public abstract class APTURLDODataTag extends ATag
{
	public static final String COMMON_PRETAGMETADATA_DESC = 
			"Sets PTURL DO data on the PORTLET REQUEST scope with ";
	
	public static final String COMMON_POSTTAGMETADATA_DESC =
		"<![CDATA[<p>]]>"+
		"This URL contains the following standard URL components: "+
		"<![CDATA[<code>]]>"+PTURLConstants.KEY_TITLE+", "+PTURLConstants.KEY_FULLURL+", "+PTURLConstants.KEY_URI+", "+
		PTURLConstants.KEY_ICONIMG+", "+PTURLConstants.KEY_ICONIMGHEIGHT+", "+PTURLConstants.KEY_ICONIMGWIDTH+
		" and "+PTURLConstants.KEY_PARAMS+"<![CDATA[</code>]]>. <![CDATA[<br>]]>"+
		"In addition, the name of each parameter in the query string of the URL in the link is also "+
		"a valid component key. So param1 could be used as a key to retrieve the value of the query string parameter "+
		"<![CDATA[<code>]]>param1=value1<![CDATA[</code>]]>. For instance, if the url is " +
        "<![CDATA[http://...server.pt?open=518&objID=58411...]]> you can get the objid data as a name/value pair stored under objid (all lower case).<![CDATA[<br>]]>";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final RequiredTagAttribute ATTRIB_ID;
	public static final OptionalTagAttribute ATTRIB_SCOPE;
	public static final OptionalTagAttribute ATTRIB_IMG;
	

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.

		ATTRIB_ID = new RequiredTagAttribute("id",
			"Name of the variable in memory where the PTURL DO data created by this tag is stored. "+
			"Specify this ID in the data attribute (most commonly the DATAVAR attribute) for any display tag consuming PTURL DO data. "+
			"Multiple data tags can share the same ID, in which case they are added to a list in the order they were processed. ",	
			AttributeType.STRING);

		ATTRIB_SCOPE = new OptionalTagAttribute("scope",
			"The scope used to store the data. See Tagdoc for the logic tag library for a list of all available scopes. ",
			AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
		
		
		
		ATTRIB_IMG = new OptionalTagAttribute("img", "This scope used to store the data. See Tagdoc for the logic tag " +
				"library for a list of all available scopes. ", AttributeType.STRING, null);
		
	}

	/** PTURL Setter Helper class **/
	TagDataSetterPTURLDOData m_URLSetter = new TagDataSetterPTURLDOData();
	
	/**
	 * 
	 * @param xpList
	 */
	public void SetPTURLDataCollection(IXPList xpList)
	{
		Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE));
		m_URLSetter.SetScope(tagScope);
		String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
		
		m_URLSetter.SetDataCollectionOnTagScope(GetState(), strDataReference, xpList);
		
	}
	
	/**
	 * 
	 * @param urlEntry
	 */
	public void SetPTURLDataEntry(IPTURL urlEntry)
	{
		Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE));
		m_URLSetter.SetScope(tagScope);
		String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
		String strImg = GetTagAttributeAsString(ATTRIB_IMG);
		if(null != strImg && urlEntry != null)
		{
			urlEntry.SetURLData(PTURLConstants.KEY_ICONIMG, strImg);
		}
		
		m_URLSetter.SetDataEntryOnTagScope(GetState(), strDataReference, urlEntry);
		
	}
	
	
	/**
	 * 
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(com.plumtree.uiinfrastructure.constants.AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles style)
	{
		// This tag does not display any JavaScript, and therefore
		// supports all access styles.
		return true;
	
	}

}
