/*
 * Created on Jan 13, 2005
 */
package com.plumtree.taglib.ptdata;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.PT_INTRINSICS;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.uitasks.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Implementation of the CommunityActions PTURL DO Data Tag
 * 
 * @author robertz
 */
public class CommunityActionsDataTag extends APTURLDODataTag
{
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	
	public static final OptionalTagAttribute TAG_INCLUDE_ONECLICK_CREATE;
	public static final OptionalTagAttribute TAG_INHERIT_TEMPLATE;
	public static final OptionalTagAttribute TAG_PAGE_TEMPLATE;
	
	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("communityactionsdata",
			"Sets PTURL DO data on the PORTLET REQUEST scope with " +
			"List of URLs to users's Community related Actions (For example " +
			"'Join Communities Editor' or 'Edit this Community'. Certain links " +
			"are only included if a user has sufficient access). An optional 'includeoneclick' tag " +
			"causes the inclusion of a link for 'one-click' page creation, using the 'inherittemplate' " +
			"and 'pagetemplate' flags to override the defaults of 'true' and 'blank page template'." +
			COMMON_POSTTAGMETADATA_DESC);

		TAG_INCLUDE_ONECLICK_CREATE = new OptionalTagAttribute("includeoneclick","Indicates inclusion of 'one-click page creation' link", AttributeType.BOOLEAN, XPConvert.ToString(false));
		TAG_INHERIT_TEMPLATE = new OptionalTagAttribute("inherittemplate","Indicates whether new page should inherit page template", AttributeType.BOOLEAN, XPConvert.ToString(true));
		TAG_PAGE_TEMPLATE = new OptionalTagAttribute("pagetemplate","Page template to use for new page", AttributeType.INT, XPConvert.ToString(PT_INTRINSICS.PT_BLANK_PAGETEMPLATEID));
	}

	 TagDataSetterPTURLDOData m_URLSetter = new TagDataSetterPTURLDOData();
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		IEnvironment env = GetEnvironment();

		XPArrayList urls = TaskAPIUICommunity.GetCommunityActionsAsURLs(TaskAPIUICommon.GetEnvTypeObject(env),
				GetTagAttributeAsBoolean(TAG_INCLUDE_ONECLICK_CREATE),
				GetTagAttributeAsBoolean(TAG_INHERIT_TEMPLATE),
				GetTagAttributeAsInt(TAG_PAGE_TEMPLATE));
		
		SetPTURLDataCollection(urls);
		
		return null;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new CommunityActionsDataTag();
	}
}