package  com.plumtree.taglib.ptdata;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.uitasks.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Implementation of the CommunityPages PTURL DO Data Tag
 * 
 */
public class CommunityPagesDataTag extends APTURLDODataTag
{
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute ATTRIB_COMMUNITYID;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("communitypagesdata",
			COMMON_PRETAGMETADATA_DESC +
			"List of URLs to Community Pages for a specified Community " +
			"(Only URLs to Community Pages a user has access to will be included) " +
			COMMON_POSTTAGMETADATA_DESC+
			"The URL will also set the <![CDATA[<code>]]>"+PTURLConstants.KEY_CURRENTMARKER+"<![CDATA[</code>]]> DO data component on a Community Page " +
			"if currently on that Community Page. ");

		ATTRIB_COMMUNITYID = new RequiredTagAttribute("commid",
			"The Object ID of the Community to list pages from.",	
			AttributeType.INT);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		IEnvironment env = GetEnvironment();
		
		int nCommunityID = GetTagAttributeAsInt(ATTRIB_COMMUNITYID);
		
		XPArrayList urls = TaskAPIUICommunity.GetCommunityPagesAsURLs(TaskAPIUICommon.GetEnvTypeObject(env), nCommunityID);
		
		SetPTURLDataCollection(urls);
		
		return null;
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new CommunityPagesDataTag();
	}
}
