/*
 * Created on Nov 30, 2007
 */
package  com.plumtree.taglib.ptdata;

import com.plumtree.openfoundation.util.XPArrayList;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.ATag;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.uitasks.*;
import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.AccessStyles;

/**
 * Implementation of the CurrentCommunityDataTag Data Tag
 * 
 */
public class CurrentCommunityDataTag extends APTURLDODataTag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		CurrentCommunityDataTag.class);

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("currcommunitydata",
			"Returns information about the current community, including a URL to the home page and optionally the " +
			"Community Name and ID, and the Community Page Name and ID. Returns nothing if user is not on a community page.");
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		IEnvironment env = GetEnvironment();
		AActivitySpace owner = TaskAPIUICommon.GetEnvTypeObject(env);

		if (!TaskAPIUICommunity.IsCurrentPageCommunityPage(owner)) {
			log.Info("Tag currcommunitydata used on a non-Community page. No data to set to datavar: '{0}' in scope '{1}'",
					GetTagAttributeAsString(ATTRIB_ID), GetTagAttributeAsString(ATTRIB_SCOPE));
			return null;
		}

		// current community URL
		XPArrayList urls = TaskAPIUICommunity.GetURLForCurrentCommunity(TaskAPIUICommon.GetEnvTypeObject(env));
		SetPTURLDataCollection(urls);
		
		return null;
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new CurrentCommunityDataTag();
	}
	
	/**
	 * 
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(com.plumtree.uiinfrastructure.constants.AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles style)
	{
		// This tag does not display any JavaScript, and therefore
		// supports all access styles.
		return true;
	}	
}