/*
 * Created on Jan 31, 2005
 */
package  com.plumtree.taglib.ptdata;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.ATag;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.uitasks.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Implementation of the CurrentCommunityPages PTURL DO Data Tag
 * 
 */
public class CurrentCommunityPagesDataTag extends APTURLDODataTag
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		CurrentCommunityPagesDataTag.class);

	
	
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("currcommunitypagesdata",
			COMMON_PRETAGMETADATA_DESC +
			"List of URLs to current Community Pages (Community Pages a " +
			"user is currently in. Only URLs to Community Pages a user has " +
			"access to will be included) " +
			COMMON_POSTTAGMETADATA_DESC+
			"The URL will also set the "+PTURLConstants.KEY_CURRENTMARKER+" DO data component on a Community Page " +
			"if currently on that Community Page. ");
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		IEnvironment env = GetEnvironment();
		
		XPArrayList urls = TaskAPIUICommunity.GetCurrentCommunityPagesAsURLs(TaskAPIUICommon.GetEnvTypeObject(env));
		if ( urls == null ) {
			log.Info("Tag currcommunitypagesdata did not find any pages in current Community. No data to set to datavar: '{0}' in scope '{1}'",
					GetTagAttributeAsString(ATTRIB_ID), GetTagAttributeAsString(ATTRIB_SCOPE));
			urls = new XPArrayList();
		}
		SetPTURLDataCollection(urls);
		
		return null;
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new CurrentCommunityPagesDataTag();
	}

}
