package com.plumtree.taglib.ptdata;

import com.plumtree.openlog.*;
import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.statichelpers.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.uitasks.*;
import com.plumtree.xpshared.htmlelements.*;

public class CurrentPageDataTag extends APTURLDODataTag 
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
		CurrentPageDataTag.class);
	
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("currpagedata",
			COMMON_PRETAGMETADATA_DESC +
			"URL for the current Community Page data. Nothing is set if the current page is not a Community page."+  
			COMMON_POSTTAGMETADATA_DESC);
	}

	@Override
	public HTMLElement DisplayTag() 
	{
		IEnvironment env = GetEnvironment();
		
		IPTURL url = TaskAPIUICommunity.GetCurrentPageAsURL(TaskAPIUICommon.GetEnvTypeObject(env));
		if ( url != null ) {
			log.Info("Tag currpagedata did not find any Page data. No data to set to datavar: '{0}' in scope '{1}'",
					GetTagAttributeAsString(ATTRIB_ID), GetTagAttributeAsString(ATTRIB_SCOPE));
			SetPTURLDataEntry(url);
		} 
		
		return null;
	}
	
	
	@Override
	public ATag Create() 
	{
		return new CurrentPageDataTag();
	}

}
