package com.plumtree.taglib.ptdata;

import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.uitasks.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Implementation of the ParentCommunity PTURL DO Data Tag
 * 
 */
public class ParentCommunityDataTag extends APTURLDODataTag
{

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute ATTRIB_COMMUNITYID;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("parentcommunitydata",
			COMMON_PRETAGMETADATA_DESC +
			"URL to Parent Community of a " +
			"specified Community (The Community above the specified Community in the " +
			"administrative hierarchy. No URL is set if the user " +
			"does not have access to the Parent Community). " +
			COMMON_POSTTAGMETADATA_DESC+
			"The URL will also set the "+PTURLConstants.KEY_CURRENTMARKER+" DO data component on a Community URL " +
			"if currently in that Community. ");

		ATTRIB_COMMUNITYID = new RequiredTagAttribute("commid",
			"The Object ID of the Community to retrieve the Parent Communities from.",	
			AttributeType.INT);
	}

	 TagDataSetterPTURLDOData m_URLSetter = new TagDataSetterPTURLDOData();
	
	public HTMLElement DisplayTag()
	{
		IEnvironment env = GetEnvironment();
	
		int nCommunityID = GetTagAttributeAsInt(ATTRIB_COMMUNITYID);
		
		IPTURL url = TaskAPIUICommunity.GetParentCommunityAsURL(TaskAPIUICommon.GetEnvTypeObject(env), nCommunityID);
		
		SetPTURLDataEntry(url);
		
		return null;
	}

	public ATag Create()
	{
		return new ParentCommunityDataTag();
	}

}
