/*
 * Created on Jan 20, 2005
 */
package  com.plumtree.taglib.ptdata;

import com.plumtree.portaluiinfrastructure.ptlink.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.taskapi.portalui.*;
import com.plumtree.uiinfrastructure.uitasks.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Implementation of the SubCommunities PTURL DO Data Tag
 * 
 */
public class SubCommunitiesDataTag extends APTURLDODataTag
{
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute ATTRIB_COMMUNITYID;
	
	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("subcommunitiesdata",
			COMMON_PRETAGMETADATA_DESC +
			"List of URLs to the Subcommunities of the specified " +
			"Community (Communities located below a Community in the " +
			"Administrative hierarchy. Only Communities a user has " +
			"access to are included). " +
			COMMON_POSTTAGMETADATA_DESC+
			"The URL will also set the "+PTURLConstants.KEY_CURRENTMARKER+" DO data component on a Community URL " +
			"if currently in that Community. ");

		ATTRIB_COMMUNITYID = new RequiredTagAttribute("commid",
			"Object ID of the Community to retrieve Subcommunities from,",	
			AttributeType.INT);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag()
	{
		IEnvironment env = GetEnvironment();

		int nCommunityID = GetTagAttributeAsInt(ATTRIB_COMMUNITYID);
		
		XPArrayList urls = TaskAPIUICommunity.GetSubCommununitiesAsURLs(TaskAPIUICommon.GetEnvTypeObject(env), nCommunityID);
		
		SetPTURLDataCollection(urls);
		
		return null;
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create()
	{
		return new SubCommunitiesDataTag();
	}
}