package com.plumtree.taglib.ptdata.test;

import junit.framework.*;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.mock.tags.*;
import com.plumtree.portaluiinfrastructure.mock.uiinfrastructure.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.displaytree.*;
import com.plumtree.portaluiinfrastructure.tags.helper.*;
import com.plumtree.portaluiinfrastructure.tags.manager.*;
import com.plumtree.taglib.ptdata.*;
import com.plumtree.uiinfrastructure.helpers.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Tests for the page link tag.
 */
public class PageLinkTagTest extends TestCase
{
	public static final String REQUEST_URL = "http://www.test.com:90/portal/server.pt?space=test";

	public MockEnvironment m_environment;

	/**
	 * mock base URL without the trailing ?
	 */
	public String BASE_URL = MockEnvironment.MOCK_BASE_URL.substring(0, MockEnvironment.MOCK_BASE_URL.length() - 1);

	public void setUp()
	{
		ConfigService.setTestInstance(new MockConfigHelper());

		m_environment = new MockEnvironment();

		MockTagLibManager tagLibs = new MockTagLibManager();
		tagLibs.SetTag(new PageLinkTag(), PageLinkTag.TAG.GetName(), PTDataTagLibraryData.LIBRARY.GetName());
		TagLibManager.setTestInstance(tagLibs);
	}

	/**
	 * This method tests storing the base URL.
	 */
	public void testBaseURL()
	{
		// initialize markup
		XMLFragmentTestHelper portletBrowseModeTagHelper = new XMLFragmentTestHelper(
			PTDataTagLibraryData.LIBRARY.GetName(), 
			PageLinkTag.TAG.GetName());
		portletBrowseModeTagHelper.AddPTAttribute(PageLinkTag.USE_BASE_URL.GetName(), "true");
		portletBrowseModeTagHelper.AddPTAttribute(PageLinkTag.SCOPE.GetName(), Scope.HTTP_REQUEST.toString());

   		Object[][] objArrMarkup = portletBrowseModeTagHelper.ToArrayEntireFragment();
   		HTMLElement result = DisplayTree.ProcessTagMarkup(m_environment, objArrMarkup);

   		String strResult = result.GetDisplayString();
   		assertEquals("The output should be empty.", "", strResult);

   		XPHashtable scopeData = TagState.GetHTTPRequestData(m_environment);

   		// Shared state variables are prefixed with a '.'
		SharedStateVariable ssVariable = (SharedStateVariable) scopeData.GetElement("." + PageLinkTag.KEY.GetDefaultValue());
		Assert.assertNotNull("Checking URL data is present.", ssVariable);

		String strURL = (String) ssVariable.GetValue();
		Assert.assertEquals("Checking URL data.", BASE_URL, strURL);
	}

	/**
	 * This method tests storing the base URL with a modified entry point.
	 */
	public void testBaseURLWithNewEntryPoint()
	{
		// initialize markup
		XMLFragmentTestHelper portletBrowseModeTagHelper = new XMLFragmentTestHelper(
			PTDataTagLibraryData.LIBRARY.GetName(), 
			PageLinkTag.TAG.GetName());
		portletBrowseModeTagHelper.AddPTAttribute(PageLinkTag.ENTRY_POINT.GetName(), "/myportal/test.pt");
		portletBrowseModeTagHelper.AddPTAttribute(PageLinkTag.USE_BASE_URL.GetName(), "true");
		portletBrowseModeTagHelper.AddPTAttribute(PageLinkTag.SCOPE.GetName(), Scope.HTTP_REQUEST.toString());

   		Object[][] objArrMarkup = portletBrowseModeTagHelper.ToArrayEntireFragment();
   		HTMLElement result = DisplayTree.ProcessTagMarkup(m_environment, objArrMarkup);

   		String strResult = result.GetDisplayString();
   		assertEquals("The output should be empty.", "", strResult);

   		XPHashtable scopeData = TagState.GetHTTPRequestData(m_environment);

   		// Shared state variables are prefixed with a '.'
		SharedStateVariable ssVariable = (SharedStateVariable) scopeData.GetElement("." + PageLinkTag.KEY.GetDefaultValue());
		Assert.assertNotNull("Checking URL data is present.", ssVariable);

		String strNewURL = GeneralHelpers.Replace(BASE_URL, "/portal/server.pt", "/myportal/test.pt"); 

		String strURL = (String) ssVariable.GetValue();
		Assert.assertEquals("Checking URL data with new http entry point.", strNewURL, strURL);
	}

	/**
	 * This method tests storing the current page URL.
	 */
	public void testPageURL()
	{
		// initialize markup
		XMLFragmentTestHelper portletBrowseModeTagHelper = new XMLFragmentTestHelper(
			PTDataTagLibraryData.LIBRARY.GetName(), 
			PageLinkTag.TAG.GetName());
		portletBrowseModeTagHelper.AddPTAttribute(PageLinkTag.SCOPE.GetName(), Scope.HTTP_REQUEST.toString());

   		Object[][] objArrMarkup = portletBrowseModeTagHelper.ToArrayEntireFragment();
   		HTMLElement result = DisplayTree.ProcessTagMarkup(m_environment, objArrMarkup);

   		String strResult = result.GetDisplayString();
   		assertEquals("The output should be empty.", "", strResult);

   		XPHashtable scopeData = TagState.GetHTTPRequestData(m_environment);

   		// Shared state variables are prefixed with a '.'
		SharedStateVariable ssVariable = (SharedStateVariable) scopeData.GetElement("." + PageLinkTag.KEY.GetDefaultValue());
		Assert.assertNotNull("Checking URL data is present.", ssVariable);

		String strURL = (String) ssVariable.GetValue();
		Assert.assertEquals("Checking URL data.", "http://localhost", strURL);
	}

	/**
	 * This method tests storing the page URL with a modified entry point.
	 */
	public void xxxtestPageURLWithNewEntryPoint()
	{
		// TODO: We can't seem to create an XPRequest with a different URL - the helper
		// functions blow up.  So we can't do anything other than a localhost url.

		// initialize markup
		XMLFragmentTestHelper portletBrowseModeTagHelper = new XMLFragmentTestHelper(
			PTDataTagLibraryData.LIBRARY.GetName(), 
			PageLinkTag.TAG.GetName());
		portletBrowseModeTagHelper.AddPTAttribute(PageLinkTag.ENTRY_POINT.GetName(), "/myportal/test.pt");
		portletBrowseModeTagHelper.AddPTAttribute(PageLinkTag.SCOPE.GetName(), Scope.HTTP_REQUEST.toString());

   		Object[][] objArrMarkup = portletBrowseModeTagHelper.ToArrayEntireFragment();
   		HTMLElement result = DisplayTree.ProcessTagMarkup(m_environment, objArrMarkup);

   		String strResult = result.GetDisplayString();
   		assertEquals("The output should be empty.", "", strResult);

   		XPHashtable scopeData = TagState.GetHTTPRequestData(m_environment);

   		// Shared state variables are prefixed with a '.'
		SharedStateVariable ssVariable = (SharedStateVariable) scopeData.GetElement("." + PageLinkTag.KEY.GetDefaultValue());
		Assert.assertNotNull("Checking URL data is present.", ssVariable);

		String strNewURL = GeneralHelpers.Replace("http://localhost", "/portal/server.pt", "/myportal/test.pt"); 

		String strURL = (String) ssVariable.GetValue();
		Assert.assertEquals("Checking URL data with new http entry point.", strNewURL, strURL);
	}
}