using System;
using XPStringUtility=com.plumtree.openfoundation.util.XPStringUtility;
using com.plumtree.portalpages.common.login;
using com.plumtree.portaluiinfrastructure.htmlconstructs;
using PTLoginHelper=com.plumtree.portaluiinfrastructure.login.PTLoginHelper;
using com.plumtree.portaluiinfrastructure.tags;
using CommonTagAttributeConstants=com.plumtree.portaluiinfrastructure.tags.common.CommonTagAttributeConstants;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.taglib.ptui
{
  /// <summary>
  /// This tag displays the 'Remember my Password' checkbox.
  /// </summary>
  public class LoginRememberMeTag : ATag
  {
    
    /// <summary>
    /// for performance 
    /// </summary>
    private const String EMPTY_STRING = "";
    public static readonly String TAG_EXAMPLE = ATag.EncodeForTagDoc("In Legacy Mode <br>" + "<pt:ptui.loginrememberme/><br>" + "<pt:logic.value pt:value=\"Remember Password\"/><br>" + "or to output name of control and customize the display <br>" + "<pt:ptui.loginrememberme pt:name=\"true\"><br>" + "<pt:core.html pt:tag=\"input\" type=\"checkbox\" name=\"$name\" /><br>" + "<pt:logic.value pt:value=\"Remember Password\"/><br>" + "</pt:ui.loginrememberme><br>");
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredParentTag PARENT;
    public static readonly OptionalTagAttribute KEY;
    public static readonly OptionalTagAttribute ATTRIB_SCOPE;
    static LoginRememberMeTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("loginrememberme", "This tag is used for the \'Remember my Password\' functionality and it" + " works in two ways, it displays \'Remember my Password\' checkbox if used" + "with out key attribute. If the \'key\' optional attribute is given a value" + "then this tag outputs the name of the inputcontrol as key parameter\'s value and displays" + "the body of the html inside it, Note that this tag displays either of the above outputs only when" + " \'AllowAutoConnect\' in portalconfig.xml is turned on. ",
       TAG_EXAMPLE);
      ATTRIB_SCOPE = new OptionalTagAttribute(CommonTagAttributeConstants.ATTRIB_SCOPE_NAME,
       CommonTagAttributeConstants.ATTRIB_SCOPE_DESC, AttributeType.STRING,
       Scope.PORTLET_REQUEST.ToString());
      KEY = new OptionalTagAttribute("key", "This takes String value as input " + "and if given a value it provides the name of the input control stored as key value and displays the body " + "if \'AllowAutoConnect\' in portalconfig.xml is turned on",
       AttributeType.STRING, "");
      PARENT = new RequiredParentTag(LoginFormTag.TAG);
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()"></seealso>
    public override HTMLElement DisplayTag()
    {
      HTMLElementCollection response = null;
      String strKeyAttrName = GetTagAttributeAsString(KEY);
      // Tag Outputs the name of the Remember Me Field with key value as 
      // parameter and also includes inner HTML only if 'AllowAutoConnect' 
      // is enabled
      if ((!XPStringUtility.EqualsIgnoreCase(strKeyAttrName, EMPTY_STRING)) && PTLoginHelper.ShowRememberMyPassword())
      {
        // Add Name of the Field to Memory and Process body.
        Scope tagScope = Scope.GetScope(GetTagAttributeAsString(ATTRIB_SCOPE));
        GetState().SetSharedVariable(strKeyAttrName, LoginHTML.PARAM_REMEMBER,
         tagScope, true);
        // Process body			
        return ProcessTagBody();
      } else
      // Displayed only if 'AllowAutoConnect' is enabled, this is where we display 
      //checkbox just to support older implementation.
      if (PTLoginHelper.ShowRememberMyPassword())
      {
        response = new HTMLElementCollection();
        // TODO CR DONH: This needs to be customizable
        String rememberPasswordText = GetEnvironment().GetString(636, "ptmsgs_portalbrowsingmsgs");
        HTMLInput myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, LoginHTML.PARAM_REMEMBER,
         rememberPasswordText);
        // TODO CR DONH: This needs to be customizable
        myInput.SetID(PTStyleID.PT_LOGIN_REMEMBER_CHECKBOX);
        // TODO CR DONH: This needs to be customizable
        myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
        response.AddInnerHTMLElement(myInput);
      }
      return response;
    }
    
    /// <seealso cref="com.plumtree.taglib.ptui.APTButtonTag#GetButtonImage(String)"></seealso>
    public virtual HTMLElement GetButtonImage(String _strAltText)
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)"></seealso>
    public override bool SupportsAccessStyle(AccessStyles _style)
    {
      // This tag does not display any JavaScript, and therefore
      // supports all access styles.
      return true;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#Create()"></seealso>
    public override ATag Create()
    {
      return new LoginRememberMeTag();
    }
  }
}