package com.plumtree.taglib.ptui;

import com.plumtree.openfoundation.util.XPStringUtility;
import com.plumtree.portalpages.common.login.*;
import com.plumtree.portaluiinfrastructure.htmlconstructs.*;
import com.plumtree.portaluiinfrastructure.login.PTLoginHelper;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.common.CommonTagAttributeConstants;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This tag displays the 'Remember my Password' checkbox.
 */
public class LoginRememberMeTag extends ATag
{
	/** for performance */
	private static final String EMPTY_STRING = "";
	
	public static final String TAG_EXAMPLE = ATag.EncodeForTagDoc(
			"In Legacy Mode <br>" +
			"<pt:ptui.loginrememberme/><br>" +
			"<pt:logic.value pt:value=\"Remember Password\"/><br>" +
			"or to output name of control and customize the display <br>" +
			"<pt:ptui.loginrememberme pt:name=\"true\"><br>" +					
			"<pt:core.html pt:tag=\"input\" type=\"checkbox\" name=\"$name\" /><br>" +
			"<pt:logic.value pt:value=\"Remember Password\"/><br>" +
			"</pt:ui.loginrememberme><br>" );
	
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
    public static final ITagMetaData TAG;
	public static final RequiredParentTag PARENT;
	public static final OptionalTagAttribute KEY;
	public static final OptionalTagAttribute ATTRIB_SCOPE;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData("loginrememberme",
				"This tag is used for the 'Remember my Password' functionality and it" +
				" works in two ways, it displays 'Remember my Password' checkbox if used" + 
				"with out key attribute. If the 'key' optional attribute is given a value" +
				"then this tag outputs the name of the inputcontrol as key parameter's value and displays" +
				"the body of the html inside it, Note that this tag displays either of the above outputs only when" +
				" 'AllowAutoConnect' in portalconfig.xml is turned on. ",TAG_EXAMPLE );
				
						
		ATTRIB_SCOPE = new OptionalTagAttribute(CommonTagAttributeConstants.ATTRIB_SCOPE_NAME,
				CommonTagAttributeConstants.ATTRIB_SCOPE_DESC,
				AttributeType.STRING, Scope.PORTLET_REQUEST.toString());
		
		KEY = new OptionalTagAttribute("key", "This takes String value as input " +
				"and if given a value it provides the name of the input control stored as key value and displays the body " +
				"if 'AllowAutoConnect' in portalconfig.xml is turned on", AttributeType.STRING, "" );

		PARENT = new RequiredParentTag(LoginFormTag.TAG);
	}

	/**
     * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
     */
    public HTMLElement DisplayTag()
    {
		HTMLElementCollection response = null;		
		
		String strKeyAttrName = GetTagAttributeAsString(KEY);

		// Tag Outputs the name of the Remember Me Field with key value as 
		// parameter and also includes inner HTML only if 'AllowAutoConnect' 
		// is enabled
		if ( (! XPStringUtility.EqualsIgnoreCase(strKeyAttrName, EMPTY_STRING )) && PTLoginHelper.ShowRememberMyPassword() )
		{
			
			// Add Name of the Field to Memory and Process body.
			
			Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE));			
			GetState().SetSharedVariable(strKeyAttrName, LoginHTML.PARAM_REMEMBER, tagScope, true);
			
			// Process body			
			return ProcessTagBody();
			
		}
		// Displayed only if 'AllowAutoConnect' is enabled, this is where we display 
		//checkbox just to support older implementation.
		else if( PTLoginHelper.ShowRememberMyPassword())
		{
			response = new HTMLElementCollection();
			// TODO CR DONH: This needs to be customizable
			String rememberPasswordText = GetEnvironment().GetString(636, "ptmsgs_portalbrowsingmsgs");
	
			HTMLInput myInput = new HTMLInput(HTMLInputTypes.CHECKBOX,
					LoginHTML.PARAM_REMEMBER, rememberPasswordText);
			// TODO CR DONH: This needs to be customizable
			myInput.SetID(PTStyleID.PT_LOGIN_REMEMBER_CHECKBOX);
			// TODO CR DONH: This needs to be customizable
			myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
			response.AddInnerHTMLElement(myInput);
		}
		
		return response;
    }

    /**
     * @see com.plumtree.taglib.ptui.APTButtonTag#GetButtonImage(String)
     */
    public HTMLElement GetButtonImage(String _strAltText)
    {
		return null;
    }

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// This tag does not display any JavaScript, and therefore
		// supports all access styles.
		return true;
	}

    /**
     * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
     */
    public ATag Create()
    {
        return new LoginRememberMeTag();
    }
}
