using System;
using com.plumtree.openfoundation.util;
using com.plumtree.portalpages.browsing.directory;
using com.plumtree.portaluiinfrastructure.ptlink;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.data;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.server;
using com.plumtree.taskapi.portalui;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.bea.alui.taglib.kdpage
{
  /// <summary>
  /// This tag only works with the Knowledge Directory activity space. It stores the list of documents in the current folder in memory.
  /// Each document data object contains the following variables: 'name' for the document's name, 'id' for the Id of the document,
  /// 'url' is the link to view the document, 'createdate' is the date that the document was created, 'modifieddate' is the date that
  /// the document was last modified, 'description' will have the decription of the document, or 'EMPTY_STRING' if there is no description,
  /// 'imagesrc' has the src for the document's img tag, and 'custompropskey' contains the key to access the documents custom properties
  /// data collection. The custom properties data collection is ordered based on the Knowledge Directory Preference Page and each property data object has a variable
  /// called 'value' that returns the value for that property.
  /// </summary>
  /// <author>Dustin Ang</author>
  public class DocumentsDataTag : AKDPageTag
  {
    private const String TAG_NAME = "documentsdata";
    private const String CUSTOM_PROP_PREFIX = "customprop";
    public const String NAME = "name";
    public const String ID = "id";
    public const String URL = "url";
    public const String CREATE_DATE = "createdate";
    public const String MODIFIED_DATE = "modifieddate";
    public const String DESC = "description";
    public const String IMAGE_SRC = "imagesrc";
    public const String CUSTOM_PROPS_KEY = "custompropskey";
    public const String TYPE_ID = "typeid";
    public const String VALUE = "value";
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly String TAG_DESCRIPTION = ATag.EncodeForTagDoc(KD_PAGE_ONLY_CONSTANT + "Stores the list of documents in the current folder in memory. " + "Each document data object contains the following variables: <br/>" + "<ul>" + "<li> <code>" + NAME + "</code> - The name of the document.</li>" + "<li> <code>" + ID + "</code> - The ID of the document.</li>" + "<li> <code>" + URL + "</code> - The link to view the document.</li>" + "<li> <code>" + CREATE_DATE + "</code> - The date that the document was created.</li>" + "<li> <code>" + MODIFIED_DATE + "</code> - The date that the document was last modified.</li>" + "<li> <code>" + DESC + "</code> - The decription of the document, or \'" + EMPTY_STRING + "\' if there is no description.</li>" + "<li> <code>" + IMAGE_SRC + "</code> - The src for the document\'s img tag.</li>" + "<li> <code>" + CUSTOM_PROPS_KEY + "</code> - The key to access the document\'s custom properties " + "data collection. The custom properties data collection is ordered based on the Knowledge Directory Preference page and each property data object has a variable " + "called <code>" + VALUE + "</code> that returns the value for that property.</li>" + "</ul>");
    static DocumentsDataTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData(TAG_NAME, TAG_DESCRIPTION);
    }
    /*
     * (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
     */
     
    public override ATag Create()
    {
      return new DocumentsDataTag();
    }
    /*
     * (non-Javadoc)
     * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
     */
     
    public override HTMLElement DisplayTag()
    {
      Scope tagScope = Scope.GetScope(GetTagAttributeAsString(ATTRIB_SCOPE));
      IDirModelRO dirModel = GetDirModel();
      IXPList docsList = new XPArrayList();
      // DCA - figure out how many documents to return
      // The actual figuring out which documents to fetch happens in the background, see DirModel:LoadFoldersAndDocuments()
      // By this point, the documents have already been fetched and we're just iterating through them to store them in memory.
      int[] arrCardRange = dirModel.GetCardRange();
      int nFirstObjectIndex = 0;
      int nLastObjectIndex = arrCardRange[DirModel.CARD_CNT];
      bool bBrowseBySearch = (DirConstants.BROWSE_BY_SEARCH == dirModel.GetBrowseBySource());
      // DCA - get documents in range
      for (int i = nFirstObjectIndex;i < nLastObjectIndex;i++)
      {
        HashtableDataObject htObject = GetDocumentDataObject(i, bBrowseBySearch);
        docsList.Add(htObject);
        // DCA - we only display custom properties if we're browsing by search
        if (bBrowseBySearch)
        {
          IXPList customPropList = GetCustomPropDataObject(i);
          GetState().SetSharedVariable(htObject.GetData(CUSTOM_PROPS_KEY),
           customPropList, tagScope, true);
        }
      }
      // DCA - add to memory
      String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
      GetState().SetSharedVariable(strDataReference, docsList, tagScope, true);
      return ProcessTagBody();
    }
    private IXPList GetCustomPropDataObject(int i)
    {
      IDirModelRO dirModel = GetDirModel();
      XPArrayList xpCustomProps = dirModel.GetPropColumns();
      IXPList customPropList = new XPArrayList();
      // DCA - check if we have custom properties to display
      if (xpCustomProps.GetSize() > 0)
      {
        // DCA - get the current document's values for those custom properties
        XPArrayList xpCustomFields = dirModel.GetCustomFields(i, xpCustomProps,
         dirModel.GetUserPrefLocale(), dirModel.GetUserPrefTimeZone());
        for (int j = 0;j < xpCustomFields.GetSize();j++)
        {
          HashtableDataObject htObject = new HashtableDataObject();
          htObject.SetData(VALUE, (String) xpCustomFields.GetElement(j));
          customPropList.Add(htObject);
        }
      }
      return customPropList;
    }
    private HashtableDataObject GetDocumentDataObject(int index, bool bBrowseBySearch)
    {
      IDirModelRO dirModel = GetDirModel();
      // Card ID
      int nCardID = XPConvert.ToInteger(dirModel.GetObjectFields(index, (bBrowseBySearch ? PT_INTRINSICS.PT_PROPERTY_OBJECTID : PT_PROPIDS.PT_PROPID_OBJECTID)));
      // Name
      String strName = (String) dirModel.GetObjectFields(index, (bBrowseBySearch ? PT_INTRINSICS.PT_PROPERTY_OBJECTNAME : PT_PROPIDS.PT_PROPID_NAME));
      // Doc Type
      int nDocType = XPConvert.ToInteger(dirModel.GetObjectFields(index, (bBrowseBySearch ? PT_INTRINSICS.PT_PROPERTY_DOCUMENTTYPEID : PT_PROPIDS.PT_PROPID_CARD_DOCUMENTTYPEID)));
      // Image uiid
      String strImageUIID = (String) dirModel.GetObjectFields(index, (bBrowseBySearch ? PT_INTRINSICS.PT_PROPERTY_OBJECTIMAGEUUID : PT_PROPIDS.PT_PROPID_IMAGEUUID));
      HTMLImg myImg = PlumtreeHelpers.GetHTMLImage(strImageUIID, true);
      String strImageBaseURL = ConfigHelper.GetImageServerBaseURL(ConfigHelper.GetIsSpaceSecure(GetEnvironment().GetCurrentHTTPRequest().IsSecureConnection()));
      myImg.SetImgBaseUrl(strImageBaseURL);
      // DCA - create a date time formate to convert dates to strings
      XPDateFormat xpDateTimeFormat = XPDateFormat.GetDateTimeInstance(XPDateFormat.SHORT,
       XPDateFormat.SHORT, dirModel.GetUserPrefLocale(), dirModel.GetUserPrefTimeZone());
      // Created
      XPDateTime dtCreatedDate = (XPDateTime) dirModel.GetObjectFields(index,
       (bBrowseBySearch ? PT_INTRINSICS.PT_PROPERTY_OBJECTCREATED : PT_PROPIDS.PT_PROPID_CREATED));
      String strCreatedDate = xpDateTimeFormat.Format(dtCreatedDate);
      // Last Modified
      XPDateTime dtLastModifiedDate = (XPDateTime) dirModel.GetObjectFields(index,
       (bBrowseBySearch ? PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED : PT_PROPIDS.PT_PROPID_LASTMODIFIED));
      String strLastModifiedDate = xpDateTimeFormat.Format(dtLastModifiedDate);
      // Description
      String strDescType = dirModel.GetDocDescriptionType();
      String strDesc = "";
      if (false == XPStringUtility.EqualsIgnoreCase(DirConstants.DIR_DOC_DESC_NONE,
       strDescType))
      {
        strDesc = (String) dirModel.GetObjectFields(index, (bBrowseBySearch ? PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY : PT_PROPIDS.PT_PROPID_DESCRIPTION));
        if (null == strDesc)
        {
          strDesc = EMPTY_STRING;
        }
        // abbreviate the description if necessary
        if (true == XPStringUtility.EqualsIgnoreCase(DirConstants.DIR_DOC_DESC_ABBR,
         strDescType))
        {
          if (DirConstants.DESCRIPTION_ABBR_LENGTH < strDesc.Length)
          {
            strDesc = XPStringUtility.Substring(strDesc, 0, DirConstants.DESCRIPTION_ABBR_LENGTH) + "...";
            }
        }
      } else
      {
        strDesc = EMPTY_STRING;
      }
      IPTURL ptDocURL = TaskAPIUIDirectory.CreateCommonOpenerDocumentURL((AActivitySpace) GetEnvironment(),
       strName, nCardID, strName);
      // DCA - create the key to access the document's custom properties
      String strCustomPropKey = CUSTOM_PROP_PREFIX + index;
      HashtableDataObject htObject = new HashtableDataObject();
      htObject.SetData(NAME, strName);
      htObject.SetData(ID, XPConvert.ToString(nCardID));
      htObject.SetData(IMAGE_SRC, myImg.GetSrc());
      htObject.SetData(URL, ptDocURL.GetFullURL());
      htObject.SetData(CREATE_DATE, strCreatedDate);
      htObject.SetData(MODIFIED_DATE, strLastModifiedDate);
      htObject.SetData(DESC, strDesc);
      htObject.SetData(CUSTOM_PROPS_KEY, strCustomPropKey);
      htObject.SetData(TYPE_ID, XPConvert.ToString(nDocType));
      return htObject;
    }
  }
}