using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.portalpages.browsing.myportal;
using com.plumtree.portaluiinfrastructure.statichelpers;
using com.plumtree.portaluiinfrastructure.tags;
using com.plumtree.portaluiinfrastructure.tags.data;
using com.plumtree.portaluiinfrastructure.tags.metadata;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlelements;
namespace com.bea.alui.taglib.portletpage
{
  /// <summary>
  /// Displays an entire portlet (header and content) using the look and feel defined in PortletContentView  
  /// </summary>
  /// <author>rozhang</author>
  public class PortletDisplayTag : ATag
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent(PTDebugHelpers.COMPONENT_PORTAL_ADAPTIVETAGS),
     "com.bea.alui.taglib.portletpage.PortletDisplayTag");
    // All public static final metadata objects should be initialized
    // in a static initializer so that we can control the order of
    // initialization.
    public static readonly ITagMetaData TAG;
    public static readonly RequiredTagAttribute DATA_VAR;
    public static readonly OptionalTagAttribute COLINDEX;
    public static readonly OptionalTagAttribute SCOPE;
    // CR DLH: I'd like to see the portletdisplay name, etc... linked to the static in that
    // class, so that if we update it before release (likely), the tagdocs will update automagically.
    // ROB: It will make the example really hard to read and maintain. I don't think we change the names
    // that often
    public static readonly String DISPLAYTAG_EXAMPLE = ATag.EncodeForTagDoc("<pt:logic.foreach pt:data=\"var1\" pt:var=\"curr\"><br/>" + "<pt:portletpage.portletdisplay pt:datavar=\"curr\"/><br/>" + "</pt:logic.foreach><br/>");
    public static readonly String DISPLAYTAG_COLLECTION_EXAMPLE = ATag.EncodeForTagDoc("Displays the first entry in the collection<br/>" + "<pt:portletpage.portletdisplay pt:datavar=\"var1\" colindex=\"0\" /><br/>");
    static PortletDisplayTag()
    {
      // The TAG member variable needs to be initialized first since it
      // can be used in other tags (i.e. as a RequiredParentTag) that
      // are referenced by member variables of this tag (i.e. as a
      // RelatedChildTag).  This is necessary for tags that have
      // circular references such as a parent / child tag relationship.
      TAG = new TagMetaData("portletdisplay", "Displays an entire portlet with header as defined from a data object, specified by the pt:datavar attribute. " + "The portlet header is displayed as defined by MyPortletContentView, which by default uses the classical look and feel from AquaLogic Interaction (ALI) 6.1.",
       PortletRegionDataTag.DATATAG_EXAMPLE + DISPLAYTAG_EXAMPLE + DISPLAYTAG_COLLECTION_EXAMPLE);
      DATA_VAR = new RequiredTagAttribute("datavar", "The key to the tag memory variable with portlet data to display. The data is set by portletpage.portletregiondata and other similar data tags. " + "This tag is also able to display a single portlet in a collection of portlet data. " + "Use the pt:colindex attribute to specify the index in the collection.");
      COLINDEX = new OptionalTagAttribute("colindex", "Used to get portlet data from a specific index in a collection if the pt:datavar attribute references a " + "portlet data collection instead of a single portlet data. The default value is 0 (the first entry in the collection). " + "The value of this attribute is ignored if pt:datavar refers to a single portlet data IDataObject. " + "Use this tag together with the logic.foreach tag to iterate over all portlets in a collection and display them all.",
       AttributeType.INT, "0");
      SCOPE = new OptionalTagAttribute("scope", "The scope used to store the data collection. See the Logic tag library for further information on scope.",
       AttributeType.STRING, Scope.PORTLET_REQUEST.ToString());
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()"></seealso>
    public override HTMLElement DisplayTag()
    {
      AActivitySpace space = (AActivitySpace) GetEnvironment();
      MyPortalContentView view = null;
      try
      {
        view = (MyPortalContentView) space.GetView(MyPortalContentView.STR_MVC_CLASS_NAME);
        }
      catch (XPException e)
      {
        throw new XPException("Could not find MyPortalContentView in the current ActivitySpace. This tag only works on Community Pages.",
         e);
      }
      // Get the tag data variable from the passed in id 
      String portletVarId = GetTagAttributeAsString(DATA_VAR);
      Scope varScope = Scope.GetScope(GetTagAttributeAsString(SCOPE));
      Object o = GetState().GetSharedVariable(portletVarId, varScope);
      if (null == o)
      {
        throw new XPIllegalArgumentException("No tag variable set with id \'" + portletVarId + "\' found in scope:" + varScope.ToString() + "." + "Verify that a data tag with that id is defined.");
        }
      IDataObject portletVar = null;
      // Handle a collection of portlet data
      if (o is IXPList)
      {
        IXPList list = (IXPList) o;
        // which index in the collection are we getting?
        int index = GetTagAttributeAsInt(COLINDEX);
        try
        {
          portletVar = (IDataObject) list.GetElement(index);
        }
        catch (XPClassCastException e)
        {
          throw new XPIllegalArgumentException("Tag collection with id \'" + portletVarId + "\' does not contain valid IDataObject objects." + " Verify that the datavar attribute refers to a variable set by a portlet data tag.");
          }
        catch (XPException ee)
        {
          throw new XPIllegalArgumentException("Invalid collection index specified for the given tag variable collection: " + portletVarId + " The collection contains: " + list.GetSize() + " while the index references an index of: " + index);
          }
      } else
      // Single portlet data
      {
        try
        {
          portletVar = (IDataObject) o;
        }
        catch (XPClassCastException e)
        {
          throw new XPIllegalArgumentException("Tag variable with id \'" + portletVarId + "\' is not of expected IDataObject type." + "Verify that a Portlet data tag with that id is defined.");
          }
      }
      // Get the portlet index
      int portletIndex = -1;
      try
      {
        portletIndex = XPConvert.ToInteger(portletVar.GetData(PortletRegionDataTag.KEY_PORTLETINDEX));
        }
      catch (XPException e)
      {
        throw new XPIllegalArgumentException("Portlet data variable \'" + portletVarId + "\' contained a non numeric portlet index. ",
         e);
      }
      // Get the display content of the portlet
      HTMLElement result = new HTMLElementCollection();
      log.Debug("Tag portletdisplay displaying portlet with index {0} from datavar {0}, scope {1}, colindex {2}.",
       portletIndex, GetTagAttributeAsString(DATA_VAR), GetTagAttributeAsString(SCOPE),
       GetTagAttributeAsInt(COLINDEX));
      // 2nd argument is never used in the method
      result.AddInnerHTMLElement(view.DisplaySinglePortlet(PortletColumnConstants.NARROW_ONE.ToInt(),
       portletIndex));
      return result;
    }
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.tags.ATag#Create()"></seealso>
    public override ATag Create()
    {
      return new PortletDisplayTag();
    }
  }
}