package com.bea.alui.taglib.portletpageeditor;

import com.plumtree.openfoundation.util.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

public class FlyoutJSTag extends ATag 
{
	/**
	 * This is the prefix of the name of the javascript object used
	 * to make the flyout work.  It is followed by the flyout ID.
	 */
	public static final String FLYOUT_OBJECT = "PTFlyout";
	/**
	 * This is the name of the javascript method used
	 * to add content to the flyout div.  It is preceded by the FLYOUT_OBJECT and flyout ID.
	 */
	public static final String ADD_CONTENT = "addContent";

	private static final String TAG_NAME = "flyoutjs";
	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final RequiredTagAttribute ATTRIB_FLYOUT_ID;
	public static final RequiredTagAttribute ATTRIB_ONCLICK;
	public static final OptionalTagAttribute ATTRIB_URL;
	public static final OptionalTagAttribute ATTRIB_SPECIFYONCLICKURL;
	public static final OptionalTagAttribute ATTRIB_HEADER_ID;
	public static final OptionalTagAttribute ATTRIB_ANCHOR_ID;
	public static final OptionalTagAttribute ATTRIB_FLYOUT_ANCHOR_TEXT;
	public static final OptionalTagAttribute ATTRIB_FLYIN_ANCHOR_TEXT;
	public static final OptionalTagAttribute ATTRIB_BASENAV_FLYOUT_ID;
	
	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData(TAG_NAME, "Adds all javascript required to create " +
				"a flyout effect on a div.");

		ATTRIB_FLYOUT_ID = PortletPageEditorTagLibraryData.ATTRIB_FLYOUTID;
		
		ATTRIB_ONCLICK = new RequiredTagAttribute("onclick",
				"The name of the onclick function to trigger the flyout effect. " +
				"Works as a toggle to display and hide the flyout.",	
				AttributeType.STRING);
		
		ATTRIB_URL = new OptionalTagAttribute("url",
				"Specifies the URL of the flyout content, retrieved the first time the flyout " +
				"effect is triggered.",	
				AttributeType.STRING, null);
		
		ATTRIB_SPECIFYONCLICKURL = new OptionalTagAttribute("specifyonclickurl",
				"Determines whether to use a passed in url for the onclick function or the default flyout URL " +
				"retrieved the first time the flyout effect is triggered.",	
				AttributeType.BOOLEAN, "false");
		
		ATTRIB_HEADER_ID = new OptionalTagAttribute("headerId",
				"Specifies the ID of the div that will be hidden when the flyout " + 
				"effect occurs (if available). It will reappear when the flyout is hidden.",	
				AttributeType.STRING, null);
		
		ATTRIB_ANCHOR_ID = new OptionalTagAttribute("anchorId",
				"Specifies the ID of the anchor tag for which text will be changed when the flyout " + 
				"effect occurs (if available). It will be changed back when the flyout is hidden.",	
				AttributeType.STRING, null);
		
		ATTRIB_FLYOUT_ANCHOR_TEXT = new OptionalTagAttribute("flyoutAnchorText",
				"Specifies the text of the anchor tag when the flyout " + 
				"effect occurs (if available).",	
				AttributeType.STRING, null);
		
		ATTRIB_FLYIN_ANCHOR_TEXT = new OptionalTagAttribute("flyinAnchorText",
				"Specifies the text of the anchor tag when the flyout " + 
				"is hidden (if available).",	
				AttributeType.STRING, null);
		
		ATTRIB_BASENAV_FLYOUT_ID = new OptionalTagAttribute("basenavFlyoutID",
				"Specifies the ID of the base navigation div that will have the flyout effect. " +
				"The base navigation div will only be used if the div ID specified in the flyoutid" + 
				"attribute is not found on the page.",	
				AttributeType.STRING, null);
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#Create()
	 */
	public ATag Create() {
		return new FlyoutJSTag();
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplayTag()
	 */
	public HTMLElement DisplayTag() {
		return null;
	}
	
	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#SupportsAccessStyle(AccessStyles)
	 */
	public boolean SupportsAccessStyle(AccessStyles _style)
	{
		// DCA - only works on standard mode
		if (AccessStyles.STANDARD.GetAccessID() == _style.GetAccessID()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @see com.plumtree.portaluiinfrastructure.tags.ATag#DisplaySharedJavascript()
	 */
	public HTMLScriptCollection DisplaySharedJavascript()
	{
		String strId = GetTagAttributeAsString(ATTRIB_FLYOUT_ID);
		String strOnclick = GetTagAttributeAsString(ATTRIB_ONCLICK);
		String strHeaderId = GetTagAttributeAsString(ATTRIB_HEADER_ID);
		String strURL = GetTagAttributeAsString(ATTRIB_URL);
		String strBaseNavFlyoutId = GetTagAttributeAsString(ATTRIB_BASENAV_FLYOUT_ID);
		boolean bSpecifyOnclickURL = GetTagAttributeAsBoolean(ATTRIB_SPECIFYONCLICKURL);
		if (null != strURL && XPStringUtility.StartsWith(strURL, "$")) {
			// DCA - URL is a variable, get value
			strURL = (String) GetAnyStateVariable(strURL);
		}
		String strAnchorId = GetTagAttributeAsString(ATTRIB_ANCHOR_ID);
		String strFlyoutAnchorText = GetTagAttributeAsString(ATTRIB_FLYOUT_ANCHOR_TEXT);
		String strFlyinAnchorText = GetTagAttributeAsString(ATTRIB_FLYIN_ANCHOR_TEXT);
		
		XPStringBuilder sb = new XPStringBuilder();
		// DCA - create the flyout object
		sb.Append(FLYOUT_OBJECT).Append(strId).Append(" = new Object();\n");
		sb.Append(FLYOUT_OBJECT).Append(strId).Append(".isOpen = false;\n");
		// wipeIn function begins
		// DCA - wipeIn function expands a div and displays its content
		sb.Append(FLYOUT_OBJECT).Append(strId).Append(".wipeIn = function(divId, bSwap){\n");
		sb.Append("    "+JSHelper.ALIDOJO_OBJECT+".fx.wipeIn({\n");
		sb.Append("	    node: divId, \n");
		sb.Append("		duration: 1000,\n");
		sb.Append("		onEnd: function(){if(bSwap && eval(\"typeof ").Append(FLYOUT_OBJECT).Append(strId)
			.Append(".swapAnchorText == 'function'\")) ").Append(FLYOUT_OBJECT).Append(strId)
			.Append(".swapAnchorText();}\n");
		sb.Append("		}).play();\n");
		sb.Append("}\n");
		// wipeIn function ends
		sb.Append("\n");
		// wipeOut function begins
		// DCA - wipeOut function collapses a div and hides its content
		sb.Append(FLYOUT_OBJECT).Append(strId).Append(".wipeOut = function(divId, bSwap) {\n");
		sb.Append("    "+JSHelper.ALIDOJO_OBJECT+".fx.wipeOut({\n");
		sb.Append("	    node: divId, \n");
		sb.Append("		duration: 1000,\n");
		sb.Append("		onEnd: function(){if(bSwap && eval(\"typeof ").Append(FLYOUT_OBJECT).Append(strId)
			.Append(".swapAnchorText == 'function'\")) ").Append(FLYOUT_OBJECT).Append(strId)
			.Append(".swapAnchorText();}\n");
		sb.Append("		}).play();\n");
		sb.Append("}\n");
		// wipeOut function ends
		sb.Append("\n");
		// DCA - if the flyout DIV is already populated, then there is no strURL
		if (null == strURL || strURL.equals("")) {
			// DCA - don't need to retrieve flyout content only need to toggle between displaying and hiding flyout
			sb.Append(FLYOUT_OBJECT).Append(strId).Append(".").Append(strOnclick).Append(" = function").Append("()\n");
			sb.Append("{\n");
			sb.Append("    if (!").Append(FLYOUT_OBJECT).Append(strId).Append(".isOpen) {\n");
			sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".isOpen = true;\n");
			sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".wipeIn('").Append(strId).Append("', true);\n");
			if (null != strHeaderId && !strHeaderId.equals("")) {
				sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".wipeOut('").Append(strHeaderId).Append("', false);\n");
			}
			sb.Append("    } else {\n");
			sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".wipeOut('").Append(strId).Append("', true);\n");
			if (null != strHeaderId && !strHeaderId.equals("")) {
				sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".wipeIn('").Append(strHeaderId).Append("', false);\n");
			}
			sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".isOpen = false;\n");
			sb.Append("    }\n");
			sb.Append("}\n");
		} else {
			// DCA - need to add javascript to send an ajax request to get flyout content
			sb.Append(FLYOUT_OBJECT).Append(strId).Append(".handleResponse = function(resp)\n"); 
			sb.Append("{ \n");
			// BT - Check whether or not we should display in the basenav flyout div or the normal page flyout div.
			if (strBaseNavFlyoutId != null && !strBaseNavFlyoutId.equals("")) {
				sb.Append("    var flyoutId;\n");
				sb.Append("    if (document.getElementById('" + strId + "')) {\n");
				sb.Append("        flyoutId = '" + strId + "';\n");
				sb.Append("    } else {\n");
				sb.Append("        flyoutId = '" + strBaseNavFlyoutId + "';\n");
				sb.Append("    }\n");
			} else {
				sb.Append("     var flyoutId = '" + strId + "';\n");
			}					
			sb.Append("    if (null != resp) {\n");
			sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".").Append(ADD_CONTENT).Append("(resp);\n");
			sb.Append("    }\n");
			//sb.Append("    ").Append(FLYOUT_OBJECT).Append(strId).Append(".wipeIn('").Append(strId).Append("', true);\n");
			sb.Append("    ").Append(FLYOUT_OBJECT).Append(strId).Append(".wipeIn(flyoutId, true);\n");
			if (null != strHeaderId && !strHeaderId.equals("")) {
				sb.Append("    ").Append(FLYOUT_OBJECT).Append(strId).Append(".wipeOut('").Append(strHeaderId).Append("', false);\n");
			}
			sb.Append("}\n");
			sb.Append("\n");
			// DCA - function that adds the content to the flyout div, use this function when the flyout is first expanded 
			// or when the content needs to be refreshed
			sb.Append(FLYOUT_OBJECT).Append(strId).Append(".").Append(ADD_CONTENT).Append(" = function(resp)\n"); 
			sb.Append("{ \n");
			// BT - Check whether or not we should display in the basenav flyout div or the normal page flyout div.
			if (strBaseNavFlyoutId != null && !strBaseNavFlyoutId.equals("")) {
				sb.Append("    var flyoutId;\n");
				sb.Append("    if (document.getElementById('" + strId + "')) {\n");
				sb.Append("        flyoutId = '" + strId + "';\n");
				sb.Append("    } else {\n");
				sb.Append("        flyoutId = '" + strBaseNavFlyoutId + "';\n");
				sb.Append("    }\n");
			} else {
				sb.Append("     var flyoutId = '" + strId + "';\n");
			}			
			sb.Append("    var div = "+JSHelper.ALIDOJO_OBJECT+".byId(flyoutId);\n");
			sb.Append("    div.innerHTML = resp.responseText;\n");
			sb.Append("    var scripts = div.getElementsByTagName('script');\n");
			sb.Append("    var head = document.getElementsByTagName('head')[0];\n");
			sb.Append("    var i=0;\n");
			sb.Append("    for (i=0; i<scripts.length; i++) {\n");
			sb.Append("    	  var script = scripts[i];\n");
			sb.Append("    	  if (script.innerHTML == '') {\n");
			sb.Append("    	  	var newscript = document.createElement('script');\n");
			sb.Append("    	  	newscript.type = script.type;\n");
			sb.Append("    	  	newscript.src = script.src;\n");
			sb.Append("    	  	head.appendChild(newscript);\n");
			sb.Append("    	  } else {\n");
			sb.Append("    	  	retVal = eval(script.innerHTML);\n");
			sb.Append("    	  }\n");
			sb.Append("    }\n");
			sb.Append("    ").Append(FLYOUT_OBJECT).Append(strId).Append(".cursorClear();\n");
			sb.Append("}\n");
			sb.Append("\n");
			// DCA - onclick function that toggles between displaying or hiding the flyout div
			sb.Append(FLYOUT_OBJECT).Append(strId).Append(".").Append(strOnclick).Append(" = function").Append("(strFlyoutURL)\n");
			sb.Append("{\n");
			// DCA - if it's not open, then get the content of the div and expand the flyout div
			sb.Append("    if (!").Append(FLYOUT_OBJECT).Append(strId).Append(".isOpen) {\n");
			sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".cursorWait();\n");
			sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".isOpen = true;\n");
			// BT - use passed in flyout URL if ATTRIB_SPECIFYONCLICKURL is true so that we can swap in
			// different flyout content in the same div.
			if (bSpecifyOnclickURL) {
				sb.Append("        var request = new PTHTTPGETRequest(").Append("strFlyoutURL").Append(", ").Append(FLYOUT_OBJECT).Append(strId).Append(".handleResponse);\n");
			} else {
				sb.Append("        var request = new PTHTTPGETRequest('").Append(strURL).Append("', ").Append(FLYOUT_OBJECT).Append(strId).Append(".handleResponse);\n");
			}
			sb.Append("        request.invoke();\n");
			sb.Append("    } else {\n");
			// BT - Check whether or not we should display in the basenav flyout div or the normal page flyout div.
			if (strBaseNavFlyoutId != null && !strBaseNavFlyoutId.equals("")) {
				sb.Append("        var flyoutId;\n");
				sb.Append("        if (document.getElementById('" + strId + "')) {\n");
				sb.Append("            flyoutId = '" + strId + "';\n");
				sb.Append("        } else {\n");
				sb.Append("            flyoutId = '" + strBaseNavFlyoutId + "';\n");
				sb.Append("        }\n");
			} else {
				sb.Append("        var flyoutId = '" + strId + "';\n");
			}			
			// DCA - if the flyout div was already expanded, then we want to collpase it
			sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".wipeOut(flyoutId, true);\n");
			// DCA - if we were provided w/ a header div, expand it back to the way it was
			if (null != strHeaderId && !strHeaderId.equals("")) {
				sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".wipeIn('").Append(strHeaderId).Append("', false);\n");
			}
			// DCA - call the rename function in case the page was renamed and to refresh the page
			sb.Append("        if (typeof renamePage == 'function') {\n");
			sb.Append("            ").Append(FLYOUT_OBJECT).Append(strId).Append(".cursorWait();\n");
			sb.Append("            setTimeout('renamePage()',900);\n");
			sb.Append("        }\n");
			sb.Append("        ").Append(FLYOUT_OBJECT).Append(strId).Append(".isOpen = false;\n");
			sb.Append("    }\n");
			sb.Append("}\n");
		}
		// DCA - helper functions that changes the cursor while the flyout div expands or collapses
		sb.Append(FLYOUT_OBJECT).Append(strId).Append(".cursorWait = function() {\n");
		sb.Append("	  document.body.style.cursor = 'wait';\n");
		sb.Append("}\n");
		sb.Append("\n");
		sb.Append(FLYOUT_OBJECT).Append(strId).Append(".cursorClear = function() {\n");
		sb.Append("	  document.body.style.cursor = 'default';\n");
		sb.Append("}\n");

		// DCA - this function switches the text on the link that expands or collapse the flyout div
		if (null != strAnchorId && !strAnchorId.equals("") && null != strFlyoutAnchorText && !strFlyoutAnchorText.equals("") && 
				null != strFlyinAnchorText && !strFlyinAnchorText.equals("")) {
			sb.Append(FLYOUT_OBJECT).Append(strId).Append(".swapAnchorText = function() {\n");
			sb.Append("   var anchor = "+JSHelper.ALIDOJO_OBJECT+".byId('").Append(strAnchorId).Append("');\n");
			sb.Append("   if (anchor) {\n");
			sb.Append("		if (anchor.innerHTML.indexOf('").Append(strFlyinAnchorText).Append("') > -1) {\n");
			sb.Append("			anchor.innerHTML = '").Append(strFlyoutAnchorText).Append("';\n");
			sb.Append("	   	} else { \n");
			sb.Append("			anchor.innerHTML = '").Append(strFlyinAnchorText).Append("';\n");
			sb.Append("	   	}\n");
			sb.Append("	  }\n");
			sb.Append("}\n");
		}
		
		HTMLScript script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
		script.AddInnerHTMLString(sb.ToString());
		HTMLScriptCollection collection = new HTMLScriptCollection();
		collection.AddInnerHTMLElement(script);
		return collection;
	}
}
