package com.bea.alui.taglib.searchpage;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.portalpages.browsing.search.results.*;
import com.plumtree.portalpages.browsing.search.results.assocobjects.*;
import com.plumtree.portaluiinfrastructure.login.*;
import com.plumtree.portaluiinfrastructure.search.*;
import com.plumtree.portaluiinfrastructure.tags.*;
import com.plumtree.portaluiinfrastructure.tags.common.*;
import com.plumtree.portaluiinfrastructure.tags.data.*;
import com.plumtree.portaluiinfrastructure.tags.metadata.*;
import com.plumtree.server.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.helpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.portalpages.browsing.portletpreview.*;
import com.plumtree.portalpages.browsing.directory.*;
import com.plumtree.portalpages.common.folder.*;
import com.plumtree.portalpages.admin.objmgr.*;

/**
 * This class implements the searchpage.searchresultsdata tag.  This tag is for use in the
 * search results page.  It fills in a list of results.
 * 
 * @author smahendr
 */
public class SearchResultsDataTag extends ASearchPageDataTag
{
	public static final String TAG_NAME = "searchresultsdata";
	public static final String SEARCH_ATTRIBUTE_NAME = "search";

	public static final String CUSTOM_SEARCH_MODEL = "custom" + SearchResultModel.STR_MVC_CLASS_NAME;

	public static final String NAME           = "name";
	public static final String DESCRIPTION    = "description";
	public static final String RANK 		  = "rank";
	public static final String RESULT_HREF	  = "resulthref";
	public static final String RESULT_ONCLICK = "resultonclick";
	public static final String RESULT_TARGET  = "resulttarget";
	public static final String ICON 		  = "icon";
	public static final String ICON_ALT_TEXT  = "iconalttext";
	public static final String ICON_HEIGHT    = "iconheight";
	public static final String ICON_WIDTH     = "iconwidth";
	public static final String LAST_MODIFIED  = "lastmodified";
	public static final String TYPE           = "type";
	public static final String STYLECLASS     = "styleclass";

	public static final String PROPERTIES_HREF = "propertieshref";
	public static final String PROPERTIES_ONCLICK = "propertiesonclick";
	public static final String IS_BEST_BET    = "isbestbet";
	public static final String IS_IN_MULTIPLE_FOLDERS = "isinmultiplefolders";

	public static final String FOLDER_PATH = "folderpath";
	public static final String FOLDER_PATH_HREF = "folderpathhref";

	public static final String PROJECT_NAME = "projectname";
	public static final String PROJECT_ONCLICK = "projectonclick";
	
	public static final String LAST_PUBLISHED_BY = "lastpublishedby";
	public static final String ASSOCIATED_OBJECTS_ONCLICK = "associatedobjectsonclick";

	public static final String GROUP_RESULTS_PREFIX = "groupresults";

	public static final String GROUP_ICON_SRC    = "groupiconsrc";
	public static final String GROUP_ICON_ALT    = "groupiconalt";
	public static final String GROUP_ICON_HEIGHT = "groupiconheight";
	public static final String GROUP_ICON_WIDTH  = "groupiconwidth";
	
	public static final String GROUP_TITLE = "grouptitle";
	public static final String GROUP_NUM_DISPLAYED = "groupnumdisplayed";
	public static final String GROUP_TOTAL_RESULTS = "grouptotalresults";
	public static final String GROUP_MORE_URL = "groupmoreurl";
	public static final String GROUP_RESULTS_KEY = "groupresultskey";
	
	public static final String DATATAG_DESCRIPTION = ATag.EncodeForTagDoc(
			"Generates the data needed to show a list of search results, and stores it in memory " +
			"using the variable name specified by the " +
			"<code>" + CommonTagAttributeConstants.ATTRIB_ID_NAME + "</code> attribute. " +
			"Each result is a DataObject with the following variables:<br/>" +
			"<ul>" +
			"<li> <code>" + NAME + "</code> - The name of the result.</li>" +
			"<li> <code>" + DESCRIPTION + "</code> - The description of the result. Not present if there is no description.</li>" +
			"<li> <code>" + RANK + "</code> - The rank of the result.</li>" +
			"<li> <code>" + RESULT_HREF + "</code> - The href of the result link (a friendly URL for portal items," +
					"full gatewayed URL for Collaboration and Publisher items).</li>" +
			"<li> <code>" + RESULT_ONCLICK + "</code> - The onclick handler for result link.  Not present if there is no onclick handler.</li>" +
			"<li> <code>" + RESULT_TARGET + "</code> - The target window name for result link.  Not present if there is no target.</li>" +
			"<li> <code>" + ICON + "</code> - The URL for the icon of the result.</li>" +
			"<li> <code>" + ICON_ALT_TEXT + "</code> - Alternate text for the icon.</li>" +
			"<li> <code>" + ICON_HEIGHT + "</code> - The height of the icon in pixels.</li>" +
			"<li> <code>" + ICON_WIDTH + "</code> - The width of the icon in pixels.</li>" +
			"<li> <code>" + LAST_MODIFIED + "</code> - A string representation (in current locale) of the last modified date.</li>" +
			"<li> <code>" + PROPERTIES_HREF + "</code> - The href of the properties link.  " +
					"Not present if there is no properties link.</li>" +
			"<li> <code>" + PROPERTIES_ONCLICK + "</code> - The onclick handler for the properties link.  " +
					"Not present if there is no properties link.</li>" +
			"<li> <code>" + IS_BEST_BET + "</code> - True if the result is a best bet, and false otherwise.</li>" +
			"<li> <code>" + IS_IN_MULTIPLE_FOLDERS + "</code> - True if the result lives in more than one Knowledge Directory folder.</li>" +
			"<li> <code>" + FOLDER_PATH_HREF + "</code> - The href for the link to the Knowledge Directory folder for this result.  Not present if there is none.</li>" +
			"<li> <code>" + FOLDER_PATH + "</code> - The name of the Knowledge Directory folder for this result.  Not present if there is none.</li>" +
			"<li> <code>" + PROJECT_ONCLICK + "</code> - The onclick handler for the link to the Collaboration project containing this result (for Collaboration results only).  Not present if not a Collaboration result.</li>" +
			"<li> <code>" + PROJECT_NAME + "</code> - The name of the Collaboration project containing this result.  Not present if no name is found.</li>" +
			"<li> <code>" + LAST_PUBLISHED_BY + "</code> - The name of the last publishing user (for Publisher results only). Not present if not a Publisher result.</li>" +
			"<li> <code>" + ASSOCIATED_OBJECTS_ONCLICK + "</code> - The onclick handler for the link to associated objects (for Publisher results only). Not present if not a Publisher result.</li>" +
			"</ul>" +
			"Use tags from the Logic library to iterate over the list and display results. " +
			"This tag only works with the Search Page Adaptive Layout, unless a new query is " +
			"provided using the " + SearchResultsDataTag.SEARCH_ATTRIBUTE_NAME + " attribute.");
	
	public static final String DATATAG_EXAMPLE = ATag.EncodeForTagDoc(
			"This simple example loops through results and shows the rank and name (as a clickable link) for each result.<br/><br/>" +
			  "<code>" +
			  "<pt:" + SearchPageTagLibraryData.LIBRARY.GetName() + "." + TAG_NAME + " pt:id=\"results\"/><br/>" +
			  "<pt:logic.foreach pt:data=\"results\" pt:var=\"result\"><br/>" +
			  "<pt:logic.value pt:value=\"$result." + RANK + "\"/>. " +
			  	"<pt:core.html pt:tag=\"a\" " + "href=\"$result." + RESULT_HREF + "\" onclick=\"$result." + RESULT_ONCLICK +
			  					"\" title=\"$result." + NAME + "\">" +
			  							"<pt:logic.value " + "pt:value=\"$result." + NAME + "\"/>" +
			  	"</pt:core.html><br/><br/>" +
			  "</pt:logic.foreach><br/>" +
			  "</code>"
			  );

	public static final String NEW_QUERY_ONLY = "This attribute only works if " +
		"you are performing a new search using the " + SEARCH_ATTRIBUTE_NAME +
		" attribute of this tag.";

	// All public static final metadata objects should be initialized
	// in a static initializer so that we can control the order of
	// initialization.
	public static final ITagMetaData TAG;
	public static final OptionalTagAttribute SEARCH;
	public static final OptionalTagAttribute SEARCH_CLASS;
//	public static final OptionalTagAttribute SEARCH_PROPS;
	public static final OptionalTagAttribute SEARCH_FOLDERS;
	public static final OptionalTagAttribute SEARCH_SUBFOLDERS;
	public static final OptionalTagAttribute MAX_RESULTS;
	public static final OptionalTagAttribute ORDERING;

	static
	{
		// The TAG member variable needs to be initialized first since it
		// can be used in other tags (i.e. as a RequiredParentTag) that
		// are referenced by member variables of this tag (i.e. as a
		// RelatedChildTag).  This is necessary for tags that have
		// circular references such as a parent / child tag relationship.
		TAG = new TagMetaData(TAG_NAME, DATATAG_DESCRIPTION, DATATAG_EXAMPLE);

		SEARCH = new OptionalTagAttribute(SEARCH_ATTRIBUTE_NAME, "This attribute allows you to perform " +
			"a search query directly in tags without user input. " +
			"Note that adding a search query to the page can significantly decrease performance. " +
			"Caching the search result on the user scope and then only using this tag if the search " +
			"result doesn't already exist can help improve performance.",
			AttributeType.STRING, "");

		SEARCH_CLASS = new OptionalTagAttribute("searchclass", "This attribute allows you " +
			"to search for a particular type of object (i.e. class ID). By default this tag " +
			"will search for all object types. " +
			NEW_QUERY_ONLY,
			AttributeType.INT, "-1");

		// This should be added back in when the search results allow retrieval of custom
		// search properties in the tag data.  Bug#69408
//		SEARCH_PROPS = new OptionalTagAttribute("searchprops", "This attribute allows you " +
//			"to specify additional search properties that will be returned with the search. " +
//			"The properties are specified as ints and can be either a single int (1) or a " +
//			"comma separated list of ints (1,2,3). " +
//			NEW_QUERY_ONLY,
//			AttributeType.STRING, "");

		SEARCH_FOLDERS = new OptionalTagAttribute("searchfolders", "This attribute allows you " +
			"to specify which folders to search. The folder IDs are specified as ints and can " +
			"be either a single int (1) or a comma separated list of ints (1,2,3).  The folders " +
			"will by Document Directory folders if the search class is documents, and admin " +
			"folders otherwise. " +
			NEW_QUERY_ONLY,
			AttributeType.STRING, "");

		SEARCH_SUBFOLDERS = new OptionalTagAttribute("searchsubfolders", "This attribute allows you " +
			"to specify whether or not subfolders should be included in the search. " +
			NEW_QUERY_ONLY,
			AttributeType.BOOLEAN, XPConvert.ToString(Boolean.FALSE));

		MAX_RESULTS = new OptionalTagAttribute("maxresults", "This attribute allows you " +
			"to set the maximum number of results returned by the search query. " +
			NEW_QUERY_ONLY,
			AttributeType.INT, "10");

		ORDERING = new OptionalTagAttribute("ordering", "This attribute allows you " +
			"to order the results by a particular search property.  The possible values include: " +
			"name (" + PT_INTRINSICS.PT_PROPERTY_OBJECTNAME + "), " +
			"description (" + PT_INTRINSICS.PT_PROPERTY_OBJECTSUMMARY + "), " +
			"created (" + PT_INTRINSICS.PT_PROPERTY_OBJECTCREATED + "), " +
			"last modified (" + PT_INTRINSICS.PT_PROPERTY_OBJECTLASTMODIFIED + "), " +
			"document type (" + PT_INTRINSICS.PT_PROPERTY_DOCUMENTTYPEID + ") - only for documents, and " +
			"content language (" + PT_INTRINSICS.PT_PROPERTY_CONTENTLANGUAGE + ") - only for documents. " +
			"Make the ordering value negative to sort in descending order, or positive to " +
			"sort in ascending order. " +
			NEW_QUERY_ONLY,
			AttributeType.INT, XPConvert.ToString(PT_INTRINSICS.PT_PROPERTY_OBJECTNAME));
	}

	@Override
	public ATag Create() {
		return new SearchResultsDataTag();
	}
	
	/**
	 * DisplayTag - Fills in the search results data tag.
	 */
	@Override
	public HTMLElement DisplayTag()
	{
		ISearchResultModelRO model;

		// This tag lets the user perform a custom search query.

		// If the user has a specific search, then they are most likely not
		// coming from a search page, so we will need to load a custom search
		// model and perform the search.
		String strQuery = GetTagAttributeAsString(SEARCH);
		if ((strQuery != null) && (strQuery.length() != 0))
		{
			AActivitySpace space = (AActivitySpace) GetEnvironment();

			// The search model may already be present on the page if we're in a space
			// where we've previously added the model, so we should only add it if it
			// doesn't already exist.
			if (!space.GetModelExists(CUSTOM_SEARCH_MODEL))
			{
				// Add the search model to this space with a new name so that we don't
				// override the existing search model (if we're on a search page).
				space.RegisterModel(SearchResultModel.STR_MVC_CLASS_NAME, CUSTOM_SEARCH_MODEL);
			}

			// load the custom model
			ISearchResultModel fullModel = (ISearchResultModel) space.GetModel(CUSTOM_SEARCH_MODEL);

			// Run the new search.
			fullModel.Search(strQuery, null, null, CreateSearchSettingCollection());

			model = (ISearchResultModelRO) fullModel;
		} else
		{
			try
			{
				// This is not a custom search, so load the default model.
				model = GetSearchResultModel();
			} catch (XPNullPointerException xpnpe)
			{
				// Custom error message since this tag is different than the others.
				throw new XPNullPointerException(GetName() + " tag can only be used in " +
					"the Search Results Page, or when a new query is provided using the " +
					SearchResultsDataTag.SEARCH_ATTRIBUTE_NAME + " attribute.", xpnpe);
			}
		}

		boolean preferPopupDocs = getPopupDocsPreference((AActivitySpace) GetEnvironment());

		Scope tagScope = Scope.GetScope( GetTagAttributeAsString(ATTRIB_SCOPE) );

		// Create array list that will hold data objects for each result.
		IXPList groupedResultsList = new XPArrayList();

	    int numGroups = model.GetNumResultGroups();
	    if (numGroups > 0)
	    {
	        // Grouped view
	        for (int grp = 0; grp < numGroups; grp++)
	        {
	        	HashtableDataObject htGroupObject = new HashtableDataObject();
	        	// Add group info
	        	addGroupHeaderInfoToDataObject(htGroupObject, model, (AActivitySpace) GetEnvironment(), grp);
	        	// Make a list of results, and add a key to the group data object.
		        int numResults = model.GetReturnedResultsInGroup(grp);

				IXPList resultsList = new XPArrayList();
		        // Loop over results in this group
		        for (int j = 0; j < numResults; j++)
		        {
		          int overallRank = model.GetResultRank(grp, j);
		          
		          HashtableDataObject htResultObject = getSingleResult(model, preferPopupDocs, overallRank);
		          resultsList.Add(htResultObject);
		        }
		        String resultsKey = GROUP_RESULTS_PREFIX + XPConvert.ToString(grp);
		        
		        htGroupObject.SetData(GROUP_RESULTS_KEY, resultsKey);
				GetState().SetSharedVariable(resultsKey, resultsList, tagScope, true);

		        groupedResultsList.Add(htGroupObject);
	        }
	    }
	    else
	    {
	        // Ungrouped view - we're going to basically make a single group with all the results, and no group header info.
			int numResults = model.GetNumDisplayableItems();
			IXPList resultsList = new XPArrayList();
			// For each result,
			for (int i = 0; i < numResults; i++)
	        {
				HashtableDataObject htObject = getSingleResult(model, preferPopupDocs, i);
				// Add the data object of the result to the list of results.
				resultsList.Add(htObject);
	        }
			if (!resultsList.IsEmpty()) {
	        	HashtableDataObject htGroupObject = new HashtableDataObject();
		        String resultsKey = GROUP_RESULTS_PREFIX;
		        
		        htGroupObject.SetData(GROUP_RESULTS_KEY, resultsKey);
				GetState().SetSharedVariable(resultsKey, resultsList, tagScope, true);
	
		        groupedResultsList.Add(htGroupObject);
			}
	    }

	    // Add grouped result list to memory
		String strDataReference = GetTagAttributeAsString(ATTRIB_ID);
		GetState().SetSharedVariable(strDataReference, groupedResultsList, tagScope, true);
		
		return ProcessTagBody();
	}

	private HashtableDataObject getSingleResult(ISearchResultModelRO model,
			boolean preferPopupDocs, int i) {
		// Create the data object.
		HashtableDataObject htObject = new HashtableDataObject();

		// **********************************************************************************************
		// The data added for each result replicates the logic in:
		// com.plumtree.portalpages.browsing.search.results.GroupedResultsViewHelper.DisplaySingleResult()
		//
		// That is the method responsible for displaying each result when not using XML page layout mode.
		// **********************************************************************************************
		
		// Add the name
		htObject.SetData(NAME, model.GetResultName(i));

		// Add the rank
		htObject.SetData(RANK, XPConvert.ToString(i + model.GetFirstDisplayableItem()));

		String resultApplication = model.GetResultApp(i);

		// Get the result link, and add href and onclick
		HTMLAnchor resultAnchor = getClickthruAnchor((AActivitySpace) GetEnvironment(), model, i, resultApplication, preferPopupDocs);
		htObject.SetData(RESULT_HREF, resultAnchor.GetHRef());
		htObject.SetData(RESULT_ONCLICK, resultAnchor.GetOnClick());
		htObject.SetData(RESULT_TARGET, resultAnchor.GetTarget());

		// Add best bet flag
		htObject.SetData(IS_BEST_BET, XPConvert.ToString(model.isResultBestBet(i)));

		// Determine if item is in multiple folders.  This is only true when
		// it is a portal card (resultApp == 'PTPORTAL' and classId == '18'), AND
		// the resultFolderID is 0. (if it's non-zero, it is only in one folder)
		boolean bIsCardAndHasMoreThanOneFolder = false;
		if(resultApplication.equals(SearchResultModel.PORTAL_APP) && model.GetResultClassID(i) == PT_CLASSIDS.PT_CATALOGCARD_ID)
			bIsCardAndHasMoreThanOneFolder = (model.GetResultFolderID(i) == 0);

		htObject.SetData(IS_IN_MULTIPLE_FOLDERS, XPConvert.ToString(bIsCardAndHasMoreThanOneFolder));

		// Add the href and onclick for the properties link, if there is one.
		HTMLAnchor propertiesAnchor = model.GetResultPropertiesURL(i, /*popup=*/ true);
		if(propertiesAnchor != null)
		{
			String href=propertiesAnchor.GetHRef();
			htObject.SetData(PROPERTIES_HREF, href);
			htObject.SetData(PROPERTIES_ONCLICK, propertiesAnchor.GetOnClick());
		}

		// Add the result's icon data.
		HTMLImg icon = (HTMLImg)model.GetResultIcon(i);
		if(icon != null)
		{
			setIconData(htObject, icon, resultApplication.equals(SearchResultModel.PORTAL_APP));
		}
		
		String subType = model.GetResultSubType(i);
		htObject.SetData(TYPE, subType);
		String styleClass = model.GetResultStyleClass(subType);
		htObject.SetData(STYLECLASS, styleClass);
		
		// Add the description (KWIC == KeyWords In Context, it's the snippet into the
		// full text content that shows search terms higlighted)
		String kwic = model.GetKWIC(i);
		if ((null != kwic) && (kwic.length() > 0) )
		{
			htObject.SetData(DESCRIPTION, kwic);
		}
		
		// Add last modified date.
		XPDateFormat dateFormat = model.GetDateFormat((AActivitySpace) GetEnvironment());
		htObject.SetData(LAST_MODIFIED, dateFormat.Format(model.GetResultLastModified(i)));

		if(resultApplication.equals(SearchResultModel.PORTAL_APP))
		{
			addPortalFolderAttributes(htObject, (AActivitySpace)GetEnvironment(), model, i, bIsCardAndHasMoreThanOneFolder);
		}
		else if(resultApplication.equals(SearchResultModel.COLLAB_APP))
		{
			addCollabProjectAttributes(htObject, model, i);
		}
		else if(resultApplication.equals(SearchResultModel.CONTENT_APP))
		{
			// Add publisher specific attributes it apptype of result is content server
			addPublisherSpecificAttributes(htObject, model, i);
		}
		return htObject;
	}

	/**
	 * Gets the HTMLAnchor for the click-through link.  We have some special cases that need
	 * to be dealt with.
	 * 
	 * @param asSpace Current Activity space
	 * @param model Search result model
	 * @param resultNum Result number to act on
	 * @param resultApplication Application result comes from
	 * @param preferPopupDocs Flag indicating whether or not results should be in a new window
	 * @return HTMLAnchor for the result
	 */
	private HTMLAnchor getClickthruAnchor(AActivitySpace asSpace, ISearchResultModelRO model, int resultNum,
			String resultApplication, boolean preferPopupDocs)
	{
	    String extraClickthroughParams = null;

	    if (asSpace.GetModelExists(LoggingHelperModel.STR_MVC_CLASS_NAME))
	    {
	      LoggingHelperModel logHelper = (LoggingHelperModel) asSpace.GetModel(LoggingHelperModel.STR_MVC_CLASS_NAME);

	      if (null != logHelper)
	      {
	    	  String loggingID = logHelper.GetLoggingID();
	    	  if(null != loggingID)
	    	  {
	    	      extraClickthroughParams = "qid=" + loggingID + "&rank=" + XPConvert.ToString(resultNum+1);
	    	  }
	      }
	    }

		HTMLAnchor clickthru;
		if(resultApplication.equals(SearchResultModel.TEMPLATING_APP))
		{
	      // Special case: for Community Templating items, clickthrough should go to
	      // a popup with a list of communities on which the associated portlet is
	      // used, as in portlet preview.

              int[] portletIDs = model.GetResultContentPortletIDs(resultNum);

              int portletId = 0;

              // "Templating" items were only indexed by pre-6.0 versions of Content Server
              // (although they could potentially still be in a search index), which would
              // associate only one portlet with each item.  So, we're basically guaranteed
              // to have exactly one item in the portletIDs array, but we might as well
              // be safe:
              if ((null != portletIDs) && (portletIDs.length > 0)) {
                portletId = portletIDs[0];
              }
	
	      ASURL templatingURL = new ASURL(asSpace);
	      templatingURL.SetLinkCreateNewSpace(PreviewPortletAS.STR_MVC_CLASS_NAME, asSpace);
	      templatingURL.SetDisplayPage(InvitePortletJSDP.STR_MVC_CLASS_NAME);
	      templatingURL.SetControl(PreviewPortletControl.STR_MVC_CLASS_NAME);
	      templatingURL.AddControlArgument(PreviewPortletControl.m_PreviewType,
	          PreviewPortletModel.SEARCHHEADER);
	      templatingURL.AddControlArgument(PreviewPortletControl.m_strPortletParam,
	          XPConvert.ToString(portletId));
	      templatingURL.AddControlArgument(PreviewPortletControl.m_strAlwaysCommunity, "y");
	
	      clickthru = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
	      clickthru.SetOnClick("contentPopUp('" +	// mj: removed 'javascript:' from function name
	                           templatingURL.GetURLAsString() + "'); return false;");
	    } 
	    else 
	    {
              // Otherwise, just use the standard clickthrough
              clickthru = model.GetResultClickthroughURL(resultNum, true, false, extraClickthroughParams);
	    }
		
		String absType = model.GetResultAbsType(resultNum);

		// Collab documents always appear in a new window.
		// Other docs appear in a popup if a preference is set (we've already
		// initialized _preferPopupDocs with the value of this pref).

		// If it's a document
		if ((null != absType) && (absType.equals(SearchResultModel.DOCUMENT_ABSTYPE)))
		{
		  if (preferPopupDocs || resultApplication.equals(SearchResultModel.COLLAB_APP))
		  {
			// set target=_blank
			clickthru.SetTarget("_blank");
		  }
		}
		else
		{
		  if (resultApplication.equals(SearchResultModel.COLLAB_APP))
		  {
			// For non-document Collab items, use javascript to open a popup
			// Get the original gatewayed href
			String clickthruURL = clickthru.GetHRef();
			clickthru.SetHRef(ConstructStaticHelpers.BLANK_URL);
			clickthru.SetOnClick("collabPopUp('" + clickthruURL + "'); return false;");
		  }
		}
		return clickthru;
	}
	
	/**
	 * Adds portal folder attributes (for documents and kd folders) to the result data object.
	 * 
	 * @param htObject Data object to add properties to
	 * @param asOwner Current activity space
	 * @param model Search result model
	 * @param resultNum Result number of the result
	 * @param bIsCardAndHasMoreThanOneFolder Flag indicating if the card is in multiple folders
	 */
	private void addPortalFolderAttributes(HashtableDataObject htObject, AActivitySpace asOwner, ISearchResultModelRO model, int resultNum, boolean bIsCardAndHasMoreThanOneFolder)
	{
		int subtype = model.GetResultClassID(resultNum);
		boolean isCard = (subtype == PT_CLASSIDS.PT_CATALOGCARD_ID);
		boolean isDocDirItem = ( isCard || (subtype == PT_CLASSIDS.PT_CATALOGFOLDER_ID));

		if (isDocDirItem)
		{
			String path  = model.GetResultPath(resultNum);
			int folderID = model.GetResultFolderID(resultNum);
	        if(isCard && folderID != 0 && bIsCardAndHasMoreThanOneFolder)
	        	folderID = 0;

	        if ((PT_CLASSIDS.PT_CATALOGFOLDER_ID == subtype) ||
	            (PT_CLASSIDS.PT_ADMIN_FOLDER_ID == subtype))
	        {
	          // for folders, the folder ID for path clickthru is just the object ID
	          folderID = model.GetResultObjectID(resultNum);
	        }

	        if ((null != path) && (path.length() > 0) && (folderID > 0))
	        {
	          path = formatPath(path);

	          ASURL pathURL = new ASURL(asOwner);

	          if (isDocDirItem)
	          {
	            pathURL.SetLinkCreateNewSpace(DirAS.STR_MVC_CLASS_NAME,
	              asOwner);
	            pathURL.SetControl(OpenSubFolderControl.STR_MVC_CLASS_NAME);
	            pathURL.AddControlArgument(OpenSubFolderControl.m_SubFolderID, XPConvert.ToString(folderID));
				pathURL.AddControlArgument(DirConstants.QS_DIRECTORY_MODE, XPConvert.ToString(DirConstants.BROWSE_MODE));
	             
	          }
	          else
	          {
	            pathURL.SetLinkCreateNewSpace(ObjMgrAS.STR_MVC_CLASS_NAME, asOwner);
	            pathURL.SetControl(ObjMgrSettingsControl.STR_MVC_CLASS_NAME);
	            pathURL.AddControlArgument(ObjMgrSettingsControl.FOLDER_ID,
	              XPConvert.ToString(folderID));
	            pathURL.AddControlArgument(ObjMgrSettingsControl.OBJECT_IDS,
	              XPConvert.ToString(subtype));
	          }

	          htObject.SetData(FOLDER_PATH, XPHttpUtility.HtmlEncode(path));
	          htObject.SetData(FOLDER_PATH_HREF, pathURL.GetURLAsString());
	        }
	    }
	}

	/**
	 * Adds collab result specific attributes.  Collaboration server results have
	 * a project name and a link to the project.
	 * 
	 * @param htObject Data object to which to add the attributes
	 * @param model Search result model
	 * @param resultNum Result number
	 */
	private void addCollabProjectAttributes(HashtableDataObject htObject, ISearchResultModelRO model, int resultNum)
	{
		String projectURL = model.GetResultCollabProjectURL(resultNum);
		if (null != projectURL)
		{
	        htObject.SetData(PROJECT_ONCLICK, "collabPopUp('" + projectURL + "'); return false;");

	        String projName = model.GetResultCollabProjectName(resultNum);

	        if (null != projName)
	        {
	        	htObject.SetData(PROJECT_NAME, projName);
	        }
	    }
	}

	/**
	 * Adds publisher specific result attributes to the data object.  Publisher items
	 * (used to be called content server items) have an additional "last published by"
	 * attribute, and a link to associated portlets and communities.
	 * 
	 * @param htObject Data objectto which to add variables
	 * @param model	   The search result model to grab data from
	 * @param resultNum  The result number of the result
	 */
	private void addPublisherSpecificAttributes(HashtableDataObject htObject, ISearchResultModelRO model, int resultNum)
	{
	  // Add the last published by user to the data object
      String pubByUser = model.GetResultContentLastPubUser(resultNum);

      if (null != pubByUser)
      {
    	  htObject.SetData(LAST_PUBLISHED_BY, XPHttpUtility.HtmlEncode(pubByUser) );
      }

      // Check if the publisher item has associated portlets
      int portletIds[] = model.GetResultContentPortletIDs(resultNum);
      if ((null != portletIds) && (0 < portletIds.length))
      {
    	// Create the link to the associated portlets
        HTMLAnchor associatedObjects = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        AActivitySpace asOwner = (AActivitySpace) GetEnvironment();
        ASURL assocObjectsURL = new ASURL(asOwner);
        assocObjectsURL.SetLinkCreateNewSpace(AssociatedObjectsAS.STR_MVC_CLASS_NAME, asOwner);
        assocObjectsURL.SetDisplayPage(AssociatedObjectsPopupDP.STR_MVC_CLASS_NAME);
        assocObjectsURL.SetControl(AssociatedObjectsControl.STR_MVC_CLASS_NAME);
        assocObjectsURL.AddControlArgument(AssociatedObjectsControl.ITEM_NAME_ARGNAME,
                                           model.GetResultName(resultNum));
        // AddControlArguments needs a string array - convert the int array to a string array.
        String[] strPortletIds = new String[portletIds.length];
        for (int j = 0; j < portletIds.length; j++) {
          strPortletIds[j] = XPConvert.ToString(portletIds[j]);
        }
        assocObjectsURL.AddControlArgument(AssociatedObjectsControl.PORTLET_LIST_ARGNAME, strPortletIds);

        associatedObjects.SetOnClick("contentPopUp('" + assocObjectsURL.GetURLAsString() + "'); return false;");
        
  	  	htObject.SetData(ASSOCIATED_OBJECTS_ONCLICK, associatedObjects.GetOnClick() );
      }
	}

	/**
	 * Sets the result icon data in the data object.
	 * 
	 * @param htObject	Data object to which icon and other icon related variables are added
	 * @param icon		The icon used to generate the data.
	 * @param isPortalIcon	Indicates if this is a portal icon or a collab/published icon.
	 */
	private void setIconData(HashtableDataObject htObject, HTMLImg icon, boolean isPortalIcon)
	{
		String iconSource = icon.GetSrc();
		if(isPortalIcon)
		{
			// Generate "http://hostname:port/imageserver/plumtree/" and prepend to icon URL.
			String strImgSrvURL = ConfigService.getInstance().GetImageServerBaseURL((AActivitySpace) GetEnvironment());

			XPStringBuilder sb = new XPStringBuilder();
			sb.Append(strImgSrvURL);
			sb.Append(iconSource);
			
			htObject.SetData(ICON, sb.ToString());
		}
		else
		{
			htObject.SetData(ICON, iconSource);
		}
		htObject.SetData(ICON_ALT_TEXT, icon.GetAlt());
		htObject.SetData(ICON_HEIGHT, icon.GetHeight());
		htObject.SetData(ICON_WIDTH, icon.GetWidth());
	}
	
	/**
	 * Get the preference that tells us if users want clicking on a result to open a new window.
	 * 
	 * @param asOwner Current activity space
	 * @return true if results should be opened in a new window.
	 */
	private boolean getPopupDocsPreference(AActivitySpace asOwner)
	{
		// Initialize _preferPopupDocs from the preference PersonalSettingNames.OPEN_DOC_NEW_WINDOW.
	    // If it's "1", then we want to open docs in a new window.
		Object[][] setting =
			(Object[][]) ((IPTSession) asOwner.GetUserSession()).GetSessionInfo().LookupPreference(PersonalSettingNames.OPEN_DOC_NEW_WINDOW, 0);
		if (null != setting) {
			Object o = setting[1][0];
			if (null != o) {
				if ((XPConvert.ToString(o)).equals("1")) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Helper method to format path string in a friendly way
	 * 
	 * @param path Path to format
	 * @return Path with max size of a 100 characters
	 */
	private String formatPath(String path)
	{
		final int MAX_LENGTH = 100;
	    int len         = path.length();

	    if (len < MAX_LENGTH)
	    {
	      return path;
	    }
	    else
	    {
	      return "... " + path.substring(len - MAX_LENGTH);
	    }
	}
		
	/**
	 * Add group header information to 
	 * @param model
	 * @param groupNum
	 */
	private void addGroupHeaderInfoToDataObject(HashtableDataObject htDataObject, ISearchResultModelRO model, AActivitySpace asOwner, int groupNum)
	{
		int numDisplayed = model.GetReturnedResultsInGroup(groupNum);

		// If there is nothing to display in this group, don't display
		// a header.
		if (0 == numDisplayed)
		{
			return;
		}

		String totalResults = model.GetTotalResultsInGroup(groupNum);
		String groupTitle = XPHttpUtility.HtmlEncode(model.GetGroupName(groupNum));

		boolean allReturned = model.isEntireGroupReturned(groupNum);
		Object groupIcon = model.GetGroupIcon(groupNum);

		if (null != groupIcon)
		{
			HTMLImg icon = (HTMLImg) groupIcon;

			String strImgSrvURL = ConfigService.getInstance().GetImageServerBaseURL((AActivitySpace) GetEnvironment());

			XPStringBuilder sb = new XPStringBuilder();
			sb.Append(strImgSrvURL);
			sb.Append(icon.GetSrc());
	        
			htDataObject.SetData(GROUP_ICON_SRC, sb.ToString());
			htDataObject.SetData(GROUP_ICON_ALT, icon.GetAlt());
			htDataObject.SetData(GROUP_ICON_HEIGHT, icon.GetHeight());
			htDataObject.SetData(GROUP_ICON_WIDTH, icon.GetWidth());
		}
		htDataObject.SetData(GROUP_TITLE, groupTitle);
		htDataObject.SetData(GROUP_NUM_DISPLAYED, XPConvert.ToString(numDisplayed));
		htDataObject.SetData(GROUP_TOTAL_RESULTS, totalResults);

		boolean moreThanOneGroup = (model.GetNumResultGroups() > 1);
		if (!allReturned && moreThanOneGroup)
		{
			HTMLAnchor anchor = (HTMLAnchor)DrilldownControl.makeURL(model, asOwner, "", groupNum);
			htDataObject.SetData(GROUP_MORE_URL, anchor.GetHRef());
		}
	}


	/**
	 * Sets up the search configuration.
	 * 
	 * @return the search settings collection.
	 */
	protected SearchSettingCollection CreateSearchSettingCollection()
	{
		SearchSettingCollection ssc = new SearchSettingCollection();

		// The search property to order the results by.
		int nSearchProperty = GetTagAttributeAsInt(ORDERING);

		ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_ORDERBY, nSearchProperty);

		// This should be added back in when the search results allow retrieval of custom
		// search properties in the tag data.  Bug#69408
		/*
		String searchProps = GetTagAttributeAsString(SEARCH_PROPS);

		int[] nProps;
		if ((searchProps != null) && (searchProps.length() > 0))
		{
			XPStringTokenizer tokens = new XPStringTokenizer(searchProps, ",");

			// Add the custom search properties, then the ordering property.
			nProps = new int[tokens.GetTokenCount() + 1];
			int nIndex = 0;
			while (tokens.HasMoreTokens())
			{
				nProps[nIndex] = XPConvert.ToInteger(tokens.GetNextToken());
				nIndex++;
			}

			// Add the ordering search property by default
			nProps[nIndex] = nSearchProperty;
		} else
		{
			// Just add the ordering property
			nProps = new int[] { nSearchProperty };
		}
*/
		// we also want to make sure that we are getting the property we are ordering
		// by in the search response since it could be searching on a custom object property.
		// The usual properties are always returned.
		ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_RET_PROPS, new int[] { nSearchProperty });

		ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_MAXRESULTS,
			new Integer(GetTagAttributeAsInt(MAX_RESULTS)));

		// By default, PTSearch retrieves objects from the portal, 
		// Collaboration Server, or Content Server.  If we don't want this
		// behavior, we need to change the setting
		// PT_SEARCHSETTING_APPS.
		// ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_APPS, new Integer(PT_SEARCH_APPS.PT_SEARCH_APPS_PORTAL));

		int nClassID = GetTagAttributeAsInt(SEARCH_CLASS);
		// -1 means to search for all class types, so we shouldn't add any restrictions to the search.
		if (nClassID != -1)
		{
			ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_OBJTYPES, new int[] { nClassID });
		}

		String searchFolders = GetTagAttributeAsString(SEARCH_FOLDERS);

		int[] nFolders = null;
		if ((searchFolders != null) && (searchFolders.length() > 0))
		{
			XPStringTokenizer tokens = new XPStringTokenizer(searchFolders, ",");

			// Add the custom search folders.
			nFolders = new int[tokens.GetTokenCount()];
			int nIndex = 0;
			while (tokens.HasMoreTokens())
			{
				nFolders[nIndex] = XPConvert.ToInteger(tokens.GetNextToken());
				nIndex++;
			}
		}

		if (nFolders != null)
		{
			if (nClassID == PT_CLASSIDS.PT_CATALOGCARD_ID)
			{
				ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_DDFOLDERS, nFolders);
			} else
			{
				ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_ADMINFOLDERS, nFolders);			
			}
		}

		ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_INCLUDE_SUBFOLDERS, GetTagAttributeAsBoolean(SEARCH_SUBFOLDERS));

		ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_ACCESS_LEVEL,
			new Integer(PT_ACCESS_LEVELS.PT_ACCESS_LEVEL_READ));

		if (PT_CLASSIDS.PT_GADGET_ID == nClassID)
		{
			ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_GADGET_FIELDS, Boolean.TRUE);
	
			// Only display mypage portlets, not community portlets.
			int[] anGadgetValues;
	
			anGadgetValues = new int[1];
			anGadgetValues[0] = PT_GADGET_TYPES.PT_GADGETTYPE_PERSONAL;
			ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_GADGET_TYPE, anGadgetValues);

			ssc.add(PT_SEARCH_SETTING.PT_SEARCHSETTING_GADGET_SEARCH_TYPE, SearchSettingCollection.GADGET_SEARCH_TYPE_NONCOMMUNITYFOLDER);
		}

		return ssc;
	}

	/*

	private String getErrorString(ISearchResultModelRO model)
	{
		SearchExecutionError error = model.GetError();
	}
	
	  public HTMLElement Display() {

		    HTMLElement c = null;

		    // If the last search failed..
		    if (null != _model.GetError())
		    {
		      // Display nothing.  The error should appear on the page because
		      // ErrorView is embedded in PlumtreeDP.
		      return null;
		    }

		    try {
		      c = _helper.DisplayCollection();
		      if (null == c) {
		        return new HTMLGenericElement("");
		      }
		      
			  _helper.InitDisplayResults();

		      HTMLElement he = null;
		      he = _helper.DisplayHeader();
		      if (null != he) c.AddInnerHTMLElement(he);

		      int numGroups = _model.GetNumResultGroups();

		      if (numGroups > 0) {
		        // Grouped view

		        for (int grp = 0; grp < numGroups; grp++) {
		          he = _helper.DisplayGroupHeader(grp);
		          if (null != he) c.AddInnerHTMLElement(he);

		          int numResults = _model.GetReturnedResultsInGroup(grp);

		          // Loop over results in this group
		          for (int j = 0; j < numResults; j++) {
		            int overallRank = _model.GetResultRank(grp, j);

		            he = _helper.DisplaySingleResult(overallRank, (1 == (j%2)), grp);
		            if (null != he) c.AddInnerHTMLElement(he);
		          }

		          he = _helper.DisplayGroupFooter(grp);
		          if (null != he) c.AddInnerHTMLElement(he);
		        }
		      } else {
		        // Ungrouped view

		        he = _helper.DisplayHeaderIfUngrouped();
		        if (null != he) c.AddInnerHTMLElement(he);

		        int numResults = _model.GetNumDisplayableItems();

		        for (int j = 0; j < numResults; j++) {
		          he = _helper.DisplaySingleResult(j, (1 == (j%2)),0);
		          if (null != he) c.AddInnerHTMLElement(he);
		        }

		        he = _helper.DisplayFooterIfUngrouped();
		        if (null != he) c.AddInnerHTMLElement(he);
		      }

		      he = _helper.DisplayFooter();
		      if (null != he) c.AddInnerHTMLElement(he);
		      
		    } catch (SearchModelException e) {
		      // Just display the exception.
		      
		        log.Error(e, e.getMessage());
		      

		      c = new HTMLGenericElement(e.getMessage());
		    } catch (HTMLException e) {
		      // Same.
		      
		        log.Error(e, e.getMessage());
		      

		      c = new HTMLGenericElement(e.getMessage());
		    }

		    return c;
		  }
*/
}
