using System;
namespace com.plumtree.uiinfrastructure.activityspace
{
  /// <summary>
  /// Constants used as arguments in the query string or in form POSTS.
  /// </summary>
  /// <author>Don Hayler</author>
  public sealed class ASConstants
  {
    // The _INPUT versions of constants are for use as hidden inputs
    // within forms The normal versions are for URLS.
    // All POSTS and GETS must include AActivitySpace information
    
    /// <summary>
    /// space name 
    /// </summary>
    public const String SPACENAME = "space";
    
    /// <summary>
    /// space name 
    /// </summary>
    public const String SPACENAME_INPUT = "in_hi_space";
    
    /// <summary>
    /// Include the space ID if you want to access a specific instance of a space (optional)
    /// If you do not include this, a new space will be created.
    /// </summary>
    public const String SPACEID = "spaceID";
    
    /// <summary>
    /// Include the space ID if you want to access a specific instance of a space (optional)
    /// If you do not include this, a new space will be created.
    /// </summary>
    public const String SPACEID_INPUT = "in_hi_spaceID";
    
    /// <summary>
    /// If this is set to 'true' then the cache will be searched for the named space
    /// and a new one will be created if it cannot be found.  This should not be used
    /// with SPACEID.
    /// </summary>
    public const String CACHED = "cached";
    
    /// <summary>
    /// cached or not 
    /// </summary>
    public const String CACHED_INPUT = "in_hi_cached";
    
    /// <summary>
    /// name of the control 
    /// </summary>
    public const String CONTROL = "control";
    
    /// <summary>
    /// name of the control 
    /// </summary>
    public const String CONTROL_INPUT = "in_hi_control";
    
    /// <summary>
    /// name of the display page 
    /// </summary>
    public const String PAGE = "page";
    
    /// <summary>
    /// name of the display page 
    /// </summary>
    public const String PAGE_INPUT = "in_hi_page";
    
    /// <summary>
    /// subspace key 
    /// </summary>
    public const String SUBSPACE = "subspace";
    
    /// <summary>
    /// subspace key 
    /// </summary>
    public const String SUBSPACE_INPUT = "in_hi_subspace";
    
    /// <summary>
    /// parent space name 
    /// </summary>
    public const String PARENTSPACENAME = "parentname";
    
    /// <summary>
    /// parent space name 
    /// </summary>
    public const String PARENTSPACENAME_INPUT = "in_hi_parentname";
    
    /// <summary>
    /// parent space id 
    /// </summary>
    public const String PARENTSPACEID = "parentid";
    
    /// <summary>
    /// parent space id 
    /// </summary>
    public const String PARENTSPACEID_INPUT = "in_hi_parentid";
    
    /// <summary>
    /// User session ID input 
    /// </summary>
    public const String USERID_INPUT = "in_hi_userid";
    
    /// <summary>
    /// JF- QS param for the URL to redirect to after login 
    /// (usually we don't need this because we do internal redirects but in mode 1, 
    /// we might have to do a 302 redirect to the login page and we don't want to loose
    /// the information on where to redirect after that).
    /// </summary>
    public const String REDIRECT_AFTER_LOGIN = "redirect";
    public const String REDIRECT_AFTER_LOGIN_INPUT = "in_hi_redirect";
    
    /// <summary>
    /// DAP- Secondary QS param for the URL to redirect to after login 
    /// required to handle complex cases involving nested PEI plugins with
    /// custom guest user login pages, or multiple (pseudo) guest user login pages.
    /// </summary>
    public const String REDIRECT_AFTER_LOGIN_SECONDARY = "redirect2";
    
    /// <summary>
    /// This parameter is for use by network monitoring tools and tells
    /// the Interpreter to clean up the HTTPSession after the request
    /// has completed.  This means it will remove everything cached on
    /// the HTTPSession to minimize the impact of network monitoring on
    /// memory usage.  This does not work for gateway requests or requests
    /// that fail in certain ways.
    /// </summary>
    public const String CLEARSESSION = "clearsession";
    
    /// <summary>
    /// Key that the cache object is stored under in the user's sandbox session
    /// AActivitySpace.GetSubSession()
    /// </summary>
    public const String SANDBOXASCACHE = "SandboxASCache";
    
    /// <summary>
    /// In 508 mode the AActivitySpace will handle views differently.  When GetView(name)
    /// is called, the space will first attempt to find the view (name + _D).  It will return
    /// that if it exists, otherwise it will return the normal view, which will be responsible
    /// for displaying 508 HTML.
    /// The _D views will be only for 508 mode.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.constants.AccessStyles.SECTION_508"></seealso>
    public const String STR_508_VIEW_SUFFIX = "_D";
    
    /// <summary>
    /// The full request URL of the previous request is stored using this
    /// key on the sandbox to allow pages to make links back to the previous page.
    /// Whether or not the request was a POST request is also stored.
    /// This information is used in HTMLConstructHelpers.GetPreviousSpaceURL().
    /// </summary>
    public const String PREVIOUS_REQUEST_URL = "PreviousRequestURL";
    public const String PREVIOUS_REQUEST_IS_POST = "PreviousRequestIsPost";
    }
}