using System;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.uiinfrastructure.activityspace
{
  /// <summary>
  /// Stores an Activityspace URL. Should be used in model components when creating URLs.
  /// Can easily be converted to ASURL in the view layer.
  /// </summary>
  /// <author>
  /// robertz
  /// Created on Oct 16, 2002
  /// </author>
  public class ASURLContainer
  {
    private AActivitySpace m_asOwner = null;
    private PTButtonType m_buttonType = null;
    private String m_aspaceID = null;
    private String m_aspaceName = null;
    private String m_ctrlName = null;
    private String m_pageName = null;
    private String m_urlName = null;
    private XPStringBuilder m_buf = null;
    
    /// <summary>
    /// Constructor
    /// </summary>
    public ASURLContainer(String aspaceName, String ctrlName, String pageName,
     AActivitySpace parent)
    {
      m_aspaceName = aspaceName;
      //m_aspaceID		= aspaceID;
      m_ctrlName = ctrlName;
      m_pageName = pageName;
      m_asOwner = parent;
    }
    
    /// <summary>
    /// Constructor
    /// </summary>
    public ASURLContainer(String aspaceName, String ctrlName, AActivitySpace parent)
    {
      m_aspaceName = aspaceName;
      //m_aspaceID		= aspaceID;
      m_ctrlName = ctrlName;
      m_asOwner = parent;
    }
    
    /// <summary>
    /// Constructor
    /// </summary>
    public ASURLContainer(String aspaceName, AActivitySpace parent)
    {
      m_aspaceName = aspaceName;
      //m_aspaceID		= aspaceID;
      m_asOwner = parent;
    }
    
    /// <summary>
    /// Add additional Querystring parameter (one that is
    /// not activityspace, control or page).
    /// </summary>
    public virtual void AddQSPair(String paramName, String paramValue)
    {
      if (null == m_buf)
      {
        m_buf = new XPStringBuilder(20);
      }
      // Make enough space in the buffer for the new name value pair. 
      m_buf.EnsureCapacity(m_buf.GetLength() + paramName.Length + paramValue.Length + 2);
      m_buf.Append("&" + paramName + "=" + paramValue);
    }
    
    /// <summary>
    /// Returns the m_asOwner.
    /// </summary>
    /// <returns>AActivitySpace the parent</returns>
    public virtual AActivitySpace GetASParent()
    {
      return m_asOwner;
    }
    
    /// <summary>
    /// Returns the m_aspaceName.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetASSpace()
    {
      if (null == m_aspaceName)
      {
        return "";
      }
      return m_aspaceName;
    }
    
    /// <summary>
    /// Returns the m_aspaceName.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetASSpaceID()
    {
      if (null == m_aspaceID)
      {
        return "";
      }
      return m_aspaceID;
    }
    
    /// <summary>
    /// Returns the m_ctrlName.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetControl()
    {
      if (null == m_ctrlName)
      {
        return "";
      }
      return m_ctrlName;
    }
    
    /// <summary>
    /// Gets the button type, if there is one
    /// </summary>
    /// <returns>button type</returns>
    public virtual PTButtonType GetImageType()
    {
      return m_buttonType;
    }
    
    /// <returns>
    /// Get all non-common activityspace paramters as a string. Empty string
    /// if nothing is set. Doesn't contain a leading '&'.
    /// </returns>
    public virtual String GetNonASQSPairs()
    {
      if (null == m_buf)
      {
        return "";
      }
      return m_buf.ToString();
    }
    
    /// <summary>
    /// Returns the m_pageName.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetPage()
    {
      if (null == m_pageName)
      {
        return "";
      }
      return m_pageName;
    }
    
    /// <summary>
    /// Pretty messy/inefficent, but coulnd't think of a better way
    /// </summary>
    /// <returns>Value for the given parameter name. null if nothing is found.</returns>
    public virtual String GetQSValue(String paramName)
    {
      String str = m_buf.ToString();
      // Make tokens of every parameter
      XPStringTokenizer tokens = new XPStringTokenizer(str, "&");
      // Look in every token
      for (int i = 0;i < tokens.GetTokenCount();i++)
      {
        String tmp = tokens.GetNextToken();
        if (-1 != XPStringUtility.IndexOf(tmp, paramName + "="))
        {
          // The value is everything after the '=' character.
          return XPStringUtility.Substring(tmp, (XPStringUtility.IndexOf(tmp,
           "=") + 1));
        }
      }
      return null;
    }
    
    /// <returns>
    /// Get the entire URL as a string, including common activityspace parameters.
    /// Doesn't contain a leading '?'.
    /// </returns>
    public virtual String GetURLAsQS()
    {
      XPStringBuilder buf = new XPStringBuilder();
      if (null != m_aspaceName)
      {
        buf.EnsureCapacity(13 + m_aspaceName.Length);
        buf.Append(ASConstants.SPACENAME + "=" + m_aspaceName);
      }
      if (null != m_aspaceID)
      {
        buf.EnsureCapacity(12 + m_aspaceID.Length);
        buf.Append(ASConstants.SPACEID + "=" + m_aspaceID);
      }
      if (null != m_ctrlName)
      {
        buf.EnsureCapacity(16 + m_aspaceName.Length);
        buf.Append("&" + ASConstants.CONTROL + "=" + m_ctrlName);
      }
      if (null != m_pageName)
      {
        buf.EnsureCapacity(12 + m_pageName.Length);
        buf.Append("&" + ASConstants.PAGE + "=" + m_pageName);
      }
      if (null != m_buf)
      {
        m_buf.EnsureCapacity(m_buf.GetLength() + 1);
        buf.Append(m_buf.ToString());
      }
      return buf.ToString();
    }
    
    /// <summary>
    /// Returns the m_urlName.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetURLName()
    {
      if (null == m_urlName)
      {
        return "";
      }
      return m_urlName;
    }
    
    /// <summary>
    /// Sets the m_aspaceName.
    /// </summary>
    /// <param name="aspaceName">The m_aspaceName to set</param>
    public virtual void SetASSpace(String aspaceName)
    {
      this.m_aspaceName = aspaceName;
    }
    
    /// <summary>
    /// Sets the m_aspaceName.
    /// </summary>
    /// <param name="aspaceName">The m_aspaceName to set</param>
    public virtual void SetASSpaceID(String aspaceID)
    {
      this.m_aspaceID = aspaceID;
    }
    
    /// <summary>
    /// Sets the button type
    /// </summary>
    /// <returns>button type</returns>
    public virtual void SetButtonType(PTButtonType type)
    {
      this.m_buttonType = type;
    }
    
    /// <summary>
    /// Sets the m_ctrlName.
    /// </summary>
    /// <param name="ctrlName">The m_ctrlName to set</param>
    public virtual void SetControl(String ctrlName)
    {
      this.m_ctrlName = ctrlName;
    }
    
    /// <summary>
    /// Sets the m_pageName.
    /// </summary>
    /// <param name="pageName">The m_pageName to set</param>
    public virtual void SetPage(String pageName)
    {
      this.m_pageName = pageName;
    }
    
    /// <summary>
    /// Sets the m_urlName.
    /// </summary>
    /// <param name="urlName">The m_urlName to set</param>
    public virtual void SetURLName(String urlName)
    {
      this.m_urlName = urlName;
    }
  }
}