using System;
namespace com.plumtree.uiinfrastructure.activityspace
{
  /// <summary>
  /// Display Pages aggregate Views and put them into an HTMLPage.
  /// They are owned by Activity Spaces and represent a view into an MVC framework.
  /// Display Pages are allowed to get and set data on the Models, if necessary.  Display
  /// Pages are viewed as an extension of Activity Spaces, and therefore are not subject
  /// to the strict data/logic/presentation separation that MVC components are.
  /// </summary>
  /// <author>Don Hayler</author>
  public interface IDisplayPage : IMVCObject
  {
    
    /// <summary>
    /// Construct the HTMLPage for Display.
    /// </summary>
    /// <param name="IWebData">
    /// The HTMLPage for display to the browser should be set on this
    /// object.  This object should not be stored anywhere.
    /// </param>
    void Display(IWebData webData);
    
    /// <summary>
    /// Return the owning Activity Space.
    /// </summary>
    AActivitySpace GetOwner();
    
    /// <summary>
    /// Initialize the Display Page.
    /// </summary>
    /// <param name="AActivitySpace">the parent AActivitySpace of this IDisplayPage.</param>
    void Init(AActivitySpace parent);
  }
}