using System;
namespace com.plumtree.uiinfrastructure.activityspace
{
  /// <summary>
  /// This interface should only be implemented to provide login functionality for
  /// the portal.  This is a special case.
  /// </summary>
  /// <author>Don Hayler</author>
  public interface ILoginControl : IControl
  {
    
    /// <summary>
    /// This method is called by the Interpreter to determine whether or not to
    /// subsequently call GetSession and perform login/logout code.
    /// If this returns false, the session will remain unchanged.
    /// </summary>
    /// <returns>boolean true implies GetSession() will be called</returns>
    bool DoGetSession();
    
    /// <summary>
    /// This method is called by the Interpreter after calling Execute on the
    /// LoginControl.  The User Session returned will be stored on the user's
    /// HTTPSession.  If null is returned, the user's session will be removed
    /// from the HTTPSession for logout.
    /// </summary>
    /// <returns>
    /// Object null implies logoff
    /// session implies login as this session
    /// </returns>
    Object GetSession();
    
    /// <summary>
    /// This method sets a redirect on the login control for use after a
    /// successful login.  This allows a user to be bounced from a page
    /// if they are not logged in, and then return to that page after they
    /// log in.
    /// </summary>
    /// <param name="Redirect">the space the original URL pointed to.</param>
    void SetRedirectForUseAfterLogin(Redirect r);
  }
}