using System;
using com.plumtree.uiinfrastructure.errordisplay;
namespace com.plumtree.uiinfrastructure.activityspace
{
  /// <summary>
  /// Interface for page control validators. Validators are objects for standardized
  /// Page Control validation and contain both server-side and client-side validation. 
  /// </summary>
  /// <author>CaiusV</author>
  public interface IValidator
  {
    
    /// <summary>
    /// Validation method. The Validation method returns a UIError object if the 
    /// object passed in fails to meet the validation criteria. Otherwise it returns
    /// null. Every class the implements IValidator should have a fully implemented
    /// Validate method for server side validation.
    /// </summary>
    /// <param name="_oObject"></param>
    /// <returns>UIError or null</returns>
    UIError Validate(Object _oObject);
    
    /// <summary>
    /// Return a JavaScript snippet for validating client side.
    /// </summary>
    /// <returns></returns>
    String GetJSValidationCode();
    
    /// <summary>
    /// Return the name of the javascript function that validates client side
    /// and arguments supplied by _sParameters.
    /// </summary>
    /// <param name="_sParameters">areguments to Javascript function</param>
    /// <returns></returns>
    String GetJSFunction(String _sParameters);
    
    /// <summary>
    /// Sets the error message for both client & server validation.
    /// </summary>
    /// <param name="_strErrorMessage">the error message</param>
    void SetErrorMessage(String _strErrorMessage);
    String GetName();
  }
}