/*
 * Created on Dec 22, 2003
 */
 
using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.uiinfrastructure.activityspace.utils
{
  /// <summary>
  /// Utility class for client browser type information.
  /// </summary>
  /// <author>DavidP</author>
  /// <author>NaveenP</author>
  public class BrowserType
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.activityspace.utils.BrowserType");
    
    /// <summary>
    /// The browser name specified in the user agent string. 
    /// </summary>
    private String m_strBrowserName = "";
    
    /// <summary>
    /// The browser version specified in the user agent string.  If we
    /// can't parse the version just assume an old browser.
    /// NOTE: This only includes the major and minor versions of the
    /// browser.  I.E. "1.3.1" will be stored as "1.3".
    /// </summary>
    private float m_nBrowserVersion = 1.0F;
    
    /// <summary>
    /// The gecko version specified in a Netscape6/Mozilla user agent string.
    /// </summary>
    private int m_nGeckoVersion = 0;
    
    /// <summary>
    /// The browser platform specified in the user agent string.
    /// </summary>
    private String m_strBrowserPlatform = "unknown";
    
    /// <summary>
    /// Constants used by this class. 
    /// </summary>
    public const String MSIE = "MSIE";
    public const String OPERA = "Opera";
    public const String MOZILLA = "Mozilla";
    public const String NETSCAPE = "Netscape";
    public const String FIREFOX = "Firefox";
    public const String SAFARI = "Safari";
    public const String WINDOWS = "Windows";
    public const String UNIX = "Unix";
    public const String SUNOS = "SunOS";
    public const String LINUX = "Linux";
    public const String MACINTOSH = "Macintosh";
    public const String UNKNOWN = "unknown";
    
    /// <summary>
    /// Constructor
    /// </summary>
    public BrowserType(IXPRequest request) : this(request.GetHeader("User-Agent"))
    {
    }
    
    /// <summary>
    /// Constructor
    /// </summary>
    public BrowserType(String userAgentString)
    {
      try
      {
        if (null == userAgentString)
        {
          m_strBrowserName = UNKNOWN;
          log.Warn("Unable to find the browser\'s name or version number from the User-Agent header. Dropping to lowest supported browser type.");
          return;
        }
        int versionStartIndex = XPStringUtility.IndexOf(userAgentString, "/");
        int versionEndIndex = XPStringUtility.IndexOf(userAgentString, " ");
        // Get the browser name.
        try
        {
          m_strBrowserName = XPStringUtility.Substring(userAgentString, 0,
           versionStartIndex);
        }
        catch (Exception e)
        {
          m_strBrowserName = UNKNOWN;
        }
        // Get the browser version.
        try
        {
          // Not all user agents will have a space in the reported
          // string.
          String agentSubstring = null;
          if (versionEndIndex < 0)
          {
            agentSubstring = XPStringUtility.Substring(userAgentString, versionStartIndex + 1);
            } else
          {
            agentSubstring = XPStringUtility.Substring(userAgentString, versionStartIndex + 1,
             versionEndIndex);
          }
          m_nBrowserVersion = GetVersionNumber(agentSubstring);
        }
        catch (Exception e)
        {
        }
        // Just use the default value.
        // MSIE lies about its name.  Of course...
        if (XPStringUtility.IndexOf(userAgentString, MSIE) != -1)
        {
          // Ex: Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)
          versionStartIndex = (XPStringUtility.IndexOf(userAgentString, MSIE) + MSIE.Length + 1);
          versionEndIndex = XPStringUtility.IndexOf(userAgentString, ";",
           versionStartIndex);
          m_strBrowserName = MSIE;
          try
          {
            m_nBrowserVersion = GetVersionNumber(XPStringUtility.Substring(userAgentString,
             versionStartIndex, versionEndIndex));
          }
          catch (Exception e)
          {
          }
        }
        // Just use the default value.
        // Netscape 6+ and Mozilla have a host of complexities, and
        // browser version isn't the whole story here...
        // A Netscape 6+ user-agent string looks something like this:
        //
        // Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.0.1) Gecko/20020823 Netscape/7.0
        //
        // Netscape 6 & Netscape 7 are both classified as Mozilla/5.0
        // But technically the codebase underneath them is the Gecko build number.
        // In this case, Gecko/20020823 means that this was built off of code from
        // the Gecko project on 8/23/2002.
        //
        // Netscape versions    Gecko versions    Mozilla versions
        // -----------------    --------------    ----------------
        // 6.0                  20001108          0.6
        // 6.0.1                20010131
        // 6.1                  20010726          0.9.1
        // 6.2                  20011019          0.9.4
        // 6.2.1                20011128          0.9.4.1
        // 6.2.2                20020314          0.9.4.2
        // 6.2.3                20020508          0.9.4.3
        // 7.0 (Mac)            20020730          1.0.1 (fictional, used by Netscape 7.0 & 7.1 Mac browsers)
        // 7.0 (Win)            20020823
        //                      20020826          1.1
        // 7.1                  20030624		  1.4
        //
        // Patrick Corcoran <patrickc@plumtree.com>   12/19/2003
        if (XPStringUtility.IndexOf(userAgentString, NETSCAPE) != -1)
        {
          m_strBrowserName = NETSCAPE;
          // Netscape example: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.0.1) Gecko/20020823 Netscape/7.0
          versionStartIndex = (XPStringUtility.IndexOf(userAgentString, NETSCAPE) + NETSCAPE.Length + 1);
          versionEndIndex = XPStringUtility.IndexOf(userAgentString, " ",
           versionStartIndex);
          if (versionEndIndex == -1)
          {
            versionEndIndex = userAgentString.Length;
          }
          try
          {
            m_nBrowserVersion = GetVersionNumber(XPStringUtility.Substring(userAgentString,
             versionStartIndex, versionEndIndex));
          }
          catch (Exception e)
          {
          }
        }
        // Just use the default value.
        if (XPStringUtility.IndexOf(userAgentString, FIREFOX) != -1)
        {
          m_strBrowserName = FIREFOX;
          // Firefox example: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.7.5) Gecko/20041107 Firefox/1.0
          versionStartIndex = (XPStringUtility.IndexOf(userAgentString, FIREFOX) + FIREFOX.Length + 1);
          versionEndIndex = XPStringUtility.IndexOf(userAgentString, " ",
           versionStartIndex);
          if (versionEndIndex == -1)
          {
            versionEndIndex = userAgentString.Length;
          }
          try
          {
            m_nBrowserVersion = GetVersionNumber(XPStringUtility.Substring(userAgentString,
             versionStartIndex, versionEndIndex));
          }
          catch (Exception e)
          {
          }
        }
        // Just use the default value.
        // Opera isn't completely honest, either...
        // Modificaton by Chris Mospaw <mospaw@polk-county.com>
        if (XPStringUtility.IndexOf(userAgentString, OPERA) != -1)
        {
          //Ex: Mozilla/4.0 (Windows NT 4.0;US) Opera 3.61  [en]
          versionStartIndex = (XPStringUtility.IndexOf(userAgentString, OPERA) + OPERA.Length + 1);
          versionEndIndex = XPStringUtility.IndexOf(userAgentString, " ",
           versionStartIndex);
          m_strBrowserName = OPERA;
          try
          {
            m_nBrowserVersion = GetVersionNumber(XPStringUtility.Substring(userAgentString,
             versionStartIndex, versionEndIndex));
          }
          catch (Exception e)
          {
          }
        }
        // Just use the default value.
        // Safari
        if (XPStringUtility.IndexOf(userAgentString, SAFARI) != -1)
        {
          // Ex: 
          // Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en-us) AppleWebKit/XX (KHTML, like Gecko) Safari/YY
          // Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/312.5.1 (KHTML, like Gecko) Safari/312.3.1
          // NOTE: Safari can have non-float version numbers such as 312.3.1
          versionStartIndex = (XPStringUtility.IndexOf(userAgentString, SAFARI) + SAFARI.Length + 1);
          m_strBrowserName = SAFARI;
          try
          {
            float browserBuild = GetVersionNumber(XPStringUtility.Substring(userAgentString,
             versionStartIndex));
            if (browserBuild < 100)
            {
              m_nBrowserVersion = 1.0F;
            } else
            if (browserBuild >= 100)
            {
              m_nBrowserVersion = 1.1F;
            }
          }
          catch (Exception e)
          {
          }
        }
        // Just use the default value
        // Try to figure out what platform.
        if ((XPStringUtility.IndexOf(userAgentString, "Windows") != -1) || (XPStringUtility.IndexOf(userAgentString,
         "WinNT") != -1) || (XPStringUtility.IndexOf(userAgentString, "Win98") != -1) || (XPStringUtility.IndexOf(userAgentString,
         "Win95") != -1))
        {
          m_strBrowserPlatform = WINDOWS;
        }
        if (XPStringUtility.IndexOf(userAgentString, "Mac") != -1)
        {
          m_strBrowserPlatform = MACINTOSH;
        }
        if (XPStringUtility.IndexOf(userAgentString, "X11") != -1)
        {
          if (XPStringUtility.IndexOf(userAgentString, SUNOS) != -1)
          {
            m_strBrowserPlatform = SUNOS;
          } else
          if (XPStringUtility.IndexOf(userAgentString, LINUX) != -1)
          {
            m_strBrowserPlatform = LINUX;
          } else
          {
            m_strBrowserPlatform = UNIX;
          }
        }
      }
      catch (Exception e)
      {
        log.Error(e, "Unable to parse browser information to determine " + "browser type, version, and platform.");
        throw new XPException("Unable to parse browser information to determine " + "browser type, version, and platform.",
         e);
      }
    }
    
    /// <summary>
    /// This helper function attempts to convert the user agent
    /// string into a float major/minor version number.
    /// </summary>
    /// <param name="_strAgent">The browser user agent string.</param>
    /// <returns>The major/minor version number as a float.</returns>
    /// <throws>
    /// XPFormatException If the agent string cannot be
    /// converted to a float.
    /// </throws>
    protected virtual float GetVersionNumber(String _strAgent)
    {
      int nFirstDecimal = XPStringUtility.IndexOf(_strAgent, '.');
      int nLastDecimal = XPStringUtility.LastIndexOf(_strAgent, '.');
      // Some agent substrings may have more than just major and minor version numbers.
      // For instance, Safari 1.3.1
      if (nFirstDecimal != nLastDecimal)
      {
        // Remove the last decimal and try again
        return GetVersionNumber(XPStringUtility.Substring(_strAgent, 0, nLastDecimal));
        }
      return XPConvert.ToFloat(_strAgent);
    }
    
    /// <returns>true if the current client browser can handle javascript, else false</returns>
    public virtual bool IsECMAScriptOK()
    {
      if (IsIE() && (GetBrowserVersion() >= 5.0F))
      {
        return true;
      } else
      if ((IsFirefox() || IsMozilla()) && (GetBrowserVersion() >= 1.0F))
      {
        return true;
      } else
      if (IsNetscape() && (GetBrowserVersion() >= 7.0F))
      {
        return true;
      } else
      if (IsOpera() && (GetBrowserVersion() >= 7.0F))
      {
        return true;
      }
      return false;
    }
    
    /// <returns>true if the current client browser can handle javascript, else false</returns>
    public virtual bool IsJSPortalMenuCompliant()
    {
      if ((IsNetscape() && (GetBrowserVersion() >= 4.0F)) || (IsIE() && !IsMac()) || IsFirefox() || IsMozilla() || (IsSafari() && GetBrowserVersion() >= 2.0F))
      {
        return true;
      } else
      {
        return false;
      }
    }
    
    /// <returns>true if the current client browser can handle javascript, else false</returns>
    public virtual bool IsJSPortletCompliant()
    {
      if (IsIE() || IsNetscape() || IsFirefox() || IsMozilla())
      {
        return true;
      } else
      {
        return false;
      }
    }
    
    /// <returns>true if the current client browser can handle javascript, else false</returns>
    public virtual bool IsDragDropLayoutCompliant()
    {
      if ((IsIE() || IsNetscape4()) && !IsMac())
      {
        return true;
      } else
      {
        return false;
      }
    }
    
    /// <returns>true if the current client browser is an Internet Explorer browser, else false</returns>
    public virtual bool IsIE()
    {
      return MSIE == m_strBrowserName;
    }
    
    /// <returns>true if the current client browser is a Safari browser, else false</returns>
    public virtual bool IsSafari()
    {
      return SAFARI == m_strBrowserName;
    }
    
    /// <returns>true if the current client browser is a Firefox browser, else false</returns>
    public virtual bool IsFirefox()
    {
      return FIREFOX == m_strBrowserName;
    }
    
    /// <returns>true if the current client browser is a Mozilla browser, else false</returns>
    public virtual bool IsMozilla()
    {
      return MOZILLA == m_strBrowserName;
    }
    
    /// <returns>true if the current client browser is a Mozilla browser, else false</returns>
    public virtual bool IsOpera()
    {
      return OPERA == m_strBrowserName;
    }
    
    /// <returns>true if the current client browser is a Netscape browser, else false</returns>
    public virtual bool IsNetscape()
    {
      return NETSCAPE == m_strBrowserName;
    }
    
    /// <returns>true if the current client browser is a Netscape 4.x browser, else false</returns>
    public virtual bool IsNetscape4()
    {
      return IsMozilla() && (m_nBrowserVersion < 5.0);
    }
    
    /// <returns>true if the current client browser is a Netscape 6.x or 7.x browser, else false</returns>
    public virtual bool IsNetscape67()
    {
      return IsNetscape() && (m_nBrowserVersion > 6.0);
    }
    
    /// <returns>true if the current client browser is a browser for Windows OS, else false</returns>
    public virtual bool IsWindows()
    {
      return WINDOWS == m_strBrowserPlatform;
    }
    
    /// <returns>true if the current client browser is a browser for Mac OS, else false</returns>
    public virtual bool IsMac()
    {
      return MACINTOSH == m_strBrowserPlatform;
    }
    
    /// <returns>true if the current client browser is running on Solaris</returns>
    public virtual bool IsSunOS()
    {
      return SUNOS == m_strBrowserName;
    }
    
    /// <returns>the browser's name</returns>
    public virtual String GetBrowserName()
    {
      return m_strBrowserName;
    }
    
    /// <summary>
    /// NOTE: This only includes the major and minor versions of the
    /// browser.  I.E. "1.3.1" will be returned as "1.3".
    /// </summary>
    /// <returns>the browser's version number, returns 1.0 if it was not found</returns>
    public virtual float GetBrowserVersion()
    {
      return m_nBrowserVersion;
    }
  }
}