using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.io;
using com.plumtree.openfoundation.util;
using com.plumtree.openkernel.config;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.application.notification;
using com.plumtree.uiinfrastructure.application.startup;
// TODO: For some reason these 3 exceptions need to be imported individually
// rather than with a '.*' for the .NET build.
using com.plumtree.uiinfrastructure.application.startup.exceptions;
using ConfigVarPackException=com.plumtree.uiinfrastructure.application.startup.exceptions.ConfigVarPackException;
using LibHomePathException=com.plumtree.uiinfrastructure.application.startup.exceptions.LibHomePathException;
using WebHomeFromConfigException=com.plumtree.uiinfrastructure.application.startup.exceptions.WebHomeFromConfigException;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.diagnostic;
using com.plumtree.uiinfrastructure.diagnostic.reporter;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.opener.types;
using com.plumtree.uiinfrastructure.pei;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.tempfile;
using com.plumtree.uiinfrastructure.util;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.xpshared.jsutil;
namespace com.plumtree.uiinfrastructure.application
{
  /// <summary>
  /// This class provides the common jumpable initialization code for both 
  /// the servlet and .aspx code to call.  The initialization can be split
  /// into a core section, and a load section.  The core method should be
  /// run only once, while the load methods can be called multiple times,
  /// to support extensible features.
  /// </summary>
  /// <author>John Osborne</author>
  /// <author>Don Hayler</author>
  /// <author>Ricky Wong</author>
  /// <author>David Phipps</author>
  public class AppWarmUp
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.AppWarmUp");
    
    /// <summary>
    /// Hashtable that stores all of the settings gathered from
    /// initialization.  It is static because we need it to be 
    /// updateable by all the static methods of this class.  It 
    /// was private to prevent tampering of the data, but is now
    /// public to allow unit testing. 
    /// </summary>
    public static XPHashtable s_htInitSettings = new XPHashtable();
    // general files constants.
    public const String SUFFIX_XML_FILE = ".xml";
    public const String SUFFIX_WAR_FILE = ".war";
    public static readonly String PATH_SEPARATOR = XPFileSystemObject.GetPathSeparator();
    // OpenKernel initialization members
    public static IOKContext m_configContext;
    public static IOKComponent m_portalComponent;
    // standard JAVA/.NET folder names.
    private const String FOLDER_JAVA_LIB = "lib";
    private const String FOLDER_DOTNET_LIB = "bin";
    private const String FOLDER_JAVA_SUBLIB = "java";
    private const String FOLDER_DOTNET_SUBLIB = "assemblies";
    // developer kits that are supported.
    public const String STR_SUPPORTED_DOTNET_SDK = "1.1";
    public const String STR_SUPPORTED_JAVA_JDK_1 = "1.3";
    public const String STR_SUPPORTED_JAVA_JDK_2 = "1.4";
    public const String STR_SUPPORTED_JAVA_JDK_3 = "1.5";
    public const String STR_SUPPORTED_JAVA_JDK_6 = "1.6";
    // folders used by the Portal.
    public const String FOLDER_I18N = "i18n";
    public const String FOLDER_PORTAL = "portal";
    public const String FOLDER_SETTINGS = "settings";
    public const String FOLDER_DYNAMIC_LOADS = "dynamicloads";
    // public static final String FOLDER_OK_SETTINGS = "openkernel";
    public const String FOLDER_COMMON = "common";
    // configuration files used by the Portal.
    public const String FILE_SERVER_CONFIG_XML = "serverconfig.xml";
    public const String FILE_PORTAL_CONFIG_XML = "portalconfig.xml";
    // WilliamA - move AppWarmUpHelper.xml to use OpenConfig consolidated file
    // public static final String FILE_APP_WARMUP_HELPER_XML = "AppWarmUpHelper.xml";
    public const String FILE_VAR_PACKS_XML = "VarPacks.xml";
    public const String FILE_ACTIVITY_SPACES_XML = "ActivitySpaces.xml";
    public const String FILE_CUSTOM_VAR_PACKS_XML = "CustomVarPacks.xml";
    public const String FILE_CUSTOM_ACTIVITY_SPACES_XML = "CustomActivitySpaces.xml";
    // parameters used in pthome.xml
    // DAP note: we'd like to retire this in favor of the app name passed 
    // in via web.xml, to reduce the number of instances 
    public const String PARAM_PTHOME_PRODUCT_NAME = "portal";
    // DAP new env variable used for pthome location
    public const String PARAM_PTHOME_ENVVAR_NAME = "PT_HOME";
    // parameters used in AppWarmUpHelper.xml.
    public const String PARAM_APPWARMUPHELPER_LIBFILE = "libfile";
    public const String PARAM_APPWARMUPHELPER_NAME = "name";
    // key names used to store settings in the Application's CacheManager.
    public const String KEY_CACHEMAN_IMAGE_SERVER_HELPER = "ImageServerHelper";
    public const String KEY_CACHEMAN_RESOURCE_MANAGER = "ResourceManager";
    public const String KEY_CACHEMAN_JS_REGISTRY = "JSRegistry";
    public const String KEY_CACHEMAN_SECURE_JS_REGISTRY = "ptSecureJSReg";
    public const String KEY_CACHEMAN_CLASS_TYPE_MANAGER = "ClassDescMgr";
    // key names used to store settings in s_htInitSettings.
    public const String KEY_APP_WARMUP_HELPER = "AppWarmUpHelper";
    public const String KEY_BASE_CONFIG_VARPACK = "BaseConfigVarPack";
    //public static final String KEY_FILE_SERVER_CONFIG = "FileServerConfig"; DAP no longer used
    //public static final String KEY_FILE_APP_WARMUP = "FileAppWarmup"; DAP no longer used
    public const String KEY_FILE_VAR_PACKS = "FileVarPacks";
    public const String KEY_FILE_ACTIVITY_SPACES = "FileActivitySpaces";
    public const String KEY_FILE_CUSTOM_VAR_PACKS = "FileCustomVarPacks";
    public const String KEY_FILE_CUSTOM_ACTIVITY_SPACES = "FileCustomActivitySpaces";
    public const String KEY_PATH_PORTAL_HOME = "PathPortalHome";
    public const String KEY_PATH_PORTAL_SETTINGS = "PathPortalSettings";
    public const String KEY_PATH_PORTAL_CONFIG = "PathPortalConfig";
    public const String KEY_PATH_PORTAL_I18N = "PathPortalI18N";
    public const String KEY_PATH_PORTAL_DYNAMIC_LOADS = "PathPortalDynamicLoads";
    public const String KEY_PATH_WEB_HOME = "PathWebHome";
    public const String KEY_PATH_LIB_HOME = "PathLibHome";
    public const String KEY_STR_APPLICATION_NAME = "ApplicationName";
    public const String KEY_STR_DEV_KIT_VERSION = "DevKitVersion";
    public const String KEY_STR_PLATFORM = "Platform";
    public const String KEY_STR_IMAGESERVER_BASE_URL = "ImageServerBaseURL";
    public const String KEY_STR_SECURE_IMAGESERVER_BASE_URL = "SecureImageServerBaseURL";
    public const String KEY_STR_IMAGESERVER_CONNECTION_URL = "ImageServerConnectionURL";
    
    /// <summary>
    /// This counter is used in the diagnostic page. 
    /// </summary>
    public const String APP_KEY_NUM_CUSTOM_VARPACKS = "NumCustomVarPacks";
    
    /// <summary>
    /// Startup notification class member, for diagnostic tool. 
    /// </summary>
    private static IStartupNotification m_warmupNotify = null;
    private static bool m_isDiagnosticTool = true;
    private const String HALTREQUEST = "Initialization halt";
    
    /// <summary>
    /// This is the first method any tool should call that wants to know if startup 
    /// is successful.  It sets a callback class used by the startup for notification.
    /// Calling this is optional.
    /// </summary>
    public static void setNotification(IStartupNotification notify)
    {
      m_warmupNotify = notify;
    }
    
    /// <summary>
    /// This is the main method used to initialize the Portal Application.
    /// There are three steps to getting the Portal up and running.  First,
    /// we perform any preinitialization checks.  If the checks succeed, we
    /// initialize our Portal.  When initialization is complete, we finally
    /// load any customized objects that we have.  The difference between
    /// the initialization and loading steps is that the loading steps can 
    /// be invoked again for reloading purposes.  The initializations steps
    /// are not designed to be rerun.
    /// </summary>
    /// <param name="_strApplicationName">Name to give to this Application.</param>
    public static void Init(String _strApplicationName)
    {
      bool breturn = false;
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "Startup successful",
       null);
      try
      {
        // save the application name.
        s_htInitSettings.PutElement(KEY_STR_APPLICATION_NAME, _strApplicationName);
        // initialization steps. note that some steps must be 
        // run before others.  for example you cannot run the
        // config helper step without first running the image
        // server helper step.
        // Register any callback reporters for startup tasks
        // If nobody has registered, then register self, for reporting on the main page
        if (m_warmupNotify == null)
        {
          DiagnosticRunner.create();
          DiagnosticRunner.instance().init();
          m_isDiagnosticTool = false;
        }
        // Run the startup tasks
        // Note: review application.startup.* to see all the tasks
        // The diagnostic tool ties into the startup tasks via a notification interface
        retVal = RunStartupTasks(_strApplicationName);
        if (retVal.getResultCode().getID() <= PT_STARTUP_RESULTCODES.WARN.getID())
        {
          // the interpreter checks this for each request.
          ASManager.INSTANCE.m_bSystemStartupSuccesful = true;
          XPSystem.Out.WriteLine("Aqualogic Interaction startup successful.");
          log.Action("Aqualogic Interaction startup successful.");
          breturn = true;
        } else
        {
          // the interpreter checks this for each request.
          ASManager.INSTANCE.m_bSystemStartupSuccesful = false;
          ASManager.INSTANCE.m_StartupFailure = retVal.getException();
          if (retVal.getException() != null)
          {
            // Output the exception to console.
            // rob: this doenst jump
            //retVal.getException().printStackTrace(System.out);
            retVal.getException().PrintStackTrace();
          }
          GenerateFailureErrorReport(null);
        }
      }
      catch (Exception e)
      {
        GenerateFailureErrorReport(e);
      }
      finally
      {
        if (m_warmupNotify != null)
        {
          try
          {
            m_warmupNotify.runEnd(retVal);
          }
          catch (Exception e)
          {
            Reporter.LogAndWriteError("Failed at Run End: " + XPException.GetInstance(e).GetMessage());
            }
        }
      }
    }
    
    /// <summary>
    /// Handle exception in startup, and general failure messages.
    /// Also handles really intensely bad errors like path missing, jars missing, dlls missing and so forth.
    /// This is for XPExceptions and Throwables caught by the standard runner.
    /// </summary>
    /// <param name="e"></param>
    public static void GenerateFailureErrorReport(Exception e)
    {
      ReturnValue retVal = null;
      // the interpreter checks this for each request.
      ASManager.INSTANCE.m_bSystemStartupSuccesful = false;
      if (e != null)
      {
        ASManager.INSTANCE.m_StartupFailure = XPException.GetInstance(e);
        }
      // report the error
      String platformName = Platform.getCurPlatform().getName();
      if (e != null)
      {
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, "Primary Startup Failure",
         XPException.GetInstance(e));
        Reporter.logEvent(99, 0, "MasterTask", platformName, "Unexpected Error",
         "Uncaught exception while running startup tasks", RecommendationHelper.ConvertExceptionToRecommendation(XPException.GetInstance(e)),
         retVal);
      }
      // Generate report; this is diag page
      EventLog[] errLogs = AppWarmUp.getStartupNotificationErrorLogs();
      String[] logReport = AppWarmUp.getStartupNotificationErrorReport();
      bool bErrDisplayed = false;
      if (errLogs != null && errLogs.Length > 0)
      {
        EventLog primaryErr = errLogs[0];
        if ((primaryErr != null) && (primaryErr.taskName != null))
        {
          Reporter.LogAndWriteFatal("Task \'" + primaryErr.taskName + "\' Failed on Startup");
          Reporter.LogAndWriteError("    Task #" + (primaryErr.taskID + 1) + " (" + primaryErr.taskName + ") Step #" + (primaryErr.taskStep + 1));
          Reporter.LogAndWriteError("--------------------------------------------------------------------------------");
          Reporter.LogAndWriteError("error:     " + primaryErr.errorMessage);
          Reporter.LogAndWriteError("platform:  " + primaryErr.platformName);
          // if there is a recommendation
          String exceptionStr = "";
          String messageStr = "";
          String secondLine = "";
          XPException ex = primaryErr.retVal.getException();
          if (ex != null)
          {
            exceptionStr = RecommendationHelper.ConvertExceptionToShortString(ex,
             false, false);
            messageStr = RecommendationHelper.ConvertExceptionToShortString(ex,
             true, false);
            secondLine = RecommendationHelper.ConvertExceptionToShortString(ex,
             false, true);
          }
          Reporter.LogAndWriteError("failure:   " + primaryErr.retVal.getResultCode().getName() + ": " + primaryErr.errorMessage);
          if (primaryErr.errorDescription.Length > Reporter.LINEWIDTH - 11)
          {
            Reporter.SystemReport(primaryErr.errorDescription, 75, 11, "recommendation: ");
            } else
          {
            Reporter.LogAndWriteError("recommendation: " + primaryErr.errorDescription);
            }
          Reporter.LogAndWriteError("exception: " + exceptionStr);
          Reporter.SystemReport(primaryErr.exceptionRecommendation, 75, 11,
           null);
          if (messageStr.Length > 1)
          {
            Reporter.SystemReport(messageStr, 75, 11, "message: ");
          }
          if (secondLine.Length > 1)
          {
            Reporter.SystemReport(secondLine, 75, 11, "secondary exception: ");
            }
          Reporter.LogAndWriteError("--------------------------------------------------------------------------------");
          bErrDisplayed = true;
        }
      }
      // Handle pointer errors in the error display itself
      if (!bErrDisplayed)
      {
        Reporter.LogAndWriteFatal("Task \'MasterTask\' Failed on Startup");
        Reporter.LogAndWriteError("Task #0 (runner) Step #0");
        Reporter.LogAndWriteError("--------------------------------------------------------------------------------");
        Reporter.LogAndWriteError("error:          unknown");
        Reporter.LogAndWriteError("platform:       " + platformName);
        Reporter.LogAndWriteError("failure:        unknown");
        Reporter.LogAndWriteError("recommendation: (contact plumtree support)");
        Reporter.LogAndWriteError("exception:      none");
        Reporter.LogAndWriteError("--------------------------------------------------------------------------------");
        }
      Reporter.LogAndWriteFatal("FAILED TO INITIALIZE SYSTEM. YOU CANNOT PROGRESS BEYOND THIS POINT.");
      }
    
    /// <summary>
    /// Get the OpenConfig context for use by OpenKernel services.
    /// </summary>
    public static IOKContext GetOKConfigContext()
    {
      return m_configContext;
    }
    
    /// <summary>
    /// Gets the error report from the startup noitication service,  Identical to the
    /// information generated by the diagnostic tool.
    /// </summary>
    /// <returns>String array with report</returns>
    public static String[] getStartupNotificationErrorReport()
    {
      if (m_warmupNotify == null)
      {
        return null;
      }
      return m_warmupNotify.getReport();
    }
    
    /// <summary>
    /// Gets the error report from the startup noitication service,  Identical to the
    /// information generated by the diagnostic tool.
    /// </summary>
    /// <returns>EventLog array with report</returns>
    public static EventLog[] getStartupNotificationErrorLogs()
    {
      if (m_warmupNotify == null)
      {
        return null;
      }
      return m_warmupNotify.getReportEventLogs();
    }
    
    /// <summary>
    /// Gets the error report from the startup noitication service,  Identical to the
    /// information generated by the diagnostic tool.
    /// </summary>
    /// <returns>String array with report</returns>
    public static void setStartupNotificationErrorLogs(EventLog[] eventVector,
     XPArrayList reportLog)
    {
      if (m_warmupNotify == null)
      {
        return;
      }
      m_warmupNotify.setReportEventLogs(eventVector, reportLog);
    }
    
    /// <summary>
    /// Helper that loads VarPacks from a config file.
    /// </summary>
    /// <param name="strVarPackFile"></param>
    /// <param name="strLibHomePath"></param>
    /// <param name="strConfigPath"></param>
    /// <param name="application"></param>
    /// <param name="bOverWrite">true implies that the Custom Object Counter will be incremented for these classes.</param>
    public static ReturnValue HelperLoadVarPacks(String strVarPackFile, String strLibHomePath,
     String strConfigPath, IApplication application, bool bOverWrite)
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "HelperLoadVarPacks successful.",
       StackTracer.MakeXPException("(internal stacktrace)"));
      try
      {
        VarPackManager vpm_VarPackManager = application.GetVarPackManager();
        log.Info("Attempting to load VarPacks specified in file " + strVarPackFile + " from library " + strLibHomePath + " with config path " + strConfigPath + ".");
        // Load VarPacks
        Object[] varPacks = XPDynamicDiscovery.GetInstancesFromXML(strVarPackFile,
         strLibHomePath);
        int nNumCustomVarPacks = 0;
        if (varPacks != null)
        {
          IVarPack varPack;
          String strVarPackID = null;
          // Initialize the VarPacks
          for (int index = 0;index < varPacks.Length;index++)
          {
            try
            {
              varPack = (IVarPack) varPacks[index];
              varPack.LoadVarPack(strConfigPath);
              strVarPackID = varPack.GetVarPackID();
              if (!vpm_VarPackManager.HasVariablePackage(strVarPackID))
              {
                // always add if it doesn't exist.
                vpm_VarPackManager.AddVariablePackage(varPack);
              } else
              {
                if (bOverWrite)
                {
                  // if it does exist, overwrite only if specified.
                  vpm_VarPackManager.AddVariablePackage(varPack);
                }
              }
              log.Debug("Loaded VarPack " + strVarPackID);
              if (bOverWrite)
              {
                // Increment the number of custom VarPacks loaded.
                nNumCustomVarPacks++;
              }
              strVarPackID = null;
            }
            catch (Exception e)
            {
              String msg = "Unable to load VarPack " + strVarPackID + " specified in file " + strVarPackFile + " from library " + strLibHomePath + ".";
              log.Fatal(e, msg);
              retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg,
               XPException.GetInstance(msg, e));
              return retVal;
            }
          }
        } else
        {
          // If Dynamic Discovery returned null, it means an error or some sort occurred.
          // Either the file or library could not be found, or a class failed to instantiate.
          String msg = "Application unable to load VarPack specified in file " + strVarPackFile + " from library " + strLibHomePath + ".";
          log.Fatal(msg);
          retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg, new XPException(msg));
          return retVal;
        }
        if (bOverWrite)
        {
          // Cache the number of custom varpacks loaded on the application
          application.GetCachingManager().SetEntry(AppWarmUp.APP_KEY_NUM_CUSTOM_VARPACKS,
           nNumCustomVarPacks);
        }
      }
      catch (Exception e)
      {
        String msg = "HelperLoadVarPacks: Unable to load VarPack.";
        log.Fatal(e, msg);
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg, XPException.GetInstance(msg,
         e));
        return retVal;
      }
      return retVal;
    }
    
    /// <summary>
    /// Helper that tests our language files.
    /// </summary>
    /// <param name="_strPortalResourceFolder"></param>
    public static ReturnValue HelperTestLanguages(String _strPortalResourceFolder)
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "LoadObjects successful.",
       StackTracer.MakeXPException("(internal stacktrace)"));
      try
      {
        // We are assuming the debug logging is on if this method was called at all.
        log.Debug("Attempting to run language file unit test.");
        XPDirectory dirMsgsRoot = new XPDirectory(_strPortalResourceFolder);
        if (!dirMsgsRoot.Exists())
        {
          String msg = "HelperTestLanguages: ***** Resource folder does not exist: " + _strPortalResourceFolder + ". *****";
          log.Error(msg);
          retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg, new XPFileNotFoundException(msg));
          return retVal;
        }
        log.Debug("Using resource folder base: " + _strPortalResourceFolder + ".");
        XPFileSystemObject[] fsoLangDirs = dirMsgsRoot.GetChildren();
        int nNumLangs = fsoLangDirs.Length;
        for (int i = 0;i < nNumLangs;i++)
        {
          String strLangPath = fsoLangDirs[i].GetFullPath();
          // We're only interested in directories, not .xml and other files.
          if (fsoLangDirs[i].IsDirectory())
          {
            XPDirectory dirLangDir = new XPDirectory(strLangPath);
            if (null == dirLangDir)
            {
              String msg = "***** Resource folder does not exist: " + strLangPath + ". *****";
              log.Error(msg);
              retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg,
               new XPFileNotFoundException(msg));
              return retVal;
            }
            if (!dirLangDir.Exists())
            {
              String msg = "***** Resource folder does not exist: " + strLangPath + ". *****";
              log.Error(msg);
              retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg,
               new XPFileNotFoundException(msg));
              return retVal;
            }
            log.Debug("Using resource file language folder: " + strLangPath + ".");
            XPFileSystemObject[] fsoResourceFiles = dirLangDir.GetChildren();
            int nNumResourceFiles = fsoResourceFiles.Length;
            for (int j = 0;j < nNumResourceFiles;j++)
            {
              String strResourceFile = fsoResourceFiles[j].GetFullPath();
              log.Debug("Loading... " + strResourceFile + ".");
              try
              {
                XPPropertyBag xpPropBag = new XPPropertyBag();
                xpPropBag.LoadFromFile(strResourceFile);
              }
              catch (Exception e)
              {
                String msg = "***** Message file failed to load: " + strResourceFile + ". *****";
                log.Error(e, msg);
                retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg,
                 XPException.GetInstance(msg, e));
                return retVal;
              }
            }
          } else
          {
            log.Debug("Ignoring... " + strLangPath + ". It is not a directory.");
            }
        }
        XPResourceManager rm = null;
        try
        {
          rm = XPResourceManager.CreateResourceManager(_strPortalResourceFolder);
          rm.LoadResources();
        }
        catch (Exception e)
        {
          String msg = "***** The multi-threaded resource file loader failed. Please see the exception. *****";
          log.Error(e, msg);
          retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg, XPException.GetInstance(msg,
           e));
          return retVal;
        }
        String[] arrSupportedLangs = rm.GetSupportedLanguages();
        if (arrSupportedLangs != null)
        {
          for (int k = 0;k < arrSupportedLangs.Length;k++)
          {
            String strTestString = "";
            try
            {
              strTestString = rm.GetString(0, "ptmsgs_portalbrowsingmsgs",
               arrSupportedLangs[k]);
            }
            catch (Exception e)
            {
              String msg = "***** Error loading resources. The folder: " + arrSupportedLangs[k] + " is problematic. *****";
              log.Error(e, msg);
              retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg,
               XPException.GetInstance(msg, e));
              return retVal;
            }
            log.Debug("Supported Language: " + arrSupportedLangs[k] + ". Sample test string: " + strTestString + ".");
            }
        }
      }
      catch (Exception e)
      {
        String msg = "HelperTestLanguages failed.";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg, XPException.GetInstance(msg,
         e));
      }
      return retVal;
    }
    
    /// <summary>
    /// Helper that determines some of the more complex paths.
    /// The WebHome and LibHome paths are constructed here.
    /// </summary>
    public static ReturnValue HelperDeterminePaths()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "HelperLoadSettings successful.",
       StackTracer.MakeXPException("(internal stacktrace)"));
      try
      {
        //////////////////////////////////
        // LOAD THE BASE CONFIG VARPACK //
        //////////////////////////////////
        ConfigVarPack vpBaseConfig = null;
        String configPath = null;
        String strLibHome = null;
        try
        {
          log.Info("Loading BaseConfigVarPack.");
          // create ConfigVarPack for specific platform.
          vpBaseConfig = new ConfigVarPack(XPConvert.ToString(s_htInitSettings.GetElement(KEY_STR_PLATFORM)));
          // load the content from the config file.
          configPath = XPConvert.ToString(XPConvert.ToString(s_htInitSettings.GetElement(KEY_PATH_PORTAL_CONFIG)));
          vpBaseConfig.LoadVarPack(XPConvert.ToString(XPConvert.ToString(s_htInitSettings.GetElement(KEY_PATH_PORTAL_CONFIG))));
          s_htInitSettings.PutElement(KEY_BASE_CONFIG_VARPACK, vpBaseConfig);
          log.Info("BaseConfigVarPack successfully loaded.");
        }
        catch (Exception ex)
        {
          String msg = "BaseConfigVarPack is badly formed or cannot be found at path: " + configPath + " error: ";
          retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg, XPException.GetInstance(msg,
           ex));
          return retVal;
        }
        ///////////////////////////////////
        // DETERMINE WEBHOME FROM CONFIG //
        ///////////////////////////////////
        String strWebHome = null;
        try
        {
          log.Info("Determining WebHome.");
          // retrieve webhome setting from base config.
          strWebHome = XPStringUtility.Trim(vpBaseConfig.GetConfigValueAsString("MainURLs",
           "WebHome", "value"));
          if (!XPStringUtility.EndsWith(XPStringUtility.ToLowerCase(strWebHome),
           SUFFIX_WAR_FILE))
          {
            // if webhome is not pointing to a WAR, then it's pointing 
            // to a path.  make sure it has a separator at the end.
            strWebHome = GeneralHelpers.FormatPath(strWebHome, PATH_SEPARATOR,
             true);
          }
          // save this path for later use.
          s_htInitSettings.PutElement(KEY_PATH_WEB_HOME, strWebHome);
        }
        catch (Exception ex)
        {
          String msg = "Error retrieving WebHome from ConfigVarPack or computing its location.  Path: " + strWebHome + " error: ";
          retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg, XPException.GetInstance(msg,
           ex));
          return retVal;
        }
        ////////////////////////////////////
        // DETERMINE LIBHOME FROM WEBHOME //
        ////////////////////////////////////
        try
        {
          log.Info("Determining LibHome.");
          if (XPStringUtility.EndsWith(XPStringUtility.ToLowerCase(strWebHome),
           SUFFIX_WAR_FILE))
          {
            // if webhome is a war file, then the libraries 
            // are there.  point libhome to webhome.
            strLibHome = strWebHome;
          } else
          {
            // since webhome is a path, try to smartly detect where libhome is.
            // given webhome, determine whether there is a standard library 
            // folder structure.  if so, use that.  if not, determine if there
            // is a web app folder structure.  if so, use that.  if not, use 
            // the webhome path as it is and assume that the libs are in there.
            String strStandardLibPath;
            String strWebAppLibPath;
            if (XPStringUtility.EqualsIgnoreCase(XPConvert.ToString(s_htInitSettings.GetElement(KEY_STR_PLATFORM)),
             ConfigVarPack.APP_PLATFORM_DOT_NET))
            {
              // the standard .NET folder structure.
              strStandardLibPath = FOLDER_DOTNET_LIB + PATH_SEPARATOR + FOLDER_DOTNET_SUBLIB;
              // the .NET web app folder structure.
              strWebAppLibPath = FOLDER_DOTNET_LIB;
            } else
            {
              // the standard JAVA folder structure.
              strStandardLibPath = FOLDER_JAVA_LIB + PATH_SEPARATOR + FOLDER_JAVA_SUBLIB;
              // the JAVA web app folder structure.
              strWebAppLibPath = FOLDER_JAVA_LIB;
            }
            // determine if the standard library folder exists.
            bool bStandardLibPathExists = new XPDirectory(strWebHome + strStandardLibPath + PATH_SEPARATOR).Exists();
            // determine if the web app folder structure exists.
            bool bWebAppLibPathExists = new XPDirectory(strWebHome + strWebAppLibPath + PATH_SEPARATOR).Exists();
            if (bStandardLibPathExists)
            {
              // if webhome contains the standard library folder 
              // structure, i.e. "lib/java" or "bin/assemblies",
              // then our libraries should be there.
              log.Info("Detected standard library folder structure.");
              strLibHome = strWebHome + strStandardLibPath + PATH_SEPARATOR;
              } else
            if (bWebAppLibPathExists)
            {
              // if webhome contains the standard webapp folder
              // structure, i.e. "lib" or "bin" and does not have
              // the standard library structure, this means that
              // the libraries have been expanded.
              log.Info("Detected expanded WebApp folder structure.");
              strLibHome = strWebHome + strWebAppLibPath + PATH_SEPARATOR;
              } else
            {
              // if webhome does not contain any known folder
              // structure, then assume that webhome already
              // points to the library folder.
              strLibHome = strWebHome;
            }
          }
          // save this path for later use.
          s_htInitSettings.PutElement(KEY_PATH_LIB_HOME, strLibHome);
        }
        catch (Exception ex)
        {
          String msg = "Error retrieving LibHome from WebHome. Path: " + strLibHome + " Error: ";
          retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg, XPException.GetInstance(msg,
           ex));
          return retVal;
        }
      }
      catch (Exception e)
      {
        String msg = "HelperDeterminePaths: Error forming valid paths to webhome and libhome.";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg, XPException.GetInstance(msg,
         e));
        return retVal;
      }
      return retVal;
    }
    
    /// <summary>
    /// Helper to return the setting value from the settings hashtable.
    /// </summary>
    /// <param name="strKey">Key for the setting value to retrieve.</param>
    /// <returns>Value for the corresponding key.</returns>
    public static Object GetSettingValue(String strKey)
    {
      return s_htInitSettings.GetElement(strKey);
    }
    
    /// <summary>
    /// Helper to return the setting value from the settings hashtable.
    /// </summary>
    /// <param name="strKey">Key for the setting value to retrieve.</param>
    /// <returns>Value for the corresponding key.</returns>
    public static String GetSettingValueAsString(String strKey)
    {
      return XPConvert.ToString(GetSettingValue(strKey));
    }
    
    /// <summary>
    /// Execute the startup tasks defined in PTStartupTasks.
    /// Note that the handling of each return value is delegated to 
    /// the notification factory.  The default handling is to continue 
    /// until a fatal error is encountered.  
    /// </summary>
    public static ReturnValue RunStartupTasks(String _strApplicationName)
    {
      ReturnValue retVal = null;
      try
      {
        // Notification
        int numTasks = PTStartupTasks.LIST.Length;
        if (m_warmupNotify != null)
        {
          bool bContinue = m_warmupNotify.runStart(numTasks);
          if (!bContinue)
          {
            // Notification subscriber orders us to not start up
            String strMsg = HALTREQUEST;
            throw new XPException(strMsg);
          }
        }
        // Preflight and init the tasks to determine how many tasks + steps
        retVal = LoopStartupTasks(_strApplicationName, true);
        if (retVal.getResultCode().getID() > PT_STARTUP_RESULTCODES.WARN.getID())
        {
          return retVal;
        }
      }
      catch (XPException e)
      {
        // Notification
        String strMsg = "FAIL initializing a startup task: " + e.GetMessage();
        // Rethrow the exception using instancing for proper exception chaining
        Reporter.LogAndWriteError(strMsg);
        throw XPException.GetInstance(strMsg, e);
      }
      try
      {
        // Execute all the startup tasks
        retVal = LoopStartupTasks(_strApplicationName, false);
      }
      catch (XPException e)
      {
        // Notification
        String strMsg = "FAIL executing a startup task: " + e.GetMessage();
        // Rethrow the exception using instancing for proper exception chaining
        Reporter.LogAndWriteError(strMsg);
        throw XPException.GetInstance(strMsg, e);
      }
      return retVal;
    }
    
    /// <summary>
    /// Actual executor for the startup tasks; handles the concept of
    /// "preflight", which the progress indicator uses to estimate
    /// the number of tasks and task-steps to run for an accurate
    /// representation of true progress.
    /// This *is* the new application warmup.  Each warmup step is 
    /// managed through this looping mechanism.
    /// </summary>
    /// <param name="bPreflightForProgress"></param>
    public static ReturnValue LoopStartupTasks(String _strApplicationName,
     bool bPreflightForProgress)
    {
      // Get the number of tasks
      int numTasks = PTStartupTasks.LIST.Length;
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "All startup tasks completed",
       null);
      ReturnValue initRet = null;
      ReturnValue stepRet = null;
      bool bContinue = true;
      // Loop through each task
      ReturnValue overallTaskRetVal = retVal;
      for (int taskLoop = 0;taskLoop < numTasks;++taskLoop)
      {
        // Get the task
        IStartupTask thisTask = PTStartupTasks.LIST[taskLoop].getTask();
        // Notification: task only 'starts' when preflight complete       
        if ((m_warmupNotify != null) && (!bPreflightForProgress))
        {
          bContinue = m_warmupNotify.taskBegin(taskLoop, thisTask.getTaskName(),
           thisTask.getTaskID());
          if (!bContinue)
          {
            String strMsg = HALTREQUEST;
            throw new XPException(strMsg);
          }
        }
        // Handle task initialization
        initRet = thisTask.init(_strApplicationName);
        // Report on task init status: call notification factory
        // This also updates progress
        if ((m_warmupNotify != null) && (bPreflightForProgress))
        {
          bContinue = m_warmupNotify.taskInitComplete(taskLoop, thisTask.getTaskName(),
           thisTask.getTaskID(), thisTask.getNumSteps(), initRet, (IMessage) thisTask);
          if (!bContinue)
          {
            String strMsg = HALTREQUEST;
            throw new XPException(strMsg);
          }
        }
        // Handle task init return value
        if ((initRet.getResultCode() == PT_STARTUP_RESULTCODES.OK) || (initRet.getResultCode() == PT_STARTUP_RESULTCODES.WARN))
        {
          // Determine the number of steps in this task
          int numSteps = thisTask.getNumSteps();
          // proceed with actual startup task; this is not preflight
          if (!bPreflightForProgress)
          {
            // Iterate through and execute each step
            // Record the worst retval you hit in a step and use that for the 
            // task return value
            overallTaskRetVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,
             "Task " + thisTask.getTaskName() + " completed", null);
            for (int stepLoop = 0;stepLoop < numSteps;++stepLoop)
            {
              // Notification
              if (m_warmupNotify != null)
              {
                bContinue = m_warmupNotify.taskStepBegin(taskLoop, thisTask.getTaskName(),
                 thisTask.getTaskID(), stepLoop);
                if (!bContinue)
                {
                  break;
                }
              }
              // Execute step
              stepRet = thisTask.executeStep(stepLoop);
              // Record worst task retval (id of retval: larger is worse)
              if (stepRet.getResultCode().getID() > overallTaskRetVal.getResultCode().getID())
              {
                overallTaskRetVal = stepRet;
              }
              // Notification
              if (m_warmupNotify != null)
              {
                bContinue = m_warmupNotify.taskStepEnd(taskLoop, thisTask.getTaskName(),
                 thisTask.getTaskID(), stepLoop, stepRet, (IMessage) thisTask);
                if (!bContinue)
                {
                  return stepRet;
                }
              }
              // Handle task step return value
              // On errors, break out of task
              // On fatal, bail out entirely and halt
              if (stepRet.getResultCode() == PT_STARTUP_RESULTCODES.ERROR)
              {
                String strMsg = "Startup Task: " + thisTask.getTaskName() + " Step #" + (stepLoop).ToString() + " Error: " + stepRet.getMsg();
                Reporter.LogAndWriteError(strMsg);
                break;
              }
              if (stepRet.getResultCode() == PT_STARTUP_RESULTCODES.FATAL)
              {
                String strMsg = "Startup Task: " + thisTask.getTaskName() + " Step #" + (stepLoop).ToString() + " Error: " + stepRet.getMsg();
                Reporter.LogAndWriteError(strMsg);
                return stepRet;
              }
            }
            // task end notification
            if (m_warmupNotify != null)
            {
              bContinue = m_warmupNotify.taskEnd(taskLoop, thisTask.getTaskName(),
               thisTask.getTaskID(), overallTaskRetVal, (IMessage) thisTask);
              if (!bContinue)
              {
                return overallTaskRetVal;
              }
            }
          }
        } else
        {
          // Initialization failed!
          // On fatal, bail out entirely and halt
          // Preflight or not we're dead 
          if (initRet.getResultCode() == PT_STARTUP_RESULTCODES.FATAL)
          {
            String strMsg = "Startup Task Initialization: " + thisTask.getTaskName() + " Fatal Error: " + stepRet.getMsg();
            Reporter.LogAndWriteError(strMsg);
          }
          return initRet;
        }
      }
      return overallTaskRetVal;
    }
  }
}