/*
 * Purpose:
 * Author: 	D. Phipps Mar 22, 2005
 */
 
using System;
using XPArrayList=com.plumtree.openfoundation.util.XPArrayList;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.diagnostic.reporter;
namespace com.plumtree.uiinfrastructure.application.notification
{
  /// <author>DavidP</author>
  public interface IStartupNotification
  {
    
    /// <summary>
    /// Called at the beginning of the startup process.
    /// </summary>
    /// <param name="number">of tasks to execute overall</param>
    /// <returns>true to continue; false to halt immediately</returns>
    bool runStart(int numTasks);
    
    /// <summary>
    /// Called when the startup process completes.  Always called, via finally.
    /// </summary>
    void runEnd(ReturnValue initRet);
    
    /// <summary>
    /// Called at the beginning of the task.  Simple start notification.
    /// </summary>
    /// <param name="taskLoop"></param>
    /// <param name="taskName"></param>
    /// <param name="taskID"></param>
    /// <returns>true to continue; false to halt immediately</returns>
    bool taskBegin(int taskLoop, String taskName, int taskID);
    
    /// <summary>
    /// Called once task initialization is complete.  Returns the number of 
    /// steps in this task.  The ReturnValue contains any errors or exceptions
    /// that may have occurred during initialization.
    /// </summary>
    /// <param name="taskLoop"></param>
    /// <param name="taskName"></param>
    /// <param name="taskID"></param>
    /// <param name="totalNumSteps"></param>
    /// <param name="initRet"></param>
    /// <returns>true to continue; false to halt immediately</returns>
    bool taskInitComplete(int taskLoop, String taskName, int taskID, int totalNumSteps,
     ReturnValue initRet, IMessage msgs);
    
    /// <summary>
    /// Called before a specific task step begins.  Simply a notifier.
    /// </summary>
    /// <param name="taskLoop"></param>
    /// <param name="taskName"></param>
    /// <param name="taskID"></param>
    /// <param name="stepLoop"></param>
    /// <returns>true to continue; false to halt immediately</returns>
    bool taskStepBegin(int taskLoop, String taskName, int taskID, int stepLoop);
    
    /// <summary>
    /// Called at the end of an individual tasks' step.  Always called, with a finally.
    /// Exceptions and error return values in the ReturnValue object.
    /// </summary>
    /// <param name="taskLoop"></param>
    /// <param name="taskName"></param>
    /// <param name="taskID"></param>
    /// <param name="stepLoop"></param>
    /// <param name="initRet"></param>
    /// <returns>true to continue; false to halt immediately</returns>
    bool taskStepEnd(int taskLoop, String taskName, int taskID, int stepLoop,
     ReturnValue initRet, IMessage msgs);
    
    /// <summary>
    /// Called at the end of a specific task.  Always called, with a finally.
    /// The ReturnValue here contains the final result of that task: the integer
    /// result code holds what the systemn feels the final result is.  Notice that 
    /// an individual step could fail with error, but the overall task simply 
    /// show a warning. 
    /// </summary>
    /// <param name="taskLoop"></param>
    /// <param name="taskName"></param>
    /// <param name="taskID"></param>
    /// <param name="initRet"></param>
    /// <returns>true to continue; false to halt immediately</returns>
    bool taskEnd(int taskLoop, String taskName, int taskID, ReturnValue initRet,
     IMessage msgs);
    
    /// <summary>
    /// Used to retrieve a comprehensive report of startup errors from the run.  
    /// </summary>
    /// <returns>Ordered array of strings that form a comprehensive startup report</returns>
    String[] getReport();
    
    /// <summary>
    /// Used to retrieve a comprehensive report of startup errors from the run.  
    /// </summary>
    /// <returns>
    /// Ordered array of eventlogs that form a comprehensive startup 
    /// ERROR report
    /// </returns>
    EventLog[] getReportEventLogs();
    
    /// <summary>
    /// Used to override generated results.  Use this to leverage the diagnostic page
    /// for non-diagnostic uses such as special error handling
    /// </summary>
    /// <param name="EventLog[]">
    /// eventVector: Ordered array of eventlogs that form a comprehensive startup 
    /// ERROR report
    /// </param>
    void setReportEventLogs(EventLog[] eventVector, XPArrayList reportLog);
    }
}