/*
 * Created on Mar 29, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
using XPConvert=com.plumtree.openfoundation.util.XPConvert;
using XPException=com.plumtree.openfoundation.util.XPException;
using OpenLogService=com.plumtree.openlog.OpenLogService;
using OpenLogger=com.plumtree.openlog.OpenLogger;
using AppWarmUp=com.plumtree.uiinfrastructure.application.AppWarmUp;
using IStartupTask=com.plumtree.uiinfrastructure.application.notification.IStartupTask;
using PT_STARTUP_RESULTCODES=com.plumtree.uiinfrastructure.application.notification.PT_STARTUP_RESULTCODES;
using ReturnValue=com.plumtree.uiinfrastructure.application.notification.ReturnValue;
using com.plumtree.uiinfrastructure.application.startup.exceptions;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.util;
using ApplicationManager=com.plumtree.uiinfrastructure.web.ApplicationManager;
namespace com.plumtree.uiinfrastructure.application.startup
{
  /// <summary>
  /// Initializes the Application.  After this point the CachingManager
  /// and VarPackManager will be available, as well as the Application.
  /// </summary>
  /// <author>DavidP</author>
  public class InitializeApplication : IStartupTask
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.startup.InitializeApplication");
    private String m_applicationName = "";
    // array of messages
    private static StartupTaskMessage[] m_diag = {new StartupTaskMessage(0,
     Platform.ALL, PT_STARTUP_RESULTCODES.FATAL, "Core startup initialization error.  Cannot proceed.",
     "RECOMMENDATION: The portal libraries are not available on this machine " + "in the expected location.  Check your PT_HOME and PORTAL_HOME environment " + "variables.  Did you install the full portal on this machine?  This tool is " + "for verifying full portal startup.")};
    public InitializeApplication() : base()
    {
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
     
    public virtual String getTaskName()
    {
      return "InitializeApplication";
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
     
    public virtual int getTaskID()
    {
      return PTStartupTasks.INITAPPLICATION.getOrdinal();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
     
    public virtual int getNumSteps()
    {
      return 1;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
     
    public virtual ReturnValue init(String _strApplicationName)
    {
      m_applicationName = _strApplicationName;
      return new ReturnValue(PT_STARTUP_RESULTCODES.OK, getTaskName() + " init successful",
       null);
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
     
    public virtual ReturnValue executeStep(int sequentialStepID)
    {
      String strApplicationName = "";
      String strLibHome = "";
      try
      {
        log.Info("Initializing Application.");
        // retrieve necessary elements from settings hashtable.
        strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
        if (strApplicationName != m_applicationName)
        {
          return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, "The application name passed to init() conflicts with the name stored in the hashtable: hashtable entry " + strApplicationName + " does not match init application name of: " + m_applicationName,
           StackTracer.MakeXPException("(internal stacktrace)"));
        }
        strLibHome = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_LIB_HOME));
        // create and initialize our application.
        ApplicationManager.GetInstance().GetApplication(strApplicationName,
         null, strLibHome);
        log.Info("Application initialized.");
      }
      catch (Exception ex)
      {
        return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, "Could not initialize the application object with libhome of: " + strLibHome + " and application name of: " + strApplicationName,
         XPException.GetInstance(ex));
      }
      return new ReturnValue(PT_STARTUP_RESULTCODES.OK, "InitializeApplication init successful",
       null);
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
     
    public virtual XPArrayList getMsg(int stepid, Platform platform, ReturnValue ret)
    {
      // Use the diagnostic helper to find the right message        
      return RecommendationHelper.getList(stepid, platform, ret.getResultCode(),
       m_diag);
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
     
    public virtual StartupTaskMessage[] getMsgArray()
    {
      return m_diag;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
     
    public virtual int getNumMsgs()
    {
      return m_diag.Length;
    }
  }
}