/*
 * Created on Apr 11, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.application;
using com.plumtree.uiinfrastructure.application.notification;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.util;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.application.startup
{
  /// <summary>
  /// Initializes the config helper. This loads the core ConfigVarPack.
  /// </summary>
  /// <author>DavidP</author>
  public class InitializeConfigHelper : IStartupTask
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.startup.InitializeConfigHelper");
    private String m_applicationName = "";
    // array of messages
    private static StartupTaskMessage[] m_diag = {new StartupTaskMessage(
    // steps are one-based; step zero is the init call
    // DAP TODO this really needs to be in an external file
    0, Platform.ALL, PT_STARTUP_RESULTCODES.FATAL, "ConfigHelper problem",
     "RECOMMENDATION: Review this exception to find the cause of this error and check the " + "relevant settings in portalconfig.xml to make sure there are no errors in the xml file.  " + "Typically, this failure indicates bad URL mappings, invalid URLs for the imageserver, or a " + "improperly configured security mode.")};
    public InitializeConfigHelper() : base()
    {
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
     
    public virtual String getTaskName()
    {
      return "InitializeConfigHelper";
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
     
    public virtual int getTaskID()
    {
      return PTStartupTasks.INITCONFIG.getOrdinal();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
     
    public virtual int getNumSteps()
    {
      return 1;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
     
    public virtual ReturnValue init(String _strApplicationName)
    {
      m_applicationName = _strApplicationName;
      return new ReturnValue(PT_STARTUP_RESULTCODES.OK, "InitializeConfigHelper init successful",
       null);
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
     
    public virtual ReturnValue executeStep(int sequentialStepID)
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "InitializeImageServerHelper successful.",
       null);
      try
      {
        log.Info("Initializing ConfigHelper.");
        // retrieve Application, ConfigVarPack, and ImgSvrURLHelper.
        String strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
        IApplication application = ApplicationManager.GetInstance().GetApplication(strApplicationName);
        ConfigVarPack vpConfig = (ConfigVarPack) application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
        ImgSvrURLHelper ptImgSvrURLHelper = (ImgSvrURLHelper) application.GetCachingManager().GetEntry(AppWarmUp.KEY_CACHEMAN_IMAGE_SERVER_HELPER);
        // initialize the config helper.  this must be done after the 
        // image server url helper instance has been initialized.
        ReturnValue ret = ConfigHelper.StaticInit(strApplicationName, vpConfig,
         ptImgSvrURLHelper);
        if (ret.getResultCode() != PT_STARTUP_RESULTCODES.OK)
        {
          retVal = ret;
        }
        log.Info("ConfigHelper initialized.");
      }
      catch (Exception ex)
      {
        String errorMessage = "Error initializing ConfigHelper: ";
        throw new XPException(errorMessage + ex.Message, ex);
      }
      return retVal;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
     
    public virtual XPArrayList getMsg(int stepid, Platform platform, ReturnValue ret)
    {
      // Use the diagnostic helper to find the right message        
      return RecommendationHelper.getList(stepid, platform, ret.getResultCode(),
       m_diag);
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
     
    public virtual StartupTaskMessage[] getMsgArray()
    {
      return m_diag;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
     
    public virtual int getNumMsgs()
    {
      return m_diag.Length;
    }
  }
}