/*
 * Created on Mar 30, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.application;
using com.plumtree.uiinfrastructure.application.notification;
using com.plumtree.uiinfrastructure.application.startup.exceptions;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.util;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.application.startup
{
  /// <summary>
  /// Initializes the ImageServerHelper.  After this point, the paths
  /// to the ImageServer and the Connection URL will be available,
  /// and the ImageServerHelper will be added to the Application Cache.
  /// </summary>
  /// <author>DavidP</author>
  public class InitializeImageServerHelper : IStartupTask
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.startup.InitializeImageServerHelper");
    private String m_applicationName = "";
    private ConfigVarPack m_vpConfig = null;
    private String m_strImageServerBaseURL = null;
    private String m_strSecureImageServerBaseURL = null;
    private String m_strImageServerConnectionURL = null;
    private IApplication m_application = null;
    // array of messages
    private static StartupTaskMessage[] m_diag = {new StartupTaskMessage(
    // steps are one-based; step zero is the init call
    // DAP TODO this really needs to be in an external file
    0, Platform.ALL, PT_STARTUP_RESULTCODES.FATAL, "ImageServerBaseURL problem",
     "RECOMMENDATION: Either you have incorrectly installed the portal, your XML " + "files are corrupt, or the user running your web application server does not " + "have access rights to the XML files.  Check that portalconfig.xml is " + "accessible to the user running the web application server and that the file " + "has valid imageserver URL entries."),
     new StartupTaskMessage(1, Platform.ALL, PT_STARTUP_RESULTCODES.FATAL,
     "Secure Imageserver Error", "RECOMMENDATION: Check the value you entered for the imageserver secure base URL " + "(ImageServerSecureBaseURL) in portalconfig.xml.  It is invalid or badly formed."),
     new StartupTaskMessage(2, Platform.ALL, PT_STARTUP_RESULTCODES.FATAL,
     "ImageServer Connection URL or Startup Error", "RECOMMENDATION: The portal requires a valid HTTP connection between it " + "and the imageserver that is independent of the external imageserver URL. " + "This URL must be valid and accessible directly from the plumtree portal server " + "machine.  Check the value of the ImageServerConnectionURL setting in portalconfig.xml " + "and validate it using a browser running directly on the plumtree portal " + "server machine.  The plumtree portal server machine may be unable to see the " + "imageserver because it may be blocked by a firewall.")};
    public InitializeImageServerHelper() : base()
    {
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
     
    public virtual String getTaskName()
    {
      return "InitializeImageServerHelper";
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
     
    public virtual int getTaskID()
    {
      return PTStartupTasks.INITIMGSERVER.getOrdinal();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
     
    public virtual int getNumSteps()
    {
      return 3;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
     
    public virtual ReturnValue init(String _strApplicationName)
    {
      m_applicationName = _strApplicationName;
      return new ReturnValue(PT_STARTUP_RESULTCODES.OK, getTaskName() + " init successful",
       null);
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
     
    public virtual ReturnValue executeStep(int sequentialStepID)
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, "Incorrect step request on init",
       StackTracer.MakeXPException("(internal stacktrace)"));
      switch (sequentialStepID){
      case 0:
      // open config file
      retVal = executeStepOne();
      break;
      case 1:
      // open config file
      retVal = executeStepTwo();
      break;
      case 2:
      // open config file
      retVal = executeStepThree();
      break;
      }
      return retVal;
    }
    
    /// <summary>
    /// Load config file, get imageserver URL, form related urls
    /// </summary>
    /// <returns></returns>
    private ReturnValue executeStepOne()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "InitializeImageServerHelper successful.",
       null);
      log.Info("Initializing ImageServerHelper.");
      // form the URL
      try
      {
        // retrieve Application and ConfigVarPack.
        String strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
        m_application = ApplicationManager.GetInstance().GetApplication(strApplicationName);
        m_vpConfig = (ConfigVarPack) m_application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
        m_strImageServerBaseURL = m_vpConfig.GetConfigValueAsString("MainURLs",
         "ImageServerBaseURL", ConfigConstants.VALUE_ATTRIBUTE);
        AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_STR_IMAGESERVER_BASE_URL,
         m_strImageServerBaseURL);
      }
      catch (Exception ex)
      {
        String errorMessage = "Failed to get ImageServerBaseURL from ConfigVarPack: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(ex));
        return retVal;
      }
      return retVal;
    }
    
    /// <summary>
    /// get imageserver URL, form related urls
    /// </summary>
    /// <returns></returns>
    private ReturnValue executeStepTwo()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "InitializeImageServerHelper successful.",
       null);
      try
      {
        m_strSecureImageServerBaseURL = m_vpConfig.GetConfigValueAsString("MainURLs",
         "ImageServerSecureBaseURL", ConfigConstants.VALUE_ATTRIBUTE);
        AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_STR_SECURE_IMAGESERVER_BASE_URL,
         m_strSecureImageServerBaseURL);
      }
      catch (Exception ex)
      {
        String errorMessage = "Failed to get SecureImageServerBaseURL from ConfigVarPack: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(ex));
        return retVal;
      }
      return retVal;
    }
    
    /// <summary>
    /// form related urls
    /// </summary>
    /// <returns></returns>
    private ReturnValue executeStepThree()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "InitializeImageServerHelper successful.",
       null);
      // form the connection url
      try
      {
        m_strImageServerConnectionURL = m_vpConfig.GetConfigValueAsString("MainURLs",
         ConfigConstants.IMAGE_SERVER_CONNECTION_URL, ConfigConstants.VALUE_ATTRIBUTE);
        AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_STR_IMAGESERVER_CONNECTION_URL,
         m_strImageServerConnectionURL);
      }
      catch (Exception ex)
      {
        String errorMessage = "Failed to get ImageServerConnectionURL from ConfigVarPack: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(ex));
        return retVal;
      }
      // init the helper
      try
      {
        // initialize our image server helper.
        ImgSvrURLHelper.Init(m_strImageServerBaseURL, m_strSecureImageServerBaseURL,
         ImgSvrURLHelper.STR_L1_DOMAIN_PLUMTREE, m_applicationName, true);
        // place an instance of the helper into the cache.
        m_application.GetCachingManager().SetEntry(AppWarmUp.KEY_CACHEMAN_IMAGE_SERVER_HELPER,
         ImgSvrURLHelper.GetInstance());
        // display URL information.
        log.Info("ImageServerBaseURL: " + m_strImageServerBaseURL);
        log.Info("SecureImageServerBaseURL: " + m_strSecureImageServerBaseURL);
        log.Info("ImageServerConnectionURL: " + m_strImageServerConnectionURL);
        log.Info("ImageServerHelper initialized.");
      }
      catch (Exception ex)
      {
        String errorMessage = "Error initializing ImageServerHelper.  Verify the imageserver you have specified is running: " + m_strImageServerBaseURL;
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(ex));
        return retVal;
      }
      return retVal;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
     
    public virtual XPArrayList getMsg(int stepid, Platform platform, ReturnValue ret)
    {
      // Use the diagnostic helper to find the right message        
      return RecommendationHelper.getList(stepid, platform, ret.getResultCode(),
       m_diag);
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
     
    public virtual StartupTaskMessage[] getMsgArray()
    {
      return m_diag;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
     
    public virtual int getNumMsgs()
    {
      return m_diag.Length;
    }
  }
}