/*
 * Created on Apr 12, 2005
 */
 
using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using XPConvert=com.plumtree.openfoundation.util.XPConvert;
using XPException=com.plumtree.openfoundation.util.XPException;
using OpenLogService=com.plumtree.openlog.OpenLogService;
using OpenLogger=com.plumtree.openlog.OpenLogger;
using AppWarmUp=com.plumtree.uiinfrastructure.application.AppWarmUp;
using IStartupTask=com.plumtree.uiinfrastructure.application.notification.IStartupTask;
using PT_STARTUP_RESULTCODES=com.plumtree.uiinfrastructure.application.notification.PT_STARTUP_RESULTCODES;
using ReturnValue=com.plumtree.uiinfrastructure.application.notification.ReturnValue;
using com.plumtree.uiinfrastructure.application.startup.exceptions;
using com.plumtree.uiinfrastructure.application.startup.messages;
using ConfigVarPack=com.plumtree.uiinfrastructure.application.varpacks.ConfigVarPack;
using com.plumtree.uiinfrastructure.util;
using ApplicationManager=com.plumtree.uiinfrastructure.web.ApplicationManager;
using IApplication=com.plumtree.uiinfrastructure.web.IApplication;
namespace com.plumtree.uiinfrastructure.application.startup
{
  /// <summary>
  /// Initializes the ResourceManager.  After this point the 
  /// ResourceManager will be available in the Application cache.
  /// The ResourceManager handles all the i18n files.
  /// TODO: This should be reloadable, but it's not.  There is a bug 
  /// filed against OpenFoundation.  When the bug is fixed, add a 
  /// method to reload the ResourceManager.
  /// </summary>
  /// <author>DavidP</author>
  public class InitializeResourceManager : IStartupTask
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.startup.InitializeResourceManager");
    private String m_applicationName = "";
    private ConfigVarPack m_vpConfig = null;
    private IApplication m_application = null;
    private String m_strResourceFolder = null;
    private int m_nDebuggingEnabled = 0;
    // array of messages
    private static StartupTaskMessage[] m_diag = {new StartupTaskMessage(
    // steps are one-based; step zero is the init call
    // DAP TODO this really needs to be in an external file
    0, Platform.ALL, PT_STARTUP_RESULTCODES.FATAL, "ResourceManager i18n Unit Test Error",
     "RECOMMENDATION: This rare error usually indicates that the i18n XML language files are " + "corrupt, or you have improperly followed instructions for extending or editing those files.  " + "Review the details in the secondary exception below."),
     new StartupTaskMessage(1, Platform.JAVA, PT_STARTUP_RESULTCODES.FATAL,
     "ResourceManager i18n Strings Loading Error (Java)", "RECOMMENDATION: This indicates an installation error, folder access rights issue, " + "corrupt i18n XML files, or incorrect customer implementation of additional language support.  " + "Since this is Java you should first check that the user you are running your J2EE web " + "application server as has full access to the i18n settings folder.  " + "To isolate other causes of this issue, try enabling debug mode in portalconfig.xml " + "(set the DebuggingMode setting to 1) and restarting your portal. This enables the i18n " + "diagnostic unit tests, which should output more detailed errors to ptspy."),
     new StartupTaskMessage(1, Platform.DOTNET, PT_STARTUP_RESULTCODES.FATAL,
     "ResourceManager i18n Strings Loading Error (.NET)", "RECOMMENDATION: This indicates an installation error, folder access rights issue, " + "corrupt i18n XML files, or incorrect customer implementation of additional language support.  " + "Since this is .NET you should first check that the IIS anonymous user has full " + "access to the i18n settings folder.  " + "To isolate other causes of this issue, try enabling debug mode in portalconfig.xml " + "(set the DebuggingMode setting to 1) and restarting your portal. This enables the i18n " + "diagnostic unit tests, which should then output more detailed errors to ptspy.")};
    public InitializeResourceManager() : base()
    {
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
     
    public virtual String getTaskName()
    {
      return "InitializeResourceManager";
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
     
    public virtual int getTaskID()
    {
      return PTStartupTasks.INITIMGSERVER.getOrdinal();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
     
    public virtual int getNumSteps()
    {
      return 2;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
     
    public virtual ReturnValue init(String _strApplicationName)
    {
      m_applicationName = _strApplicationName;
      return new ReturnValue(PT_STARTUP_RESULTCODES.OK, getTaskName() + " init successful",
       null);
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
     
    public virtual ReturnValue executeStep(int sequentialStepID)
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, "Incorrect step request on init",
       StackTracer.MakeXPException("(internal stacktrace)"));
      switch (sequentialStepID){
      case 0:
      // open config file
      retVal = executeStepOne();
      break;
      case 1:
      // load and process resources
      retVal = executeStepTwo();
      break;
      }
      return retVal;
    }
    
    /// <summary>
    /// Load config file, get imageserver URL, form related urls
    /// </summary>
    /// <returns></returns>
    private ReturnValue executeStepOne()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "InitializeImageServerHelper successful.",
       null);
      log.Info("Initializing Resource Manager.");
      try
      {
        // retrieve Application and other strings.
        m_applicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
        m_application = ApplicationManager.GetInstance().GetApplication(m_applicationName);
        m_vpConfig = (ConfigVarPack) m_application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
        m_strResourceFolder = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_I18N));
        try
        {
          // retrieve debugging mode.  
          m_nDebuggingEnabled = m_vpConfig.GetConfigValueAsInt("SystemProperties",
           "DebuggingMode", "value");
        }
        catch (Exception ex)
        {
          m_nDebuggingEnabled = 0;
          log.Warn(ex, "Debugging mode unknown, skipping language file test.");
          }
      }
      catch (Exception ex)
      {
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, "Could not determine the i18n resource folder.  Check the ConfigVarPack settings for i18n.",
         XPException.GetInstance(ex));
        return retVal;
      }
      return retVal;
    }
    
    /// <summary>
    /// Load config file, get imageserver URL, form related urls
    /// </summary>
    /// <returns></returns>
    private ReturnValue executeStepTwo()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "InitializeImageServerHelper successful.",
       null);
      log.Info("Initializing Resource Manager.");
      try
      {
        if (m_nDebuggingEnabled == 1)
        {
          // if debugging is enabled run language test.
          log.Info("Running language file unit test.");
          retVal = AppWarmUp.HelperTestLanguages(m_strResourceFolder);
          if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
          {
            log.Info("Language file unit test completed.");
          } else
          {
            return retVal;
          }
        }
      }
      catch (Exception ex)
      {
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, "Resource Manager error executing unit test on i18n resources:  Corrupted i18n language files.",
         XPException.GetInstance(ex));
        return retVal;
      }
      try
      {
        // resource manager is a singleton, cannot be reloaded.		    
        XPResourceManager rmResourceManager = XPResourceManager.CreateResourceManager(m_strResourceFolder);
        rmResourceManager.LoadResources();
        // save the resource manager into the application cache.
        m_application.GetCachingManager().SetEntry(AppWarmUp.KEY_CACHEMAN_RESOURCE_MANAGER,
         rmResourceManager);
        log.Info("Resource Manager initialized.");
      }
      catch (Exception ex)
      {
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, "Resource Manager loading error:",
         XPException.GetInstance(ex));
        return retVal;
      }
      return retVal;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
     
    public virtual XPArrayList getMsg(int stepid, Platform platform, ReturnValue ret)
    {
      // Use the diagnostic helper to find the right message        
      return RecommendationHelper.getList(stepid, platform, ret.getResultCode(),
       m_diag);
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
     
    public virtual StartupTaskMessage[] getMsgArray()
    {
      return m_diag;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
     
    public virtual int getNumMsgs()
    {
      return m_diag.Length;
    }
  }
}