/*
 * Created on Apr 15, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.application;
using com.plumtree.uiinfrastructure.application.notification;
using com.plumtree.uiinfrastructure.application.startup.exceptions;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.util;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.application.startup
{
  /// <summary>
  /// Initializes VarPacks.  This method will load the VarPacks specified
  /// in the VarPacks.xml file, as well as any additional VarPacks as 
  /// declared by the AppWarmUpHelper.  After this point, the Application
  /// VarPackManager will contain all of the initial VarPacks.
  /// </summary>
  /// <author>DavidP</author>
  public class InitializeVarPacks : IStartupTask
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.startup.InitializeVarPacks");
    private String m_applicationName = "";
    private IAppWarmUpHelper m_ptAppWarmUpHelper = null;
    internal IApplication m_application = null;
    internal String m_strLibHome = "";
    internal String m_strConfigPath = "";
    internal String m_strVarPacksFile = "";
    // array of messages
    private static StartupTaskMessage[] m_diag = {new StartupTaskMessage(
    // steps are one-based; step zero is the init call
    // DAP TODO this really needs to be in an external file
    0, Platform.JAVA, PT_STARTUP_RESULTCODES.FATAL, "InitializeVarPacks: Problem processing varpacks in VarPacks.xml.",
     "Java Specific: This often indicates a bad portal install, or a mismatch " + "between the core Aqualogic Interaction JAR files in PORTAL_HOME/lib/java and " + "PORTAL_HOME/webapp/portal.war. Check that all JAR files are present " + "and their dependencies are satisfied.  " + "It can *also* indicate you have edited a core XML file incorrectly.  " + "The VarPacks affected include SecureASVarPack," + "DevicesVarPack, and URLASMappingsVarPack.  Check the relevant settings " + "in the portalconfig.xml file."),
     new StartupTaskMessage(0, Platform.DOTNET, PT_STARTUP_RESULTCODES.FATAL,
     "InitializeVarPacks: Problem processing varpacks in VarPacks.xml.", ".NET Specific: This often indicates a bad portal install, or a mismatch " + "between the core Aqualogic Interaction DLL assembly files in PORTAL_HOME\\bin\\assemblies " + "and PORTAL_HOME\\webapp\\portal\\web. Check that all DLL files are present " + "and their assembly dependencies are satisfied. " + "It can *also* indicate you have edited a core XML file incorrectly.  " + "The VarPacks affected include SecureASVarPack," + "DevicesVarPack, and URLASMappingsVarPack.  Check the relevant settings " + "in the portalconfig.xml file."),
     new StartupTaskMessage(1, Platform.JAVA, PT_STARTUP_RESULTCODES.FATAL,
     "InitializeVarPacks: Problem intiializing the custom varpacks in " + "portaluiinfrastructure.",
     "Verify that the settings in portalconfig.xml have reasonable values and are not corrupt." + "The VarPacks affected include PTDirPrefsVarPack,PTDocTypesVarPack," + "PTTimeZoneVarPack,PTSSOVarPack, and PTNavSettingsVarPack.  Check the relevant settings " + "in the portalconfig.xml file."),
     new StartupTaskMessage(1, Platform.DOTNET, PT_STARTUP_RESULTCODES.FATAL,
     "InitializeVarPacks: Problem intiializing the custom varpacks in " + "portaluiinfrastructure.",
     "Verify that the settings in portalconfig.xml have reasonable values and are not corrupt." + "The VarPacks affected include PTDirPrefsVarPack,PTDocTypesVarPack," + "PTTimeZoneVarPack,PTSSOVarPack, and PTNavSettingsVarPack.  Check the relevant settings " + "in the portalconfig.xml file.")};
    public InitializeVarPacks() : base()
    {
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
     
    public virtual String getTaskName()
    {
      return "InitializeVarPacks";
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
     
    public virtual int getTaskID()
    {
      return PTStartupTasks.INITVARPACKS.getOrdinal();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
     
    public virtual int getNumSteps()
    {
      return 2;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
     
    public virtual ReturnValue init(String _strApplicationName)
    {
      m_applicationName = _strApplicationName;
      return new ReturnValue(PT_STARTUP_RESULTCODES.OK, getTaskName() + " init successful",
       null);
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
     
    public virtual ReturnValue executeStep(int sequentialStepID)
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, "Incorrect step request on init",
       StackTracer.MakeXPException("(internal stacktrace)"));
      switch (sequentialStepID){
      case 0:
      // open config file
      retVal = executeStepOne();
      break;
      case 1:
      // load config file
      retVal = executeStepTwo();
      break;
      }
      return retVal;
    }
    
    /// <summary>
    /// Load config file
    /// </summary>
    /// <returns></returns>
    private ReturnValue executeStepOne()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "InitializeVarPacks load config file successful",
       null);
      try
      {
        log.Info("Initializing VarPacks.");
        /////////////////////////////////////////
        // PROCESSING VARPACKS IN VARPACKS.XML //
        /////////////////////////////////////////
        // retrieving necessary elements.
        String strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
        m_application = ApplicationManager.GetInstance().GetApplication(strApplicationName);
        m_strLibHome = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_LIB_HOME));
        m_strConfigPath = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_CONFIG));
        m_strVarPacksFile = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_FILE_VAR_PACKS));
        log.Info("Processing file: " + m_strVarPacksFile);
        // loads the VarPacks specified in VarPacks.xml.  -- false means they won't override
        // previously loaded varpacks, since they're the first varpacks to be loaded.
        AppWarmUp.HelperLoadVarPacks(m_strVarPacksFile, m_strLibHome, m_strConfigPath,
         m_application, false);
      }
      catch (Exception ex)
      {
        String errorMessage = "Error initializing VarPacks: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(ex));
      }
      return retVal;
    }
    
    /// <summary>
    /// Load config file
    /// </summary>
    /// <returns></returns>
    private ReturnValue executeStepTwo()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "InitializeVarPacks load config file successful",
       null);
      try
      {
        log.Info("Process Helper Varpacks (portaluiinfrastructure)");
        /////////////////////////////////////////
        // PROCESSING APPWARMUPHELPER VARPACKS //
        /////////////////////////////////////////
        // retrieving necessary elements.
        m_ptAppWarmUpHelper = (IAppWarmUpHelper) AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_APP_WARMUP_HELPER);
        if (null != m_ptAppWarmUpHelper)
        {
          String strPlatform = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_PLATFORM));
          // initializes any VarPacks from the AppWarmUpHelper.
          m_ptAppWarmUpHelper.InitializeVarPacks(m_application.GetVarPackManager(),
           m_strConfigPath, strPlatform);
        }
        log.Info("VarPacks initialized.");
      }
      catch (Exception ex)
      {
        String errorMessage = "Error initializing VarPacks: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(ex));
      }
      return retVal;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
     
    public virtual XPArrayList getMsg(int stepid, Platform platform, ReturnValue ret)
    {
      // Use the diagnostic helper to find the right message        
      return RecommendationHelper.getList(stepid, platform, ret.getResultCode(),
       m_diag);
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
     
    public virtual StartupTaskMessage[] getMsgArray()
    {
      return m_diag;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
     
    public virtual int getNumMsgs()
    {
      return m_diag.Length;
    }
  }
}