/*
 * Purpose:	Generic loader for all activityspace tasks
 * Author: 	D. Phipps May 18, 2005
 */
 
using System;
using com.plumtree.openfoundation.io;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.application;
using com.plumtree.uiinfrastructure.application.notification;
using com.plumtree.uiinfrastructure.application.startup.exceptions;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.login;
using com.plumtree.uiinfrastructure.opener.types;
using com.plumtree.uiinfrastructure.pei;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.util;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.application.startup
{
  /// <summary>
  /// Large bundled task that handles all the dynamic loads related to activityspaces, PEIs and such.
  /// </summary>
  /// <author>DavidP</author>
  public class LoadCustomLoads : IStartupTask
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.startup.LoadCustomLoads");
    private String m_applicationName = "";
    private IAppWarmUpHelper m_ptAppWarmUpHelper = null;
    private IApplication m_application = null;
    private String m_strImageServerBaseURL = null;
    private String m_strSecureImageServerBaseURL = null;
    private String m_strImageServerConnectionURL = null;
    private ConfigVarPack m_vpConfig = null;
    // array of messages
    private static StartupTaskMessage[] m_diag = {new StartupTaskMessage(
    // steps are one-based; step zero is the init call
    // DAP TODO this really needs to be in an external file
    0, Platform.DOTNET, PT_STARTUP_RESULTCODES.FATAL, "LoadCustomActivitySpaces: XML or refpath problem.",
     "RECOMMENDATION: Verify your reference path is correct.  Check that the " + "relevant .NET assemblies " + "in the customactivityspaces.xml file are present in both " + "PORTAL_HOME/bin/assemblies as well as PORTAL_HOME/webapp/portal/web/bin.  " + "Check secondary details below."),
     new StartupTaskMessage(0, Platform.JAVA, PT_STARTUP_RESULTCODES.FATAL,
     "LoadCustomActivitySpaces: XML or classpath problem.", "RECOMMENDATION: Verify your classpath is correct.  Check that the relevant " + "JAR files in the customactivityspaces.xml file are present in both " + "PORTAL_HOME/lib/java as well as inside the PORTAL_HOME/webapp/portal.war " + "in the WEB-INF/lib subdirectory.  " + "Check secondary details below."),
     new StartupTaskMessage(1, Platform.DOTNET, PT_STARTUP_RESULTCODES.FATAL,
     "LoadCustomManagers: XML or refpath problem.", "RECOMMENDATION: Verify your reference path is correct.  Check that the " + "relevant .NET assemblies " + "in the customactivityspaces.xml and customtags.xml files are present in both " + "PORTAL_HOME/bin/assemblies as well as PORTAL_HOME/webapp/portal/web/bin.  " + "Check secondary details below."),
     new StartupTaskMessage(1, Platform.JAVA, PT_STARTUP_RESULTCODES.FATAL,
     "LoadCustomManagers: XML or classpath problem.", "RECOMMENDATION: Verify your classpath is correct.  Check that the relevant " + "JAR files in the customactivityspaces.xml and customtags.xml files are present in both " + "PORTAL_HOME/lib/java as well as inside the PORTAL_HOME/webapp/portal.war " + "in the WEB-INF/lib subdirectory.  " + "Check secondary details below."),
     new StartupTaskMessage(2, Platform.DOTNET, PT_STARTUP_RESULTCODES.FATAL,
     "LoadCustomVarPacks: Varpack loading problem.", "RECOMMENDATION: Verify your reference path is correct.  Check that the " + "relevant .NET assemblies in the customvarpacks.xml file are present in both " + "PORTAL_HOME/bin/assemblies as well as PORTAL_HOME/webapp/portal/web/bin.  " + "Check secondary details below."),
     new StartupTaskMessage(2, Platform.JAVA, PT_STARTUP_RESULTCODES.FATAL,
     "LoadCustomVarPacks: Varpack loading problem.", "RECOMMENDATION: Verify your classpath is correct.  Check that the relevant " + "JAR files in the customvarpacks.xml file are present in both " + "PORTAL_HOME/lib/java as well as inside the PORTAL_HOME/webapp/portal.war " + "in the WEB-INF/lib subdirectory.  " + "Check secondary details below."),
     new StartupTaskMessage(3, Platform.DOTNET, PT_STARTUP_RESULTCODES.FATAL,
     "LoadDynamicLoads: Dynamic loading problem.", "RECOMMENDATION: Verify your reference path is correct.  Check that the " + "relevant .NET assemblies in the customvarpacks.xml file are present in both " + "PORTAL_HOME/bin/assemblies as well as PORTAL_HOME/webapp/portal/web/bin.  " + "Check secondary details below."),
     new StartupTaskMessage(3, Platform.JAVA, PT_STARTUP_RESULTCODES.FATAL,
     "LoadDynamicLoads: Dynamic loading problem.", "RECOMMENDATION: Verify your classpath is correct.  Check that the relevant " + "JAR files in the customvarpacks.xml file are present in both " + "PORTAL_HOME/lib/java as well as inside the PORTAL_HOME/webapp/portal.war " + "in the WEB-INF/lib subdirectory.  " + "Check secondary details below."),
     new StartupTaskMessage(4, Platform.DOTNET, PT_STARTUP_RESULTCODES.FATAL,
     "LoadObjects: Dynamic loading problem.", "RECOMMENDATION: The ClassTypeDescManager supports custom object types. " + "Verify your reference path is correct.  Check that the " + "relevant .NET assemblies in the customvarpacks.xml file are present in both " + "PORTAL_HOME/bin/assemblies as well as PORTAL_HOME/webapp/portal/web/bin.  " + "Check secondary details below."),
     new StartupTaskMessage(4, Platform.JAVA, PT_STARTUP_RESULTCODES.FATAL,
     "LoadObjects: Dynamic loading problem.", "RECOMMENDATION: The ClassTypeDescManager supports custom object types. " + "Verify your classpath is correct.  Check that the relevant " + "JAR files in the customvarpacks.xml file are present in both " + "PORTAL_HOME/lib/java as well as inside the PORTAL_HOME/webapp/portal.war " + "in the WEB-INF/lib subdirectory.  " + "Check secondary details below.")};
    public LoadCustomLoads() : base()
    {
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
     
    public virtual String getTaskName()
    {
      return "LoadCustomLoads";
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
     
    public virtual int getTaskID()
    {
      return PTStartupTasks.LOADCUSTOMLOADS.getOrdinal();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
     
    public virtual int getNumSteps()
    {
      // TODO: This should really be done automatically based on the error message array
      return 5;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
     
    public virtual ReturnValue init(String _strApplicationName)
    {
      m_applicationName = _strApplicationName;
      return new ReturnValue(PT_STARTUP_RESULTCODES.OK, getTaskName() + " init successful",
       null);
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
     
    public virtual ReturnValue executeStep(int sequentialStepID)
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, "Incorrect step request on init",
       StackTracer.MakeXPException("(internal stacktrace)"));
      switch (sequentialStepID){
      case 0:
      retVal = LoadCustomActivitySpaces();
      break;
      case 1:
      retVal = LoadCustomManagers();
      break;
      case 2:
      retVal = LoadCustomVarPacks();
      break;
      case 3:
      retVal = LoadDynamicLoads();
      break;
      case 4:
      retVal = LoadObjects();
      break;
      }
      return retVal;
    }
    
    /// <summary>
    /// Loads custom Activity Spaces by processing the CustomActivitySpaces.xml
    /// through the ASManager and any other managers specified in AppWarmUpHelper.
    /// </summary>
    /// <returns>ReturnValue</returns>
    public virtual ReturnValue LoadCustomActivitySpaces()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "LoadCustomActivitySpaces successful",
       StackTracer.MakeXPException("(internal stacktrace)"));
      try
      {
        log.Info("Loading Custom Activity Spaces.");
        String strCustomActivitySpacesFile = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_FILE_CUSTOM_ACTIVITY_SPACES));
        String strLibHome = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_LIB_HOME));
        ///////////////////////////////////
        // PROCESS SPACES WITH ASMANAGER //
        ///////////////////////////////////
        try
        {
          log.Info("Loading custom AS / MVC classes into ASManager.");
          // loading custom activity spaces.
          ASManager.INSTANCE.LoadCustomClasses(strLibHome, strCustomActivitySpacesFile);
          log.Info("ASManager custom loading complete.");
        }
        catch (Exception ex)
        {
          String errorMessage = "Error loading custom AS / MVC classes into ASManager: ";
          retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
           XPException.GetInstance(ex));
          return retVal;
        }
        log.Info("Custom Activity Spaces and MVC classes loaded.");
        return retVal;
      }
      catch (Exception ex)
      {
        String errorMessage = "Error loading custom activity spaces and MVC classes: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(ex));
      }
      return retVal;
    }
    
    /// <summary>
    /// Loads custom Managers by processing the CustomActivitySpaces.xml and
    /// CustomTags.xml files through the NavManager and TagLibManager
    /// and any other managers specified in AppWarmUpHelper.
    /// </summary>
    /// <returns>ReturnValue</returns>
    public virtual ReturnValue LoadCustomManagers()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "LoadCustomManagers successful",
       StackTracer.MakeXPException("(internal stacktrace)"));
      try
      {
        log.Info("Loading Custom Managers.");
        String strLibHome = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_LIB_HOME));
        /////////////////////////////////////////
        // PROCESS SPACES WITH APPWARMUPHELPER //
        /////////////////////////////////////////
        // retrieving necessary elements.
        IAppWarmUpHelper ptAppWarmUpHelper = (IAppWarmUpHelper) AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_APP_WARMUP_HELPER);
        if (null != ptAppWarmUpHelper)
        {
          String strConfigPath = AppWarmUp.GetSettingValueAsString(AppWarmUp.KEY_PATH_PORTAL_CONFIG);
          try
          {
            log.Info("Loading Custom Managers.");
            // load custom managers based on custom AS and tag files.
            retVal = ptAppWarmUpHelper.LoadManagers(strLibHome, strConfigPath);
            if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
            {
              log.Info("Custom Managers loaded.");
            }
          }
          catch (Exception ex)
          {
            String errorMessage = "Error loading Custom Managers: ";
            retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
             XPException.GetInstance(ex));
            return retVal;
          }
        }
        log.Info("Custom Managers loaded.");
        return retVal;
      }
      catch (Exception ex)
      {
        String errorMessage = "Error loading custom managers: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(ex));
      }
      return retVal;
    }
    
    /// <summary>
    /// Loads custom VarPacks by processing the CustomVarPacks.xml
    /// and any other VarPacks specified in AppWarmUpHelper.
    /// </summary>
    public virtual ReturnValue LoadCustomVarPacks()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "LoadCustomVarPacks successful.",
       StackTracer.MakeXPException("(internal stacktrace)"));
      try
      {
        log.Info("Loading Custom VarPacks.");
        // retrieving necessary elements.
        String strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
        IApplication application = ApplicationManager.GetInstance().GetApplication(strApplicationName);
        String strCustomVarPacksFile = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_FILE_CUSTOM_VAR_PACKS));
        String strLibHome = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_LIB_HOME));
        String strConfigPath = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_CONFIG));
        String strPlatform = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_PLATFORM));
        ////////////////////////////////////////////
        // PROCESS VARPACKS IN CUSTOMVARPACKS.XML //
        ////////////////////////////////////////////
        try
        {
          log.Info("Loading Custom VarPacks from CustomVarPacks.xml");
          // load custom varpacks, overwritten as they occur.
          retVal = AppWarmUp.HelperLoadVarPacks(strCustomVarPacksFile, strLibHome,
           strConfigPath, application, true);
          if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
          {
            log.Info("Custom VarPacks from CustomVarPacks.xml loaded.");
          } else
          {
            return retVal;
          }
        }
        catch (Exception ex)
        {
          String errorMessage = "Error loading Custom VarPacks from CustomVarPacks.xml: ";
          retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
           XPException.GetInstance(ex));
          return retVal;
        }
        /////////////////////////////////////////
        // PROCESS VARPACKS IN APPWARMUPHELPER //
        /////////////////////////////////////////
        // retrieving necessary elements.
        IAppWarmUpHelper ptAppWarmUpHelper = (IAppWarmUpHelper) AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_APP_WARMUP_HELPER);
        if (null != ptAppWarmUpHelper)
        {
          try
          {
            log.Info("Loading Custom VarPacks from AppWarmUpHelper.");
            // load custom managers based on custom AS file.
            retVal = ptAppWarmUpHelper.LoadVarPacks(application.GetVarPackManager(),
             strConfigPath, strPlatform);
            if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
            {
              log.Info("Custom VarPacks from AppWarmUpHelper loaded.");
            }
          }
          catch (Exception ex)
          {
            String errorMessage = "Error loading Custom VarPacks from AppWarmUpHelper: ";
            retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
             XPException.GetInstance(ex));
            return retVal;
          }
        }
        if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
        {
          log.Info("Custom VarPacks loaded.");
        }
      }
      catch (Exception te)
      {
        String errorMessage = "Error accessing aqppwarmup hashtable entries: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(te));
        return retVal;
      }
      return retVal;
    }
    
    /// <summary>
    /// Loads elements from the dynamic loads folder, including
    /// any PEIs, provider types, etc.
    /// </summary>
    public virtual ReturnValue LoadDynamicLoads()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "LoadDynamicLoads successful.",
       StackTracer.MakeXPException("(internal stacktrace)"));
      try
      {
        log.Info("Loading Dynamic Loads.");
        // retrieving necessary elements.
        String strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
        IApplication application = ApplicationManager.GetInstance().GetApplication(strApplicationName);
        String strDynamicLoadsPath = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_DYNAMIC_LOADS));
        String strLibHome = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_LIB_HOME));
        // load custom varpacks, overwritten as they occur.
        retVal = HelperLoadSettings(application, strDynamicLoadsPath, strLibHome);
        // take old style PEIs (i.e. ILoginActions) and wrap them so they
        // implement the new interface (i.e. ILoginActions2)
        ConvertOldPEIInterfaces(application);
        if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
        {
          log.Info("Dynamic Loads loaded.");
        }
      }
      catch (Exception ex)
      {
        String errorMessage = "Error loading Dynamic Loads: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(ex));
        return retVal;
      }
      return retVal;
    }
    
    /// <summary>
    /// Loads CustomerObjects.  This method will load any additional
    /// objects specified here, and any additional objects as specified 
    /// by the AppWarmUpHelper.
    /// </summary>
    public static ReturnValue LoadObjects()
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "LoadObjects successful.",
       StackTracer.MakeXPException("(internal stacktrace)"));
      try
      {
        log.Info("Loading Custom Objects.");
        // retrieving necessary elements.
        String strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
        IApplication application = ApplicationManager.GetInstance().GetApplication(strApplicationName);
        ConfigVarPack vpConfig = (ConfigVarPack) application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID);
        String strImageServerBaseURL = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_IMAGESERVER_BASE_URL));
        String strSecureImageServerBaseURL = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_SECURE_IMAGESERVER_BASE_URL));
        String strImageServerConnectionURL = XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_IMAGESERVER_CONNECTION_URL));
        IAppWarmUpHelper ptAppWarmUpHelper = (IAppWarmUpHelper) AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_APP_WARMUP_HELPER);
        CachingManager cmCachingManager = application.GetCachingManager();
        // initialize static memebers.
        LoginHandlers.StaticInit(vpConfig);
        try
        {
          // Load ClassTypeDescManager
          log.Info("Loading the class type descriptor.");
          // LoadSettings and the Resouces must be called before ClassTypeDescManager is instantiated	
          ClassTypeManager openerTypeMgr = new ClassTypeManager(cmCachingManager);
          cmCachingManager.SetEntry(AppWarmUp.KEY_CACHEMAN_CLASS_TYPE_MANAGER,
           openerTypeMgr);
        }
        catch (Exception ex)
        {
          String errorMessage = "Failed to load ClassTypeDescManager!";
          log.Fatal(ex, errorMessage);
          retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
           XPException.GetInstance(ex));
          return retVal;
        }
        // initialize custom objects from the helper.
        if (null != ptAppWarmUpHelper)
        {
          retVal = ptAppWarmUpHelper.LoadObjects(vpConfig, strApplicationName,
           strImageServerBaseURL, strSecureImageServerBaseURL, application.GetCachingManager());
          }
        if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
        {
          log.Info("Custom Objects loaded.");
        }
      }
      catch (Exception ex)
      {
        String errorMessage = "Error loading Custom Objects: ";
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, errorMessage,
         XPException.GetInstance(ex));
      }
      return retVal;
    }
    
    /// <summary>
    /// Helper that loads settings from a folder.
    /// </summary>
    /// <param name="app"></param>
    /// <param name="strSettingsFolder"></param>
    /// <param name="strLibHomePath"></param>
    public static ReturnValue HelperLoadSettings(IApplication app, String strSettingsFolder,
     String strLibHomePath)
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "HelperLoadSettings successful.",
       StackTracer.MakeXPException("(internal stacktrace)"));
      try
      {
        XPDirectory settingsDir = new XPDirectory(strSettingsFolder);
        if (settingsDir.IsDirectory())
        {
          XPFileSystemObject[] fileObjects = settingsDir.GetChildren();
          String strFileName;
          int nLength = fileObjects.Length;
          int nEndIndex;
          Object[] settings = null;
          String strCacheString;
          for (int i = 0;i < nLength;i++)
          {
            // There are only two types of objects in file system directories:
            // files and directories.  Therefore, if not a directory, it must
            // be a file. If directory, recurse to it.
            if (fileObjects[i].IsDirectory())
            {
              XPDirectory dir = (XPDirectory) fileObjects[i];
              log.Info("Attempting Dynamic Discovery in folder {0}...", dir.GetName());
              String subDirPath = GeneralHelpers.FormatPath(dir.GetFullPath(),
               AppWarmUp.PATH_SEPARATOR, true);
              HelperLoadSettings(app, subDirPath, strLibHomePath);
            } else
            {
              strFileName = fileObjects[i].GetName();
              nEndIndex = XPStringUtility.IndexOf(strFileName, AppWarmUp.SUFFIX_XML_FILE);
              // Ignore non-xml files
              if ((nEndIndex >= 0) && (strFileName.Length == AppWarmUp.SUFFIX_XML_FILE.Length + nEndIndex))
              {
                strCacheString = XPStringUtility.Substring(strFileName, 0,
                 nEndIndex);
                log.Info("Attempting Dynamic Discovery of: {0}", strCacheString);
                // make sure we catch openfoundation bugs
                settings = null;
                try
                {
                  settings = XPDynamicDiscovery.GetInstancesFromXML(strSettingsFolder + strFileName,
                   strLibHomePath);
                  if (null != settings)
                  {
                    log.Info(" Found: " + settings.Length + " instances of interface described in " + strCacheString);
                    }
                }
                catch (Exception ex)
                {
                  String msg = "Fatal Exception loading lib file: " + strSettingsFolder + strFileName;
                  log.Fatal(ex, msg);
                  retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg,
                   XPException.GetInstance(msg, ex));
                  return retVal;
                }
                if (settings == null)
                {
                  String msg = "Dynamic Discovery returned a null object for the \'" + strCacheString + "\' setting.  Cannot load.";
                  log.Error(msg);
                  retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg,
                   new XPException(msg));
                  return retVal;
                }
                app.GetCachingManager().SetEntry(strCacheString, settings);
                }
            }
          }
        }
      }
      catch (Exception e)
      {
        String msg = "HelperLoadSettings: Unable to load custom settings.";
        log.Fatal(e, msg);
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL, msg, XPException.GetInstance(msg,
         e));
        return retVal;
      }
      return retVal;
    }
    
    /// <summary>
    /// Helper that converts older PEI interfaces into new PEI
    /// interfaces using helper objects that wrap both types
    /// of interfaces.
    /// TODO: This conversion should probably be specified in
    /// the xml file used to load the PEI so this can be done
    /// automatically in the future instead of hardcoding it here.
    /// </summary>
    /// <param name="app"></param>
    public static void ConvertOldPEIInterfaces(IApplication app)
    {
      // TODO: Bug#44530
      // This needs to be implemented more flexibly so it can convert PEIs
      // that live in portal* projects.
      // Have the wrapper objects dynamically discovered by a single xml file.
      // They would all implement a PEIWrapper interface. which would include
      // the following methods:
      // String GetPEIStorageName();
      // boolean SetPEIImplementation(Object _PEIImplementation);
      // We would then load up the original PEIs and wrapper objects, loop
      // through the wrapper objects, get the discovered PEIs for each wrapper
      // object, create new wrapper objects for each PEI instance, and store the
      // list back under the same name.
      // Get the original interfaces
      Object[] originalPEIs = (Object[]) app.GetCachingManager().GetEntry("LoginActions");
      if (null != originalPEIs)
      {
        Object[] newPEIWrappers = new Object[originalPEIs.Length];
        for (int x = 0;x < originalPEIs.Length;x++)
        {
          newPEIWrappers[x] = new LoginPEIContainer((ILoginActions) originalPEIs[x]);
          }
        app.GetCachingManager().SetEntry("LoginActions", newPEIWrappers);
        }
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
     
    public virtual XPArrayList getMsg(int stepid, Platform platform, ReturnValue ret)
    {
      // Use the diagnostic helper to find the right message        
      return RecommendationHelper.getList(stepid, platform, ret.getResultCode(),
       m_diag);
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
     
    public virtual StartupTaskMessage[] getMsgArray()
    {
      return m_diag;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
     
    public virtual int getNumMsgs()
    {
      return m_diag.Length;
    }
  }
}