/*
 * Created on Apr 13, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.application;
using com.plumtree.uiinfrastructure.application.notification;
using com.plumtree.uiinfrastructure.application.startup.messages;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.uiinfrastructure.util;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.application.startup
{
  /// <summary>
  /// Loads the Version Information.
  /// </summary>
  /// <author>DavidP</author>
  public class LoadVersionHelper : IStartupTask
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.application.startup.LoadVersionHelper");
    private String m_applicationName = "";
    private ConfigVarPack m_vpConfig = null;
    private String m_strImageServerBaseURL = null;
    private String m_strSecureImageServerBaseURL = null;
    private String m_strImageServerConnectionURL = null;
    private IApplication m_application = null;
    // array of messages
    private static StartupTaskMessage[] m_diag = {new StartupTaskMessage(
    // steps are one-based; step zero is the init call
    // DAP TODO this really needs to be in an external file
    0, Platform.ALL, PT_STARTUP_RESULTCODES.FATAL, "LoadVersionHelper problem",
     "RECOMMENDATION: Check that version.xml is located in the correct directory " + "(I.E. plumtree\\settings\\portal) and is not corrupt.")};
    public LoadVersionHelper() : base()
    {
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
     
    public virtual String getTaskName()
    {
      return "LoadVersionHelper";
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
     
    public virtual int getTaskID()
    {
      return PTStartupTasks.LOADVERSION.getOrdinal();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
     
    public virtual int getNumSteps()
    {
      return 1;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
     
    public virtual ReturnValue init(String _strApplicationName)
    {
      m_applicationName = _strApplicationName;
      return new ReturnValue(PT_STARTUP_RESULTCODES.OK, "LoadVersionHelper init successful",
       null);
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
     
    public virtual ReturnValue executeStep(int sequentialStepID)
    {
      ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "Version information loaded.",
       null);
      try
      {
        log.Info("Loading Version: version.xml.");
        // uses version.xml config file to initialize.
        VersionHelper versionHelper = new VersionHelper(XPConvert.ToString(AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_CONFIG)));
        // set the static content.
        versionHelper.LoadVersionXML();
        log.Info(VersionHelper.GetVersionMessage());
      }
      catch (Exception ex)
      {
        retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK, "Error loading Version information: ",
         XPException.GetInstance(ex));
      }
      return retVal;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
     
    public virtual XPArrayList getMsg(int stepid, Platform platform, ReturnValue ret)
    {
      // Use the diagnostic helper to find the right message        
      return RecommendationHelper.getList(stepid, platform, ret.getResultCode(),
       m_diag);
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
     
    public virtual StartupTaskMessage[] getMsgArray()
    {
      return m_diag;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
     
    public virtual int getNumMsgs()
    {
      return m_diag.Length;
    }
  }
}