/*
 * Created on Mar 13, 2005
 */
 
using System;
using com.plumtree.uiinfrastructure.application.notification;
namespace com.plumtree.uiinfrastructure.application.startup
{
  /// <summary>
  /// Enumerator for startup tasks for uiinfrastructure.
  /// Add each new startup class to this helper.
  /// Other startup tasks must be added to the task manager 
  /// dynamically.
  /// This enumerator automatically numbers each task sequentially.
  /// </summary>
  /// <author>DavidP</author>
  public class PTStartupTasks
  {
    // private task instance, ordinal management values
    private static int m_nextOrdinal = 0;
    private IStartupTask m_task;
    private int m_ordinal = 0;
    // public startup task elements
    //	 WARNING: Do NOT change the order of these entries; they are highly interdependent
    public static readonly PTStartupTasks INITBASE = new PTStartupTasks(new InitPortalObjects(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks DEVKIT = new PTStartupTasks(new CheckDevKit(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks INITPATHS = new PTStartupTasks(new InitializePaths(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks INITAPPLICATION = new PTStartupTasks(new InitializeApplication(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks INITAPPWARMUP = new PTStartupTasks(new InitializeAppWarmUpHelper(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks INITIMGSERVER = new PTStartupTasks(new InitializeImageServerHelper(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks INITCONFIG = new PTStartupTasks(new InitializeConfigHelper(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks INITRESOURCE = new PTStartupTasks(new InitializeResourceManager(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks LOADVERSION = new PTStartupTasks(new LoadVersionHelper(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks INITAS = new PTStartupTasks(new InitializeActivitySpaces(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks INITVARPACKS = new PTStartupTasks(new InitializeVarPacks(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks INITIMGSERVERII = new PTStartupTasks(new InitializeImageServerHelperPartII(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks INITOBJECTS = new PTStartupTasks(new InitializeObjects(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks LOADCUSTOMLOADS = new PTStartupTasks(new LoadCustomLoads(),
     ++m_nextOrdinal);
    public static readonly PTStartupTasks CHECKDBVER = new PTStartupTasks(new CheckDBVersion(),
     ++m_nextOrdinal);
    // public consumable enumeratable array list of tasks
    // this list must be static and structured for order preservation
    // WARNING: Do NOT change the order of these entries; they are highly interdependent
    // NOTE: Even though this variable is static and final, the elements in the array
    // could be changed.  Do not do this!
    public static readonly PTStartupTasks[] LIST = {INITBASE, DEVKIT, INITPATHS,
     INITAPPLICATION, INITAPPWARMUP, INITIMGSERVER, INITCONFIG, INITRESOURCE,
     LOADVERSION, INITAS, INITVARPACKS, INITIMGSERVERII, INITOBJECTS, LOADCUSTOMLOADS,
     CHECKDBVER};
    // private constructor
    private PTStartupTasks(IStartupTask task, int ordinal)
    {
      m_ordinal = ordinal;
      m_task = task;
    }
    
    /// <returns>
    /// the task interface for this enumerated task 
    /// instance
    /// </returns>
    public virtual IStartupTask getTask()
    {
      return m_task;
    }
    
    /// <returns>
    /// the integer ordinal (one-based) of this 
    /// enumerated task instance
    /// </returns>
    public virtual int getOrdinal()
    {
      return m_ordinal;
    }
  }
}