using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.approval
{
  /// <summary>
  /// This View represents a section of the main page
  /// of the Misc Editor.
  /// This View is printed out by Misc2DP.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class CommentView : EditorView
  {
    // ***CHANGE_HERE*** Change the name of the class.
    // ***COMMENT_1*** Delete the comments prefixed by CHANGE_HERE and COMMENT
    //       when you are done.
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.approval.CommentView");
    // ***CHANGE_HERE*** Set the class name.
    // ***COMMENT_1*** All the Views in Plumtree need to have a unique name.
    public const String STR_MVC_CLASS_NAME = "CommentView";
    // Strings to localize.
    // ***CHANGE_HERE*** List of the Strings that need to be localized.
    // ***COMMENT_1*** The names should all start with "_ui_".
    // HTML Object names.
    // ***CHANGE_HERE*** Declare some constans for each
    //        HTML object you are using in the display method
    // ***COMMENT_1*** You should follow the HTMLElements naming standards:
    //       <select> starts with in_se
    //       <input button> starts with in_bu
    //       <input text> starts with in_tx
    //       <input checkbox> starts with in_cb
    //       <input radio> starts with in_ra
    //       <textarea> starts with in_ta
    
    /// <summary>
    /// Localized String: Unable to display the HTML for PopUpView (Display). 
    /// </summary>
    public const String HTMLINPUT_POPUPSTRING = "in_tx_PopUpString";
    public const String HTMLBUTTON_DONE = "in_bu_done";
    public const String HTMLBUTTON_CANCEL = "in_bu_cancel";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new CommentView();
    }
    
    /// <summary>
    /// Returns the HTML for the View.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public override HTMLElement Display()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLInput myInput;
      try
      {
        // ***CHANGEHERE*** Write the HTML for your View.
        // ***COMMENT_1*** The HTML below can be used as a reference
        //       to write some HTMLElements.
        // ***COMMENT_2*** Also look at PropMainView and PropAdvView in template1
        //      for more HTML example.
        result.AddInnerHTMLString("\n");
        HTMLComment myComment = new HTMLComment("Begin: Display PopUpView");
        result.AddInnerHTMLElement(myComment);
        result.AddInnerHTMLString("\n");
        myTable = new HTMLTable();
        result.AddInnerHTMLElement(myTable);
        myTable.SetWidth("100%");
        myTable.SetBorder("0");
        myTable.SetCellPadding("5");
        myTable.SetCellSpacing("0");
        // 1st Row begin
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetNoWrap(true);
        // ***EXAMPLE*** How to display an input text?
        // ***<input text>*** begins here
        myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_POPUPSTRING,
         "");
        myCell.AddInnerHTMLElement(myInput);
        myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
        // ***EXAMPLE*** This is how you can HTMLEncode your Strings
        myInput.SetValue("bob");
        myInput.SetMaxLength(63);
        myInput.SetSize(50);
        // ***<input text>*** ends here
        // 1st Row end
        // 2nd Row begin
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        // ***EXAMPLE*** How to implement a repost action?
        // ***Done Action*** begins here
        myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_DONE, "");
        myCell.AddInnerHTMLElement(myInput);
        myInput.SetValue(m_asOwner.GetString(32, ResourceConstants.INFRASTRUCTURE));
        myInput.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
        myInput.SetOnClick("OnDone();");
        // ***Done Action*** ends here
        // ***Cancel Action*** begins here
        myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_CANCEL,
         "");
        myCell.AddInnerHTMLElement(myInput);
        myInput.SetValue(m_asOwner.GetString(33, ResourceConstants.INFRASTRUCTURE));
        myInput.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
        myInput.SetOnClick("window.close();");
        // ***Cancel Action*** ends here
        // 2nd Row end
        result.AddInnerHTMLString("\n");
        myComment = new HTMLComment("End: Display PopUpView");
        result.AddInnerHTMLElement(myComment);
        result.AddInnerHTMLString("\n");
      }
      catch (Exception e)
      {
        // ***CHANGE_HERE*** Update the error messages (log and exception).
        log.Error(e, "PopUpView::Display An error occured when trying to display the HTML.");
        throw new XPException(m_asOwner.GetString(34, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return result;
    }
    
    /// <summary>
    /// This method returns the HTMLScript that encompasses the Javascript on the page.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    /// <returns>HTMLScript</returns>
    public override HTMLScript DisplayJavascript()
    {
      HTMLScript myScript = new HTMLScript("text/javaScript");
      XPStringBuilder sb;
      try
      {
        // JF- OnDone JavaScript function
        sb = new XPStringBuilder(200);
        sb.Append("function OnDone()\n");
        sb.Append("   window.close();\n");
        sb.Append("}\n");
        myScript.AddInnerHTMLString(sb.ToString());
      }
      catch (HTMLException he)
      {
      }
      return myScript;
    }
    // ***CHANGE_HERE*** Update the class name below.
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
  }
}