using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.web;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.common
{
  /// <summary>
  /// This class implements a frame set page.
  /// </summary>
  /// <author>robertz</author>
  public abstract class FrameSetDP : IDisplayPage
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.common.FrameSetDP");
    
    /// <summary>
    /// Frame names for JS manipulation 
    /// </summary>
    public const String TOP_FRAME = "TopFrame";
    public const String BODY_FRAME = "BodyFrame";
    public const String BOTTOM_FRAME = "BottomFrame";
    
    /// <summary>
    /// Activity Space 
    /// </summary>
    protected AActivitySpace m_asOwner;
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IDisplayPage#display()"></seealso>
    public virtual void Display(IWebData webData)
    {
      // Create an HTMLPage
      HTMLPage myPage = null;
      myPage = new HTMLPage();
      HTMLHead myHead = new HTMLHead();
      myPage.SetHead(myHead);
      HTMLBody myBody = new HTMLBody();
      myBody.SetMarginWidth(CommonHTMLStrings.ZERO);
      myBody.SetMarginHeight(CommonHTMLStrings.ZERO);
      myBody.SetTopMargin(CommonHTMLStrings.ZERO);
      myBody.SetLeftMargin(CommonHTMLStrings.ZERO);
      try
      {
        HTMLElementCollection styleSheetLinks = m_asOwner.GetStyleSheetLinks();
        if (null != styleSheetLinks)
        {
          myHead.AddInnerHTMLElement(styleSheetLinks);
        }
        String strFrameSetTitle = XPHttpUtility.HtmlEncode(GetFrameSetTitle());
        HTMLTitle myTitle = new HTMLTitle(strFrameSetTitle);
        myHead.AddInnerHTMLElement(myTitle);
        HTMLFrameSet frameset = new HTMLFrameSet();
        frameset.SetRows(GetFrameSetAttributes());
        frameset.SetTitle(strFrameSetTitle);
        HTMLFrame frameHeader = new HTMLFrame();
        frameHeader.SetFrameBorder(CommonHTMLStrings.ZERO);
        frameHeader.SetName(GetFrameNameTop());
        String strFrameTitleTop = GetFrameTitleTop();
        if (null != strFrameTitleTop)
        {
          frameHeader.SetTitle(strFrameTitleTop);
        }
        ASURL frameURLTop = GetFrameURLTop();
        String strFrameURLTop;
        if (null != frameURLTop)
        {
          strFrameURLTop = frameURLTop.GetURLAsString();
        } else
        {
          strFrameURLTop = GetFrameURLStringTop();
        }
        frameHeader.SetSrc(strFrameURLTop);
        if (!GetFrameScrollTop())
        {
          frameHeader.SetScrolling("no");
        }
        frameset.AddInnerHTMLElement(frameHeader);
        HTMLFrame frameBody = new HTMLFrame();
        frameBody.SetFrameBorder(CommonHTMLStrings.ZERO);
        frameBody.SetName(GetFrameNameBody());
        String strFrameTitleBody = GetFrameTitleBody();
        if (null != strFrameTitleBody)
        {
          frameBody.SetTitle(strFrameTitleBody);
        }
        frameBody.SetSrc(GetFrameURLBody().GetURLAsString());
        if (!GetFrameScrollBody())
        {
          frameBody.SetScrolling("no");
        }
        frameset.AddInnerHTMLElement(frameBody);
        ASURL bottomURL = GetFrameURLBottom();
        if (bottomURL != null)
        {
          HTMLFrame frameFooter = new HTMLFrame();
          frameFooter.SetFrameBorder(CommonHTMLStrings.ZERO);
          frameFooter.SetName(GetFrameNameBottom());
          String strFrameTitleBottom = GetFrameTitleBottom();
          if (null != strFrameTitleBottom)
          {
            frameFooter.SetTitle(strFrameTitleBottom);
          }
          frameFooter.SetSrc(bottomURL.GetURLAsString());
          if (!GetFrameScrollBottom())
          {
            frameFooter.SetScrolling("no");
          }
          frameset.AddInnerHTMLElement(frameFooter);
        }
        /*
         HTMLElementCollection noFramesBody = GetNoFrames();
         if (null != noFramesBody)
         {
         HTMLElement noFrame = new HTMLElementCollection();
         noFrame.AddInnerHTMLString("<NOFRAMES>");
         noFrame.AddInnerHTMLElement(noFramesBody);
         noFrame.AddInnerHTMLString("</NOFRAMES>");
         
         // This is not currently supported.
         myPage.SetNoFrames(noFrame);
         }
         */
         
        myPage.SetFrameSet(frameset);
      }
      catch (Exception e)
      {
        log.Error(e, "Frameset unable to set frames.");
        // Perhaps this should just raise an error that goes to the error page?
        myPage = new HTMLPage();
        myBody = new HTMLBody();
        try
        {
          myPage.SetBody(myBody);
          HTMLTable myTable = new HTMLTable();
          myBody.AddInnerHTMLElement(myTable);
          HTMLTableRow myRow = new HTMLTableRow();
          myTable.AddInnerHTMLElement(myRow);
          HTMLTableCell myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          myCell.AddInnerHTMLString(m_asOwner.GetString(36, ResourceConstants.INFRASTRUCTURE));
          }
        catch (Exception ee)
        {
          log.Error(ee, "Frameset unable to display error frame.");
        }
      }
      webData.SetData(myPage);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public abstract String GetName();
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#GetParent()"></seealso>
    public virtual AActivitySpace GetOwner()
    {
      return m_asOwner;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Init(AActivitySpace)"></seealso>
    public virtual void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
    }
    /////////////////////////////////
    // Methods to optionally override
    /////////////////////////////////
    
    /// <summary>
    /// Set Frame Scroll Body
    /// </summary>
    /// <returns>boolean true implies scrolling</returns>
    public virtual bool GetFrameScrollBody()
    {
      return false;
    }
    
    /// <summary>
    /// Set Frame Scroll Bottom
    /// </summary>
    /// <returns>boolean true implies scrolling</returns>
    public virtual bool GetFrameScrollBottom()
    {
      return false;
    }
    
    /// <summary>
    /// Set Frame Scroll Top
    /// </summary>
    /// <returns>boolean true implies scrolling</returns>
    public virtual bool GetFrameScrollTop()
    {
      return false;
    }
    
    /// <summary>
    /// Set FrameSet Attributes
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetFrameSetAttributes()
    {
      return "50px,50%";
    }
    
    /// <summary>
    /// Set Frame URL Bottom
    /// </summary>
    /// <returns>ASURL, null implies no bottom frame</returns>
    public virtual ASURL GetFrameURLBottom()
    {
      return null;
    }
    
    /// <summary>
    /// Set Frame URL Top
    /// This method is only called if the GetFrameURLTop
    /// method returns null; 
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetFrameURLStringTop()
    {
      return null;
    }
    
    /// <summary>
    /// Set Frame Name Top
    /// The frame name and title are used by 508 users.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetFrameNameTop()
    {
      return TOP_FRAME;
    }
    
    /// <summary>
    /// Set Frame Name Body
    /// The frame name and title are used by 508 users.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetFrameNameBody()
    {
      return BODY_FRAME;
    }
    
    /// <summary>
    /// Set Frame Name Bottom
    /// The frame name and title are used by 508 users.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetFrameNameBottom()
    {
      return BOTTOM_FRAME;
    }
    
    /// <summary>
    /// Set Frame Title Top
    /// The frame name and title are used by 508 users.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetFrameTitleTop()
    {
      return null;
    }
    
    /// <summary>
    /// Set Frame Title Body
    /// The frame name and title are used by 508 users.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetFrameTitleBody()
    {
      return null;
    }
    
    /// <summary>
    /// Set Frame Title Bottom
    /// The frame name and title are used by 508 users.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetFrameTitleBottom()
    {
      return null;
    }
    
    /// <summary>
    /// Set NOFRAMES Body Element
    /// The NOFRAMES element is useful for 508 users.
    /// </summary>
    /// <returns>HTMLElementCollection</returns>
    public virtual HTMLElementCollection GetNoFrames()
    {
      return null;
    }
    ///////////////////
    // Abstract methods
    ///////////////////
    
    /// <summary>
    /// Set FrameSet Title
    /// This string will be HTML Encoded for
    /// safe display by this class.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetFrameSetTitle();
    
    /// <summary>
    /// Set Frame URL Body
    /// </summary>
    /// <returns>ASURL</returns>
    public abstract ASURL GetFrameURLBody();
    
    /// <summary>
    /// Set Frame URL Top
    /// </summary>
    /// <returns>ASURL</returns>
    public abstract ASURL GetFrameURLTop();
  }
}