using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.errordisplay;
// Framework Imports
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.common
{
  /// <author>NaveenP</author>
  public class PassthruControl : IControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.common.PassthruControl");
    
    /// <summary>
    /// Object Name for AS 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "PassthruControl";
    public const String QS_PASSTHRU_CONTROL = "passthruControl";
    public const String QS_MAIN_CONTROL = "mainControl";
    protected IModel m_model;
    protected AActivitySpace m_asOwner;
    
    /// <returns>Redirect</returns>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      String strPassthruName;
      String strMainName;
      // Find pagination ID
      String[] sValues = (String[]) arguments.GetElement(QS_PASSTHRU_CONTROL);
      if (sValues != null)
      {
        strPassthruName = sValues[0];
      } else
      {
        log.Warn("Unable to find passthru control.");
        strPassthruName = "";
      }
      sValues = (String[]) arguments.GetElement(QS_MAIN_CONTROL);
      if (sValues != null)
      {
        strMainName = sValues[0];
      } else
      {
        log.Warn("Unable to find main control.");
        strMainName = "";
      }
      IControl passthruControl = m_asOwner.GetControl(strPassthruName);
      IControl mainControl = m_asOwner.GetControl(strMainName);
      if (passthruControl != null)
      {
        passthruControl.CheckActionSecurityAndExecute(arguments);
      }
      if (mainControl != null)
      {
        return mainControl.CheckActionSecurityAndExecute(arguments);
      } else
      {
        UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
         m_asOwner.GetString(2423, ResourceConstants.PORTALADMINMSGS), m_asOwner.GetString(2424,
         ResourceConstants.PORTALADMINMSGS));
        UIErrorHandler.RaiseError(myError, m_asOwner.GetSubSession(), m_asOwner.GetApplication());
        return null;
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new PassthruControl();
    }
    
    /// <summary>
    /// Return the class name.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Inits the class with the associated model and activity space.
    /// If this method is overridden in a sub-class, this version of
    /// Init must be called as super.Init();
    /// </summary>
    /// <param name="model"></param>
    /// <param name="space"></param>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      m_model = model;
      m_asOwner = space;
    }
  }
}