using System;
using XPHashtable=com.plumtree.openfoundation.util.XPHashtable;
using XPSystem=com.plumtree.openfoundation.util.XPSystem;
namespace com.plumtree.uiinfrastructure.common
{
  /// <summary>
  /// The TimeTracker Class is used to measure time differences, in milliseconds.
  /// </summary>
  /// <author>
  /// scottc
  /// Created on May 3, 2003
  /// </author>
  public class TimeTracker
  {
    private long m_startTime;
    private XPHashtable m_checkpoints;
    public TimeTracker()
    {
      m_startTime = XPSystem.GetCurrentTimeMilliSec();
    }
    
    /// <summary>
    /// The TimeTracker is reset.  Alternative to creating a new one.
    /// </summary>
    public virtual void Reset()
    {
      m_startTime = XPSystem.GetCurrentTimeMilliSec();
      m_checkpoints = null;
    }
    
    /// <summary>
    /// The time in milliseconds since the TimeTracker creation or the last Reset()
    /// is returned.
    /// </summary>
    /// <returns>long</returns>
    public virtual long GetTotalTime()
    {
      return XPSystem.GetCurrentTimeMilliSec() - m_startTime;
    }
    
    /// <summary>
    /// SetCheckpoint is used to create a time checkpiont for later retrieval.
    /// The checkpoint is identified by a string, and stored in an internal
    /// XPHashtable with that string as a key.
    /// </summary>
    /// <param name="checkpoint"></param>
    public virtual void SetCheckpoint(String checkpoint)
    {
      if (m_checkpoints == null)
      {
        m_checkpoints = new XPHashtable();
      }
      m_checkpoints.PutElement(checkpoint, new TimeContainer(XPSystem.GetCurrentTimeMilliSec()));
      }
    
    /// <summary>
    /// GetCheckpointTimeFromStart attempts to retrieve a checkpoint with the key given.
    /// It returns -1 if there is no checkpoint by the given name.
    /// It returns the time in milliseconds between the creation of the TimeTracker 
    /// or its last Reset() and the checkpoint time.
    /// </summary>
    /// <param name="checkpoint"></param>
    /// <returns>long the time between the start of the time and the checkpoint</returns>
    public virtual long GetCheckpointTimeFromStart(String checkpoint)
    {
      if (m_checkpoints == null)
      {
        return -1;
      } else
      {
        TimeContainer myTimeContainer = (TimeContainer) m_checkpoints.GetElement(checkpoint);
        if (myTimeContainer == null)
        {
          return -1;
        } else
        {
          return myTimeContainer.GetTime() - m_startTime;
        }
      }
    }
    
    /// <summary>
    /// GetCheckpointTimeFromStart attempts to retrieve the two checkpoints with
    /// the keys given.
    /// It returns -1 if either of the checkpoints do not exist.
    /// It returns the time in milliseconds between the start checkpoint and the
    /// end checkpoint time.
    /// </summary>
    /// <param name="startCheckpoint"></param>
    /// <param name="endCheckpoint"></param>
    /// <returns>long the time between the two checkpoints</returns>
    public virtual long GetCheckpointTimeFromCheckpoint(String startCheckpoint,
     String endCheckpoint)
    {
      if (m_checkpoints == null)
      {
        return -1;
      } else
      {
        TimeContainer myStartTimeContainer = (TimeContainer) m_checkpoints.GetElement(startCheckpoint);
        if (myStartTimeContainer == null)
        {
          return -1;
        }
        TimeContainer myEndTimeContainer = (TimeContainer) m_checkpoints.GetElement(endCheckpoint);
        if (myEndTimeContainer == null)
        {
          return -1;
        }
        return myEndTimeContainer.GetTime() - myStartTimeContainer.GetTime();
        }
    }
  }
}