/*
 * Purpose:
 * Author: 	D. Phipps Feb 24, 2005
 */
 
using System;
using com.plumtree.uiinfrastructure.diagnostic.reporter;
using com.plumtree.uiinfrastructure.diagnostic.testinterface;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.application.notification;
namespace com.plumtree.uiinfrastructure.diagnostic
{
  /// <summary>
  /// Manages the test results for a single task.  Used to build up a 
  /// report array that can be used to record results.
  /// Has hooks back to progress manager class.
  /// </summary>
  /// <author>DavidP</author>
  public class IndividualTestProgressTracker : IProgressTracker
  {
    private int m_min = 0;
    private int m_max = 1;
    private int m_current = 0;
    private int m_currentTask = 0;
    private int m_totalTasks = 0;
    private int m_completedSteps = 0;
    private bool m_bCompleted = false;
    private String m_name = "";
    private XPStringBuilder m_log = new XPStringBuilder();
    private ProgressManager m_mgr = null;
    private bool m_bVerbose = false;
    // Array of test objects and their state
    private ReturnValue[] m_testResults = null;
    
    /// <summary>
    /// Sets the number of tasks to be run
    /// </summary>
    public virtual void init(int iTask, int iNumTasks, bool verbose)
    {
      m_currentTask = iTask;
      m_totalTasks = iNumTasks;
      m_bCompleted = false;
      m_bVerbose = verbose;
    }
    public virtual void setName(String name)
    {
      m_name = name;
    }
    
    /// <summary>
    /// Returns number of tasks to be run
    /// </summary>
    /// <returns></returns>
    public virtual int getNumTasks()
    {
      return m_totalTasks;
    }
    
    /// <summary>
    /// Returns number of tasks to be run
    /// </summary>
    /// <returns></returns>
    public virtual int getCurTask()
    {
      return m_currentTask;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#getMax()
     */
     
    public virtual int getNumSteps()
    {
      return m_max;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#setNumSteps(int)
     */
     
    public virtual void setNumSteps(int max)
    {
      m_max = max;
      m_testResults = new ReturnValue[max + 2];
    }
    public virtual void setRetval(int step, ReturnValue retval)
    {
      if (step < m_max + 1)
      {
        m_testResults[step] = retval;
      }
    }
    public virtual ReturnValue getRetval(int step)
    {
      if (step < m_max + 1)
      {
        return m_testResults[step];
      }
      return null;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#increment()
     */
     
    public virtual void increment()
    {
      m_current++;
      if (m_current > m_max + 1)
      {
        m_current = m_max + 1;
        complete();
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#decrement()
     */
     
    public virtual void decrement()
    {
      m_current--;
      if (m_current < m_min)
      {
        m_current = m_min;
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#complete()
     */
     
    public virtual void complete()
    {
      // TODO: resource string?
      m_bCompleted = true;
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#notify(java.lang.String)
     */
     
    public virtual void notifyevent(String message, bool verbose)
    {
      m_log.Append(message);
      // spy logging, if valid
      if (verbose && m_bVerbose)
      {
        Reporter.LogAndWrite(message);
      } else
      if (!verbose)
      {
        Reporter.LogAndWrite(message);
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#notify(java.lang.String)
     */
     
    public virtual void notifyevent(int iStep, ReturnValue retval)
    {
      m_log.Append(retval.getMsg());
      m_log.Append("\n");
      // spy logging, if valid
      Reporter.LogAndWrite(retval.getMsg());
    }
    
    /// <summary>
    /// Distinct hook back to manager for notify markers
    /// </summary>
    /// <param name="mgr"></param>
    public virtual void setManager(ProgressManager mgr)
    {
      m_mgr = mgr;
    }
    
    /// <summary>
    /// Returns true if the overall task completed, no matter what the status
    /// </summary>
    /// <author>DavidP</author>
    public virtual bool getCompleted()
    {
      return m_bCompleted;
    }
    
    /// <summary>
    /// sets the completed boolean
    /// </summary>
    public virtual void setCompleted(bool bCompleted)
    {
      m_bCompleted = bCompleted;
    }
    public virtual int getCompletedSteps()
    {
      return m_current;
    }
  }
}