/*
 * Author: 	D. Phipps Feb 24, 2005
 */
 
using System;
using com.plumtree.uiinfrastructure.application.notification;
namespace com.plumtree.uiinfrastructure.diagnostic.testinterface
{
  /// <summary>
  /// Used to track the progress of a single task.  An array
  /// of these objects can form reports of total startup.
  /// </summary>
  /// <author>DavidP</author>
  public interface IProgressTracker
  {
    
    /// <summary>
    /// Sets the number of tasks to be run
    /// </summary>
    void init(int iTask, int iNumTasks, bool verbose);
    
    /// <summary>
    /// Tracks th ename of this task.
    /// </summary>
    /// <param name="name"></param>
    void setName(String name);
    
    /// <summary>
    /// Returns number of tasks to be run
    /// </summary>
    /// <returns></returns>
    int getNumTasks();
    
    /// <summary>
    /// Returns number of tasks to be run
    /// </summary>
    /// <returns></returns>
    int getCurTask();
    
    /// <summary>
    /// Returns maximum value of progress indicator for this test
    /// </summary>
    /// <returns></returns>
    int getNumSteps();
    
    /// <summary>
    /// Gets a specific return value for a step.  Step 0 is init.
    /// </summary>
    /// <param name="min"></param>
    ReturnValue getRetval(int step);
    
    /// <summary>
    /// Sets a specific return value for a step.  Step 0 is init.
    /// </summary>
    /// <param name="min"></param>
    void setRetval(int step, ReturnValue retval);
    
    /// <summary>
    /// Sets the maximum for progress tracking
    /// </summary>
    /// <param name="min"></param>
    void setNumSteps(int max);
    
    /// <summary>
    /// Called by testrunner for each test step
    /// </summary>
    void increment();
    
    /// <summary>
    /// Called by testrunner when a test step fails 
    /// and has to be backtracked
    /// </summary>
    void decrement();
    
    /// <summary>
    /// Called by testrunner when test completes
    /// </summary>
    void complete();
    
    /// <summary>
    /// Called by testrunner to add a message to the message log
    /// </summary>
    /// <param name="message"></param>
    void notifyevent(String message, bool verbose);
    
    /// <summary>
    /// Called by testrunner to add a message to the message log
    /// </summary>
    /// <param name="message"></param>
    void notifyevent(int iStep, ReturnValue retval);
    
    /// <summary>
    /// Returns true if the overall task completed, no matter what the status
    /// </summary>
    /// <author>DavidP</author>
    bool getCompleted();
    
    /// <summary>
    /// sets the completed boolean
    /// </summary>
    void setCompleted(bool bCompleted);
    
    /// <summary>
    /// Gets the number of steps completed
    /// </summary>
    int getCompletedSteps();
  }
}