using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.common;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.uiinfrastructure.tips;
using com.plumtree.uiinfrastructure.statichelpers;
// Framework Imports
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.editor
{
  /// <summary>
  /// This is class is used to print out  the form tag, some hidden input and the
  /// postToSelf Javascript function. Editor display pages should extend this
  /// class.
  /// </summary>
  /// <author>Julie Fournier</author>
  public abstract class EditorDP : AFormDP
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.editor.EditorDP");
    
    /// <summary>
    /// JF- Associated AS class. 
    /// </summary>
    protected EditorAS m_asOwner;
    
    /// <summary>
    /// Javascript to add the head. 
    /// </summary>
    private HTMLScript m_Script = null;
    
    /// <summary>
    /// JF- This is for the content table cell spacing.  It will add some space
    /// on the left and on the right side of the table.  People who don't want
    /// that space can set it to 0 using the  SetContentTableCellSpacing
    /// method.
    /// </summary>
    private String m_sContentTableCellSpacing = "15";
    
    /// <summary>
    /// Returns the String to display on the left navigation menu,  linking to
    /// that page.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetLeftNavString();
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public abstract override String GetName();
    
    /// <summary>
    /// Add some JavaScript dynamically to this DP. This is a "one time"
    /// JavaScript. It will be deleted after it's printed out. For example, can
    /// be used to open another window.
    /// </summary>
    /// <param name="_script">the script to add to the page</param>
    public virtual void AddScriptToHead(HTMLScript _script)
    {
      m_Script = _script;
    }
    
    /// <summary>
    /// Creates the content table without the left nav bar.
    /// </summary>
    /// <param name="_Views"></param>
    /// <param name="_sSubHeaderTitle">
    /// Title for the sub header. Pass in null if you
    /// don't want the sub header. Warning: if you pass in null, any
    /// code from GetHTMLBelowSubHeader  will not be displayed either.
    /// </param>
    /// <returns>HTMLTable</returns>
    public virtual HTMLTable CreateContentTable(HTMLElement _Views, String _sSubHeaderTitle)
    {
      // Each Editor DP will have to call that method and pass in its array of Views.
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      // Editor Content table
      myTable = new HTMLTable();
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellPadding("0");
      myTable.SetCellSpacing(this.m_sContentTableCellSpacing);
      HTMLElement myHTML = GetHTMLBelowSubHeader();
      if (null != _sSubHeaderTitle || null != myHTML)
      {
        // 1st Row - sub header
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetWidth("100%");
        myCell.SetVAlign("top");
        if (null != _sSubHeaderTitle)
        {
          myCell.AddInnerHTMLElement(GetContentHeaderTable(_sSubHeaderTitle));
          }
        if (null != myHTML)
        {
          // JF- Add HTML from Editor below the sub header (e.g.: additional buttons, instructions...)
          myCell.AddInnerHTMLElement(myHTML);
        }
      }
      // 2nd Row - page content
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.AddInnerHTMLElement(_Views);
      return myTable;
    }
    
    /// <summary>
    /// Creates a table with the left nav bar and puts innerElement next to the
    /// left nav bar.
    /// </summary>
    /// <param name="_Views"></param>
    /// <returns>HTMLTable</returns>
    public virtual HTMLTable CreateLeftNavBar(HTMLElement _Views)
    {
      // JF- The HTML below puts together the left nav from EditorLeftNavView 
      // and the Views passed in argument.
      // Each Editor DP will have to call that method and pass in its array of Views.
      HTMLTable myMainTable;
      HTMLTableRow myMainTableRow;
      HTMLTableCell myMainTableCell;
      EditorLeftNavView myLeftNavView = (EditorLeftNavView) m_asOwner.GetView(EditorLeftNavView.STR_MVC_CLASS_NAME);
      // Start Main table
      myMainTable = new HTMLTable();
      myMainTable.SetBorder("0");
      myMainTable.SetCellPadding("0");
      myMainTable.SetCellSpacing("10");
      myMainTable.SetWidth("100%");
      // Main table 1st Row
      myMainTableRow = new HTMLTableRow();
      myMainTable.AddInnerHTMLElement(myMainTableRow);
      if (!((IEditorModelRO) m_asOwner.GetModel(m_asOwner.m_strModelName)).GetHideLeftNav())
      {
        // Row 1 - Cell 1
        myMainTableCell = new HTMLTableCell();
        myMainTableRow.AddInnerHTMLElement(myMainTableCell);
        myMainTableCell.SetWidth("225");
        myMainTableCell.SetVAlign("top");
        // Display Left Navigation Menu
        myMainTableCell.AddInnerHTMLElement(myLeftNavView.Display());
        myMainTableCell.AddInnerHTMLElement((m_asOwner.GetView(TipsView.STR_MVC_CLASS_NAME).Display()));
        // Display Spacer table
        myMainTableCell.AddInnerHTMLElement(GetSpacerTable());
      }
      // Row 2 - Cell 2
      myMainTableCell = new HTMLTableCell();
      myMainTableRow.AddInnerHTMLElement(myMainTableCell);
      myMainTableCell.SetWidth("100%");
      myMainTableCell.SetVAlign("top");
      //	add the error display view.
      myMainTableCell.AddInnerHTMLElement(m_asOwner.GetView(ErrorView.STR_MVC_CLASS_NAME).Display());
      // JF- Content header with page title
      // We get the title from the EditorLeftNavView because 
      // that title is whatever String was selected in the left 
      // navigation menu. 
      myMainTableCell.AddInnerHTMLElement(GetContentHeaderTable(((EditorDP) m_asOwner.GetCurrentPage()).GetPageTitle()));
      HTMLElement myHTML = GetHTMLBelowSubHeader();
      if (null != myHTML)
      {
        // JF- Banner with help link and instructions
        myMainTableCell.AddInnerHTMLElement(myHTML);
      }
      // Editor Content table
      HTMLTable myTable = new HTMLTable();
      myMainTableCell.AddInnerHTMLElement(myTable);
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellPadding("0");
      myTable.SetCellSpacing(this.m_sContentTableCellSpacing);
      // 1st Row
      HTMLTableRow myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      HTMLTableCell myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      // Editor content
      myCell.AddInnerHTMLElement(_Views);
      return myMainTable;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.AFormDP#DisplayHTMLBeforeForm()"></seealso>
    public override HTMLElementCollection DisplayHTMLBeforeForm()
    {
      HTMLElementCollection myResult = base.DisplayHTMLBeforeForm();
      if (AccessStyles.STANDARD != m_asOwner.GetAccessStyle())
      {
        if (null == myResult)
        {
          myResult = new HTMLElementCollection();
        }
        // JF- Prints out the Header View.
        EditorHeaderView myHeaderView = (EditorHeaderView) m_asOwner.GetView(GetEditorHeaderViewName());
        myResult.AddInnerHTMLElement(myHeaderView.Display());
        myResult.AddInnerHTMLElement(m_asOwner.GetView(ErrorView.STR_MVC_CLASS_NAME).Display());
        }
      return myResult;
    }
    
    /// <summary>
    /// Returns the Browser title. For the Editor Framework, the browser title
    /// is the same as the  left navigation String of the current page.
    /// </summary>
    /// <returns>String</returns>
    public override String GetFormTitle()
    {
      return GetLeftNavString();
    }
    
    /// <summary>
    /// Returns the page title. By default it is the same as the left navigation string.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetPageTitle()
    {
      return XPHttpUtility.HtmlEncode(GetLeftNavString());
    }
    
    /// <summary>
    /// Returns the associated activity space class.
    /// </summary>
    /// <returns>AActivitySpace</returns>
    public override AActivitySpace GetOwner()
    {
      return m_asOwner;
    }
    
    /// <summary>
    /// Inits the class with the associated activity space passed in argument.
    /// </summary>
    /// <param name="parent"></param>
    public override void Init(AActivitySpace parent)
    {
      base.Init(parent);
      try
      {
        try
        {
          m_asOwner = (EditorAS) parent;
        }
        catch (Exception e)
        {
          throw XPException.GetInstance(e);
        }
      }
      catch (XPClassCastException cce)
      {
        log.Error("EditorDP::Init Unable to cast the Object " + parent + " to EditorAS. The following exception occured: " + cce);
        // JF- No need to extract, this is an unexpected error.
        throw new XPException("Unable to cast the Object " + parent + " to EditorAS.",
         cce);
      }
      // JF- In 508 mode we are using the buttons from AFormDP. 
      // The rest of the time, we are using the buttons from EditorHeaderView.
      SetShow508FormFooter(true);
    }
    
    /// <summary>
    /// Returns the HTML for that Page.
    /// </summary>
    /// <returns>HTMLElement</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AFormDP#PageDisplay(HTMLTableRow)"></seealso>
    public override HTMLElement PageDisplay()
    {
      HTMLElementCollection myResult = new HTMLElementCollection();
      // MJD- Header prints out before form in 508.
      // JF- No help banner in 508 mode
      if (AccessStyles.STANDARD == m_asOwner.GetAccessStyle())
      {
        // JF- Prints out the Header View.
        EditorHeaderView myHeaderView = (EditorHeaderView) m_asOwner.GetView(GetEditorHeaderViewName());
        myResult.AddInnerHTMLElement(myHeaderView.Display());
        // JF- Get and display the sub banner.	
        PTConstruct mySubBanner = m_asOwner.GetSubBanner();
        if (null != mySubBanner)
        {
          myResult.AddInnerHTMLElement(mySubBanner.GetHTMLElement());
        }
      }
      return myResult;
    }
    
    /// <summary>
    /// This is a hook to allow editors to supply a different header view if
    /// necessary. The editor returned here must be a subclass EditorHeaderView. 
    /// </summary>
    /// <returns>String</returns>
    protected virtual String GetEditorHeaderViewName()
    {
      return EditorHeaderView.STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// This method lets you set the cellspacing value for the content table.
    /// This is optional. The default is 15 and will work for most of Editors.
    /// However, if you want to get rid of the empty space on the left and
    /// right sides of the table, you might want to set it to 0.
    /// </summary>
    /// <param name="_sContentTableCellSpacing"></param>
    public virtual void SetContentTableCellSpacing(String _sContentTableCellSpacing)
    {
      m_sContentTableCellSpacing = _sContentTableCellSpacing;
    }
    
    /// <summary>
    /// Display the Javascripts for the Views.
    /// </summary>
    /// <returns>HTMLScriptCollection</returns>
    protected override HTMLScriptCollection DisplayJavaScriptFromChild()
    {
      HTMLScriptCollection myScripts = base.DisplayJavaScriptFromChild();
      // JF- Add JavaScript for the left navigation
      myScripts.AddInnerHTMLElement(m_asOwner.GetView(EditorLeftNavView.STR_MVC_CLASS_NAME).DisplayJavascript());
      if (null != m_Script)
      {
        // JF- Add other JavaScript
        myScripts.AddInnerHTMLElement(m_Script);
        // JF- Delete the script because we use it only once.
        m_Script = null;
      }
      // Add Javascript from DisplayJavascript PEI
      HTMLScriptCollection myPEIScripts = DisplayJavascriptHelper.DoDisplayJavascriptPEI(m_asOwner);
      if (myPEIScripts != null)
      {
        myScripts.AddInnerHTMLElement(myPEIScripts);
      }
      return myScripts;
    }
    
    /// <summary>
    /// Implement this method to add HTML below the sub header.
    /// </summary>
    /// <returns>HTMLElement</returns>
    protected virtual HTMLElement GetHTMLBelowSubHeader()
    {
      return HTMLConstructHelpers.GetSubHeader("", this.m_asOwner);
    }
    
    /// <summary>
    /// Helper method: returns page header table.
    /// </summary>
    /// <param name="_sTitle"></param>
    /// <returns>HTMLElement</returns>
    private HTMLElement GetContentHeaderTable(String _sTitle)
    {
      // JF- The HTML below displays the page header (not the main Editor header but the sub header).
      // The title printed out in that header is passed in argument.
      HTMLElementCollection myResult = new HTMLElementCollection();
      PTConstruct myContentHeader;
      try
      {
        myResult.AddInnerHTMLString("\n");
        HTMLComment myComment = new HTMLComment("Begin: Editor Page header with page title");
        myResult.AddInnerHTMLElement(myComment);
        myResult.AddInnerHTMLString("\n");
        // JF- Get the construct for the content header banner
        if (m_asOwner.GetIsAccessStyleStandard())
        {
          myContentHeader = UIConstructFactory.GetConstruct(PTBannerType.WIDE);
          myContentHeader.ReplaceNestedElementsWithHTMLString(PTBannerType.WIDE.TITLE_ELEMENT_NAME,
           _sTitle);
        } else
        {
          myContentHeader = UIConstructFactory.GetConstruct(PTBannerType.WIDE_508);
          myContentHeader.ReplaceNestedElementsWithHTMLString(PTBannerType.WIDE_508.TITLE_ELEMENT_NAME,
           _sTitle);
        }
        myResult.AddInnerHTMLElement(myContentHeader.GetHTMLElement());
        myResult.AddInnerHTMLString("\n");
        myComment = new HTMLComment("End: Editor Page header with page title");
        myResult.AddInnerHTMLElement(myComment);
        myResult.AddInnerHTMLString("\n");
      }
      catch (Exception e)
      {
        log.Error(e, "EditorDP::GetContentHeaderTable An error occured when trying to display the HTML.");
        }
      return myResult;
    }
    
    /// <summary>
    /// Helper method: returns empty table.
    /// </summary>
    /// <returns>HTMLElement</returns>
    private HTMLElement GetSpacerTable()
    {
      HTMLElementCollection myResult = new HTMLElementCollection();
      PTConstruct myConstruct;
      HTMLImg myImage;
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      try
      {
        myResult.AddInnerHTMLString("\n");
        HTMLComment myComment = new HTMLComment("Begin: Spacer table");
        myResult.AddInnerHTMLElement(myComment);
        myResult.AddInnerHTMLString("\n");
        myTable = new HTMLTable();
        myResult.AddInnerHTMLElement(myTable);
        myTable.SetBorder("0");
        myTable.SetWidth("100%");
        myTable.SetCellPadding("0");
        myTable.SetCellSpacing("0");
        // 1st Row
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER);
        myImage = (HTMLImg) myConstruct.GetHTMLElement();
        myImage.SetAlt("");
        myImage.SetWidth("225");
        myImage.SetHeight("1");
        myImage.SetBorder("0");
        myCell.AddInnerHTMLElement(myImage);
        myResult.AddInnerHTMLString("\n");
        myComment = new HTMLComment("End: Spacer table");
        myResult.AddInnerHTMLElement(myComment);
        myResult.AddInnerHTMLString("\n");
      }
      catch (Exception e)
      {
        log.Error(e, "EditorDP::GetSpacerTable An error occured when trying to display the HTML.");
        }
      return myResult;
    }
  }
}