using System;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.htmlconstructs;
// Framework Imports
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.editor
{
  /// <summary>
  /// This class implements the HTML to print out the Editor header.
  /// This View is printed out by EditorDP.
  /// Editor Views should NOT extend this class.
  /// They should extend EditorView instead.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class EditorHeaderView : EditorView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.editor.EditorHeaderView");
    
    /// <summary>
    /// Object Name for AS 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "EditorHeader";
    
    /// <summary>
    /// Finish button input name 
    /// </summary>
    public const String HTMLBUTTON_EDITOR_FINISH = "in_bu_Done";
    
    /// <summary>
    /// Cancel button input name 
    /// </summary>
    public const String HTMLBUTTON_EDITOR_CANCEL = "in_bu_Cancel";
    
    /// <summary>
    /// Apply button input name 
    /// </summary>
    public const String HTMLBUTTON_EDITOR_APPLY = "in_bu_Apply";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new EditorHeaderView();
    }
    
    /// <summary>
    /// Returns HTML for the Editor banner.
    /// Also displays the Editor title returned by GetEditorTitle
    /// in the associated AS class.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public override HTMLElement Display()
    {
      HTMLElementCollection myResult = new HTMLElementCollection();
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLImg myImage;
      HTMLH1 headerH1;
      PTConstruct myConstruct;
      try
      {
        myResult.AddInnerHTMLString("\n");
        HTMLComment myComment = new HTMLComment("Begin: Header main table");
        myResult.AddInnerHTMLElement(myComment);
        myResult.AddInnerHTMLString("\n");
        myTable = new HTMLTable();
        myResult.AddInnerHTMLElement(myTable);
        myTable.SetBorder("0");
        myTable.SetWidth("100%");
        myTable.SetCellPadding("0");
        myTable.SetCellSpacing("0");
        // 1st Row
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.BAN_HEADER_BG);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetWidth("10");
        myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER);
        myImage = (HTMLImg) myConstruct.GetHTMLElement();
        myImage.SetAlt("");
        myImage.SetWidth("10");
        myImage.SetHeight("1");
        myImage.SetBorder("0");
        myCell.AddInnerHTMLElement(myImage);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetWidth("100%");
        myCell.SetHeight("30");
        myCell.SetStyleClass(PTStyleClass.BAN_TEXT);
        myCell.SetNoWrap(true);
        headerH1 = new HTMLH1();
        myCell.AddInnerHTMLElement(headerH1);
        headerH1.AddInnerHTMLString(XPHttpUtility.HtmlEncode(((EditorAS) m_asOwner).GetEditorTitle()));
        headerH1.SetStyleClass(PTStyleClass.BAN_HEADER);
        // JF- Done and Cancel buttons will be displayed at the bottom of the page by AFormDP if we are in 508 mode.
        if (AccessStyles.STANDARD == m_asOwner.GetAccessStyle())
        {
          myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          myCell.SetWidth("300");
          myCell.SetAlign("right");
          myCell.SetStyleClass(PTStyleClass.BAN_TEXT);
          myCell.AddInnerHTMLString("\n");
          myComment = new HTMLComment("Begin: Header buttons table");
          myCell.AddInnerHTMLElement(myComment);
          myCell.AddInnerHTMLString("\n");
          HTMLElement myButtons = AddHeaderButtons();
          if (null != myButtons)
          {
            myCell.AddInnerHTMLElement(myButtons);
          }
        }
        // End Header buttons table
        myCell.AddInnerHTMLString("\n");
        myComment = new HTMLComment("End: Header buttons table");
        myCell.AddInnerHTMLElement(myComment);
        myCell.AddInnerHTMLString("\n");
        myResult.AddInnerHTMLString("\n<!-- End: Header main table -->\n");
        }
      catch (Exception e)
      {
        log.Error(e, "EditorHeaderView::Display TAn error occured when trying to display the HTML.");
        throw new XPException(m_asOwner.GetString(40, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return myResult;
    }
    
    /// <summary>
    /// Helper method to display the header buttons.
    /// </summary>
    /// <returns>HTMLElement</returns>
    protected virtual HTMLElement AddHeaderButtons()
    {
      HTMLImg myImage;
      HTMLInput myInput;
      PTConstruct myConstruct;
      // Begin Header buttons table
      HTMLTable mySubTable = new HTMLTable();
      mySubTable.SetBorder("0");
      mySubTable.SetCellPadding("5");
      mySubTable.SetCellSpacing("0");
      // button table 1st Row
      HTMLTableRow mySubRow = new HTMLTableRow();
      mySubTable.AddInnerHTMLElement(mySubRow);
      HTMLTableCell mySubCell = new HTMLTableCell();
      mySubRow.AddInnerHTMLElement(mySubCell);
      mySubCell.SetHeight("20");
      mySubCell.SetAlign("right");
      mySubCell.SetVAlign("middle");
      mySubCell.SetNoWrap(true);
      // JF- Only display the finish and apply buttons if the it's not read-only mode.
      if (!((IEditorModelRO) m_model).GetIsEditorReadOnly())
      {
        // JF- The Apply button is optional.
        if (((IEditorModelRO) m_model).GetEnableApplyButton())
        {
          // Apply button
          myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_EDITOR_APPLY,
           "");
          mySubCell.AddInnerHTMLElement(myInput);
          myInput.SetValue(m_asOwner.GetString(87, ResourceConstants.INFRASTRUCTURE));
          myInput.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
          myInput.SetOnClick("postToSelf(" + EditorRepostControl.POSTTOSELF_ACTION_APPLY_CHANGES + "); return false;");
          mySubCell.AddInnerHTMLString(SPACE);
        }
        // Done button
        myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_EDITOR_FINISH,
         "");
        mySubCell.AddInnerHTMLElement(myInput);
        myInput.SetValue(m_asOwner.GetString(38, ResourceConstants.INFRASTRUCTURE));
        myInput.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
        myInput.SetOnClick("postToSelf(" + EditorRepostControl.POSTTOSELF_ACTION_FINISH + "); return false;");
        mySubCell.AddInnerHTMLString(SPACE);
      }
      // Cancel button
      myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_EDITOR_CANCEL,
       "");
      mySubCell.AddInnerHTMLElement(myInput);
      myInput.SetValue(m_asOwner.GetString(39, ResourceConstants.INFRASTRUCTURE));
      myInput.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
      myInput.SetOnClick("postToSelf(" + EditorRepostControl.POSTTOSELF_ACTION_CANCEL + "); return false;");
      mySubCell = new HTMLTableCell();
      mySubRow.AddInnerHTMLElement(mySubCell);
      mySubCell.SetWidth("1");
      myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER);
      myImage = (HTMLImg) myConstruct.GetHTMLElement();
      myImage.SetAlt("");
      myImage.SetWidth("1");
      myImage.SetHeight("1");
      myImage.SetBorder("0");
      mySubCell.AddInnerHTMLElement(myImage);
      return mySubTable;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
  }
}