using System;
using com.plumtree.uiinfrastructure.activityspace;
// Framework Imports
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.editor
{
  /// <summary>
  /// This class is a helper class for the EditorModel.  This is NOT a real
  /// EditorModel or RepostModel.  The saveSection method will have to be called
  /// by  the main EditorModel class of the Editor.  This class can be used to
  /// implement common or plug-in sections  and can also be used as a secondary
  /// Model class if the main  one is getting too big and you want to split the
  /// code into  multiple classes.
  /// </summary>
  /// <author>Julie Fournier</author>
  public abstract class EditorHelperModel : IModel
  {
    
    /// <summary>
    /// Associated Activity Space. 
    /// </summary>
    public AActivitySpace m_asOwner;
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public abstract Object Create();
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetName();
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()"></seealso>
    public virtual void CleanupTempData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()"></seealso>
    public virtual void CleanupAllData()
    {
    }
    
    /// <summary>
    /// Inits the Model with the Activity Space.
    /// </summary>
    /// <param name="parent"></param>
    public virtual void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
    }
    
    /// <summary>
    /// Code to save the section represented by this Model.
    /// </summary>
    /// <param name="_htFormData">Form posted data.</param>
    /// <returns>int Status for server side validation</returns>
    public abstract int SaveSection(XPHashtable _htFormData);
  }
}