using System;
using com.plumtree.uiinfrastructure.activityspace;
// Framework Imports
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.editor
{
  /// <summary>
  /// This class is a helper class for the EditorRepostControl.  This is NOT a
  /// real control. The PerformRepostAction method  will have to be called by the
  /// main RepostControl class of the Editor.  This class can be used to
  /// implement common or plug-in sections  and can also be used as a secondary
  /// RepostControl class if the main  one is getting too big and you want to
  /// split the code into  multiple classes.
  /// </summary>
  /// <author>Julie Fournier</author>
  public abstract class EditorHelperRepostControl : IControl
  {
    
    /// <summary>
    /// Associated Activity Space. 
    /// </summary>
    protected AActivitySpace m_asOwner;
    
    /// <summary>
    /// Associated Model. 
    /// </summary>
    protected IModel m_model;
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public abstract Object Create();
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetName();
    
    /// <summary>
    /// This method is not used here.  This calss is not a real control.  It's
    /// just a helper for the Repost Control.
    /// </summary>
    /// <param name="arguments"></param>
    /// <returns>Redirect</returns>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      throw new XPException("ERROR in EditorHelperRepostControl::Execute, this function should NOT be called. " + "This class is not a real Control this is only a Helper Repost Control. It can only be used with a real " + "RepostControl and the PerformRepostAction method should be called from that RepostControl.");
      }
    
    /// <summary>
    /// Inits the class with the associated model and activity space.
    /// </summary>
    /// <param name="model"></param>
    /// <param name="space"></param>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      m_model = model;
      m_asOwner = space;
    }
    
    /// <summary>
    /// This method is a dispatcher. For each repost action  it should perform
    /// the right action.
    /// </summary>
    /// <param name="_nAction">Action mode.</param>
    /// <param name="_htFormData">Form posted data.</param>
    public abstract void PerformRepostAction(int _nAction, XPHashtable _htFormData);
    }
}