using System;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.editor
{
  /// <author>
  /// Julie Fournier
  /// This class implements the HTML to print out the Editor left navigation.
  /// This View is printed out by EditorDP.
  /// Editor Views should NOT extend this class.
  /// They should extend EditorView instead.
  /// </author>
  public class EditorLeftNavView : EditorView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.editor.EditorLeftNavView");
    
    /// <summary>
    /// Object Name for AS 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "EditorLeftNav";
    
    /// <summary>
    /// HTML object: hidden input for the space name when we want to redirect to another AS. 
    /// </summary>
    public const String HTMLHIDDEN_EDITOR_REDIRECT_SPACENAME = "in_hi_EditorRedirectSpaceName";
    
    /// <summary>
    /// HTML object: hidden input for the page name when we want to redirect to another page. 
    /// </summary>
    public const String HTMLHIDDEN_EDITOR_REDIRECT_PAGENAME = "in_hi_EditorRedirectPageName";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new EditorLeftNavView();
    }
    
    /// <summary>
    /// Returns the HTML for the left navigation.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public override HTMLElement Display()
    {
      HTMLElementCollection myResult = new HTMLElementCollection();
      PTConstruct myConstruct;
      HTMLImg myImage;
      HTMLAnchor myAnchor;
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLInput myInput;
      HTMLSpan mySpan;
      PTConstruct myMenuHeader;
      try
      {
        String sPageNameToSelect = m_asOwner.GetCurrentPage().GetName();
        LeftNavMenu mEditorMenu = ((EditorAS) m_asOwner).GetLeftMenu();
        XPArrayList alSubMenus = mEditorMenu.GetArraySubMenus();
        LeftNavSubMenu smCurrentSubMenu;
        LeftNavLink lCurrentLink;
        XPArrayList alLinks;
        String sPage;
        String sASName;
        String sMenuString;
        String sJSName;
        EditorDP dpPage;
        myResult.AddInnerHTMLString("\n");
        HTMLComment myComment = new HTMLComment("Begin: Left Navigation");
        myResult.AddInnerHTMLElement(myComment);
        myResult.AddInnerHTMLString("\n");
        // JF- Hidden input for the new AS name (used by goToPage JS function)
        myInput = new HTMLInput(HTMLInputTypes.HIDDEN, HTMLHIDDEN_EDITOR_REDIRECT_SPACENAME,
         "");
        myResult.AddInnerHTMLElement(myInput);
        // JF- Hidden input for the new page name (used by goToPage JS function)
        myInput = new HTMLInput(HTMLInputTypes.HIDDEN, HTMLHIDDEN_EDITOR_REDIRECT_PAGENAME,
         "");
        myResult.AddInnerHTMLElement(myInput);
        // JF- Loop through the sub menus.
        for (int i = 0;i < alSubMenus.GetSize();i++)
        {
          smCurrentSubMenu = (LeftNavSubMenu) alSubMenus.GetElement(i);
          if (!smCurrentSubMenu.GetHide())
          {
            alLinks = smCurrentSubMenu.GetArrayLinks();
            myResult.AddInnerHTMLString("\n");
            myComment = new HTMLComment("Begin: menu " + i);
            myResult.AddInnerHTMLElement(myComment);
            myResult.AddInnerHTMLString("\n");
            myTable = new HTMLTable();
            myResult.AddInnerHTMLElement(myTable);
            myTable.SetBorder("0");
            myTable.SetWidth("100%");
            myTable.SetCellPadding("0");
            myTable.SetCellSpacing("0");
            // 1st Row
            myRow = new HTMLTableRow();
            myTable.AddInnerHTMLElement(myRow);
            myRow.SetStyleClass(PTStyleClass.MENU_HEADER_BG);
            myCell = new HTMLTableCell();
            myRow.AddInnerHTMLElement(myCell);
            // JF- Get the construct for the menu header
            myMenuHeader = UIConstructFactory.GetConstruct(PTBannerType.MENU);
            myMenuHeader.ReplaceNestedElementsWithHTMLString(PTBannerType.MENU.TITLE_ELEMENT_NAME,
             smCurrentSubMenu.GetTitle());
            myCell.AddInnerHTMLElement(myMenuHeader.GetHTMLElement());
            // JF- For each sub menu, display each link.
            for (int j = 0;j < alLinks.GetSize();j++)
            {
              lCurrentLink = (LeftNavLink) alLinks.GetElement(j);
              if (!lCurrentLink.GetHide())
              {
                sPage = lCurrentLink.GetDPName();
                sASName = lCurrentLink.GetASName();
                sJSName = lCurrentLink.GetJSValidationName();
                dpPage = (EditorDP) m_asOwner.GetPage(sPage);
                sMenuString = XPHttpUtility.HtmlEncode(dpPage.GetLeftNavString());
                // item Row
                myRow = new HTMLTableRow();
                myTable.AddInnerHTMLElement(myRow);
                myCell = new HTMLTableCell();
                myRow.AddInnerHTMLElement(myCell);
                myCell.SetWidth("100%");
                myCell.SetHeight("25");
                myCell.SetStyleClass(PTStyleClass.MENU_TEXT);
                myCell.AddInnerHTMLString(SPACE);
                mySpan = new HTMLSpan();
                mySpan.SetStyleClass(PTStyleClass.ALERT_REQUIRED_TEXT);
                mySpan.AddInnerHTMLString("*");
                if (sPageNameToSelect.Equals(sPage))
                {
                  // JF- If this corresponds to the DP displayed on the
                  // right side of the page, then don't display as link, 
                  // just text and also keep that text in a memeber variable
                  // so we can display it later in the sub header (see EditorDP).
                  myCell.AddInnerHTMLString(sMenuString);
                  if (lCurrentLink.GetMandatory())
                  {
                    myCell.AddInnerHTMLElement(mySpan);
                    // JF- If the user is going to a mandatory page, we also need to remove it from the array.
                    mEditorMenu.RemovePageFromMandatoryArray(sPage);
                  }
                } else
                {
                  myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
                  myAnchor.SetOnClick("goToPage(\'" + sASName + "\', \'" + sPage + "\',\'" + sJSName + "\'); return false;");
                  myAnchor.AddInnerHTMLString(sMenuString);
                  if (lCurrentLink.GetMandatory())
                  {
                    myAnchor.AddInnerHTMLElement(mySpan);
                  }
                  myCell.AddInnerHTMLElement(myAnchor);
                }
              }
            }
            // Last Row
            myRow = new HTMLTableRow();
            myTable.AddInnerHTMLElement(myRow);
            myCell = new HTMLTableCell();
            myRow.AddInnerHTMLElement(myCell);
            myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER);
            myImage = (HTMLImg) myConstruct.GetHTMLElement();
            myImage.SetAlt("");
            myImage.SetWidth("1");
            myImage.SetHeight("10");
            myImage.SetBorder("0");
            myCell.AddInnerHTMLElement(myImage);
            myResult.AddInnerHTMLString("\n");
            myComment = new HTMLComment("End: menu " + i);
            myResult.AddInnerHTMLElement(myComment);
            myResult.AddInnerHTMLString("\n");
          }
        }
        myResult.AddInnerHTMLString("\n");
        myComment = new HTMLComment("End: Left Navigation");
        myResult.AddInnerHTMLElement(myComment);
        myResult.AddInnerHTMLString("\n");
      }
      catch (Exception e)
      {
        GeneralHelpers.SetUIError(m_asOwner, OpenLogService.GetComponent("UI_Infrastructure"),
         "EditorLeftNavView::Display", e, false, UIErrorLevel.SERIOUS, m_asOwner.GetString(125,
         ResourceConstants.INFRASTRUCTURE));
      }
      return myResult;
    }
    
    /// <summary>
    /// Returns Java Script for goToPage function.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    /// <returns>HTMLScript</returns>
    public override HTMLScript DisplayJavascript()
    {
      HTMLScript myScript;
      try
      {
        XPStringBuilder sb = new XPStringBuilder(300);
        myScript = new HTMLScript("text/javaScript");
        sb.Append("function goToPage(varASName,varNewPage,varJSValidationName)\n");
        sb.Append("{\n");
        sb.Append("  if (0 != varJSValidationName.length) \n");
        sb.Append("  {\n");
        sb.Append("    if (! eval(varJSValidationName)())\n");
        sb.Append("    {\n");
        sb.Append("      return;\n");
        sb.Append("    }\n");
        sb.Append("  }\n");
        sb.Append("    document." + EditorDP.HTMLFORM_NAME + "." + HTMLHIDDEN_EDITOR_REDIRECT_SPACENAME + ".value= varASName;\n");
        sb.Append("    document." + EditorDP.HTMLFORM_NAME + "." + HTMLHIDDEN_EDITOR_REDIRECT_PAGENAME + ".value= varNewPage;\n");
        sb.Append("    postToSelf(" + EditorRepostControl.POSTTOSELF_ACTION_GOTOPAGE + ");\n");
        sb.Append("}\n");
        myScript.AddInnerHTMLString(sb.ToString());
      }
      catch (Exception e)
      {
        log.Error(e, "EditorHeaderView::DisplayJavascript An error occured when trying to display the JavaScript.");
        throw new XPException(m_asOwner.GetString(42, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return myScript;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
  }
}