using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.htmlconstructs;
// Framework Imports
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.editor
{
  /// <summary>
  /// This is the View class for the Editor Framework. It implements the Init
  /// method and should be extended by any View.
  /// </summary>
  /// <author>Julie Fournier</author>
  public abstract class EditorView : IView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.editor.EditorView");
    
    /// <summary>
    /// Constant for the HTML sign "&nbsp;". 
    /// </summary>
    public const String SPACE = "&nbsp;";
    // 
    
    /// <summary>
    /// JF- Associated Activity Space 
    /// </summary>
    protected AActivitySpace m_asOwner;
    
    /// <summary>
    /// JF- Associated read-only Model 
    /// </summary>
    protected IModelRO m_model;
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public abstract Object Create();
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetName();
    
    /// <summary>
    /// Returns a short empty table. This is the one you should use in your
    /// View,  just below the View header (the one returned by GetHeaderView).
    /// </summary>
    /// <returns>HTMLTable</returns>
    public static HTMLTable GetShortSpacerTable()
    {
      HTMLTable mySpacerTable = new HTMLTable();
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLImg myImage;
      PTConstruct myConstruct;
      try
      {
        mySpacerTable.SetWidth("100%");
        mySpacerTable.SetBorder("0");
        mySpacerTable.SetCellPadding("0");
        mySpacerTable.SetCellSpacing("0");
        // 1st Row begin
        myRow = new HTMLTableRow();
        mySpacerTable.AddInnerHTMLElement(myRow);
        // Empty Cell
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER);
        myImage = (HTMLImg) myConstruct.GetHTMLElement();
        myImage.SetAlt("");
        myImage.SetWidth("1");
        myImage.SetHeight("5");
        myImage.SetBorder("0");
        myCell.AddInnerHTMLElement(myImage);
      }
      catch (Exception e)
      {
        log.Error(e, "EditorView::GetShortSpacerTable The following error occured when trying to display the HTML.");
        }
      // JF- no need to throw an exception, this is not critical.
      return mySpacerTable;
    }
    
    /// <summary>
    /// Returns an empty table. This is the one used to separate each  section
    /// in any Editor page. Note that this is a static method so  you can also
    /// call it from any DP.
    /// </summary>
    /// <returns>HTMLTable</returns>
    public static HTMLTable GetSpacerTable()
    {
      HTMLTable mySpacerTable = new HTMLTable();
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLImg myImage;
      PTConstruct myConstruct;
      try
      {
        mySpacerTable.SetWidth("100%");
        mySpacerTable.SetBorder("0");
        mySpacerTable.SetCellPadding("0");
        mySpacerTable.SetCellSpacing("0");
        // 1st Row begin
        myRow = new HTMLTableRow();
        mySpacerTable.AddInnerHTMLElement(myRow);
        // Empty Cell
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER);
        myImage = (HTMLImg) myConstruct.GetHTMLElement();
        myImage.SetAlt("");
        myImage.SetWidth("1");
        myImage.SetHeight("15");
        myImage.SetBorder("0");
        myCell.AddInnerHTMLElement(myImage);
      }
      catch (Exception e)
      {
        log.Error(e, "EditorView::GetSpacerTable The following error occured when trying to display the HTML.");
        }
      // JF- no need to throw an exception, this is not critical.
      return mySpacerTable;
    }
    
    /// <summary>
    /// Adds an empty Cell to the Row passed in argument.
    /// </summary>
    /// <param name="_myRow"></param>
    public virtual void AddEmptyCellToRow(HTMLTableRow _myRow)
    {
      this.AddEmptyCellToRow(_myRow, "");
    }
    
    /// <summary>
    /// Adds an empty Cell of the specified width to the Row passed in argument.
    /// </summary>
    /// <param name="_myRow">_myRow</param>
    /// <param name="_sWidth">strWidth</param>
    public virtual void AddEmptyCellToRow(HTMLTableRow _myRow, String _sWidth)
    {
      HTMLTableCell myCell;
      try
      {
        myCell = new HTMLTableCell();
        _myRow.AddInnerHTMLElement(myCell);
        myCell.SetNoWrap(true);
        if (!"".Equals(_sWidth))
        {
          myCell.SetWidth(_sWidth);
        }
        myCell.AddInnerHTMLString(SPACE);
      }
      catch (Exception e)
      {
        log.Error(e, "EditorView::AddEmptyCellToRow The following error occured when trying to display the HTML.");
        throw new XPException(m_asOwner.GetString(46, ResourceConstants.INFRASTRUCTURE),
         e);
      }
    }
    
    /// <summary>
    /// Adds an empty Row to the Table passed in argument.
    /// </summary>
    /// <param name="_myTable"></param>
    public virtual void AddEmptyRowToTable(HTMLTable _myTable)
    {
      this.AddEmptyRowToTable(_myTable, "3");
    }
    
    /// <summary>
    /// Adds an empty Row of the specified height to the Table passed in
    /// argument.
    /// </summary>
    /// <param name="_myTable">_myTable</param>
    /// <param name="_sHeight">strHeight</param>
    public virtual void AddEmptyRowToTable(HTMLTable _myTable, String _sHeight)
    {
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      try
      {
        myRow = new HTMLTableRow();
        _myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetNoWrap(true);
        myCell.SetAutoFit(true);
        myCell.SetHeight(_sHeight);
      }
      catch (Exception e)
      {
        log.Error(e, "EditorView::AddEmptyRowToTable The following error occured when trying to display the HTML.");
        throw new XPException(m_asOwner.GetString(45, ResourceConstants.INFRASTRUCTURE),
         e);
      }
    }
    
    /// <summary>
    /// This method returns the HTMLElement that encompasses the output of the
    /// View.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public abstract HTMLElement Display();
    
    /// <summary>
    /// This method returns the HTMLScript that encompasses the Javascript on
    /// the page.
    /// </summary>
    /// <returns>HTMLScript</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    public virtual HTMLScript DisplayJavascript()
    {
      return null;
    }
    
    /// <summary>
    /// Returns the header for the View.
    /// </summary>
    /// <param name="_asOwner">The Activity Space this view is using.</param>
    /// <param name="_sHeaderTitle">Title you want to display in the header.</param>
    /// <returns>HTMLTable</returns>
    protected virtual HTMLElement GetHeaderViewHelper(AActivitySpace _asOwner,
     String _sHeaderTitle)
    {
      PTConstruct myViewBanner;
      try
      {
        // JF- Get the construct for the View banner
        if (_asOwner.GetIsAccessStyleStandard())
        {
          myViewBanner = UIConstructFactory.GetConstruct(PTBannerType.OBJECT,
           true, this.m_asOwner);
          myViewBanner.ReplaceNestedElementsWithHTMLString(PTBannerType.OBJECT.TITLE_ELEMENT_NAME,
           _sHeaderTitle);
          myViewBanner.ReplaceNestedElementsWithHTMLString(PTBannerType.OBJECT.SUBTITLE_ELEMENT_NAME,
           "");
        } else
        {
          myViewBanner = UIConstructFactory.GetConstruct(PTBannerType.OBJECT_508,
           true, this.m_asOwner);
          myViewBanner.ReplaceNestedElementsWithHTMLString(PTBannerType.OBJECT_508.TITLE_ELEMENT_NAME,
           SPACE + _sHeaderTitle);
          myViewBanner.ReplaceNestedElementsWithHTMLString(PTBannerType.OBJECT_508.SUBTITLE_ELEMENT_NAME,
           "");
        }
      }
      catch (Exception e)
      {
        log.Error(e, "EditorView::GetHeaderView The following error occured when trying to display the HTML.");
        throw new XPException(m_asOwner.GetString(47, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return myViewBanner.GetHTMLElement();
    }
    
    /// <summary>
    /// Returns the header for the view.  In the case that your Activity
    /// Space is not extended from EditorAS, but your view is, you will
    /// need to overwrite GetHeaderView with this same code, with the 
    /// exception that you will be passing your m_asOwner.  In here, the
    /// m_asOwner is expected to be of type EditorAS.
    /// </summary>
    /// <param name="_sHeaderTitle">Title you want to display in the header.</param>
    /// <returns>HTMLTable</returns>
    public virtual HTMLElement GetHeaderView(String _sHeaderTitle)
    {
      return GetHeaderViewHelper(m_asOwner, _sHeaderTitle);
    }
    
    /// <summary>
    /// Inits the View with the read-only Model and the Activity Space.
    /// </summary>
    /// <param name="model"></param>
    /// <param name="parent"></param>
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      m_model = model;
      try
      {
        try
        {
          m_asOwner = (EditorAS) parent;
        }
        catch (Exception e)
        {
          throw XPException.GetInstance(e);
        }
      }
      catch (XPClassCastException cce)
      {
        log.Error(cce, "EditorView::Init Unable to cast the Object " + parent + " to EditorAS. The following exception occured");
        // JF- No need to extract, this is an unexpected error.
        throw new XPException("Unable to cast the Object " + parent + " to EditorAS.",
         cce);
      }
    }
  }
}