using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.errordisplay
{
  /// <author>
  /// Ben Geislinger
  /// This abstract view provides common functionality for concrete
  /// view implementaions for displaying error messages.
  /// </author>
  public abstract class AErrorView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.errordisplay.AErrorView");
    // Associated Activity Space
    protected AActivitySpace m_asOwner;
    // Associated read-only model
    protected IErrorModelRO m_model;
    
    /// <summary>
    /// This method returns the HTMLElement that encompasses the output of the View.
    /// </summary>
    public virtual HTMLElement Display()
    {
      bool bDisplaySeparatePageErrors = this.m_model.GetDisplaySeparatePageErrors();
      IXPEnumerator eErrors = m_model.GetErrors();
      ErrorDisplayHelper helper = new ErrorDisplayHelper(m_asOwner, eErrors);
      helper.SetDisplaySeparatePageErrors(bDisplaySeparatePageErrors);
      return helper.DisplayErrors();
    }
    
    /// <summary>
    /// This method returns the HTMLScript that encompasses the Javascript on the page.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    /// <returns>HTMLScript</returns>
    public virtual HTMLScript DisplayJavascript()
    {
      return null;
    }
    
    /// <param name="model"></param>
    /// <param name="parent"></param>
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      if (model == null)
      {
        log.Error("Initialized with null model.");
      }
      if (parent == null)
      {
        log.Error("Initialized with null model.");
      }
      m_model = (AErrorModel) model;
      m_asOwner = parent;
    }
  }
}