using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.errordisplay
{
  /// <author>
  /// Ben Geislinger
  /// This concrete model extends AErrorModel, and provides basic functionality
  /// for displaying error messages.
  /// <br /><br />
  /// This model makes use of the base UIErrorHandler class.  This class can not make
  /// use of any plumtree-specific functionality, and for this reason, this
  /// error model class is somewhat limited.  Portal display pages should use
  /// instead the PTErrorModel class found in the portalinfrastructure package.
  /// </author>
  public class ErrorModel : AErrorModel
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.errordisplay.ErrorModel");
    public const String STR_MVC_CLASS_NAME = "ErrorModel";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.errordisplay.AErrorModel#ClearError()"></seealso>
    public override void ClearError(String sKey, ISessionManager mySessionManager,
     IApplication myApplication)
    {
      UIErrorHandler.ClearError(sKey, mySessionManager, myApplication);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.errordisplay.AErrorModel#ClearErrors()"></seealso>
    public override void ClearErrors(ISessionManager mySessionManager, IApplication myApplication)
    {
      UIErrorHandler.ClearErrors(mySessionManager, myApplication);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.errordisplay.AErrorModel#ClearErrorsForASChange()"></seealso>
    public override void ClearErrorsForASChange(String sASName, ISessionManager mySessionManager,
     IApplication myApplication)
    {
      UIErrorHandler.ClearErrorsForASChange(sASName, mySessionManager, myApplication);
      }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new ErrorModel();
    }
    
    /// <summary>
    /// Returns false.  Only errors not requiring a spearate page
    /// will be displayed.  This fact makes this model suitable for
    /// MVCs embedded within other display pages.
    /// </summary>
    /// <returns>boolean false</returns>
    public override bool GetDisplaySeparatePageErrors()
    {
      return false;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.errordisplay.AErrorModel#GetErrors()"></seealso>
    public override IXPEnumerator GetErrors(ISessionManager session, IApplication myApplication,
     AActivitySpace activitySpace, bool bGetErrorsAssociatedWithAnAS)
    {
      log.Debug("GetErrors()");
      return UIErrorHandler.GetErrors(session, myApplication, activitySpace,
       bGetErrorsAssociatedWithAnAS, UIErrorHandler.GetDisplayAdminErrors(activitySpace));
      }
    
    /// <summary>
    /// Returns true.  Only errors associated with the current activity space
    /// will be returned by the GetErrors methods.
    /// </summary>
    /// <returns>boolean true</returns>
    public override bool GetErrorsAssociatedWithAnAS()
    {
      return true;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IModel#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Inits the Model with the Activity Space.
    /// </summary>
    /// <param name="parent"></param>
    public override void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
    }
  }
}