using System;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.uiinfrastructure.errordisplay
{
  /// <author>
  /// BenG
  /// This concrete view class extends AErrorView.
  /// <br /><br />
  /// Although the existing error view classes are essentially functionally
  /// equivalent, it is necessary to provide multiple views for those cases in
  /// which multiple error displaying MVCs are embedded in the same page.
  /// <br /><br />
  /// For example, all display pages extending PlumtreeDP automatically contain
  /// an error display MVC.  This MVC is responsible for displaying errors
  /// that do not require a separate display page.  The display page that is
  /// responsible for displaying those errors that do require a separate display
  /// page ALSO extends PlumtreeDP, simply so that it looks like the other plumtree
  /// pages, and contains the banner and whatnot.  This display page then contains
  /// two MVCs responsible for displaying error messages: the one that comes with
  /// PlumtreeDP and displays errors not requiring a separate page, and the one that
  /// comes in the separate page display page and is responsible for displaying those
  /// errors that do require a separate page.  Although this particular view would
  /// be suitable for both MVCs embedded in this page, it is not possible to
  /// embed the same view multiple times by the design of the activity space
  /// framework. Perhaps this will change, but for now, there are multiple error
  /// display view classes that do the same thing.
  /// </author>
  public class ErrorView : AErrorView,IView
  {
    
    /// <summary>
    /// MVC object name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "ErrorView";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new ErrorView();
    }
    // TODOCR JF 12/10/02 - There is no reason to have this class.
    // This class may be useful because the AErrorView class requires
    // you to implement the necessary classes when you subclass, so you
    // can't mess it up as easily by forgetting one of the methods.
    //
    // The AS lets you have a storage name when you Register a View. 
    // You can register the same View twice in the same AS with 2 different storage names.
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IView#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
  }
}