using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.errordisplay
{
  /// <author>BenG</author>
  public class SinglePageErrorControl : IControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.errordisplay.SinglePageErrorControl");
    public const String STR_MVC_CLASS_NAME = "SinglePageErrorControl";
    public const String PARAM_CLEAR_AND_RETURN = "in_hi_clearandreturn";
    private AActivitySpace m_asOwner;
    private SinglePageErrorModel m_asModel;
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IControl#execute()"></seealso>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      log.Debug("Entering Execute()");
      Redirect myRedirect = null;
      bool bo;
      String[] sValues = (String[]) arguments.GetElement(SinglePageErrorControl.PARAM_CLEAR_AND_RETURN);
      if (sValues != null)
      {
        try
        {
          bo = XPConvert.ToBoolean(sValues[0]);
        }
        catch (Exception ex)
        {
          log.Error(ex, "Unable to convert PARAM_CLEAR_AND_RETURN to boolean: " + sValues[0]);
          bo = false;
        }
        if (bo)
        {
          log.Debug("About to clear and return.");
          UIErrorHandler.ClearErrorsRequiringASeparateDisplayPage(this.m_asOwner.GetSubSession(),
           this.m_asOwner.GetApplication(), this.m_asOwner, false, UIErrorHandler.GetDisplayAdminErrors(this.m_asOwner));
          AActivitySpace asParent = this.m_asOwner.GetParentSpace();
          if (asParent != null)
          {
            log.Debug("Got a parent, going home.");
            myRedirect = new Redirect();
            myRedirect.SetLinkGetCachedSpace(asParent);
          } else
          {
            log.Error("Couldn\'t get parent to return to..");
          }
        }
      }
      return myRedirect;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new SinglePageErrorControl();
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IControl#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IControl#Init(IModelRW)"></seealso>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      m_asModel = (SinglePageErrorModel) model;
      m_asOwner = space;
    }
  }
}