using System;
namespace com.plumtree.uiinfrastructure.errordisplay
{
  /// <author>
  /// Ben Geislinger
  /// This class provides an enumeration of the various levels of error scope.  The
  /// scope of an error determines the audience for which the error message is
  /// intended.
  /// </author>
  public class UIErrorScope
  {
    
    /// <summary>
    /// Use the CURRENT_USER scope to create an error that will be displayed
    /// only to the current user.  Such errors are stored on the current users
    /// app-server session.
    /// </summary>
    public static readonly UIErrorScope CURRENT_USER = new UIErrorScope("current_user");
    
    /// <summary>
    /// Use the ADMIN_USERS scope to create an error that will be displayed
    /// only to administrative users.
    /// </summary>
    public static readonly UIErrorScope ADMIN_USERS = new UIErrorScope("admin_users");
    
    /// <summary>
    /// Use the ALL_USERS scope to create an error that will be displayed
    /// to all portal users.
    /// </summary>
    public static readonly UIErrorScope ALL_USERS = new UIErrorScope("all_users");
    
    /// <summary>
    /// the error scope string 
    /// </summary>
    private String m_strErrorScope;
    
    /// <summary>
    /// Creates a new UIErrorScope object.
    /// </summary>
    /// <param name="s"></param>
    private UIErrorScope(String strErrorScope)
    {
      m_strErrorScope = strErrorScope;
    }
    
    /// <summary>
    /// Call this method to obtain the String representation of this scope.
    /// </summary>
    /// <returns>String</returns>
    public override String ToString()
    {
      return m_strErrorScope;
    }
  }
}