using System;
using com.plumtree.uiinfrastructure.editor;
// Framework Imports
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.expandablelist
{
  /// <summary>
  /// This class handles the repost actions for the  Expandable List framework.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class ExpListControl : EditorHelperRepostControl
  {
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "ExpList";
    
    /// <summary>
    /// Will add one or more items to the list. 
    /// </summary>
    public const int POSTTOSELF_ACTION_ADD = 30;
    
    /// <summary>
    /// Will delete one or more items from the list. 
    /// </summary>
    public const int POSTTOSELF_ACTION_DELETE = 31;
    
    /// <summary>
    /// Will open the tree to select some items. 
    /// </summary>
    public const int POSTTOSELF_ACTION_OPENTREE = 32;
    
    /// <summary>
    /// Will move the corresponding row. 
    /// </summary>
    public const int POSTTOSELF_ACTION_MOVEROW = 33;
    
    /// <summary>
    /// Will resort by the selected column. 
    /// </summary>
    public const int POSTTOSELF_ACTION_SORTBYCOLUMN = 34;
    
    /// <summary>
    /// Will reverse the sort of the current sorted column. 
    /// </summary>
    public const int POSTTOSELF_ACTION_REVERSESORT = 35;
    
    /// <summary>
    /// Will modify the corresponding row. 
    /// </summary>
    public const int POSTTOSELF_ACTION_MODIFYROW = 36;
    
    /// <summary>
    /// Will add one or more items to the list using IPR. 
    /// </summary>
    public const int POSTTOSELF_ACTION_ADD_IPR = 37;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new ExpListControl();
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Calls the method associated to the repost action. Editors using this
    /// framework need to call that method from  the PerformRepostAction method
    /// of the main RepostControl.
    /// </summary>
    /// <param name="_nAction"></param>
    /// <param name="_htFormData"></param>
    public override void PerformRepostAction(int _nAction, XPHashtable _htFormData)
    {
      ExpListModel myExpListModel = (ExpListModel) m_model;
      if (_nAction == myExpListModel.GetUniqueInt(POSTTOSELF_ACTION_ADD))
      {
        myExpListModel.AddRowsOnRepost(_htFormData);
      } else
      if (_nAction == myExpListModel.GetUniqueInt(POSTTOSELF_ACTION_ADD_IPR))
      {
        myExpListModel.AddRowsOnRepost(_htFormData);
        if (m_asOwner.GetIsInPlaceRefreshEnabled())
        {
          // set a flag so that display pages will show in place 
          // content instead of normal content on the next pass.
          m_asOwner.SetIsInPlaceRefreshDisplayOn(true);
        }
      } else
      if (_nAction == myExpListModel.GetUniqueInt(POSTTOSELF_ACTION_DELETE))
      {
        myExpListModel.DeleteRowsOnRepost(_htFormData);
        if (m_asOwner.GetIsInPlaceRefreshEnabled())
        {
          // set a flag so that display pages will show in place 
          // content instead of normal content on the next pass.
          m_asOwner.SetIsInPlaceRefreshDisplayOn(true);
        }
      } else
      if (_nAction == myExpListModel.GetUniqueInt(POSTTOSELF_ACTION_REVERSESORT))
      {
        myExpListModel.ReverseCurrentSort();
        if (m_asOwner.GetIsInPlaceRefreshEnabled())
        {
          // set a flag so that display pages will show in place 
          // content instead of normal content on the next pass.
          m_asOwner.SetIsInPlaceRefreshDisplayOn(true);
        }
      } else
      if (_nAction == myExpListModel.GetUniqueInt(POSTTOSELF_ACTION_SORTBYCOLUMN))
      {
        myExpListModel.SortByColumn(_htFormData);
        if (m_asOwner.GetIsInPlaceRefreshEnabled())
        {
          // set a flag so that display pages will show in place 
          // content instead of normal content on the next pass.
          m_asOwner.SetIsInPlaceRefreshDisplayOn(true);
        }
      } else
      if (_nAction == myExpListModel.GetUniqueInt(POSTTOSELF_ACTION_MOVEROW))
      {
        myExpListModel.MoveRow(_htFormData);
      } else
      if (_nAction == myExpListModel.GetUniqueInt(POSTTOSELF_ACTION_MODIFYROW))
      {
        myExpListModel.ModifyRow(_htFormData);
      }
    }
  }
}