using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.form.pagination;
using com.plumtree.uiinfrastructure.editor;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.openlog;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.expandablelist
{
  /// <summary>
  /// This View contains the HTML for the Expandable List
  /// Framework.
  /// </summary>
  /// <author>Julie Fournier</author>
  public abstract class ExpListView : EditorView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.expandablelist.ExpListView");
    
    /// <summary>
    /// JF- HTML Object name: Delete button 
    /// </summary>
    public const String HTMLBUTTON_DELETE = "in_bu_Delete";
    
    /// <summary>
    /// JF- HTML Object name: Check all checkbox 
    /// </summary>
    public const String HTMLCHECKBOX_CHECKALL = "in_cb_CheckAll";
    
    /// <summary>
    /// JF- HTML Object name: Delete row checkbox 
    /// </summary>
    public const String HTMLCHECKBOX_DELROW = "in_cb_DelRow";
    
    /// <summary>
    /// JF- HTML Object name: Column index 
    /// </summary>
    public const String HTMLHIDDEN_COLUMNID = "in_hi_ColumnID";
    
    /// <summary>
    /// JF- HTML Object name: Row index 
    /// </summary>
    public const String HTMLHIDDEN_ROWINDEX = "in_hi_RowIndex";
    
    /// <summary>
    /// JF- HTML Object name: Move row action 
    /// </summary>
    public const String HTMLHIDDEN_MOVEACTION = "in_hi_MoveAction";
    
    /// <summary>
    /// RWW - HTML ID: Entire Expandable List. 
    /// </summary>
    public const String HTMLID_EXPLIST = "id_ExpListView";
    
    /// <summary>
    /// Variable is updated when iterating through rows in Display(). Maybe be used in child classes. 
    /// </summary>
    protected int m_nLastVisibleRow = -1;
    
    /// <summary>
    /// Returns the HTML for the View.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public override HTMLElement Display()
    {
      HTMLElementCollection resultWrapper = new HTMLElementCollection();
      HTMLSpan result = new HTMLSpan();
      result.SetID(((IExpListModelRO) m_model).GetUniqueString(HTMLID_EXPLIST));
      resultWrapper.AddInnerHTMLElement(result);
      HTMLTable myTable;
      HTMLTableRow myRow;
      try
      {
        result.AddInnerHTMLString("\n");
        HTMLComment myComment = new HTMLComment("Begin: Display ExpListView");
        result.AddInnerHTMLElement(myComment);
        result.AddInnerHTMLString("\n");
        // Header
        result.AddInnerHTMLString("\n");
        myComment = new HTMLComment("Begin: Display Section header");
        result.AddInnerHTMLElement(myComment);
        result.AddInnerHTMLString("\n");
        if (0 < ((IExpListModelRO) m_model).GetSectionTitle().Length)
        {
          String sPagination = "";
          if (((IExpListModelRO) m_model).IsPaginationEnabled() && (0 != ((IExpListModelRO) m_model).GetReadOnlyItems().GetCount()))
          {
            sPagination = SPACE;
            sPagination+=XPResourceManager.FormatMessage(m_asOwner.GetString(692,
             "ptmsgs_portalcommonmsgs"), ((IExpListModelRO) m_model).GetCurrentLowerObjectCount(),
             ((IExpListModelRO) m_model).GetCurrentUpperObjectCount(), ((IExpListModelRO) m_model).GetReadOnlyItems().GetCount());
            }
          result.AddInnerHTMLElement(GetHeaderView(((IExpListModelRO) m_model).GetSectionTitle() + sPagination));
          }
        result.AddInnerHTMLString("\n");
        myComment = new HTMLComment("End: Display Section header");
        result.AddInnerHTMLElement(myComment);
        result.AddInnerHTMLString("\n");
        // Action Bar
        // JF- We don't display the action bar if the list is read only
        if (!((IExpListModelRO) m_model).GetIsListReadOnly())
        {
          HTMLTable myActionBar = GetHTMLForActionBar();
          // JF- The child class could also decide to not have an action bar by returning null
          if (null != myActionBar)
          {
            result.AddInnerHTMLString("\n");
            myComment = new HTMLComment("Begin: Display Action Bar");
            result.AddInnerHTMLElement(myComment);
            result.AddInnerHTMLString("\n");
            result.AddInnerHTMLElement(myActionBar);
            result.AddInnerHTMLString("\n");
            myComment = new HTMLComment("End: Display Action Bar");
            result.AddInnerHTMLElement(myComment);
            result.AddInnerHTMLString("\n");
          }
        }
        // Below Action Bar
        HTMLTable myBelowActionBar = GetHTMLBelowActionBar();
        // AGC- This adds a table between the action bar and the list.
        if (null != myBelowActionBar)
        {
          result.AddInnerHTMLString("\n");
          myComment = new HTMLComment("Begin: Display Below Action Bar");
          result.AddInnerHTMLElement(myComment);
          result.AddInnerHTMLString("\n");
          result.AddInnerHTMLElement(myBelowActionBar);
          result.AddInnerHTMLString("\n");
          myComment = new HTMLComment("End: Display Below Action Bar");
          result.AddInnerHTMLElement(myComment);
          result.AddInnerHTMLString("\n");
        }
        String sInstructions = ((IExpListModelRO) m_model).GetTableInstructions();
        if (null != sInstructions)
        {
          result.AddInnerHTMLElement(EditorView.GetShortSpacerTable());
          result.AddInnerHTMLElement(GetHTMLForInstructions(sInstructions,
           false));
          result.AddInnerHTMLElement(EditorView.GetShortSpacerTable());
        }
        // JF- Get the Object array from the Model.
        IPTGrowableSortedArrayWrapperRO saData = ((IExpListModelRO) m_model).GetReadOnlyItems();
        // Main Table
        result.AddInnerHTMLString("\n");
        myComment = new HTMLComment("Begin: Display ExpList Main Table");
        result.AddInnerHTMLElement(myComment);
        result.AddInnerHTMLString("\n");
        // Hidden input for the row index (used by MoveRow to know which row to move)
        HTMLInput myInput = new HTMLInput(HTMLInputTypes.HIDDEN, ((IExpListModelRO) m_model).GetUniqueString(HTMLHIDDEN_ROWINDEX),
         "");
        result.AddInnerHTMLElement(myInput);
        // Hidden input for the move action (used by MoveRow to know which type of move this is)
        myInput = new HTMLInput(HTMLInputTypes.HIDDEN, ((IExpListModelRO) m_model).GetUniqueString(HTMLHIDDEN_MOVEACTION),
         "");
        result.AddInnerHTMLElement(myInput);
        // Hidden input for the column ID (used on change sort to know which column to sort)
        myInput = new HTMLInput(HTMLInputTypes.HIDDEN, ((IExpListModelRO) m_model).GetUniqueString(HTMLHIDDEN_COLUMNID),
         "");
        result.AddInnerHTMLElement(myInput);
        myTable = new HTMLTable();
        result.AddInnerHTMLElement(myTable);
        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.SetBorder(CommonHTMLStrings.ZERO);
        myTable.SetCellPadding(CommonHTMLStrings.ZERO);
        myTable.SetCellSpacing(CommonHTMLStrings.TWO);
        if (CheckListContainsVisibleRows(saData))
        {
          // 1st Row begin - sub Header
          myTable.AddInnerHTMLElement(GetHTMLForSubHeader());
          // 1st Row end - sub Header
          // JF- Loop through the list and display the rows.
          m_nLastVisibleRow = -1;
          int nStartIndex = ((IExpListModelRO) m_model).GetCurrentLowerObjectCount() - 1;
          int nEndIndex = ((IExpListModelRO) m_model).GetCurrentUpperObjectCount() - 1;
          for (int nRowIndex = nStartIndex;nRowIndex <= nEndIndex;nRowIndex++)
          {
            if (CheckRowIsVisible(nRowIndex))
            {
              // Row nRowIndex - begin
              myRow = new HTMLTableRow();
              myTable.AddInnerHTMLElement(myRow);
              // JF- Set row background
              SetRowStyle(myRow, nRowIndex);
              if (!(((IExpListModelRO) m_model).GetIsListReadOnly() || ((IExpListModelRO) m_model).IsCheckBoxesDisabled()))
              {
                // Display delete checkbox
                myRow.AddInnerHTMLElement(GetHTMLForCheckBox(nRowIndex));
                }
              // Display rest of the row
              myRow.AddInnerHTMLElement(GetHTMLForRow(saData, nRowIndex));
              // Update last visible row
              m_nLastVisibleRow = nRowIndex;
            }
          }
          // Row nRowIndex - end
          HTMLTableRow myBottomRow = GetHTMLForBottomRow();
          if (null != myBottomRow)
          {
            myTable.AddInnerHTMLElement(myBottomRow);
          }
          // Below Main Table
          HTMLTable myBelowMainTable = GetHTMLBelowMainTable();
          if (null != myBelowMainTable)
          {
            result.AddInnerHTMLElement(new HTMLComment("Begin: Display Below Main Table"));
            result.AddInnerHTMLElement(myBelowMainTable);
            result.AddInnerHTMLElement(new HTMLComment("End: Display Below Main Table"));
            }
          if (((IExpListModelRO) m_model).IsPaginationEnabled())
          {
            result.AddInnerHTMLElement(GetHTMLForPaginationLinks((IExpListModelRO) m_model));
            }
          try
          {
            if (((IExpListModelRO) m_model).GetUnusedProperties().GetSize() == 0)
            {
              myTable.AddInnerHTMLElement(GetHTMLForAllRowsAdded());
            }
          }
          catch (XPException e)
          {
            log.Warn("ExpListView::Getting unused properties not supported by model " + m_model.GetName());
            }
        } else
        {
          myTable.AddInnerHTMLElement(GetHTMLForEmptyList());
        }
        result.AddInnerHTMLString("\n");
        myComment = new HTMLComment("End: Display ExpList Main Table");
        result.AddInnerHTMLElement(myComment);
        result.AddInnerHTMLString("\n");
        result.AddInnerHTMLString("\n");
        myComment = new HTMLComment("End: Display ExpListView");
        result.AddInnerHTMLElement(myComment);
        result.AddInnerHTMLString("\n");
      }
      catch (Exception e)
      {
        log.Error(e, "ExpListView::Display An error occured when trying to display the HTML.");
        throw new XPException(m_asOwner.GetString(52, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return resultWrapper;
    }
    
    /// <summary>
    /// Iterate through list in order to determine whether there are any visible rows.
    /// </summary>
    /// <param name="saData"></param>
    /// <returns>boolean</returns>
    private bool CheckListContainsVisibleRows(IPTGrowableSortedArrayWrapperRO saData)
    {
      for (int nRowIndex = 0;nRowIndex < saData.GetCount();nRowIndex++)
      {
        if (CheckRowIsVisible(nRowIndex))
        {
          return true;
        }
      }
      return false;
    }
    
    /// <summary>
    /// Returns Java Script for goToPage function.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    /// <returns>HTMLScript</returns>
    public override HTMLScript DisplayJavascript()
    {
      HTMLScript myScript = null;
      myScript = new HTMLScript("text/javaScript");
      try
      {
        XPStringBuilder sb;
        sb = new XPStringBuilder(300);
        // JavaScript to sort by column	
        sb.Append("function " + ((IExpListModelRO) m_model).GetUniqueString("SortBy") + "(varColumnID)\n");
        sb.Append("{\n");
        sb.Append("  document." + EditorDP.HTMLFORM_NAME + "." + ((IExpListModelRO) m_model).GetUniqueString(HTMLHIDDEN_COLUMNID) + ".value= varColumnID;\n");
        if (m_asOwner.GetIsInPlaceRefreshEnabled())
        {
          sb.Append(InPlaceRefreshHelper.GetJSSendPOSTOnAFormDP(m_asOwner.GetCurrentNormalizedBaseURL(),
           ((IExpListModelRO) m_model).GetUniqueString(HTMLID_EXPLIST), ((IExpListModelRO) m_model).GetUniqueInt(ExpListControl.POSTTOSELF_ACTION_SORTBYCOLUMN))).Append("\n");
          } else
        {
          sb.Append("  postToSelf(" + ((IExpListModelRO) m_model).GetUniqueInt(ExpListControl.POSTTOSELF_ACTION_SORTBYCOLUMN) + ");\n");
          }
        sb.Append("}\n");
        myScript.AddInnerHTMLString(sb.ToString());
        sb = new XPStringBuilder(300);
        // JavaScript to sort by column	
        sb.Append("function " + ((IExpListModelRO) m_model).GetUniqueString("DeleteRows") + "()\n");
        sb.Append("{\n");
        sb.Append("  if(0 != " + ((IExpListModelRO) m_model).GetUniqueString("CountCheckedRows") + "()) \n");
        sb.Append("  {\n");
        String sDelRowConfirmMsg = ((IExpListModelRO) m_model).GetDelRowConfirmMsg();
        if ((null != sDelRowConfirmMsg) && (0 != sDelRowConfirmMsg.Length))
        {
          sb.Append("    if (confirm(\"" + sDelRowConfirmMsg + "\"))");
        }
        if (m_asOwner.GetIsInPlaceRefreshEnabled())
        {
          sb.Append(InPlaceRefreshHelper.GetJSSendPOSTOnAFormDP(m_asOwner.GetCurrentNormalizedBaseURL(),
           ((IExpListModelRO) m_model).GetUniqueString(HTMLID_EXPLIST), ((IExpListModelRO) m_model).GetUniqueInt(ExpListControl.POSTTOSELF_ACTION_DELETE))).Append("\n");
          } else
        {
          sb.Append("    postToSelf(" + ((IExpListModelRO) m_model).GetUniqueInt(ExpListControl.POSTTOSELF_ACTION_DELETE) + ");\n");
          }
        sb.Append("  }\n");
        sb.Append("  else\n");
        sb.Append("  {\n");
        sb.Append("    alert(\'" + m_asOwner.GetString(60, ResourceConstants.INFRASTRUCTURE) + "\');\n");
        sb.Append("  }\n");
        sb.Append("}\n");
        myScript.AddInnerHTMLString(sb.ToString());
        sb = new XPStringBuilder(300);
        // JavaScript to return the number of checked rows.
        sb.Append("function " + ((IExpListModelRO) m_model).GetUniqueString("CountCheckedRows") + "()\n");
        sb.Append("{ \n");
        sb.Append("  var frm = document." + EditorDP.HTMLFORM_NAME + "; \n");
        sb.Append("  var len = frm.elements.length; \n");
        sb.Append("  var nValue = frm." + ((IExpListModelRO) m_model).GetUniqueString(HTMLCHECKBOX_CHECKALL) + ".checked; \n");
        sb.Append("  var i = 0; \n");
        sb.Append("  var nTotal = 0; \n");
        sb.Append("  \n");
        sb.Append("  for(i=0 ; i<len ; i++) \n");
        sb.Append("  { \n");
        sb.Append("    if (frm.elements[i].name == \'" + ((IExpListModelRO) m_model).GetUniqueString(HTMLCHECKBOX_DELROW) + "\') \n");
        sb.Append("    { \n");
        sb.Append("       if (frm.elements[i].checked) \n");
        sb.Append("       { \n");
        sb.Append("          nTotal++; \n");
        sb.Append("       } \n");
        sb.Append("    } \n");
        sb.Append("  } \n");
        sb.Append("  return (nTotal);\n");
        sb.Append("}\n");
        myScript.AddInnerHTMLString(sb.ToString());
        sb = new XPStringBuilder(300);
        // JavaScript to check all the rows
        sb.Append("function " + ((IExpListModelRO) m_model).GetUniqueString("CheckRows") + "()\n");
        sb.Append("{ \n");
        sb.Append("  var frm = document." + EditorDP.HTMLFORM_NAME + "; \n");
        sb.Append("  var len = frm.elements.length; \n");
        sb.Append("  var nValue = frm." + ((IExpListModelRO) m_model).GetUniqueString(HTMLCHECKBOX_CHECKALL) + ".checked; \n");
        sb.Append("  var i = 0; \n");
        sb.Append("  \n");
        sb.Append("  for(i=0 ; i<len ; i++) \n");
        sb.Append("  { \n");
        sb.Append("    if (frm.elements[i].name == \'" + ((IExpListModelRO) m_model).GetUniqueString(HTMLCHECKBOX_DELROW) + "\') \n");
        sb.Append("    { \n");
        sb.Append("       frm.elements[i].checked = nValue;\n");
        sb.Append("    } \n");
        sb.Append("  } \n");
        sb.Append("}\n");
        myScript.AddInnerHTMLString(sb.ToString());
        if (((IExpListModelRO) m_model).GetEnableMoveRowActions())
        {
          sb = new XPStringBuilder(300);
          // JavaScript to sort by column			
          sb.Append("function " + ((IExpListModelRO) m_model).GetUniqueString("MoveRow") + "(varMoveAction, varRowIndex)\n");
          sb.Append("{\n");
          sb.Append("  document." + EditorDP.HTMLFORM_NAME + "." + ((IExpListModelRO) m_model).GetUniqueString(HTMLHIDDEN_ROWINDEX) + ".value= varRowIndex;\n");
          sb.Append("  document." + EditorDP.HTMLFORM_NAME + "." + ((IExpListModelRO) m_model).GetUniqueString(HTMLHIDDEN_MOVEACTION) + ".value= varMoveAction;\n");
          sb.Append("  postToSelf(\'" + ((IExpListModelRO) m_model).GetUniqueInt(ExpListControl.POSTTOSELF_ACTION_MOVEROW) + "\');\n");
          sb.Append("}\n");
          myScript.AddInnerHTMLString(sb.ToString());
        }
        String sEmptyListAlert = ((IExpListModelRO) m_model).GetAlertMessageForEmptyList();
        if (null != sEmptyListAlert)
        {
          sb = new XPStringBuilder(300);
          // JavaScript to check if the list is not empty	
          // JF- Note that this method is not called anywhere in the framework. 
          // It has to be registered as a validation function inside the Editor
          sb.Append("function " + ((IExpListModelRO) m_model).GetUniqueString("CheckEmptyList") + "()\n");
          sb.Append("{\n");
          sb.Append("  \n");
          if (0 == ((IExpListModelRO) m_model).GetReadOnlyItems().GetCount())
          {
            sb.Append("  alert(\"" + sEmptyListAlert + "\"); \n");
            sb.Append("  return false;\n");
          } else
          {
            sb.Append("  return true;\n");
          }
          sb.Append("}\n");
          myScript.AddInnerHTMLString(sb.ToString());
        }
      }
      catch (Exception e)
      {
        log.Error(e, "ExpListView::DisplayJavascript An error occured when trying to display the JavaScript.");
        throw new XPException(m_asOwner.GetString(53, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return myScript;
    }
    
    /// <summary>
    /// Will be called for each row.
    /// Should return the HTML for row _nRowIndex.
    /// </summary>
    /// <param name="_saData"></param>
    /// <param name="_nRowIndex"></param>
    /// <returns>HTMLElement</returns>
    public abstract HTMLElement GetHTMLForRow(IPTGrowableSortedArrayWrapperRO _saData,
     int _nRowIndex);
    
    /// <summary>
    /// Empty method that sub classes can override to add
    /// actions to the action bar (like a create object action).
    /// </summary>
    /// <param name="_myRow">Action bar row object.</param>
    protected virtual void AddHTMLToActionBar(HTMLTableRow _myRow)
    {
    }
    
    /// <summary>
    /// Helper method to determine whether or not row is visible.  This method is used by Display().
    /// </summary>
    /// <param name="_nRowIndex"></param>
    /// <returns>True if row should be displayed, otherwise false.</returns>
    protected virtual bool CheckRowIsVisible(int _nRowIndex)
    {
      return true;
    }
    
    /// <summary>
    /// Returns the HTML for the space below the Action bar.
    /// (The one with the add and delete icons).
    /// </summary>
    /// <returns>HTMLTable</returns>
    protected virtual HTMLTable GetHTMLBelowActionBar()
    {
      return null;
    }
    
    /// <summary>
    /// Helper method to return the HTML for the instruction table.
    /// </summary>
    /// <param name="_sIntructions"></param>
    /// <returns>HTMLTable</returns>
    private HTMLTable GetHTMLForInstructions(String _sIntructions, bool _bIsWarning)
    {
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      myTable = new HTMLTable();
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellPadding("5");
      myTable.SetCellSpacing("0");
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.OBJECT_CONTENT_BG);
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetWidth("100%");
      if (_bIsWarning)
      {
        myCell.SetStyleClass(PTStyleClass.ALERT_ERROR_SUBTITLE);
      } else
      {
        myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
      }
      myCell.AddInnerHTMLString(_sIntructions);
      return myTable;
    }
    
    /// <summary>
    /// Returns the HTML for the last row of the main table.
    /// </summary>
    /// <returns>HTMLTable</returns>
    protected virtual HTMLTableRow GetHTMLForBottomRow()
    {
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the space below the main table,
    /// after all rows have been displayed.
    /// </summary>
    /// <returns>HTMLTable</returns>
    protected virtual HTMLTable GetHTMLBelowMainTable()
    {
      return null;
    }
    
    /// <summary>
    /// Returns the HTML for the Action bar.
    /// (The one with the add and delete icons).
    /// </summary>
    /// <returns>HTMLTable</returns>
    protected virtual HTMLTable GetHTMLForActionBar()
    {
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      myTable = new HTMLTable();
      myTable.SetBorder("0");
      myTable.SetCellPadding("0");
      myTable.SetCellSpacing("0");
      // 1st Row begin (there is only one row)
      myRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(myRow);
      myRow.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);
      // DCA - check if add button wants to be shown
      if (!((IExpListModelRO) m_model).IsAddButtonDisabled())
      {
        // Add icon cell
        myRow.AddInnerHTMLElement(GetHTMLForAddAction());
        // Empty Cell
        myRow.AddInnerHTMLElement(GetHTMLForEmptyCellInActionBar());
      }
      // Add Extra HTML to the action bar (like a "create" button)
      AddHTMLToActionBar(myRow);
      // Empty Cell
      myRow.AddInnerHTMLElement(GetHTMLForEmptyCellInActionBar());
      // JF- No need to display the delete icon, if the list is empty.
      if (0 != ((IExpListModelRO) m_model).GetReadOnlyItems().GetCount())
      {
        // Delete icon cell
        myRow.AddInnerHTMLElement(GetHTMLForDeleteAction());
      }
      // Last Empty Cell
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetWidth("100%");
      myCell.SetHeight("25");
      myCell.AddInnerHTMLString(SPACE);
      // 1st Row ends
      return myTable;
    }
    
    /// <summary>
    /// Returns the HTML for the add cell in the action bar.
    /// </summary>
    /// <returns>HTMLTableCell</returns>
    protected virtual HTMLTableCell GetHTMLForAddAction()
    {
      HTMLTableCell myCell;
      PTConstruct myConstruct;
      HTMLImg myImage;
      HTMLImg myAddIcon;
      HTMLAnchor myAnchor;
      HTMLSpan mySpan;
      myCell = new HTMLTableCell();
      myCell.SetHeight("25");
      myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
      myCell.SetNoWrap(true);
      // Add icon
      myConstruct = UIConstructFactory.GetConstruct(PTImageType.ACTION_ADD,
       true, this.m_asOwner);
      myAddIcon = (HTMLImg) myConstruct.GetHTMLElement();
      myAddIcon.SetAlt("");
      myAddIcon.SetAlign("absmiddle");
      // Object icon
      myConstruct = UIConstructFactory.GetConstruct(((IExpListModelRO) m_model).GetAddIcon(),
       true, this.m_asOwner);
      myImage = (HTMLImg) myConstruct.GetHTMLElement();
      myImage.SetAlt("");
      myImage.SetAlign("absmiddle");
      myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
      myAnchor.AddInnerHTMLString(SPACE);
      myAnchor.SetOnClick("postToSelf(\'" + ((IExpListModelRO) m_model).GetUniqueInt(ExpListControl.POSTTOSELF_ACTION_ADD) + "\'); return false;");
      myAnchor.AddInnerHTMLElement(myAddIcon);
      myAnchor.AddInnerHTMLElement(myImage);
      mySpan = new HTMLSpan();
      myAnchor.AddInnerHTMLElement(mySpan);
      mySpan.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);
      // Add label
      mySpan.AddInnerHTMLString(((IExpListModelRO) m_model).GetAddLabel());
      mySpan.AddInnerHTMLString(SPACE);
      mySpan.AddInnerHTMLString(SPACE);
      myCell.AddInnerHTMLElement(myAnchor);
      return myCell;
    }
    
    /// <summary>
    /// Returns the HTML for the checkbox on row _nRowIndex.
    /// </summary>
    /// <param name="_nRowIndex"></param>
    /// <returns>HTMLTableCell</returns>
    protected virtual HTMLTableCell GetHTMLForCheckBox(int _nRowIndex)
    {
      HTMLTableCell myCell;
      HTMLInput myInput;
      myCell = new HTMLTableCell();
      myCell.SetHeight("30");
      myCell.SetAlign("center");
      myCell.SetVAlign("middle");
      myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
      // checkbox
      myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, ((IExpListModelRO) m_model).GetUniqueString(HTMLCHECKBOX_DELROW),
       ((IExpListModelRO) m_model).GetDeleteIconAlt());
      myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
      myInput.SetValue(XPConvert.ToString(_nRowIndex));
      myCell.AddInnerHTMLElement(myInput);
      return (myCell);
    }
    
    /// <summary>
    /// Returns the HTML for the delete cell in the action bar.
    /// </summary>
    /// <returns>HTMLTableCell</returns>
    protected virtual HTMLTableCell GetHTMLForDeleteAction()
    {
      HTMLTableCell myCell;
      PTConstruct myConstruct;
      HTMLImg myImage;
      HTMLAnchor myAnchor;
      myCell = new HTMLTableCell();
      myCell.SetHeight("25");
      myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
      myCell.SetNoWrap(true);
      // Delete icon
      myConstruct = UIConstructFactory.GetConstruct(((IExpListModelRO) m_model).GetDeleteIcon(),
       true, this.m_asOwner);
      myImage = (HTMLImg) myConstruct.GetHTMLElement();
      myImage.SetAlt(m_asOwner.GetString(122, ResourceConstants.INFRASTRUCTURE));
      myImage.SetAlign("absmiddle");
      myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
      myAnchor.SetOnClick(((IExpListModelRO) m_model).GetUniqueString("DeleteRows") + "(\'" + ((IExpListModelRO) m_model).GetUniqueInt(ExpListControl.POSTTOSELF_ACTION_DELETE) + "\'); return false;");
      myAnchor.AddInnerHTMLElement(myImage);
      myCell.AddInnerHTMLElement(myAnchor);
      return myCell;
    }
    
    /// <summary>
    /// Helper method for empty cell between actions
    /// in the action bar.
    /// </summary>
    /// <returns>HTMLTableCell</returns>
    protected virtual HTMLTableCell GetHTMLForEmptyCellInActionBar()
    {
      HTMLTableCell myCell;
      PTConstruct myConstruct;
      HTMLImg myImage;
      myCell = new HTMLTableCell();
      myCell.SetHeight("25");
      myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_DIVIDER);
      myConstruct = UIConstructFactory.GetConstruct(PTImageType.SPACER, true,
       this.m_asOwner);
      myImage = (HTMLImg) myConstruct.GetHTMLElement();
      myImage.SetAlt("");
      myImage.SetWidth("1");
      myImage.SetHeight("1");
      myImage.SetBorder("0");
      myCell.AddInnerHTMLElement(myImage);
      return myCell;
    }
    
    /// <summary>
    /// The HTML returned by this method will be printed out
    /// when the list is empty.
    /// </summary>
    /// <returns>HTMLTableRow</returns>
    protected virtual HTMLTableRow GetHTMLForEmptyList()
    {
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLP myP;
      myRow = new HTMLTableRow();
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetWidth("100%");
      myCell.SetStyleClass(PTStyleClass.TIPS_TEXT);
      myP = new HTMLP();
      myCell.AddInnerHTMLElement(myP);
      HTMLSpan mySpan = new HTMLSpan();
      myP.AddInnerHTMLElement(mySpan);
      mySpan.SetStyleClass(PTStyleClass.ALERT_ERROR_TITLE);
      mySpan.AddInnerHTMLString(((IExpListModelRO) m_model).GetEmptyListMainMessage());
      myP.AddInnerHTMLElement(new HTMLBr());
      myP.AddInnerHTMLString(((IExpListModelRO) m_model).GetEmptyListInstructions());
      return myRow;
    }
    
    /// <summary>
    /// The HTML returned by this method will be printed out
    /// when all the rows in the list are already added and there are no more 
    /// rows to add.
    /// </summary>
    /// <returns>HTMLTableRow</returns>
    protected virtual HTMLTableRow GetHTMLForAllRowsAdded()
    {
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLP myP;
      myRow = new HTMLTableRow();
      myCell = new HTMLTableCell();
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetWidth("100%");
      myCell.SetColSpan("3");
      myCell.SetStyleClass(PTStyleClass.TIPS_TEXT);
      myP = new HTMLP();
      myP.SetStyleClass(PTStyleClass.ALERT_BG);
      myCell.AddInnerHTMLElement(myP);
      HTMLSpan mySpan = new HTMLSpan();
      myP.AddInnerHTMLElement(mySpan);
      mySpan.SetStyleClass(PTStyleClass.ALERT_TITLE);
      mySpan.AddInnerHTMLString(m_asOwner.GetString(3217, ResourceConstants.PORTALADMINMSGS));
      myP.AddInnerHTMLElement(new HTMLBr());
      myP.AddInnerHTMLString(m_asOwner.GetString(3218, ResourceConstants.PORTALADMINMSGS));
      return myRow;
    }
    
    /// <summary>
    /// Returns the HTML to display the pagination links.
    /// </summary>
    /// <returns>HTMLTable</returns>
    protected virtual HTMLTable GetHTMLForPaginationLinks(IExpListModelRO _model)
    {
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      myTable = new HTMLTable();
      myRow = new HTMLTableRow();
      myCell = new HTMLTableCell();
      myTable.AddInnerHTMLElement(myRow);
      myTable.SetWidth("100%");
      myRow.AddInnerHTMLElement(myCell);
      myCell.SetWidth("100%");
      myCell.SetAlign(CommonHTMLStrings.RIGHT);
      myCell.AddInnerHTMLElement(PaginationControl.CreatePaginationLinks(m_asOwner,
       _model.GetListKey(), _model.GetReadOnlyItems().GetCount(), _model.GetCurrentLowerObjectCount(),
       _model.GetCurrentUpperObjectCount(), _model.GetPaginationSize(), _model.GetModelStorageName()));
      return myTable;
    }
    
    /// <summary>
    /// Helper method for empty cell between actions
    /// in the action bar.
    /// </summary>
    /// <returns>HTMLTableCell</returns>
    protected virtual HTMLTableCell GetHTMLForMoveRowActions(int _nRowIndex)
    {
      HTMLTableCell myCell = new HTMLTableCell();
      if (1 < ((IExpListModelRO) m_model).GetReadOnlyItems().GetCount())
      {
        PTConstruct myConstruct;
        HTMLImg myImage;
        HTMLAnchor myAnchor;
        // JF- We are building an HTML Cell with the 4 move row icons: first, up, down, last
        myCell.SetAlign("right");
        myCell.SetVAlign("middle");
        myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
        myCell.SetNoWrap(true);
        myCell.AddInnerHTMLString(SPACE);
        // JF- Build the link for the Move First icon
        myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        myAnchor.SetOnClick(((IExpListModelRO) m_model).GetUniqueString("MoveRow") + "(\'" + ExpListModel.MOVEACTION_FIRST + "\', \'" + _nRowIndex + "\'); return false;");
        myConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_ARROW_TOP,
         true, this.m_asOwner);
        // JF- Add the Move First icon
        myImage = (HTMLImg) myConstruct.GetHTMLElement();
        myImage.SetAlt(m_asOwner.GetString(56, ResourceConstants.INFRASTRUCTURE));
        myAnchor.AddInnerHTMLElement(myImage);
        myCell.AddInnerHTMLElement(myAnchor);
        // JF- Build the link for the Move Up icon
        myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        myAnchor.SetOnClick(((IExpListModelRO) m_model).GetUniqueString("MoveRow") + "(\'" + ExpListModel.MOVEACTION_UP + "\', \'" + _nRowIndex + "\'); return false;");
        myConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_ARROW_UP,
         true, this.m_asOwner);
        // JF- Add the Move Up icon
        myImage = (HTMLImg) myConstruct.GetHTMLElement();
        myImage.SetAlt(m_asOwner.GetString(57, ResourceConstants.INFRASTRUCTURE));
        myAnchor.AddInnerHTMLElement(myImage);
        myCell.AddInnerHTMLElement(myAnchor);
        // JF- Build the link for the Move Down icon
        myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        myAnchor.SetOnClick(((IExpListModelRO) m_model).GetUniqueString("MoveRow") + "(\'" + ExpListModel.MOVEACTION_DOWN + "\', \'" + _nRowIndex + "\'); return false;");
        myConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_ARROW_DOWN,
         true, m_asOwner);
        // JF- Add the Move Down icon
        myImage = (HTMLImg) myConstruct.GetHTMLElement();
        myImage.SetAlt(m_asOwner.GetString(58, ResourceConstants.INFRASTRUCTURE));
        myAnchor.AddInnerHTMLElement(myImage);
        myCell.AddInnerHTMLElement(myAnchor);
        // JF- Build the link for the Move Last icon
        myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        myAnchor.SetOnClick(((IExpListModelRO) m_model).GetUniqueString("MoveRow") + "(\'" + ExpListModel.MOVEACTION_LAST + "\', \'" + _nRowIndex + "\'); return false;");
        myConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_ARROW_BOTTOM,
         true, this.m_asOwner);
        // JF- Add the Move Last icon
        myImage = (HTMLImg) myConstruct.GetHTMLElement();
        myImage.SetAlt(m_asOwner.GetString(59, ResourceConstants.INFRASTRUCTURE));
        myAnchor.AddInnerHTMLElement(myImage);
        myCell.AddInnerHTMLElement(myAnchor);
      }
      myCell.AddInnerHTMLString(SPACE);
      return myCell;
    }
    
    /// <summary>
    /// Returns the HTML for the sub header.
    /// </summary>
    /// <returns>HTMLTableRow</returns>
    protected virtual HTMLTableRow GetHTMLForSubHeader()
    {
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLAnchor myAnchor;
      HTMLInput myInput;
      HTMLImg myImage;
      PTConstruct myConstruct;
      myRow = new HTMLTableRow();
      myRow.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);
      if (!((IExpListModelRO) m_model).IsCheckBoxesDisabled() && !((IExpListModelRO) m_model).GetIsListReadOnly())
      {
        // Row 1 - Cell 1 - checkbox
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetWidth("20");
        myCell.SetAlign("center");
        myCell.SetVAlign("middle");
        myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
        // checkbox
        myInput = new HTMLInput(HTMLInputTypes.CHECKBOX, ((IExpListModelRO) m_model).GetUniqueString(HTMLCHECKBOX_CHECKALL),
         ((IExpListModelRO) m_model).GetAddIconAlt());
        myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
        myInput.SetOnClick(((IExpListModelRO) m_model).GetUniqueString("CheckRows") + "();");
        myCell.AddInnerHTMLElement(myInput);
      }
      Object[][] oArrExpListColInfo = ((IExpListModelRO) m_model).GetArrExpListColInfo();
      int nColumnID;
      // JF- We loop through the column array and print out each one
      for (int i = 0;i < oArrExpListColInfo.Length;i++)
      {
        // Row 1 - Cell i
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetWidth((String) oArrExpListColInfo[i][ExpListModel.EXPLIST_COLUMN_WIDTH]);
        myCell.SetColSpan((String) oArrExpListColInfo[i][ExpListModel.EXPLIST_COLUMN_COLSPAN]);
        myCell.SetHeight("20");
        myCell.SetNoWrap(((IExpListModelRO) m_model).GetWrapColumnHeaders());
        myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);
        myCell.AddInnerHTMLString(SPACE);
        // JF- Print out the column title with information we get from the array (oArrExpListColInfo)
        if (XPConvert.ToBoolean(oArrExpListColInfo[i][ExpListModel.EXPLIST_COLUMN_ENABLESORT]))
        {
          // JF- If the sort is enabled for this column we need to know what to display:
          // - If the list is not sorted by this column, we display just a link to sort by this column
          // - If the list is currently sorted by this column, we display an arrow that shows the current 
          //   sort direction and a link to reverse the sort.
          nColumnID = XPConvert.ToInteger(oArrExpListColInfo[i][ExpListModel.EXPLIST_COLUMN_ID]);
          // JF- The list is currently sorted by this column
          if (nColumnID == ((IExpListModelRO) m_model).GetMainColumnSort())
          {
            // JF- We build the link to reverse the sort
            myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
            if (m_asOwner.GetIsInPlaceRefreshEnabled())
            {
              myAnchor.SetOnClick(InPlaceRefreshHelper.GetJSSendPOSTOnAFormDP(m_asOwner.GetCurrentNormalizedBaseURL(),
               ((IExpListModelRO) m_model).GetUniqueString(HTMLID_EXPLIST),
               ((IExpListModelRO) m_model).GetUniqueInt(ExpListControl.POSTTOSELF_ACTION_REVERSESORT)));
              } else
            {
              myAnchor.SetOnClick("postToSelf(\'" + ((IExpListModelRO) m_model).GetUniqueInt(ExpListControl.POSTTOSELF_ACTION_REVERSESORT) + "\'); return false;");
              }
            myCell.AddInnerHTMLElement(myAnchor);
            myAnchor.AddInnerHTMLString((String) oArrExpListColInfo[i][ExpListModel.EXPLIST_COLUMN_TITLE]);
            // JF- We figure out the sort direction to know which arrow to display
            if (PTSortedArrayWrapper.PTSAW_SORT_ASCENDING == ((IExpListModelRO) m_model).GetMainColumnSortDirection())
            {
              myConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_ARROW_DESCEND,
               true, this.m_asOwner);
              myImage = (HTMLImg) myConstruct.GetHTMLElement();
              myImage.SetAlt(m_asOwner.GetString(55, ResourceConstants.INFRASTRUCTURE));
              myImage.SetAlign("absmiddle");
              myAnchor.AddInnerHTMLElement(myImage);
            } else
            {
              myConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_ARROW_ASCEND,
               true, this.m_asOwner);
              myImage = (HTMLImg) myConstruct.GetHTMLElement();
              myImage.SetAlt(m_asOwner.GetString(54, ResourceConstants.INFRASTRUCTURE));
              myImage.SetAlign("absmiddle");
              myAnchor.AddInnerHTMLElement(myImage);
            }
          } else
          // JF- The list is currently NOT sorted by this column
          {
            // JF- We build the link to sort by this column
            myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
            myAnchor.SetOnClick(((IExpListModelRO) m_model).GetUniqueString("SortBy") + "(\'" + oArrExpListColInfo[i][ExpListModel.EXPLIST_COLUMN_ID] + "\'); return false;");
            myCell.AddInnerHTMLElement(myAnchor);
            myAnchor.AddInnerHTMLString((String) oArrExpListColInfo[i][ExpListModel.EXPLIST_COLUMN_TITLE]);
            }
        } else
        // JF- The sort was not enabled, just print out the title
        // (we don't need to worry about the link or the arrows)
        {
          myCell.AddInnerHTMLString((String) oArrExpListColInfo[i][ExpListModel.EXPLIST_COLUMN_TITLE]);
          }
      }
      return myRow;
    }
    
    /// <summary>
    /// Helper method to set the style of a row.  This method is used by Display().
    /// </summary>
    /// <param name="_myRow"></param>
    /// <param name="_nRowIndex"></param>
    protected virtual void SetRowStyle(HTMLTableRow _myRow, int _nRowIndex)
    {
      // JF- Set row background
      if (0 == (_nRowIndex % 2))
      {
        _myRow.SetStyleClass(PTStyleClass.LIST_ITEM_TWO_BG);
      } else
      {
        _myRow.SetStyleClass(PTStyleClass.LIST_ITEM_ONE_BG);
      }
    }
    
    /// <summary>
    /// Returns the HTML to display the hidden inputs needed for the pagination.
    /// Those inputs have to be displayed only once per page.
    /// </summary>
    /// <returns>HTMLElement</returns>
    public static HTMLElement GetHiddenInputsForPagination()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLInput myInput;
      // Hidden input for pagination ID (class ID)
      myInput = new HTMLInput(HTMLInputTypes.HIDDEN, PaginationControl.HTMLINPUT_PAGINATION_ID,
       "");
      result.AddInnerHTMLElement(myInput);
      // Hidden input for paginate from
      myInput = new HTMLInput(HTMLInputTypes.HIDDEN, PaginationControl.HTMLINPUT_PAGINATE_FROM,
       "");
      result.AddInnerHTMLElement(myInput);
      // Hidden input for paginate to
      myInput = new HTMLInput(HTMLInputTypes.HIDDEN, PaginationControl.HTMLINPUT_PAGINATE_TO,
       "");
      result.AddInnerHTMLElement(myInput);
      // Hidden input for model storage name
      myInput = new HTMLInput(HTMLInputTypes.HIDDEN, PaginationControl.HTMLINPUT_MODEL_STORAGE_NAME,
       "");
      result.AddInnerHTMLElement(myInput);
      return result;
    }
    
    /// <summary>
    /// Returns the JavaScripts needed for the pagination.
    /// Those JavaScripts have to be displayed only once per page.
    /// </summary>
    /// <param name="_asOwner"></param>
    /// <returns>HTMLScript</returns>
    public static HTMLScript GetJavaScriptsForPagination(AActivitySpace _asOwner)
    {
      HTMLScript myScript = new HTMLScript("text/javaScript");
      myScript.AddInnerHTMLString(PaginationControl.CreatePaginationJavascript(_asOwner));
      return myScript;
    }
  }
}