using System;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.expandablelist;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.htmlconstructs;
// Framework Imports
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.expandablelist.growablelist
{
  /// <summary>
  /// This is the implementation of the Expandable List
  /// framework for the Growable Array.
  /// Is constains the HTML to display the Growable Array.
  /// It gets the data to display from GrowableListModel.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class GrowableListView : ExpListView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.expandablelist.growablelist.GrowableListView");
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "GrowableList";
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String HTMLINPUT_ITEM = "in_tx_Item";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new GrowableListView();
    }
    
    /// <summary>
    /// Returns Java Script for goToPage function.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    /// <returns>HTMLScript</returns>
    public override HTMLScript DisplayJavascript()
    {
      HTMLScript myScript = base.DisplayJavascript();
      if (null == myScript)
      {
        myScript = new HTMLScript("text/javaScript");
      }
      try
      {
        int nClientValidation = ((IGrowableListModelRO) m_model).GetClientValidationType();
        if (GrowableListModel.CLIENT_VALIDATION_TYPE_NONE != nClientValidation)
        {
          XPStringBuilder sb;
          sb = new XPStringBuilder(300);
          // JavaScript validation
          sb.Append("function " + ((IExpListModelRO) m_model).GetUniqueString("ValidateRows") + "()\n");
          sb.Append("{\n");
          sb.Append("  var intRowCount = " + ((IExpListModelRO) m_model).GetReadOnlyItems().GetCount() + ";\n");
          sb.Append("  var strInputName = \'" + ((IGrowableListModelRO) m_model).GetUniqueString(HTMLINPUT_ITEM) + "\';\n");
          sb.Append("  var strFormElem = \'window.document." + AFormDP.HTMLFORM_NAME + ".\' + strInputName;\n");
          sb.Append("  \n");
          sb.Append("  for(var i = 0; i < intRowCount; i ++) \n");
          sb.Append("  {\n");
          if (GrowableListModel.CLIENT_VALIDATION_TYPE_NON_EMPTY_STRING == nClientValidation)
          {
            sb.Append("    if (0 == eval(strFormElem + i + \".value.length\"))\n");
            } else
          if (GrowableListModel.CLIENT_VALIDATION_TYPE_STRING_NO_SPACE == nClientValidation)
          {
            sb.Append("    if (-1 != eval(strFormElem + i + \".value.indexOf(\' \',0)\"))\n");
            } else
          if (GrowableListModel.CLIENT_VALIDATION_TYPE_VALID_URL == nClientValidation)
          {
            sb.Append("    if (((0 != eval(strFormElem + i + \".value.indexOf(\'http://\',0)\"))\n");
            sb.Append("        && (0 != eval(strFormElem + i + \".value.indexOf(\'https://\',0)\")))\n");
            sb.Append("       || (-1 != eval(strFormElem + i + \".value.indexOf(\' \',0)\")))\n");
            }
          sb.Append("    {\n");
          sb.Append("      alert(\"" + ((IGrowableListModelRO) m_model).GetAlertMessage() + "\");\n");
          sb.Append("      eval(strFormElem + i + \".focus()\");\n");
          sb.Append("      return false;\n");
          sb.Append("    }\n");
          sb.Append("  }\n");
          sb.Append("}\n");
          myScript.AddInnerHTMLString(sb.ToString());
        }
      }
      catch (Exception e)
      {
        log.Error(e, "GrowableListView::DisplayJavascript An error occured when trying to display the JavaScript.");
        throw new XPException(m_asOwner.GetString(35, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return myScript;
    }
    
    /// <summary>
    /// Returns the HTML for the row _nRowIndex.
    /// Each row contains just an icon and input text.
    /// </summary>
    /// <param name="_saData"></param>
    /// <param name="_nRowIndex"></param>
    /// <returns>HTMLElement</returns>
    public override HTMLElement GetHTMLForRow(IPTGrowableSortedArrayWrapperRO _saData,
     int _nRowIndex)
    {
      HTMLTableCell myCell;
      HTMLInput myInput;
      myCell = new HTMLTableCell();
      myCell.SetWidth("100%");
      myCell.SetHeight("30");
      myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
      myCell.SetNoWrap(true);
      myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      // row icon
      PTImageType imgRowIcon = ((IGrowableListModelRO) m_model).GetRowIcon();
      if (null != imgRowIcon)
      {
        myCell.AddInnerHTMLElement(GetHTMLForIcon(imgRowIcon));
        myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      }
      // input text
      myInput = new HTMLInput(HTMLInputTypes.TEXT, ((IGrowableListModelRO) m_model).GetUniqueString(HTMLINPUT_ITEM) + _nRowIndex,
       "");
      myCell.AddInnerHTMLElement(myInput);
      myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
      myInput.SetValue((String) _saData.GetItem(_nRowIndex, GrowableListModel.EXPLIST_SORTEDARRAY_PROPID_INPUTTEXT));
      myInput.SetMaxLength("100");
      myInput.SetSize("80");
      return myCell;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Helper method to get the HTML for the icon.
    /// (Each row has an icon before the input text.)
    /// </summary>
    /// <returns>HTMLImg</returns>
    protected virtual HTMLImg GetHTMLForIcon(PTImageType _imgRowIcon)
    {
      HTMLImg myImage;
      PTConstruct myConstruct;
      myConstruct = UIConstructFactory.GetConstruct(_imgRowIcon, true, this.m_asOwner);
      myImage = (HTMLImg) myConstruct.GetHTMLElement();
      myImage.SetAlt("");
      myImage.SetAlign("absmiddle");
      return (myImage);
    }
  }
}