using System;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.expandablelist;
using com.plumtree.uiinfrastructure.form;
// Framework Imports
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.uiinfrastructure.expandablelist.multicolumnlist
{
  /// <summary>
  /// This is the implementation of the Expandable List framework for the Growable
  /// Array. The Growable Array is a list of input texts. On add, a new input
  /// text populated with the default value will be added at the bottom of the
  /// list.
  /// </summary>
  /// <author>JohnH</author>
  public class MultiColumnListModel : ExpListModel,IMultiColumnListModelRO
  {
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "MultiColumnList";
    
    /// <summary>
    /// Number total of columns 
    /// </summary>
    public int m_intNumColumns = 2;
    
    /// <summary>
    /// Icon to display on each row, just before the input text. 
    /// </summary>
    protected PTImageType m_imgRowIcon;
    
    /// <summary>
    /// This is used on add to populate the input text of the newly added row.
    /// Note, it MUST be initialized explicitly by the client.
    /// </summary>
    protected String[] m_arrDefaultValue;
    
    /// <summary>
    /// The names of the form items for each column.
    /// </summary>
    protected String[] m_arrFormDataKeys;
    
    /// <summary>
    /// Adds one row at the bottom of the list and sets the following values:
    /// First column: m_nNextIndex Second column: m_sDefaultValue
    /// </summary>
    /// <param name="_htFormData">Form data</param>
    public override void AddRowsOnRepost(XPHashtable _htFormData)
    {
      String[][] arrDefault = new String[m_intNumColumns][];
      for (int j2c_index = 0;j2c_index < m_intNumColumns;j2c_index++)
      {
        arrDefault[j2c_index] = new String[1];
      }
      for (int i = 0;i < m_intNumColumns;i++)
      {
        arrDefault[i][0] = m_arrDefaultValue[i];
      }
      AddRowsToList(arrDefault);
    }
    
    /// <summary>
    /// Adds an array of new rows to the list.
    /// </summary>
    /// <param name="_aData"></param>
    public virtual void AddRowsToList(Object[][] _aData)
    {
      // rob: the 2d data array passed in and the 2d array we passing in to the addrows method
      // is inversed: oRow[row][col], _aData[col][row]
      Object[][] oRow = new Object[_aData[0].Length][];
      for (int j2c_index = 0;j2c_index < _aData[0].Length;j2c_index++)
      {
        oRow[j2c_index] = new Object[m_intNumColumns + 1];
      }
      for (int rows = 0;rows < _aData[0].Length;rows++)
      {
        oRow[rows][EXPLIST_SORTEDARRAY_PROPID_INDEX] = m_nNextIndex;
        m_nNextIndex++;
        for (int cols = 0;cols < m_intNumColumns;cols++)
        {
          oRow[rows][cols + 1] = _aData[cols][rows];
        }
      }
      base.GetItems().AddRows(oRow);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new MultiColumnListModel();
    }
    
    /// <summary>
    /// This method should ge used by the client to get back the array of values
    /// at the end of the Editor and store them.
    /// </summary>
    /// <returns>Object[][]</returns>
    public virtual Object[][] GetListValues()
    {
      Object[][] aListValues = new Object[m_intNumColumns][];
      for (int j2c_index = 0;j2c_index < m_intNumColumns;j2c_index++)
      {
        aListValues[j2c_index] = new Object[m_saData.GetCount()];
      }
      for (int i = 0;i < m_saData.GetCount();i++)
      {
        for (int j = 1;j <= m_intNumColumns;j++)
        {
          aListValues[j - 1][i] = (String) m_saData.GetItem(i, j);
        }
      }
      return aListValues;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Returns the numColumns.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetNumColumns()
    {
      return m_intNumColumns;
    }
    
    /// <summary>
    /// Returns the icon to display on each row before the input text. If the
    /// value is null, no icon is displayed.
    /// </summary>
    /// <returns>PTImageType</returns>
    public virtual PTImageType GetRowIcon()
    {
      return m_imgRowIcon;
    }
    
    /// <summary>
    /// Initializes the expandable list.
    /// </summary>
    /// <param name="_sSectionTitle">Main title.</param>
    /// <param name="_sSectionSubTitle">Sub title</param>
    /// <param name="_imgAddIcon">Icon for the add action.</param>
    /// <param name="_sAddLabel">Text to display next to the add icon.</param>
    /// <param name="_nListKey">
    /// In case you have multiple Expandable Lists on the same
    /// DP, they should have a different key (otherwise, just set it to
    /// 1).
    /// </param>
    public virtual void InitializeMultiColumnList(String _sSectionTitle, String[] _sSectionSubTitle,
     PTImageType _imgAddIcon, String _sAddLabel, int _nListKey)
    {
      //Set the width based on the number of columns
      int intColWidth = 100 / m_intNumColumns;
      XPStringBuilder sb = new XPStringBuilder(5);
      sb.Append(intColWidth).Append("%");
      Object[][] arrExpListColInfo = new Object[m_intNumColumns][];
      for (int j2c_index = 0;j2c_index < m_intNumColumns;j2c_index++)
      {
        arrExpListColInfo[j2c_index] = new Object[5];
      }
      for (int i = 0;i < m_intNumColumns;i++)
      {
        arrExpListColInfo[i][EXPLIST_COLUMN_ID] = i + 1;
        arrExpListColInfo[i][EXPLIST_COLUMN_TITLE] = _sSectionSubTitle[i];
        arrExpListColInfo[i][EXPLIST_COLUMN_WIDTH] = sb.ToString();
        arrExpListColInfo[i][EXPLIST_COLUMN_COLSPAN] = "1";
        arrExpListColInfo[i][EXPLIST_COLUMN_ENABLESORT] = false;
      }
      base.InitializeExpandableList(_sSectionTitle, arrExpListColInfo, _imgAddIcon,
       _sAddLabel, _nListKey);
    }
    
    /// <summary>
    /// Initalizes the list (PTGrowableSortedArrayWrapper) with a column mapping
    /// and an array of sort. The growable array will always be sorted by the
    /// 1st column ascending.
    /// </summary>
    /// <param name="_saData"></param>
    public override void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
    {
      // JF- Column mapping
      int[] arrColumns = new int[m_intNumColumns + 1];
      arrColumns[0] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
      for (int i = 1;i <= m_intNumColumns;i++)
      {
        arrColumns[i] = i;
      }
      // JF- Sort array
      // The growable array will always be sorted by the 1st column.
      int[][] arrSort = new int[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        arrSort[j2c_index] = new int[2];
      }
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      // JF- We don't initalize the list itself (null below)
      // At first it's empty and we will be initalized if needed using the
      // AddRowsToList method.
      _saData.Initialize(arrColumns, null, arrSort, true);
    }
    
    /// <summary>
    /// Sets a new default value. By default this is an array of empty strings.
    /// This value is used on add to populate the input text of the newly added
    /// row.
    /// </summary>
    /// <param name="_sDefaultValue">New value</param>
    public virtual void SetDefaultValue(String[] _sDefaultValue)
    {
      m_arrDefaultValue = _sDefaultValue;
    }
    
    /// <summary>
    /// Sets the base name for the form data items in each column.
    /// </summary>
    /// <param name="_sDefaultValue">New value</param>
    public virtual void SetFormDataKeys(String[] _arrFormDataKeys)
    {
      m_arrFormDataKeys = new String[_arrFormDataKeys.Length];
      for (int i = 0;i < _arrFormDataKeys.Length;i++)
      {
        m_arrFormDataKeys[i] = GetUniqueString(_arrFormDataKeys[i]);
      }
    }
    
    /// <summary>
    /// Sets the numColumns.
    /// </summary>
    /// <param name="numColumns">The numColumns to set</param>
    public virtual void SetNumColumns(int numColumns)
    {
      m_intNumColumns = numColumns;
    }
    
    /// <summary>
    /// Sets the icon to display on each row before the input text. By default,
    /// no icon is displayed.
    /// </summary>
    /// <param name="_imgRowIcon"></param>
    public virtual void SetRowIcon(PTImageType _imgRowIcon)
    {
      m_imgRowIcon = _imgRowIcon;
    }
    
    /// <summary>
    /// Saves the data on the row _nRowIndex.
    /// </summary>
    /// <param name="_saData">List to update the data on row _nRowIndex</param>
    /// <param name="_nRowIndex">Index of the row to save.</param>
    /// <param name="_htFormData">Data posted by the form.</param>
    /// <returns>int Status for server side validation.</returns>
    protected override int SaveRow(PTGrowableSortedArrayWrapper _saData, int _nRowIndex,
     XPHashtable _htFormData)
    {
      XPStringBuilder sb = new XPStringBuilder(100);
      sb.Append(MultiColumnListView.HTMLINPUT_ITEM);
      for (int i = 1;i <= m_intNumColumns;i++)
      {
        String sDataKey;
        if (m_arrFormDataKeys != null)
        {
          // use the client-specified key names -jas
          sDataKey = m_arrFormDataKeys[i - 1];
        } else
        {
          // generate the default name -jas
          sDataKey = GetUniqueString(sb.Append(i).ToString());
        }
        sDataKey+=_nRowIndex;
        String[] aDataValue;
        // JF- For each input text posted by the form, we get the value back
        // and set the new value in the array.
        if (_htFormData.ContainsKey(sDataKey))
        {
          aDataValue = (String[]) _htFormData.GetElement(sDataKey);
          if ((null != aDataValue) && !(0 == aDataValue.Length))
          {
            _saData.SetItem(_nRowIndex, i, aDataValue[0]);
          }
        } else
        {
          _saData.SetItem(_nRowIndex, i, "");
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
  }
}