using System;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.expandablelist;
using com.plumtree.uiinfrastructure.htmlconstructs;
// Framework Imports
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.expandablelist.multicolumnlist
{
  /// <summary>
  /// This is an implementation of the Expandable List framework for the Multi
  /// Column Array. Is constains the HTML to display the Array. It gets the data
  /// to display from MultiColumnListModel.
  /// </summary>
  /// <author>JohnH</author>
  public class MultiColumnListView : ExpListView
  {
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "MultiColumnList";
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String HTMLINPUT_ITEM = "in_tx_Item";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new MultiColumnListView();
    }
    
    /// <summary>
    /// Returns the HTML for the row _nRowIndex. Each row contains just an icon
    /// and input text for each column.
    /// </summary>
    /// <param name="_saData"></param>
    /// <param name="_nRowIndex"></param>
    /// <returns>HTMLElement</returns>
    public override HTMLElement GetHTMLForRow(IPTGrowableSortedArrayWrapperRO _saData,
     int _nRowIndex)
    {
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLTableCell myCell;
      HTMLInput myInput;
      int intColWidth = 100 / ((IMultiColumnListModelRO) m_model).GetNumColumns();
      XPStringBuilder sbColPercent = new XPStringBuilder(5);
      sbColPercent.Append(intColWidth).Append("%");
      myCell = new HTMLTableCell();
      myCell.SetWidth(sbColPercent.ToString());
      myCell.SetHeight("30");
      myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
      myCell.SetNoWrap(true);
      myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      // row icon
      PTImageType imgRowIcon = ((IMultiColumnListModelRO) m_model).GetRowIcon();
      if (null != imgRowIcon)
      {
        myCell.AddInnerHTMLElement(GetHTMLForIcon(imgRowIcon));
        myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      }
      XPStringBuilder sbFieldName = new XPStringBuilder(100);
      sbFieldName.Append(HTMLINPUT_ITEM);
      for (int i = 1;i <= ((IMultiColumnListModelRO) m_model).GetNumColumns();
      i++)
      {
        sbFieldName.Append(i);
        //Only add a new cell if this isn't the first one
        if (i > 1)
        {
          myCell = new HTMLTableCell();
          myCell.SetWidth(sbColPercent.ToString());
          myCell.SetHeight("30");
          myCell.SetStyleClass(PTStyleClass.LIST_TEXT);
          myCell.SetNoWrap(true);
        }
        myInput = new HTMLInput(HTMLInputTypes.TEXT, ((IMultiColumnListModelRO) m_model).GetUniqueString(sbFieldName.ToString()) + _nRowIndex,
         "");
        myCell.AddInnerHTMLElement(myInput);
        myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
        myInput.SetValue((String) _saData.GetItem(_nRowIndex, i));
        myInput.SetSize(sbColPercent.ToString());
        result.AddInnerHTMLElement(myCell);
      }
      if (((IMultiColumnListModelRO) m_model).GetEnableMoveRowActions())
      {
        result.AddInnerHTMLElement(base.GetHTMLForMoveRowActions(_nRowIndex));
        }
      // input text
      return result;
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Helper method to get the HTML for the icon. (Each row has an icon before
    /// the input text.)
    /// </summary>
    /// <param name="_imgRowIcon"></param>
    /// <returns>HTMLImg</returns>
    protected virtual HTMLImg GetHTMLForIcon(PTImageType _imgRowIcon)
    {
      HTMLImg myImage;
      PTConstruct myConstruct;
      myConstruct = UIConstructFactory.GetConstruct(_imgRowIcon, true, this.m_asOwner);
      myImage = (HTMLImg) myConstruct.GetHTMLElement();
      myImage.SetAlt("");
      myImage.SetAlign(CommonHTMLStrings.ABSMIDDLE);
      return myImage;
    }
    
    /// <returns></returns>
    protected override HTMLTableRow GetHTMLForSubHeader()
    {
      HTMLTableRow myRow = base.GetHTMLForSubHeader();
      IXPEnumerator enumItems = myRow.GetInnerHTMLElements(false);
      if (enumItems != null)
      {
        HTMLElement eTemp = null;
        while (enumItems.MoveNext())
        {
          eTemp = (HTMLElement) enumItems.GetCurrent();
        }
        if (null != eTemp)
        {
          ((HTMLTableCell) eTemp).SetColSpan(CommonHTMLStrings.TWO);
        }
      }
      return myRow;
    }
  }
}