using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.arraywrapper;
using com.plumtree.uiinfrastructure.expandablelist.multicolumnlist;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.uiinfrastructure.expandablelist.multicolumnlist.sorted
{
  /// <summary>
  /// This model allows you to use a sorted multi column list.
  /// </summary>
  /// <author>DonH</author>
  public class SortedMultiColumnListModel : MultiColumnListModel
  {
    
    /// <summary>
    /// Class name for the activity space 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "SortedMultiColumnList";
    
    /// <summary>
    /// Zero based column id for the default column to sort by.
    /// </summary>
    protected int m_nDefaultSortColumn;
    
    /// <summary>
    /// Initializes the expandable list.
    /// </summary>
    /// <param name="_sSectionTitle">Main title.</param>
    /// <param name="_sSectionSubTitle">Sub title</param>
    /// <param name="_bEnableSort">for each column</param>
    /// <param name="_nDefaultSortColumn">the zero based column id for the default column to sort by.</param>
    /// <param name="_imgAddIcon">Icon for the add action.</param>
    /// <param name="_sAddLabel">Text to display next to the add icon.</param>
    /// <param name="_nListKey">
    /// In case you have multiple Expandable Lists on the same
    /// DP, they should have a different key (otherwise, just set it to
    /// 1).
    /// </param>
    public virtual void InitializeMultiColumnList(String _sSectionTitle, String[] _sSectionSubTitle,
     bool[] _bEnableSort, int _nDefaultSortColumn, PTImageType _imgAddIcon,
     String _sAddLabel, int _nListKey)
    {
      //Set the width based on the number of columns
      int intColWidth = 100 / m_intNumColumns;
      XPStringBuilder sb = new XPStringBuilder(5);
      sb.Append(intColWidth).Append("%");
      m_nDefaultSortColumn = _nDefaultSortColumn;
      Object[][] arrExpListColInfo = new Object[m_intNumColumns][];
      for (int j2c_index = 0;j2c_index < m_intNumColumns;j2c_index++)
      {
        arrExpListColInfo[j2c_index] = new Object[5];
      }
      for (int i = 0;i < m_intNumColumns;i++)
      {
        arrExpListColInfo[i][EXPLIST_COLUMN_ID] = i + 1;
        arrExpListColInfo[i][EXPLIST_COLUMN_TITLE] = _sSectionSubTitle[i];
        arrExpListColInfo[i][EXPLIST_COLUMN_WIDTH] = sb.ToString();
        arrExpListColInfo[i][EXPLIST_COLUMN_COLSPAN] = "1";
        arrExpListColInfo[i][EXPLIST_COLUMN_ENABLESORT] = _bEnableSort[i];
        }
      base.InitializeExpandableList(_sSectionTitle, arrExpListColInfo, _imgAddIcon,
       _sAddLabel, _nListKey);
    }
    
    /// <summary>
    /// Initalizes the list (PTGrowableSortedArrayWrapper) with a column mapping
    /// and a sort array. The growable array will always be sorted by the
    /// 1st column ascending.
    /// </summary>
    /// <param name="_saData"></param>
    public override void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
    {
      // JF- Column mapping
      int[] arrColumns = new int[m_intNumColumns + 1];
      arrColumns[0] = EXPLIST_SORTEDARRAY_PROPID_INDEX;
      for (int i = 1;i <= m_intNumColumns;i++)
      {
        arrColumns[i] = i;
      }
      // JF- Sort array
      // The growable array will always be sorted by the 1st column.
      int[][] arrSort = new int[1][];
      for (int j2c_index = 0;j2c_index < 1;j2c_index++)
      {
        arrSort[j2c_index] = new int[2];
      }
      // This is not zero based.
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID] = m_nDefaultSortColumn + 1;
      arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION] = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      // JF- We don't initalize the list itself (null below)
      // At first it's empty and we will be initalized if needed using the
      // AddRowsToList method.
      _saData.Initialize(arrColumns, null, arrSort, true);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new SortedMultiColumnListModel();
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
  }
}