using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.openlog;
using com.plumtree.openfoundation.io;
namespace com.plumtree.uiinfrastructure.filedownload
{
  /// <author>
  /// adolfoc
  /// A simple page that does a binary download. NOTE: This page does not
  /// support downloading filenames with multi-byte characters!
  /// </author>
  public class DownloadBinaryDP : IDisplayPage
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.filedownload.DownloadBinaryDP");
    public const String STR_MVC_CLASS_NAME = "DownloadBinaryDP";
    private AActivitySpace m_asOwner = null;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Display(IWebData)"></seealso>
    public virtual void Display(IWebData webData)
    {
      try
      {
        FileDownloadModel fdlModel = (FileDownloadModel) m_asOwner.GetModel(FileDownloadModel.STR_MVC_CLASS_NAME);
        String strFileToDownload = (String) m_asOwner.GetSubSession().GetAttribute(fdlModel.GetSessionKeyToDownload());
        XPFileInputStream xpfo = new XPFileInputStream(strFileToDownload);
        byte[] arOut = null;
        // well, we think it's at least one... We'll find out - won't we.
        int nBytesToRead = 1;
        // The following code is the not correct code, but is temporarily broken
        // because of bug# 17877. When that bug is fixed, this code should be fixed.
        // XPFileInputStream.Read should return -1 when it reaches the end of File.
        // However, because of 17877 it doesn't in .NET - it returns 0. Furthermore,
        // because of 17877, nBytesToRead always shows the full size of the stream in
        // .NET, even though it is updated properly in java.
        int nBytesRead = 1;
        while ((0 < nBytesRead) && (0 < nBytesToRead))
        {
          nBytesToRead = xpfo.GetAvailableBytes();
          byte[] arBytes = new byte[nBytesToRead];
          nBytesRead = xpfo.Read(arBytes);
          if (0 < nBytesRead)
          {
            arOut = ConcatByteArrays(arOut, arBytes);
          }
        }
        // arOut should contain the binary contents of the file now.
        webData.SetData(arOut);
        // Set the content type - if specified. The WebData object automatically
        // sets a reasonable default if the content type is not specified.
        String strContentType = fdlModel.GetContentTypeToDownload();
        if ((null != strContentType) && (0 < strContentType.Length))
        {
          webData.SetHeader(HTTPHeaderConstants.CONTENT_TYPE, strContentType);
          }
        XPFile xpfOut = new XPFile(strFileToDownload);
        webData.SetHeader(HTTPHeaderConstants.CONTENT_DISPOSITION, "attachment; filename=" + xpfOut.GetName());
        ((WebData) webData).SetSuppressAutomaticHeaders(true);
      }
      // AGC - Future Support - if necessary.
      // For I.E. we must URL-Encode the file name in order to pass filenames
      // with multi-byte characters. It doesn't work in Netscape though, so we
      // must not URL-Encode it in that case. Unfortunately, this requires knowledge
      // of the browser type.... yeesh.
      catch (Exception e)
      {
        // This needs better error handling.
        log.Error(e, "Unknown Error attempting to download file.");
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#GetOwner()"></seealso>
    public virtual AActivitySpace GetOwner()
    {
      return m_asOwner;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Init(AActivitySpace)"></seealso>
    public virtual void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
    }
    
    /// <param name="arByteA"></param>
    /// <param name="arByteB"></param>
    /// <returns></returns>
    private byte[] ConcatByteArrays(byte[] arByteA, byte[] arByteB)
    {
      if (null == arByteA)
      {
        return arByteB;
      }
      if (null == arByteB)
      {
        return arByteA;
      }
      int nTotalLength = arByteA.Length + arByteB.Length;
      int nOffset = arByteA.Length;
      byte[] arResult = new byte[nTotalLength];
      // Copy over A first, then B
      for (int nIndex = 0;nIndex < arResult.Length;nIndex++)
      {
        if (nIndex < nOffset)
        {
          // We're in A territory
          arResult[nIndex] = arByteA[nIndex];
        } else
        {
          // Now we copy B
          arResult[nIndex] = arByteB[nIndex - nOffset];
        }
      }
      return arResult;
    }
  }
}