using System;
using com.plumtree.openlog;
using XPFile=com.plumtree.openfoundation.io.XPFile;
using com.plumtree.openfoundation.util;
using XPHttpUtility=com.plumtree.openfoundation.web.XPHttpUtility;
using com.plumtree.uiinfrastructure.activityspace;
using ResourceConstants=com.plumtree.uiinfrastructure.constants.ResourceConstants;
using PTStyleClass=com.plumtree.xpshared.htmlconstructs.PTStyleClass;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.filedownload
{
  /// <author>adolfoc</author>
  public class DownloadFilePopupDP : IDisplayPage
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.filedownload.DownloadFilePopupDP");
    
    /// <summary>
    /// Class name for the activity space
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "DownloadFilePopupDP";
    // Strings to localize.
    
    /// <summary>
    /// HTML Object name: close button 
    /// </summary>
    public const String HTMLBUTTON_CLOSE = "in_bu_Close";
    
    /// <summary>
    /// Parent AS.
    /// </summary>
    private AFileDownloadAS m_asOwner;
    
    /// <summary>
    /// Associated read-only Model.
    /// </summary>
    private FileDownloadModel m_model;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new DownloadFilePopupDP();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Display(IWebData)"></seealso>
    public virtual void Display(IWebData webData)
    {
      HTMLPage myPage = new HTMLPage();
      // Set the HTMLPage as the data to be sent to the browser.
      webData.SetData(myPage);
      HTMLHead myHead = new HTMLHead();
      myPage.SetHead(myHead);
      //		GetFormTitle should already be encoded.
      HTMLTitle myTitle = new HTMLTitle(m_asOwner.GetString(582, ResourceConstants.PORTALBROWSINGMSGS));
      HTMLBody myBody = new HTMLBody();
      myPage.SetBody(myBody);
      HTMLTable myMainTable;
      HTMLTable myTable;
      HTMLTableRow myMainTableRow;
      HTMLTableRow myRow;
      HTMLTableCell myMainTableCell;
      HTMLTableCell myCell;
      HTMLInput myInput;
      HTMLAnchor myAnchor;
      try
      {
        // AGC - Create the URL that actually commits a download.
        ASURL asurlDownload = new ASURL(m_asOwner);
        asurlDownload.SetLinkCreateNewSpace(AFileDownloadAS.STR_MVC_CLASS_NAME,
         m_asOwner);
        asurlDownload.SetDisplayPage(DownloadBinaryDP.STR_MVC_CLASS_NAME);
        asurlDownload.SetControl(FileDownloadControl.STR_MVC_CLASS_NAME);
        asurlDownload.AddControlArgument(FileDownloadControl._QS_FD_CONTENT_TYPE,
         "application/octet-stream");
        asurlDownload.AddControlArgument(FileDownloadControl._QS_FD_SESSION_KEY,
         ((FileDownloadModel) m_model).GetSessionKeyToDownload());
        myHead.AddInnerHTMLElement(myTitle);
        // <META  HTTP-EQUIV="REFRESH" CONTENT="0; URL=http://www.redmeat.com">
        HTMLMeta myMeta = new HTMLMeta("2;URL=" + asurlDownload.GetURLAsString());
        myMeta.SetHTTPEquiv("REFRESH");
        myHead.AddInnerHTMLElement(myMeta);
        // Add the stylesheets.
        myHead.AddInnerHTMLElement(m_asOwner.GetStyleSheetLinks());
        // Main table
        myMainTable = new HTMLTable();
        myBody.AddInnerHTMLElement(myMainTable);
        myMainTable.SetWidth("100%");
        myMainTable.SetBorder("0");
        myMainTable.SetCellPadding("0");
        myMainTable.SetCellSpacing("0");
        // Main table = row 1
        myMainTableRow = new HTMLTableRow();
        myMainTable.AddInnerHTMLElement(myMainTableRow);
        myMainTableCell = new HTMLTableCell();
        myMainTableRow.AddInnerHTMLElement(myMainTableCell);
        myMainTableCell.SetHeight("60");
        myMainTableCell.SetWidth("100%");
        myMainTableCell.SetStyleClass(PTStyleClass.DIALOGUE_BG);
        myMainTableCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        // Main table = row 2
        myMainTableRow = new HTMLTableRow();
        myMainTable.AddInnerHTMLElement(myMainTableRow);
        myMainTableCell = new HTMLTableCell();
        myMainTableRow.AddInnerHTMLElement(myMainTableCell);
        // Content table
        myTable = new HTMLTable();
        myMainTableCell.AddInnerHTMLElement(myTable);
        myTable.SetWidth("100%");
        myTable.SetHeight("200");
        myTable.SetBorder("0");
        myTable.SetCellPadding("0");
        myTable.SetCellSpacing("10");
        // Row 1 - begin
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetWidth("100%");
        myCell.SetHeight("20");
        myCell.SetStyleClass(PTStyleClass.DIALOGUE_NOTICE_HEADER);
        myCell.AddInnerHTMLString(m_asOwner.GetString(582, ResourceConstants.PORTALBROWSINGMSGS));
        // Row 1 - end
        // Row 2 - begin
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetWidth("100%");
        myCell.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
        myCell.AddInnerHTMLString(m_asOwner.GetString(583, ResourceConstants.PORTALBROWSINGMSGS));
        myCell.AddInnerHTMLElement(new HTMLBr());
        myCell.AddInnerHTMLElement(new HTMLBr());
        String strFileToDownload = (String) m_asOwner.GetSubSession().GetAttribute(m_model.GetSessionKeyToDownload());
        XPFile xpf = new XPFile(strFileToDownload);
        myAnchor = new HTMLAnchor(asurlDownload.GetURLAsString());
        myAnchor.AddInnerHTMLString(XPHttpUtility.HtmlEncode(xpf.GetName()));
        myCell.AddInnerHTMLElement(myAnchor);
        // Row 2 - end
        // Row 3 - begin
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetWidth("100%");
        myCell.SetAlign("center");
        myCell.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
        // Button to close the window
        myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_CLOSE, "");
        myCell.AddInnerHTMLElement(myInput);
        myInput.SetValue(m_asOwner.GetString(584, ResourceConstants.PORTALBROWSINGMSGS));
        myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
        myInput.SetOnClick("window.close();");
        myCell.AddInnerHTMLElement(new HTMLBr());
        myCell.AddInnerHTMLElement(new HTMLBr());
        // Row 3 - end
        // Row 6 - begin
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetWidth("100%");
        myCell.SetHeight("100%");
        myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        // Row 6 - end
        // Main table = row 3
        myMainTableRow = new HTMLTableRow();
        myMainTable.AddInnerHTMLElement(myMainTableRow);
        myMainTableCell = new HTMLTableCell();
        myMainTableRow.AddInnerHTMLElement(myMainTableCell);
        myMainTableCell.SetHeight("160");
        myMainTableCell.SetWidth("100%");
        myMainTableCell.SetStyleClass(PTStyleClass.DIALOGUE_BG);
        myMainTableCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        // AGC- Now add our javascript
        HTMLScript myScript = DisplayJavascript();
        myHead.AddInnerHTMLElement(myScript);
      }
      catch (Exception e)
      {
        log.Error(e, "Display An error occured when trying to display the HTML.");
        throw new XPException(m_asOwner.GetString(585, ResourceConstants.PORTALBROWSINGMSGS),
         e);
      }
      webData.SetData(myPage);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    public virtual HTMLScript DisplayJavascript()
    {
      HTMLScript myScript = new HTMLScript("text/javaScript");
      XPStringBuilder sb;
      try
      {
        sb = new XPStringBuilder(300);
        // AGC- redirect the parent window when we popup.
        sb.Append("  window.opener.postToSelf(\'" + FileDownloadControl.POSTTOSELF_ACTION_REDIRECT_AFTER_DOWNLOAD + "\'); \n");
        myScript.AddInnerHTMLString(sb.ToString());
      }
      catch (HTMLException he)
      {
        log.Error(he, "DisplayJavascript An error occured when trying to display the JavaScript.");
        }
      return myScript;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#GetOwner()"></seealso>
    public virtual AActivitySpace GetOwner()
    {
      return m_asOwner;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO,AActivitySpace)"></seealso>
    public virtual void Init(AActivitySpace parent)
    {
      m_model = (FileDownloadModel) parent.GetModel(FileDownloadModel.STR_MVC_CLASS_NAME);
      m_asOwner = (AFileDownloadAS) parent;
    }
  }
}