using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.form
{
  /// <summary>
  /// This AS provides a base implementation for an HTML Form.
  /// </summary>
  /// <author>Don Hayler</author>
  public abstract class AFormAS : AActivitySpace,IFormAS
  {
    
    /// <summary>
    /// image server variable 
    /// </summary>
    public const String JAVASCRIPT_GVAR_PTIMGSVRIMGSURL = "g_PTImgSvrImgsURL";
    
    /// <summary>
    /// Shared image server variable 
    /// </summary>
    public const String JAVASCRIPT_GVAR_SHAREDIMGSVRIMGSURL = "g_SharedImgSvrImgsURL";
    
    /// <summary>
    /// help server variable 
    /// </summary>
    public const String JAVASCRIPT_GVAR_PTHELPSVRBASEURL = "g_HelpSvrBaseURL";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public abstract override String GetName();
    
    /// <summary>
    /// This is a helper method that all classes that inherit from AFormDP can override
    /// if they like.  The method provides a hook into the head so that DP's can add javascript.
    /// Simply override and return and HTMLScript.  Also, it is best to return the results of the
    /// DisplayJavaScript functions from the appropriate views.
    /// Override this method to Display javascript from child IViews
    /// </summary>
    /// <returns>HTMLScriptCollection the Javascript from the Child IViews</returns>
    public virtual HTMLScriptCollection DisplayCommonJavaScript()
    {
      HTMLScriptCollection scripts = new HTMLScriptCollection();
      HTMLScript myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      scripts.AddInnerHTMLElement(myScript);
      myScript.AddInnerHTMLString("var " + JAVASCRIPT_GVAR_PTIMGSVRIMGSURL + " = \"" + ConfigHelper.GetPortalImageServerImagesURL(this) + "\";\n" + "var " + JAVASCRIPT_GVAR_SHAREDIMGSVRIMGSURL + " = \"" + ConfigHelper.GetSharedImageServerImagesURL(this) + "\";");
      myScript.AddInnerHTMLString("var " + JAVASCRIPT_GVAR_PTHELPSVRBASEURL + " = \"" + ConfigHelper.GetImageServerBaseURL(this) + "\";\n");
      // JSRegistry include of jsrobohelp component
      scripts.AddInnerHTMLElement(ConfigHelper.GetJSIncluderJSComponentInclude(ConfigHelper.JSCOMPONENT_JSROBOHELPER,
       this));
      /*
       * SAMPLE
       *
       * scripts.AddInnerHTMLElement(m_asOwner.GetView(ObjectView.STR_MVC_CLASS_NAME).DisplayJavascript());
       */
       
      return scripts;
    }
    
    /// <summary>
    /// Returns the name of the repost control class.
    /// E.g.: PropRepostControl.GetName()
    /// This can also be overridden on a per DP basis.  See AFormDP#GetRepostControlName()
    /// The repost control does not necessarily need to extend RepostControl.  A form
    /// can be submitted to any type of control as long as it knows how to properly
    /// interpret the form.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetRepostControlName()
    {
      // This method needs to be implemented by all sub-classes that use the repost
      // framework.
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AActivitySpace#Init()"></seealso>
    public override void Init()
    {
      base.Init();
      RegisterView(FormHeaderView.STR_MVC_CLASS_NAME, null);
      RegisterView(MediumFormHeaderView.STR_MVC_CLASS_NAME, null);
      RegisterView(FormFooterView.STR_MVC_CLASS_NAME, null);
    }
  }
}