using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.form
{
  /// <summary>
  /// This class implements the HTML to print out the Done and Cancel
  /// at the bottom of the form in small pop-up style.
  /// This View is printed out by AFormDP.
  /// This view needs to be 508 compliant
  /// To turn this header on ... go to AFormDP::SetShowFormFooter()
  /// This is the small pop-up footer.
  /// </summary>
  /// <author>Don Hayler</author>
  public class FormFooterView : IView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.form.FormFooterView");
    
    /// <summary>
    /// MVC Object Name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "FormFooter";
    
    /// <summary>
    /// HTML Input name for finish button 
    /// </summary>
    public const String HTMLBUTTON_EDITOR_FINISH = "in_bu_Done";
    
    /// <summary>
    /// HTML Input name for cancel button 
    /// </summary>
    public const String HTMLBUTTON_EDITOR_CANCEL = "in_bu_Cancel";
    
    /// <summary>
    /// internal MVC data 
    /// </summary>
    private AActivitySpace m_asOwner;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new FormFooterView();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Display()"></seealso>
    public virtual HTMLElement Display()
    {
      bool bNoJavascript = (m_asOwner.GetAccessStyle() != AccessStyles.STANDARD);
      HTMLElementCollection myResult = new HTMLElementCollection();
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLInput myInput;
      try
      {
        myResult.AddInnerHTMLElement(new HTMLComment("Begin: Footer buttons table"));
        myTable = new HTMLTable();
        myResult.AddInnerHTMLElement(myTable);
        myTable.SetBorder("0");
        myTable.SetWidth("100%");
        myTable.SetCellPadding("0");
        myTable.SetCellSpacing("10");
        myTable.SetStyleClass(PTStyleClass.BAN_HEADER_BG);
        // 1st Row
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.BAN_HEADER_BG);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetAlign(CommonHTMLStrings.CENTER);
        if (bNoJavascript)
        {
          // 508 & low bandwidth buttons
          // If we cannot use javascript, we need special submit buttons
          // Done button
          // "OK"
          String strOK = m_asOwner.GetString(66, ResourceConstants.INFRASTRUCTURE);
          myInput = new HTMLInput(HTMLInputTypes.SUBMIT, RepostControl.HTMLSUBMIT_508_POSTTOSELF + RepostControl.POSTTOSELF_ACTION_FINISH,
           "");
          myCell.AddInnerHTMLElement(myInput);
          // "OK"
          myInput.SetValue(strOK);
          // "OK"
          myInput.SetAlt(strOK);
          myInput.SetAccessKey(m_asOwner.GetString(26, "ptmsgs_portal508quickkeymsgs"));
          myInput.SetTitle(m_asOwner.GetString(28, "ptmsgs_portal508quickkeymsgs"));
          myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
          myCell.AddInnerHTMLString("&nbsp;");
          // Cancel button
          String strCancel = m_asOwner.GetString(67, 
          // "Cancel"
          ResourceConstants.INFRASTRUCTURE);
          myInput = new HTMLInput(HTMLInputTypes.SUBMIT, RepostControl.HTMLSUBMIT_508_POSTTOSELF + RepostControl.POSTTOSELF_ACTION_CANCEL,
           "");
          myCell.AddInnerHTMLElement(myInput);
          // "Cancel"
          myInput.SetValue(strCancel);
          // "Cancel"
          myInput.SetAlt(strCancel);
          myInput.SetAccessKey(m_asOwner.GetString(27, "ptmsgs_portal508quickkeymsgs"));
          myInput.SetTitle(m_asOwner.GetString(29, "ptmsgs_portal508quickkeymsgs"));
          myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
        } else
        {
          // Normal buttons
          // Done button
          myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_EDITOR_FINISH,
           "");
          myCell.AddInnerHTMLElement(myInput);
          // "Done"
          myInput.SetValue(m_asOwner.GetString(66, ResourceConstants.INFRASTRUCTURE));
          myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
          myInput.SetOnClick("postToSelf(" + RepostControl.POSTTOSELF_ACTION_FINISH + "); return false;");
          // Cancel button
          myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_EDITOR_CANCEL,
           "");
          myCell.AddInnerHTMLElement(myInput);
          // "Cancel"
          myInput.SetValue(m_asOwner.GetString(67, ResourceConstants.INFRASTRUCTURE));
          myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
          myInput.SetOnClick("postToSelf(" + RepostControl.POSTTOSELF_ACTION_CANCEL + "); return false;");
          }
        myCell.AddInnerHTMLElement(new HTMLComment("End: Footer buttons table"));
        }
      catch (Exception e)
      {
        log.Error(e, "Display: The following error occured when trying to display the HTML:");
        throw new XPException(m_asOwner.GetString(68, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return myResult;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    public virtual HTMLScript DisplayJavascript()
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    // Strings to localize
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO,AActivitySpace)"></seealso>
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      m_asOwner = parent;
    }
  }
}