using System;
namespace com.plumtree.uiinfrastructure.form
{
  /// <summary>
  /// FormPageTypes is an emumeration of the various form page types allowed.
  /// </summary>
  public class FormPageTypes
  {
    
    /// <summary>
    /// The standard form style for normal windows.
    /// A normal page will usually consist of an editor or forms page.
    /// Either could be in the main browser window or be a popup with tabs.
    /// These forms have multiple pages and most likely use the Editor
    /// framework. The tops of the forms have a titlebar banner with a
    /// Finish and Cancel button.
    /// </summary>
    public static readonly FormPageTypes NORMAL_PAGE = new FormPageTypes();
    
    /// <summary>
    /// The style for medium pop-ups.
    /// This UI is used for complex, 1-page form popups, such as for adding
    /// objects, displaying a tree, etc. The dialog contains a dark blue
    /// title banner with help text below in a gray rectangle. A forms area
    /// will be located below that and may have a Section titlebar and a
    /// toolbar. The window function buttons are usually OK and Cancel and
    /// are located in the upper right of the Titlebar banner.
    /// Note that there is an exception to when you use the Medium popup.
    /// When there is more than one page, which happens with tabbed popups,
    /// the Normal page should be used instead of the Medium popup.
    /// </summary>
    public static readonly FormPageTypes MEDIUM_POPUP = new FormPageTypes();
    
    /// <summary>
    /// The style for small pop-ups.
    /// This UI is used for things like custom messages (HTML), warnings,
    /// and simple dialogs (no more than 2 input boxes or controls). So,
    /// for instance, a Create Folder would use the Small Popup, having
    /// only a title, help text, and a name and description input box.
    /// For Small Popups, the buttons are:
    /// 1. at the bottom of the popup
    /// 2. are centered
    /// 3. Usually will be OK or OK and Cancel buttons.
    /// </summary>
    public static readonly FormPageTypes SMALL_POPUP = new FormPageTypes();
    
    /// <summary>
    /// The style for custom pages.
    /// This UI is used when you don't want Finish and Cancel buttons on
    /// your page and only want to use the form.
    /// </summary>
    public static readonly FormPageTypes NO_BUTTONS = new FormPageTypes();
    
    /// <summary>
    /// Creates a new FormPageTypes object.
    /// </summary>
    /// <param name="_nAccessID"></param>
    private FormPageTypes()
    {
    }
  }
}