using System;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.form
{
  /// <summary>
  /// This interface is used along with RepostControl and IRepostModel
  /// to partially implement the logic of the repost action.
  /// Those two classes are used by the Editor Framework to help implementing
  /// the repost mechanism on any Editor page.
  /// They can also be used by any other Activity Space.
  /// See instructions in RepostControl.java on how to use that
  /// logic outside the Editor Framework.
  /// </summary>
  /// <author>Julie Fournier</author>
  public interface IFormAS
  {
    
    /// <summary>
    /// Return the common javascript that should go in all AFormDPs in this AS.
    /// Override this method to Display javascript in all AFormDPs in an AFormAS.
    /// </summary>
    /// <returns>HTMLScriptCollection the common Javascript for all forms in this space.</returns>
    HTMLScriptCollection DisplayCommonJavaScript();
    
    /// <summary>
    /// Returns the name of the repost control class.
    /// E.g.: PropRepostControl.GetName()
    /// </summary>
    /// <returns>String</returns>
    String GetRepostControlName();
  }
}