using System;
using com.plumtree.uiinfrastructure.activityspace;
// Framework Imports
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.form
{
  /// <summary>
  /// This interface is used along with RepostControl and IRepostAS
  /// to partially implement the logic of the repost action.
  /// Those two classes are used by the Editor Framework to help implementing
  /// the repost mechanism on any Editor page.
  /// They can also be used by any other Activity Space.
  /// See instructions in RepostControl.java on how to use that
  /// logic outside the Editor Framework.
  /// </summary>
  /// <author>Julie Fournier</author>
  public interface IRepostModel : IModel
  {
    
    /// <summary>
    /// Implement some code to execute when the user clicks on "Finish".
    /// </summary>
    void Finish();
    
    /// <summary>
    /// Returns the Redirect object
    /// (this is were we need to go at the end of the Editor).
    /// </summary>
    /// <returns>Redirect</returns>
    Redirect GetRedirectAfterFinish();
    
    /// <summary>
    /// Calls the right save function according to the page name passed in argument.
    /// </summary>
    /// <param name="_sPageName"></param>
    /// <param name="_htFormData"></param>
    int SavePage(String _sPageName, XPHashtable _htFormData);
  }
}