using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlelements;
using com.plumtree.xpshared.htmlconstructs;
namespace com.plumtree.uiinfrastructure.form
{
  /// <summary>
  /// This is a com.plumtree.uiinfrastructure.activityspace.ILink
  /// that works within an AFormAS.  When clicked, the link will repost the
  /// form, then run the specified control with the provided args.
  /// Currently just a link within a single page; does not support setting
  /// an alternate space or display page.
  /// </summary>
  /// <author>CraigS</author>
  public class PassthruURL : ILinkElement
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.form.PassthruURL");
    private HTMLElementCollection _internals = new HTMLElementCollection();
    private String _control;
    private XPStringBuilder _args = new XPStringBuilder(200);
    
    /// <param name="strName"></param>
    /// <param name="strValue"></param>
    public virtual void AddControlArgument(String strName, String strValue)
    {
      _args.Append(LinkConstants.AMPERSAND).Append(strName).Append(LinkConstants.EQUALS).Append(XPHttpUtility.UrlEncode(strValue,
       LinkConstants.UTF_8_ENCODING));
    }
    
    /// <summary>
    /// This method allows you to add a control argument to the HTML URL.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.ILink#AddQueryStringPair(String,String[])">This URL encodes the value for you</seealso>
    /// <param name="strName"></param>
    /// <param name="strValues"></param>
    public virtual void AddControlArgument(String strName, String[] strValues)
    {
      if (strValues == null)
      {
        log.Warn("AddControlArgument called with null String[]");
        return;
      }
      for (int i = 0;i < strValues.Length;i++)
      {
        AddControlArgument(strName, strValues[i]);
      }
    }
    
    /// <summary>
    /// This method allows you to add a control argument to the HTML URL.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String,String)"></seealso>
    /// <param name="strName"></param>
    /// <param name="nValue"></param>
    public virtual void AddControlArgument(String strName, int nValue)
    {
      AddEncodedControlArgument(strName, XPConvert.ToString(nValue));
    }
    
    /// <summary>
    /// This method allows you to add a control argument to the HTML URL.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String,String)"></seealso>
    /// <param name="strName"></param>
    /// <param name="booleanValue"></param>
    public virtual void AddControlArgument(String strName, bool bValue)
    {
      AddEncodedControlArgument(strName, XPConvert.ToString(bValue));
    }
    
    /// <summary>
    /// This method allows you to add a control argument to the HTML URL with no value.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String,String)"></seealso>
    /// <param name="strName"></param>
    /// <param name="booleanValue"></param>
    public virtual void AddControlArgument(String strName)
    {
      _args.Append(LinkConstants.AMPERSAND).Append(strName).Append(LinkConstants.EQUALS);
      }
    
    /// <param name="element"></param>
    public virtual void AddInnerHTMLElement(HTMLElement element)
    {
      _internals.AddInnerHTMLElement(element);
    }
    
    /// <param name="string"></param>
    /// <throws>HTMLException</throws>
    public virtual void AddInnerHTMLString(String strHTML)
    {
      this.AddInnerHTMLElement(new HTMLGenericElement(strHTML));
    }
    
    /// <returns></returns>
    public virtual HTMLElement GetURLAsHTMLElement()
    {
      XPStringBuilder sb = new XPStringBuilder(200);
      // TODO: fix this.  This whole class should just be a static method that returns
      // an ASURL or something.  This is totally the wrong way to do this.
      sb.Append(AFormDP.JAVASCRIPT_SUBMIT_TO_CONTROL).Append("(\'").Append(_control).Append("\',\'").Append(_args.ToString()).Append("\'); return false;");
      HTMLAnchor result = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
      result.SetOnClick(sb.ToString());
      try
      {
        result.AddInnerHTMLElement(_internals);
      }
      catch (HTMLException e)
      {
        log.Error(e, "Unable to add elements to pass thru url as HTML element.");
        throw XPException.GetInstance(e);
      }
      return result;
    }
    
    /// <param name="strControl"></param>
    public virtual void SetControl(String strControl)
    {
      _control = strControl;
    }
    
    /// <param name="strPage"></param>
    public virtual void SetDisplayPage(String strPage)
    {
    }
    // not implemented
    
    /// <param name="string"></param>
    public virtual void SetLabelString(String strLabel)
    {
      AddInnerHTMLString(strLabel);
    }
    
    /// <param name="strSpace"></param>
    /// <param name="parent"></param>
    /// <returns></returns>
    public virtual bool SetLinkCreateNewSpace(String strSpace, AActivitySpace parent)
    {
      // not implemented
      return true;
    }
    
    /// <param name="space"></param>
    /// <returns></returns>
    public virtual bool SetLinkGetCachedSpace(AActivitySpace space)
    {
      // not implemented
      return true;
    }
    
    /// <param name="strSpace"></param>
    /// <param name="strSpaceID"></param>
    /// <returns></returns>
    public virtual bool SetSpaceGetCached(String strSpace, String strSpaceID)
    {
      // not implemented
      return true;
    }
    
    /// <param name="strSpace"></param>
    /// <param name="parent"></param>
    /// <returns></returns>
    public virtual bool SetLinkGetSpaceIfCached(String strSpace, AActivitySpace parent)
    {
      // not implemented
      return true;
    }
    
    /// <param name="sSubSpaceKey"></param>
    public virtual void SetSubSpaceKey(String sSubSpaceKey)
    {
    }
    // not implemented
    
    /// <summary>
    /// This is a helper method to add qs arguments.  This assumes that you
    /// have already URL encoded the values if necessary.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.ILink#AddQueryStringPair(String,String)"></seealso>
    /// <param name="strName"></param>
    /// <param name="strValue"></param>
    private void AddEncodedControlArgument(String strName, String strValue)
    {
      _args.Append(LinkConstants.AMPERSAND).Append(strName).Append(LinkConstants.EQUALS).Append(strValue);
      }
    public virtual bool SetLinkToExternalURL(String url)
    {
      return false;
    }
  }
}