using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.form
{
  /// <summary>
  /// This is a simply pop-up display page.
  /// </summary>
  public abstract class SimplePopUpDP : IDisplayPage
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.form.SimplePopUpDP");
    
    /// <summary>
    /// HTML Object: view button 
    /// </summary>
    public const String HTMLBUTTON_CLOSE = "in_bu_Close";
    
    /// <returns></returns>
    public abstract String GetTitle();
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public abstract String GetName();
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.AFormDP#PageDisplay()"></seealso>
    protected abstract HTMLElement PageDisplay();
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.AFormDP#DisplayJavaScriptFromChild()"></seealso>
    protected abstract HTMLScriptCollection DisplayJavaScriptFromChild();
    protected AActivitySpace m_asOwner;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Display()">
    /// Warning: PlumtreeDP overrides this, so if you change things here, you probably
    /// need to change it there as well.
    /// </seealso>
    public virtual void Display(IWebData pageData)
    {
      // Calculate whether or not to display javascript.
      bool bNoJavascript = !(GetUseJavascript());
      HTMLPage myPage = new HTMLPage();
      // Set the HTMLPage as the data to be sent to the browser.
      pageData.SetData(myPage);
      if (bNoJavascript)
      {
        myPage.SetDo508Checking(true);
      }
      HTMLHead myHead = new HTMLHead();
      myPage.SetHead(myHead);
      // GetFormTitle should already be encoded.
      HTMLTitle myTitle = new HTMLTitle(GetTitle());
      HTMLBody myBody = new HTMLBody();
      myPage.SetBody(myBody);
      myBody.SetMarginHeight("0");
      myBody.SetMarginWidth("0");
      myBody.SetLeftMargin("0");
      myBody.SetTopMargin("0");
      try
      {
        myHead.AddInnerHTMLElement(myTitle);
        myHead.AddInnerHTMLElement(GetOwner().GetStyleSheetLinks());
        // Add javascript if we are in standard access style (i.e. not 508)
        if (!bNoJavascript)
        {
          myHead.AddInnerHTMLElement(DisplayJavaScriptFromChild());
        }
        // JF- Create the table for the sub banner (description text)
        HTMLTable myTable = new HTMLTable();
        myBody.AddInnerHTMLElement(myTable);
        myTable.SetBorder("0");
        myTable.SetWidth("100%");
        myTable.SetCellPadding("5");
        myTable.SetCellSpacing("0");
        HTMLTableRow myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.DIR_HEADER_BG);
        // display sub header message
        HTMLTableCell myCell = new HTMLTableCell();
        myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
        myRow.AddInnerHTMLElement(myCell);
        HTMLSpan mySpan = new HTMLSpan();
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.SetStyleClass(PTStyleClass.DIR_HEADER);
        mySpan.AddInnerHTMLString(GetTitle());
        myCell = new HTMLTableCell();
        myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetAlign(CommonHTMLStrings.RIGHT);
        // JF- Close button
        HTMLInput myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_CLOSE,
         "");
        myCell.AddInnerHTMLElement(myInput);
        myInput.SetValue(m_asOwner.GetString(1958, "ptmsgs_portaladminmsgs"));
        myInput.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
        myInput.SetOnClick("window.close(); return false;");
        myBody.AddInnerHTMLElement(PageDisplay());
      }
      catch (HTMLException he)
      {
        log.Error(he, "Display: The following HTML error occured when trying to display the HTML:");
        throw new XPException(he);
      }
      catch (Exception e)
      {
        log.Error(e, "Display: The following error occured when trying to display the HTML:");
        throw XPException.GetInstance(e);
      }
    }
    
    /// <summary>
    /// This method determines whether to use Javascript.  On most pages,
    /// only the STANDARD portal uses JS.  However, on portal, community, and
    /// the default profile pages, the low-bandwidth portal can display JS.  In these
    /// cases, this method is overridden to allow JS for Standard and LB (which is the
    /// same as NOT 5_08).
    /// </summary>
    /// <returns>boolean</returns>
    public virtual bool GetUseJavascript()
    {
      return (GetOwner().GetAccessStyle() == AccessStyles.STANDARD);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#GetOwner()"></seealso>
    public virtual AActivitySpace GetOwner()
    {
      return m_asOwner;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Init(AActivitySpace)"></seealso>
    public virtual void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
    }
  }
}