using System;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.uiinfrastructure.form.pagination
{
  /// <summary>
  /// This interface provides the information necessary to
  /// create the pagination bar.
  /// </summary>
  /// <author>Don Hayler</author>
  public interface IPaginationModelRO : IModelRO
  {
    
    /// <summary>
    /// This method returns the current lower count for which objects to display.
    /// This number is the 'n' in (Displaying n - m of o)
    /// </summary>
    /// <returns>int the current lower count for which objects to display.</returns>
    int GetCurrentLowerObjectCount();
    
    /// <summary>
    /// This method returns the current upper count for which objects to display.
    /// This number is the 'm' in (Displaying n - m of o)
    /// </summary>
    /// <returns>int the current upper count for which objects to display.</returns>
    int GetCurrentUpperObjectCount();
    
    /// <summary>
    /// This method returns the total number of objects available for display.
    /// This number is the 'o' in (Displaying n - m of o)
    /// </summary>
    /// <returns>int the total number of objects available for display.</returns>
    int GetNumObjects();
    
    /// <summary>
    /// This is the number of objects displayed per page.
    /// This name is the value of (m - n)
    /// </summary>
    /// <returns>int the total number of objects available for display per page</returns>
    int GetPaginationSize();
  }
}