using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.arraywrapper;
namespace com.plumtree.uiinfrastructure.form.sort
{
  /// <summary>
  /// This control sets the sort to use.
  /// </summary>
  /// <author>Don Hayler</author>
  public class SortControl : IControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.form.sort.SortControl");
    
    /// <summary>
    /// Object Name for AS 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "SortControl";
    
    /// <summary>
    /// The index of the header to sort on 
    /// </summary>
    public const String HTMLINPUT_SORT_ID = "in_hi_SortID";
    
    /// <summary>
    /// The index of the header to sort on 
    /// </summary>
    public const String HTMLINPUT_SORT_HEADER = "in_hi_SortHeader";
    
    /// <summary>
    /// the sort direction
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.arraywrapper.PTSortedArrayWrapper"></seealso>
    public const String HTMLINPUT_SORT_DIRECTION = "in_hi_SortDirection";
    private ISortModel m_asModel;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IControl#CheckActionSecurityAndExecute(XPHashtable)"></seealso>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      int nSortID;
      int nSortHeader;
      int nSortDirection;
      String[] sValues = (String[]) arguments.GetElement(HTMLINPUT_SORT_ID);
      if (sValues != null)
      {
        nSortID = XPConvert.ToInteger(sValues[0]);
      } else
      {
        log.Error("Unable to find sort ID.  Using default.");
        nSortID = 1;
      }
      sValues = (String[]) arguments.GetElement(HTMLINPUT_SORT_HEADER);
      if (sValues != null)
      {
        nSortHeader = XPConvert.ToInteger(sValues[0]);
      } else
      {
        log.Error("Unable to find sort header index.  Using default.");
        nSortHeader = 1;
      }
      sValues = (String[]) arguments.GetElement(HTMLINPUT_SORT_DIRECTION);
      if (sValues != null)
      {
        nSortDirection = XPConvert.ToInteger(sValues[0]);
        if ((nSortDirection != PTSortedArrayWrapper.PTSAW_SORT_ASCENDING) && (nSortDirection != PTSortedArrayWrapper.PTSAW_SORT_DESCENDING))
        {
          log.Error("Sort direction neither ascending nor descending, using default: " + nSortDirection);
          nSortDirection = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
        }
      } else
      {
        log.Error("Unable to find sort direction.  Using default.");
        nSortDirection = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
      }
      m_asModel.SetSort(nSortID, nSortHeader, nSortDirection);
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new SortControl();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IControl#Init(IModel,AActivitySpace)"></seealso>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      try
      {
        m_asModel = (ISortModel) model;
      }
      catch (Exception e)
      {
        log.Error(e, "Initialized with wrong type of model.  Expecting ISortModel.");
        }
    }
  }
}