using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.form.status
{
  /// <summary>
  /// This class contains the HTML and Javascript for placing a status
  /// bar into your HTML.  While it implements IView, this class does 
  /// not need to be loaded into an AS in the traditional manner. 
  /// Plenty of static methods allow you to insert this code into your
  /// existing DPs and Views as an optional/external component.
  /// </summary>
  /// <author>RickyW</author>
  public class StatusView : IView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.form.status.StatusView");
    public const String STR_MVC_CLASS_NAME = "StatusView";
    private const String ELEMID_TABLE = "StatusViewTable";
    private const String ELEMID_CELL = "StatusViewCell";
    private const String FILENAME_JS_STATUSPAGE = "ptstatuspage.js";
    private const String JS_METHOD_SHOW_IN_BODY = "PTStatusPage.replaceBodyWithStatusAndShow";
    protected AActivitySpace m_asOwner;
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IView#Init(com.plumtree.uiinfrastructure.activityspace.IModelRO, com.plumtree.uiinfrastructure.activityspace.AActivitySpace)
     */
     
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      m_asOwner = parent;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
     */
     
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
     */
     
    public virtual Object Create()
    {
      return new StatusView();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
     */
     
    public virtual HTMLElement Display()
    {
      String strLangID = (String) PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE,
       m_asOwner);
      return StatusView.GetDisplay(strLangID);
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
     */
     
    public virtual HTMLScript DisplayJavascript()
    {
      return StatusView.GetDisplayJavascript(m_asOwner.GetIsSecuredSpace());
      }
    
    /// <summary>
    /// Alternative way to get the display from this view 
    /// without the need to create an instance of it.
    /// </summary>
    /// <returns>This method returns the HTMLElement for the view.</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Display()"></seealso>
    public static HTMLElement GetDisplay(String strLang)
    {
      HTMLElementCollection result;
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLImg myImage;
      HTMLSpan mySpan;
      // new collection.
      result = new HTMLElementCollection();
      try
      {
        // SPAN - Main span.
        mySpan = new HTMLSpan();
        mySpan.SetStyleClass(PTStyleClass.DIALOGUE_BG);
        result.AddInnerHTMLElement(mySpan);
        // TABLE - Main table.
        myTable = new HTMLTable();
        myTable.SetID(StatusView.ELEMID_TABLE);
        myTable.SetBorder("0");
        myTable.SetWidth("100%");
        myTable.SetHeight("90%");
        myTable.SetCellPadding("0");
        myTable.SetCellSpacing("10");
        myTable.SetStyle("display:none");
        mySpan.AddInnerHTMLElement(myTable);
        // ROW - Main row.
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myCell.SetID(StatusView.ELEMID_CELL);
        myCell.SetAlign("center");
        myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
        myRow.AddInnerHTMLElement(myCell);
        // CELL 1 - Status Text.
        myCell.AddInnerHTMLString(ConfigHelper.GetResourceManager().GetString(157,
         "ptmsgs_infrastructure", strLang));
        myCell.AddInnerHTMLElement(new HTMLBr());
        myCell.AddInnerHTMLElement(new HTMLBr());
        // CELL 2 - Status Image.
        PTConstruct myConstruct = UIConstructFactory.GetConstruct(PTImageType.ICON_STATUS_BAR);
        myImage = (HTMLImg) myConstruct.GetHTMLElement();
        myImage.SetAlt(ConfigHelper.GetResourceManager().GetString(158, "ptmsgs_infrastructure",
         strLang));
        myCell.AddInnerHTMLElement(myImage);
      }
      catch (Exception ex)
      {
        log.Error(ex, "Error displaying the HTML for this view.");
        throw new XPException("Error displaying the HTML for this view.",
         ex);
      }
      return result;
    }
    
    /// <summary>
    /// Alternative way to get the Javascript from this view 
    /// without the need to create an instance of it.
    /// </summary>
    /// <returns>This method returns the HTMLScript for the view.</returns>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    public static HTMLScript GetDisplayJavascript(bool bIsSecureSpace)
    {
      HTMLScript myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      myScript.SetSrc(ConfigHelper.GetPortalImageServerJavaScriptURL(bIsSecureSpace) + StatusView.FILENAME_JS_STATUSPAGE);
      return myScript;
    }
    
    /// <summary>
    /// Gets the Javascript method that shows the status bar in place of the body.
    /// </summary>
    /// <returns>The Javascript method that shows the status bar in place of the body.</returns>
    public static String GetJSShowStatusInBody()
    {
      return StatusView.JS_METHOD_SHOW_IN_BODY + "(\'" + StatusView.ELEMID_TABLE + "\');\n";
      }
    
    /// <summary>
    /// Returns the method from GetJSShowStatusInBody but additionally wraps it in a
    /// check that makes sure the the repost argument is one of the values in the 
    /// repost values.
    /// </summary>
    /// <param name="strRepostArgument">The argument that should contain your value.</param>
    /// <param name="aRepostValues">The values to check your argument against.</param>
    /// <param name="strSpacing">Optional spacing for formatting purposes.</param>
    /// <returns>Check-Wrapped Javascript method that shows the status bar in place of the body.</returns>
    public static String GetJSShowStatusInBodyRestricted(String strRepostArgument,
     int[] aRepostValues, String strSpacing)
    {
      if (null == aRepostValues)
      {
        return strSpacing + GetJSShowStatusInBody();
      } else
      {
        if (null == strSpacing)
        {
          strSpacing = "";
        }
        XPStringBuilder sb = new XPStringBuilder(300);
        sb.Append(strSpacing).Append("if (");
        for (int ii = 0;ii < aRepostValues.Length;ii++)
        {
          sb.Append("(").Append(strRepostArgument).Append(" == ").Append(aRepostValues[ii]).Append(") || ");
          }
        sb.Append("false)\n");
        sb.Append(strSpacing).Append("{\n");
        sb.Append(strSpacing).Append(strSpacing).Append(StatusView.GetJSShowStatusInBody());
        sb.Append(strSpacing).Append("}\n");
        return sb.ToString();
      }
    }
  }
}