using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.help
{
  /// <author>
  /// donh
  /// This Display Page provides a frame with a "back" navigation
  /// link for 508 users.
  /// </author>
  public class HelpBackFrame : IDisplayPage
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.help.HelpBackFrame");
    
    /// <summary>
    /// MVC object name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HelpBackFrame";
    protected HelpFrameAS m_asOwner;
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Display(com.plumtree.uiinfrastructure.activityspace.IWebData)
     */
     
    public virtual void Display(IWebData webData)
    {
      HTMLPage myPage = new HTMLPage();
      // Set the HTMLPage as the data to be sent to the browser.
      webData.SetData(myPage);
      myPage.SetDo508Checking(true);
      HTMLHead myHead = new HTMLHead();
      myPage.SetHead(myHead);
      String strGoBack = m_asOwner.GetString(151, "ptmsgs_infrastructure");
      HTMLTitle myTitle = new HTMLTitle(strGoBack);
      HTMLBody myBody = new HTMLBody();
      myPage.SetBody(myBody);
      myBody.SetMarginHeight("0");
      myBody.SetMarginWidth("0");
      myBody.SetLeftMargin("0");
      myBody.SetTopMargin("0");
      try
      {
        myHead.AddInnerHTMLElement(myTitle);
        myHead.AddInnerHTMLElement(GetOwner().GetStyleSheetLinks());
        // Body
        HTMLTable helpTable = new HTMLTable();
        myBody.AddInnerHTMLElement(helpTable);
        HTMLTableRow helpRow = new HTMLTableRow();
        helpTable.AddInnerHTMLElement(helpRow);
        helpRow.SetAlign(CommonHTMLStrings.CENTER);
        HTMLTableCell helpCell = new HTMLTableCell();
        helpRow.AddInnerHTMLElement(helpCell);
        // TODO: This should reference a constant, but we don't have those
        // constants in uiinfrastructure yet.
        helpCell.SetStyleClass(new StyleClass("navSidebarText"));
        HTMLAnchor help = new HTMLAnchor(CommonHTMLStrings.POUND_SYMBOL);
        if (null == m_asOwner.m_strPreviousRequestURL)
        {
          // If we don't know the previous URL, just go to the base URL.
          m_asOwner.m_strPreviousRequestURL = ConfigHelper.GetApplicationBaseURL(m_asOwner);
          }
        help.SetHRef(m_asOwner.m_strPreviousRequestURL);
        help.AddInnerHTMLString(strGoBack);
        help.SetTarget("_parent");
        help.SetAccessKey(m_asOwner.GetString(3, ResourceConstants.PORTAL508QUICKKEYMSGS));
        help.SetTitle(m_asOwner.GetString(47, ResourceConstants.PORTAL508QUICKKEYMSGS));
        helpCell.AddInnerHTMLElement(help);
      }
      catch (HTMLException he)
      {
        log.Error(he, "Display: The following HTML error occured when trying to display the HTML:");
        throw new XPException(he);
      }
      catch (Exception e)
      {
        log.Error(e, "Display: The following error occured when trying to display the HTML:");
        throw XPException.GetInstance(e);
      }
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IDisplayPage#GetOwner()
     */
     
    public virtual AActivitySpace GetOwner()
    {
      return m_asOwner;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Init(com.plumtree.uiinfrastructure.activityspace.AActivitySpace)
     */
     
    public virtual void Init(AActivitySpace parent)
    {
      m_asOwner = (HelpFrameAS) parent;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
     */
     
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
  }
}