using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.helpers
{
  /// <summary>
  /// A service that provides access to the config helper object.
  /// </summary>
  public class ConfigService : IConfigHelper
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.helpers.ConfigService");
    //the singleton instance
    private static IConfigHelper m_HelperInstance;
    // This stores whether or not the factory is in test mode.
    private static bool m_bIsProduction = false;
    // Protected constructor to ensure controlled access to the singleton.
    protected ConfigService()
    {
    }
    
    /// <returns>The singleton config helper.</returns>
    public static IConfigHelper getInstance()
    {
      if (m_HelperInstance == null)
      {
        m_bIsProduction = true;
        m_HelperInstance = new ConfigService();
      }
      return m_HelperInstance;
    }
    
    /// <summary>
    /// Unit tests should use this method to overwrite/teardown existing helper singleton.
    /// DO NOT USE for production code. 
    /// NOTE: When finished with a unit test, this should be set back to null so as not
    /// to interfere with other tests.
    /// </summary>
    public static void setTestInstance(IConfigHelper testHelper)
    {
      if (m_bIsProduction)
      {
        String strMessage = "Can\'t override ConfigHelper instance for testing... ConfigService has already been initialized.";
        log.Error(strMessage);
        throw new XPException(strMessage);
      }
      m_HelperInstance = testHelper;
    }
    public virtual String GetJSIncluderComponentPath(String strJSComponentName,
     String strImageServerURL)
    {
      return ConfigHelper.GetJSIncluderComponentPath(strJSComponentName, strImageServerURL);
      }
    public virtual HTMLElementCollection GetJSIncluderJSComponentInclude(String strJSComponentName,
     AActivitySpace owner)
    {
      return ConfigHelper.GetJSIncluderJSComponentInclude(strJSComponentName,
       owner);
    }
    public virtual HTMLElementCollection GetJSIncluderJSComponentInclude(String strJSComponentName,
     bool bEnableDebug, AActivitySpace owner, XPLocale locale)
    {
      return ConfigHelper.GetJSIncluderJSComponentInclude(strJSComponentName,
       bEnableDebug, owner, locale);
    }
    public virtual String GetJSIncluderVersionPrefix(String strJSComponentName)
    {
      return ConfigHelper.GetJSIncluderVersionPrefix(strJSComponentName);
      }
    public virtual String GetApplicationBaseURL(String _sNormalizedBaseURL,
     bool _bSecure)
    {
      return ConfigHelper.GetApplicationBaseURL(_sNormalizedBaseURL, _bSecure);
      }
    public virtual String GetApplicationBaseURL(AActivitySpace asSpace)
    {
      return ConfigHelper.GetApplicationBaseURL(asSpace);
    }
    public virtual String GetApplicationBaseURL(String _sNormalizedBaseURL)
    {
      return ConfigHelper.GetApplicationBaseURL(_sNormalizedBaseURL);
    }
    public virtual String GetApplicationBaseURLToAS(String _sNormalizedBaseURL,
     String _sASName, bool _bIsSubSpace)
    {
      return ConfigHelper.GetApplicationBaseURLToAS(_sNormalizedBaseURL, _sASName,
       _bIsSubSpace);
    }
    public virtual String GetHTTPEntryPoint()
    {
      return ConfigHelper.GetHTTPEntryPoint();
    }
    public virtual String GetMainServerApplicationURL(bool _bIsSecure)
    {
      return ConfigHelper.GetMainServerApplicationURL(_bIsSecure);
    }
    public virtual String GetImageServerBaseURL(AActivitySpace asSpace)
    {
      return ConfigHelper.GetImageServerBaseURL(asSpace);
    }
    public virtual String GetNormalizedBaseURL(String _sURL)
    {
      return ConfigHelper.GetNormalizedBaseURL(_sURL);
    }
  }
}