using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.helpers
{
  public interface IConfigHelper
  {
    
    /// <summary>
    /// JF- Returns the Application URL according to the URL Mapping and the security.
    /// Warning: depending on the security mode, the security in argument might not make sense.
    /// E.g.: in mode 0, it doesn't make sense to request the secure URL
    /// (and it might just return an empty URL)
    /// The normalized base URL is available either cached on the AActivitySpace or by calling
    /// ConfigHelper.GetNormalizedBaseURL with the current request URL.
    /// </summary>
    /// <param name="_sNormalizedBaseURL">the normalized base URL from the current request.</param>
    /// <param name="_bSecure">HTTP or HTTPS</param>
    /// <returns>String</returns>
    String GetApplicationBaseURL(String _sNormalizedBaseURL, bool _bSecure);
    
    /// <summary>
    /// JF- Returns the Application URL according to the URL Mapping.
    /// Warning: In mode 1, it returns the non secure URL by default.
    /// </summary>
    /// <param name="AActivitySpace">the current activity space.</param>
    /// <returns>String</returns>
    String GetApplicationBaseURL(AActivitySpace asSpace);
    
    /// <summary>
    /// JF- Returns the Application URL according to the URL Mapping.
    /// Warning: In mode 1, it returns the non secure URL by default.
    /// GetApplicationBaseURL(AActivitySpace) should be used if possible.
    /// </summary>
    /// <param name="_sNormalizedBaseURL">
    /// Normalized Base URL from the AActivity Space
    /// (or ConfigHelper.GetNormalizedBaseURL).
    /// </param>
    /// <returns>String</returns>
    String GetApplicationBaseURL(String _sNormalizedBaseURL);
    
    /// <summary>
    /// JF- Returns the Application URL according to the URL Mapping and
    /// determines the security according to the space name in argument.
    /// </summary>
    /// <param name="_sNormalizedBaseURL">
    /// Normalized Base URL from the AActivity Space.
    /// (or ConfigHelper.GetNormalizedBaseURL).
    /// </param>
    /// <param name="_sASName">Name of the destination Activity Space</param>
    /// <param name="_bIsSubSpace">Is the target AS a sub space?</param>
    /// <returns>String</returns>
    String GetApplicationBaseURLToAS(String _sNormalizedBaseURL, String _sASName,
     bool _bIsSubSpace);
    
    /// <summary>
    /// JF- Returns the Main server URL from x_config.xml
    /// </summary>
    /// <param name="_bIsSecure">HTTP or HTTPS</param>
    /// <returns>String</returns>
    String GetMainServerApplicationURL(bool _bIsSecure);
    
    /// <summary>
    /// JF- Returns the value of the HTTP Entry point from x_config.xml 
    /// E.g.: server.pt
    /// </summary>
    /// <returns>String</returns>
    String GetHTTPEntryPoint();
    
    /// <param name="strJSComponentName">JSComponent name included through JSIncluder Example: " jsxml" or "jsutil"</param>
    /// <returns></returns>
    String GetJSIncluderVersionPrefix(String strJSComponentName);
    
    /// <param name="strJSComponentName"></param>
    /// <param name="strImageServerURL"></param>
    /// <returns>the path to a particular JS component</returns>
    String GetJSIncluderComponentPath(String strJSComponentName, String strImageServerURL);
    
    /// <summary>
    /// HTMLElement with a script blocks including the javascript files for the given jscomponent. 
    /// </summary>
    /// <param name="strJSComponentName">JSComponent name. For example "jsxml" or "jsutil", the common components have constants defined in ConfigHelper.</param>
    /// <param name="owner">Current activityspace</param>
    /// <returns></returns>
    HTMLElementCollection GetJSIncluderJSComponentInclude(String strJSComponentName,
     AActivitySpace owner);
    
    /// <summary>
    /// HTMLElement with a script blocks including the javascript files for the given jscomponent. 
    /// </summary>
    /// <param name="strJSComponentName">JSComponent name. For example "jsxml" or "jsutil", the common components have constants defined in ConfigHelper.</param>
    /// <param name="owner">Current activityspace</param>
    /// <returns></returns>
    HTMLElementCollection GetJSIncluderJSComponentInclude(String strJSComponentName,
     bool bEnableDebug, AActivitySpace owner, XPLocale locale);
    
    /// <summary>
    /// This method used to return the Portal Image Server Base URL.
    /// </summary>
    /// <param name="AActivitySpace">the space that this image is in.</param>
    /// <returns>String the Portal image server base URL.</returns>
    String GetImageServerBaseURL(AActivitySpace asSpace);
    
    /// <summary>
    /// DLH- Helper method to return the normalized base URL for use
    /// in determining the correct application base url.
    /// </summary>
    /// <param name="_sURL"></param>
    /// <returns>String the normalized base URL</returns>
    String GetNormalizedBaseURL(String _sURL);
  }
}