using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.htmlconstructs
{
  /// <author>
  /// Ben Geislinger
  /// Use this class to obtain PTConstruct objects.
  /// <br /><br />
  /// Objects of this class cannot be instantiated.  This class only provides a
  /// static method for obtaining instances of PTConstruct objects.
  /// </author>
  public class UIConstructFactory
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.htmlconstructs.UIConstructFactory");
    
    /// <summary>
    /// Creates a new UIConstructFactory object.  No need for anyone to ever call
    /// this method.
    /// </summary>
    private UIConstructFactory()
    {
    }
    
    /// <summary>
    /// Call this method to obtain a PTConstruct.
    /// <br /><br />
    /// </summary>
    /// <param name="type">
    /// Pass the type of construct you wish to obtain.  Construct
    /// types are available as static members of the various PTConstructType
    /// classes.
    /// </param>
    public static PTConstruct GetConstruct(PTConstructType type)
    {
      log.Debug("Entering GetConstruct - type: " + type);
      //	no point in trying to update the strings if we
      //	dont have an activity space
      return GetConstruct(type, false, null);
    }
    
    /// <summary>
    /// Call this method to obtain a PTConstruct.
    /// <br /><br />
    /// </summary>
    /// <param name="type">
    /// Pass the type of construct you wish to obtain.  Construct
    /// types are available as static members of the various PTConstructType
    /// classes.
    /// <br /><br />
    /// </param>
    /// <param name="bUpdateStrings">
    /// Pass a boolean value indicating whether or not
    /// you wish for the strings of this banner to be updated automatically.
    /// If you pass true, all appropriate strings will be replaced with
    /// localized versions.  If you pass false, all strings will be left with
    /// they prototypical values.  You can update the strings yourself, using
    /// the string dictionary returned by the PTBannerType.GetStringDictionary
    /// method.
    /// </param>
    public static PTConstruct GetConstruct(PTConstructType type, bool bUpdateStrings,
     AActivitySpace activitySpace)
    {
      log.Debug("GetConstruct - type: " + type);
      log.Debug("GetConstruct() - bUpdateStrings: " + XPConvert.ToString(bUpdateStrings));
      HTMLElement e = null;
      HTMLElement eClone = null;
      PTConstruct construct = null;
      //   Get the prototypical element.
      try
      {
        e = type.GetHTMLElement();
      }
      catch (Exception ex)
      {
      }
      if (e == null)
      {
        log.Error("GetConstruct() - Could not obtain the requested construct: " + type);
        } else
      {
        //   Clone the prototype, so the client obtains a fresh table.
        try
        {
          log.Debug("GetConstruct() - About to call GetClone().");
          eClone = (HTMLElement) e.GetClone();
          log.Debug("GetConstruct() - eClone: " + eClone);
        }
        catch (Exception ex)
        {
          // todo
          log.Error(ex, "GetConstruct() - The element could not be cloned.");
          throw new XPException("The element could not be cloned.", ex);
        }
        construct = new PTConstruct(type, eClone);
        log.Debug("GetConstruct() - construct: " + construct);
        if (bUpdateStrings)
        {
          //   If we're supposed to, update the strings.
          UIFactoryHelper.updateStrings(construct, activitySpace);
          UIFactoryHelper.updateImgAlts(construct, activitySpace);
        }
      }
      //    e is not null;
      return construct;
    }
  }
}