/*
 * Created on May 18, 2006
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openkernel.config;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.application;
namespace com.plumtree.uiinfrastructure.httpcompression
{
  public class CompressionSettings
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.httpcompression.CompressionSettings");
    public const String HTTPCOMPRESSION_CONTEXT = "httpCompression";
    // DCA - browser buckets for configuring compression settings
    public const String IE5_CONTEXT = "IE5";
    public const String IE5_MAC_CONTEXT = "IE5-Mac";
    public const String IE6_CONTEXT = "IE6";
    public const String GECKO_CONTEXT = "Gecko";
    public const String SAFARI_CONTEXT = "Safari";
    public const String OPERA5PLUS_CONTEXT = "Opera5+";
    public const String UNKNOWN_CONTEXT = "Unknown";
    public const String COMPRESSION_TYPE = "http://www.plumtree.com/config/component/types/httpcompression";
    // DCA - content types available for compression
    public static readonly String TEXT_TYPE = CompressionSettings.COMPRESSION_TYPE + "/text";
    public static readonly String IMAGE_TYPE = CompressionSettings.COMPRESSION_TYPE + "/image";
    public static readonly String APP_TYPE = CompressionSettings.COMPRESSION_TYPE + "/application";
    public static readonly String AUDIO_TYPE = CompressionSettings.COMPRESSION_TYPE + "/audio";
    public static readonly String VIDEO_TYPE = CompressionSettings.COMPRESSION_TYPE + "/video";
    public static readonly String MULT_TYPE = CompressionSettings.COMPRESSION_TYPE + "/multipart";
    public static readonly String MESS_TYPE = CompressionSettings.COMPRESSION_TYPE + "/message";
    public static readonly String MODEL_TYPE = CompressionSettings.COMPRESSION_TYPE + "/model";
    public const String ENABLE_COMPRESSION_SETTING = "enableHttpCompression";
    public const String MIN_COMPRESSION_BYTES_SETTING = "minCompressBytes";
    public const String FAVOR_GZIP_SETTING = "favorGZIP";
    public const String COMPRESSION_LEVEL_SETTING = "compressionLevel";
    public const String DEFAULT_COMPRESSION_SETTING = "default";
    public const String FORCE_COMPRESSION_SETTING = "forceCompression";
    public const String DISABLE_COMPRESSION_SETTING = "disableCompression";
    public static readonly CompressionSettings INSTANCE = new CompressionSettings();
    private IOKComponent m_okCompressionComponent;
    // Hashtable version of the httpcompression.xml file
    private XPHashtable m_htBrowserContextsMap;
    private bool m_bInitialized = false;
    private CompressionSettings()
    {
      m_htBrowserContextsMap = new XPHashtable();
    }
    public virtual void Init()
    {
      if (m_bInitialized)
      {
        log.Error("CompressionSettings::Init Init attempted twice.  Ignoring second attempt.");
        return;
      }
      IOKContext okCompresseionContext = null;
      try
      {
        // DCA - get the general compression settings context
        okCompresseionContext = AppWarmUp.GetOKConfigContext().getConfig().getContext(HTTPCOMPRESSION_CONTEXT);
        }
      catch (Exception e)
      {
        // compression xml was not found
        log.Warn("CompressionSettings::Init httpcompression.xml was not found. Compression will be disabled.",
         e);
        return;
      }
      m_okCompressionComponent = okCompresseionContext.getComponentByType(COMPRESSION_TYPE);
      if (IsCompressionEnabled())
      {
        // DCA - load each browser group
        LoadBrowserContext(IE5_CONTEXT);
        LoadBrowserContext(IE5_MAC_CONTEXT);
        LoadBrowserContext(IE6_CONTEXT);
        LoadBrowserContext(GECKO_CONTEXT);
        LoadBrowserContext(SAFARI_CONTEXT);
        LoadBrowserContext(OPERA5PLUS_CONTEXT);
        LoadBrowserContext(UNKNOWN_CONTEXT);
      } else
      {
        log.Debug("CompressionSettings::Init Global Compression has been configured to be disabled. Compression Settings will not be loaded.");
        }
      m_bInitialized = true;
    }
    private void LoadBrowserContext(String strBrowserContextKey)
    {
      log.Debug("CompressionSettings::LoadBrowserContext Loading browser context " + strBrowserContextKey);
      XPHashtable htBrowserContextMap = new XPHashtable();
      m_htBrowserContextsMap.PutElement(strBrowserContextKey, htBrowserContextMap);
      IOKContext okBrowserContext = null;
      try
      {
        okBrowserContext = AppWarmUp.GetOKConfigContext().getConfig().getContext(strBrowserContextKey);
        }
      catch (Exception e)
      {
        // compression xml was not found
        log.Warn("CompressionSettings::LoadBrowserContext Browser context " + strBrowserContextKey + " was not found. Compression for browsers in this context is disabled.",
         e);
        return;
      }
      // DCA - load each content type group for the specified browser group
      LoadCompressionSettings(TEXT_TYPE, htBrowserContextMap, okBrowserContext);
      LoadCompressionSettings(IMAGE_TYPE, htBrowserContextMap, okBrowserContext);
      LoadCompressionSettings(APP_TYPE, htBrowserContextMap, okBrowserContext);
      LoadCompressionSettings(AUDIO_TYPE, htBrowserContextMap, okBrowserContext);
      LoadCompressionSettings(VIDEO_TYPE, htBrowserContextMap, okBrowserContext);
      LoadCompressionSettings(MULT_TYPE, htBrowserContextMap, okBrowserContext);
      LoadCompressionSettings(MESS_TYPE, htBrowserContextMap, okBrowserContext);
      LoadCompressionSettings(MODEL_TYPE, htBrowserContextMap, okBrowserContext);
      }
    private void LoadCompressionSettings(String strContentGroupKey, XPHashtable htBrowserContextMap,
     IOKContext okBrowserContext)
    {
      log.Debug("CompressionSettings::LoadCompressionSettings Loading compression settings " + strContentGroupKey + " for browser context " + okBrowserContext.getName());
      IOKComponent okContentTypeComponent;
      try
      {
        okContentTypeComponent = okBrowserContext.getComponentByType(strContentGroupKey);
        }
      catch (Exception e)
      {
        // component was not found
        log.Warn("CompressionSettings::LoadCompressionSettings Compression settings " + strContentGroupKey + " for browser context " + okBrowserContext.getName() + " was not found. Compression for this group of content type for this browser group is disabled.",
         e);
        htBrowserContextMap.PutElement(strContentGroupKey + DEFAULT_COMPRESSION_SETTING,
         false);
        htBrowserContextMap.PutElement(strContentGroupKey + FORCE_COMPRESSION_SETTING,
         new XPHashtable());
        return;
      }
      // DCA - store the default compression setting for this content type group for this browser
      bool bDefault = okContentTypeComponent.getSettingValue(DEFAULT_COMPRESSION_SETTING,
       false);
      htBrowserContextMap.PutElement(strContentGroupKey + DEFAULT_COMPRESSION_SETTING,
       bDefault);
      String strExceptionSettingKey;
      // DCA - determine what type of exception is in this component
      if (bDefault)
      {
        strExceptionSettingKey = DISABLE_COMPRESSION_SETTING;
      } else
      {
        strExceptionSettingKey = FORCE_COMPRESSION_SETTING;
      }
      XPHashtable htContentTypes = new XPHashtable();
      htBrowserContextMap.PutElement(strContentGroupKey + strExceptionSettingKey,
       htContentTypes);
      // DCA - load exceptions into a hashtable for easy look up.
      String strExceptions = okContentTypeComponent.getSettingValue(strExceptionSettingKey,
       "");
      XPStringTokenizer stExceptionTokens = new XPStringTokenizer(strExceptions,
       ";", false);
      while (stExceptionTokens.HasMoreElements())
      {
        String strContentType = stExceptionTokens.GetNextElement();
        htContentTypes.PutElement(strContentType, strContentType);
      }
    }
    public virtual bool GetDefaultCompressionSetting(String strBrowserGroupKey,
     String strContentGroupKey)
    {
      XPHashtable htBrowserContextMap = (XPHashtable) m_htBrowserContextsMap.GetElement(strBrowserGroupKey);
      if (null == htBrowserContextMap)
      {
        return false;
      }
      bool bDefault = (bool) htBrowserContextMap.GetElement(strContentGroupKey + DEFAULT_COMPRESSION_SETTING);
      return XPConvert.ToBoolean(bDefault);
    }
    public virtual XPHashtable GetForceCompressionTypes(String strBrowserGroupKey,
     String strContentGroupKey)
    {
      XPHashtable htBrowserContextMap = (XPHashtable) m_htBrowserContextsMap.GetElement(strBrowserGroupKey);
      if (null == htBrowserContextMap)
      {
        return new XPHashtable();
      }
      XPHashtable htContentTypes = (XPHashtable) htBrowserContextMap.GetElement(strContentGroupKey + FORCE_COMPRESSION_SETTING);
      return htContentTypes;
    }
    public virtual XPHashtable GetDisableCompressionTypes(String strBrowserGroupKey,
     String strContentGroupKey)
    {
      XPHashtable htBrowserContextMap = (XPHashtable) m_htBrowserContextsMap.GetElement(strBrowserGroupKey);
      if (null == htBrowserContextMap)
      {
        return new XPHashtable();
      }
      XPHashtable htContentTypes = (XPHashtable) htBrowserContextMap.GetElement(strContentGroupKey + DISABLE_COMPRESSION_SETTING);
      return htContentTypes;
    }
    public virtual bool IsCompressionEnabled()
    {
      if (null == m_okCompressionComponent)
      {
        return false;
      }
      bool val = m_okCompressionComponent.getSettingValue(ENABLE_COMPRESSION_SETTING,
       false);
      return val;
    }
    public virtual int GetMinCompressionBytes()
    {
      if (null == m_okCompressionComponent)
      {
        return 512;
      }
      int val = m_okCompressionComponent.getSettingValue(MIN_COMPRESSION_BYTES_SETTING,
       512);
      return val;
    }
    public virtual bool IsGZIPFavored()
    {
      if (null == m_okCompressionComponent)
      {
        return false;
      }
      bool val = m_okCompressionComponent.getSettingValue(FAVOR_GZIP_SETTING,
       false);
      return val;
    }
    public virtual int GetCompressionLevel()
    {
      if (null == m_okCompressionComponent)
      {
        return 5;
      }
      int val = m_okCompressionComponent.getSettingValue(COMPRESSION_LEVEL_SETTING,
       5);
      return val;
    }
  }
}