/*
 * Created on May 11, 2006
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.util;
namespace com.plumtree.uiinfrastructure.httpcompression
{
  public class HttpCompressionUtil : IHttpCompressionUtil
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.httpcompression.HttpCompressionUtil");
    public const int UNINITIALIZED = -1;
    public const int NO_COMPRESSION = 0;
    // compression types
    // deflate compression with gzip package and CRC (RFC 1952)
    public const int GZIP = 1;
    // deflate compression with no packaging (RFC 1951)
    public const int DEFLATE = 2;
    // deflate compression with zlib packaging (RFC 1950)
    public const int DEFLATE_WRAPPED = 3;
    public const String GZIP_STR = "gzip";
    public const String DEFLATE_STR = "deflate";
    private int m_nCompressionType;
    private String m_strAcceptEncoding;
    private BrowserType m_btBrowser;
    private String m_strContentType;
    public HttpCompressionUtil(IXPRequest xpRequest, BrowserType browserType)
    {
      m_strAcceptEncoding = xpRequest.GetHeader("accept-encoding");
      m_btBrowser = browserType;
      m_nCompressionType = UNINITIALIZED;
    }
    public virtual void SetContentType(String contentType)
    {
      m_strContentType = contentType;
    }
    
    /// <summary>
    /// Analyzes the request and determines what kind of compression should be used
    /// for this client browser, if any.
    /// </summary>
    private void DetermineCompression()
    {
      // DCA - first check whether the browser accepts compression
      if (m_strAcceptEncoding != null)
      {
        // DCA - determine whether browser can handle compression for content type
        bool bCompressible = IsCompressible();
        String strCompression = "no compression";
        if (XPStringUtility.IndexOf(m_strAcceptEncoding, DEFLATE_STR) != -1)
        {
          // favor deflate over gzip (faster or no CRC).
          if (bCompressible)
          {
            m_nCompressionType = DEFLATE;
            strCompression = DEFLATE_STR;
          }
        }
        if (XPStringUtility.IndexOf(m_strAcceptEncoding, GZIP_STR) != -1)
        {
          if (CompressionSettings.INSTANCE.IsGZIPFavored() || UNINITIALIZED == m_nCompressionType)
          {
            if (bCompressible)
            {
              m_nCompressionType = GZIP;
              strCompression = GZIP_STR;
            }
          }
        }
        log.Debug("HttpCompressionUtil::DetermineCompression Using compression method: " + strCompression);
        } else
      {
        log.Debug("HttpCompressionUtil::DetermineCompression accept-encoding setting was not found in the request header.");
        }
      if (UNINITIALIZED == m_nCompressionType)
      {
        log.Debug("HttpCompressionUtil::DetermineCompression Disabling compression.");
        m_nCompressionType = NO_COMPRESSION;
      }
    }
    
    /// <summary>
    /// Private utility method for browserCompressionType() that inspects the User-Agent string and rejects
    /// unsupported browsers.  Can potentially be driven from a config file. 
    /// Returns true if the browser is supported for compression.
    /// </summary>
    private bool IsCompressible()
    {
      // DCA - get the browser group of the browser
      String strBrowserGroupKey = null;
      if (m_btBrowser.IsIE() && m_btBrowser.GetBrowserVersion() >= 6)
      {
        strBrowserGroupKey = CompressionSettings.IE6_CONTEXT;
      } else
      if (m_btBrowser.IsIE() && m_btBrowser.GetBrowserVersion() >= 5)
      {
        if (m_btBrowser.IsMac())
        {
          strBrowserGroupKey = CompressionSettings.IE5_MAC_CONTEXT;
        } else
        {
          strBrowserGroupKey = CompressionSettings.IE5_CONTEXT;
        }
      } else
      if (m_btBrowser.IsNetscape67() || m_btBrowser.IsMozilla() || m_btBrowser.IsFirefox())
      {
        strBrowserGroupKey = CompressionSettings.GECKO_CONTEXT;
      } else
      if (m_btBrowser.IsOpera() && m_btBrowser.GetBrowserVersion() >= 5)
      {
        strBrowserGroupKey = CompressionSettings.OPERA5PLUS_CONTEXT;
      } else
      if (m_btBrowser.IsSafari())
      {
        strBrowserGroupKey = CompressionSettings.SAFARI_CONTEXT;
      } else
      {
        strBrowserGroupKey = CompressionSettings.UNKNOWN_CONTEXT;
      }
      log.Debug("HttpCompressionUtil::IsCompressible The current browser has been determined to belong to browser group: " + strBrowserGroupKey);
      // DCA - figure out the default setting for the content group
      String strContentGroupKey = null;
      String strContentTypeKey = null;
      int nEnd = XPStringUtility.IndexOf(m_strContentType, ";");
      if (XPStringUtility.IndexOf(m_strContentType, "text/") != -1)
      {
        strContentGroupKey = CompressionSettings.TEXT_TYPE;
        strContentTypeKey = GetContentTypeKey(XPStringUtility.IndexOf(m_strContentType,
         "text/") + 5, nEnd);
      } else
      if (XPStringUtility.IndexOf(m_strContentType, "image/") != -1)
      {
        strContentGroupKey = CompressionSettings.IMAGE_TYPE;
        strContentTypeKey = GetContentTypeKey(XPStringUtility.IndexOf(m_strContentType,
         "image/") + 6, nEnd);
      } else
      if (XPStringUtility.IndexOf(m_strContentType, "audio/") != -1)
      {
        strContentGroupKey = CompressionSettings.AUDIO_TYPE;
        strContentTypeKey = GetContentTypeKey(XPStringUtility.IndexOf(m_strContentType,
         "audio/") + 6, nEnd);
      } else
      if (XPStringUtility.IndexOf(m_strContentType, "video/") != -1)
      {
        strContentGroupKey = CompressionSettings.VIDEO_TYPE;
        strContentTypeKey = GetContentTypeKey(XPStringUtility.IndexOf(m_strContentType,
         "video/") + 6, nEnd);
      } else
      if (XPStringUtility.IndexOf(m_strContentType, "application/") != -1)
      {
        strContentGroupKey = CompressionSettings.APP_TYPE;
        strContentTypeKey = GetContentTypeKey(XPStringUtility.IndexOf(m_strContentType,
         "application/") + 12, nEnd);
      } else
      if (XPStringUtility.IndexOf(m_strContentType, "multipart/") != -1)
      {
        strContentGroupKey = CompressionSettings.MULT_TYPE;
        strContentTypeKey = GetContentTypeKey(XPStringUtility.IndexOf(m_strContentType,
         "multipart/") + 10, nEnd);
      } else
      if (XPStringUtility.IndexOf(m_strContentType, "message/") != -1)
      {
        strContentGroupKey = CompressionSettings.MESS_TYPE;
        strContentTypeKey = GetContentTypeKey(XPStringUtility.IndexOf(m_strContentType,
         "message/") + 8, nEnd);
      } else
      if (XPStringUtility.IndexOf(m_strContentType, "model/") != -1)
      {
        strContentGroupKey = CompressionSettings.MODEL_TYPE;
        strContentTypeKey = GetContentTypeKey(XPStringUtility.IndexOf(m_strContentType,
         "model/") + 6, nEnd);
      } else
      {
        // DCA - we don't know the content type, return false
        return false;
      }
      log.Debug("HttpCompressionUtil::IsCompressible The current content\'s type is " + m_strContentType);
      bool bDefaultSetting = CompressionSettings.INSTANCE.GetDefaultCompressionSetting(strBrowserGroupKey,
       strContentGroupKey);
      log.Debug("HttpCompressionUtil::IsCompressible Default compression setting for content type group is " + XPConvert.ToString(bDefaultSetting));
      if (bDefaultSetting)
      {
        // check for exception to disable
        XPHashtable htContentTypes = CompressionSettings.INSTANCE.GetDisableCompressionTypes(strBrowserGroupKey,
         strContentGroupKey);
        if (htContentTypes.ContainsKey(strContentTypeKey))
        {
          log.Debug("HttpCompressionUtil::IsCompressible An exception was found. Overriding default setting returning false.");
          return false;
        }
      } else
      {
        // check for exception to enable
        XPHashtable htContentTypes = CompressionSettings.INSTANCE.GetForceCompressionTypes(strBrowserGroupKey,
         strContentGroupKey);
        if (htContentTypes.ContainsKey(strContentTypeKey))
        {
          log.Debug("HttpCompressionUtil::IsCompressible An exception was found. Overriding default setting returning true.");
          return true;
        }
      }
      return bDefaultSetting;
    }
    private String GetContentTypeKey(int nStart, int nEnd)
    {
      if (nEnd != -1)
      {
        return XPStringUtility.Substring(m_strContentType, nStart, nEnd);
        } else
      {
        return XPStringUtility.Substring(m_strContentType, nStart);
      }
    }
    public virtual int GetCompressionType()
    {
      return m_nCompressionType;
    }
    public virtual void DisableCompression()
    {
      m_nCompressionType = NO_COMPRESSION;
    }
    public virtual bool IsCompressionDisabled()
    {
      if (m_nCompressionType == UNINITIALIZED)
      {
        DetermineCompression();
      }
      return m_nCompressionType == NO_COMPRESSION;
    }
    public virtual bool IsCompressionDeflate()
    {
      if (m_nCompressionType == UNINITIALIZED)
      {
        DetermineCompression();
      }
      return m_nCompressionType == DEFLATE;
    }
    public virtual bool IsCompressionDeflateWrapped()
    {
      if (m_nCompressionType == UNINITIALIZED)
      {
        DetermineCompression();
      }
      return m_nCompressionType == DEFLATE_WRAPPED;
    }
    public virtual bool IsCompressionGZIP()
    {
      if (m_nCompressionType == UNINITIALIZED)
      {
        DetermineCompression();
      }
      return m_nCompressionType == GZIP;
    }
    public virtual int GetMinCompressionBytes()
    {
      return CompressionSettings.INSTANCE.GetMinCompressionBytes();
    }
    public virtual int GetCompressionLevel()
    {
      return CompressionSettings.INSTANCE.GetCompressionLevel();
    }
  }
}