using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.interpreter
{
  /// <summary>
  /// IMVCController defines the interface of an MVC Controller, which the 
  /// Interpreter will interact to invoke the application logic. The application
  /// logic is typically written based on activityspace-related objects (Control,
  /// Model, View, Redirect, etc). 
  /// </summary>
  /// <author>WilliamA</author>
  public interface IMVCController
  {
    
    /// <summary>
    /// This method invokes the OnPageStart PEI. The PEI will be invoked with the
    /// activity space in the given redirect if any. Otherwise, it will be 
    /// invoked with the activity space in the requestData.
    /// </summary>
    /// <param name="redirect">Redirect to be processed, if any</param>
    /// <param name="requestData">the request</param>
    /// <param name="hostname">the hostname of current portal instance</param>
    void DoOnPageStartActions(Redirect redirect, RequestData requestData,
     String hostname);
    
    /// <summary>
    /// This method performs the core control flow execution of activityspace
    /// related logics. It processes the given redirect (if any) in the context
    /// of the requestData. This method may return a final Redirect object,
    /// which typically is a Redirect containing a DisplayPage object, or
    /// a flag to indicate an HTTP redirect.
    /// </summary>
    /// <param name="redirect">Redirect to be processed, if any</param>
    /// <param name="requestData">the request</param>
    /// <returns>the final Redirect, either a DisplayPage or an HTTP redirect</returns>
    Redirect HandleRedirect(Redirect redirect, RequestData requestData);
    
    /// <summary>
    /// This method invokes the OnPageFinish PEI.
    /// </summary>
    /// <param name="requestData">the request</param>
    /// <param name="hostname">the hostname of current portal instance</param>
    /// <returns>any page action comments</returns>
    HTMLElement DoOnPageFinishActions(RequestData requestData, String hostname);
    }
}