using System;
using XPException=com.plumtree.openfoundation.util.XPException;
namespace com.plumtree.uiinfrastructure.interpreter
{
  /// <summary>
  /// This class is used to convey an error or exception information back to
  /// the Interpreter. Upon receiving this exception, the Interpreter will
  /// display an error page containing a (localized) error message from this 
  /// exception, if there is any. 
  /// </summary>
  /// <author>William Adjie-Winoto</author>
  public class InterpreterException : XPException
  {
    public const int INVALID_ID = -1;
    public const int INVALID_HTTP_STATUS_CODE = -1;
    private int m_nMessageStringID = -1;
    private String m_strResourceFileName = null;
    private String m_strNonLocalizedMessage = null;
    private int m_nHTTPStatusCode = -1;
    // CONSTRUCTORS
    
    /// <summary>
    /// Default Constructor
    /// </summary>
    public InterpreterException()
    {
      m_nMessageStringID = INVALID_ID;
      m_strResourceFileName = null;
      m_strNonLocalizedMessage = null;
      m_nHTTPStatusCode = INVALID_HTTP_STATUS_CODE;
    }
    
    /// <summary>
    /// Contructor.
    /// </summary>
    /// <param name="_nMessageStringID">the message string ID in the resource file</param>
    /// <param name="_strResourceFileName">the name of the resource file</param>
    /// <param name="_strNonLocalizedMessage">the non-localized version of the message</param>
    public InterpreterException(int _nMessageStringID, String _strResourceFileName,
     String _strNonLocalizedMessage)
    {
      m_nMessageStringID = _nMessageStringID;
      m_strResourceFileName = _strResourceFileName;
      m_strNonLocalizedMessage = _strNonLocalizedMessage;
      m_nHTTPStatusCode = INVALID_HTTP_STATUS_CODE;
    }
    
    /// <summary>
    /// Contructor.
    /// </summary>
    /// <param name="_nMessageStringID">the message string ID in the resource file</param>
    /// <param name="_strResourceFileName">the name of the resource file</param>
    /// <param name="_strNonLocalizedMessage">the non-localized version of the message</param>
    /// <param name="_nHttpStatusCode">the HTTP status code for the HTTP response</param>
    public InterpreterException(int _nMessageStringID, String _strResourceFileName,
     String _strNonLocalizedMessage, int _nHttpStatusCode)
    {
      m_nMessageStringID = _nMessageStringID;
      m_strResourceFileName = _strResourceFileName;
      m_strNonLocalizedMessage = _strNonLocalizedMessage;
      m_nHTTPStatusCode = _nHttpStatusCode;
    }
    // METHODS
    public virtual int GetMessageStringID()
    {
      return m_nMessageStringID;
    }
    public virtual void SetMessageStringID(int _nId)
    {
      m_nMessageStringID = _nId;
    }
    public virtual String GetResourceFileName()
    {
      return m_strResourceFileName;
    }
    public virtual void GetResourceFileName(String _strFilename)
    {
      m_strResourceFileName = _strFilename;
    }
    public virtual String GetNonLocalizedMessage()
    {
      return m_strNonLocalizedMessage;
    }
    public virtual void SetNonLocalizedMessage(String _strNonLocalizedMessage)
    {
      m_strNonLocalizedMessage = _strNonLocalizedMessage;
    }
    public virtual int GetHTTPStatusCode()
    {
      return m_nHTTPStatusCode;
    }
    public virtual void SetHTTPStatusCode(int _nStatusCode)
    {
      m_nHTTPStatusCode = _nStatusCode;
    }
  }
}