using System;
using com.plumtree.uiinfrastructure.interpreter;
namespace com.plumtree.uiinfrastructure.interpreter.filter
{
  /// <summary>
  /// This is a pass-through filter; both the PreFilter and PostFilter methods
  /// have the default implementation that returns a boolean false. This is a
  /// utility class to make it easier to implement only one of the filter methods 
  /// (either the PreFilter or the PostFilter) without implementing the other 
  /// (which by default will be a pass through).
  /// </summary>
  /// <author>WilliamA</author>
  public class BaseInterpreterFilter : IInterpreterFilter
  {
    
    /// <summary>
    /// A pass-through PreFilter. It always returns false.
    /// </summary>
    public virtual bool PreFilter(RequestData requestData)
    {
      return false;
    }
    
    /// <summary>
    /// A pass-through PostFilter.  
    /// </summary>
    public virtual void PostFilter(RequestData requestData)
    {
    }
  }
}