using System;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.interpreter.filter.utils;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.interpreter.filter
{
  /// <summary>
  /// A filter to handle gateway request.
  /// </summary>
  /// <author>Daniilk  (Gateway)</author>
  /// <author>WilliamA (Writing this new class)</author>
  public class GatewayFilter : BaseInterpreterFilter
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.interpreter.filter.GatewayFilter");
    
    /// <summary>
    /// This method detects if the incoming request is a gateway request.
    /// If it is, it will take over the processing of the request, and return
    /// a boolean true to indicate that Interpreter shall not process the
    /// request further. 
    /// </summary>
    public override bool PreFilter(RequestData requestData)
    {
      IXPRequest request = requestData.m_xpRequest;
      /*
       * FIRST OF ALL WE NEED TO FIGURE OUT IF THIS IS A GATEWAY REQUEST OR NOT. GATEWAY REQUESTS
       * ARE SPECIAL AND THEY TAKE DIFFERENT CODE PATH.
       * 		-- moose
       */
       
      String strURL = request.GetRequestURL();
      int iQSIndex = XPStringUtility.IndexOf(strURL, "?");
      if (-1 != iQSIndex)
      {
        strURL = XPStringUtility.Substring(strURL, 0, iQSIndex);
      }
      // at this point strURL does not have a query string!!!!
      // Use the gateway handle request if this is a gateway url
      if (IsGatewayURL(strURL))
      {
        log.Debug("Calling HandleGatewayRequest(...) for URL [" + request.GetRequestURL() + "]");
        // If we need to login from the gateway, we let the
        // interpreter handle that.			
        return GatewayHandlers.HandleGatewayRequest(requestData);
      }
      /*
       * ================================================================================================
       * END OF SPECIAL PROCESSING FOR THE GATEWAY REQUESTS
       * ================================================================================================
       */
       
      return false;
    }
    protected virtual bool IsGatewayURL(String url)
    {
      return (-1 != XPStringUtility.IndexOf(url, GatewayHandlers.GATEWAY_UNIQUE));
      }
  }
}