using System;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace.utils;
using com.plumtree.uiinfrastructure.httpcompression;
using com.plumtree.uiinfrastructure.interpreter;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.interpreter.filter
{
  /// <author>
  /// scottc
  /// HttpCompressionFilter impliments the java servlet Filter interface.
  /// It compresses HTTP content using deflate or gzip, detecting browser support,
  /// compressing, and configuring headers appropriately.
  /// </author>
  public class HttpCompressionFilter : BaseInterpreterFilter
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.interpreter.filter.HttpCompressionFilter");
    public override bool PreFilter(RequestData requestData)
    {
      if (CompressionSettings.INSTANCE.IsCompressionEnabled())
      {
        log.Debug("HttpCompressionFilter::PreFilter Global Compression has been configured to be enabled.");
        BrowserType browserType = (BrowserType) requestData.m_smSandbox.GetAttribute(Interpreter.BROWSERTYPE);
        IXPRequest xpRequest = requestData.m_xpRequest;
        IXPResponse xpResponse = requestData.m_xpResponse;
        if (null != browserType)
        {
          // DCA - wrapping XPResponse to use compression
          XPCompressionResponse compressionResponse = new XPCompressionResponse(xpResponse,
           new HttpCompressionUtil(xpRequest, browserType));
          requestData.m_xpResponse = compressionResponse;
          requestData.m_wdPageData.setResponse(requestData.m_xpResponse);
          } else
        {
          log.Warn("HttpCompressionFilter::PreFilter BrowserType class not found, compression will not be used.");
          }
      } else
      {
        log.Info("HttpCompressionFilter::PreFilter Global Compression has been configured to be disabled.");
        }
      return false;
    }
  }
}