using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.web;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.login
{
  /// <summary>
  /// This class is used to access some of the application specific data that would
  /// normally be available through the AActivitySpace.
  /// </summary>
  public class ApplicationData
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.login.ApplicationData");
    private IXPRequest m_Request;
    private IWebData m_WebData;
    private IApplication m_Application;
    private ISessionManager m_SubSession;
    private ISessionManager m_PersistentSubSession;
    public ApplicationData(IXPRequest _Request, IWebData _WebData, IApplication _Application,
     ISessionManager _SubSession, ISessionManager _PersistentSubSession)
    {
      m_Request = _Request;
      m_WebData = _WebData;
      m_Application = _Application;
      m_SubSession = _SubSession;
      m_PersistentSubSession = _PersistentSubSession;
    }
    
    /// <summary>
    /// This method returns the application.
    /// </summary>
    /// <returns>IApplication</returns>
    public virtual IApplication GetApplication()
    {
      return m_Application;
    }
    
    /// <summary>
    /// This method returns the sub session.
    /// This sub session will be cleared on logout,
    /// and therefore is a good place for storing variables
    /// needed for the lifetime of the user session.
    /// </summary>
    /// <returns>ISessionManager</returns>
    public virtual ISessionManager GetSubSession()
    {
      return m_SubSession;
    }
    
    /// <summary>
    /// This method returns the persistent sub session.
    /// This persistent sub session will not be cleared on logout.
    /// You should use the regular SubSession unless you have a 
    /// specific reason to use this persistent sub session.
    /// </summary>
    /// <returns>ISessionManager</returns>
    public virtual ISessionManager GetPersistentSubSession()
    {
      return m_PersistentSubSession;
    }
    
    /// <summary>
    /// This method adds the specified cookie.
    /// </summary>
    /// <param name="String">the cookie name.</param>
    /// <param name="String">the cookie value.</param>
    public virtual void SetCookie(String strCookieName, String strCookieValue)
    {
      m_WebData.AddCookie(new XPCookie(strCookieName, strCookieValue));
    }
    
    /// <summary>
    /// This method adds the specified cookie.
    /// </summary>
    /// <param name="_xpCookie">the XPCookie to add.</param>
    public virtual void SetCookie(XPCookie _xpCookie)
    {
      m_WebData.AddCookie(_xpCookie);
    }
    
    /// <summary>
    /// This method returns the requested cookie.
    /// </summary>
    /// <returns>String the cookie value. null if cookie was not found.</returns>
    public virtual String GetCookie(String strCookieName)
    {
      if ((strCookieName == null) || ("".Equals(strCookieName)))
      {
        return null;
      }
      XPCookie[] ca = m_Request.GetCookies();
      String sCookieVal = null;
      if (ca != null)
      {
        int iNum = ca.Length;
        XPCookie c;
        for (int x = 0;x < iNum;x++)
        {
          c = ca[x];
          log.Debug("ApplicationData.GetCookie() examining cookie: " + c.GetName());
          if (strCookieName.Equals(c.GetName()))
          {
            log.Debug("Found cookie.");
            sCookieVal = c.GetValue();
            return sCookieVal;
          }
        }
      }
      //	for each cookie
      //	got cookies
      return sCookieVal;
    }
    
    /// <summary>
    /// This method returns the requested header.
    /// </summary>
    /// <returns>String the header value. null if header was not found.</returns>
    public virtual String GetHeader(String strHeaderName)
    {
      if ((strHeaderName == null) || ("".Equals(strHeaderName)))
      {
        return null;
      }
      return m_Request.GetHeader(strHeaderName);
    }
    
    /// <summary>
    /// JF- Sets the content language on the WebData.
    /// </summary>
    /// <param name="_sLocale"></param>
    public virtual void SetContentLanguage(String _sLocale)
    {
      m_WebData.SetContentLanguage(_sLocale);
    }
    
    /// <summary>
    /// This method sets the requested header.
    /// </summary>
    /// <param name="String">the header name</param>
    /// <param name="String">the header value</param>
    public virtual void SetHeader(String strHeaderName, String strHeaderValue)
    {
      m_WebData.SetHeader(strHeaderName, strHeaderValue);
    }
    
    /// <summary>
    /// This method sets the status code on the Response.
    /// </summary>
    /// <param name="_nStatus">Status code</param>
    public virtual void SetStatus(int _nStatus)
    {
      m_WebData.SetStatus(_nStatus);
    }
    
    /// <summary>
    /// Returns the current request.
    /// </summary>
    /// <returns>IXPRequest</returns>
    public virtual IXPRequest GetRequest()
    {
      return m_Request;
    }
    
    /// <summary>
    /// JF- Returns the current request URL.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetRequestURL()
    {
      return m_Request.GetRequestURL();
    }
    
    /// <summary>
    /// JF- Returns the current server name from XPRequest.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetServerName()
    {
      return m_Request.GetServerName();
    }
    
    /// <summary>
    /// JF- Returns the current port number from XPRequest.
    /// </summary>
    /// <returns>int</returns>
    public virtual int GetServerPort()
    {
      return m_Request.GetServerPort();
    }
    
    /// <summary>
    /// JF- Returns the value of the paramter in argument.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetParameterValue(String _sParamName)
    {
      return m_Request.GetParameterValue(_sParamName);
    }
  }
}