using System;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.uiinfrastructure.login
{
  /// <author>
  /// DonH
  /// This class contains the result of a failed login attempt.
  /// </author>
  public class FailedLoginResult
  {
    
    /// <summary>
    /// Localized login error message suitable for display to user. 
    /// </summary>
    private String m_strError;
    
    /// <summary>
    /// The Redirect, if one of the PEIs returned a failed login
    /// redirect. May be null.
    /// </summary>
    private Redirect m_Redirect;
    
    /// <summary>
    /// Creates a new FailedLoginResult object with a custom error message.
    /// </summary>
    /// <param name="_strError">The custom error message for the end user.</param>
    public FailedLoginResult(String _strError)
    {
      m_strError = _strError;
      m_Redirect = null;
    }
    
    /// <summary>
    /// Creates a new FailedLoginResult object for a failed
    /// login that was redirected from a PEI.
    /// </summary>
    /// <param name="redirect">where to redirect the user to (specified by a login PEI).</param>
    public FailedLoginResult(Redirect _redirect)
    {
      m_Redirect = _redirect;
      m_strError = null;
    }
    
    /// <summary>
    /// Creates a new FailedLoginResult object for a failed
    /// login that was redirected from a PEI.
    /// </summary>
    /// <param name="_strError">The custom error message for the end user (maybe ignored).</param>
    /// <param name="redirect">where to redirect the user to (specified by a login PEI).</param>
    public FailedLoginResult(String _strError, Redirect _redirect)
    {
      m_Redirect = _redirect;
      m_strError = _strError;
    }
    
    /// <summary>
    /// Get the error message for this failed login.
    /// </summary>
    /// <returns>String the error message for this failed login.</returns>
    public virtual String GetErrorMessage()
    {
      return m_strError;
    }
    
    /// <summary>
    /// Get the redirect for this failed login.
    /// </summary>
    /// <returns>Redirect the redirect for this failed login.</returns>
    public virtual Redirect GetFailureRedirect()
    {
      return m_Redirect;
    }
  }
}