using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.pei;
namespace com.plumtree.uiinfrastructure.login
{
  /// <summary>
  /// This helper object wraps up Login PEIs that implement
  /// either ILoginActions or ILoginActions2 and makes them
  /// behave the same.
  /// </summary>
  /// <author>DonH</author>
  public class LoginPEIContainer : ILoginActions2
  {
    protected ILoginActions m_LoginActionsPEI;
    protected ILoginActions2 m_LoginActions2PEI;
    public LoginPEIContainer(ILoginActions _pei)
    {
      m_LoginActionsPEI = _pei;
      if (_pei is ILoginActions2)
      {
        m_LoginActions2PEI = (ILoginActions2) _pei;
      } else
      {
        m_LoginActions2PEI = null;
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.pei.ILoginActions2#OnFailedLoginDoRedirect(com.plumtree.uiinfrastructure.login.ApplicationData,java.lang.String,com.plumtree.openfoundation.util.XPException)"></seealso>
    public virtual Redirect OnFailedLoginDoRedirect(ApplicationData _appData,
     String _strUserName, XPException _xpError)
    {
      if (null != m_LoginActions2PEI)
      {
        return m_LoginActions2PEI.OnFailedLoginDoRedirect(_appData, _strUserName,
         _xpError);
      }
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.pei.ILoginActions#OnAfterLogin(java.lang.Object,com.plumtree.uiinfrastructure.login.ApplicationData)"></seealso>
    public virtual Redirect OnAfterLogin(Object _oUserSession, ApplicationData _appData)
    {
      return m_LoginActionsPEI.OnAfterLogin(_oUserSession, _appData);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.pei.ILoginActions#OnBeforeLogout(java.lang.Object,com.plumtree.uiinfrastructure.login.ApplicationData)"></seealso>
    public virtual Redirect OnBeforeLogout(Object _oUserSession, ApplicationData _appData)
    {
      return m_LoginActionsPEI.OnBeforeLogout(_oUserSession, _appData);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.pei.ILoginActions#OnBeforeLogin(com.plumtree.uiinfrastructure.login.ApplicationData,java.lang.String)"></seealso>
    public virtual Redirect OnBeforeLogin(ApplicationData _appData, String _strUserName)
    {
      return m_LoginActionsPEI.OnBeforeLogin(_appData, _strUserName);
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.pei.ILoginActions#OnFailedLogin(com.plumtree.uiinfrastructure.login.ApplicationData,java.lang.String,com.plumtree.openfoundation.util.XPException)"></seealso>
    public virtual String OnFailedLogin(ApplicationData _appData, String _strUserName,
     XPException _xpError)
    {
      return m_LoginActionsPEI.OnFailedLogin(_appData, _strUserName, _xpError);
      }
    
    /// <summary>
    /// This method is overridden so that when we try to display
    /// the contents of the PEI lists we get an appropriate value
    /// instead of just seeing wrapper objects.
    /// </summary>
    /// <seealso cref="java.lang.Object#toString()"></seealso>
    public override String ToString()
    {
      return m_LoginActionsPEI.ToString();
    }
  }
}