using System;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.uiinfrastructure.login
{
  /// <summary>
  /// This class contains the result of an attempted login.
  /// </summary>
  public class LoginResult
  {
    
    /// <summary>
    /// The user session, if successful.  Null if login failed. 
    /// </summary>
    public Object m_UserSession;
    
    /// <summary>
    /// The Redirect, if one of the PEIs returned a redirect. May be null. 
    /// </summary>
    public Redirect m_Redirect;
    
    /// <summary>
    /// Whether or not the login was succesful 
    /// </summary>
    public bool m_bSuccess = false;
    
    /// <summary>
    /// Whether or not the login was redirected before the login attempt 
    /// </summary>
    public bool m_bRedirectedBeforeLogin = false;
    
    /// <summary>
    /// Whether or not the login is for the guest user 
    /// </summary>
    public bool m_bIsGuest = false;
    
    /// <summary>
    /// Localized login error message suitable for display to user. Will be
    /// null if attempt succesful.
    /// </summary>
    public String m_strError;
    
    /// <summary>
    /// Creates a new LoginResult object for a succesful login.
    /// </summary>
    /// <param name="userSession">The new user session object.</param>
    /// <param name="redirect">where to redirect the user to (specified by a login PEI).</param>
    /// <param name="bIsGuestUser"></param>
    public LoginResult(Object userSession, Redirect redirect, bool bIsGuestUser)
    {
      m_UserSession = userSession;
      m_Redirect = redirect;
      m_bSuccess = true;
      m_bIsGuest = bIsGuestUser;
      m_strError = null;
    }
    
    /// <summary>
    /// Creates a new LoginResult object for a failed login.
    /// </summary>
    /// <param name="userSession">The new user session object.</param>
    /// <param name="redirect">where to redirect the user to (specified by a login PEI).</param>
    /// <param name="strError">
    /// An error message suitable for display to an end user
    /// (ie localized, and not revealing too much information).  Should not be null.
    /// </param>
    /// <param name="bIsGuestUser"></param>
    public LoginResult(Object userSession, Redirect redirect, String strError,
     bool bIsGuestUser)
    {
      // We do not return the login exception to the clients of this code
      // because displaying it to the end user (or putting it in an HTMLComment)
      // is a security hole for people trying to guess user names and passwords.
      // If they can see the original error (user not found), then they can
      // easily create a list of users.
      m_UserSession = userSession;
      m_Redirect = redirect;
      m_bSuccess = false;
      m_bIsGuest = bIsGuestUser;
      m_strError = strError;
    }
    
    /// <summary>
    /// Creates a new LoginResult object for a login that was
    /// redirected before the login attempt.
    /// </summary>
    /// <param name="redirect">where to redirect the user to (specified by a login PEI).</param>
    public LoginResult(Redirect redirect)
    {
      m_UserSession = null;
      m_Redirect = redirect;
      m_bRedirectedBeforeLogin = true;
      m_bSuccess = true;
      m_bIsGuest = true;
      m_strError = null;
    }
  }
}