using System;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.interpreter;
namespace com.plumtree.uiinfrastructure.login
{
  /// <summary>
  /// This singleton class controls SSO redirects.  To implement your own SSO
  /// redirect code, simply implement an instance of ISSOHelper and initialize this
  /// class with your helper using the StaticInit method.
  /// Created on Mar 13, 2003
  /// </summary>
  /// <author>mikej</author>
  public sealed class SSOHelper : ISSOHelper
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.login.SSOHelper");
    private static ISSOHelper s_SSOHelper = null;
    private static bool s_bInitialized = false;
    
    /// <summary>
    /// Use this class static to access the one and only instance of
    /// SSOHelper.
    /// </summary>
    public static readonly SSOHelper INSTANCE = new SSOHelper();
    
    /// <summary>
    /// Private constructor to ensure there is only one instance.
    /// </summary>
    private SSOHelper()
    {
    }
    
    /// <summary>
    /// Static initialization method.  Use this method to add custom user
    /// session object handling code.
    /// This method does not need to be called.
    /// </summary>
    /// <param name="ISSOHelper">custom SSO code</param>
    public static void StaticInit(ISSOHelper SSOHelper)
    {
      if (s_bInitialized)
      {
        log.Error("SSOHelper StaticInit attempted twice.  Ignoring second attempt.");
        return;
      }
      s_SSOHelper = SSOHelper;
      s_bInitialized = true;
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.login.ISSOHelper#IsSSOEnabled()
     */
     
    public bool IsSSOEnabled()
    {
      if (s_SSOHelper == null)
      {
        return false;
      }
      return s_SSOHelper.IsSSOEnabled();
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.login.ISSOHelper#DoLoginRedirect(com.plumtree.uiinfrastructure.activityspace.RequestData, java.lang.String)
     */
     
    public Redirect DoLoginRedirect(RequestData tempData, String sQueryString)
    {
      if (s_SSOHelper == null)
      {
        return null;
      }
      return s_SSOHelper.DoLoginRedirect(tempData, sQueryString);
    }
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.login.ISSOHelper#DoLogoutRedirect(com.plumtree.uiinfrastructure.activityspace.RequestData)
     */
     
    public bool DoLogoutRedirect(RequestData tempData)
    {
      if (s_SSOHelper == null)
      {
        return false;
      }
      return s_SSOHelper.DoLogoutRedirect(tempData);
    }
  }
}