using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.opener;
using com.plumtree.uiinfrastructure.statichelpers;
namespace com.plumtree.uiinfrastructure.opener.plugins
{
  /// <summary>
  /// This plugin enables the Common Opener to open objects using
  /// the more human readable "open" parameter instead of the
  /// "clsID" parameter.  The "open" is processed into its
  /// ClassID equivalent and put back into the QS.
  /// </summary>
  public class OpenPlugin : IOpenerPlugins
  {
    
    /// <seealso cref="com.plumtree.portaluiinfrastructure.opener.IOpenerPlugins#ProcessPlugIn(RedirectData,AActivitySpace,Object)"></seealso>
    public virtual RedirectData ProcessPlugIn(RedirectData _redirectData,
     AActivitySpace _asCurrentSpace, Object _userSession)
    {
      // get the "open" parameter from the querystring. 
      String strOpen = HTTPHelpers.GetArgumentAsString(_redirectData.GetQSArguments(),
       IOpenerConstants.QS_OPEN, null);
      if (null != strOpen && strOpen.Length > 0)
      {
        int nClassID = -1;
        try
        {
          // Try to convert the "open" parameter into a numeric classID
          char firstChar = XPStringUtility.CharAt(strOpen, 0);
          if (XPCharacterUtility.IsASCIIDecimalDigit(firstChar))
          {
            nClassID = XPConvert.ToInteger(strOpen);
          }
        }
        catch (Exception e)
        {
          // This is an expected result for UI Objects, which
          // have non-numeric class keys and no object ID.
          // Ignore the exception.
          nClassID = -1;
        }
        if (nClassID != -1)
        {
          // put ClassID in return data, overwriting if it exists. 
          _redirectData.AddQSArgument(IOpenerConstants.QS_CLASSID, nClassID);
          }
      }
      return _redirectData;
    }
  }
}