using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.errordisplay;
using com.plumtree.uiinfrastructure.opener;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.opener.types
{
  /// <summary>
  /// TODO: Document
  /// </summary>
  /// <author>DonH</author>
  public class ClassTypeManager : IClassTypeManager
  {
    ////////////////////////////
    // IClassTypeDesc Members //
    ////////////////////////////
    
    /// <summary>
    /// A map of loaded ClassTypeDescriptors.  The key is the ClassID for
    /// PTClassTypeDesc objects and ClassName for UIClassTypeDesc objects.
    /// </summary>
    protected XPHashtable m_mapClassTypes;
    
    /// <summary>
    /// An array of all loaded IClassType objects. 
    /// </summary>
    protected IClassTypeDesc[] m_aIClassTypes;
    
    /// <param name="cmCachingManager"></param>
    public ClassTypeManager(CachingManager cmCachingManager)
    {
      ///////////////////////////////////
      // IClassTypeDesc Initialization //
      ///////////////////////////////////
      // load the dynamically discovered ClassTypeDesc objects.
      Object[] oaClassTypes = (Object[]) cmCachingManager.GetEntry("ClassTypeDesc");
      IClassTypeDesc[] aIClassTypes = new IClassTypeDesc[oaClassTypes.Length];
      for (int i = 0;i < oaClassTypes.Length;i++)
      {
        aIClassTypes[i] = (IClassTypeDesc) oaClassTypes[i];
      }
      m_aIClassTypes = aIClassTypes;
      m_mapClassTypes = new XPHashtable();
      ////////////////////////////
      // General Initialization //
      ////////////////////////////
      InitializeMapOfClassTypes();
    }
    private void InitializeMapOfClassTypes()
    {
      for (int i = 0;i < m_aIClassTypes.Length;i++)
      {
        m_mapClassTypes.PutElement(m_aIClassTypes[i].GetClassKey(), m_aIClassTypes[i]);
        }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.opener.types.IOpenerTypeManager#GetDefaultOpenerMode(java.lang.String)"></seealso>
    public virtual int GetDefaultOpenerMode(String _strClassKey)
    {
      try
      {
        return GetClassTypeDescItem(_strClassKey).GetDefaultOpenerMode();
        }
      catch (Exception ex)
      {
        throw new XPException("Error retrieving default opener mode " + "for ClassKey [" + _strClassKey + "]",
         ex);
      }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.opener.types.IOpenerTypeManager#GetOpenerRedirect(java.lang.String,int,com.plumtree.openfoundation.util.XPHashtable,com.plumtree.uiinfrastructure.activityspace.AActivitySpace,java.lang.Object)"></seealso>
    public virtual Redirect GetOpenerRedirect(String _strClassKey, int _nOpenerMode,
     XPHashtable _htArguments, AActivitySpace _asParent, Object _userSession)
    {
      switch (_nOpenerMode){
      case IOpenerConstants.OPENER_MODE_CREATE:
      return GetClassTypeDescItem(_strClassKey).GetRedirectCreateNew(_htArguments,
       _asParent, _userSession);
      goto case IOpenerConstants.OPENER_MODE_EDIT;
      case IOpenerConstants.OPENER_MODE_EDIT:
      return GetClassTypeDescItem(_strClassKey).GetRedirectOpenObject(_htArguments,
       _asParent, _userSession);
      goto case IOpenerConstants.OPENER_MODE_VIEW;
      case IOpenerConstants.OPENER_MODE_VIEW:
      return GetClassTypeDescItem(_strClassKey).GetRedirectViewObject(_htArguments,
       _asParent, _userSession);
      goto case IOpenerConstants.OPENER_MODE_VIEW_META_DATA;
      case IOpenerConstants.OPENER_MODE_VIEW_META_DATA:
      return GetClassTypeDescItem(_strClassKey).GetRedirectViewObjectMetaData(_htArguments,
       _asParent, _userSession);
      goto case IOpenerConstants.OPENER_MODE_SEARCH_BROWSE;
      case IOpenerConstants.OPENER_MODE_SEARCH_BROWSE:
      return GetClassTypeDescItem(_strClassKey).GetRedirectSearchBrowse(_htArguments,
       _asParent, _userSession);
      goto case IOpenerConstants.OPENER_MODE_SEARCH_EDIT;
      case IOpenerConstants.OPENER_MODE_SEARCH_EDIT:
      return GetClassTypeDescItem(_strClassKey).GetRedirectSearchEdit(_htArguments,
       _asParent, _userSession);
      goto default;
      default:
      // unsupported opener mode.										
      UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,
       null, _asParent.GetString(485, "ptmsgs_portalinfrastructure"), null);
      UIErrorHandler.RaiseError(myError, _asParent.GetSubSession(), _asParent.GetApplication());
      throw new XPException("The OpenerMode argument specified is invalid.");
      break;
      }
    }
    
    /// <summary>
    /// Returns IClassTypeDesc for the particular ClassKey.
    /// </summary>
    /// <param name="_strClassKey">
    /// For PTClassTypes, this is the ClassID, for 
    /// UIClassTypes, this is the ClassName.
    /// </param>
    /// <returns>IClassTypeDesc for the particular ClassKey.</returns>
    protected virtual IClassTypeDesc GetClassTypeDescItem(String _strClassKey)
    {
      try
      {
        IClassTypeDesc item = (IClassTypeDesc) m_mapClassTypes.GetElement(_strClassKey);
        return item;
      }
      catch (Exception ex)
      {
        throw new XPException("The ClassKey [" + _strClassKey + "] is invalid.",
         ex);
      }
    }
  }
}