using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.uiinfrastructure.opener.types
{
  /// <summary>
  /// Interface for IClassTypeDesc.  
  /// </summary>
  /// <author>John Osborne</author>
  public interface IClassTypeDesc
  {
    
    /// <summary>
    /// Returns the class type.
    /// </summary>
    /// <returns>The class type.</returns>
    ClassTypeEnum GetClassType();
    
    /// <summary>
    /// Returns the key for this class type.
    /// </summary>
    /// <returns>The key for this class type.</returns>
    String GetClassKey();
    
    /// <summary>
    /// Returns the default opener mode for this class type.
    /// </summary>
    /// <returns>The default opener mode for this class type.</returns>
    int GetDefaultOpenerMode();
    
    /// <summary>
    /// This method is called by the Common Opener in the create
    /// mode.  Depending on what class type you are dealing with,
    /// the Common Opener will invoke the correct implemented 
    /// method.  Some class types may need to override this so
    /// that they can put additional control arguments in.
    /// </summary>
    /// <param name="_htArguments">Arguments to reach the new space.</param>
    /// <param name="_asParent">The parent activity space.</param>
    /// <param name="_userSession">The current session.</param>
    /// <returns>
    /// A redirect that will bring you to the create 
    /// activity space for the particular class/object.
    /// </returns>
    Redirect GetRedirectCreateNew(XPHashtable _htArguments, AActivitySpace _asParent,
     Object _userSession);
    
    /// <summary>
    /// This method is called by the Common Opener in the edit
    /// mode.  Depending on what class type you are dealing with,
    /// the Common Opener will invoke the correct implemented 
    /// method.  Some class types may need to override this so
    /// that they can put additional control arguments in.
    /// </summary>
    /// <param name="_htArguments">Arguments to reach the new space.</param>
    /// <param name="_asParent">The parent activity space.</param>
    /// <param name="_userSession">The current session.</param>
    /// <returns>
    /// A redirect that will bring you to the edit 
    /// activity space for the particular class/object.
    /// </returns>
    Redirect GetRedirectOpenObject(XPHashtable _htArguments, AActivitySpace _asParent,
     Object _userSession);
    
    /// <summary>
    /// This method is called by the Common Opener in the search
    /// basic mode.  This is a helper method that will return one
    /// of the four main modes.
    /// </summary>
    /// <param name="_htArguments">Arguments to reach the new space.</param>
    /// <param name="_asParent">The parent activity space.</param>
    /// <param name="_userSession">The current session.</param>
    /// <returns>
    /// A redirect that will bring you to the meta 
    /// activity space for the particular class/object.
    /// </returns>
    Redirect GetRedirectSearchBrowse(XPHashtable _htArguments, AActivitySpace _asParent,
     Object _userSession);
    
    /// <summary>
    /// This method is called by the Common Opener in the search
    /// basic mode.  This is a helper method that will return one
    /// of the four main modes.
    /// </summary>
    /// <param name="_htArguments">Arguments to reach the new space.</param>
    /// <param name="_asParent">The parent activity space.</param>
    /// <param name="_userSession">The current session.</param>
    /// <returns>
    /// A redirect that will bring you to the meta 
    /// activity space for the particular class/object.
    /// </returns>
    Redirect GetRedirectSearchEdit(XPHashtable _htArguments, AActivitySpace _asParent,
     Object _userSession);
    
    /// <summary>
    /// This method is called by the Common Opener in the view
    /// mode.  Depending on what class type you are dealing with,
    /// the Common Opener will invoke the correct implemented 
    /// method.  Some class types may need to override this so
    /// that they can put additional control arguments in.
    /// </summary>
    /// <param name="_htArguments">Arguments to reach the new space.</param>
    /// <param name="_asParent">The parent activity space.</param>
    /// <param name="_userSession">The current session.</param>
    /// <returns>
    /// A redirect that will bring you to the view 
    /// activity space for the particular class/object.
    /// </returns>
    Redirect GetRedirectViewObject(XPHashtable _htArguments, AActivitySpace _asParent,
     Object _userSession);
    
    /// <summary>
    /// This method is called by the Common Opener in the meta
    /// mode.  Depending on what class type you are dealing with,
    /// the Common Opener will invoke the correct implemented 
    /// method.  Some class types may need to override this so
    /// that they can put additional control arguments in.
    /// </summary>
    /// <param name="_htArguments">Arguments to reach the new space.</param>
    /// <param name="_asParent">The parent activity space.</param>
    /// <param name="_userSession">The current session.</param>
    /// <returns>
    /// A redirect that will bring you to the meta 
    /// activity space for the particular class/object.
    /// </returns>
    Redirect GetRedirectViewObjectMetaData(XPHashtable _htArguments, AActivitySpace _asParent,
     Object _userSession);
  }
}