using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.uiinfrastructure.pagecontrols.htmldatetimepicker;
using com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker;
namespace com.plumtree.uiinfrastructure.pagecontrols.datepicker
{
  /// <summary>
  /// Abstract class extended by all DatePickers.
  /// </summary>
  /// <author>DorisL</author>
  /// <author>CaiusV</author>
  public abstract class ADatePickerComponent : AComponent,IPageControl
  {
    
    /// <summary>
    /// Index constant for array of options: option value 
    /// </summary>
    public const int OPTION_VALUE = 0;
    
    /// <summary>
    /// Index constant for array of options: text to display on option 
    /// </summary>
    public const int OPTION_TEXT = 1;
    
    /// <summary>
    /// some other helpful static Strings 
    /// </summary>
    public const String AM = "AM";
    public const String PM = "PM";
    public const String H = "H";
    public const String EMPTY = "";
    public virtual XPDateTime GetDateAsDateTime()
    {
      return null;
    }
    public virtual void SetBlankDate()
    {
    }
    public virtual void SetDefaultDate(int _nMonth, int _nDay, int _nYear)
    {
    }
    public abstract String GetSelectedDateAsString();
    public abstract String GetDateInputName();
    public abstract String GetPTDatepickerVarName();
    public virtual XPDateTime GetDateTimeAsDateTime()
    {
      return null;
    }
    
    /// <summary>
    /// Initialize a new DatePicker with the preset date empty.
    /// </summary>
    /// <param name="_strFormName">Name of the form where this DatePicker appears</param>
    /// <param name="_ptPageMgr">the PageMgr responsible for this control</param>
    /// <param name="_strUniqueInstanceID">unique id for this instance</param>
    public abstract void Init(String _strFormName, PageMgr _ptPageMgr, String _strUniqueInstanceID);
    
    /// <summary>
    /// Initialize a new DatePicker with the preset date set.
    /// </summary>
    /// <param name="_strFormName">Name of the form where this DatePicker appears</param>
    /// <param name="_dtDefaultDateTime">the preset datetime</param>
    /// <param name="_ptPageMgr">the PageMgr responsible for this control</param>
    /// <param name="_strUniqueInstanceID">unique id for this instance</param>
    public abstract void Init(String _strFormName, XPDateTime _dtDefaultDateTime,
     PageMgr _ptPageMgr, String _strUniqueInstanceID);
    
    /// <summary>
    /// Return the XPDateTime for the specific data picker
    /// </summary>
    /// <param name="_htRequestParameters">name/value pairs in the request</param>
    /// <param name="_strUniqueInstanceID">unique id for this control</param>
    /// <param name="_xpLocale">Locale for this control</param>
    /// <param name="_xpTimeZone">TimeZone for this control</param>
    /// <param name="boolean">_bDoValidation , set to true if date must be set (pulldowns cannot be empty)</param>
    /// <param name="AccessStyles">_asAccessStyle, the page's mode</param>
    /// <returns>XPDateTime</returns>
    public static XPDateTime GetDateTime(XPHashtable _htRequestParameters,
     String _strUniqueInstanceID, XPLocale _xpLocale, XPTimeZone _xpTimeZone,
     bool _bDoValidation, AccessStyles _asAccessStyle)
    {
      if (_asAccessStyle.Equals(AccessStyles.STANDARD))
      {
        return JSDateTimePicker.GetDateTime(_htRequestParameters, _strUniqueInstanceID,
         _xpLocale, _xpTimeZone);
      } else
      {
        return HTMLDateTimePicker.GetDateTime(_htRequestParameters, _strUniqueInstanceID,
         _xpLocale, _xpTimeZone, _bDoValidation);
      }
    }
    public abstract com.plumtree.uiinfrastructure.pagecontrols.framework.ComponentMgr GetComponentMgr(com.plumtree.uiinfrastructure.pagecontrols.framework.PageMgr x0);
    public abstract String GetComponentMgrName();
  }
}