using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.application.varpacks;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;
using com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker;
using com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
using com.plumtree.xpshared.jsutil;
namespace com.plumtree.uiinfrastructure.pagecontrols.datepicker
{
  /// <summary>
  /// Page Control for HTML Date Picker.
  /// </summary>
  /// <author>Doris Lin</author>
  public class DatePicker : ADatePickerComponent,IPageControl
  {
    
    /// <summary>
    /// for logging 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.datepicker.DatePicker");
    
    /// <summary>
    /// String constant for month select drop down component 
    /// </summary>
    public const String MONTH_DROP_DOWN = "SDD_MONTH";
    
    /// <summary>
    /// String constant for day select drop down component 
    /// </summary>
    public const String DAY_DROP_DOWN = "SDD_DAY";
    
    /// <summary>
    /// String constant for year select drop down component 
    /// </summary>
    public const String YEAR_DROP_DOWN = "SDD_YEAR";
    
    /// <summary>
    /// String constant for min year, max year, min num of days, max num of days 
    /// </summary>
    public const int MIN_YEAR = 1950;
    public const int NUM_OF_YEARS = 71;
    public const int MIN_DAY = 0;
    public const int MAX_DAY = 31;
    public const int NUM_OF_MONTHS = 12;
    
    /// <summary>
    /// Constant for Month Names 
    /// </summary>
    public const int BLANK = 0;
    
    /// <summary>
    /// Constant String for BLANK 
    /// </summary>
    public const String STR_BLANK = "0";
    
    /// <summary>
    /// Delimeters in the date pattern 
    /// </summary>
    public const String DELIMETERS = " /-,.";
    
    /// <summary>
    /// Reg. expression for delimeters in date pattern 
    /// </summary>
    public const String REG_EXP_DELEMETERS = "[,\\s./-]";
    
    /// <summary>
    /// Variables to save the order in which the month, day and year should appear (ranging from 0-2) 
    /// </summary>
    private static int m_nOrderM;
    private static int m_nOrderd;
    private static int m_nOrdery;
    
    /// <summary>
    /// mvc class name for datepicker component 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "DatePicker_D";
    
    /// <summary>
    /// component's base name 
    /// </summary>
    public const String STR_COMP_BASE_NAME = "in_se_pc";
    
    /// <summary>
    /// Variables to save the month, day, and year 
    /// </summary>
    protected String m_strMonth;
    protected String m_strDay;
    protected String m_strYear;
    
    /// <summary>
    /// HTMLSelectDropDown Components 
    /// </summary>
    private HTMLSelectDropDown m_ptHTMLMonthSelectDropDown;
    private HTMLSelectDropDown m_ptHTMLDaySelectDropDown;
    private HTMLSelectDropDown m_ptHTMLYearSelectDropDown;
    
    /// <summary>
    /// DateFormat object 
    /// </summary>
    private XPDateFormat m_dfDateFormat;
    
    /// <summary>
    /// Variables to save the number of M's, d's, and y's 
    /// </summary>
    private int m_nNumM;
    private int m_nNumd;
    
    /// <summary>
    /// String array to save the delimeters for the date pattern 
    /// </summary>
    private XPMatch[] m_nDateDelims = new XPMatch[3];
    
    /// <summary>
    /// Array containing month names 
    /// </summary>
    private String[][] m_arrMonthNames;
    
    /// <summary>
    /// unique string id for control 
    /// </summary>
    private String m_strUniqueInstanceID;
    
    /// <summary>
    /// Creates a new DatePicker for 508 mode
    /// </summary>
    public DatePicker()
    {
    }
    
    /// <summary>
    /// initializes a new HTMLDatePicker object with the month, day and year blank
    /// </summary>
    /// <param name="_strFormName">Name of the form where this DatePicker appears</param>
    /// <param name="_ptPageMgr">the PageMgr responsible for this control</param>
    /// <param name="_strUniqueInstanceID">unique id for this instance</param>
    public override void Init(String _strFormName, PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
      XPResourceManager rm = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");
      m_strUniqueInstanceID = _strUniqueInstanceID;
      String[][] arrDropDownValueText;
      //	create dateformat object for medium format and the given locale
      m_dfDateFormat = XPDateFormat.GetDateInstance(XPDateFormat.MEDIUM, _ptPageMgr.GetLocale(),
       _ptPageMgr.GetTimeZone());
      // extract and save the information from the pattern
      ExtractPatternInfo(m_dfDateFormat.ToPattern());
      // initialize the month select drop down component
      m_arrMonthNames = new String[NUM_OF_MONTHS + 1][];
      for (int j2c_index = 0;j2c_index < NUM_OF_MONTHS + 1;j2c_index++)
      {
        m_arrMonthNames[j2c_index] = new String[2];
      }
      // fill the array with either month names or numbers, depending on the num. of M's
      switch (m_nNumM){
      case 1:
      FillSingleDigitArray(m_arrMonthNames);
      break;
      case 2:
      FillDoubleDigitArray(m_arrMonthNames);
      break;
      case 3:
      FillMonthNameArray(m_arrMonthNames);
      break;
      }
      m_ptHTMLMonthSelectDropDown = (HTMLSelectDropDown) m_ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
      m_ptHTMLMonthSelectDropDown.Init(m_ptPageMgr, MONTH_DROP_DOWN + _strUniqueInstanceID,
       m_arrMonthNames, rm.GetString(148, "ptmsgs_infrastructure", m_ptPageMgr.GetLangID()));
      // initialize the day select drop down component
      arrDropDownValueText = new String[MAX_DAY + 1][];
      for (int j2c_index = 0;j2c_index < MAX_DAY + 1;j2c_index++)
      {
        arrDropDownValueText[j2c_index] = new String[2];
      }
      // fill the array with either "01" or "1" format, depending on the num of d's
      switch (m_nNumd){
      case 1:
      FillSingleDigitArray(arrDropDownValueText);
      break;
      case 2:
      FillDoubleDigitArray(arrDropDownValueText);
      break;
      }
      m_ptHTMLDaySelectDropDown = (HTMLSelectDropDown) m_ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
      m_ptHTMLDaySelectDropDown.Init(m_ptPageMgr, DAY_DROP_DOWN + _strUniqueInstanceID,
       arrDropDownValueText, rm.GetString(149, "ptmsgs_infrastructure", m_ptPageMgr.GetLangID()));
      // initialize the year select drop down component
      arrDropDownValueText = new String[NUM_OF_YEARS + 1][];
      for (int j2c_index = 0;j2c_index < NUM_OF_YEARS + 1;j2c_index++)
      {
        arrDropDownValueText[j2c_index] = new String[2];
      }
      FillYearArray(arrDropDownValueText);
      m_ptHTMLYearSelectDropDown = (HTMLSelectDropDown) m_ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
      m_ptHTMLYearSelectDropDown.Init(m_ptPageMgr, YEAR_DROP_DOWN + _strUniqueInstanceID,
       arrDropDownValueText, rm.GetString(150, "ptmsgs_infrastructure", m_ptPageMgr.GetLangID()));
      // set the default date to be blank
      SetBlankDate();
    }
    
    /// <summary>
    /// Initializes a new DatePicker Page Control with the given datetime as the
    /// preset value.
    /// </summary>
    /// <param name="_strFormName">Name of the form where this DatePicker appears</param>
    /// <param name="_xpDateTime">Preset DateTime</param>
    /// <param name="_ptPageMgr">the PageMgr responsible for this control</param>
    /// <param name="_strUniqueInstanceID">unique id for this instance</param>
    public override void Init(String _strFormName, XPDateTime _xpDateTime,
     PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      this.Init(_strFormName, _ptPageMgr, _strUniqueInstanceID);
      XPCalendar myXPCalendar = XPCalendar.GetInstance(_ptPageMgr.GetLocale());
      if (_xpDateTime != null)
      {
        myXPCalendar.SetTime(_xpDateTime);
        m_ptHTMLMonthSelectDropDown.SetHTMLComponentValue(EMPTY + myXPCalendar.Get(XPCalendar.MONTH) + EMPTY);
        m_ptHTMLDaySelectDropDown.SetHTMLComponentValue(EMPTY + myXPCalendar.Get(XPCalendar.DATE) + EMPTY);
        m_ptHTMLYearSelectDropDown.SetHTMLComponentValue(EMPTY + myXPCalendar.Get(XPCalendar.YEAR) + EMPTY);
        m_strMonth = EMPTY + myXPCalendar.Get(XPCalendar.YEAR) + EMPTY;
        m_strDay = EMPTY + myXPCalendar.Get(XPCalendar.DATE) + EMPTY;
        m_strYear = EMPTY + myXPCalendar.Get(XPCalendar.YEAR) + EMPTY;
      }
    }
    
    /// <summary>
    /// Sets the month, day, and year of drop down menus to blank
    /// </summary>
    public override void SetBlankDate()
    {
      m_ptHTMLMonthSelectDropDown.SetHTMLComponentValue(EMPTY + DatePicker.BLANK + EMPTY);
      m_ptHTMLDaySelectDropDown.SetHTMLComponentValue(EMPTY + DatePicker.BLANK + EMPTY);
      m_ptHTMLYearSelectDropDown.SetHTMLComponentValue(EMPTY + DatePicker.BLANK + EMPTY);
      }
    
    /// <summary>
    /// Returns the base name of this HTML OpenComponent.
    /// </summary>
    /// <returns>String</returns>
    protected virtual String GetHTMLComponentBaseName()
    {
      return STR_COMP_BASE_NAME;
    }
    
    /// <summary>
    /// Returns the date as a String in the pattern format specified by the locale, null
    /// values in memeber variables, m_strYear, m_strMonth, or m_strDay indicate an invalid
    /// date so return the default, and empty String.
    /// </summary>
    /// <returns>String the date as a String</returns>
    public override String GetSelectedDateAsString()
    {
      if (null == m_strYear || null == m_strMonth || null == m_strDay)
      {
        return EMPTY;
      }
      switch (m_nOrderd){
      case 0:
      if (m_nOrdery == 1)
      {
        return m_strDay + (m_nDateDelims[0]).ToString() + m_strYear + (m_nDateDelims[1]).ToString() + m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT];
        } else
      {
        return m_strDay + (m_nDateDelims[0]).ToString() + m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT] + (m_nDateDelims[1]).ToString() + m_strYear;
        }
      goto case 1;
      case 1:
      if (m_nOrdery == 0)
      {
        return m_strYear + (m_nDateDelims[0]).ToString() + m_strDay + (m_nDateDelims[1]).ToString() + m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT];
        } else
      {
        return m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT] + (m_nDateDelims[0]).ToString() + m_strDay + (m_nDateDelims[1]).ToString() + " " + m_strYear;
        }
      goto case 2;
      case 2:
      if (m_nOrdery == 0)
      {
        return m_strYear + (m_nDateDelims[0]).ToString() + m_strDay + (m_nDateDelims[1]).ToString() + m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT];
        } else
      {
        return m_strDay + (m_nDateDelims[0]).ToString() + m_strYear + (m_nDateDelims[1]).ToString() + m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT];
        }
      break;
      }
      return EMPTY;
    }
    
    /// <summary>
    /// This function returns an XPDateTime object that represents the date and
    /// the user has chosen using the HTMLDatePicker.
    /// </summary>
    /// <returns>XPDateTime</returns>
    public override XPDateTime GetDateAsDateTime()
    {
      XPCalendar myXPCalendar;
      try
      {
        myXPCalendar = XPCalendar.GetInstance(XPConvert.ToInteger(m_strYear),
         XPConvert.ToInteger(m_strMonth), XPConvert.ToInteger(m_strDay));
        }
      catch (XPException e)
      {
        log.Error(e, "unable to retreive datetime");
        return null;
      }
      return myXPCalendar.GetTime();
    }
    
    /// <summary>
    /// Returns the HTML for this component.
    /// </summary>
    /// <returns>HTMLElementCollection the HTML to display this control</returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      int nOrder = 0;
      try
      {
        // determine which order the components should be displayed
        for (int i = 0;i < 3;i++)
        {
          if (m_nOrderM == nOrder)
          {
            // add month drop down component 
            retValue.AddInnerHTMLElement(GetMonthDropDownHTML());
          } else
          if (m_nOrderd == nOrder)
          {
            // day drop down component 
            retValue.AddInnerHTMLElement(GetDayDropDownHTML());
          } else
          {
            //year drop down component
            retValue.AddInnerHTMLElement(GetYearDropDownHTML());
          }
          // add the delimeter after each component except for the last component
          if (i < 2)
          {
            retValue.AddInnerHTMLString((m_nDateDelims[i]).GetValue());
          }
          nOrder++;
        }
      }
      catch (XPException e)
      {
        log.Error(e, "cannot retreive componenet HTML");
      }
      return retValue;
    }
    
    /// <summary>
    /// Store the new value of this HTML component.
    /// </summary>
    /// <param name="_htRequestParameters">name/value pairs for the request</param>
    /// <returns>int - the status of the page</returns>
    public override int StoreUserInput(XPHashtable _htRequestParameters)
    {
      String[] data;
      m_ptHTMLMonthSelectDropDown.StoreUserInput(_htRequestParameters);
      m_ptHTMLDaySelectDropDown.StoreUserInput(_htRequestParameters);
      m_ptHTMLYearSelectDropDown.StoreUserInput(_htRequestParameters);
      //reset values for month, day, and year
      m_strMonth = STR_BLANK;
      m_strDay = STR_BLANK;
      m_strYear = STR_BLANK;
      data = (String[]) _htRequestParameters.GetElement(GetHTMLComponentBaseName() + "_" + MONTH_DROP_DOWN + m_strUniqueInstanceID);
      if (data != null)
      {
        m_strMonth = data[0];
      }
      data = (String[]) _htRequestParameters.GetElement(GetHTMLComponentBaseName() + "_" + DAY_DROP_DOWN + m_strUniqueInstanceID);
      if (data != null)
      {
        m_strDay = data[0];
      }
      data = (String[]) _htRequestParameters.GetElement(GetHTMLComponentBaseName() + "_" + YEAR_DROP_DOWN + m_strUniqueInstanceID);
      if (data != null)
      {
        m_strYear = data[0];
      }
      // if all the fields are blank, then the page is still valid
      if (m_strMonth.Equals(STR_BLANK) && m_strDay.Equals(STR_BLANK) && m_strYear.Equals(STR_BLANK))
      {
        return RepostControl.PAGE_STATUS_VALID;
      }
      // check if the date is valid, an invalid date will not be set
      try
      {
        XPCalendar.GetInstance(XPConvert.ToInteger(m_strYear), XPConvert.ToInteger(m_strMonth),
         XPConvert.ToInteger(m_strDay));
      }
      catch (XPException e)
      {
        log.Error(e, "invalid date");
        m_ptPageMgr.AddErrorMessage("153", ResourceConstants.INFRASTRUCTURE);
        return RepostControl.PAGE_STATUS_INVALID;
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// The HTMLElementCollectiion for the month drop down
    /// </summary>
    /// <returns>HTMLElementCollection</returns>
    private HTMLElementCollection GetMonthDropDownHTML()
    {
      m_ptHTMLMonthSelectDropDown = (HTMLSelectDropDown) (m_ptPageMgr.GetComponentMgr(HTMLSelectDropDown.STR_MVC_CLASS_NAME)).GetComponent(MONTH_DROP_DOWN + m_strUniqueInstanceID);
      m_ptHTMLMonthSelectDropDown.SetHTMLComponentStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
      return m_ptHTMLMonthSelectDropDown.GetComponentHTML();
    }
    
    /// <summary>
    /// The HTMLElementCollection for the day drop down
    /// </summary>
    /// <returns>HTMLElementCollection</returns>
    private HTMLElementCollection GetDayDropDownHTML()
    {
      m_ptHTMLDaySelectDropDown = (HTMLSelectDropDown) (m_ptPageMgr.GetComponentMgr(HTMLSelectDropDown.STR_MVC_CLASS_NAME)).GetComponent(DAY_DROP_DOWN + m_strUniqueInstanceID);
      m_ptHTMLDaySelectDropDown.SetHTMLComponentStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
      return m_ptHTMLDaySelectDropDown.GetComponentHTML();
    }
    
    /// <summary>
    /// The HTMLElementCollection for the year drop down
    /// </summary>
    /// <returns>HTMLElementCollection for the year dropdown</returns>
    private HTMLElementCollection GetYearDropDownHTML()
    {
      m_ptHTMLYearSelectDropDown = (HTMLSelectDropDown) (m_ptPageMgr.GetComponentMgr(HTMLSelectDropDown.STR_MVC_CLASS_NAME)).GetComponent(YEAR_DROP_DOWN + m_strUniqueInstanceID);
      m_ptHTMLYearSelectDropDown.SetHTMLComponentStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
      return m_ptHTMLYearSelectDropDown.GetComponentHTML();
    }
    
    /// <summary>
    /// Fill in the array to create the select drop down component for months
    /// </summary>
    /// <param name="_arrDropDownValueText">text for the month dropdown</param>
    private void FillMonthNameArray(String[][] _arrDropDownValueText)
    {
      XPCalendar myXPCalendar;
      XPDateTime myXPDateTime;
      XPDateFormat myXPDateFormat;
      String dateString;
      _arrDropDownValueText[0][OPTION_VALUE] = EMPTY + DatePicker.BLANK + EMPTY;
      _arrDropDownValueText[0][OPTION_TEXT] = " ";
      for (int i = 1;i < _arrDropDownValueText.Length;i++)
      {
        _arrDropDownValueText[i][OPTION_VALUE] = EMPTY + i + EMPTY;
        myXPCalendar = XPCalendar.GetInstance(2004, i, 1);
        myXPDateTime = myXPCalendar.GetTime();
        myXPDateFormat = XPDateFormat.GetDateInstance(XPDateFormat.MEDIUM,
         m_ptPageMgr.GetLocale(), m_ptPageMgr.GetTimeZone());
        dateString = myXPDateFormat.Format(myXPDateTime);
        _arrDropDownValueText[i][OPTION_TEXT] = GetMonthName(dateString);
        }
    }
    
    /// <summary>
    /// Fill in the array to create the select drop down component for day with format d
    /// </summary>
    /// <param name="_arrDropDownValueText">text for the dropdown</param>
    private void FillSingleDigitArray(String[][] _arrDropDownValueText)
    {
      _arrDropDownValueText[0][OPTION_VALUE] = EMPTY + DatePicker.BLANK + EMPTY;
      _arrDropDownValueText[0][OPTION_TEXT] = " ";
      for (int i = 1;i < _arrDropDownValueText.Length;i++)
      {
        _arrDropDownValueText[i][OPTION_VALUE] = EMPTY + i + EMPTY;
        _arrDropDownValueText[i][OPTION_TEXT] = EMPTY + i + EMPTY;
      }
    }
    
    /// <summary>
    /// Fill in the array to create the select drop down component for day with format dd
    /// </summary>
    /// <param name="_arrDropDownValueText">text for the dropdown</param>
    private void FillDoubleDigitArray(String[][] _arrDropDownValueText)
    {
      _arrDropDownValueText[0][OPTION_VALUE] = EMPTY + DatePicker.BLANK + EMPTY;
      _arrDropDownValueText[0][OPTION_TEXT] = " ";
      for (int i = 1;i < _arrDropDownValueText.Length;i++)
      {
        _arrDropDownValueText[i][OPTION_VALUE] = EMPTY + i + EMPTY;
        if (i < 10)
        {
          _arrDropDownValueText[i][OPTION_TEXT] = "0" + i + EMPTY;
        } else
        {
          _arrDropDownValueText[i][OPTION_TEXT] = EMPTY + i + EMPTY;
        }
      }
    }
    
    /// <summary>
    /// Fill in the array to create the select drop down component for year
    /// </summary>
    /// <param name="_arrDropDownValueText">text for the year dropdown</param>
    private void FillYearArray(String[][] _arrDropDownValueText)
    {
      _arrDropDownValueText[0][OPTION_VALUE] = EMPTY + DatePicker.BLANK + EMPTY;
      _arrDropDownValueText[0][OPTION_TEXT] = " ";
      for (int i = 1;i < (NUM_OF_YEARS + 1);i++)
      {
        _arrDropDownValueText[i][OPTION_VALUE] = EMPTY + ((i - 1) + MIN_YEAR) + EMPTY;
        _arrDropDownValueText[i][OPTION_TEXT] = EMPTY + ((i - 1) + MIN_YEAR) + EMPTY;
        }
    }
    
    /// <summary>
    /// create the month name drop down menu based on the format given by the locale.
    /// </summary>
    /// <param name="_strDate">the date</param>
    /// <returns>the month name</returns>
    private String GetMonthName(String _strDate)
    {
      XPStringTokenizer myXPStrTokenizer = new XPStringTokenizer(_strDate,
       DELIMETERS);
      for (int i = 0;myXPStrTokenizer.HasMoreTokens();i++)
      {
        if (i >= m_nOrderM)
        {
          return myXPStrTokenizer.GetNextToken();
        } else
        {
          myXPStrTokenizer.GetNextToken();
        }
      }
      return null;
    }
    
    /// <summary>
    /// Reset the index for the order in which the month, year, and day should appear to be between (0-2)
    /// instead of their index position in the date pattern
    /// </summary>
    private void ResetIndex()
    {
      if (m_nOrderM == 0)
      {
        if (m_nOrderd > m_nOrdery)
        {
          m_nOrderd = 2;
          m_nOrdery = 1;
        } else
        {
          m_nOrderd = 1;
          m_nOrdery = 2;
        }
      } else
      if (m_nOrderd == 0)
      {
        if (m_nOrderM > m_nOrdery)
        {
          m_nOrderM = 2;
          m_nOrdery = 1;
        } else
        {
          m_nOrderM = 1;
          m_nOrdery = 2;
        }
      } else
      {
        if (m_nOrderd > m_nOrderM)
        {
          m_nOrderd = 2;
          m_nOrderM = 1;
        } else
        {
          m_nOrderd = 1;
          m_nOrderM = 2;
        }
      }
    }
    
    /// <summary>
    /// ExtractPatternInfo saves information about the date pattern such as the number of M's, and d's and the
    /// index (order) in which they appear in the pattern.  The delimeters between the month, day, and year will
    /// also be saved in an array.
    /// </summary>
    /// <param name="_strPattern">pattern with date format instructions</param>
    private void ExtractPatternInfo(String _strPattern)
    {
      XPMatch month;
      XPMatch day;
      XPMatch year;
      // record the number of M
      month = XPRegex.Match(_strPattern, "M+");
      m_nNumM = month.GetLength();
      m_nOrderM = month.GetIndex();
      // record the number of y
      year = XPRegex.Match(_strPattern, "y+");
      m_nOrdery = year.GetIndex();
      // record the number of d
      day = XPRegex.Match(_strPattern, "d+");
      m_nNumd = day.GetLength();
      m_nOrderd = day.GetIndex();
      //record the delimeters
      m_nDateDelims = XPRegex.Matches(_strPattern, REG_EXP_DELEMETERS);
      ResetIndex();
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new DatePicker();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Returns the name of the PTDatepicker JavaScript variable name. In this
    /// case there isn't one so return null;
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetPTDatepickerVarName()
    {
      return null;
    }
    
    /// <summary>
    /// Returns the name of the date input. In this
    /// case there isn't one so return null;
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetDateInputName()
    {
      return null;
    }
    public override ComponentMgr GetComponentMgr(PageMgr _ptPageMgr)
    {
      JSRegistry ptJSRegistry = JSRegistry.GetRegistry(_ptPageMgr.GetImgSvrBaseURL(),
       _ptPageMgr.GetImageServerConnectionURL());
      return new JSDateTimePickerMgr(ptJSRegistry, new JSDatePickerVersionMgr(),
       _ptPageMgr);
    }
    
    /// <summary>
    /// Return the name of the component manager responsible for this page control
    /// </summary>
    /// <returns>the name of the ComponentMgr for this class</returns>
    public override String GetComponentMgrName()
    {
      return JSDateTimePickerMgr.STR_CLASS_NAME;
    }
    
    /// <summary>
    /// Currently validation on the date/time picker page controls is not implemented.
    /// This method returns null.
    /// </summary>
    public override XPArrayList Validate()
    {
      return null;
    }
    
    /// <summary>
    /// Currently validation on the date/time picker page controls is not implemented.
    /// This method returns null.
    /// </summary>
    public override XPArrayList GetValidators()
    {
      return null;
    }
  }
}