using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using OpenLogService=com.plumtree.openlog.OpenLogService;
using OpenLogger=com.plumtree.openlog.OpenLogger;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.pagecontrols.datepicker;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.uiinfrastructure.pagecontrols.timepicker;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmldatetimepicker
{
  /// <author>DorisL</author>
  public class HTMLDateTimePicker : DatePicker
  {
    public String STR_MVC_CLASS_NAME = "DateTimePicker_D";
    
    /// <summary>
    /// The time-picker part of this component. 
    /// </summary>
    private TimePicker m_tpTimePicker;
    
    /// <summary>
    /// unique instance id 
    /// </summary>
    private String m_strUniqueInstanceID;
    
    /// <summary>
    /// Some static String used by the datetimepicker 
    /// </summary>
    private static String EMPTY = "";
    
    /// <summary>
    /// logger 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmldatetimepicker.HTMLDateTimePicker");
    
    /// <summary>
    /// Empty constructor Creates a new HTMLDateTimePicker
    /// </summary>
    public HTMLDateTimePicker()
    {
    }
    
    /// <summary>
    /// Initialize this HTMLDateTimePicker
    /// </summary>
    /// <param name="_strFormName">
    /// the name of the form where this HTMLDateTimePicker
    /// appears
    /// </param>
    /// <param name="_ptPageMgr">the PageMgr that owns this HTMLDateTimePicker</param>
    /// <param name="_strUniqueInstanceID">a unique id for the HTMLDateTimePicker</param>
    public override void Init(String _strFormName, PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      base.Init(_strFormName, _ptPageMgr, _strUniqueInstanceID);
      m_tpTimePicker = (TimePicker) _ptPageMgr.AddPageControl(TimePicker.STR_MVC_CLASS_NAME);
      m_tpTimePicker.Init(_ptPageMgr, EMPTY, _strUniqueInstanceID);
    }
    
    /// <summary>
    /// Initialize this HTMLDateTimePicker and set it to the default datetime
    /// if that datetime isn't null.
    /// </summary>
    /// <param name="_strFormName">
    /// the name of the form where this HTMLDateTimePicker
    /// appears
    /// </param>
    /// <param name="_dtDefaultDateTime">to set the HTMLDateTimePicker to</param>
    /// <param name="_ptPageMgr">the PageMgr that owns this HTMLDateTimePicker</param>
    /// <param name="_strUniqueInstanceID">a unique id for the HTMLDateTimePicker</param>
    public override void Init(String _strFormName, XPDateTime _dtDefaultDateTime,
     PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      base.Init(_strFormName, _dtDefaultDateTime, _ptPageMgr, _strUniqueInstanceID);
      m_strUniqueInstanceID = _strUniqueInstanceID;
      //	Set the date format object - use thM format
      XPDateFormat dfDateTimeFormat = XPDateFormat.GetDateTimeInstance(XPDateFormat.MEDIUM,
       XPDateFormat.MEDIUM, _ptPageMgr.GetLocale(), _ptPageMgr.GetTimeZone());
      // Get a calendar
      XPCalendar cal = XPCalendar.GetInstance(_ptPageMgr.GetLocale());
      cal.SetTimeZone(_ptPageMgr.GetTimeZone());
      // Check for the 24 hour clock pattern
      bool b24Hour = false;
      String strDateTimePattern = dfDateTimeFormat.ToPattern();
      if (XPStringUtility.IndexOf(strDateTimePattern, H) != -1)
      {
        b24Hour = true;
      }
      if (null != _dtDefaultDateTime)
      {
        cal.SetTime(_dtDefaultDateTime);
        if (b24Hour)
        {
          m_tpTimePicker = (TimePicker) _ptPageMgr.AddPageControl(TimePicker.STR_MVC_CLASS_NAME);
          m_tpTimePicker.Init(cal.Get(XPCalendar.HOUR_OF_DAY), cal.Get(XPCalendar.MINUTE),
           _ptPageMgr, _strUniqueInstanceID);
        } else
        if (cal.Get(XPCalendar.AM_PM) == 0)
        {
          m_tpTimePicker = (TimePicker) _ptPageMgr.AddPageControl(TimePicker.STR_MVC_CLASS_NAME);
          m_tpTimePicker.Init(cal.Get(XPCalendar.HOUR), cal.Get(XPCalendar.MINUTE),
           dfDateTimeFormat.GetAmPmStrings()[0], _ptPageMgr, _strUniqueInstanceID);
          } else
        {
          m_tpTimePicker = (TimePicker) _ptPageMgr.AddPageControl(TimePicker.STR_MVC_CLASS_NAME);
          m_tpTimePicker.Init(cal.Get(XPCalendar.HOUR), cal.Get(XPCalendar.MINUTE),
           dfDateTimeFormat.GetAmPmStrings()[1], _ptPageMgr, _strUniqueInstanceID);
          }
      } else
      {
        m_tpTimePicker = (TimePicker) _ptPageMgr.AddPageControl(TimePicker.STR_MVC_CLASS_NAME);
        m_tpTimePicker.Init(_ptPageMgr, EMPTY, _strUniqueInstanceID);
      }
    }
    
    /// <summary>
    /// This method returns the AM/PM part of the time. If we are using 24-hour
    /// time this method returns null.
    /// </summary>
    /// <returns>The AM/PM part of the time String.</returns>
    public virtual String GetAMPM()
    {
      return m_tpTimePicker.GetAMPM();
    }
    
    /// <summary>
    /// Provides a means of retrieving the HTML Input name for the AMPM portion
    /// of the date picker.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetAMPMInputName()
    {
      return m_tpTimePicker.GetAMPMInputName();
    }
    
    /// <summary>
    /// This method returns the date/time-picker HTML.
    /// </summary>
    /// <returns>The HTMLElementCollection that is the date and time picker.</returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retVal = base.GetComponentHTML();
      retVal.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      retVal.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      retVal.AddInnerHTMLElement(m_tpTimePicker.GetComponentHTML());
      return retVal;
    }
    
    /// <summary>
    /// This function returns an XPDateTime object that represents the date and
    /// time the user has chosen using the date/time-picker HTML control.
    /// </summary>
    /// <returns>XPDateTime</returns>
    public override XPDateTime GetDateTimeAsDateTime()
    {
      XPCalendar myXPCalendar;
      int nHours = 0;
      int nTime;
      String strAMPM = null;
      try
      {
        strAMPM = GetAMPM();
        // check to see if the am/pm is set
        if (strAMPM != null)
        {
          // special check for 12th  hr am/pm setting
          if ((nTime = XPConvert.ToInteger(m_tpTimePicker.GetHours())) == 12)
          {
            // if AM, substract 12
            if (strAMPM.Equals(AM))
            {
              nHours = nTime - 12;
            } else
            {
              nHours = nTime;
            }
          } else
          // check whether need to add 12 hrs depending on if am/pm is set
          if (m_tpTimePicker.GetAMPM().Equals(PM))
          {
            nHours = XPConvert.ToInteger(m_tpTimePicker.GetHours()) + 12;
            } else
          {
            nHours = XPConvert.ToInteger(m_tpTimePicker.GetHours());
          }
        } else
        {
          nHours = XPConvert.ToInteger(m_tpTimePicker.GetHours());
        }
        myXPCalendar = XPCalendar.GetInstance(XPConvert.ToInteger(m_strYear),
         XPConvert.ToInteger(m_strMonth), XPConvert.ToInteger(m_strDay), nHours,
         XPConvert.ToInteger(m_tpTimePicker.GetMinutes()));
      }
      catch (XPException e)
      {
        log.Error(e, "unable to retreive datetime");
        return null;
      }
      return myXPCalendar.GetTime();
    }
    
    /// <summary>
    /// This method returns the date and time as a String.
    /// </summary>
    /// <returns>The date/time string.</returns>
    public virtual String GetDateTimeAsString()
    {
      return GetSelectedDateAsString() + " " + m_tpTimePicker.GetTimeAsEnglishString();
      }
    
    /// <summary>
    /// This method returns the hours part of the time as a String.
    /// </summary>
    /// <returns>The hours.</returns>
    public virtual String GetHours()
    {
      return m_tpTimePicker.GetHours();
    }
    
    /// <summary>
    /// Provides a means of retrieving the HTML Input name for the hours portion
    /// of the date picker.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetHoursInputName()
    {
      return m_tpTimePicker.GetHoursInputName();
    }
    
    /// <summary>
    /// This method returns the minutes part of the time as a String.
    /// </summary>
    /// <returns>The minutes.</returns>
    public virtual String GetMinutes()
    {
      return m_tpTimePicker.GetMinutes();
    }
    
    /// <summary>
    /// Provides a means of retrieving the HTML Input name for the minutes
    /// portion of the date picker.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetMinutesInputName()
    {
      return m_tpTimePicker.GetMinutesInputName();
    }
    
    /// <summary>
    /// This method serves as a getter for "sub-component" HTML. The
    /// date/time-picker is made up of two sub-components: the date-picker and
    /// the time-picker. The constants for component sub-index should be
    /// specified in the component file.
    /// </summary>
    /// <param name="_nSubComponentIndex">
    /// The sub-component index
    /// (N_SUB_COMPONENT_INDEX_DATE_PICKER or
    /// N_SUB_COMPONENT_INDEX_TIME_PICKER).
    /// </param>
    /// <returns>The HTML for the sub-component requested.</returns>
    public override HTMLElementCollection GetSubComponentHTMLByIndex(int _nSubComponentIndex)
    {
      if (_nSubComponentIndex == 0)
      {
        return base.GetComponentHTML();
      } else
      if (_nSubComponentIndex == 1)
      {
        return m_tpTimePicker.GetComponentHTML();
      }
      return null;
    }
    
    /// <summary>
    /// This method returns just the time part of the String.
    /// </summary>
    /// <returns>The time String.</returns>
    public virtual String GetTimeAsString()
    {
      return m_tpTimePicker.GetTimeAsString();
    }
    
    /// <summary>
    /// Stores the user input after the request is sent back.
    /// </summary>
    /// <param name="_htRequestParameters">The XPHashtable of request name/value pairs.</param>
    public override int StoreUserInput(XPHashtable _htRequestParameters)
    {
      int nDateStatus;
      int nTimeStatus;
      nDateStatus = base.StoreUserInput(_htRequestParameters);
      nTimeStatus = m_tpTimePicker.StoreUserInput(_htRequestParameters);
      if (nDateStatus == RepostControl.PAGE_STATUS_INVALID)
      {
        return nDateStatus;
      }
      return nTimeStatus;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLDateTimePicker();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Return the XPDateTime for the specific date picker
    /// </summary>
    /// <param name="_htRequestParameters"></param>
    /// <param name="_strUniqueInstanceID"></param>
    /// <param name="_xpLocale"></param>
    /// <param name="_xpTimeZone"></param>
    /// <param name="boolean">_bDoValidation , set to true if date must be set (pulldowns cannot be empty)</param>
    /// <returns>XPDateTime</returns>
    public static XPDateTime GetDateTime(XPHashtable _htRequestParameters,
     String _strUniqueInstanceID, XPLocale _xpLocale, XPTimeZone _xpTimeZone,
     bool _bDoValidation)
    {
      XPCalendar myCalendar = null;
      String strMonth = STR_BLANK;
      String strDay = STR_BLANK;
      String strYear = STR_BLANK;
      String strHours = null;
      String strMinutes = null;
      String strAMPM = null;
      int nHours = 0;
      int nTime;
      String[] data;
      // get the month
      data = (String[]) _htRequestParameters.GetElement(STR_COMP_BASE_NAME + "_" + MONTH_DROP_DOWN + _strUniqueInstanceID);
      if (data != null)
      {
        strMonth = data[0];
      }
      // get the day
      data = (String[]) _htRequestParameters.GetElement(STR_COMP_BASE_NAME + "_" + DAY_DROP_DOWN + _strUniqueInstanceID);
      if (data != null)
      {
        strDay = data[0];
      }
      // get the year
      data = (String[]) _htRequestParameters.GetElement(STR_COMP_BASE_NAME + "_" + YEAR_DROP_DOWN + _strUniqueInstanceID);
      if (data != null)
      {
        strYear = data[0];
      }
      // get the hour
      data = (String[]) _htRequestParameters.GetElement(TimePicker.HTMLSELECT_HOURS + _strUniqueInstanceID);
      if (data != null)
      {
        strHours = data[0];
      }
      // get the minutes
      data = (String[]) _htRequestParameters.GetElement(TimePicker.HTMLSELECT_MINUTES + _strUniqueInstanceID);
      if (data != null)
      {
        strMinutes = data[0];
      }
      //null values in hours minutes indicate and invalid datetime
      if (null == strHours || null == strMinutes)
      {
        log.Error("strHours, strMinutes cannot be null");
        return null;
      }
      // get am/pm
      data = (String[]) _htRequestParameters.GetElement(TimePicker.HTMLSELECT_AMPM + _strUniqueInstanceID);
      if (data != null && !(data[0].Equals(EMPTY)))
      {
        strAMPM = data[0];
      }
      try
      {
        // check to see if the am/pm is set
        if (null != strAMPM)
        {
          // special check for 12th  hr am/pm setting
          if ((nTime = XPConvert.ToInteger(strHours)) == 12)
          {
            // if AM, substract 12
            if (strAMPM.Equals(AM))
            {
              nHours = nTime - 12;
            } else
            {
              nHours = nTime;
            }
          } else
          // check whether need to add 12 hrs depending on if am/pm is set
          if (strAMPM.Equals(PM))
          {
            nHours = XPConvert.ToInteger(strHours) + 12;
          } else
          {
            nHours = XPConvert.ToInteger(strHours);
          }
        } else
        {
          nHours = XPConvert.ToInteger(strHours);
        }
        if (_bDoValidation)
        {
          //performs validation
          myCalendar = XPCalendar.GetInstance(XPConvert.ToInteger(strYear),
           XPConvert.ToInteger(strMonth), XPConvert.ToInteger(strDay), nHours,
           XPConvert.ToInteger(strMinutes));
        } else
        {
          // if all the fields are blank, then return null;
          if (strMonth.Equals(STR_BLANK) && strDay.Equals(STR_BLANK) && strYear.Equals(STR_BLANK))
          {
            return null;
          }
        }
      }
      catch (XPException e)
      {
        log.Error(e, "unable to get datetime");
        return null;
      }
      // set calendar with correct time zone and locale to return the xpdatetime object
      myCalendar = XPCalendar.GetInstance(_xpTimeZone, _xpLocale);
      myCalendar.Set(XPConvert.ToInteger(strYear), XPConvert.ToInteger(strMonth),
       XPConvert.ToInteger(strDay), nHours, XPConvert.ToInteger(strMinutes));
      return myCalendar.GetTime();
    }
  }
}