using System;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using PTStyleClass=com.plumtree.xpshared.htmlconstructs.PTStyleClass;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <summary>
  /// Check box page control handles display and values of a check box.
  /// </summary>
  /// <author>CaiusV</author>
  public class Button : AHTMLComponent,IPageControl
  {
    
    /// <summary>
    /// logger 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlbuttons.HTMLButton");
    
    /// <summary>
    /// base name constant 
    /// </summary>
    private const String STR_HTML_INPUT_BASE_NAME = "in_bu_pc";
    
    /// <summary>
    /// class name constant 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "Button";
    
    /// <summary>
    /// the HTMLInput object this wraps 
    /// </summary>
    public HTMLInput m_ptHTMLInput;
    
    /// <summary>
    /// Default constructor. Creates a new HTMLCheckBox object.
    /// </summary>
    public Button()
    {
    }
    
    /// <summary>
    /// Initializes a Button object.
    /// </summary>
    /// <param name="_ptPageMgr">PageManager</param>
    /// <param name="_strUniqueInstanceID">Unique Instance ID</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     AActivitySpace _ptActivitySpace)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      m_ptHTMLInput = new HTMLInput(HTMLInputTypes.BUTTON_TYPE, GetHTMLComponentName(),
       "");
    }
    
    /// <summary>
    /// Returns the base name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return STR_HTML_INPUT_BASE_NAME;
    }
    
    /// <summary>
    /// Returns the HTML for this component. 
    /// </summary>
    /// <returns>HTMLElementCollection for this button</returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      m_ptHTMLInput.SetValue(GetHTMLComponentValue());
      retValue.AddInnerHTMLElement(m_ptHTMLInput);
      return retValue;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new Button();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Sets the style class value of this object.
    /// </summary>
    /// <param name="_style">style class value</param>
    public virtual void SetStyleClass(PTStyleClass _style)
    {
      m_ptHTMLInput.SetStyleClass(_style);
    }
    
    /// <summary>
    /// Returns the style class value of this object.
    /// </summary>
    /// <returns>the sytle class value of the object.</returns>
    public virtual String GetStyleClass()
    {
      return m_ptHTMLInput.GetStyleClass();
    }
  }
}