using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using RepostControl=com.plumtree.uiinfrastructure.form.RepostControl;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <summary>
  /// Check box page control handles display and values of a check box.
  /// </summary>
  /// <author>CaiusV</author>
  public class HTMLCheckBox : AHTMLComponent,IPageControl
  {
    
    /// <summary>
    /// logger 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLCheckBox");
    
    /// <summary>
    /// base name constant 
    /// </summary>
    private const String STR_HTML_INPUT_BASE_NAME = "in_cb_pc";
    
    /// <summary>
    /// class name constant 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HTMLCheckBox";
    
    /// <summary>
    /// hold this check box's 'checked' state 
    /// </summary>
    private bool bChecked = false;
    
    /// <summary>
    /// the HTMLInput object this wraps 
    /// </summary>
    public HTMLInput m_ptHTMLInput;
    
    /// <summary>
    /// Default constructor. Creates a new empty HTMLCheckBox object.
    /// </summary>
    public HTMLCheckBox()
    {
    }
    
    /// <summary>
    /// Initializes a new HTMLCheckBox object.
    /// </summary>
    /// <param name="_ptPageMgr">PageManager</param>
    /// <param name="_strUniqueInstanceID">Unique Instance ID</param>
    /// <param name="_strCols">Check Box Value</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     String _strValue)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      SetHTMLComponentValue(_strValue);
      m_ptHTMLInput = new HTMLInput(HTMLInputTypes.CHECKBOX_TYPE, GetHTMLComponentName(),
       "");
    }
    
    /// <summary>
    /// Returns the base name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return STR_HTML_INPUT_BASE_NAME;
    }
    
    /// <summary>
    /// Returns the HTML for this component.
    /// </summary>
    /// <returns></returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      m_ptHTMLInput.SetValue(GetHTMLComponentValue());
      m_ptHTMLInput.SetChecked(bChecked);
      retValue.AddInnerHTMLElement(m_ptHTMLInput);
      retValue.AddInnerHTMLString(GetLabel());
      return retValue;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLCheckBox();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Store the new value of this HTML component. Sets bChecked to true
    /// if the value of this checkbox is present in _htRequestParameters.
    /// </summary>
    /// <param name="_htRequestParameters"></param>
    public override int StoreUserInput(XPHashtable _htRequestParameters)
    {
      String strValue = GetHTMLComponentValue();
      bChecked = false;
      Object oElement = _htRequestParameters.GetElement(GetHTMLComponentName());
      if (oElement != null)
      {
        String[] aStrValues = (String[]) oElement;
        for (int i = 0;i < aStrValues.Length;i++)
        {
          if (strValue != null && strValue.Equals(aStrValues[i]))
          {
            bChecked = true;
          }
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Sets the boolean checked attribute of this checkbox.
    /// </summary>
    /// <param name="isChecked">boolean</param>
    public virtual void SetChecked(bool isChecked)
    {
      bChecked = isChecked;
    }
    
    /// <summary>
    /// Returns the boolean checked attribute of this checkbox
    /// </summary>
    /// <returns>boolean checked</returns>
    public virtual bool IsChecked()
    {
      return bChecked;
    }
  }
}