using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using RepostControl=com.plumtree.uiinfrastructure.form.RepostControl;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <summary>
  /// HTMLCheckBoxGroup class supports grouping of related checkboxes.
  /// </summary>
  /// <author>CaiusV</author>
  public class HTMLCheckBoxGroup : AHTMLComponent,IPageControl
  {
    
    /// <summary>
    /// for logging 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLCheckBoxGroup");
    
    /// <summary>
    /// base name for input 
    /// </summary>
    private const String STR_HTML_INPUT_BASE_NAME = "in_cb_pc";
    
    /// <summary>
    /// this class name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HTMLCheckBoxGroup";
    
    /// <summary>
    /// the array of checkboxes owned by this group 
    /// </summary>
    public XPArrayList m_checkBoxes;
    
    /// <summary>
    /// delimits checkboxes 
    /// </summary>
    protected HTMLElementCollection m_ptDelimiter = null;
    
    /// <summary>
    /// Default constructor, creates a new HTMLCheckBox object
    /// </summary>
    public HTMLCheckBoxGroup()
    {
    }
    
    /// <summary>
    /// Initializes a new ACheckBoxGroup object.
    /// </summary>
    /// <param name="_ptPageMgr">the PageMgr where this object belongs</param>
    /// <param name="_strUniqueInstanceID">id for this obejct</param>
    /// <param name="_strValues">array of values for checkboxes</param>
    /// <param name="__strLabels">array of labels for checkboxes</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     String[] _strValues, String[] _strLabels)
    {
      if (null != _strLabels && _strLabels.Length != _strValues.Length)
      {
        log.Error("Values array not equal to labels array");
        _strLabels = null;
      }
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      m_checkBoxes = new XPArrayList(_strValues.Length);
      for (int i = 0;i < _strValues.Length;i++)
      {
        HTMLCheckBox checkBox = (HTMLCheckBox) _ptPageMgr.AddPageControl(HTMLCheckBox.STR_MVC_CLASS_NAME);
        checkBox.Init(_ptPageMgr, _strUniqueInstanceID, _strValues[i]);
        if (null != _strLabels)
        {
          checkBox.SetLabel(_strLabels[i]);
        }
        m_checkBoxes.Add(checkBox);
      }
    }
    
    /// <summary>
    /// Returns the base name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return STR_HTML_INPUT_BASE_NAME;
    }
    
    /// <summary>
    /// Returns the HTML for this component.
    /// </summary>
    /// <returns></returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      for (int i = 0;i < m_checkBoxes.GetSize();i++)
      {
        HTMLCheckBox checkBox = (HTMLCheckBox) m_checkBoxes.GetElement(i);
        retValue.AddInnerHTMLElement(checkBox.GetComponentHTML());
        if (null != m_ptDelimiter && i < (m_checkBoxes.GetSize() - 1))
        {
          retValue.AddInnerHTMLElement(m_ptDelimiter);
        }
      }
      return retValue;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLCheckBoxGroup();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Store the values of this HTML component.
    /// </summary>
    /// <param name="_htRequestParameters"></param>
    public override int StoreUserInput(XPHashtable _htRequestParameters)
    {
      for (int i = 0;i < m_checkBoxes.GetSize();i++)
      {
        HTMLCheckBox checkBox = (HTMLCheckBox) m_checkBoxes.GetElement(i);
        String strValue = checkBox.GetHTMLComponentValue();
        Object oElement = _htRequestParameters.GetElement(checkBox.GetHTMLComponentName());
        if (oElement != null)
        {
          String[] aStrValues = (String[]) oElement;
          for (int j = 0;j < aStrValues.Length;j++)
          {
            if (strValue != null && strValue.Equals(aStrValues[j]))
            {
              checkBox.SetChecked(true);
            }
          }
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Return an XPArrayList containing the selected values.
    /// </summary>
    /// <returns>XPArrayList of selected values</returns>
    public virtual XPArrayList GetSelectedValues()
    {
      XPArrayList alReturnValue = new XPArrayList();
      for (int i = 0;i < m_checkBoxes.GetSize();i++)
      {
        HTMLCheckBox checkBox = (HTMLCheckBox) m_checkBoxes.GetElement(i);
        if (checkBox.IsChecked())
        {
          String strValue = checkBox.GetHTMLComponentValue();
          alReturnValue.Add(strValue);
        }
      }
      return alReturnValue;
    }
    
    /// <summary>
    /// Set the selected values for this object.
    /// </summary>
    /// <param name="arrSelectedValues"></param>
    public virtual void SetSelectedValues(XPArrayList arrSelectedValues)
    {
      for (int i = 0;i < m_checkBoxes.GetSize();i++)
      {
        HTMLCheckBox checkBox = (HTMLCheckBox) m_checkBoxes.GetElement(i);
        if (arrSelectedValues.Contains(checkBox.GetHTMLComponentValue()))
        {
          checkBox.SetChecked(true);
        }
      }
    }
    
    /// <returns>Returns the m_ptDelimiter.</returns>
    public virtual HTMLElementCollection GetDelimiter()
    {
      return m_ptDelimiter;
    }
    
    /// <param name="delimiter">The m_ptDelimiter to set.</param>
    public virtual void SetDelimiter(HTMLElementCollection delimiter)
    {
      m_ptDelimiter = delimiter;
    }
  }
}