using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using RepostControl=com.plumtree.uiinfrastructure.form.RepostControl;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <summary>
  /// HTMLCheckBoxGroupPlus class supports grouping of related checkboxplus objects.
  /// </summary>
  /// <author>CaiusV</author>
  public class HTMLCheckBoxGroupPlus : AHTMLComponent,IPageControl
  {
    
    /// <summary>
    /// for logging 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLCheckBoxGroupPlus");
    
    /// <summary>
    /// this class name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HTMLCheckBoxGroupPlus";
    
    /// <summary>
    /// base name for input 
    /// </summary>
    private const String STR_HTML_INPUT_BASE_NAME = "in_cb_pc";
    
    /// <summary>
    /// icons to associate with checkboxes 
    /// </summary>
    private HTMLImg[] m_ptIcons;
    
    /// <summary>
    /// ASURLs to associate with checkboxes 
    /// </summary>
    private XPArrayList m_ptURLs;
    
    /// <summary>
    /// Descriptions to associate with checkboxes 
    /// </summary>
    private String[] m_sDescs;
    
    /// <summary>
    /// the array of checkboxes owned by this group 
    /// </summary>
    protected XPArrayList m_checkBoxes;
    
    /// <summary>
    /// Default constructor, creates a new HTMLCheckBox object
    /// </summary>
    public HTMLCheckBoxGroupPlus()
    {
    }
    
    /// <summary>
    /// Initializes a new ACheckBoxGroupPlus object.
    /// </summary>
    /// <param name="_ptPageMgr">the PageMgr responsible for this control</param>
    /// <param name="_strUniqueInstanceID">unique id</param>
    /// <param name="_strValues">the values for the checkboxes</param>
    /// <param name="_strLabels">the labels for the checkboxes</param>
    /// <param name="_ptIcons">icons for the checkboxes</param>
    /// <param name="_ptURLs">ASURLs for the checkboxes</param>
    /// <param name="_sDescs">Descriptions for the checkboxes</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     String[] _strValues, String[] _strLabels, HTMLImg[] _ptIcons, XPArrayList _ptURLs,
     String[] _sDescs)
    {
      if (null != _ptIcons && _ptIcons.Length != _strValues.Length)
      {
        log.Error("Values array not equal to icons array");
        _ptIcons = null;
      }
      if (null != _ptURLs && _ptURLs.GetSize() != _strValues.Length)
      {
        log.Error("Values array not equal to links array");
        _sDescs = null;
      }
      if (null != _sDescs && _sDescs.Length != _strValues.Length)
      {
        log.Error("Values array not equal to descriptions array");
        _sDescs = null;
      }
      m_ptURLs = _ptURLs;
      m_ptIcons = _ptIcons;
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      m_checkBoxes = new XPArrayList(_strValues.Length);
      for (int i = 0;i < _strValues.Length;i++)
      {
        HTMLCheckBoxPlus checkBoxPlus = (HTMLCheckBoxPlus) _ptPageMgr.AddPageControl(HTMLCheckBoxPlus.STR_MVC_CLASS_NAME);
        checkBoxPlus.Init(_ptPageMgr, _strUniqueInstanceID, _strValues[i]);
        if (null != _strLabels)
        {
          checkBoxPlus.SetLabel(_strLabels[i]);
        }
        if (null != _ptIcons)
        {
          checkBoxPlus.SetIcon(_ptIcons[i]);
        }
        if (null != _ptURLs)
        {
          checkBoxPlus.SetURL((ASURL) _ptURLs.GetElement(i));
        }
        if (null != _sDescs)
        {
          checkBoxPlus.SetDescription(_sDescs[i]);
        }
        m_checkBoxes.Add(checkBoxPlus);
      }
    }
    
    /// <summary>
    /// Returns the HTML for this component. This is not a method that is often used.
    /// Rather, iterate over each CheckBoxPlus object and render them -- this way 
    /// offers more flexible display options.
    /// </summary>
    /// <returns>HTMLElementCollection for display in a view</returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      for (int i = 0;i < m_checkBoxes.GetSize();i++)
      {
        retValue.AddInnerHTMLElement(GetElement(i).GetComponentHTML());
      }
      return retValue;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLCheckBoxGroupPlus();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Return the number of CheckBoxPlus objects for this group.
    /// </summary>
    /// <returns>the number of CheckBoxPlus objects</returns>
    public virtual int GetSize()
    {
      return m_checkBoxes.GetSize();
    }
    
    /// <summary>
    /// Return a specific CheckBoxPlus object by index.
    /// </summary>
    /// <param name="_iIndex">the index</param>
    /// <returns>the CheckBoxPlus object at the index.</returns>
    public virtual HTMLCheckBoxPlus GetElement(int _iIndex)
    {
      return (HTMLCheckBoxPlus) m_checkBoxes.GetElement(_iIndex);
    }
    
    /// <summary>
    /// Store the values of this HTML component.
    /// </summary>
    /// <param name="_htRequestParameters"></param>
    public override int StoreUserInput(XPHashtable _htRequestParameters)
    {
      for (int i = 0;i < m_checkBoxes.GetSize();i++)
      {
        HTMLCheckBoxPlus checkBoxPlus = (HTMLCheckBoxPlus) m_checkBoxes.GetElement(i);
        String strValue = checkBoxPlus.GetHTMLComponentValue();
        Object oElement = _htRequestParameters.GetElement(checkBoxPlus.GetHTMLComponentName());
        if (oElement != null)
        {
          String[] aStrValues = (String[]) oElement;
          for (int j = 0;j < aStrValues.Length;j++)
          {
            if (strValue != null && strValue.Equals(aStrValues[j]))
            {
              checkBoxPlus.SetChecked(true);
            }
          }
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Return an XPArrayList containing the selected values.
    /// </summary>
    /// <returns>XPArrayList of selected values</returns>
    public virtual XPArrayList GetSelectedValues()
    {
      XPArrayList alReturnValue = new XPArrayList();
      for (int i = 0;i < m_checkBoxes.GetSize();i++)
      {
        HTMLCheckBoxPlus checkBox = (HTMLCheckBoxPlus) m_checkBoxes.GetElement(i);
        if (checkBox.IsChecked())
        {
          String strValue = checkBox.GetHTMLComponentValue();
          alReturnValue.Add(strValue);
        }
      }
      return alReturnValue;
    }
    
    /// <summary>
    /// Returns the base name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return STR_HTML_INPUT_BASE_NAME;
    }
  }
}