using System;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <summary>
  /// Check box page control handles display and values of a check box.
  /// </summary>
  /// <author>CaiusV</author>
  public class HTMLCheckBoxPlus : AHTMLComponent,IPageControl
  {
    
    /// <summary>
    /// logger 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLCheckBoxPlus");
    
    /// <summary>
    /// base name constant 
    /// </summary>
    private const String STR_HTML_INPUT_BASE_NAME = "in_cb_pc";
    
    /// <summary>
    /// class name constant 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HTMLCheckBoxPlus";
    
    /// <summary>
    /// icon to associate with checkbox 
    /// </summary>
    private HTMLImg m_ptIcon;
    
    /// <summary>
    /// ASURL to associate with checkbox 
    /// </summary>
    private ASURL m_ptURL;
    
    /// <summary>
    /// Description to associate with checkbox 
    /// </summary>
    private String m_sDescription;
    
    /// <summary>
    /// hold this check box's 'checked' state 
    /// </summary>
    protected bool bChecked = false;
    
    /// <summary>
    /// the HTMLInput object this wraps 
    /// </summary>
    public HTMLInput m_ptHTMLInput;
    
    /// <summary>
    /// Default constructor. Creates a new HTMLCheckBox object.
    /// </summary>
    public HTMLCheckBoxPlus()
    {
    }
    
    /// <summary>
    /// Initializes a new HTMLCheckBoxPlus object.
    /// </summary>
    /// <param name="_ptPageMgr">PageManager</param>
    /// <param name="_strUniqueInstanceID">Unique Instance ID</param>
    /// <param name="_strCols">Check Box Value</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     String _strValue)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      SetHTMLComponentValue(_strValue);
      m_ptHTMLInput = new HTMLInput(HTMLInputTypes.CHECKBOX_TYPE, GetHTMLComponentName(),
       "");
    }
    
    /// <summary>
    /// Returns the base name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return STR_HTML_INPUT_BASE_NAME;
    }
    
    /// <summary>
    /// Returns the HTML for this component.
    /// </summary>
    /// <returns></returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      m_ptHTMLInput.SetValue(GetHTMLComponentValue());
      m_ptHTMLInput.SetChecked(bChecked);
      retValue.AddInnerHTMLElement(m_ptHTMLInput);
      return retValue;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLCheckBoxPlus();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <returns>Returns the m_ptIcon.</returns>
    public virtual HTMLImg GetIcon()
    {
      return m_ptIcon;
    }
    
    /// <param name="icon">The m_ptIcon to set.</param>
    public virtual void SetIcon(HTMLImg icon)
    {
      m_ptIcon = icon;
    }
    
    /// <returns>Returns the m_ptURL.</returns>
    public virtual ASURL GetURL()
    {
      return m_ptURL;
    }
    
    /// <param name="m_pturl">The m_ptURL to set.</param>
    public virtual void SetURL(ASURL m_pturl)
    {
      m_ptURL = m_pturl;
    }
    
    /// <returns>Returns the m_sDescription.</returns>
    public virtual String GetDescription()
    {
      return m_sDescription;
    }
    
    /// <param name="description">The m_sDescription to set.</param>
    public virtual void SetDescription(String description)
    {
      m_sDescription = description;
    }
    
    /// <summary>
    /// Sets the boolean checked attribute of this checkbox.
    /// </summary>
    /// <param name="isChecked">boolean</param>
    public virtual void SetChecked(bool isChecked)
    {
      bChecked = isChecked;
    }
    
    /// <summary>
    /// Returns the boolean checked attribute of this checkbox.
    /// </summary>
    /// <param name="isChecked">boolean</param>
    public virtual bool IsChecked()
    {
      return bChecked;
    }
  }
}