using System;
using XPResourceManager=com.plumtree.openfoundation.internationalization.XPResourceManager;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using IPTGrowableSortedArrayWrapperRO=com.plumtree.uiinfrastructure.arraywrapper.IPTGrowableSortedArrayWrapperRO;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <summary>
  /// HTMLNameDescLang page control handles display and values of a page control
  /// made up of a text box text area and a select dropdown. The select is
  /// populated with default language options. This is a common construct
  /// throughout the admin ui.
  /// </summary>
  /// <author>CaiusV</author>
  public class HTMLNameDescLang : AHTMLComponent,IPageControl
  {
    
    /// <summary>
    /// for logging 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLNameDescLang");
    
    /// <summary>
    /// base name constant 
    /// </summary>
    private const String STR_HTML_INPUT_BASE_NAME = "in_tx_pc";
    
    /// <summary>
    /// class name constant 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HTMLNameDescLang";
    
    /// <summary>
    /// the textbox for this object 
    /// </summary>
    public HTMLTextBox m_TextBox;
    
    /// <summary>
    /// the selectdropdown for this object 
    /// </summary>
    public HTMLSelectDropDown m_SelectDropDown;
    
    /// <summary>
    /// the textarea for this object 
    /// </summary>
    public HTMLTextArea m_TextArea;
    
    /// <summary>
    /// primary language 
    /// </summary>
    private String m_strPrimaryLang;
    
    /// <summary>
    /// stores already selected langs 
    /// </summary>
    private IPTGrowableSortedArrayWrapperRO m_ptGSArrayWrapper;
    
    /// <summary>
    /// array of language options 
    /// </summary>
    private XPArrayList m_alOptions;
    
    /// <summary>
    /// Default constructor. Creates a new HTMLTextAndDropDown object.
    /// </summary>
    public HTMLNameDescLang()
    {
    }
    
    /// <summary>
    /// Initializes a new HTMLNameDescLang object. Creates and initializes
    /// the HTMLTextBox, HTMLTextArea and HTMLSelectDropdown objects 
    /// contained by this object. Populates the default languages for the
    /// select.
    /// </summary>
    /// <param name="_ptPageMgr">The component manager for this component</param>
    /// <param name="_strUniqueInstanceID">The unique ID for this component.</param>
    /// <param name="_arrDropDownValueText">The dropdown values</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     XPResourceManager _ptResourceManager, String _strPrimaryLang)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      // the primary language
      m_strPrimaryLang = _strPrimaryLang;
      // set up the textbox page control
      m_TextBox = (HTMLTextBox) _ptPageMgr.AddPageControl(HTMLTextBox.STR_MVC_CLASS_NAME);
      m_TextBox.Init(_ptPageMgr, _strUniqueInstanceID);
      m_SelectDropDown = (HTMLSelectDropDown) _ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
      // set up the dropdown page control	
      SetLangChoices(_ptResourceManager);
      m_SelectDropDown.Init(_ptPageMgr, _strUniqueInstanceID, m_alOptions,
       "");
      // set up the htmltextarea page control
      m_TextArea = (HTMLTextArea) m_ptPageMgr.AddPageControl(HTMLTextArea.STR_MVC_CLASS_NAME);
      m_TextArea.Init(m_ptPageMgr, _strUniqueInstanceID, 50, 3);
    }
    
    /// <summary>
    /// Returns the base name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return STR_HTML_INPUT_BASE_NAME;
    }
    
    /// <summary>
    /// Returns the HTML for this component.
    /// </summary>
    /// <returns>HTML for this object</returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      if (null != GetHTMLComponentStyleClass())
      {
        m_TextBox.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
        m_SelectDropDown.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
        m_TextArea.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
        }
      retValue.AddInnerHTMLElement(m_TextBox.GetComponentHTML());
      retValue.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      retValue.AddInnerHTMLElement(m_TextArea.GetComponentHTML());
      retValue.AddInnerHTMLString(CommonHTMLStrings.SPACE);
      retValue.AddInnerHTMLElement(m_SelectDropDown.GetComponentHTML());
      return retValue;
    }
    
    /// <summary>
    /// Return the component values of the TextBox and SelectDropdown
    /// separated by a space.
    /// </summary>
    public override String GetHTMLComponentValue()
    {
      return m_TextBox.GetHTMLComponentValue() + " " + m_SelectDropDown.GetHTMLComponentValue() + " " + m_TextArea.GetHTMLComponentValue();
      }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLNameDescLang();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// For every Validator associated with this object execute it's Validate method.
    /// Return an Arraylist of UIError objects generated from executing the Validators.
    /// Return and empty Arraylist if no problems were found.
    /// </summary>
    /// <returns>Arraylist of UIErrors.</returns>
    public override XPArrayList Validate()
    {
      XPArrayList myErrorList = base.Validate();
      if (m_TextBox.Validate().GetSize() > 0)
      {
        myErrorList.Add(m_TextBox.Validate().GetElement(0));
      }
      if (m_TextArea.Validate().GetSize() > 0)
      {
        myErrorList.Add(m_TextArea.Validate().GetElement(0));
      }
      if (m_SelectDropDown.Validate().GetSize() > 0)
      {
        myErrorList.Add(m_SelectDropDown.Validate().GetElement(0));
      }
      return myErrorList;
    }
    
    /// <summary>
    /// Return a string made up of all the JSValidationCode for each of the
    /// Validator's associated with this component.
    /// </summary>
    /// <returns>JavaScript validation code</returns>
    public override String GetJSValidationCode()
    {
      XPStringBuilder sb = new XPStringBuilder(1250);
      sb.Append(base.GetJSValidationCode());
      sb.Append(m_TextBox.GetJSValidationCode());
      sb.Append(m_TextArea.GetJSValidationCode());
      sb.Append(m_SelectDropDown.GetJSValidationCode());
      return sb.ToString();
    }
    
    /// <summary>
    /// Set the options of the member variable m_SelectDropDown with the appropriate
    /// languages.
    /// </summary>
    /// <param name="_ptResourceManager">XPResourceManager with language options</param>
    public virtual void SetLangChoices(XPResourceManager _ptResourceManager)
    {
      XPArrayList usedLangs = new XPArrayList();
      m_alOptions = new XPArrayList();
      usedLangs.Add(m_strPrimaryLang);
      if (null != m_ptGSArrayWrapper)
      {
        for (int i = 0;i < m_ptGSArrayWrapper.GetCount();++i)
        {
          String strDisallowedLang = (String) m_ptGSArrayWrapper.GetItem(i,
           3);
          strDisallowedLang = XPStringUtility.ToLowerCase(strDisallowedLang);
          usedLangs.Add(strDisallowedLang);
        }
      }
      String[] asKnownLangs = _ptResourceManager.GetSupportedLanguages();
      int size = asKnownLangs.Length;
      for (int i = 0;i < size;i++)
      {
        if (!usedLangs.Contains(asKnownLangs[i]))
        {
          HTMLOption myOption = new HTMLOption(asKnownLangs[i]);
          myOption.AddInnerHTMLString(_ptResourceManager.GetDisplayLocaleFromLocaleID(asKnownLangs[i],
           m_ptPageMgr.GetLangID()));
          m_alOptions.Add(myOption);
        }
      }
      m_SelectDropDown.SetOptions(m_alOptions);
    }
    
    /// <summary>
    /// Returns member variable m_ptGSArrayWrapper.
    /// </summary>
    /// <returns>m_ptGSArrayWrapper member variable</returns>
    public virtual IPTGrowableSortedArrayWrapperRO GetData()
    {
      return m_ptGSArrayWrapper;
    }
    
    /// <summary>
    /// Sets the member variable m_ptGSArrayWrapper
    /// </summary>
    /// <param name="data"></param>
    public virtual void SetData(IPTGrowableSortedArrayWrapperRO data)
    {
      m_ptGSArrayWrapper = data;
    }
    
    /// <summary>
    /// Set the primary language member variable.
    /// </summary>
    /// <param name="_sPrimaryLang">the primary language</param>
    public virtual void SetPrimaryLanguage(String _sPrimaryLang)
    {
      m_strPrimaryLang = _sPrimaryLang;
    }
  }
}