using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using RepostControl=com.plumtree.uiinfrastructure.form.RepostControl;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <summary>
  /// HTMLRadioButtonGroup class supports grouping of related radio buttons.
  /// </summary>
  /// <author>CaiusV</author>
  public class HTMLRadioButtonGroup : AHTMLComponent,IPageControl
  {
    
    /// <summary>
    /// for logging 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLRadioButtonGroup");
    
    /// <summary>
    /// base name for input 
    /// </summary>
    private const String STR_HTML_INPUT_BASE_NAME = "in_ra_pc";
    
    /// <summary>
    /// this class name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HTMLRadioButtonGroup";
    
    /// <summary>
    /// the array of ARadioButtons owned by this group 
    /// </summary>
    protected XPArrayList m_radioButtons;
    
    /// <summary>
    /// delimits checkboxes 
    /// </summary>
    private HTMLElementCollection m_ptDelimiter = null;
    
    /// <summary>
    /// Default constructor. Creates a new HTMLRadioButtonGroup object.
    /// </summary>
    public HTMLRadioButtonGroup()
    {
    }
    
    /// <summary>
    /// Initializes a new HTMLRadioButtonGroup object made up of radio buttons with values
    /// passed in as the _arrValues argument.
    /// </summary>
    /// <param name="_ptPageMgr">PageManager</param>
    /// <param name="_strUniqueInstanceID">Unique Instance ID</param>
    /// <param name="_arrValues">values for radio buttons</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     String[] _arrValues)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      m_radioButtons = new XPArrayList(_arrValues.Length);
      for (int i = 0;i < _arrValues.Length;i++)
      {
        HTMLRadioButton radioButton = (HTMLRadioButton) _ptPageMgr.AddPageControl(HTMLRadioButton.STR_MVC_CLASS_NAME);
        radioButton.Init(_ptPageMgr, _strUniqueInstanceID, _arrValues[i]);
        m_radioButtons.Add(radioButton);
      }
    }
    
    /// <summary>
    /// Returns the base name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return STR_HTML_INPUT_BASE_NAME;
    }
    
    /// <summary>
    /// Returns the HTML for this component.
    /// </summary>
    /// <returns></returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      for (int i = 0;i < m_radioButtons.GetSize();i++)
      {
        HTMLRadioButton radioButton = (HTMLRadioButton) m_radioButtons.GetElement(i);
        retValue.AddInnerHTMLElement(radioButton.GetComponentHTML());
        if (null != m_ptDelimiter && i < (m_radioButtons.GetSize() - 1))
        {
          retValue.AddInnerHTMLElement(m_ptDelimiter);
        }
      }
      return retValue;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLRadioButtonGroup();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Store the values of this HTML component.
    /// </summary>
    /// <param name="_htRequestParameters"></param>
    public override int StoreUserInput(XPHashtable _htRequestParameters)
    {
      for (int i = 0;i < m_radioButtons.GetSize();i++)
      {
        HTMLRadioButton radioButton = (HTMLRadioButton) m_radioButtons.GetElement(i);
        String strValue = radioButton.GetHTMLComponentValue();
        bool bChecked = false;
        Object oElement = _htRequestParameters.GetElement(radioButton.GetHTMLComponentName());
        if (oElement != null)
        {
          String[] aStrValues = (String[]) oElement;
          for (int j = 0;j < aStrValues.Length;j++)
          {
            if (strValue != null && strValue.Equals(aStrValues[j]))
            {
              radioButton.SetChecked(true);
            }
          }
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Return the currently selected value of the radio button group.
    /// </summary>
    /// <returns>XPArrayList of selected values</returns>
    public virtual String GetSelectedValue()
    {
      for (int i = 0;i < m_radioButtons.GetSize();i++)
      {
        HTMLRadioButton radioButton = (HTMLRadioButton) m_radioButtons.GetElement(i);
        if (radioButton.IsChecked())
        {
          return radioButton.GetHTMLComponentValue();
        }
      }
      return null;
    }
    
    /// <summary>
    /// Set the currently selected value of the radio button group.
    /// </summary>
    /// <returns>XPArrayList of selected values</returns>
    public virtual void SetSelectedValue(String strSelectedValue)
    {
      for (int i = 0;i < m_radioButtons.GetSize();i++)
      {
        HTMLRadioButton radioButton = (HTMLRadioButton) m_radioButtons.GetElement(i);
        if (strSelectedValue.Equals(radioButton.GetHTMLComponentValue()))
        {
          radioButton.SetChecked(true);
        }
      }
    }
    
    /// <returns>Returns the m_ptDelimiter.</returns>
    public virtual HTMLElementCollection GetDelimiter()
    {
      return m_ptDelimiter;
    }
    
    /// <param name="delimiter">The m_ptDelimiter to set.</param>
    public virtual void SetDelimiter(HTMLElementCollection delimiter)
    {
      m_ptDelimiter = delimiter;
    }
  }
}