using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using RepostControl=com.plumtree.uiinfrastructure.form.RepostControl;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <summary>
  /// HTMLRadioButtonGroupPlus class supports grouping of related radiobuttons.
  /// </summary>
  /// <author>CaiusV</author>
  public class HTMLRadioButtonGroupPlus : AHTMLComponent,IPageControl
  {
    
    /// <summary>
    /// for logging 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLRadioButtonGroupPlus");
    
    /// <summary>
    /// this class name 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HTMLRadioButtonGroupPlus";
    
    /// <summary>
    /// base name for input 
    /// </summary>
    private const String STR_HTML_INPUT_BASE_NAME = "in_ra_pc";
    
    /// <summary>
    /// icons to associate with radiobuttons 
    /// </summary>
    private HTMLImg[] m_ptIcons;
    
    /// <summary>
    /// ASURLs to associate with radiobuttons 
    /// </summary>
    private XPArrayList m_ptURLs;
    
    /// <summary>
    /// Descriptions to associate with radiobuttons 
    /// </summary>
    private String[] m_sDescs;
    
    /// <summary>
    /// the array of ARadioButtons owned by this group 
    /// </summary>
    protected XPArrayList m_radioButtons;
    
    /// <summary>
    /// Default constructor, creates a new HTMLRadioButton object
    /// </summary>
    public HTMLRadioButtonGroupPlus()
    {
    }
    
    /// <summary>
    /// Initializes a new ARadioButtonGroupPlus object. Assign values, labels, icons
    /// ASURLs and descriptions by passing in the appropriate arrays.
    /// </summary>
    /// <param name="_ptPageMgr">PageManager</param>
    /// <param name="_strUniqueInstanceID">Unique Instance ID</param>
    /// <param name="_strValues">values for the radio buttons</param>
    /// <param name="_strLabels">labels for the radio buttons</param>
    /// <param name="_ptIcons">icons  for the radio buttons</param>
    /// <param name="_ptURLs">ASURLs for the radio buttons</param>
    /// <param name="_sDescs">text descriptions for the radio buttons</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     String[] _strValues, String[] _strLabels, HTMLImg[] _ptIcons, XPArrayList _ptURLs,
     String[] _sDescs)
    {
      if (null != _ptIcons && _ptIcons.Length != _strValues.Length)
      {
        log.Error("Values array not equal to icons array");
        _ptIcons = null;
      }
      if (null != _ptURLs && _ptURLs.GetSize() != _strValues.Length)
      {
        log.Error("Values array not equal to links array");
        _sDescs = null;
      }
      if (null != _sDescs && _sDescs.Length != _strValues.Length)
      {
        log.Error("Values array not equal to descriptions array");
        _sDescs = null;
      }
      m_ptURLs = _ptURLs;
      m_ptIcons = _ptIcons;
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      m_radioButtons = new XPArrayList(_strValues.Length);
      for (int i = 0;i < _strValues.Length;i++)
      {
        HTMLRadioButtonPlus radioPlus = (HTMLRadioButtonPlus) _ptPageMgr.AddPageControl(HTMLRadioButtonPlus.STR_MVC_CLASS_NAME);
        radioPlus.Init(_ptPageMgr, _strUniqueInstanceID, _strValues[i]);
        if (null != _strLabels)
        {
          radioPlus.SetLabel(_strLabels[i]);
        }
        if (null != _ptIcons)
        {
          radioPlus.SetIcon(_ptIcons[i]);
        }
        if (null != _ptURLs)
        {
          radioPlus.SetURL((ASURL) _ptURLs.GetElement(i));
        }
        if (null != _sDescs)
        {
          radioPlus.SetDescription(_sDescs[i]);
        }
        m_radioButtons.Add(radioPlus);
      }
    }
    
    /// <summary>
    /// Returns the HTML for this component.
    /// </summary>
    /// <returns></returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      for (int i = 0;i < m_radioButtons.GetSize();i++)
      {
        retValue.AddInnerHTMLElement(GetElement(i).GetComponentHTML());
      }
      return retValue;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLRadioButtonGroupPlus();
    }
    
    /// <summary>
    /// Store the values of this HTML component.
    /// </summary>
    /// <param name="_htRequestParameters"></param>
    public override int StoreUserInput(XPHashtable _htRequestParameters)
    {
      for (int i = 0;i < m_radioButtons.GetSize();i++)
      {
        HTMLRadioButtonPlus radioButtonPlus = (HTMLRadioButtonPlus) m_radioButtons.GetElement(i);
        String strValue = radioButtonPlus.GetHTMLComponentValue();
        bool bChecked = false;
        Object oElement = _htRequestParameters.GetElement(radioButtonPlus.GetHTMLComponentName());
        if (oElement != null)
        {
          String[] aStrValues = (String[]) oElement;
          for (int j = 0;j < aStrValues.Length;j++)
          {
            if (strValue != null && strValue.Equals(aStrValues[j]))
            {
              radioButtonPlus.SetChecked(true);
            }
          }
        }
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// Return the currently selected value of the radio button group.
    /// </summary>
    /// <returns>XPArrayList of selected values</returns>
    public virtual String GetSelectedValue()
    {
      for (int i = 0;i < m_radioButtons.GetSize();i++)
      {
        HTMLRadioButtonPlus radioButtonPlus = (HTMLRadioButtonPlus) m_radioButtons.GetElement(i);
        if (radioButtonPlus.IsChecked())
        {
          return radioButtonPlus.GetHTMLComponentValue();
        }
      }
      return null;
    }
    
    /// <summary>
    /// Set the currently selected value of the radio button group.
    /// </summary>
    /// <returns>XPArrayList of selected values</returns>
    public virtual void SetSelectedValue(String strSelectedValue)
    {
      for (int i = 0;i < m_radioButtons.GetSize();i++)
      {
        HTMLRadioButtonPlus radioButtonPlus = (HTMLRadioButtonPlus) m_radioButtons.GetElement(i);
        if (strSelectedValue.Equals(radioButtonPlus.GetHTMLComponentValue()))
        {
          radioButtonPlus.SetChecked(true);
        }
      }
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Return the number of RadioButtons in this group
    /// </summary>
    /// <returns>number of RadioButtons</returns>
    public virtual int GetSize()
    {
      return m_radioButtons.GetSize();
    }
    
    /// <summary>
    /// Return the RadioButtonPlus object at the given index.
    /// </summary>
    /// <param name="_iIndex">the index of the returned button</param>
    /// <returns>the RadioButtonPlus object at the given index</returns>
    public virtual HTMLRadioButtonPlus GetElement(int _iIndex)
    {
      return (HTMLRadioButtonPlus) m_radioButtons.GetElement(_iIndex);
    }
    
    /// <summary>
    /// Returns the base name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return STR_HTML_INPUT_BASE_NAME;
    }
  }
}