using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <summary>
  /// JF- Page Control for HTML Select Drop Down.
  /// </summary>
  /// <author>Julie Fournier</author>
  public class HTMLSelectDropDown : AHTMLComponent,IPageControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLSelectDropDown");
    
    /// <summary>
    /// JF- Index constant for array of options: option value 
    /// </summary>
    public const int OPTION_VALUE = 0;
    
    /// <summary>
    /// JF- Index constant for array of options: text to display on option 
    /// </summary>
    public const int OPTION_TEXT = 1;
    public const String STR_MVC_CLASS_NAME = "HTMLSelectDropDown";
    
    /// <summary>
    /// JF- Array of option value/text to populate the drop down menu. 
    /// </summary>
    private String[][] m_arrDropDownValueText;
    
    /// <summary>
    /// array of HTMLOptions to populate the drop down menu 
    /// </summary>
    private XPArrayList m_alOptions;
    
    /// <summary>
    /// the title describing the htmlselectdropdown menu 
    /// </summary>
    private String m_strTitle;
    
    /// <summary>
    /// the HTMLSelect that this object wraps 
    /// </summary>
    internal HTMLSelect htmlSelect;
    public HTMLSelectDropDown()
    {
    }
    
    /// <summary>
    /// JF- Creates a new HTMLSelectDropDown object.
    /// </summary>
    /// <param name="_ptPageMgr">PageManager</param>
    /// <param name="_strUniqueInstanceID">Unique Instance ID</param>
    /// <param name="_arrDropDownValueText">array of names and values for select dropdown</param>
    /// <param name="_strTitle">title of dropdown</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     String[][] _arrDropDownValueText, String _strTitle)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      if (null == _arrDropDownValueText)
      {
        log.Error("_arrDropDownValueText cannot be null");
      }
      m_arrDropDownValueText = _arrDropDownValueText;
      m_strTitle = _strTitle;
    }
    
    /// <summary>
    /// Creates a new HTMLSelectDropDown object.
    /// </summary>
    /// <param name="_ptPageMgr">PageManager</param>
    /// <param name="_strUniqueInstanceID">Unique Instance ID</param>
    /// <param name="_alOptions">XPArrayList of options for the select to use</param>
    /// <param name="_strTitle">title of dropdown</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     XPArrayList _alOptions, String _strTitle)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      if (null == _alOptions)
      {
        log.Error("_alOptions cannot be null");
      }
      m_alOptions = _alOptions;
      m_strTitle = _strTitle;
    }
    
    /// <summary>
    /// JF- Returns the base name of this HTML OpenComponent.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return "in_se_pc";
    }
    
    /// <summary>
    /// JF- Returns the HTML for this component. The names and values of the options
    /// in the select dropdown are either specified by the array argument in the constructor
    /// or by the member variable m_alOptions which may be set with the SetOptions method, or
    /// initailized in the alternate constructor.
    /// </summary>
    /// <returns>HTMLElementCollection</returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      XPArrayList myOptions = new XPArrayList();
      HTMLElementCollection retValue = new HTMLElementCollection();
      HTMLSelect htmlSelect = new HTMLSelect(GetHTMLComponentName(), "");
      htmlSelect.SetSize(GetHTMLComponentSize());
      htmlSelect.SetTitle(m_strTitle);
      if (null != GetHTMLComponentStyleClass())
      {
        htmlSelect.SetStyleClass(GetHTMLComponentStyleClass());
      }
      HTMLOption myOption;
      // either populate the select with the contents of the String[][] 
      // m_arrDropDownValueText or the XPArrayList m_alOptions
      if (null != m_alOptions)
      {
        for (int i = 0;i < m_alOptions.GetSize();i++)
        {
          myOption = (HTMLOption) m_alOptions.GetElement(i);
          htmlSelect.AddInnerHTMLElement(myOption);
          myOptions.Add(myOption);
        }
      } else
      {
        for (int i = 0;i < m_arrDropDownValueText.Length;i++)
        {
          myOption = new HTMLOption(m_arrDropDownValueText[i][OPTION_VALUE]);
          myOption.AddInnerHTMLString(m_arrDropDownValueText[i][OPTION_TEXT]);
          htmlSelect.AddInnerHTMLElement(myOption);
          myOptions.Add(myOption);
        }
      }
      // if the current value is one of the options then set it
      String myCurrentValue = GetHTMLComponentValue();
      for (int i = 0;i < myOptions.GetSize();i++)
      {
        myOption = (HTMLOption) myOptions.GetElement(i);
        if (XPStringUtility.Compare(myOption.GetValue(), GetHTMLComponentValue()) == 0)
        {
          htmlSelect.SetValue(GetHTMLComponentValue());
        }
      }
      htmlSelect.SetDoSort(true);
      retValue.AddInnerHTMLElement(htmlSelect);
      return retValue;
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLSelectDropDown();
    }
    
    /// <param name="options">The m_alOptions to set.</param>
    public virtual void SetOptions(XPArrayList _alOptions)
    {
      m_alOptions = _alOptions;
    }
  }
}