using System;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <summary>
  /// HTMLTextAndDropDown page control handles display and values of a page control
  /// made up of a text box and a select dropdown.
  /// </summary>
  /// <author>CaiusV</author>
  public class HTMLTextAndDropDown : AHTMLComponent,IPageControl
  {
    
    /// <summary>
    /// for logging 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLTextAndDropDown");
    
    /// <summary>
    /// base name constant 
    /// </summary>
    private const String STR_HTML_INPUT_BASE_NAME = "in_tx_pc";
    
    /// <summary>
    /// class name constant 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HTMLTextAndDropDown";
    
    /// <summary>
    /// the textbox for this object 
    /// </summary>
    public HTMLTextBox m_TextBox;
    
    /// <summary>
    /// the selectdropdown for this object 
    /// </summary>
    public HTMLSelectDropDown m_SelectDropDown;
    
    /// <summary>
    /// Default constructor. Creates a new HTMLTextAndDropDown object.
    /// </summary>
    public HTMLTextAndDropDown()
    {
    }
    
    /// <summary>
    /// Initializes a new HTMLTextAndDropDown object. Creates and initializes
    /// the HTMLTextBox and HTMLSelectDropdown objects contained by this object.
    /// </summary>
    /// <param name="_ptPageMgr">The component manager for this component</param>
    /// <param name="_strUniqueInstanceID">The unique ID for this component.</param>
    /// <param name="_arrDropDownValueText">The dropdown values</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     String[][] _arrDropDownValueText)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      m_TextBox = (HTMLTextBox) _ptPageMgr.AddPageControl(HTMLTextBox.STR_MVC_CLASS_NAME);
      m_TextBox.Init(_ptPageMgr, _strUniqueInstanceID);
      m_SelectDropDown = (HTMLSelectDropDown) _ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
      m_SelectDropDown.Init(_ptPageMgr, _strUniqueInstanceID, _arrDropDownValueText,
       "foo");
    }
    
    /// <summary>
    /// Returns the base name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return STR_HTML_INPUT_BASE_NAME;
    }
    
    /// <summary>
    /// Returns the HTML for this component.
    /// </summary>
    /// <returns>HTML for this object</returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      retValue.AddInnerHTMLElement(m_TextBox.GetComponentHTML());
      retValue.AddInnerHTMLElement(m_SelectDropDown.GetComponentHTML());
      if (null != GetHTMLComponentStyleClass())
      {
        m_TextBox.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
        m_SelectDropDown.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
        }
      return retValue;
    }
    
    /// <summary>
    /// Return the component values of the TextBox and SelectDropdown
    /// separated by a space.
    /// </summary>
    public override String GetHTMLComponentValue()
    {
      return m_TextBox.GetHTMLComponentValue() + " " + m_SelectDropDown.GetHTMLComponentValue();
      }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLTextAndDropDown();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
  }
}