using System;
using XPConvert=com.plumtree.openfoundation.util.XPConvert;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols
{
  /// <author>CaiusV</author>
  public class HTMLTextArea : AHTMLComponent,IPageControl
  {
    
    /// <summary>
    /// for logging 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLTextArea");
    
    /// <summary>
    /// base name constant 
    /// </summary>
    private const String STR_HTML_INPUT_BASE_NAME = "in_ta_pc";
    
    /// <summary>
    /// class name constant 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "HTMLTextArea";
    
    /// <summary>
    /// HTML TextArea Cols 
    /// </summary>
    private int m_iHTMLComponentCols;
    
    /// <summary>
    /// HTML TextArea Rows 
    /// </summary>
    private int m_iHTMLComponentRows;
    
    /// <summary>
    /// Default constructor. Creates a new HTMLTextArea object.
    /// </summary>
    public HTMLTextArea()
    {
    }
    
    /// <summary>
    /// Initializes a ATextArea object.
    /// </summary>
    /// <param name="_ptPageMgr">PageManager</param>
    /// <param name="_strUniqueInstanceID">Unique Instance ID</param>
    /// <param name="_iCols">Text Area columns</param>
    /// <param name="_iRows">Text Area rows</param>
    public virtual void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
     int _iCols, int _iRows)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      SetComponentCols(_iCols);
      SetComponentRows(_iRows);
    }
    
    /// <summary>
    /// Returns the base name of this HTML Component.
    /// </summary>
    /// <returns>String</returns>
    protected override String GetHTMLComponentBaseName()
    {
      return STR_HTML_INPUT_BASE_NAME;
    }
    
    /// <summary>
    /// Returns the HTML for this component.
    /// </summary>
    /// <returns></returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retValue = new HTMLElementCollection();
      com.plumtree.xpshared.htmlelements.HTMLTextArea htmlTextArea = new com.plumtree.xpshared.htmlelements.HTMLTextArea(XPConvert.ToString(GetComponentCols()),
       XPConvert.ToString(GetComponentRows()));
      htmlTextArea.SetName(GetHTMLComponentName());
      htmlTextArea.AddInnerHTMLString(GetHTMLComponentValue());
      retValue.AddInnerHTMLElement(htmlTextArea);
      if (null != GetHTMLComponentStyleClass())
      {
        htmlTextArea.SetStyleClass(GetHTMLComponentStyleClass());
      }
      return retValue;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new HTMLTextArea();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Return m_sHTMLComponentCols, coloumn size for the text area.
    /// </summary>
    /// <returns>text area columns.</returns>
    public virtual int GetComponentCols()
    {
      return m_iHTMLComponentCols;
    }
    
    /// <summary>
    /// Return m_sHTMLComponentRows, row size for the text area.
    /// </summary>
    /// <returns>text area rows.</returns>
    public virtual int GetComponentRows()
    {
      return m_iHTMLComponentRows;
    }
    
    /// <summary>
    /// Set the column size (m_iHTMLComponentCols) for textarea.
    /// </summary>
    /// <param name="_cols">column size</param>
    public virtual void SetComponentCols(int _cols)
    {
      m_iHTMLComponentCols = _cols;
    }
    
    /// <summary>
    /// Set the rpw size (m_iHTMLComponentRows) for textarea.
    /// </summary>
    /// <param name="_ros">column size</param>
    public virtual void SetComponentRows(int _rows)
    {
      m_iHTMLComponentRows = _rows;
    }
  }
}