using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using OpenLogService=com.plumtree.openlog.OpenLogService;
using OpenLogger=com.plumtree.openlog.OpenLogger;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.pagecontrols.datepicker;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using JSDateTimePickerMgr=com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.JSDateTimePickerMgr;
using com.plumtree.xpshared.htmlelements;
using JSRegistry=com.plumtree.xpshared.jsutil.JSRegistry;
namespace com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker
{
  /// <author>John Osborne, John Hogan</author>
  public class JSDatePicker : ADatePickerComponent,IPageControl
  {
    
    /// <summary>
    /// The name for the HTML div tag that the datepicker uses. 
    /// </summary>
    private const String STR_DP_DIV_NAME = "dpDiv";
    
    /// <summary>
    /// The base name for the JS variable name of the date-picker. 
    /// </summary>
    private const String STR_DP_VAR_NAME = "varName";
    
    /// <summary>
    /// The base name of the date-picker input textbox. 
    /// </summary>
    public const String STR_DP_DATE_INPUT_NAME = "in_tx_DPInput";
    
    /// <summary>
    /// The base name of the date-picker button. 
    /// </summary>
    private const String STR_DP_BUTTON_INPUT_NAME = "in_bu_DPButton";
    
    /// <summary>
    /// The name for the calendar button. 
    /// </summary>
    private String m_strButtonName;
    
    /// <summary>
    /// The name for the date input text box 
    /// </summary>
    private String m_strDateInputName;
    
    /// <summary>
    /// The name of the form in which the datepicker resides 
    /// </summary>
    private String m_strFormName;
    
    /// <summary>
    /// The String value of the currently selected date. 
    /// </summary>
    private String m_strSelectedDate;
    
    /// <summary>
    /// The name of the date-picker variables. 
    /// </summary>
    private String m_strVarName;
    
    /// <summary>
    /// A date formatter to help with formatting and parsing. 
    /// </summary>
    private XPDateFormat m_dfDateFormat;
    
    /// <summary>
    /// MVC class name for page control 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "DatePicker";
    
    /// <summary>
    /// Variable to store a unique ID for the DatePickMgr associated with this DatePicker *
    /// </summary>
    public String m_strDatePickerMgrID = "";
    
    /// <summary>
    /// logger 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker.JSDatePicker");
    
    /// <summary>
    /// Static EN-us locale used for formatting dates exchanged with client side javascript 
    /// </summary>
    protected static readonly XPLocale m_oUSLocale = new XPLocale("EN", "us");
    
    /// <summary>
    /// Create a new instance of JSDatePicker 
    /// </summary>
    public JSDatePicker()
    {
    }
    
    /// <summary>
    /// Initialize the JSDatePicker.
    /// </summary>
    /// <param name="_strFormName">The name of the form in which the datepicker resides</param>
    /// <param name="_ptPageMgr">
    /// The datepicker page manager for this instance of
    /// the datepicker
    /// </param>
    /// <param name="_strUniqueInstanceID">
    /// The string that will be appended to
    /// variables to make them unique
    /// </param>
    public override void Init(String _strFormName, PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      if (null == _strUniqueInstanceID)
      {
        _strUniqueInstanceID = EMPTY;
      }
      m_strFormName = _strFormName;
      m_dfDateFormat = XPDateFormat.GetDateInstance(XPDateFormat.MEDIUM, m_oUSLocale,
       _ptPageMgr.GetTimeZone());
      m_strSelectedDate = EMPTY;
      m_strDateInputName = AppendUniqueIDToElementName(STR_DP_DATE_INPUT_NAME);
      m_strButtonName = AppendUniqueIDToElementName(STR_DP_BUTTON_INPUT_NAME);
      m_strVarName = AppendUniqueIDToElementName(STR_DP_VAR_NAME);
    }
    
    /// <summary>
    /// Initialize a new JSDatePicker object.
    /// </summary>
    /// <param name="_strFormName">The name of the form in which the datepicker resides</param>
    /// <param name="_strDefaultDate">The initial date selected</param>
    /// <param name="_ptPageMgr">
    /// The datepicker page manager for this instance of
    /// the datepicker
    /// </param>
    /// <param name="_strUniqueInstanceID">
    /// The string that will be appended to
    /// variables to make them unique
    /// </param>
    public virtual void Init(String _strFormName, String _strDefaultDate,
     PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      if (null == _strUniqueInstanceID)
      {
        _strUniqueInstanceID = EMPTY;
      }
      m_strFormName = _strFormName;
      m_dfDateFormat = XPDateFormat.GetDateInstance(XPDateFormat.MEDIUM, m_oUSLocale,
       _ptPageMgr.GetTimeZone());
      if ((_strDefaultDate == null) || _strDefaultDate.Equals(EMPTY))
      {
        m_strSelectedDate = EMPTY;
      } else
      {
        try
        {
          m_dfDateFormat.Parse(_strDefaultDate);
        }
        catch (XPException e)
        {
          log.Error(e, "invalid date format " + _strDefaultDate);
          m_strSelectedDate = EMPTY;
        }
      }
      m_strDateInputName = AppendUniqueIDToElementName(STR_DP_DATE_INPUT_NAME);
      m_strButtonName = AppendUniqueIDToElementName(STR_DP_BUTTON_INPUT_NAME);
      m_strVarName = AppendUniqueIDToElementName(STR_DP_VAR_NAME);
    }
    
    /// <summary>
    /// Initailize a new JSDatePicker object.
    /// </summary>
    /// <param name="_strFormName">The name of the form in which the datepicker resides</param>
    /// <param name="_dtDefaultTime">The initial date selected</param>
    /// <param name="_ptPageMgr">The page manager where this control is registered</param>
    /// <param name="_strUniqueInstanceID">
    /// The string that will be appended to
    /// variables to make them unique
    /// </param>
    public override void Init(String _strFormName, XPDateTime _dtDefaultTime,
     PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      base.Init(_ptPageMgr, _strUniqueInstanceID);
      if (null == _strUniqueInstanceID)
      {
        _strUniqueInstanceID = EMPTY;
      }
      m_strFormName = _strFormName;
      m_dfDateFormat = XPDateFormat.GetDateInstance(XPDateFormat.MEDIUM, m_oUSLocale,
       _ptPageMgr.GetTimeZone());
      if (null != _dtDefaultTime)
      {
        try
        {
          m_strSelectedDate = m_dfDateFormat.Format(_dtDefaultTime);
        }
        catch (XPException e)
        {
          log.Error(e, "invalid date " + _dtDefaultTime, _dtDefaultTime);
          m_strSelectedDate = EMPTY;
        }
      } else
      {
        m_strSelectedDate = EMPTY;
      }
      m_strDateInputName = AppendUniqueIDToElementName(STR_DP_DATE_INPUT_NAME);
      m_strButtonName = AppendUniqueIDToElementName(STR_DP_BUTTON_INPUT_NAME);
      m_strVarName = AppendUniqueIDToElementName(STR_DP_VAR_NAME);
    }
    
    /// <summary>
    /// This method returns the datepicker HTML.
    /// </summary>
    /// <returns>The HTMLElementCollection that is the datepicker.</returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retHTMLCollection = new HTMLElementCollection();
      HTMLScript myScript;
      XPCalendar xpCalendar;
      int nFirstDayOfWeek;
      xpCalendar = XPCalendar.GetInstance(m_ptPageMgr.GetTimeZone(), m_ptPageMgr.GetLocale());
      nFirstDayOfWeek = xpCalendar.GetFirstDayOfWeek();
      //Add the script elements
      myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      XPStringBuilder sb = new XPStringBuilder(200);
      sb.Append("var " + m_strVarName + " = new PTDatepicker(");
      // div name
      sb.Append("\"" + STR_DP_DIV_NAME + m_strDatePickerMgrID + "\",");
      sb.Append("\"" + m_strVarName + "\",");
      // var name - this needs to be unique
      // form name
      sb.Append("\"" + m_strFormName + "\",");
      sb.Append("\"" + m_strDateInputName + "\",");
      // name for the input text box
      // the name for the button
      sb.Append("\"" + m_strButtonName + "\",");
      // the default value
      sb.Append("\"" + m_strSelectedDate + "\",");
      sb.Append("\"" + m_dfDateFormat.ToPattern() + "\",");
      // Date format used to communicate with the Date Picker
      // first day of the week
      sb.Append(nFirstDayOfWeek).Append(",");
      sb.Append("\"" + m_ptPageMgr.GetImgSvrBaseURL() + "plumtree/common/private/js/" + JSDatePickerMgr.STR_JS_COMPONENT_NAME + "/" + "LATEST" + "/images/" + "\",");
      // No Validation
      sb.Append("false,");
      sb.Append("\"\",");
      sb.Append("\"\",");
      sb.Append("\"\",");
      sb.Append("\"\",");
      sb.Append("\"\",");
      sb.Append("\"\",");
      sb.Append("\"" + m_ptPageMgr.GetLocale().GetLanguage() + "_" + m_ptPageMgr.GetLocale().GetCountry() + "\"");
      //Locale
      sb.Append(");");
      myScript.AddInnerHTMLString(sb.ToString());
      retHTMLCollection.AddInnerHTMLElement(myScript);
      myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      myScript.AddInnerHTMLString(m_strVarName + ".writeControl();");
      retHTMLCollection.AddInnerHTMLElement(myScript);
      return retHTMLCollection;
    }
    
    /// <summary>
    /// This method returns the datepicker HTML.
    /// </summary>
    /// <param name="_strPickerType">the picker type.</param>
    /// <returns>The HTMLElementCollection that is the datepicker.</returns>
    public virtual HTMLElementCollection GetComponentHTML(String _strPickerType)
    {
      HTMLElementCollection retHTMLCollection = new HTMLElementCollection();
      HTMLScript myScript;
      XPCalendar xpCalendar;
      int nFirstDayOfWeek;
      xpCalendar = XPCalendar.GetInstance(m_ptPageMgr.GetTimeZone(), m_ptPageMgr.GetLocale());
      nFirstDayOfWeek = xpCalendar.GetFirstDayOfWeek();
      //Add the script elements
      myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      XPStringBuilder sb = new XPStringBuilder(200);
      sb.Append("var " + m_strVarName + " = new PTDatepicker(");
      // div name
      sb.Append("\"" + STR_DP_DIV_NAME + m_strDatePickerMgrID + "\",");
      sb.Append("\"" + m_strVarName + "\",");
      // var name - this needs to be unique
      // form name
      sb.Append("\"" + m_strFormName + "\",");
      sb.Append("\"" + m_strDateInputName + "\",");
      // name for the input text box
      // the name for the button
      sb.Append("\"" + m_strButtonName + "\",");
      // the default value
      sb.Append("\"" + m_strSelectedDate + "\",");
      sb.Append("\"" + m_dfDateFormat.ToPattern() + "\",");
      // Date format used to communicate with the Date Picker
      // first day of the week
      sb.Append(nFirstDayOfWeek).Append(",");
      sb.Append("\"" + m_ptPageMgr.GetImgSvrBaseURL() + "plumtree/common/private/js/" + JSDatePickerMgr.STR_JS_COMPONENT_NAME + "/" + "LATEST" + "/images/" + "\",");
      // No Validation
      sb.Append("false,");
      sb.Append("\"\",");
      sb.Append("\"\",");
      sb.Append("\"\",");
      sb.Append("\"\",");
      sb.Append("\"\",");
      sb.Append("\"\",");
      sb.Append("\"" + m_ptPageMgr.GetLocale().GetLanguage() + "_" + m_ptPageMgr.GetLocale().GetCountry() + "\"");
      //Locale
      sb.Append(");");
      myScript.AddInnerHTMLString(sb.ToString());
      retHTMLCollection.AddInnerHTMLElement(myScript);
      myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      myScript.AddInnerHTMLString(m_strVarName + ".writeControl();");
      retHTMLCollection.AddInnerHTMLElement(myScript);
      return retHTMLCollection;
    }
    
    /// <summary>
    /// Provides a means of retrieving the HTML Input name for the date portion
    /// of the date picker.  This allows for using javascript to
    /// programmatically set input values, for example.
    /// </summary>
    /// <returns>String</returns>
    public override String GetDateInputName()
    {
      return m_strDateInputName;
    }
    
    /// <summary>
    /// Returns the name of the PTDatepicker JavaScript variable name
    /// </summary>
    /// <returns>String</returns>
    public override String GetPTDatepickerVarName()
    {
      return m_strVarName;
    }
    
    /// <summary>
    /// Returns the date selected in the date picker as a string
    /// </summary>
    /// <returns>String</returns>
    public override String GetSelectedDateAsString()
    {
      return m_strSelectedDate;
    }
    
    /// <summary>
    /// Stores the user input after the request is sent back.
    /// </summary>
    /// <param name="_htRequestParameters">The XPHashtable of request name/value pairs.</param>
    public override int StoreUserInput(XPHashtable _htRequestParameters)
    {
      String[] data = (String[]) _htRequestParameters.GetElement(m_strDateInputName);
      if ((null != data) && !(data[0].Equals(EMPTY)))
      {
        m_strSelectedDate = data[0];
      } else
      {
        m_strSelectedDate = EMPTY;
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// This function returns an XPDateTime object that represents the date and
    /// time the user has chosen using the date/time-picker HTML control.
    /// </summary>
    /// <returns>XPDateTime</returns>
    public override XPDateTime GetDateAsDateTime()
    {
      XPDateTime xpdtResult = null;
      try
      {
        XPDateFormat myDateFormat = XPDateFormat.GetDateInstance(XPDateFormat.MEDIUM,
         m_oUSLocale, m_ptPageMgr.GetTimeZone());
        String myDateString = GetSelectedDateAsString();
        xpdtResult = myDateFormat.Parse(myDateString);
      }
      catch (XPException e)
      {
        log.Error(e, "selected date: " + GetSelectedDateAsString() + " is invalid");
        return null;
      }
      return xpdtResult;
    }
    public virtual void SetDatePickerMgrID(String strDatePickerMgrID)
    {
      m_strDatePickerMgrID = strDatePickerMgrID;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new JSDatePicker();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    public override ComponentMgr GetComponentMgr(PageMgr _ptPageMgr)
    {
      JSRegistry ptJSRegistry = JSRegistry.GetRegistry(_ptPageMgr.GetImgSvrBaseURL(),
       _ptPageMgr.GetImageServerConnectionURL());
      return new JSDateTimePickerMgr(ptJSRegistry, new JSDatePickerVersionMgr(),
       _ptPageMgr);
    }
    
    /// <summary>
    /// Return the name of the component manager responsible for this page control
    /// </summary>
    public override String GetComponentMgrName()
    {
      return JSDateTimePickerMgr.STR_CLASS_NAME;
    }
    
    /// <summary>
    /// Currently validation on the date/time picker page controls is not implemented.
    /// This method returns null.
    /// </summary>
    public override XPArrayList Validate()
    {
      return null;
    }
    
    /// <summary>
    /// Currently validation on the date/time picker page controls is not implemented.
    /// This method returns null.
    /// </summary>
    public override XPArrayList GetValidators()
    {
      return null;
    }
  }
}