using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
using com.plumtree.xpshared.jsutil;
namespace com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker
{
  /// <summary>
  /// This is the component manager class for the JS date-picker. Note: there is
  /// also JS date/time-picker if you need date and time input.
  /// </summary>
  /// <author>JohnO</author>
  public class JSDatePickerMgr : JSComponentMgr
  {
    
    /// <summary>
    /// The name for JS controls of this type. 
    /// </summary>
    public const String STR_JS_COMPONENT_NAME = "jsdatepicker";
    
    /// <summary>
    /// The ID for the HTML div tag that the datepicker uses. 
    /// </summary>
    private const String STR_DP_DIV_ID = "dpDiv";
    
    /// <summary>
    /// Unique ID for this datepick manager. *
    /// </summary>
    private String m_strJSDatePickerMgrID = "";
    
    /// <summary>
    /// Creates a new JSDatePickerMgr object.
    /// </summary>
    /// <param name="_ptJSRegistry">The JS Registry is needed to get the JS includes</param>
    /// <param name="_ptJSCompVerMgr">
    /// The component version manager interface allows us
    /// to the version of this control that is registered with the
    /// product as a whole
    /// </param>
    /// <param name="_ptPageMgr">
    /// The locale that should be used by the datepickers
    /// under the control of this manager
    /// </param>
    public JSDatePickerMgr(JSRegistry _ptJSRegistry, IJSComponentVersionMgr _ptJSCompVerMgr,
     PageMgr _ptPageMgr) : base(STR_JS_COMPONENT_NAME, _ptJSRegistry, _ptJSCompVerMgr,
     _ptPageMgr)
    {
    }
    
    /// <summary>
    /// This method returns the HTML that needs to be written-out before the JS
    /// datepicker form is written out. Specifically, this is HTML that defines
    /// a div. Note that this HTML is of page scope, meaning that it only needs
    /// to be written out one time even if there are multiple date-pickers on
    /// the page.
    /// </summary>
    /// <returns>The pre-form HTML for this</returns>
    public override HTMLElementCollection GetPagePreFormContent()
    {
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLDiv myDiv = new HTMLDiv();
      myDiv.SetID(STR_DP_DIV_ID);
      myDiv.SetStyleClass(PTStyleClass.CAL_DIV);
      result.AddInnerHTMLElement(myDiv);
      return result;
    }
    
    /// <summary>
    /// This method returns the HTML that needs to be written-out before the JS
    /// datepicker form is written out. Specifically, this is HTML that defines
    /// a div. Note that this HTML is of page scope, meaning that it only needs
    /// to be written out one time even if there are multiple date-pickers on
    /// the page.
    /// </summary>
    /// <param name="strDatePickerMgrID">
    /// Unique ID for datepickers managed by this object.
    /// Use this method when there might be several datepick managers on a page, e.g. MyPage.
    /// </param>
    /// <returns>The pre-form HTML for this</returns>
    public virtual HTMLElementCollection GetPagePreFormContent(String strDatePickerMgrID)
    {
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLDiv myDiv = new HTMLDiv();
      myDiv.SetID(STR_DP_DIV_ID + strDatePickerMgrID);
      myDiv.SetStyleClass(PTStyleClass.CAL_DIV);
      result.AddInnerHTMLElement(myDiv);
      m_strJSDatePickerMgrID = strDatePickerMgrID;
      return result;
    }
    
    /// <summary>
    /// This method allows you to register a component with the component
    /// manager. The order in which you register components is the order in
    /// which calls will be made when doing any looping operation--storing or
    /// printing. If you attempt to re-register the same component, it will
    /// overwrite the one that existed there before.
    /// </summary>
    /// <param name="_ptComponent">The component that you wish to register.</param>
    public override void RegisterComponent(IPageControl _ptComponent)
    {
      base.RegisterComponent(_ptComponent);
      if (!"".Equals(m_strJSDatePickerMgrID) && _ptComponent is JSDatePicker)
      {
        ((JSDatePicker) _ptComponent).SetDatePickerMgrID(m_strJSDatePickerMgrID);
        }
    }
    
    /// <summary>
    /// Get the ID of the preform DIV element used by datepickers managed by this object.
    /// </summary>
    /// <returns></returns>
    public virtual String GetPreFormDivID()
    {
      return STR_DP_DIV_ID + m_strJSDatePickerMgrID;
    }
  }
}