using System;
using com.plumtree.openfoundation.internationalization;
using com.plumtree.openfoundation.util;
using OpenLogService=com.plumtree.openlog.OpenLogService;
using OpenLogger=com.plumtree.openlog.OpenLogger;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
using com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker;
using com.plumtree.uiinfrastructure.pagecontrols.timepicker;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker
{
  /// <summary>
  /// This class defines the JSDateTimePicker control.
  /// </summary>
  /// <author>John Osborne</author>
  /// <author>Caius van Nouhuys</author>
  public class JSDateTimePicker : JSDatePicker,IPageControl
  {
    
    /// <summary>
    /// The time-picker part of this component. 
    /// </summary>
    private TimePicker m_tpTimePicker;
    public const String STR_MVC_CLASS_NAME = "DateTimePicker";
    
    /// <summary>
    /// logger 
    /// </summary>
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.JSDateTimePicker");
    
    /// <summary>
    /// Default empty constructor.
    /// </summary>
    public JSDateTimePicker()
    {
    }
    
    /// <summary>
    /// Initialize a new JSDateTimePicker.
    /// </summary>
    /// <param name="_strFormName">The name of the form in which the datepicker resides</param>
    /// <param name="_strDefaultDate">The initial date selected</param>
    /// <param name="_ptPageMgr">
    /// The datepicker page manager for this instance of
    /// the datepicker
    /// </param>
    /// <param name="_strUniqueInstanceID">
    /// The string that will be appended to
    /// variables to make them unique
    /// </param>
    public override void Init(String _strFormName, String _strDefaultDate,
     PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      base.Init(_strFormName, _strDefaultDate, _ptPageMgr, _strUniqueInstanceID);
      m_tpTimePicker = new TimePicker();
      m_tpTimePicker.Init(_ptPageMgr, EMPTY, _strUniqueInstanceID);
    }
    
    /// <summary>
    /// Initialize a new JSDateTimePicker.
    /// </summary>
    /// <param name="_strFormName">The name of the form in which the datepicker resides</param>
    /// <param name="_ptPageMgr">
    /// The datepicker page manager for this instance of
    /// the datepicker
    /// </param>
    /// <param name="_strUniqueInstanceID">
    /// The string that will be appended to
    /// variables to make them unique
    /// </param>
    public override void Init(String _strFormName, PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      base.Init(_strFormName, _ptPageMgr, _strUniqueInstanceID);
      m_tpTimePicker = new TimePicker();
      m_tpTimePicker.Init(_ptPageMgr, EMPTY, _strUniqueInstanceID);
    }
    
    /// <summary>
    /// Initialize a new JSDateTimePicker.
    /// </summary>
    /// <param name="_strFormName">
    /// The name of the form in which this component
    /// resides.
    /// </param>
    /// <param name="_dtDefaultDateTime">The default date-time.</param>
    /// <param name="_ptComponentMgr">
    /// The component manager for this instance of the
    /// date/time-picker.
    /// </param>
    /// <param name="_strUniqueInstanceID">
    /// The unique ID for this instance of the
    /// date/time-picker.
    /// </param>
    public override void Init(String _strFormName, XPDateTime _dtDefaultDateTime,
     PageMgr _ptPageMgr, String _strUniqueInstanceID)
    {
      base.Init(_strFormName, _dtDefaultDateTime, _ptPageMgr, _strUniqueInstanceID);
      //Set the format object - use the MEDIUM format
      XPDateFormat dfDateTimeFormat = XPDateFormat.GetDateTimeInstance(XPDateFormat.MEDIUM,
       XPDateFormat.MEDIUM, _ptPageMgr.GetLocale(), _ptPageMgr.GetTimeZone());
      // Get a calendar
      XPCalendar cal = XPCalendar.GetInstance(_ptPageMgr.GetLocale());
      cal.SetTimeZone(_ptPageMgr.GetTimeZone());
      // Check for the 24 hour clock pattern
      bool b24Hour = false;
      String strDateTimePattern = dfDateTimeFormat.ToPattern();
      if (XPStringUtility.IndexOf(strDateTimePattern, H) != -1)
      {
        b24Hour = true;
      }
      if (null != _dtDefaultDateTime)
      {
        cal.SetTime(_dtDefaultDateTime);
        if (b24Hour)
        {
          m_tpTimePicker = new TimePicker();
          m_tpTimePicker.Init(cal.Get(XPCalendar.HOUR_OF_DAY), cal.Get(XPCalendar.MINUTE),
           _ptPageMgr, _strUniqueInstanceID);
        } else
        if (cal.Get(XPCalendar.AM_PM) == 0)
        {
          m_tpTimePicker = new TimePicker();
          m_tpTimePicker.Init(cal.Get(XPCalendar.HOUR), cal.Get(XPCalendar.MINUTE),
           dfDateTimeFormat.GetAmPmStrings()[0], _ptPageMgr, _strUniqueInstanceID);
          } else
        {
          m_tpTimePicker = new TimePicker();
          m_tpTimePicker.Init(cal.Get(XPCalendar.HOUR), cal.Get(XPCalendar.MINUTE),
           dfDateTimeFormat.GetAmPmStrings()[1], _ptPageMgr, _strUniqueInstanceID);
          }
      } else
      {
        m_tpTimePicker = new TimePicker();
        m_tpTimePicker.Init(_ptPageMgr, EMPTY, _strUniqueInstanceID);
      }
    }
    
    /// <summary>
    /// This method returns the AM/PM part of the time. If we are using 24-hour
    /// time this method returns null.
    /// </summary>
    /// <returns>The AM/PM part of the time String.</returns>
    public virtual String GetAMPM()
    {
      return m_tpTimePicker.GetAMPM();
    }
    
    /// <summary>
    /// Provides a means of retrieving the HTML Input name for the AMPM portion
    /// of the date picker.  This allows for using javascript to
    /// programmatically set input values, for example.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetAMPMInputName()
    {
      return m_tpTimePicker.GetAMPMInputName();
    }
    
    /// <summary>
    /// This method returns the date/time-picker HTML.
    /// </summary>
    /// <returns>The HTMLElementCollection that is the date and time picker.</returns>
    public override HTMLElementCollection GetComponentHTML()
    {
      HTMLElementCollection retVal = base.GetComponentHTML(STR_MVC_CLASS_NAME);
      retVal.AddInnerHTMLElement(m_tpTimePicker.GetComponentHTML());
      return retVal;
    }
    
    /// <summary>
    /// This function returns an XPDateTime object that represents the date and
    /// time the user has chosen using the date/time-picker HTML control.
    /// </summary>
    /// <returns>XPDateTime</returns>
    public override XPDateTime GetDateTimeAsDateTime()
    {
      XPDateTime xpdtResult = null;
      XPDateFormat dfDateTimeFormat = XPDateFormat.GetDateTimeInstance(XPDateFormat.MEDIUM,
       XPDateFormat.MEDIUM, m_oUSLocale, m_ptPageMgr.GetTimeZone());
      try
      {
        String dateString = GetSelectedDateAsString();
        String timeString = m_tpTimePicker.GetTimeAsString();
        xpdtResult = dfDateTimeFormat.Parse(dateString + " " + timeString);
        }
      catch (XPException e)
      {
        log.Error(e, "Unable to parse datetime");
        return null;
      }
      return xpdtResult;
    }
    
    /// <summary>
    /// This method returns the date and time as a String.
    /// </summary>
    /// <returns>The date/time string.</returns>
    public virtual String GetDateTimeAsString()
    {
      return GetSelectedDateAsString() + " " + m_tpTimePicker.GetTimeAsString();
      }
    
    /// <summary>
    /// This method returns the hours part of the time as a String.
    /// </summary>
    /// <returns>The hours.</returns>
    public virtual String GetHours()
    {
      return m_tpTimePicker.GetHours();
    }
    
    /// <summary>
    /// Provides a means of retrieving the HTML Input name for the hours portion
    /// of the date picker.  This allows for using javascript to
    /// programmatically set input values, for example.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetHoursInputName()
    {
      return m_tpTimePicker.GetHoursInputName();
    }
    
    /// <summary>
    /// This method returns the minutes part of the time as a String.
    /// </summary>
    /// <returns>The minutes.</returns>
    public virtual String GetMinutes()
    {
      return m_tpTimePicker.GetMinutes();
    }
    
    /// <summary>
    /// Provides a means of retrieving the HTML Input name for the minutes
    /// portion of the date picker.  This allows for using javascript to
    /// programmatically set input values, for example.
    /// </summary>
    /// <returns>String</returns>
    public virtual String GetMinutesInputName()
    {
      return m_tpTimePicker.GetMinutesInputName();
    }
    
    /// <summary>
    /// This method serves as a getter for "sub-component" HTML. The
    /// date/time-picker is made up of two sub-components: the date-picker and
    /// the time-picker. The constants for component sub-index should be
    /// specified in the component file.
    /// </summary>
    /// <param name="_nSubComponentIndex">
    /// The sub-component index
    /// (N_SUB_COMPONENT_INDEX_DATE_PICKER or
    /// N_SUB_COMPONENT_INDEX_TIME_PICKER).
    /// </param>
    /// <returns>The HTML for the sub-component requested.</returns>
    public override HTMLElementCollection GetSubComponentHTMLByIndex(int _nSubComponentIndex)
    {
      if (_nSubComponentIndex == 0)
      {
        return base.GetComponentHTML(STR_MVC_CLASS_NAME);
      } else
      if (_nSubComponentIndex == 1)
      {
        return m_tpTimePicker.GetComponentHTML();
      }
      return null;
    }
    
    /// <summary>
    /// This method returns just the time part of the String.
    /// </summary>
    /// <returns>The time String.</returns>
    public virtual String GetTimeAsString()
    {
      return m_tpTimePicker.GetTimeAsString();
    }
    
    /// <summary>
    /// Stores the user input after the request is sent back.
    /// </summary>
    /// <param name="_htRequestParameters">The XPHashtable of request name/value pairs.</param>
    public override int StoreUserInput(XPHashtable _htRequestParameters)
    {
      base.StoreUserInput(_htRequestParameters);
      m_tpTimePicker.StoreUserInput(_htRequestParameters);
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public override Object Create()
    {
      return new JSDateTimePicker();
    }
    
    /// <summary>
    /// Return the name of the MVC object.
    /// </summary>
    /// <returns>the name of the object.</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Return the XPDateTime for the specific date picker
    /// </summary>
    /// <param name="_htRequestParameters">The XPHashtable of request name/value pairs.</param>
    /// <param name="_strDateInputName">unique instance name for this control</param>
    /// <param name="_xpLocale">XPLocale for this control</param>
    /// <param name="_xpTimeZone">XPTimeZone for this control</param>
    /// <returns>
    /// XPDateTime the XPDateTime derived from the request and various
    /// parameters passed in
    /// </returns>
    public static XPDateTime GetDateTime(XPHashtable _htRequestParameters,
     String _strDateInputName, XPLocale _xpLocale, XPTimeZone _xpTimeZone)
    {
      String[] data;
      XPDateTime myDateTime = null;
      String strHours = null;
      String strMinutes = null;
      String strAMPM = null;
      XPCalendar myCalendar = null;
      XPDateFormat dfDateFormat = XPDateFormat.GetDateInstance(XPDateFormat.MEDIUM,
       _xpLocale, _xpTimeZone);
      XPDateFormat defaultDateFormat = XPDateFormat.GetDateInstance(XPDateFormat.MEDIUM,
       XPLocale.GetDefault(), _xpTimeZone);
      XPDateFormat defaultUSDateFormat = XPDateFormat.GetDateInstance(XPDateFormat.MEDIUM,
       JSDatePicker.m_oUSLocale, _xpTimeZone);
      data = (String[]) _htRequestParameters.GetElement(JSDatePicker.STR_DP_DATE_INPUT_NAME + "_" + _strDateInputName);
      if ((null != data) && !(data[0].Equals(EMPTY)))
      {
        // first try the default USDateFormat
        try
        {
          myDateTime = defaultUSDateFormat.Parse(data[0]);
        }
        catch (Exception e)
        {
          myDateTime = null;
        }
        // if that didn't work try the default locale for the OS
        if (myDateTime == null)
        {
          try
          {
            myDateTime = defaultDateFormat.Parse(data[0]);
          }
          catch (Exception e)
          {
            myDateTime = null;
          }
        }
        // if that didn't work try the given locale
        if (myDateTime == null)
        {
          try
          {
            myDateTime = dfDateFormat.Parse(data[0]);
          }
          catch (Exception e)
          {
            myDateTime = null;
          }
        }
        // if that still didn't work set the date time to a new datetime
        if (myDateTime == null)
        {
          log.Error("unable to parse datetime. " + data[0] + " using default date.");
          myDateTime = new XPDateTime();
        }
      }
      //  extract time values
      // you may have asked for the 'localized' date, but there are no 'localized' time
      // variables so the first thing to do is remove '_localized' if it's in the input
      // name
      int indexOfLocalized = XPStringUtility.IndexOf(_strDateInputName, "_localized");
      if (indexOfLocalized != -1)
      {
        _strDateInputName = XPStringUtility.Substring(_strDateInputName, 0,
         indexOfLocalized);
      }
      data = (String[]) _htRequestParameters.GetElement(TimePicker.HTMLSELECT_HOURS + _strDateInputName);
      if (data != null)
      {
        strHours = data[0];
      }
      data = (String[]) _htRequestParameters.GetElement(TimePicker.HTMLSELECT_MINUTES + _strDateInputName);
      if (data != null)
      {
        strMinutes = data[0];
      }
      data = (String[]) _htRequestParameters.GetElement(TimePicker.HTMLSELECT_AMPM + _strDateInputName);
      if (data != null && !(data[0].Equals(EMPTY)))
      {
        strAMPM = data[0];
      }
      myCalendar = XPCalendar.GetInstance(_xpTimeZone, _xpLocale);
      if (myDateTime == null)
      {
        //null values in hours minutes indicate and invalid datetime
        if (null == strHours || null == strHours)
        {
          log.Error("strHours, strHours cannot be null");
          return null;
        }
        myCalendar.Set(XPCalendar.HOUR, XPConvert.ToInteger(strHours));
        myCalendar.Set(XPCalendar.MINUTE, XPConvert.ToInteger(strMinutes));
        if (null != strAMPM && strAMPM.Equals(AM))
        {
          myCalendar.Set(XPCalendar.AM_PM, XPCalendar.AM);
        } else
        if (null != strAMPM && strAMPM.Equals(PM))
        {
          myCalendar.Set(XPCalendar.AM_PM, XPCalendar.PM);
        }
        return myCalendar.GetTime();
      } else
      // myDateTime is not null
      {
        XPCalendar myTmpCalendar = XPCalendar.GetInstance(_xpTimeZone, _xpLocale);
        myTmpCalendar.SetTime(myDateTime);
        int iHours = XPConvert.ToInteger(strHours);
        int iMins = XPConvert.ToInteger(strMinutes);
        myCalendar.Set(myTmpCalendar.Get(XPCalendar.YEAR), myTmpCalendar.Get(XPCalendar.MONTH),
         myTmpCalendar.Get(XPCalendar.DAY_OF_MONTH), iHours, iMins);
        if (null != strAMPM && strAMPM.Equals(AM))
        {
          myCalendar.Set(XPCalendar.AM_PM, XPCalendar.AM);
        } else
        if (null != strAMPM && strAMPM.Equals(PM))
        {
          myCalendar.Set(XPCalendar.AM_PM, XPCalendar.PM);
        }
        return myCalendar.GetTime();
      }
    }
  }
}