using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.pagecontrols.framework;
namespace com.plumtree.uiinfrastructure.pagecontrols.timepicker
{
  /// <author>DorisL</author>
  public abstract class ATimePicker : AComponent,IPageControl
  {
    
    /// <summary>
    /// Creates a new TimePicker object based on a complete time string in
    /// XPDateFormat.MEDIUM format
    /// </summary>
    /// <param name="_strTime">- The time string to parse</param>
    /// <param name="_ptComponentMgr">- The component manager</param>
    /// <param name="_strUniqueInstanceID">
    /// - The unique identifying string to
    /// distinguish multiple time pickers on the page
    /// </param>
    public abstract void Init(PageMgr _ptPageMgr, String _strTime, String _strUniqueInstanceID);
    
    /// <summary>
    /// Creates a new TimePicker object.
    /// </summary>
    /// <param name="_nHours">The hours value to initially select</param>
    /// <param name="_nMinutes">The minutes value to initially select</param>
    /// <param name="_strAMPM">The AM/PM string to initially select</param>
    /// <param name="_ptComponentMgr">The component manager</param>
    /// <param name="_strUniqueInstanceID">
    /// The unique identifier string used to
    /// distinguish multiple copies of this component on the same page
    /// </param>
    public abstract void Init(int _nHours, int _nMinutes, String _strAMPM,
     PageMgr _ptPageMgr, String _strUniqueInstanceID);
    
    /// <summary>
    /// Creates a new TimePicker object.
    /// </summary>
    /// <param name="_nHours">The hours value to initially select</param>
    /// <param name="_nMinutes">The minutes value to initially select</param>
    /// <param name="_ptComponentMgr">The component manager</param>
    /// <param name="_strUniqueInstanceID">
    /// The unique identifier string used to
    /// distinguish multiple copies of this component on the same page
    /// </param>
    public abstract void Init(int _nHours, int _nMinutes, PageMgr _ptPageMgr,
     String _strUniqueInstanceID);
    
    /// <summary>
    /// Returns the AMPM string
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetAMPM();
    
    /// <summary>
    /// Provides a means of retrieving the HTML Input name for the AMPM portion
    /// of the date picker.  This allows for using javascript to
    /// programmatically set input values, for example.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetAMPMInputName();
    
    /// <summary>
    /// Returns the hours string
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetHours();
    
    /// <summary>
    /// Provides a means of retrieving the HTML Input name for the hours portion
    /// of the date picker.  This allows for using javascript to
    /// programmatically set input values, for example.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetHoursInputName();
    
    /// <summary>
    /// Returns the minutes string
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetMinutes();
    
    /// <summary>
    /// Provides a means of retrieving the HTML Input name for the minutes
    /// portion of the date picker.  This allows for using javascript to
    /// programmatically set input values, for example.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetMinutesInputName();
    
    /// <summary>
    /// Returns the time portion of the date and time chosen by the user as a
    /// String.
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetTimeAsString();
    
    /// <summary>
    /// Returns the time chosen by the user as a
    /// String in the English format: "h:mm:ss a".
    /// </summary>
    /// <returns>String</returns>
    public abstract String GetTimeAsEnglishString();
    public abstract com.plumtree.uiinfrastructure.pagecontrols.framework.ComponentMgr GetComponentMgr(com.plumtree.uiinfrastructure.pagecontrols.framework.PageMgr x0);
    public abstract String GetComponentMgrName();
  }
}