using System;
namespace com.plumtree.uiinfrastructure.pagination
{
  /// <author>
  /// CraigS
  /// General interface for pagination (read-only part).
  /// </author>
  public interface IPaginationRO
  {
    
    /// <summary>
    /// Returns the index of the first displayable item.  On the
    /// first page of results, this will be 1; on subsequent
    /// pages it will be 11, 21, and so on (if there are 10
    /// items per page).
    /// </summary>
    int GetFirstDisplayableItem();
    
    /// <summary>
    /// Returns the user's desired results per page.  This may differ
    /// from the number of currently displayable items if the total number
    /// of items is short or if the user is looking at the last page.
    /// or if the user is looking at the last result page.
    /// </summary>
    int GetItemsPerPage();
    
    /// <summary>
    /// Returns the number of results actually returned.
    /// </summary>
    int GetNumDisplayableItems();
    
    /// <summary>
    /// Returns the current sort order.
    /// </summary>
    String GetSortOrder();
    
    /// <summary>
    /// Returns the total number of items, some of which may not
    /// currently be displayable.
    /// </summary>
    int GetTotalItems();
  }
}