using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.pei
{
  /// <summary>
  /// This class has been created to allow developers the ability to add some
  /// Javascript functionality to the script section on many main pages, such as
  /// Plumtree banner pages and editor pages. Other pages may not get this
  /// javascript displayed on them.
  /// <br /><br />
  /// NOTE: Should this interface be implemented more than once within the
  /// Dynamic Discovery process, the implementations will be processed in the
  /// order that they are displayed within the representative XML file. All
  /// implementations will be called.
  /// </summary>
  public interface IDisplayJavascriptActions
  {
    
    /// <summary>
    /// This method allows Javascript to be displayed in the script sections of
    /// many main pages, including Plumtree banner pages and editor pages.
    /// <br /><br />
    /// AActivitySpace.GetCurrentPage() can be used to determine what the
    /// current page is, if some custom processing needs to occur.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <returns>HTMLScript the HTML Script to be displayed</returns>
    HTMLScript DisplayJavascript(AActivitySpace _asCurrentSpace);
  }
}