using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.login;
namespace com.plumtree.uiinfrastructure.pei
{
  /// <summary>
  /// This interface was created to allow developers the ability to create
  /// functions that can be dynamically discovered by the Plumtree application
  /// for the purposes of performing some functionality when a particular
  /// login based event occurs.
  /// <br /><br />
  /// NOTE: Should this interface be implemented more than once within the
  /// Dynamic Discovery process, the implementations will be processed in the
  /// order that they are displayed within the representative XML file. All
  /// implementations will be called.
  /// <br /><br />
  /// NOTE: In the case where a Redirect object is returned, the first 
  /// implementation of the interface that actually returns a valid Redirect will 
  /// be the one that will be processed.
  /// <br /><br />
  /// NOTE: To return an error message on a failed login event, use the
  /// OnFailedLogin method in the ILoginActions interface instead.
  /// <br /><br />
  /// NOTE: PEIs implementing the ILoginActions2 interface are loaded exactly
  /// the same as PEIs implementing the ILoginActions interface.
  /// </summary>
  public interface ILoginActions2 : ILoginActions
  {
    
    /// <summary>
    /// This function allows for some functionality to occur if a user should
    /// not login successfully and then possibly do a redirect to someplace
    /// other than the standard Login page.
    /// </summary>
    /// <param name="_appData">
    /// - This parameter provides access to some of the
    /// application data that would normally be available through the
    /// current Activity Space
    /// </param>
    /// <param name="_strUserName">- This is the name of the user who failed on login..</param>
    /// <param name="_xpError">- the error that caused login to fail.  May be null</param>
    /// <returns>
    /// Redirect - Location that the User will be redirected after the
    /// functionality has completed.
    /// </returns>
    Redirect OnFailedLoginDoRedirect(ApplicationData _appData, String _strUserName,
     XPException _xpError);
  }
}