using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.uiinfrastructure.pei
{
  /// <summary>
  /// This interface was created to allow developers the ability to create
  /// functions that can be dynamically discovered by the Plumtree application
  /// for the purposes of performing some functionality when a particular
  /// object creation or modification based event occurs.
  /// <br /><br />
  /// </summary>
  public interface IOpenerActions
  {
    
    /// <summary>
    /// This function will perform some functionality just before we use the Common
    /// Opener to open an object (or direct to an Activity Space).  To override the
    /// destination to Redirect to, the function will optionally return a Redirect.
    /// If multiple implementations are processed, they will be processed according
    /// to the order that they appearin the XML file.  Processing will stop once an
    /// implemention returns a valid Redirect, however. 
    /// </summary>
    /// <param name="_nClassID">
    /// - Unchecked Class ID of the object that is being edited.
    /// -1 if the object to be opened does not use class IDs and object IDs
    /// (such as Activity Spaces).
    /// </param>
    /// <param name="_nObjectID">
    /// - Unchecked Object ID of the object that is being edited.
    /// -1 if the object to be opened does not use class IDs and object IDs
    /// (such as Activity Spaces).
    /// </param>
    /// <param name="_strClassKey">
    /// - The class key of this object.  For objects that use
    /// class IDs and object IDs this will generally be the class ID.
    /// </param>
    /// <param name="_htQSArguments">
    /// - The query string arguments from the link to the
    /// Common Opener.
    /// </param>
    /// <param name="_rRedirect">
    /// -  The redirect generated by the Common Opener.  This
    /// will be used unless a new redirect is returned by a PEI.
    /// </param>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_userSession">- Plumtree session object for the current user</param>
    /// <returns>
    /// Redirect - Optional location that the User will be redirected after 
    /// the functionality has completed.  Leave this null otherwise.
    /// </returns>
    Redirect OnBeforeOpen(int _nClassID, int _nObjectID, String _strClassKey,
     XPHashtable _htQSArguments, Redirect _rRedirect, AActivitySpace _asCurrentSpace,
     Object _userSession);
  }
}