using System;
using com.plumtree.xpshared.htmlelements;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.uiinfrastructure.pei
{
  /// <summary>
  /// This class has been created to allow developers the ability to add some
  /// functionality to the beginning and end of any and every page that is
  /// displayed within the Plumtree Portal. The amount of functionality that is
  /// available is going to be and should be extremely limited as this could
  /// affect performance a great deal.  
  /// <br /><br />
  /// NOTE: Should this interface be implemented more than once within the
  /// Dynamic Discovery process, the implementations will be processed in the
  /// order that they are displayed within the representative XML file. All
  /// implementations will be called.
  /// <br /><br />
  /// NOTE: These functions will primarily be implemented within the cases of a 
  /// MyPortal Page, Community Page, Directory Page, and Login Page. As these pages 
  /// all include Gateway requests, we will not be worrying about Gateway requests, 
  /// just the outer containers request.
  /// <br /><br />
  /// NOTE: All information needed to use as a key for timestamps should be
  /// available from the Server Name and Activity Space, such as User ID,
  /// Activity Space Name, current Display Page Name and so on.  There is also a
  /// possibility that the _oUserSession passed in to OnPageStart() and
  /// OnPageFinish() will be different (such as when you click finish after
  /// editing the mypage layout - the request will start in the edit layout
  /// space, but finish in the myPage space).
  /// </summary>
  public interface IPageActions
  {
    
    /// <summary>
    /// This function allows for some functionality to be created at the very
    /// end of the processing of the page.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_oUserSession">- Session object for the current user</param>
    /// <param name="_strServerName">- Server from which the page request originated.</param>
    /// <returns>
    /// HTMLComment - The return object will be a HTMLComment that will
    /// be added to the end of the page being displayed. If no
    /// information is needed to be returned, then just return Null.
    /// </returns>
    HTMLComment OnPageFinish(AActivitySpace _asCurrentSpace, Object _oUserSession,
     String _strServerName);
    
    /// <summary>
    /// This function allows for some functionality to be created at the very
    /// beginning of the page.
    /// </summary>
    /// <param name="_asCurrentSpace">
    /// - Provides access to the current MVC classes as
    /// well as the Application and the HTTP Session
    /// </param>
    /// <param name="_oUserSession">- Session object for the current user</param>
    /// <param name="_strServerName">- Server from which the page request began.</param>
    void OnPageStart(AActivitySpace _asCurrentSpace, Object _oUserSession,
     String _strServerName);
  }
}