using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.errordisplay;
namespace com.plumtree.uiinfrastructure.statichelpers
{
  /// <author>
  /// robertz
  /// Static helper functions
  /// </author>
  public class GeneralHelpers
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.statichelpers.GeneralHelpers");
    public static String FormatPath(String _strPath, String _strDelimiter,
     bool _bForceEndsWith)
    {
      XPStringBuilder buf = new XPStringBuilder(_strPath);
      int i = 0;
      int nStart = 0;
      int nEnd;
      bool bAlreadyEndsWith = false;
      //SS - Iterate through our string
      while (i < buf.GetLength())
      {
        //SS - We see the start of our replaceable tokens ('\' or '/')
        if ((buf.CharAt(i) == '\\') || (buf.CharAt(i) == '/'))
        {
          nStart = i;
          //SS - Iterate through our replaceable tokens
          while ((buf.CharAt(i) == '\\') || (buf.CharAt(i) == '/'))
          {
            if (++i == buf.GetLength())
            {
              break;
            }
          }
          nEnd = i;
          //SS - Replace the tokens with our delimiter and start our
          //counter back at the new buffer position (because we
          //have changed our string)
          buf.Replace(nStart, nEnd, _strDelimiter);
          i = nStart + _strDelimiter.Length - 1;
        }
        i++;
      }
      //SS - Now we check to correct for ends-with functionality
      String strIntermediate = buf.ToString();
      XPStringBuilder bufIntermediate = new XPStringBuilder(strIntermediate);
      nEnd = strIntermediate.Length;
      nStart = strIntermediate.Length - _strDelimiter.Length;
      //SS - Check to see if the string already ends with our delimiter
      if (XPStringUtility.CompareIgnoreCase(bufIntermediate.Substring(nStart,
       nEnd), _strDelimiter) == 0)
      {
        bAlreadyEndsWith = true;
      }
      //SS - If we have to end with the delimiter and it's already there, return the string.
      //Otherwise add it and return the string.
      if (_bForceEndsWith)
      {
        if (bAlreadyEndsWith)
        {
          return strIntermediate;
        }
        buf.Append(_strDelimiter);
        return (buf.ToString());
      }
      //SS - We do not want the string to return with the delimiter so
      //return the string if it doesn't end with the delimiter,
      //or remove the delimiter if the string does end with it (the delimiter)
      if (!bAlreadyEndsWith)
      {
        return strIntermediate;
      }
      buf.Remove(nStart, nEnd);
      return (buf.ToString());
    }
    
    /// <summary>
    /// replace substrings within string.
    /// </summary>
    /// <param name="s">string to be modified</param>
    /// <param name="sub">sub-string to search for</param>
    /// <param name="with">string to replace sub-string</param>
    /// <returns>
    /// string replaced with new sub-string or the unmodified
    /// string if no match could be found
    /// </returns>
    public static String Replace(String s, String sub, String with)
    {
      int c = 0;
      int i = XPStringUtility.IndexOf(s, sub, c);
      if (i == -1)
      {
        return s;
      }
      XPStringBuilder buf = new XPStringBuilder(s.Length + with.Length);
      do
      {
        buf.Append(XPStringUtility.Substring(s, c, i));
        buf.Append(with);
        c = i + sub.Length;
      } while ((i = XPStringUtility.IndexOf(s, sub, c)) != -1);
      if (c < s.Length)
      {
        buf.Append(XPStringUtility.Substring(s, c, s.Length));
      }
      return buf.ToString();
    }
    
    /// <summary>
    /// Adds spaces into long strings so that it can be displayed with wrapping
    /// in HTML without having to truncate.
    /// </summary>
    public static String BreakDownLongWordsInString(String strIn, int maxCharsPerWord)
    {
      XPStringTokenizer tokenizer = new XPStringTokenizer(strIn, " ");
      strIn = "";
      String strToken;
      String strTokenSubstring;
      while (tokenizer.HasMoreTokens())
      {
        strToken = tokenizer.GetNextToken();
        if (strToken.Length <= maxCharsPerWord)
        {
          strIn+=(strToken + " ");
        } else
        {
          while (strToken.Length > maxCharsPerWord)
          {
            strTokenSubstring = XPStringUtility.Substring(strToken, 0, maxCharsPerWord);
            strIn+=(strTokenSubstring + " ");
            strToken = XPStringUtility.Substring(strToken, maxCharsPerWord,
             strToken.Length);
          }
          strIn+=(strToken + " ");
        }
      }
      return strIn;
    }
    
    /// <summary>
    /// Description: This function returns a shortened version of a string that
    /// is too long. It will shorten the string to intMaxNumberOfCharacters and
    /// append a "..."
    /// </summary>
    /// <param name="strInput">String to check.</param>
    /// <param name="nMaxChars">
    /// Maximum number of characters in a string before
    /// truncation.  The length truncated string will
    /// be no longer than this number plus an additional
    /// 3 characters for the '...'.
    /// </param>
    /// <param name="bMaximumWordLength">
    /// variable is to specify that you want to
    /// shorten the string only if it has a word in it
    /// longer than nMaxChars.
    /// bMaximumWordLength is used to determine whether or not any word in the
    /// string will be too long and cause the HTML to look bad.
    /// Warning: If the string is going to be HTML Encoded, remember to truncate it
    /// first, and then HTML encode it.  Otherwise you can get truncated international
    /// language codes such as &#... instead of an actual character.
    /// Examples
    /// --------
    /// TruncateLongString("123456789 123456 12345678 123", 10, True)
    /// Returns: "123456789 123456 12345678 123"
    /// TruncateLongString("123456789 123456 12345678 123", 10, False)
    /// Returns: "123456789..."
    /// TruncateLongString("123456789 123456 1234567890123 123", 12, True)
    /// Returns: "123456789 12..."
    /// TruncateLongString("123456789 123456 1234567890123 123", 10, False)
    /// Returns: "123456789..."
    /// TruncateLongString("123456789 123456 1234567890123 123", 70, False)
    /// Returns: "123456789 123456 1234567890123 123"
    /// TruncateLongString("1234", 2, False)
    /// Returns: "12..."
    /// </param>
    public static String TruncateLongString(String strInput, int nMaxChars,
     bool bMaximumWordLength)
    {
      if (!(strInput.Length > nMaxChars))
      {
        // If the string is already small enough, just return it.
        return strInput;
      }
      if (bMaximumWordLength)
      {
        // We are going to search the string for any words over maxChars
        // and only truncate if we find long words.
        int nWordLength = 0;
        for (int i = 0;i < strInput.Length;i++, nWordLength++)
        {
          if (' ' == XPStringUtility.CharAt(strInput, i))
          {
            // We are at a new word.  Reset the word length counter.
            nWordLength = 0;
          }
          if (nWordLength >= nMaxChars)
          {
            // There is a word somewhere in the string that is too
            // long, so just truncate the string
            return (XPStringUtility.Trim(XPStringUtility.Substring(strInput,
             0, nMaxChars)) + "...");
          }
        }
        return strInput;
      } else
      {
        // If we're not worried about long words, just truncate the string.
        return (XPStringUtility.Trim(XPStringUtility.Substring(strInput, 0,
         nMaxChars)) + "...");
      }
    }
    
    /// <summary>
    /// This method escapes a String for use in Javascript.
    /// This code was adapted from Studio's code (Patrick Corcoran)
    /// </summary>
    /// <param name="String">the String to be escaped</param>
    /// <returns>String the escaped version of the input</returns>
    public static String EscapeJavascript(String strInput)
    {
      if (strInput == null)
      {
        return "";
      }
      int n = strInput.Length;
      XPStringBuilder sb = new XPStringBuilder(n);
      for (int i = 0;i < n;i++)
      {
        char c = XPStringUtility.CharAt(strInput, i);
        switch (c){
        case '\n':
        sb.Append("\\n");
        break;
        case '\r':
        sb.Append("");
        break;
        case '\t':
        sb.Append(" ");
        break;
        case '\'':
        sb.Append("\\\'");
        break;
        case '\"':
        sb.Append("&quot;");
        break;
        //case '"':
        //	sb.Append("\\u0022");
        //
        //	break;
        case '\\':
        sb.Append("\\\\");
        break;
        case '\u00a5':
        sb.Append("&yen;");
        break;
        case '<':
        sb.Append("\\u003c");
        break;
        case '>':
        sb.Append("\\u003e");
        break;
        default:
        sb.Append(c);
        break;
        }
      }
      return sb.ToString();
    }
    
    /// <summary>
    /// This method takes a String with numbers separated by ','s and splits it into
    /// an int array.  If there is an error, the rest of the array is filled with 0s.
    /// </summary>
    /// <param name="String">the comma separated string of numbers to convert</param>
    /// <returns>int[] the result</returns>
    public static int[] ConvertToInts(String data)
    {
      if (data == null)
      {
        return new int[0];
      }
      XPStringTokenizer st = new XPStringTokenizer(data, ",");
      int[] result = new int[st.GetTokenCount()];
      int i = 0;
      try
      {
        for (i = 0;st.HasMoreTokens();i++)
        {
          result[i] = XPConvert.ToInteger(st.GetNextToken());
        }
      }
      catch (Exception e)
      {
        log.Warn(e, "ConvertToInts: Bad data - expected numeric data separated by \',\'s, got : " + data);
        // null out rest of array
        for (;i < result.Length;i++)
        {
          result[i] = 0;
        }
      }
      return result;
    }
    
    /// <summary>
    /// This method converts the expanded object types int[] into a comma
    /// separated String
    /// </summary>
    /// <param name="int[]"></param>
    /// <returns>String</returns>
    public static String ConvertToString(int[] nInts)
    {
      if ((nInts == null) || (nInts.Length < 1))
      {
        return "";
      }
      // Build the string
      XPStringBuilder sb = new XPStringBuilder(nInts.Length * 3);
      for (int i = 0;i < nInts.Length;i++)
      {
        sb.Append(nInts[i]);
        if (i < (nInts.Length - 1))
        {
          sb.Append(",");
        }
      }
      return sb.ToString();
    }
    
    /// <summary>
    /// This helper function sets a UI Error with the messages specified, and optionally
    /// throws an exception.  The UI Error will contain the strings specified only, but the
    /// Debug log entry will also contain the text of the exception (e.getMessage())
    /// </summary>
    /// <author>- JohnH</author>
    /// <param name="asOwner">- The activity space in which the error was thrown</param>
    /// <param name="strFunctionName">- The function that trapped the error (e.g. MyModel::CheckEditorAccessAndStart)</param>
    /// <param name="uiComponent">- The UI component in which the error occurred, e.g. OpenLogService.GetComponent("Portal_UI_Infrastructure")</param>
    /// <param name="e">- The exception that has been thrown</param>
    /// <param name="bThrow">- Throw an XPException in addition to setting the error?</param>
    /// <param name="uiErrorLevel">- The error level (e.g. UIErrorLevel.SERIOUS)</param>
    /// <param name="strErrorMessage">- The error message string to be displayed</param>
    public static void SetUIError(AActivitySpace asOwner, OpenLogComponent uiComponent,
     String strFunctionName, Exception e, bool bThrow, UIErrorLevel uiErrorLevel,
     String strErrorMessage)
    {
      XPStringBuilder sbShortMsg = new XPStringBuilder(100);
      XPStringBuilder sbLongMsg = new XPStringBuilder(150);
      //Build the error string
      sbShortMsg.Append(strErrorMessage);
      if (null != e)
      {
        sbLongMsg.Append(strErrorMessage).Append(e.Message);
      } else
      {
        sbLongMsg.Append(strErrorMessage);
      }
      //Raise the UI error
      UIError myError = new UIError(uiErrorLevel, UIErrorScope.CURRENT_USER,
       null, sbShortMsg.ToString(), sbLongMsg.ToString());
      UIErrorHandler.RaiseError(myError, asOwner.GetSubSession(), asOwner.GetApplication());
      //Add the exception text and trace it
      if (null != e)
      {
        log.Error(e, strFunctionName + sbShortMsg.ToString());
      } else
      {
        log.Error(strFunctionName + sbShortMsg.ToString());
      }
      //Optionally throw the exception
      if (bThrow)
      {
        if (null != e)
        {
          throw XPException.GetInstance(strErrorMessage, e);
        } else
        {
          throw new XPException(strErrorMessage);
        }
      }
    }
    
    /// <summary>
    /// Helper method that checks whether or not a string contains unsafe
    /// characters, such as angle braces for HTML or curly braces for
    /// javascript.
    /// This is useful for determining if someone is trying to do a javascript hack.
    /// </summary>
    /// <param name="String">the string to process</param>
    /// <returns>boolean true implies log a message in PTSpy if the string contains unsafe characters.</returns>
    public static bool ContainsUnsafeCharacters(String _strStringIn, bool bLogErrors)
    {
      if (null == _strStringIn)
      {
        return false;
      }
      if ((XPStringUtility.IndexOf(_strStringIn, '<') != -1) || (XPStringUtility.IndexOf(_strStringIn,
       '>') != -1) || (XPStringUtility.IndexOf(_strStringIn, '{') != -1) || (XPStringUtility.IndexOf(_strStringIn,
       '}') != -1) || (XPStringUtility.IndexOf(_strStringIn, ';') != -1) || (XPStringUtility.IndexOf(_strStringIn,
       '\'') != -1) || (XPStringUtility.IndexOf(_strStringIn, '\"') != -1) || (XPStringUtility.IndexOf(_strStringIn,
       '\\') != -1))
      {
        if (bLogErrors)
        {
          log.Error("Query string contained unsafe characters, such as angle braces or curly braces.  Possible hack attempt: " + _strStringIn);
          }
        return true;
      }
      return false;
    }
  }
}