using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.help;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.statichelpers
{
  // NOTE: this should be moved to htmlconstructs project (in common, Ben's code) to the static
  // helpers there.
  public class HTMLConstructHelpers
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.statichelpers.HTMLConstructHelpers");
    public const String STR_508_HELP_DIR = "508";
    public const String STR_STANDARD_HELP_DIR = "std";
    public static HTMLElement GetSubHeader(String strText, AActivitySpace sp)
    {
      //	Get the banner, set the title
      PTBannerType myType = PTBannerType.DIRECTORY_CONTENT;
      PTConstruct construct;
      try
      {
        construct = UIConstructFactory.GetConstruct(myType);
        //	Set the title on the construct.
        //	ReplaceNestedElementsWithHTMLString() takes an ElementName, and a string.
        //	It finds the element within the construct having the specified ElementName,
        //	and sets the inner html of this element to be the specified string
        construct.ReplaceNestedElementsWithHTMLString(myType.TITLE_ELEMENT_NAME,
         strText);
        HTMLAnchor help = HTMLConstructHelpers.GetHelpAnchor(sp);
        if (sp.GetAccessStyle() != AccessStyles.STANDARD)
        {
          help.SetAccessKey(sp.GetString(11, "ptmsgs_portal508quickkeymsgs"));
          }
        construct.AddElementToNestedElement(PTBannerType.DIRECTORY_CONTENT.HELP_CONTAINER_ELEMENT_NAME,
         help);
      }
      catch (Exception e)
      {
        log.Error("HTMLConstructHelpers::GetSubHeader: " + sp.GetString(74,
         ResourceConstants.INFRASTRUCTURE) + "  " + "The following exception occured: " + e);
        throw new XPException(sp.GetString(74, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return construct.GetHTMLElement();
    }
    public static String Get508HelpBaseUrl(AActivitySpace sp)
    {
      String sHelpUrl = ConfigHelper.GetPortalImageServerHelpURL(sp);
      String sLang = null;
      try
      {
        sLang = (String) PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE,
         sp);
      }
      catch (Exception ex)
      {
        sLang = "en";
      }
      XPStringBuilder sb = new XPStringBuilder(300);
      sb.Append(sHelpUrl);
      sb.Append(STR_508_HELP_DIR);
      sb.Append("/");
      sb.Append(sLang);
      sb.Append("/");
      return sb.ToString();
    }
    public static String GetRoboHelpBaseUrl(AActivitySpace sp)
    {
      String sHelpUrl = ConfigHelper.GetPortalImageServerHelpURL(sp);
      String sLang = null;
      try
      {
        sLang = (String) PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE,
         sp);
      }
      catch (Exception ex)
      {
        sLang = "en";
      }
      XPStringBuilder sb = new XPStringBuilder(300);
      sb.Append(sHelpUrl);
      // We know that robohelp is always standard access style, because
      // robohelp does not support 508.
      sb.Append(STR_STANDARD_HELP_DIR);
      sb.Append("/");
      sb.Append(sLang);
      // This returns the main portal help page, and the javascript link
      // then points the main page to the specific sub-page.
      sb.Append("/portalhelp.htm");
      return sb.ToString();
    }
    public static String GetHelpTopic(AActivitySpace sp)
    {
      String sHelpTopic = sp.GetHelpTopic();
      if (sHelpTopic == null)
      {
        sHelpTopic = "portalhelp";
      }
      return XPStringUtility.ToLowerCase(sHelpTopic);
    }
    public static String Get508HelpUrl(AActivitySpace sp)
    {
      String strCurrentRequestURL = sp.GetCurrentRequestURL();
      // Always ends with ?
      String strBaseURL = sp.GetCurrentNormalizedBaseURL();
      // Make sure both URLS either have or don't have ?'s
      if (!XPStringUtility.EndsWith(strCurrentRequestURL, "?"))
      {
        if (XPStringUtility.EndsWith(strBaseURL, "?"))
        {
          strBaseURL = XPStringUtility.Substring(strBaseURL, 0, strBaseURL.Length - 1);
          }
      }
      // If this is a POST, link to the current space.
      if (0 == XPStringUtility.CompareIgnoreCase(strBaseURL, strCurrentRequestURL))
      {
        ASURL thisSpace = new ASURL(sp);
        thisSpace.SetLinkGetCachedSpace(sp);
        strCurrentRequestURL = thisSpace.GetURLAsString();
      }
      ASURL helpURL = new ASURL(sp);
      helpURL.SetLinkCreateNewSpace(HelpFrameAS.STR_MVC_CLASS_NAME, sp);
      helpURL.SetControl(HelpControl.STR_MVC_CLASS_NAME);
      helpURL.AddControlArgument(HelpControl.STR_HELP_TOPIC, GetHelpTopic(sp));
      helpURL.AddControlArgument(HelpControl.STR_PREVIOUS_URL, strCurrentRequestURL);
      helpURL.SetDisplayPage(HelpFrameSetDP.STR_MVC_CLASS_NAME);
      return helpURL.GetURLAsString();
    }
    public static String GetHelpUrlForXUIPage(AActivitySpace sp)
    {
      return ImgSvrURLHelper.GetInstance().GetRootURL(sp.GetIsSecuredSpace()) + GetHelpTopic(sp);
      }
    public static HTMLAnchor GetHelpAnchor(AActivitySpace sp)
    {
      HTMLElementCollection text = new HTMLElementCollection();
      text.AddInnerHTMLString(sp.GetString(337, ResourceConstants.PORTALCOMMONMSGS));
      return GetHelpAnchor(sp, text, sp.GetString(338, ResourceConstants.PORTALCOMMONMSGS));
      }
    public static HTMLAnchor GetHelpAnchor(AActivitySpace sp, HTMLElement innerElement,
     String strHelpHint)
    {
      HTMLAnchor help = new HTMLAnchor("#");
      AccessStyles accessStyles = sp.GetAccessStyle();
      if (accessStyles == AccessStyles.STANDARD)
      {
        String sHelpTopic = GetHelpTopic(sp);
        // JF- Special case for XUI pages. We have to backwards compatible with 4.5 XUI servers.
        if ((-1 != XPStringUtility.IndexOf(sHelpTopic, ".html")) || (-1 != XPStringUtility.IndexOf(sHelpTopic,
         ".htm")))
        {
          String strJSUtilVersionPrefix = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSUTIL);
          // a value of 1 for the IsFancyWindow argument of PTWindowUtil.openHelpWindow
          // means to use just scrollbars and nothing else, which is what we want for 
          // our help window - see PTUtil.js for more details.
          help.SetOnClick(strJSUtilVersionPrefix + "PTWindowUtil.openHelpWindow( \'" + HTMLConstructHelpers.GetHelpUrlForXUIPage(sp) + "\', \'600\', \'600\', 1 ); return false;");
          } else
        {
          if (0 != sHelpTopic.Length)
          {
            sHelpTopic = sHelpTopic + ".htm";
          }
          help.SetOnClick("PTRoboHelp.launchHelp( \'" + sHelpTopic + "\', \'" + HTMLConstructHelpers.GetRoboHelpBaseUrl(sp) + "\', \'600\', \'600\' ); return false;");
          }
      } else
      {
        help.SetHRef(HTMLConstructHelpers.Get508HelpUrl(sp));
      }
      HTMLImg myHelpIcon = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.ICON_HELP,
       true, sp).GetHTMLElement();
      help.AddInnerHTMLElement(myHelpIcon);
      myHelpIcon.SetAlt(strHelpHint);
      myHelpIcon.SetWidth("16");
      myHelpIcon.SetHeight("16");
      myHelpIcon.SetAlign(CommonHTMLStrings.ABSMIDDLE);
      myHelpIcon.SetBorder(CommonHTMLStrings.ZERO);
      help.SetTitle(strHelpHint);
      help.AddInnerHTMLElement(innerElement);
      return help;
    }
    
    /// <summary>
    /// This function returns an Action Bar "button", which is an HTMLAnchor with
    /// images, text, and an onclick event attached.  The first parameter
    /// specifies whether the button will have no (0), an Add (1), or a Create
    /// (2) image just to the left of the main image, typically the icon of the
    /// object being added or created.
    /// </summary>
    /// <author>JohnH</author>
    /// <param name="intAddOrCreate">- 0 for none, 1 for Add, 2 for Create</param>
    /// <param name="myImage">- The image to appear on the button</param>
    /// <param name="strImageAlt">- The alt tag to be associated with the above image</param>
    /// <param name="strButtonText">- The text to appear on the button</param>
    /// <param name="strOnClick">- The on click function to be attached to the button</param>
    /// <param name="sp">- The activity space from whence this function was invoked</param>
    /// <returns>HTMLAnchor</returns>
    public static HTMLAnchor GetActionBarAnchor(int intAddOrCreate, HTMLImg myImage,
     String strImageAlt, String strButtonText, String strOnClick, AActivitySpace sp)
    {
      HTMLAnchor myAnchor;
      HTMLSpan mySpan;
      HTMLImg myActionImage;
      PTConstruct myConstruct = null;
      try
      {
        myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
        myAnchor.SetOnClick(strOnClick);
        myAnchor.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        //Add the images to the "button"
        if (intAddOrCreate > 0)
        {
          //Add
          if (1 == intAddOrCreate)
          {
            //Add the ACTION_ADD image
            myConstruct = ConstructFactory.GetConstruct(PTImageType.ACTION_ADD);
            } else
          if (2 == intAddOrCreate)
          {
            //Add the ACTION_ADD image
            myConstruct = ConstructFactory.GetConstruct(PTImageType.ACTION_CREATE);
            }
          myActionImage = (HTMLImg) myConstruct.GetHTMLElement();
          myActionImage.SetAlt("");
          myActionImage.SetAlign("absmiddle");
          myAnchor.AddInnerHTMLElement(myActionImage);
        }
        //Now add the passed in image
        myAnchor.AddInnerHTMLElement(myImage);
        myImage.SetAlt(strImageAlt);
        myImage.SetAlign("absmiddle");
        //Add the text for the "button"
        if (!(strButtonText.Equals("")))
        {
          mySpan = new HTMLSpan();
          myAnchor.AddInnerHTMLElement(mySpan);
          mySpan.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);
          mySpan.AddInnerHTMLString(strButtonText);
          mySpan.AddInnerHTMLString(CommonHTMLStrings.SPACE);
          mySpan.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        }
      }
      catch (Exception e)
      {
        log.Error("HTMLConstructHelpers::GetActionBarAnchor: " + sp.GetString(74,
         ResourceConstants.INFRASTRUCTURE) + "  " + "The following exception occured: " + e);
        throw new XPException(sp.GetString(74, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return myAnchor;
    }
    
    /// <summary>
    /// Returns an "Action Bar" banner, an HTMLTable of style actionbar_object
    /// containing an arbitrary number of "buttons" (HTMLAnchors with images and
    /// text attached) or other HTMLElements.  The Elements are passed in as an
    /// collection, along with the view name and activity space where this banner
    /// will be displayed
    /// </summary>
    /// <author>JohnH</author>
    /// <param name="myElements">- An array of pre-constructed HTMLElements</param>
    /// <param name="strViewName">
    /// - The name of the view where this banner will be
    /// displayed
    /// </param>
    /// <param name="sp">- The activity space this banner will reside in</param>
    /// <returns>HTMLElement</returns>
    public static HTMLElement GetActionBar(HTMLElementCollection myElements,
     String strViewName, AActivitySpace sp)
    {
      HTMLTable myTable = new HTMLTable();
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLImg myImg;
      XPArrayList myArrayList;
      int intNumElements;
      try
      {
        myTable.SetPreComment(sp.GetString(93, ResourceConstants.INFRASTRUCTURE) + ": " + strViewName);
        myTable.SetWidth("100%");
        myTable.SetBorder("0");
        myTable.SetCellPadding("0");
        myTable.SetCellSpacing("0");
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);
        myArrayList = myElements.GetInnerHTMLElementsAsArrayList(false);
        intNumElements = myArrayList.GetSize();
        //Add the buttons
        for (int i = 0;i < intNumElements;i++)
        {
          //Add a cell
          myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          myCell.SetHeight("25");
          myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
          myCell.SetNoWrap(true);
          myCell.AddInnerHTMLElement((HTMLElement) myArrayList.GetElement(i));
          myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
          // divider
          if (i < (intNumElements - 1))
          {
            myCell = new HTMLTableCell();
            myRow.AddInnerHTMLElement(myCell);
            myCell.SetHeight("25");
            myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_DIVIDER);
            PTConstruct myConstruct = ConstructFactory.GetConstruct(PTImageType.SPACER);
            myImg = (HTMLImg) myConstruct.GetHTMLElement();
            myImg.SetAlt("");
            myImg.SetHeight("1");
            myImg.SetWidth("1");
            myImg.SetBorder("0");
            myCell.AddInnerHTMLElement(myImg);
          } else
          {
            myCell.SetWidth("100%");
          }
        }
        myTable.SetPostComment(sp.GetString(94, ResourceConstants.INFRASTRUCTURE) + ": " + strViewName);
        }
      catch (Exception e)
      {
        log.Error("HTMLConstructHelpers::GetActionBar: " + sp.GetString(74,
         ResourceConstants.INFRASTRUCTURE) + "  " + "The following exception occured: " + e);
        throw new XPException(sp.GetString(74, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return myTable;
    }
    
    /// <summary>
    /// This helper method determines the appropriate previous request URL
    /// to go pack to the previous page.  This will just go directly to the
    /// previous space if the last request was a POST.
    /// This method assumes that you are switching between Activity Spaces.
    /// It does not work well if you want to go back to the previous URL
    /// in the same space.
    /// This can also be a problem if the user refreshes the page, as this
    /// link will then point to the current space.
    /// </summary>
    /// <param name="asOwner"></param>
    /// <returns>String URL to go back to</returns>
    public static String GetPreviousSpaceURL(AActivitySpace asOwner)
    {
      String strPreviousRequestURL = (String) asOwner.GetSubSession().GetAttribute(ASConstants.PREVIOUS_REQUEST_URL);
      bool bPreviousRequestIsPost = false;
      Object oIsPost = asOwner.GetSubSession().GetAttribute(ASConstants.PREVIOUS_REQUEST_IS_POST);
      if (null != oIsPost)
      {
        bPreviousRequestIsPost = XPConvert.ToBoolean(oIsPost);
      }
      if (bPreviousRequestIsPost)
      {
        // If the previous request was a POST, we don't know the
        // actual URL, so just go to the cached space.
        AActivitySpace prevSpace = asOwner.GetParentSpace();
        if (null != prevSpace)
        {
          ASURL prevSpaceURL = new ASURL(asOwner);
          prevSpaceURL.SetLinkGetCachedSpace(prevSpace);
          return prevSpaceURL.GetURLAsString();
        }
      } else
      if ((null != strPreviousRequestURL) && (!"".Equals(strPreviousRequestURL)))
      {
        // Use the previous URL.
        return strPreviousRequestURL;
      }
      // If all else fails, just go to the base URL.
      return ConfigHelper.GetApplicationBaseURL(asOwner);
    }
  }
}