using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.activityspace;
namespace com.plumtree.uiinfrastructure.statichelpers
{
  /// <summary>
  /// This class contains helpers for dealing with HTTP, QS,
  /// JS, and other URL related issues.
  /// </summary>
  /// <author>RickyW</author>
  public class HTTPHelpers
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.statichelpers.HTTPHelpers");
    
    /// <summary>
    /// This helper method retrieves a single argument from the
    /// argument hashtable.  If the argument does not exist or
    /// is null, the default value will be used.  
    /// </summary>
    /// <param name="_htArguments">Arguments to reach the new space.</param>
    /// <param name="_strKey">The key for the argument you want.</param>
    /// <param name="_nDefault">Value to return if argument is null.</param>
    /// <returns>The specified argument from the argument hashtable. </returns>
    public static int GetArgumentAsInt(XPHashtable _htArguments, String _strKey,
     int _nDefault)
    {
      int nRetVal = _nDefault;
      String[] aStrValues = null;
      try
      {
        aStrValues = (String[]) _htArguments.GetElement(_strKey);
        if ((null != aStrValues) && (!aStrValues[0].Equals("")))
        {
          nRetVal = XPConvert.ToInteger(aStrValues[0]);
        }
      }
      catch (Exception ex)
      {
        log.Warn("Error retrieving int from arguments, using default value.");
        }
      return nRetVal;
    }
    
    /// <summary>
    /// This helper method retrieves a single argument from the
    /// argument hashtable.  If the argument does not exist or
    /// is null, the default value will be used.  
    /// </summary>
    /// <param name="_htArguments">Arguments to reach the new space.</param>
    /// <param name="_strKey">The key for the argument you want.</param>
    /// <param name="_strDefault">Value to return if argument is null.</param>
    /// <returns>The specified argument from the argument hashtable. </returns>
    public static String GetArgumentAsString(XPHashtable _htArguments, String _strKey,
     String _strDefault)
    {
      String strRetVal = _strDefault;
      String[] aStrValues = null;
      try
      {
        aStrValues = (String[]) _htArguments.GetElement(_strKey);
        if (null != aStrValues)
        {
          strRetVal = aStrValues[0];
        }
      }
      catch (Exception ex)
      {
        log.Warn("Error retrieving string from arguments, using default value.");
        }
      return strRetVal;
    }
    
    /// <summary>
    /// This helper method retrieves a single argument from the
    /// argument hashtable.  If the argument does not exist or
    /// is null, the default value will be used.  
    /// </summary>
    /// <param name="_htArguments">Arguments to reach the new space.</param>
    /// <param name="_strKey">The key for the argument you want.</param>
    /// <param name="_bDefault">Value to return if argument is null.</param>
    /// <returns>The specified argument from the argument hashtable. </returns>
    public static bool GetArgumentAsBoolean(XPHashtable _htArguments, String _strKey,
     bool _bDefault)
    {
      bool bRetVal = _bDefault;
      String[] aStrValues = null;
      try
      {
        aStrValues = (String[]) _htArguments.GetElement(_strKey);
        if (null != aStrValues)
        {
          bRetVal = XPConvert.ToBoolean(aStrValues[0]);
        }
      }
      catch (Exception ex)
      {
        log.Warn("Error retrieving boolean from arguments, using default value.");
        }
      return bRetVal;
    }
    
    /// <summary>
    /// This helper adds a QS name to your XPStringBuilder.  If the 
    /// argument name is null, nothing is added.
    /// </summary>
    /// <param name="_sbArgs">XPStringBuilder to append to.</param>
    /// <param name="_strArgName">Name of name/value pair.</param>
    public static void AppendQSArgument(XPStringBuilder _sbArgs, String _strArgName)
    {
      if (null != _strArgName)
      {
        _sbArgs.Append("&");
        _sbArgs.Append(_strArgName);
      }
    }
    
    /// <summary>
    /// This helper adds a QS name/value pair to your XPStringBuilder.  
    /// If the argument name or value is null or empty, nothing is added.
    /// </summary>
    /// <param name="_sbArgs">XPStringBuilder to append to.</param>
    /// <param name="_strArgName">Name of name/value pair.</param>
    /// <param name="_strArgValue">Value of name/value pair.</param>
    public static void AppendQSArgument(XPStringBuilder _sbArgs, String _strArgName,
     String _strArgValue)
    {
      AppendQSArgument(_sbArgs, _strArgName, _strArgValue, false);
    }
    
    /// <summary>
    /// This helper adds a QS name/value pair to your XPStringBuilder.  
    /// If the argument name or value is null or empty, nothing is added.
    /// </summary>
    /// <param name="_sbArgs">XPStringBuilder to append to.</param>
    /// <param name="_strArgName">Name of name/value pair.</param>
    /// <param name="_strArgValue">Value of name/value pair.</param>
    /// <param name="_bLeaveOutAmpersand">True to leave out the leading "&".</param>
    public static void AppendQSArgument(XPStringBuilder _sbArgs, String _strArgName,
     String _strArgValue, bool _bLeaveOutAmpersand)
    {
      if ((null != _strArgName) && (null != _strArgValue) && (!XPStringUtility.EqualsIgnoreCase(_strArgValue,
       "")))
      {
        if (!_bLeaveOutAmpersand)
        {
          _sbArgs.Append("&");
        }
        _sbArgs.Append(_strArgName);
        _sbArgs.Append("=");
        _sbArgs.Append(_strArgValue);
      }
    }
    
    /// <param name="_sbArgs"></param>
    /// <param name="_strArgName"></param>
    /// <param name="_nArgValue"></param>
    public static void AppendQSArgument(XPStringBuilder _sbArgs, String _strArgName,
     int _nArgValue)
    {
      AppendQSArgument(_sbArgs, _strArgName, _nArgValue, false);
    }
    
    /// <summary>
    /// This helper adds a QS name/value pair to your XPStringBuilder.  If 
    /// the argument name is null, or if the argument value is -1, nothing 
    /// is added.
    /// </summary>
    /// <param name="_sbArgs">XPStringBuilder to append to.</param>
    /// <param name="_strArgName">Name of name/value pair.</param>
    /// <param name="_nArgValue">Value of name/value pair.</param>
    /// <param name="_bLeaveOutAmpersand">True to leave out the leading "&".</param>
    public static void AppendQSArgument(XPStringBuilder _sbArgs, String _strArgName,
     int _nArgValue, bool _bLeaveOutAmpersand)
    {
      if ((null != _strArgName) && (-1 != _nArgValue))
      {
        if (!_bLeaveOutAmpersand)
        {
          _sbArgs.Append("&");
        }
        _sbArgs.Append(_strArgName);
        _sbArgs.Append("=");
        _sbArgs.Append(_nArgValue);
      }
    }
    
    /// <summary>
    /// This helper generates the QS argument string from the given
    /// argument name.  If the argument name is null, empty string
    /// is returned.
    /// </summary>
    /// <param name="_strArgName">Name of name/value pair.</param>
    /// <returns>Argument string to add to your QS.</returns>
    public static String GetQSArgString(String _strArgName)
    {
      if (null != _strArgName)
      {
        return "&" + _strArgName;
      } else
      {
        return "";
      }
    }
    
    /// <summary>
    /// This helper adds a QS name/value pair to your XPStringBuilder.  
    /// If the argument name or value is null or empty, nothing is added.
    /// </summary>
    /// <param name="_strArgName">Name of name/value pair.</param>
    /// <param name="_strArgValue">Value of name/value pair.</param>
    /// <returns>Argument string to add to your QS.</returns>
    public static String GetQSArgString(String _strArgName, String _strArgValue)
    {
      if ((null != _strArgName) && (null != _strArgValue) && (!XPStringUtility.EqualsIgnoreCase(_strArgValue,
       "")))
      {
        return "&" + _strArgName + "=" + _strArgValue;
      } else
      {
        return "";
      }
    }
    
    /// <summary>
    /// This helper adds a QS name/value pair to your XPStringBuilder.  If 
    /// the argument name is null, or if the argument value is -1, nothing 
    /// is added.
    /// </summary>
    /// <param name="_strArgName">Name of name/value pair.</param>
    /// <param name="_nArgValue">Value of name/value pair.</param>
    /// <returns>Argument string to add to your QS.</returns>
    public static String GetQSArgString(String _strArgName, int _nArgValue)
    {
      if ((null != _strArgName) && (-1 != _nArgValue))
      {
        return "&" + _strArgName + "=" + _nArgValue;
      } else
      {
        return "";
      }
    }
    
    /// <summary>
    /// Helper method to determine if the given string contains characters
    /// that we have deemed to be unsafe for URLs, indicating possible 
    /// hack attempts.
    /// </summary>
    /// <author>JohnO</author>
    /// <param name="__strInput"></param>
    /// <returns>True if string contains unsafe characters.</returns>
    public static bool StringHasUnsafeURLCharacters(String _strInput)
    {
      if (null == _strInput)
      {
        return false;
      }
      if ((XPStringUtility.IndexOf(_strInput, '<') != -1) || (XPStringUtility.IndexOf(_strInput,
       '>') != -1) || (XPStringUtility.IndexOf(_strInput, ';') != -1) || (XPStringUtility.IndexOf(_strInput,
       '\"') != -1) || (XPStringUtility.IndexOf(_strInput, '\'') != -1) || (XPStringUtility.IndexOf(_strInput,
       '\\') != -1))
      {
        log.Error("Unsafe URL characters detected, possible hack attempt: " + _strInput);
        return true;
      }
      return false;
    }
    
    /// <summary>
    /// Returns a query string for the redirect containing all the control arguments.
    /// Copied from com.plumtree.uiinfrastructure.activityspace.Redirect.
    /// </summary>
    /// <returns>String The querystring, excluding the question mark.</returns>
    public static String GetQueryString(XPHashtable _htArguments)
    {
      XPStringBuilder sb = new XPStringBuilder();
      IXPEnumerator enumerator = _htArguments.GetKeys();
      // mj: iterate over all control arguments
      while (enumerator.MoveNext())
      {
        String strName = XPHttpUtility.UrlEncode((String) enumerator.GetCurrent(),
         LinkConstants.UTF_8_ENCODING);
        String[] arrValues = (String[]) _htArguments.GetElement(strName);
        // mj: check whether there are any values
        if ((arrValues != null) && (arrValues.Length > 0))
        {
          for (int i = 0;i < arrValues.Length;i++)
          {
            // mj: append the attribute name
            sb.Append(strName).Append('=');
            // mj: if this value is null, use an empty string
            if (arrValues[i] != null)
            {
              sb.Append(XPHttpUtility.UrlEncode(arrValues[i], LinkConstants.UTF_8_ENCODING));
              }
            sb.Append('&');
          }
        } else
        {
          // mj: no values; append an empty string
          sb.Append(strName).Append('=').Append('&');
        }
      }
      // mj: remove the trailing ampersand, if any
      if (sb.GetLength() > 0)
      {
        sb.SetLength(sb.GetLength() - 1);
      }
      return sb.ToString();
    }
    
    /// <returns></returns>
    public static String GetParameterArrayAsString(String[] listParamNames,
     String[] listParamValues)
    {
      CheckIntegrityOfParameterArrays(listParamNames, listParamValues);
      if (null == listParamNames && null == listParamValues)
      {
        return null;
      }
      XPStringBuilder buf = new XPStringBuilder(listParamNames.Length * 8 * 2);
      buf.Append("&");
      for (int i = 0;i < listParamNames.Length;i++)
      {
        buf.Append(listParamNames[i] + "=" + listParamValues[i] + "&");
      }
      // remove last & char.
      buf.RemoveCharAt(buf.GetLength() - 1);
      return buf.ToString();
    }
    
    /// <param name="strParamNames"></param>
    /// <param name="strParamValues"></param>
    /// <returns></returns>
    public static void CheckIntegrityOfParameterArrays(String[] strParamNames,
     String[] strParamValues)
    {
      String errorMsg = null;
      if (null == strParamNames && null == strParamValues)
      {
        return;
      }
      // They both can't be null so one of the arguments are null, error 
      if (null == strParamNames)
      {
        errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamNames argument has a null value.";
        } else
      if (null == strParamValues)
      {
        errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamValues argument has a  null value.";
        }
      // Length of arrays don't match
      if ((strParamNames != null) && (strParamNames.Length != strParamValues.Length))
      {
        errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamNames.length = " + strParamNames.Length + " strParamValues.length = " + strParamValues.Length;
        }
      if (null != errorMsg)
      {
        throw new XPIllegalArgumentException(errorMsg);
      }
    }
    
    /// <returns></returns>
    public static String GetParameterListAsString(XPArrayList listParamNames,
     XPArrayList listParamValues)
    {
      CheckIntegrityOfParameterArrays(listParamNames, listParamValues);
      if (null == listParamNames && null == listParamValues)
      {
        return null;
      }
      XPStringBuilder buf = new XPStringBuilder(listParamNames.GetSize() * 8 * 2);
      buf.Append("&");
      for (int i = 0;i < listParamNames.GetSize();i++)
      {
        buf.Append(listParamNames.GetElement(i) + "=" + listParamValues.GetElement(i) + "&");
        }
      // remove last & char.
      buf.RemoveCharAt(buf.GetLength() - 1);
      return buf.ToString();
    }
    
    /// <param name="strParamNames"></param>
    /// <param name="strParamValues"></param>
    /// <returns></returns>
    public static void CheckIntegrityOfParameterArrays(XPArrayList strParamNames,
     XPArrayList strParamValues)
    {
      String errorMsg = null;
      if (null == strParamNames && null == strParamValues)
      {
        return;
      }
      // They both can be null so one of the arguments are null, error 
      if (null == strParamNames)
      {
        errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamNames argument has a null value.";
        } else
      if (null == strParamValues)
      {
        errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamValues argument has a  null value.";
        }
      // Length of arrays don't match
      if ((strParamNames != null) && (strParamNames.GetSize() != strParamValues.GetSize()))
      {
        errorMsg = "Length of strParamNames and strParamValues arrays does not match,  strParamNames.length = " + strParamNames.GetSize() + " strParamValues.length = " + strParamValues.GetSize();
        }
      if (null != errorMsg)
      {
        throw new XPIllegalArgumentException(errorMsg);
      }
    }
  }
}