using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
using com.plumtree.openlog;
using com.plumtree.uiinfrastructure.interpreter.filter.utils;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.statichelpers
{
  /// <author>daniilk</author>
  public class HttpUtils
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.statichelpers.HttpUtils");
    public const String HTTP_SCHEME = "http";
    public const String HTTPS_SCHEME = "https";
    public const String HASHTABLE_CONTROL_KEY_URL = "MyURL";
    public const String HASHTABLE_CONTROL_KEY_HTTP_HEADERS = "RequestHTTPHeaders";
    public const String RETURN_URI = "MyReturnURL";
    
    /// <param name="strRawHeaders"></param>
    /// <param name="strHeaderNameToFind"></param>
    /// <returns></returns>
    public static String GetHeaderValue(String strRawHeaders, String strHeaderNameToFind)
    {
      if (null != strRawHeaders)
      {
        int iHeaderStart = 0;
        int iHeaderEnd = XPStringUtility.IndexOf(strRawHeaders, "\r\n");
        while (iHeaderEnd > -1)
        {
          String strHeaderName = "";
          String strHeaderValue = "";
          /* Get the current header */
          String strHeader = XPStringUtility.Substring(strRawHeaders, iHeaderStart,
           iHeaderEnd);
          /* move to the next header, since we are not using this variables later
           * in the loop anyways and they need to be updated no-matter-what, so
           * i'm updating them here.
           */
           
          iHeaderStart = iHeaderEnd + 2;
          iHeaderEnd = XPStringUtility.IndexOf(strRawHeaders, "\r\n", iHeaderStart);
          int iHeaderNameEnd = XPStringUtility.IndexOf(strHeader, ":");
          if (iHeaderNameEnd < 0)
          {
            /* skip this wierd line with no ":". BTW, this will happen for the
             * first line as it contains the status code and looks like
             * HTTP/1.1 200 OK and obviously does not contain a colon.
             */
             
            continue;
          }
          strHeaderName = XPStringUtility.Substring(strHeader, 0, iHeaderNameEnd);
          /* we want to skip ":", hence "+1" */
          strHeaderValue = XPStringUtility.Substring(strHeader, iHeaderNameEnd + 1,
           strHeader.Length);
          if (XPStringUtility.EqualsIgnoreCase(strHeaderName, strHeaderNameToFind))
          {
            return strHeaderValue;
          }
        }
      }
      return "";
    }
    
    /// <param name="request"></param>
    /// <returns></returns>
    public static String GetRawHeaders(IXPRequest request)
    {
      XPStringBuilder strRawHeaders = new XPStringBuilder(10240);
      IXPEnumerator xpenumHeaders = request.GetHeaderNames();
      while (xpenumHeaders.MoveNext())
      {
        String strHeaderName = (String) xpenumHeaders.GetCurrent();
        String strHeaderValue = request.GetHeader(strHeaderName);
        strRawHeaders.Append(strHeaderName);
        strRawHeaders.Append(": ");
        strRawHeaders.Append(strHeaderValue);
        strRawHeaders.Append("\r\n");
      }
      return strRawHeaders.ToString();
    }
    
    /// <summary>
    /// This method calls XPHttpUtility.UrlDecode, which is fairly
    /// expensive, and therefore this method should be used sparingly
    /// and the results should be cached per request (they are already
    /// cached on the ActivitySpace and RequestData objects).
    /// </summary>
    /// <param name="xpRequest"></param>
    /// <returns>the request full URL</returns>
    public static String GetRequestFullURL(IXPRequest xpRequest)
    {
      /* NOTE: different security modes need to be handled correctly!!! */
      String strQS = xpRequest.GetQueryString();
      String strRequestServer = xpRequest.GetServerName();
      int iRequestPort = xpRequest.GetServerPort();
      String strRequestScheme = xpRequest.GetScheme();
      // NOTICE: WE HAVE TO CALL GetRequestURI and NOT GetRequestURL because OpenFoundation
      // on .NET returns URL-decoded version of GetRequestURL and URL-encoded version for 
      // GetRequestURI (it is trickier than that, but if you want to verify it try 
      // URL of the form http://machinename/mytest/%25D1%2582%25D0%25B5%25D1%2581%25D1%2582.html)
      //
      // In Java you will get %25D1%2582%25D0%25B5%25D1%2581%25D1%2582.html from both calls
      // and on .NET you will get
      // 
      // %D1%82%D0%B5%D1%81%D1%82.html from GetRequestURI()
      // and
      // blah.html from GetRequestURL() where blah is russian word for test
      //
      // 		-- Moose, Martok release cycle.
      // 
      String strRequestURL = xpRequest.GetRequestURI();
      /* URL-decoded it in case it was encoded. */
      strRequestServer = strRequestScheme + "://" + strRequestServer + ":" + iRequestPort;
      strRequestURL = strRequestServer + strRequestURL;
      /* if it is not null and not an empty string. In java it is always null
       * in .NET it is an empty string.
       */
       
      if ((null != strQS) && (strQS.Length > 0))
      {
        strRequestURL+=("?" + strQS);
      }
      return strRequestURL;
    }
    
    /// <param name="strURL"></param>
    /// <param name="sm"></param>
    public static void SetReturnURI(String strURL, ISessionManager sm)
    {
      sm.SetAttribute(RETURN_URI, strURL);
    }
    public static String GetSafeURL(String strUnsafeURL)
    {
      /* Check if someone tries to fool us by spoofing the URL and trying
       * to add bogus headers that will be sent to the browser.
       *
       * /gateway/PTARGS_0_1_1952_0_0_43/http://moose/ShowHeaders1.asp?a=b%0D%0ASetCookie:%20BogusValue%0A%0D
       *
       * In this case what we get from OpenFoundation is a URL that is already
       * decoded (filed a bug on it) So, %0D%0A will be converted to "\r\n". Imagine a
       * situation when a user does not have access to this link (let's say she clicks on
       * it from the e-mail program and does not have a session yet). In this case we will
       * store this URL on a session in the decoded form and then when the user logs in
       * we will take this URL from the session and put is a value of the location header
       * in the redirect response. However, since the string on a session has NL and CR in it
       * we will actually redirect the user to [..../moose/ShowHeaders.asp?a=b] and also add
       * a new header that sets a cookie. This method here is just a cheesy work-around. The
       * real problem should be fixed in OpenFoundation. While GetParameterValue should
       * clearly return URL-decoded value of the specified parameter, call to GetQueryString
       * should NOT return URL-decoded version and instead should provide access to raw
       * URL that was sent by the browser including query string.
       *
       * The following code checks for that and just trunkates the string at the first
       * "\r" or "\n".
       *
       * 		-- moose
       */
       
      int iCRIndex = XPStringUtility.IndexOf(strUnsafeURL, "\r");
      int iNLIndex = XPStringUtility.IndexOf(strUnsafeURL, "\n");
      String strSafeURL = strUnsafeURL;
      if (iCRIndex > -1 || iNLIndex > -1)
      {
        log.Warn("GetSafeURL(): Possible hacker attack detected. URL [" + strUnsafeURL + "]");
        if (iNLIndex == -1)
        {
          strSafeURL = XPStringUtility.Substring(strUnsafeURL, 0, iCRIndex);
          } else
        if (iCRIndex == -1)
        {
          strSafeURL = XPStringUtility.Substring(strUnsafeURL, 0, iNLIndex);
          } else
        if (iCRIndex > iNLIndex)
        {
          strSafeURL = XPStringUtility.Substring(strUnsafeURL, 0, iNLIndex);
          } else
        {
          strSafeURL = XPStringUtility.Substring(strUnsafeURL, 0, iCRIndex);
          }
      }
      return strSafeURL;
    }
    public static String GetNormalizedURL(String strGWURL, bool bSecure)
    {
      String strNormBaseURL = ConfigHelper.GetNormalizedBaseURL(strGWURL);
      String strBaseURL = ConfigHelper.GetApplicationBaseURL(strNormBaseURL,
       bSecure);
      int iIndexGW = XPStringUtility.IndexOf(strGWURL, GatewayHandlers.GATEWAY_UNIQUE);
      int iIndexQS = XPStringUtility.IndexOf(strGWURL, "?");
      /* strBaseURL returns base url which ends with question mark.
       * In case someone will ever decide to fix it, we will take an
       * intelligent approach and search for "?" first, instead of blindly
       * removing the last character.
       * 		-- Moose
       */
       
      if (null != strBaseURL)
      {
        int iQuestionMarkLocation = XPStringUtility.IndexOf(strBaseURL, "?");
        if (-1 != iQuestionMarkLocation)
        {
          strBaseURL = XPStringUtility.Substring(strBaseURL, 0, iQuestionMarkLocation);
          }
      }
      // if this is NOT a gateway URL
      if (-1 == iIndexGW)
      {
        // and it does not have a query string
        if (-1 == iIndexQS)
        {
          // DCA - make sure we don't cut off friendly URLs
          String sHTTPEntryPoint = ConfigHelper.GetHTTPEntryPoint();
          int iIndexFURL = XPStringUtility.IndexOf(strGWURL, sHTTPEntryPoint + "/");
          if (-1 == iIndexFURL)
          {
            // return just a base URL
            return strBaseURL;
          } else
          {
            return strBaseURL + XPStringUtility.Substring(strGWURL, iIndexFURL + sHTTPEntryPoint.Length);
            }
        } else
        {
          // otherwise
          return strBaseURL + XPStringUtility.Substring(strGWURL, iIndexQS);
          }
      } else
      // this is a gateway URL
      {
        return strBaseURL + XPStringUtility.Substring(strGWURL, iIndexGW);
        }
    }
    public static String GetNormalizedURL(String strURL)
    {
      bool bAccessedThroughHTTPS = XPStringUtility.StartsWith(XPStringUtility.ToLowerCase(strURL),
       HTTPS_SCHEME);
      // modes 2 and 3 only allow secure URLs, so we just enforce them here.
      bool bSecureOnlyMode = ConfigHelper.GetSecurityMode() > 1;
      return GetNormalizedURL(strURL, bAccessedThroughHTTPS || bSecureOnlyMode);
      }
  }
}