using System;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.statichelpers
{
  /// <summary>
  /// This class is a helper class to insert "In-Place Refresh" functionality
  /// to your Activity Space.  Most of the helpers deal with inserting JS 
  /// method calls and functions into your views and display pages.
  /// </summary>
  /// <author>RickyW</author>
  /// <author>Caiusv</author>
  public class InPlaceRefreshHelper
  {
    
    /// <summary>
    /// Helper method to get the JavaScript call to sent an in place POST to
    /// the server, assuming you're on a page that uses AFormDP.  This method
    /// wraps in-place refresh functionality, allowing you to replace the 
    /// innerHTML of the specified ID.  The repostAction will enable the server
    /// to decide what content to return in the response.
    /// </summary>
    /// <param name="_strURL">The URL to post to, generally GetCurrentNormalizedBaseURL.</param>
    /// <param name="_refreshElementID">The ID of the element to replace innerHTML.</param>
    /// <param name="_nRepostAction">The repostAction for the server to decide content.</param>
    /// <param name="_strDocument">
    /// The name of the document where the in-place refresh
    /// happens. This should be identified if it ocurrs in another window
    /// for example. A _strDocument value of null defaults to 'document'
    /// </param>
    /// <returns>The JavaScript method call for SendPOSTOnAFormDP.</returns>
    public static String GetJSSendPOSTOnAFormDP(String _strURL, String _refreshElementID,
     int _nRepostAction, String _strDocument)
    {
      XPStringBuilder sbMethodJS = new XPStringBuilder(300);
      if (null == _strDocument)
      {
        _strDocument = "document";
      }
      String strJSXMLPrefixString = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSXML);
      if (XPStringUtility.EndsWith(strJSXMLPrefixString, "."))
      {
        strJSXMLPrefixString = XPStringUtility.Substring(strJSXMLPrefixString,
         0, strJSXMLPrefixString.Length - 1);
      }
      sbMethodJS.Append("    var myForm = " + _strDocument + ".mainform;\n");
      // TODOCR: What is this magic number 6?  Why don't we just calculate the value of myRepostAction
      // here in Java instead of making javascript do it?
      sbMethodJS.Append("    var myRepostAction = (" + _nRepostAction + ") ? " + _nRepostAction + " : 6;\n");
      sbMethodJS.Append("    myForm[\'in_hi_PostToSelf\'].value = " + _nRepostAction + ";\n");
      sbMethodJS.Append("    var formRequest = new " + strJSXMLPrefixString + ".PTHTTPPOSTRequest");
      sbMethodJS.Append("(");
      sbMethodJS.Append("\'").Append(_strURL).Append("\', ");
      sbMethodJS.Append("myForm, ");
      sbMethodJS.Append(_strDocument + ".getElementById(\'" + _refreshElementID + "\'), ");
      sbMethodJS.Append("\'sync\', ");
      sbMethodJS.Append("myRepostAction");
      sbMethodJS.Append(");\n");
      sbMethodJS.Append("    formRequest.invoke();");
      return sbMethodJS.ToString();
    }
    
    /// <summary>
    /// Helper method to get the JavaScript call to sent an in place POST to
    /// the server, assuming you're on a page that uses AFormDP.  This method
    /// wraps in-place refresh functionality, allowing you to replace the 
    /// innerHTML of the specified ID.  The repostAction will enable the server
    /// to decide what content to return in the response. This method calls the
    /// four argument version of the method supplying a document argument of null.
    /// </summary>
    /// <param name="_strURL">The URL to post to, generally GetCurrentNormalizedBaseURL.</param>
    /// <param name="_refreshElementID">The ID of the element to replace innerHTML.</param>
    /// <param name="_nRepostAction">The repostAction for the server to decide content.</param>
    /// <returns>The JavaScript method call for SendPOSTOnAFormDP.</returns>
    public static String GetJSSendPOSTOnAFormDP(String strURL, String refreshElementID,
     int nRepostAction)
    {
      return GetJSSendPOSTOnAFormDP(strURL, refreshElementID, nRepostAction,
       null);
    }
    
    /// <summary>
    /// Helper method to get the JavaScript call to sent an in place GET to
    /// the server, assuming you're on a page that uses AFormDP.  This method
    /// wraps in-place refresh functionality, allowing you to replace the 
    /// innerHTML of the specified ID.  The repostAction will enable the server
    /// to decide what content to return in the response.
    /// </summary>
    /// <param name="strURL">The URL to post to, generally GetCurrentNormalizedBaseURL.</param>
    /// <param name="refreshElementID">The ID of the element to replace innerHTML.</param>
    /// <param name="nRepostAction">The repostAction for the server to decide content.</param>
    /// <returns>The JavaScript method call for SendGETOnAFormDP.</returns>
    public static String GetJSSendGETOnAFormDP(String strURL, String refreshElementID,
     int nRepostAction)
    {
      XPStringBuilder sbMethodJS = new XPStringBuilder(300);
      String strJSXMLPrefixString = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSXML);
      if (XPStringUtility.EndsWith(strJSXMLPrefixString, "."))
      {
        strJSXMLPrefixString = XPStringUtility.Substring(strJSXMLPrefixString,
         0, strJSXMLPrefixString.Length - 1);
      }
      sbMethodJS.Append("    var myForm = document.mainform;\n");
      // TODOCR: What is this magic number 6?  Why don't we just calculate the value of myRepostAction
      // here in Java instead of making javascript do it?
      sbMethodJS.Append("    var myRepostAction = (" + nRepostAction + ") ? " + nRepostAction + " : 6;\n");
      sbMethodJS.Append("    myForm[\'in_hi_PostToSelf\'].value = " + nRepostAction + ";\n");
      sbMethodJS.Append("    var formRequest = new " + strJSXMLPrefixString + ".PTHTTPGETRequest");
      sbMethodJS.Append("(");
      sbMethodJS.Append("\'").Append(strURL).Append("\', ");
      sbMethodJS.Append("myForm, ");
      sbMethodJS.Append("document.getElementById(\"" + refreshElementID + "\"), ");
      sbMethodJS.Append("\'sync\', ");
      sbMethodJS.Append("myRepostAction");
      sbMethodJS.Append(");\n");
      sbMethodJS.Append("    formRequest.invoke();");
      return sbMethodJS.ToString();
    }
  }
}