using System;
using com.plumtree.uiinfrastructure.editor;
// Framework Imports
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.tips
{
  /// <summary>
  /// This control provides control functionality for views and models
  /// responsible for displaying error messages.
  /// </summary>
  /// <author>BenG</author>
  public class TipsControl : EditorHelperRepostControl
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.tips.TipsControl");
    public const String STR_MVC_CLASS_NAME = "TipsControl";
    public const String PARAM_CLOSE = "in_hi_close";
    //	todo: remove HACK: silly number to avoid conflict
    public const int POSTTOSELF_ACTION_TOGGLE_TIP = 7555;
    protected TipsModel m_model;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new TipsControl();
    }
    
    /// <summary>
    /// Returns the Class name.
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Calls the method associated to the repost action.
    /// Editors using this framework need to call that method from
    /// the PerformRepostAction method of the main RepostControl.
    /// </summary>
    /// <param name="_nAction"></param>
    /// <param name="_htFormData"></param>
    public override void PerformRepostAction(int _nAction, XPHashtable _htFormData)
    {
      log.Debug("PerformRepostAction()");
      if (_nAction == POSTTOSELF_ACTION_TOGGLE_TIP)
      {
        this.PerformToggleTipAction();
      }
    }
    
    /// <summary>
    /// Performs repost actions in Read-Only mode.
    /// </summary>
    /// <param name="_nAction"></param>
    protected virtual void PerformReadOnlyAction(int _nAction)
    {
      // JF- Implement this method if you need to execute some
      // code on repost when the Editor is Read-Only.
      if (_nAction == POSTTOSELF_ACTION_TOGGLE_TIP)
      {
        this.PerformToggleTipAction();
      }
    }
    public virtual void PerformToggleTipAction()
    {
      TipsModel tm = (TipsModel) base.m_asOwner.GetModel(TipsModel.STR_MVC_CLASS_NAME);
      bool bOld = tm.GetDoDisplayTipsBox();
      tm.SetDoDisplayTipsBox(!bOld);
    }
  }
}