using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.openlog;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.tips
{
  /// <summary>
  /// This is an abstract model extended by various concrete error model.
  /// </summary>
  /// <author>Beng</author>
  public class TipsModel : IModel,ITipsModelRO
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.tips.TipsModel");
    // TODO we should probably put these constants somewhere
    // so they can be used consistently.  this is being used
    // in HTMLConstructHelpers too.
    public const String STR_508_HELP_DIR = "508";
    public const String STR_STANDARD_HELP_DIR = "std";
    public const String STR_MVC_CLASS_NAME = "TipsModel";
    // JF- Associated Activity Space
    protected AActivitySpace m_asOwner;
    private HTMLElement eDefault = null;
    private HTMLElement eSecondaryTip = null;
    private HTMLElement eTip = null;
    private String sDefaultTipTitle = null;
    private String sSecondaryTipTile = null;
    private String sTipTitle = null;
    private String sTipGeneratingClass = null;
    private bool bDoDisplay = true;
    private bool bDoDisplaySet = false;
    private bool bDisplayGeneratingClass = false;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()"></seealso>
    public virtual void CleanupTempData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()"></seealso>
    public virtual void CleanupAllData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new TipsModel();
    }
    
    /// <returns></returns>
    public virtual HTMLElement GetDefaultTip()
    {
      return eDefault;
    }
    
    /// <returns></returns>
    public virtual String GetDefaultTipTitle()
    {
      return sDefaultTipTitle;
    }
    
    /// <returns></returns>
    public virtual bool GetDoDisplayTipsBox()
    {
      log.Debug("GetDoDisplayTipsBox()");
      if (!bDoDisplaySet)
      {
        ISessionManager sm = this.m_asOwner.GetSubSession();
        // This is cached on the session so that users can
        // turn tips on or off and it will follow them
        // across Activity Spaces.
        // NOTE: DonH guessed this from looking at the source.  May not be true.
        Object o = sm.GetAttribute("displayTips");
        if (o != null)
        {
          bDoDisplay = XPConvert.ToBoolean(o);
        } else
        {
          bDoDisplay = true;
        }
        bDoDisplaySet = true;
      }
      return bDoDisplay;
    }
    public virtual void SetGeneratingClass(String s)
    {
      sTipGeneratingClass = s;
    }
    public virtual String GetGeneratingClass()
    {
      return sTipGeneratingClass;
    }
    public virtual bool GetDisplayGeneratingClass()
    {
      return bDisplayGeneratingClass;
    }
    public virtual void SetDisplayGeneratingClass(bool b)
    {
      bDisplayGeneratingClass = b;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IModel#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <returns></returns>
    public virtual HTMLElement GetSecondaryTip()
    {
      return eSecondaryTip;
    }
    
    /// <returns></returns>
    public virtual String GetSecondaryTipTitle()
    {
      return sSecondaryTipTile;
    }
    
    /// <returns></returns>
    public virtual HTMLElement GetTip()
    {
      return eTip;
    }
    
    /// <returns></returns>
    public virtual String GetTipTitle()
    {
      return sTipTitle;
    }
    
    /// <summary>
    /// Inits the Model with the Activity Space.
    /// </summary>
    /// <param name="parent"></param>
    public virtual void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
      this.SetDefaultTipTitle(this.m_asOwner.GetString(1591, ResourceConstants.PORTALBROWSINGMSGS));
      HTMLElement eDefaultTip = new HTMLElementCollection();
      HTMLElement e;
      e = new HTMLGenericElement("Default Tip");
      try
      {
        eDefaultTip.AddInnerHTMLElement(e);
      }
      catch (Exception ex)
      {
      }
      this.SetDefaultTip(eDefaultTip);
    }
    
    /// <param name="e"></param>
    public virtual void SetDefaultTip(HTMLElement e)
    {
      eDefault = e;
    }
    
    /// <param name="s"></param>
    public virtual void SetDefaultTipTitle(String s)
    {
      sDefaultTipTitle = s;
    }
    
    /// <param name="b"></param>
    public virtual void SetDoDisplayTipsBox(bool b)
    {
      bDoDisplay = b;
      ISessionManager sm = this.m_asOwner.GetSubSession();
      // This is cached on the session so that users can
      // turn tips on or off and it will follow them
      // across Activity Spaces.
      // NOTE: DonH guessed this from looking at the source.  May not be true.
      sm.SetAttribute("displayTips", b);
    }
    
    /// <param name="e"></param>
    public virtual void SetSecondarTip(HTMLElement e)
    {
      HTMLElement eLocalized = GetModifiedTip(e);
      eSecondaryTip = eLocalized;
    }
    
    /// <param name="s"></param>
    public virtual void SetSecondarTipTitle(String s)
    {
      sSecondaryTipTile = s;
    }
    
    /// <param name="e"></param>
    public virtual void SetTip(HTMLElement e)
    {
      HTMLElement eLocalized = GetModifiedTip(e);
      eTip = eLocalized;
    }
    private HTMLElement GetModifiedTip(HTMLElement e)
    {
      //	todo, this is not done correctly.  There's no guarantee that
      //	e is ready to display.  This conversion must happen
      //	at display time.
      String sTip = e.GetDisplayString();
      String sHelpBaseUrl = ConfigHelper.GetPortalImageServerHelpURL(this.m_asOwner);
      String sLang = null;
      String sAccessStyle = null;
      try
      {
        sLang = (String) PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LANGUAGE,
         this.m_asOwner);
      }
      catch (Exception ex)
      {
        sLang = "en";
      }
      try
      {
        if (m_asOwner.GetIsAccessStyleStandard())
        {
          sAccessStyle = TipsModel.STR_STANDARD_HELP_DIR;
        } else
        {
          // TODO: This is not the correct way to support 508 Help.
          // This is not a priority because tips are not used in 508.
          // This should link to the HelpFrame to provide a back link.
          sAccessStyle = TipsModel.STR_508_HELP_DIR;
        }
      }
      catch (Exception ex)
      {
        sAccessStyle = TipsModel.STR_STANDARD_HELP_DIR;
      }
      XPStringBuilder sb = new XPStringBuilder(300);
      sHelpBaseUrl = XPStringUtility.ForceEndsWith(sHelpBaseUrl, "/");
      sb.Append(sHelpBaseUrl);
      sb.Append(sAccessStyle);
      sb.Append("/");
      sb.Append(sLang);
      sb.Append("/");
      String sHelpUrl = sb.ToString();
      sb = new XPStringBuilder(300);
      int i;
      // TODOCR: This is not a good way to do this and should be redone.
      String sPlaceHolder = "$$HBU$$";
      String sLastBit = sTip;
      i = XPStringUtility.IndexOf(sTip, sPlaceHolder);
      while (i != -1)
      {
        sb.Append(XPStringUtility.Substring(sTip, 0, i));
        sb.Append(sHelpUrl);
        sTip = XPStringUtility.Substring(sTip, i + sPlaceHolder.Length, sTip.Length);
        sLastBit = sTip;
        i = XPStringUtility.IndexOf(sTip, sPlaceHolder);
      }
      sb.Append(sLastBit);
      HTMLGenericElement eLocalized = new HTMLGenericElement(sb.ToString());
      return eLocalized;
    }
    
    /// <param name="s"></param>
    public virtual void SetTipTitle(String s)
    {
      sTipTitle = s;
    }
    public virtual void DisableTips()
    {
      // This will cause the TipsView to display nothing.
      eTip = null;
    }
  }
}