using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.htmlconstructs;
using com.plumtree.openlog;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.tips
{
  /// <author>Ben Geislinger</author>
  public class TipsView : IView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.tips.TipsView");
    public const String STR_MVC_CLASS_NAME = "TipsView";
    // Associated Activity Space
    protected AActivitySpace m_asOwner;
    // Associated read-only model
    protected ITipsModelRO m_model;
    private HTMLImg tipImg = null;
    //	private String _ui_tipsImgAlt = "Tips";
    //	private String _ui_tipsBannerTitle = "Tips Banner Title";
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new TipsView();
    }
    
    /// <summary>
    /// This method returns the HTMLElement that encompasses the output of the View.
    /// </summary>
    public virtual HTMLElement Display()
    {
      HTMLElement eOut = null;
      HTMLTable myTable = null;
      HTMLTableRow myRow = null;
      HTMLTableCell myCell = null;
      HTMLElement eTip = null;
      PTConstruct myConstruct;
      PTConstruct myButtonConstruct;
      PTButtonType buttonType;
      PTBannerType bannerType;
      HTMLAnchor anchor;
      HTMLSpan mySpan;
      eOut = new HTMLElementCollection();
      if (this.m_model.GetTip() == null)
      {
        return eOut;
      }
      if (tipImg == null)
      {
        tipImg = (HTMLImg) UIConstructFactory.GetConstruct(PTImageType.ICON_TIPS_BULB,
         true, this.m_asOwner).GetHTMLElement();
        tipImg.SetAlt(this.m_asOwner.GetString(1590, ResourceConstants.PORTALBROWSINGMSGS));
        }
      try
      {
        bannerType = PTBannerType.TIPS;
        myConstruct = UIConstructFactory.GetConstruct(bannerType, true, this.m_asOwner);
        myConstruct.ReplaceNestedElementsWithHTMLString(bannerType.TITLE_ELEMENT_NAME,
         this.m_asOwner.GetString(1591, ResourceConstants.PORTALBROWSINGMSGS));
        if (this.m_model.GetDoDisplayTipsBox())
        {
          buttonType = PTButtonType.COLLAPSE_TIPS;
        } else
        {
          buttonType = PTButtonType.EXPAND_TIPS;
        }
        myButtonConstruct = UIConstructFactory.GetConstruct(buttonType, true,
         this.m_asOwner);
        if (this.m_model.GetDoDisplayTipsBox())
        {
          ConstructFactory.SetAltOnButtonImages(myButtonConstruct, this.m_asOwner.GetString(1588,
           ResourceConstants.PORTALBROWSINGMSGS));
          ConstructFactory.SetTitleOnButtonImages(myButtonConstruct, this.m_asOwner.GetString(1588,
           ResourceConstants.PORTALBROWSINGMSGS));
        } else
        {
          ConstructFactory.SetAltOnButtonImages(myButtonConstruct, this.m_asOwner.GetString(1589,
           ResourceConstants.PORTALBROWSINGMSGS));
          ConstructFactory.SetTitleOnButtonImages(myButtonConstruct, this.m_asOwner.GetString(1589,
           ResourceConstants.PORTALBROWSINGMSGS));
        }
        myButtonConstruct.ReplaceNestedElementsWithHTMLString(buttonType.TITLE_ELEMENT_NAME,
         "");
        anchor = (HTMLAnchor) myButtonConstruct.GetNestedElement(buttonType.ANCHOR_ELEMENT_NAME);
        anchor.SetHRef(CommonHTMLStrings.POUND_SYMBOL);
        anchor.SetOnClick("postToSelf(" + TipsControl.POSTTOSELF_ACTION_TOGGLE_TIP + "); return false;");
        myConstruct.AddElementToNestedElement(bannerType.EXPAND_COLLAPSE_CONTAINER_ELEMENT_NAME,
         myButtonConstruct.GetHTMLElement());
        eOut.AddInnerHTMLElement(myConstruct.GetHTMLElement());
      }
      catch (Exception ex)
      {
        log.Error(ex, "Unable to display Tips View.");
      }
      if (this.m_model.GetDoDisplayTipsBox())
      {
        try
        {
          myTable = new HTMLTable();
          myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
          myTable.SetCellPadding(CommonHTMLStrings.FIVE);
          myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
          eOut.AddInnerHTMLElement(myTable);
          myRow = new HTMLTableRow();
          myRow.SetStyleClass(PTStyleClass.TIPS_LIGHT_BG);
          myTable.AddInnerHTMLElement(myRow);
          myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          String sTitle = this.m_model.GetTipTitle();
          if (sTitle == null)
          {
            sTitle = this.m_model.GetDefaultTipTitle();
          }
          if (sTitle != null)
          {
            mySpan = new HTMLSpan();
            mySpan.SetStyleClass(PTStyleClass.TIPS_TITLE);
            myCell.AddInnerHTMLElement(tipImg);
            myCell.AddInnerHTMLElement(mySpan);
            mySpan.AddInnerHTMLString(sTitle);
          }
          if (this.m_model.GetDisplayGeneratingClass())
          {
            String sGClass = this.m_model.GetGeneratingClass();
            if (sGClass != null)
            {
              String sOut;
              if (sGClass.Length > 30)
              {
                sOut = XPStringUtility.Substring(sGClass, 0, 30);
                sOut = sOut + "<br>";
                sOut = sOut + XPStringUtility.Substring(sGClass, 30, sGClass.Length);
                } else
              {
                sOut = sGClass;
              }
              myRow = new HTMLTableRow();
              myRow.SetStyleClass(PTStyleClass.TIPS_LIGHT_BG);
              myTable.AddInnerHTMLElement(myRow);
              myCell = new HTMLTableCell();
              myCell.SetStyleClass(PTStyleClass.TIPS_TEXT);
              myRow.AddInnerHTMLElement(myCell);
              myCell.AddInnerHTMLString("This tip set within the class: " + sOut);
              }
          }
          myRow = new HTMLTableRow();
          myRow.SetStyleClass(PTStyleClass.TIPS_LIGHT_BG);
          myTable.AddInnerHTMLElement(myRow);
          myCell = new HTMLTableCell();
          myCell.SetStyleClass(PTStyleClass.TIPS_TEXT);
          myRow.AddInnerHTMLElement(myCell);
          eTip = this.m_model.GetTip();
          if (eTip != null)
          {
            myCell.AddInnerHTMLElement(eTip);
          }
          eTip = this.m_model.GetSecondaryTip();
          if (eTip != null)
          {
            sTitle = this.m_model.GetSecondaryTipTitle();
            if (sTitle != null)
            {
              myRow = new HTMLTableRow();
              myRow.SetStyleClass(PTStyleClass.TIPS_SHADED_BG);
              myTable.AddInnerHTMLElement(myRow);
              myCell = new HTMLTableCell();
              myRow.AddInnerHTMLElement(myCell);
              mySpan = new HTMLSpan();
              mySpan.SetStyleClass(PTStyleClass.TIPS_SUBTITLE);
              myCell.AddInnerHTMLElement(mySpan);
              mySpan.AddInnerHTMLString(sTitle);
            }
            myRow = new HTMLTableRow();
            myRow.SetStyleClass(PTStyleClass.TIPS_SHADED_BG);
            myTable.AddInnerHTMLElement(myRow);
            myCell = new HTMLTableCell();
            myCell.SetStyleClass(PTStyleClass.TIPS_TEXT);
            myRow.AddInnerHTMLElement(myCell);
            myCell.AddInnerHTMLElement(eTip);
          }
        }
        catch (Exception ex)
        {
        }
      }
      //	do display tips box
      return eOut;
    }
    
    /// <summary>
    /// This method returns the HTMLScript that encompasses the Javascript on the page.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    /// <returns>HTMLScript</returns>
    public virtual HTMLScript DisplayJavascript()
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.xpshared.activityspace.IView#Init(IModel,AActivitySpace)"></seealso>
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      if (model == null)
      {
        log.Error("Initialized with null model.");
      }
      if (parent == null)
      {
        log.Error("Initialized with null model.");
      }
      m_model = (ITipsModelRO) model;
      m_asOwner = parent;
    }
  }
}