using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <author>Don Hayler</author>
  public abstract class ATreeAS : AFormAS,IFormAS
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.tree.ATreeAS");
    
    /// <summary>
    /// storage name for tree model (no matter what type of model it is) 
    /// </summary>
    public const String TREEMODEL = "TreeModel";
    
    /// <summary>
    /// storage name for ClosePopUpDP (used for closing the TreeDP only) 
    /// </summary>
    public const String TREECLOSEPOPUP = "TreeClosePopUp";
    
    /// <summary>
    /// anchor name for tree bar. 
    /// </summary>
    public const String TREE_BAR_ANCHOR = "treeBarAnchor";
    
    /// <summary>
    /// The maximum length of names of objects when displayed.
    /// This is actually the max length a word can be before the
    /// entire string gets truncated.
    /// </summary>
    public const int MAX_DISPLAYED_NAME_LENGTH = 40;
    // We are in a frame, so we need to call parent, not window
    
    /// <summary>
    /// Non-localized Strings 
    /// </summary>
    public const String JS_PARENT_CLOSE = "parent.close();\n";
    // member variables on objects returned by Javascript submit call
    
    /// <summary>
    /// The name of the javascript array member variable for the object name 
    /// </summary>
    public const String OBJECT_NAME = "Name";
    
    /// <summary>
    /// The name of the javascript array member variable for the object id 
    /// </summary>
    public const String OBJECT_OBJECT_ID = "ObjectID";
    
    /// <summary>
    /// The name of the javascript array member variable for the class id 
    /// </summary>
    public const String OBJECT_CLASS_ID = "ClassID";
    ///////////////////
    // Abstract Methods
    ///////////////////
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AActivitySpace#GetName()"></seealso>
    public abstract override String GetName();
    
    /// <summary>
    /// This method is called at the end of every HTTP request by the Interpreter.
    /// This method then calls IModel.CleanupTempData() on all IModels registered
    /// using RegisterModel().
    /// This method is intended for cleaning up temporary data that is only used
    /// for the lifetime of a single request, like DB query results.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AActivitySpace#CleanupTempData()"></seealso>
    public sealed override void CleanupTempData()
    {
      base.CleanupTempData();
      SetCurrentPage(GetPage(TreeFrameDP.STR_MVC_CLASS_NAME));
    }
    
    /// <summary>
    /// This method handles the ClosePopUpDP and passes everything else to the base
    /// implementation.  It adds javascript to the ClosePopUpDP that sets the selected
    /// items on the opener window (if it's a remote tree) and calls a javascript
    /// function on the opener window.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AActivitySpace#GetPage(String)"></seealso>
    public override IDisplayPage GetPage(String pageName)
    {
      if (ClosePopUpDP.STR_MVC_CLASS_NAME.Equals(pageName))
      {
        // This is called when TreeDP is canceled.
        // This should only be called if Cancel was clicked, not Finish.
        ClosePopUpDP result = (ClosePopUpDP) base.GetPage(pageName);
        try
        {
          HTMLScript myScript = new HTMLScript("text/javaScript");
          // We are in a frame, so we need to call parent, not window
          myScript.AddInnerHTMLString(JS_PARENT_CLOSE);
          result.SetScript(myScript);
        }
        catch (HTMLException he)
        {
          log.Error(he, "Error generating JS for ClosePopUpDP.");
        }
        return result;
      } else
      if (TREECLOSEPOPUP.Equals(pageName))
      {
        // This is called when TreeDP is finished.
        // This should only be called if Finish was clicked, not Cancel.
        ClosePopUpDP result = (ClosePopUpDP) base.GetPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
        ITreeModelRO model = (ITreeModelRO) GetModel(TREEMODEL);
        if (model != null)
        {
          String strOpenerFunction = model.GetParentFormSubmitFunction();
          int nSize = 20;
          if (model.GetSubmitMode() == TreeConstants.N_JAVASCRIPTSUBMIT)
          {
            nSize+=100;
          }
          if ((strOpenerFunction != null) && (!"".Equals(strOpenerFunction)))
          {
            nSize+=100;
          }
          XPStringBuilder sb = new XPStringBuilder(nSize);
          if (model.GetSubmitMode() == TreeConstants.N_JAVASCRIPTSUBMIT)
          {
            ITreeModelRO myModel = (ITreeModelRO) GetModel(TREEMODEL);
            XPArrayList mySelectedItems = myModel.GetSelectedItems();
            sb.Append("myArray = new Array(").Append(mySelectedItems.GetSize()).Append(");\n");
            sb.Append("\n");
            // Make sure we use the unencoded version of the name here
            // (although it is still truncated)
            for (int i = 0;i < mySelectedItems.GetSize();i++)
            {
              sb.Append("myArray[").Append(i).Append("] = new Object();\n");
              sb.Append("myArray[").Append(i).Append("].").Append(OBJECT_NAME).Append(" = \'").Append(GeneralHelpers.EscapeJavascript(((TreeLeafNode) mySelectedItems.GetElement(i)).GetName())).Append("\';\n");
              sb.Append("myArray[").Append(i).Append("].").Append(OBJECT_OBJECT_ID).Append(" = \'").Append(((TreeLeafNode) mySelectedItems.GetElement(i)).m_nObjectID).Append("\';\n");
              sb.Append("myArray[").Append(i).Append("].").Append(OBJECT_CLASS_ID).Append(" = \'").Append(((TreeLeafNode) mySelectedItems.GetElement(i)).m_nClassID).Append("\';\n");
              sb.Append(GetTreeSpecificArrayDataToAppend(i));
              sb.Append("\n");
            }
          }
          // Add method to call on opening form, if it exists
          if ((strOpenerFunction != null) && (!"".Equals(strOpenerFunction)))
          {
            sb.Append("if ( parent.opener.window[\'").Append(JSHelper.escapeJS(strOpenerFunction)).Append("\'] ) {\n");
            if (model.GetSubmitMode() == TreeConstants.N_JAVASCRIPTSUBMIT)
            {
              sb.Append("  parent.opener.window[\'").Append(JSHelper.escapeJS(strOpenerFunction)).Append("\'](myArray);\n");
              } else
            {
              sb.Append("  parent.opener.window[\'").Append(JSHelper.escapeJS(strOpenerFunction)).Append("\']();\n");
              }
            sb.Append("} else {\n");
            sb.Append("  alert(\'" + GetString(75, ResourceConstants.INFRASTRUCTURE) + "\');\n");
            sb.Append("}\n");
            sb.Append("\n");
          }
          // We are in a frame, so we need to call parent, not window
          sb.Append(JS_PARENT_CLOSE);
          try
          {
            HTMLScript myScript = new HTMLScript("text/javaScript");
            myScript.AddInnerHTMLString(sb.ToString());
            result.SetScript(myScript);
          }
          catch (HTMLException he)
          {
            log.Error(he, "Error generating JS for TreeDP ClosePopUpDP.");
            }
        }
        return result;
      }
      return base.GetPage(pageName);
    }
    
    /// <summary>
    /// Override this method to attach additional information specific to your implementation of the tree.
    /// Below is an example of how to do this.
    /// ITreeModelRO myModel\u0009    = (ITreeModelRO) GetModel(TREEMODEL);
    /// XPArrayList mySelectedItems = myModel.GetSelectedItems();
    /// XPStringBuilder sb = new XPStringBuilder();
    /// sb.Append("myArray[").Append(iSelectedItemIndex).Append("].")
    /// .Append(OBJECT_CLASS_ID).Append(" = '")
    /// .Append(((TreeLeafNode) mySelectedItems.GetElement(iSelectedItemIndex)).m_nClassID)
    /// .Append("';\n");
    /// return sb.ToString();
    /// </summary>
    /// <param name="index">of the selected Item</param>
    /// <returns>String</returns>
    public virtual String GetTreeSpecificArrayDataToAppend(int iSelectedItemIndex)
    {
      return "";
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostAS#GetRepostControlName()"></seealso>
    public override String GetRepostControlName()
    {
      return TreeRepostControl.STR_MVC_CLASS_NAME;
    }
    /////////////////
    // Helper methods
    /////////////////
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.AActivitySpace#Init()">
    /// Sub classes need to provide their own implementation that calls into this
    /// implementation and adds all of the MVC elements (other than DPs).  The model
    /// should be stored as this.TREEMODEL.
    /// </seealso>
    public override void Init()
    {
      base.Init();
      // Main Frame set
      // This must be registered first so it is the default page
      RegisterPage(new TreeFrameDP());
      // Header DP
      AFormDP myHeader = new TreeHeaderDP();
      //		myHeader.SetShowFormHeader(true);
      myHeader.AddToValidationFunction(TreeHeaderDP.JAVASCRIPT_SUBMIT_TREE);
      RegisterPage(myHeader);
      // Main Tree DP
      AFormDP myPage = (AFormDP) new TreeDP();
      myPage.AddToValidationFunction(TreeView.JAVASCRIPT_SUBMIT_TO_PARENT);
      RegisterPage(myPage);
      // Close pop up
      RegisterPage(new ClosePopUpDP());
      // Turn on current page tracking.  This is necessary because the form header
      // calls GetFormTitle() on the current page, which it is assuming is the page
      // that it is in.
      SetTrackCurrentPage(true);
    }
  }
}