using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This model handles generating a tree for admin objects.
  /// </summary>
  /// <author>Don Hayler</author>
  public abstract class ATreeModel : ITreeModel,ITreeModelRO,IRepostModel
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.tree.ATreeModel");
    private static int m_nINITIAL_UNIQUE_ID = 1;
    protected AActivitySpace m_asOwner;
    protected String m_strActionSubTitle = null;
    protected String m_strOnClickFunction = null;
    protected String m_strParentFormItems = null;
    protected String m_strParentFormName = null;
    protected String m_strParentFormSpaceID = null;
    protected String m_strParentFormSubmit = null;
    protected String m_strTitle = null;
    protected TreeCompositeNode m_Root;
    protected XPHashtable m_xpmSelectedItems;
    protected bool m_bFlatList = false;
    protected bool m_bAllChecked = false;
    protected bool m_bMultiSelect = false;
    protected bool m_bShowRoot = true;
    protected bool m_bEmptyList = false;
    protected bool m_bAllowEmptyTreeSubmission = false;
    protected bool m_bHelpEnabled = false;
    protected int m_nSelectMode = TreeConstants.N_LEAFSELECT;
    protected int m_nSelectOptions = TreeConstants.N_LOOSELEAF;
    protected int m_nSubmitMode = TreeConstants.N_SERVERSUBMIT;
    protected int m_nLastExpandedNodeID = -1;
    // This counter provides the next unique ID for internal tree IDs.
    private int m_nNextUniqueID = m_nINITIAL_UNIQUE_ID;
    // This counter provides the next unique ID for internal tree IDs.
    private const String UNIQUE_NAME_PREFIX = ",UniqueTreeName";
    // AGC- A simple method to reset the SelectedItems hash map.
    // It doesn't seem trivial to allow an external list of selected
    // items to be added. This may have to be done in a subclass.
    // For now, I'm going to assume that clearing the selected items
    // might be good enough for my purposes.
    public virtual void ClearSelectedItems()
    {
      m_xpmSelectedItems = new XPHashtable();
    }
    
    /// <summary>
    /// Expand or collapse a node in the tree.  This method is responsible
    /// for updating the last expanded node member (m_nLastExpandedNodeID).
    /// </summary>
    /// <param name="boolean">bExpand</param>
    /// <param name="int">nNodeID the unique node of the ID in the tree</param>
    public virtual void ExpandNode(bool bExpand, int nNodeID)
    {
      // Set the last expanded node id so we can go to its html anchor
      // when we display the page.
      m_nLastExpandedNodeID = nNodeID;
      // If we are expanding, there should be new nodes that may not be checked
      if (bExpand)
      {
        m_bAllChecked = false;
      }
    }
    
    /// <summary>
    /// Returns the most recently expanded or collapsed node in the tree.
    /// </summary>
    /// <param name="int">
    /// nNodeID the unique node of the ID in the tree that was
    /// most recently expanded or collapsed.  -1 implies no node was expanded
    /// or collapsed in the last repost.
    /// </param>
    public virtual int GetLastExpandedNodeID()
    {
      return m_nLastExpandedNodeID;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()"></seealso>
    public virtual void CleanupTempData()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()"></seealso>
    public virtual void CleanupAllData()
    {
    }
    
    /// <summary>
    /// This method is used by the ASManager to return new instances of managed
    /// objects.
    /// </summary>
    /// <returns>A new instance of the managed class (i.e. return new Foo();)</returns>
    public abstract Object Create();
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#Finish()"></seealso>
    public virtual void Finish()
    {
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetActionSubTitle()"></seealso>
    public virtual String GetActionSubTitle()
    {
      if (m_strActionSubTitle == null)
      {
        // This means a developer missed setting the title.  Should we
        // report an error here?
        return "";
      }
      return m_strActionSubTitle;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetHelpEnabled()"></seealso>
    public virtual bool GetHelpEnabled()
    {
      return m_bHelpEnabled;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetAllowEmptySubmission()"></seealso>
    public virtual bool GetAllowEmptySubmission()
    {
      return m_bAllowEmptyTreeSubmission;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetDisplayingFlatList()"></seealso>
    public virtual bool GetDisplayingFlatList()
    {
      return m_bFlatList;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetDisplayingFlatList()"></seealso>
    public virtual bool GetAllChecked()
    {
      return m_bAllChecked;
    }
    
    /// <returns></returns>
    public virtual bool GetMultipleSelect()
    {
      return m_bMultiSelect;
    }
    ///////////////////////////
    // RO Methods from IModelRO
    ///////////////////////////
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public abstract String GetName();
    
    /// <returns></returns>
    public virtual String GetOnClickFunction()
    {
      return m_strOnClickFunction;
    }
    
    /// <returns></returns>
    public virtual String GetParentFormName()
    {
      return m_strParentFormName;
    }
    
    /// <returns></returns>
    public virtual String GetParentFormSpaceID()
    {
      return m_strParentFormSpaceID;
    }
    
    /// <returns></returns>
    public virtual String GetParentFormSubmitFunction()
    {
      return m_strParentFormSubmit;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#GetRedirectAfterFinish()"></seealso>
    public virtual Redirect GetRedirectAfterFinish()
    {
      // Close the Tree pop-up
      Redirect result = new Redirect();
      result.SetLinkGetCachedSpace((AActivitySpace) m_asOwner);
      result.SetDisplayPage(ATreeAS.TREECLOSEPOPUP);
      return result;
    }
    ///////////////////////////////
    // RO Methods from ITreeModelRO
    ///////////////////////////////
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetTreeIsEmpty()"></seealso>
    public virtual bool GetTreeIsEmpty()
    {
      return m_bEmptyList;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetRootTreeIterator()"></seealso>
    public virtual TreeCompositeIterator GetRootTreeIterator()
    {
      // Make sure the tree accurately displays the selected items.
      RefreshTree();
      return new TreeCompositeIterator(m_Root, m_asOwner);
    }
    
    /// <returns></returns>
    public virtual int GetSelectMode()
    {
      return m_nSelectMode;
    }
    
    /// <returns></returns>
    public virtual int GetSelectOptions()
    {
      return m_nSelectOptions;
    }
    ///////////////////
    // For parent space
    ///////////////////
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetSelectedItems()"></seealso>
    public virtual XPArrayList GetSelectedItems()
    {
      // Add the elements to a sorted dictionary so they come out
      // in a reasonable order
      XPSortedDictionary tempDictionary = new XPSortedDictionary();
      TreeLeafNode tempNode;
      IXPEnumerator temp = m_xpmSelectedItems.GetElements();
      int i = 0;
      while (temp.MoveNext())
      {
        tempNode = (TreeLeafNode) temp.GetCurrent();
        // We need to make the names unique so we don't collapse multiple nodes with the same name
        // to the same element and lose objects.
        tempDictionary.PutElement(tempNode.GetName() + UNIQUE_NAME_PREFIX + i,
         new TreeLeafNode(tempNode));
        i++;
      }
      XPArrayList result = new XPArrayList();
      temp = tempDictionary.GetElements();
      while (temp.MoveNext())
      {
        result.Add(temp.GetCurrent());
      }
      return result;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetShowRoot()"></seealso>
    public virtual bool GetShowRoot()
    {
      return m_bShowRoot;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetSubmitMode()"></seealso>
    public virtual int GetSubmitMode()
    {
      return m_nSubmitMode;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModelRO#GetTitle()"></seealso>
    public virtual String GetTitle()
    {
      if (m_strTitle == null)
      {
        // This means a developer forgot to set the title.  Should
        // we report this somehow?
        return "";
      }
      return m_strTitle;
    }
    ////////////////////////////
    // Write Methods from IModel
    ////////////////////////////
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IModel#Init(AActivitySpace)"></seealso>
    public virtual void Init(AActivitySpace parent)
    {
      m_asOwner = parent;
    }
    /////////////////
    // Helper methods
    /////////////////
    
    /// <summary>
    /// This method goes through the selected items list and makes sure
    /// that all nodes (including multiply instanced nodes) are correctly selected.
    /// </summary>
    public virtual void RefreshTree()
    {
      IXPEnumerator temp = m_xpmSelectedItems.GetElements();
      while (temp.MoveNext())
      {
        TreeLeafNode curNode = (TreeLeafNode) temp.GetCurrent();
        if (curNode.m_bMultiplyInstanced)
        {
          // For groups-in-groups (users can be in multiple groups)
          m_Root.SetSelected(curNode.m_nClassID, curNode.m_nObjectID);
        } else
        {
          curNode.m_bSelected = true;
        }
      }
    }
    ////////////////////////////////
    // Write Methods from ITreeModel
    ////////////////////////////////
    public virtual void ResetTree()
    {
      m_Root = null;
      ResetUniqueIDs();
      m_bShowRoot = true;
      m_bFlatList = false;
      m_bAllChecked = false;
      m_bEmptyList = false;
      m_bAllowEmptyTreeSubmission = false;
      m_bHelpEnabled = false;
      m_xpmSelectedItems = new XPHashtable();
      m_nLastExpandedNodeID = -1;
    }
    public virtual void ResetUniqueIDs()
    {
      m_nNextUniqueID = m_nINITIAL_UNIQUE_ID;
    }
    //////////////////////////////////
    // Write Methods from IRepostModel
    //////////////////////////////////
    
    /// <seealso cref="com.plumtree.uiinfrastructure.form.IRepostModel#SavePage(String,XPHashtable)"></seealso>
    /// <returns>int Page status for server side validation</returns>
    public virtual int SavePage(String _sPageName, XPHashtable _htFormData)
    {
      IXPEnumerator keys = _htFormData.GetKeys();
      IXPEnumerator values = _htFormData.GetElements();
      while (keys.MoveNext())
      {
        values.MoveNext();
        log.Debug("Save Page: key = " + keys.GetCurrent() + " value = " + ((String[]) values.GetCurrent())[0]);
        }
      String strSelectedInputName = null;
      if (_htFormData.GetElement(TreeView.HTMLBUTTON_RADIO) != null)
      {
        strSelectedInputName = TreeView.HTMLBUTTON_RADIO;
      } else
      if (_htFormData.GetElement(TreeView.HTMLBUTTON_CHECKBOX) != null)
      {
        strSelectedInputName = TreeView.HTMLBUTTON_CHECKBOX;
      }
      XPHashtable xpmTempSelected = new XPHashtable();
      // Get pre-selected nodes (tree only, not search)
      String[] strTemp = (String[]) _htFormData.GetElement(TreeView.HTMLHIDDEN_SELECTEDIDS);
      if (strTemp != null)
      {
        // tree mode
        int[] nPreSelectedNodeIDs = GeneralHelpers.ConvertToInts(strTemp[0]);
        int nNode;
        String[] strSelectedItems = (String[]) _htFormData.GetElement(strSelectedInputName);
        if (strSelectedItems != null)
        {
          // Find and set the new selected elements
          for (int i = 0;i < strSelectedItems.Length;i++)
          {
            nNode = -1;
            try
            {
              try
              {
                nNode = XPConvert.ToInteger(strSelectedItems[i]);
              }
              catch (Exception e)
              {
                throw XPException.GetInstance(e);
              }
            }
            catch (XPFormatException fe)
            {
              log.Error("Save Page: ID of selected item in invalid format: " + strSelectedItems[i]);
              log.Error(fe, "Format error.");
              nNode = -1;
            }
            if (nNode >= m_nINITIAL_UNIQUE_ID)
            {
              TreeLeafNode node = m_Root.GetNode(nNode);
              if (node != null)
              {
                node.m_bSelected = true;
                xpmTempSelected.PutElement(GetKeyFromNode(node), node);
              } else
              {
                log.Warn("Save Page: bad selected element: node ID = " + nNode);
                }
            }
          }
        }
        // Find items that have been un-selected and remove them from the list.
        for (int i = 0;i < nPreSelectedNodeIDs.Length;i++)
        {
          if (nPreSelectedNodeIDs[i] >= m_nINITIAL_UNIQUE_ID)
          {
            TreeLeafNode node = m_Root.GetNode(nPreSelectedNodeIDs[i]);
            if (node != null)
            {
              if (!xpmTempSelected.ContainsKey(GetKeyFromNode(node)))
              {
                // Item was un-selected, remove it
                m_xpmSelectedItems.RemoveElement(GetKeyFromNode(node));
                // make sure we unselect it so it doesn't show up selected in the tree
                node.m_bSelected = false;
              }
            } else
            {
              log.Warn("Save Page: bad pre-selected element: node ID = " + nPreSelectedNodeIDs[i]);
              }
          }
        }
      }
      IXPEnumerator temp = xpmTempSelected.GetElements();
      // Clear the selected list if we have a selection in single select mode
      if ((!m_bMultiSelect) && (xpmTempSelected.GetSize() > 0))
      {
        m_xpmSelectedItems.Clear();
        // Add the single selected item
        if (temp.MoveNext())
        {
          TreeLeafNode curNode = (TreeLeafNode) temp.GetCurrent();
          m_xpmSelectedItems.PutElement(GetKeyFromNode(curNode), curNode);
          } else
        {
          log.Error("Tree Save Page: Couldn\'t access single selected item.");
          }
      } else
      {
        // Add all selected items to the list
        while (temp.MoveNext())
        {
          TreeLeafNode curNode = (TreeLeafNode) temp.GetCurrent();
          m_xpmSelectedItems.PutElement(GetKeyFromNode(curNode), curNode);
          }
      }
      // Check if the user checked the check all box
      m_bAllChecked = false;
      String[] strCheckAll = (String[]) _htFormData.GetElement(TreeView.HTMLINPUT_CHECK_ALL);
      if (strCheckAll != null)
      {
        // This will only be here if it is checked.
        m_bAllChecked = true;
      }
      return RepostControl.PAGE_STATUS_VALID;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModel#RemoveFromSelection(int">classID,int objectID)</seealso>
    public virtual void RemoveFromSelection(int classID, int objectID)
    {
      // mel: This method was added to assist in fixing 42066
      try
      {
        // Ignore remove if there are no elements in the tree
        if (m_Root == null)
        {
          return;
        }
        TreeLeafNode node = m_Root.GetNode(classID, objectID);
        // Ignore remove if no element exists identified by classID,objectID
        if (node == null)
        {
          return;
        }
        String nodeKey = GetKeyFromNode(node);
        if (m_xpmSelectedItems.ContainsKey(nodeKey))
        {
          m_xpmSelectedItems.RemoveElement(nodeKey);
          node.m_bSelected = false;
        }
      }
      catch (Exception anyException)
      {
        log.Error(anyException, "ATreeModel.RemoveFromSelection Failed to remove classID,objectID from the tree selection:" + classID + "," + objectID);
        }
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModel#SetActionSubTitle(String)"></seealso>
    public virtual void SetActionSubTitle(String strSubTitle)
    {
      m_strActionSubTitle = strSubTitle;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModel#SetAllowEmptySubmission(boolean)"></seealso>
    public virtual void SetAllowEmptySubmission(bool bAllow)
    {
      m_bAllowEmptyTreeSubmission = bAllow;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModel#SetHelpTopic(String)"></seealso>
    public virtual void SetHelpTopic(String strHelpTopic)
    {
      m_asOwner.SetHelpTopic(strHelpTopic);
      m_bHelpEnabled = true;
    }
    
    /// <param name="nIDs"></param>
    public abstract void SetClassIDsToDisplay(int[] nIDs);
    
    /// <param name="bMultiSelect"></param>
    public virtual void SetMultipleSelect(bool bMultiSelect)
    {
      m_bMultiSelect = bMultiSelect;
    }
    
    /// <param name="nIDs"></param>
    public abstract void SetObjectIDsToDisplay(int[] nIDs);
    
    /// <param name="nIDs"></param>
    public abstract void SetObjectIDsToHide(int[] nIDs);
    
    /// <param name="nIDs"></param>
    public abstract void SetObjectIDsToSelect(int[] nIDs);
    
    /// <returns></returns>
    public virtual int GetNextUniqueID()
    {
      return m_nNextUniqueID++;
    }
    
    /// <param name="name"></param>
    public virtual void SetOnClickFunction(String name)
    {
      m_strOnClickFunction = name;
    }
    
    /// <param name="name"></param>
    public virtual void SetParentFormName(String name)
    {
      m_strParentFormName = name;
    }
    
    /// <param name="name"></param>
    public virtual void SetParentFormSpaceID(String name)
    {
      m_strParentFormSpaceID = name;
    }
    
    /// <param name="name"></param>
    public virtual void SetParentFormSubmitFunction(String name)
    {
      m_strParentFormSubmit = name;
    }
    
    /// <param name="nSelectMode"></param>
    public virtual void SetSelectMode(int nSelectMode)
    {
      m_nSelectMode = nSelectMode;
    }
    
    /// <param name="nSelectOptions"></param>
    public virtual void SetSelectOptions(int nSelectOptions)
    {
      m_nSelectOptions = nSelectOptions;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModel#SetShowRoot(boolean)"></seealso>
    public virtual void SetShowRoot(bool bShowRoot)
    {
      m_bShowRoot = bShowRoot;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModel#SetSubmitMode(int)"></seealso>
    public virtual void SetSubmitMode(int nMode)
    {
      m_nSubmitMode = nMode;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.tree.ITreeModel#SetTitle(String)"></seealso>
    public virtual void SetTitle(String strTitle)
    {
      m_strTitle = strTitle;
    }
    
    /// <param name="strKey"></param>
    /// <returns></returns>
    protected virtual TreeCompositeNode GetCompositeIfAlreadySelected(String strKey)
    {
      TreeCompositeNode myFolder = (TreeCompositeNode) m_xpmSelectedItems.GetElement(strKey);
      if (myFolder == null)
      {
        return null;
      }
      if (myFolder.m_nTreeNodeID != -1)
      {
        log.Warn("attempting to create a new tree folder node that is already selected.  Key " + strKey);
        } else
      {
        TreeCompositeNode temp = new TreeCompositeNode(GetNextUniqueID());
        temp.SetNameAndDescription(myFolder);
        temp.m_nObjectID = myFolder.m_nObjectID;
        temp.m_nClassID = myFolder.m_nClassID;
        temp.m_strLocation = myFolder.m_strLocation;
        temp.m_bSelected = myFolder.m_bSelected;
        temp.m_Image = myFolder.m_Image;
        temp.m_OpenImage = myFolder.m_OpenImage;
        return temp;
      }
      return null;
    }
    
    /// <param name="strKey"></param>
    /// <returns></returns>
    protected virtual TreeLeafNode GetLeafIfAlreadySelected(String strKey)
    {
      TreeLeafNode myLeaf = (TreeLeafNode) m_xpmSelectedItems.GetElement(strKey);
      if (myLeaf == null)
      {
        return null;
      }
      if (myLeaf.m_nTreeNodeID != -1)
      {
        log.Warn("attempting to create a new tree node that is already selected.  Key " + strKey);
        } else
      {
        // Should this copy everything?  expanded, etc...?
        TreeLeafNode temp = new TreeLeafNode(GetNextUniqueID());
        temp.SetNameAndDescription(myLeaf);
        temp.m_nObjectID = myLeaf.m_nObjectID;
        temp.m_nClassID = myLeaf.m_nClassID;
        temp.m_strLocation = myLeaf.m_strLocation;
        temp.m_bSelected = myLeaf.m_bSelected;
        temp.m_Image = myLeaf.m_Image;
        return temp;
      }
      return null;
    }
    
    /// <summary>
    /// This function creates a unique key given a node.
    /// It is used to store items in XPHashtables.
    /// Override this function if your tree's nodes
    /// can't be differentiated by "classid, objectid"
    /// </summary>
    /// <param name="node"></param>
    /// <returns>String</returns>
    protected virtual String GetKeyFromNode(TreeLeafNode node)
    {
      return node.m_nClassID + "," + node.m_nObjectID;
    }
  }
}