using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This interface provides read only access to the tree.
  /// </summary>
  /// <author>Don Hayler</author>
  public interface ITreeModelRO : IModelRO
  {
    
    /// <summary>
    /// Get the action sub-title for the tree popup
    /// I.E. select a user
    /// </summary>
    /// <returns>String the non-HTML Encoded action sub-title</returns>
    String GetActionSubTitle();
    
    /// <summary>
    /// Gets whether or not help is enabled
    /// </summary>
    /// <returns>boolean true implies enabled.</returns>
    bool GetHelpEnabled();
    
    /// <summary>
    /// Whether or not users are allowed to click finish
    /// on trees without selecting anything.
    /// </summary>
    /// <returns>boolean true implies allow</returns>
    bool GetAllowEmptySubmission();
    
    /// <summary>
    /// This method returns whether or not the tree is displaying as
    /// a flat list, or as a tree.  Flat lists are displayed for
    /// smaller numbers of items.
    /// </summary>
    /// <result>boolean true implies flat list</result>
    bool GetDisplayingFlatList();
    
    /// <summary>
    /// This method returns whether or not the check all box in the
    /// tree should be pre-selected or not.
    /// </summary>
    /// <result>boolean true implies check all box should be pre-selected.</result>
    bool GetAllChecked();
    
    /// <returns></returns>
    bool GetMultipleSelect();
    
    /// <returns></returns>
    String GetOnClickFunction();
    
    /// <returns></returns>
    String GetParentFormName();
    
    /// <returns></returns>
    String GetParentFormSpaceID();
    
    /// <returns></returns>
    String GetParentFormSubmitFunction();
    ///////////////////////
    // Methods for the view
    ///////////////////////
    
    /// <returns></returns>
    bool GetTreeIsEmpty();
    
    /// <returns></returns>
    TreeCompositeIterator GetRootTreeIterator();
    
    /// <returns></returns>
    int GetSelectMode();
    
    /// <returns></returns>
    int GetSelectOptions();
    
    /// <returns></returns>
    int GetLastExpandedNodeID();
    ///////////////////////////////
    // Methods for the parent space
    ///////////////////////////////
    
    /// <summary>
    /// This returns an array of the currently selected objects/folders in the tree.
    /// </summary>
    /// <returns>XPArrayList of TreeLeafNode objects</returns>
    XPArrayList GetSelectedItems();
    
    /// <summary>
    /// This method returns whether or not to show the root folder.
    /// </summary>
    /// <result>boolean true implies show root (default)</result>
    bool GetShowRoot();
    
    /// <returns></returns>
    int GetSubmitMode();
    
    /// <summary>
    /// Get the title for the tree popup
    /// </summary>
    /// <returns>String the non-HTML Encoded title</returns>
    String GetTitle();
  }
}