using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.statichelpers;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This control is used to re-open a previously opened tree and refresh
  /// the contents (which objects are selected, etc...).
  /// </summary>
  /// <author>Don Hayler</author>
  public class RefreshTreeControl : IControl
  {
    
    /// <summary>
    /// name of the refresh tree control 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "RefreshTree";
    protected ATreeAS m_asOwner;
    
    /// <summary>
    /// Internal data *
    /// </summary>
    protected ITreeModel m_asModel;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IControl#CheckActionSecurityAndExecute(XPHashtable)"></seealso>
    public virtual Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      String[] sValues;
      // Object ids to select (class id, object id pairs)
      sValues = (String[]) arguments.GetElement(AStartTreeControl.SELECTED_ITEMS);
      if (sValues != null)
      {
        // Check for referenced long strings
        String strSelectedObjects = TreeDataHelpers.UnReferenceQueryStringData(sValues,
         m_asOwner);
        if (strSelectedObjects == null)
        {
          // Use the normal data if there were no long strings referenced
          strSelectedObjects = sValues[0];
        }
        m_asModel.SetObjectIDsToSelect(GeneralHelpers.ConvertToInts(strSelectedObjects));
        }
      // Object ids to hide (class id, object id pairs)
      sValues = (String[]) arguments.GetElement(AStartTreeControl.HIDDEN_ITEMS);
      if (sValues != null)
      {
        // Check for referenced long strings
        String strHiddenObjects = TreeDataHelpers.UnReferenceQueryStringData(sValues,
         m_asOwner);
        if (strHiddenObjects == null)
        {
          // Use the normal data if there were no long strings referenced
          strHiddenObjects = sValues[0];
        }
        m_asModel.SetObjectIDsToHide(GeneralHelpers.ConvertToInts(strHiddenObjects));
        }
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new RefreshTreeControl();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IControl#Init(IModel,AActivitySpace)"></seealso>
    public virtual void Init(IModel model, AActivitySpace space)
    {
      m_asModel = (ITreeModel) model;
      m_asOwner = (ATreeAS) space;
    }
  }
}