using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.statichelpers;
using com.plumtree.openlog;
using com.plumtree.xpshared.htmlconstructs;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// Display the action sub-title for the tree frame.
  /// </summary>
  /// <author>Don Hayler</author>
  public class TreeBarView : IView
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.uiinfrastructure.tree.TreeBarView");
    
    /// <summary>
    /// name of the tree bar view 
    /// </summary>
    public const String STR_MVC_CLASS_NAME = "TreeBar";
    private AActivitySpace m_asOwner;
    
    /// <summary>
    /// internal data 
    /// </summary>
    private ITreeModelRO m_asModel;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public virtual Object Create()
    {
      return new TreeBarView();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Display()"></seealso>
    public virtual HTMLElement Display()
    {
      HTMLTable myTable = new HTMLTable();
      myTable.SetWidth("100%");
      myTable.SetBorder("0");
      myTable.SetCellSpacing("0");
      myTable.SetCellPadding("3");
      myTable.SetSummary("Action title bar table");
      HTMLTableRow myTableRow;
      HTMLTableCell myTableCell;
      try
      {
        // Title
        myTableRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myTableRow);
        myTableRow.SetStyleClass(PTStyleClass.DIALOGUE_HEADER_BG);
        myTableCell = new HTMLTableCell();
        myTableRow.AddInnerHTMLElement(myTableCell);
        myTableCell.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
        myTableCell.SetHeight("20");
        myTableCell.SetWidth("80%");
        HTMLSpan mySpan = new HTMLSpan();
        myTableCell.AddInnerHTMLElement(mySpan);
        mySpan.SetStyleClass(PTStyleClass.DIALOGUE_SUBTITLE);
        mySpan.AddInnerHTMLString("&nbsp;");
        mySpan.AddInnerHTMLEncodedString(m_asModel.GetActionSubTitle());
        if (m_asModel.GetHelpEnabled())
        {
          myTableCell = new HTMLTableCell();
          myTableRow.AddInnerHTMLElement(myTableCell);
          myTableCell.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
          myTableCell.SetHeight("20");
          myTableCell.SetWidth("20%");
          myTableCell.SetAlign("right");
          mySpan = new HTMLSpan();
          myTableCell.AddInnerHTMLElement(mySpan);
          mySpan.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
          mySpan.AddInnerHTMLElement(HTMLConstructHelpers.GetHelpAnchor(m_asOwner));
          }
      }
      catch (Exception e)
      {
        log.Error(e, "Display: The following error occured when trying to display the HTML.");
        }
      return myTable;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()"></seealso>
    public virtual HTMLScript DisplayJavascript()
    {
      return null;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public virtual String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO,AActivitySpace)"></seealso>
    public virtual void Init(IModelRO model, AActivitySpace parent)
    {
      m_asModel = (ITreeModelRO) model;
      m_asOwner = parent;
    }
  }
}