using System;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// This class contains tree constants.
  /// </summary>
  public sealed class TreeConstants
  {
    
    /// <summary>
    /// Select Modes 
    /// </summary>
    public const int N_COMPOSITESELECT = 1;
    public const int N_LEAFSELECT = 2;
    public const int N_LEAFCOMPOSITESELECT = 3;
    public const int N_NOSELECT = 4;
    
    /// <summary>
    /// Select Options:
    /// Sometimes leaf nodes can act as both leaves and composites.
    /// This mode limits selection of leaves to exclude those dual
    /// nodes (such as user groups).  Default behavior is to
    /// include these nodes in N_LEAFSELECT select mode (N_LOOSELEAF).
    /// </summary>
    public const int N_LOOSELEAF = 1;
    public const int N_STRICTLEAF = 2;
    
    /// <summary>
    /// Submit Modes:
    /// SERVERSUBMIT is for internal portal developers
    /// JAVASCRIPTSUBMIT is for projects external to the server,
    /// such as collab and content.
    /// </summary>
    public const int N_SERVERSUBMIT = 1;
    public const int N_JAVASCRIPTSUBMIT = 2;
    
    /// <summary>
    /// Common string for javascript window.open window attributes.
    /// This should be used for all trees in the portal so we can
    /// maintain a common look and feel.
    /// </summary>
    private const String STR_TREE_WINDOW_ATTRIBUTES = "location=no,menubar=no,resizable=yes,height=400,width=400";
    
    /// <summary>
    /// Common string for javascript window.open window attributes.
    /// This should be used for all trees in the portal so we can
    /// maintain a common look and feel.
    /// </summary>
    public static String GetJavascriptTreeWindowAttributes()
    {
      return STR_TREE_WINDOW_ATTRIBUTES;
    }
  }
}