using System;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.form;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.tree
{
  /// <summary>
  /// The tree repost control handles the common actions in the tree,
  /// such as expanding nodes.
  /// </summary>
  /// <author>Don Hayler</author>
  public class TreeRepostControl : RepostControl
  {
    public const String STR_MVC_CLASS_NAME = "TreeRepost";
    public const String m_strEXPAND = "in_hi_Expand";
    public const String m_strFOLDERID = "in_hi_FolderID";
    public const int POSTTOSELF_EXPAND = 10;
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IControl#Execute()"></seealso>
    public override Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
    {
      Redirect result = base.CheckActionSecurityAndExecute(arguments);
      if (m_nAction == POSTTOSELF_ACTION_CANCEL)
      {
        // If canceling, we want to use the normal close pop up, not the
        // special treeDP close popup with extra javascript
        result = new Redirect();
        result.SetLinkGetCachedSpace(m_asOwner);
        result.SetDisplayPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
      }
      if (result == null)
      {
        result = new Redirect();
        result.SetLinkGetCachedSpace(m_asOwner);
        result.SetDisplayPage(TreeDP.STR_MVC_CLASS_NAME);
      }
      return result;
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()"></seealso>
    public override Object Create()
    {
      return new TreeRepostControl();
    }
    
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()"></seealso>
    public override String GetName()
    {
      return STR_MVC_CLASS_NAME;
    }
    
    /// <summary>
    /// Switch between the action modes and call  the associated method for each
    /// action.
    /// </summary>
    /// <param name="_nAction"></param>
    protected override void PerformAction(int _nAction)
    {
      base.PerformAction(_nAction);
      //***CHANGE_HERE*** if you don't have any repost action, delete the switch statement.
      switch (_nAction){
      //***CHANGE_HERE*** Create a new case for each of your action.
      case POSTTOSELF_EXPAND:
      // Expand or contract a node in the tree
      bool bExpand = XPConvert.ToBoolean(((String[]) m_htFormData.GetElement(m_strEXPAND))[0]);
      int nID = XPConvert.ToInteger(((String[]) m_htFormData.GetElement(m_strFOLDERID))[0]);
      ((ITreeModel) m_model).ExpandNode(bExpand, nID);
      break;
      }
    }
  }
}