using System;
using com.plumtree.openfoundation.web;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.uiinfrastructure.web;
namespace com.plumtree.uiinfrastructure.uitasks
{
  /// <summary>
  /// This class provides access to the environment that a
  /// tag is executed in.
  /// </summary>
  public interface IEnvironment
  {
    
    /// <summary>
    /// This method returns the current access style (i.e. Section
    /// 508 compliance, low bandwidth, or standard).
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.constants.AccessStyles"></seealso>
    /// <returns>AccessStyles The access style</returns>
    AccessStyles GetAccessStyle();
    
    /// <summary>
    /// Return the HTTP Request for the current request.
    /// </summary>
    /// <returns>the current request</returns>
    IXPRequest GetCurrentHTTPRequest();
    
    /// <summary>
    /// Return the HTTP Response for the current request.
    /// </summary>
    /// <returns>the current response</returns>
    IXPResponse GetCurrentHTTPResponse();
    
    /// <summary>
    /// Return the normalized base url of the current request.
    /// The normalized base url is essentially the request url
    /// up to and including the '?'.
    /// <p />
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.statichelpers.ConfigHelpers#GetNormalizedBaseURL(String)"></seealso>
    /// <returns>The normalized base url of the current request</returns>
    String GetCurrentNormalizedBaseURL();
    
    /// <summary>
    /// Return the full url of the current request.
    /// </summary>
    /// <returns>the url of the current request</returns>
    String GetCurrentRequestURL();
    
    /// <summary>
    /// Gets the current language of the user.
    /// <p />
    /// E.G. en, fr, jp
    /// </summary>
    /// <returns>The current language of the user</returns>
    String GetLanguage();
    
    /// <summary>
    /// Gets the current locale of the user.
    /// <p />
    /// E.G. en-us, en-gb, fr-fr, jp-jp
    /// </summary>
    /// <returns>The current locale of the user.</returns>
    String GetLocale();
    
    /// <summary>
    /// Gets the time zone of the current user.
    /// <p />
    /// E.G. America/Los_Angeles
    /// </summary>
    /// <returns>The time zone of the user.</returns>
    String GetTimeZone();
    
    /// <summary>
    /// This method gets localized strings for an
    /// internationalized string file based on the user's
    /// locale.
    /// <p />
    /// Available string files:
    /// <br />
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.constants.ResourceConstants">
    /// <p />
    /// Custom string files can also be added to the portal.
    /// </seealso>
    /// <param name="_nID">The ID of the string to retrieve</param>
    /// <param name="_strStringFile">The name of the string file to use</param>
    /// <returns>String The localized string</returns>
    String GetString(int _nID, String _strStringFile);
    
    /// <summary>
    /// Return the user's Session.  This is a generic Object that
    /// can be used for different kinds of sessions.  This will
    /// have to be cast to the correct type of session (IPTSession
    /// for the portal).
    /// </summary>
    /// <returns>Object the user's session</returns>
    Object GetUserSession();
    
    /// <summary>
    /// This method gets Variable Packages.
    /// </summary>
    /// <deprecated>Variable packages are no longer available directly in tags.  In the portal, the Environment object can be cast to an Activity Space and VarPacks can be retrieved that way.</deprecated>
    /// <param name="String">_strVarPack The ID of the requested VarPack</param>
    /// <returns>IVarPack The requested VarPack</returns>
    IVarPack GetVarPack(String _strVarPack);
  }
}