/*
 * Created on Jan 12, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
namespace com.plumtree.uiinfrastructure.utils
{
  /// <summary>
  /// Generates a Javascript 1D array in a String.
  /// Only the array itself is generated, no variable declaration or assignment or trailing semicolon.
  /// <p />
  /// Ex:
  /// <c>
  /// JSArrayList jsarray = new JSArrayList(3);
  /// for ( int i = 0; i < 3; i++ )
  /// {
  /// jsarray.AddNewElement();
  /// jsarray.SetValue(0, "val"+0)
  /// jsarray.SetValue(1, "val"+1)
  /// jsarray.SetValue(2, "val"+2)
  /// }
  /// jsarray.ToString();
  /// </c>
  /// <p />
  /// Generates the string:
  /// <p />
  /// "[[val00, val01, val02],[val10, val11,val12],[val20,val21,val23]]"
  /// </summary>
  /// <author>robertz</author>
  public class JS1DArrayBuilder
  {
    private XPArrayList m_list;
    private int m_nCurrIndex = 0;
    
    /// <param name="nColumnCount"></param>
    public JS1DArrayBuilder()
    {
      m_list = new XPArrayList();
    }
    
    /// <returns></returns>
    public virtual int AddNewStringValueElement(String strValue)
    {
      return m_list.Add("\"" + strValue + "\"");
    }
    
    /// <param name="strValue"></param>
    /// <returns></returns>
    public virtual int AddNewReferenceElement(String strValue)
    {
      return m_list.Add(strValue);
    }
    
    /// <returns></returns>
    public virtual String ToString()
    {
      XPStringBuilder buf = new XPStringBuilder();
      buf.Append("[");
      for (int nRowIndex = 0;nRowIndex < m_list.GetSize();nRowIndex++)
      {
        if (nRowIndex != 0)
        {
          buf.Append(",");
        }
        buf.Append(m_list.GetElement(nRowIndex));
      }
      buf.Append("]");
      return buf.ToString();
    }
  }
}