/*
 * Created on Jan 12, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.statichelpers;
namespace com.plumtree.uiinfrastructure.utils
{
  /// <summary>
  /// Generates a Javascript 2D array in a String.
  /// Only the array itself is generated, no variable declaration or assignment or trailing semicolon.
  /// <p />
  /// Ex:
  /// <c>
  /// JSArrayList jsarray = new JSArrayList(3);
  /// for ( int i = 0; i < 3; i++ )
  /// {
  /// jsarray.AddNewElement();
  /// jsarray.SetValue(0, "val"+i+"0")
  /// jsarray.SetValue(1, "val"+i+"1")
  /// jsarray.SetValue(2, "val"+i+"2")
  /// }
  /// jsarray.ToString();
  /// </c>z
  /// <p />
  /// Generates the string:
  /// <p />
  /// "[[val00, val01, val02],[val10, val11,val12],[val20,val21,val23]]"
  /// </summary>
  /// <author>robertz</author>
  public class JS2DArrayBuilder
  {
    private XPArrayList m_list;
    private int m_nCurrIndex = 0;
    private int m_nSetColumnCount = 0;
    
    /// <param name="nColumnCount">Number of columns in the array.</param>
    public JS2DArrayBuilder(int nColumnCount)
    {
      m_list = new XPArrayList();
      m_nSetColumnCount = nColumnCount;
    }
    
    /// <summary>
    /// Adds a new row at the end.
    /// </summary>
    /// <returns></returns>
    public virtual int AddNewElement()
    {
      return m_list.Add(new String[m_nSetColumnCount]);
    }
    
    /// <param name="nColumn">Column index of the current row</param>
    /// <param name="strValue">Value to assign to the specified column. The value will be surrounded with quotes to define a string</param>
    public virtual void SetColumnStringValue(int nColumn, String strValue)
    {
      String[] arColumns = (String[]) m_list.GetElement(m_list.GetSize() - 1);
      if (nColumn < 0 || nColumn > m_nSetColumnCount)
      {
        throw new XPIllegalArgumentException("Invalid column index specified: " + nColumn + ". Must be a positive value less than the maximum columns.");
        }
      arColumns[nColumn] = (strValue == null) ? "" : "\'" + JSHelper.escapeJS(strValue) + "\'";
      }
    
    /// <TODO>MARTOK add another method for simple value or change the name</TODO>
    /// <param name="nColumn">Column index of the current row</param>
    /// <param name="strValue"></param>
    public virtual void SetColumnReference(int nColumn, String strValue)
    {
      String[] arColumns = (String[]) m_list.GetElement(m_nCurrIndex);
      if (nColumn < 0 || nColumn > m_nSetColumnCount)
      {
        throw new XPIllegalArgumentException("Invalid column index specified: " + nColumn + ". Must be a positive value less than the maximum columns.");
        }
      arColumns[nColumn] = (strValue == null) ? "" : strValue;
    }
    
    /// <returns>The size of the current array.</returns>
    public virtual int GetSize()
    {
      return m_list.GetSize();
    }
    
    /// <returns></returns>
    public virtual String ToString()
    {
      XPStringBuilder buf = new XPStringBuilder();
      buf.Append("[");
      for (int nRowIndex = 0;nRowIndex < m_list.GetSize();nRowIndex++)
      {
        if (nRowIndex != 0)
        {
          buf.Append(",");
        }
        buf.Append(" [");
        String[] arValues = (String[]) m_list.GetElement(nRowIndex);
        for (int nColIndex = 0;nColIndex < m_nSetColumnCount;nColIndex++)
        {
          if (nColIndex != 0)
          {
            buf.Append(",");
          }
          buf.Append(arValues[nColIndex]);
        }
        buf.Append("]");
      }
      buf.Append("]");
      return buf.ToString();
    }
  }
}