/*
 * Created on Jan 14, 2005
 */
 
using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.statichelpers;
namespace com.plumtree.uiinfrastructure.utils
{
  /// <summary>
  /// Class description
  /// </summary>
  /// <author>robertz</author>
  public class JSFunctionCallBuilder
  {
    private String m_strFunctionName = "";
    private XPStringBuilder buf = new XPStringBuilder();
    private bool m_bSemiColon = false;
    public JSFunctionCallBuilder(String strFunctionName)
    {
      m_strFunctionName = strFunctionName;
    }
    
    /// <summary>
    /// Argument value will be put inside quotes.
    /// E.g. functionName("string1", "string2");
    /// </summary>
    /// <param name="strStringValue"></param>
    public virtual void AddStringValueArgument(String strStringValue)
    {
      if (buf.GetLength() != 0)
      {
        buf.Append(",");
      }
      buf.Append("\"" + JSHelper.escapeJS(strStringValue) + "\"");
    }
    
    /// <summary>
    /// Argument value is enclosed in escaped quotes. Used when
    /// a function call is used within a javascript string.
    /// E.g.
    /// <script>
    /// var foo = "<a href=\"# \ onclick=\"somefunction(\"stringval\", variableval)\" ">text</a> ""
    /// </summary>
    public virtual void AddStringValueArgumentUsingSingleQuotes(String strStringValue)
    {
      if (buf.GetLength() != 0)
      {
        buf.Append(",");
      }
      buf.Append("\'" + JSHelper.escapeJS(strStringValue) + "\'");
    }
    
    /// <summary>
    /// Javascript variable reference
    /// E.g. functionName(var1, var2);
    /// </summary>
    public virtual void AddVariableArgument(String strVariable)
    {
      if (buf.GetLength() != 0)
      {
        buf.Append(",");
      }
      buf.Append(strVariable);
    }
    public virtual void UseSemiColon()
    {
      m_bSemiColon = true;
    }
    public virtual String ToString()
    {
      return m_strFunctionName + "(" + buf.ToString() + ")" + ((m_bSemiColon) ? ";" : "");
      }
  }
}