using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.errordisplay;
using AValidator=com.plumtree.uiinfrastructure.pagecontrols.framework.AValidator;
namespace com.plumtree.uiinfrastructure.validators
{
  /// <summary>
  /// Validates that an object is within a given int range bounded by m_intMax & m_intMin.
  /// </summary>
  /// <author>caiusv</author>
  public class IntRangeValidator : AValidator,IValidator
  {
    public String VALIDATOR_NAME = "IntRangeValidator";
    
    /// <summary>
    /// name of the form where this validator appears (used by JavaScript validation)
    /// </summary>
    public String m_strFormName;
    
    /// <summary>
    /// name of the input which this validator works on (used by JavaScript validation) 
    /// </summary>
    public String m_strInputName;
    
    /// <summary>
    /// the defualt value of the input 
    /// </summary>
    private String m_strDefaultValue;
    private int m_intMax;
    private int m_intMin;
    
    /// <summary>
    /// Creates a new IntRangeValidator.
    /// </summary>
    /// <param name="_sFormName">name of the form whre this appears</param>
    /// <param name="_sInputName">name of the input element</param>
    /// <param name="_sDefaultValue">default value of the input element</param>
    /// <param name="_iMax">max value for the int validated</param>
    /// <param name="_iMin">min value for the int validated</param>
    public IntRangeValidator(String _sFormName, String _sInputName, String _sDefaultValue,
     int _iMax, int _iMin)
    {
      m_strFormName = _sFormName;
      m_strInputName = _sInputName;
      m_strDefaultValue = _sDefaultValue;
      m_intMax = _iMax;
      m_intMin = _iMin;
    }
    
    /// <summary>
    /// Return a UIError object if the object passed in is not an int within the range 
    /// specified by the m_intMax & m_intMin.
    /// </summary>
    /// <returns>a UIError containing the error message set with SetErrorMessage()</returns>
    public virtual UIError Validate(Object _oObject)
    {
      String strErrorMsg = GetErrorMessage();
      int myInt = 0;
      UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
       strErrorMsg, strErrorMsg);
      if (null == _oObject)
      {
        return myError;
      }
      try
      {
        myInt = XPConvert.ToInteger(_oObject);
      }
      catch (Exception e)
      {
        return myError;
      }
      if (myInt > m_intMax || myInt < m_intMin)
      {
        return myError;
      }
      return null;
    }
    
    /// <summary>
    /// Returns the JavaScript Validation method that may be used on a page to validate
    /// associated form input.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IValidator#GetJSValidationCode()"></seealso>
    public virtual String GetJSValidationCode()
    {
      XPStringBuilder sb = new XPStringBuilder(1250);
      sb.Append("<script type=\"text/javascript\">\n");
      sb.Append("function isIntInRange(inputName)\n");
      sb.Append("{\n");
      sb.Append("    if (PTNumberUtil.isInteger(parseInt(inputName.value)) " + "&& inputName.value < " + m_intMax + " " + "&& inputName.value > " + m_intMin + ")\n");
      sb.Append("    {\n");
      sb.Append("        return true;\n");
      sb.Append("    }\n");
      sb.Append("    else");
      sb.Append("    {\n");
      sb.Append("        alert(\"" + GetErrorMessage() + "\");\n");
      sb.Append("        inputName.value = \"" + m_strDefaultValue + "\";\n");
      sb.Append("        return false;\n");
      sb.Append("    }\n");
      sb.Append("}\n");
      sb.Append("</script>\n");
      return sb.ToString();
    }
    
    /// <summary>
    /// Return the name of the JavaScript validation method along with whatever
    /// parameters the script needs.
    /// </summary>
    public virtual String GetJSFunction(String _sParameters)
    {
      if (null != _sParameters)
      {
        return "isIntInRange(" + _sParameters + ")";
      } else
      {
        return "isIntInRange(" + m_strFormName + "." + m_strInputName + ")";
        }
    }
    
    /// <summary>
    /// Return the name of this Validator
    /// </summary>
    public virtual String GetName()
    {
      return VALIDATOR_NAME;
    }
  }
}