using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.errordisplay;
using AValidator=com.plumtree.uiinfrastructure.pagecontrols.framework.AValidator;
namespace com.plumtree.uiinfrastructure.validators
{
  /// <summary>
  /// Validates that a input value is less than a given length
  /// </summary>
  /// <author>caiusv</author>
  public class LengthValidator : AValidator,IValidator
  {
    public String VALIDATOR_NAME = "LengthValidator";
    
    /// <summary>
    /// name of the form where this validator appears (used by JavaScript validation)
    /// </summary>
    public String m_strFormName;
    
    /// <summary>
    /// name of the input which this validator works on (used by JavaScript validation) 
    /// </summary>
    public String m_strInputName;
    private int m_intMax;
    private int m_intMin;
    
    /// <summary>
    /// Creates a new PositiveIntValidator.
    /// </summary>
    /// <param name="_sFormName">name of the form whre this appears</param>
    /// <param name="_sInputName">name of the input element</param>
    /// <param name="_sDefaultValue">default value of the input element</param>
    public LengthValidator(String _sFormName, String _sInputName, int _iMax,
     int _iMin)
    {
      m_strFormName = _sFormName;
      m_strInputName = _sInputName;
      m_intMax = _iMax;
      m_intMin = _iMin;
    }
    public virtual UIError Validate(Object _oObject)
    {
      UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
       GetErrorMessage(), GetErrorMessage());
      if (null == _oObject)
      {
        return myError;
      }
      try
      {
        String myString = XPConvert.ToString(_oObject);
        if (myString.Length > m_intMax || myString.Length < m_intMin)
        {
          return myError;
        }
      }
      catch (Exception e)
      {
        return myError;
      }
      return null;
    }
    
    /// <summary>
    /// Returns the JavaScript Validation method that may be used on a page to validate
    /// associated form input.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IValidator#GetJSValidationCode()"></seealso>
    public virtual String GetJSValidationCode()
    {
      XPStringBuilder sb = new XPStringBuilder(1250);
      sb.Append("<script type=\"text/javascript\">\n");
      sb.Append("function validateLength(inputName)\n");
      sb.Append("{\n");
      sb.Append("    if (inputName.value.length > " + m_intMax + " || inputName.value.length < " + m_intMin + ")\n");
      sb.Append("    {\n");
      sb.Append("        alert(\"" + GetErrorMessage() + "\");\n");
      sb.Append("        return false;\n");
      sb.Append("    }\n");
      sb.Append("    else");
      sb.Append("    {\n");
      sb.Append("        return true;\n");
      sb.Append("    }\n");
      sb.Append("}\n");
      sb.Append("</script>\n");
      return sb.ToString();
    }
    
    /// <summary>
    /// Return the name of the JavaScript validation method along with whatever
    /// parameters the script needs.
    /// </summary>
    public virtual String GetJSFunction(String _sParameters)
    {
      if (null != _sParameters)
      {
        return "validateLength(" + _sParameters + ")";
      } else
      {
        return "validateLength(" + m_strFormName + "." + m_strInputName + ")";
        }
    }
    
    /// <summary>
    /// Return the name of this Validator
    /// </summary>
    public virtual String GetName()
    {
      return VALIDATOR_NAME;
    }
  }
}