using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.errordisplay;
using AValidator=com.plumtree.uiinfrastructure.pagecontrols.framework.AValidator;
namespace com.plumtree.uiinfrastructure.validators
{
  /// <summary>
  /// A PageControl Validator to check that the value of the page control is not
  /// null or an empty String
  /// </summary>
  /// <author>caiusv</author>
  public class NotEmptyValidator : AValidator,IValidator
  {
    private String VALIDATOR_NAME = "NotEmptyValidator";
    
    /// <summary>
    /// name of the form where this validator appears (used by JavaScript validation)
    /// </summary>
    public String m_strFormName;
    
    /// <summary>
    /// name of the input which this validator works on (used by JavaScript validation) 
    /// </summary>
    public String m_strInputName;
    
    /// <summary>
    /// Empty constructor creates a new NotEmptyValidator
    /// </summary>
    public NotEmptyValidator(String _sFormName, String _sInputName)
    {
      m_strFormName = _sFormName;
      m_strInputName = _sInputName;
    }
    public virtual UIError Validate(Object _oObject)
    {
      UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
       GetErrorMessage(), GetErrorMessage());
      if (null == _oObject)
      {
        return myError;
      }
      try
      {
        if (XPStringUtility.Compare(XPConvert.ToString(_oObject), "") == 0)
        {
          return myError;
        }
      }
      catch (Exception e)
      {
        return myError;
      }
      return null;
    }
    
    /// <summary>
    /// Returns the JavaScript Validation method that may be used on a page to validate
    /// associated form input.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IValidator#GetJSValidationCode()"></seealso>
    public virtual String GetJSValidationCode()
    {
      XPStringBuilder sb = new XPStringBuilder(1250);
      sb.Append("<script type=\"text/javascript\">\n");
      sb.Append("function isEmpty(elem)\n");
      sb.Append("{\n");
      sb.Append("    var str = elem.value;\n");
      sb.Append("    var re = /.+/;\n");
      sb.Append("    if (!str.match(re))\n");
      sb.Append("    {\n");
      sb.Append("        alert(\"" + GetErrorMessage() + "\");\n");
      sb.Append("        return false;");
      sb.Append("    }\n");
      sb.Append("    else\n");
      sb.Append("    {\n");
      sb.Append("    \t   alert(\"boo!\");\n");
      sb.Append("        return true;\n");
      sb.Append("    }\n");
      sb.Append("}\n");
      sb.Append("</script>\n");
      return sb.ToString();
    }
    
    /// <summary>
    /// Return the name of the JavaScript validation method along with whatever
    /// parameters the script needs.
    /// </summary>
    public virtual String GetJSFunction(String _sParameters)
    {
      if (null != _sParameters)
      {
        return "isEmpty(" + _sParameters + ")";
      } else
      {
        return "isEmpty(" + m_strFormName + "." + m_strInputName + ")";
      }
    }
    
    /// <summary>
    /// Return the name of this Validator
    /// </summary>
    public virtual String GetName()
    {
      return VALIDATOR_NAME;
    }
  }
}