using System;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.errordisplay;
using AValidator=com.plumtree.uiinfrastructure.pagecontrols.framework.AValidator;
namespace com.plumtree.uiinfrastructure.validators
{
  /// <summary>
  /// Validates that an object is a positive int.
  /// </summary>
  /// <author>caiusv</author>
  public class PositiveIntValidator : AValidator,IValidator
  {
    public String VALIDATOR_NAME = "PositiveIntValidator";
    
    /// <summary>
    /// the defualt value of the input 
    /// </summary>
    private String m_strDefaultValue;
    
    /// <summary>
    /// name of the form where this validator appears (used by JavaScript validation)
    /// </summary>
    public String m_strFormName;
    
    /// <summary>
    /// name of the input which this validator works on (used by JavaScript validation) 
    /// </summary>
    public String m_strInputName;
    
    /// <summary>
    /// Creates a new PositiveIntValidator.
    /// </summary>
    /// <param name="_sFormName">name of the form whre this appears</param>
    /// <param name="_sInputName">name of the input element</param>
    /// <param name="_sDefaultValue">default value of the input element</param>
    public PositiveIntValidator(String _sFormName, String _sInputName, String _sDefaultValue)
    {
      m_strFormName = _sFormName;
      m_strInputName = _sInputName;
      m_strDefaultValue = _sDefaultValue;
    }
    
    /// <summary>
    /// Return a UIError object if the object passed in is not a positive int.
    /// </summary>
    public virtual UIError Validate(Object _oObject)
    {
      String strErrorMsg = GetErrorMessage();
      UIError myError = new UIError(UIErrorLevel.FRIENDLY, UIErrorScope.CURRENT_USER,
       strErrorMsg, strErrorMsg);
      if (null == _oObject)
      {
        return myError;
      }
      try
      {
        if (XPConvert.ToInteger(_oObject) < 1)
        {
          return myError;
        }
      }
      catch (Exception e)
      {
        return myError;
      }
      return null;
    }
    
    /// <summary>
    /// Returns the JavaScript Validation method that may be used on a page to validate
    /// associated form input.
    /// </summary>
    /// <seealso cref="com.plumtree.uiinfrastructure.activityspace.IValidator#GetJSValidationCode()"></seealso>
    public virtual String GetJSValidationCode()
    {
      XPStringBuilder sb = new XPStringBuilder(1250);
      sb.Append("<script type=\"text/javascript\">\n");
      sb.Append("function isPositiveInt(inputName)\n");
      sb.Append("{\n");
      sb.Append("    if (!PTNumberUtil.isPositiveInteger(parseInt(inputName.value)))\n");
      sb.Append("    {\n");
      sb.Append("        alert(\"" + GetErrorMessage() + "\");\n");
      sb.Append("        inputName.value = \"" + m_strDefaultValue + "\";\n");
      sb.Append("        return false;\n");
      sb.Append("    }\n");
      sb.Append("    return true;\n");
      sb.Append("}\n");
      sb.Append("</script>\n");
      return sb.ToString();
    }
    
    /// <summary>
    /// Return the name of the JavaScript validation method along with whatever
    /// parameters the script needs.
    /// </summary>
    public virtual String GetJSFunction(String _sParameters)
    {
      if (null != _sParameters)
      {
        return "isPositiveInt(" + _sParameters + ")";
      } else
      {
        return "isPositiveInt(" + m_strFormName + "." + m_strInputName + ")";
        }
    }
    
    /// <summary>
    /// Return the name of this Validator
    /// </summary>
    public virtual String GetName()
    {
      return VALIDATOR_NAME;
    }
  }
}