using System;
//import com.plumtree.xpfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.xpshared.htmlconstructs
{
  /// <author>
  /// beng  Use this class to obtain PTConstruct objects. <br />
  /// <br />
  /// Objects of this class cannot be instantiated.  This class only
  /// provides a  static method for obtaining instances of PTConstruct
  /// objects.
  /// </author>
  public class ConstructFactory
  {
    //
    //    private static XPLogger myLogger = XPLogManager.GetLogger(ConstructFactory.class);
    //
    //
    //    private static boolean bDebug = myLogger.IsDebugEnabled();
    //
    //
    //    private static boolean bError = myLogger.IsErrorEnabled();
    //
    //
    //    private static boolean bWarn = myLogger.IsWarnEnabled();
    //
    //
    //    private static boolean bInfo = myLogger.IsInfoEnabled();
    //
    //
    //    private static boolean bFatal = myLogger.IsFatalEnabled();
    
    /// <summary>
    /// Creates a new PTConstructFactory object.  No need for anyone to ever
    /// call this method.
    /// </summary>
    private ConstructFactory()
    {
    }
    
    /// <summary>
    /// Call this method to obtain a PTConstruct. <br /><br />
    /// </summary>
    /// <param name="type">
    /// Pass the type of construct you wish to obtain.  Construct
    /// types are available as static members of the various
    /// PTConstructType classes.
    /// </param>
    /// <returns></returns>
    public static PTConstruct GetConstruct(PTConstructType type)
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering GetConstruct - type: " + type);
      //        }
      return GetConstruct(type, false);
    }
    
    /// <summary>
    /// Call this method to obtain a PTConstruct. <br /><br />
    /// </summary>
    /// <param name="type">
    /// Pass the type of construct you wish to obtain.  Construct
    /// types are available as static members of the various
    /// PTConstructType classes. <br /><br />
    /// </param>
    /// <param name="bUpdateStrings">
    /// Pass a boolean value indicating whether or not you
    /// wish for the strings of this banner to be updated automatically.
    /// If you pass true, all appropriate strings will be replaced with
    /// localized versions.  If you pass false, all strings will be left
    /// with they prototypical values.  You can update the strings
    /// yourself, using the string dictionary returned by the
    /// PTBannerType.GetStringDictionary method.
    /// </param>
    /// <returns></returns>
    public static PTConstruct GetConstruct(PTConstructType type, bool bUpdateStrings)
    {
      //        if (bInfo)
      //        {
      //            myLogger.Info("Entering GetConstruct - type: " + type);
      //            myLogger.Info("bUpdateStrings: " + bUpdateStrings);
      //        }
      HTMLElement e = null;
      HTMLElement eClone = null;
      PTConstruct construct = null;
      //   Get the prototypical element.
      try
      {
        e = type.GetHTMLElement();
      }
      catch (Exception ex)
      {
      }
      if (e == null)
      {
      } else
      //            if (bError)
      //            {
      //                myLogger.Error("Could not obtain the requested construct: " + type);
      //            }
      {
        //   Clone the prototype, so the client obtains a fresh table.
        try
        {
          //                if (bDebug)
          //                {
          //                    myLogger.Debug("About to call GetClone().");
          //                }
          eClone = (HTMLElement) e.GetClone();
        }
        //                if (bDebug)
        //                {
        //                    myLogger.Debug("eClone: " + eClone);
        //                }
        catch (Exception ex)
        {
          // todo
          //                if (bError)
          //                {
          //                    myLogger.Error("The element could not be cloned.");
          //                }
          throw new XPException("The element could not be cloned.");
        }
        construct = new PTConstruct(type, eClone);
        //
        //            if (bDebug)
        //            {
        //                myLogger.Debug("construct: " + construct);
        //            }
        if (bUpdateStrings)
        {
          //   If we're supposed to, update the strings.
          FactoryHelper.updateStrings(construct);
          FactoryHelper.updateImgAlts(construct);
        }
      }
      //    e is not null;
      return construct;
    }
    
    /// <summary>
    /// Call this method to set the alt attribute values on the images included
    /// within a construct that is one of PTButtonType constructs.
    /// </summary>
    /// <param name="construct">Pass a construct obtained with a PTButtonType</param>
    /// <param name="sAlt">Pass a String alt attribute value</param>
    public static void SetAltOnButtonImages(PTConstruct construct, String sAlt)
    {
      HTMLImg i;
      PTButtonType type;
      try
      {
        type = (PTButtonType) construct.GetType();
      }
      catch (Exception ex)
      {
        return;
      }
      try
      {
        i = (HTMLImg) construct.GetNestedElement(type.IMG_ELEMENT_NAME);
        if (i != null)
        {
          i.SetAlt(sAlt);
        }
      }
      catch (Exception ex)
      {
      }
      // do nothing
      try
      {
        i = (HTMLImg) construct.GetNestedElement(type.IMG_2_ELEMENT_NAME);
        if (i != null)
        {
          i.SetAlt(sAlt);
        }
      }
      catch (Exception e)
      {
      }
    }
    // do nothing
    
    /// <summary>
    /// Call this method to set the title attribute values on the images included
    /// within a construct that is one of PTButtonType constructs.
    /// </summary>
    /// <param name="construct">Pass a construct obtained with a PTButtonType</param>
    /// <param name="sTitle">Pass a String alt attribute value</param>
    public static void SetTitleOnButtonImages(PTConstruct construct, String sTitle)
    {
      HTMLImg i;
      PTButtonType type;
      try
      {
        type = (PTButtonType) construct.GetType();
      }
      catch (Exception ex)
      {
        return;
      }
      try
      {
        i = (HTMLImg) construct.GetNestedElement(type.IMG_ELEMENT_NAME);
        if (i != null)
        {
          i.SetTitle(sTitle);
        }
      }
      catch (Exception ex)
      {
      }
      // do nothing
      try
      {
        i = (HTMLImg) construct.GetNestedElement(type.IMG_2_ELEMENT_NAME);
        if (i != null)
        {
          i.SetTitle(sTitle);
        }
      }
      catch (Exception e)
      {
      }
    }
  }
}
// do nothing
