using System;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.xpshared.htmlconstructs
{
  /// <author>
  /// beng
  /// The static members of this class provide PTBannerType objects that may be
  /// used to obtain constructs from the PTConstructFactory.    These constructs
  /// describe banners commonly appearing in the portal ui.
  /// <br /><br />
  /// A banner is typically a table with a title string, and possibly buttons or
  /// other html elements.
  /// </author>
  public class PTBannerType : PTConstructType
  {
    //	  //   We share the logger for efficiency.
    //	  private static XPLogger myLogger = XPLogManager.GetLogger(PTBannerType.class);
    //
    //	  //   pre-determining which levels are enabled allows the jvm to avoid
    //	  //   concatenating parameters of the logging methods.
    //	  private static boolean bDebug = myLogger.IsDebugEnabled();
    //	  private static boolean bError = myLogger.IsErrorEnabled();
    //	  private static boolean bWarn  = myLogger.IsWarnEnabled();
    //	  private static boolean bInfo  = myLogger.IsInfoEnabled();
    //	  private static boolean bFatal = myLogger.IsFatalEnabled();
    private String sName;
    private HTMLTable myTable;
    private ElementName eName;
    private XPHashtable myStringMap;
    //	  private XPStringDictionary sdImgToAltId;
    private String sDescription;
    private static XPHashtable xpmNameToTableElement = new XPHashtable();
    private static XPHashtable xpmNameToStringDictionary = new XPHashtable();
    private static XPHashtable xpmNameToImgToAltIdMap = new XPHashtable();
    private static XPHashtable xpmNameToDescription = new XPHashtable();
    private static XPArrayList alTypes = new XPArrayList();
    private static XPHashtable xpmImgToAltId;
    private XPHashtable xpmElementNames = new XPHashtable();
    
    /// <summary>
    /// Use this ElementName to obtain the HTMLElement containing
    /// the banner's help button.
    /// </summary>
    public ElementName HELP_CONTAINER_ELEMENT_NAME = null;
    public ElementName FOLDER_IMG_CONTAINER_ELEMENT_NAME = null;
    
    /// <summary>
    /// Use this ElementName to obtain the HTMLElement containing
    /// the banner's title.
    /// </summary>
    public ElementName TITLE_ELEMENT_NAME = null;
    
    /// <summary>
    /// Use this ElementName to obtain the HTMLElement containing
    /// the banner's subtitle.
    /// </summary>
    public ElementName SUBTITLE_ELEMENT_NAME = null;
    
    /// <summary>
    /// Use this ElementName to obtain the HTMLElement containing
    /// the banner folder image.
    /// </summary>
    public ElementName FOLDER_IMG_ELEMENT_NAME = null;
    
    /// <summary>
    /// Use this ElementNameto obtain the HTMLElement that
    /// should contain a collapse or expand button.  Banner's do not come
    /// from the factory with these buttons, but should be inserted into this
    /// element.
    /// </summary>
    public ElementName EXPAND_COLLAPSE_CONTAINER_ELEMENT_NAME = null;
    
    /// <summary>
    /// Use this ElementName ID to obtain the HTMLElement containing
    /// the banner's extendable cell.  This cell is where you should insert
    /// additional buttons etc.  Not all banner's have such a cell.
    /// </summary>
    public ElementName EXTENDABLE_CELL_ELEMENT_NAME = null;
    
    /// <summary>
    /// Use this ElementName ID to obtain the HTMLElement containing
    /// the banner's secondary extendable cell.  This cell is where you should
    /// insert additional buttons etc.
    /// </summary>
    public ElementName EXTENDABLE_CELL2_ELEMENT_NAME = null;
    
    /// <summary>
    /// Use
    /// </summary>
    public ElementName RIGHT_SIDE_EXTENDABLE_CELL_ELEMENT_NAME = null;
    public const String ELEMENT_NAME_ID_RIGHT_SIDE_EXTENDABLE_CELL = "rsec_enid";
    
    /// <summary>
    /// Use this ElementName ID to obtain an ElementName from a specific
    /// PTBannerType that will allow you to obtain the HTMLElement containing
    /// the banner's title.
    /// </summary>
    public const String ELEMENT_NAME_ID_HELP_CONTAINER = "help_container_enid";
    
    /// <summary>
    /// Use this ElementName ID to obtain an ElementName from a specific
    /// PTBannerType that will allow you to obtain the HTMLElement containing
    /// the banner's title.
    /// </summary>
    public const String ELEMENT_NAME_ID_TITLE = "banner_title_enid";
    
    /// <summary>
    /// Use this ElementName ID to obtain an ElementName from a specific
    /// PTBannerType that will allow you to obtain the HTMLElement containing
    /// the banner's subtitle.
    /// </summary>
    public const String ELEMENT_NAME_ID_SUBTITLE = "banner_subtitle_enid";
    
    /// <summary>
    /// Use this ElementName ID to obtain an ElementName from a specific
    /// PTBannerType that will allow you to obtain the HTMLElement containing
    /// the folder image.
    /// </summary>
    public const String ELEMENT_NAME_ID_FOLDER_IMG = "banner_folderimg_enid";
    public const String ELEMENT_NAME_ID_FOLDER_IMG_CONTAINER = "banner_folderimg_con_edid";
    
    /// <summary>
    /// Use this ElementName ID to obtain an ElementName from a specific
    /// PTBannerType that will allow you to obtain the HTMLElement that
    /// should contain a collapse or expand button.  Banner's do not come from the
    /// factory with these buttons, but should be inserted into this
    /// element.
    /// </summary>
    public const String ELEMENT_NAME_ID_EXPAND_COLLAPSE_CONTAINER = "banner_expcollcont_enid";
    
    /// <summary>
    /// Use this ElementName ID to obtain an ElementName from a specific
    /// PTBannerType that will allow you to obtain the HTMLElement containing
    /// the banner's extendable cell.  This cell is where you should insert
    /// additional buttons etc.  Not all banner's have such a cell.
    /// </summary>
    public const String ELEMENT_NAME_ID_EXTENDABLE_CELL = "banner_extcell_enid";
    
    /// <summary>
    /// Use this ElementName ID to obtain an ElementName from a specific
    /// PTBannerType that will allow you to obtain the HTMLElement containing
    /// the banner's extendable cell.  This cell is where you should insert
    /// additional buttons etc.  Not all banner's have such a cell.
    /// </summary>
    public const String ELEMENT_NAME_ID_EXTENDABLE_CELL_2 = "banner_extcell2_enid";
    private static bool bInitCalled = false;
    
    /// <summary>
    /// C# handles static blocks differently than java.  In C#, the static
    /// initializer is run after all private members are initialized.  For this
    /// reason, we must guarantee that the following static initialization code
    /// is run before the private constructor is called.
    /// </summary>
    private static void staticInit()
    {
      if (bInitCalled)
      {
        return;
      }
      bInitCalled = true;
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("Entering static block.");
      //		  }
      HTMLElementCollection myCollection = null;
      HTMLGenericElement myGenericElement = null;
      HTMLTable myTable = null;
      HTMLTableRow myRow = null;
      HTMLTableCell myCell = null;
      HTMLSpan mySpan = null;
      HTMLImg myImg = null;
      XPStringDictionary sdTemp = null;
      HTMLSpan folderIconSpan = null;
      PTConstruct myConstruct = null;
      String sBannerName;
      //////////////////////////////////////////////////////////
      //   FOLDER
      sBannerName = "folder";
      myCollection = new HTMLElementCollection();
      try
      {
        myConstruct = ConstructFactory.GetConstruct(PTImageType.ICON_FOLDER_OPEN,
         false);
      }
      catch (Exception e)
      {
      }
      // do nothing
      myImg = (HTMLImg) myConstruct.GetHTMLElement();
      myImg.SetAlign("absMiddle");
      String sImgName = PTBannerType.GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_FOLDER_IMG,
       sBannerName);
      try
      {
        myImg.SetElementName(sImgName);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      sdTemp = new XPStringDictionary();
      sdTemp.PutElement(sImgName, "folderImgAltID");
      try
      {
        mySpan = new HTMLSpan();
        mySpan.SetElementName(GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_EXPAND_COLLAPSE_CONTAINER,
         sBannerName));
        //   add the collapse button container
        myCollection.AddInnerHTMLElement(mySpan);
        //   add a space
        //			myCollection.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        //   add the folder icon
        folderIconSpan = new HTMLSpan();
        folderIconSpan.SetStyleClass(PTStyleClass.DIR_HEADER);
        folderIconSpan.AddInnerHTMLElement(myImg);
        folderIconSpan.SetElementName(PTBannerType.GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_FOLDER_IMG_CONTAINER,
         sBannerName));
        mySpan = new HTMLSpan();
        mySpan.AddInnerHTMLString(sBannerName);
        mySpan.SetElementName(PTBannerType.GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_TITLE,
         sBannerName));
        mySpan.SetStyleClass(PTStyleClass.DIR_HEADER);
        //   add the title
        myCollection.AddInnerHTMLElement(mySpan);
      }
      catch (Exception e)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error("Couln't add stuff to elementcollection.");
      //			  }
      //   Do not set the title element name, because we set it above.
      GetButtonBanner(sBannerName, myCollection, null, PTStyleClass.DIR_HEADER_BG,
       PTStyleClass.DIR_TEXT, PTStyleClass.DIR_BTN, "25", true, sdTemp, folderIconSpan);
      xpmNameToDescription.PutElement(sBannerName, "Use this banner above the list of folders.");
      //////////////////////////////////////////////////////////
      //   OBJECT
      sBannerName = "object";
      //   <table> <tr> <td> <span EXTENDABLE> <span TITLE> <span SUBSTITLE>
      myTable = new HTMLTable();
      myRow = new HTMLTableRow();
      try
      {
        myTable.SetPreComment("begin " + sBannerName + " banner");
        myTable.SetPostComment("end " + sBannerName + " banner");
        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.SetCellPadding(CommonHTMLStrings.ZERO);
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        myTable.SetBorder(CommonHTMLStrings.ZERO);
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.OBJECT_HEADER_BG);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
        myCell.SetHeight("25");
        myCell.SetVAlign("middle");
        mySpan = new HTMLSpan();
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.SetElementName(GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_EXPAND_COLLAPSE_CONTAINER,
         sBannerName));
        mySpan = new HTMLSpan();
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.SetElementName(GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_EXTENDABLE_CELL,
         sBannerName));
        mySpan = new HTMLSpan();
        myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.SetElementName(GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_TITLE,
         sBannerName));
        mySpan.SetStyleClass(PTStyleClass.OBJECT_HEADER);
        mySpan.AddInnerHTMLString("PLACEHOLDER_TITLE");
        mySpan = new HTMLSpan();
        mySpan.SetStyleClass(PTStyleClass.OBJECT_TEXT);
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.SetElementName(GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_SUBTITLE,
         sBannerName));
        mySpan.AddInnerHTMLString("PLACEHOLDER_SUBTITLE");
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error("couldn't make banner: " + he);
      //			  }
      xpmNameToTableElement.PutElement(sBannerName, myTable);
      xpmNameToDescription.PutElement(sBannerName, "Use this banner above a list of managed objects.  This banner should appear immediately above the OBJECT_ACTION_BAR banner.");
      //////////////////////////////////////////////////////////
      //   OBJECT_508
      sBannerName = "object_508";
      //   <table> <tr> <td> <span EXTENDABLE> <h3 TITLE> <h4 SUBSTITLE>
      myTable = new HTMLTable();
      myRow = new HTMLTableRow();
      try
      {
        myTable.SetPreComment("begin " + sBannerName + " banner");
        myTable.SetPostComment("end " + sBannerName + " banner");
        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.SetCellPadding(CommonHTMLStrings.ZERO);
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        myTable.SetBorder(CommonHTMLStrings.ZERO);
        myTable.AddInnerHTMLElement(myRow);
        myRow.SetStyleClass(PTStyleClass.DIR_HEADER_BG);
        myCell = new HTMLTableCell();
        myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
        myCell.SetHeight("25");
        myCell.SetVAlign("middle");
        myRow.AddInnerHTMLElement(myCell);
        mySpan = new HTMLSpan();
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.SetElementName(GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_EXPAND_COLLAPSE_CONTAINER,
         sBannerName));
        mySpan = new HTMLSpan();
        myCell.AddInnerHTMLElement(mySpan);
        mySpan.SetElementName(GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_EXTENDABLE_CELL,
         sBannerName));
        HTMLH3 myHeaderH3 = new HTMLH3();
        myHeaderH3.SetElementName(GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_TITLE,
         sBannerName));
        myHeaderH3.AddInnerHTMLString("PLACEHOLDER_TITLE");
        myHeaderH3.SetStyleClass(PTStyleClass.NAV_SIDEBARSECTION_HEADER);
        myCell.AddInnerHTMLElement(myHeaderH3);
        HTMLH4 myHeaderH4 = new HTMLH4();
        myCell.AddInnerHTMLElement(myHeaderH4);
        myHeaderH4.SetElementName(GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_SUBTITLE,
         sBannerName));
        myHeaderH4.SetStyleClass(PTStyleClass.NAV_SIDEBARSUBSUBSECTION_HEADER);
        myHeaderH4.AddInnerHTMLString("PLACEHOLDER_SUBTITLE");
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error("couldn't make banner: " + he);
      //			  }
      xpmNameToTableElement.PutElement(sBannerName, myTable);
      xpmNameToDescription.PutElement(sBannerName, "Use this banner above a list of managed objects.  This banner should appear immediately above the OBJECT_ACTION_BAR banner.");
      //////////////////////////////////////////////////////////
      //   NARROW_COLLAPSABLE_EXPANDED
      sBannerName = "narrow_collapsable_expanded";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      myTable = GetBannerStyle1(sBannerName, "PLACEHOLDER TITLE", false);
      //////////////////////////////////////////////////////////
      //   NARROW_COLLAPSABLE_COLLAPSED
      sBannerName = "narrow_collapsable_collapsed";
      //
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      myTable = GetBannerStyle1(sBannerName, "PLACEHOLDER TITLE", true);
      //////////////////////////////////////////////////////////
      //   PORTLET
      sBannerName = "portlet";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      mySpan = new HTMLSpan();
      try
      {
        mySpan.SetElementName(PTBannerType.GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_TITLE,
         sBannerName));
        mySpan.AddInnerHTMLString("PLACEHOLDER_TITLE");
        mySpan.SetStyleClass(PTStyleClass.PLATPORTLET_NARROW_HEADER);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(he);
      //			  }
      GetButtonBanner(sBannerName, mySpan, PTStyleClass.PLATPORTLET_HEADER_BG,
       PTStyleClass.PLATPORTLET_HEADER_BG, PTStyleClass.PLATPORTLET_SUBHEAD_BG,
       PTStyleClass.PLATPORTLET_SUBHEAD_BG, "18", false);
      xpmNameToDescription.PutElement(sBannerName, "This banner should be used above each portlet.");
      //////////////////////////////////////////////////////////
      //   MOST_RECENTLY_USED_EXPANDED
      sBannerName = "most_recently_used_expanded";
      //
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      myTable = GetBannerStyle1(sBannerName, "Most Recently Used", false);
      //////////////////////////////////////////////////////////
      //   NARROW
      sBannerName = "narrow";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      //////////////////////////////////////////////////////////
      //   WIDE
      sBannerName = "wide";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      mySpan = new HTMLSpan();
      try
      {
        mySpan.SetElementName(PTBannerType.GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_TITLE,
         sBannerName));
        mySpan.AddInnerHTMLString(CommonHTMLStrings.SPACE + "PLACEHOLDER_TITLE");
        mySpan.SetStyleClass(PTStyleClass.DIR_HEADER);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(he);
      //			  }
      //      GetEmptyBanner( sBannerName, myGenericElement, PTStyleClass.MENU_HEADER_BG,
      //         PTStyleClass.MENU_TEXT, "22", true );
      GetButtonBanner(sBannerName, mySpan, PTStyleClass.DIR_HEADER_BG, PTStyleClass.DIR_HEADER_BG,
       PTStyleClass.DIR_TEXT, PTStyleClass.DIR_BTN, "25", false);
      //////////////////////////////////////////////////////////
      //   WIDE_508
      sBannerName = "wide_508";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      HTMLH2 myHeaderH2 = new HTMLH2();
      try
      {
        myHeaderH2.SetElementName(PTBannerType.GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_TITLE,
         sBannerName));
        myHeaderH2.AddInnerHTMLString(CommonHTMLStrings.SPACE + "PLACEHOLDER_TITLE");
        myHeaderH2.SetStyleClass(PTStyleClass.BAN_SUBHEADER);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //      GetEmptyBanner( sBannerName, myGenericElement, PTStyleClass.MENU_HEADER_BG,
      //         PTStyleClass.MENU_TEXT, "22", true );
      GetButtonBanner(sBannerName, myHeaderH2, PTStyleClass.DIR_HEADER_BG,
       PTStyleClass.DIR_HEADER_BG, PTStyleClass.DIR_TEXT, PTStyleClass.DIR_BTN,
       "25", false);
      //////////////////////////////////////////////////////////
      //   MENU
      sBannerName = "menu";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      myCollection = new HTMLElementCollection();
      AddCollapseExpandSpanToCollection(myCollection, sBannerName);
      mySpan = new HTMLSpan();
      try
      {
        myCollection.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        myCollection.AddInnerHTMLElement(mySpan);
        mySpan.SetElementName(PTBannerType.GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_TITLE,
         sBannerName));
        mySpan.AddInnerHTMLString("PLACEHOLDER_TITLE");
        mySpan.SetStyleClass(PTStyleClass.MENU_HEADER);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(he);
      //			  }
      GetButtonBanner(sBannerName, myCollection, PTStyleClass.MENU_HEADER_BG,
       PTStyleClass.MENU_HEADER_BG, PTStyleClass.MENU_TEXT, PTStyleClass.MENU_TEXT,
       "20", false);
      xpmNameToDescription.PutElement(sBannerName, "This banner should be used above a list of items, on the left side of the administrative sections of the portal.");
      //////////////////////////////////////////////////////////
      //   TIPS
      sBannerName = "tips";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      myCollection = new HTMLElementCollection();
      AddCollapseExpandSpanToCollection(myCollection, sBannerName);
      mySpan = new HTMLSpan();
      try
      {
        myCollection.AddInnerHTMLElement(mySpan);
        mySpan.SetElementName(PTBannerType.GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_TITLE,
         sBannerName));
        mySpan.AddInnerHTMLString("PLACEHOLDER_TITLE");
        mySpan.SetStyleClass(PTStyleClass.TIPS_HEADER);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(he);
      //			  }
      GetButtonBanner(sBannerName, myCollection, PTStyleClass.TIPS_HEADER_BG,
       PTStyleClass.TIPS_HEADER_BG, PTStyleClass.TIPS_TEXT, PTStyleClass.TIPS_TEXT,
       "18", false);
      xpmNameToDescription.PutElement(sBannerName, "This banner should only be used above the tips box.");
      //////////////////////////////////////////////////////////
      //   NARROW2
      sBannerName = "narrow_2";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      //////////////////////////////////////////////////////////
      //   WIDE2
      sBannerName = "wide_2";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      myGenericElement = new HTMLGenericElement(sBannerName);
      GetEmptyBanner(sBannerName, myGenericElement, PTStyleClass.MENU_HEADER_BG,
       PTStyleClass.MENU_TEXT, "22", true);
      //////////////////////////////////////////////////////////
      //   OBJECT_ACTION_BAR
      sBannerName = "object_action_bar";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      myTable = new HTMLTable();
      myRow = new HTMLTableRow();
      try
      {
        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.SetBorder(CommonHTMLStrings.ZERO);
        myTable.SetCellPadding(CommonHTMLStrings.ZERO);
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        myRow.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
        myCell.SetHeight("25");
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_EXTENDABLE_CELL,
         sBannerName));
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_DIVIDER);
        myCell.SetHeight("25");
        myConstruct = ConstructFactory.GetConstruct(PTImageType.SPACER);
        myImg = (HTMLImg) myConstruct.GetHTMLElement();
        myImg.SetWidth("1");
        myCell.AddInnerHTMLElement(myImg);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
        myCell.SetHeight("25");
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_EXTENDABLE_CELL_2,
         sBannerName));
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_DIVIDER);
        myCell.SetHeight("25");
        myConstruct = ConstructFactory.GetConstruct(PTImageType.SPACER);
        myImg = (HTMLImg) myConstruct.GetHTMLElement();
        myImg.SetWidth("1");
        myCell.AddInnerHTMLElement(myImg);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
        myCell.SetHeight("25");
        myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetAlign(CommonHTMLStrings.RIGHT);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
        myCell.SetHeight("25");
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_HELP_CONTAINER,
         sBannerName));
        myRow.AddInnerHTMLElement(myCell);
      }
      /*
       myCell = new HTMLTableCell();
       
       myCell.SetNoWrap(true);
       myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
       myCell.SetHeight("25");
       myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
       
       myRow.AddInnerHTMLElement(myCell);
       
       */
       
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(he);
      //			  }
      catch (Exception ex)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(ex);
      //			  }
      xpmNameToTableElement.PutElement(sBannerName, myTable);
      xpmNameToDescription.PutElement(sBannerName, "This banner should be used in conjunction with the OBJECT banner.  This banner has multiple extendable cells to which buttons can be added.");
      //////////////////////////////////////////////////////////
      //   OBJECT_ACTION_BAR_CONTENT
      sBannerName = "object_action_bar_content";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      myTable = new HTMLTable();
      myRow = new HTMLTableRow();
      try
      {
        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.SetBorder(CommonHTMLStrings.ZERO);
        myTable.SetCellPadding("5");
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        myRow.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myCell.AddInnerHTMLString("_TITLE_");
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
        myCell.SetVAlign("top");
        myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_TITLE,
         sBannerName));
        myRow.AddInnerHTMLElement(myCell);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(he);
      //			  }
      catch (Exception ex)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(ex);
      //			  }
      xpmNameToTableElement.PutElement(sBannerName, myTable);
      xpmNameToDescription.PutElement(sBannerName, "This... ");
      //////////////////////////////////////////////////////////
      //   DIRECTORY_CONTENT
      sBannerName = "directory_content";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      myTable = new HTMLTable();
      myRow = new HTMLTableRow();
      try
      {
        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.SetBorder(CommonHTMLStrings.ZERO);
        myTable.SetCellPadding("5");
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        myRow.SetStyleClass(PTStyleClass.DIR_CONTENT_BG);
        myTable.AddInnerHTMLElement(myRow);
        mySpan = new HTMLSpan();
        mySpan.SetStyleClass(PTStyleClass.DIR_CONTENT_TEXT);
        mySpan.AddInnerHTMLString("_TITLE_");
        myCell = new HTMLTableCell();
        myCell.AddInnerHTMLElement(mySpan);
        //			  myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
        myCell.SetVAlign("top");
        myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_TITLE,
         sBannerName));
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
        myCell.SetVAlign("top");
        myCell.SetNoWrap(true);
        myCell.SetAlign(CommonHTMLStrings.RIGHT);
        myCell.SetWidth(CommonHTMLStrings.ZERO);
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_HELP_CONTAINER,
         sBannerName));
        myRow.AddInnerHTMLElement(myCell);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(he);
      //			  }
      catch (Exception ex)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(ex);
      //			  }
      xpmNameToTableElement.PutElement(sBannerName, myTable);
      xpmNameToDescription.PutElement(sBannerName, "This... ");
      //////////////////////////////////////////////////////////
      //   DIRECTORY_ACTION_BAR
      sBannerName = "directory_action_bar";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      myTable = new HTMLTable();
      myRow = new HTMLTableRow();
      try
      {
        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.SetBorder(CommonHTMLStrings.ZERO);
        myTable.SetCellPadding(CommonHTMLStrings.ZERO);
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        myRow.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_BG);
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_BTN);
        myCell.SetHeight("25");
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_EXTENDABLE_CELL,
         sBannerName));
        myCell.SetPreComment(ELEMENT_NAME_ID_EXTENDABLE_CELL);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_DIVIDER);
        myCell.SetHeight("25");
        myConstruct = ConstructFactory.GetConstruct(PTImageType.SPACER);
        myImg = (HTMLImg) myConstruct.GetHTMLElement();
        myImg.SetWidth("1");
        myImg.SetAlt("");
        myCell.AddInnerHTMLElement(myImg);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_BTN);
        myCell.SetHeight("25");
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_EXTENDABLE_CELL_2,
         sBannerName));
        myCell.SetPreComment(ELEMENT_NAME_ID_EXTENDABLE_CELL_2);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_DIVIDER);
        myCell.SetHeight("25");
        myConstruct = ConstructFactory.GetConstruct(PTImageType.SPACER);
        myImg = (HTMLImg) myConstruct.GetHTMLElement();
        myImg.SetWidth("1");
        myCell.AddInnerHTMLElement(myImg);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_BTN);
        myCell.SetHeight("25");
        myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetAlign(CommonHTMLStrings.RIGHT);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_BTN);
        myCell.SetHeight("25");
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_RIGHT_SIDE_EXTENDABLE_CELL,
         sBannerName));
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_BTN);
        myCell.SetHeight("25");
        myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        myRow.AddInnerHTMLElement(myCell);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(he);
      //			  }
      catch (Exception ex)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(ex);
      //			  }
      xpmNameToTableElement.PutElement(sBannerName, myTable);
      xpmNameToDescription.PutElement(sBannerName, "This banner contains only buttons.  This banner contains multiple extendable cells to which buttons can be added.");
      //////////////////////////////////////////////////////////
      //   BANNER_ACTION_BAR
      sBannerName = "banner_action_bar";
      //
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      myTable = new HTMLTable();
      myRow = new HTMLTableRow();
      try
      {
        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.SetBorder(CommonHTMLStrings.ZERO);
        myTable.SetCellPadding(CommonHTMLStrings.ZERO);
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        myRow.SetStyleClass(PTStyleClass.ACTIONBAR_BAN_BG);
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_BTN);
        myCell.SetHeight("25");
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_EXTENDABLE_CELL,
         sBannerName));
        myCell.SetPreComment(ELEMENT_NAME_ID_EXTENDABLE_CELL);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_DIVIDER);
        myCell.SetHeight("25");
        myConstruct = ConstructFactory.GetConstruct(PTImageType.SPACER);
        myImg = (HTMLImg) myConstruct.GetHTMLElement();
        myImg.SetWidth("1");
        myImg.SetAlt("");
        myCell.AddInnerHTMLElement(myImg);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_BTN);
        myCell.SetHeight("25");
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_EXTENDABLE_CELL_2,
         sBannerName));
        myCell.SetPreComment(ELEMENT_NAME_ID_EXTENDABLE_CELL_2);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_DIVIDER);
        myCell.SetHeight("25");
        myConstruct = ConstructFactory.GetConstruct(PTImageType.SPACER);
        myImg = (HTMLImg) myConstruct.GetHTMLElement();
        myImg.SetWidth("1");
        myCell.AddInnerHTMLElement(myImg);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
        myCell.SetHeight("25");
        myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetAlign(CommonHTMLStrings.RIGHT);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BTN);
        myCell.SetHeight("25");
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_HELP_CONTAINER,
         sBannerName));
        myRow.AddInnerHTMLElement(myCell);
        myCell = new HTMLTableCell();
        myCell.SetNoWrap(true);
        myCell.SetStyleClass(PTStyleClass.ACTIONBAR_DIR_BTN);
        myCell.SetHeight("25");
        myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        myRow.AddInnerHTMLElement(myCell);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(he);
      //			  }
      catch (Exception ex)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(ex);
      //			  }
      xpmNameToTableElement.PutElement(sBannerName, myTable);
      xpmNameToDescription.PutElement(sBannerName, "This banner contains only buttons.  This banner contains multiple extendable cells to which buttons can be added.");
      //////////////////////////////////////////////////////////
      //   CONTENT_ACCESS_PRIVILEGES
      sBannerName = "content_access_privileges";
      //////////////////////////////////////////////////////////
      //   SEARCH_EXPANDED
      sBannerName = "search_expanded";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      //////////////////////////////////////////////////////////
      //   TIP_EXPANDED
      sBannerName = "tip_expanded";
      //		  if (bDebug)
      //		  {
      //			  myLogger.Debug("About to prototype banner: " + sBannerName);
      //		  }
      //////////////////////////////////////////////////////////
      //   ADMIN_DIRECTORY_2
      sBannerName = "admin_directory_2";
    }
    //
    //		  if (bDebug)
    //		  {
    //			  myLogger.Debug("Leaving static block.");
    //		  }
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    /// <deprecated></deprecated>
    public static readonly PTBannerType NARROW_COLLAPSABLE_EXPANDED = new PTBannerType("narrow_collapsable_expanded");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    /// <deprecated></deprecated>
    public static readonly PTBannerType NARROW_COLLAPSABLE_COLLAPSED = new PTBannerType("narrow_collapsable_collapsed");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType PORTLET = new PTBannerType("portlet");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    /// <deprecated></deprecated>
    public static readonly PTBannerType NARROW = new PTBannerType("narrow");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType WIDE = new PTBannerType("wide");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType WIDE_508 = new PTBannerType("wide_508");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    /// <deprecated></deprecated>
    public static readonly PTBannerType NARROW_2 = new PTBannerType("narrow_2");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    /// <deprecated></deprecated>
    public static readonly PTBannerType WIDE_2 = new PTBannerType("wide_2");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType SEARCH_EXPANDED = new PTBannerType("search_expanded");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    /// <deprecated></deprecated>
    public static readonly PTBannerType TIP_EXPANDED = new PTBannerType("tip_expanded");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    /// <deprecated></deprecated>
    public static readonly PTBannerType ADMIN_DIRECTORY_2 = new PTBannerType("admin_directory_2");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    /// <deprecated></deprecated>
    public static readonly PTBannerType MOST_RECENTLY_USED_EXPANDED = new PTBannerType("most_recently_used_expanded");
    
    /// <deprecated></deprecated>
    public static readonly PTBannerType CONTENT_ACCESS_PRIVILEGES = new PTBannerType("content_access_privileges");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType FOLDER = new PTBannerType("folder");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType OBJECT = new PTBannerType("object");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType OBJECT_508 = new PTBannerType("object_508");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType MENU = new PTBannerType("menu");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType TIPS = new PTBannerType("tips");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType OBJECT_ACTION_BAR = new PTBannerType("object_action_bar");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType DIRECTORY_ACTION_BAR = new PTBannerType("directory_action_bar");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType OBJECT_ACTION_BAR_CONTENT = new PTBannerType("object_action_bar_content");
    
    /// <summary>
    /// This PTBannerType is to be used for
    /// </summary>
    public static readonly PTBannerType BANNER_ACTION_BAR = new PTBannerType("banner_action_bar");
    public static readonly PTBannerType DIRECTORY_CONTENT = new PTBannerType("directory_content");
    static PTBannerType()
    {
      alTypes.Add(PTBannerType.FOLDER);
      alTypes.Add(PTBannerType.OBJECT);
      alTypes.Add(PTBannerType.WIDE);
      alTypes.Add(PTBannerType.PORTLET);
      alTypes.Add(PTBannerType.TIPS);
      alTypes.Add(PTBannerType.MENU);
      alTypes.Add(PTBannerType.OBJECT_ACTION_BAR);
      alTypes.Add(PTBannerType.OBJECT_ACTION_BAR_CONTENT);
      alTypes.Add(PTBannerType.DIRECTORY_ACTION_BAR);
      alTypes.Add(PTBannerType.DIRECTORY_CONTENT);
      alTypes.Add(PTBannerType.BANNER_ACTION_BAR);
    }
    /////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////
    //
    //      CONSTRUCTOR
    //
    
    /// <summary>
    /// Creates a new PTBannerType object.
    /// </summary>
    /// <param name="sTypeName"></param>
    private PTBannerType(String sTypeName)
    {
      //		  if (bInfo)
      //		  {
      //			  myLogger.Info("Entering PTBannerType() - sTypeName: " + sTypeName);
      //		  }
      PTBannerType.staticInit();
      sName = sTypeName;
      ElementName enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_RIGHT_SIDE_EXTENDABLE_CELL,
       sName));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_RIGHT_SIDE_EXTENDABLE_CELL,
       enTemp);
      this.RIGHT_SIDE_EXTENDABLE_CELL_ELEMENT_NAME = enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_FOLDER_IMG_CONTAINER,
       sName));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_FOLDER_IMG_CONTAINER, enTemp);
      this.FOLDER_IMG_CONTAINER_ELEMENT_NAME = enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_FOLDER_IMG,
       sName));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_FOLDER_IMG, enTemp);
      this.FOLDER_IMG_ELEMENT_NAME = enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_HELP_CONTAINER,
       sName));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_HELP_CONTAINER, enTemp);
      HELP_CONTAINER_ELEMENT_NAME = enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_TITLE,
       sName));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_TITLE, enTemp);
      TITLE_ELEMENT_NAME = enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_EXTENDABLE_CELL,
       sName));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_EXTENDABLE_CELL, enTemp);
      EXTENDABLE_CELL_ELEMENT_NAME = enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_SUBTITLE,
       sName));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_SUBTITLE, enTemp);
      SUBTITLE_ELEMENT_NAME = enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_EXTENDABLE_CELL_2,
       sName));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_EXTENDABLE_CELL_2, enTemp);
      EXTENDABLE_CELL2_ELEMENT_NAME = enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_EXPAND_COLLAPSE_CONTAINER,
       sName));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_EXPAND_COLLAPSE_CONTAINER,
       enTemp);
      EXPAND_COLLAPSE_CONTAINER_ELEMENT_NAME = enTemp;
      try
      {
        myTable = (HTMLTable) xpmNameToTableElement.GetElement(sName);
      }
      catch (Exception e)
      {
        // do nothing
        //			  if (bError)
        //			  {
        //				  myLogger.Error("e: " + e);
        //			  }
        myTable = null;
      }
      try
      {
        myStringMap = (XPHashtable) xpmNameToStringDictionary.GetElement(sName);
        }
      catch (Exception e)
      {
        myStringMap = null;
      }
      try
      {
        //			  sdImgToAltId = (XPStringDictionary) xpmNameToImgToAltIdMap.GetElement(sName);
        xpmImgToAltId = (XPHashtable) xpmNameToImgToAltIdMap.GetElement(sName);
        }
      catch (Exception e)
      {
        xpmImgToAltId = null;
      }
      try
      {
        sDescription = (String) xpmNameToDescription.GetElement(sName);
      }
      catch (Exception e)
      {
      }
      // do nothing
      if (sDescription == null)
      {
        sDescription = sName + " banner description has not been specified.";
        }
    }
    
    /// <returns></returns>
    public override String GetDescription()
    {
      return sDescription;
    }
    
    /// <param name="bannerName"></param>
    /// <param name="sTitle"></param>
    /// <param name="bCollapsed"></param>
    /// <returns></returns>
    private static HTMLTable GetBannerStyle1(String bannerName, String sTitle,
     bool bCollapsed)
    {
      //		  if (bInfo)
      //		  {
      //			  myLogger.Info("Entering GetBannerStyle1()");
      //			  myLogger.Info("bannerName: " + bannerName);
      //			  myLogger.Info("sTitle: " + sTitle);
      //		  }
      HTMLSpan mySpan;
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      HTMLElement myElement;
      mySpan = new HTMLSpan();
      try
      {
        mySpan.SetStyleClass(PTStyleClass.DIR_HEADER);
        mySpan.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        mySpan.AddInnerHTMLString(sTitle);
        mySpan.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_TITLE,
         bannerName));
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      PTConstruct myConstruct = null;
      try
      {
        if (bCollapsed)
        {
          myConstruct = ConstructFactory.GetConstruct(PTButtonType.EXPAND_AREA,
           false);
        } else
        {
          myConstruct = ConstructFactory.GetConstruct(PTButtonType.COLLAPSE_AREA,
           false);
        }
      }
      catch (Exception e)
      {
      }
      // do nothing
      myElement = myConstruct.GetHTMLElement();
      myTable = new HTMLTable();
      XPStringBuilder sb = new XPStringBuilder(200);
      sb.Append("begin ");
      sb.Append(bannerName);
      sb.Append(" banner");
      myTable.SetPreComment(sb.ToString());
      sb = new XPStringBuilder(200);
      sb.Append("end ");
      sb.Append(bannerName);
      sb.Append(" banner");
      myTable.SetPostComment(sb.ToString());
      myRow = new HTMLTableRow();
      try
      {
        myTable.SetCellPadding(CommonHTMLStrings.ZERO);
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        myTable.SetStyleClass(PTStyleClass.BAN_HEADER_BG);
        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetHeight("22");
        myCell.SetWidth("22");
        myCell.SetStyleClass(PTStyleClass.BAN_HELP_TEXT);
        myCell.AddInnerHTMLElement(myElement);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.AddInnerHTMLElement(mySpan);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error("Couldn't add elements to cell. he: " + he);
      //			  }
      XPStringDictionary sdTemp;
      sdTemp = new XPStringDictionary();
      //   Gadget banner title should not be updated by the factory
      sdTemp.PutElement(GetElementNameAsString(ELEMENT_NAME_ID_TITLE, bannerName),
       PTConstructType.DO_NOT_UPDATE);
      xpmNameToStringDictionary.PutElement(bannerName, sdTemp);
      xpmNameToTableElement.PutElement(bannerName, myTable);
      //      XPArrayList alTemp = new XPArrayList();
      if (bCollapsed)
      {
        //         alTemp.Add( PTButtonType.EXPAND_AREA.GetImgElementNameToAltIdMap() );
        xpmImgToAltId = PTButtonType.EXPAND_AREA.GetImgElementNameToAltIdMap();
        } else
      {
        //         alTemp.Add( PTButtonType.COLLAPSE_AREA.GetImgElementNameToAltIdMap() );
        xpmImgToAltId = PTButtonType.COLLAPSE_AREA.GetImgElementNameToAltIdMap();
        }
      //      sdTemp = PTFactoryHelper.combineStringDictionaries( alTemp.GetEnumerator() );
      //		  xpmNameToImgToAltIdMap.PutElement(bannerName, sdTemp);
      return myTable;
    }
    
    /// <summary>
    /// GetButtonBanner without an img-to-alt-id map
    /// </summary>
    private static HTMLTable GetButtonBanner(String bannerName, HTMLElement heTitle,
     PTStyleClass tableClass, PTStyleClass rowClass, PTStyleClass titleCellClass,
     PTStyleClass buttonCellClass, String sCellHeight, bool bSetTitleElementName)
    {
      return GetButtonBanner(bannerName, heTitle, tableClass, rowClass, titleCellClass,
       buttonCellClass, sCellHeight, bSetTitleElementName, null, null);
    }
    
    /// <summary>
    /// GetButtonBanner with all possible parameters
    /// </summary>
    private static HTMLTable GetButtonBanner(String bannerName, HTMLElement heTitle,
     PTStyleClass tableClass, PTStyleClass rowClass, PTStyleClass titleCellClass,
     PTStyleClass buttonCellClass, String sCellHeight, bool bSetTitleElementName,
     XPStringDictionary imgNameToAltId, HTMLElement ePreTitle)
    {
      //		  if (bInfo)
      //		  {
      //			  myLogger.Info("Entering GetButtonBanner()");
      //			  myLogger.Info("bannerName: " + bannerName);
      //			  myLogger.Info("heTitle: " + heTitle);
      //		  }
      HTMLTable myTable;
      HTMLTableRow myRow;
      HTMLTableCell myCell;
      myTable = new HTMLTable();
      XPStringBuilder sb = new XPStringBuilder(200);
      sb.Append("begin ");
      sb.Append(bannerName);
      sb.Append(" banner");
      myTable.SetPreComment(sb.ToString());
      sb = new XPStringBuilder(200);
      sb.Append("end ");
      sb.Append(bannerName);
      sb.Append(" banner");
      myTable.SetPostComment(sb.ToString());
      myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
      myTable.SetHeight(sCellHeight);
      myRow = new HTMLTableRow();
      XPArrayList alButtonDictionaries = null;
      XPArrayList alButtonImgAltMaps = new XPArrayList();
      //   if we were passed an img to alt-id map, then we add it
      //   to our collection of img to alt-id maps.
      if (imgNameToAltId != null)
      {
        alButtonImgAltMaps.Add(imgNameToAltId);
      }
      try
      {
        myRow.SetStyleClass(rowClass);
        myTable.SetCellPadding(CommonHTMLStrings.ZERO);
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);
        myTable.SetBorder(CommonHTMLStrings.ZERO);
        myTable.SetStyleClass(tableClass);
        myTable.AddInnerHTMLElement(myRow);
        if (ePreTitle != null)
        {
          //	pre title element
          myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          myCell.SetStyleClass(titleCellClass);
          myCell.SetHeight(sCellHeight);
          myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
          myCell.SetWidth("3%");
          myCell.AddInnerHTMLElement(ePreTitle);
        } else
        {
          myCell = new HTMLTableCell();
          myRow.AddInnerHTMLElement(myCell);
          myCell.SetStyleClass(titleCellClass);
          myCell.SetHeight(sCellHeight);
          myCell.SetWidth("5");
        }
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetStyleClass(titleCellClass);
        myCell.SetHeight(sCellHeight);
        myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
        if (bSetTitleElementName)
        {
          myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_TITLE,
           bannerName));
        }
        myCell.AddInnerHTMLElement(heTitle);
        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);
        myCell.SetStyleClass(buttonCellClass);
        myCell.SetAlign(CommonHTMLStrings.RIGHT);
        myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
        myCell.SetNoWrap(true);
        myCell.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_EXTENDABLE_CELL,
         bannerName));
        /*
         myCell = new HTMLTableCell();
         myRow.AddInnerHTMLElement(myCell);
         
         myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
         */
         
        myRow = new HTMLTableRow();
        myTable.AddInnerHTMLElement(myRow);
        myCell = new HTMLTableCell();
        myCell.SetAutoFit(true);
        myRow.AddInnerHTMLElement(myCell);
      }
      catch (HTMLException he)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error("Couldn't add elements to cell. he: " + he);
      //			  }
      XPStringDictionary sdTemp;
      if (alButtonDictionaries != null)
      {
        sdTemp = FactoryHelper.combineStringDictionaries(alButtonDictionaries.GetEnumerator());
        } else
      {
        sdTemp = new XPStringDictionary();
      }
      //   Gadget banner title should not be updated by the factory
      sdTemp.PutElement(GetElementNameAsString(ELEMENT_NAME_ID_TITLE, bannerName),
       PTConstructType.DO_NOT_UPDATE);
      xpmNameToStringDictionary.PutElement(bannerName, sdTemp);
      if (alButtonImgAltMaps != null)
      {
        sdTemp = FactoryHelper.combineStringDictionaries(alButtonImgAltMaps.GetEnumerator());
        XPStringDictionary sdExisting = null;
        sdExisting = (XPStringDictionary) xpmNameToImgToAltIdMap.GetElement(bannerName);
        //   if there's already a string dictionary stored here, then
        //   combine it with the one we just created.
        if (sdExisting != null)
        {
          sdTemp = FactoryHelper.combineStringDictionaries(sdTemp, sdExisting);
          }
        //			  if (bDebug)
        //			  {
        //				  myLogger.Debug("Img alt map sdTemp: " + sdTemp);
        //			  }
        xpmNameToImgToAltIdMap.PutElement(bannerName, sdTemp);
      }
      xpmNameToTableElement.PutElement(bannerName, myTable);
      return myTable;
    }
    
    /// <param name="bannerName"></param>
    /// <param name="heTitle"></param>
    /// <param name="tableClass"></param>
    /// <param name="cellClass"></param>
    /// <param name="sCellHeight"></param>
    /// <param name="bSetTitleElementName"></param>
    private static void GetEmptyBanner(String bannerName, HTMLElement heTitle,
     PTStyleClass tableClass, PTStyleClass cellClass, String sCellHeight,
     bool bSetTitleElementName)
    {
      //		  if (bInfo)
      //		  {
      //			  myLogger.Info("Entering GetEmptyBanner()");
      //			  myLogger.Info("bannerName: " + bannerName);
      //			  myLogger.Info("heTitle: " + heTitle);
      //		  }
      GetButtonBanner(bannerName, heTitle, tableClass, null, cellClass, cellClass,
       sCellHeight, bSetTitleElementName);
    }
    
    /// <returns></returns>
    public static IXPEnumerator GetBannerTypeNames()
    {
      return xpmNameToTableElement.GetKeys();
    }
    
    /// <summary>
    /// Call this method to obtain an enumeration of all available PTBannerType
    /// objects.
    /// </summary>
    public static IXPEnumerator GetTypes()
    {
      return alTypes.GetEnumerator();
    }
    
    /// <returns></returns>
    public virtual ElementName GetElementName()
    {
      return eName;
    }
    public override HTMLElement GetHTMLElement()
    {
      if (myTable == null)
      {
        //			  if (bError)
        //			  {
        //				  myLogger.Error("No HTMLElement available for this type. this: " + this);
        //			  }
        throw new XPException("This PTBannerType was not initialized properly.");
        }
      return (HTMLElement) myTable;
    }
    
    /// <summary>
    /// Call this method to obtain the string dictionary for this banner
    /// type.
    /// <br /><br />
    /// A construct's string dictionary contains, as keys, the name's of
    /// those html elements that must be updated with localized strings.
    /// The values corresponding to these keys are the id's used to obtain
    /// the corresponding localized string.  For example, if a banner's
    /// string dictionary contained one pair of strings:
    /// <br /><br />
    /// &nbsp;&nbsp;&nbsp;"title" - "titleID"
    /// <br /><br />
    /// then, the element named "title" would need to be updated with the
    /// localized string retrieved with the id "titleID".
    /// <br /><br />
    /// see PTBannerFactory.GetTableBanner().
    /// </summary>
    public override XPHashtable GetStringMap()
    {
      return myStringMap;
    }
    public override XPHashtable GetImgElementNameToAltIdMap()
    {
      return xpmImgToAltId;
    }
    
    /// <param name="sElementNameID"></param>
    /// <param name="sBannerName"></param>
    /// <returns></returns>
    private static String GetElementNameAsString(String sElementNameID, String sBannerName)
    {
      String sReturn = sElementNameID + "_" + sBannerName;
      return sReturn;
    }
    public override ElementName GetElementName(String sElementNameID)
    {
      ElementName eReturn = null;
      //		  if (bInfo)
      //		  {
      //			  myLogger.Info("Entering GetElementName( String )");
      //		  }
      try
      {
        eReturn = (ElementName) xpmElementNames.GetElement(sElementNameID);
        }
      catch (Exception e)
      {
      }
      // do nothing
      //			  if (bError)
      //			  {
      //				  myLogger.Error(sElementNameID + " is not a valid element name for this construct type.");
      //			  }
      if (eReturn == null)
      {
      }
      //			  if (bError)
      //			  {
      //				  myLogger.Error(sElementNameID + " is not a valid element name for this construct type.");
      //			  }
      return eReturn;
    }
    
    /// <param name="collection"></param>
    /// <param name="sBannerName"></param>
    private static void AddCollapseExpandSpanToCollection(HTMLElementCollection collection,
     String sBannerName)
    {
      HTMLSpan mySpan;
      mySpan = new HTMLSpan();
      try
      {
        collection.AddInnerHTMLElement(mySpan);
        mySpan.SetElementName(PTBannerType.GetElementNameAsString(PTBannerType.ELEMENT_NAME_ID_EXPAND_COLLAPSE_CONTAINER,
         sBannerName));
        mySpan.SetStyleClass(PTStyleClass.TIPS_TEXT);
      }
      catch (HTMLException he)
      {
      }
    }
    // do nothing
    //			  if (bError)
    //			  {
    //				  myLogger.Error(he);
    //			  }
    
    /// <returns></returns>
    public override String ToString()
    {
      return XPStringUtility.ToUpperCase(sName);
    }
  }
}