using System;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.xpshared.htmlconstructs
{
  /// <summary>
  /// An object of this class can be used to contruct the familiar
  /// breadcrumb path. <br />
  /// <br />
  /// The breadcrumb path construct contains an ordered list of text
  /// and/or links, separated by a separator character.
  /// </summary>
  /// <author>beng</author>
  public class PTBreadcrumbPath
  {
    
    /// <summary>
    /// The separator should use breaking spaces so that the breadcrumb can wrap between folders. 
    /// </summary>
    public String sSeparator = " : ";
    private XPArrayList alElements;
    
    /// <summary>
    /// Creates a new PTBreadcrumbPath object.
    /// </summary>
    public PTBreadcrumbPath()
    {
      alElements = new XPArrayList();
    }
    
    /// <summary>
    /// Call this method to obtain an HTMLElement that describes the state of
    /// this breadcrumb path.
    /// </summary>
    /// <returns></returns>
    /// <throws>Exception</throws>
    public virtual HTMLElement GetPathAsHTMLElement()
    {
      IXPEnumerator enumElements = alElements.GetEnumerator();
      HTMLSpan mySpan = new HTMLSpan();
      HTMLElement eSeparator = new HTMLGenericElement(sSeparator);
      mySpan.SetStyleClass(PTStyleClass.BREADCRUMB_TEXT);
      HTMLElement e;
      int iNumElements = alElements.GetSize();
      int i = 1;
      while (enumElements.MoveNext())
      {
        e = (HTMLElement) enumElements.GetCurrent();
        mySpan.AddInnerHTMLElement(e);
        if (i < iNumElements)
        {
          mySpan.AddInnerHTMLElement(eSeparator);
        }
        i++;
      }
      return mySpan;
    }
    
    /// <summary>
    /// Call this method to append an HTMLAnchor to the end of this path.
    /// </summary>
    /// <param name="a"></param>
    public virtual void appendAnchorItem(HTMLAnchor a)
    {
      alElements.Add(a);
    }
    
    /// <summary>
    /// Call this method to append a regular string to the end of this path.
    /// This string should be plain text, and contain no html.
    /// </summary>
    /// <param name="s"></param>
    public virtual void appendTextItem(String s)
    {
      HTMLGenericElement e = new HTMLGenericElement(s);
      alElements.Add(e);
    }
  }
}