using System;
//import com.plumtree.xpfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.xpshared.htmlconstructs
{
  /// <author>
  /// BenG  The static members of this class provide PTButtonType objects
  /// that may be used to obtain constructs from the PTConstructFactory.
  /// These constructs describe buttons commonly appearing in the portal
  /// ui. <br />
  /// <br />
  /// A button is typically a span containing an anchor containing an img
  /// and possibly a title.  Buttons can also be input elements.
  /// </author>
  public class PTButtonType : PTConstructType
  {
    //
    //    private static XPLogger myLogger = XPLogManager.GetLogger(PTButtonType.class);
    //
    //
    //    private static boolean bDebug = myLogger.IsDebugEnabled();
    //
    //
    //    private static boolean bError = myLogger.IsErrorEnabled();
    //
    //
    //    private static boolean bWarn = myLogger.IsWarnEnabled();
    //
    //
    //    private static boolean bInfo = myLogger.IsInfoEnabled();
    //
    //
    //    private static boolean bFatal = myLogger.IsFatalEnabled();
    private static XPArrayList alTypes = new XPArrayList();
    private static XPHashtable xpmNameToElement = new XPHashtable();
    private static XPHashtable xpmNameToDescription = new XPHashtable();
    private static XPHashtable xpmNameToStringDictionary = new XPHashtable();
    private static XPHashtable xpmNameToImgToAltId = new XPHashtable();
    
    /// <summary>
    /// Use this constant to obtain the ElementName indicating this button's img
    /// element.
    /// </summary>
    public const String ELEMENT_NAME_ID_IMG = "button_img_enid";
    
    /// <summary>
    /// Use this constant to obtain the ElementName indicating this button's img
    /// element.
    /// </summary>
    public const String ELEMENT_NAME_ID_IMG_2 = "button_img2_enid";
    
    /// <summary>
    /// Use this constant to obtain the ElementName indicating this button's
    /// title element.
    /// </summary>
    public const String ELEMENT_NAME_ID_TITLE = "button_title_enid";
    
    /// <summary>
    /// Use this constant to obtain the ElementName indicating this button's
    /// anchor element.  Not all button's have anchors, some have inputs,  so
    /// it's possible that this element name does not indicate a valid  html
    /// element.
    /// </summary>
    public const String ELEMENT_NAME_ID_ANCHOR = "button_anchor_enid";
    private static bool bInitCalled = false;
    private const int iUpAltId = 1;
    private const int iDownAltId = 1;
    private const int iTopAltId = 1;
    private const int iBottomAltId = 1;
    private const int iHelpAltId = 1;
    private const int iApproveAltId = 1;
    private const int iRejectAltId = 1;
    private const int iHoldAltId = 1;
    private const int iAddMeAltId = 1;
    
    /// <summary>
    /// Abort Job button 
    /// </summary>
    private const int iAbortAltId = 1;
    private const int iClearTitleId = 1;
    private const int iFinishAltId = 1;
    private const int iMigrateObjectsAltId = 1;
    private const int iEnableAltId = 1;
    private const int iDisableAltId = 1;
    private const int iCreatePortletAltId = 1;
    private const int iApproveAllAltId = 1;
    private const int iRejectAllAltId = 1;
    private const int iHoldAllAltId = 1;
    private const int iApplyAltId = 1;
    private const int iLinkSettingsAltId = 1;
    private const int iSecurityAltId = 1;
    private const int iBulkAddAltId = 1;
    private const int iPreferencesAltId = 1;
    private const int iMoveAltId = 1;
    private const int iCopyAltId = 1;
    private const int iRenameAltId = 1;
    private const int iCreateFolderAltId = 1;
    private const int iAddUrlAltId = 1;
    private const int iLoginAltId = 0;
    private const int iCreateAccountAltId = 1;
    private const int iSearchAltId = 1;
    private const int iFilterAltId = 1;
    private const int iGoUpAltId = 1;
    private const int iSubmitAltId = 1;
    private const int iBrowseAltId = 1;
    private const int iNewPageAltId = 1;
    private const int iNewSubcommunityAltId = 1;
    private const int iNewPageTemplateAltId = 1;
    private const int iNewBannerAltId = 1;
    private const int iNewFooterAltId = 1;
    private const int iPreviewAltId = 1;
    private const int iSelectCommunityTemplateAltId = 1;
    private const int iRefreshAltId = 1;
    private const int iNewCommunityTemplateAltId = 1;
    private const int iAddPortletAltId = 1;
    private const int iSelectPageTemplateAltId = 1;
    private const int iSelectLayoutAltId = 1;
    private const int iRenamePageAltId = 1;
    private const int iAddMembersAltId = 1;
    private const int iCreateCustomGroupAltId = 1;
    private const int iSendToOthersAltId = 1;
    private const int iCloseAltId = 1;
    private const int iClearAltId = 1;
    private const int iDeleteAltId = 1;
    private const int iCollapseAltId = 1;
    private const int iExpandAltId = 1;
    private const int iAddGroupAltId = 1;
    private const int iEditAltId = 1;
    private const int iUnlockAltId = 1;
    private const int iCreateAltId = 1;
    private const int iSelectAltId = 1;
    private const int iUnlockTitleId = 1;
    private const int iSendToOthersTitleId = 1;
    private const int iCreateAccountTitleId = 1;
    private const int iLoginTitleId = 0;
    private const int iHelpTitleId = 1;
    private const int iApproveTitleId = 1;
    private const int iRejectTitleId = 1;
    private const int iHoldTitleId = 1;
    private const int iApproveAllTitleId = 1;
    private const int iRejectAllTitleId = 1;
    private const int iHoldAllTitleId = 1;
    private const int iFinishTitleId = 1;
    private const int iEnableTitleId = 1;
    private const int iDisableTitleId = 1;
    private const int iApplyTitleId = 1;
    
    /// <summary>
    /// Use this PTButtonType to obtain a go_up button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType HELP = new PTButtonType("help");
    
    /// <summary>
    /// Use this PTButtonType to obtain a go_up button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SEARCH = new PTButtonType("search");
    
    /// <summary>
    /// Use this PTButtonType to obtain a go_up button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType CLEAR = new PTButtonType("clear");
    
    /// <summary>
    /// Use this PTButtonType to obtain a go_up button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType CLEAR_DIR_ACTIONBAR = new PTButtonType("clear_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a go_up button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType CLOSE = new PTButtonType("close");
    
    /// <summary>
    /// Use this PTButtonType to obtain a go_up button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType GO_UP = new PTButtonType("go_up");
    
    /// <summary>
    /// Use this PTButtonType to obtain a collapse area button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType COLLAPSE_AREA = new PTButtonType("collapse_area");
    
    /// <summary>
    /// Use this PTButtonType to obtain a collapse area button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType COLLAPSE_TIPS = new PTButtonType("collapse_tips");
    
    /// <summary>
    /// Use this PTButtonType to obtain an expand area button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType EXPAND_AREA = new PTButtonType("expand_area");
    
    /// <summary>
    /// Use this PTButtonType to obtain an expand area button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType EXPAND_TIPS = new PTButtonType("expand_tips");
    
    /// <summary>
    /// Use this PTButtonType to obtain a finish button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType FINISH = new PTButtonType("finish");
    
    /// <summary>
    /// Use this PTButtonType to obtain an edit button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType EDIT = new PTButtonType("edit");
    
    /// <summary>
    /// Use this PTButtonType to obtain an edit button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType EDIT_ACTIONBAR = new PTButtonType("edit_actionbar");
    public static readonly PTButtonType ADD_ME_ACTIONBAR = new PTButtonType("add_me_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain an edit button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType EDIT_DIR_ACTIONBAR = new PTButtonType("edit_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain an edit button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType EDIT_FOLDER_DIR_ACTIONBAR = new PTButtonType("edit_folder_dir_actionbar");
    public static readonly PTButtonType CONTENT_SNAPSHOT_HEADER = new PTButtonType("content_snapshot_header");
    
    /// <summary>
    /// Use this PTButtonType to obtain an edit button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType EDIT_OBJ_ACTIONBAR = new PTButtonType("edit_obj_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain an edit button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType EDIT_HEADER = new PTButtonType("edit_header");
    
    /// <summary>
    /// Use this PTButtonType to obtain a create button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType CREATE = new PTButtonType("create");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType CREATE_ACTIONBAR = new PTButtonType("create_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType CREATE_DIR_ACTIONBAR = new PTButtonType("create_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType CREATE_OBJ_ACTIONBAR = new PTButtonType("create_obj_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a select button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SELECT = new PTButtonType("select");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType ADD_GROUP = new PTButtonType("add_group");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType DELETE = new PTButtonType("delete");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType NEW_PAGE = new PTButtonType("new_page");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType NEW_SUBCOMMUNITY = new PTButtonType("new_subcommunity");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType NEW_COMMUNITY_TEMPLATE = new PTButtonType("new_community_template");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType NEW_PAGE_TEMPLATE = new PTButtonType("new_page_template");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType NEW_BANNER = new PTButtonType("new_banner");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType NEW_FOOTER = new PTButtonType("new_footer");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType PREVIEW = new PTButtonType("preview");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SELECT_COMMUNITY_TEMPLATE = new PTButtonType("select_community_template");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SELECT_PAGE_TEMPLATE = new PTButtonType("select_page_template");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType ADD_PORTLET = new PTButtonType("add_portlet");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType CREATE_PORTLET = new PTButtonType("create_portlet");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SELECT_LAYOUT = new PTButtonType("select_layout");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType RENAME_PAGE = new PTButtonType("rename_page");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType ADD_MEMBERS = new PTButtonType("add_members");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType CREATE_CUSTOM_GROUP = new PTButtonType("create_custom_group");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SEND_TO_OTHERS = new PTButtonType("send_to_others");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SEND_TO_OTHERS_BANNER_ACTIONBAR = new PTButtonType("send_to_others_banner_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType REFRESH = new PTButtonType("refresh");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType FILTER = new PTButtonType("filter");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType LOGIN = new PTButtonType("login");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType ADD_URL = new PTButtonType("addurl");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SELECT_ACTIONBAR = new PTButtonType("select_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SELECT_DIR_ACTIONBAR = new PTButtonType("select_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SELECT_ALL_DIR_ACTIONBAR = new PTButtonType("select_all_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType UNSELECT_ALL_DIR_ACTIONBAR = new PTButtonType("unselect_all_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SELECT_OBJ_ACTIONBAR = new PTButtonType("select_obj_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType REFRESH_DIR_ACTIONBAR = new PTButtonType("refresh_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType REFRESH_OBJ_ACTIONBAR = new PTButtonType("refresh_obj_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType DELETE_DIR_ACTIONBAR = new PTButtonType("delete_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType DELETE_OBJ_ACTIONBAR = new PTButtonType("delete_obj_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SUBMIT_HEADER = new PTButtonType("submit_header");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SUBMIT_DIR_ACTIONBAR = new PTButtonType("submit_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType BROWSE_HEADER = new PTButtonType("browse_header");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType BROWSE = new PTButtonType("browse");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType CREATE_FOLDER_DIR_ACTIONBAR = new PTButtonType("create_folder_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType MOVE_DIR_ACTIONBAR = new PTButtonType("move_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType COPY_DIR_ACTIONBAR = new PTButtonType("copy_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType RENAME_DIR_ACTIONBAR = new PTButtonType("rename_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SECURITY_DIR_ACTIONBAR = new PTButtonType("security_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType BULK_ADD_DIR_ACTIONBAR = new PTButtonType("bulk_add_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType PREFERENCES_DIR_ACTIONBAR = new PTButtonType("preferences_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType APPROVE_OBJECT_ACTIONBAR = new PTButtonType("approve_object_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType UNAPPROVE_OBJECT_ACTIONBAR = new PTButtonType("unapprove_object_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType LINK_SETTINGS_OBJECT_ACTIONBAR = new PTButtonType("link_settings_object_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType APPROVE_ALL_DIR_ACTIONBAR = new PTButtonType("approve_all_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType REJECT_ALL_DIR_ACTIONBAR = new PTButtonType("reject_all_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType HOLD_ALL_DIR_ACTIONBAR = new PTButtonType("hold_all_dir_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType APPROVE = new PTButtonType("approve");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType REJECT = new PTButtonType("reject");
    
    /// <summary>
    /// Use this PTButtonType to obtain an Abort Job button from the Construct
    /// factory
    /// </summary>
    public static readonly PTButtonType ABORT_JOB = new PTButtonType("abort_job");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType HOLD = new PTButtonType("hold");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType APPLY_OBJECT_ACTIONBAR = new PTButtonType("apply_object_actionbar");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType UP = new PTButtonType("up");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType DOWN = new PTButtonType("down");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SORT_UP = new PTButtonType("sort_up");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType SORT_DOWN = new PTButtonType("sort_down");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType TOP = new PTButtonType("top");
    
    /// <summary>
    /// Use this PTButtonType to obtain a add group button from the
    /// ConstructFactory.
    /// </summary>
    public static readonly PTButtonType BOTTOM = new PTButtonType("bottom");
    public static readonly PTButtonType FINISH_DIR_ACTIONBAR = new PTButtonType("finish_dir_actionbar");
    public static readonly PTButtonType UNLOCK_OBJ_ACTIONBAR = new PTButtonType("unlock_obj_actionbar");
    public static readonly PTButtonType CREATE_PORTLET_DIR_ACTIONBAR = new PTButtonType("create_portlet_dir_actionbar");
    public static readonly PTButtonType CREATE_ACCOUNT = new PTButtonType("create_account");
    public static readonly PTButtonType MIGRATE_OBJECTS_DIR_ACTIONBAR = new PTButtonType("migrate_objects_dir_actionbar");
    public static readonly PTButtonType ENABLE_DIR_ACTIONBAR = new PTButtonType("enable_dir_actionbar");
    public static readonly PTButtonType DISABLE_DIR_ACTIONBAR = new PTButtonType("disable_dir_actionbar");
    static PTButtonType()
    {
      alTypes.Add(PTButtonType.UP);
      alTypes.Add(PTButtonType.DOWN);
      alTypes.Add(PTButtonType.SORT_UP);
      alTypes.Add(PTButtonType.SORT_DOWN);
      alTypes.Add(PTButtonType.TOP);
      alTypes.Add(PTButtonType.BOTTOM);
      alTypes.Add(PTButtonType.HELP);
      alTypes.Add(PTButtonType.ADD_ME_ACTIONBAR);
      alTypes.Add(PTButtonType.SUBMIT_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.SUBMIT_HEADER);
      alTypes.Add(PTButtonType.BROWSE);
      alTypes.Add(PTButtonType.BROWSE_HEADER);
      alTypes.Add(PTButtonType.CONTENT_SNAPSHOT_HEADER);
      alTypes.Add(PTButtonType.SEARCH);
      alTypes.Add(PTButtonType.GO_UP);
      alTypes.Add(PTButtonType.COLLAPSE_AREA);
      alTypes.Add(PTButtonType.EXPAND_AREA);
      alTypes.Add(PTButtonType.FINISH);
      alTypes.Add(PTButtonType.FINISH_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.EDIT);
      alTypes.Add(PTButtonType.EDIT_ACTIONBAR);
      alTypes.Add(PTButtonType.EDIT_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.EDIT_OBJ_ACTIONBAR);
      alTypes.Add(PTButtonType.EDIT_HEADER);
      alTypes.Add(PTButtonType.APPROVE_ALL_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.REJECT_ALL_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.HOLD_ALL_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.APPROVE);
      alTypes.Add(PTButtonType.REJECT);
      alTypes.Add(PTButtonType.HOLD);
      alTypes.Add(PTButtonType.APPLY_OBJECT_ACTIONBAR);
      alTypes.Add(PTButtonType.CREATE);
      alTypes.Add(PTButtonType.CREATE_ACTIONBAR);
      alTypes.Add(PTButtonType.CREATE_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.CREATE_OBJ_ACTIONBAR);
      alTypes.Add(PTButtonType.CREATE_FOLDER_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.SECURITY_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.BULK_ADD_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.PREFERENCES_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.MOVE_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.COPY_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.RENAME_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.APPROVE_OBJECT_ACTIONBAR);
      alTypes.Add(PTButtonType.UNAPPROVE_OBJECT_ACTIONBAR);
      alTypes.Add(PTButtonType.LINK_SETTINGS_OBJECT_ACTIONBAR);
      alTypes.Add(PTButtonType.SELECT);
      alTypes.Add(PTButtonType.SELECT_ACTIONBAR);
      alTypes.Add(PTButtonType.SELECT_ALL_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.UNSELECT_ALL_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.SELECT_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.SELECT_OBJ_ACTIONBAR);
      alTypes.Add(PTButtonType.ADD_GROUP);
      alTypes.Add(PTButtonType.CLOSE);
      alTypes.Add(PTButtonType.DELETE);
      alTypes.Add(PTButtonType.DELETE_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.DELETE_OBJ_ACTIONBAR);
      alTypes.Add(PTButtonType.CLEAR);
      alTypes.Add(PTButtonType.CLEAR_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.NEW_PAGE);
      alTypes.Add(PTButtonType.NEW_SUBCOMMUNITY);
      alTypes.Add(PTButtonType.NEW_COMMUNITY_TEMPLATE);
      alTypes.Add(PTButtonType.NEW_PAGE_TEMPLATE);
      alTypes.Add(PTButtonType.NEW_BANNER);
      alTypes.Add(PTButtonType.NEW_FOOTER);
      alTypes.Add(PTButtonType.PREVIEW);
      alTypes.Add(PTButtonType.SELECT_COMMUNITY_TEMPLATE);
      alTypes.Add(PTButtonType.SELECT_PAGE_TEMPLATE);
      alTypes.Add(PTButtonType.ADD_PORTLET);
      alTypes.Add(PTButtonType.SELECT_LAYOUT);
      alTypes.Add(PTButtonType.RENAME_PAGE);
      alTypes.Add(PTButtonType.ADD_MEMBERS);
      alTypes.Add(PTButtonType.CREATE_CUSTOM_GROUP);
      alTypes.Add(PTButtonType.SEND_TO_OTHERS);
      alTypes.Add(PTButtonType.SEND_TO_OTHERS_BANNER_ACTIONBAR);
      alTypes.Add(PTButtonType.REFRESH);
      alTypes.Add(PTButtonType.REFRESH_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.REFRESH_OBJ_ACTIONBAR);
      alTypes.Add(PTButtonType.FILTER);
      alTypes.Add(PTButtonType.LOGIN);
      alTypes.Add(PTButtonType.CREATE_ACCOUNT);
      alTypes.Add(PTButtonType.ADD_URL);
      alTypes.Add(PTButtonType.UNLOCK_OBJ_ACTIONBAR);
      alTypes.Add(PTButtonType.CREATE_PORTLET);
      alTypes.Add(PTButtonType.CREATE_PORTLET_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.MIGRATE_OBJECTS_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.ENABLE_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.DISABLE_DIR_ACTIONBAR);
      alTypes.Add(PTButtonType.ABORT_JOB);
    }
    public ElementName ANCHOR_ELEMENT_NAME = null;
    public ElementName IMG_ELEMENT_NAME = null;
    public ElementName IMG_2_ELEMENT_NAME = null;
    public ElementName TITLE_ELEMENT_NAME = null;
    internal ElementName eName;
    internal HTMLElement myElement;
    internal String sDescription;
    internal String sName;
    internal XPHashtable myStringMap;
    internal XPHashtable xpmImgToAltId;
    private XPHashtable xpmElementNames = new XPHashtable();
    
    /// <summary>
    /// Creates a new PTButtonType object.
    /// </summary>
    /// <param name="s">Pass the name of the button type.</param>
    private PTButtonType(String s)
    {
      PTButtonType.staticInit();
      sName = s;
      myElement = (HTMLElement) xpmNameToElement.GetElement(s);
      ElementName enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_TITLE,
       s));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_TITLE, enTemp);
      TITLE_ELEMENT_NAME = enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_IMG,
       s));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_IMG, enTemp);
      IMG_ELEMENT_NAME = enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_IMG_2,
       s));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_IMG_2, enTemp);
      IMG_2_ELEMENT_NAME = enTemp;
      enTemp = new ElementName(GetElementNameAsString(ELEMENT_NAME_ID_ANCHOR,
       s));
      xpmElementNames.PutElement(ELEMENT_NAME_ID_ANCHOR, enTemp);
      ANCHOR_ELEMENT_NAME = enTemp;
      try
      {
        myStringMap = (XPHashtable) xpmNameToStringDictionary.GetElement(s);
        }
      catch (Exception e)
      {
        myStringMap = null;
      }
      try
      {
        //            sdImgToAltId = (XPStringDictionary) xpmNameToImgToAltId.GetElement(s);
        xpmImgToAltId = (XPHashtable) xpmNameToImgToAltId.GetElement(s);
      }
      catch (Exception e)
      {
        xpmImgToAltId = null;
      }
      try
      {
        sDescription = (String) xpmNameToDescription.GetElement(s);
      }
      catch (Exception e)
      {
      }
      // do nothing
      if (sDescription == null)
      {
        sDescription = s + " button description has not been specified.";
        }
    }
    //        if (bDebug)
    //        {
    //            myLogger.Debug("Entering PTButtonType() - s: " + s);
    //            myLogger.Debug("myElement: " + myElement);
    //            myLogger.Debug("myElement.GetElementName(): " + myElement.GetElementName());
    //        }
    
    /// <returns></returns>
    public static IXPEnumerator GetButtonTypeNames()
    {
      return xpmNameToElement.GetKeys();
    }
    
    /// <returns></returns>
    public override String GetDescription()
    {
      return sDescription;
    }
    
    /// <param name="sElementNameID"></param>
    /// <returns></returns>
    /// <throws>Exception</throws>
    public override ElementName GetElementName(String sElementNameID)
    {
      ElementName eReturn = null;
      try
      {
        eReturn = (ElementName) xpmElementNames.GetElement(sElementNameID);
        }
      catch (Exception e)
      {
      }
      // do nothing
      //            if (bError)
      //            {
      //                myLogger.Error(sElementNameID + " is not a valid element name for this construct type.");
      //            }
      return eReturn;
    }
    
    /// <summary>
    /// Call this method to obtain the HTMLElement prototype of this button. You
    /// should never use this HTMLElement in an HTML page.  This element should
    /// be cloned before use.  In general, use PTContstructFactory to obtain
    /// usable constructs.
    /// </summary>
    /// <returns>HTMLElement prototype</returns>
    public override HTMLElement GetHTMLElement()
    {
      return myElement;
    }
    //   package-private
    
    /// <returns></returns>
    public override XPHashtable GetStringMap()
    {
      return myStringMap;
    }
    
    /// <returns></returns>
    public static IXPEnumerator GetTypes()
    {
      return alTypes.GetEnumerator();
    }
    //   package-private
    
    /// <returns></returns>
    public override XPHashtable GetImgElementNameToAltIdMap()
    {
      return xpmImgToAltId;
    }
    
    /// <returns></returns>
    public override String ToString()
    {
      return XPStringUtility.ToUpperCase(sName);
    }
    
    /// <param name="sElementNameID"></param>
    /// <param name="sButtonName"></param>
    /// <returns></returns>
    private static String GetElementNameAsString(String sElementNameID, String sButtonName)
    {
      String sReturn = sElementNameID + "_" + sButtonName;
      return sReturn;
    }
    
    /// <param name="sButtonName"></param>
    /// <param name="outterSpanStyle"></param>
    /// <param name="titleSpanStyle"></param>
    /// <param name="image1"></param>
    /// <param name="image2"></param>
    /// <param name="iAltId"></param>
    /// <param name="bDisplayImageFirst"></param>
    private static void MakeSimpleButton(String sButtonName, PTStyleClass outterSpanStyle,
     PTStyleClass titleSpanStyle, PTImageType image1, PTImageType image2,
     int iAltId, bool bDisplayImageFirst)
    {
      MakeSimpleButton(sButtonName, -1, outterSpanStyle, titleSpanStyle, image1,
       image2, iAltId, bDisplayImageFirst);
    }
    
    /// <param name="sButtonName"></param>
    /// <param name="iTitle"></param>
    /// <param name="outterSpanStyle"></param>
    /// <param name="titleSpanStyle"></param>
    /// <param name="image1"></param>
    /// <param name="image2"></param>
    /// <param name="iAltId"></param>
    /// <param name="bDisplayImageFirst"></param>
    private static void MakeSimpleButton(String sButtonName, int iTitle, PTStyleClass outterSpanStyle,
     PTStyleClass titleSpanStyle, PTImageType image1, PTImageType image2,
     int iAltId, bool bDisplayImageFirst)
    {
      HTMLSpan mySpan;
      HTMLAnchor myAnchor;
      HTMLImg myImg;
      HTMLImg mySpacerImg = null;
      PTConstruct myConstruct;
      XPHashtable xpmMyImgToAltId = null;
      try
      {
        mySpan = new HTMLSpan();
        xpmNameToElement.PutElement(sButtonName, mySpan);
        mySpan.SetPreComment("begin " + sButtonName + " button");
        mySpan.SetPostComment("end " + sButtonName + " button");
        mySpan.SetStyleClass(outterSpanStyle);
        myAnchor = new HTMLAnchor("#");
        myAnchor.SetElementName(GetElementNameAsString(ELEMENT_NAME_ID_ANCHOR,
         sButtonName));
        mySpan.AddInnerHTMLElement(myAnchor);
        try
        {
          myConstruct = ConstructFactory.GetConstruct(PTImageType.SPACER);
          mySpacerImg = (HTMLImg) myConstruct.GetHTMLElement();
        }
        catch (Exception ex)
        {
        }
        // do nothing
        if (!bDisplayImageFirst)
        {
          mySpan = new HTMLSpan();
          mySpan.AddInnerHTMLElement(mySpacerImg);
          if (titleSpanStyle != null)
          {
            mySpan.SetStyleClass(titleSpanStyle);
          }
          String sTitleElementName = GetElementNameAsString(PTButtonType.ELEMENT_NAME_ID_TITLE,
           sButtonName);
          mySpan.SetElementName(sTitleElementName);
          //	if the title index is -1, then we just add
          //	the span, but don't add anything to the title
          //	map.  This way, the client can set the title, but
          //	nothing will appear in the construct straight from the
          //	factory.
          if (iTitle != -1)
          {
            //	if the title index is not -1, then
            //	we add it to the map.  This string
            //	will then appear localized in the
            //	construct
            XPHashtable m = new XPHashtable();
            m.PutElement(sTitleElementName, iTitle);
            PTButtonType.xpmNameToStringDictionary.PutElement(sButtonName,
             m);
          }
          myAnchor.AddInnerHTMLElement(mySpan);
        }
        if (image1 != null)
        {
          myImg = null;
          try
          {
            myConstruct = ConstructFactory.GetConstruct(image1);
            myImg = (HTMLImg) myConstruct.GetHTMLElement();
          }
          catch (Exception e)
          {
          }
          // do nothing
          myImg.SetElementName(GetElementNameAsString(PTButtonType.ELEMENT_NAME_ID_IMG,
           sButtonName));
          myImg.SetAlign("absMiddle");
          myImg.SetAlt("ALT_PLACEHOLDER");
          xpmMyImgToAltId = new XPHashtable();
          xpmMyImgToAltId.PutElement(myImg.GetElementName(), iAltId);
          myAnchor.AddInnerHTMLElement(myImg);
          xpmNameToImgToAltId.PutElement(sButtonName, xpmMyImgToAltId);
        }
        // image1 != null
        if (image2 != null)
        {
          myImg = null;
          try
          {
            myConstruct = ConstructFactory.GetConstruct(image2);
            myImg = (HTMLImg) myConstruct.GetHTMLElement();
          }
          catch (Exception e)
          {
          }
          // do nothing
          myImg.SetElementName(GetElementNameAsString(PTButtonType.ELEMENT_NAME_ID_IMG_2,
           sButtonName));
          myImg.SetAlign("absMiddle");
          myImg.SetAlt("ALT_PLACEHOLDER");
          if (xpmMyImgToAltId == null)
          {
            xpmMyImgToAltId = new XPHashtable();
          }
          xpmMyImgToAltId.PutElement(myImg.GetElementName(), iAltId);
          myAnchor.AddInnerHTMLElement(myImg);
          xpmNameToImgToAltId.PutElement(sButtonName, xpmMyImgToAltId);
        }
        // image2 != null
        if (bDisplayImageFirst)
        {
          mySpan = new HTMLSpan();
          mySpan.AddInnerHTMLElement(mySpacerImg);
          if (titleSpanStyle != null)
          {
            mySpan.SetStyleClass(titleSpanStyle);
          }
          String sTitleElementName = GetElementNameAsString(PTButtonType.ELEMENT_NAME_ID_TITLE,
           sButtonName);
          mySpan.SetElementName(sTitleElementName);
          //	if the title index is -1, then we just add
          //	the span, but don't add anything to the title
          //	map.  This way, the client can set the title, but
          //	nothing will appear in the construct straight from the
          //	factory.
          if (iTitle != -1)
          {
            //	if the title index is not -1, then
            //	we add it to the map.  This string
            //	will then appear localized in the
            //	construct.
            XPHashtable m = new XPHashtable();
            m.PutElement(sTitleElementName, iTitle);
            PTButtonType.xpmNameToStringDictionary.PutElement(sButtonName,
             m);
          }
          myAnchor.AddInnerHTMLElement(mySpan);
          mySpan = new HTMLSpan();
          mySpan.AddInnerHTMLString(CommonHTMLStrings.SPACE);
          if (titleSpanStyle != null)
          {
            mySpan.SetStyleClass(titleSpanStyle);
          }
          myAnchor.AddInnerHTMLElement(mySpan);
        }
      }
      catch (HTMLException he)
      {
      }
    }
    // do nothing
    
    /// <summary>
    /// C# handles static blocks differently than java.  In C#, the static
    /// initializer is run after all private members are initialized.  For this
    /// reason, we must guarantee that the following static initialization code
    /// is run before the private constructor is called.
    /// </summary>
    /// <throws>Error</throws>
    private static void staticInit()
    {
      if (bInitCalled)
      {
        return;
      }
      bInitCalled = true;
      HTMLInput myInput = null;
      String sButtonName;
      //        if (bDebug)
      //        {
      //            myLogger.Debug("Entering static block.");
      //        }
      try
      {
        ////////////////////////////////////////////////////
        //   SORT_UP
        sButtonName = "sort_up";
        MakeSimpleButton(sButtonName, PTStyleClass.LIST_PAGINATION_TEXT, PTStyleClass.LIST_PAGINATION_TEXT,
         PTImageType.ICON_ARROW_UP, null, iUpAltId, false);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   SORT_DOWN
        sButtonName = "sort_down";
        MakeSimpleButton(sButtonName, PTStyleClass.LIST_PAGINATION_TEXT, PTStyleClass.LIST_PAGINATION_TEXT,
         PTImageType.ICON_ARROW_DOWN, null, iDownAltId, false);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   UP
        sButtonName = "up";
        MakeSimpleButton(sButtonName, PTStyleClass.LIST_PAGINATION_TEXT, PTStyleClass.LIST_PAGINATION_TEXT,
         PTImageType.ICON_ARROW_UP, null, iUpAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   DOWN
        sButtonName = "down";
        MakeSimpleButton(sButtonName, PTStyleClass.LIST_PAGINATION_TEXT, PTStyleClass.LIST_PAGINATION_TEXT,
         PTImageType.ICON_ARROW_DOWN, null, iDownAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   TOP
        sButtonName = "top";
        MakeSimpleButton(sButtonName, PTStyleClass.LIST_PAGINATION_TEXT, PTStyleClass.LIST_PAGINATION_TEXT,
         PTImageType.ICON_ARROW_TOP, null, iTopAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   BOTTOM
        sButtonName = "bottom";
        MakeSimpleButton(sButtonName, PTStyleClass.LIST_PAGINATION_TEXT, PTStyleClass.LIST_PAGINATION_TEXT,
         PTImageType.ICON_ARROW_BOTTOM, null, iBottomAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   HELP
        sButtonName = "help";
        MakeSimpleButton(sButtonName, iHelpTitleId, PTStyleClass.BAN_HELP_TEXT,
         PTStyleClass.BAN_HELP_TEXT, PTImageType.ICON_HELP, null, iHelpAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   APPROVE
        sButtonName = "approve";
        MakeSimpleButton(sButtonName, iApproveTitleId, PTStyleClass.ACTIONBAR_BAN_BTN,
         PTStyleClass.LIST_TEXT, PTImageType.ACTION_APPROVE, null, iApproveAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   REJECT
        sButtonName = "reject";
        MakeSimpleButton(sButtonName, iRejectTitleId, PTStyleClass.LIST_TEXT,
         PTStyleClass.LIST_TEXT, PTImageType.ACTION_ALL_REJECT, null, iRejectAltId,
         false);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   HOLD
        sButtonName = "hold";
        MakeSimpleButton(sButtonName, iHoldTitleId, PTStyleClass.LIST_TEXT,
         PTStyleClass.LIST_TEXT, PTImageType.ACTION_ALL_HOLD, null, iHoldAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   FINISH_DIR_ACTIONBAR
        sButtonName = "finish_dir_actionbar";
        MakeSimpleButton(sButtonName, iFinishTitleId, PTStyleClass.ACTIONBAR_DIR_BTN,
         PTStyleClass.ACTIONBAR_TEXT, PTImageType.ACTION_APPROVE_ALL, null,
         iFinishAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   MIGRATE_OBJECTS_DIR_ACTIONBAR
        sButtonName = "migrate_objects_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_OBJECT_MIGRATION, null, iMigrateObjectsAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   ENABLE_DIR_ACTIONBAR
        sButtonName = "enable_dir_actionbar";
        MakeSimpleButton(sButtonName, iEnableTitleId, PTStyleClass.ACTIONBAR_DIR_BTN,
         PTStyleClass.ACTIONBAR_TEXT, PTImageType.ACTION_ENABLE, null, iEnableAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   DISABLE_DIR_ACTIONBAR
        sButtonName = "disable_dir_actionbar";
        MakeSimpleButton(sButtonName, iDisableTitleId, PTStyleClass.ACTIONBAR_DIR_BTN,
         PTStyleClass.ACTIONBAR_TEXT, PTImageType.ACTION_DISABLE, null, iDisableAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   CREATE_PORTLET_DIR_ACTIONBAR
        sButtonName = "create_portlet_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_NEW, PTImageType.ICON_PORTLET, iCreatePortletAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   APPROVE_ALL_DIR_ACTIONBAR
        sButtonName = "approve_all_dir_actionbar";
        MakeSimpleButton(sButtonName, iApproveAllTitleId, PTStyleClass.ACTIONBAR_DIR_BTN,
         PTStyleClass.ACTIONBAR_TEXT, PTImageType.ACTION_APPROVE_ALL, null,
         iApproveAllAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   CLEAR_DIR_ACTIONBAR
        sButtonName = "clear_dir_actionbar";
        MakeSimpleButton(sButtonName, iClearTitleId, PTStyleClass.ACTIONBAR_DIR_BTN,
         PTStyleClass.ACTIONBAR_TEXT, PTImageType.ACTION_CANCEL, null, iClearAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   REJECT_ALL_DIR_ACTIONBAR
        sButtonName = "reject_all_dir_actionbar";
        MakeSimpleButton(sButtonName, iRejectAllTitleId, PTStyleClass.ACTIONBAR_DIR_BTN,
         PTStyleClass.ACTIONBAR_TEXT, PTImageType.ACTION_ALL_REJECT, null,
         iRejectAllAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   HOLD_ALL_DIR_ACTIONBAR
        sButtonName = "hold_all_dir_actionbar";
        MakeSimpleButton(sButtonName, iHoldAllTitleId, PTStyleClass.ACTIONBAR_DIR_BTN,
         PTStyleClass.ACTIONBAR_TEXT, PTImageType.ACTION_ALL_HOLD, null, iHoldAllAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   APPLY_OBJECT_ACTIONBAR
        sButtonName = "apply_object_actionbar";
        MakeSimpleButton(sButtonName, iApplyTitleId, PTStyleClass.ACTIONBAR_OBJECT_BTN,
         PTStyleClass.ACTIONBAR_TEXT, PTImageType.ACTION_APPLY, null, iApplyAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   APPROVE_OBJECT_ACTIONBAR
        sButtonName = "approve_object_actionbar";
        MakeSimpleButton(sButtonName, iApproveTitleId, PTStyleClass.ACTIONBAR_OBJECT_BTN,
         PTStyleClass.ACTIONBAR_TEXT, PTImageType.ACTION_APPROVE, null, iApproveAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   UNAPPROVE_OBJECT_ACTIONBAR
        sButtonName = "unapprove_object_actionbar";
        MakeSimpleButton(sButtonName, iRejectTitleId, PTStyleClass.ACTIONBAR_OBJECT_BTN,
         PTStyleClass.ACTIONBAR_TEXT, PTImageType.ACTION_UNAPPROVE, null,
         iRejectAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   LINK_SETTINGS_OBJECT_ACTIONBAR
        sButtonName = "link_settings_object_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_SETTING_LINK, null, iLinkSettingsAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   SECURITY_DIR_ACTIONBAR
        sButtonName = "security_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_SECURITY, null, iSecurityAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   BULK_ADD_DIR_ACTIONBAR
        sButtonName = "bulk_add_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_BULK_ADD, null, iBulkAddAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   PREFERENCES_DIR_ACTIONBAR
        sButtonName = "preferences_dir_actionbar";
        /*PTImageType.ACTION_PREFERENCE*/
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.TODO_REPLACE_ME, null, iPreferencesAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   MOVE_DIR_ACTIONBAR
        sButtonName = "move_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_MOVE, PTImageType.ICON_FOLDER, iMoveAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   COPY_DIR_ACTIONBAR
        sButtonName = "copy_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_COPY, null, iCopyAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   RENAME_DIR_ACTIONBAR
        sButtonName = "rename_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_RENAME, null, iRenameAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   CREATE_FOLDER_DIR_ACTIONBAR
        sButtonName = "create_folder_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_NEW, PTImageType.ICON_FOLDER, iCreateFolderAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   ADD_URL
        sButtonName = "addurl";
        /* PTImageType.ACTION_LINK_ADD */
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_BAN_BTN, PTStyleClass.ACTIONBAR_BAN_TEXT,
         PTImageType.ACTION_ADD, null, iAddUrlAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this button to let the user add an URL to a list of URLs.  See the <a taret=\"new\" href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/crawlers(9-9)/crawler7.htm\">spec</a>.");
        ////////////////////////////////////////////////////
        //   LOGIN
        sButtonName = "login";
        MakeSimpleButton(sButtonName, iLoginTitleId, PTStyleClass.LOGIN_TEXT,
         null, PTImageType.ICON_LOGIN, null, iLoginAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this button to let the user log in to the portal.");
        ////////////////////////////////////////////////////
        //   CREATE_ACCOUNT
        sButtonName = "create_account";
        /*PTImageType.LOGIN*/
        MakeSimpleButton(sButtonName, iCreateAccountTitleId, PTStyleClass.LOGIN_TEXT,
         null, PTImageType.TODO_REPLACE_ME, null, iCreateAccountAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this button to let the user create an account in the portal.");
        ////////////////////////////////////////////////////
        //   SEARCH
        sButtonName = "search";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECTSEARCH_TEXT, null,
         PTImageType.ACTION_SEARCH_BUTTON, null, iSearchAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this button to let the user issue a search.");
        ////////////////////////////////////////////////////
        //   FILTER
        sButtonName = "filter";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, null, PTImageType.ICON_FILTER,
         null, iFilterAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this button to...");
        ////////////////////////////////////////////////////
        //   REFRESH
        sButtonName = "refresh";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, null, PTImageType.ACTION_REFRESH,
         null, iRefreshAltId, true);
        ////////////////////////////////////////////////////
        //   REFRESH_DIR_ACTIONBAR
        sButtonName = "refresh_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_REFRESH, null, iRefreshAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this refresh button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/directoryActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   REFRESH_OBJ_ACTIONBAR
        sButtonName = "refresh_obj_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, null,
         PTImageType.ACTION_REFRESH, null, iRefreshAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this refresh button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/objectActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   GO_UP
        sButtonName = "go_up";
        MakeSimpleButton(sButtonName, PTStyleClass.DIR_BTN_TEXT, null, PTImageType.ACTION_FOLDER_UP,
         PTImageType.ICON_FOLDER, iGoUpAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this button to let the user navigate up one level in the directory.  This button should be added to FOLDER banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/fileCrawler/adminSubExpandedTemplate.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   SUBMIT_HEADER
        sButtonName = "submit_header";
        MakeSimpleButton(sButtonName, PTStyleClass.DIR_BTN, PTStyleClass.DIR_BTN_TEXT,
         PTImageType.ACTION_SUBMIT, null, iSubmitAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "This button should be added to FOLDER banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/fileCrawler/adminSubExpandedTemplate.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   SUBMIT_DIR_ACTIONBAR
        sButtonName = "submit_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_SUBMIT, null, iSubmitAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   BROWSE_HEADER
        sButtonName = "browse_header";
        MakeSimpleButton(sButtonName, PTStyleClass.DIR_BTN, PTStyleClass.DIR_BTN_TEXT,
         PTImageType.ACTION_BROWSE, null, iBrowseAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "This button should be added to FOLDER banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/fileCrawler/adminSubExpandedTemplate.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   BROWSE
        sButtonName = "browse";
        MakeSimpleButton(sButtonName, PTStyleClass.DIR_BTN, PTStyleClass.DIR_BTN_TEXT,
         PTImageType.ACTION_BROWSE, null, iBrowseAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "This button should be added to FOLDER banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/fileCrawler/adminSubExpandedTemplate.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   NEW_PAGE
        sButtonName = "new_page";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_CREATE, null, iNewPageAltId, true);
        ////////////////////////////////////////////////////
        //   NEW_SUBCOMMUNITY
        sButtonName = "new_subcommunity";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_CREATE, PTImageType.ICON_COMMUNITY, iNewSubcommunityAltId,
         true);
        ////////////////////////////////////////////////////
        //   NEW_COMMUNITY_TEMPLATE
        sButtonName = "new_community_template";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_CREATE, PTImageType.ICON_COMMUNITY_TEMPLATE, iNewCommunityTemplateAltId,
         true);
        ////////////////////////////////////////////////////
        //   NEW_PAGE_TEMPLATE
        sButtonName = "new_page_template";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_CREATE, PTImageType.ICON_PAGE_TEMPLATE, iNewPageTemplateAltId,
         true);
        ////////////////////////////////////////////////////
        //   NEW_BANNER
        sButtonName = "new_banner";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_NEW, PTImageType.ICON_BANNER, iNewBannerAltId,
         true);
        ////////////////////////////////////////////////////
        //   NEW_FOOTER
        sButtonName = "new_footer";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_NEW, PTImageType.ICON_FOOTER, iNewFooterAltId,
         true);
        ////////////////////////////////////////////////////
        //   PREVIEW
        sButtonName = "preview";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ICON_PREVIEW, null, iPreviewAltId, true);
        ////////////////////////////////////////////////////
        //   SELECT_COMMUNITY_TEMPLATE
        sButtonName = "select_community_template";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_SELECT, PTImageType.ICON_COMMUNITY_TEMPLATE, iSelectCommunityTemplateAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "This button should be added to OBJECT_ACTIONBAR banners.  See the <a href=\"http://ptproto/odyssey/odysseyCommunities/createCommunity.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   SELECT_PAGE_TEMPLATE
        sButtonName = "select_page_template";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_SELECT, PTImageType.ICON_PAGE_TEMPLATE, iSelectPageTemplateAltId,
         true);
        ////////////////////////////////////////////////////
        //   ADD_PORTLET
        sButtonName = "add_portlet";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_ADD, PTImageType.ICON_PORTLET, iAddPortletAltId,
         true);
        ////////////////////////////////////////////////////
        //   CREATE_PORTLET
        sButtonName = "create_portlet";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_NEW, PTImageType.ICON_PORTLET, iCreatePortletAltId,
         true);
        ////////////////////////////////////////////////////
        //   SELECT_LAYOUT
        sButtonName = "select_layout";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_SELECT, PTImageType.ICON_LAYOUT, iSelectLayoutAltId,
         true);
        ////////////////////////////////////////////////////
        //   RENAME_PAGE
        sButtonName = "rename_page";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_BAN_BTN, PTStyleClass.BAN_TEXT,
         PTImageType.ACTION_RENAME, PTImageType.ICON_PAGE_TEMPLATE, iRenamePageAltId,
         true);
        ////////////////////////////////////////////////////
        //   ADD_MEMBERS
        sButtonName = "add_members";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_ADD, PTImageType.ICON_MEMBER, iAddMembersAltId,
         true);
        ////////////////////////////////////////////////////
        //   CREATE_CUSTOM_GROUP
        sButtonName = "create_custom_group";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_CREATE, PTImageType.ICON_GROUP, iCreateCustomGroupAltId,
         true);
        ////////////////////////////////////////////////////
        //   SEND_TO_OTHERS
        sButtonName = "send_to_others";
        MakeSimpleButton(sButtonName, iSendToOthersTitleId, PTStyleClass.ACTIONBAR_BAN_BTN,
         PTStyleClass.ACTIONBAR_BAN_TEXT, PTImageType.ACTION_SEND, null, iSendToOthersAltId,
         true);
        ////////////////////////////////////////////////////
        //   SEND_TO_OTHERS
        sButtonName = "send_to_others_banner_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_BAN_BTN, PTStyleClass.ACTIONBAR_BAN_TEXT,
         PTImageType.ACTION_SEND, null, iFinishAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "");
        ////////////////////////////////////////////////////
        //   CLOSE
        sButtonName = "close";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ICON_BUTTON_CLOSE, null, iCloseAltId, true);
        ////////////////////////////////////////////////////
        //   CLEAR
        sButtonName = "clear";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ICON_BUTTON_CLOSE, null, iClearAltId, true);
        ////////////////////////////////////////////////////
        //   DELETE
        sButtonName = "delete";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_DELETE, null, iDeleteAltId, true);
        ////////////////////////////////////////////////////
        //   DELETE_DIR_ACTIONBAR
        sButtonName = "delete_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_DELETE, null, iDeleteAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this delete button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/directoryActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   DELETE_OBJ_ACTIONBAR
        sButtonName = "delete_obj_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_DELETE, null, iDeleteAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this delete button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/objectActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   COLLAPSE_AREA
        sButtonName = "collapse_area";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_BAN_BTN, PTStyleClass.ACTIONBAR_BAN_TEXT,
         PTImageType.ICON_ARROW_EXPAND, null, iCollapseAltId, true);
        //   COLLAPSE_TIPS
        sButtonName = "collapse_tips";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_TEXT, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ICON_ARROW_EXPAND, null, iCollapseAltId, false);
        ////////////////////////////////////////////////////
        //   EXPAND_AREA
        sButtonName = "expand_area";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_BAN_BTN, PTStyleClass.ACTIONBAR_BAN_TEXT,
         PTImageType.ICON_ARROW_COLLAPSE, null, iExpandAltId, true);
        ////////////////////////////////////////////////////
        //   EXPAND_TIPS
        sButtonName = "expand_tips";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_TEXT, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ICON_ARROW_COLLAPSE, null, iExpandAltId, false);
        ////////////////////////////////////////////////////
        //   ADD_GROUP
        sButtonName = "add_group";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_ADD, PTImageType.ICON_GROUP, iAddGroupAltId, true);
        ////////////////////////////////////////////////////
        //   FINISH
        sButtonName = "finish";
        myInput = new HTMLInput(HTMLInputTypes.BUTTON_TYPE, "in_bu_finish",
         "Finish!");
        xpmNameToElement.PutElement(sButtonName, myInput);
        ////////////////////////////////////////////////////
        //   EDIT
        sButtonName = "edit";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_EDIT, null, iEditAltId, true);
        ////////////////////////////////////////////////////
        //   EDIT_ACTIONBAR
        sButtonName = "edit_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_EDIT, null, iEditAltId, true);
        ////////////////////////////////////////////////////
        //   EDIT_DIR_ACTIONBAR
        sButtonName = "edit_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_EDIT, null, iEditAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this edit button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/directoryActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   EDIT_FOLDER_DIR_ACTIONBAR
        sButtonName = "edit_folder_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_EDIT_SMALL, PTImageType.ICON_FOLDER, iEditAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "Use this edit button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/directoryActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   EDIT_OBJ_ACTIONBAR
        sButtonName = "edit_obj_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_EDIT, null, iEditAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this edit button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/objectActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   UNLOCK_OBJ_ACTIONBAR
        sButtonName = "unlock_obj_actionbar";
        MakeSimpleButton(sButtonName, iUnlockTitleId, PTStyleClass.ACTIONBAR_OBJECT_BTN,
         PTStyleClass.ACTIONBAR_TEXT, PTImageType.ACTION_UNLOCK, null, iUnlockAltId,
         true);
        xpmNameToDescription.PutElement(sButtonName, "Use this edit button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/objectActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   EDIT_HEADER
        sButtonName = "edit_header";
        MakeSimpleButton(sButtonName, PTStyleClass.DIR_BTN, PTStyleClass.DIR_BTN_TEXT,
         PTImageType.ACTION_EDIT, null, iEditAltId, true);
        ////////////////////////////////////////////////////
        //   CONTENT_SNAPSHOT_HEADER
        sButtonName = "content_snapshot_header";
        MakeSimpleButton(sButtonName, PTStyleClass.DIR_BTN, PTStyleClass.DIR_BTN_TEXT,
         PTImageType.ACTION_CONTENT_SNAPSHOT, null, iEditAltId, true);
        ////////////////////////////////////////////////////
        //   CREATE
        sButtonName = "create";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_CREATE, null, iCreateAltId, true);
        ////////////////////////////////////////////////////
        //   CREATE_ACTIONBAR
        sButtonName = "create_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_CREATE, null, iCreateAltId, true);
        ////////////////////////////////////////////////////
        //   ADD_ME_ACTIONBAR
        sButtonName = "add_me_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_ADD, PTImageType.ICON_USER, iAddMeAltId, true);
        ////////////////////////////////////////////////////
        //   CREATE_DIR_ACTIONBAR
        sButtonName = "create_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_CREATE, null, iCreateAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this create button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/directoryActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   CREATE_OBJ_ACTIONBAR
        sButtonName = "create_obj_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_CREATE, null, iCreateAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this create button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/objectActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   ABORT_JOB
        sButtonName = "abort_job";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.OBJECT_TEXT,
         PTImageType.ACTION_ABORT, null, iAbortAltId, true);
        ////////////////////////////////////////////////////
        //   SELECT
        sButtonName = "select";
        MakeSimpleButton(sButtonName, PTStyleClass.OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_SELECT, null, iSelectAltId, true);
        ////////////////////////////////////////////////////
        //   SELECT_ACTIONBAR
        sButtonName = "select_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_SELECT, null, iSelectAltId, true);
        ////////////////////////////////////////////////////
        //   SELECT_DIR_ACTIONBAR
        sButtonName = "select_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_SELECT, null, iSelectAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this select button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/directoryActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   SELECT_ALL_DIR_ACTIONBAR
        sButtonName = "select_all_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_ALL_CHECK, null, iSelectAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this select button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/directoryActionBar.htm\" target=\"new\">spec</a>.");
        // //////////////////////////////////////////////////
        //   UNSELECT_ALL_DIR_ACTIONBAR
        sButtonName = "unselect_all_dir_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_DIR_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_ALL_UNCHECK, null, iSelectAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this select button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/directoryActionBar.htm\" target=\"new\">spec</a>.");
        ////////////////////////////////////////////////////
        //   SELECT_OBJ_ACTIONBAR
        sButtonName = "select_obj_actionbar";
        MakeSimpleButton(sButtonName, PTStyleClass.ACTIONBAR_OBJECT_BTN, PTStyleClass.ACTIONBAR_TEXT,
         PTImageType.ACTION_SELECT, null, iSelectAltId, true);
        xpmNameToDescription.PutElement(sButtonName, "Use this select button in DIRECTORY_ACTION_BAR banners.  See the <a href=\"http://ptproto/prototypes/Validated%20-%20HTML4.01/Platform/editors/admin/commonComponents/objectActionBar.htm\" target=\"new\">spec</a>.");
        }
      catch (HTMLException he)
      {
        throw new Exception("Error in PTButtonType - " + he.Message);
      }
    }
  }
}
//        if (bDebug)
//        {
//            myLogger.Debug("Leaving static block.");
//        }
