using System;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.xpshared.htmlconstructs
{
  /// <author>
  /// beng         All classes representing types of constructs must
  /// extend this class.
  /// </author>
  public abstract class PTConstructType
  {
    
    /// <summary>
    /// Use this as the value for those strings which should not be
    /// automatically updated by the factory.  For example, a gadget banner's
    /// title is not a string that is extracted and localized, but rather a
    /// string that simply needs to be filled in.
    /// </summary>
    public const String DO_NOT_UPDATE = "&?!!Bzzrt,";
    
    /// <returns></returns>
    public abstract String GetDescription();
    
    /// <summary>
    /// Call this method to obtain an ElementName object indicating a specific
    /// html element found within this construct.     <br />
    /// <br />
    /// For example, to obtain an ElementName object indicating the html
    /// element containing the wide banner's title, do the following:     <br />
    /// <br />
    /// <c>    myEName = PTBannerType.WIDE.GetElementName(
    /// PTBannerType.ELEMENT_NAME_ID_TITLE );     </c><br /><br />
    /// </summary>
    /// <param name="sElementNameID">
    /// Pass a constant indicating which ElementName you
    /// wish to obtain.  Each concrete PTConstructType provides a unique
    /// set     of constants for obtaining ElementName objects.
    /// </param>
    /// <returns></returns>
    /// <throws>Exception</throws>
    public abstract ElementName GetElementName(String sElementNameID);
    
    /// <summary>
    /// Call this method to obtain the string dictionary for this     construct. <br />
    /// <br />
    /// A construct's string dictionary contains, as keys, the name's of those
    /// html elements that must be updated with localized strings. The values
    /// corresponding to these keys are the id's used to obtain the
    /// corresponding localized string.  For example, if a banner's string
    /// dictionary containes one pair of strings:     <br />
    /// <br />
    /// &nbsp;&nbsp;&nbsp;"title" - "titleID"     <br />
    /// <br />
    /// then, the element named "title" would need to be updated with the
    /// localized string retrieved with the id "titleID".
    /// </summary>
    /// <returns></returns>
    public abstract XPHashtable GetStringMap();
    
    /// <summary>
    /// Call this method to obtain a prototype HTMLElement for this particular
    /// construct type.  This prototype should neither be used, nor modified,
    /// though it may be examined or cloned.     <br />
    /// <br />
    /// If you wish to obtain an HTMLElement representation of this construct
    /// type that is all your own, with which you're free to do as you please,
    /// use the PTConstructFactory.GetConstruct() method.
    /// </summary>
    /// <returns></returns>
    /// <throws>Exception</throws>
    public virtual HTMLElement GetHTMLElement()
    {
      throw new XPException();
    }
    
    /// <summary>
    /// Call this method to obtain a string dictionary mapping element names to
    /// img alt id's.  An element name indicates the name of a particular
    /// HTMLImg element within this construct.  An img alt id indicates the id
    /// that may be used to obtain an alt string.
    /// </summary>
    /// <returns></returns>
    public abstract XPHashtable GetImgElementNameToAltIdMap();
  }
}