using System;
//import com.plumtree.xpfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.constants;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.xpshared.htmlconstructs
{
  /// <author>
  /// JohnH  This construct returns an input box and a combo box for an
  /// integer and time increments (minutes, hours, days, etc.)
  /// respectively.
  /// </author>
  public class PTDurationCombo
  {
    
    /// <summary>
    /// Localized String: Millisecond(s) 
    /// </summary>
    public const String _ui_ViewMilliseconds = "Millisecond(s)";
    
    /// <summary>
    /// Localized String: Second(s) 
    /// </summary>
    public const String _ui_ViewSeconds = "Second(s)";
    
    /// <summary>
    /// Localized String: Minute(s) 
    /// </summary>
    public const String _ui_ViewMinutes = "Minute(s)";
    
    /// <summary>
    /// Localized String: Hour(s) 
    /// </summary>
    public const String _ui_ViewHours = "Hour(s)";
    
    /// <summary>
    /// Localized String: Day(s) 
    /// </summary>
    public const String _ui_ViewDays = "Day(s)";
    
    /// <summary>
    /// Localized String: Week(s) 
    /// </summary>
    public const String _ui_ViewWeeks = "Week(s)";
    
    /// <summary>
    /// Localized String: Month(s) 
    /// </summary>
    public const String _ui_ViewMonths = "Month(s)";
    
    /// <summary>
    /// Localized String: Year(s) 
    /// </summary>
    public const String _ui_ViewYears = "Year(s)";
    public const int DISPLAYMILLISECONDS = 1;
    public const int DISPLAYSECONDS = 2;
    public const int DISPLAYMINUTES = 4;
    public const int DISPLAYHOURS = 8;
    public const int DISPLAYDAYS = 16;
    public const int DISPLAYWEEKS = 32;
    public const int DISPLAYMONTHS = 64;
    public const int DISPLAYYEARS = 128;
    
    /// <summary>
    /// Creates a new PTDurationCombo object.
    /// </summary>
    public PTDurationCombo()
    {
    }
    
    /// <param name="nAvailableDurations"></param>
    /// <param name="lMilliseconds"></param>
    /// <returns></returns>
    public static int BestDurationTypeForMilliseconds(int nAvailableDurations,
     long lMilliseconds)
    {
      long l = (long) 12L * 4L * 7L * 24L * 60L * 60L * 1000L;
      if (0 != (PTDurationCombo.DISPLAYYEARS & nAvailableDurations))
      {
        if (0 == (lMilliseconds % l))
        {
          return PTDurationCombo.DISPLAYYEARS;
        }
      }
      l = (long) 4L * 7L * 24L * 60L * 60L * 1000L;
      if (0 != (PTDurationCombo.DISPLAYMONTHS & nAvailableDurations))
      {
        if (0 == (lMilliseconds % l))
        {
          return PTDurationCombo.DISPLAYMONTHS;
        }
      }
      if (0 != (PTDurationCombo.DISPLAYWEEKS & nAvailableDurations))
      {
        if (0 == (lMilliseconds % (7L * 24L * 60L * 60L * 1000L)))
        {
          return PTDurationCombo.DISPLAYWEEKS;
        }
      }
      if (0 != (PTDurationCombo.DISPLAYDAYS & nAvailableDurations))
      {
        if (0 == (lMilliseconds % (24 * 60 * 60 * 1000)))
        {
          return PTDurationCombo.DISPLAYDAYS;
        }
      }
      if (0 != (PTDurationCombo.DISPLAYHOURS & nAvailableDurations))
      {
        if (0 == (lMilliseconds % (60 * 60 * 1000)))
        {
          return PTDurationCombo.DISPLAYHOURS;
        }
      }
      if (0 != (PTDurationCombo.DISPLAYMINUTES & nAvailableDurations))
      {
        if (0 == (lMilliseconds % (60 * 1000)))
        {
          return PTDurationCombo.DISPLAYMINUTES;
        }
      }
      if (0 != (PTDurationCombo.DISPLAYSECONDS & nAvailableDurations))
      {
        if (0 == (lMilliseconds % 1000))
        {
          return PTDurationCombo.DISPLAYSECONDS;
        }
      }
      if (0 != (PTDurationCombo.DISPLAYMILLISECONDS & nAvailableDurations))
      {
        return PTDurationCombo.DISPLAYMILLISECONDS;
      }
      return 0;
    }
    
    /// <param name="lMilliseconds"></param>
    /// <param name="nDurationType"></param>
    /// <returns></returns>
    public static long DurationFromMilliseconds(long lMilliseconds, int nDurationType)
    {
      long nTimeoutMilliseconds = lMilliseconds;
      switch (nDurationType){
      case PTDurationCombo.DISPLAYYEARS:
      nTimeoutMilliseconds = nTimeoutMilliseconds / 12;
      goto case PTDurationCombo.DISPLAYMONTHS;
      case PTDurationCombo.DISPLAYMONTHS:
      nTimeoutMilliseconds = nTimeoutMilliseconds / 4;
      goto case PTDurationCombo.DISPLAYWEEKS;
      case PTDurationCombo.DISPLAYWEEKS:
      nTimeoutMilliseconds = nTimeoutMilliseconds / 7;
      goto case PTDurationCombo.DISPLAYDAYS;
      case PTDurationCombo.DISPLAYDAYS:
      nTimeoutMilliseconds = nTimeoutMilliseconds / 24;
      goto case PTDurationCombo.DISPLAYHOURS;
      case PTDurationCombo.DISPLAYHOURS:
      nTimeoutMilliseconds = nTimeoutMilliseconds / 60;
      goto case PTDurationCombo.DISPLAYMINUTES;
      case PTDurationCombo.DISPLAYMINUTES:
      nTimeoutMilliseconds = nTimeoutMilliseconds / 60;
      goto case PTDurationCombo.DISPLAYSECONDS;
      case PTDurationCombo.DISPLAYSECONDS:
      nTimeoutMilliseconds = nTimeoutMilliseconds / 1000;
      break;
      }
      return nTimeoutMilliseconds;
    }
    
    /// <param name="nDuration"></param>
    /// <param name="nDurationType"></param>
    /// <returns></returns>
    public static long DurationToMilliseconds(long nDuration, int nDurationType)
    {
      long nTimeoutMilliseconds = nDuration;
      switch (nDurationType){
      case PTDurationCombo.DISPLAYYEARS:
      nTimeoutMilliseconds = 12 * nTimeoutMilliseconds;
      goto case PTDurationCombo.DISPLAYMONTHS;
      case PTDurationCombo.DISPLAYMONTHS:
      nTimeoutMilliseconds = 4 * nTimeoutMilliseconds;
      goto case PTDurationCombo.DISPLAYWEEKS;
      case PTDurationCombo.DISPLAYWEEKS:
      nTimeoutMilliseconds = 7 * nTimeoutMilliseconds;
      goto case PTDurationCombo.DISPLAYDAYS;
      case PTDurationCombo.DISPLAYDAYS:
      nTimeoutMilliseconds = 24 * nTimeoutMilliseconds;
      goto case PTDurationCombo.DISPLAYHOURS;
      case PTDurationCombo.DISPLAYHOURS:
      nTimeoutMilliseconds = 60 * nTimeoutMilliseconds;
      goto case PTDurationCombo.DISPLAYMINUTES;
      case PTDurationCombo.DISPLAYMINUTES:
      nTimeoutMilliseconds = 60 * nTimeoutMilliseconds;
      goto case PTDurationCombo.DISPLAYSECONDS;
      case PTDurationCombo.DISPLAYSECONDS:
      nTimeoutMilliseconds = 1000 * nTimeoutMilliseconds;
      break;
      }
      return nTimeoutMilliseconds;
    }
    
    /// <summary>
    /// This returns an HTML INPUT box followed by a combo box populated  with
    /// time increments.
    /// </summary>
    /// <param name="strDurationElemName">
    /// - The name for the HTML INPUT box containing
    /// the numeric duration
    /// </param>
    /// <param name="strUnitsElemName">
    /// - The name for the SELECT drop down containing
    /// the units
    /// </param>
    /// <param name="intDisplayFlags">
    /// - An integer indicating which durations to
    /// display in the select.  The choices are exposed by this class as
    /// public constants: DISPLAYMINUTES = 1, DISPLAYHOURS = 2,
    /// DISPLAYDAYS = 4, DISPLAYWEEKS = 8.  These can be combined by
    /// adding up the choices.  Thus,  a value of 3 would display
    /// minutes and hours.
    /// </param>
    /// <param name="strSetDuration">- Sets the value for the duration input</param>
    /// <param name="intSetUnits">- Sets the integer value for the Units Select element</param>
    /// <returns></returns>
    /// <throws>Exception</throws>
    /// <throws>XPException</throws>
    public static HTMLElement GetDurationCombo(String strDurationElemName,
     String strUnitsElemName, int intDisplayFlags, String strSetDuration,
     int intSetUnits, AActivitySpace asOwner)
    {
      HTMLElementCollection result = new HTMLElementCollection();
      HTMLSelect mySelect;
      HTMLOption myOption;
      HTMLSpan mySpan;
      HTMLInput myInput;
      try
      {
        myInput = new HTMLInput(HTMLInputTypes.TEXT, strDurationElemName,
         strDurationElemName);
        //This is a number input box, so set the size = 3
        myInput.SetSize(3);
        myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
        if (!strSetDuration.Equals(""))
        {
          myInput.SetValue(strSetDuration);
        }
        result.AddInnerHTMLElement(myInput);
        result.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        // Add the second element
        mySelect = new HTMLSelect(strUnitsElemName, strUnitsElemName);
        mySelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
        //Determine which options to display through bitwise comparison
        if ((intDisplayFlags & DISPLAYMILLISECONDS) != 0)
        {
          myOption = new HTMLOption(asOwner.GetString(113, ResourceConstants.INFRASTRUCTURE));
          myOption.SetValue(XPConvert.ToString(DISPLAYMILLISECONDS));
          if (DISPLAYMILLISECONDS == intSetUnits)
          {
            myOption.SetSelected(true);
          }
          mySpan = new HTMLSpan();
          myOption.AddInnerHTMLElement(mySpan);
          mySpan.AddInnerHTMLString(asOwner.GetString(113, ResourceConstants.INFRASTRUCTURE));
          mySelect.AddInnerHTMLElement(myOption);
        }
        if ((intDisplayFlags & DISPLAYSECONDS) != 0)
        {
          myOption = new HTMLOption(asOwner.GetString(114, ResourceConstants.INFRASTRUCTURE));
          myOption.SetValue(XPConvert.ToString(DISPLAYSECONDS));
          if (DISPLAYSECONDS == intSetUnits)
          {
            myOption.SetSelected(true);
          }
          mySpan = new HTMLSpan();
          myOption.AddInnerHTMLElement(mySpan);
          mySpan.AddInnerHTMLString(asOwner.GetString(114, ResourceConstants.INFRASTRUCTURE));
          mySelect.AddInnerHTMLElement(myOption);
        }
        if ((intDisplayFlags & DISPLAYMINUTES) != 0)
        {
          myOption = new HTMLOption(asOwner.GetString(115, ResourceConstants.INFRASTRUCTURE));
          myOption.SetValue(XPConvert.ToString(DISPLAYMINUTES));
          if (DISPLAYMINUTES == intSetUnits)
          {
            myOption.SetSelected(true);
          }
          mySpan = new HTMLSpan();
          myOption.AddInnerHTMLElement(mySpan);
          mySpan.AddInnerHTMLString(asOwner.GetString(115, ResourceConstants.INFRASTRUCTURE));
          mySelect.AddInnerHTMLElement(myOption);
        }
        if ((intDisplayFlags & DISPLAYHOURS) != 0)
        {
          myOption = new HTMLOption(asOwner.GetString(116, ResourceConstants.INFRASTRUCTURE));
          myOption.SetValue(XPConvert.ToString(DISPLAYHOURS));
          if (DISPLAYHOURS == intSetUnits)
          {
            myOption.SetSelected(true);
          }
          mySpan = new HTMLSpan();
          myOption.AddInnerHTMLElement(mySpan);
          mySpan.AddInnerHTMLString(asOwner.GetString(116, ResourceConstants.INFRASTRUCTURE));
          mySelect.AddInnerHTMLElement(myOption);
        }
        if ((intDisplayFlags & DISPLAYDAYS) != 0)
        {
          myOption = new HTMLOption(asOwner.GetString(117, ResourceConstants.INFRASTRUCTURE));
          myOption.SetValue(XPConvert.ToString(DISPLAYDAYS));
          if (DISPLAYDAYS == intSetUnits)
          {
            myOption.SetSelected(true);
          }
          mySpan = new HTMLSpan();
          myOption.AddInnerHTMLElement(mySpan);
          mySpan.AddInnerHTMLString(asOwner.GetString(117, ResourceConstants.INFRASTRUCTURE));
          mySelect.AddInnerHTMLElement(myOption);
        }
        if ((intDisplayFlags & DISPLAYWEEKS) != 0)
        {
          myOption = new HTMLOption(asOwner.GetString(118, ResourceConstants.INFRASTRUCTURE));
          myOption.SetValue(XPConvert.ToString(DISPLAYWEEKS));
          if (DISPLAYWEEKS == intSetUnits)
          {
            myOption.SetSelected(true);
          }
          mySpan = new HTMLSpan();
          myOption.AddInnerHTMLElement(mySpan);
          mySpan.AddInnerHTMLString(asOwner.GetString(118, ResourceConstants.INFRASTRUCTURE));
          mySelect.AddInnerHTMLElement(myOption);
        }
        if ((intDisplayFlags & DISPLAYMONTHS) != 0)
        {
          myOption = new HTMLOption(asOwner.GetString(119, ResourceConstants.INFRASTRUCTURE));
          myOption.SetValue(XPConvert.ToString(DISPLAYMONTHS));
          if (DISPLAYMONTHS == intSetUnits)
          {
            myOption.SetSelected(true);
          }
          mySpan = new HTMLSpan();
          myOption.AddInnerHTMLElement(mySpan);
          mySpan.AddInnerHTMLString(asOwner.GetString(119, ResourceConstants.INFRASTRUCTURE));
          mySelect.AddInnerHTMLElement(myOption);
        }
        if ((intDisplayFlags & DISPLAYYEARS) != 0)
        {
          myOption = new HTMLOption(asOwner.GetString(120, ResourceConstants.INFRASTRUCTURE));
          myOption.SetValue(XPConvert.ToString(DISPLAYYEARS));
          if (DISPLAYYEARS == intSetUnits)
          {
            myOption.SetSelected(true);
          }
          mySpan = new HTMLSpan();
          myOption.AddInnerHTMLElement(mySpan);
          mySpan.AddInnerHTMLString(asOwner.GetString(120, ResourceConstants.INFRASTRUCTURE));
          mySelect.AddInnerHTMLElement(myOption);
        }
        result.AddInnerHTMLElement(mySelect);
      }
      catch (Exception e)
      {
        throw new XPException(asOwner.GetString(121, ResourceConstants.INFRASTRUCTURE),
         e);
      }
      return result;
    }
  }
}