using System;
//import com.plumtree.xpfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.xpshared.htmlconstructs
{
  /// <author>
  /// sample         This class generates html necessary to construct a
  /// table containing DIVs of table rows that are revealed and hidden
  /// from the last visible row of the table.     A total of five calls
  /// will generate all of the necessary     elements:     1)
  /// setBlankQueryCondition(PTQueryCondition)  // defines the row
  /// template 2) generateTable (numberOfRows)    // gets a table of
  /// hidden rows     3) getJavaScriptFunctions() // gets necessary
  /// script elements 4) getAddConditionButton(strAdd) // button to
  /// reveal a row 5) getRemoveLastConditionButton(strRemove)  // button
  /// to hide last row     The "Add Condition" and "Remove Last
  /// Condition" buttons     must be retrieved with separate calls.
  /// </author>
  public class PTDynamicQueryTable
  {
    //
    //    private static XPLogger myLogger = XPLogManager.GetLogger("com.plumtree.xpshared.htmlconstructs.PTDynamicQueryTable");
    //
    //
    //    private static boolean bDebug = myLogger.IsDebugEnabled();
    //
    //
    //    private static boolean bError = myLogger.IsErrorEnabled();
    //
    //
    //    private static boolean bWarn = myLogger.IsWarnEnabled();
    //
    //
    //    private static boolean bInfo = myLogger.IsInfoEnabled();
    //
    //
    //    private static boolean bFatal = myLogger.IsFatalEnabled();
    private static String _ui_DisplayHTMLError = "Unable to render query table due to construct.";
    private PTQueryCondition m_BlankQC = null;
    private String m_strName;
    private XPArrayList m_listDefinedRows = null;
    private bool m_bAndOperator = true;
    private int m_Depth = 0;
    private int m_Rows = 0;
    
    /// <summary>
    /// Creates a new PTDynamicQueryTable object.
    /// </summary>
    /// <param name="strName"></param>
    /// <param name="Depth"></param>
    public PTDynamicQueryTable(String strName, int Depth)
    {
      m_Depth = Depth;
      m_strName = strName;
      m_Rows = 0;
    }
    
    /// <param name="bOperator"></param>
    public virtual void setAllMustPass(bool bOperator)
    {
      m_bAndOperator = bOperator;
    }
    
    /// <param name="qc"></param>
    public virtual void setBlankQueryCondition(PTQueryCondition qc)
    {
      qc.makeInVisible();
      m_BlankQC = qc;
    }
    
    /// <param name="nRows"></param>
    /// <throws>XPException</throws>
    public virtual void setTableSize(int nRows)
    {
      if ((nRows > 0) && (m_Rows == 0))
      {
        m_Rows = nRows;
      } else
      {
        throw new XPException("Row count must be greater than zero and cannot be set more than once.");
        }
    }
    
    /// <param name="strActiveClause"></param>
    /// <param name="groupCount"></param>
    /// <param name="result"></param>
    /// <throws>Exception</throws>
    public static void AddGlobalElements(String strActiveClause, int groupCount,
     HTMLElementCollection result)
    {
      HTMLInput activeClause = new HTMLInput(HTMLInputTypes.HIDDEN, "in_hi_activeclause",
       "");
      activeClause.SetID("activeclause");
      activeClause.SetValue(strActiveClause);
      result.AddInnerHTMLElement(activeClause);
      HTMLInput groupAction = new HTMLInput(HTMLInputTypes.HIDDEN, "in_hi_groupaction",
       "");
      groupAction.SetID("groupaction");
      groupAction.SetValue("");
      result.AddInnerHTMLElement(groupAction);
      HTMLInput totalGroups = new HTMLInput(HTMLInputTypes.HIDDEN, "in_hi_totalgroups",
       "");
      totalGroups.SetID("totalgroups");
      totalGroups.SetValue(XPConvert.ToString(groupCount));
      result.AddInnerHTMLElement(totalGroups);
    }
    
    /// <param name="searchstring"></param>
    /// <param name="AllMustPass"></param>
    /// <param name="result"></param>
    /// <throws>Exception</throws>
    public static void AddTopLevelSearchBoxAndOperator(String searchstring,
     bool AllMustPass, HTMLElementCollection result)
    {
      HTMLTable myTable = new HTMLTable();
      result.AddInnerHTMLElement(myTable);
      myTable.SetWidth("600");
      myTable.SetBorder("0");
      myTable.SetCellPadding("5");
      myTable.SetCellSpacing("0");
      HTMLTableRow textSearchRow = new HTMLTableRow();
      HTMLTableRow operatorRow = new HTMLTableRow();
      myTable.AddInnerHTMLElement(textSearchRow);
      myTable.AddInnerHTMLElement(operatorRow);
      textSearchRow.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);
      operatorRow.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);
      HTMLTableCell textCell = new HTMLTableCell();
      HTMLTableCell opCell = new HTMLTableCell();
      textSearchRow.AddInnerHTMLElement(textCell);
      operatorRow.AddInnerHTMLElement(opCell);
      textCell.SetAlign("CENTER");
      textCell.SetWidth("100%");
      opCell.SetAlign("CENTER");
      opCell.SetWidth("100%");
      HTMLInput input = new HTMLInput(HTMLInputTypes.TEXT_TYPE, "in_tx_fulltext",
       "");
      input.SetValue(searchstring);
      input.SetSize(40);
      textCell.SetStyleClass(PTStyleClass.MENU_TEXT);
      textCell.AddInnerHTMLString("Basic Fields Search: ");
      textCell.AddInnerHTMLElement(input);
      HTMLInput andRB = new HTMLInput(HTMLInputTypes.RADIO, "in_ra_topoperator",
       "");
      andRB.SetValue("and");
      andRB.SetChecked(AllMustPass);
      opCell.SetStyleClass(PTStyleClass.MENU_TEXT);
      opCell.AddInnerHTMLElement(andRB);
      opCell.AddInnerHTMLString("And");
      HTMLInput orRB = new HTMLInput(HTMLInputTypes.RADIO, "in_ra_topoperator",
       "");
      orRB.SetValue("or");
      orRB.SetChecked(!AllMustPass);
      opCell.AddInnerHTMLElement(orRB);
      opCell.AddInnerHTMLString("Or");
    }
    
    /// <param name="strLabel"></param>
    /// <returns></returns>
    /// <throws>Exception</throws>
    public static HTMLElement GetAddConditionButton(String strLabel)
    {
      if ((strLabel == null) || ("").Equals(strLabel))
      {
        strLabel = "+ ADD STATEMENT";
      }
      HTMLInput addButton = new HTMLInput(HTMLInputTypes.BUTTON_TYPE, "in_bu_NewRow",
       "");
      addButton.SetValue(strLabel);
      addButton.SetOnClick("adjustRow(getActiveClause(), true);");
      addButton.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
      return addButton;
    }
    
    /// <param name="strLabel"></param>
    /// <returns></returns>
    /// <throws>Exception</throws>
    public static HTMLElement GetAddNewClauseButton(String strLabel)
    {
      if ((strLabel == null) || ("").Equals(strLabel))
      {
        strLabel = "+ ADD NEW GROUP";
      }
      HTMLInput addGroupButton = new HTMLInput(HTMLInputTypes.BUTTON_TYPE,
       "in_bu_AddGroup", "");
      addGroupButton.SetValue(strLabel);
      addGroupButton.SetOnClick("adjustGroups(true);");
      addGroupButton.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
      return addGroupButton;
    }
    
    /// <returns></returns>
    public virtual bool GetAllMustPass()
    {
      return m_bAndOperator;
    }
    
    /// <param name="iIndex"></param>
    /// <returns></returns>
    /// <throws>XPException</throws>
    public virtual PTQueryCondition GetDefinedQueryCondition(int iIndex)
    {
      if ((iIndex + 1) > m_listDefinedRows.GetSize())
      {
        throw new XPException("Requested condition is not yet in list");
      }
      return (PTQueryCondition) m_listDefinedRows.GetElement(iIndex);
    }
    
    /// <param name="submitString"></param>
    /// <returns></returns>
    /// <throws>Exception</throws>
    public static HTMLElement GetJavaScript(String submitString)
    {
      XPStringBuilder sb = new XPStringBuilder();
      HTMLScript script = script = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
      sb.Append("\nfunction AdjustOperator(FormName, RowNumber, val)\n");
      sb.Append("{\n");
      sb.Append("\n");
      sb.Append("\tvar valPair = val.split(\"|\");\n");
      sb.Append("\tvar Type = \'Text\';\n");
      sb.Append("\tif (valPair[1]==\'3\' || valPair[1]==\'5\')\n");
      sb.Append("\t\tType = \'Number\';\n");
      sb.Append("\tif (valPair[1]==\'7\')\n");
      sb.Append("\t\tType = \'Date\';\n");
      sb.Append("\n");
      sb.Append("\t// First shut all of them off, then turn on the appropriate operator drop down\n");
      sb.Append("\tvar OpDiv = document.getElementById(FormName + \'_divText_\' + RowNumber);\n");
      sb.Append("\tOpDiv.style.display = \'none\';\n");
      sb.Append("\tOpDiv = document.getElementById(FormName + \'_divNumber_\' + RowNumber);\n");
      sb.Append("\tOpDiv.style.display = \'none\';\n");
      sb.Append("\tOpDiv = document.getElementById(FormName + \'_divDate_\' + RowNumber);\n");
      sb.Append("\tOpDiv.style.display = \'none\';\n");
      sb.Append("\n");
      sb.Append("\tOpDiv = document.getElementById(FormName + \'_div\' + Type + \'_\' + RowNumber);\n");
      sb.Append("\tOpDiv.style.display = \'\';\n");
      sb.Append("}\n");
      sb.Append("\n");
      sb.Append("\nfunction adjustRow(FormName, bAdd)\n");
      sb.Append("{\n");
      sb.Append("\n");
      sb.Append("\t\tvar disp = (bAdd) ? \"\" : \"none\";\n");
      sb.Append("\t\tvar maxrows = document.getElementById(FormName + \'_maxrows\');\n");
      sb.Append("\t\tvar rowcount = document.getElementById(FormName + \'_revealed\');\n");
      sb.Append("\t\tvar current = rowcount.value;\n");
      sb.Append("\t\tvar max = maxrows.value;\n");
      //		sb.Append("		alert('COUNT = ' + current + ' MAX = ' + max);\n");
      sb.Append("\t\tif (((current == max) && bAdd) || (!bAdd && (current ==0))) return;\n");
      sb.Append("\n");
      sb.Append("\t\tif (bAdd)\n");
      sb.Append("\t\t\tcurrent++;\n");
      sb.Append("\n");
      //		sb.Append("		alert('INCREMENT COUNT = ' + current + ' MAX = ' + max);\n");
      sb.Append("\t\tvar propDiv = document.getElementById(FormName + \"_divPropRow_\" + current);\n");
      sb.Append("\t\tpropDiv.style.display = disp;\n");
      sb.Append("\t\tvar propSel = document.getElementById(FormName +\"_PropSelect_\" + current);\n");
      sb.Append("\t\tvar types = propSel.options[propSel.selectedIndex].value.split(\"|\");\n");
      sb.Append("\t\tvar OpDiv;\n");
      sb.Append("\t\tif (types[1] == 3 || types[1] == 5)\n");
      sb.Append("\t\t\tOpDiv = document.getElementById(FormName + \"_divNumber_\" + current);\n");
      sb.Append("\t\tif (types[1] == 7)\n");
      sb.Append("\t\t\tOpDiv = document.getElementById(FormName + \"_divDate_\" + current);\n");
      sb.Append("\t\tif (types[1] == 8 || types[1] == 72)\n");
      sb.Append("\t\t\tOpDiv = document.getElementById(FormName + \"_divText_\" + current);\n");
      sb.Append("\t\tOpDiv.style.display = disp;\n");
      sb.Append("\n");
      sb.Append("\t\tvar valueDiv = document.getElementById(FormName + \"_divCompareval_\" + current);\n");
      sb.Append("\t\tvalueDiv.style.display = disp;\n");
      sb.Append("\t\tif(!bAdd)\n");
      sb.Append("\t\t\tcurrent--;\n");
      sb.Append("\t\trowcount.value = current;\n");
      sb.Append("\n");
      sb.Append("}\n");
      sb.Append("\n\n");
      sb.Append("function getActiveClause()\n");
      sb.Append("{\n");
      sb.Append("\n");
      sb.Append("\tvar clause = document.getElementById(\'activeclause\');\n");
      sb.Append("\tvar formname = clause.value;\n");
      sb.Append(" return formname;\n");
      sb.Append("}\n");
      sb.Append("\n");
      sb.Append("\nfunction setActiveClause(NewGroupNum, IsChecked)\n");
      sb.Append("{\n");
      sb.Append("\tif (!IsChecked)\n");
      sb.Append("\t{ // it doesn\'t make sense to uncheck a group\n");
      sb.Append("\t\tvar ThisCheckBox = document.getElementById(NewGroupNum+\'_checked\');\n");
      sb.Append("\t\tThisCheckBox.checked = true;\n");
      sb.Append("\t\treturn;\n");
      sb.Append("\t}\n");
      sb.Append("\n");
      sb.Append("\tvar clause = document.getElementById(\'activeclause\');\n");
      sb.Append("\tvar oldfocus = clause.value;\n");
      sb.Append(" var vChecked = document.getElementById(oldfocus+\'_checked\');\n");
      sb.Append("\tclause.value = NewGroupNum;\n");
      sb.Append("\tif (!vChecked.checked)\n");
      sb.Append("\t\treturn;\n");
      sb.Append("\tvChecked.checked = false;\n");
      sb.Append("}\n");
      sb.Append("\n");
      sb.Append("\nfunction adjustGroups(bAdd)\n");
      sb.Append("{\n");
      sb.Append("\tvar action = document.getElementById(\'groupaction\');\n");
      sb.Append("\tif (bAdd)\n");
      sb.Append("\t\taction.value = \'ADD\';\n");
      sb.Append("\telse\n");
      sb.Append("\t\taction.value = \'REMOVE\';\n");
      sb.Append(submitString);
      sb.Append("}\n");
      script.AddInnerHTMLString(sb.ToString());
      return script;
    }
    
    /// <param name="strLabel"></param>
    /// <returns></returns>
    /// <throws>Exception</throws>
    public static HTMLElement GetRemoveClauseButton(String strLabel)
    {
      if ((strLabel == null) || ("").Equals(strLabel))
      {
        strLabel = "- REMOVE GROUP";
      }
      HTMLInput removeGroupButton = new HTMLInput(HTMLInputTypes.BUTTON_TYPE,
       "in_bu_RemoveGroup", "");
      removeGroupButton.SetValue(strLabel);
      removeGroupButton.SetOnClick("adjustGroups(false);");
      removeGroupButton.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
      return removeGroupButton;
    }
    
    /// <param name="strLabel"></param>
    /// <returns></returns>
    /// <throws>Exception</throws>
    public static HTMLElement GetRemoveLastConditionButton(String strLabel)
    {
      if ((strLabel == null) || ("").Equals(strLabel))
      {
        strLabel = "- REMOVE STATEMENT";
      }
      HTMLInput minusButton = new HTMLInput(HTMLInputTypes.BUTTON_TYPE, "in_bu_RemoveRow",
       "");
      minusButton.SetValue(strLabel);
      minusButton.SetOnClick("adjustRow(getActiveClause(), false);");
      minusButton.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
      return minusButton;
    }
    
    /// <param name="result"></param>
    /// <throws>XPException</throws>
    public virtual void addTableToCollection(HTMLElementCollection result)
    {
      // this spacer table is used for indenting clauses
      HTMLTable SpacerTable = new HTMLTable();
      SpacerTable.SetWidth("100%");
      SpacerTable.SetBorder("0");
      SpacerTable.SetCellPadding("0");
      SpacerTable.SetCellSpacing("0");
      HTMLTableRow SpacerRow = new HTMLTableRow();
      HTMLTableCell SpacerCell = new HTMLTableCell();
      HTMLTableCell ClauseCell = new HTMLTableCell();
      // the inner table
      HTMLTable myTable = new HTMLTable();
      if (m_listDefinedRows == null)
      {
        m_listDefinedRows = new XPArrayList();
      }
      try
      {
        // IF NEEDED, add the spacer column to the spacer row
        if (m_Depth > 0)
        {
          PTConstruct spacerConstruct = ConstructFactory.GetConstruct(PTImageType.SPACER,
           true);
          HTMLImg img = (HTMLImg) spacerConstruct.GetHTMLElement();
          img.SetWidth(XPConvert.ToString(m_Depth * 50));
          SpacerCell.AddInnerHTMLElement(img);
          SpacerRow.AddInnerHTMLElement(SpacerCell);
        }
        // ATTACH SPACER TABLE TO RESULT
        result.AddInnerHTMLElement(SpacerTable);
        // ATTACH ROW TO SPACER TABLE
        SpacerTable.AddInnerHTMLElement(SpacerRow);
        // ATTACH CELL TO ROW
        SpacerRow.AddInnerHTMLElement(ClauseCell);
        // ATTACH INNER TABLE TO CELL
        ClauseCell.AddInnerHTMLElement(myTable);
        if (m_BlankQC == null)
        {
          throw new XPException("Blank query condition must be set before generating table");
          }
        myTable.SetWidth("600");
        myTable.SetBorder("0");
        //used to be 5
        myTable.SetCellPadding("1");
        myTable.SetCellSpacing("0");
        HTMLTableCell titleCell = new HTMLTableCell();
        titleCell.SetWidth("100%");
        titleCell.SetHeight("20");
        /*PTStyleClass.OBJECT_TEXT*/
        titleCell.SetStyleClass(PTStyleClass.OBJECT_HEADER);
        titleCell.SetColSpan("2");
        titleCell.SetAlign("absLeft");
        HTMLInput checkbox = new HTMLInput(HTMLInputTypes.CHECKBOX, "in_cb_thisform",
         "");
        checkbox.SetValue(m_strName);
        checkbox.SetID(m_strName + "_checked");
        if (m_strName.Equals("1"))
        {
          checkbox.SetChecked(true);
        } else
        {
          checkbox.SetChecked(false);
        }
        checkbox.SetOnClick("setActiveClause(this.value, this.checked);");
        titleCell.AddInnerHTMLElement(checkbox);
        titleCell.AddInnerHTMLString("Grouping " + m_strName);
        //HTMLSpan mySpan = new HTMLSpan();
        //titleCell.AddInnerHTMLElement(mySpan);
        //mySpan.SetStyleClass(PTStyleClass.OBJECT_HEADER);
        //mySpan.AddInnerHTMLString("Grouping " + m_strName);
        HTMLTableRow titleRow = new HTMLTableRow();
        titleRow.SetStyleClass(PTStyleClass.OBJECT_HEADER_BG);
        titleRow.AddInnerHTMLElement(titleCell);
        // the operator selection
        HTMLTableCell GroupOps = new HTMLTableCell();
        GroupOps.SetNoWrap(true);
        HTMLInput andRB = new HTMLInput(HTMLInputTypes.RADIO, "in_ra_groupoperator_" + m_strName,
         m_strName);
        andRB.SetID(m_strName + "_and");
        andRB.SetValue("and");
        andRB.SetChecked(m_bAndOperator);
        GroupOps.AddInnerHTMLElement(andRB);
        GroupOps.AddInnerHTMLString("And");
        HTMLInput orRB = new HTMLInput(HTMLInputTypes.RADIO, "in_ra_groupoperator_" + m_strName,
         m_strName);
        orRB.SetID(m_strName + "_or");
        orRB.SetValue("or");
        orRB.SetChecked(!m_bAndOperator);
        GroupOps.AddInnerHTMLElement(orRB);
        GroupOps.AddInnerHTMLString("Or&nbsp&nbsp&nbsp&nbsp");
        GroupOps.SetHeight("20");
        /*PTStyleClass.OBJECT_TEXT*/
        GroupOps.SetStyleClass(PTStyleClass.OBJECT_HEADER);
        GroupOps.SetAlign("Right");
        titleRow.AddInnerHTMLElement(GroupOps);
        myTable.AddInnerHTMLElement(titleRow);
        int tableSize = m_listDefinedRows.GetSize();
        if (m_Rows == 0)
        {
          throw new XPException("Table size set before generating table");
          }
        PTQueryCondition qc;
        // now we simply add multiple identical rows
        for (int i = 0;i < m_Rows;i++)
        {
          if (i < tableSize)
          {
            qc = (PTQueryCondition) m_listDefinedRows.GetElement(i);
            qc.makeVisible();
          } else
          {
            qc = m_BlankQC;
          }
          myTable.AddInnerHTMLElement(qc.GetTableRow(m_strName, XPConvert.ToString(i + 1)));
          }
        // finally add a row that will be used by the
        HTMLInput maxrows = new HTMLInput(HTMLInputTypes.HIDDEN, "in_hi_maxrows_" + m_strName,
         "");
        maxrows.SetID(m_strName + "_maxrows");
        maxrows.SetValue(XPConvert.ToString(m_Rows));
        ClauseCell.AddInnerHTMLElement(maxrows);
        //result.AddInnerHTMLElement(maxrows);
        HTMLInput revealed = new HTMLInput(HTMLInputTypes.HIDDEN, "in_hi_revealed_" + m_strName,
         "");
        revealed.SetID(m_strName + "_revealed");
        revealed.SetValue(XPConvert.ToString(tableSize));
        ClauseCell.AddInnerHTMLElement(revealed);
        //result.AddInnerHTMLElement(revealed);
        HTMLInput depth = new HTMLInput(HTMLInputTypes.HIDDEN, "in_hi_depth_" + m_strName,
         "");
        depth.SetID(m_strName + "_depth");
        depth.SetValue(XPConvert.ToString(m_Depth));
        ClauseCell.AddInnerHTMLElement(depth);
      }
      //result.AddInnerHTMLElement(depth);
      //result.AddInnerHTMLElement(myTable);
      catch (Exception e)
      {
        //            if (bError)
        //            {
        //                myLogger.Error("PTDynamicQueryTable::generateTable: The following error occured when trying to display the HTML: " + e.getMessage());
        //            }
        throw new XPException(_ui_DisplayHTMLError, e);
      }
    }
    
    /// <param name="qc"></param>
    public virtual void loadDefinedQueryCondition(PTQueryCondition qc)
    {
      if (m_listDefinedRows == null)
      {
        m_listDefinedRows = new XPArrayList();
      }
      m_listDefinedRows.Add(qc);
    }
    
    /// <param name="iIndex"></param>
    /// <throws>XPException</throws>
    public virtual void removeDefinedQueryCondition(int iIndex)
    {
      if ((iIndex + 1) > m_listDefinedRows.GetSize())
      {
        throw new XPException("Requested condition is not yet in list");
      }
      m_listDefinedRows.RemoveAt(iIndex);
    }
  }
}