using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// <br /><br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/global.html#h-7.5.1">Spec</a>
  /// </summary>
  public class HTMLBody : HTMLElement
  {
    internal static int iNextID = 0;
    private static readonly byte[] BEGINTAG = HTMLElement.ToByteArray("\n\n\n\n<body");
    private static readonly byte[] ENDTAG = HTMLElement.ToByteArray("\n</body>\n");
    private const String BODY = "body";
    private static readonly byte[] MARGINWIDTH_ATTR = HTMLElement.ToByteArray(" marginwidth=\"");
    private static readonly byte[] MARGINHEIGHT_ATTR = HTMLElement.ToByteArray(" marginheight=\"");
    private static readonly byte[] LEFTMARGIN_ATTR = HTMLElement.ToByteArray(" leftmargin=\"");
    private static readonly byte[] TOPMARGIN_ATTR = HTMLElement.ToByteArray(" topmargin=\"");
    private static readonly byte[] ONACTIVATE_ATTR = HTMLElement.ToByteArray(" onactivate=\"");
    private static readonly byte[] ONAFTERPRINT_ATTR = HTMLElement.ToByteArray(" onafterprint=\"");
    private static readonly byte[] ONBEFOREACTIVATE_ATTR = HTMLElement.ToByteArray(" onbeforeactivate=\"");
    private static readonly byte[] ONBEFOREUNLOAD_ATTR = HTMLElement.ToByteArray(" onbeforeunload=\"");
    private static readonly byte[] ONBLUR_ATTR = HTMLElement.ToByteArray(" onblur=\"");
    private static readonly byte[] ONCONTROLSET_ATTR = HTMLElement.ToByteArray(" oncontrolset=\"");
    private static readonly byte[] ONDEACTIVATE_ATTR = HTMLElement.ToByteArray(" ondeactivate=\"");
    private static readonly byte[] ONERROR_ATTR = HTMLElement.ToByteArray(" onerror=\"");
    private static readonly byte[] ONFOCUS_ATTR = HTMLElement.ToByteArray(" onfocus=\"");
    private static readonly byte[] ONHELP_ATTR = HTMLElement.ToByteArray(" onhelp=\"");
    private static readonly byte[] ONLOAD_ATTR = HTMLElement.ToByteArray(" onload=\"");
    private static readonly byte[] ONRESIZE_ATTR = HTMLElement.ToByteArray(" onresize=\"");
    private static readonly byte[] ONRESIZEEND_ATTR = HTMLElement.ToByteArray(" onresizeend=\"");
    private static readonly byte[] ONRESIZESTART_ATTR = HTMLElement.ToByteArray(" onresizestart=\"");
    private static readonly byte[] ONUNLOAD_ATTR = HTMLElement.ToByteArray(" onunload=\"");
    //    private static XPLogger myLogger = XPLogManager.GetLogger("com.plumtree.xpshared.htmlelements.HTMLBody");
    //    private static boolean bDebug = myLogger.IsDebugEnabled();
    //    private static boolean bError = myLogger.IsErrorEnabled();
    //    private static boolean bWarn = myLogger.IsWarnEnabled();
    //    private static boolean bInfo = myLogger.IsInfoEnabled();
    //    private static boolean bFatal = myLogger.IsFatalEnabled();
    private String marginHeight = null;
    private String marginLeft = null;
    private String marginTop = null;
    private String marginWidth = null;
    private String sOnActivate = null;
    private String sOnAfterPrint = null;
    private String sOnBeforeActivate = null;
    private String sOnBeforeUnload = null;
    private String sOnBlur = null;
    private String sOnControlSet = null;
    private String sOnDeactivate = null;
    private String sOnError = null;
    private String sOnFocus = null;
    private String sOnHelp = null;
    private String sOnLoad = null;
    private String sOnResize = null;
    private String sOnResizeEnd = null;
    private String sOnResizeStart = null;
    private String sOnUnload = null;
    
    /// <summary>
    /// Creates a new HTMLBody object.
    /// </summary>
    public HTMLBody()
    {
    }
    
    /// <summary>
    /// Get the value of the leftmargin attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetLeftMargin()
    {
      return marginLeft;
    }
    
    /// <summary>
    /// Get the value of the marginheight attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetMarginHeight()
    {
      return marginHeight;
    }
    
    /// <summary>
    /// Get the value of the marginwidth attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetMarginWidth()
    {
      return marginWidth;
    }
    
    /// <summary>
    /// Get the value of the onactivate attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnActivate()
    {
      return sOnActivate;
    }
    
    /// <summary>
    /// Get the value of the onafterprint attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnAfterPrint()
    {
      return sOnAfterPrint;
    }
    
    /// <summary>
    /// Get the value of the onbeforeactivate attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnBeforeActivate()
    {
      return sOnBeforeActivate;
    }
    
    /// <summary>
    /// Get the value of the onbeforeunload attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnBeforeUnload()
    {
      return sOnBeforeUnload;
    }
    
    /// <summary>
    /// Get the value of the onblur attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnBlur()
    {
      return sOnBlur;
    }
    
    /// <summary>
    /// Get the value of the oncontrolset attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnControlSet()
    {
      return sOnControlSet;
    }
    
    /// <summary>
    /// Get the value of the ondeactivate attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnDeactivate()
    {
      return sOnDeactivate;
    }
    
    /// <summary>
    /// Get the value of the onerror attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnError()
    {
      return sOnError;
    }
    
    /// <summary>
    /// Get the value of the onfocus attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnFocus()
    {
      return sOnFocus;
    }
    
    /// <summary>
    /// Get the value of the onhelp attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnHelp()
    {
      return sOnHelp;
    }
    
    /// <summary>
    /// Get the value of the onload attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnLoad()
    {
      return sOnLoad;
    }
    
    /// <summary>
    /// Get the value of the onresize attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnResize()
    {
      return sOnResize;
    }
    
    /// <summary>
    /// Get the value of the onresizeend attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnResizeEnd()
    {
      return sOnResizeEnd;
    }
    
    /// <summary>
    /// Get the value of the onresizestart attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnResizeStart()
    {
      return sOnResizeStart;
    }
    
    /// <summary>
    /// Get the value of the onunload attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetOnUnload()
    {
      return sOnUnload;
    }
    
    /// <summary>
    /// Get the value of the topmargin attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetTopMargin()
    {
      return marginTop;
    }
    
    /// <summary>
    /// Set the value of the leftmargin attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetLeftMargin(String s)
    {
      marginLeft = s;
    }
    
    /// <summary>
    /// Set the value of the marginheight attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetMarginHeight(String s)
    {
      marginHeight = s;
    }
    
    /// <summary>
    /// Set the value of the marginwidth attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetMarginWidth(String s)
    {
      marginWidth = s;
    }
    
    /// <summary>
    /// Set the value of the onactivate attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnActivate(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnActivate = s;
    }
    
    /// <summary>
    /// Set the value of the onafterprint attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnAfterPrint(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnAfterPrint = s;
    }
    
    /// <summary>
    /// Set the value of the onbeforeactivate attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnBeforeActivate(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnBeforeActivate = s;
    }
    
    /// <summary>
    /// Set the value of the onbeforeunload attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnBeforeUnload(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnBeforeUnload = s;
    }
    
    /// <summary>
    /// Set the value of the onblur attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnBlur(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnBlur = s;
    }
    
    /// <summary>
    /// Set the value of the oncontrolset attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnControlSet(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnControlSet = s;
    }
    
    /// <summary>
    /// Set the value of the ondeactivate attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnDeactivate(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnDeactivate = s;
    }
    
    /// <summary>
    /// Set the value of the onerror attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnError(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnError = s;
    }
    
    /// <summary>
    /// Set the value of the onfocus attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnFocus(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnFocus = s;
    }
    
    /// <summary>
    /// Set the value of the onhelp attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnHelp(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnHelp = s;
    }
    
    /// <summary>
    /// Set the value of the onload attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnLoad(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnLoad = s;
    }
    
    /// <summary>
    /// Set the value of the onresize attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnResize(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnResize = s;
    }
    
    /// <summary>
    /// Set the value of the onresizeend attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnResizeEnd(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnResizeEnd = s;
    }
    
    /// <summary>
    /// Set the value of the onresizestart attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnResizeStart(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnResizeStart = s;
    }
    
    /// <summary>
    /// Set the value of the onunload attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetOnUnload(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnUnload = s;
    }
    
    /// <summary>
    /// Set the value of the topmargin attribute.
    /// </summary>
    /// <param name="s">Pass a valid String value.</param>
    public virtual void SetTopMargin(String s)
    {
      marginTop = s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGINTAG, sb, response);
      if (sOnActivate != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onactivate attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(ONACTIVATE_ATTR, sb, response);
        WriteNugget(sOnActivate, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnAfterPrint != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onafterprint attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONAFTERPRINT_ATTR, sb, response);
        WriteNugget(sOnAfterPrint, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnBeforeActivate != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onbeforeactivate attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONBEFOREACTIVATE_ATTR, sb, response);
        WriteNugget(sOnBeforeActivate, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnBeforeUnload != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onbeforeunload attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONBEFOREUNLOAD_ATTR, sb, response);
        WriteNugget(sOnBeforeUnload, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnBlur != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onblur attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONBLUR_ATTR, sb, response);
        WriteNugget(sOnBlur, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnControlSet != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the oncontrolset attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONCONTROLSET_ATTR, sb, response);
        WriteNugget(sOnControlSet, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnDeactivate != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the ondeactivate attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONDEACTIVATE_ATTR, sb, response);
        WriteNugget(sOnDeactivate, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnError != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onerror attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONERROR_ATTR, sb, response);
        WriteNugget(sOnError, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnFocus != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onfocus attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONFOCUS_ATTR, sb, response);
        WriteNugget(sOnFocus, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnHelp != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onhelp attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONHELP_ATTR, sb, response);
        WriteNugget(sOnHelp, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnLoad != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onload attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONLOAD_ATTR, sb, response);
        WriteNugget(sOnLoad, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnResize != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onresize attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONRESIZE_ATTR, sb, response);
        WriteNugget(sOnResize, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnResizeEnd != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onresizeend attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONRESIZEEND_ATTR, sb, response);
        WriteNugget(sOnResizeEnd, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnResizeStart != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onresizestart attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONRESIZESTART_ATTR, sb, response);
        WriteNugget(sOnResizeStart, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sOnUnload != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onunload attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                	/*Exception e = new Exception();
        //                	StackTraceElement[] elts = e.getStackTrace();
        //                	String stackMsg = "";
        //                	for (int i = 0; i < elts.length; ++i)
        //                	{
        //                		stackMsg += elts[i].toString();
        //                		stackMsg += "\n";
        //                	}
        //                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
        //                }
        WriteNugget(HTMLBody.ONUNLOAD_ATTR, sb, response);
        WriteNugget(sOnUnload, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (marginWidth != null)
      {
        WriteNugget(MARGINWIDTH_ATTR, sb, response);
        WriteNugget(marginWidth, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (marginHeight != null)
      {
        WriteNugget(MARGINHEIGHT_ATTR, sb, response);
        WriteNugget(marginHeight, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (marginLeft != null)
      {
        WriteNugget(LEFTMARGIN_ATTR, sb, response);
        WriteNugget(marginLeft, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (marginTop != null)
      {
        WriteNugget(TOPMARGIN_ATTR, sb, response);
        WriteNugget(marginTop, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(ENDTAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = BODY + iNextID;
      iNextID++;
      return s;
    }
  }
}