using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// Use this class to render an html br element.
  /// </summary>
  public class HTMLBr : HTMLElement
  {
    internal static int iNextID = 0;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<br");
    
    /// <summary>
    /// Creates a new HTMLBr object.
    /// </summary>
    public HTMLBr()
    {
    }
    
    /// <summary>
    /// Clients may not nest html with a br element.
    /// </summary>
    /// <param name="e">HTMLElement</param>
    /// <throws>HTMLException This method will throw an exception</throws>
    public override void AddInnerHTMLElement(HTMLElement e)
    {
      throw new HTMLException("You may not nest html within a <br> element.");
      }
    
    /// <summary>
    /// Clients may not nest html with a br element.
    /// </summary>
    /// <param name="s">String</param>
    /// <throws>HTMLException This method will throw an exception</throws>
    public override void AddInnerHTMLString(String s)
    {
      throw new HTMLException("You may not nest html within a <br> element.");
      }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    /// <throws>HTMLException</throws>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
    }
    //WriteNugget(END_TAG, sb, response);
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean false.</returns>
    protected override bool ProvidesClosingTag()
    {
      return false;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "br" + iNextID;
      iNextID++;
      return s;
    }
  }
}