using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// Use this class to render an html comment.
  /// </summary>
  public class HTMLComment : HTMLElement
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.xpshared.htmlelements.HTMLComment");
    internal static int iNextID = 0;
    
    /// <summary>
    /// Creates a new HTMLComment object.
    /// </summary>
    /// <param name="s">Pass the String comment.</param>
    public HTMLComment(String s)
    {
      try
      {
        base.AddInnerHTMLString(s);
      }
      catch (HTMLException he)
      {
        log.Error(he, "Unable to add comment string to parent class.");
      }
    }
    
    /// <summary>
    /// HTMLElement.InternalWrite rewritten for HTMLComment.
    /// </summary>
    /// <param name="bFormat">
    /// Pass a boolean indicating whether or not to format the
    /// output.
    /// </param>
    /// <param name="sbLocal">
    /// Pass an XPStringBuilder to which you wish the String
    /// appended.  Pass null if you do not wish the string appended to
    /// an XPStringBuilder.
    /// </param>
    /// <param name="response">
    /// Pass an IXPResponse to which you wish the String written.
    /// Pass null if you do not wish the string written to an
    /// IXPResponse.
    /// </param>
    /// <throws>HTMLException</throws>
    protected override void InternalWrite(bool bFormat, XPStringBuilder sbLocal,
     IXPResponse response)
    {
      bool bMethodFormat = bFormat;
      if (bForceFormattingOff)
      {
        bMethodFormat = false;
      }
      if (bMethodFormat)
      {
        appendTabs(sbLocal, response);
      }
      //   concrete subclasses implement the following
      AppendBeginningOfOpeningTag(bMethodFormat, sbLocal, response);
      GetInnerHTML(sbLocal, response, false);
      //   append the closing tag
      AppendClosingTag(bMethodFormat, sbLocal, response);
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    /// <throws>HTMLException</throws>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(CommonHTMLStrings.BEGIN_COMMENT, sb, response);
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(CommonHTMLStrings.END_COMMENT, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean false.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "comment" + iNextID;
      iNextID++;
      return s;
    }
  }
}