using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// This class can be used to display an HTML div element. <br />
  /// <br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/global.html#h-7.5.4">Spec</a><br /><br />
  /// </summary>
  public class HTMLDiv : HTMLElement
  {
    internal static int iNextID = 0;
    private const String DIV = "div";
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<div");
    private static readonly byte[] ONSELECTSTART_ATTR = HTMLElement.ToByteArray(" onselectstart=\"");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</div>");
    private String sOnSelectStart = null;
    
    /// <summary>
    /// Creates a new HTMLDiv object.
    /// </summary>
    public HTMLDiv()
    {
    }
    
    /// <returns></returns>
    public virtual String GetOnSelectStart()
    {
      return sOnSelectStart;
    }
    
    /// <param name="s"></param>
    public virtual void SetOnSelectStart(String s)
    {
      if (!HTMLElement.bAllowEvents)
      {
      }
      //            if (bError)
      //            {
      //                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      sOnSelectStart = s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      if (sOnSelectStart != null)
      {
        if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
        {
        }
        //                if (bError)
        //                {
        //                    myLogger.Error("You are not allowed to set the onselectstart attribute value because the client of this html tree is attempting to render a 508-compliant page.");
        //                    myLogger.Error("this: " + this);
        //                }
        WriteNugget(HTMLDiv.ONSELECTSTART_ATTR, sb, response);
        WriteNugget(sOnSelectStart, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = DIV + iNextID;
      iNextID++;
      return s;
    }
  }
}