using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <version>$Revision: #1 $</version>
  /// <author>$author$</author>
  public class HTMLEm : HTMLElement
  {
    internal static int iNextID = 0;
    private const String EM = "em";
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<em");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</em>");
    
    /// <summary>
    /// Creates a new HTMLEm object.
    /// </summary>
    public HTMLEm()
    {
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = EM + iNextID;
      iNextID++;
      return s;
    }
  }
}