using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// <br /><br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/present/frames.html#h-16.2.1">Spec</a><br /><br />
  /// </summary>
  public class HTMLFrameSet : HTMLElement
  {
    private static int iNextID = 0;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<frameset");
    private static readonly byte[] ROWS_ATTR = HTMLElement.ToByteArray(" rows=\"");
    private static readonly byte[] COLS_ATTR = HTMLElement.ToByteArray(" cols=\"");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</frameset>");
    private String sCols = null;
    private String sRows = null;
    
    /// <summary>
    /// Creates a new HTMLFrameSet object.
    /// </summary>
    public HTMLFrameSet()
    {
    }
    
    /// <summary>
    /// Get the value of the cols attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetCols()
    {
      return sCols;
    }
    
    /// <summary>
    /// Get the value of the rows attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetRows()
    {
      return sRows;
    }
    
    /// <summary>
    /// Call this method to set the value of the cols attribute.
    /// </summary>
    /// <param name="s">A valid String value.</param>
    public virtual void SetCols(String s)
    {
      sCols = s;
    }
    
    /// <summary>
    /// Call this method to set the value of the rows attribute.
    /// </summary>
    /// <param name="s">A valid String value.</param>
    public virtual void SetRows(String s)
    {
      sRows = s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      if (sRows != null)
      {
        WriteNugget(ROWS_ATTR, sb, response);
        WriteNugget(sRows, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sCols != null)
      {
        WriteNugget(COLS_ATTR, sb, response);
        WriteNugget(sCols, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "frameset" + iNextID;
      iNextID++;
      return s;
    }
  }
}