using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// This class can be used to represent HTML elements for which no specific
  /// class exists. <br />
  /// <br />
  /// It is far better thing to use a class specific to the element you wish  to
  /// render, than to use this class.
  /// </summary>
  public class HTMLGenericElement : HTMLElement
  {
    internal static int iNextID = 0;
    internal String sHTML;
    
    /// <summary>
    /// Call this method to create an HTMLGenericElement object.     <br /><br />
    /// </summary>
    /// <param name="s">Pass any valid HTML string.</param>
    public HTMLGenericElement(String s)
    {
      sHTML = s;
    }
    ////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////
    
    /// <summary>
    /// Call this method to set the html string for this object.
    /// </summary>
    /// <param name="s">Pass a valid html string.</param>
    public virtual void SetDisplayString(String s)
    {
      sHTML = s;
    }
    ///////////////////////////////////////////////
    //   The following method are overriden for HTMLElement, because they are
    //   meaningless for this class.
    
    /// <summary>
    /// Method rewritten to disallow calling on an HTMLGenericElement
    /// </summary>
    /// <param name="s">String</param>
    /// <throws>HTMLException This method will throw an exception.</throws>
    public override void SetName(String s)
    {
      throw new HTMLException("You may not call SetName on this object.");
      }
    
    /// <summary>
    /// Method rewritten to disallow calling on an HTMLGenericElement
    /// </summary>
    /// <param name="s">String</param>
    /// <throws>HTMLException This method will throw an exception.</throws>
    public override void SetOnClick(String s)
    {
      throw new HTMLException("You may not call SetOnClick on this object.");
      }
    
    /// <summary>
    /// Method rewritten to disallow calling on an HTMLGenericElement
    /// </summary>
    /// <param name="s">String</param>
    /// <throws>HTMLException This method will throw an exception.</throws>
    public override void SetOnKeyDown(String s)
    {
      throw new HTMLException("You may not call SetOnKeyDown on this object.");
      }
    
    /// <summary>
    /// Method rewritten to disallow calling on an HTMLGenericElement
    /// </summary>
    /// <param name="s">String</param>
    /// <throws>HTMLException This method will throw an exception.</throws>
    public override void SetOnKeyPress(String s)
    {
      throw new HTMLException("You may not call SetOnKeyPress on this object.");
      }
    
    /// <summary>
    /// Method rewritten to disallow calling on an HTMLGenericElement
    /// </summary>
    /// <param name="s">String</param>
    /// <throws>HTMLException This method will throw an exception.</throws>
    public override void SetOnMouseOut(String s)
    {
      throw new HTMLException("You may not call SetOnMouseOut on this object.");
      }
    
    /// <summary>
    /// Method rewritten to disallow calling on an HTMLGenericElement
    /// </summary>
    /// <param name="s">String</param>
    /// <throws>HTMLException This method will throw an exception.</throws>
    public override void SetOnMouseOver(String s)
    {
      throw new HTMLException("You may not call SetOnMouseOver on this object.");
      }
    
    /// <summary>
    /// Method rewritten to disallow calling on an HTMLGenericElement
    /// </summary>
    /// <param name="s">String</param>
    /// <throws>HTMLException This method will throw an exception.</throws>
    public override void SetStyle(String s)
    {
      throw new HTMLException("You may not call SetStyle on this object.");
      }
    
    /// <summary>
    /// Method rewritten to disallow calling on an HTMLGenericElement
    /// </summary>
    /// <param name="s">String</param>
    /// <throws>HTMLException This method will throw an exception.</throws>
    public override void SetStyleClass(StyleClass s)
    {
      throw new HTMLException("You may not call SetStyleClass on this object.");
      }
    
    /// <summary>
    /// Method rewritten to disallow calling on an HTMLGenericElement
    /// </summary>
    /// <param name="e">HTMLElement</param>
    /// <throws>HTMLException This method will throw an exception.</throws>
    public override void AddInnerHTMLElement(HTMLElement e)
    {
      throw new HTMLException("This class does not support adding nested HTML.");
      }
    
    /// <summary>
    /// Method rewritten to disallow calling on an HTMLGenericElement
    /// </summary>
    /// <param name="s">HTMLElement</param>
    /// <throws>HTMLException This method will throw an exception.</throws>
    public override void AddInnerHTMLString(String s)
    {
      throw new HTMLException("This class does not support adding nested HTML.");
      }
    // override HTMLElement.InternalWrite
    
    /// <summary>
    /// HTMLElement.InternalWrite() rewritten.
    /// </summary>
    /// <param name="bFormat">
    /// Pass a boolean indicating whether or not to format the
    /// output.
    /// </param>
    /// <param name="sbLocal">
    /// Pass an XPStringBuilder to which you wish the String
    /// appended.  Pass null if you do not wish the string appended to
    /// an XPStringBuilder.
    /// </param>
    /// <param name="response">
    /// Pass an IXPResponse to which you wish the String written.
    /// Pass null if you do not wish the string written to an
    /// IXPResponse.
    /// </param>
    /// <throws>HTMLException</throws>
    protected override void InternalWrite(bool bFormat, XPStringBuilder sbLocal,
     IXPResponse response)
    {
      bool bMethodFormat = bFormat;
      if (bForceFormattingOff)
      {
        bMethodFormat = false;
      }
      String sComment = GetPreComment();
      if ((sComment != null) && bPrintComments)
      {
        if (bMethodFormat)
        {
          appendTabs(sbLocal, response);
        }
        WriteNugget(CommonHTMLStrings.BEGIN_COMMENT, sbLocal, response);
        WriteNugget(sComment, sbLocal, response);
        WriteNugget(CommonHTMLStrings.END_COMMENT, sbLocal, response);
        if (bMethodFormat)
        {
          WriteNugget(sNewLine, sbLocal, response);
        }
      }
      if (bMethodFormat)
      {
        appendTabs(sbLocal, response);
      }
      WriteNugget(sHTML, sbLocal, response);
      if (bMethodFormat)
      {
        WriteNugget(sNewLine, sbLocal, response);
      }
      sComment = GetPostComment();
      if ((sComment != null) && bPrintComments)
      {
        if (bMethodFormat)
        {
          appendTabs(sbLocal, response);
        }
        WriteNugget(CommonHTMLStrings.BEGIN_COMMENT, sbLocal, response);
        WriteNugget(sComment, sbLocal, response);
        WriteNugget(CommonHTMLStrings.END_COMMENT, sbLocal, response);
        if (bMethodFormat)
        {
          WriteNugget(sNewLine, sbLocal, response);
        }
      }
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
    }
    //   do nothing
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
    }
    //   do nothing
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean false.</returns>
    protected override bool ProvidesClosingTag()
    {
      return false;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "generic" + iNextID;
      iNextID++;
      return s;
    }
  }
}