using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// Use this class to render an html head element.
  /// <br /><br />
  /// <a target="new" href="http://www.w3.org/TR/html4/struct/global.html#h-7.4.1">Spec</a>
  /// <br /><br />
  /// </summary>
  public class HTMLHead : HTMLElement
  {
    private HTMLTitle myTitle;
    private static byte[] BEGINTAG = HTMLElement.ToByteArray("\n\n<head");
    private static byte[] ENDTAG = HTMLElement.ToByteArray("\n</head>");
    internal static int iNextID = 0;
    //
    //    //   We share the logger for efficiency.
    //    private static XPLogger myLogger = XPLogManager.GetLogger(HTMLHead.class);
    //
    //    //   pre-determining which levels are enabled allows the jvm to avoid
    //    //   concatenating parameters of the logging methods.
    //    private static boolean bDebug = myLogger.IsDebugEnabled();
    //    private static boolean bError = myLogger.IsErrorEnabled();
    //    private static boolean bWarn  = myLogger.IsWarnEnabled();
    //    private static boolean bInfo  = myLogger.IsInfoEnabled();
    //    private static boolean bFatal = myLogger.IsFatalEnabled();
    
    /// <summary>
    /// Creates a new HTMLHead object.
    /// </summary>
    public HTMLHead()
    {
      myTitle = null;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "head" + iNextID;
      iNextID++;
      return s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGINTAG, sb, response);
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never
    /// be called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(ENDTAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag.
    /// Some elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <deprecated>Use addInnerHTMLElement( HTMLTitle )</deprecated>
    public virtual void SetTitle(HTMLTitle t)
    {
      t.SetLevel(GetLevel() + 1);
      myTitle = t;
    }
    //  OVERRIDE these methods from HTMLElement to ensure
    //  that only appropriate elements are added within this <head>.
    
    /// <summary>
    /// HTMLElement.addInnerHTMLElement rewritten.
    /// </summary>
    /// <param name="e">HTMLElement</param>
    /// <throws>
    /// HTMLException Throws an exception if e is not an HTMLScript, 
    /// HTMLLink, HTMLTitle, HTMLScriptCollection, or HTMLElementCollection 
    /// element.
    /// </throws>
    public override void AddInnerHTMLElement(HTMLElement e)
    {
      if (!(e is HTMLScript) && !(e is HTMLLink) && !(e is HTMLTitle) && !(e is HTMLScriptCollection) && !(e is HTMLMeta) && !(e is HTMLElementCollection))
      {
        throw new HTMLException("You may only add HTMLLink, HTMLMeta, HTMLScriptCollection, HTMLTitle, HTMLElementCollection, and HTMLScript elements to an HTMLHead element.");
        }
      if (e is HTMLElementCollection)
      {
      }
      //IXPEnumerator enumItems = e.GetInnerHTMLElements(false);
      //            if (enumItems != null)
      //            {
      //                HTMLElement eTemp;
      //
      //                while (enumItems.MoveNext())
      //                {
      //                    eTemp = (HTMLElement) enumItems.GetCurrent();
      //
      ////                    if (bDebug)
      ////                    {
      ////                        myLogger.Debug("eTemp: " + eTemp);
      ////                    }
      //
      //                } //   for each inner element
      //
      //            } //   got inner elements
      // e instanceof HTMLElementCollection
      base.AddInnerHTMLElement(e);
    }
    
    /// <summary>
    /// Method rewritten to throw an exception if called.  Use 
    /// addInnerHTMLElement().
    /// </summary>
    /// <param name="s">String</param>
    /// <throws>HTMLException Throws an exception.</throws>
    public override void AddInnerHTMLString(String s)
    {
      throw new HTMLException("You may only add HTMLElement objects to an HTMLHead element.");
      }
  }
}