using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// Use this class to render an html img element. <br />
  /// <br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/objects.html#h-13.2">Spec</a><br /><br />
  /// </summary>
  public class HTMLImg : HTMLElement
  {
    internal static int iNextID = 0;
    
    /// <summary>
    /// Set this to false to cause the HTMLImg object to not enforce that the
    /// alt attribute is set to a valid String.
    /// </summary>
    public static bool bEnforceAlt = false;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<img src=\"");
    private static readonly byte[] ALT_ATTR = HTMLElement.ToByteArray(" alt=\"");
    private static readonly byte[] BORDER_ATTR = HTMLElement.ToByteArray(" border=\"");
    private static readonly byte[] HEIGHT_ATTR = HTMLElement.ToByteArray(" height=\"");
    private static readonly byte[] WIDTH_ATTR = HTMLElement.ToByteArray(" width=\"");
    private static readonly byte[] ALIGN_ATTR = HTMLElement.ToByteArray(" align=\"");
    private static readonly byte[] USEMAP_ATTR = HTMLElement.ToByteArray(" usemap=\"");
    private static readonly byte[] ENDTAG = HTMLElement.ToByteArray("</img>");
    internal String sAlign = null;
    internal String sAlt;
    internal String sBorder = null;
    internal String sHeight = null;
    internal String sSrc;
    internal String sWidth = null;
    internal String sUseMap = null;
    private bool bUseDisplayOptionsBaseUrl = true;
    
    /// <param name="src">Pass the source string of this image.</param>
    /// <param name="alt">Pass the alt string of this image.</param>
    public HTMLImg(String src, String alt)
    {
      sSrc = src;
      sAlt = alt;
      sAlign = null;
    }
    
    /// <param name="src">Pass the source string of this image.</param>
    /// <param name="alt">Pass the alt string of this image.</param>
    /// <param name="height">Pass the height string of this image.</param>
    /// <param name="width">Pass the width string of this image.</param>
    public HTMLImg(String src, String alt, String height, String width)
    {
      sSrc = src;
      sAlt = alt;
      sHeight = height;
      sWidth = width;
    }
    //   to be deprectated
    
    /// <summary>
    /// Get the value of the align attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetAlign()
    {
      return sAlign;
    }
    
    /// <summary>
    /// Get the value of the alt attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetAlt()
    {
      return sAlt;
    }
    //   to be deprectated
    
    /// <summary>
    /// Get the value of the border attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetBorder()
    {
      return sBorder;
    }
    
    /// <summary>
    /// Get the value of the height attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetHeight()
    {
      return sHeight;
    }
    public virtual bool GetUseDisplayOptionsBaseUrl()
    {
      return bUseDisplayOptionsBaseUrl;
    }
    
    /// <summary>
    /// Get the value of the width attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetWidth()
    {
      return sWidth;
    }
    
    /// <summary>
    /// Get the value of the usemap attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public virtual String GetUseMap()
    {
      return sUseMap;
    }
    //   to be deprectated
    
    /// <summary>
    /// Set the value of the align atribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetAlign(String s)
    {
      sAlign = s;
    }
    
    /// <summary>
    /// Set the value of the alt atribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetAlt(String s)
    {
      sAlt = s;
    }
    //   to be deprectated
    
    /// <summary>
    /// Set the value of the border atribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetBorder(String s)
    {
      sBorder = s;
    }
    
    /// <summary>
    /// Set the value of the height atribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetHeight(String s)
    {
      sHeight = s;
    }
    public virtual void SetUseDisplayOptionsBaseUrl(bool b)
    {
      bUseDisplayOptionsBaseUrl = b;
    }
    
    /// <summary>
    /// Set the value of the width atribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetWidth(String s)
    {
      sWidth = s;
    }
    
    /// <summary>
    /// Set the value of the usemap atribute.
    /// </summary>
    /// <param name="s">String value.</param>
    public virtual void SetUseMap(String s)
    {
      sUseMap = s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    /// <throws>HTMLException</throws>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      if (!bUseDisplayOptionsBaseUrl)
      {
        //            if (bDebug)
        //            {
        //                myLogger.Debug("Not using display options base url.");
        //            }
        WriteNugget(sSrc, sb, response);
      } else
      {
        //            if (bDebug)
        //            {
        //                myLogger.Debug("Using display root base url.");
        //            }
        //	we're using the display options base url
        DisplayOptions options = this.GetDisplayOptions();
        String sBaseUrl = null;
        if (options != null)
        {
          sBaseUrl = options.GetImgBaseUrl();
        }
        //
        //            if (bDebug)
        //            {
        //                myLogger.Debug("sBaseUrl: " + sBaseUrl);
        //            }
        if (sBaseUrl == null)
        {
          //                if (bError)
          //                {
          //                    myLogger.Error("The base URL for images must be set on the root display object.");
          //                    myLogger.Error("eDisplayRoot: " + eDisplayRoot);
          //                    myLogger.Error("sSrc: " + sSrc);
          //                }
          WriteNugget(sSrc, sb, response);
        } else
        {
          WriteNugget(sBaseUrl, sb, response);
          WriteNugget(sSrc, sb, response);
        }
      }
      WriteNugget(QUOTE, sb, response);
      if (sAlt == null)
      {
        if (bEnforceAlt)
        {
        }
      }
      //                if (bError)
      //                {
      //                    myLogger.Error("You must set the alt attribute in img with src " + sSrc + ".");
      //                }
      if (sAlt != null)
      {
        WriteNugget(ALT_ATTR, sb, response);
        WriteNugget(sAlt, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sBorder != null)
      {
        WriteNugget(BORDER_ATTR, sb, response);
        WriteNugget(sBorder, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sAlign != null)
      {
        WriteNugget(ALIGN_ATTR, sb, response);
        WriteNugget(sAlign, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sHeight != null)
      {
        WriteNugget(HEIGHT_ATTR, sb, response);
        WriteNugget(sHeight, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sWidth != null)
      {
        WriteNugget(WIDTH_ATTR, sb, response);
        WriteNugget(sWidth, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sUseMap != null)
      {
        WriteNugget(USEMAP_ATTR, sb, response);
        WriteNugget(sUseMap, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    public virtual String GetSrc()
    {
      if (!bUseDisplayOptionsBaseUrl)
      {
        return sSrc;
      } else
      {
        //	we're using the display options base url
        DisplayOptions options = this.GetDisplayOptions();
        String sBaseUrl = null;
        if (options != null)
        {
          sBaseUrl = options.GetImgBaseUrl();
        }
        if (sBaseUrl == null)
        {
          return sSrc;
        } else
        {
          return sBaseUrl + sSrc;
        }
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(ENDTAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean false.</returns>
    protected override bool ProvidesClosingTag()
    {
      return false;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "img" + iNextID;
      iNextID++;
      return s;
    }
  }
}