using System;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// This class contains constants identifying the various types of INPUT
  /// element.  These constants can be used to instantiate HTMLInput type
  /// objects. <br /><br />
  /// </summary>
  /// <seealso cref="HTMLInput#HTMLInput(int,String,String,String)"></seealso>
  public class HTMLInputTypes
  {
    public const int BUTTON = 1;
    public const int CHECKBOX = 2;
    public const int FILE = 3;
    public const int HIDDEN = 4;
    public const int IMAGE = 5;
    public const int PASSWORD = 6;
    public const int RADIO = 7;
    public const int RESET = 8;
    public const int SUBMIT = 9;
    public const int TEXT = 10;
    public static readonly HTMLInputTypes BUTTON_TYPE = new HTMLInputTypes("button",
     BUTTON);
    public static readonly HTMLInputTypes CHECKBOX_TYPE = new HTMLInputTypes("checkbox",
     CHECKBOX);
    public static readonly HTMLInputTypes FILE_TYPE = new HTMLInputTypes("file",
     FILE);
    public static readonly HTMLInputTypes HIDDEN_TYPE = new HTMLInputTypes("hidden",
     HIDDEN);
    public static readonly HTMLInputTypes IMAGE_TYPE = new HTMLInputTypes("image",
     IMAGE);
    public static readonly HTMLInputTypes PASSWORD_TYPE = new HTMLInputTypes("password",
     PASSWORD);
    public static readonly HTMLInputTypes RADIO_TYPE = new HTMLInputTypes("radio",
     RADIO);
    public static readonly HTMLInputTypes RESET_TYPE = new HTMLInputTypes("reset",
     RESET);
    public static readonly HTMLInputTypes SUBMIT_TYPE = new HTMLInputTypes("submit",
     SUBMIT);
    public static readonly HTMLInputTypes TEXT_TYPE = new HTMLInputTypes("text",
     TEXT);
    private String sTypeName = null;
    private int iType;
    //  this class shouldn't be constructed.
    
    /// <summary>
    /// Creates a new HTMLInputTypes object.
    /// </summary>
    /// <param name="s">String type name</param>
    /// <param name="i">int type int.</param>
    private HTMLInputTypes(String s, int i)
    {
      sTypeName = s;
      iType = i;
    }
    
    /// <summary>
    /// Get the int representation of this type.
    /// </summary>
    /// <returns>int</returns>
    public virtual int ToInt()
    {
      return iType;
    }
    
    /// <summary>
    /// Get the string representation of this type.
    /// </summary>
    /// <returns>String</returns>
    public virtual String ToString()
    {
      return sTypeName;
    }
    
    /// <summary>
    /// Call to convert an int type value to a String.
    /// </summary>
    /// <param name="iType">int type respresentation</param>
    /// <returns>String representation</returns>
    public static String ToString(int iType)
    {
      String sTemp = null;
      switch (iType){
      case HTMLInputTypes.TEXT:
      sTemp = "text";
      break;
      case HTMLInputTypes.SUBMIT:
      sTemp = "submit";
      break;
      case HTMLInputTypes.BUTTON:
      sTemp = "button";
      break;
      case HTMLInputTypes.CHECKBOX:
      sTemp = "checkbox";
      break;
      case HTMLInputTypes.FILE:
      sTemp = "file";
      break;
      case HTMLInputTypes.HIDDEN:
      sTemp = "hidden";
      break;
      case HTMLInputTypes.IMAGE:
      sTemp = "image";
      break;
      case HTMLInputTypes.PASSWORD:
      sTemp = "password";
      break;
      case HTMLInputTypes.RADIO:
      sTemp = "radio";
      break;
      case HTMLInputTypes.RESET:
      sTemp = "reset";
      break;
      }
      return sTemp;
    }
  }
}