using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// Title:        HTMLList Description: Copyright:    Copyright (c) 2002
  /// Company:      Plumtree Software
  /// </summary>
  /// <version>1.0</version>
  /// <author>Chris Bucchere</author>
  public class HTMLList : HTMLElement
  {
    internal static int iNextID = 0;
    private int m_listType;
    
    /// <summary>
    /// Creates a new HTMLList object.
    /// </summary>
    /// <param name="listType"></param>
    /// <throws>HTMLException</throws>
    public HTMLList(int listType)
    {
      m_listType = listType;
    }
    
    /// <summary>
    /// Obtain an int indicating which type of list this is.  See HTMLListTypes
    /// class
    /// </summary>
    /// <returns>int type</returns>
    public virtual int GetListType()
    {
      return m_listType;
    }
    
    /// <param name="e"></param>
    /// <throws>HTMLException</throws>
    public override void AddInnerHTMLElement(HTMLElement e)
    {
      if (!(e is HTMLListItem))
      {
        throw new HTMLException("You can only and HTMLListItem object to HTMLList objects.");
        }
      base.AddInnerHTMLElement(e);
      HTMLListItem it = (HTMLListItem) e;
      it.SetParentList(this);
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(HTMLListTypes.GetListOpeningTag(m_listType), sb, response);
      }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(HTMLListTypes.GetListClosingTag(m_listType), sb, response);
      }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "list" + iNextID;
      iNextID++;
      return s;
    }
  }
}