using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// Title:        HTMLListItem Description: Copyright:    Copyright (c) 2002
  /// Company:      Plumtree Software
  /// </summary>
  /// <version>1.0</version>
  /// <author>Chris Bucchere</author>
  public class HTMLListItem : HTMLElement
  {
    internal static int iNextID = 0;
    private HTMLList m_parentList;
    private bool m_isDefinition;
    
    /// <summary>
    /// Creates a new HTMLListItem object.
    /// </summary>
    /// <throws>HTMLException</throws>
    public HTMLListItem()
    {
      m_isDefinition = false;
    }
    public virtual bool IsDefinition()
    {
      return m_isDefinition;
    }
    
    /// <param name="parentList"></param>
    public virtual void SetParentList(HTMLList parentList)
    {
      m_parentList = parentList;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(HTMLListTypes.GetListItemOpeningTag(m_parentList.GetListType(),
       m_isDefinition), sb, response);
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(HTMLListTypes.GetListItemClosingTag(m_parentList.GetListType(),
       m_isDefinition), sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "li" + iNextID;
      iNextID++;
      return s;
    }
  }
}