using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// Use this class to render an html map element. <br />
  /// <br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/objects.html#h-13.2">Spec</a><br /><br />
  /// </summary>
  public class HTMLMap : HTMLElement
  {
    internal static int iNextID = 0;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<map");
    private static readonly byte[] ID_ATTR = HTMLElement.ToByteArray(" id=\"");
    private static readonly byte[] NAME_ATTR = HTMLElement.ToByteArray(" name=\"");
    private static readonly byte[] ENDTAG = HTMLElement.ToByteArray("</map>");
    internal String sName = null;
    internal String sID = null;
    
    /// <summary>
    /// Constructs a new HTMLMap
    /// </summary>
    public HTMLMap()
    {
    }
    
    /// <summary>
    /// Constructs a new HTMLMap
    /// </summary>
    /// <param name="name">Pass the name string of this map.</param>
    /// <param name="id">Pass the id string of this map.</param>
    public HTMLMap(String name, String id)
    {
      sName = name;
      sID = id;
    }
    
    /// <summary>
    /// Get the value of the id attribute.
    /// </summary>
    /// <returns>String value.</returns>
    public override String GetID()
    {
      return sID;
    }
    
    /// <summary>
    /// Get the value of the name attribute
    /// </summary>
    /// <returns>String</returns>
    public override String GetName()
    {
      return sName;
    }
    
    /// <summary>
    /// Sets the value of the id attribute.
    /// </summary>
    /// <param name="strID">The id to set</param>
    public override void SetID(String strID)
    {
      this.sID = strID;
    }
    
    /// <summary>
    /// Sets the value of the name attribute.
    /// </summary>
    /// <param name="sHref">The name to set</param>
    public override void SetName(String strName)
    {
      this.sName = strName;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    /// <throws>HTMLException</throws>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      if (sID != null)
      {
        WriteNugget(ID_ATTR, sb, response);
        WriteNugget(sID, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (sName != null)
      {
        WriteNugget(NAME_ATTR, sb, response);
        WriteNugget(sName, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(ENDTAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean false.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "img" + iNextID;
      iNextID++;
      return s;
    }
  }
}