using System;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/global.html#h-7.4.4.2">See the
  /// spec</a>
  /// </summary>
  public class HTMLMeta : HTMLElement
  {
    private static readonly byte[] BEGINTAG = HTMLElement.ToByteArray("<meta");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</meta>");
    private static readonly byte[] CONTENT_ATTR = HTMLElement.ToByteArray(" content=\"");
    private static readonly byte[] HTTPEQUIV_ATTR = HTMLElement.ToByteArray(" http-equiv=\"");
    private const String SPACE = " ";
    internal static int iNextID = 0;
    private String myContent;
    private String sHttpEquiv;
    
    /// <param name="sContent">Pass the content string of this meta.</param>
    /// <throws>HTMLException</throws>
    public HTMLMeta(String sContent)
    {
      myContent = sContent;
      sHttpEquiv = null;
    }
    
    /// <returns></returns>
    public virtual String GetHTTPEquiv()
    {
      return sHttpEquiv;
    }
    
    /// <param name="s"></param>
    public virtual void SetHTTPEquiv(String s)
    {
      sHttpEquiv = s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGINTAG, sb, response);
      if (sHttpEquiv != null)
      {
        WriteNugget(HTTPEQUIV_ATTR, sb, response);
        WriteNugget(sHttpEquiv, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
      if (myContent != null)
      {
        WriteNugget(CONTENT_ATTR, sb, response);
        WriteNugget(myContent, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean false.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "meta" + iNextID;
      iNextID++;
      return s;
    }
  }
}