using System;
//import com.plumtree.openfoundation.logging.*;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// <br /><br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/interact/scripts.html#h-18.2.1">Spec</a>
  /// </summary>
  public class HTMLScript : HTMLElement
  {
    internal static int iNextID = 0;
    //	For 508 compliance we cannot allow script tags.
    
    /// <summary>
    /// Toggle this false to log errors when this object is     created. 
    /// </summary>
    public static bool bAllowScript = true;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<script");
    private static readonly byte[] TYPE_ATTR = HTMLElement.ToByteArray(" type=\"");
    private static readonly byte[] SRC_ATTR = HTMLElement.ToByteArray(" src=\"");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</script>");
    
    /// <deprecated></deprecated>
    public const String TYPE_JAVASCIPT = "text/javascript";
    public const String TYPE_JAVASCRIPT = "text/javascript";
    internal String myType;
    internal String src;
    
    /// <param name="sType">
    /// Pass the type string of this script.  See public type
    /// members.
    /// </param>
    public HTMLScript(String sType)
    {
      //        if (!bAllowScript)
      //        {
      //            if (bError)
      //            {
      //                String sDesc = "You are not allowed to use the HTMLScript class because this portal is attempting to render a 508-compliant page.";
      //
      //                Exception e = new Exception(sDesc);
      //
      //                myLogger.Error(sDesc, e);
      //            }
      //        }
      myType = sType;
      src = null;
    }
    
    /// <summary>
    /// Call this method to get the value of the src attribute.
    /// </summary>
    /// <returns>String value</returns>
    public virtual String GetSrc()
    {
      return src;
    }
    
    /// <summary>
    /// Call this method to set the src attribute value.
    /// </summary>
    /// <param name="s">String value</param>
    public virtual void SetSrc(String s)
    {
      src = s;
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
      WriteNugget(TYPE_ATTR, sb, response);
      WriteNugget(myType, sb, response);
      WriteNugget(QUOTE, sb, response);
      if (src != null)
      {
        WriteNugget(SRC_ATTR, sb, response);
        WriteNugget(src, sb, response);
        WriteNugget(QUOTE, sb, response);
      }
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "script" + iNextID;
      iNextID++;
      return s;
    }
  }
}