using System;
using com.plumtree.openlog;
using com.plumtree.openfoundation.util;
using com.plumtree.openfoundation.web;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// <br /><br />
  /// <a target="new"
  /// href="http://www.w3.org/TR/html4/struct/global.html#h-7.4.2">Spec</a>
  /// </summary>
  public class HTMLTitle : HTMLElement
  {
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
     "com.plumtree.xpshared.htmlelements.HTMLTitle");
    internal static int iNextID = 0;
    private static readonly byte[] BEGIN_TAG = HTMLElement.ToByteArray("<title");
    private static readonly byte[] END_TAG = HTMLElement.ToByteArray("</title>");
    
    /// <param name="s">Pass the title string value.</param>
    public HTMLTitle(String s)
    {
      try
      {
        base.AddInnerHTMLString(s);
      }
      catch (HTMLException he)
      {
        log.Info(he, "Unable to add HTML title to parent class.");
      }
    }
    
    /// <summary>
    /// This method generates the beginning of the opening tag.  The abstract
    /// HTMLElement superclass will provide the remainder of the opening tag,
    /// repleate with attributes common to all html elements, such as name, and
    /// id, and class.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendBeginningOfOpeningTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(BEGIN_TAG, sb, response);
    }
    
    /// <summary>
    /// This method generates the closing tag of this html element.  If the
    /// providesClosingTag method returns false, then this method will never be
    /// called.
    /// </summary>
    /// <param name="bFormat">boolean indicating whether or not to format.</param>
    /// <param name="sb">XPStringBuilder</param>
    /// <param name="response">IXPResponse</param>
    protected override void AppendClosingTag(bool bFormat, XPStringBuilder sb,
     IXPResponse response)
    {
      WriteNugget(END_TAG, sb, response);
    }
    
    /// <summary>
    /// Indicates whether or not this HTML element provides a closing tag. Some
    /// elements only provide opening tags, such as the &lt;br&gt; element.
    /// </summary>
    /// <returns>boolean true.</returns>
    protected override bool ProvidesClosingTag()
    {
      return true;
    }
    
    /// <summary>
    /// Get the next id value.  This is used by the GetClone method to ensure
    /// that cloned object have new ID attribute values.
    /// </summary>
    /// <returns>A valid id string.</returns>
    internal override String GetNextID()
    {
      String s = "title" + iNextID;
      iNextID++;
      return s;
    }
  }
}