using System;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// This class represents a style class defined in an HTML
  /// style sheet.
  /// To use a custom style class, simply create a new StyleClass
  /// instance with your custom style and pass it into the HTMLElement.
  /// <br /><br />
  /// For example, if you have an HTMLTable object, and you wish to set
  /// a custom style class, you can do this:
  /// <br /><br />
  /// <c>
  /// myTable.SetStyleClass( new StyleClass("customclass" );
  /// </c>
  /// <br /><br />
  /// The resulting html will look something like:
  /// <br /><br />
  /// &lt;table class="customclass"&gt; ...
  /// <br /><br />
  /// </summary>
  public class StyleClass
  {
    /* bug 23279 fix: allow public creation */
    protected static bool bAllowPublicCreation = true;
    protected String sClass;
    
    /// <summary>
    /// Creates a new StyleClass object.
    /// </summary>
    /// <param name="strStyleClass">The name of this style class.</param>
    /// <throws>
    /// HTMLException This method throws an exception if public creation
    /// of StyleClass objects is disallowed.
    /// </throws>
    public StyleClass(String strStyleClass)
    {
      if (!bAllowPublicCreation)
      {
        throw new HTMLException("This class may not be created publicly.  You must use a pre-instantiated StyleClass object.");
        }
      sClass = strStyleClass;
    }
    
    /// <summary>
    /// Call this method to obtain the name of this style class.
    /// </summary>
    /// <returns>String name</returns>
    public virtual String GetStyleName()
    {
      return sClass;
    }
  }
}