using System;
namespace com.plumtree.xpshared.htmlelements
{
  /// <summary>
  /// This interface is implemented by all HTML element-generating classes that
  /// generate HTML elements that may contain tab indexes.  A tab index is a
  /// property that guides the clients browser as the client attempts to tab
  /// through form elements. HTML browsers will generally tab to form elements
  /// with lower tab indexes before moving on to form elements with higher tab
  /// indexes.  Form elements with equal tab indexes will generally be tabbed
  /// through in the order by which they were sent to the browser.  Form elements
  /// with tab indexes are generally tabbed through before form elements without
  /// tab indexes. Tab indexes are set and returned as simple Java String
  /// objects.  Classes implementing this interface are responsible for verifying
  /// that the passed String does represent a valid tab index, specifically an
  /// integer between 0 and some big integer.
  /// </summary>
  public interface TabableHTMLElement
  {
    
    /// <summary>
    /// Call this method to get the value of the tabindex attribute.
    /// </summary>
    /// <returns>The String value of the tabindex attribute.</returns>
    String GetTabIndex();
    
    /// <summary>
    /// Call this method to set the value of the tabindex attribute.
    /// </summary>
    /// <param name="s">Pass the String value.</param>
    void SetTabIndex(String s);
    
    /// <summary>
    /// Call this method to remove the tab index from this HTMLElement.
    /// </summary>
    void ClearTabIndex();
  }
}