package com.plumtree.debug;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;

// Uncomment the following imports if you uncomment main() for testing:
// import com.plumtree.openkernel.factory.*;
// import com.plumtree.openkernel.config.*;

/////////////////////////////////////////////////////////////////////////////
// PTDebug class declaration
// 		Glue code to plug UI traceing into different architecture of 
//		OpenKernel's logging mechanism
//
// Note: Overhauled by D. Phipps to move to OpenFoundation XP style
// 		Location tracing added 3/18/04 DAP
//      Added to Martok and modified to use the latest openlog - 05/20/05 WilliamA
/////////////////////////////////////////////////////////////////////////////
/**
 * This class is deprecated; it exists only for backward compatibility. 
 * Please use OpenLogger and OpenLogService in com.plumtree.openlog
 * package.
 * 
 * @deprecated Do not use this class. Please use OpenLogger and OpenLogService 
 * in com.plumtree.openlog package.
 */
public final class PTDebug {

    private static XPArrayList m_pComponentLoggers = null;
        
    private static boolean m_bIsLoggerInitialized = false;
    
    /////  Private Constructor  /////
    //
    // Private constructor so no instances are created outside this class.
    // This class only has static members it never should be instantiated.
    //
    private PTDebug() 
    {
    }

    /////////////////////////////
    // Public Static Functions //

    /////  Generic Trace  /////
    // Note: All other traces functions use this at its core
    //
    public static void Trace(int nComponent, int nTraceType,
                             String strMessage, XPException ex) 
	{
		// Get the appropriate logger	
		OpenLogger logger = FindLogger( nComponent );
		
		// trace the message
        if (nTraceType == TraceType.Debug) 
        {
			if (!logger.IsDebugEnabled()) {
				return;
			}
			if (null == ex)
				logger.Debug(strMessage);
			else
				logger.Debug(ex, strMessage);
        }
        else if (nTraceType == TraceType.Info) 
        {
			if (!logger.IsInfoEnabled()) {
				return;
			}
			if (null == ex)
				logger.Info(strMessage);
			else
				logger.Info(ex, strMessage);	
		}
        else if (nTraceType == TraceType.Warn) 
        {
			if (!logger.IsWarnEnabled()) {
				return;
			}
			if (null == ex)
				logger.Warn(strMessage);
			else
				logger.Warn(ex, strMessage);
		}
        else if (nTraceType == TraceType.Error) 
        {
			if (!logger.IsErrorEnabled()) {
				return;
			}
			if (null == ex)
				logger.Error(strMessage);
			else
				logger.Error(ex, strMessage);				
		}
        else if (nTraceType == TraceType.Fatal) 
        {
			if (!logger.IsFatalEnabled()) {
				return;
			}
			if (null == ex)
				logger.Fatal(strMessage);
			else
				logger.Fatal(ex, strMessage);
        }				
		else if (nTraceType == TraceType.Action) 
		{
			if (!logger.IsActionEnabled()) {
				return;
			}
			logger.Action(strMessage);
		}
        // other tracing types are currently not supported
        // function and performance tracing only occur in javaportal 
        // and can't be done through glue code
    }
        
    /*
     * FindLogger():	Find the logger and init the array if needed
     */
    public static OpenLogger FindLogger( int nComponent )
    {
		OpenLogger logger = null;
		
		// init the logger 
		if (!m_bIsLoggerInitialized)
		{
			Init();
		}
		
		// Get the appropriate logger
		logger = (OpenLogger)m_pComponentLoggers.GetElement(nComponent);

		// If the component does not exist, just use the first one 
		// in the list (Tracing), rather than returning null.		
		if (null == logger) 
		{
			logger = (OpenLogger)m_pComponentLoggers.GetElement(Component.Tracing);			
		}
		
		return logger;    	
    }
    
	/**
	 * Init():	Loads up the component arraylist with loggers
	 * 			Also initializes the OpenKernel logger if needed
	 * 			Note that we do *not* init openkernel itself here
	 **/
    public static void Init()
    {
		if (m_bIsLoggerInitialized)
			return;
			
		// Initialize the openkernel logging service
		// LogManager.GetInstance().StartBroadcaster(); 
			
		// Load up the array of named loggers corresponding to UI areas
		m_pComponentLoggers = new XPArrayList();
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Dummy, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Tracing, PTDebug.class));	
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.PlumtreeDLL, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.GadgetProviders, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.AuthSourceProviders, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.PTUtility, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.UIUtilities, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Search, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Database, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Job, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.CommonLib, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Gateway, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Transformers, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.WebDownloaderParallel, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.DataSourceProviders, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.SOAP, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Wrappers, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.WebServicesSearch, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.WebServicesDirectoryAPI, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.WebServicesPortletAPI, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.WebServicesQueryAPI, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.WebServicesQueryInterfaceAPI, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.WebServicesSessionAPI, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.UI_Infrastructure, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Portal_UI_Infrastructure, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Portal_Admin, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Portal_Browsing, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Portal_Common, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Portal_Portlets, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Portal_Devices, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Portal_UserProfile, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Portal_Providers, PTDebug.class));
		m_pComponentLoggers.Add(OpenLogService.GetLogger(UILogComponents.Unified_Cache, PTDebug.class)); 

		m_bIsLoggerInitialized = true;
    }

	/////  Trace to PTSpy with exception information (exception type and stack)  /////
	public static void Trace(int nComponent, int nTraceType, 
		Exception ex) 
	{
		if (ex == null)
			Trace(nComponent, nTraceType, "null exception", null); 
		else
			Trace(nComponent, nTraceType, new XPException(ex)); 
	}
	
	/////  Trace to PTSpy with exception information (exception type and stack)  /////
	public static void Trace(int nComponent, int nTraceType, 
		String message, Throwable ex) 
	{
		if (ex == null)
			Trace(nComponent, nTraceType, message, null); 
		else
			Trace(nComponent, nTraceType, message, new XPException(ex)); 
	}	
	
    /////  Trace to PTSpy with exception information (exception type and stack)  /////
    public static void Trace(int nComponent, int nTraceType, 
    	XPException ex) 
    {
        // Do a simple trace of the exception type //
        if (ex != null) {
            Trace(nComponent, nTraceType, ex.toString(), ex);
        }
        else {
            Trace(nComponent, nTraceType, "null exception", null );
        }
    }
      
	/////  Trace to PTSpy with message
	public static void Trace(int nComponent, int nTraceType, 
		String message) 
	{
		// Do a simple trace of the exception type //
		Trace(nComponent, nTraceType, message, null );
	}

    /**
     * Unsupported ancient api which hangs around because of old tests
	 * @deprecated
	 */
	public static void TraceStack(int nComponent, int nTraceType,
                                  String strMessage) 
	{
		// No longer supported
    }

/////  Determine if any of the global flags are set for the given component  /////
    public static boolean AreAnyFlagsSet(int nComponent, int nFlags) {
        throw new XPException(
            "Method PTDebug.AreAnyFlagsSet() is not supported.");
    }

/////  Determine if all of the global flags are set for the given component  /////
    public static boolean AreAllFlagsSet(int nComponent, int nFlags) {
        throw new XPException(
            "Method PTDebug.AreAllFlagsSet() is not supported.");
    }

/////  Determines if extra debug code (checking parameters) should be executed for the component  /////
    public static boolean IsDebugEnabled(int nComponent) {
        return AreAnyFlagsSet(nComponent, TraceType.Debug);
    }

/////  Determines if debug-severity tracing is enabled for the given component  /////
    public static boolean IsDebugTracingEnabled(int nComponent) {
		OpenLogger logger = FindLogger( nComponent );    	
        return logger.IsDebugEnabled(); 
    }

/////  Determines if info-severity tracing is enabled for the given component  /////
    public static boolean IsInfoTracingEnabled(int nComponent) {
		OpenLogger logger = FindLogger( nComponent );
        return logger.IsInfoEnabled();
    }

/////  Determines if warn-severity tracing is enabled for the given component  /////
    public static boolean IsWarnTracingEnabled(int nComponent) {
		OpenLogger logger = FindLogger( nComponent );    	
        return logger.IsWarnEnabled();
    }

/////  Determines if error-severity tracing is enabled for the given component  /////
    public static boolean IsErrorTracingEnabled(int nComponent) {
		OpenLogger logger = FindLogger( nComponent );    	
        return logger.IsErrorEnabled();
    }

/////  Determines if fatal-severity tracing is enabled for the given component  /////
    public static boolean IsFatalTracingEnabled(int nComponent) {
		OpenLogger logger = FindLogger( nComponent );    	
        return logger.IsFatalEnabled();
    }

/////  Determines if function tracing is enabled for the given component  /////
    public static boolean IsFunctionTracingEnabled(int nComponent) {
		OpenLogger logger = FindLogger( nComponent );    	
        return logger.IsFunctionEnabled();
    }

/////  Determines if performance tracing is enabled for the given component  /////
    public static boolean IsPerformanceTracingEnabled(int nComponent) {
		OpenLogger logger = FindLogger( nComponent );    	
        return logger.IsPerformanceEnabled();
    }

/////  Determines if action tracing is enabled for the given component  /////
	  public static boolean IsActionTracingEnabled(int nComponent) {
		OpenLogger logger = FindLogger( nComponent );	
		return logger.IsActionEnabled();
	  }
/////  Gets all the flags for the component (the actual DWORD stored for the component is returned)  /////
    public static int GetComponentFlags(int nComponent) {
        throw new XPException(
            "Method PTDebug.GetComponentFlags() is not supported.");
    }

/////  Sets the component's flags to the new value (the actual DWORD for the component is replaced with the new value)  /////
    public static void SetComponentFlags(int nComponent, int nNewFlags) {
        throw new XPException(
            "Method PTDebug.SetComponentFlags() is not supported.");
    }
    
//	returns the stack trace for this exception
	public static String GetStack( XPException e )
	{
		return e.GetStackTrace();
	}

	/////  Returns the stack given a throwable object  /////
	public static String GetStack (Throwable e)
	{
		return XPException.GetInstance(e).GetStackTrace();
	}

	/*
	public static void main(String[] args) {

		// initialize OK        
		String PATH_SEPARATOR = XPSystem.GetDirectorySeparator();
		// create the path to serverconfig.xml's directory
		String strPTHome = XPEnvironment.GetEnvironmentVariable("PT_HOME");
		String strPlumtreeSettingsDir = strPTHome + PATH_SEPARATOR + "settings" + PATH_SEPARATOR;

		//MES- Initialize OpenKernel, telling OpenConfig where to get it's stuff
		IOKContext configContext = OKConfigFactory.createInstance(strPlumtreeSettingsDir, "portal");
	    	
		//MES- Set up the log service
		//	NOTE: We are using the "backwards compatible" mode to do this
		OpenLogService.initialize(configContext);

		PTDebug.Trace(Component.Portal_UI_Infrastructure,TraceType.Info,"Info Test");

		PTDebug.Trace(Component.UIUtilities,TraceType.Error,new XPException("Exception Stack Trace Test"));
	}
	*/
}
