package com.plumtree.uiinfrastructure.activityspace;

/**
 * Constants used as arguments in the query string or in form POSTS.
 * 
 * @author Don Hayler
 */
public final class ASConstants
{
	// The _INPUT versions of constants are for use as hidden inputs
	// within forms The normal versions are for URLS.

	// All POSTS and GETS must include AActivitySpace information

	/** space name */
	public static final String SPACENAME	   = "space";

	/** space name */
	public static final String SPACENAME_INPUT = "in_hi_space";

	/**
	 * Include the space ID if you want to access a specific instance of a space (optional)
	 * If you do not include this, a new space will be created.
	 */
	public static final String SPACEID		 = "spaceID";

	/**
	 * Include the space ID if you want to access a specific instance of a space (optional)
	 * If you do not include this, a new space will be created.
	 */
	public static final String SPACEID_INPUT = "in_hi_spaceID";

	/**
	 * If this is set to 'true' then the cache will be searched for the named space
	 * and a new one will be created if it cannot be found.  This should not be used
	 * with SPACEID.
	 */
	public static final String CACHED	    = "cached";

	/** cached or not */
	public static final String CACHED_INPUT = "in_hi_cached";

	/** Optional arguments on QS/forms */

	/** name of the control */
	public static final String CONTROL				 = "control";

	/** name of the control */
	public static final String CONTROL_INPUT		 = "in_hi_control";

	/** name of the display page */
	public static final String PAGE					 = "page";

	/** name of the display page */
	public static final String PAGE_INPUT			 = "in_hi_page";

	/** subspace key */
	public static final String SUBSPACE				 = "subspace";

	/** subspace key */
	public static final String SUBSPACE_INPUT		 = "in_hi_subspace";

	/** parent space name */
	public static final String PARENTSPACENAME		 = "parentname";

	/** parent space name */
	public static final String PARENTSPACENAME_INPUT = "in_hi_parentname";

	/** parent space id */
	public static final String PARENTSPACEID	   = "parentid";

	/** parent space id */
	public static final String PARENTSPACEID_INPUT = "in_hi_parentid";

	/** User session ID input */
	public static final String USERID_INPUT = "in_hi_userid";
	
	/** 
	 * JF- QS param for the URL to redirect to after login 
	 * (usually we don't need this because we do internal redirects but in mode 1, 
	 * we might have to do a 302 redirect to the login page and we don't want to loose
	 * the information on where to redirect after that).
	 */
	public static final String REDIRECT_AFTER_LOGIN = "redirect";
	
	public static final String REDIRECT_AFTER_LOGIN_INPUT = "in_hi_redirect";

	/** 
	 * DAP- Secondary QS param for the URL to redirect to after login 
	 * required to handle complex cases involving nested PEI plugins with
	 * custom guest user login pages, or multiple (pseudo) guest user login pages.
	 */
	public static final String REDIRECT_AFTER_LOGIN_SECONDARY = "redirect2";
	
	/**
	 * This parameter is for use by network monitoring tools and tells
	 * the Interpreter to clean up the HTTPSession after the request
	 * has completed.  This means it will remove everything cached on
	 * the HTTPSession to minimize the impact of network monitoring on
	 * memory usage.  This does not work for gateway requests or requests
	 * that fail in certain ways.
	 */
	public static final String CLEARSESSION = "clearsession";

	/**
	 * Key that the cache object is stored under in the user's sandbox session
	 * AActivitySpace.GetSubSession()
	 */
	public static final String SANDBOXASCACHE = "SandboxASCache";

	/**
	 * In 508 mode the AActivitySpace will handle views differently.  When GetView(name)
	 * is called, the space will first attempt to find the view (name + _D).  It will return
	 * that if it exists, otherwise it will return the normal view, which will be responsible
	 * for displaying 508 HTML.
	 *
	 * The _D views will be only for 508 mode.
	 *
	 * @see com.plumtree.uiinfrastructure.constants.AccessStyles.SECTION_508
	 */
	public static final String STR_508_VIEW_SUFFIX = "_D";

	/**
	 * The full request URL of the previous request is stored using this
	 * key on the sandbox to allow pages to make links back to the previous page.
	 * Whether or not the request was a POST request is also stored.
	 * 
	 * This information is used in HTMLConstructHelpers.GetPreviousSpaceURL().
	 */
	public static final String PREVIOUS_REQUEST_URL = "PreviousRequestURL";
	public static final String PREVIOUS_REQUEST_IS_POST = "PreviousRequestIsPost";
}