package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * Stores an Activityspace URL. Should be used in model components when creating URLs.
 * Can easily be converted to ASURL in the view layer.
 * 
 * @author robertz
 * Created on Oct 16, 2002
 */
public class ASURLContainer
{
	private AActivitySpace m_asOwner = null;
	private PTButtonType m_buttonType = null;
	private String m_aspaceID		  = null;
	private String m_aspaceName		  = null;
	private String m_ctrlName = null;
	private String m_pageName = null;
	private String m_urlName  = null;
	private XPStringBuilder m_buf = null;

	/**
	 * Constructor
	 */
	public ASURLContainer(String aspaceName, String ctrlName, String pageName,
		AActivitySpace parent)
	{
		m_aspaceName     = aspaceName;

		//m_aspaceID		= aspaceID;
		m_ctrlName     = ctrlName;
		m_pageName     = pageName;
		m_asOwner	   = parent;
	}

	/**
	 * Constructor
	 */
	public ASURLContainer(String aspaceName, String ctrlName,
		AActivitySpace parent)
	{
		m_aspaceName     = aspaceName;

		//m_aspaceID		= aspaceID;
		m_ctrlName     = ctrlName;
		m_asOwner	   = parent;
	}

	/**
	 * Constructor
	 */
	public ASURLContainer(String aspaceName, AActivitySpace parent)
	{
		m_aspaceName     = aspaceName;

		//m_aspaceID		= aspaceID;
		m_asOwner = parent;
	}

	/**
	 * Add additional Querystring parameter (one that is
	 * not activityspace, control or page).
	 *
	 */
	public void AddQSPair(String paramName, String paramValue)
	{
		if (null == m_buf)
		{
			m_buf = new XPStringBuilder(20);
		}

		// Make enough space in the buffer for the new name value pair. 
		m_buf.EnsureCapacity(m_buf.GetLength() + paramName.length() +
			paramValue.length() + 2);
		m_buf.Append("&" + paramName + "=" + paramValue);
	}

	/**
	 * Returns the m_asOwner.
	 * @return AActivitySpace the parent
	 */
	public AActivitySpace GetASParent()
	{
		return m_asOwner;
	}

	/**
	 * Returns the m_aspaceName.
	 * @return String
	 */
	public String GetASSpace()
	{
		if (null == m_aspaceName)
		{
			return "";
		}

		return m_aspaceName;
	}

	/**
	 * Returns the m_aspaceName.
	 * @return String
	 */
	public String GetASSpaceID()
	{
		if (null == m_aspaceID)
		{
			return "";
		}

		return m_aspaceID;
	}

	/**
	 * Returns the m_ctrlName.
	 * @return String
	 */
	public String GetControl()
	{
		if (null == m_ctrlName)
		{
			return "";
		}

		return m_ctrlName;
	}

	/**
	 * Gets the button type, if there is one
	 * @return button type
	 */
	public PTButtonType GetImageType()
	{
		return m_buttonType;
	}

	/**
	 * @return Get all non-common activityspace paramters as a string. Empty string
	 * if nothing is set. Doesn't contain a leading '&'.
	 */
	public String GetNonASQSPairs()
	{
		if (null == m_buf)
		{
			return "";
		}

		return m_buf.ToString();
	}

	/**
	 * Returns the m_pageName.
	 * @return String
	 */
	public String GetPage()
	{
		if (null == m_pageName)
		{
			return "";
		}

		return m_pageName;
	}

	/**
	 * Pretty messy/inefficent, but coulnd't think of a better way
	 *
	 *
	 * @return Value for the given parameter name. null if nothing is found.
	 *
	 */
	public String GetQSValue(String paramName)
	{
		String str = m_buf.ToString();

		// Make tokens of every parameter
		XPStringTokenizer tokens = new XPStringTokenizer(str, "&");

		// Look in every token
		for (int i = 0; i < tokens.GetTokenCount(); i++)
		{
			String tmp = tokens.GetNextToken();

			if (-1 != tmp.indexOf(paramName + "="))
			{
				// The value is everything after the '=' character.
				return tmp.substring((tmp.indexOf("=") + 1));
			}
		}

		return null;
	}

	/**
	 * @return Get the entire URL as a string, including common activityspace parameters.
	 * Doesn't contain a leading '?'.
	 */
	public String GetURLAsQS()
	{
		XPStringBuilder buf = new XPStringBuilder();

		if (null != m_aspaceName)
		{
			buf.EnsureCapacity(13 + m_aspaceName.length());
			buf.Append(ASConstants.SPACENAME + "=" + m_aspaceName);
		}

		if (null != m_aspaceID)
		{
			buf.EnsureCapacity(12 + m_aspaceID.length());
			buf.Append(ASConstants.SPACEID + "=" + m_aspaceID);
		}

		if (null != m_ctrlName)
		{
			buf.EnsureCapacity(16 + m_aspaceName.length());
			buf.Append("&" + ASConstants.CONTROL + "=" + m_ctrlName);
		}

		if (null != m_pageName)
		{
			buf.EnsureCapacity(12 + m_pageName.length());
			buf.Append("&" + ASConstants.PAGE + "=" + m_pageName);
		}

		if (null != m_buf)
		{
			m_buf.EnsureCapacity(m_buf.GetLength() + 1);
			buf.Append(m_buf.ToString());
		}

		return buf.ToString();
	}

	/**
	 * Returns the m_urlName.
	 * @return String
	 */
	public String GetURLName()
	{
		if (null == m_urlName)
		{
			return "";
		}

		return m_urlName;
	}

	/**
	 * Sets the m_aspaceName.
	 * @param aspaceName The m_aspaceName to set
	 */
	public void SetASSpace(String aspaceName)
	{
		this.m_aspaceName = aspaceName;
	}

	/**
	 * Sets the m_aspaceName.
	 * @param aspaceName The m_aspaceName to set
	 */
	public void SetASSpaceID(String aspaceID)
	{
		this.m_aspaceID = aspaceID;
	}

	/**
	 * Sets the button type
	 * @return button type
	 */
	public void SetButtonType(PTButtonType type)
	{
		this.m_buttonType = type;
	}

	/**
	 * Sets the m_ctrlName.
	 * @param ctrlName The m_ctrlName to set
	 */
	public void SetControl(String ctrlName)
	{
		this.m_ctrlName = ctrlName;
	}

	/**
	 * Sets the m_pageName.
	 * @param pageName The m_pageName to set
	 */
	public void SetPage(String pageName)
	{
		this.m_pageName = pageName;
	}

	/**
	 * Sets the m_urlName.
	 * @param urlName The m_urlName to set
	 */
	public void SetURLName(String urlName)
	{
		this.m_urlName = urlName;
	}
}

