package com.plumtree.uiinfrastructure.activityspace;

/**
 * Display Pages aggregate Views and put them into an HTMLPage.
 * They are owned by Activity Spaces and represent a view into an MVC framework.
 *
 * Display Pages are allowed to get and set data on the Models, if necessary.  Display
 * Pages are viewed as an extension of Activity Spaces, and therefore are not subject
 * to the strict data/logic/presentation separation that MVC components are.
 * 
 * @author Don Hayler
 */
public interface IDisplayPage extends IMVCObject
{
	/**
	 * Construct the HTMLPage for Display.
	 *
	 * @param IWebData The HTMLPage for display to the browser should be set on this
	 *             object.  This object should not be stored anywhere.
	 */
	public void Display(IWebData webData);

	/**
	 * Return the owning Activity Space.
	 */
	public AActivitySpace GetOwner();

	/**
	 * Initialize the Display Page.
	 *
	 * @param AActivitySpace the parent AActivitySpace of this IDisplayPage.
	 */
	public void Init(AActivitySpace parent);
}

