package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.openfoundation.web.*;

/**
 * This interface should only be implemented by controls that need to perform
 * special HTTP things, such as reading HTTP headers, getting/setting cookies,
 * and file upload.  These are special cases, and most controls should not use
 * this.
 *
 * The gateway needs access to the raw request in order to get all the necessary
 * information that is not normally passed from the interpreter to the AS. In
 * particular, gateway needs access to the raw body and raw headers.
 * 
 * @author Don Hayler
 */
public interface IHTTPControl extends IControl
{
	/**
	 * This method provides access to the XPRequest.  It will be called
	 * by the Interpreter before CheckActionSecurityAndExecute() is called.
	 * 
	 * Although these parameters will need to be cached so they can be used
	 * during CheckActionSecurityAndExecute(), they should be nulled out
	 * at the end of that method to make sure they are not leaked, as the
	 * control is stored on the HTTPSession.
	 *
	 * @param IXPRequest
	 * @param IWebData
	 */
	public void SetHTTPItems(IXPRequest request, IWebData pageData);
}

