package com.plumtree.uiinfrastructure.activityspace;

/**
 * This interface should only be implemented to provide login functionality for
 * the portal.  This is a special case.
 * 
 * @author Don Hayler
 */
public interface ILoginControl extends IControl
{
	/**
	 * This method is called by the Interpreter to determine whether or not to
	 * subsequently call GetSession and perform login/logout code.
	 *
	 * If this returns false, the session will remain unchanged.
	 *
	 * @return boolean true implies GetSession() will be called
	 */
	public boolean DoGetSession();

	/**
	 * This method is called by the Interpreter after calling Execute on the
	 * LoginControl.  The User Session returned will be stored on the user's
	 * HTTPSession.  If null is returned, the user's session will be removed
	 * from the HTTPSession for logout.
	 *
	 * @return Object null implies logoff
	 *                 session implies login as this session
	 */
	public Object GetSession();

	/**
	 * This method sets a redirect on the login control for use after a
	 * successful login.  This allows a user to be bounced from a page
	 * if they are not logged in, and then return to that page after they
	 * log in.
	 *
	 * @param Redirect the space the original URL pointed to.
	 */
	public void SetRedirectForUseAfterLogin(Redirect r);
}

