package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.openfoundation.util.XPHashtable;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author DorisL
 *
 * interface for page controls
 */
public interface IPageControl extends IMVCObject, IManagedObject {

	/**
	 * This method serves as a getter for component HTML. Sub-classes must
	 * implement this method.
	 *
	 * @return The HTML for the component itself.
	 */
	public HTMLElementCollection GetComponentHTML();
	
	/**
	 * This method returns the string used to uniquely idenify this component
	 * on a page.
	 *
	 * @return
	 */
	public String GetUniqueInstanceID();
	
	/**
	 * This abstract method stores the input submitted by the user. It should
	 * be called after a POST or GET. In the case of the date picker, it would
	 * store the date.
	 *
	 * @param _htRequestParameters A hashtable containing the request's name-
	 * 		  value pairs.
	 */
	public int StoreUserInput(XPHashtable _htRequestParameters);
	
	/**
	 * Return a ComponentMgr of the type responsisble for this PageControl.
	 * 
	 * @param _pageMgr	this PageMgr
	 * @return	a ComponentMgr
	 */
	public ComponentMgr GetComponentMgr(PageMgr _pageMgr);
	
	/**
	 * Return the name of the ComponentMgr of the type responsible for this
	 * Page Control.
	 * 
	 * @return the name of the ComponentMgr
	 */
	public String GetComponentMgrName();
}
