package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.uiinfrastructure.errordisplay.*;

/**
 * Interface for page control validators. Validators are objects for standardized
 * Page Control validation and contain both server-side and client-side validation. 
 *  
 * @author CaiusV
 */
public interface IValidator 
{

	/**
	 * Validation method. The Validation method returns a UIError object if the 
	 * object passed in fails to meet the validation criteria. Otherwise it returns
	 * null. Every class the implements IValidator should have a fully implemented
	 * Validate method for server side validation.
	 * 
	 * @param _oObject
	 * @return UIError or null
	 */
	public UIError Validate(Object _oObject);
		
	/**
	 * Return a JavaScript snippet for validating client side.
	 * 
	 * @return
	 */
	public String GetJSValidationCode();
	
	/**
	 * Return the name of the javascript function that validates client side
	 * and arguments supplied by _sParameters.
	 * 
	 * @param _sParameters areguments to Javascript function
	 * @return
	 */
	public String GetJSFunction(String _sParameters);

	/**
	 * Sets the error message for both client & server validation.
	 * 
	 * @param _strErrorMessage	the error message
	 */
	public void SetErrorMessage(String _strErrorMessage);
	
	public String GetName();
}
