package com.plumtree.uiinfrastructure.activityspace;

import com.plumtree.xpshared.htmlelements.*;

/**
 * Views are responsible for generating HTML in the MVC framework.  They query their
 * model for data and build HTML from it.  The links and forms in that HTML should be
 * constructed so the Interpreter can use the controls to process events.
 *
 * Views should encapsulate the presentation of the AActivitySpace.  As an MVC component,
 * Views are subject to the strict separation between data, logic, and presentation.
 *
 * It is a requirement that Views not contain any server code other than constants
 * (i.e. PT_PROPIDS.PT_PROPID_NAME).  IPTQueryResults should not be used in Views,
 * rather, Models should provide accesor methods for them.
 *
 * It is also a requirement that Views not depend on Models for HTML.  Views can use
 * helper classes (Strategies, delegation, etc...) to generate HTML, but Views should
 * never expect a model to return an HTMLElement or HTMLConstruct to them.  Models can
 * return text Strings for display in HTMLElements.
 *
 * Obviously, there should be a good way to refer to views without having to
 * manually type in a text string.  There should be some sort of package identifier
 * used to get these.
 * 
 * @author Don Hayler
 */
public interface IView extends IMVCObject, IManagedObject
{
	/**
	 * This method returns the HTMLElement that encompasses the output of the View.
	 */
	public HTMLElement Display();

	/**
	 * This method returns the HTMLScript that encompasses the Javascript on the page.
	 *
	 * This method should be called by IDisplayPages and the resulting Javascript should
	 * be put in the HTMLHead element.  This can return null.
	 */
	public HTMLScript DisplayJavascript();

	/**
	 * This method initializes the View with the read only model interface it is
	 * supposed to draw it's data from as well as it's parent ActivitySpace
	 * (necessary to create URLS).
	 */
	public void Init(IModelRO model, AActivitySpace parent);
}

