/*
 * Purpose:	
 * Author: 	D. Phipps Mar 22, 2005
 */
package com.plumtree.uiinfrastructure.application.notification;

import com.plumtree.openfoundation.util.XPArrayList;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.diagnostic.reporter.*;

/**
 * @author DavidP
 */
public interface IStartupNotification
{
    /**
     * Called at the beginning of the startup process.
     * 
     * @param number of tasks to execute overall
     * @return true to continue; false to halt immediately
     */
    boolean runStart(int numTasks);
    
    /**
     * Called when the startup process completes.  Always called, via finally.
     */
    void runEnd(ReturnValue initRet);
    
    /**
     * Called at the beginning of the task.  Simple start notification.
     * 
     * @param taskLoop
     * @param taskName
     * @param taskID
     * @return true to continue; false to halt immediately
     */
    boolean taskBegin( int taskLoop, String taskName, int taskID );
    
    /**
     * Called once task initialization is complete.  Returns the number of 
     * steps in this task.  The ReturnValue contains any errors or exceptions
     * that may have occurred during initialization.
     * 
     * @param taskLoop
     * @param taskName
     * @param taskID
     * @param totalNumSteps
     * @param initRet
     * @return true to continue; false to halt immediately
     */
    boolean taskInitComplete( int taskLoop, String taskName, int taskID, int totalNumSteps, ReturnValue initRet, IMessage msgs );
    
    /**
     * Called before a specific task step begins.  Simply a notifier.
     * 
     * @param taskLoop
     * @param taskName
     * @param taskID
     * @param stepLoop
     * @return true to continue; false to halt immediately
     */
    boolean taskStepBegin( int taskLoop, String taskName, int taskID, int stepLoop );
    
    /**
     * Called at the end of an individual tasks' step.  Always called, with a finally.
     * Exceptions and error return values in the ReturnValue object.
     * 
     * @param taskLoop
     * @param taskName
     * @param taskID
     * @param stepLoop
     * @param initRet
     * @return true to continue; false to halt immediately
     */
    boolean taskStepEnd( int taskLoop, String taskName, int taskID, int stepLoop, ReturnValue initRet, IMessage msgs );
    
    /**
     * Called at the end of a specific task.  Always called, with a finally.
     * The ReturnValue here contains the final result of that task: the integer
     * result code holds what the systemn feels the final result is.  Notice that 
     * an individual step could fail with error, but the overall task simply 
     * show a warning. 
     * 
     * @param taskLoop
     * @param taskName
     * @param taskID
     * @param initRet
     * @return true to continue; false to halt immediately
     */
    boolean taskEnd( int taskLoop, String taskName, int taskID, ReturnValue initRet, IMessage msgs );
    
    /**
     * Used to retrieve a comprehensive report of startup errors from the run.  
     * 
     * @return Ordered array of strings that form a comprehensive startup report
     */
    String[] getReport();
    
    /**
     * Used to retrieve a comprehensive report of startup errors from the run.  
     * 
     * @return Ordered array of eventlogs that form a comprehensive startup 
     * 			ERROR report
     */
    EventLog[] getReportEventLogs();
    
    /**
     * Used to override generated results.  Use this to leverage the diagnostic page
     * for non-diagnostic uses such as special error handling
     * 
     * @param EventLog[] eventVector: Ordered array of eventlogs that form a comprehensive startup 
     * 			ERROR report
     */
    void setReportEventLogs(EventLog[] eventVector, XPArrayList reportLog);
}
