/*
 * Created on Mar 13, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.application.startup.exceptions.*;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.util.*;

/**
 * Check the version of the JDK / .NET runtime we are on.
 * Pretty basic test.
 *  
 * @author DavidP
 */
public class CheckDevKit implements IStartupTask
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.application.startup.CheckDevKit");
    
    /**
     * Recommendations for platform minimums.
     * @author DavidP
     */
    // array of messages
    private static StartupTaskMessage m_diag[] = 
    {
        // steps are one-based; step zero is the init call
        // recommender picks error most appropriate for step, platform, errlevel            
        // DAP TODO this really needs to be in an external file
        new StartupTaskMessage( 
                0, Platform.DOTNET,PT_STARTUP_RESULTCODES.FATAL,
                ".NET Runtime Version Error", 
                "RECOMMENDATION: You have not installed the correct version of the .NET runtime " +
                "required to run the " +
                "portal.  It must be .NET Foundation 1.1 or higher."
                ),
        new StartupTaskMessage( 
                0, Platform.JAVA,PT_STARTUP_RESULTCODES.FATAL,
                "Java Runtime Version Error", 
                "RECOMMENDATION: You have not installed the correct version of the Java JRE (JDK)" +
                "required to run the portal.  It must be JDK 1.4.2 or higher."
                )
    };
    
    public String getTaskName()
    {
        // Note: Thanks to j2c limitations, can't use CheckDevKit.class.getSimpleName();
        return "CheckDevKit";
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
    public int getTaskID()
    {
        return PTStartupTasks.DEVKIT.getOrdinal();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
    public int getNumSteps()
    {
        return 1;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init()
     */
    public ReturnValue init(String _strApplicationName)
    {
        return new ReturnValue(PT_STARTUP_RESULTCODES.OK,"CheckDevKit init successful",null);
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
    public ReturnValue executeStep(int sequentialStepID)
    {
        ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"CheckDevKit successful.",null);
  	   
  		log.Info("Initializing CheckDevKit.");
 		
 		try
 		{		 	        
			// retrieve the JAVA/.NET Dev Kit version.
			String strDevKitVersion = XPSystem.GetDevKitVersion();
			AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_STR_DEV_KIT_VERSION, strDevKitVersion);
			Platform platform = Platform.getCurPlatform();
			
			if (XPSystem.IsRunningInDotNet())
			{
				// commenting this check out because 2.0 is now a requirement- CVN
				//TODO perhaps put another check in for other versions in the future? - CVN
				// output the version of .NET SDK to console.
			    //String strSDK = "The detected .NET SDK is: " + strDevKitVersion;
				
//				if (!XPStringUtility.StartsWith(strDevKitVersion, AppWarmUp.STR_SUPPORTED_DOTNET_SDK))
//				{
//					String strErrorString = "Aqualogic Interaction's UI Infrastructure does NOT support this version: " + strSDK;
//					return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,strErrorString,
//					        StackTracer.MakeXPException(strErrorString));
//				}
			}
			else 
			{
				// output the version of JAVA JDK to console.
			    String strSDK = "The detected Java JDK version is: " + strDevKitVersion;
				
				if (!(XPStringUtility.StartsWith(strDevKitVersion, AppWarmUp.STR_SUPPORTED_JAVA_JDK_1) || 
				    XPStringUtility.StartsWith(strDevKitVersion, AppWarmUp.STR_SUPPORTED_JAVA_JDK_2) ||
				    XPStringUtility.StartsWith(strDevKitVersion, AppWarmUp.STR_SUPPORTED_JAVA_JDK_3) ||
				    XPStringUtility.StartsWith(strDevKitVersion, AppWarmUp.STR_SUPPORTED_JAVA_JDK_6) ))
				{
				    //Commenting out this check per bug 64452. You can now try to run on supported java version
					// though it may not work, it logs an error -- CVN
					String strErrorString = "WebCenter Interaction's UI Infrastructure does NOT support this version: " + strSDK; 
					log.Error(strErrorString);
				    //return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,strErrorString,
					//        StackTracer.MakeXPException(strErrorString));				
				}
				
				// check for supported java version
				if (platform.isCurrentPlatformSupported() == false)
				{
				    String strErrorString = "The platform '" + platform.getName() + "' is not supported.";
					return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,strErrorString,
					        StackTracer.MakeXPException(strErrorString));							    
				}
			}
			
			String result = "CheckDevKit startup successful. Platform is '"+ platform.getName() + "'.  Fully supported.";
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,result,null);
		}
		catch( Throwable e )
		{
		    String errorMessage = "CheckDevKit Version Problem: ";
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
			        errorMessage,
			        XPException.GetInstance(e));
		}
		
		return retVal;
    }

    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
    public XPArrayList getMsg( int stepid, Platform platform, ReturnValue ret )
    {
        // Use the diagnostic helper to find the right message        
        return RecommendationHelper.getList(stepid,platform,ret.getResultCode(),m_diag);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
    public StartupTaskMessage[] getMsgArray()
    {
        return m_diag;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
    public int getNumMsgs()
    {
        return m_diag.length;
    }
}
