/*
 * Created on Apr 13, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.application.startup.exceptions.*;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.util.*;

/**
 * Loads the production activity spaces for the portal.  
 * @author DavidP
 */
public class InitializeActivitySpaces implements IStartupTask
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.application.startup.InitializeActivitySpaces");

    private String m_applicationName = "";
    private String m_strActivitySpacesFile = "";
	private String m_strLibHome = "";
	private IAppWarmUpHelper m_ptAppWarmUpHelper=null; 
    
    // array of messages
    private static StartupTaskMessage m_diag[] = 
    {
        // steps are one-based; step zero is the init call
        // DAP TODO this really needs to be in an external file
        new StartupTaskMessage( 
                0, Platform.ALL,PT_STARTUP_RESULTCODES.FATAL,
                "LoadActivitySpace: ASManager.INSTANCE.LoadClasses problem.", 
                "RECOMMENDATION: Check that ActivitySpaces.xml only refers to internal Aqualogic Interaction " +
                "activity space libraries.  Check that the libraries are available both in the " +
                "%PORTAL_HOME% library as well as the web application tree (WAR file or .NET web " +
                "app folder).  Note that the portal requires the same library be in two places " +
                "to support dynamic discovery.  Verify that ActivitySpaces.xml is not corrupt."
                                ),
        new StartupTaskMessage( 
                1, Platform.ALL,PT_STARTUP_RESULTCODES.FATAL,
                "LoadActivitySpace problem: managers will not load.", 
                "RECOMMENDATION: Check that version.xml is located in the correct directory " +
                "and is not corrupt."                )
    };    

    
    /**
     * 
     */
    public InitializeActivitySpaces()
    {
        super();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
    public String getTaskName()
    {
        return "InitializeActivitySpaces";
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
    public int getTaskID()
    {
        return PTStartupTasks.INITAS.getOrdinal();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
    public int getNumSteps()
    {
        return 2;
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
    public ReturnValue init(String _strApplicationName)
    {
    	m_applicationName = _strApplicationName;
    	return new ReturnValue(PT_STARTUP_RESULTCODES.OK,getTaskName()+" init successful",null);
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
    public ReturnValue executeStep(int sequentialStepID)
    {
		ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Incorrect step request on init",
		        StackTracer.MakeXPException("(internal stacktrace)"));
		switch( sequentialStepID )
		{
			case 0:
			    // open config file
				retVal = executeStepOne();
				break;
			case 1:
			    // load config file
				retVal = executeStepTwo();
				break;
		}
		return retVal;
    }
    
    public ReturnValue executeStepOne()
    {
        ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,getTaskName()+" step one loaded successfully",null);
	    log.Info("Initializing Activity Spaces.");

	    m_strActivitySpacesFile = XPConvert.ToString(AppWarmUp.s_htInitSettings.
	        GetElement(AppWarmUp.KEY_FILE_ACTIVITY_SPACES));
	    m_strLibHome = XPConvert.ToString(AppWarmUp.s_htInitSettings.
	        GetElement(AppWarmUp.KEY_PATH_LIB_HOME));

	    //////////////////////////
	    // INITIALIZE ASMANAGER //
	    //////////////////////////
	    
		try
		{
		    log.Info("Initializing AS with ASManager.");
		    
		    // loading activity space classes defined in ActivitySpaces.xml.
		    ASManager.INSTANCE.LoadClasses(m_strLibHome, m_strActivitySpacesFile);
		    
			log.Info("ASManager initialized.");
			
			/////////////////////////////////////////
			// INITIALIZE APPWARMUPHELPER MANAGERS //
			/////////////////////////////////////////
			
		    m_ptAppWarmUpHelper = (IAppWarmUpHelper) 
		    	AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_APP_WARMUP_HELPER);
		}
		catch (Throwable ex)
		{
		    String errorMessage = "Error initializing AS with ASManager: ";
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,errorMessage,XPException.GetInstance(ex));
			return retVal;
		}
	    
	    return retVal;
    }

    public ReturnValue executeStepTwo()
    {
        ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,getTaskName()+" step one loaded successfully",null);
	    if (null != m_ptAppWarmUpHelper)
	    {
			try
			{
			    String strConfigPath = AppWarmUp.GetSettingValueAsString(AppWarmUp.KEY_PATH_PORTAL_CONFIG);
			    log.Info("Initializing AppWarmUpHelper Managers.");
	
			    // load custom managers based on non-custom file.
			    retVal = m_ptAppWarmUpHelper.InitializeManagers(m_strLibHome, 
			        strConfigPath);
			    if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
			        log.Info("AppWarmUpHelper Managers initialized.");
			}
			catch (Throwable ex)
			{
			    String errorMessage = "Error initializing AppWarmUpHelper Managers: ";
			    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,errorMessage,XPException.GetInstance(ex));
			    return retVal;
			}
	    }
	    if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
	        log.Info("Activity Spaces initialized.");
	    return retVal;
    }

    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
    public XPArrayList getMsg( int stepid, Platform platform, ReturnValue ret )
    {
        // Use the diagnostic helper to find the right message        
        return RecommendationHelper.getList(stepid,platform,ret.getResultCode(),m_diag);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
    public StartupTaskMessage[] getMsgArray()
    {
        return m_diag;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
    public int getNumMsgs()
    {
        return m_diag.length;
    }    
}
