/*
 * Created on Mar 29, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.util.XPConvert;
import com.plumtree.openfoundation.util.XPDynamicDiscovery;
import com.plumtree.openfoundation.util.XPException;
import com.plumtree.openfoundation.xml.dom.XPNodeList;
import com.plumtree.openlog.OpenLogService;
import com.plumtree.openlog.OpenLogger;
import com.plumtree.uiinfrastructure.application.AppWarmUp;
import com.plumtree.uiinfrastructure.application.IAppWarmUpHelper;
import com.plumtree.uiinfrastructure.application.notification.IStartupTask;
import com.plumtree.uiinfrastructure.application.notification.PT_STARTUP_RESULTCODES;
import com.plumtree.uiinfrastructure.application.notification.ReturnValue;
import com.plumtree.uiinfrastructure.application.startup.exceptions.*;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.application.varpacks.ConfigVarPack;
import com.plumtree.uiinfrastructure.util.*;
import com.plumtree.uiinfrastructure.web.ApplicationManager;
import com.plumtree.uiinfrastructure.web.IApplication;
import com.plumtree.openkernel.config.*;

/**
 * @author DavidP
 */
public class InitializeAppWarmUpHelper implements IStartupTask
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.application.startup.InitializeAppWarmUpHelper");

	// OpenConfig component type
	public static final String PORTAL_APPWARMUPHELPER_COMPONENT_TYPE = "http://www.plumtree.com/config/component/types/portal/appwarmuphelper";
    
    private String m_applicationName = "";
    private IAppWarmUpHelper m_ptAppWarmUpHelper = null;
    // private XPDocument m_document = null;
	private IOKComponent m_component = null;    
    private String m_strLibHome = "";
    private String m_strAppWarmUpFile = "";
    private XPNodeList m_libFiles = null;
    private String m_strLibFileName = "";
    private ConfigVarPack m_vpConfig = null;    
    
    // array of messages
    private static StartupTaskMessage m_diag[] = 
    {
        // steps are one-based; step zero is the init call
        // DAP TODO this really needs to be in an external file
        new StartupTaskMessage( 
                0, Platform.ALL, PT_STARTUP_RESULTCODES.FATAL,
                "Unable to locate or read application warmup XML (AppWarmUpHelper setting) in portalconfig.xml", 
                "RECOMMENDATION: Check that portalconfig.xml is in the portal settings directory " +
                "and is accessible to the user running the web application, and contains the setting " +
                "for AppWarmUpHelper.  Check for security rights.  Verify that the file has the correct " +
                "default entry and is not corrupt."
                ),
        new StartupTaskMessage( 
                1, Platform.JAVA, PT_STARTUP_RESULTCODES.FATAL,
                "Unable to dynamically load AppWarmup instances", 
                "RECOMMENDATION: This usually indicates missing JAR files from the classpath. Has " +
                "your install been tampered with? Please check all relevant environment variables " +
                "and ensure that the relevant jar files (uiinfrastructure, portaluiinfrastructure) " +
                " are present in both lib/java and webapp/portal.war. " +
                "Please ensure that \"WebHome\" setting in the portalconfig.xml is configured properly."
                ),
        new StartupTaskMessage( 
                1, Platform.DOTNET, PT_STARTUP_RESULTCODES.FATAL,
                "Unable to dynamically load AppWarmup instances", 
                "RECOMMENDATION: This usually indicates missing DLL assembly files from the refpath. Has " +
                "your install been tampered with? Please check all relevant environment variables " +
                "and ensure that the relevant DLL files (uiinfrastructure, portaluiinfrastructure) are " +
                "present in both PORTAL_HOME/bin/assemblies and PORTAL_HOME/webapp/portal/web/bin. " +
				"Please ensure that \"WebHome\" setting in the portalconfig.xml is configured properly."
                ),
        new StartupTaskMessage( 
                2, Platform.JAVA, PT_STARTUP_RESULTCODES.FATAL,
                "Unable to dynamically load AppWarmup ConfigVarPack", 
                "RECOMMENDATION: This usually indicates missing JAR files from the classpath. Has " +
                "your install been tampered with? Please check all relevant environment variables " +
                "and ensure that the relevant jar files (uiinfrastructure, portaluiinfrastructure) are " +
                "present in both lib/java and webapp/portal.war.  This can also mean core XML files have " +
                "been tampered with."
                ),
        new StartupTaskMessage( 
                2, Platform.DOTNET, PT_STARTUP_RESULTCODES.FATAL,
                "Unable to dynamically load AppWarmup ConfigVarPack", 
                "RECOMMENDATION: This usually indicates missing DLL assembly files from the refpath. Has " +
                "your install been tampered with? Please check all relevant environment variables " +
                "and ensure that the relevant DLL files (uiinfrastructure, portaluiinfrastructure) are " +
                "present in both PORTAL_HOME/bin/assemblies and PORTAL_HOME/webapp/portal/web/bin. This " +
                "can also mean core XML files have been tampered with."
                ),
    };    
    /**
	 * Initializes the AppWarmUpHelper.  The AppWarmUpHelper is a class that
	 * can be implemented to perform custom initializations specific to your
	 * application.  After this point, an AppWarmUpHelper instance will be 
	 * available, as well as the final ConfigVarPack (because the 
	 * AppWarmUpHelper has the option to override the BaseConfigVarPack).
	 * 
	 * TODO: The ConfigVarPack should be a reloadable item.  We should have
	 * a method that tries to reload the ConfigVarPacks, first the Base one,
	 * and then the AppWarmUpHelper one if it exists.
     */
    public InitializeAppWarmUpHelper()
    {
        super();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
    public String getTaskName()
    {
        return "InitializeAppWarmUpHelper";
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
    public int getTaskID()
    {
        return PTStartupTasks.INITAPPWARMUP.getOrdinal();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
    public int getNumSteps()
    {
        return 3;
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
    public ReturnValue init(String _strApplicationName)
    {
    	m_applicationName = _strApplicationName;
        return new ReturnValue(PT_STARTUP_RESULTCODES.OK,getTaskName()+" init successful",null);
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
    public ReturnValue executeStep(int sequentialStepID)
    {
		ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Incorrect step request on init",
		        StackTracer.MakeXPException("(internal stacktrace)"));
		switch( sequentialStepID )
		{
			case 0:
			    // open config file
				retVal = executeStepOne();
				break;
			case 1:
			    // load config file
				retVal = executeStepTwo();
				break;
			case 2:
			    // process the loaded config file: load classes specified
				retVal = executeStepThree();
				break;
		}
		return retVal;
    }

    /**
     * Load config file
     * @return
     */
    private ReturnValue executeStepOne()
    {
        ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeAppWarmup load config file successful",null);
        
		//////////////////////////////////////////
		// CREATING INSTANCE OF APPWARMUPHELPER //
		//////////////////////////////////////////

	    m_ptAppWarmUpHelper = null;
		
		try
		{
		    log.Info("Initializing AppWarmUpHelper.");

		    // retrieving arguments to use.
		    m_strLibHome = XPConvert.ToString(AppWarmUp.s_htInitSettings.
		        GetElement(AppWarmUp.KEY_PATH_LIB_HOME));
		    if (m_strLibHome == null || m_strLibHome.length() < 1)
				return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
				        "Could not form path to appwarmup config file '"+m_strAppWarmUpFile+"': ",
				        StackTracer.MakeXPException("(internal stacktrace)"));
		    
		    try 
		    {
		        // load the AppWarmUpHelper.xml config file.

				// Use OpenConfig to read the settings
				IOKContext configContext = AppWarmUp.GetOKConfigContext();	
				m_component = configContext.getComponentByType(PORTAL_APPWARMUPHELPER_COMPONENT_TYPE);
				if (null == m_component)
				{
					throw new XPException("Cannot retrieve component type: " + PORTAL_APPWARMUPHELPER_COMPONENT_TYPE);
				}
		    }
		    catch (Throwable ex)
		    {    
				retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
						"OpenConfig setting load error",
				        XPException.GetInstance(ex));
		    }
		}
	    catch (Throwable ex)
	    {
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
			        "Could not form path to appwarmup config file '"+m_strAppWarmUpFile+"': ",
			        XPException.GetInstance(ex));
	    }
	    
	    return retVal;
    }

    /**
     * Process config file; load other app warmup instances
     * @return
     */
    private ReturnValue executeStepTwo()
    {
        ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeAppWarmup process config file successful",null);		    

        try
        {        	
		    // get the entries listed in the config file.		
			IOKSetting[] settings = m_component.querySettings();

			if ((settings == null) || 
				(settings.length == 0))
			{
				throw new XPException("Config file has no entries for AppWarmUpHelper component. ");
			}
			if (settings.length != 1)
			{
				throw new XPException("Only 1 entry allowed in config file for AppWarmUpHelper component. ");				
			}

			int nSettingType = settings[0].getDataType();
			if (nSettingType != OKConfigDataTypes.TYPE_STRING)
			{
				throw new XPException("Invalid data type for AppWarmUpHelper setting:" + nSettingType);
			}
			
			m_strLibFileName = settings[0].getStringValue();

			log.Info("Loading AppWarmUpHelper instance from library: " + m_strLibFileName);

			Object[] helperInstances = null;

			try
			{
				helperInstances = XPDynamicDiscovery.GetInstancesFromLib(
				    m_strLibHome, "uiinfrastructure", m_strLibFileName,
					"com.plumtree.uiinfrastructure.application.IAppWarmUpHelper");
			}
			catch( Throwable ex )
			{
			    String errorMessage = "Error retrieving instances from library; cannot continue without AppWarmUpHelper. ";		    
			    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
			            errorMessage,XPException.GetInstance(ex));
			    return retVal;
			}

			if (null == helperInstances)
			{
			    String errorMessage = "No AppWarmUpHelper instances found in library; cannot continue without AppWarmUpHelper. ";
			    return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
			            errorMessage,null);
			}
			if (helperInstances.length != 1)
			{
			    String errorMessage = "Only 1 AppWarmUpHelper instance allowed in library; only the first AppWarmUpHelper will be used. ";
			    return new ReturnValue(PT_STARTUP_RESULTCODES.WARN,
			            errorMessage,null);
			}
			
			m_ptAppWarmUpHelper = (IAppWarmUpHelper) helperInstances[0];
		        
			// save AppWarmUpHelper for later use.
			AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_APP_WARMUP_HELPER, m_ptAppWarmUpHelper);

			log.Info("AppWarmUpHelper initialized.");
		} 
		catch (Throwable ex)
		{				
		    String errorMessage = "Error initializing AppWarmUpHelper; cannot continue without AppWarmUpHelper. ";		    
			log.Warn(ex, errorMessage + ex.getMessage());		    
		    return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
		            errorMessage,XPException.GetInstance(ex));
		}	
		
		return retVal;
	}

    /**
     * If the app warmup helper was loaded - i.e., portaluiinfrastructure is wired up - then
     * process the instrcutions and load the related classes.
     * @return
     */
	private ReturnValue executeStepThree()
	{
        ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeAppWarmup load and process config varpack successful!",null);		    

		////////////////////////////////////////////////
		// LOADING CONFIGVARPACK FROM APPWARMUPHELPER //
		////////////////////////////////////////////////
        String strConfigPath="";
		if (null != m_ptAppWarmUpHelper)
		{
			try 
			{
			    log.Info("Overriding ConfigVarPack with AppWarmUpHelper ConfigVarPack.");
	
			    String strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.
			        GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
			    IApplication application = ApplicationManager.GetInstance().
			    	GetApplication(strApplicationName);
			    strConfigPath = XPConvert.ToString(AppWarmUp.s_htInitSettings.
			        GetElement(AppWarmUp.KEY_PATH_PORTAL_CONFIG));
			    String strPlatform = XPConvert.ToString(AppWarmUp.s_htInitSettings.
			        GetElement(AppWarmUp.KEY_STR_PLATFORM));
	
				m_vpConfig = m_ptAppWarmUpHelper.LoadConfigVarPack(
				    strConfigPath, strPlatform);		    
	
				if (m_vpConfig == null)
				{
					// Fatal if no ConfigVarPack is loaded (due to no AppWarmUpHelper or else)
					// as none of the Main URLs will be set (imageserver URL etc).
				    String errorMessage = "Fatal Error loading ConfigVarPack: ";
				    log.Info(errorMessage+"'"+strConfigPath+"': ");			    
					retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
					        errorMessage+"'"+strConfigPath+"': ",
					        null);
				}
				else
				{
				    application.GetVarPackManager().AddVariablePackage(m_vpConfig);
				    String errorMessage = "AppWarmUpHelper ConfigVarPack available; ConfigVarPack overridden.";
				    log.Info(errorMessage);
					retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,
					        errorMessage,
					        null);
				}	
			}
			catch (Throwable ex)
			{
			    String errorMessage = "Fatal Error loading ConfigVarPack: ";
				retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
				        errorMessage+"'"+strConfigPath+"': ",
				        XPException.GetInstance(ex));
			}
		}
        else
        {
            // Fatal if no ConfigVarPack is loaded (due to no AppWarmUpHelper or else)
			// as none of the Main URLs will be set (imageserver URL etc).
		    String errorMessage = "AppWarmUpHelper empty, no config varpack processing will occur, cannot continue (is this the portal?)";		    
			log.Warn(errorMessage);		    
		    return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
		            errorMessage,null);
            
        }
		
		return retVal;
    }
	
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
    public XPArrayList getMsg( int stepid, Platform platform, ReturnValue ret )
    {
        // Use the diagnostic helper to find the right message        
        return RecommendationHelper.getList(stepid,platform,ret.getResultCode(),m_diag);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
    public StartupTaskMessage[] getMsgArray()
    {
        return m_diag;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
    public int getNumMsgs()
    {
        return m_diag.length;
    }	
}
