/*
 * Created on Mar 29, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.util.XPConvert;
import com.plumtree.openfoundation.util.XPException;
import com.plumtree.openlog.OpenLogService;
import com.plumtree.openlog.OpenLogger;
import com.plumtree.uiinfrastructure.application.AppWarmUp;
import com.plumtree.uiinfrastructure.application.notification.IStartupTask;
import com.plumtree.uiinfrastructure.application.notification.PT_STARTUP_RESULTCODES;
import com.plumtree.uiinfrastructure.application.notification.ReturnValue;
import com.plumtree.uiinfrastructure.application.startup.exceptions.*;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.util.*;
import com.plumtree.uiinfrastructure.web.ApplicationManager;

/**
 * Initializes the Application.  After this point the CachingManager
 * and VarPackManager will be available, as well as the Application.
 * @author DavidP
 */
public class InitializeApplication implements IStartupTask
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.application.startup.InitializeApplication");
		
	private String m_applicationName="";
	
    // array of messages
    private static StartupTaskMessage m_diag[] = 
    {
        new StartupTaskMessage( 
                0, Platform.ALL,PT_STARTUP_RESULTCODES.FATAL,
                "Core startup initialization error.  Cannot proceed.", 
                "RECOMMENDATION: The portal libraries are not available on this machine " +
                "in the expected location.  Check your PT_HOME and PORTAL_HOME environment " +
                "variables.  Did you install the full portal on this machine?  This tool is " +
                "for verifying full portal startup."
                )
    };	

	/**
	 * 
	 */
	public InitializeApplication() {
		super();
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
	 */
	public String getTaskName() {
		return "InitializeApplication";
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
	 */
	public int getTaskID() 
	{
		return PTStartupTasks.INITAPPLICATION.getOrdinal();
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
	 */
	public int getNumSteps() 
	{
		return 1;
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
	 */
	public ReturnValue init(String _strApplicationName) 
	{
    	m_applicationName = _strApplicationName;
    	return new ReturnValue(PT_STARTUP_RESULTCODES.OK,getTaskName()+" init successful",null);
	}

	/* (non-Javadoc)
	 * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
	 */
	public ReturnValue executeStep(int sequentialStepID) 
	{
	    String strApplicationName = "";
	    String strLibHome = "";		   
		try 
		{
		    log.Info("Initializing Application.");
		    
			// retrieve necessary elements from settings hashtable.
		    strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.
		        GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
		    if (strApplicationName != m_applicationName)
		    {
				return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
						"The application name passed to init() conflicts with the name stored in the hashtable: hashtable entry "+strApplicationName+" does not match init application name of: "+m_applicationName,
						StackTracer.MakeXPException("(internal stacktrace)"));		    	
		    }
		    strLibHome = XPConvert.ToString(AppWarmUp.s_htInitSettings.
		        GetElement(AppWarmUp.KEY_PATH_LIB_HOME));		   

		    // create and initialize our application.
		    ApplicationManager.GetInstance().GetApplication(
			    strApplicationName, null, strLibHome);			

			log.Info("Application initialized.");
		}
		catch (Throwable ex)
		{
			return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
					"Could not initialize the application object with libhome of: "+strLibHome+" and application name of: "+strApplicationName,
					XPException.GetInstance(ex));
		} 
		
		return new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeApplication init successful",null);
	}
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
    public XPArrayList getMsg( int stepid, Platform platform, ReturnValue ret )
    {
        // Use the diagnostic helper to find the right message        
        return RecommendationHelper.getList(stepid,platform,ret.getResultCode(),m_diag);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
    public StartupTaskMessage[] getMsgArray()
    {
        return m_diag;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
    public int getNumMsgs()
    {
        return m_diag.length;
    }
}
