/*
 * Created on Apr 11, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.util.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * Initializes the config helper. This loads the core ConfigVarPack.
 * @author DavidP
 */
public class InitializeConfigHelper implements IStartupTask
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.application.startup.InitializeConfigHelper");

	private String m_applicationName = "";
    
    // array of messages
    private static StartupTaskMessage m_diag[] = 
    {
        // steps are one-based; step zero is the init call
        // DAP TODO this really needs to be in an external file
        new StartupTaskMessage( 
                0, Platform.ALL,PT_STARTUP_RESULTCODES.FATAL,
                "ConfigHelper problem", 
                "RECOMMENDATION: Review this exception to find the cause of this error and check the " +
                "relevant settings in portalconfig.xml to make sure there are no errors in the xml file.  " +
				"Typically, this failure indicates bad URL mappings, invalid URLs for the imageserver, or a " +
                "improperly configured security mode."
                )
    };
    
    /**
     * 
     */
    public InitializeConfigHelper()
    {
        super();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
    public String getTaskName()
    {
        return "InitializeConfigHelper";
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
    public int getTaskID()
    {
        return PTStartupTasks.INITCONFIG.getOrdinal();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
    public int getNumSteps()
    {
        return 1;
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
    public ReturnValue init(String _strApplicationName)
    {
    	m_applicationName = _strApplicationName;
        return new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeConfigHelper init successful",null);
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
    public ReturnValue executeStep(int sequentialStepID)
    {
        ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeImageServerHelper successful.",null);
        
		try 
		{		    
		    log.Info("Initializing ConfigHelper.");

		    // retrieve Application, ConfigVarPack, and ImgSvrURLHelper.
		    String strApplicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.
		        GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
		    IApplication application = ApplicationManager.GetInstance().
		    	GetApplication(strApplicationName);
		    ConfigVarPack vpConfig = (ConfigVarPack) application.GetVarPackManager().
		    	GetVariablePackage(ConfigVarPack.VARPACK_ID);
		    ImgSvrURLHelper ptImgSvrURLHelper = (ImgSvrURLHelper) application.
		    	GetCachingManager().GetEntry(AppWarmUp.KEY_CACHEMAN_IMAGE_SERVER_HELPER);
		    
			// initialize the config helper.  this must be done after the 
		    // image server url helper instance has been initialized.
			ReturnValue ret = ConfigHelper.StaticInit(strApplicationName, vpConfig, ptImgSvrURLHelper);
			if (ret.getResultCode() != PT_STARTUP_RESULTCODES.OK)
			    retVal = ret;
			
			log.Info("ConfigHelper initialized.");
		} 
		catch (Throwable ex) 
		{
		    String errorMessage = "Error initializing ConfigHelper: ";
			throw new XPException(errorMessage + ex.getMessage(), ex);
		}

        return retVal;
    }

    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
    public XPArrayList getMsg( int stepid, Platform platform, ReturnValue ret )
    {
        // Use the diagnostic helper to find the right message        
        return RecommendationHelper.getList(stepid,platform,ret.getResultCode(),m_diag);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
    public StartupTaskMessage[] getMsgArray()
    {
        return m_diag;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
    public int getNumMsgs()
    {
        return m_diag.length;
    }	
}
