/*
 * Created on Mar 13, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.openfoundation.io.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.application.startup.exceptions.*;
import com.plumtree.uiinfrastructure.application.startup.exceptions.ConfigVarPackException;
import com.plumtree.uiinfrastructure.application.startup.exceptions.LibHomePathException;
import com.plumtree.uiinfrastructure.application.startup.exceptions.WebHomeFromConfigException;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.util.*;
import com.plumtree.xpshared.config.*;

/**
 * Initializes all the path variables used by AppWarmup.
 * 
 * @author DavidP
 */
public class InitializePaths implements IStartupTask
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.application.startup.InitializePaths");
	
	private String m_applicationName="";
	
    // array of messages
    private static StartupTaskMessage m_diag[] = 
    {
        new StartupTaskMessage( 
                0, Platform.JAVA,PT_STARTUP_RESULTCODES.FATAL,
                "Core PORTAL HOME path initialization error.  Cannot proceed.", 
                "RECOMMENDATION: This usually indicates that the PT_HOME environment variable points to an " +
                "invalid pthome.xml, or the contents of pthome.xml are invalid.  Please check the portal " +
                "product path values in pthome.xml carefully for errors.  Since you are running " +
                "a Java portal, check that the user running the web application server has read access " +
                "to the settings files (E.G. pthome.xml)."
                ),
        new StartupTaskMessage(                
                0, Platform.DOTNET,PT_STARTUP_RESULTCODES.FATAL,
                "Core PORTAL HOME path initialization error.  Cannot proceed.", 
                "RECOMMENDATION: This usually indicates that the PT_HOME environment variable points to an " +
                "invalid pthome.xml, or the contents of pthome.xml are invalid.  Please check the portal " +
                "product path values in pthome.xml carefully for errors.  Since you are running a .NET " +
                "portal, please check that your NETWORK SERVICE user has appropriate access rights to " +
                "the settings folder."
                ),
        new StartupTaskMessage( 
                1, Platform.JAVA,PT_STARTUP_RESULTCODES.FATAL,
                "Secondary PATH initialization error.  Cannot proceed.", 
                "RECOMMENDATION: This indicates problems with the folder structure beneath the root " +
                "directory pointed to by the PT_HOME environment variable. Check that PT_HOME and " +
                "pthome.xml point to a valid installation.  Since you are running a Java portal, check " +
                "that the user running the web application server has read access to the settings files " +
                "(E.G. pthome.xml)."
                ),
        new StartupTaskMessage( 
                1, Platform.DOTNET,PT_STARTUP_RESULTCODES.FATAL,
                "Secondary PATH initialization error.  Cannot proceed.", 
                "RECOMMENDATION: This indicates problems with the folder structure beneath the root " +
                "directory pointed to by the PT_HOME environment variable. Check that PT_HOME and " +
                "pthome.xml point to a valid installation.  Since you are running a .NET portal, " +
                "please check that your NETWORK SERVICE user has appropriate access rights to the settings " +
                "folder."
                ),
        new StartupTaskMessage( 
                2, Platform.ALL,PT_STARTUP_RESULTCODES.FATAL,
                "Web Home initialization error.  Cannot proceed.", 
                "RECOMMENDATION: This means that the web application home directory is not defined or is invalid."
                )
    };
    
    
    public String getTaskName()
    {
        // Note: Thanks to j2c limitations, can't use CheckDevKit.class.getSimpleName();
        return "InitializePaths";
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
    public int getTaskID()
    {
        return PTStartupTasks.INITPATHS.getOrdinal();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
    public int getNumSteps()
    {
        return 3;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init()
     */
    public ReturnValue init(String _strApplicationName)
    {
    	m_applicationName = _strApplicationName;
    	return new ReturnValue(PT_STARTUP_RESULTCODES.OK,getTaskName()+" init successful",null);
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
	public ReturnValue executeStep(int sequentialStepID)
	{
		ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
		        "Incorrect step request on InitializePaths",
		        StackTracer.MakeXPException("(internal stacktrace)"));
		switch( sequentialStepID )
		{
			case 0:
				retVal = executeStepOne();
				break;
			case 1:
				retVal = executeStepTwo();
				break;
			case 2:
				retVal = executeStepThree();
				break;
		}
		return retVal;
	}

	/**
	 * Determine portal home path
	 */
	public ReturnValue executeStepOne()
	{
		log.Info("Initializing Portal Paths.");

		////////////////////////////////
		// DETERMINE PORTAL HOME PATH //
		////////////////////////////////
		
		try
		{
			// retrieve portal home path from pthome.xml.
			String strPortalHome = GeneralHelpers.FormatPath(ConfigPathResolver.
				GetProductPath(m_applicationName), AppWarmUp.PATH_SEPARATOR, true);
		    
			AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_PATH_PORTAL_HOME, strPortalHome);
		}
		catch (Throwable te)
		{
			String errorMessage = "Error retrieving PORTAL_HOME path: " + te.getMessage();
			return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,errorMessage,XPException.GetInstance(errorMessage, te));				
		}
		
		return new ReturnValue(PT_STARTUP_RESULTCODES.OK,"Portal home path determined.",null);
	}

	/**
	 * Determine other paths
	 */
	public ReturnValue executeStepTwo()
	{
		//////////////////////////////////
		// DETERMINE OTHER PORTAL PATHS //
		//////////////////////////////////
	    ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"All paths have been determined.",null);
		String strTempPath = null;
		boolean isValid = false;
		String strPortalHome = (String)AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_HOME);
		try
		{
			// verify all these folders exist and are readable
			String strPTHome = GeneralHelpers.FormatPath(
				XPEnvironment.GetEnvironmentVariable(AppWarmUp.PARAM_PTHOME_ENVVAR_NAME), 
				AppWarmUp.PATH_SEPARATOR, true);
	
			// path to PT_HOME\settings\portal folder.
			String strConfigPath = ConfigPathResolver.GetOpenConfigPath();
			strTempPath = GeneralHelpers.FormatPath(strConfigPath + 
				AppWarmUp.PATH_SEPARATOR + AppWarmUp.FOLDER_PORTAL + AppWarmUp.PATH_SEPARATOR, 
				AppWarmUp.PATH_SEPARATOR, true);
			checkDirectoryExists(strTempPath,"pthome.xml points to invalid portal settings directory");
			AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_PATH_PORTAL_SETTINGS, strTempPath);
	
			// path to PT_HOME\settings\portal folder (copy).
			strTempPath = (String)AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_SETTINGS);
			AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_PATH_PORTAL_CONFIG, strTempPath);
	
			// path to PORTAL_HOME\i18n folder.
			strTempPath = GeneralHelpers.FormatPath(strPortalHome + AppWarmUp.FOLDER_I18N + AppWarmUp.PATH_SEPARATOR, 
				AppWarmUp.PATH_SEPARATOR, true);
			checkDirectoryExists(strTempPath,"The i18n directory does not exist");
			AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_PATH_PORTAL_I18N, strTempPath);
	
			// path to PT_HOME\settings\portal\dynamicloads folder.
			strTempPath = GeneralHelpers.FormatPath(AppWarmUp.s_htInitSettings.GetElement(
				AppWarmUp.KEY_PATH_PORTAL_SETTINGS) + AppWarmUp.FOLDER_DYNAMIC_LOADS + AppWarmUp.PATH_SEPARATOR, 
				AppWarmUp.PATH_SEPARATOR, true);
			checkDirectoryExists(strTempPath,"The dynamicloads directory does not exist");
			AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_PATH_PORTAL_DYNAMIC_LOADS, strTempPath);

			// path to PT_HOME\settings\portal\VarPacks.xml file.
			strTempPath = GeneralHelpers.FormatPath(AppWarmUp.s_htInitSettings.GetElement(
				AppWarmUp.KEY_PATH_PORTAL_SETTINGS) + AppWarmUp.FILE_VAR_PACKS_XML, AppWarmUp.PATH_SEPARATOR, false);
			checkFileExists(strTempPath,"The VarPacks.xml file does not exist");
			AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_FILE_VAR_PACKS, strTempPath);
	
			// path to PT_HOME\settings\portal\ActivitySpaces.xml file.
			strTempPath = GeneralHelpers.FormatPath(AppWarmUp.s_htInitSettings.GetElement(
				AppWarmUp.KEY_PATH_PORTAL_SETTINGS) + AppWarmUp.FILE_ACTIVITY_SPACES_XML, AppWarmUp.PATH_SEPARATOR, false);
			checkFileExists(strTempPath,"The ActivitySpaces.xml file does not exist");
			AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_FILE_ACTIVITY_SPACES, strTempPath);
	
			// path to PT_HOME\settings\portal\CustomVarPacks.xml file.
			strTempPath = GeneralHelpers.FormatPath(AppWarmUp.s_htInitSettings.GetElement(
				AppWarmUp.KEY_PATH_PORTAL_SETTINGS) + AppWarmUp.FILE_CUSTOM_VAR_PACKS_XML, AppWarmUp.PATH_SEPARATOR, false);
			checkFileExists(strTempPath,"The CustomVarPacks.xml file does not exist");
			AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_FILE_CUSTOM_VAR_PACKS, strTempPath);
	
			// path to PT_HOME\settings\portal\CustomActivitySpaces.xml file.
			strTempPath = GeneralHelpers.FormatPath(AppWarmUp.s_htInitSettings.GetElement(
				AppWarmUp.KEY_PATH_PORTAL_SETTINGS) + AppWarmUp.FILE_CUSTOM_ACTIVITY_SPACES_XML, AppWarmUp.PATH_SEPARATOR, false);
			checkFileExists(strTempPath,"The CustomActivitySpaces.xml file does not exist");
			AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_FILE_CUSTOM_ACTIVITY_SPACES, strTempPath);
	
			////////////////////////////
			// DETERMINE THE PLATFORM //
			////////////////////////////
	
			if (XPSystem.IsRunningInDotNet())
			{
				AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_STR_PLATFORM, ConfigVarPack.APP_PLATFORM_DOT_NET);
			}
			else
			{
				AppWarmUp.s_htInitSettings.PutElement(AppWarmUp.KEY_STR_PLATFORM, ConfigVarPack.APP_PLATFORM_JAVA);
			}
		}
		catch (Throwable te)
		{
		    String reason = RecommendationHelper.ConvertExceptionToRecommendation( XPException.GetInstance(te) );
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,reason,XPException.GetInstance(te));				
		}
		
		return retVal;
	}

	/**
	 * Determine webhome and libhome
	 */
	public ReturnValue executeStepThree()
	{		
		///////////////////////////////////
		// DETERMINE WEBHOME AND LIBHOME //
		///////////////////////////////////
	    ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializePaths determination successful.",null);
		
		try
		{
			// retrieve portal home path from pthome.xml.
			retVal = AppWarmUp.HelperDeterminePaths();
		}
		catch( ConfigVarPackException te )
		{
			String errorMessage = "Cannot locate the base config varpack.  Error is: " + te.getMessage();
			return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,errorMessage,XPException.GetInstance(errorMessage, te));				
		}
		catch( LibHomePathException te )
		{
			String errorMessage = "Cannot locate the library home path.  Error is: " + te.getMessage();
			return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,errorMessage,XPException.GetInstance(errorMessage, te));				
		}
		catch( WebHomeFromConfigException te )
		{
			String errorMessage = "Cannot compute or locate the web home path.  Error is: " + te.getMessage();
			return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,errorMessage,XPException.GetInstance(errorMessage, te));			
		}
		catch (Throwable te)
		{
			String errorMessage = "Error retrieving WEBOME and LIBHOME paths.  Major issue with install tree or PT_HOME env. var: " + te.getMessage();
			return new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,errorMessage,XPException.GetInstance(errorMessage, te));				
		}

		//////////////////////////////////////
		// LOG CORE INITIALIZATION MESSAGES //
		//////////////////////////////////////
		if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
		{
			log.Info("The platform is: " +
				AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_STR_PLATFORM));
			log.Info("The path to Portal Home is: " + 
				AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_HOME));
			log.Info("The path to the configuration files is: " + 
				AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_CONFIG));
			log.Info("The path to the dynamicloads files is: " + 
				AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_DYNAMIC_LOADS));
			log.Info("The path to the resource files is: " + 
				AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_I18N));
			log.Info("The path to WebHome is: " + 
				AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_WEB_HOME));
			log.Info("The path to LibHome is: " + 
				AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_LIB_HOME));
			
			log.Info("Portal Paths initialized.");
		}
		
		return retVal;
    }

	
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
    public XPArrayList getMsg( int stepid, Platform platform, ReturnValue ret )
    {
        // Use the diagnostic helper to find the right message        
        return RecommendationHelper.getList(stepid,platform,ret.getResultCode(),m_diag);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
    public StartupTaskMessage[] getMsgArray()
    {
        return m_diag;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
    public int getNumMsgs()
    {
        return m_diag.length;
    }	
    
    /**
     * Check that the directory exists
     * @param szPath
     * @param szMessage
     */
    private void checkDirectoryExists(String szPath, String szMessage)
    {
	    boolean isValid = false;
	    try
	    {
	        XPDirectory dirItself = new XPDirectory(szPath);
	        isValid = dirItself.Exists();
	    }
	    catch( XPException e )
	    {
	        throw XPException.GetInstance(szMessage+": "+szPath,e);
	    }
	    if (!isValid)
	    {
	        StackTracer.ThrowNewXPFileNotFoundException(szMessage+": "+szPath);
	    }						        
    }
    
    /**
     * Check that the file exists
     * @param szPath
     * @param szMessage
     */
    private void checkFileExists(String szPath, String szMessage)
    {
	    boolean isValid = false;
	    XPFile fileItself = null;
	    try
	    {
	        fileItself = new XPFile(szPath);
	        isValid = fileItself.Exists();
	    }
	    catch( XPException e )
	    {
	        throw XPException.GetInstance(szMessage+": "+szPath,e);
	    }
	    if (!isValid)
	    {
	        StackTracer.ThrowNewXPFileNotFoundException(szMessage+": "+szPath);
	    }						        
    }

}
