/*
 * Created on Apr 12, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.util.XPConvert;
import com.plumtree.openfoundation.util.XPException;
import com.plumtree.openlog.OpenLogService;
import com.plumtree.openlog.OpenLogger;
import com.plumtree.uiinfrastructure.application.AppWarmUp;
import com.plumtree.uiinfrastructure.application.notification.IStartupTask;
import com.plumtree.uiinfrastructure.application.notification.PT_STARTUP_RESULTCODES;
import com.plumtree.uiinfrastructure.application.notification.ReturnValue;
import com.plumtree.uiinfrastructure.application.startup.exceptions.*;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.application.varpacks.ConfigVarPack;
import com.plumtree.uiinfrastructure.util.*;
import com.plumtree.uiinfrastructure.web.ApplicationManager;
import com.plumtree.uiinfrastructure.web.IApplication;

/**
 * Initializes the ResourceManager.  After this point the 
 * ResourceManager will be available in the Application cache.
 * The ResourceManager handles all the i18n files.
 * 
 * TODO: This should be reloadable, but it's not.  There is a bug 
 * filed against OpenFoundation.  When the bug is fixed, add a 
 * method to reload the ResourceManager.
 * 
 * @author DavidP
 */
public class InitializeResourceManager implements IStartupTask
{
	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.application.startup.InitializeResourceManager");
    private String m_applicationName = "";
    private ConfigVarPack m_vpConfig = null;
    private IApplication m_application = null; 
    private String m_strResourceFolder = null;
    private int m_nDebuggingEnabled = 0;
    
    // array of messages
    private static StartupTaskMessage m_diag[] = 
    {
        // steps are one-based; step zero is the init call
        // DAP TODO this really needs to be in an external file
        new StartupTaskMessage( 
                0, Platform.ALL,PT_STARTUP_RESULTCODES.FATAL,
                "ResourceManager i18n Unit Test Error", 
                "RECOMMENDATION: This rare error usually indicates that the i18n XML language files are " +
                "corrupt, or you have improperly followed instructions for extending or editing those files.  " +
                "Review the details in the secondary exception below."
                ),
        new StartupTaskMessage( 
                1, Platform.JAVA,PT_STARTUP_RESULTCODES.FATAL,
                "ResourceManager i18n Strings Loading Error (Java)", 
                "RECOMMENDATION: This indicates an installation error, folder access rights issue, " +
                "corrupt i18n XML files, or incorrect customer implementation of additional language support.  "+
				"Since this is Java you should first check that the user you are running your J2EE web " +
				"application server as has full access to the i18n settings folder.  " + 
                "To isolate other causes of this issue, try enabling debug mode in portalconfig.xml " +
                "(set the DebuggingMode setting to 1) and restarting your portal. This enables the i18n " +
                "diagnostic unit tests, which should output more detailed errors to ptspy."
                ),
        new StartupTaskMessage( 
                1, Platform.DOTNET,PT_STARTUP_RESULTCODES.FATAL,
                "ResourceManager i18n Strings Loading Error (.NET)", 
                "RECOMMENDATION: This indicates an installation error, folder access rights issue, " +
                "corrupt i18n XML files, or incorrect customer implementation of additional language support.  "+
				"Since this is .NET you should first check that the IIS anonymous user has full " +
				"access to the i18n settings folder.  " + 
                "To isolate other causes of this issue, try enabling debug mode in portalconfig.xml " +
                "(set the DebuggingMode setting to 1) and restarting your portal. This enables the i18n " +
                "diagnostic unit tests, which should then output more detailed errors to ptspy."
                )
    };    

    /**
     * 
     */
    public InitializeResourceManager()
    {
        super();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
    public String getTaskName()
    {
        return "InitializeResourceManager";
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
    public int getTaskID()
    {
        return PTStartupTasks.INITIMGSERVER.getOrdinal();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
    public int getNumSteps()
    {
        return 2;
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
    public ReturnValue init(String _strApplicationName)
    {
    	m_applicationName = _strApplicationName;
        return new ReturnValue(PT_STARTUP_RESULTCODES.OK,getTaskName()+" init successful",null);
    }


    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
    public ReturnValue executeStep(int sequentialStepID)
    {
		ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Incorrect step request on init",
		        StackTracer.MakeXPException("(internal stacktrace)"));
		switch( sequentialStepID )
		{
			case 0:
			    // open config file
				retVal = executeStepOne();
				break;
			case 1:
			    // load and process resources
				retVal = executeStepTwo();
				break;
		}
		return retVal;
    }
    
    /**
     * Load config file, get imageserver URL, form related urls
     * @return
     */
    private ReturnValue executeStepOne()
    {
        ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeImageServerHelper successful.",null);
	   
	    log.Info("Initializing Resource Manager.");

	    try 
		{		    
		    // retrieve Application and other strings.
	        m_applicationName = XPConvert.ToString(AppWarmUp.s_htInitSettings.
		        GetElement(AppWarmUp.KEY_STR_APPLICATION_NAME));
		    m_application = ApplicationManager.GetInstance().
		    	GetApplication(m_applicationName);
		    m_vpConfig = (ConfigVarPack) m_application.GetVarPackManager().
		    	GetVariablePackage(ConfigVarPack.VARPACK_ID);
		    m_strResourceFolder = XPConvert.ToString(AppWarmUp.s_htInitSettings.
		        GetElement(AppWarmUp.KEY_PATH_PORTAL_I18N));

		    try 
		    {
		        // retrieve debugging mode.  
			    m_nDebuggingEnabled = m_vpConfig.GetConfigValueAsInt(
			        "SystemProperties", "DebuggingMode", "value");
		    }
		    catch (Throwable ex)
		    {
		        m_nDebuggingEnabled = 0;
				log.Warn(ex, "Debugging mode unknown, skipping language file test.");								        
		    }
		}
		catch (Throwable ex) 
		{
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
			        "Could not determine the i18n resource folder.  Check the ConfigVarPack settings for i18n.",
			        XPException.GetInstance(ex));
			return retVal;
		}
		
		return retVal;
    }
		    
    /**
     * Load config file, get imageserver URL, form related urls
     * @return
     */
    private ReturnValue executeStepTwo()
    {
        ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeImageServerHelper successful.",null);
	   
	    log.Info("Initializing Resource Manager.");
		    
	    try 
		{		    
			if (m_nDebuggingEnabled == 1)
			{
			    // if debugging is enabled run language test.
				log.Info("Running language file unit test."); 
				retVal = AppWarmUp.HelperTestLanguages(m_strResourceFolder);
				if (retVal.getResultCode() == PT_STARTUP_RESULTCODES.OK)
				    log.Info("Language file unit test completed.");		
				else
				    return retVal;
			}
		} 
		catch (Throwable ex) 
		{
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
			        "Resource Manager error executing unit test on i18n resources:  Corrupted i18n language files.",
			        XPException.GetInstance(ex));
			return retVal;

		}

		try
		{
		    // resource manager is a singleton, cannot be reloaded.		    
			XPResourceManager rmResourceManager = XPResourceManager.
				CreateResourceManager(m_strResourceFolder);
			rmResourceManager.LoadResources();

			// save the resource manager into the application cache.
			m_application.GetCachingManager().SetEntry(
			    AppWarmUp.KEY_CACHEMAN_RESOURCE_MANAGER, rmResourceManager);
			
		    log.Info("Resource Manager initialized.");
		} 
		catch (Throwable ex) 
		{
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
			        "Resource Manager loading error:",
			        XPException.GetInstance(ex));
			return retVal;

		}
		return retVal;
    }

    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
    public XPArrayList getMsg( int stepid, Platform platform, ReturnValue ret )
    {
        // Use the diagnostic helper to find the right message        
        return RecommendationHelper.getList(stepid,platform,ret.getResultCode(),m_diag);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
    public StartupTaskMessage[] getMsgArray()
    {
        return m_diag;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
    public int getNumMsgs()
    {
        return m_diag.length;
    }    
}
