/*
 * Created on Apr 13, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.application.startup.messages.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.uiinfrastructure.util.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * Loads the Version Information.
 * @author DavidP
 */
public class LoadVersionHelper implements IStartupTask
{

	private static OpenLogger log = OpenLogService.GetLogger(
			OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.application.startup.LoadVersionHelper");
    private String m_applicationName = "";
    private ConfigVarPack m_vpConfig = null;
    private String m_strImageServerBaseURL = null;
    private String m_strSecureImageServerBaseURL = null;
    private String m_strImageServerConnectionURL = null;
    private IApplication m_application = null; 

    
    // array of messages
    private static StartupTaskMessage m_diag[] = 
    {
        // steps are one-based; step zero is the init call
        // DAP TODO this really needs to be in an external file
        new StartupTaskMessage( 
                0, Platform.ALL,PT_STARTUP_RESULTCODES.FATAL,
                "LoadVersionHelper problem", 
                "RECOMMENDATION: Check that version.xml is located in the correct directory " +
                "(I.E. plumtree\\settings\\portal) and is not corrupt."                )
    };    
    
    /**
     * 
     */
    public LoadVersionHelper()
    {
        super();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskName()
     */
    public String getTaskName()
    {
        return "LoadVersionHelper";
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getTaskID()
     */
    public int getTaskID()
    {
        return PTStartupTasks.LOADVERSION.getOrdinal();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#getNumSteps()
     */
    public int getNumSteps()
    {
        return 1;
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#init(java.lang.String)
     */
    public ReturnValue init(String _strApplicationName)
    {
    	m_applicationName = _strApplicationName;
        return new ReturnValue(PT_STARTUP_RESULTCODES.OK,"LoadVersionHelper init successful",null);
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.application.notification.IStartupTask#executeStep(int)
     */
    public ReturnValue executeStep(int sequentialStepID)
    {
        ReturnValue retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"Version information loaded.",null);
		try
		{
		    log.Info("Loading Version: version.xml.");
		    
			// uses version.xml config file to initialize.
		    VersionHelper versionHelper = new VersionHelper(XPConvert.ToString(
			    AppWarmUp.s_htInitSettings.GetElement(AppWarmUp.KEY_PATH_PORTAL_CONFIG)));
			
			// set the static content.
			versionHelper.LoadVersionXML();
			
			log.Info(VersionHelper.GetVersionMessage());
		}
		catch (Throwable ex)
		{
		    retVal = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"Error loading Version information: ",
		            XPException.GetInstance(ex));
		}			
        
        return retVal;
    }

    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsg(int)
     */
    public XPArrayList getMsg( int stepid, Platform platform, ReturnValue ret )
    {
        // Use the diagnostic helper to find the right message        
        return RecommendationHelper.getList(stepid,platform,ret.getResultCode(),m_diag);
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getMsgArray()
     */
    public StartupTaskMessage[] getMsgArray()
    {
        return m_diag;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.text.IMessage#getNumMsgs()
     */
    public int getNumMsgs()
    {
        return m_diag.length;
    }    
}
