/*
 * Created on Mar 13, 2005
 */
package com.plumtree.uiinfrastructure.application.startup;

import com.plumtree.uiinfrastructure.application.notification.*;

/**
 * Enumerator for startup tasks for uiinfrastructure.
 * Add each new startup class to this helper.
 * Other startup tasks must be added to the task manager 
 * dynamically.
 * This enumerator automatically numbers each task sequentially.
 * 
 * @author DavidP
 */
public class PTStartupTasks
{    
	// private task instance, ordinal management values
	private static int m_nextOrdinal = 0;
	private IStartupTask m_task;
	private int m_ordinal = 0;
    
	// public startup task elements
//	 WARNING: Do NOT change the order of these entries; they are highly interdependent
    public static final PTStartupTasks INITBASE			= new PTStartupTasks(new InitPortalObjects(), ++m_nextOrdinal);
	public static final PTStartupTasks DEVKIT			= new PTStartupTasks(new CheckDevKit(), ++m_nextOrdinal);
	public static final PTStartupTasks INITPATHS		= new PTStartupTasks(new InitializePaths(), ++m_nextOrdinal);
	public static final PTStartupTasks INITAPPLICATION	= new PTStartupTasks(new InitializeApplication(), ++m_nextOrdinal);
	public static final PTStartupTasks INITAPPWARMUP	= new PTStartupTasks(new InitializeAppWarmUpHelper(), ++m_nextOrdinal);
	public static final PTStartupTasks INITIMGSERVER	= new PTStartupTasks(new InitializeImageServerHelper(), ++m_nextOrdinal);
	public static final PTStartupTasks INITCONFIG		= new PTStartupTasks(new InitializeConfigHelper(), ++m_nextOrdinal);
	public static final PTStartupTasks INITRESOURCE		= new PTStartupTasks(new InitializeResourceManager(), ++m_nextOrdinal);
	public static final PTStartupTasks LOADVERSION		= new PTStartupTasks(new LoadVersionHelper(), ++m_nextOrdinal);
	public static final PTStartupTasks INITAS			= new PTStartupTasks(new InitializeActivitySpaces(), ++m_nextOrdinal);
	public static final PTStartupTasks INITVARPACKS		= new PTStartupTasks(new InitializeVarPacks(), ++m_nextOrdinal);
	public static final PTStartupTasks INITIMGSERVERII	= new PTStartupTasks(new InitializeImageServerHelperPartII(), ++m_nextOrdinal);
	public static final PTStartupTasks INITOBJECTS		= new PTStartupTasks(new InitializeObjects(), ++m_nextOrdinal);
	public static final PTStartupTasks LOADCUSTOMLOADS	= new PTStartupTasks(new LoadCustomLoads(), ++m_nextOrdinal);
	public static final PTStartupTasks CHECKDBVER		= new PTStartupTasks(new CheckDBVersion(), ++m_nextOrdinal);
    
	// public consumable enumeratable array list of tasks
	// this list must be static and structured for order preservation
	// WARNING: Do NOT change the order of these entries; they are highly interdependent
    // NOTE: Even though this variable is static and final, the elements in the array
    // could be changed.  Do not do this!
	public static final PTStartupTasks[] LIST = 
		{
	        INITBASE,
			DEVKIT,
			INITPATHS,
			INITAPPLICATION,
			INITAPPWARMUP,
			INITIMGSERVER,
			INITCONFIG,
			INITRESOURCE,
			LOADVERSION,
			INITAS,
			INITVARPACKS,
			INITIMGSERVERII,
			INITOBJECTS,
			LOADCUSTOMLOADS,
			CHECKDBVER
		};
    
	// private constructor
	private PTStartupTasks( IStartupTask task, int ordinal )
	{
	    m_ordinal = ordinal;
		m_task = task;
	}
    
	/**
	 * @return the task interface for this enumerated task 
	 * instance
	 */
	public IStartupTask getTask()
	{
		return m_task;
	}
    
	/**
	 * @return the integer ordinal (one-based) of this 
	 * enumerated task instance
	 */
	public int getOrdinal()
	{
		return m_ordinal;
	}
}
