/*
 * Created on Apr 4, 2005
 */
package com.plumtree.uiinfrastructure.application.startup.messages;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.util.*;

/**
 * Processes array of recommendations to arrive at a set of reasonable 
 * suggestions best tailored to match the user's situation.
 * 
 * @author DavidP
 */
public class RecommendationHelper
{
    /**
     * 
     */
    public RecommendationHelper()
    {
        super();
    }


    /**
     * Implements lookup method that locates the most appropriate recommendation(s) 
     * 
     * @param taskstepid
     * @param platformid
     * @param retval
     * @param m_diag
     * @return
     */
    public static XPArrayList getList(int taskstepid,Platform platformid,PT_STARTUP_RESULTCODES retval,StartupTaskMessage[] m_diag)
    {
        XPArrayList recommendations = new XPArrayList();
        
        if (m_diag.length > 0)
        {
            // Match all specifics and add to the stack
            for (int loop = 0; loop < m_diag.length; ++loop)
            {
                StartupTaskMessage elt = m_diag[loop];
                if (elt.getStepID() == taskstepid)
                {
                    if (elt.getPlatform() == platformid)
                    {
                        if (elt.getRetval() == retval)
                        {
                            // most specific
                            recommendations.Add(elt);
                        }
                        else if (elt.getRetval() == null)
                        {
                            // somewhat specific
                            recommendations.Add(elt);                            
                        }
                    }
                    else if ((elt.getPlatform() == Platform.JAVA)&&(Platform.isPlatformAnyJava(platformid)))
                    {
                        // not very specific
                        recommendations.Add(elt);
                    }
                    else if ((elt.getPlatform() == Platform.DOTNET)&&(Platform.isPlatformAnyDotnet(platformid)))
                    {
                        // not very specific
                        recommendations.Add(elt);
                    }
                    else if (elt.getPlatform() == Platform.ALL)
                    {
                        // not very specific
                        recommendations.Add(elt);
                    }
                }
                else
                {
                    // pretty darn vague - forget it
                }
            }
        }
        
        return recommendations;
    }
    
	/**
	 * Converts exception to a short reasonable string
	 * 
	 * @param te
	 * @return
	 */
	public static final String ConvertExceptionToShortString( XPException te, boolean getMessage, boolean getSecondLine )
	{
	    String exceptionName = "";
	    String fullexception = XPException.ExceptionToString(te);
	    String message = "";
	    String firstLine = "";
	    String secondLine = "";
	    String cr = "";
	    Platform curPlatform = Platform.getCurPlatform();
	    
	    // get the linefeed char for this platform
	    if ((curPlatform == Platform.WIN32_JAVA)||(curPlatform == Platform.WIN32_DOTNET))
	    	cr = "\r\n";
	    else
	    	cr = "\n";
	    
	    // pare down the name to its minimum
	    if (fullexception.length() > 0)
	    {
		    // get the first line of the chained exception
		    char[] eol = new char[3];
		    eol[0] = (char)0x0d;
		    eol[1] = (char)0x00;
		    eol[2] = (char)0x00;
		    XPStringTokenizer tok = new XPStringTokenizer(fullexception,new String(eol),false);
		    if (tok.HasMoreTokens())
		    {
		    	firstLine = tok.GetNextToken();
		    	if (firstLine != null && firstLine.length() > 1)
		    	{
			    	if (tok.HasMoreTokens())
			    	{
			    		String str2 = tok.GetNextToken();
			    		if (str2 != null && str2.length() > 1)
			    		{
			    		    secondLine = str2.substring(1,str2.length());
			    		    
			    		    // skip those lines showing traces *inside* XPException, e.g., these two lines:
			    		    //	at com.plumtree.openfoundation.util.XPException.GetInstance(XPException.java:383)
							//  at com.plumtree.openfoundation.util.XPException.GetInstance(XPException.java:350) 
			    		    while ((secondLine.indexOf("com.plumtree.openfoundation.util.XPException") != -1) 
			    		    	&& tok.HasMoreTokens())
		    		    	{
								str2 = tok.GetNextToken();
								if (str2 != null && str2.length() > 1)
								{
									secondLine = str2.substring(1,str2.length());
								} 
		    		    	}
			    		}
		    		}
			    		
			    	// compute base exception name
			    	int iLastColon = firstLine.indexOf(":");
			    	String exceptionOnly = firstLine;
			    	if (iLastColon != -1)
			    	{
						exceptionOnly = firstLine.substring(0,iLastColon);
						if ((iLastColon+1) < firstLine.length())
						{
							message = XPStringUtility.Substring(firstLine, iLastColon+1);
						}
			    	}

			    	int iLastDot = exceptionOnly.lastIndexOf(".");
			    	if ((iLastDot != -1) && (iLastDot < exceptionOnly.length())&&
			    	        ((firstLine.length()-iLastDot-1)>(iLastDot+1)))
			    	{
			    		String subStr = exceptionOnly.substring(iLastDot+1,exceptionOnly.length());
			    		int colon = subStr.indexOf(":");
			    		if (colon != -1)
			    		    exceptionName = subStr.substring(0,colon);
			    		else if (subStr != null && subStr.length() > 1)
			    		    exceptionName = subStr;
		    		}
		    	}
	    	} 
	    }
	    else
	    {
	    	exceptionName = fullexception.substring(0,50);
	    }
	    
	    if (getSecondLine)
	    {
	        return secondLine;
	    }
	    
	    if (getMessage)
	    {
	        return message;
	    }
	    
	    return exceptionName;
	}
    
	/**
	 * Converts stack trace string to a short reasonable string
	 * 
	 * @param te
	 * @return
	 */
	public static final String ConvertStackTraceToShortString( String stackTrace, boolean getSecondLine )
	{
	    String exceptionName = "";
	    String fullexception = stackTrace;
	    String firstLine = "";
	    String secondLine = "";
	    String cr = "";
	    Platform curPlatform = Platform.getCurPlatform();
	    
	    // get the linefeed char for this platform
	    if ((curPlatform == Platform.WIN32_JAVA)||(curPlatform == Platform.WIN32_DOTNET))
	    	cr = "\r\n";
	    else
	    	cr = "\n";
	    
	    // pare down the name to its minimum
	    if (fullexception.length() > 0)
	    {
		    // get the first line of the chained exception
		    char[] eol = new char[3];
		    eol[0] = (char)0x0d;
		    eol[1] = (char)0x00;
		    eol[2] = (char)0x00;
		    XPStringTokenizer tok = new XPStringTokenizer(fullexception,new String(eol),false);
		    if (tok.HasMoreTokens())
		    {
		    	firstLine = tok.GetNextToken();
		    	if (firstLine != null && firstLine.length() > 1)
		    	{
			    	if (tok.HasMoreTokens())
			    	{
			    		String str2 = tok.GetNextToken();
			    		if (str2 != null && str2.length() > 1)
			    		{
			    		    secondLine = str2.substring(1,str2.length());

							// skip those lines showing traces *inside* XPException, e.g., these two lines:
							//	at com.plumtree.openfoundation.util.XPException.GetInstance(XPException.java:383)
							//  at com.plumtree.openfoundation.util.XPException.GetInstance(XPException.java:350) 
							while ((secondLine.indexOf("com.plumtree.openfoundation.util.XPException") != -1) 
								&& tok.HasMoreTokens())
							{
								str2 = tok.GetNextToken();
								if (str2 != null && str2.length() > 1)
								{
									secondLine = str2.substring(1,str2.length());
								} 
							}
			    		}
		    		}
			    		
			    	// compute base exception name
			    	int iLastDot = firstLine.lastIndexOf(".");
			    	if ((iLastDot != -1) && (iLastDot < firstLine.length())&&
			    	        ((firstLine.length()-iLastDot-1)>(iLastDot+1)))
			    	{
			    		String subStr = firstLine.substring(iLastDot+1,firstLine.length());
			    		int colon = subStr.indexOf(":");
			    		if (colon != -1)
			    		    exceptionName = subStr.substring(0,colon);
			    		else if (subStr != null && subStr.length() > 1)
			    		    exceptionName = subStr;
		    		}
		    	}
	    	} 
	    }
	    else
	    {
	    	exceptionName = fullexception.substring(0,50);
	    }
	    
	    if (getSecondLine)
	        return secondLine;
	    
	    return exceptionName;
	}
	
	/**
	 * Returns a recommendation string for an exception
	 * @param te
	 * @return
	 */
	public static final String ConvertExceptionToRecommendation( XPException te )
	{
	    // retrieve the two messages
	    String exceptionName = ConvertExceptionToShortString(te,false,false);
	    String message = ConvertExceptionToShortString(te,true,false);
	    String secondLine = ConvertExceptionToShortString(te,false,true);
	    
	    return ConvertExceptionToRecommendationFromShortName( exceptionName, message, secondLine, te );
	}    
		
	/**
	 * Returns a recommendation string for an exception's shortname, more reusable
	 * @param te
	 * @return
	 */
	public static final String ConvertExceptionToRecommendationFromShortName( String exceptionName, 
	        String messageStr, String secondLine, XPException te )
	{
	    String message = "";
	    String cr = "";
	    Platform curPlatform = Platform.getCurPlatform();
	    String exceptionMsg = XPException.GetInstance(te).GetMessage();
	    boolean bNative = false;
	    boolean bOKSearch = false;
	    
	    // is this one of the devastating errors?
	    if (curPlatform.isJava())
	    {
		    if (exceptionMsg.indexOf("Native library") != -1)
		        bNative = true;
		    if (exceptionMsg.indexOf("openkernelsearch") != -1)
		        bOKSearch = true;
	    }
	    else
	    {
		    if ((exceptionMsg.indexOf("File or assembly") != -1)&&(exceptionMsg.indexOf("System.IO.FileNotFoundException") == -1))
		        bNative = true;
		    if (exceptionMsg.indexOf("openkernelsearch") != -1)
		        bOKSearch = true;	        
	    }
	    
	    // get the linefeed char for this platform
	    if ((curPlatform == Platform.WIN32_JAVA)||(curPlatform == Platform.WIN32_DOTNET))
	    	cr = "\r\n";
	    else
	    	cr = "\n";
	    
	    // search library path
	    boolean bDevastating = false;
	    if (bNative && bOKSearch)
	    {
	        if (curPlatform.isDotnet())
	        {
	            message = "TDE: Portal or Search native libraries are not on your PATH.  Normally, the " +
            		"installer sets up your PATH correctly. Please verify all native libraries have " +
            		"been installed and that your system PATH correctly points to them. The installer " +
            		"sets up the following environment variables, and then uses them on your PATH: " +
            		"PORTALLIB_PATH; PTHREADS_PATH; OUTSIDEIN_PATH; INXIGHT_PATH; ICU_PATH. Please " +
            		"manually tune and verify each " +
            		"entry on your PATH and run again.";
	            bDevastating = true;
	        }
	        else if (curPlatform.isSolaris() || curPlatform.isHPUX() || curPlatform.isAIX() || curPlatform.isLinux())
	        {
	            message = "TDE: Portal or Search native libraries are not on your LD_LIBRARY_PATH.  " +
	            		"Normally, the " +
	        		"installer sets up your LD_LIBRARY_PATH correctly. Please verify all native libraries have " +
	        		"been installed and that your system PATH correctly points to them. The installer " +
	        		"sets up the following environment variables, and then uses them on your LD_LIBRARY_PATH: " +
	        		"PORTALLIB_PATH; PTHREADS_PATH; OUTSIDEIN_PATH; INXIGHT_PATH; ICU_PATH.  Typically, " +
	        		"this problem occurs on Unix platforms because you have not sourced the pthome.sh " +
	        		"environement configuring shell script (run '. ./pthome.sh' in your PT_HOME directory).  " +
	        		"Please manually tune and verify each " +
	        		"entry on your LD_LIBRARY_PATH and run again.";    
	            bDevastating = true;
	        }
	        else // java on windows
	        {
	            message = "TDE: Portal or Search native libraries are not on your PATH.  Normally, the " +
	        		"installer sets up your PATH correctly. Please verify all native libraries have " +
	        		"been installed and that your system PATH correctly points to them. The installer " +
	        		"sets up the following environment variables, and then uses them on your PATH: " +
	        		"PORTALLIB_PATH; PTHREADS_PATH; OUTSIDEIN_PATH; INXIGHT_PATH; ICU_PATH.  Please " +
	        		"manually tune and verify each " +
	        		"entry on your PATH and run again.";
	            bDevastating = true;
	        }
	    }
	    else if (bNative)
	    {
	        if (curPlatform.isDotnet())
	        {
	            message = "TDE: Portal native libraries are not in your PATH.  Normally, the " +
            		"installer sets up your PATH correctly. Please verify that all native libraries have " +
            		"been installed and that your system PATH correctly points to the libraries. " +
            		"Please manually tune and verify each entry on your PATH and run again.";
	            bDevastating = true;
	        }
	        else if (curPlatform.isSolaris() || curPlatform.isHPUX() || curPlatform.isAIX() || curPlatform.isLinux())
	        {
	            message = "TDE: Portal native libraries are not on your LD_LIBRARY_PATH.  Normally, the " +
	        		"installer sets up your LD_LIBRARY_PATH correctly. Please verify all native libraries have " +
	        		"been installed and that your system PATH correctly points to them. Typically, " +
	        		"this problem occurs on Unix platforms because you have not sourced the pthome.sh " +
	        		"environement configuring shell script (run '. ./pthome.sh' in your PT_HOME directory).  " +
	        		"Please manually tune and verify each " +
	        		"entry on your LD_LIBRARY_PATH and run again.";    
	            bDevastating = true;
	        }
	        else // java on windows
	        {
	            message = "TDE: Portal native libraries are not on your PATH.  Normally, the " +
	        		"installer sets up your PATH correctly. Please verify that all native libraries have " +
	        		"been installed and that your system PATH correctly points to the libraries. Please " +
	        		"manually tune and verify each entry on your PATH and run again.";
	            bDevastating = true;
	        }
	    }
	    
	    // Exception name comparison chain:
	    if (!bDevastating)
	    {
		    if (exceptionName.compareToIgnoreCase("OKConfigException") == 0)
		    {
		    	message = "OpenKernel configuration failure. This usually means INVALID DATABASE SETTINGS. Run ptconfig " +
		    			"to verify your database connection.  If this fails, check the web application server user's access rights " +
		    			"to the file system.  Finally, verify the XML settings files are not corrupt.";
		    } else
			if (exceptionName.compareToIgnoreCase("OKConfigIllegalAccessException") == 0)
			{
				message = "OpenKernel configuration illegal access exception. Check the web application server user's access " +
						"rights to the file system.";
			} else
			if (exceptionName.compareToIgnoreCase("OKConfigInvalidNameException") == 0)
			{
				message = "OpenKernel configuration invalid name exception. You have an error in one of your XML configuration files under PT_HOME/settings, perhaps because you have two components with identical names or types.  Note that OpenConfig requires unique names acrosss *all* XML files in the settings directory.";
			} else
			if (exceptionName.compareToIgnoreCase("OKConfigItemNotFoundException") == 0)
			{
				message = "OpenKernel configuration item not found. This means the OpenKernel XML Configuration " +
						"file CANNOT BE FOUND.  This means either a bad install, or the value of the environment variable " +
						"PT_HOME is invalid.  Verify the XML settings files for the portal are properly installed and configured.";
			} else
			if (exceptionName.compareToIgnoreCase("OKConfigNullValueException") == 0)
			{
				message = "OpenKernel configuration null value exception. Check the XML settings files for corruption.";
			} else
			if (exceptionName.compareToIgnoreCase("OKConfigProviderReadException") == 0)
			{
				message = "OpenKernel configuration provider read exception. Check network access.";
			} else
			if (exceptionName.compareToIgnoreCase("OKConfigUninitializedSettingException") == 0)
			{
				message = "OpenKernel configuration uninitialized setting. Check the XML settings files for " +
						"corruption.";
			} else
			if (exceptionName.compareToIgnoreCase("OKCounterInitException") == 0)
			{
				message = "OpenKernel counter initialization exception. Check that the web application server user's " +
						"rights allow counter creation.";
			} else
			if (exceptionName.compareToIgnoreCase("OKCountersException") == 0)
			{
				message = "OpenKernel counter exception. Check that the web application server user's rights " +
						"allow counter creation.";
			} else
			if (exceptionName.compareToIgnoreCase("OKCryptoException") == 0)
			{
				message = "OpenKernel cryptography exception. Check for corruption in the database or XML files.  " +
						"Check the classpath and PATH for accidental upgrade errors.";
			} else
			if (exceptionName.compareToIgnoreCase("OKDataLengthException") == 0)
			{
				message = "OpenKernel database data length exception. Check the database for errors and report " +
						"the errors to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKDBException") == 0)
			{
				message = "OpenKernel database exception. Try running ptconfig verify your database connectivity; " +
						"you may need to report this error to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKDBTxnInconsistent") == 0)
			{
				message = "OpenKernel inconsistent database transaction exception. Try running ptconfig and verify your database connectivity; " +
						"you may need to report this error to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKException") == 0)
			{
				message = "OpenKernel exception with no further data. This is an unusually bad error.  Check your classpath, " +
						"PATH, and installation.  This may be a JNI error because you've mixed old and new versions of the " +
						"WebCenter Interaction on your PATH.";
			} else
			if (exceptionName.compareToIgnoreCase("OKFactoryException") == 0)
			{
				message = "OpenKernel factory exception. Check for available memory.  This could mean that the OpenKernel " +
						"factory is unable to create new objects because there is not enough memory available.";
			} else
			if (exceptionName.compareToIgnoreCase("OKHttpException") == 0)
			{
				message = "OpenKernel HTTP exception. Check network access to the affected resource or remote " +
						"server.";
			} else
			if (exceptionName.compareToIgnoreCase("OKHttpHostResolutionException") == 0)
			{
				message = "OpenKernel HTTP host resolution exception. Check network access to the affected resource " +
						"or remote server.  Check DNS name resolution.";
			} else
			if (exceptionName.compareToIgnoreCase("OKHttpInvalidStateException") == 0)
			{
				message = "OpenKernel HTTP invalid state exception. Check network access to the affected resource or " +
						"remote server.  This may indicate an unusual HTTP error.";
			} else
			if (exceptionName.compareToIgnoreCase("OKHttpUnknownProxyHostException") == 0)
			{
				message = "OpenKernel HTTP unknown proxy host exception. Check network access to the affected resource " +
						"or remote server.  Check DNS name resolution to the remote host or proxy server.";
			} else
			if (exceptionName.compareToIgnoreCase("OKInvalidAlgorithmParameterException") == 0)
			{
				message = "OpenKernel invalid cryptography algorithm parameter exception. An incorrect cryptography algorithm was " +
						"selected. Report this error to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKInvalidCipherTextException") == 0)
			{
				message = "OpenKernel invalid cipher text exception. Check the secret key stored in the database. " +
						"Try to regenerate a new secret key on the server settings utility page.";
			} else
			if (exceptionName.compareToIgnoreCase("OKInvalidMethodCallException") == 0)
			{
				message = "OpenKernel invalid method call exception. Check the classpath and PATH to insure you are " +
						"not mixing references to two versions of WebCenter Interaction.";
			} else
			if (exceptionName.compareToIgnoreCase("OKInvalidParamException") == 0)
			{
				message = "OpenKernel invalid parameter exception. Report this error to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKJMXConfigException") == 0)
			{
				message = "OpenKernel JMX (Java Management Extensions) config exception. " +
						"Check the JMX configuration data for your web application server.  " +
						"This is usually related to the JMX host server rather than the WebCenter Interaction server.";
			} else
			if (exceptionName.compareToIgnoreCase("OKJMXException") == 0)
			{
				message = "OpenKernel JMX (Java Management Extensions) exception. Check the JMX configuration data for your web application server.  " +
						"This is usually related to the JMX host server rather than the WebCenter Interaction server.";
			} else
			if (exceptionName.compareToIgnoreCase("OKJNICounterException") == 0)
			{
				message = "OpenKernel JNI (Java Native Interface) Counter. Check the PATH to insure the right binaries are accessible.  " +
						"Check that the user running the web application has appropriate counter access rights.";
			} else
			if (exceptionName.compareToIgnoreCase("OKMustBeOverriddenException") == 0)
			{
				message = "OpenKernel MustBeOverridden exception. Report this error to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKNoSuchAlgorithmException") == 0)
			{
				message = "OpenKernel  no such algorithm exception. Cryptography error.  Report this error to Oracle " +
						"Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKNoSuchPaddingException") == 0)
			{
				message = "OpenKernel cryptography No Such Padding exception. Cryptography error.  Report this error to Oracle " +
						"Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKNoSuchProviderException") == 0)
			{
				message = "OpenKernel no such provider exception. May be an incomplete installation. Report this error " +
						"to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKNotImplementedException") == 0)
			{
				message = "OpenKernel Not Implemented exception.  Report this error to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKObjectNotInitializedException") == 0)
			{
				message = "OpenKernel object not initialized exception. Report this error to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKSoapException") == 0)
			{
				message = "OpenKernel SOAP (Simple Object Access Protocol) exception. Verify remote server connections and network access to your " +
						"SOAP provider remote hosts.";
			} else
			if (exceptionName.compareToIgnoreCase("OKSQLBindParamInconsistent") == 0)
			{
				message = "OpenKernel SQL Bind Parameter Inconsistency exception. Verify your database is not corrupt. " +
						"Report this error to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKSQLDupKeyException") == 0)
			{
				message = "OpenKernel SQL duplicate key exception. Verify your database is not corrupt.  Try rebuilding " +
						"your indexes. Report this error to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKSQLException") == 0)
			{
				message = "OpenKernel SQL exception. Verify your database is not corrupt.  Try rebuilding your " +
						"indexes. Report this error to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKThreadInterruptedException") == 0)
			{
				message = "OpenKernel Interrupted Thread exception. An interrupted thread is usually a recoverable " +
						"error.  If this prevents startup, report this error to Oracle Support.";
			} else
			if (exceptionName.compareToIgnoreCase("OKUnhandledNativeException") == 0)
			{
				message = "OpenKernel unhandled native exception. Check your PATH; this usually indicates a JNI " +
						"error due to incorrect installation.  Check that the user running your web application server has " +
						"rights to access the native libraries.";
			} 
			else if (exceptionName.compareToIgnoreCase("XPSecurityException") == 0)
			{
				message = "OpenFoundation security exception. Check that the user running this application has the rights " +
						"to access the settings and library files.";
			}		
			else if (exceptionName.compareToIgnoreCase("XPDirectoryNotFoundException") == 0)
			{
				message = "OpenFoundation Directory Not Found exception. Check that you have " +
						"a successful installation.  Verify the directories pointed to by the " +
						"PT_HOME and PORTAL_HOME environment variables.";
			}
			else if (exceptionName.compareToIgnoreCase("XPIllegalAccessException") == 0)
			{
				message = "OpenFoundation Illegal Access exception. Usually means an application security problem " +
						"of some kind.  Verify the security requirements of your application " +
						"server.  On .NET, verify that the correct strongly-named dlls are being used.";
			}		
			else if (exceptionName.compareToIgnoreCase("XPInvocationTargetException") == 0)
			{
				message = "OpenFoundation InvocationTarget exception. Quite rare and usually " +
						"indicates an invalid install, or a reinstall on top of an older " +
						"server that does not have matching signatures.  Contact Oracle " +
						"support.";
			}		
			else if (exceptionName.compareToIgnoreCase("XPNoSuchMethodException") == 0)
			{
				message = "OpenFoundation NoSuchMethod exception. Quite rare and usually " +
						"indicates an invalid install, or a reinstall on top of an older " +
						"server that does not have matching signatures.  Contact Oracle " +
						"support.";
			}		
			else if (exceptionName.compareToIgnoreCase("XPArrayTypeMismatchException") == 0)
			{
				message = "OpenFoundation ArrayTypeMismatch exception. Quite rare and usually " +
						"indicates an invalid install, or a reinstall on top of an older " +
						"server that does not have matching signatures.  Contact Oracle " +
						"support.";
			}		
			else if (exceptionName.compareToIgnoreCase("XPClassNotFoundException") == 0)
			{
				message = "OpenFoundation ClassNotFound exception. This can occur with custom " +
						"dynamically-loaded code when there is a mismatch between libraries located " +
						"in the two library directories.  This can sometimes occur when a library is " +
						"present in both locations, but one is out of date compared to the other; " +
						"you must be careful to update both. Check that the files located in the " +
						"%PORTAL_HOME%/lib/java " +
						"or %PORTAL_HOME%/bin/assemblies directory exactly matches the library " +
						"files included in the WAR file or the " +
						"%PORTAL_HOME%/webapp/portal/web/bin directory.  This may also indicate a " +
						"bad install.";
			}		
			else if (exceptionName.compareToIgnoreCase("XPIndexOutOfBoundsException") == 0)
			{
				message = "OpenFoundation IndexOutOfBounds exception. This can occur through a variety of " +
						"ways; it usually indicates either an improper install or a settings file is " +
						"missing.  Verify the settings files are present in the correct location " +
						"(I.E. alui\\settings\\portal).  If this occurs during " +
						"a web crawl, the original error may be very difficult to locate, and " +
						"you should contact Oracle Support.";
			}		
			else if (exceptionName.compareToIgnoreCase("XPMalformedURLException") == 0)
			{
				message = "OpenFoundation MalformedURL exception. May indicate a bad URL on a web service, " +
						"remote server, or activity space.  Try to isolate what caused the exception. Somewhat " +
						"rare, since much of the portal does URL validation.";
			}		
			else if (exceptionName.compareToIgnoreCase("XPNotImplementedException") == 0)
			{
				message = "OpenFoundation NotImplemented exception. Indicates you've called either a portal " +
						"server or openkernel API call that is not yet implemented on the specific platform " +
						"you are running on.";
			}		
			else if (exceptionName.compareToIgnoreCase("XPParseException") == 0)
			{
				message = "OpenFoundation Parse exception. Usually indicates corruption of an XML file, a " +
						"setting within that file, or corruption of a database persisted property bag.  " +
						"Try to determine the root cause, and then contact Oracle Support.";
			}		
			else if (exceptionName.compareToIgnoreCase("XPThreadInterruptedException") == 0)
			{
				message = "OpenFoundation Thread Interrupted exception. Thread interrupts may occur during " +
						"long web or document crawls, or due to a browser being terminated in the middle " +
						"of uploading a document or while receiving a web page response.  Try to determine " +
						"the root cause of the exception.  May not necessarily be serious.";
			}		
			else if ((exceptionName.compareToIgnoreCase("XPFileNotFoundException") == 0)||
			        (exceptionName.compareToIgnoreCase("XPNullPointerException") == 0)||
			        (exceptionName.compareToIgnoreCase("XPIllegalArgumentException") == 0)||
					(exceptionName.compareToIgnoreCase("XPIOException") == 0)||
			        (exceptionName.compareToIgnoreCase("XPException") == 0))
			{
				// DavidP add on the message component
			    if ((te.GetMessage() != null)&&(te.GetMessage().length() > 0))
			    {
			        message = te.GetMessage();
			        secondLine = "";
			        if (te.GetInnerException() != null)
			        {
			            String innerMsg = te.GetStackTrace();
			            String firstStack = ConvertStackTraceToShortString( innerMsg, false );
			            String secondStack = ConvertStackTraceToShortString( innerMsg, true );
		                secondLine = "Stack Trace: " + firstStack;
		                if (secondStack != null)
		                    secondLine += cr + secondStack;
			        }
			    }
			    else  if (te.GetInnerException() != null)
			    {
		            String innerMsg = te.GetStackTrace();
		            String firstStack = ConvertStackTraceToShortString( innerMsg, false );
		            String secondStack = ConvertStackTraceToShortString( innerMsg, true );
	                message = "Stack Trace: " + firstStack;
	                if (secondStack != null)
	                    message += cr + secondStack;
			    } else {
					message = exceptionName;			    	
			    }
			}
			else
			{
				message = exceptionName;
			}
	    }
		
	    if (message.length() > 1)
	        message = message + cr;
		
		return message;
	}
}
