package com.plumtree.uiinfrastructure.approval;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 *
 *
 * @author $author$
 */
public class ApprovalControl extends EditorRepostControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.approval.ApprovalControl");


	public static final String STR_MVC_CLASS_NAME = "ApprovalControl";


	public static final String PARAM_APPLY		  = "in_hi_apply";


	public static final String PARAM_FINISH		  = "in_hi_finish";


	public static final String PARAM_ADD_COMMENT  = "in_hi_addcom";


	public static final String PARAM_APPROVE_ALL  = "in_hi_aprall";


	public static final String PARAM_REJECT_ALL   = "in_hi_rejall";


	public static final String PARAM_HOLD_ALL     = "in_hi_hoall";


	public static final String PARAM_CURRENT_TYPE = "in_hi_cutyp";


	public static final String PARAM_PREV_PAGE = "in_hi_prep";


	public static final String PARAM_NEXT_PAGE = "in_hi_nextp";


	public static final String PARAM_GOTO_PAGE = "in_hi_gotop";


	public static final String PARAM_GOTO_TYPE = "in_hi_gotot";


	public static final String PARAM_EDIT_ITEM = "in_hi_editit";


	public static final String PARAM_CLEAR = "in_hi_clear";


	public final static int POSTTOSELF_ACTION_SORT_SUBMITTEDBY = 18;


	public final static int POSTTOSELF_ACTION_SORT_NAME = 19;


	public final static int POSTTOSELF_ACTION_SORT_DATE = 20;


	public final static int POSTTOSELF_ACTION_APPLY		    = 10;


	public final static int POSTTOSELF_ACTION_PREVIOUS_PAGE = 11;


	public final static int POSTTOSELF_ACTION_NEXT_PAGE   = 12;


	public final static int POSTTOSELF_ACTION_ADD_COMMENT = 13;


	public final static int POSTTOSELF_ACTION_GOTO_TYPE   = 14;


	public final static int POSTTOSELF_ACTION_APPROVE_ALL = 15;


	public final static int POSTTOSELF_ACTION_REJECT_ALL = 16;


	public final static int POSTTOSELF_ACTION_HOLD_ALL = 17;


	public final static int POSTTOSELF_ACTION_CLEAR = 21;


	public final static int POSTTOSELF_ACTION_SELECT_ALL = 22;

	
	public static final int POSTTOSELF_ACTION_GOTO_RANGE = 23;

	public final static int POSTTOSELF_ACTION_UNSELECT_ALL = 24;

	XPHashtable htArgs = null;

	/**
	 *
	 *
	 * @param arguments
	 * @param type
	 */
	public void Apply(XPHashtable arguments, ApprovalItemType type)
	{
		IXPEnumerator enumKeys;
		String sKey;
		int iRadioIndex;
		int iItemIndex;
		int iDescriptionIndex;
		int iNameIndex;
		int iClearIndex;
		int iCommentIndex;

		String sPrimaryID;
		String sInstanceID;
		String sTemp;
		String sVal;
		String[] saVals;

		AItemDescriptor item		 = null;
		AInstanceDescriptor instance = null;

		
			log.Debug("ENTERING Apply() - type: " + type);
		

		enumKeys = arguments.GetKeys();

		while (enumKeys.MoveNext())
		{
			sKey     = (String) enumKeys.GetCurrent();

			iRadioIndex		  = -1;
			iItemIndex		  = -1;
			iCommentIndex     = -1;
			iClearIndex		  = -1;

			iRadioIndex     = sKey.lastIndexOf(ApprovalView.INSTANCE_RADIO_PREFIX);

			iItemIndex     = sKey.lastIndexOf(ApprovalView.ITEM_RADIO_PREFIX);

			iCommentIndex     = sKey.lastIndexOf(ApprovalView.COMMENT_PREFIX);

			iDescriptionIndex     = sKey.lastIndexOf(ApprovalView.DESCRIPTION_PREFIX);

			iClearIndex = sKey.lastIndexOf(ApprovalView.ITEM_CLEAR_PREFIX);

			iNameIndex = sKey.lastIndexOf(ApprovalView.NAME_PREFIX );

			if (iRadioIndex != -1)
			{
				sTemp     = sKey.substring(sKey.indexOf("X") + 1, sKey.length());

				sPrimaryID     = sTemp.substring(0, sTemp.indexOf("X"));

				sInstanceID = sTemp.substring(sTemp.indexOf("X") + 1,
						sTemp.length());

				
					log.Debug("Apply() - Found an instance radio button.");
					log.Debug("Apply() - sPrimaryID: " + sPrimaryID);
					log.Debug("Apply() - sInstanceID: " + sInstanceID);
				

				item = ((AApprovalModel) m_model).GetItemDescriptor(sPrimaryID);

				if (item == null)
				{
					
						log.Error("Apply() - Couldn't find item: " + sPrimaryID);
					

					UIError error = new UIError(UIErrorLevel.SERIOUS,
							UIErrorScope.CURRENT_USER,
							"Couldnt find the item you were attempting to approve.",
							"Couldn't find item: " + sPrimaryID);

					error.SetActivitySpace(((AApprovalModel) m_model).m_asOwner.GetName());
					error.SetClearWithActivitySpaceChange(true);
					error.SetManualClear(true);

					UIErrorHandler.RaiseError(error,
						((AApprovalModel) m_model).m_asOwner.GetSubSession(),
						((AApprovalModel) m_model).m_asOwner.GetApplication());

					continue;
				}
				else
				{
					
						log.Debug("Apply() - Found the item, about to get the instance.");
					

					instance = item.GetInstance(sInstanceID);
				}

				saVals     = (String[]) arguments.GetElement(sKey);

				sVal = saVals[0];

				if (instance == null)
				{
					
						log.Error("Apply() - Couldn't find the instance.");
					
				}
				else
				{
					
						log.Debug("Apply() - Found the instance, about to set the status.");
					

					if (ApprovalView.VAL_APPROVE.equals(sVal))
					{
						instance.Approve();
					}
					else if (ApprovalView.VAL_REJECT.equals(sVal))
					{
						instance.Reject();
					}
					else if (ApprovalView.VAL_HOLD.equals(sVal))
					{
						instance.Hold();
					}
				}
				 // instance != null
			}
			else if (iItemIndex != -1)
			{
				
					log.Debug("Apply() - Found an item radio button. iItemIndex: " +
						iItemIndex);
					log.Debug("Apply() - Found an item radio button. sKey: " + sKey);
				

				sPrimaryID     = sKey.substring(sKey.indexOf("X") + 1,
						sKey.length());

				item = ((AApprovalModel) m_model).GetItemDescriptor(sPrimaryID);

				if (item == null)
				{
					
						log.Error("Couldn't find item: " +
							sPrimaryID);
					

					UIError error = new UIError(UIErrorLevel.SERIOUS,
							UIErrorScope.CURRENT_USER,
							"Couldnt find the item you were attempting to approve.",
							"Couldn't find item: " + sPrimaryID);

					error.SetActivitySpace(((AApprovalModel) m_model).m_asOwner.GetName());
					error.SetClearWithActivitySpaceChange(true);
					error.SetManualClear(true);

					UIErrorHandler.RaiseError(error,
						((AApprovalModel) m_model).m_asOwner.GetSubSession(),
						((AApprovalModel) m_model).m_asOwner.GetApplication());

					continue;
				}

				saVals     = (String[]) arguments.GetElement(sKey);

				sVal = saVals[0];

				
					log.Debug("Apply() - sPrimaryID: " + sPrimaryID);
					log.Debug("Apply() - sVal: " + sVal);
				

				if (item != null)
				{
					if (ApprovalView.VAL_APPROVE.equals(sVal))
					{
						item.Approve();
					}
					else if (ApprovalView.VAL_REJECT.equals(sVal))
					{
						item.Reject();
					}
					else if (ApprovalView.VAL_HOLD.equals(sVal))
					{
						item.Hold();
					}
				}
				 // item != null
			}
			 //   item radio button
			else if (iCommentIndex != -1)
			{
				//   comment
				sPrimaryID     = sKey.substring(sKey.indexOf("X") + 1,
						sKey.length());

				item = ((AApprovalModel) m_model).GetItemDescriptor(sPrimaryID);

				if (item == null)
				{
					
						log.Error("Couldn't find item: " +
							sPrimaryID);
					

					UIError error = new UIError(UIErrorLevel.SERIOUS,
							UIErrorScope.CURRENT_USER,
							"Couldnt find the item you were attempting to approve.",
							"Couldn't find item: " + sPrimaryID);

					error.SetActivitySpace(((AApprovalModel) m_model).m_asOwner.GetName());
					error.SetClearWithActivitySpaceChange(true);
					error.SetManualClear(true);

					UIErrorHandler.RaiseError(error,
						((AApprovalModel) m_model).m_asOwner.GetSubSession(),
						((AApprovalModel) m_model).m_asOwner.GetApplication());

					continue;
				}

				saVals = (String[]) arguments.GetElement(sKey);

				item.SetComment(saVals[0]);
			}
			else if (iDescriptionIndex != -1)
			{
				//   description
				sPrimaryID     = sKey.substring(sKey.indexOf("X") + 1,
						sKey.length());

				item = ((AApprovalModel) m_model).GetItemDescriptor(sPrimaryID);

				if (item == null)
				{
					
						log.Error("Couldn't find item: " +
							sPrimaryID);
					

					UIError error = new UIError(UIErrorLevel.SERIOUS,
							UIErrorScope.CURRENT_USER,
							"Couldnt find the item you were attempting to approve.",
							"Couldn't find item: " + sPrimaryID);

					error.SetActivitySpace(((AApprovalModel) m_model).m_asOwner.GetName());
					error.SetClearWithActivitySpaceChange(true);
					error.SetManualClear(true);
					UIErrorHandler.RaiseError(error,
						((AApprovalModel) m_model).m_asOwner.GetSubSession(),
						((AApprovalModel) m_model).m_asOwner.GetApplication());

					continue;
				}

				saVals = (String[]) arguments.GetElement(sKey);

				item.SetDescription(saVals[0]);
			}
			else if (iClearIndex != -1)
			{
				sPrimaryID     = sKey.substring(sKey.indexOf("X") + 1,
						sKey.length());

				item = ((AApprovalModel) m_model).GetItemDescriptor(sPrimaryID);

				if (item == null)
				{
					
						log.Error("Couldn't find item: " +
							sPrimaryID);
					

					UIError error = new UIError(UIErrorLevel.SERIOUS,
							UIErrorScope.CURRENT_USER,
							"Couldnt find the item you were attempting to approve.",
							"Couldn't find item: " + sPrimaryID);

					error.SetActivitySpace(((AApprovalModel) m_model).m_asOwner.GetName());
					error.SetClearWithActivitySpaceChange(true);
					error.SetManualClear(true);
					UIErrorHandler.RaiseError(error,
						((AApprovalModel) m_model).m_asOwner.GetSubSession(),
						((AApprovalModel) m_model).m_asOwner.GetApplication());

					continue;
				}


				saVals = (String[]) arguments.GetElement(sKey);

				sVal = saVals[0];

				if( sVal != null )
				{
					item.SetClear( true );
				}

			}
			else if( iNameIndex != -1 )
			{
				//   name
				sPrimaryID     = sKey.substring(sKey.indexOf("X") + 1,
						sKey.length());

				item = ((AApprovalModel) m_model).GetItemDescriptor(sPrimaryID);

				if (item == null)
				{
					
						log.Error("Couldn't find item: " +
							sPrimaryID);
					

					UIError error = new UIError(UIErrorLevel.SERIOUS,
							UIErrorScope.CURRENT_USER,
							"Couldnt find the item you were attempting to approve.",
							"Couldn't find item: " + sPrimaryID);

					error.SetActivitySpace(((AApprovalModel) m_model).m_asOwner.GetName());
					error.SetClearWithActivitySpaceChange(true);
					error.SetManualClear(true);

					UIErrorHandler.RaiseError(error,
						((AApprovalModel) m_model).m_asOwner.GetSubSession(),
						((AApprovalModel) m_model).m_asOwner.GetApplication());

					continue;
				}

				saVals = (String[]) arguments.GetElement(sKey);

				item.SetName(saVals[0]);


			}


		}
		 //   for each parameter

		int iCurrent = ((AApprovalModel) m_model).GetCurrentPageIndex(type);

		if (!((AApprovalModel) m_model).IsPageAvailable(type, iCurrent + 1))
		{
			((AApprovalModel) m_model).GotoPage(type, 1);
		}
	}

	/**
	 *
	 *
	 * @param type
	 */
	public void ApproveAll(ApprovalItemType type)
	{
		AItemDescriptor item = null;

		
			log.Debug("ENTERING ApproveAll() - type: " + type.GetName());
		

		IXPEnumerator eTemp = ((AApprovalModel) m_model).GetItemDescriptors(type);

		while (eTemp.MoveNext())
		{
			item = (AItemDescriptor) eTemp.GetCurrent();

			item.Approve();
		}
	}

	/**
	 *
	 *
	 * @param type
	 * @param bSelect true is select, false is unselect
	 */
	public void SelectUnselectAll(ApprovalItemType type, boolean bSelect)
	{
		AItemDescriptor item = null;

		IXPEnumerator eTemp = ((AApprovalModel) m_model).GetItemDescriptors(type);

		while (eTemp.MoveNext())
		{
			item = (AItemDescriptor) eTemp.GetCurrent();

			item.SetClear( bSelect );

		}
	}


	/**
	 * @see com.plumtree.xpshared.activityspace.IControl#execute()
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		
			log.Debug("ENTERING Execute()");
		

		htArgs = arguments;

		String[] saVals;
		ApprovalItemType typeTemp;
		int oi     = 0;
		String sID = null;

		ApprovalItemType t = ((AApprovalModel) m_model).GetDefaultItemType();

		//    ADD COMMENT
		saVals = (String[]) arguments.GetElement(ApprovalControl.PARAM_ADD_COMMENT);

		if (saVals != null)
		{
			// item id
			sID = saVals[0];
		}

		if (sID != null)
		{
			if (!"".equals(sID))
			{
				((AApprovalModel) m_model).SetAddComment(true);
				((AApprovalModel) m_model).SetItemID(sID);
			}
		}

		//    PREVIOUS PAGE
		typeTemp     = null;
		saVals		 = (String[]) arguments.GetElement(ApprovalControl.PARAM_PREV_PAGE);

		if (saVals != null)
		{
			typeTemp = t.GetType(saVals[0]);
		}

		if (typeTemp != null)
		{
			((AApprovalModel) m_model).PreviousPage(typeTemp);
		}

		//    NEXT PAGE
		typeTemp     = null;
		saVals		 = (String[]) arguments.GetElement(ApprovalControl.PARAM_NEXT_PAGE);

		if (saVals != null)
		{
			typeTemp = t.GetType(saVals[0]);
		}

		if (typeTemp != null)
		{
			((AApprovalModel) m_model).NextPage(typeTemp);
		}

		//    GOTO TYPE
		typeTemp     = null;
		saVals		 = (String[]) arguments.GetElement(ApprovalControl.PARAM_GOTO_TYPE);

		if (saVals != null)
		{
			typeTemp = t.GetType(saVals[0]);
		}

		if (typeTemp != null)
		{
			try
			{
				saVals = (String[]) arguments.GetElement(ApprovalControl.PARAM_GOTO_PAGE);

				if (saVals != null)
				{
					oi = XPConvert.ToInteger(saVals[0]);
				}
			}
			catch (Exception ex)
			{
				// do nothing
			}

			if (oi != 0)
			{
				((AApprovalModel) m_model).GotoPage(typeTemp, oi);
			}
		}

		//    CURRENT TYPE
		saVals = (String[]) arguments.GetElement(ApprovalControl.PARAM_CURRENT_TYPE);

		if (saVals != null)
		{
			((AApprovalModel) m_model).SetCurrentType(t.GetType(saVals[0]));
		}

		//    EDIT ITEM
		saVals = (String[]) arguments.GetElement(ApprovalControl.PARAM_EDIT_ITEM);

		if (saVals != null)
		{
			AItemDescriptor item = ((AApprovalModel) m_model).GetItemDescriptor(saVals[0]);

			if (item != null)
			{
				Redirect r = item.GetEditRedirect();

				((AApprovalModel) m_model).ClearCache();

				return r;
			}
		}

		return super.CheckActionSecurityAndExecute(arguments);
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IControl#setData(String, String)
	 */
	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ApprovalControl();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 *
	 *
	 * @param type
	 */
	public void HoldAll(ApprovalItemType type)
	{
		if (type == null)
		{
			return;
		}

		log.Debug("ENTERING HoldAll() - type: " + type.GetName());

		IXPEnumerator enumerator;
		AItemDescriptor item = null;

		enumerator = ((AApprovalModel) m_model).GetItemDescriptors(type);

		while (enumerator.MoveNext())
		{
			item = (AItemDescriptor) enumerator.GetCurrent();

			item.Hold();
		}
	}

	/**
	 *
	 */
	public void PerformFinishAction()
	{
		
			log.Debug("ENTERING PerformFinishAction()");
		

		Apply(this.m_htFormData, ((AApprovalModel) m_model).GetCurrentType());

		this.ExecuteStatusChanges();

		super.PerformFinishAction();
	}

	/**
	 *
	 *
	 * @param type
	 */
	public void RejectAll(ApprovalItemType type)
	{
		AItemDescriptor item = null;

		if (type == null)
		{
			return;
		}

		
			log.Debug("ENTERING RejectAll() - type: " + type.GetName());
		

		IXPEnumerator eTemp = ((AApprovalModel) m_model).GetItemDescriptors(type);

		while (eTemp.MoveNext())
		{
			item = (AItemDescriptor) eTemp.GetCurrent();

			item.Reject();
		}
	}


	void InitializeClearState( ApprovalItemType type )
	{
		IXPEnumerator eItems = ((AApprovalModel) this.m_model).GetItemDescriptors( type );

		AItemDescriptor i = null;

		while( eItems.MoveNext() )
		{
			i = (AItemDescriptor) eItems.GetCurrent();
			i.SetClear( false );
		}
	}



	/**
	 *
	 *
	 * @param _nAction
	 */
	protected void PerformAction(int _nAction)
	{
		
			log.Debug("ENTERING PerformAction()");
		

		super.PerformAction(_nAction);

		switch (_nAction)
		{
			case POSTTOSELF_ACTION_SELECT_ALL:

				SelectUnselectAll(((AApprovalModel) m_model).GetCurrentType(), true);

				break;

			case POSTTOSELF_ACTION_UNSELECT_ALL:

			SelectUnselectAll(((AApprovalModel) m_model).GetCurrentType(), false);

				break;


		case POSTTOSELF_ACTION_SORT_DATE:
			SortDate();

			break;

		case POSTTOSELF_ACTION_SORT_NAME:

			
				log.Debug("Sort Name");
			

			SortName();

			break;

		case POSTTOSELF_ACTION_SORT_SUBMITTEDBY:

			
				log.Debug("Sort Submittedby");
			

			SortSubmittedBy();

			break;

		case POSTTOSELF_ACTION_APPLY:

			
				log.Debug("Apply Changes");
			

			ExecuteStatusChanges();

			break;

		case POSTTOSELF_ACTION_CLEAR:

			
				log.Debug("Clear");
			


			InitializeClearState(((AApprovalModel) m_model).GetCurrentType() );

			Apply( this.htArgs, ((AApprovalModel) m_model).GetCurrentType() );

			ExecuteStatusChanges();

			break;

		case POSTTOSELF_ACTION_APPROVE_ALL:

			
				log.Debug("Approve All");
			

			ApproveAll(((AApprovalModel) m_model).GetCurrentType());

			break;

		case POSTTOSELF_ACTION_REJECT_ALL:

			
				log.Debug("Reject All");
			

			RejectAll(((AApprovalModel) m_model).GetCurrentType());

			break;

		case POSTTOSELF_ACTION_HOLD_ALL:

			
				log.Debug("Hold All");
			

			HoldAll(((AApprovalModel) m_model).GetCurrentType());

			break;			
			
		case POSTTOSELF_ACTION_GOTO_RANGE:
			log.Info("GOTO RANGE");

			int nFromRange;
			int nToRange;
			String[] sValues;

			// From
			sValues = (String[]) 
				m_htFormData.GetElement(ApprovalView.HTMLINPUT_FROM);

			if (sValues == null)
			{
				
					log.Error("could not find the FROM range value in query string.");
				

				return;
			}
			else
			{
				nFromRange = (XPConvert.ToInteger(sValues[0]) - 1);
			}

			// To
			sValues = (String[]) 
				m_htFormData.GetElement(ApprovalView.HTMLINPUT_TO);

			if (sValues == null)
			{
				
					log.Error("could not find the TO range value in query string.");
				

				return;
			}
			else
			{
				nToRange = (XPConvert.ToInteger(sValues[0]) - 1);
			}

			/* use the model to set the indices */
			ApprovalItemType type = ((AApprovalModel) this.m_model).GetCurrentType();
			((AApprovalModel) this.m_model).SetPageLowIndex(nFromRange);
			((AApprovalModel) this.m_model).SetPageHighIndex(nToRange);
			
			/* clear cache */
			((AApprovalModel) this.m_model).ClearCache(type);

			return;

		}
	}

	/**
	 *
	 */
	protected void SaveCurrentPage()
	{
		
			log.Debug("ENTERING SaveCurrentPage()");
		

		Apply(this.m_htFormData, ((AApprovalModel) m_model).GetCurrentType());
	}

	/**
	 *
	 */
	private void ExecuteStatusChanges()
	{
		
			log.Debug("ENTERING ExecuteStatusChanges()");
		

		IXPEnumerator eTemp		 = ((AApprovalModel) m_model).GetAllCachedItems();
		IXPEnumerator eInstances;

		AItemDescriptor item;
		AInstanceDescriptor instance;

		while (eTemp.MoveNext())
		{
			item = null;

			try
			{
				item = (AItemDescriptor) eTemp.GetCurrent();
			}
			catch (Exception ex)
			{
				
					log.Error("ExecuteStatusChanges() - Couldn't get item descriptor: " +
						eTemp.GetCurrent());
				
			}

			if (item != null)
			{
				
					log.Debug("ExecuteStatusChanges() - item ID: " + item.GetID());
				

				item.Execute();

				eInstances = item.GetInstances();

				if (eInstances != null)
				{
					while (eInstances.MoveNext())
					{
						instance = (AInstanceDescriptor) eInstances.GetCurrent();

						
							log.Debug("ExecuteStatusChanges() - instance ID: " +
								instance.GetID());
						

						instance.Execute();
					}
				}
			}
		}

		((AApprovalModel) m_model).ClearCache();
	}

	/**
	 *
	 */
	private void SortDate()
	{
		
			log.Debug("SortDate()");
		

		((AApprovalModel) m_model).SetDoSortName(false);
		((AApprovalModel) m_model).SetDoSortSubmittedBy(false);
		((AApprovalModel) m_model).SetDoSortDate(true);

		((AApprovalModel) m_model).ClearCache();
	}

	/**
	 *
	 */
	private void SortName()
	{
		
			log.Debug("SortName()");
		

		((AApprovalModel) m_model).SetDoSortName(true);
		((AApprovalModel) m_model).SetDoSortSubmittedBy(false);
		((AApprovalModel) m_model).SetDoSortDate(false);

		((AApprovalModel) m_model).ClearCache();
	}

	/**
	 *
	 */
	private void SortSubmittedBy()
	{
		
			log.Debug("SortSubmittedBy()");
		

		((AApprovalModel) m_model).SetDoSortName(false);
		((AApprovalModel) m_model).SetDoSortSubmittedBy(true);
		((AApprovalModel) m_model).SetDoSortDate(false);

		((AApprovalModel) m_model).ClearCache();
	}
}

