package com.plumtree.uiinfrastructure.approval;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.editor.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * View class for approving items
 *
 * @author BenG
 */
public class ApprovalView extends EditorView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.approval.ApprovalView");

	/** The unique identifier for the class */
	public static final String STR_MVC_CLASS_NAME = "Approval";

	/** Comment textarea */
	public static final String COMMENT_PREFIX     = "in_ta_cmntX";

	/** Name text box */
	public static final String NAME_PREFIX     = "in_ta_nmX";

	/** Description text area */
	public static final String DESCRIPTION_PREFIX = "in_ta_dscrX";

	/** Instance radio button */
	public static final String INSTANCE_RADIO_PREFIX = "in_ra_insX";

	/** Item radio button */
	public static final String ITEM_RADIO_PREFIX = "in_ra_itX";

	/** Item clear radio button */
	public static final String ITEM_CLEAR_PREFIX = "in_cb_itX";

	/** Approve constant */
	public static final String VAL_APPROVE = "appr";

	/** Reject constant */
	public static final String VAL_REJECT = "reje";

	/** Hold constant */
	public static final String VAL_HOLD = "hold";

	/** HTML navigation fields */
	public static final String HTMLINPUT_FROM	    = "in_tx_From";
	public static final String HTMLINPUT_TO		    = "in_tx_To";
	public static final String HTMLBUTTON_GOTORANGE = "in_bu_GotoRange";
	public static final String JAVASCRIPT_CARDRANGE = "GoToCardRange";
	
	private HTMLImg folderImg		    = null;
	private HTMLImg imgNext = null;
	private HTMLImg imgPrev = null;
	private XPHashtable xmpItemImage		  = new XPHashtable();
	private int iMaxNumPageSelectorsDisplayed = 333;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ApprovalView();
	}

	/**
	 * Displays the HTML for the view
	 *
	 * @return HTMLElement
	 */
	public HTMLElement Display()
	{
		return Display(null);
	}

	private XPDateFormat GetDateFormat()
	{
		XPDateFormat dateFormat;

		String strLocale = (String) PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LOCALE, m_asOwner);

		XPLocale l = new XPLocale(strLocale);

		dateFormat = XPDateFormat.GetDateInstance( XPDateFormat.SHORT, l, null );
			
		return dateFormat;	
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
	 */
	public HTMLElement Display(ApprovalItemType type)
	{
		HTMLElement result = new HTMLElementCollection();

		HTMLInput myInput;

		HTMLTable myTable = null;

		if( type == null )
		{

			myTable = new HTMLTable();
			myTable.SetWidth( CommonHTMLStrings.ONE_HUNDRED_PERCENT );
			myTable.SetBorder( CommonHTMLStrings.ZERO );


			result.AddInnerHTMLElement( myTable );

			this.AddNoItemsRowToTable( myTable, PTStyleClass.LIST_ITEM_ONE_BG );

			return result;
		}



		
			log.Debug("Entering Display().");
			log.Debug("Display() - type: " + type.GetName());
		

		String sTypeName = type.GetName();

		HTMLScript myScript;
		XPStringBuilder sb = new XPStringBuilder();

		String sFormName = "mainform";

		try
		{
			myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
			result.AddInnerHTMLElement(myScript);

			if (((AApprovalModel) m_model).GetAddComment())
			{
				// The URL to open the pop-up
				ASURL myPopUp = new ASURL(m_asOwner);
				myPopUp.SetLinkGetCachedSpace(m_asOwner);
				myPopUp.SetDisplayPage(CommentDP.STR_MVC_CLASS_NAME);
				myPopUp.SetControl(ApprovalControl.STR_MVC_CLASS_NAME);

				sb.Append("    window.open('").Append(myPopUp.GetURLAsString())
				.Append("', '_blank3', 'location=no,menubar=no,height=200,width=400');\n");
			}

			sb.Append("function changePrimary( sItemID )\n");
			sb.Append("{\n");
			sb.Append("	\n");

			sb.Append("			var form = document.");
			sb.Append(sFormName);
			sb.Append(";\n");
			sb.Append("			var sRadioName = '");
			sb.Append(ITEM_RADIO_PREFIX);
			sb.Append("' + sItemID;\n");

			sb.Append("			var instanceRadioName = '");

			sb.Append(INSTANCE_RADIO_PREFIX);
			sb.Append("' + sItemID;\n");

			sb.Append("			var formelements = form.elements;\n");
			sb.Append("			var iNumElements = formelements.length;\n");
			sb.Append("			var formElement;\n");
			sb.Append("			var checkedValue;\n");

			sb.Append("			for( x = 0; x < iNumElements; x++ )\n");
			sb.Append("			{\n");
			sb.Append("				formElement = formelements[ x ];\n");

			sb.Append("				if( formElement.name == sRadioName )\n");
			sb.Append("				{\n");
			sb.Append("					if( formElement.checked )\n");
			sb.Append("					{\n");

			sb.Append("						checkedValue = formElement.value;\n");
			sb.Append("					}\n");
			sb.Append("				}\n");
			sb.Append("			}\n");

			sb.Append("			for( x = 0; x < iNumElements; x++ )\n");
			sb.Append("			{\n");
			sb.Append("				formElement = formelements[ x ];\n");

			sb.Append(
				"				if( formElement.name.indexOf( instanceRadioName ) != -1 )\n");
			sb.Append("				{\n");
			sb.Append("					if( formElement.value == checkedValue )\n");
			sb.Append("					{\n");
			sb.Append("						formElement.checked = true;\n");
			sb.Append("					}\n");
			sb.Append("					else\n");
			sb.Append("					{\n");
			sb.Append("						formElement.checked = false;\n");
			sb.Append("					}\n");

			sb.Append("				}\n");
			sb.Append("			}\n");

			sb.Append("	return true;\n");
			sb.Append("}\n");

			sb.Append("\n");
			sb.Append("function changeInstance( sItemID )\n");
			sb.Append("{\n");
			sb.Append("	\n");

			sb.Append("			var form = document.");
			sb.Append(sFormName);
			sb.Append(";\n");
			sb.Append("			var sRadioName = '");
			sb.Append(ITEM_RADIO_PREFIX);
			sb.Append("' + sItemID;\n");

			sb.Append("			var formelements = form.elements;\n");
			sb.Append("			var iNumElements = formelements.length;\n");
			sb.Append("			var formElement;\n");
			sb.Append("			var checkedValue;\n");

			sb.Append("			for( x = 0; x < iNumElements; x++ )\n");
			sb.Append("			{\n");
			sb.Append("				formElement = formelements[ x ];\n");

			sb.Append("				if( formElement.name == sRadioName )\n");
			sb.Append("				{\n");
			sb.Append("					if( formElement.checked )\n");
			sb.Append("					{\n");

			sb.Append("						formElement.checked = false;\n");

			sb.Append("					}\n");
			sb.Append("				}\n");
			sb.Append("			}\n");

			sb.Append("	\n");
			sb.Append("	return true;\n");
			sb.Append("}\n");





			sb.Append("function gotoApprovalPage( page, type )\n");
			sb.Append("{\n");



			sb.Append("var form = document." );
			sb.Append( sFormName );
			sb.Append(";\n");

			sb.Append("var ea = form.elements;\n");

			sb.Append("	var i;\n");
			sb.Append("	var anInput;\n");
			sb.Append("	var gotoInput;\n");
			sb.Append("	var typeInput;\n");

			sb.Append("	for( i = 0; i < ea.length; i++ )\n");
			sb.Append("	{\n");
			sb.Append("		anInput = ea[ i ];\n");

			sb.Append("		if( anInput.name == '" );
			sb.Append(ApprovalControl.PARAM_GOTO_PAGE);
			sb.Append("' )\n");
			sb.Append("		{\n");
			sb.Append("			gotoInput = anInput;\n");
			sb.Append("		}\n");

			sb.Append("		if( anInput.name == '" );
			sb.Append(ApprovalControl.PARAM_GOTO_TYPE);
			sb.Append("' )\n");
			sb.Append("		{\n");
			sb.Append("			typeInput = anInput;\n");
			sb.Append("		}\n");

			sb.Append("	}\n");

			sb.Append("	gotoInput.value = page;\n");

			sb.Append("	typeInput.value = type;\n");

			sb.Append("	form.submit();\n");


			sb.Append("}\n");








			sb.Append("function prev( typename )\n");
			sb.Append("{\n");

			sb.Append("			var form = document.");
			sb.Append(sFormName);
			sb.Append(";\n");


			sb.Append("	var prev;\n");

			sb.Append("var ea = form.elements;\n");

			sb.Append("	var i;\n");
			sb.Append("	var anInput;\n");


			sb.Append("	for( i = 0; i < ea.length; i++ )\n");
			sb.Append("	{\n");
			sb.Append("		anInput = ea[ i ];\n");

			sb.Append("		if( anInput.name == '" );
			sb.Append(ApprovalControl.PARAM_PREV_PAGE);
			sb.Append("' )\n");
			sb.Append("		{\n");
			sb.Append("			prev = anInput;\n");
			sb.Append("		}\n");

			sb.Append("	}\n");

			sb.Append("	prev.value = typename;\n");

			sb.Append("	form.submit();\n");

			sb.Append("}\n");






			sb.Append("function editItem( it )\n");
			sb.Append("{\n");

			sb.Append("			var form = document.");
			sb.Append(sFormName);
			sb.Append(";\n");

			sb.Append("var ea = form.elements;\n");

			sb.Append("	var i;\n");
			sb.Append(" var item;\n");
			sb.Append("	var anInput;\n");


			sb.Append("	for( i = 0; i < ea.length; i++ )\n");
			sb.Append("	{\n");
			sb.Append("		anInput = ea[ i ];\n");

			sb.Append("		if( anInput.name == '" );
			sb.Append(ApprovalControl.PARAM_EDIT_ITEM);
			sb.Append("' )\n");
			sb.Append("		{\n");
			sb.Append("			item = anInput;\n");
			sb.Append("		}\n");

			sb.Append("	}\n");


			sb.Append("	item.value = it;\n");

			sb.Append("	form.submit();\n");

			sb.Append("}\n");





			sb.Append("function next( typename )\n");
			sb.Append("{\n");

			sb.Append("			var form = document.");
			sb.Append(sFormName);
			sb.Append(";\n");

			sb.Append("	var prev;\n");

			sb.Append("var ea = form.elements;\n");

			sb.Append("	var i;\n");
			sb.Append("	var anInput;\n");


			sb.Append("	for( i = 0; i < ea.length; i++ )\n");
			sb.Append("	{\n");
			sb.Append("		anInput = ea[ i ];\n");

			sb.Append("		if( anInput.name == '" );
			sb.Append(ApprovalControl.PARAM_NEXT_PAGE);
			sb.Append("' )\n");
			sb.Append("		{\n");
			sb.Append("			prev = anInput;\n");
			sb.Append("		}\n");

			sb.Append("	}\n");


			sb.Append("	prev.value = typename;\n");

			sb.Append("	form.submit();\n");

			sb.Append("}\n");







			sb.Append("function expand( typename )\n");
			sb.Append("{\n");

			sb.Append("			var form = document.");
			sb.Append(sFormName);
			sb.Append(";\n");

			sb.Append("	var currentType;\n");

			sb.Append("var ea = form.elements;\n");

			sb.Append("	var i;\n");
			sb.Append("	var anInput;\n");


			sb.Append("	for( i = 0; i < ea.length; i++ )\n");
			sb.Append("	{\n");
			sb.Append("		anInput = ea[ i ];\n");

			sb.Append("		if( anInput.name == '" );
			sb.Append(ApprovalControl.PARAM_CURRENT_TYPE);
			sb.Append("' )\n");
			sb.Append("		{\n");
			sb.Append("			currentType = anInput;\n");
			sb.Append("		}\n");

			sb.Append("	}\n");


			sb.Append("	currentType.value = typename;\n");

			sb.Append("	form.submit();\n");

			sb.Append("}\n");




			sb.Append("function collapse( typename )\n");
			sb.Append("{\n");

			sb.Append("			var form = document.");
			sb.Append(sFormName);
			sb.Append(";\n");

			sb.Append("	var currentType = form.");
			sb.Append(ApprovalControl.PARAM_CURRENT_TYPE);
			sb.Append(";\n");


			sb.Append("var ea = form.elements;\n");

			sb.Append("	var i;\n");
			sb.Append("	var anInput;\n");


			sb.Append("	for( i = 0; i < ea.length; i++ )\n");
			sb.Append("	{\n");
			sb.Append("		anInput = ea[ i ];\n");

			sb.Append("		if( anInput.name == '" );
			sb.Append(ApprovalControl.PARAM_CURRENT_TYPE);
			sb.Append("' )\n");
			sb.Append("		{\n");
			sb.Append("			currentType = anInput;\n");
			sb.Append("		}\n");

			sb.Append("	}\n");


			sb.Append("	currentType.value = 'none';\n");

			sb.Append("	form.submit();\n");

			sb.Append("}\n");




			sb.Append("function doFinish()\n");
			sb.Append("{\n");

			sb.Append("			var form = document.");
			sb.Append(sFormName);
			sb.Append(";\n");

			sb.Append("	var doFinish;\n");


			sb.Append("var ea = form.elements;\n");

			sb.Append("	var i;\n");
			sb.Append("	var anInput;\n");


			sb.Append("	for( i = 0; i < ea.length; i++ )\n");
			sb.Append("	{\n");
			sb.Append("		anInput = ea[ i ];\n");

			sb.Append("		if( anInput.name == '" );
			sb.Append(ApprovalControl.PARAM_FINISH);
			sb.Append("' )\n");
			sb.Append("		{\n");
			sb.Append("			doFinish = anInput;\n");
			sb.Append("		}\n");

			sb.Append("	}\n");


			sb.Append("	doFinish.value = \"true\";\n");
			sb.Append("form.submit();\n");

			sb.Append("}\n");


			
			/* javascript for view items */
			if (((AApprovalModel) this.m_model).DisplayViewItems() &&
				((AApprovalModel) this.m_model).GetTypeCount(type) > 0)
			{
				int nObjectCount = ((IApprovalModelRO) m_model).GetTypeCount(type);

				sb.Append("function IsNumeric(strString) ");
				sb.Append("   //  check for valid numeric strings	 \n");
				sb.Append("   { \n");
				sb.Append("   var strValidChars = \"0123456789\"; \n");
				sb.Append("   var strChar; \n");
				sb.Append("   var blnResult = true; \n");
				sb.Append(" \n");
				sb.Append("   if (strString.length == 0) return false; \n");
				sb.Append(" \n");
				sb.Append(
					"   //  test strString consists of valid characters listed above \n");
				sb.Append(
					"   for (i = 0; i < strString.length && blnResult == true; i++) \n");
				sb.Append("      { \n");
				sb.Append("      strChar = strString.charAt(i); \n");
				sb.Append("      if (strValidChars.indexOf(strChar) == -1) \n");
				sb.Append("         { \n");
				sb.Append("         blnResult = false; \n");
				sb.Append("         } \n");
				sb.Append("      } \n");
				sb.Append("   return blnResult; \n");
				sb.Append("   } \n");
	
				sb.Append("function WithInRange(nValue)\n");
				sb.Append("{\n");
				sb.Append("  if (nValue < 1 || nValue > ").Append(nObjectCount).Append(")\n");
				sb.Append("  {\n");
	
				sb.Append("      alert(\"" +
					m_asOwner.GetString(466, ResourceConstants.PORTALINFRASTRUCTURE) +
					"\");\n");
				sb.Append("      return false;\n");
				sb.Append("  }\n");
				sb.Append("  return true;\n");
				sb.Append("}\n");
	
				sb.Append("function CheckNumericToFromValues()\n");
				sb.Append("{\n");
				sb.Append("  if (false == IsNumeric(document." + AFormDP.HTMLFORM_NAME +
					"." + HTMLINPUT_FROM + ".value))\n");
				sb.Append("  {\n");
	
				sb.Append("      alert(\"" +
					m_asOwner.GetString(466, ResourceConstants.PORTALINFRASTRUCTURE) +
					"\");\n");
				sb.Append("      return false;\n");
				sb.Append("  } else if (false == IsNumeric(document." +
					AFormDP.HTMLFORM_NAME + "." + HTMLINPUT_TO + ".value))\n");
				sb.Append("  {\n");
	
				sb.Append("      alert(\"" +
					m_asOwner.GetString(466, ResourceConstants.PORTALINFRASTRUCTURE) +
					"\");\n");
				sb.Append("      return false;\n");
				sb.Append("  }\n");
				sb.Append("  return true;\n");
				sb.Append("}\n");
	
				sb.Append("function ").Append(JAVASCRIPT_CARDRANGE).Append("()\n");
				sb.Append("{\n");
				sb.Append("  if (true == CheckNumericToFromValues())\n");
				sb.Append("  {\n");
				sb.Append("  	var nBegin = parseInt(document.").Append(AFormDP.HTMLFORM_NAME)
				.Append(".").Append(HTMLINPUT_FROM).Append(".value);\n");
				sb.Append("  	var nEnd = parseInt(document.").Append(AFormDP.HTMLFORM_NAME)
				.Append(".").Append(HTMLINPUT_TO).Append(".value);\n");
				sb.Append("     if (nBegin > nEnd)\n");
				sb.Append("     {\n");
	
				sb.Append("        alert(\"" +
					m_asOwner.GetString(467, ResourceConstants.PORTALINFRASTRUCTURE) +
					"\");\n");
				sb.Append(
					"     } else if(WithInRange(nBegin)) \n");
				sb.Append("     {\n");
				sb.Append("        ").Append(AFormDP.JAVASCRIPT_SUBMIT_FORM)
				.Append("(").Append(ApprovalControl.POSTTOSELF_ACTION_GOTO_RANGE)
				.Append(");\n");
				sb.Append("     }\n");
				sb.Append("  }\n");
				sb.Append("}\n");
			}

			myScript.AddInnerHTMLString(sb.ToString());


			// Sets the dologin parameter
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ApprovalControl.PARAM_APPLY, ApprovalControl.PARAM_APPLY,
					"Approval input");
			
			myInput.SetValue(XPConvert.ToString(false));
			result.AddInnerHTMLElement(myInput);

			// Sets the dologin parameter
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ApprovalControl.PARAM_APPROVE_ALL,
					ApprovalControl.PARAM_APPROVE_ALL, "Approve All input");
					
			myInput.SetValue(XPConvert.ToString(false));
			result.AddInnerHTMLElement(myInput);

			// Sets the dologin parameter
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ApprovalControl.PARAM_HOLD_ALL,
					ApprovalControl.PARAM_HOLD_ALL, "Hold input");
					
			myInput.SetValue(XPConvert.ToString(false));
			result.AddInnerHTMLElement(myInput);

			// Sets the dologin parameter
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ApprovalControl.PARAM_REJECT_ALL,
					ApprovalControl.PARAM_REJECT_ALL, "Reject All input");
					
			myInput.SetValue(XPConvert.ToString(false));
			result.AddInnerHTMLElement(myInput);

			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ApprovalControl.PARAM_PREV_PAGE,
					ApprovalControl.PARAM_PREV_PAGE, "Previous page input");
					
			myInput.SetValue(XPConvert.ToString(false));
			result.AddInnerHTMLElement(myInput);

			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ApprovalControl.PARAM_NEXT_PAGE,
					ApprovalControl.PARAM_NEXT_PAGE, "Next page input");
					
			myInput.SetValue(XPConvert.ToString(false));
			result.AddInnerHTMLElement(myInput);

			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ApprovalControl.PARAM_GOTO_PAGE,
					ApprovalControl.PARAM_GOTO_PAGE, "Goto page input");
			myInput.SetValue("");
			result.AddInnerHTMLElement(myInput);

			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ApprovalControl.PARAM_GOTO_TYPE,
					ApprovalControl.PARAM_GOTO_TYPE, "Goto type input");
			myInput.SetValue("");
			result.AddInnerHTMLElement(myInput);

			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ApprovalControl.PARAM_EDIT_ITEM,
					ApprovalControl.PARAM_EDIT_ITEM, "Edit item input");
			myInput.SetValue("");
			result.AddInnerHTMLElement(myInput);

			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ApprovalControl.PARAM_CURRENT_TYPE,
					ApprovalControl.PARAM_CURRENT_TYPE, "Current type input");
			myInput.SetValue(sTypeName);
			result.AddInnerHTMLElement(myInput);

			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ApprovalControl.PARAM_ADD_COMMENT,
					ApprovalControl.PARAM_ADD_COMMENT, "Add comment input");
			result.AddInnerHTMLElement(myInput);
		}
		catch (HTMLException he)
		{
			
				log.Error(he, "Couldn't construct form.");
			
		}



		if( ((IApprovalModelRO) m_model).DisplayItemTypeHeaderAboveActionbar() )
		{
			AddItemTypeHeader( result, type );
		}


		AddActionBar(result, type);

		AddSpacer(result, "5");

		try
		{
			myTable = new HTMLTable();
			result.AddInnerHTMLElement(myTable);

			myTable.SetCellSpacing("2");
			myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
		}
		catch (HTMLException he)
		{
		}

		if (!((IApprovalModelRO) m_model).AddColumnsHeaderForEachItem())
		{
			// if we're not adding the columns header for each item, we'll add
			//	the columns header for the entire collection of items for each
			//	type.
			AddColumnsHeader(myTable);
		}

		if (type != null)
		{	
			AddType(type, myTable);
		}

		return result;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Returns the URL Image type
	 *
	 * @return PTImageType
	 */
	protected PTImageType GetItemURLImgType()
	{
		return PTImageType.ICON_DOCUMENT_URL;
	}

	/**
	 * JHQ: This method will be called by AddActionBar() and will add action 
	 * buttons to the action bar.  This allows an editor that is extending
	 * this class to add extra functionality to the approve view.  This method
	 * should be overridden.
	 * 
	 * @param _actionBarConstruct
	 */
	public void AddActionButtons(PTConstruct _actionConstruct, PTBannerType _bannerType)
	{
		return;
	}
	
	/**
	 * Adds the action bar to the approval page
	 *
	 * @param result HTMLElement
	 */
	private void AddActionBar(HTMLElement result, ApprovalItemType type)
	{
		PTConstruct myConstruct;
		PTConstruct myButtonConstruct;
		HTMLElement eTemp;
		HTMLAnchor myAnchor;
		PTButtonType myBType;

		PTBannerType myBannerType = PTBannerType.OBJECT_ACTION_BAR;

		boolean bDisable    = ((AApprovalModel) this.m_model).GetDisableRadioSelection();
		boolean bAllowClear = ((AApprovalModel) this.m_model).GetAllowClearingOfApprovalRequests();
		boolean bDisplaySelectAll = ((AApprovalModel) this.m_model).GetDisplaySelectAllButton();
		
		try
		{
			myConstruct = UIConstructFactory.GetConstruct(myBannerType);

			if (!bDisable)
			{
				//	If approval is disabled, then we shouldn't show these
				//	buttons.


				//   APPLY button
				myBType     = PTButtonType.APPLY_OBJECT_ACTIONBAR;

				myButtonConstruct = UIConstructFactory.GetConstruct(myBType,
						true, this.m_asOwner);


				myButtonConstruct.ReplaceNestedElementsWithHTMLString(myBType.TITLE_ELEMENT_NAME,
					m_asOwner.GetString(19, ResourceConstants.INFRASTRUCTURE));

				myAnchor = (HTMLAnchor) myButtonConstruct.GetNestedElement(myBType.ANCHOR_ELEMENT_NAME);

				myAnchor.SetOnClick("postToSelf(" +
					ApprovalControl.POSTTOSELF_ACTION_APPLY + "); return false;");

				myConstruct.AddElementToNestedElement(myBannerType.EXTENDABLE_CELL_ELEMENT_NAME,
					myButtonConstruct.GetHTMLElement());

				ConstructFactory.SetAltOnButtonImages( myButtonConstruct, m_asOwner.GetString(19, ResourceConstants.INFRASTRUCTURE) );
				ConstructFactory.SetTitleOnButtonImages( myButtonConstruct, m_asOwner.GetString(19, ResourceConstants.INFRASTRUCTURE) );





				//   APPROVE_ALL button
				myBType     = PTButtonType.APPROVE_ALL_DIR_ACTIONBAR;

				myButtonConstruct = UIConstructFactory.GetConstruct(myBType,
						true, this.m_asOwner);


				myButtonConstruct.ReplaceNestedElementsWithHTMLString(myBType.TITLE_ELEMENT_NAME,
					m_asOwner.GetString(16, ResourceConstants.INFRASTRUCTURE));

				myAnchor = (HTMLAnchor) myButtonConstruct.GetNestedElement(myBType.ANCHOR_ELEMENT_NAME);

				myAnchor.SetOnClick("postToSelf(" +
					ApprovalControl.POSTTOSELF_ACTION_APPROVE_ALL + "); return false;");

				myConstruct.AddElementToNestedElement(myBannerType.EXTENDABLE_CELL_ELEMENT_NAME,
					myButtonConstruct.GetHTMLElement());

				ConstructFactory.SetAltOnButtonImages( myButtonConstruct, m_asOwner.GetString(16, ResourceConstants.INFRASTRUCTURE) );
				ConstructFactory.SetTitleOnButtonImages( myButtonConstruct, m_asOwner.GetString(16, ResourceConstants.INFRASTRUCTURE) );



				//   REJECT_ALL button
				myBType     = PTButtonType.REJECT_ALL_DIR_ACTIONBAR;

				myButtonConstruct = UIConstructFactory.GetConstruct(myBType,
						true, this.m_asOwner);


				myButtonConstruct.ReplaceNestedElementsWithHTMLString(myBType.TITLE_ELEMENT_NAME,
					m_asOwner.GetString(17, ResourceConstants.INFRASTRUCTURE));

				myAnchor = (HTMLAnchor) myButtonConstruct.GetNestedElement(myBType.ANCHOR_ELEMENT_NAME);

				myAnchor.SetOnClick("postToSelf(" +
					ApprovalControl.POSTTOSELF_ACTION_REJECT_ALL + "); return false;");

				myConstruct.AddElementToNestedElement(myBannerType.EXTENDABLE_CELL_ELEMENT_NAME,
					myButtonConstruct.GetHTMLElement());


				ConstructFactory.SetAltOnButtonImages( myButtonConstruct, m_asOwner.GetString(17, ResourceConstants.INFRASTRUCTURE) );
				ConstructFactory.SetTitleOnButtonImages( myButtonConstruct, m_asOwner.GetString(17, ResourceConstants.INFRASTRUCTURE) );



				//   HOLD_ALL button
				myBType     = PTButtonType.HOLD_ALL_DIR_ACTIONBAR;

				myButtonConstruct = UIConstructFactory.GetConstruct(myBType,
						true, this.m_asOwner);


				myButtonConstruct.ReplaceNestedElementsWithHTMLString(myBType.TITLE_ELEMENT_NAME,
					m_asOwner.GetString(18, ResourceConstants.INFRASTRUCTURE));

				myAnchor = (HTMLAnchor) myButtonConstruct.GetNestedElement(myBType.ANCHOR_ELEMENT_NAME);

				//            myAnchor.SetOnClick("javaScript:doHoldAll();");
				myAnchor.SetOnClick("postToSelf(" +
					ApprovalControl.POSTTOSELF_ACTION_HOLD_ALL + "); return false;");

				myConstruct.AddElementToNestedElement(myBannerType.EXTENDABLE_CELL_ELEMENT_NAME,
					myButtonConstruct.GetHTMLElement());


				ConstructFactory.SetAltOnButtonImages( myButtonConstruct, m_asOwner.GetString(18, ResourceConstants.INFRASTRUCTURE) );
				ConstructFactory.SetTitleOnButtonImages( myButtonConstruct, m_asOwner.GetString(18, ResourceConstants.INFRASTRUCTURE) );


				//	approval is not disabled
			}

			if (bAllowClear)
			{
				//   CLEAR button
				myBType     = PTButtonType.CLEAR_DIR_ACTIONBAR;

				myButtonConstruct = UIConstructFactory.GetConstruct(myBType,
						true, this.m_asOwner);

				myButtonConstruct.ReplaceNestedElementsWithHTMLString(myBType.TITLE_ELEMENT_NAME,
					this.m_asOwner.GetString( 102, ResourceConstants.INFRASTRUCTURE ));

				myAnchor = (HTMLAnchor) myButtonConstruct.GetNestedElement(myBType.ANCHOR_ELEMENT_NAME);

				myAnchor.SetOnClick("postToSelf(" +
					ApprovalControl.POSTTOSELF_ACTION_CLEAR + "); return false;");

				myConstruct.AddElementToNestedElement(myBannerType.EXTENDABLE_CELL_ELEMENT_NAME,
					myButtonConstruct.GetHTMLElement());


				ConstructFactory.SetAltOnButtonImages( myButtonConstruct, m_asOwner.GetString(102, ResourceConstants.INFRASTRUCTURE) );
				ConstructFactory.SetTitleOnButtonImages( myButtonConstruct, m_asOwner.GetString(102, ResourceConstants.INFRASTRUCTURE) );


			}

			if( bDisplaySelectAll )
			{
				//   Select all button
				myBType     = PTButtonType.SELECT_ALL_DIR_ACTIONBAR;

				myButtonConstruct = UIConstructFactory.GetConstruct(myBType,
						true, this.m_asOwner);

				myButtonConstruct.ReplaceNestedElementsWithHTMLString(myBType.TITLE_ELEMENT_NAME,
					this.m_asOwner.GetString( 421, ResourceConstants.PORTALINFRASTRUCTURE ));

				myAnchor = (HTMLAnchor) myButtonConstruct.GetNestedElement(myBType.ANCHOR_ELEMENT_NAME);

				myAnchor.SetOnClick("postToSelf(" +
					ApprovalControl.POSTTOSELF_ACTION_SELECT_ALL + "); return false;");

				myConstruct.AddElementToNestedElement(myBannerType.EXTENDABLE_CELL_ELEMENT_NAME,
					myButtonConstruct.GetHTMLElement());

				ConstructFactory.SetAltOnButtonImages( myButtonConstruct, m_asOwner.GetString(421, ResourceConstants.PORTALINFRASTRUCTURE) );
				ConstructFactory.SetTitleOnButtonImages( myButtonConstruct, m_asOwner.GetString(421, ResourceConstants.PORTALINFRASTRUCTURE) );
				
				// Unselect all button
				myBType     = PTButtonType.UNSELECT_ALL_DIR_ACTIONBAR;

				myButtonConstruct = UIConstructFactory.GetConstruct(myBType,
				   true, this.m_asOwner);

				myButtonConstruct.ReplaceNestedElementsWithHTMLString(myBType.TITLE_ELEMENT_NAME,
					this.m_asOwner.GetString( 144, ResourceConstants.INFRASTRUCTURE));

				myAnchor = (HTMLAnchor) myButtonConstruct.GetNestedElement(myBType.ANCHOR_ELEMENT_NAME);

				myAnchor.SetOnClick("postToSelf(" +
					ApprovalControl.POSTTOSELF_ACTION_UNSELECT_ALL + "); return false;");

				myConstruct.AddElementToNestedElement(myBannerType.EXTENDABLE_CELL_ELEMENT_NAME,
					myButtonConstruct.GetHTMLElement());

				ConstructFactory.SetAltOnButtonImages( myButtonConstruct, m_asOwner.GetString(421, ResourceConstants.PORTALINFRASTRUCTURE) );
				ConstructFactory.SetTitleOnButtonImages( myButtonConstruct, m_asOwner.GetString(421, ResourceConstants.PORTALINFRASTRUCTURE) );
			}

			/* JHQ: allow other editors to add their own buttons */
			AddActionButtons(myConstruct,myBannerType);

			/* JHQ: View Items */
			if (((AApprovalModel) this.m_model).DisplayViewItems())
			{
				AddViewItems(type, myConstruct);
			}
			
			
			eTemp = myConstruct.GetHTMLElement();

			result.AddInnerHTMLElement(eTemp);
		}
		catch (Exception ex)
		{
			
				log.Error(ex, "Couldnt make actionbar.");
			
		}
	}

	// View Items
	private void AddViewItems(ApprovalItemType _type, PTConstruct _banner)
	{
		if (((IApprovalModelRO) m_model).GetTypeCount(_type) < 1)
		{
			return;
		}
		
		int iLow  = ((IApprovalModelRO) m_model).GetLowItemIndexForCurrentPage(_type) + 1;
		int iHigh = ((IApprovalModelRO) m_model).GetHighItemIndexForCurrentPage(_type) + 1;
		
		_banner.AddHTMLStringToNestedElement(
			PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL2_ELEMENT_NAME,
			CommonHTMLStrings.SPACE);
		
		/* View Items */
		HTMLSpan mySpan = new HTMLSpan();
		_banner.AddElementToNestedElement(
			PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL2_ELEMENT_NAME,
			mySpan);

		mySpan.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);
		mySpan.AddInnerHTMLString(
			m_asOwner.GetString(464, ResourceConstants.PORTALINFRASTRUCTURE));

		_banner.AddHTMLStringToNestedElement(
			PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL2_ELEMENT_NAME,
			CommonHTMLStrings.SPACE);

		/* lower bound */
		HTMLInput myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_FROM, "");
		_banner.AddElementToNestedElement(
			PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL2_ELEMENT_NAME,
			myInput);

		myInput.SetValue(XPConvert.ToString(iLow));
		myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
		myInput.SetMaxLength(8);
		myInput.SetSize(5);

		_banner.AddHTMLStringToNestedElement(
			PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL2_ELEMENT_NAME,
			CommonHTMLStrings.SPACE);

		/* to */
		mySpan = new HTMLSpan();
		_banner.AddElementToNestedElement(
			PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL2_ELEMENT_NAME,
			mySpan);

		mySpan.SetStyleClass(PTStyleClass.ACTIONBAR_TEXT);
		mySpan.AddInnerHTMLString(
			m_asOwner.GetString(465, ResourceConstants.PORTALINFRASTRUCTURE));

		_banner.AddHTMLStringToNestedElement(
			PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL2_ELEMENT_NAME,
			CommonHTMLStrings.SPACE);

		/* upper bound */
		myInput = new HTMLInput(HTMLInputTypes.TEXT, HTMLINPUT_TO, "");
		_banner.AddElementToNestedElement(
			PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL2_ELEMENT_NAME,
			myInput);
		
		myInput.SetValue(XPConvert.ToString(iHigh));
		myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
		myInput.SetMaxLength(8);
		myInput.SetSize(5);

		_banner.AddHTMLStringToNestedElement(
			PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL2_ELEMENT_NAME,
			CommonHTMLStrings.SPACE);

		/* goto range */
		myInput = new HTMLInput(HTMLInputTypes.BUTTON, HTMLBUTTON_GOTORANGE, "");
		_banner.AddElementToNestedElement(
			PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL2_ELEMENT_NAME,
			myInput);

		myInput.SetValue(">>");
		myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
		myInput.SetOnClick(JAVASCRIPT_CARDRANGE + "(); return false;");

		_banner.AddHTMLStringToNestedElement(
			PTBannerType.OBJECT_ACTION_BAR.EXTENDABLE_CELL2_ELEMENT_NAME,
			CommonHTMLStrings.SPACE);
	}
	
	/**
	 * Adds columns to the header on the approval page
	 *
	 * @param myTable The table to add columns to
	 */
	private void AddColumnsHeader(HTMLTable myTable)
	{
		HTMLTableRow myRow   = null;
		HTMLTableCell myCell = null;

		boolean bAllowClear = ((AApprovalModel) this.m_model).GetAllowClearingOfApprovalRequests();

		boolean bDisable = ((AApprovalModel) this.m_model).GetDisableRadioSelection();

		boolean bDisplayStatusColumn = ((AApprovalModel) this.m_model).GetDisplayStatusColumn();

		boolean bDisplaySelectAll = ((AApprovalModel) this.m_model).GetDisplaySelectAllButton();

		try
		{
			myRow     = new HTMLTableRow();

			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myRow.SetStyleClass(PTStyleClass.LIST_SORT_HEADER_BG);

			if (bAllowClear)
			{
				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);

				myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
				myCell.SetAlign(CommonHTMLStrings.CENTER);
				myCell.SetStyleClass(PTStyleClass.LIST_TEXT);



				if( bDisplaySelectAll )
				{

					HTMLInput myInput = new HTMLInput( HTMLInputTypes.CHECKBOX_TYPE, "blah", "Select All" );

					myInput.SetOnClick( "checkAll(this); return true;" );
				}


			}


		}
		catch (Exception ex)
		{
		}

		if (!bDisable)
		{
			try
			{
				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);

				myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
				myCell.SetAlign(CommonHTMLStrings.CENTER);
				myCell.SetStyleClass(PTStyleClass.LIST_TEXT);


				AddImageToCell(myCell, PTImageType.ICON_APPROVE,
					m_asOwner.GetString(21, ResourceConstants.INFRASTRUCTURE));
			}
			catch (Exception ex)
			{
				
					log.Error("Couldn't get approve image.");
				
			}

			try
			{
				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);

				myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
				myCell.SetAlign(CommonHTMLStrings.CENTER);
				myCell.SetStyleClass(PTStyleClass.LIST_TEXT);


				AddImageToCell(myCell, PTImageType.ICON_REJECT,
					m_asOwner.GetString(22, ResourceConstants.INFRASTRUCTURE));
			}
			catch (Exception ex)
			{
				
					log.Error("Couldn't get reject image.");
				
			}

			try
			{
				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);

				myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
				myCell.SetAlign(CommonHTMLStrings.CENTER);
				myCell.SetStyleClass(PTStyleClass.LIST_TEXT);


				AddImageToCell(myCell, PTImageType.ICON_HOLD,
					m_asOwner.GetString(23, ResourceConstants.INFRASTRUCTURE));
			}
			catch (Exception ex)
			{
				
					log.Error("Couldn't get hold image.");
				
			}
		}
		 // radio buttons not disabled

		try
		{
			// NAME
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("60%");
			myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);


			if (((IApprovalModelRO) m_model).IsNameSortable())
			{
				HTMLAnchor myAnchor = new HTMLAnchor("#");

				myAnchor.SetOnClick("postToSelf(" +
					ApprovalControl.POSTTOSELF_ACTION_SORT_NAME + "); return false;");


				myAnchor.AddInnerHTMLString(m_asOwner.GetString(25,
						ResourceConstants.INFRASTRUCTURE));

				myCell.AddInnerHTMLElement(myAnchor);
			}
			else
			{

				myCell.AddInnerHTMLString(m_asOwner.GetString(25,
						ResourceConstants.INFRASTRUCTURE));
			}

			if (bDisplayStatusColumn)
			{
				//	STATUS
				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);

				myCell.SetWidth("20%");
				myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);

				myCell.AddInnerHTMLString(this.m_asOwner.GetString( 103, ResourceConstants.INFRASTRUCTURE ));
			}

			//	SUBMITTED BY
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("20%");
			myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);

			if (((IApprovalModelRO) m_model).IsSubmittedBySortable())
			{
				HTMLAnchor myAnchor = new HTMLAnchor("#");

				myAnchor.SetOnClick("postToSelf(" +
					ApprovalControl.POSTTOSELF_ACTION_SORT_SUBMITTEDBY +
					"); return false;");


				myAnchor.AddInnerHTMLString(m_asOwner.GetString(26,
						ResourceConstants.INFRASTRUCTURE));

				myCell.AddInnerHTMLElement(myAnchor);
			}
			else
			{

				myCell.AddInnerHTMLString(m_asOwner.GetString(26,
						ResourceConstants.INFRASTRUCTURE));
			}

			//	DATE
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("20%");
			myCell.SetStyleClass(PTStyleClass.LIST_SORT_HEADER);

			if (((IApprovalModelRO) m_model).IsDateSortable())
			{
				HTMLAnchor myAnchor = new HTMLAnchor("#");

				myAnchor.SetOnClick("postToSelf(" +
					ApprovalControl.POSTTOSELF_ACTION_SORT_DATE + "); return false;");


				myAnchor.AddInnerHTMLString(m_asOwner.GetString(27,
						ResourceConstants.INFRASTRUCTURE));

				myCell.AddInnerHTMLElement(myAnchor);
			}
			else
			{

				myCell.AddInnerHTMLString(m_asOwner.GetString(27,
						ResourceConstants.INFRASTRUCTURE));
			}
		}
		catch (HTMLException he)
		{
		}
	}

	/**
	 * Adds a description row to the approval table
	 *
	 * @param myTable The table to be added to
	 * @param rowClass The StyleClass for the row to be added
	 * @param sDescription The description to be added
	 */
	private void AddDescriptionRowToTable(HTMLTable myTable,
		PTStyleClass rowClass, boolean bComment, String sDescription,
		String sItemID, boolean bWritable)
	{
		HTMLTableCell myCell;

		HTMLTableRow myRow	    = null;
		HTMLTextArea myTextArea;

		if (sDescription != null)
		{
			try
			{
				myRow = new HTMLTableRow();
				myTable.AddInnerHTMLElement(myRow);

				myRow.SetVAlign(CommonHTMLStrings.TOP);
				myRow.SetStyleClass(rowClass);

				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);

				myCell.SetVAlign(CommonHTMLStrings.TOP);
				myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
				myCell.SetNoWrap(true);
				myCell.SetWidth("0%");

				HTMLSpan mySpan = new HTMLSpan();
				mySpan.SetStyleClass(PTStyleClass.OBJECT_SUBTITLE);

				myCell.AddInnerHTMLElement(mySpan);


				String sPrompt = m_asOwner.GetString(28, ResourceConstants.INFRASTRUCTURE);

				if (bComment)
				{

					sPrompt = m_asOwner.GetString(29, ResourceConstants.INFRASTRUCTURE);
				}

				mySpan.AddInnerHTMLString(sPrompt);

				mySpan.AddInnerHTMLString(CommonHTMLStrings.SPACE);

				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);

				myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);

				myCell.SetVAlign(CommonHTMLStrings.TOP);
				myCell.SetAlign(CommonHTMLStrings.LEFT);

				myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);

				if (bWritable)
				{
					String sPref = DESCRIPTION_PREFIX;

					if (bComment)
					{
						sPref = COMMENT_PREFIX;
					}

					myTextArea = new HTMLTextArea("80", "2");

					myTextArea.SetName(sPref + sItemID);

					myTextArea.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);

					myTextArea.AddInnerHTMLEncodedString(  sDescription );

					myCell.AddInnerHTMLElement(myTextArea);
				}
				else
				{

					String sOut = null;

					if( sDescription.length() > 60 )
					{
						sOut = sDescription.substring(0, 60) + "...";
					}
					else
					{
						sOut = sDescription;
					}


					myCell.AddInnerHTMLEncodedString(sOut);
					myCell.SetName("comment" + sItemID);

					if (bComment)
					{
						HTMLInput myInput = new HTMLInput(HTMLInputTypes.BUTTON,
								"in_bu_name", "id", "alt");

						myInput.SetValue("Add Comment");

						myInput.SetOnClick("addComment( '" +
							sItemID + "');");

						myCell.AddInnerHTMLElement(myInput);
					}
				}
			}
			catch (Exception ex)
			{
				
					log.Error(ex, "AddDescriptionRowToTable.");
				
			}
		}
	}

	/**
	 * Adds an image to a cell
	 *
	 * @param c The cell to add the images to
	 * @param i The image type to add
	 * @param sAlt the Alt for the image
	 */
	private void AddImageToCell(HTMLTableCell c, PTImageType i, String sAlt)
	{
		PTConstruct myConstruct;
		HTMLImg myImg;

		try
		{
			myConstruct     = UIConstructFactory.GetConstruct(i);

			myImg = (HTMLImg) myConstruct.GetHTMLElement();
			myImg.SetAlt(sAlt);

			c.AddInnerHTMLElement(myImg);
		}
		catch (Exception ex)
		{
			
				log.Error(ex, "couldn't add image: " + i + " to cell.");
			
		}
	}

	/**
	 * Adds a row containing an item instance to be approved
	 *
	 * @param myTable The table to be added to
	 * @param Item The instance descriptor
	 * @param instance The instance
	 * @param rowClass The row class
	 */
	private void AddInstanceItemRowToTable(HTMLTable myTable,
		AItemDescriptor item, AInstanceDescriptor instance,
		PTStyleClass rowClass) //String sItemID, String sInstanceID, PTStyleClass rowClass)
	{
		HTMLTableRow myRow   = null;
		HTMLTableCell myCell;

		String sItemID     = item.GetID();
		String sInstanceID = instance.GetID();
		HTMLElement ePath  = instance.GetPath();

		boolean bDisable = ((AApprovalModel) this.m_model).GetDisableRadioSelection();

		String sOnChange = "changeInstance( '" + sItemID + "' );";

		try
		{
			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myRow.SetStyleClass(rowClass);
		}
		catch (HTMLException he)
		{
		}

		String sOptionRadioName = INSTANCE_RADIO_PREFIX + sItemID + "X" +
			sInstanceID;

		try
		{
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
			myCell.SetAlign(CommonHTMLStrings.CENTER);
			myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

			AddRadioToCell(myCell, sOptionRadioName, VAL_APPROVE, "alt",
				sOnChange, null, instance.GetApprove(), bDisable);
		}
		catch (Exception ex)
		{
			
			
		}

		try
		{
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
			myCell.SetAlign(CommonHTMLStrings.CENTER);
			myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

			AddRadioToCell(myCell, sOptionRadioName, VAL_REJECT, "alt",
				sOnChange, null, instance.GetReject(), bDisable);
		}
		catch (Exception ex)
		{
			
			
		}

		try
		{
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
			myCell.SetAlign(CommonHTMLStrings.CENTER);
			myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

			AddRadioToCell(myCell, sOptionRadioName, VAL_HOLD, "alt",
				sOnChange, null, instance.GetHold(), bDisable);
		}
		catch (Exception ex)
		{
			
			
		}

		try
		{
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
			myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
			myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

			myCell.SetColSpan("3");

			myCell.AddInnerHTMLElement(GetFolderImg());

			myCell.AddInnerHTMLElement(ePath);
		}
		catch (Exception ex)
		{
			
			
		}
	}

	/**
	 * Adds an the header to the item type section of the approval page
	 *
	 * @param result The HTMLElement to be added to
	 * @param type The approval item type
	 * @param bCollapsed Collapsed or not
	 */
	private void AddItemTypeHeader(HTMLElement result, ApprovalItemType type)
	{
		PTConstruct myConstruct;
		HTMLElement eTemp;
		PTBannerType myBannerType = PTBannerType.OBJECT;

		int iLow  = ((IApprovalModelRO) m_model).GetLowItemIndexForCurrentPage(type);
		int iHigh = ((IApprovalModelRO) m_model).GetHighItemIndexForCurrentPage(type);
		int iMax  = ((IApprovalModelRO) m_model).GetTypeCount(type);

		String sSubTitle;

		if (iMax == 0)
		{
//			sSubTitle = ((IApprovalModelRO) m_model).GetEmptyTypeSectionSubTitle(type);

			//	no subtitle for now, it appears redundant.
			sSubTitle = "";
		}
		else
		{
			iLow++;
			iHigh++;

			sSubTitle = XPResourceManager.FormatMessage(m_asOwner.GetString(1689, ResourceConstants.PORTALADMINMSGS),
				new Integer(iLow), new Integer(iHigh), new Integer(iMax));

		}

		int iNameID = type.GetLocalizedNameIDSingular();

		if( iMax > 1 )
		{
			iNameID = type.GetLocalizedNameIDPlural();
		}

		String sDisplayName = m_asOwner.GetString(iNameID, ResourceConstants.PORTALADMINMSGS);


		try
		{
			myConstruct = UIConstructFactory.GetConstruct(myBannerType);

			myConstruct.ReplaceNestedElementsWithElement(myBannerType.TITLE_ELEMENT_NAME,
				this.GetItemImg(type, null));

			myConstruct.AddHTMLStringToNestedElement(myBannerType.TITLE_ELEMENT_NAME,
				CommonHTMLStrings.SPACE);

			myConstruct.AddHTMLStringToNestedElement(myBannerType.TITLE_ELEMENT_NAME,
				sDisplayName);


			myConstruct.ReplaceNestedElementsWithHTMLString(myBannerType.SUBTITLE_ELEMENT_NAME,
				sSubTitle);

			eTemp = myConstruct.GetHTMLElement();

			result.AddInnerHTMLElement(eTemp);

			eTemp.SetID(type.GetName());
		}
		catch (Exception ex)
		{
			log.Error(ex, "Couldnt make item type header.");
		}
	}

	/**
	 * Adds a row to a table with no items in it
	 *
	 * @param myTable The table to be added to
	 * @param rowClass The style class for the row
	 */
	private void AddNoItemsRowToTable(HTMLTable myTable, PTStyleClass rowClass)
	{
		HTMLTableCell myCell;
		HTMLTableRow myRow = null;

		try
		{
			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myRow.SetVAlign(CommonHTMLStrings.TOP);
			myRow.SetStyleClass(rowClass);

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetVAlign(CommonHTMLStrings.TOP);
			myCell.SetStyleClass(PTStyleClass.OBJECT_TEXT);
			myCell.SetAutoFit(true);

			String s = ((IApprovalModelRO) m_model).GetNoItemsDescription();

			if( s == null )
			{
				s = this.m_asOwner.GetString( 104, ResourceConstants.INFRASTRUCTURE );
			}

			myCell.AddInnerHTMLString( s );
		}
		catch (Exception ex)
		{
			
			
		}
	}

	/**
	 * Adds the previous and next page links
	 *
	 * @param myTable The table to be added to
	 * @param type The approval item type
	 */
	private void AddPaginator(HTMLTable myTable, ApprovalItemType type)
	{
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLAnchor myAnchor;

		this.GetPrevImg();
		this.GetNextImg();

		String sPrevHRef = "prev( '" + type.GetName() + "' );";
		String sNextHRef = "next( '" + type.GetName() + "' );";

		boolean bPrev = ((IApprovalModelRO) m_model).IsPreviousPageAvailable(type);
		boolean bNext = ((IApprovalModelRO) m_model).IsNextPageAvailable(type);

		try
		{
			myRow = new HTMLTableRow();
			myRow.SetAlign(CommonHTMLStrings.RIGHT);
			myRow.SetStyleClass(PTStyleClass.LIST_PAGINATION_TEXT);

			myTable.AddInnerHTMLElement(myRow);

			myCell = new HTMLTableCell();
			myCell.SetAutoFit(true);
			myCell.SetAlign(CommonHTMLStrings.RIGHT);
			myCell.SetStyleClass(PTStyleClass.LIST_PAGINATION_TEXT);
			myRow.AddInnerHTMLElement(myCell);

			if (bPrev)
			{
				myAnchor = new HTMLAnchor("#");
				myAnchor.SetOnClick(sPrevHRef);

				myAnchor.AddInnerHTMLElement(imgPrev);


				myAnchor.AddInnerHTMLString(m_asOwner.GetString(30,
						ResourceConstants.INFRASTRUCTURE));

				myCell.AddInnerHTMLElement(myAnchor);

				myCell.AddInnerHTMLString("|");
			}

			if (bPrev || bNext)
			{
				int iNumPages		  = ((IApprovalModelRO) m_model).GetNumPagesAvailable(type);
				int iCurrentPageIndex = ((IApprovalModelRO) m_model).GetCurrentPageIndex(type);
				int iLow;
				int iHigh;
				String sPageIndex;

				boolean bFirstPage = true;

				if (iNumPages > iMaxNumPageSelectorsDisplayed)
				{
					iNumPages = iMaxNumPageSelectorsDisplayed;
				}

				for (int x = 1; x <= iNumPages; x++)
				{
					iLow	  = ((IApprovalModelRO) m_model).GetLowItemIndexForGivenPage(x -
							1, type);
					iHigh     = ((IApprovalModelRO) m_model).GetHighItemIndexForGivenPage(x -
							1, type);

					++iLow;
					++iHigh;

					String s;

					if (iLow == iHigh)
					{
						s = "" + iLow;
					}
					else
					{
						s     = "" + iLow;
						s     = s + "-" + iHigh;
					}

					sPageIndex = XPConvert.ToString(x);

					if (!bFirstPage)
					{
						myCell.AddInnerHTMLString("|");
					}

					myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
					myAnchor.SetOnClick("gotoApprovalPage( '" +
							sPageIndex + "', '" + type.GetName() + "' );");

					if ((x - 1) == iCurrentPageIndex)
					{
						myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
						myCell.AddInnerHTMLString(s);
						myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
					}
					else
					{
						myAnchor.AddInnerHTMLString(s);

						myCell.AddInnerHTMLString(" ");
						myCell.AddInnerHTMLElement(myAnchor);
						myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
					}

					bFirstPage = false;
				}
			}
			 //	bPrev || bNext

			if (bNext)
			{
				myCell.AddInnerHTMLString("|");

				myAnchor = new HTMLAnchor("#");
				myAnchor.SetOnClick(sNextHRef);


				myAnchor.AddInnerHTMLString(m_asOwner.GetString(31,
						ResourceConstants.INFRASTRUCTURE));
				myAnchor.AddInnerHTMLElement(imgNext);

				myCell.AddInnerHTMLElement(myAnchor);
			}
		}
		catch (HTMLException he)
		{
			
				log.Error("couldn't build paginator, he: " + he);
			
		}
	}

	/**
	 * Adds a row to the approval table
	 *
	 * @param myTable The table
	 * @param rowClass The class of row
	 * @param sItemName The item being approved
	 * @param sSubmittedBy Submitted by
	 * @param sItemID The item id
	 * @param sPreviewHRef The preview link
	 * @param sEditHRef The edit link
	 * @param submittedDate Date submitted
	 */
	private void AddPrimaryItemRowToTable(HTMLTable myTable,
		PTStyleClass rowClass, AItemDescriptor item) //String sItemName, String sSubmittedBy, String sItemID, String sPreviewHRef, String sEditHRef, XPDateTime submittedDate)
	{
		HTMLTableRow myRow   = null;
		HTMLTableCell myCell;
		HTMLInput myInput;
		HTMLTextArea myTextArea;
		
		//boolean bIE			 = true;

		boolean bAllowClear = ((AApprovalModel) this.m_model).GetAllowClearingOfApprovalRequests();

		boolean bDisplayStatusColumn = ((AApprovalModel) this.m_model).GetDisplayStatusColumn();

		boolean bDisable = ((AApprovalModel) this.m_model).GetDisableRadioSelection();

		String sItemName		 = item.GetName();
		String sSubmittedBy		 = item.GetSubmittedBy();
		String sDescription		 = item.GetDescription();
		String sItemID			 = item.GetID();
		String sStatus			 = item.GetStatus();
		String sPreviewHRef		 = item.GetPreviewHRef();
		Redirect editRedirect    = item.GetEditRedirect();
		XPDateTime submittedDate = item.GetDateSubmitted();

		
			log.Debug("AddPrimaryItemRowToTable()");
			log.Debug("AddPrimaryItemRowToTable() - sItemName: " + sItemName);
			log.Debug("AddPrimaryItemRowToTable() - item.GetApprove(): " +
				item.GetApprove());
			log.Debug("AddPrimaryItemRowToTable() - item.GetReject(): " +
				item.GetReject());
			log.Debug("AddPrimaryItemRowToTable() - item.GetHold(): " +
				item.GetHold());
		



		PTConstruct myConstruct;
		HTMLAnchor myAnchor;
		HTMLElement eTemp;

		String sItemRadioName = ITEM_RADIO_PREFIX + sItemID;

		String sOnChange = "changePrimary( '" + sItemID + "' );";

		try
		{
			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);

			myRow.SetStyleClass(rowClass);
		}
		catch (HTMLException he)
		{
		}

		if (bAllowClear)
		{
			try
			{
				if (((AApprovalModel) this.m_model).GetAllowClearingOfItem(item))
				{
					myCell = new HTMLTableCell();
					myRow.AddInnerHTMLElement(myCell);

					myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
					myCell.SetAlign(CommonHTMLStrings.CENTER);
					myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

					String sName = ApprovalView.ITEM_CLEAR_PREFIX + sItemID;

					myInput = new HTMLInput(HTMLInputTypes.CHECKBOX_TYPE,
							sName, "alt");

					if( item.GetClear() )
					{
						myInput.SetChecked( true );
					}



					myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);

					myCell.AddInnerHTMLElement(myInput);
				}
			}
			catch (Exception ex)
			{
			}
		}

		if (!bDisable)
		{
			try
			{
				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);

				myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
				myCell.SetAlign(CommonHTMLStrings.CENTER);
				myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

				AddRadioToCell(myCell, sItemRadioName,
					ApprovalView.VAL_APPROVE, this.m_asOwner.GetString( 107, ResourceConstants.INFRASTRUCTURE ), sOnChange, null,
					item.GetApprove(), bDisable);
			}
			catch (Exception ex)
			{
				
				
			}

			try
			{
				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);

				myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
				myCell.SetAlign(CommonHTMLStrings.CENTER);
				myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

				AddRadioToCell(myCell, sItemRadioName, ApprovalView.VAL_REJECT,
				this.m_asOwner.GetString( 108, ResourceConstants.INFRASTRUCTURE ), sOnChange, null, item.GetReject(), bDisable);
			}
			catch (Exception ex)
			{
				
				
			}

			try
			{
				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);

				myCell.SetVAlign(CommonHTMLStrings.MIDDLE);
				myCell.SetAlign(CommonHTMLStrings.CENTER);
				myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

				AddRadioToCell(myCell, sItemRadioName, ApprovalView.VAL_HOLD,
				this.m_asOwner.GetString( 109, ResourceConstants.INFRASTRUCTURE ), sOnChange, null, item.GetHold(), bDisable);
			}
			catch (Exception ex)
			{
				
				
			}
		}
		 // radio buttons not disabled.

		try
		{
			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("60%");
			myCell.SetVAlign(CommonHTMLStrings.TOP);
			myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

			/* type icon - if we have a preview href, then we will add this
			 * image an anchor to the preview... 
			 * otherwise we just use this plane image 
			 */
			HTMLImg i = this.GetItemImg(item.GetType(), item);
			i.SetBorder("0");
			
			if (sPreviewHRef != null)
			{
				/* JHQ 021004 - bug 28179: open preview in new window */  
				myAnchor = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
				
				String strJSUtilVersionPrefix = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSUTIL);
				
				myAnchor.SetOnClick(strJSUtilVersionPrefix+"PTWindowUtil.openWindow('" + 
					JSHelper.JSEncodeString(sPreviewHRef) + "', '"+ 
					sItemID + "', '600','800', 1); return false;");
 
				myAnchor.AddInnerHTMLElement(i);
				myCell.AddInnerHTMLElement(myAnchor);
			}
			else
			{
				myCell.AddInnerHTMLElement(i);
			}


			/* Title */
			if( ((IApprovalModelRO) m_model).IsTitleEditable())
			{
				String sNameName = NAME_PREFIX + item.GetID();

				myTextArea = new HTMLTextArea("60", "2");
				myCell.AddInnerHTMLElement(myTextArea);

				myTextArea.SetName(sNameName);
				myTextArea.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);
				myTextArea.AddInnerHTMLEncodedString(  sItemName );
			}
			else
			{
				myCell.AddInnerHTMLEncodedString( sItemName );
				myCell.AddInnerHTMLString( CommonHTMLStrings.SPACE );
				myCell.AddInnerHTMLString( CommonHTMLStrings.SPACE );
			}


// JHQ: this preveiw link is now part of the object type image above.
//
//  			if (sPreviewHRef != null)
//  			{
//  				try
//  				{
//  					myConstruct = UIConstructFactory.GetConstruct(PTButtonType.PREVIEW,
//  							true, this.m_asOwner);
//  
//  					myConstruct.ReplaceNestedElementsWithHTMLString(PTButtonType.PREVIEW.TITLE_ELEMENT_NAME,
//  						"");
//  
//  
//  					ConstructFactory.SetAltOnButtonImages( myConstruct, this.m_asOwner.GetString( 106, ResourceConstants.INFRASTRUCTURE ) );
//						ConstructFactory.SetTitleOnButtonImages( myConstruct, this.m_asOwner.GetString( 106, ResourceConstants.INFRASTRUCTURE ) );	
//  
//  
//  					myAnchor = (HTMLAnchor) myConstruct.GetNestedElement(PTButtonType.PREVIEW.ANCHOR_ELEMENT_NAME);
//  
//  					myAnchor.SetHRef(sPreviewHRef);
//  
//  					eTemp = myConstruct.GetHTMLElement();
//  
//  					myCell.AddInnerHTMLElement(eTemp);
//  				}
//  				catch (Exception ex)
//  				{
//  					
//  						PTDebug.Trace(OpenLogService.GetComponent("UI_Infrastructure"),
//  							TraceType.Error,
//  							"Couldn't get preview button, ex: " + ex);
//  					
//  				}
//  			}


			if (editRedirect != null)
			{
				try
				{
					myConstruct = UIConstructFactory.GetConstruct(PTButtonType.EDIT,
							true, this.m_asOwner);

					myConstruct.ReplaceNestedElementsWithHTMLString(PTButtonType.EDIT.TITLE_ELEMENT_NAME,
						"");


					ConstructFactory.SetAltOnButtonImages( myConstruct, this.m_asOwner.GetString( 105, ResourceConstants.INFRASTRUCTURE ) );
					ConstructFactory.SetTitleOnButtonImages( myConstruct, this.m_asOwner.GetString( 105, ResourceConstants.INFRASTRUCTURE ) );


					myAnchor = (HTMLAnchor) myConstruct.GetNestedElement(PTButtonType.EDIT.ANCHOR_ELEMENT_NAME);

					myAnchor.SetOnClick("editItem( '" +
						item.GetID() + "' );");

					eTemp = myConstruct.GetHTMLElement();

					myCell.AddInnerHTMLElement(eTemp);
				}
				catch (Exception ex)
				{
					log.Error(ex, "Couldn't get edit button.");
				}
			}
			
			/*TODO JHQ: This is where we should add the "add folder" icon */
			
			  

			if (((IApprovalModelRO) m_model).DisplayDescriptionImmediatelyUnderTitle())
			{
				if (sDescription != null)
				{
					myCell.AddInnerHTMLElement(new HTMLBr());


					String sPrompt = m_asOwner.GetString(28, ResourceConstants.INFRASTRUCTURE);

					HTMLB myB = new HTMLB();
					myB.AddInnerHTMLEncodedString( sPrompt );

					myCell.AddInnerHTMLElement( myB );
					myCell.AddInnerHTMLString( CommonHTMLStrings.SPACE );
					myCell.AddInnerHTMLEncodedString(sDescription);
				}
			}

			if (bDisplayStatusColumn)
			{
				myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);

				myCell.SetWidth("20%");
				myCell.SetVAlign(CommonHTMLStrings.TOP);
				myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

				if (sStatus != null)
				{
					myCell.AddInnerHTMLString(sStatus);
				}
			}

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("20%");
			myCell.SetVAlign(CommonHTMLStrings.TOP);
			myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

			if (sSubmittedBy != null)
			{
				myCell.AddInnerHTMLString(sSubmittedBy);
			}

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);
			myCell.SetVAlign(CommonHTMLStrings.TOP);

			myCell.SetWidth("20%");
			myCell.SetStyleClass(PTStyleClass.LIST_TEXT);

			if (submittedDate != null)
			{
				myCell.AddInnerHTMLString( GetDateFormat().Format( submittedDate ) );
			}

		}
		catch (HTMLException he)
		{
			
				log.Error("he: " + he);
			
		}
	}

	/**
	 * Function for adding a radio button to a cell in the approval table
	 *
	 * @param myCell The cell to be added to
	 * @param sRadioName The radio button name
	 * @param sRadioVal The value for the radio button
	 * @param sRadioAlt The Alt tag for the radio button
	 * @param sOnClick The onclick event string
	 * @param sOnChange The onchange event string
	 */
	private void AddRadioToCell(HTMLTableCell myCell, String sRadioName,
		String sRadioVal, String sRadioAlt, String sOnClick, String sOnChange,
		boolean bSelected, boolean bDisable)
	{
		HTMLInput myInput;

		try
		{
			myInput = new HTMLInput(HTMLInputTypes.RADIO, sRadioName, sRadioAlt);

			myInput.SetValue(sRadioVal);

			myInput.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);

			if (bDisable)
			{
				myInput.SetDisabled(true);
			}

			if (bSelected)
			{
				myInput.SetChecked(true);
			}

			if (sOnClick != null)
			{
				myInput.SetOnClick(sOnClick);
			}

			if (sOnChange != null)
			{
				myInput.SetOnChange(sOnChange);
			}

			myCell.AddInnerHTMLElement(myInput);
		}
		catch (Exception ex)
		{
			
				log.Error("Couldn't add radio named: " + sRadioName +
					" to cell. ex: " + ex);
			
		}
	}

	/**
	 * Adds a spacer image to a table
	 *
	 * @param result The element to add a spacer to
	 * @param sHeight The height of the spacer
	 */
	private void AddSpacer(HTMLElement result, String sHeight)
	{
		PTConstruct myConstruct;
		HTMLImg myImg;

		try
		{
			myConstruct     = UIConstructFactory.GetConstruct(PTImageType.SPACER);

			myImg = (HTMLImg) myConstruct.GetHTMLElement();

			myImg.SetHeight(sHeight);

			result.AddInnerHTMLElement(myImg);
		}
		catch (Exception ex)
		{
			
				log.Error("couldn't make spacer img: " + ex);
			
		}
	}

	/**
	 * Adds a spacer row to a table
	 *
	 * @param t The table to be added to
	 * @param sHeight The height of the spacer image
	 */
	private void AddSpacerRowToTable(HTMLTable t, String sHeight)
	{
		HTMLTableRow r  = new HTMLTableRow();
		HTMLTableCell c = new HTMLTableCell();

		try
		{
			t.AddInnerHTMLElement(r);

			r.AddInnerHTMLElement(c);

			c.SetAutoFit(true);
			c.SetHeight(sHeight);

			this.AddImageToCell(c, PTImageType.SPACER, "");
		}
		catch (Exception ex)
		{
			log.Error(ex, "Couldn't add spacer row to table.");
		}
	}

	public void AddEditItemLink(HTMLTable myTable, ApprovalItemType type, String strSelectedItems, AItemDescriptor item)
	{
		return;
	}

	/**
	 * Adds an item of the specified type to the approval type
	 *
	 * @param type The type of item being approved
	 * @param myTable The table to which the item will be added
	 */
	private void AddType(ApprovalItemType type, HTMLTable myTable)
	{
		PTStyleClass descriptionRowStyle = PTStyleClass.LIST_ITEM_ONE_BG;

		if (((IApprovalModelRO) m_model).GetDisplayTypeHeader(type))
		{
			int iNumItems = ((IApprovalModelRO) m_model).GetTypeCount(type);

			if ((iNumItems > 0) ||
					(((IApprovalModelRO) m_model).GetDisplayHeadersForItemTypesWithNoItems()))
			{
				AddItemTypeHeader(myTable, type);
			}
		}

		
			log.Debug("About to call GetItemDescriptors for type: " + type.GetName());
		

		IXPEnumerator enumItems     = ((IApprovalModelRO) m_model).GetItemDescriptors(type);
		IXPEnumerator enumInstances;

		AItemDescriptor item;
		int iItemCount			    = 0;

		AInstanceDescriptor instance;
		PTStyleClass sClass;

		if (enumItems != null)
		{
			while (enumItems.MoveNext())
			{
				if (((IApprovalModelRO) m_model).AddColumnsHeaderForEachItem())
				{
					this.AddColumnsHeader(myTable);
				}

				item = (AItemDescriptor) enumItems.GetCurrent();

				iItemCount++;


				if( ( iItemCount % 2 ) == 0 )
				{
					sClass = PTStyleClass.LIST_ITEM_ONE_BG;
					descriptionRowStyle = PTStyleClass.LIST_ITEM_ONE_BG;
				}
				else
				{
					sClass = PTStyleClass.LIST_ITEM_TWO_BG;
					descriptionRowStyle = PTStyleClass.LIST_ITEM_TWO_BG;
				}



				AddPrimaryItemRowToTable(myTable, sClass, item);

				if (((IApprovalModelRO) m_model).GetDisplayInstances(type))
				{
					enumInstances = item.GetInstances();

					if (enumInstances != null)
					{
						String strSelectedItems = "";
						while (enumInstances.MoveNext())
						{
							instance = (AInstanceDescriptor) enumInstances.GetCurrent();
							this.AddInstanceItemRowToTable(myTable, item,
								instance, sClass); //instance.GetPath(), item.GetID(), instance.GetID(), PTStyleClass.LIST_ITEM_TWO_BG);

							strSelectedItems += ",17," + XPConvert.ToString(instance.GetPTFolderID());
						}
						
						if (strSelectedItems.length() > 0)
							AddEditItemLink(myTable, type, strSelectedItems.substring(1), item);
					}
				}
				 //   display instances

				HTMLTable descriptionTable = new HTMLTable();

				try
				{
					descriptionTable.SetCellPadding("2");

					HTMLTableRow myRow = new HTMLTableRow();
					myTable.AddInnerHTMLElement(myRow);

					HTMLTableCell myCell = new HTMLTableCell();
					myRow.AddInnerHTMLElement(myCell);

					myCell.SetAutoFit(true);

					myCell.AddInnerHTMLElement(descriptionTable);

					descriptionTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
					descriptionTable.SetCellSpacing("2");
				}
				catch (Exception ex)
				{
				}

				if (!((IApprovalModelRO) m_model).DisplayDescriptionImmediatelyUnderTitle())
				{
					// only add a description row if the description is not
					//	drawn with the title.
					this.AddDescriptionRowToTable(descriptionTable,
						descriptionRowStyle, false, item.GetDescription(),
						item.GetID(),
						((IApprovalModelRO) m_model).IsDescriptionEditable());
				}

				if (((IApprovalModelRO) m_model).DisplayItemComment())
				{
					this.AddDescriptionRowToTable(descriptionTable,
						descriptionRowStyle, true, item.GetComment(), item.GetID(),
						true);
				}
				
				this.AddSpacerRowToTable(myTable, "15");
			}
			 // for each item

			AddPaginator(myTable, type);
		}
		 // enumItems != null

		if (iItemCount == 0)
		{
			AddNoItemsRowToTable(myTable, PTStyleClass.LIST_ITEM_ONE_BG);
		}
	}

	/**
	 * Retrieves the Folder image from the construct factory
	 *
	 * @return HTMLImg
	 */
	private HTMLImg GetFolderImg()
	{
		PTConstruct myConstruct;

		if (folderImg == null)
		{
			try
			{
				myConstruct     = UIConstructFactory.GetConstruct(PTImageType.ICON_FOLDER,
						true, this.m_asOwner);

				folderImg = (HTMLImg) myConstruct.GetHTMLElement();

				folderImg.SetAlt(m_asOwner.GetString(78, ResourceConstants.INFRASTRUCTURE));

				folderImg.SetAlign(CommonHTMLStrings.ABSMIDDLE);
			}
			catch (Exception ex)
			{
				log.Error(ex, "Couldn't get item img.");
			}
		}

		return folderImg;
	}

	/**
	 * Retrieves the Item image from the construct factory
	 *
	 * @param type The approval item type
	 * @return HTMLImg
	 */
	private HTMLImg GetItemImg(ApprovalItemType type, AItemDescriptor item )
	{
		PTConstruct myConstruct;


		//	Try to get it from the cache

		HTMLImg imgTemp = null;

		if( item != null )
		{
			//	try to get item-specific image

			imgTemp = (HTMLImg) xmpItemImage.GetElement( item.GetID() );

		}
		else
		{
			//	try to get type-specific image

			imgTemp = (HTMLImg) xmpItemImage.GetElement( type.GetName() );


		}





		//	if we couldn't get an image from the cache, create
		//	one, and cache it.

		if (imgTemp == null)
		{
			if(item != null )
			{
				//	we have an item, try to get an item-specific image and cache it.

				String sUUID = item.GetImgUUID();

				if( sUUID != null )
				{
					//	we were passed an item, and that item has an img uuid,
					//	we'll use this to obtain a more specific img.
					
					String sImgUrl = "portal/public/img/sml" + XPStringUtility.ToLowerCase(sUUID) + ".gif";

					int iAltIndex = type.GetLocalizedNameIDSingular();

					String sAlt = m_asOwner.GetString(iAltIndex, ResourceConstants.PORTALADMINMSGS);

					imgTemp = new HTMLImg( sImgUrl, sAlt );

					imgTemp.SetAlign(CommonHTMLStrings.ABSMIDDLE);

					xmpItemImage.PutElement(item.GetID(), imgTemp);
					
				}
				else
				{
					//	we have an item, but no uuid, just use the type image


					try
					{
						myConstruct     = UIConstructFactory.GetConstruct(type.GetItemImageType(),
								true, this.m_asOwner);

						imgTemp = (HTMLImg) myConstruct.GetHTMLElement();

						int iAltIndex = type.GetLocalizedNameIDSingular();

						imgTemp.SetAlt(m_asOwner.GetString(iAltIndex, ResourceConstants.PORTALADMINMSGS));

						imgTemp.SetAlign(CommonHTMLStrings.ABSMIDDLE);


						//	store it indexed by the item id, so we don't go through
						//	this again.

						xmpItemImage.PutElement(item.GetID(), imgTemp);

					}
					catch (Exception ex)
					{
						log.Error(ex, "Couldn't get item img.");
					}
				}


			}
			else
			{

				//	item is null, this is just for the type.


				try
				{
					myConstruct     = UIConstructFactory.GetConstruct(type.GetItemImageType(),
							true, this.m_asOwner);

					imgTemp = (HTMLImg) myConstruct.GetHTMLElement();

					int iAltIndex = type.GetLocalizedNameIDSingular();

					imgTemp.SetAlt(m_asOwner.GetString(iAltIndex, ResourceConstants.PORTALADMINMSGS));

					imgTemp.SetAlign(CommonHTMLStrings.ABSMIDDLE);


					//	store it indexed by the type

					xmpItemImage.PutElement(type.GetName(), imgTemp);

				}
				catch (Exception ex)
				{
					log.Error(ex, "Couldn't get item img.");
				}


			}	//	no item



		}	//	we couldn't get one from the cache, we had to get one fresh
			//	and cache it.

		return imgTemp;
	}

	/**
	 * Retrieves the Next image from the construct factory
	 */
	private void GetNextImg()
	{
		PTConstruct myConstruct;

		if (imgNext == null)
		{
			try
			{
				myConstruct     = UIConstructFactory.GetConstruct(PTImageType.ICON_ARROW_NEXT,
						true, this.m_asOwner);

				imgNext = (HTMLImg) myConstruct.GetHTMLElement();

				imgNext.SetAlt(this.m_asOwner.GetString( 111, ResourceConstants.INFRASTRUCTURE ));

				imgNext.SetAlign(CommonHTMLStrings.ABSMIDDLE);
			}
			catch (Exception ex)
			{
				log.Error(ex, "Couldn't get item img.");
			}
		}
	}

	/**
	 * Retrieves the Previous image from the construct factory
	 */
	private void GetPrevImg()
	{
		PTConstruct myConstruct;

		if (imgPrev == null)
		{
			try
			{
				myConstruct     = UIConstructFactory.GetConstruct(PTImageType.ICON_ARROW_PREVIOUS);

				imgPrev = (HTMLImg) myConstruct.GetHTMLElement();

				imgPrev.SetAlt(this.m_asOwner.GetString( 110, ResourceConstants.INFRASTRUCTURE ));

				imgPrev.SetAlign(CommonHTMLStrings.ABSMIDDLE);
			}
			catch (Exception ex)
			{
				log.Error(ex, "Couldn't get item img.");
			}
		}
	}
}


