package com.plumtree.uiinfrastructure.common;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.pei.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This helper class handles the processing of the DisplayJavascript PEIs
 * and returns the end result scripts.
 * 
 * @author donh
 */
public class DisplayJavascriptHelper implements IDisplayJavascriptHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.common.DisplayJavascriptHelper");

	//the singleton instance
	private static IDisplayJavascriptHelper m_HelperInstance;

	// This stores whether or not the factory is in test mode.
	private static boolean m_bIsProduction = false;

	// Private constructor to ensure controlled access to the singleton.
	private DisplayJavascriptHelper() {}

	/**
	 * @return The singleton config helper.
	 */
	public static IDisplayJavascriptHelper getInstance()
    {
		if (m_HelperInstance == null)
		{
			m_bIsProduction = true;
			m_HelperInstance = new DisplayJavascriptHelper();
		}
		return m_HelperInstance;
	}

    /**
     * Unit tests should use this method to overwrite/teardown existing helper singleton.
     * DO NOT USE for production code. 
     */
	public static void setTestInstance(IDisplayJavascriptHelper testHelper)
	{
		if (m_bIsProduction)
		{
			String strMessage = "Can't override ConfigHelper instance for testing... ConfigService has already been initialized.";
			log.Error(strMessage);
			throw new XPException(strMessage);
		}

		m_HelperInstance = testHelper;
	}

    public HTMLScriptCollection DoDisplayJavascriptPEIs(AActivitySpace _asSession)
    {
    	return DisplayJavascriptHelper.DoDisplayJavascriptPEI(_asSession);
    }

    /**
     * This helper method handles the processing of the DisplayJavascript PEIs.
     *
     * @param _asSession The current space
     *
     * @return HTMLScriptCollection the resulting javascript to be added to the pages.
     * Can be null.
     * @deprecated
     */
    public static HTMLScriptCollection DoDisplayJavascriptPEI(AActivitySpace _asSession)
    {
        Object[] oa;
        Object o;
		IDisplayJavascriptActions iDisplayJavascript;
        HTMLScript script;
        HTMLScriptCollection result = null;
        boolean bGotScript = false;

        oa = (Object[]) _asSession.GetApplication().GetCachingManager().GetEntry("DisplayJavascriptActions");

        // DO DISPLAY JAVASCRIPT
        if (oa != null)
        {
        	result = new HTMLScriptCollection();

            for (int x = 0; x < oa.length; x++)
            {
                o = oa[x];

                log.Debug("DoDisplayJavascriptPEI() - About to call DisplayJavascript on an: " + o);

                if (o != null)
                {
                    iDisplayJavascript = (IDisplayJavascriptActions) o;
                    script             = iDisplayJavascript.DisplayJavascript(_asSession);

                    if (script != null)
                    {
                        result.AddInnerHTMLElement(script);
                        bGotScript = true;
                    }
                }
            }
        }

		if (bGotScript)
		{
	        return result;
		} else
		{
			return null;
		}
    }
}