package com.plumtree.uiinfrastructure.common;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class implements a frame set page.
 *
 * @author robertz
 */
public abstract class FrameSetDP implements IDisplayPage
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.common.FrameSetDP");

	/** Frame names for JS manipulation */
	public static final String TOP_FRAME    = "TopFrame";

	public static final String BODY_FRAME   = "BodyFrame";

	public static final String BOTTOM_FRAME = "BottomFrame";

	/** Activity Space */
	protected AActivitySpace m_asOwner;

	/**
	 * @see com.plumtree.xpshared.activityspace.IDisplayPage#display()
	 */
	public void Display(IWebData webData)
	{
		// Create an HTMLPage
		HTMLPage myPage = null;
		myPage = new HTMLPage();

		HTMLHead myHead = new HTMLHead();
		myPage.SetHead(myHead);

		HTMLBody myBody = new HTMLBody();

		myBody.SetMarginWidth(CommonHTMLStrings.ZERO);
		myBody.SetMarginHeight(CommonHTMLStrings.ZERO);
		myBody.SetTopMargin(CommonHTMLStrings.ZERO);
		myBody.SetLeftMargin(CommonHTMLStrings.ZERO);

		try
		{
			HTMLElementCollection styleSheetLinks = m_asOwner.GetStyleSheetLinks();
			if (null != styleSheetLinks)
			{
				myHead.AddInnerHTMLElement(styleSheetLinks);
			}

			String strFrameSetTitle = XPHttpUtility.HtmlEncode(GetFrameSetTitle());
			HTMLTitle myTitle = new HTMLTitle(strFrameSetTitle);
			myHead.AddInnerHTMLElement(myTitle);

			HTMLFrameSet frameset = new HTMLFrameSet();
			frameset.SetRows(GetFrameSetAttributes());
			frameset.SetTitle(strFrameSetTitle);

			HTMLFrame frameHeader = new HTMLFrame();
			frameHeader.SetFrameBorder(CommonHTMLStrings.ZERO);
			frameHeader.SetName(GetFrameNameTop());

			String strFrameTitleTop = GetFrameTitleTop();
			if (null != strFrameTitleTop)
			{
				frameHeader.SetTitle(strFrameTitleTop);
			}

			ASURL frameURLTop = GetFrameURLTop();
			String strFrameURLTop;

			if (null != frameURLTop)
			{
				strFrameURLTop = frameURLTop.GetURLAsString();
			} else
			{
				strFrameURLTop = GetFrameURLStringTop();
			}

			frameHeader.SetSrc(strFrameURLTop);

			if (!GetFrameScrollTop())
			{
				frameHeader.SetScrolling("no");
			}

			frameset.AddInnerHTMLElement(frameHeader);

			HTMLFrame frameBody = new HTMLFrame();
			frameBody.SetFrameBorder(CommonHTMLStrings.ZERO);
			frameBody.SetName(GetFrameNameBody());

			String strFrameTitleBody = GetFrameTitleBody();
			if (null != strFrameTitleBody)
			{
				frameBody.SetTitle(strFrameTitleBody);
			}

			frameBody.SetSrc(GetFrameURLBody().GetURLAsString());

			if (!GetFrameScrollBody())
			{
				frameBody.SetScrolling("no");
			}

			frameset.AddInnerHTMLElement(frameBody);

			ASURL bottomURL = GetFrameURLBottom();

			if (bottomURL != null)
			{
				HTMLFrame frameFooter = new HTMLFrame();
				frameFooter.SetFrameBorder(CommonHTMLStrings.ZERO);
				frameFooter.SetName(GetFrameNameBottom());

				String strFrameTitleBottom = GetFrameTitleBottom();
				if (null != strFrameTitleBottom)
				{
					frameFooter.SetTitle(strFrameTitleBottom);
				}

				frameFooter.SetSrc(bottomURL.GetURLAsString());

				if (!GetFrameScrollBottom())
				{
					frameFooter.SetScrolling("no");
				}

				frameset.AddInnerHTMLElement(frameFooter);
			}

/*
			HTMLElementCollection noFramesBody = GetNoFrames();
			if (null != noFramesBody)
			{
				HTMLElement noFrame = new HTMLElementCollection();
				noFrame.AddInnerHTMLString("<NOFRAMES>");
				noFrame.AddInnerHTMLElement(noFramesBody);
				noFrame.AddInnerHTMLString("</NOFRAMES>");

				// This is not currently supported.
				myPage.SetNoFrames(noFrame);
			}
*/

			myPage.SetFrameSet(frameset);
		}
		catch (Exception e)
		{
			
				log.Error(e, "Frameset unable to set frames.");
			

			// Perhaps this should just raise an error that goes to the error page?

			myPage     = new HTMLPage();
			myBody     = new HTMLBody();

			try
			{
				myPage.SetBody(myBody);

				HTMLTable myTable = new HTMLTable();
				myBody.AddInnerHTMLElement(myTable);

				HTMLTableRow myRow = new HTMLTableRow();
				myTable.AddInnerHTMLElement(myRow);

				HTMLTableCell myCell = new HTMLTableCell();
				myRow.AddInnerHTMLElement(myCell);


				myCell.AddInnerHTMLString(m_asOwner.GetString(36,
						ResourceConstants.INFRASTRUCTURE));
			}
			catch (Exception ee)
			{
				
					log.Error(ee, "Frameset unable to display error frame.");
				
			}
		}

		webData.SetData(myPage);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	abstract public String GetName();

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IDisplayPage#GetParent()
	 */
	public AActivitySpace GetOwner()
	{
		return m_asOwner;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner = parent;
	}

	/////////////////////////////////
	// Methods to optionally override
	/////////////////////////////////

	/**
	 * Set Frame Scroll Body
	 *
	 * @return boolean true implies scrolling
	 */
	public boolean GetFrameScrollBody()
	{
		return false;
	}

	/**
	 * Set Frame Scroll Bottom
	 *
	 * @return boolean true implies scrolling
	 */
	public boolean GetFrameScrollBottom()
	{
		return false;
	}

	/**
	 * Set Frame Scroll Top
	 *
	 * @return boolean true implies scrolling
	 */
	public boolean GetFrameScrollTop()
	{
		return false;
	}

	/**
	 * Set FrameSet Attributes
	 *
	 * @return String
	 */
	public String GetFrameSetAttributes()
	{
		return "50px,50%";
	}

	/**
	 * Set Frame URL Bottom
	 *
	 * @return ASURL, null implies no bottom frame
	 */
	public ASURL GetFrameURLBottom()
	{
		return null;
	}

	/**
	 * Set Frame URL Top
	 *
	 * This method is only called if the GetFrameURLTop
	 * method returns null; 
	 * 
	 * @return String
	 */
	public String GetFrameURLStringTop()
	{
		return null;
	}

	/**
	 * Set Frame Name Top
	 *
	 * The frame name and title are used by 508 users.
	 * 
	 * @return String
	 */
	public String GetFrameNameTop()
	{
		return TOP_FRAME;
	}

	/**
	 * Set Frame Name Body
	 *
	 * The frame name and title are used by 508 users.
	 * 
	 * @return String
	 */
	public String GetFrameNameBody()
	{
		return BODY_FRAME;
	}

	/**
	 * Set Frame Name Bottom
	 *
	 * The frame name and title are used by 508 users.
	 * 
	 * @return String
	 */
	public String GetFrameNameBottom()
	{
		return BOTTOM_FRAME;
	}

	/**
	 * Set Frame Title Top
	 *
	 * The frame name and title are used by 508 users.
	 * 
	 * @return String
	 */
	public String GetFrameTitleTop()
	{
		return null;
	}

	/**
	 * Set Frame Title Body
	 *
	 * The frame name and title are used by 508 users.
	 * 
	 * @return String
	 */
	public String GetFrameTitleBody()
	{
		return null;
	}

	/**
	 * Set Frame Title Bottom
	 *
	 * The frame name and title are used by 508 users.
	 * 
	 * @return String
	 */
	public String GetFrameTitleBottom()
	{
		return null;
	}

	/**
	 * Set NOFRAMES Body Element
	 *
	 * The NOFRAMES element is useful for 508 users.
	 * 
	 * @return HTMLElementCollection
	 */
	public HTMLElementCollection GetNoFrames()
	{
		return null;
	}

	///////////////////
	// Abstract methods
	///////////////////

	/**
	 * Set FrameSet Title
	 *
	 * This string will be HTML Encoded for
	 * safe display by this class.
	 *
	 * @return String
	 */
	abstract public String GetFrameSetTitle();

	/**
	 * Set Frame URL Body
	 *
	 * @return ASURL
	 */
	abstract public ASURL GetFrameURLBody();

	/**
	 * Set Frame URL Top
	 *
	 * @return ASURL
	 */
	abstract public ASURL GetFrameURLTop();
}

