package com.plumtree.uiinfrastructure.common;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.errordisplay.*;

// Framework Imports
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 * @author NaveenP
 */
public class PassthruControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.common.PassthruControl");

	/** Object Name for AS */
	public static final String STR_MVC_CLASS_NAME = "PassthruControl";

	public static final String QS_PASSTHRU_CONTROL = "passthruControl";

	public static final String QS_MAIN_CONTROL = "mainControl";

	protected IModel m_model;
	
	protected AActivitySpace m_asOwner;

	/**
	 *
	 * @return Redirect
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		String strPassthruName, strMainName;
		
		// Find pagination ID
		String[] sValues = (String[]) arguments.GetElement(QS_PASSTHRU_CONTROL);

		if (sValues != null)
		{
			strPassthruName = sValues[0];
		}
		else
		{
			
				log.Warn("Unable to find passthru control.");
			

			strPassthruName = "";
		}
		
		sValues = (String[]) arguments.GetElement(QS_MAIN_CONTROL);

		if (sValues != null)
		{
			strMainName = sValues[0];
		}
		else
		{
			
				log.Warn("Unable to find main control.");
			

			strMainName = "";
		}
		
		IControl passthruControl = m_asOwner.GetControl(strPassthruName);
		IControl mainControl = m_asOwner.GetControl(strMainName); 
		
		if(passthruControl != null)
		{
			passthruControl.CheckActionSecurityAndExecute(arguments);
		}
		
		if(mainControl != null)
		{
			return mainControl.CheckActionSecurityAndExecute(arguments);
		} else {
			UIError myError = new UIError(UIErrorLevel.FRIENDLY,
														UIErrorScope.CURRENT_USER,
										m_asOwner.GetString(2423, ResourceConstants.PORTALADMINMSGS),
										m_asOwner.GetString(2424, ResourceConstants.PORTALADMINMSGS));
	
			UIErrorHandler.RaiseError(myError,
				m_asOwner.GetSubSession(), m_asOwner.GetApplication());
				
			return null;
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new PassthruControl();
	}

	/**
	 * Return the class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Inits the class with the associated model and activity space.
	 *
	 * If this method is overridden in a sub-class, this version of
	 * Init must be called as super.Init();
	 *
	 * @param model
	 * @param space
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		m_model 	  = model;
		m_asOwner     = space;
	}
}

