/*
 * Purpose:	
 * Author: 	D. Phipps Feb 24, 2005
 */
package com.plumtree.uiinfrastructure.diagnostic;

import com.plumtree.uiinfrastructure.diagnostic.reporter.*;
import com.plumtree.uiinfrastructure.diagnostic.testinterface.*;
import com.plumtree.openfoundation.util.*;

import com.plumtree.uiinfrastructure.application.notification.*;

/**
 * Manages the test results for a single task.  Used to build up a 
 * report array that can be used to record results.
 * 
 * Has hooks back to progress manager class.
 * 
 * @author DavidP
 */
public class IndividualTestProgressTracker implements IProgressTracker
{
    private int m_min = 0;
    private int m_max = 1;
    private int m_current = 0;
    private int m_currentTask = 0;
    private int m_totalTasks = 0;
    private int m_completedSteps = 0;
    private boolean m_bCompleted = false;
    private String m_name = "";
    private XPStringBuilder m_log = new XPStringBuilder(); 
    private ProgressManager m_mgr = null;
    private boolean m_bVerbose = false;
    
    // Array of test objects and their state
    private ReturnValue[] m_testResults = null;

    
    /**
     * Sets the number of tasks to be run
     */
    public void init(int iTask, int iNumTasks, boolean verbose )
    {
    	m_currentTask = iTask;
    	m_totalTasks = iNumTasks;
    	m_bCompleted = false;
    	m_bVerbose = verbose;
    }
    
    public void setName( String name )
    {
    	m_name = name;
    }

    /**
     * Returns number of tasks to be run
     * @return
     */
    public int getNumTasks()
    {
    	return m_totalTasks;
    }

    /**
     * Returns number of tasks to be run
     * @return
     */
    public int getCurTask()
    {
    	return m_currentTask;
    }
    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#getMax()
     */
    public int getNumSteps()
    {
        return m_max;
    }

    
    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#setNumSteps(int)
     */
    public void setNumSteps(int max)
    {
        m_max = max;
        m_testResults = new ReturnValue[max+2];
    }
    
    public void setRetval( int step, ReturnValue retval )
    {
    	if (step < m_max+1)
    	{
    		m_testResults[step] = retval;
    	}
    }
    
    public ReturnValue getRetval( int step )
    {
    	if (step<m_max+1)
    	{
    		return m_testResults[step];
    	}
    	return null;
    }

    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#increment()
     */
    public void increment()
    {
        m_current++;
        if (m_current > m_max+1)
        {
            m_current = m_max+1;
            complete();
        }
    }

    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#decrement()
     */
    public void decrement()
    {
        m_current--;
        if (m_current < m_min)
        {
            m_current = m_min;
        }
    }

    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#complete()
     */
    public void complete()
    {
        // TODO: resource string?
        m_bCompleted = true;
    }

    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#notify(java.lang.String)
     */
    public void notifyevent(String message, boolean verbose)
    {
        m_log.Append(message);
        
        // spy logging, if valid
        if (verbose && m_bVerbose)
            Reporter.LogAndWrite(message);
        else if (!verbose)
            Reporter.LogAndWrite(message);
    }


    /* (non-Javadoc)
     * @see com.plumtree.diagtool.testinterface.IProgressTracker#notify(java.lang.String)
     */
    public void notifyevent(int iStep, ReturnValue retval)
    {
        m_log.Append(retval.getMsg());
        m_log.Append("\n");
        
        // spy logging, if valid
        Reporter.LogAndWrite(retval.getMsg());
    }    
    /**
     * Distinct hook back to manager for notify markers
     * @param mgr
     */
    public void setManager( ProgressManager mgr )
    {
        m_mgr = mgr;
    }
    
    /**
     * Returns true if the overall task completed, no matter what the status
     * @author DavidP
     */
    public boolean getCompleted()
    {
        return m_bCompleted;
    }

	/**
	 * sets the completed boolean
	 */
	public void setCompleted(boolean bCompleted)
	{
	    m_bCompleted = bCompleted;
	}
	
	public int getCompletedSteps()
	{
	    return m_current;
	}
}