/*
 * Author: 	D. Phipps Feb 24, 2005
 */
package com.plumtree.uiinfrastructure.diagnostic.testinterface;

import com.plumtree.uiinfrastructure.application.notification.*;

/**
 * Used to track the progress of a single task.  An array
 * of these objects can form reports of total startup.
 * 			
 * @author DavidP
 */
public interface IProgressTracker
{
    /**
     * Sets the number of tasks to be run
     */
    public void init(int iTask, int iNumTasks, boolean verbose );
    
    /**
     * Tracks th ename of this task.
     * @param name
     */
    public void setName( String name );

    /**
     * Returns number of tasks to be run
     * @return
     */
    public int getNumTasks();

    /**
     * Returns number of tasks to be run
     * @return
     */
    public int getCurTask();

    /**
     * Returns maximum value of progress indicator for this test
     * @return
     */
    public int getNumSteps();
    
    /**
     * Gets a specific return value for a step.  Step 0 is init.
     * @param min
     */
    public ReturnValue getRetval( int step );
    
    /**
     * Sets a specific return value for a step.  Step 0 is init.
     * @param min
     */
    public void setRetval( int step, ReturnValue retval );    
    /**
     * Sets the maximum for progress tracking
     * @param min
     */
    public void setNumSteps( int max );
    
    /**
     * Called by testrunner for each test step
     */
    public void increment();

    /**
     * Called by testrunner when a test step fails 
     * and has to be backtracked
     */
    public void decrement();

    /**
     * Called by testrunner when test completes
     */
    public void complete();

    /** 
     * Called by testrunner to add a message to the message log
     * @param message
     */
    public void notifyevent( String message, boolean verbose );

    /** 
     * Called by testrunner to add a message to the message log
     * @param message
     */
    public void notifyevent( int iStep, ReturnValue retval );

    /**
     * Returns true if the overall task completed, no matter what the status
     * @author DavidP
     */
    public boolean getCompleted();
    
    /**
     * sets the completed boolean
     */
    public void setCompleted(boolean bCompleted);
    
    /**
     * Gets the number of steps completed
     */
    public int getCompletedSteps();
}
