package com.plumtree.uiinfrastructure.editor;

import com.plumtree.uiinfrastructure.activityspace.*;

// Framework Imports
import com.plumtree.openfoundation.util.*;

/**
 * This class is a helper class for the EditorModel.  This is NOT a real
 * EditorModel or RepostModel.  The saveSection method will have to be called
 * by  the main EditorModel class of the Editor.  This class can be used to
 * implement common or plug-in sections  and can also be used as a secondary
 * Model class if the main  one is getting too big and you want to split the
 * code into  multiple classes.
 *
 * @author Julie Fournier
 */
public abstract class EditorHelperModel implements IModel
{
	/** Associated Activity Space. */
	public AActivitySpace m_asOwner;

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	abstract public Object Create();

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public abstract String GetName();

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
	}

	/**
	 * Inits the Model with the Activity Space.
	 *
	 * @param parent
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner = parent;
	}

	/**
	 * Code to save the section represented by this Model.
	 *
	 * @param _htFormData Form posted data.
	 *
	 * @return int Status for server side validation
	 */
	public abstract int SaveSection(XPHashtable _htFormData);
}

