package com.plumtree.uiinfrastructure.editor;

import com.plumtree.uiinfrastructure.activityspace.*;

// Framework Imports
import com.plumtree.openfoundation.util.*;

/**
 * This class is a helper class for the EditorRepostControl.  This is NOT a
 * real control. The PerformRepostAction method  will have to be called by the
 * main RepostControl class of the Editor.  This class can be used to
 * implement common or plug-in sections  and can also be used as a secondary
 * RepostControl class if the main  one is getting too big and you want to
 * split the code into  multiple classes.
 *
 * @author Julie Fournier
 */
public abstract class EditorHelperRepostControl implements IControl
{
	/** Associated Activity Space. */
	protected AActivitySpace m_asOwner;

	/** Associated Model. */
	protected IModel m_model;

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	abstract public Object Create();

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public abstract String GetName();

	/**
	 * This method is not used here.  This calss is not a real control.  It's
	 * just a helper for the Repost Control.
	 *
	 * @param arguments
	 *
	 * @return Redirect
	 *
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		throw new XPException(
			"ERROR in EditorHelperRepostControl::Execute, this function should NOT be called. " +
			"This class is not a real Control this is only a Helper Repost Control. It can only be used with a real " +
			"RepostControl and the PerformRepostAction method should be called from that RepostControl.");
	}

	/**
	 * Inits the class with the associated model and activity space.
	 *
	 * @param model
	 * @param space
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		m_model		  = model;
		m_asOwner     = space;
	}

	/**
	 * This method is a dispatcher. For each repost action  it should perform
	 * the right action.
	 *
	 * @param _nAction Action mode.
	 * @param _htFormData Form posted data.
	 */
	public abstract void PerformRepostAction(int _nAction,
		XPHashtable _htFormData);
}

