package com.plumtree.uiinfrastructure.errordisplay;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author Ben Geislinger
 *
 * This abstract view provides common functionality for concrete
 * view implementaions for displaying error messages.
 */
public abstract class AErrorView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.errordisplay.AErrorView");

	protected AActivitySpace m_asOwner; // Associated Activity Space
	protected IErrorModelRO m_model; // Associated read-only model

	/**
	 * This method returns the HTMLElement that encompasses the output of the View.
	 */
	public HTMLElement Display()
	{
		boolean bDisplaySeparatePageErrors = this.m_model.GetDisplaySeparatePageErrors();

		IXPEnumerator eErrors = m_model.GetErrors();

		ErrorDisplayHelper helper = new ErrorDisplayHelper(m_asOwner, eErrors);

		helper.SetDisplaySeparatePageErrors(bDisplaySeparatePageErrors);

		return helper.DisplayErrors();
	}

	/**
	 * This method returns the HTMLScript that encompasses the Javascript on the page.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 *
	 * @return HTMLScript
	 */
	public HTMLScript DisplayJavascript()
	{
		return null;
	}

	/**
	 *
	 * @param model
	 * @param parent
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		if (model == null)
		{
			log.Error("Initialized with null model.");
		}

		if (parent == null)
		{
			log.Error("Initialized with null model.");
		}

		m_model		  = (AErrorModel) model;
		m_asOwner     = parent;
	}
}