package com.plumtree.uiinfrastructure.errordisplay;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 * @author Ben Geislinger
 *
 * This concrete model extends AErrorModel, and provides basic functionality
 * for displaying error messages.
 *
 * <br><br>
 *
 * This model makes use of the base UIErrorHandler class.  This class can not make
 * use of any plumtree-specific functionality, and for this reason, this
 * error model class is somewhat limited.  Portal display pages should use
 * instead the PTErrorModel class found in the portalinfrastructure package.
 */
public class ErrorModel extends AErrorModel
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.errordisplay.ErrorModel");

	public static final String STR_MVC_CLASS_NAME = "ErrorModel";

	/**
	 * @see com.plumtree.uiinfrastructure.errordisplay.AErrorModel#ClearError()
	 */
	public void ClearError(String sKey, ISessionManager mySessionManager,
		IApplication myApplication)
	{
		UIErrorHandler.ClearError(sKey, mySessionManager, myApplication);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.errordisplay.AErrorModel#ClearErrors()
	 */
	public void ClearErrors(ISessionManager mySessionManager,
		IApplication myApplication)
	{
		UIErrorHandler.ClearErrors(mySessionManager, myApplication);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.errordisplay.AErrorModel#ClearErrorsForASChange()
	 */
	public void ClearErrorsForASChange(String sASName,
		ISessionManager mySessionManager, IApplication myApplication)
	{
		UIErrorHandler.ClearErrorsForASChange(sASName, mySessionManager,
			myApplication);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ErrorModel();
	}

	/**
	 * Returns false.  Only errors not requiring a spearate page
	 *     will be displayed.  This fact makes this model suitable for
	 *     MVCs embedded within other display pages.
	 *
	 * @return boolean false
	 */
	public boolean GetDisplaySeparatePageErrors()
	{
		return false;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.errordisplay.AErrorModel#GetErrors()
	 */
	public IXPEnumerator GetErrors(ISessionManager session,
		IApplication myApplication, AActivitySpace activitySpace,
		boolean bGetErrorsAssociatedWithAnAS)
	{
		
			log.Debug("GetErrors()");
		

		return UIErrorHandler.GetErrors(session, myApplication, activitySpace,
			bGetErrorsAssociatedWithAnAS,
			UIErrorHandler.GetDisplayAdminErrors(activitySpace));
	}

	/**
	 * Returns true.  Only errors associated with the current activity space
	 *     will be returned by the GetErrors methods.
	 *
	 * @return boolean true
	 */
	public boolean GetErrorsAssociatedWithAnAS()
	{
		return true;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IModel#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Inits the Model with the Activity Space.
	 *
	 * @param parent
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner = parent;
	}
}
