package com.plumtree.uiinfrastructure.errordisplay;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;

/**
 * @author BenG
 *
 */
public class SinglePageErrorControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.errordisplay.SinglePageErrorControl");


	public static final String STR_MVC_CLASS_NAME = "SinglePageErrorControl";

	public static final String PARAM_CLEAR_AND_RETURN = "in_hi_clearandreturn";

	private AActivitySpace m_asOwner;

	private SinglePageErrorModel m_asModel;

	/**
	 * @see com.plumtree.xpshared.activityspace.IControl#execute()
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		
			log.Debug("Entering Execute()");
		

		Redirect myRedirect = null;
		boolean bo;

		String[] sValues = (String[]) arguments.GetElement(SinglePageErrorControl.PARAM_CLEAR_AND_RETURN);

		if (sValues != null)
		{
			try
			{
				bo = XPConvert.ToBoolean(sValues[0]);
			}
			catch (Exception ex)
			{
				
					log.Error(ex, "Unable to convert PARAM_CLEAR_AND_RETURN to boolean: " + sValues[0]);
				

				bo = false;
			}

			if (bo)
			{
				
					log.Debug("About to clear and return.");
				

				UIErrorHandler.ClearErrorsRequiringASeparateDisplayPage(this.m_asOwner.GetSubSession(),
					this.m_asOwner.GetApplication(), this.m_asOwner, false,
					UIErrorHandler.GetDisplayAdminErrors(this.m_asOwner));

				AActivitySpace asParent = this.m_asOwner.GetParentSpace();

				if (asParent != null)
				{
					
						log.Debug("Got a parent, going home.");
					

					myRedirect = new Redirect();
					myRedirect.SetLinkGetCachedSpace(asParent);
				}
				else
				{
					
						log.Error("Couldn't get parent to return to..");
					
				}
			}
		}

		return myRedirect;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new SinglePageErrorControl();
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IControl#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IControl#Init(IModelRW)
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		m_asModel     = (SinglePageErrorModel) model;
		m_asOwner     = space;
	}
}

