package com.plumtree.uiinfrastructure.errordisplay;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author BenG
 *
 * This view is responsible for displaying those errors requiring their
 * own display page.  It re-uses the error display from the parent class
 * and adds a 'clear errors and return' link.
 */
public class SinglePageErrorView extends AErrorView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.errordisplay.SinglePageErrorView");

	/** MVC object name */
	public static final String STR_MVC_CLASS_NAME = "SinglePageErrorView";

	private static final String sFormName		= "singlePageErrorForm";

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new SinglePageErrorView();
	}

	/**
	 * This method returns the HTMLElement that encompasses the output of the Section.
	 */
	public HTMLElement Display()
	{
		// Get the errors displayed by the parent.
		HTMLElement eParentErrors = super.Display();

		HTMLForm myForm     = null;
		HTMLInput myInput   = null;

		try
		{
			myForm = new HTMLForm(sFormName);

			myForm.SetAction(ConfigHelper.GetApplicationBaseURLToAS(m_asOwner.GetCurrentNormalizedBaseURL(),
				m_asOwner.GetName(), m_asOwner.IsSubSpace()));
			myForm.SetMethod("post");
			myForm.SetName(sFormName);

			// Sets the space name for the event to be passed to the server
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ASConstants.SPACENAME_INPUT, ASConstants.SPACENAME_INPUT,
					"This is an input");
			myInput.SetValue(m_asOwner.GetName());
			myForm.AddInnerHTMLElement(myInput);

			// Sets the space ID for the event to be passed to the server
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ASConstants.SPACEID_INPUT, ASConstants.SPACEID_INPUT,
					"This is an input");
			myInput.SetValue(m_asOwner.GetSpaceID());
			myForm.AddInnerHTMLElement(myInput);

			// Sets the control name for the event to be passed to the server
			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					ASConstants.CONTROL_INPUT, ASConstants.CONTROL_INPUT,
					"This is an input");
			myInput.SetValue(SinglePageErrorControl.STR_MVC_CLASS_NAME);
			myForm.AddInnerHTMLElement(myInput);

			myInput = new HTMLInput(HTMLInputTypes.HIDDEN,
					SinglePageErrorControl.PARAM_CLEAR_AND_RETURN,
					SinglePageErrorControl.PARAM_CLEAR_AND_RETURN,
					"This is an input");
			myInput.SetValue(XPConvert.ToString(false));
			myForm.AddInnerHTMLElement(myInput);

			// TODOCR JF 12/10/02 - Use DisplayJavaScript for JavaScripts. This way they will be printed out in the head.
			HTMLScript myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
			XPStringBuilder sb  = new XPStringBuilder();

			sb.Append("function clearAndReturn()\n");
			sb.Append("{\n");
			sb.Append("var myForm = document.");
			sb.Append(sFormName);
			sb.Append(";\n");
			sb.Append("var car = myForm.");
			sb.Append(SinglePageErrorControl.PARAM_CLEAR_AND_RETURN);
			sb.Append(";\n");
			sb.Append("car.value = \"true\";\n");
			sb.Append("myForm.submit();\n");
			sb.Append("}\n");

			myScript.AddInnerHTMLString(sb.ToString());

			myForm.AddInnerHTMLElement(myScript);

			myForm.AddInnerHTMLElement(eParentErrors);

			HTMLAnchor aReturn = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL); 
			aReturn.SetOnClick("clearAndReturn(); return false;"); 

			aReturn.AddInnerHTMLString(m_asOwner.GetString(52,
					ResourceConstants.PORTALCOMMONMSGS));

//			The return anchor was removed because, more often than not, the space
//			that raised the error is in a bad state, and returning will just raise the
//			error again.
//			myForm.AddInnerHTMLElement(aReturn);
		}
		catch (Exception ex)
		{
			
				log.Error(ex, "Unable to display single page error view.");
			
		}

		return myForm;
	}

	/**
	 * @see com.plumtree.xpshared.activityspace.IView#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
}

