package com.plumtree.uiinfrastructure.errordisplay;

/**
 * @author Ben Geislinger
 *
 * This class enumerates the various levels of error available.  Error levels
 * might affect the display of the message, or the logging of error information.
 */
public class UIErrorLevel
{
	/** WARNING errors should be used to inform the user of something that is
	 *     possibly relevant, but requires no action on the user's part. For example,
	 *     if a user has read-only access to an editor, a warning may be displayed
	 *     indicating this fact.
	 *
	 */
	public static final UIErrorLevel WARNING = new UIErrorLevel("warning");

	/** FRIENDLY errors should be used to inform the user that an input needs
	 *     to be modified or an action needs to be taken.
	 *
	 */
	public static final UIErrorLevel FRIENDLY = new UIErrorLevel("friendly");

	/**
	 * @deprecated I don't know what this is for.  Someone wanted it, but
	 *     I think that, if anything, developer should be a scope of error rather
	 *     than a level of error. -bg
	 */
	public static final UIErrorLevel DEVELOPER = new UIErrorLevel("developer");

	/** SERIOUS errors should be used to inform the user that something is wrong
	 *     and a specific feature cannot be used.  Possibly something needs to be
	 *     configured correctly, or an administrator needs to do something.
	 *
	 */
	public static final UIErrorLevel SERIOUS = new UIErrorLevel("serious");

	/** FATAL errors should be used to inform the user that the portal is not
	 *     working.  The database is down, or corrupt.  Immediate administrative
	 *     action must be taken.
	 *
	 */
	public static final UIErrorLevel FATAL = new UIErrorLevel("fatal");
	private String sName;

	/**
	 * Creates a new UIErrorLevel object.
	 *
	 * @param s String name
	 */
	private UIErrorLevel(String s)
	{
		sName = s;
	}
}
