package com.plumtree.uiinfrastructure.errordisplay;

/**
 * @author Ben Geislinger
 *
 * This class provides an enumeration of the various levels of error scope.  The
 * scope of an error determines the audience for which the error message is
 * intended.
 */
public class UIErrorScope
{
	/**
	 * Use the CURRENT_USER scope to create an error that will be displayed
	 * only to the current user.  Such errors are stored on the current users
	 * app-server session.
	 */
	public static final UIErrorScope CURRENT_USER = new UIErrorScope(
			"current_user");

	/**
	 * Use the ADMIN_USERS scope to create an error that will be displayed
	 * only to administrative users.
	 */
	public static final UIErrorScope ADMIN_USERS = new UIErrorScope(
			"admin_users");

	/**
	 * Use the ALL_USERS scope to create an error that will be displayed
	 * to all portal users.
	 */
	public static final UIErrorScope ALL_USERS = new UIErrorScope("all_users");

	/** the error scope string */
	private String m_strErrorScope;

	/**
	 * Creates a new UIErrorScope object.
	 *
	 * @param s
	 */
	private UIErrorScope(String strErrorScope)
	{
		m_strErrorScope = strErrorScope;
	}

	/**
	 * Call this method to obtain the String representation of this scope.
	 *
	 * @return String
	 */
	public String toString()
	{
		return m_strErrorScope;
	}
}

