package com.plumtree.uiinfrastructure.expandablelist;

import com.plumtree.uiinfrastructure.editor.*;

// Framework Imports
import com.plumtree.openfoundation.util.*;

/**
 * This class handles the repost actions for the  Expandable List framework.
 *
 * @author Julie Fournier
 */  
public class ExpListControl extends EditorHelperRepostControl
{
	/** Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "ExpList";

	/** Will add one or more items to the list. */
	public static final int POSTTOSELF_ACTION_ADD = 30;

	/** Will delete one or more items from the list. */
	public static final int POSTTOSELF_ACTION_DELETE = 31;

	/** Will open the tree to select some items. */
	public static final int POSTTOSELF_ACTION_OPENTREE = 32;

	/** Will move the corresponding row. */
	public static final int POSTTOSELF_ACTION_MOVEROW = 33;

	/** Will resort by the selected column. */
	public static final int POSTTOSELF_ACTION_SORTBYCOLUMN = 34;

	/** Will reverse the sort of the current sorted column. */
	public static final int POSTTOSELF_ACTION_REVERSESORT = 35;

	/** Will modify the corresponding row. */
	public static final int POSTTOSELF_ACTION_MODIFYROW = 36;
	
   	/** Will add one or more items to the list using IPR. */
   	public static final int POSTTOSELF_ACTION_ADD_IPR = 37;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new ExpListControl();
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

   	/**
   	 * Calls the method associated to the repost action. Editors using this
   	 * framework need to call that method from  the PerformRepostAction method
   	 * of the main RepostControl.
   	 *
   	 * @param _nAction
   	 * @param _htFormData
   	 */
   	public void PerformRepostAction(int _nAction, XPHashtable _htFormData)
   	{
   		ExpListModel myExpListModel = (ExpListModel) m_model;
   
   		if (_nAction == myExpListModel.GetUniqueInt(POSTTOSELF_ACTION_ADD))
   		{
   			myExpListModel.AddRowsOnRepost(_htFormData);
   		}
   		else if (_nAction == myExpListModel.GetUniqueInt(POSTTOSELF_ACTION_ADD_IPR))
   		{
   			myExpListModel.AddRowsOnRepost(_htFormData);

   			if (m_asOwner.GetIsInPlaceRefreshEnabled())
 			{
 				// set a flag so that display pages will show in place 
 				// content instead of normal content on the next pass.
 				m_asOwner.SetIsInPlaceRefreshDisplayOn(true);
 			}
   		}
   		else if (_nAction == myExpListModel.GetUniqueInt(
   					POSTTOSELF_ACTION_DELETE))
   		{
   			myExpListModel.DeleteRowsOnRepost(_htFormData);
 			
 			if (m_asOwner.GetIsInPlaceRefreshEnabled())
 			{
 				// set a flag so that display pages will show in place 
 				// content instead of normal content on the next pass.
 				m_asOwner.SetIsInPlaceRefreshDisplayOn(true);
 			}
   		}
   		else if (_nAction == myExpListModel.GetUniqueInt(
   					POSTTOSELF_ACTION_REVERSESORT))
   		{
   			myExpListModel.ReverseCurrentSort();
 
 			if (m_asOwner.GetIsInPlaceRefreshEnabled())
 			{
 				// set a flag so that display pages will show in place 
 				// content instead of normal content on the next pass.
 				m_asOwner.SetIsInPlaceRefreshDisplayOn(true);
 			}
   		}
   		else if (_nAction == myExpListModel.GetUniqueInt(
   					POSTTOSELF_ACTION_SORTBYCOLUMN))
   		{
   			myExpListModel.SortByColumn(_htFormData);

   			if (m_asOwner.GetIsInPlaceRefreshEnabled())
 			{
 				// set a flag so that display pages will show in place 
 				// content instead of normal content on the next pass.
 				m_asOwner.SetIsInPlaceRefreshDisplayOn(true);
 			}
   		}
   		else if (_nAction == myExpListModel.GetUniqueInt(
   					POSTTOSELF_ACTION_MOVEROW))
   		{
   			myExpListModel.MoveRow(_htFormData);
   		}
   		else if (_nAction == myExpListModel.GetUniqueInt(
   					POSTTOSELF_ACTION_MODIFYROW))
   		{
   			myExpListModel.ModifyRow(_htFormData);
   		}
   	}

}

