package com.plumtree.uiinfrastructure.expandablelist.multicolumnlist.sorted;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;
import com.plumtree.uiinfrastructure.expandablelist.multicolumnlist.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 * This model allows you to use a sorted multi column list.
 * 
 * @author DonH
 */
public class SortedMultiColumnListModel extends MultiColumnListModel
{
	/** Class name for the activity space */
	public static final String STR_MVC_CLASS_NAME = "SortedMultiColumnList";

	/**
	 * Zero based column id for the default column to sort by.
	 */
	protected int m_nDefaultSortColumn;

	/**
	 * Initializes the expandable list.
	 *
	 * @param _sSectionTitle Main title.
	 * @param _sSectionSubTitle Sub title
	 * @param _bEnableSort for each column
	 * @param _nDefaultSortColumn the zero based column id for the default column to sort by.
	 * @param _imgAddIcon Icon for the add action.
	 * @param _sAddLabel Text to display next to the add icon.
	 * @param _nListKey In case you have multiple Expandable Lists on the same
	 *        DP, they should have a different key (otherwise, just set it to
	 *        1).
	 */
	public void InitializeMultiColumnList(String _sSectionTitle,
		String[] _sSectionSubTitle, Boolean[] _bEnableSort, int _nDefaultSortColumn,
		PTImageType _imgAddIcon, String _sAddLabel, int _nListKey)
	{
		//Set the width based on the number of columns
		int intColWidth    = 100 / m_intNumColumns;
		XPStringBuilder sb = new XPStringBuilder(5);
		sb.Append(intColWidth).Append("%");

		m_nDefaultSortColumn = _nDefaultSortColumn;

		Object[][] arrExpListColInfo = new Object[m_intNumColumns][5];

		for (int i = 0; i < m_intNumColumns; i++)
		{
			arrExpListColInfo[i][EXPLIST_COLUMN_ID]			    = new Integer(i +
					1);
			arrExpListColInfo[i][EXPLIST_COLUMN_TITLE]		    = _sSectionSubTitle[i];
			arrExpListColInfo[i][EXPLIST_COLUMN_WIDTH]		    = sb.ToString();
			arrExpListColInfo[i][EXPLIST_COLUMN_COLSPAN]	    = "1";
			arrExpListColInfo[i][EXPLIST_COLUMN_ENABLESORT]     = _bEnableSort[i];
		}

		super.InitializeExpandableList(_sSectionTitle, arrExpListColInfo,
			_imgAddIcon, _sAddLabel, _nListKey);
	}

	/**
	 * Initalizes the list (PTGrowableSortedArrayWrapper) with a column mapping
	 * and a sort array. The growable array will always be sorted by the
	 * 1st column ascending.
	 *
	 * @param _saData
	 */
	public void InitializeSortedArray(PTGrowableSortedArrayWrapper _saData)
	{
		// JF- Column mapping
		int[] arrColumns = new int[m_intNumColumns + 1];
		arrColumns[0] = EXPLIST_SORTEDARRAY_PROPID_INDEX;

		for (int i = 1; i <= m_intNumColumns; i++)
		{
			arrColumns[i] = i;
		}

		// JF- Sort array
		// The growable array will always be sorted by the 1st column.
		int[][] arrSort = new int[1][2];
		// This is not zero based.
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_PROPID]		  = m_nDefaultSortColumn + 1;
		arrSort[0][PTSortedArrayWrapper.PTSAW_SORT_ROW_DIRECTION]     = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;

		// JF- We don't initalize the list itself (null below)
		// At first it's empty and we will be initalized if needed using the
		// AddRowsToList method.
		_saData.Initialize(arrColumns, null, arrSort, true);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new SortedMultiColumnListModel();
	}

	/**
	 * Returns the Class name.
	 *
	 * @return String
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
}
