/**
 * Created on Dec 16, 2002
 * by adolfoc
 */
package com.plumtree.uiinfrastructure.filedownload;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.openfoundation.util.*;

/**
 * @author adolfoc
 */
public class FileDownloadControl implements IControl
{
	public static final String STR_MVC_CLASS_NAME = "FileDownloadControl";

	/**
	 * NON-Localized String: This is a query string argument
	 * for the storage key for managing access to the session.
	 * The repost control looks for this query string argument, takes
	 * the string from it and looks in the user Session in the location
	 * marked by the value of _QS_FILEPATH_SESSION_KEY.
	 *
	 * Why do we do this? We cannot send the filepath in the QS because it
	 * could be spoofed by a malevolent user to download any file on the
	 * Web Server. We secure this download function by only making it possible
	 * to download filepaths placed on the session (which should not normally
	 * be directly accessible by users). If we wish to improve security later,
	 * we can apply a transformation on the filepath in the Session such that
	 * this control can only use filepaths explicitly transformed for use with
	 * this control (just in case the user finds some other place where a filepath
	 * is stored on the session).
	 */
	public static final String _QS_FD_SESSION_KEY  = "fdSession";

	public static final String _QS_FD_CONTENT_TYPE = "fdContentType";

	/**
	 * This is posted after the popup window pops - so that the parent knows to
	 * redirect.
	 */
	public static final int POSTTOSELF_ACTION_REDIRECT_AFTER_DOWNLOAD = 45;
	protected AActivitySpace m_asOwner								  = null;
	protected IModel m_model										  = null;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#Execute(XPHashtable)
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		// TODO: add code to: take in the arguments and use them to find the file path
		// to download on the session.
		// Set that file path in our model so that it can get the file as binary data and
		// pass it to DownloadBinaryDP for display.
		FileDownloadModel m_fdlModel = (FileDownloadModel) m_model;

		if (arguments.ContainsKey(_QS_FD_SESSION_KEY))
		{
			String[] data = (String[]) arguments.GetElement(_QS_FD_SESSION_KEY);

			if ((0 != data.length) && (null != data[0]))
			{
				m_fdlModel.SetSessionKeyToDownload((String) data[0]);
			}
		}

		if (arguments.ContainsKey(_QS_FD_CONTENT_TYPE))
		{
			String[] data = (String[]) arguments.GetElement(_QS_FD_CONTENT_TYPE);

			if ((0 != data.length) && (null != data[0]))
			{
				m_fdlModel.SetContentTypeToDownload((String) data[0]);
			}
		}

		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new FileDownloadControl();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#Init(IModel, AActivitySpace)
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		// Store these for use later.
		m_asOwner     = space;
		m_model		  = model;
	}
}

