package com.plumtree.uiinfrastructure.form;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This AS provides a base implementation for an HTML Form.
 * 
 * @author Don Hayler
 */
public abstract class AFormAS extends AActivitySpace implements IFormAS
{
	/** Javascript Global Variables */

	/** image server variable */
	public static final String JAVASCRIPT_GVAR_PTIMGSVRIMGSURL     = "g_PTImgSvrImgsURL";

	/** Shared image server variable */
	public static final String JAVASCRIPT_GVAR_SHAREDIMGSVRIMGSURL = "g_SharedImgSvrImgsURL";

	/** help server variable */
	public static final String JAVASCRIPT_GVAR_PTHELPSVRBASEURL = "g_HelpSvrBaseURL";


	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public abstract String GetName();

	/**
	 * This is a helper method that all classes that inherit from AFormDP can override
	 * if they like.  The method provides a hook into the head so that DP's can add javascript.
	 * Simply override and return and HTMLScript.  Also, it is best to return the results of the
	 * DisplayJavaScript functions from the appropriate views.
	 *
	 * Override this method to Display javascript from child IViews
	 *
	 * @return HTMLScriptCollection the Javascript from the Child IViews
	 */
	public HTMLScriptCollection DisplayCommonJavaScript()
	
	{
		HTMLScriptCollection scripts = new HTMLScriptCollection();

		HTMLScript myScript = new HTMLScript(HTMLScript.TYPE_JAVASCRIPT);
		scripts.AddInnerHTMLElement(myScript);
		myScript.AddInnerHTMLString("var " + JAVASCRIPT_GVAR_PTIMGSVRIMGSURL +
			" = \"" + ConfigHelper.GetPortalImageServerImagesURL(this) + "\";\n" + "var " +
			JAVASCRIPT_GVAR_SHAREDIMGSVRIMGSURL + " = \"" +
			ConfigHelper.GetSharedImageServerImagesURL(this) + "\";");
			
			
		myScript.AddInnerHTMLString("var " + JAVASCRIPT_GVAR_PTHELPSVRBASEURL +
			" = \"" + ConfigHelper.GetImageServerBaseURL(this) + "\";\n" );			
			

		// JSRegistry include of jsrobohelp component
		scripts.AddInnerHTMLElement(ConfigHelper.GetJSIncluderJSComponentInclude(ConfigHelper.JSCOMPONENT_JSROBOHELPER, this));

		/*
		* SAMPLE
		*
		* scripts.AddInnerHTMLElement(m_asOwner.GetView(ObjectView.STR_MVC_CLASS_NAME).DisplayJavascript());
		*/
		return scripts;
	}

	/**
	 * Returns the name of the repost control class.
	 * E.g.: PropRepostControl.GetName()
	 *
	 * This can also be overridden on a per DP basis.  See AFormDP#GetRepostControlName()
	 *
	 * The repost control does not necessarily need to extend RepostControl.  A form
	 * can be submitted to any type of control as long as it knows how to properly
	 * interpret the form.
	 *
	 * @return String
	 */
	public String GetRepostControlName()
	{
		// This method needs to be implemented by all sub-classes that use the repost
		// framework.
		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.AActivitySpace#Init()
	 */
	public void Init()
	{
		super.Init();

		RegisterView(FormHeaderView.STR_MVC_CLASS_NAME, null);
		RegisterView(MediumFormHeaderView.STR_MVC_CLASS_NAME, null);
		RegisterView(FormFooterView.STR_MVC_CLASS_NAME, null);
	}
}

