package com.plumtree.uiinfrastructure.form;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.htmlconstructs.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class implements the HTML to print out the Done and Cancel
 * at the top of the form. This View is printed out by AFormDP.  To
 * turn this header on ... go to AFormDP::SetShowFormHeader()
 * 
 * @author Don Hayler
 */
public abstract class AFormHeaderView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.form.AFormHeaderView");

	/** Button names */
	public static final String HTMLBUTTON_EDITOR_FINISH = "in_bu_Done";
	public static final String HTMLBUTTON_EDITOR_CANCEL = "in_bu_Cancel";

	/** Parent Activity Space */
	protected AActivitySpace m_asOwner;

	/**
	 * This method returns the text for the finish button.  It can be
	 * overridden to change the text.
	 * 
	 * @return String the text for the finish button.
	 */
	protected String GetFinishButtonText()
	{

		return m_asOwner.GetString(70, ResourceConstants.INFRASTRUCTURE);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
	 */
	public HTMLElement Display()
	{
		HTMLElementCollection myResult = new HTMLElementCollection();

		HTMLTable myTable;
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLImg myImage;
		HTMLInput myInput;
		PTConstruct myConstruct;
		HTMLH1 headerH1;
		
		try
		{
			myResult.AddInnerHTMLElement(new HTMLComment(
					"Begin: Header main table"));

			myTable = new HTMLTable();
			myResult.AddInnerHTMLElement(myTable);
			myTable.SetBorder("0");
			myTable.SetWidth("100%");
			myTable.SetCellPadding("0");
			myTable.SetCellSpacing("0");

			// 1st Row
			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);
			myRow.SetStyleClass(PTStyleClass.BAN_HEADER_BG);

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetWidth("10");
			myConstruct     = UIConstructFactory.GetConstruct(PTImageType.SPACER,
					true, this.m_asOwner);
			myImage		    = (HTMLImg) myConstruct.GetHTMLElement();
			myImage.SetAlt("");
			myImage.SetWidth("10");
			myImage.SetHeight("1");
			myImage.SetBorder("0");
			myCell.AddInnerHTMLElement(myImage);

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);
			myCell.SetWidth("100%");
			myCell.SetHeight("30");
			myCell.SetStyleClass(PTStyleClass.BAN_TEXT);
			myCell.SetNoWrap(false);
			
			headerH1 = new HTMLH1();
			myCell.AddInnerHTMLElement(headerH1);
			// This is assumed to be HTML Encoded already
			headerH1.AddInnerHTMLString(((AFormDP) (m_asOwner.GetCurrentPage())).GetFormTitle());
			headerH1.SetStyleClass(PTStyleClass.BAN_HEADER);

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);
			myCell.SetWidth("300");
			myCell.SetAlign("right");
			myCell.SetStyleClass(PTStyleClass.BAN_TEXT);
			myCell.AddInnerHTMLElement(new HTMLComment(
					"Begin: Header buttons table"));

			// Begin Header buttons table
			HTMLTable mySubTable = new HTMLTable();
			myCell.AddInnerHTMLElement(mySubTable);
			mySubTable.SetBorder("0");
			mySubTable.SetCellPadding("5");
			mySubTable.SetCellSpacing("0");

			// button table 1st Row
			HTMLTableRow mySubRow = new HTMLTableRow();
			mySubTable.AddInnerHTMLElement(mySubRow);

			HTMLTableCell mySubCell = new HTMLTableCell();
			mySubRow.AddInnerHTMLElement(mySubCell);
			mySubCell.SetHeight("20");
			mySubCell.SetAlign("right");
			mySubCell.SetVAlign("middle");
			mySubCell.SetNoWrap(true);

			// Done button
			myInput = new HTMLInput(HTMLInputTypes.BUTTON,
					HTMLBUTTON_EDITOR_FINISH, "");
			mySubCell.AddInnerHTMLElement(myInput);

			myInput.SetValue(GetFinishButtonText());
			myInput.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
			myInput.SetOnClick("postToSelf(" +
				RepostControl.POSTTOSELF_ACTION_FINISH + "); return false;");

			mySubCell.AddInnerHTMLString("&nbsp;");

			// Cancel button
			myInput = new HTMLInput(HTMLInputTypes.BUTTON,
					HTMLBUTTON_EDITOR_CANCEL, "");
			mySubCell.AddInnerHTMLElement(myInput);


			myInput.SetValue(m_asOwner.GetString(71, ResourceConstants.INFRASTRUCTURE));
			myInput.SetStyleClass(PTStyleClass.FORM_EDITOR_BTN_TEXT);
			myInput.SetOnClick("postToSelf(" +
				RepostControl.POSTTOSELF_ACTION_CANCEL + "); return false;");

			mySubCell = new HTMLTableCell();
			mySubRow.AddInnerHTMLElement(mySubCell);
			mySubCell.SetWidth("1");
			myConstruct     = UIConstructFactory.GetConstruct(PTImageType.SPACER,
					true, this.m_asOwner);
			myImage		    = (HTMLImg) myConstruct.GetHTMLElement();
			myImage.SetAlt("");
			myImage.SetWidth("1");
			myImage.SetHeight("1");
			myImage.SetBorder("0");
			mySubCell.AddInnerHTMLElement(myImage);

			// End Header buttons table
			myCell.AddInnerHTMLElement(new HTMLComment(
					"End: Header buttons table"));

			// 3rd Row
			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);
			myRow.SetStyleClass(PTStyleClass.BAN_HEADER_BG);

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);
			myCell.SetColSpan("3");

			myResult.AddInnerHTMLElement(new HTMLComment(
					"End: Header main table"));
		}
		catch (Exception e)
		{
			
				log.Error(e, "Display: The following error occured when trying to display the HTML:");
			


			throw new XPException(m_asOwner.GetString(72,
					ResourceConstants.INFRASTRUCTURE), e);
		}

		return myResult;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		return null;
	}

	// Strings to localize





	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO,
	 *      AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		m_asOwner = parent;
	}

	///////////////////
	// abstract methods
	///////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	abstract public Object Create();

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	abstract public String GetName();
}

