package com.plumtree.uiinfrastructure.form;

import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This Display Page adds client specified Javascript to the page
 * and then closes the window.  This is useful for closing pop-ups
 * after they are finished.
 * 
 * @author Don Hayler
 */
public class ClosePopUpDP implements IDisplayPage
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.form.ClosePopUpDP");

	/** Object Name */
	public static final String STR_MVC_CLASS_NAME = "ClosePopUp";

	/** static javascript to close the window */
	private static final String JAVASCRIPT_CLOSE = "window.close();";

	/** Javascript to call before closing the window */
	private HTMLScript m_Script = null;

	/** Whether or not this popup should just close */
	private boolean m_bShouldClose = true;
	
	/** The parent activity space */
	private AActivitySpace m_asOwner;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Display(IWebData)
	 */
	public void Display(IWebData webData)
	{
		HTMLPage myPage = new HTMLPage();

		// Set the HTMLPage as the data to be sent to the browser.
		webData.SetData(myPage);

		HTMLHead myHead = new HTMLHead();
		myPage.SetHead(myHead);

		HTMLBody myBody = new HTMLBody();
		myPage.SetBody(myBody);

		HTMLScript closeScript = new HTMLScript("text/javaScript");

		try
		{
		    // DCA - add JSPortlet scripts
			HTMLScriptCollection jsxmlScript = new HTMLScriptCollection();
			jsxmlScript.AddInnerHTMLElement( ConfigHelper.GetJSIncluderJSComponentInclude(ConfigHelper.JSCOMPONENT_JSPORTLET, GetOwner()) );
			myHead.AddInnerHTMLElement(jsxmlScript);
			
			// Add client defined script
			if (m_Script != null)
			{
				myHead.AddInnerHTMLElement(m_Script);
			}

			// Close the window
			if (m_bShouldClose)
			{
				closeScript.AddInnerHTMLString(JAVASCRIPT_CLOSE);
			}

			myHead.AddInnerHTMLElement(closeScript);
		}
		catch (HTMLException he)
		{
			log.Error(he, "Display: HTML Error while attempting to display the HTML.");
		}
		catch (Exception e)
		{
			log.Error(e, "Display: Error while attempting to display the HTML.");
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IDisplayPage#GetParent()
	 */
	public AActivitySpace GetOwner()
	{
		return m_asOwner;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IDisplayPage#Init(AActivitySpace)
	 */
	public void Init(AActivitySpace parent)
	{
	    m_asOwner = parent;
	}

	/**
	 * Set the script that will be added before the window is closed. This is
	 * where you should call javascript functions on the opening window if
	 * desired.
	 *
	 * @param script the script to add before closing
	 */
	public void SetScript(HTMLScript script)
	{
		m_Script = script;
	}

	/**
	 * This function sets a boolean to check whether the ClosePopUpDP should
	 * actually close.  This is useful for pages that redirect and would like
	 * to set some JavaScript but not close.  A good example of this is a page
	 * that uses frames and needs the parent.location to change - a redirect
	 * object doesn't work, and ClosePopupDP gives us the script. so we call
	 * this function to keep the window open.
	 *
	 * @param bShouldClose
	 */
	public void SetToClose(boolean bShouldClose)
	{
		m_bShouldClose = bShouldClose;
	}
}

