package com.plumtree.uiinfrastructure.form;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This class implements the HTML to print out the Done and Cancel
 * at the bottom of the form in small pop-up style.
 * This View is printed out by AFormDP.
 *
 * This view needs to be 508 compliant
 *
 * To turn this header on ... go to AFormDP::SetShowFormFooter()
 * 
 * This is the small pop-up footer.
 * 
 * @author Don Hayler
 */
public class FormFooterView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.form.FormFooterView");

	/** MVC Object Name */
	public static final String STR_MVC_CLASS_NAME = "FormFooter";

	/** HTML Input name for finish button */
	public static final String HTMLBUTTON_EDITOR_FINISH = "in_bu_Done";

	/** HTML Input name for cancel button */
	public static final String HTMLBUTTON_EDITOR_CANCEL = "in_bu_Cancel";

	/** internal MVC data */
	private AActivitySpace m_asOwner;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new FormFooterView();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
	 */
	public HTMLElement Display()
	{
		boolean bNoJavascript = (m_asOwner.GetAccessStyle() != AccessStyles.STANDARD);

		HTMLElementCollection myResult = new HTMLElementCollection();

		HTMLTable myTable;
		HTMLTableRow myRow;
		HTMLTableCell myCell;
		HTMLInput myInput;

		try
		{
			myResult.AddInnerHTMLElement(new HTMLComment(
					"Begin: Footer buttons table"));

			myTable = new HTMLTable();
			myResult.AddInnerHTMLElement(myTable);
			myTable.SetBorder("0");
			myTable.SetWidth("100%");
			myTable.SetCellPadding("0");
			myTable.SetCellSpacing("10");
			myTable.SetStyleClass(PTStyleClass.BAN_HEADER_BG);

			// 1st Row
			myRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myRow);
			myRow.SetStyleClass(PTStyleClass.BAN_HEADER_BG);

			myCell = new HTMLTableCell();
			myRow.AddInnerHTMLElement(myCell);

			myCell.SetAlign(CommonHTMLStrings.CENTER);

			if (bNoJavascript)
			{
				// 508 & low bandwidth buttons
				// If we cannot use javascript, we need special submit buttons
				// Done button
				String strOK = m_asOwner.GetString(66, ResourceConstants.INFRASTRUCTURE); // "OK"
				myInput = new HTMLInput(HTMLInputTypes.SUBMIT,
						RepostControl.HTMLSUBMIT_508_POSTTOSELF +
						RepostControl.POSTTOSELF_ACTION_FINISH, "");
				myCell.AddInnerHTMLElement(myInput);


				myInput.SetValue(strOK); // "OK"
				myInput.SetAlt(strOK); // "OK"
				myInput.SetAccessKey(m_asOwner.GetString(26, "ptmsgs_portal508quickkeymsgs"));
				myInput.SetTitle(m_asOwner.GetString(28, "ptmsgs_portal508quickkeymsgs"));
				myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);

				myCell.AddInnerHTMLString("&nbsp;");

				// Cancel button
				String strCancel = m_asOwner.GetString(67,
						ResourceConstants.INFRASTRUCTURE); // "Cancel"
				myInput = new HTMLInput(HTMLInputTypes.SUBMIT,
						RepostControl.HTMLSUBMIT_508_POSTTOSELF +
						RepostControl.POSTTOSELF_ACTION_CANCEL, "");
				myCell.AddInnerHTMLElement(myInput);


				myInput.SetValue(strCancel); // "Cancel"
				myInput.SetAlt(strCancel); // "Cancel"
				myInput.SetAccessKey(m_asOwner.GetString(27, "ptmsgs_portal508quickkeymsgs"));
				myInput.SetTitle(m_asOwner.GetString(29, "ptmsgs_portal508quickkeymsgs"));
				myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
			}
			else
			{
				// Normal buttons
				// Done button
				myInput = new HTMLInput(HTMLInputTypes.BUTTON,
						HTMLBUTTON_EDITOR_FINISH, "");
				myCell.AddInnerHTMLElement(myInput);


				myInput.SetValue(m_asOwner.GetString(66, ResourceConstants.INFRASTRUCTURE)); // "Done"
				myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
				myInput.SetOnClick("postToSelf(" +
					RepostControl.POSTTOSELF_ACTION_FINISH +
					"); return false;");

				// Cancel button
				myInput = new HTMLInput(HTMLInputTypes.BUTTON,
						HTMLBUTTON_EDITOR_CANCEL, "");
				myCell.AddInnerHTMLElement(myInput);


				myInput.SetValue(m_asOwner.GetString(67, ResourceConstants.INFRASTRUCTURE)); // "Cancel"
				myInput.SetStyleClass(PTStyleClass.FORM_BTN_TEXT);
				myInput.SetOnClick("postToSelf(" +
					RepostControl.POSTTOSELF_ACTION_CANCEL +
					"); return false;");
			}

			myCell.AddInnerHTMLElement(new HTMLComment(
					"End: Footer buttons table"));
		}
		catch (Exception e)
		{
			
				log.Error(e, "Display: The following error occured when trying to display the HTML:");
			


			throw new XPException(m_asOwner.GetString(68,
					ResourceConstants.INFRASTRUCTURE), e);
		}

		return myResult;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	// Strings to localize





	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO, AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		m_asOwner = parent;
	}
}

