package com.plumtree.uiinfrastructure.form;

import com.plumtree.uiinfrastructure.activityspace.*;

// Framework Imports
import com.plumtree.openfoundation.util.*;

/**
 * This interface is used along with RepostControl and IRepostAS
 * to partially implement the logic of the repost action.
 * Those two classes are used by the Editor Framework to help implementing
 * the repost mechanism on any Editor page.
 * They can also be used by any other Activity Space.
 *
 * See instructions in RepostControl.java on how to use that
 * logic outside the Editor Framework.
 *
 * @author Julie Fournier
 */
public interface IRepostModel extends IModel
{
	/**
	 * Implement some code to execute when the user clicks on "Finish".
	 *
	 */
	public void Finish();

	/**
	 * Returns the Redirect object
	 * (this is were we need to go at the end of the Editor).
	 *
	 * @return Redirect
	 */
	public Redirect GetRedirectAfterFinish();

	/**
	 * Calls the right save function according to the page name passed in argument.
	 *
	 * @param _sPageName
	 * @param _htFormData
	 */
	public int SavePage(String _sPageName, XPHashtable _htFormData);
}

