package com.plumtree.uiinfrastructure.form;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.xpshared.htmlconstructs.*;

/**
 *  This is a com.plumtree.uiinfrastructure.activityspace.ILink
 *  that works within an AFormAS.  When clicked, the link will repost the
 *  form, then run the specified control with the provided args.
 *  Currently just a link within a single page; does not support setting
 *  an alternate space or display page.
 *
 * @author CraigS
 */
public class PassthruURL implements ILinkElement
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.form.PassthruURL");

	private HTMLElementCollection _internals = new HTMLElementCollection();
	private String _control;
	private XPStringBuilder _args			 = new XPStringBuilder(200);

	/**
	 *
	 *
	 * @param strName
	 * @param strValue
	 */
	public void AddControlArgument(String strName, String strValue)
	{
		_args.Append(LinkConstants.AMPERSAND).Append(strName).Append(LinkConstants.EQUALS)
      .Append(XPHttpUtility.UrlEncode(strValue, LinkConstants.UTF_8_ENCODING));
	}

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddQueryStringPair(String, String[])
	 *
	 * This URL encodes the value for you
	 *
	 * @param strName
	 * @param strValues
	 */
	public void AddControlArgument(String strName, String[] strValues)
	{
		if (strValues == null)
		{
			
				log.Warn("AddControlArgument called with null String[]");
			

			return;
		}

		for (int i = 0; i < strValues.length; i++)
		{
			AddControlArgument(strName, strValues[i]);
		}
	}

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName
	 * @param nValue
	 */
	public void AddControlArgument(String strName, int nValue)
	{
		AddEncodedControlArgument(strName, XPConvert.ToString(nValue));
	}

	/**
	 * This method allows you to add a control argument to the HTML URL.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName
	 * @param booleanValue
	 */
	public void AddControlArgument(String strName, boolean bValue)
	{
		AddEncodedControlArgument(strName, XPConvert.ToString(bValue));
	}

	/**
	 * This method allows you to add a control argument to the HTML URL with no value.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddControlArgument(String, String)
	 *
	 * @param strName
	 * @param booleanValue
	 */
	public void AddControlArgument(String strName)
	{
		_args.Append(LinkConstants.AMPERSAND).Append(strName).Append(LinkConstants.EQUALS);
	}

	/**
	 *
	 *
	 * @param element
	 *
	 */
	public void AddInnerHTMLElement(HTMLElement element)
	{
		_internals.AddInnerHTMLElement(element);
	}

	/**
	 *
	 *
	 * @param string
	 *
	 * @throws HTMLException
	 */
	public void AddInnerHTMLString(String strHTML)
	{
		this.AddInnerHTMLElement(new HTMLGenericElement(strHTML));
	}

	/**
	 *
	 *
	 * @return
	 */
	public HTMLElement GetURLAsHTMLElement()
	{
		XPStringBuilder sb = new XPStringBuilder(200);

  		// TODO: fix this.  This whole class should just be a static method that returns
  		// an ASURL or something.  This is totally the wrong way to do this.

		sb.Append(AFormDP.JAVASCRIPT_SUBMIT_TO_CONTROL).Append("('")
		.Append(_control).Append("','").Append(_args.ToString())
		.Append("'); return false;");

		HTMLAnchor result = new HTMLAnchor(ConstructStaticHelpers.BLANK_URL);
		result.SetOnClick(sb.ToString());

		try
		{
			result.AddInnerHTMLElement(_internals);
		}
		catch (HTMLException e)
		{
			
				log.Error(e, "Unable to add elements to pass thru url as HTML element.");
			

			throw XPException.GetInstance(e);
		}

		return result;
	}

	/**
	 *
	 *
	 * @param strControl
	 */
	public void SetControl(String strControl)
	{
		_control = strControl;
	}

	/**
	 *
	 *
	 * @param strPage
	 */
	public void SetDisplayPage(String strPage)
	{
		// not implemented
	}

	/**
	 *
	 *
	 * @param string
	 *
	 */
	public void SetLabelString(String strLabel)
	{
		AddInnerHTMLString(strLabel);
	}

	/**
	 *
	 *
	 * @param strSpace
	 * @param parent
	 *
	 * @return
	 */
	public boolean SetLinkCreateNewSpace(String strSpace, AActivitySpace parent)
	{
		// not implemented
		return true;
	}

	/**
	 *
	 *
	 * @param space
	 *
	 * @return
	 */
	public boolean SetLinkGetCachedSpace(AActivitySpace space)
	{
		// not implemented
		return true;
	}

	/**
	 *
	 *
	 * @param strSpace
	 * @param strSpaceID
	 *
	 * @return
	 */
	public boolean SetSpaceGetCached(String strSpace, String strSpaceID)
	{
		// not implemented
		return true;
	}

	/**
	 *
	 *
	 * @param strSpace
	 * @param parent
	 *
	 * @return
	 */
	public boolean SetLinkGetSpaceIfCached(String strSpace, AActivitySpace parent)
	{
		// not implemented
		return true;
	}

	/**
	 *
	 *
	 * @param sSubSpaceKey
	 */
	public void SetSubSpaceKey(String sSubSpaceKey)
	{
		// not implemented
	}

	/**
	 * This is a helper method to add qs arguments.  This assumes that you
	 * have already URL encoded the values if necessary.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.ILink#AddQueryStringPair(String, String)
	 *
	 * @param strName
	 * @param strValue
	 */
	private void AddEncodedControlArgument(String strName, String strValue)
	{
		_args.Append(LinkConstants.AMPERSAND).Append(strName).Append(LinkConstants.EQUALS).Append(strValue);
	}

	public boolean SetLinkToExternalURL(String url) {
		return false;
	}
	
	/** Logging setup */
}

