package com.plumtree.uiinfrastructure.form.sort;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.arraywrapper.*;

/**
 * This control sets the sort to use.
 * 
 * @author Don Hayler
 */
public class SortControl implements IControl
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.form.sort.SortControl");

	/** Object Name for AS */
	public static final String STR_MVC_CLASS_NAME = "SortControl";

	/** Form element names */
	/** The index of the header to sort on */
	public static final String HTMLINPUT_SORT_ID = "in_hi_SortID";

	/** The index of the header to sort on */
	public static final String HTMLINPUT_SORT_HEADER = "in_hi_SortHeader";

	/** the sort direction
	 * @see com.plumtree.uiinfrastructure.arraywrapper.PTSortedArrayWrapper
	 */
	public static final String HTMLINPUT_SORT_DIRECTION = "in_hi_SortDirection";
	private ISortModel m_asModel;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#CheckActionSecurityAndExecute(XPHashtable)
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		int nSortID;
		int nSortHeader;
		int nSortDirection;

		String[] sValues = (String[]) arguments.GetElement(HTMLINPUT_SORT_ID);

		if (sValues != null)
		{
			nSortID = XPConvert.ToInteger(sValues[0]);
		}
		else
		{
			
				log.Error("Unable to find sort ID.  Using default.");
			

			nSortID = 1;
		}

		sValues = (String[]) arguments.GetElement(HTMLINPUT_SORT_HEADER);

		if (sValues != null)
		{
			nSortHeader = XPConvert.ToInteger(sValues[0]);
		}
		else
		{
			
				log.Error("Unable to find sort header index.  Using default.");
			

			nSortHeader = 1;
		}

		sValues = (String[]) arguments.GetElement(HTMLINPUT_SORT_DIRECTION);

		if (sValues != null)
		{
			nSortDirection = XPConvert.ToInteger(sValues[0]);

			if ((nSortDirection != PTSortedArrayWrapper.PTSAW_SORT_ASCENDING) &&
					(nSortDirection != PTSortedArrayWrapper.PTSAW_SORT_DESCENDING))
			{
				
					log.Error("Sort direction neither ascending nor descending, using default: " +
						nSortDirection);
				

				nSortDirection = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
			}
		}
		else
		{
			
				log.Error("Unable to find sort direction.  Using default.");
			

			nSortDirection = PTSortedArrayWrapper.PTSAW_SORT_ASCENDING;
		}

		m_asModel.SetSort(nSortID, nSortHeader, nSortDirection);

		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new SortControl();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#Init(IModel, AActivitySpace)
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		try
		{
			m_asModel = (ISortModel) model;
		}
		catch (Exception e)
		{
			
				log.Error(e, "Initialized with wrong type of model.  Expecting ISortModel.");
			
		}
	}
}

