package com.plumtree.uiinfrastructure.helpers;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * A service that provides access to the config helper object.
 */
public class ConfigService implements IConfigHelper 
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.helpers.ConfigService");

	//the singleton instance
	private static IConfigHelper m_HelperInstance;

	// This stores whether or not the factory is in test mode.
	private static boolean m_bIsProduction = false;

	// Protected constructor to ensure controlled access to the singleton.
	protected ConfigService() {}

	/**
	 * @return The singleton config helper.
	 */
	public static IConfigHelper getInstance()
    {
		if (m_HelperInstance == null)
		{
			m_bIsProduction = true;
			m_HelperInstance = new ConfigService();
		}
		return m_HelperInstance;
	}

    /**
     * Unit tests should use this method to overwrite/teardown existing helper singleton.
     * DO NOT USE for production code. 
     * 
     * NOTE: When finished with a unit test, this should be set back to null so as not
     * to interfere with other tests.
     */
	public static void setTestInstance(IConfigHelper testHelper)
	{
		if (m_bIsProduction)
		{
			String strMessage = "Can't override ConfigHelper instance for testing... ConfigService has already been initialized.";
			log.Error(strMessage);
			throw new XPException(strMessage);
		}

		m_HelperInstance = testHelper;
	}

	public String GetJSIncluderComponentPath(String strJSComponentName, String strImageServerURL)
	{
		return ConfigHelper.GetJSIncluderComponentPath(strJSComponentName, strImageServerURL);
	}

	public HTMLElementCollection GetJSIncluderJSComponentInclude(String strJSComponentName, AActivitySpace owner)
	{
		return ConfigHelper.GetJSIncluderJSComponentInclude(strJSComponentName, owner);
	}

	public HTMLElementCollection GetJSIncluderJSComponentInclude(String strJSComponentName, boolean bEnableDebug, AActivitySpace owner, XPLocale locale)
	{
		return ConfigHelper.GetJSIncluderJSComponentInclude(strJSComponentName, bEnableDebug, owner, locale);
	}

	public String GetJSIncluderVersionPrefix(String strJSComponentName)
	{
		return ConfigHelper.GetJSIncluderVersionPrefix(strJSComponentName);
	}

	public String GetApplicationBaseURL(String _sNormalizedBaseURL, boolean _bSecure) {
		return ConfigHelper.GetApplicationBaseURL(_sNormalizedBaseURL, _bSecure);
	}

	public String GetApplicationBaseURL(AActivitySpace asSpace) {
		return ConfigHelper.GetApplicationBaseURL(asSpace);
	}

	public String GetApplicationBaseURL(String _sNormalizedBaseURL) {
		return ConfigHelper.GetApplicationBaseURL(_sNormalizedBaseURL);
	}

	public String GetApplicationBaseURLToAS(String _sNormalizedBaseURL, String _sASName, boolean _bIsSubSpace) {
		return ConfigHelper.GetApplicationBaseURLToAS(_sNormalizedBaseURL, _sASName, _bIsSubSpace);
	}

	public String GetHTTPEntryPoint() {
		return ConfigHelper.GetHTTPEntryPoint();
	}

	public String GetMainServerApplicationURL(boolean _bIsSecure) {
		return ConfigHelper.GetMainServerApplicationURL(_bIsSecure);
	}
	
	public String GetImageServerBaseURL(AActivitySpace asSpace) {
		return ConfigHelper.GetImageServerBaseURL(asSpace);
	}

	public String GetNormalizedBaseURL(String _sURL) {
		return ConfigHelper.GetNormalizedBaseURL(_sURL);
	}
}