package com.plumtree.uiinfrastructure.helpers;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

public interface IConfigHelper
{
	/**
	 * JF- Returns the Application URL according to the URL Mapping and the security.
	 * Warning: depending on the security mode, the security in argument might not make sense.
	 * E.g.: in mode 0, it doesn't make sense to request the secure URL
	 * (and it might just return an empty URL)
	 *
	 * The normalized base URL is available either cached on the AActivitySpace or by calling
	 * ConfigHelper.GetNormalizedBaseURL with the current request URL.
	 * 
	 * @param _sNormalizedBaseURL the normalized base URL from the current request.
	 * @param _bSecure HTTP or HTTPS
	 *
	 * @return String
	 */
	public String GetApplicationBaseURL(String _sNormalizedBaseURL, boolean _bSecure);

	/**
	 * JF- Returns the Application URL according to the URL Mapping.
	 * Warning: In mode 1, it returns the non secure URL by default.
	 *
	 * @param AActivitySpace the current activity space.
	 *
	 * @return String
	 */
	public String GetApplicationBaseURL(AActivitySpace asSpace);

	/**
	 * JF- Returns the Application URL according to the URL Mapping.
	 * Warning: In mode 1, it returns the non secure URL by default.
	 * 
	 * GetApplicationBaseURL(AActivitySpace) should be used if possible.
	 *
	 * @param _sNormalizedBaseURL Normalized Base URL from the AActivity Space
	 * (or ConfigHelper.GetNormalizedBaseURL).
	 *
	 * @return String
	 */
	public String GetApplicationBaseURL(String _sNormalizedBaseURL);
	
	/**
	 * JF- Returns the Application URL according to the URL Mapping and
	 * determines the security according to the space name in argument.
	 *
	 * @param _sNormalizedBaseURL Normalized Base URL from the AActivity Space.
	 * (or ConfigHelper.GetNormalizedBaseURL).
	 * @param _sASName Name of the destination Activity Space
	 * @param _bIsSubSpace Is the target AS a sub space?
	 *
	 * @return String
	 */
	public String GetApplicationBaseURLToAS(String _sNormalizedBaseURL,
		String _sASName, boolean _bIsSubSpace);

	/**
	 * JF- Returns the Main server URL from x_config.xml
	 *
	 * @param _bIsSecure HTTP or HTTPS
	 *
	 * @return String
	 */
	public String GetMainServerApplicationURL(boolean _bIsSecure);
	
	/**
	 * JF- Returns the value of the HTTP Entry point from x_config.xml 
	 * E.g.: server.pt
	 * 
	 * @return String
	 */
	public String GetHTTPEntryPoint();

	/**
	 * 
	 * @param strJSComponentName JSComponent name included through JSIncluder Example: " jsxml" or "jsutil"
	 * @return
	 */
	public String GetJSIncluderVersionPrefix(String strJSComponentName);

	/**
	 * @param strJSComponentName
	 * @param strImageServerURL
	 * @return the path to a particular JS component
	 */
	public String GetJSIncluderComponentPath(String strJSComponentName, String strImageServerURL);
	
	/**
	 * HTMLElement with a script blocks including the javascript files for the given jscomponent. 
	 * 
	 * @param strJSComponentName	JSComponent name. For example "jsxml" or "jsutil", the common components have constants defined in ConfigHelper.
	 * @param owner	Current activityspace
	 * @return 
	 */
	public HTMLElementCollection GetJSIncluderJSComponentInclude(String strJSComponentName, AActivitySpace owner);
	
	/**
	 * HTMLElement with a script blocks including the javascript files for the given jscomponent. 
	 * 
	 * @param strJSComponentName	JSComponent name. For example "jsxml" or "jsutil", the common components have constants defined in ConfigHelper.
	 * @param owner	Current activityspace
	 * @return 
	 */
	public HTMLElementCollection GetJSIncluderJSComponentInclude(String strJSComponentName, boolean bEnableDebug, AActivitySpace owner,  XPLocale locale);

	/**
	 * This method used to return the Portal Image Server Base URL.
	 *
	 * @param AActivitySpace the space that this image is in.
	 * @return String the Portal image server base URL.
	 */
	public String GetImageServerBaseURL(AActivitySpace asSpace);
	
	/**
	 * DLH- Helper method to return the normalized base URL for use
	 * in determining the correct application base url.
	 *
	 * @param _sURL
	 * @return String the normalized base URL
	 */
	public String GetNormalizedBaseURL(String _sURL);
}
