package com.plumtree.uiinfrastructure.htmlconstructs;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author Ben Geislinger
 *
 *     This class implements functionality used by various classes in this
 *     package.  This functionality is not intended for public use.
 *
 */
public class UIFactoryHelper extends FactoryHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.htmlconstructs.UIFactoryHelper");

	/**
	 *
	 * @param construct
	 */
	public static void updateImgAlts(PTConstruct construct)
	{
		updateImgAlts(construct, null);
	}

	/**
	 * This method updates HTMLImg objects nested within a construct.  The alt
	 *     attribute of each updated HTMLImg object is set to the localized
	 *     String.
	 *
	 * @param t Pass a PTConstructType
	 * @param construct Pass a PTConstruct
	 */
	public static void updateImgAlts(PTConstruct construct, AActivitySpace activitySpace)
	{
		
			log.Debug("updateImgAlts()");
		

		PTConstructType t = construct.GetType();

		XPHashtable m = t.GetImgElementNameToAltIdMap();

		if (m != null)
		{
			IXPEnumerator eKeys = m.GetKeys();

			String sElementName;
			String sStringID;
			int iID;
			HTMLElement eTemp;
			HTMLImg imgTemp;
			boolean bGotIt	    = false;

			String sNewString = "";

			while (eKeys.MoveNext())
			{
				sElementName     = (String) eKeys.GetCurrent();

				sStringID     = null;
				iID		  = -1;
				bGotIt		  = false;

				try
				{
					Object o = m.GetElement(sElementName);

					if (o != null)
					{
						iID	       = XPConvert.ToInteger(o);
						bGotIt     = true;
					}
					else
					{
						bGotIt = false;
					}
				}
				catch (Exception ex)
				{
					bGotIt = false;
				}

				
					log.Debug("updateImgAlts() - sElementName: " + sElementName);
					log.Debug("updateImgAlts() - sStringID: " + sStringID);
					log.Debug("updateImgAlts() - oiID: " + iID);
				

				if (bGotIt)
				{
					eTemp = null;

					try
					{
						eTemp = construct.GetNestedElement(new ElementName(
									sElementName));
					}
					catch (Exception e)
					{
						
							log.Error("updateImgAlts() - couldn't get nested element named: " +
								sElementName);
						
					}

					if (eTemp != null)
					{
						if (eTemp instanceof HTMLImg)
						{
							try
							{
								sNewString = activitySpace.GetString(iID,
										ResourceConstants.SAMPLEMSGS);
							}
							catch (Exception ex)
							{
							}

							imgTemp = (HTMLImg) eTemp;

							imgTemp.SetAlt(sNewString);
						}
						else
						{
							
								log.Error("updateImgAlts() - This element is not an HTMLImg.");
							
						}
						 //   element is not a HTMLImg
					}
				}
				 //   string id is not DO_NOT_UPDATE
			}
			 //   for each key/val pair in the string dictionary
		}
		 //   if this type offers a string dictionary.
	}

	/**
	 *
	 * @param construct
	 */
	public static void updateStrings(PTConstruct construct)
	{
		updateStrings(construct, null);
	}

	/**    This method updates all of the appropriate strings within a construct
	 *     with localized versions of those strings.
	 *     <br><br>
	 *         Each construct offers a StrinDictionary containing
	 *         ElementName/stringid pairs.  The ElementName indicates a nested
	 *         HTMLElement object, the string id indicates a localized string.
	 *         <br><br>
	 *         This method obtains each nested element indicated by the set of
	 *         ElementName objects.  If that nested HTMLElement is an
	 *         HTMLGenericElement, then it is replaced with the corresponding localized
	 *         string.  If the nested element is any other type of HTMLElement object,
	 *         the corresponding localized string is nested within the nested element.

	 *     @param t Pass a PTConstructType.  This type will be used to obtain
	 *     a string dictionary describing which internal htmlelements need to
	 *     be updated.
	 *     <br><br>
	 *     @param construct Pass a PTConstruct.  Nested elements with names
	 *     equivalent to those specified by the string dictionary will be updated.
	 *
	 *
	 */
	public static void updateStrings(PTConstruct construct, AActivitySpace activitySpace)
	{
		PTConstructType t = construct.GetType();

		
			log.Debug("updateStrings()");
		

		if (activitySpace == null)
		{
			//	todo perhaps log an error?
			return;
		}

		XPHashtable m = t.GetStringMap();

		if (m != null)
		{
			
				log.Debug("updateStrings() - Got string map.");
			

			IXPEnumerator eKeys = m.GetKeys();

			String sElementName;
			String sStringID;

			HTMLElement eTemp;

			String sNewString;

			while (eKeys.MoveNext())
			{
				sElementName     = (String) eKeys.GetCurrent();
				sStringID = null;

				int iId = -1;

				try
				{
					Object o = m.GetElement(sElementName);

					if (o != null)
					{
						iId	   = XPConvert.ToInteger(o);
					}
				}
				catch (Exception ex)
				{
				}

				
					log.Debug("updateStrings() - sElementName: " + sElementName);
					log.Debug("updateStrings() - sStringID: " + sStringID);
				

				if (iId != -1)
				{
					try
					{
						sNewString = activitySpace.GetString(iId, ResourceConstants.SAMPLEMSGS);
					}
					catch (Exception ex)
					{
						
							log.Error(ex, "updateStrings() - Couldn't retrieve message with id: " +
								iId);
						

						sNewString = "";
					}
				}
				else
				{
					// iID = -1
					sNewString = "";
				}

				eTemp = null;

				try
				{
					eTemp = construct.GetNestedElement(new ElementName(
								sElementName));
				}
				catch (Exception e)
				{
					
						log.Error("updateStrings() - couldn't get nested element named: " +
							sElementName);
					
				}

				if (eTemp != null)
				{
					if (eTemp instanceof HTMLGenericElement)
					{
						HTMLGenericElement ge = (HTMLGenericElement) eTemp;

						ge.SetDisplayString(sNewString);
					}
					else
					{
						eTemp.ClearInnerHTMLElements();

						try
						{
							eTemp.AddInnerHTMLString(sNewString);
						}
						catch (HTMLException he)
						{
							
								log.Debug("updateStrings() - Couldn't add string.");
							
						}
					}
					 //   element is not a HTMLGenericElement
				}
			}
			 //   for each key/val pair in the string dictionary
		}
		 //   if this type offers a string dictionary.
	}
}

