/*
 * Created on May 18, 2006
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.plumtree.uiinfrastructure.httpcompression;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openkernel.config.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.application.*;

public class CompressionSettings {

    private static OpenLogger log = OpenLogService.GetLogger(
            OpenLogService.GetComponent("UI_Infrastructure"),
            "com.plumtree.uiinfrastructure.httpcompression.CompressionSettings");
    
    public static final String HTTPCOMPRESSION_CONTEXT = "httpCompression";
    
    // DCA - browser buckets for configuring compression settings
    public static final String IE5_CONTEXT = "IE5";
    public static final String IE5_MAC_CONTEXT = "IE5-Mac";
    public static final String IE6_CONTEXT = "IE6";
    public static final String GECKO_CONTEXT = "Gecko";
    public static final String SAFARI_CONTEXT = "Safari";
    public static final String OPERA5PLUS_CONTEXT = "Opera5+";
    public static final String UNKNOWN_CONTEXT = "Unknown";
    
    public static final String COMPRESSION_TYPE = "http://www.plumtree.com/config/component/types/httpcompression";
    
    // DCA - content types available for compression
    public static final String TEXT_TYPE = CompressionSettings.COMPRESSION_TYPE + "/text";
    public static final String IMAGE_TYPE = CompressionSettings.COMPRESSION_TYPE + "/image";
    public static final String APP_TYPE = CompressionSettings.COMPRESSION_TYPE + "/application";
    public static final String AUDIO_TYPE = CompressionSettings.COMPRESSION_TYPE + "/audio";
    public static final String VIDEO_TYPE = CompressionSettings.COMPRESSION_TYPE + "/video";
    public static final String MULT_TYPE = CompressionSettings.COMPRESSION_TYPE + "/multipart";
    public static final String MESS_TYPE = CompressionSettings.COMPRESSION_TYPE + "/message";
    public static final String MODEL_TYPE = CompressionSettings.COMPRESSION_TYPE + "/model";
    
    public static final String ENABLE_COMPRESSION_SETTING = "enableHttpCompression";
    public static final String MIN_COMPRESSION_BYTES_SETTING = "minCompressBytes";
    public static final String FAVOR_GZIP_SETTING = "favorGZIP";
    public static final String COMPRESSION_LEVEL_SETTING = "compressionLevel";
    
    public static final String DEFAULT_COMPRESSION_SETTING = "default";
    public static final String FORCE_COMPRESSION_SETTING = "forceCompression";
    public static final String DISABLE_COMPRESSION_SETTING = "disableCompression";

    public static final CompressionSettings INSTANCE = new CompressionSettings();
    
    private IOKComponent m_okCompressionComponent;
    // Hashtable version of the httpcompression.xml file
    private XPHashtable m_htBrowserContextsMap;
    private boolean m_bInitialized = false;
    
    private CompressionSettings() {
        m_htBrowserContextsMap = new XPHashtable();
    }
    
    public void Init() {
        if (m_bInitialized)
        {
            log.Error("CompressionSettings::Init Init attempted twice.  Ignoring second attempt.");
            return;
        }
        
        IOKContext okCompresseionContext = null;
        try {
            // DCA - get the general compression settings context
            okCompresseionContext = AppWarmUp.GetOKConfigContext().getConfig().getContext(HTTPCOMPRESSION_CONTEXT);
        } catch (Exception e) {
            // compression xml was not found
            log.Warn("CompressionSettings::Init httpcompression.xml was not found. Compression will be disabled.", e);
            return;
        }
        m_okCompressionComponent = okCompresseionContext.getComponentByType(COMPRESSION_TYPE);
        
        if (IsCompressionEnabled()) {
            // DCA - load each browser group
            LoadBrowserContext(IE5_CONTEXT);
            LoadBrowserContext(IE5_MAC_CONTEXT);
            LoadBrowserContext(IE6_CONTEXT);
            LoadBrowserContext(GECKO_CONTEXT);
            LoadBrowserContext(SAFARI_CONTEXT);
            LoadBrowserContext(OPERA5PLUS_CONTEXT);
            LoadBrowserContext(UNKNOWN_CONTEXT);
        } else {
            log.Debug("CompressionSettings::Init Global Compression has been configured to be disabled. Compression Settings will not be loaded.");
        }
        m_bInitialized = true;
    }
    
    private void LoadBrowserContext(String strBrowserContextKey) {
        log.Debug("CompressionSettings::LoadBrowserContext Loading browser context " + strBrowserContextKey);
        XPHashtable htBrowserContextMap = new XPHashtable();
        m_htBrowserContextsMap.PutElement(strBrowserContextKey, htBrowserContextMap);
        IOKContext okBrowserContext = null;
        try {
            okBrowserContext = AppWarmUp.GetOKConfigContext().getConfig().getContext(strBrowserContextKey);
        } catch (Exception e) {
            // compression xml was not found
            log.Warn("CompressionSettings::LoadBrowserContext Browser context " + strBrowserContextKey + " was not found. Compression for browsers in this context is disabled.", e);
            return;
        } 
        
        // DCA - load each content type group for the specified browser group
        LoadCompressionSettings(TEXT_TYPE, htBrowserContextMap, okBrowserContext);
        LoadCompressionSettings(IMAGE_TYPE, htBrowserContextMap, okBrowserContext);
        LoadCompressionSettings(APP_TYPE, htBrowserContextMap, okBrowserContext);
        LoadCompressionSettings(AUDIO_TYPE, htBrowserContextMap, okBrowserContext);
        LoadCompressionSettings(VIDEO_TYPE, htBrowserContextMap, okBrowserContext);
        LoadCompressionSettings(MULT_TYPE, htBrowserContextMap, okBrowserContext);
        LoadCompressionSettings(MESS_TYPE, htBrowserContextMap, okBrowserContext);
        LoadCompressionSettings(MODEL_TYPE, htBrowserContextMap, okBrowserContext);
    }

    private void LoadCompressionSettings(String strContentGroupKey, XPHashtable htBrowserContextMap, IOKContext okBrowserContext) {
        log.Debug("CompressionSettings::LoadCompressionSettings Loading compression settings " + strContentGroupKey + " for browser context " + okBrowserContext.getName());
        IOKComponent okContentTypeComponent;
        try {
            okContentTypeComponent = okBrowserContext.getComponentByType(strContentGroupKey);
        } catch (Exception e) {
            // component was not found
            log.Warn("CompressionSettings::LoadCompressionSettings Compression settings " + strContentGroupKey + " for browser context " + okBrowserContext.getName() + 
                " was not found. Compression for this group of content type for this browser group is disabled.", e);
            htBrowserContextMap.PutElement(strContentGroupKey+DEFAULT_COMPRESSION_SETTING, Boolean.FALSE);
            htBrowserContextMap.PutElement(strContentGroupKey+FORCE_COMPRESSION_SETTING, new XPHashtable());
            return;
        } 
        
        // DCA - store the default compression setting for this content type group for this browser
        boolean bDefault = okContentTypeComponent.getSettingValue(DEFAULT_COMPRESSION_SETTING, false);
        htBrowserContextMap.PutElement(strContentGroupKey+DEFAULT_COMPRESSION_SETTING, Boolean.valueOf(bDefault));
        String strExceptionSettingKey;
        // DCA - determine what type of exception is in this component
        if (bDefault) {
            strExceptionSettingKey = DISABLE_COMPRESSION_SETTING;
        } else {
            strExceptionSettingKey = FORCE_COMPRESSION_SETTING;
        }
        XPHashtable htContentTypes = new XPHashtable();
        htBrowserContextMap.PutElement(strContentGroupKey+strExceptionSettingKey, htContentTypes);
        
        // DCA - load exceptions into a hashtable for easy look up.
        String strExceptions = okContentTypeComponent.getSettingValue(strExceptionSettingKey, "");
        XPStringTokenizer stExceptionTokens = new XPStringTokenizer(strExceptions, ";", false);
        while (stExceptionTokens.HasMoreElements()) {
            String strContentType = stExceptionTokens.GetNextElement();
            htContentTypes.PutElement(strContentType, strContentType);
        }
    }

    public boolean GetDefaultCompressionSetting(String strBrowserGroupKey, String strContentGroupKey) {
        XPHashtable htBrowserContextMap = (XPHashtable) m_htBrowserContextsMap.GetElement(strBrowserGroupKey);
        if (null == htBrowserContextMap) {
            return false;
        }
        Boolean bDefault = (Boolean) htBrowserContextMap.GetElement(strContentGroupKey+DEFAULT_COMPRESSION_SETTING);
        return XPConvert.ToBoolean(bDefault);
    }
    
    public XPHashtable GetForceCompressionTypes(String strBrowserGroupKey, String strContentGroupKey) {
        XPHashtable htBrowserContextMap = (XPHashtable) m_htBrowserContextsMap.GetElement(strBrowserGroupKey);
        if (null == htBrowserContextMap) {
            return new XPHashtable();
        }
        XPHashtable htContentTypes =  (XPHashtable) htBrowserContextMap.GetElement(strContentGroupKey+FORCE_COMPRESSION_SETTING);
        return htContentTypes;
    }
    
    public XPHashtable GetDisableCompressionTypes(String strBrowserGroupKey, String strContentGroupKey) {
        XPHashtable htBrowserContextMap = (XPHashtable) m_htBrowserContextsMap.GetElement(strBrowserGroupKey);
        if (null == htBrowserContextMap) {
            return new XPHashtable();
        }
        XPHashtable htContentTypes =  (XPHashtable) htBrowserContextMap.GetElement(strContentGroupKey+DISABLE_COMPRESSION_SETTING);
        return htContentTypes;
    }
    
    public boolean IsCompressionEnabled() {
        if (null == m_okCompressionComponent) {
            return false;
        }
        boolean val = m_okCompressionComponent.getSettingValue(ENABLE_COMPRESSION_SETTING, false);
        return val;
    }
    
    public int GetMinCompressionBytes() {
        if (null == m_okCompressionComponent) {
            return 512;
        }
        int val = m_okCompressionComponent.getSettingValue(MIN_COMPRESSION_BYTES_SETTING, 512);
        return val;
    }
    
    public boolean IsGZIPFavored() {
        if (null == m_okCompressionComponent) {
            return false;
        }
        boolean val = m_okCompressionComponent.getSettingValue(FAVOR_GZIP_SETTING, false);
        return val;
    }
    
    public int GetCompressionLevel() {
        if (null == m_okCompressionComponent) {
            return 5;
        }
        int val = m_okCompressionComponent.getSettingValue(COMPRESSION_LEVEL_SETTING, 5);
        return val;
    }
}
