package com.plumtree.uiinfrastructure.interpreter;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * IMVCController defines the interface of an MVC Controller, which the 
 * Interpreter will interact to invoke the application logic. The application
 * logic is typically written based on activityspace-related objects (Control,
 * Model, View, Redirect, etc). 
 *    
 * @author WilliamA
 */
public interface IMVCController {

	/**
	 * This method invokes the OnPageStart PEI. The PEI will be invoked with the
	 * activity space in the given redirect if any. Otherwise, it will be 
	 * invoked with the activity space in the requestData.
	 * 
	 * @param redirect Redirect to be processed, if any
	 * @param requestData the request 
	 * @param hostname the hostname of current portal instance
	 */
	void DoOnPageStartActions(Redirect redirect, RequestData requestData, String hostname);
	
	/**
	 * This method performs the core control flow execution of activityspace
	 * related logics. It processes the given redirect (if any) in the context
	 * of the requestData. This method may return a final Redirect object,
	 * which typically is a Redirect containing a DisplayPage object, or
	 * a flag to indicate an HTTP redirect.
	 *   
	 * @param redirect Redirect to be processed, if any
	 * @param requestData the request
	 * @return the final Redirect, either a DisplayPage or an HTTP redirect
	 */
	Redirect HandleRedirect(Redirect redirect, RequestData requestData);
	
	/**
	 * This method invokes the OnPageFinish PEI.
	 * 
	 * @param requestData the request
	 * @param hostname the hostname of current portal instance
	 * @return any page action comments
	 */
	HTMLElement DoOnPageFinishActions(RequestData requestData, String hostname);
}
