package com.plumtree.uiinfrastructure.interpreter;

import com.plumtree.openfoundation.util.XPException;

/**
 * This class is used to convey an error or exception information back to
 * the Interpreter. Upon receiving this exception, the Interpreter will
 * display an error page containing a (localized) error message from this 
 * exception, if there is any. 
 *  
 * @author William Adjie-Winoto
 */
public class InterpreterException extends XPException {
  
    public static final int INVALID_ID = -1;
	public static final int INVALID_HTTP_STATUS_CODE = -1;

	private int m_nMessageStringID = -1;
	private String m_strResourceFileName = null;
    private String m_strNonLocalizedMessage = null;
	private int m_nHTTPStatusCode = -1;
	
	// CONSTRUCTORS
	/**
	 * Default Constructor
	 */
	public InterpreterException() {

		m_nMessageStringID = INVALID_ID;
		m_strResourceFileName = null;
		m_strNonLocalizedMessage = null;
		m_nHTTPStatusCode = INVALID_HTTP_STATUS_CODE;
	}

	/**
	 * Contructor.
	 * @param _nMessageStringID the message string ID in the resource file
	 * @param _strResourceFileName the name of the resource file
	 * @param _strNonLocalizedMessage the non-localized version of the message
	 */
	public InterpreterException(int _nMessageStringID, String _strResourceFileName,
		String _strNonLocalizedMessage) {

		m_nMessageStringID = _nMessageStringID;
		m_strResourceFileName = _strResourceFileName;
		m_strNonLocalizedMessage = _strNonLocalizedMessage;
		m_nHTTPStatusCode = INVALID_HTTP_STATUS_CODE;
	}
	
	/**
	 * Contructor.
	 * @param _nMessageStringID the message string ID in the resource file
	 * @param _strResourceFileName the name of the resource file
	 * @param _strNonLocalizedMessage the non-localized version of the message
	 * @param _nHttpStatusCode the HTTP status code for the HTTP response
	 */
	public InterpreterException(int _nMessageStringID, String _strResourceFileName,
		String _strNonLocalizedMessage, int _nHttpStatusCode) {

		m_nMessageStringID = _nMessageStringID;
		m_strResourceFileName = _strResourceFileName;
		m_strNonLocalizedMessage = _strNonLocalizedMessage;
		m_nHTTPStatusCode = _nHttpStatusCode;
	}
	
	// METHODS
    public int GetMessageStringID()
    {
    	return m_nMessageStringID;
    }

	public void SetMessageStringID(int _nId)
	{
		m_nMessageStringID = _nId;
	}
    
        
	public String GetResourceFileName()
	{
		return m_strResourceFileName;
	}

	public void GetResourceFileName(String _strFilename)
	{
		m_strResourceFileName = _strFilename;
	}
		
	public String GetNonLocalizedMessage()
	{
		return m_strNonLocalizedMessage;
	}
	
	public void SetNonLocalizedMessage(String _strNonLocalizedMessage)
	{
		m_strNonLocalizedMessage = _strNonLocalizedMessage;
	}
		
	public int GetHTTPStatusCode()
	{
		return m_nHTTPStatusCode;
	}
	
	public void SetHTTPStatusCode(int _nStatusCode)
	{
		m_nHTTPStatusCode = _nStatusCode;
	}
}
