package com.plumtree.uiinfrastructure.interpreter;

import com.plumtree.uiinfrastructure.activityspace.*; 

/**
 * This exception is typically thrown by the MVCController class. There are 3 kinds 
 * of security-related checks inside the MVCController: <br />
 * 1. Secure activity space check in relation to security mode 1 <br />
 * 2. Users basic access right check whether he/she has the privilege to 
 * access the AS <br />
 * 3. Individual controls action security check <br />
 * Hence, an UnauthorizedAccessException can be thrown due to either of the above.
 * 
 * @author William Adjie-Winoto
 */
public class UnauthorizedAccessException extends InterpreterException {

	private Redirect m_Redirect = null;
	
	public UnauthorizedAccessException() 
	{
		m_Redirect = null;
	}
	
	public UnauthorizedAccessException(Redirect r) 
	{
		m_Redirect = r;
	}
	
	/**
	 * This method returns the Redirect object, if any, where the Interpreter should 
	 * redirect the user to (due to the exception occuring). For example, in the case 
	 * of a Secure Activity Space violation and the security mode is 1, the Redirect 
	 * object returned here will be an HTTP redirect to the corresponding https URL. 
	 * If the method returns null, the Interpreter will redirect the user (by default)
	 * either to an error page, or to a login page if the user is guest (by calling 
	 * the Authenticator.GetOrSendLoginChallenge()).
	 * 
	 * @return a Redirect object, if any.
	 */
	public Redirect GetRedirect()
	{
		return m_Redirect;
	}
}
