package com.plumtree.uiinfrastructure.interpreter.filter;

import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.interpreter.*;
import com.plumtree.uiinfrastructure.interpreter.filter.utils.*;

/**
 * A filter to handle gateway request.
 * 
 * @author Daniilk  (Gateway)
 * @author WilliamA (Writing this new class)
 */
public class GatewayFilter extends BaseInterpreterFilter {

	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.interpreter.filter.GatewayFilter");
		
	/**
	 * This method detects if the incoming request is a gateway request.
	 * If it is, it will take over the processing of the request, and return
	 * a boolean true to indicate that Interpreter shall not process the
	 * request further. 
	 */
	public boolean PreFilter(RequestData requestData)
	{
		IXPRequest request = requestData.m_xpRequest;
				
		/*
		 * FIRST OF ALL WE NEED TO FIGURE OUT IF THIS IS A GATEWAY REQUEST OR NOT. GATEWAY REQUESTS
		 * ARE SPECIAL AND THEY TAKE DIFFERENT CODE PATH.
		 * 		-- moose
		 */
		String strURL = request.GetRequestURL();
			
		int iQSIndex = strURL.indexOf("?");
		
		if (-1 != iQSIndex)
		{
			strURL = strURL.substring(0, iQSIndex);
		}
		
		// at this point strURL does not have a query string!!!!
		
		// Use the gateway handle request if this is a gateway url
		if ( IsGatewayURL(strURL) )
		{
			log.Debug("Calling HandleGatewayRequest(...) for URL [" 
				  + request.GetRequestURL() + "]");

			// If we need to login from the gateway, we let the
			// interpreter handle that.			
			return GatewayHandlers.HandleGatewayRequest(requestData);
		}

		/*
		 * ================================================================================================
		 * END OF SPECIAL PROCESSING FOR THE GATEWAY REQUESTS
		 * ================================================================================================
		 */

		return false;
	}
	
	protected boolean IsGatewayURL(String url)
	{
		return (-1 != url.indexOf(GatewayHandlers.GATEWAY_UNIQUE));
	}
	
}
