package com.plumtree.uiinfrastructure.interpreter.filter; 

import com.plumtree.openfoundation.web.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.utils.*;
import com.plumtree.uiinfrastructure.httpcompression.*;
import com.plumtree.uiinfrastructure.interpreter.*;
import com.plumtree.uiinfrastructure.web.*;


/**
 * @author scottc
 * HttpCompressionFilter impliments the java servlet Filter interface.
 * It compresses HTTP content using deflate or gzip, detecting browser support,
 * compressing, and configuring headers appropriately.
 */
public class HttpCompressionFilter extends BaseInterpreterFilter {
	
    private static OpenLogger log = OpenLogService.GetLogger(
            OpenLogService.GetComponent("UI_Infrastructure"),
            "com.plumtree.uiinfrastructure.interpreter.filter.HttpCompressionFilter");
            
    public boolean PreFilter(RequestData requestData) 
    {   
        if (CompressionSettings.INSTANCE.IsCompressionEnabled()) {
            log.Debug("HttpCompressionFilter::PreFilter Global Compression has been configured to be enabled.");
            BrowserType browserType = (BrowserType) requestData.m_smSandbox.GetAttribute(Interpreter.BROWSERTYPE);
            IXPRequest xpRequest = requestData.m_xpRequest;
            IXPResponse xpResponse = requestData.m_xpResponse;
            if (null != browserType) {
                // DCA - wrapping XPResponse to use compression
                XPCompressionResponse compressionResponse = new XPCompressionResponse(xpResponse, new HttpCompressionUtil(xpRequest, browserType));
                requestData.m_xpResponse = compressionResponse;
                requestData.m_wdPageData.setResponse(requestData.m_xpResponse);
            } else {
                log.Warn("HttpCompressionFilter::PreFilter BrowserType class not found, compression will not be used.");
            }
        } else {
            log.Info("HttpCompressionFilter::PreFilter Global Compression has been configured to be disabled.");
        }
        return false;
    }
	
}