package com.plumtree.uiinfrastructure.login;

import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * @author DonH
 *
 * This class contains the result of a failed login attempt.
 */
public class FailedLoginResult
{
    /** Localized login error message suitable for display to user. */
    private String m_strError;

    /**
     * The Redirect, if one of the PEIs returned a failed login
     * redirect. May be null.
     **/
    private Redirect m_Redirect;

    /**
     * Creates a new FailedLoginResult object with a custom error message.
     *
     * @param _strError The custom error message for the end user.
     */
    public FailedLoginResult(String _strError)
    {
        m_strError = _strError;
        m_Redirect = null;
    }

	/**
	 * Creates a new FailedLoginResult object for a failed
	 * login that was redirected from a PEI.
	 *
	 * @param redirect where to redirect the user to (specified by a login PEI).
	 */
	public FailedLoginResult(Redirect _redirect)
	{
		m_Redirect = _redirect;
		m_strError = null;
	}

	/**
	 * Creates a new FailedLoginResult object for a failed
	 * login that was redirected from a PEI.
	 *
     * @param _strError The custom error message for the end user (maybe ignored).
	 * @param redirect where to redirect the user to (specified by a login PEI).
	 */
	public FailedLoginResult(String _strError, Redirect _redirect)
	{
		m_Redirect = _redirect;
		m_strError = _strError;
	}

	/**
	 * Get the error message for this failed login.
	 * 
	 * @return String the error message for this failed login.
	 */
	public String GetErrorMessage()
	{
		return m_strError;
	}

	/**
	 * Get the redirect for this failed login.
	 * 
	 * @return Redirect the redirect for this failed login.
	 */
	public Redirect GetFailureRedirect()
	{
		return m_Redirect;
	}
}