package com.plumtree.uiinfrastructure.login;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.pei.*;

/**
 * This helper object wraps up Login PEIs that implement
 * either ILoginActions or ILoginActions2 and makes them
 * behave the same.
 * 
 * @author DonH
 */
public class LoginPEIContainer implements ILoginActions2
{
	protected ILoginActions m_LoginActionsPEI;
	protected ILoginActions2 m_LoginActions2PEI;

	public LoginPEIContainer(ILoginActions _pei)
	{
		m_LoginActionsPEI = _pei;

		if (_pei instanceof ILoginActions2)
		{
			m_LoginActions2PEI = (ILoginActions2) _pei;
		} else
		{
			m_LoginActions2PEI = null;
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.pei.ILoginActions2#OnFailedLoginDoRedirect(com.plumtree.uiinfrastructure.login.ApplicationData, java.lang.String, com.plumtree.openfoundation.util.XPException)
	 */
	public Redirect OnFailedLoginDoRedirect(
		ApplicationData _appData,
		String _strUserName,
		XPException _xpError)
	{
		if (null != m_LoginActions2PEI)
		{
			return m_LoginActions2PEI.OnFailedLoginDoRedirect
				(_appData, _strUserName, _xpError);
		}

		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.pei.ILoginActions#OnAfterLogin(java.lang.Object, com.plumtree.uiinfrastructure.login.ApplicationData)
	 */
	public Redirect OnAfterLogin(
		Object _oUserSession,
		ApplicationData _appData)
	{
		return m_LoginActionsPEI.OnAfterLogin(_oUserSession, _appData);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.pei.ILoginActions#OnBeforeLogout(java.lang.Object, com.plumtree.uiinfrastructure.login.ApplicationData)
	 */
	public Redirect OnBeforeLogout(
		Object _oUserSession,
		ApplicationData _appData)
	{
		return m_LoginActionsPEI.OnBeforeLogout(_oUserSession, _appData);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.pei.ILoginActions#OnBeforeLogin(com.plumtree.uiinfrastructure.login.ApplicationData, java.lang.String)
	 */
	public Redirect OnBeforeLogin(
		ApplicationData _appData,
		String _strUserName)
	{
		return m_LoginActionsPEI.OnBeforeLogin(_appData, _strUserName);
	}

	/**
	 * @see com.plumtree.uiinfrastructure.pei.ILoginActions#OnFailedLogin(com.plumtree.uiinfrastructure.login.ApplicationData, java.lang.String, com.plumtree.openfoundation.util.XPException)
	 */
	public String OnFailedLogin(
		ApplicationData _appData,
		String _strUserName,
		XPException _xpError)
	{
		return m_LoginActionsPEI.OnFailedLogin(_appData, _strUserName, _xpError);
	}

	/**
	 * This method is overridden so that when we try to display
	 * the contents of the PEI lists we get an appropriate value
	 * instead of just seeing wrapper objects.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return m_LoginActionsPEI.toString();
	}
}