package com.plumtree.uiinfrastructure.login;

import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * This class contains the result of an attempted login.
 */
public class LoginResult
{
    /** The user session, if successful.  Null if login failed. */
    public Object m_UserSession;

    /** The Redirect, if one of the PEIs returned a redirect. May be null. */
    public Redirect m_Redirect;

	/** Whether or not the login was succesful */
	public boolean m_bSuccess = false;

	/** Whether or not the login was redirected before the login attempt */
	public boolean m_bRedirectedBeforeLogin = false;

    /** Whether or not the login is for the guest user */
    public boolean m_bIsGuest = false;

    /**
     * Localized login error message suitable for display to user. Will be
     * null if attempt succesful.
     */
    public String m_strError;

    /**
     * Creates a new LoginResult object for a succesful login.
     *
     * @param userSession The new user session object.
     * @param redirect where to redirect the user to (specified by a login PEI).
	 * @param bIsGuestUser
     */
    public LoginResult(Object userSession, Redirect redirect, boolean bIsGuestUser)
    {
        m_UserSession = userSession;
        m_Redirect    = redirect;
        m_bSuccess    = true;
        m_bIsGuest    = bIsGuestUser;
        m_strError    = null;
    }

    /**
     * Creates a new LoginResult object for a failed login.
     *
     * @param userSession The new user session object.
     * @param redirect where to redirect the user to (specified by a login PEI).
     * @param strError An error message suitable for display to an end user
     * (ie localized, and not revealing too much information).  Should not be null.
	 * @param bIsGuestUser
     */
    public LoginResult(Object userSession, Redirect redirect, String strError, boolean bIsGuestUser)
    {
		// We do not return the login exception to the clients of this code
		// because displaying it to the end user (or putting it in an HTMLComment)
		// is a security hole for people trying to guess user names and passwords.
		// If they can see the original error (user not found), then they can
		// easily create a list of users.

        m_UserSession = userSession;
        m_Redirect    = redirect;
        m_bSuccess    = false;
        m_bIsGuest    = bIsGuestUser;
        m_strError    = strError;
    }

	/**
	 * Creates a new LoginResult object for a login that was
	 * redirected before the login attempt.
	 *
	 * @param redirect where to redirect the user to (specified by a login PEI).
	 */
	public LoginResult(Redirect redirect)
	{
		m_UserSession = null;
		m_Redirect    = redirect;
		m_bRedirectedBeforeLogin = true;
		m_bSuccess    = true;
		m_bIsGuest    = true;
		m_strError    = null;
	}
}
