package com.plumtree.uiinfrastructure.login;

import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.interpreter.*;

/**
 * This singleton class controls SSO redirects.  To implement your own SSO
 * redirect code, simply implement an instance of ISSOHelper and initialize this
 * class with your helper using the StaticInit method.
 * 
 * Created on Mar 13, 2003
 * 
 * @author mikej
 */
public final class SSOHelper implements ISSOHelper {
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.login.SSOHelper");


    private static ISSOHelper s_SSOHelper = null;
    private static boolean s_bInitialized = false;

    /**
     * Use this class static to access the one and only instance of
     * SSOHelper.
     */
    public static final SSOHelper INSTANCE = new SSOHelper();

    /**
     * Private constructor to ensure there is only one instance.
     */
    private SSOHelper()
    {
    }

    /**
     * Static initialization method.  Use this method to add custom user
     * session object handling code.
     * 
     * This method does not need to be called.
     * 
     * @param ISSOHelper custom SSO code
     */
    public static final void StaticInit(ISSOHelper SSOHelper)
    {
        if (s_bInitialized)
        {
            
                log.Error("SSOHelper StaticInit attempted twice.  Ignoring second attempt.");
            

            return;
        }

        s_SSOHelper = SSOHelper;
        s_bInitialized = true;
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.login.ISSOHelper#IsSSOEnabled()
     */
    public boolean IsSSOEnabled() {
        if (s_SSOHelper == null)
        {
            return false;
        }
		
		return s_SSOHelper.IsSSOEnabled();
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.login.ISSOHelper#DoLoginRedirect(com.plumtree.uiinfrastructure.activityspace.RequestData, java.lang.String)
     */
    public Redirect DoLoginRedirect(RequestData tempData, String sQueryString) {
        if (s_SSOHelper == null)
        {
            return null;
        }
		
        return s_SSOHelper.DoLoginRedirect(tempData, sQueryString);
    }

    /* (non-Javadoc)
     * @see com.plumtree.uiinfrastructure.login.ISSOHelper#DoLogoutRedirect(com.plumtree.uiinfrastructure.activityspace.RequestData)
     */
    public boolean DoLogoutRedirect(RequestData tempData) {
        if (s_SSOHelper == null)
        {
            return false;
        }
		
        return s_SSOHelper.DoLogoutRedirect(tempData);
    }

}

