package com.plumtree.uiinfrastructure.opener.plugins;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.opener.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

/**
 * This plugin enables the Common Opener to open objects using
 * the more human readable "open" parameter instead of the
 * "clsID" parameter.  The "open" is processed into its
 * ClassID equivalent and put back into the QS.
 */
public class OpenPlugin implements IOpenerPlugins
{
	/**
	 * @see com.plumtree.portaluiinfrastructure.opener.IOpenerPlugins#ProcessPlugIn(RedirectData, AActivitySpace, Object)
	 */
	public RedirectData ProcessPlugIn( RedirectData _redirectData,
		AActivitySpace _asCurrentSpace, Object _userSession)
	{
		// get the "open" parameter from the querystring. 
		String strOpen = HTTPHelpers.GetArgumentAsString(_redirectData.
			GetQSArguments(), IOpenerConstants.QS_OPEN, null);

		if (null != strOpen && strOpen.length() > 0)
		{
			int nClassID = -1;
			try
			{
				// Try to convert the "open" parameter into a numeric classID
				char firstChar = XPStringUtility.CharAt(strOpen, 0);
				if (XPCharacterUtility.IsASCIIDecimalDigit(firstChar))
				{
				    nClassID = XPConvert.ToInteger(strOpen);
				}
			} 
			catch (Exception e)
			{
				// This is an expected result for UI Objects, which
				// have non-numeric class keys and no object ID.
				// Ignore the exception.
				nClassID = -1;
			}

			if (nClassID != -1)
			{
				// put ClassID in return data, overwriting if it exists. 
				_redirectData.AddQSArgument(IOpenerConstants.QS_CLASSID, nClassID);
			}
		} 

		return _redirectData;
	}
}