package com.plumtree.uiinfrastructure.opener.types;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.opener.*;
import com.plumtree.uiinfrastructure.web.*;

/**
 * TODO: Document
 * 
 * @author DonH
 */
public class ClassTypeManager implements IClassTypeManager
{
	////////////////////////////
	// IClassTypeDesc Members //
	////////////////////////////
	
	/**
	 * A map of loaded ClassTypeDescriptors.  The key is the ClassID for
	 * PTClassTypeDesc objects and ClassName for UIClassTypeDesc objects.
	 **/
	protected XPHashtable m_mapClassTypes;

	/** An array of all loaded IClassType objects. */
	protected IClassTypeDesc[] m_aIClassTypes;

	/**
	 * @param cmCachingManager
	 */
	public ClassTypeManager(CachingManager cmCachingManager)
	{
		///////////////////////////////////
		// IClassTypeDesc Initialization //
		///////////////////////////////////
	    
		// load the dynamically discovered ClassTypeDesc objects.
		Object[] oaClassTypes = (Object[]) cmCachingManager.GetEntry("ClassTypeDesc");
		IClassTypeDesc[] aIClassTypes = new IClassTypeDesc[oaClassTypes.length];

		for (int i = 0; i < oaClassTypes.length; i++)
		{
			aIClassTypes[i] = (IClassTypeDesc) oaClassTypes[i];
		}

		m_aIClassTypes = aIClassTypes;
		m_mapClassTypes = new XPHashtable();

		////////////////////////////
		// General Initialization //
		////////////////////////////
		
		InitializeMapOfClassTypes();
	}

	private final void InitializeMapOfClassTypes()
	{
		for (int i = 0; i < m_aIClassTypes.length; i++)
		{
			m_mapClassTypes.PutElement(m_aIClassTypes[i].GetClassKey(), 
				m_aIClassTypes[i]);
		}		    
	}

	/**
	 * @see com.plumtree.uiinfrastructure.opener.types.IOpenerTypeManager#GetDefaultOpenerMode(java.lang.String)
	 */
	public int GetDefaultOpenerMode(String _strClassKey)
	{
		try
		{
			return GetClassTypeDescItem(_strClassKey).GetDefaultOpenerMode();		
		}
		catch (Exception ex)
		{
			throw new XPException("Error retrieving default opener mode " +
				"for ClassKey [" + _strClassKey + "]", ex);
		}
	}

	/**
	 * @see com.plumtree.uiinfrastructure.opener.types.IOpenerTypeManager#GetOpenerRedirect(java.lang.String, int, com.plumtree.openfoundation.util.XPHashtable, com.plumtree.uiinfrastructure.activityspace.AActivitySpace, java.lang.Object)
	 */
	public Redirect GetOpenerRedirect(
		String _strClassKey,
		int _nOpenerMode,
		XPHashtable _htArguments,
		AActivitySpace _asParent,
		Object _userSession)
	{
		switch (_nOpenerMode)
		{
			case IOpenerConstants.OPENER_MODE_CREATE:
				return GetClassTypeDescItem(_strClassKey).GetRedirectCreateNew(
					_htArguments, _asParent, _userSession);		

			case IOpenerConstants.OPENER_MODE_EDIT:
				return GetClassTypeDescItem(_strClassKey).GetRedirectOpenObject(
					_htArguments, _asParent, _userSession);		

			case IOpenerConstants.OPENER_MODE_VIEW:
				return GetClassTypeDescItem(_strClassKey).GetRedirectViewObject(
					_htArguments, _asParent, _userSession);
			
			case IOpenerConstants.OPENER_MODE_VIEW_META_DATA:
				return GetClassTypeDescItem(_strClassKey).GetRedirectViewObjectMetaData(
					_htArguments, _asParent, _userSession);
			
			case IOpenerConstants.OPENER_MODE_SEARCH_BROWSE:
				return GetClassTypeDescItem(_strClassKey).GetRedirectSearchBrowse(
					_htArguments,_asParent, _userSession);
			
			case IOpenerConstants.OPENER_MODE_SEARCH_EDIT:
				return GetClassTypeDescItem(_strClassKey).GetRedirectSearchEdit(
					_htArguments, _asParent, _userSession);
			
			default:
				// unsupported opener mode.										
				UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER, 
					null, _asParent.GetString(485, "ptmsgs_portalinfrastructure"), null);
				UIErrorHandler.RaiseError(myError, _asParent.GetSubSession(), 
					_asParent.GetApplication());			
				throw new XPException("The OpenerMode argument specified is invalid.");
		}
	}

	/**
	 * Returns IClassTypeDesc for the particular ClassKey.
	 *
	 * @param _strClassKey For PTClassTypes, this is the ClassID, for 
	 * 		  UIClassTypes, this is the ClassName.
	 *
	 * @return IClassTypeDesc for the particular ClassKey.
	 */
	protected IClassTypeDesc GetClassTypeDescItem(String _strClassKey)
	{
		try
		{
			IClassTypeDesc item = (IClassTypeDesc) m_mapClassTypes.
				GetElement(_strClassKey);

			return item;
		}
		catch (Exception ex)
		{
			throw new XPException("The ClassKey [" + _strClassKey + "] is invalid.", ex);
		}
	}
}