package com.plumtree.uiinfrastructure.opener.types;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * Interface for IClassTypeDesc.  
 * 
 * @author John Osborne
 */
public interface IClassTypeDesc
{
	/**
	 * Returns the class type.
	 * 
	 * @return The class type.
	 */
	public ClassTypeEnum GetClassType();

	/**
	 * Returns the key for this class type.
	 * 
	 * @return The key for this class type.
	 */
	public String GetClassKey();

	/**
	 * Returns the default opener mode for this class type.
	 *
	 * @return The default opener mode for this class type.
	 */
	public int GetDefaultOpenerMode();

	/**
	 * This method is called by the Common Opener in the create
	 * mode.  Depending on what class type you are dealing with,
	 * the Common Opener will invoke the correct implemented 
	 * method.  Some class types may need to override this so
	 * that they can put additional control arguments in.
	 *
	 * @param _htArguments Arguments to reach the new space.
	 * @param _asParent The parent activity space.
	 * @param _userSession The current session.
	 *
	 * @return A redirect that will bring you to the create 
	 * activity space for the particular class/object.
	 */
	public Redirect GetRedirectCreateNew(XPHashtable _htArguments,
		AActivitySpace _asParent, Object _userSession);

	/**
	 * This method is called by the Common Opener in the edit
	 * mode.  Depending on what class type you are dealing with,
	 * the Common Opener will invoke the correct implemented 
	 * method.  Some class types may need to override this so
	 * that they can put additional control arguments in.
	 *
	 * @param _htArguments Arguments to reach the new space.
	 * @param _asParent The parent activity space.
	 * @param _userSession The current session.
	 *
	 * @return A redirect that will bring you to the edit 
	 * activity space for the particular class/object.
	 */
	public Redirect GetRedirectOpenObject(XPHashtable _htArguments,
		AActivitySpace _asParent, Object _userSession);

	/**
	 * This method is called by the Common Opener in the search
	 * basic mode.  This is a helper method that will return one
	 * of the four main modes.
	 *
	 * @param _htArguments Arguments to reach the new space.
	 * @param _asParent The parent activity space.
	 * @param _userSession The current session.
	 *
	 * @return A redirect that will bring you to the meta 
	 * activity space for the particular class/object.
	 */
	public Redirect GetRedirectSearchBrowse(XPHashtable _htArguments,
		AActivitySpace _asParent, Object _userSession);

	/**
	 * This method is called by the Common Opener in the search
	 * basic mode.  This is a helper method that will return one
	 * of the four main modes.
	 *
	 * @param _htArguments Arguments to reach the new space.
	 * @param _asParent The parent activity space.
	 * @param _userSession The current session.
	 *
	 * @return A redirect that will bring you to the meta 
	 * activity space for the particular class/object.
	 */
	public Redirect GetRedirectSearchEdit(XPHashtable _htArguments,
		AActivitySpace _asParent, Object _userSession);
		
	/**
	 * This method is called by the Common Opener in the view
	 * mode.  Depending on what class type you are dealing with,
	 * the Common Opener will invoke the correct implemented 
	 * method.  Some class types may need to override this so
	 * that they can put additional control arguments in.
	 *
	 * @param _htArguments Arguments to reach the new space.
	 * @param _asParent The parent activity space.
	 * @param _userSession The current session.
	 *
	 * @return A redirect that will bring you to the view 
	 * activity space for the particular class/object.
	 */
	public Redirect GetRedirectViewObject(XPHashtable _htArguments,
		AActivitySpace _asParent, Object _userSession);

	/**
	 * This method is called by the Common Opener in the meta
	 * mode.  Depending on what class type you are dealing with,
	 * the Common Opener will invoke the correct implemented 
	 * method.  Some class types may need to override this so
	 * that they can put additional control arguments in.
	 *
	 * @param _htArguments Arguments to reach the new space.
	 * @param _asParent The parent activity space.
	 * @param _userSession The current session.
	 *
	 * @return A redirect that will bring you to the meta 
	 * activity space for the particular class/object.
	 */
	public Redirect GetRedirectViewObjectMetaData(XPHashtable _htArguments,
		AActivitySpace _asParent, Object _userSession);
}

