package com.plumtree.uiinfrastructure.opener.types.classtypes;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.opener.*;
import com.plumtree.uiinfrastructure.opener.types.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

/**
 * This psuedo class type represents the ID to use to tell the
 * Common Opener to open Activity Spaces.  Although the Common
 * Opener is supposed to open objects, there are places in the
 * Portal that are not related to objects, and thus this class
 * type will allow you to go to those pages.
 * 
 * @author RickyW
 */
public class UIActivitySpaces extends AUIClassTypeDesc
{
    /** Name that identifies this UIClassTypeDesc */
    public static final String UICLASSTYPEDESC_CLASSNAME = "space";

	// The following query string parameters are necessary so
	// we don't confuse the parameters used to open the Common
	// Opener space with the parameters used to open the
	// target space. 

	/** QueryString paramenter for defining AS name. */
	public static final String QS_ACTIVITY_SPACE_NAME = "name";

	/** QueryString paramenter for defining AS ID or to use a cached space. */
	public static final String QS_ACTIVITY_SPACE_ID = "id";

	/** QueryString paramenter value for using a cached space. */
	public static final String QS_SPACE_ID_CACHED = "cached";

	/** QueryString paramenter for defining parent AS name. */
	public static final String QS_PARENT_SPACE_NAME = "psname";

	/** QueryString paramenter for defining parent AS ID. */
	public static final String QS_PARENT_SPACE_ID = "psid";

	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IClassTypeDesc#GetClassType()
	 */
	public ClassTypeEnum GetClassType()
	{
	    return ClassTypeEnum.UICLASSTYPE;
	}
	
	/* (non-Javadoc)
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IUIClassTypeDesc#GetClassName()
	 */
	public String GetClassName()
	{
	    return UICLASSTYPEDESC_CLASSNAME;
	}
	
	/*
	 * @see com.plumtree.portaluiinfrastructure.classtypedescriptors.IClassTypeDesc#GetRedirectViewObject(com.plumtree.openfoundation.util.XPHashtable, com.plumtree.uiinfrastructure.activityspace.AActivitySpace, com.plumtree.server.IPTSession)
	 */
	public Redirect GetRedirectViewObject(XPHashtable _htArguments, AActivitySpace _asParent, Object _userSession)
	{
	    // retrieve the Activity Space parameters from the query string.
		String strActivitySpaceName = HTTPHelpers.GetArgumentAsString(
			_htArguments, UIActivitySpaces.QS_ACTIVITY_SPACE_NAME, null);
		String strActivitySpaceID = HTTPHelpers.GetArgumentAsString(
			_htArguments, UIActivitySpaces.QS_ACTIVITY_SPACE_ID, null);
		String strParentActivitySpaceName = HTTPHelpers.GetArgumentAsString(
			_htArguments, UIActivitySpaces.QS_PARENT_SPACE_NAME, null);
		String strParentActivitySpaceID = HTTPHelpers.GetArgumentAsString(
			_htArguments, UIActivitySpaces.QS_PARENT_SPACE_ID, null);
		
		if (null == strActivitySpaceName)
		{
		    // TODO - change error message, it's using the gateway one right now.
			UIError myError = new UIError(UIErrorLevel.SERIOUS, UIErrorScope.CURRENT_USER,  
				null, _asParent.GetString(497, "ptmsgs_portalinfrastructure"), null);
			UIErrorHandler.RaiseError(myError, _asParent.GetSubSession(), 
				_asParent.GetApplication());
						
			throw new XPException("An Activity Space Name argument is required in the " +
			    "Common Opener URL to open Activity Spaces.");			
		}

		Redirect asRedirect = new Redirect();
	    
		// the Open and AS args should not be passed.
		_htArguments.RemoveElement(IOpenerConstants.QS_OPEN);
		_htArguments.RemoveElement(IOpenerConstants.QS_MODE);
		_htArguments.RemoveElement(UIActivitySpaces.QS_ACTIVITY_SPACE_NAME);
		_htArguments.RemoveElement(UIActivitySpaces.QS_ACTIVITY_SPACE_ID);
		_htArguments.RemoveElement(UIActivitySpaces.QS_PARENT_SPACE_NAME);
		_htArguments.RemoveElement(UIActivitySpaces.QS_PARENT_SPACE_ID);

		// Remove the arguments used to access the opener space
		_htArguments.RemoveElement(ASConstants.SPACENAME);
		_htArguments.RemoveElement(ASConstants.SPACENAME_INPUT);
		_htArguments.RemoveElement(ASConstants.CACHED);

		// other infrastructure arguments that should not be passed.
		_htArguments.RemoveElement(HttpUtils.HASHTABLE_CONTROL_KEY_URL);
		_htArguments.RemoveElement(HttpUtils.HASHTABLE_CONTROL_KEY_HTTP_HEADERS);

		// Always create a new space.  If the opener url has extra
		// arguments, such as space ID or cached, these will overwrite
		// the base ones in the redirect.
		asRedirect.SetLinkCreateNewSpace(strActivitySpaceName, _asParent);

		// Add the correct space ID information, if present
		if (null != strActivitySpaceID)
		{
			if (0 == XPStringUtility.Compare(strActivitySpaceID, QS_SPACE_ID_CACHED))
			{
				asRedirect.AddControlArgument(ASConstants.CACHED, true);
			} else
			{
				asRedirect.AddControlArgument(ASConstants.SPACEID, strActivitySpaceID);
			}
		}

		if (null != _htArguments)
		{
			// if there are QS arguments pass them through.
		    asRedirect.AddControlArguments(_htArguments);
		}

		// Only replace the parent space information if it was specifically passed in
		// (i.e. from ASURL).  Otherwise, links formed from the opener won't have the
		// correct parent space information.
		if (null != strParentActivitySpaceName)
		{
			// Add the correct parent space information to the redirect
			asRedirect.AddControlArgument(ASConstants.PARENTSPACENAME, strParentActivitySpaceName);
			asRedirect.AddControlArgument(ASConstants.PARENTSPACEID, strParentActivitySpaceID);
	
			// Add the correct parent space information back to the hash table
			// of arguments because the common opener control uses them.
			_htArguments.PutElement(ASConstants.PARENTSPACENAME, new String[] { strParentActivitySpaceName });
			_htArguments.PutElement(ASConstants.PARENTSPACEID, new String[] { strParentActivitySpaceID });
		}

		return asRedirect;
	}	
}