package com.plumtree.uiinfrastructure.pagecontrols.datepicker;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.uiinfrastructure.pagecontrols.htmldatetimepicker.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.*;

/**
 * Abstract class extended by all DatePickers.
 * 
 * @author DorisL
 * @author CaiusV
 */
public abstract class ADatePickerComponent extends AComponent implements IPageControl
{

	/** Index constant for array of options: option value */
	public static final int OPTION_VALUE = 0;

	/** Index constant for array of options: text to display on option */
	public static final int OPTION_TEXT = 1;
	
	/** some other helpful static Strings */
	public static final String AM = "AM";
	public static final String PM = "PM";
	public static final String H = "H";
	public static final String EMPTY = "";
	
	public XPDateTime GetDateAsDateTime()
	{
		return null;
	}

	public void SetBlankDate()
	{
	}

	public void SetDefaultDate(int _nMonth, int _nDay, int _nYear)
	{
	}

	public abstract String GetSelectedDateAsString();

	public abstract String GetDateInputName();

	public abstract String GetPTDatepickerVarName();

	public XPDateTime GetDateTimeAsDateTime()
	{
		return null;
	}

	/**
	 * Initialize a new DatePicker with the preset date empty.
	 * 
	 * @param _strFormName Name of the form where this DatePicker appears
	 * @param _ptPageMgr the PageMgr responsible for this control
	 * @param _strUniqueInstanceID unique id for this instance
	 */
	public abstract void Init(
		String _strFormName,
		PageMgr _ptPageMgr,
		String _strUniqueInstanceID);

	/**
	 * 
	 * Initialize a new DatePicker with the preset date set.
	 * 
	 * @param _strFormName Name of the form where this DatePicker appears
	 * @param _dtDefaultDateTime the preset datetime
	 * @param _ptPageMgr the PageMgr responsible for this control
	 * @param _strUniqueInstanceID unique id for this instance
	 */
	public abstract void Init(
		String _strFormName,
		XPDateTime _dtDefaultDateTime,
		PageMgr _ptPageMgr,
		String _strUniqueInstanceID);

	/**
	 * Return the XPDateTime for the specific data picker
	 * 
	 * @param _htRequestParameters name/value pairs in the request
	 * @param _strUniqueInstanceID unique id for this control
	 * @param _xpLocale Locale for this control
	 * @param _xpTimeZone TimeZone for this control
	 * @param boolean _bDoValidation , set to true if date must be set (pulldowns cannot be empty)
	 * @param AccessStyles _asAccessStyle, the page's mode
	 *
	 *@return XPDateTime
	 */
	public static XPDateTime GetDateTime(
		XPHashtable _htRequestParameters,
		String _strUniqueInstanceID,
		XPLocale _xpLocale,
		XPTimeZone _xpTimeZone,
		boolean _bDoValidation,
		AccessStyles _asAccessStyle)
	{
		if (_asAccessStyle.equals(AccessStyles.STANDARD))
		{
			return JSDateTimePicker.GetDateTime(_htRequestParameters,_strUniqueInstanceID,_xpLocale, _xpTimeZone);
		}
		else
		{
			return HTMLDateTimePicker.GetDateTime(_htRequestParameters,_strUniqueInstanceID,_xpLocale, _xpTimeZone,_bDoValidation);
		}
	}

}
