package com.plumtree.uiinfrastructure.pagecontrols.datepicker;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.xpshared.jsutil.*;

/**
 * Page Control for HTML Date Picker.
 *
 * @author Doris Lin
 */
public class DatePicker extends ADatePickerComponent implements IPageControl {
	/** for logging */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
															 "com.plumtree.uiinfrastructure.pagecontrols.datepicker.DatePicker");
	

	/** String constant for month select drop down component */
	public static final String MONTH_DROP_DOWN = "SDD_MONTH";

	/** String constant for day select drop down component */
	public static final String DAY_DROP_DOWN = "SDD_DAY";

	/** String constant for year select drop down component */
	public static final String YEAR_DROP_DOWN = "SDD_YEAR";

	/** String constant for min year, max year, min num of days, max num of days */
	public static final int MIN_YEAR 		= 1950;
	public static final int NUM_OF_YEARS 	= 71;
	public static final int MIN_DAY 		= 0;
	public static final int MAX_DAY 		= 31;
	public static final int NUM_OF_MONTHS = 12;

	/** Constant for Month Names */
	public static final int BLANK = 0;
	
	/** Constant String for BLANK */
	public static final String STR_BLANK = "0";

	/** Delimeters in the date pattern */
	public static final String DELIMETERS = " /-,.";

	/** Reg. expression for delimeters in date pattern */
	public static final String REG_EXP_DELEMETERS = "[,\\s./-]";

	/** Variables to save the order in which the month, day and year should appear (ranging from 0-2) */
	private static int m_nOrderM;
	private static int m_nOrderd;
	private static int m_nOrdery;

	/** mvc class name for datepicker component */
	public static final String STR_MVC_CLASS_NAME = "DatePicker_D";
	
	/** component's base name */
	public static final String STR_COMP_BASE_NAME = "in_se_pc";

	/** Variables to save the month, day, and year */
	protected String m_strMonth;
	protected String m_strDay;
	protected String m_strYear;

	/** HTMLSelectDropDown Components */
	private HTMLSelectDropDown m_ptHTMLMonthSelectDropDown;
	private HTMLSelectDropDown m_ptHTMLDaySelectDropDown;
	private HTMLSelectDropDown m_ptHTMLYearSelectDropDown;

	/** DateFormat object */
	private XPDateFormat m_dfDateFormat;

	/** Variables to save the number of M's, d's, and y's */
	private int m_nNumM;
	private int m_nNumd;

	/** String array to save the delimeters for the date pattern */
	private XPMatch[] m_nDateDelims = new XPMatch[3];

	/** Array containing month names */
	private String[][] m_arrMonthNames;

	/** unique string id for control */
	private String m_strUniqueInstanceID;

	/**
	 * Creates a new DatePicker for 508 mode
	 */
	public DatePicker()
	{
	}

	/**
	 * initializes a new HTMLDatePicker object with the month, day and year blank
	 *
	 * @param _strFormName Name of the form where this DatePicker appears
	 * @param _ptPageMgr the PageMgr responsible for this control
	 * @param _strUniqueInstanceID unique id for this instance
	 */
	public void Init(
		String _strFormName,
		PageMgr _ptPageMgr,
		String _strUniqueInstanceID)
	{
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		
		IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
		XPResourceManager rm = (XPResourceManager) application.GetCachingManager().GetEntry("ResourceManager");

		m_strUniqueInstanceID = _strUniqueInstanceID;

		String[][] arrDropDownValueText;

		//	create dateformat object for medium format and the given locale
		m_dfDateFormat = XPDateFormat.GetDateInstance(
				XPDateFormat.MEDIUM,
				_ptPageMgr.GetLocale(),
				_ptPageMgr.GetTimeZone());

		// extract and save the information from the pattern
		ExtractPatternInfo(m_dfDateFormat.ToPattern());

		// initialize the month select drop down component
		m_arrMonthNames = new String[NUM_OF_MONTHS + 1][2];

		// fill the array with either month names or numbers, depending on the num. of M's
		switch (m_nNumM)
		{
			case 1 :
				FillSingleDigitArray(m_arrMonthNames);
				break;
			case 2 :
				FillDoubleDigitArray(m_arrMonthNames);
				break;
			case 3 :
				FillMonthNameArray(m_arrMonthNames);
				break;
		}

		m_ptHTMLMonthSelectDropDown =
			(HTMLSelectDropDown) m_ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
		m_ptHTMLMonthSelectDropDown.Init(
			m_ptPageMgr,
			MONTH_DROP_DOWN + _strUniqueInstanceID,
			m_arrMonthNames, rm.GetString(148, "ptmsgs_infrastructure", m_ptPageMgr.GetLangID()));

		// initialize the day select drop down component
		arrDropDownValueText = new String[MAX_DAY + 1][2];

		// fill the array with either "01" or "1" format, depending on the num of d's
		switch (m_nNumd)
		{
			case 1 :
				FillSingleDigitArray(arrDropDownValueText);
				break;
			case 2 :
				FillDoubleDigitArray(arrDropDownValueText);
				break;
		}

		m_ptHTMLDaySelectDropDown =
			(HTMLSelectDropDown) m_ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
		m_ptHTMLDaySelectDropDown.Init(
			m_ptPageMgr,
			DAY_DROP_DOWN + _strUniqueInstanceID,
			arrDropDownValueText, 
			rm.GetString(149, "ptmsgs_infrastructure", m_ptPageMgr.GetLangID()));

		// initialize the year select drop down component
		arrDropDownValueText = new String[NUM_OF_YEARS + 1][2];
		FillYearArray(arrDropDownValueText);
		m_ptHTMLYearSelectDropDown =
			(HTMLSelectDropDown) m_ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
		m_ptHTMLYearSelectDropDown.Init(
			m_ptPageMgr,
			YEAR_DROP_DOWN + _strUniqueInstanceID,
			arrDropDownValueText,  rm.GetString(150, "ptmsgs_infrastructure", m_ptPageMgr.GetLangID()));
			

		// set the default date to be blank
		SetBlankDate();
	}

	/**
	 * Initializes a new DatePicker Page Control with the given datetime as the
	 * preset value.
	 *
	 * @param _strFormName Name of the form where this DatePicker appears
	 * @param _xpDateTime Preset DateTime
	 * @param _ptPageMgr the PageMgr responsible for this control
	 * @param _strUniqueInstanceID unique id for this instance
	 */
	public void Init(
		String _strFormName,
		XPDateTime _xpDateTime,
		PageMgr _ptPageMgr,
		String _strUniqueInstanceID) 
	{
		this.Init(_strFormName, _ptPageMgr, _strUniqueInstanceID);

		XPCalendar myXPCalendar = XPCalendar.GetInstance(_ptPageMgr.GetLocale());

		if (_xpDateTime != null) 
		{
			myXPCalendar.SetTime(_xpDateTime);

			m_ptHTMLMonthSelectDropDown.SetHTMLComponentValue(
					EMPTY + myXPCalendar.Get(XPCalendar.MONTH) + EMPTY);
			m_ptHTMLDaySelectDropDown.SetHTMLComponentValue(
					EMPTY + myXPCalendar.Get(XPCalendar.DATE) + EMPTY);
			m_ptHTMLYearSelectDropDown.SetHTMLComponentValue(
					EMPTY + myXPCalendar.Get(XPCalendar.YEAR) + EMPTY);
				
			m_strMonth 	= EMPTY + myXPCalendar.Get(XPCalendar.YEAR) + EMPTY;
			m_strDay 	= EMPTY + myXPCalendar.Get(XPCalendar.DATE) + EMPTY;
			m_strYear 	= EMPTY + myXPCalendar.Get(XPCalendar.YEAR) + EMPTY;
		}
	}

	/**
	 * Sets the month, day, and year of drop down menus to blank
	 */
	public void SetBlankDate()
	{
		m_ptHTMLMonthSelectDropDown.SetHTMLComponentValue(EMPTY + DatePicker.BLANK + EMPTY);
		m_ptHTMLDaySelectDropDown.SetHTMLComponentValue(EMPTY + DatePicker.BLANK + EMPTY);
		m_ptHTMLYearSelectDropDown.SetHTMLComponentValue(EMPTY + DatePicker.BLANK + EMPTY);
	}

	/**
	 * Returns the base name of this HTML OpenComponent.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName() 
	{
		return STR_COMP_BASE_NAME;
	}

	/**
	 * Returns the date as a String in the pattern format specified by the locale, null
	 * values in memeber variables, m_strYear, m_strMonth, or m_strDay indicate an invalid
	 * date so return the default, and empty String.
	 * 
	 * @return String the date as a String
	 */
	public String GetSelectedDateAsString() 
	{
		if (null == m_strYear || null == m_strMonth || null == m_strDay)
		{
			return EMPTY;
		}
		switch (m_nOrderd)
		{
			case 0 :
				if (m_nOrdery == 1) 
				{
					return m_strDay + (m_nDateDelims[0]).ToString() + m_strYear
						+ (m_nDateDelims[1]).ToString()
						+ m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT];
				}
				else
				{
					return m_strDay
						+ (m_nDateDelims[0]).ToString()
						+ m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT]
						+ (m_nDateDelims[1]).ToString()
						+ m_strYear;
				}

			case 1 :
				if (m_nOrdery == 0)
				{
					return m_strYear
						+ (m_nDateDelims[0]).ToString()
						+ m_strDay
						+ (m_nDateDelims[1]).ToString()
						+ m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT];
				}
				else
				{
					return m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT]
						+ (m_nDateDelims[0]).ToString()
						+ m_strDay
						+ (m_nDateDelims[1]).ToString()
						+ " "
						+ m_strYear;
				}

			case 2 :
				if (m_nOrdery == 0)
				{
					return m_strYear
						+ (m_nDateDelims[0]).ToString()
						+ m_strDay
						+ (m_nDateDelims[1]).ToString()
						+ m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT];
				}
				else
				{
					return m_strDay
						+ (m_nDateDelims[0]).ToString()
						+ m_strYear
						+ (m_nDateDelims[1]).ToString()
						+ m_arrMonthNames[XPConvert.ToInteger(m_strMonth)][OPTION_TEXT];
				}
		}
		return EMPTY;
	}

	/**
	 * This function returns an XPDateTime object that represents the date and
	 *  the user has chosen using the HTMLDatePicker.
	 *
	 * @return XPDateTime
	 */
	public XPDateTime GetDateAsDateTime()
	{
		XPCalendar myXPCalendar;
		try { 
			myXPCalendar = XPCalendar.GetInstance(
					XPConvert.ToInteger(m_strYear),
					XPConvert.ToInteger(m_strMonth),
					XPConvert.ToInteger(m_strDay));
		} 
		catch (XPException e)
		{
			log.Error(e, "unable to retreive datetime");
			return null;
		}
		return myXPCalendar.GetTime();
	}

	/**
	 * Returns the HTML for this component.
	 *
	 * @return HTMLElementCollection the HTML to display this control
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		int nOrder = 0;
		try {
			// determine which order the components should be displayed
			for (int i = 0; i < 3; i++)
			{
				if (m_nOrderM == nOrder) 
				{
					// add month drop down component 
					retValue.AddInnerHTMLElement(GetMonthDropDownHTML());
				} 
				else if (m_nOrderd == nOrder)
				{
					// day drop down component 
					retValue.AddInnerHTMLElement(GetDayDropDownHTML());
				} 
				else 
				{
					//year drop down component
					retValue.AddInnerHTMLElement(GetYearDropDownHTML());
				}

				// add the delimeter after each component except for the last component
				if (i < 2)
				{
					retValue.AddInnerHTMLString((m_nDateDelims[i]).GetValue());
				}

				nOrder++;
			}
		} 
		catch (XPException e) 
		{
			log.Error(e, "cannot retreive componenet HTML");
		}

		return retValue;
	}

	/**
	* Store the new value of this HTML component.
	*
	* @param _htRequestParameters name/value pairs for the request
	* @return int - the status of the page
	*/
	public int StoreUserInput(XPHashtable _htRequestParameters)
	{
		String[] data;
		m_ptHTMLMonthSelectDropDown.StoreUserInput(_htRequestParameters);
		m_ptHTMLDaySelectDropDown.StoreUserInput(_htRequestParameters);
		m_ptHTMLYearSelectDropDown.StoreUserInput(_htRequestParameters);
		
		//reset values for month, day, and year
		m_strMonth = STR_BLANK; m_strDay = STR_BLANK; m_strYear = STR_BLANK;
		
		data = (String[]) _htRequestParameters.GetElement(
				GetHTMLComponentBaseName()
					+ "_"
					+ MONTH_DROP_DOWN
					+ m_strUniqueInstanceID);
		if (data != null)
		{
			m_strMonth = data[0];
		}
		data = (String[]) _htRequestParameters.GetElement(
				GetHTMLComponentBaseName()
					+ "_"
					+ DAY_DROP_DOWN
					+ m_strUniqueInstanceID);
		if (data != null)
		{
			m_strDay = data[0];
		}
		data = (String[]) _htRequestParameters.GetElement(
				GetHTMLComponentBaseName()
					+ "_"
					+ YEAR_DROP_DOWN
					+ m_strUniqueInstanceID);
		if (data != null)
		{
			m_strYear = data[0];
		}

		// if all the fields are blank, then the page is still valid
		if (m_strMonth.equals(STR_BLANK) && m_strDay.equals(STR_BLANK) && m_strYear.equals(STR_BLANK))
		{
			return RepostControl.PAGE_STATUS_VALID;
		}

		// check if the date is valid, an invalid date will not be set
		try 
		{
			XPCalendar.GetInstance(
				XPConvert.ToInteger(m_strYear),
				XPConvert.ToInteger(m_strMonth),
				XPConvert.ToInteger(m_strDay));
		} 
		catch (XPException e)
		{
			log.Error(e, "invalid date");
			m_ptPageMgr.AddErrorMessage("153", ResourceConstants.INFRASTRUCTURE);		
			return RepostControl.PAGE_STATUS_INVALID;
		}

		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * The HTMLElementCollectiion for the month drop down
	 * @return HTMLElementCollection
	 */
	private HTMLElementCollection GetMonthDropDownHTML()
	{
		m_ptHTMLMonthSelectDropDown = (HTMLSelectDropDown)
				(m_ptPageMgr.GetComponentMgr(HTMLSelectDropDown.STR_MVC_CLASS_NAME)).GetComponent(
						MONTH_DROP_DOWN + m_strUniqueInstanceID);
		m_ptHTMLMonthSelectDropDown.SetHTMLComponentStyleClass(
			PTStyleClass.FORM_PULLDOWN_TEXT);

		return m_ptHTMLMonthSelectDropDown.GetComponentHTML();
	}

	/**
	* The HTMLElementCollection for the day drop down
	* @return HTMLElementCollection
	*/
	private HTMLElementCollection GetDayDropDownHTML()
	{
		m_ptHTMLDaySelectDropDown = (HTMLSelectDropDown)
				(m_ptPageMgr.GetComponentMgr(HTMLSelectDropDown.STR_MVC_CLASS_NAME)).GetComponent(
						DAY_DROP_DOWN + m_strUniqueInstanceID);
		m_ptHTMLDaySelectDropDown.SetHTMLComponentStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
		return m_ptHTMLDaySelectDropDown.GetComponentHTML();
	}

	/**
	 * The HTMLElementCollection for the year drop down
	 * @return HTMLElementCollection for the year dropdown
	 */
	private HTMLElementCollection GetYearDropDownHTML()
	{
		m_ptHTMLYearSelectDropDown = (HTMLSelectDropDown)
				(m_ptPageMgr.GetComponentMgr(HTMLSelectDropDown.STR_MVC_CLASS_NAME)).GetComponent(
						YEAR_DROP_DOWN + m_strUniqueInstanceID);
		m_ptHTMLYearSelectDropDown.SetHTMLComponentStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
		return m_ptHTMLYearSelectDropDown.GetComponentHTML();
	}

	/**
	 * Fill in the array to create the select drop down component for months
	 * 
	 * @param _arrDropDownValueText text for the month dropdown
	 */
	private void FillMonthNameArray(String[][] _arrDropDownValueText)
	{
		XPCalendar myXPCalendar;
		XPDateTime myXPDateTime;
		XPDateFormat myXPDateFormat;
		String dateString;
		_arrDropDownValueText[0][OPTION_VALUE] = EMPTY + DatePicker.BLANK + EMPTY;
		_arrDropDownValueText[0][OPTION_TEXT] = " ";
		for (int i = 1; i < _arrDropDownValueText.length; i++)
		{
			_arrDropDownValueText[i][OPTION_VALUE] = EMPTY + i + EMPTY;
			myXPCalendar = XPCalendar.GetInstance(2004, i, 1);
			myXPDateTime = myXPCalendar.GetTime();
			myXPDateFormat = XPDateFormat.GetDateInstance(
					XPDateFormat.MEDIUM,
					m_ptPageMgr.GetLocale(),
					m_ptPageMgr.GetTimeZone());
			dateString = myXPDateFormat.Format(myXPDateTime);
			_arrDropDownValueText[i][OPTION_TEXT] = GetMonthName(dateString);
		}
	}

	/**
	 * Fill in the array to create the select drop down component for day with format d
	 * 
	 * @param  _arrDropDownValueText text for the dropdown
	 */
	private void FillSingleDigitArray(String[][] _arrDropDownValueText)
	{
		_arrDropDownValueText[0][OPTION_VALUE] = EMPTY + DatePicker.BLANK + EMPTY;
		_arrDropDownValueText[0][OPTION_TEXT] = " ";
		for (int i = 1; i < _arrDropDownValueText.length; i++) 
		{
			_arrDropDownValueText[i][OPTION_VALUE] = EMPTY + i + EMPTY;
			_arrDropDownValueText[i][OPTION_TEXT] = EMPTY + i + EMPTY;
		}
	}

	/**
	 * Fill in the array to create the select drop down component for day with format dd
	 * @param  _arrDropDownValueText text for the dropdown
	 */
	private void FillDoubleDigitArray(String[][] _arrDropDownValueText) 
	{
		_arrDropDownValueText[0][OPTION_VALUE] = EMPTY + DatePicker.BLANK + EMPTY;
		_arrDropDownValueText[0][OPTION_TEXT] = " ";
		for (int i = 1; i < _arrDropDownValueText.length; i++) 
		{
			_arrDropDownValueText[i][OPTION_VALUE] = EMPTY + i + EMPTY;
			if (i < 10) 
			{
				_arrDropDownValueText[i][OPTION_TEXT] = "0" + i + EMPTY;
			}
			else
			{
				_arrDropDownValueText[i][OPTION_TEXT] = EMPTY + i + EMPTY;
			}
		}
	}

	/**
	 * Fill in the array to create the select drop down component for year
	 * @param  _arrDropDownValueText text for the year dropdown
	 */
	private void FillYearArray(String[][] _arrDropDownValueText)
	{
		_arrDropDownValueText[0][OPTION_VALUE] = EMPTY + DatePicker.BLANK + EMPTY;
		_arrDropDownValueText[0][OPTION_TEXT] = " ";
		for (int i = 1; i < (NUM_OF_YEARS + 1); i++)
		{
			_arrDropDownValueText[i][OPTION_VALUE] = EMPTY + ((i - 1) + MIN_YEAR) + EMPTY;
			_arrDropDownValueText[i][OPTION_TEXT]  = EMPTY + ((i - 1) + MIN_YEAR) + EMPTY;
		}
	}

	/**
	 * create the month name drop down menu based on the format given by the locale.
	 * 
	 * @param _strDate the date
	 * @return the month name
	 */
	private String GetMonthName(String _strDate)
	{
		XPStringTokenizer myXPStrTokenizer = new XPStringTokenizer(_strDate, DELIMETERS);
		for (int i = 0; myXPStrTokenizer.HasMoreTokens(); i++) 
		{
			if (i >= m_nOrderM) 
			{
				return myXPStrTokenizer.GetNextToken();
			} 
			else
			{
				myXPStrTokenizer.GetNextToken();
			}
		}
		return null;
	}

	/**
	 * Reset the index for the order in which the month, year, and day should appear to be between (0-2)
	 * instead of their index position in the date pattern
	 */
	private void ResetIndex()
	{
		if (m_nOrderM == 0)
		{
			if (m_nOrderd > m_nOrdery) 
			{
				m_nOrderd = 2;
				m_nOrdery = 1;
			} 
			else
			{
				m_nOrderd = 1;
				m_nOrdery = 2;
			}
		}
		else if (m_nOrderd == 0)
		{
			if (m_nOrderM > m_nOrdery)
			{
				m_nOrderM = 2;
				m_nOrdery = 1;
			}
			else
			{
				m_nOrderM = 1;
				m_nOrdery = 2;
			}
		} 
		else
		{
			if (m_nOrderd > m_nOrderM)
			{
				m_nOrderd = 2;
				m_nOrderM = 1;
			}
			else
			{
				m_nOrderd = 1;
				m_nOrderM = 2;
			}
		}
	}

	/**
	 * ExtractPatternInfo saves information about the date pattern such as the number of M's, and d's and the
	 * index (order) in which they appear in the pattern.  The delimeters between the month, day, and year will
	 * also be saved in an array.
	 * 
	 * @param _strPattern pattern with date format instructions
	 */
	private void ExtractPatternInfo(String _strPattern)
	{
		XPMatch month;
		XPMatch day;
		XPMatch year;

		// record the number of M
		month = XPRegex.Match(_strPattern, "M+");
		m_nNumM = month.GetLength();
		m_nOrderM = month.GetIndex();

		// record the number of y
		year = XPRegex.Match(_strPattern, "y+");
		m_nOrdery = year.GetIndex();

		// record the number of d
		day = XPRegex.Match(_strPattern, "d+");
		m_nNumd = day.GetLength();
		m_nOrderd = day.GetIndex();

		//record the delimeters
		m_nDateDelims = XPRegex.Matches(_strPattern, REG_EXP_DELEMETERS);

		ResetIndex();
	}

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new DatePicker();
	}

	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName() 
	{
		return STR_MVC_CLASS_NAME;
	}
	
	/**
	 * Returns the name of the PTDatepicker JavaScript variable name. In this
	 * case there isn't one so return null;
	 *
	 * @return the name of the object.
	 */
	public String GetPTDatepickerVarName()
	{
		return null;
	}
	
	/**
	 * Returns the name of the date input. In this
	 * case there isn't one so return null;
	 *
	 * @return the name of the object.
	 */
	public String GetDateInputName()
	{
		return null;
	}
	
	public ComponentMgr GetComponentMgr(PageMgr _ptPageMgr)
	{
		JSRegistry ptJSRegistry = JSRegistry.GetRegistry(_ptPageMgr.GetImgSvrBaseURL(),
					_ptPageMgr.GetImageServerConnectionURL());
		return new JSDateTimePickerMgr(ptJSRegistry, new JSDatePickerVersionMgr(), _ptPageMgr);
	}
	
	/**
	 * Return the name of the component manager responsible for this page control
	 *
	 * @return the name of the ComponentMgr for this class
	 */
	public String GetComponentMgrName()
	{
		return JSDateTimePickerMgr.STR_CLASS_NAME;
	}
	
	/**
	 * Currently validation on the date/time picker page controls is not implemented.
	 * This method returns null.
	 */
	public XPArrayList Validate()
	{
		return null;
	}
	
	/**
	 * Currently validation on the date/time picker page controls is not implemented.
	 * This method returns null.
	 */
	public XPArrayList GetValidators()
	{
		return null;
	}

}
