package com.plumtree.uiinfrastructure.pagecontrols.htmldatetimepicker;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.OpenLogService;
import com.plumtree.openlog.OpenLogger;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.pagecontrols.datepicker.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.uiinfrastructure.pagecontrols.timepicker.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author DorisL
 *
 */
public class HTMLDateTimePicker extends DatePicker {

	public String STR_MVC_CLASS_NAME = "DateTimePicker_D";

	/** The time-picker part of this component. */
	private TimePicker m_tpTimePicker;

	/** unique instance id */
	private String m_strUniqueInstanceID;

	/** Some static String used by the datetimepicker */
	private static String EMPTY = "";
	
	/** logger */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
															 "com.plumtree.uiinfrastructure.pagecontrols.htmldatetimepicker.HTMLDateTimePicker");

	/**
	 * Empty constructor Creates a new HTMLDateTimePicker
	 */
	public HTMLDateTimePicker()
	{
	}

	/**
	 * Initialize this HTMLDateTimePicker
	 *
	 * @param  _strFormName the name of the form where this HTMLDateTimePicker
	 * 				appears
	 * @param  _ptPageMgr the PageMgr that owns this HTMLDateTimePicker
	 * @param  _strUniqueInstanceID a unique id for the HTMLDateTimePicker
	 */
	public void Init(String _strFormName,
		PageMgr _ptPageMgr,
		String _strUniqueInstanceID)
	{
		super.Init(_strFormName, _ptPageMgr, _strUniqueInstanceID);
		m_tpTimePicker = (TimePicker) _ptPageMgr.AddPageControl(TimePicker.STR_MVC_CLASS_NAME);
		m_tpTimePicker.Init(_ptPageMgr, EMPTY, _strUniqueInstanceID);
	}

	/**
	 * Initialize this HTMLDateTimePicker and set it to the default datetime
	 * if that datetime isn't null.
	 *
	 * @param  _strFormName the name of the form where this HTMLDateTimePicker
	 * 				appears
	 * @param  _dtDefaultDateTime to set the HTMLDateTimePicker to
	 * @param  _ptPageMgr the PageMgr that owns this HTMLDateTimePicker
	 * @param  _strUniqueInstanceID a unique id for the HTMLDateTimePicker
	 */
	public void Init(
		String _strFormName,
		XPDateTime _dtDefaultDateTime,
		PageMgr _ptPageMgr,
		String _strUniqueInstanceID) 
	{
		super.Init(
			_strFormName,
			_dtDefaultDateTime,
			_ptPageMgr,
			_strUniqueInstanceID);
		
		m_strUniqueInstanceID = _strUniqueInstanceID;

		//	Set the date format object - use thM format
		XPDateFormat dfDateTimeFormat = XPDateFormat.GetDateTimeInstance(
				XPDateFormat.MEDIUM,
				XPDateFormat.MEDIUM,
				_ptPageMgr.GetLocale(),
				_ptPageMgr.GetTimeZone());

		// Get a calendar
		XPCalendar cal = XPCalendar.GetInstance(_ptPageMgr.GetLocale());
		cal.SetTimeZone(_ptPageMgr.GetTimeZone());

		// Check for the 24 hour clock pattern
		boolean b24Hour = false;
		String strDateTimePattern = dfDateTimeFormat.ToPattern();

		if (strDateTimePattern.indexOf(H) != -1)
		{
			b24Hour = true;
		}

		if (null != _dtDefaultDateTime)
		{
			cal.SetTime(_dtDefaultDateTime);
			if (b24Hour)
			{
				m_tpTimePicker = (TimePicker) _ptPageMgr.AddPageControl(TimePicker.STR_MVC_CLASS_NAME);
				m_tpTimePicker.Init(
					cal.Get(XPCalendar.HOUR_OF_DAY),
					cal.Get(XPCalendar.MINUTE),
					_ptPageMgr,
					_strUniqueInstanceID);
			} 
			else if (cal.Get(XPCalendar.AM_PM) == 0) 
			{
				m_tpTimePicker = (TimePicker) _ptPageMgr.AddPageControl(TimePicker.STR_MVC_CLASS_NAME);
				m_tpTimePicker.Init(
					cal.Get(XPCalendar.HOUR),
					cal.Get(XPCalendar.MINUTE),
					dfDateTimeFormat.GetAmPmStrings()[0],
					_ptPageMgr,
					_strUniqueInstanceID);
			} 
			else
			{
				m_tpTimePicker = (TimePicker) _ptPageMgr.AddPageControl(TimePicker.STR_MVC_CLASS_NAME);
				m_tpTimePicker.Init(
					cal.Get(XPCalendar.HOUR),
					cal.Get(XPCalendar.MINUTE),
					dfDateTimeFormat.GetAmPmStrings()[1],
					_ptPageMgr,
					_strUniqueInstanceID);
			}
		} 
		else 
		{
			m_tpTimePicker = (TimePicker) _ptPageMgr.AddPageControl(TimePicker.STR_MVC_CLASS_NAME);
			m_tpTimePicker.Init(_ptPageMgr, EMPTY, _strUniqueInstanceID);
		}
	}

	/**
	 * This method returns the AM/PM part of the time. If we are using 24-hour
	 * time this method returns null.
     *
     * @return The AM/PM part of the time String.
     */
	public String GetAMPM()
	{
		return m_tpTimePicker.GetAMPM();
	}

	/**
     * Provides a means of retrieving the HTML Input name for the AMPM portion
     * of the date picker.
     *
     * @return String
     */
	public String GetAMPMInputName() 
	{
		return m_tpTimePicker.GetAMPMInputName();
	}

	/**
     * This method returns the date/time-picker HTML.
     *
     * @return The HTMLElementCollection that is the date and time picker.
     */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retVal = super.GetComponentHTML();
		retVal.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		retVal.AddInnerHTMLString(CommonHTMLStrings.SPACE);
		retVal.AddInnerHTMLElement(m_tpTimePicker.GetComponentHTML());
		return retVal;
	}

	/**
     * This function returns an XPDateTime object that represents the date and
     * time the user has chosen using the date/time-picker HTML control.
     *
     * @return XPDateTime
     */
	public XPDateTime GetDateTimeAsDateTime()
	{
		XPCalendar myXPCalendar;
		int nHours = 0;
		int nTime;
		String strAMPM = null;
		try 
		{
			strAMPM = GetAMPM();
			// check to see if the am/pm is set
			if (strAMPM != null) 
			{
				// special check for 12th  hr am/pm setting
				if ((nTime = XPConvert.ToInteger(m_tpTimePicker.GetHours())) == 12) 
				{
					// if AM, substract 12
					if (strAMPM.equals(AM))
					{
						nHours = nTime - 12;
					} 
					else 
					{
						nHours = nTime;
					}
				}
				// check whether need to add 12 hrs depending on if am/pm is set
				else if (m_tpTimePicker.GetAMPM().equals(PM)) 
				{
					nHours = XPConvert.ToInteger(m_tpTimePicker.GetHours()) + 12;
				} 
				else
				{
					nHours = XPConvert.ToInteger(m_tpTimePicker.GetHours());
				}
			} 
			else 
			{
				nHours = XPConvert.ToInteger(m_tpTimePicker.GetHours());
			}
			myXPCalendar = XPCalendar.GetInstance(
					XPConvert.ToInteger(m_strYear),
					XPConvert.ToInteger(m_strMonth),
					XPConvert.ToInteger(m_strDay),
					nHours,
					XPConvert.ToInteger(m_tpTimePicker.GetMinutes()));
		} 
		catch (XPException e) 
		{
			log.Error(e, "unable to retreive datetime");
			return null;
		}
		return myXPCalendar.GetTime();
	}

	/**
     * This method returns the date and time as a String.
     *
     * @return The date/time string.
     */
	public String GetDateTimeAsString() {
		return GetSelectedDateAsString() + " " + m_tpTimePicker.GetTimeAsEnglishString();
	}

	/**
	 * This method returns the hours part of the time as a String.
	 *
	 * @return The hours.
	 */
	public String GetHours() {
		return m_tpTimePicker.GetHours();
	}

	/**
     * Provides a means of retrieving the HTML Input name for the hours portion
     * of the date picker.
     *
     * @return String
     */
	public String GetHoursInputName() {
		return m_tpTimePicker.GetHoursInputName();
	}

	/**
     * This method returns the minutes part of the time as a String.
     *
     * @return The minutes.
     */
	public String GetMinutes() {
		return m_tpTimePicker.GetMinutes();
	}

	/**
     * Provides a means of retrieving the HTML Input name for the minutes
     * portion of the date picker.
     *
     * @return String
     */
	public String GetMinutesInputName() {
		return m_tpTimePicker.GetMinutesInputName();
	}

	/**
	 * This method serves as a getter for "sub-component" HTML. The
	 * date/time-picker is made up of two sub-components: the date-picker and
	 * the time-picker. The constants for component sub-index should be
	 * specified in the component file.
	 *
	 * @param _nSubComponentIndex The sub-component index
	 * 		(N_SUB_COMPONENT_INDEX_DATE_PICKER or
	 *      	N_SUB_COMPONENT_INDEX_TIME_PICKER).
	 *
	 * @return The HTML for the sub-component requested.
	 */
	public HTMLElementCollection GetSubComponentHTMLByIndex(int _nSubComponentIndex)
	{
		if (_nSubComponentIndex == 0)
		{
			return super.GetComponentHTML();
		} 
		else if (_nSubComponentIndex == 1)
		{
			return m_tpTimePicker.GetComponentHTML();
		}
		return null;
	}

	/**
	 * This method returns just the time part of the String.
	 *
	 * @return The time String.
	 */
	public String GetTimeAsString()
	{
		return m_tpTimePicker.GetTimeAsString();
	}

	/**
	 * Stores the user input after the request is sent back.
	 *
	 * @param _htRequestParameters The XPHashtable of request name/value pairs.
	 */
	public int StoreUserInput(XPHashtable _htRequestParameters) 
	{
		int nDateStatus;
		int nTimeStatus;
		nDateStatus = super.StoreUserInput(_htRequestParameters);
		nTimeStatus = m_tpTimePicker.StoreUserInput(_htRequestParameters);
		if (nDateStatus == RepostControl.PAGE_STATUS_INVALID)
		{
			return nDateStatus;
		}
		return nTimeStatus;
	}

	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new HTMLDateTimePicker();
	}

	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName() 
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Return the XPDateTime for the specific date picker
	 * 
	 *@param _htRequestParameters
	 *@param _strUniqueInstanceID 
	 *@param _xpLocale
	 *@param _xpTimeZone
	 *@param boolean _bDoValidation , set to true if date must be set (pulldowns cannot be empty)
	 *
	 *@return XPDateTime
	 */
	public static XPDateTime GetDateTime (
		XPHashtable _htRequestParameters,
		String _strUniqueInstanceID,
		XPLocale _xpLocale,
		XPTimeZone _xpTimeZone,
		boolean _bDoValidation)
	{
		XPCalendar myCalendar 	= null;
		String strMonth 		= STR_BLANK;
		String strDay 			= STR_BLANK;
		String strYear 			= STR_BLANK;
		String strHours 		= null; 
		String strMinutes 		= null;
		String strAMPM 			= null;
		int nHours 				= 0;
		int nTime;
		String[] data;

		// get the month
		data = (String[]) _htRequestParameters.GetElement(
				STR_COMP_BASE_NAME + "_" + MONTH_DROP_DOWN + _strUniqueInstanceID);		
		if (data != null)
		{
			strMonth = data[0];
		}
		
		// get the day
		data = (String[]) _htRequestParameters.GetElement(
				STR_COMP_BASE_NAME + "_" + DAY_DROP_DOWN + _strUniqueInstanceID);
		if (data != null)
		{
			strDay = data[0];
		}
		
		// get the year
		data = (String[]) _htRequestParameters.GetElement(
				STR_COMP_BASE_NAME + "_" + YEAR_DROP_DOWN + _strUniqueInstanceID);
		if (data != null) {
			strYear = data[0];
		}

		// get the hour
		data = (String[]) _htRequestParameters.GetElement(
				TimePicker.HTMLSELECT_HOURS + _strUniqueInstanceID);
		if (data != null)
		{
			strHours = data[0];
		}
		
		// get the minutes
		data =
			(String[]) _htRequestParameters.GetElement(
				TimePicker.HTMLSELECT_MINUTES + _strUniqueInstanceID);
		if (data != null)
		{
			strMinutes = data[0];
		}
		
		//null values in hours minutes indicate and invalid datetime
		if (null == strHours || null == strMinutes)
		{
			log.Error("strHours, strMinutes cannot be null");
			return null;	
		}

		// get am/pm
		data = (String[]) _htRequestParameters.GetElement(
				TimePicker.HTMLSELECT_AMPM + _strUniqueInstanceID);
		if (data != null && !(data[0].equals(EMPTY)))
		{
			strAMPM = data[0];
		}

		try {
			// check to see if the am/pm is set
			if (null != strAMPM)
			{
				// special check for 12th  hr am/pm setting
				if ((nTime = XPConvert.ToInteger(strHours)) == 12) 
				{
					// if AM, substract 12
					if (strAMPM.equals(AM))
					{
						nHours = nTime - 12;
					} 
					else
					{
						nHours = nTime;
					}
				}
				// check whether need to add 12 hrs depending on if am/pm is set
				else if (strAMPM.equals(PM)) 
				{
					nHours = XPConvert.ToInteger(strHours) + 12;
				} 
				else
				{
					nHours = XPConvert.ToInteger(strHours);
				}
			} else {
				nHours = XPConvert.ToInteger(strHours);
			}

			if (_bDoValidation)
			{
				//performs validation
				myCalendar =
				XPCalendar.GetInstance(
					XPConvert.ToInteger(strYear),
					XPConvert.ToInteger(strMonth),
					XPConvert.ToInteger(strDay),
					nHours,
					XPConvert.ToInteger(strMinutes));
			}
			else
			{
				// if all the fields are blank, then return null;
				if (strMonth.equals(STR_BLANK) && strDay.equals(STR_BLANK) 
						&& strYear.equals(STR_BLANK))
				{
					return null;
				}
			}

		} 
		catch (XPException e) 
		{
			log.Error(e, "unable to get datetime");
			return null;	
		}
		// set calendar with correct time zone and locale to return the xpdatetime object
		myCalendar = XPCalendar.GetInstance(_xpTimeZone, _xpLocale);
		myCalendar.Set(
				XPConvert.ToInteger(strYear),
				XPConvert.ToInteger(strMonth),
				XPConvert.ToInteger(strDay), 
				nHours,
				XPConvert.ToInteger(strMinutes));

		return myCalendar.GetTime();
	}
}
