package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlconstructs.PTStyleClass;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Check box page control handles display and values of a check box.
 * 
 * @author CaiusV
 */
public class Button extends AHTMLComponent implements IPageControl
{
	/** logger */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
															 "com.plumtree.uiinfrastructure.pagecontrols.htmlbuttons.HTMLButton");
	/** base name constant */
	private static final String STR_HTML_INPUT_BASE_NAME = "in_bu_pc";

	/** class name constant */	
	public static final String STR_MVC_CLASS_NAME = "Button";

	/** the HTMLInput object this wraps */
	public HTMLInput m_ptHTMLInput;

	/**
	 * Default constructor. Creates a new HTMLCheckBox object.
	 */
	public Button()
	{	
	}
	
	/**
	 * Initializes a Button object.
	 * 
	 * @param _ptPageMgr	PageManager
	 * @param _strUniqueInstanceID	Unique Instance ID
	 */
	public void Init(PageMgr _ptPageMgr, 
						String _strUniqueInstanceID,
						AActivitySpace _ptActivitySpace)
	{
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		m_ptHTMLInput = new HTMLInput(HTMLInputTypes.BUTTON_TYPE, GetHTMLComponentName(), "");
	}
	
	/**
	 * Returns the base name of this HTML Component.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName()
	{
		return STR_HTML_INPUT_BASE_NAME;
	}

	/**
	 * Returns the HTML for this component. 
	 *
	 * @return HTMLElementCollection for this button
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		m_ptHTMLInput.SetValue(GetHTMLComponentValue());
		retValue.AddInnerHTMLElement(m_ptHTMLInput);		
		return retValue;
	}
	
	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new Button();
	}
	
	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
	
	/**
	 * Sets the style class value of this object.
	 * 
	 * @param _style style class value
	 */
	public void SetStyleClass(PTStyleClass _style)
	{
		m_ptHTMLInput.SetStyleClass(_style);
	}

	/**
	 * Returns the style class value of this object.
	 * 
	 * @return the sytle class value of the object.
	 */	
	public String GetStyleClass()
	{
		return m_ptHTMLInput.GetStyleClass(); 
	}
}
