package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.RepostControl;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Check box page control handles display and values of a check box.
 * 
 * @author CaiusV
 */
public class HTMLCheckBox extends AHTMLComponent implements IPageControl
{
	/** logger */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
															 "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLCheckBox");
	/** base name constant */
	private static final String STR_HTML_INPUT_BASE_NAME = "in_cb_pc";

	/** class name constant */	
	public static final String STR_MVC_CLASS_NAME = "HTMLCheckBox";
	
	/** hold this check box's 'checked' state */
	private boolean bChecked = false;
	
	/** the HTMLInput object this wraps */
	public HTMLInput m_ptHTMLInput;

	/**
	 * Default constructor. Creates a new empty HTMLCheckBox object.
	 */
	public HTMLCheckBox()
	{	
	}
	
	/**
	 * Initializes a new HTMLCheckBox object.
	 * 
	 * @param _ptPageMgr	PageManager
	 * @param _strUniqueInstanceID	Unique Instance ID
	 * @param _strCols	Check Box Value
	 */
	public void Init(PageMgr _ptPageMgr, 
						String _strUniqueInstanceID, 
						String _strValue)
	{
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		SetHTMLComponentValue(_strValue);
		m_ptHTMLInput = new HTMLInput(HTMLInputTypes.CHECKBOX_TYPE, GetHTMLComponentName(), "");
	}
	
	/**
	 * Returns the base name of this HTML Component.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName()
	{
		return STR_HTML_INPUT_BASE_NAME;
	}

	/**
	 * Returns the HTML for this component.
	 *
	 * @return
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		m_ptHTMLInput.SetValue(GetHTMLComponentValue());
		m_ptHTMLInput.SetChecked(bChecked);
		retValue.AddInnerHTMLElement(m_ptHTMLInput);
		retValue.AddInnerHTMLString(GetLabel());
		return retValue;
	}
	
	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new HTMLCheckBox();
	}
	
	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Store the new value of this HTML component. Sets bChecked to true
	 * if the value of this checkbox is present in _htRequestParameters.
	 *
	 * @param _htRequestParameters
	 */
	public int StoreUserInput(XPHashtable _htRequestParameters)
	{
		String strValue = GetHTMLComponentValue();
		bChecked = false;
		Object oElement = _htRequestParameters.GetElement(GetHTMLComponentName());
		if (oElement != null)
		{
			String[] aStrValues = (String[]) oElement;
			for(int i = 0; i < aStrValues.length; i++) {
				if (strValue != null && strValue.equals(aStrValues[i])) {
					bChecked = true;					
				}
			}
		}
		return RepostControl.PAGE_STATUS_VALID;
	}

	/**
	 * Sets the boolean checked attribute of this checkbox.
	 * 
	 * @param isChecked boolean
	 */	
	public void SetChecked(boolean isChecked)
	{
		bChecked = isChecked;
	}

	/**
	 * Returns the boolean checked attribute of this checkbox
	 * 
	 * @return boolean checked
	 */	
	public boolean IsChecked()
	{
		return bChecked;
	}
}
