package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.RepostControl;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * HTMLCheckBoxGroup class supports grouping of related checkboxes.
 * 
 * @author CaiusV
 */
public class HTMLCheckBoxGroup extends AHTMLComponent implements IPageControl
{
	/** for logging */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLCheckBoxGroup");
	
	/** base name for input */
	private static final String STR_HTML_INPUT_BASE_NAME = "in_cb_pc";
	
	/** this class name */
	public static final String STR_MVC_CLASS_NAME = "HTMLCheckBoxGroup";
	
	/** the array of checkboxes owned by this group */ 
	public XPArrayList m_checkBoxes;
	
	/** delimits checkboxes */
	protected HTMLElementCollection m_ptDelimiter = null;

	/**
	 * Default constructor, creates a new HTMLCheckBox object
	 */
	public HTMLCheckBoxGroup()
	{	
	}	
	
	/**
	 * Initializes a new ACheckBoxGroup object.
	 * 
	 * @param _ptPageMgr	the PageMgr where this object belongs
	 * @param _strUniqueInstanceID id for this obejct
	 * @param _strValues array of values for checkboxes
	 * @param __strLabels array of labels for checkboxes
	 */
	public void Init(PageMgr _ptPageMgr, 
						String _strUniqueInstanceID, 
						String[] _strValues,
						String[] _strLabels)
	{
		if (null != _strLabels && _strLabels.length != _strValues.length)
		{
			log.Error("Values array not equal to labels array");
			_strLabels = null;
		}
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		m_checkBoxes = new XPArrayList(_strValues.length);
		for (int i = 0; i < _strValues.length; i++)
		{
			HTMLCheckBox checkBox = (HTMLCheckBox) _ptPageMgr.AddPageControl(HTMLCheckBox.STR_MVC_CLASS_NAME);			
			checkBox.Init(_ptPageMgr, _strUniqueInstanceID, _strValues[i]);
			if (null != _strLabels)
			{
				checkBox.SetLabel(_strLabels[i]);
			}
			m_checkBoxes.Add(checkBox);
		}
	}
	
	/**
	 * Returns the base name of this HTML Component.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName()
	{
		return STR_HTML_INPUT_BASE_NAME;
	}

	/**
	 * Returns the HTML for this component.
	 *
	 * @return
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		for (int i = 0; i < m_checkBoxes.GetSize(); i++)
		{
			HTMLCheckBox checkBox = (HTMLCheckBox)m_checkBoxes.GetElement(i);		
			retValue.AddInnerHTMLElement(checkBox.GetComponentHTML());
			if (null != m_ptDelimiter && i < (m_checkBoxes.GetSize() -1))
			{
				retValue.AddInnerHTMLElement(m_ptDelimiter);
			}
		}		
		return retValue;
	}
	
	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new HTMLCheckBoxGroup();
	}
	
	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Store the values of this HTML component.
	 *
	 * @param _htRequestParameters
	 */
	public int StoreUserInput(XPHashtable _htRequestParameters)
	{
		for (int i = 0; i < m_checkBoxes.GetSize(); i++)
		{
			HTMLCheckBox checkBox = (HTMLCheckBox)m_checkBoxes.GetElement(i);
			String strValue = checkBox.GetHTMLComponentValue();
			Object oElement = _htRequestParameters.GetElement(checkBox.GetHTMLComponentName());
			if (oElement != null)
			{
				String[] aStrValues = (String[]) oElement;
				for(int j = 0; j < aStrValues.length; j++)
				{
					if (strValue != null && strValue.equals(aStrValues[j]))
					{
						checkBox.SetChecked(true);					
					}
				}
			}
		}
		return RepostControl.PAGE_STATUS_VALID;
	}


	/**
	 * Return an XPArrayList containing the selected values.
	 * 
	 * @return XPArrayList of selected values
	 */	
	public XPArrayList GetSelectedValues()
	{
		XPArrayList alReturnValue = new XPArrayList();
		for (int i = 0; i < m_checkBoxes.GetSize(); i++)
		{
			HTMLCheckBox checkBox = (HTMLCheckBox)m_checkBoxes.GetElement(i);
			if (checkBox.IsChecked())
			{
				String strValue = checkBox.GetHTMLComponentValue();
				alReturnValue.Add(strValue);				
			}
		}
		return alReturnValue;	
	}

	/**
	 * Set the selected values for this object.
	 * 
	 * @param arrSelectedValues
	 */	
	public void SetSelectedValues(XPArrayList arrSelectedValues)
	{
		for (int i = 0; i < m_checkBoxes.GetSize(); i++)
		{
			HTMLCheckBox checkBox = (HTMLCheckBox)m_checkBoxes.GetElement(i);
			if (arrSelectedValues.Contains(checkBox.GetHTMLComponentValue()))
			{
				checkBox.SetChecked(true);
			}
		}
	}
	
	/**
	 * @return Returns the m_ptDelimiter.
	 */
	public HTMLElementCollection GetDelimiter() {
		return m_ptDelimiter;
	}
	
	/**
	 * @param delimiter The m_ptDelimiter to set.
	 */
	public void SetDelimiter(HTMLElementCollection delimiter) {
		m_ptDelimiter = delimiter;
	}
}
