package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.RepostControl;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * HTMLCheckBoxGroupPlus class supports grouping of related checkboxplus objects.
 * 
 * @author CaiusV
 */
public class HTMLCheckBoxGroupPlus extends AHTMLComponent implements IPageControl
{
	/** for logging */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLCheckBoxGroupPlus");
	
	/** this class name */
	public static final String STR_MVC_CLASS_NAME = "HTMLCheckBoxGroupPlus";

	/** base name for input */
	private static final String STR_HTML_INPUT_BASE_NAME = "in_cb_pc";
	
	/** icons to associate with checkboxes */
	private HTMLImg[] m_ptIcons;
	
	/** ASURLs to associate with checkboxes */
	private XPArrayList m_ptURLs;
	
	/** Descriptions to associate with checkboxes */
	private String[] m_sDescs;

	/** the array of checkboxes owned by this group */ 
	protected XPArrayList m_checkBoxes;
	
	/**
	 * Default constructor, creates a new HTMLCheckBox object
	 */
	public HTMLCheckBoxGroupPlus()
	{	
	}	
	
	/**
	 * Initializes a new ACheckBoxGroupPlus object.
	 * 
	 * @param _ptPageMgr the PageMgr responsible for this control
	 * @param _strUniqueInstanceID unique id
	 * @param _strValues	the values for the checkboxes
	 * @param _strLabels	the labels for the checkboxes
	 * @param _ptIcons		icons for the checkboxes
	 * @param _ptURLs		ASURLs for the checkboxes
	 * @param _sDescs		Descriptions for the checkboxes
	 */
	public void Init(PageMgr _ptPageMgr, 
						String _strUniqueInstanceID, 
						String[] _strValues,
						String[] _strLabels,
						HTMLImg[] _ptIcons,
						XPArrayList _ptURLs,
						String[] _sDescs)
	{
		if (null != _ptIcons && _ptIcons.length != _strValues.length)
		{
			log.Error("Values array not equal to icons array");
			_ptIcons = null;
		}
		if (null != _ptURLs && _ptURLs.GetSize() != _strValues.length)
		{
			log.Error("Values array not equal to links array");
			_sDescs = null;
		}
		if (null != _sDescs && _sDescs.length != _strValues.length)
		{
			log.Error("Values array not equal to descriptions array");
			_sDescs = null;
		}
		m_ptURLs = _ptURLs;
		m_ptIcons = _ptIcons;
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		m_checkBoxes = new XPArrayList(_strValues.length);
		for (int i = 0; i < _strValues.length; i++)
		{
			HTMLCheckBoxPlus checkBoxPlus = (HTMLCheckBoxPlus) _ptPageMgr.AddPageControl(HTMLCheckBoxPlus.STR_MVC_CLASS_NAME);			
			checkBoxPlus.Init(_ptPageMgr, _strUniqueInstanceID, _strValues[i]);
			if (null != _strLabels)
			{
				checkBoxPlus.SetLabel(_strLabels[i]);
			}
			if (null != _ptIcons)
			{
				checkBoxPlus.SetIcon(_ptIcons[i]);
			}
			if (null != _ptURLs)
			{
				checkBoxPlus.SetURL((ASURL)_ptURLs.GetElement(i));
			}
			if (null != _sDescs)
			{
				checkBoxPlus.SetDescription(_sDescs[i]);
			}
			m_checkBoxes.Add(checkBoxPlus);
		}
	}

	/**
	 * Returns the HTML for this component. This is not a method that is often used.
	 * Rather, iterate over each CheckBoxPlus object and render them -- this way 
	 * offers more flexible display options.
	 *
	 * @return HTMLElementCollection for display in a view
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		for (int i = 0; i < m_checkBoxes.GetSize(); i++)
		{
			retValue.AddInnerHTMLElement(GetElement(i).GetComponentHTML());
		}		
		return retValue;
	}
	
	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new HTMLCheckBoxGroupPlus();
	}
	
	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
	
	/**
	 * Return the number of CheckBoxPlus objects for this group.
	 * 
	 * @return the number of CheckBoxPlus objects
	 */
	public int GetSize()
	{
		return m_checkBoxes.GetSize();
	}
	
	/**
	 * Return a specific CheckBoxPlus object by index.
	 * 
	 * @param _iIndex the index
	 * @return the CheckBoxPlus object at the index.
	 */
	public HTMLCheckBoxPlus GetElement(int _iIndex)
	{
		return (HTMLCheckBoxPlus)m_checkBoxes.GetElement(_iIndex);		
	}
	
	/**
	 * Store the values of this HTML component.
	 *
	 * @param _htRequestParameters
	 */
	public int StoreUserInput(XPHashtable _htRequestParameters)
	{
		for (int i = 0; i < m_checkBoxes.GetSize(); i++)
		{
			HTMLCheckBoxPlus checkBoxPlus = (HTMLCheckBoxPlus)m_checkBoxes.GetElement(i);
			String strValue = checkBoxPlus.GetHTMLComponentValue();
			Object oElement = _htRequestParameters.GetElement(checkBoxPlus.GetHTMLComponentName());
			if (oElement != null)
			{
				String[] aStrValues = (String[]) oElement;
				for(int j = 0; j < aStrValues.length; j++)
				{
					if (strValue != null && strValue.equals(aStrValues[j]))
					{
						checkBoxPlus.SetChecked(true);					
					}
				}
			}
		}
		return RepostControl.PAGE_STATUS_VALID;
	}
	
	/**
	 * Return an XPArrayList containing the selected values.
	 * 
	 * @return XPArrayList of selected values
	 */	
	public XPArrayList GetSelectedValues()
	{
		XPArrayList alReturnValue = new XPArrayList();
		for (int i = 0; i < m_checkBoxes.GetSize(); i++)
		{
			HTMLCheckBoxPlus checkBox = (HTMLCheckBoxPlus)m_checkBoxes.GetElement(i);
			if (checkBox.IsChecked())
			{
				String strValue = checkBox.GetHTMLComponentValue();
				alReturnValue.Add(strValue);				
			}
		}
		return alReturnValue;	
	}

	/**
	 * Returns the base name of this HTML Component.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName()
	{
		return STR_HTML_INPUT_BASE_NAME;
	}
}
