package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Check box page control handles display and values of a check box.
 * 
 * @author CaiusV
 */
public class HTMLCheckBoxPlus extends AHTMLComponent implements IPageControl
{
	/** logger */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
															 "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLCheckBoxPlus");
	/** base name constant */
	private static final String STR_HTML_INPUT_BASE_NAME = "in_cb_pc";

	/** class name constant */	
	public static final String STR_MVC_CLASS_NAME = "HTMLCheckBoxPlus";
	
	/** icon to associate with checkbox */
	private HTMLImg m_ptIcon;
	
	/** ASURL to associate with checkbox */
	private ASURL m_ptURL;
	
	/** Description to associate with checkbox */
	private String m_sDescription;
	
	/** hold this check box's 'checked' state */
	protected boolean bChecked = false;

	/** the HTMLInput object this wraps */
	public HTMLInput m_ptHTMLInput;
	
	/**
	 * Default constructor. Creates a new HTMLCheckBox object.
	 */
	public HTMLCheckBoxPlus()
	{	
	}
	
	/**
	 * Initializes a new HTMLCheckBoxPlus object.
	 * 
	 * @param _ptPageMgr	PageManager
	 * @param _strUniqueInstanceID	Unique Instance ID
	 * @param _strCols	Check Box Value
	 */
	public void Init(PageMgr _ptPageMgr, 
						String _strUniqueInstanceID, 
						String _strValue)
	{
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		SetHTMLComponentValue(_strValue);
		m_ptHTMLInput = new HTMLInput(HTMLInputTypes.CHECKBOX_TYPE, GetHTMLComponentName(), "");
	}
	
	/**
	 * Returns the base name of this HTML Component.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName()
	{
		return STR_HTML_INPUT_BASE_NAME;
	}

	/**
	 * Returns the HTML for this component.
	 *
	 * @return
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		m_ptHTMLInput.SetValue(GetHTMLComponentValue());
		m_ptHTMLInput.SetChecked(bChecked);
		retValue.AddInnerHTMLElement(m_ptHTMLInput);
		return retValue;
	}
	
	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new HTMLCheckBoxPlus();
	}
	
	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
	
	/**
	 * @return Returns the m_ptIcon.
	 */
	public HTMLImg GetIcon() {
		return m_ptIcon;
	}
	/**
	 * @param icon The m_ptIcon to set.
	 */
	public void SetIcon(HTMLImg icon) {
		m_ptIcon = icon;
	}
	/**
	 * @return Returns the m_ptURL.
	 */
	public ASURL GetURL() {
		return m_ptURL;
	}
	/**
	 * @param m_pturl The m_ptURL to set.
	 */
	public void SetURL(ASURL m_pturl) {
		m_ptURL = m_pturl;
	}
	/**
	 * @return Returns the m_sDescription.
	 */
	public String GetDescription() {
		return m_sDescription;
	}
	/**
	 * @param description The m_sDescription to set.
	 */
	public void SetDescription(String description) {
		m_sDescription = description;
	}
	
	/**
	 * Sets the boolean checked attribute of this checkbox.
	 * 
	 * @param isChecked boolean
	 */	
	public void SetChecked(boolean isChecked)
	{
		bChecked = isChecked;
	}
	
	/**
	 * Returns the boolean checked attribute of this checkbox.
	 * 
	 * @param isChecked boolean
	 */	
	public boolean IsChecked()
	{
		return bChecked;
	}
}
