package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.RepostControl;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * HTMLRadioButtonGroup class supports grouping of related radio buttons.
 * 
 * @author CaiusV
 */
public class HTMLRadioButtonGroup extends AHTMLComponent implements IPageControl
{

	/** for logging */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
				"com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLRadioButtonGroup");
	
	/** base name for input */
	private static final String STR_HTML_INPUT_BASE_NAME = "in_ra_pc";
	
	/** this class name */
	public static final String STR_MVC_CLASS_NAME = "HTMLRadioButtonGroup";
	
	/** the array of ARadioButtons owned by this group */ 
	protected XPArrayList m_radioButtons;
	
	/** delimits checkboxes */
	private HTMLElementCollection m_ptDelimiter = null;

	/**
	 * Default constructor. Creates a new HTMLRadioButtonGroup object.
	 */
	public HTMLRadioButtonGroup()
	{	
	}
	
	/**
	 * Initializes a new HTMLRadioButtonGroup object made up of radio buttons with values
	 * passed in as the _arrValues argument.
	 * 
	 * @param _ptPageMgr	PageManager
	 * @param _strUniqueInstanceID	Unique Instance ID
	 * @param _arrValues values for radio buttons
	 */
	public void Init(PageMgr _ptPageMgr, 
						String _strUniqueInstanceID, 
						String[] _arrValues)
	{
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		m_radioButtons = new XPArrayList(_arrValues.length);
		for (int i = 0; i < _arrValues.length; i++)
		{
			HTMLRadioButton radioButton = (HTMLRadioButton) _ptPageMgr.AddPageControl(HTMLRadioButton.STR_MVC_CLASS_NAME);			
			radioButton.Init(_ptPageMgr, _strUniqueInstanceID, _arrValues[i]);
			m_radioButtons.Add(radioButton);
		}
	}
	
	/**
	 * Returns the base name of this HTML Component.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName()
	{
		return STR_HTML_INPUT_BASE_NAME;
	}

	/**
	 * Returns the HTML for this component.
	 *
	 * @return
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		for (int i = 0; i < m_radioButtons.GetSize(); i++)
		{
			HTMLRadioButton radioButton = (HTMLRadioButton)m_radioButtons.GetElement(i);		
			retValue.AddInnerHTMLElement(radioButton.GetComponentHTML());
			if (null != m_ptDelimiter && i < (m_radioButtons.GetSize() -1))
			{
				retValue.AddInnerHTMLElement(m_ptDelimiter);
			}
		}		
		return retValue;
	}
	
	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new HTMLRadioButtonGroup();
	}
	
	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * Store the values of this HTML component.
	 *
	 * @param _htRequestParameters
	 */
	public int StoreUserInput(XPHashtable _htRequestParameters)
	{
		for (int i = 0; i < m_radioButtons.GetSize(); i++)
		{
			HTMLRadioButton radioButton = (HTMLRadioButton)m_radioButtons.GetElement(i);
			String strValue = radioButton.GetHTMLComponentValue();
			boolean bChecked = false;
			Object oElement = _htRequestParameters.GetElement(radioButton.GetHTMLComponentName());
			if (oElement != null)
			{
				String[] aStrValues = (String[]) oElement;
				for(int j = 0; j < aStrValues.length; j++)
				{
					if (strValue != null && strValue.equals(aStrValues[j]))
					{
						radioButton.SetChecked(true);					
					}
				}
			}
		}
		return RepostControl.PAGE_STATUS_VALID;
	}


	/**
	 * Return the currently selected value of the radio button group.
	 * 
	 * @return XPArrayList of selected values
	 */	
	public String GetSelectedValue()
	{
		for (int i = 0; i < m_radioButtons.GetSize(); i++)
		{
			HTMLRadioButton radioButton = (HTMLRadioButton)m_radioButtons.GetElement(i);
			if (radioButton.IsChecked())
			{
				return  radioButton.GetHTMLComponentValue();				
			}
		}
		return null;	
	}

	/**
	 * Set the currently selected value of the radio button group.
	 * 
	 * @return XPArrayList of selected values
	 */
	public void SetSelectedValue(String strSelectedValue)
	{
		for (int i = 0; i < m_radioButtons.GetSize(); i++)
		{
			HTMLRadioButton radioButton = (HTMLRadioButton)m_radioButtons.GetElement(i);
			if (strSelectedValue.equals(radioButton.GetHTMLComponentValue()))
			{
				radioButton.SetChecked(true);
			}
		}
	}
		
	/**
	 * @return Returns the m_ptDelimiter.
	 */
	public HTMLElementCollection GetDelimiter() {
		return m_ptDelimiter;
	}
	
	/**
	 * @param delimiter The m_ptDelimiter to set.
	 */
	public void SetDelimiter(HTMLElementCollection delimiter) {
		m_ptDelimiter = delimiter;
	}
}
