package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.form.RepostControl;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * HTMLRadioButtonGroupPlus class supports grouping of related radiobuttons.
 * 
 * @author CaiusV
 */
public class HTMLRadioButtonGroupPlus extends AHTMLComponent implements IPageControl
{
	/** for logging */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
			"com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLRadioButtonGroupPlus");
	
	/** this class name */
	public static final String STR_MVC_CLASS_NAME = "HTMLRadioButtonGroupPlus";

	/** base name for input */
	private static final String STR_HTML_INPUT_BASE_NAME = "in_ra_pc";
	
	/** icons to associate with radiobuttons */
	private HTMLImg[] m_ptIcons;
	
	/** ASURLs to associate with radiobuttons */
	private XPArrayList m_ptURLs;
	
	/** Descriptions to associate with radiobuttons */
	private String[] m_sDescs;

	/** the array of ARadioButtons owned by this group */ 
	protected XPArrayList m_radioButtons;
	
	/**
	 * Default constructor, creates a new HTMLRadioButton object
	 */
	public HTMLRadioButtonGroupPlus()
	{	
	}	
	
	/**
	 * Initializes a new ARadioButtonGroupPlus object. Assign values, labels, icons
	 * ASURLs and descriptions by passing in the appropriate arrays.
	 * 
	 * @param _ptPageMgr	PageManager
	 * @param _strUniqueInstanceID	Unique Instance ID
	 * @param _strValues values for the radio buttons
	 * @param _strLabels labels for the radio buttons
	 * @param _ptIcons   icons  for the radio buttons
	 * @param _ptURLs    ASURLs for the radio buttons
	 * @param _sDescs    text descriptions for the radio buttons
	 */
	public void Init(PageMgr _ptPageMgr, 
						String _strUniqueInstanceID, 
						String[] _strValues,
						String[] _strLabels,
						HTMLImg[] _ptIcons,
						XPArrayList _ptURLs,
						String[] _sDescs)
	{
		if (null != _ptIcons && _ptIcons.length != _strValues.length)
		{
			log.Error("Values array not equal to icons array");
			_ptIcons = null;
		}
		if (null != _ptURLs && _ptURLs.GetSize() != _strValues.length)
		{
			log.Error("Values array not equal to links array");
			_sDescs = null;
		}
		if (null != _sDescs && _sDescs.length != _strValues.length)
		{
			log.Error("Values array not equal to descriptions array");
			_sDescs = null;
		}
		m_ptURLs = _ptURLs;
		m_ptIcons = _ptIcons;
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		m_radioButtons = new XPArrayList(_strValues.length);
		for (int i = 0; i < _strValues.length; i++)
		{
			HTMLRadioButtonPlus radioPlus = (HTMLRadioButtonPlus) _ptPageMgr.AddPageControl(HTMLRadioButtonPlus.STR_MVC_CLASS_NAME);			
			radioPlus.Init(_ptPageMgr, _strUniqueInstanceID, _strValues[i]);
			if (null != _strLabels)
			{
				radioPlus.SetLabel(_strLabels[i]);
			}
			if (null != _ptIcons)
			{
				radioPlus.SetIcon(_ptIcons[i]);
			}
			if (null != _ptURLs)
			{
				radioPlus.SetURL((ASURL)_ptURLs.GetElement(i));
			}
			if (null != _sDescs)
			{
				radioPlus.SetDescription(_sDescs[i]);
			}
			m_radioButtons.Add(radioPlus);
		}
	}

	/**
	 * Returns the HTML for this component.
	 *
	 * @return
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		for (int i = 0; i < m_radioButtons.GetSize(); i++)
		{
			retValue.AddInnerHTMLElement(GetElement(i).GetComponentHTML());
		}		
		return retValue;
	}
	
	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new HTMLRadioButtonGroupPlus();
	}
	
	/**
	 * Store the values of this HTML component.
	 *
	 * @param _htRequestParameters
	 */
	public int StoreUserInput(XPHashtable _htRequestParameters)
	{
		for (int i = 0; i < m_radioButtons.GetSize(); i++)
		{
			HTMLRadioButtonPlus radioButtonPlus = (HTMLRadioButtonPlus)m_radioButtons.GetElement(i);
			String strValue = radioButtonPlus.GetHTMLComponentValue();
			boolean bChecked = false;
			Object oElement = _htRequestParameters.GetElement(radioButtonPlus.GetHTMLComponentName());
			if (oElement != null)
			{
				String[] aStrValues = (String[]) oElement;
				for(int j = 0; j < aStrValues.length; j++)
				{
					if (strValue != null && strValue.equals(aStrValues[j]))
					{
						radioButtonPlus.SetChecked(true);					
					}
				}
			}
		}
		return RepostControl.PAGE_STATUS_VALID;
	}
	
	/**
	 * Return the currently selected value of the radio button group.
	 * 
	 * @return XPArrayList of selected values
	 */	
	public String GetSelectedValue()
	{
		for (int i = 0; i < m_radioButtons.GetSize(); i++)
		{
			HTMLRadioButtonPlus radioButtonPlus = (HTMLRadioButtonPlus)m_radioButtons.GetElement(i);
			if (radioButtonPlus.IsChecked())
			{
				return  radioButtonPlus.GetHTMLComponentValue();				
			}
		}
		return null;	
	}

	/**
	 * Set the currently selected value of the radio button group.
	 * 
	 * @return XPArrayList of selected values
	 */
	public void SetSelectedValue(String strSelectedValue)
	{
		for (int i = 0; i < m_radioButtons.GetSize(); i++)
		{
			HTMLRadioButtonPlus radioButtonPlus = (HTMLRadioButtonPlus)m_radioButtons.GetElement(i);
			if (strSelectedValue.equals(radioButtonPlus.GetHTMLComponentValue()))
			{
				radioButtonPlus.SetChecked(true);
			}
		}
	}
	
	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
	
	/**
	 * Return the number of RadioButtons in this group
	 * 
	 * @return number of RadioButtons
	 */
	public int GetSize()
	{
		return m_radioButtons.GetSize();
	}
	
	/**
	 * Return the RadioButtonPlus object at the given index.
	 * 
	 * @param _iIndex the index of the returned button
	 * @return the RadioButtonPlus object at the given index
	 */
	public HTMLRadioButtonPlus GetElement(int _iIndex)
	{
		return (HTMLRadioButtonPlus)m_radioButtons.GetElement(_iIndex);		
	}
	
	/**
	 * Returns the base name of this HTML Component.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName()
	{
		return STR_HTML_INPUT_BASE_NAME;
	}
}
