package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlelements.*;


/**
 * JF- Page Control for HTML Select Drop Down.
 *
 * @author Julie Fournier
 */
public class HTMLSelectDropDown extends AHTMLComponent implements IPageControl
{
    private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
                                                             "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLSelectDropDown");

    /** JF- Index constant for array of options: option value */
    public static final int OPTION_VALUE = 0;

    /** JF- Index constant for array of options: text to display on option */
    public static final int OPTION_TEXT = 1;
    public static final String STR_MVC_CLASS_NAME = "HTMLSelectDropDown";

    /** JF- Array of option value/text to populate the drop down menu. */
    private String[][] m_arrDropDownValueText;
    
    /** array of HTMLOptions to populate the drop down menu */ 
    private XPArrayList m_alOptions;
    
    /** the title describing the htmlselectdropdown menu */
    private String m_strTitle;

    /** the HTMLSelect that this object wraps */
    HTMLSelect htmlSelect;
    
    public HTMLSelectDropDown()
    {
    }

    /**
     * JF- Creates a new HTMLSelectDropDown object.
     * 
	 * @param _ptPageMgr	PageManager
	 * @param _strUniqueInstanceID	Unique Instance ID
     * @param _arrDropDownValueText array of names and values for select dropdown
     * @param _strTitle title of dropdown
     */
    public void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
                     String[][] _arrDropDownValueText,
					 String _strTitle)
    {
        super.Init(_ptPageMgr, _strUniqueInstanceID);
        if (null == _arrDropDownValueText)
        {
        	log.Error("_arrDropDownValueText cannot be null");
        }
        m_arrDropDownValueText = _arrDropDownValueText;
        m_strTitle = _strTitle;
    }
    
    /**
     * Creates a new HTMLSelectDropDown object.
     * 
	 * @param _ptPageMgr	PageManager
	 * @param _strUniqueInstanceID	Unique Instance ID
     * @param _alOptions XPArrayList of options for the select to use
     * @param _strTitle title of dropdown
     */
    public void Init(PageMgr _ptPageMgr, String _strUniqueInstanceID,
                     XPArrayList _alOptions, 
					 String _strTitle)
    {
        super.Init(_ptPageMgr, _strUniqueInstanceID);
        if (null == _alOptions)
        {
        	log.Error("_alOptions cannot be null");
        }
        m_alOptions = _alOptions;
        m_strTitle = _strTitle;
    }

    /**
     * JF- Returns the base name of this HTML OpenComponent.
     *
     * @return String
     */
    protected String GetHTMLComponentBaseName()
    {
        return "in_se_pc";
    }

    /**
     * JF- Returns the HTML for this component. The names and values of the options
     * in the select dropdown are either specified by the array argument in the constructor
     * or by the member variable m_alOptions which may be set with the SetOptions method, or
     * initailized in the alternate constructor.
     *
     * @return HTMLElementCollection
     */
    public HTMLElementCollection GetComponentHTML()
    {
        XPArrayList myOptions = new XPArrayList();
    	HTMLElementCollection retValue = new HTMLElementCollection();
        HTMLSelect htmlSelect = new HTMLSelect(GetHTMLComponentName(), "");
        htmlSelect.SetSize(GetHTMLComponentSize());
        htmlSelect.SetTitle(m_strTitle);

        if (null != GetHTMLComponentStyleClass())
        {
            htmlSelect.SetStyleClass(GetHTMLComponentStyleClass());
        }

        HTMLOption myOption;
        
        // either populate the select with the contents of the String[][] 
        // m_arrDropDownValueText or the XPArrayList m_alOptions
        if (null != m_alOptions)
        {
        	for(int i = 0; i < m_alOptions.GetSize(); i++)
        	{
        		myOption = (HTMLOption)m_alOptions.GetElement(i);
        		htmlSelect.AddInnerHTMLElement(myOption);
        		myOptions.Add(myOption);
        	}
        }
        else
        {
        	for (int i = 0; i < m_arrDropDownValueText.length; i++)
	        {
	            myOption = new HTMLOption(m_arrDropDownValueText[i][OPTION_VALUE]);
	            myOption.AddInnerHTMLString(m_arrDropDownValueText[i][OPTION_TEXT]);
	            htmlSelect.AddInnerHTMLElement(myOption);
        		myOptions.Add(myOption);
	        }
        }

        // if the current value is one of the options then set it
    	String myCurrentValue = GetHTMLComponentValue();
    	for (int i = 0; i < myOptions.GetSize(); i++)
    	{
    		myOption = (HTMLOption)myOptions.GetElement(i);
    		if (XPStringUtility.Compare(myOption.GetValue(),GetHTMLComponentValue()) == 0)
    		{
    			htmlSelect.SetValue(GetHTMLComponentValue());
    		}	
    	}
    	
        htmlSelect.SetDoSort(true);
        retValue.AddInnerHTMLElement(htmlSelect);
        return retValue;
    }

    /**
     * Return the name of the MVC object.
     *
     * @return the name of the object.
     */
    public String GetName()
    {
        return STR_MVC_CLASS_NAME;
    }

    /**
     * This method is used by the ASManager to return new instances of managed
     * objects.
     *
     * @return A new instance of the managed class (i.e. return new Foo();)
     */
    public Object Create()
    {
        return new HTMLSelectDropDown();
    }

	/**
	 * @param options The m_alOptions to set.
	 */
	public void SetOptions(XPArrayList _alOptions) {
		m_alOptions = _alOptions;
	}
}
