package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * HTMLTextAndDropDown page control handles display and values of a page control
 * made up of a text box and a select dropdown.
 * 
 * @author CaiusV
 */
public class HTMLTextAndDropDown extends AHTMLComponent implements IPageControl
{

	/** for logging */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
				"com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLTextAndDropDown");
				
	/** base name constant */
	private static final String STR_HTML_INPUT_BASE_NAME = "in_tx_pc";
	
	/** class name constant */
	public static final String STR_MVC_CLASS_NAME = "HTMLTextAndDropDown";

	/** the textbox for this object */
	public HTMLTextBox m_TextBox;
	
	/** the selectdropdown for this object */
	public HTMLSelectDropDown m_SelectDropDown;
	
	/**
	 * Default constructor. Creates a new HTMLTextAndDropDown object.
	 */
	public HTMLTextAndDropDown()
	{	
	}

	/**
	 * Initializes a new HTMLTextAndDropDown object. Creates and initializes
	 * the HTMLTextBox and HTMLSelectDropdown objects contained by this object.
	 * 
	 * @param _ptPageMgr The component manager for this component
	 * @param _strUniqueInstanceID The unique ID for this component.
	 * @param _arrDropDownValueText The dropdown values
	 */
	public void Init(PageMgr _ptPageMgr, 
			String _strUniqueInstanceID,
			String[][] _arrDropDownValueText)
	{
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		m_TextBox = (HTMLTextBox) _ptPageMgr.AddPageControl(HTMLTextBox.STR_MVC_CLASS_NAME);
		m_TextBox.Init(_ptPageMgr, _strUniqueInstanceID);
		m_SelectDropDown = (HTMLSelectDropDown) _ptPageMgr.AddPageControl(HTMLSelectDropDown.STR_MVC_CLASS_NAME);
		m_SelectDropDown.Init(_ptPageMgr, _strUniqueInstanceID, _arrDropDownValueText, "foo");
	}
	
	/**
	 * Returns the base name of this HTML Component.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName()
	{
		return STR_HTML_INPUT_BASE_NAME;
	}


	/**
	 * Returns the HTML for this component.
	 *
	 * @return HTML for this object
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		retValue.AddInnerHTMLElement(m_TextBox.GetComponentHTML());
		retValue.AddInnerHTMLElement(m_SelectDropDown.GetComponentHTML());
		if (null != GetHTMLComponentStyleClass())
		{
			m_TextBox.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
			m_SelectDropDown.SetHTMLComponentStyleClass(GetHTMLComponentStyleClass());
		}
		
		return retValue;
	}

	/**
	 * Return the component values of the TextBox and SelectDropdown
	 * separated by a space.
	 */
	public String GetHTMLComponentValue()
	{
		return m_TextBox.GetHTMLComponentValue() + " " +
			m_SelectDropDown.GetHTMLComponentValue();
	}
	
	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create()
	{
		return new HTMLTextAndDropDown();
	}
	
	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}
}