package com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols;

import com.plumtree.openfoundation.util.XPConvert;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author CaiusV
 */
public class HTMLTextArea extends AHTMLComponent implements IPageControl
{
	/** for logging */
	private static OpenLogger log = OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
															 "com.plumtree.uiinfrastructure.pagecontrols.htmlpagecontrols.HTMLTextArea");
	/** base name constant */
	private static final String STR_HTML_INPUT_BASE_NAME = "in_ta_pc";
	
	/** class name constant */
	public static final String STR_MVC_CLASS_NAME = "HTMLTextArea";

	/** HTML TextArea Cols */
	private int m_iHTMLComponentCols;
	
	/** HTML TextArea Rows */
	private int m_iHTMLComponentRows;

	/**
	 * Default constructor. Creates a new HTMLTextArea object.
	 */
	public HTMLTextArea()
	{	
	}

	/**
	 * Initializes a ATextArea object.
	 * 
	 * @param _ptPageMgr	PageManager
	 * @param _strUniqueInstanceID	Unique Instance ID
	 * @param _iCols	Text Area columns
	 * @param _iRows	Text Area rows
	 */
	public void Init(PageMgr _ptPageMgr, 
						String _strUniqueInstanceID, 
						int _iCols, 
						int _iRows)
	{
		super.Init(_ptPageMgr, _strUniqueInstanceID);
		SetComponentCols(_iCols);
		SetComponentRows(_iRows);
	}
	
	/**
	 * Returns the base name of this HTML Component.
	 *
	 * @return String
	 */
	protected String GetHTMLComponentBaseName()
	{
		return STR_HTML_INPUT_BASE_NAME;
	}

	/**
	 * Returns the HTML for this component.
	 *
	 * @return
	 */
	public HTMLElementCollection GetComponentHTML()
	{
		HTMLElementCollection retValue = new HTMLElementCollection();
		com.plumtree.xpshared.htmlelements.HTMLTextArea htmlTextArea = 
				new com.plumtree.xpshared.htmlelements.HTMLTextArea(
						XPConvert.ToString(GetComponentCols()),
						XPConvert.ToString(GetComponentRows()));
		htmlTextArea.SetName(GetHTMLComponentName());
		htmlTextArea.AddInnerHTMLString(GetHTMLComponentValue());
		retValue.AddInnerHTMLElement(htmlTextArea);
		
		if (null != GetHTMLComponentStyleClass())
		{
			htmlTextArea.SetStyleClass(GetHTMLComponentStyleClass());
		}
		
		return retValue;
	}
	
	/**
	 * This method is used by the ASManager to return new instances of managed
	 * objects.
	 *
	 * @return A new instance of the managed class (i.e. return new Foo();)
	 */
	public Object Create(){
		return new HTMLTextArea();
	}
	
	/**
	 * Return the name of the MVC object.
	 *
	 * @return the name of the object.
	 */
	public String GetName(){
		return STR_MVC_CLASS_NAME;
	}
	
	/**
	 * Return m_sHTMLComponentCols, coloumn size for the text area.
	 * @return text area columns.
	 */
	public int GetComponentCols() {
		return m_iHTMLComponentCols;
	}

	/**
	 * Return m_sHTMLComponentRows, row size for the text area.
	 * 
	 * @return text area rows.
	 */
	public int GetComponentRows() {
		return m_iHTMLComponentRows;
	}

	/**
	 * Set the column size (m_iHTMLComponentCols) for textarea.
	 * 
	 * @param _cols column size
	 */
	public void SetComponentCols(int _cols) {
		m_iHTMLComponentCols = _cols;
	}

	/**
	 * Set the rpw size (m_iHTMLComponentRows) for textarea.
	 * 
	 * @param _ros column size
	 */
	public void SetComponentRows(int _rows) {
		m_iHTMLComponentRows = _rows;
	}
}
