package com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.*;

import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.xpshared.jsutil.*;

/**
 * This is the component manager class for the JS date-picker. Note: there is
 * also JS date/time-picker if you need date and time input.
 *
 * @author JohnO
 */
public class JSDatePickerMgr extends JSComponentMgr
{
	/** The name for JS controls of this type. */
	public static final String STR_JS_COMPONENT_NAME = "jsdatepicker";

	/** The ID for the HTML div tag that the datepicker uses. */
	private static final String STR_DP_DIV_ID = "dpDiv";
	
	/** Unique ID for this datepick manager. **/
	private String m_strJSDatePickerMgrID = "";
	/**
	 * Creates a new JSDatePickerMgr object.
	 *
	 * @param _ptJSRegistry The JS Registry is needed to get the JS includes
	 * @param _ptJSCompVerMgr The component version manager interface allows us
	 * 		  to the version of this control that is registered with the
	 * 		  product as a whole
	 * @param _ptPageMgr The locale that should be used by the datepickers
	 * 		  under the control of this manager
	 */
	public JSDatePickerMgr(JSRegistry _ptJSRegistry, IJSComponentVersionMgr _ptJSCompVerMgr, PageMgr _ptPageMgr)
	{
		super(STR_JS_COMPONENT_NAME, _ptJSRegistry, _ptJSCompVerMgr, _ptPageMgr);
	}

	/**
	 * This method returns the HTML that needs to be written-out before the JS
	 * datepicker form is written out. Specifically, this is HTML that defines
	 * a div. Note that this HTML is of page scope, meaning that it only needs
	 * to be written out one time even if there are multiple date-pickers on
	 * the page.
	 *
	 * @return The pre-form HTML for this
	 */
	public HTMLElementCollection GetPagePreFormContent()
	{
		HTMLElementCollection result = new HTMLElementCollection();

		HTMLDiv myDiv = new HTMLDiv();
		myDiv.SetID(STR_DP_DIV_ID);
		myDiv.SetStyleClass(PTStyleClass.CAL_DIV);
		result.AddInnerHTMLElement(myDiv);
	
		return result;
	}
	
	/**
	 * This method returns the HTML that needs to be written-out before the JS
	 * datepicker form is written out. Specifically, this is HTML that defines
	 * a div. Note that this HTML is of page scope, meaning that it only needs
	 * to be written out one time even if there are multiple date-pickers on
	 * the page.
	 *
	 *@param strDatePickerMgrID Unique ID for datepickers managed by this object.
	 *Use this method when there might be several datepick managers on a page, e.g. MyPage.
	 * @return The pre-form HTML for this
	 */
	public HTMLElementCollection GetPagePreFormContent(String strDatePickerMgrID)
	{
		HTMLElementCollection result = new HTMLElementCollection();

		HTMLDiv myDiv = new HTMLDiv();
		myDiv.SetID(STR_DP_DIV_ID+strDatePickerMgrID);
		myDiv.SetStyleClass(PTStyleClass.CAL_DIV);
		result.AddInnerHTMLElement(myDiv);
	
		m_strJSDatePickerMgrID = strDatePickerMgrID;
		
		return result;
	}
	
	
	 /**
     * This method allows you to register a component with the component
     * manager. The order in which you register components is the order in
     * which calls will be made when doing any looping operation--storing or
     * printing. If you attempt to re-register the same component, it will
     * overwrite the one that existed there before.
     * 
     *
     * @param _ptComponent The component that you wish to register.
     */
    public void RegisterComponent(IPageControl _ptComponent)
    {
        super.RegisterComponent(_ptComponent);
		if ( !"".equals(m_strJSDatePickerMgrID) &&  _ptComponent instanceof JSDatePicker  )
		{
			((JSDatePicker)_ptComponent).SetDatePickerMgrID(m_strJSDatePickerMgrID);
		}
		
    }
	
	/**
	 * Get the ID of the preform DIV element used by datepickers managed by this object.
	 * 
	 * @return
	 */
	public String GetPreFormDivID()
	{
		return STR_DP_DIV_ID + m_strJSDatePickerMgrID;
		
	}
	
	
}

