package com.plumtree.uiinfrastructure.pagecontrols.timepicker;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;

/**
 * @author DorisL
 */
public abstract class ATimePicker extends AComponent implements IPageControl {
	
	/**
	 * Creates a new TimePicker object based on a complete time string in
	 * XPDateFormat.MEDIUM format
	 *
	 * @param _strTime - The time string to parse
	 * @param _ptComponentMgr - The component manager
	 * @param _strUniqueInstanceID - The unique identifying string to
	 * 		  distinguish multiple time pickers on the page
	 */
	public abstract void Init(
		PageMgr _ptPageMgr,
		String _strTime,
		String _strUniqueInstanceID);

	/**
	 * Creates a new TimePicker object.
	 *
	 * @param _nHours The hours value to initially select
	 * @param _nMinutes The minutes value to initially select
	 * @param _strAMPM The AM/PM string to initially select
	 * @param _ptComponentMgr The component manager
	 * @param _strUniqueInstanceID The unique identifier string used to
	 * 		  distinguish multiple copies of this component on the same page
	 */
	public abstract void Init(
		int _nHours,
		int _nMinutes,
		String _strAMPM,
		PageMgr _ptPageMgr,
		String _strUniqueInstanceID);

	/**
	 * Creates a new TimePicker object.
	 *
	 * @param _nHours The hours value to initially select
	 * @param _nMinutes The minutes value to initially select
	 * @param _ptComponentMgr The component manager
	 * @param _strUniqueInstanceID The unique identifier string used to
	 * 		  distinguish multiple copies of this component on the same page
	 */
	public abstract void Init(
		int _nHours,
		int _nMinutes,
		PageMgr _ptPageMgr,
		String _strUniqueInstanceID);

	/**
	 * Returns the AMPM string
	 *
	 * @return String
	 */
	public abstract String GetAMPM();

	/**
	 * Provides a means of retrieving the HTML Input name for the AMPM portion
	 * of the date picker.  This allows for using javascript to
	 * programmatically set input values, for example.
	 *
	 * @return String
	 */
	public abstract String GetAMPMInputName();

	/**
	 * Returns the hours string
	 *
	 * @return String
	 */
	public abstract String GetHours();

	/**
	 * Provides a means of retrieving the HTML Input name for the hours portion
	 * of the date picker.  This allows for using javascript to
	 * programmatically set input values, for example.
	 *
	 * @return String
	 */
	public abstract String GetHoursInputName();

	/**
	 * Returns the minutes string
	 *
	 * @return String
	 */
	public abstract String GetMinutes();

	/**
	 * Provides a means of retrieving the HTML Input name for the minutes
	 * portion of the date picker.  This allows for using javascript to
	 * programmatically set input values, for example.
	 *
	 * @return String
	 */
	public abstract String GetMinutesInputName();

	/**
	 * Returns the time portion of the date and time chosen by the user as a
	 * String.
	 *
	 * @return String
	 */
	public abstract String GetTimeAsString();

	/**
	 * Returns the time chosen by the user as a
	 * String in the English format: "h:mm:ss a".
	 *
	 * @return String
	 */
	public abstract String GetTimeAsEnglishString();
}
