package com.plumtree.uiinfrastructure.pagecontrols.timepicker;

import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatepicker.*;
import com.plumtree.uiinfrastructure.pagecontrols.jsdatetimepicker.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.xpshared.jsutil.*;


/**
 *
 * @author JohnO
 */
public class TimePicker extends ATimePicker implements IPageControl
{
	/** some helpful static Strings */
	public static final String AM 	 = "AM";
	public static final String PM 	 = "PM";
	public static final String H 	 = "H";
    public static final String EMPTY = "";
    public static final String DOUBLE0 	= "00";
    public static final String TWELVE= "12";
    
	private static OpenLogger log = 
    	OpenLogService.GetLogger(OpenLogService.GetComponent("UI_Infrastructure"),
        	"com.plumtree.uiinfrastructure.pagecontrols.timepicker.TimePicker");

    /**
     * The constant part of the name for the hours select list that will be
     * returned by the time-picker. The unique identifier string will be
     * appended to this name on construction of the object.
     */
    public static final String HTMLSELECT_HOURS = "in_se_TPHours";

    /**
     * The constant part of the name for the minutes select list that will  be
     * returned by the time picker. The unique identifier string will be
     * appended to this name on construction of the object.
     */
    public static final String HTMLSELECT_MINUTES = "in_se_TPMinutes";

    /**
     * The constant part of the name for the AM/PM select list that will
     * optionally be returned by the date picker. The unique identifier string
     * will be appended to this name on construction of the object.
     */
    public static final String HTMLSELECT_AMPM = "in_se_TPAMPM";
    public static final String STR_MVC_CLASS_NAME = "TimePicker";

    /** The hour AM/PM variable. */
    private String m_strAMPM = "";
    
	private String m_strLangID;
	
	/** static EN-us locale object used for formatting dates exchenaged with client side */
	private static XPLocale m_oUSLocale = new XPLocale("EN","us");

    /**
     * The final name for the optional AM/PM select list - set on construction
     */
    private String m_strAMPMSelectName;

    /** The hour member variable. */
    private String m_strHours = EMPTY;

    /** The final name for the hours select list - set on construction */
    private String m_strHoursSelectName;

    /** The final name for the minutes select list - set on construction */
    private String m_strMinutes = EMPTY;

    /** The name for the minutes select list. */
    private String m_strMinutesSelectName;

    /** Is this 24-hour mode or 12-hour mode? */
    private boolean m_b24hour;

    /** The delimiter used to separate hours, minutes, seconds - ":" or "." */
    private String m_strDelimiter;
    private boolean m_bIsAMPMBeforeTime;
    private boolean m_bIsAMPMBeforeTimeWithSpace;

    public TimePicker()
    {
    }

    /**
     * Creates a new TimePicker object based on a complete time string in
     * XPDateFormat.MEDIUM format using the given time as the preset value.
     *
     * @param _ptPageMgr - The page manager
     * @param _strTime - The time string to parse
     * @param _strUniqueInstanceID - The unique identifying string to
     *                   distinguish multiple time pickers on the page
     */
    public void Init(PageMgr _ptPageMgr,
    		String _strTime,
            String _strUniqueInstanceID)
    {
        super.Init(_ptPageMgr, _strUniqueInstanceID);
        
		m_strLangID = _ptPageMgr.GetLangID();

        XPDateTime xpdtTime = null;

        //Get a datetime format object
        XPDateFormat xpdfTimeFormat = XPDateFormat.GetTimeInstance(XPDateFormat.MEDIUM,
                                                                   _ptPageMgr.GetLocale(),
                                                                   _ptPageMgr.GetTimeZone());

        //Get a calendar
        XPCalendar xpCal = XPCalendar.GetInstance(_ptPageMgr.GetTimeZone(),
                                                  _ptPageMgr.GetLocale());

        //If the MEDIUM format string for this locale contains "H", it is a 24 hour clock
        m_b24hour = false;

        String strDateTimePattern = xpdfTimeFormat.ToPattern();

        if (strDateTimePattern.indexOf(H) != -1)
        {
            m_b24hour = true;
        }

        //Try to parse the time string that the client has passed in
        try
        {
            xpdtTime = xpdfTimeFormat.Parse(_strTime);

            //If we get this far, the parse worked and we can use the calendar object to extract the hours, minutes, etc.
            xpCal.SetTime(xpdtTime);

            m_strMinutes = XPConvert.ToString(xpCal.Get(XPCalendar.MINUTE));

            if (!m_b24hour)
            {
                m_strHours = XPConvert.ToString(xpCal.Get(XPCalendar.HOUR));

                String[] arrAMPM = xpdfTimeFormat.GetAmPmStrings();
                m_strAMPM = arrAMPM[xpCal.Get(XPCalendar.AM_PM)];
            }
            else
            {
                m_strHours = XPConvert.ToString(xpCal.Get(XPCalendar.HOUR_OF_DAY));
                m_strAMPM = null;
            }
        }
        catch (Exception e)
        {
            //Couldn't parse the time string - just set defaults
            if (m_b24hour)
            {
                m_strHours = DOUBLE0;
                m_strAMPM = null;
            }
            else
            {
                m_strHours = TWELVE;
                m_strAMPM = xpdfTimeFormat.GetAmPmStrings()[0];
            }

            m_strMinutes = DOUBLE0;
        }

        m_strHoursSelectName 	= HTMLSELECT_HOURS + _strUniqueInstanceID;
        m_strMinutesSelectName 	= HTMLSELECT_MINUTES + _strUniqueInstanceID;
        m_strAMPMSelectName 	= HTMLSELECT_AMPM + _strUniqueInstanceID;
        m_strDelimiter 			= GetDelimiter(xpdfTimeFormat.ToPattern());

        char cPatternFirstChar = (xpdfTimeFormat.ToPattern()).charAt(0);

        if ((cPatternFirstChar == 'a') || (cPatternFirstChar == 't')) // AM/PM is represented by "a" in Java and "tt" in .NET
        {
            m_bIsAMPMBeforeTime = true;
            m_bIsAMPMBeforeTimeWithSpace = (xpdfTimeFormat.ToPattern()).charAt(0) == ' ';
        }
    }

    /**
     * Creates a new TimePicker object.
     *
     * @param _nHours The hours value to initially select
     * @param _nMinutes The minutes value to initially select
     * @param _strAMPM The AM/PM string to initially select
     * @param _ptComponentMgr The component manager
     * @param _strUniqueInstanceID The unique identifier string used to
     *                   distinguish multiple copies of this component on the same page
     */
    public void Init(int _nHours,
    		int _nMinutes,
			String _strAMPM,
            PageMgr _ptPageMgr,
			String _strUniqueInstanceID)
    {
        super.Init(_ptPageMgr, _strUniqueInstanceID);
        
		m_strLangID = _ptPageMgr.GetLangID();

        m_b24hour = false;

        if ((_nHours > 12) || (_nHours < 1))
        {
            m_strHours = TWELVE;
        }
        else
        {
            m_strHours = XPConvert.ToString(_nHours);
        }

        if ((_nMinutes > 59) || (_nMinutes < 0))
        {
            m_strMinutes = DOUBLE0;
        }
        else
        {
            m_strMinutes = XPConvert.ToString(_nMinutes);
        }

        m_strAMPM = _strAMPM;
        m_strHoursSelectName 	= HTMLSELECT_HOURS + _strUniqueInstanceID;
        m_strMinutesSelectName 	= HTMLSELECT_MINUTES + _strUniqueInstanceID;
        m_strAMPMSelectName 	= HTMLSELECT_AMPM + _strUniqueInstanceID;

        //		Get a datetime format object
        XPDateFormat xpdfTimeFormat = XPDateFormat.GetTimeInstance(XPDateFormat.MEDIUM,
                                                                   _ptPageMgr.GetLocale(),
                                                                   _ptPageMgr.GetTimeZone());

        m_strDelimiter = GetDelimiter(xpdfTimeFormat.ToPattern());

        char cPatternFirstChar = (xpdfTimeFormat.ToPattern()).charAt(0);

        if ((cPatternFirstChar == 'a') || (cPatternFirstChar == 't')) // AM/PM is represented by "a" in Java and "tt" in .NET
        {
            m_bIsAMPMBeforeTime = true;
            m_bIsAMPMBeforeTimeWithSpace = (xpdfTimeFormat.ToPattern()).charAt(0) == ' ';
        }
    }

    /**
     * Initialize a new TimePicker object.
     *
     * @param _nHours The hours value to initially select
     * @param _nMinutes The minutes value to initially select
     * @param _ptPageMgr The page manager
     * @param _strUniqueInstanceID The unique identifier string used to
     *                   distinguish multiple copies of this component on the same page
     */
    public void Init(int _nHours, 
    		int _nMinutes, 
			PageMgr _ptPageMgr,
            String _strUniqueInstanceID)
    {
        super.Init(_ptPageMgr, _strUniqueInstanceID);
        
		m_strLangID = _ptPageMgr.GetLangID();

        m_b24hour = true;

        if ((_nHours > 24) || (_nHours < 0))
        {
            m_strHours = DOUBLE0;
        }
        else
        {
            m_strHours = XPConvert.ToString(_nHours);
        }

        if ((_nMinutes > 59) || (_nMinutes < 0))
        {
            m_strMinutes = DOUBLE0;
        }
        else
        {
            m_strMinutes = XPConvert.ToString(_nMinutes);
        }

        m_strAMPM = null;
        m_strHoursSelectName 	= HTMLSELECT_HOURS + _strUniqueInstanceID;
        m_strMinutesSelectName 	= HTMLSELECT_MINUTES + _strUniqueInstanceID;
        m_strAMPMSelectName 	= HTMLSELECT_AMPM + _strUniqueInstanceID;

        //		Get a datetime format object
        XPDateFormat xpdfTimeFormat = XPDateFormat.GetTimeInstance(XPDateFormat.MEDIUM,
                                                                   _ptPageMgr.GetLocale(),
                                                                   _ptPageMgr.GetTimeZone());

        m_strDelimiter = GetDelimiter(xpdfTimeFormat.ToPattern());

        char cPatternFirstChar = (xpdfTimeFormat.ToPattern()).charAt(0);

        if ((cPatternFirstChar == 'a') || (cPatternFirstChar == 't')) // AM/PM is represented by "a" in Java and "tt" in .NET
        {
            m_bIsAMPMBeforeTime = true;
            m_bIsAMPMBeforeTimeWithSpace = (xpdfTimeFormat.ToPattern()).charAt(0) == ' ';
        }
    }

    /**
     * Returns the AMPM string
     *
     * @return String
     */
    public String GetAMPM()
    {
        return m_strAMPM;
    }

    /**
     * Provides a means of retrieving the HTML Input name for the AMPM portion
     * of the date picker.  This allows for using javascript to
     * programmatically set input values, for example.
     *
     * @return String
     */
    public String GetAMPMInputName()
    {
        return m_strAMPMSelectName;
    }

    /**
     * Returns the time picker HTML, which consists of either 2 or three select
     * lists, depending on whether AMPM notation is used in the locale
     *
     * @return HTMLElementCollection
     */
    public HTMLElementCollection GetComponentHTML()
    {
        HTMLElementCollection retValue = new HTMLElementCollection();
        HTMLSelect mySelect;
        HTMLOption myOption;
        XPDateFormat xpdfTimeFormat;
        String strHours;
        
		IApplication application = ApplicationManager.GetInstance().
				GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
		XPResourceManager rm = (XPResourceManager) 
				application.GetCachingManager().GetEntry("ResourceManager");

        // Get the time drop downs
        mySelect = new HTMLSelect(m_strHoursSelectName, m_strHoursSelectName);
        mySelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
        mySelect.SetTitle(rm.GetString(154, "ptmsgs_infrastructure", m_strLangID));

        if (m_b24hour)
        {
            String strHoursToSelect = m_strHours;

            if (1 == m_strHours.length())
            {
                strHoursToSelect = XPConvert.ToString(0) + m_strHours;
            }

            // For 24 hour style...
            for (int i = 0; i < 24; i++)
            {
                // Prepend a "0" to complete a time. (i.e. 03:30 as opposed to 3:30 for a 24-hour clock
                strHours = XPConvert.ToString(i);

                if (i < 10)
                {
                    strHours = XPConvert.ToString(0) + strHours;
                }

                myOption = new HTMLOption(strHours);
                myOption.SetValue(strHours);
                myOption.AddInnerHTMLString(strHours);
                mySelect.AddInnerHTMLElement(myOption);

                if (strHoursToSelect.equals(strHours))
                {
                    myOption.SetSelected(true);
                }
            }
        }
        else
        {
            // For 12-hour time...
            for (int i = 1; i < 13; i++)
            {
                strHours = XPConvert.ToString(i);
                myOption = new HTMLOption(strHours);
                myOption.SetValue(strHours);
                myOption.AddInnerHTMLString(strHours);
                mySelect.AddInnerHTMLElement(myOption);

                if (m_strHours.equals(strHours))
                {
                    myOption.SetSelected(true);
                }
            }
        }

        // Now add the hours select to the result
        retValue.AddInnerHTMLElement(mySelect);

        retValue.AddInnerHTMLString(CommonHTMLStrings.SPACE);
        retValue.AddInnerHTMLString(m_strDelimiter);
        retValue.AddInnerHTMLString(CommonHTMLStrings.SPACE);

        // Now for the minutes drop-down
        mySelect = new HTMLSelect(m_strMinutesSelectName, m_strMinutesSelectName);
        mySelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
        mySelect.SetTitle(rm.GetString(155, "ptmsgs_infrastructure", m_strLangID));

        for (int i = 0; i < 60; i++)
        {
            String strTime = XPConvert.ToString(i);
            myOption = new HTMLOption(strTime);
            if (m_strMinutes.equals(strTime) ||
                    m_strMinutes.equals(XPConvert.ToString(0) + strTime))
            {
                myOption.SetSelected(true);
            }
            // Prepend a "0" for minute values less than 10
            if (i < 10)
            {
                strTime = XPConvert.ToString(0) + strTime;
            }
            myOption.SetValue(strTime);
            myOption.AddInnerHTMLString(strTime);
            mySelect.AddInnerHTMLElement(myOption);
        }

        retValue.AddInnerHTMLElement(mySelect);

        // We only want the AM/PM if it's not a 24 hour clock.
        if (!m_b24hour)
        {
            xpdfTimeFormat = XPDateFormat.GetTimeInstance(XPDateFormat.MEDIUM,
            		super.GetPageMgr().GetLocale(),
                    super.GetPageMgr().GetTimeZone());
            mySelect = new HTMLSelect(m_strAMPMSelectName, m_strAMPMSelectName);
            mySelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
            mySelect.SetTitle(rm.GetString(156, "ptmsgs_infrastructure", m_strLangID));

            String[] arrAMPM = xpdfTimeFormat.GetAmPmStrings();

            for (int i = 0; i < arrAMPM.length; i++)
            {
                myOption = new HTMLOption(arrAMPM[i]);
                myOption.SetValue(arrAMPM[i]);
                myOption.AddInnerHTMLString(arrAMPM[i]);
                mySelect.AddInnerHTMLElement(myOption);

                if (m_strAMPM.equals(arrAMPM[i]))
                {
                    myOption.SetSelected(true);
                }
            }

            retValue.AddInnerHTMLElement(mySelect);
        }

        return retValue;
    }

    /**
     * Returns the hours string
     *
     * @return String
     */
    public String GetHours()
    {
        return m_strHours;
    }

    /**
     * Provides a means of retrieving the HTML Input name for the hours portion
     * of the date picker.  This allows for using javascript to
     * programmatically set input values, for example.
     *
     * @return String
     */
    public String GetHoursInputName()
    {
        return m_strHoursSelectName;
    }

    /**
     * Returns the minutes string
     *
     * @return String
     */
    public String GetMinutes()
    {
        return m_strMinutes;
    }

    /**
     * Provides a means of retrieving the HTML Input name for the minutes
     * portion of the date picker.  This allows for using javascript to
     * programmatically set input values, for example.
     *
     * @return String
     */
    public String GetMinutesInputName()
    {
        return m_strMinutesSelectName;
    }

    /**
     * Returns the time portion of the date and time chosen by the user as a
     * String.
     *
     * @return String
     */
    public String GetTimeAsString()
    {
        return ConstructTime(m_strHours, m_strMinutes, m_strAMPM);
    }

    /**
     * Helper method to construct the time and return it as a String
     *
     * @return String
     */
    private String ConstructTime(String _sHours,
    		String _sMinutes,
			String _sAMPM)
    {
        String sTime = EMPTY;

        // If either the hours or minutes are blank we return a blank string.
        if ((_sHours.equals(EMPTY)) || (_sMinutes.equals(EMPTY)))
        {
            return (sTime);
        }

        // Create a calendar with the current locale and timezone
        XPCalendar cal = XPCalendar.GetInstance(m_ptPageMgr.GetTimeZone(),
                                                m_ptPageMgr.GetLocale());

        // Create a DateFormat object with the current locale and timezone so we can format the time later
        XPDateFormat dfTimeFormat = XPDateFormat.GetTimeInstance(XPDateFormat.MEDIUM,
                												 m_oUSLocale,
                                                                 m_ptPageMgr.GetTimeZone());

        // We set the hour using HOUR_OF_DAY or just HOUR depending if am/pm is used or no
        if ((null == _sAMPM) || (0 == _sAMPM.length()))
        {
            cal.Set(XPCalendar.HOUR_OF_DAY, XPConvert.ToInteger(_sHours));
        }
        else
        {
            cal.Set(XPCalendar.HOUR, XPConvert.ToInteger(_sHours));

            // Get AM/PM Strings because they are different in every language.
            String[] arrAMPM = dfTimeFormat.GetAmPmStrings();

            // Set am/pm
            if (_sAMPM.equalsIgnoreCase(arrAMPM[XPCalendar.AM]))
            {
                cal.Set(XPCalendar.AM_PM, XPCalendar.AM);
            }
            else
            {
                cal.Set(XPCalendar.AM_PM, XPCalendar.PM);
            }
        }

        // Finally set the minutes and the seconds
        cal.Set(XPCalendar.MINUTE, XPConvert.ToInteger(_sMinutes));
        cal.Set(XPCalendar.SECOND, 0);

        // Then get the DateTime object and transform it to a String (in the current language format)
        XPDateTime dTime = cal.GetTime();

        try
        {
            sTime = dfTimeFormat.Format(dTime);
        }
        catch (Exception e)
        {
            log.Error("Unable to parse the time. The following exception occured: " +
                      e);
        }

        return sTime;
    }

    /**
     * Returns the time in the format format: "h:mm:ss a" the default US en time format.
     * Note this is a poorly named method and in fact what's returned is the time formatted
     * to the default format of the server.
     *
     * @return String selected time formatted to the default format of the server.
     */
    public String GetTimeAsEnglishString()
    {
        return TransformTimeToNewLocale(GetTimeAsString(),
                						m_oUSLocale,
                                        XPLocale.GetDefault(),
                                        m_ptPageMgr.GetTimeZone());
    }

    /**
     * Transforms a time from one format to another based on the Locale.
     *
     * @param _sTimeAsString
     * @param _lCurrentLocale
     * @param _lNewLocale
     *
     * @return String
     */
    public static String TransformTimeToNewLocale(String _sTimeAsString,
                                                  XPLocale _lCurrentLocale,
                                                  XPLocale _lNewLocale,
                                                  XPTimeZone _tzCurrentTimeZone)
    {
        String sNewTime = EMPTY;

        // JF- Construct the XPDateFormat in the current locale
        XPDateFormat dfTimeFormat = XPDateFormat.GetTimeInstance(XPDateFormat.MEDIUM,
                                                                 _lCurrentLocale,
                                                                 _tzCurrentTimeZone);

        try
        {
            // JF- Then transform it to something that is not locale specific
            XPDateTime dtTime = dfTimeFormat.Parse(_sTimeAsString);

            //	JF- Construct another XPDateFormat in the new locale
            XPDateFormat dfNewTimeFormat = XPDateFormat.GetTimeInstance(XPDateFormat.MEDIUM,
                                                                        _lNewLocale,
                                                                        _tzCurrentTimeZone);

            // JF- And finally get the time in the new format
            sNewTime = dfNewTimeFormat.Format(dtTime);
        }
        catch (Exception e)
        {
            log.Error("Unable to transform the following time: " +
                      _sTimeAsString + " to this new locale: " + _lNewLocale.toString() +
                      " . The following exception occured: " + e);
        }

        return sNewTime;
    }

    /**
    *  Saves the items selected by the user into member variables
    *
    * @param _htRequestParameters - A hashtable containing the contents of the
    *                   HTML form  the user has submitted
    * @return int - status of the page
    */
    public int StoreUserInput(XPHashtable _htRequestParameters)
    {
        String[] data;

        data = (String[]) _htRequestParameters.GetElement(m_strHoursSelectName);

        if ((null != data) && !(data[0].equals(EMPTY)))
        {
            m_strHours = data[0];
        }

        data = (String[]) _htRequestParameters.GetElement(m_strMinutesSelectName);

        if ((null != data) && !(data[0].equals(EMPTY)))
        {
            m_strMinutes = data[0];
        }

        data = (String[]) _htRequestParameters.GetElement(m_strAMPMSelectName);

        if ((null != data) && !(data[0].equals(EMPTY)))
        {
            m_strAMPM = data[0];
        }

        return RepostControl.PAGE_STATUS_VALID;
    }

    /**
     * Return the locale-appropriate delimiter string. Either a '.' or a ':'.
     *  
     * @param _strPattern the pattern to derive the deilimiter string from.
     * @return the delimiter String, Either a '.' or a ':'
     */
    private String GetDelimiter(String _strPattern)
    {
        for (int i = 0; i < _strPattern.length(); i++)
        {
            if (_strPattern.charAt(i) == '.')
            {
                return (".");
            }
        }

        return (":");
    }

    /**
     * This method is used by the ASManager to return new instances of managed
     * objects.
     *
     * @return A new instance of the managed class (i.e. return new Foo();)
     */
    public Object Create()
    {
        return new TimePicker();
    }

    /**
     * Return the name of the MVC object.
     *
     * @return the name of the object.
     */
    public String GetName()
    {
        return STR_MVC_CLASS_NAME;
    }
    
    /**
     * Return a new ComponentMgr of the type that manages this class (a JSDateTimePickerMgr).
     */
	public ComponentMgr GetComponentMgr(PageMgr _ptPageMgr)
	{
		JSRegistry ptJSRegistry = JSRegistry.GetRegistry(_ptPageMgr.GetImgSvrBaseURL(),
					_ptPageMgr.GetImageServerConnectionURL());
		return new JSDateTimePickerMgr(ptJSRegistry, new JSDatePickerVersionMgr(), _ptPageMgr);
	}
	
	/**
	 * Return the name of the component manager responsible for this page control
	 */
	public String GetComponentMgrName()
	{
		return JSDateTimePickerMgr.STR_CLASS_NAME;
	}
	
	/**
	 * Currently validation on the date/time picker page controls is not implemented.
	 * This method returns null.
	 */
	public XPArrayList Validate()
	{
		return null;
	}
	
	/**
	 * Currently validation on the date/time picker page controls is not implemented.
	 * This method returns null.
	 */
	public XPArrayList GetValidators()
	{
		return null;
	}
}
