package com.plumtree.uiinfrastructure.pei;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;

/**
 * This interface was created to allow developers the ability to create
 * functions that can be dynamically discovered by the Plumtree application
 * for the purposes of performing some functionality when a particular
 * object creation or modification based event occurs.
 * <br><br>
 */
public interface IOpenerActions
{
    /**
     * This function will perform some functionality just before we use the Common
     * Opener to open an object (or direct to an Activity Space).  To override the
     * destination to Redirect to, the function will optionally return a Redirect.
     * 
     * If multiple implementations are processed, they will be processed according
     * to the order that they appearin the XML file.  Processing will stop once an
     * implemention returns a valid Redirect, however. 
     *
     * @param _nClassID - Unchecked Class ID of the object that is being edited.
     *        -1 if the object to be opened does not use class IDs and object IDs
     *        (such as Activity Spaces).
     * @param _nObjectID - Unchecked Object ID of the object that is being edited.
     *        -1 if the object to be opened does not use class IDs and object IDs
     *        (such as Activity Spaces).
     * @param _strClassKey - The class key of this object.  For objects that use
     *        class IDs and object IDs this will generally be the class ID.
     * @param _htQSArguments - The query string arguments from the link to the
     *        Common Opener.
     * @param _rRedirect -  The redirect generated by the Common Opener.  This
     *        will be used unless a new redirect is returned by a PEI.
     * @param _asCurrentSpace - Provides access to the current MVC classes as
     *        well as the Application and the HTTP Session
     * @param _userSession - Plumtree session object for the current user
     * 
     * @return Redirect - Optional location that the User will be redirected after 
     *         the functionality has completed.  Leave this null otherwise.
     */
    public Redirect OnBeforeOpen(int _nClassID, int _nObjectID,
    	String _strClassKey, XPHashtable _htQSArguments, Redirect _rRedirect,
    	AActivitySpace _asCurrentSpace, Object _userSession);
}