package com.plumtree.uiinfrastructure.statichelpers;

import com.plumtree.jsincluder.*;
import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.activityspace.utils.*;
import com.plumtree.uiinfrastructure.application.*;
import com.plumtree.uiinfrastructure.application.notification.*;
import com.plumtree.uiinfrastructure.application.startup.exceptions.*;
import com.plumtree.uiinfrastructure.application.varpacks.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.opener.types.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.xpshared.jsutil.*;
import com.plumtree.uiinfrastructure.login.*;

/**
 * This class contains helper methods to get variables from the configVarPack.
 *
 * This is also used to get data that is generated from the config var pack, along
 * with some logic from other places, like the image server urls.  These should only
 * be retrieved from this static helper.
 *
 * NOTE DonH: if the base config varpack reloads, this will not currently get reloaded.
 * Reloading wouldn't be threadsafe anyways.
 *
 * @author Don Hayler
 * @author Julie Fournier - Application URLs, URL Mapping and Security Modes
 */
public class ConfigHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.statichelpers.ConfigHelper");

	/** JF- HTTP Scheme */
	public static final String HTTP_SCHEME = "http://";

	/** JF- HTTPS Scheme */
	public static final String HTTPS_SCHEME = "https://";

	/** private variables */
	private static String s_strApplicationName = "";
	private static boolean s_bInitialized = false;

	private static boolean s_bIsJava = false;
	private static boolean s_bIsDotNet = false;

	/** JF- Constant for the position if the Application URL in the array. */
	private static final int URL_MAPPING_APPLICATIONURL = 0;

	/** JF- Constant for the position if the Secure Application URL in the array. */
	private static final int URL_MAPPING_SECUREAPPLICATIONURL = 1;

	/** JF- Value of the default entry in the mapping. */
	private static final String URL_MAPPING_DEFAULT_ENTRY = "*";

	/** JF- This is to be able to identify a gatway URL */
	public static final String GATEWAY_URL = "/gateway/PTARGS_";

	/**
	 * JF- Hashtable of URL Mapping
	 * Key: URL from Request
	 * Value: 1D String array with 2 values: Application URL and Secure Application URL
	 */
	private static XPHashtable m_htURLMapping;

	/** JF- Security Mode */
	private static int m_nSecurityMode;

	/** JF- Main server application URL. */
	private static String m_sMainServerApplicationURL = "";

	/** JF- Main server secure application URL. */
	private static String m_sMainServerSecureApplicationURL = "";

	/** JF- HTTP Entry point */
	private static String m_sHTTPEntryPoint = "";

	/** JYS- HTTP Entry point */
	private static String m_sSSOEntryPoint = "";

	/** Performance Metrics HTMLComments enabled by default */
	private static boolean m_bArePerformanceCommentsEnabled = true;
	private static boolean m_bArePerformanceStacktracesEnabled = true;
	
	private static int m_nImageserverConnectTimeout = 30;

	/////////////
	// Non-secure
	/////////////

	/**
	 * Non-secure Image Server Base URL
	 * This URL is just the beginning part (i.e. "http://myserver:8080/ptimages/").
	 * It will have a trailing slash if the ImgSvrURLHelper is set for trailing slashes.
	 * This is set by default.
	 * 
	 * rjz:  ImageServerBaseURL has the domain name ('plumtree') in the URL, created ImageServerRootURL to return the
	 * "true" root ImageServerURL.

	 */
	private static String m_sNonSecureImageServerRootURL;
	
	private static String m_sNonSecureImageServerBaseURL;

	/** Portal Image Server Base URL. */
	private static String m_sNonSecurePortalImageServerBaseURL;

	/** Portal Image Server URL to Help files. */
	private static String m_sNonSecurePortalImageServerHelpURL;

	/** Portal Image Server URL to Images. */
	private static String m_sNonSecurePortalImageServerImagesURL;

	/** Portal Image Server URL to JavaScripts. */
	private static String m_sNonSecurePortalImageServerJavaScriptURL;

	/** Shared Image Server Base URL. */
	private static String m_sNonSecureSharedImageServerBaseURL;

	/** Shared Image Server URL to Images. */
	private static String m_sNonSecureSharedImageServerImagesURL;

	/** Shared Image Server URL to JavaScripts. */
	private static String m_sNonSecureSharedImageServerJavaScriptURL;

	/** Shared Image Server URL to Style sheets. */
	private static String m_sNonSecureSharedImageServerStylesURL;


	/////////
	// Secure
	/////////

	/**
	 * Secure Image Server Root URL
	 * This URL is just the beginning part (i.e. "http://myserver:8080/ptimages/").
	 * It will have a trailing slash if the ImgSvrURLHelper is set for trailing slashes.
	 * This is set by default.
	 * 
	 * rjz:  ImageServerBaseURL has the domain name ('plumtree') in the URL, created ImageServerRootURL to return the
	 * "true" root ImageServerURL.
	 */
	private static String m_sSecureImageServerRootURL;
	
	private static String m_sSecureImageServerBaseURL;

	/** Portal Image Server Base URL. */
	private static String m_sSecurePortalImageServerBaseURL;

	/** Portal Image Server URL to Help files. */
	private static String m_sSecurePortalImageServerHelpURL;

	/** Portal Image Server URL to Images. */
	private static String m_sSecurePortalImageServerImagesURL;

	/** Portal Image Server URL to JavaScripts. */
	private static String m_sSecurePortalImageServerJavaScriptURL;

	/** Shared Image Server Base URL. */
	private static String m_sSecureSharedImageServerBaseURL;

	/** Shared Image Server URL to Images. */
	private static String m_sSecureSharedImageServerImagesURL;

	/** Shared Image Server URL to JavaScripts. */
	private static String m_sSecureSharedImageServerJavaScriptURL;

	/** Shared Image Server URL to Style sheets. */
	private static String m_sSecureSharedImageServerStylesURL;

	/** Shared HTML doc type specification */
	private static String m_sHTMLDocType;


	public static final String JSCOMPONENT_JSXML = "jsxml";
	
	public static final String JSCOMPONENT_JSPORTLET = "jsportlet";
	
	public static final String JSCOMPONENT_JSUTIL = "jsutil";
	
	public static final String JSCOMPONENT_JSDATEPICKER = "jsdatepicker";
	
	public static final String JSCOMPONENT_JSPORTALMENU = "jsportalmenus";
	
	public static final String JSCOMPONENT_JSROBOHELPER = "jsrobohelp";
	
	/**
	 * Static init method.  Can only be called once.
	 *
	 * @param String the main application name
	 * @param ConfigVarPack
	 * @param ImgSvrURLHelper
	 * @return ReturnValue describing succes or failure
	 */
	public static ReturnValue StaticInit(String strApplicationName, ConfigVarPack vpConfig,
		ImgSvrURLHelper ptImageServerHelper)
	{
	    ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeConfigHelper successful.",null);
	    
		if (s_bInitialized)
		{
			log.Error("Attempting to initialize confighelper a second time.  Ignoring.");	
			return retVal;
		}

		try
		{
			s_bInitialized = true;
	
			s_strApplicationName = strApplicationName;
	
			// Non secure URLs
			m_sNonSecureImageServerRootURL = ptImageServerHelper.GetRootURL(false);
			
			m_sNonSecureImageServerBaseURL = ptImageServerHelper.GetDomainRootURL(false);
			
			m_sNonSecureSharedImageServerBaseURL	= ptImageServerHelper.GetURL(false, true, "");
			m_sNonSecureSharedImageServerImagesURL = ptImageServerHelper.GetURL(false, true, ImgSvrURLHelper.STR_L4_FILE_TYPE_IMG);
			m_sNonSecureSharedImageServerJavaScriptURL = ptImageServerHelper.GetURL(false, true, ImgSvrURLHelper.STR_L4_FILE_TYPE_JS);
			m_sNonSecureSharedImageServerStylesURL = ptImageServerHelper.GetURL(false, true, ImgSvrURLHelper.STR_L4_FILE_TYPE_CSS);
	
			m_sNonSecurePortalImageServerBaseURL	= ptImageServerHelper.GetURL(false, false, "");
			m_sNonSecurePortalImageServerHelpURL	= ptImageServerHelper.GetURL(false, false, ImgSvrURLHelper.STR_L4_FILE_TYPE_HELP);
			m_sNonSecurePortalImageServerImagesURL = ptImageServerHelper.GetURL(false, false, ImgSvrURLHelper.STR_L4_FILE_TYPE_IMG);
			m_sNonSecurePortalImageServerJavaScriptURL = ptImageServerHelper.GetURL(false, false, ImgSvrURLHelper.STR_L4_FILE_TYPE_JS);
	
			// Secure URLs
			m_sSecureImageServerRootURL = ptImageServerHelper.GetRootURL(true);
			
			m_sSecureImageServerBaseURL = ptImageServerHelper.GetDomainRootURL(true);
	
			m_sSecureSharedImageServerBaseURL	= ptImageServerHelper.GetURL(true, true, "");
			m_sSecureSharedImageServerImagesURL = ptImageServerHelper.GetURL(true, true, ImgSvrURLHelper.STR_L4_FILE_TYPE_IMG);
			m_sSecureSharedImageServerJavaScriptURL = ptImageServerHelper.GetURL(true, true, ImgSvrURLHelper.STR_L4_FILE_TYPE_JS);
			m_sSecureSharedImageServerStylesURL = ptImageServerHelper.GetURL(true, true, ImgSvrURLHelper.STR_L4_FILE_TYPE_CSS);
	
			m_sSecurePortalImageServerBaseURL	= ptImageServerHelper.GetURL(true, false, "");
			m_sSecurePortalImageServerHelpURL	= ptImageServerHelper.GetURL(true, false, ImgSvrURLHelper.STR_L4_FILE_TYPE_HELP);
			m_sSecurePortalImageServerImagesURL = ptImageServerHelper.GetURL(true, false, ImgSvrURLHelper.STR_L4_FILE_TYPE_IMG);
			m_sSecurePortalImageServerJavaScriptURL = ptImageServerHelper.GetURL(true, false, ImgSvrURLHelper.STR_L4_FILE_TYPE_JS);
	
			DetermineDocType( vpConfig );
	
			retVal = InitSecurityMode(vpConfig);
			if (retVal.getResultCode()==PT_STARTUP_RESULTCODES.FATAL)
			    return retVal;
			
			InitHTTPEntryPoint(vpConfig);
			InitSSOEntryPoint(vpConfig);
			BuildMainServerApplicationURLs(vpConfig);
			InitImageserverConnectTimeout(vpConfig);
			
			// Note: Check mapping urls against security mode for obvious errors 
			retVal = LoadMappingURLs(vpConfig);
			if (retVal.getResultCode()==PT_STARTUP_RESULTCODES.FATAL)
			    return retVal;
	
			retVal = SetPerformanceCommentsEnabled(vpConfig);
			if (retVal.getResultCode()==PT_STARTUP_RESULTCODES.FATAL)
			    return retVal;
			
			SetPlatform(vpConfig);
		}
		catch( Exception e )
		{
		    retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Config Helper intiialization FAILURE.",
		            XPException.GetInstance(e));
		}
		
		return retVal;
	}

	/**
	 * Whether or not the config helper has been initialized properly.
	 *
	 * @return true implies this has been initialized.
	 */
	public static boolean IsInitialized()
	{
		return s_bInitialized;
	}

	/**
	 * JF- Returns the Application URL according to the URL Mapping and the security.
	 * Warning: depending on the security mode, the security in argument might not make sense.
	 * E.g.: in mode 0, it doesn't make sense to request the secure URL
	 * (and it might just return an empty URL)
	 *
	 * The normalized base URL is available either cached on the AActivitySpace or by calling
	 * ConfigHelper.GetNormalizedBaseURL with the current request URL.
	 * 
	 * @param _sNormalizedBaseURL the normalized base URL from the current request.
	 * @param _bSecure HTTP or HTTPS
	 *
	 * @return String
	 */
	public static final String GetApplicationBaseURL(String _sNormalizedBaseURL, boolean _bSecure)
	{
		String[] arrApplicationURLs = null;
		
		
		log.Debug("Getting ApplicationBaseURL for NormalizedBaseURL: " + _sNormalizedBaseURL );
		if (m_htURLMapping.ContainsKey(_sNormalizedBaseURL))
		{			
			// JF- sRequestURL was found in the mapping, we get the corresponding value.			
			log.Debug("Found the ApplicationBaseURL in the Mapping!" );
			arrApplicationURLs = (String[]) m_htURLMapping.GetElement(_sNormalizedBaseURL);
		}
		else
		{
			log.Debug("Could not find the ApplicationBaseURL in the Mapping!" );
			//RT - Bug 9381128: Here we should check if the normalizedbaseurl is secure and if it then
			// we check the secureapplication url value rather than http key and if the mapping exist then
			// we use that mapping instead of default fallback.
			if(_bSecure && (-1 != XPStringUtility.ToLowerCase(_sNormalizedBaseURL).indexOf(HTTPS_SCHEME)))
			{
				log.Debug("Looking through all the https values of the mapping" );
				IXPEnumerator elementItr = m_htURLMapping.GetElements();
				String[] arrMappedURLs;
				while (elementItr.MoveNext())
				{				
					arrMappedURLs = (String[])elementItr.GetCurrent();
					
					if(XPStringUtility.EqualsIgnoreCase(arrMappedURLs[URL_MAPPING_SECUREAPPLICATIONURL], _sNormalizedBaseURL)){
						log.Debug("Found the mapping by comparing https values" );
						arrApplicationURLs = arrMappedURLs;
						break;
					}
				}					
			}
			if(null == arrApplicationURLs)
			{
				log.Debug("Request URL " + _sNormalizedBaseURL + " was not found in the mapping, assigning default value");
				// JF- sRequestURL was not found in the mapping, we get the default value.
				arrApplicationURLs = (String[]) m_htURLMapping.GetElement(URL_MAPPING_DEFAULT_ENTRY);
			}			
		}
		
		// JF- In mode 0, it can either be http or https, based on the security of sRequestURL
		if ((ConfigVarPack.SECURITY_MODE_NO_SSL == m_nSecurityMode) 
			&& (-1 != XPStringUtility.ToLowerCase(_sNormalizedBaseURL).indexOf(HTTPS_SCHEME)))
		{
			_bSecure = true;
		}

		if (_bSecure)
		{
			// JF- If the URL from the mapping is the default value,
			// it means that we need to return the RequestURL
			if ((URL_MAPPING_DEFAULT_ENTRY.equals(arrApplicationURLs[URL_MAPPING_SECUREAPPLICATIONURL]))
				|| (("").equals(arrApplicationURLs[URL_MAPPING_SECUREAPPLICATIONURL])))
			{
				log.Debug("Returning ApplicationBaseURL: " + _sNormalizedBaseURL);
				return _sNormalizedBaseURL;
			}
			// JF- Return the Secure Application URL from the Mapping
			else
			{
				log.Debug("Returning ApplicationBaseURL: " + arrApplicationURLs[URL_MAPPING_SECUREAPPLICATIONURL]);
				return arrApplicationURLs[URL_MAPPING_SECUREAPPLICATIONURL];
			}
		}
		else
		{
			// JF- If the URL from the mapping is the default value,
			// it means that we need to return the RequestURL
			if ((URL_MAPPING_DEFAULT_ENTRY.equals(arrApplicationURLs[URL_MAPPING_APPLICATIONURL]))
				|| (("").equals(arrApplicationURLs[URL_MAPPING_APPLICATIONURL])))
			{
				log.Debug("Returning ApplicationBaseURL which is default value in the mapping: " + _sNormalizedBaseURL);
				return _sNormalizedBaseURL;
			}
			// JF- Return the Application URL from the Mapping
			else
			{
				log.Debug("Returning ApplicationBaseURL from the mapping: " + arrApplicationURLs[URL_MAPPING_SECUREAPPLICATIONURL]);
				return arrApplicationURLs[URL_MAPPING_APPLICATIONURL];
			}
		}
	}

	/**
	 * JF- Returns the Application URL according to the URL Mapping.
	 * Warning: In mode 1, it returns the non secure URL by default.
	 *
	 * @param AActivitySpace the current activity space.
	 *
	 * @return String
	 */
	public static final String GetApplicationBaseURL(AActivitySpace asSpace)
	{
		return GetApplicationBaseURL(asSpace.GetCurrentNormalizedBaseURL());
	}

	/**
	 * JF- Returns the Application URL according to the URL Mapping.
	 * Warning: In mode 1, it returns the non secure URL by default.
	 * 
	 * GetApplicationBaseURL(AActivitySpace) should be used if possible.
	 *
	 * @param _sNormalizedBaseURL Normalized Base URL from the AActivity Space
	 * (or ConfigHelper.GetNormalizedBaseURL).
	 *
	 * @return String
	 */
	public static final String GetApplicationBaseURL(String _sNormalizedBaseURL)
	{
		// JF- In modes 2 and 3, returns the secure URL
		if ((ConfigVarPack.SECURITY_MODE_SSL_ALL == m_nSecurityMode)
			|| (ConfigVarPack.SECURITY_MODE_SSL_ACCELERATOR == m_nSecurityMode))
		{
			return GetApplicationBaseURL(_sNormalizedBaseURL, true);
		}
		// JF- In modes 0 and 1, returns the non secure URL
		// Note that in mode 0, the security will be checked later based on the security of _sRequestURL
		else
		{
			return GetApplicationBaseURL(_sNormalizedBaseURL, false);
		}
	}
	
	/**
	 * JF- Returns the Application URL according to the URL Mapping and
	 * determines the security according to the space name in argument.
	 *
	 * @param _sNormalizedBaseURL Normalized Base URL from the AActivity Space.
	 * (or ConfigHelper.GetNormalizedBaseURL).
	 * @param _sASName Name of the destination Activity Space
	 * @param _bIsSubSpace Is the target AS a sub space?
	 *
	 * @return String
	 */
	public static final String GetApplicationBaseURLToAS(String _sNormalizedBaseURL,
		String _sASName, boolean _bIsSubSpace)
	{
		if ((_bIsSubSpace) && (_sNormalizedBaseURL.startsWith(ConfigHelper.HTTPS_SCHEME)))
		{
			return GetApplicationBaseURL(_sNormalizedBaseURL, true);
		}
		else
		{
			return GetApplicationBaseURL(_sNormalizedBaseURL, IsSpaceSecured(_sASName));
		}
	}

	/**
	 * JF- Returns the Main server URL from x_config.xml
	 *
	 * @param _bIsSecure HTTP or HTTPS
	 *
	 * @return String
	 */
	public static final String GetMainServerApplicationURL(boolean _bIsSecure)
	{
		if (_bIsSecure)
		{
			return m_sMainServerSecureApplicationURL;
		}
		else
		{
			return m_sMainServerApplicationURL;
		}
	}
	
	/**
	 * JF- Retuns the value of the HTTP Entry point from x_config.xml 
	 * E.g.: server.pt
	 * 
	 * @return String
	 */
	public static final String GetHTTPEntryPoint()
	{
		return m_sHTTPEntryPoint;	
	}

	/**
	 * Returns whether or not the performance metrics comments displayed
	 * in the HTML source at the end of each page are enabled.
	 * 
	 * This value should be false if web caching and acceleration software
	 * is being used, otherwise pages will never have static content.
	 * 
	 * @return boolean true implies the comments are displayed.
	 */
	public static final boolean GetPerformanceCommentsEnabled()
	{
		return m_bArePerformanceCommentsEnabled;
	}
	
	/**
	 * Returns whether or not the performance metrics stacktraces displayed
	 * in the HTML source at the end of each page are enabled.
	 * 
	 * This value should be false if web caching and acceleration software
	 * is being used, otherwise pages will never have static content.
	 * 
	 * @return boolean true implies the stacktraces are displayed.
	 */
	public static final boolean GetPerformanceStacktracesEnabled()
	{
		return m_bArePerformanceStacktracesEnabled;
	}

	/**
	 * Returns the security of the Activty Space.
	 * 
	 * @param _bIsConnectionSecure
	 * @return boolean
	 */
	public static final boolean GetIsSpaceSecure(boolean _bIsConnectionSecure)
	{
		if (ConfigVarPack.SECURITY_MODE_SSL_ACCELERATOR == ConfigHelper.GetSecurityMode())
		{
			return true;
		}
		else
		{
			return _bIsConnectionSecure;
		}
	}
	
	/**
	 * Helper method to know whether the Redirect is going to a sub space of the current space of no.
	 * 
	 * @param _myRedirect
	 * @return boolean
	 */
	public static final boolean IsGoingToSubSpace(Redirect _myRedirect)
	{
		boolean bIsSubSpace = false;
				
		// JF- Check if the destination space is a sub space of the current space
		if ((null != _myRedirect.GetSubSpaceKey()) && ("").equals(_myRedirect.GetSubSpaceKey()))
		{
			bIsSubSpace = true;
		}
		
		return bIsSubSpace;
	}

	/**
	 * JF- Helper method to get the settings from the Config var pack and populate the mapping.
	 * DAP - added a bit of validation for startup cycle
	 *
	 * @param _vpConfig
	 */
	private static final ReturnValue LoadMappingURLs(ConfigVarPack _vpConfig)
	{
	    ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeConfigHelper successful.",null);
	    
	    try
	    {
			XPHashtable htConfigXMLSettings = _vpConfig.GetConfigXMLSettings();
			String sSection = ConfigConstants.URL_MAPPING_SECTION;
			XPArrayList localApplicationURLMapping = new XPArrayList();
			XPArrayList localSecureApplicationURLMapping = new XPArrayList();
			m_htURLMapping = new XPHashtable();
			
			int i = 0;
			Object oURLFromRequest, oApplicationURL, oSecureApplicationURL;
	
			while (htConfigXMLSettings.ContainsKey(sSection + ConfigConstants.URL_FROM_REQUEST_SUBSECTION + i))
			{
				oURLFromRequest = htConfigXMLSettings.GetElement(sSection +
					ConfigConstants.URL_FROM_REQUEST_SUBSECTION + i);
				oApplicationURL = htConfigXMLSettings.GetElement(sSection +
					ConfigConstants.APPLICATION_URL_SUBSECTION + i);
				oSecureApplicationURL = htConfigXMLSettings.GetElement(sSection +
					ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + i);
	
				AddToURLMapping(i, oURLFromRequest, oApplicationURL, oSecureApplicationURL);
				
				if (oApplicationURL != null)
				    localApplicationURLMapping.Add((String)oApplicationURL);
				if (oSecureApplicationURL != null)
				    localSecureApplicationURLMapping.Add((String)oSecureApplicationURL);
	
				i ++;
			}
	
			if (0 == m_htURLMapping.GetSize())
			{
				log.Fatal("Invalid Mapping URL: The " + ConfigConstants.URL_MAPPING_SECTION
						+ " section was not found in portalconfig.xml.");
				
				retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Invalid Mapping URL: The " + ConfigConstants.URL_MAPPING_SECTION
					+ " section was not found in portalconfig.xml.",StackTracer.MakeXPException("URLMapping section is null!"));
			}
	
			if (!m_htURLMapping.ContainsKey(URL_MAPPING_DEFAULT_ENTRY))
			{
				log.Fatal("Invalid Mapping URL: The URL Mapping doesn't contain the default entry. " +
						"Check the URLMapping section in portalconfig.xml and" +
						" add one parameter called URLFromRequest0 and equals to \"*\".");
				
				retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Invalid Mapping URL: The URL Mapping doesn't contain the default entry. " +
						"Check the URLMapping section in portalconfig.xml and " +
						"add one parameter called URLFromRequest0 and equals to \"*\".",
						StackTracer.MakeXPException("URLMapping contains no default mapping!"));
			}
			
			// Handle the case where there is no mapping, and security mode 1 has been selected
			
			// JF- If it's the default entry, check the values of the application URLs
			switch (m_nSecurityMode)
			{
				// JF- In mode 0 we can have * for both the application URL and the secure application URL
				// It's not a problem because we figure out the security of the application URL 
				// based on the incoming URL.
				case ConfigVarPack.SECURITY_MODE_NO_SSL:
				    // no checks
			        break;
	
				case ConfigVarPack.SECURITY_MODE_SSL_CRITICAL_AS_ONLY:
					// JF- In mode 1, the application URL cannot be * because we need both the
					// non secure and the secure application URL
					if (0 == localApplicationURLMapping.GetSize())
					{
						log.Fatal("Invalid Mapping URL: In security mode 1, you must have specified an application URL mapping because we " +
								"need both the non secure and the secure application URL");
						retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
						        "Invalid Mapping URL: In security mode 1, you must have specified an application URL mapping because we " +
								"need both the non secure and the secure application URL",
								StackTracer.MakeXPException("URLMapping security mode rules violation!"));
					}
					else
					{
				        IXPEnumerator enum1 = localApplicationURLMapping.GetEnumerator();
				        while (enum1.MoveNext() == true)
				        {
				            String appMap = (String)enum1.GetCurrent();
					        if (appMap.compareTo(URL_MAPPING_DEFAULT_ENTRY) == 0)
					        {
								log.Fatal("Invalid Mapping URL: In security mode 1, the application URL cannot be \"*\" because we " +
									"need both the non secure and the secure application URL");
								retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
								        "Invalid Mapping URL: In security mode 1, the application URL cannot be \"*\" because we " +
								        "need both the non secure and the secure application URL",
								        StackTracer.MakeXPException("URLMapping security mode rules violation!"));
					        }
					    }
					}
				    break;
	
				case ConfigVarPack.SECURITY_MODE_SSL_ALL:
				    // JF- In mode 2, the secure application URL, cannot be * because if the incoming URL 
				    // was http we need to force the redirect to https				    
					if (0 == localSecureApplicationURLMapping.GetSize())
					{
						log.Fatal("Invalid Mapping URL: In security mode 2, you must have specified a secure application URL mapping.");
						retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
						        "Invalid Mapping URL: In security mode 1,  you must have specified a secure application URL mapping.",
								StackTracer.MakeXPException("URLMapping security mode rules violation!"));
					}
					else
					{
				        IXPEnumerator enum1 = localSecureApplicationURLMapping.GetEnumerator();
				        while (enum1.MoveNext() == true)
				        {
				            String appMap = (String)enum1.GetCurrent();
					        if (appMap.compareTo(URL_MAPPING_DEFAULT_ENTRY) == 0)
					        {
								log.Fatal("Invalid Mapping URL: In security mode 2, the secure application URL cannot be \"*\" " +
										"because if the incoming URL was http we need to force the redirect to https");
								retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
								        "Invalid Mapping URL: In security mode 2, the secure application URL cannot be \"*\" " +
										"because if the incoming URL was http we need to force the redirect to https",
								        StackTracer.MakeXPException("URLMapping security mode rules violation!"));
					        }
					    }
					}
					break;
					
				case ConfigVarPack.SECURITY_MODE_SSL_ACCELERATOR:
				    // JF- In mode 3, the secure application URL, cannot be * because the incoming URL will 
				    // be http (coming from the bigIP) whereas the application URL has to be https
					if (0 == localSecureApplicationURLMapping.GetSize())
					{
						log.Fatal("Invalid Mapping URL: In security mode 3, you must have specified a secure application URL mapping.");
						retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
						        "Invalid Mapping URL: In security mode 3, you must have specified a secure application URL mapping.",
								StackTracer.MakeXPException("URLMapping security mode rules violation!"));
					}
					else
					{
				        IXPEnumerator enum1 = localSecureApplicationURLMapping.GetEnumerator();
				        while (enum1.MoveNext() == true)
				        {
				            String appMap = (String)enum1.GetCurrent();
					        if (appMap.compareTo(URL_MAPPING_DEFAULT_ENTRY) == 0)
					        {
								log.Fatal("Invalid Mapping URL: In security mode 3, the secure application URL cannot be \"*\" " +
										"because the incoming URL will be http (coming from the bigIP) whereas the application URL has to be https");
								retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,
								        "Invalid Mapping URL: In security mode 3, the secure application URL cannot be \"*\" " +
								        "because the incoming URL will be http (coming from the bigIP) whereas the application URL has to be https",
								        StackTracer.MakeXPException("URLMapping security mode rules violation!"));
					        }
					    }
					}				    
				    break;
			}
	    }
	    catch( Exception e )
	    {
	        retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Error loading the mapping URLs from " +
	        		"portalconfig.xml.  Please check that they are both correct and match your security mode " +
	        		"specific requirements.",
	        		XPException.GetInstance(e));
	    }
	
		return retVal;
	}

	/**
	 * JF- Helper method to get the settings from the Config var pack and set
	 * whether or not the performance comments are enabled.
	 *
	 * @param _vpConfig
	 */
	private static final ReturnValue SetPerformanceCommentsEnabled(ConfigVarPack _vpConfig)
	{
	    ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeConfigHelper successful.",null);
		String strPerformanceCommentsEnabled = _vpConfig.GetConfigValueAsString
			(ConfigConstants.SYSTEM_PROPERTIES_SECTION,
			ConfigConstants.PERFORMANCE_COMMENTS_SUBSECTION,
			ConfigConstants.VALUE_ATTRIBUTE);

		if ((strPerformanceCommentsEnabled != null) &&
			!("".equals(strPerformanceCommentsEnabled)))
		{
			try
			{
				int nPerformanceCommentsEnabled = XPConvert.ToInteger(strPerformanceCommentsEnabled);
				if (nPerformanceCommentsEnabled == 0)
				{
					m_bArePerformanceCommentsEnabled = false;
					m_bArePerformanceStacktracesEnabled = false;
				} 
				else if (nPerformanceCommentsEnabled == 1)
				{
					m_bArePerformanceCommentsEnabled = true;
					m_bArePerformanceStacktracesEnabled = true;
				}
				else if (nPerformanceCommentsEnabled == 2)
				{
					m_bArePerformanceCommentsEnabled = true;
					m_bArePerformanceStacktracesEnabled = false;
				} 
				else if (nPerformanceCommentsEnabled == 3)
				{
					m_bArePerformanceCommentsEnabled = false;
					m_bArePerformanceStacktracesEnabled = true;
				}
				else 
				{
					log.Error("Invalid Performance Comments value (" + strPerformanceCommentsEnabled
							+ ") in System Properties section in portalconfig.xml.  Using default value ("
							+ XPConvert.ToString(m_bArePerformanceCommentsEnabled) + ").");
				}
			} 
			catch (Exception e)
			{				
				log.Error(e, "Unable to read Performance Comments value (" + strPerformanceCommentsEnabled
					+ ") in System Properties section in portalconfig.xml.  Using default value ("
					+ XPConvert.ToString(m_bArePerformanceCommentsEnabled) + ").");
			
				
				retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.WARN,"Unable to read Performance Comments value (" + strPerformanceCommentsEnabled
						+ ") in System Properties section in portalconfig.xml.  Using default value ("
						+ XPConvert.ToString(m_bArePerformanceCommentsEnabled) + ").",null);
			}
		}
		return retVal;
	}

	/**
	 * JF- Helper method to check the URLs and add them to the mapping if they are correct
	 *
	 * @param _nIndex
	 * @param _oURLFromRequest
	 * @param _oApplicationURL
	 * @param _oSecureApplicationURL
	 */
	private static final ReturnValue AddToURLMapping(int _nIndex,
		Object _oURLFromRequest, Object _oApplicationURL, Object _oSecureApplicationURL)
	{
	    ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeConfigHelper successful.",null);
		String sURLFromRequest="";

		try
		{
			sURLFromRequest = (String) _oURLFromRequest;
		}
		catch (Exception e)
		{
			
			log.Fatal("Invalid Mapping URL: Mapping URLs have to be Strings. " +
					"Check the value of " + ConfigConstants.URL_FROM_REQUEST_SUBSECTION
					+ _nIndex + " in portalconfig.xml.");

			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Invalid Mapping URL: Value of " +
				ConfigConstants.URL_FROM_REQUEST_SUBSECTION + _nIndex +" is invalid in portalconfig.xml",StackTracer.MakeXPException("URL Mapping Rules Violation"));
		}
		
		// JF- If the URL is an empty String, we change it to *
		sURLFromRequest = CheckEmptyURL(sURLFromRequest);

		// JF- Is it the default entry?
		// DAP master try catch for error interpretation
		try
		{
			if (URL_MAPPING_DEFAULT_ENTRY.equals(sURLFromRequest))
			{
				// JF- We can't have more than one default entry
				if (m_htURLMapping.ContainsKey(URL_MAPPING_DEFAULT_ENTRY))
				{
					
					log.Fatal("Invalid Mapping URL: You can have only one entry with the default value \"*\". " +
							"Check the value of " + ConfigConstants.URL_FROM_REQUEST_SUBSECTION
							+ _nIndex + " in portalconfig.xml.");				
	
					retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Invalid Mapping URL: "
							+ "You can have only one entry with the default value \"*\". " +
							"Check the value of " + ConfigConstants.URL_FROM_REQUEST_SUBSECTION
							+ _nIndex + " in portalconfig.xml.",StackTracer.MakeXPException("URL Mapping Rules Violation"));
					
				}
				else
				{
					// JF- If it's the default entry, check the values of the application URLs
					switch (m_nSecurityMode)
					{
						// JF- In mode 0 we can have * for both the application URL and the secure application URL
						// It's not a problem because we figure out the security of the application URL 
						// based on the incoming URL.
						case ConfigVarPack.SECURITY_MODE_NO_SSL:
						_oApplicationURL = CheckEmptyURL(_oApplicationURL);
						_oSecureApplicationURL = CheckEmptyURL(_oSecureApplicationURL);
						
						if (!URL_MAPPING_DEFAULT_ENTRY.equals(_oApplicationURL))
						{
							// JF- If it's not *, make sure this is a valid HTTP URL
							ValidateMappingURL(_oApplicationURL, ConfigConstants.APPLICATION_URL_SUBSECTION + _nIndex);
							ValidateHTTPMappingURL((String) _oApplicationURL,
								ConfigConstants.APPLICATION_URL_SUBSECTION + _nIndex);
						}
						
						if (!URL_MAPPING_DEFAULT_ENTRY.equals(_oSecureApplicationURL))
						{		
							// JF- If it's not *, make sure this is a valid HTTPS URL
							ValidateMappingURL(_oSecureApplicationURL, ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
							ValidateHTTPSMappingURL((String) _oSecureApplicationURL,
								ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
						}
						break;
	
						case ConfigVarPack.SECURITY_MODE_SSL_CRITICAL_AS_ONLY:
						// JF- In mode 1, the application URL cannot be * because we need both the
						// non secure and the secure application URL
	
						// JF- Make sure the Application URL and Secure Application URL are correct
						ValidateMappingURL(_oApplicationURL, ConfigConstants.APPLICATION_URL_SUBSECTION + _nIndex);
						ValidateHTTPMappingURL((String) _oApplicationURL,
							ConfigConstants.APPLICATION_URL_SUBSECTION + _nIndex);
	
						ValidateMappingURL(_oSecureApplicationURL, ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
						ValidateHTTPSMappingURL((String) _oSecureApplicationURL,
							ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
						break;
	
						case ConfigVarPack.SECURITY_MODE_SSL_ALL:
						case ConfigVarPack.SECURITY_MODE_SSL_ACCELERATOR:
						// JF- In mode 2, the secure application URL, cannot be * because if the incoming URL 
						// was http we need to force the redirect to https
						// JF- In mode 3, the secure application URL, cannot be * because the incoming URL will 
						// be http (coming from the bigIP) whereas the application URL has to be https
						ValidateMappingURL(_oSecureApplicationURL, ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
						ValidateHTTPSMappingURL((String) _oSecureApplicationURL,
							ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
						break;
					}
				}
			}
			else
			{
				// JF- This is not the default entry, make sure this is a valid URL
				ValidateMappingURL(sURLFromRequest, ConfigConstants.URL_FROM_REQUEST_SUBSECTION + _nIndex);
	
				switch (m_nSecurityMode)
				{
					case ConfigVarPack.SECURITY_MODE_NO_SSL:
					case ConfigVarPack.SECURITY_MODE_SSL_CRITICAL_AS_ONLY:
					// JF- In mode 0:
					// RequestURL can be either HTTP or HTTPS
					// ApplicationURL has to be a valid HTTP URL
					// ApplicationSecureURL has to be a valid HTTPS URL
					
					// JF- In mode 1:
					// RequestURL has to be a valid URL (HTTP or HTTPS)
					// ApplicationURL has to be a valid HTTP URL
					// ApplicationSecureURL has to be a valid HTTPS URL
								 
					ValidateMappingURL(_oApplicationURL, ConfigConstants.APPLICATION_URL_SUBSECTION + _nIndex);
					ValidateHTTPMappingURL((String) _oApplicationURL,
						ConfigConstants.APPLICATION_URL_SUBSECTION + _nIndex);
						
					ValidateMappingURL(_oSecureApplicationURL, ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
					ValidateHTTPSMappingURL((String) _oSecureApplicationURL,
						ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
					break;
	
					case ConfigVarPack.SECURITY_MODE_SSL_ALL:
					// JF- In mode 2:
					// RequestURL can be either http or https
					// ApplicationURL is not use, no need to check it
					// ApplicationSecureURL has to be a valid HTTPS URL
	
					ValidateMappingURL(_oSecureApplicationURL, ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
					ValidateHTTPSMappingURL((String) _oSecureApplicationURL,
						ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
					break;
	
					case ConfigVarPack.SECURITY_MODE_SSL_ACCELERATOR:
					// JF- In mode 3:
					// RequestURL can be either HTTP (through the BigIP) or HTTPS (direct request)
					// ApplicationURL is not use, no need to check it
					// ApplicationSecureURL has to be a valid HTTPS URL
					ValidateMappingURL(_oSecureApplicationURL, ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
					ValidateHTTPSMappingURL((String) _oSecureApplicationURL,
						ConfigConstants.SECURE_APPLICATION_URL_SUBSECTION + _nIndex);
					break;
				}
			}
		}
		catch( XPException e )
		{
		    //map to returnval
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Invalid Mapping URL Error",
			        XPException.GetInstance(e));
		}

		// JF- We have checked all the URLs, we now add them to the mapping
		String[] arrApplicationURLs = new String[]{GetURLForMapping(_oApplicationURL),
			GetURLForMapping(_oSecureApplicationURL)};

		Object oOldValue = m_htURLMapping.PutElement(GetURLForMapping(sURLFromRequest), arrApplicationURLs);

		if (null != oOldValue)
		{
			log.Fatal("Invalid Mapping URL: You can have only one entry with the key \""
					+ sURLFromRequest + "\". " + "Check the value of " +
					ConfigConstants.URL_FROM_REQUEST_SUBSECTION + _nIndex + " in portalconfig.xml.");			
			retVal = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Invalid Mapping URL: You can have only one entry with the key \""
				+ sURLFromRequest + "\". " + "Check the value of " + ConfigConstants.URL_FROM_REQUEST_SUBSECTION
				+ _nIndex + " in portalconfig.xml.",StackTracer.MakeXPException("URL Mapping Rules Violation"));
		}
		else
		{
			log.Info("Added new value to URL Mapping: " + _nIndex + " - "
					+ GetURLForMapping(sURLFromRequest) + " --> "
					+ GetURLForMapping(_oApplicationURL) + " and "
					+ GetURLForMapping(_oSecureApplicationURL) + ".");
		}
		
		return retVal;
	}
	
	/**
	 * Helper method to check if the URL is empty and if it is, change it to 
	 * the default value.
	 * 
	 * @param _sURL
	 * 
	 * @return String
	 */
	private static String CheckEmptyURL(Object _oURL)
	{
		String sURL = URL_MAPPING_DEFAULT_ENTRY;
		
		try
		{
			sURL = (String) _oURL;	
		}
		catch (Exception e)
		{
			// JF- Just use the default value if could not cast the URL to a String
			return sURL;
		}
		
		if (0 == sURL.length())
		{
			sURL = URL_MAPPING_DEFAULT_ENTRY;
		}
		
		return sURL;
		
	}

	/**
	 * JF- Helper method to check if the URL starts with http://
	 *
	 * @param _sURL
	 * @param _sConfigXMLParamName
	 */
	private static final void ValidateHTTPMappingURL(String _sURL, String _sConfigXMLParamName)
	{
		if (!_sURL.startsWith(HTTP_SCHEME))
		{
			
				log.Fatal("Invalid Mapping URL: Value of " +
					_sConfigXMLParamName + " in portalconfig.xml should start with \"" + HTTP_SCHEME + "\".");
			

			throw new XPException("Invalid Mapping URL: Value of " +
				_sConfigXMLParamName + " in portalconfig.xml should start with \"" + HTTP_SCHEME + "\".");
		}
	}

	/**
	 * JF- Helper method to check if the URL starts with https://
	 *
	 * @param _sURL
	 * @param _sConfigXMLParamName
	 */
	private static final void ValidateHTTPSMappingURL(String _sURL, String _sConfigXMLParamName)
	{
		if (!_sURL.startsWith(HTTPS_SCHEME))
		{
			
				log.Fatal("Invalid Mapping URL: Value of " +
					_sConfigXMLParamName + " in portalconfig.xml should start with \"" + HTTPS_SCHEME + "\".");
			

			throw new XPException("Invalid Mapping URL: Value of " +
				_sConfigXMLParamName + " in portalconfig.xml should start with \"" + HTTPS_SCHEME + "\".");
		}
	}

	/**
	 * JF- Helper method to check if the URL is not null, starts with http
	 * and ends with the HTTPEntry point.
	 *
	 * @param _oURL
	 * @param _sConfigXMLParamName
	 */
	private static final void ValidateMappingURL(Object _oURL, String _sConfigXMLParamName)
	{
		String sURL;

		if (null == _oURL)
		{
			
				log.Fatal("Invalid Mapping URL: Value of " +
					_sConfigXMLParamName + " in portalconfig.xml cannot be null.");
			

			throw new XPException("Invalid Mapping URL: Value of " +
				_sConfigXMLParamName + " in portalconfig.xml cannot be null.");
		}

		sURL = (String) _oURL;

		if (!sURL.startsWith("http"))
		{
			
				log.Fatal("Invalid Mapping URL: Value of " +
					_sConfigXMLParamName + " in portalconfig.xml is not a valid URL.");
			

			throw new XPException("Invalid Mapping URL: Value of " +
				_sConfigXMLParamName + " in portalconfig.xml is not a valid URL.");
		}
	}

	/**
	 * DLH- Helper method to return the normalized base URL for use
	 * in determining the correct application base url.
	 *
	 * @param _sURL
	 * @return String the normalized base URL
	 */
	public static final String GetNormalizedBaseURL(String _sURL)
	{
		// JF- Modify the sRequestURL to ignore ports 80 and 443
		String sNormalizedURL = GetURLForMapping(_sURL);

		if (-1 != sNormalizedURL.indexOf(m_sHTTPEntryPoint))
		{
			sNormalizedURL = sNormalizedURL.substring(0,
 				sNormalizedURL.indexOf(m_sHTTPEntryPoint) + m_sHTTPEntryPoint.length());
		}
		else if (-1 != sNormalizedURL.indexOf(XPStringUtility.ToLowerCase(GATEWAY_URL)))
		{
			sNormalizedURL = sNormalizedURL.substring(0,
 				sNormalizedURL.indexOf(XPStringUtility.ToLowerCase(GATEWAY_URL)));
		}
		else if (-1 != sNormalizedURL.indexOf("?"))
		{
			sNormalizedURL = sNormalizedURL.substring(0,
 				sNormalizedURL.indexOf("?") + 1);
		}

		if (!sNormalizedURL.endsWith("?"))
		{
			sNormalizedURL += "?";
		}

		if (-1 != sNormalizedURL.indexOf(XPStringUtility.ToLowerCase(m_sSSOEntryPoint)))
		{
			sNormalizedURL = sNormalizedURL.substring(0, 
				sNormalizedURL.indexOf(XPStringUtility.ToLowerCase(m_sSSOEntryPoint))) 
				+ m_sHTTPEntryPoint 
				+ sNormalizedURL.substring(sNormalizedURL.indexOf(XPStringUtility.ToLowerCase(m_sSSOEntryPoint)) + m_sSSOEntryPoint.length(), 
				sNormalizedURL.length());
		}

		return sNormalizedURL;
	}

	private static final void InitSSOEntryPoint(ConfigVarPack _vpConfig)
	{
		m_sSSOEntryPoint = _vpConfig.GetConfigValueAsString(ConfigConstants.SYSTEM_PROPERTIES_SECTION,
				ConfigConstants.SSO_SERVLET_NAME_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE);

		// JF- The HTTP Entry point should not have any question mark at the end
		if (m_sSSOEntryPoint.endsWith("?"))
		{
			m_sSSOEntryPoint = m_sSSOEntryPoint.substring(0, m_sSSOEntryPoint.length() - 1);
		}

		
			log.Info("Config value: SSO Entry point is : " +
			m_sSSOEntryPoint);
		
	}

	/**
	 * JF- Helper method to return the final URL that should be added to the mapping.
	 *
	 * @param _oURL
	 */
	private static final String GetURLForMapping(Object _oURL)
	{
		if ((null == _oURL) || ("").equals(_oURL))
		{
			return "";
		}

		String sURL = (String) _oURL;

		if (URL_MAPPING_DEFAULT_ENTRY.equals(sURL))
		{
			return sURL;
		}

		String sNewURL;

		// JF- Ignore ports 80 and 443
		if (-1 != sURL.indexOf(":80/"))
		{
			sNewURL = GeneralHelpers.Replace(sURL, ":80/", "/");
		}
		else if (-1 != sURL.indexOf(":443/"))
		{
			sNewURL = GeneralHelpers.Replace(sURL, ":443/", "/");
		}
		else
		{
			sNewURL = sURL;
		}

		if (!sNewURL.endsWith("?"))
		{
			sNewURL += "?";
		}

		return XPStringUtility.ToLowerCase(sNewURL);
	}

	/**
	 * JF- Helper method to init the HTTP Entry point member variable.
	 *
	 * @param _vpConfig
	 */
	private static final void InitHTTPEntryPoint(ConfigVarPack _vpConfig)
	{
		m_sHTTPEntryPoint = _vpConfig.GetConfigValueAsString(ConfigConstants.SYSTEM_PROPERTIES_SECTION,
				ConfigConstants.HTTP_ENTRY_POINT_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE);

		// JF- The HTTP Entry point should not have any question mark at the end
		if (m_sHTTPEntryPoint.endsWith("?"))
		{
			m_sHTTPEntryPoint = m_sHTTPEntryPoint.substring(0, m_sHTTPEntryPoint.length() - 1);
		}

		
			log.Info("Config value: HTTP Entry point is : " +
				m_sHTTPEntryPoint);
		


		if (0 == m_sHTTPEntryPoint.length())
		{
			
				log.Fatal("Config value: HTTP Entry point cannot be empty. " +
					"Check the value of " + ConfigConstants.HTTP_ENTRY_POINT_SUBSECTION
					+ " in portalconfig.xml.");
			

			throw new XPException("HTTP Entry point is invalid.");
		}

	}

 	/**
	 * JF- Helper method to init the main server URL (both secure and non secure).
	 *
	 * @param _vpConfig
	 */
 	private static final void BuildMainServerApplicationURLs(ConfigVarPack _vpConfig)
	{
		// JF- Get the port number
		String sPortNumber = _vpConfig.GetConfigValueAsString(ConfigConstants.SYSTEM_PROPERTIES_SECTION,
			ConfigConstants.HTTP_PORT_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE);

		// JF- Keep the port number only if it's valid and not 80
		if ((null != sPortNumber) && !sPortNumber.equals("0") && !sPortNumber.equals("") &&
				!sPortNumber.equals("80"))
		{
			sPortNumber = ":" + sPortNumber;
		}
		else
		{
			sPortNumber = "";
		}

		// JF- Get the secure port number
		String sSecurePortNumber = _vpConfig.GetConfigValueAsString(ConfigConstants.SYSTEM_PROPERTIES_SECTION,
			ConfigConstants.HTTP_SECURE_PORT_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE);

		// JF- Keep the port number only if it's valid and not 80
		if ((null != sSecurePortNumber) && !sSecurePortNumber.equals("0") && !sSecurePortNumber.equals("") &&
				!sSecurePortNumber.equals("443"))
		{
			sSecurePortNumber = ":" + sSecurePortNumber;
		}
		else
		{
			sSecurePortNumber = "";
		}

		// JF- Get the server name
		String sServerName = _vpConfig.GetConfigValueAsString(ConfigConstants.SYSTEM_PROPERTIES_SECTION,
			ConfigConstants.SERVER_NAME_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE);


		// JF- Get the Virtual Directory Path
		String sVirtualDirectoryPath = _vpConfig.GetConfigValueAsString(ConfigConstants.SYSTEM_PROPERTIES_SECTION,
			ConfigConstants.VIRTUAL_DIRECTORY_PATH_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE);

		// JF- If no value was entered we assume there is no virtual directory
		if (0 == sVirtualDirectoryPath.length())
		{
			sVirtualDirectoryPath = "/";
		}

		// JF- Make sure the path starts with /
		if (!sVirtualDirectoryPath.startsWith("/"))
		{
			
				log.Fatal("Config value: Virtual Directory path should start with a \"/\". " +
					"Check the value of VirtualDirectoryPath in portalconfig.xml.");
			

			throw new XPException("HTTP Entry point is invalid.");
		}

		if (!sVirtualDirectoryPath.endsWith("/"))
		{
			sVirtualDirectoryPath += "/";
		}

		// JF- Build main server application URL
		m_sMainServerApplicationURL = HTTP_SCHEME + sServerName + sPortNumber;
		m_sMainServerApplicationURL += sVirtualDirectoryPath + m_sHTTPEntryPoint + "?";

		
			log.Info("Config value: Main Server Application Base URL is : " +
				m_sMainServerApplicationURL);
		

		// JF- Build main server secure application URL
		m_sMainServerSecureApplicationURL = HTTPS_SCHEME + sServerName + sSecurePortNumber;
		m_sMainServerSecureApplicationURL += sVirtualDirectoryPath + m_sHTTPEntryPoint + "?";

		
			log.Info("Config value: Main Server Secure Application Base URL is : " +
				m_sMainServerSecureApplicationURL);
		
	}

	/**
	 * JF- Helper method to init the Security Mode member variable.
	 *
	 * @param _vpConfig
	 */
	private static final ReturnValue InitSecurityMode(ConfigVarPack _vpConfig)
	{
	    ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeConfigHelper successful.",null);
	    
		m_nSecurityMode = _vpConfig.GetConfigValueAsInt(ConfigConstants.SECURITY_SECTION,
			ConfigConstants.SECURITY_MODE_SUBSECTION, ConfigConstants.VALUE_ATTRIBUTE);
		
		log.Info("Config value: Security Mode is : " + m_nSecurityMode);
		

		if ((m_nSecurityMode > 3) || (m_nSecurityMode < 0))
		{
		    log.Fatal("Config value: The Security Mode is invalid. Only Security Modes 0, 1, 2 and 3 are supported." +
					"Check the value of " + ConfigConstants.SECURITY_MODE_SUBSECTION + " in portalconfig.xml.");
		    retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Security Mode is invalid. Value '"+
		            (new Integer(m_nSecurityMode)).toString()+"'is outside the range of 0 to 3",StackTracer.MakeXPException("Security Mode Violation"));
		}
		
		return retVal;
	}

	/**
	 * DAP- Helper method to init the Imageserver Connect Timeout or Disable member variable.
	 * This value is used to determine whether or not we ping the imageserver on startup, and if so, 
	 * for how long. 
	 *
	 * @param _vpConfig
	 */
	private static final ReturnValue InitImageserverConnectTimeout(ConfigVarPack _vpConfig)
	{
	    ReturnValue retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.OK,"InitializeConfigHelper successful.",null);
	    try
	    {
	        m_nImageserverConnectTimeout = _vpConfig.GetConfigValueAsInt(ConfigConstants.MAIN_URLS_SECTION,
	                ConfigConstants.IMAGE_SERVER_CONNECTION_URL_TIMEOUT, ConfigConstants.VALUE_ATTRIBUTE);
		
	        log.Info("Config value: Imageserver Connect Timeout is : " + m_nImageserverConnectTimeout);
	    }
	    catch( Exception e )
	    {
		    log.Fatal("Config value: The ImageServerConnectionURLTimeout in portalconfig.xml is invalid.  Are you sure it is an integer?");
		    retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Imageserver Connect Timeout is invalid. Are you sure it is an integer?",
		            XPException.GetInstance(e));	        
	    }

		if ((m_nImageserverConnectTimeout < -1) || (m_nImageserverConnectTimeout > 6000))
		{
		    log.Fatal("Config value: The Imageserver Connect Timeout is invalid. Please limit yourself to the numeric range -1 (disabled) to 6000 (100 minutes).");
		    retVal  = new ReturnValue(PT_STARTUP_RESULTCODES.FATAL,"Imageserver Connect Timeout is invalid. Value '"+
		            (new Integer(m_nSecurityMode)).toString()+"'is outside the range of -1 (disabled) to 6000",
		            StackTracer.MakeXPException("Imageserver Connect Timeout rules violation"));
		} 
		
		return retVal;
	}
	
	/**
	 * This method returns the imageserver connection timeout integer in seconds
	 *
	 * @return int the timeout
	 */
	public static final int GetImageserverConnectTimeout()
	{
		return m_nImageserverConnectTimeout;
	}
	
	/**
	 * This method returns the main application name
	 *
	 * @return String the main application name
	 */
	public static final String GetMainApplicationName()
	{
		return s_strApplicationName;
	}


	/**	This method returns the document type specification for use with
	 * 	the HTMLPage objects created for each DisplayPage.
	 *
	 * 	@return String HTML document type
	 *
	 */
	public static final String GetHTMLDocType()
	{
		return m_sHTMLDocType;
	}


	private static void DetermineDocType( ConfigVarPack vpConfig )
	{
		//	This config variable can be either a string representing an integer,
		//	or a string representing a doc type.  If it's an integer string, i.e. "1",
		//	then it is mapped to the corresponding doc type.  If it's a string
		//	that cannot be converted to an integer, then it's assumed that the
		//	string is a valid document type.
		String strDocType = vpConfig.GetConfigValueAsString( "SystemProperties", "DoctypeSpecification", "value" );

		int nDocType;

		try
		{
			nDocType = XPConvert.ToInteger(strDocType);
		}
		catch (Exception e)
		{
			//	This config variable can be either a string representing an integer,
			//	or a string representing a doc type.  If it's a string
			//	that cannot be converted to an integer, then it's assumed that the
			//	string is a valid document type.

			//	config value couldn't be converted to an integer.
			nDocType = DocTypeConstants.DOCTYPE_CUSTOM_STRING;
		}

		switch( nDocType )
		{
			case DocTypeConstants.DOCTYPE_NONE:
			{
				//	none
				m_sHTMLDocType = null;
				break;
			}

			case DocTypeConstants.DOCTYPE_HTML_3_2:
			{
				//	HTML 3.2
				m_sHTMLDocType = HTMLPage.DOCTYPE_3_2;
				break;
			}

			case DocTypeConstants.DOCTYPE_HTML_4_0:
			{
				//	HTML 4.0 transitional
				m_sHTMLDocType = HTMLPage.DOCTYPE_4_0_TRANSITIONAL;
				break;
			}

			case DocTypeConstants.DOCTYPE_HTML_4_0_Frameset:
			{
				//	HTML 4.0 frameset
				m_sHTMLDocType = HTMLPage.DOCTYPE_4_0_FRAMESET;
				break;
			}

			case DocTypeConstants.DOCTYPE_HTML_4_0_STRICT:
			{
				//	HTML 4.0 strict
				m_sHTMLDocType = HTMLPage.DOCTYPE_4_0_STRICT;
				break;
			}

			case DocTypeConstants.DOCTYPE_CUSTOM_STRING:
			{
				// Assume this is a string doc type in config file.
				m_sHTMLDocType = strDocType;
				break;
			}

			default:
			{
				log.Error("Config value: The DoctypeSpecification is invalid (" + strDocType + "). Setting doctype to none.");

				//	none
				m_sHTMLDocType = null;
				break;
			}
		}

		HTMLPage.DOCTYPE_RENDERED = m_sHTMLDocType;
	}

	
	/**
	 * This method used to return the Image Server Base URL.
	 * Example: http://server/imageserver/
	 * 
	 * @param boolean true implies secure
	 * @return String the image server base URL.
	 */
	public static final String GetImageServerRootURL(boolean bSecure)
	{
		if (bSecure)
		{
			return m_sSecureImageServerRootURL;
		} 
		else
		{
			return m_sNonSecureImageServerRootURL;
		}
	}
	
	/**
	 * This method used to return the Image Server Root URL.
	 * Example: http://server/imageserver/
	 *
	 * @param AActivitySpace the space that this image is in.
	 * @return String the image server base URL.
	 */
	public static final String GetImageServerRootURL(AActivitySpace asOwner)
	{
		return GetImageServerRootURL(asOwner.GetIsSecuredSpace());
	}
	
	/**
	 * This method used to return the Image Server Base URL.
	 *Example: http://server/imageserver/plumtree
	 *
	 * @param boolean true implies secure
	 * @return String the image server base URL.
	 */
	public static final String GetImageServerBaseURL(boolean bSecure)
	{
		if (bSecure)
		{
			return m_sSecureImageServerBaseURL;
		} 
		else
		{
			return m_sNonSecureImageServerBaseURL;
		}
	}

	/**
	 * This method used to return the Image Server Base URL.
	 *Example: http://server/imageserver/plumtree
	 *
	 * @param AActivitySpace the space that this image is in.
	 * @return String the image server base URL.
	 */
	public static final String GetImageServerBaseURL(AActivitySpace asOwner)
	{
		return GetImageServerBaseURL(asOwner.GetIsSecuredSpace());
	}

	/**
	 * This method used to return the Shared Image Server Base URL.
	 *
	 * @param boolean true implies secure
	 * @return String the Shared image server base URL.
	 */
	public static final String GetSharedImageServerBaseURL(boolean bSecure)
	{
		if (bSecure)
		{
			return m_sSecureSharedImageServerBaseURL;
		} else
		{
			return m_sNonSecureSharedImageServerBaseURL;
		}
	}

	/**
	 * This method used to return the Shared Image Server Base URL.
	 *
	 * @param AActivitySpace the space that this image is in.
	 * @return String the Shared image server base URL.
	 */
	public static final String GetSharedImageServerBaseURL(AActivitySpace asOwner)
	{
		return GetSharedImageServerBaseURL(asOwner.GetIsSecuredSpace());
	}

	/**
	 * This method used to return the Shared Image Server Images URL.
	 *
	 * @param boolean true implies secure
	 * @return String the Shared image server Images URL.
	 */
	public static final String GetSharedImageServerImagesURL(boolean bSecure)
	{
		if (bSecure)
		{
			return m_sSecureSharedImageServerImagesURL;
		} else
		{
			return m_sNonSecureSharedImageServerImagesURL;
		}
	}

	/**
	 * This method used to return the Shared Image Server Images URL.
	 *
	 * @param AActivitySpace the space that this image is in.
	 * @return String the Shared image server Images URL.
	 */
	public static final String GetSharedImageServerImagesURL(AActivitySpace asOwner)
	{
		return GetSharedImageServerImagesURL(asOwner.GetIsSecuredSpace());
	}

	/**
	 * This method used to return the Shared Image Server JavaScript URL.
	 *
	 * @param boolean true implies secure
	 * @return String the Shared image server JavaScript URL.
	 */
	public static final String GetSharedImageServerJavaScriptURL(boolean bSecure)
	{
		if (bSecure)
		{
			return m_sSecureSharedImageServerJavaScriptURL;
		} else
		{
			return m_sNonSecureSharedImageServerJavaScriptURL;
		}
	}

	/**
	 * This method used to return the Shared Image Server JavaScript URL.
	 *
	 * @param AActivitySpace the space that this image is in.
	 * @return String the Shared image server JavaScript URL.
	 */
	public static final String GetSharedImageServerJavaScriptURL(AActivitySpace asOwner)
	{
		return GetSharedImageServerJavaScriptURL(asOwner.GetIsSecuredSpace());
	}

	/**
	 * This method used to return the Shared Image Server Styles URL.
	 *
	 * @param boolean true implies secure
	 * @return String the Shared image server Styles URL.
	 */
	public static final String GetSharedImageServerStylesURL(boolean bSecure)
	{
		if (bSecure)
		{
			return m_sSecureSharedImageServerStylesURL;
		} else
		{
			return m_sNonSecureSharedImageServerStylesURL;
		}
	}

	/**
	 * This method used to return the Shared Image Server Styles URL.
	 *
	 * @param AActivitySpace the space that this image is in.
	 * @return String the Shared image server Styles URL.
	 */
	public static final String GetSharedImageServerStylesURL(AActivitySpace asOwner)
	{
		return GetSharedImageServerStylesURL(asOwner.GetIsSecuredSpace());
	}
	
	/**
	 * Returns the base URL to the style sheets on the image server.
	 *
	 * @param AActivitySpace current AS
	 * 
	 * @return String URL to the style sheets on the image server
	 */
	public static final String GetStyleSheetsBaseURL(AActivitySpace asOwner)
	{
		return GetImageServerBaseURL(asOwner.GetIsSecuredSpace()) + "common/public/css/";
	}
	
	/**
	 * Returns the base URL to the style sheets on the image server.
	 *
	 * @param bSecure security of the URL
	 * 
	 * @return String URL to the style sheets on the image server
	 */
	public static final String GetStyleSheetsBaseURL(boolean bSecure)
	{
		return GetImageServerBaseURL(bSecure) + "common/public/css/";
	}
	
	/**
	 * This method used to return the Portal Image Server Base URL.
	 *
	 * @param boolean true implies secure
	 * @return String the Portal image server base URL.
	 */
	public static final String GetPortalImageServerBaseURL(boolean bSecure)
	{
		if (bSecure)
		{
			return m_sSecurePortalImageServerBaseURL;
		} else
		{
			return m_sNonSecurePortalImageServerBaseURL;
		}
	}

	/**
	 * This method returns the Image Server Connection URL. This URL is used anytime
	 * the Portal Server needs to talk to the Image Server. This happens when
	 * retrieving JSRegistry information, for example.
	 *
	 * @return String Portal image server connection URL.
	 */
	public static final String GetPortalImageServerConnectionURL(AActivitySpace asOwner)
	{
		return ConfigHelper.GetConfigVariableAsString(ConfigConstants.MAIN_URLS_SECTION, ConfigConstants.IMAGE_SERVER_CONNECTION_URL, ConfigConstants.VALUE_ATTRIBUTE, asOwner);
	}

	/**
	 * This method used to return the Portal Image Server Base URL.
	 *
	 * @param AActivitySpace the space that this image is in.
	 * @return String the Portal image server base URL.
	 */
	public static final String GetPortalImageServerBaseURL(AActivitySpace asOwner)
	{
		return GetPortalImageServerBaseURL(asOwner.GetIsSecuredSpace());
	}

	/**
	 * This method used to return the Portal Image Server Images URL.
	 *
	 * @param boolean true implies secure
	 * @return String the Portal image server Images URL.
	 */
	public static final String GetPortalImageServerImagesURL(boolean bSecure)
	{
		if (bSecure)
		{
			return m_sSecurePortalImageServerImagesURL;
		} else
		{
			return m_sNonSecurePortalImageServerImagesURL;
		}
	}

	/**
	 * This method used to return the Portal Image Server Images URL.
	 *
	 * @param AActivitySpace the space that this image is in.
	 * @return String the Portal image server Images URL.
	 */
	public static final String GetPortalImageServerImagesURL(AActivitySpace asOwner)
	{
		return GetPortalImageServerImagesURL(asOwner.GetIsSecuredSpace());
	}

	/**
	 * This method used to return the Portal Image Server JavaScript URL.
	 *
	 * @param boolean true implies secure
	 * @return String the Portal image server JavaScript URL.
	 */
	public static final String GetPortalImageServerJavaScriptURL(boolean bSecure)
	{
		if (bSecure)
		{
			return m_sSecurePortalImageServerJavaScriptURL;
		} else
		{
			return m_sNonSecurePortalImageServerJavaScriptURL;
		}
	}

	/**
	 * This method used to return the Portal Image Server JavaScript URL.
	 *
	 * @param AActivitySpace the space that this image is in.
	 * @return String the Portal image server JavaScript URL.
	 */
	public static final String GetPortalImageServerJavaScriptURL(AActivitySpace asOwner)
	{
		return GetPortalImageServerJavaScriptURL(asOwner.GetIsSecuredSpace());
	}

	/**
	 * This method used to return the Portal Image Server Help URL.
	 *
	 * @param boolean true implies secure
	 * @return String the Portal image server Help URL.
	 */
	public static final String GetPortalImageServerHelpURL(boolean bSecure)
	{
		if (bSecure)
		{
			return m_sSecurePortalImageServerHelpURL;
		} else
		{
			return m_sNonSecurePortalImageServerHelpURL;
		}
	}

	/**
	 * This method used to return the Portal Image Server Help URL.
	 *
	 * @param AActivitySpace the space that this image is in.
	 * @return String the Portal image server Help URL.
	 */
	public static final String GetPortalImageServerHelpURL(AActivitySpace asOwner)
	{
		return GetPortalImageServerHelpURL(asOwner.GetIsSecuredSpace());
	}

	
	/**
	 * @deprecated Use GetCommonJSIncludes(String _strComponentName, XPLocale locale, boolean _bIsSecureImgSvr) instead
	 * 
	 * @param _strComponentName
	 * @param strLang
	 * @param _bIsSecureImgSvr
	 * @return
	 */
	public static final HTMLElementCollection GetCommonJSIncludes(String _strComponentName, String strLangID, boolean _bIsSecureImgSvr)
	{
		return  GetCommonJSIncludes(_strComponentName, strLangID,  _bIsSecureImgSvr, JSComponent.INCLUDE_ALL);
	}
	
	/**
	 * @deprecated GetCommonJSIncludes(String _strComponentName, XPLocale locale, boolean _bIsSecureImgSvr, String strIncludeType)
	 * 
	 * @param _strComponentName
	 * @param locale
	 * @param _bIsSecureImgSvr
	 * @param strIncludeType
	 * @return
	 */
	public static final HTMLElementCollection GetCommonJSIncludes(String _strComponentName, String strLangID, boolean _bIsSecureImgSvr, String strIncludeType)
	{
		XPLocale locale = new XPLocale(strLangID);
		
		IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());
  
  		JSRegistry jsReg = null;
  		if (_bIsSecureImgSvr)
  		{
  			jsReg = (JSRegistry) application.GetCachingManager()
  				.GetEntry("ptSecureJSReg");
  		}
  		else
  		{
  			jsReg = (JSRegistry) application.GetCachingManager()
  				.GetEntry("JSRegistry");
  		}
  
  
  		String strHTMLLinks = jsReg.GetJSComponent(_strComponentName).GenIncludes(strIncludeType, locale );
  
  		HTMLElementCollection myLinksElement = null;
  
  		try
  		{
  			myLinksElement = new HTMLElementCollection();
  			myLinksElement.AddInnerHTMLString(strHTMLLinks);
  		}
  		catch (Exception e)
  		{
  			throw XPException.GetInstance(e);
  		}
  
  		return myLinksElement;
		
	}
	
	
	/**
	 * @deprecated No use anymore, always uses LATEST
	 * Always returns "window.". 
	 * @param strComponentName Name of the jscomponent with . Example: "jsutil", "jsxml"
	 * @return	Version "window."
	 */
	public static String GetJSIncluderComponentVersion(String strJSComponentName)
	{
		
		return "LATEST";
	}

	/**
	 * 
	 * @param strJSComponentName JSComponent name included through JSIncluder Example: " jsxml" or "jsutil"
	 * @return
	 */
	public static  String GetJSIncluderVersionPrefix(String strJSComponentName)
	{
		return "window.";
		
	}
	
	public static String GetJSIncluderComponentPath(String strJSComponentName, String strImageServerURL)
	{
		 return JSIncluder.GetIncluder(strImageServerURL).GetBaseURL(strJSComponentName );
	}
	
	/**
	 * HTMLElement with a script blocks including the javascript files for the given jscomponent. 
	 * 
	 * @param strJSComponentName	JSComponent name. For example "jsxml" or "jsutil", the common components have constants defined in ConfigHelper.
	 * @param owner	Current activityspace
	 * @return 
	 */
	public static HTMLElementCollection GetJSIncluderJSComponentInclude(String strJSComponentName, AActivitySpace owner)
	{
		String strLocaleID = (String) PersonalSettingsHelper.GetPersonalSettingValue(PersonalSettingsHelper.LOCALE, owner);
		String strImageServerURL = GetImageServerRootURL(owner);
		
		XPLocale locale = new XPLocale(strLocaleID);
		HTMLGenericElement el = new HTMLGenericElement( JSIncluder.GetIncluder(strImageServerURL).GenerateIncludes(strJSComponentName, locale));
		HTMLElementCollection ret = new HTMLElementCollection();
		ret.AddInnerHTMLElement(el);
		return ret;
	}
	

	
	/**
	 * HTMLElement with a script blocks including the javascript files for the given jscomponent. 
	 * 
	 * @param strJSComponentName	JSComponent name. For example "jsxml" or "jsutil", the common components have constants defined in ConfigHelper.
	 * @param owner	Current activityspace
	 * @return 
	 */
	public static HTMLElementCollection GetJSIncluderJSComponentInclude(String strJSComponentName, boolean bEnableDebug, AActivitySpace owner,  XPLocale locale)
	{
		String strImageServerURL = GetImageServerRootURL(owner);
		JSIncluder includer = JSIncluder.GetIncluder(strImageServerURL);
		includer.SetDebugEnabled(bEnableDebug);
		HTMLElementCollection ele = new HTMLElementCollection();
		
		ele.AddInnerHTMLElement( new HTMLGenericElement( includer.GenerateIncludes(strJSComponentName, locale)) );
		
		return ele;
	
	}
	
	/**
	 * Helper method to access an int variable from portalconfig.xml.
	 *
	 * @param strSection strSection
	 * @param strSubSection strSubSection
	 * @param strAttribute strAttribute
	 * @param space the current space
	 *
	 * @return int the requested variable
	 */
	public static int GetConfigVariableAsInt(String strSection,
		String strSubSection, String strAttribute, AActivitySpace space)
	{
		return ((ConfigVarPack) space.GetApplication().GetVarPackManager()
				.GetVariablePackage(ConfigVarPack.VARPACK_ID)).GetConfigValueAsInt(strSection, strSubSection, strAttribute);
	}

	/**
	 * Helper method to access an int variable from portalconfig.xml.
	 *
	 * @param strSection strSection
	 * @param strSubSection strSubSection
	 * @param strAttribute strAttribute
	 * @param space the current space
	 *
	 * @return int the requested variable
	 * 
	 */
	public static int GetConfigVariableAsInt(String strSection,
		String strSubSection, String strAttribute, Application application)
	{
		try {
			return ((ConfigVarPack) application.GetVarPackManager()
			.GetVariablePackage(ConfigVarPack.VARPACK_ID)).GetConfigValueAsInt(strSection, strSubSection, strAttribute);
		} catch ( XPException e ) {
			throw new XPException("Unable to retrieve the requested variable: "+strAttribute, e);
		}
	}

	/**
	 * Helper method to access a String variable from portalconfig.xml.
	 *
	 * @param strSection
	 * @param strSubSection
	 * @param strAttribute
	 * @param owner Current Activityspace
	 *
	 * @return <code>String</code> with the requested variable value, or <code>null</code> if the variable is not found, or cannot be retrieved.
	 */
	public static String GetConfigVariableAsString(String strSection, String strSubSection, String strAttribute, AActivitySpace owner)
	{
			return ((ConfigVarPack) owner.GetApplication().GetVarPackManager()
					.GetVariablePackage(ConfigVarPack.VARPACK_ID)).GetConfigValueAsString(strSection, strSubSection, strAttribute);
		
	}

	/**
	 * Helper method to access a String variable from portalconfig.xml.
	 *
	 * @param strSection
	 * @param strSubSection
	 * @param strAttribute
	 * @param application the current IApplication
	 *
	 * @return <code>String</code> with the requested variable value, or <code>null</code> if the variable is not found, or cannot be retrieved.
	 */
	public static String GetConfigVariableAsString(String strSection, String strSubSection, String strAttribute, IApplication app)
	{
		return ((ConfigVarPack) app.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID)).GetConfigValueAsString(strSection,
				strSubSection, strAttribute);
	}

	/**
	 * Helper method to access a String variable from portalconfig.xml.
	 *
	 * @param strSection strSection
	 * @param strSubSection strSubSection
	 * @param strAttribute strAttribute
	 * @param space the current space
	 *
	 * @return String the requested variable
	 */
	public static Object[] GetConfigVariables(String strSection,
		String strSubSection, String strAttribute, AActivitySpace space)
	{
		return ((ConfigVarPack) space.GetApplication().GetVarPackManager()
		.GetVariablePackage(ConfigVarPack.VARPACK_ID)).GetConfigValues(strSection,
			strSubSection, strAttribute);
	}

	/**
	 * Helper method to access a String variable from portalconfig.xml.
	 *
	 * @param strSection strSection
	 * @param strSubSection strSubSection
	 * @param strAttribute strAttribute
	 * @param space the current space
	 *
	 * @return String the requested variable
	 */
	public static Object[] GetConfigVariables(String strSection,
		String strSubSection, String strAttribute, IApplication application)
	{
		return ((ConfigVarPack) application.GetVarPackManager()
		.GetVariablePackage(ConfigVarPack.VARPACK_ID)).GetConfigValues(strSection,
			strSubSection, strAttribute);
	}

	/**
	 * Returns full URL to the admin site without any question mark.
	 *
	 *
	 * @return String
	 */
	public static final String GetFullAdminSiteURL()
	{
		IApplication application;
		String sAdminSiteURL;

		application		   = ApplicationManager.GetInstance().GetApplication(s_strApplicationName);
		sAdminSiteURL     = ((ConfigVarPack) application.GetVarPackManager().GetVariablePackage(ConfigVarPack.VARPACK_ID)).GetConfigValueAsString("MainURLs", "AdminSiteBaseURL", "value");

		if (sAdminSiteURL.endsWith("/"))
		{
			// remove the trailing slash, if there is one.
			sAdminSiteURL = sAdminSiteURL.substring(0,sAdminSiteURL.length() -1);
		}

		return  sAdminSiteURL;
	}

	/**
	 *
	 *
	 * @return
	 */
	public static final XPResourceManager GetResourceManager()
	{
		IApplication application = ApplicationManager.GetInstance()
			.GetApplication(s_strApplicationName);

		XPResourceManager rm = (XPResourceManager) application.GetCachingManager()
			.GetEntry("ResourceManager");

		return rm;
	}

	/**
	 * Returns the security mode value from portalconfig.xml.
	 *
	 * @return int
	 */
	public static final int GetSecurityMode()
	{
		return m_nSecurityMode;
	}

	/**
	 * This method returns the class type description manager.
	 */
	public static final IClassTypeManager GetTypeDescrManager()
	{
		IApplication application = ApplicationManager.GetInstance().GetApplication(AppConstants.MAIN_APPLICATION_NAME.ToString());

		return (IClassTypeManager) application.GetCachingManager().GetEntry(AppWarmUp.KEY_CACHEMAN_CLASS_TYPE_MANAGER);
	}

	/**
	 * Returns true if this AS shoudl be accessed through HTTPS.
	 * Note: in security mode 2 and 3, true is always returned.
	 *
	 * @param _sASName
	 *
	 * @return boolean
	 */
	public static final boolean IsSpaceSecured(String _sASName)
	{
		boolean bIsSecure;

		switch (m_nSecurityMode)
		{
		case ConfigVarPack.SECURITY_MODE_SSL_CRITICAL_AS_ONLY:

			IApplication application;
			SecureASVarPack vpSecureAS;

			// JF- Get the Sercure AS var pack where we have the list of all the secured AS.
			application = ApplicationManager.GetInstance().GetApplication(s_strApplicationName);
			vpSecureAS = (SecureASVarPack) application.GetVarPackManager()
				.GetVariablePackage(SecureASVarPack.VARPACK_ID);

			if (vpSecureAS.IsSpaceSecured(_sASName))
			{
				bIsSecure = true;
			}
			else
			{
				bIsSecure = false;
			}

			break;

		case ConfigVarPack.SECURITY_MODE_SSL_ALL:
		case ConfigVarPack.SECURITY_MODE_SSL_ACCELERATOR:
			bIsSecure = true;

			break;

		default:
			bIsSecure = false;
		}

		return bIsSecure;
	}

	/**
	 * JF- Helper method to figure out if the login cookie should be secure or not.
	 * 
	 * @return boolean - true if it should be secure
	 */
	public static boolean IsLoginTokenSecure(String strCurrentRequestURL)
	{
		// JF- In mode 0, the user has the choice to browse http or https. 
		// We need to check the request URL to know if the cookie should be secure or no.
		// Bug 50181 - Similarly for WebDAV request (to avoid multiple prompts
		// for login when WebDAV is on security mode 1)
		if (ConfigVarPack.SECURITY_MODE_NO_SSL == ConfigHelper.GetSecurityMode() ||
			HTTPBasicAuthenticator.GetInstance().IsBasicAuthURL(strCurrentRequestURL)) 
		{
			return XPStringUtility.StartsWith(strCurrentRequestURL, HTTPS_SCHEME);
		}
		// JF- For the other modes, we just check if the Login AS is secure
		else
		{
			return ConfigHelper.IsSpaceSecured(LoginHandlers.LOGINSPACE);
		}
	}

	/**
	 * JF- Helper method to get the settings from the Config var pack and set
	 * the current platform.
	 *
	 * @param _vpConfig
	 */
	private static final void SetPlatform(ConfigVarPack _vpConfig)
	{
		s_bIsDotNet = _vpConfig.GetIsDotNet();
		s_bIsJava = _vpConfig.GetIsJava();
	}

	/**
	 * Returns whether or not the current platform is .NET.
	 *
	 * @return boolean true implies .NET
	 */
	public static final boolean GetIsDotNet()
	{
		return s_bIsDotNet;
	}

	/**
	 * Returns whether or not the current platform is Java.
	 *
	 * @return boolean true implies Java
	 */
	public static final boolean GetIsJava()
	{
		return s_bIsJava;
	}
}
