package com.plumtree.uiinfrastructure.statichelpers;

import com.plumtree.openfoundation.util.*;

/**
 * This class is a helper class to insert "In-Place Refresh" functionality
 * to your Activity Space.  Most of the helpers deal with inserting JS 
 * method calls and functions into your views and display pages.
 *
 * @author RickyW
 * @author Caiusv
 */
public class InPlaceRefreshHelper
{

    /**
     * Helper method to get the JavaScript call to sent an in place POST to
     * the server, assuming you're on a page that uses AFormDP.  This method
     * wraps in-place refresh functionality, allowing you to replace the 
     * innerHTML of the specified ID.  The repostAction will enable the server
     * to decide what content to return in the response.
     * 
     * @param _strURL The URL to post to, generally GetCurrentNormalizedBaseURL.
     * @param _refreshElementID The ID of the element to replace innerHTML.
     * @param _nRepostAction The repostAction for the server to decide content.
     * @param _strDocument The name of the document where the in-place refresh
     * 			happens. This should be identified if it ocurrs in another window
     * 			for example. A _strDocument value of null defaults to 'document'
     * 
     * @return The JavaScript method call for SendPOSTOnAFormDP.
     */
    public static final String GetJSSendPOSTOnAFormDP(String _strURL, 
            String _refreshElementID,
            int _nRepostAction,
            String _strDocument)
    {       
        XPStringBuilder sbMethodJS = new XPStringBuilder(300);
        
	    if (null == _strDocument)
	    {
	        _strDocument = "document";
	    }
        String strJSXMLPrefixString = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSXML);
	    if ( XPStringUtility.EndsWith(strJSXMLPrefixString, ".")  ) {
	    	strJSXMLPrefixString = XPStringUtility.Substring(strJSXMLPrefixString, 0, strJSXMLPrefixString.length()-1);
	    }
	    
	    sbMethodJS.Append("    var myForm = "+_strDocument+".mainform;\n");
		// TODOCR: What is this magic number 6?  Why don't we just calculate the value of myRepostAction
		// here in Java instead of making javascript do it?
	    sbMethodJS.Append("    var myRepostAction = (" + _nRepostAction + ") ? " + _nRepostAction +" : 6;\n");
	    sbMethodJS.Append("    myForm['in_hi_PostToSelf'].value = " + _nRepostAction + ";\n");
        sbMethodJS.Append("    var formRequest = new " + strJSXMLPrefixString + ".PTHTTPPOSTRequest");
        sbMethodJS.Append("(");
        sbMethodJS.Append("'").Append(_strURL).Append("', ");
        sbMethodJS.Append("myForm, ");
        sbMethodJS.Append(_strDocument + ".getElementById('"+_refreshElementID+"'), ");
        sbMethodJS.Append("'sync', ");
        sbMethodJS.Append("myRepostAction");
        sbMethodJS.Append(");\n");
        sbMethodJS.Append("    formRequest.invoke();");
        
        
        return sbMethodJS.ToString();
    }
    
    /**
     * Helper method to get the JavaScript call to sent an in place POST to
     * the server, assuming you're on a page that uses AFormDP.  This method
     * wraps in-place refresh functionality, allowing you to replace the 
     * innerHTML of the specified ID.  The repostAction will enable the server
     * to decide what content to return in the response. This method calls the
     * four argument version of the method supplying a document argument of null.
     * 
     * @param _strURL The URL to post to, generally GetCurrentNormalizedBaseURL.
     * @param _refreshElementID The ID of the element to replace innerHTML.
     * @param _nRepostAction The repostAction for the server to decide content.
     * 
     * @return The JavaScript method call for SendPOSTOnAFormDP.
     */
    public static final String GetJSSendPOSTOnAFormDP(String strURL, 
            String refreshElementID,
            int nRepostAction)
    {
        return GetJSSendPOSTOnAFormDP(strURL, refreshElementID, nRepostAction, null);   
    }

    /**
     * Helper method to get the JavaScript call to sent an in place GET to
     * the server, assuming you're on a page that uses AFormDP.  This method
     * wraps in-place refresh functionality, allowing you to replace the 
     * innerHTML of the specified ID.  The repostAction will enable the server
     * to decide what content to return in the response.
     * 
     * @param strURL The URL to post to, generally GetCurrentNormalizedBaseURL.
     * @param refreshElementID The ID of the element to replace innerHTML.
     * @param nRepostAction The repostAction for the server to decide content.
     * 
     * @return The JavaScript method call for SendGETOnAFormDP.
     */
    public static final String GetJSSendGETOnAFormDP(String strURL, String refreshElementID, int nRepostAction)
    {       
        XPStringBuilder sbMethodJS = new XPStringBuilder(300);

	    String strJSXMLPrefixString = ConfigHelper.GetJSIncluderVersionPrefix(ConfigHelper.JSCOMPONENT_JSXML);
	    if ( XPStringUtility.EndsWith(strJSXMLPrefixString, ".")  ) {
	    	strJSXMLPrefixString = XPStringUtility.Substring(strJSXMLPrefixString, 0, strJSXMLPrefixString.length()-1);
	    }
	    
	    sbMethodJS.Append("    var myForm = document.mainform;\n");
	    // TODOCR: What is this magic number 6?  Why don't we just calculate the value of myRepostAction
	    // here in Java instead of making javascript do it?
	    sbMethodJS.Append("    var myRepostAction = (" + nRepostAction + ") ? " + nRepostAction +" : 6;\n");
	    sbMethodJS.Append("    myForm['in_hi_PostToSelf'].value = " + nRepostAction + ";\n");
        sbMethodJS.Append("    var formRequest = new " + strJSXMLPrefixString + ".PTHTTPGETRequest");
        sbMethodJS.Append("(");
        sbMethodJS.Append("'").Append(strURL).Append("', ");
        sbMethodJS.Append("myForm, ");
        sbMethodJS.Append("document.getElementById(\""+refreshElementID+"\"), ");
        sbMethodJS.Append("'sync', ");
        sbMethodJS.Append("myRepostAction");
        sbMethodJS.Append(");\n");
        sbMethodJS.Append("    formRequest.invoke();");
        
        return sbMethodJS.ToString();
    }

}
