package com.plumtree.uiinfrastructure.statichelpers;

import com.plumtree.openlog.*;
import com.plumtree.openfoundation.io.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.xml.dom.*;



/**
 * @author NaveenP
 * 
 * Loads the version.xml and gives us a String to print with the version info.
 */
public class VersionHelper
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.statichelpers.VersionHelper");

	public static final String RELEASE 		= "release";
	public static final String MAJOR 		= "major";
	public static final String MINOR 		= "minor";
	public static final String SERVICEPACK	= "servicepack";
	public static final String CHANGELIST	= "changelist";
	public static final String DATE			= "date";
	public static final String TIME			= "time";
	
	private static String m_strRelease;
	private static String m_strMajor;
	private static String m_strMinor;
	private static String m_strServicePack;
	private static String m_strChangelist;
	private static String m_strDate;
	private static String m_strTime;
	private static String m_strVersionMessage = "Version.xml has not been loaded.";
	
	
	/** The path to the folder in which x_config.xml resides. */
	protected String m_strConfigFilePath = "";
	
	public VersionHelper(String _strConfigFilePath)
	{
		m_strConfigFilePath = _strConfigFilePath;
	}
	
	/**
	 * Loads the version.xml and calls private function to get the info.
	 */
	public void LoadVersionXML()
	{
		String strFileSeparator = XPFileSystemObject.GetPathSeparator();

		String strVersionFileName = "version.xml";
		
		String strVersionPath = m_strConfigFilePath + strFileSeparator + strVersionFileName;
		strVersionPath = GeneralHelpers.FormatPath(strVersionPath, strFileSeparator, false);

		
			log.Info("Loading Version XML document from " + strVersionPath);
		

		XPDocument xmlDoc = XPBuilder.LoadDocument(strVersionPath);

		if (xmlDoc == null)
		{
			
				log.Error("Could not load Version XML document from " + strVersionPath);
			

			throw new XPException("Could not load Version XML document from " + strVersionPath);
		}

		LoadSettingsIntoStaticVars(xmlDoc);

		
			log.Info("Finished loading Version XML document.");
		
	}

	/**
	 * Helper method to load the settings from version.xml and put them in a XPHashtable.
	 * Also creates the version message.
	 */
	private void LoadSettingsIntoStaticVars(XPDocument xmlDoc)
	{
		if (xmlDoc == null)
		{
			throw new XPException("Version XML document not found.");
		}

		XPElement nTopNode = xmlDoc.GetDocumentElement();

		
			log.Info("The top root node of version.xml is " + nTopNode.GetNodeName());
		

		if (nTopNode == null)
		{
			throw new XPException("Version XML document top node not found.");
		}

		XPNodeList nlFirstLevel = nTopNode.GetChildNodes();
		if (nlFirstLevel == null)
		{
			throw new XPException("Version XML document first level nodes not found.");
		}
		
		XPHashtable tableVersionValues = new XPHashtable();
		String sName = "", sValue = "";
		for(int i = 0; i < nlFirstLevel.GetLength(); i++)
		{
			XPNode node = nlFirstLevel.Item(i);
			
			if (node instanceof XPElement)
			{
				sName = node.GetNodeName();
				XPNode child = node.GetFirstChild();
				sValue = ((XPText) child).GetData();
				tableVersionValues.PutElement(sName, sValue);
			} 
		}
		
		m_strRelease 		= (String) tableVersionValues.GetElement(RELEASE);
		m_strMajor 			= (String) tableVersionValues.GetElement(MAJOR);
		m_strMinor	 		= (String) tableVersionValues.GetElement(MINOR);
		m_strServicePack	= (String) tableVersionValues.GetElement(SERVICEPACK);
		m_strChangelist		= (String) tableVersionValues.GetElement(CHANGELIST);
		m_strDate 			= (String) tableVersionValues.GetElement(DATE);
		m_strTime 			= (String) tableVersionValues.GetElement(TIME);
		
		m_strVersionMessage = "Portal Version: " + m_strRelease + ", Changelist: " + m_strChangelist + ", Build Date: " + m_strDate + " at " + m_strTime;  
	}
	
	/**
	 * Get the version message to be printed at the bottom of each page.
	 * 
	 * @return String
	 */
	public static String GetVersionMessage()
	{
		return m_strVersionMessage;	
	}
	
    public static String GetMajorVersion()
    {
        return m_strMajor; 
    }
    
    public static String GetMinorVersion()
    {
        return m_strMinor; 
    }
    
    public static String GetServicePackVersion()
    {
        return m_strServicePack; 
    }
    
    public static String GetChangelistVersion()
    {
        return m_strChangelist; 
    }
}
