package com.plumtree.uiinfrastructure.tips;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.web.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.openlog.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 *     This is an abstract model extended by various concrete error model.
 *
 * @author Beng
 */
public class TipsModel implements IModel, ITipsModelRO
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.tips.TipsModel");

	// TODO we should probably put these constants somewhere
	// so they can be used consistently.  this is being used
	// in HTMLConstructHelpers too.
	public static final String STR_508_HELP_DIR = "508";
	public static final String STR_STANDARD_HELP_DIR = "std";

	public static final String STR_MVC_CLASS_NAME = "TipsModel";

	// JF- Associated Activity Space
	protected AActivitySpace m_asOwner;
	private HTMLElement eDefault	  = null;
	private HTMLElement eSecondaryTip = null;
	private HTMLElement eTip		  = null;
	private String sDefaultTipTitle  = null;
	private String sSecondaryTipTile = null;
	private String sTipTitle		 = null;
	private String sTipGeneratingClass = null;
	private boolean bDoDisplay = true;
	private boolean bDoDisplaySet = false;
	private boolean bDisplayGeneratingClass = false;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupTempData()
	 */
	public void CleanupTempData()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IModel#CleanupAllData()
	 */
	public void CleanupAllData()
	{
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new TipsModel();
	}

	/**
	 *
	 *
	 * @return
	 */
	public HTMLElement GetDefaultTip()
	{
		return eDefault;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetDefaultTipTitle()
	{
		return sDefaultTipTitle;
	}

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetDoDisplayTipsBox()
	{
		
			log.Debug("GetDoDisplayTipsBox()");
		

		if (!bDoDisplaySet)
		{
			ISessionManager sm = this.m_asOwner.GetSubSession();

			// This is cached on the session so that users can
			// turn tips on or off and it will follow them
			// across Activity Spaces.
			// NOTE: DonH guessed this from looking at the source.  May not be true.
			Object o   = sm.GetAttribute("displayTips");

			if (o != null)
			{
				bDoDisplay = XPConvert.ToBoolean(o);
			}
			else
			{
				bDoDisplay = true;
			}

			bDoDisplaySet = true;
		}

		return bDoDisplay;
	}


	public void SetGeneratingClass( String s )
	{
		sTipGeneratingClass = s;
	}

	public String GetGeneratingClass()
	{
		return sTipGeneratingClass;
	}


	public boolean GetDisplayGeneratingClass()
	{
		return bDisplayGeneratingClass;
	}

	public void SetDisplayGeneratingClass( boolean b )
	{
		bDisplayGeneratingClass = b;
	}


	/**
	 * @see com.plumtree.xpshared.activityspace.IModel#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 *
	 *
	 * @return
	 */
	public HTMLElement GetSecondaryTip()
	{
		return eSecondaryTip;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetSecondaryTipTitle()
	{
		return sSecondaryTipTile;
	}

	/**
	 *
	 *
	 * @return
	 */
	public HTMLElement GetTip()
	{
		return eTip;
	}

	/**
	 *
	 *
	 * @return
	 */
	public String GetTipTitle()
	{
		return sTipTitle;
	}

	/**
	 * Inits the Model with the Activity Space.
	 *
	 * @param parent
	 */
	public void Init(AActivitySpace parent)
	{
		m_asOwner = parent;

		this.SetDefaultTipTitle(this.m_asOwner.GetString( 1591, ResourceConstants.PORTALBROWSINGMSGS ));

		HTMLElement eDefaultTip = new HTMLElementCollection();

		HTMLElement e;


		e = new HTMLGenericElement( "Default Tip" );


		try
		{
			eDefaultTip.AddInnerHTMLElement(e);
		}
		catch (Exception ex)
		{
		}

		this.SetDefaultTip(eDefaultTip);


	}

	/**
	 *
	 *
	 * @param e
	 */
	public void SetDefaultTip(HTMLElement e)
	{
		eDefault = e;
	}

	/**
	 *
	 *
	 * @param s
	 */
	public void SetDefaultTipTitle(String s)
	{
		sDefaultTipTitle = s;
	}

	/**
	 *
	 *
	 * @param b
	 */
	public void SetDoDisplayTipsBox(boolean b)
	{
		bDoDisplay = b;

		ISessionManager sm = this.m_asOwner.GetSubSession();

		// This is cached on the session so that users can
		// turn tips on or off and it will follow them
		// across Activity Spaces.
		// NOTE: DonH guessed this from looking at the source.  May not be true.
		sm.SetAttribute("displayTips", new Boolean(b));
	}

	/**
	 *
	 *
	 * @param e
	 */
	public void SetSecondarTip(HTMLElement e)
	{
		HTMLElement eLocalized = GetModifiedTip( e );
		eSecondaryTip = eLocalized;
	}

	/**
	 *
	 *
	 * @param s
	 */
	public void SetSecondarTipTitle(String s)
	{
		sSecondaryTipTile = s;
	}

	/**
	 *
	 *
	 * @param e
	 */
	public void SetTip(HTMLElement e)
	{
		HTMLElement eLocalized = GetModifiedTip( e );
		eTip = eLocalized;
	}

	private HTMLElement GetModifiedTip( HTMLElement e )
	{
		//	todo, this is not done correctly.  There's no guarantee that
		//	e is ready to display.  This conversion must happen
		//	at display time.

		String sTip = e.GetDisplayString();


		String sHelpBaseUrl = ConfigHelper.GetPortalImageServerHelpURL( this.m_asOwner );

		String sLang = null;
		String sAccessStyle = null;

		try
		{
			sLang = (String) PersonalSettingsHelper.GetPersonalSettingValue( PersonalSettingsHelper.LANGUAGE, this.m_asOwner );
		}
		catch( Exception ex )
		{
			sLang = "en";
		}

		try
		{
			if (m_asOwner.GetIsAccessStyleStandard())
			{
				sAccessStyle = TipsModel.STR_STANDARD_HELP_DIR;
			}
			else
			{
				// TODO: This is not the correct way to support 508 Help.
				// This is not a priority because tips are not used in 508.
				// This should link to the HelpFrame to provide a back link.
				sAccessStyle = TipsModel.STR_508_HELP_DIR; 
			} 			
		}
		catch(Exception ex)
		{
			sAccessStyle = TipsModel.STR_STANDARD_HELP_DIR;
		}

		XPStringBuilder sb = new XPStringBuilder( 300 );
		
		sHelpBaseUrl = XPStringUtility.ForceEndsWith(sHelpBaseUrl, "/");

		sb.Append( sHelpBaseUrl );
		sb.Append( sAccessStyle );
		sb.Append( "/");
		sb.Append( sLang );
		sb.Append( "/" );

		String sHelpUrl = sb.ToString();

		sb = new XPStringBuilder( 300 );
		int i;

		// TODOCR: This is not a good way to do this and should be redone.
		String sPlaceHolder = "$$HBU$$";
		String sLastBit = sTip;

		i = sTip.indexOf( sPlaceHolder );

		while( i != -1 )
		{
			sb.Append( sTip.substring( 0, i ) );
			sb.Append( sHelpUrl );

			sTip = sTip.substring( i + sPlaceHolder.length(), sTip.length() );

			sLastBit = sTip;

			i = sTip.indexOf( sPlaceHolder );
		}

		sb.Append( sLastBit );

		HTMLGenericElement eLocalized = new HTMLGenericElement( sb.ToString() );

		return eLocalized;
	}

	/**
	 *
	 *
	 * @param s
	 */
	public void SetTipTitle(String s)
	{
		sTipTitle = s;
	}

	public void DisableTips()
	{
		// This will cause the TipsView to display nothing.
		eTip = null;
	}
}

