package com.plumtree.uiinfrastructure.tree;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * @author Don Hayler
 */
public abstract class ATreeAS extends AFormAS implements IFormAS
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.tree.ATreeAS");

	/** storage name for tree model (no matter what type of model it is) */
	public static final String TREEMODEL = "TreeModel";

	/** storage name for ClosePopUpDP (used for closing the TreeDP only) */
	public static final String TREECLOSEPOPUP = "TreeClosePopUp";

	/** anchor name for tree bar. */
	public static final String TREE_BAR_ANCHOR = "treeBarAnchor";

	/**
	 * The maximum length of names of objects when displayed.
	 * This is actually the max length a word can be before the
	 * entire string gets truncated.
	 **/
	public static final int MAX_DISPLAYED_NAME_LENGTH = 40;

	/** Non-localized Strings */

	// We are in a frame, so we need to call parent, not window
	public static final String JS_PARENT_CLOSE = "parent.close();\n";

	// member variables on objects returned by Javascript submit call

	/** The name of the javascript array member variable for the object name */
	public static final String OBJECT_NAME	    = "Name";

	/** The name of the javascript array member variable for the object id */
	public static final String OBJECT_OBJECT_ID = "ObjectID";

	/** The name of the javascript array member variable for the class id */
	public static final String OBJECT_CLASS_ID = "ClassID";

	///////////////////
	// Abstract Methods
	///////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.AActivitySpace#GetName()
	 */
	abstract public String GetName();

	/**
	 * This method is called at the end of every HTTP request by the Interpreter.
	 * This method then calls IModel.CleanupTempData() on all IModels registered
	 * using RegisterModel().
	 *
	 * This method is intended for cleaning up temporary data that is only used
	 * for the lifetime of a single request, like DB query results.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.AActivitySpace#CleanupTempData()
	 */
	public final void CleanupTempData()
	{
		super.CleanupTempData();

		SetCurrentPage(GetPage(TreeFrameDP.STR_MVC_CLASS_NAME));
	}

	/**
	 * This method handles the ClosePopUpDP and passes everything else to the base
	 * implementation.  It adds javascript to the ClosePopUpDP that sets the selected
	 * items on the opener window (if it's a remote tree) and calls a javascript
	 * function on the opener window.
	 *
	 * @see com.plumtree.uiinfrastructure.activityspace.AActivitySpace#GetPage(String)
	 */
	public IDisplayPage GetPage(String pageName)
	{
		if (ClosePopUpDP.STR_MVC_CLASS_NAME.equals(pageName))
		{
			// This is called when TreeDP is canceled.
			// This should only be called if Cancel was clicked, not Finish.
			ClosePopUpDP result = (ClosePopUpDP) super.GetPage(pageName);

			try
			{
				HTMLScript myScript = new HTMLScript("text/javaScript");

				// We are in a frame, so we need to call parent, not window
				myScript.AddInnerHTMLString(JS_PARENT_CLOSE);
				result.SetScript(myScript);
			}
			catch (HTMLException he)
			{
				
					log.Error(he, "Error generating JS for ClosePopUpDP.");
				
			}

			return result;
		}
		else if (TREECLOSEPOPUP.equals(pageName))
		{
			// This is called when TreeDP is finished.
			// This should only be called if Finish was clicked, not Cancel.
			ClosePopUpDP result = (ClosePopUpDP) super.GetPage(ClosePopUpDP.STR_MVC_CLASS_NAME);
			ITreeModelRO model  = (ITreeModelRO) GetModel(TREEMODEL);

			if (model != null)
			{
				String strOpenerFunction = model.GetParentFormSubmitFunction();

				int nSize = 20;

				if (model.GetSubmitMode() == TreeConstants.N_JAVASCRIPTSUBMIT)
				{
					nSize += 100;
				}

				if ((strOpenerFunction != null) &&
						(!"".equals(strOpenerFunction)))
				{
					nSize += 100;
				}

				XPStringBuilder sb = new XPStringBuilder(nSize);

				if (model.GetSubmitMode() == TreeConstants.N_JAVASCRIPTSUBMIT)
				{
					ITreeModelRO myModel	    = (ITreeModelRO) GetModel(TREEMODEL);
					XPArrayList mySelectedItems = myModel.GetSelectedItems();

					sb.Append("myArray = new Array(")
					.Append(mySelectedItems.GetSize()).Append(");\n");
					sb.Append("\n");

					// Make sure we use the unencoded version of the name here
					// (although it is still truncated)
					for (int i = 0; i < mySelectedItems.GetSize(); i++)
					{
						sb.Append("myArray[").Append(i).Append("] = new Object();\n");
						sb.Append("myArray[").Append(i).Append("].")
						.Append(OBJECT_NAME).Append(" = '")
						.Append(GeneralHelpers.EscapeJavascript(((TreeLeafNode) mySelectedItems.GetElement(i)).GetName()))
						.Append("';\n");
						sb.Append("myArray[").Append(i).Append("].")
						.Append(OBJECT_OBJECT_ID).Append(" = '")
						.Append(((TreeLeafNode) mySelectedItems.GetElement(i)).m_nObjectID)
						.Append("';\n");
						sb.Append("myArray[").Append(i).Append("].")
						.Append(OBJECT_CLASS_ID).Append(" = '")
						.Append(((TreeLeafNode) mySelectedItems.GetElement(i)).m_nClassID)
						.Append("';\n");
						sb.Append(GetTreeSpecificArrayDataToAppend(i));
						sb.Append("\n");
					}
				}

				// Add method to call on opening form, if it exists
				if ((strOpenerFunction != null) &&
						(!"".equals(strOpenerFunction)))
				{
					sb.Append("if ( parent.opener.window['").Append(JSHelper.escapeJS(strOpenerFunction))
					.Append("'] ) {\n");

					if (model.GetSubmitMode() == TreeConstants.N_JAVASCRIPTSUBMIT)
					{
						sb.Append("  parent.opener.window['").Append(JSHelper.escapeJS(strOpenerFunction))
						.Append("'](myArray);\n");
					}
					else
					{
						sb.Append("  parent.opener.window['").Append(JSHelper.escapeJS(strOpenerFunction))
						.Append("']();\n");
					}

					sb.Append("} else {\n");


					sb.Append("  alert('" +
						GetString(75, ResourceConstants.INFRASTRUCTURE) + "');\n");
					sb.Append("}\n");
					sb.Append("\n");
				}

				// We are in a frame, so we need to call parent, not window
				sb.Append(JS_PARENT_CLOSE);

				try
				{
					HTMLScript myScript = new HTMLScript("text/javaScript");
					myScript.AddInnerHTMLString(sb.ToString());
					result.SetScript(myScript);
				}
				catch (HTMLException he)
				{
					
						log.Error(he, "Error generating JS for TreeDP ClosePopUpDP.");
					
				}
			}

			return result;
		}

		return super.GetPage(pageName);
	}
	
	/**
	 * Override this method to attach additional information specific to your implementation of the tree.
	 * Below is an example of how to do this.
	 * 
	 * 		ITreeModelRO myModel	    = (ITreeModelRO) GetModel(TREEMODEL);
	 * 		XPArrayList mySelectedItems = myModel.GetSelectedItems();
	 * 
	 * 		XPStringBuilder sb = new XPStringBuilder();
	 * 		sb.Append("myArray[").Append(iSelectedItemIndex).Append("].")
	 * 			.Append(OBJECT_CLASS_ID).Append(" = '")
	 * 			.Append(((TreeLeafNode) mySelectedItems.GetElement(iSelectedItemIndex)).m_nClassID)
	 * 			.Append("';\n");
	 * 		return sb.ToString();
	 * 
	 * @param index of the selected Item
	 * @return String
	 */
	public String GetTreeSpecificArrayDataToAppend(int iSelectedItemIndex)
	{
		return "";
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.IRepostAS#GetRepostControlName()
	 */
	public String GetRepostControlName()
	{
		return TreeRepostControl.STR_MVC_CLASS_NAME;
	}

	/////////////////
	// Helper methods
	/////////////////

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.AActivitySpace#Init()
	 *
	 * Sub classes need to provide their own implementation that calls into this
	 * implementation and adds all of the MVC elements (other than DPs).  The model
	 * should be stored as this.TREEMODEL.
	 */
	public void Init()
	{
		super.Init();

		// Main Frame set
		// This must be registered first so it is the default page
		RegisterPage(new TreeFrameDP());

		// Header DP
		AFormDP myHeader = new TreeHeaderDP();
//		myHeader.SetShowFormHeader(true);
		myHeader.AddToValidationFunction(TreeHeaderDP.JAVASCRIPT_SUBMIT_TREE);
		RegisterPage(myHeader);

		// Main Tree DP
		AFormDP myPage = (AFormDP) new TreeDP();
		myPage.AddToValidationFunction(TreeView.JAVASCRIPT_SUBMIT_TO_PARENT);
		RegisterPage(myPage);

		// Close pop up
		RegisterPage(new ClosePopUpDP());

		// Turn on current page tracking.  This is necessary because the form header
		// calls GetFormTitle() on the current page, which it is assuming is the page
		// that it is in.
		SetTrackCurrentPage(true);
	}
}

