package com.plumtree.uiinfrastructure.tree;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.openfoundation.util.*;

/**
 * This interface provides read only access to the tree.
 *
 * @author Don Hayler
 */
public interface ITreeModelRO extends IModelRO
{
	/**
	 * Get the action sub-title for the tree popup
	 * I.E. select a user
	 *
	 * @return String the non-HTML Encoded action sub-title
	 */
	public String GetActionSubTitle();

	/**
	 * Gets whether or not help is enabled
	 *
	 * @return boolean true implies enabled.
	 */
	public boolean GetHelpEnabled();

	/**
	 * Whether or not users are allowed to click finish
	 * on trees without selecting anything.
	 *
	 * @return boolean true implies allow
	 */
	public boolean GetAllowEmptySubmission();

	/**
	 * This method returns whether or not the tree is displaying as
	 * a flat list, or as a tree.  Flat lists are displayed for
	 * smaller numbers of items.
	 *
	 * @result boolean true implies flat list
	 */
	public boolean GetDisplayingFlatList();

	/**
	 * This method returns whether or not the check all box in the
	 * tree should be pre-selected or not.
	 *
	 * @result boolean true implies check all box should be pre-selected.
	 */
	public boolean GetAllChecked();

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetMultipleSelect();

	/**
	 *
	 *
	 * @return
	 */
	public String GetOnClickFunction();

	/**
	 *
	 *
	 * @return
	 */
	public String GetParentFormName();

	/**
	 *
	 *
	 * @return
	 */
	public String GetParentFormSpaceID();

	/**
	 *
	 *
	 * @return
	 */
	public String GetParentFormSubmitFunction();

	///////////////////////
	// Methods for the view
	///////////////////////

	/**
	 *
	 *
	 * @return
	 */
	public boolean GetTreeIsEmpty();

	/**
	 *
	 *
	 * @return
	 */
	public TreeCompositeIterator GetRootTreeIterator();

	/**
	 *
	 *
	 * @return
	 */
	public int GetSelectMode();

	/**
	 *
	 *
	 * @return
	 */
	public int GetSelectOptions();

	/**
	 *
	 *
	 * @return
	 */
	public int GetLastExpandedNodeID();

	///////////////////////////////
	// Methods for the parent space
	///////////////////////////////

	/**
	 * This returns an array of the currently selected objects/folders in the tree.
	 *
	 * @return XPArrayList of TreeLeafNode objects
	 */
	public XPArrayList GetSelectedItems();

	/**
	 * This method returns whether or not to show the root folder.
	 *
	 * @result boolean true implies show root (default)
	 */
	public boolean GetShowRoot();

	/**
	 *
	 *
	 * @return
	 */
	public int GetSubmitMode();

	/**
	 * Get the title for the tree popup
	 *
	 * @return String the non-HTML Encoded title
	 */
	public String GetTitle();
}

