package com.plumtree.uiinfrastructure.tree;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

/**
 * This control is used to re-open a previously opened tree and refresh
 * the contents (which objects are selected, etc...).
 * 
 * @author Don Hayler
 */
public class RefreshTreeControl implements IControl
{
	/** name of the refresh tree control */
	public static final String STR_MVC_CLASS_NAME = "RefreshTree";
	protected ATreeAS m_asOwner;

	/** Internal data **/
	protected ITreeModel m_asModel;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#CheckActionSecurityAndExecute(XPHashtable)
	 */
	public Redirect CheckActionSecurityAndExecute(XPHashtable arguments)
	{
		String[] sValues;

		// Object ids to select (class id, object id pairs)
		sValues = (String[]) arguments.GetElement(AStartTreeControl.SELECTED_ITEMS);

		if (sValues != null)
		{
			// Check for referenced long strings
			String strSelectedObjects = TreeDataHelpers.UnReferenceQueryStringData
				(sValues, m_asOwner);

			if (strSelectedObjects == null)
			{
				// Use the normal data if there were no long strings referenced
				strSelectedObjects = sValues[0];
			}

			m_asModel.SetObjectIDsToSelect(GeneralHelpers.ConvertToInts(strSelectedObjects));
		}

		// Object ids to hide (class id, object id pairs)
		sValues = (String[]) arguments.GetElement(AStartTreeControl.HIDDEN_ITEMS);

		if (sValues != null)
		{
			// Check for referenced long strings
			String strHiddenObjects = TreeDataHelpers.UnReferenceQueryStringData
				(sValues, m_asOwner);

			if (strHiddenObjects == null)
			{
				// Use the normal data if there were no long strings referenced
				strHiddenObjects = sValues[0];
			}

			m_asModel.SetObjectIDsToHide(GeneralHelpers.ConvertToInts(strHiddenObjects));
		}

		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new RefreshTreeControl();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IControl#Init(IModel, AActivitySpace)
	 */
	public void Init(IModel model, AActivitySpace space)
	{
		m_asModel     = (ITreeModel) model;
		m_asOwner     = (ATreeAS) space;
	}
}

