package com.plumtree.uiinfrastructure.tree;

import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.statichelpers.*;
import com.plumtree.openlog.*;
import com.plumtree.xpshared.htmlconstructs.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * Display the action sub-title for the tree frame.
 * 
 * @author Don Hayler
 */
public class TreeBarView implements IView
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.uiinfrastructure.tree.TreeBarView");

	/** name of the tree bar view */
	public static final String STR_MVC_CLASS_NAME = "TreeBar";
	private AActivitySpace m_asOwner;

	/** internal data */
	private ITreeModelRO m_asModel;

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IManagedObject#Create()
	 */
	public Object Create()
	{
		return new TreeBarView();
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Display()
	 */
	public HTMLElement Display()
	{
		HTMLTable myTable = new HTMLTable();
		myTable.SetWidth("100%");
		myTable.SetBorder("0");
		myTable.SetCellSpacing("0");
		myTable.SetCellPadding("3");
		myTable.SetSummary("Action title bar table");

		HTMLTableRow myTableRow;
		HTMLTableCell myTableCell;

		try
		{
			// Title
			myTableRow = new HTMLTableRow();
			myTable.AddInnerHTMLElement(myTableRow);
			myTableRow.SetStyleClass(PTStyleClass.DIALOGUE_HEADER_BG);

			myTableCell = new HTMLTableCell();
			myTableRow.AddInnerHTMLElement(myTableCell);
			myTableCell.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
			myTableCell.SetHeight("20");

			myTableCell.SetWidth("80%");

			HTMLSpan mySpan = new HTMLSpan();
			myTableCell.AddInnerHTMLElement(mySpan);
			mySpan.SetStyleClass(PTStyleClass.DIALOGUE_SUBTITLE);

			mySpan.AddInnerHTMLString("&nbsp;");
			mySpan.AddInnerHTMLEncodedString(m_asModel.GetActionSubTitle());

			if (m_asModel.GetHelpEnabled())
			{
				myTableCell = new HTMLTableCell();
				myTableRow.AddInnerHTMLElement(myTableCell);
				myTableCell.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);
				myTableCell.SetHeight("20");
				myTableCell.SetWidth("20%");
				myTableCell.SetAlign("right");

				mySpan = new HTMLSpan();
				myTableCell.AddInnerHTMLElement(mySpan);
				mySpan.SetStyleClass(PTStyleClass.DIALOGUE_TEXT);

				mySpan.AddInnerHTMLElement(HTMLConstructHelpers.GetHelpAnchor(m_asOwner));
			}
		}
		catch (Exception e)
		{
			
				log.Error(e, "Display: The following error occured when trying to display the HTML.");
			
		}

		return myTable;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#DisplayJavascript()
	 */
	public HTMLScript DisplayJavascript()
	{
		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IView#Init(IModelRO, AActivitySpace)
	 */
	public void Init(IModelRO model, AActivitySpace parent)
	{
		m_asModel     = (ITreeModelRO) model;
		m_asOwner     = parent;
	}
}

