package com.plumtree.uiinfrastructure.tree;

/**
 * This class contains tree constants.
 */
public final class TreeConstants
{
	/** Select Modes */
	public static final int N_COMPOSITESELECT     = 1;


	public static final int N_LEAFSELECT		  = 2;


	public static final int N_LEAFCOMPOSITESELECT = 3;


	public static final int N_NOSELECT 			  = 4;


	/**
	 * Select Options:
	 * Sometimes leaf nodes can act as both leaves and composites.
	 * This mode limits selection of leaves to exclude those dual
	 * nodes (such as user groups).  Default behavior is to
	 * include these nodes in N_LEAFSELECT select mode (N_LOOSELEAF).
	 */
	public static final int N_LOOSELEAF = 1;
	public static final int N_STRICTLEAF = 2;

	/**
	 * Submit Modes:
	 *
	 * SERVERSUBMIT is for internal portal developers
	 *
	 * JAVASCRIPTSUBMIT is for projects external to the server,
	 * such as collab and content.
	 */
	public static final int N_SERVERSUBMIT     = 1;


	public static final int N_JAVASCRIPTSUBMIT = 2;

	/**
	 * Common string for javascript window.open window attributes.
	 * This should be used for all trees in the portal so we can
	 * maintain a common look and feel.
	 */
	private static final String STR_TREE_WINDOW_ATTRIBUTES = "location=no,menubar=no,resizable=yes,height=400,width=400";

	/**
	 * Common string for javascript window.open window attributes.
	 * This should be used for all trees in the portal so we can
	 * maintain a common look and feel.
	 */
	public static String GetJavascriptTreeWindowAttributes()
	{
		return STR_TREE_WINDOW_ATTRIBUTES;
	}
}

