package com.plumtree.uiinfrastructure.tree;

import com.plumtree.uiinfrastructure.common.*;
import com.plumtree.uiinfrastructure.form.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * The header displays common code and the header bar at
 * the top of the tree.
 * 
 * @author Don Hayler
 */
public class TreeHeaderDP extends AFormDP
{
	/** Tree Header object name */
	public static final String STR_MVC_CLASS_NAME = "TreeHeader";

	/** JS to submit the tree form in the body frame. */
	public static final String JAVASCRIPT_SUBMIT_TREE = "submitTreeForm";

	public static final String JAVASCRIPT_SUBMIT_TRIES = "nSubmitTries";

	/**
	 * @see com.plumtree.uiinfrastructure.form.AFormDP#GetFormPageType()
	 */
	public FormPageTypes GetFormPageType()
	{
		// default implementation
		return FormPageTypes.NORMAL_PAGE;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.AFormDP#GetFormTitle()
	 */
	public String GetFormTitle()
	{
		ITreeModelRO myModel = (ITreeModelRO) m_asOwner.GetModel(ATreeAS.TREEMODEL);

		// AFormDP requires this to be encoded
		return XPHttpUtility.HtmlEncode(myModel.GetTitle());
	}

	/**
	 * @see com.plumtree.uiinfrastructure.activityspace.IMVCObject#GetName()
	 */
	public String GetName()
	{
		return STR_MVC_CLASS_NAME;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.AFormDP#PageDisplay()
	 */
	public HTMLElement PageDisplay()
	{
		return null;
	}

	/**
	 * @see com.plumtree.uiinfrastructure.form.AFormDP#DisplayJavaScriptFromChild()
	 *
	 * This is a helper method that all classes that inherit from AFormDP can override
	 * if they like.  The method provides a hook into the head so that DP's can add javascript.
	 * Simply override and return and HTMLScript.  Also, it is best to return the results of the
	 * DisplayJavaScript functions from the appropriate views.
	 */
	protected HTMLScriptCollection DisplayJavaScriptFromChild()
	{
		HTMLScriptCollection scripts = new HTMLScriptCollection();

		XPStringBuilder sb = new XPStringBuilder(50);
		sb.Append("\n");
		sb.Append("var ").Append(JAVASCRIPT_SUBMIT_TRIES).Append(" = 0;\n");
		sb.Append("\n");
		sb.Append("function ").Append(JAVASCRIPT_SUBMIT_TREE).Append("()\n");
		sb.Append("{\n");

		// Check if body has been loaded yet, and retry later if it hasn't.
		sb.Append("  if (parent.").Append(FrameSetDP.BODY_FRAME).Append(".")
		.Append(TreeView.JAVASCRIPT_PAGE_LOADED).Append(" == false) {\n");
		sb.Append("    if (").Append(JAVASCRIPT_SUBMIT_TRIES).Append(" < 5) {\n");
		sb.Append("      ").Append(JAVASCRIPT_SUBMIT_TRIES).Append("++;\n");
		// The page is not loaded yet.  Try to run this function again in one second
		sb.Append("      setTimeout(\"").Append(JAVASCRIPT_SUBMIT_TREE).Append("()\", 1000);\n");
		sb.Append("    } else {\n");
		sb.Append("      ").Append(JAVASCRIPT_SUBMIT_TRIES).Append(" = 0;\n");
		sb.Append("    }\n");
		sb.Append("    return false;\n");
		sb.Append("  }\n");

		sb.Append("  parent.").Append(FrameSetDP.BODY_FRAME).Append(".")
		.Append(JAVASCRIPT_SUBMIT_FORM).Append("(")
		.Append(RepostControl.POSTTOSELF_ACTION_FINISH).Append(");\n");
		sb.Append("  return false;\n");
		sb.Append("}\n");

		HTMLScript myScript = new HTMLScript("text/javaScript");
		myScript.AddInnerHTMLString(sb.ToString());
		scripts.AddInnerHTMLElement(myScript);

		return scripts;
	}
}

