/*
 * Created on Jan 12, 2005
 */
package com.plumtree.uiinfrastructure.utils;

import com.plumtree.openfoundation.util.*;

/**
 * Generates a Javascript 1D array in a String.
 * Only the array itself is generated, no variable declaration or assignment or trailing semicolon.
 * <p>
 * Ex:
 * <code>
 * 	JSArrayList jsarray = new JSArrayList(3);
 *  	for ( int i = 0; i < 3; i++ )
 *  	{
 * 		jsarray.AddNewElement();
 * 		jsarray.SetValue(0, "val"+0)
 * 		jsarray.SetValue(1, "val"+1)
 * 		jsarray.SetValue(2, "val"+2)
 * 	}
 * 	jsarray.ToString();
 * </code>
 * <p>
 * Generates the string:
 * <p>
 * "[[val00, val01, val02],[val10, val11,val12],[val20,val21,val23]]"
 * 
 * 
 * @author robertz
 */
public class JS1DArrayBuilder
{
	
	private XPArrayList m_list;
	
	private int m_nCurrIndex = 0;

	
	/**
	 * 
	 * @param nColumnCount
	 */
	public JS1DArrayBuilder()
	{
		m_list = new XPArrayList();
		
	}
	
	/**
	 * 
	 * @return
	 */
	public int AddNewStringValueElement(String strValue)
	{
			return m_list.Add("\""+strValue+"\"");
	}
	
	/**
	 * 
	 * @param strValue
	 * @return
	 */
	public int AddNewReferenceElement(String strValue)
	{
			return m_list.Add(strValue);
	}
	
	/**
	 * 
	 * @return
	 */
	public String ToString()
	{
		XPStringBuilder buf = new XPStringBuilder();
		buf.Append("[");
		
		for ( int nRowIndex = 0; nRowIndex < m_list.GetSize(); nRowIndex++ ) 
		{

			if ( nRowIndex != 0 ) 
			{
					buf.Append(",");
			}
			buf.Append(m_list.GetElement(nRowIndex));
			
		}
		buf.Append("]");
		
		return buf.ToString();
	}
}
