/*
 * Created on Jan 12, 2005
 */
package com.plumtree.uiinfrastructure.utils;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

/**
 * Generates a Javascript 2D array in a String.
 * Only the array itself is generated, no variable declaration or assignment or trailing semicolon.
 * <p>
 * Ex:
 * <code>
 * 	JSArrayList jsarray = new JSArrayList(3);
 *  	for ( int i = 0; i < 3; i++ )
 *  	{
 * 		jsarray.AddNewElement();
 * 		jsarray.SetValue(0, "val"+i+"0")
 * 		jsarray.SetValue(1, "val"+i+"1")
 * 		jsarray.SetValue(2, "val"+i+"2")
 * 	}
 * 	jsarray.ToString();
 * </code>z
 * <p>
 * Generates the string:
 * <p>
 * "[[val00, val01, val02],[val10, val11,val12],[val20,val21,val23]]"
 * 
 * 
 * @author robertz
 */
public class JS2DArrayBuilder
{
	
	private XPArrayList m_list;
	
	private int m_nCurrIndex = 0;

	
	private int m_nSetColumnCount = 0;
	/**
	 * 
	 * @param nColumnCount Number of columns in the array.
	 */
	public JS2DArrayBuilder( int nColumnCount)
	{
		m_list = new XPArrayList();
		
		m_nSetColumnCount = nColumnCount;
	}
	
	/**
	 * Adds a new row at the end.
	 * 
	 * @return
	 */
	public int AddNewElement()
	{
			
			return m_list.Add(new String[m_nSetColumnCount]);
			
	}
	
	/**
	 * 
	 * @param nColumn Column index of the current row
	 * @param strValue Value to assign to the specified column. The value will be surrounded with quotes to define a string 
	 */
	public void SetColumnStringValue(int nColumn, String strValue)
	{
		String[] arColumns = (String[])m_list.GetElement(m_list.GetSize()-1);
		if ( nColumn < 0 || nColumn > m_nSetColumnCount ) {
			throw new XPIllegalArgumentException("Invalid column index specified: "+nColumn+". Must be a positive value less than the maximum columns.");
		}
		
		arColumns[nColumn] = (strValue == null ) ? "" : "'"+ JSHelper.escapeJS(strValue)+"'";
	}
	
	/**
	 * @TODO MARTOK add another method for simple value or change the name
	 * 
	 * @param nColumn Column index of the current row
	 * @param strValue
	 */
	public void SetColumnReference(int nColumn, String strValue)
	{
		String[] arColumns = (String[])m_list.GetElement(m_nCurrIndex);
		if ( nColumn < 0 || nColumn > m_nSetColumnCount ) {
			throw new XPIllegalArgumentException("Invalid column index specified: "+nColumn+". Must be a positive value less than the maximum columns.");
		}
		
		arColumns[nColumn] = (strValue == null ) ? "" : strValue;
	}
	
	/**
	 * 
	 * @return The size of the current array.
	 */
	public int GetSize()
	{
		return m_list.GetSize();
	}
	
	/**
	 * 
	 * @return
	 */
	public String ToString()
	{
		XPStringBuilder buf = new XPStringBuilder();
		buf.Append("[");
		
		for ( int nRowIndex = 0; nRowIndex < m_list.GetSize(); nRowIndex++ ) 
		{
			
			if ( nRowIndex != 0 ) 
			{
					buf.Append(",");
			}
			buf.Append(" [");
			
			String[] arValues = (String[])m_list.GetElement(nRowIndex);
			
			for ( int nColIndex = 0; nColIndex < m_nSetColumnCount; nColIndex++ )
			{
				if ( nColIndex != 0 ) {
					buf.Append(",");
				}
				buf.Append(arValues[nColIndex]);
			}
			buf.Append("]");
			
		}
		buf.Append("]");
		
		return buf.ToString();
	}
}
