/*
 * Created on Jan 14, 2005
 */
package com.plumtree.uiinfrastructure.utils;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.statichelpers.*;

/**
 * Class description
 * 
 * @author robertz
 */
public class JSFunctionCallBuilder
{
		private String m_strFunctionName = "";
		private XPStringBuilder buf = new XPStringBuilder();
		private boolean m_bSemiColon = false;
		
		public JSFunctionCallBuilder(String strFunctionName)
		{
			m_strFunctionName = strFunctionName;
		}
		
		
		
		
		
		/**
		 * Argument value will be put inside quotes.
		 * E.g. functionName("string1", "string2");
		 * 
		 * @param strStringValue
		 */
		public void AddStringValueArgument(String strStringValue)
		{
			if ( buf.GetLength() != 0) {
				buf.Append(",");
			}
			
			buf.Append("\""+JSHelper.escapeJS(strStringValue)+"\"");
		}
		
		/**
		 * Argument value is enclosed in escaped quotes. Used when
		 * a function call is used within a javascript string.
		 * 
		 * E.g.
		 * 
		 * <script>
		 * var foo = "<a href=\"# \ onclick=\"somefunction(\"stringval\", variableval)\" ">text</a> ""
		 * 
		 */
		public void AddStringValueArgumentUsingSingleQuotes(String strStringValue)
		{
			if ( buf.GetLength() != 0) {
				buf.Append(",");
			}
			buf.Append("\'"+JSHelper.escapeJS(strStringValue)+"\'");
		}
		
		/**
		 * Javascript variable reference
		 * E.g. functionName(var1, var2);
		 */
		public void AddVariableArgument(String strVariable)
		{
			if ( buf.GetLength() != 0) {
				buf.Append(",");
			}
			buf.Append(strVariable);
		}
		
		public void UseSemiColon()
		{
			m_bSemiColon = true;
		}
		
		public String ToString()
		{
			return m_strFunctionName+"("+buf.ToString()+")"+((m_bSemiColon) ? ";" : "");
		}
		
}

