package com.plumtree.uiinfrastructure.validators;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.AValidator;

/**
 * Validates that a input value is less than a given length
 * 
 * @author caiusv
 */
public class LengthValidator extends AValidator implements IValidator {

	public String VALIDATOR_NAME = "LengthValidator";
	
	/** name of the form where this validator appears (used by JavaScript validation)*/
	public String m_strFormName;
	
	/** name of the input which this validator works on (used by JavaScript validation) */
	public String m_strInputName;
	
	private int m_intMax;
	private int m_intMin;
	
	/**
	 * Creates a new PositiveIntValidator.
	 * 
	 * @param _sFormName name of the form whre this appears
	 * @param _sInputName	name of the input element
	 * @param _sDefaultValue	default value of the input element
	 */
	public LengthValidator(String _sFormName, 
			String _sInputName,
			int _iMax,
			int _iMin)
	{
		m_strFormName 		= _sFormName;
		m_strInputName 		= _sInputName;
		m_intMax			= _iMax;
		m_intMin			= _iMin;
	}
	
	public UIError Validate(Object _oObject)
	{

		UIError myError =
			new UIError(
				UIErrorLevel.FRIENDLY,
				UIErrorScope.CURRENT_USER,
				GetErrorMessage(),
				GetErrorMessage());
		if (null == _oObject)
		{
			return myError;
		}
		try {
			String myString = XPConvert.ToString(_oObject);
			if (myString.length() > m_intMax || myString.length() < m_intMin)
			{
				return myError;
			}
		}
		catch (Exception e)
		{
			return myError;
		}
		return null;
	}

	/**
	 * Returns the JavaScript Validation method that may be used on a page to validate
	 * associated form input.
	 * 
	 * @see com.plumtree.uiinfrastructure.activityspace.IValidator#GetJSValidationCode()
	 */
	public String GetJSValidationCode() {
		XPStringBuilder sb = new XPStringBuilder(1250);
		sb.Append("<script type=\"text/javascript\">\n");
		sb.Append("function validateLength(inputName)\n");
		sb.Append("{\n");
		sb.Append("    if (inputName.value.length > "+m_intMax+" || inputName.value.length < "+m_intMin+")\n");
		sb.Append("    {\n");
		sb.Append("        alert(\""+GetErrorMessage()+"\");\n");
		sb.Append("        return false;\n");
		sb.Append("    }\n");
		sb.Append("    else");
		sb.Append("    {\n");
		sb.Append("        return true;\n");
		sb.Append("    }\n");
		sb.Append("}\n");
		sb.Append("</script>\n");
		return sb.toString();
	}
	
	/**
	 * Return the name of the JavaScript validation method along with whatever
	 * parameters the script needs.
	 */
	public String GetJSFunction(String _sParameters)
	{
		if(null != _sParameters)
		{
			return "validateLength("+_sParameters+")";
		}
		else
		{
			return "validateLength("+m_strFormName+"."+m_strInputName+")";
		}
	}
	
	/**
	 * Return the name of this Validator
	 */
	public String GetName()
	{
		return VALIDATOR_NAME;
	}
}
