package com.plumtree.uiinfrastructure.validators;

import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.errordisplay.*;
import com.plumtree.uiinfrastructure.pagecontrols.framework.AValidator;

/**
 * Validates that an object is a positive int.
 * 
 * @author caiusv
 */
public class PositiveIntValidator extends AValidator implements IValidator {

	public String VALIDATOR_NAME = "PositiveIntValidator";
	
	/** the defualt value of the input */
	private String m_strDefaultValue;
	
	/** name of the form where this validator appears (used by JavaScript validation)*/
	public String m_strFormName;
	
	/** name of the input which this validator works on (used by JavaScript validation) */
	public String m_strInputName;
	
	/**
	 * Creates a new PositiveIntValidator.
	 * 
	 * @param _sFormName name of the form whre this appears
	 * @param _sInputName	name of the input element
	 * @param _sDefaultValue	default value of the input element
	 */
	public PositiveIntValidator(String _sFormName, 
			String _sInputName,
			String _sDefaultValue)
	{
		m_strFormName 		= _sFormName;
		m_strInputName 		= _sInputName;
		m_strDefaultValue 	= _sDefaultValue;
	}
	
	/**
	 * Return a UIError object if the object passed in is not a positive int.
	 */
	public UIError Validate(Object _oObject)
	{
		String strErrorMsg = GetErrorMessage();

		UIError myError =
			new UIError(
				UIErrorLevel.FRIENDLY,
				UIErrorScope.CURRENT_USER,
				strErrorMsg,
				strErrorMsg);
		if (null == _oObject)
		{
			return myError;
		}
		try {
			if (XPConvert.ToInteger(_oObject) < 1)
			{
				return myError;
			}
		}
		catch (Exception e)
		{
			return myError;
		}
		return null;
	}

	/**
	 * Returns the JavaScript Validation method that may be used on a page to validate
	 * associated form input.
	 * 
	 * @see com.plumtree.uiinfrastructure.activityspace.IValidator#GetJSValidationCode()
	 */
	public String GetJSValidationCode() {
		XPStringBuilder sb = new XPStringBuilder(1250);
		sb.Append("<script type=\"text/javascript\">\n");
		sb.Append("function isPositiveInt(inputName)\n");
		sb.Append("{\n");
		sb.Append("    if (!PTNumberUtil.isPositiveInteger(parseInt(inputName.value)))\n");
		sb.Append("    {\n");
		sb.Append("        alert(\""+GetErrorMessage()+"\");\n");
		sb.Append("        inputName.value = \""+m_strDefaultValue+"\";\n");
		sb.Append("        return false;\n");
		sb.Append("    }\n");
		sb.Append("    return true;\n");
		sb.Append("}\n");
		sb.Append("</script>\n");
		return sb.toString();
	}
	
	/**
	 * Return the name of the JavaScript validation method along with whatever
	 * parameters the script needs.
	 */
	public String GetJSFunction(String _sParameters)
	{
		if(null != _sParameters)
		{
			return "isPositiveInt("+_sParameters+")";
		}
		else
		{
			return "isPositiveInt("+m_strFormName+"."+m_strInputName+")";
		}
	}
	
	/**
	 * Return the name of this Validator
	 */
	public String GetName()
	{
		return VALIDATOR_NAME;
	}
}
