package com.plumtree.xpshared.htmlconstructs;

//import com.plumtree.xpfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;


/**
 *
 *
 * @author beng  This class implements functionality used by various classes in
 *         this package.  This functionality is not intended for public use.
 */
public class FactoryHelper
{
//
//    private static XPLogger myLogger = XPLogManager.GetLogger(FactoryHelper.class);
//
//
//    private static boolean bDebug = myLogger.IsDebugEnabled();
//
//
//    private static boolean bError = myLogger.IsErrorEnabled();
//
//
//    private static boolean bWarn = myLogger.IsWarnEnabled();
//
//
//    private static boolean bInfo = myLogger.IsInfoEnabled();
//
//
//    private static boolean bFatal = myLogger.IsFatalEnabled();

    /**
     * This method takes a string dictionary, and constructs an HTML table from
     * the values in this dictionary.  This method is to be used purely for
     * debuggin purposes, the HTMLTable returned has no true display value.
     *
     * @param sd
     *
     * @return
     */
    public static HTMLTable XPDictionaryToTable(XPStringDictionary sd)
    {
        HTMLTable myTable = new HTMLTable();

        HTMLTableRow myRow;
        HTMLTableCell myCell;
        String sKey;
        String sVal;
        IXPEnumerator eKeys = sd.GetKeys();

        while (eKeys.MoveNext())
        {
            sKey = (String) eKeys.GetCurrent();
            sVal = sd.GetElement(sKey);

            try
            {
                myRow = new HTMLTableRow();

                myTable.AddInnerHTMLElement(myRow);

                myCell = new HTMLTableCell();

                myCell.AddInnerHTMLString("KEY: " + sKey);

                myRow.AddInnerHTMLElement(myCell);

                myCell = new HTMLTableCell();

                myCell.AddInnerHTMLString("VAL: " + sVal);

                myRow.AddInnerHTMLElement(myCell);
            }
            catch (HTMLException he)
            {
				// do nothing
            }
        }

        return myTable;
    }

    /**
     * This method takes an IXPEnumerator of XPHashtable objects, and  combines all
     * of the object/object pairs of all of these maps into a single XPHashtable
     * object.
     *
     * @param e
     *
     * @return
     */
    public static XPHashtable combineMaps(IXPEnumerator e)
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering method combineMaps");
//        }

        XPHashtable mResult = new XPHashtable();

        IXPEnumerator eKeys;

        Object oKey;
        Object oVal;

        XPHashtable map;

        while (e.MoveNext())
        {
            //   for each dictionary
            try
            {
                map = (XPHashtable) e.GetCurrent();
            }
            catch (Exception ex)
            {
                map = null;
            }

            if (map != null)
            {
//                if (bDebug)
//                {
//                    myLogger.Debug("Got a map.");
//                }

                eKeys = map.GetKeys();

                while (eKeys.MoveNext())
                {
                    //   for each key
                    oKey = eKeys.GetCurrent();

                    oVal = map.GetElement(oKey);
//
//                    if (bDebug)
//                    {
//                        myLogger.Debug("Got a key: " + oKey);
//                    }

                    mResult.PutElement(oKey, oVal);
                }
            }
        } //   for each map

        return mResult;
    }

    /**
     * Call this method to add two XPStringDictionary objects.
     *
     * @param sd1 Pass an XPStringDictionary
     * @param sd2 Pass another XPStringDictionary
     *
     * @return Returns an XPStringDictionary containing mappings from both
     *         parameter XPStringDictionary objects.
     */
    public static XPStringDictionary combineStringDictionaries(XPStringDictionary sd1, XPStringDictionary sd2)
    {
        XPStringDictionary dResult = new XPStringDictionary();

        IXPEnumerator eKeys;
        String sKey;
        String sVal;

        eKeys = sd1.GetKeys();

        while (eKeys.MoveNext())
        {
            sKey = (String) eKeys.GetCurrent();
            sVal = sd1.GetElement(sKey);

            dResult.PutElement(sKey, sVal);
        }

        eKeys = sd2.GetKeys();

        while (eKeys.MoveNext())
        {
            sKey = (String) eKeys.GetCurrent();
            sVal = sd2.GetElement(sKey);

            dResult.PutElement(sKey, sVal);
        }

        return dResult;
    }

    /**
     * This method takes an ArrayList of StringDictionary objects, and combines
     * all of the key/value pairs of all of these dictionaries into a single
     * dictionary.
     *
     * @param e
     *
     * @return
     */
    public static XPStringDictionary combineStringDictionaries(IXPEnumerator e)
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering method combineStringDictionaries");
//        }

        XPStringDictionary dResult = new XPStringDictionary();

        IXPEnumerator eKeys;
        String sKey;
        String sVal;
        XPStringDictionary sd;

        while (e.MoveNext())
        {
            //   for each dictionary
            try
            {
                sd = (XPStringDictionary) e.GetCurrent();
            }
            catch (Exception ex)
            {
                sd = null;
            }

            if (sd != null)
            {
//                if (bDebug)
//                {
//                    myLogger.Debug("Got a dicitonary.");
//                }

                eKeys = sd.GetKeys();

                while (eKeys.MoveNext())
                {
                    //   for each key
                    sKey = (String) eKeys.GetCurrent();

                    sVal = sd.GetElement(sKey);

                    dResult.PutElement(sKey, sVal);
                }
            }
        } //   for each dictionary

        return dResult;
    }

    /**
     * This method updates HTMLImg objects nested within a construct.  The alt
     * attribute of each updated HTMLImg object is set to the localized
     * String.
     *
     * @param construct Pass a PTConstruct
     */
    public static void updateImgAlts(PTConstruct construct)
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering updateImgAlts()");
//        }

        PTConstructType t = construct.GetType();

        //      XPArrayList alInnerElements = construct.GetAllElements();
        //      IXPEnumerator eInnerElements;
        //        XPStringDictionary d = t.GetImgElementNameToAltIdMap();
        XPHashtable m = t.GetImgElementNameToAltIdMap();

        //      if( bDebug )
        //      {
        //         myLogger.Debug( "alInnerElements.GetCount(): " + alInnerElements.GetSize() );
        //         myLogger.Debug( "d: " + d );
        //      }
        if (m != null)
        {
            IXPEnumerator eKeys = m.GetKeys();

            String sElementName;
            String sStringID    = "";
            HTMLElement eTemp;
            HTMLImg imgTemp;

            String sNewString;

            while (eKeys.MoveNext())
            {
                sElementName = (String) eKeys.GetCurrent();

                Object o = m.GetElement(sElementName);

                if (o != null)
                {
					sStringID = XPConvert.ToString(o);
                }

                sNewString = "new_" + sStringID;

                eTemp = null;

                try
                {
                    eTemp = construct.GetNestedElement(new ElementName(sElementName));
                }
                catch (Exception e)
                {
					// do nothing

//                    if (bError)
//                    {
//                        myLogger.Error("couldn't get nested element named: " + sElementName);
//                    }
                }

                if (eTemp != null)
                {
                    if (eTemp instanceof HTMLImg)
                    {
                        imgTemp = (HTMLImg) eTemp;

                        imgTemp.SetAlt(sNewString);
                    }
                    else
                    {
//                        if (bError)
//                        {
//                            myLogger.Error("This element is not an HTMLImg.");
//                        }
                    } //   element is not a HTMLImg
                }
            } //   for each key/val pair in the string dictionary
        } //   if this type offers a string dictionary.
    }

    /**
     * This method updates all of the appropriate strings within a construct
     * with localized versions of those strings. <br>
     * <br>
     * Each construct offers a StrinDictionary containing ElementName/stringid
     * pairs.  The ElementName indicates a nested HTMLElement object, the
     * string id indicates a localized string.     <br>
     * <br>
     * This method obtains each nested element indicated by the set of
     * ElementName objects.  If that nested HTMLElement is an
     * HTMLGenericElement, then it is replaced with the corresponding
     * localized     string.  If the nested element is any other type of
     * HTMLElement object,     the corresponding localized string is nested
     * within the nested element.
     *
     * @param construct Pass a PTConstruct.  Nested elements with names
     *        equivalent to those specified by the string dictionary will be
     *        updated.
     */
    public static void updateStrings(PTConstruct construct)
    {
        PTConstructType t = construct.GetType();

//        if (bInfo)
//        {
//            myLogger.Info("Entering updateStrings()");
//        }

        XPHashtable m = t.GetStringMap();

        if (m != null)
        {
            IXPEnumerator eKeys = m.GetKeys();

            String sElementName;
            String sStringID    = "";
            HTMLElement eTemp;

            String sNewString;

            while (eKeys.MoveNext())
            {
                sElementName = (String) eKeys.GetCurrent();

                try
                {
                    Object o = m.GetElement(sElementName);

                    if (o != null)
                    {
						sStringID = XPConvert.ToString(o);
                    }
                }
                catch (Exception ex)
                {
                }

//                if (bDebug)
//                {
//                    myLogger.Debug("sElementName: " + sElementName);
//                    myLogger.Debug("oiId: " + oiId);
//                }

                if (sStringID.equals(PTConstructType.DO_NOT_UPDATE))
                {
                }
                else
                {
                    //	todo - The new string should be obtained from the
                    //	localization dictionary
                    sNewString = "new_" + sStringID;

                    eTemp = null;

                    try
                    {
                        eTemp = construct.GetNestedElement(new ElementName(sElementName));
                    }
                    catch (Exception e)
                    {
						// do nothing

//                        if (bError)
//                        {
//                            myLogger.Error("couldn't get nested element named: " + sElementName);
//                        }
                    }

                    if (eTemp != null)
                    {
                        if (eTemp instanceof HTMLGenericElement)
                        {
                            HTMLGenericElement ge = (HTMLGenericElement) eTemp;

                            ge.SetDisplayString(sNewString);
                        }
                        else
                        {
                            eTemp.ClearInnerHTMLElements();

                            try
                            {
                                eTemp.AddInnerHTMLString(sNewString);
                            }
                            catch (HTMLException he)
                            {
								// do nothing

//                                if (bDebug)
//                                {
//                                    myLogger.Debug("Couldn't add string.");
//                                }
                            }
                        } //   element is not a HTMLGenericElement
                    }
                } //   string id is not DO_NOT_UPDATE
            } //   for each key/val pair in the string dictionary
        } //   if this type offers a string dictionary.
    }
}
