package com.plumtree.xpshared.htmlconstructs;

import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;


/**
 * An object of this class can be used to contruct the familiar
 *         breadcrumb path. <br>
 *         <br>
 *         The breadcrumb path construct contains an ordered list of text
 *         and/or links, separated by a separator character.
 *
 * @author beng
 */
public class PTBreadcrumbPath
{
    /** The separator should use breaking spaces so that the breadcrumb can wrap between folders. */
    public String sSeparator = " : ";


    private XPArrayList alElements;

    /**
     * Creates a new PTBreadcrumbPath object.
     */
    public PTBreadcrumbPath()
    {
        alElements = new XPArrayList();
    }

    /**
     * Call this method to obtain an HTMLElement that describes the state of
     * this breadcrumb path.
     *
     * @return
     *
     * @throws Exception
     */
    public HTMLElement GetPathAsHTMLElement()
    {
        IXPEnumerator enumElements     = alElements.GetEnumerator();
        HTMLSpan mySpan        = new HTMLSpan();
        HTMLElement eSeparator = new HTMLGenericElement(sSeparator);

        mySpan.SetStyleClass(PTStyleClass.BREADCRUMB_TEXT);

        HTMLElement e;

        int iNumElements = alElements.GetSize();

        int i = 1;

        while (enumElements.MoveNext())
        {
            e = (HTMLElement) enumElements.GetCurrent();

            mySpan.AddInnerHTMLElement(e);

            if (i < iNumElements)
            {
                mySpan.AddInnerHTMLElement(eSeparator);
            }

            i++;
        }

        return mySpan;
    }

    /**
     * Call this method to append an HTMLAnchor to the end of this path.
     *
     * @param a
     */
    public void appendAnchorItem(HTMLAnchor a)
    {
        alElements.Add(a);
    }

    /**
     * Call this method to append a regular string to the end of this path.
     * This string should be plain text, and contain no html.
     *
     * @param s
     */
    public void appendTextItem(String s)
    {
        HTMLGenericElement e = new HTMLGenericElement(s);

        alElements.Add(e);
    }
}
