package com.plumtree.xpshared.htmlconstructs;

import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;


/**
 *
 * @author beng         All classes representing types of constructs must
 *         extend this class.
 */
public abstract class PTConstructType
{
    /**
     * Use this as the value for those strings which should not be
     * automatically updated by the factory.  For example, a gadget banner's
     * title is not a string that is extracted and localized, but rather a
     * string that simply needs to be filled in.
     */
    public static final String DO_NOT_UPDATE = "&?!!Bzzrt,";

    /**
     * @return
     */
    abstract public String GetDescription();

    /**
     * Call this method to obtain an ElementName object indicating a specific
     * html element found within this construct.     <br>
     * <br>
     * For example, to obtain an ElementName object indicating the html
     * element containing the wide banner's title, do the following:     <br>
     * <br>
     * <code>    myEName = PTBannerType.WIDE.GetElementName(
     * PTBannerType.ELEMENT_NAME_ID_TITLE );     </code><br><br>
     *
     * @param sElementNameID Pass a constant indicating which ElementName you
     *        wish to obtain.  Each concrete PTConstructType provides a unique
     *        set     of constants for obtaining ElementName objects.
     *
     * @return
     *
     * @throws Exception
     */
    public abstract ElementName GetElementName(String sElementNameID);

    /**
     * Call this method to obtain the string dictionary for this     construct. <br>
     * <br>
     * A construct's string dictionary contains, as keys, the name's of those
     * html elements that must be updated with localized strings. The values
     * corresponding to these keys are the id's used to obtain the
     * corresponding localized string.  For example, if a banner's string
     * dictionary containes one pair of strings:     <br>
     * <br>
     * &nbsp;&nbsp;&nbsp;"title" - "titleID"     <br>
     * <br>
     * then, the element named "title" would need to be updated with the
     * localized string retrieved with the id "titleID".
     *
     * @return
     */
    public abstract XPHashtable GetStringMap();

    /**
     * Call this method to obtain a prototype HTMLElement for this particular
     * construct type.  This prototype should neither be used, nor modified,
     * though it may be examined or cloned.     <br>
     * <br>
     * If you wish to obtain an HTMLElement representation of this construct
     * type that is all your own, with which you're free to do as you please,
     * use the PTConstructFactory.GetConstruct() method.
     *
     * @return
     *
     * @throws Exception
     */
    public HTMLElement GetHTMLElement()
    {
        throw new XPException();
    }

    /**
     * Call this method to obtain a string dictionary mapping element names to
     * img alt id's.  An element name indicates the name of a particular
     * HTMLImg element within this construct.  An img alt id indicates the id
     * that may be used to obtain an alt string.
     *
     * @return
     */
    public abstract XPHashtable GetImgElementNameToAltIdMap();
}
