package com.plumtree.xpshared.htmlconstructs;



import com.plumtree.openfoundation.internationalization.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;
import com.plumtree.xpshared.jsutil.*;


/**
 * @author JohnH
 *
 * Use this class to obtain PTDatePicker objects.
 * <br><br>
 * Use of this construct requires the inclusion of several HTML elements to
 * make it work.  First, the following DIV must be placed before the FORM tag
 * on the page: <div id="calDiv" class="cal-Div">&nbsp;</div> These all can be
 * implemented using the methods of this construct.  Call GetDatePickerIncludes to
 * retrieve an HTMLElementCollection that can be used in the Display Page
 * classes of editors to insert script and the style link into the HEAD, for
 * example.  The DIV is retrieved as an HTMLElement by calling the GetCalDiv
 * method.
 */

public class PTDatePicker
{
    private static String _ui_DisplayHTMLError = "Unable to render DatePicker Control.";

	public static final int DATE_PICKER_CHANGE_LIST = 81555;


	public PTDatePicker()
	{

	}

	/** This returns an HTMLElement containing the DIV tag that is used
	 * to control location of the date picker.  It must be placed after the BODY but
	 * before the form tag.
	 */
	public static HTMLElement GetCalDiv()
	{

//		if( bInfo )
//		{
//			myLogger.Info( "Entering GetCalDiv" );
//		}


		HTMLElementCollection result = new HTMLElementCollection();

		try
		{
			//Display the DIV tag for the calendar control
			HTMLDiv myDiv = new HTMLDiv();
			myDiv.SetID("calDiv");
			myDiv.SetStyleClass(PTStyleClass.CAL_DIV);
			result.AddInnerHTMLElement(myDiv);

		} catch (Exception e) {
//			if (bError) {
//				myLogger.Error("PTDatePicker::GetCalDiv The following error occured when trying to display the HTML: " + e.getMessage());
//			}
			throw new XPException(_ui_DisplayHTMLError, e);
		}

		return result;
	}


    /**	This returns an HTML INPUT box followed by a calendar button which, when
	 * pressed, displays a calendar from which a date can be chosen. * 	<br><br>
	 *  @param strImageServerURL - The URL to where the date picker images are stored.  This URL must contain the trailing slash.
	 * 	@param strVarName - A unique variable used in writing out the script tag
	 *  @param strFormName - The name of the form the control is displayed in
	 *  @param strInputName - The name for the input box generated by the construct
	 *  @param strButtonName - The name for the button generated by the construct
	 * 	@param strStartDate - The string to populate initially in the date control
	 */
	public static HTMLElement GetDatePicker(String strImageServerURL, String strVarName, String strFormName, String strInputName, String strButtonName, String strStartDate )
	{

//		if( bInfo )
//		{
//			myLogger.Info( "Entering GetDatePicker" );
//		}


		HTMLElementCollection result = new HTMLElementCollection();
		HTMLScript myScript;
		//String strNow;

		try
		{
			//Add a javascript function
            myScript = new HTMLScript( "text/javascript" );

            XPStringBuilder strFunction = new XPStringBuilder("var " + strVarName + " = new PTDatepicker(");
            strFunction.Append( "\"calDiv\"," ); //DIV name
            strFunction.Append( "\"" + strVarName + "\"," ); // var name - this needs to be unique
            strFunction.Append( "\"" + strFormName + "\"," ); //form name
            strFunction.Append( "\"" + strInputName + "\"," ); //name for the input text box
            strFunction.Append( "\"" + strButtonName + "\"," ); //name for the button
            strFunction.Append( "\"" + strStartDate + "\"," ); //Default value
            strFunction.Append( "\"MMM d, yyyy\","); //Date format
            strFunction.Append( "0," );
            strFunction.Append( "\"" + strImageServerURL + "common/private/js/jsdatepicker/" + DATE_PICKER_CHANGE_LIST + "/images/" + "\"" );
            strFunction.Append( ");" );

            myScript.AddInnerHTMLString( strFunction.toString() );
            result.AddInnerHTMLElement( myScript );

            myScript = new HTMLScript( "text/javascript" );
			myScript.AddInnerHTMLString( strVarName + ".writeControl();" );
			result.AddInnerHTMLElement( myScript );

		} catch (Exception e) {
//			if (bError) {
//				myLogger.Error("PTDatePicker::GetDatePicker The following error occured when trying to display the HTML: " + e.getMessage());
//			}
			throw new XPException(_ui_DisplayHTMLError, e);
		}

		return result;
	}

	/**
	 * @deprecated Use GetDatePickerIncludes(XPLocale locale, JSRegistry _jsRegistry)
	 * This function returns the SCRIPT includes for the date picker
	 *
	 * @param _strLangID The language string
	 * @param _jsRegistry The javascript registry
	 *
	 * @return an HTMLElementCollection containing the script includes for the
	 * date picker
	 *
	 * @throws XPException
	 */
	public static HTMLElement GetDatePickerIncludes(String strLangID,  JSRegistry _jsRegistry)
	{
		HTMLElementCollection retVal = null;

		try
		{
			XPLocale locale = new XPLocale(strLangID);
			String strHTMLLinks = _jsRegistry.GetJSComponent("jsdatepicker", DATE_PICKER_CHANGE_LIST).GenIncludes(locale);

			retVal = new HTMLElementCollection();
			retVal.AddInnerHTMLString(strHTMLLinks);
		}
		catch (Exception e)
		{
			throw new XPException("PTDatePicker::GetDatePickerIncludes The following error occured when trying to display the HTML: " + e.getMessage(), e);
		}

		return retVal;
	}
	
	/**
	 * This function returns the SCRIPT includes for the date picker
	 *
	 * @param _strLangID The language string
	 * @param _jsRegistry The javascript registry
	 *
	 * @return an HTMLElementCollection containing the script includes for the
	 * date picker
	 *
	 * @throws XPException
	 */
	public static HTMLElement GetDatePickerIncludes(XPLocale locale, JSRegistry _jsRegistry)
	{
		HTMLElementCollection retVal = null;

		try
		{
			String strHTMLLinks = _jsRegistry.GetJSComponent("jsdatepicker", DATE_PICKER_CHANGE_LIST).GenIncludes(locale);

			retVal = new HTMLElementCollection();
			retVal.AddInnerHTMLString(strHTMLLinks);
		}
		catch (Exception e)
		{
			throw new XPException("PTDatePicker::GetDatePickerIncludes The following error occured when trying to display the HTML: " + e.getMessage(), e);
		}

		return retVal;
	}

}

