package com.plumtree.xpshared.htmlconstructs;

//import com.plumtree.xpfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.uiinfrastructure.activityspace.*;
import com.plumtree.uiinfrastructure.constants.*;
import com.plumtree.xpshared.htmlelements.*;


/**
 *
 *
 * @author JohnH  This construct returns an input box and a combo box for an
 *         integer and time increments (minutes, hours, days, etc.)
 *         respectively.
 */
public class PTDurationCombo
{

    /** Localized String: Millisecond(s) */
    public static final String _ui_ViewMilliseconds = "Millisecond(s)";

    /** Localized String: Second(s) */
    public static final String _ui_ViewSeconds = "Second(s)";

    /** Localized String: Minute(s) */
    public static final String _ui_ViewMinutes = "Minute(s)";

    /** Localized String: Hour(s) */
    public static final String _ui_ViewHours = "Hour(s)";

    /** Localized String: Day(s) */
    public static final String _ui_ViewDays = "Day(s)";

    /** Localized String: Week(s) */
    public static final String _ui_ViewWeeks = "Week(s)";

    /** Localized String: Month(s) */
    public static final String _ui_ViewMonths = "Month(s)";

    /** Localized String: Year(s) */
    public static final String _ui_ViewYears = "Year(s)";


    public static final int DISPLAYMILLISECONDS = 1;


    public static final int DISPLAYSECONDS = 2;


    public static final int DISPLAYMINUTES = 4;


    public static final int DISPLAYHOURS = 8;


    public static final int DISPLAYDAYS = 16;


    public static final int DISPLAYWEEKS = 32;


    public static final int DISPLAYMONTHS = 64;


    public static final int DISPLAYYEARS = 128;

    /**
     * Creates a new PTDurationCombo object.
     */
    public PTDurationCombo()
    {
    }

    /**
     *
     *
     * @param nAvailableDurations
     * @param lMilliseconds
     *
     * @return
     */
    public static int BestDurationTypeForMilliseconds(int nAvailableDurations, long lMilliseconds)
    {
        long l = (long) 12L * 4L * 7L * 24L * 60L * 60L * 1000L;

        if (0 != (PTDurationCombo.DISPLAYYEARS & nAvailableDurations))
        {
            if (0 == (lMilliseconds % l))
            {
                return PTDurationCombo.DISPLAYYEARS;
            }
        }

        l = (long) 4L * 7L * 24L * 60L * 60L * 1000L;

        if (0 != (PTDurationCombo.DISPLAYMONTHS & nAvailableDurations))
        {
            if (0 == (lMilliseconds % l))
            {
                return PTDurationCombo.DISPLAYMONTHS;
            }
        }

        if (0 != (PTDurationCombo.DISPLAYWEEKS & nAvailableDurations))
        {
            if (0 == (lMilliseconds % (7L * 24L * 60L * 60L * 1000L)))
            {
                return PTDurationCombo.DISPLAYWEEKS;
            }
        }

        if (0 != (PTDurationCombo.DISPLAYDAYS & nAvailableDurations))
        {
            if (0 == (lMilliseconds % (24 * 60 * 60 * 1000)))
            {
                return PTDurationCombo.DISPLAYDAYS;
            }
        }

        if (0 != (PTDurationCombo.DISPLAYHOURS & nAvailableDurations))
        {
            if (0 == (lMilliseconds % (60 * 60 * 1000)))
            {
                return PTDurationCombo.DISPLAYHOURS;
            }
        }

        if (0 != (PTDurationCombo.DISPLAYMINUTES & nAvailableDurations))
        {
            if (0 == (lMilliseconds % (60 * 1000)))
            {
                return PTDurationCombo.DISPLAYMINUTES;
            }
        }

        if (0 != (PTDurationCombo.DISPLAYSECONDS & nAvailableDurations))
        {
            if (0 == (lMilliseconds % 1000))
            {
                return PTDurationCombo.DISPLAYSECONDS;
            }
        }

        if (0 != (PTDurationCombo.DISPLAYMILLISECONDS & nAvailableDurations))
        {
            return PTDurationCombo.DISPLAYMILLISECONDS;
        }

        return 0;
    }

    /**
     *
     *
     * @param lMilliseconds
     * @param nDurationType
     *
     * @return
     */
    public static long DurationFromMilliseconds(long lMilliseconds, int nDurationType)
    {
        long nTimeoutMilliseconds = lMilliseconds;

        switch (nDurationType)
        {
        case PTDurationCombo.DISPLAYYEARS:
            nTimeoutMilliseconds = nTimeoutMilliseconds / 12;

        case PTDurationCombo.DISPLAYMONTHS:
            nTimeoutMilliseconds = nTimeoutMilliseconds / 4;

        case PTDurationCombo.DISPLAYWEEKS:
            nTimeoutMilliseconds = nTimeoutMilliseconds / 7;

        case PTDurationCombo.DISPLAYDAYS:
            nTimeoutMilliseconds = nTimeoutMilliseconds / 24;

        case PTDurationCombo.DISPLAYHOURS:
            nTimeoutMilliseconds = nTimeoutMilliseconds / 60;

        case PTDurationCombo.DISPLAYMINUTES:
            nTimeoutMilliseconds = nTimeoutMilliseconds / 60;

        case PTDurationCombo.DISPLAYSECONDS:
            nTimeoutMilliseconds = nTimeoutMilliseconds / 1000;
        }

        return nTimeoutMilliseconds;
    }

    /**
     *
     *
     * @param nDuration
     * @param nDurationType
     *
     * @return
     */
    public static long DurationToMilliseconds(long nDuration, int nDurationType)
    {
        long nTimeoutMilliseconds = nDuration;

        switch (nDurationType)
        {
        case PTDurationCombo.DISPLAYYEARS:
            nTimeoutMilliseconds = 12 * nTimeoutMilliseconds;

        case PTDurationCombo.DISPLAYMONTHS:
            nTimeoutMilliseconds = 4 * nTimeoutMilliseconds;

        case PTDurationCombo.DISPLAYWEEKS:
            nTimeoutMilliseconds = 7 * nTimeoutMilliseconds;

        case PTDurationCombo.DISPLAYDAYS:
            nTimeoutMilliseconds = 24 * nTimeoutMilliseconds;

        case PTDurationCombo.DISPLAYHOURS:
            nTimeoutMilliseconds = 60 * nTimeoutMilliseconds;

        case PTDurationCombo.DISPLAYMINUTES:
            nTimeoutMilliseconds = 60 * nTimeoutMilliseconds;

        case PTDurationCombo.DISPLAYSECONDS:
            nTimeoutMilliseconds = 1000 * nTimeoutMilliseconds;
        }

        return nTimeoutMilliseconds;
    }

    /**
     * This returns an HTML INPUT box followed by a combo box populated  with
     * time increments.
     *
     * @param strDurationElemName - The name for the HTML INPUT box containing
     *        the numeric duration
     * @param strUnitsElemName - The name for the SELECT drop down containing
     *        the units
     * @param intDisplayFlags - An integer indicating which durations to
     *        display in the select.  The choices are exposed by this class as
     *        public constants: DISPLAYMINUTES = 1, DISPLAYHOURS = 2,
     *        DISPLAYDAYS = 4, DISPLAYWEEKS = 8.  These can be combined by
     *        adding up the choices.  Thus,  a value of 3 would display
     *        minutes and hours.
     * @param strSetDuration - Sets the value for the duration input
     * @param intSetUnits - Sets the integer value for the Units Select element
     *
     * @return
     *
     * @throws Exception
     * @throws XPException
     */
    public static HTMLElement GetDurationCombo(String strDurationElemName, String strUnitsElemName, int intDisplayFlags, String strSetDuration, int intSetUnits, AActivitySpace asOwner)
    {
        HTMLElementCollection result = new HTMLElementCollection();
        HTMLSelect mySelect;
        HTMLOption myOption;
        HTMLSpan mySpan;
        HTMLInput myInput;

        try
        {
            myInput = new HTMLInput(HTMLInputTypes.TEXT, strDurationElemName, strDurationElemName);


            //This is a number input box, so set the size = 3
            myInput.SetSize(3);
            myInput.SetStyleClass(PTStyleClass.FORM_INPUT_BOX_TEXT);

            if (!strSetDuration.equals(""))
            {
                myInput.SetValue(strSetDuration);
            }

            result.AddInnerHTMLElement(myInput);
        	result.AddInnerHTMLString(CommonHTMLStrings.SPACE);

            // Add the second element
            mySelect = new HTMLSelect(strUnitsElemName, strUnitsElemName);
            mySelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);

            //Determine which options to display through bitwise comparison
            if ((intDisplayFlags & DISPLAYMILLISECONDS) != 0)
            {
                myOption = new HTMLOption(asOwner.GetString(113, ResourceConstants.INFRASTRUCTURE));
                myOption.SetValue(XPConvert.ToString(DISPLAYMILLISECONDS));

                if (DISPLAYMILLISECONDS == intSetUnits)
                {
                    myOption.SetSelected(true);
                }

                mySpan = new HTMLSpan();
                myOption.AddInnerHTMLElement(mySpan);
                mySpan.AddInnerHTMLString(asOwner.GetString(113, ResourceConstants.INFRASTRUCTURE));
                mySelect.AddInnerHTMLElement(myOption);
            }

            if ((intDisplayFlags & DISPLAYSECONDS) != 0)
            {
                myOption = new HTMLOption(asOwner.GetString(114, ResourceConstants.INFRASTRUCTURE));
                myOption.SetValue(XPConvert.ToString(DISPLAYSECONDS));

                if (DISPLAYSECONDS == intSetUnits)
                {
                    myOption.SetSelected(true);
                }

                mySpan = new HTMLSpan();
                myOption.AddInnerHTMLElement(mySpan);
                mySpan.AddInnerHTMLString(asOwner.GetString(114, ResourceConstants.INFRASTRUCTURE));
                mySelect.AddInnerHTMLElement(myOption);
            }

            if ((intDisplayFlags & DISPLAYMINUTES) != 0)
            {
                myOption = new HTMLOption(asOwner.GetString(115, ResourceConstants.INFRASTRUCTURE));
                myOption.SetValue(XPConvert.ToString(DISPLAYMINUTES));

                if (DISPLAYMINUTES == intSetUnits)
                {
                    myOption.SetSelected(true);
                }

                mySpan = new HTMLSpan();
                myOption.AddInnerHTMLElement(mySpan);
                mySpan.AddInnerHTMLString(asOwner.GetString(115, ResourceConstants.INFRASTRUCTURE));
                mySelect.AddInnerHTMLElement(myOption);
            }

            if ((intDisplayFlags & DISPLAYHOURS) != 0)
            {
                myOption = new HTMLOption(asOwner.GetString(116, ResourceConstants.INFRASTRUCTURE));
                myOption.SetValue(XPConvert.ToString(DISPLAYHOURS));

                if (DISPLAYHOURS == intSetUnits)
                {
                    myOption.SetSelected(true);
                }

                mySpan = new HTMLSpan();
                myOption.AddInnerHTMLElement(mySpan);
                mySpan.AddInnerHTMLString(asOwner.GetString(116, ResourceConstants.INFRASTRUCTURE));
                mySelect.AddInnerHTMLElement(myOption);
            }

            if ((intDisplayFlags & DISPLAYDAYS) != 0)
            {
                myOption = new HTMLOption(asOwner.GetString(117, ResourceConstants.INFRASTRUCTURE));
                myOption.SetValue(XPConvert.ToString(DISPLAYDAYS));

                if (DISPLAYDAYS == intSetUnits)
                {
                    myOption.SetSelected(true);
                }

                mySpan = new HTMLSpan();
                myOption.AddInnerHTMLElement(mySpan);
                mySpan.AddInnerHTMLString(asOwner.GetString(117, ResourceConstants.INFRASTRUCTURE));
                mySelect.AddInnerHTMLElement(myOption);
            }

            if ((intDisplayFlags & DISPLAYWEEKS) != 0)
            {
                myOption = new HTMLOption(asOwner.GetString(118, ResourceConstants.INFRASTRUCTURE));
                myOption.SetValue(XPConvert.ToString(DISPLAYWEEKS));

                if (DISPLAYWEEKS == intSetUnits)
                {
                    myOption.SetSelected(true);
                }

                mySpan = new HTMLSpan();
                myOption.AddInnerHTMLElement(mySpan);
                mySpan.AddInnerHTMLString(asOwner.GetString(118, ResourceConstants.INFRASTRUCTURE));
                mySelect.AddInnerHTMLElement(myOption);
            }

            if ((intDisplayFlags & DISPLAYMONTHS) != 0)
            {
                myOption = new HTMLOption(asOwner.GetString(119, ResourceConstants.INFRASTRUCTURE));
                myOption.SetValue(XPConvert.ToString(DISPLAYMONTHS));

                if (DISPLAYMONTHS == intSetUnits)
                {
                    myOption.SetSelected(true);
                }

                mySpan = new HTMLSpan();
                myOption.AddInnerHTMLElement(mySpan);
                mySpan.AddInnerHTMLString(asOwner.GetString(119, ResourceConstants.INFRASTRUCTURE));
                mySelect.AddInnerHTMLElement(myOption);
            }

            if ((intDisplayFlags & DISPLAYYEARS) != 0)
            {
                myOption = new HTMLOption(asOwner.GetString(120, ResourceConstants.INFRASTRUCTURE));
                myOption.SetValue(XPConvert.ToString(DISPLAYYEARS));

                if (DISPLAYYEARS == intSetUnits)
                {
                    myOption.SetSelected(true);
                }

                mySpan = new HTMLSpan();
                myOption.AddInnerHTMLElement(mySpan);
                mySpan.AddInnerHTMLString(asOwner.GetString(120, ResourceConstants.INFRASTRUCTURE));
                mySelect.AddInnerHTMLElement(myOption);
            }

            result.AddInnerHTMLElement(mySelect);
        }
        catch (Exception e)
        {
            throw new XPException(asOwner.GetString(121, ResourceConstants.INFRASTRUCTURE), e);
        }

        return result;
    }
}
