
package com.plumtree.xpshared.htmlconstructs;

//import com.plumtree.xpfoundation.logging.*;
import com.plumtree.openfoundation.util.*;

import com.plumtree.xpshared.htmlelements.*;

/**
 * This class contains an enumeration of all images appearing in the Plumtree
 * portal ui. <br>
 * <br>
 * An HTMLImg object can be obtained for any one of these images from the
 * PTConstructFactory in the following way: <br>
 * <br>
 * HTMLImg i = (HTMLImg) PTConstructFactory.GetConstruct( PTImageType.MY_IMAGE
 * ).GetHTMLElement(); <br>
 * <br>
 * If you wish to add an image to this class, search for the string "ADD_HERE",
 * and you will find comments describing what to do.
 * 
 * @author beng
 */
public class PTImageType extends PTConstructType
{

	private static XPArrayList		alTypes							= new XPArrayList();

	private static XPHashtable		xpmNameToFile					= new XPHashtable();
	private static XPHashtable		xpmNameToHeight					= new XPHashtable();
	private static XPHashtable		xpmNameToWidth					= new XPHashtable();

	private static boolean			bInitCalled						= false;

	public static final PTImageType	ICON_LOCKBOX					= new PTImageType("icon_lockbox");
	public static final PTImageType	ICON_STATUS_BAR					= new PTImageType("icon_status_bar");
	public static final PTImageType	ICON_FOLDER_CRAWLER				= new PTImageType("icon_folder_crawler");
	public static final PTImageType	ICON_FOLDER_CRAWLER_OPEN		= new PTImageType("icon_folder_crawler_open");
	public static final PTImageType	ICON_COMMUNITIES_MENU			= new PTImageType("icon_communities_menu");
	public static final PTImageType	ICON_EDIT_MENU					= new PTImageType("icon_edit_menu");
	public static final PTImageType	ICON_BROWSE_MENU				= new PTImageType("icon_browse_menu");
	public static final PTImageType	ICON_MYPAGE_MENU				= new PTImageType("icon_mypage_menu");
	
	public static final PTImageType	ACTION_COMMUNITIES_EDIT			= new PTImageType("action_communities_edit");
	public static final PTImageType	ACTION_COMMUNITIES_JOIN			= new PTImageType("action_communities_join");
	public static final PTImageType	ACTION_COMMUNITIES_UNSUB		= new PTImageType("action_communities_unsub");
	public static final PTImageType	ACTION_MYPAGE_NEW				= new PTImageType("action_mypage_new");
	public static final PTImageType	ACTION_PORTLET_ADD				= new PTImageType("action_portlet_add");
	public static final PTImageType	ACTION_PORTLET_PREF				= new PTImageType("action_portlet_pref");
	public static final PTImageType	ACTION_UNSUBSCRIBE_SMALL		= new PTImageType("action_unsubscribe_small");
	public static final PTImageType	ACTION_ARROW_UP_BTN				= new PTImageType("action_arrow_up_btn");
	public static final PTImageType	ACTION_ARROW_DOWN_BTN			= new PTImageType("action_arrow_down_btn");
	public static final PTImageType	ACTION_ARROW_RIGHT_BTN			= new PTImageType("action_arrow_right_btn");
	public static final PTImageType	ACTION_ARROW_LEFT_BTN			= new PTImageType("action_arrow_left_btn");
	public static final PTImageType	ACTION_DELETE_BTN				= new PTImageType("action_delete_btn");
	public static final PTImageType	ACTION_CONTENT_SNAPSHOT			= new PTImageType("action_content_snapshot");

	public static final PTImageType	ICON_508						= new PTImageType("icon_508");
	public static final PTImageType	ICON_DEVICE						= new PTImageType("icon_device");
	public static final PTImageType	ICON_PORTLET_EXPAND				= new PTImageType("icon_portlet_expand");
	public static final PTImageType	ICON_PORTLET_COLLAPSE			= new PTImageType("icon_portlet_collapse");
	public static final PTImageType	ICON_PORTLET_EDIT				= new PTImageType("icon_portlet_edit");
	public static final PTImageType	ICON_SEARCH_RESULT				= new PTImageType("icon_search_result");
	
	public static final PTImageType	ACTION_SEARCH_NETWORK_OUT		= new PTImageType("action_search_network_out");
	public static final PTImageType	ACTION_SEARCH_NETWORK_IN		= new PTImageType("action_search_network_in");

	public static final PTImageType	ICON_WARNING					= new PTImageType("icon_warning");
	public static final PTImageType	ICON_EVENTS_PREVIEW				= new PTImageType("icon_events_preview");

	public static final PTImageType	ACTION_SUBSCRIBE				= new PTImageType("action_subscribe");
	public static final PTImageType	ACTION_SUBSCRIBE_SMALL			= new PTImageType("action_subscribe_small");

	public static final PTImageType	ICON_EDIT_DISABLED				= new PTImageType("icon_edit_disabled");

	public static final PTImageType	ACTION_DETACH_SMALL				= new PTImageType("action_detach_small");
	public static final PTImageType	ACTION_PREFS					= new PTImageType("action_prefs");

	public static final PTImageType	ICON_FOLDER_SUBPORTAL			= new PTImageType("icon_folder_subportal");
	public static final PTImageType	ICON_FOLDER_SUBPORTAL_OPEN		= new PTImageType("icon_folder_subportal_open");
	public static final PTImageType	ICON_UNAPPROVE					= new PTImageType("icon_unapprove");
	public static final PTImageType	ICON_ARROW_TOP_DISABLED			= new PTImageType("icon_arrow_top_disabled");
	public static final PTImageType	ICON_ARROW_UP_DISABLED			= new PTImageType("icon_arrow_up_disabled");
	public static final PTImageType	ICON_ARROW_DOWN_DISABLED		= new PTImageType("icon_arrow_down_disabled");
	public static final PTImageType	ICON_ARROW_BOTTOM_DISABLED		= new PTImageType("icon_arrow_bottom_disabled");
	public static final PTImageType	ICON_ADMINISTRATION				= new PTImageType("icon_administration");
	public static final PTImageType	ICON_FOLDER_PORTLET_OPEN		= new PTImageType("icon_folder_portlet_open");
	public static final PTImageType	ICON_FOLDER_WORKFLOW_OPEN		= new PTImageType("icon_folder_workflow_open");
	public static final PTImageType	ICON_FOLDER_COMMUNITY_OPEN		= new PTImageType("icon_folder_community_open");
	public static final PTImageType	ICON_FOLDER_ROOT_OPEN			= new PTImageType("icon_folder_root_open");

	public static final PTImageType	ACTION_ACTIONBAR_COLLAPSE		= new PTImageType("action_actionbar_collapse");
	public static final PTImageType	ACTION_ACTIONBAR_EXPAND			= new PTImageType("action_actionbar_expand");
	public static final PTImageType	ACTION_ADD						= new PTImageType("action_add");
	public static final PTImageType	ACTION_ALL_CHECK				= new PTImageType("action_all_check");
	public static final PTImageType	ACTION_ALL_UNCHECK				= new PTImageType("action_all_uncheck");
	public static final PTImageType	ACTION_BROWSE					= new PTImageType("action_browse");
	public static final PTImageType	ACTION_BULK_ADD					= new PTImageType("action_bulk_add");
	public static final PTImageType	ACTION_CANCEL					= new PTImageType("action_cancel");
	public static final PTImageType	ACTION_CHECK_COLUMN				= new PTImageType("action_check_column");
	public static final PTImageType	ACTION_CHECK_ROW				= new PTImageType("action_check_row");
	public static final PTImageType	ACTION_COLLAPSE_ALL				= new PTImageType("action_collapse_all");
	public static final PTImageType	ACTION_COPY						= new PTImageType("action_copy");
	public static final PTImageType	ACTION_CREATE					= new PTImageType("action_create");
	public static final PTImageType	ACTION_DELETE					= new PTImageType("action_delete");
	public static final PTImageType	ACTION_DELETE_MENU				= new PTImageType("action_delete_menu");
	public static final PTImageType	ACTION_DOWNLOAD					= new PTImageType("action_download");
	public static final PTImageType	ACTION_EDIT						= new PTImageType("action_edit");
	public static final PTImageType	ACTION_EDIT_DISABLED			= new PTImageType("action_edit_disabled");
	public static final PTImageType	ACTION_EDIT_SMALL				= new PTImageType("action_edit_small");
	public static final PTImageType	ACTION_EXPAND_ALL				= new PTImageType("action_expand_all");
	public static final PTImageType	ACTION_FOLDER_TOP				= new PTImageType("action_folder_top");
	public static final PTImageType	ACTION_FOLDER_UP				= new PTImageType("action_folder_up");
	public static final PTImageType	ACTION_MOVE						= new PTImageType("action_move");
	public static final PTImageType	ACTION_NEW						= new PTImageType("action_new");
	public static final PTImageType	ACTION_REFRESH					= new PTImageType("action_refresh");
	public static final PTImageType	ACTION_PORTLET_REFRESH					= new PTImageType("action_portlet_refresh");
	public static final PTImageType	ACTION_PORTLET_HELP					= new PTImageType("action_portlet_help");
	public static final PTImageType	ACTION_PORTLET_EDIT					= new PTImageType("action_portlet_edit");
	public static final PTImageType	ACTION_PORTLET_MENU					= new PTImageType("action_portlet_menu");
	public static final PTImageType	ACTION_PORTLET_COLLAPSE					= new PTImageType("action_portlet_collapse");
	public static final PTImageType	ACTION_PORTLET_EXPAND					= new PTImageType("action_portlet_expand");
	public static final PTImageType	ACTION_PORTLET_REMOVE					= new PTImageType("action_portlet_remove");
	
	public static final PTImageType	ACTION_REGISTER					= new PTImageType("action_register");
	public static final PTImageType	ACTION_REMOVE					= new PTImageType("action_remove");
	public static final PTImageType	ACTION_RENAME					= new PTImageType("action_rename");
	public static final PTImageType	ACTION_SAVE						= new PTImageType("action_save");
	public static final PTImageType	ACTION_SEARCH_BUTTON			= new PTImageType("action_search_button");
	public static final PTImageType	ACTION_SELECT					= new PTImageType("action_select");
	public static final PTImageType	ACTION_SEND						= new PTImageType("action_send");
	public static final PTImageType	ACTION_SORT						= new PTImageType("action_sort");
	public static final PTImageType	ACTION_SUBMIT					= new PTImageType("action_submit");
	public static final PTImageType	ACTION_TOPBESTBET_BUTTON		= new PTImageType("action_topbestbet_button");
	public static final PTImageType	ACTION_UPLOAD					= new PTImageType("action_upload");

	public static final PTImageType	ICON_APPROVE					= new PTImageType("icon_approve");
	public static final PTImageType	ICON_ARROW_ASCEND				= new PTImageType("icon_arrow_ascend");
	public static final PTImageType	ICON_ARROW_BOTTOM				= new PTImageType("icon_arrow_bottom");
	public static final PTImageType	ICON_ARROW_COLLAPSE				= new PTImageType("icon_arrow_collapse");
	public static final PTImageType	ICON_ARROW_DESCEND				= new PTImageType("icon_arrow_descend");
	public static final PTImageType	ICON_ARROW_DOWN					= new PTImageType("icon_arrow_down");
	public static final PTImageType	ICON_ARROW_EXPAND				= new PTImageType("icon_arrow_expand");
	public static final PTImageType	ICON_ARROW_MENU					= new PTImageType("icon_arrow_menu");
	public static final PTImageType	ICON_ARROW_NEXT					= new PTImageType("icon_arrow_next");
	public static final PTImageType	ICON_ARROW_PREVIOUS				= new PTImageType("icon_arrow_previous");
	public static final PTImageType	ICON_ARROW_TOP					= new PTImageType("icon_arrow_top");
	public static final PTImageType	ICON_ARROW_UP					= new PTImageType("icon_arrow_up");
	public static final PTImageType	ICON_BUTTON_EDIT				= new PTImageType("icon_button_edit");
	public static final PTImageType	ICON_CALENDAR					= new PTImageType("icon_calendar");
	public static final PTImageType	ICON_COLOR						= new PTImageType("icon_color");
	public static final PTImageType	ICON_DATE_PICKER				= new PTImageType("icon_date_picker");
	public static final PTImageType	ICON_DOCUMENT					= new PTImageType("icon_document");
	public static final PTImageType	ICON_DOCUMENTS					= new PTImageType("icon_documents");
	public static final PTImageType	ICON_DOCUMENT_URL				= new PTImageType("icon_document_url");
	public static final PTImageType	ICON_EDIT						= new PTImageType("icon_edit");
	public static final PTImageType	ICON_ERROR						= new PTImageType("icon_error");
	public static final PTImageType	ICON_FOLDER						= new PTImageType("icon_folder");
	public static final PTImageType	ICON_FOLDER_OPEN				= new PTImageType("icon_folder_open");
	public static final PTImageType	ICON_HELP						= new PTImageType("icon_help");
	public static final PTImageType	ICON_HOME						= new PTImageType("icon_home");
	public static final PTImageType	ICON_INFORMATION				= new PTImageType("icon_information");
	public static final PTImageType	ICON_MEMBER						= new PTImageType("icon_member");
	public static final PTImageType	ICON_NOTICE						= new PTImageType("icon_notice");
	public static final PTImageType	ICON_PORTLET					= new PTImageType("icon_portlet");
	public static final PTImageType	ICON_PREFS						= new PTImageType("icon_prefs");
	public static final PTImageType	ICON_PREVIEW					= new PTImageType("icon_preview");
	public static final PTImageType	ICON_PROGRESS					= new PTImageType("icon_progress");
	public static final PTImageType	ICON_REJECT						= new PTImageType("icon_reject");
	public static final PTImageType	ICON_SECURITY					= new PTImageType("icon_security");
	public static final PTImageType	ICON_TIPS_BULB					= new PTImageType("icon_tips_bulb");
	public static final PTImageType	ICON_TREE_COLLAPSE				= new PTImageType("icon_tree_collapse");
	public static final PTImageType	ICON_TREE_EXPAND				= new PTImageType("icon_tree_expand");
	public static final PTImageType	ICON_URL						= new PTImageType("icon_url");

	public static final PTImageType	SPACER							= new PTImageType("spacer");

	public static final PTImageType	ACTION_ABORT					= new PTImageType("action_abort");
	public static final PTImageType	ACTION_ALL_HOLD					= new PTImageType("action_all_hold");
	public static final PTImageType	ACTION_ALL_REJECT				= new PTImageType("action_all_reject");
	public static final PTImageType	ACTION_APPLY					= new PTImageType("action_apply");
	public static final PTImageType	ACTION_APPROVE					= new PTImageType("action_approve");
	public static final PTImageType	ACTION_APPROVE_ALL				= new PTImageType("action_approve_all");
	public static final PTImageType	ACTION_DISABLE					= new PTImageType("action_disable");
	public static final PTImageType	ACTION_ENABLE					= new PTImageType("action_enable");
	public static final PTImageType	ACTION_SECURITY					= new PTImageType("action_security");
	public static final PTImageType	ACTION_FILTER					= new PTImageType("action_filter");
	public static final PTImageType	ACTION_HISTORY_AUDIT			= new PTImageType("action_history_audit");
	public static final PTImageType	ACTION_OBJECT_MIGRATE			= new PTImageType("action_object_migrate");
	public static final PTImageType	ACTION_OBJECT_MIGRATION			= new PTImageType("action_object_migration");
	public static final PTImageType	ACTION_PROPERTIES				= new PTImageType("action_properties");
	public static final PTImageType	ACTION_RESET					= new PTImageType("action_reset");
	public static final PTImageType	ACTION_RUN_ONCE					= new PTImageType("action_run_once");
	public static final PTImageType	ACTION_SETTING_LINK				= new PTImageType("action_setting_link");
	public static final PTImageType	ACTION_UNAPPROVE				= new PTImageType("action_unapprove");
	public static final PTImageType	ACTION_UNLOCK					= new PTImageType("action_unlock");
	public static final PTImageType	ACTION_UNREGISTER				= new PTImageType("action_unregister");

	public static final PTImageType	ICON_ACTIVITY_RIGHT				= new PTImageType("icon_activity_right");
	public static final PTImageType	ICON_ACTIVITY_RIGHT_CUSTOM		= new PTImageType("icon_activity_right_custom");
	public static final PTImageType	ICON_ACTIVITY_RIGHT_INTRINSIC	= new PTImageType("icon_activity_right_intrinsic");
	public static final PTImageType	ICON_AUTH_SOURCE				= new PTImageType("icon_auth_source");
	public static final PTImageType	ICON_AUTOMATION_SERVER			= new PTImageType("icon_automation_server");
	public static final PTImageType	ICON_BANNER						= new PTImageType("icon_banner");
	public static final PTImageType	ICON_BUTTON_CLOSE				= new PTImageType("icon_button_close");
	public static final PTImageType	ICON_COLUMN						= new PTImageType("icon_column");
	public static final PTImageType	ICON_COMMUNITY					= new PTImageType("icon_community");
	public static final PTImageType	ICON_CRAWLER					= new PTImageType("icon_crawler");
	public static final PTImageType	ICON_DISPLAY_OPTIONS			= new PTImageType("icon_display_options");
	public static final PTImageType	ICON_DOCUMENT_TYPE				= new PTImageType("icon_document_type");
	public static final PTImageType	ICON_EXCLUSION					= new PTImageType("icon_exclusion");
	public static final PTImageType	ICON_EXTENSION					= new PTImageType("icon_extension");
	public static final PTImageType	ICON_FILTER						= new PTImageType("icon_filter");
	public static final PTImageType	ICON_FOLDER_COMMUNITY			= new PTImageType("icon_folder_community");
	public static final PTImageType	ICON_FOOTER						= new PTImageType("icon_footer");
	public static final PTImageType	ICON_GROUP						= new PTImageType("icon_group");
	public static final PTImageType	ICON_HOLD						= new PTImageType("icon_hold");
	public static final PTImageType	ICON_INCLUSION					= new PTImageType("icon_inclusion");
	public static final PTImageType	ICON_JOB						= new PTImageType("icon_job");
	public static final PTImageType	ICON_KNOWLEDGE_DIRECTORY		= new PTImageType("icon_knowledge_directory");
	public static final PTImageType	ICON_LAYOUT						= new PTImageType("icon_layout");
	public static final PTImageType	ICON_LOCALE_SETTINGS			= new PTImageType("icon_locale_settings");
	public static final PTImageType	ICON_LOGIN						= new PTImageType("icon_login");
	public static final PTImageType	ICON_MOBILE_SETTINGS			= new PTImageType("icon_mobile_settings");
	public static final PTImageType	ICON_MYPAGE						= new PTImageType("icon_mypage");
	public static final PTImageType	ICON_OPERATION					= new PTImageType("icon_operation");
	public static final PTImageType	ICON_PACKAGE					= new PTImageType("icon_package");
	public static final PTImageType	ICON_PAGE_TEMPLATE				= new PTImageType("icon_page_template");
	public static final PTImageType	ICON_PROFILE					= new PTImageType("icon_profile");
	public static final PTImageType	ICON_PROFILE_PAGE				= new PTImageType("icon_profile_page");
	public static final PTImageType	ICON_PROFILE_PRIVATE			= new PTImageType("icon_profile_private");
	public static final PTImageType	ICON_PROFILE_PUBLIC				= new PTImageType("icon_profile_public");
	public static final PTImageType	ICON_PROPERTY					= new PTImageType("icon_property");
	public static final PTImageType	ICON_RESOURCE					= new PTImageType("icon_resource");
	public static final PTImageType	ICON_ROW						= new PTImageType("icon_row");
	public static final PTImageType	ICON_SCHEME						= new PTImageType("icon_scheme");
	public static final PTImageType	ICON_SERVER						= new PTImageType("icon_server");
	public static final PTImageType	ICON_SMART_SORT					= new PTImageType("action_smart_sort");
	public static final PTImageType	ICON_USER						= new PTImageType("icon_user");
	public static final PTImageType	ICON_USER_LIST					= new PTImageType("icon_user_list");
	public static final PTImageType	ICON_VALUE						= new PTImageType("icon_value");
	public static final PTImageType	ICON_WEB_SERVICE				= new PTImageType("icon_web_service");

	public static final PTImageType	LAYOUT0							= new PTImageType("layout0");
	public static final PTImageType	LAYOUT1							= new PTImageType("layout1");
	public static final PTImageType	LAYOUT10						= new PTImageType("layout10");
	public static final PTImageType	LAYOUT11						= new PTImageType("layout11");
	public static final PTImageType	LAYOUT12						= new PTImageType("layout12");
	public static final PTImageType	LAYOUT2							= new PTImageType("layout2");
	public static final PTImageType	LAYOUT3							= new PTImageType("layout3");
	public static final PTImageType	LAYOUT4							= new PTImageType("layout4");
	public static final PTImageType	LAYOUT5							= new PTImageType("layout5");
	public static final PTImageType	LAYOUT6							= new PTImageType("layout6");
	public static final PTImageType	LAYOUT7							= new PTImageType("layout7");
	public static final PTImageType	LAYOUT8							= new PTImageType("layout8");
	public static final PTImageType	LAYOUT9							= new PTImageType("layout9");

	public static final PTImageType	TEMPLATE_PORTLET_BAN			= new PTImageType("template_portlet_ban");
	public static final PTImageType	TEMPLATE_PORTLET_FOOTER			= new PTImageType("template_portlet_footer");
	public static final PTImageType	TEMPLATE_PORTLET_FREEFORM		= new PTImageType("template_portlet_freeform");
	public static final PTImageType	TEMPLATE_PORTLET_NARROW			= new PTImageType("template_portlet_narrow");
	public static final PTImageType	TEMPLATE_PORTLET_WIDE			= new PTImageType("template_portlet_wide");

	public static final PTImageType	TODO_REPLACE_ME					= new PTImageType("todo_replace_me");

	public static final PTImageType	ICON_DRAFT						= new PTImageType("icon_draft");
	public static final PTImageType	ICON_MESSAGE_SENT				= new PTImageType("icon_message_sent");
	public static final PTImageType	ICON_MOBILE						= new PTImageType("icon_mobile");
	public static final PTImageType	ICON_PORTLET_BUNDLE				= new PTImageType("icon_portlet_bundle");
	public static final PTImageType	ICON_PUBLISH					= new PTImageType("icon_publish");
	public static final PTImageType	ICON_SEARCH						= new PTImageType("icon_search");
	public static final PTImageType	ICON_SEARCH_TOPBESTBET			= new PTImageType("icon_search_topbestbet");
	public static final PTImageType	ICON_WORKFLOW					= new PTImageType("icon_workflow");
	public static final PTImageType	ICON_COMMUNITY_TEMPLATE			= new PTImageType("icon_community_template");
	public static final PTImageType	ICON_DATA_SOURCE				= new PTImageType("icon_data_source");
	public static final PTImageType	ICON_EXTERNAL_OPERATION			= new PTImageType("icon_external_operation");
	public static final PTImageType	ICON_INVITATION					= new PTImageType("icon_invitation");
	public static final PTImageType	ICON_NETWORK_SEARCH				= new PTImageType("icon_network_search");
	public static final PTImageType	ICON_PORTLET_TEMPLATE			= new PTImageType("icon_portlet_template");
	public static final PTImageType	ICON_PROFILE_SOURCE				= new PTImageType("icon_profile_source");
	public static final PTImageType	ICON_REMOTE_SERVER				= new PTImageType("icon_remote_server");
	public static final PTImageType	ICON_SAVED_SEARCH				= new PTImageType("icon_saved_search");
	public static final PTImageType	ICON_SUB_PORTAL					= new PTImageType("icon_sub_portal");
	
	public static final PTImageType	ICON_SUBPORTAL_RULE				= new PTImageType("icon_subportal_rule");

	public static final PTImageType	ACTION_SEARCH_BUTTON_ADVANCED	= new PTImageType("action_search_button_advanced");
	public static final PTImageType	ACTION_SEARCH_BUTTON_NETWORK	= new PTImageType("action_search_button_network");

	public static final PTImageType	LOGO_SML_WHT					= new PTImageType("pt_logo_sml_wht");
	public static final PTImageType	LOGO_POWERED_BY_JAVA			= new PTImageType("pt_logo_poweredbyjava");
    public static final PTImageType LOGO_HEADER_BEA                 = new PTImageType("logo_header_bea");
    public static final PTImageType LOGO_HEADER_SPLASH              = new PTImageType("logo_header_splash");
    public static final PTImageType LOGO_BANNER_ORACLE              = new PTImageType("banner_logo");

	static
	{
		alTypes.Add(ICON_LOCKBOX);
		alTypes.Add(ICON_STATUS_BAR);

		alTypes.Add(ICON_FOLDER_CRAWLER);
		alTypes.Add(ICON_FOLDER_CRAWLER_OPEN);

		alTypes.Add(ICON_COMMUNITIES_MENU);
		alTypes.Add(ICON_EDIT_MENU);
		alTypes.Add(ICON_BROWSE_MENU);
		alTypes.Add(ICON_MYPAGE_MENU);

		alTypes.Add(ACTION_COMMUNITIES_EDIT);
		alTypes.Add(ACTION_COMMUNITIES_JOIN);
		alTypes.Add(ACTION_COMMUNITIES_UNSUB);
		alTypes.Add(ACTION_MYPAGE_NEW);
		alTypes.Add(ACTION_PORTLET_ADD);
		alTypes.Add(ACTION_PORTLET_PREF);

		alTypes.Add(ACTION_ABORT);
		alTypes.Add(ACTION_ACTIONBAR_COLLAPSE);
		alTypes.Add(ACTION_ACTIONBAR_EXPAND);
		alTypes.Add(ACTION_ADD);
		alTypes.Add(ACTION_ALL_CHECK);
		alTypes.Add(ACTION_ALL_HOLD);
		alTypes.Add(ACTION_ALL_REJECT);
		alTypes.Add(ACTION_ALL_UNCHECK);
		alTypes.Add(ACTION_APPLY);
		alTypes.Add(ACTION_APPROVE);
		alTypes.Add(ACTION_APPROVE_ALL);

		alTypes.Add(ACTION_ARROW_DOWN_BTN);
		alTypes.Add(ACTION_ARROW_LEFT_BTN);
		alTypes.Add(ACTION_ARROW_RIGHT_BTN);
		alTypes.Add(ACTION_ARROW_UP_BTN);

		alTypes.Add(ACTION_BROWSE);
		alTypes.Add(ACTION_BULK_ADD);
		alTypes.Add(ACTION_CANCEL);
		alTypes.Add(ACTION_CHECK_COLUMN);
		alTypes.Add(ACTION_CHECK_ROW);
		alTypes.Add(ACTION_COLLAPSE_ALL);
		alTypes.Add(ACTION_CONTENT_SNAPSHOT);
		alTypes.Add(ACTION_COPY);
		alTypes.Add(ACTION_CREATE);
		alTypes.Add(ACTION_DELETE);
		alTypes.Add(ACTION_DELETE_MENU);
		alTypes.Add(ACTION_DELETE_BTN);
		alTypes.Add(ACTION_DETACH_SMALL);
		alTypes.Add(ACTION_DISABLE);
		alTypes.Add(ACTION_DOWNLOAD);
		alTypes.Add(ACTION_EDIT);
		alTypes.Add(ACTION_EDIT_DISABLED);
		alTypes.Add(ACTION_EDIT_SMALL);
		alTypes.Add(ACTION_ENABLE);
		alTypes.Add(ACTION_EXPAND_ALL);
		alTypes.Add(ACTION_FILTER);

		//        alTypes.Add(ACTION_FOLDER_DELETED);
		alTypes.Add(ACTION_FOLDER_TOP);
		alTypes.Add(ACTION_FOLDER_UP);
		alTypes.Add(ACTION_HISTORY_AUDIT);
		alTypes.Add(ACTION_MOVE);
		alTypes.Add(ACTION_NEW);
		alTypes.Add(ACTION_OBJECT_MIGRATE);
		alTypes.Add(ACTION_OBJECT_MIGRATION);
		alTypes.Add(ACTION_PREFS);
		alTypes.Add(ACTION_PROPERTIES);
		alTypes.Add(ACTION_REFRESH);
		
		alTypes.Add(ACTION_PORTLET_REFRESH);
		alTypes.Add(ACTION_PORTLET_COLLAPSE);
		alTypes.Add(ACTION_PORTLET_EXPAND);
		alTypes.Add(ACTION_PORTLET_REMOVE);
		alTypes.Add(ACTION_PORTLET_MENU);
		alTypes.Add(ACTION_PORTLET_EDIT);
		alTypes.Add(ACTION_PORTLET_HELP);
		
		
		alTypes.Add(ACTION_REGISTER);
		alTypes.Add(ACTION_REMOVE);
		alTypes.Add(ACTION_RENAME);
		alTypes.Add(ACTION_RESET);
		alTypes.Add(ACTION_RUN_ONCE);
		alTypes.Add(ACTION_SAVE);
		alTypes.Add(ACTION_SEARCH_BUTTON);
		alTypes.Add(ACTION_TOPBESTBET_BUTTON);
		
		alTypes.Add(ACTION_SEARCH_BUTTON_ADVANCED);
		alTypes.Add(ACTION_SEARCH_BUTTON_NETWORK);

		//        alTypes.Add( ACTION_SEARCH_BUTTON_FEDERATED );
		alTypes.Add(ACTION_SEARCH_NETWORK_IN);
		alTypes.Add(ACTION_SEARCH_NETWORK_OUT);
		alTypes.Add(ACTION_SECURITY);
		alTypes.Add(ACTION_SELECT);
		alTypes.Add(ACTION_SEND);
		alTypes.Add(ACTION_SETTING_LINK);
		alTypes.Add(ACTION_SORT);
		alTypes.Add(ACTION_SUBSCRIBE);
		alTypes.Add(ACTION_SUBSCRIBE_SMALL);
		alTypes.Add(ACTION_SUBMIT);
		alTypes.Add(ACTION_UNAPPROVE);
		alTypes.Add(ACTION_UNLOCK);
		alTypes.Add(ACTION_UNREGISTER);
		alTypes.Add(ACTION_UNSUBSCRIBE_SMALL);
		alTypes.Add(ACTION_UPLOAD);

		//        alTypes.Add(ACTION_UTILITY_EDIT);
		alTypes.Add(ICON_508);
		alTypes.Add(ICON_ACTIVITY_RIGHT);
		alTypes.Add(ICON_ACTIVITY_RIGHT_CUSTOM);
		alTypes.Add(ICON_ACTIVITY_RIGHT_INTRINSIC);
		alTypes.Add(ICON_ADMINISTRATION);
		alTypes.Add(ICON_APPROVE);
		alTypes.Add(ICON_ARROW_ASCEND);
		alTypes.Add(ICON_ARROW_BOTTOM);
		alTypes.Add(ICON_ARROW_BOTTOM_DISABLED);
		alTypes.Add(ICON_ARROW_COLLAPSE);
		alTypes.Add(ICON_ARROW_DESCEND);
		alTypes.Add(ICON_ARROW_DOWN);
		alTypes.Add(ICON_ARROW_DOWN_DISABLED);
		alTypes.Add(ICON_ARROW_EXPAND);
		alTypes.Add(ICON_ARROW_MENU);
		alTypes.Add(ICON_ARROW_NEXT);
		alTypes.Add(ICON_ARROW_PREVIOUS);
		alTypes.Add(ICON_ARROW_TOP);
		alTypes.Add(ICON_ARROW_TOP_DISABLED);
		alTypes.Add(ICON_ARROW_UP);
		alTypes.Add(ICON_ARROW_UP_DISABLED);
		alTypes.Add(ICON_AUTH_SOURCE);
		alTypes.Add(ICON_AUTOMATION_SERVER);
		alTypes.Add(ICON_BANNER);
		alTypes.Add(ICON_BUTTON_CLOSE);
		alTypes.Add(ICON_BUTTON_EDIT);
		alTypes.Add(ICON_CALENDAR);

		//        alTypes.Add( ICON_CHECK_IN );
		//        alTypes.Add(ICON_CHECK_OUT);
		alTypes.Add(ICON_COLOR);
		alTypes.Add(ICON_COLUMN);
		alTypes.Add(ICON_COMMUNITY);
		alTypes.Add(ICON_COMMUNITY_TEMPLATE);
		alTypes.Add(ICON_CRAWLER);
		alTypes.Add(ICON_DATA_SOURCE);
		alTypes.Add(ICON_DATE_PICKER);
		alTypes.Add(ICON_DEVICE);
		alTypes.Add(ICON_DISPLAY_OPTIONS);
		alTypes.Add(ICON_DOCUMENT);
		alTypes.Add(ICON_DOCUMENTS);
		alTypes.Add(ICON_DOCUMENT_URL);
		alTypes.Add(ICON_DOCUMENT_TYPE);
		alTypes.Add(ICON_DRAFT);
		alTypes.Add(ICON_EDIT);
		alTypes.Add(ICON_EDIT_DISABLED);
		alTypes.Add(ICON_ERROR);
		alTypes.Add(ICON_EVENTS_PREVIEW);
		alTypes.Add(ICON_EXCLUSION);
		alTypes.Add(ICON_EXTENSION);
		alTypes.Add(ICON_EXTERNAL_OPERATION);
		alTypes.Add(ICON_FILTER);
		alTypes.Add(ICON_FOLDER);
		alTypes.Add(ICON_FOLDER_COMMUNITY);
		alTypes.Add(ICON_FOLDER_OPEN);
		alTypes.Add(ICON_FOLDER_COMMUNITY_OPEN);
		alTypes.Add(ICON_FOLDER_PORTLET_OPEN);
		alTypes.Add(ICON_FOLDER_ROOT_OPEN);
		alTypes.Add(ICON_FOLDER_SUBPORTAL);
		alTypes.Add(ICON_FOLDER_SUBPORTAL_OPEN);
		alTypes.Add(ICON_FOLDER_WORKFLOW_OPEN);
		alTypes.Add(ICON_FOOTER);
		alTypes.Add(ICON_GROUP);
		alTypes.Add(ICON_HELP);
		alTypes.Add(ICON_HOLD);
		alTypes.Add(ICON_HOME);
		alTypes.Add(ICON_INCLUSION);
		alTypes.Add(ICON_INFORMATION);
		alTypes.Add(ICON_INVITATION);
		alTypes.Add(ICON_JOB);

		//        alTypes.Add(ICON_JOB_SERVER);
		alTypes.Add(ICON_KNOWLEDGE_DIRECTORY);
		alTypes.Add(ICON_LAYOUT);
		alTypes.Add(ICON_LOCALE_SETTINGS);
		alTypes.Add(ICON_LOGIN);

		//        alTypes.Add( ICON_MAP_FOLDER );
		alTypes.Add(ICON_MEMBER);
		alTypes.Add(ICON_MESSAGE_SENT);
		alTypes.Add(ICON_MOBILE);
		alTypes.Add(ICON_MOBILE_SETTINGS);
		alTypes.Add(ICON_MYPAGE);
		alTypes.Add(ICON_NETWORK_SEARCH);
		alTypes.Add(ICON_NOTICE);
		alTypes.Add(ICON_OPERATION);
		alTypes.Add(ICON_PACKAGE);
		alTypes.Add(ICON_PAGE_TEMPLATE);
		alTypes.Add(ICON_PORTLET);
		alTypes.Add(ICON_PORTLET_BUNDLE);
		alTypes.Add(ICON_PORTLET_COLLAPSE);
		alTypes.Add(ICON_PORTLET_EDIT);
		alTypes.Add(ICON_PORTLET_EXPAND);
		alTypes.Add(ICON_PORTLET_TEMPLATE);
		alTypes.Add(ICON_PREFS);
		alTypes.Add(ICON_PREVIEW);
		alTypes.Add(ICON_PROFILE);
		alTypes.Add(ICON_PROFILE_PAGE);
		alTypes.Add(ICON_PROFILE_PRIVATE);
		alTypes.Add(ICON_PROFILE_PUBLIC);
		alTypes.Add(ICON_PROFILE_SOURCE);
		alTypes.Add(ICON_PROGRESS);
		alTypes.Add(ICON_PROPERTY);
		alTypes.Add(ICON_PUBLISH);
		alTypes.Add(ICON_REJECT);
		alTypes.Add(ICON_REMOTE_SERVER);
		alTypes.Add(ICON_RESOURCE);
		alTypes.Add(ICON_ROW);
		alTypes.Add(ICON_SAVED_SEARCH);
		alTypes.Add(ICON_SCHEME);
		alTypes.Add(ICON_SEARCH);
		alTypes.Add(ICON_SEARCH_TOPBESTBET);
		alTypes.Add(ICON_SEARCH_RESULT);

		//        alTypes.Add(ICON_SEARCH_PREFS);
		alTypes.Add(ICON_SECURITY);
		alTypes.Add(ICON_SERVER);

		//        alTypes.Add( ICON_SITE_MAP_LINK );
		alTypes.Add(ICON_SMART_SORT);

		//        alTypes.Add(ACTION_SMART_SORT);
		alTypes.Add(ICON_SUB_PORTAL);

		//        alTypes.Add( ICON_TAXONOMIST );
		//        alTypes.Add(ICON_TEMPLATE_PROJECT);
		alTypes.Add(ICON_TIPS_BULB);
		alTypes.Add(ICON_TREE_COLLAPSE);
		alTypes.Add(ICON_TREE_EXPAND);
		alTypes.Add(ICON_UNAPPROVE);
		alTypes.Add(ICON_URL);
		alTypes.Add(ICON_USER);
		alTypes.Add(ICON_USER_LIST);
		alTypes.Add(ICON_VALUE);
		alTypes.Add(ICON_WARNING);
		alTypes.Add(ICON_WEB_SERVICE);

		alTypes.Add(ICON_WORKFLOW);

		alTypes.Add(LAYOUT0);
		alTypes.Add(LAYOUT1);
		alTypes.Add(LAYOUT10);
		alTypes.Add(LAYOUT11);
		alTypes.Add(LAYOUT12);
		alTypes.Add(LAYOUT2);
		alTypes.Add(LAYOUT3);
		alTypes.Add(LAYOUT4);
		alTypes.Add(LAYOUT5);
		alTypes.Add(LAYOUT6);
		alTypes.Add(LAYOUT7);
		alTypes.Add(LAYOUT8);
		alTypes.Add(LAYOUT9);

		alTypes.Add(LOGO_SML_WHT);

		alTypes.Add(SPACER);

		alTypes.Add(TEMPLATE_PORTLET_BAN);
		alTypes.Add(TEMPLATE_PORTLET_FOOTER);
		alTypes.Add(TEMPLATE_PORTLET_FREEFORM);
		alTypes.Add(TEMPLATE_PORTLET_NARROW);
		alTypes.Add(TEMPLATE_PORTLET_WIDE);
		
		alTypes.Add(ICON_SUBPORTAL_RULE);

        alTypes.Add(LOGO_HEADER_BEA);
        alTypes.Add(LOGO_HEADER_SPLASH);
        alTypes.Add(LOGO_BANNER_ORACLE);
        
		alTypes.Add(TODO_REPLACE_ME);
	}

	public static final String		ELEMENT_NAME_ID_IMG				= "img_enid";

	private ElementName				eName;

	private String					sDescription;
	private String					sFileName;
	private String					sHeight;
	private String					sName;
	private String					sWidth;

	/**
	 * Creates a new PTImageType object.
	 * 
	 * @param s
	 */
	private PTImageType(String s) {
		PTImageType.staticInit();

		sName = s;
		eName = new ElementName(s + "_img");

		sFileName = (String) xpmNameToFile.GetElement(sName);
		sHeight = (String) xpmNameToHeight.GetElement(sName);
		sWidth = (String) xpmNameToWidth.GetElement(sName);

		if (sFileName == null)
		{
			//            if (bError)
			//            {
			//                myLogger.Error("Couldnt' get file name for PTImageType: " + s);
			//            }
		}

		sDescription = "Image " + s + " uses file: " + sFileName + ", height: " + sHeight + ", width: " + sWidth;
	}

	/**
	 * 
	 * 
	 * @return
	 */
	public String GetDescription()
	{
		return sDescription;
	}

	/**
	 * 
	 * 
	 * @return
	 */
	public static IXPEnumerator GetImageNames()
	{
		//   if a name doesn't have an associated file, it's not a valid
		//   image name.
		return xpmNameToFile.GetKeys();
	}

	/**
	 * 
	 * 
	 * @param sElementNameID
	 * 
	 * @return
	 * 
	 * @throws XPException
	 */
	public ElementName GetElementName(String sElementNameID)
	{
		ElementName eReturn;

		if (ELEMENT_NAME_ID_IMG.equals(sElementNameID))
		{
			eReturn = eName;
		} else
		{
			throw new XPException("This is not a valid element name id.  See public enum.");
		}

		return eReturn;
	}

	/**
	 * 
	 * 
	 * @return
	 */
	public String GetFileName()
	{
		return sFileName;
	}

	/**
	 * 
	 * 
	 * @return
	 */
	public HTMLElement GetHTMLElement()
	{
		String sPath = GetFileName();

		HTMLImg i = new HTMLImg(sPath, null, this.GetHeight(), this.GetWidth());

		// To be deprecated
		i.SetBorder("0");

		if (this == PTImageType.SPACER)
		{
			i.SetAlt("");
		}

		return (HTMLElement) i;
	}

	/**
	 * 
	 * 
	 * @return
	 */
	public String GetHeight()
	{
		return sHeight;
	}

	/**
	 * Images don't come with alts because we don't know offhand what they will
	 * be used for.
	 * 
	 * @return
	 */
	public XPHashtable GetImgElementNameToAltIdMap()
	{
		return null;
	}

	/**
	 * 
	 * 
	 * @return
	 */
	public XPHashtable GetStringMap()
	{
		return null;
	}

	/**
	 * Call this method to obtain an enumeration of all available PTImageType
	 * objects.
	 * 
	 * @return
	 */
	public static IXPEnumerator GetTypes()
	{
		return alTypes.GetEnumerator();
	}

	/**
	 * 
	 * 
	 * @return
	 */
	public String GetWidth()
	{
		return sWidth;
	}

	/**
	 * 
	 * 
	 * @return
	 */
	public String toString()
	{
		return XPStringUtility.ToUpperCase(sName);
	}

	/**
	 * 
	 * 
	 * @param sName
	 * @param sFileName
	 * @param sHeight
	 * @param sWidth
	 */
	private static void InitializeImage(String sName, String sFileName, String sHeight, String sWidth)
	{
		xpmNameToFile.PutElement(sName, sFileName);
		xpmNameToHeight.PutElement(sName, sHeight);
		xpmNameToWidth.PutElement(sName, sWidth);
	}

	/**
	 *  
	 */
	private static void staticInit()
	{
		if (bInitCalled)
		{
			return;
		}

		bInitCalled = true;

		InitializeImage("icon_lockbox", "portal/public/img/icon_lockbox.gif", "20", "20");
		InitializeImage("icon_status_bar", "portal/public/img/icon_status_bar.gif", "16", "233");
		InitializeImage("icon_folder_crawler", "portal/private/img/icon_folder_crawler.gif", "20", "20");
		InitializeImage("icon_folder_crawler_open", "portal/private/img/icon_folder_crawler_open.gif", "20", "20");
		InitializeImage("icon_communities_menu", "portal/public/img/icon_communities_menu.gif", "20", "20");
		InitializeImage("icon_edit_menu", "portal/public/img/icon_edit_menu.gif", "20", "20");
		InitializeImage("icon_browse_menu", "portal/public/img/icon_browse_menu.gif", "20", "20");
		InitializeImage("icon_mypage_menu", "portal/public/img/icon_mypage_menu.gif", "20", "20");
		InitializeImage("action_communities_edit", "portal/private/img/action_communities_edit.gif", "20", "20");
		InitializeImage("action_communities_join", "portal/private/img/action_communities_join.gif", "20", "20");
		InitializeImage("action_communities_unsub", "portal/private/img/action_communities_unsub.gif", "20", "20");
		InitializeImage("action_mypage_new", "portal/private/img/action_mypage_new.gif", "20", "20");
		InitializeImage("action_portlet_add", "portal/private/img/action_portlet_add.gif", "20", "20");
		InitializeImage("action_portlet_pref", "portal/private/img/action_portlet_pref.gif", "20", "20");
		InitializeImage("action_unsubscribe_small", "portal/public/img/action_unsubscribe_small.gif", "25", "10");
		InitializeImage("action_arrow_left_btn", "portal/public/img/action_arrow_left_btn.gif", "20", "20");
		InitializeImage("action_arrow_right_btn", "portal/public/img/action_arrow_right_btn.gif", "20", "20");
		InitializeImage("action_arrow_down_btn", "portal/public/img/action_arrow_down_btn.gif", "20", "20");
		InitializeImage("action_arrow_up_btn", "portal/public/img/action_arrow_up_btn.gif", "20", "20");
		InitializeImage("action_delete_btn", "portal/public/img/action_delete_btn.gif", "20", "20");
		InitializeImage("action_content_snapshot", "portal/private/img/action_content_snapshot.gif", "25", "25");
		InitializeImage("icon_508", "portal/public/img/icon_508.gif", "20", "20");
		InitializeImage("icon_device", "portal/public/img/icon_device.gif", "20", "20");
		InitializeImage("icon_portlet_expand", "portal/public/img/icon_portlet_expand.gif", "16", "16");
		InitializeImage("icon_portlet_collapse", "portal/public/img/icon_portlet_collapse.gif", "16", "16");
		InitializeImage("icon_portlet_edit", "portal/public/img/icon_portlet_edit.gif", "16", "16");
		InitializeImage("icon_search_result", "portal/public/img/icon_search_result.gif", "20", "15");
		InitializeImage("action_search_network_out", "portal/public/img/action_search_network_out.gif", "20", "20");
		InitializeImage("action_search_network_in", "portal/public/img/action_search_network_in.gif", "20", "20");
		InitializeImage("icon_warning", "portal/public/img/icon_warning.gif", "20", "20");
		InitializeImage("icon_events_preview", "portal/private/img/action_event_preview.gif", "25", "25");
		InitializeImage("action_subscribe", "portal/public/img/action_subscribe.gif", "25", "25");
		InitializeImage("action_subscribe_small", "portal/public/img/action_subscribe_small.gif", "25", "10");
		InitializeImage("icon_edit_disabled", "portal/public/img/icon_edit_disabled.gif", "20", "20");
		InitializeImage("action_detach_small", "portal/public/img/action_detach_small.gif", "25", "9");
		InitializeImage("action_prefs", "portal/public/img/action_prefs.gif", "25", "7");
		InitializeImage("icon_folder_subportal", "portal/private/img/icon_folder_subportal.gif", "20", "20");
		InitializeImage("icon_folder_subportal_open", "portal/private/img/icon_folder_subportal_open.gif", "20", "20");
		InitializeImage("icon_arrow_top_disabled", "portal/public/img/icon_arrow_top_disabled.gif", "20", "15");
		InitializeImage("icon_arrow_up_disabled", "portal/public/img/icon_arrow_up_disabled.gif", "20", "15");
		InitializeImage("icon_arrow_down_disabled", "portal/public/img/icon_arrow_down_disabled.gif", "20", "15");
		InitializeImage("icon_arrow_bottom_disabled", "portal/public/img/icon_arrow_bottom_disabled.gif", "20", "15");
		InitializeImage("icon_administration", "portal/public/img/icon_administration.gif", "20", "20");
		InitializeImage("icon_folder_workflow_open", "portal/private/img/icon_folder_workflow_open.gif", "20", "20");
		InitializeImage("icon_folder_portlet_open", "portal/private/img/icon_folder_portlet_open.gif", "20", "20");
		InitializeImage("icon_folder_community_open", "portal/private/img/icon_folder_community_open.gif", "20", "20");
		InitializeImage("icon_folder_root_open", "portal/private/img/icon_folder_root_open.gif", "20", "20");
		InitializeImage("icon_unapprove", "portal/public/img/icon_unapprove.gif", "20", "10");
		InitializeImage("action_expand_all", "portal/public/img/action_expand_all.gif", "25", "25");
		InitializeImage("action_collapse_all", "portal/public/img/action_collapse_all.gif", "25", "25");
		InitializeImage("action_actionbar_collapse", "portal/public/img/action_actionbar_collapse.gif", "25", "25");
		InitializeImage("action_actionbar_expand", "portal/public/img/action_actionbar_expand.gif", "25", "25");
		InitializeImage("action_add", "portal/public/img/action_add.gif", "25", "6");
		InitializeImage("action_all_check", "portal/public/img/action_all_check.gif", "25", "25");
		InitializeImage("action_all_uncheck", "portal/public/img/action_all_uncheck.gif", "25", "25");
		InitializeImage("action_browse", "portal/public/img/action_browse.gif", "25", "25");
		InitializeImage("action_bulk_add", "portal/public/img/action_bulk_add.gif", "25", "25");
		InitializeImage("action_check_column", "portal/public/img/action_check_column.gif", "25", "25");
		InitializeImage("action_check_row", "portal/public/img/action_check_row.gif", "25", "25");
		InitializeImage("action_cancel", "portal/public/img/action_cancel.gif", "25", "25");
		InitializeImage("action_copy", "portal/public/img/action_copy.gif", "25", "25");
		InitializeImage("action_create", "portal/public/img/action_create.gif", "25", "25");
		InitializeImage("action_delete", "portal/public/img/action_delete.gif", "25", "25");
		InitializeImage("action_delete_menu", "portal/public/img/action_delete_menu.gif", "20", "20");
		InitializeImage("action_download", "portal/public/img/action_download.gif", "25", "25");
		InitializeImage("action_edit", "portal/public/img/action_edit.gif", "25", "25");
		InitializeImage("action_edit_disabled", "portal/public/img/action_edit_disabled.gif", "25", "25");
		InitializeImage("action_edit_small", "portal/public/img/action_edit_small.gif", "25", "10");
		InitializeImage("action_folder_deleted", "portal/public/img/action_folder_deleted.gif", "25", "20");
		InitializeImage("action_folder_top", "portal/public/img/action_folder_top.gif", "25", "8");
		InitializeImage("action_folder_up", "portal/public/img/action_folder_up.gif", "25", "8");
		InitializeImage("action_move", "portal/public/img/action_move.gif", "25", "10");
		InitializeImage("action_new", "portal/public/img/action_new.gif", "25", "6");
		InitializeImage("action_refresh", "portal/public/img/action_refresh.gif", "25", "25");
		
		InitializeImage("action_portlet_refresh", "portal/public/img/action_portlet_refresh.gif", "13", "13");
		InitializeImage("action_portlet_help", "portal/public/img/action_portlet_help.gif", "13", "13");
		InitializeImage("action_portlet_menu", "portal/public/img/action_portlet_menu.gif", "13", "9");
		InitializeImage("action_portlet_edit", "portal/public/img/action_portlet_edit.gif", "13", "13");
		InitializeImage("action_portlet_expand", "portal/public/img/action_portlet_expand.gif", "13", "13");
		InitializeImage("action_portlet_collapse", "portal/public/img/action_portlet_collapse.gif", "13", "13");
		InitializeImage("action_portlet_remove", "portal/public/img/action_portlet_remove.gif", "13", "13");
		
		
		
		InitializeImage("action_register", "portal/public/img/action_register.gif", "25", "25");
		InitializeImage("action_remove", "portal/public/img/action_remove.gif", "25", "25");
		InitializeImage("action_rename", "portal/public/img/action_rename.gif", "25", "25");
		InitializeImage("action_save", "portal/public/img/action_save.gif", "25", "25");
		InitializeImage("action_search_button", "portal/public/img/action_search_button.gif", "20", "30");
		InitializeImage("action_topbestbet_button", "portal/public/img/action_search_topbestbet.gif", "20", "30");
		InitializeImage("action_select", "portal/public/img/action_select.gif", "25", "6");
		InitializeImage("action_send", "portal/public/img/action_send.gif", "25", "25");
		InitializeImage("action_sort", "portal/public/img/action_sort.gif", "25", "25");
		InitializeImage("action_submit", "portal/public/img/action_submit.gif", "25", "25");
		InitializeImage("action_upload", "portal/public/img/action_upload.gif", "25", "25");
		InitializeImage("icon_approve", "portal/public/img/icon_approve.gif", "20", "10");
		InitializeImage("icon_arrow_ascend", "portal/public/img/icon_arrow_ascend.gif", "20", "11");
		InitializeImage("icon_arrow_bottom", "portal/public/img/icon_arrow_bottom.gif", "20", "15");
		InitializeImage("icon_arrow_collapse", "portal/public/img/icon_arrow_collapse.gif", "16", "16");
		InitializeImage("icon_arrow_descend", "portal/public/img/icon_arrow_descend.gif", "20", "11");
		InitializeImage("icon_arrow_down", "portal/public/img/icon_arrow_down.gif", "20", "15");
		InitializeImage("icon_arrow_expand", "portal/public/img/icon_arrow_expand.gif", "16", "16");
		InitializeImage("icon_arrow_menu", "portal/public/img/icon_arrow_menu.gif", "16", "16");
		InitializeImage("icon_arrow_next", "portal/public/img/icon_arrow_next.gif", "16", "16");
		InitializeImage("icon_arrow_previous", "portal/public/img/icon_arrow_previous.gif", "16", "16");
		InitializeImage("icon_arrow_top", "portal/public/img/icon_arrow_top.gif", "20", "15");
		InitializeImage("icon_arrow_up", "portal/public/img/icon_arrow_up.gif", "20", "15");
		InitializeImage("icon_button_edit", "portal/public/img/icon_button_edit.gif", "20", "20");
		InitializeImage("icon_calendar", "portal/public/img/icon_calendar.gif", "20", "19");
		InitializeImage("icon_check_out", "collab/img/icon_check_out.gif", "20", "7");
		InitializeImage("icon_color", "portal/public/img/icon_color.gif", "20", "20");
		InitializeImage("icon_date_picker", "portal/public/img/icon_date_picker.gif", "20", "25");
		InitializeImage("icon_document", "portal/public/img/icon_document.gif", "20", "14");
		InitializeImage("icon_documents", "portal/public/img/icon_documents.gif", "20", "20");
		InitializeImage("icon_document_url", "portal/public/img/icon_document_url.gif", "20", "20");
		InitializeImage("icon_edit", "portal/public/img/icon_edit.gif", "20", "20");
		InitializeImage("icon_error", "portal/public/img/icon_error.gif", "20", "20");
		InitializeImage("icon_folder", "portal/public/img/icon_folder.gif", "20", "18");
		InitializeImage("icon_folder_open", "portal/public/img/icon_folder_open.gif", "20", "20");
		InitializeImage("icon_help", "portal/public/img/icon_help.gif", "16", "16");
		InitializeImage("icon_home", "portal/public/img/icon_home.gif", "20", "20");
		InitializeImage("icon_information", "portal/public/img/icon_information.gif", "20", "20");
		InitializeImage("icon_member", "portal/public/img/icon_member.gif", "20", "18");
		InitializeImage("icon_notice", "portal/public/img/icon_notice.gif", "20", "20");
		InitializeImage("icon_portlet", "portal/public/img/icon_portlet.gif", "20", "20");
		InitializeImage("icon_prefs", "portal/public/img/icon_prefs.gif", "20", "7");
		InitializeImage("icon_preview", "portal/public/img/icon_preview.gif", "8", "20");
		InitializeImage("icon_reject", "portal/public/img/icon_reject.gif", "20", "20");
		InitializeImage("icon_security", "portal/public/img/icon_security.gif", "20", "20");
		InitializeImage("icon_tips_bulb", "portal/public/img/icon_tips_bulb.gif", "20", "20");
		InitializeImage("icon_tree_collapse", "portal/public/img/icon_tree_collapse.gif", "20", "20");
		InitializeImage("icon_tree_expand", "portal/public/img/icon_tree_expand.gif", "20", "20");
		InitializeImage("icon_url", "portal/public/img/icon_url.gif", "20", "20");
		InitializeImage("spacer", "portal/public/img/sp.gif", "1", "1");
		InitializeImage("action_abort", "portal/private/img/action_abort.gif", "25", "25");
		InitializeImage("action_all_hold", "portal/private/img/action_all_hold.gif", "25", "25");
		InitializeImage("action_all_reject", "portal/public/img/action_all_reject.gif", "25", "25");
		InitializeImage("action_apply", "portal/private/img/action_apply.gif", "25", "25");
		InitializeImage("action_approve", "portal/private/img/action_approve.gif", "25", "25");
		InitializeImage("action_approve_all", "portal/private/img/action_approve_all.gif", "25", "25");
		InitializeImage("action_enable", "portal/public/img/action_enable.gif", "25", "25");
		InitializeImage("action_disable", "portal/public/img/action_disable.gif", "25", "25");
		InitializeImage("action_security", "portal/public/img/action_security.gif", "25", "25");
		InitializeImage("action_filter", "portal/private/img/action_filter.gif", "25", "25");
		InitializeImage("action_history_audit", "portal/private/img/action_history_audit.gif", "25", "25");
		InitializeImage("action_object_migrate", "portal/private/img/action_object_migrate.gif", "25", "25");
		InitializeImage("action_object_migration", "portal/private/img/action_object_migration.gif", "25", "25");
		InitializeImage("action_properties", "portal/private/img/action_properties.gif", "25", "25");
		InitializeImage("action_reset", "portal/private/img/action_reset.gif", "25", "25");
		InitializeImage("action_run_once", "portal/private/img/action_run_once.gif", "25", "25");
		InitializeImage("action_setting_link", "portal/private/img/action_setting_link.gif", "25", "25");
		InitializeImage("action_unapprove", "portal/private/img/action_unapprove.gif", "25", "25");
		InitializeImage("action_unlock", "portal/private/img/action_unlock.gif", "25", "25");
		InitializeImage("action_unregister", "portal/private/img/action_unregister.gif", "25", "25");
		InitializeImage("action_utility_edit", "portal/private/img/action_utility_edit.gif", "25", "25");
		InitializeImage("icon_activity_right", "portal/private/img/icon_activity_right.gif", "20", "20");
		InitializeImage("icon_activity_right_custom", "portal/private/img/icon_activity_right_custom.gif", "20", "20");
		InitializeImage("icon_activity_right_intrinsic", "portal/private/img/icon_activity_right_intrinsic.gif", "20", "20");
		InitializeImage("icon_auth_source", "portal/public/img/icon_auth_source.gif", "20", "16");
		InitializeImage("icon_automation_server", "portal/private/img/icon_automation_server.gif", "20", "20");
		InitializeImage("icon_banner", "portal/private/img/icon_banner.gif", "20", "14");
		InitializeImage("icon_button_close", "portal/public/img/icon_button_close.gif", "20", "17");
		InitializeImage("icon_column", "portal/private/img/icon_column.gif", "20", "20");
		InitializeImage("icon_community", "portal/public/img/icon_community.gif", "20", "20");
		InitializeImage("icon_crawler", "portal/public/img/icon_crawler.gif", "20", "20");
		InitializeImage("icon_display_options", "portal/private/img/icon_display_options.gif", "20", "18");
		InitializeImage("icon_document_type", "portal/public/img/icon_document_type.gif", "20", "14");
		InitializeImage("icon_exclusion", "portal/private/img/icon_exclusion.gif", "20", "18");
		InitializeImage("icon_extension", "portal/private/img/icon_extension.gif", "20", "20");
		InitializeImage("icon_filter", "portal/public/img/icon_filter.gif", "20", "20");
		InitializeImage("icon_folder_community", "portal/private/img/icon_folder_community.gif", "20", "20");
		InitializeImage("icon_footer", "portal/private/img/icon_footer.gif", "20", "14");
		InitializeImage("icon_group", "portal/public/img/icon_group.gif", "20", "16");
		InitializeImage("icon_hold", "portal/private/img/icon_hold.gif", "20", "20");
		InitializeImage("icon_inclusion", "portal/private/img/icon_inclusion.gif", "20", "18");
		InitializeImage("icon_job", "portal/public/img/icon_job.gif", "20", "16");
		InitializeImage("icon_job_server", "portal/private/img/icon_job_server.gif", "16", "16");
		InitializeImage("icon_knowledge_directory", "portal/private/img/icon_knowledge_directory.gif", "20", "20");
		InitializeImage("icon_layout", "portal/private/img/icon_layout.gif", "20", "20");
		InitializeImage("icon_login", "portal/public/img/icon_login.gif", "20", "15");
		InitializeImage("icon_locale_settings", "portal/private/img/icon_locale_settings.gif", "20", "18");
		InitializeImage("icon_mobile_settings", "portal/private/img/icon_mobile_settings.gif", "20", "18");
		InitializeImage("icon_mypage", "portal/private/img/icon_mypage.gif", "20", "20");
		InitializeImage("icon_operation", "portal/private/img/icon_operation.gif", "20", "20");
		InitializeImage("icon_package", "portal/private/img/icon_package.gif", "20", "16");
		InitializeImage("icon_page_template", "portal/public/img/icon_page_template.gif", "20", "14");
		InitializeImage("icon_profile", "portal/private/img/icon_profile.gif", "20", "10");
		InitializeImage("icon_profile_page", "portal/private/img/icon_profile_page.gif", "20", "18");
		InitializeImage("icon_profile_private", "portal/private/img/icon_profile_private.gif", "20", "13");
		InitializeImage("icon_profile_public", "portal/private/img/icon_profile_public.gif", "20", "10");
		InitializeImage("icon_progress", "portal/public/img/icon_progress.gif", "30", "250");
		InitializeImage("icon_property", "portal/private/img/icon_property.gif", "20", "16");
		InitializeImage("icon_resource", "portal/private/img/icon_resource.gif", "20", "18");
		InitializeImage("icon_row", "portal/private/img/icon_row.gif", "20", "20");
		InitializeImage("icon_scheme", "portal/private/img/icon_scheme.gif", "20", "20");
		InitializeImage("icon_search_prefs", "portal/private/img/icon_search_prefs.gif", "25", "25");
		InitializeImage("icon_server", "portal/private/img/icon_server.gif", "20", "20");
		InitializeImage("icon_smart_sort", "portal/private/img/icon_smart_sort.gif", "16", "16");
		InitializeImage("action_smart_sort", "portal/private/img/action_smart_sort.gif", "25", "25");
		InitializeImage("icon_template_project", "portal/private/img/icon_template_project.gif", "16", "16");
		InitializeImage("icon_user", "portal/public/img/icon_user.gif", "20", "9");
		InitializeImage("icon_user_list", "portal/private/img/icon_user_list.gif", "20", "18");
		InitializeImage("icon_value", "portal/private/img/icon_value.gif", "20", "20");
		InitializeImage("icon_web_service", "portal/public/img/icon_web_service.gif", "20", "20");
		InitializeImage("layout0", "portal/private/img/layout0.gif", "108", "92");
		InitializeImage("layout1", "portal/private/img/layout1.gif", "108", "92");
		InitializeImage("layout10", "portal/private/img/layout10.gif", "108", "92");
		InitializeImage("layout11", "portal/private/img/layout11.gif", "108", "92");
		InitializeImage("layout12", "portal/private/img/layout12.gif", "108", "92");
		InitializeImage("layout2", "portal/private/img/layout2.gif", "108", "92");
		InitializeImage("layout3", "portal/private/img/layout3.gif", "108", "92");
		InitializeImage("layout4", "portal/private/img/layout4.gif", "108", "92");
		InitializeImage("layout5", "portal/private/img/layout5.gif", "108", "92");
		InitializeImage("layout6", "portal/private/img/layout6.gif", "108", "92");
		InitializeImage("layout7", "portal/private/img/layout7.gif", "108", "92");
		InitializeImage("layout8", "portal/private/img/layout8.gif", "108", "92");
		InitializeImage("layout9", "portal/private/img/layout9.gif", "108", "92");
		InitializeImage("template_portlet_ban", "portal/private/img/template_portlet_ban.gif", "108", "92");
		InitializeImage("template_portlet_footer", "portal/private/img/template_portlet_footer.gif", "108", "92");
		InitializeImage("template_portlet_freeform", "portal/private/img/template_portlet_freeform.gif", "108", "92");
		InitializeImage("template_portlet_narrow", "portal/private/img/template_portlet_narrow.gif", "108", "92");
		InitializeImage("template_portlet_wide", "portal/private/img/template_portlet_wide.gif", "108", "92");
		InitializeImage("icon_check_in", "collab/img/icon_check_in.gif", "20", "7");
		//        InitializeImage("icon_check_out", "icon_check_out.gif", "20", "7" );
		InitializeImage("icon_draft", "portal/public/img/icon_draft.gif", "20", "14");
		InitializeImage("icon_message_sent", "portal/public/img/icon_message_sent.gif", "20", "20");
		InitializeImage("icon_mobile", "portal/public/img/icon_mobile.gif", "20", "11");
		InitializeImage("icon_portlet_bundle", "portal/public/img/icon_portlet_bundle.gif", "20", "20");
		InitializeImage("icon_publish", "portal/public/img/icon_publish.gif", "20", "20");
		InitializeImage("icon_search", "portal/public/img/icon_search.gif", "20", "11");
		InitializeImage("icon_search_topbestbet", "portal/public/img/icon_search_topbestbet.gif", "16", "16");
		InitializeImage("icon_workflow", "portal/private/img/icon_workflow.gif", "20", "18");
		InitializeImage("icon_community_template", "portal/public/img/icon_community_template.gif", "20", "18");
		InitializeImage("icon_data_source", "portal/private/img/icon_data_source.gif", "20", "20");
		InitializeImage("icon_external_operation", "portal/private/img/icon_external_operation.gif", "20", "20");
		InitializeImage("icon_invitation", "portal/private/img/icon_invitation.gif", "20", "20");
		InitializeImage("icon_network_search", "portal/public/img/icon_search_network.gif", "20", "20");
		InitializeImage("icon_portlet_template", "portal/public/img/icon_portlet_template.gif", "20", "20");
		InitializeImage("icon_profile_source", "portal/private/img/icon_profile_source.gif", "20", "20");
		InitializeImage("icon_remote_server", "portal/public/img/icon_server_remote.gif", "20", "20");
		InitializeImage("icon_saved_search", "portal/public/img/icon_search_saved.gif", "20", "20");
		InitializeImage("icon_site_map_link", "portal/private/img/icon_knowledge_directory.gif", "16", "16");
		InitializeImage("icon_map_folder", "portal/private/img/icon_map_folder.gif", "16", "16");
		InitializeImage("icon_sub_portal", "portal/private/img/icon_sub_portal.gif", "20", "18");
		InitializeImage("icon_taxonomist", "portal/private/img/icon_taxonomist.gif", "16", "16");
		InitializeImage("action_search_button_advanced", "portal/private/img/action_search_button_advanced.gif", "20", "20");
		InitializeImage("action_search_button_network", "portal/private/img/action_search_button_network.gif", "20", "20");
		InitializeImage("action_search_button_federated", "portal/private/img/action_search_button_federated.gif", "16", "16");
		InitializeImage("pt_logo_sml_wht", "portal/public/img/PT_logo_sm_wht.gif", "50", "125");
		InitializeImage("pt_logo_poweredbyjava", "portal/public/img/pt_logo_poweredbyjava.gif", "126", "65");
		InitializeImage("icon_subportal_rule", "portal/private/img/icon_subportal_rule.gif", "16", "16");
        InitializeImage("logo_header_bea", "portal/public/img/logo_header_bea.gif", "77", "448");
        InitializeImage("logo_header_splash", "portal/public/img/logo_header_splash.jpg", "77", "409");
        InitializeImage("banner_logo", "portal/private/img/banner_logo.gif", "33", "421");
		InitializeImage("todo_replace_me", "portal/private/img/todo_replace_me.gif", "20", "20");
		
	}
}
