package com.plumtree.xpshared.htmlconstructs;

//import com.plumtree.xpfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;


/**
 *
 *
 * @author beng  This class generates an HTMLElement that renders an item in a
 *         list.
 */
public abstract class PTListItem
{
//
//    private static XPLogger myLogger = XPLogManager.GetLogger(PTListItem.class);
//
//
//    private static boolean bDebug = myLogger.IsDebugEnabled();
//
//
//    private static boolean bError = myLogger.IsErrorEnabled();
//
//
//    private static boolean bWarn = myLogger.IsWarnEnabled();
//
//
//    private static boolean bInfo = myLogger.IsInfoEnabled();
//
//
//    private static boolean bFatal = myLogger.IsFatalEnabled();


    private HTMLAnchor aPrimary = null;


    private HTMLAnchor aProperties = null;


    private HTMLImg myApprovalStateImg = null;


    private HTMLImg myEditItemImg = null;


    private HTMLImg myItemTypeImg = null;


    private HTMLImg myPreviewItemImg = null;


    private HTMLInputTypes selectionType = null;


    private PTStyleClass tableClass = PTStyleClass.LIST_TEXT;


    private PTStyleClass titleStyleClass = PTStyleClass.LIST_SUBTITLE;


    private String sDescription = null;


    private String sEditHRef;


    private String sHeight = "30";


    private String sInputAlt = null;


    private String sInputName = null;


    private String sInputValue = null;


    private String sPreviewHRef;


    private String sTitle;


    private String sTitleHRef;


    private StyleClass myRowStyleClass = PTStyleClass.LIST_ITEM_ONE_BG;


    private XPArrayList alSubLinks;


    private XPStringDictionary sdProperties;


    private boolean bAlwaysDrawEditItemCell = false;


    private boolean bAlwaysDrawItemTypeCell = false;


    private boolean bAlwaysDrawPreviewItemCell = false;


    private boolean bAlwaysDrawSelectionInputCell = false;


    private boolean bChecked = false;


    private boolean m_bNew = false;


    private boolean bSelectionInputEnabled = false;


    private boolean m_bUpdated = false;


    private int iMaxNumSubLinksDisplayed = 5;


	private boolean bAddBreaks = true;


	private HTMLInput hiSelection = null;


	protected abstract HTMLElement GetNewIndicator();


	protected abstract HTMLElement GetUpdatedIndicator();


	private HTMLImg imgSpacer = null;

	private HTMLImg titleSpacer = null;

	private HTMLAnchor editAnchor = null;

	private HTMLAnchor previewAnchor = null;


    /**
     * Call this method to construct a new PTListItem. <br><br>
     *
     * @param title Pass a String that is the title of this item.
     * @param titleHRef Pass a String that is the href of this title.
     */
    public PTListItem(String title, String titleHRef )
    {
        sTitle     = title;
        sTitleHRef = titleHRef;

        aPrimary = new HTMLAnchor(titleHRef);

        try
        {
            aPrimary.AddInnerHTMLString(sTitle);
            aPrimary.SetTitle( title );
        }
        catch (Exception ex)
        {
//            if (bError)
//            {
//                myLogger.Error(ex.getMessage());
//            }
        }
    }


	public HTMLAnchor GetEditAnchor()
	{
		return editAnchor;
	}

	public HTMLAnchor GetPreviewAnchor()
	{
		return previewAnchor;
	}



    /**
     * Call this method to construct a new PTListItem. <br><br>
     * <br><br>
     * Use this constructor if you wish for the title of this item to be just
     * 	a String, as opposed to an anchor.
     *
     *
     * @param title Pass a String that is the title of this item.
     */
    public PTListItem( String title )
    {
    	sTitle = title;
    }



    /**
     * Call this method to construct a new PTListItem. <br><br>
     *
     * @param a Pass the HTMLAnchor that will appear as the title of this item.
     */
    public PTListItem(HTMLAnchor a)
    {
        aPrimary = a;

        sTitleHRef = a.GetHRef();
    }


    public void SetTitle( String s )
    {
    	sTitle = s;
    }


    public void SetTitle( String s, String sHref )
    {
        sTitle     = s;
        sTitleHRef = sHref;

        aPrimary = new HTMLAnchor(sHref);

        try
        {
            aPrimary.AddInnerHTMLString(sTitle);
            aPrimary.SetTitle( s );
        }
        catch (Exception ex)
        {
//            if (bError)
//            {
//                myLogger.Error(ex.getMessage());
//            }
        }
    }


    /**	Call this method to supress line breaks added to the main cell of
     * 	this item.  A break after the title anchor will still be drawn.
     *
     *
     */
    public void SuppressBreaksInMainCell()
    {
    	bAddBreaks = false;
    }

    /**	Call this method to allow line breaks added to the main cell of
     * 	this item.  This is the default state of a list item, so calling this
     * 	method is unnecessary unless SuppressBreaksInMainCell has been called
     * 	previously.
     *
     */
    public void AllowBreaksInMainCell()
    {
    	bAddBreaks = true;
    }


    /**
     * Call this method to add HTMLTableCell objects describing this item to
     * the passed HTMLTableRow. <br><br>
     *
     * @param myRow HTMLTableRow
     *
     * @throws HTMLException
     */
    public void AddItemCellsToRow(HTMLTableRow myRow)
    {

        HTMLTableCell myCell;
        HTMLAnchor myAnchor;
        HTMLSpan mySpan;
        HTMLBr myBr                    = new HTMLBr();
        HTMLGenericElement eCommaSpace = new HTMLGenericElement(", ");

        PTConstruct myConstruct = null;


		if( imgSpacer == null )
		{

	        try
	        {
	            myConstruct = ConstructFactory.GetConstruct(PTImageType.SPACER);
	        }
	        catch (Exception ex)
	        {
	        }

	        if (myConstruct != null)
	        {
	            imgSpacer = (HTMLImg) myConstruct.GetHTMLElement();
	        }
		}

		if( titleSpacer == null )
		{

			try
			{
				myConstruct = ConstructFactory.GetConstruct(PTImageType.SPACER);
			}
			catch (Exception ex)
			{
			}

			if (myConstruct != null)
			{
				titleSpacer = (HTMLImg) myConstruct.GetHTMLElement();
			}

			titleSpacer.SetHeight( "13" );
			titleSpacer.SetWidth( "1" );

		}


		//	SELECTION INPUT CELL

        if (this.bSelectionInputEnabled)
        {
            myCell = new HTMLTableCell();
            myRow.AddInnerHTMLElement(myCell);

            myCell.SetStyleClass(tableClass);
            myCell.SetWidth("0%");
            myCell.SetHeight(sHeight);
            myCell.SetAlign("middle");
            myCell.SetVAlign("top");

            myCell.AddInnerHTMLElement( hiSelection );

        }
        else
        {
            if (bAlwaysDrawSelectionInputCell)
            {
                // dummy cell
                myCell = new HTMLTableCell();
                myRow.AddInnerHTMLElement(myCell);

                myCell.SetStyleClass(tableClass);
            }
        }




        //	ITEM-TYPE / APPROVAL-STATE CELL

        if ((myItemTypeImg != null) || (myApprovalStateImg != null))
        {
            myCell = new HTMLTableCell();
            myRow.AddInnerHTMLElement(myCell);

            myCell.SetStyleClass(tableClass);
            myCell.SetWidth("0%");
            myCell.SetHeight(sHeight);
            myCell.SetAlign("middle");
            myCell.SetVAlign("top");

            if (myApprovalStateImg != null)
            {
                myCell.AddInnerHTMLElement(myApprovalStateImg);
            }

            if (myItemTypeImg != null)
            {
                myCell.AddInnerHTMLElement(myItemTypeImg);
            }
        }
        else
        {
            if (bAlwaysDrawItemTypeCell)
            {
                // dummy cell
                myCell = new HTMLTableCell();
                myRow.AddInnerHTMLElement(myCell);

                myCell.SetStyleClass(tableClass);
            }
        }




		//	MAIN CELL

        myCell = new HTMLTableCell();
        myRow.AddInnerHTMLElement(myCell);

		//	we cannot set the style class on this cell because it seems
		//	to override the style on the title anchor.

        myCell.SetWidth("100%");
        myCell.SetHeight(sHeight);
        myCell.SetVAlign("top");

        mySpan = new HTMLSpan();
        mySpan.SetStyleClass(titleStyleClass);


		if( aPrimary != null )
		{
			//	if we have a primary anchor, then this
			//	is the title
	        mySpan.AddInnerHTMLElement(aPrimary);
		}
		else
		{
			//	otherwise, it's just the title string
			mySpan.AddInnerHTMLString( sTitle );
		}


        myCell.AddInnerHTMLElement(mySpan);

        if (m_bNew)
        {
            myCell.AddInnerHTMLElement( GetNewIndicator() );

        }

        if (m_bUpdated)
        {
            myCell.AddInnerHTMLElement( GetUpdatedIndicator() );
        }


		myCell.AddInnerHTMLElement( titleSpacer );


		//	this break is always drawn, even if breaks are not
		//	added to the remainder of the main cell
        myCell.AddInnerHTMLElement(myBr);



        if (sdProperties != null)
        {
//            if (bDebug)
//            {
//                myLogger.Debug("AddItemCellsToRow - sdProperties is not null.");
//            }

            String sPropKey;
            String sPropVal;
            boolean bFirstProperty = true;

			mySpan = new HTMLSpan();
			mySpan.SetStyleClass( tableClass );

			myCell.AddInnerHTMLElement( mySpan );

            IXPEnumerator eKeys = sdProperties.GetKeys();

            while (eKeys.MoveNext())
            {
                if (!bFirstProperty)
                {
                    myCell.AddInnerHTMLString(CommonHTMLStrings.SPACE);
                }

                bFirstProperty = false;

                sPropKey = (String) eKeys.GetCurrent();
                sPropVal = sdProperties.GetElement(sPropKey);

                mySpan.AddInnerHTMLString(sPropKey);
                mySpan.AddInnerHTMLString(": ");
                mySpan.AddInnerHTMLString(sPropVal);
            }


			if( bAddBreaks )
	            mySpan.AddInnerHTMLElement(myBr);

        }

        if (sDescription != null)
        {
//            if (bDebug)
//            {
//                myLogger.Debug("AddItemCellsToRow - sDescription is not null.");
//            }

			mySpan = new HTMLSpan();
			mySpan.SetStyleClass( tableClass );
			myCell.AddInnerHTMLElement( mySpan );

            mySpan.AddInnerHTMLString(sDescription);

            if( bAddBreaks )
	            mySpan.AddInnerHTMLElement(myBr);

        }

        if (aProperties != null)
        {
//            if (bDebug)
//            {
//                myLogger.Debug("AddItemCellsToRow - aProperties is not null.");
//            }

			mySpan = new HTMLSpan();
			mySpan.SetStyleClass( tableClass );
			myCell.AddInnerHTMLElement( mySpan );

            mySpan.AddInnerHTMLElement(aProperties);

            if( bAddBreaks )
	            mySpan.AddInnerHTMLElement(myBr);

        }

        if (alSubLinks != null)
        {
            IXPEnumerator eLinks = alSubLinks.GetEnumerator();

			mySpan = new HTMLSpan();
			mySpan.SetStyleClass( tableClass );
			myCell.AddInnerHTMLElement( mySpan );

            boolean bFirstAnchor = true;
            int iNumLink         = 0;
            PTListItemSubLink slTemp;
            boolean bNew;
            boolean bUpdated;

            while (eLinks.MoveNext())
            {
                iNumLink++;

                if (iNumLink > iMaxNumSubLinksDisplayed)
                {
                	if (iNumLink == (iMaxNumSubLinksDisplayed + 1))
                	{
                		// Only add the ellipses once
	                    mySpan.AddInnerHTMLString("...");
                	}

                    continue;
                }

                if (!bFirstAnchor)
                {
                    mySpan.AddInnerHTMLElement(eCommaSpace);
                }

                slTemp = (PTListItemSubLink) eLinks.GetCurrent();

                myAnchor = slTemp.GetAnchor();
                bNew     = slTemp.GetNew();
                bUpdated = slTemp.GetUpdated();

                mySpan.AddInnerHTMLElement(myAnchor);

                if (bNew)
                {
                    mySpan.AddInnerHTMLElement( GetNewIndicator() );
                }

                if (bUpdated)
                {
                    mySpan.AddInnerHTMLElement( GetUpdatedIndicator() );
                }

                bFirstAnchor = false;
            }

			if( bAddBreaks )
	            mySpan.AddInnerHTMLElement(myBr);

        }

        myCell.AddInnerHTMLElement(imgSpacer);




		//	PREVIEW IMAGE CELL


        if (myPreviewItemImg != null)
        {
            myCell = new HTMLTableCell();
            myRow.AddInnerHTMLElement(myCell);

            myCell.SetStyleClass(tableClass);
            myCell.SetNoWrap(true);
            myCell.SetWidth("20");
            myCell.SetHeight(sHeight);
            myCell.SetAlign("middle");
            myCell.SetVAlign("top");

			if( previewAnchor == null )
			{
	            previewAnchor = new HTMLAnchor(sPreviewHRef);
				previewAnchor.AddInnerHTMLElement(myPreviewItemImg);
			}

			myCell.AddInnerHTMLElement(previewAnchor);


        }
        else
        {
            if (bAlwaysDrawPreviewItemCell)
            {
                // dummy cell
                myCell = new HTMLTableCell();
                myRow.AddInnerHTMLElement(myCell);

                myCell.SetStyleClass(tableClass);
            }
        }



		//	EDIT IMAGE CELL

        if (myEditItemImg != null)
        {
            myCell = new HTMLTableCell();
            myRow.AddInnerHTMLElement(myCell);

            myCell.SetStyleClass(tableClass);
            myCell.SetNoWrap(true);
            myCell.SetWidth("20");
            myCell.SetHeight(sHeight);
            myCell.SetAlign("middle");
            myCell.SetVAlign("top");

			if( sEditHRef != null )
			{

				if( editAnchor == null )
				{
					editAnchor = new HTMLAnchor(sEditHRef);
					editAnchor.AddInnerHTMLElement(myEditItemImg);
				}

				myCell.AddInnerHTMLElement(editAnchor);

			}
			else
			{
				myCell.AddInnerHTMLElement( myEditItemImg );
			}

        }
        else
        {
            if (bAlwaysDrawEditItemCell)
            {
                // dummy cell
                myCell = new HTMLTableCell();
                myRow.AddInnerHTMLElement(myCell);

                myCell.SetStyleClass(tableClass);
            }
        }


    }

    /**
     * Call this method to add a name/value property pair to this item.
     *
     * @param sName String name
     * @param sVal String value
     */
    public void AddProperty(String sName, String sVal)
    {
        if (sdProperties == null)
        {
            sdProperties = new XPStringDictionary();
        }

        sdProperties.PutElement(sName, sVal);
//
//        if (bDebug)
//        {
//            myLogger.Debug("AddProperty - sName: " + sName + ", sVal: " + sVal);
//        }
    }

    /**
     * Call this method to add a sublink to this list item. Each sublink can be
     * new and/or updated. <br><br>
     *
     * @param a HTMLAnchor
     * @param m_bNew boolean
     * @param m_bUpdated boolean
     */
    public void AddSubLink(HTMLAnchor a, boolean bNew, boolean bUpdated)
    {
        if (alSubLinks == null)
        {
            alSubLinks = new XPArrayList();
        }

        PTListItemSubLink sl = new PTListItemSubLink();

        sl.SetAnchor(a);
        sl.SetNew(bNew);
        sl.SetUpdated(bUpdated);

        alSubLinks.Add(sl);
    }

    /**
     * Call this method to cause this item to display a checkbox input.
     * <br><br>
     *
     * @param checkboxName Pass a String that is the name of this checkbox.
     * @param checkboxValue
     * @param checkboxAlt Pass a String that is the alt attribute of this
     *        checkbox.
     * @param bSelected whether on not this input is checked
     */
    public void EnableCheckbox(String checkboxName, String checkboxValue, String checkboxAlt, boolean bSelected)
    {
        EnableSelectionInput(HTMLInputTypes.CHECKBOX_TYPE, checkboxName, checkboxValue, checkboxAlt, bSelected);
    }

    /**
     * Call this method to cause this item to display a selection input
     * allowing the user to select this item among the list of items. <br><br>
     *
     * @param type HTMLInputType should be either radio or checkbox.
     * @param inputName String input name
     * @param inputValue String input value
     * @param inputAlt String input alt
     * @param bSelected whether on not this input is checked
     */
    public void EnableSelectionInput(HTMLInputTypes type, String inputName, String inputValue, String inputAlt, boolean bSelected)
    {
        this.bSelectionInputEnabled = true;
        this.sInputName             = inputName;
        this.sInputValue            = inputValue;
        this.sInputAlt              = inputAlt;
        this.selectionType          = type;
        this.bChecked               = bSelected;


		hiSelection = new HTMLInput(selectionType, this.sInputName, "id");

		hiSelection.SetStyleClass(PTStyleClass.FORM_CHECKBOX_TEXT);
		hiSelection.SetValue(sInputValue);
		hiSelection.SetAlt(sInputAlt);


		if (bChecked)
		{
			// Only set selected if it is true (don't waste space, default is false)
			hiSelection.SetChecked(bChecked);
		}

    }

    public HTMLInput GetSelectionInput()
    {
    	return hiSelection;
    }


    /**
     *
     *
     * @return
     */
    public String GetCellHeight()
    {
        return sHeight;
    }

    /**
     * Call this method to obtain an HTMLTable that will render this list item
     * as it is currently configured.
     *
     * @return
     *
     * @throws HTMLException
     */
    public HTMLTable GetItemAsTable()
    {
        HTMLTable myTable;

        myTable = new HTMLTable();

        myTable.SetWidth(CommonHTMLStrings.ONE_HUNDRED_PERCENT);
        myTable.SetCellPadding("3");
        myTable.SetCellSpacing(CommonHTMLStrings.ZERO);

        HTMLTableRow myRow = new HTMLTableRow();
        myRow.SetStyleClass(myRowStyleClass);

        myTable.AddInnerHTMLElement(myRow);

        AddItemCellsToRow(myRow);

        return myTable;
    }

    /**
     * Call this method to force this item to draw the HTMLTableCell in which
     * the edit button is nested whether the edit button is set or     not. By
     * default, this cell is not drawn, but if you wish for some     items in
     * a table to contain this button while others do not, then this
     * HTMLTableCell should always be drawn.
     *
     * @param b boolean
     */
    public void SetAlwaysDrawEditItemCell(boolean b)
    {
        bAlwaysDrawEditItemCell = b;
    }

    /**
     * Call this method to force this item to draw the HTMLTableCell in which
     * the preview button is nested whether the preview button is set or not.
     * By default, this cell is not drawn, but if you wish for some items in a
     * table to contain this button while others do not, then this
     * HTMLTableCell should always be drawn.
     *
     * @param b boolean
     */
    public void SetAlwaysDrawItemPreviewCell(boolean b)
    {
        bAlwaysDrawPreviewItemCell = b;
    }

    /**
     * Call this method to force this item to draw the HTMLTableCell in which
     * the item type image is nested whether the item type image is set or
     * not.  By default, this cell is not drawn, but if you wish for some
     * items in a table to contain this image while others do not, then this
     * HTMLTableCell should always be drawn.
     *
     * @param b boolean
     */
    public void SetAlwaysDrawItemTypeCell(boolean b)
    {
        bAlwaysDrawItemTypeCell = b;
    }

    /**
     * Call this method to force this item to draw the HTMLTableCell in which
     * the selection input is nested whether the selection input is set or
     * not.  By default, this cell is not drawn, but if you wish for some
     * items in a table to contain this input while others do not, then this
     * HTMLTableCell should always be drawn.
     *
     * @param b boolean
     */
    public void SetAlwaysDrawSelectionInputCell(boolean b)
    {
        bAlwaysDrawSelectionInputCell = b;
    }

    /**
     *
     *
     * @param image
     * @param imgAlt
     */
    public void SetApprovalStateImage(PTImageType image, String imgAlt)
    {
        myApprovalStateImg = (HTMLImg) image.GetHTMLElement();
        myApprovalStateImg.SetAlt(imgAlt);
    }

    /**
     *
     *
     * @param s
     */
    public void SetCellHeight(String s)
    {
        sHeight = s;
    }

    /**
     * Call this method to set the description for this list item.
     *
     * @param s String
     */
    public void SetDescription(String s)
    {
        sDescription = s;
    }

    /**
     * Call this method to set the image/href that allows users to edit this
     * item. <br>
     * <br>
     * Calling this method will cause the generated table to contain one
     * additional column. <br><br>
     *
     * @param image Pass a PTImageType that contains the edit button image.
     * @param imgAlt Pass a String that is the alt attribute of this image.
     * @param editHRef Pass the HRef that redirect the user to a page allowing
     *        this item to be edited.  Pass null, and the edit button will be
     * 		  merely and edit image.
     */
    public void SetItemEditButton(PTImageType image, String imgAlt, String editHRef)
    {
        myEditItemImg = (HTMLImg) image.GetHTMLElement();
        myEditItemImg.SetAlt(imgAlt);
        sEditHRef = editHRef;


        if( sEditHRef != null )
        {
			editAnchor = new HTMLAnchor(sEditHRef);
			editAnchor.AddInnerHTMLElement(myEditItemImg);
        }
    }




    /**
     * Call this method to set the image/href that allows users to preview this
     * item. <br>
     * <br>
     * Calling this method will cause the generated table to contain one
     * additional column. <br><br>
     *
     * @param image Pass a PTImageType that contains the preview button image.
     * @param imgAlt Pass a String that is the alt attribute of this image.
     * @param previewHRef Pass the HRef that redirect the user to a page
     *        allowing this item to be pre viewed.
     */
    public void SetItemPreviewButton(PTImageType image, String imgAlt, String previewHRef)
    {
        myPreviewItemImg = (HTMLImg) image.GetHTMLElement();
        myPreviewItemImg.SetAlt(imgAlt);
        sPreviewHRef = previewHRef;

		previewAnchor = new HTMLAnchor(sPreviewHRef);
		previewAnchor.AddInnerHTMLElement(myPreviewItemImg);
    }

    /**
     * Call this method to set the image that indicates the type of this item. <br>
     * <br>
     * Calling this method will cause the generated table to contain one
     * additional column. <br><br>
     *
     * @param image Pass a PTImageType that indicates the image describing this
     *        item's type.
     * @param imgAlt Pass a String that is the alt attribute of this image.
     */
    public void SetItemTypeImage(PTImageType image, String imgAlt)
    {
        myItemTypeImg = (HTMLImg) image.GetHTMLElement();
        myItemTypeImg.SetAlt(imgAlt);
    }

    /**
     *
     *
     * @param image
     */
    public void SetItemTypeImage(HTMLImg image)
    {
        myItemTypeImg = image;
    }

    /**
     * Call this method to set the maximum number of sub links displayed with
     * this item.   <br><br>
     *
     * @param i int
     */
    public void SetMaxNumSubLinksDisplayed(int i)
    {
        iMaxNumSubLinksDisplayed = i;
    }

    /**
     * Call this method to toggle the newness of this list item.
     *
     * @param b boolean
     */
    public void SetNew(boolean b)
    {
        m_bNew = b;
    }

    /**
     * Call this method to set the properties link on this list item.
     *
     * @param a HTMLAnchor link
     */
    public void SetPropertiesLink(HTMLAnchor a)
    {
        aProperties = a;
    }

    /**
     * Call this method to set the style class for the HTMLTableRow rendering
     * this item. <br><br>
     *
     * @param styleClass Pass a PTStyleClass.
     */
    public void SetRowStyleClass(PTStyleClass styleClass)
    {
        myRowStyleClass = styleClass;
    }

    /**
     *
     *
     * @param s
     */
    public void SetTableStyleClass(PTStyleClass s)
    {
        tableClass = s;
    }

    /**
     *
     *
     * @param s
     */
    public void SetTitleStyleClass(PTStyleClass s)
    {
        titleStyleClass = s;
    }

    /**
     * Call this method to toggle the updatedness of this list item.
     *
     * @param b boolean
     */
    public void SetUpdated(boolean b)
    {
        m_bUpdated = b;
    }

}
