package com.plumtree.xpshared.htmlconstructs;

import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;


/**
 *
 *
 * @author sample
 *         This class generates html necessary to construct a
 *         single query condition against the array of properties passed in
 */
public class PTQueryCondition
{

    public static final int TYPE_LONG = 3;


    public static final int TYPE_DOUBLE = 5;


    public static final int TYPE_DATE = 7;


    public static final int TYPE_STRING = 8;


    public static final int TYPE_REF = 72;


    public static final int PT_FILTEROP_EQ = 1; // ==


    public static final int PT_FILTEROP_EQ_NOCASE = 101; // == (and force case-insensitive)


    public static final int PT_FILTEROP_NE = 2; // !=


    public static final int PT_FILTEROP_LT = 3; // <


    public static final int PT_FILTEROP_GT = 4; // >


    public static final int PT_FILTEROP_LTE = 5; // <=


    public static final int PT_FILTEROP_GTE = 6; // >=


    public static final int PT_FILTEROP_CONTAINS = 7; // substring search


    public static final int PT_FILTEROP_DOESNOTCONTAIN = 8; // substring search (negative case)


    public static final int PT_FILTEROP_ISEMPTY = 9; // is property empublic static final int PTy?  (Empublic static final int PTy means that this property is not available from the card.)


    public static final int PT_FILTEROP_ISNOTEMPTY = 10; // is property not empublic static final int PTy?  (Empublic static final int PTy means that this property is not available from the card.)


    public static final int PT_FILTEROP_IN = 13; // IN


    public static final int PT_FILTEROP_NUMERIC_EQ = 14;


    public static final int PT_FILTEROP_NUMERIC_NOTEQ = 15;
//
//
//    private static XPLogger myLogger = XPLogManager.GetLogger("com.plumtree.xpshared.htmlconstructs.PTQueryCondition");
//
//
//    private static boolean bDebug = myLogger.IsDebugEnabled();
//
//
//    private static boolean bError = myLogger.IsErrorEnabled();
//
//
//    private static boolean bWarn = myLogger.IsWarnEnabled();
//
//
//    private static boolean bInfo = myLogger.IsInfoEnabled();
//
//
//    private static boolean bFatal = myLogger.IsFatalEnabled();


    private static String _ui_DisplayHTMLError = "Unable to render query condition due to construct.";


    private int m_Operator;


    private int m_SelectedID;


    private String m_strValue;


    private XPHashtable m_htProperties;


    private boolean m_bVisible = false;

    /**
     *
     *
     * @param ID
     * @param operator
     * @param strValue
     */
    public void setSelected(int nID, int nOperator, String strValue)
    {
        //String[] data = (String[]) m_htFormData.GetElement(QS_EDITOR_TYPE);
        m_SelectedID = nID;
        m_Operator   = nOperator;
        m_strValue   = strValue;
    }

    /**
     *
     *
     * @return
     */
    public PTQueryCondition CreateDuplicate()
    {
        PTQueryCondition acopy = new PTQueryCondition();

        IXPEnumerator keys   = m_htProperties.GetKeys();
        IXPEnumerator values = m_htProperties.GetElements();
        int[] vals       = null;

        while (keys.MoveNext())
        {
            values.MoveNext();
            vals = (int[]) values.GetCurrent();
            acopy.addProperty(XPConvert.ToString(keys.GetCurrent()), vals[0], vals[1]);
        }

        if (m_SelectedID > 0)
        {
            acopy.setSelected(m_SelectedID, m_Operator, m_strValue);
        }

        if (m_bVisible)
        {
            acopy.makeVisible(); // constructor already sets value to false
        }

        return acopy;
    }

    /**
     *
     *
     * @param strTag
     * @param strStatementID
     *
     * @return
     * @throws XPException
     */
    public HTMLElement GetTableRow(String strTag, String strStatementID)
    {
        HTMLTableRow row = new HTMLTableRow();
        String strID     = "_" + strTag + "_" + strStatementID;

        try
        {
            row.SetStyleClass(PTStyleClass.ACTIONBAR_OBJECT_BG);

            // first let's build the select control...
            HTMLTableCell propCell = new HTMLTableCell();
            propCell.SetWidth("200");
            propCell.SetAlign("absLeft");

            HTMLSelect myPropSelect = new HTMLSelect("in_se_PropSelect" + strID, "");
            myPropSelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);
            myPropSelect.SetID(strTag + "_PropSelect_" + strStatementID);
            myPropSelect.SetOnChange("AdjustOperator('" + strTag + "', '" + strStatementID + "', this.options[this.selectedIndex].value);");

            IXPEnumerator keys   = m_htProperties.GetKeys();
            IXPEnumerator values = m_htProperties.GetElements();

            int[] vals       = null;
            int selectedType = 0;

            while (keys.MoveNext())
            {
                values.MoveNext();
                vals = (int[]) values.GetCurrent();

                HTMLOption myOption = new HTMLOption(XPConvert.ToString(vals[0]) + "|" + XPConvert.ToString(vals[1]));

                if (vals[0] == m_SelectedID)
                {
                    myOption.SetSelected(true);
                    selectedType = vals[1];
                }

                myOption.AddInnerHTMLString((String) keys.GetCurrent());
                myPropSelect.AddInnerHTMLElement(myOption);
            }

            HTMLDiv tableCellDiv = new HTMLDiv();
            tableCellDiv.SetID(strTag + "_divPropRow_" + strStatementID);

            if (!m_bVisible)
            {
                tableCellDiv.SetStyle("display: none;");
            }

            tableCellDiv.AddInnerHTMLElement(myPropSelect);
            propCell.AddInnerHTMLElement(tableCellDiv);
            row.AddInnerHTMLElement(propCell);

            // now we should construct the conditional operator elements
            // reviewing this you might be inclined to say that there is
            // enough repetition here to warrant a generic function for
            // generating the comparison drop downs.  unfortunately these
            // operator drop downs will probably never change so its more
            // trouble than its worth
            // first Text property operators...
            // this cell will contain all of the possible operator drop downs...
            HTMLTableCell propCellOperator = new HTMLTableCell();
            propCellOperator.SetWidth("150");
            propCellOperator.SetAlign("absLeft");

            String strDisp  = ""; //"position:absolute;left:450;";
            HTMLDiv Textdiv = new HTMLDiv();

            Textdiv.SetID(strTag + "_divText_" + strStatementID);

            if ((selectedType == TYPE_STRING || selectedType == TYPE_REF) && m_bVisible)
            {
                Textdiv.SetStyle(strDisp);
            }
            else
            {
                Textdiv.SetStyle(strDisp + "display: none;");
            }

            HTMLSelect TextOpSelect = new HTMLSelect("in_se_OpText" + strID, "");
            TextOpSelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);

            HTMLOption Option1 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_CONTAINS));
            Option1.AddInnerHTMLString("Contains");

            if (PT_FILTEROP_CONTAINS == m_Operator)
            {
                Option1.SetSelected(true);
            }

            TextOpSelect.AddInnerHTMLElement(Option1);

            HTMLOption Option2 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_ISEMPTY));
            Option2.AddInnerHTMLString("Contains No Value");

            if (PT_FILTEROP_ISEMPTY == m_Operator)
            {
                Option2.SetSelected(true);
            }

            TextOpSelect.AddInnerHTMLElement(Option2);

            Textdiv.AddInnerHTMLElement(TextOpSelect);
            propCellOperator.AddInnerHTMLElement(Textdiv);

            // second, Date property operators...
            HTMLDiv Datediv = new HTMLDiv();

            Datediv.SetID(strTag + "_divDate_" + strStatementID);

            if ((selectedType == TYPE_DATE) && m_bVisible)
            {
                Datediv.SetStyle(strDisp);
            }
            else
            {
                Datediv.SetStyle(strDisp + "display: none;");
            }

            HTMLSelect DateOpSelect = new HTMLSelect("in_se_OpDate" + strID, "");
            DateOpSelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);

            HTMLOption Option3 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_GT));
            Option3.AddInnerHTMLString("Comes After");

            if (PT_FILTEROP_GT == m_Operator)
            {
                Option3.SetSelected(true);
            }

            DateOpSelect.AddInnerHTMLElement(Option3);

            HTMLOption Option4 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_LT));
            Option4.AddInnerHTMLString("Comes Before");

            if (PT_FILTEROP_LT == m_Operator)
            {
                Option4.SetSelected(true);
            }

            DateOpSelect.AddInnerHTMLElement(Option4);

            HTMLOption Option5 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_NUMERIC_EQ));
            Option5.AddInnerHTMLString("Is");

            if (PT_FILTEROP_NUMERIC_EQ == m_Operator)
            {
                Option5.SetSelected(true);
            }

            DateOpSelect.AddInnerHTMLElement(Option5);

            HTMLOption Option6 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_NUMERIC_NOTEQ));
            Option6.AddInnerHTMLString("Is Not");

            if (PT_FILTEROP_NUMERIC_NOTEQ == m_Operator)
            {
                Option6.SetSelected(true);
            }

            DateOpSelect.AddInnerHTMLElement(Option6);

            Datediv.AddInnerHTMLElement(DateOpSelect);
            propCellOperator.AddInnerHTMLElement(Datediv);

            // finally, the numeric operators
            HTMLDiv Numberdiv = new HTMLDiv();

            Numberdiv.SetID(strTag + "_divNumber_" + strStatementID);

            if ((selectedType == TYPE_LONG || selectedType == TYPE_DOUBLE) && m_bVisible)
            {
                Numberdiv.SetStyle(strDisp);
            }
            else
            {
                Numberdiv.SetStyle(strDisp + "display: none;");
            }

            HTMLSelect NumberOpSelect = new HTMLSelect("in_se_OpNumber" + strID, "");
            NumberOpSelect.SetStyleClass(PTStyleClass.FORM_PULLDOWN_TEXT);

            HTMLOption Option7 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_GT));
            Option7.AddInnerHTMLString("Is Greater Than");

            if (PT_FILTEROP_GT == m_Operator)
            {
                Option7.SetSelected(true);
            }

            NumberOpSelect.AddInnerHTMLElement(Option7);

            HTMLOption Option8 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_LT));
            Option8.AddInnerHTMLString("Is Less Than");

            if (PT_FILTEROP_LT == m_Operator)
            {
                Option8.SetSelected(true);
            }

            NumberOpSelect.AddInnerHTMLElement(Option8);

            HTMLOption Option9 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_NUMERIC_EQ));
            Option9.AddInnerHTMLString("Is");

            if (PT_FILTEROP_NUMERIC_EQ == m_Operator)
            {
                Option9.SetSelected(true);
            }

            NumberOpSelect.AddInnerHTMLElement(Option9);

            HTMLOption Option10 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_NUMERIC_NOTEQ));
            Option10.AddInnerHTMLString("Is Not");

            if (PT_FILTEROP_NUMERIC_NOTEQ == m_Operator)
            {
                Option10.SetSelected(true);
            }

            NumberOpSelect.AddInnerHTMLElement(Option10);

            HTMLOption Option11 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_GTE));
            Option11.AddInnerHTMLString("Is Greater Than or Equals");

            if (PT_FILTEROP_GTE == m_Operator)
            {
                Option11.SetSelected(true);
            }

            NumberOpSelect.AddInnerHTMLElement(Option11);

            HTMLOption Option12 = new HTMLOption(XPConvert.ToString(PT_FILTEROP_LTE));
            Option12.AddInnerHTMLString("Is Less Than or Equals");

            if (PT_FILTEROP_LTE == m_Operator)
            {
                Option12.SetSelected(true);
            }

            NumberOpSelect.AddInnerHTMLElement(Option12);

            Numberdiv.AddInnerHTMLElement(NumberOpSelect);
            propCellOperator.AddInnerHTMLElement(Numberdiv);

            row.AddInnerHTMLElement(propCellOperator);

            // and now the input element
            HTMLTableCell inputCell = new HTMLTableCell();
            inputCell.SetWidth("250");
            inputCell.SetAlign("absLeft");

            HTMLInput input = new HTMLInput(HTMLInputTypes.TEXT_TYPE, "in_tx_compareval" + strID, "");
            input.SetStyleClass(PTStyleClass.FORM_TEXTBOX_TEXT);
            input.SetValue(m_strValue);

            HTMLDiv CompareDiv = new HTMLDiv();
            CompareDiv.SetID(strTag + "_divCompareval_" + strStatementID);

            if (!m_bVisible)
            {
                CompareDiv.SetStyle("display: none;");
            }

            CompareDiv.AddInnerHTMLElement(input);

            inputCell.AddInnerHTMLElement(CompareDiv);
            row.AddInnerHTMLElement(inputCell);
        }
        catch (Exception e)
        {
//            if (bError)
//            {
//                myLogger.Error("PTQueryCondition::getDivTableRow: The following error occured when trying to display the HTML: " + e.getMessage());
//            }

            throw new XPException(_ui_DisplayHTMLError, e);
        }

        return row;
    }

    /**
     *
     *
     * @param strName
     * @param ID
     * @param ObjectType
     */
    public void addProperty(String strName, int nID, int nObjectType)
    {
        if (m_htProperties == null)
        {
            m_htProperties = new XPHashtable();
        }

        int[] arIDandType;
        arIDandType    = new int[2];
        arIDandType[0] = nID;
        arIDandType[1] = nObjectType;

        m_htProperties.PutElement(strName, arIDandType);
    }

    /**
     *
     */
    public void makeInVisible()
    {
        m_bVisible = false;
    }

    /**
     * the object type passed in as the second argument to this     method
     * should correspond to one of the types defined at the     top of this
     * file
     */
    public void makeVisible()
    {
        m_bVisible = true;
    }
}

