package com.plumtree.xpshared.htmlconstructs;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openlog.*;
import com.plumtree.xpshared.htmlelements.StyleClass;

/**
 * @author beng
 *	
 * 	This class contains references to static StyleClass objects.  These
 * 	StyleClass objects can be used to set the style class on HTMLElement object.
 * 	<br><br>
 * 	For example, if you have an HTMLTable object, and you wish to set the
 * 	style class, you can do this:
 * 	<br><br>
 * 	<code>
 * 	myTable.SetStyleClass( PTStyleClass.HEADER );
 * 	</code>
 * 	<br><br>
 * 	The resulting html will look something like:
 * 	<br><br>
 * 	&lt;table class="header"&gt; ...
 * 	<br><br>
 * 
 * The constructor for PTStyleClass is protected because
 * this class is meant only for standard Plumtree Styles.
 * 
 * To create a new Style for use with an HTML Element,
 * create a new StyleClass object instead.
 * 	<br><br>
 * 	For example, if you have an HTMLTable object, and you wish to set
 *  a custom style class, you can do this:
 * 	<br><br>
 * 	<code>
 * 	myTable.SetStyleClass( new StyleClass("customclass" );
 * 	</code>
 * 	<br><br>
 * 	The resulting html will look something like:
 * 	<br><br>
 * 	&lt;table class="customclass"&gt; ...
 * 	<br><br>
 */
public class PTStyleClass extends StyleClass
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.xpshared.htmlconstructs.PTStyleClass");

	public String toString()
	{
		return sClass;
	}

	private static boolean bPreviousAllowPublicCreation;

	/**
	 * The constructor for PTStyleClass is protected because
	 * this class is meant only for standard Plumtree Styles.
	 * 
	 * To create a new Style for use with an HTML Element,
	 * create a new StyleClass object instead.
	 * 
	 * @param strStyleClass The style text.
	 */
	protected PTStyleClass( String strStyleClass )
	{	
		super( strStyleClass );
	}

	public static PTStyleClass NAV_SIDEBAR_SUB_SUBSECTION_BG = null;
	public static PTStyleClass OBJECT_BTN_TEXT = null;

	public static PTStyleClass FOOT_HEADER_BG = null;
	public static PTStyleClass FOOT_SUBHEAD = null;
	public static PTStyleClass FOOT_TITLE = null;
	public static PTStyleClass FOOT_SUBTITLE = null;
	public static PTStyleClass FOOT_TEXT = null;

	public static PTStyleClass ACTIONBAR_BAN_BG = null;	
	public static PTStyleClass ACTIONBAR_CAL_BG = null;
	public static PTStyleClass ACTIONBAR_DIALOGUE_BG = null;
	public static PTStyleClass ACTIONBAR_DIR_BG = null;
	public static PTStyleClass ACTIONBAR_PORTLET_BG = null;
	public static PTStyleClass ACTIONBAR_OBJECT_BG = null;
	
	public static PTStyleClass ACTIONBAR_TEXT = null;
	public static PTStyleClass ACTIONBAR_BAN_TEXT = null;
	public static PTStyleClass ACTIONBAR_BAN_DISABLED_TEXT = null;
	public static PTStyleClass ACTIONBAR_BAN_BTN = null;
	public static PTStyleClass ACTIONBAR_DIALOGUE_BTN = null;
	public static PTStyleClass ACTIONBAR_DIR_BTN = null;
	public static PTStyleClass ACTIONBAR_PORTLET_BTN = null;
	public static PTStyleClass ACTIONBAR_OBJECT_BTN = null;
	public static PTStyleClass ACTIONBAR_BAN_DIVIDER = null;
	public static PTStyleClass ACTIONBAR_OBJECT_DIVIDER = null;
	public static PTStyleClass ACTIONBAR_PORTLET_DIVIDER = null;
	public static PTStyleClass ACTIONBAR_DIR_DIVIDER = null;
	public static PTStyleClass ACTIONBAR_DIALOGUE_DIVIDER = null;
	
	public static PTStyleClass ALERT_ERROR_TEXT = null;
	public static PTStyleClass ALERT_WARNING_TEXT = null;
	public static PTStyleClass ALERT_CONFIRM_TEXT = null;
	public static PTStyleClass ALERT_TITLE = null;
	public static PTStyleClass ALERT_SUBTITLE = null;

	public static PTStyleClass ALERT_DIALOG_BG = null;
	public static PTStyleClass ALERT_DIALOG_TITLE = null;
	public static PTStyleClass ALERT_DIALOG_TEXT = null;
	public static PTStyleClass ALERT_ERROR_TITLE = null;
	public static PTStyleClass ALERT_ERROR_SUBTITLE = null;
	public static PTStyleClass ALERT_ERROR_DESCRIPTION = null;
	public static PTStyleClass ALERT_REQUIRED_TEXT = null;
	public static PTStyleClass ALERT_BG = null;	
	public static PTStyleClass ALERT_TEXT = null;	

	public static PTStyleClass BAN_TOPBAR_BG = null;
	public static PTStyleClass BAN_HEADER_BG = null;
	public static PTStyleClass BAN_HEADER = null;
	public static PTStyleClass BAN_SUBHEADER = null;
	public static PTStyleClass BAN_TEXT = null;
	public static PTStyleClass BAN_GREETING_TEXT = null;
	public static PTStyleClass BAN_HELP_TEXT = null;

	public static PTStyleClass BREADCRUMB_TEXT = null;

	public static PTStyleClass CAL_DIV = null;	
	public static PTStyleClass CAL_HEADER_BG = null;
	public static PTStyleClass CAL_MONTH_TOGGLE_BG = null;
	public static PTStyleClass CAL_ACTIONBAR_BG = null;
	public static PTStyleClass CAL_DATE_BG = null;
	public static PTStyleClass CAL_BORDER = null;
	public static PTStyleClass CAL_HEADER = null;
	public static PTStyleClass CAL_SUBHEADER = null;
	public static PTStyleClass CAL_TITLE = null;
	public static PTStyleClass CAL_SUBTITLE = null;
	public static PTStyleClass CAL_MONTH_TOGGLE_HEADER = null;
	public static PTStyleClass CAL_TEXT = null;
	public static PTStyleClass CAL_ACTIONBAR_TEXT = null;
	public static PTStyleClass CAL_DATE_TEXT = null;
	public static PTStyleClass CAL_DAYS_TEXT = null;
	public static PTStyleClass CAL_DAYS_DIM_TEXT = null;
	

	public static PTStyleClass DIR_HEADER_BG = null;
	public static PTStyleClass DIR_ACTIONBAR_BG = null;
	public static PTStyleClass DIR_HEADER = null;
	public static PTStyleClass DIR_TEXT = null;
	public static PTStyleClass DIR_BTN_TEXT = null;
	public static PTStyleClass DIR_BTN = null;
	public static PTStyleClass DIR_DIVIDER = null;

	public static PTStyleClass DIR_CONTENT_SUBTITLE = null;
	public static PTStyleClass DIR_CONTENT_TEXT = null;
	public static PTStyleClass DIR_CONTENT_BG = null;

	public static PTStyleClass CUSTOMAPP_HEADER_BG = null;
	public static PTStyleClass CUSTOMAPP_SUBHEAD_BG = null;
	public static PTStyleClass CUSTOMAPP_LIGHT_BG = null;
	public static PTStyleClass CUSTOMAPP_DK_GRAY_BG = null;
	public static PTStyleClass CUSTOMAPP_MD_GRAY_BG = null;
	public static PTStyleClass CUSTOMAPP_LT_GRAY_BG = null;
	public static PTStyleClass CUSTOMAPP_BORDER = null;
	public static PTStyleClass CUSTOMAPP_HEADER = null;
	public static PTStyleClass CUSTOMAPP_SUBHEAD = null;
	public static PTStyleClass CUSTOMAPP_TITLE = null;
	public static PTStyleClass CUSTOMAPP_SUBTITLE = null;


	public static PTStyleClass DIALOGUE_BG = null;
	public static PTStyleClass DIALOGUE_HEADER_BG = null;
	public static PTStyleClass DIALOGUE_MED_BG = null;
	public static PTStyleClass DIALOGUE_LIGHT_BG = null;
	public static PTStyleClass DIALOGUE_GRAY_BG = null;
	public static PTStyleClass DIALOGUE_TAB_ON_BG = null;
	public static PTStyleClass DIALOGUE_TAB_OFF_BG = null;
	public static PTStyleClass DIALOGUE_SEARCH_BG = null;
//	public static PTStyleClass DIALOGUE_ACTIONBAR_BG = null;
	public static PTStyleClass DIALOGUE_HEADER = null;
	public static PTStyleClass DIALOGUE_NOTICE_HEADER = null;
	public static PTStyleClass DIALOGUE_SUBHEAD = null;
	public static PTStyleClass DIALOGUE_TITLE = null;
	public static PTStyleClass DIALOGUE_SUBTITLE = null;
	public static PTStyleClass DIALOGUE_TEXT = null;
	public static PTStyleClass DIALOGUE_TAB_TEXT = null;
	public static PTStyleClass DIALOGUE_FORM_TEXT = null;
	public static PTStyleClass DIALOGUE_MRU_TEXT = null;
//	public static PTStyleClass DIALOGUE_ACTIONBAR_TEXT = null;
//	public static PTStyleClass DIALOGUE_BTN = null;

	

	public static PTStyleClass FORM_PULLDOWN_TEXT = null;
	public static PTStyleClass FORM_INPUT_BOX_TEXT = null;
	public static PTStyleClass FORM_BTN_TEXT = null;
	public static PTStyleClass FORM_EDITOR_BTN_TEXT = null;
	public static PTStyleClass FORM_CHECKBOX_TEXT = null;
	public static PTStyleClass FORM_RADIO = null;
	public static PTStyleClass FORM_SELECT_TEXT = null;
	public static PTStyleClass FORM_TEXTBOX_TEXT = null;

	/**
	 * This has been deprecated.  Use LIST_SORT_HEADER_BG  
	 * @deprecated	
	 */
	public static PTStyleClass LIST_HEADER_BG = null;
	
	/**
	 * This has been deprecated.  Use LIST_SORT_HEADER
	 * @deprecated
	 */
	public static PTStyleClass LIST_HEADER = null;
	
	public static PTStyleClass LIST_SORT_HEADER_BG = null;
	public static PTStyleClass LIST_SORT_HEADER = null;
	
	
	
	public static PTStyleClass LIST_ITEM_ONE_BG = null;
	public static PTStyleClass LIST_ITEM_TWO_BG = null;
	public static PTStyleClass LIST_SUBHEAD = null;
	public static PTStyleClass LIST_SUBTITLE = null;
	public static PTStyleClass LIST_TEXT = null;
	public static PTStyleClass LIST_SUBJECT_TEXT = null;
	public static PTStyleClass LIST_READ_TEXT = null;
	public static PTStyleClass LIST_UNREAD_TEXT = null;
	public static PTStyleClass LIST_DATE_TEXT = null;
	public static PTStyleClass LIST_PAGINATION_TEXT = null;		
	public static PTStyleClass LIST_ITALIC_TEXT = null;
	public static PTStyleClass LIST_BOLD_ITALIC_TEXT = null;
	public static PTStyleClass LIST_HILITE_TEXT = null;
	public static PTStyleClass LIST_NEW_TEXT = null;
	public static PTStyleClass LIST_UPDATED_TEXT = null;

	public static PTStyleClass MENU_HEADER_BG = null;
	public static PTStyleClass MENU_HEADER = null;
	public static PTStyleClass MENU_TEXT = null;
	
	
	public static PTStyleClass LOGIN_HEADER = null;
	public static PTStyleClass LOGIN_SUBHEAD = null;
	public static PTStyleClass LOGIN_TITLE = null;
	public static PTStyleClass LOGIN_SUBTITLE = null;
	public static PTStyleClass LOGIN_TEXT = null;
	
	


//	public static PTStyleClass OBJECT_ACTIONBAR_BG = null;
	public static PTStyleClass OBJECT_SUBHEAD = null;
	public static PTStyleClass OBJECT_TITLE = null;
	public static PTStyleClass OBJECT_SUBTITLE = null;
//	public static PTStyleClass OBJECT_ACTIONBAR_TEXT = null;
	public static PTStyleClass OBJECT_DIRECTORY_TEXT = null;
	public static PTStyleClass OBJECT_BTN = null;
	public static PTStyleClass OBJECT_HEADER_BG = null;	
	public static PTStyleClass OBJECT_HEADER = null;	
	public static PTStyleClass OBJECT_TEXT = null;	
	public static PTStyleClass OBJECT_CONTENT_BG = null;

	public static PTStyleClass OBJECTSEARCH_BG = null;
	public static PTStyleClass OBJECTSEARCH_HEADER = null;
	public static PTStyleClass OBJECTSEARCH_TEXT = null;

	public static PTStyleClass PAGELAYOUT_COLUMN_BG = null;
	public static PTStyleClass PAGELAYOUT_CONTENT_BG = null;
	

	public static PTStyleClass PLATPORTLET_HEADER_BG = null;
	public static PTStyleClass PLATPORTLET_SUBHEAD_BG = null;
	public static PTStyleClass PLATPORTLET_LIGHT_BG = null;
	public static PTStyleClass PLATPORTLET_DK_GRAY_BG = null;
	public static PTStyleClass PLATPORTLET_MD_GRAY_BG = null;
	public static PTStyleClass PLATPORTLET_LT_GRAY_BG = null;
//	public static PTStyleClass PLATPORTLET_ACTIONBAR_BG = null;
	public static PTStyleClass PLATPORTLET_BORDER = null;	
	public static PTStyleClass PLATPORTLET_MANDATORY_HEADER_BG = null;
	public static PTStyleClass PLATPORTLET_MANDATORY_BG = null;
	public static PTStyleClass PLATPORTLET_NARROW_HEADER = null;
//	public static PTStyleClass PLATPORTLET_NARROW_SUBHEAD = null;
	public static PTStyleClass PLATPORTLET_WIDE_HEADER = null;
//	public static PTStyleClass PLATPORTLET_WIDE_SUBHEAD = null;
//	public static PTStyleClass PLATPORTLET_TITLE = null;
//	public static PTStyleClass PLATPORTLET_SUBTITLE = null;
//	public static PTStyleClass PLATPORTLET_TEXT = null;
//	public static PTStyleClass PLATPORTLET_ACTIONBAR_TEXT = null;
	

	public static PTStyleClass TIPS_HEADER_BG = null;
	public static PTStyleClass TIPS_LIGHT_BG = null;
	public static PTStyleClass TIPS_SHADED_BG = null;
	public static PTStyleClass TIPS_HEADER = null;
	public static PTStyleClass TIPS_SUBHEAD = null;
	public static PTStyleClass TIPS_TITLE = null;
	public static PTStyleClass TIPS_TEXT = null;	
	public static PTStyleClass TIPS_SUBTITLE = null;
	
	public static PTStyleClass NAV_TOPBARDROPBAR_BG = null;
	public static PTStyleClass NAV_MIDTAB_BG = null;
	public static PTStyleClass NAV_MIDTABSPACER_BG = null;
	public static PTStyleClass NAV_TABON_BG = null;
	public static PTStyleClass NAV_TABOFF_BG = null;
	public static PTStyleClass NAV_TABSPACER_BG = null;
	public static PTStyleClass NAV_TABDROPBAR_BG = null;
	public static PTStyleClass NAV_TAB_UP_TEXT = null;
	public static PTStyleClass NAV_SIDEBARSECTION_BG = null;
	public static PTStyleClass NAV_SIDEBARSUBSECTION_BG = null;
	
	public static PTStyleClass NAV_PULLDOWN_TEXT = null;
	public static PTStyleClass NAV_HEADER = null;
	public static PTStyleClass NAV_SUBHEADER = null;
	public static PTStyleClass NAV_TABDROPBAR_SUBHEADER = null;
	public static PTStyleClass NAV_TOPBARDROPBAR_TEXT = null;
	public static PTStyleClass NAV_TOPBARSTRONG_TEXT = null;
	public static PTStyleClass NAV_MIDTAB_TEXT = null;
	public static PTStyleClass NAV_TAB_TEXT = null;
	public static PTStyleClass NAV_TABDROPBAR_TEXT = null;
	public static PTStyleClass NAV_SIDEBARSECTION_HEADER = null;
	public static PTStyleClass NAV_SIDEBARSUBSECTION_HEADER = null;
	public static PTStyleClass NAV_SIDEBARSUBSUBSECTION_HEADER = null;
	public static PTStyleClass NAV_SIDEBAR_SUBHEADER = null;
	public static PTStyleClass NAV_SIDEBAR_TEXT = null;
	public static PTStyleClass NAV_TOPBAR_TEXT = null;
	public static PTStyleClass NAV_TAB_ON_WHITE_BG = null;
	
		
	public static PTStyleClass NAV_MIDTAB_DROPDOWN_TEXT = null;
	public static PTStyleClass NAV_MIDTAB_DROPDOWN_ACTION_TEXT = null;
	public static PTStyleClass NAV_MIDTAB_DROPDOWN_BG = null;
	public static PTStyleClass NAV_MIDTAB_BTN = null;

	public static PTStyleClass CUSTOMAPP_TEXT = null;
	
	/**
	 * These styles are only for Section 508 compliance
	 * for people with disabilities.  They are not used
	 * for pages in normal mode.
	 */
	public static PTStyleClass D_HIDDEN_BLACK_TEXT = null;
	public static PTStyleClass D_HIDDEN_WHITE_TEXT = null;
	public static PTStyleClass D_HIDDEN_LTGRAY_TEXT = null;
	public static PTStyleClass D_HIDDEN_YELLOW_TEXT = null;
	
	/* PBC: Added for MARTOK */
	public static PTStyleClass BG_DARK = null;
	public static PTStyleClass EXTERNAL_CONTENT = null;
	public static PTStyleClass LAYOUT_ONE_COLUMN = null;
	public static PTStyleClass LAYOUT_THREE_COLUMN = null;
	public static PTStyleClass LAYOUT_TWO_COLUMN_NARROW_WIDE = null;
	public static PTStyleClass LAYOUT_TWO_COLUMN_WIDE_NARROW = null;
	public static PTStyleClass PORTAL_CONTENT = null;
	public static PTStyleClass PORTLET_BASE = null;
	public static PTStyleClass PORTLET_BODY = null;
	public static PTStyleClass PORTLET_BUTTONS = null;
	public static PTStyleClass PORTLET_COLLAPSE_BUTTON = null;
	public static PTStyleClass PORTLET_REMOVE_BUTTON = null;
	public static PTStyleClass PORTLET_HELP_BUTTON = null;
	public static PTStyleClass PORTLET_PREFS_BUTTON = null;
	public static PTStyleClass PORTLET_REFRESH_BUTTON = null;
	public static PTStyleClass PORTLET_TITLE = null;
	
	/* Private Constants */

	// Used in object instances such as pages and portlets.
	//
	// These Strings get concatenated with an instance variable to form
	// a unique class for each object. They are private because they
	// should only be accessed via the GetClassStringForPageUser() method.
	private static PTStyleClass PT_PAGE_COMMUNITY_PREFIX = null;
	private static PTStyleClass PT_PAGE_USER_CLASS_PREFIX = null;


	static
	{
		//	This semi-hack allows this subclass to create the superclass, while
		//	public creation of the super-class might not be allowed.
		bPreviousAllowPublicCreation = StyleClass.bAllowPublicCreation;
		StyleClass.bAllowPublicCreation = true;

		try
		{
			NAV_SIDEBAR_SUB_SUBSECTION_BG = new PTStyleClass("navSidebarSubSubsectionBg");
			OBJECT_BTN_TEXT = new PTStyleClass("objectBtnText");
			CUSTOMAPP_TEXT = new PTStyleClass( "customappText" );

//			CONTENTCANVAS_HEADER_BG = new PTStyleClass( "contentcanvasHeaderBg" );
//			CONTENTCANVAS_DISABLED_HEADER_BG = new PTStyleClass( "contentcanvasDisabledHeaderBg" );
//			CONTENTCANVAS_DISABLED_BG = new PTStyleClass( "contentcanvasDisabledBg" );
		
			FOOT_HEADER_BG = new PTStyleClass( "footHeaderBg" );
			FOOT_SUBHEAD = new PTStyleClass( "footSubhead" );
			FOOT_TITLE = new PTStyleClass( "footTitle" );
			FOOT_SUBTITLE = new PTStyleClass( "footSubtitle" );
			FOOT_TEXT = new PTStyleClass( "footText" );
//			FOOT_HEADER2 = new PTStyleClass( "footHeader2" );
//			FOOT_SUBHEAD2 = new PTStyleClass( "footSubhead2" );
//			FOOT_TITLE2 = new PTStyleClass( "footTitle2" );
//			FOOT_SUBTITLE2 = new PTStyleClass( "footSubtitle2" );
//			FOOT_TEXT2 = new PTStyleClass( "footText2" );
	
			NAV_MIDTAB_DROPDOWN_TEXT = new PTStyleClass( "navMidtabDropdownText" );
			NAV_MIDTAB_DROPDOWN_ACTION_TEXT = new PTStyleClass( "navMidtabDropdownActionText" );
			NAV_MIDTAB_DROPDOWN_BG = new PTStyleClass( "navMidtabDropdownBg" );
			NAV_MIDTAB_BTN = new PTStyleClass( "navMidtabBtn" );
			NAV_TAB_ON_WHITE_BG = new PTStyleClass( "navTabOnWhiteBg" );
			
			CAL_DIV = new PTStyleClass( "cal-Div" );
			
			TIPS_TEXT = new PTStyleClass( "tipsText" );			
			TIPS_HEADER_BG = new PTStyleClass( "tipsHeaderBg" );
			TIPS_LIGHT_BG = new PTStyleClass( "tipsLightBg" );
			TIPS_SHADED_BG = new PTStyleClass( "tipsShadedBg" );
			TIPS_HEADER = new PTStyleClass( "tipsHeader" );
			TIPS_SUBHEAD = new PTStyleClass( "tipsSubhead" );
			TIPS_TITLE = new PTStyleClass( "tipsTitle" );
			TIPS_SUBTITLE = new PTStyleClass( "tipsSubtitle" );

			PAGELAYOUT_COLUMN_BG = new PTStyleClass( "pagelayoutColumnBg" );
			PAGELAYOUT_CONTENT_BG = new PTStyleClass( "pagelayoutContentBg" );
			
			PLATPORTLET_HEADER_BG = new PTStyleClass( "platportletHeaderBg" );
			PLATPORTLET_SUBHEAD_BG = new PTStyleClass( "platportletSubheadBg" );
			PLATPORTLET_LIGHT_BG = new PTStyleClass( "platportletLightBg" );
			PLATPORTLET_DK_GRAY_BG = new PTStyleClass( "platportletDkGrayBg" );
			PLATPORTLET_MD_GRAY_BG = new PTStyleClass( "platportletMdGrayBg" );
			PLATPORTLET_LT_GRAY_BG = new PTStyleClass( "platportletLtGrayBg" );
//			PLATPORTLET_ACTIONBAR_BG = new PTStyleClass( "platportletActionbarBg" );
			PLATPORTLET_BORDER = new PTStyleClass( "platportletBorder" );
			PLATPORTLET_MANDATORY_HEADER_BG = new PTStyleClass( "platportletMandatoryHeaderBg" );
			PLATPORTLET_MANDATORY_BG = new PTStyleClass( "platportletMandatoryBg" );			
			PLATPORTLET_NARROW_HEADER = new PTStyleClass( "platportletNarrowHeader" );
//			PLATPORTLET_NARROW_SUBHEAD = new PTStyleClass( "platportletNarrowSubhead" );
			PLATPORTLET_WIDE_HEADER = new PTStyleClass( "platportletWideHeader" );
//			PLATPORTLET_WIDE_SUBHEAD = new PTStyleClass( "platportletWideSubhead" );
//			PLATPORTLET_TITLE = new PTStyleClass( "platportletTitle" );
//			PLATPORTLET_SUBTITLE = new PTStyleClass( "platportletSubtitle" );
//			PLATPORTLET_TEXT = new PTStyleClass( "platportletText" );
//			PLATPORTLET_ACTIONBAR_TEXT = new PTStyleClass( "platportletActionbarText" );

//			OBJECT_ACTIONBAR_BG = new PTStyleClass( "objectActionbarBg" );
			OBJECT_SUBHEAD = new PTStyleClass( "objectSubhead" );
			OBJECT_TITLE = new PTStyleClass( "objectTitle" );
			OBJECT_SUBTITLE = new PTStyleClass( "objectSubtitle" );
//			OBJECT_ACTIONBAR_TEXT = new PTStyleClass( "objectActionbarText" );
			OBJECT_DIRECTORY_TEXT = new PTStyleClass( "objectDirectoryText" );
			OBJECT_BTN = new PTStyleClass( "objectBtn" );
			
			OBJECTSEARCH_BG = new PTStyleClass( "objectsearchBg" );
			OBJECTSEARCH_HEADER = new PTStyleClass( "objectsearchHeader" );
			OBJECTSEARCH_TEXT = new PTStyleClass( "objectsearchText" );
			
			OBJECT_CONTENT_BG =new PTStyleClass( "objectContentBg" );	
			
			MENU_HEADER_BG = new PTStyleClass( "menuHeaderBg" );
			MENU_HEADER = new PTStyleClass( "menuHeader" );
			MENU_TEXT = new PTStyleClass( "menuText" );

			LIST_ITALIC_TEXT = new PTStyleClass( "listItalicText" );
			LIST_BOLD_ITALIC_TEXT = new PTStyleClass( "listBoldItalicText" );

			LIST_SORT_HEADER_BG = new PTStyleClass( "listSortHeaderBg" );
			LIST_SORT_HEADER = new PTStyleClass( "listSortHeader" );

			LIST_HEADER_BG = new PTStyleClass( "listHeaderBg" );
			LIST_ITEM_ONE_BG = new PTStyleClass( "listItemOneBg" );
			LIST_ITEM_TWO_BG = new PTStyleClass( "listItemTwoBg" );
			LIST_HEADER = new PTStyleClass( "listHeader" );
			LIST_SUBHEAD = new PTStyleClass( "listSubhead" );
			LIST_SUBTITLE = new PTStyleClass( "listSubtitle" );
			LIST_TEXT = new PTStyleClass( "listText" );
			LIST_SUBJECT_TEXT = new PTStyleClass( "listSubjectText" );
			LIST_READ_TEXT = new PTStyleClass( "listReadText" );
			LIST_UNREAD_TEXT = new PTStyleClass( "listUnreadText" );
			LIST_DATE_TEXT = new PTStyleClass( "listDateText" );
			LIST_PAGINATION_TEXT = new PTStyleClass( "listPaginationText" );
			LIST_HILITE_TEXT = new PTStyleClass( "listHiliteText" );
			LIST_NEW_TEXT = new PTStyleClass( "listNewText" );
			LIST_UPDATED_TEXT = new PTStyleClass( "listUpdatedText" );
	
			LOGIN_HEADER = new PTStyleClass( "loginHeader" );
			LOGIN_SUBHEAD = new PTStyleClass( "loginSubhead" );
			LOGIN_TITLE = new PTStyleClass( "loginTitle" );
			LOGIN_SUBTITLE = new PTStyleClass( "loginSubtitle" );
			LOGIN_TEXT = new PTStyleClass( "loginText" );
			
			FORM_PULLDOWN_TEXT = new PTStyleClass( "formPulldownText" );
			FORM_INPUT_BOX_TEXT = new PTStyleClass( "formInputBoxText" );
			FORM_BTN_TEXT = new PTStyleClass( "formBtnText" );
			FORM_EDITOR_BTN_TEXT = new PTStyleClass( "formEditorBtnText" );
			FORM_CHECKBOX_TEXT = new PTStyleClass( "formCheckboxText" );
			FORM_RADIO = new PTStyleClass( "formRadio" );
			FORM_SELECT_TEXT = new PTStyleClass( "formSelectText" );
			FORM_TEXTBOX_TEXT = new PTStyleClass( "formTextboxText" );
			
			DIALOGUE_BG = new PTStyleClass( "dialogueBg" );
			DIALOGUE_HEADER_BG = new PTStyleClass( "dialogueHeaderBg" );
			DIALOGUE_MED_BG = new PTStyleClass( "dialogueMedBg" );
			DIALOGUE_LIGHT_BG = new PTStyleClass( "dialogueLightBg" );
			DIALOGUE_GRAY_BG = new PTStyleClass( "dialogueGrayBg" );
			DIALOGUE_TAB_ON_BG = new PTStyleClass( "dialogueTabOnBg" );
			DIALOGUE_TAB_OFF_BG = new PTStyleClass( "dialogueTabOffBg" );
			DIALOGUE_SEARCH_BG = new PTStyleClass( "dialogueSearchBg" );
//			DIALOGUE_ACTIONBAR_BG = new PTStyleClass( "dialogueActionbarBg" );
			DIALOGUE_HEADER = new PTStyleClass( "dialogueHeader" );
			DIALOGUE_NOTICE_HEADER = new PTStyleClass( "dialogueNoticeHeader" );
			DIALOGUE_SUBHEAD = new PTStyleClass( "dialogueSubhead" );
			DIALOGUE_TITLE = new PTStyleClass( "dialogueTitle" );
			DIALOGUE_SUBTITLE = new PTStyleClass( "dialogueSubtitle" );
			DIALOGUE_TEXT = new PTStyleClass( "dialogueText" );
			DIALOGUE_TAB_TEXT = new PTStyleClass( "dialogueTabText" );
			DIALOGUE_FORM_TEXT = new PTStyleClass( "dialogueFormText" );
			DIALOGUE_MRU_TEXT = new PTStyleClass( "dialogueMRUText" );
//			DIALOGUE_ACTIONBAR_TEXT = new PTStyleClass( "dialogueActionbarText" );
//			DIALOGUE_BTN = new PTStyleClass( "dialogueBtn" );

			CUSTOMAPP_HEADER_BG = new PTStyleClass( "customappHeaderBg" );
			CUSTOMAPP_SUBHEAD_BG = new PTStyleClass( "customappSubheadBg" );
			CUSTOMAPP_LIGHT_BG = new PTStyleClass( "customappLightBg" );
			CUSTOMAPP_DK_GRAY_BG = new PTStyleClass( "customappDkGrayBg" );
			CUSTOMAPP_MD_GRAY_BG = new PTStyleClass( "genericMdGrayBg" );
			CUSTOMAPP_LT_GRAY_BG = new PTStyleClass( "customappLtGrayBg" );
			CUSTOMAPP_BORDER = new PTStyleClass( "customappBorder" );
			CUSTOMAPP_HEADER = new PTStyleClass( "customappHeader" );
			CUSTOMAPP_SUBHEAD = new PTStyleClass( "customappSubhead" );
			CUSTOMAPP_TITLE = new PTStyleClass( "customappTitle" );
			CUSTOMAPP_SUBTITLE = new PTStyleClass( "customappSubtitle" );
			CUSTOMAPP_TEXT = new PTStyleClass( "customappText" );

			DIR_HEADER_BG = new PTStyleClass( "dirHeaderBg" );
			DIR_ACTIONBAR_BG = new PTStyleClass( "dirActionbarBg" );
			DIR_HEADER = new PTStyleClass( "dirHeader" );
			DIR_TEXT = new PTStyleClass( "dirText" );
			DIR_BTN_TEXT = new PTStyleClass( "dirBtnText" );
			DIR_BTN = new PTStyleClass( "dirBtn" );
			DIR_DIVIDER = new PTStyleClass( "dirDivider" );
			DIR_CONTENT_SUBTITLE = new PTStyleClass( "dirContentSubtitle" );
			DIR_CONTENT_TEXT = new PTStyleClass( "dirContentText" );
			DIR_CONTENT_BG = new PTStyleClass( "dirContentBg" );
			
			CAL_HEADER_BG = new PTStyleClass( "calHeaderBg" );
			CAL_MONTH_TOGGLE_BG = new PTStyleClass( "calMonthToggleBg" );
			CAL_ACTIONBAR_BG = new PTStyleClass( "calActionbarBg" );
			CAL_DATE_BG = new PTStyleClass( "calDateBg" );
			CAL_BORDER = new PTStyleClass( "calBorder" );
			CAL_HEADER = new PTStyleClass( "calHeader" );
			CAL_SUBHEADER = new PTStyleClass( "calSubheader" );
			CAL_TITLE = new PTStyleClass( "calTitle" );
			CAL_SUBTITLE = new PTStyleClass( "calSubtitle" );
			CAL_MONTH_TOGGLE_HEADER = new PTStyleClass( "calMonthToggleHeader" );
			CAL_TEXT = new PTStyleClass( "calText" );
			CAL_ACTIONBAR_TEXT = new PTStyleClass( "calActionbarText" );
			CAL_DATE_TEXT = new PTStyleClass( "calDateText" );
			CAL_DAYS_TEXT = new PTStyleClass( "calDaysText" );
			CAL_DAYS_DIM_TEXT = new PTStyleClass( "calDaysDimText" );

			BREADCRUMB_TEXT = new PTStyleClass( "breadcrumbText" );
			
			BAN_TOPBAR_BG = new PTStyleClass( "banTopbarBg" );
			BAN_HEADER_BG = new PTStyleClass( "banHeaderBg" );
			BAN_HEADER = new PTStyleClass( "banHeader" );
			BAN_SUBHEADER = new PTStyleClass( "banSubhead" );
			BAN_TEXT = new PTStyleClass( "banText" );
			BAN_GREETING_TEXT = new PTStyleClass( "banGreetingText" );
			BAN_HELP_TEXT = new PTStyleClass( "banHelpText" );
			
			ALERT_ERROR_TEXT = new PTStyleClass( "alertErrorText" );
			ALERT_WARNING_TEXT = new PTStyleClass( "alertWarningText" );
			ALERT_CONFIRM_TEXT = new PTStyleClass( "alertConfirmText" );

		   	ALERT_DIALOG_BG = new PTStyleClass( "alertDialogueBg" );
		   	ALERT_DIALOG_TITLE = new PTStyleClass( "alertDialogueTitle" );
		   	ALERT_DIALOG_TEXT = new PTStyleClass( "alertDialogueText" );
		   	ALERT_ERROR_TITLE = new PTStyleClass( "alertErrorTitle" );
		   	ALERT_ERROR_SUBTITLE = new PTStyleClass( "alertErrorSubtitle" );
		   	ALERT_ERROR_DESCRIPTION = new PTStyleClass( "alertErrorDescription" );
		   	ALERT_REQUIRED_TEXT = new PTStyleClass( "alertRequiredText" );
			ALERT_BG = new PTStyleClass( "alertBg" );
			ALERT_TEXT = new PTStyleClass( "alertText" );
			ALERT_TITLE = new PTStyleClass( "alertTitle" );
			ALERT_SUBTITLE = new PTStyleClass( "alertSubtitle" );

			ACTIONBAR_BAN_DISABLED_TEXT = new PTStyleClass( "actionbarBanDisabledText" );
			ACTIONBAR_BAN_BG = new PTStyleClass( "actionbarBanBg" );
			ACTIONBAR_CAL_BG = new PTStyleClass( "actionbarCalBg" );
			ACTIONBAR_DIALOGUE_BG = new PTStyleClass( "actionbarDialogueBg" );
			ACTIONBAR_DIR_BG = new PTStyleClass( "actionbarDirBg" );
			ACTIONBAR_PORTLET_BG = new PTStyleClass( "actionbarPortletBg" );
			ACTIONBAR_OBJECT_BG = new PTStyleClass( "actionbarObjectBg" );
			ACTIONBAR_TEXT = new PTStyleClass( "actionbarText" );
			ACTIONBAR_BAN_TEXT = new PTStyleClass( "actionbarBanText" );
			ACTIONBAR_BAN_BTN = new PTStyleClass( "actionbarBanBtn" );
			ACTIONBAR_DIALOGUE_BTN = new PTStyleClass( "actionbarDialogueBtn" );
			ACTIONBAR_DIR_BTN = new PTStyleClass( "actionbarDirBtn" );
			ACTIONBAR_PORTLET_BTN = new PTStyleClass( "actionbarPortletBtn" );
			ACTIONBAR_OBJECT_BTN = new PTStyleClass( "actionbarObjectBtn" );
			ACTIONBAR_BAN_DIVIDER = new PTStyleClass( "actionbarBanDivider" );
			ACTIONBAR_OBJECT_DIVIDER = new PTStyleClass( "actionbarObjectDivider" );
			ACTIONBAR_PORTLET_DIVIDER = new PTStyleClass( "actionbarPortletDivider" );
			ACTIONBAR_DIR_DIVIDER = new PTStyleClass( "actionbarDirDivider" );
			ACTIONBAR_DIALOGUE_DIVIDER = new PTStyleClass( "actionbarDialogueDivider" );

			NAV_HEADER 					= new PTStyleClass( "navHeader" );
			NAV_MIDTAB_BG 				= new PTStyleClass( "navMidtabBg" );
			NAV_MIDTAB_TEXT 			= new PTStyleClass( "navMidtabText" );
			NAV_MIDTABSPACER_BG 		= new PTStyleClass( "navMidtabSpacerBg" );
			NAV_PULLDOWN_TEXT 			= new PTStyleClass( "navPulldownText" );
			NAV_SIDEBAR_SUBHEADER 		= new PTStyleClass( "navSidebarSubheader" );
			NAV_SIDEBAR_TEXT 			= new PTStyleClass( "navSidebarText" );
			NAV_SIDEBARSECTION_BG 		= new PTStyleClass( "navSidebarSectionBg" );
			NAV_SIDEBARSECTION_HEADER 	= new PTStyleClass( "navSidebarSectionHeader" );
			NAV_SIDEBARSUBSECTION_BG 	= new PTStyleClass( "navSidebarSubsectionBg" );
			NAV_SIDEBARSUBSECTION_HEADER = new PTStyleClass( "navSidebarSubsectionHeader" );
			NAV_SIDEBARSUBSUBSECTION_HEADER = new PTStyleClass( "navSidebarSubSubsectionHeader" );
			NAV_SUBHEADER 				= new PTStyleClass( "navSubhead" );
			NAV_TAB_TEXT 				= new PTStyleClass( "navTabText" );
			NAV_TABDROPBAR_BG 			= new PTStyleClass( "navTopDropbarBg" );
			NAV_TABDROPBAR_SUBHEADER 	= new PTStyleClass( "navTabDropbarSubhead" );
			NAV_TABDROPBAR_TEXT 		= new PTStyleClass( "navTabDropbarText" );
			NAV_TABOFF_BG 				= new PTStyleClass( "navTabOffBg" );
			NAV_TABON_BG 				= new PTStyleClass( "navTabOnBg" );
			NAV_TABSPACER_BG 			= new PTStyleClass( "navTabSpacerBg" );
			NAV_TOPBARDROPBAR_BG 		= new PTStyleClass( "navTabbarDropbarBg" );
			NAV_TOPBARDROPBAR_TEXT 		= new PTStyleClass( "navTopbarDropbarText" );
			NAV_TOPBARSTRONG_TEXT 		= new PTStyleClass( "navTopbarStrongText" );
			NAV_TOPBAR_TEXT 			= new PTStyleClass( "navTopbarText" );
			NAV_TAB_UP_TEXT				= new PTStyleClass( "navTabUpText");
			
			OBJECT_TEXT = new PTStyleClass( "objectText" );
			OBJECT_HEADER_BG = new PTStyleClass( "objectHeaderBg" );
			OBJECT_HEADER = new PTStyleClass( "objectHeader" );

			/**
			 * These styles are only for Section 508 compliance
			 * for people with disabilities.  They are not used
			 * for pages in normal mode.  The style classes
			 * need to be prefixed with d_, which is done
			 * automatically in 508 mode.
			 */
			D_HIDDEN_BLACK_TEXT 		= new PTStyleClass( "hiddenBlkText" );
			D_HIDDEN_WHITE_TEXT 		= new PTStyleClass( "hiddenWhtText" );
			D_HIDDEN_LTGRAY_TEXT 		= new PTStyleClass( "hiddenLtgrayText" );
			D_HIDDEN_YELLOW_TEXT 		= new PTStyleClass( "hiddenYelText" );
			
			/* PBC: Added for MARTOK */

			// Init public
			BG_DARK							= new PTStyleClass( "bgDark" );
			EXTERNAL_CONTENT				= new PTStyleClass( "externalContent" );
			LAYOUT_ONE_COLUMN				= new PTStyleClass( "layoutOneColumn" );
			LAYOUT_THREE_COLUMN				= new PTStyleClass( "layoutThreeColumn" );
			LAYOUT_TWO_COLUMN_NARROW_WIDE	= new PTStyleClass( "layoutTwoColumnNarrowWide" );
			LAYOUT_TWO_COLUMN_WIDE_NARROW	= new PTStyleClass( "layoutTwoColumnWideNarrow" );
			PORTAL_CONTENT					= new PTStyleClass( "portalContent" );
			PORTLET_BASE					= new PTStyleClass( "portletBase" );
			PORTLET_BODY					= new PTStyleClass( "portletBody" );
			PORTLET_BUTTONS					= new PTStyleClass( "portletButtons" );
			PORTLET_COLLAPSE_BUTTON			= new PTStyleClass( "portletCollapseButton" );
			PORTLET_REMOVE_BUTTON			= new PTStyleClass( "portletRemoveButton" );
			PORTLET_HELP_BUTTON				= new PTStyleClass( "portletHelpButton" );
			PORTLET_PREFS_BUTTON			= new PTStyleClass( "portletPrefsButton" );
			PORTLET_REFRESH_BUTTON			= new PTStyleClass( "portletRefreshButton" );
			PORTLET_TITLE					= new PTStyleClass( "portletTitle" );
			
			// Init private
			PT_PAGE_COMMUNITY_PREFIX		= new PTStyleClass( "ptCommunity-" );
			PT_PAGE_USER_CLASS_PREFIX		= new PTStyleClass( "ptPageUser-" );
		}
		catch( Exception e )
		{
			// do nothing			
			log.Error(e, "Couldnt initialize styles.");
		}

		//	semi-hack.  return the state of the super-class to whatever it was 
		//	before we created our static member vars.
		
		StyleClass.bAllowPublicCreation = bPreviousAllowPublicCreation;
	}
	
	public static String GetClassStringForCommunity(String id)
	{
		return ((id == null) ? "" : PT_PAGE_COMMUNITY_PREFIX + id);
	}
	
	public static String GetClassStringForPageUser(String login)
	{
		return ((login == null) ? "" : PT_PAGE_USER_CLASS_PREFIX + XPStringUtility.ToLowerCase(login));
	}
}