package com.plumtree.xpshared.htmlconstructs;

import com.plumtree.openfoundation.util.*;
import com.plumtree.xpshared.htmlelements.*;

/**
 * This construct creates a style sheet HTMLLink.  Eventually,
 * this should programmatically determine the correct style sheet
 * based upon the user's preferences and language/locale.
 */
public class PTStyleSheetLink
{
	// TODOCR: This should not be prefixed with PT since it is in the uiinfrastructure project.


    private String m_strFullURL;


    private String m_strLanguage;


    private String m_strStyleSheetName;


    private boolean useFullURL = false;

    /**
     * Pass in the style sheet name (without the -en and without the .css), and
     * the user's language.  For instance, if you want to use
     * mainstyle14-en.css, you would pass in "mainstyle14" as the first
     * argument, and "en" as the second.
     *
     * @param sStyleSheetName URL to style sheet
     * @param sUserLanguage
     *
     * @throws XPException
     */
    public PTStyleSheetLink(String sStyleSheetName, String sUserLanguage)
    {
        if ("".equals(sStyleSheetName) || (sStyleSheetName == null))
        {
            throw new XPException("PTStyleSheetLink: the style sheet name cannot be null.");
        }

        if ("".equals(sUserLanguage) || (sUserLanguage == null))
        {
            throw new XPException("PTStyleSheetLink: the language cannot be null.");
        }

        m_strStyleSheetName = sStyleSheetName;
        m_strLanguage       = sUserLanguage;
        useFullURL          = false;
    }

    /**
     * Pass in an HTMLLink object with the complete stylesheet link, overriding
     * base URL on this object
     *
     * @param link with full URL to style sheet
     *
     * @throws XPException
     */
    public PTStyleSheetLink(HTMLLink link)
    {
        if ((link == null) || "".equals(link.GetHRef()))
        {
            throw new XPException("PTStyleSheetLink: the full style sheet URL cannot be null.");
        }

        m_strFullURL = link.GetHRef();
        m_strLanguage = link.GetLang();
        useFullURL   = true;
    }

    /**
     * Pass in the full URL; ignore the base URL set on this object.
     *
     * @param sFullURL URL to style sheet
     *
     * @throws XPException
     */
    public PTStyleSheetLink(String sFullURL)
    {
        if ("".equals(sFullURL) || (sFullURL == null))
        {
            throw new XPException("PTStyleSheetLink: the full style sheet URL cannot be null.");
        }

        m_strFullURL = sFullURL;
    }

    /**
     *
     *
     * @return
     *
     * @throws Exception
     */
    public HTMLElement GetLinkAsHTMLElement(String _strStyleSheetsBaseURL)
    {
        HTMLLink myLink = new HTMLLink();

        myLink.SetRel("StyleSheet");
        myLink.SetHRef(CalculateStyleSheetURL(_strStyleSheetsBaseURL));
        myLink.SetType("text/css");
        myLink.SetLangLocal(m_strLanguage);

        return myLink;
    }

    /**
     *
     *
     * @return
     */
    public String GetStyleSheetAsString(String _strStyleSheetsBaseURL)
    {
        return CalculateStyleSheetURL(_strStyleSheetsBaseURL);
    }

    /**
     *
     *
     * @return
     */
    private String CalculateStyleSheetURL(String _strStyleSheetsBaseURL)
    {
        if (useFullURL == false)
        {
        	return _strStyleSheetsBaseURL + XPStringUtility.ToLowerCase(m_strStyleSheetName) 
            	+ "-" + XPStringUtility.ToLowerCase(m_strLanguage) + ".css";
        }
        else
        {
            return m_strFullURL;
        }
    }
}
