package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * <br><br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/struct/dirlang.html#h-8.2.4">Spec</a><br><br>
 */
public class HTMLBdo extends HTMLElement
{
    private static final String LTR = "ltr";
    private static final String RTL = "rtl";
    private static final String BDO = "bdo";

    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<bdo");
    private static final byte[] DIR_ATTR = HTMLElement.ToByteArray(" dir=\"");
    private static final byte[] CLOSE_BRACKET = HTMLElement.ToByteArray(">");
    private static final byte[] END_TAG = HTMLElement.ToByteArray("</bdo>");

    private static int iNextID = 0;

    private String sDir;

    private String sLang;

    /**
     * Creates a new HTMLBdo object.
     */
    public HTMLBdo()
    {
        sDir  = LTR;
        sLang = null;
    }

    /**
     * Get the value of the dir attribute.
     *
     * @return String value.
     */
    public String GetDir()
    {
        return sDir;
    }

    /**
     * Call this method to set the value of the dir attribute.
     *
     * @param s Pass the String value.
     *
     * @throws HTMLException Passing an invalid dir value with cause an
     *         exception     to be thrown.
     */
    public void SetDir(String s)
    {
        if (!(s.equals(LTR)) && !(s.equals(RTL)))
        {
            throw new HTMLException("The dir attribute on HTMLBdo must be 'ltr' or 'rtl'.");
        }

        sDir = s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);

        if (sDir != null)
        {
            WriteNugget(DIR_ATTR, sb, response);
            WriteNugget(sDir, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        WriteNugget(CLOSE_BRACKET, sb, response);
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = BDO + iNextID;
        iNextID++;

        return s;
    }
}
