package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * <br><br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/struct/global.html#h-7.5.1">Spec</a>
 */
public class HTMLBody extends HTMLElement
{
    static int iNextID = 0;

    private static final byte[] BEGINTAG = HTMLElement.ToByteArray("\n\n\n\n<body");
    private static final byte[] ENDTAG = HTMLElement.ToByteArray("\n</body>\n");
    private static final String BODY = "body";
    private static final byte[] MARGINWIDTH_ATTR = HTMLElement.ToByteArray(" marginwidth=\"");
    private static final byte[] MARGINHEIGHT_ATTR = HTMLElement.ToByteArray(" marginheight=\"");
    private static final byte[] LEFTMARGIN_ATTR = HTMLElement.ToByteArray(" leftmargin=\"");
    private static final byte[] TOPMARGIN_ATTR = HTMLElement.ToByteArray(" topmargin=\"");
    private static final byte[] ONACTIVATE_ATTR = HTMLElement.ToByteArray(" onactivate=\"");
    private static final byte[] ONAFTERPRINT_ATTR = HTMLElement.ToByteArray(" onafterprint=\"");
    private static final byte[] ONBEFOREACTIVATE_ATTR = HTMLElement.ToByteArray(" onbeforeactivate=\"");
    private static final byte[] ONBEFOREUNLOAD_ATTR = HTMLElement.ToByteArray(" onbeforeunload=\"");
    private static final byte[] ONBLUR_ATTR = HTMLElement.ToByteArray(" onblur=\"");
    private static final byte[] ONCONTROLSET_ATTR = HTMLElement.ToByteArray(" oncontrolset=\"");
    private static final byte[] ONDEACTIVATE_ATTR = HTMLElement.ToByteArray(" ondeactivate=\"");
    private static final byte[] ONERROR_ATTR = HTMLElement.ToByteArray(" onerror=\"");
    private static final byte[] ONFOCUS_ATTR = HTMLElement.ToByteArray(" onfocus=\"");
    private static final byte[] ONHELP_ATTR = HTMLElement.ToByteArray(" onhelp=\"");
    private static final byte[] ONLOAD_ATTR = HTMLElement.ToByteArray(" onload=\"");
    private static final byte[] ONRESIZE_ATTR = HTMLElement.ToByteArray(" onresize=\"");
    private static final byte[] ONRESIZEEND_ATTR = HTMLElement.ToByteArray(" onresizeend=\"");
    private static final byte[] ONRESIZESTART_ATTR = HTMLElement.ToByteArray(" onresizestart=\"");
    private static final byte[] ONUNLOAD_ATTR = HTMLElement.ToByteArray(" onunload=\"");

//    private static XPLogger myLogger = XPLogManager.GetLogger("com.plumtree.xpshared.htmlelements.HTMLBody");
//    private static boolean bDebug = myLogger.IsDebugEnabled();
//    private static boolean bError = myLogger.IsErrorEnabled();
//    private static boolean bWarn = myLogger.IsWarnEnabled();
//    private static boolean bInfo = myLogger.IsInfoEnabled();
//    private static boolean bFatal = myLogger.IsFatalEnabled();

    private String marginHeight = null;
    private String marginLeft = null;
    private String marginTop = null;
	private String marginWidth = null;
	private String sOnActivate = null;
	private String sOnAfterPrint = null;
	private String sOnBeforeActivate = null;
	private String sOnBeforeUnload = null;
	private String sOnBlur = null;
	private String sOnControlSet = null;
	private String sOnDeactivate = null;
	private String sOnError = null;
	private String sOnFocus = null;
	private String sOnHelp = null;
	private String sOnLoad = null;
	private String sOnResize = null;
	private String sOnResizeEnd = null;
	private String sOnResizeStart = null;
	private String sOnUnload = null;

    /**
     * Creates a new HTMLBody object.
     */
    public HTMLBody()
    {
    }

    /**
     * Get the value of the leftmargin attribute.
     *
     * @return String value.
     */
    public String GetLeftMargin()
    {
        return marginLeft;
    }

    /**
     * Get the value of the marginheight attribute.
     *
     * @return String value.
     */
    public String GetMarginHeight()
    {
        return marginHeight;
    }

    /**
     * Get the value of the marginwidth attribute.
     *
     * @return String value.
     */
    public String GetMarginWidth()
    {
        return marginWidth;
    }

    /**
     * Get the value of the onactivate attribute.
     *
     * @return String value.
     */
    public String GetOnActivate()
    {
        return sOnActivate;
    }

    /**
     * Get the value of the onafterprint attribute.
     *
     * @return String value.
     */
    public String GetOnAfterPrint()
    {
        return sOnAfterPrint;
    }

    /**
     * Get the value of the onbeforeactivate attribute.
     *
     * @return String value.
     */
    public String GetOnBeforeActivate()
    {
        return sOnBeforeActivate;
    }

    /**
     * Get the value of the onbeforeunload attribute.
     *
     * @return String value.
     */
    public String GetOnBeforeUnload()
    {
        return sOnBeforeUnload;
    }

    /**
     * Get the value of the onblur attribute.
     *
     * @return String value.
     */
    public String GetOnBlur()
    {
        return sOnBlur;
    }

    /**
     * Get the value of the oncontrolset attribute.
     *
     * @return String value.
     */
    public String GetOnControlSet()
    {
        return sOnControlSet;
    }

    /**
     * Get the value of the ondeactivate attribute.
     *
     * @return String value.
     */
    public String GetOnDeactivate()
    {
        return sOnDeactivate;
    }

    /**
     * Get the value of the onerror attribute.
     *
     * @return String value.
     */
    public String GetOnError()
    {
        return sOnError;
    }

    /**
     * Get the value of the onfocus attribute.
     *
     * @return String value.
     */
    public String GetOnFocus()
    {
        return sOnFocus;
    }

    /**
     * Get the value of the onhelp attribute.
     *
     * @return String value.
     */
    public String GetOnHelp()
    {
        return sOnHelp;
    }

    /**
     * Get the value of the onload attribute.
     *
     * @return String value.
     */
    public String GetOnLoad()
    {
        return sOnLoad;
    }

    /**
     * Get the value of the onresize attribute.
     *
     * @return String value.
     */
    public String GetOnResize()
    {
        return sOnResize;
    }

    /**
     * Get the value of the onresizeend attribute.
     *
     * @return String value.
     */
    public String GetOnResizeEnd()
    {
        return sOnResizeEnd;
    }

    /**
     * Get the value of the onresizestart attribute.
     *
     * @return String value.
     */
    public String GetOnResizeStart()
    {
        return sOnResizeStart;
    }

    /**
     * Get the value of the onunload attribute.
     *
     * @return String value.
     */
    public String GetOnUnload()
    {
        return sOnUnload;
    }

    /**
     * Get the value of the topmargin attribute.
     *
     * @return String value.
     */
    public String GetTopMargin()
    {
        return marginTop;
    }



    /**
     * Set the value of the leftmargin attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetLeftMargin(String s)
    {
        marginLeft = s;
    }



    /**
     * Set the value of the marginheight attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetMarginHeight(String s)
    {
        marginHeight = s;
    }



    /**
     * Set the value of the marginwidth attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetMarginWidth(String s)
    {
        marginWidth = s;
    }

    /**
     * Set the value of the onactivate attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnActivate(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnActivate = s;
    }

    /**
     * Set the value of the onafterprint attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnAfterPrint(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnAfterPrint = s;
    }

    /**
     * Set the value of the onbeforeactivate attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnBeforeActivate(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnBeforeActivate = s;
    }

    /**
     * Set the value of the onbeforeunload attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnBeforeUnload(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnBeforeUnload = s;
    }

    /**
     * Set the value of the onblur attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnBlur(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnBlur = s;
    }

    /**
     * Set the value of the oncontrolset attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnControlSet(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnControlSet = s;
    }

    /**
     * Set the value of the ondeactivate attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnDeactivate(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnDeactivate = s;
    }

    /**
     * Set the value of the onerror attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnError(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnError = s;
    }

    /**
     * Set the value of the onfocus attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnFocus(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnFocus = s;
    }

    /**
     * Set the value of the onhelp attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnHelp(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnHelp = s;
    }

    /**
     * Set the value of the onload attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnLoad(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnLoad = s;
    }

    /**
     * Set the value of the onresize attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnResize(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnResize = s;
    }

    /**
     * Set the value of the onresizeend attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnResizeEnd(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnResizeEnd = s;
    }

    /**
     * Set the value of the onresizestart attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnResizeStart(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnResizeStart = s;
    }

    /**
     * Set the value of the onunload attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetOnUnload(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnUnload = s;
    }



    /**
     * Set the value of the topmargin attribute.
     *
     * @param s Pass a valid String value.
     */
    public void SetTopMargin(String s)
    {
        marginTop = s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGINTAG, sb, response);

        if (sOnActivate != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onactivate attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(ONACTIVATE_ATTR, sb, response);
            WriteNugget(sOnActivate, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnAfterPrint != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onafterprint attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONAFTERPRINT_ATTR, sb, response);
            WriteNugget(sOnAfterPrint, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnBeforeActivate != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onbeforeactivate attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONBEFOREACTIVATE_ATTR, sb, response);
            WriteNugget(sOnBeforeActivate, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnBeforeUnload != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onbeforeunload attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONBEFOREUNLOAD_ATTR, sb, response);
            WriteNugget(sOnBeforeUnload, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnBlur != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onblur attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONBLUR_ATTR, sb, response);
            WriteNugget(sOnBlur, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnControlSet != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the oncontrolset attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONCONTROLSET_ATTR, sb, response);
            WriteNugget(sOnControlSet, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnDeactivate != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the ondeactivate attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONDEACTIVATE_ATTR, sb, response);
            WriteNugget(sOnDeactivate, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnError != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onerror attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONERROR_ATTR, sb, response);
            WriteNugget(sOnError, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnFocus != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onfocus attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONFOCUS_ATTR, sb, response);
            WriteNugget(sOnFocus, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnHelp != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onhelp attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONHELP_ATTR, sb, response);
            WriteNugget(sOnHelp, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnLoad != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onload attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONLOAD_ATTR, sb, response);
            WriteNugget(sOnLoad, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnResize != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onresize attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONRESIZE_ATTR, sb, response);
            WriteNugget(sOnResize, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnResizeEnd != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onresizeend attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONRESIZEEND_ATTR, sb, response);
            WriteNugget(sOnResizeEnd, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnResizeStart != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onresizestart attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONRESIZESTART_ATTR, sb, response);
            WriteNugget(sOnResizeStart, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnUnload != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onunload attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                	/*Exception e = new Exception();
//                	StackTraceElement[] elts = e.getStackTrace();
//                	String stackMsg = "";
//                	for (int i = 0; i < elts.length; ++i)
//                	{
//                		stackMsg += elts[i].toString();
//                		stackMsg += "\n";
//                	}
//                	myLogger.Error("Body's stack trace is: " + stackMsg );*/
//                }
            }

            WriteNugget(HTMLBody.ONUNLOAD_ATTR, sb, response);
            WriteNugget(sOnUnload, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (marginWidth != null)
        {
            WriteNugget(MARGINWIDTH_ATTR, sb, response);
            WriteNugget(marginWidth, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (marginHeight != null)
        {
            WriteNugget(MARGINHEIGHT_ATTR, sb, response);
            WriteNugget(marginHeight, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (marginLeft != null)
        {
            WriteNugget(LEFTMARGIN_ATTR, sb, response);
            WriteNugget(marginLeft, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (marginTop != null)
        {
            WriteNugget(TOPMARGIN_ATTR, sb, response);
            WriteNugget(marginTop, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(ENDTAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = BODY + iNextID;
        iNextID++;

        return s;
    }
}
