package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * Use this class to render an html br element.
 */
public class HTMLBr extends HTMLElement
{
    static int iNextID = 0;

    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<br");

    /**
     * Creates a new HTMLBr object.
     */
    public HTMLBr()
    {
    }

    /**
     * Clients may not nest html with a br element.
     *
     * @param e HTMLElement
     *
     * @throws HTMLException This method will throw an exception
     */
    public void AddInnerHTMLElement(HTMLElement e) throws HTMLException
    {
        throw new HTMLException("You may not nest html within a <br> element.");
    }

    /**
     * Clients may not nest html with a br element.
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception
     */
    public void AddInnerHTMLString(String s) throws HTMLException
    {
        throw new HTMLException("You may not nest html within a <br> element.");
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     *
     * @throws HTMLException
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response) throws HTMLException
    {
        WriteNugget(BEGIN_TAG, sb, response);
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
    	//WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean false.
     */
    protected boolean ProvidesClosingTag()
    {
        return false;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "br" + iNextID;
        iNextID++;

        return s;
    }
}
