package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openlog.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * Use this class to render an html comment.
 */
public class HTMLComment extends HTMLElement
{
	private static OpenLogger log = OpenLogService.GetLogger(
		OpenLogService.GetComponent("UI_Infrastructure"),
		"com.plumtree.xpshared.htmlelements.HTMLComment");

    static int iNextID = 0;

    /**
     * Creates a new HTMLComment object.
     *
     * @param s Pass the String comment.
     */
    public HTMLComment(String s)
    {
        try
        {
            super.AddInnerHTMLString(s);
        }
        catch (HTMLException he)
        {
			log.Error(he, "Unable to add comment string to parent class.");
        }
    }


    /**
     * HTMLElement.InternalWrite rewritten for HTMLComment.
     *
     * @param bFormat Pass a boolean indicating whether or not to format the
     *        output.
     * @param sbLocal Pass an XPStringBuilder to which you wish the String
     *        appended.  Pass null if you do not wish the string appended to
     *        an XPStringBuilder.
     * @param response Pass an IXPResponse to which you wish the String written.
     *        Pass null if you do not wish the string written to an
     *        IXPResponse.
     *
     * @throws HTMLException
     */
    protected void InternalWrite(boolean bFormat, XPStringBuilder sbLocal, IXPResponse response)
    {
        boolean bMethodFormat = bFormat;

        if (bForceFormattingOff)
        {
            bMethodFormat = false;
        }

        if (bMethodFormat)
        {
            appendTabs(sbLocal, response);
        }


        //   concrete subclasses implement the following
        AppendBeginningOfOpeningTag(bMethodFormat, sbLocal, response);

        GetInnerHTML(sbLocal, response, false);


        //   append the closing tag
        AppendClosingTag(bMethodFormat, sbLocal, response);
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     *
     * @throws HTMLException
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(CommonHTMLStrings.BEGIN_COMMENT, sb, response);
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(CommonHTMLStrings.END_COMMENT, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean false.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {

        String s = "comment" + iNextID;
        iNextID++;

        return s;
    }
}
