package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * This class can be used to display an HTML div element. <br>
 * <br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/struct/global.html#h-7.5.4">Spec</a><br><br>
 */
public class HTMLDiv extends HTMLElement
{
    static int iNextID = 0;

    private static final String DIV = "div";
    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<div");
    private static final byte[] ONSELECTSTART_ATTR = HTMLElement.ToByteArray(" onselectstart=\"");
    private static final byte[] END_TAG = HTMLElement.ToByteArray("</div>");

    private String sOnSelectStart = null;

    /**
     * Creates a new HTMLDiv object.
     */
    public HTMLDiv()
    {
    }

    /**
     *
     * @return
     */
    public String GetOnSelectStart()
    {
        return sOnSelectStart;
    }

    /**
     *
     * @param s
     */
    public void SetOnSelectStart(String s)
    {
        if (!HTMLElement.bAllowEvents)
        {
//            if (bError)
//            {
//                String sDesc = "The client of this htmlelement tree hopes to generate 508-compliant html.  This method shouldn't be called.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
        }

        sOnSelectStart = s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);

        if (sOnSelectStart != null)
        {
            if (!HTMLElement.bAllowEvents || GetDo508CheckingForThisInstance())
            {
//                if (bError)
//                {
//                    myLogger.Error("You are not allowed to set the onselectstart attribute value because the client of this html tree is attempting to render a 508-compliant page.");
//                    myLogger.Error("this: " + this);
//                }
            }

            WriteNugget(HTMLDiv.ONSELECTSTART_ATTR, sb, response);
            WriteNugget(sOnSelectStart, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = DIV + iNextID;
        iNextID++;

        return s;
    }
}
