package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * This class can be used to represent HTML elements for which no specific
 * class exists. <br>
 * <br>
 * It is far better thing to use a class specific to the element you wish  to
 * render, than to use this class.
 */
public class HTMLElementCollection extends HTMLElement
{
    static int iNextID = 0;

    /**
     * Creates a new HTMLElementCollection object.
     */
    public HTMLElementCollection()
    {
    }

    ///////////////////////////////////////////////
    //   The following method are overriden for HTMLElement, because they are
    //   meaningless for this class.

    /**
     * Method rewritten to disallow calling on an HTMLElementCollection
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public final void SetName(String s)
    {
        throw new HTMLException("You may not call SetName on an HTMLElementCollection.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLElementCollection
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public final void SetOnClick(String s)
    {
        throw new HTMLException("You may not call SetOnClick on an HTMLElementCollection.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLElementCollection
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public final void SetOnKeyDown(String s)
    {
        throw new HTMLException("You may not call SetOnKeyDown on an HTMLElementCollection.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLElementCollection
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public final void SetOnKeyPress(String s)
    {
        throw new HTMLException("You may not call SetOnKeyPress on an HTMLElementCollection.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLElementCollection
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public final void SetOnMouseOut(String s)
    {
        throw new HTMLException("You may not call SetOnMouseOut on an HTMLElementCollection.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLElementCollection
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public final void SetOnMouseOver(String s)
    {
        throw new HTMLException("You may not call SetOnMouseOver on an HTMLElementCollection.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLElementCollection
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public final void SetStyle(String s)
    {
        throw new HTMLException("You may not call SetStyle on an HTMLElementCollection.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLElementCollection
     *
     * @param s StyleClass
     *
     * @throws HTMLException This method will throw an exception.
     */
    public void SetStyleClass(StyleClass s)
    {
        throw new HTMLException("You may not call SetStyleClass on an HTMLElementCollection.");
    }

    // override HTMLElement.InternalWrite.  this version doesn't do any of
    //   the tag generation stuff, it only writes comments and nested html.

    /**
     * HTMLElement.InternalWrite rewritten for HTMLElementCollection.
     *
     * @param bFormat Pass a boolean indicating whether or not to format the
     *        output.
     * @param sbLocal Pass an XPStringBuilder to which you wish the String
     *        appended.  Pass null if you do not wish the string appended to
     *        an XPStringBuilder.
     * @param response Pass an IXPResponse to which you wish the String written.
     *        Pass null if you do not wish the string written to an
     *        IXPResponse.
     *
     * @throws HTMLException
     */
    protected void InternalWrite(boolean bFormat, XPStringBuilder sbLocal, IXPResponse response)
    {
		boolean bMethodFormat = bFormat;

		if (bForceFormattingOff)	{ bMethodFormat = false; }
		if (bForceFormattingOn)		{ bMethodFormat = true; }

		bDisplayDidPause = false;

		if( this.GetChildInMidRender() != null )
		{
			this.bResumingAndChildInMidRender = true;
		}

		if( this.GetLastChildFullyRendered() != null )
		{
			bResumingAndChildFinished = true;
		}

		if( !bResumingAndChildInMidRender && !bResumingAndChildFinished )
		{
			String sComment = GetPreComment();

			if ((sComment != null) && bPrintComments)
			{
				if (bMethodFormat)
				{
					appendTabs(sbLocal, response);
				}

				WriteNugget(BEGIN_COMMENT, sbLocal, response);
				WriteNugget(sComment, sbLocal, response);
				WriteNugget(END_COMMENT, sbLocal, response);

				if (bMethodFormat)
				{
					WriteNugget(sNewLine, sbLocal, response);
				}
			}

			if (bMethodFormat)
			{
				appendTabs(sbLocal, response);
			}

			GetInnerHTML(sbLocal, response, bMethodFormat);

			if( bDisplayDidPause )
			{
				return;
			}

			if (bMethodFormat)
			{
				WriteNugget(sNewLine, sbLocal, response);
			}

			sComment = GetPostComment();

			if ((sComment != null) && bPrintComments)
			{
				if (bMethodFormat)
				{
					appendTabs(sbLocal, response);
				}

				WriteNugget(BEGIN_COMMENT, sbLocal, response);
				WriteNugget(sComment, sbLocal, response);
				WriteNugget(END_COMMENT, sbLocal, response);

				if (bMethodFormat)
				{
					WriteNugget(sNewLine, sbLocal, response);
				}

			}


		}	//	not resuming
		else
		{
			// resuming
			GetInnerHTML(sbLocal, response, bMethodFormat);

			if( bDisplayDidPause )
			{
				return;
			}

			if (bMethodFormat)
			{
				WriteNugget(sNewLine, sbLocal, response);
			}

			String sComment = GetPostComment();

			if ((sComment != null) && bPrintComments)
			{
				if (bMethodFormat)
				{
					appendTabs(sbLocal, response);
				}

				WriteNugget(BEGIN_COMMENT, sbLocal, response);
				WriteNugget(sComment, sbLocal, response);
				WriteNugget(END_COMMENT, sbLocal, response);

				if (bMethodFormat)
				{
					WriteNugget(sNewLine, sbLocal, response);
				}
			}
		}	//	resuming


/*

        boolean bMethodFormat = bFormat;

        if (bForceFormattingOff)
        {
            bMethodFormat = false;
        }

        String sComment = GetPreComment();

        if ((sComment != null) && bPrintComments)
        {
            if (bMethodFormat)
            {
                appendTabs(sbLocal, response);
            }

            WriteNugget(BEGIN_COMMENT, sbLocal, response);

            WriteNugget(sComment, sbLocal, response);

            WriteNugget(END_COMMENT, sbLocal, response);

            if (bMethodFormat)
            {
                WriteNugget(sNewLine, sbLocal, response);
            }
        }

        GetInnerHTML(sbLocal, response, bMethodFormat);

        sComment = GetPostComment();

        if ((sComment != null) && bPrintComments)
        {
            if (bMethodFormat)
            {
                appendTabs(sbLocal, response);
            }

            WriteNugget(BEGIN_COMMENT, sbLocal, response);

            WriteNugget(sComment, sbLocal, response);

            WriteNugget(END_COMMENT, sbLocal, response);

            if (bMethodFormat)
            {
                WriteNugget(sNewLine, sbLocal, response);
            }
        }
*/

    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        //   do nothing
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        //   do nothing
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean false.
     */
    protected boolean ProvidesClosingTag()
    {
        return false;
    }

    ////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "generic" + iNextID;
        iNextID++;

        return s;
    }
}
