package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * <br><br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/present/frames.html#h-16.2.1">Spec</a><br><br>
 */
public class HTMLFrameSet extends HTMLElement
{
    private static int iNextID = 0;

    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<frameset");
    private static final byte[] ROWS_ATTR = HTMLElement.ToByteArray(" rows=\"");
    private static final byte[] COLS_ATTR = HTMLElement.ToByteArray(" cols=\"");
    private static final byte[] END_TAG = HTMLElement.ToByteArray("</frameset>");

    private String sCols = null;
    private String sRows = null;

    /**
     * Creates a new HTMLFrameSet object.
     */
    public HTMLFrameSet()
    {
    }

    /**
     * Get the value of the cols attribute.
     *
     * @return String value.
     */
    public String GetCols()
    {
        return sCols;
    }

    /**
     * Get the value of the rows attribute.
     *
     * @return String value.
     */
    public String GetRows()
    {
        return sRows;
    }

    /**
     * Call this method to set the value of the cols attribute.
     *
     * @param s A valid String value.
     */
    public void SetCols(String s)
    {
        sCols = s;
    }

    /**
     * Call this method to set the value of the rows attribute.
     *
     * @param s A valid String value.
     */
    public void SetRows(String s)
    {
        sRows = s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);

        if (sRows != null)
        {
            WriteNugget(ROWS_ATTR, sb, response);
            WriteNugget(sRows, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sCols != null)
        {
            WriteNugget(COLS_ATTR, sb, response);
            WriteNugget(sCols, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "frameset" + iNextID;
        iNextID++;

        return s;
    }
}
