package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * This class can be used to represent HTML elements for which no specific
 * class exists. <br>
 * <br>
 * It is far better thing to use a class specific to the element you wish  to
 * render, than to use this class.
 */
public class HTMLGenericElement extends HTMLElement
{
    static int iNextID = 0;

    String sHTML;

    /**
     * Call this method to create an HTMLGenericElement object.     <br><br>
     *
     * @param s Pass any valid HTML string.
     */
    public HTMLGenericElement(String s)
    {
        sHTML = s;
    }

    ////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////

    /**
     * Call this method to set the html string for this object.
     *
     * @param s Pass a valid html string.
     */
    public void SetDisplayString(String s)
    {
        sHTML = s;
    }

    ///////////////////////////////////////////////
    //   The following method are overriden for HTMLElement, because they are
    //   meaningless for this class.

    /**
     * Method rewritten to disallow calling on an HTMLGenericElement
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public void SetName(String s) throws HTMLException
    {
        throw new HTMLException("You may not call SetName on this object.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLGenericElement
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public void SetOnClick(String s) throws HTMLException
    {
        throw new HTMLException("You may not call SetOnClick on this object.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLGenericElement
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public void SetOnKeyDown(String s) throws HTMLException
    {
        throw new HTMLException("You may not call SetOnKeyDown on this object.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLGenericElement
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public void SetOnKeyPress(String s) throws HTMLException
    {
        throw new HTMLException("You may not call SetOnKeyPress on this object.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLGenericElement
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public void SetOnMouseOut(String s) throws HTMLException
    {
        throw new HTMLException("You may not call SetOnMouseOut on this object.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLGenericElement
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public void SetOnMouseOver(String s) throws HTMLException
    {
        throw new HTMLException("You may not call SetOnMouseOver on this object.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLGenericElement
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public void SetStyle(String s) throws HTMLException
    {
        throw new HTMLException("You may not call SetStyle on this object.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLGenericElement
     *
     * @param s String
     *
     * @throws HTMLException This method will throw an exception.
     */
    public void SetStyleClass(StyleClass s) throws HTMLException
    {
        throw new HTMLException("You may not call SetStyleClass on this object.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLGenericElement
     *
     * @param e HTMLElement
     *
     * @throws HTMLException This method will throw an exception.
     */
    public void AddInnerHTMLElement(HTMLElement e) throws HTMLException
    {
        throw new HTMLException("This class does not support adding nested HTML.");
    }

    /**
     * Method rewritten to disallow calling on an HTMLGenericElement
     *
     * @param s HTMLElement
     *
     * @throws HTMLException This method will throw an exception.
     */
    public void AddInnerHTMLString(String s) throws HTMLException
    {
        throw new HTMLException("This class does not support adding nested HTML.");
    }

    // override HTMLElement.InternalWrite

    /**
     * HTMLElement.InternalWrite() rewritten.
     *
     * @param bFormat Pass a boolean indicating whether or not to format the
     *        output.
     * @param sbLocal Pass an XPStringBuilder to which you wish the String
     *        appended.  Pass null if you do not wish the string appended to
     *        an XPStringBuilder.
     * @param response Pass an IXPResponse to which you wish the String written.
     *        Pass null if you do not wish the string written to an
     *        IXPResponse.
     *
     * @throws HTMLException
     */
    protected void InternalWrite(boolean bFormat, XPStringBuilder sbLocal, IXPResponse response) throws HTMLException
    {
        boolean bMethodFormat = bFormat;

        if (bForceFormattingOff)
        {
            bMethodFormat = false;
        }

        String sComment = GetPreComment();

        if ((sComment != null) && bPrintComments)
        {
            if (bMethodFormat)
            {
                appendTabs(sbLocal, response);
            }

            WriteNugget(CommonHTMLStrings.BEGIN_COMMENT, sbLocal, response);
            WriteNugget(sComment, sbLocal, response);
            WriteNugget(CommonHTMLStrings.END_COMMENT, sbLocal, response);

            if (bMethodFormat)
            {
                WriteNugget(sNewLine, sbLocal, response);
            }
        }

        if (bMethodFormat)
        {
            appendTabs(sbLocal, response);
        }

        WriteNugget(sHTML, sbLocal, response);

        if (bMethodFormat)
        {
            WriteNugget(sNewLine, sbLocal, response);
        }

        sComment = GetPostComment();

        if ((sComment != null) && bPrintComments)
        {
            if (bMethodFormat)
            {
                appendTabs(sbLocal, response);
            }

            WriteNugget(CommonHTMLStrings.BEGIN_COMMENT, sbLocal, response);
            WriteNugget(sComment, sbLocal, response);
            WriteNugget(CommonHTMLStrings.END_COMMENT, sbLocal, response);

            if (bMethodFormat)
            {
                WriteNugget(sNewLine, sbLocal, response);
            }
        }
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        //   do nothing
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        //   do nothing
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean false.
     */
    protected boolean ProvidesClosingTag()
    {
        return false;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "generic" + iNextID;
        iNextID++;

        return s;
    }
}
