package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**        Use this class to render an html head element.
 *        <br><br>
 *      <a target="new" href="http://www.w3.org/TR/html4/struct/global.html#h-7.4.1">Spec</a>
 *        <br><br>
 *
 */
public class HTMLHead extends HTMLElement
{
    private HTMLTitle myTitle;

    private static byte[] BEGINTAG = HTMLElement.ToByteArray("\n\n<head");
    private static byte[] ENDTAG   = HTMLElement.ToByteArray("\n</head>");

    static int iNextID = 0;
//
//    //   We share the logger for efficiency.
//    private static XPLogger myLogger = XPLogManager.GetLogger(HTMLHead.class);
//
//    //   pre-determining which levels are enabled allows the jvm to avoid
//    //   concatenating parameters of the logging methods.
//    private static boolean bDebug = myLogger.IsDebugEnabled();
//    private static boolean bError = myLogger.IsErrorEnabled();
//    private static boolean bWarn  = myLogger.IsWarnEnabled();
//    private static boolean bInfo  = myLogger.IsInfoEnabled();
//    private static boolean bFatal = myLogger.IsFatalEnabled();

    /**
     * Creates a new HTMLHead object.
     */
    public HTMLHead()
    {
        myTitle = null;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     *     that cloned object have new ID attribute values.
     * 
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "head" + iNextID;
        iNextID++;

        return s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     *     HTMLElement superclass will provide the remainder of the opening tag,
     *     repleate with attributes common to all html elements, such as name, and
        *    id, and class.
     * 
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGINTAG, sb, response);
    }

    /**
     * This method generates the closing tag of this html element.  If the
     *     providesClosingTag method returns false, then this method will never
     *     be called.
     * 
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(ENDTAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag.
     * Some elements only provide opening tags, such as the &lt;br&gt; element.
     * 
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**    @deprecated Use addInnerHTMLElement( HTMLTitle )
     * 
     */
    public void SetTitle(HTMLTitle t)
    {
        t.SetLevel(GetLevel() + 1);

        myTitle = t;
    }



    //  OVERRIDE these methods from HTMLElement to ensure
    //  that only appropriate elements are added within this <head>.

    /**
     * HTMLElement.addInnerHTMLElement rewritten.
     * 
     * @param e HTMLElement
     * @throws HTMLException Throws an exception if e is not an HTMLScript, 
     * 	HTMLLink, HTMLTitle, HTMLScriptCollection, or HTMLElementCollection 
     * 	element.
     */
    public void AddInnerHTMLElement(HTMLElement e) 
    {
        if (!(e instanceof HTMLScript) && !(e instanceof HTMLLink) && !(e instanceof HTMLTitle) && !(e instanceof HTMLScriptCollection) && !(e instanceof HTMLMeta) && !(e instanceof HTMLElementCollection))
        {
            throw new HTMLException("You may only add HTMLLink, HTMLMeta, HTMLScriptCollection, HTMLTitle, HTMLElementCollection, and HTMLScript elements to an HTMLHead element.");
        }

        if (e instanceof HTMLElementCollection)
        {
            //IXPEnumerator enumItems = e.GetInnerHTMLElements(false);

//            if (enumItems != null)
//            {
//                HTMLElement eTemp;
//
//                while (enumItems.MoveNext())
//                {
//                    eTemp = (HTMLElement) enumItems.GetCurrent();
//
////                    if (bDebug)
////                    {
////                        myLogger.Debug("eTemp: " + eTemp);
////                    }
//
//                } //   for each inner element
//
//            } //   got inner elements

        } // e instanceof HTMLElementCollection

        super.AddInnerHTMLElement(e);
    }

    /**
     * Method rewritten to throw an exception if called.  Use 
     * 	addInnerHTMLElement().
     * 
     * @param s String
     * @throws HTMLException Throws an exception.
     */
    public void AddInnerHTMLString(String s) 
    {
        throw new HTMLException("You may only add HTMLElement objects to an HTMLHead element.");
    }
}
