package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * Use this class to render an html img element. <br>
 * <br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/struct/objects.html#h-13.2">Spec</a><br><br>
 */
public class HTMLImg extends HTMLElement
{
    static int iNextID = 0;

    /**
     * Set this to false to cause the HTMLImg object to not enforce that the
     * alt attribute is set to a valid String.
     */
    public static boolean bEnforceAlt = false;

    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<img src=\"");
    private static final byte[] ALT_ATTR = HTMLElement.ToByteArray(" alt=\"");
    private static final byte[] BORDER_ATTR = HTMLElement.ToByteArray(" border=\"");
    private static final byte[] HEIGHT_ATTR = HTMLElement.ToByteArray(" height=\"");
    private static final byte[] WIDTH_ATTR = HTMLElement.ToByteArray(" width=\"");
    private static final byte[] ALIGN_ATTR = HTMLElement.ToByteArray(" align=\"");
	private static final byte[] USEMAP_ATTR = HTMLElement.ToByteArray(" usemap=\"");
	private static final byte[] ENDTAG = HTMLElement.ToByteArray("</img>");

    String sAlign = null;
    String sAlt;
    String sBorder = null;
    String sHeight = null;
    String sSrc;
    String sWidth = null;
	String sUseMap = null;

    private boolean bUseDisplayOptionsBaseUrl = true;

    /**
     *
     * @param src Pass the source string of this image.
     * @param alt Pass the alt string of this image.
     */
    public HTMLImg(String src, String alt)
    {
        sSrc   = src;
        sAlt   = alt;
        sAlign = null;
    }

    /**
     *
     * @param src Pass the source string of this image.
     * @param alt Pass the alt string of this image.
     * @param height Pass the height string of this image.
     * @param width Pass the width string of this image.
     */
    public HTMLImg(String src, String alt, String height, String width)
    {
        sSrc = src;
        sAlt = alt;

        sHeight = height;
        sWidth  = width;
    }

    //   to be deprectated

    /**
     * Get the value of the align attribute.
     *
     * @return String value.
     */
    public String GetAlign()
    {
        return sAlign;
    }

    /**
     * Get the value of the alt attribute.
     *
     * @return String value.
     */
    public String GetAlt()
    {
        return sAlt;
    }

    //   to be deprectated

    /**
     * Get the value of the border attribute.
     *
     * @return String value.
     */
    public String GetBorder()
    {
        return sBorder;
    }

    /**
     * Get the value of the height attribute.
     *
     * @return String value.
     */
    public String GetHeight()
    {
        return sHeight;
    }

    /**
     */
    public boolean GetUseDisplayOptionsBaseUrl()
    {
        return bUseDisplayOptionsBaseUrl;
    }

    /**
     * Get the value of the width attribute.
     *
     * @return String value.
     */
    public String GetWidth()
    {
        return sWidth;
    }
    
	/**
	 * Get the value of the usemap attribute.
	 *
	 * @return String value.
	 */
	public String GetUseMap()
	{
		return sUseMap;
	}


    //   to be deprectated

    /**
     * Set the value of the align atribute.
     *
     * @param s String value.
     */
    public void SetAlign(String s)
    {
        sAlign = s;
    }

    /**
     * Set the value of the alt atribute.
     *
     * @param s String value.
     */
    public void SetAlt(String s)
    {
        sAlt = s;
    }

    //   to be deprectated

    /**
     * Set the value of the border atribute.
     *
     * @param s String value.
     */
    public void SetBorder(String s)
    {
        sBorder = s;
    }

    /**
     * Set the value of the height atribute.
     *
     * @param s String value.
     */
    public void SetHeight(String s)
    {
        sHeight = s;
    }

    /**
     */
    public void SetUseDisplayOptionsBaseUrl(boolean b)
    {
        bUseDisplayOptionsBaseUrl = b;
    }

    /**
     * Set the value of the width atribute.
     *
     * @param s String value.
     */
    public void SetWidth(String s)
    {
        sWidth = s;
    }

	/**
	 * Set the value of the usemap atribute.
	 *
	 * @param s String value.
	 */
	public void SetUseMap(String s)
	{
		sUseMap= s;
	}

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     *
     * @throws HTMLException
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);

        if (!bUseDisplayOptionsBaseUrl)
        {
//            if (bDebug)
//            {
//                myLogger.Debug("Not using display options base url.");
//            }

            WriteNugget(sSrc, sb, response);
        }
        else
        {
//            if (bDebug)
//            {
//                myLogger.Debug("Using display root base url.");
//            }

            //	we're using the display options base url
            DisplayOptions options = this.GetDisplayOptions();
            String sBaseUrl          = null;

            if (options != null)
            {
                sBaseUrl = options.GetImgBaseUrl();
            }
//
//            if (bDebug)
//            {
//                myLogger.Debug("sBaseUrl: " + sBaseUrl);
//            }

            if (sBaseUrl == null)
            {
//                if (bError)
//                {
//                    myLogger.Error("The base URL for images must be set on the root display object.");
//                    myLogger.Error("eDisplayRoot: " + eDisplayRoot);
//                    myLogger.Error("sSrc: " + sSrc);
//                }

                WriteNugget(sSrc, sb, response);
            }
            else
            {
                WriteNugget(sBaseUrl, sb, response);
                WriteNugget(sSrc, sb, response);
            }
        }

        WriteNugget(QUOTE, sb, response);

        if (sAlt == null)
        {
            if (bEnforceAlt)
            {
//                if (bError)
//                {
//                    myLogger.Error("You must set the alt attribute in img with src " + sSrc + ".");
//                }
            }
        }

        if (sAlt != null)
        {
            WriteNugget(ALT_ATTR, sb, response);
            WriteNugget(sAlt, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sBorder != null)
        {
            WriteNugget(BORDER_ATTR, sb, response);
            WriteNugget(sBorder, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sAlign != null)
        {
            WriteNugget(ALIGN_ATTR, sb, response);
            WriteNugget(sAlign, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sHeight != null)
        {
            WriteNugget(HEIGHT_ATTR, sb, response);
            WriteNugget(sHeight, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sWidth != null)
        {
            WriteNugget(WIDTH_ATTR, sb, response);
            WriteNugget(sWidth, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

		if (sUseMap != null)
		{
			WriteNugget(USEMAP_ATTR, sb, response);
			WriteNugget(sUseMap, sb, response);
			WriteNugget(QUOTE, sb, response);
		}
    }
    
    public String GetSrc() {
    	if (!bUseDisplayOptionsBaseUrl)
        {
            return sSrc;
        }
        else
        {
            //	we're using the display options base url
            DisplayOptions options = this.GetDisplayOptions();
            String sBaseUrl          = null;

            if (options != null)
            {
                sBaseUrl = options.GetImgBaseUrl();
            }
            if (sBaseUrl == null)
            {
            	return sSrc;
            }
            else
            {
            	return sBaseUrl + sSrc;
            }
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
    	WriteNugget(ENDTAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean false.
     */
    protected boolean ProvidesClosingTag()
    {
        return false;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "img" + iNextID;
        iNextID++;

        return s;
    }
}
