package com.plumtree.xpshared.htmlelements;

/**
 * This class contains constants identifying the various types of INPUT
 * element.  These constants can be used to instantiate HTMLInput type
 * objects. <br><br>
 *
 * @see HTMLInput#HTMLInput( int, String, String, String )
 */
public class HTMLInputTypes
{

    public static final int BUTTON = 1;
    public static final int CHECKBOX = 2;
    public static final int FILE = 3;
    public static final int HIDDEN = 4;
    public static final int IMAGE = 5;
    public static final int PASSWORD = 6;
    public static final int RADIO = 7;
    public static final int RESET = 8;
    public static final int SUBMIT = 9;
    public static final int TEXT = 10;

    public static final HTMLInputTypes BUTTON_TYPE = new HTMLInputTypes("button", BUTTON);
    public static final HTMLInputTypes CHECKBOX_TYPE = new HTMLInputTypes("checkbox", CHECKBOX);
    public static final HTMLInputTypes FILE_TYPE = new HTMLInputTypes("file", FILE);
    public static final HTMLInputTypes HIDDEN_TYPE = new HTMLInputTypes("hidden", HIDDEN);
    public static final HTMLInputTypes IMAGE_TYPE = new HTMLInputTypes("image", IMAGE);
    public static final HTMLInputTypes PASSWORD_TYPE = new HTMLInputTypes("password", PASSWORD);
    public static final HTMLInputTypes RADIO_TYPE = new HTMLInputTypes("radio", RADIO);
    public static final HTMLInputTypes RESET_TYPE = new HTMLInputTypes("reset", RESET);
    public static final HTMLInputTypes SUBMIT_TYPE = new HTMLInputTypes("submit", SUBMIT);
    public static final HTMLInputTypes TEXT_TYPE = new HTMLInputTypes("text", TEXT);

    private String sTypeName = null;

    private int iType;

    //  this class shouldn't be constructed.

    /**
     * Creates a new HTMLInputTypes object.
     *
     * @param s String type name
     * @param i int type int.
     */
    private HTMLInputTypes(String s, int i)
    {
        sTypeName = s;
        iType     = i;
    }

    /**
     * Get the int representation of this type.
     *
     * @return int
     */
    public int ToInt()
    {
        return iType;
    }

    /**
     * Get the string representation of this type.
     *
     * @return String
     */
    public String ToString()
    {
        return sTypeName;
    }

    /**
     * Call to convert an int type value to a String.
     *
     * @param iType int type respresentation
     *
     * @return String representation
     */
    public static String ToString(int iType)
    {
        String sTemp = null;

        switch (iType)
        {
        case HTMLInputTypes.TEXT:
            sTemp = "text";

            break;

        case HTMLInputTypes.SUBMIT:
            sTemp = "submit";

            break;

        case HTMLInputTypes.BUTTON:
            sTemp = "button";

            break;

        case HTMLInputTypes.CHECKBOX:
            sTemp = "checkbox";

            break;

        case HTMLInputTypes.FILE:
            sTemp = "file";

            break;

        case HTMLInputTypes.HIDDEN:
            sTemp = "hidden";

            break;

        case HTMLInputTypes.IMAGE:
            sTemp = "image";

            break;

        case HTMLInputTypes.PASSWORD:
            sTemp = "password";

            break;

        case HTMLInputTypes.RADIO:
            sTemp = "radio";

            break;

        case HTMLInputTypes.RESET:
            sTemp = "reset";

            break;
        }

        return sTemp;
    }
}
