package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * This
 */
public class HTMLLayer extends HTMLElement
{
    static int iNextID = 0;

    private static final String LAYER = "layer";
    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<layer");
    private static final byte[] END_TAG = HTMLElement.ToByteArray("</layer>");
    private static final byte[] ZINDEX_ATTR = HTMLElement.ToByteArray(" z-index=\"");

    private String sZIndex = null;

    /**
     * Creates a new HTMLLayer object.
     */
    public HTMLLayer()
    {
    }

    /**
     * Get the value of the zindex attribute.
     *
     * @return String value.
     */
    public String GetZIndex()
    {
        return sZIndex;
    }

    /**
     * Set the value of the zindex attribute.
     *
     * @param s String value.
     */
    public void SetZIndex(String s)
    {
        sZIndex = s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);

        if (sZIndex != null)
        {
            WriteNugget(ZINDEX_ATTR, sb, response);
            WriteNugget(sZIndex, sb, response);
            WriteNugget(CommonHTMLStrings.QUOTE, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = LAYER + iNextID;
        iNextID++;

        return s;
    }
}
