package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * Title:        HTMLList Description: Copyright:    Copyright (c) 2002
 * Company:      Plumtree Software
 *
 * @version 1.0
 * @author Chris Bucchere
 */
public class HTMLList extends HTMLElement
{
    static int iNextID = 0;

    private int m_listType;

    /**
     * Creates a new HTMLList object.
     *
     * @param listType
     *
     * @throws HTMLException
     */
    public HTMLList(int listType)
    {
        m_listType = listType;
    }

    /**
     * Obtain an int indicating which type of list this is.  See HTMLListTypes
     * class
     *
     * @return int type
     */
    public int GetListType()
    {
        return m_listType;
    }

    /**
     *
     * @param e
     *
     * @throws HTMLException
     */
    public void AddInnerHTMLElement(HTMLElement e)
    {
        if (!(e instanceof HTMLListItem))
        {
            throw new HTMLException("You can only and HTMLListItem object to HTMLList objects.");
        }

        super.AddInnerHTMLElement(e);

        HTMLListItem it = (HTMLListItem) e;

        it.SetParentList(this);
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(HTMLListTypes.GetListOpeningTag(m_listType), sb, response);
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(HTMLListTypes.GetListClosingTag(m_listType), sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "list" + iNextID;
        iNextID++;

        return s;
    }
}
