package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * Title:        HTMLListItem Description: Copyright:    Copyright (c) 2002
 * Company:      Plumtree Software
 *
 * @version 1.0
 * @author Chris Bucchere
 */
public class HTMLListItem extends HTMLElement
{
    static int iNextID = 0;

    private HTMLList m_parentList;

    private boolean m_isDefinition;

    /**
     * Creates a new HTMLListItem object.
     *
     * @throws HTMLException
     */
    public HTMLListItem()
    {
        m_isDefinition = false;
    }

    /**
     *
     */
    public boolean IsDefinition()
    {
        return m_isDefinition;
    }

    /**
     *
     * @param parentList
     */
    public void SetParentList(HTMLList parentList)
    {
        m_parentList = parentList;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(HTMLListTypes.GetListItemOpeningTag(m_parentList.GetListType(), m_isDefinition), sb, response);
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(HTMLListTypes.GetListItemClosingTag(m_parentList.GetListType(), m_isDefinition), sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "li" + iNextID;
        iNextID++;

        return s;
    }
}
