package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * Use this class to render an html map element. <br>
 * <br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/struct/objects.html#h-13.2">Spec</a><br><br>
 */
public class HTMLMap extends HTMLElement
{
	static int iNextID = 0;

	private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<map");
	private static final byte[] ID_ATTR = HTMLElement.ToByteArray(" id=\"");
	private static final byte[] NAME_ATTR = HTMLElement.ToByteArray(" name=\"");
	private static final byte[] ENDTAG = HTMLElement.ToByteArray("</map>");


	String sName = null;

	String sID = null;

	/**
	 * Constructs a new HTMLMap
	 * 
	 */
	public HTMLMap()
	{
	}

	/**
	 * Constructs a new HTMLMap
	 * 
	 * @param name Pass the name string of this map.
	 * @param id Pass the id string of this map.
	 */
	public HTMLMap(String name, String id)
	{
		sName = name;
		sID = id;
	}

	/**
	 * Get the value of the id attribute.
	 *
	 * @return String value.
	 */
	public String GetID()
	{
		return sID;
	}

	/**
	 * Get the value of the name attribute
	 * 
	 * @return String
	 */
	public String GetName()
	{
		return sName;
	}

	/**
	 * Sets the value of the id attribute.
	 * @param strID The id to set
	 */
	public void SetID(String strID)
	{
		this.sID = strID;
	}

	/**
	 * Sets the value of the name attribute.
	 * @param sHref The name to set
	 */
	public void SetName(String strName)
	{
		this.sName = strName;
	}

	/**
	 * This method generates the beginning of the opening tag.  The abstract
	 * HTMLElement superclass will provide the remainder of the opening tag,
	 * repleate with attributes common to all html elements, such as name, and
	 * id, and class.
	 *
	 * @param bFormat boolean indicating whether or not to format.
	 * @param sb XPStringBuilder
	 * @param response IXPResponse
	 *
	 * @throws HTMLException
	 */
	protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
	{
		WriteNugget(BEGIN_TAG, sb, response);

		if (sID != null)
		{
			WriteNugget(ID_ATTR, sb, response);
			WriteNugget(sID, sb, response);
			WriteNugget(QUOTE, sb, response);
		}

		if (sName != null)
		{
			WriteNugget(NAME_ATTR, sb, response);
			WriteNugget(sName, sb, response);
			WriteNugget(QUOTE, sb, response);
		}

	}

	/**
	 * This method generates the closing tag of this html element.  If the
	 * providesClosingTag method returns false, then this method will never be
	 * called.
	 *
	 * @param bFormat boolean indicating whether or not to format.
	 * @param sb XPStringBuilder
	 * @param response IXPResponse
	 */
	protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
	{
		WriteNugget(ENDTAG, sb, response);
	}

	/**
	 * Indicates whether or not this HTML element provides a closing tag. Some
	 * elements only provide opening tags, such as the &lt;br&gt; element.
	 *
	 * @return boolean false.
	 */
	protected boolean ProvidesClosingTag()
	{
		return true;
	}

	/**
	 * Get the next id value.  This is used by the GetClone method to ensure
	 * that cloned object have new ID attribute values.
	 *
	 * @return A valid id string.
	 */
	String GetNextID()
	{
		String s = "img" + iNextID;
		iNextID++;

		return s;
	}

}
