package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * <a target="new"
 * href="http://www.w3.org/TR/html4/struct/global.html#h-7.4.4.2">See the
 * spec</a>
 */
public class HTMLMeta extends HTMLElement
{
    private static final byte[] BEGINTAG = HTMLElement.ToByteArray("<meta");
	private static final byte[] END_TAG = HTMLElement.ToByteArray("</meta>");
    private static final byte[] CONTENT_ATTR = HTMLElement.ToByteArray(" content=\"");
    private static final byte[] HTTPEQUIV_ATTR = HTMLElement.ToByteArray(" http-equiv=\"");

    private static final String SPACE = " ";
    static int iNextID                = 0;


    private String myContent;


    private String sHttpEquiv;

    /**
     *
     * @param sContent Pass the content string of this meta.
     *
     * @throws HTMLException
     */
    public HTMLMeta(String sContent)
    {
        myContent  = sContent;
        sHttpEquiv = null;
    }

    /**
     *
     * @return
     */
    public String GetHTTPEquiv()
    {
        return sHttpEquiv;
    }

    /**
     *
     * @param s
     */
    public void SetHTTPEquiv(String s)
    {
        sHttpEquiv = s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGINTAG, sb, response);

        if (sHttpEquiv != null)
        {
            WriteNugget(HTTPEQUIV_ATTR, sb, response);
            WriteNugget(sHttpEquiv, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (myContent != null)
        {
            WriteNugget(CONTENT_ATTR, sb, response);
            WriteNugget(myContent, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
    	WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean false.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "meta" + iNextID;
        iNextID++;

        return s;
    }
}
