package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * <br><br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/interact/scripts.html#h-18.2.1">Spec</a>
 */
public class HTMLScript extends HTMLElement
{
    static int iNextID = 0;

    //	For 508 compliance we cannot allow script tags.

    /** Toggle this false to log errors when this object is     created. */
    public static boolean bAllowScript = true;

    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<script");
    private static final byte[] TYPE_ATTR = HTMLElement.ToByteArray(" type=\"");
    private static final byte[] SRC_ATTR = HTMLElement.ToByteArray(" src=\"");
    private static final byte[] END_TAG = HTMLElement.ToByteArray("</script>");

	/**
	 *
	 * @deprecated
	 */
	public static final String TYPE_JAVASCIPT = "text/javascript";
	public static final String TYPE_JAVASCRIPT = "text/javascript";

    String myType;
    String src;

    /**
     *
     * @param sType Pass the type string of this script.  See public type
     *        members.
     */
    public HTMLScript(String sType)
    {
//        if (!bAllowScript)
//        {
//            if (bError)
//            {
//                String sDesc = "You are not allowed to use the HTMLScript class because this portal is attempting to render a 508-compliant page.";
//
//                Exception e = new Exception(sDesc);
//
//                myLogger.Error(sDesc, e);
//            }
//        }

        myType = sType;
        src    = null;
    }

    /**
     * Call this method to get the value of the src attribute.
     *
     * @return String value
     */
    public String GetSrc()
    {
        return src;
    }

    /**
     * Call this method to set the src attribute value.
     *
     * @param s String value
     */
    public void SetSrc(String s)
    {
        src = s;
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);

        WriteNugget(TYPE_ATTR, sb, response);
        WriteNugget(myType, sb, response);
        WriteNugget(QUOTE, sb, response);

        if (src != null)
        {
            WriteNugget(SRC_ATTR, sb, response);
            WriteNugget(src, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "script" + iNextID;
        iNextID++;

        return s;
    }
}
