package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 *
 * @version $Revision: #1 $
 * @author $author$
 */
public class HTMLScriptCollection extends HTMLElement
{
    static int iNextID = 0;

    /**
     * Creates a new HTMLScriptCollection object.
     */
    public HTMLScriptCollection()
    {
    }

    ///////////////////////////////////////////////
    //   The following method are overriden for HTMLElement, because they are
    //   meaningless for this class.

    /**
     * This method rewritten to prevent usage.
     *
     * @param s String
     *
     * @throws HTMLException Throws an HTMLException
     */
    public final void SetName(String s)
    {
        throw new HTMLException("You may not call SetName on an HTMLScriptCollection.");
    }

    /**
     * This method rewritten to prevent usage.
     *
     * @param s String
     *
     * @throws HTMLException Throws an HTMLException
     */
    public final void SetOnClick(String s)
    {
        throw new HTMLException("You may not call SetOnClick on an HTMLScriptCollection.");
    }

    /**
     * This method rewritten to prevent usage.
     *
     * @param s String
     *
     * @throws HTMLException Throws an HTMLException
     */
    public final void SetOnKeyDown(String s)
    {
        throw new HTMLException("You may not call SetOnKeyDown on an HTMLScriptCollection.");
    }

    /**
     * This method rewritten to prevent usage.
     *
     * @param s String
     *
     * @throws HTMLException Throws an HTMLException
     */
    public final void SetOnKeyPress(String s)
    {
        throw new HTMLException("You may not call SetOnKeyPress on an HTMLScriptCollection.");
    }

    /**
     * This method rewritten to prevent usage.
     *
     * @param s String
     *
     * @throws HTMLException Throws an HTMLException
     */
    public final void SetOnMouseOut(String s)
    {
        throw new HTMLException("You may not call SetOnMouseOut on an HTMLScriptCollection.");
    }

    /**
     * This method rewritten to prevent usage.
     *
     * @param s String
     *
     * @throws HTMLException Throws an HTMLException
     */
    public final void SetOnMouseOver(String s)
    {
        throw new HTMLException("You may not call SetOnMouseOver on an HTMLScriptCollection.");
    }

    /**
     * This method rewritten to prevent usage.
     *
     * @param s String
     *
     * @throws HTMLException Throws an HTMLException
     */
    public final void SetStyle(String s)
    {
        throw new HTMLException("You may not call SetStyle on an HTMLScriptCollection.");
    }

    /**
     * This method rewritten to prevent usage.
     *
     * @param s String
     *
     * @throws HTMLException Throws an HTMLException
     */
    public void SetStyleClass(StyleClass s)
    {
        throw new HTMLException("You may not call SetStyleClass on an HTMLScriptCollection.");
    }

    /**
     * HTMLElement.addInnerHTMLElement rewritten.
     *
     * @param e Pass an HTMLScript or an HTMLScriptCollection object.
     *
     * @throws HTMLException Throws an exception if the parameter is not
     *         correct.
     */
    public void AddInnerHTMLElement(HTMLElement e)
    {
//        if (!((e instanceof HTMLScript) || (e instanceof HTMLScriptCollection)))
//        {
//            throw new HTMLException("You may only nest HTMLScript or HTMLScriptCollection elements within an HTMLScriptCollection.");
//        }

        super.AddInnerHTMLElement(e);
    }

    /**
     * This method rewritten to prevent usage.
     *
     * @param s String
     *
     * @throws HTMLException Throws an HTMLException
     */
    public void AddInnerHTMLString(String s)
    {
        //throw new HTMLException("You may only nest elements within an HTMLScriptCollection: String " + s);
    }

    // override HTMLElement.InternalWrite.  this version doesn't do any of
    //   the tag generation stuff, it only writes comments and nested html.

    /**
     * HTMLElement.InternalWrite rewritten.
     *
     * @param bFormat Pass a boolean indicating whether or not to format the
     *        output.
     * @param sbLocal Pass an XPStringBuilder to which you wish the String
     *        appended.  Pass null if you do not wish the string appended to
     *        an XPStringBuilder.
     * @param response Pass an IXPResponse to which you wish the String written.
     *        Pass null if you do not wish the string written to an
     *        IXPResponse.
     *
     * @throws HTMLException
     */
    protected void InternalWrite(boolean bFormat, XPStringBuilder sbLocal, IXPResponse response)
    {
        boolean bMethodFormat = bFormat;

        if (bForceFormattingOff)
        {
            bMethodFormat = false;
        }

        String sComment = GetPreComment();

        if ((sComment != null) && bPrintComments)
        {
            if (bMethodFormat)
            {
                appendTabs(sbLocal, response);
            }

            WriteNugget(BEGIN_COMMENT, sbLocal, response);
            WriteNugget(sComment, sbLocal, response);
            WriteNugget(END_COMMENT, sbLocal, response);

            if (bMethodFormat)
            {
                WriteNugget(sNewLine, sbLocal, response);
            }
        }

        GetInnerHTML(sbLocal, response, bMethodFormat);

        sComment = GetPostComment();

        if ((sComment != null) && bPrintComments)
        {
            if (bMethodFormat)
            {
                appendTabs(sbLocal, response);
            }

            WriteNugget(BEGIN_COMMENT, sbLocal, response);

            WriteNugget(sComment, sbLocal, response);

            WriteNugget(END_COMMENT, sbLocal, response);

            if (bMethodFormat)
            {
                WriteNugget(sNewLine, sbLocal, response);
            }
        }
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        //   do nothing
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param reponse IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse reponse)
    {
        //   do nothing
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return false;
    }

    ////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "htmlscriptcollection" + iNextID;
        iNextID++;

        return s;
    }
}
