package com.plumtree.xpshared.htmlelements;

//import com.plumtree.openfoundation.logging.*;
import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;


/**
 * This class can be used to display an HTML b element. <br><br>
 */
public class HTMLStrong extends HTMLElement
{
    static int iNextID = 0;

    private static final String STRONG = "strong";
    private static final byte[] BEGIN_TAG = HTMLElement.ToByteArray("<strong");
    private static final byte[] END_TAG = HTMLElement.ToByteArray("</strong>");

    /**
     * Creates a new HTMLStrong object.
     */
    public HTMLStrong()
    {
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(BEGIN_TAG, sb, response);
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(END_TAG, sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = STRONG + iNextID;
        iNextID++;

        return s;
    }
}
