package com.plumtree.xpshared.htmlelements;

import com.plumtree.openfoundation.util.*;
import com.plumtree.openfoundation.web.*;

/**
 * Use this HTMLElement object to generate an html table tag. <br>
 * <br>
 * <a target="new"
 * href="http://www.w3.org/TR/html4/struct/tables.html#h-11.2.1">Spec</a>
 */
public class HTMLTable extends HTMLElement
{
    static int iNextID = 0;

    public static boolean bEnforceSummary = false;

    private static final byte[] BEGINTAG 	= HTMLElement.ToByteArray("<table");
    private static final byte[] ALIGN_ATTR 	= HTMLElement.ToByteArray(" align=\"");
    private static final byte[] BGCOLOR_ATTR = HTMLElement.ToByteArray(" bgcolor=\"");
    private static final byte[] BORDERCOLOR_ATTR = HTMLElement.ToByteArray(" bordercolor=\"");
    private static final byte[] CELLPADDING_ATTR = HTMLElement.ToByteArray(" cellpadding=\"");
    private static final byte[] CELLSPACING_ATTR = HTMLElement.ToByteArray(" cellspacing=\"");
    private static final byte[] HEIGHT_ATTR = HTMLElement.ToByteArray(" height=\"");
    private static final byte[] WIDTH_ATTR = HTMLElement.ToByteArray(" width=\"");
    private static final byte[] BORDER_ATTR = HTMLElement.ToByteArray(" border=\"");
    private static final byte[] SUMMARY_ATTR = HTMLElement.ToByteArray(" summary=\"");
    private static final byte[] ON_SELECT_START_ATTR = HTMLElement.ToByteArray(" onselectstart=\"");
    private static final byte[] ENDTAG = HTMLElement.ToByteArray("</table>");

    private String sAlign;
    private String sBGColor;
    private String sBorder;
    private String sBorderColor;
    private String sCellPadding;
    private String sCellSpacing;
    private String sHeight;
    private String sOnSelectStart;
    private String sSummary;
    private String sWidth;
    private boolean bErrorCaught = false;
    private boolean bHousingForm = false;
    private boolean bValidated;
    private int iNumColsSpanned;
    private int iNumRows;

    /**
     * Creates a new HTMLTable object.
     */
    public HTMLTable()
    {
        sAlign       = null;
        sBGColor     = null;
        sBorderColor = null;
        sCellPadding = null;
        sCellSpacing = null;
        sHeight      = null;
        sWidth       = null;
        sBorder      = null;

        iNumColsSpanned = -1;
        iNumRows        = 0;
        bValidated      = false;

		sOnSelectStart = null;
        sSummary = null;
    }

    /**
     * Call this method to get the value of the align attribute.
     *
     * @return String value.
     */
    public String GetAlign()
    {
        return sAlign;
    }

    /**
     * Call this method to get the value of the bgcolor attribute.
     *
     * @return String value.
     */
    public String GetBGColor()
    {
        return sBGColor;
    }

    /**
     * Call this method to get the value of the border attribute.
     *
     * @return String value.
     */
    public String GetBorder()
    {
        return sBorder;
    }

    /**
     * Call this method to get the value of the bordercolor attribute.
     *
     * @return String value.
     */
    public String GetBorderColor()
    {
        return sBorderColor;
    }

    /**
     * Call this method to get the value of the cellpadding attribute.
     *
     * @return String value.
     */
    public String GetCellPadding()
    {
        return sCellPadding;
    }

    /**
     * Call this method to get the value of the cellspacing attribute.
     *
     * @return String value.
     */
    public String GetCellSpacing()
    {
        return sCellSpacing;
    }

    /**
     * Call this method to obtain an clone of this table.
     *
     * @return Object that is an HTMLTable.
     */
    public Object GetClone()
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering GetClone()");
//        }

        HTMLTable tClone = (HTMLTable) super.GetClone();

        IXPEnumerator xpEnumerator = tClone.GetInnerHTMLElements(false);
        HTMLTableRow r;

        while (xpEnumerator.MoveNext())
        {
//            if (bDebug)
//            {
//                myLogger.Debug("Setting parent table on inner row.");
//            }

            r = (HTMLTableRow) xpEnumerator.GetCurrent();

            r.SetParentTable(this);
        }

        return tClone;
    }

    /**
     * Call this method to get the value of the height attribute.
     *
     * @return String value.
     */
    public String GetHeight()
    {
        return sHeight;
    }

    /**
     * Call this method to determine the number of columns spanned by this
     * table.  This value may change if rows are added or removed. <br>
     * <br>
     * Calling this method may be expensive, as the table needs to be
     * evaluated to determine this information.
     *
     * @return
     *
     * @throws HTMLException
     */
    public int GetNumColsSpanned()
    {
        Validate();

        return iNumColsSpanned;
    }

    /**
     * Call this method to get the value of the onselectstart attribute.
     *
     * @return String value.
     */
    public String GetOnSelectStart()
    {
        return sOnSelectStart;
    }

    /**
     * Call this method to get the value of the summary attribute.
     *
     * @return String value.
     */
    public String GetSummary()
    {
        return sSummary;
    }

    /**
     * Call this method to get the value of the width attribute.
     *
     * @return String value.
     */
    public String GetWidth()
    {
        return sWidth;
    }

    /**
     * Call this method to set the align attribute value.
     *
     * @param s String value.
     */
    public void SetAlign(String s)
    {
        sAlign = s;
    }

    /**
     * Call this method to set the bgcolor attribute value.
     *
     * @param s String value.
     */
    public void SetBGColor(String s)
    {
        sBGColor = s;
    }

    /**
     * Call this method to set the border attribute value.
     *
     * @param s String value.
     */
    public void SetBorder(String s)
    {
        sBorder = s;
    }

    /**
     * Call this method to set the bordercolor attribute value.
     *
     * @param s String value.
     */
    public void SetBorderColor(String s)
    {
        sBorderColor = s;
    }

    /**
     * Call this method to set the cellpadding attribute value.
     *
     * @param s String value.
     */
    public void SetCellPadding(String s)
    {
        sCellPadding = s;
    }

    /**
     * Call this method to set the cellspacing attribute value.
     *
     * @param s String value.
     */
    public void SetCellSpacing(String s)
    {
        sCellSpacing = s;
    }

    /**
     * Call this method to set the height attribute value.
     *
     * @param s String value.
     */
    public void SetHeight(String s)
    {
        sHeight = s;
    }

    /**
     * Call this method to set the onselectstart attribute value.
     * 
     * This is IE specific, and has been added so that developers
     * can turn off text selection in tables as this can cause
     * problems with fancier javascript code.
     *
     * @param s String value.
     */
    public void SetOnSelectStart(String s)
    {
        sOnSelectStart = s;
    }

    /**
     * Call this method to set the summary attribute value.
     *
     * @param s String value.
     */
    public void SetSummary(String s)
    {
        sSummary = s;
    }

    /**
     * Call this method to set the width attribute value.
     *
     * @param s String value.
     */
    public void SetWidth(String s)
    {
        sWidth = s;
    }

    /**
     * Call this method to add a HTMLTableRow object to this table.  The
     * HTMLTableRow passed as a parameter must have been constructed with a
     * reference to this very HTMLTable.
     *
     * @param e
     *
     * @throws HTMLException
     */
    public void AddInnerHTMLElement(HTMLElement e)
    {
        if (!(e instanceof HTMLTableRow) && !(e instanceof HTMLElementCollection) && !(e instanceof HTMLForm))
        {
            throw new HTMLException("You may only add HTMLRow, HTMLForm, or HTMLElementCollection elements to an HTMLTable.");
        }

        if (e instanceof HTMLForm)
        {
            this.bHousingForm = true;

            HTMLForm f = (HTMLForm) e;

            f.SetParentTable(this);

            super.AddInnerHTMLElement(f);
        }
        else if (e instanceof HTMLElementCollection)
        {
            IXPEnumerator enumItems = e.GetInnerHTMLElements(false);

            if (enumItems != null)
            {
                HTMLElement eTemp;

                while (enumItems.MoveNext())
                {
                    eTemp = (HTMLElement) enumItems.GetCurrent();

//                    if (bDebug)
//                    {
//                        myLogger.Debug("eTemp: " + eTemp);
//                    }

                    if (!(eTemp instanceof HTMLTableRow))
                    {
                        throw new HTMLException("You may only add an HTMLElementCollection containing HTMLTableRow elements to an HTMLTable element.  You are attempting to add an: " + eTemp);
                    }

                    ProcessRowForAdding((HTMLTableRow) eTemp);
                } //   for each inner element
            } //   got inner elements
        } // e instanceof HTMLElementCollection
        else
        {
            // it's just a row being added
            ProcessRowForAdding((HTMLTableRow) e);
        }


        //   we must validate anew.
        Invalidate();
    }

    /**
     * This method generates the beginning of the opening tag.  The abstract
     * HTMLElement superclass will provide the remainder of the opening tag,
     * repleate with attributes common to all html elements, such as name, and
     * id, and class.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     *
     * @throws HTMLException
     */
    protected void AppendBeginningOfOpeningTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        Validate();

        WriteNugget(BEGINTAG, sb, response);

        if (sAlign != null)
        {
            WriteNugget(ALIGN_ATTR, sb, response);
            WriteNugget(sAlign, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sBGColor != null)
        {
            WriteNugget(BGCOLOR_ATTR, sb, response);
            WriteNugget(sBGColor, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sBorderColor != null)
        {
            WriteNugget(BORDERCOLOR_ATTR, sb, response);
            WriteNugget(sBorderColor, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sCellPadding != null)
        {
            WriteNugget(CELLPADDING_ATTR, sb, response);
            WriteNugget(sCellPadding, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sCellSpacing != null)
        {
            WriteNugget(CELLSPACING_ATTR, sb, response);
            WriteNugget(sCellSpacing, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sHeight != null)
        {
            WriteNugget(HEIGHT_ATTR, sb, response);
            WriteNugget(sHeight, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sWidth != null)
        {
            WriteNugget(WIDTH_ATTR, sb, response);
            WriteNugget(sWidth, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sBorder != null)
        {
            WriteNugget(BORDER_ATTR, sb, response);
            WriteNugget(sBorder, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sOnSelectStart != null)
        {
            WriteNugget(ON_SELECT_START_ATTR, sb, response);
            WriteNugget(sOnSelectStart, sb, response);
            WriteNugget(QUOTE, sb, response);
        }

        if (sSummary != null)
        {
            WriteNugget(SUMMARY_ATTR, sb, response);
            WriteNugget(sSummary, sb, response);
            WriteNugget(QUOTE, sb, response);
        }
        else
        {
            if (bEnforceSummary || GetDo508CheckingForThisInstance())
            {
                if (!bErrorCaught)
                {
                    //	this prevents infinite loop.
                    bErrorCaught = true;

//                    if (bError)
//                    {
//                        myLogger.Error("You must set the summary attribute on this HTMLTable.");
//                    }

                    bErrorCaught = false;
                }
            }
        }
    }

    /**
     * This method generates the closing tag of this html element.  If the
     * providesClosingTag method returns false, then this method will never be
     * called.
     *
     * @param bFormat boolean indicating whether or not to format.
     * @param sb XPStringBuilder
     * @param response IXPResponse
     */
    protected void AppendClosingTag(boolean bFormat, XPStringBuilder sb, IXPResponse response)
    {
        WriteNugget(ENDTAG, sb, response);

        AppendClosingLineBreak(sb, response);
    }

    /**
     * Indicates whether or not this HTML element provides a closing tag. Some
     * elements only provide opening tags, such as the &lt;br&gt; element.
     *
     * @return boolean true.
     */
    protected boolean ProvidesClosingTag()
    {
        return true;
    }

    /**
     * Call to determine whether or not this HTMLTable is currently validated.
     *
     * @return boolean value.
     */
    boolean IsValidated()
    {
        return bValidated;
    }

    /**
     * Get the next id value.  This is used by the GetClone method to ensure
     * that cloned object have new ID attribute values.
     *
     * @return A valid id string.
     */
    String GetNextID()
    {
        String s = "table" + iNextID;
        iNextID++;

        return s;
    }

    /**
     * This method adds an HTMLRow to this table.
     *
     * @param r Pass an HTMLRow object.
     *
     * @throws HTMLException This method throws an exception if for any reason
     *         the row passed isn't appropriate.
     */
    void ProcessRowForAdding(HTMLTableRow r)
    {
        if (r.GetParentTable() == null)
        {
            r.SetParentTable(this);
        }
        else
        {
            if (r.GetParentTable() != this)
            {
                throw new HTMLException("You may only add a given HTMLTableRow to a single given HTMLTable.");
            }
        }

        super.AddInnerHTMLElement(r);
    }

    //   package-private access level

    /**
     * This method should be called whenever a change has been made to any
     * member HTMLTableCell objects that would cause this  HTMLTableRow to
     * need to validate.
     */
    void Invalidate()
    {
//        if (bInfo)
//        {
//            myLogger.Info("HTMLTable::invalidate() ");
//        }

        bValidated = false;


        //   we know not how many cols we span.
        iNumColsSpanned = -1;
    }

    /**
     * Call this method to prepare this HTMLTable for display. <br>
     * <br>
     * If this HTMLTable is already validated, then this method will not do
     * anything. <br>
     * <br>
     * <br>
     * this method will iterate through this table's constituent rows.   Each
     * row will first be validated. <br>
     * <br>
     * The number of columns spanned by this table will be determined. <br>
     * <br>
     * This HTMLTable will then enter a validated state, and be ready for
     * display.
     *
     * @throws HTMLException
     */
    private void Validate()
    {
//        if (bInfo)
//        {
//            myLogger.Info("Entering validate()");
//        }

        //   If this table is already validated, or we're not validating tables,
        //   just return.
        if (bValidated)
        {
//            if (bDebug)
//            {
//                myLogger.Debug("Already validated, returning.");
//            }

            return;
        }
//
//        if (bInfo)
//        {
//            myLogger.Info("validate() - proceeding");
//        }

//        if (HTMLElement.bEnforceMaxTableDepth)
//        {
//            if (bDebug)
//            {
//                myLogger.Debug("validate() - about to test for excessive table nesting depth.");
//            }
//
//            if (this.GetTableLevel() > iMaxTableDepth)
//            {
//                if (bError)
//                {
//                    myLogger.Error("This table is nested too deeply.");
//                }
//
//                //                throw new HTMLTable.TableNestingTooDeepException("The maximum table depth is: " + this.iMaxTableDepth);
//            }
//        }

        IXPEnumerator e;

        e = GetInnerHTMLElements(false);

        if (this.bHousingForm)
        {
            e.MoveNext();

            HTMLForm f = (HTMLForm) e.GetCurrent();

            e = f.GetInnerHTMLElements(false);
        }

        HTMLTableRow r;

        int iMaxColsSpanned = 0;
        int iRowIndex       = 0;

        if (e == null)
        {
//            if (bError)
//            {
//                myLogger.Error(s);
//            }

            //			throw new HTMLException( s );
        }
        else
        {
            while (e.MoveNext())
            {
            	HTMLElement eTemp = (HTMLElement) e.GetCurrent();

                if(!(eTemp instanceof HTMLTableRow))
                {
                	continue;
                }

                r = (HTMLTableRow) e.GetCurrent();

                r.SetIndexInTable(iRowIndex);

                r.Validate();

                if (!r.GetHasAutoFitElement())
                {
                    //  the row being added does not have an autofit element.
//                    if (bDebug)
//                    {
//                        myLogger.Debug("validate() - Found a row without an autofit element.");
//                    }

                    int iRowColsSpanned = r.GetNumColsSpanned();
//
//                    if (bDebug)
//                    {
//                        myLogger.Debug("validate() - iNumColsSpanned: " + iNumColsSpanned);
//                        myLogger.Debug("validate() - iRowColsSpanned: " + iRowColsSpanned);
//                    }

                    if (iNumColsSpanned == -1)
                    {
                        iNumColsSpanned = iRowColsSpanned;
                    }
                    else if (iRowColsSpanned != iNumColsSpanned)
                    {
                        //  the row being added does not span the
                        //  the appropriate number of columns.
                        //String sErrorDescription = "The row at index " + iRowIndex + " spans " + r.GetNumColsSpanned() + " columns, where it should span " + iNumColsSpanned;

//                        if (bError)
//                        {
//                            myLogger.Error(sErrorDescription);
//                        }

                        //	                    throw new InvalidColumnSpanException(sErrorDescription);
                    }

                    iMaxColsSpanned = iNumColsSpanned;
                } //  the row being added does not have an autofit element.
                else
                {
                    //  the row being added has an autofit element.
//                    if (bDebug)
//                    {
//                        myLogger.Debug("validate() - Found a row WITH an autofit element.");
//                    }

                    int iRowColSpan = r.GetNumColsSpanned();
//
//                    if (bDebug)
//                    {
//                        myLogger.Debug("validate() - iRowColSpan: " + iRowColSpan);
//                    }

                    if (iRowColSpan > iMaxColsSpanned)
                    {
                        //   We need to determine the maximum number of columns
                        //   spanned by any autofit row.  In the event that this table
                        //   consists only of autofit rows, the rows with fewer elements will
                        //   need to autofit themselves to match the row with the most
                        //   elements.  Remember, when asked how many columns they span, rows
                        //   with autofit elements assume that
                        //   the autofit element spans only 1 column.
                        iMaxColsSpanned = iRowColSpan;
                    }

                    //  If we haven't yet determined how many columns we span, then
                    //  we shouldn't perform a check.
                    if (iNumColsSpanned != -1)
                    {
                        //  We know how many rows we span.
                        if (iRowColSpan > iNumColsSpanned)
                        {
                            //  the row being added spans more columns than
                            //   a row without an autofit element.
                            //String sErrorDescription = "The row at index " + iRowIndex + " spans " + iRowColSpan + " columns, where it should span " + iNumColsSpanned;

//                            if (bError)
//                            {
//                                myLogger.Error(sErrorDescription);
//                            }

                            //	                        throw new InvalidColumnSpanException(sErrorDescription);
                        }
                    }
                } //  the row added has an autofit element.

                iRowIndex++;
            } // for each row
        } // e != null

        if (iNumColsSpanned == -1)
        {
            //   we were unable to determine how many columns we should span.
            //   This might be because all of the rows had autofit elements.
            //   In this case, this table must span the number of columns
            //   of the row spanning the greatest number of columns.
            iNumColsSpanned = iMaxColsSpanned;
        }

        bValidated = true;
    }
}
