package com.plumtree.xpshared.htmlelements;

/**
 * This class represents a style class defined in an HTML
 * style sheet.
 * 
 * To use a custom style class, simply create a new StyleClass
 * instance with your custom style and pass it into the HTMLElement.
 * 	<br><br>
 * 	For example, if you have an HTMLTable object, and you wish to set
 *  a custom style class, you can do this:
 * 	<br><br>
 * 	<code>
 * 	myTable.SetStyleClass( new StyleClass("customclass" );
 * 	</code>
 * 	<br><br>
 * 	The resulting html will look something like:
 * 	<br><br>
 * 	&lt;table class="customclass"&gt; ...
 * 	<br><br>
 */
public class StyleClass
{
	/* bug 23279 fix: allow public creation */
    static protected boolean bAllowPublicCreation = true;

    protected String sClass;

    /**
     * Creates a new StyleClass object.
     *
     * @param strStyleClass The name of this style class.
     *
     * @throws HTMLException This method throws an exception if public creation
     *         of StyleClass objects is disallowed.
     */
    public StyleClass(String strStyleClass)
    {
        if (!bAllowPublicCreation)
        {
            throw new HTMLException("This class may not be created publicly.  You must use a pre-instantiated StyleClass object.");
        }

        sClass = strStyleClass;
    }

    /**
     * Call this method to obtain the name of this style class.
     *
     * @return String name
     */
    public String GetStyleName()
    {
        return sClass;
    }
}
