using System;
using com.plumtree.openfoundation.web;
using com.plumtree.uiinfrastructure.activityspace;
using com.plumtree.uiinfrastructure.web;
using com.plumtree.xpshared.htmlelements;
namespace com.plumtree.uiinfrastructure.login
{
  /// <summary>
  /// This interface defines the extensible login helper code.  This is used
  /// by the Interpreter and SSO.
  /// </summary>
  public interface ILoginHelper
  {
    
    /// <summary>
    /// Attempts to create and connect a new user session.
    /// </summary>
    /// <returns>a valid userSession</returns>
    Object AttemptLogin(String sUserName, String sPassword, Object token,
     bool bUseNumericConnect, ISessionManager session, IApplication application,
     String sRequestURL);
    
    /// <summary>
    /// Attempts to create and connect a new default guest user session.
    /// </summary>
    /// <returns>a valid guest userSession</returns>
    Object AttemptGuestLogin(Object token, ISessionManager session, IApplication application,
     String sRequestURL);
    
    /// <summary>
    /// Multiple-guest-user support. Attempts to create a new user session
    /// and connect as the given guest user.
    /// </summary>
    /// <returns>a valid guest userSession</returns>
    Object AttemptGuestLogin(int nUserID, Object token, ISessionManager session,
     IApplication application, String sRequestURL);
    
    /// <summary>
    /// Attempts to create and connect a new user session.
    /// Throws an exception if login fails.
    /// </summary>
    /// <returns>a valid user session</returns>
    //ES - add XPRequest
    Object AttemptLogin(String sLoginToken, ISessionManager session, IApplication application,
     String sRequestURL, IXPRequest request);
    
    /// <summary>
    /// Attempts to create and connect a new user session.
    /// This method is used for Basic Auth logins.  Unfortunately, in Basic Auth
    /// the user name and password are encoded in the character encoding of the
    /// client machine, and the server does not know what that is.  Therefore
    /// we have to guess the appropriate encoding.  The user credentials
    /// passed to this method need to be unencoded as win-1252, shift-jis, Big5,
    /// etc... before they can be used.  The credentials are basic auth credentials;
    /// user name and password are separated by a colon.
    /// </summary>
    /// <returns>a valid userSession</returns>
    Object AttemptBasicAuthLogin(String sEncodedCredentials);
    
    /// <summary>
    /// This method gets the user ID from the supplied user
    /// session object.
    /// </summary>
    /// <param name="Object">the user session</param>
    /// <returns>
    /// String the user ID (return "" instead of null
    /// if necessary)
    /// </returns>
    String GetUserSessionID(Object userSession);
    
    /// <summary>
    /// This method gets the user Name from the supplied user
    /// session object.
    /// </summary>
    /// <param name="Object">the user session</param>
    /// <returns>
    /// String the user Name (return "" instead of null
    /// if necessary)
    /// </returns>
    String GetUserSessionName(Object userSession);
    
    /// <summary>
    /// This method returns whether or not the user is a guest user.
    /// </summary>
    /// <param name="Object">the user session</param>
    /// <returns>boolean true implies the user is a guest user.</returns>
    bool GetIsGuestUser(Object userSession);
    
    /// <summary>
    /// This method gets an error message suitable for display to the user
    /// from the supplied exception (i.e. localized and without any details
    /// that a hacker could exploit, such as whether or not that particular
    /// user name is valid).
    /// </summary>
    /// <param name="Exception">the login exception to interpret</param>
    /// <param name="ISessionManager">the current subsession to use</param>
    /// <param name="IApplication">the application</param>
    /// <returns>
    /// String the error message for the user (return "" instead of null
    /// if necessary)
    /// </returns>
    String GetDetailedFailureMessage(Exception exception, ISessionManager subSession,
     IApplication application);
    
    /// <summary>
    /// Call this method to obtain a redirect to the default space.
    /// </summary>
    /// <param name="ILink">
    /// user supplied link that will get filled in with the
    /// proper parameters. (Redirect or ASURL object)
    /// </param>
    /// <param name="Object">userSession</param>
    /// <param name="AActivitySpace"></param>
    /// <returns>ILink to the default page for this user.</returns>
    ILink GetDefaultPageRedirect(ILink suppliedLink, Object userSession, AActivitySpace asOwner);
    
    /// <summary>
    /// Call this method to obtain a redirect to the default space.
    /// </summary>
    /// <param name="ILink">
    /// user supplied link that will get filled in with the
    /// proper parameters. (Redirect or ASURL object)
    /// </param>
    /// <param name="Object">userSession</param>
    /// <param name="IXPRequest">request</param>
    /// <returns>ILink to the default page for this user.</returns>
    ILink GetDefaultPageRedirect(ILink suppliedLink, Object userSession, IXPRequest request);
    
    /// <summary>
    /// Call this method to set an autologin token for the user.
    /// </summary>
    /// <param name="oWebData"></param>
    /// <param name="oUserSession"></param>
    /// <param name="nExpires">cookie expiration time in seconds.  -1 means use default value.</param>
    /// <param name="bIsPersistentCookie">
    /// false means don't set an expires time
    /// (expire at end of session).
    /// </param>
    /// <param name="bIsSecure">whether or not the login cookie is secure</param>
    void SetAutologinToken(IWebData oWebData, Object oUserSession, int nExpires,
     bool bIsPersistentCookie, bool bIsSecure);
    
    /// <summary>
    /// Checks if the current COM process has been recycled - retuns a new session in the new process if it has, otherwise returns null.
    /// </summary>
    /// <param name="userSession">Current user session</param>
    /// <returns>
    /// An IPTSession created in new COM process with the current user logged in if the current process has been recycled.
    /// null if no recycling happened.
    /// </returns>
    Object HandleCOMPlusProcessRecycled(Object userSession);
    
    /// <summary>
    /// Returns an HTMLElementCollection of style sheet links.  A style sheet is
    /// named "stylesheetname-language", for instance "mainstyle12-en".
    /// </summary>
    /// <param name="userSession"></param>
    /// <param name="AActivitySpace"></param>
    HTMLElementCollection GetStyleSheetLinks(Object userSession, AActivitySpace asOwner);
    }
}